// AD9162_Dac_SubDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef AD9162_Dac_SubDevice_MbH
#define AD9162_Dac_SubDevice_MbH

#include <X6Support_Mb.h>
#include <JESDSubDevice_Stream_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IRequires_AD9162_Dac_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_AD9162_Dac_Io : public IRequires_JESDDAC
{
public:

    // Customization
    virtual void SetDacLaneConfiguration(unsigned int device) = 0;

    // Io Interface
    virtual void         DacSpi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int DacSpi(unsigned int device, unsigned int address) = 0;

    virtual bool DacChannelEnabled(int ch) = 0;
    virtual void DacChannelEnabled(int ch, bool state) = 0;

    // Customization Interface
    virtual ~IRequires_AD9162_Dac_Io()
        {}
};

//===========================================================================
//  CLASS IAD9162_Dac_Loader -- Information and Parameter Loading for AD9162_Dac
//===========================================================================

class IAD9162_Dac_Loader
{
public:

    virtual ~IAD9162_Dac_Loader()
        {}

    virtual void SetInterpolationRate(unsigned int value) = 0;
    virtual void SetDecodeMode(unsigned int value) = 0;
    virtual void SetHbFilterBw(unsigned int value) = 0;
    virtual void SetEnInvSinc(bool value) = 0;
    virtual void SetNcoMode(unsigned int value) = 0;
    virtual void SetNcoSideband(unsigned int value) = 0;
    virtual void SetNcoFreq(double value) = 0;
    virtual void SetNcoPhase(int value) = 0;
    virtual void SetNcoOnlyAmplitude(int value) = 0;
};

//===========================================================================
//  CLASS AD9162_DAC -- JESD D/A Support "SubDevice"
//===========================================================================

class AD9162_DAC : public IX6SubDevice, public IAD9162_Dac_Loader
{
public:
    AD9162_DAC(IRequires_AD9162_Dac_Io * owner, unsigned int jesd_device_id);
    virtual ~AD9162_DAC();

    //
    //  Methods
    void         Spi(unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int address);

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void ConfigureHardware();

    double CalcDacJesdLineRate(double SamplingRate);
    double CalcLmfcRate();
    bool   ValidSampleRate(string & text);
    void   GetJesdLinkStatus();

    void  SetFullScaleCurrent_mA(double value)
        {  FFullScaleCurrent_mA = value;  }
    double  GetFullScaleCurrent_mA()
        {  return FFullScaleCurrent_mA;  }

    // DAC
    virtual void SetInterpolationRate(unsigned int value);
    virtual void SetDecodeMode(unsigned int value);
    virtual void SetHbFilterBw(unsigned int value);
    virtual void SetEnInvSinc(bool value);
    virtual void SetNcoMode(unsigned int value);
    virtual void SetNcoSideband(unsigned int value);
    virtual void SetNcoFreq(double value);
    virtual void SetNcoPhase(int value);
    virtual void SetNcoOnlyAmplitude(int value);

    //
    //  Configuration Customization
    vector<unsigned int> LanePN_Inverted;       // 1 = inverted, 0 not inverted
    vector<unsigned int> PhysToLogical_LaneMap; // physical to logical lanes mapping

private:
    IRequires_AD9162_Dac_Io *  MDC;
    const unsigned int         DeviceID;   // ID for proper JESD DAC Device ID
    JESD_DACDevice &           JesdDac() {  return MDC->JesdDac(DeviceID);  }

    void         ConfigureDacStartUp();
    void         ConfigureDacClkDll();
    void         WaitForDacClkDllLock();
    void         ConfigureFullScaleCurrent();
    void         ConfigureDacNco();
    void         CalcNcoParams();
    void         ConfigureJesdStartUp();
    void         ConfigureDacSerDesPll();
    void         ConfigureJesdParams();
    unsigned int CalcChecksum();
    void         ConfigureDacLanes();
    void         ConfigureSyncProcedure();
    void         GetRcvdILASConfiguration();
    void         CalcDacParams();
    bool         ChannelEnabled(int ch);
    void         ChannelEnabled(int ch, bool state);

    enum DecodeModes { dmNRZ, dmMixMode, dmRZ, dm2xNRZ };
    enum HbFilterBws { hfb80, hfb90 };
    enum NcoModes { nmNone, nmModulator, nmNcoOnly };
    enum NcoSidebands { nsbUpper, nsbLower };

    double       FFullScaleCurrent_mA;

    // DAC
    unsigned int InterpolationRate;
    DecodeModes  DecodeMode;
    HbFilterBws  HbFilterBw;
    bool         EnInvSinc;
    NcoModes     NcoMode;
    NcoSidebands NcoSideband;
    double       NcoFreq;
    int          NcoPhase;
    int          NcoOnlyAmplitude;

    double       DacSamplingRate;
    unsigned int InterpolationFactor;
    unsigned int NumConverters;
    unsigned int OctetsPerFrame;
    unsigned int NumLanes;
    double       LineRate;
    unsigned int SamplesPerFrame;
    bool         HighDensityMode;
    bool         EnNcoDmMode;
    s_ii64       NcoFtw;
    s_ii64       NcoDelta;
    ii64         NcoModulus;
    bool         ErrorOccurred;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
