// AD9680_SubDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef AD9680_SubDevice_MbH
#define AD9680_SubDevice_MbH

#include <X6Support_Mb.h>
#include <JESDSubDevice_Stream_Mb.h>
#include <FmcSupport_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IRequires_AD9680_ADC_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_AD9680_ADC_Io : public IRequires_JESDADC
{
public:

    // Customization
    virtual void SetAdcLaneConfiguration(unsigned int device) = 0;

    // Io Interface
    virtual void         AdcSpi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int AdcSpi(unsigned int device, unsigned int address) = 0;

    virtual bool         AdcChannelEnabled(int ch) = 0;
    virtual void         AdcChannelEnabled(int ch, bool state) = 0;
    virtual unsigned int AdcActiveChannels() = 0;

    // Customization Interface
    virtual ~IRequires_AD9680_ADC_Io()
        {}
};

//===========================================================================
//  CLASS IAD9680_ADC_Loader -- Information and Parameter Loading for AD9680
//===========================================================================

class IAD9680_ADC_Loader
{
public:

    virtual ~IAD9680_ADC_Loader()
                {}

    virtual unsigned int EffectiveAdcChannels() = 0;
    virtual unsigned int EffectiveAdcDevices() = 0;

    // ADC
    virtual void SetAdcChipOperatingMode(const unsigned int Mode) = 0;
    virtual void SetAdcFullBwDecimationRate(const unsigned int value) = 0;
    virtual void SetDdcSyncMode(const unsigned int Mode) = 0;

    virtual void AdcDeviceTestPattern(unsigned int pattern1, unsigned int pattern2,
                                      unsigned int pattern3, unsigned int pattern4) = 0;

    // ADC Input Channel
    virtual void SetOverrideChannelSetting(const bool value,const unsigned int AdcChIndex) = 0;
    virtual void SetHighFreqSetting(const bool value,const unsigned int AdcChIndex) = 0;
    virtual void SetLowFreqOperation(const bool value,const unsigned int AdcChIndex) = 0;
    virtual void SetBufferControl1(const unsigned int value,const unsigned int AdcChIndex) = 0;
    virtual void SetBufferControl2(const unsigned int value,const unsigned int AdcChIndex) = 0;
    virtual void SetInputCapacitance(const unsigned int value,const unsigned int AdcChIndex) = 0;
    virtual void SetInputTermination(const unsigned int value,const unsigned int AdcChIndex) = 0;
    virtual void SetAdcTestMode(const unsigned int value,const unsigned int AdcChIndex) = 0;

    // DDC
    virtual void SetMixer(const unsigned int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetGain(const unsigned int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetIFintermediateFreq(const unsigned int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetDecimationRate(const unsigned int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetComplexToReal(const bool enabled,const unsigned int DdcIndex) = 0;
    virtual void SetNCOfreq(const int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetNCOphase(const int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetQinput(const unsigned int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetIinput(const unsigned int Mode,const unsigned int DdcIndex) = 0;
    virtual void SetQoutput(const bool enabled,const unsigned int DdcIndex) = 0;
    virtual void SetIoutput(const bool enabled,const unsigned int DdcIndex) = 0;
};

//===========================================================================
//  CLASS AD9680_ADC -- JESD A/D Support "SubDevice"
//===========================================================================

class AD9680_ADC : public IX6SubDevice, public IAD9680_ADC_Loader
{
public:
    AD9680_ADC(IRequires_AD9680_ADC_Io * owner, unsigned int jesd_device_id);
    virtual ~AD9680_ADC();

    unsigned int Bits();
    void         OverrideChannelEnabled();
    double       CalcAdcJesdLineRate(double SamplingRate);

    //
    //  Methods
    void         Spi(unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int address);

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void ConfigureHardware();

    // IAD9680_ADC_Loader Implementation
    virtual unsigned int EffectiveAdcChannels();
    virtual unsigned int EffectiveAdcDevices();

    // ADC
    virtual void SetAdcChipOperatingMode(const unsigned int Mode);
    virtual void SetAdcFullBwDecimationRate(const unsigned int value);
    virtual void SetDdcSyncMode(const unsigned int Mode);

    void AdcDeviceTestPattern(unsigned int pattern1, unsigned int pattern2,
                              unsigned int pattern3, unsigned int pattern4);

    // ADC Input Channel
    virtual void SetOverrideChannelSetting(const bool value,const unsigned int AdcChIndex);
    virtual void SetHighFreqSetting(const bool value,const unsigned int AdcChIndex);
    virtual void SetLowFreqOperation(const bool value,const unsigned int AdcChIndex);
    virtual void SetBufferControl1(const unsigned int value,const unsigned int AdcChIndex);
    virtual void SetBufferControl2(const unsigned int value,const unsigned int AdcChIndex);
    virtual void SetInputCapacitance(const unsigned int value,const unsigned int AdcChIndex);
    virtual void SetInputTermination(const unsigned int value,const unsigned int AdcChIndex);
    virtual void SetAdcTestMode(const unsigned int value,const unsigned int AdcChIndex);

    // DDC
    virtual void SetMixer(const unsigned int Mode,const unsigned int DdcIndex);
    virtual void SetGain(const unsigned int Mode,const unsigned int DdcIndex);
    virtual void SetIFintermediateFreq(const unsigned int Mode,const unsigned int DdcIndex);
    virtual void SetDecimationRate(const unsigned int Mode,const unsigned int DdcIndex);
    virtual void SetComplexToReal(const bool enabled,const unsigned int DdcIndex);
    virtual void SetNCOfreq(const int Mode,const unsigned int DdcIndex);
    virtual void SetNCOphase(const int Mode,const unsigned int DdcIndex);
    virtual void SetQinput(const unsigned int Mode,const unsigned int DdcIndex);
    virtual void SetIinput(const unsigned int Mode,const unsigned int DdcIndex);
    virtual void SetQoutput(const bool enabled,const unsigned int DdcIndex);
    virtual void SetIoutput(const bool enabled,const unsigned int DdcIndex);

    //
    //  Configuration Customization
    vector<unsigned int> LanePN_Inverted;       // 1 = inverted, 0 not inverted
    vector<unsigned int> PhysToLogical_LaneMap; // physical to logical lanes mapping

private:
    IRequires_AD9680_ADC_Io * MDC;
    const unsigned int        DeviceID;   // ID for proper JESD ADC Device ID
    JESD_ADCDevice &          JesdAdc() {  return MDC->JesdAdc(DeviceID);  }

    void         AdcInit();
    void         ConfigureAdcInputs();
    void         ConfigureAdcCh(unsigned int AdcChIndex);
    void         ConfigureJesdParams();
    void         ConfigureDdcParams();
    void         ConfigureDdcRegisters(unsigned int DdcIndex);
    void         ConfigureCommonRegs();
    bool         AllLanesInverted();
    void         CalcAdcParams();
    unsigned int CalcDdcDecRate(unsigned int DdcIndex);
    bool         IsInDdcMode();
    bool         ChannelEnabled(int ch);
    void         ChannelEnabled(int ch, bool state);
    unsigned int ActiveChannels();
    unsigned int log2(unsigned int value);

    enum AdcChipOperatingModes { comFullBandwidth, comDdc0, comDdc0Ddc1, comAllFour };

    // ADC
    unsigned int AdcChipOperatingMode;
    unsigned int AdcFullBwDecimationRate;
    unsigned int DdcSyncMode;

    unsigned int AdcTestPattern1;
    unsigned int AdcTestPattern2;
    unsigned int AdcTestPattern3;
    unsigned int AdcTestPattern4;

    struct AdcChSettings
    {
        bool         OverrideChannelSetting;
        bool         HighFreqSetting;
        bool         LowFreqOperation;
        unsigned int BufferControl1;
        unsigned int BufferControl2;
        unsigned int InputCapacitance;
        unsigned int InputTermination;
        unsigned int AdcTestMode;
    };
    AdcChSettings AdcCh[2];

    struct DdcSetting
    {
        unsigned int Mixer;
        unsigned int Gain;
        unsigned int IFintermediateFreq;
        unsigned int DecimationRate;
        bool         ComplexToReal;
        int          NCOfreq;
        int          NCOphase;
        unsigned int Qinput;
        unsigned int Iinput;
        bool         Qoutput;
        bool         Ioutput;
    };
    DdcSetting DDC[4];

    unsigned int ChipDecimationRate;
    bool         ChipQignore;
    unsigned int NumConverters;
    unsigned int NumLanes;
    double       LineRate;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
