//
// ADF4355_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef ADF4355_MbH
#define ADF4355_MbH

#include <TimebaseDevice_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>
#include <VcxoControl_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS ADF4355PllTuningEvent  --  Override PLL Tuning Parameters
//===========================================================================

class ADF4355PllTuningEvent : public OpenWire::Event
{
public:
    unsigned int &RDiv2;
    unsigned int &RMul2;
    unsigned int &R;
    unsigned int &N_Int;
    unsigned int &N_Frac1;
    unsigned int &N_Frac2;
    unsigned int &N_Mod2;
    unsigned int &OD;

    ADF4355PllTuningEvent(unsigned int & rDiv2,  unsigned int & rMul2,
                          unsigned int & r,  unsigned int & n_Int,
                          unsigned int & n_Frac1, unsigned int & n_Frac2, unsigned int & n_Mod2,
                          unsigned int & od)
        : RDiv2(rDiv2), RMul2(rMul2), R(r), N_Int(n_Int),
          N_Frac1(n_Frac1), N_Frac2(n_Frac2), N_Mod2(n_Mod2), OD(od)
    {
    }
};

//===========================================================================
//  CLASS ADF4355Registers  -- Addresses for Device Registers
//===========================================================================

class ADF4355Registers
{
public:
    unsigned int   Control_Addr;
    unsigned int   SpiIntf_Addr;
};

//=============================================================================
//  CLASS ADF4355  --  PLL timebase control
//=============================================================================

class ADF4355 : public PllTimebaseDevice
{
public:
    enum AdfType { adf4355, adf4355_2, adf4356  };
    enum AdfMode { modeIntOnly, modeFractional };  // programming mode to use
    enum AdfReferenceSource { refPrimary=0x0 };
    enum AdfDevice { Dev0=0x1, Dev1=0x2, Dev_Both=0x3 };
    //
    //  Member Functions
    ADF4355(IAddressingSpace &space, ADF4355Registers & regs);
    virtual ~ADF4355();

    void SetType(AdfType type)  {  FChipType=type;  }
    void SetOperatingMode(AdfMode mode)  {  FOperatingMode=mode;  }

    void Init();
    void Reapply();
    void DeviceRouting(AdfDevice routing) { FDeviceRouting = routing; }

    //  Select the Input reference
    void ReferenceSource(AdfReferenceSource value);
    AdfReferenceSource  ReferenceSource()
        {  return FReferenceSource;  }

    bool  Locked() const;

    int             MaxOutputDivider() const;

    //  Raw configuration accesses
    virtual void         Port(unsigned int reg, unsigned int value);
    virtual unsigned int Port(unsigned int reg);
    void            ApplyImage(unsigned int reg);
    void            PortImage(unsigned int reg, unsigned int value);
    unsigned int    PortImage(unsigned int reg);

    OpenWire::EventHandler<ADF4355PllTuningEvent> OnPllTuning;

    double  PhaseDetectorFrequency() const           // PDF
                {  return FPhaseDetectorFrequency;  }
    void    SetPhaseDetectorFrequency(double freq)
                {  FPhaseDetectorFrequency = freq;  }
    double  ChannelSpacingFrequency() const         // channel spacing frequency
                {  return FChannelSpacingFrequency;  }
    void    SetFChannelSpacingFrequency(double freq)
                {  FChannelSpacingFrequency = freq;  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  Class TuneMatchResults
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class TuneMatchResults
    {
    public:
        double CandidateVco;
        unsigned int OD;

        TuneMatchResults(double vco, unsigned int od)
            :  CandidateVco(vco), OD(od)
        {}
    };


protected:
    //
    //  Member Data
    AdfType                    FChipType;
    AdfMode                    FOperatingMode;
    AdfReferenceSource         FReferenceSource;

    std::vector<unsigned int>  RegisterImage;
    std::vector<float>         OutputFrequencyLimit;

    double  FPhaseDetectorFrequency;
    double  FChannelSpacingFrequency;
    double  FVcxoFrequency;

    AdfDevice   FDeviceRouting;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    virtual void    OnSetPower(bool & new_state);

    virtual double  DefaultPhaseDetectorFrequency() const = 0;
    virtual unsigned int  DefaultChannelSpacingValue() const = 0;

    unsigned int Calculate_NMod2_fromSpacing(double PFD, double channel_spacing_freq);
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    enum DualModulusPrescaler { pre_4_5, pre_8_9 };               // SetPrescaler()
    enum PhaseDetectPolarity { pdpNegative, pdpPositive };        // SetPhaseDetectorPolarity()
    enum MuxoutLogicLevels { ldp_1p8v, ldp_3p3v};                 // SetMuxoutLevelSelect()
    enum ReferenceInOptions { refSingleEnded, refDifferential };  // SetReferenceMode()
    enum ChargePumpCurrentOptions { cpc_0p31, cpc_0p63, cpc_0p94,
                                    cpc_1p25, cpc_1p56, cpc_1p88,
                                    cpc_2p19, cpc_2p50, cpc_2p81,
                                    cpc_3p13, cpc_3p44, cpc_3p75,
                                    cpc_4p06, cpc_4p38, cpc_4p69,
                                    cpc_5p00};                    // SetChargePumpCurrent()
    enum MuxOutOptions { muxThreeState, muxDVdd, muxSDgnd,
                         muxRoutput, muxNoutput, muxAnalogLock,
                         muxDigitalLock     };                    // SetMuxOut()
    enum OutputPowerOptions { out_n4dbm, out_n1dbm,
                              out_p2dbm, out_p5dbm   };           // SetRfOutputPower(), SetAuxOutputPower()

    enum FeedbackSelectOptions { fsDivided, fsFundamental  };     // SetFeedbackSelect()

    enum LockDetectModeOptions { ldmFractionalN, ldmIntegerN  };  // SetLockDetectMode()
    enum FractionalNPrecisionOptions { ldp_5ns, ldp_6ns,
                                       ldp_8ns, ldp_12ns };       // SetFractionalNPrecision()
    enum LossOfLockCycleCountOptions { lol_1024, lol_2048,
                                       lol_4096, lol_8192  };     // SetLossOfLockCycleCount()

    void SetVcoAutocal(bool state);
    void SetPrescaler(DualModulusPrescaler prescaler);
    DualModulusPrescaler GetPrescaler();
    void SetN_Int(unsigned int n_int);
    unsigned int GetN_Int();

    void SetN_Frac1(unsigned int n_Frac1);
    unsigned int GetN_Frac1();

    void SetN_Frac2(unsigned int n_Frac2);
    unsigned int GetN_Frac2();
    void SetN_Mod2(unsigned int n_Mod2);
    unsigned int GetN_Mod2();

    void SetOutputPhase(unsigned int phase_value);
    void SetOutputPhaseAdjustEnabled(bool state);
    void SetOutputPhaseResyncEnabled(bool state);
    void SetSigmaDeltaLoadResetDisabled(bool state);

    void SetCounterReset(bool state);
    void SetCpThreeStateEnabled(bool state);
    void SetPowerDown(bool state);
    void SetPhaseDetectorPolarity(PhaseDetectPolarity polarity);
    void SetMuxoutLevelSelect(MuxoutLogicLevels level_setting);
    void SetReferenceMode(ReferenceInOptions ref_mode);
    void SetChargePumpCurrent(ChargePumpCurrentOptions current_setting);
    float GetChargePumpCurrent();
    void SetReg6_DoubleBuffering(bool state);
    void SetR(unsigned int R);
    unsigned int GetR();
    void SetRDiv2(unsigned int rDiv2);
    unsigned int GetRDiv2();
    void SetRMul2(unsigned int rMul2);
    unsigned int GetRMul2();
    void SetMuxOut(MuxOutOptions option);

    void SetRfOutputPower(OutputPowerOptions option);
    void SetRfOutputEnabled(bool state);
    void SetAuxOutputPower(OutputPowerOptions option);
    void SetAuxOutputEnabled(bool state);
    void SetMTLDEnabled(bool state);
    void SetChargePumpBleedCurrent(float current_uA);
    void SetFeedbackSelect(FeedbackSelectOptions option);
    FeedbackSelectOptions GetFeedbackSelect();
    void SetBleedCurrentEnabled(bool state);
    void SetGatedBleedEnabled(bool state);
    void SetBleedPolarity(bool bleed_pos);

    void SetLockDetectMode(LockDetectModeOptions option);
    void SetFractionalNPrecision(FractionalNPrecisionOptions option);
    void SetLossOfLockModeEnabled(bool state);
    void SetLossOfLockCycleCount(LossOfLockCycleCountOptions option);
    void SetLESyncEnabled(bool state);

    void SetReg9(double pfd);
    void SetSynthesizerLockTimeout(unsigned int value);
    void SetAutomaticLevelCalTimeout(unsigned int value);
    void SetTimeout(unsigned int value);
    void SetVcoBandDivision(unsigned int value);

    void SetReg10(double pfd);
    void VTuneAdcEnabled(bool enabled);
    void VTuneAdcConversionEnabled(bool enabled);
    void VTuneAdcClockDivider(unsigned int divider);

    void PhaseResyncClockDivider(unsigned int divider);

    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);

    unsigned int  GetDivider(unsigned int output);

    void  SetReferenceSource(AdfReferenceSource source);

    void  SetDeviceRouting(AdfDevice device);
    void  SetDevicePowerDown(ADF4355::AdfDevice device, bool state);
    void  SetDeviceEnable(ADF4355::AdfDevice device, bool state);
    unsigned int  GetMuxOut();

    //  Other Registers
    void  InitDataRegisters();
    void  WriteDataRegisters();

    //
    //  PLL Configuration Info
    bool    TuneMatch(std::vector<TuneMatchResults> &TM_results);
    bool    FindVcoRate(double& VcoRate, unsigned int& OD);

    //  piece parts for loading N_Frac2, N_Mod2
    void SetN_Frac2_LSB(unsigned int n_Frac2_l);
    unsigned int GetN_Frac2_LSB();

    void SetN_Mod2_LSB(unsigned int n_Mod2_l);
    unsigned int GetN_Mod2_LSB();

    void SetN_Frac2_MSB(unsigned int n_Frac2_h);
    unsigned int GetN_Frac2_MSB();

    void SetN_Mod2_MSB(unsigned int n_Mod2_h);
    unsigned int GetN_Mod2_MSB();

    void    SetVcoRange(AdfType chip_type);

    //-------------------------------------------------------------
    // CLASS InternalVcxo  --  In-chip VCXO interface
    //-------------------------------------------------------------

    class InternalVcxo : public IVcxoControl
    {
    public:
        InternalVcxo(ADF4355 & pll)  :  Pll(pll)  {}
        virtual ~InternalVcxo()  {}

        virtual double  Min() {  return FVcoRange_Min;  }
        virtual double  Max() {  return FVcoRange_Max;  }

        virtual bool    CanTuneTo(double freq);
        virtual double  BestTunableFrequency(double freq);

        virtual void    SetVcoFrequency(double freq);
        virtual double  GetVcoFrequency();
        virtual void    SetRange(double /*min*/, double /*max*/)
        {  /* not available  */ }


        void    Frequency(double freq)
        {  SetVcoFrequency(freq);   }
        double  Frequency()
        {  return GetVcoFrequency();   }
        double  FrequencyActual()
        {  return GetVcoFrequency();   }

        void    Load(double r0_min, double r0_max)
        {
            FVcoRange_Min = r0_min;   FVcoRange_Max = r0_max;
        }
    private:
        ADF4355 & Pll;
        double     FVcoRange_Min;
        double     FVcoRange_Max;
    };


    friend class ADF4355::InternalVcxo;

    InternalVcxo        LocalVcxo_Standard;
    IVcxoControl &      LocalVcxo();

    //-------------------------------------------------------------
    // CLASS PllControlRegister  --
    //-------------------------------------------------------------
    class PllControlRegister : public Register
    {
    public:
        typedef Register inherited;
        RegisterBitGroup   Device;
        RegisterBitGroup   RF_PowerDown;
        RegisterBitGroup   EnablePLL;
        RegisterBitGroup   MuxOut;
        RegisterBit        Spi_Ready;

        PllControlRegister(IAddressingSpace &space, int offset)
            :  inherited(space, offset),
              Device(*this, 0, 2),
              RF_PowerDown(*this, 12, 2),
              EnablePLL(*this, 16, 2),
              MuxOut(*this, 24, 2),
              Spi_Ready(*this, 31)
        {  }

    };


    PllControlRegister  ControlReg;
    Register            SpiReg;

    // Fields
    bool                FPowered;

    double              FVcoFrequency;

    //
    //  PLL Device Implementations
    void         FrequencyModeUpdate();
    void         DividerModeUpdate();

    //
    void         SetOutputDividers(double VcoFreq, unsigned int OutDiv);

    virtual bool CheckDivider(unsigned int output, unsigned int divide_by);
    virtual void GetNearestDividers(unsigned int output, float candidate, int & below, int & above);
    unsigned int BestDividerGreaterThan(float min_divide_by);
    unsigned int BestDividerLessThan(float max_divide_by);

    void  InitDividerSupport();

protected:
    typedef std::vector<unsigned int> ValidDividerArray;
    typedef std::map<unsigned int, unsigned int> ValidDividerMap;
    typedef std::map<unsigned int, unsigned int> InverseDividerMap;

    // Mapping for  Output Dividers
    ValidDividerArray    Output_DividerValues;
    ValidDividerMap      Output_ValidDividers;
    InverseDividerMap    Output_InverseDividers;

    bool          IsValidDivider(unsigned int divide_by);
    unsigned int  ToDividerInput(unsigned int divide_by);
    unsigned int  FromDividerInput(unsigned int divider_field);

public:
    Trace         Debug;

private:
    unsigned int N_Mod2_Max();
    unsigned int delay_us;
    unsigned int GetRegCount();
};

#ifdef _MSC_VER
#pragma warning(pop)
#endif

} // namespace Innovative
#endif

