//
// Ad9508_Tb_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Ad9508_Tb_MbH
#define Ad9508_Tb_MbH

#include <TimebaseDevice_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ad9508_Tb  --  PLL timebase control
//=============================================================================

class Ad9508_Tb : public PllTimebaseDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad9508_Tb::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   PllControl_Addr;
        unsigned int   PllSpi_Addr;
    };


    //
    //  Member Functions
    Ad9508_Tb(IAddressingSpace &space, MapRegisters & regs, unsigned int instance);
    virtual ~Ad9508_Tb();

    void   Init();
    void   Reapply();

    int                 MaxOutputDivider() const;

    //  Raw configuration accesses
    virtual void            Port(unsigned short reg, unsigned int value);
    virtual unsigned int    Port(unsigned short reg);

    void OutputSyncPin(bool state);

protected:
    //
    //  Member Data
    std::vector<int>           RegisterImage;
    std::vector<float>         OutputFrequencyLimit;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;

    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    //   These values fill the field for OutputMode for each output. The LvCmos
    //     settings are for half of the field - combine one N and one P setting
    //     for the full configuration
    enum  OutputCfgSettings { ocLvds_0p50, ocLvds_0p75, ocLvds,
                             ocLvds_1p25, ocHstl,  ocHstl_boost,
                             ocCmos, oc_CmosOther
                           };
    enum  OutputDriverPhaseSettings { odpForceHigh, odpNormal, odpInverting, odpForceLow };


    void  SetOutputSync(unsigned int output, bool is_synced);
    void  SetOutputDriverPhaseMode(unsigned int output, unsigned int setting);
    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting, bool should_cache_setting=true);
    void  SetCmosOutputConfig(unsigned int output, bool useP, unsigned int P_phase, bool useN, unsigned int N_Phase)
        {
        SetCmosP_Enabled(output, useP);
        SetCmosP_DriverPhaseMode(output, P_phase);
        SetCmosN_Enabled(output, useN);
        SetCmosN_DriverPhaseMode(output, N_Phase);
        }

    void  SetCmosP_Enabled(unsigned int output, bool enabled);
    void  SetCmosP_DriverPhaseMode(unsigned int output, unsigned int setting);
    void  SetCmosN_Enabled(unsigned int output, bool enabled);
    void  SetCmosN_DriverPhaseMode(unsigned int output, unsigned int setting);



    void  SetPhaseAdjust(unsigned int output, unsigned int divider_setting, unsigned int phase_setting);
    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    //
    //  Internal "Getters"
    unsigned int  GetDivider(unsigned int output);
    bool          GetDividerEnable(unsigned int output);

    void  ReadDataRegisters();
    void  WriteDataRegisters();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllStatusRegister  -- PLL status/SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Sync;
        RegisterBit           SpiReady;
        RegisterBit           SpiDataValid;

        PllStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9508_Tb::PllStatusRegister", space, offset),
              Sync(*this, 8),
              SpiReady(*this, 30), SpiDataValid(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiControlRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup      Address;
        RegisterBit           PllSelect;
        RegisterBit           Rw;
        RegisterBitGroup      Data;

        SpiControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9508_Tb::SpiControlRegister", space, offset),
              Address(*this, 0, 10), PllSelect(*this, 10),
              Rw(*this, 12), Data(*this, 16, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiStatusRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Data;

        SpiStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9508_Tb::SpiStatusRegister", space, offset),
              Data(*this, 24, 8)
            {
            }
    };

    PllStatusRegister   PllStatusReg;
    SpiControlRegister  SpiControlReg;
    SpiStatusRegister   SpiStatusReg;
    unsigned int        Instance;

    //
    //  PLL Device Implementations
    void            SettingsInit();
    void            FrequencyModeUpdate();
    void            DividerModeUpdate();

    //
    void            SetOutputDividers(double VcoFreq, unsigned int OutDiv);

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by);  }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above)
        {  return Ad9508GetNearestDividers(candidate, below, above);  }

protected:

    //double RoundToNearestInt(double number);

    //  Dividers
    bool         IsValidDivider(unsigned int divide_by);
    void         Ad9508GetNearestDividers(float candidate, int& below, int& above);
    unsigned int BestDividerGreaterThan(float min_divide_by);
    unsigned int BestDividerLessThan(float max_divide_by);

    int          OutputOffset(int bank, int offset);

public:
    Trace         Debug;

private:
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

