//
// Ad9510Duo_Tb_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Ad9510Duo_Tb_Mb.h>
#include <SystemSupport_Mb.h>
#include <algorithm>
#include <limits>
#include <cmath>
#include <Magic_Mb.h>
#include <Trace_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
static const double _VcoLow = 100.0e6;
static const double _VcoHigh = 140.0e6;

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ad9510Duo_Tb  --  Cascaded PLL timebase control
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class Ad9510_Tb
//-----------------------------------------------------------------------------

const unsigned int  TotalReferences  = 2;   // Primary PLL reference count
const unsigned int  TotalOutputs     = 4;   // Divider output count

//-----------------------------------------------------------------------------
//  constructor for class Ad9510Duo_Tb
//-----------------------------------------------------------------------------

Ad9510Duo_Tb::Ad9510Duo_Tb(Ad9510_Tb * primary, Ad9508_Tb * divider)
    :  PllTimebaseDevice(TotalReferences, TotalOutputs),
       //FOutputClock(oVco),
       PrimaryPll(primary), Divider(divider),
       FPostDividerStyle(Ad951xAutoTune::pdsAd9508),
       FLimitMasterPll(false),  FMasterPll_MaxOutput(0.0)
{
    //
    //  Set Default Input Configuration
    SourceInUse(0, true);    SetSourceMapping(0, IX6ClockIo::rsExternal);   // primary = external
    SourceInUse(1, true);    SetSourceMapping(1, IX6ClockIo::rsInternal);   // secondary = internal
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


//-----------------------------------------------------------------------------
//  destructor for class Ad9510Duo_Tb
//-----------------------------------------------------------------------------

Ad9510Duo_Tb::~Ad9510Duo_Tb()
{
    delete PrimaryPll;
    PrimaryPll = 0;

	delete Divider;
    Divider = 0;
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::LoadMasterPll()  --  Load master PLL to Duo
//-----------------------------------------------------------------------------

void   Ad9510Duo_Tb::LoadMasterPll(Ad9510_Tb * master)
{
    if (PrimaryPll)
        delete PrimaryPll;
    PrimaryPll = master;
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::LoadSlavePll()  --  Load slave PLL to Duo
//-----------------------------------------------------------------------------

void   Ad9510Duo_Tb::LoadSlavePll(Ad9508_Tb * slave)
{
    if (Divider)
        delete Divider;
    Divider = slave;
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::Init()  --  Init after both PLLs are loaded
//-----------------------------------------------------------------------------

void  Ad9510Duo_Tb::Init()
{
    if (!IsValid())
        return;

    // Copy reference rates from Primary PLL...
    for (unsigned int i=0; i<Sources(); i++)
        {
        ReferenceFrequency(i, PrimaryPll->ReferenceFrequency(i));
        }

    //  Set up Primary PLL
    PrimaryPll->AutoReapply(true);
    PrimaryPll->AllowPostDivider(true);
    PrimaryPll->AutoTuner().PostDividerStyle(FPostDividerStyle);
    PrimaryPll->AutoTuner().EnforceOutputMax(FLimitMasterPll);
    PrimaryPll->AutoTuner().PllOutputMaxRate(FMasterPll_MaxOutput);
    //PrimaryPll->PFDPolarity(true);

    PrimaryPll->Debug.Log("Ad9510Duo_Tb::PostDividerStyle = ", FPostDividerStyle);
    PrimaryPll->Debug.Log("Ad9510Duo_Tb::LimitMasterPll = ", FLimitMasterPll);
    PrimaryPll->Debug.Log("Ad9510Duo_Tb::MasterPll_MaxOutput = ", FMasterPll_MaxOutput);

    PrimaryPll->Init();


    //  Set up Divider
    Divider->AutoReapply(true);
    //Divider->PFDPolarity(true);

    Divider->Init();

}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::Reapply()  --  Reapply settings to hardware from saved state
//-----------------------------------------------------------------------------

void  Ad9510Duo_Tb::Reapply()
{
    if (!IsValid())
        return;

    // Copy input reference rates to Primary PLL references....
    for (unsigned int i=0; i<Sources(); i++)
        {
        PrimaryPll->ReferenceFrequency(i, ReferenceFrequency(i));
        }

    //PrimaryPll->Reference(Reference());
    PrimaryPll->Reapply();

    Divider->Reference(PrimaryPll->EffectiveFrequencyActual());
    Divider->Reapply();

    // Copy output rates from Divider into my Output Rate Actual array....
    for (unsigned int i=0; i<Outputs(); i++)
        {
        OutputFreqActualArray[i] = Divider->OutputFrequencyActual(i);
        }
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::ReferenceSource()  --
//-----------------------------------------------------------------------------

void  Ad9510Duo_Tb::ReferenceSource(Ad9510_Tb::AdReferenceSource value)
{
    if (!IsValid())
        return;
    SetSourceSelection(value);  // inform base of current source
    PrimaryPll->ReferenceSource(value);
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::ReferenceSource()  --
//-----------------------------------------------------------------------------

Ad9510_Tb::AdReferenceSource  Ad9510Duo_Tb::ReferenceSource()
{
    if (!IsValid())
        return Ad9510_Tb::refInternal;
    return PrimaryPll->ReferenceSource();
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::ExtClock_Mode()  --
//-----------------------------------------------------------------------------

void  Ad9510Duo_Tb::ExtClock_Mode()
{
    if (!IsValid())
        return;
    PrimaryPll->ExtClock_Mode();
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::OutputSource()  -- setter
//-----------------------------------------------------------------------------

void  Ad9510Duo_Tb::OutputSource(Ad9510_Tb::AdOutputSource value)
{
    UpdateMode = (value==Ad9510_Tb::outPll) ? updFrequency : updDivider;
    PrimaryPll->OutputSource(value);
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::OutputSource()  -- getter
//-----------------------------------------------------------------------------

Ad9510_Tb::AdOutputSource   Ad9510Duo_Tb::OutputSource() const
{
    if (!IsValid())
        return Ad9510_Tb::outPll;
    return PrimaryPll->OutputSource();
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::ExternalClockSelect()  --
//-----------------------------------------------------------------------------

void  Ad9510Duo_Tb::ExternalClockSelect(Ad9510_Tb::AdExternalSelect value)
{
    if (!IsValid())
        return;
    PrimaryPll->ExternalClockSelect(value);
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::ExternalClockSelect()  --
//-----------------------------------------------------------------------------

Ad9510_Tb::AdExternalSelect  Ad9510Duo_Tb::ExternalClockSelect() const
{
    if (!IsValid())
        return Ad9510_Tb::csFrontPanel;
    return PrimaryPll->ExternalClockSelect();
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::FrequencyModeUpdate()  --
//-----------------------------------------------------------------------------

void Ad9510Duo_Tb::FrequencyModeUpdate()
{
    // Copy Sources to Primary
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::DividerModeUpdate()  --
//-----------------------------------------------------------------------------

void Ad9510Duo_Tb::DividerModeUpdate()
{
}

void Ad9510Duo_Tb::SetAdcOutputRates(double adc_rate)
{
    if (!IsValid())
        return;
    unsigned int multiplier = PrimaryPll->AdcMultiplierFactor();
    PrimaryPll->Adc().Frequency(adc_rate/multiplier);  // don't "multiplier" twice
    Divider->Reference(PrimaryPll->EffectiveFrequencyActual());
    Divider->Adc().Frequency(adc_rate);

    // Copy output rates from Divider into my Output Rate array....
    for (unsigned int i=0; i<Outputs(); i++)
        {
        OutputFrequency(i, Divider->OutputFrequency(i));
        OutputFreqActualArray[i] = Divider->OutputFrequencyActual(i);
        }
}

void Ad9510Duo_Tb::SetDacOutputRates(double dac_rate)
{
    if (!IsValid())
        return;

    unsigned int multiplier = PrimaryPll->DacMultiplierFactor();
    PrimaryPll->Dac().Frequency(dac_rate/multiplier); // don't "multiplier" twice
    Divider->Reference(PrimaryPll->EffectiveFrequencyActual());
    Divider->Dac().Frequency(dac_rate);

    // Copy output rates from Divider into my Output Rate array....
    for (unsigned int i=0; i<Outputs(); i++)
        {
        OutputFrequency(i, Divider->OutputFrequency(i));
        OutputFreqActualArray[i] = Divider->OutputFrequencyActual(i);
        }
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::Decimation()  --  Return decimation factor
//-----------------------------------------------------------------------------

int	Ad9510Duo_Tb::Decimation() const
{
    if (!IsValid())
        return 1;

    return PrimaryPll->AutoTuner().Decimation();
}

void Ad9510Duo_Tb::SetDividerHdw(unsigned int /*output*/, unsigned int /*divide_by*/)
{

}

bool Ad9510Duo_Tb::CheckDivider(unsigned int /*output*/, unsigned int /*divide_by*/)
{
    return true;
}

void Ad9510Duo_Tb::GetNearestDividers(unsigned int /*output*/, float /*candidate*/, int& /*below*/, int& /*above*/)
{
    //if (IsValid())
    //    return Divider->GetNearestDividers(output, candidate, below, above);
}

void Ad9510Duo_Tb::SetOutEnabledHdw(unsigned int /*output*/, bool /*enabled*/)
{

}

unsigned int Ad9510Duo_Tb::AdcMasterOutput()
    {
    if (!IsValid())
        return 0;
    else
        return Divider->AdcMasterOutput();
    }

unsigned int Ad9510Duo_Tb::DacMasterOutput()
{
    if (!IsValid())
        return 0;
    else
        return Divider->DacMasterOutput();
}

unsigned int Ad9510Duo_Tb::AdcMultiplierFactor()
{
    if (!IsValid())
        return 1;
    else
        return PrimaryPll->AdcMultiplierFactor();
}

unsigned int Ad9510Duo_Tb::DacMultiplierFactor()
{
    if (!IsValid())
        return 1;
    else
        return PrimaryPll->DacMultiplierFactor();
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::DoSetFrequency()  --  Calculate settings for a frequency
//-----------------------------------------------------------------------------
/*
void Ad9510Duo_Tb::DoSetFrequency(double freq)
{
	// Adjust Frequency to accomodate clock factor
	freq *= FClockFactor;

	Tuner.ReferenceRate(Reference());
	Tuner.SampleRate(freq);

	Tuner.Calculate();
	//
    //  Apply Reference value to Clock
    PrimaryPll->Reference(Reference());
    PrimaryPll->Debug.Log("DUO Input Frequency =", freq);
	// Drive the master clock at upper end of usable range to minimize jitter
	double f = (FOutputClock==oVco) ? Tuner.ActualSampleRateBeforePostDivider() : Reference();
    PrimaryPll->Frequency(f);
    PrimaryPll->Debug.Log("DUO Clock Actual =", PrimaryPll->FrequencyActual());
	// Drive the slave clock at the requested rate
	Divider->Reference(Clock->FrequencyActual());
    PrimaryPll->Debug.Log("DUO Divider Freq =", freq);
	double f2 = (FOutputClock==oVco) ? Tuner.ActualSampleRateBeforeDecimation() : freq;
	Divider->Frequency(f2);

    PrimaryPll->Debug.Log("DUO Divider Actual =", Divider->FrequencyActual());
    PrimaryPll->Debug.Log("DUO Divider Ref =", Divider->Reference());
}
*/



//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::Locked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  Ad9510Duo_Tb::Locked() const
{
    return PrimaryPll->Locked();
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::Port()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void Ad9510Duo_Tb::Port(int device, unsigned short reg, unsigned int value)
{
	if (device)
		Divider->Port(reg, value);
	else
        PrimaryPll->Port(reg, value);
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::Port()  --  Read from PLL data port
//-----------------------------------------------------------------------------

unsigned int Ad9510Duo_Tb::Port(int device, unsigned short reg)
{
	unsigned char result;

	if (device)
		result = Divider->Port(reg);
	else
        result = PrimaryPll->Port(reg);

	return result;
}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::PllTuningEvent()  --
//-----------------------------------------------------------------------------

//void Ad9510Duo_Tb::HandlePllTuning (PllTuningEvent& e)
//{
//    OnPllTuning.Execute(e);
//}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::PllDividerEvent()  --
//-----------------------------------------------------------------------------

//void Ad9510Duo_Tb::HandlePllDividing (PllDividerEvent& e)
//{
//    OnPllDividing.Execute(e);
//}

//-----------------------------------------------------------------------------
//  Ad9510Duo_Tb::IsValid()  -- check for valid subcomponents
//-----------------------------------------------------------------------------

bool Ad9510Duo_Tb::IsValid() const
{
    if (PrimaryPll==0 || Divider==0)
        return false;
    else
        return true;
}

} // namespace Innovative
