//
// Ad9510Duo_Tb_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Ad9510Duo_Tb_MbH
#define Ad9510Duo_Tb_MbH

#include <Ad9510_Tb_Mb.h>
#include <Ad9508_Tb_Mb.h>
#include <X6Support_Mb.h>

namespace Innovative
{


#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ad9510Duo_Tb  --  Cascaded PLL timebase control
//=============================================================================

class Ad9510Duo_Tb : public PllTimebaseDevice
{

public:
	//
	//  Member Functions
    Ad9510Duo_Tb(Ad9510_Tb * primary=0, Ad9508_Tb * divider=0);
    virtual ~Ad9510Duo_Tb();

    void   LoadMasterPll(Ad9510_Tb * master);
    void   LoadSlavePll(Ad9508_Tb * slave);

    void   PostDividerStyle( Ad951xAutoTune::PostDividerStyles style)
                {  FPostDividerStyle = style;   }
    void   MasterPll_MaxOutput( double rate )
                {  FLimitMasterPll = true, FMasterPll_MaxOutput = rate;   }
    void   MasterPll_NoLimit()
                {  FLimitMasterPll = false, FMasterPll_MaxOutput = 0.0;   }

    // Methods
    void                Init();
    void                Reapply();

    //  Select the Input reference
    void  ReferenceSource(Ad9510_Tb::AdReferenceSource value);
    Ad9510_Tb::AdReferenceSource  ReferenceSource();

    //  Select Output Mode
    bool  IsIn_Pll_Mode()
             {  return OutputSource() == Ad9510_Tb::outPll;  }
    bool  IsIn_ExtClock_Mode()
             {  return IsIn_Pll_Mode() == false;  }
    void  Pll_Mode()
            {  OutputSource(Ad9510_Tb::outPll);  }
    void  ExtClock_Mode();

    void  OutputSource(Ad9510_Tb::AdOutputSource value);
    Ad9510_Tb::AdOutputSource  OutputSource() const;

    void  ExternalClockSelect(Ad9510_Tb::AdExternalSelect value);
    Ad9510_Tb::AdExternalSelect  ExternalClockSelect() const;

    virtual void  FrequencyModeUpdate();
    virtual void  DividerModeUpdate();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

	//  Raw configuration accesses
    virtual void 			Port(int device, unsigned short reg, unsigned int value);
    virtual unsigned int    Port(int device, unsigned short reg);

    bool				Locked() const;
    int		            Decimation() const;


    Ad9510_Tb *     Master()  {  return PrimaryPll;  }
    Ad9508_Tb *     Slave()   {  return Divider;     }

    //
    //  Supports modified Clock Factors
    virtual unsigned int AdcMultiplierFactor(); // "Clock Factor" - device needs faster rate than sample rate
    virtual unsigned int DacMultiplierFactor();

protected:

    //
    //  Derived Class
    virtual void  SetDividerHdw(unsigned int output, unsigned int divide_by);
    virtual bool  CheckDivider(unsigned int output, unsigned int divide_by);
    virtual void  GetNearestDividers(unsigned int output, float candidate,
                                    int & below,
                                    int & above);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

private:
    // Fields
    Ad9510_Tb *     PrimaryPll;
    Ad9508_Tb *     Divider;

    Ad951xAutoTune::PostDividerStyles  FPostDividerStyle;
    bool                               FLimitMasterPll;
    double                             FMasterPll_MaxOutput;

    bool IsValid() const;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative

#endif

