//
// Ad9510_Tb_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Ad9510_Tb_Mb.h>
#include <SystemSupport_Mb.h>
#include <algorithm>
#include <limits>
#include <cmath>
#include <Magic_Mb.h>
#include <Trace_Mb.h>
#include <sstream>
#include <FloatingPtSupport_Mb.h>
#include <X6Support_Mb.h>

//---------------------------------------------------------------------------
static const double _VcoLow = 100.0e6;
static const double _VcoHigh = 140.0e6;

namespace Innovative
{

//=============================================================================
//  CLASS Ad9510_Tb  --  Ad9510_Tb Pll Implementation
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class Ad9510_Tb
//-----------------------------------------------------------------------------

const unsigned int  TotalReferences  = 2;
const unsigned int  TotalOutputs     = 8;
const unsigned int  Total_Bank0_LvPecl_Outputs = 4;
const unsigned int  Total_Bank1_LvPecl_Outputs = 4;

const int           SpiRegisterCount = 0x2D;
const float         DefaultMaxOutFreq = 1650.0e6;

const unsigned int  kMaxOutputDivider = 32;

//-----------------------------------------------------------------------------
//  constructor for class Ad9510_Tb
//-----------------------------------------------------------------------------

Ad9510_Tb::Ad9510_Tb(IAddressingSpace &space, MapRegisters & regs, unsigned int instance)
    : PllTimebaseDevice(TotalReferences, TotalOutputs),
      RegisterImage(SpiRegisterCount),
      Vco(new RangedVcxoControl(_VcoLow, _VcoHigh)),
      Tuner(Vco),
      Instance(instance),
      FReferenceSource(refInternal), FOutputSource(outPll),
      FExternalClockSrc(csFrontPanel),
      PllControlReg(space, regs.PllControl_Addr),
      PllStatusReg(space,  regs.PllControl_Addr),
      SpiControlReg(space, regs.PllSpi_Addr),
      SpiStatusReg(space,  regs.PllSpi_Addr),
      Debug(true)
{
    ConfigureDividerInformation();

    Tuner.ReferenceRate(Reference());
    Tuner.Multiplier(1);
    Tuner.AllowDecimation(false);

    for (unsigned int i=0; i<TotalOutputs; i++)
        {
        OutputFrequencyLimit.push_back(DefaultMaxOutFreq);
        }
    //
    //  Set Default Input Configuration
    SourceInUse(0, true);    SetSourceMapping(0, IX6ClockIo::rsExternal);   // primary = external
    SourceInUse(1, true);    SetSourceMapping(1, IX6ClockIo::rsInternal);   // secondary = internal

}

//-----------------------------------------------------------------------------
//  destructor for class Ad9510_Tb
//-----------------------------------------------------------------------------

Ad9510_Tb::~Ad9510_Tb()
{
    delete Vco;
    Vco=0;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Init()  --  Initialize Chip
//-----------------------------------------------------------------------------

void Ad9510_Tb::Init()
{
    Settings.Pll.PhaseDetectorFrequency = DefaultPhaseDetectorFrequency();    

    InitSerialPort();      // initialize device serial port

    //ReadDataRegisters();   // read initial state from chip
    SettingsInit();        // common INIT settings
    OnInit();
    Update();              // Update state of the device based on sources

    //WriteDataRegisters();
    SetReg08(Settings.Pll.R08);
    SetReg09(Settings.Pll.ChargePumpCurrent);
    SetReg0D(Settings.Pll.R0D);

    UpdateRegisters();     // activate new settings

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SettingsInit()  --  Initial Settings Loading
//-----------------------------------------------------------------------------
//   Note board specific implementations can override these in OnInit

void Ad9510_Tb::SettingsInit()
{
    //  Default LvPecl voltage level to chip default
    for (unsigned int i=0; i<Total_Bank0_LvPecl_Outputs; i++)
        SetOutVoltageHdw(i, v810mv);
    //  Default Lvds current level to chip default
    for (unsigned int i=Total_Bank0_LvPecl_Outputs; i<TotalOutputs; i++)
        SetOutCurrentHdw(i, c3500uA);


}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Reapply()  --  Reapply settings to hardware from saved state
//-----------------------------------------------------------------------------

void Ad9510_Tb::Reapply()
{
    InitSerialPort();      // initialize device serial port

    SetOutputSource(FOutputSource);   // reapply output source

    OnReapply();
    Update();    // Update state of the device based on sources

    //WriteDataRegisters();
    SetReg08(Settings.Pll.R08);
    SetReg09(Settings.Pll.ChargePumpCurrent);
    SetReg0D(Settings.Pll.R0D);

    // Apply output configuration
    for (unsigned int i=0; i<Settings.Out_Bank0.size(); i++)
        Port(0x3C+i, Settings.Out_Bank0[i].OutputCfgReg);
    for (unsigned int i=0; i<Settings.Out_Bank1.size(); i++)
        Port(LvdsDividerOffset(Total_Bank0_LvPecl_Outputs+i), Settings.Out_Bank1[i].OutputCfgReg);

    UpdateRegisters();     // activate new settings
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::ExternalClockSelect()  --  Set external clock selection
//-----------------------------------------------------------------------------

void  Ad9510_Tb::ExternalClockSelect(AdExternalSelect value)
{
    FExternalClockSrc = value;
    SetExternalClockSelect(value);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::ExternalClockSelect()  --  Read external clock selection
//-----------------------------------------------------------------------------

Ad9510_Tb::AdExternalSelect  Ad9510_Tb::ExternalClockSelect() const
{
    return FExternalClockSrc;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::FrequencyModeUpdate()  --  Update using Frequency as baseline
//-----------------------------------------------------------------------------

void Ad9510_Tb::FrequencyModeUpdate()
{
    ScopedTraceRef ccc(Debug, "Ad9510_Tb::FrequencyModeUpdate()");

    // Use MasterFreq here, as it has the Multiplier factored in
    if ((MasterOutputFrequency() == 0.0) || (Reference() == 0.0))
        return;

    //
    //  Configure Tuner to Calculate New Frequency
    //Tuner.AllowOddDivisors(Settings.Pll.AllowAsymmetricDutyCycle);
    Tuner.ReferenceRate(Reference());
    Tuner.SampleRate(MasterOutputFrequency());   // or set Tuner.Multiplier and use Frequency()?
    // Ensure 50/50 duty cycle output - how?

    Tuner.Calculate();

    int R       = Tuner.R();
    int P       = Tuner.P();
    int P_bits  = Tuner.P_bits();
    int B       = Tuner.B();
    int A       = Tuner.A();
    int OD      = Tuner.OutputDivider();

    Debug.Log("Ad9510_Tb::R = ", R);
    Debug.Log("Ad9510_Tb::P = ", P);
    Debug.Log("Ad9510_Tb::P_bits = ", P_bits);
    Debug.Log("Ad9510_Tb::B = ", B);
    Debug.Log("Ad9510_Tb::A = ", A);
    Debug.Log("Ad9510_Tb::OD = ", OD);
    Debug.Log("Ad9510_Tb::PostDivider = ", Tuner.PostDividerValue());
    Debug.Log("Ad9510_Tb::Decimation = ", Tuner.Decimation());

    Debug.Log("Pll Tuning Event");
    Ad9510Tb_PllTuningEvent e(R, P_bits, B, A, OD);
    OnPllTuning.Execute(e);

    P = PBitsToP(P_bits);
//    Debug.Log("Hard Code Override R, B");
//    R = 100;
//    B = 1400;


    Debug.Log("Ad9510_Tb::R = ", R);
    Debug.Log("Ad9510_Tb::P = ", P);
    Debug.Log("Ad9510_Tb::P_bits = ", P_bits);
    Debug.Log("Ad9510_Tb::B = ", B);
    Debug.Log("Ad9510_Tb::A = ", A);
    Debug.Log("Ad9510_Tb::OutDiv = ", OD);
//    Debug.Log("Ad9510_Tb::PostDivider = ", Tuner.PostDividerValue());
//    Debug.Log("Ad9510_Tb::Decimation = ", Tuner.Decimation());

    // Cache Resultant Information
    Settings.Pll.VcoFrequency                  = Tuner.ActualVcoRate();
    Settings.Pll.PhaseDetectorFrequency        = Tuner.PfdRate();
    Settings.Pll.PhaseDetectorFrequency_Actual = Tuner.PfdRateActual();
    Settings.Pll.ActualRateBeforePostDivider   = Tuner.ActualSampleRateBeforePostDivider();
    Settings.Pll.ActualRateBeforeDecimation    = Tuner.ActualSampleRateBeforeDecimation();

    Debug.Log("Ad9510_Tb::VcoFrequency = ", Settings.Pll.VcoFrequency);
    Debug.Log("Ad9510_Tb::PFD Rate = ", Settings.Pll.PhaseDetectorFrequency);
    Debug.Log("Ad9510_Tb::PFD Rate Actual= ", Settings.Pll.PhaseDetectorFrequency_Actual);

    Debug.Log("Ad9510_Tb::Decimation = ", Tuner.Decimation());

    // Set the Vco to the frequency desired
    Vco->SetVcoFrequency(Settings.Pll.VcoFrequency);

    SetA(A);
    SetB(B);
    SetP(P, B);
    SetR(R);

    SetOutputDividers(Settings.Pll.VcoFrequency, OD);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::DividerModeUpdate() -- Update state of dividers
//-----------------------------------------------------------------------------
//  In this mode, the reference is sent to the outputs, with possible division

void  Ad9510_Tb::DividerModeUpdate()
{
    ScopedTraceRef ccc(Debug, "Ad9510_Tb::DividerModeUpdate()");
    // Use MasterFreq here, as it has the Multiplier factored in
    if ((MasterOutputFrequency() == 0.0) || (Reference() == 0.0))
        return;

    //
    //  Configure Tuner to Calculate Dividers, Post Dividers and Decimation
    //Tuner.AllowOddDivisors(Settings.Pll.AllowAsymmetricDutyCycle);
    Tuner.ReferenceRate(Reference());
    Tuner.SampleRate(MasterOutputFrequency());   // or set Tuner.Multiplier and use Frequency()?

    Tuner.Calculate_External();

    unsigned int OD = Tuner.OutputDivider();

    Debug.Log("Ad9510_Tb::OutDiv = ", OD);
    Debug.Log("Ad9510_Tb::PostDivider = ", Tuner.PostDividerValue());
    Debug.Log("Ad9510_Tb::Decimation = ", Tuner.Decimation());

    SetOutputDividers(Reference(), OD);

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutputDividers()  --
//-----------------------------------------------------------------------------
/*
 *


    // encode outdiv for register use
    int outdiv_bits = ToDividerValue(outdiv);
    Debug.Log("Ad9510_Tb outdiv bits: ", outdiv_bits);

    // disable output dividers if outdiv = 1
    int outdiv_bypass = ToBypassValue(outdiv);


    ConfigureOutputs(outdiv_bits, outdiv_bypass);
    Debug.Log("Pll Divider Event");

*/

void  Ad9510_Tb::SetOutputDividers(double VcoFreq, unsigned int OutDiv)
{
    unsigned int BestDivider = OutDiv;
    double       OutputFreq  = VcoFreq/BestDivider;

    for (unsigned int output=0; output<TotalOutputs; output++)
        {
        if (OutputInUse(output)==true)
            {
            float   CandidateDivider = static_cast<float>(BestDivider);
            double  CandidateFreq    = OutputFreq;
            //
            //  If the Output is the Master, use the best values.
            //     if not the master, and Frequency is unspecified, then
            //     the make the output a copy of Master.
            //     Otherwise, figure the best divider possible for each output
            if ((output!=FMasterOutput) && (OutputFrequency(output)!=0.0))
                {
                CandidateFreq = OutputFrequency(output);
                CandidateDivider = static_cast<float>(VcoFreq / CandidateFreq);

                int above, below, divider;
                GetNearestDividers(output, CandidateDivider, below, above);

                // Find which divider is closest to desired frequency
                double centerFreq = VcoFreq/CandidateDivider;
                double hiFreq     = VcoFreq/below;
                double lowFreq    = VcoFreq/above;
                if (std::abs(centerFreq-lowFreq) > std::abs(hiFreq-centerFreq))
                    divider = below;
                else
                    divider = above;

                CandidateDivider = static_cast<float>(divider);
                }

            //
            //  Check for frequency exceeding applied Limits
            if (OutputFrequencyLimit[output]>=CandidateFreq)
                OutputDivider(output, static_cast<unsigned int>(CandidateDivider));
            else
                {
                double MinDivideRatio = VcoFreq/OutputFrequencyLimit[output];
                OutputDivider(output, BestDividerGreaterThan(static_cast<float>(MinDivideRatio)));
                }
            //
            //  Calculate Acutal Frequency
            if (OutputDividerArray[output]!=0)  // if illegal divider, skip it
                {
                OutputFreqActualArray[output] = VcoFreq / OutputDividerArray[output];
                }
            }
        else
            {
            OutputFreqActualArray[output] = 0.0;
            }
        }
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::LoadVco()  --  Replace Vco with a new Vco object
//-----------------------------------------------------------------------------

void Ad9510_Tb::LoadVco(IVcxoControl * aVco)
{
    if (aVco)   // we need a valid Vco object to work
        {
        IVcxoControl * tmp = Vco;
        Vco = aVco;
        Tuner.LoadVco(Vco);
        delete tmp;
        }
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Decimation()  --  Return decimation factor
//-----------------------------------------------------------------------------

int	Ad9510_Tb::Decimation() const
{
    return Tuner.Decimation();  // set to 1 if decimation not allowed
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::MaxOutputDivider()  --  Return max output post-divider
//-----------------------------------------------------------------------------

int Ad9510_Tb::MaxOutputDivider() const
{
    return kMaxOutputDivider;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Locked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  Ad9510_Tb::Locked() const
{
    return PllStatusReg.Locked.Value();
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::PFDPolarity()  --  Set Phase-Frequency Detector (PFD) Polarity
//                             0 = Negative (Default), 1 = Positive.
//-----------------------------------------------------------------------------

void Ad9510_Tb::PFDPolarity(bool state)
{
    Settings.Pll.R08.PFDPolarity = state;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::PFDPolarity()  --  Return Phase-Frequency Detector (PFD) Polarity
//-----------------------------------------------------------------------------

bool Ad9510_Tb::PFDPolarity() const
{
    return Settings.Pll.R08.PFDPolarity;
}

////-----------------------------------------------------------------------------
////  Ad9510_Tb::AllowAsymmetricDutyCycle()  --
////-----------------------------------------------------------------------------

//void Ad9510_Tb::AllowAsymmetricDutyCycle(bool state)
//{
//    Settings.Pll.AllowAsymmetricDutyCycle = state;
//}

////-----------------------------------------------------------------------------
////  Ad9510_Tb::AllowAsymmetricDutyCycle()  --
////-----------------------------------------------------------------------------

//bool Ad9510_Tb::AllowAsymmetricDutyCycle() const
//{
//    return Settings.Pll.AllowAsymmetricDutyCycle;
//}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::ChargePumpCurrent()  --  Set charge-pump current mode
//-----------------------------------------------------------------------------

void Ad9510_Tb::ChargePumpCurrent(AdCpCurrent mode)
{
    Settings.Pll.ChargePumpCurrent = mode;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::ChargePumpCurrent()  --  Query charge-pump current mode
//-----------------------------------------------------------------------------

Ad9510_Tb::AdCpCurrent	Ad9510_Tb::ChargePumpCurrent() const
{
    return Settings.Pll.ChargePumpCurrent;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetReferenceSource()  --
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetReferenceSource(Ad9510_Tb::AdReferenceSource source)
{
    PllControlReg.ReferenceSource.Value((source == refExternal) ? 0 : 1);

    Debug.Log("Ad9510_Tb::SetReferenceSource()");
    Debug.Log("Control Value()", PllControlReg.Value(), 16);
    Debug.Log("Status Value()", PllStatusReg.Value(), 16);

    SetSourceSelection(source);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutputSource()  --
//-----------------------------------------------------------------------------

void  Ad9510_Tb::SetOutputSource(Ad9510_Tb::AdOutputSource source)
{
    // all clock inputs on, PLL output is as selected
    unsigned char ClkOut = (source==outPll)? 0 : 1;
    Port(0x45, ClkOut);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetExternalClockSelect()  --  Set the Source for the divider matrix
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetExternalClockSelect(AdExternalSelect source)
{
    PllControlReg.ExtClockSource.Value((source == csFrontPanel) ? 1 : 0);

    Debug.Log("Ad9510_Tb::SetExternalClockSelect()");
    Debug.Log("Control Value()", PllControlReg.Value(), 16);
    Debug.Log("Status Value()", PllStatusReg.Value(), 16);
}



//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetDividerHdw()  --  Set divider register
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetDividerHdw(unsigned int output, unsigned int divider_setting)
{
    if (!IsValidDivider(divider_setting))
        return;

    if (output < TotalOutputs)
        {
        int divider_bits = ToDividerValue(divider_setting);
        Port(OutputDividerOffset(output, 0), divider_bits);

        int bypass_bits = ToBypassValue(divider_setting);
        Port(OutputDividerOffset(output, 1), bypass_bits);
        }
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::OutputDividerOffset()  -- Find register address for divider settings
//-----------------------------------------------------------------------------

int  Ad9510_Tb::OutputDividerOffset(int bank, int offset)
{
    int base;
    switch(bank)
        {
        default:
        case 0:   base=0x48;  break;
        case 1:   base=0x4a;  break;
        case 2:   base=0x4c;  break;
        case 3:   base=0x4e;  break;
        case 4:   base=0x50;  break;
        case 5:   base=0x52;  break;
        case 6:   base=0x54;  break;
        case 7:   base=0x56;  break;
        }

    return base+offset;
}


//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutputConfig()  --  Set Bank 1 output configuration
//-----------------------------------------------------------------------------
//  If output is on bank 0, skip config

void  Ad9510_Tb::SetOutputConfig(unsigned int output, unsigned int cfg_setting)
{
    if (output >= TotalOutputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)  // no configuration for LvPecl bank
        return;

    unsigned int lvpecl_idx = output - Total_Bank0_LvPecl_Outputs;   // adjust to bank 1 base

    //  write output config, bank 1
    PtrBitManipulator Reg(&Settings.Out_Bank1[lvpecl_idx].OutputCfgReg);
    Reg.Bit(3, cfg_setting ? true : false);

    // Apply to hardware
    Port(LvdsDividerOffset(output), Reg.Value());
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutVoltageHdw()  --  Apply the voltage level state to hdw
//-----------------------------------------------------------------------------
//  Bank 0 only (LvPecl)

void Ad9510_Tb::SetOutVoltageHdw(unsigned int output, unsigned int volt_level)
{
    if (output >= Total_Bank0_LvPecl_Outputs)
        return;

    PtrBitManipulator Reg(&Settings.Out_Bank0[output].OutputCfgReg);
    Reg.Field(2, 2, volt_level);

    // Apply to hardware
    Port(0x3C+output, Reg.Value());

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutCurrentHdw()  --  Apply the current state to hdw
//-----------------------------------------------------------------------------
//  Bank 1 only (Lvds)

void Ad9510_Tb::SetOutCurrentHdw(unsigned int output, unsigned int current)
{
    if (output >= TotalOutputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)
        return;

    output -= Total_Bank0_LvPecl_Outputs;

    PtrBitManipulator Reg(&Settings.Out_Bank1[output].OutputCfgReg);
    Reg.Field(1, 2, current);

    // Apply to hardware
    Port(0x40+output, Reg.Value());

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutEnabledHdw()  --  Apply the enable/disable state to hdw
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetOutEnabledHdw(unsigned int output, bool enabled)
{
    if (output >= TotalOutputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)
        SetOutEnabledHdw_Bank0(output, enabled);
    else
        SetOutEnabledHdw_Bank1(output-Total_Bank0_LvPecl_Outputs, enabled);

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutEnabledHdw_Bank0()  --  Bank 0 Impl
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetOutEnabledHdw_Bank0(unsigned int output, bool enabled)
{
    // output valididty already checked
    unsigned int cfg_setting = (enabled) ? 0x0 : 0x3 ;

    PtrBitManipulator Reg(&Settings.Out_Bank0[output].OutputCfgReg);
    Reg.Field(0, 2, cfg_setting);

    // Apply to hardware
    Port(0x3C+output, Reg.Value());
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetOutEnabledHdw_Bank1()  --  Bank 1 Impl
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetOutEnabledHdw_Bank1(unsigned int output, bool enabled)
{
    // output valididty already checked
    PtrBitManipulator Reg(&Settings.Out_Bank1[output].OutputCfgReg);
    Reg.Bit(0, !enabled);

    // Apply to hardware
    Port(0x40+output, Reg.Value());
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::LvdsDividerOffset()  -- Find register address for Lvds divider settings
//-----------------------------------------------------------------------------

int  Ad9510_Tb::LvdsDividerOffset(int bank)
{
    int base;
    switch(bank)
        {
        default:
        case 4:   base=0x40;  break;
        case 5:   base=0x41;  break;
        case 6:   base=0x42;  break;
        case 7:   base=0x43;  break;
        }

    return base;
}


//-----------------------------------------------------------------------------
//  Ad9510_Tb::InitSerialPort()  --
//-----------------------------------------------------------------------------

void Ad9510_Tb::InitSerialPort()
{
    // initialize device serial port
    Port(0x00, 0xB0);
    Port(0x00, 0x90);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::UpdateRegisters()  --  update registers
//-----------------------------------------------------------------------------

void Ad9510_Tb::UpdateRegisters()
{
    Port(0x58, 0x04);     // set soft SYNC
    Port(0x5A, 0x01);
    Port(0x58, 0x00);     // Release soft SYNC
    Port(0x5A, 0x01);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetR()  --  Set PLL R value
//-----------------------------------------------------------------------------

void  Ad9510_Tb::SetR(unsigned int R)
{
    Port(0x0C, (R & 0xff));      // R LSBs
    Port(0x0B, (R >> 8));        // R MSBs
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetP()  --  Set PLL P value
//-----------------------------------------------------------------------------

void  Ad9510_Tb::SetP(unsigned int P, unsigned int B)
{
    // encode P value for register use
    unsigned int P_bits;
    switch(P)
        {
        default:
        case  1:    P_bits = 0;        break;
        case  2:    P_bits = 1;        break;
        case  3:    P_bits = 7;        break;
        case  4:    P_bits = 3;        break;
        case  8:    P_bits = 4;        break;
        case 16:    P_bits = 5;        break;
        case 32:    P_bits = 6;        break;
        }
    Port(0x0A, ((B == 1) ? 0x40 : 0x0) | (P_bits << 2));
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetA()  --  Set PLL P value
//-----------------------------------------------------------------------------

void  Ad9510_Tb::SetA(unsigned int A)
{
      // Load A register
    Port(0x04, A);				// A
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetB()  --  Set PLL B value
//-----------------------------------------------------------------------------

void  Ad9510_Tb::SetB(unsigned int B)
{
    // Load both portions of B register
    Port(0x05, (B >> 8));		// B MSBs
    Port(0x06, (B & 0xff));		// B LSBs
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetReg08()  --  Set Register 08 value
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetReg08(const Ad9510_Tb::Register08Fields& r08)
{
    MemoryBitManipulator R08(0);

    // assemble fields
    R08.Bit(6, r08.PFDPolarity);
    R08.Field(2, 4, r08.StatusPinCfg);
    R08.Field(0, 2, r08.ChargePumpMode);

    // write to hardware
    Port(0x08, R08.Value());
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetReg08()  --  Set Register 08 value
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetReg09(unsigned int charge_pump_current_setting)
{
    MemoryBitManipulator R09(0);

    // assemble fields
    R09.Field(4, 3, charge_pump_current_setting);

    // write to hardware
    Port(0x09, R09.Value());

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::SetReg0D()  --  Set Register 08 value
//-----------------------------------------------------------------------------

void Ad9510_Tb::SetReg0D(const Register0DFields & r0d)
{
    MemoryBitManipulator R0D(0);

    // assemble fields
    R0D.Field(0, 2, r0d.AntiBacklash);
    R0D.Bit(5, r0d.ShortDLD);
    R0D.Bit(6, r0d.DisableDLD);

    // write to hardware
    Port(0x0d, R0D.Value());

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Port()  --  Write to device data port
//-----------------------------------------------------------------------------

void Ad9510_Tb::Port(unsigned short reg, unsigned int value)
{
    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !PllStatusReg.SpiReady.Value(); ++i)
        Sleep(1);
    if (!PllStatusReg.SpiReady.Value())
        Debug.Log("PLL SPI WR ready timeout error!");

    bool device_bit = (Instance) ? true : false ;  // Dev 0 -> bit is 0

    SpiControlReg.Address.Value(reg);
    SpiControlReg.Data.Value(value);
    SpiControlReg.Rw.Value(0);
    SpiControlReg.PllSelect.Value(device_bit);
    SpiControlReg.Apply();

    std::stringstream msg;
    msg << "Ad9510_Tb::Port[0x" << std::hex << static_cast<int>(reg)
        << "]=0x" << static_cast<int>(value);
    Debug.Log(msg.str());

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Port()  --  Read from device data port
//-----------------------------------------------------------------------------

unsigned int Ad9510_Tb::Port(unsigned short reg)
{
    unsigned char result;

    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !PllStatusReg.SpiReady.Value(); ++i)
        Sleep(1);
    if (!PllStatusReg.SpiReady.Value())
        Debug.Log("PLL SPI RD ready timeout error!");

    bool device_bit = (Instance) ? true : false ;  // Dev 0 -> bit is 0

    SpiControlReg.Address.Value(reg);
    SpiControlReg.Rw.Value(1);
    SpiControlReg.PllSelect.Value(device_bit);
    SpiControlReg.Apply();

    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !PllStatusReg.SpiDataValid.Value(); ++i)
        Sleep(1);
    if (!PllStatusReg.SpiDataValid.Value())
        Debug.Log("PLL SPI RD data valid timeout error!");

    result = SpiStatusReg.Data.Value();

    return result;
    }

//-----------------------------------------------------------------------------
//  Ad9510_Tb::ConfigureDividerInformation()
//-----------------------------------------------------------------------------

void Ad9510_Tb::ConfigureDividerInformation()
{
    ValidDividerValues.push_back(1);  ValidDividerValues.push_back(2);
    ValidDividerValues.push_back(4);  ValidDividerValues.push_back(6);
    ValidDividerValues.push_back(8);  ValidDividerValues.push_back(10);
    ValidDividerValues.push_back(12); ValidDividerValues.push_back(14);
    ValidDividerValues.push_back(16); ValidDividerValues.push_back(18);
    ValidDividerValues.push_back(20); ValidDividerValues.push_back(22);
    ValidDividerValues.push_back(24); ValidDividerValues.push_back(26);
    ValidDividerValues.push_back(28); ValidDividerValues.push_back(30);
    ValidDividerValues.push_back(32);

    //  Fill the conversion information structures
    //     Valid dividers is keyed by divide_by, value is Input Value
    Ad9510_ValidDividers[1]  = 0xF0;       Ad9510_ValidDividers[2]  = 0x00;
    Ad9510_ValidDividers[4]  = 0x11;       Ad9510_ValidDividers[6]  = 0x22;
    Ad9510_ValidDividers[8]  = 0x33;       Ad9510_ValidDividers[10] = 0x44;
    Ad9510_ValidDividers[12] = 0x55;       Ad9510_ValidDividers[14] = 0x66;
    Ad9510_ValidDividers[16] = 0x77;       Ad9510_ValidDividers[18] = 0x88;
    Ad9510_ValidDividers[20] = 0x99;       Ad9510_ValidDividers[22] = 0xaa;
    Ad9510_ValidDividers[24] = 0xbb;       Ad9510_ValidDividers[26] = 0xcc;
    Ad9510_ValidDividers[28] = 0xdd;       Ad9510_ValidDividers[30] = 0xee;
    Ad9510_ValidDividers[32] = 0xff;

    ValidDividerValues_Odd.push_back(1);  ValidDividerValues_Odd.push_back(2);
    ValidDividerValues_Odd.push_back(3);   ValidDividerValues_Odd.push_back(4);
    ValidDividerValues_Odd.push_back(5);   ValidDividerValues_Odd.push_back(6);
    ValidDividerValues_Odd.push_back(7);   ValidDividerValues_Odd.push_back(8);
    ValidDividerValues_Odd.push_back(9);   ValidDividerValues_Odd.push_back(10);
    ValidDividerValues_Odd.push_back(11);  ValidDividerValues_Odd.push_back(12);
    ValidDividerValues_Odd.push_back(13);  ValidDividerValues_Odd.push_back(14);
    ValidDividerValues_Odd.push_back(15);  ValidDividerValues_Odd.push_back(16);
    ValidDividerValues_Odd.push_back(17);  ValidDividerValues_Odd.push_back(18);
    ValidDividerValues_Odd.push_back(19);  ValidDividerValues_Odd.push_back(20);
    ValidDividerValues_Odd.push_back(21);  ValidDividerValues_Odd.push_back(22);
    ValidDividerValues_Odd.push_back(23);  ValidDividerValues_Odd.push_back(24);
    ValidDividerValues_Odd.push_back(25);  ValidDividerValues_Odd.push_back(26);
    ValidDividerValues_Odd.push_back(27);  ValidDividerValues_Odd.push_back(28);
    ValidDividerValues_Odd.push_back(29);  ValidDividerValues_Odd.push_back(30);
    ValidDividerValues_Odd.push_back(31);  ValidDividerValues_Odd.push_back(32);

    //  Fill the conversion information structures
    //     Valid dividers is keyed by divide_by, value is Input Value
    Ad9510_ValidDividers_Odd[1]  = 0xF0;       Ad9510_ValidDividers_Odd[2]  = 0x00;
    Ad9510_ValidDividers_Odd[3]  = 0x01;       Ad9510_ValidDividers_Odd[4]  = 0x11;
    Ad9510_ValidDividers_Odd[5]  = 0x12;       Ad9510_ValidDividers_Odd[6]  = 0x22;
    Ad9510_ValidDividers_Odd[7]  = 0x23;       Ad9510_ValidDividers_Odd[8]  = 0x33;
    Ad9510_ValidDividers_Odd[9]  = 0x34;       Ad9510_ValidDividers_Odd[10] = 0x44;
    Ad9510_ValidDividers_Odd[11]  = 0x45;       Ad9510_ValidDividers_Odd[12] = 0x55;
    Ad9510_ValidDividers_Odd[13]  = 0x56;       Ad9510_ValidDividers_Odd[14] = 0x66;
    Ad9510_ValidDividers_Odd[15]  = 0x67;       Ad9510_ValidDividers_Odd[16] = 0x77;
    Ad9510_ValidDividers_Odd[17]  = 0x78;       Ad9510_ValidDividers_Odd[18] = 0x88;
    Ad9510_ValidDividers_Odd[19]  = 0x89;       Ad9510_ValidDividers_Odd[20] = 0x99;
    Ad9510_ValidDividers_Odd[21]  = 0x9a;       Ad9510_ValidDividers_Odd[22] = 0xaa;
    Ad9510_ValidDividers_Odd[23]  = 0xab;       Ad9510_ValidDividers_Odd[24] = 0xbb;
    Ad9510_ValidDividers_Odd[25]  = 0xbc;       Ad9510_ValidDividers_Odd[26] = 0xcc;
    Ad9510_ValidDividers_Odd[27]  = 0xcd;       Ad9510_ValidDividers_Odd[28] = 0xdd;
    Ad9510_ValidDividers_Odd[29]  = 0xde;       Ad9510_ValidDividers_Odd[30] = 0xee;
    Ad9510_ValidDividers_Odd[31]  = 0xef;       Ad9510_ValidDividers_Odd[32] = 0xff;

}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::IsValidDivider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  Ad9510_Tb::IsValidDivider(unsigned int divide_by)
{
    ValidDividerMap::iterator iter = Ad9510_ValidDividers.find(divide_by);
    return iter != Ad9510_ValidDividers.end();
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::Ad951xGetNearestDividers()  -- find divider set around a candidate
//-----------------------------------------------------------------------------

void   Ad9510_Tb::Ad9510GetNearestDividers(float candidate, int & below, int & above)
{
    if (IsAboutEqual(candidate, NearestIntegerPart(candidate)))
        {
        //  We are essentially an integer, so treat as such.
        unsigned int int_candidate(static_cast<unsigned int>(NearestInteger(candidate)));
        if (IsValidDivider(int_candidate))
            {
            above = below = int_candidate;
            return;
            }
        }
    above = BestDividerGreaterThan(candidate);
    below = BestDividerLessThan(candidate);
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::BestDividerGreaterThan() -- returns next largest valid Divider
//-----------------------------------------------------------------------------

unsigned int  Ad9510_Tb::BestDividerGreaterThan(float min_divide_by)
{
    unsigned int CandidateD = 0;
    for (ValidDividerArray::iterator div = ValidDividerValues.begin();
             div !=  ValidDividerValues.end();  ++div)
        {
        CandidateD = *div;
        if (static_cast<float>(CandidateD) > min_divide_by)
            break;
        }
    return CandidateD;
}

//-----------------------------------------------------------------------------
//  Ad9510_Tb::BestDividerLessThan() -- returns next smallest valid Divider
//-----------------------------------------------------------------------------

unsigned int  Ad9510_Tb::BestDividerLessThan(float max_divide_by)
{
    unsigned int CandidateD = 0;
    for (ValidDividerArray::reverse_iterator div = ValidDividerValues.rbegin();
             div !=  ValidDividerValues.rend();  ++div)
        {
        CandidateD = *div;
        if (static_cast<float>(CandidateD) < max_divide_by)
            break;
        }
    return CandidateD;
}

//---------------------------------------------------------------------------
//  Ad9510_Tb::ToDividerValue()  -- "Divider" register value from a divider
//---------------------------------------------------------------------------
//  Calculate bits written given a divider

int  Ad9510_Tb::ToDividerValue(int divide_by)
{
	const int odd = (divide_by < 32) ? (divide_by & 1) : 0;
	const int lo = (divide_by >> 1) - 1;
	const int hi = lo + odd;
    int divider_bits = ( (lo << 4) | hi );
    return divider_bits;
}

//---------------------------------------------------------------------------
//  Ad9510_Tb::ToBypassValue()  -- "Bypass" register value from a divider
//---------------------------------------------------------------------------

int  Ad9510_Tb::ToBypassValue(int divide_by)
{
	if (divide_by==1)
		return 0x80;	
	else
		return 0x00;
}

//---------------------------------------------------------------------------
//  Ad9510_Tb::PBitsToP()  -- "Bypass" register value from a divider
//---------------------------------------------------------------------------

int  Ad9510_Tb::PBitsToP(int P_bits)
{
    P_bits &= 7;
    static int PMap[] = {1, 2, 2, 4, 8, 16, 32, 3};
    return PMap[P_bits];
    }


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  STRUCT Ad9510_Tb::HdwSettings
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//---------------------------------------------------------------------------
//  constructor for class  Ad9510_Tb::HdwSettings --
//---------------------------------------------------------------------------

Ad9510_Tb::HdwSettings::HdwSettings()
    :  Out_Bank0(Total_Bank0_LvPecl_Outputs),
       Out_Bank1(Total_Bank0_LvPecl_Outputs)
{
}


} // namespace Innovative
