//
// Ad9510_Tb_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Ad9510_Tb_MbH
#define Ad9510_Tb_MbH

#include <TimebaseDevice_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>
#include <VcxoControl_Mb.h>
#include <Ad9511Support_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS Ad9510Tb_PllTuningEvent  --  Calculate PLL tuning parameters
//===========================================================================

class Ad9510Tb_PllTuningEvent : public OpenWire::Event
{
public:
    int &R, &P, &B, &A, &Outdiv;

    Ad9510Tb_PllTuningEvent(int& r, int& p, int& b, int& a, int& outdiv)
        : R(r), P(p), B(b), A(a), Outdiv(outdiv)
        {
        }
};

//=============================================================================
//  CLASS Ad9510_Tb  --  PLL timebase control
//=============================================================================

class Ad9510_Tb : public PllTimebaseDevice
{
public:
    enum AdReferenceSource { refExternal, refInternal };
    enum AdOutputSource { outExtClk, outPll };
    enum AdExternalSelect { csFrontPanel, csCarrier };
    enum AdCpCurrent { cpc600,  cpc1200, cpc1800, cpc2400, cpc3000,
                       cpc3600, cpc4200, cpc4800 };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad9510_Tb::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   PllControl_Addr;
        unsigned int   PllSpi_Addr;
    };

    //
    //  Member Functions
    Ad9510_Tb(IAddressingSpace &space, MapRegisters & regs, unsigned int instance);
    ~Ad9510_Tb();

    void   Init();
    void   Reapply();

    int    MaxOutputDivider() const;


    void  SetPhaseDetectorFrequency(double freq)
        {  Settings.Pll.PhaseDetectorFrequency = freq;  }
    double  PhaseDetectorFrequency() const
        {  return Settings.Pll.PhaseDetectorFrequency;  }
    virtual double  DefaultPhaseDetectorFrequency() const = 0;  // initial value


    //  Select the Input reference
    void  ReferenceSource(AdReferenceSource value)
             { FReferenceSource = value;  SetReferenceSource(value); }
    AdReferenceSource  ReferenceSource()
             {  return FReferenceSource;  }

    //  Select Output Mode
    bool  IsIn_Pll_Mode()
             {  return FOutputSource == outPll;  }
    bool  IsIn_ExtClock_Mode()
             {  return IsIn_Pll_Mode() == false;  }
    void  Pll_Mode()
             {  OutputSource(outPll);  }
    void  ExtClock_Mode()
             {
             //ReferenceSource(refExternal);  // must use clock 1
             OutputSource(outExtClk);
             }
    void  OutputSource(AdOutputSource value)
             {
             FOutputSource = value;
             UpdateMode = (value==outPll) ? updFrequency : updDivider;
             SetOutputSource(value);
             }
    AdOutputSource  OutputSource() const
             { return FOutputSource; }

    void  ExternalClockSelect(AdExternalSelect value);
    AdExternalSelect  ExternalClockSelect() const;


    //  Tuner Configuration
    Ad951xAutoTune &  AutoTuner()  {  return Tuner;  }

    void 	AllowOddDivisors(bool state)
        { Tuner.AllowOddDivisors(state); }
    bool	AllowOddDivisors() const
        {  return Tuner.AllowOddDivisors();  }
    void 	AllowDecimation(bool state)
        { Tuner.AllowDecimation(state); }
    bool	AllowDecimation() const
        {  return Tuner.AllowDecimation();  }
    void 	AllowPostDivider(bool state)
        { Tuner.PostDivider(state); }
    bool	AllowPostDivider() const
        {  return Tuner.PostDivider();  }

    bool				Locked() const;

    int		            Decimation() const;

//    void 				AllowAsymmetricDutyCycle(bool state);
//    bool				AllowAsymmetricDutyCycle() const;

    void 				PFDPolarity(bool state);
    bool				PFDPolarity() const;

    void 				ChargePumpCurrent(AdCpCurrent mode);
    AdCpCurrent	        ChargePumpCurrent() const;


    void                VcoRange(double low, double high)
                            {  Vco->SetRange(low, high);  }
    void                LoadVco(IVcxoControl * aVco);
    IVcxoControl*       Vcxo() const  { return Vco;  }

    //  Raw configuration accesses
    virtual void 			Port(unsigned short reg, unsigned int value);
    virtual unsigned int 	Port(unsigned short reg);

    OpenWire::EventHandler<Ad9510Tb_PllTuningEvent> OnPllTuning;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    //   These values fill the field for OutputMode for each output. The LvCmos
    //     settings are for half of the field - combine one N and one P setting
    //     for the full configuration
//    enum  OutputCfgSettings { ocLvds_0p50, ocLvds_0p75, ocLvds,
//                             ocLvds_1p25, ocHstl,  ocHstl_boost,
//                             ocCmos, oc_CmosOther
//                           };
//    enum  OutputDriverPhaseSettings { odpForceHigh, odpNormal, odpInverting, odpForceLow };
    enum AdCpMode   { cpmHighZ, cpmPumpUp, cpmPumpDown, cpmNormal };
    enum StatusPinOpts { spOff, spDigitalLockH, spN, spDigitalLockL, spR, spAnalogLockN,
                         spA, spPrescaler, spPfdUp, spPfdDown, spRefLostH, spTristate,
                         spAnalogLockP, spNotLockedH, spNotLockedL, spRefLostL };
    enum  AntiBacklashOpts {  abw1p3, abw2p9, abw6p0, abw1p3x, abwDefault=abw1p3};
    enum  AdVoltLevel  { v500mv, v340mv, v810mv, v660mv };
    enum  AdLvdsCurrent  { c1750uA, c3500uA, c5250uA, c7000uA };
    enum  OutputCfgSettings { ocLvds=0x0, ocLvCmos=0x1 };

    //
    //  Configuration Data Classes
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  STRUCT Register08Fields
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct Register08Fields
    {
        bool           PFDPolarity;
        StatusPinOpts  StatusPinCfg;
        AdCpMode       ChargePumpMode;

        Register08Fields() : PFDPolarity(true), StatusPinCfg(spDigitalLockH), ChargePumpMode(cpmNormal)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  STRUCT Register0DFields
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct Register0DFields
    {
        AntiBacklashOpts  AntiBacklash;
        bool              ShortDLD;
        bool              DisableDLD;

        Register0DFields() : AntiBacklash(abwDefault), ShortDLD(false), DisableDLD(false)
            {}
    };

        struct LvPeclSettings
        {
            unsigned int OutputCfgReg;

            LvPeclSettings()
                {
                OutputCfgReg = 0;
                }
        };

        struct LvdsSettings
        {
            unsigned int OutputCfgReg;       // 0x40 bank register image
            LvdsSettings()
                {
                OutputCfgReg = 0;
                }
        };

        struct PllSettings
        {
            AdCpCurrent  ChargePumpCurrent;
            bool         PllPowerDownX;
            Register08Fields  R08;
            Register0DFields  R0D;
            double       PhaseDetectorFrequency;
            double       PhaseDetectorFrequency_Actual;
            double       VcoFrequency;
            double	     ActualRateBeforePostDivider;
            double	     ActualRateBeforeDecimation;



            PllSettings() : ChargePumpCurrent(cpc600),
                            PhaseDetectorFrequency(0.0), PhaseDetectorFrequency_Actual(0.0),
                            VcoFrequency(0.0), ActualRateBeforePostDivider(0.0), ActualRateBeforeDecimation(0.0)
                {
                }
        };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  STRUCT HdwSettings - cached data
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct HdwSettings
    {
        PllSettings     Pll;
        std::vector<LvPeclSettings>  Out_Bank0;
        std::vector<LvdsSettings>    Out_Bank1;

        HdwSettings();
    };


    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    void  SetOutVoltageHdw(unsigned int output, unsigned int volt_setting);
    void  SetOutCurrentHdw(unsigned int output, unsigned int current_setting);

    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting);
    void  SetReferenceSource(AdReferenceSource source);
    void  SetOutputSource(AdOutputSource source);
    void  SetExternalClockSelect(AdExternalSelect source);
    void  InitSerialPort();
    void  UpdateRegisters();

    void SetR(unsigned int R);
    void SetP(unsigned int P, unsigned int B);
    void SetA(unsigned int A);
    void SetB(unsigned int B);
    void SetReg08(const Register08Fields & r08);
    void SetReg09(unsigned int charge_pump_current_setting);
    void SetReg0D(const Register0DFields & r0d);

    //
    //  PLL Device Implementations
    void            SettingsInit();
    void            FrequencyModeUpdate();
    void            DividerModeUpdate();

//    void  ReadDataRegisters();
//    void  WriteDataRegisters();


protected:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllControlRegister  -- PLL misc control interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit     ReferenceSource;
        RegisterBit     ExtClockSource;
        RegisterBit     Sync1;
        RegisterBit     Sync2;

        PllControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9510_Tb::PllControlRegister", space, offset),
              ReferenceSource(*this, 5),  ExtClockSource(*this, 6),
              Sync1(*this, 7),  Sync2(*this, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllStatusRegister  -- PLL status/SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Locked;
        RegisterBit           SpiReady;
        RegisterBit           SpiDataValid;

        PllStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9510_Tb::PllStatusRegister", space, offset),
              Locked(*this, 13),
              SpiReady(*this, 30), SpiDataValid(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiControlRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup      Address;
        RegisterBit           PllSelect;
        RegisterBit           Rw;
        RegisterBitGroup      Data;

        SpiControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9510_Tb::SpiControlRegister", space, offset),
              Address(*this, 0, 10), PllSelect(*this, 10),
              Rw(*this, 12), Data(*this, 16, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiStatusRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Data;

        SpiStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9510_Tb::SpiStatusRegister", space, offset),
              Data(*this, 24, 8)
            {
            }
    };

    //
    //  Member Data
    std::vector<int>           RegisterImage;
    std::vector<float>         OutputFrequencyLimit;
    IVcxoControl *             Vco;
    Ad951xAutoTune             Tuner;
    unsigned int               Instance;
    AdReferenceSource          FReferenceSource;
    AdOutputSource             FOutputSource;
    AdExternalSelect           FExternalClockSrc;


    HdwSettings     Settings;

    PllControlRegister  PllControlReg;
    PllStatusRegister   PllStatusReg;
    SpiControlRegister  SpiControlReg;
    SpiStatusRegister   SpiStatusReg;

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by);  }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above)
        {  return Ad9510GetNearestDividers(candidate, below, above);  }

    typedef std::vector<unsigned int> ValidDividerArray;
    typedef std::map<unsigned int, unsigned int> ValidDividerMap;

    //  Dividers
    ValidDividerArray    ValidDividerValues;
    ValidDividerMap      Ad9510_ValidDividers;
    ValidDividerArray    ValidDividerValues_Odd;
    ValidDividerMap      Ad9510_ValidDividers_Odd;

    bool         IsValidDivider(unsigned int divide_by);
    void         Ad9510GetNearestDividers(float candidate, int& below, int& above);
    unsigned int BestDividerGreaterThan(float min_divide_by);
    unsigned int BestDividerLessThan(float max_divide_by);

    int  		 ToDividerValue(int divide_by);
    int  		 ToBypassValue(int divide_by);
    int          PBitsToP(int P_bits);

    int          OutputDividerOffset(int bank, int offset);


    void  SetOutputDividers(double VcoFreq, unsigned int OutDiv);
    void  SetOutEnabledHdw_Bank0(unsigned int output, bool enabled);
    void  SetOutEnabledHdw_Bank1(unsigned int output, bool enabled);

    void  ConfigureDividerInformation();


    int   LvdsDividerOffset(int bank);

public:
    Trace			Debug;
private:
};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

