//
// Ad9511Duo_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Ad9511Duo_Mb.h>
#include <SystemSupport_Mb.h>
#include <algorithm>
#include <limits>
#include <cmath>
#include <Magic_Mb.h>
#include <Trace_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
static const double _VcoLow = 100.0e6;
static const double _VcoHigh = 140.0e6;

namespace Innovative
{

namespace Ad9511DuoSupport
{

//=============================================================================
//  CLASS Ad9511DuoMasterClock  --  Master PLL timebase control
//=============================================================================

class Ad9511DuoMasterClock : public Ad9511Pfd
{
	typedef Ad9511Pfd inherited;

public:
	//
	//  Member Functions
	Ad9511DuoMasterClock(int index, AddressingSpace &space, int ctl)
		: inherited(index, space, ctl)
		{  Debug.Enabled(true);  }
	Ad9511DuoMasterClock(double freq, double ref,
		bool enabled, int index, AddressingSpace & space, int ctl)
		: inherited(freq, ref, enabled, index, space, ctl)
		{  Debug.Enabled(true);  }

protected:

	virtual double  PhaseDetectorFrequency() const
		{   return 100.0e3;  }
};


//=============================================================================
//  CLASS Master  --  Master clock in cascade
//=============================================================================

class Master : public Ad9511DuoMasterClock
{
	typedef Ad9511DuoMasterClock inherited;

public:
	Master(int index, AddressingSpace &space, int ctl, Ad9511Duo * owner)
	   : inherited(index, space, ctl), Owner(owner)
       {  Debug.Enabled(true); Tuner.PostDivider(false); Tuner.AllowDecimation(false); }
	Master(double freq, double ref, bool enabled, int index,
	   AddressingSpace & space, int ctl, Ad9511Duo * owner)
	   : inherited(freq, ref, enabled, index, space, ctl), Owner(owner)
	   {  Debug.Enabled(true); Tuner.PostDivider(false); Tuner.AllowDecimation(false); }

protected:
    // Overrides
    virtual void ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
        {
        Port(0x3C, 0x08);         // Clock 0 LVPECL on @810mV
        for (unsigned int i = 0; i < 7; ++i)
            Port(0x3D+i, 0x03);   // Clock 1..7 disabled

        // ...This stock behavior is to link Master out to slave input (as in X3Servo)
        // Identical duty cycle and phase for all outputs
        for (int i = 0; i < 8; ++i)
            {
            Port(0x48+i*2, outdiv_bits);	        // duty cycle
            Port(0x48+(i*2+1), outdiv_bypass);	    // phase
            }

        //  Allow user to optionally change 'normal' output divider setting
        Owner->ConfigureMasterOutputs(*this, outdiv_bits, outdiv_bypass);


        //  Allow user to change anything he feels like changing
        PllConfigureOutputEvent e(*Owner->Clock, outdiv_bits, outdiv_bypass);
        Owner->OnPllConfigureClockOutput.Execute(e);
        }

private:
	Ad9511Duo *    Owner;
};

//=============================================================================
//  CLASS Slave  --  Slave clock in cascade
//=============================================================================

class Slave : public Ad9511
{
public:
    Slave(int index, AddressingSpace &space, int ctl, Ad9511Duo * owner)
        : Ad9511(index, space, ctl), Owner(owner)
		{  Debug.Enabled(true);  }
	Slave(double freq, double ref, bool enabled, int index,
	   AddressingSpace & space, int ctl, Ad9511Duo * owner)
	   : Ad9511(freq, ref, enabled, index, space, ctl), Owner(owner)
	   {  Debug.Enabled(true);  }

protected:
    // Overrides
    // No default configuration here...override required
    virtual void ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
        {
        //  Apply device specific slave output configuration
        Owner->ConfigureSlaveOutputs(*this, outdiv_bits, outdiv_bypass);

        //  Allow user to change anything he feels like changing
        PllConfigureOutputEvent e(*Owner->Divider, outdiv_bits, outdiv_bypass);
        Owner->OnPllConfigureDividerOutput.Execute(e);
        }

private:
    Ad9511Duo *    Owner;
};

}  // end namespace Ad9511DuoSupport
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ad9511Duo  --  Cascaded PLL timebase control
//=============================================================================

//-----------------------------------------------------------------------------
//  constructor for class Ad9511Duo
//-----------------------------------------------------------------------------

Ad9511Duo::Ad9511Duo(int index, AddressingSpace & space, int ctl_master, int ctl_slave)
	: ClockBase(index), FOutputClock(oVco),
      Clock(new Ad9511DuoSupport::Master(index, space, ctl_master, this)),
      Divider(new Ad9511DuoSupport::Slave(index, space, ctl_slave, this)),
      Tuner(0)
{
    Clock->VcoRange(_VcoLow, _VcoHigh);

	Divider->VcoRange(_VcoLow, _VcoHigh);

    Tuner.LoadVco(Clock->Vcxo());
    Tuner.Multiplier(1);
    Tuner.PostDivider(true);
    Tuner.AllowDecimation(true);

    Divider->OutputClock(Ad9511::oExternal);

    Clock->PFDPolarity(true);
    Divider->PFDPolarity(true);

    Clock->OnPllTuning.SetEvent(this, &Ad9511Duo::HandlePllTuning);
    Clock->OnPllDividing.SetEvent(this, &Ad9511Duo::HandlePllDividing);
}

//-----------------------------------------------------------------------------
//  constructor for class Ad9511Duo
//-----------------------------------------------------------------------------

Ad9511Duo::Ad9511Duo(double freq, double ref, bool enabled, int index,
				 AddressingSpace & space, int ctl_master, int ctl_slave)
	: ClockBase(freq, ref, enabled, index), FOutputClock(oVco),
      Clock(new Ad9511DuoSupport::Master(freq, ref, enabled, index, space, ctl_master, this)),
      Divider(new Ad9511DuoSupport::Slave(freq, ref, enabled, index, space, ctl_slave, this)),
      Tuner(0)
{
    Clock->VcoRange(_VcoLow, _VcoHigh);

    Divider->VcoRange(_VcoLow, _VcoHigh);

    Tuner.LoadVco(Clock->Vcxo());
    Tuner.Multiplier(1);
    Tuner.PostDivider(true);
    Tuner.AllowDecimation(true);

	DoSetFrequency(freq);
	DoSetEnabled(enabled);

    Divider->OutputClock(Ad9511::oExternal);

    Clock->PFDPolarity(true);
    Divider->PFDPolarity(true);

    Clock->OnPllTuning.SetEvent(this, &Ad9511Duo::HandlePllTuning);
    Clock->OnPllDividing.SetEvent(this, &Ad9511Duo::HandlePllDividing);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


//-----------------------------------------------------------------------------
//  destructor for class Ad9511Duo
//-----------------------------------------------------------------------------

Ad9511Duo::~Ad9511Duo()
{
    delete Clock;
    Clock = 0;

	delete Divider;
    Divider = 0;
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Max()  --  Set maximum output rate
//-----------------------------------------------------------------------------

void Ad9511Duo::Max(double value)
{
    Clock->Max(value);
    Divider->Max(value);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Decimation()  --  Return decimation factor
//-----------------------------------------------------------------------------

int	Ad9511Duo::Decimation() const
{
    if (FOutputClock==oVco)
        return Tuner.Decimation();
    else
        return 1;
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Max()  --  Return maximum output rate
//-----------------------------------------------------------------------------

double Ad9511Duo::Max() const
{
	return Clock->Max();
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Min()  --  Set minimum output rate
//-----------------------------------------------------------------------------

void Ad9511Duo::Min(double value)
{
	Clock->Min(value);
	Divider->Min(value);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Min()  --  Return minimum output rate
//-----------------------------------------------------------------------------

double Ad9511Duo::Min() const
{
    return Clock->Min() / MaxOutputDivider();
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::MaxOutputDivider()  --  Return max output post-divider
//-----------------------------------------------------------------------------

int Ad9511Duo::MaxOutputDivider() const
{
	return Clock->MaxOutputDivider() * Divider->MaxOutputDivider();
}

//---------------------------------------------------------------------------
//  Ad9511Duo::OutputClock() -  Specify output
//---------------------------------------------------------------------------

void Ad9511Duo::OutputClock(IIOutputClock value)
{
    Clock->OutputClock(value==oExternal ? Ad9511::oExternal : Ad9511::oVco);
    Divider->OutputClock(Ad9511::oExternal);

	FOutputClock = value;
}

//---------------------------------------------------------------------------
//  Ad9511Duo::Mux() -  Query output
//---------------------------------------------------------------------------

Ad9511Duo::IIOutputClock Ad9511Duo::OutputClock() const
{
	return FOutputClock;
}

//---------------------------------------------------------------------------
//  Ad9511Duo::PllStatus() -  Query lock status
//---------------------------------------------------------------------------

bool Ad9511Duo::PllStatus() const
{
	return Clock->PllStatus() && Divider->PllStatus();
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::GetFrequencyActual()  --  Calculate actual clock rate
//-----------------------------------------------------------------------------

double Ad9511Duo::GetFrequencyActual() const
{
	return Divider->FrequencyActual()/FClockFactor;
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::SetFrequency()  --  Calculate settings for a frequency
//-----------------------------------------------------------------------------

void Ad9511Duo::SetFrequency(double freq)
{
	DoSetFrequency(freq);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::DoSetFrequency()  --  Calculate settings for a frequency
//-----------------------------------------------------------------------------
void Ad9511Duo::DoSetFrequency(double freq)
{
	// Adjust Frequency to accomodate clock factor
	freq *= FClockFactor;

	Tuner.ReferenceRate(Reference());
	Tuner.SampleRate(freq);

	if (FOutputClock == oVco)
		Tuner.Calculate();
	else
		Tuner.Calculate_External();
		
	//
    //  Apply Reference value to Clock
    Clock->Reference(Reference());
    Clock->Debug.Log("DUO Input Frequency =", freq);
	// Drive the master clock at upper end of usable range to minimize jitter
	double f =  Tuner.ActualSampleRateBeforePostDivider();
	Clock->Frequency(f);
	Clock->Debug.Log("DUO Clock Actual =", Clock->FrequencyActual());
	// Drive the slave clock at the requested rate
	Divider->Reference(Clock->FrequencyActual());
	Clock->Debug.Log("DUO Divider Freq =", freq);
	double f2 =  Tuner.ActualSampleRateBeforeDecimation() ;
	Divider->Frequency(f2);

    Clock->Debug.Log("DUO Divider Actual =", Divider->FrequencyActual());
	Clock->Debug.Log("DUO Divider Ref =", Divider->Reference());
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Locked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  Ad9511Duo::Locked() const
{
	//return (Clock->Locked() && Divider->Locked()) ? true : false;
	return Clock->Locked();
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::SetEnabled()  --  Activate / Deactivate output
//-----------------------------------------------------------------------------

void  Ad9511Duo::SetEnabled(bool state)
{
	DoSetEnabled(state);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::DoSetEnabled()  --  Activate / Deactivate output
//-----------------------------------------------------------------------------

void  Ad9511Duo::DoSetEnabled(bool state)
{
    Clock->Enabled(state);
    Divider->Enabled(state);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Reapply()  --  Reapply settings to hardware from saved state
//-----------------------------------------------------------------------------

void  Ad9511Duo::Reapply()
{
    Clock->Reference(Reference());
	Frequency(FFrequency);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Port()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void Ad9511Duo::Port(int device, unsigned char reg, unsigned char value)
{
	if (device)
		Divider->Port(reg, value);
	else
		Clock->Port(reg, value);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::Port()  --  Read from PLL data port
//-----------------------------------------------------------------------------

unsigned char Ad9511Duo::Port(int device, unsigned char reg)
{
	unsigned char result;

	if (device)
		result = Divider->Port(reg);
	else
		result = Clock->Port(reg);

	return result;
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::PllTuningEvent()  --
//-----------------------------------------------------------------------------

void Ad9511Duo::HandlePllTuning (PllTuningEvent& e)
{
    OnPllTuning.Execute(e);
}

//-----------------------------------------------------------------------------
//  Ad9511Duo::PllDividerEvent()  --
//-----------------------------------------------------------------------------

void Ad9511Duo::HandlePllDividing (PllDividerEvent& e)
{
    OnPllDividing.Execute(e);
}

} // namespace Innovative
