//
// Ad9511Duo_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Ad9511Duo_MbH
#define Ad9511Duo_MbH

#include <Ad9511_Mb.h>

namespace Innovative
{

namespace Ad9511DuoSupport
{
    class Master;
    class Slave;
}

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ad9511Duo  --  Cascaded PLL timebase control
//=============================================================================

class Ad9511Duo : public ClockBase
{
    friend class Ad9511DuoSupport::Master;
    friend class Ad9511DuoSupport::Slave;

public:
	enum IIOutputClock { oExternal, oVco };

	//
	//  Member Functions
	Ad9511Duo(int index, AddressingSpace &space, int ctl_master, int ctl_slave);
	Ad9511Duo(double freq, double ref, bool enabled, int index,
		   AddressingSpace & space, int ctl_master, int ctl_slave);
    virtual ~Ad9511Duo();

	void 				OutputClock(IIOutputClock value);
	IIOutputClock		OutputClock() const;

	bool				Locked() const;
	int		            Decimation() const;
	bool				PllStatus() const;

	// Methods
	void                Reapply();

	// Output frequency range information
	void 				Max(double value);
	double				Max() const;

	void 				Min(double value);
	double 				Min() const;
	int  				MaxOutputDivider() const;

    virtual void    ConfigureMasterOutputs(Ad9511 & /*clk*/, int /*outdiv_bits*/, int /*outdiv_bypass*/)
        {  }
    virtual void    ConfigureSlaveOutputs(Ad9511 & /*clk*/, int /*outdiv_bits*/, int /*outdiv_bypass*/)
		{  }

	//  Raw configuration accesses
	virtual void 			Port(int device, unsigned char reg, unsigned char value);
	virtual unsigned char 	Port(int device, unsigned char reg);

	void 	AllowOddDivisors(bool state)
        { Tuner.AllowOddDivisors(state); }
	bool	AllowOddDivisors() const
		{  return Tuner.AllowOddDivisors();  }
    void    PfdRate(double value)
        {  Tuner.PfdRate(value);  Master_SubTuner().PfdRate(value); }

    OpenWire::EventHandler<PllTuningEvent> OnPllTuning;
    OpenWire::EventHandler<PllDividerEvent> OnPllDividing;
    OpenWire::EventHandler<PllConfigureOutputEvent> OnPllConfigureClockOutput;
    OpenWire::EventHandler<PllConfigureOutputEvent> OnPllConfigureDividerOutput;

    
    virtual double  PhaseDetectorFrequency() const
        {   return 100.0e3;  }
    virtual void   SetPhaseDetectorFrequency(double)
        { }

    //  Tuner Configuration
    Ad951xAutoTune &  AutoTuner()  {  return Tuner;  }

    //  SubTuner Configuration
    Ad951xAutoTune &  Master_SubTuner()  {  return Clock->AutoTuner();  }
    Ad951xAutoTune &  Slave_SubTuner()  {  return Divider->AutoTuner();  }

public:
    // Overrides
	virtual void    SetFrequency(double freq);
	virtual double  GetFrequencyActual() const;
	virtual void    SetEnabled(bool state);

private:
    // Fields
    IIOutputClock	FOutputClock;

    Ad9511Pfd *     Clock;
    Ad9511 *        Divider;
    Ad951xAutoTune  Tuner;
    // Support
	void 			DoSetFrequency(double freq);
	void 			DoSetEnabled(bool state);

    void HandlePllTuning (PllTuningEvent& e);
    void HandlePllDividing (PllDividerEvent& e);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative

#endif

