// Ad9511Support_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Ad9511Support_MbH
#define Ad9511Support_MbH


#include <vector>

namespace Innovative
{

//---------------------------------------------------------------------------
//  class Ad951xAutoTune
//---------------------------------------------------------------------------

class Ad951xAutoTune
{
public:
    typedef std::vector<int> 	IntArray;
    enum PostDividerStyles {  pdsAd9511, pdsAd9508, pdsAd9508_SDF  };

    // Constructors
	Ad951xAutoTune(IVcxoControl *Vco);

	// Methods
    bool Calculate();              // PLL mode calculation
    bool Calculate_External();     // Divider Mode calculation

    void LoadVco(IVcxoControl *aVco)
        {  FVco = aVco; }
	// Properties
	void 	AllowDecimation(bool state)
		{  FAllowDecimation = state;  }
	bool	AllowDecimation() const
		{  return FAllowDecimation;  }

	void 	AllowOddDivisors(bool state);
	bool	AllowOddDivisors() const
		{  return FAllowOddDivisors;  }

	void 	ReferenceRate(double f)
		{  FReferenceRate = f;  }
	double	ReferenceRate() const
		{  return FReferenceRate;  }

	void 	SampleRate(double f)
		{  FSampleRate = f;  }
	double	SampleRate() const
		{  return FSampleRate;  }

	void 	Multiplier(int value)
		{  FMultiplier = value;  }
	int		Multiplier() const
		{  return FMultiplier;  }

	void 	PfdRate(double f)
		{  FPfdRate = f;  }
    double	PfdRate() const
        {  return FPfdRate;  }
    double	PfdRateActual() const
        {  return FActualPfdRate;  }

	void 	PfdError(double f)
		{  FPfdError = f;  }
	double	PfdError() const
		{  return FPfdError;  }

	void 	PostDivider(bool state);
    bool	PostDivider() const
		{  return FPostDivider;  }
    void    PostDividerStyle(PostDividerStyles style)
        {  FPostDividerStyle = style;  ConfigurePostDividers();  }

    void	EnforceOutputMax(bool enforce)
        {
        FEnforceOutputMax = enforce;
        }
    void	PllOutputMaxRate(double rate)
        {
        FPllOutputMaxRate = rate;
        }

	// Read-only
	double	ClockRate() const
		{  return FSampleRate * FMultiplier;  }
	double	LowestRate() const
		{  return FLowestRate;  }
	double	HighestRate() const
		{  return FHighestRate;  }
	double	ActualSampleRate() const
		{  return FActualSampleRate;  }
	double	ActualSampleRateBeforePostDivider() const
		{  return FActualSampleRateBeforePostDivider;  }
	double	ActualSampleRateBeforeDecimation() const
		{  return FActualSampleRateBeforeDecimation;  }

	double	ActualVcoRate() const
		{  return FActualVcoRate;  }
	double	VcoLow() const
        {  return FVco->Min();  }
	double	VcoHigh() const
        {  return FVco->Max();  }

    //  Results
    int  Prescaler() const    {  return Best.P;  }
    int  P() const            {  return Prescaler();  }
    int  P_bits() const       {  return Best.P_bits;  }
    int  B() const            {  return Best.B;  }
    int  A() const            {  return Best.A;  }
    int  R() const            {  return Best.R;  }
    int	 Decimation() const    {  return Best.Decimation;  }
    int  OutputDivider() const {  return Best.D[0];  }
    int  PostDividerValue() const   {  return Best.D[1];  }

	// Data
	struct
	{
		// Data
		int P;
		int P_bits;
		int B;
		int A;
		int R;
		int D[2];
		int Decimation;

    } Best;

private:
	double	FSampleRate;
	int		FMultiplier;
	double	FLowestRate;
	double	FHighestRate;
	double 	FReferenceRate;
	double 	FPfdRate;
	double 	FPfdError;
	bool	FPostDivider;
    PostDividerStyles  FPostDividerStyle;
    double	FActualPfdRate;
	double	FActualSampleRate;
    double  FActualSampleRateBeforePostDivider;
    double  FActualSampleRateBeforeDecimation;
	double	FActualVcoRate;
	bool	FAllowOddDivisors;
	bool	FAllowDecimation;

    bool    FEnforceOutputMax;
    double  FPllOutputMaxRate;

    IVcxoControl *FVco;

	// Data
	IntArray	Dividers;
	IntArray	PostDividers;
	IntArray	Prescalers;

	// Support
	void ConfigureValidDividers(IntArray & dividers);
	void ConfigureValidPrescalers();
    void ConfigurePostDividers();

    void  AdvancedPBACalculation(int N, int & P, int & P_bits, int & B, int & A, double VcoRate);
};


} // namespace Innovative

#endif
