//
// Ad9511_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Ad9511_MbH
#define Ad9511_MbH

#include <Clockbase_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>
#include <VcxoControl_Mb.h>
#include <Ad9511Support_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS PllTuningEvent  --  Calculate PLL tuning parameters
//===========================================================================

class PllTuningEvent : public OpenWire::Event
{
public:
    int &R, &P, &B, &A, &Outdiv;

    PllTuningEvent(int& r, int& p, int& b, int& a, int& outdiv)
        : R(r), P(p), B(b), A(a), Outdiv(outdiv)
        {
        }
};

//===========================================================================
// CLASS PllDividerEvent  --  Calculate PLL divider parameters
//===========================================================================

class PllDividerEvent : public OpenWire::Event
{
public:
    int &Divider;

    PllDividerEvent(int& divider)
        : Divider(divider)
        {
        }
};

//===========================================================================
// CLASS PllConfigureOutputEvent  --  Configure PLL outputs
//===========================================================================

class PllConfigureOutputEvent : public OpenWire::Event
{
public:
    class Ad9511 & Pll;
    int Bits;
    int Bypass;

    PllConfigureOutputEvent(Ad9511 & clk, int outdiv_bits, int outdiv_bypass)
        : Pll(clk), Bits(outdiv_bits), Bypass(outdiv_bypass)
        {
        }
};

//=============================================================================
//  CLASS Ad9511  --  PLL timebase control
//=============================================================================

class Ad9511 : public ClockBase
{
public:
    enum IIOutputClock { oExternal, oVco };
    enum IIReferenceClock { rInternal, rExternal };
    enum IIChargePumpCurrent { c600, c1200, c1800, c2400, c3000, c3600, c4200, c4800 };

    //
    //  Member Functions
    Ad9511(int index, AddressingSpace &space, int ctl);
    Ad9511(double freq, double ref, bool enabled, int index,
           AddressingSpace & space, int ctl);
    ~Ad9511()
        {  delete Vco;  Vco=0;  }

    void 				OutputClock(IIOutputClock value);
    IIOutputClock		OutputClock() const;

    virtual void 		        ReferenceClock(IIReferenceClock value);
    virtual IIReferenceClock	ReferenceClock() const;

    bool				Locked() const;
    int		            Decimation() const;
    int 				OutputDivider() const;
    void 				OutputDivider(int value);

    void 				AllowAsymmetricDutyCycle(bool state);
    bool				AllowAsymmetricDutyCycle() const;

    void 				PFDPolarity(bool state);
    bool				PFDPolarity() const;

    void 				ChargePumpCurrent(IIChargePumpCurrent mode);
    IIChargePumpCurrent	ChargePumpCurrent() const;

    bool				PllStatus() const
        {  return FPllStatus;  }

    void                UseAdvancedPBACalculation(bool state)
                            {  FUseAdvancedPBACalculation = state;  }

    // Methods
    void                Reapply();

    // Output frequency range information
    void 				Max(double value);
    double				Max() const;

    void 				Min(double value);
    double 				Min() const;
    int  				MaxOutputDivider() const;

    void                VcoRange(double low, double high)
                            {  Vco->SetRange(low, high);  }
    void                LoadVco(IVcxoControl * aVco);
    IVcxoControl*       Vcxo() const
        { return Vco;       }

    //  Raw configuration accesses
    virtual void 			Port(unsigned char reg, unsigned char value);
    virtual unsigned char 	Port(unsigned char reg);

    ShadowRegister &        PllControlReg()
        {  return ControlReg;  }

    //  Tuner Configuration
    Ad951xAutoTune &  AutoTuner()  {  return Tuner;  }

    void 	AllowOddDivisors(bool state)
        { Tuner.AllowOddDivisors(state); }
    bool	AllowOddDivisors() const
        {  return Tuner.AllowOddDivisors();  }
    void 	AllowDecimation(bool state)
        { Tuner.AllowDecimation(state); }
    bool	AllowDecimation() const
        {  return Tuner.AllowDecimation();  }
    void    PfdRate(double value)
        {  Tuner.PfdRate(value);  }
    void    PfdError(double value)
        {  Tuner.PfdError(value);  }

    OpenWire::EventHandler<PllTuningEvent> OnPllTuning;
    OpenWire::EventHandler<PllDividerEvent> OnPllDividing;
    OpenWire::EventHandler<PllConfigureOutputEvent> OnPllConfigureOutput;
    OpenWire::EventHandler<PllConfigureOutputEvent> OnAfterPllConfigureOutput;

protected:
    // Overrides
    virtual void    SetFrequency(double freq);
    virtual double  GetFrequencyActual() const;
    virtual void    SetEnabled(bool state);
    virtual void    ConfigureOutputs(int outdiv_bits, int outdiv_bypass);

    //
    //  Member Data
    IVcxoControl *  Vco;
    Ad951xAutoTune  Tuner;
    int 			outdiv;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllControlRegister  -- PLL misc control interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit     ReferenceSource;

        PllControlRegister(AddressingSpace &space, int offset)
            : inherited("Ad9511::PllControlRegister", space, offset),
              ReferenceSource(*this, 0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllStatusRegister  -- PLL misc control interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;
        ReadOnlyRegisterBit		Lock;

    public:
        PllStatusRegister(AddressingSpace &space, int offset)
        :  inherited(space, offset),
           Lock(*this, 31)
        {  }

        // Properties
        bool Locked() const
            {  return Lock.Value() ? true : false;  }
    };

    PllControlRegister		ControlReg;
    PllStatusRegister		StatusReg;

    // Fields
    double				FFrequencyActual;
    IIOutputClock		FOutputClock;
    IIReferenceClock    FReferenceClock;
    bool				FPllStatus;
    bool				FAllowAsymmetricDutyCycle;
    bool				FPFDPolarity;
    IIChargePumpCurrent	FChargePumpCurrent;
    bool                FUseAdvancedPBACalculation;

    // Support
    void 			DoSetFrequency(double freq);
    void 			DoSetEnabled(bool state);
    double 			Round(double original);
    int  			ToDividerValue(int divide_by);
    int  			ToBypassValue(int divide_by);

public:
    Trace			Debug;
};

//=============================================================================
//  CLASS Ad9511Pfd  --  PLL timebase control. Same as AD9511
// but optimized for comparing at the phase-frequency-detector frequency.
//=============================================================================

class Ad9511Pfd : public Ad9511
{
public:
    //
    //  Member Functions
    Ad9511Pfd(int index, AddressingSpace &space, int ctl);
    Ad9511Pfd(double freq, double ref, bool enabled, int index,
           AddressingSpace & space, int ctl);
    ~Ad9511Pfd()
        {}

    virtual double PhaseDetectorFrequency() const = 0;
    virtual void   SetPhaseDetectorFrequency(double)
    {}
};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

