//
// Ad9516_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Ad9516_MbH
#define Ad9516_MbH

#include <Clockbase_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <sstream>
#include <map>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ad9516  --  PLL timebase control
//=============================================================================

class Ad9516 : public ClockBase
{
public:
	enum IIOutputClock { oExternal, oVco };
	enum IIReferenceClock { rExternal, rInternal };

	//
	//  Member Functions
	Ad9516(int index, AddressingSpace &space, int ctl, int spi);
	Ad9516(double freq, double ref, bool enabled, int index,
		   AddressingSpace & space, int ctl, int spi);

	void 				        OutputClock(IIOutputClock value);
	IIOutputClock		        OutputClock() const;

	virtual void 		        ReferenceClock(IIReferenceClock value);
	virtual IIReferenceClock	ReferenceClock() const;

	virtual bool				Locked() const;

	int 				OutputDivider() const;
	void 				OutputDivider(int value);

	void 				AllowAsymmetricDutyCycle(bool state);
	bool				AllowAsymmetricDutyCycle() const;

	void 				PFDPolarity(bool state);
	bool				PFDPolarity() const;

	// Methods
	virtual void        Reset();
	virtual void        Reapply();

	// Output frequency range information
	void 				Max(double value);
	double				Max() const;

	void 				Min(double value);
	double 				Min() const;
	int  				MaxOutputDivider() const;

    void                VcoRange(double low, double high)
                            { VcoLow = low;  VcoHigh = high;  }

    //  Raw configuration accesses
	virtual void 			Port(unsigned short reg, unsigned char value);
	virtual unsigned char 	Port(unsigned short reg);

protected:
	// Overrides
	virtual void    SetFrequency(double freq);
	virtual double  GetFrequencyActual() const;
	virtual void    SetEnabled(bool state);
	virtual void    ConfigureOutputs(int outdiv_bits, int outdiv_bypass);

	//
	//  Member Data
	// VCO operational frequency range, Hz
	double 			VcoLow;
	double 			VcoHigh;
	int 			outdiv;
    int             vcodiv;
    int             totaldiv;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS PllControlRegister  -- PLL misc control interface
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class PllControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit     Reset;
		RegisterBit     ClockSource;
		RegisterBit     ReferenceSource;

		PllControlRegister(AddressingSpace &space, int offset)
			: inherited("Ad9516::PllControlRegister", space, offset),
			  Reset(*this, 0), ClockSource(*this, 1),
              ReferenceSource(*this, 2)
			{
			//  Default setup ????
			}
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS PllStatusRegister  -- PLL SPI interface
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class PllStatusRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit           Locked;

		PllStatusRegister(AddressingSpace &space, int offset)
			: inherited("Ad9516::PllStatusRegister", space, offset),
			  Locked(*this, 31)
			{
			//  Default setup ????
			}
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS SpiControlRegister  -- PLL SPI interface
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class SpiControlRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup      Data;
		RegisterBitGroup      Address;
		RegisterBit           Rw;

		SpiControlRegister(AddressingSpace &space, int offset)
			: inherited("Ad9516::SpiControlRegister", space, offset),
			  Data(*this, 0, 8), Address(*this, 8, 10), Rw(*this, 18)
			{
			//  Default setup ????
			}
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS SpiStatusRegister  -- PLL SPI interface
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class SpiStatusRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit           Ready;
		RegisterBit           DataValid;
		RegisterBitGroup      Data;

		SpiStatusRegister(AddressingSpace &space, int offset)
			: inherited("Ad9516::SpiStatusRegister", space, offset),
			  Ready(*this, 22), DataValid(*this, 23), Data(*this, 24, 8)
			{
			//  Default setup ????
			}
    };

	SpiControlRegister  SpiControlReg;
	SpiStatusRegister   SpiStatusReg;
	PllControlRegister  PllControlReg;
	PllStatusRegister   PllStatusReg;

	// Fields
    IIOutputClock       FOutputClock;
    IIReferenceClock    FReferenceClock;
	double				FFrequencyActual;
	bool				FPFDPolarity;

	// Support
	void 			DoSetFrequency(double freq);
	void 			DoSetEnabled(bool state);
	double 			Round(double original);
    void            CalculateDivisors(double & freq);

public:
  	Trace			Debug;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative

#endif

