//
// Ad9516_Tb_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Ad9516_Tb_Mb.h>
#include <SystemSupport_Mb.h>
#include <algorithm>
#include <limits>
#include <cmath>
#include <sstream>
#include <Trace_Mb.h>
#include <sstream>
#include <FloatingPtSupport_Mb.h>

//---------------------------------------------------------------------------

using namespace std;

namespace Innovative
{

//=============================================================================
//  CLASS Ad9516_Tb  --  Ad9516 PLL (TimebaseDevice derived version)
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class Ad9516_Tb
//-----------------------------------------------------------------------------

const unsigned int  Total_Ad9516_Bank0_Outputs     = 6;   // we only support the 6 LVDS outputs

//-----------------------------------------------------------------------------
//  constructor for class Ad9516_Tb
//-----------------------------------------------------------------------------

Ad9516_Tb::Ad9516_Tb(IAd951x_Tb_Hdw * hdw_intf)
    :  Ad951x_Base_Tb(hdw_intf, Total_Ad9516_Bank0_Outputs)
{

}

//-----------------------------------------------------------------------------
//  destructor for class Ad9516_Tb
//-----------------------------------------------------------------------------


Ad9516_Tb::~Ad9516_Tb()
{
}

//-----------------------------------------------------------------------------
//  Ad9516_Tb::Init_DeviceIDCodes()  --  Fill Device ID code list with values
//-----------------------------------------------------------------------------

void Ad9516_Tb::Init_DeviceIDCodes()
{
    DeviceIDCodes.clear();
    DeviceIDCodes.push_back(0x01);
    DeviceIDCodes.push_back(0x41);
    DeviceIDCodes.push_back(0x81);
    DeviceIDCodes.push_back(0x43);
    DeviceIDCodes.push_back(0xC3);
}

//-----------------------------------------------------------------------------
//  Ad9516_Tb::Init_Bank0_OffsetMap()  --  register offsets for bank 0
//-----------------------------------------------------------------------------

void Ad9516_Tb::Init_Bank0_OffsetMap()
{
    Bank0_OffsetMap.clear();
    Bank0_OffsetMap.push_back(0);
    Bank0_OffsetMap.push_back(1);
    Bank0_OffsetMap.push_back(2);
    Bank0_OffsetMap.push_back(3);
    Bank0_OffsetMap.push_back(4);
    Bank0_OffsetMap.push_back(5);
}

//=============================================================================
//  CLASS Ad9517_Tb  --  Ad9516 PLL (TimebaseDevice derived version)
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class Ad9517_Tb
//-----------------------------------------------------------------------------

const unsigned int  Total_Ad9517_Bank0_Outputs     = 4;   // we support all outputs
const unsigned int  Total_Ad9517_Bank1_Outputs     = 4;

//-----------------------------------------------------------------------------
//  constructor for class Ad9517_Tb
//-----------------------------------------------------------------------------

Ad9517_Tb::Ad9517_Tb(IAd951x_Tb_Hdw * hdw_intf)
    :  Ad951x_Base_Tb(hdw_intf, Total_Ad9517_Bank0_Outputs, Total_Ad9517_Bank1_Outputs)
{

}

//-----------------------------------------------------------------------------
//  destructor for class Ad9517_Tb
//-----------------------------------------------------------------------------

Ad9517_Tb::~Ad9517_Tb()
{
}

//-----------------------------------------------------------------------------
//  Ad9517_Tb::Init_DeviceIDCodes()  --  Fill Device ID list with values
//-----------------------------------------------------------------------------

void Ad9517_Tb::Init_DeviceIDCodes()
{
    DeviceIDCodes.clear();
    DeviceIDCodes.push_back(0x11);
    DeviceIDCodes.push_back(0x51);
    DeviceIDCodes.push_back(0x91);
    DeviceIDCodes.push_back(0x53);
    DeviceIDCodes.push_back(0xD3);
}

//-----------------------------------------------------------------------------
//  Ad9517_Tb::Init_Bank0_OffsetMap()  --  register offsets for bank 0
//-----------------------------------------------------------------------------

void Ad9517_Tb::Init_Bank0_OffsetMap()
{
    Bank0_OffsetMap.clear();
    Bank0_OffsetMap.push_back(0);
    Bank0_OffsetMap.push_back(1);
    Bank0_OffsetMap.push_back(4);
    Bank0_OffsetMap.push_back(5);
}

//=============================================================================
//  CLASS Ad951x_Base_Tb  --  Ad9516 PLL (TimebaseDevice derived version)
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class Ad951x_Base_Tb
//-----------------------------------------------------------------------------

const unsigned int  TotalReferences  = 2;   // 3?

static struct VcoRangesType {
    double Low;
    double High;
} AD951xVcoRanges[] = {
    {2550.0e6, 2950.0e6},// AD951x-0 default values
    {2300.0e6, 2650.0e6},// AD951x-1 default values
    {2050.0e6, 2330.0e6},// AD951x-2 default values
    {1750.0e6, 2250.0e6},// AD951x-3 default values
    {1450.0e6, 1800.0e6} // AD951x-4 default values  - default, overridden with chip ID
};

//-----------------------------------------------------------------------------
//  constructor for class Ad951x_Base_Tb
//-----------------------------------------------------------------------------

Ad951x_Base_Tb::Ad951x_Base_Tb(IAd951x_Tb_Hdw * hdw_intf, unsigned int bank0_outputs, unsigned int bank1_outputs)
    : PllTimebaseDevice(TotalReferences, bank0_outputs+bank1_outputs),
      Hdw(hdw_intf),
      Vco(new RangedVcxoControl(AD951xVcoRanges[4].Low, AD951xVcoRanges[4].High)),
      Total_Bank0_LvPecl_Outputs(bank0_outputs), Total_Bank1_Lvds_Outputs(bank1_outputs),
      Total_Outputs(bank0_outputs+bank1_outputs),
      FReferenceSource(refInternal), FOutputSource(outPll),
      FExternalClockSrc(csFrontPanel), FInternalVco(true),
      Debug(true)
{
    Settings.Out_Bank0.resize(Total_Bank0_LvPecl_Outputs);
    Settings.Out_Bank1.resize(Total_Bank1_Lvds_Outputs);

    ConfigureDividerInformation();
}

//-----------------------------------------------------------------------------
//  destructor for class Ad951x_Base_Tb
//-----------------------------------------------------------------------------

Ad951x_Base_Tb::~Ad951x_Base_Tb()
{
    delete Hdw;  Hdw = 0;
    delete Vco;  Vco = 0;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::LoadVco()  --  Replace Vco with a new Vco object
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::LoadVco(IVcxoControl * aVco)
{
    if (aVco)   // we need a valid Vco object to work
        {
        IVcxoControl * tmp = Vco;
        Vco = aVco;
        delete tmp;
        }
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Init()  --  Initialize Chip
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::Init()
{
    Debug.Log("Reset PLL");
    Reset();

    Init_DeviceIDCodes();
    Init_Bank0_OffsetMap();

    if (FInternalVco)
        {
        // Change VCO range, based on PLL device ID
        size_t id = DeviceId();
        if (id < sizeof(AD951xVcoRanges) / sizeof(VcoRangesType))
            VcoRange(AD951xVcoRanges[id].Low, AD951xVcoRanges[id].High);

        stringstream msg;
        msg << "PLL ID: " << id;
        Debug.Log(msg.str());
        }

    SettingsInit();        // common INIT settings
    OnInit();
    Update();    // Update state of the device based on sources

    UpdateRegisters();  // apply changes to hardware.
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SettingsInit()  --  Initial Settings Loading
//-----------------------------------------------------------------------------
//   Note board specific implementations can override these in OnInit

void Ad951x_Base_Tb::SettingsInit()
{
    // Differential reference mode
    Port(0x1c, 0x01);

    // default anti-backlash, status pin DC (default)
    Port(0x17, 0x00);

    // If we don't support the LVDS/LVCMOS outputs, power them down
    if (Total_Bank1_Lvds_Outputs)
        {
        Port(0x140, 0x01);       // OUT6 power-down
        Port(0x141, 0x01);       // OUT7 power-down
        Port(0x142, 0x01);       // OUT8 power-down
        Port(0x143, 0x01);       // OUT9 power-down
        }

    //  Default LvPecl voltage level to chip default
    for (unsigned int i=0; i<Total_Bank0_LvPecl_Outputs; i++)
        SetOutVoltageHdw(i, v780mv);
    //  Default Lvds current level to chip default
    for (unsigned int i=Total_Bank0_LvPecl_Outputs; i<Total_Outputs; i++)
        SetOutCurrentHdw(i, c3500uA);

    Settings.Pll.PFDPolarity = true;
    Settings.Pll.ChargePumpCurrent = cpc4800;
    Settings.Pll.ChargePumpMode = cpmNormal;
    Settings.Pll.PllPowerDown = true;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Reapply()  --  Reapply settings to hardware from saved state
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::Reapply()
{
    OnReapply();

    Update();    // Update state of the device based on sources

    //  Update SelA, SelB bits & set to hardware
    bool selA;
    bool selB;
    bool selC;
    if (FReferenceSource==refInternal)
        selB = false;
    else
        selB = true;
    if (FExternalClockSrc==csFrontPanel)
        selA = false;
    else
        selA = true;
    if (FOutputSource==outPll)
        selC = false;
    else
        selC = true;
    Hdw->SetSelA(selA);
    Hdw->SetSelB(selB);
    Hdw->SetSelC(selC);

//    WriteDataRegisters();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::InternalVco()  --  Set external vco
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::InternalVco(bool state)
{
    FInternalVco = state;
    if (state)
        ValidVcoDividerValues = ValidVcoDividerValues_InternalVco;
    else
        ValidVcoDividerValues = ValidVcoDividerValues_ExternalVco;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::ExternalClockSelect()  --  Set external clock selection
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::ExternalClockSelect(AdExternalSelect value)
{
    FExternalClockSrc = value;
    SetExternalClockSelect(value);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::ExternalClockSelect()  --  Read external clock selection
//-----------------------------------------------------------------------------

Ad951x_Base_Tb::AdExternalSelect  Ad951x_Base_Tb::ExternalClockSelect() const
{
    return FExternalClockSrc;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::DeviceId()  --  Read PLL device ID
//-----------------------------------------------------------------------------

size_t Ad951x_Base_Tb::DeviceId()
{
    unsigned int value = Port(0x03) & 0xff;

    size_t result = -1;
    for (size_t i = 0; i < DeviceIDCodes.size(); ++i)
        if (value == DeviceIDCodes[i])
            result = i;

    return result;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::FrequencyModeUpdate()  --  Configure outputs to a set frequency
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::FrequencyModeUpdate()
{
    ScopedTraceRef ccc(Debug, "Ad951x_Base_Tb::FrequencyModeUpdate");

    if ((Frequency() == 0.0) || (Reference() == 0.0))
        return;

    // initialize device serial port
    InitSerialPort();

    //  Calculate our VCO Frequency (and maybe VcoDivider)
    CalculateDividers();

    // Calculate N and R using PhaseDetectorFrequency and VcoFrequency
    unsigned int R;
    unsigned int N;
    CalculateNR(N, R);

    //  Display "actual" Vco frequency
    Settings.VcoFrequency = Reference() * N / R;
    Debug.Log("Ad9516::Actual Vco Freq = ", Settings.VcoFrequency);

    //  Calculate PBA trio  ...N = P*B+A
    unsigned int P(0);
    unsigned int B(0);
    unsigned int A(0);
    AdvancedPBACalculation(N, P, B, A, Settings.VcoFrequency);

    Debug.Log("Ad951x::N = ", N);
    Debug.Log("Ad951x::R = ", R);
    Debug.Log("Ad951x::P = ", P);
    Debug.Log("Ad951x::B = ", B);
    Debug.Log("Ad951x::A = ", A);

    Debug.Log("Pll Tuning Event");
    Ad951xPllTuningEvent e(this, R, P, B, A);
    OnPllTuning.Execute(e);

    Debug.Log("Final Values");
    Debug.Log("Ad951x::N = ", N);
    Debug.Log("Ad951x::R = ", R);
    Debug.Log("Ad951x::P = ", P);
    Debug.Log("Ad951x::B = ", B);
    Debug.Log("Ad951x::A = ", A);

    //  Update VCO Frequency
    Settings.VcoFrequency = Reference() * (P * B + A) / R;
    Debug.Log("Ad951x::Final Vco Freq = ", Settings.VcoFrequency);
    //
    //  Update VCO (this does nothing if VCO is not programmable...
    Vco->SetVcoFrequency(Settings.VcoFrequency);    // reprogram Vcxo

    // Set PLL configuration
    SetR(R);
    SetP(P, B);
    SetB(B);
    SetA(A);

    // Power up PLL
    Settings.Pll.PllPowerDown = false;
    ApplyChargePumpCfg();

    // VCO as input to VCO divider
    SetOutputSource(outPll);

    SetVcoDividerHdw(Settings.VcoDivider);

    SetOutputDividers();

    // update registers
    UpdateRegisters();

    // Cal VCO
    CalibrateVco();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::DividerModeUpdate()  --  Configure outputs by preset dividers
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::DividerModeUpdate()
{
    ScopedTraceRef ccc(Debug, "Ad951x_Base_Tb::DividerModeUpdate");
    if ((Frequency() == 0.0) || (Reference() == 0.0))
        return;

    // initialize device serial port
    InitSerialPort();

    // Route external clock as source
    SetOutputSource(outExtClk);
    // Power-down PLL
    Settings.Pll.PllPowerDown = true;
    ApplyChargePumpCfg();


    CalculateExtClk_Dividers();
    //SetVcoDividerHdw(Settings.VcoDivider);
    //Debug.Log("Vco Divider: ",   Settings.VcoDivider);

    //unsigned int OutDiv = NearestInteger(Reference()/Frequency());
    //OutDiv = std::max<unsigned int>(OutDiv, 1);

    SetOutputDividers();

    // update registers
    UpdateRegisters();

    // Cal VCO
    CalibrateVco();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::CalculateDividers()  --  Derive best-fit divisors
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::CalculateDividers()          // (or vco rate?)
{
    //
    //  Start with the master frequency - see if direct output mode works
    if (Vco->CanTuneTo(Frequency()))
        {
        //  Bypass mode
        Settings.VcoFrequency = Frequency();
        Settings.VcoDivider = 0;      // undetermined so far
        Settings.Master.PassThrough = true;
        Settings.Master.Divider = 1;  // don't care
        }
    else
        {
        unsigned int outdiv(1);
        unsigned int vcodiv(1);
        // we will use the divider chain - Freq_in must be < 1600 MHz
        bool in_range = false;
        for(ValidDividerArray::iterator vcoIter = ValidVcoDividerValues.begin();
                    vcoIter != ValidVcoDividerValues.end(); ++vcoIter)
            {
            vcodiv = *vcoIter;

            for(ValidDividerArray::reverse_iterator divIter = ValidDividerValues.rbegin();
                    divIter != ValidDividerValues.rend(); ++divIter)
                {
                outdiv = *divIter;
                if (Vco->CanTuneTo(Frequency() * outdiv * vcodiv))
                    {
                    //  Frequency at OutDivider start must be less than 1600 MHz
                    if (Frequency() * outdiv <= 1600.0e6)
                        {
                        in_range = true;
                        Settings.VcoFrequency = Frequency() * outdiv * vcodiv;
                        Settings.VcoDivider = vcodiv;
                        Settings.Master.PassThrough = false;
                        Settings.Master.Divider = outdiv;
                        }
                    break;
                    }
                }
            if (in_range)
                break;
            }
        //  We failed to find a match, so use an endpoint
        if(!in_range)
            {
            if (Settings.VcoFrequency > Vco->Max())
                {
                Settings.VcoFrequency = Vco->Max();
                Settings.VcoDivider = ValidVcoDividerValues[0];  // pick smallest vco divider
                }
            else
                {
                Settings.VcoFrequency = Vco->Min();
                Settings.VcoDivider = ValidVcoDividerValues[0];  // pick smallest vco divider
                }
            //Settings.VcoDivider = vcodiv;     // I give up
            Settings.Master.PassThrough = false;
            Settings.Master.Divider = outdiv; // I give up
            }
        }
    //
    //  If we didn't find a good VCO divider from the main frequency, try again
    if (Settings.VcoDivider == 0)
        {
        FindVcoDivider();
        }
    Debug.Log("Vco Frequency: ", Settings.VcoFrequency);
    Debug.Log("Vco Divider: ",   Settings.VcoDivider);
    Debug.Log("Master Passthru: ", Settings.Master.PassThrough);
    Debug.Log("Master Divider: ",  Settings.Master.Divider );

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::CalculateExtClk_Dividers()  --  Ext Clock Mode calculation
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::CalculateExtClk_Dividers()
{
    //  We know the "VCO" frequency
    Settings.VcoFrequency = Reference();  // Actually the input freq to dividers
    Settings.VcoDivider = 1;   // we don't use it, so set it to this invalid value

    unsigned int outdiv;

    //  First, hunt for an exact matching divider set
    // we will use the divider chain - Freq_in must be < 1600 MHz
    bool in_range = false;

    for(ValidDividerArray::reverse_iterator divIter = ValidDividerValues.rbegin();
            divIter != ValidDividerValues.rend(); ++divIter)
        {
        outdiv = *divIter;
        if (IsAboutEqual(Settings.VcoFrequency, Frequency() * outdiv ))
            {
            //  Frequency at OutDivider start must be less than 1600 MHz
            if (Frequency() * outdiv <= 1600.0e6)
                {
                in_range = true;
                Settings.Master.PassThrough = (outdiv==1) ? true : false;
                Settings.Master.Divider = outdiv;
                }
            break;
            }
        }

        //  We failed to find an exact match, so lets use a coarser method
        if (!in_range)
            {
            float TotalDivider = static_cast<float>(Settings.VcoFrequency / Frequency());
            int outdiv_below;
            int outdiv_above;
            Ad951xGetNearestDividers(TotalDivider, outdiv_below, outdiv_above);

            //  Frequency at OutDivider start must be less than 1600 MHz
            if (Frequency() * outdiv_above <= 1600.0e6)
                {
                in_range = true;
                Settings.Master.PassThrough = (outdiv_above==1) ? true : false;
                Settings.Master.Divider = outdiv_above;
                }
            else if (Frequency() * outdiv_below <= 1600.0e6)
                {
                in_range = true;
                Settings.Master.PassThrough = (outdiv_below==1) ? true : false;
                Settings.Master.Divider = outdiv_below;
                }
        }

    Debug.Log("Reference: ",  Settings.VcoFrequency);
    //Debug.Log("Vco Divider: ",   Settings.VcoDivider);
    Debug.Log("Master Passthru: ", Settings.Master.PassThrough);
    Debug.Log("Master Divider: ",  Settings.Master.Divider );

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::FindVcoDivider() -- Find reasonable VCO divider
//-----------------------------------------------------------------------------
//  Settings.VcoFrequency is already set to correct rate

void Ad951x_Base_Tb::FindVcoDivider()
{
    unsigned int vcodiv(1);
    for (ValidDividerArray::iterator vcoIter = ValidVcoDividerValues.begin();
                    vcoIter != ValidVcoDividerValues.end(); ++vcoIter)
        {
        vcodiv = *vcoIter;
        double DividerInFrequency = Settings.VcoFrequency/vcodiv;
        //
        //  Constraint #1: Input Rate to Dividers can't exceed 1600.0 MHz
        if ( DividerInFrequency > 1600.0e6)
            continue;   // this divider is no good
        //
        //  If Master IS in bypass mode, use another input to find divider range
        bool all_ok = true;
        for (unsigned int output=0; output<Total_Outputs; output++)
            {
            //  If this output is unused, the Master, or slaved to Master, skip it
            if (OutputInUse(output)==false || (output==FMasterOutput) ||
                    (OutputFrequency(output)==0.0))
                continue;   // if we don't use it, skip it
            if (IsAboutEqual(Settings.VcoFrequency, OutputFrequency(output)))
                continue;   // don't use if also a bypass
            //
            //  if we can't cover this frequency in range, fail and keep looking
            if (((DividerInFrequency/MaxOutputDivider()<=OutputFrequency(output))
                         && (OutputFrequency(output)<=DividerInFrequency)) == false)
                {
                all_ok = false;
                break;
                }
            }
        if (all_ok)
            {
            Settings.VcoDivider = vcodiv;
            break;
            }
        }
    if (Settings.VcoDivider == 0)
        {
        // as nobody cares, pick a legal value
        Settings.VcoDivider = vcodiv;
        }
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::CalculateNR() -- Note: Passed by reference!
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::CalculateNR(unsigned int & N, unsigned int & R)
{
    // 1. Since we want F_Ref / R = PhaseDetectorFrequency:
    R = static_cast<int>(std::floor(Reference() / PhaseDetectorFrequency() + 0.5f));
    N = static_cast<int>(std::floor(Settings.VcoFrequency * R/Reference()  + 0.5f));
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::AdvancedPBACalculation()  --  Advanced Calculation
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::AdvancedPBACalculation(unsigned int N, unsigned int & P,
                            unsigned int & B, unsigned int & A, double VcoRate)
{
    // calc P, B, A terms such that N = P*B+A
    Debug.Log("Ad9511 Advanced P,B,A for VcoRate: ", VcoRate);
    Debug.Log("   and N: ", N);
    //  We need to pick a P that reduces B and A input to 300 MHz, and does
    //    not violate P input rate restrictions for Dual Modulus Modes
    bool DM_Mode(false);
    if (VcoRate<=300.0e6)
        {
        P = 1; DM_Mode = false;
        }
    else if (VcoRate<=600.0e6)
        {
        P = 2;  DM_Mode = false;
        }
    else if (VcoRate<=900.0e6)
        {
        P = 3;  DM_Mode = false;
        }
    else if (VcoRate<=1000.0e6)
        {
        P = 4;  DM_Mode = true;
        }
    else if (VcoRate<=2400.0e6)
        {
        P = 8;  DM_Mode = true;
        }
    else if (VcoRate<=3000.0e6)
        {
        P = 16;  DM_Mode = true;
        }
    else  // also 3000.0e6
        {
        P = 32;  DM_Mode = true;
        }
    //
    //  Calculate B from P and N
    static const unsigned int MaxB = 1 << 13;
    B = std::max(1u, std::min(N/P, MaxB-1));
    if (B == 2)
        {
        if (DM_Mode)
            {
            B = 1;  // A will cover the gap
            }
        else
            {
            B = 1;
            P = 2;  // single mod P=2
            }
        }
    //
    //  Calculate A
    if (DM_Mode)
        A = std::max(std::min(N - (P * B), 63u), 0u);
    else
        A = 0;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutputDividers()  --
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::SetOutputDividers()
{
    double DividerInFrequency = Settings.VcoFrequency/Settings.VcoDivider;

    for (unsigned int output=0; output<Total_Outputs; output++)
        {
        if (OutputInUse(output)==false)
            {
            OutputFreqActualArray[output] = 0.0;
            continue;
            }
        //
        //  If the output is the Master, or follows the master we already
        //    know the divider settings
        bool    PassThrough;
        float   CandidateDivider;
        double  CandidateOutputFreq;

        if ((output==FMasterOutput) || (OutputFrequency(output)==0.0))
            {
            PassThrough = Settings.Master.PassThrough;
            CandidateDivider = static_cast<float>(Settings.Master.Divider);
            CandidateOutputFreq = Frequency();
            }
        else
            {
            CandidateOutputFreq = OutputFrequency(output);
            CandidateDivider = static_cast<float>(DividerInFrequency / CandidateOutputFreq);
            if (IsAboutEqual(Settings.VcoFrequency, CandidateOutputFreq))
                {
                PassThrough = true;
                CandidateDivider = 1.0;  // make sure this is valid divider
                }
            else if (CandidateDivider<1.0)
                {
                PassThrough = true;
                CandidateDivider = 1.0;  // make sure this is valid divider
                }
            else
                {
                int above, below, divider;
                GetNearestDividers(output, CandidateDivider, below, above);

                // Find which divider is closest to desired frequency
                double centerFreq = DividerInFrequency/CandidateDivider;
                double hiFreq     = DividerInFrequency/below;
                double lowFreq    = DividerInFrequency/above;
                if (std::abs(centerFreq-lowFreq) > std::abs(hiFreq-centerFreq))
                    divider = below;
                else
                    divider = above;

                PassThrough = false;
                CandidateDivider = static_cast<float>(divider);
                }
            }

        //  Apply to hardware
        Debug.Log("Output: ",     output);
        Debug.Log("Passthrough: ", PassThrough);
        Debug.Log("Divider: ",    static_cast<unsigned int>(CandidateDivider));

        OutputDivider(output, static_cast<unsigned int>(CandidateDivider));
        SetDividerPassThrough(output, PassThrough);
        //
        //  Calculate Actual Frequency
        if (OutputDividerArray[output]!=0)  // if illegal divider, skip it
            {
            if (PassThrough)
                OutputFreqActualArray[output] = Settings.VcoFrequency;
            else
                OutputFreqActualArray[output] = DividerInFrequency / OutputDividerArray[output];
            }
        Debug.Log("Out Freq: ",   OutputFrequency(output));
        Debug.Log("FreqActual: ", OutputFreqActualArray[output]);
        }
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::InitSerialPort()  --
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::InitSerialPort()
{
    Port(0x00, 0x18);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::UpdateRegisters()  --
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::UpdateRegisters()
{
    Port(0x232, 0x01);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::CalibrateVco()  --
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::CalibrateVco()
{
    Port(0x18, 00);
    Port(0x232, 0x01);
    Port(0x18, 01);
    Port(0x232, 0x01);

    Sleep(1);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Max()  --  Set maximum output rate
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::Max(double value)
{
     Vco->SetRange(Vco->Min(), value);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Max()  --  Return maximum output rate
//-----------------------------------------------------------------------------

double Ad951x_Base_Tb::Max() const
{
    return Vco->Max();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Min()  --  Set minimum output rate
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::Min(double value)
{
    Vco->SetRange(value * MaxOutputDivider(), Vco->Max());
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Min()  --  Return minimum output rate
//-----------------------------------------------------------------------------

double Ad951x_Base_Tb::Min() const
{
    return Vco->Min() / MaxOutputDivider();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::MaxOutputDivider()  --  Return max output post-divider
//-----------------------------------------------------------------------------

int Ad951x_Base_Tb::MaxOutputDivider() const
{
    return 32;        // this ignores the VcoDivider
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Locked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  Ad951x_Base_Tb::Locked() const
{
    return Hdw->IsLocked();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::PFDPolarity()  --  Set Phase-Frequency Detector (PFD) Polarity
//                             0 = Negative (Default), 1 = Positive.
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::PFDPolarity(bool state)
{
    Settings.Pll.PFDPolarity = state;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::PFDPolarity()  --  Return Phase-Frequency Detector (PFD) Polarity
//-----------------------------------------------------------------------------

bool Ad951x_Base_Tb::PFDPolarity() const
{
    return Settings.Pll.PFDPolarity;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Reset()  --  Reset state of all internal registers
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::Reset()
{
    Hdw->ResetPll();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Port()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::Port(unsigned short reg, unsigned char value)
{
    Hdw->SpiPort(reg, value);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Port()  --  Read from PLL data port
//-----------------------------------------------------------------------------

unsigned char Ad951x_Base_Tb::Port(unsigned short reg)
{
    unsigned char result;
    result = Hdw->SpiPort(reg);
    return result;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetReferenceSource()  --  Set reference source input
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetReferenceSource(AdReferenceSource source)
{
    unsigned int source_no;
    switch (source)
        {
        default:
        case refExternal:
            source_no = 1;
            break;
        case refInternal:
            source_no = 0;
            break;
        }

    //  (We use an external mux to change reference sources)
    //  Differential reference mode
    Port(0x1c, 0x01);

    SetSourceSelection(source_no);  // inform base class
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutputSource()  --  Set the Source for the divider matrix
//-----------------------------------------------------------------------------
//    The output divider matrix can configured in three ways
//       0x00 - use external in through VCO divider (required if rate > 1600.0 MHz
//       0x01 - bypass VCO divider (and passthru)
//       0x02 - use PLL through VCO divider
//

void Ad951x_Base_Tb::SetOutputSource(AdOutputSource source)
{
    unsigned int cfg_setting;
    switch (source)
        {
        default:
        case outPll:        cfg_setting = (FInternalVco) ? 0x2 : 0x1 ; break;
        case outExtClk:     cfg_setting = (Reference()<=1600e6) ? 0x01 : 0x0; break;
        }

    Port(0x1e1, cfg_setting);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetExternalClockSelect()  --  Set the Source for the divider matrix
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetExternalClockSelect(AdExternalSelect /*source*/)
{
    // unimplemented?
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetDividerPassThrough()  --  Set direct pass thru mode
//-----------------------------------------------------------------------------
//  Bank 0 only

void Ad951x_Base_Tb::SetDividerPassThrough(unsigned int output, bool pass_thru)
{
    if (output >= Total_Bank0_LvPecl_Outputs)
        return;

    // Apply to hardware
    int divider_offset = ToBank0_Offset(output)/2;      // 2 outputs share common divider
    int channel_offset = divider_offset*3;

    //  Divider Register 2 - Passthru/DCCOFF setting
    PtrBitManipulator Reg_D2(&Settings.Out_Bank0[output].DividerCfgReg[2]);
    Reg_D2.Bit(1, pass_thru);
    Port(0x192+channel_offset, Reg_D2.Value()); // Apply to hardware
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetDividerHdw()  --  Apply the divider state to hdw
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetDividerHdw(unsigned int output, unsigned int divider_setting)
{
    if (output >= Total_Outputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)
        SetDividerHdw_Bank0(output, divider_setting);
    else
        SetDividerHdw_Bank1(output-Total_Bank0_LvPecl_Outputs, divider_setting);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetDividerHdw_Bank0()  --  Bank 0 Impl
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetDividerHdw_Bank0(unsigned int output, unsigned int divider_setting)
{
    // output validity already checked
    LvPeclSettings & OutSettings(Settings.Out_Bank0[output]);

    // calculate duty-cycle bits and bypass for divider
    int outdiv_bits = GetOutdivBits(divider_setting, OutSettings.DutyCycle);

    int outdiv_bypass =(divider_setting == 1)  ? 0x80 : 0x0 ;

    bool dccoff_bit = (OutSettings.DutyCycle==0.0) ? false : true;

    // Apply to hardware
    int divider_offset = ToBank0_Offset(output)/2;      // 2 outputs share common divider
    int channel_offset = divider_offset*3;
    Port(0x190+channel_offset, outdiv_bits);
    // Enable/bypass divider
    Port(0x191+channel_offset, outdiv_bypass);   // Possible option to start high?

    //  Divider Register 2 - Passthru/DCCOFF setting
    PtrBitManipulator Reg_D2(&OutSettings.DividerCfgReg[2]);
    Reg_D2.Bit(0, dccoff_bit);
    Port(0x192+channel_offset, Reg_D2.Value()); // Apply to hardware

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetDividerHdw_Bank1()  --  Bank 0 Impl
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetDividerHdw_Bank1(unsigned int output, unsigned int divider_setting)
{
    // output validity already checked

    LvdsSettings & OutSettings(Settings.Out_Bank1[output]);

    // calculate duty-cycle bits and bypass for divider
    int outdiv_bits = GetOutdivBits(divider_setting, OutSettings.DutyCycle);

    bool outdiv_bypass_0 = (divider_setting == 1);
    bool outdiv_bypass_1 = true;   // always bypass second stage

    bool dccoff_bit = (OutSettings.DutyCycle==0.0) ? false : true;

    int divider_offset = output/2;      // 2 outputs share common divider
    int channel_offset = divider_offset*5;

    //  Divider Register 0 - Stage 1 divider bits
    PtrBitManipulator Reg_D0(&OutSettings.DividerCfgReg[0]);
    Reg_D0.Value(outdiv_bits);
    Port(0x199+channel_offset, Reg_D0.Value()); // Apply to hardware
    //  Divider Register 1 - Phase offset bits (ignored)
    //PtrBitManipulator Reg_D1(&OutSettings.DividerCfgReg[1]);
    //Reg_D1.Value( ??? );
    //Port(0x19a+channel_offset, Reg_D1.Value()); // Apply to hardware
    //  Divider Register 2 - Stage 2 divider bits
    PtrBitManipulator Reg_D2(&OutSettings.DividerCfgReg[2]);
    Reg_D2.Value(0);  //  ...don't matter, always bypassed
    Port(0x19b+channel_offset, Reg_D2.Value()); // Apply to hardware
    //  Divider Register 3 - Bypass bits
    PtrBitManipulator Reg_D3(&OutSettings.DividerCfgReg[3]);
    Reg_D3.Bit(4, outdiv_bypass_0);
    Reg_D3.Bit(5, outdiv_bypass_1);
    Port(0x19c+channel_offset, Reg_D3.Value()); // Apply to hardware
    //  Divider Register 4 - DCCOFF setting
    PtrBitManipulator Reg_D4(&OutSettings.DividerCfgReg[4]);
    Reg_D4.Bit(0, dccoff_bit);
    Port(0x19d+channel_offset, Reg_D4.Value()); // Apply to hardware
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::GetOutdivBits()  --  Calculate OutDiv value
//-----------------------------------------------------------------------------

unsigned int  Ad951x_Base_Tb::GetOutdivBits(unsigned int divider_setting, float duty_cycle )
{
    unsigned int lo;
    unsigned int hi;
    unsigned int odd;

    if (duty_cycle==0.0)
        {
        // calculate 50 pct duty-cycle bits for divider
        odd = divider_setting & 1;
        lo = (divider_setting >> 1) - 1;
        hi = lo + odd;
        }
    else
        {
        hi = static_cast<unsigned int>(divider_setting*duty_cycle);
        lo = divider_setting - hi;
        hi -= 1;
        lo -= 1;
        if (hi>0xF || lo>0xF)  // if parts are invalid, recalc using 50 pct duty cycle
            return GetOutdivBits(divider_setting, 0.0);
        }

    return ( (lo << 4) | hi );
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutputDutyCycle()  --  Set non default duty cycle
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetOutputDutyCycle(unsigned int output, float duty_cycle)
{
    if (output >= Total_Outputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)
        Settings.Out_Bank0[output].DutyCycle = duty_cycle;
    else
        Settings.Out_Bank1[output-Total_Bank0_LvPecl_Outputs].DutyCycle = duty_cycle;

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutEnabledHdw()  --  Apply the enable/disable state to hdw
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetOutEnabledHdw(unsigned int output, bool enabled)
{
    if (output >= Total_Outputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)
        SetOutEnabledHdw_Bank0(output, enabled);
    else
        SetOutEnabledHdw_Bank1(output-Total_Bank0_LvPecl_Outputs, enabled);

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutEnabledHdw_Bank0()  --  Bank 0 Impl
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetOutEnabledHdw_Bank0(unsigned int output, bool enabled)
{
    // output valididty already checked
    unsigned int cfg_setting = (enabled) ? 0x0 : 0x3 ;

    PtrBitManipulator Reg(&Settings.Out_Bank0[output].OutputCfgReg);
    Reg.Field(0, 2, cfg_setting);

    // Apply to hardware
    Port(0xF0+ToBank0_Offset(output), Reg.Value());
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutEnabledHdw_Bank1()  --  Bank 1 Impl
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetOutEnabledHdw_Bank1(unsigned int output, bool enabled)
{
    // output valididty already checked
    PtrBitManipulator Reg(&Settings.Out_Bank1[output].OutputCfgReg);
    Reg.Bit(0, !enabled);

    // Apply to hardware
    Port(0x140+output, Reg.Value());
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutVoltageHdw()  --  Apply the voltage level state to hdw
//-----------------------------------------------------------------------------
//  Bank 0 only (LvPecl)

void Ad951x_Base_Tb::SetOutVoltageHdw(unsigned int output, unsigned int volt_level)
{
    if (output >= Total_Bank0_LvPecl_Outputs)
        return;

    PtrBitManipulator Reg(&Settings.Out_Bank0[output].OutputCfgReg);
    Reg.Field(2, 2, volt_level);

    // Apply to hardware
    Port(0xF0+ToBank0_Offset(output), Reg.Value());

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutCurrentHdw()  --  Apply the current state to hdw
//-----------------------------------------------------------------------------
//  Bank 1 only (Lvds)

void Ad951x_Base_Tb::SetOutCurrentHdw(unsigned int output, unsigned int current)
{
    if (output >= Total_Outputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)
        return;

    output -= Total_Bank0_LvPecl_Outputs;

    PtrBitManipulator Reg(&Settings.Out_Bank1[output].OutputCfgReg);
    Reg.Field(1, 2, current);

    // Apply to hardware
    Port(0x140+output, Reg.Value());

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetVcoDividerHdw()  --  Apply the divider state to hdw
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::SetVcoDividerHdw(unsigned int divider_setting)
{
    if (IsValidVcoDivider(divider_setting) == false)
        return;

    Port(0x1e0, Ad951x_ValidVcoDividers[divider_setting]);

}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::ApplyChargePumpCfg()  --  from Settings.Pll
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::ApplyChargePumpCfg()
{
    unsigned int cfg_setting = 0;

    PtrBitManipulator Reg(&cfg_setting);
    Reg.Field(0, 2, ( Settings.Pll.PllPowerDown) ? 0x01 : 0x00 );
    Reg.Field(2, 2, static_cast<unsigned int>(Settings.Pll.ChargePumpMode));
    Reg.Field(4, 3, static_cast<unsigned int>(Settings.Pll.ChargePumpCurrent));
    Reg.Bit(7, Settings.Pll.PFDPolarity!=true);   // 0 = positive, 1 = negative

    Port(0x10, cfg_setting);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetOutputConfig()  --  Set Bank 1 output configuration
//-----------------------------------------------------------------------------
//  If output is on bank 0, skip config

void  Ad951x_Base_Tb::SetOutputConfig(unsigned int output, unsigned int cfg_setting)
{
    if (output >= Total_Outputs)
        return;

    if (output < Total_Bank0_LvPecl_Outputs)  // no configuration for LvPecl bank
        return;
    output -= Total_Bank0_LvPecl_Outputs;   // adjust to bank 1 base

    //  write output config, bank 1
    PtrBitManipulator Reg(&Settings.Out_Bank1[output].OutputCfgReg);
    Reg.Bit(3, cfg_setting ? true : false);

    // Apply to hardware
    Port(0x140+output, Reg.Value());
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetR()  --  Set PLL R value
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::SetR(unsigned int R)
{
    Port(0x11, (R & 0xff));      // R LSBs
    Port(0x12, (R >> 8));        // R MSBs
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetP()  --  Set PLL P value
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::SetP(unsigned int P, unsigned int B)
{
    // encode P value for register use
    int P_bits;
    switch(P)
        {
        default:
        case  1:    P_bits = 0;        break;
        case  2:    P_bits = 1;        break;
        case  3:    P_bits = 7;        break;
        case  4:    P_bits = 3;        break;
        case  8:    P_bits = 4;        break;
        case 16:    P_bits = 5;        break;
        case 32:    P_bits = 6;        break;
        }
    Port(0x16, ((B == 1) ? 0x08 : 0x0) | P_bits);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetA()  --  Set PLL P value
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::SetA(unsigned int A)
{
      // Load A register
    Port(0x13, A);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::SetB()  --  Set PLL B value
//-----------------------------------------------------------------------------

void  Ad951x_Base_Tb::SetB(unsigned int B)
{
    // Load both portions of B register
    Port(0x14, (B & 0xff));
    Port(0x15, (B >> 8));
}

//-----------------------------------------------------------------------------
// Ad951x_Base_Tb::ConfigureDividerInformation()
//-----------------------------------------------------------------------------

void Ad951x_Base_Tb::ConfigureDividerInformation()
{
    ValidDividerValues.push_back(1);  ValidDividerValues.push_back(2);
    ValidDividerValues.push_back(4);  ValidDividerValues.push_back(6);
    ValidDividerValues.push_back(8);  ValidDividerValues.push_back(10);
    ValidDividerValues.push_back(12); ValidDividerValues.push_back(14);
    ValidDividerValues.push_back(16); ValidDividerValues.push_back(18);
    ValidDividerValues.push_back(20); ValidDividerValues.push_back(22);
    ValidDividerValues.push_back(24); ValidDividerValues.push_back(26);
    ValidDividerValues.push_back(28); ValidDividerValues.push_back(30);
    ValidDividerValues.push_back(32);

    //  Fill the conversion information structures
    //     Valid dividers is keyed by divide_by, value is Input Value
    Ad951x_ValidDividers[1]  = 0xF0;       Ad951x_ValidDividers[2]  = 0x00;
    Ad951x_ValidDividers[4]  = 0x11;       Ad951x_ValidDividers[6]  = 0x22;
    Ad951x_ValidDividers[8]  = 0x33;       Ad951x_ValidDividers[10] = 0x44;
    Ad951x_ValidDividers[12] = 0x55;       Ad951x_ValidDividers[14] = 0x66;
    Ad951x_ValidDividers[16] = 0x77;       Ad951x_ValidDividers[18] = 0x88;
    Ad951x_ValidDividers[20] = 0x99;       Ad951x_ValidDividers[22] = 0xaa;
    Ad951x_ValidDividers[24] = 0xbb;       Ad951x_ValidDividers[26] = 0xcc;
    Ad951x_ValidDividers[28] = 0xdd;       Ad951x_ValidDividers[30] = 0xee;
    Ad951x_ValidDividers[32] = 0xff;

    ValidVcoDividerValues_InternalVco.push_back(2);  ValidVcoDividerValues_InternalVco.push_back(3);
    ValidVcoDividerValues_InternalVco.push_back(4);  ValidVcoDividerValues_InternalVco.push_back(5);
    ValidVcoDividerValues_InternalVco.push_back(6);

    ValidVcoDividerValues_ExternalVco.push_back(1);

    ValidVcoDividerValues = ValidVcoDividerValues_InternalVco;

    //  Fill the conversion information structures
    //     Valid dividers is keyed by divide_by, value is Input Value
    Ad951x_ValidVcoDividers[2]  = 0x00;       Ad951x_ValidVcoDividers[3]  = 0x01;
    Ad951x_ValidVcoDividers[4]  = 0x02;       Ad951x_ValidVcoDividers[5]  = 0x03;
    Ad951x_ValidVcoDividers[6]  = 0x04;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::IsValidDivider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  Ad951x_Base_Tb::IsValidDivider(unsigned int divide_by)
{
    ValidDividerMap::iterator iter = Ad951x_ValidDividers.find(divide_by);
    return iter != Ad951x_ValidDividers.end();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::IsValidVcoDivider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  Ad951x_Base_Tb::IsValidVcoDivider(unsigned int divide_by)
{
    ValidDividerMap::iterator iter = Ad951x_ValidVcoDividers.find(divide_by);
    return iter != Ad951x_ValidVcoDividers.end();
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::Ad951xGetNearestDividers()  -- find divider set around a candidate
//-----------------------------------------------------------------------------

void   Ad951x_Base_Tb::Ad951xGetNearestDividers(float candidate, int & below, int & above)
{
    if (IsAboutEqual(candidate, NearestIntegerPart(candidate)))
        {
        //  We are essentially an integer, so treat as such.
        unsigned int int_candidate(static_cast<unsigned int>(NearestInteger(candidate)));
        if (IsValidDivider(int_candidate))
            {
            above = below = int_candidate;
            return;
            }
        }
    above = BestDividerGreaterThan(candidate);
    below = BestDividerLessThan(candidate);
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::BestDividerGreaterThan() -- returns next largest valid Divider
//-----------------------------------------------------------------------------

unsigned int  Ad951x_Base_Tb::BestDividerGreaterThan(float min_divide_by)
{
    unsigned int CandidateD = 0;
    for (ValidDividerArray::iterator div = ValidDividerValues.begin();
             div !=  ValidDividerValues.end();  ++div)
        {
        CandidateD = *div;
        if (static_cast<float>(CandidateD) > min_divide_by)
            break;
        }
    return CandidateD;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::BestDividerLessThan() -- returns next smallest valid Divider
//-----------------------------------------------------------------------------

unsigned int  Ad951x_Base_Tb::BestDividerLessThan(float max_divide_by)
{
    unsigned int CandidateD = 0;
    for (ValidDividerArray::reverse_iterator div = ValidDividerValues.rbegin();
             div !=  ValidDividerValues.rend();  ++div)
        {
        CandidateD = *div;
        if (static_cast<float>(CandidateD) < max_divide_by)
            break;
        }
    return CandidateD;
}

//-----------------------------------------------------------------------------
//  Ad951x_Base_Tb::ToBank0_Offset() -- map channel number to register offset
//-----------------------------------------------------------------------------
//  For output bank 0

unsigned int Ad951x_Base_Tb::ToBank0_Offset(unsigned int ch)
{
    if (ch < Bank0_OffsetMap.size())
        return Bank0_OffsetMap[ch];
    else
        return 0;
}


//=============================================================================
//  CLASS BasicAd9516_Tb_Hdw  --  Hardware Interface for Ad951x_Base_Tb
//=============================================================================
//  "normal" version (X5)

//-----------------------------------------------------------------------------
//  constructor for class BasicAd9516_Tb_Hdw
//-----------------------------------------------------------------------------

BasicAd9516_Tb_Hdw::BasicAd9516_Tb_Hdw(IAddressingSpace & space, MapRegisters & regs)
    : SpiControlReg(space, regs.PllSpi_Addr), SpiStatusReg(space, regs.PllSpi_Addr),
      PllControlReg(space, regs.PllControl_Addr), PllStatusReg(space, regs.PllControl_Addr)
{
}
//---------------------------------------------------------------------------
//  BasicAd9516_Tb_Hdw::SetSelA() -  Specify output
//---------------------------------------------------------------------------

void BasicAd9516_Tb_Hdw::SetSelA(bool value)
{
    PllControlReg.ClockSource.Value(value);
}

//---------------------------------------------------------------------------
//  BasicAd9516_Tb_Hdw::SetSelB() -  Select reference clock source
//---------------------------------------------------------------------------

void BasicAd9516_Tb_Hdw::SetSelB(bool value)
{
    PllControlReg.ReferenceSource.Value(value);
}

//-----------------------------------------------------------------------------
//  BasicAd9516_Tb_Hdw::IsLocked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  BasicAd9516_Tb_Hdw::IsLocked()
{
    return PllStatusReg.Locked.Value() ? true : false;
}

//-----------------------------------------------------------------------------
//  BasicAd9516_Tb_Hdw::ResetPll()  --  Reset PLL
//-----------------------------------------------------------------------------

void  BasicAd9516_Tb_Hdw::ResetPll()
{
    PllControlReg.Reset.Set();
    Sleep(1);
    PllControlReg.Reset.Clear();
}

//-----------------------------------------------------------------------------
//  BasicAd9516_Tb_Hdw::SpiPort()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void BasicAd9516_Tb_Hdw::SpiPort(unsigned short reg, unsigned char value)
{
    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !SpiStatusReg.Ready.Value(); ++i)
        Sleep(1);
    if (!SpiStatusReg.Ready.Value())
        Debug.Log("PLL SPI ready timeout error!");

    SpiControlReg.Address.Value(reg);
    SpiControlReg.Data.Value(value);
    SpiControlReg.Rw.Value(0);
    SpiControlReg.Apply();

//    std::stringstream msg;
//    msg << "Ad9516::Port[0x" << std::hex << static_cast<int>(reg)
//        << "]=0x" << static_cast<int>(value);
//    Debug.Log(msg.str());
}

//-----------------------------------------------------------------------------
//  BasicAd9516_Tb_Hdw::SpiPort()  --  Read from PLL data port
//-----------------------------------------------------------------------------

unsigned char BasicAd9516_Tb_Hdw::SpiPort(unsigned short reg)
{
    unsigned char result;

    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !SpiStatusReg.Ready.Value(); ++i)
        Sleep(1);
    if (!SpiStatusReg.Ready.Value())
        Debug.Log("PLL SPI ready timeout error!");

    SpiControlReg.Address.Value(reg);
    SpiControlReg.Rw.Value(1);
    SpiControlReg.Apply();

    // Wait for previous operation to complete
    while (!SpiStatusReg.DataValid.Value())
        Sleep(1);

    result = SpiStatusReg.Data.Value();

/*
    std::stringstream msg;
    msg << std::hex << "0x" << result << "=Ad9516::Port[0x"
    << static_cast<int>(reg) << "]";
    Debug.Log(msg.str());
*/
    return result;
}


//=============================================================================
//  CLASS X6Ad9516_Tb_Hdw  --  Hardware Interface for Ad951x_Base_Tb
//=============================================================================
//  "X6" version

//-----------------------------------------------------------------------------
//  constructor for class X6Ad9516_Tb_Hdw
//-----------------------------------------------------------------------------

X6Ad9516_Tb_Hdw::X6Ad9516_Tb_Hdw(IAddressingSpace & space, MapRegisters & regs)
    : SpiControlReg(space, regs.PllSpi_Addr), SpiStatusReg(space, regs.PllSpi_Addr),
      PllControlReg(space, regs.PllControl_Addr), PllStatusReg(space, regs.PllControl_Addr)
{
}

//---------------------------------------------------------------------------
//  X6Ad9516_Tb_Hdw::SetSelA() -  Specify output
//---------------------------------------------------------------------------

void X6Ad9516_Tb_Hdw::SetSelA(bool value)
{
    PllControlReg.ClockSource.Value(value);
}

//---------------------------------------------------------------------------
//  X6Ad9516_Tb_Hdw::SetSelB() -  Select reference clock source
//---------------------------------------------------------------------------

void X6Ad9516_Tb_Hdw::SetSelB(bool value)
{
    PllControlReg.ReferenceSource.Value(value);
}

//-----------------------------------------------------------------------------
//  X6Ad9516_Tb_Hdw::IsLocked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  X6Ad9516_Tb_Hdw::IsLocked()
{
    return PllStatusReg.Locked.Value() ? true : false;
}

//-----------------------------------------------------------------------------
//  X6Ad9516_Tb_Hdw::ResetPll()  --  Reset PLL
//-----------------------------------------------------------------------------

void  X6Ad9516_Tb_Hdw::ResetPll()
{
    PllControlReg.Reset.Clear();   // negative logic
    Sleep(1);
    PllControlReg.Reset.Set();
}

//-----------------------------------------------------------------------------
//  X6Ad9516_Tb_Hdw::SpiPort()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void X6Ad9516_Tb_Hdw::SpiPort(unsigned short reg, unsigned char value)
{
    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !PllStatusReg.SpiReady.Value(); ++i)
        Sleep(1);
    if (!PllStatusReg.SpiReady.Value())
        Debug.Log("PLL SPI ready timeout error!");

    SpiControlReg.Address.Value(reg);
    SpiControlReg.Data.Value(value);
    SpiControlReg.Rw.Value(0);
    SpiControlReg.Apply();

    std::stringstream msg;
    msg << "Ad951x::Port[0x" << std::hex << static_cast<int>(reg)
        << "]=0x" << static_cast<int>(value);
    Debug.Log(msg.str());

}
//-----------------------------------------------------------------------------
//  X6Ad9516_Tb_Hdw::SpiPort()  --  Read from PLL data port
//-----------------------------------------------------------------------------

unsigned char X6Ad9516_Tb_Hdw::SpiPort(unsigned short reg)
{
    unsigned char result;

    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !PllStatusReg.SpiReady.Value(); ++i)
        Sleep(1);
    if (!PllStatusReg.SpiReady.Value())
        Debug.Log("PLL SPI ready timeout error!");

    SpiControlReg.Address.Value(reg);
    SpiControlReg.Rw.Value(1);
    SpiControlReg.Apply();

    // Wait for previous operation to complete
    while (!PllStatusReg.SpiDataValid.Value())
        Sleep(1);

    result = SpiStatusReg.Data.Value();

/*
    std::stringstream msg;
    msg << std::hex << "0x" << result << "=Ad951x::Port[0x"
    << static_cast<int>(reg) << "]";
    Debug.Log(msg.str());
*/
    return result;
}

} // namespace Innovative
