//
// Ad9516_Tb_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Ad9516_Tb_MbH
#define Ad9516_Tb_MbH

#include <TimebaseDevice_Mb.h>
#include <VcxoControl_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Events_Mb.h>
#include <Trace_Mb.h>
#include <sstream>
#include <map>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS IAd951x_Tb_Hdw  --  Hardware Interface for Ad9516_Tb
//=============================================================================
//  This class keeps the Ad9516_Tb isolated from register changes or other issues

class IAd951x_Tb_Hdw
{
public:
    virtual ~IAd951x_Tb_Hdw() {}

    virtual void  SetSelA(bool setting) = 0;
    virtual void  SetSelB(bool setting) = 0;
    virtual void  SetSelC(bool /*setting*/) {}

    virtual bool  IsLocked() = 0;
    virtual void  ResetPll() = 0;

    virtual void           SpiPort(unsigned short reg, unsigned char value) = 0;
    virtual unsigned char  SpiPort(unsigned short reg) = 0;

    virtual void  ResetFpgaPll()    {}
    virtual bool  IsFpgaPllLocked()  {  return false;  }

};

typedef IAd951x_Tb_Hdw IAd9516_Tb_Hdw;

//===========================================================================
// CLASS Ad951xPllTuningEvent  --  Override PLL Tuning Parameters
//===========================================================================

class Ad951xPllTuningEvent : public OpenWire::Event
{
public:
    class Ad951x_Base_Tb * Sender;
    unsigned int &R, &P, &B, &A;

    Ad951xPllTuningEvent(Ad951x_Base_Tb * sender, unsigned int& r, unsigned int& p,
                            unsigned int& b, unsigned int& a)
        : Sender(sender), R(r), P(p), B(b), A(a)
        {
        }
};

typedef Ad951xPllTuningEvent Ad16PllTuningEvent;

//=============================================================================
//  CLASS Ad951x_Base_Tb  --  Ad9516 PLL (TimebaseDevice derived version)
//=============================================================================

class Ad951x_Base_Tb : public PllTimebaseDevice
{
public:
    enum AdReferenceSource { refExternal, refInternal };
    enum AdOutputSource { outExtClk, outPll };
    enum AdExternalSelect { csFrontPanel, csP16 };

    //
    //  Member Functions
    //  Bank 0 = N  LVPECL outputs..  Bank 1 = N LVDS/CMOS outputs
    Ad951x_Base_Tb(IAd951x_Tb_Hdw * hdw_intf, unsigned int bank0_outputs,
                                              unsigned int bank1_outputs=0);
    ~Ad951x_Base_Tb();

    void   Init();
    void   Reapply();

    //  Select the Input reference
    void  ReferenceSource(AdReferenceSource value)
             { FReferenceSource = value;  SetReferenceSource(value); }
    AdReferenceSource  ReferenceSource()
             {  return FReferenceSource;  }
    //  Select output signal source
    void  OutputSource(AdOutputSource value)
             {
             FOutputSource = value;
             UpdateMode = (value==outPll) ? updFrequency : updDivider;
             SetOutputSource(value);
             }
    AdOutputSource  OutputSource() const
             { return FOutputSource; }
    //  Use External VCO or internal
    bool InternalVco()  {  return FInternalVco; }
    void InternalVco(bool state);
    //  Set the External Signal Source
    void  ExternalClockSelect(AdExternalSelect value);
    AdExternalSelect  ExternalClockSelect() const;
    size_t               DeviceId();


    bool   Locked() const;

    void                AllowAsymmetricDutyCycle(bool state);
    bool                AllowAsymmetricDutyCycle() const;

    void                PFDPolarity(bool state);
    bool                PFDPolarity() const;

    virtual void        SetPhaseDetectorFrequency(double /*pfd_freq*/)
                              { }   // may be implemented in board specific class
                              
    // Methods
    virtual void        Reset();

    // Output frequency range information
    void                 Max(double value);
    double               Max() const;

    void                 Min(double value);
    double               Min() const;
    int                  MaxOutputDivider() const;

    void                 VcoRange(double low, double high)
                            {  Vco->SetRange(low, high);  }
    void                 LoadVco(IVcxoControl * aVco);

    //  Raw configuration accesses
    void             Port(unsigned short reg, unsigned char value);
    unsigned char    Port(unsigned short reg);

    //  Event Hooks
    OpenWire::EventHandler<Ad951xPllTuningEvent>  OnPllTuning;

    IAd951x_Tb_Hdw *  HdwIntf()  {  return Hdw;  }

protected:
    //  Chip Personality Interface
    virtual void    Init_DeviceIDCodes() = 0;
    virtual void    Init_Bank0_OffsetMap() = 0;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    //virtual void    OnSetPower(bool & new_state);
    virtual double  PhaseDetectorFrequency() const = 0;
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    enum AdCpCurrent { cpc600,  cpc1200, cpc1800, cpc2400, cpc3000,
                       cpc3600, cpc4200, cpc4800 };
    enum AdCpMode   { cpmHichZ, cpmPumpUp, cpmPumpDown, cpmNormal };
    enum AdVoltLevel  { v400mv, v600mv, v780mv, v960mv };
    enum AdLvdsCurrent  { c1750uA, c3500uA, c5250uA, c7000uA };
    enum OutputCfgSettings { ocLvds=0x0, ocLvCmos=0x1 };
    //
    //  Internal "Setters"
    void  SetReferenceSource(AdReferenceSource source);
    void  SetOutputSource(AdOutputSource source);
    void  SetExternalClockSelect(AdExternalSelect src);
    void  SetDividerPassThrough(unsigned int output, bool passthru);
    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    virtual void  SetOutVoltageHdw(unsigned int output, unsigned int volt_setting);
    virtual void  SetOutCurrentHdw(unsigned int output, unsigned int current_setting);
    virtual void  SetVcoDividerHdw(unsigned int divider_setting);
    void  ApplyChargePumpCfg();   // from Settings.Pll
    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting);
    void  SetOutputDutyCycle(unsigned int output, float duty_cycle);
    //
    //  Bank 0, 1 Impl for primary Internal Setters
    void  SetOutEnabledHdw_Bank0(unsigned int output, bool enabled);
    void  SetOutEnabledHdw_Bank1(unsigned int output, bool enabled);
    void  SetDividerHdw_Bank0(unsigned int output, unsigned int divider_setting);
    void  SetDividerHdw_Bank1(unsigned int output, unsigned int divider_setting);

    unsigned int  GetOutdivBits(unsigned int divider_setting, float duty_cycle );
    //
    //  Pll Programming Registers
    void  SetR(unsigned int R);
    void  SetP(unsigned int P, unsigned int B);
    void  SetB(unsigned int B);
    void  SetA(unsigned int A);
    //
    //  Internal Operations
    void  InitSerialPort();
    void  UpdateRegisters();
    void  CalibrateVco();

    //
    //  Member Data
    IAd951x_Tb_Hdw *  Hdw;
    IVcxoControl *    Vco;

    const unsigned int  Total_Bank0_LvPecl_Outputs;
    const unsigned int  Total_Bank1_Lvds_Outputs;
    const unsigned int  Total_Outputs;

    // Fields
    AdReferenceSource   FReferenceSource;
    AdOutputSource      FOutputSource;
    AdExternalSelect    FExternalClockSrc;
    bool                FInternalVco;

    struct PllSettings
    {
        bool         PFDPolarity;
        AdCpCurrent  ChargePumpCurrent;
        AdCpMode     ChargePumpMode;
        bool         PllPowerDown;
    };

    struct LvPeclSettings
    {
        unsigned int OutputCfgReg;       // 0xF0 bank register image
        unsigned int DividerCfgReg[3];   // 0x190 bank register image
        float        DutyCycle;

        LvPeclSettings()
            {
            OutputCfgReg = 0;  DividerCfgReg[0] = 0; DividerCfgReg[1] = 0;
            DividerCfgReg[2] = 0; DutyCycle = 0.0;
            }
    };

    struct LvdsSettings
    {
        unsigned int OutputCfgReg;       // 0x140 bank register image
        unsigned int DividerCfgReg[5];   // 0x199 bank register image
        float        DutyCycle;

        LvdsSettings()
            {
            OutputCfgReg = 0;  DividerCfgReg[0] = 0; DividerCfgReg[1] = 0;
            DividerCfgReg[2] = 0; DividerCfgReg[3] = 0; DividerCfgReg[4] = 0;
            DutyCycle = 0.0;
            }
    };

    struct MasterSettings
    {
        bool         PassThrough;
        unsigned int Divider;
    };

    struct HdwSettings
    {
        MasterSettings  Master;
        PllSettings     Pll;
        std::vector<LvPeclSettings>  Out_Bank0;
        std::vector<LvdsSettings>    Out_Bank1;
        double          VcoFrequency;
        unsigned int    VcoDivider;
    };

    HdwSettings     Settings;
    //
    //  PLL Device Implementations
    void            SettingsInit();
    void            FrequencyModeUpdate();
    void            DividerModeUpdate();
    // Support
    void             CalculateDividers();
    void             CalculateExtClk_Dividers();
    void             FindVcoDivider();
    void             CalculateNR(unsigned int & N, unsigned int & R);
    void             AdvancedPBACalculation(unsigned int N, unsigned int & P,
                            unsigned int & B, unsigned int & A, double VcoRate);
    void             SetOutputDividers();

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by);  }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above)
        {  return Ad951xGetNearestDividers(candidate, below, above);  }

    //
    //  Ad9516 Divider Info Functions
public:
    //unsigned int  ToDivider(unsigned int divider_idx);
    //unsigned int  ToDividerIdx(unsigned int divider);
    bool          IsValidDivider(unsigned int divide_by);
    bool          IsValidVcoDivider(unsigned int divide_by);

    unsigned int  BestDividerGreaterThan(float min_divide_by);
    unsigned int  BestDividerLessThan(float max_divide_by);

    //unsigned int  ToDividerInput(unsigned int divide_by);
    //unsigned int  FromDividerInput(unsigned int divider_bits);


protected:
    typedef std::vector<unsigned int> ValidDividerArray;
    typedef std::map<unsigned int, unsigned int> ValidDividerMap;
    //typedef std::map<unsigned int, unsigned int> InverseDividerMap;
    //typedef std::map<unsigned int, std::vector<unsigned int> > PhaseAdjustArrayMap;

    //InverseDividerMap    CdceInverseDividers;
    //PhaseAdjustArrayMap  CdcePhaseArray;

protected:
    std::vector<unsigned int>  DeviceIDCodes;
    std::vector<unsigned int>  Bank0_OffsetMap;

    ValidDividerArray    ValidDividerValues;
    ValidDividerMap      Ad951x_ValidDividers;

    ValidDividerArray    ValidVcoDividerValues;
    ValidDividerArray    ValidVcoDividerValues_InternalVco;
    ValidDividerArray    ValidVcoDividerValues_ExternalVco;
    ValidDividerMap      Ad951x_ValidVcoDividers;

    void  ConfigureDividerInformation();
    void  Ad951xGetNearestDividers(float candidate, int & below, int & above);

    unsigned int ToBank0_Offset(unsigned int ch);

public:
    Trace            Debug;
};

//=============================================================================
//  CLASS Ad9516_Tb  --  Ad9516 PLL (TimebaseDevice derived version)
//=============================================================================

class Ad9516_Tb : public Ad951x_Base_Tb
{
public:
    //
    //  Member Functions
    Ad9516_Tb(IAd951x_Tb_Hdw * hdw_intf);
    virtual ~Ad9516_Tb();

protected:

    virtual void    Init_DeviceIDCodes();
    virtual void    Init_Bank0_OffsetMap();
};

//=============================================================================
//  CLASS Ad9517_Tb  --  Ad9517 PLL (TimebaseDevice derived version)
//=============================================================================

class Ad9517_Tb : public Ad951x_Base_Tb
{
public:
    //
    //  Member Functions
    Ad9517_Tb(IAd951x_Tb_Hdw * hdw_intf);
    virtual ~Ad9517_Tb();

protected:

    virtual void    Init_DeviceIDCodes();
    virtual void    Init_Bank0_OffsetMap();
};

//=============================================================================
//  CLASS BasicAd9516_Tb_Hdw  --  Hardware Interface for Ad9516_Tb
//=============================================================================
//  "normal" version (X5)

class BasicAd9516_Tb_Hdw : public IAd951x_Tb_Hdw
{
public:
    //
    //  Interface Implementation
    virtual void  SetSelA(bool setting);
    virtual void  SetSelB(bool setting);
    virtual bool  IsLocked();
    virtual void  ResetPll();
    virtual void           SpiPort(unsigned short reg, unsigned char value);
    virtual unsigned char  SpiPort(unsigned short reg);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BasicAd9516_Tb_Hdw::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   PllControl_Addr;
        unsigned int   PllSpi_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllControlRegister  -- PLL misc control interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit     Reset;
        RegisterBit     ClockSource;
        RegisterBit     ReferenceSource;

        PllControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9516::PllControlRegister", space, offset),
              Reset(*this, 0), ClockSource(*this, 1),
              ReferenceSource(*this, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllStatusRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Locked;

        PllStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9516::PllStatusRegister", space, offset),
              Locked(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiControlRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup      Data;
        RegisterBitGroup      Address;
        RegisterBit           Rw;

        SpiControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9516::SpiControlRegister", space, offset),
              Data(*this, 0, 8), Address(*this, 8, 10), Rw(*this, 18)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiStatusRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Ready;
        RegisterBit           DataValid;
        RegisterBitGroup      Data;

        SpiStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad9516::SpiStatusRegister", space, offset),
              Ready(*this, 22), DataValid(*this, 23), Data(*this, 24, 8)
            {
            }
    };
    //
    //  Member Functions
    BasicAd9516_Tb_Hdw( IAddressingSpace & space, MapRegisters & regs );

    SpiControlRegister  SpiControlReg;
    SpiStatusRegister   SpiStatusReg;
    PllControlRegister  PllControlReg;
    PllStatusRegister   PllStatusReg;
public:
    Trace            Debug;

};

//=============================================================================
//  CLASS X6Ad9516_Tb_Hdw  --  Hardware Interface for Ad9516_Tb
//=============================================================================
//  "X6" version

class X6Ad9516_Tb_Hdw : public IAd951x_Tb_Hdw
{
public:
    //
    //  Interface Implementation
    virtual void  SetSelA(bool setting);
    virtual void  SetSelB(bool setting);
    virtual bool  IsLocked();
    virtual void  ResetPll();
    virtual void           SpiPort(unsigned short reg, unsigned char value);
    virtual unsigned char  SpiPort(unsigned short reg);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Ad9516_Tb_Hdw::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   PllControl_Addr;
        unsigned int   PllSpi_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllControlRegister  -- PLL misc control interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit     Reset;
        RegisterBit     ReferenceSource;
        RegisterBit     ClockSource;

        PllControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad951x::PllControlRegister", space, offset),
              Reset(*this, 1), ReferenceSource(*this, 5),
              ClockSource(*this, 6)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PllStatusRegister  -- PLL status/SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PllStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Locked;
        RegisterBit           SpiReady;
        RegisterBit           SpiDataValid;

        PllStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad951x::PllStatusRegister", space, offset),
              Locked(*this, 4), SpiReady(*this, 30), SpiDataValid(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiControlRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup      Address;
        RegisterBit           Rw;
        RegisterBitGroup      Data;

        SpiControlRegister(IAddressingSpace &space, int offset)
            : inherited("Ad951x::SpiControlRegister", space, offset),
              Address(*this, 0, 10), Rw(*this, 12), Data(*this, 16, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiStatusRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Data;

        SpiStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad951x::SpiStatusRegister", space, offset),
              Data(*this, 24, 8)
            {
            }
    };
    //
    //  Member Functions
    X6Ad9516_Tb_Hdw( IAddressingSpace & space, MapRegisters & regs );

    SpiControlRegister  SpiControlReg;
    SpiStatusRegister   SpiStatusReg;
    PllControlRegister  PllControlReg;
    PllStatusRegister   PllStatusReg;
    
public:
    Trace            Debug;

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

