// AdDac9122_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef AdDac9122_MbH
#define AdDac9122_MbH

#include <Trace_Mb.h>
//#include <X6Support_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

namespace AdDac9122
{
//===========================================================================
//  CLASS IDacToHardware  -- Interface to link device to rest of system
//===========================================================================

class IDacToHardware
{
public:

    virtual unsigned short  Spi(int addr) = 0;
    virtual void            Spi(int addr, unsigned short value) = 0;

    virtual void            DacReset(bool state) = 0;

    virtual ~IDacToHardware()
        {}
};

//===========================================================================
//  CLASS DacDevice  -- Per-DAC controls for AdDac9122
//===========================================================================

class DacDevice
{
public:

    //
    //  Public Data
    struct DacSedStatus
        {
        unsigned short R7;
        unsigned short R67;
        unsigned short R70;
        unsigned short R71;
        unsigned short R72;
        unsigned short R73;
        };

    DacSedStatus  SedStatus;

    //
    //  Constructors
    DacDevice(IDacToHardware * hdw);
    virtual ~DacDevice();

    enum Constants { DeviceChannels = 2 };

    // Methods
    void                Reset();
    void                ChannelEnabled(int idx, bool value)
                            {  FChannelEnabled[idx] = value; }
    bool                ChannelEnabled(int idx)
                            {  return FChannelEnabled[idx];  }

    void                Compare_I(int idx, short value)
                            {  FCompare_I[idx] = value; }
    short               Compare_I(int idx)
                            {  return FCompare_I[idx];  }
    void                Compare_Q(int idx, short value)
                            {  FCompare_Q[idx] = value; }
    short               Compare_Q(int idx)
                            {  return FCompare_Q[idx];  }

    void                PreconfigHardware();
    void                ConfigureHardware();

    void                SED_Mode(bool state)
                             {  FEnterSEDMode = state;  }
    void                GetSED_Status();

    void                Write_I_Offset(unsigned int value);
    void                Write_Q_Offset(unsigned int value);
    void                Write_I_Gain(unsigned int value);
    void                Write_Q_Gain(unsigned int value);

protected:
    IDacToHardware *    Hdw;
    bool                FEnterSEDMode;
    bool                FChannelEnabled[DeviceChannels];
    unsigned short      FCompare_I[2];
    unsigned short      FCompare_Q[2];
    unsigned int        Offset_I;
    unsigned int        Gain_I;
    unsigned int        Offset_Q;
    unsigned int        Gain_Q;

    Trace               Debug;

    void  Setup_Phase1();
    void  Setup_Phase2(unsigned int fifo_offset);
    bool  Setup_Phase5(unsigned int & thermo_rb);
    void  Setup_Phase9();
    void  Setup_Phase10();
    void  Setup_Phase11();

    void          SleepDacs(bool state);
    bool          IsThermometerValueLegal(unsigned int thermo_value);
    unsigned int  ThermometerValueDecode(unsigned int thermo_value);
    void          Load_Compare_I(unsigned int index);
    void          Load_Compare_Q(unsigned int index);

    //  SPI primitives
    void          SpiBit(int addr, int bit, bool state);

    void          SpiField(int addr, int start_bit, int size, unsigned int data);
    unsigned int  SpiField(int addr, int start_bit, int size);

    void          SpiValue(int addr, int data);
    unsigned int  SpiValue(int addr);


};

//===========================================================================
//  CLASS IDacCalibratorToHdw  -- Interface of Calibrator back to Hardware
//===========================================================================
/*
class IDacCalibratorToHdw
{
public:
    enum EngDeviceState  { engDevice0, engDevice1, engAll };

    virtual void          ApplyDacGainCoefficients() = 0;
    virtual void          DacReset() = 0;
    virtual void          PreconfigureHardware() = 0;
    virtual double        ClockFrequencyActualMhz() = 0;

    virtual void          SetTestMode_Alternating() = 0;
    virtual void          SetTestMode_DoubleAlternating() = 0;
    virtual void          SetTestEnable(bool state) = 0;
    virtual void          SetSoftwareUnframedTrigger() = 0;
    virtual void          DeviceRun(bool state, EngDeviceState which) = 0;

    virtual DacDevice &   Dac(unsigned int idx) = 0;

    virtual ~IDacCalibratorToHdw()
        {}
};
*/
//===========================================================================
//  CLASS DacCalibrator  -- DAC calibration procedure for TiDac6682
//===========================================================================
/*
class DacCalibrator : public IDacCalibrator
{
public:
    DacCalibrator(IDacCalibrationRegister * reg, IDacCalibratorToHdw * hdw);
    virtual ~DacCalibrator();

    virtual void  Calibrate();

protected:
    IDacCalibrationRegister *  Reg;
    IDacCalibratorToHdw *      Hdw;

    Trace         Debug;

    void          DoInternalCalibration();
    unsigned int  GetInternalCalibrationResults();

    //  Single/Dual Device methods
    virtual unsigned int GetShouldBeValue();
    virtual bool         ShouldCalibrate();
            bool         ShiftCountCheck();

    virtual void  SetDacTestMode();
    virtual void  SetDacRunControl(bool state);
    virtual void  ReadDacDelay(unsigned int & delay_0, unsigned int & delay_1);
    virtual void  LoadDacDelay(unsigned int delay_0, unsigned int delay_1);
};

//===========================================================================
//  CLASS DualDacCalibrator  -- DAC calibration procedure for dual TiDac6682
//===========================================================================

class DualDacCalibrator : public DacCalibrator
{
public:
    DualDacCalibrator(IDacCalibrationRegister * reg, IDacCalibratorToHdw * hdw);
    virtual ~DualDacCalibrator();

protected:
    //  Single/Dual Device methods
    virtual unsigned int GetShouldBeValue();
    virtual bool         ShouldCalibrate();

    virtual void  SetDacTestMode();
    virtual void  SetDacRunControl(bool state);
    virtual void  ReadDacDelay(unsigned int & delay_0, unsigned int & delay_1);
    virtual void  LoadDacDelay(unsigned int delay_0, unsigned int delay_1);
};
*/
} // end namespace AdDac9122


//---------------------------------------------------------------------------
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif


}; // end namespace Innovative
//---------------------------------------------------------------------------
#endif

