// AdcStats_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef AdcStats_MbH
#define AdcStats_MbH

#include <IppFloatDG_Mb.h>
#include <Fourier_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <WindowType_Mb.h>
#include <ProcessEvents_Mb.h>
#include <limits>
#include <math.h>
#include <Stats_Mb.h>
#include <Sample_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//---------------------------------------------------------------------------
class AdcStats
{
public:
    enum IISpectrumMode { smAmplitude, smPower };

	//  Ctors
    AdcStats();
    virtual ~AdcStats();

    //  Member Functions
    void        Reset();
    void        Analyze();
	void        Analyze(float const * buffer, size_t size);
	void        Analyze(double const * buffer, size_t size);
	void        Analyze(const Innovative::IppFloatDG & data);
	void        Analyze(const Innovative::FloatDG & data);
	void        Analyze(const Innovative::DoubleDG & data);
	void        Analyze(const Innovative::ComplexDG & data);
	void        Datasample(double value);
    void        Datasample(Sample value);
	void        AnalyzePowerSpectrum(size_t stride);

    unsigned int Harmonic(int index) const
        { return FHarmonic[index];  }
    double      Enob() const
        { return FEnob;  }
    double      HarmonicDistortion() const
        { return FHd;  }
    double      THD() const
        {  return FThd;  }
    double      Thd() const
        {  return FThd;  }
    double      dBThd() const
        {  return FdBThd;  }
    double      Sinad() const
        { return FSinad;  }
    double      SFDR() const
		{  return FSfdr;  }
	double      Psd() const
		{  return FPsd;  }
	double      NoiseFloor() const
		{  return FNoiseFloor;  }
	double      Sfdr() const
		{  return FSfdr;  }
    double      Snr() const
        {  return FSnr;  }
    double      MaxSnr() const
        { return GetMaxSnr();  }
    double      PowerSpec(unsigned int index) const
        { return FPowerSpec[index];  }
    unsigned int PeakIndex(unsigned int idx) const
        {  return FPeakIndex[idx];  }
    unsigned int AmplitudePeakIndex() const
        {  return FAmplitudePeakIndex;  }
	unsigned int SpurPeakIndex() const
		{  return FSpurPeakIndex;  }
	double      SpurPeakAmplitude() const
		{  return FSpurPeakAmplitude;  }
	double      PhaseAngle(unsigned int index)
		{  return FPhase[index];  }
	unsigned int Points() const
		{  return FPoints;  }
	unsigned int    Margin() const
		{  return std::max<unsigned int>(FReach/1000, 1);  }

	void        ClearExclusions()
		{  FExcluded.clear();  }
	void        Exclude(unsigned int low, unsigned int high)
		{  FExcluded.push_back(Range(low, high));  }

	// Properties
	double RmsNoise() const
		{ return FRmsNoise;  }
	double RmsPeak() const
		{ return FRmsSignal;  }

	unsigned int Extent(unsigned int index) const
		{  return FExtent[index];  }
	void Extent(unsigned int index, unsigned int value)
		{  FExtent[index]=value;  }

	double dB(unsigned int index) const
		{  return FdB[index];  }
	void   dB(unsigned int index, double value)
        {  FdB[index] = value;  }

	double 	Amplitude(unsigned int index) const
		{  return FAmplitude[index];  }
	void 	Amplitude(unsigned int index, double value)
		{  FAmplitude[index] = value;  }

	void Size(size_t value)
		{  SetSize(value);  }
	size_t Size() const
		{  return FSize;  }
    void 	Reach(unsigned int samples)
        {  SetReach(samples);  }
	unsigned int Reach() const
		{  return FReach;  }

	int    Hunt() const
		{  return FHunt;  }
	void   Hunt(int value)
		{  FHunt=value;  }

	int    HarmonicWidth() const
		{ return FHarmonicWidth;  }
	void   HarmonicWidth(int value)
		{  FHarmonicWidth=value;  }

	bool   KeepDc() const
		{  return FKeepDc;  }
	void   KeepDc(bool value)
		{  FKeepDc=value;  }

	bool   KeepFs2() const
		{  return FKeepFs2;  }
	void   KeepFs2(bool value)
		{  FKeepFs2=value;  }

	bool   Rescaled() const
		{  return FRescaled;  }
    void   Rescaled(bool value)
        {  FRescaled=value;  }

    void   SpectrumMode(IISpectrumMode value)
        {  FSpectrumMode = value;  }
    IISpectrumMode SpectrumMode() const
        {  return FSpectrumMode;  }

    double Jitter(int bits, double freq_in) const;

    // Data
    Innovative::Fourier         FftData;
    Innovative::Statistics      PSStats;
    Innovative::Statistics      DataStats;

    void    Order(int value)
        {  SetOrder(value);  }
    int     Order() const
        {  return FOrder;  }

    void    Bits(int value)
        {  SetBits(value);  }
    int     Bits() const
        {  return FBits;  }

    void        Window(Innovative::WindowTypes value)
        {  SetWindow(value);  }
    WindowTypes  Window() const
        {  return FWindow;  }

    void    Harmonics(unsigned int value)
        {  SetHarmonics(value);  }
    unsigned int  Harmonics() const
        {  return FHarmonics;  }

    void    ScaleFactor(double value)
        {  FScaleFactor=value;  }
    double  ScaleFactor() const
        {  return FScaleFactor;  }

    void    Offset(double value)
        {  FOffset=value;  }
    double  Offset() const
        {  return FOffset;  }

    void    Intermodulation(bool state)
        {  FIntermodulation = state;  }
    bool    Intermodulation() const
        {  return FIntermodulation;  }

    OpenWire::EventHandler<ProcessCompletionEvent>   OnAnalyzed;

private:
    typedef std::vector<double> DoubleArray;
    typedef std::vector<unsigned int> IndexArray;
    class Range
    {
    public:
        Range(unsigned int low, unsigned int high)
            : Low(low), High(high)
            {  }
        unsigned int Low;
        unsigned int High;
    };
	typedef std::vector<Range>  RangeArray;

    // Data
    int                         FBits;
    double                      FRmsNoise;      // RMS noise of power spectrum
	double                      FRmsNoisePlusDistortion;      // RMS noise of power spectrum + THD
    double                      FRmsSignal;       // Peak value of power spectrum
    std::vector<int>            FHarmonic;      // Indicies to first six harmonics
    double                      FEnob;          // Effective number of bits
    double                      FHd;            // Harmonic distortion
    double                      FThd;           // Total harmonic distortion in %
    double                      FdBThd;         // Total harmonic distortion in dB
    double                      FSinad;         // SINAD for A/D
    double                      FSfdr;          // Spectral free dynamic range
    double                      FSnr;           // Actual Signal to Noise
	double                      FPsd;           // Power spectum density
	double                      FNoiseFloor;
    std::vector<unsigned int>	FExtent;        // Indicies around mode of PS
    DoubleArray                 FPowerSpec;     // Power spectrum of input data
    DoubleArray                 FdB;            // Normalized amplitude spectrum
    DoubleArray                 FAmplitude;     // Normalized amplitude spectrum
    DoubleArray                 FPhase;         // Normalized phase response
    unsigned int                FReach;
	unsigned int                FMargin;
    size_t                		FSize;
	size_t	                	FPoints;
    int                         FHunt;
    int                         FHarmonicWidth;
    bool                        FKeepDc;
    bool                        FKeepFs2;
    bool                        FRescaled;
    size_t                      FOrder;
    Innovative::WindowTypes     FWindow;
    unsigned int                FHarmonics;
	double                      FScaleFactor;
	double                      FOffset;
    bool                        FIntermodulation;
    unsigned int                FPeakIndex[2];
    unsigned int                FSpurPeakIndex;
	double                      FSpurPeakAmplitude;
	unsigned int                FAmplitudePeakIndex;
    IISpectrumMode              FSpectrumMode;
	RangeArray                  FExcluded;
	int                         FStride;

	double                      ZerodB;
    double                      MindB;
	Statistics                  Noise;
	Statistics                  Distortion;
	Statistics                  Signal;

	//  Setter getters
    void     SetOrder(int order);
    void     SetBits(int bits);
    void     SetHarmonics(unsigned int value);
    void     SetWindow(Innovative::WindowTypes window);
	unsigned int   GetExtent(unsigned int index);
	void     SetExtent(unsigned int index, unsigned int value);
	unsigned int   GetHarmonic(unsigned int index);
	double   GetPowerSpec(unsigned int index);
	double   GetMaxSnr() const;
	void     SetdB(unsigned int index, double value);
	double   GetdB(unsigned int index);
	void     SetSize(size_t samples);
	void     SetReach(size_t samples);

	// Spectra calculations
    void            CalculateExtents(unsigned int peak, unsigned int *extent);
    unsigned int    CalculatePeakIndex2(unsigned int peak);
    void            ReferenceSpectrum();
    void            PowerSpectrum();
    void            analyze();
    void            analyzeTHD();
    void            analyzeIM();
    void            Initialize();
    bool            Within(int value, int lower, int upper);    
    double          NoisePowerBandwidth() const;
    double          WindowGain() const;
    void            Calculate_dBV();
    void            Calculate_dBm();
	void            Pivot(DoubleArray & array, int axis);
    void            Merge(DoubleArray & array);
    bool            IsHarmonic(int idx);
    bool            IsExcluded(int idx);
    void            RetainLargestSpur(unsigned int idx, double value);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative

#endif

