//  AlertSystem_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//
#ifndef AlertSystem_MbH
#define AlertSystem_MbH

#include <Events_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS AlertSignalEvent  --  Event fired when Alert is detected
//===========================================================================

class AlertSignalEvent : public OpenWire::Event
{
public:
    size_t  SenderTag;
    size_t  TimeStamp;
    int  Argument;

public:
    AlertSignalEvent( size_t sender_tag, size_t timeStamp, int argument )
        :  SenderTag( sender_tag ), TimeStamp( timeStamp ), Argument( argument )
        {
        }
};

//===========================================================================
// CLASS  AlertData  --  Alert Data Cookie
//===========================================================================

typedef  OpenWire::EventHandler<AlertSignalEvent> *  AlertSignalEventHandlerPtr;

class AlertData
{
public:
    int           Key;
    int           EnableBit;
    unsigned int  StatusMask;
    bool          PassThru;
    bool          Enabled;
    AlertSignalEventHandlerPtr   AlertEvent;
    int           Field;
    //
    //  Member Functions
    AlertData(int key, int bit, unsigned int statusMask,
              bool passThru,  AlertSignalEventHandlerPtr alertEvent,
              int field=0 )
        :  Key(key), EnableBit(bit), StatusMask(statusMask),
           PassThru(passThru), Enabled(false), AlertEvent(alertEvent),
           Field(field)
        { }
};

//===========================================================================
// CLASS  AlertSystem  --  Alert Processing Class
//===========================================================================

class AlertSystem
{
public:
    //
    //  Member Functions
    AlertSystem(int alertPid)
            :  AlertSenderTag(0), AlertPid(alertPid)
            {}
    virtual ~AlertSystem()
            {}

    void  SenderTag(size_t tag)
            {  AlertSenderTag = tag;  };
    size_t   SenderTag() const
            {  return AlertSenderTag;  };

    void  ClearAlerts();
    void  LoadAlert(const AlertData & alert);

    bool  AlertTest(int pid)
            {  return pid==AlertPid;  }
            
    virtual bool  CheckAndProcess(const Buffer & buffer);

    virtual void  Open() = 0;
    virtual void  Configure() = 0;

protected:
    //
    //  Member Functions
    void  DoAlertEnable(int key, bool state);
    void  DoAlertPassThru(int key, bool state);
    void  DoAlertClear(int key);

    AlertData *  FindAlertByKey(int key);
    AlertData *  FindAlertByStatusMask(unsigned int mask);

    //
    //  Overridable Alert Info Cracker Functions
    virtual int   ExtractAlertTimestamp(const Buffer & alert);
    virtual int   ExtractAlertArgument(const Buffer & alert, int bit, int field);
    //
    //  Member Data
    size_t                  AlertSenderTag;
    const int               AlertPid;
    std::vector<AlertData>  Alerts;

    virtual void  ApplyAlertClearHdw(unsigned int bit);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

//---------------------------------------------------------------------------
#endif
