// AlignedBlock_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef AlignedBlock_MbH
#define AlignedBlock_MbH

#include <stdlib.h>
#include <SyncObjects_Mb.h>
#include <AccessDatagrams_Mb.h>
#include <AlignedMemoryPool_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS AlignedBlock  -- Chunk of Managed Memory
//=============================================================================

class AlignedBlock
{
public:
    friend class AlignedBlockHandle;

    AlignedBlock(size_t size_in_integers = 0 );
    AlignedBlock(const AlignedBlock & other);
    ~AlignedBlock();

    void  Resize(size_t sizeInInts );
    void  CopyFrom(const AlignedBlock & other);

    const AlignedBlock & Self() const
        {  return *this;  }
    AlignedBlock & Self() 
        {  return *this;  }

    unsigned int  UseCount() const
        {  return ReferenceCount;  }
    size_t        SizeInInts() const
        {  return Size;  }
    size_t        AllocatedInInts() const
        {  return AllocatedSize;  }

    int *  Base()
        {  return Ptr.Address();  }
    const int *  ConstBase() const
        {  return Ptr.Address();  }

    void *      operator new( size_t size );
    void        operator delete( void *ptr );

private:
    //
    //  Member Data
    unsigned int  ReferenceCount;
    size_t        AllocatedSize;
    size_t        Size;

    AlignedMemoryPtr            Ptr;
    AtomicAccess<unsigned int>  Atomic;
    //
    //  Member Functions
    void           AddReference();
    unsigned int   RemoveReference();       // returns current count

    void           Allocate(size_t size_in_ints);
    void           Deallocate();
    void           Swap(AlignedBlock & other);

    unsigned int   CalculateAllocatedSize(size_t size_in_ints);

};

//==============================================================================
//  CLASS AlignedBlockHandle  -- Reference Counter Handle to a AlignedBlock
//==============================================================================

class AlignedBlockHandle
{
public:
    enum AutoCreate { createBlock };

    AlignedBlockHandle(AlignedBlock * block = 0);
    AlignedBlockHandle(AutoCreate tag, size_t size_in_integers);
    AlignedBlockHandle(const AlignedBlockHandle & other);
    ~AlignedBlockHandle();

    AlignedBlock *  operator->()
        {  return Blk;  }
    AlignedBlock *  operator->() const
        {  return Blk;  }
    AlignedBlock &  operator*()
        {  return *Blk;  }
    AlignedBlock &  operator*() const
        {  return *Blk;  }

    bool  IsNotNull() const
            {  return Blk!=0;  };
    bool  IsNull() const
            {  return Blk==0;  };
    bool  IsUnique() const
            {  return IsNotNull() && Blk->UseCount()==1;  };

    void  MakeUnique();
    void  DeepCopy(AlignedBlockHandle & other);
    void  Swap(AlignedBlockHandle & other);

    void  Resize(size_t size_in_integers);

private:
    AlignedBlock * Blk;

};

//==============================================================================
//  CLASS AlignedBlockDatagram  -- Datagram access to Aligned Block Data
//==============================================================================
//  We construct an external interface rather than support it directly

class AlignedBlockDatagram : public AccessDatagram<unsigned int>
{
public:
    //
    //  Member Functions
    AlignedBlockDatagram(AlignedBlock & block)
        :  AccessDatagram<unsigned int>(BuildInterface(block)),
           FBlock(block)
        {}
    ~AlignedBlockDatagram()
        {  delete Intf;  }     // since we allocated it in BuildInterface

    AlignedBlock &  Block()  {  return FBlock;  };

protected:
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS AlignedBlockDatagrammable  -- Datagrammable Interface for ABs
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class AlignedBlockDatagrammable : public IDatagrammable
        {

        public:
            AlignedBlockDatagrammable(AlignedBlock & block)
                :   FBlock(block)
                {}

            virtual size_t      DatagramSize() 
                                {  return FBlock.SizeInInts();  }
            virtual int *         DatagramBasePtr()
                                {  return FBlock.Base();  }
            virtual bool          MakeWritable()
                                {  return false;  }   
            virtual void          Resize(size_t /*size_in_ints*/)
                                {   }  
            AlignedBlock &  FBlock;
        };

    IDatagrammable * BuildInterface(AlignedBlock & block)
        {  return new AlignedBlockDatagrammable(block);  }

private:
    //
    //  Member Data
    AlignedBlock &  FBlock;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif

