//  AlignedMemoryPool_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef AlignedMemoryPool_MbH
#define AlignedMemoryPool_MbH

#include <stddef.h>
#include <string>

#include <SyncObjects_Mb.h>
#include <SingletonManager_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS AlignedMemoryPtr -- Pointer to chunk of aligned memory
//==============================================================================

class AlignedMemoryPtr
{
public:
    explicit AlignedMemoryPtr(char * origin);
    explicit AlignedMemoryPtr(size_t size_in_bytes);
    AlignedMemoryPtr();
    ~AlignedMemoryPtr();

    AlignedMemoryPtr        New(size_t bytes);
    void                    Delete();
    bool                    IsNull() const;

    char *                  Origin() const;
    int *                   Address() const;

private:
    char *                  FOrigin;
    int *                   FAddress;

    int *                   AddressFromOrigin(char * origin);
};

//=============================================================================
//  CLASS AlignedMemoryManager  --  manages aligned memory pool
//=============================================================================

class AlignedMemoryList;
class AlignedMemoryPool;

class AlignedMemoryManager : public OpenWire::SingletonService
{
public:
    //
    //  Client Interface
    static AlignedMemoryPtr        Allocate(size_t size_in_bytes);
    static void                    Deallocate(AlignedMemoryPtr data, size_t size_in_bytes);

    static void                    ReleaseAll();
    static void                    Preallocate(size_t size_in_bytes, size_t count);

    static void                    Info(std::string & result);

    AlignedMemoryManager();
    ~AlignedMemoryManager();

private:
    //
    //  Singleton Implementation Methods
    AlignedMemoryPtr      Allocate_Impl(size_t size_in_bytes );
    void                  Deallocate_Impl(AlignedMemoryPtr data, size_t size_in_bytes);
    void                  ReleaseAll_Impl();
    void                  Preallocate_Impl(size_t size_in_bytes, size_t count);
    void                  Info_Impl(std::string & result);

    AlignedMemoryList *   GetAMList_forSize(size_t size_in_bytes);

    // Data
    AtomicAccess<unsigned int>	    Atomic;
    AlignedMemoryPool *             Pool;

    static AlignedMemoryManager *  Instance;
    static unsigned int            AllocationCount;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

//---------------------------------------------------------------------------
#endif
