//  BddFile_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "BddFile_Mb.h"
#include <stdio.h>

namespace Innovative
{

//---------------------------------------------------------------------------
//  BddInitObj:: - InitBddFile(BinView& graph)Set up BDD File for an output file
//---------------------------------------------------------------------------

void BddInitObj::InitBddFile(BinView& graph)
{
    // Optionally skip u
    BinFile(graph.BinFile());
    PathSpec spec(graph.BinFile());
    spec.Ext(".bdd");
    if (FileExists(spec.Full()) && !OverwriteBdd)
        return;

    remove(spec.Full().c_str());

    const int limit = 1 << (FBits-1);


    graph.Time().LowerLimit(-limit);
    graph.Time().UpperLimit(limit);
    graph.Time().Break(1);
    graph.Time().SeamSize((PacketdataSize)*sizeof(int));
    graph.Time().AnnotateSeams(true);


    graph.Fft().LowerLimit(-150);
    graph.Fft().ScaleYType(BinViewOptions::FftOptions::sLog);
    graph.Fft().ScaleXType(BinViewOptions::FftOptions::sLog);
    graph.Fft().CriteriaThreshold(75);
    int pktsize = std::min(PacketdataSize, 0x10000u);
    BinViewOptions::FftOptions::IIPoints pts(static_cast<BinViewOptions::FftOptions::IIPoints>(
        TrailingZeroCount(pktsize)-7));
    graph.Fft().Points(pts);
    graph.Fft().Window(BinViewOptions::FftOptions::wBlackmann);
    graph.Fft().Average(1);

    graph.Text().DataFormat("%11.3f");
    graph.System().UpdateRate(BinViewOptions::SystemOptions::ms1000);
    graph.Leap(PacketdataSize);
    graph.SignificantBits(FBits);
    graph.Polarity(Isunipolar ? BinView::pUnsigned : BinView::pSigned);
    graph.DataSpan(300);
    graph.DataType(BinView::tInt);
    graph.Units("mV");
    graph.Samples(1000);
    graph.SampleRate(SamplerateKhz);

    graph.Channels(Activechannels);
    graph.Devices(Activedevices);

    graph.System().Source(Fileformat);
    graph.NullDataValue(.12345e-19f);
    graph.InputSpan(InputSpan_mv);
    graph.ScalingEnabled(true);

    ApplyCustomSettings(graph);

    graph.Save();
}

//---------------------------------------------------------------------------
//  BddInitObj::ApplyCustomSettings() - default no customization impl
//---------------------------------------------------------------------------

void BddInitObj::ApplyCustomSettings(BinView & /*graph*/)
{
     //DO NOTHING
}





}
