// BinviewPlotter_App.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Application/BinviewPlotter_App.h>
#include <SystemSupport_Mb.h>
#include <BufferDatagrams_Mb.h>

using namespace Innovative;
using namespace std;

namespace Innovative
{

//===========================================================================
//  CLASS BinviewPlotter  --
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class BinviewPlotter
//---------------------------------------------------------------------------

BinviewPlotter::BinviewPlotter(unsigned int instance)
    : FUpdateRate(1.0), FInstance(instance), Elapsed(0),
	  Data(new RtDataSet), FIsStarted(false)
{
	Instance(instance);
}

//---------------------------------------------------------------------------
//  destructor for class BinviewPlotter
//---------------------------------------------------------------------------

BinviewPlotter::~BinviewPlotter()
{
    Stop();

    delete Data;
    Data = 0;
}

//---------------------------------------------------------------------------
//  Create a new instance from default.. useful for arrays of plotters.
//---------------------------------------------------------------------------

void BinviewPlotter::Instance(unsigned int NewInstance)
{
	FInstance = NewInstance;

	std::stringstream tag;
    tag << "BinviewPlotter_" << FInstance;

    Innovative::Application App;
    Bdd.BinFile(App.Path() + tag.str() + ".bin");

    Bdd.Restore();

	Bdd.System().ServerSlotName(tag.str());

	// Initialize BDD
    Bdd.System().Source(BinViewOptions::SystemOptions::sMapped);

    Bdd.Save();
}

//---------------------------------------------------------------------------
//  Create a new instance from default.. useful for arrays of plotters.
//---------------------------------------------------------------------------

unsigned int BinviewPlotter::Instance() const
{
	return FInstance;		
}

//------------------------------------------------------------------------
// BinviewPlotter::Start()
//------------------------------------------------------------------------

void BinviewPlotter::Start()
{
	if (IsStarted())
		return;

	Bdd.Save();

	Data->Enabled(false);

    Data->FileName(Bdd.BinFile());
    Data->Type(static_cast<DataSet::IIDataType>(Bdd.DataType()));
	Data->Channels(Bdd.Channels());
	Data->Devices(Bdd.Devices());
	Data->Bits(Bdd.SignificantBits());
	Data->Complex(Bdd.Complex());
	Data->Size(Bdd.Channels()*Bdd.Samples()*Data->SampleSize());

	Data->ReadOnly(false);
	Data->AutoCreate(true);

	Data->Enabled(true);

	Bdd.Plot();
    Elapsed = Rtc.Now()-UpdatePeriod();
	
	FIsStarted = true;
}

//------------------------------------------------------------------------
// BinviewPlotter::Stop()
//------------------------------------------------------------------------

void BinviewPlotter::Stop()
{
	if (IsStarted()==false)
		return;
		
    Data->Enabled(false);

	Bdd.Quit();
    Bdd.Restore();
	FIsStarted = false;
}

//---------------------------------------------------------------------------
//  BinviewPlotter::Update() --
//---------------------------------------------------------------------------

void BinviewPlotter::Update(const Buffer & buffer)
{
	if (IsStarted()==false)
		return;

	double Now = Rtc.Now();
	double Delta = Now - Elapsed;
    if (Delta < UpdatePeriod())
        return;
    Elapsed = Now;

	// Data is untyped, for purposes of movement into MM file.
	CharDG Wave(buffer);
	const size_t packet_elements = Wave.SizeInBytes()/Data->SampleSize();
	const size_t dataset_elements = static_cast<const size_t>(Data->Size()/Data->SampleSize());
	const size_t elements = std::min<size_t>(dataset_elements, packet_elements);
	// Since we are extracting data linearly .. fake the number of channels
	int SaveChannels = Data->Channels();
	Data->Channels(1);
	int idx = 0;
	Sample s(&Wave[idx], Data->SubsampleSize(), Data->Complex());
	Data->Element(idx, s);
	for (size_t i = 1; i < elements; ++i)
		{
		idx += Data->SampleSize();
		Sample s(&Wave[idx], Data->SubsampleSize(), Data->Complex());
		Data->CurrentSample(s);
		}

	Data->Channels(SaveChannels);		// Restore 
	Bdd.Refresh();
}


} // namespace

