//---------------------------------------------------------------------------

#include "BlockChecker_App.h"
#include <RandomGen_Mb.h>
#include <DataLogger_Mb.h>
#include <BinView_Mb.h>
#include <BinView_Mb.h>
#include <Poco/Random.h>

//---------------------------------------------------------------------------

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//==============================================================================
//  CLASS IBlockCheckerTester  --  Interface to create a generated pattern
//==============================================================================
//------------------------------------------------------------------------------
//  IBlockCheckerTester::FullCompare() -- compare entire buffer
//------------------------------------------------------------------------------

bool  IBlockCheckerTester::FullCompare(Innovative::Buffer & test, Innovative::Buffer & should_be)
{
    if (test.SizeInInts() != should_be.SizeInInts())
        return false;

    UIntegerDG TestDG(test);
	UIntegerDG ShouldBeDG(should_be);

    for (unsigned int i=0; i<TestDG.size(); ++i)
        if (TestDG[i] != ShouldBeDG[i])
            return false;

    return true;
}

//------------------------------------------------------------------------------
//  IBlockCheckerTester::QuickCompare() -- compare ends of a buffer
//------------------------------------------------------------------------------
//  Test continuity (missing data) rather than full accuracy

bool  IBlockCheckerTester::QuickCompare(Innovative::Buffer & test, Innovative::Buffer & should_be)
{
    if (test.SizeInInts() != should_be.SizeInInts())
        return false;

    UIntegerDG TestDG(test);
	UIntegerDG ShouldBeDG(should_be);

    if (TestDG[0] != ShouldBeDG[0])
        return false;
    else if (TestDG[TestDG.size()-1] != ShouldBeDG[ShouldBeDG.size()-1])
        return false;
    else
        return true;
}

//==============================================================================
//  CLASS PatternBlockTester  --  Use a single pattern block as a template
//==============================================================================
//------------------------------------------------------------------------------
//  PatternBlockTester::TestInit()
//------------------------------------------------------------------------------

void  PatternBlockTester::TestInit()
{
    Pattern.Resize(BlockSize());
    FillPatternBuffer();
}

//------------------------------------------------------------------------------
//  PatternBlockTester::RecvPatternGenerate() - return the pattern buffer
//------------------------------------------------------------------------------

void  PatternBlockTester::RecvPatternGenerate(Innovative::Buffer & out_buffer)
{
    out_buffer = Pattern;
}

//------------------------------------------------------------------------------
//  PatternBlockTester::XmitPatternGenerate() - return the pattern buffer
//------------------------------------------------------------------------------

void  PatternBlockTester::XmitPatternGenerate(Innovative::Buffer & out_buffer)
{
    out_buffer = Pattern;
}

//==============================================================================
//  CLASS RampBlockTester  --  Standard Ramp Block Tester
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class RampBlockTester()
//------------------------------------------------------------------------------

RampBlockTester::RampBlockTester()
{
}

//------------------------------------------------------------------------------
//  RampBlockTester::Compare()
//------------------------------------------------------------------------------

bool  RampBlockTester::Compare(Innovative::Buffer & test, Innovative::Buffer & should_be )
{
    return FullCompare(test, should_be);
}

//------------------------------------------------------------------------------
//  RampBlockTester::FillPatternBuffer()
//------------------------------------------------------------------------------

void  RampBlockTester::FillPatternBuffer()
{
	UIntegerDG PatternDG(Pattern);

	PacketBufferHeader PatternHeader(Pattern);
    PatternHeader.PeripheralId(BlockPid());
    PatternHeader.DataSize(Pattern.SizeInInts());

    for (unsigned int i=0; i<PatternDG.size(); ++i)
        PatternDG[i] = i;
}

//==============================================================================
//  CLASS RampBlockTester16  --  Standard Ramp Block Tester
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class RampBlockTester16()
//------------------------------------------------------------------------------

RampBlockTester16::RampBlockTester16()
{
}

//------------------------------------------------------------------------------
//  RampBlockTester16::Compare()
//------------------------------------------------------------------------------

bool  RampBlockTester16::Compare(Innovative::Buffer & test, Innovative::Buffer & should_be )
{
    return FullCompare(test, should_be);
}

//------------------------------------------------------------------------------
//  RampBlockTester16::FillPatternBuffer()
//------------------------------------------------------------------------------

void  RampBlockTester16::FillPatternBuffer()
{
    UShortDG PatternDG(Pattern);

    PacketBufferHeader PatternHeader(Pattern);
    PatternHeader.PeripheralId(BlockPid());
    PatternHeader.DataSize(Pattern.SizeInInts());

    for (unsigned int i=0; i<PatternDG.size(); ++i)
        PatternDG[i] = Ramp++;
}

//------------------------------------------------------------------------------
//  RampBlockTester16::RecvPatternGenerate() - return the pattern buffer
//------------------------------------------------------------------------------

void  RampBlockTester16::RecvPatternGenerate(Innovative::Buffer & out_buffer)
{
    FillPatternBuffer();
    out_buffer = Pattern;
}

//==============================================================================
//  CLASS QuickRampBlockTester  --  Quick Ramp Block Tester (continuity only)
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class QuickRampBlockTester()
//------------------------------------------------------------------------------

QuickRampBlockTester::QuickRampBlockTester()
{
}

//------------------------------------------------------------------------------
//  QuickRampBlockTester::Compare()
//------------------------------------------------------------------------------

bool  QuickRampBlockTester::Compare(Innovative::Buffer & test, Innovative::Buffer & should_be )
{
    return QuickCompare(test, should_be);
}

//==============================================================================
//  CLASS AlternatingBitsBlockTester  --  A5-5A Ramp Block Tester
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class AlternatingBitsBlockTester()
//------------------------------------------------------------------------------

AlternatingBitsBlockTester::AlternatingBitsBlockTester()
{
}

//------------------------------------------------------------------------------
//  AlternatingBitsBlockTester::Compare()
//------------------------------------------------------------------------------

bool  AlternatingBitsBlockTester::Compare(Innovative::Buffer & test, Innovative::Buffer & should_be )
{
    return FullCompare(test, should_be);
}

//------------------------------------------------------------------------------
//  AlternatingBitsBlockTester::FillPatternBuffer()
//------------------------------------------------------------------------------

void  AlternatingBitsBlockTester::FillPatternBuffer()
{
	UIntegerDG PatternDG(Pattern);

	PacketBufferHeader PatternHeader(Pattern);
    PatternHeader.PeripheralId(BlockPid());
    PatternHeader.DataSize(Pattern.SizeInInts());

    for (unsigned int i=0; i<PatternDG.size(); i+=2)
        {
        PatternDG[i] = 0xa5a5a5a5;
        PatternDG[i+1] = 0x5a5a5a5a;
        }
}

//==============================================================================
//  CLASS WalkingBitBlockTester  --  Walking Bit Block Tester
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class WalkingBitBlockTester()
//------------------------------------------------------------------------------

WalkingBitBlockTester::WalkingBitBlockTester()
{
}

//------------------------------------------------------------------------------
//  WalkingBitBlockTester::Compare()
//------------------------------------------------------------------------------

bool  WalkingBitBlockTester::Compare(Innovative::Buffer & test, Innovative::Buffer & should_be )
{
    return FullCompare(test, should_be);
}

//------------------------------------------------------------------------------
//  WalkingBitBlockTester::FillPatternBuffer()
//------------------------------------------------------------------------------

void  WalkingBitBlockTester::FillPatternBuffer()
{
	UIntegerDG PatternDG(Pattern);

	PacketBufferHeader PatternHeader(Pattern);
    PatternHeader.PeripheralId(BlockPid());
    PatternHeader.DataSize(Pattern.SizeInInts());

    for (unsigned int i=0; i<PatternDG.size(); i+=1)
        {
        PatternDG[i] = 1<<i;
//        PatternDG[i+1] = 1<<1;
        }
}

//==============================================================================
//  CLASS RandomBitsBlockTester  --  A5-5A Ramp Block Tester
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class RandomBitsBlockTester()
//------------------------------------------------------------------------------

RandomBitsBlockTester::RandomBitsBlockTester()
{
}

//------------------------------------------------------------------------------
//  RandomBitsBlockTester::Compare()
//------------------------------------------------------------------------------

bool  RandomBitsBlockTester::Compare(Innovative::Buffer & test, Innovative::Buffer & should_be )
{
    return FullCompare(test, should_be);
}

//------------------------------------------------------------------------------
//  RandomBitsBlockTester::FillPatternBuffer()
//------------------------------------------------------------------------------

void  RandomBitsBlockTester::FillPatternBuffer()
{
	PacketBufferHeader PatternHeader(Pattern);
    PatternHeader.PeripheralId(BlockPid());
    PatternHeader.DataSize(Pattern.SizeInInts());

	IntegerDG p(Pattern);
	Poco::Random Gen;
	Gen.seed();
	for (size_t i = 0; i < p.size(); ++i)
		p[i] = Gen.next();
}

//==============================================================================
//  CLASS BlockChecker  --  Check block data against a generated pattern
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class BlockChecker()
//------------------------------------------------------------------------------

BlockChecker::BlockChecker(TestModes mode)
    : mMode(mode), mMaxBlocks(20), mBlocksWithErrors(0), CustomTest(0)
{
    SwitchMode();
}

//------------------------------------------------------------------------------
//  BlockChecker::Init()
//------------------------------------------------------------------------------

void  BlockChecker::Init()
{
    mBlocksWithErrors = 0;
    ErrorBlocks.clear();
    ShouldBeBlocks.clear();

    CurrentTest->TestInit();
}

//------------------------------------------------------------------------------
//  BlockChecker::Check() -- Check Block against 'test pattern'
//------------------------------------------------------------------------------

void  BlockChecker::Check(Innovative::Buffer & in)
{
    Buffer should_be( in.SizeInInts() );
    //
    //  Generate pattern
    CurrentTest->RecvPatternGenerate(should_be);

    bool error = ! CurrentTest->Compare(in, should_be);
    //
    //  Error block processing
    if (error || mKeepFirst)
        {
        // count this as an error...
        mBlocksWithErrors++;

        if (ErrorBlocks.size() >= mMaxBlocks)
            return;
        //
        //  Save blocks in store
        ErrorBlocks.push_back(in);
        ShouldBeBlocks.push_back(should_be);
        }
}

//------------------------------------------------------------------------------
//  BlockChecker::Fill() -- Fill block from enerator
//------------------------------------------------------------------------------

void  BlockChecker::Fill(Innovative::Buffer & out)
{
    //
    //  Generate pattern
    CurrentTest->XmitPatternGenerate(out);
}

//------------------------------------------------------------------------------
//  BlockChecker::SwitchMode() -- Switch current operating mode
//------------------------------------------------------------------------------

void  BlockChecker::SwitchMode()
{
    switch (mMode)
        {
        default:
        case tmWalkingBitsTest:
            CurrentTest = &WalkingBitsTest;
            break;
        case tmRampTest:
            CurrentTest = &RampTest;
            break;
        case tmQuickRampTest:
            CurrentTest = &QuickRampTest;
            break;
        case tmAlternatingBitsTest:
            CurrentTest = &AltBitsBlockTest;
            break;
        case tmRandomBitsTest:
            CurrentTest = &RandomTest;
            break;
        case tm16BitRamp:
            CurrentTest = &RampTest16;
            break;
        case tmCustomTest:
            CurrentTest = CustomTest;
            break;
        }
}

//------------------------------------------------------------------------------
//  BlockChecker::BlockPid() -- Read Block Pid
//------------------------------------------------------------------------------

unsigned int  BlockChecker::BlockPid()
{
    if (CurrentTest)
        return CurrentTest->BlockPid();
    else
        return 0;
}

//------------------------------------------------------------------------------
//  BlockChecker::BlockPid() -- Write Block Pid
//------------------------------------------------------------------------------

void BlockChecker::BlockPid(unsigned int pid)
{
    if (CurrentTest)
        CurrentTest->BlockPid(pid);
}

//------------------------------------------------------------------------------
//  BlockChecker::BlockSize() -- Read Block Size
//------------------------------------------------------------------------------

unsigned int  BlockChecker::BlockSize()
{
    if (CurrentTest)
        return CurrentTest->BlockSize();
    else
        return 0;
}

//------------------------------------------------------------------------------
//  BlockChecker::BlockSize() -- Write Block Size
//------------------------------------------------------------------------------

void BlockChecker::BlockSize(unsigned int data_size_ints)
{
    if (CurrentTest)
        CurrentTest->BlockSize(data_size_ints);
}

//------------------------------------------------------------------------------
//  BlockChecker::LogToDisk() -- Log block data to disk
//------------------------------------------------------------------------------

void  BlockChecker::LogToDisk(const std::string & file_name)
{
    DataLogger   Logger;
    BinView      Bv;

    Logger.FileName(file_name);
    Bv.BinFile(Logger.FileName());

    Logger.Start();
    // Log Error Packets
    for (unsigned int i=0; i<ErrorBlocks.size(); i++)
        {
        //  Log Error Packets
        Buffer & Eb(ErrorBlocks[i]);
	    PacketBufferHeader EbHeader(Eb);
        EbHeader.PeripheralId(0x0);                // Error blocks header = 0
        Logger.LogWithHeader(ErrorBlocks[i]);

        //  Log "Should Be" Packets
        Buffer & Sb(ShouldBeBlocks[i]);
	    PacketBufferHeader SbHeader(Sb);
        SbHeader.PeripheralId(0x1);
        Logger.LogWithHeader(ShouldBeBlocks[i]);    // Should be  header = 1
        }

    Logger.Stop();

    if (ErrorBlocks.empty())
        return;

    // Init Bdd File
	Bv.Time().LowerLimit(0x80000001);
	Bv.Time().UpperLimit(0x7fffffff);
	Bv.Time().Break(1);

    Buffer & Eb_0(ErrorBlocks[0]);
    PacketBufferHeader Eb0_Header(Eb_0);
    unsigned int PacketDataSize = static_cast<unsigned int>(Eb0_Header.DataSize());
    const unsigned int PidCount = 2;
	Bv.Time().SeamSize(PidCount*PacketDataSize*sizeof(int));
    Bv.Time().AnnotateSeams(true);

    Bv.Fft().LowerLimit(-150);
    Bv.Fft().ScaleYType(BinViewOptions::FftOptions::sLog);
    Bv.Fft().ScaleXType(BinViewOptions::FftOptions::sLog);
    Bv.Fft().CriteriaThreshold(75);
    Bv.Fft().Window(BinViewOptions::FftOptions::wBlackmann);
    Bv.Fft().Average(1);

    Bv.Text().DataFormat("%11.3x");
    Bv.System().UpdateRate(BinViewOptions::SystemOptions::ms1000);
    Bv.Leap(PacketDataSize);
    Bv.SignificantBits(32);
    Bv.Polarity(BinView::pSigned);
    Bv.DataSpan(300);
    Bv.DataType(BinView::tInt);
    Bv.Units("mV");
    Bv.Samples(1000);
    Bv.Channels(2);
    Bv.Devices(2);
    Bv.System().Source(BinViewOptions::SystemOptions::sPacketFile);
    Bv.NullDataValue(.12345e-19f);
    float rate = static_cast<float>(100e6/1000.);
    Bv.SampleRate(rate);
    Bv.InputSpan(2000);
    Bv.ScalingEnabled(false);

    Bv.Save();

}
#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative

