// FiclIo_App.cpp
//
//  Ficl Support Linkage

#include "FiclIo_App.h"
#include <sstream>

using namespace std;

//===========================================================================
//  CLASS FiclIo  -- Ficl Support for Application
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FiclIo
//---------------------------------------------------------------------------

FiclIo::FiclIo(IFicl_UserInterface * ficlUI)
	:  FiclUI(ficlUI)
{
	Ficl.OnDisplayText.SetEvent(this, &FiclIo::HandleFiclOnDisplayText);
}

//---------------------------------------------------------------------------
//  constructor for class FiclIo
//---------------------------------------------------------------------------

FiclIo::FiclIo(const std::string & tag, IFicl_UserInterface * ficlUI)
	:  FiclUI(ficlUI), FTag(tag)
{
	Ficl.OnDisplayText.SetEvent(this, &FiclIo::HandleFiclOnDisplayText);
}

//---------------------------------------------------------------------------
//  constructor for class FiclIo
//---------------------------------------------------------------------------

FiclIo::FiclIo(const std::string & tag, IFicl_UserInterface * ficlUI, const std::string & help_command)
	:  FiclUI(ficlUI), FTag(tag), FHelpCommand(help_command)
{
	Ficl.OnDisplayText.SetEvent(this, &FiclIo::HandleFiclOnDisplayText);
}

//---------------------------------------------------------------------------
//  destructor for class FiclIo
//---------------------------------------------------------------------------

FiclIo::~FiclIo()
{
}

//---------------------------------------------------------------------------
//  FiclIo::FiclIo_Init() -- Initialization of Ficl
//---------------------------------------------------------------------------

void  FiclIo::FiclIo_Init()
{
    Ficl.Start();
}

//-----------------------------------------------------------------------------
//  FiclIo::HandleFiclOnDisplayText()  --
//-----------------------------------------------------------------------------

void FiclIo::HandleFiclOnDisplayText(Innovative::ProcessStatusEvent & event)
{
    FiclUI->FiclLog(event.Message);
}

//-----------------------------------------------------------------------------
//  FiclIo::FiclIo_OnCommandTextComplete()  -- Command Text Ready
//-----------------------------------------------------------------------------
//  Qt seems to not give us the keypress handler but we have a 'complete' signal

void FiclIo::FiclIo_OnCommandTextComplete()
{
    string cmd = FiclUI->FiclReadCommandString();
    Execute(cmd);
}

//-----------------------------------------------------------------------------
//  FiclIo::FiclIo_OnCommandKeyPress()  -- Key Press Handler
//-----------------------------------------------------------------------------

void FiclIo::FiclIo_OnCommandKeyPress(char &Key)
{
	if (Key != 0x0d)
		return;

    FiclIo_OnCommandTextComplete();
}

//-----------------------------------------------------------------------------
//  FiclIo::Evaluate()  -- Command processor
//-----------------------------------------------------------------------------

void  FiclIo::Evaluate(const string & cmd)
{
    stringstream txt;
    txt << cmd << " ";
    Ficl.Display(txt.str());

    Ficl.Evaluate(cmd);
    Ficl.Display("\n");

    FiclUI->FiclCommandComplete();
}

//-----------------------------------------------------------------------------
//  FiclIo::Execute()  -- Command processor
//-----------------------------------------------------------------------------

void  FiclIo::Execute(const string & cmd)
{
    stringstream txt;
    txt << cmd << " ";
    Ficl.Display(txt.str());

    Ficl.Execute(cmd);
    Ficl.Display("\n");

    FiclUI->FiclCommandComplete();
}

//---------------------------------------------------------------------------
//  FiclIo::Port() --  Read a register in Port Momory, Log results
//---------------------------------------------------------------------------

void FiclIo::Port(const string & address)
{
    stringstream msg;
    msg << address << " p@ x.";
    Execute(msg.str());
}

//---------------------------------------------------------------------------
//  FiclIo::Port() --  Write a register in Port Momory, Log results
//---------------------------------------------------------------------------

void FiclIo::Port(const string & address, const string & data)
{
    stringstream msg;
    msg << data << " " << address << " p!";
    Execute(msg.str());
}

//---------------------------------------------------------------------------
//  FiclIo::Logic() --  Read a register in Logic memory
//---------------------------------------------------------------------------

void FiclIo::Logic(const string & address)
{
    stringstream msg;
    msg << address << " l@ x.";
    Execute(msg.str());
}

//---------------------------------------------------------------------------
//  FiclIo::Logic() --  Write a register in Logic Memory
//---------------------------------------------------------------------------

void FiclIo::Logic(const string & address, const string & data)
{
    stringstream msg;
    msg << data << " " << address << " l!";
    Execute(msg.str());
}

//---------------------------------------------------------------------------
//  FiclIo::Device() --  Read a register in device memory
//---------------------------------------------------------------------------

void FiclIo::Device(const string & address, const string & device)
{
    stringstream msg;
    msg << address << " " << device << " d@ x.";
    Execute(msg.str());
}

//---------------------------------------------------------------------------
//  FiclIo::Device() --  Write a register in device Memory
//---------------------------------------------------------------------------

void FiclIo::Device(const string & address, const string & device, const string & data)
{
    stringstream msg;
    msg << data << " " << address << " " << device << " d!";
    Execute(msg.str());
}





