//  FiclIo_App.h
//
//  Application Io Extensions for FICL support
//---------------------------------------------------------------------------

#ifndef FiclIo_AppH
#define FiclIo_AppH

#include <Ficl/FiclSystem_Mb.h>

//===========================================================================
//  CLASS IFicl_UserInterface  -- Interface for callbacks to UI from Ficl
//===========================================================================

class IFicl_UserInterface
{
public:
    virtual ~IFicl_UserInterface()  {  }

    virtual void         FiclLog(const std::string & a_string) = 0;
    virtual std::string  FiclReadCommandString() = 0;
    virtual void         FiclCommandComplete() = 0;
};

//===========================================================================
//  CLASS FiclIo  -- FICL Io Support Class
//===========================================================================

class FiclIo
{
public:
    FiclIo(IFicl_UserInterface * ficlUI);
    FiclIo(const std::string & tag, IFicl_UserInterface * ficlUI);
    FiclIo(const std::string & tag, IFicl_UserInterface * ficlUI, const std::string & help_command);
    ~FiclIo();

    Innovative::FiclSystem & FiclSys()
        {  return Ficl;  }

    void  FiclIo_Init();
    void  FiclIo_OnCommandTextComplete();
    void  FiclIo_OnCommandKeyPress(char &Key);

    void  Evaluate(const std::string & cmd);      // no prompt at end
    void  Execute(const std::string & cmd);       // prints prompt at end

    void  SetHelpCommandText(const std::string & help_command)
        {  FHelpCommand = help_command;  }

    // Module register access
    void Port(const std::string & address, const std::string & data);
    void Port(const std::string & address);

    void Logic(const std::string & address, const std::string & data);
    void Logic(const std::string & address);

    void Device(const std::string & address, const std::string & device,
                const std::string & data);
    void Device(const std::string & address, const std::string & device);

    void ExecuteDebugScript(const std::string & script_file)
            {  Execute(std::string("load ") + script_file);  }

    void Help()
            {  Execute(FHelpCommand);  }

    std::string  Tag()
            {  return FTag;  }

protected:

private:
    Innovative::FiclSystem     Ficl;
    IFicl_UserInterface *      FiclUI;
    std::string                FTag;
    std::string                FHelpCommand;

    void    HandleFiclOnDisplayText(Innovative::ProcessStatusEvent & event);
};


//==============================================================================
//  CLASS  FiclGroup  -- collection of FiclIo packs
//==============================================================================
//  Objects are owned by the FiclGroup

class FiclGroup
{
public:
    FiclGroup()
        {}
    ~FiclGroup()
        {
        Clear();
        }

    void Clear()
        {
        while( Group.size() > 0)
            {
            delete Group.back();
            Group.pop_back();
            }
        }

    void Append(FiclIo * value)
        {   Group.push_back(value);   }

    size_t  Size()
        {   return Group.size();  }

    FiclIo &  Ref_At(size_t idx)
        {   return *Group[idx];   }
    FiclIo &  operator [](size_t idx)
        {   return Ref_At(idx);   }

private:
    //
    //
    std::vector<FiclIo *> Group;
};


#endif
