//---------------------------------------------------------------------------

#ifndef McImageChecker_AppH
#define McImageChecker_AppH


#include <BufferDatagrams_Mb.h>
#include <BufferHeader_Mb.h>
#include <vector>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IImageCheckerTester  --  Interface to create a generated pattern
//==============================================================================

class IImageCheckerTester
{
public:
    IImageCheckerTester()
        {}
    virtual ~IImageCheckerTester()
        {}

    //  Properties
    virtual void  TestInit() = 0;
    virtual void  RecvPatternGenerate(unsigned int channel, Innovative::McBuffer & out_buffer) = 0;
    virtual bool  Compare(Innovative::McImage & test, Innovative::McBuffer & should_be ) = 0;

protected:
    bool FullCompare(Innovative::McImage & test, Innovative::McBuffer & should_be);
    bool QuickCompare(Innovative::McImage & test, Innovative::McBuffer & should_be);

};

//==============================================================================
//  CLASS LogicRampsImageTester  --
//==============================================================================

class LogicRampsImageTester : public IImageCheckerTester
{
public:
    LogicRampsImageTester() : Ramps(8)
        {}
    virtual ~LogicRampsImageTester()
        {}
    // Properties
    // Methods
    virtual void  TestInit();
    virtual void  RecvPatternGenerate(unsigned int channel, Innovative::McBuffer & out_buffer);

    virtual bool  Compare(Innovative::McImage & test, Innovative::McBuffer & should_be )
                        {   return FullCompare(test, should_be);  }

protected:
    std::vector<unsigned int>  Ramps;
};

//==============================================================================
//  CLASS QuickLogicRampsImageTester  --
//==============================================================================

class QuickLogicRampsImageTester : public LogicRampsImageTester
{
public:
    QuickLogicRampsImageTester()
        {}
    virtual ~QuickLogicRampsImageTester()
        {}
    // Properties
    // Methods
    virtual bool  Compare(Innovative::McImage & test, Innovative::McBuffer & should_be )
            {   return QuickCompare(test, should_be);  }

protected:

};

//==============================================================================
//  CLASS McImageChecker  --  Check Image data against a generated pattern
//==============================================================================

class McImageChecker
{
public:
    enum TestModes {  tmLogicRampsTest, tmQuickLogicRampsTest,
                      tmCustomTest };

    McImageChecker(TestModes mode = tmLogicRampsTest);

    TestModes     Mode()  { return mMode; }
    void          Mode(TestModes mode) {  mMode = mode;  SwitchMode(); }
    void          Mode(int mode) {  Mode(static_cast<TestModes>(mode)); }

    void  Init();
//    void  Fill(Innovative::McImage & out);
    void  Check(Innovative::McImage & in);
    void  LoadCustomTest(IImageCheckerTester * custom)
            {  CustomTest = custom;  }

    void  LogToDisk(const std::string & file_name);

    unsigned int  MaxImages()  { return mMaxImages; }
    void          MaxImages(unsigned int max) {  mMaxImages = max;  }
    bool          KeepFirstImages()  { return mKeepFirst; }
    void          KeepFirstImages(bool state) {  mKeepFirst = state;  }

    int       ImagesWithErrors() { return mImagesWithErrors; }
    int       ImagesSaved() {  return static_cast<int>(ErrorImages.size());  }
    Innovative::Buffer &      ErrorImage(int idx)
            {  return ErrorImages[idx];  }
    Innovative::Buffer &      ShouldBeImage(int idx)
            {  return ShouldBeImages[idx];  }
    //
    //  Standard Tests
    LogicRampsImageTester         LogicRampsTest;
    QuickLogicRampsImageTester    QuickLogicRampsTest;

private:
    IImageCheckerTester * CurrentTest;
    TestModes     mMode;
    unsigned int  mMaxImages;
    int           mImagesWithErrors;
    bool          mKeepFirst;

    IImageCheckerTester * CustomTest;

    std::vector<Innovative::McBuffer>      ErrorImages;
    std::vector<Innovative::McBuffer>      ShouldBeImages;

    void  SwitchMode();

};

//==============================================================================
//  CLASS IMcBufferCheckerTester  --  Interface to create a generated pattern
//==============================================================================

class IMcBufferCheckerTester
{
public:
    IMcBufferCheckerTester()
        {}
    virtual ~IMcBufferCheckerTester()
        {}

    //  Properties
    virtual void  TestInit() = 0;
    virtual void  RecvPatternGenerate(unsigned int channel, Innovative::McBuffer & out_buffer) = 0;
    virtual void  XmitPatternGenerate(unsigned int channel, Innovative::McBuffer & out_buffer) = 0;
    virtual bool  Compare(Innovative::McBuffer & test, Innovative::McBuffer & should_be ) = 0;

protected:
    bool FullCompare(Innovative::McBuffer & test, Innovative::McBuffer & should_be);
    bool QuickCompare(Innovative::McBuffer & test, Innovative::McBuffer & should_be);

};

//==============================================================================
//  CLASS LogicRampsMcBufferTester  --
//==============================================================================

class LogicRampsMcBufferTester : public IMcBufferCheckerTester
{
public:
    LogicRampsMcBufferTester() : Ramps(8), TxRamps(8)
        {}
    virtual ~LogicRampsMcBufferTester()
        {}
    // Properties
    // Methods
    virtual void  TestInit();
    virtual void  RecvPatternGenerate(unsigned int channel, Innovative::McBuffer & out_buffer);
    virtual void  XmitPatternGenerate(unsigned int channel, Innovative::McBuffer & out_buffer);

    virtual bool  Compare(Innovative::McBuffer & test, Innovative::McBuffer & should_be )
                        {   return FullCompare(test, should_be);  }

protected:
    std::vector<unsigned int>  Ramps;
    std::vector<unsigned int>  TxRamps;
};

//==============================================================================
//  CLASS QuickLogicRampsMcBufferTester  --
//==============================================================================

class QuickLogicRampsMcBufferTester : public LogicRampsMcBufferTester
{
public:
    QuickLogicRampsMcBufferTester()
        {}
    virtual ~QuickLogicRampsMcBufferTester()
        {}
    // Properties
    // Methods
    virtual bool  Compare(Innovative::McBuffer & test, Innovative::McBuffer & should_be )
            {   return QuickCompare(test, should_be);  }

protected:

};

//==============================================================================
//  CLASS McBufferChecker  --  Check Buffer data against a generated pattern
//==============================================================================

class McBufferChecker
{
public:
    enum TestModes {  tmLogicRampsTest, tmQuickLogicRampsTest,
                      tmCustomTest };

    McBufferChecker(TestModes mode = tmLogicRampsTest);

    TestModes     Mode()  { return mMode; }
    void          Mode(TestModes mode) {  mMode = mode;  SwitchMode(); }
    void          Mode(int mode) {  Mode(static_cast<TestModes>(mode)); }

    void  Init();
    void  Fill(unsigned int channel, Innovative::McBuffer & out);
    void  Check(Innovative::McBuffer & in);
    void  LoadCustomTest(IMcBufferCheckerTester * custom)
            {  CustomTest = custom;  }

    void  LogToDisk(const std::string & file_name);

    unsigned int  MaxImages()  { return mMaxImages; }
    void          MaxImages(unsigned int max) {  mMaxImages = max;  }
    bool          KeepFirstImages()  { return mKeepFirst; }
    void          KeepFirstImages(bool state) {  mKeepFirst = state;  }

    int       ImagesWithErrors() { return mImagesWithErrors; }
    int       ImagesSaved() {  return static_cast<int>(ErrorImages.size());  }
    Innovative::Buffer &      ErrorImage(int idx)
            {  return ErrorImages[idx];  }
    Innovative::Buffer &      ShouldBeImage(int idx)
            {  return ShouldBeImages[idx];  }
    //
    //  Standard Tests
    LogicRampsMcBufferTester         LogicRampsTest;
    QuickLogicRampsMcBufferTester    QuickLogicRampsTest;

private:
    IMcBufferCheckerTester * CurrentTest;
    TestModes     mMode;
    unsigned int  mMaxImages;
    int           mImagesWithErrors;
    bool          mKeepFirst;

    IMcBufferCheckerTester * CustomTest;

    std::vector<Innovative::McBuffer>      ErrorImages;
    std::vector<Innovative::McBuffer>      ShouldBeImages;

    void  SwitchMode();

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
