//  MultiLoggerManager_App.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.

#include <Application/MultiLoggerManager_App.h>
#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>
#include <sstream>
#include <algorithm>
#include <FileSupport_Mb.h>
namespace   Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS MultiLoggerManager - Manages setup of multiple logger and viewers
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class MultiLoggerManager
//------------------------------------------------------------------------------

MultiLoggerManager::MultiLoggerManager()
    :  FRoot("")
{
}

//------------------------------------------------------------------------------
//  destructor for class MultiLoggerManager
//------------------------------------------------------------------------------

MultiLoggerManager::~MultiLoggerManager()
{
    Cleanup();
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Init() --  Initialize the manager
//------------------------------------------------------------------------

void  MultiLoggerManager::Init(unsigned int loggers_to_make)
{
    Cleanup();

    Data.resize(loggers_to_make);

    for (unsigned int i=0; i<Data.size(); i++)
        {
        Data[i].Init(FRoot, i);
        }
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Cleanup() --  Get rid of all logger
//------------------------------------------------------------------------

void  MultiLoggerManager::Cleanup()
{
    for (unsigned int i=0; i<Data.size(); i++)
        {
        //  Delete the loggers & graph
        Data[i].Cleanup();
        }
}

//------------------------------------------------------------------------
//  MultiLoggerManager::StartAll() --  Start all loggers
//------------------------------------------------------------------------

void  MultiLoggerManager::StartAll()
{
    for (unsigned int i=0; i<Data.size(); i++)
        {
        Data[i].AtStartConfig();

        if (Data[i].LoggerEnable || Data[i].PlotEnable)
            Data[i].Logger->Start();
        }
}

//------------------------------------------------------------------------
//  MultiLoggerManager::StopAll() --  Stop all loggers, pop up graphs
//------------------------------------------------------------------------

void  MultiLoggerManager::StopAll()
{
    for (unsigned int i=0; i<Data.size(); i++)
        {
        if (Data[i].LoggerEnable || Data[i].PlotEnable)
            {           
            //  Stop Logging
            Data[i].Logger->Stop();

            //  Lauch graph view of channel
            LaunchGraph(i);

            Data[i].AtStopConfig();
            }
        }

}

//------------------------------------------------------------------------
//  MultiLoggerManager::QuitAll() --  Quit all Graphs
//------------------------------------------------------------------------

void  MultiLoggerManager::QuitAll()
{
    for (unsigned int i=0; i<Data.size(); i++)
        {
        if (Data[i].PlotEnable==false)
            continue;
        //  Quit Graphing
        Data[i].Graph->Quit();
        //  Restore to get any changes
        Data[i].Graph->Restore();
        }
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Logger() --  Return a ref to a particular logger
//------------------------------------------------------------------------

Innovative::DataLogger &  MultiLoggerManager::Logger(unsigned int idx)
{
    return *(Data[idx].Logger);
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Graph() --  Return a ref to a particular graph
//------------------------------------------------------------------------

Innovative::BinView & MultiLoggerManager::Graph(unsigned int idx)
{
    return *(Data[idx].Graph);
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Graph() --  Return a ref to a particular graph
//------------------------------------------------------------------------

BddInitObj &  MultiLoggerManager::BddFile(unsigned int idx)
{
    return *(Data[idx].BddFile);
}

//------------------------------------------------------------------------
//  MultiLoggerManager::FileNamer() --  Access IncrementalFileName for ch
//------------------------------------------------------------------------

IncrementalFileName &MultiLoggerManager::FileNamer(unsigned int idx)
{
    return Data[idx].IFN;
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Id() --  Set ID field
//------------------------------------------------------------------------

void  MultiLoggerManager::Id(unsigned int idx, unsigned int id)
{
    Data[idx].Id = id & 0xFF;    // PID is only 8 bits wide
}

//------------------------------------------------------------------------
//  MultiLoggerManager::LaunchGraph() -- Create BDD, Launch Graph
//------------------------------------------------------------------------

void  MultiLoggerManager::LaunchGraph(unsigned int idx)
{
    Data[idx].LaunchGraph();
}

//------------------------------------------------------------------------
//  MultiLoggerManager::IsDataLoggingCompleted() --
//------------------------------------------------------------------------

bool  MultiLoggerManager::IsDataLoggingCompleted()
{
    bool AllCompleted(true);
    for (unsigned int i=0; i<Data.size(); i++)
        {
        if (Data[i].LoggerEnable==false)
            continue;   // don't count this one
            if (Data[i].WordsToLog==0)
            AllCompleted = false;   // this one never is completed
        else if (Data[i].WordCount < Data[i].WordsToLog)
            AllCompleted = false;
        // if you get here, this log is completed, so AllCompleted stays the same
        }

    return AllCompleted;
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Log() -- Log data to appropriate logger
//------------------------------------------------------------------------

void  MultiLoggerManager::Log(Innovative::PmcBuffer & buffer)
{
    unsigned int idx = FindIdxForBuffer(buffer);
    if (idx>=Count())
        return;

    Data[idx].Log(buffer);
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Log() -- Log data to appropriate logger
//------------------------------------------------------------------------

void MultiLoggerManager::Log(McBuffer& buffer)
{
    unsigned int idx = FindIdxForBuffer(buffer);
    if (idx>=Count())
        return;

    Data[idx].Log(buffer);
}

//------------------------------------------------------------------------
//  MultiLoggerManager::Log() -- Log data to appropriate logger
//------------------------------------------------------------------------

void MultiLoggerManager::Log(McImage& image)
{
    unsigned int idx = FindIdxForBuffer(image);
    if (idx>=Count())
        return;

    Data[idx].Log(image);
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::FindIdxForBuffer() -- Find logger for buffer
//------------------------------------------------------------------------------

unsigned int  MultiLoggerManager::FindIdxForBuffer(Innovative::PmcBuffer & buffer)
{
    PmcHeaderDatagram header(buffer);
    for (unsigned int i=0; i<Count(); i++)
        {
        if (Data[i].Id == static_cast<unsigned int>(header.PeripheralId()))
            return i;
        }
    return static_cast<unsigned int>(-1);
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::FindIdxForBuffer() -- Find logger for buffer
//------------------------------------------------------------------------------

unsigned int MultiLoggerManager::FindIdxForBuffer(McBuffer& buffer)
{
    McHeaderDatagram header(buffer);
    for (unsigned int i=0; i<Count(); i++)
        {
        if (Data[i].Id == static_cast<unsigned int>(header.ChannelId()))
            return i;
        }
    return static_cast<unsigned int>(-1);
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::FindIdxForBuffer() -- Find logger for buffer
//------------------------------------------------------------------------------

unsigned int MultiLoggerManager::FindIdxForBuffer(McImage& image)
{
    McHeaderDatagram header(image);
    for (unsigned int i=0; i<Count(); i++)
        {
        if (Data[i].Id == static_cast<unsigned int>(header.ChannelId()))
            return i;
        }
    return static_cast<unsigned int>(-1);
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::LoggerEnable() -- Set a Logger enable flag
//------------------------------------------------------------------------------

void  MultiLoggerManager::LoggerEnable(size_t idx, bool enable)
{
    if (idx < Count())
        {
        Data[idx].LoggerEnable = enable;
        }
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::PlotEnable() -- Set a Plot enable flag
//------------------------------------------------------------------------------

void  MultiLoggerManager::PlotEnable(size_t idx, bool enable)
{
    if (idx < Count())
        {
        Data[idx].PlotEnable = enable;
        }
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::OverwriteBdd() -- Set a OverrideBdd flag
//------------------------------------------------------------------------------

void  MultiLoggerManager::OverwriteBdd(size_t idx, bool overwrite)
{
    if (idx < Count())
        {
        Data[idx].OverwriteBdd = overwrite;
        }
}


//------------------------------------------------------------------------------
//  MultiLoggerManager::UniqueFileName() -- Set a unique name flag
//------------------------------------------------------------------------------

void  MultiLoggerManager::UniqueFileName(size_t idx, bool uniquefilename)
{
    if (idx < Count())
        {
        Data[idx].UniqueFileName = uniquefilename;
        }
}
//------------------------------------------------------------------------------
//  MultiLoggerManager::WordsToLog() -- load word count
//------------------------------------------------------------------------------

void  MultiLoggerManager::WordsToLog(size_t idx, ii64 words)
{
    if (idx < Count())
        {
        Data[idx].WordsToLog = words;
        }
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::SamplesToLog() -- load word count via samples
//------------------------------------------------------------------------------

void  MultiLoggerManager::SamplesToLog(size_t idx, ii64 samples)
{
    ii64 words(samples);
    if (idx < Count())
        {
        words *= SamplesPerWord(BddFile(idx).Bits());
        }
    WordsToLog(idx, words);

}

//------------------------------------------------------------------------------
//  MultiLoggerManager::WordsToLog() -- load word count
//------------------------------------------------------------------------------

void  MultiLoggerManager::WordsToLog(const std::vector<ii64> & words)
{
    size_t count = std::min(Count(), words.size());
    for (size_t i=0; i<count; i++)
        {
        WordsToLog(i, words[i]);
        }
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::SamplesToLog() -- load sample count
//------------------------------------------------------------------------------

void  MultiLoggerManager::SamplesToLog(const std::vector<ii64> & samples)
{
    size_t count = std::min(Count(), samples.size());
    for (size_t i=0; i<count; i++)
        {
        SamplesToLog(i, samples[i]);
        }
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::SamplesPerWord() -- samples per 32 bit word
//------------------------------------------------------------------------------

unsigned int  MultiLoggerManager::SamplesPerWord(unsigned int bits)
{
    if (bits <= 32 && bits > 16)
        return 1;
    else if (bits <= 16 && bits > 8)
        return 2;
    else if (bits <= 8)
        return 4;
    else
        return 1;

}

//==============================================================================
//  CLASS MultiLoggerManager::MLM_Data - Single channel MLM class
//==============================================================================
//------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::Cleanup() --  Get rid of logger, graph
//------------------------------------------------------------------------

void  MultiLoggerManager::MLM_Data::Cleanup()
{
    //  Delete the loggers & graph
    delete Logger;   Logger = 0;
    delete Graph;    Graph = 0;
    delete BddFile;  BddFile = 0;
}

//------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::Init() --  Initialize the manager
//------------------------------------------------------------------------

void  MultiLoggerManager::MLM_Data::Init(const std::string & root, unsigned int i)
{
    //  Make the loggers & graph
    Logger = new Innovative::DataLogger;
    Graph = new Innovative::BinView;
    BddFile = new Innovative::BddInitObj;

    std::stringstream fn;
    fn << root << "Ch" << i ;

    IFN.Base(fn.str());
    IFN.Tag("_Test");
    IFN.Ext(".bin");

    std::stringstream ss;
    ss << "MLM_Ch" << i ;
    Graph->System().ServerSlotName(ss.str());

    Id = i;
}

//------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::AtStartConfig() --  Standard At Start
//------------------------------------------------------------------------

void  MultiLoggerManager::MLM_Data::AtStartConfig()
{
    //std::string str0 = IFN.FileName();
    Logger->FileName(IFN.FileName());
    Graph->BinFile(IFN.FileName());

    // Load current settings as a starting point
    Graph->Restore();

    BddFile->BinFile(Graph->BinFile());
    BddFile->OverWriteBdd(OverwriteBdd);
    WordCount = 0;
}

//------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::AtStopConfig() --  Standard At Stop
//------------------------------------------------------------------------

void MultiLoggerManager::MLM_Data::AtStopConfig()
{
    if (UniqueFileName)
        IFN.Increment();
    //std::string str = IFN.FileName();
}

//------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::Log() -- Log data to appropriate logger
//------------------------------------------------------------------------

void  MultiLoggerManager::MLM_Data::Log(Innovative::PmcBuffer & buffer)
{
    if (LoggerEnable || PlotEnable)
        if ((WordsToLog==0) || (WordCount < WordsToLog))  // log forever if 0
            {
            Logger->LogWithHeader(buffer);
            WordCount += buffer.SizeInInts();
            }

}
//------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::Log() -- Log data to appropriate logger
//------------------------------------------------------------------------

void MultiLoggerManager::MLM_Data::Log(McBuffer& buffer)
{
    if (LoggerEnable || PlotEnable)
        if ((WordsToLog==0) || (WordCount < WordsToLog))  // log forever if 0
            {
            Logger->Log(buffer);
            WordCount += buffer.SizeInInts();
            }
}

//------------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::Log() -- Log data to appropriate logger
//------------------------------------------------------------------------------

void MultiLoggerManager::MLM_Data::Log(McImage& image)
{
    //  Log Image Data in the appropriate file
    if (LoggerEnable || PlotEnable)
        Logger->Log(image.DatagramBasePtr(), image.DatagramSize());
        if ((WordsToLog==0) || (WordCount < WordsToLog))  // log forever if 0
            {
            WordCount += image.SizeInInts();
            }
}

//-------------------------------------------------------------------------------
//  MultiLoggerManager::MLM_Data::LaunchGraph() -- Create BDD, Launch Graph
//-------------------------------------------------------------------------------

void  MultiLoggerManager::MLM_Data::LaunchGraph()
{
    //  Save changes to BDD file
    BddFile->InitBddFile(*Graph);

    if (Logger->Logged() && PlotEnable)
        Graph->Plot();
}



#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

