//  MultiLoggerManager_App.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef MultiLoggerManager_AppH
#define MultiLoggerManager_AppH

#include <BinView_Mb.h>
#include <DataLogger_Mb.h>
#include <BddFile_Mb.h>

//---------------------------------------------------------------------------

namespace Innovative
{

//==============================================================================
//  CLASS MultiLoggerManager - Manages setup of multiple logger and viewers
//==============================================================================

class MultiLoggerManager
{
public:
    MultiLoggerManager();
    ~MultiLoggerManager();

    //  Config
    void  Root(std::string root_path)
              {  FRoot = root_path;  }
    void  Init(unsigned int loggers_to_make);

    bool    IsDataLoggingCompleted();

    size_t  Count() {  return Data.size();  }
    Innovative::DataLogger & Logger(unsigned int idx);
    Innovative::BinView &    Graph(unsigned int idx);
    Innovative::BddInitObj & BddFile(unsigned int idx);
    Innovative::IncrementalFileName & FileNamer(unsigned int idx);

    void  Id(unsigned int idx, unsigned int id_val);

    void  StartAll();
    void  StopAll();
    void  QuitAll();

    void  Log(Innovative::PmcBuffer & buffer);
    void  Log(Innovative::McBuffer & buffer);
    void  Log(Innovative::McImage & image);

    void  LoggerEnable(size_t idx, bool enable);
    void  PlotEnable(size_t idx, bool enable);
    void  OverwriteBdd(size_t idx, bool overwrite);
    void  WordsToLog(size_t idx, ii64 words);
    void  SamplesToLog(size_t idx, ii64 samples);  // requires BDD file "bits" to be loaded first
    void  UniqueFileName(size_t idx, bool uniquefilename);
    //------------------------------------------------------------------------------
    //  MultiLoggerManager::LoggerEnable() -- Set Logger enable flags
    //------------------------------------------------------------------------------

    template <class T>
    void LoggerEnable(const std::vector<T>& enable)
    {
        size_t count = std::min(Count(), enable.size());
        for (size_t i=0; i<count; i++)
            {
            LoggerEnable(i, static_cast<bool>(enable[i]));
            }
    }

    //------------------------------------------------------------------------------
    //  MultiLoggerManager::PlotEnable() -- Set Plot enable flags
    //------------------------------------------------------------------------------

    template <class T>
    void  PlotEnable(const std::vector<T> & enable)
    {
        size_t count = std::min(Count(), enable.size());
        for (size_t i=0; i<count; i++)
            {
            PlotEnable(i, static_cast<bool>(enable[i]));
            }
    }

    //------------------------------------------------------------------------------
    //  MultiLoggerManager::OverwriteBdd() -- Set Plot enable flags
    //------------------------------------------------------------------------------

    template <class T>
    void  OverwriteBdd(const std::vector<T> & overwrite)
    {
        size_t count = std::min(Count(), overwrite.size());
        for (size_t i=0; i<count; i++)
            {
            OverwriteBdd(i, static_cast<bool>(overwrite[i]));
            }
    }


    //------------------------------------------------------------------------------
    //  MultiLoggerManager::UniqueName() -- Set Unique name flags
    //------------------------------------------------------------------------------

    template <class T>
    void  UniqueFileName(const std::vector<T> & uniquefilename)
    {
        size_t count = std::min(Count(), uniquefilename.size());
        for (size_t i=0; i<count; i++)
            {
            UniqueFileName(i, static_cast<bool>(uniquefilename[i]));
            }
    }
    void  WordsToLog(const std::vector<ii64> & words);
    void  SamplesToLog(const std::vector<ii64> & samples);

private:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS MLM_Data -
    //
    class MLM_Data
    {
    public:
        Innovative::DataLogger *  Logger;
        Innovative::BinView *     Graph;
        Innovative::BddInitObj *  BddFile;
        IncrementalFileName       IFN;
        unsigned int              Id;
        bool                      PlotEnable;
        bool                      LoggerEnable;
        bool                      OverwriteBdd;
        bool                      UniqueFileName;
        ii64                      WordCount;
        ii64                      WordsToLog;

        MLM_Data()
            : Logger(0), Graph(0), BddFile(0),
              IFN("dummy", ".dum"), Id(0),
              PlotEnable(false), LoggerEnable(false),
              OverwriteBdd(false), UniqueFileName(false),
              WordCount(0), WordsToLog(0)
            {}

        void   Cleanup();
        void   Init(const std::string& root, unsigned int i);
        void   AtStartConfig();
        void   AtStopConfig();

        void   Log(PmcBuffer & buffer);
        void   Log(McBuffer & buffer);
        void   Log(McImage & image);

        void   LaunchGraph();
    };   

    std::vector<MLM_Data>  Data;
    std::string            FRoot;

    void   Cleanup();
    void   LaunchGraph(unsigned int idx);

    unsigned int FindIdxForBuffer(Innovative::PmcBuffer & buffer);
    unsigned int FindIdxForBuffer(Innovative::McBuffer & buffer);
    unsigned int FindIdxForBuffer(Innovative::McImage & image);
    unsigned int SamplesPerWord(unsigned int bits);
};

}  // namespace Innovative
//---------------------------------------------------------------------------
#endif
