//---------------------------------------------------------------------------
// Oscilloscope_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#include <Exception_Mb.h>
#include <Application/Oscilloscope_App.h>
#include <string>
#include <sstream>
#include <Trace_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <windows.h>

using namespace std;

namespace Innovative
{

//===========================================================================
//  Class OscilloscopeDllFunctions  --  Oscilloscope DLL access
//===========================================================================

class OscilloscopeDllFunctions
{
    friend class Oscilloscope;
    friend class Oscilloscope::TriggerControl;
    friend class Oscilloscope::AmpControl;
    friend class Oscilloscope::FormControl;
    friend class Oscilloscope::CellControl;

	//
	//  Member Functions
	OscilloscopeDllFunctions();
	~OscilloscopeDllFunctions();

    int     MaxInstances() const
        {  return FMaxInstances;  }

	//
	//  ...Function Pointer Typedefs
    typedef int (__cdecl * DLLPROC_1)(int Prm);
    typedef int (__cdecl * DLLPROC_2)(int Prm , const char  *P_IniName, const char *P_IniSuffix);
    typedef int (__cdecl * DLLPROC_3)(int Prm, double * ArrDbl);
    typedef double (__cdecl * DLLPROC_4)(int Prm);
    typedef int (__cdecl * DLLPROC_5)(int Prm, int value);
    typedef int (__cdecl * DLLPROC_6)(int Prm, int p1, int p2);
    typedef int (__cdecl * DLLPROC_7)(int Prm, double value);
    typedef int (__cdecl * DLLPROC_8)(int Prm, const char * value);
    typedef int (__cdecl * DLLPROC_9)(int Prm, int v1, double v2);
    typedef double (__cdecl * DLLPROC_10)(int Prm, int v);

	//
	//  Member Data
	HINSTANCE               Instance;
	bool                    Ok;
    int                     FMaxInstances;
    Trace                   Debug;

	DLLPROC_1  AtOpenLib;
	DLLPROC_2  Create;
	DLLPROC_1  Destroy;
	DLLPROC_1  Show;
	DLLPROC_1  Hide;
	DLLPROC_3  ShowNext;
	DLLPROC_1  QuickUpdate;
	DLLPROC_1  CleanBuffers;
	DLLPROC_3  ExternalNext;
	DLLPROC_1  GetFormLeft;
	DLLPROC_1  GetFormTop;
	DLLPROC_1  GetFormWidth;
	DLLPROC_1  GetFormHeight;
	DLLPROC_1  GetScreenWidth;
	DLLPROC_1  GetScreenHeight;
	DLLPROC_1  GetCellPixelSize;
	DLLPROC_5  SetCellPixelSize;
	DLLPROC_4  GetCellSampleSize;
	DLLPROC_7  SetCellSampleSize;
	DLLPROC_1  GetPanelState;
	DLLPROC_5  SetPanelState;
	DLLPROC_6  SetFormPosition;
	DLLPROC_6  SetFormSize;
	DLLPROC_8  SetCaption;
	DLLPROC_9  SetAmpScale;
	DLLPROC_10 GetAmpScale;
	DLLPROC_9  SetAmpOffset;
	DLLPROC_10 GetAmpOffset;
	DLLPROC_9  SetTriggerLevel;
	DLLPROC_10 GetTriggerLevel;
	DLLPROC_5  SetTriggerSource;
	DLLPROC_1  GetTriggerSource;
	DLLPROC_5  SetTriggerEdge;
	DLLPROC_1  GetTriggerEdge;
	DLLPROC_1  GetTrigHoldOffSmplsNum;
	DLLPROC_5  SetTrigHoldOffSmplsNum;
};

//---------------------------------------------------------------------------
//  constructor for class OscilloscopeDllFunctions
//---------------------------------------------------------------------------

OscilloscopeDllFunctions::OscilloscopeDllFunctions()
	: Instance(0)
	, Ok(true)
    , Debug(true)    
	, AtOpenLib(0)
	, Create(0)
	, Destroy(0)
	, Show(0)
	, Hide(0)
	, ShowNext(0)
	, QuickUpdate(0)
	, CleanBuffers(0)
	, ExternalNext(0)
	, GetFormLeft(0)
	, GetFormTop(0)
	, GetFormWidth(0)
	, GetFormHeight(0)
	, GetScreenWidth(0)
	, GetScreenHeight(0)
    , GetCellPixelSize(0)
    , GetCellSampleSize(0)
    , SetCellSampleSize(0)
	, GetPanelState(0)
	, SetPanelState(0)
    , SetFormPosition(0)
    , SetFormSize(0)
    , SetCaption(0) 
    , SetAmpScale(0)    
    , GetAmpScale(0)
    , SetAmpOffset(0)    
    , GetAmpOffset(0)
    , SetTriggerLevel(0)
    , GetTriggerLevel(0)
    , SetTriggerSource(0)
    , GetTriggerSource(0)
    , SetTriggerEdge(0)
    , GetTriggerEdge(0)
    , GetTrigHoldOffSmplsNum(0)
    , SetTrigHoldOffSmplsNum(0)    
{
    const char * lib = "Osc_DLL.dll";

	//
	//  Attempt to Open the DLL & get its functions
	Instance = LoadLibraryA(lib);
	if (Instance==NULL)
		{
		//
		//  Could not open DLL. Add error to event log
    	std::stringstream msg;
	    msg << "Could not open " << lib;
        Debug.Log(msg.str());

		Ok = false;
		return;
		}
	//
	//  Get pointers to functions
	AtOpenLib = (DLLPROC_1)GetProcAddress(Instance, "AtOpenLib");
	if (!AtOpenLib)
		Ok = false;

	ShowNext = (DLLPROC_3)GetProcAddress(Instance, "ShowNext");
	if (!ShowNext)
		Ok = false;

	Create = (DLLPROC_2)GetProcAddress(Instance, "ScopeCreate");
	if (!Create)
		Ok = false;

	Destroy = (DLLPROC_1)GetProcAddress(Instance, "ScopeDestroy");
	if (!Destroy)
		Ok = false;

	Show = (DLLPROC_1)GetProcAddress(Instance, "ScopeShow");
	if (!Show)
		Ok = false;

	Hide = (DLLPROC_1)GetProcAddress(Instance, "ScopeHide");
	if (!Hide)
		Ok = false;

	QuickUpdate = (DLLPROC_1)GetProcAddress(Instance, "QuickUpDate");
	if (!QuickUpdate)
		Ok = false;

	CleanBuffers = (DLLPROC_1)GetProcAddress(Instance, "ScopeCleanBuffers");
	if (!CleanBuffers)
		Ok = false;

	ExternalNext = (DLLPROC_3)GetProcAddress(Instance, "ExternalNext");
	if (!ExternalNext)
		Ok = false;

	GetFormLeft = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetFormLeft");
	if (!GetFormLeft)
		Ok = false;

	GetFormTop = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetFormTop");
	if (!GetFormTop)
		Ok = false;

	GetFormWidth = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetFormWidth");
	if (!GetFormWidth)
		Ok = false;

	GetFormHeight = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetFormHeight");
	if (!GetFormHeight)
		Ok = false;

	GetScreenWidth = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetScreenWidth");
	if (!GetScreenWidth)
		Ok = false;

	GetScreenHeight = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetScreenHeight");
	if (!GetScreenHeight)
		Ok = false;

	GetCellPixelSize = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetCellPixelSize");
	if (!GetCellPixelSize)
		Ok = false;

	SetCellPixelSize = (DLLPROC_5)GetProcAddress(Instance, "ScopeSetCellPixelSize");
	if (!SetCellPixelSize)
		Ok = false;

	GetCellSampleSize = (DLLPROC_4)GetProcAddress(Instance, "ScopeGetCellSampleSize");
	if (!GetCellSampleSize)
		Ok = false;

	SetCellSampleSize = (DLLPROC_7)GetProcAddress(Instance, "ScopeSetCellSampleSize");
	if (!SetCellSampleSize)
		Ok = false;

	GetPanelState = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetPanelState");
	if (!GetPanelState)
		Ok = false;

	SetPanelState = (DLLPROC_5)GetProcAddress(Instance, "ScopeSetPanelState");
	if (!SetPanelState)
		Ok = false;

	SetFormPosition = (DLLPROC_6)GetProcAddress(Instance, "ScopeSetFormPos");
	if (!SetFormPosition)
		Ok = false;

	SetFormSize = (DLLPROC_6)GetProcAddress(Instance, "ScopeSetFormSize");
	if (!SetFormSize)
		Ok = false;

	SetCaption = (DLLPROC_8)GetProcAddress(Instance, "ScopeSetCaption");
	if (!SetCaption)
		Ok = false;

	SetAmpScale = (DLLPROC_9)GetProcAddress(Instance, "ScopeSetAmpScale");
	if (!SetAmpScale)
		Ok = false;

	GetAmpScale = (DLLPROC_10)GetProcAddress(Instance, "ScopeGetAmpScale");
	if (!GetAmpScale)
		Ok = false;

	SetAmpOffset = (DLLPROC_9)GetProcAddress(Instance, "ScopeSetAmpOffset");
	if (!SetAmpOffset)
		Ok = false;

	GetAmpOffset = (DLLPROC_10)GetProcAddress(Instance, "ScopeGetAmpOffset");
	if (!GetAmpOffset)
		Ok = false;

	SetTriggerLevel = (DLLPROC_9)GetProcAddress(Instance, "ScopeSetTriggerLevel");
	if (!SetTriggerLevel)
		Ok = false;

	GetTriggerLevel = (DLLPROC_10)GetProcAddress(Instance, "ScopeGetTriggerLevel");
	if (!GetTriggerLevel)
		Ok = false;

	SetTriggerSource = (DLLPROC_5)GetProcAddress(Instance, "ScopeSetTriggerSourse");
	if (!SetTriggerSource)
		Ok = false;

	GetTriggerSource = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetTriggerSourse");
	if (!GetTriggerSource)
		Ok = false;

	SetTriggerEdge = (DLLPROC_5)GetProcAddress(Instance, "ScopeSetActiveTriggerEdge");
	if (!SetTriggerEdge)
		Ok = false;

	GetTriggerEdge = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetActiveTriggerEdge");
	if (!GetTriggerEdge)
		Ok = false;

	GetTrigHoldOffSmplsNum = (DLLPROC_1)GetProcAddress(Instance, "ScopeGetTrigHoldOffSmplsNum");
	if (!GetTrigHoldOffSmplsNum)
		Ok = false;

	SetTrigHoldOffSmplsNum = (DLLPROC_5)GetProcAddress(Instance, "ScopeSetTrigHoldOffSmplsNum");
	if (!SetTrigHoldOffSmplsNum)
		Ok = false;

	std::stringstream msg;
	msg << lib << " open status: " << (Ok ? "ok" : "failed");
    Debug.Log(msg.str());

    FMaxInstances = AtOpenLib(0);
    msg.str("");
	msg << "Max instances: " << FMaxInstances;
}

//---------------------------------------------------------------------------
//  destructor for class OscilloscopeDllFunctions
//---------------------------------------------------------------------------

OscilloscopeDllFunctions::~OscilloscopeDllFunctions()
{
	//
	//  Free the DLL instance
	if (!Instance)
		return;

	FreeLibrary(Instance);
	Instance = 0;
}

//===========================================================================
//  CLASS Oscilloscope  --
//===========================================================================

OscilloscopeDllFunctions * Oscilloscope::Dll = 0;
unsigned int Oscilloscope::Instances = 0;

//---------------------------------------------------------------------------
// Oscilloscope::Oscilloscope() --  Ctor
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

Oscilloscope::Oscilloscope()
    : Trigger(*this), Amp(*this), Form(*this), Cell(*this),
      Info(new UnpairedChannelInfo(2, 16, 0, false)), FMaxBeams(3)
{
    Create("", "");
}

//---------------------------------------------------------------------------
// Oscilloscope::Oscilloscope() --  Ctor
//---------------------------------------------------------------------------

Oscilloscope::Oscilloscope(const string & file)
    : Trigger(*this), Amp(*this), Form(*this), Cell(*this),
      Info(new UnpairedChannelInfo(2, 16, 0, false)), FMaxBeams(3)
{
    Create(file, "");
}

//---------------------------------------------------------------------------
// Oscilloscope::Oscilloscope() --  Ctor
//---------------------------------------------------------------------------

Oscilloscope::Oscilloscope(const string & file, const string & section)
    : Trigger(*this), Amp(*this), Form(*this), Cell(*this),
      Info(new UnpairedChannelInfo(2, 16, 0, false)), FMaxBeams(3)
{
    Create(file, section);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  Oscilloscope::Create() --  Load DLL, if not already in memory
//---------------------------------------------------------------------------

void Oscilloscope::Create(const string & file, const string & section)
{
    if (!Instances++)
        Dll = new OscilloscopeDllFunctions;

    if (Dll->Ok==false)
        throw MalibuException("Unable to open Oscilloscope DLL");

    Handle = Dll->Create(0, file.c_str(), section.c_str());
    if ((Handle <= 0) || (Handle >= Dll->FMaxInstances))
        throw MalibuException("Unable to create Oscilloscope instance");
}

//---------------------------------------------------------------------------
// Oscilloscope::~Oscilloscope() --  Dtor
//---------------------------------------------------------------------------

Oscilloscope::~Oscilloscope()
{
    Form.Hide();
    
    delete Info;
    Info = 0;

    if (Instances)
        {
        --Instances;
        return;
        }

    Dll->Destroy(Handle);

//	if (Dll->Destroy(Handle) != Handle)
//		volatile int x = 0;	//  throw MalibuException("Unable to destroy Oscilloscope instance");

	delete Dll;
	Dll = 0;
}

//---------------------------------------------------------------------------
//  Oscilloscope::Format() --  Specify data buffer format
//---------------------------------------------------------------------------

void Oscilloscope::Format(int channels, int bits, bool unipolar)
{
    delete Info;
    Info = new UnpairedChannelInfo(channels, bits, 0, unipolar);

    for (int ch = 0; ch < channels; ++ch)
        Info->Enabled(ch, true);
}

//---------------------------------------------------------------------------
//  Oscilloscope::Enable() --  Get draw state of specified beam
//---------------------------------------------------------------------------

bool Oscilloscope::Enable(int ch) const
{
    return Info->Enabled(ch);
}

//---------------------------------------------------------------------------
//  Oscilloscope::Format() --  Set draw state of specified beam
//---------------------------------------------------------------------------

void Oscilloscope::Enable(int ch, bool state)
{
    Info->Enabled(ch, state);
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Show() --  Display scope form
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::Show()
{
    return Dll().Show(Handle()) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Hide() --  Hode scope form
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::Hide()
{
    return Dll().Hide(Handle()) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::Plot() --  Plot single point, all beams
//---------------------------------------------------------------------------

bool Oscilloscope::Plot(double * data, bool external)
{
    if (external)
        return Dll->ExternalNext(Handle, data) == Handle;
    else
        return Dll->ShowNext(Handle, data) == Handle;
}

//---------------------------------------------------------------------------
//  Oscilloscope::Plot() --  Plot buffer of points, all beams, specified
//                           number of samples
//---------------------------------------------------------------------------

void Oscilloscope::Plot(Buffer & buffer, int samples)
{
    int bits = Info->Bits();
	samples = std::min<int>(samples, static_cast<int>(buffer.SizeInBytes()/(Info->BytesPerSample()*Info->Channels())));

    // Plot waveform elements
    if (bits <=8)
        {
        if (Info->Unipolar())
            plot<unsigned char>(buffer, samples);
        else
            plot<char>(buffer, samples);
        }
    else if (bits <= 16)
        {
        if (Info->Unipolar())
            plot<unsigned short>(buffer, samples);
        else
            plot<short>(buffer, samples);
        }
    else
        {
        if (Info->Unipolar())
            plot<unsigned int>(buffer, samples);
        else
            plot<int>(buffer, samples);
        }
}

//---------------------------------------------------------------------------
//  Oscilloscope::Plot() --  Plot buffer of points, all beams, all samples
//---------------------------------------------------------------------------

void Oscilloscope::Plot(Buffer & buffer)
{
    int samples = static_cast<int>(buffer.SizeInBytes()/(Info->BytesPerSample()*Info->Channels()));

    Plot(buffer, samples);
}

//---------------------------------------------------------------------------
//  Oscilloscope::Update() --  Refresh the form, rarely needed
//---------------------------------------------------------------------------

bool Oscilloscope::Update()
{
    return Dll->QuickUpdate(Handle) == Handle;
}

//---------------------------------------------------------------------------
//  Oscilloscope::CleanBuffers() --
//---------------------------------------------------------------------------

bool Oscilloscope::CleanBuffers()
{
    return Dll->CleanBuffers(Handle) == Handle;
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Left() --  Return leftmost form position
//---------------------------------------------------------------------------

int Oscilloscope::FormControl::Left() const
{
    return Dll().GetFormLeft(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Top() --  Return topmost form position
//---------------------------------------------------------------------------

int Oscilloscope::FormControl::Top() const
{
    return Dll().GetFormTop(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Width() --  Return form width
//---------------------------------------------------------------------------

int Oscilloscope::FormControl::Width() const
{
    return Dll().GetFormWidth(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Height() --  Return height of form
//---------------------------------------------------------------------------

int Oscilloscope::FormControl::Height() const
{
    return Dll().GetFormHeight(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Position() --  Set top/left corner of form
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::Position(int left, int top)
{
    return Dll().SetFormPosition(Handle(), left, top) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Size() --  Set height/width of form
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::Size(int width, int height)
{
    return Dll().SetFormSize(Handle(), width, height) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::ScreenWidth() --  Return width of plot area
//---------------------------------------------------------------------------

int Oscilloscope::FormControl::ScreenWidth() const
{
    return Dll().GetScreenWidth(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::ScreenHeight() --  Return height of plot area
//---------------------------------------------------------------------------

int Oscilloscope::FormControl::ScreenHeight() const
{
    return Dll().GetScreenHeight(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::CellControl::PixelSize() --  Return size of pixel
//---------------------------------------------------------------------------

int Oscilloscope::CellControl::PixelSize() const
{
    return Dll().GetCellPixelSize(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::CellControl::PixelSize() --  Set size of pixel
//---------------------------------------------------------------------------

bool Oscilloscope::CellControl::PixelSize(int size)
{
    return Dll().SetCellPixelSize(Handle(), size) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::CellControl::SampleSize() --  Get size of sample
//---------------------------------------------------------------------------

double Oscilloscope::CellControl::SampleSize() const
{
    return Dll().GetCellSampleSize(Handle());
}

//---------------------------------------------------------------------------
//  Oscilloscope::CellControl::SampleSize() --  Set size of sample
//---------------------------------------------------------------------------

bool Oscilloscope::CellControl::SampleSize(double value)
{
    return Dll().SetCellSampleSize(Handle(), value) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::PanelVisible() --  Control panel visibility
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::PanelVisible() const
{
    return Dll().GetPanelState(Handle()) == 2;
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::PanelVisible() --  Query panel visibility
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::PanelVisible(bool state)
{
    return Dll().SetPanelState(Handle(), state ? 1 : 0) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::FormControl::Caption() --  Set form caption
//---------------------------------------------------------------------------

bool Oscilloscope::FormControl::Caption(const std::string & value)
{
    return Dll().SetCaption(Handle(), value.c_str()) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::Scale() --  Set amp scale for specified beam
//---------------------------------------------------------------------------

bool Oscilloscope::AmpControl::Scale(int beam, double value)
{
    return Dll().SetAmpScale(Handle(), beam, value) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::Scale() --  Get amp scale for specified beam
//---------------------------------------------------------------------------

double Oscilloscope::AmpControl::Scale(int beam) const
{
    return Dll().GetAmpScale(Handle(), beam);
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::Offset() --  Set amp offset for specified beam
//---------------------------------------------------------------------------

bool Oscilloscope::AmpControl::Offset(int beam, double value)
{
    return Dll().SetAmpOffset(Handle(), beam, value) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::Offset() --  Get amp offset for specified beam
//---------------------------------------------------------------------------

double Oscilloscope::AmpControl::Offset(int beam) const
{
    return Dll().GetAmpOffset(Handle(), beam);
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::AutoScale() --  Automatically scale per format
//---------------------------------------------------------------------------

void Oscilloscope::AmpControl::AutoScale()
{
    int amplitude = (1 << Owner.Info->Bits())/4;
    int offset = Owner.Info->Unipolar() ? 0 : -amplitude/2;

    for (int beam = 0; beam < Owner.MaxBeams(); ++beam)
        Scale(beam, (amplitude+offset)*FGain[beam]);
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::Gain() --  Set scale factor for specified beam
//---------------------------------------------------------------------------

double Oscilloscope::AmpControl::Gain(int beam) const
{
    return FGain[beam];
}

//---------------------------------------------------------------------------
//  Oscilloscope::AmpControl::Gain() --  Get scale factor for specified beam
//---------------------------------------------------------------------------

void Oscilloscope::AmpControl::Gain(int beam, double value)
{
    FGain[beam] = value;
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::Level() --  Set trigger level, specified beam
//---------------------------------------------------------------------------

bool Oscilloscope::TriggerControl::Level(int beam, double value)
{
    return Dll().SetTriggerLevel(Handle(), beam, value) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::Level() --  Get trigger level, specified beam
//---------------------------------------------------------------------------

double Oscilloscope::TriggerControl::Level(int beam) const
{
    return Dll().GetTriggerLevel(Handle(), beam);
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::Source() --  Set trigger source
//---------------------------------------------------------------------------

bool Oscilloscope::TriggerControl::Source(IISource src)
{
    return Dll().SetTriggerSource(Handle(), src) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerSource() --  Get trigger source
//---------------------------------------------------------------------------

Oscilloscope::TriggerControl::IISource Oscilloscope::TriggerControl::Source() const
{
    return static_cast<TriggerControl::IISource>(Dll().GetTriggerSource(Handle()));
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::Edge() --  Set trigger edge polarity
//---------------------------------------------------------------------------

bool Oscilloscope::TriggerControl::Edge(IIEdge type)
{
    int value;
    switch (type)
        {
        default:
        case teNone:        value = 0;      break;
        case teRising:      value = 1;      break;
        case teFalling:     value = -1;     break;
        }

    return Dll().SetTriggerSource(Handle(), value) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::TriggerEdge() --  Get trigger edge polarity
//---------------------------------------------------------------------------

Oscilloscope::TriggerControl::IIEdge Oscilloscope::TriggerControl::Edge() const
{
    int value = Dll().GetTriggerEdge(Owner.Handle);
    if (value < 0)
        return teFalling;
    else if (value > 0)
        return teRising;
    else
        return teNone;
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::Holdoff() --  Set trigger holdoff
//---------------------------------------------------------------------------

bool Oscilloscope::TriggerControl::Holdoff(int samples)
{
    return Dll().SetTrigHoldOffSmplsNum(Handle(), samples) == Handle();
}

//---------------------------------------------------------------------------
//  Oscilloscope::TriggerControl::TriggerHoldoff() --  Get trigger holdoff
//---------------------------------------------------------------------------

int Oscilloscope::TriggerControl::Holdoff() const
{
    return Dll().GetTrigHoldOffSmplsNum(Owner.Handle);
}

//---------------------------------------------------------------------------
//  Oscilloscope::MaxInstances() --  Return max instances of scope
//---------------------------------------------------------------------------

int Oscilloscope::MaxInstances() const
{
    return Dll->FMaxInstances;
}

//---------------------------------------------------------------------------
//  Oscilloscope::Instance() --  Return current instance handle (count)
//---------------------------------------------------------------------------

int Oscilloscope::Instance() const
{
    return Handle;
}

//---------------------------------------------------------------------------
//  Oscilloscope::MaxBeams() --  Return max number of beams per plot
//---------------------------------------------------------------------------

int Oscilloscope::MaxBeams() const
{
    return FMaxBeams;
}

} // namespace


