// ScriptPlayer_App.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Application/ScriptPlayer_App.h>
#include <SystemSupport_Mb.h>
#include <cstring>

using namespace Innovative;
using namespace std;

namespace Innovative
{

//==============================================================================
//  CLASS  Script
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class Script -- ctor
//------------------------------------------------------------------------------

ScriptPlayer::ScriptPlayer()
{
    Script.OnCommand.SetEvent(this, &ScriptPlayer::HandleScriptCommand);
    Script.OnMessage.SetEvent(this, &ScriptPlayer::HandleScriptMessage);
}

//------------------------------------------------------------------------------
//  destructor for class Script
//------------------------------------------------------------------------------

ScriptPlayer::~ScriptPlayer()
{
    Script.OnCommand.ClearEvent();
    Script.OnMessage.ClearEvent();
}

//---------------------------------------------------------------------------
//  ScriptPlayer::Execute() --  Log script message
//---------------------------------------------------------------------------

void ScriptPlayer::Execute(const std::string & file)
{
    Script.Execute(file);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::HandleScriptMessage() --  Log script message
//---------------------------------------------------------------------------

void ScriptPlayer::HandleScriptMessage(ProcessStatusEvent & Event)
{
    Log(Event.Message);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::HandleScriptCommand() --  Dispatch script command
//---------------------------------------------------------------------------

void ScriptPlayer::HandleScriptCommand(Innovative::ProcessStatusEvent & Event)
{
    const char * cmd = Event.Message.c_str();

    // Logic store
    if (strcmp(cmd, "l!") == 0)
        DoLogicStore();
    // Logic fetch
    if (strcmp(cmd, "l@") == 0)
        DoLogicFetch();
    // Port store
    if (strcmp(cmd, "p!") == 0)
        DoPortStore();
    // Port fetch
    if (strcmp(cmd, "p@") == 0)
        DoPortFetch();
    // Dwell
    if (strcmp(cmd, "ms") == 0)
        DoDelay();
}

//---------------------------------------------------------------------------
//  ScriptPlayer::DoLogicStore() --  Store to logic register
//---------------------------------------------------------------------------

void ScriptPlayer::DoLogicStore()
{
    int addr = Script.Pop();
    int value = Script.Pop();
    Logic(addr, value);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::DoLogicFetch() --  Fetch from logic register
//---------------------------------------------------------------------------

void ScriptPlayer::DoLogicFetch()
{
    int addr = Script.Pop();
    Logic(addr);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::DoPortStore() --  Store to PCI register
//---------------------------------------------------------------------------

void ScriptPlayer::DoPortStore()
{
	int addr = Script.Pop();
	int value = Script.Pop();
	Port(addr, value);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::DoPortFetch() --  Fetch from PCI register
//---------------------------------------------------------------------------

void ScriptPlayer::DoPortFetch()
{
    int addr = Script.Pop();
    Port(addr);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::DoDelay() --  Spin for a while
//---------------------------------------------------------------------------

void ScriptPlayer::DoDelay()
{
    int value = Script.Pop();
    Innovative::Sleep(value);

    std::stringstream msg;
    msg << " Sleep " << value << " mS";
    Log(std::string(msg.str()));
}

//---------------------------------------------------------------------------
//  ScriptPlayer::Port() --  Read a register in Port Momory, Log results
//---------------------------------------------------------------------------

ii32 ScriptPlayer::Port(int address)
{
    int value = GetPort(address);

    std::stringstream msg;
    msg << std::hex << "0x" << address << " p@=" << "0x" << value;
    Log(std::string(msg.str()));

    return value;
}

//---------------------------------------------------------------------------
//  ScriptPlayer::Port() --  Write a register in Port Momory, Log results
//---------------------------------------------------------------------------

void ScriptPlayer::Port(int address, ii32 data)
{
    std::stringstream msg;
    msg << std::hex << "0x" << data << " 0x" << address << " p!";
    Log(std::string(msg.str()));

    SetPort(address, data);
}

//---------------------------------------------------------------------------
//  ScriptPlayer::Logic() --  Read a register in Logic memory
//---------------------------------------------------------------------------

ii32 ScriptPlayer::Logic(int address)
{
    int value = GetLogic(address);

    std::stringstream msg;
    msg << std::hex << "0x" << address << " l@=" << "0x" << value;
    Log(std::string(msg.str()));

    return value;
}

//---------------------------------------------------------------------------
//  ScriptPlayer::Logic() --  Write a register in Logic Memory
//---------------------------------------------------------------------------

void ScriptPlayer::Logic(int address, ii32 data)
{
    std::stringstream msg;
    msg << std::hex << "0x" << data << " 0x" << address << " l!";
    Log(std::string(msg.str()));

    SetLogic(address, data);
}



} // namespace

