//---------------------------------------------------------------------------



#include "TimeTraceLog_App.h"
#include <StopWatch_Mb.h>
#include <Mutex_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{
//---------------------------------------------------------------------------
//  housekeeping  for Delta T Logging
//---------------------------------------------------------------------------

const int TotalLogs = 5;

std::vector<StopWatch *> * TimerStore = 0;
std::vector<TimeLogData> * TimeLog = 0;
Mutex *  TimeMutex = 0;

//---------------------------------------------------------------------------
//  Init_DeltaT_Log() -- Initialize Delta T Log
//---------------------------------------------------------------------------

void  Init_DeltaT_Log()
{
    if (TimerStore==0)
        {
        TimerStore = new std::vector<StopWatch *>(TotalLogs);
        for (unsigned int i=0; i<TimerStore->size(); ++i)
            {
            (*TimerStore)[i] = new StopWatch;
            (*TimerStore)[i]->Start();
            }
        }
    if (TimeLog==0)
        {
        TimeLog = new std::vector<TimeLogData>;
        }
    if (TimeMutex==0)
        {
        TimeMutex = new Mutex;
        }

}

//---------------------------------------------------------------------------
//  Clear_DeltaT_Log() -- Clear delta T Log
//---------------------------------------------------------------------------

void  Clear_DeltaT_Log()
{
    TimeLog->clear();   // clear log
}

//---------------------------------------------------------------------------
//  DeltaT_Log() -- Add entry to a Delta T Log
//---------------------------------------------------------------------------

void  DeltaT_Log(int which, int type, int arg)
{
    Init_DeltaT_Log();
    MutexedLock Lock(*TimeMutex);
    if ((unsigned int)which >= TimerStore->size())
        return;

    double Delta = (*TimerStore)[which]->Stop();
    (*TimerStore)[which]->Start();
    TimeLogData item(type, arg, Delta );
    TimeLog->push_back(item);

}

//---------------------------------------------------------------------------
//  DeltaTLog_Vec() -- Return vector pointer
//---------------------------------------------------------------------------

std::vector<TimeLogData> *  DeltaTLog_Vec()
{
    return TimeLog;   // return log
}

}

