//  XU_OutputManager_App.h
//
//  Copyright 2016 Innovative Integration
//---------------------------------------------------------------------------

#ifndef XU_OutputManager_AppH
#define XU_OutputManager_AppH

#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>
#include <DataPlayer_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS XU_OutputManager -- Coordinate Output for XU output streams
//===========================================================================

class XU_OutputManager
{
public:
    enum PlayModes { pmContinuousPlay, pmOneBlock };

    XU_OutputManager(unsigned int channels);
    ~XU_OutputManager()  {  }

    void SetMode(PlayModes newMode) {  Mode=newMode;  }

    void Start();
    void Stop();

    void Play(unsigned int ch, McImage& Image);
    void Play(unsigned int ch, McBuffer& Buffer);

    bool PlayerOpened(unsigned int ch);
    bool AllPlayersOpened();
    DataPlayer &  Player(unsigned int ch)  {  return PerCh[ch].Player;  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_OutputManager::PerChannelArgs  -- Args for each XU Channel
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct PerChannelArgs
    {
        std::string     SourceFilename;
        unsigned int    CacheSizeInts;
    };

    PerChannelArgs &  Settings(unsigned int ch)  {  return PerArg[ch];  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_OutputManager::PerChannelInfo  -- Data for each XU Channel
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct PerChannelInfo
    {
        unsigned int    ChannelID;
        DataPlayer      Player;
        bool            PlayerOpened;
        McBuffer        Cache;
        size_t          CacheOffset;

        PerChannelInfo()
            :  ChannelID(0), PlayerOpened(false), CacheOffset(0)
        {}

    };

    void AllocateCaches();
    void ReleaseCaches();

private:

    PlayModes  Mode;
    std::vector<PerChannelArgs> PerArg;
    std::vector<PerChannelInfo> PerCh;

    PerChannelInfo &  Info(unsigned int ch)  {  return PerCh[ch];  }

    void  TruncateImage(McImage & Image, size_t size_ints);

};

}  // namespace Innovative


#endif // XU_OutputManager_AppH
