// AtroposDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <AtroposDeviceMap_Mb.h>
#include <AtroposMemoryMap_Mb.h>
#include <AtroposDeviceDriver_Mb.h>
#include <SystemSupport_Mb.h>


#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
	#pragma warning(disable : 4616)
    #pragma warning(disable : 8004)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

//---------------------------------------------------------
//  Atropos Constants Definition

//=============================================================================
//  CLASS Atropos_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Atropos_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class Atropos_Pll
//---------------------------------------------------------------------------

Atropos_Pll::Atropos_Pll(AtroposDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg, omNestedZeroDelay, tsmLogic), Owner(owner), At_Init(false)
{
    OutputSyncMode(osmSysRefSync);
    SetMasterOutputSelection(AdcMasterOutput());  // Use AdcMaster as guideline output
    Allow_Pll1_FeedbackAdjustment();
    AutoReapply(false);

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class Atropos_Pll
//---------------------------------------------------------------------------

Atropos_Pll::~Atropos_Pll()
{
}

//---------------------------------------------------------------------------
//  Atropos_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  Atropos_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::OnInit()");
    //
    //  Input Configuration
    SetInputConfig(0, icCmos);
    SetInputConfig(1, icBipolar);
    SetInputConfig(2, icBipolar);

    //  Device Powered/Enabled Setup
    //PowerDownOutputGroup(12/2, true);

    //  Analog Delay Config
    PowerDownOutputAnalogDelay(10, false);   // not powered down
    PowerDownOutputGlitchlessAnalogDelay(10, false);
    PowerDownOutputHalfStep(10, false);
    SetAnalogDelayMode(10, true);   // mess with duty cycle
    //SetAnalogDelayMode(10, false);   // don't mess with duty cycle
    SetAnalogDelay(10, 0);           // initial delay of 0
    //
    //  Set Output Configuration                                   //   REV A            REV B+
    OutputInUse(0,  true);    SetOutputConfig(0,  ocLvPecl_1600);  //  FP Clk 0 (J3)  | FP Clk 1 (J4)
    OutputInUse(1,  false);   SetOutputConfig(1,  ocLvPecl_1600);  //  FP Trg 0 (J7)  | FP Trg 1 (J8)
    OutputInUse(2,  true);    SetOutputConfig(2,  ocLvPecl_1600);  //  FP Clk 1 (J4)  | FP Clk 2 (J5)
    OutputInUse(3,  false);   SetOutputConfig(3,  ocLvPecl_1600);  //  FP Trg 1 (J8)  | FP Trg 2 (J9)
    OutputInUse(4,  true);    SetOutputConfig(4,  ocLvPecl_1600);  //  FP Clk 2 (J5)  | FP Clk 3 (J6)
    OutputInUse(5,  false);   SetOutputConfig(5,  ocLvPecl_1600);  //  FP Trg 2 (J9)  | FP Trg 3 (J10)
    OutputInUse(6,  true);    SetOutputConfig(6,  ocLvPecl_1600);  //  FP Clk 3 (J6)  | <feedback>
    OutputInUse(7,  false);   SetOutputConfig(7,  ocLvPecl_1600);  //  FP Trg 3 (J10) | (unused)
    OutputInUse(8,  true);    SetOutputConfig(8,  ocLvPecl_1600);  //  S2P_CLK        | S2P_CLK
    OutputInUse(9,  false);   SetOutputConfig(9,  ocDisabled);     //  (unused)       | (unused)
    OutputInUse(10, true);    SetOutputConfig(10, ocLvPecl_1600);  //  P2S_CLK        | P2S_CLK
    OutputInUse(11, true);    SetOutputConfig(11, ocLvPecl_1600);  //  TRIG_LPBK      | TRIG_LPBK (into S2P)
    OutputInUse(12, true);    SetOutputConfig(12, ocLvPecl_1600);  //  FPGA_CLK       | FP Clk 0 (J3)
    OutputInUse(13, false);   SetOutputConfig(13, ocLvPecl_1600);  //  FPGA_CLK2      | FP Trg 0 (J7)

    FeedbackOutput(fbm_Dclk6);  // set output to use as PLL feedback in 0-delay modes

    //  Set Reduced Frequency Limits for Outputs (Preset to 3200.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[0] = 1000.0e6;
    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refCLKin2);    // Set to use internal reference

    EnableAllOutputs(false);         // don't apply now
    EnableAllTriggerOutputs(false);  // don't apply now

    At_Init = true;
    Logic_SerDes_Disable();
}

//---------------------------------------------------------------------------
//  Atropos_Pll::OnReapply()
//---------------------------------------------------------------------------

void  Atropos_Pll::OnReapply()
{
    EnableAllTriggerOutputs(false); // don't apply now (?)

    Logic_SerDes_Disable();
}

//---------------------------------------------------------------------------
//  Atropos_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  Atropos_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  Atropos_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  Atropos_Pll::DacMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  Atropos_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool Atropos_Pll::IsFeedbackOutputAdc()
{
    return true;
}

//---------------------------------------------------------------------------
//  Atropos_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  Atropos_Pll::SetAdcOutputRates(double adc_rate)
{
    unsigned int sysref_1_11_div = 16; //(adc_rate<1450.0e6) ? 16 : 8 ;
    unsigned int sysref_8_10_div = (adc_rate<1450.0e6) ? 1 : 2 ;

    OutputFrequency(0,   adc_rate);
    //OutputFrequency(1,   adc_rate/sysref_1_11_div);
    OutputFrequency(2,   adc_rate);
    //OutputFrequency(3,   adc_rate);
    OutputFrequency(4,   adc_rate);
    //OutputFrequency(5,   adc_rate);
    OutputFrequency(6,   adc_rate);
    //OutputFrequency(7,   adc_rate);
    OutputFrequency(8,   adc_rate/sysref_8_10_div);
    OutputFrequency(10,   adc_rate/sysref_8_10_div);
    OutputFrequency(11,   adc_rate/sysref_1_11_div);
    OutputFrequency(12,   adc_rate);
    //OutputFrequency(13,   adc_rate);
}

//---------------------------------------------------------------------------
//  Atropos_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  Atropos_Pll::SetDacOutputRates(double /*dac_rate*/)
{
    // none
}

//---------------------------------------------------------------------------
//  Atropos_Pll::EnableAllOutputs() -- enable/disable external outputs
//---------------------------------------------------------------------------

void  Atropos_Pll::EnableAllOutputs(bool enabled, bool auto_apply)
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::EnableAllOutputs()");
    Debug.Log("enabled=", enabled);

    OutputEnabled_AutoApply(0, enabled, auto_apply);
    //OutputEnabled_AutoApply(1, enabled);
    OutputEnabled_AutoApply(2, enabled, auto_apply);
    //OutputEnabled_AutoApply(3, enabled);
    OutputEnabled_AutoApply(4, enabled, auto_apply);
    //OutputEnabled_AutoApply(5, enabled);
    OutputEnabled_AutoApply(6, enabled, auto_apply);
   // OutputEnabled_AutoApply(7, enabled);
    OutputEnabled_AutoApply(8, enabled, auto_apply);
    OutputEnabled_AutoApply(9, enabled, auto_apply);
    OutputEnabled_AutoApply(10, enabled, auto_apply);
    OutputEnabled_AutoApply(11, enabled, auto_apply);
    OutputEnabled_AutoApply(12, enabled, auto_apply);

    //WriteDataRegisters();

}

//---------------------------------------------------------------------------
//  Atropos_Pll::EnableAllTriggerOutputs() -- enable/disable trigger outputs
//---------------------------------------------------------------------------

void  Atropos_Pll::EnableAllTriggerOutputs(bool enabled, bool auto_apply)
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::EnableAllTriggerOutputs()");
    Debug.Log("enabled=", enabled);

    OutputEnabled_AutoApply(1, enabled, auto_apply);
    OutputEnabled_AutoApply(3, enabled, auto_apply);
    OutputEnabled_AutoApply(5, enabled, auto_apply);
    OutputEnabled_AutoApply(7, enabled, auto_apply);
    OutputEnabled_AutoApply(13, enabled, auto_apply);

    //WriteDataRegisters();

}

//-----------------------------------------------------------------------------
//  Atropos_Pll::OutputEnabled_AutoApply()
//-----------------------------------------------------------------------------

void  Atropos_Pll::OutputEnabled_AutoApply(unsigned int output, bool enabled, bool auto_apply)
{
    OutputEnabledArray[output] = enabled;
    SetOutEnabledHdw_autoapply(output, enabled, auto_apply);  // auto apply
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::OnAfterOutputSync()  --  Serializer/Deserializer configuration
//-----------------------------------------------------------------------------

void Atropos_Pll::OnAfterOutputSync()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::OnAfterOutputSync()");

    // Skip the calibration sequence after OnInit()
    if (At_Init)
        {
        At_Init = false;
        return;
        }

    //
    //  Call Logic to SerDes S2P calibration
    Logic_S2P_Calibration();

    //  Setup the trigger path in the LMK04820
    TriggerPathSetup();
    //
    //  Call Logic to SerDes P2S calibration
    Logic_P2S_Calibration();


    Latency_Calibration();

    Debug.Log("Enable Triggers");
    bool ApplyNow = true;
    EnableAllTriggerOutputs(true, ApplyNow);

    Debug.Log("Select B Off");
    Owner->SerDes().S2P_SelectB.Clear();
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::Logic_S2P_Calibration()  --  S2P Calibration
//-----------------------------------------------------------------------------

void  Atropos_Pll::Logic_S2P_Calibration()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::Logic_S2P_Calibration()");

    //  S2P Reset should be on...

    //  Set S2P Mode
    Debug.Log("...Set S2P Mode");
    bool mode = (Frequency()<1450.0e6) ? true : false ;
    Owner->SerDes().S2P_Sdr_DdrN_Mode.Value(mode);

    // Select B Input
    Debug.Log("...Select input B");
    Owner->SerDes().S2P_SelectB.Set();

    for (int reps=0; reps<5; reps++)
        {
        // Start Cal
        Debug.Log("Start S2P Calibration");
        Owner->SerDes().S2P_Reset.Clear();
        Innovative::Sleep(1);

        // Check Cal Done
        bool Result = Owner->SerDes().S2P_CalDone.Value();
        Debug.Log("Cal Result: ", Result);

        if (Result)
            return;

        //  Set S2P Reset
        Owner->SerDes().S2P_Reset.Set();
        Innovative::Sleep(1);
        Debug.Log("Cal FAILED, retry");
        }
    Debug.Log("Too many retries, abandoning cal");
    LastOutputSyncStatus = false;
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::TriggerPathSetup()  --  Setup the trigger path in the LMK04820
//-----------------------------------------------------------------------------

void  Atropos_Pll::TriggerPathSetup()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::TriggerPathSetup()");

    const bool ApplyNow = true;

    // ( set SYSREF_PD, SYSREF_DDLY_PD, SYSREF_PLSR_PD)  // go away
    Debug.Log("...power down SYSREF");
    PowerDownAllSysref(true);
    PowerDownSysrefDigitalDelay(true);
    PowerDownSysrefPulseGen(true, ApplyNow);  // update register
    //
    //  Set Trigger mode to "LowLatency"
    TriggerSyncMode(tsmLowLatency);
    TriggerSyncSetup();             // call here?
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::Logic_P2S_Calibration()  --  P2S Calibration
//-----------------------------------------------------------------------------

void  Atropos_Pll::Logic_P2S_Calibration()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::Logic_P2S_Calibration()");

    //  P2S Reset should be on...

    //  Set P2S Mode
    Debug.Log("...Set P2S Mode");
    bool mode = (Frequency()<1450.0e6) ? true : false ;
    Owner->SerDes().P2S_Sdr_DdrN_Mode.Value(mode);
    Owner->SerDes().P2S_PulserCalEnable.Clear();

    //  Adjust the clock delay so P2S serial output is captured reliably
    //  by the S2P when loopedback through the LMK04820 to the S2P SinB input
    P2S_ClockAdjust();

    //  Adjust the P2S parallel data capture by controlling the PCLK phase used
    //  in the FPGA to update the data bus
    P2S_CaptureAdjust();
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::P2S_ClockAdjust()  --  P2S Clock adjust
//-----------------------------------------------------------------------------

void  Atropos_Pll::P2S_ClockAdjust()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::P2S_ClockAdjust()");

    Debug.Log("Load P2S Data = 0x0707");
    Owner->SerDes().P2S_Samples.Value(0x707);

    Debug.Log("Set P2S Override...");
    Owner->SerDes().P2S_DataOverride.Set();

    Debug.Log("Start P2S Calibration");
    Owner->SerDes().P2S_Reset.Clear();
    Innovative::Sleep(1);

    //  Test all valid delays
    unsigned int DV_size = 24;
    std::vector<bool> DataValidResults(DV_size+1);
    for (unsigned int delay=0; delay<DV_size; delay++)
        {
        //  Set delay and check it
        AnalogDelay_10(delay);
        //  save the result
        DataValidResults[delay] = Owner->SerDes().S2P_DataValid.Value();
        }
    DataValidResults[DV_size] = false;  // sentinel end step

    unsigned int MaxValidStartIdx(0);
    unsigned int MaxValidEndIdx(0);
    unsigned int ValidStartIdx(0);
    unsigned int ValidEndIdx(0);
    for (unsigned int delay=0; delay<DV_size+1; delay++)
        {
        if (DataValidResults[delay])
            {
            // we are in a valid region now...extend the current region
            if (ValidEndIdx-ValidStartIdx==0)  // true if was just invalid
                {
                ValidStartIdx = delay;   // starting a new valid region
                }
            ValidEndIdx = delay+1;  // grow the valid region by one
            }
        else
            {
            // we are not in a valid region now
            if (ValidEndIdx-ValidStartIdx!=0) // true if was valid last time
                {
                if (ValidEndIdx-ValidStartIdx > MaxValidEndIdx-MaxValidStartIdx)
                    {
                    // if the longest run of valid entries, save it
                    MaxValidEndIdx = ValidEndIdx;
                    MaxValidStartIdx = ValidStartIdx;
                    }
                }
            ValidStartIdx = ValidEndIdx = delay;   // start=end means region is now invalid
            }
        }

    Debug.Log("Max Valid Region Start Delay=", MaxValidStartIdx);
    Debug.Log("Max Valid Region End Delay=", MaxValidEndIdx);
    Debug.Log("Max Valid Region Width=", MaxValidEndIdx-MaxValidStartIdx);

    if (Frequency())  // guard for 0 frequencies
        Debug.Log("Estimated Width=", (unsigned int)(1.0/Frequency()/25.0e-12));
    else
        Debug.Log("Estimated Width=Zero Freq");

    unsigned int CalibratedDelay = MaxValidStartIdx + (MaxValidEndIdx-MaxValidStartIdx)/2;
    Debug.Log("CalibratedDelay=", CalibratedDelay);
    AnalogDelay_10(CalibratedDelay);

    // do shift check with 0x0707
    bool nowAligned = ShiftCheck();
    Debug.Log("0x0707 Shift Check Result:", nowAligned);
    LastOutputSyncStatus &= nowAligned;
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::P2S_CaptureAdjust()  --  P2S data capture adjust
//-----------------------------------------------------------------------------

void  Atropos_Pll::P2S_CaptureAdjust()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::P2S_CaptureAdjust()");

    Debug.Log("Load P2S Data = 0x00FF");
    Owner->SerDes().P2S_Samples.Value(0x0FF);
    Owner->SerDes().P2S_UpdateFallingEdge.Clear();

    // do shift check with clear UpdateFallingEdge
    bool nowAligned = ShiftCheck();
    Debug.Log("0x00FF Shift Check Result:", nowAligned);
    if (nowAligned==false)
        {
        Debug.Log("First Shift Check Failed..Reset phase and retry");
        Owner->SerDes().P2S_UpdateFallingEdge.Set();
        nowAligned = ShiftCheck(); // try one last time...
        if (!nowAligned)
            {
            Debug.Log("Final Shift Check Failed");
            LastOutputSyncStatus = false;
            }
        }
    Owner->SerDes().S2P_DataCheckEnable.Clear();
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::ShiftCheck()  -- check alignment
//-----------------------------------------------------------------------------

bool  Atropos_Pll::ShiftCheck()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::ShiftCheck()");

    bool nowAligned(false);
    for (unsigned int shift=0; shift<16; shift++)
        {
        //Debug.Log("Data Check Disable");
        Owner->SerDes().S2P_DataCheckEnable.Clear();

        //Debug.Log("Data Check Enable");
        Owner->SerDes().S2P_DataCheckEnable.Set();

        Sleep(10);

        Debug.Log("S2P_DataAligned=", Owner->SerDes().S2P_DataAligned.Value());
        if (Owner->SerDes().S2P_DataAligned.Value())
            {
            Debug.Log("Aligned after shift", shift);
            nowAligned = true;
            break;
            }

        // shift one more time
        Owner->SerDes().S2P_SyncPulse.Set();
        Owner->SerDes().S2P_SyncPulse.Clear();
        }
    return nowAligned;
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::Latency_Calibration()  --  P2S-S2P Latency Calibration
//-----------------------------------------------------------------------------

void  Atropos_Pll::Latency_Calibration()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::Latency_Calibration()");

    Debug.Log("Pulser Calibration");
    Owner->SerDes().P2S_DataOverride.Clear();
    Sleep(11);
    Owner->SerDes().P2S_PulserCalEnable.Set();

    Sleep(100);
    bool pulserDone = Owner->SerDes().P2S_PulserCalDone.Value();
    Debug.Log("Pulser Cal Done Result:", pulserDone);
    LastOutputSyncStatus &= pulserDone;

    Debug.Log("End Pulser Cal");
    Owner->SerDes().P2S_PulserCalEnable.Clear();
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::AnalogDelay_10()  --
//-----------------------------------------------------------------------------

void  Atropos_Pll::AnalogDelay_10(unsigned int value)
{
    //ScopedTraceRef ccc(Debug, "Atropos_Pll::AnalogDelay_10()");
    Debug.Log("delay value=", value);
    //Debug.Log("S2P_DataValid=", Owner->SerDes().S2P_DataValid.Value());
    //Debug.Log("S2P_DataAligned=", Owner->SerDes().S2P_DataAligned.Value());

    //Debug.Log("Data Check Disable");
    Owner->SerDes().S2P_DataCheckEnable.Clear();

    // PLL update
    bool ApplyNow=true;
    SetAnalogDelay(10, value, ApplyNow);

    // Dummy read to wait for this SPI to finish
    unsigned int data = Port(0x12B);
    data = data;  // silence unused variable noise.
    //Debug.Log("Wait for SPI Dummy Read", data, 16);

    //Debug.Log("Data Check Enable");
    Owner->SerDes().S2P_DataCheckEnable.Set();

    Sleep(10);

    Debug.Log("S2P_DataValid=", Owner->SerDes().S2P_DataValid.Value());
    Debug.Log("S2P_DataAligned=", Owner->SerDes().S2P_DataAligned.Value());
}

//-----------------------------------------------------------------------------
//  Atropos_Pll::Logic_SerDes_Disable()  --
//-----------------------------------------------------------------------------

void  Atropos_Pll::Logic_SerDes_Disable()
{
    ScopedTraceRef ccc(Debug, "Atropos_Pll::Logic_SerDes_Disable()");

    Debug.Log(" Reset the P2S and S2P Interface Modules");
    Owner->SerDes().S2P_Reset.Set();
    Owner->SerDes().P2S_Reset.Set();
}

//===========================================================================
//  CLASS AtroposDeviceMap  -- Register & low level support for Sbc_ComEx
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class AtroposDeviceMap
//---------------------------------------------------------------------------

using namespace Atropos_MemoryMap;

namespace  // nameless
{
X6Info::MapRegisters AtroposInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmSubrevision,        //  BitstreamDate_Addr
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

AtroposLedReset::MapRegisters AtroposPeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

QSpiFpaLoader::MapRegisters AtroposVsPromRegAddrs =
    {
    mmVsProm_ControlAddr,  //  AddressControl_Addr;
    mmVsProm_Data,         //  Data_Addr
    mmVsProm_Opcode,       //  Opcode_Addr;
    };

X6Thermal::MapRegisters ThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

AtroposLMK20_Clock::MapRegisters AtroposLMKClockRegAddrs =
    {
    mmPllControl,       //  AfePllControl_Addr;
    mmPllSpi,           //  AfePllSpi_Addr
    mmPllExtClkSelect,  //  ExtClkSourceSelect_Addr
    };

Lmk04820Registers  Atropos_Lmk04820Registers =
{
    mmPllControl,    //  Control_Addr;
    mmPllSpi,        //  SpiIntf_Addr

};

AtroposTriggerSystem::MapRegisters AtroposTriggerRegAddrs =
    {
    mmExtSyncSelect,    //  ExtSyncSelect_Addr;
    mmSerDesControl,    //  SerDesControl_Addr
    mmSofwareTrigger,   //  SoftwareTrigger_Addr
    mmTriggerControl,   //  TriggerConfig_Addr;
    mmDecimation,       //  TriggerDecimation_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters AtroposPriRegAddrs =
    {
    mmPriTriggerEn,      //  PriTriggerEnable_Addr;
    mmPriControl,        //  PriInterval_Addr;
    mmPriPulseDelay,     //  PriTriggerParam_Addr;
    mmPriPulseWidth      //  PriCaptureWindow_Addr;
    };

EEpromIntfStyle ModernStyle = intfModernStyle;

}  // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class AtroposDeviceMap
//---------------------------------------------------------------------------

AtroposDeviceMap::AtroposDeviceMap()
    :  IUsesPmcEeprom        (LogicMemory, mmIdromI2C, 2048, ModernStyle),
       DeviceDriver          (new AtroposDeviceDriver),
       WB_Periph             (LogicMemory, wbPeripheral),
       WB_Loader             (LogicMemory, wbLogicLoader),
       WB_Thermal            (LogicMemory, wbTempControl),
       Info_SDev             (this, WB_Periph, PortMemory, AtroposInfoRegAddrs),
       Periph_SDev           (this, WB_Periph, AtroposPeripheralRegAddrs),
       Clock_SDev            (this, WB_Periph, AtroposLMKClockRegAddrs),
       Trigger_SDev          (this, WB_Periph, AtroposTriggerRegAddrs),
       Pri_SDev              (this, true, WB_Periph, AtroposPriRegAddrs),
       Thermal_SDev          (this, WB_Thermal, ThermalRegAddrs),
       VsProm_SDev           (this, QSpiFpaLoader::romMicron_N25Q128, WB_Loader, AtroposVsPromRegAddrs),
       PciControlReg         (PortMemory, mmPciControl),
	   InterruptStatusReg    (PortMemory, mmInterruptStatus),
       InterruptEnableReg    (PortMemory, mmInterruptEnable),
       Isr(*this), Debug(true)
{
    ConstructCustomParts();
}

AtroposDeviceMap * anAtropos()
{
    AtroposDeviceMap * wombat = new AtroposDeviceMap;
    return wombat;
}

//---------------------------------------------------------------------------
//  destructor for class AtroposDeviceMap
//---------------------------------------------------------------------------

AtroposDeviceMap::~AtroposDeviceMap()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t AtroposDeviceMap::BoardCount()
{
	return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  AtroposDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
    return true;
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  AtroposDeviceMap::Name() const
{
    switch (Info().PciLogicType())
        {
        case 4: return "Atropos";
        default: return "Unknown";
        }
    //return "Unknown";
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::FpgaName() --  Return name string for FPGA type
//---------------------------------------------------------------------------

std::string  AtroposDeviceMap::FpgaName(short Number) const
{
     switch (Number)
        {
        case 0x00: return "LX75T";
        case 0x01: return "LX25T";
        default: return "Unknown";
        }
    //return "Unknown";
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  AtroposDeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Open()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Open()
{
    if(DeviceDriver->Opened())
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));
    LogicMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->LogicMemoryAddress()));

    InterruptEnable(false);
    Isr.Open();

    //
    //  After Open Behavior
    OpenHardware();

}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Close()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Close()
{
    if(!DeviceDriver->Opened())
        return;

    InterruptEnable(false);

    Isr.Close();
    DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Reset()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Reset()
{
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::MReset()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::MReset()
{
    Debug.Log("+AtroposDeviceMap::MReset()");
    PciControlReg.BusmasterReset.Value(false);  // apply BM Reset (active low)
    PciControlReg.BusmasterReset.Value(true);   // release BM Reset
    PciControlReg.MasterReset.Value(true);      // Apply "Board" reset (autoclear)
    PciControlReg.MasterReset.Value(false);     // but we need to clear shadow
    Debug.Log("-AtroposDeviceMap::MReset()");
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Start() - turn on clocks & triggers
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Start()
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::Start()");

    Atropos_Pll & AtrPll = dynamic_cast<Atropos_Pll &>(Clock_SDev.RawClockDevice().Pll());
    AtrPll.EnableAllOutputs(true);  // don't apply now

    PreconfigureHardware();
    ConfigureHardware();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Stop()  turn off clocks & triggers
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Stop()
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::Stop()");

    Atropos_Pll & AtrPll = dynamic_cast<Atropos_Pll &>(Clock_SDev.RawClockDevice().Pll());
    bool ApplyNow = true;
    AtrPll.EnableAllOutputs(false, ApplyNow);
    AtrPll.EnableAllTriggerOutputs(false, ApplyNow);

    AtrPll.ResetSerDes();

    StopHardware();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  AtroposDeviceMap::Target() const
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::MakeFmcAfePllObject() -- Get Pri Device object
//---------------------------------------------------------------------------

Lmk04820 *  AtroposDeviceMap::MakeFmcAfePllObject()
{
    return new Atropos_Pll(this, WB_Periph, Atropos_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 AtroposDeviceMap::PeekPort(int offset)
{
    ii32 * PortBasePtr = PortMemory.Base();
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void AtroposDeviceMap::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = PortMemory.Base();
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32 AtroposDeviceMap::PeekLogic(int offset)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    return LogicBasePtr[offset];
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void AtroposDeviceMap::PokeLogic(int offset, ii32 value)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    LogicBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Led()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::Led(bool value)
{
    PciControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::Led()
//---------------------------------------------------------------------------

bool  AtroposDeviceMap::Led() const
{
    return PciControlReg.Led;
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::InterruptEvent() --
//---------------------------------------------------------------------------

Event * AtroposDeviceMap::InterruptEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::InterruptDepth() --
//---------------------------------------------------------------------------

size_t AtroposDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::InterruptAck() --
//---------------------------------------------------------------------------

unsigned int AtroposDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void AtroposDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::InterruptDispatch()");
    // dispatch interrupt
    //Gps_SDev.Gps().DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void AtroposDeviceMap::InterruptEnable(bool state)
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::InterruptEnable()");
    Debug.Log("state=", (int)state);

    InterruptEnableReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    Clock_SDev.ConstructCustomParts();
    Trigger_SDev.ConstructCustomParts();
    Pri_SDev.ConstructCustomParts();
    Thermal_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::OpenHardware()");
    //
    //  Reset the device
    MReset();
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    Clock_SDev.OpenHardware();
    Trigger_SDev.OpenHardware();
    Pri_SDev.OpenHardware();
    Thermal_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  AtroposDeviceMap::PreconfigureHardware()
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::PreconfigureHardware()");
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    Clock_SDev.PreconfigureHardware();
    Trigger_SDev.PreconfigureHardware();
    Pri_SDev.PreconfigureHardware();
    Thermal_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  AtroposDeviceMap::ConfigureHardware()
{
    ScopedTraceRef ccc(Debug, "AtroposDeviceMap::ConfigureHardware()");
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    Clock_SDev.ConfigureHardware();
    Trigger_SDev.ConfigureHardware();
    Pri_SDev.ConfigureHardware();
    Thermal_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::StopHardware() -- set hardware for 'off' condition
//---------------------------------------------------------------------------

void  AtroposDeviceMap::StopHardware()
{
    // Subdevice Configuration
    Periph_SDev.StopHardware();
}


//---------------------------------------------------------------------------
//  AtroposDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  AtroposDeviceMap::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new AtroposFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  AtroposDeviceMap::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new AtroposPriClockDevice(this, 1);
}



} // namespace Innovative


