// AtroposDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef AtroposDeviceMap_MbH
#define AtroposDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <FmcSubDevice_Stream_Mb.h>      // Thermal Bus
#include <X6SubDevice_Stream_Mb.h>       // PRI
#include <K7Support_Mb.h>
#include <K7SubDevice_Misc_Mb.h>
#include <SimpleInterrupt_Mb.h>
#include <PmcEeprom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS ISuppliesAtroposRequirements -- Atropos Requirements
//===========================================================================

class ISuppliesAtroposRequirements : public IRequires_Nothing,
                                     public IRequires_Lmk20FmcClocks,
                                     public IRequires_FrameCalcDevice,
                                     public IRequires_PriClockDevice
{
public:
    virtual ~ISuppliesAtroposRequirements()
        {}

};

class AtroposDeviceMap;

//==============================================================================
//  CLASS Atropos_Pll -- Atropos PLL Chip (Specialization of Lmk04820)
//==============================================================================

class Atropos_Pll : public Lmk04820
{
public:
    Atropos_Pll(AtroposDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~Atropos_Pll();

    void  EnableAllOutputs(bool enabled, bool auto_apply=false);
    void  EnableAllTriggerOutputs(bool enabled, bool auto_apply=false);
    void  OutputEnabled_AutoApply(unsigned int output, bool enabled, bool auto_apply);
    void  ResetSerDes()  {  Logic_SerDes_Disable();  }

    void  AnalogDelay_10(unsigned int value);

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 40e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    AtroposDeviceMap *   Owner;
    double               FTrueFrequency;
    double               FPDF;
    bool                 At_Init;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

    //  Serdes Logic Hdw Interface Methods
    virtual void   OnAfterOutputSync();
    void  Logic_SerDes_Disable();
    void  Logic_S2P_Calibration();
    void  TriggerPathSetup();
    void  Logic_P2S_Calibration();
    void  P2S_ClockAdjust();
    void  P2S_CaptureAdjust();
    bool  ShiftCheck();
    void  Latency_Calibration();
};


//===========================================================================
//  CLASS AtroposDeviceMap  -- Register support for K7 System Unit
//===========================================================================

class AtroposDeviceMap :  public ISuppliesAtroposRequirements,
                          public IUsesPmcEeprom,
                          public ISimpleInterrupt
{
public:
    friend class Atropos;
    //friend class AtroposGPS_Callback;
        //
    //  Member Functions
    AtroposDeviceMap(/*Atropos * owner*/);
    virtual ~AtroposDeviceMap();

    // Methods
    virtual size_t  InterruptDepth();
	size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);

    void            Open();
    void            Close();
    void            Reset();
    void            Start();
    void            Stop();

    std::string         Name() const;
    std::string         FpgaName(short Number) const;
    std::string         Revision() const;

    void            Led(bool value);
    bool            Led() const;

    //  Hardware Device Access
    const IX6LogicVersionInfo &     Info() const      {  return Info_SDev;  }
    IFpaLoader &                    VsProm()          {  return VsProm_SDev.VsProm();  }
    IUsesPmcEeprom &                IdRom()           {  return *this;   }
    IX6ThermalDevice &              Thermal()         {  return Thermal_SDev;  }

    AtroposTriggerSystem::SerDesControlRegister & SerDes()  { return Trigger_SDev.SerDes(); }

    //  Subdevice Customization Handlers

    //  Requirements Supplier Methods
    virtual class IPriClockDevice *      MakePriClockDevice(PriDeviceCode which);
    virtual Lmk04820 *                   MakeFmcAfePllObject();
    virtual IFrameCalcDevice *           MakeFrameCalcDevice(FrameDeviceCode which);

    //  Debug Support
    ii32            PeekPort(int offset);
    void            PokePort(int offset, ii32 value);
    ii32            PeekLogic(int offset);
    void            PokeLogic(int offset, ii32 value);

    // Properties
    void            Target(int target);
    int             Target() const;

	AddressingSpace    PortMemory;
    AddressingSpace    LogicMemory;

    // Debug
    QSpiFpaLoader &                    DVsProm()          {  return VsProm_SDev;  }

    void   MReset();    // Master Reset

    virtual double  StandardClockFrequency()
                        {  return Clock_SDev.Clock().Frequency();   }

protected:
    //
    //  Member Data
    InnovativeKernel::DualRegionIsrDeviceDriver *  DeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Loader;
    WishboneBusSpace       WB_Thermal;
    //
    //  Subdevices
    X6Info                           Info_SDev;
    AtroposLedReset                  Periph_SDev;
    AtroposLMK20_Clock               Clock_SDev;
    AtroposTriggerSystem             Trigger_SDev;
    X6AFE_PulseRepetitionInterval    Pri_SDev;
    X6Thermal                        Thermal_SDev;
    QSpiFpaLoader                    VsProm_SDev;
    //
    //  Registers
    //
	//  Class PciControlRegister
    class PciControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       MasterReset;
        RegisterBit       Led;
        RegisterBit       BusmasterReset;

	public:
        PciControlRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
                MasterReset( *this, 0 ),
                Led( *this, 1 ),
                BusmasterReset( *this, 3 )
            {
            //  Default setup ????
            }
    };

	//  ...Interrupt Registers
	PciControlRegister                 	PciControlReg;
    Register                            InterruptStatusReg;
    ShadowRegister                      InterruptEnableReg;

private:
    // Interface
    virtual Event *         InterruptEvent();
    virtual unsigned int    InterruptAck();
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);

    void    ConstructCustomParts();
    void    OpenHardware();
    void    PreconfigureHardware();
    void    ConfigureHardware();
    void    StopHardware();

    // Data
    SimpleInterrupt         Isr;
    Trace                   Debug;
};

//=============================================================================
//  CLASS AtroposPriClockDevice  --  Normal impl to access clock for PRI
//=============================================================================

class AtroposPriClockDevice : public IPriClockDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->StandardClockFrequency();  }
    virtual int   CalculatePriDivisor()
        {  return Divisor;  }

    AtroposPriClockDevice(AtroposDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~AtroposPriClockDevice()
        {}

private:
    AtroposDeviceMap * Owner;
    int Divisor;
};

//=============================================================================
//  CLASS AtroposFrameCalcDevice  --  Supply Divider on Frame ticks
//=============================================================================

class AtroposFrameCalcDevice : public IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor()
        {  return Divisor;  }

    AtroposFrameCalcDevice(AtroposDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~AtroposFrameCalcDevice()
        {}

private:
    AtroposDeviceMap * Owner;
    int Divisor;
};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
