//---------------------------------------------------------------------------
// Atropos_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Atropos_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS Atropos  -- Time/Trigger baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Atropos
//---------------------------------------------------------------------------

Atropos::Atropos()
{
}

//---------------------------------------------------------------------------
//  destructor for class Atropos
//---------------------------------------------------------------------------

Atropos::~Atropos()
{
}

//---------------------------------------------------------------------------
//  Atropos::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t Atropos::BoardCount()
{
	return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  Atropos::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> Atropos::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  Atropos::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string Atropos::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  Atropos::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  Atropos::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  Atropos::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Atropos::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  Atropos::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Atropos::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  Atropos::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  Atropos::GreenLed() const
{
    return Device.Periph_SDev.GreenLed();
}

//---------------------------------------------------------------------------
//  Atropos::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  Atropos::GreenLed( bool value )
{
    Device.Periph_SDev.GreenLed(value);
}

//---------------------------------------------------------------------------
//  Atropos::BlueLed() -- Read the Blue LED state
//---------------------------------------------------------------------------

bool  Atropos::BlueLed() const
{
    return Device.Periph_SDev.BlueLed();
}

//---------------------------------------------------------------------------
//  Atropos::BlueLed() -- Set the Blue LED state of the board
//---------------------------------------------------------------------------

void  Atropos::BlueLed( bool value )
{
    Device.Periph_SDev.BlueLed(value);
}

//---------------------------------------------------------------------------
//  Atropos::SoftwareTrigger() -- Raise or lower the software trigger
//---------------------------------------------------------------------------

void  Atropos::SoftwareTrigger( bool state )
{
    Device.Trigger_SDev.SoftwareTrigger(state);
}

//---------------------------------------------------------------------------
//  Atropos::MasterTrigger() -- Raise or lower the master trigger
//---------------------------------------------------------------------------

void  Atropos::MasterTrigger( bool state )
{
    Device.Trigger_SDev.MasterTrigger(state);
}

//---------------------------------------------------------------------------
//  Atropos::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  Atropos::Open()
{
    Device.Open();
}

//---------------------------------------------------------------------------
//  Atropos::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  Atropos::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  Atropos::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  Atropos::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  Atropos::Start() -- Start the clocks and triggers
//---------------------------------------------------------------------------

void  Atropos::Start()
{
    Device.Start();
}

//---------------------------------------------------------------------------
//  Atropos::Stop() -- Stop the clocks and triggers
//---------------------------------------------------------------------------

void  Atropos::Stop()
{
    Device.Stop();
}

//---------------------------------------------------------------------------
//  Atropos::Info() --
//---------------------------------------------------------------------------

const IX6LogicVersionInfo & Atropos::Info() const
{
    return Device.Info();
}

//---------------------------------------------------------------------------
//  Atropos::VsProm() --
//---------------------------------------------------------------------------

IFpaLoader &  Atropos::VsProm()
{
	return Device.VsProm();
}

//---------------------------------------------------------------------------
//  Atropos::Thermal() --
//---------------------------------------------------------------------------

IX6ThermalDevice &  Atropos::Thermal()
{
	return Device.Thermal();
}

//---------------------------------------------------------------------------
//  Atropos::Clock() -- return interface to Clock Manager
//---------------------------------------------------------------------------

IAtroposClockIo &  Atropos::Clock()
{
    return Device.Clock_SDev.Clock();
}

//---------------------------------------------------------------------------
//  Atropos::AtroposRawClockDevices() -- return raw clock device interface
//---------------------------------------------------------------------------

ILmk20RawClockDevices &  Atropos::RawClockDevice()
{
    return Device.Clock_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  Atropos::Trigger() -- Interface to Atropos Triggering
//---------------------------------------------------------------------------

IAtroposTriggerCfg &  Atropos::Trigger()
{
    return Device.Trigger_SDev.Trigger();
}

//---------------------------------------------------------------------------
//  Atropos::Pulse() -- Interface to Atropos PRI configuration
//---------------------------------------------------------------------------

IPulseRepetitionIntervalIntf &  Atropos::Pulse()
{
    return Device.Pri_SDev;
}

//---------------------------------------------------------------------------
//  Atropos::IdRom() --
//---------------------------------------------------------------------------

IUsesPmcEeprom & Atropos::IdRom()
{
    return Device.IdRom();
}

//---------------------------------------------------------------------------
//  Atropos::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  Atropos::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  Atropos::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  Atropos::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  Atropos::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  Atropos::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  Atropos::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  Atropos::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  Atropos::AfterFiclStart() --
//---------------------------------------------------------------------------

void  Atropos::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate(" 0 constant Fan 1 constant Pll");

    engine.Evaluate(": ?Atropos "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Fan  Pll\" "
                    "    cr  "
                    "    cr .\" ---- Atropos Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?Atropos  : display this message \" "
                    "    cr  ; ");
    //engine.Evaluate(": ?VpxEpc  ?Atropos  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("Atropos-specific wordset loaded \n" );
}

//---------------------------------------------------------------------------
//  Atropos::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  Atropos::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  Atropos::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  Atropos::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  Atropos::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  Atropos::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  Atropos::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  Atropos::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  Atropos::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  Atropos::Do_DFetch(int dev, int addr)
{
    const int kFan = 0; const int kPll = 1;

    switch (dev)
        {
        default:
            return 0;
        case kFan:
            return Device.Thermal().Port(addr);
        case kPll:
            return Device.Clock_SDev.Pll().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  Atropos::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  Atropos::Do_DStore(int dev, int addr, int value)
{
    const int kFan = 0; const int kPll = 1;


    switch (dev)
        {
        default:
            break;
        case kFan:
            Device.Thermal().Port(addr, value);
            break;
        case kPll:
            Device.Clock_SDev.Pll().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  Atropos::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  Atropos::Do_DFreq(int dev, float freq)
{
    const int kPll = 1;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.Clock_SDev.Pll().Frequency(freq);
            Device.Clock_SDev.Pll().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  Atropos::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  Atropos::Do_DFreqActual(int dev)
{
    const int kPll = 1;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.Clock_SDev.Pll().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
