// Atropos_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Atropos_MbH
#define Atropos_MbH

#include <DataTypes_Mb.h>
#include <AtroposDeviceMap_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS Atropos  -- Atropos baseboard class
//===========================================================================

class Atropos : public IFiclTarget
{

public:
    // Enumerations

    //
    //  Member Functions
    Atropos();
    virtual ~Atropos();

    //  Informational Methods
	size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
	bool                        LicenseCheck(unsigned int & code);

    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 FpgaName(short Number) const
        {  return Device.FpgaName(Number);  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    int                         Target();
    void                        Target(int value);

    bool                        GreenLed() const;
    void                        GreenLed(bool value);

    bool                        BlueLed() const;
    void                        BlueLed(bool value);

    void                        SoftwareTrigger(bool state);
    void                        MasterTrigger(bool state);

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    virtual void                Start();
    virtual void                Stop();

    //
    //  Properties

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    const IX6LogicVersionInfo &         Info() const;
    IAtroposClockIo &                   Clock();
    ILmk20RawClockDevices &             RawClockDevice();
    IAtroposTriggerCfg &                Trigger();
    IPulseRepetitionIntervalIntf &      Pulse();

    IFpaLoader &                        VsProm();
    IX6ThermalDevice &                  Thermal();
    IUsesPmcEeprom &                    IdRom();

public: //Pseudo protected

    AtroposDeviceMap * Debug() { return &Device; }

protected:
    //
    //  Member Data
    AtroposDeviceMap       Device;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( Atropos & board );
    friend AddressingSpace & PortMemorySpace( Atropos & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( Atropos & board )
{
    return board.Device.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( Atropos & board )
{
    return board.Device.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

