// BandStop_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <BandStop_Mb.h>
#include <math.h>
#include <ipps.h>               // ipps.h is not redistributable!

//---------------------------------------------------------------------------
namespace   Innovative
{

//---------------------------------------------------------------------------
static const float Pi = static_cast<float>(4.0*atan(1.0));

//---------------------------------------------------------------------------
//  BandStop::BandStop() -  Ctor
//---------------------------------------------------------------------------

BandStop::BandStop()
{
}

//---------------------------------------------------------------------------
//  BandStop::BandStop() -  Copy ctor
//---------------------------------------------------------------------------

BandStop::BandStop(const BandStop & other)
    : inherited(other)
{
}

//---------------------------------------------------------------------------
//  BandStop::operator = - 
//---------------------------------------------------------------------------

BandStop & BandStop::operator = ( const BandStop & other )
{
    inherited::operator = ( other );

    return *this;
}

//---------------------------------------------------------------------------
//  BandStop::CalculateCoefficients() -
//---------------------------------------------------------------------------

void BandStop::CalculateCoefficients()
{
	float CutLow = (FFreqLow / FSampleRate) * 2.0f;
	float CutHigh = (FFreqHigh / FSampleRate) * 2.0f;

	//---------------------------------------------
	// adjust the number of coefficients to be ODD
	//---------------------------------------------
    size_t count = FCoefficient.size();
	size_t CoefNum = count;
	if (count % 2 == 0)
		++CoefNum;

	size_t HalfLen = (CoefNum - 1) / 2;

	//--------------------------------------------------------
	// Allocate memory for coefficients if length changed
	//--------------------------------------------------------
	FCoefficient.resize(CoefNum);

	//------------------
	// Bandpass filter
	//------------------
	FCoefficient[HalfLen] = CutHigh - CutLow;
	for (size_t Cnt = 1; Cnt <= HalfLen; Cnt++)
		{
		float   TmpFloat = Pi * Cnt;
		FCoefficient[HalfLen + Cnt] = 2.0f *
			sin((CutHigh - CutLow) / 2.0f * TmpFloat) *
			cos((CutHigh + CutLow) / 2.0f * TmpFloat) /
			TmpFloat;
		FCoefficient[HalfLen - Cnt] = FCoefficient[HalfLen + Cnt];
		}

	//------------------------------
	// multiplying with a window
	//------------------------------
	//  TmpFloat := 2.0 * Pi / (CoefNum - 1.0);
	ApplyWindow(&FCoefficient[0], CoefNum, FWindowType, FAlpha);
	ippsMulC_32f_I(-1, &FCoefficient[0], static_cast<int>(CoefNum));

	//------------------------------
	// Normalize GAIN to 1 ( Step 1 )
	//------------------------------
	float   Sum = 0.0f;
	for (size_t Cnt = 0; Cnt < CoefNum; Cnt++)
		{
		if (Cnt % 2 == 0)
			Sum += FCoefficient[Cnt];   // poly(-1), even +, odd -
		else
			Sum -= FCoefficient[Cnt];
		}

	++FCoefficient[HalfLen];
	++Sum;

	//------------------------------
	// Normalize GAIN to 1 ( Step 2 )
	//------------------------------
	if (Sum < 0)
		Sum = -Sum;

	ippsDivC_32f_I(Sum, &FCoefficient[0], static_cast<int>(CoefNum));
}

} // Innovative
