//  BaseboardCpuInterfaces_Mb.h
//
//  Contains standard implementation of Baseboard Cpus
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef BaseboardCpuInterfaces_MbH
#define BaseboardCpuInterfaces_MbH


#include <BaseboardInterfaces_Mb.h>
#include <PacketDeviceSupport_Mb.h>
#include <HpiEngine_Mb.h>
#include <CoffThread_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS IUsesHpiControl -- Low level hardware interface class.
//===========================================================================

class IUsesHpiControl
{
public:
    virtual void ResetBaseBoard() = 0;
    virtual  ~IUsesHpiControl()
    	{}
};

//===========================================================================
//  CLASS IUsesStandardHpiCpuBase -- User Interface to mix-in onboard CPU & Hpi
//===========================================================================
//  Standard Interface implementation for Onboard CPU support...

class IUsesStandardHpiCpu : public IUsesOnboardHpiCpu
{
    friend class IUsesMatadorCpu;

public:
    //
    // Methods
    IUsesStandardHpiCpu(IUsesHpiControl & hpic,
		                HpiEngineAddresses & HpiAddr,
                        HpiCeSpaceValues & HpiCe);

    ~IUsesStandardHpiCpu();

    virtual void  Boot();
    virtual void  DownloadCoff(const std::string & Filename);

    virtual CoffFile & CoffImage();
    //
    // Methods
    virtual HpiEngine & Hpi();

protected:
    //
    //  Member Data
    bool                   mHpiLoadVerify;
    HpiEngine              mHpi;
	IUsesHpiControl	&	   HpiControl;
	CoffThread             Coff;
    Trace                  Debug;

    void  Open(ii32 * port_memory_addr);

    void  HandleMemWrite(MemWriteEvent & event);
    void  HandleOnProgress(ProcessProgressEvent & event);
    void  HandleOnLoadMsg(ProcessStatusEvent & event);
    void  HandleOnComplete(ProcessCompletionEvent & event);

    // Support Methods
    void  Cold();
};

//===========================================================================
//  CLASS IUsesRestrictedHpiCpu -- User Interface to mix-in onboard CPU & Hpi
//===========================================================================

class IUsesRestrictedHpiCpu : public IUsesStandardHpiCpu
{
public:
    //
    // Methods
    IUsesRestrictedHpiCpu(IUsesHpiControl & hpic,
		                  HpiEngineAddresses & HpiAddr,
                          HpiCeSpaceValues & HpiCe);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative

#endif
