//  BaseboardInterfaces_Mb.h
//
//  Contains some common interfaces for baseboard objects
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef BaseboardInterfaces_MbH
#define BaseboardInterfaces_MbH

#include <ProcessEvents_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class CoffFile;
class HpiEngine;

//===========================================================================
//  CLASS IUsesOnboardCpu -- User Interface to mix-in onboard CPU
//===========================================================================
//  Standard Interface for Onboard CPU support...

class IUsesOnboardCpu
{
public:
    virtual ~IUsesOnboardCpu()
        {  }

    //
    // Methods
    virtual void  Boot() = 0;
    virtual void  DownloadCoff(const std::string & Filename) = 0;
    
    virtual CoffFile & CoffImage() = 0;

    // Events
    OpenWire::EventHandler<ProcessCompletionEvent>          OnBeforeCoffLoad;
    OpenWire::ThunkedEventHandler<ProcessProgressEvent>     OnCoffLoadProgress;
    OpenWire::ThunkedEventHandler<ProcessStatusEvent>       OnCoffLoadMessage;
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>   OnCoffLoadComplete;
};

//===========================================================================
//  CLASS IUsesOnboardHpiCpu -- User Interface to mix-in onboard CPU & Hpi
//===========================================================================
//  Standard Interface for Onboard CPU support...

class IUsesOnboardHpiCpu : public IUsesOnboardCpu
{
public:
    //
    // Methods
    virtual HpiEngine & Hpi() = 0;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative

#endif

