// BinFile_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#include <BinFile_Mb.h>
#include <fstream>
#include <sstream>
#include <Exception_Mb.h>

namespace   Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)	
	#pragma warning(disable : 4996)		
#endif

//===========================================================================
// CLASS CIIBinFileException  --  BIN Image File exception base
//===========================================================================

class CIIBinFileException : public MalibuException
{
public:
    virtual const char* what() const throw() {return "class CIIBinFileException";}
};

//===========================================================================
// CLASS CIIBinFileErrorException  --  BIN Image File  exception
//===========================================================================

class CIIBinFileErrorException : public CIIBinFileException
{
public:
    virtual const char* what() const throw() {return "class CIIBinFileErrorException";}
};

//===========================================================================
// CLASS CIIBinFileFormatException  --  BIN Image File exception
//===========================================================================

class CIIBinFileFormatException : public CIIBinFileException
{
public:
    virtual const char* what() const throw() {return "class CIIBinFileFormatException";}
};

//===========================================================================
// CLASS BinFile  --  Motorola S-record file reader
//===========================================================================


//---------------------------------------------------------------------------
//  BinFile::BinFile() --  Ctor
//---------------------------------------------------------------------------

BinFile::BinFile(std::string&  filename) 
	: FFileName(filename)
{
}

//---------------------------------------------------------------------------
//  BinFile::~BinFile() --  Dtor
//---------------------------------------------------------------------------

BinFile::~BinFile()
{
}

//---------------------------------------------------------------------------
//  BinFile::GetChecksum() --  Calculate 32-bit checksum
//---------------------------------------------------------------------------

int BinFile::GetChecksum() const
{
    int         Result = 0;
    size_t      Elements = Image.size() / sizeof(int);
    const int*  p = reinterpret_cast < const int * > (Image.data());
    for (size_t i = 0; i < Elements; ++i)
        Result += *p++;

    return Result;
}

//-----------------------------------------------------------------------------
//  BinFile::FileExists() -- return true if file exists...
//-----------------------------------------------------------------------------

bool BinFile::FileExists(std::string filename)
{
    std::ifstream in(filename.c_str(), std::ios_base::binary);

    return in.good();
}

//-----------------------------------------------------------------------------
//  BinFile::LoadFromFile() -- Fill string list from the file
//-----------------------------------------------------------------------------

void BinFile::LoadFromFile()
{
    std::ifstream in(FFileName.c_str(), std::ios_base::binary);

    if (!FileExists(FFileName))
        throw CIIBinFileErrorException();   // ("Bin file not found!");

    in.seekg (0, std::ios::end);
    std::streamoff length = in.tellg();
    in.seekg (0, std::ios::beg);

    Image.erase();

    std::streamoff count = 0;
    while (!in.eof())
        {
        //  extract blocks from the file and append to Image
        char    bk[512];
        in.read(bk, 512);
        Image.append(bk, static_cast<unsigned int>(in.gcount()));

        // Progress indicator
        count += in.gcount();
		ProcessProgressEvent ppe(static_cast<int>((count * 100)/length));
        OnProgress.Execute(ppe);
        }
	
	ProcessProgressEvent ppe100(100);
	ProcessCompletionEvent pce;

    OnProgress.Execute(ppe100);
    OnComplete.Execute(pce);
}

#ifdef _MSC_VER
	#pragma warning(pop)	
#endif

} // namespace Innovative

