// Binview_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//



#include <BinView_Mb.h>
#include <StringSupport_Mb.h>
#include <SystemSupport_Mb.h>
#if defined(WIN32) || defined(WIN64)
#include <Mailslot_Win.h>
#include <Registry_Win.h>
#include <Shellapi.h>
#endif


namespace Innovative
{

class BinViewImpl
{
public:
#if defined(WIN32) || defined(WIN64)
	BinViewImpl()
		: hProcess(0), FOpen(false)  {  }
	~BinViewImpl()
		{  Close();  }

	bool        Open(const std::string name)
		{
		if (FOpen)
			return true;

		FOpen = Mail.OpenClient(name);
		return FOpen;
		}
	void        Close()
		{
		if (FOpen)
			Mail.CloseClient();
		FOpen = false;
		}

	// Data
	MailSlot    Mail;
	HANDLE      hProcess;
	bool        FOpen;

#else
    BinViewImpl()  {  }
#endif	
};

namespace BinViewOptions
{

//========================================================================
//  CLASS TimeOptions -- Common options for BinView plots
//========================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

TimeOptions::TimeOptions()
    : FLowerLimit(-32767), FUpperLimit(32767), FScaleMode(sAuto),
	  FScaleXType(sLinear), FScaleYType(sLinear), FCursorMode(cmZoom),
	  FBreak(100), FAnnotateSeams(false), FSeamSize(16384),
	  FEnableDigitalView(false), FDigitalViewChannel(0)
{
}

//---------------------------------------------------------------------------
// TimeOptions::Save() -
//---------------------------------------------------------------------------

void TimeOptions::Save(IniFile & Ini)
{
	Ini.Section = "Graph";
	Ini.Write("Lower Limit", FLowerLimit);
	Ini.Write("Upper Limit", FUpperLimit);
	Ini.Write("Y Axis Scaling", FScaleMode);
	Ini.Write("X Axis Scale", FScaleXType);
	Ini.Write("Y Axis Scale", FScaleYType);
	Ini.Write("Cursor Mode", FCursorMode);
	Ini.Write("TimeTitle", FTimeTitle);
	Ini.Write("ConstellationTitle", FConstellationTitle);

	Ini.Section = "Data";
	Ini.Write("Break", FBreak);
	Ini.Write("Annotate Seams", FAnnotateSeams);
	Ini.Write("Seam Size", FSeamSize);
	Ini.Write("Enable Digital View", FEnableDigitalView);
	Ini.Write("Digital View Channel", FDigitalViewChannel);
}

//---------------------------------------------------------------------------
// TimeOptions::Restore() -
//---------------------------------------------------------------------------

void TimeOptions::Restore(IniFile & Ini)
{
	Ini.Section = "Graph";
	FLowerLimit = Ini.Read("Lower Limit", FLowerLimit);
	FUpperLimit = Ini.Read("Upper Limit", FUpperLimit);
	FScaleMode = (IIScaleMode)Ini.Read("Y Axis Scaling", FScaleMode);
	FScaleXType = (IIScaleType)Ini.Read("X Axis Scale", FScaleXType);
	FScaleYType = (IIScaleType)Ini.Read("Y Axis Scale", FScaleYType);
	FCursorMode = (IICursorMode)Ini.Read("Cursor Mode", FCursorMode);
	FTimeTitle = Ini.Read("TimeTitle", FTimeTitle);
	FConstellationTitle = Ini.Read("ConstellationTitle", FConstellationTitle);

	Ini.Section = "Data";
	FBreak = static_cast<float>(Ini.Read("Break", FBreak));
	FAnnotateSeams = Ini.Read("Annotate Seams", FAnnotateSeams);
	FSeamSize = Ini.Read("Seam Size", FSeamSize);
	FEnableDigitalView = Ini.Read("Enable Digital View", FEnableDigitalView);
	FDigitalViewChannel = Ini.Read("Digital View Channel", FDigitalViewChannel);
}

//========================================================================
//  CLASS FttOptions -- Common options for BinView plots
//========================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

FftOptions::FftOptions()
	: FLowerLimit(-150), FUpperLimit(0), FScaleMode(sAuto),
	  FScaleXType(sLinear), FScaleYType(sLog), FCursorMode(cmZoom),
	  FPoints(p1024), FWindow(wBlackmann), FAverage(1),
	  FLeftExtent(0), FRightExtent(0), FCriteria(cSnr), FCriteriaThreshold(75)
{
}

//---------------------------------------------------------------------------
// FftOptions::Save() - Save all properties to BDD
//---------------------------------------------------------------------------

void FftOptions::Save(IniFile & Ini)
{
	Ini.Section = "Graph";
	Ini.Write("FFT Lower Limit", FLowerLimit);
	Ini.Write("FFT Upper Limit", FUpperLimit);
	Ini.Write("FFT Y Axis Scaling", FScaleMode);
	Ini.Write("FFT X Axis Scale", FScaleXType);
	Ini.Write("FFT Y Axis Scale", FScaleYType);
	Ini.Write("FFT Cursor Mode", FCursorMode);
	Ini.Write("FFT Points", FPoints);
	Ini.Write("FFT Window", FWindow);
	Ini.Write("FFT Average", FAverage);
	Ini.Write("FFT Left Extent", FLeftExtent);
	Ini.Write("FFT Right Extent", FRightExtent);
	Ini.Write("FftTitle", FFftTitle);
	Ini.Write("CfftTitle", FCfftTitle);

	Ini.Section = "Data";
	Ini.Write("Criteria", FCriteria);
	Ini.Write("Criteria Threshold", FCriteriaThreshold);
}

//---------------------------------------------------------------------------
// FftOptions::Restore() - Restore all properties from BDD
//---------------------------------------------------------------------------

void FftOptions::Restore(IniFile & Ini)
{
	Ini.Section = "Graph";
	FLowerLimit = Ini.Read("FFT Lower Limit", FLowerLimit);
	FUpperLimit = Ini.Read("FFT Upper Limit", FUpperLimit);
	FScaleMode = (IIScaleMode)Ini.Read("FFT Y Axis Scaling", FScaleMode);
	FScaleXType = (IIScaleType)Ini.Read("FFT X Axis Scale", FScaleXType);
	FScaleYType = (IIScaleType)Ini.Read("FFT Y Axis Scale", FScaleYType);
	FCursorMode = (IICursorMode)Ini.Read("FFT Cursor Mode", FCursorMode);
	FPoints = (IIPoints)Ini.Read("FFT Points", FPoints);
	FWindow = (IIWindow)Ini.Read("FFT Window", FWindow);
	FAverage = Ini.Read("FFT Average", FAverage);
	FLeftExtent = Ini.Read("FFT Left Extent", FLeftExtent);
	FRightExtent = Ini.Read("FFT Right Extent", FRightExtent);
	FFftTitle = Ini.Read("FftTitle", FFftTitle);
	FCfftTitle = Ini.Read("CfftTitle", FCfftTitle);

	Ini.Section = "Data";
	FCriteria = (IICriteria)Ini.Read("Criteria", FCriteria);
	FCriteriaThreshold = static_cast<float>(Ini.Read("Criteria Threshold", FCriteriaThreshold));
}

//========================================================================
//  CLASS TextOptions -- Common options for BinView plots
//========================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

TextOptions::TextOptions()
	: FAddressFormat(fDecimal)
{
//	FDataFormat = "%11.3f";
}

//---------------------------------------------------------------------------
// TextOptions::Save() - Save all properties to BDD
//---------------------------------------------------------------------------

void TextOptions::Save(IniFile & Ini)
{
	Ini.Section = "Data";
	Ini.Write("Address Format", FAddressFormat);
    Ini.Write("Data Format", FDataFormat);
}

//---------------------------------------------------------------------------
// TextOptions::Restore() - Restore all properties from BDD
//---------------------------------------------------------------------------

void TextOptions::Restore(IniFile & Ini)
{
    Ini.Section = "Data";
    FAddressFormat = (IIAddressFormat)Ini.Read("Address Format", FAddressFormat);
    FDataFormat = Ini.Read("Data Format", FDataFormat);
}

//========================================================================
//  CLASS SystemOptions -- Common options for BinView plots
//========================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

SystemOptions::SystemOptions()
	: FSource(sFile), FBoardType(ibNone), FTarget(0), FUpdateRate(ms1000),
	  FScsiDevice(-1), FScsiFileId(-1), FActivePage(pTime),
      FActiveTimePage(pAmplitude), FActiveFrequencyPage(pReal),
      FTop(0), FLeft(0), FHeight(0), FWidth(0)
{
    FServerSlotName = "BinView";

#if defined(WIN32) || defined(WIN64)
	UserRegistry Reg("BinView", "Innovative Integration");
	FBinViewPath = Reg.Read("Application", ".");
#else
	FBinViewPath = ".";
#endif
}

//---------------------------------------------------------------------------
// SystemOptions::Save() - Save all properties to BDD
//---------------------------------------------------------------------------

void SystemOptions::Save(IniFile & Ini)
{
    Ini.Section = "System";
    Ini.Write("Board Type", FBoardType);
    Ini.Write("Target", FTarget);
    Ini.Write("Update Rate", FUpdateRate);
    Ini.Write("Active Page", (int)FActivePage);
    Ini.Write("Active Time Page", (int)FActiveTimePage);
    Ini.Write("Active Frequency Page", (int)FActiveFrequencyPage);
    Ini.Write("Source", FSource);
    Ini.Write("Server Slot Name", FServerSlotName);
    Ini.Section = "Scsi";
    Ini.Write("Scsi Device", FScsiDevice);
    Ini.Write("Scsi File ID", FScsiFileId);
    if (FHeight)
        {
        Ini.Write("Top", FTop);
        Ini.Write("Left", FLeft);
        Ini.Write("Height", FHeight);
        Ini.Write("Width", FWidth);
        }
}

//---------------------------------------------------------------------------
// SystemOptions::Restore() - Restore all properties from BDD
//---------------------------------------------------------------------------

void SystemOptions::Restore(IniFile & Ini)
{
    Ini.Section = "System";
    FBoardType = (IIBoardType)Ini.Read("Board Type", FBoardType);
    FTarget = Ini.Read("Target", FTarget);
    FUpdateRate = (IIUpdateRate)Ini.Read("Update Rate", FUpdateRate);
    FActivePage = (IIActivePage)Ini.Read("Active Page", (int)FActivePage);
    FActiveTimePage = (IIActiveTimePage)Ini.Read("Active Time Page", (int)FActiveTimePage);
    FActiveFrequencyPage = (IIActiveFrequencyPage)Ini.Read("Active Frequency Page", (int)FActiveFrequencyPage);
    FSource = (IISource)Ini.Read("Source", FSource);
    FServerSlotName = Ini.Read("Server Slot Name", FServerSlotName);
    Ini.Section = "Scsi";
    FScsiDevice = Ini.Read("Scsi Device", FScsiDevice);
    FScsiFileId = Ini.Read("Scsi File ID", FScsiFileId);
    FTop = Ini.Read("Top", FTop);
    FLeft = Ini.Read("Left", FLeft);
    FHeight = Ini.Read("Height", FHeight);
    FWidth = Ini.Read("Width", FWidth);
}

} // end namespace BinViewOptions

//========================================================================
//  CLASS BinView -- Binview INI file generator class
//========================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

BinView::BinView()
    : Impl(new BinViewImpl),
	  FBigEndian(false),
      FBinFile("Plot.bin"),
      FChannelEnable(0),
      FChannels(1),
      FCurrentOffset(0),
      FCurrentChannel(0),
      FDataSpan(300),
      FDataType(tInt),
      FDcOffset(0),
      FDevices(1),
      FEventHeaderBytes(0),
      FEventTrailerBytes(0),
      FGang(1),
      FGraphEnable(0),
      FHeaderBytes(0),
      FInputSpan(0.0),
      FLeap(100),
      FNullDataValue(.123456e-7f),
      FPolarity(pSigned),
      FReadOnly(true),
      FSampleRate(0),
      FSamples(1000),
      FScalingEnabled(false),
      FSignificantBits(16),
      FTrailerBytes(0),
      FUnits("mV"), 
      FComplex(false)
{
}

//---------------------------------------------------------------------------
//  Destructor
//---------------------------------------------------------------------------

BinView::~BinView()
{
    Quit();

    delete Impl;
    Impl = 0;
}

//---------------------------------------------------------------------------
// BinView::Save() - Save all properties to BDD
//---------------------------------------------------------------------------

void BinView::Save()
{
    PathSpec spec(FBinFile);
    spec.Ext(".bdd");
    IniFile Ini(spec.Full());

    //CodeSite->SendMsg("Update");
    FTimeOptions.Save(Ini);
    FFftOptions.Save(Ini);
    FTextOptions.Save(Ini);
    FSystemOptions.Save(Ini);

    Ini.Section = "Data";
    Ini.Write("Channel Enable", (ii64)FChannelEnable);
    Ini.Write("Header Bytes", FHeaderBytes);
    Ini.Write("Trailer Bytes", FTrailerBytes);
    Ini.Write("Event Header", FEventHeaderBytes);
    Ini.Write("Event Trailer", FEventTrailerBytes);
    Ini.Write("Sample Rate", FSampleRate);
    Ini.Write("Significant Bits", FSignificantBits);
    Ini.Write("Data Polarity", FPolarity);
    Ini.Write("Data Type", FDataType);
    Ini.Write("Data Span", FDataSpan);
    Ini.Write("Enable Scaling", FScalingEnabled);
    Ini.Write("Input Span", FInputSpan);
    Ini.Write("Units", FUnits);
    Ini.Write("Dc Offset", FDcOffset);
    Ini.Write("Size", FSamples);
    Ini.Write("Read Only", FReadOnly);
    Ini.Write("Big Endian", FBigEndian);
    Ini.Write("Number of Channels", FChannels);
    Ini.Write("Number of Devices", FDevices);
    Ini.Write("Current Offset", FCurrentOffset);
    Ini.Write("Current Channel", FCurrentChannel);
    Ini.Write("File Name", FBinFile);
    Ini.Write("Complex", FComplex);

    Ini.Section = "Graph";
    Ini.Write("Graph Enable", (ii64)FGraphEnable);
    Ini.Write("Leap", FLeap);
    Ini.Write("NullDataValue", FNullDataValue);

    Ini.Section = "Comments";
    Ini.DeleteSection();
    Ini.Write("CommentLines", static_cast<int>(FComments.Count()));
    Ini.Write("CommentLine", FComments, true);
    Ini.Write("SeeAlso", FSeeAlso);

    Ini.Update();
}

//---------------------------------------------------------------------------
// BinView::Restore() - Restore all properties from BDD
//---------------------------------------------------------------------------

void BinView::Restore()
{
    PathSpec spec(FBinFile);
    spec.Ext(".bdd");
    IniFile Ini(spec.Full());

    //CodeSite->SendMsg("Restore");
    FTimeOptions.Restore(Ini);
    FFftOptions.Restore(Ini);
    FTextOptions.Restore(Ini);
    FSystemOptions.Restore(Ini);

    Ini.Section = "Data";
    FChannelEnable = Ini.Read("Channel Enable", (ii64)FChannelEnable);
    FHeaderBytes = Ini.Read("Header Bytes", FHeaderBytes);
    FTrailerBytes = Ini.Read("Trailer Bytes", FTrailerBytes);
    FEventHeaderBytes = Ini.Read("Event Header", FEventHeaderBytes);
    FEventTrailerBytes = Ini.Read("Event Trailer", FEventTrailerBytes);
    FSignificantBits = Ini.Read("Significant Bits", FSignificantBits);
    FPolarity = (IIPolarity)Ini.Read("Data Polarity", FPolarity);
    FDataSpan = Ini.Read("Data Span", FDataSpan);
    FScalingEnabled = Ini.Read("Enable Scaling", FScalingEnabled);
    FInputSpan = Ini.Read("Input Span", FInputSpan);
    FUnits = Ini.Read("Units", FUnits);
    FDcOffset = Ini.Read("Dc Offset", FDcOffset);
    FSamples = Ini.Read("Size", FSamples);
    FReadOnly = Ini.Read("Read Only", FReadOnly);
    FBigEndian = Ini.Read("Big Endian", FBigEndian);
    FChannels = Ini.Read("Number of Channels", FChannels);
    FDevices = Ini.Read("Number of Devices", FDevices);
    FCurrentOffset = Ini.Read("Current Offset", FCurrentOffset);
    FCurrentChannel = Ini.Read("Current Channel", FCurrentChannel);
    FSampleRate = Ini.Read("Sample Rate", FSampleRate);
    FComplex = Ini.Read("Complex", FComplex);

    Ini.Section = "Graph";
    FGraphEnable = Ini.Read("Graph Enable", FGraphEnable);
    FLeap = Ini.Read("Leap", FLeap);
    FNullDataValue = Ini.Read("NullDataValue", FNullDataValue);
    FDataType = (IIDataType)Ini.Read("Data Type", FDataType);

    Ini.Section = "Comments";
    FComments.Clear();
    Ini.Read("CommentLine", FComments, true);
    FSeeAlso = Ini.Read("SeeAlso", FSeeAlso);
}

//---------------------------------------------------------------------------
// BinView::Plot() -- Launch Binview application with specified BDD
//---------------------------------------------------------------------------

bool BinView::Plot()
{
    if (BinviewCommand(std::string("PLOT ") + BinFile()))
        return true;

    if (!FSystemOptions.BinViewPath().size())
        return false;

    PathSpec spec(BinFile());
    spec.Ext(".bdd");

    std::string File = FSystemOptions.BinViewPath();
	std::string Parms = spec.Full();

#if defined(WIN32) || defined(WIN64)

    SHELLEXECUTEINFO ExecInfo;
    memset(&ExecInfo, 0, sizeof(SHELLEXECUTEINFO));

    ExecInfo.cbSize = sizeof(SHELLEXECUTEINFO);
    ExecInfo.lpFile = File.c_str();
    ExecInfo.lpParameters = Parms.c_str();
	ExecInfo.lpDirectory = NULL;
	ExecInfo.lpVerb = NULL;
    ExecInfo.nShow = SW_NORMAL;
	ExecInfo.fMask = SEE_MASK_NOCLOSEPROCESS;
    ExecInfo.hwnd = NULL;
    ExecInfo.hInstApp = NULL;

    if (!ShellExecuteEx(&ExecInfo))
        return false;

    if(Impl)
		{
		Impl->hProcess = ExecInfo.hProcess;
		return (Impl->hProcess != 0);
		}
    else
        return false;

#else
	return true;
#endif
}

//---------------------------------------------------------------------------
// BinView::Quit() -- Close Binview application
//---------------------------------------------------------------------------

bool BinView::Quit()
{
#if defined(WIN32) || defined(WIN64)
	if (Impl->hProcess == 0)
		return true;

	bool result = BinviewCommand("QUIT");
	if (!result)
		return false;

	// Wait for a while for the process to shut down
	for (int i = 0; i < 40; ++i)
		{
		int state = WaitForSingleObject(Impl->hProcess, 100);
		if (state != WAIT_TIMEOUT)
			break;
		}

	Impl->Close();
	Impl->hProcess = 0;

	return result;

#else
	return true;
#endif
}

//---------------------------------------------------------------------------
// BinView::Refresh() -- Refreshes Binview graph
//---------------------------------------------------------------------------

bool BinView::Refresh()
{
	return BinviewCommand("REFRESH");
}

//---------------------------------------------------------------------------
// BinView::BinviewCommand() -- Issue command to Binview server
//---------------------------------------------------------------------------

bool BinView::BinviewCommand(std::string cmd)
{
#if defined(WIN32) || defined(WIN64)
	if (!Impl->Open(System().ServerSlotName()))
		return false;

	if (!Impl->Mail.Write(cmd))
		return false;
#endif

	return (cmd.size() ? true : false);
}

//---------------------------------------------------------------------------
// BinView::Running() -- Check if BV still running
//---------------------------------------------------------------------------

bool BinView::Running() const
{
	bool result = false;

#if defined(WIN32) || defined(WIN64)
	int state = WaitForSingleObject(Impl->hProcess, 0);
	result = (state == WAIT_TIMEOUT);
#endif

    return result;
}

} // namespace Innovative

//---------------------------------------------------------------------------
