//
// Binview_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Binview_MbH
#define Binview_MbH

#include <IniFile_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class BinView;
class BinViewImpl;

namespace BinViewOptions
{

//========================================================================
//  CLASS TimeOptions -- Common options for BinView plots
//========================================================================

class TimeOptions
{
    friend class Innovative::BinView;

public:
    enum IIScaleMode { sAuto, sManual };
    enum IIScaleType { sLinear, sLog };
    enum IICursorMode { cmZoom, cmMeasure };

    //
    // Constructor
    TimeOptions();

	// Properties
	int				LowerLimit() const
		{  return FLowerLimit;  }
	TimeOptions & 	LowerLimit(int limit)
		{  FLowerLimit=limit;  return *this;  }
		
	int				UpperLimit() const
		{  return FUpperLimit;  }
	TimeOptions & 	UpperLimit(int limit)
		{  FUpperLimit=limit;  return *this;  }
		
	IIScaleMode		ScaleMode() const
		{  return FScaleMode;  }
	TimeOptions & 	ScaleMode(IIScaleMode mode)
		{  FScaleMode=mode;  return *this;  }

	IIScaleType		ScaleXType() const
		{  return FScaleXType;  }
	TimeOptions & 	ScaleXType(IIScaleType type)
		{  FScaleXType=type;  return *this;  }

	IIScaleType		ScaleYType() const
		{  return FScaleYType;  }
	TimeOptions & 	ScaleYType(IIScaleType type)
		{  FScaleYType=type;  return *this;  }

	IICursorMode	CursorMode() const
		{  return FCursorMode;  }
	TimeOptions & 	CursorMode(IICursorMode mode)
		{  FCursorMode=mode;  return *this;  }

	float			Break() const
		{  return FBreak;  }
	TimeOptions & 	Break(float mag)
		{  FBreak=mag;  return *this;  }
		
	bool			AnnotateSeams() const
		{  return FAnnotateSeams;  }
	TimeOptions & 	AnnotateSeams(bool state)
		{  FAnnotateSeams=state;  return *this;  }

	int			SeamSize() const
		{  return FSeamSize;  }
	TimeOptions & 	SeamSize(int size)
		{  FSeamSize=size;  return *this;  }

	bool			EnableDigitalView() const
		{  return FEnableDigitalView;  }
	TimeOptions & 	EnableDigitalView(bool state)
		{  FEnableDigitalView=state;  return *this;  }

	int 			DigitalViewChannel() const
		{  return FDigitalViewChannel;  }
	TimeOptions & 	DigitalViewChannel(int value)
		{  FDigitalViewChannel=value;  return *this;  }

	std::string		TimeTitle() const
		{  return FTimeTitle;  }
	TimeOptions & 	TimeTitle(const std::string & value)
		{  FTimeTitle=value;  return *this;  }

	std::string		ConstellationTitle() const
		{  return FConstellationTitle;  }
	TimeOptions & 	ConstellationTitle(const std::string & value)
		{  FConstellationTitle=value;  return *this;  }

private:
	// Methods
    void Save(IniFile & Ini);
    void Restore(IniFile & Ini);

    // Data
    int                     FLowerLimit;
    int                     FUpperLimit;
    IIScaleMode             FScaleMode;
    IIScaleType             FScaleXType;
    IIScaleType             FScaleYType;
    IICursorMode            FCursorMode;
    float                   FBreak;
    bool                    FAnnotateSeams;
    int                     FSeamSize;
    bool                    FEnableDigitalView;
	int                     FDigitalViewChannel;
	std::string				FTimeTitle;
	std::string				FConstellationTitle;
};

//========================================================================
//  CLASS FftOptions -- Common options for BinView plots
//========================================================================

class FftOptions
{
    friend class Innovative::BinView;

public:
    enum IIScaleMode { sAuto, sManual };
    enum IIScaleType { sLinear, sLog };
    enum IICursorMode { cmZoom, cmMeasure };
    enum IIPoints { p128, p256, p512, p1024, p2048, p4096, p8192, p16384,
                    p32768, p65536, p131072, p262144, p524288, p1048576,
                    p2097152, p4194304  };
    enum IIWindow { wNone, wBartlett, wBlackmann, wHamming, wHanning,
                    wKaiser };
    enum IICriteria { cSnr, cSinad, cEnob, cSfdr, cThd };

    //
    // Constructor
    FftOptions();

	// Properties
	int				LowerLimit() const
		{  return FLowerLimit;  }
	FftOptions & 	LowerLimit(int limit)
		{  FLowerLimit=limit;  return *this;  }
		
	int				UpperLimit() const
		{  return FUpperLimit;  }
	FftOptions & 	UpperLimit(int limit)
		{  FUpperLimit=limit;  return *this;  }

	IIScaleMode		ScaleMode() const
		{  return FScaleMode;  }
	FftOptions & 	ScaleMode(IIScaleMode mode)
		{  FScaleMode=mode;  return *this;  }

	IIScaleType		ScaleXType() const
		{  return FScaleXType;  }
	FftOptions & 	ScaleXType(IIScaleType type)
		{  FScaleXType=type;  return *this;  }

	IIScaleType		ScaleYType() const
		{  return FScaleYType;  }
	FftOptions & 	ScaleYType(IIScaleType type)
		{  FScaleYType=type;  return *this;  }

	IICursorMode	CursorMode() const
		{  return FCursorMode;  }
	FftOptions & 	CursorMode(IICursorMode mode)
		{  FCursorMode=mode;  return *this;  }

	IICriteria		Criteria() const
		{  return FCriteria;  }
	FftOptions & 	Criteria(IICriteria criteria)
		{  FCriteria=criteria;  return *this;  }

	float			CriteriaThreshold() const
		{  return FCriteriaThreshold;  }
	FftOptions & 	CriteriaThreshold(float value)
		{  FCriteriaThreshold=value;  return *this;  }

	IIPoints		Points() const
		{  return FPoints;  }
	FftOptions & 	Points(IIPoints value)
		{  FPoints=value;  return *this;  }

	IIWindow		Window() const
		{  return FWindow;  }
	FftOptions & 	Window(IIWindow value)
		{  FWindow=value;  return *this;  }

	int				Average() const
		{  return FAverage;  }
	FftOptions & 	Average(int value)
		{  FAverage=value;  return *this;  }

	int				LeftExtent() const
		{  return FLeftExtent;  }
	FftOptions & 	LeftExtent(int value)
		{  FLeftExtent=value;  return *this;  }

	int				RightExtent() const
		{  return FRightExtent;  }
	FftOptions & 	RightExtent(int value)
		{  FRightExtent=value;  return *this;  }

	std::string		FftTitle() const
		{  return FFftTitle;  }
	FftOptions & 	FftTitle(const std::string & value)
		{  FFftTitle=value;  return *this;  }

	std::string		CfftTitle() const
		{  return FCfftTitle;  }
	FftOptions & 	CfftTitle(const std::string & value)
		{  FCfftTitle=value;  return *this;  }

private:
    // Methods
    void Save(IniFile & Ini);
    void Restore(IniFile & Ini);

    // Data
    int                     FLowerLimit;
    int                     FUpperLimit;
    IIScaleMode             FScaleMode;
    IIScaleType             FScaleXType;
    IIScaleType             FScaleYType;
    IICursorMode            FCursorMode;
    IIPoints                FPoints;
    IIWindow                FWindow;
    int                     FAverage;
    int                     FLeftExtent;
    int                     FRightExtent;
    IICriteria              FCriteria;
	float                   FCriteriaThreshold;
	std::string				FFftTitle;
	std::string				FCfftTitle;
};

//========================================================================
//  CLASS TextOptions -- Common options for BinView plots
//========================================================================

class TextOptions
{
	friend class Innovative::BinView;

public:
	enum IIAddressFormat { fDecimal, fHex };

	TextOptions();

	// Properties
	IIAddressFormat		AddressFormat() const
		{  return FAddressFormat;  }
	TextOptions & 		AddressFormat(IIAddressFormat value)
		{  FAddressFormat=value;  return *this;  }

	std::string			DataFormat() const
		{  return FDataFormat;  }
	TextOptions & 		DataFormat(const std::string & value)
		{  FDataFormat=value;  return *this;  }

private:
    // Methods
    void Save(IniFile & Ini);
    void Restore(IniFile & Ini);

    // Data
    IIAddressFormat         FAddressFormat;
    std::string             FDataFormat;
};

//========================================================================
//  CLASS SystemOptions -- Common options for BinView plots
//========================================================================

class SystemOptions 
{
    friend class Innovative::BinView;

public:
    enum IIBoardType { ibNone, ibAdc64, ibM44, ibPc31, ibPc32, ibPc44,
                       ibPci32, ibPci44, ibSbc31, ibSbc32, ibSbc54,
                       ibM62, ibQ62, ibSbc62, ibChico, ibLast };
    enum IIActivePage { pTime, pFrequency, pText, pSummary, pServer };
    enum IIActiveTimePage { pAmplitude, pConstellation };
    enum IIActiveFrequencyPage { pReal, pComplex };
    enum IISource { sFile, sMapped, sPacketFile, sVelo, sVita };
    enum IIUpdateRate { ms100, ms250, ms500, ms1000, ms2000 };

    SystemOptions();

	std::string         BinViewPath() const
		{  return FBinViewPath;  }
	SystemOptions &     BinViewPath(std::string)
		{  return *this;  }

    // Properties
	IIBoardType			BoardType() const
		{  return FBoardType;  }
	SystemOptions & 	BoardType(IIBoardType value)
		{  FBoardType=value;  return *this;  }
		
	int					Target() const
		{  return FTarget;  }
	SystemOptions & 	Target(int value)
		{  FTarget=value;  return *this;  }

	IIUpdateRate		UpdateRate() const
		{  return FUpdateRate;  }
	SystemOptions & 	UpdateRate(IIUpdateRate value)
		{  FUpdateRate=value;  return *this;  }
		
	IIActivePage		ActivePage() const
		{  return FActivePage;  }
	SystemOptions & 	ActivePage(IIActivePage value)
		{  FActivePage=value;  return *this;  }

	IIActiveTimePage	ActiveTimePage() const
		{  return FActiveTimePage;  }
	SystemOptions & 	ActiveTimePage(IIActiveTimePage value)
		{  FActiveTimePage=value;  return *this;  }

	IIActiveFrequencyPage	ActiveFrequencyPage() const
		{  return FActiveFrequencyPage;  }
	SystemOptions & 	ActiveFrequencyPage(IIActiveFrequencyPage value)
		{  FActiveFrequencyPage=value;  return *this;  }

	IISource			Source() const
		{  return FSource;  }
	SystemOptions & 	Source(IISource value)
		{  FSource=value;  return *this;  }

	int					ScsiDevice() const
		{  return FScsiDevice;  }
	SystemOptions & 	ScsiDevice(int value)
		{  FScsiDevice=value;  return *this;  }

	int					ScsiFileId() const
		{  return FScsiFileId;  }
	SystemOptions & 	ScsiFileId(int value)
		{  FScsiFileId=value;  return *this;  }

	std::string         ServerSlotName() const
		{  return FServerSlotName;  }
	SystemOptions & 	ServerSlotName(std::string value)
		{  FServerSlotName=value;  return *this;  }

	int					Top() const
		{  return FTop;  }
	SystemOptions & 	Top(int value)
		{  FTop=value;  return *this;  }

	int					Left() const
		{  return FLeft;  }
	SystemOptions & 	Left(int value)
		{  FLeft=value;  return *this;  }

	int					Height() const
		{  return FHeight;  }
	SystemOptions & 	Height(int value)
		{  FHeight=value;  return *this;  }

	int					Width() const
		{  return FWidth;  }
	SystemOptions & 	Width(int value)
		{  FWidth=value;  return *this;  }

private:
    // Methods
    void Save(IniFile & Ini);
    void Restore(IniFile & Ini);

    // Data
    IISource                    FSource;
    IIBoardType                 FBoardType;
    int                         FTarget;
    IIUpdateRate                FUpdateRate;
    int                         FScsiDevice;
    int                         FScsiFileId;
    std::string                 FBinViewPath;
    IIActivePage                FActivePage;
    IIActiveTimePage            FActiveTimePage;
    IIActiveFrequencyPage       FActiveFrequencyPage;
    std::string                 FServerSlotName;
    int                         FTop;
    int                         FLeft;
    int                         FHeight;
    int                         FWidth;
};

} // end namespace BinViewOptions

//========================================================================
//  CLASS BinView -- Binview file viewer access class
//========================================================================

class BinView
{
public:
    enum IIPolarity { pUnsigned, pSigned };
    enum IIDataType { tInt, tIeeeFloat, tTiFloat, tALaw, tuLaw };

    // Ctor
    BinView();
    ~BinView();

	// Methods
    bool                    Plot();
    bool                    Quit();
    bool                    Refresh();
	bool                    Running() const;

    void                    Save();
    void                    Restore();

	BinViewOptions::TimeOptions &			Time()
		                            {  return FTimeOptions;  }
	BinViewOptions::FftOptions &			Fft()
		                            {  return FFftOptions;  }
	BinViewOptions::TextOptions &			Text()
		                            {  return FTextOptions;  }    
	BinViewOptions::SystemOptions &			System()
		                            {  return FSystemOptions;  }    

    // Properties
	bool				GraphEnabled(int i) const
		{  return FGraphEnable & ((ii64)1 << i) ? true : false;  }
	BinView & 			GraphEnabled(int i, bool value)
		{
        int mask = 1 << i;
        FGraphEnable &= ~mask;
        FGraphEnable |= value ? mask : 0;
        return *this;
        }

	bool				ChannelEnabled(int i) const
		{  return FChannelEnable & ((ii64)1 << i) ? true : false;  }
	BinView & 			ChannelEnabled(int i, bool value)
		{
        int mask = 1 << i;
        FChannelEnable &= ~mask;
        FChannelEnable |= value ? mask : 0;
        return *this;
        }

	std::string			BinFile() const
		{  return FBinFile;  }
	BinView & 			BinFile(std::string value)
		{  FBinFile=value;  return *this;  }

	int					Leap() const
		{  return FLeap;  }
	BinView & 			Leap(int value)
		{  FLeap=value;  return *this;  }

	float				NullDataValue() const
		{  return FNullDataValue;  }
	BinView & 			NullDataValue(float value)
		{  FNullDataValue=value;  return *this;  }

	int					HeaderBytes() const
		{  return FHeaderBytes;  }
	BinView & 			HeaderBytes(int value)
		{  FHeaderBytes=value;  return *this;  }

	int					TrailerBytes() const
		{  return FTrailerBytes;  }
	BinView & 			TrailerBytes(int value)
		{  FTrailerBytes=value;  return *this;  }

	int					EventHeaderBytes() const
		{  return FEventHeaderBytes;  }
	BinView & 			EventHeaderBytes(int value)
		{  FEventHeaderBytes=value;  return *this;  }

	int					EventTrailerBytes() const
		{  return FEventTrailerBytes;  }
	BinView & 			EventTrailerBytes(int value)
		{  FEventTrailerBytes=value;  return *this;  }

	float				SampleRate() const
		{  return FSampleRate;  }
	BinView & 			SampleRate(float value)
		{  FSampleRate=value;  return *this;  }

	int					SignificantBits() const
		{  return FSignificantBits;  }
	BinView & 			SignificantBits(int value)
		{  FSignificantBits=value;  return *this;  }

	IIPolarity			Polarity() const
		{  return FPolarity;  }
	BinView & 			Polarity(IIPolarity value)
		{  FPolarity=value;  return *this;  }

	IIDataType			DataType() const
		{  return FDataType;  }
	BinView & 			DataType(IIDataType value)
		{  FDataType=value;  return *this;  }

	int					DataSpan() const
		{  return FDataSpan;  }
	BinView & 			DataSpan(int value)
		{  FDataSpan=value;  return *this;  }

	bool				ScalingEnabled() const
		{  return FScalingEnabled;  }
	BinView & 			ScalingEnabled(bool value)
		{  FScalingEnabled=value;  return *this;  }

	float				InputSpan() const
		{  return FInputSpan;  }
	BinView & 			InputSpan(float value)
		{  FInputSpan=value;  return *this;  }

	std::string			Units() const
		{  return FUnits;  }
	BinView & 			Units(std::string value)
		{  FUnits=value;  return *this;  }

	float				DcOffset() const
		{  return FDcOffset;  }
	BinView & 			DcOffset(float value)
		{  FDcOffset=value;  return *this;  }

	int					Samples() const
		{  return FSamples;  }
	BinView & 			Samples(int value)
		{  FSamples=value;  return *this;  }

	bool				ReadOnly() const
		{  return FReadOnly;  }
	BinView & 			ReadOnly(bool value)
		{  FReadOnly=value;  return *this;  }

	bool				BigEndian() const
		{  return FBigEndian;  }
	BinView & 			BigEndian(bool value)
		{  FBigEndian=value;  return *this;  }

	int					Channels() const
		{  return FChannels;  }
	BinView & 			Channels(int value)
		{  FChannels=value;  return *this;  }

	int					Devices() const
		{  return FDevices;  }
	BinView & 			Devices(int value)
		{  FDevices=value;  return *this;  }

	int					CurrentOffset() const
		{  return FCurrentOffset;  }
	BinView & 			CurrentOffset(int value)
		{  FCurrentOffset=value;  return *this;  }

	int					CurrentChannel() const
		{  return FCurrentChannel;  }
	BinView & 			CurrentChannel(int value)
		{  FCurrentChannel=value;  return *this;  }

	int					Gang() const
		{  return FGang;  }
	BinView & 			Gang(int value)
		{  FGang=value;  return *this;  }

	bool				Complex() const
		{  return FComplex;  }
	BinView & 			Complex(bool value)
		{  FComplex=value;  return *this;  }

    StringList &        Comments()
        {  return FComments;  }

private:
    // Fields
	BinViewOptions::TimeOptions				FTimeOptions;
	BinViewOptions::FftOptions				FFftOptions;
	BinViewOptions::TextOptions				FTextOptions;
	BinViewOptions::SystemOptions			FSystemOptions;
    BinViewImpl *                           Impl;

    bool                    FBigEndian;
    std::string             FBinFile;
    ii64                    FChannelEnable;
    int                     FChannels;
    StringList              FComments;
    int                     FCurrentOffset;
    int                     FCurrentChannel;
    int                     FDataSpan;
    IIDataType              FDataType;
    float                   FDcOffset;
    int                     FDevices;
	int                     FEventHeaderBytes;
    int                     FEventTrailerBytes;
    int						FGang;
    ii64                    FGraphEnable;
    int                     FHeaderBytes;
    float                   FInputSpan;
    int                     FLeap;
    float                   FNullDataValue;
    IIPolarity              FPolarity;
    bool                    FReadOnly;
    float                   FSampleRate;
    int                     FSamples;
    bool                    FScalingEnabled;
    std::string             FSeeAlso;
    int                     FSignificantBits;
    int                     FTrailerBytes;
    std::string             FUnits;
	bool					FComplex;

	// Methods
    bool                    BinviewCommand(std::string cmd);

	//class TermBox &		    Term();
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
