//  BitFile_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <BitFile_Mb.h>
#include <Exception_Mb.h>
#include <fstream>
#include <cstring>
#include <Magic_Mb.h>

/* first 13 bytes of a bit file */
static unsigned char preamble[] = {0, 9, 15, 240, 15, 240, 15, 240, 15, 240, 0, 0, 1};
const int preamble_size = sizeof(preamble);

using namespace std;

namespace   Innovative
{

//---------------------------------------------------------------------------
//  BitFile::BitFile() --
//---------------------------------------------------------------------------

BitFile::BitFile()
    : section("?")
{
}

//---------------------------------------------------------------------------
//  BitFile::~BitFile() --
//---------------------------------------------------------------------------

BitFile::~BitFile()
{
}

//---------------------------------------------------------------------------
//  BitFile::Parse() -- Parse the entire header, load image buffer
//---------------------------------------------------------------------------

void BitFile::Parse()
{
	file.open(FFileName.c_str(), std::ios_base::in | std::ios::binary | std::ios::ate);

    if (file.fail())
        {
        Error("Unable to read BIT file");
        return;
        }

	filesize = file.tellg();
	file.seekg(0);

	// Preable
	Preamble();
    Progress(10);

	// Filename
	ReadSection(FTag, ReadSectionHeader());
    Progress(20);

	// Part name
	ReadSection(FPart, ReadSectionHeader());
    Progress(30);

	// Creation date
	ReadSection(FDate, ReadSectionHeader());
    Progress(40);

	// Creation time
	ReadSection(FTime, ReadSectionHeader());
    Progress(50);

	// Bitstream length
	ReadLength();
    Progress(100);

    file.close();

    Complete(0);
}

//---------------------------------------------------------------------------
// BitFile::Preamble
//
// Read the first 13 bytes of the bit file.  Discards the 13 bytes but
// verifies that they are correct.
//---------------------------------------------------------------------------

void BitFile::Preamble()
{
    CharArray temp(preamble_size);

	/* read header */
	file.read(reinterpret_cast<char*>(&temp[0]), preamble_size);
	if (file.fail())
		Error("Unable to read BIT preamble");

	/* verify header is correct */
	int t = memcmp(&temp[0], preamble, preamble_size);
	if (t)
		Error("BIT preamble mismatch");
}

//---------------------------------------------------------------------------
// BitFile::ReadSectionHeader()  --  Read the header of a bit file section.
//    The section letter is placed in string section
//---------------------------------------------------------------------------

int BitFile::ReadSectionHeader()
{
	// get section letter
	file.read(&section[0], 1);
	if (file.fail())
		Error("Error reading section letter");

	// read length
	char lenbuf[2];
	file.read(lenbuf, 2);
	if (file.fail())
		Error("Invalid section length");

	// convert 2-byte length to an int
	return (((int)lenbuf[0]) <<8) | lenbuf[1];
}

//---------------------------------------------------------------------------
// BitFile::ReadSection()  --  Read section into array
//---------------------------------------------------------------------------

void BitFile::ReadSection(CharArray & buf, unsigned int /*length*/)
{
	// get section data
	file.read(reinterpret_cast<char*>(&buf[0]), buf.size());
	if (file.fail() || (buf[buf.size()-1] != 0))
		Error("Unable to read bit image");
}

//---------------------------------------------------------------------------
// BitFile::ReadSection()  --  Read section into string
//---------------------------------------------------------------------------

void BitFile::ReadSection(string & s, unsigned int length)
{
    CharArray buf(length);

	// get section data
	file.read(reinterpret_cast<char*>(&buf[0]), length);
	if (file.fail() || (buf[length-1] != 0))
		Error("Unable to read bit header");

    s = std::string(reinterpret_cast<char*>(&buf[0]));
}

//---------------------------------------------------------------------------
// BitFile::ReadLength()  --  Return length of section
//---------------------------------------------------------------------------

int BitFile::ReadLength()
{
	CharArray buf(4);
	unsigned int length;

	// get section, make sure it's "e"
	file.read(&section[0], 1);
	if (file.fail() || (section[0] != 'e'))
		Error("Invalid BIT image ID");

	// get length
	file.read(reinterpret_cast<char*>(&buf[0]), 4);
	if (file.fail())
		Error("Invalid BIT image length");

	// convert 4-byte length to an int
	length = (((int)buf[0]) <<24) | (((int)buf[1]) <<16)
	         | (((int)buf[2]) <<8) | buf[3];

    Buffer.resize(length);
    file.read(reinterpret_cast<char*>(&Buffer[0]), length);
    if (file.fail())
		Error("Unable to read BIT image");

    for (unsigned int i = 0; i < length; ++i)
        Buffer[i] = BitReverse(Buffer[i]);

	return length;
}

} // namespace
