//  BufferDatagrams_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef BufferDatagrams_MbH
#define BufferDatagrams_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>
#include <complex>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS Complex  -- alias of real complex class
//==============================================================================
typedef std::complex<float> Complex;

//==============================================================================
//  CLASS IntegerDG -- Datagram wrapping Buffer; provides access as 'int'
//==============================================================================

class IntegerDG : public AccessDatagram<int>
{
public:
    typedef AccessDatagram<int> BaseClass;
    //
    //  Member Functions
    IntegerDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IntegerDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
        
private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS UIntegerDG -- Datagram wrapping Buffer; provides access as 'unsigned int'
//==============================================================================

class UIntegerDG : public AccessDatagram<unsigned int>
{
public:
    typedef AccessDatagram<unsigned int> BaseClass;
    //
    //  Member Functions
    UIntegerDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    UIntegerDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
        
private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS FloatDG -- Datagram wrapping a buffer; provides access as 'float'
//==============================================================================

class FloatDG : public AccessDatagram<float>
{
public:
    typedef AccessDatagram<float> BaseClass;
    //
    //  Member Functions
    FloatDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    FloatDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }

private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS DoubleDG -- Datagram wrapping a buffer; provides access as 'double'
//==============================================================================

class DoubleDG : public AccessDatagram<double>
{
public:
    typedef AccessDatagram<double> BaseClass;
    //
    //  Member Functions
    DoubleDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    DoubleDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }

private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS ComplexDG -- Datagram wrapping a buffer; provides access as 'complex'
//==============================================================================

class ComplexDG : public AccessDatagram<Complex>
{
public:
    typedef AccessDatagram<Complex> BaseClass;
    //
    //  Member Functions
    ComplexDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    ComplexDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }

    Buffer     Real() const;
    Buffer     Imaginary() const;

private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS ShortDG -- Datagram wrapping a buffer; provides access as 'short'
//==============================================================================

class ShortDG : public AccessDatagram<short>
{
public:
    typedef AccessDatagram<short> BaseClass;
    //
    //  Member Functions
    ShortDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    ShortDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
        
private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS UShortDG -- Datagram wrapping a buffer; provides access as 'unsigned short'
//==============================================================================

class UShortDG : public AccessDatagram<unsigned short>
{
public:
    typedef AccessDatagram<unsigned short> BaseClass;
    //
    //  Member Functions
    UShortDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    UShortDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
        
private:
    Buffer & FBuffer;
};

//==============================================================================
//  CLASS CharDG -- Datagram wrapping a buffer; provides access as 'char'
//==============================================================================

class CharDG : public AccessDatagram<char>
{
public:
    typedef AccessDatagram<char> BaseClass;
    //
    //  Member Functions
    CharDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    CharDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
        
private:
    Buffer & FBuffer;
};


//==============================================================================
//  CLASS UCharDG -- Datagram wrapping a buffer; provides access as ' unsigned char'
//==============================================================================

class UCharDG : public AccessDatagram<unsigned char>
{
public:
    typedef AccessDatagram<unsigned char> BaseClass;
    //
    //  Member Functions
    UCharDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    UCharDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }

private:
    Buffer & FBuffer;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
