//  BufferHeader_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "BufferHeader_Mb.h"
#include "HardwareRegister_Mb.h"

namespace   Innovative
{
//---------------------------------------------------------------------------
//  PmcBuffer_HeaderSizeInts(() -- Return natural header size
//---------------------------------------------------------------------------

unsigned int  PmcBuffer_HeaderSizeInts()
{
    PmcBufferManager PBM;
    return PBM.HeaderSizeInts();
}

//---------------------------------------------------------------------------
//  VeloBuffer_HeaderSizeInts(() -- Return natural header size
//---------------------------------------------------------------------------

unsigned int  VeloBuffer_HeaderSizeInts()
{
    VeloBufferManager VBM;
    return VBM.HeaderSizeInts();
}

//---------------------------------------------------------------------------
//  VitaBuffer_HeaderSizeInts(() -- Return natural header size
//---------------------------------------------------------------------------

unsigned int  VitaBuffer_HeaderSizeInts()
{
    VitaBufferManager VBM;
    return VBM.HeaderSizeInts();
}

//---------------------------------------------------------------------------
//  McBuffer_HeaderSizeInts(() -- Return natural header size
//---------------------------------------------------------------------------

unsigned int  McBuffer_HeaderSizeInts()
{
    McBufferManager VBM;
    return VBM.HeaderSizeInts();
}

//==============================================================================
//  CLASS PacketBufferHeader  --  Header Access Datagram for Packet Buffers
//==============================================================================

const unsigned int PidFieldStartBit     = 24;
const unsigned int PktSizeFieldStartBit =  0;

const unsigned int PidFieldSize         =  8;
const unsigned int PktSizeFieldSize     = 24;

//---------------------------------------------------------------------------
//  PacketBufferHeader::PeripheralId() -- read peripheral id field
//---------------------------------------------------------------------------

unsigned short  PacketBufferHeader::PeripheralId() const
{
    PtrBitManipulator Header0( &Data[0] );

    return Header0.Field(PidFieldStartBit, PidFieldSize);
}

//---------------------------------------------------------------------------
//  PacketBufferHeader::PeripheralId(unsigned short) -- write peripheral id
//---------------------------------------------------------------------------

void  PacketBufferHeader::PeripheralId(unsigned short value)
{
    write_split_n_update();
    PtrBitManipulator Header0( &Data[0] );

    Header0.Field(PidFieldStartBit, PidFieldSize, value);
}

//---------------------------------------------------------------------------
//  PacketBufferHeader::PacketSize() -- read PacketSize id
//---------------------------------------------------------------------------

size_t  PacketBufferHeader::PacketSize() const
{
    PtrBitManipulator Header0( &Data[0] );

    return Header0.Field(PktSizeFieldStartBit, PktSizeFieldSize);
}

//---------------------------------------------------------------------------
//  PacketBufferHeader::PacketSize(unsigned int) -- write PacketSize id
//---------------------------------------------------------------------------

void  PacketBufferHeader::PacketSize(size_t value)
{
    write_split_n_update();
    PtrBitManipulator Header0( &Data[0] );

    Header0.Field(PktSizeFieldStartBit, PktSizeFieldSize, static_cast<unsigned int>(value));
}

//---------------------------------------------------------------------------
//  PacketBufferHeader::DataSize() -- read Size of Data in packet
//---------------------------------------------------------------------------

size_t PacketBufferHeader::DataSize() const
{
    return PacketSize()-size();
}

//---------------------------------------------------------------------------
//  PacketBufferHeader::DataSize(size_t) -- write Size of Data of packet
//---------------------------------------------------------------------------

void  PacketBufferHeader::DataSize(size_t value)
{
    PacketSize(value+size());
}


//==============================================================================
//  CLASS NetworkBufferHeader  --  Header Access Datagram for Network Buffers
//==============================================================================

//---------------------------------------------------------------------------
//  NetworkBufferHeader::Type() --
//---------------------------------------------------------------------------

unsigned int NetworkBufferHeader::Type() const
{
    return Data[0];
}

//---------------------------------------------------------------------------
//  NetworkBufferHeader::Type() --
//---------------------------------------------------------------------------

void NetworkBufferHeader::Type(unsigned int which)
{
    write_split_n_update();
    Data[0] = which;
}

//==============================================================================
//  CLASS VitaHeaderDatagram  --  Header Access Datagram for Packet Buffers
//==============================================================================
//
//  IF Header Fields and Sizes
const unsigned int IF_Address  = 0x0;     // offset of IF word in Header

const unsigned int IF_PacketSize_StartBit   =  0;
const unsigned int IF_PacketCount_StartBit  = 16;
const unsigned int IF_TSF_StartBit          = 20;     // can't be 0
const unsigned int IF_TSI_StartBit          = 22;     // can't be 0
const unsigned int IF_CTRR_StartBit         = 24;     // fixed
const unsigned int IF_PacketType_StartBit   = 28;

const unsigned int IF_PacketSize_Width      = 16;
const unsigned int IF_PacketCount_Width     =  4;
const unsigned int IF_TSF_Width             =  2;
const unsigned int IF_TSI_Width             =  2;
const unsigned int IF_CTRR_Width            =  4;
const unsigned int IF_PacketType_Width      =  4;
//
//  StreamID Fields and Sizes
const unsigned int SID_Address  = 0x1;     // offset of SID word in Header

const unsigned int SID_StreamId_StartBit         =  0;
const unsigned int SID_DestinationMask_StartBit  = 16;

const unsigned int SID_StreamId_Width            = 16;
const unsigned int SID_DestinationMask_Width     = 16;

//  ClassOUI Fields and Sizes
const unsigned int CLS_OUI_Address    = 0x2;
//  ClassInfo Fields and Sizes
const unsigned int CLS_INF_Address    = 0x3;

const unsigned int CLS_INF_SOF_StartBit  = 16;
const unsigned int CLS_INF_EOF_StartBit  = 17;


//  Timestamp Integer Seconds Fields and Sizes
const unsigned int TS_IS_Address      = 0x4;
//  Timestamp Fractional Seconds MSW Fields and Sizes
const unsigned int TS_FS_MSW_Address  = 0x5;
//  Timestamp Fractional Seconds LSW Fields and Sizes
const unsigned int TS_FS_LSW_Address  = 0x6;

//  Other Information
const unsigned int VitaTrailerSize  = 1;

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::PacketSize() -- read Packet Size
//---------------------------------------------------------------------------

size_t  VitaHeaderDatagram::PacketSize() const
{
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    return HeaderIF.Field(IF_PacketSize_StartBit, IF_PacketSize_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::PacketSize(unsigned int) -- write Packet Size
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::PacketSize(size_t value)
{
    write_split_n_update();
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    HeaderIF.Field(IF_PacketSize_StartBit, IF_PacketSize_Width, static_cast<unsigned int>(value));
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::DataSize() -- read Size of Data in packet
//---------------------------------------------------------------------------

size_t VitaHeaderDatagram::DataSize() const
{
    return PacketSize() - (size()+VitaTrailerSize);   // subtract off header & trailer sizes
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::DataSize(size_t) -- write Size of Data of packet
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::DataSize(size_t value)
{
    PacketSize(value + (size()+VitaTrailerSize));    // subtract off header & trailer sizes
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::PacketCount() -- read PacketCount field
//---------------------------------------------------------------------------

int  VitaHeaderDatagram::PacketCount() const
{
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    return HeaderIF.Field(IF_PacketCount_StartBit, IF_PacketCount_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::PacketCount(int) -- write PacketCount
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::PacketCount(int type)
{
    write_split_n_update();
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    HeaderIF.Field(IF_PacketCount_StartBit, IF_PacketCount_Width, static_cast<unsigned int>(type));
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::Ctrr_Init() -- Initialize CTRR field
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::Ctrr_Init()
{
    write_split_n_update();
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    HeaderIF.Field(IF_CTRR_StartBit, IF_CTRR_Width, 0xC);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TSI() -- read TSI field
//---------------------------------------------------------------------------

int  VitaHeaderDatagram::TSI() const
{
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    return HeaderIF.Field(IF_TSI_StartBit, IF_TSI_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TSI(int) -- write TSI
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::TSI(int option)
{
    write_split_n_update();
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    HeaderIF.Field(IF_TSI_StartBit, IF_TSI_Width, static_cast<unsigned int>(option));
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TSI(TSI_Options) -- write TSI
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::TSI(TSI_Options option)
{
    TSI((int)option);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TSF() -- read TSF field
//---------------------------------------------------------------------------

int  VitaHeaderDatagram::TSF() const
{
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    return HeaderIF.Field(IF_TSF_StartBit, IF_TSF_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TSF(int) -- write TSF
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::TSF(int option)
{
    write_split_n_update();
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    HeaderIF.Field(IF_TSF_StartBit, IF_TSF_Width, static_cast<unsigned int>(option));
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TSF(TSF_Options) -- write TSF
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::TSF(TSF_Options option)
{
    TSF((int)option);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::PacketType() -- read PacketType field
//---------------------------------------------------------------------------

int  VitaHeaderDatagram::PacketType() const
{
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    return HeaderIF.Field(IF_PacketType_StartBit, IF_PacketType_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::PacketType(int) -- write PacketType
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::PacketType(int type)
{
    write_split_n_update();
    PtrBitManipulator HeaderIF( &Data[IF_Address] );

    HeaderIF.Field(IF_PacketType_StartBit, IF_PacketType_Width, static_cast<unsigned int>(type));
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::StartFrame() -- read SOF field
//---------------------------------------------------------------------------

bool  VitaHeaderDatagram::StartFrame() const
{
    PtrBitManipulator HeaderCLSINF( &Data[CLS_INF_Address] );

    return HeaderCLSINF.Bit(CLS_INF_SOF_StartBit);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::StartFrame(bool) -- write SOF
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::StartFrame(bool state)
{
    write_split_n_update();
    PtrBitManipulator HeaderCLSINF( &Data[CLS_INF_Address] );

    HeaderCLSINF.Bit(CLS_INF_SOF_StartBit, state);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::EndFrame() -- read EOF field
//---------------------------------------------------------------------------

bool  VitaHeaderDatagram::EndFrame() const
{
    PtrBitManipulator HeaderCLSINF( &Data[CLS_INF_Address] );

    return HeaderCLSINF.Bit(CLS_INF_EOF_StartBit);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::EndFrame(bool) -- write EOF
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::EndFrame(bool state)
{
    write_split_n_update();
    PtrBitManipulator HeaderCLSINF( &Data[CLS_INF_Address] );

    HeaderCLSINF.Bit(CLS_INF_EOF_StartBit, state);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::StreamId() -- read Stream id field
//---------------------------------------------------------------------------

unsigned short  VitaHeaderDatagram::StreamId() const
{
    PtrBitManipulator HeaderSID( &Data[SID_Address] );

    return HeaderSID.Field(SID_StreamId_StartBit, SID_StreamId_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::StreamId(unsigned short) -- write Stream id
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::StreamId(unsigned short value)
{
    write_split_n_update();
    PtrBitManipulator HeaderSID( &Data[SID_Address] );

    HeaderSID.Field(SID_StreamId_StartBit, SID_StreamId_Width, value);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::DestinationMask() -- read Dest Mask field
//---------------------------------------------------------------------------

unsigned short  VitaHeaderDatagram::DestinationMask() const
{
    PtrBitManipulator HeaderSID( &Data[SID_Address] );

    return HeaderSID.Field(SID_DestinationMask_StartBit, SID_DestinationMask_Width);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::DestinationMask(unsigned short) -- write Dest Mask
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::DestinationMask(unsigned short value)
{
    write_split_n_update();
    PtrBitManipulator HeaderSID( &Data[SID_Address] );

    HeaderSID.Field(SID_DestinationMask_StartBit, SID_DestinationMask_Width, value);
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TS_Seconds() -- read Timestamp Integer Seconds field
//---------------------------------------------------------------------------

int  VitaHeaderDatagram::TS_Seconds() const
{
    return Data[TS_IS_Address];
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TS_Seconds(int) -- write Timestamp Integer Secs field
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::TS_Seconds(int seconds)
{
    write_split_n_update();

    Data[TS_IS_Address] = seconds;
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TS_FSeconds() -- read Timestamp Fractional Secs field
//---------------------------------------------------------------------------

ii64  VitaHeaderDatagram::TS_FSeconds() const
{
    ii64 msw = static_cast<ii64>(Data[TS_FS_MSW_Address]);
    ii64 lsw = static_cast<ii64>(Data[TS_FS_LSW_Address]);
    return (msw<<32) | lsw;
}

//---------------------------------------------------------------------------
//  VitaHeaderDatagram::TS_FSeconds(ii64) -- write TS Fractional Secs field
//---------------------------------------------------------------------------

void  VitaHeaderDatagram::TS_FSeconds(ii64 fraction)
{
    write_split_n_update();

    Data[TS_FS_MSW_Address] = static_cast<int>(fraction>>32);
    Data[TS_FS_LSW_Address] = static_cast<int>(fraction);
}

//---------------------------------------------------------------------------

//==============================================================================
//  CLASS McHeaderDatagram  --  Header Access Datagram for MultiChan Buffers
//==============================================================================
//
//  Header 0 Fields and Sizes
const unsigned int H0_Address  = 0x0;     // offset of IF word in Header

const unsigned int H0_PacketSize_StartBit   =  0;
//const unsigned int H0_PadBytes_StartBit     = 23;
const unsigned int H0_Channel_StartBit      = 28;

const unsigned int H0_PacketSize_Width      = 28;
//const unsigned int H0_PadBytes_Width        =  5;
const unsigned int H0_Channel_Width         =  4;


//---------------------------------------------------------------------------
//  McHeaderDatagram::PacketSize() -- read Packet Size
//---------------------------------------------------------------------------

size_t  McHeaderDatagram::PacketSize() const
{
    PtrBitManipulator Header0( &Data[H0_Address] );

    return Header0.Field(H0_PacketSize_StartBit, H0_PacketSize_Width);
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::PacketSize(unsigned int) -- write Packet Size
//---------------------------------------------------------------------------

void  McHeaderDatagram::PacketSize(size_t value)
{
    write_split_n_update();
    PtrBitManipulator Header0( &Data[H0_Address] );

    Header0.Field(H0_PacketSize_StartBit, H0_PacketSize_Width, static_cast<unsigned int>(value));
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::DataSize() -- read Size of Data in packet
//---------------------------------------------------------------------------

size_t McHeaderDatagram::DataSize() const
{
    return PacketSize();   // don't subtract off header size
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::DataSize(size_t) -- write Size of Data of packet
//---------------------------------------------------------------------------

void  McHeaderDatagram::DataSize(size_t value)
{
    PacketSize(value);    // don't add on header size
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::Padding() -- read Padding field
//---------------------------------------------------------------------------

size_t  McHeaderDatagram::Padding() const
{
//    PtrBitManipulator Header0( &Data[H0_Address] );

//    return Header0.Field(H0_PadBytes_StartBit, H0_PadBytes_Width);
    return 0;
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::Padding(size_t) --
//---------------------------------------------------------------------------

void  McHeaderDatagram::Padding(size_t /*channel*/)
{
//    write_split_n_update();
//    PtrBitManipulator Header0( &Data[H0_Address] );

//    Header0.Field(H0_PadBytes_StartBit, H0_PadBytes_Width, static_cast<unsigned int>(channel));
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::ChannelId() -- read PacketCount field
//---------------------------------------------------------------------------

unsigned short  McHeaderDatagram::ChannelId() const
{
    PtrBitManipulator Header0( &Data[H0_Address] );

    return Header0.Field(H0_Channel_StartBit, H0_Channel_Width);
}

//---------------------------------------------------------------------------
//  McHeaderDatagram::ChannelId(unsigned short) -- write Id
//---------------------------------------------------------------------------

void  McHeaderDatagram::ChannelId(unsigned short channel)
{
    write_split_n_update();
    PtrBitManipulator Header0( &Data[H0_Address] );

    Header0.Field(H0_Channel_StartBit, H0_Channel_Width, static_cast<unsigned int>(channel));
}

}  // namespace Innovative
