//---------------------------------------------------------------------------
//  BufferHeader_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef BufferHeader_MbH
#define BufferHeader_MbH

#include <Buffer_Mb.h>
#include <DataTypes_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

unsigned int  PmcBuffer_HeaderSizeInts();
unsigned int  VeloBuffer_HeaderSizeInts();
unsigned int  VitaBuffer_HeaderSizeInts();
unsigned int  McBuffer_HeaderSizeInts();

//==============================================================================
//  CLASS BufferHeaderDatagram  --  Base class for Header Manip?
//==============================================================================

class BufferHeaderDatagram : public AccessDatagram<unsigned int>
{
public:
    //
    //  Member Functions
    BufferHeaderDatagram(Buffer & buffer)
        {  LoadInterface(buffer);  }
    BufferHeaderDatagram(const Buffer & buffer)
        {  LoadInterface(buffer);  }
    BufferHeaderDatagram(unsigned int * start_of_header, unsigned int header_size_ints)
        {  LoadInterface(start_of_header, header_size_ints);  }
    // ...raw Interface Passing
    BufferHeaderDatagram(IDatagrammable * interfaceX, bool /*use_raw_intf*/)
        {  load_intf(interfaceX);  }

    ~BufferHeaderDatagram()
        {    }

protected:
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS BufferHeaderDatagrammable  -- Datagrammable Hdr for Headers
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class BufferHeaderDatagrammable : public IDatagrammable
        {
        public:
            BufferHeaderDatagrammable() : FBuffer(0)
                {}

            void  Init(Buffer & buffer)
                {  FBuffer = &buffer;  }

            virtual size_t        DatagramSize()
                                {  return FBuffer->HeaderSizeInInts();  }
            virtual int *         DatagramBasePtr()
                                {  return FBuffer->Header();  }
            virtual bool MakeWritable()
                {
                bool buffer_has_changed = (FBuffer->IsUniqueHeader()==false);
                FBuffer->MakeUniqueHeader();
                return buffer_has_changed;
                }
            virtual void Resize(size_t /*size_in_ints*/) 
                {
                // no support for resizing Headers
                }
                
            Buffer *  FBuffer;
        };
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS ConstBufferHeaderDatagrammable  -- Const Datagrammable
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class ConstBufferHeaderDatagrammable : public IDatagrammable
        {
        public:
            ConstBufferHeaderDatagrammable()
                {}

            void  Init(const Buffer & buffer)
                {  FBuffer = &buffer;  }

            virtual size_t        DatagramSize()
                                {  return FBuffer->HeaderSizeInInts();  }
            virtual int *         DatagramBasePtr()
                                {  return const_cast<int *>(FBuffer->ConstHeader());  }
            virtual bool MakeWritable()
                {
                bool buffer_has_changed = false;
                return buffer_has_changed;
                }
            virtual void Resize(size_t /*size_in_ints*/) 
                {
                // no support for resizing Headers
                }
                
            const Buffer *  FBuffer;
        };
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS OverlayHeaderDatagrammable  -- Overlay header on memory 
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class OverlayHeaderDatagrammable : public IDatagrammable
        {
        public:
            OverlayHeaderDatagrammable()
                {}

            void  Init(unsigned int * start_of_header, unsigned int header_size_ints)
                {  FBuffer = start_of_header;  HdrSize = header_size_ints;  }

            virtual size_t        DatagramSize()
                                {  return HdrSize; }
            virtual int *         DatagramBasePtr()
                                {  return const_cast<int *>(reinterpret_cast<volatile int *>(FBuffer));  }
            virtual bool MakeWritable()
                {
                bool buffer_has_changed = false;
                return buffer_has_changed;
                }
            virtual void Resize(size_t /*size_in_ints*/) 
                {
                // no support for resizing Headers
                }
                
            volatile unsigned int * FBuffer;
            unsigned int            HdrSize;
        };

private:
    BufferHeaderDatagrammable        BufferIntf;
    ConstBufferHeaderDatagrammable   CBufferIntf;
    OverlayHeaderDatagrammable       OverlayIntf;

    void  LoadInterface(Buffer & buffer)
        {
        BufferIntf.Init(buffer);
        load_intf(&BufferIntf);
        }

    void  LoadInterface(const Buffer & buffer)
        {
        CBufferIntf.Init(buffer);
        load_intf(&CBufferIntf);
        }

    void  LoadInterface(unsigned int * start_of_header, unsigned int hdr_size_ints)
        {
        OverlayIntf.Init(start_of_header, hdr_size_ints);
        load_intf(&OverlayIntf);
        }

};


//==============================================================================
//  CLASS PacketBufferHeader  --  Header Access Datagram for Packet Buffers
//==============================================================================

class PacketBufferHeader : public BufferHeaderDatagram
{
public:
    //
    //  Member Functions
    PacketBufferHeader(Buffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    PacketBufferHeader(const Buffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    PacketBufferHeader(unsigned int * start_of_header)
        :  BufferHeaderDatagram(start_of_header, PmcBuffer_HeaderSizeInts())
        {}
    PacketBufferHeader(unsigned int * start_of_header, unsigned int header_size_ints)
        :  BufferHeaderDatagram(start_of_header, header_size_ints)
        {}

    unsigned short  PeripheralId() const;
    void            PeripheralId(unsigned short value);
    size_t          PacketSize() const;
    void            PacketSize(size_t size);
    size_t          DataSize() const;
    void            DataSize(size_t size);

};

typedef  PacketBufferHeader  PacketHeaderDatagram;   // alias

//==============================================================================
//  CLASS NetworkBufferHeader  --  Header Access Datagram for Network Buffers
//==============================================================================

class NetworkBufferHeader : public BufferHeaderDatagram
{
public:
    //
    //  Member Functions
    NetworkBufferHeader(Buffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    NetworkBufferHeader(const Buffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    NetworkBufferHeader(unsigned int * start_of_header)
        :  BufferHeaderDatagram(start_of_header, PmcBuffer_HeaderSizeInts())
        {}

    unsigned int    Type() const;
    void            Type(unsigned int which);
};

//==============================================================================
//  CLASS PmcHeaderDatagram  --  Header Access Datagram for Pmc Buffers
//==============================================================================

class PmcHeaderDatagram : public PacketBufferHeader
{
public:
    //
    //  Member Functions
    PmcHeaderDatagram(PmcBuffer & buffer)
        :  PacketBufferHeader(buffer)
        {}
    PmcHeaderDatagram(const PmcBuffer & buffer)
        :  PacketBufferHeader(buffer)
        {}
    PmcHeaderDatagram(unsigned int * start_of_header)
        :  PacketBufferHeader(start_of_header)
        {}
};

//==============================================================================
//  CLASS VeloHeaderDatagram  --  Header Access Datagram for Velo Buffers
//==============================================================================

class VeloHeaderDatagram : public PacketBufferHeader
{
public:
    //
    //  Member Functions
    VeloHeaderDatagram(VeloBuffer & buffer)
        :  PacketBufferHeader(buffer)
        {}
    VeloHeaderDatagram(const VeloBuffer & buffer)
        :  PacketBufferHeader(buffer)
        {}
    VeloHeaderDatagram(unsigned int * start_of_header)
        :  PacketBufferHeader(start_of_header, VeloBuffer_HeaderSizeInts())
        {}
};

//==============================================================================
//  CLASS VitaHeaderDatagram  --  Header Access Datagram for Packet Buffers
//==============================================================================

class VitaHeaderDatagram : public BufferHeaderDatagram
{
public:
    enum TSI_Options { tsiUtc=1, tsiGPS, tsiOther };
    enum TSF_Options { tsfSamples=1, tsfRealtime, tsfFree };
    //
    //  Member Functions
    VitaHeaderDatagram(VitaBuffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    VitaHeaderDatagram(const VitaBuffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    VitaHeaderDatagram(unsigned int * start_of_header)
        :  BufferHeaderDatagram(start_of_header, VitaBuffer_HeaderSizeInts())
        {}

    size_t          PacketSize() const;
    void            PacketSize(size_t size);
    size_t          DataSize() const;
    void            DataSize(size_t size);

    unsigned short  StreamId() const;
    void            StreamId(unsigned short value);
    unsigned short  DestinationMask() const;
    void            DestinationMask(unsigned short value);

    void            Ctrr_Init();

    int             TSI() const;
    void            TSI(int option);      // not 0.
    void            TSI(TSI_Options option);

    int             TSF() const;
    void            TSF(int option);
    void            TSF(TSF_Options option);

    int             PacketType() const;
    void            PacketType(int type);
    int             PacketCount() const;
    void            PacketCount(int type);
	
    bool            StartFrame() const;
    void            StartFrame(bool state);
    bool            EndFrame() const;
    void            EndFrame(bool state);

    int             TS_Seconds() const;       // integer seconds
    void            TS_Seconds(int seconds);

    ii64            TS_FSeconds() const;      // fractional seconds
    void            TS_FSeconds(ii64 fraction);

    //  other Methods TBD

};

//==============================================================================
//  CLASS McHeaderDatagram  --  Header Access Datagram for Mc Buffers
//==============================================================================

class McHeaderDatagram : public BufferHeaderDatagram
{
public:
    //
    //  Member Functions
    McHeaderDatagram(McBuffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    McHeaderDatagram(const McBuffer & buffer)
        :  BufferHeaderDatagram(buffer)
        {}
    McHeaderDatagram(unsigned int * start_of_header)
        :  BufferHeaderDatagram(start_of_header, McBuffer_HeaderSizeInts())
        {}
    McHeaderDatagram(McImage & image)
        :  BufferHeaderDatagram(image.RawH(), image.HeaderSize())
        {}
    McHeaderDatagram(const McImage & image)
        :  BufferHeaderDatagram(image.ConstRawH(), image.HeaderSize())
        {}

    unsigned short  ChannelId() const;
    void            ChannelId(unsigned short channel);

    size_t          PacketSize() const;
    void            PacketSize(size_t size);
    size_t          DataSize() const;
    void            DataSize(size_t size);

    size_t          Padding() const;        // in characters
    void            Padding(size_t size);

protected:


};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
