//  BufferTrailer_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "BufferTrailer_Mb.h"
#include "HardwareRegister_Mb.h"

namespace   Innovative
{

//==============================================================================
//  CLASS VitaTrailerDatagram  --  Trailer Access Datagram for Packet Buffers
//==============================================================================
//
//  TL Fields and Sizes
const unsigned int TL_Address  = 0x0;     // offset of TL (Trailer word)
//
const unsigned int TL_ContextCount_StartBit  =  0;
const unsigned int TL_E_StartBit             =  7;
const unsigned int TL_Padding_StartBit       =  8;
const unsigned int TL_State_StartBit         = 12;
const unsigned int TL_Ones_StartBit          = 20;   // fixed 0xF
const unsigned int TL_Enables_StartBit       = 24;
//
const unsigned int TL_ContextCount_Width     =  7;
const unsigned int TL_E_Width                =  1;   // use bit io
const unsigned int TL_Padding_Width          =  4;
const unsigned int TL_State_Width            =  8;
const unsigned int TL_Ones_Width             =  4;
const unsigned int TL_Enables_Width          =  8;

//------------------------------------------------------------------------------
//  VitaTrailerDatagram::Padding() -- read Padding
//------------------------------------------------------------------------------

size_t  VitaTrailerDatagram::Padding() const
{
    PtrBitManipulator TrailerTL( &Data[TL_Address] );

    return TrailerTL.Field(TL_Padding_StartBit, TL_Padding_Width);
}

//------------------------------------------------------------------------------
//  VitaTrailerDatagram::Padding(size_t) -- write Padding
//------------------------------------------------------------------------------

void  VitaTrailerDatagram::Padding(size_t value)
{
    write_split_n_update();
    PtrBitManipulator TrailerTL( &Data[TL_Address] );

    TrailerTL.Field(TL_Padding_StartBit, TL_Padding_Width, static_cast<unsigned int>(value));
}

//------------------------------------------------------------------------------
//  VitaTrailerDatagram::Ones_Init() -- Initialize Ones field
//------------------------------------------------------------------------------

void  VitaTrailerDatagram::Ones_Init()
{
    write_split_n_update();
    PtrBitManipulator TrailerTL( &Data[TL_Address] );

    //  Field must be all ones
    TrailerTL.Field(TL_Ones_StartBit, TL_Ones_Width, 0xF);
}

//------------------------------------------------------------------------------
//  VitaTrailerDatagram::IsOnes() -- Check Ones field
//------------------------------------------------------------------------------

bool  VitaTrailerDatagram::IsOnes()
{
    write_split_n_update();
    PtrBitManipulator TrailerTL( &Data[TL_Address] );

    //  Field must be all ones
    return TrailerTL.Field(TL_Ones_StartBit, TL_Ones_Width) == 0xf;
}

//---------------------------------------------------------------------------
}  // namespace Innovative
