//---------------------------------------------------------------------------
//  BufferTrailer_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef BufferTrailer_MbH
#define BufferTrailer_MbH

#include <Buffer_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS BufferTrailerDatagram  --  Base class for Trailer Manip?
//==============================================================================

class BufferTrailerDatagram : public AccessDatagram<unsigned int>
{
public:
    //
    //  Member Functions
    BufferTrailerDatagram(Buffer & buffer)
        {  LoadInterface(buffer);  }
    BufferTrailerDatagram(const Buffer & buffer)
        {  LoadInterface(buffer);  }
    BufferTrailerDatagram(unsigned int * start_of_Trailer)
        {  LoadInterface(start_of_Trailer, 1);  }
    ~BufferTrailerDatagram()
        {   }

protected:
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS BufferTrailerDatagrammable  -- Datagrammable Hdr for Trailers
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class BufferTrailerDatagrammable : public IDatagrammable
        {
        public:
            BufferTrailerDatagrammable()
                :  FBuffer(0)
                {}

            void  Init(Buffer & buffer)
                {  FBuffer = &buffer;  }

            virtual size_t        DatagramSize()
                                {  return FBuffer->TrailerSizeInInts();  }
            virtual int *         DatagramBasePtr()
                                {  return FBuffer->Trailer();  }
            virtual bool MakeWritable()
                {
                bool buffer_has_changed = (FBuffer->IsUniqueTrailer()==false);
                FBuffer->MakeUniqueTrailer();
                return buffer_has_changed;
                }
            virtual void Resize(size_t /*size_in_ints*/) 
                {
                // no support for resizing Trailers
                }
                
            Buffer *  FBuffer;
        };
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS ConstBufferTrailerDatagrammable  -- Const Datagrammable
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class ConstBufferTrailerDatagrammable : public IDatagrammable
        {
        public:
            ConstBufferTrailerDatagrammable()
                :  FBuffer(0)
                {}

            void  Init(const Buffer & buffer)
                {  FBuffer = &buffer;  }

            virtual size_t        DatagramSize()
                                {  return FBuffer->TrailerSizeInInts();  }
            virtual int *         DatagramBasePtr()
                                {  return const_cast<int *>(FBuffer->ConstTrailer());  }
            virtual bool MakeWritable()
                {
                bool buffer_has_changed = false;
                return buffer_has_changed;
                }
            virtual void Resize(size_t /*size_in_ints*/) 
                {
                // no support for resizing Trailers
                }
                
            const Buffer *  FBuffer;
        };
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //  CLASS OverlayTrailerDatagrammable  -- Overlay Trailer on memory 
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    class OverlayTrailerDatagrammable : public IDatagrammable
        {
        public:
            OverlayTrailerDatagrammable()
                :  FBuffer(0)
                {}

            void  Init(unsigned int * start_of_trailer, unsigned int /*trailer_size_ints*/)
                {  FBuffer = start_of_trailer;  }

            virtual size_t        DatagramSize()
                                {  return 1;  }  // Assume standard 1 word Trailer
            virtual int *         DatagramBasePtr()
                                {  return const_cast<int *>(reinterpret_cast<volatile int *>(FBuffer));  }
            virtual bool MakeWritable()
                {
                bool buffer_has_changed = false;
                return buffer_has_changed;
                }
            virtual void Resize(size_t /*size_in_ints*/) 
                {
                // no support for resizing Trailers
                }
                
            volatile unsigned int * FBuffer;
        };

private:
    BufferTrailerDatagrammable        BufferIntf;
    ConstBufferTrailerDatagrammable   CBufferIntf;
    OverlayTrailerDatagrammable       OverlayIntf;

    void  LoadInterface(Buffer & buffer)
        {
        BufferIntf.Init(buffer);
        load_intf(&BufferIntf);
        }

    void  LoadInterface(const Buffer & buffer)
        {
        CBufferIntf.Init(buffer);
        load_intf(&CBufferIntf);
        }

    void  LoadInterface(unsigned int * start_of_trailer, unsigned int trailer_size_ints)
        {
        OverlayIntf.Init(start_of_trailer, trailer_size_ints);
        load_intf(&OverlayIntf);
        }
};

//==============================================================================
//  CLASS VitaTrailerDatagram  --  Trailer Access Datagram for VITA Buffers
//==============================================================================

class VitaTrailerDatagram : public BufferTrailerDatagram
{
public:
    //
    //  Member Functions
    VitaTrailerDatagram(VitaBuffer & buffer)
        :  BufferTrailerDatagram(buffer)
        {}
    VitaTrailerDatagram(const VitaBuffer & buffer)
        :  BufferTrailerDatagram(buffer)
        {}
    VitaTrailerDatagram(unsigned int * start_of_Trailer)
        :  BufferTrailerDatagram(start_of_Trailer)
        {}

    size_t          Padding() const;
    void            Padding(size_t size);

    void            Ones_Init();    // Initialize Ones field
    bool            IsOnes();       // Check Ones field

    //  other Methods TBD

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
