// C3xFloat_Mb.h
//
//  u-law, A-law and linear PCM conversions.
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//
//  The TMS320C3x/C4x does not use the IEEE standard representation of
//     floating point numbers.  The two representations must be translated
//     either on the host or target for them to be understood.
//

//---------------------------------------------------------------------------
#ifndef __C3x_Float_H__
#define __C3x_Float_H__

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class TIFloat
{
public: 
    //
    // Constructors
    TIFloat(float x)
        {  _IEEE = x;  _TI = toTI(x);  };
    TIFloat(int x)
        {  _TI = x;  _IEEE = toIEEE(x);  };

    // Cast conversion to from TIFloat to float
    operator float()
        {  return  _IEEE;  };    
    float asIEEE()
        {  return _IEEE;  };
    int asTI()
        {  return _TI;  };                
        
private:
    //  These structures give the alternate representations of the 
    //  floating point numbers   
    struct c30float
        {
        signed   int    mantissa:23;
        unsigned int    sign:1;
        signed   int    exponent:8;
        };

    struct ieeefloat
        {
        unsigned int    mantissa:23;
        unsigned int    exponent:8;
        unsigned int    sign:1;
        };

    //  Conversion structure
    union ti_flt
        {
        int                 in;
        float               flt;
        struct c30float     ti_str;
        struct ieeefloat    ieee_str;
        };

    // Data 
    float   _IEEE;
    int     _TI;

    // Support Methods
    //   Convert TI C3x/4x format into IEEE-754 single precision format
    float toIEEE(int x)
    {
        ti_flt c30;
        ti_flt ieee;
        c30.in = x;
            
        if (c30.ti_str.exponent == 0-128)
            return(0.0f);
        //
        //  The sign is the same in both formats 
        ieee.ieee_str.sign = c30.ti_str.sign;    
        //
        //  If the input C30 value is negative, negate it 
        if (c30.ti_str.sign)
            {
            //
            // Calculate twos-complement of mantissa 
            c30.ti_str.mantissa = ~c30.ti_str.mantissa + 1;

            if (!c30.ti_str.mantissa)
                c30.ti_str.exponent++;
            }
        //
        // The exponent in the IEEE format is 127 added to the actual value 
        //   (OFFSET 127 format)
        ieee.ieee_str.exponent = (c30.ti_str.exponent == -128) 
            ? 0 : c30.ti_str.exponent + 127;
        //
        // The mantissas are identical 
        ieee.ieee_str.mantissa = c30.ti_str.mantissa;      

        return (ieee.flt);
    };

    // Convert from IEEE-754 single precision format into TI C3x/4x format
    int toTI(float x)
    {
        ti_flt c30;
        ti_flt ieee;
        ieee.flt = x;

        if (ieee.in == 0)
            return(0x80000000);
        //
        //  The mantissas are identical 
        c30.ti_str.mantissa = ieee.ieee_str.mantissa;    
        //
        //  The exponent in the IEEE format is 127 added to the actual value 
        //     (known as OFFSET 127)
        //
        int expo = int(ieee.ieee_str.exponent);
        c30.ti_str.exponent = (ieee.ieee_str.exponent == 0) 
            ? -128 : expo - 127;
        //
        //  The sign is the same in both formats 
        c30.ti_str.sign = ieee.ieee_str.sign;    
        //
        //  If the input IEEE value is negative, negate the C30 mantissa 
        if (ieee.ieee_str.sign)
            {
            //
            // Calculate twos-complement of mantissa 
            c30.ti_str.mantissa = ~c30.ti_str.mantissa + 1;

            if (!c30.ti_str.mantissa)
                c30.ti_str.exponent--;
            }

        return (c30.in);
    };
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif

