// C64xDspDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <C64xDspDeviceMap_Mb.h>
#include <C64xDspDeviceDriver_Mb.h>
#include <Exception_Mb.h>


using namespace std;

namespace
{
const int   tiCpuBaseAddr  = (0x01800000 >> 2);
const int   tiCpuEmifABase = (0x01800000 >> 2) - tiCpuBaseAddr;
const int   tiCpuEmifBBase = (0x01a80000 >> 2) - tiCpuBaseAddr;

const int   tiCpuRSTSRC    = (0x01c00000 >> 2) - tiCpuBaseAddr;
const int   tiCpuPMDCSR    = (0x01c00004 >> 2) - tiCpuBaseAddr;
const int   tiCpuPCIIS     = (0x01c00008 >> 2) - tiCpuBaseAddr;
const int   tiCpuPCIIEN    = (0x01c0000c >> 2) - tiCpuBaseAddr;
const int   tiCpuDSPMA     = (0x01c00010 >> 2) - tiCpuBaseAddr;
const int   tiCpuPCIMA     = (0x01c00014 >> 2) - tiCpuBaseAddr;
const int   tiCpuPCIMC     = (0x01c00018 >> 2) - tiCpuBaseAddr;
const int   tiCpuCDSPA     = (0x01c0001c >> 2) - tiCpuBaseAddr;
const int   tiCpuCPCIA     = (0x01c00020 >> 2) - tiCpuBaseAddr;
const int   tiCpuCCNT      = (0x01c00024 >> 2) - tiCpuBaseAddr;
const int   tiCpuHALT      = (0x01c00028 >> 2) - tiCpuBaseAddr;
const int   tiCpuHSR       = (0x01c1fff0 >> 2) - tiCpuBaseAddr;
const int   tiCpuHDCR      = (0x01c1fff4 >> 2) - tiCpuBaseAddr;
const int   tiCpuDSSP      = (0x01c1fff8 >> 2) - tiCpuBaseAddr;
const int   tiCpuReserved  = (0x01c1fffc >> 2) - tiCpuBaseAddr;
const int   tiCpuEEADD     = (0x01c20000 >> 2) - tiCpuBaseAddr;
const int   tiCpuEEDAT     = (0x01c20004 >> 2) - tiCpuBaseAddr;
const int   tiCpuEECTL     = (0x01c20008 >> 2) - tiCpuBaseAddr;

//
//  DSP Type IDs
//  const int  ctC641xTypeId           = 0x00;
//
//  ...Firmware Versioning
const int   MinFirmwareRev = 0x00;

}  // end nameless namespace

//---------------------------------------------------------------------------
namespace   Innovative
{

//===========================================================================
//  CLASS C64xDspDeviceMap  -- 
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class C64xDspDeviceMap
//---------------------------------------------------------------------------

TiBmDeviceMapRegisters C64xCpuRegAddrs =
    {
    tiCpuHSR,             // StatusRegister_Addr;
    tiCpuDSSP,            // DspPageRegister_Addr;
    tiCpuHDCR,            // HostToDspCtlRegister_Addr;
    tiCpuEmifABase,       // CpuEmifARegister_Addr
    tiCpuEEADD,           // Eeprom address
    tiCpuEEDAT,           // Eeprom data
    tiCpuEECTL,           // Eeprom control/status
    };


//---------------------------------------------------------------------------
//  constructor for class C64xDspDeviceMap
//---------------------------------------------------------------------------

C64xDspDeviceMap::C64xDspDeviceMap(C64xDspDeviceDriver::DeviceMode mode)
    :  TiBusmasterDeviceMap(new C64xDspDeviceDriver(mode), C64xCpuRegAddrs),
       DDeviceDriver(dynamic_cast<C64xDspDeviceDriver*>(DeviceDriver))
{
}

//---------------------------------------------------------------------------
//  C64xDspDeviceMap::~C64xDspDeviceMap() - Dtor
//---------------------------------------------------------------------------

C64xDspDeviceMap::~C64xDspDeviceMap()
{
    Close();
}

//---------------------------------------------------------------------------
//  C64xDspDeviceMap::Open() - 
//---------------------------------------------------------------------------

void C64xDspDeviceMap::Open()
{
    if ( DeviceDriver->Opened() )
        return;
    //
    //  Call base class Open()
    TiBusmasterDeviceMap::Open();

    IUsesStandardC64xCpu::Open();
}

//---------------------------------------------------------------------------
//  C64xDspDeviceMap::Close() - 
//---------------------------------------------------------------------------

void C64xDspDeviceMap::Close()
{
}

//---------------------------------------------------------------------------
//  C64xDspDeviceMap::DeviceInfo() --
//---------------------------------------------------------------------------

InnovativeKernel::DeviceEntry & C64xDspDeviceMap::DeviceInfo()
{
    return DDeviceDriver->PciConfig();
}


} // Innovative
