//
// Cdce18005_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef CDCE18005_MbH
#define CDCE18005_MbH

#include <TimebaseDevice_Mb.h>
#include <CDCE_Conversion_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS Cdce18005Registers  -- Addresses for Device Registers
//===========================================================================

class Cdce18005Registers
{
public:
     unsigned int   SpiIntf_Addr;
};

//=============================================================================
//  CLASS Cdce18005  --  Programmable MUX object
//=============================================================================

class Cdce18005 : public MuxTimebaseDevice, public CdceConversion
{
public:
    //
    //  Member Functions
    Cdce18005(IAddressingSpace &space, Cdce18005Registers & regs);
    virtual ~Cdce18005()  {  }

    void   Init();
    void   Reapply();

    //  Power state
    void  Powered(bool value);
    bool  Powered()
             {  return FPowered;  }

    //
    //  Public CONFIG Functions
    //----------------------------------------------------------------------------
    enum MuxSelect { muxPrimaryIn, muxSecondaryIn, muxAuxIn };

    //  Raw configuration accesses
    virtual void            Port(unsigned char reg, unsigned int value);
    virtual unsigned int    Port(unsigned char reg);

protected:

    Register        SpiReg;
    bool            FPowered;

    std::vector<int>  RegisterImage;
    std::vector<unsigned int>  OutputConfigField;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    virtual void    OnSetPower(bool & new_state) = 0;
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    enum MuxSettings { msPrimaryIn=0x0, msSecondaryIn=0x2, msAuxIn=0x1 };
    enum InputCfgSettings { icLvPecl=0x2, icLvds=0x3, icLvCmos=0x0 };
    //   These values fill the field for OutputMode for each output. The LvCmos
    //     settings are for half of the field - combine one N and one P setting
    //     for the full configuration
    enum OutputCfgSettings { ocDisabled=0x1A, ocLvPecl=0x20, ocLvds=0x3A,
                             ocLvCmos_P_Active=0x00, ocLvCmos_P_Inverted=0x01,
                             ocLvCmos_P_Low=0x03, ocLvCmos_P_TriStated=0x02,
                             ocLvCmos_N_Active=0x00, ocLvCmos_N_Inverted=0x04,
                             ocLvCmos_N_Low=0x0c, ocLvCmos_N_TriStated=0x08,
                           };
    enum OutputSyncSettings { osPrimary=0x1, osSecondary=0x2, osAux=0x0 };
    enum AuxOutputEnable { aeDisabled=0x0, aeEnabled=0x1 };
    enum AuxOutputSettings { aoFromDivider2=0x0, aoFromDivider3=0x1 };

    void  SetInputConfig(unsigned int cfg_setting);
    void  SetInputBufferConfig(bool dc_terminated,
                               bool primary_term_disabled, bool sec_term_disabled,
                               bool prim_bias, bool sec_bias, bool failsafe  );
    void  SetOutputDividerSync(unsigned int setting);
    void  SetHighSwingLvpecl(unsigned int output, bool setting);
    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting, bool should_cache_setting=true);
    void  SetAuxOutputConfig(bool enable, unsigned int aux_setting);

    virtual void  SetMuxHdw(unsigned int output, unsigned int mux_source)
        {  SetMux(output, ToMuxSetting(static_cast<MuxSelect>(mux_source)));   }

    void  SetPhaseAdjust(unsigned int output, unsigned int divider_setting, unsigned int phase_setting);
    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    void  SetDividerEnable(unsigned int output, bool state);

    MuxSettings  ToMuxSetting(MuxSelect mux)
    {
        switch (mux)
            {
            default:
            case muxPrimaryIn:   return msPrimaryIn;
            case muxSecondaryIn: return msSecondaryIn;
            case muxAuxIn:       return msAuxIn;
            }
   }

    void  ReadDataRegisters();
    void  WriteDataRegisters();

    void  WriteMustBeValues();

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by);  }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above)
        {  return CdceGetNearestDividers(candidate, below, above);  }

private:
    void  SetMux(unsigned int output, unsigned int mux_setting);

public:
    Trace       Debug;
};

//==============================================================================
//  ClockMux_Cdce18005 -- Implements Interface to 'standard' clock Mux
//==============================================================================
/*
class ClockMux_Cdce18005 : public Cdce18005//,  public IClockMuxDevice
{
public:
    ClockMux_Cdce18005(IAddressingSpace & space, Cdce18005Registers & regs)
        :  Cdce18005( space, regs )
        {  }
    virtual  ~ClockMux_Cdce18005()  {}

    virtual void  Source(unsigned int output, unsigned int source)
        {  SetMuxSelection(output, source);   }
    virtual void  Divider(unsigned int output, unsigned int divide_by)
        {  OutputDivider(output, divide_by);  }
    virtual void  DividerEnabled(unsigned int output, bool enabled)
        {  SetDividerEnable(output, enabled);  }
    virtual void  Phase(unsigned int output, unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(output, divide_by, phase);  }
    virtual void  Boost(unsigned int output, bool enabled)
        {  SetHighSwingLvpecl(output, enabled);  }

//    virtual double  SourceFrequency(unsigned int input) const
//        {  return Cdce18005::SourceFrequency(input);  }
//    virtual double  OutputFrequencyActual(unsigned int output) const
//        {  return Cdce18005::OutputFrequencyActual(output);  }


    virtual void  UpdateUsingDividers(bool divider_mode_enabled)
        {  UpdateMode = (divider_mode_enabled) ? updDivider : updFrequency;  }
    virtual void  UpdateUsingFrequency(bool freq_mode_enabled)
        {  UpdateUsingDividers( ! freq_mode_enabled );  }

    //
    //  Internal interface
    virtual void          CM_Port(unsigned int addr, unsigned int data)
        {  Port(addr, data);  }
    virtual unsigned int  CM_Port(unsigned int addr)
        {  return Port(addr);  }
        
    virtual void  CM_Init()
        {  Init();  }
    virtual void  CM_Reapply()
        {  Reapply();  }

};
*/

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative

//---------------------------------------------------------------------------
#endif
