//
// Cdce62005_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef CDCE62005_MbH
#define CDCE62005_MbH

#include <TimebaseDevice_Mb.h>
#include <CDCE_Conversion_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>
#include <VcxoControl_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS Cdce62005PllTuningEvent  --  Override PLL Tuning Parameters
//===========================================================================

class Cdce62005PllTuningEvent : public OpenWire::Event
{
public:
    unsigned int &R_P;
    unsigned int &M;
    unsigned int &I;
    unsigned int &FB_A;
    unsigned int &FB_B;
    unsigned int &OD;

    Cdce62005PllTuningEvent(unsigned int & r_p,  unsigned int & m,
                            unsigned int & i,    unsigned int & fb_a,
                            unsigned int & fb_b, unsigned int & od     )
        : R_P(r_p), M(m), I(i), FB_A(fb_a), FB_B(fb_b), OD(od)
        {
        }
};

//===========================================================================
//  CLASS Cdce62005Registers  -- Addresses for Device Registers
//===========================================================================

class Cdce62005Registers
{
public:
    unsigned int   Control_Addr;
    unsigned int   SpiIntf_Addr;
};

//=============================================================================
//  CLASS Cdce62005  --  PLL timebase control
//=============================================================================

class Cdce62005 : public PllTimebaseDevice, public CdceConversion
{
public:
    enum CdReferenceSource { refNone, refPrimary=0x1, refSecondary=0x2, refAux=0x4,
                                refAutoSelectRef=0x3, refAutoSelectAll=0x7 };
    enum CdOutputSource { fromPll, fromExtClk };
    //
    //  Member Functions
    Cdce62005(IAddressingSpace &space, Cdce62005Registers & regs);
    virtual ~Cdce62005();

    void   Init();
    void   Reapply();

    //  Power state
    void  Powered(bool value);
    bool  Powered()
             {  return FPowered;  }
    //  Select the Input reference
    void  ReferenceSource(CdReferenceSource value)
             { FReferenceSource = value;  SetReferenceSource(value); }
    CdReferenceSource  ReferenceSource()
             {  return FReferenceSource;  }
    //  Select Output Mode
    void  Pll_Mode()
             {  OutputSource(fromPll);  }
    void  ExtClock_Mode()
             {  OutputSource(fromExtClk);  }
    void  OutputSource(CdOutputSource value)
             {
             FOutputSource = value;
             UpdateMode = (value==fromPll) ? updFrequency : updDivider;
             SetOutputSource(value);
             }
    CdOutputSource  OutputSource() const
             { return FOutputSource; }
    bool  IsIn_Pll_Mode()
             {  return FOutputSource == fromPll;  }
    bool  IsIn_ExtClock_Mode()
             {  return IsIn_Pll_Mode() == false;  }

    bool                Locked() const;

    int                 MaxOutputDivider() const;

    //  Raw configuration accesses
    virtual void            Port(unsigned char reg, unsigned int value);
    virtual unsigned int    Port(unsigned char reg);

    OpenWire::EventHandler<Cdce62005PllTuningEvent> OnPllTuning;

protected:
    //
    //  Member Data
    CdReferenceSource   FReferenceSource;
    CdOutputSource      FOutputSource;
    std::vector<int>           RegisterImage;
    std::vector<float>         OutputFrequencyLimit;
    std::vector<unsigned int>  OutputConfigField;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    virtual void    OnSetPower(bool & new_state);
    virtual double  PhaseDetectorFrequency() const = 0;

    virtual class IVcxoControl &   Vcxo() {  return LocalVcxo;  }
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    enum InputCfgSettings { icLvPecl=0x2, icLvds=0x3, icLvCmos=0x0 };
    //   These values fill the field for OutputMode for each output. The LvCmos
    //     settings are for half of the field - combine one N and one P setting
    //     for the full configuration
    enum OutputCfgSettings { ocDisabled=0x1A, ocLvPecl=0x20, ocLvds=0x3A,
                             ocLvCmos_P_Active=0x00, ocLvCmos_P_Inverted=0x01,
                             ocLvCmos_P_Low=0x03, ocLvCmos_P_TriStated=0x02,
                             ocLvCmos_N_Active=0x00, ocLvCmos_N_Inverted=0x04,
                             ocLvCmos_N_Low=0x0c, ocLvCmos_N_TriStated=0x08,
                           };
    enum OutputMuxSource { osPrimaryRef=0x0, osSecondaryRef=0x2, osSmartMux=0x1, osVco=0x3 };

    enum AuxOutputEnable { aeDisabled=0x0, aeEnabled=0x1 };
    enum AuxOutputSettings { aoFromDivider2=0x0, aoFromDivider3=0x1 };
    enum ExtRefControlSettings { xrFromInternalRegister=0x1, xrUseExternalPin=0x0 };

    void  SetInputConfig(unsigned int cfg_setting);
    void  SetInputBufferConfig(bool dc_terminated, bool primary_input_term_disabled,
                               bool secondary_input_term_disabled,
                               bool prim_bias, bool sec_bias, bool failsafe );

    void  SetHighSwingLvpecl(unsigned int output, bool setting);
    void  SetHighSwingLvds(unsigned int output, bool setting)
                {  SetHighSwingLvpecl(output, setting);  }
    void  SetHighSwing(unsigned int output, bool setting)
                {  SetHighSwingLvpecl(output, setting);  }
    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting, bool should_cache_setting=true);


    void  SetAuxOutputConfig(bool enable, unsigned int aux_setting);

    void  SetReferenceSource(CdReferenceSource source);

    void  SetLockConfig(bool analog_lock);
    void  SetOutputMuxSource(unsigned int output, unsigned int source_code);

    void  SetExternalReferenceControl(bool useInternalSelect);
    void  SetExternalReferenceSelect(bool usePrimaryReference);

    void  SetPhaseAdjust(unsigned int output, unsigned int divider_setting, unsigned int phase_setting);
    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    void  SetDividerEnable(unsigned int output, bool state);

    void  SetVcoRange(bool isHighRange);
    void  SetOutputSource(CdOutputSource source);
    //
    //  Internal "Getters"
    unsigned int  GetDivider(unsigned int output);
    bool          GetDividerEnable(unsigned int output);
    //
    //  Pll Programming Registers
    void  SetR_P(unsigned int R_P);
    void  SetM(unsigned int M);
    void  SetI(unsigned int I);
    void  SetFB_A(unsigned int FB_A);
    void  SetFB_B(unsigned int FB_B);
    void  SetVco_Ps(unsigned int Vco_Ps);

    unsigned int  GetR_P();
    unsigned int  GetM();
    unsigned int  GetI();
    unsigned int  GetFB_A();
    unsigned int  GetFB_B();
    unsigned int  GetVco_Ps();

    void  ReadDataRegisters();
    void  WriteDataRegisters();
    void  SignalSync();

    double  MaxMInputFrequency()
                {  return 250.0e6;  };
    double  MaxR_PInputFrequency()
                {  return 500.0e6;  };

    void  CalculateRPMI(unsigned int & R_P, unsigned int & M, unsigned int & I);
    void  CalculateOD_FB(unsigned int & OD, unsigned int & FB_A, unsigned int & FB_B,
                              double & VcoFreq, double Pfd );
    void  CalculateFB(unsigned int & FB_A, unsigned int & FB_B, double VcoFreq, double Pfd);

    //-------------------------------------------------------------
    // CLASS InternalVcxo  --  In-chip VCXO interface
    //-------------------------------------------------------------

    class InternalVcxo : public IVcxoControl
    {
    public:
        virtual double  Min() {  return Pll.Vco_Min();  }
        virtual double  Max() {  return Pll.Vco_Max();  }
        virtual bool    CanTuneTo(double freq) {  return Pll.Vco_CanTuneTo(freq);  }
        virtual double  BestTunableFrequency(double freq)
                              {  return Pll.Vco_BestTunableFrequency(freq);  }

        virtual void    SetVcoFrequency(double freq)
                              {  Pll.Vco_SetVcoFrequency(freq);  }
        virtual void    SetRange(double /*min*/, double /*max*/)
                              {  /* not available  */ }

        InternalVcxo(Cdce62005 & pll)  :  Pll(pll)  {}
        virtual ~InternalVcxo()  {}

        void    Frequency(double freq)
                    {  SetVcoFrequency(freq);   }
        double  Frequency()
                    {  return Pll.Vco_GetVcoFrequency();   }
        double  FrequencyActual()
                    {  return Pll.Vco_GetVcoFrequency();   }

    private:
        Cdce62005 & Pll;
    };
    friend class Cdce62005::InternalVcxo;

    InternalVcxo  LocalVcxo;

    //-------------------------------------------------------------
    // CLASS PllControlRegister  --
    //-------------------------------------------------------------
    class PllControlRegister : public ShadowRegister
    {
    public:
        typedef ShadowRegister inherited;
        RegisterBit        PowerOn;
        RegisterBit        ResetOff;
        RegisterBit        SpiEnable;
        RegisterBit        UsePrimaryReference;
        RegisterBit        PllSync;

        PllControlRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           PowerOn(*this, 0),
           ResetOff(*this, 1),
           SpiEnable(*this, 2),
           UsePrimaryReference(*this, 5),
           PllSync(*this, 7)
        {  }

    };

    class StatusRegister : public ReadOnlyRegister
    {
    public:
        typedef ReadOnlyRegister inherited;
        ReadOnlyRegisterBit		Lock;

        StatusRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           Lock(*this, 4)
        {  }

        // Properties
        bool Locked() const
            {  return Lock.Value() ? true : false;  }
    };

    PllControlRegister  ControlReg;
    StatusRegister      StatusReg;
    Register            SpiReg;

    // Fields
    bool                FPowered;

    //
    //  PLL Device Implementations
    void            SettingsInit();
    void            FrequencyModeUpdate();
    void            DividerModeUpdate();

    //
    void            SetOutputDividers(double VcoFreq, unsigned int OutDiv);

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by);  }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above)
        {  return CdceGetNearestDividers(candidate, below, above);  }

    void  InitDividerSupport();

protected:
    // Mapping for Feedback A dividers
    ValidDividerArray    FB_A_DividerValues;
    ValidDividerMap      FB_A_ValidDividers;
    InverseDividerMap    FB_A_InverseDividers;

    unsigned int  To_FB_A_DividerInput(unsigned int divide_by);
    unsigned int  From_FB_A_DividerInput(unsigned int divider_bits);
    bool  IsValid_FB_A_Divider(unsigned int divide_by);

    // Mapping for Feedback B (Bypass) dividers
    ValidDividerArray    FB_B_DividerValues;
    ValidDividerMap      FB_B_ValidDividers;
    InverseDividerMap    FB_B_InverseDividers;

    unsigned int  To_FB_B_DividerInput(unsigned int divide_by);
    unsigned int  From_FB_B_DividerInput(unsigned int divider_bits);
    bool  IsValid_FB_B_Divider(unsigned int divide_by);

    // Mapping for Vco Out Prescaler divider
    ValidDividerArray    Vco_Ps_DividerValues;
    ValidDividerMap      Vco_Ps_ValidDividers;
    InverseDividerMap    Vco_Ps_InverseDividers;

    unsigned int  To_Vco_Ps_DividerInput(unsigned int divide_by);
    unsigned int  From_Vco_Ps_DividerInput(unsigned int divider_bits);
    bool  IsValid_Vco_Ps_Divider(unsigned int divide_by);

    //  Internal VCO implementations
    double  Vco_Min();
    double  Vco_Max();
    bool    Vco_CanTuneTo(double freq);
    double  Vco_BestTunableFrequency(double freq);
    void    Vco_SetVcoFrequency(double freq);
    double  Vco_GetVcoFrequency();
    void    Vco_Calibrate();

    double  Vco_Gap0Min() { return 785.0e6;   }
    double  Vco_Gap0Max() { return 875.0e6;   }

    double  FVcoFrequency;

public:
    Trace         Debug;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

