//
// Cdce72010_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Cdce72010_MbH
#define Cdce72010_MbH

#include <TimebaseDevice_Mb.h>
#include <CDCE_Conversion_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS CdcePllTuningEvent  --  Override PLL Tuning Parameters
//===========================================================================

class CdcePllTuningEvent : public OpenWire::Event
{
public:
    int &M, &R, &N, &Fb, &X;

    CdcePllTuningEvent(int& m, int& r, int& n, int& fb, int &x)
        : M(m), R(r), N(n), Fb(fb), X(x)
        {
        }
};

//===========================================================================
//  CLASS Cdce72010Registers  -- Addresses for Device Registers
//===========================================================================

class Cdce72010Registers
{
public:
    unsigned int   Control_Addr;
    unsigned int   SpiIntf_Addr;
};

//=============================================================================
//  CLASS Cdce72010  --  PLL timebase control
//=============================================================================

class Cdce72010 : public PllTimebaseDevice, public CdceConversion
{
public:
    enum CdReferenceSource { refNone, refPrimary, refSecondary, refAutoSelect };
    enum CdOutputSource { outPll, outAux };
    //
    //  Member Functions
    Cdce72010(IAddressingSpace &space, Cdce72010Registers & regs);
    virtual ~Cdce72010();

    void   Init();
    void   Reapply();

    //  Power state
    void  Powered(bool value);
    bool  Powered()
             {  return FPowered;  }
    //  Select the Input reference
    void  ReferenceSource(CdReferenceSource value)
             { FReferenceSource = value;  SetReferenceSource(value); }
    CdReferenceSource  ReferenceSource()
             {  return FReferenceSource;  }
    //  Select output signal
    void  OutputSource(CdOutputSource value);
    CdOutputSource  OutputSource() const
             { return FOutputSource; }


    bool				Locked() const;

    int  				MaxOutputDivider() const;

    virtual void    SetPhaseDetectorFrequency(double /*pfd_freq*/)
                        { }   // may be implemented in board specific class
    //
    //  Public CONFIG Functions
    //----------------------------------------------------------------------------
    void LoadFreq(int m, int n, int fb, int divider);

    struct InfoBlock
    {
        unsigned int M;
        unsigned int R;
        unsigned int N;
        unsigned int Fb;
        unsigned int X;
        double RefFrequency;
        double VcoFrequency;
        double FrequencyActual;
    };
    void ReadInfo(InfoBlock & info);

    //  Raw configuration accesses
    virtual void 			Port(unsigned char reg, unsigned int value);
    virtual unsigned int    Port(unsigned char reg);

    OpenWire::EventHandler<CdcePllTuningEvent> OnPllTuning;

protected:
    //
    //  Member Data
    CdReferenceSource   FReferenceSource;
    CdOutputSource      FOutputSource;
    std::vector<int>    RegisterImage;
    std::vector<float>         OutputFrequencyLimit;
    std::vector<unsigned int>  OutputConfigField;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    virtual void    OnSetPower(bool & new_state);
    virtual double  PhaseDetectorFrequency() const = 0;

    virtual class IVcxoControl &   Vcxo() = 0;
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    enum InputCfgSettings { icLvPecl=0x1, icLvds=0x3, icLvCmos=0x0 };
    enum VcoCfgSettings   { vcLvPecl=0x1, vcLvds=0x3, vcLvCmos=0x0 };
    //   These values fill the field for OutputMode for each output. The LvCmos
    //     settings are for half of the field - combine one N and one P setting
    //     for the full configuration
    enum OutputCfgSettings { ocDisabled=0x1A, ocLvPecl=0x20, ocLvds=0x3A,
                             ocLvCmos_P_Active=0x00, ocLvCmos_P_Inverted=0x01,
                             ocLvCmos_P_Low=0x03, ocLvCmos_P_TriStated=0x02,
                             ocLvCmos_N_Active=0x00, ocLvCmos_N_Inverted=0x04,
                             ocLvCmos_N_Low=0x0c, ocLvCmos_N_TriStated=0x08,
                           };
    enum AuxInputEnable { aeDisabled=0x1, aeEnabled=0x0 };
    enum AuxInputSettings { aiNegPinBias=0x1, aiNormal=0x0 };
    //
    //  Internal "Setters"
    void  SetPllEnable(bool enabled);
    void  SetFBDividerEnable(bool enabled);
    void  SetInputConfig(unsigned int cfg_setting);
    void  SetInputBufferConfig(bool dc_terminated, bool input_term_disabled,
                               bool prim_bias, bool sec_bias, bool failsafe  );
    void  SetVcoConfig(unsigned int cfg_setting);
    void  SetVcoBufferConfig(bool dc_terminated, bool hysteresis_enabled,
                             bool input_term_disabled, bool vco_bias  );
    void  SetHighSwingLvpecl(unsigned int output, bool setting);
    void  SetHighSwingLvds(unsigned int output, bool setting)
                {  SetHighSwingLvpecl(output, setting);  }
    void  SetHighSwing(unsigned int output, bool setting)
                {  SetHighSwingLvpecl(output, setting);  }
    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting, bool should_cache_setting=true);
    void  SetAuxInputConfig(bool enable, unsigned int aux_setting);
    void  SetReferenceSource(CdReferenceSource source);
    void  SetOutputSource(CdOutputSource source);
    void  SetFBClockType(bool clock_is_cml);   //  false = CMOS clock, true = CML clock
    void  SetLockConfig(bool analog_lock);

    void  SetExternalReferenceControl(bool useExternalPin);
    void  SetExternalReferenceSelect(bool usePrimaryReference);

    void  SetPhaseAdjust(unsigned int output, unsigned int divider_setting, unsigned int phase_setting);
    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    void  SetDividerEnable(unsigned int output, bool state);
    //
    //  Internal "Getters"
    unsigned int  GetDivider(unsigned int output);
    bool          GetDividerEnable(unsigned int output);
    //
    //  Pll Programming Registers
    void  SetM(unsigned int M);
    void  SetR(unsigned int R);
    void  SetN(unsigned int N);
    void  SetFB(unsigned int FB);
    void  SetX(unsigned int X);

    unsigned int  GetM();
    unsigned int  GetR();
    unsigned int  GetN();
    unsigned int  GetFB();
    unsigned int  GetX();

    void  ReadDataRegisters();
    void  WriteDataRegisters();

    double  MaxMInputFrequency()
                {  return 250.0e6;  };
    double  MaxNInputFrequency()
                {  return 250.0e6;  };

    class PllControlRegister : public ShadowRegister
    {
    public:
        typedef ShadowRegister inherited;
        RegisterBit		   PowerOn;
        RegisterBit		   ResetOff;
        RegisterBit		   SpiEnable;
        RegisterBit		   UsePrimaryReference;

        PllControlRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           PowerOn(*this, 0),
           ResetOff(*this, 1),
           SpiEnable(*this, 2),
           UsePrimaryReference(*this, 5)
        {  }

    };

    class StatusRegister : public ReadOnlyRegister
    {
    public:
        typedef ReadOnlyRegister inherited;
        ReadOnlyRegisterBit		Lock;

        StatusRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           Lock(*this, 4)
        {  }

        // Properties
        bool Locked() const
            {  return Lock.Value() ? true : false;  }
    };

    PllControlRegister  ControlReg;
    StatusRegister		StatusReg;
    Register		    SpiReg;

    // Fields
    bool                FPowered;

    //
    //  PLL Device Implementations
	void            SettingsInit();
    void 			FrequencyModeUpdate();
    void            DividerModeUpdate();
    unsigned int    ExtractDividerFactor(unsigned int FB, unsigned int N);
    //
    void            SetOutputDividers(double VcoFreq, unsigned int OutDiv);

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by);  }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above)
        {  return CdceGetNearestDividers(candidate, below, above);  }

    unsigned int  RoundToBestInt(double number);

public:
      Trace			Debug;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

