//
// CDCE_Conversion_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <CDCE_Conversion_Mb.h>
#include <algorithm>
#include <FloatingPtSupport_Mb.h>

namespace Innovative
{

//=============================================================================
//  CLASS CdceConversion --  Common Conversions for CDCE dividers
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class CdceConversion
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//  constructor for class CdceConversion
//-----------------------------------------------------------------------------

CdceConversion::CdceConversion()
{
    CdceDividerValues.push_back(1);   CdceDividerValues.push_back(2);
    CdceDividerValues.push_back(3);   CdceDividerValues.push_back(4);
    CdceDividerValues.push_back(5);   CdceDividerValues.push_back(6);
    CdceDividerValues.push_back(8);   CdceDividerValues.push_back(10);
    CdceDividerValues.push_back(12);  CdceDividerValues.push_back(16);
    CdceDividerValues.push_back(18);  CdceDividerValues.push_back(20);
    CdceDividerValues.push_back(24);  CdceDividerValues.push_back(28);
    CdceDividerValues.push_back(30);  CdceDividerValues.push_back(32);
    CdceDividerValues.push_back(36);  CdceDividerValues.push_back(40);
    CdceDividerValues.push_back(42);  CdceDividerValues.push_back(48);
    CdceDividerValues.push_back(50);  CdceDividerValues.push_back(56);
    CdceDividerValues.push_back(60);  CdceDividerValues.push_back(64);
    CdceDividerValues.push_back(70);  CdceDividerValues.push_back(80);

    CdceDividerFactors.push_back(2);   CdceDividerFactors.push_back(3);
    CdceDividerFactors.push_back(5);   CdceDividerFactors.push_back(7);

    //  Fill the conversion information structures
    //     Valid dividers is keyed by divide_by, value is Input Value
    CdceValidDividers[1]  = 0x20;       CdceValidDividers[2]  = 0x40;
    CdceValidDividers[3]  = 0x41;       CdceValidDividers[4]  = 0x42;
    CdceValidDividers[5]  = 0x43;       CdceValidDividers[6]  = 0x01;
    CdceValidDividers[8]  = 0x02;       CdceValidDividers[10] = 0x03;
    CdceValidDividers[12] = 0x05;       CdceValidDividers[16] = 0x06;
    CdceValidDividers[18] = 0x09;       CdceValidDividers[20] = 0x07;
    CdceValidDividers[24] = 0x0a;       CdceValidDividers[28] = 0x18;
    CdceValidDividers[30] = 0x0b;       CdceValidDividers[32] = 0x0e;
    CdceValidDividers[36] = 0x15;       CdceValidDividers[40] = 0x0f;
    CdceValidDividers[42] = 0x19;       CdceValidDividers[48] = 0x16;
    CdceValidDividers[50] = 0x13;       CdceValidDividers[56] = 0x1a;
    CdceValidDividers[60] = 0x17;       CdceValidDividers[64] = 0x1e;
    CdceValidDividers[70] = 0x1b;       CdceValidDividers[80] = 0x1f;

    //  Fill the conversion information structures
    //     Valid dividers is keyed by Input Value, value is divide_by
    CdceInverseDividers[0x20]  =  1;       CdceInverseDividers[0x40]  =  2;
    CdceInverseDividers[0x41]  =  3;       CdceInverseDividers[0x42]  =  4;
    CdceInverseDividers[0x43]  =  5;       CdceInverseDividers[0x00]  =  4;
    CdceInverseDividers[0x01]  =  6;       CdceInverseDividers[0x02]  =  8;
    CdceInverseDividers[0x03]  =  10;      CdceInverseDividers[0x04]  =  8;
    CdceInverseDividers[0x05]  =  12;      CdceInverseDividers[0x06]  =  16;
    CdceInverseDividers[0x07]  =  20;      CdceInverseDividers[0x08]  =  12;
    CdceInverseDividers[0x09]  =  18;      CdceInverseDividers[0x0a]  =  24;
    CdceInverseDividers[0x0b]  =  30;      CdceInverseDividers[0x0c]  =  16;
    CdceInverseDividers[0x0d]  =  24;      CdceInverseDividers[0x0e]  =  32;
    CdceInverseDividers[0x0f]  =  40;      CdceInverseDividers[0x10]  =  20;
    CdceInverseDividers[0x11]  =  30;      CdceInverseDividers[0x12]  =  40;
    CdceInverseDividers[0x13]  =  50;      CdceInverseDividers[0x14]  =  24;
    CdceInverseDividers[0x15]  =  36;      CdceInverseDividers[0x16]  =  48;
    CdceInverseDividers[0x17]  =  60;      CdceInverseDividers[0x18]  =  28;
    CdceInverseDividers[0x19]  =  42;      CdceInverseDividers[0x1a]  =  56;
    CdceInverseDividers[0x1b]  =  70;      CdceInverseDividers[0x1c]  =  32;
    CdceInverseDividers[0x1d]  =  48;      CdceInverseDividers[0x1e]  =  64;
    CdceInverseDividers[0x1f]  =  80;

    //  Divide by 1 Phase Adjust values
    std::vector<unsigned int> DivBy_1_Phase;
    DivBy_1_Phase.push_back(0x00);
    CdcePhaseArray[1] = DivBy_1_Phase;

    //  Divide by 2 Phase Adjust values
    std::vector<unsigned int> DivBy_2_Phase;
    DivBy_2_Phase.push_back(0x00);      DivBy_2_Phase.push_back(0x01);
    CdcePhaseArray[2] = DivBy_2_Phase;

    //  Divide by 3 Phase Adjust values
    std::vector<unsigned int> DivBy_3_Phase;
    DivBy_3_Phase.push_back(0x00);      DivBy_3_Phase.push_back(0x01);
    DivBy_3_Phase.push_back(0x02);
    CdcePhaseArray[3] = DivBy_3_Phase;

    //  Divide by 4 Phase Adjust values
    std::vector<unsigned int> DivBy_4_Phase;
    DivBy_4_Phase.push_back(0x00);      DivBy_4_Phase.push_back(0x01);
    DivBy_4_Phase.push_back(0x02);      DivBy_4_Phase.push_back(0x03);
    CdcePhaseArray[4] = DivBy_4_Phase;

    //  Divide by 5 Phase Adjust values
    std::vector<unsigned int> DivBy_5_Phase;
    DivBy_5_Phase.push_back(0x00);      DivBy_5_Phase.push_back(0x01);
    DivBy_5_Phase.push_back(0x02);      DivBy_5_Phase.push_back(0x03);
    DivBy_5_Phase.push_back(0x04);
    CdcePhaseArray[5] = DivBy_5_Phase;

    //  Divide by 6 Phase Adjust values
    std::vector<unsigned int> DivBy_6_Phase;
    DivBy_6_Phase.push_back(0x00);      DivBy_6_Phase.push_back(0x01);
    DivBy_6_Phase.push_back(0x02);      DivBy_6_Phase.push_back(0x40);
    DivBy_6_Phase.push_back(0x41);      DivBy_6_Phase.push_back(0x42);
    CdcePhaseArray[6] = DivBy_6_Phase;

    //  Divide by 8 Phase Adjust values
    std::vector<unsigned int> DivBy_8_Phase;
    DivBy_8_Phase.push_back(0x00);      DivBy_8_Phase.push_back(0x01);
    DivBy_8_Phase.push_back(0x02);      DivBy_8_Phase.push_back(0x03);
    DivBy_8_Phase.push_back(0x40);      DivBy_8_Phase.push_back(0x41);
    DivBy_8_Phase.push_back(0x42);      DivBy_8_Phase.push_back(0x43);
    CdcePhaseArray[8] = DivBy_8_Phase;

    //  Divide by 10 Phase Adjust values
    std::vector<unsigned int> DivBy_10_Phase;
    DivBy_10_Phase.push_back(0x00);      DivBy_10_Phase.push_back(0x01);
    DivBy_10_Phase.push_back(0x02);      DivBy_10_Phase.push_back(0x03);
    DivBy_10_Phase.push_back(0x04);      DivBy_10_Phase.push_back(0x40);
    DivBy_10_Phase.push_back(0x41);      DivBy_10_Phase.push_back(0x42);
    DivBy_10_Phase.push_back(0x43);      DivBy_10_Phase.push_back(0x44);
    CdcePhaseArray[10] = DivBy_10_Phase;

    //  Divide by 12 Phase Adjust values
    std::vector<unsigned int> DivBy_12_Phase;
    DivBy_12_Phase.push_back(0x00);      DivBy_12_Phase.push_back(0x01);
    DivBy_12_Phase.push_back(0x02);      DivBy_12_Phase.push_back(0x08);
    DivBy_12_Phase.push_back(0x09);      DivBy_12_Phase.push_back(0x0a);
    DivBy_12_Phase.push_back(0x10);      DivBy_12_Phase.push_back(0x11);
    DivBy_12_Phase.push_back(0x12);      DivBy_12_Phase.push_back(0x18);
    DivBy_12_Phase.push_back(0x19);      DivBy_12_Phase.push_back(0x1a);
    CdcePhaseArray[12] = DivBy_12_Phase;

    //  Divide by 16 Phase Adjust values
    std::vector<unsigned int> DivBy_16_Phase;
    DivBy_16_Phase.push_back(0x00);      DivBy_16_Phase.push_back(0x01);
    DivBy_16_Phase.push_back(0x02);      DivBy_16_Phase.push_back(0x03);
    DivBy_16_Phase.push_back(0x08);      DivBy_16_Phase.push_back(0x09);
    DivBy_16_Phase.push_back(0x0a);      DivBy_16_Phase.push_back(0x0b);
    DivBy_16_Phase.push_back(0x10);      DivBy_16_Phase.push_back(0x11);
    DivBy_16_Phase.push_back(0x12);      DivBy_16_Phase.push_back(0x13);
    DivBy_16_Phase.push_back(0x18);      DivBy_16_Phase.push_back(0x19);
    DivBy_16_Phase.push_back(0x1a);      DivBy_16_Phase.push_back(0x1b);
    CdcePhaseArray[16] = DivBy_16_Phase;

    //  Divide by 18 Phase Adjust values
    std::vector<unsigned int> DivBy_18_Phase;
    DivBy_18_Phase.push_back(0x00);      DivBy_18_Phase.push_back(0x01);
    DivBy_18_Phase.push_back(0x02);      DivBy_18_Phase.push_back(0x08);
    DivBy_18_Phase.push_back(0x09);      DivBy_18_Phase.push_back(0x0a);
    DivBy_18_Phase.push_back(0x10);      DivBy_18_Phase.push_back(0x11);
    DivBy_18_Phase.push_back(0x12);      DivBy_18_Phase.push_back(0x18);
    DivBy_18_Phase.push_back(0x19);      DivBy_18_Phase.push_back(0x1a);
    DivBy_18_Phase.push_back(0x20);      DivBy_18_Phase.push_back(0x21);
    DivBy_18_Phase.push_back(0x22);      DivBy_18_Phase.push_back(0x28);
    DivBy_18_Phase.push_back(0x29);      DivBy_18_Phase.push_back(0x2a);
    CdcePhaseArray[18] = DivBy_18_Phase;

    //  Divide by 20 Phase Adjust values
    std::vector<unsigned int> DivBy_20_Phase;
    DivBy_20_Phase.push_back(0x00);      DivBy_20_Phase.push_back(0x01);
    DivBy_20_Phase.push_back(0x02);      DivBy_20_Phase.push_back(0x03);
    DivBy_20_Phase.push_back(0x04);      DivBy_20_Phase.push_back(0x08);
    DivBy_20_Phase.push_back(0x09);      DivBy_20_Phase.push_back(0x0a);
    DivBy_20_Phase.push_back(0x0b);      DivBy_20_Phase.push_back(0x0c);
    DivBy_20_Phase.push_back(0x10);      DivBy_20_Phase.push_back(0x11);
    DivBy_20_Phase.push_back(0x12);      DivBy_20_Phase.push_back(0x13);
    DivBy_20_Phase.push_back(0x14);      DivBy_20_Phase.push_back(0x18);
    DivBy_20_Phase.push_back(0x19);      DivBy_20_Phase.push_back(0x1a);
    DivBy_20_Phase.push_back(0x1b);      DivBy_20_Phase.push_back(0x1c);
    CdcePhaseArray[20] = DivBy_20_Phase;

    //  Divide by 24 Phase Adjust values
    std::vector<unsigned int> DivBy_24_Phase;
    DivBy_24_Phase.push_back(0x00);      DivBy_24_Phase.push_back(0x01);
    DivBy_24_Phase.push_back(0x02);      DivBy_24_Phase.push_back(0x03);
    DivBy_24_Phase.push_back(0x08);      DivBy_24_Phase.push_back(0x09);
    DivBy_24_Phase.push_back(0x0a);      DivBy_24_Phase.push_back(0x0b);
    DivBy_24_Phase.push_back(0x10);      DivBy_24_Phase.push_back(0x11);
    DivBy_24_Phase.push_back(0x12);      DivBy_24_Phase.push_back(0x13);
    DivBy_24_Phase.push_back(0x18);      DivBy_24_Phase.push_back(0x19);
    DivBy_24_Phase.push_back(0x1a);      DivBy_24_Phase.push_back(0x1b);
    DivBy_24_Phase.push_back(0x20);      DivBy_24_Phase.push_back(0x21);
    DivBy_24_Phase.push_back(0x22);      DivBy_24_Phase.push_back(0x23);
    DivBy_24_Phase.push_back(0x28);      DivBy_24_Phase.push_back(0x29);
    DivBy_24_Phase.push_back(0x2a);      DivBy_24_Phase.push_back(0x2b);
    CdcePhaseArray[24] = DivBy_24_Phase;

    //  Divide by 28 Phase Adjust values
    std::vector<unsigned int> DivBy_28_Phase;
    DivBy_28_Phase.push_back(0x00);      DivBy_28_Phase.push_back(0x01);
    DivBy_28_Phase.push_back(0x08);      DivBy_28_Phase.push_back(0x09);
    DivBy_28_Phase.push_back(0x10);      DivBy_28_Phase.push_back(0x11);
    DivBy_28_Phase.push_back(0x18);      DivBy_28_Phase.push_back(0x19);
    DivBy_28_Phase.push_back(0x20);      DivBy_28_Phase.push_back(0x21);
    DivBy_28_Phase.push_back(0x28);      DivBy_28_Phase.push_back(0x29);
    DivBy_28_Phase.push_back(0x30);      DivBy_28_Phase.push_back(0x31);
    DivBy_28_Phase.push_back(0x40);      DivBy_28_Phase.push_back(0x41);
    DivBy_28_Phase.push_back(0x48);      DivBy_28_Phase.push_back(0x49);
    DivBy_28_Phase.push_back(0x50);      DivBy_28_Phase.push_back(0x51);
    DivBy_28_Phase.push_back(0x58);      DivBy_28_Phase.push_back(0x59);
    DivBy_28_Phase.push_back(0x60);      DivBy_28_Phase.push_back(0x61);
    DivBy_28_Phase.push_back(0x68);      DivBy_28_Phase.push_back(0x69);
    DivBy_28_Phase.push_back(0x70);      DivBy_28_Phase.push_back(0x71);
    CdcePhaseArray[28] = DivBy_28_Phase;

    //  Divide by 30 Phase Adjust values
    std::vector<unsigned int> DivBy_30_Phase;
    DivBy_30_Phase.push_back(0x00);      DivBy_30_Phase.push_back(0x01);
    DivBy_30_Phase.push_back(0x02);      DivBy_30_Phase.push_back(0x03);
    DivBy_30_Phase.push_back(0x04);      DivBy_30_Phase.push_back(0x08);
    DivBy_30_Phase.push_back(0x09);      DivBy_30_Phase.push_back(0x0a);
    DivBy_30_Phase.push_back(0x0b);      DivBy_30_Phase.push_back(0x0c);
    DivBy_30_Phase.push_back(0x10);      DivBy_30_Phase.push_back(0x11);
    DivBy_30_Phase.push_back(0x12);      DivBy_30_Phase.push_back(0x13);
    DivBy_30_Phase.push_back(0x14);      DivBy_30_Phase.push_back(0x18);
    DivBy_30_Phase.push_back(0x19);      DivBy_30_Phase.push_back(0x1a);
    DivBy_30_Phase.push_back(0x1b);      DivBy_30_Phase.push_back(0x1c);
    DivBy_30_Phase.push_back(0x20);      DivBy_30_Phase.push_back(0x21);
    DivBy_30_Phase.push_back(0x22);      DivBy_30_Phase.push_back(0x23);
    DivBy_30_Phase.push_back(0x24);      DivBy_30_Phase.push_back(0x28);
    DivBy_30_Phase.push_back(0x29);      DivBy_30_Phase.push_back(0x2a);
    DivBy_30_Phase.push_back(0x2b);      DivBy_30_Phase.push_back(0x2c);
    CdcePhaseArray[30] = DivBy_30_Phase;

    //  Divide by 32 Phase Adjust values
    std::vector<unsigned int> DivBy_32_Phase;
    DivBy_32_Phase.push_back(0x00);      DivBy_32_Phase.push_back(0x01);
    DivBy_32_Phase.push_back(0x02);      DivBy_32_Phase.push_back(0x03);
    DivBy_32_Phase.push_back(0x08);      DivBy_32_Phase.push_back(0x09);
    DivBy_32_Phase.push_back(0x0a);      DivBy_32_Phase.push_back(0x0b);
    DivBy_32_Phase.push_back(0x10);      DivBy_32_Phase.push_back(0x11);
    DivBy_32_Phase.push_back(0x12);      DivBy_32_Phase.push_back(0x13);
    DivBy_32_Phase.push_back(0x18);      DivBy_32_Phase.push_back(0x19);
    DivBy_32_Phase.push_back(0x1a);      DivBy_32_Phase.push_back(0x1b);
    DivBy_32_Phase.push_back(0x20);      DivBy_32_Phase.push_back(0x21);
    DivBy_32_Phase.push_back(0x22);      DivBy_32_Phase.push_back(0x23);
    DivBy_32_Phase.push_back(0x28);      DivBy_32_Phase.push_back(0x29);
    DivBy_32_Phase.push_back(0x2a);      DivBy_32_Phase.push_back(0x2b);
    DivBy_32_Phase.push_back(0x30);      DivBy_32_Phase.push_back(0x31);
    DivBy_32_Phase.push_back(0x32);      DivBy_32_Phase.push_back(0x33);
    DivBy_32_Phase.push_back(0x38);      DivBy_32_Phase.push_back(0x39);
    DivBy_32_Phase.push_back(0x3a);      DivBy_32_Phase.push_back(0x3b);
    CdcePhaseArray[32] = DivBy_32_Phase;

    //  Divide by 36 Phase Adjust values
    std::vector<unsigned int> DivBy_36_Phase;
    DivBy_36_Phase.push_back(0x00);      DivBy_36_Phase.push_back(0x01);
    DivBy_36_Phase.push_back(0x02);      DivBy_36_Phase.push_back(0x08);
    DivBy_36_Phase.push_back(0x09);      DivBy_36_Phase.push_back(0x0a);
    DivBy_36_Phase.push_back(0x10);      DivBy_36_Phase.push_back(0x11);
    DivBy_36_Phase.push_back(0x12);      DivBy_36_Phase.push_back(0x18);
    DivBy_36_Phase.push_back(0x19);      DivBy_36_Phase.push_back(0x1a);
    DivBy_36_Phase.push_back(0x20);      DivBy_36_Phase.push_back(0x21);
    DivBy_36_Phase.push_back(0x22);      DivBy_36_Phase.push_back(0x28);
    DivBy_36_Phase.push_back(0x29);      DivBy_36_Phase.push_back(0x2a);
    DivBy_36_Phase.push_back(0x40);      DivBy_36_Phase.push_back(0x41);
    DivBy_36_Phase.push_back(0x42);      DivBy_36_Phase.push_back(0x48);
    DivBy_36_Phase.push_back(0x49);      DivBy_36_Phase.push_back(0x4a);
    DivBy_36_Phase.push_back(0x50);      DivBy_36_Phase.push_back(0x51);
    DivBy_36_Phase.push_back(0x52);      DivBy_36_Phase.push_back(0x58);
    DivBy_36_Phase.push_back(0x59);      DivBy_36_Phase.push_back(0x5a);
    DivBy_36_Phase.push_back(0x60);      DivBy_36_Phase.push_back(0x61);
    DivBy_36_Phase.push_back(0x62);      DivBy_36_Phase.push_back(0x68);
    DivBy_36_Phase.push_back(0x69);      DivBy_36_Phase.push_back(0x6a);
    CdcePhaseArray[36] = DivBy_36_Phase;

    //  Divide by 40 Phase Adjust values
    std::vector<unsigned int> DivBy_40_Phase;
    DivBy_40_Phase.push_back(0x00);      DivBy_40_Phase.push_back(0x01);
    DivBy_40_Phase.push_back(0x02);      DivBy_40_Phase.push_back(0x03);
    DivBy_40_Phase.push_back(0x04);      DivBy_40_Phase.push_back(0x08);
    DivBy_40_Phase.push_back(0x09);      DivBy_40_Phase.push_back(0x0a);
    DivBy_40_Phase.push_back(0x0b);      DivBy_40_Phase.push_back(0x0c);
    DivBy_40_Phase.push_back(0x10);      DivBy_40_Phase.push_back(0x11);
    DivBy_40_Phase.push_back(0x12);      DivBy_40_Phase.push_back(0x13);
    DivBy_40_Phase.push_back(0x14);      DivBy_40_Phase.push_back(0x18);
    DivBy_40_Phase.push_back(0x19);      DivBy_40_Phase.push_back(0x1a);
    DivBy_40_Phase.push_back(0x1b);      DivBy_40_Phase.push_back(0x1c);
    DivBy_40_Phase.push_back(0x20);      DivBy_40_Phase.push_back(0x21);
    DivBy_40_Phase.push_back(0x22);      DivBy_40_Phase.push_back(0x23);
    DivBy_40_Phase.push_back(0x24);      DivBy_40_Phase.push_back(0x28);
    DivBy_40_Phase.push_back(0x29);      DivBy_40_Phase.push_back(0x2a);
    DivBy_40_Phase.push_back(0x2b);      DivBy_40_Phase.push_back(0x2c);
    DivBy_40_Phase.push_back(0x30);      DivBy_40_Phase.push_back(0x31);
    DivBy_40_Phase.push_back(0x32);      DivBy_40_Phase.push_back(0x33);
    DivBy_40_Phase.push_back(0x34);      DivBy_40_Phase.push_back(0x38);
    DivBy_40_Phase.push_back(0x39);      DivBy_40_Phase.push_back(0x3a);
    DivBy_40_Phase.push_back(0x3b);      DivBy_40_Phase.push_back(0x3c);
    CdcePhaseArray[40] = DivBy_40_Phase;

    //  Divide by 42 Phase Adjust values
    std::vector<unsigned int> DivBy_42_Phase;
    DivBy_42_Phase.push_back(0x00);      DivBy_42_Phase.push_back(0x01);
    DivBy_42_Phase.push_back(0x02);      DivBy_42_Phase.push_back(0x08);
    DivBy_42_Phase.push_back(0x09);      DivBy_42_Phase.push_back(0x0a);
    DivBy_42_Phase.push_back(0x10);      DivBy_42_Phase.push_back(0x11);
    DivBy_42_Phase.push_back(0x12);      DivBy_42_Phase.push_back(0x18);
    DivBy_42_Phase.push_back(0x19);      DivBy_42_Phase.push_back(0x1a);
    DivBy_42_Phase.push_back(0x20);      DivBy_42_Phase.push_back(0x21);
    DivBy_42_Phase.push_back(0x22);      DivBy_42_Phase.push_back(0x28);
    DivBy_42_Phase.push_back(0x29);      DivBy_42_Phase.push_back(0x2a);
    DivBy_42_Phase.push_back(0x30);      DivBy_42_Phase.push_back(0x31);
    DivBy_42_Phase.push_back(0x32);      DivBy_42_Phase.push_back(0x40);
    DivBy_42_Phase.push_back(0x41);      DivBy_42_Phase.push_back(0x42);
    DivBy_42_Phase.push_back(0x48);      DivBy_42_Phase.push_back(0x49);
    DivBy_42_Phase.push_back(0x4a);      DivBy_42_Phase.push_back(0x50);
    DivBy_42_Phase.push_back(0x51);      DivBy_42_Phase.push_back(0x52);
    DivBy_42_Phase.push_back(0x58);      DivBy_42_Phase.push_back(0x59);
    DivBy_42_Phase.push_back(0x5a);      DivBy_42_Phase.push_back(0x60);
    DivBy_42_Phase.push_back(0x61);      DivBy_42_Phase.push_back(0x62);
    DivBy_42_Phase.push_back(0x68);      DivBy_42_Phase.push_back(0x69);
    DivBy_42_Phase.push_back(0x6a);      DivBy_42_Phase.push_back(0x70);
    DivBy_42_Phase.push_back(0x71);      DivBy_42_Phase.push_back(0x72);
    CdcePhaseArray[42] = DivBy_42_Phase;

    //  Divide by 48 Phase Adjust values
    std::vector<unsigned int> DivBy_48_Phase;
    DivBy_48_Phase.push_back(0x00);      DivBy_48_Phase.push_back(0x01);
    DivBy_48_Phase.push_back(0x02);      DivBy_48_Phase.push_back(0x03);
    DivBy_48_Phase.push_back(0x08);      DivBy_48_Phase.push_back(0x09);
    DivBy_48_Phase.push_back(0x0a);      DivBy_48_Phase.push_back(0x0b);
    DivBy_48_Phase.push_back(0x10);      DivBy_48_Phase.push_back(0x11);
    DivBy_48_Phase.push_back(0x12);      DivBy_48_Phase.push_back(0x13);
    DivBy_48_Phase.push_back(0x18);      DivBy_48_Phase.push_back(0x19);
    DivBy_48_Phase.push_back(0x1a);      DivBy_48_Phase.push_back(0x1b);
    DivBy_48_Phase.push_back(0x20);      DivBy_48_Phase.push_back(0x21);
    DivBy_48_Phase.push_back(0x22);      DivBy_48_Phase.push_back(0x23);
    DivBy_48_Phase.push_back(0x28);      DivBy_48_Phase.push_back(0x29);
    DivBy_48_Phase.push_back(0x2a);      DivBy_48_Phase.push_back(0x2b);
    DivBy_48_Phase.push_back(0x40);      DivBy_48_Phase.push_back(0x41);
    DivBy_48_Phase.push_back(0x42);      DivBy_48_Phase.push_back(0x43);
    DivBy_48_Phase.push_back(0x48);      DivBy_48_Phase.push_back(0x49);
    DivBy_48_Phase.push_back(0x4a);      DivBy_48_Phase.push_back(0x4b);
    DivBy_48_Phase.push_back(0x50);      DivBy_48_Phase.push_back(0x51);
    DivBy_48_Phase.push_back(0x52);      DivBy_48_Phase.push_back(0x53);
    DivBy_48_Phase.push_back(0x58);      DivBy_48_Phase.push_back(0x59);
    DivBy_48_Phase.push_back(0x5a);      DivBy_48_Phase.push_back(0x5b);
    DivBy_48_Phase.push_back(0x60);      DivBy_48_Phase.push_back(0x61);
    DivBy_48_Phase.push_back(0x62);      DivBy_48_Phase.push_back(0x63);
    DivBy_48_Phase.push_back(0x68);      DivBy_48_Phase.push_back(0x69);
    DivBy_48_Phase.push_back(0x6a);      DivBy_48_Phase.push_back(0x6b);
    CdcePhaseArray[48] = DivBy_48_Phase;

    //  Divide by 50 Phase Adjust values
    std::vector<unsigned int> DivBy_50_Phase;
    DivBy_50_Phase.push_back(0x00);      DivBy_50_Phase.push_back(0x01);
    DivBy_50_Phase.push_back(0x02);      DivBy_50_Phase.push_back(0x03);
    DivBy_50_Phase.push_back(0x04);      DivBy_50_Phase.push_back(0x08);
    DivBy_50_Phase.push_back(0x09);      DivBy_50_Phase.push_back(0x0a);
    DivBy_50_Phase.push_back(0x0b);      DivBy_50_Phase.push_back(0x0c);
    DivBy_50_Phase.push_back(0x10);      DivBy_50_Phase.push_back(0x11);
    DivBy_50_Phase.push_back(0x12);      DivBy_50_Phase.push_back(0x13);
    DivBy_50_Phase.push_back(0x14);      DivBy_50_Phase.push_back(0x18);
    DivBy_50_Phase.push_back(0x19);      DivBy_50_Phase.push_back(0x1a);
    DivBy_50_Phase.push_back(0x1b);      DivBy_50_Phase.push_back(0x1c);
    DivBy_50_Phase.push_back(0x20);      DivBy_50_Phase.push_back(0x21);
    DivBy_50_Phase.push_back(0x22);      DivBy_50_Phase.push_back(0x23);
    DivBy_50_Phase.push_back(0x24);      DivBy_50_Phase.push_back(0x40);
    DivBy_50_Phase.push_back(0x41);      DivBy_50_Phase.push_back(0x42);
    DivBy_50_Phase.push_back(0x43);      DivBy_50_Phase.push_back(0x44);
    DivBy_50_Phase.push_back(0x48);      DivBy_50_Phase.push_back(0x49);
    DivBy_50_Phase.push_back(0x4a);      DivBy_50_Phase.push_back(0x4b);
    DivBy_50_Phase.push_back(0x4c);      DivBy_50_Phase.push_back(0x50);
    DivBy_50_Phase.push_back(0x51);      DivBy_50_Phase.push_back(0x52);
    DivBy_50_Phase.push_back(0x53);      DivBy_50_Phase.push_back(0x54);
    DivBy_50_Phase.push_back(0x58);      DivBy_50_Phase.push_back(0x59);
    DivBy_50_Phase.push_back(0x5a);      DivBy_50_Phase.push_back(0x5b);
    DivBy_50_Phase.push_back(0x5c);      DivBy_50_Phase.push_back(0x60);
    DivBy_50_Phase.push_back(0x61);      DivBy_50_Phase.push_back(0x62);
    DivBy_50_Phase.push_back(0x63);      DivBy_50_Phase.push_back(0x64);
    CdcePhaseArray[50] = DivBy_50_Phase;

    //  Divide by 56 Phase Adjust values
    std::vector<unsigned int> DivBy_56_Phase;
    DivBy_56_Phase.push_back(0x00);      DivBy_56_Phase.push_back(0x01);
    DivBy_56_Phase.push_back(0x02);      DivBy_56_Phase.push_back(0x03);
    DivBy_56_Phase.push_back(0x08);      DivBy_56_Phase.push_back(0x09);
    DivBy_56_Phase.push_back(0x0a);      DivBy_56_Phase.push_back(0x0b);
    DivBy_56_Phase.push_back(0x10);      DivBy_56_Phase.push_back(0x11);
    DivBy_56_Phase.push_back(0x12);      DivBy_56_Phase.push_back(0x13);
    DivBy_56_Phase.push_back(0x18);      DivBy_56_Phase.push_back(0x19);
    DivBy_56_Phase.push_back(0x1a);      DivBy_56_Phase.push_back(0x1b);
    DivBy_56_Phase.push_back(0x20);      DivBy_56_Phase.push_back(0x21);
    DivBy_56_Phase.push_back(0x22);      DivBy_56_Phase.push_back(0x23);
    DivBy_56_Phase.push_back(0x28);      DivBy_56_Phase.push_back(0x29);
    DivBy_56_Phase.push_back(0x2a);      DivBy_56_Phase.push_back(0x2b);
    DivBy_56_Phase.push_back(0x30);      DivBy_56_Phase.push_back(0x31);
    DivBy_56_Phase.push_back(0x32);      DivBy_56_Phase.push_back(0x33);
    DivBy_56_Phase.push_back(0x40);      DivBy_56_Phase.push_back(0x41);
    DivBy_56_Phase.push_back(0x42);      DivBy_56_Phase.push_back(0x43);
    DivBy_56_Phase.push_back(0x48);      DivBy_56_Phase.push_back(0x49);
    DivBy_56_Phase.push_back(0x4a);      DivBy_56_Phase.push_back(0x4b);
    DivBy_56_Phase.push_back(0x50);      DivBy_56_Phase.push_back(0x51);
    DivBy_56_Phase.push_back(0x52);      DivBy_56_Phase.push_back(0x53);
    DivBy_56_Phase.push_back(0x58);      DivBy_56_Phase.push_back(0x59);
    DivBy_56_Phase.push_back(0x5a);      DivBy_56_Phase.push_back(0x5b);
    DivBy_56_Phase.push_back(0x60);      DivBy_56_Phase.push_back(0x61);
    DivBy_56_Phase.push_back(0x62);      DivBy_56_Phase.push_back(0x63);
    DivBy_56_Phase.push_back(0x68);      DivBy_56_Phase.push_back(0x69);
    DivBy_56_Phase.push_back(0x6a);      DivBy_56_Phase.push_back(0x6b);
    DivBy_56_Phase.push_back(0x70);      DivBy_56_Phase.push_back(0x71);
    DivBy_56_Phase.push_back(0x72);      DivBy_56_Phase.push_back(0x73);
    CdcePhaseArray[56] = DivBy_56_Phase;

    //  Divide by 60 Phase Adjust values
    std::vector<unsigned int> DivBy_60_Phase;
    DivBy_60_Phase.push_back(0x00);      DivBy_60_Phase.push_back(0x01);
    DivBy_60_Phase.push_back(0x02);      DivBy_60_Phase.push_back(0x03);
    DivBy_60_Phase.push_back(0x04);      DivBy_60_Phase.push_back(0x08);
    DivBy_60_Phase.push_back(0x09);      DivBy_60_Phase.push_back(0x0a);
    DivBy_60_Phase.push_back(0x0b);      DivBy_60_Phase.push_back(0x0c);
    DivBy_60_Phase.push_back(0x10);      DivBy_60_Phase.push_back(0x11);
    DivBy_60_Phase.push_back(0x12);      DivBy_60_Phase.push_back(0x13);
    DivBy_60_Phase.push_back(0x14);      DivBy_60_Phase.push_back(0x18);
    DivBy_60_Phase.push_back(0x19);      DivBy_60_Phase.push_back(0x1a);
    DivBy_60_Phase.push_back(0x1b);      DivBy_60_Phase.push_back(0x1c);
    DivBy_60_Phase.push_back(0x20);      DivBy_60_Phase.push_back(0x21);
    DivBy_60_Phase.push_back(0x22);      DivBy_60_Phase.push_back(0x23);
    DivBy_60_Phase.push_back(0x24);      DivBy_60_Phase.push_back(0x28);
    DivBy_60_Phase.push_back(0x29);      DivBy_60_Phase.push_back(0x2a);
    DivBy_60_Phase.push_back(0x2b);      DivBy_60_Phase.push_back(0x2c);
    DivBy_60_Phase.push_back(0x40);      DivBy_60_Phase.push_back(0x41);
    DivBy_60_Phase.push_back(0x42);      DivBy_60_Phase.push_back(0x43);
    DivBy_60_Phase.push_back(0x44);      DivBy_60_Phase.push_back(0x48);
    DivBy_60_Phase.push_back(0x49);      DivBy_60_Phase.push_back(0x4a);
    DivBy_60_Phase.push_back(0x4b);      DivBy_60_Phase.push_back(0x4c);
    DivBy_60_Phase.push_back(0x50);      DivBy_60_Phase.push_back(0x51);
    DivBy_60_Phase.push_back(0x52);      DivBy_60_Phase.push_back(0x53);
    DivBy_60_Phase.push_back(0x54);      DivBy_60_Phase.push_back(0x58);
    DivBy_60_Phase.push_back(0x59);      DivBy_60_Phase.push_back(0x5a);
    DivBy_60_Phase.push_back(0x5b);      DivBy_60_Phase.push_back(0x5c);
    DivBy_60_Phase.push_back(0x60);      DivBy_60_Phase.push_back(0x61);
    DivBy_60_Phase.push_back(0x62);      DivBy_60_Phase.push_back(0x63);
    DivBy_60_Phase.push_back(0x64);      DivBy_60_Phase.push_back(0x68);
    DivBy_60_Phase.push_back(0x69);      DivBy_60_Phase.push_back(0x6a);
    DivBy_60_Phase.push_back(0x6b);      DivBy_60_Phase.push_back(0x6c);
    CdcePhaseArray[60] = DivBy_60_Phase;

    //  Divide by 64 Phase Adjust values
    std::vector<unsigned int> DivBy_64_Phase;
    DivBy_64_Phase.push_back(0x00);      DivBy_64_Phase.push_back(0x01);
    DivBy_64_Phase.push_back(0x02);      DivBy_64_Phase.push_back(0x03);
    DivBy_64_Phase.push_back(0x08);      DivBy_64_Phase.push_back(0x09);
    DivBy_64_Phase.push_back(0x0a);      DivBy_64_Phase.push_back(0x0b);
    DivBy_64_Phase.push_back(0x10);      DivBy_64_Phase.push_back(0x11);
    DivBy_64_Phase.push_back(0x12);      DivBy_64_Phase.push_back(0x13);
    DivBy_64_Phase.push_back(0x18);      DivBy_64_Phase.push_back(0x19);
    DivBy_64_Phase.push_back(0x1a);      DivBy_64_Phase.push_back(0x1b);
    DivBy_64_Phase.push_back(0x20);      DivBy_64_Phase.push_back(0x21);
    DivBy_64_Phase.push_back(0x22);      DivBy_64_Phase.push_back(0x23);
    DivBy_64_Phase.push_back(0x28);      DivBy_64_Phase.push_back(0x29);
    DivBy_64_Phase.push_back(0x2a);      DivBy_64_Phase.push_back(0x2b);
    DivBy_64_Phase.push_back(0x30);      DivBy_64_Phase.push_back(0x31);
    DivBy_64_Phase.push_back(0x32);      DivBy_64_Phase.push_back(0x33);
    DivBy_64_Phase.push_back(0x38);      DivBy_64_Phase.push_back(0x39);
    DivBy_64_Phase.push_back(0x3a);      DivBy_64_Phase.push_back(0x3b);
    DivBy_64_Phase.push_back(0x40);      DivBy_64_Phase.push_back(0x41);
    DivBy_64_Phase.push_back(0x42);      DivBy_64_Phase.push_back(0x43);
    DivBy_64_Phase.push_back(0x48);      DivBy_64_Phase.push_back(0x49);
    DivBy_64_Phase.push_back(0x4a);      DivBy_64_Phase.push_back(0x4b);
    DivBy_64_Phase.push_back(0x50);      DivBy_64_Phase.push_back(0x51);
    DivBy_64_Phase.push_back(0x52);      DivBy_64_Phase.push_back(0x53);
    DivBy_64_Phase.push_back(0x58);      DivBy_64_Phase.push_back(0x59);
    DivBy_64_Phase.push_back(0x5a);      DivBy_64_Phase.push_back(0x5b);
    DivBy_64_Phase.push_back(0x60);      DivBy_64_Phase.push_back(0x61);
    DivBy_64_Phase.push_back(0x62);      DivBy_64_Phase.push_back(0x63);
    DivBy_64_Phase.push_back(0x68);      DivBy_64_Phase.push_back(0x69);
    DivBy_64_Phase.push_back(0x6a);      DivBy_64_Phase.push_back(0x6b);
    DivBy_64_Phase.push_back(0x70);      DivBy_64_Phase.push_back(0x71);
    DivBy_64_Phase.push_back(0x72);      DivBy_64_Phase.push_back(0x73);
    DivBy_64_Phase.push_back(0x78);      DivBy_64_Phase.push_back(0x79);
    DivBy_64_Phase.push_back(0x7a);      DivBy_64_Phase.push_back(0x7b);
    CdcePhaseArray[64] = DivBy_64_Phase;

    //  Divide by 70 Phase Adjust values
    std::vector<unsigned int> DivBy_70_Phase;
    DivBy_70_Phase.push_back(0x00);      DivBy_70_Phase.push_back(0x01);
    DivBy_70_Phase.push_back(0x02);      DivBy_70_Phase.push_back(0x03);
    DivBy_70_Phase.push_back(0x04);      DivBy_70_Phase.push_back(0x08);
    DivBy_70_Phase.push_back(0x09);      DivBy_70_Phase.push_back(0x0a);
    DivBy_70_Phase.push_back(0x0b);      DivBy_70_Phase.push_back(0x0c);
    DivBy_70_Phase.push_back(0x10);      DivBy_70_Phase.push_back(0x11);
    DivBy_70_Phase.push_back(0x12);      DivBy_70_Phase.push_back(0x13);
    DivBy_70_Phase.push_back(0x14);      DivBy_70_Phase.push_back(0x18);
    DivBy_70_Phase.push_back(0x19);      DivBy_70_Phase.push_back(0x1a);
    DivBy_70_Phase.push_back(0x1b);      DivBy_70_Phase.push_back(0x1c);
    DivBy_70_Phase.push_back(0x20);      DivBy_70_Phase.push_back(0x21);
    DivBy_70_Phase.push_back(0x22);      DivBy_70_Phase.push_back(0x23);
    DivBy_70_Phase.push_back(0x24);      DivBy_70_Phase.push_back(0x28);
    DivBy_70_Phase.push_back(0x29);      DivBy_70_Phase.push_back(0x2a);
    DivBy_70_Phase.push_back(0x2b);      DivBy_70_Phase.push_back(0x2c);
    DivBy_70_Phase.push_back(0x30);      DivBy_70_Phase.push_back(0x31);
    DivBy_70_Phase.push_back(0x32);      DivBy_70_Phase.push_back(0x33);
    DivBy_70_Phase.push_back(0x34);      DivBy_70_Phase.push_back(0x40);
    DivBy_70_Phase.push_back(0x41);      DivBy_70_Phase.push_back(0x42);
    DivBy_70_Phase.push_back(0x43);      DivBy_70_Phase.push_back(0x44);
    DivBy_70_Phase.push_back(0x48);      DivBy_70_Phase.push_back(0x49);
    DivBy_70_Phase.push_back(0x4a);      DivBy_70_Phase.push_back(0x4b);
    DivBy_70_Phase.push_back(0x4c);      DivBy_70_Phase.push_back(0x50);
    DivBy_70_Phase.push_back(0x51);      DivBy_70_Phase.push_back(0x52);
    DivBy_70_Phase.push_back(0x53);      DivBy_70_Phase.push_back(0x54);
    DivBy_70_Phase.push_back(0x58);      DivBy_70_Phase.push_back(0x59);
    DivBy_70_Phase.push_back(0x5a);      DivBy_70_Phase.push_back(0x5b);
    DivBy_70_Phase.push_back(0x5c);      DivBy_70_Phase.push_back(0x60);
    DivBy_70_Phase.push_back(0x61);      DivBy_70_Phase.push_back(0x62);
    DivBy_70_Phase.push_back(0x63);      DivBy_70_Phase.push_back(0x64);
    DivBy_70_Phase.push_back(0x68);      DivBy_70_Phase.push_back(0x69);
    DivBy_70_Phase.push_back(0x6a);      DivBy_70_Phase.push_back(0x6b);
    DivBy_70_Phase.push_back(0x6c);      DivBy_70_Phase.push_back(0x70);
    DivBy_70_Phase.push_back(0x71);      DivBy_70_Phase.push_back(0x72);
    DivBy_70_Phase.push_back(0x73);      DivBy_70_Phase.push_back(0x74);
    CdcePhaseArray[70] = DivBy_70_Phase;

    //  Divide by 80 Phase Adjust values
    std::vector<unsigned int> DivBy_80_Phase;
    DivBy_80_Phase.push_back(0x00);      DivBy_80_Phase.push_back(0x01);
    DivBy_80_Phase.push_back(0x02);      DivBy_80_Phase.push_back(0x03);
    DivBy_80_Phase.push_back(0x04);      DivBy_80_Phase.push_back(0x08);
    DivBy_80_Phase.push_back(0x09);      DivBy_80_Phase.push_back(0x0a);
    DivBy_80_Phase.push_back(0x0b);      DivBy_80_Phase.push_back(0x0c);
    DivBy_80_Phase.push_back(0x10);      DivBy_80_Phase.push_back(0x11);
    DivBy_80_Phase.push_back(0x12);      DivBy_80_Phase.push_back(0x13);
    DivBy_80_Phase.push_back(0x14);      DivBy_80_Phase.push_back(0x18);
    DivBy_80_Phase.push_back(0x19);      DivBy_80_Phase.push_back(0x1a);
    DivBy_80_Phase.push_back(0x1b);      DivBy_80_Phase.push_back(0x1c);
    DivBy_80_Phase.push_back(0x20);      DivBy_80_Phase.push_back(0x21);
    DivBy_80_Phase.push_back(0x22);      DivBy_80_Phase.push_back(0x23);
    DivBy_80_Phase.push_back(0x24);      DivBy_80_Phase.push_back(0x28);
    DivBy_80_Phase.push_back(0x29);      DivBy_80_Phase.push_back(0x2a);
    DivBy_80_Phase.push_back(0x2b);      DivBy_80_Phase.push_back(0x2c);
    DivBy_80_Phase.push_back(0x30);      DivBy_80_Phase.push_back(0x31);
    DivBy_80_Phase.push_back(0x32);      DivBy_80_Phase.push_back(0x33);
    DivBy_80_Phase.push_back(0x34);      DivBy_80_Phase.push_back(0x38);
    DivBy_80_Phase.push_back(0x39);      DivBy_80_Phase.push_back(0x3a);
    DivBy_80_Phase.push_back(0x3b);      DivBy_80_Phase.push_back(0x3c);
    DivBy_80_Phase.push_back(0x40);      DivBy_80_Phase.push_back(0x41);
    DivBy_80_Phase.push_back(0x42);      DivBy_80_Phase.push_back(0x43);
    DivBy_80_Phase.push_back(0x44);      DivBy_80_Phase.push_back(0x48);
    DivBy_80_Phase.push_back(0x49);      DivBy_80_Phase.push_back(0x4a);
    DivBy_80_Phase.push_back(0x4b);      DivBy_80_Phase.push_back(0x4c);
    DivBy_80_Phase.push_back(0x50);      DivBy_80_Phase.push_back(0x51);
    DivBy_80_Phase.push_back(0x52);      DivBy_80_Phase.push_back(0x53);
    DivBy_80_Phase.push_back(0x54);      DivBy_80_Phase.push_back(0x58);
    DivBy_80_Phase.push_back(0x59);      DivBy_80_Phase.push_back(0x5a);
    DivBy_80_Phase.push_back(0x5b);      DivBy_80_Phase.push_back(0x5c);
    DivBy_80_Phase.push_back(0x60);      DivBy_80_Phase.push_back(0x61);
    DivBy_80_Phase.push_back(0x62);      DivBy_80_Phase.push_back(0x63);
    DivBy_80_Phase.push_back(0x64);      DivBy_80_Phase.push_back(0x68);
    DivBy_80_Phase.push_back(0x69);      DivBy_80_Phase.push_back(0x6a);
    DivBy_80_Phase.push_back(0x6b);      DivBy_80_Phase.push_back(0x6c);
    DivBy_80_Phase.push_back(0x70);      DivBy_80_Phase.push_back(0x71);
    DivBy_80_Phase.push_back(0x72);      DivBy_80_Phase.push_back(0x73);
    DivBy_80_Phase.push_back(0x74);      DivBy_80_Phase.push_back(0x78);
    DivBy_80_Phase.push_back(0x79);      DivBy_80_Phase.push_back(0x7a);
    DivBy_80_Phase.push_back(0x7b);      DivBy_80_Phase.push_back(0x7c);
    CdcePhaseArray[80] = DivBy_80_Phase;
}

//-----------------------------------------------------------------------------
//  CdceConversion::ToDivider() -- find divide_by value from list of dividers
//-----------------------------------------------------------------------------

unsigned int  CdceConversion::ToDivider(unsigned int divider_idx)
{
	return CdceDividerValues[divider_idx];
}

//-----------------------------------------------------------------------------
//  CdceConversion::ToDividerIdx() -- find index in list from divide_by value
//-----------------------------------------------------------------------------

unsigned int  CdceConversion::ToDividerIdx(unsigned int divider)
{
    ValidDividerArray::iterator iter =
        std::find(CdceDividerValues.begin(), CdceDividerValues.end(), divider);

    if (iter==CdceDividerValues.end())
        return 0;  // not found, return first divider
    else
        return static_cast<unsigned int>(iter - CdceDividerValues.begin());   // convert to IDX
}

//-----------------------------------------------------------------------------
//  CdceConversion::IsValidDivider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  CdceConversion::IsValidDivider(unsigned int divide_by)
{
	ValidDividerMap::iterator iter = CdceValidDividers.find(divide_by);
	return iter != CdceValidDividers.end();
}

//-----------------------------------------------------------------------------
//  CdceConversion::BestDividerGreaterThan() -- returns next largest valid Divider
//-----------------------------------------------------------------------------

unsigned int  CdceConversion::BestDividerGreaterThan(float min_divide_by)
{
    unsigned int CandidateD = 0;
    for (ValidDividerArray::iterator div = CdceDividerValues.begin();
             div !=  CdceDividerValues.end();  ++div)
        {
        CandidateD = *div;
        if (static_cast<float>(CandidateD) > min_divide_by)
            break;
        }
    return CandidateD;
}

//-----------------------------------------------------------------------------
//  CdceConversion::BestDividerLessThan() -- returns next smallest valid Divider
//-----------------------------------------------------------------------------

unsigned int  CdceConversion::BestDividerLessThan(float max_divide_by)
{
    unsigned int CandidateD = 0;
    for (ValidDividerArray::reverse_iterator div = CdceDividerValues.rbegin();
             div !=  CdceDividerValues.rend();  ++div)
        {
        CandidateD = *div;
        if (static_cast<float>(CandidateD) < max_divide_by)
            break;
        }
    return CandidateD;
}

//-----------------------------------------------------------------------------
//  CdceConversion::ToDividerInput() -- Returns input value for divide_by
//-----------------------------------------------------------------------------
//  this maps divider ratios to the field value in the chip hardware

unsigned int  CdceConversion::ToDividerInput(unsigned int divide_by)
{
	if (IsValidDivider(divide_by))
        return CdceValidDividers[divide_by];
    else
        return CdceValidDividers[1];   // if invalid divider, divide by 1.
}

//-----------------------------------------------------------------------------
//  CdceConversion::FromDividerInput() -- Returns divide_by for input value
//-----------------------------------------------------------------------------
//  this maps divider bits in the field value in the chip hardware to divide_by result

unsigned int  CdceConversion::FromDividerInput(unsigned int divider_bits)
{
	InverseDividerMap::iterator iter = CdceInverseDividers.find(divider_bits);

	if (iter != CdceInverseDividers.end())
        return CdceInverseDividers[divider_bits];
    else
        return 1;   // if invalid divider_bits, claim it is divide by 1
}

//-----------------------------------------------------------------------------
//  CdceConversion::ToPhaseAdjustInput() -- Returns input value for phase_adjust
//-----------------------------------------------------------------------------
//  this maps phase adjust phases to the field value in the chip hardware
//    values depend on current divider value

unsigned int  CdceConversion::ToPhaseAdjustInput(unsigned int divide_by, unsigned int phase)
{
    if (phase >= divide_by)
        return 0x00;     // invalid phase
	if (IsValidDivider(divide_by)==false)
        return 0x00;     // invalid divide-by
    else
        {
        //  find the vector
        PhaseAdjustArrayMap::iterator iter = CdcePhaseArray.find(divide_by);
        //  return the phase value
	    return (iter->second)[phase];
        }

}

//-----------------------------------------------------------------------------
//  CdceConversion::GetNearestDividers()  -- find divider set around a candidate
//-----------------------------------------------------------------------------

void   CdceConversion::CdceGetNearestDividers(float candidate, int & below, int & above)
{
    if (IsAboutEqual(candidate, NearestIntegerPart(candidate)))
        {
        //  We are essentially an integer, so treat as such.
        unsigned int int_candidate(static_cast<unsigned int>(NearestInteger(candidate)));
        if (IsValidDivider(int_candidate))
            {
            above = below = int_candidate;
            return;
            }
        }
    above = BestDividerGreaterThan(candidate);
    below = BestDividerLessThan(candidate);

}



} // namespace Innovative
