//
// CDCE_Conversion_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef CDCE_Conversion_MbH
#define CDCE_Conversion_MbH

#include <vector>
#include <map>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS CdceConversion --  Common Conversions for CDCE dividers
//=============================================================================
//  The TI CDCE parts have 'strange' mappings of divider numbers and
//     phase delays. These are implemented here for common use in part
//     implementation objects
//

class CdceConversion
{
public:
    CdceConversion();
    virtual ~CdceConversion()  {  }

    unsigned int  ToDivider(unsigned int divider_idx);
    unsigned int  ToDividerIdx(unsigned int divider);
    bool          IsValidDivider(unsigned int divide_by);

    unsigned int  BestDividerGreaterThan(float min_divide_by);
    unsigned int  BestDividerLessThan(float max_divide_by);

    unsigned int  ToDividerInput(unsigned int divide_by);
    unsigned int  FromDividerInput(unsigned int divider_bits);

    unsigned int  ToPhaseAdjustInput(unsigned int divide_by, unsigned int phase);

protected:
    typedef std::vector<unsigned int> ValidDividerArray;
    typedef std::map<unsigned int, unsigned int> ValidDividerMap;
    typedef std::map<unsigned int, unsigned int> InverseDividerMap;
    typedef std::map<unsigned int, std::vector<unsigned int> > PhaseAdjustArrayMap;

    ValidDividerMap      CdceValidDividers;
    InverseDividerMap    CdceInverseDividers;
    PhaseAdjustArrayMap  CdcePhaseArray;

protected:
    ValidDividerArray    CdceDividerValues;
    ValidDividerArray    CdceDividerFactors;

    void  CdceGetNearestDividers(float candidate, int & below, int & above);

};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative
#endif

