// 
// CalMemBase_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef CalMemBase_MbH
#define CalMemBase_MbH

#include <PmcEeprom_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS IUsesCalibration  --  Calibration memory access class for analog I/O
//=============================================================================

class IUsesCalibration : public IUsesPmcEeprom
{
public:
	// Ctor
    IUsesCalibration(AddressingSpace & space, unsigned int addr, int size,
		             int channels, int one);

    // Properties
	IUsesCalibration &	Calibrated(bool state);
	bool				Calibrated() const;

    float 				Offset(int ch) const;
    IUsesCalibration &	Offset(int ch, float value);

	float 				Gain(int ch) const;
    IUsesCalibration &	Gain(int ch, float value);

    //  Loading functions
	virtual void		ApplyRomCoefficients();
    virtual void        ApplyDefaultCoefficients();

protected:
	// Data
	int					Channels;
    int                 FOne;

	// Methods
	unsigned int 		ToFixed(float value);
	float 				FromFixed(unsigned int value);

	int					One() const;


	// Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(int ch, int value) = 0;
    virtual void 		SetCalibrationGain(int ch, int value) = 0;
};

//=============================================================================
//  CLASS IUsesSectionCalibration  --  Calib memory class for analog I/O
//=============================================================================
//  Uses a section to access the IDROM for calibration (allowing sharing)

class IUsesSectionCalibration
{
public:
	// Ctor
    IUsesSectionCalibration(PmcIdromSection section, int channels, int one);
    virtual ~IUsesSectionCalibration()
        {  }

    // Properties
	IUsesSectionCalibration &	Calibrated(bool state);
	bool				        Calibrated() const;

    float 				        Offset(int ch) const;
    IUsesSectionCalibration &	Offset(int ch, float value);

	float 				        Gain(int ch) const;
    IUsesSectionCalibration &	Gain(int ch, float value);

    //  Loading functions
	void		ApplyRomCoefficients();
    void        ApplyDefaultCoefficients();

protected:
	// Data
    PmcIdromSection     Section;
	int					Channels;
    int                 FOne;

	// Methods
	unsigned int 		ToFixed(float value);
	float 				FromFixed(unsigned int value);

	int					One() const;

	// Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(int ch, int value) = 0;
    virtual void 		SetCalibrationGain(int ch, int value) = 0;
};

//=============================================================================
//  CLASS IUsesRangedSectionCalibration  --  Calib memory class for Multiple Gain Ranges
//=============================================================================
//  Uses a section to access the IDROM for calibration (allowing sharing)

typedef  std::vector<PmcIdromSection>   PmcIdromSectionArray;

class IUsesRangedSectionCalibration
{
public:
	// Ctor
    IUsesRangedSectionCalibration(int channels, int one);
    virtual ~IUsesRangedSectionCalibration()
        {  }

    void                            AppendRange(const PmcIdromSection & section);
    void                            ClearRanges();
    size_t                          Ranges() const;

    // Properties
	IUsesRangedSectionCalibration &	Calibrated(size_t gain_range, bool state);
	bool				            Calibrated(size_t gain_range) const;

    float 				            Offset(size_t gain_range, int ch) const;
    IUsesRangedSectionCalibration &	Offset(size_t gain_range, int ch, float value);

	float 				            Gain(size_t gain_range, int ch) const;
    IUsesRangedSectionCalibration &	Gain(size_t gain_range, int ch, float value);

    //  Loading functions
	void		                    ApplyRomCoefficients(size_t gain_range);
    void                            ApplyDefaultCoefficients(size_t gain_range);

protected:
	// Data
    PmcIdromSectionArray     SectionArray;
	int					     Channels;
    int                      FOne;

	// Methods
	unsigned int 		ToFixed(float value);
	float 				FromFixed(unsigned int value);

	int					One() const;

	// Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(size_t gain_range, int ch, int value) = 0;
    virtual void 		SetCalibrationGain(size_t gain_range, int ch, int value) = 0;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative

#endif
