
//===========================================================================
//  CLASS CardSharpGpsPlugin  -- Hardware Plugin impl for Jackson Labs CSAC
//===========================================================================

//---------------------------------------------------------------------------
// CardSharpCarrierDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATioN CORPORATioN PROPRIETARY INFORMATioN
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative IntegraTion CorporaTion and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative IntegraTion CorporaTion.
//  All Rights Reserved.
//

#include <string.h>
#include <fcntl.h>
#include <CardSharpMemoryMap_Mb.h>
#include <CardSharpCarrierDeviceMap_Mb.h>
#include <CardSharpDeviceDriver_Mb.h>
#include <CardSharp_Mb.h>
#include <K7Support_Mb.h>
#include <SystemSupport_Mb.h>
#include <Gps_Csac_Mb.h>
#include <Gps_Ieee1588_Mb.h>

#include <Poco/Foundation.h>
#include <Poco/Thread.h>
#include <Poco/Activity.h>
#include <Poco/Timespan.h>
#include <Poco/Timestamp.h>
#include <Poco/Mutex.h>
#include <Poco/Event.h>
#include <sys/ioctl.h>


namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS CardSharpGpsPlugin  -- Hardware Plugin impl
//===========================================================================

class CardSharpGpsPlugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() { Owner->ArmGpsTrigger();  };

    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)  // antenna not implemented in hardware
                          {  return gps_value;  }
    virtual bool    PGPS_IsLocked(bool /*gps_value*/)
                          {  return Owner->IsLocked();  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return Owner->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  Owner->UartData(c); }
    virtual char             PGPS_UartData()
                                {  return Owner->UartData();  }
    virtual void             PGPS_UartBaud(IGps::BaudRates baud)
                                {     }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return IGps::b115200; /*convert(Owner->GetBaud());*/  }

    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/)  {}
    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {  Owner->GpsEnabled(state);  }
    virtual bool          PGPS_HdwEnabled()
                                {  return Owner->GpsEnabled();  }

    CardSharpGpsPlugin(CardSharpGpsUart * owner);
    virtual ~CardSharpGpsPlugin()
        {}

private:
    CardSharpGpsUart * Owner;

};

//---------------------------------------------------------------------------
//  constructor for class CardSharpGpsPlugin
//---------------------------------------------------------------------------

CardSharpGpsPlugin::CardSharpGpsPlugin(CardSharpGpsUart * owner)
    : Owner(owner)
{
}

//---------------------------------------------------------------------------
//   CardSharpGpsPlugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void CardSharpGpsPlugin::PGPS_OnInit()
{
     Owner->Uart(0).PutString("SYNC:OUT:1PPS:RESET ON\r\n");    // PPS from power up
     Owner->Uart(0).PutString("GPS:GPZDA 1\r\n");              // emit GPZDA once per second
     Owner->Uart(0).PutString("GPS:GPGGA 1\r\n");              // emit GPGGA once per second
}


//===========================================================================
//  CLASS CardSharpIeee1588Plugin  -- Hardware Plugin impl for IEEE1588 intf
//===========================================================================

class CardSharpIeee1588Plugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() {  GpsUart->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)  // antenna not implemented in hardware
                          {  return gps_value;  }
    virtual bool    PGPS_IsLocked(bool gps_value)
                          {  return gps_value;  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return GpsUart->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  GpsUart->Uart(0).Port(c); }  // or leave blank?
    virtual char             PGPS_UartData()
                                {  return GpsUart->Uart(0).Port();  }

    virtual void             PGPS_UartBaud(IGps::BaudRates /*baud*/)   // don't support changing baud rates?
                                {  /*IeeeDev->Uart(0).SetBaud(baud);*/  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return IGps::b115200;  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {
                                GpsUart->GpsEnabled(state);
                                }
    virtual bool          PGPS_HdwEnabled()
                                {  return GpsUart->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/)  {}

    CardSharpIeee1588Plugin(CardSharpGpsUart * gps_uart, BabushkaIeee1588 * ieee_dev);
    virtual ~CardSharpIeee1588Plugin()
        {}

private:
    CardSharpGpsUart * GpsUart;
    BabushkaIeee1588 * IeeeDev;
};

//---------------------------------------------------------------------------
//  constructor for class CardSharpIeee1588Plugin
//---------------------------------------------------------------------------

CardSharpIeee1588Plugin::CardSharpIeee1588Plugin(CardSharpGpsUart * gps_uart, BabushkaIeee1588 * ieee_dev)
    : GpsUart(gps_uart), IeeeDev(ieee_dev)
{
}

//---------------------------------------------------------------------------
//   CardSharpIeee1588Plugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void CardSharpIeee1588Plugin::PGPS_OnInit()
{
    GpsUart->Ieee1588_MezzanineReset(false);
    Sleep(1);
    GpsUart->Ieee1588_MezzanineReset(true);
    Sleep(1);
    GpsUart->Ieee1588_MezzanineReset(false);
    Sleep(1);

    // we can't config thru serial port

    // but can we do the IEEE init here?
    IeeeDev->InitIeee1588();

}

//===========================================================================
//  CLASS CardSharpGPS_Callback -- X6 GPS Hardware Callback Interface
//===========================================================================

class CardSharpGPS_Callback : public IGPS_Callback
{
public:
	CardSharpGPS_Callback(CardSharpDeviceMap * owner)
        :  Owner(owner)
        {}
    virtual ~CardSharpGPS_Callback()
        {}

    virtual void InterruptEnable(bool state)
                    {  Owner->InterruptEnable(state);  }

private:
    CardSharpDeviceMap *  Owner;
};

//===========================================================================
//  CLASS CardSharpInfo -- Information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class CardSharpInfo
//---------------------------------------------------------------------------

CardSharpInfo::CardSharpInfo(IAddressingSpace & space,  MapRegisters & regs)
    :  FpgaLogicVersionReg(space, regs.Info_Addr),
       FpgaLogicSubRevReg(space, regs.Subrev_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class CardSharpInfo
//---------------------------------------------------------------------------

CardSharpInfo::~CardSharpInfo()
{
}

//---------------------------------------------------------------------------
//  CardSharpInfo::OpenHardware()
//---------------------------------------------------------------------------

void  CardSharpInfo::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  CardSharpInfo::ConfigureHardware()
//---------------------------------------------------------------------------

void  CardSharpInfo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  CardSharpInfo::PciLogicRevision() -- PCI Rev Code
//---------------------------------------------------------------------------

short CardSharpInfo::PciLogicRevision() const
{
    return 0;
}

//---------------------------------------------------------------------------
//  CardSharpInfo::PciLogicFamily() -- PCI Family Code
//---------------------------------------------------------------------------

short CardSharpInfo::PciLogicFamily() const
{
    return 0;
}

//---------------------------------------------------------------------------
//  CardSharpInfo::FpgaLogicVersion() -- FPGA Version Code
//---------------------------------------------------------------------------

short CardSharpInfo::FpgaLogicVersion() const
{
    return FpgaLogicVersionReg.Version.Value();
}

//---------------------------------------------------------------------------
//  CardSharpInfo::FpgaHardwareVariant() -- FPGA Hardware Variant Code
//---------------------------------------------------------------------------

short CardSharpInfo::FpgaHardwareVariant() const
{
    return FpgaLogicVersionReg.HdwVariant.Value();
}

//---------------------------------------------------------------------------
//  CardSharpInfo::PciLogicPcb() -- Hardware Board Rev Code
//---------------------------------------------------------------------------

short CardSharpInfo::PciLogicPcb() const
{
    return 0;
}

//---------------------------------------------------------------------------
//  CardSharpInfo::PciLogicType() -- Hardware Board Type Code
//---------------------------------------------------------------------------

short CardSharpInfo::PciLogicType() const
{
    return FpgaLogicVersionReg.BoardHwType.Value();
}

//---------------------------------------------------------------------------
//  CardSharpInfo::FpgaChipType() -- FPGA Chip Type Code
//---------------------------------------------------------------------------

short CardSharpInfo::FpgaChipType() const
{
    return FpgaLogicVersionReg.FpgaType.Value();
}

//---------------------------------------------------------------------------
//  CardSharpInfo::BoardRevisionBuiltFor() -- Which revision is this logic for?
//---------------------------------------------------------------------------
//  0xF = all versions...

short CardSharpInfo::BoardRevisionBuiltFor() const
{
    return 0;
}

//---------------------------------------------------------------------------
//  CardSharpInfo::FpgaLogicSubrevision() -- FPGA Logic Subrev Code
//---------------------------------------------------------------------------

short CardSharpInfo::FpgaLogicSubrevision() const
{
    return 0;
}

//---------------------------------------------------------------------------
//  CardSharpInfo::PciLogicRawValue() -- read entire register
//---------------------------------------------------------------------------

unsigned int  CardSharpInfo::PciLogicRawValue() const
{
    return 0;
}

//---------------------------------------------------------------------------
//  CardSharpInfo::FpgaInformationRawValue() -- read entire register
//---------------------------------------------------------------------------

unsigned int  CardSharpInfo::FpgaInformationRawValue() const
{
    return FpgaLogicVersionReg.Value();
}

//---------------------------------------------------------------------------
//  CardSharpInfo::FpgaSubRevRawValue() -- read entire register
//---------------------------------------------------------------------------

unsigned int  CardSharpInfo::FpgaSubRevRawValue() const
{
    return FpgaLogicSubRevReg.Value();
}

//===========================================================================
//  CLASS CardSharpPeripherals --  Peripheral "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class CardSharpPeripherals
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class CardSharpPeripherals
//---------------------------------------------------------------------------

CardSharpPeripherals::CardSharpPeripherals(IAddressingSpace & space, MapRegisters & regs)
    :  ResetNLedReg(space, regs.ResetReg_Addr), MemTestReg(space, regs.MemTestReg_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class CardSharpPeripherals
//---------------------------------------------------------------------------

CardSharpPeripherals::~CardSharpPeripherals()
{
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::ConstructCustomParts()
//---------------------------------------------------------------------------

void  CardSharpPeripherals::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::OpenHardware()
//---------------------------------------------------------------------------

void  CardSharpPeripherals::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::ConfigureHardware()
//---------------------------------------------------------------------------

void  CardSharpPeripherals::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::GreenLed() -- Read the Green Led state
//---------------------------------------------------------------------------

bool  CardSharpPeripherals::GreenLed()
{
    return ResetNLedReg.GreenLed.Value();
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::GreenLed() -- Set the Green Led state
//---------------------------------------------------------------------------

void  CardSharpPeripherals::GreenLed( bool value )
{
    ResetNLedReg.GreenLed.Value(value);
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::FrontPanelLed() -- Read the FrontPanel Led state
//---------------------------------------------------------------------------

bool  CardSharpPeripherals::FrontPanelLed()
{
    return ResetNLedReg.FrontPanelLed.Value();
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::FrontPanelLed() -- Set the FrontPanel Led state
//---------------------------------------------------------------------------

void  CardSharpPeripherals::FrontPanelLed( bool value )
{
    ResetNLedReg.FrontPanelLed.Value(value);
}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::AppReset() -- Apply "Application" Reset
//---------------------------------------------------------------------------

void  CardSharpPeripherals::AppReset()
{
    Debug.Log("App Reset Applied");
    //
    //  Reset User Logic
    ResetNLedReg.FpgaReset.Set();
    Sleep(1);
    ResetNLedReg.FpgaReset.Clear();
    Sleep(1);

}

//---------------------------------------------------------------------------
//  CardSharpPeripherals::FmcCRun() --  (FMC only)
//---------------------------------------------------------------------------

void  CardSharpPeripherals::FmcCRun(bool state)
{
    if (state)
        Debug.Log("FMC Carrier Run ON");
    else
        Debug.Log("FMC Carrier Run OFF");

    ResetNLedReg.FmcCarrierRun.Value(state);
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::StartPlMemTest() --
//---------------------------------------------------------------------------
void  CardSharpPeripherals::StartPlMemTest()
    {
    MemTestReg.MemTestEnable.Value(1);
    }

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::CheckPlMemTest() -- 5 minutes after calling StartPlMemTest, true if test passed
//---------------------------------------------------------------------------

bool  CardSharpPeripherals::CheckPlMemTest()
    {
    bool  TestPassed = !MemTestReg.MemTestError.Value();

    MemTestReg.MemTestEnable.Value(0);

    return TestPassed;
    }

using namespace CardsharpMemoryMap;
namespace  // nameless
{

const size_t  FMC_Slot_Count = 1;

CardSharpInfo::MapRegisters CardSharpInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmSubrevision         //  Subrev_Addr
    };


CardSharpPeripherals::MapRegisters CardSharpPeripRegAddrs =
    {
    mmReset              //  Reset_Addr
    };
}


//CardSharpDeviceMap * aCardSharp()
//{
//    CardSharpDeviceMap * wombat = new CardSharpDeviceMap;
//    return wombat;
//}

//===========================================================================
//  CLASS IIsAnCardSharpDeviceMap  --Fmc daughter card linkage interface class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for IIsAnCardSharpDeviceMap
//---------------------------------------------------------------------------

IIsAnCardSharpDeviceMap::IIsAnCardSharpDeviceMap(size_t total_slots)
{
    if (total_slots--)
        FmcDaughters.push_back(&FmcSlot_Prime);
    if (total_slots--)
        FmcDaughters.push_back(&FmcSlot_RTM);
}

//---------------------------------------------------------------------------
//  IIsAnCardSharpDeviceMap::LinkFmcDaughterTo() -- connect daughter to map
//---------------------------------------------------------------------------

bool  IIsAnCardSharpDeviceMap::LinkFmcDaughterTo(IFmcDaughterCard * daughter, size_t slot)
{
    if (slot>=FmcDaughters.size())
        return false;

    if (daughter)
        {
        FmcDaughters[slot]->Link(daughter);
        return true;
        }
    else
        return false;
}

//---------------------------------------------------------------------------
//  IIsAnCardSharpDeviceMap::UnlinkFmcDaughterFrom() -- unlink daughter
//---------------------------------------------------------------------------

void IIsAnCardSharpDeviceMap::UnlinkFmcDaughterFrom(size_t slot)
{
    if (slot>=FmcDaughters.size())
        return;
    FmcDaughters[slot]->Unlink();
}

namespace  // nameless
{
BabushkaIeee1588::MapRegisters SbcDuoBabushkaIeee1588Addrs =
    {
    wbBabushkaBase,           //   UartWishbone_Base;
    mmBab1588_Ctrl,           //   Ctrl_Addr
    mmBab1588_ClkId_Lo,       //   ClkId_Lo_Addr;
    mmBab1588_ClkId_Hi,       //   ClkId_Hi_Addr;
    mmBab1588_Time,           //   Time_Addr;
    mmBab1588_EEPromI2C,      //   EEPromI2C_Addr
    };
}  // end nameless namespace


//==============================================================================
//  CLASS PpsDeviceHandlerImpl --
//==============================================================================

//===========================================================================
//  CLASS PpsDeviceHandlerImpl  -- Collects interrupt status words.
//===========================================================================
class CardSharpDeviceMap;
class PpsDeviceHandlerImpl
{
public:
	PpsDeviceHandlerImpl(CardSharpDeviceMap *ow);
	~PpsDeviceHandlerImpl();

	bool Enable();
	bool Disable();

protected:
	void runActivity();

private:
	CardSharpDeviceMap 						   *owner;
	Poco::Activity<PpsDeviceHandlerImpl>     	activity;
	int 				 				     	fd;
};

//------------------------------------------------------------------------------
//  PpsDeviceHandlerImpl::PpsDeviceHandlerImpl() --
//------------------------------------------------------------------------------

PpsDeviceHandlerImpl::PpsDeviceHandlerImpl(CardSharpDeviceMap *ow) : owner(ow),
		activity(this, &PpsDeviceHandlerImpl::runActivity), fd(-1)
{
}

//------------------------------------------------------------------------------
//  PpsDeviceHandlerImpl::~PpsDeviceHandlerImpl() --
//------------------------------------------------------------------------------

PpsDeviceHandlerImpl::~PpsDeviceHandlerImpl()
{
	if (!activity.isStopped())
	{
		activity.stop();
		activity.wait();

		if (fd != -1) close(fd);
	}
}


//------------------------------------------------------------------------------
//  PpsDeviceHandlerImpl::Enable() --
//------------------------------------------------------------------------------

bool PpsDeviceHandlerImpl::Enable()
{
	// Open the device only once
	if (fd != -1)
		{
		fd = open("/dev/ii_gps_pps", O_RDONLY);

		if (fd < 0)
			throw std::runtime_error("Could not connect PPS signal");
		}

	activity.start();
	return true;
}

//------------------------------------------------------------------------------
//  PpsDeviceHandlerImpl::Disable() --
//------------------------------------------------------------------------------

bool PpsDeviceHandlerImpl::Disable()
{
	activity.stop();
	activity.wait();
	return true;
}

//------------------------------------------------------------------------------
//  PpsDeviceHandlerImpl::runActivity() --
//------------------------------------------------------------------------------

void PpsDeviceHandlerImpl::runActivity()
{
	while (!activity.isStopped())
	{
		ioctl(fd, 0);

	    owner->InterruptDispatch(0);
	}
}

//---------------------------------------------------------------------------
//  constructor for class CardSharpDeviceMap
//---------------------------------------------------------------------------

CardSharpDeviceMap::CardSharpDeviceMap(CardSharp *owner)
	: MultiChanDeviceMap(owner, new CardSharpDeviceDriver),
	  IIsAnCardSharpDeviceMap(FMC_Slot_Count),
      WB_Periph             (LogicMemory, wbPeripheral),
      Info_SDev             (WB_Periph, CardSharpInfoRegAddrs),
      Periph_SDev           (WB_Periph, CardSharpPeripRegAddrs),
      Gps_SDev              (this, new CardSharpGPS_Callback(this)),
      SerialReg             (GpsUart(0), GpsUart(1)),
      BabIeee1588           (SerialReg,  SbcDuoBabushkaIeee1588Addrs),
      PpsCtrl				(new PpsDeviceHandlerImpl(this))
{
	AtConstructCustomPartTime();

}

//---------------------------------------------------------------------------
//  destructor for class CardSharpDeviceMap
//---------------------------------------------------------------------------

CardSharpDeviceMap::~CardSharpDeviceMap()
{

}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::DumpIsrStatus(IntVec & vec)
{
    vec.clear();
    for (int i = 0; i < CardSharpDeviceDriver::LOG->Count; i++)
        vec.push_back(CardSharpDeviceDriver::LOG->Status[i]);
}


//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  CardSharpDeviceMap::Name() const
{
    switch (Info().PciLogicType())
        {
        case 4: return "CardSharp";
        default: return "Unknown";
        }
    return "Unknown";
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::FpgaName() --  Return name string for FPGA type
//---------------------------------------------------------------------------

std::string  CardSharpDeviceMap::FpgaName(short Number) const
{
     switch (Number)
        {
        case 0x00: return "Zynq";
        default: return "Unknown";
        }
    return "Unknown";
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  CardSharpDeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A'  + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Open()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::Open()
{
    InterruptEnable(false);

	MultiChanDeviceMap::Open();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Close()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::Close()
{
    InterruptEnable(false);

	MultiChanDeviceMap::Close();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::PreOpenHardware()
//---------------------------------------------------------------------------

void CardSharpDeviceMap::PreOpenHardware()
{
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::OnSingleStreamAtStartConfig()
//---------------------------------------------------------------------------

void CardSharpDeviceMap::OnSingleStreamAtStartConfig()
{
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::OnBeforeStreamStart()
//---------------------------------------------------------------------------

void CardSharpDeviceMap::OnBeforeStreamStart()
{
	//
	//  Start execution
	Periph_SDev.FmcCRun(true);
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::OnAfterStreamStart()
//---------------------------------------------------------------------------

void CardSharpDeviceMap::OnAfterStreamStart()
{
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::OnBeforeStreamStop()
//---------------------------------------------------------------------------

void CardSharpDeviceMap::OnBeforeStreamStop()
{

}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Reset()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::Reset()
{
    //
    //  Clear Int Status queue
    //DeviceDriver->ClearInterruptStatusQueue();
    // DebugLog.clear();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Preconfig() -- Non-repeated 'Manual' Configuration
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::Preconfig()
{
	MultiChanDeviceMap::AtPreconfigureTime();
    // "activate" one time at-start preconfiguration
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Start() - turn on clocks & triggers
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::Start()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::Start()");

	MultiChanDeviceMap::AtStreamStartTime();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::Stop()  turn off clocks & triggers
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::Stop()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::Stop()");

	MultiChanDeviceMap::AtStreamStopTime();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 CardSharpDeviceMap::PeekPort(int offset)
{
    return MultiChanDeviceMap::PeekPort(offset);
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void CardSharpDeviceMap::PokePort(int offset, ii32 value)
{
    return MultiChanDeviceMap::PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32 CardSharpDeviceMap::PeekLogic(int offset)
{
    return MultiChanDeviceMap::PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void CardSharpDeviceMap::PokeLogic(int offset, ii32 value)
{
    return MultiChanDeviceMap::PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::ConstructCustomParts()
{
	MultiChanDeviceMap::ConstructCustomParts();

    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    Gps_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::OpenHardware()");
    //
    //  Reset the device
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    Gps_SDev.OpenHardware();

    //  Call daughter card
    //  ...make memory spaces on daughter valid
    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;

        FmcDaughter(i)->PortMemory.Link( PortMemory );
        FmcDaughter(i)->LogicMemory.Link( LogicMemory );

        FmcDaughter(i)->InitCardPower();

        FmcDaughter(i)->OpenHardware();
        }
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::CloseHardware()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::CloseHardware()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::CloseHardware()");

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->CloseHardware();
        FmcDaughter(i)->RemoveCardPower();
        }

}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::StopHardware()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::StopHardware()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::StopHardware()");

}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::StreamStopHardware()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::StreamStopHardware()");

	Periph_SDev.FmcCRun(false);

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->StreamStopHardware();
        }
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::PreconfigureHardware()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::PreconfigureHardware()");
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    Gps_SDev.PreconfigureHardware();

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->PreconfigureHardware();
        }

}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::ConfigureHardware()
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::ConfigureHardware()");
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    Gps_SDev.ConfigureHardware();

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->ConfigureHardware();
        }
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::AfterFiclStart() --
//---------------------------------------------------------------------------

void  CardSharpDeviceMap::AfterFiclStart(FiclSystem & engine)
{
    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->AfterFiclStart(engine);
        }
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void CardSharpDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::InterruptDispatch()");
    // dispatch UART interrupt
    Gps_SDev.Gps().DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void CardSharpDeviceMap::InterruptEnable(bool state)
{
    ScopedTraceRef ccc(Debug, "CardSharpDeviceMap::InterruptEnable()");
    Debug.Log("state=", (int)state);

    if (state)
    	PpsCtrl->Enable();
    else
    	PpsCtrl->Disable();

}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::GetGpsCount() --
//---------------------------------------------------------------------------
//  Count of GPS devices to be created and installed

size_t  CardSharpDeviceMap::GetGpsCount()
{
    return 2;
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::MakeOneGpsDevice() -- Build custom Gps
//---------------------------------------------------------------------------
//  Will be called once per GPS device set up.  Use IDX to distinguish them

IGps *  CardSharpDeviceMap::MakeOneGpsDevice(size_t idx)
{
   //  Make our local Gps objects (subdev will delete it)
   if (idx==0)
       return new Gps_CSACPlugin(new CardSharpGpsPlugin(&Gps_SDev));
   else
       return new Gps_Ieee1588Plugin(new CardSharpIeee1588Plugin(&Gps_SDev, &Babushka()) );
}

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::StartPlMemTest() --
//---------------------------------------------------------------------------
void  CardSharpDeviceMap::StartPlMemTest()
    {
    Periph_SDev.StartPlMemTest();
    }

//---------------------------------------------------------------------------
//  CardSharpDeviceMap::CheckPlMemTest() -- 5 minutes after calling StartPlMemTest, true if test passed
//---------------------------------------------------------------------------

bool  CardSharpDeviceMap::CheckPlMemTest()
    {
    return Periph_SDev.CheckPlMemTest();
    }

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
