#ifndef CARDSHARPCARRIER_DEVICEMAP_MbH
#define CARDSHARPCARRIER_DEVICEMAP_MbH

#include <DeviceDriverCardSharp_Mb.h>
#include <K7Support_Mb.h>
#include <FmcSupport_Mb.h>
#include <X6Support_Mb.h>
#include <SafeLink_Mb.h>
#include <HardwareRegister_Mb.h>
#include <StreamDeviceEvents_Mb.h>
#include <MultiChanDeviceMap_Mb.h>
#include <MultiChanStreamSupport_Mb.h>
#include <DeviceDriver_Mb.h>
#include <CardSharpSubDevice_Misc_Mb.h>

#include <SbcDuoDeviceMap_Mb.h>
#include <memory>

namespace Innovative {

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS CardSharpSysLedReset -- LED/Reset register "SubDevice"
//===========================================================================

//===========================================================================
//  CLASS CardSharpPeripherals -- X6 Peripheral section "SubDevice"
//===========================================================================

class CardSharpPeripherals: public IX6SubDevice {
public:
	//=======================================================================
	//  CLASS CardSharpPeripherals::MapRegisters  -- Device Register Offsets
	//=======================================================================
	class MapRegisters {
	public:
		unsigned int ResetReg_Addr;
		unsigned int MemTestReg_Addr;
	};

	CardSharpPeripherals(IAddressingSpace & space, MapRegisters & regs);
	virtual ~CardSharpPeripherals();

	//  Methods
	bool GreenLed();
	void GreenLed(bool value);

	bool FrontPanelLed();
	void FrontPanelLed(bool value);

	void AppReset();
	void FmcCRun(bool state);

	void  StartPlMemTest();
	// Wait 5 minutes before ...
	bool  CheckPlMemTest();

	//  Internal Interface
	virtual void ConstructCustomParts();
	virtual void OpenHardware();
	virtual void ConfigureHardware();

	bool MemTest();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS CardSharpPeripherals::ResetNLedRegister  --  Module User Logic ID info reg
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class ResetNLedRegister: public Register {
		typedef Register inherited;

	public:
		RegisterBit FpgaReset;
		RegisterBit GreenLed;
		RegisterBit FrontPanelLed;
		RegisterBit FmcCarrierRun;

		ResetNLedRegister(IAddressingSpace & space, unsigned int offset) :
				inherited(space, offset), FpgaReset(*this, 0), GreenLed(*this,
						1), FrontPanelLed(*this, 2), FmcCarrierRun(*this, 4) {
		}
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS CardSharpPeripherals::MemTestRegister  --
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class MemTestRegister: public Register {
		typedef Register inherited;

	public:
		ReadOnlyRegisterBitGroup 	InitDone;
		RegisterBitGroup 		MemTestEnable;
		ReadOnlyRegisterBitGroup 	MemTestError;


		MemTestRegister(IAddressingSpace & space, unsigned int offset) :
				inherited(space, offset), InitDone(*this, 4, 2),
				MemTestEnable(*this, 12, 2), MemTestError(*this, 16, 2) {
		}
	};


private:

	ResetNLedRegister ResetNLedReg;
	MemTestRegister   MemTestReg;

};

//===========================================================================
//  CLASS CardSharpInfo -- Information "SubDevice"
//===========================================================================

class CardSharpInfo: public IX6SubDevice, public IX6LogicVersionInfo {
public:
	//=======================================================================
	//  CLASS CardSharpInfo::MapRegisters  -- Device Register Offsets
	//=======================================================================
	class MapRegisters {
	public:
		unsigned int Info_Addr;
		unsigned int Subrev_Addr;
	};

    CardSharpInfo(IAddressingSpace & space, MapRegisters & regs);
	virtual ~CardSharpInfo();

	virtual short PciLogicRevision() const;
	virtual short PciLogicFamily() const;    // Board Family (X6)

	virtual short FpgaLogicVersion() const;
	virtual short PciLogicPcb() const;       // Board Revision
	virtual short PciLogicType() const;      // Board Type
	virtual short FpgaHardwareVariant() const;
	virtual short FpgaChipType() const;

	virtual short FpgaLogicSubrevision() const;
	virtual short BoardRevisionBuiltFor() const;

	virtual unsigned int PciLogicRawValue() const;
	virtual unsigned int FpgaInformationRawValue() const;
	virtual unsigned int FpgaSubRevRawValue() const;

	//  Internal Interface
	virtual void OpenHardware();
	virtual void ConfigureHardware();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS FpgaLogicVersionRegister  --   Module User Logic ID info reg
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class FpgaLogicVersionRegister: public ReadOnlyRegister {
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBitGroup Version;
		ReadOnlyRegisterBitGroup HdwVariant;
		ReadOnlyRegisterBitGroup BoardHwType;
		ReadOnlyRegisterBitGroup FpgaType;

		FpgaLogicVersionRegister(IAddressingSpace & space, unsigned int offset) :
				inherited(space, offset), Version(*this, 0, 16), HdwVariant(
						*this, 16, 8), BoardHwType(*this, 24, 4), FpgaType(
						*this, 28, 2) {
		}
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS FpgaLogicSubRevRegister  --   Module User Logic ID info reg
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class FpgaLogicSubRevRegister: public ReadOnlyRegister {
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBitGroup Rev;

		FpgaLogicSubRevRegister(IAddressingSpace & space, unsigned int offset) :
				inherited(space, offset), Rev(*this, 24, 8) {
		}
	};


private:
	FpgaLogicVersionRegister FpgaLogicVersionReg;
	FpgaLogicSubRevRegister FpgaLogicSubRevReg;

};

//===========================================================================
//  CLASS IIsAnCardSharpDeviceMap  --Fmc daughter card linkage interface class
//===========================================================================

class IIsAnCardSharpDeviceMap {

public:
	IIsAnCardSharpDeviceMap(size_t total_slots);
	virtual ~IIsAnCardSharpDeviceMap() {
	}

	virtual bool LinkFmcDaughterTo(IFmcDaughterCard * daughter, size_t slot);
	virtual void UnlinkFmcDaughterFrom(size_t slot);

protected:
	typedef SafeLink<IFmcDaughterCard, SafeFmcDaughterCardImpl> DevMapSafeLink;

	DevMapSafeLink FmcSlot_Prime;
	DevMapSafeLink FmcSlot_RTM;

	DevMapSafeLink & FmcDaughter(size_t slot) {
		return *(FmcDaughters[slot]);
	}
	size_t FmcSlots() {
		return FmcDaughters.size();
	}

	std::vector<DevMapSafeLink *> FmcDaughters;

};
#ifdef _MSC_VER
#pragma warning(pop)
#endif

//===========================================================================
//  CLASS CardSharpDeviceMap  -- Register & low level support for VpxBoard
//===========================================================================
class CardSharp;

class CardSharpDeviceMap: public IRequires_MultiGps,
						  public IGPS_Callback,
		                  public MultiChanDeviceMap,
						  public IIsAnCardSharpDeviceMap
{

	friend class CardSharpGPS_Callback;
	friend class PpsDeviceHandlerImpl;
public:
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS CardSharpDeviceMap::MapWishbones  -- Wishbone Offsets
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class MapWishbones {
	public:
		unsigned int WbApp_Offset;
	};

	friend class CardSharp;
	friend class FmcCarrierBoard;
	friend class FmcCarrierInputDevice;
	friend class FmcCarrierOutputDevice;
	friend class FmcDaughterCardSafeLink;
	//
	//  Member Functions
	CardSharpDeviceMap(CardSharp *owner);
	virtual ~CardSharpDeviceMap();
	//
	//  Make this property public

	// Methods
	// Version
	std::string Name() const;
	std::string FpgaName(short Number) const;
	std::string Revision() const;

	virtual void Open();
	virtual void Close();
	virtual void Reset();
	virtual void Start();
	virtual void Stop();

	void  Preconfig();

    BabushkaIeee1588 &                    Babushka()        {  return BabIeee1588;  }
    IMulticharUart &                      GpsUart(int dev)  {  return Gps_SDev.Uart(dev);  }

    const IX6LogicVersionInfo & Info() const 				{  return Info_SDev;}

    virtual size_t             GetGpsCount();
    virtual IGps *             MakeOneGpsDevice(size_t idx);

    void  StartPlMemTest();
    bool  CheckPlMemTest();

public: // pseudo-protected


    virtual void PreOpenHardware();
    virtual void OnSingleStreamAtStartConfig();
    virtual void OnBeforeStreamStart();
    virtual void OnAfterStreamStart();
    virtual void OnBeforeStreamStop();

	virtual void ConstructCustomParts();// call in most derived ctor
	virtual void OpenHardware();
	virtual void PreconfigureHardware();
	virtual void ConfigureHardware();
	virtual void StreamStopHardware();
	virtual void StopHardware();
	virtual void CloseHardware();
	//
	//  Subdevice Customization - implement board specific pieces
	//
	//  Busmaster Intf functions

	typedef std::vector<int> IntVec;
	void DumpIsrStatus(IntVec & vec);

	//  Debug Support
	ii32 PeekPort(int offset);
	void PokePort(int offset, ii32 value);
	ii32 PeekLogic(int offset);
	void PokeLogic(int offset, ii32 value);

protected:
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);

	//
	//  Wishbone Bus Spaces
	WishboneBusSpace WB_Periph;


	//
	//  Subdevices
	CardSharpInfo         		 Info_SDev;
	CardSharpPeripherals  		 Periph_SDev;
	CardSharpGpsUart      		 Gps_SDev;
    SbcDuo_SerialRegister   	 SerialReg;
    BabushkaIeee1588        	 BabIeee1588;
    std::unique_ptr<class PpsDeviceHandlerImpl> PpsCtrl;
	Trace 						 Debug;

	// Data
	// ???  May not be implemented
	void LocalLoopbackMode(bool state);
	void FmcCarrierRun(bool state);

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void  AfterFiclStart(FiclSystem & engine);
};

#ifdef _MSC_VER
#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}
;
// Innovative

#endif // CARDSHARPCARRIER_DEVICEMAP_MbH
