#ifndef CARDSHARPMEMORYMAP_MbH
#define CARDSHARPMEMORYMAP_MbH

namespace CardsharpMemoryMap {
	//
	//  Logic Memory Register Offsets
	//
	//  Cardsharp Unit "Standard" WB section offsets
	const unsigned int wbPeripheral 	 = 0x0000;
	const unsigned int wbPacketizer		 = 0x0300;
	const unsigned int wbAlerts			 = 0x0400;
	const unsigned int wbDigIo			 = 0x0600;
	const unsigned int wbMatlab			 = 0x0700;
	const unsigned int wbFMCIF 		 	 = 0x0C00;
	const unsigned int wbApp	 		 = 0x1A00;

	//
	//  K7System Wishbone offsets
	//
	//  WB0 - Peripheral Registers
	const unsigned int mmInformation 	 = 0x00;
	const unsigned int mmReset 		 	 = 0x01;
	const unsigned int mmSubrevision 	 = 0x03;
	const unsigned int mmBypassVitaPad 	 = 0x04;
	const unsigned int mmPlDdr3		 	 = 0x07;
	const unsigned int mmPid 		 	 = 0x08;
	const unsigned int mmQSFP0Ctrl	 	 = 0x10;
	const unsigned int mmQSFP0I2c	 	 = 0x11;
	const unsigned int mmQSFP1Ctrl	 	 = 0x12;
	const unsigned int mmQSFP1I2c	 	 = 0x13;
	const unsigned int mmQSFPxI2c	 	 = 0x14;

	//
	//  WB1 - FMC2 Interface
	const unsigned int mmInterfaceStatus = 0x00;
	const unsigned int mmI2CInterface 	 = 0x01;
	const unsigned int mmFMCId 			 = 0x02;

	//
	//  WB3 - Packetizer
	const unsigned int mmPktDataChEn 	= 0x00;
	const unsigned int mmPktAuxHdr2     = 0x01;
	const unsigned int mmAlertPktHdr 	= 0x02;
	const unsigned int mmDataPktHdr 	= 0x03;
	const unsigned int mmForcePktSize 	= 0x22;
	const unsigned int mmPktTimer 		= 0x23;

	//
	//  WB4 - Alerts
	const unsigned int mmAlertEn		 = 0x00;
	const unsigned int mmAlertCtrl		 = 0x01;
	const unsigned int mmAlertSw		 = 0x02;

	//
	//  WB6 - P16 DIO
	const unsigned int mmDioOut			 = 0x00;
	const unsigned int mmDioEn			 = 0x02;

	//
	//  WB - Application
	const unsigned int mmAppRun			 = 0x00;

	//
	//  Babushka Register Map
	//
	//  ...the Duo has a serial link to an alternate logic that holds (for now)
	//      registers that form an additional serial link (like nesting Babushka dolls).
	//
	//  SbcDuo Unit "Standard" WB section offsets
	const unsigned int  wbBabushkaBase     = 0x900;

	//
	//  pseudo WB9 - Babushka 1588 and UART
	const unsigned int  mmBab1588_Ctrl     = 0x00;
	const unsigned int  mmBab1588_ClkId_Lo = 0x01;
	const unsigned int  mmBab1588_ClkId_Hi = 0x02;
	const unsigned int  mmBab1588_Time     = 0x03;
	const unsigned int  mmBab1588_EEPromI2C= 0x04;
}

#endif // CARDSHARPMEMORYMAP_MbH
