//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef CARDSHARPBOARD_MbH
#define CARDSHARPBOARD_MbH


#include <X6Support_Mb.h>
#include <CardSharpCarrierDeviceMap_Mb.h>
#include <MultiChanBoard_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS CardSharp  -- CardSharp baseboard class
//===========================================================================

class CardSharp : public MultiChanBoard
{

public:
    // Enumerations

    //
    //  Member Functions
    CardSharp();
    virtual ~CardSharp();

    //  Informational Methods
    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    int                         Target();
    void                        Target(int value);

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    virtual void                Start();
    virtual void                Stop();

    void 						Preconfig();
    //
    //  Properties

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    const IX6LogicVersionInfo &         Info() const;
    const IMulticharUart &         	    Gps() const;
    IX6_GpsIo &                         GpsCtrl();

    BabushkaIeee1588 &                  Babushka()        {  return Device.Babushka();  }
    void  SelectGps(size_t idx)
        {
    	Device.Gps_SDev.SelectGps(idx);
        }

    // Fmc connect interface
    bool ConnectTo(IFmcDaughterCard * daughter, unsigned int slot=0 );
    void DisconnectFrom(unsigned int slot);


public: //Pseudo protected

    CardSharpDeviceMap * Debug() { return &Device; }

protected:
    //
    //  Member Data
    CardSharpDeviceMap       Device;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( CardSharp & board );
    friend AddressingSpace & PortMemorySpace( CardSharp & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
};


//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( CardSharp & board )
{
    return board.Device.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( CardSharp & board )
{
    return board.Device.PortMemory;
}




#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative



#endif // CARDSHARPBOARD_MbH
