//
// ChannelInfo.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef ChannelInfo_MbH
#define ChannelInfo_MbH

#include <vector>
#include <ProcessEvents_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS BlkChannelInfo  --  Channel/Device configuration
//=============================================================================
//  ...For some modules we require that channels be allocated in a restricted
//     way -- in a contiguous block from the lowest channel to the number enabled.
//     This channel info base class enforces this requirement by not allowing 
//     individual channel enables.

class BlkChannelInfo
{
public:
    // Ctor
    BlkChannelInfo(int devices, int ch_per_dev, int bits,
                   int muxChannelCount, bool unipolar);
    virtual ~BlkChannelInfo();

    // Methods
    int                 Range() const;
    int                 Offset() const;
    int                 BytesPerSample() const;
    int                 BytesPerEvent() const;
    int                 IntsPerSample() const;
    int                 IntsPerEvent() const;

    bool                Enabled(int ch) const;
    bool                DevEnabled(int dev) const;
    BlkChannelInfo &    EnableChannels( int ch_count );
    BlkChannelInfo &    EnableDevices( int dev_count );
    BlkChannelInfo &    DisableAll();
    BlkChannelInfo &    EnableAll();

    int                 Channels() const;
	int					MuxChannels() const;
    int                 Devices() const;
    int                 ActiveChannels() const;
    int                 ActiveDevices() const;
    int                 ActiveChannelIndex(int N) const;
    int                 ActiveDeviceIndex(int N) const;
    int                 PhysicalChannelIndex(int N) const;
    int                 PhysicalDeviceIndex(int N) const;

    BlkChannelInfo &    Bits(int bits);
    int                 Bits() const;

    BlkChannelInfo &    Unipolar(bool state);
    bool                Unipolar() const;

    OpenWire::EventHandler<OpenWire::NotifyEvent>  OnHardwareUpdate;

protected:
    typedef std::vector<int>    DevArray;

    // Fields
    DevArray        FEnabled;
    int             FBits;
    bool            FUnipolar;
    int             FRange;
    int             FOffset;
    int             FBytesPerSample;
    int             FBytesPerEvent;
    int             FActiveChannels;
    int				FChannelsPerDevice;
	const int		FMuxChannels;
	int				FActiveMuxChannels;
	DevArray		FPhysicalToActiveChannel;
	DevArray		FActiveToPhysicalChannel;
	DevArray		FPhysicalToActiveDevice;
	DevArray		FActiveToPhysicalDevice;

    // Methods
    void            Update();
	void 			BuildMaps();
    int             DeviceOf(int ch) const;
};

//=============================================================================
//  CLASS ChannelInfo  --  Channel/Device configuration
//=============================================================================
//  ...This base class allows individual device enabling.

class ChannelInfo :  public BlkChannelInfo
{
public:
    //
    //  Member functions
    ChannelInfo &    Enabled(int ch, bool state);
    ChannelInfo &    DevEnabled(int dev, bool state);

	using BlkChannelInfo::Enabled;
	using BlkChannelInfo::DevEnabled;
  
protected:
    // Ctor
    ChannelInfo(int devices, int ch_per_dev, int bits, int muxChannelCount, bool unipolar);

};

extern ChannelInfo * ChannelInfoPtr(BlkChannelInfo * blkCI);

//=============================================================================
//  CLASS UnpairedBlkChannelInfo  --  
//=============================================================================

class UnpairedBlkChannelInfo :  public BlkChannelInfo
{
public:
    // Ctor
    UnpairedBlkChannelInfo(int devices, int bits, int muxChannelCount, bool unipolar=false);

};

//=============================================================================
//  CLASS  PairedBlkChannelInfo  -- Paired, block channel cfg
//=============================================================================

class  PairedBlkChannelInfo :  public BlkChannelInfo
{
public:
    // Ctor
    PairedBlkChannelInfo(int devices, int bits, int muxChannelCount, bool unipolar=false);
};

//=============================================================================
//  CLASS PairedChannelInfo  --  Channel Info for 16 bit or less paired hdw
//=============================================================================

class PairedChannelInfo : public ChannelInfo
{
public:
    // Ctor
    PairedChannelInfo(int devices, int bits, int muxChannelCount, bool unipolar=false);
};

//=============================================================================
//  CLASS UnpairedChannelInfo  --  Channel Info for 17 bit or more unpaired hdw
//=============================================================================

class UnpairedChannelInfo : public ChannelInfo
{
public:
    // Ctor
    UnpairedChannelInfo(int devices, int bits, int muxChannelCount, bool unipolar=false);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative

#endif
