// ChicoDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <ChicoDeviceMap_Mb.h>
#include <ChicoDeviceDriver_Mb.h>
#include <SystemSupport_Mb.h>

using namespace std;
using namespace Innovative;

namespace
{
                            //   = 0x10000;
const int  hwGainListMemoryReg   = 0x10400;
                            //   = 0x10800;
const int  hwDioDataReg          = 0x10c00;
const int  hwDioConfigReg        = 0x11000;
                            //   = 0x11400;
                            //   = 0x11800;
const int  hwTimer0Period        = 0x11c00;
const int  hwTimer1Period        = 0x12000;
                            //   = 0x12400;
                            //   = 0x12800;
const int  hwFpaReg              = 0x12c00;
const int  hwFifoLevelControl    = 0x13000;
const int  hwHombreDspSerial     = 0x13400;
const int  hwDspLinkPort0        = 0x13800;
const int  hwDspLinkPort1        = 0x13C00;
const int  hwChicoTimerCfg       = 0x14000;
const int  hwPciInterruptEnable  = 0x14400;
                            //   = 0x14800;
                            //   = 0x14c00;
const int  hwBmReadAddressReg    = 0x15000;
const int  hwBmWriteAddressReg   = 0x15400;
const int  hwBmRegionSizeReg     = 0x15800;
const int  hwSequencerAddr       = 0x15c00;
const int  hwChicoSeqCount       = 0x16000;
const int  hwSequencerInstr      = 0x16400;
                            //   = 0x16800;
                            //   = 0x16c00;
                            //   = 0x17000;
const int  hwGainListCountReg    = 0x17400;
const int  hwGainListAddrReg     = 0x17800;
const int  hwSyncLinkReg0        = 0x17c00;
const int  hwSyncLinkReg1        = 0x18000;
                            //   = 0x18400;
const int  hwDspControlReg       = 0x18800;
                            //   = 0x18c00;
const int  hwInterruptStatus     = 0x19000;
const int  hwControlStatusReg    = 0x19400;
                            //   = 0x19800;
const int  hwPciSequencerMailbox = 0x19c00;
                            //   = 0x1a000;
                            //   = 0x1a400;
                            //   = 0x1a800;
const int  hwPlusTimer0          = 0x1ac00;
const int  hwPlusTimer1          = 0x1b000;
const int  hwPlusTimerOut        = 0x1b400;
const int  hwPlusDdsOut          = 0x1b800;
const int  hwDspStatusReg        = 0x1bc00;
const int  hwPlusSeqCount0       = 0x1c000;
const int  hwPlusSeqCount1       = 0x1c400;

const int  hwPlusPxiStarOut      = 0x1c800;
const int  hwPlusPxiTrigger0Out  = 0x1cc00;
const int  hwPlusPxiTrigger1Out  = 0x1d000;
const int  hwPlusPxiTrigger2Out  = 0x1d400;
const int  hwPlusPxiTrigger3Out  = 0x1d800;
const int  hwPlusPxiTrigger4Out  = 0x1dc00;
const int  hwPlusPxiTrigger5Out  = 0x1e000;
const int  hwPlusPxiTrigger6Out  = 0x1e400;
const int  hwPlusPxiTrigger7Out  = 0x1e800;

}
//---------------------------------------------------------------------------
namespace       Innovative
{

//---------------------------------------------------------------------------
//typedef std::list<int>  InterruptStatusArray;

//===========================================================================
//  CLASS ChicoDeviceMap  --
//===========================================================================

//---------------------------------------------------------------------------
//  ChicoDeviceMap::ChicoDeviceMap() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif
ChicoDeviceMap::ChicoDeviceMap() :
    mControlRegister( mPorts, hwControlStatusReg ),
    DioConfigReg(mPorts, hwDioConfigReg ),
	DioDataReg(mPorts, hwDioDataReg )
{
	 mDeviceDriver = new ChicoDeviceDriver;
}
//---------------------------------------------------------------------------
// ChicoDeviceMap::~ChicoDeviceMap() --
//---------------------------------------------------------------------------

ChicoDeviceMap::~ChicoDeviceMap()
{
	Close();
	delete mDeviceDriver;
}

//---------------------------------------------------------------------------

void ChicoDeviceMap::Open()
{
	if (mDeviceDriver->Opened())
		return;

	mDeviceDriver->Open();
	mPorts.Link(mDeviceDriver->PortMemoryAddress());
//    ClearControlRegister();
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::Close() --
//---------------------------------------------------------------------------

void ChicoDeviceMap::Close()
{
	mDeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::Reset() --
//------------------------------------------------------------------------------
 
void ChicoDeviceMap::Reset()
{
    mControlRegister.BaseboardReset.Clear();
	Innovative::Sleep(20);
    mControlRegister.BaseboardReset.Set();
}
//---------------------------------------------------------------------------
//  ChicoDeviceMap::GetLed() --
//------------------------------------------------------------------------------
 
bool ChicoDeviceMap::Led() const
{
	return mControlRegister.Led;
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::SetLed() --
//------------------------------------------------------------------------------

void ChicoDeviceMap::Led(bool value)
{
	mControlRegister.Led = value;
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::Target() --
//------------------------------------------------------------------------------

unsigned int  ChicoDeviceMap::Target() const
{
	return mDeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::SetTarget() --
//------------------------------------------------------------------------------

void  ChicoDeviceMap::Target(unsigned int target)
{
	mDeviceDriver->Target(target);
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::DioData() --
//------------------------------------------------------------------------------
void ChicoDeviceMap::DioData(int value)
{
	DioDataReg.Value(value);
}
//---------------------------------------------------------------------------
//  ChicoDeviceMap::DioData() --
//------------------------------------------------------------------------------
int  ChicoDeviceMap::DioData()
{
    return DioDataReg.Value();
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::DioConfig() --
//------------------------------------------------------------------------------
void ChicoDeviceMap::DioConfig(int mask)
{
    DioConfigReg.Value(mask);
}

//---------------------------------------------------------------------------
//  ChicoDeviceMap::DioConfig() --
//------------------------------------------------------------------------------
int  ChicoDeviceMap::DioConfig()
{
    return DioConfigReg.Value();
}

//==============================================================================
//  CLASS ChicoDeviceMap::ControlRegister  --
//==============================================================================
//------------------------------------------------------------------------------
// ChicoDeviceMap::ControlRegister::ControlRegister() --
//------------------------------------------------------------------------------

ChicoDeviceMap::ControlRegister::ControlRegister(AddressingSpace&  space,
    int offset)
    : ShadowRegister(space, offset), Led(*this, 1), BaseboardReset(*this, 3)
{
}

/*
//===========================================================================
//  CLASS ChicoDeviceMap::ChicoDeviceMap  --
//===========================================================================
//---------------------------------------------------------------------------
// ChicoDeviceMap::ControlStatusRegister::ControlStatusRegister() --
//------------------------------------------------------------------------------

ChicoDeviceMap::ControlStatusRegister::ControlStatusRegister(AddressingSpace&  space,
    int offset)
    : Register(space, offset)
{
}

*/

} // Innovative

