// ClockGeneratorDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DeviceDriver_Mb.h>
#include <ClockGeneratorDeviceMap_Mb.h>
#include <ClockGenerator_Mb.h>
#include <ClockGeneratorMemoryMap.h>
#include <ClockGeneratorDeviceDriver_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <cmath>
#include <limits>


#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace ClockGenerator_MemoryMap;

//=============================================================================
//  CLASS ClockGeneratorPll  --  PLL timebase control
//=============================================================================

class ClockGeneratorPll : public Ad9511Pfd
{
	typedef Ad9511Pfd inherited;

public:
	//
	//  Member Functions
	ClockGeneratorPll(ClockGeneratorDeviceMap * owner, int index, AddressingSpace & space, int ctl)
		: inherited(index, space, ctl), Owner(owner)
		{
		ChargePumpCurrent(c4800);
        AllowDecimation(false);
		}
	ClockGeneratorPll(ClockGeneratorDeviceMap * owner, double freq, double ref,
		bool enabled, int index, AddressingSpace & space, int ctl)
		: inherited(freq, ref, enabled, index, space, ctl), Owner(owner)
		{
		ChargePumpCurrent(c4800);
        AllowDecimation(false);
		}

protected:
	// Data
	ClockGeneratorDeviceMap	* 	Owner;

	// Methods
	virtual double  PhaseDetectorFrequency() const
		{   return Owner->PfdFrequency();  }
};

//=============================================================================
//  CLASS ClockGeneratorDeviceMap  --  
//=============================================================================
//---------------------------------------------------------------------------
//  constructor for class ClockGeneratorDeviceMap
//---------------------------------------------------------------------------

ClockGeneratorDeviceMap::ClockGeneratorDeviceMap(ClockGenerator * owner)
	: IUsesVirtexJtagLoader(PortMemory, mmJtag),
	  IUsesPciLogicVersionInfo(PortMemory, mmPciVersion),
	  Owner(owner), FSyncSource(sTcxo), FTraceEnable(false),
	  FPfdFrequency(1.0e6),
	  DeviceDriver(new ClockGeneratorDeviceDriver),
	  ControlReg(PortMemory, mmControl),
	  StatusReg(PortMemory, mmStatus),
	  MuxReg(PortMemory, mmOutputMux)
{
	Sync = new Ics843002(0, PortMemory, mmPllControl);
	Dds = new Ad9952(0, PortMemory, mmDdsInstruction, mmDdsData, mmDdsControl);
	Pll = new ClockGeneratorPll(this, 0, PortMemory, mmClockControl);
}

//---------------------------------------------------------------------------
//  destructor for class ClockGeneratorDeviceMap
//---------------------------------------------------------------------------

ClockGeneratorDeviceMap::~ClockGeneratorDeviceMap()
{
	delete Pll;
	Pll = 0;

	delete Dds;
	Dds = 0;

	delete Sync;
	Sync = 0;
	
	delete DeviceDriver;
	DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t ClockGeneratorDeviceMap::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Open()
//---------------------------------------------------------------------------

void  ClockGeneratorDeviceMap::Open()
{
	if( DeviceDriver->Opened() )
		return;

    DeviceDriver->Open();
    //
	//  Link register definitions to device addresses
    PortMemory.Link( ( ii32 *)DeviceDriver->PortMemoryAddress() );

//    ClearControlRegister();
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Close()
//---------------------------------------------------------------------------

void  ClockGeneratorDeviceMap::Close()
{
	DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Reset()
//---------------------------------------------------------------------------

void  ClockGeneratorDeviceMap::Reset()
{
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  ClockGeneratorDeviceMap::Target() const
{
	return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  ClockGeneratorDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Led()
//---------------------------------------------------------------------------

void  ClockGeneratorDeviceMap::Led(bool value)
{
    ControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Led()
//---------------------------------------------------------------------------

bool  ClockGeneratorDeviceMap::Led() const
{
	return ControlReg.Led;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 ClockGeneratorDeviceMap::PeekPort(int offset)
{
	ii32 * PortBasePtr = static_cast<ii32 *>(DeviceDriver->PortMemoryAddress());
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::PokePort(int offset, ii32 value)
{
	ii32 * PortBasePtr = static_cast<ii32 *>(DeviceDriver->PortMemoryAddress());
	PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::SyncSource() -
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::SyncSource(IISyncSource src)
{
	FSyncSource = src;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::SyncSource() -
//---------------------------------------------------------------------------

ClockGeneratorDeviceMap::IISyncSource ClockGeneratorDeviceMap::SyncSource() const
{
	return FSyncSource;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::OutputClock() -  Specify output
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::OutputClock(IIOutputClock value)
{
	FOutputClock = value;
	if (FOutputClock == oExternal)
		{
		MuxReg.PllEnable.Value(false);
		}
	else
		{
		MuxReg.PllEnable.Value(true);
		}
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Mux() -  Query output
//---------------------------------------------------------------------------

ClockGeneratorDeviceMap::IIOutputClock ClockGeneratorDeviceMap::OutputClock() const
{
	return FOutputClock;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::SyncFrequency() -  Set sync clock input freq
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::SyncFrequency(double frequency)
{
	FSyncFrequency = frequency;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::SyncFrequency() -  Query sync input clock freq
//---------------------------------------------------------------------------

double ClockGeneratorDeviceMap::SyncFrequency() const
{
	return FSyncFrequency;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::AllowAsymmetricDutyCycle() -
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::AllowAsymmetricDutyCycle(bool state)
{
	Pll->AllowAsymmetricDutyCycle(state);
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::AllowAsymmetricDutyCycle() -
//---------------------------------------------------------------------------

bool ClockGeneratorDeviceMap::AllowAsymmetricDutyCycle() const
{
	return Pll->AllowAsymmetricDutyCycle();
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Frequency() -  Set output clock frequency
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::Frequency(double frequency)
{
    FFrequency = frequency;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Frequency() -  Return output clock frequency
//---------------------------------------------------------------------------

double ClockGeneratorDeviceMap::Frequency() const
{
	double result;

	switch (FOutputClock)
		{
        default:
		case oInternal:
			result = Pll->FrequencyActual();
			break;
		case oExternal:
			result = FFrequency;
			break;
		}

	return result;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::OutputEnable() -  Set output enable state
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::OutputEnable(IIOutputEnable state)
{
	FOutputEnable = state;
	int field;
	switch(state)
		{
		case oActive:	field = 3;		break;
		case oLow:		field = 1;		break;
        default:
		case oTristate:	field = 2;		break;
		}

	MuxReg.OutputEnable.Value(field);
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::OutputEnable() -  Query output enable state
//---------------------------------------------------------------------------

ClockGeneratorDeviceMap::IIOutputEnable ClockGeneratorDeviceMap::OutputEnable() const
{
	return FOutputEnable;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Start() -  Start clock
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::Start()
{
	if (FOutputClock == oExternal)
        return;
        
	if (FSyncSource == sSync)
		Sync->Source(Ics843002::rcSyncIn);
	else
		Sync->Source(Ics843002::rcTcxo);

	Sync->SourceFrequency(SyncFrequency());

	// Sync drives DDS reference at fixed rate
	Sync->Frequency(40.0e6);

	Dds->Reference(Sync->FrequencyActual());

	if (FFrequency < Pll->Min())
		{
		// DDS drives output
		int divider = std::min(static_cast<int>(Dds->Max()/FFrequency) & ~1, Pll->MaxOutputDivider());
		Pll->OutputDivider(divider);
		Dds->Frequency(FFrequency*divider);
		Pll->OutputClock(Ad9511::oExternal);
        }
	else
		{
		// PLL drives output
		double f;
		if (FFrequency > Dds->Max())
			{
			if(FFrequency/Dds->Max() > 2.0)
				f = FFrequency/Dds->Max();
			else
				f = 2.0;
			}
		else
			{
			if(FFrequency/Dds->Max() > 2.0)
				f = 1.0/(FFrequency/Dds->Max());
			else
				f = 0.5;
			}


        // Round the factor up to the next largest integer
		// to allow better optimization of the downstream AD9511 PFD divider
		int divisor = static_cast<int>(std::ceil(f));
		double best_remainder = std::numeric_limits<double>::max();
		double best_divisor = divisor;
		double dds_freq;
		do
			{
			f = static_cast<double>(divisor);
			dds_freq = FFrequency / f;
			double remainder = std::fmod(FFrequency, f);
			if (remainder < best_remainder)
				{
				best_remainder = remainder;
				best_divisor = divisor;
				}
			++divisor;
			}
		while ( best_remainder && (dds_freq > 1.0e6) );
		f = static_cast<double>(best_divisor);
        
		Dds->Frequency(FFrequency/f);
		Pll->OutputClock(Ad9511::oVco);
		}

	Pll->Reference(Dds->Frequency());
	Pll->Frequency(FFrequency);
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::Locked() -  Return clock lock status
//---------------------------------------------------------------------------

bool ClockGeneratorDeviceMap::Locked() const
{
	return Sync->Locked() && Pll->Locked();
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::TraceEnable() -  Enable/disable trace log
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::TraceEnable(bool state)
{
	FTraceEnable = state;

	Sync->Debug.Enabled(state);
	Dds->Debug.Enabled(state);
	Pll->Debug.Enabled(state);
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::TraceEnable() -  Query trace log state
//---------------------------------------------------------------------------

bool ClockGeneratorDeviceMap::TraceEnable() const
{
	return FTraceEnable;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::PfdFrequency() -
//---------------------------------------------------------------------------

void ClockGeneratorDeviceMap::PfdFrequency(float value)
{
	FPfdFrequency = value;
}

//---------------------------------------------------------------------------
//  ClockGeneratorDeviceMap::PfdFrequency() -
//---------------------------------------------------------------------------

float ClockGeneratorDeviceMap::PfdFrequency() const
{
	return FPfdFrequency;
}



} // namespace Innovative


