// ClockGeneratorDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef ClockGeneratorDeviceMap_MbH
#define ClockGeneratorDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <Magic_Mb.h>
#include <ICS843002_Mb.h>
#include <Ad9952_Mb.h>
#include <Ad9511_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class ClockGenerator;

//===========================================================================
//  CLASS ClockGeneratorDeviceMap  -- Register & low level support for ClockGenerator
//===========================================================================

class ClockGeneratorDeviceMap :
	public IUsesVirtexJtagLoader,
    public IUsesPciLogicVersionInfo
{
	friend class ClockGenerator;

public:
	enum IISyncSource { sSync, sTcxo };
	enum IIOutputClock { oInternal, oExternal };
	enum IIOutputEnable { oActive, oLow, oTristate };

	//
	//  Member Functions
	ClockGeneratorDeviceMap(ClockGenerator * owner);
	virtual ~ClockGeneratorDeviceMap();

    size_t          BoardCount();
	void            Open();
	void            Close();
	void            Reset();
    void            Start();

	ii32  PeekPort(int offset);
	void  PokePort(int offset, ii32 value);

	// Properties
	void                SyncSource(IISyncSource src);
	IISyncSource		SyncSource() const;

	void                SyncFrequency(double frequency);
	double              SyncFrequency() const;

	void                Frequency(double frequency);
	double              Frequency() const;

	void                OutputClock(IIOutputClock value);
	IIOutputClock		OutputClock() const;

	void 				OutputEnable(IIOutputEnable state);
	IIOutputEnable		OutputEnable() const;

	void 				AllowAsymmetricDutyCycle(bool state);
	bool				AllowAsymmetricDutyCycle() const;

	void                Target(int target);
	int                 Target() const;

	void                Led(bool value);
	bool                Led() const;

	bool                Locked() const;

	void 				TraceEnable(bool state);
	bool				TraceEnable() const;

	void 				PfdFrequency(float value);
	float  				PfdFrequency() const;

protected:
	ClockGenerator *    Owner;

protected:
	//
	//  Class OutputControlRegister
	class OutputControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit       PllEnable;
		RegisterBitGroup  OutputEnable;

	public:
		OutputControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   PllEnable(*this, 0), OutputEnable(*this, 1, 2)
			{
			//  Default setup ????
			}
	};

	//
	//  Class PciControlRegister
	class PciControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit       Led;

	public:
		PciControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Led(*this, 1)
			{
			//  Default setup ????
			}
	};

	//
	//  Class PciStatusRegister
	class PciStatusRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBit       Slot64;

	public:
		PciStatusRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Slot64(*this, 0)
			{
			//  Default setup ????
			}
	};

	// Fields
	IISyncSource					 	FSyncSource;
	double                          	FSyncFrequency;
	IIOutputClock					   	FOutputClock;
	IIOutputEnable						FOutputEnable;
	double								FFrequency;
	bool								FTraceEnable;
	float 								FPfdFrequency;

	//
	//  ...Registers
	Ics843002 *                        	Sync;
	Ad9952 *						   	Dds;
	Ad9511 * 					  		Pll;
	AddressingSpace                    	PortMemory;
	InnovativeKernel::DeviceDriver *   	DeviceDriver;
	PciControlRegister                 	ControlReg;
	PciStatusRegister                  	StatusReg;
	OutputControlRegister				MuxReg;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
