// ClockGenerator_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <ClockGenerator_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS ClockGenerator  -- Digital Transmitter PMC module object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class ClockGenerator
//---------------------------------------------------------------------------

ClockGenerator::ClockGenerator()
    :  Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class ClockGenerator
//---------------------------------------------------------------------------

ClockGenerator::~ClockGenerator()
{
    Close();
}

//---------------------------------------------------------------------------
//  ClockGenerator::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t ClockGenerator::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  ClockGenerator::Open()
{
    Device.Open();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  ClockGenerator::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Reset() -- Cold-start initialization
//---------------------------------------------------------------------------

void ClockGenerator::Reset()
{
    return Device.Reset();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  ClockGenerator::Target() const
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  ClockGenerator::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  ClockGenerator::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  ClockGenerator::Led() const
{
	return Device.Led();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  ClockGenerator::Led( bool value )
{
	Device.Led(value);
}

//---------------------------------------------------------------------------
//  ClockGenerator::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  ClockGenerator::PeekPort(int offset )
{
	return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  ClockGenerator::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  ClockGenerator::PokePort( int offset, ii32 value )
{
	Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  ClockGenerator::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & ClockGenerator::PciLogicVersion()
{
	return Device;
}

//---------------------------------------------------------------------------
//  ClockGenerator::Eeprom() -- Return reference to EEProm object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & ClockGenerator::Eeprom()
{
	return Device;
}


//---------------------------------------------------------------------------
//  ClockGenerator::Sync() --  Return reference to ICS83002 sync PLL
//---------------------------------------------------------------------------

Ics843002 & ClockGenerator::Sync()
{
	return *Device.Sync;
}

//---------------------------------------------------------------------------
//  ClockGenerator::Dds() --  Return reference to AD9952 DDS
//---------------------------------------------------------------------------

Ad9952 & ClockGenerator::Dds()
{
	return *Device.Dds;
}

//---------------------------------------------------------------------------
//  ClockGenerator::Pll() --  Return reference to AD9511 PLL
//---------------------------------------------------------------------------

Ad9511 & ClockGenerator::Pll()
{
	return *Device.Pll;
}

//---------------------------------------------------------------------------
//  ClockGenerator::SyncSource() --  Set source of clock to Sync PLL
//---------------------------------------------------------------------------

void ClockGenerator::SyncSource(IISyncSource src)
{
	Device.SyncSource(static_cast<ClockGeneratorDeviceMap::IISyncSource>(src));
}

//---------------------------------------------------------------------------
//  ClockGenerator::SyncSource() --  Query source of clock to Sync PLL
//---------------------------------------------------------------------------

ClockGenerator::IISyncSource ClockGenerator::SyncSource() const
{
	return static_cast<ClockGenerator::IISyncSource>(Device.SyncSource());
}

//---------------------------------------------------------------------------
//  ClockGenerator::SyncFrequency() --  Set Sync PLL input clock frequency
//---------------------------------------------------------------------------

void ClockGenerator::SyncFrequency(double frequency)
{
	Device.SyncFrequency(frequency);
}

//---------------------------------------------------------------------------
//  ClockGenerator::SyncFrequency() --  Query Sync PLL input clock frequency
//---------------------------------------------------------------------------

double ClockGenerator::SyncFrequency() const
{
	return Device.SyncFrequency();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Start() --  Starts ClockGen
//---------------------------------------------------------------------------

void ClockGenerator::Start()
{
	Device.Start();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Frequency() --  Set output frequency
//---------------------------------------------------------------------------

void ClockGenerator::Frequency(double frequency)
{
	Device.Frequency(frequency);
}

//---------------------------------------------------------------------------
//  ClockGenerator::Frequency() --  Query output frequency
//---------------------------------------------------------------------------

double ClockGenerator::Frequency() const
{
	return Device.Frequency();
}

//---------------------------------------------------------------------------
//  ClockGenerator::Clock() --  Specify output clock source
//---------------------------------------------------------------------------
void  ClockGenerator::OutputClock(IIOutputClock value)
{
	Device.OutputClock(static_cast<ClockGeneratorDeviceMap::IIOutputClock>(value));
}

//---------------------------------------------------------------------------
//  ClockGenerator::Clock() --  Query output clock source
//---------------------------------------------------------------------------

ClockGenerator::IIOutputClock ClockGenerator::OutputClock() const
{
	return static_cast<ClockGenerator::IIOutputClock>(Device.OutputClock());
}

//---------------------------------------------------------------------------
//  ClockGenerator::Locked() --  Query clock lock state
//---------------------------------------------------------------------------

bool  ClockGenerator::Locked() const
{
	return Device.Locked();
}

//---------------------------------------------------------------------------
//  ClockGenerator::OutputEnable() --  Set mux output output state
//---------------------------------------------------------------------------

void ClockGenerator::OutputEnable(IIOutputEnable state)
{
	Device.OutputEnable(static_cast<ClockGeneratorDeviceMap::IIOutputEnable>(state));
}

//---------------------------------------------------------------------------
//  ClockGenerator::OutputEnable() --  Query mux output enable state
//---------------------------------------------------------------------------

ClockGenerator::IIOutputEnable ClockGenerator::OutputEnable() const
{
	return static_cast<ClockGenerator::IIOutputEnable>(Device.OutputEnable());
}

//---------------------------------------------------------------------------
//  ClockGenerator::TraceEnable() --  Set trace log state
//---------------------------------------------------------------------------

void ClockGenerator::TraceEnable(bool state)
{
	Device.TraceEnable(state);
}

//---------------------------------------------------------------------------
//  ClockGenerator::TraceEnable() --  Query trace log state
//---------------------------------------------------------------------------

bool  ClockGenerator::TraceEnable() const
{
	return Device.TraceEnable();
}

//---------------------------------------------------------------------------
//  ClockGenerator::AllowAsymmetricDutyCycle() --  
//---------------------------------------------------------------------------

void ClockGenerator::AllowAsymmetricDutyCycle(bool state)
{
	Device.AllowAsymmetricDutyCycle(state);
}

//---------------------------------------------------------------------------
//  ClockGenerator::AllowAsymmetricDutyCycle() --
//---------------------------------------------------------------------------

bool ClockGenerator::AllowAsymmetricDutyCycle() const
{
	return Device.AllowAsymmetricDutyCycle();
}

//---------------------------------------------------------------------------
//  ClockGenerator::BusMasterSize() --
//---------------------------------------------------------------------------

void ClockGenerator::BusMasterSize(int /*bytes*/)
{
    // A dummy function used only for applet compatibility
}

//---------------------------------------------------------------------------
//  ClockGenerator::BusMasterSize() --
//---------------------------------------------------------------------------

int  ClockGenerator::BusMasterSize() const
{
    // A dummy function used only for applet compatibility
	return 0;
}

//---------------------------------------------------------------------------
//  ClockGenerator::PfdFrequency() -
//---------------------------------------------------------------------------

void ClockGenerator::PfdFrequency(float value)
{
	Device.PfdFrequency(value);
}

//---------------------------------------------------------------------------
//  ClockGenerator::PfdFrequency() -
//---------------------------------------------------------------------------

float ClockGenerator::PfdFrequency() const
{
	return Device.PfdFrequency();
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

}  // namespace Innovative
