// ClockGenerator_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef ClockGenerator_MbH
#define ClockGenerator_MbH

#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <ClockGeneratorDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class ClockGeneratorDeviceMap;

//===========================================================================
//  CLASS ClockGenerator  -- Digital Transmitter PMC module object
//===========================================================================

class ClockGenerator
{
    friend class ClockGeneratorDeviceMap;

public:
	enum IISyncSource { sSync, sTcxo };
	enum IIOutputClock { oInternal, oExternal };
	enum IIOutputEnable { oActive, oLow, oTristate };

	//
	//  Member Functions
	ClockGenerator();
	~ClockGenerator();

    size_t          BoardCount();
    void            Open();
    void            Close();
	void            Reset();
    void            Start();
    
	ii32  PeekPort( int offset );
	void  PokePort( int offset, ii32 value );

	// Properties
	void  				Led(bool value);
	bool  				Led() const;

	void                SyncSource(IISyncSource src);
	IISyncSource		SyncSource() const;

	void                SyncFrequency(double frequency);
	double              SyncFrequency() const;

	void                Frequency(double frequency);
	double              Frequency() const;

	void                OutputClock(IIOutputClock value);
	IIOutputClock		OutputClock() const;

	void                OutputEnable(IIOutputEnable state);
	IIOutputEnable		OutputEnable() const;

	void 				AllowAsymmetricDutyCycle(bool state);
	bool				AllowAsymmetricDutyCycle() const;

	void                Target(int target);
	int                 Target() const;

	bool                Locked() const;

	void 				TraceEnable(bool state);
	bool				TraceEnable() const;

    void                BusMasterSize(int bytes);
	int                 BusMasterSize() const;

	void 				PfdFrequency(float value);
	float  				PfdFrequency() const;

	ClockGeneratorDeviceMap * Debug() { return &Device; };

	// EEprom programming
	IUsesVirtexJtagLoader & Eeprom();

	// Trigger Support

	// Methods
	Ics843002 &         Sync();
	Ad9952 &         	Dds();
	Ad9511 &         	Pll();
	PmcIdrom &          IdRom();

    PciLogicVersionInfoIntf & PciLogicVersion();

protected:
    //
    //  Member Data
    ClockGeneratorDeviceMap             Device;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

