//
// ClockBase_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#include <Clockbase_Mb.h>
                
namespace Innovative
{

//=============================================================================
//  CLASS ClockBase  --  Clock source base class
//=============================================================================
//-----------------------------------------------------------------------------
//  constructor for class ClockBase  --  Default ctor
//-----------------------------------------------------------------------------

ClockBase::ClockBase(int index)
    : Noncopyable(), FFrequency(0.), FReference(1.0e6), FEnabled(false),
      FIndex(index), FClockFactor(1), FReferenceCalibrationFactor(1.0)

{
}

//-----------------------------------------------------------------------------
//  constructor for class ClockBase  --  Enable/Disable & parms
//-----------------------------------------------------------------------------

ClockBase::ClockBase(double freq, double ref, bool enabled, int index)
    : Noncopyable(), FFrequency(freq), FReference(ref), 
      FEnabled(enabled), FIndex(index), FClockFactor(1), 
      FReferenceCalibrationFactor(1.0)
{
}

//-----------------------------------------------------------------------------
//  destructor for class ClockBase
//-----------------------------------------------------------------------------

ClockBase::~ClockBase()
{
}

//-----------------------------------------------------------------------------
//  ClockBase::Period()  --  Set period between clock pulses
//-----------------------------------------------------------------------------

ClockBase & ClockBase::Period(double period)
{
    Frequency(1.0/period);
    return *this;
}

//-----------------------------------------------------------------------------
//  ClockBase::Period()  --  Retreive period between clock pulses
//-----------------------------------------------------------------------------

double ClockBase::Period() const
{
    if (FFrequency == 0.0)
        return 0.0;
        
    return 1.0/FFrequency;
}

//-----------------------------------------------------------------------------
//  ClockBase::Frequency()  --  
//-----------------------------------------------------------------------------

double ClockBase::Frequency() const         
{  
    return GetFrequency();
}

//-----------------------------------------------------------------------------
//  ClockBase::Frequency()  --  
//-----------------------------------------------------------------------------

ClockBase & ClockBase::Frequency(double freq)
{
    FFrequency = freq;

    if (FReference)
        SetFrequency(freq);

    return *this;
}

//-----------------------------------------------------------------------------
//  ClockBase::FrequencyActual()  --
//-----------------------------------------------------------------------------

double ClockBase::FrequencyActual() const
{
    return GetFrequencyActual();
}

//-----------------------------------------------------------------------------
//  ClockBase::ClockFactor()  -- get clock factor
//-----------------------------------------------------------------------------

double ClockBase::ClockFactor() const
{
    return GetClockFactor();
}

//-----------------------------------------------------------------------------
//  ClockBase::ClockFactor()  -- set clock factor
//-----------------------------------------------------------------------------

ClockBase &  ClockBase::ClockFactor(double value)
{
    SetClockFactor(value);
    return *this;
}

//-----------------------------------------------------------------------------
//  ClockBase::ReferenceCalibrationFactor()  -- get clock factor
//-----------------------------------------------------------------------------

double ClockBase::ReferenceCalibrationFactor() const
{
    return GetReferenceCalibrationFactor();
}

//-----------------------------------------------------------------------------
//  ClockBase::ReferenceCalibrationFactor()  -- set clock factor
//-----------------------------------------------------------------------------

ClockBase &  ClockBase::ReferenceCalibrationFactor(double value)
{
    if ( value < .5 || value > 1.5)    // disallow dubious corrections
        value = 1.0;

    FReferenceCalibrationFactor = value;
    SetReferenceCalibrationFactor(value);

    return *this;
}

//-----------------------------------------------------------------------------
//  ClockBase::SetReferenceCalibrationFactor()  -- set reference correction
//-----------------------------------------------------------------------------

void ClockBase::SetReferenceCalibrationFactor(double /*value*/)
{
}

//-----------------------------------------------------------------------------
//  ClockBase::GetReferenceCalibrationFactor()  -- return reference correction
//-----------------------------------------------------------------------------

double  ClockBase::GetReferenceCalibrationFactor() const
{
    return FReferenceCalibrationFactor;
}

//-----------------------------------------------------------------------------
//  ClockBase::Reference()  --
//-----------------------------------------------------------------------------

ClockBase & ClockBase::Reference(double freq)
{
    FReference = freq;
    return *this;
}

//-----------------------------------------------------------------------------
//  ClockBase::Reference()  --
//-----------------------------------------------------------------------------

double ClockBase::Reference() const
{
    return FReference * FReferenceCalibrationFactor;
}

//-----------------------------------------------------------------------------
//  ClockBase::Enabled()  --  
//-----------------------------------------------------------------------------

ClockBase & ClockBase::Enabled(bool state)
{
    SetEnabled(state);
    return *this; 
}

//-----------------------------------------------------------------------------
//  ClockBase::Enabled()  --  
//-----------------------------------------------------------------------------

bool ClockBase::Enabled() const         
{  
    return FEnabled;    
}

//-----------------------------------------------------------------------------
//  ClockBase::Index()  --  
//-----------------------------------------------------------------------------

int ClockBase::Index() const        
{  
    return FIndex;
}

} // namespace Innovative



