// 
// ClockBase_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef ClockBase_MbH
#define ClockBase_MbH

#include <NonCopyable_Mb.h>

namespace Innovative
{

//=============================================================================
//  CLASS ClockBase  --  Clock source base class
//=============================================================================

class ClockBase : public Noncopyable
{
public:
    // Ctors
    ClockBase(int index);
    ClockBase(double freq, double ref, bool enabled, int index);
    virtual ~ClockBase();

    // Properties
    double          Frequency() const;
    ClockBase &     Frequency(double freq);

    ClockBase &     Period(double period);
    double          Period() const;

    ClockBase &     Reference(double freq);
    double          Reference() const;

    ClockBase &     Enabled(bool state);
    bool            Enabled() const;

    ClockBase &     ClockFactor(double value);
    double          ClockFactor() const;

    ClockBase &     ReferenceCalibrationFactor(double value);
    double          ReferenceCalibrationFactor() const;

    // Methods
    double          FrequencyActual() const;
    int             Index() const;

protected:
    // Virtuals
    virtual void    SetReferenceCalibrationFactor(double value);
    virtual double  GetReferenceCalibrationFactor() const;
    virtual void    SetFrequency(double freq) = 0;
    virtual double  GetFrequency() const
        {  return FFrequency;  }
    virtual double  GetClockFactor() const
        {  return FClockFactor;  }
    virtual void    SetClockFactor(double value)
        {  FClockFactor=value;  }

    virtual double  GetFrequencyActual() const = 0;
    virtual void    SetEnabled(bool state) = 0;

protected:
    //
    //  Member Data
    double                  FFrequency;
    double                  FReference;
    bool                    FEnabled;
    int                     FIndex;
    double                  FClockFactor;
    double                  FReferenceCalibrationFactor;
};


} // namespace Innovative
         
#endif
