// CoffThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <CoffThread_Mb.h>
#include <StringSupport_Mb.h>
#include <DebugSupport_Mb.h>
#include <DataTypes_Mb.h>

namespace   Innovative
{

//===========================================================================
// CLASS: CoffThread - COFF4 background-loader class
//===========================================================================

//---------------------------------------------------------------------------
//  Default constructor
//---------------------------------------------------------------------------

CoffThread::CoffThread()
    : inherited("CoffThread"), Loading(Event::ManualReset),
      Parsing(Event::ManualReset)
{
    Resume();
}

//---------------------------------------------------------------------------
//  Default destructor
//---------------------------------------------------------------------------

CoffThread::~CoffThread()
{
}

//------------------------------------------------------------------------------
//  CoffThread::Init() -- Init multi-objects
//------------------------------------------------------------------------------

void CoffThread::Init()
{
    inherited::Init();
	AddObject(Loading, sLoading);
    AddObject(Parsing, sParsing);
}

//---------------------------------------------------------------------------
//  CoffThread::Execute() --
//---------------------------------------------------------------------------

void CoffThread::Execute()
{
    while (!Terminated())
        {
        // WaitResult is either a) Suicide event, b) Load/Parse event
        int WaitResult = WaitFor();
        
        if (WaitResult == sParsing)
            {
            Parsing.Clear();
            DoParse();
            }
        else if (WaitResult == sLoading)
            {
            Loading.Clear();
            DoLoad();
            }
        }
}

//---------------------------------------------------------------------------
//  CoffThread::Load() --  Initiate background load
//---------------------------------------------------------------------------

void CoffThread::Load(const std::string & Filename)
{
    FFilename = Filename;
    Loading.Set();
}

//---------------------------------------------------------------------------
//  CoffThread::DoLoad() --  Load COFF image to specified target
//---------------------------------------------------------------------------

int CoffThread::DoLoad()
{
    if (!DoParse())
        return 0;

    return Image.Load(FFilename.c_str());
}

//---------------------------------------------------------------------------
//  CoffThread::Parse() --  Initiate COFF file parse
//---------------------------------------------------------------------------

void CoffThread::Parse(const std::string & infile)
{
    FFilename = infile;
    Parsing.Set();
}

//---------------------------------------------------------------------------
//  CoffThread::DoParse() --  Background parse COFF file header
//---------------------------------------------------------------------------

int CoffThread::DoParse()
{
    return Image.Parse(FFilename.c_str());
}


} // namespace Innovative
