// CommandLineIntf_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef CommandLineIntf_MbH
#define CommandLineIntf_MbH
#include <sstream>          // Avoids compiler error with _algobase.h
#include <vector>
#include <memory>
#include <string>
#include <iterator>

namespace Innovative
{

//==============================================================================
//  CLASS CommandLineArguments  --
//==============================================================================

class CommandLineArguments
{
    typedef std::vector<char*>  CharArray;

public:
    CommandLineArguments()
      {  }
    CommandLineArguments(const CommandLineArguments & other)
      {  Copy(other);  }
    virtual ~CommandLineArguments()
        {  Clear();  }
    char *  operator [] (size_t idx)
        {  return args[idx];  }
    size_t  size()
        {  return args.size();  }

    // Forward iterators.
    CharArray::iterator begin()
        {  return args.begin();  }
    const CharArray::const_iterator begin() const
        {  return args.begin();  }

    CharArray::iterator end()
        {  return args.end();  }
    const CharArray::const_iterator end() const
        {  return args.end();  }

    // Reverse iterators.
    CharArray::reverse_iterator rbegin()
        {  return args.rbegin();  }
    const CharArray::const_reverse_iterator rbegin() const
        {  return args.rbegin();  }

    CharArray::reverse_iterator rend()
        {  return args.rend();  }
    const CharArray::const_reverse_iterator rend() const
        {  return args.rend();  }


    CommandLineArguments & operator = (const CommandLineArguments & other)
        {
            if (this != &other) // protect against invalid self-assignment
                Copy(other);

            // by convention, always return *this
            return *this;
        }


    void Add(const std::string & txt)
        {
        const char * s = txt.c_str();
        size_t len = txt.size();
        char * d = new char[len+1];
        for (size_t i = 0; i < len; ++i)
            d[i] = s[i];
        d[len] = 0;
        args.push_back(d);
        }

private:

    std::vector<char*>  args;

    // Delete entire contents
    void Clear()
    {
        for (CharArray::iterator i = args.begin(); i != args.end(); ++i)
            {
            delete [] *i;
            *i = 0;
            }

        args.clear();
    }
    // Copy entire contents from other
    void Copy(const CommandLineArguments & other)
    {
        Clear();
        for (size_t i = 0; i < other.args.size(); ++i)
            Add(other.args[i]);
    }
};

extern CommandLineArguments GetCommandLineArguments();

} // Innovative

#endif
