// CommonFourier_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <CommonFourier_Mb.h>
#include <ipp.h>   // ipps.h is not redistributable!
#include <cmath>
#include <Magic_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <algorithm>

using namespace std;

namespace   Innovative
{

//===========================================================================
//  CLASS CommonFourier  --
//===========================================================================

//---------------------------------------------------------------------------
// CommonFourier::CommonFourier() --  Constructor
//---------------------------------------------------------------------------

CommonFourier::CommonFourier()
	: FNormalization(Normalize), FOrder(10), FSamples(1024), FReach(512)
{
}

//---------------------------------------------------------------------------
// CommonFourier::CommonFourier() -- Copy constructor
//---------------------------------------------------------------------------

CommonFourier::CommonFourier(const CommonFourier & other)
	: FNormalization(other.FNormalization), FOrder(other.FOrder),
	  FSamples(other.FSamples), FReach(other.FReach)
{
}

//---------------------------------------------------------------------------
// CommonFourier::~CommonFourier() --  Destructor
//---------------------------------------------------------------------------

CommonFourier::~CommonFourier()
{
	Stop();
}

//---------------------------------------------------------------------------
//  CommonFourier::operator = --  Assignment operator
//---------------------------------------------------------------------------

CommonFourier & CommonFourier::operator = (const CommonFourier & other)
{
	FNormalization = other.FNormalization;
	FOrder = other.FOrder;
	FSamples = other.FSamples;
	FReach = other.FReach;

	return *this;
}

//---------------------------------------------------------------------------
//  CommonFourier::Order() --  Setter method for Order property
//---------------------------------------------------------------------------

void  CommonFourier::Order( int value )
{
	FOrder = value;
	FSamples = (1 << value);
	FReach = FSamples >> 1;

	Start();
}

//---------------------------------------------------------------------------
//  CommonFourier::Samples() --
//---------------------------------------------------------------------------

void CommonFourier::Samples( int value )
{
    // Set order to set both FSamples and FOrder together
	Order(TrailingZeroCount(NextLargestPowerOfTwo(value) / 2));
}

//---------------------------------------------------------------------------
//  CommonFourier::Start() --
//---------------------------------------------------------------------------

bool CommonFourier::Start()
{
	Stop();

	IppComplexDG TimeDG(mTime);
	TimeDG.Resize(FSamples);

	IppComplexDG FrequencyDG(mFrequency);
	FrequencyDG.Resize(FSamples);

	bool result = true;

	switch( Normalization() )
	  {
	  case NormalizeNone:
		result &= alloc(IPP_FFT_NODIV_BY_ANY, ippAlgHintAccurate);
		break;

	  case Normalize:
		result &= alloc(IPP_FFT_DIV_FWD_BY_N, ippAlgHintAccurate);
		break;

	  case NormalizeSqrt:
		result &= alloc(IPP_FFT_DIV_BY_SQRTN, ippAlgHintAccurate);
		break;
	  }

	return result;
}

//---------------------------------------------------------------------------
//  CommonFourier::alloc() --
//---------------------------------------------------------------------------

bool CommonFourier::alloc(int flag, int hint)
{
	bool result = true;
	IppStatus status;

	// get sizes for required buffers
	int sizeSpec = 0;
	int sizeInit = 0;
	int sizeBuffer = 0;

	status = ippsFFTGetSize_C_32fc(Order(), flag, static_cast<IppHintAlgorithm>(hint),
		&sizeSpec, &sizeInit, &sizeBuffer);
	result &= (status == ippStsNoErr);

	mWorking.resize(sizeBuffer);

	std::vector<unsigned char> mInit;
	mInit.resize(sizeInit);

	/// initialize FFT specification structure
	mSpec.resize(sizeSpec);

	FFTSpec_C_32fc ** pSpec = reinterpret_cast<FFTSpec_C_32fc**>(&mState);

	status = ippsFFTInit_C_32fc(pSpec, Order(), flag, static_cast<IppHintAlgorithm>(hint),
		reinterpret_cast<Ipp8u*>(&(mSpec.front())), reinterpret_cast<Ipp8u*>(&(mInit.front())));
	result &= (status == ippStsNoErr);

	return result;
}

//---------------------------------------------------------------------------
//  CommonFourier::Stop() --
//---------------------------------------------------------------------------

void CommonFourier::Stop()
{
}

} // namespace Innovative
