#ifndef COPIOUS_BOARD_H
#define COPIOUS_BOARD_H

#include <DataTypes_Mb.h>
#include <Copious_DeviceMap_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative {

//===========================================================================
//  CLASS Copious  -- Copious baseboard class
//===========================================================================

class Copious : public IFiclTarget
{
public:
    enum LedColor {Red, Green};
    //
    //  Member Functions
    Copious();
    virtual ~Copious();

    //  Informational Methods
    size_t                      BoardCount();
    bool                        LicenseCheck(unsigned int & code);

    std::string                 Name() const;
    std::string                 Revision() const;

    int                         Target();
    void                        Target(int value);

    bool                        Led(LedColor color);
//    void                        Led(bool value);

//    void                        CustomInterruptSource(SbcDuoInterruptSources src);
//    SbcDuoInterruptSources      CustomInterruptSource();

    void                        MasterTrigger(bool state);
    void                        SoftwareTrigger(bool state);
//    void                        ArmPpsTrigger();

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    virtual void                Start();
    virtual void                Stop();

    //
    //  Properties
    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

protected:
    //
    //  Member Data
    Copious_DeviceMap       Device;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( Copious & board );
    friend AddressingSpace & PortMemorySpace( Copious & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( Copious & board )
{
    return board.Device.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( Copious & board )
{
    return board.Device.PortMemory;
}


}


#endif // COPIOUS_BOARD_H
