#include <Copious_DeviceMap_Mb.h>
#include <Copious_DeviceDriver_Mb.h>
#include <Copious_MemoryMap_Mb.h>

namespace Innovative {

//===========================================================================
//  CLASS FmcCHdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

template<class T>
class FmcCHdwPluginImpl : public IPacketDevicePlugin
{
public:
    FmcCHdwPluginImpl( T * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        {  PDM->CloseHardware();  }

    T * PDM;
};

using namespace Copious_MemoryMap;

namespace  // nameless
{
PacketDeviceRegisters X6DRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
    mmLinkControl,        // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters X6RegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

X6Peripherals::MapRegisters X6PeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

X6PacketizerCfg::MapRegisters X6PacketizerRegAddrs =
    {
    mmPktzer_Enable,          //  PacketizerEnable_Addr
    mmPktzer_AuxHdr,          //  AuxHeader_Addr
    mmPktzer_VeloInHdr,       //  VelociaInHeader_Addr
    mmPktzer_ForceSize,       //  PacketizerForceSize_Addr
    mmPktzer_IntervalPeriod,  //  PacketizerIntervalPeriod_Addr
    };

X6AlertSystem::MapRegisters X6AlertsRegAddrs =
    {
    mmAlert_Enable,       //  AlertEnable_Addr
    mmAlert_Control,      //  AlertControl_Addr
    mmAlert_Software,     //  SoftwareAlert_Addr;
    mmAlert_Clear,        //  ClearAlert_Addr;
    };

X6DepacketizerCfg::MapRegisters X6DepacketizerRegAddrs =
    {
    mmVeloOutTestLB,      //  TestLoopbackPid_Addr;
    mmVeloOutDataPid,     //  OutgoingDataPid_Addr
    };

X6Info::MapRegisters CopiousInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmSubrevision,        //  BitstreamDate_Addr - not used!
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };
}

//---------------------------------------------------------------------------
//  Local constants
//---------------------------------------------------------------------------
namespace {

const int    MaxInDevices      = 2;
const int    ChannelsPerDevice = 2;
const int    MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const unsigned int VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int VeloOutStreamCount  = 1;   // Number of Velocia Outgoing PIDs
const unsigned int VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

}

Copious_DeviceMap::Copious_DeviceMap() :
       PacketDeviceImpl(new Copious_DeviceDriver, X6DRegAddrs),
       PacketDeviceMap (this, X6RegAddrs, 0),
       IIsAnFmcCarrierDeviceMap(FmcSlotCount),
       IUsesStreamDeviceEventCallbacks(this),
       WB_Periph          (LogicMemory,Copious_MemoryMap::wbPeripheral),
       WB_Packetizer      (LogicMemory,Copious_MemoryMap::wbPacketizer),
       WB_Alerts          (LogicMemory,Copious_MemoryMap::wbAlerts),
       Info_SDev          (this, WB_Periph, PortMemory, CopiousInfoRegAddrs),
       Periph_SDev        (this, WB_Periph, X6PeripheralRegAddrs),
       Velo_SDev          (this, WB_Packetizer, X6PacketizerRegAddrs),
       Alerts_SDev        (this, WB_Alerts, X6AlertsRegAddrs),
       VeloOut_SDev       (this, WB_Periph, X6DepacketizerRegAddrs)
{
    ConstructCustomParts();
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new FmcCHdwPluginImpl<Copious_DeviceMap>(this) );

    BDeviceDriver = dynamic_cast<InnovativeKernel::DualRegionDeviceDriver*>(DeviceDriver);
    //  Enable use of dedicated register to signal Kernel Cache ready
    BDeviceDriver->EnableDedicatedKernelCacheRelease(PortMemory, Copious_MemoryMap::mmKernelCacheAvail);

}

Copious_DeviceMap::~Copious_DeviceMap()
{

}

string Copious_DeviceMap::Name() const
{
    return "COPious";
}

string Copious_DeviceMap::Revision() const
{
    char ch = 'A' + static_cast<char>(Info().PciLogicPcb());
    return string(1, ch);
}

const IX6LogicVersionInfo &Copious_DeviceMap::Info() const
{
    return Info_SDev;
}

unsigned int Copious_DeviceMap::GetVeloInStreamCount()
{
    return VeloInStreamCount;
}

unsigned int Copious_DeviceMap::GetVeloOutStreamCount()
{
    return VeloOutStreamCount;
}

IIsAVitaPacketStreamBoard *Copious_DeviceMap::GetBaseboardPtr()
{

}

unsigned int Copious_DeviceMap::GetPatternModeAlignment()
{

}

unsigned int Copious_DeviceMap::GetPatternModeScaleFactor()
{

}

bool Copious_DeviceMap::GetPatternModeSingleMemoryFlag()
{

}

void Copious_DeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    Velo_SDev.ConstructCustomParts();
    Alerts_SDev.ConstructCustomParts();
    VeloOut_SDev.ConstructCustomParts();

}

void Copious_DeviceMap::OpenHardware()
{
    MasterReset();

    Periph_SDev.AppReset();

    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    Velo_SDev.OpenHardware();
    Alerts_SDev.OpenHardware();
    VeloOut_SDev.OpenHardware();
}

void Copious_DeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    Velo_SDev.PreconfigureHardware();
    Alerts_SDev.PreconfigureHardware();
    VeloOut_SDev.PreconfigureHardware();
}

void Copious_DeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();
    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    Velo_SDev.ConfigureHardware();
    Alerts_SDev.ConfigureHardware();
    VeloOut_SDev.ConfigureHardware();
}

void Copious_DeviceMap::StreamStopHardware()
{

}

void Copious_DeviceMap::CloseHardware()
{

}

bool Copious_DeviceMap::AlertTest(unsigned int pid)
{
    //  return TRUE if PID is an alert PID.
    //  return FALSE if not
    return Alerts_SDev.AlertTest(pid);
}

bool Copious_DeviceMap::AlertCheck(const Buffer &buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alerts_SDev.CheckAndProcess(buffer);
}

void Copious_DeviceMap::DumpIsrStatus(std::vector<int> &vec)
{
    vec.clear();
    for (int i = 0; i < Copious_DeviceDriver::LOG->Count; i++)
        vec.push_back(Copious_DeviceDriver::LOG->Status[i]);
}

}

