#ifndef COPIOUS_DEVICEMAP_H
#define COPIOUS_DEVICEMAP_H

#include <DeviceDriver_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <FmcCarrierBoard_Mb.h>
#include <PmcEeprom_Mb.h>

namespace Innovative {


class Copious_DeviceMap : public PacketDeviceImpl,
                          public PacketDeviceMap,
                          public IIsAnFmcCarrierDeviceMap,
                          public IUsesStreamDeviceEventCallbacks,
                          public ISuppliesFmcCarrierRequirements
{
    enum {FmcSlotCount = 1};

public:
    Copious_DeviceMap();
    virtual ~Copious_DeviceMap();

    // Methods
    std::string     Name() const;
    std::string     Revision() const;

    //  Hardware Device Access
    const IX6LogicVersionInfo &     Info() const;


// implement virtual interfaces
    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();

    virtual IIsAVitaPacketStreamBoard *   GetBaseboardPtr();
    virtual unsigned int                  GetPatternModeAlignment();
    virtual unsigned int                  GetPatternModeScaleFactor();
    virtual bool                          GetPatternModeSingleMemoryFlag();

public: // pseudo-protected

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    virtual void    CloseHardware();
    //
    //  Subdevice Customization - implement board specific pieces
    //
    //  Busmaster Intf functions
    virtual bool    AlertTest(unsigned int pid);
    virtual bool    AlertCheck(const Buffer & buffer);

    void            DumpIsrStatus(std::vector<int>& vec);

protected:
    InnovativeKernel::DualRegionDeviceDriver *  BDeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Packetizer;
    WishboneBusSpace       WB_Alerts;
    //
    //  Subdevices
    X6Info               Info_SDev;
    X6Peripherals        Periph_SDev;
    X6PacketizerCfg      Velo_SDev;
    X6AlertSystem        Alerts_SDev;
    X6DepacketizerCfg    VeloOut_SDev;
    //
    Trace                Debug;
};

}



#endif // COPIOUS_DEVICEMAP_H
