// CountedPtr_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef CountedPtr_MbH
#define CountedPtr_MbH

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

template <class T> class CountedPtr
{
public:
    typedef T element_type;

    explicit CountedPtr(T* p = 0) // allocate a new counter
        : itsCounter(0) {if (p) itsCounter = new counter(p);}
    ~CountedPtr()
    {
        Release();
    }
    
    CountedPtr(const CountedPtr& r) throw()
    {
        acquire(r.itsCounter);
    }
    
    CountedPtr& operator=(const CountedPtr& r)
    {
        if (this != &r)
            {
            Release();
            acquire(r.itsCounter);
            }
            
        return *this;
    }

    T& operator*()  const throw()   { return *itsCounter->ptr;}
    T* operator->() const throw()   { return itsCounter->ptr;}
    T* get()        const throw()   { return itsCounter ? itsCounter->ptr : 0; }
    bool unique()   const throw()
        { return (itsCounter ? itsCounter->count == 1 : true); }

private:

    struct counter
    {
        counter(T* p = 0, unsigned c = 1) :
            ptr(p), count(c)
        {
        }

        T          *ptr;
        unsigned    count;
    } *itsCounter;

    void acquire(counter* c) throw()
    { // increment the count
        itsCounter = c;
        if( c )
          ++ c->count;
    }

    void Release()
    { // decrement the count, delete if it is 0
        if (itsCounter)
            {
            if (--itsCounter->count == 0)
                {
                delete itsCounter->ptr;
                delete itsCounter;
                }
            itsCounter = 0;
            }
            
    }
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative;

#endif
