//---------------------------------------------------------------------------
// DAC3xJ82_SubDevice_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.

#include <DAC3484_SubDevice_Mb.h>
#include <SystemSupport_Mb.h>
#include <cmath>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

namespace DAC3484
{
//===========================================================================
//  CLASS DAC3484_DAC -- JESD A/D Support "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DAC3484_DAC
//---------------------------------------------------------------------------

const unsigned int DacNumConverters = 4;  // constant for DAC3484
const unsigned int DacLaneCount     = 8;

//---------------------------------------------------------------------------
//  constructor for class DAC3484_DAC
//---------------------------------------------------------------------------

DAC3484_DAC::DAC3484_DAC(IRequires_DAC3484_DAC_Io * mdc)
    : MDC(mdc)
{
}

//---------------------------------------------------------------------------
//  destructor for class DAC3484_DAC
//---------------------------------------------------------------------------

DAC3484_DAC::~DAC3484_DAC()
{
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::ConstructCustomParts()
//---------------------------------------------------------------------------

void  DAC3484_DAC::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::OpenHardware()
//---------------------------------------------------------------------------

void  DAC3484_DAC::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::ConfigureHardware()
//---------------------------------------------------------------------------

void  DAC3484_DAC::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    DacInit();
    ConfigureDacPll();
    ConfigureDacDigBlocks();
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::LoadDacClockRates() - load needed clock rates
//---------------------------------------------------------------------------

void DAC3484_DAC::LoadDacClockRates(double dacSamplingRate, double dacEffectiveRate)
{
    Debug.Log("DAC3484_DAC::LoadDacClockRates()");

    DacSamplingRate = dacSamplingRate;
    DacClkRate      = dacEffectiveRate;
    Debug.Log("end DAC3484_DAC::LoadDacClockRates()");
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::DacInit()
//---------------------------------------------------------------------------

void DAC3484_DAC::DacInit()
{
    Debug.Log("DAC3484_DAC::DacInit()");

    Debug.Log("DAC Interpolation Rate = ", InterpolationRate);
    Debug.Log("DAC Number of converters = ", DacNumConverters);

    // Soft reset the DAC
    Spi(0x3,0xF001);
    Sleep(10);

    // Read the Vendor and Version ID register
    Debug.Log("Dac Vendor and Version ID config127 = ", Spi(0x7f), 16);

    Debug.Log("end DAC3484_DAC::DacInit()");
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::ConfigureDacPll()
//---------------------------------------------------------------------------

void DAC3484_DAC::ConfigureDacPll()
{
    Debug.Log("DAC3484_DAC::ConfigureDacPll()");

    DacPllEnabled = (DacSamplingRate == DacClkRate)? false : true;

    Debug.Log("DacSamplingRate(MHz) =", (DacSamplingRate/1.0e6));
    Debug.Log("DacClkRate(MHz) =", (DacClkRate/1.0e6));

    CalcDacPllParams();

    MemoryBitManipulator Data(0);

    // Set config26 register
    Data.Value(Spi(0x1A));
    Data.Field(10, 6, Pll_Vco);
    Data.Bit(0, (((DacCh[1].CouplingMode == ocmAc)? true : false) || !ChannelEnabled(1)));  // set dacd_sleep
    Data.Bit(1, (((DacCh[1].CouplingMode == ocmDc)? true : false) || !ChannelEnabled(1)));  // set dacc_sleep
    Data.Bit(2, (((DacCh[0].CouplingMode == ocmAc)? true : false) || !ChannelEnabled(0)));  // set dacb_sleep
    Data.Bit(3, (((DacCh[0].CouplingMode == ocmDc)? true : false) || !ChannelEnabled(0)));  // set daca_sleep
    Data.Bit(5, !DacPllEnabled);  // set pll_sleep
    Spi(0x1A, Data.Value());

    // Set config24 register
    Data.Value(Spi(0x18));
    Data.Field(3, 3, Pll_P);
    Data.Bit(10, DacPllEnabled);   // set pll_ena
    Data.Bit(12, !DacPllEnabled);  // set pll_reset
    Spi(0x18, Data.Value());

    if (DacPllEnabled)
        {
        // Set config25 register
        Data.Value(Spi(0x19));
        Data.Field(2, 2, Pll_VcoITune);
        Data.Field(4, 4, Pll_N);
        Data.Field(8, 8, Pll_M);
        Spi(0x19, Data.Value());

        // Check the loop filter voltage
        Sleep(1);
        Data.Value(Spi(0x18));
        unsigned int Pll_Lfvolt = Data.Value() & 0x7;

        // Check the PLL lock
        bool DacPllLock = (Data.Value() & 0x7) ? false : true;;
        Debug.Log("DAC PLL loop filter voltage =", Pll_Lfvolt);
        Debug.Log("DAC PLL locked: ", DacPllLock);
        }

    Debug.Log("end DAC3484_DAC::ConfigureDacPll()");
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::CalcDacPllParams()
//---------------------------------------------------------------------------

void DAC3484_DAC::CalcDacPllParams()
{
    Debug.Log("DAC3484_DAC::CalcDacPllParams()");

    if (!DacPllEnabled)
        {
        Pll_N = Pll_M = Pll_P = 0;
        Pll_Vco = 0;
        Pll_VcoITune = 0;
        Debug.Log("Dac PLL disabled");
        return;
        }

    // Calculate the Prescaler value
    const double VcoLmin = 3700.0e6;
    const double VcoLmax = 4660.0e6;
    const double VcoHmin = 4440.0e6;
    const double VcoHmax = 5600.0e6;
    bool         InVcoLRange;
    bool         InVcoHRange;
    double VcoFreq = 5000.0e6;  // Initialize to a high value

    const unsigned int Pll_PAry[] = {2,3,4,5,6,7,8,9,4,6,8,10,12,14,16,18};
    const unsigned int Pll_PArySize = sizeof(Pll_PAry)/sizeof(unsigned int);
    double freq;

    // Initialize
    Pll_P = 0;

    // Loop over the prescaler values to find the max value
    for (unsigned int i=0; i<Pll_PArySize; i++)
        {
        // Calculate the candidate Vco frequency
        freq = Pll_PAry[i] * DacSamplingRate;

        InVcoLRange = false;
        InVcoHRange = false;
        if (freq >= VcoLmin && freq <= VcoLmax)
            InVcoLRange = true;
        if (freq >= VcoHmin && freq <= VcoHmax)
            InVcoHRange = true;

        // Latch the Vco frequency and prescaler index
        if ((InVcoLRange || InVcoHRange) && (Pll_PAry[i]>=Pll_PAry[Pll_P]))
            {
            VcoFreq = freq;
            Pll_P = i;
            }
        }

    // The 2 VCOs overlap over the range of VcoHmin-VcoLmax, calculate the center
    // frequency of this range to use it to select one of the VCOs
    double VcoCenter = (VcoHmin + VcoLmax)/2;

    // Select the 4GHz or 5GHz Vco (true for 4GHz Vco)
    //Pll_VcoSel = (VcoFreq < VcoCenter)? true : false;

    Debug.Log("DAC PLL VcoFreq(MHz) =", VcoFreq/1.0e6);
    Debug.Log("DAC PLL P =", Pll_PAry[Pll_P]);

    double VcoError;
    double MinVcoError = 5000.0;

    // Loop over the Pll_Vco values and find the closest matching value
//    for (unsigned int VcoRangeAdj=0; VcoRangeAdj<64; VcoRangeAdj++)
//        {
//        if (Pll_VcoSel)     // 4GHz Vco
//            VcoError = (0.089703*VcoRangeAdj*VcoRangeAdj)+(8.8312*VcoRangeAdj)+3752.5-(VcoFreq/1.0e6);
//        else                // 5GHz Vco
//            VcoError = (0.10998*VcoRangeAdj*VcoRangeAdj)+(10.574*VcoRangeAdj)+4446.3-(VcoFreq/1.0e6);

//        if (fabs(VcoError) < MinVcoError)
//            {
//            MinVcoError = fabs(VcoError);
//            Pll_Vco = VcoRangeAdj;
//            }
//        }

    Debug.Log("DAC PLL MinVcoError =", MinVcoError);
    Debug.Log("DAC PLL Pll_Vco =", Pll_Vco);

    // Set the core bias current depending on the selected Vco
    //Pll_VcoITune = (Pll_VcoSel)? 2 : 3;

    double PFD;
    const double MaxPFD = 650.0e6;
    bool   PllMFound;

    // Initialize (to be used when no match is found)
    Pll_N = 0;
    Pll_M = 0;

    // Loop over the Pll_N and Pll_M values to find the proper settings
    for (unsigned int N=1; N<=32; N++)
        {
        PllMFound = false;
        PFD = DacClkRate/N;
        if (PFD > MaxPFD)  // skip if PFD is too fast
            continue;
        for (unsigned int M=1; M<=256; M++)
            {
            if (PFD * M == DacSamplingRate)
                {
                Pll_N = N-1;
                Pll_M = M-1;
                PllMFound = true;
                Debug.Log("DAC PLL PFD(MHz) =", PFD/1.0e6);
                Debug.Log("DAC PLL N =", N);
                Debug.Log("DAC PLL M =", M);
                break;
                }
            }
        if (PllMFound)
            break;
        }

    Debug.Log("end DAC3484_DAC::CalcDacPllParams()");
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::ConfigureDacDigBlocks()
//---------------------------------------------------------------------------

void DAC3484_DAC::ConfigureDacDigBlocks()
{
    Debug.Log("DAC3484_DAC::ConfigureDacDigBlocks()");

    MemoryBitManipulator Data(0);

    // Set config1 register
    Data.Value(Spi(0x1));
    Data.Bit(4, DacCh[1].Compliment);
    Data.Bit(5, DacCh[1].Compliment);
    Data.Bit(6, DacCh[0].Compliment);
    Data.Bit(7, DacCh[0].Compliment);
    Spi(0x1, Data.Value());

    // Set config2 register
    Data.Value(0);
    Data.Bit(1, 1);     // Set twos to enable 2s complement data format
    Data.Bit(4, DacEnFNco);
    Data.Bit(5, DacEnMixerGain);
    Data.Bit(6, DacEnMixer);
    Spi(0x2, Data.Value());

    // Set config3 register
    Data.Value(Spi(0x3));
    Data.Field(12, 4, DacCoarseGain);
    Spi(0x3, Data.Value());

    const int QmcOffsetMaxPos = (1<<12)-1; //4095
    const int QmcOffsetMaxNeg = -(1<<12);  //-4096
    // Check the QmcOffset values if within range, saturate if not within range
    DacCh[0].QmcOffset = std::min(QmcOffsetMaxPos,DacCh[0].QmcOffset);
    DacCh[0].QmcOffset = std::max(QmcOffsetMaxNeg,DacCh[0].QmcOffset);
    DacCh[1].QmcOffset = std::min(QmcOffsetMaxPos,DacCh[1].QmcOffset);
    DacCh[1].QmcOffset = std::max(QmcOffsetMaxNeg,DacCh[1].QmcOffset);

    // Set config8 register
    Data.Value(Spi(0x8));
    Data.Field(0, 13, DacCh[0].QmcOffset);
    Spi(0x8, Data.Value());

    // Set config9 register
    Data.Value(Spi(0x9));
    Data.Field(0, 13, DacCh[1].QmcOffset);
    Spi(0x9, Data.Value());

    const unsigned int QmcGainMax = (1<<11)-1; //2047
    // Check the QmcGain values if within range, saturate if not within range
    DacCh[0].QmcGain = std::min(QmcGainMax,DacCh[0].QmcGain);
    DacCh[1].QmcGain = std::min(QmcGainMax,DacCh[1].QmcGain);

    // Set config12 register
    Data.Value(Spi(0xC));
    Data.Field(0, 11, DacCh[0].QmcGain);
    Spi(0xC, Data.Value());

    // Set config13 register
    Data.Value(Spi(0xD));
    Data.Field(0, 11, DacCh[1].QmcGain);
    Data.Field(12, 4, DacCMixSelect);
    Spi(0xD, Data.Value());

    const int QmcPhaseMaxPos = (1<<11)-1; //2047
    const int QmcPhaseMaxNeg = -(1<<11);  //-2048
    // Check the DacQmcPhase value if within range, saturate if not within range
    DacQmcPhase = std::min(QmcPhaseMaxPos,DacQmcPhase);
    DacQmcPhase = std::max(QmcPhaseMaxNeg,DacQmcPhase);

    // Set config16 register
    Data.Value(Spi(0x10));
    Data.Field(0, 12, DacQmcPhase);
    Spi(0x10, Data.Value());

    const unsigned int DacFNcoPhaseOffsetMax = (1<<16)-1; //65535;
    // Check the DacFNcoPhaseOffset value if within range, saturate if not within range
    DacFNcoPhaseOffset = std::min(DacFNcoPhaseOffsetMax,DacFNcoPhaseOffset);

    // Set config18 register
    Spi(0x12, DacFNcoPhaseOffset);

    const s_ii64 DacFNcoPhaseIncMaxPos = ((static_cast<s_ii64>(1)) <<47)-1;
    const s_ii64 DacFNcoPhaseIncMaxNeg = -((static_cast<s_ii64>(1)) <<47);
    DacFNcoPhaseInc = std::min(DacFNcoPhaseIncMaxPos,DacFNcoPhaseInc);
    DacFNcoPhaseInc = std::max(DacFNcoPhaseIncMaxNeg,DacFNcoPhaseInc);

    // Set config20,21,22 registers
    Spi(0x14, (DacFNcoPhaseInc & 0xFFFF));
    Spi(0x15, ((DacFNcoPhaseInc>>16) & 0xFFFF));
    Spi(0x16, ((DacFNcoPhaseInc>>32) & 0xFFFF));

    // Set config34 register
    // (output assignment specific to FMC1000)
    // Outputs A&C are AC coupled, B&D are DC coupled
    Data.Value(0);
    Data.Field(0,  2, ((DacCh[1].CouplingMode == ocmDc)? 1:3));  // Set pathd_out_sel
    Data.Field(2,  2, ((DacCh[1].CouplingMode == ocmAc)? 1:3));  // Set pathc_out_sel
    Data.Field(4,  2, ((DacCh[0].CouplingMode == ocmDc)? 0:2));  // Set pathb_out_sel
    Data.Field(6,  2, ((DacCh[0].CouplingMode == ocmAc)? 0:2));  // Set patha_out_sel
    Data.Field(8,  2, 3);                     // Set pathd_in_sel
    Data.Field(10, 2, 2);                     // Set pathc_in_sel
    Data.Field(12, 2, 1);                     // Set pathb_in_sel
    Data.Field(14, 2, 0);                     // Set patha_in_sel
    Spi(0x22, Data.Value());

    // Set config38 register
//    Data.Value(Spi(0x26));
//    Data.Bit(0, DacEnZeroDither);
//    Data.Field(4, 4, DacDitherAmount);
//    Data.Bit(8, DacCh[0].EnDitherMixer);
//    Data.Bit(9, DacCh[1].EnDitherMixer);
//    Data.Bit(12, DacCh[0].EnDither);
//    Data.Bit(13, DacCh[1].EnDither);
//    Spi(0x26, Data.Value());

    // Set config30 register
    Spi(0x1E, 0x9999);

    // Set config31 register
    Spi(0x1F, 0x9940);

    // Set config32 register
    //Spi(0x20, 0x8000);

    // Toggle the sif_sync
    Spi(0x1F, 0x9942);
    Spi(0x1F, 0x9940);

    Debug.Log("end DAC3484_DAC::ConfigureDacDigBlocks()");
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::CalcDacClkFrequency() -- Calculate the DAC clock frequency
//---------------------------------------------------------------------------

double DAC3484_DAC::CalcDacClkFrequency(double SamplingRate)
{
    double MaxfDac = 1250.0e6;
    double DacClkFrequency = (SamplingRate/InterpolationRate);

    while (DacClkFrequency > MaxfDac)
        DacClkFrequency /=2;

    return DacClkFrequency;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacInterpolationRate() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacInterpolationRate(const unsigned int value)
{
    DacInterpolationRate = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnInvSinc() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnInvSinc(const bool value)
{
    DacEnInvSinc = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnFNco() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnFNco(const bool value)
{
    DacEnFNco = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnMixerGain() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnMixerGain(const bool value)
{
    DacEnMixerGain = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnMixer() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnMixer(const bool value)
{
    DacEnMixer = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacCoarseGain() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacCoarseGain(const unsigned int value)
{
    DacCoarseGain = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacCMixSelect() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacCMixSelect(const unsigned int value)
{
    DacCMixSelect = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnQmc() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnQmc(const bool value)
{
    DacEnQmc = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacQmcPhase() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacQmcPhase(const int value)
{
    DacQmcPhase = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacFNcoPhaseOffset() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacFNcoPhaseOffset(const unsigned int value)
{
    DacFNcoPhaseOffset = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacFNcoPhaseInc() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacFNcoPhaseInc(const s_ii64 value)
{
    DacFNcoPhaseInc = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnZeroDither() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnZeroDither(const bool value)
{
    DacEnZeroDither = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacDitherAmount() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacDitherAmount(const unsigned int value)
{
    DacDitherAmount = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacOutputCouplingMode() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacOutputCouplingMode(const unsigned int value,const unsigned int DacChIndex)
{
    DacCh[DacChIndex].CouplingMode = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacCompliment() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacCompliment(const bool value,const unsigned int DacChIndex)
{
    DacCh[DacChIndex].Compliment = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacQmcOffset() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacQmcOffset(const int value,const unsigned int DacChIndex)
{
    DacCh[DacChIndex].QmcOffset = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacQmcGain() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacQmcGain(const unsigned int value,const unsigned int DacChIndex)
{
    DacCh[DacChIndex].QmcGain = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnDither() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnDither(const bool value,const unsigned int DacChIndex)
{
    DacCh[DacChIndex].EnDither = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::SetDacEnDitherMixer() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::SetDacEnDitherMixer(const bool value,const unsigned int DacChIndex)
{
    DacCh[DacChIndex].EnDitherMixer = value;
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::Write_I_Offset() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::Write_I_Offset(unsigned int value)
{
    //not done yet
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::Write_Q_Offset() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::Write_Q_Offset(unsigned int value)
{
    //not done yet
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::Write_I_Gain() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::Write_I_Gain(unsigned int value)
{
    //not done yet
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::Write_Q_Gain() --
//---------------------------------------------------------------------------

void  DAC3484_DAC::Write_Q_Gain(unsigned int value)
{
    //not done yet
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  DAC3484_DAC::Spi(unsigned int address, unsigned int data)
{
    MDC->DacSpi(address, data);
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  DAC3484_DAC::Spi(unsigned int address)
{
    return MDC->DacSpi(address);
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::ChannelEnabled() -- read cached channel enable state
//---------------------------------------------------------------------------

bool  DAC3484_DAC::ChannelEnabled(int ch)
{
    return MDC->DacChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::ChannelEnabled() -- set channel enable state
//---------------------------------------------------------------------------

void DAC3484_DAC::ChannelEnabled(int ch, bool state)
{
    return MDC->DacChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  DAC3484_DAC::log2()  -- Calculate Log2
//---------------------------------------------------------------------------

unsigned int DAC3484_DAC::log2(unsigned int value)
{
    unsigned int log = 0;
    while (value != 0)
        {
        value >>= 1;
        log++;
        }
    return log-1;
}

}  // namespace DAC3484

}  // namespace Innovative



