// DAC3xJ82_SubDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef DAC3484_SubDevice_MbH
#define DAC3484_SubDevice_MbH

#include <X6Support_Mb.h>
#include <JESDSubDevice_Stream_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IRequires_DAC3484_DAC_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_DAC3484_DAC_Io
{
public:

    // Io Interface
    virtual void         DacSpi(unsigned int address, unsigned int data) = 0;
    virtual unsigned int DacSpi(unsigned int address) = 0;

    virtual bool DacChannelEnabled(int ch) = 0;
    virtual void DacChannelEnabled(int ch, bool state) = 0;

    // Customization Interface
    virtual ~IRequires_DAC3484_DAC_Io()
        {}
};

namespace DAC3484
{
//===========================================================================
//  CLASS IDAC3484_DAC_Loader -- Information and Parameter Loading for DAC3484
//===========================================================================

class IDAC3484_DAC_Loader
{
public:

    virtual ~IDAC3484_DAC_Loader()
        {}

    virtual void SetDacInterpolationRate(const unsigned int value) = 0;
    virtual void SetDacEnInvSinc(const bool value) = 0;
    virtual void SetDacEnFNco(const bool value) = 0;
    virtual void SetDacEnMixerGain(const bool value) = 0;
    virtual void SetDacEnMixer(const bool value) = 0;
    virtual void SetDacCoarseGain(const unsigned int value) = 0;
    virtual void SetDacCMixSelect(const unsigned int value) = 0;
    virtual void SetDacEnQmc(const bool value) = 0;
    virtual void SetDacQmcPhase(const int value) = 0;
    virtual void SetDacFNcoPhaseOffset(const unsigned int value) = 0;
    virtual void SetDacFNcoPhaseInc(const s_ii64 value) = 0;
    virtual void SetDacEnZeroDither(const bool value) = 0;
    virtual void SetDacDitherAmount(const unsigned int value) = 0;
    virtual void SetDacOutputCouplingMode(const unsigned int value,const unsigned int DacChIndex) = 0;
    virtual void SetDacCompliment(const bool value,const unsigned int DacChIndex) = 0;
    virtual void SetDacQmcOffset(const int value,const unsigned int DacChIndex) = 0;
    virtual void SetDacQmcGain(const unsigned int value,const unsigned int DacChIndex) = 0;
    virtual void SetDacEnDither(const bool value,const unsigned int DacChIndex) = 0;
    virtual void SetDacEnDitherMixer(const bool value,const unsigned int DacChIndex) = 0;
};

//===========================================================================
//  CLASS DAC3484_DAC -- D/A Support "SubDevice"
//===========================================================================

class DAC3484_DAC : public IX6SubDevice, public IDAC3484_DAC_Loader
{
public:
    DAC3484_DAC(IRequires_DAC3484_DAC_Io * owner);
    virtual ~DAC3484_DAC();

    //
    //  Methods
    void         Spi(unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int address);

    void         ChannelEnabled(int ch, bool state);
    bool         ChannelEnabled(int ch);

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void ConfigureHardware();

    void LoadDacClockRates(double dacSamplingRate, double dacEffectiveRate);

    double CalcDacClkFrequency(double SamplingRate);

    // DAC
    virtual void SetDacInterpolationRate(const unsigned int value);
    virtual void SetDacEnInvSinc(const bool value);
    virtual void SetDacEnFNco(const bool value);
    virtual void SetDacEnMixerGain(const bool value);
    virtual void SetDacEnMixer(const bool value);
    virtual void SetDacCoarseGain(const unsigned int value);
    virtual void SetDacCMixSelect(const unsigned int value);
    virtual void SetDacEnQmc(const bool value);
    virtual void SetDacQmcPhase(const int value);
    virtual void SetDacFNcoPhaseOffset(const unsigned int value);
    virtual void SetDacFNcoPhaseInc(const s_ii64 value);
    virtual void SetDacEnZeroDither(const bool value);
    virtual void SetDacDitherAmount(const unsigned int value);
    virtual void SetDacOutputCouplingMode(const unsigned int value,const unsigned int DacChIndex);
    virtual void SetDacCompliment(const bool value,const unsigned int DacChIndex);
    virtual void SetDacQmcOffset(const int value,const unsigned int DacChIndex);
    virtual void SetDacQmcGain(const unsigned int value,const unsigned int DacChIndex);
    virtual void SetDacEnDither(const bool value,const unsigned int DacChIndex);
    virtual void SetDacEnDitherMixer(const bool value,const unsigned int DacChIndex);

    virtual void Write_I_Offset(unsigned int value);
    virtual void Write_Q_Offset(unsigned int value);

    virtual void Write_I_Gain(unsigned int value);
    virtual void Write_Q_Gain(unsigned int value);

private:
    IRequires_DAC3484_DAC_Io *  MDC;
    const unsigned int          DeviceID;   // ID for proper JESD DAC Device ID

    void         DacInit();
    void         ConfigureDacPll();
    void         CalcDacPllParams();
    void         ConfigureDacDigBlocks();
    unsigned int log2(unsigned int value);

    enum DacSerDesLaneRates { lrFull, lrHalf, lrQuarter, lrEighth };
    enum DacOutputCouplingModes { ocmDc, ocmAc };

    // DAC
    unsigned int DacInterpolationRate;
    bool         DacEnInvSinc;
    bool         DacEnFNco;
    bool         DacEnMixerGain;
    bool         DacEnMixer;
    unsigned int DacCoarseGain;
    unsigned int DacCMixSelect;
    bool         DacEnQmc;
    int          DacQmcPhase;
    unsigned int DacFNcoPhaseOffset;
    s_ii64       DacFNcoPhaseInc;
    bool         DacEnZeroDither;
    unsigned int DacDitherAmount;

    struct DacChSettings
    {
        unsigned int CouplingMode;
        bool         Compliment;
        int          QmcOffset;
        unsigned int QmcGain;
        bool         EnDither;
        bool         EnDitherMixer;
    };
    DacChSettings DacCh[2];

    unsigned int InterpolationRate;
    unsigned int NumLanes;
    double       LineRate;
    double       DacSamplingRate;
    double       DacClkRate;
    bool         DacPllEnabled;
    unsigned int Pll_N;
    unsigned int Pll_M;
    unsigned int Pll_P;
    unsigned int Pll_Vco;
    unsigned int Pll_VcoITune;
    unsigned int DacSerDesLaneRate;
    bool         SerDesPllVRange;
    unsigned int SerDesPllRefClkDiv;
    unsigned int SerDesPllMpy;
};

}   // end namespace DAC3484

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
