// DataLogger_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef DataLogger_MbH
#define DataLogger_MbH

#include <DataTypes_Mb.h>
#include <Events_Mb.h>
#include <string>
#include <ProgressEvent_Mb.h>
#include <Buffer_Mb.h>
#include <ProcessEvents_Mb.h>
#include <DriveIo_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS DataLogger  --  
//=============================================================================

class DataLogger
{
public:
    // ctor
    DataLogger();
    DataLogger(const DataLogger & other);
    ~DataLogger();

    // Assignment
    DataLogger & operator = (const DataLogger & other);

    // Methods
	bool        Start();
    bool        Stop();

	bool        Log(const Buffer & buffer);
	bool        Log(const int * data, unsigned int size_in_ints);
	bool        LogWithHeader(const Buffer & buffer);
	ii64        Logged() const;

	// Properties
	void        Ceiling(const ii64 & value);
	ii64        Ceiling() const;

	void        FileName(const std::string & value);
	std::string FileName() const;

	void        Enabled(bool state);
	bool        Enabled() const;

	void        SoftCeiling(bool state);
	bool        SoftCeiling() const;

	void        WriteThrough(bool state);
	bool        WriteThrough() const;

	void        CoalesceWrites(bool state);
	bool        CoalesceWrites() const;

	// Events
	OpenWire::EventHandler<ProgressEvent>           OnProgress;
	OpenWire::EventHandler<StartBufferEvent>        OnStart;
	OpenWire::EventHandler<OpenWire::NotifyEvent>   OnStop;

protected:
	// Fields
    ii64                FLogged;
    ii64                FCeiling;
	bool                FEnabled;
	bool                FSoftCeiling;
	bool				FCoalesceWrites;
	size_t				Residual;
	Buffer				Cache;

	// Data
	DriveIo             mFile;

	// Methods
	bool LogCoalesced(const Buffer & buffer);
	bool LogUncoalesced(const Buffer & buffer);
	bool DoLogging(const void *data, unsigned int bytes);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
