// DataPlayer_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DataPlayer_Mb.h>
#include <BufferHeader_Mb.h>
#include <cstring>

using namespace std;

namespace Innovative
{

//=============================================================================
//  CLASS DataPlayer  --  
//=============================================================================
#ifdef _MSC_VER		
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
//---------------------------------------------------------------------------
// DataPlayer::DataPlayer() --
//---------------------------------------------------------------------------

DataPlayer::DataPlayer()
	: FMode(pmStop),
	  FEnabled(true),
	  FPlayed(0), CurrentPercent(0)
{
}

//---------------------------------------------------------------------------
// DataPlayer::DataPlayer() --
//---------------------------------------------------------------------------

DataPlayer::DataPlayer(const DataPlayer & other)
    : FMode(other.FMode),
	  FEnabled(other.FEnabled),
	  FPlayed(other.FPlayed), CurrentPercent(0),
      mFile(other.mFile)
{
	OnProgress = other.OnProgress;
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// DataPlayer::~DataPlayer() --
//---------------------------------------------------------------------------

DataPlayer::~DataPlayer()
{
	Stop();
}

//---------------------------------------------------------------------------
//  DataPlayer::operator = --
//---------------------------------------------------------------------------

DataPlayer & DataPlayer::operator = (const DataPlayer & other)
{
    // protect against invalid self-assignment
    if (this != &other)
        {
        FMode = other.FMode;
        FEnabled = other.FEnabled;
        FPlayed = other.FPlayed;
        mFile = other.mFile;

        OnProgress = other.OnProgress;
        }

	return *this;
}

//---------------------------------------------------------------------------
//  DataPlayer::SetEnabled() --
//---------------------------------------------------------------------------

void DataPlayer::Enabled(bool value)
{
    FEnabled = value;
}

//---------------------------------------------------------------------------
//  DataPlayer::Enabled() --
//---------------------------------------------------------------------------

bool DataPlayer::Enabled() const
{
	return FEnabled;
}

//---------------------------------------------------------------------------
//  DataPlayer::Start() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4996)
#endif
bool DataPlayer::Start()
{
	if (mFile.Opened() || !mFile.FileName().length())
		return false;

    FPlayed = 0;
    CurrentPercent = 0;

    try
        {
        if (!mFile.Open())
            return false;

        ii64 mLength = mFile.Size();
        mFile.Position(0);

        StartBufferEvent event(mLength);
        OnStart.Execute(event);

        ProgressEvent e(0);
        OnProgress.Execute(e);

        return true;
        }
    catch (...)
        {
        //  catch any exception - we remain unopened
        return false;
        }

}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  DataPlayer::Stop() --
//---------------------------------------------------------------------------

bool DataPlayer::Stop()
{
	if (!mFile.Opened())
		return false;

	mFile.Close();

    OpenWire::NotifyEvent event;
    OnStop.Execute(event);

    return true;
}

//---------------------------------------------------------------------------
//  DataPlayer::Play() --
//---------------------------------------------------------------------------

unsigned int DataPlayer::Play(int * data, unsigned int size_in_ints)
{
	if (!mFile.Opened() || !size_in_ints)
		return 0;

    unsigned int size_in_bytes = size_in_ints*sizeof(int);
    char * dst = reinterpret_cast<char *>(data);

    while (size_in_bytes)
        {
		unsigned int read = mFile.Read(dst, size_in_bytes);

        bool eof = read != size_in_bytes;
		FPlayed += read;
		size_in_bytes -= read;
		dst += read;

        //  Progress Message
        double percent = FPlayed * 100.0 / mFile.Size();
        int Percent = static_cast<int>(percent);

        if (CurrentPercent != Percent)
            {
            ProgressEvent e(Percent);
            OnProgress.Execute(e);
            CurrentPercent = Percent;
            }

		// Handle EOF condition
		if (eof)
			{
			switch (FMode)
				{
				case pmStop:
                    Stop();
					return (size_in_ints*sizeof(int)-size_in_bytes)/sizeof(int);

				case pmRepeat:
					mFile.Position(0);
                    FPlayed = 0;
					break;

				case pmZero:
					memset(dst, 0, size_in_bytes);
					return size_in_ints;
				}
			}
            
		}

	return size_in_ints;
}

//---------------------------------------------------------------------------
//  DataPlayer::Play() -- Play into MemoryBuffer
//---------------------------------------------------------------------------

unsigned int DataPlayer::Play(Buffer & buffer)
{
    buffer.MakeUnique();
    return Play(buffer.Data(), static_cast<unsigned int>(buffer.SizeInInts()));
}

//---------------------------------------------------------------------------
//  DataPlayer::Play() -- Play into a Datagrammable memory area
//---------------------------------------------------------------------------

unsigned int DataPlayer::Play(IDatagrammable & dg_object)
{
    dg_object.MakeWritable();
    return Play(dg_object.DatagramBasePtr(), static_cast<unsigned int>(dg_object.DatagramSize()));
}

//---------------------------------------------------------------------------
//  DataPlayer::PlayWithHeader() -- Play into PmcBuffer (Headers in file)
//---------------------------------------------------------------------------

unsigned int DataPlayer::PlayWithHeader(Buffer & buffer)
{
    buffer.MakeUniqueHeader();
    Play(buffer.Header(), static_cast<unsigned int>(buffer.HeaderSizeInInts()));
    buffer.MakeUnique();

    //  Use virtual "peek header" method to get packet data size
    size_t data_size = buffer.ReadHeaderField_DataSize();
    buffer.Resize(data_size);
    unsigned int rtnSize = Play(buffer);

    //  If we have a trailer, fill that too.
    unsigned int TrailerSize = static_cast<unsigned int>(buffer.TrailerSizeInInts());
    if (TrailerSize)
        {
        buffer.MakeUniqueTrailer();
        Play(buffer.Trailer(), TrailerSize);
        }
    return rtnSize;
}

//---------------------------------------------------------------------------
//  DataPlayer::Played() --
//---------------------------------------------------------------------------

ii64 DataPlayer::Played() const
{
	return FPlayed;
}

//---------------------------------------------------------------------------
//  DataPlayer::Total() -- Total bytes in File
//---------------------------------------------------------------------------

ii64 DataPlayer::Total() const
{
	return mFile.Size();;
}

//---------------------------------------------------------------------------
//  DataPlayer::FileName() --
//---------------------------------------------------------------------------

void DataPlayer::FileName(const string & value)
{
	mFile.FileName(value);
}

//---------------------------------------------------------------------------
//  DataPlayer::FileName() --
//---------------------------------------------------------------------------

string DataPlayer::FileName() const
{
	return mFile.FileName();
}

//---------------------------------------------------------------------------
//  DataPlayer::Mode() --
//---------------------------------------------------------------------------

void DataPlayer::Mode(IIMode value)
{
	FMode = value;
}

//---------------------------------------------------------------------------
//  DataPlayer::Mode() --
//---------------------------------------------------------------------------

DataPlayer::IIMode DataPlayer::Mode() const
{
	return FMode;
}

} // namespace Innovative



