// DataSetBase_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

//---------------------------------------------------------------------------
#ifndef DataSetBase_MbH
#define DataSetBase_MbH
//---------------------------------------------------------------------------

#include <Sample_Mb.h>
#include <DataTypes_Mb.h>
#include <Events_Mb.h>
#include <Trace_Mb.h>
#include <string>
#include <vector>
#include <ProcessEvents_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class IniFile;

//===========================================================================
// CLASS ParseProgressEvent  --  Fractional completion indicator
//===========================================================================

class ParseProgressEvent : public OpenWire::Event
{
public:
    const int Percent;
	bool Abort;

    ParseProgressEvent(int percent, bool abort=false)
        :  Percent(percent), Abort(abort)
        {
        }
};

//===========================================================================
// Class DatasetIncident - class for storing Graph Annotations and Alerts
//===========================================================================

class DatasetIncident
{
public:
	enum PacketType { Alert, Velocia, Velo, Vita };
	typedef std::vector<int> IntArray;

	// Ctor
	DatasetIncident(int id=0, ii32 idx=0)
		: Index(0), Id(id), Size(0), Argument(0), Seconds(0), Subseconds(0),
		  Origin(0), Sample(idx), Padding(0),
		  RangedIncident(false), Type(Velocia)
		{  }
	virtual ~DatasetIncident()
		{  }

	ii64                Index;
	int             	Id;
	int             	Size;
	int 				Argument;
	int         		Seconds;
	ii64        		Subseconds;
	ii64                Offset;
	ii64        		Origin;
	ii64        		Sample;
	ii64                Padding;
	bool        		RangedIncident;
	IntArray			Raw;
	PacketType          Type;
	virtual std::string Format() const;
	virtual std::string Description() const;
	virtual std::string Label() const;

	bool operator ==(const DatasetIncident & rhs) const;
	bool operator !=(const DatasetIncident & rhs) const;
	bool operator < (const DatasetIncident & rhs) const;
};

typedef std::vector<DatasetIncident>  DatasetIncidentList;

//===========================================================================
// Class DataSet  --  Base class for sampled data sets
//===========================================================================

class DataSet
{
public:
    enum IIDataType { Int, Float, TiFloat, ALaw, MuLaw };
    enum IIPolarity { Unsigned, Signed };

    // Ctor
                        DataSet();
    virtual 			~DataSet();
    void                Seek(ii64 sample);
    void                Flush();
    void                ReadIniFile(const std::string & value);
    void                WriteIniFile(const std::string & value);
	virtual void        FillIncidentList(ii64 /*StartIdx*/, ii64 /*EndIdx*/)
		{  }
	void        		SortIncidentList();

	//  Scans to end of the file, parsing packets to the end.
	void  ScanToEnd()
		{
		Sample dummy;
		dummy = Element( Samples() );
		}

	// Properties
	void				CurrentSample(const Sample & value);
	Sample		        CurrentSample();

    void				Element(ii64 index, const Sample & value);
    Sample		        Element(ii64 index);

    unsigned int 		Mask() const
        {  return FMask;  }

    void 				Size(ii64 value);
    ii64 				Size() const;

    ii64 				SizeInEvents() const
        {  return Size()/EventSize();  }

    ii64 				Samples() const;

    void 				ReadOnly(bool value)
        {  FReadOnly=value;  }
    bool				ReadOnly() const
        {  return FReadOnly;  };

    void				BigEndian(bool value)
        {  FBigEndian=value;  }
    bool				BigEndian() const
        {  return FBigEndian;  }

    void				ScalingEnabled(bool value)
        {  FScalingEnabled = value;  }
    bool				ScalingEnabled() const
        {  return FScalingEnabled;  }

    void				Enabled(bool value);
    bool  				Enabled() const;

    void				Header(int value);
    int 				Header() const
        {  return FHeader;  }

    void				Trailer(int value);
    int 				Trailer() const
        {  return FTrailer;  }

    void				EventHeader(int value);
    int 				EventHeader() const
        {  return FEventHeader;  }

    void				EventTrailer(int value);
    int 				EventTrailer() const
        {  return FEventTrailer;  }

    void				Channels(int value);
    int 				Channels() const
		{  return FChannels;  }

	void				Devices(int value);
    int 				Devices() const
        {  return FDevices;  }

    void				Channel(int value);
    int 				Channel() const
        {  return FChannel;  }

	void				Polarity(IIPolarity value);
	IIPolarity			Polarity() const
        {  return FPolarity;  }

    void				Type(IIDataType value);
    IIDataType			Type() const
        {  return FType;  }

    void				Offset(float value);
    float				Offset() const
        {  return FOffset;  }

    void				Bits(int value);
    int 				Bits() const
        {  return FBits;  }

        void			SubsampleSize(int /*value*/)
        {  }
    int 				SubsampleSize() const
        {  return FSubsampleSize;  }

        void			SampleSize(int /*value*/)
        {  }
    int 				SampleSize() const
        {  return FSampleSize;  }

        void			EventSize(int /*value*/)
        {  }
    int 				EventSize() const
        {  return FEventSize;  }

    void				ScaleFactor(float value);
    float				ScaleFactor() const
        {  return FScaleFactor;  }

    void				NullDataValue(double value)
        {  FNullDataValue = value;  }
    double 				NullDataValue() const
        {  return FNullDataValue;  }

    void				SampleRate(float value)
        {  FSampleRate = value;  }
    float 				SampleRate() const
        {  return FSampleRate;  }

    void				FileName(std::string value)
        {  SetFileName(value);	}
    std::string			FileName() const
        {  return FFileName;  }

    void				Complex(bool state);
    bool			    Complex() const
        {  return FComplex;  }

    unsigned int        Subsamples() const;
	unsigned int        Subchannels() const;

	bool                Unipolar() const
		{  return FPolarity == Unsigned;  }
	bool                Progress(ii64 sample_idx);

	void                Abort(bool state)
		{  FAbort = state;  }
	bool                Abort() const
		{  return FAbort;  }

	// Data
	DatasetIncidentList  Incidents;

    // Events
	OpenWire::EventHandler<ParseProgressEvent> OnProgress;

protected:
	void                Init();
	void                CorrectEndian(char *input);

	// Override Methods
	virtual bool        ImageOpen() = 0;
	virtual void        ImageClose() = 0;
	virtual void        ImageInit() = 0;
	virtual void        PreAccess()  {  }
	virtual void        PostAccess()  {  }
	virtual void        ImageSampleFetch(Subsample & sample) = 0;
	virtual void        ImageSampleStore(Subsample & sample) = 0;
	virtual void        ImageSeek(ii64 offset) = 0;
	virtual ii64     	ImageSize() const = 0;
	virtual void        ImageSize(ii64 size) = 0;
	virtual void        ImageFlush() = 0;
	virtual void        ImageReadIniFile(IniFile & /*file*/)    {  }
	virtual void        ImageWriteIniFile(IniFile & /*file*/)   {  }

	// Fields
	bool                FEnabled;           // Data accessible if true
	bool                FScalingEnabled;    // Enable scale factors if true
	int                 FHeader;            // # bytes before data in image
	int                 FTrailer;           // # bytes after data in image
	int                 FEventHeader;       // # bytes before data in event
    int                 FEventTrailer;      // # bytes after data in event
    int                 FChannels;          // # channels per event
    int                 FDevices;           // # devices per event
    int                 FChannel;           // Primary event channel
    IIDataType          FType;              // char, int, float, TI flt
    int                 FBits;              // # Active bits/sample
    float               FOffset;            // Offset correction factor
    float               FScaleFactor;       // I/O Scalefactor
    IIPolarity        	FPolarity;          // Polarity of dataset
    int                 FSampleSize;        // Length of sample, in bytes
    int                 FSubsampleSize;     // Length of subsample, in bytes
    int                 FEventSize;         // # bytes per event
    unsigned int        FMask;              // Used bits/sample
    OpenWire::EventHandler<OpenWire::NotifyEvent>	Dependent;
    bool                FBigEndian;         // Big endian data format
    double              FNullDataValue;     // Value returned on FP error
    bool                FReadOnly;          // Open for read-only access
    float               FSampleRate;
    std::string         FFileName;
    bool                FComplex;           // Complex IQ data set channel data
    Trace				Debug;
	bool                FAbort;

    // Setter/Getters
    virtual void 		SetFileName(const std::string & name);

private:
    // Data
    int             Shift;              // # fallow bits/sample

    void          	Nop(int /*value*/) { };
    void          	Nop(ii64 /*value*/) { };
    void          	SetScaleFactor(float scalefactor);
    void          	SetOffset(float offset);
    void          	SetType(IIDataType type);
    void          	SetPolarity(IIPolarity polarity);
    void          	SetEnabled(bool state);
    bool          	GetEnabled();
    void          	SetBits(int bits);
    void          	SetHeader(int bytes);
    void          	SetTrailer(int bytes);
    void          	SetEventHeader(int bytes);
    void          	SetEventTrailer(int bytes);
    void          	SetChannels(int channels);
    void          	SetChannel(int channel);
    void          	SetDevices(int devices);
    Sample          GetElement(ii64 index);
    void          	SetElement(ii64 index, Sample value);
    void          	SetSample(Sample value);
    Sample          GetSample();

    void      		SetSize(ii64 size);
    ii64 			GetSize() const;
};

//===========================================================================
// Class EventStreamDataSet  --  Base class for sampled data sets
//===========================================================================
//   An event stream is one where internal format is a continuous array
//      of single formatted events. One sample of each channel per event

class EventStreamDataSet : public DataSet
{
public:
    enum DataType { Int, Float, TiFloat, ALaw, MuLaw };
    enum PolarityType { Unsigned, Signed };

    // Ctor
                        EventStreamDataSet();
    virtual 			~EventStreamDataSet();

protected:

    // Overridden Methods
    virtual void        ImageInit();
    virtual void        ImageSampleFetch(Subsample & sample);
    virtual void        ImageSampleStore(Subsample & sample);
    virtual void        PreAccess();
    virtual void        PostAccess();

    // Override Methods
    virtual void        ImageFetch(Subsample & sample) = 0;
    virtual void        ImageStore(Subsample & sample) = 0;
    virtual void        ImageSkip(ii64 offset) = 0;

    // Fields
private:
    // Data
    int                 HeaderSkip;
    int                 TrailerSkip;

    void                Skip(int bytes);

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative


#endif
