//---------------------------------------------------------------------------
// Date_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "Date_Mb.h"

#include <Exception_Mb.h>
#include <cstdio>
#include <cstring>

using namespace std;
namespace Innovative
{

//=============================================================================
//  CLASS Date  --  Date management class
//=============================================================================

//--------------------- Class Varaibles ------------------------------------------------

int				Date::DisplayFormat		= Date::MDY;
unsigned int	Date::DisplayOptions	= 0;
unsigned short	Date::DefaultCentury    = 1900;
unsigned int   	Date::startDST=((Date::April<<8) + 1);
unsigned int   	Date::startSTD=((Date::October<<8) + 31);


static const char *dayname[]= {"Sunday", "Monday", "Tuesday",
    "Wednesday", "Thursday", "Friday", "Saturday"} ;

static const char *mname[] = {"January", "February", "March", "April",
    "May", "June", "July", "August", "September", "October",
    "November", "December"};

static const char	szBCE[]				= " B.C.E.";

static char GauDays[]	= { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

static int	DaysSoFar[][13] =
			{
			{0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365},
			{0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366}
			};

//---------------------------------------------------------------------------
//  Date::InvalidDay() --
//---------------------------------------------------------------------------

const char * Date::InvalidDay()
{
    return "invalid day";
}

//---------------------------------------------------------------------------
//  Date::InvalidDate() --
//---------------------------------------------------------------------------

const char * Date::InvalidDate()
{
    return "invalid date";
}

//---------------------------------------------------------------------------
//  Date::InvalidMonth() --
//---------------------------------------------------------------------------

const char * Date::InvalidMonth()
{
    return "invalid month";
}

//---------------------------------------------------------------------------
//  Date::Months() --
//---------------------------------------------------------------------------

const char * Date::Months(int idx)
{
    return mname[idx-1];
}

//---------------------------------------------------------------------------
//  Date::DayName() --
//---------------------------------------------------------------------------

const char * Date::DayName(int idx)
{
    return dayname[idx-1];
}


//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date()
{
	month 	= NonMonth;
	day 	= 0;
	year 	= 0;
	julian 	= 0L;

	day_of_week = NonDay;
}


//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date (long j)
    : julian(j)
{
    julian_to_mdy();
}


//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date (short m, short d, short y)
    : month(m), day(d), year(y)
{
	if ((month > 0) == 0)
        throw MalibuException("Invalid month");
	if ((month < 13) == 0)
        throw MalibuException("Invalid month");

	if ((day > 0) == 0)
        throw MalibuException("Invalid day");
	if ((day < 32) == 0)
        throw MalibuException("Invalid day");

	mdy_to_julian();
}

//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date (const tm &TM)
{
    month	= (TM.tm_mon + 1);
    day		= TM.tm_mday;
    year	= (TM.tm_year + 1900);

    mdy_to_julian();
}

//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date (const Date &dt)
{
    month = dt.month;
    day   = dt.day;
    year  = dt.year;

    mdy_to_julian();
}

//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date (int weeknum, int dow, short m, short y)
    : month(m), day(1), year(y)
{
    int	d;

    // weeknum = 0 mean "the last"
    if ((weeknum > -1) == 0)
        throw MalibuException("Invalid week number");
    if ((weeknum < 6) == 0)
        throw MalibuException("Invalid week number");

    if ((dow >  NonDay) == 0)
        throw MalibuException("Invalid day of week");
    if ((dow <= Saturday) == 0)
        throw MalibuException("Invalid day of week");

    if ((m > NonMonth) == 0)
        throw MalibuException("Invalid month");
    if ((m <= December) == 0)
        throw MalibuException("Invalid month");

    mdy_to_julian ();

    if (weeknum ==0)
        AddMonths(1);

    d = (dow - day_of_week);
    julian = julian + d;

    if (d > -1)
        weeknum--;

    AddWeeks(weeknum);
}

//---------------------------------------------------------------------------
//  Date::Date() --  Ctor
//---------------------------------------------------------------------------

Date::Date (const char *dat)
{
	tm			*local_time;
	time_t		 timer;
	const char	*pDat;	

	timer	   = time(NULL);
	local_time = localtime(&timer);

#ifdef VXWORKS
	if ( (*dat == '.') ||  (StringCompare(dat, "TODAY") == 0) )
#else
	if ( (*dat == '.') || (stricmp(dat, "TODAY") == 0) )
#endif		
    	{
        month = (local_time->tm_mon + 1);
        day   =  local_time->tm_mday;
        year  = (local_time->tm_year + 1900);
	    }
	else
    	{
        int  	nlen=strlen(dat);
        int		i;
        int		datetype;
        int		mon;

        //
        //  Possible date string formats are those that are generated by the
        //  Date class itself!  The corresponding possible string lengths are
        //  also listed (with ranges from shortest to longest string for that
        //  format).
        //
		//	MDY:		03/23/1993				=> 6-10,  13-17
		//	COLLATE:	19930323				=> 8
		//	EUROPEAN:	23 March 1993			=> 13,	  20
		//	FULL,ABBR.: Tue, Mar 23, 1993		=> 16-17, 23-24
		//	FULL:		Tuesday, March 23, 1993	=> 22-23, 29-30
        //
        //  These dates may also have B.C.E. appended at the end, thus we have
        //  the second set of string lengths with 7 more characters!
        //

		if (isalpha(dat[0]))
            datetype = Date::FULL;

		else if (nlen == 20)
            datetype = Date::EUROPEAN;

		else if (nlen == 13)
            datetype = (isalpha(dat[5]) ? Date::EUROPEAN : Date::MDY);

		else
            datetype = Date::MDY;		// or COLLATE - both done at default:

		switch(datetype)
    		{
			case Date::EUROPEAN:
                day = atoi(dat);

				pDat  = strchr(dat, ' ')+1;
				i = 12;

				while (i > 0)
    				{
					if (strnicmp(pDat, Months(i), 3) == 0)
					    break;

					i--;
	    			}

				mon   =  i;
				pDat  = strchr(pDat, ' ')+1;
                year  = atoi(pDat);
				break;

			case Date::FULL:
				pDat = strchr(dat, ' ')+1;		// Skip the day info

				i = 12;

				while (i > 0)
    				{
                    if (strnicmp(pDat, Months(i), 3) == 0)
                        break;

                    i--;
	    			}

				mon	  = i;

				pDat  = strchr(pDat, ' ')+1;
                day   = atoi(pDat);

				pDat  = strchr(pDat, ' ')+1;
                year  = atoi(pDat);
				break;


			default:
				i = strspn(dat,"0123456789");

				if (i == nlen)		// It's entirely number --> COLLATE
    				{
                    // This line sometimes produces a warning because the compiler
                    // assumes that "(dat[0]-'0')*1000" can go beyond the range of a
                    // short.  But the lines above it make sure it won't get higher
                    // than 9000, so every thing cool.
                    year  = ((dat[0]-'0')*1000 + (dat[1]-'0') *100  +
                             (dat[2]-'0')*10   + (dat[3]-'0'));

                    mon   = (dat[4]-'0')*10   + (dat[5]-'0');
                    day   = ((dat[6]-'0')*10   + (dat[7]-'0'));
		    		}
	    		else
			    	{
                    pDat  = &dat[i];
                    mon   = atoi(dat);		// Month is number before that
                    day   = atoi(pDat+1);	// day is number after that.

                    pDat  = strchr(pDat+1, *pDat); // Find next occurence of the first delimiter

                    if (pDat == NULL)				// If no 2nd delim, assume current year.
                        year = local_time->tm_year + 1900;
                    else
                        year = atoi(pDat+1);
        		    }
				break;
    		}

        //
        //  Convert B.C.E. year to proper value!
        //

		if (strchr(dat, '.') != NULL)
            year = -year;


		month = (enum Months) mon;

		//
		//  Verify values!
		//
		if ((month <= 0) || (day <= 0))
		    {
			month  	= NonMonth;
			day 	= 0;
			year 	= 0;
    		}

		if ( (year > 0) && (year < 100) )
            year = year + DefaultCentury;

	    }

	mdy_to_julian ();
}


//=============================================================================
//  Conversion operations
//=============================================================================

Date::operator const char *() const
{
    return (formatDate());
}

//=============================================================================
//  Date Arithmetic
//=============================================================================

//---------------------------------------------------------------------------
//  operator +=() --
//---------------------------------------------------------------------------

Date &Date::operator += (long i)
{
    julian += i;

    julian_to_mdy();

    return *this;
}

//---------------------------------------------------------------------------
//  operator -() --
//---------------------------------------------------------------------------

Date &Date::operator -= (long i)
{
    julian -= i;
    julian_to_mdy();

    return *this;
}

//---------------------------------------------------------------------------
//  operator ++() --
//---------------------------------------------------------------------------

Date Date::operator ++ ()
{
    julian++;

    julian_to_mdy();

    return *this;
}
 
//---------------------------------------------------------------------------
//  operator --() --
//---------------------------------------------------------------------------

Date Date::operator -- ()
{
    julian--;

    julian_to_mdy();

    return *this;
}

//---------------------------------------------------------------------------
//  operator ++() --
//---------------------------------------------------------------------------

Date Date::operator ++(int)
{
    Date temp=*this;				// TML - Necessary to save current
                                    // value of (*this) in order to
    julian++;						// simulate postfix operation!
    julian_to_mdy();

    return temp;
}

//---------------------------------------------------------------------------
//  operator --() --
//---------------------------------------------------------------------------

Date Date::operator --(int)
{
    Date temp=*this;				// TML - Necessary to save current
                                    // value of (*this) in order to
    julian--;						// simulate postfix operation!

    julian_to_mdy();

    return temp;
}


//=============================================================================
//  Ostream operations
//=============================================================================

//---------------------------------------------------------------------------
//  operator <<() --
//---------------------------------------------------------------------------

ostream & operator << (ostream &os, const Date &dt)
{
	return os << dt.formatDate();
}

//=============================================================================
//  Conversion routines
//=============================================================================

void Date::julian_to_wday()
{
    day_of_week = (enum Wday) ((julian + 2) % 7 + 1);
}

static const int OCT5_1582 =        2299160L;		// "really" 15-Oct-1582
static const int OCT14_1582 =   	2299169L;		// "really"  4-Oct-1582
static const int JAN1_1 =           1721423L;

static const int YEAR =          	365;
static const int FOUR_YEARS =     	1461;
static const int CENTURY =          36524L;
static const int FOUR_CENTURIES =   146097L;

//---------------------------------------------------------------------------
//  Date::julian_to_mdy() --
//---------------------------------------------------------------------------

void Date::julian_to_mdy()
{
	long	z,y;
	short 	m,d;
	int 	lp;

	z = julian+1;
	if (z >= OCT5_1582)
    	{
		z -= JAN1_1;
        z  = z + (z/CENTURY) - (z/FOUR_CENTURIES) - 2;
		z += JAN1_1;
	    }

    // Remove leap years before current year
    z = z - ((z-YEAR) / FOUR_YEARS);
	y = z / YEAR;

	d = (short) (z - (y * YEAR));

    // our base year in 4713BC
    y = y - 4712;
	if (y < 1)
		y--;

    // lp = 1 if this is a leap year.
    lp = !(y & 3);

	if (d==0)
    	{
        y--;
        d = YEAR + lp;
	    }

    // guess at month
    m  = d/30;

	while (DaysSoFar[lp][m] >=d)
        m--;	// Correct guess.

    d = d - DaysSoFar[lp][m];

	day = d;

    month = m+1;

    year = y;

    julian_to_wday();
}

//---------------------------------------------------------------------------
//  Date::mdy_to_julian() --
//
// The original here was far more complicated then it needed to be.
// What we need to keep in mind is the simple rule:
//	  Before 10/4/1585, a leap year occured every 4 years.
//	  After  10/15/1585, leap years were skipped on centuries
//		not divisible by 400. Plus 10 days were skipped to adjust
//		for the past error.
//---------------------------------------------------------------------------

void Date::mdy_to_julian()
{
	int		a;
	int		work_year=year;
	long	j;
	int 	lp;

	// correct for negative year  (-1 = 1BC = year 0)
	if (work_year < 0)
        work_year++;

    // lp = 1 if this is a leap year.
    lp = !(work_year & 3);

	j =
		((work_year-1) / 4)		+		// Plus ALL leap years
		DaysSoFar[lp][month-1]	+
		day					+
		(work_year * 365L)	+		// Days in years
		 JAN1_1 			+
		 -366;						// adjustments

	// deal with Gregorian calendar
	if (j >= OCT14_1582)
    	{
		a = (int)(work_year/100);
		j = j+ 2 - a + a/4;			// Skip days which didn't exist.
	    }

	julian = j;

    julian_to_wday();
}


//==========================================================================
// Format routines
//==========================================================================

//---------------------------------------------------------------------------
//  Date::formatDate() --
//---------------------------------------------------------------------------

char *Date::formatDate (int type) const
{
    static char buf[40];

#ifdef VXWORKS
    memset(buf, '\0', sizeof(buf));
#else
	strnset(buf, '\0', sizeof(buf));
#endif
	
	switch (type)
	{
		case Date::DAY:
			if ( (day_of_week < 1) || (day_of_week > 7) )
				strcpy(buf, InvalidDay());
			else
				strncpy(buf, DayName(day_of_week),
					(DisplayOptions & Abbreviation) ? ABBR_LENGTH : 9);
			break;

		case Date::MONTH:
			if ( (month < 1) || (month > 12) )
				strcpy(buf, InvalidMonth());
			else
                strncpy(buf, Months(month),
                        (DisplayOptions & Abbreviation) ? ABBR_LENGTH : 9);
			break;

		case Date::FULL:
			if ( (month < 1) || (month > 12) || (day_of_week < 0) ||
				 (day_of_week > 7) )
    			{
				strcpy(buf, InvalidDate());
	    		}
            else
                {
				int	y = abs(year);
				if ((DisplayOptions & NoCentury)  && (y > 1899) )
				    y = y % 100;

                strncpy( buf, DayName(day_of_week),
                    (DisplayOptions & Abbreviation) ? ABBR_LENGTH : 9);

                strcat( buf, ", ");
                strncat( buf, Months(month),
                    (DisplayOptions & Abbreviation) ? ABBR_LENGTH : 9);

                strcat( buf, " ");
                sprintf( buf+strlen(buf), "%d, %02d", day, y );
                if (year < 0)
                    strcat(buf,szBCE);
                }
			break;

		case Date::EUROPEAN:
			if ( (month < 1) || (month > 12) || (day_of_week < 0) ||
				 (day_of_week > 7) )
    			{
				strcpy(buf, InvalidDate());
	    		}
			else
                {
   				int	y = year;
				if ((DisplayOptions & NoCentury)  && (y > 1899) )
                    y = y % 100;

                sprintf(buf,"%d ",	day);

                strncat(buf, Months(month),
                        (DisplayOptions & Abbreviation) ? ABBR_LENGTH : 9);

                sprintf( buf+strlen(buf), " %02d", y );

                if (year < 0)
                    strcat(buf,szBCE);
                }
			break;

		case Date::COLLATE:
            // Can't generate COLLATE form for BCE dates.
            if (day==0 || month==0 || year<=0)
				strcpy(buf, InvalidDate());
			else
				sprintf( buf, "%04d%02d%02d", year, month, day);
			break;

		case Date::MDY:
		default:
			if (day==0 || month==0 || year==0)
				strcpy(buf, InvalidDate());
			else
    			{
				int	y = year;
				if ((DisplayOptions & NoCentury)  && (y > 1899) )
                    y = y % 100;

				sprintf( buf+strlen(buf), "%1d/%1d/%02d", month, day, y);
	    		}
			break;
    	}

    return (buf);
}

//---------------------------------------------------------------------------
//  Date::Century() --
//---------------------------------------------------------------------------

int Date::Century(short century)
{
	int	oldcent = DefaultCentury;

	if ( (century > 0) && (century < 100) )
        century = century * 100;

	DefaultCentury = century;

	return(oldcent);
}

//---------------------------------------------------------------------------
//  Date::Option() --
//---------------------------------------------------------------------------

bool Date::Option( int option, int action )
{
    bool retv = true;

    switch (option)
        {
        case NoCentury:
            if ( action )
                DisplayOptions |= NoCentury;
            else
                DisplayOptions &= (~NoCentury);
            break;

        case Abbreviation:
            if ( action )
                DisplayOptions |= Abbreviation;
            else
                DisplayOptions &= (~Abbreviation);
            break;

        default:
            retv = false;
            break;
        }

    return(retv);
}

//==========================================================================
//  Miscellaneous Routines
//==========================================================================

//---------------------------------------------------------------------------
//  Date::DayOfYear() --
//---------------------------------------------------------------------------

int Date::DayOfYear() const
{
	Date temp(1, 1, year);

	return julian - temp.julian + 1;
}

//---------------------------------------------------------------------------
//  Date::isLeapYear() --
//---------------------------------------------------------------------------

int Date::isLeapYear() const
{
	return  ( (year >= 1582) ?
              ((year % 4 == 0)  &&  (((year % 100) != 0)  ||  ((year % 400) == 0)) ):
			  (year % 4 == 0) );
}

//---------------------------------------------------------------------------
//  Date::isDaylightSavingsTime() --
//---------------------------------------------------------------------------

bool Date::isDaylightSavingsTime() const
{
    // Initialize start of DST and STD
    Date tempDST( (startDST >> 8), (startDST & 255), year ) ;
    Date tempSTD( (startSTD >> 8), (startSTD & 255), year ) ;

    // DST begins first Sunday in April
    tempDST += -tempDST.NumberOfDay() + 8 ;
    // STD begins last Sunday in October
    tempSTD -= (tempSTD.NumberOfDay() - 1) ;

    return( (bool) (julian >= tempDST.julian && julian < tempSTD.julian) ) ;
}
 
//---------------------------------------------------------------------------
//  Date::DaylightSavingsTime() --
//---------------------------------------------------------------------------

bool Date::DaylightSavingsTime(unsigned nMonth, unsigned nDay)
{
    if (nMonth < 13 && nDay < 32)
        {
        startDST = (nMonth << 8) + nDay;
        return true;
        }
    else
        return false;
}
 
//---------------------------------------------------------------------------
//  Date::StandardTime() --
//---------------------------------------------------------------------------

bool Date::StandardTime(unsigned nMonth, unsigned nDay)
{
    if (nMonth < 13 && nDay < 32)
        {
        startSTD = (nMonth << 8) + nDay;
        return true;
        }
    else
        return false;
}

//---------------------------------------------------------------------------
//  Date::Set() --
//---------------------------------------------------------------------------


Date &Date::Set()
{
	tm			*local_time;
	time_t		timer;

	timer 	  = time(NULL);
	local_time = localtime(&timer);

    month = local_time->tm_mon + 1;
    day   = local_time->tm_mday;
    year  = local_time->tm_year + 1900;

	mdy_to_julian();

	return *this;
}
 
//---------------------------------------------------------------------------
//  &Date::Set() --
//---------------------------------------------------------------------------

Date &Date::Set(unsigned int nMonth, unsigned int nDay, unsigned int nYear)
{
    month = nMonth;
    year  = (nYear > 9999) ? 0 : nYear;

    day   = (nDay < DaysInMonth()) ? nDay : DaysInMonth();

	mdy_to_julian();

	return *this;
}

//---------------------------------------------------------------------------
//  &Date::Set() --
//---------------------------------------------------------------------------

Date &Date::Set(long j)
{
	julian = j;

	julian_to_mdy();

	return *this;
}

//---------------------------------------------------------------------------
//  &Date::Set() --
//---------------------------------------------------------------------------

Date &Date::Set(int weeknum, int dow, short m, short y)
{
    int	d;

    // weeknum = 0 mean "the last"
    if ((weeknum > -1) == 0)
        throw MalibuException("Invalid week");
    if ((weeknum < 6) == 0)
        throw MalibuException("Invalid week");

    if ((dow >  NonDay) == 0)
        throw MalibuException("Invalid day of week");
    if ((dow <= Saturday) == 0)
        throw MalibuException("Invalid day of week");

    if ((m > NonMonth) == 0)
        throw MalibuException("Invalid month");
    if ((m <= December) == 0)
        throw MalibuException("Invalid month");

    Set(m,1,y);

    mdy_to_julian ();

    if (weeknum ==0)
        AddMonths(1);

    d = (dow - day_of_week);
    julian = julian + d;

    if (d > 0)
        weeknum--;

    // Julian_to_mdy done in AddWeeks()
    AddWeeks(weeknum);

    return(*this);
}
 
//---------------------------------------------------------------------------
//  Date::DaysInMonth() --
//---------------------------------------------------------------------------

unsigned int Date::DaysInMonth() const
{
    return GauDays[month] + (month==2 && isLeapYear());
}
 
//---------------------------------------------------------------------------
//  &Date::AddMonths() --
//---------------------------------------------------------------------------

Date &Date::AddMonths(int nCount)
{
    int nDays;

    month += nCount;

	while (month < 1)
    	{
        month += 12;
        year--;
    	}

	while (month > 12)
    	{
        month -= 12;
        year++;
	    }

    nDays = DaysInMonth();

    if (day > nDays)                    // Bump up the month by one if day is
        {                               // greater than # of days in new month
        month++;                        // and assigned the difference as the
        day -= nDays;                   // day value for the new month! - TML
        }

	mdy_to_julian();

	return *this;
}
 
//---------------------------------------------------------------------------
//  Date::AddWeeks() --
//---------------------------------------------------------------------------

Date & Date::AddWeeks(int nCount)
{
    Set(julian + static_cast<long>(nCount*7));
    return *this;
}
 
//---------------------------------------------------------------------------
//  &Date::AddYears() --
//---------------------------------------------------------------------------

Date &Date::AddYears(int nCount)
{
    year += nCount;
    mdy_to_julian();

    return *this;
}
 
//---------------------------------------------------------------------------
//  Date::WeekOfMonth() --
//---------------------------------------------------------------------------

int Date::WeekOfMonth() const
{
	// Abs day includes the days from previous month that fills up
	// the begin. of the week.
    int nAbsDay = day + FirstDayOfMonth()-1;
    return (nAbsDay-NumberOfDay())/7 + 1;
}

//---------------------------------------------------------------------------
//  Date::WeekOfYear() --
//---------------------------------------------------------------------------

int Date::WeekOfYear() const
{
    Date   doTemp(1, 1, year);
    return ((julian - doTemp.julian+1)/7) + 1;
}

} // namespace Innovative

