// Date_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Date_MbH
#define Date_MbH

#include <iostream>
#include <cstdlib>
#include <ctime>

#ifdef VXWORKS
#include "StringSupport_Mb.h"
#endif

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Date
{
    #define ABBR_LENGTH 3

public:
    enum FormatType {MDY, DAY, MONTH, FULL, EUROPEAN, COLLATE};
    enum {Off, On};
    enum {BufferSize=40};
    enum Actions { NoCentury = 0x02, Abbreviation = 0x04};
    enum Wday 	{NonDay=0, Sunday=1, Monday, Tuesday, Wednesday, Thursday, Friday, Saturday};
    enum Months   {NonMonth=0, January=1, February, March, April, May, June, July, August,
                            September, October, November, December};

    // Ctors
    Date ();
    Date (short m, short d, short y);
    Date (long 	j);
    Date (const char * dat);
    Date (const Date &dt);
    Date (const tm &TM);
    Date (int weeknum, int dow, short m, short y);
    // Dtor
    virtual ~Date() {}

    operator const char *() const;        // Date to character - via type casting

    // Basic Operators
    inline Date operator + (long i)	const
        {  return Date(julian + i);  }
    inline Date operator + (int i) const
        {  return Date(julian + (long)i);  }
    inline Date operator - (long i)	const
        {  return Date(julian - i);  }
    inline Date operator - (int i) const
        {  return Date(julian - (long)i);  }
    inline long operator - (const Date &dt)	const
        {  return ( julian - dt.julian );  }

    Date &operator += (long i);
    Date &operator -= (long i);

    Date  operator ++ ();               // Prefix increment
    Date  operator -- ();               // Prefix decrement

    Date  operator ++ (int);            // Postfix increment
    Date  operator -- (int);            // Postfix decrement

    inline int operator <  (const Date &dt) const
        {  return(julian <  dt.julian);  }
    inline int operator <= (const Date &dt) const
        {  return(julian <= dt.julian);  }
    inline int operator >  (const Date &dt) const
        {  return(julian >  dt.julian);  }
    inline int operator >= (const Date &dt) const
        {  return(julian >= dt.julian);  }
    inline int operator == (const Date &dt) const
        {  return(julian == dt.julian);  }
    inline int operator != (const Date &dt) const
        {  return(julian != dt.julian);  }

    inline static void Format (enum FormatType format)
        {  DisplayFormat = format;  }
    static bool		Option (int option, int action=On);
    static int   	Century(short century);

    friend std::ostream &operator << (std::ostream &os, enum FormatType ft)
        {
        Date::Format(ft);
        return(os);
        }

    char *formatDate(int type=DisplayFormat) const;

    // returns julian date
    inline long	julDate() const
        {  return(julian);  }
    // returns relative date since Jan. 1
    int	DayOfYear()	const;

    // returns true if leap year, false if not
    int		isLeapYear() const;
    // returns true if date is within Daylight
    bool 	isDaylightSavingsTime() const;
    // Savings Time (DST), false if not

    // Sets the month and day which DST and STD date begins!  This will
    // enable isDaylightSavingsTime() to return the correct result for regions other than
    // North America.  Returns true if month and day values are valid, false
    // otherwise - TML
    static bool	DaylightSavingsTime(unsigned nMonth, unsigned nDay);
    static bool	StandardTime(unsigned nMonth, unsigned nDay);

    // These 'Set's modify the date object and actually SET it.
    // They all return a reference to self (*this)
    // Sets to current system date
    Date & Set();
    Date & Set(long lJulian);
    Date & Set(unsigned int nMonth, unsigned int nDay, unsigned int nYear);
    Date & Set(int weeknum, int dow, short m, short y);

    // May also pass neg# to decrement
    Date & AddMonths(int nCount = 1);
    Date & AddWeeks(int nCount = 1);
    Date & AddYears(int nCount = 1);

    // Number of days in month (1..31)
    unsigned int DaysInMonth() const ;

    // Numeric Week Of Month  (1..6)
    int	WeekOfMonth() const;
    // Numeric Week Of Year  (1..52)
    int	WeekOfYear() const;

    // First Day Of Month	(1..7)
    inline int	FirstDayOfMonth() const
        {  return Date(month, 1, year).NumberOfDay();  }

    // Numeric Day of date object
    inline int Day()const
        {  return day;  }

    //	Day Of Week
    // Character ('Sunday'..'Saturday')
    inline const char * DayOfWeek()	const
        {  return(formatDate(DAY));  }
    // (1..7)
    inline int NumberOfDay() const
        {  return day_of_week;  }

    // eg. 1992
    inline int NumberOfYear()	const
        {  return year;  }

    // Month Number (1..12)
    inline int NumberOfMonth()	const
        {  return month;  }

    // First Date Of Month
    inline Date BeginningOfMonth()const
        {  return(Date(month, 1, year));  }

    // Last Date Of Month
    inline Date EndOfMonth() const
        {  return((Date(month, 1, year).AddMonths(1))-1);  }

    // First Date Of Year
    inline Date BeginningOfYear() const
        {  return(Date(1, 1, year));  }

    // Last Date Of Year
    inline Date EndOfYear() const
        {  return(Date(1, 1, year+1)-1);  }

    // Character Month name
    inline const char * NameOfMonth() const
        {  return(formatDate(MONTH));  }    
    
#if !defined(strnicmp) && defined(VXWORKS)
    int strnicmp(const char *c1,const char *c2, int l) { return Innovative::StringCompare(c1,c2,l);}
#endif

protected:
    unsigned long julian;	// see julDate();  days since 1/1/4713 B.C.
    short	month;			// see NumberOfMonth()
    short 	day;			// see Day()
    short 	year;			// see NumberOfYear()
    char 	day_of_week;	// see NumberOfDay();	1 = Sunday, ... 7 = Saturday
    char	separator;

private:
    static			int		DisplayFormat;
    static unsigned int		DisplayOptions;
    static unsigned short	DefaultCentury;
    static unsigned int 	startDST;
    static unsigned int 	startSTD;

    void julian_to_mdy();         // convert julian day to mdy
    void julian_to_wday();        // convert julian day to day_of_week
    void mdy_to_julian();         // convert mdy to julian day

    static const char * InvalidDay();
    static const char * InvalidDate();
    static const char * InvalidMonth();
    static const char * Months(int idx);
    static const char * DayName(int idx);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
