//---------------------------------------------------------------------------
// Ddc.cpp
//
// DDC logic-specific data flow and hardware I/O
//
//  INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.

//---------------------------------------------------------------------------
#if 0

#include <Ddc_Mb.h>
#include <math.h>
#include <Magic_Mb.h>
#include <Exception_Mb.h>
#include <Magic_Mb.h>
#include <StringSupport_Mb.h>
#include <SystemSupport_Mb.h>
#include <FileSupport_Mb.h>
#include <PacketFileDataSet_Mb.h>
#include <FileDataSet_Mb.h>
#include <Thread_Mb.h>
#include <FcfFile_Mb.h>
#include <algorithm>
#include <fstream>


using namespace Innovative;
using namespace DdcMemoryMap;
using namespace std;

namespace Innovative
{

#ifdef __BORLANDC__
    #pragma option push
    #pragma warn -8004
#endif


// Enable only for extended register diagnosis
#ifdef ExtRegDiagnosis

//===========================================================================
//  CLASS RegisterNames  --  Map register offset to name
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class RegisterNames
//---------------------------------------------------------------------------

RegisterNames::RegisterNames()
{
    Map[DdcMemoryMap::mmDdsSource] = "DdsSource";
    Map[DdcMemoryMap::mmDdsFrequency] = "DdsFrequency";
    Map[DdcMemoryMap::mmCicControl] = "CicControl";
    Map[DdcMemoryMap::mmCfirControl] = "CfirControl";
    Map[DdcMemoryMap::mmPfirControl] = "PfirControl";
    Map[DdcMemoryMap::mmFirLsb] = "FirLsb";
    Map[DdcMemoryMap::mmCicLsb] = "CicLsb";
    Map[DdcMemoryMap::mmAcCouple] = "AcCouple";
    Map[DdcMemoryMap::mmDdcChannel] = "DdcChannel";
    Map[DdcMemoryMap::mmPowerMeter] = "PowerMeter";
    Map[DdcMemoryMap::mmOverflow] = "Overflow";
    Map[DdcMemoryMap::mmDataType] = "DataSource";
    Map[DdcMemoryMap::mmPacketSize] = "PacketSize";
}

std::string  RegisterNames::Name(int offset)
{
    if (Map.count(offset))
        return Map.find(offset)->second;
    else
        return "";
}

//===========================================================================
//  CLASS DdcShadowRegister  --  Debug version of ShadowRegister, which
//                               echoes value written to log window
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DdcShadowRegister
//---------------------------------------------------------------------------

DdcShadowRegister::DdcShadowRegister(AddressingSpace &space, int offset)
    : inherited(space, offset), Offset(offset)
{
}

//------------------------------------------------------------------------------
//  DdcShadowRegister::ShadowRegisterChanged() --
//------------------------------------------------------------------------------

void DdcShadowRegister::ShadowRegisterChanged()
{
//	Debug.Enabled(true);
//	std::stringstream msg;
//	msg << "Reg[0x" << std::hex << Offset << "] = 0x" << mShadowValue
//		<< "  (" << Reg.Name(Offset) << ")";
//	Debug.Log(msg.str().c_str());
    Apply();
}

#endif


//===========================================================================
//  CLASS DdcCombSubFilter  --
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DdcSubFilter
//---------------------------------------------------------------------------

DdcCombSubFilter::DdcCombSubFilter(AddressingSpace & space, int DecimationReg, unsigned int ch)
    : DeciReg(space, DecimationReg), FBStart(0), FChannel(ch)
{
}

void DdcCombSubFilter::Load(const string & file, const string & section)
{
    IniFile Ini(file,false,100000,false);
    Ini.Section = section;
    //  Read the BStart value
    FBStart =  Ini.Read("BStart", 0);
    FDecimationRate = Ini.Read("CicRate", 128);
}

void DdcCombSubFilter::Apply()
{
    // Set decimation rate
    DeciReg.Value(FDecimationRate);
}

//---------------------------------------------------------------------------
//  DdcFirSubFilter:: --  Load coefficients into filter
//---------------------------------------------------------------------------

DdcFirSubFilter::DdcFirSubFilter(AddressingSpace & space, int reg, unsigned int ch)
    :LoadReg(space, reg), FChannel(ch)
{
}

void DdcFirSubFilter::Load(const string & file, const string & section)
{
    IniFile Ini(file,false,100000,false);
    Ini.Section = section;
    //  Read the BStart value
    FBStart =  Ini.Read("BStart", 0);
    //  Read the Taps from the INI file
    FTaps = Ini.Read("Taps", 0);
    FTapsToLoad = Ini.Read("TapsToLoad", 0);

    if (FTapsToLoad<=0)
        FTapsToLoad = FTaps;
    LoadOrder.clear();	// start from the begining
    Data.clear();
    for (unsigned int i = 0; i < FTapsToLoad; ++i)
        {
        std::string idx = "Index" + IntToString(i);
        std::string tag = "Coeff" + IntToString(i);
        LoadOrder.push_back(Ini.Read(idx, i));
        Data.push_back(Ini.Read(tag, 0));
        }
}

void DdcFirSubFilter::Apply()
{
    int i;
    int Len = (int) Data.size();
//	for (IntArray::iterator i = Data.begin(); i != Data.end(); ++i)
//		LoadReg.Value(*i);
        // By hardware definition, the filter values must be in the following seq.
        // 1. No of Taps 2. Channel Number 3. Filter values.
    LoadReg.Value(FTapsToLoad);
    LoadReg.Value(FChannel);
    for (i = 0; i < Len; i++)
        {
        LoadReg.Value(Data[LoadOrder[i]]);
        }
}

//===========================================================================
//  CLASS DdcFilter  --
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class DdcFilter
//---------------------------------------------------------------------------

DdcFilter::DdcFilter(AddressingSpace & space, unsigned int ch)
    : Cic(space, DdcMemoryMap::mmCicControl, ch),
      Cfir(space, DdcMemoryMap::mmCfirControl, ch),
      Pfir(space, DdcMemoryMap::mmPfirControl, ch),
      FChannel(ch),
      FirBStartReg(space, DdcMemoryMap::mmFirLsb),
      CicBStartReg(space, DdcMemoryMap::mmCicLsb)
{
}

//---------------------------------------------------------------------------
//  DdcFilter::Load() --
//---------------------------------------------------------------------------
bool DdcFilter::Load(const std::string & file)
{
    Cic.Load(file, "CIC");
    Cfir.Load(file, "CFir");
    Pfir.Load(file, "PFir");

    return (Cic.BStart() < 3) && (Cfir.BStart() < 7) && (Pfir.BStart() < 7);
}

//---------------------------------------------------------------------------
//  DdcFilter::Apply() --  Load coefficients into filter
//---------------------------------------------------------------------------

void DdcFilter::Apply()
{
        // By hardware definition, the filter values must be in the following seq.
        // 1. No of Taps 2. Channel Number 3. Filter values.
        // For now simply do step #1.
        // Load Bstart values.
    CicBStartReg.Lsb.Value(Cic.BStart());
    FirBStartReg.CfirLsb.Value(Cfir.BStart());
    FirBStartReg.PfirLsb.Value(Pfir.BStart());
    Cic.Apply();
    Innovative::Sleep(10);
    Cfir.Apply();
    Innovative::Sleep(10);
    Pfir.Apply();
    Innovative::Sleep(10);
}

//===========================================================================
//  CLASS DdcChannel  -- Register & low level support for DDC Ddcs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DdcChannel
//---------------------------------------------------------------------------

DdcChannel::DdcChannel(float & SampleRate, AddressingSpace & space, unsigned char ch)
    : CurrentSampleRate(SampleRate),
      FFrequency(1.0e6),
      FChannel(ch),
      Filter(space, ch),
      TuningFreqGenerator(space, DdcMemoryMap::mmDdsFrequency),
      SourceReg(space, DdcMemoryMap::mmDdsSource),
      AcCoupleReg(space, DdcMemoryMap::mmAcCouple),
      ChannelReg(space, DdcMemoryMap::mmDdcChannel),
      OverflowReg(space, DdcMemoryMap::mmOverflow)
{
}

//---------------------------------------------------------------------------
//  DdcChannel::Channel() --  Return channel index
//---------------------------------------------------------------------------

unsigned int DdcChannel::Channel() const
{
    return FChannel;
}

//---------------------------------------------------------------------------
//  DdcChannel::Source() --  Set input source
//---------------------------------------------------------------------------

void DdcChannel::Source(unsigned char value)
{
    ChannelReg.Channel.Value(FChannel);
    SourceReg.Value(value);
}

//---------------------------------------------------------------------------
//  DdcChannel::Source() --  Get input source
//---------------------------------------------------------------------------

unsigned char DdcChannel::Source() const
{
    return static_cast<unsigned char>(SourceReg.Value());
}

//---------------------------------------------------------------------------
//  DdcChannel::Load() --  Parse and load coef file into filter
//---------------------------------------------------------------------------

bool DdcChannel::LoadFilter(const string & file)
{
    bool RtnVal;
    RtnVal = Filter.Load(file);
    return RtnVal;
}

//---------------------------------------------------------------------------
//  DdcChannel::IsOverflow() --
//---------------------------------------------------------------------------

int DdcChannel::IsOverflow()
{
    ChannelReg.Channel.Value(FChannel);
    return OverflowReg.Value();
}
//---------------------------------------------------------------------------
//  DdcChannel::ApplyTuningFreq() --  Apply cached settings to hardware
//---------------------------------------------------------------------------

void DdcChannel::ApplyTuningFreq(float value)
{
    FFrequency = value;
    ChannelReg.Channel.Value(FChannel);
//	TuningFreqGenerator.Frequency(DdcGroup->SampleRate(), value);
    TuningFreqGenerator.Frequency(CurrentSampleRate, value);
}

//---------------------------------------------------------------------------
//  DdcChannel::Apply() --  Apply cached settings to hardware
//---------------------------------------------------------------------------

void DdcChannel::ApplyFilterSettings()
{
    Filter.Apply();
}

//===========================================================================
//  CLASS PowerMeter  --
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class PowerMeter
//---------------------------------------------------------------------------

PowerMeter::PowerMeter(AddressingSpace & space)
    : MeasureReg(space, mmPowerMeter), CtlReg(space, mmPowerControl)
{
}

//------------------------------------------------------------------------------
//  PowerMeter::dBm() --  Return current power, in dBm
//------------------------------------------------------------------------------

float PowerMeter::dBm() const
{
    float PwrOf2;
    int iTmp = MeasureReg.Value();
    if (iTmp <= 0)
        {
        return -77.0;		// some most -ve value
        }
    PwrOf2 = static_cast<float> (pow(2.0, 27.0));
    float fTmp = static_cast<float>(iTmp)/(PwrOf2*50);
    fTmp = static_cast<float>(log10(fTmp*1000.0));
    return 10.0f * fTmp;
}


//===========================================================================
//  CLASS DdsGenerator  --
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DdsGenerator
//---------------------------------------------------------------------------

DdsGenerator::DdsGenerator(AddressingSpace & space, int offset)
    : DdsScaleFactor(static_cast<float>(pow(2.0, 32.0))),
      TuningReg(space, offset)
{
}

//---------------------------------------------------------------------------
//  DdsGenerator::Frequency() --  Set DDS frequency
//---------------------------------------------------------------------------

void DdsGenerator::Frequency(float FSampleRate, float value)
{
    int result = static_cast<int>(DdsScaleFactor * value / FSampleRate);
    TuningReg.Value(result);
}

//---------------------------------------------------------------------------
//  DdsGenerator::Frequency() --  Get DDS frequency
//---------------------------------------------------------------------------

float DdsGenerator::Frequency(float FSampleRate) const
{
    return (TuningReg.Value() * FSampleRate) / DdsScaleFactor;
}

#ifdef __BORLANDC__
    #pragma option pop
#endif


} // Innovative


#endif
