//---------------------------------------------------------------------------
// Ddc.h
//
// DDC logic-specific data flow and hardware I/O
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef DdcH
#define DdcH
#if 0

#include <HardwareRegister_Mb.h>
#include <StringVector_Mb.h>
#include <ProcessEvents_Mb.h>
#include <Trace_Mb.h>
#include <BinView_Mb.h>
#include <sstream>
#include <map>

//---------------------------------------------------------------------------

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

	// It is expected that DDC Functionality will use the same
	// register set across products.  Thus the register are
	// defined here.
namespace DdcMemoryMap
{
	//
	//  User Logic Memory Registers
	//
	// H/W configuration read Register.
	const unsigned int  mmHwConfig            = 0x16;
	// Control
	const unsigned int  mmDdsSource           = 0x28;
	const unsigned int  mmDdcReset            = 0x34;
	// Tuning
	const unsigned int  mmDdsFrequency        = 0x29;
	// Filters
	const unsigned int  mmCicControl          = 0x2a;
	const unsigned int  mmCfirControl         = 0x2b;
	const unsigned int  mmPfirControl         = 0x2c;
	const unsigned int  mmFirLsb              = 0x2d;
	const unsigned int  mmCicLsb              = 0x15;
	const unsigned int  mmAcCouple            = 0x2f;
	const unsigned int  mmDdcChannel          = 0x2e;
	// Status
	const unsigned int  mmPowerMeter          = 0x09;
	const unsigned int  mmPowerControl        = 0x12;
	// Flow
	const unsigned int  mmOverflow            = 0x07;
	const unsigned int  mmDataType            = 0x35;
	const unsigned int  mmPacketSize          = 0x36;

	const unsigned int  mmTestGenFrequency    = 0x37;


} // end namespace DdcMemoryMap



#define ExtRegDiagnosis

// Enable only for extended register diagnosis
#ifdef ExtRegDiagnosis

//===========================================================================
//  CLASS RegisterNames  --  Map register offset to name
//===========================================================================

class RegisterNames
{
public:
	RegisterNames();
	std::string     Name(int offset);

private:
	typedef std::map<int, std::string>  StringMap;
	StringMap   Map;
};

//===========================================================================
//  CLASS DdcShadowRegister  --  Debug version of ShadowRegister, which
//                               echoes value written to log window
//===========================================================================

class DdcShadowRegister : public ShadowRegister
{
	typedef ShadowRegister inherited;

public:
	DdcShadowRegister(AddressingSpace &space, int offset);

protected:
//	Trace           Debug;
	int             Offset;
	RegisterNames   Reg;

	virtual void ShadowRegisterChanged();
};
#else
typedef ShadowRegister DdcShadowRegister;
#endif

//===========================================================================
//  CLASS DdcSubFilter  --
//===========================================================================
class DdcCombSubFilter
{

	public:
	DdcCombSubFilter(AddressingSpace & space, int DecimationReg, unsigned int ch);

	// Methods
	void        Apply();
	void        Load(const std::string & file, const std::string & section);

	// Properties
	unsigned int BStart() const
		{  return FBStart;  }

   unsigned int DecimationRate() const
		{	return FDecimationRate; }

	private:
	// Registers
	DdcShadowRegister   DeciReg;

	// Data
	unsigned int    FBStart;
	unsigned int   	FDecimationRate;
	unsigned int 	FChannel;

};

class DdcFirSubFilter
{
public:
	typedef std::vector<int> IntArray;
	DdcFirSubFilter(AddressingSpace & space, int reg, unsigned int ch);

	// Methods
	void        Apply();
	void        Load(const std::string & file, const std::string & section);

	// Properties
	unsigned int BStart() const
		{  return FBStart;  }
	unsigned int TapsToLoad() const
		{  return FTapsToLoad;  }
	size_t Taps() const
		{  return Data.size();  }

private:
	// Registers
	DdcShadowRegister   LoadReg;

	// Data
	IntArray        Data;
	IntArray		LoadOrder;
	unsigned int    FBStart;
	unsigned int    FTaps;
	unsigned int    FTapsToLoad;
	unsigned int    FChannel;
};

//===========================================================================
//  CLASS DdcFilter  --
//===========================================================================

class DdcFilter
{

public:
	typedef std::vector<std::string> StringArray;

	DdcFilter(AddressingSpace & space, unsigned int ch);

	// Methods
	void        Apply();
	bool        Load(const std::string & file);

	unsigned int    DecimationRate() const
		{	return (Cic.DecimationRate() * 4); }	// Cfir & Pfir have fixed decimation rate

	private:
	// Registers
	DdcCombSubFilter   Cic;
	DdcFirSubFilter    Cfir;
	DdcFirSubFilter    Pfir;
	unsigned int    FChannel;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS FirBstartRegister  --
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class FirBstartRegister : public DdcShadowRegister
	{
		typedef DdcShadowRegister inherited;

	public:
		RegisterBitGroup    CfirLsb;
		RegisterBitGroup    PfirLsb;

		FirBstartRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  CfirLsb(*this, 0, 7), PfirLsb(*this, 8, 7)
			{
			}
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS CicBstartRegister  --
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class CicBstartRegister : public DdcShadowRegister
	{
		typedef DdcShadowRegister inherited;

	public:
		RegisterBitGroup    Lsb;

		CicBstartRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Lsb(*this, 0, 3)
			{
			}
	};

	FirBstartRegister           FirBStartReg;
	CicBstartRegister           CicBStartReg;
};

//===========================================================================
//  CLASS DdsGenerator  --
//===========================================================================

class DdsGenerator
{
public:
	DdsGenerator(AddressingSpace & space, int offset);

	// Properties
	void           		 Frequency(float FSampleRate, float value);
	float          		 Frequency(float FSampleRate) const;

private:
	const float     	 DdsScaleFactor;
	// Registers
	DdcShadowRegister    TuningReg;
};

//===========================================================================
//  CLASS DdcChannel  --
//===========================================================================

class DdcChannel
{

public:
	// Ctor
	DdcChannel(float & SampleRate, AddressingSpace & space, unsigned char ch);

	// Methods
	void            ApplyFilterSettings();
	void 			ApplyTuningFreq(float value);
	bool            LoadFilter(const std::string & file);
	unsigned int    Channel() const;
	int             IsOverflow();

	// Properties
	float           Frequency() const;

	void            Source(unsigned char value);
	unsigned char   Source() const;

	void            AcCouple(int state)
		{  AcCoupleReg.Enable = state ? 0 : 1;  }

	bool            AcCouple() const;

	unsigned int    DecimationRate() const
		{	return Filter.DecimationRate(); }

	private:
	// Fields
//	DdcCollective * 		DdcGroup;
	float &				CurrentSampleRate;
	float               FFrequency;
	unsigned int        FChannel;
	DdcFilter           Filter;
	DdsGenerator		TuningFreqGenerator;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS AcCoupleRegister  --
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class AcCoupleRegister : public DdcShadowRegister
	{
		typedef DdcShadowRegister inherited;

	public:
		RegisterBit         Enable;

		AcCoupleRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Enable(*this, 31)
			{
			}
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS ChannelSelectRegister  --
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class ChannelSelectRegister : public DdcShadowRegister
	{
		typedef DdcShadowRegister inherited;

	public:
		RegisterBitGroup    Channel;

		ChannelSelectRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Channel(*this, 0, 3)
			{
			}
	};

	// Registers
	DdcShadowRegister           SourceReg;
	AcCoupleRegister            AcCoupleReg;
	ChannelSelectRegister       ChannelReg;
	Register                    OverflowReg;

	// Data
	Innovative::StringVector    File;
	BinView                     Graph;
};

//===========================================================================
//  CLASS PowerMeter  --
//===========================================================================

class PowerMeter
{

public:
	PowerMeter(AddressingSpace & space);

	// Methods
	float       dBm() const;

	// Properties
	void        Samples(int value)
		{  CtlReg.Samples.Value(value);  }
	int         Samples() const
		{  return CtlReg.Samples.Value();  }

	void        Imaginary(bool state)
		{  CtlReg.Imaginary.Value(state ? 1 : 0);  }
	bool        Imaginary() const
		{  return CtlReg.Imaginary.Value() ? true : false;  }

	void        Cooked(bool state)
		{  CtlReg.Cooked.Value(state ? 1 : 0);  }
	bool        Cooked() const
		{  return CtlReg.Cooked.Value() ? true : false;  }

	void        Channel(int ch)
		{  CtlReg.Channel.Value(ch);  }
	int         Channel() const
		{  return CtlReg.Channel.Value();  }

private:
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS PowerControlRegister  -- Powermeter control
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class PowerControlRegister : public DdcShadowRegister
	{
		typedef DdcShadowRegister inherited;

	public:
		RegisterBitGroup    Samples;
		RegisterBit         Imaginary;
		RegisterBit         Cooked;
		RegisterBitGroup    Channel;

		PowerControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Samples(*this, 0, 5), Imaginary(*this, 16),
			  Cooked(*this, 17), Channel(*this, 18, 8)
			{
			}
	};

	// Registers
	ReadOnlyRegister	    MeasureReg;
	PowerControlRegister    CtlReg;
};


//===========================================================================
//  CLASS DdcCollective  --
//===========================================================================
template <class T>
class DdcCollective : public BaseboardExtension<T>
{
//	friend class X5_DDC;
	friend class DdcChannel;
	typedef BaseboardExtension<T> inherited;

public:
	typedef std::vector<DdcChannel *> DdcArray;
	typedef DdcArray::iterator iterator;
	typedef DdcArray::reverse_iterator reverse_iterator;

	// Ctor
	DdcCollective(T & owner, int NumOfChannels, float MaxFreq)
	: inherited(owner),
          NumOfDDCChannels(NumOfChannels),
          FMaxSampleRate(MaxFreq),
          FSampleRate(MaxFreq),
          TestGenerator(LogicMemorySpace(owner), DdcMemoryMap::mmTestGenFrequency),
		  Power(LogicMemorySpace(owner)),
          HwConfigReg(LogicMemorySpace(owner),   DdcMemoryMap::mmHwConfig),
          PacketSizeReg(LogicMemorySpace(owner), DdcMemoryMap::mmPacketSize),
          DataTypeReg(LogicMemorySpace(owner),   DdcMemoryMap::mmDataType),
          ResetReg(LogicMemorySpace(owner),      DdcMemoryMap::mmDdcReset)

		{
		for (int i = 0; i < NumOfChannels; ++i)
//			ADdcChannelArray.push_back(new DdcChannel(this, LogicMemorySpace(owner), i));
			ADdcChannelArray.push_back(new DdcChannel(FSampleRate, LogicMemorySpace(owner), i));
		}

	~DdcCollective()
		{
		for (size_t i = 0; i < ADdcChannelArray.size(); ++i)
			delete ADdcChannelArray[i];
		}

	DdcChannel operator [] (int idx) const
		{ return *ADdcChannelArray.at(idx); }
	DdcChannel & operator [] (int idx)
		{ return *ADdcChannelArray.at(idx); }

	// Methods
	void 			Reset()
		{
		ResetReg.Value(1);
		ResetReg.Value(0);
		}

	// Properties
	void            RawData(int state)
		{ DataTypeReg.Value(state ? 1 : 0); }
	bool            RawData() const
		{ return DataTypeReg.Value() ? true : false; }

	void            PacketSize(unsigned int words)
		{ PacketSizeReg.Size.Value(words); }
	
	unsigned int    PacketSize() const
		{ return PacketSizeReg.Size.Value() - 2; }

	void            SampleRate(float value)
		{ FSampleRate = value; }
	float           SampleRate() const
		{ return FSampleRate; }

	void			ApplyTestGenFreq(float value)
		{ TestGenerator.Frequency(FSampleRate, value); }

	void			SetPowerMeter(int NumOfPoints, int MonitorCh, bool cooked, bool imaginary)
		{
		Power.Samples(NumOfPoints);
		Power.Channel(MonitorCh);
		Power.Cooked(cooked);
		Power.Imaginary(imaginary);
		}

	void            GetHwConfig(unsigned int &RtnNumA2ds, int & RtnNumDDCs, char & RtnHwType)
	{
		unsigned int RegValue = HwConfigReg.Value();
		RtnNumDDCs = static_cast<int>(RegValue & 0x0FFFF);		// 1st 16 bits.
		RtnNumA2ds = (RegValue >> 16) & 0x0F;	// 16-19
		RtnHwType = static_cast<char>((RegValue >> 20) & 0x0F);	// 20 onwards
	}

	float			GetPwrMeterVal()
					{ return Power.dBm(); }

private:
	// Data
	int				NumOfDDCChannels;
	float			FMaxSampleRate;
	float       	FSampleRate;
	DdcArray    	ADdcChannelArray;
	DdsGenerator    TestGenerator;
	PowerMeter      Power;
	ReadOnlyRegister HwConfigReg;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS PacketSizeRegister  --
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class PacketSizeRegister : public DdcShadowRegister
	{
		typedef DdcShadowRegister inherited;

	public:
		RegisterBitGroup    Size;
		RegisterBitGroup    Channel;

		PacketSizeRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Size(*this, 0, 24), Channel(*this, 24, 8)
			{
			}
	};

	PacketSizeRegister      PacketSizeReg;
	DdcShadowRegister       DataTypeReg;
	DdcShadowRegister       ResetReg;
};

template <class T>
class X5_DDC : public T
{
public:
	// Ctor
	X5_DDC(int NumOfChannels, float MaxFreq)
	: Ddc(*this, NumOfChannels, MaxFreq)
	{
	}

	// Data
	DdcCollective<T>     Ddc;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}   // end namespace Innovative
#endif

#endif
