//DDS Sine Wave Generator (c) Copyright 2003 -- Daniel Wallner This
//  software is free. You can redistribute it and/or modify it under the terms
//  of the GNU General Public License as published by the Free Software
//  Foundation.

#ifndef Dds_MbH
#define Dds_MbH

#include <cmath>
#include <vector>
#include <Trace_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS Dds  --
//=============================================================================

class Dds
{
public:
    void Init(const unsigned int precision /* bits */, const double amplitude)
    {
		double Pi = 4.0*std::atan(1.0);
		unsigned int tLength = static_cast<unsigned int>(Pi * (1 << precision) + 1.);

        m_Debug.Log("Creating DDS table entries ", static_cast<int>(tLength));

        m_Table.resize(tLength);

        for (unsigned int i = 0; i < tLength; i++)
            m_Table[i] = amplitude * std::sin(2. * Pi * i / tLength);

        m_Phase = 0.;
    }

    void Advance(const double frequency, const double fs)
    {
        double  length = static_cast<double>(m_Table.size());
        m_Phase += frequency * length / fs;
        if (m_Phase > length)
            m_Phase -= length;
    }

    //double Sin() { return m_Table[unsigned int(round(m_Phase))]; }
    double  Sin()
        {  return m_Table[static_cast<unsigned int>(m_Phase)];  }

private:
    double              m_Phase;
    std::vector<double> m_Table;
	Trace				m_Debug;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
