//  DeviceDriverBmManager_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DeviceDriverBmManager_Mb.h>
#include <SystemSupport_Mb.h>
#include <DeviceDriver_Mb.h>
#include <algorithm>

namespace InnovativeKernel
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS  StandardPacketBmManager  --  Manage Busmaster Memory Alloc & HW Load
//=============================================================================
//  Implementation for 'normal' Packet Stream devices:
//      -- One contiguous region in each direction
//      -- Regions allocated by WinDriver
//---------------------------------------------------------------------------
//  constructor for class StandardPacketBmManager()
//---------------------------------------------------------------------------
//  note - we don't own the MDC.
#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

StandardPacketBmManager::StandardPacketBmManager(IPmRequires_PortBaseAddr * mdc,
                                   Osal::OsDevice & device, MapRegisters & regs)
    : MDC(mdc), Device(device), MyUI(this),
      BmRingBaseAddress_Outgoing(PortMemorySpace, regs.RingBaseOutgoing_Addr),
      BmRingBaseAddress_Incoming(PortMemorySpace, regs.RingBaseIncoming_Addr),
      BmRingBaseAddressMSW_Outgoing(PortMemorySpace, regs.RingBaseOutgoingMSW_Addr),
      BmRingBaseAddressMSW_Incoming(PortMemorySpace, regs.RingBaseIncomingMSW_Addr),
      BmRingSize_Outgoing(PortMemorySpace, regs.RingSizeOutgoing_Addr),
      BmRingSize_Incoming(PortMemorySpace, regs.RingSizeIncoming_Addr)
{
}
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//------------------------------------------------------------------------------
//  StandardPacketBmManager::AllocateRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Allocate using standard contiguous BM Region

void  StandardPacketBmManager::AllocateRegion(BmRegions region, ii32 region_size_words)
{
    Device.AllocateMemoryBlock(GetBmMemoryBlock(region), region_size_words*sizeof(int));
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::FreeRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Free standard contiguous BM Region

void  StandardPacketBmManager::FreeRegion(BmRegions region)
{
    Device.FreeMemoryBlock(GetBmMemoryBlock(region));
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::RequestResources() -- Request custom Resources
//------------------------------------------------------------------------------

void  StandardPacketBmManager::RequestResources()
{
   PortMemorySpace.Link( MDC->GetPortBaseAddr() );
}

//---------------------------------------------------------------------------
//  StandardPacketBmManager::ApplyDeviceInfo() - Apply Region info to hdw
//---------------------------------------------------------------------------

void  StandardPacketBmManager::ApplyDeviceInfo()
{
    BmRingBaseAddressMSW_Outgoing.Value(static_cast<ii32>(reinterpret_cast<ii64>(BusMasterMemory_Outgoing.PhysAddr)>>32));
    BmRingBaseAddress_Outgoing.Value(static_cast<ii32>(reinterpret_cast<ii64>(BusMasterMemory_Outgoing.PhysAddr)));
    BmRingBaseAddressMSW_Incoming.Value(static_cast<ii32>(reinterpret_cast<ii64>(BusMasterMemory_Incoming.PhysAddr)>>32));
    BmRingBaseAddress_Incoming.Value(static_cast<ii32>(reinterpret_cast<ii64>(BusMasterMemory_Incoming.PhysAddr)));

    BmRingSize_Outgoing.Value( BusMasterMemory_Outgoing.Size / sizeof(int) );
    BmRingSize_Incoming.Value( BusMasterMemory_Incoming.Size / sizeof(int) );
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::GetBmMemoryBlock() -- return block for region
//------------------------------------------------------------------------------

MemoryBlock & StandardPacketBmManager::GetBmMemoryBlock( BmRegions which )
{
    switch (which)
        {
        default:
        case bmIncoming:
            return BusMasterMemory_Incoming;
        case bmOutgoing:
            return BusMasterMemory_Outgoing;
        }
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::FindOffsetForPhysPtr_Incoming() --
//------------------------------------------------------------------------------
//  Find integer offset from base of Incoming Data of this Phys Memory Pointer

ii32  StandardPacketBmManager::FindOffsetForPhysPtr_Incoming( ii32 phys_addr )
{
    ii32 * base_ptr = BusMasterMemory_Incoming.PhysAddr;
    ii64 full_phys_addr = phys_addr;

    ii64 raw_base = reinterpret_cast<ii64>(base_ptr);
    if ((raw_base>>32) != 0)
        {
        ii64 raw_msw = raw_base>>32;
        ii64 raw_lsw = raw_base&0xFFFFFFFF;
        // if address spans a seam, add one to the msw
        ii64 phys_msw = (raw_lsw<phys_addr) ? raw_msw : raw_msw+1;
        full_phys_addr |= phys_msw<<32;
        }
    ii32 * target_ptr = reinterpret_cast<ii32 *>(full_phys_addr);  // current pointer

    ptrdiff_t offset = target_ptr - base_ptr;
    return static_cast<ii32>(offset);
}


//------------------------------------------------------------------------------
//  StandardPacketBmManager::FindOffsetForPhysPtr_Outgoing() --
//------------------------------------------------------------------------------
//  Find integer offset from base of Outgoing Data of this Phys Memory Pointer

ii32  StandardPacketBmManager::FindOffsetForPhysPtr_Outgoing( ii32 phys_addr )
{
    ii32 * base_ptr = BusMasterMemory_Outgoing.PhysAddr;
    ii64 full_phys_addr = phys_addr;

    ii64 raw_base = reinterpret_cast<ii64>(base_ptr);
    if ((raw_base>>32) != 0)
        {
        ii64 raw_msw = raw_base>>32;
        ii64 raw_lsw = raw_base&0xFFFFFFFF;
        // if address spans a seam, add one to the msw
        ii64 phys_msw = (raw_lsw<phys_addr) ? raw_msw : raw_msw+1;
        full_phys_addr |= phys_msw<<32;
        }
    ii32 * target_ptr = reinterpret_cast<ii32 *>(full_phys_addr);  // current pointer

    ptrdiff_t offset = target_ptr - base_ptr;
    return static_cast<ii32>(offset);
}

//=============================================================================
//  CLASS  StandardPacketBmManager::StandardPMUI  --  UI Classes
//=============================================================================
//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::IncomingBusMasterAddress()
//------------------------------------------------------------------------------
//  Base Address of Memory Region

ii32 *  StandardPacketBmManager::StandardPMUI::IncomingBusMasterAddress() const
{
    return PBM->BusMasterMemory_Incoming.Addr;
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::OutgoingBusMasterAddress()
//------------------------------------------------------------------------------
//  Base Address of Memory Region

ii32 *  StandardPacketBmManager::StandardPMUI::OutgoingBusMasterAddress() const
{
    return PBM->BusMasterMemory_Outgoing.Addr;
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::IncomingPhysicalAddress()
//------------------------------------------------------------------------------
//  Physical Address of Memory Region

ii32 *  StandardPacketBmManager::StandardPMUI::IncomingPhysicalAddress() const
{
    return PBM->BusMasterMemory_Incoming.PhysAddr;
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::OutgoingPhysicalAddress()
//------------------------------------------------------------------------------
//  Physical Address of Memory Region

ii32 *  StandardPacketBmManager::StandardPMUI::OutgoingPhysicalAddress() const
{
    return PBM->BusMasterMemory_Outgoing.PhysAddr;
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::IncomingPhysicalMemOffset()
//------------------------------------------------------------------------------
//  Find offset of address in Incoming Region

ii32  StandardPacketBmManager::StandardPMUI::IncomingPhysicalMemOffset(ii32 phys_addr)
{
    return PBM->FindOffsetForPhysPtr_Incoming( phys_addr );
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::OutgoingPhysicalMemOffset()
//------------------------------------------------------------------------------
//  Find offset of address in Outgoing Region

ii32  StandardPacketBmManager::StandardPMUI::OutgoingPhysicalMemOffset(ii32 phys_addr)
{
    return PBM->FindOffsetForPhysPtr_Outgoing( phys_addr );
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::ReapplyDeviceInfo()
//------------------------------------------------------------------------------
//  Reapply device info

void  StandardPacketBmManager::StandardPMUI::ReapplyDeviceInfo()
{
    PBM->ApplyDeviceInfo();
}

//------------------------------------------------------------------------------
//  StandardPacketBmManager::StandardPMUI::PeriodicUpdate()
//------------------------------------------------------------------------------
//  "in-flight update" callbacks

void  StandardPacketBmManager::StandardPMUI::PeriodicUpdate()
{
   // do nothing
}

//=============================================================================
//  CLASS  ScatterGatherPacketBmManager  --  Manage BM Memory Alloc & HW Load
//=============================================================================
//  Implementation for 'Scatter/Gather' Packet Stream devices:
//      -- One contiguous region in each direction
//      -- Regions allocated by WinDriver
//---------------------------------------------------------------------------
//  constructor for class ScatterGatherPacketBmManager()
//---------------------------------------------------------------------------
//  note - we don't own the MDC.
#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

ScatterGatherPacketBmManager::ScatterGatherPacketBmManager(IPmRequires_PortBaseAddr * mdc,
                                   Osal::OsDevice & device, MapRegisters & regs)
    : MDC(mdc), Device(device), MyUI(this),
      PageSize_Incoming(PortMemorySpace, regs.IncomingPageSize_Addr),
      PageSize_Outgoing(PortMemorySpace, regs.OutgoingPageSize_Addr),
      PageQueueLSW_Incoming(PortMemorySpace, regs.IncomingPageQueueLSW_Addr),
      PageQueueLSW_Outgoing(PortMemorySpace, regs.OutgoingPageQueueLSW_Addr),
      PageQueueMSW_Incoming(PortMemorySpace, regs.IncomingPageQueueMSW_Addr),
      PageQueueMSW_Outgoing(PortMemorySpace, regs.OutgoingPageQueueMSW_Addr),
      PageQueueSize_Incoming(PortMemorySpace, regs.IncomingPageQueueSize_Addr),
      PageQueueSize_Outgoing(PortMemorySpace, regs.OutgoingPageQueueSize_Addr)
{
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::AllocateRegion() -- Scatter/Gather Allocation
//------------------------------------------------------------------------------
//  Allocate using scatter gather region

void  ScatterGatherPacketBmManager::AllocateRegion(BmRegions region, ii32 region_size_words)
{
    //Device.AllocateScatterBlock(GetScatterBlock(region), region_size_words*sizeof(int));
    Device.AllocateMemoryBlock(GetBmMemoryBlock(region), GetPageQueueSize(region, region_size_words)*sizeof(ii64));
    LoadBusMasterPageList(region);
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::FreeRegion() --  Scatter/Gather Allocation
//------------------------------------------------------------------------------
//  Free using scatter gather region

void  ScatterGatherPacketBmManager::FreeRegion(BmRegions region)
{
    Device.FreeMemoryBlock(GetBmMemoryBlock(region));
    //Device.FreeScatterBlock(GetScatterBlock(region));
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::RequestResources() -- Request custom Resources
//------------------------------------------------------------------------------

void  ScatterGatherPacketBmManager::RequestResources()
{
   PortMemorySpace.Link( MDC->GetPortBaseAddr() );
}

//---------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ApplyDeviceInfo() - Apply Region info to hdw
//---------------------------------------------------------------------------

void  ScatterGatherPacketBmManager::ApplyDeviceInfo()
{
    //  Set page size register
    PageSize_Incoming.Value(GetPageSizeInts(bmIncoming));
    PageSize_Outgoing.Value(GetPageSizeInts(bmOutgoing));

    SetPageListAddress(bmIncoming);
    SetPageListAddress(bmOutgoing);

    // Set page list size register
    PageQueueSize_Incoming.Value(static_cast<ii32>(GetScatterBlock(bmIncoming).Pages()));
    PageQueueSize_Outgoing.Value(static_cast<ii32>(GetScatterBlock(bmOutgoing).Pages()));
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::GetScatterBlock() -- return block for region
//------------------------------------------------------------------------------

ScatterBlock & ScatterGatherPacketBmManager::GetScatterBlock( BmRegions which )
{
    switch (which)
        {
        default:
        case bmIncoming:
            return BusMasterMemory_Incoming;
        case bmOutgoing:
            return BusMasterMemory_Outgoing;
        }
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::GetBmMemoryBlock() -- return block for region
//------------------------------------------------------------------------------

MemoryBlock & ScatterGatherPacketBmManager::GetBmMemoryBlock( BmRegions which )
{
    switch (which)
        {
        default:
        case bmIncoming:
            return BusMasterPageList_Incoming;
        case bmOutgoing:
            return BusMasterPageList_Outgoing;
        }
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::GetPageSizeInts() -- Size of memory pages
//------------------------------------------------------------------------------
//  Return page size

unsigned int  ScatterGatherPacketBmManager::GetPageSizeInts( BmRegions which )
{
    switch (which)
        {
        default:
        case bmIncoming:
            return (ii32)BusMasterMemory_Incoming.PageSize/sizeof(int);
        case bmOutgoing:
            return (ii32)BusMasterMemory_Outgoing.PageSize/sizeof(int);
        }
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::GetPageQueueSize() -- Size of Page Block
//------------------------------------------------------------------------------
//  Number of entries in the Page Queue

unsigned int  ScatterGatherPacketBmManager::GetPageQueueSize(BmRegions which, unsigned int region_size_words)
{
    return region_size_words / GetPageSizeInts(which);
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::SetPageListAddress() -- Load Page Address
//------------------------------------------------------------------------------
//  Number of entries in the Page Queue

void  ScatterGatherPacketBmManager::SetPageListAddress(BmRegions which)
{
    switch (which)
        {
        default:
        case bmIncoming:
            {
            ii64 PhysAddr = reinterpret_cast<ii64>(GetBmMemoryBlock(which).PhysAddr);
            PageQueueMSW_Incoming.Value(static_cast<ii32>(PhysAddr>>32));            // Load MSW, then LSW
            PageQueueLSW_Incoming.Value(static_cast<ii32>(PhysAddr&0xFFFFFFFF));
            }
        case bmOutgoing:
            {
            ii64 PhysAddr = reinterpret_cast<ii64>(GetBmMemoryBlock(which).PhysAddr);
            PageQueueMSW_Outgoing.Value(static_cast<ii32>(PhysAddr>>32));            // Load MSW, then LSW
            PageQueueLSW_Outgoing.Value(static_cast<ii32>(PhysAddr&0xFFFFFFFF));
            }
        }
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::LoadBusMasterPageList() -- Load Pages
//------------------------------------------------------------------------------
//  Load Page List into Bm List Memory

void  ScatterGatherPacketBmManager::LoadBusMasterPageList(BmRegions which)
{
    MemoryBlock & MB(GetBmMemoryBlock(which));
    ScatterBlock & SB(GetScatterBlock(which));

    ii64 * Destination = reinterpret_cast<ii64 *>(MB.Addr);

    size_t page_queue_size = SB.Pages();
    SB.Reset();
    for (unsigned int i=0; i<page_queue_size; i++)
        {
        *Destination++ = static_cast<ii64>(SB.NextPageAddr());
        SB.Increment();
        }
}


//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::FindOffsetForPhysPtr_Incoming() --
//------------------------------------------------------------------------------
//  Find integer offset from base of Incoming Data of this Phys Memory Pointer

ii32  ScatterGatherPacketBmManager::FindOffsetForPhysPtr_Incoming( ii32 phys_addr )
{
    ii32 offset = FindOffsetForPhysPtr(GetScatterBlock(bmIncoming), phys_addr );
    return offset;
}


//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::FindOffsetForPhysPtr_Outgoing() --
//------------------------------------------------------------------------------
//  Find integer offset from base of Outgoing Data of this Phys Memory Pointer

ii32  ScatterGatherPacketBmManager::FindOffsetForPhysPtr_Outgoing( ii32 phys_addr )
{
    ii32 offset = FindOffsetForPhysPtr(GetScatterBlock(bmOutgoing), phys_addr );
    return offset;
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::FindOffsetForPhysPtr_Outgoing() --
//------------------------------------------------------------------------------
//  Find integer offset from base of Outgoing Data of this Phys Memory Pointer

ii32  ScatterGatherPacketBmManager::FindOffsetForPhysPtr( ScatterBlock & block, ii32 phys_addr )
{
    size_t page_size = block.PageSize;
    for (size_t page=0; page<block.Pages(); page++)
        {
        size_t page_phys_addr = block.PageList[page];
        if ((page_phys_addr <= phys_addr) &&
                (phys_addr < page_phys_addr+page_size))
            {
            // address "In" this page, return offset
            ii32 offset = static_cast<ii32>(page*page_size + (phys_addr-page_phys_addr));
            return offset;
            }
        }
    //  address not in *any* page, return 0.
    return 0;
}

//=============================================================================
//  CLASS  ScatterGatherPacketBmManager::ScatterGatherPMUI  --  UI Classes
//=============================================================================
//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::IncomingBusMasterAddress()
//------------------------------------------------------------------------------
//  Base Address of Memory Region

ii32 *  ScatterGatherPacketBmManager::ScatterGatherPMUI::IncomingBusMasterAddress() const
{
    return PBM->BusMasterMemory_Incoming.Memory.Address();
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::OutgoingBusMasterAddress()
//------------------------------------------------------------------------------
//  Base Address of Memory Region

ii32 *  ScatterGatherPacketBmManager::ScatterGatherPMUI::OutgoingBusMasterAddress() const
{
    return PBM->BusMasterMemory_Outgoing.Memory.Address();
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::IncomingPhysicalAddress()
//------------------------------------------------------------------------------
//  Physical Address of Memory Region

ii32 *  ScatterGatherPacketBmManager::ScatterGatherPMUI::IncomingPhysicalAddress() const
{
    return 0; // Not applicable
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::OutgoingPhysicalAddress()
//------------------------------------------------------------------------------
//  Physical Address of Memory Region

ii32 *  ScatterGatherPacketBmManager::ScatterGatherPMUI::OutgoingPhysicalAddress() const
{
    return 0; // Not applicable
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::IncomingPhysicalMemOffset()
//------------------------------------------------------------------------------
//  Find offset of address in Incoming Region

ii32  ScatterGatherPacketBmManager::ScatterGatherPMUI::IncomingPhysicalMemOffset(ii32 phys_addr)
{
    return PBM->FindOffsetForPhysPtr_Incoming( phys_addr );
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::OutgoingPhysicalMemOffset()
//------------------------------------------------------------------------------
//  Find offset of address in Outgoing Region

ii32  ScatterGatherPacketBmManager::ScatterGatherPMUI::OutgoingPhysicalMemOffset(ii32 phys_addr)
{
    return PBM->FindOffsetForPhysPtr_Outgoing( phys_addr );
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::ReapplyDeviceInfo()
//------------------------------------------------------------------------------
//  Reapply device info

void  ScatterGatherPacketBmManager::ScatterGatherPMUI::ReapplyDeviceInfo()
{
    PBM->ApplyDeviceInfo();
}

//------------------------------------------------------------------------------
//  ScatterGatherPacketBmManager::ScatterGatherPMUI::PeriodicUpdate()
//------------------------------------------------------------------------------
//  "in-flight update" callbacks

void  ScatterGatherPacketBmManager::ScatterGatherPMUI::PeriodicUpdate()
{
    //PBM->PushPages_Incoming();
    //PBM->PushPages_Outgoing();
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif


#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS  StandardMcbmHardwareManager  --  Manage Busmaster Memory Alloc & HW Load
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class StandardMcbmHardwareManager
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//  constructor for class StandardMcbmHardwareManager()
//---------------------------------------------------------------------------
//  note - we don't own the MDC.
#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace MultiChanStandardMemoryMap;

StandardMcbmHardwareManager::StandardMcbmHardwareManager(IPmRequires_MultiChanDevice * mdc)
    :  MDC(mdc), //Device(device), MyUI(this),
       XuCsrReg(PortMemorySpace, mmXuControlStatusRegister),
       XuInterruptEnableReg(PortMemorySpace, mmXuInterruptEnableRegister),
       XuIntQueuesDoneReg(PortMemorySpace, mmXuInterruptQueuesDone),

       XuRxChannelSelectReg(PortMemorySpace, mmXuRxChannelSelect),
       XuRxBusmasterLoAddrReg(PortMemorySpace, mmXuRxBaseAddr_Low),  // uses select
       XuRxBusmasterHiAddrReg(PortMemorySpace, mmXuRxBaseAddr_High), // uses select
       XuRxBusmasterSizeBytesReg(PortMemorySpace, mmXuRxBufferSize), // uses select
       XuRxBusmasterNotifySizeReg(PortMemorySpace, mmXuRxNotifySize), // uses select

       XuTxChannelSelectReg(PortMemorySpace, mmXuTxChannelSelect),
       XuTxBusmasterLoAddrReg(PortMemorySpace, mmXuTxBaseAddr_Low),   // uses select
       XuTxBusmasterHiAddrReg(PortMemorySpace, mmXuTxBaseAddr_High),  // uses select
       XuTxBusmasterSizeBytesReg(PortMemorySpace, mmXuTxBufferSize),  // uses select
       XuTxBusmasterNotifySizeReg(PortMemorySpace, mmXuTxNotifySize), // uses select

       XuRxCreditAddReg(PortMemorySpace, mmXuRxCreditAdd),
       XuRxCreditReadReg(PortMemorySpace, mmXuRxCreditAdd),
       XuTxCreditAddReg(PortMemorySpace, mmXuTxCreditAdd),
       XuTxCreditReadReg(PortMemorySpace, mmXuTxCreditAdd),

       XuRxInterruptSampleQueueReg(PortMemorySpace, mmXuRxInterruptQueue),
       XuRxSampleQDepthReg(PortMemorySpace, mmXuRxInterruptQueueDepth),
       XuRxTimerPeriodReg(PortMemorySpace, mmXuRxInterruptTimeout),
       XuRxTimerEnableReg(PortMemorySpace, mmXuRxInterruptTimeoutEnable),

       XuTxInterruptSampleQueueReg(PortMemorySpace, mmXuTxInterruptQueue),
       XuTxSampleQDepthReg(PortMemorySpace, mmXuTxInterruptQueueDepth),
       XuTxArbRequestLimitReg(PortMemorySpace, mmXuTxArbitratorRequestLimit),

       XuAlertSampleQueueReg(PortMemorySpace, mmXuAlertInterruptQueue),
       XuAlertSampleQDepthReg(PortMemorySpace, mmXuAlertInterruptQueueDepth),
       XuAlertDataReg(PortMemorySpace, mmXuAlertPayloadRead),

       XuExternalSampleQueueReg(PortMemorySpace, mmXuExternalInterruptQueue),
       XuExternalSampleQDepthReg(PortMemorySpace, mmXuExternalInterruptQueueDepth),
       XuExternalIntEnableReg(PortMemorySpace, mmXuExternalInterruptEnable),

       XuDeviceLinkCSRReg(PortMemorySpace, mmXuDeviceLinkCSR),

       RegEnd(0)
{
}
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//------------------------------------------------------------------------------
//  StandardMcbmHardwareManager::RequestResources() -- Request custom Resources
//------------------------------------------------------------------------------

void  StandardMcbmHardwareManager::RequestResources()
{
    PortMemorySpace.Link( MDC->GetPortBaseAddr() );

    // Register Inits
    //XuCsrReg.BusmasterReset.Set();

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::ApplyDeviceInfo() - Apply Region info to hdw
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::ApplyDeviceInfo()
{
    XuCsrReg.BusmasterReset.Clear();  // not in reset
    BusmasteringOff();
    //
    //  Apply Incoming Info
    for (unsigned int chan=0; chan<MDC->GetMaxIncomingRegionCount(); chan++)
        {
        StandardMcbm::ChannelRegionInfo * info = MDC->GetIncomingRegionInfo(chan);

        // set selector register
        XuRxChannelSelectReg.Channel.Value(chan);

        // set size in bytes (must be 4k bound??)
        XuRxBusmasterSizeBytesReg.Value(info->SizeInBytes());

        // set physical address
        ii64 phys_addr = reinterpret_cast<ii64>(info->PhysAddr());
        XuRxBusmasterLoAddrReg.Value(phys_addr & 0xFFFFFFFF);
        XuRxBusmasterHiAddrReg.Value((phys_addr>>32) & 0xF);
        }
    //
    //  Apply Outgoing Info
    for (unsigned int chan=0; chan<MDC->GetMaxOutgoingRegionCount(); chan++)
        {
        StandardMcbm::ChannelRegionInfo * info = MDC->GetOutgoingRegionInfo(chan);

        // set selector register
        XuTxChannelSelectReg.Channel.Value(chan);

        // set size in bytes (must be 4k bound??)
        XuTxBusmasterSizeBytesReg.Value(info->SizeInBytes());

        // set physical address
        ii64 phys_addr = reinterpret_cast<ii64>(info->PhysAddr());
        XuTxBusmasterLoAddrReg.Value(phys_addr & 0xFFFFFFFF);
        XuTxBusmasterHiAddrReg.Value((phys_addr>>32) & 0xF);
        }
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::ApplyBusmasterNotifySizes() -
//---------------------------------------------------------------------------
//  Apply Busmaster Notify sizes to Hardware i.e. "Packet Sizes"

void  StandardMcbmHardwareManager::ApplyBusmasterNotifySizes()
{
    //
    //  Apply Incoming Info
    for (unsigned int chan=0; chan<MDC->GetMaxIncomingRegionCount(); chan++)
        {
        StandardMcbm::ChannelRegionInfo * info = MDC->GetIncomingRegionInfo(chan);

        // set selector register
        XuRxChannelSelectReg.Channel.Value(chan);

        // set size in WORDs (must be 1k bound??)
        XuRxBusmasterNotifySizeReg.Size.Value(info->NotifySize());
        }
    //
    //  Apply Outgoing Info
    for (unsigned int chan=0; chan<MDC->GetMaxOutgoingRegionCount(); chan++)
        {
        StandardMcbm::ChannelRegionInfo * info = MDC->GetOutgoingRegionInfo(chan);

        // set selector register
        XuTxChannelSelectReg.Channel.Value(chan);

        // set size in WORDs (must be 1k bound??)
        XuTxBusmasterNotifySizeReg.Size.Value(info->NotifySize());
        }
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::Led()
//---------------------------------------------------------------------------

bool  StandardMcbmHardwareManager::Led()
{
    return XuCsrReg.Led.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::Led()
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::Led( bool value )
{
    XuCsrReg.Led = value ? true : false;
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::BusmasteringOn() -- Turn On Busmastering
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::BusmasteringOn()
{
    //
    //  Begin full duplex busmastering
    XuCsrReg.AllowBusmastering.Set();
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::BusmasteringOff() -- Turn Off Busmastering
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::BusmasteringOff()
{
    //
    //  End full duplex busmastering
    XuCsrReg.AllowBusmastering.Clear();
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::BusmasterReset() -- prep for Busmaster
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::BusmasterReset()
{
    XuCsrReg.BusmasterReset.Set();
    Innovative::Sleep(10);
    XuCsrReg.BusmasterReset.Clear();
    Innovative::Sleep(10);
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::MasterReset() - "power on" reset
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::MasterReset()
{
    XuCsrReg.MasterReset.Set();
    Innovative::Sleep(10);
    XuCsrReg.MasterReset.Clear();
    Innovative::Sleep(10);
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::EnableBusmasterInterrupt() -- BM Ints
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::EnableBusmasterInterrupt( bool enable )
{
    if ( enable )
        {
        XuInterruptEnableReg.RxData.Set();
        XuInterruptEnableReg.TxData.Set();
        XuInterruptEnableReg.Alerts.Set();
        XuInterruptEnableReg.External.Set();
        }
    else
        {
        XuInterruptEnableReg.RxData.Clear();
        XuInterruptEnableReg.TxData.Clear();
        XuInterruptEnableReg.Alerts.Clear();
        XuInterruptEnableReg.External.Clear();
        }
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::IssueInitialRecvCredit() -
//---------------------------------------------------------------------------
//  Issue credit to all enabled recv channels.

void  StandardMcbmHardwareManager::IssueInitialRecvCredit()
{
    for (size_t chan=0; chan<MDC->GetInUseIncomingRegionCount(); chan++)
        {
        StandardMcbm::ChannelRegionInfo *  CRI = MDC->GetIncomingRegionInfo(chan);

        if (CRI->Allocated)
            {
            // Give slightly less credit to avoid full/empty confusion in ring
            GrantIncomingCredit(chan, CRI->SizeInWords());
            }
        }

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GrantIncomingCredit() -
//---------------------------------------------------------------------------
//  Issue credit to one of the Rx channels

void  StandardMcbmHardwareManager::GrantIncomingCredit(size_t chan, unsigned int credits)
{
    Innovative::CriticalSectionArea  guard(RxCreditLock);

    XuRxCreditAddReg.Channel.Value(static_cast<ii32>(chan));
    XuRxCreditAddReg.Credit.Value(credits);

    XuRxCreditAddReg.Apply();    // actally apply to hardware

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GrantOutgoingCredit() -
//---------------------------------------------------------------------------
//  Issue credit to one of the Tx channels

void  StandardMcbmHardwareManager::GrantOutgoingCredit(size_t chan, unsigned int credits)
{
    Innovative::CriticalSectionArea  guard(TxCreditLock);

    XuTxCreditAddReg.Channel.Value(static_cast<ii32>(chan));
    XuTxCreditAddReg.Credit.Value(credits);

    XuTxCreditAddReg.Apply();    // actally apply to hardware

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GetCurrentIncomingCredit() - Get Image Headers for Recv INT
//---------------------------------------------------------------------------
//  Report credit level on remote hardware

size_t StandardMcbmHardwareManager::GetCurrentIncomingCredit(size_t chan)
{
    Innovative::CriticalSectionArea  guard(RxCreditLock);

    // set selector register
    XuRxChannelSelectReg.Channel.Value(static_cast<ii32>(chan));

    return XuRxCreditReadReg.Credit.Value();

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GetCurrentOutgoingCredit() -
//---------------------------------------------------------------------------
//  Report credit level on remote hardware

size_t StandardMcbmHardwareManager::GetCurrentOutgoingCredit(size_t chan)
{
    Innovative::CriticalSectionArea  guard(TxCreditLock);

    // set selector register
    XuTxChannelSelectReg.Channel.Value(static_cast<ii32>(chan));

    return XuTxCreditReadReg.Credit.Value();

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GetIncomingQueueData() - Get Image Headers for Recv INT
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::GetIncomingQueueData(Innovative::McImagePack & images)
{
    size_t incoming_words = XuRxSampleQDepthReg.Value();

    // One Queue Word per packet

    images.clear();
    for (size_t i=0; i<incoming_words; i++)
        {
        unsigned int hdr_0 = XuRxInterruptSampleQueueReg.Value();  // read header word

        Innovative::McImage pkt;
        Innovative::AccessDatagram<unsigned int> HdrDG(pkt.Header());
        //
        //  Fill Header (only 1)
        HdrDG[0] = hdr_0;
        //
        //  Add image stub to list
        images.push_back(pkt);
        }

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GetOutgoingQueueData() - Get Image Headers for Xmit INT
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::GetOutgoingQueueData(Innovative::McImagePack& images)
{
    size_t outgoing_words = XuTxSampleQDepthReg.Value();

    // One Queue Word per packet

    images.clear();
    for (size_t i=0; i<outgoing_words; i++)
        {
        unsigned int hdr_0 = XuTxInterruptSampleQueueReg.Value();  // read header word

        Innovative::McImage pkt;
        Innovative::AccessDatagram<unsigned int> HdrDG(pkt.Header());
        //
        //  Fill Header (only 1)
        HdrDG[0] = hdr_0;
        //
        //  Add image stub to list
        images.push_back(pkt);
        }

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GetAlertQueueData() - Get Alerts for INT
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::GetAlertQueueData(Innovative::McAlertPack& alerts)
{
    size_t alert_count = XuAlertSampleQDepthReg.QDepth.Value();

    alerts.clear();
    for (size_t i=0; i<alert_count; i++)
        {
        Innovative::McAlert alert;
        LoadAlertData(alert);
        //
        //  Add object to list to list
        alerts.push_back(alert);
        }

}
//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::LoadAlertData() -
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::LoadAlertData(Innovative::IDatagrammable & store)
{
    size_t alert_words = XuAlertSampleQueueReg.Value();  // must be >=3 ?

    Innovative::AccessDatagram<unsigned int> Alert(store);

    Alert.Resize(alert_words);
    size_t limit = std::min(Alert.size(), alert_words);

    for (size_t i=0; i<limit; i++)
        {
        Alert[i] = XuAlertDataReg.Value();
        }

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::LinkRate() - Return link rate in "Gens"
//---------------------------------------------------------------------------

int StandardMcbmHardwareManager::LinkRate()
{
    unsigned int rate_code = XuDeviceLinkCSRReg.LinkRate.Value();
    switch (rate_code)
        {
        case 1:
        case 2:
            return rate_code;
        case 4:
            return 3;
        default:
            return 0;  // Gen 0 = error!
        }
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::LinkWidth() - return Link Width in Bits
//---------------------------------------------------------------------------

int StandardMcbmHardwareManager::LinkWidth()
{
    unsigned int rate = XuDeviceLinkCSRReg.LinkWidth.Value();
    if (rate>16)
        return 0;
    else
        return rate;  // in bits

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::GetExtIntQueueData() - Get Ext Int Data
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::GetExtIntQueueData(Innovative::McExternalIntPack & extInts)
{
    size_t ext_int_count = XuExternalSampleQDepthReg.Value();

    // Two Queue Words per ext int

    extInts.clear();
    for (size_t i=0; i<ext_int_count; i++)
        {
        unsigned int data_0 = XuExternalSampleQueueReg.Value();  // read data 0

        Innovative::McExternalInt ext_int_data(data_0);
        //
        //  Add object to list
        extInts.push_back(ext_int_data);
        }

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::EnableExtInterrupt() -
//---------------------------------------------------------------------------

void  StandardMcbmHardwareManager::EnableExtInterrupt(int ext_int, ExtIntTypes which, bool state)
{
    switch (which)
        {
        default:
        case eiRisingEdge:
            XuExternalIntEnableReg.RisingEdgeInt.Bit(ext_int, state);
            break;
        case eiFallingEdge:
            XuExternalIntEnableReg.FallingEdgeInt.Bit(ext_int, state);
            break;
        case eiBoth:
            XuExternalIntEnableReg.RisingEdgeInt.Bit(ext_int, state);
            XuExternalIntEnableReg.FallingEdgeInt.Bit(ext_int, state);
            break;
        }

}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::EnableChannelLoopback() -
//---------------------------------------------------------------------------
//  Enables looping back TX stream to RX stream for data testing

void StandardMcbmHardwareManager::EnableChannelLoopback(bool state)
{
    XuCsrReg.ChannelLoopbackEnable.Value(state);
}

//---------------------------------------------------------------------------
//  StandardMcbmHardwareManager::SignalIntQueuesDone() - Signal for next INT
//---------------------------------------------------------------------------

void StandardMcbmHardwareManager::SignalIntQueuesDone()
{
    XuIntQueuesDoneReg.Value(0);
}



} // namespace InnovativeKernel

