//  DeviceDriverBmManager_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//
#ifndef DeviceDriverBmManager_MbH
#define DeviceDriverBmManager_MbH

#include <DataTypes_Mb.h>
#include <Osal_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SyncObjects_Mb.h>
#include <AccessDatagrams_Mb.h>

#include <MultiChanStreamSupport_Mb.h>

//---------------------------------------------------------------------------

namespace InnovativeKernel
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS  IDeviceDriverBmManagerUI  --  Manage Busmaster Memory Alloc & HW Load
//=============================================================================
//  Interface for the UI client (streaming classes)

class IDeviceDriverBmManagerUI
{
public:

    // Raw "app" address pointers
    virtual ii32 *  IncomingBusMasterAddress() const = 0;
    virtual ii32 *  OutgoingBusMasterAddress() const = 0;

    virtual ii32 *  IncomingPhysicalAddress() const = 0;
    virtual ii32 *  OutgoingPhysicalAddress() const = 0;

    virtual ii32    IncomingPhysicalMemOffset( ii32 phys_addr ) = 0;
    virtual ii32    OutgoingPhysicalMemOffset( ii32 phys_addr ) = 0;

    virtual void    ReapplyDeviceInfo() = 0;  // "Stream Configure" Info Load
    virtual void    PeriodicUpdate() = 0;     // "in-flight update call

    virtual ~IDeviceDriverBmManagerUI()
        {}
};

//=============================================================================
//  CLASS  IDeviceDriverBmManager  --  Manage Busmaster Memory Alloc & HW Load
//=============================================================================
//  Interface for "Device Drivers"

class IDeviceDriverBmManager
{
public:
    enum BmRegions { bmIncoming, bmOutgoing };

    virtual IDeviceDriverBmManagerUI &  UI() = 0;  // UI for driver clients

    virtual void  AllocateRegion(BmRegions region, ii32 region_size_words) = 0;
    virtual void  FreeRegion(BmRegions region) = 0;

    virtual void  RequestResources() = 0;  // call back for needed info

    virtual void  ApplyDeviceInfo() = 0;   // "After Device Open" Info Load

    virtual ~IDeviceDriverBmManager()
        {}
};

//=============================================================================
//  CLASS  IPmRequires_PortBaseAddr() --  "Requirements" class
//=============================================================================

class IPmRequires_PortBaseAddr
{
public:
    virtual ii32 *  GetPortBaseAddr() = 0;

    virtual ~IPmRequires_PortBaseAddr()
        {}
};


//=============================================================================
//  CLASS  IMcbmHardwareManager  --  Multi-Chan register block interface
//=============================================================================
//  Interface for "Device Drivers"

class IMcbmHardwareManager
{
public:
    enum  ExtIntTypes  { eiRisingEdge, eiFallingEdge, eiBoth };

    virtual void  RequestResources() = 0;  // call back for needed info
    virtual void  ApplyDeviceInfo() = 0;   // "After Device Open" Info Load

    virtual void  GetIncomingQueueData(Innovative::McImagePack & images) = 0;
    virtual void  GetOutgoingQueueData(Innovative::McImagePack & images) = 0;
    virtual void  GetAlertQueueData(Innovative::McAlertPack & alerts) = 0;
    virtual void  GetExtIntQueueData(Innovative::McExternalIntPack & extInts) = 0;
    virtual void  SignalIntQueuesDone() = 0;
    virtual void  GrantIncomingCredit(size_t chan, unsigned int credits) = 0;
    virtual void  GrantOutgoingCredit(size_t chan, unsigned int credits) = 0;

    virtual size_t GetCurrentIncomingCredit(size_t chan) = 0;
    virtual size_t GetCurrentOutgoingCredit(size_t chan) = 0;

    virtual void   EnableExtInterrupt(int ext_int, ExtIntTypes which, bool state) = 0;
    virtual void   EnableChannelLoopback(bool state) = 0;

    virtual ~IMcbmHardwareManager()
        {}
};


//=============================================================================
//  CLASS  StandardPacketBmManager  --  Manage Busmaster Memory Alloc & HW Load
//=============================================================================
//  Implementation for 'normal' Packet Stream devices:
//      -- One contiguous region in each direction
//      -- Regions allocated by WinDriver

class StandardPacketBmManager : public IDeviceDriverBmManager
{
public:
    //=======================================================================
    //  CLASS StandardPacketBmManager::MapRegisters -- Device Reg Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   RingBaseOutgoing_Addr;
        unsigned int   RingBaseIncoming_Addr;
        unsigned int   RingBaseOutgoingMSW_Addr;
        unsigned int   RingBaseIncomingMSW_Addr;
        unsigned int   RingSizeOutgoing_Addr;
        unsigned int   RingSizeIncoming_Addr;
    };

    StandardPacketBmManager(IPmRequires_PortBaseAddr * mdc,
                            Osal::OsDevice & device, MapRegisters & regs);
    virtual ~StandardPacketBmManager()
        {}

    virtual IDeviceDriverBmManagerUI &  UI()
                {  return MyUI;  }

    //
    //  General Device Driver Interface
    virtual void  AllocateRegion(BmRegions region, ii32 region_size_words);
    virtual void  FreeRegion(BmRegions region);
    virtual void  RequestResources();
    virtual void  ApplyDeviceInfo();   

private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  StandardPacketBmManager::StandardPMUI  -- UI Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class StandardPMUI : public IDeviceDriverBmManagerUI
    {
    public:
        // Raw "app" address pointers
        virtual ii32 *  IncomingBusMasterAddress() const;
        virtual ii32 *  OutgoingBusMasterAddress() const;

        virtual ii32 *  IncomingPhysicalAddress() const;
        virtual ii32 *  OutgoingPhysicalAddress() const;

        virtual ii32    IncomingPhysicalMemOffset( ii32 phys_addr );
        virtual ii32    OutgoingPhysicalMemOffset( ii32 phys_addr );

        virtual void    ReapplyDeviceInfo();
        virtual void    PeriodicUpdate();

        StandardPMUI(StandardPacketBmManager * pbm) : PBM(pbm)
            {}
        virtual ~StandardPMUI()
            {}

    private:
        StandardPacketBmManager * PBM;
    };

    friend class StandardPMUI;

    IPmRequires_PortBaseAddr * MDC;
    Osal::OsDevice &           Device;
    StandardPMUI               MyUI;

    MemoryBlock  BusMasterMemory_Incoming;  // Incoming Device BusMaster memory.
    MemoryBlock  BusMasterMemory_Outgoing;  // Outgoing Device BusMaster memory.

    //
    //  Methods
    MemoryBlock &  GetBmMemoryBlock( BmRegions which );
    ii32           FindOffsetForPhysPtr_Incoming( ii32 phys_addr );
    ii32           FindOffsetForPhysPtr_Outgoing( ii32 phys_addr );

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ChannelSizeRegister  --  Packet Streaming "normal" channel reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ChannelSizeRegister : public Innovative::ShadowRegister
    {
        typedef Innovative::ShadowRegister inherited;

    public:
        Innovative::RegisterBitGroup   Size;

    public:
        ChannelSizeRegister( Innovative::IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               Size( *this, 0, 24 )
            {  }
    };

    //  Registers
    Innovative::AddressingSpace  PortMemorySpace;

    Innovative::ShadowRegister  BmRingBaseAddress_Outgoing;
    Innovative::ShadowRegister  BmRingBaseAddress_Incoming;
    Innovative::ShadowRegister  BmRingBaseAddressMSW_Outgoing;
    Innovative::ShadowRegister  BmRingBaseAddressMSW_Incoming;
    ChannelSizeRegister         BmRingSize_Outgoing;
    ChannelSizeRegister         BmRingSize_Incoming;

};

//=============================================================================
//  CLASS  ScatterGatherPacketBmManager  --  Manage BM Memory Alloc & HW Load
//=============================================================================
//  Implementation for 'Scatter/Gather' Packet Stream devices:
//      -- One contiguous region in each direction
//      -- Regions allocated by WinDriver

class ScatterGatherPacketBmManager : public IDeviceDriverBmManager
{
public:
    //=======================================================================
    //  CLASS ScatterGatherPacketBmManager::MapRegisters -- Device Reg Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   IncomingPageSize_Addr;
        unsigned int   OutgoingPageSize_Addr;
        unsigned int   IncomingPageQueueLSW_Addr;
        unsigned int   OutgoingPageQueueLSW_Addr;
        unsigned int   IncomingPageQueueMSW_Addr;
        unsigned int   OutgoingPageQueueMSW_Addr;
        unsigned int   IncomingPageQueueSize_Addr;
        unsigned int   OutgoingPageQueueSize_Addr;
    };

    ScatterGatherPacketBmManager(IPmRequires_PortBaseAddr * mdc,
                            Osal::OsDevice & device, MapRegisters & regs);
    virtual ~ScatterGatherPacketBmManager()
        {}

    virtual IDeviceDriverBmManagerUI &  UI()
                {  return MyUI;  }

    //
    //  General Device Driver Interface
    virtual void  AllocateRegion(BmRegions region, ii32 region_size_words);
    virtual void  FreeRegion(BmRegions region);
    virtual void  RequestResources();
    virtual void  ApplyDeviceInfo();   

private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  ScatterGatherPacketBmManager::ScatterGatherPMUI  -- UI Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ScatterGatherPMUI : public IDeviceDriverBmManagerUI
    {
    public:
        // Raw "app" address pointers
        virtual ii32 *  IncomingBusMasterAddress() const;
        virtual ii32 *  OutgoingBusMasterAddress() const;

        virtual ii32 *  IncomingPhysicalAddress() const;
        virtual ii32 *  OutgoingPhysicalAddress() const;

        virtual ii32    IncomingPhysicalMemOffset( ii32 phys_addr );
        virtual ii32    OutgoingPhysicalMemOffset( ii32 phys_addr );

        virtual void    ReapplyDeviceInfo();
        virtual void    PeriodicUpdate();

        ScatterGatherPMUI(ScatterGatherPacketBmManager * pbm) : PBM(pbm)
            {}
        virtual ~ScatterGatherPMUI()
            {}

    private:
        ScatterGatherPacketBmManager * PBM;
    };

    friend class ScatterGatherPMUI;

    IPmRequires_PortBaseAddr * MDC;
    Osal::OsDevice &           Device;
    ScatterGatherPMUI          MyUI;

    ScatterBlock  BusMasterMemory_Incoming;    // Incoming Device BM memory.
    ScatterBlock  BusMasterMemory_Outgoing;    // Outgoing Device BM memory.
    MemoryBlock   BusMasterPageList_Incoming;  // Incoming Device Page Table memory.
    MemoryBlock   BusMasterPageList_Outgoing;  // Outgoing Device Page Table memory.
    //
    //  Methods
    ScatterBlock &  GetScatterBlock( BmRegions which );
    MemoryBlock &   GetBmMemoryBlock( BmRegions which );
    unsigned int    GetPageSizeInts( BmRegions which );
    unsigned int    GetPageQueueSize(BmRegions which, unsigned int region_size_words);

    void            SetPageListAddress(BmRegions which);

    ii32            FindOffsetForPhysPtr_Incoming( ii32 phys_addr );
    ii32            FindOffsetForPhysPtr_Outgoing( ii32 phys_addr );
    ii32            FindOffsetForPhysPtr( ScatterBlock & block, ii32 phys_addr );

    void            LoadBusMasterPageList(BmRegions which);

    //  Registers
    Innovative::AddressingSpace  PortMemorySpace;

    Innovative::Register    PageSize_Incoming;
    Innovative::Register    PageSize_Outgoing;
    Innovative::Register    PageQueueLSW_Incoming;
    Innovative::Register    PageQueueLSW_Outgoing;
    Innovative::Register    PageQueueMSW_Incoming;
    Innovative::Register    PageQueueMSW_Outgoing;
    Innovative::Register    PageQueueSize_Incoming;
    Innovative::Register    PageQueueSize_Outgoing;

};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS ChannelRegionInfo() --  ChannelRegionScratchpad
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

namespace StandardMcbm
{

class ChannelRegionInfo
{
public:

    MemoryBlock   MB;  // enabled field needed?
    unsigned int  Requested_BMSize;  // in ints
    bool          Allocated;
    unsigned int  NotifyPktSize;

    ChannelRegionInfo() :  Requested_BMSize(0), Allocated(false)
        {}
    virtual ~ChannelRegionInfo()
        {}

    unsigned int  SizeInWords()
        {  return Requested_BMSize;  }
    unsigned int  SizeInBytes()
        {  return SizeInWords()*sizeof(int);  }
    unsigned int  NotifySize()
        {  return NotifyPktSize;  }

    unsigned int *  Addr()
        {  return MB.Addr;  }
    unsigned int *  PhysAddr()
        {  return MB.PhysAddr;  }

private:
};

}  // end namespace StandardMcbm

//=============================================================================
//  CLASS  IPmRequires_MultiChanDevice() --  "Requirements" class
//=============================================================================

class IPmRequires_MultiChanDevice : public IPmRequires_PortBaseAddr
{
public:
    virtual size_t  GetMaxIncomingRegionCount() = 0;
    virtual size_t  GetMaxOutgoingRegionCount() = 0;
    virtual size_t  GetInUseIncomingRegionCount() = 0;
    virtual size_t  GetInUseOutgoingRegionCount() = 0;
    virtual StandardMcbm::ChannelRegionInfo *  GetIncomingRegionInfo(size_t chan) = 0;
    virtual StandardMcbm::ChannelRegionInfo *  GetOutgoingRegionInfo(size_t chan) = 0;
    virtual InnovativeKernel::IMultiChanOwnerDM & IDevMap() = 0;

    virtual ~IPmRequires_MultiChanDevice()
        {}
};

//=============================================================================
//  CLASS  Supplies_MultiChanDevice() --  "Supplies" class
//=============================================================================

class Supplies_MultiChanDevice : public IPmRequires_MultiChanDevice
{
public:
    virtual ~Supplies_MultiChanDevice()
        {}
};

//=============================================================================
//  CLASS  StandardMcbmHardwareManager  --  Manage Registers for Mcbm Boards
//=============================================================================
//  Implementation for 'normal' port memory region hardware

class StandardMcbmHardwareManager : public IMcbmHardwareManager
{
public:
    //=======================================================================
    //  CLASS StandardMcbmHardwareManager::MapRegisters -- Device Reg Offsets
    //=======================================================================
//    class MapRegisters
//    {
//    public:
//        // Hardcoded for now
//    };

    StandardMcbmHardwareManager(IPmRequires_MultiChanDevice* mdc);
    virtual ~StandardMcbmHardwareManager()
        {}

    //
    //  General Device Driver Interface
    virtual void  RequestResources();
    virtual void  ApplyDeviceInfo();
    virtual void  GetIncomingQueueData(Innovative::McImagePack& images);
    virtual void  GetOutgoingQueueData(Innovative::McImagePack& images);
    virtual void  GetAlertQueueData(Innovative::McAlertPack& alerts);
    virtual void  GetExtIntQueueData(Innovative::McExternalIntPack & extInts);
    virtual void  SignalIntQueuesDone();
    virtual void  GrantIncomingCredit(size_t chan, unsigned int credits);
    virtual void  GrantOutgoingCredit(size_t chan, unsigned int credits);
    virtual void  EnableExtInterrupt(int ext_int, ExtIntTypes which, bool state);
    virtual void  EnableChannelLoopback(bool state);

    virtual size_t GetCurrentIncomingCredit(size_t chan);
    virtual size_t GetCurrentOutgoingCredit(size_t chan);

    bool  Led();
    void  Led(bool value);

    void  BusmasteringOn();
    void  BusmasteringOff();

    void  BusmasterReset();
    void  MasterReset();

    void  EnableBusmasterInterrupt( bool enable );

    void  IssueInitialRecvCredit();


    void  ApplyBusmasterNotifySizes();


    //  Alerts
    void          LoadAlertData(Innovative::IDatagrammable & store);

    //  Lane Control/Status
    int   LinkRate();   // return X as in Gen1, GenX.
    int   LinkWidth();  // return 0 if unlinked

private:

    IPmRequires_MultiChanDevice * MDC;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuControlStatusRegister  --  Control/Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuControlStatusRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBit       MasterReset;
        Innovative::RegisterBit       Led;
        Innovative::RegisterBit       BusmasterReset;
        Innovative::RegisterBit       AllowBusmastering;
        Innovative::RegisterBit       BusmasteringStopped;
        Innovative::RegisterBit       ChannelLoopbackEnable;
        Innovative::RegisterBitGroup  DMACore_RevisionCode;
        Innovative::RegisterBitGroup  DMACore_HardwareFamily;

    public:
        XuControlStatusRegister( Innovative::IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               MasterReset( *this, 0 ),
               Led( *this, 1 ),
               BusmasterReset( *this, 3 ),
               AllowBusmastering( *this, 4 ),
               BusmasteringStopped( *this, 5 ),
               ChannelLoopbackEnable( *this, 8),
               DMACore_RevisionCode( *this, 12, 4),
               DMACore_HardwareFamily( *this, 16, 4)
            {
                Led.Set();  // we like this as a default better
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuInterruptEnableRegister  --  Control/Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  NOTE:  InterruptStatusRegister has same format (only read by KerPlug)

    class XuInterruptEnableRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBit       RxData;    // incoming
        Innovative::RegisterBit       TxData;    // outgoing
        Innovative::RegisterBit       Alerts;
        Innovative::RegisterBit       External;
        Innovative::RegisterBit       DisableHostInts;   // test only

    public:
        XuInterruptEnableRegister( Innovative::IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               RxData( *this, 0 ),
               TxData( *this, 1 ),
               Alerts( *this, 2 ),
               External( *this, 3 ),
               DisableHostInts( *this, 4 )
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuChannelSelectorRegister  --  Set Channel to access
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuChannelSelectorRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup  Channel;

    public:
        XuChannelSelectorRegister( Innovative::IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               Channel( *this, 0, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuBusmasterHiAddrRegister  --  Busmaster Addr High Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuBusmasterHiAddrRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup  Msb;

    public:
        XuBusmasterHiAddrRegister( Innovative::IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               Msb( *this, 0, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuBusmasterNotifySizeRegister  --  Busmaster Notify Size Reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuBusmasterNotifySizeRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup  Size;

    public:
        XuBusmasterNotifySizeRegister( Innovative::IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               Size( *this, 0, 28)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuCreditAddRegister  --  Credit Add Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuCreditAddRegister : public Innovative::CachedShadowRegister
    {
        typedef Innovative::CachedShadowRegister inherited;

    public:
        Innovative::RegisterBitGroup   Credit;
        Innovative::RegisterBitGroup   Channel;

    public:
        XuCreditAddRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              Credit( *this, 0, 28 ),
              Channel( *this, 28, 4 )
            {
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuCreditReadbackRegister  --  Credit Report Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuCreditReadbackRegister : public Innovative::ReadOnlyRegister
    {
        typedef Innovative::ReadOnlyRegister inherited;

    public:
        Innovative::ReadOnlyRegisterBitGroup   Credit;
        Innovative::ReadOnlyRegisterBitGroup   Channel;

    public:
        XuCreditReadbackRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              Credit( *this, 0, 28 ),
              Channel( *this, 28, 4 )
            {
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuSampleQDepthRegister  --  Int Queue Depth
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuSampleQDepthRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup   QDepth;

    public:
        XuSampleQDepthRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              QDepth( *this, 0, 9 )
            {
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuRxTimerPeriodRegister  --  Interrupt Period
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuRxTimerPeriodRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup   Period;

    public:
        XuRxTimerPeriodRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              Period( *this, 0, 8 )
            {
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuTxArbRequestLimitRegister  --  Interrupt Period
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuTxArbRequestLimitRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup   RequestLimit;

    public:
        XuTxArbRequestLimitRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              RequestLimit( *this, 0, 8 )
            {
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuExternalIntEnableRegister  --  Ext Interrupt Enable
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuExternalIntEnableRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup   RisingEdgeInt;
        Innovative::RegisterBitGroup   FallingEdgeInt;

    public:
        XuExternalIntEnableRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              RisingEdgeInt( *this, 0, 12 ),
              FallingEdgeInt( *this, 12, 12 )
            {
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XuDeviceLinkCSRRegister  --  Link Information
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class XuDeviceLinkCSRRegister : public Innovative::Register
    {
        typedef Innovative::Register inherited;

    public:
        Innovative::RegisterBitGroup   MaxReadRequestSize;
        Innovative::RegisterBitGroup   MaxPayloadSize;
        Innovative::RegisterBitGroup   LinkRate;
        Innovative::RegisterBitGroup   LinkWidth;

    public:
        XuDeviceLinkCSRRegister( Innovative::IAddressingSpace &space, int offset )
            : inherited( space, offset ),
              MaxReadRequestSize( *this, 14, 3 ),
              MaxPayloadSize( *this, 17, 3 ),
              LinkRate( *this, 23, 3 ),
              LinkWidth( *this, 26, 5 )
            {
            }

    };
    //  Registers
    Innovative::AddressingSpace     PortMemorySpace;

    XuControlStatusRegister         XuCsrReg;
    XuInterruptEnableRegister       XuInterruptEnableReg;
    Innovative::Register            XuIntQueuesDoneReg;

    XuChannelSelectorRegister       XuRxChannelSelectReg;
    Innovative::Register            XuRxBusmasterLoAddrReg;
    XuBusmasterHiAddrRegister       XuRxBusmasterHiAddrReg;
    Innovative::Register            XuRxBusmasterSizeBytesReg;
    XuBusmasterNotifySizeRegister   XuRxBusmasterNotifySizeReg;

    XuChannelSelectorRegister       XuTxChannelSelectReg;
    Innovative::Register            XuTxBusmasterLoAddrReg;
    XuBusmasterHiAddrRegister       XuTxBusmasterHiAddrReg;
    Innovative::Register            XuTxBusmasterSizeBytesReg;
    XuBusmasterNotifySizeRegister   XuTxBusmasterNotifySizeReg;

    XuCreditAddRegister             XuRxCreditAddReg;
    XuCreditReadbackRegister        XuRxCreditReadReg;
    XuCreditAddRegister             XuTxCreditAddReg;
    XuCreditReadbackRegister        XuTxCreditReadReg;

    Innovative::Register            XuRxInterruptSampleQueueReg;
    XuSampleQDepthRegister          XuRxSampleQDepthReg;
    XuRxTimerPeriodRegister         XuRxTimerPeriodReg;
    Innovative::Register            XuRxTimerEnableReg;

    Innovative::Register            XuTxInterruptSampleQueueReg;
    XuSampleQDepthRegister          XuTxSampleQDepthReg;
    XuTxArbRequestLimitRegister     XuTxArbRequestLimitReg;

    Innovative::Register            XuAlertSampleQueueReg;
    XuSampleQDepthRegister          XuAlertSampleQDepthReg;
    Innovative::Register            XuAlertDataReg;

    Innovative::Register            XuExternalSampleQueueReg;
    XuSampleQDepthRegister          XuExternalSampleQDepthReg;
    XuExternalIntEnableRegister     XuExternalIntEnableReg;

    XuDeviceLinkCSRRegister         XuDeviceLinkCSRReg;

    Innovative::CriticalSection  RxCreditLock;
    Innovative::CriticalSection  TxCreditLock;

    int RegEnd;


};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace InnovativeKernel

#endif
