// DeviceDriver_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifdef __BORLANDC__
#include <mem.h>
#endif

#include <DeviceDriver_Mb.h>
#include <vector>
#include <algorithm>
#include <Exception_Mb.h>
#include <StringSupport_Mb.h>
#include <SingletonManager_Mb.h>
#include <sstream>

namespace
{

//=============================================================================
//  CLASS BOARDINFO  --  
//=============================================================================

class BOARDINFO
{

public:
    std::string     Name;
    std::string     DeviceBaseName;
    int             VendorID;
    int             DeviceID;
    unsigned int    SubID;
    bool            CheckSubID;

public:
    BOARDINFO(const std::string & name, const std::string & device_base_name, int vendor_id,
              int device_id, int sub_id, bool check_sub_id) :
    Name(name),
    DeviceBaseName(device_base_name),
    VendorID(vendor_id),
    DeviceID(device_id),
    SubID(sub_id),
    CheckSubID(check_sub_id)
    {
    }

    BOARDINFO(const BOARDINFO&  other) :
    Name(other.Name),
    DeviceBaseName(other.DeviceBaseName),
    VendorID(other.VendorID),
    DeviceID(other.DeviceID),
    SubID(other.SubID),
    CheckSubID(other.CheckSubID)
    {
    }

public:
    bool operator == (const BOARDINFO&other)
        {
        if (Name != other.Name)
            return false;

        if (DeviceBaseName != other.DeviceBaseName)
            return false;

        if (VendorID != other.VendorID)
            return false;

        if (DeviceID != other.DeviceID)
            return false;

        if (SubID != other.SubID)
            return false;

        if (CheckSubID != other.CheckSubID)
            return false;

        return true;
        }

    BOARDINFO &operator=(const BOARDINFO&  other)
    {
        Name = other.Name;
        DeviceBaseName = other.DeviceBaseName;
        VendorID = other.VendorID;
        DeviceID = other.DeviceID;
        SubID = other.SubID;
        CheckSubID = other.CheckSubID;

        return *this;
    }
};

//=============================================================================
//  CLASS BoardInfoListSingleton  --
//=============================================================================

class BoardInfoListSingleton : public OpenWire::SingletonService
{
public:
    std::vector<BOARDINFO> BoardInfoList;

private:
    static BoardInfoListSingleton *Instance;

public:
    static BoardInfoListSingleton *GetInstance()
    {
        if( ! Instance )
            Instance = new BoardInfoListSingleton();

        return Instance;
    }

};

BoardInfoListSingleton *BoardInfoListSingleton::Instance = NULL;

} // namespace


namespace InnovativeKernel
{

//---------------------------------------------------------------------------
//  RegisterPciDevice() --
//---------------------------------------------------------------------------

void RegisterPciDevice(const std::string & name, const std::string & device_base_name,
                       int vendor_id, int device_id, int sub_id,
                       bool check_sub_id)
{
    BOARDINFO   ABoardInfo(name, device_base_name, vendor_id, device_id, sub_id,
                           check_sub_id);
    if (std::find( BoardInfoListSingleton::GetInstance()->BoardInfoList.begin(),
                  BoardInfoListSingleton::GetInstance()->BoardInfoList.end(),
                  ABoardInfo) == BoardInfoListSingleton::GetInstance()->BoardInfoList.end())
        BoardInfoListSingleton::GetInstance()->BoardInfoList.push_back(ABoardInfo);
}

//---------------------------------------------------------------------------
//  BoardEnumData::Count() --
//---------------------------------------------------------------------------

size_t BoardEnumData::Count()
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList.size();
}

//---------------------------------------------------------------------------
//  BoardEnumData::BoardName() --
//---------------------------------------------------------------------------
 
std::string BoardEnumData::BoardName(size_t i)
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList[i].Name;
}

//---------------------------------------------------------------------------
//  BoardEnumData::DriverName() --
//---------------------------------------------------------------------------

std::string BoardEnumData::DriverName(size_t i)
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList[i].DeviceBaseName;
}

//---------------------------------------------------------------------------
//  BoardEnumData::VendorId() --
//---------------------------------------------------------------------------
 
int BoardEnumData::VendorId(size_t i)
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList[i].VendorID;
}

//---------------------------------------------------------------------------
//  BoardEnumData::DeviceId() --
//---------------------------------------------------------------------------
 
int BoardEnumData::DeviceId(size_t i)
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList[i].DeviceID;
}

//---------------------------------------------------------------------------
//  BoardEnumData::SubsystemId() --
//---------------------------------------------------------------------------
 
int BoardEnumData::SubsystemId(size_t i)
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList[i].SubID;
}

//---------------------------------------------------------------------------
//  BoardEnumData::CheckSubsystemId() --
//---------------------------------------------------------------------------
 
bool BoardEnumData::CheckSubsystemId(size_t i)
{
    return BoardInfoListSingleton::GetInstance()->BoardInfoList[i].CheckSubID;
}

//---------------------------------------------------------------------------
//  BoardEnumData::IndexOf() --
//---------------------------------------------------------------------------

int BoardEnumData::IndexOf(const std::string & board_name)
{
    for (size_t i = 0; i < BoardInfoListSingleton::GetInstance()->BoardInfoList.size(); ++i)
        {
        std::string name = BoardName(i);
        if (name == board_name)
            return static_cast<int>(i);

        }

    return -1;
    //throw Innovative::MalibuException("Board name not found");
}

//==============================================================================
//  CLASS StandardOsDeviceDebugUI -- OsDevice Debug Call-up Interface
//==============================================================================

class StandardOsDeviceDebugUI : public Osal::IOsDeviceDebugUI
{
public:
    StandardOsDeviceDebugUI(class DeviceDriver * dd)
        :  DD(dd)
        {}
    virtual ~StandardOsDeviceDebugUI() {}

    virtual void  IsrLog(unsigned int isr_log_data)
        {  DD->AppendToIsrLog(isr_log_data);  }
    virtual void  TraceLog(int code, int data)
        {  DD->AppendToTraceLog(code, data);  }

private:
    DeviceDriver * DD;
};

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif
//=============================================================================
//  CLASS DeviceDriver  --
//=============================================================================
//---------------------------------------------------------------------------
//  constructor for class DeviceDriver()
//---------------------------------------------------------------------------

DeviceDriver::DeviceDriver(const PciId & id, int port_region)
    :  Device(id, new StandardOsDeviceDebugUI(this)), FTarget(0),
       PortRegion(port_region), FOpened(false), DDUI(0)
{
    //Device.scanPci();
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class DeviceDriver()
//---------------------------------------------------------------------------

DeviceDriver::~DeviceDriver()
{
    //  Close once in the most derived class...
    if (FOpened)
        Close();

    if (DDUI)
        {
        delete DDUI;
        DDUI = 0;
        }
}

//---------------------------------------------------------------------------
//  DeviceDriver::BoardCount() -- Number of enumerated boards of this type
//---------------------------------------------------------------------------

size_t DeviceDriver::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  DeviceDriver::Open() -- Open Driver
//---------------------------------------------------------------------------

bool  DeviceDriver::Open()
{
    if ( FOpened )
       return true;
    //
    //  Attempt to open device
    Device.DeviceOpen(FTarget);

    if ( FillCardinfo() == false )
        {
        Device.DeviceClose();
        Error("Failed to fill CardInfo structure.");
        return false;
        }

    //  Tag object as open
    FOpened = true;
    return true;

}

//---------------------------------------------------------------------------
//  DeviceDriver::Close() -- Close Driver
//---------------------------------------------------------------------------

bool  DeviceDriver::Close()
{
    if( ! FOpened )
        return true;

    //
    //  Release all held memory resources allocated on Open().
    ReleaseResources();
    //
    //  Close device
    Device.DeviceClose();

    FOpened = false;
    return true;
}

//----------------------------------------------------------------------------
//  DeviceDriver::Target() -- change target number
//----------------------------------------------------------------------------

void  DeviceDriver::Target( int target )
{
    if ( FTarget == target )
        return;

	bool WasOpened = Opened();
    if ( WasOpened )
        Close();

    FTarget = target;

	if ( WasOpened )
		Open();
}

//---------------------------------------------------------------------------
//  DeviceDriver::FillCardinfo() -- Read Card Information & save
//---------------------------------------------------------------------------

bool  DeviceDriver::FillCardinfo()
{
    //
    //  This "configurator" function can be overridden by derived classes to
    //    specialize the setup of the driver class based on the resources.
    InitializeResources();

	//
    //  Load the PciConfig Device Entry Information
    //Device.ReadPciConfig(PciCfgData);
    //PciCfgData.PositionFrom(Device.BoardLocation());
    
	//
    //  This "configurator" function can be overridden by derived classes to
    //    specialize the setup of the driver class based on the resources.
    FinalizeResources();
    return true;
}

//------------------------------------------------------------------------------
//  DeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//  Remember to call the base class version to init the base class data!
//
void  DeviceDriver::InitializeResources()
{
    //
    //  Init Port Memory Structure
    Device.InitializeMappedMemory(PortMemory, PortRegion);
}
//
//------------------------------------------------------------------------------
//  DeviceDriver::FinalizeResources() -- Complete setup of resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!
//
void  DeviceDriver::FinalizeResources()
{
}

//------------------------------------------------------------------------------
//  DeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  DeviceDriver::ReleaseResources()
{
}

//------------------------------------------------------------------------------
//  DeviceDriver::KpQueueDepth() -- Return depth of KP ISR queue
//------------------------------------------------------------------------------

size_t DeviceDriver::KpQueueDepth()
{
    return Device.KpQueueDepth();
}

//------------------------------------------------------------------------------
//  DeviceDriver::Error() -- throw exception
//------------------------------------------------------------------------------

void  DeviceDriver::Error(const std::string & msg)
{
    //
    //  Precede error text with Board ID code: [Ven, Dev, Targ]
    std::stringstream txt;
    InnovativeKernel::PciId id = Device.BoardId();
    txt << "[" << id.VendorId << ", " << id.DeviceId << ", " << FTarget << "] ";
    txt << msg;
    Device.Error(txt.str());
//    throw Innovative::MalibuException(txt.str());
}

//---------------------------------------------------------------------------
//  DeviceDriver::PortMemoryAddress() -- return Port memory address
//---------------------------------------------------------------------------

ii32 *  DeviceDriver::PortMemoryAddress()
{
    if ( PortMemory.WasMapped )
        return (ii32 *)PortMemory.Addr;
    else
        return NULL;
}

//---------------------------------------------------------------------------
//  DeviceDriver::AppendToIsrLog() -- implement to do ISR Logging
//---------------------------------------------------------------------------

void  DeviceDriver::AppendToIsrLog(unsigned int /*isr_log_data*/)
{
}

//---------------------------------------------------------------------------
//  DeviceDriver::AppendToTraceLog() -- implement to do ISR Logging
//---------------------------------------------------------------------------

void  DeviceDriver::AppendToTraceLog(int code, int data)
{
    if (DDUI)
        DDUI->TraceLog(code, data);
}

//=============================================================================
//  CLASS DualPortDeviceDriver  --
//=============================================================================

const unsigned int DualPortDeviceRegionIdx = 0;

//---------------------------------------------------------------------------
//  constructor for class DualPortDeviceDriver()
//---------------------------------------------------------------------------

DualPortDeviceDriver::DualPortDeviceDriver(const PciId & id, int port_region,
                                            int dual_port_region)
    :  DeviceDriver(id, port_region),
       DualPortRegion(dual_port_region)
{
}

//---------------------------------------------------------------------------
//  destructor for class DualPortDeviceDriver()
//---------------------------------------------------------------------------

DualPortDeviceDriver::~DualPortDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//------------------------------------------------------------------------------
//  DualPortDeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//
void  DualPortDeviceDriver::InitializeResources()
{
    inherited::InitializeResources();
    //
    //  Init DualPort Memory Structure
    Device.InitializeMappedMemory(DualPortMemory, DualPortRegion);
}

//------------------------------------------------------------------------------
//  DualPortDeviceDriver::FinalizeResources() -- Complete setup of memory blks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  DualPortDeviceDriver::FinalizeResources()
{
    inherited::FinalizeResources();
}

//------------------------------------------------------------------------------
//  DualPortDeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  DualPortDeviceDriver::ReleaseResources()
{
    inherited::ReleaseResources();
}

//=============================================================================
//  CLASS IsrDeviceDriver  --
//=============================================================================
//---------------------------------------------------------------------------
//  constructor for class IsrDeviceDriver()
//---------------------------------------------------------------------------

IsrDeviceDriver::IsrDeviceDriver(const PciId & id,
                    unsigned int intStatusByteOffset, unsigned int intStatusMask,
                    unsigned int intStatusRegion, KernelIsrType intHandlerType)
    : DeviceDriver(id),
      mKernelIsrType(intHandlerType), 
      mIntStatusByteOffset(intStatusByteOffset),
      mIntStatusMask(intStatusMask), 
      mIntStatusRegion(intStatusRegion),
      mIsrEvent(Innovative::Event::AutoReset)
{
}

//---------------------------------------------------------------------------
//  IsrDeviceDriver::Open() -- Open Driver
//---------------------------------------------------------------------------

bool IsrDeviceDriver::Open()
{
    if (!DeviceDriver::Open())
        return false;

    // Enable interrupts
    InterruptInstall();
    if (!InterruptEnable())
        {
        Close();
//        Error( mTargetName + ": Failed interrupt enable within open" );
        FOpened = false;
        return false;
        }

    //  Tag object as open
    return true;
}

//---------------------------------------------------------------------------
//  IsrDeviceDriver::Close() -- Close Driver
//---------------------------------------------------------------------------

bool IsrDeviceDriver::Close()
{
    if( ! FOpened )
        return true;

    bool Status = true;

    // Disable interrupts
    Status &= InterruptDisable();
    Status &= InterruptDeinstall();

	mIsrEvent.Clear();

    Status &= inherited::Close();
    return Status;
}

//------------------------------------------------------------------------------
//  IsrDeviceDriver::InterruptEnable() --
//------------------------------------------------------------------------------

bool  IsrDeviceDriver::InterruptEnable()
{
    Device.InterruptEnable();
    return true;
}

//---------------------------------------------------------------------------
//  IsrDeviceDriver::InterruptDisable() --
//------------------------------------------------------------------------------

bool IsrDeviceDriver::InterruptDisable()
{
    Device.InterruptDisable();
    return true;
}

//---------------------------------------------------------------------------
//  IsrDeviceDriver::InterruptDeinstall() --
//------------------------------------------------------------------------------

bool  IsrDeviceDriver::InterruptDeinstall()
{
    Device.InterruptDeinstall();
    return true;
}

//---------------------------------------------------------------------------
//  IsrDeviceDriver::InterruptInstall() --
//------------------------------------------------------------------------------

bool IsrDeviceDriver::InterruptInstall()
{
    //
    //  Create or Load the event fired when the driver sees an Interrupt
    mIsrEvent.Clear();
    //
    //  Set up Int Extension Region for your board
    InitializeInterruptDeviceExtension();
    //
    //  Install Interrupt into driver
    Device.InterruptInstall(&mIsrEvent);
    return true;
}

//----------------------------------------------------------------------------
//   IsrDeviceDriver::InterruptDepth() -- Returns Int Status Queue Depth
//----------------------------------------------------------------------------

size_t  IsrDeviceDriver::InterruptDepth()
{
    size_t temp = Device.IsrQueueDepth();
    return temp;
}

//----------------------------------------------------------------------------
//   IsrDeviceDriver::InterruptAck() -- Extracts a point from the Int Queue
//----------------------------------------------------------------------------

unsigned int   IsrDeviceDriver::InterruptAck()
{
    unsigned int Return = Device.DequeueIsrStatus();
    // Debug
    //mIsrInfo.Log(Return);    // pushed down into OSAL fetch
    return Return;
}

//---------------------------------------------------------------------------
//  IsrDeviceDriver::AppendToIsrLog() -- implement to do ISR Logging
//---------------------------------------------------------------------------

void  IsrDeviceDriver::AppendToIsrLog(unsigned int isr_log_data)
{
    mIsrInfo.Log(isr_log_data);
}

//------------------------------------------------------------------------------
//  IsrDeviceDriver::ClearInterruptStatusQueue() -- Reset Interrupt Queue
//------------------------------------------------------------------------------

void  IsrDeviceDriver::ClearInterruptStatusQueue()
{
    if (FOpened==false)
        return;

    // Reset Queue of Int Status Data
    Device.ClearInterruptQueue();
}

//------------------------------------------------------------------------------
//  IsrDeviceDriver::InitializeInterruptDeviceExtension() -- Init Int Data
//------------------------------------------------------------------------------

void  IsrDeviceDriver::InitializeInterruptDeviceExtension()
{
    //
    //  Initialize Transfer Structure
    Device.InitDeviceExtension(mKernelIsrType, mIntStatusByteOffset,
                                    mIntStatusMask );

}

//------------------------------------------------------------------------------
//  IsrDeviceDriver::EnableDedicatedKernelCacheRelease() --
//------------------------------------------------------------------------------

void  IsrDeviceDriver::EnableDedicatedKernelCacheRelease(Innovative::IAddressingSpace & space, int offset)
{
    //
    //  Pass it down
    Device.EnableDedicatedKernelCacheRelease(space, offset);

}

//=============================================================================
//  CLASS BusMasterDeviceDriver  --
//=============================================================================
//---------------------------------------------------------------------------
//  constructor for class BusMasterDeviceDriver()
//---------------------------------------------------------------------------

BusMasterDeviceDriver::BusMasterDeviceDriver(const PciId & id,
                                             unsigned int intStatusByteOffset,
                                             unsigned int intStatusMask,
                                             unsigned int intStatusRegion,
                                             KernelIsrType intHandlerType )
    : IsrDeviceDriver(id, intStatusByteOffset,
                      intStatusMask, intStatusRegion, intHandlerType),
      FBusMasterSizeWords_Incoming(0x200000/sizeof(int)),
      FBusMasterSizeWords_Outgoing(0x200000/sizeof(int)),
      is64bitCapable(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class BusMasterDeviceDriver()
//---------------------------------------------------------------------------

BusMasterDeviceDriver::~BusMasterDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//  Remember to call the base class version to init the base class data!
//
void  BusMasterDeviceDriver::InitializeResources()
{
    inherited::InitializeResources();

    AllocateBmRegion( bmIncoming, FBusMasterSizeWords_Incoming );
    try
        {
        AllocateBmRegion( bmOutgoing, FBusMasterSizeWords_Outgoing );
        }
    catch (Innovative::MalibuException & /*e*/)
        {
        FreeBmRegion(bmIncoming);
        throw;
        }
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::FinalizeResources() -- Complete setup of resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  BusMasterDeviceDriver::FinalizeResources()
{
    inherited::FinalizeResources();
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  BusMasterDeviceDriver::ReleaseResources()
{
    inherited::ReleaseResources();
    //  Unmap the BusMaster Address before exiting (if need be)
    FreeBmRegion(bmIncoming);
    FreeBmRegion(bmOutgoing);
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::BusMasterSizeWords() --
//------------------------------------------------------------------------------

void BusMasterDeviceDriver::BusMasterSizeWords(unsigned int words)
{
    FBusMasterSizeWords_Incoming = FBusMasterSizeWords_Outgoing = words;
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::IncomingBusMasterSizeWords() --
//------------------------------------------------------------------------------

void BusMasterDeviceDriver::IncomingBusMasterSizeWords(unsigned int words)
{
    FBusMasterSizeWords_Incoming  = words;
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::IncomingBusMasterSizeWords() --
//------------------------------------------------------------------------------

ii32 BusMasterDeviceDriver::IncomingBusMasterSizeWords() const
{
    return FBusMasterSizeWords_Incoming;
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::OutgoingBusMasterSizeWords() --
//------------------------------------------------------------------------------

void BusMasterDeviceDriver::OutgoingBusMasterSizeWords(unsigned int words)
{
    FBusMasterSizeWords_Outgoing = words;
}

//------------------------------------------------------------------------------
//  BusMasterDeviceDriver::OutgoingBusMasterSizeWords() --
//------------------------------------------------------------------------------

ii32 BusMasterDeviceDriver::OutgoingBusMasterSizeWords() const
{
    return FBusMasterSizeWords_Outgoing;
}

//=============================================================================
//  CLASS SimpleBusMasterDeviceDriver  --
//=============================================================================
//---------------------------------------------------------------------------
//  constructor for class SimpleBusMasterDeviceDriver()
//---------------------------------------------------------------------------

SimpleBusMasterDeviceDriver::SimpleBusMasterDeviceDriver(const PciId & id,
                                             unsigned int intStatusByteOffset,
                                             unsigned int intStatusMask,
                                             unsigned int intStatusRegion,
                                             KernelIsrType intHandlerType )
    : BusMasterDeviceDriver(id, intStatusByteOffset,
                      intStatusMask, intStatusRegion, intHandlerType)
{
}

//---------------------------------------------------------------------------
//  destructor for class SimpleBusMasterDeviceDriver()
//---------------------------------------------------------------------------

SimpleBusMasterDeviceDriver::~SimpleBusMasterDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::AllocateBmRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Allocate using standard contiguous BM Region

void  SimpleBusMasterDeviceDriver::AllocateBmRegion(BmRegions region, ii32 region_size_words)
{
    Device.AllocateMemoryBlock(GetBmMemoryBlock(region), region_size_words*sizeof(int), is64bitCapable);
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::FreeBmRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Allocate using standard contiguous BM Region

void  SimpleBusMasterDeviceDriver::FreeBmRegion(BmRegions region)
{
    Device.FreeMemoryBlock(GetBmMemoryBlock(region), is64bitCapable);
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::IncomingBusMasterAddress() --
//------------------------------------------------------------------------------

ii32 * SimpleBusMasterDeviceDriver::IncomingBusMasterAddress() const
{
    return BusMasterMemory_Incoming.Addr;
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::IncomingBusMasterPhysicalAddress() --
//------------------------------------------------------------------------------

ii32 * SimpleBusMasterDeviceDriver::IncomingBusMasterPhysicalAddress() const
{
    return BusMasterMemory_Incoming.PhysAddr;
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::OutgoingBusMasterAddress() --
//------------------------------------------------------------------------------

ii32 * SimpleBusMasterDeviceDriver::OutgoingBusMasterAddress() const
{
    return BusMasterMemory_Outgoing.Addr;
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::OutgoingBusMasterPhysicalAddress() --
//------------------------------------------------------------------------------

ii32 * SimpleBusMasterDeviceDriver::OutgoingBusMasterPhysicalAddress() const
{
    return BusMasterMemory_Outgoing.PhysAddr;
}

//------------------------------------------------------------------------------
//  SimpleBusMasterDeviceDriver::GetBmMemoryBlock() -- return block for region
//------------------------------------------------------------------------------

MemoryBlock & SimpleBusMasterDeviceDriver::GetBmMemoryBlock( BmRegions which )
{
    switch (which)
        {
        default:
        case bmIncoming:
            return BusMasterMemory_Incoming;
        case bmOutgoing:
            return BusMasterMemory_Outgoing;
        }
}

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
//=============================================================================
//  CLASS PacketBusMasterDeviceDriver  -- Base for Packet Stream Devices
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PacketBusMasterDeviceDriver
//---------------------------------------------------------------------------

namespace
{
    //  Normal Memory Map
    const unsigned int  mmBmRingBaseAddress   = 0x15000;
    const unsigned int  mmBmRingSize          = 0x15400;
}

StandardPacketBmManager::MapRegisters  StdPktRegAddrs =
    {
    mmBmRingBaseAddress + 0,   //  RingBaseOutgoing_Addr
    mmBmRingBaseAddress + 1,   //  RingBaseIncoming_Addr
    mmBmRingBaseAddress + 2,   //  RingBaseOutgoingMSW_Addr
    mmBmRingBaseAddress + 3,   //  RingBaseIncomingMSW_Addr
    mmBmRingSize + 0,          //  RingSizeOutgoing_Addr
    mmBmRingSize + 1,          //  RingSizeIncoming_Addr
    };
    
namespace
{
    //  ScatterGather Memory Map
    const unsigned int  mmBmSGPageQueueAddress  = 0x15000;
    const unsigned int  mmBmPageSize            = 0x15400;
    const unsigned int  mmBmPageQueueSize       = 0x15800;
}

ScatterGatherPacketBmManager::MapRegisters ScatGPktRegAddrs =
    {
    mmBmPageSize + 1,            //  IncomingPageSize_Addr
    mmBmPageSize + 0,            //  OutgoingPageSize_Addr
    mmBmSGPageQueueAddress + 1,  //  IncomingPageQueueLSW_Addr
    mmBmSGPageQueueAddress + 0,  //  OutgoingPageQueueLSW_Addr
    mmBmSGPageQueueAddress + 3,  //  IncomingPageQueueMSW_Addr
    mmBmSGPageQueueAddress + 2,  //  OutgoingPageQueueMSW_Addr
    mmBmPageQueueSize + 1,       //  IncomingPageQueueSize_Addr
    mmBmPageQueueSize + 0,       //  OutgoingPageQueueSize_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class PacketBusMasterDeviceDriver
//---------------------------------------------------------------------------

PacketBusMasterDeviceDriver::PacketBusMasterDeviceDriver(const PciId & id,
                                             unsigned int intStatusByteOffset,
                                             unsigned int intStatusMask,
                                             unsigned int intStatusRegion )
    : BusMasterDeviceDriver(id, intStatusByteOffset, intStatusMask, intStatusRegion),
      BmMgr(0), BmMgrMode(bmmLegacy), PortBA_Supplier(this)
{
    Load_BusmasterManager( new StandardPacketBmManager(&PortBA_Supplier, Device, StdPktRegAddrs) );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif
//---------------------------------------------------------------------------
//  destructor for class PacketBusMasterDeviceDriver
//---------------------------------------------------------------------------

PacketBusMasterDeviceDriver::~PacketBusMasterDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();

    delete BmMgr;
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::FinalizeResources() -- Complete setup of resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  PacketBusMasterDeviceDriver::FinalizeResources()
{
    inherited::FinalizeResources();

    // We are now open, so memory space pointers are valid
    BmMgr->RequestResources();

    // Manager is fully configured, so load the hardware regions
    BmMgr->ApplyDeviceInfo();

}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::ToMgrBmRegion() -- Converter
//------------------------------------------------------------------------------
IDeviceDriverBmManager::BmRegions  PacketBusMasterDeviceDriver::ToMgrBmRegion(BmRegions region)
{
    return static_cast<IDeviceDriverBmManager::BmRegions>(region);
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::AllocateBmRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Allocate using standard contiguous BM Region

void  PacketBusMasterDeviceDriver::AllocateBmRegion(BmRegions region, ii32 region_size_words)
{
    BmMgr->AllocateRegion(ToMgrBmRegion(region), region_size_words);
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::FreeBmRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Allocate using standard contiguous BM Region

void  PacketBusMasterDeviceDriver::FreeBmRegion(BmRegions region)
{
    BmMgr->FreeRegion(ToMgrBmRegion(region));
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::Load_BusmasterManager() --
//------------------------------------------------------------------------------
//  Replace BM Manager with another

void  PacketBusMasterDeviceDriver::Load_BusmasterManager( IDeviceDriverBmManager * mgr )
{
    if (BmMgr)
        delete BmMgr;   // clean up old manager

    BmMgr = mgr;
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::LegacyMode() --
//------------------------------------------------------------------------------
//  Legacy mode BM Memory Mgr

void  PacketBusMasterDeviceDriver::LegacyMode()
{
    if (Mode()==bmmLegacy)
        return;   // don't need to switch

    Load_BusmasterManager( new StandardPacketBmManager(&PortBA_Supplier, Device, StdPktRegAddrs) );
    BmMgrMode = bmmLegacy;
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::ScatterMode() --
//------------------------------------------------------------------------------
//  ScatterGather mode BM Memory Mgr

void  PacketBusMasterDeviceDriver::ScatterMode()
{
    if (Mode()==bmmScatter)
        return;   // don't need to switch

    Load_BusmasterManager( new ScatterGatherPacketBmManager(&PortBA_Supplier, Device, ScatGPktRegAddrs) );
    BmMgrMode = bmmScatter;
}

//------------------------------------------------------------------------------
//  PacketBusMasterDeviceDriver::CustomMode() --
//------------------------------------------------------------------------------
//  Custom mode BM Memory Mgr

void  PacketBusMasterDeviceDriver::CustomMode(IDeviceDriverBmManager * mgr)
{
    Load_BusmasterManager( mgr );
    BmMgrMode = bmmCustom;
}

//=============================================================================
//  CLASS DualRegionDeviceDriver  --
//=============================================================================

const unsigned int DualRegionDeviceRegionIdx = 0;

//---------------------------------------------------------------------------
//  constructor for class DualRegionDeviceDriver()
//---------------------------------------------------------------------------

DualRegionDeviceDriver::DualRegionDeviceDriver(const PciId & id,
                                               unsigned int intStatusByteOffset,
                                               unsigned int intStatusMask)
    :  PacketBusMasterDeviceDriver(id, intStatusByteOffset, intStatusMask,
                             DualRegionDeviceRegionIdx), LogicDeviceBar(1)
{
}

//---------------------------------------------------------------------------
//  constructor for class DualRegionDeviceDriver()
//---------------------------------------------------------------------------

DualRegionDeviceDriver::DualRegionDeviceDriver(const PciId & id,
                                               unsigned int intStatusByteOffset,
                                               unsigned int intStatusMask,
                                               unsigned int logicDeviceBar)
    :  PacketBusMasterDeviceDriver(id, intStatusByteOffset, intStatusMask,
                             DualRegionDeviceRegionIdx), LogicDeviceBar(logicDeviceBar)
{
}

//---------------------------------------------------------------------------
//  destructor for class DualRegionDeviceDriver()
//---------------------------------------------------------------------------

DualRegionDeviceDriver::~DualRegionDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//-----------------------------------------------------------------------------
//  DualRegionDeviceDriver::LogicMemoryAddress() -- return Logic memory addr
//-----------------------------------------------------------------------------

ii32 *  DualRegionDeviceDriver::LogicMemoryAddress()
{
    if ( LogicMemory.WasMapped )
        return (ii32 *)LogicMemory.Addr;
    else
        return NULL;
}

//------------------------------------------------------------------------------
//  DualRegionDeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//
void  DualRegionDeviceDriver::InitializeResources()
{
    inherited::InitializeResources();
    //
    //  Init Logic Memory Structure
    Device.InitializeMappedMemory(LogicMemory, LogicDeviceBar, 1);   // Region Idx always 1
}

//------------------------------------------------------------------------------
//  DualRegionDeviceDriver::FinalizeResources() -- Complete setup of memory blks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  DualRegionDeviceDriver::FinalizeResources()
{
    inherited::FinalizeResources();
}

//------------------------------------------------------------------------------
//  DualRegionDeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  DualRegionDeviceDriver::ReleaseResources()
{
    inherited::ReleaseResources();
}

//=============================================================================
//  CLASS DualRegionIsrDeviceDriver  --
//=============================================================================

const unsigned int DualRegionIsrDeviceRegionIdx = 0;

//---------------------------------------------------------------------------
//  constructor for class DualRegionIsrDeviceDriver()
//---------------------------------------------------------------------------

DualRegionIsrDeviceDriver::DualRegionIsrDeviceDriver(const PciId & id,
                                                     unsigned int intStatusByteOffset,
                                                     unsigned int intStatusMask)
    :  IsrDeviceDriver(id, intStatusByteOffset, intStatusMask,
                             DualRegionIsrDeviceRegionIdx), LogicDeviceBar(1)
{
}

//---------------------------------------------------------------------------
//  constructor for class DualRegionIsrDeviceDriver()
//---------------------------------------------------------------------------

DualRegionIsrDeviceDriver::DualRegionIsrDeviceDriver(const PciId & id,
                                                     unsigned int intStatusByteOffset,
                                                     unsigned int intStatusMask,
                                                     unsigned int logicDeviceBar)
    :  IsrDeviceDriver(id, intStatusByteOffset, intStatusMask,
                             DualRegionIsrDeviceRegionIdx), LogicDeviceBar(logicDeviceBar)
{
}

//---------------------------------------------------------------------------
//  destructor for class DualRegionIsrDeviceDriver()
//---------------------------------------------------------------------------

DualRegionIsrDeviceDriver::~DualRegionIsrDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//-----------------------------------------------------------------------------
//  DualRegionIsrDeviceDriver::LogicMemoryAddress() -- return Logic memory addr
//-----------------------------------------------------------------------------

ii32 *  DualRegionIsrDeviceDriver::LogicMemoryAddress()
{
    if ( LogicMemory.WasMapped )
        return (ii32 *)LogicMemory.Addr;
    else
        return NULL;
}

//------------------------------------------------------------------------------
//  DualRegionIsrDeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//
void  DualRegionIsrDeviceDriver::InitializeResources()
{
    inherited::InitializeResources();
    //
    //  Init Logic Memory Structure
    Device.InitializeMappedMemory(LogicMemory, LogicDeviceBar, 1);   // Region Idx always 1
}

//------------------------------------------------------------------------------
//  DualRegionIsrDeviceDriver::FinalizeResources() -- Complete setup of memory blks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  DualRegionIsrDeviceDriver::FinalizeResources()
{
    inherited::FinalizeResources();
}

//------------------------------------------------------------------------------
//  DualRegionIsrDeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  DualRegionIsrDeviceDriver::ReleaseResources()
{
    inherited::ReleaseResources();
}

//=============================================================================
//  CLASS TiCpuDeviceDriver  --
//=============================================================================

const unsigned int TiCpuDeviceRegionIdx = 1;

//---------------------------------------------------------------------------
//  constructor for class TiCpuDeviceDriver()
//---------------------------------------------------------------------------

TiCpuDeviceDriver::TiCpuDeviceDriver(const PciId & id,
                                     unsigned int intStatusByteOffset,
                                     unsigned int intStatusMask)
    :  SimpleBusMasterDeviceDriver(id, intStatusByteOffset, intStatusMask,
                             TiCpuDeviceRegionIdx, kIsrWriteback)
{
}

//---------------------------------------------------------------------------
//  destructor for class TiCpuDeviceDriver()
//---------------------------------------------------------------------------

TiCpuDeviceDriver::~TiCpuDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//-----------------------------------------------------------------------------
//  TiCpuDeviceDriver::CpuRegisterMemoryAddress() -- return Register memory addr
//-----------------------------------------------------------------------------

ii32 *  TiCpuDeviceDriver::CpuRegisterMemoryAddress()
{
    if ( CpuRegisterMemory.WasMapped )
        return (ii32 *)CpuRegisterMemory.Addr;
    else
        return NULL;
}

//------------------------------------------------------------------------------
//  TiCpuDeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//
void  TiCpuDeviceDriver::InitializeResources()
{
    inherited::InitializeResources();
    //
    //  Init Cpu Memory Structure
    Device.InitializeMappedMemory(CpuRegisterMemory, 1);
}

//------------------------------------------------------------------------------
//  TiCpuDeviceDriver::FinalizeResources() -- Complete setup of memory blks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  TiCpuDeviceDriver::FinalizeResources()
{
    inherited::FinalizeResources();
}

//------------------------------------------------------------------------------
//  TiCpuDeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  TiCpuDeviceDriver::ReleaseResources()
{
    inherited::ReleaseResources();
}

//=============================================================================
//  CLASS MultiChannelBMDeviceDriver  --
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class MultiChannelBMDeviceDriver()
//---------------------------------------------------------------------------

namespace
{
    const int  mcbmStandardPortMemoryBar = 0;

    const unsigned int  mcbmTotalChannelRegions = 8;
    //const unsigned int  mcbmIncomingChannelRegions = 8;
    //const unsigned int  mcbmOutgoingChannelRegions = 8;

}


//---------------------------------------------------------------------------
//  constructor for class MultiChannelBMDeviceDriver()
//---------------------------------------------------------------------------

MultiChannelBMDeviceDriver::MultiChannelBMDeviceDriver(IMultiChanOwnerDM& odm, const PciId & id,
                                                       unsigned int intStatusByteOffset,
                                                       unsigned int intStatusMask,
                                                       unsigned int logicDeviceBar )
    : IsrDeviceDriver(id, intStatusByteOffset,
                      intStatusMask, mcbmStandardPortMemoryBar, kIsrStandard),
      ODM(odm),
      Incoming(mcbmTotalChannelRegions),
      Outgoing(mcbmTotalChannelRegions),
      LogicDeviceBar(logicDeviceBar), HdwMgr(this),
      IncomingCfg(*this, Incoming, "MultiChannelBMDeviceDriver::IncomingConfig"),
      OutgoingCfg(*this, Outgoing, "MultiChannelBMDeviceDriver::OutgoingConfig"),
      is64bitCapable(false)
{

}

//---------------------------------------------------------------------------
//  destructor for class MultiChannelBMDeviceDriver()
//---------------------------------------------------------------------------

MultiChannelBMDeviceDriver::~MultiChannelBMDeviceDriver()
{
    //  Close once in the most derived class that grabs resources...
    if (Opened())
        Close();
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::InitializeResources() -- Init resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the initialization
//    of memory blocks from the resource region here.
//  Remember to call the base class version to init the base class data!
//
void  MultiChannelBMDeviceDriver::InitializeResources()
{
    inherited::InitializeResources();

    //
    //  Init Logic Memory Structure
    Device.InitializeMappedMemory(LogicMemory, LogicDeviceBar, 1);   // Region Idx always 1

    try
        {
        ODM.TextLog().Log("Allocate Incoming Regions");
        for (size_t i=0; i<Incoming.size(); i++)
            {
            ODM.TextLog().Log("Region=", i);
            AllocateBmRegion(Incoming[i]);
            }
        ODM.TextLog().Log("Allocate Outgoing Regions");
        for (size_t i=0; i<Outgoing.size(); i++)
            {
            ODM.TextLog().Log("Region=", i);
            AllocateBmRegion(Outgoing[i]);
            }
        }
    catch (Innovative::MalibuException & /*e*/)
        {
        FreeAllBmRegions();
        throw;
        }
    ODM.TextLog().Log("End Initialize Resources");
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::FinalizeResources() -- Complete setup of resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to insert the complete setup
//    of memory blocks here.
//  Remember to call the base class version to set up the base class data!

void  MultiChannelBMDeviceDriver::FinalizeResources()
{
    ODM.TextLog().Log("+++FinalizeResources()");
    inherited::FinalizeResources();

    // We are now open, so memory space pointers are valid
    HdwMgr.RequestResources();

    HdwMgr.MasterReset();
    HdwMgr.BusmasterReset();

    // Manager is fully configured, so load the hardware regions
    ODM.TextLog().Log("Load Busmaster Address Info");
    HdwMgr.ApplyDeviceInfo();

    //  Dump Physical Addresses
    for (size_t i=0; i<Incoming.size(); i++)
        {
        ODM.TextLog().Log("Incoming Region=", i);
        ODM.TextLog().Log("PhysAddr=", reinterpret_cast<ii64>(Incoming[i].PhysAddr()), 16);
        }
    for (size_t i=0; i<Outgoing.size(); i++)
        {
        ODM.TextLog().Log("Outgoing Region=", i);
        ODM.TextLog().Log("PhysAddr=", reinterpret_cast<ii64>(Outgoing[i].PhysAddr()), 16);
        }

    ODM.TextLog().Log("---FinalizeResources()");
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::ReleaseResources() -- release held resource blocks
//------------------------------------------------------------------------------
//  This virtual function allows derived classes to release their personal
//    memory blocks here.
//  Remember to call the base class version to free the base class data!

void  MultiChannelBMDeviceDriver::ReleaseResources()
{
    ODM.TextLog().Log("+++MultiChannelBMDeviceDriver::ReleaseResources(");

    inherited::ReleaseResources();

    //  Unmap the BusMaster Regions before exiting
    FreeAllBmRegions();

    ODM.TextLog().Log("---MultiChannelBMDeviceDriver::ReleaseResources(");
}


//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::AllocateBmRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  Allocate using standard contiguous BM Region

void  MultiChannelBMDeviceDriver::AllocateBmRegion(StandardMcbm::ChannelRegionInfo & CRI)
{
    ODM.TextLog().Log("Allocate Region Size = ", CRI.Requested_BMSize);
    if (CRI.Requested_BMSize==0)
        {
        CRI.Requested_BMSize = 0x4000;
        ODM.TextLog().Log("...Instead using Size = ", CRI.Requested_BMSize);
        }
    CRI.Allocated = false;
    Device.AllocateMemoryBlock(CRI.MB, CRI.SizeInBytes(), is64bitCapable);
    //  The above might throw, but if we get here, it didn't
    CRI.Allocated = true;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::FreeBmRegion() -- default BM Memory Allocation
//------------------------------------------------------------------------------
//  free allocated BM Region

void  MultiChannelBMDeviceDriver::FreeBmRegion(StandardMcbm::ChannelRegionInfo & CRI)
{
    CRI.Allocated = false;
    Device.FreeMemoryBlock(CRI.MB, is64bitCapable);
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::FreeAllBmRegions() --
//------------------------------------------------------------------------------
//  free allocated BM Region

void  MultiChannelBMDeviceDriver::FreeAllBmRegions()
{
    for (size_t i=0; i<Incoming.size(); i++)
        {
        ODM.TextLog().Log("Incoming Channel = ", i);
        ODM.TextLog().Log("Free Region Size = ", Incoming[i].Requested_BMSize);
        if (Incoming[i].Allocated)
            FreeBmRegion(Incoming[i]);
        }
    for (size_t i=0; i<Outgoing.size(); i++)
        {
        ODM.TextLog().Log("Outgoing Channel = ", i);
        ODM.TextLog().Log("Free Region Size = ", Outgoing[i].Requested_BMSize);
        if (Outgoing[i].Allocated)
            FreeBmRegion(Outgoing[i]);
        }
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::LogicMemoryAddress() -- return Logic memory addr
//-----------------------------------------------------------------------------

ii32 *  MultiChannelBMDeviceDriver::LogicMemoryAddress()
{
    if ( LogicMemory.WasMapped )
        return (ii32 *)LogicMemory.Addr;
    else
        return NULL;
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::PortMemoryAddress() -- return Logic memory addr
//-----------------------------------------------------------------------------

ii32 *  MultiChannelBMDeviceDriver::PortMemoryAddress()
{
    if ( PortMemory.WasMapped )
        return (ii32 *)PortMemory.Addr;
    else
        return NULL;
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::GetMaxIncomingRegionCount() -- Intf to Manager
//-----------------------------------------------------------------------------

size_t  MultiChannelBMDeviceDriver::GetMaxIncomingRegionCount()
{
    return IncomingCfg.MaxMcChannels();
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::GetMaxOutgoingRegionCount() -- Intf to Manager
//-----------------------------------------------------------------------------

size_t  MultiChannelBMDeviceDriver::GetMaxOutgoingRegionCount()
{
    return OutgoingCfg.MaxMcChannels();
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::GetInUseIncomingRegionCount() -- Intf to Manager
//-----------------------------------------------------------------------------

size_t MultiChannelBMDeviceDriver::GetInUseIncomingRegionCount()
{
    return IncomingCfg.McChannels();
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::GetInUseOutgoingRegionCount() -- Intf to Manager
//-----------------------------------------------------------------------------

size_t MultiChannelBMDeviceDriver::GetInUseOutgoingRegionCount()
{
    return OutgoingCfg.McChannels();
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::GetIncomingRegionInfo() -- Intf to Manager
//-----------------------------------------------------------------------------

StandardMcbm::ChannelRegionInfo *  MultiChannelBMDeviceDriver::GetIncomingRegionInfo(size_t chan)
{
    return &Incoming[chan];
}

//-----------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::GetOutgoingRegionCount() -- Intf to Manager
//-----------------------------------------------------------------------------

StandardMcbm::ChannelRegionInfo *  MultiChannelBMDeviceDriver::GetOutgoingRegionInfo(size_t chan)
{
    return &Outgoing[chan];
}


//=============================================================================
//  CLASS MultiChannelBMDeviceDriver::DirectionConfig
//=============================================================================
//------------------------------------------------------------------------------
//  housekeeping for class MultiChannelBMDeviceDriver::DirectionConfig
//------------------------------------------------------------------------------

const unsigned int kDefaultUnusedChannelAllocSize = 16384;   // must be multiple of 4K;

//------------------------------------------------------------------------------
//  constructor for class MultiChannelBMDeviceDriver::DirectionConfig
//------------------------------------------------------------------------------

MultiChannelBMDeviceDriver::DirectionConfig::DirectionConfig(MultiChannelBMDeviceDriver& owner,
                                                             std::vector<StandardMcbm::ChannelRegionInfo>& info,
                                                             const std::string & Tag)
    :  Owner(owner), Info(info), TagString(Tag),
       ChannelActive(MaxMcChannels()), CurrentMcChannels(MaxMcChannels()),
       UnusedChannelAllocSize(kDefaultUnusedChannelAllocSize)
{
    std::fill_n(ChannelActive.begin(), MaxMcChannels(), 0u);  // set all elements to "false"
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::MsgText() -- standard text info
//------------------------------------------------------------------------------

std::string   MultiChannelBMDeviceDriver::DirectionConfig::MsgText(const std::string & first, const std::string & last)
{
    return first + TagString + last;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::SetAllBusMasterSize() -- set all
//------------------------------------------------------------------------------

void MultiChannelBMDeviceDriver::DirectionConfig::SetAllBusMasterSize(ii32 words)
{
    std::string in_text(MsgText("+++ ", " words="));
    Owner.ODM.TextLog().Log(in_text, words);

    for (unsigned int chan=0; chan<Info.size(); chan++)
        {
        if (chan < McChannels())
            BusMasterSize(chan, words);
        else
            BusMasterSize(chan, UnusedChannelAllocSize);  // use standard size for regions not used
        }

    std::string out_text(MsgText("--- ", " "));
    Owner.ODM.TextLog().Log(out_text);
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::BusMasterSize() --
//------------------------------------------------------------------------------

void MultiChannelBMDeviceDriver::DirectionConfig::BusMasterSize(int chan, ii32 words)
{
    std::string in_text(MsgText("+++ ", "::BusMasterSize(chan, words) "));
    Owner.ODM.TextLog().Log(in_text, words);
    Owner.ODM.TextLog().Log("      chan =", chan);
    Owner.ODM.TextLog().Log("      words =", words);

    const unsigned int FourKByte = 0x400;
    if (chan < static_cast<int>(Info.size()))
        {
        //  Adjust size to be integral 1K words
        if (words%FourKByte)
            words = (words/FourKByte+1)*FourKByte;
        //  Save size
        Info[chan].Requested_BMSize = words;
        }

    std::string out_text(MsgText("--- ", "::BusMasterSize(chan, words) "));
    Owner.ODM.TextLog().Log(out_text);
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::BusMasterSize() --
//------------------------------------------------------------------------------

ii32 MultiChannelBMDeviceDriver::DirectionConfig::BusMasterSize(int chan)
{
    if (chan < static_cast<int>(Owner.Incoming.size()))
        return Info[chan].SizeInWords();
    else
        return 0;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::SetAllNotifySize() -- set all
//------------------------------------------------------------------------------

void MultiChannelBMDeviceDriver::DirectionConfig::SetAllNotifySize(ii32 words)
{
    for (int chan=0; chan<static_cast<int>(Info.size()); chan++)
        {
        NotifySize(chan, words);
        }
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::NotifySize() --
//------------------------------------------------------------------------------

void MultiChannelBMDeviceDriver::DirectionConfig::NotifySize(int chan, ii32 words)
{
    std::string in_text(MsgText("+++ ", "::NotifySize(chan, words) "));
    Owner.ODM.TextLog().Log(in_text, words);
    Owner.ODM.TextLog().Log("      chan =", chan);
    Owner.ODM.TextLog().Log("      words =", words);

    if (chan < static_cast<int>(Owner.Incoming.size()))
        {
        Info[chan].NotifyPktSize = words;
        }
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::NotifySize() --
//------------------------------------------------------------------------------

ii32 MultiChannelBMDeviceDriver::DirectionConfig::NotifySize(int chan)
{
    if (chan < static_cast<int>(Owner.Incoming.size()))
        return Info[chan].NotifySize();
    else
        return 0;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::MaxMcChannels() --
//------------------------------------------------------------------------------
//  System Max Mc Channels

ii32 MultiChannelBMDeviceDriver::DirectionConfig::MaxMcChannels()
{
    return mcbmTotalChannelRegions;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::McChannels() --
//------------------------------------------------------------------------------
//  In Use Mc Channels

ii32 MultiChannelBMDeviceDriver::DirectionConfig::McChannels()
{
    return CurrentMcChannels;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::McChannelActive() --
//------------------------------------------------------------------------------
//  In Use Mc Channels "active"

bool MultiChannelBMDeviceDriver::DirectionConfig::McChannelActive(int ch)
{
    if (ch < static_cast<int>(McChannels()))
        return ChannelActive[ch]!=0u;
    else
        return false;   // if not in use, will never be active
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::SetMcChannels() --
//------------------------------------------------------------------------------
//  SET In Use Mc Channels

void MultiChannelBMDeviceDriver::DirectionConfig::SetMcChannels(ii32 channels)
{
    CurrentMcChannels = channels;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::SetUnusedMcChannelsSize() --
//------------------------------------------------------------------------------

void MultiChannelBMDeviceDriver::DirectionConfig::SetUnusedMcChannelsSize(ii32 words)
{
    UnusedChannelAllocSize = words;
}

//------------------------------------------------------------------------------
//  MultiChannelBMDeviceDriver::DirectionConfig::SetMcChannelActive() --
//------------------------------------------------------------------------------
//  SET an in-use channel to be Active, or inactive

void MultiChannelBMDeviceDriver::DirectionConfig::SetMcChannelActive(int chan, bool isActive)
{
    if (chan < static_cast<int>(MaxMcChannels()))
        ChannelActive[chan] = (isActive!=0) ? 1 : 0;
}

} // namespace InnovativeKernel

