//  DeviceDriver_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef DeviceDriver_MbH
#define DeviceDriver_MbH

#include <DeviceDriverBmManager_Mb.h>
#include <Osal_Mb.h>
#include <DataTypes_Mb.h>
#include <string>
#include <Event_Mb.h>
#include <DeviceEnum_Mb.h>
#include <KernelTypes_Mb.h>

#include <MultiChanStreamSupport_Mb.h>

namespace InnovativeKernel
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//---------------------------------------------------------------------------
//struct BoardInfo
//{
//    DeviceEntry Enum;
//
//    ULONG       DriverVersion;
//    ULONG       DriverDLLVersion;
//};
//---------------------------------------------------------------------------
////---------------------------------------------------------------------------
//struct IoPortBlock
//{
//    ULONG           Base;       // Base Port of Block
//    ULONG           Size;       // Block Size in bytes
//    MemoryType      MemoryType; // Port or Memory Mapped
//    bool            WasMapped;  // Was Mapped or not
//};
//
//---------------------------------------------------------------------------
class IsrLog
{
public:
    int   Count;
    bool  Wrapped;
    int   Tally;
    int   Status[1000];

    IsrLog() : Count(0), Wrapped(false), Tally(0) {}
    void Clear() { Count=0; Wrapped=false; Tally=0; }
    void Erase()
        {
        Clear();
        for (int i=0; i<1000; i++)
             Status[i] = 0;
        };
    void Log(int value)
        {
        ++Tally;
        if (Count<1000)
            {
            Status[Count++] = value;
            if (Count>=1000)
                {
                Count = 0;
                Wrapped = true;
                }
            }
        }

};

//==============================================================================
//  CLASS IDeviceDriverDebugUI -- OsDevice Debug Call-up Interface
//==============================================================================

class IDeviceDriverDebugUI
{
public:
    virtual ~IDeviceDriverDebugUI() {}

    virtual void  TraceLog(int code, int data) = 0;

};

//=============================================================================
//   CLASS DeviceDriver  -- Base class for DriverX Device Drivers
//=============================================================================

class  DeviceDriver
{
public:
    friend class StandardOsDeviceDebugUI;
    //
    //  Member Functions
    DeviceDriver(const PciId & id, int port_region=0);
    virtual ~DeviceDriver();

    virtual bool         Open();
    virtual bool         Close();
    //
    //  ...Properties
    size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);
    bool            Opened() { return FOpened; }
    void            Target( int target );
    int             Target() { return FTarget; }
    InnovativeKernel::PciPosition   Location()
        { return Device.BoardLocation(); }

    ii32 *   PortMemoryAddress();
    ii32 *   PortPhysicalAddress()
        {  return PortMemory.PhysAddr;  }
    ii32     PortSizeWords()
        {  return PortMemory.Size/sizeof(int);  }

    void                Error(const std::string & text );
    size_t              KpQueueDepth();
    void                LoadDebugUI(IDeviceDriverDebugUI * ddui)
            {  DDUI = ddui;  }
    unsigned int        Test();

    // Info readback
    DeviceEntry &        PciConfig()
        {  return PciCfgData;  }

    Osal::OsDevice  &ODevice() {    return Device;   }

protected:
    bool  FillCardinfo();
    //
    //  Overridable methods if changes needed
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();

    virtual void  AppendToIsrLog(unsigned int isr_log_data);
    virtual void  AppendToTraceLog(int code, int data);

protected:
    //
    //  Member Data
    Osal::OsDevice   Device;
    int              FTarget;       // Device No
    MemoryBlock      PortMemory;    // Device IO ports memory region.
    int              PortRegion;
    DeviceEntry      PciCfgData;
    bool             FOpened;
    IDeviceDriverDebugUI *  DDUI;
};

//=============================================================================
//  CLASS DualPortDeviceDriver  -- DPRam + Periph Region Busmastering Device
//=============================================================================

class  DualPortDeviceDriver : public DeviceDriver
{
    typedef DeviceDriver inherited;

public:
    //
    //  Member Functions
    DualPortDeviceDriver(const PciId & id,
                         int port_region=0,
                         int dual_port_region=1);
    virtual ~DualPortDeviceDriver();

    ii32 *   DualPortMemoryAddress()
        { return DualPortMemory.Addr;  }

    ii32 *   DualPortPhysicalAddress()
        {  return DualPortMemory.PhysAddr;  }
    ii32     DualPortSizeWords()
        {  return (ii32)DualPortMemory.Size/sizeof(int);  }

protected:
    //
    //  Member Data
    MemoryBlock  DualPortMemory;
    int          DualPortRegion;
    //
    //  Member Functions
    //  ...Overridable Configuration methods
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();
};

//=============================================================================
//   CLASS IsrDeviceDriver  -- Base class for interrupt-driven DriverX Drivers
//=============================================================================

class  IsrDeviceDriver : public DeviceDriver
{
    typedef DeviceDriver inherited;

public:
    //
    //  Member Functions
    IsrDeviceDriver(const PciId & id,
                    unsigned int intStatusByteOffset,
                    unsigned int intStatusMask,
                    unsigned int intStatusRegion = 0,
                    KernelIsrType intHandlerType = kIsrStandard);

    // Methods
    virtual bool         Open();
    virtual bool         Close();

    void                 InitializeInterruptDeviceExtension();
    void                 ClearInterruptStatusQueue();
    Innovative::Event *  IsrEvent() { return &mIsrEvent; };
    size_t               InterruptDepth();
    unsigned int         InterruptAck();

    IsrLog &             ILOG() {  return mIsrInfo;  }

    void  EnableDedicatedKernelCacheRelease(Innovative::IAddressingSpace & space, int offset);

protected:
    bool  InterruptInstall();
    bool  InterruptDeinstall();
    bool  InterruptEnable();
    bool  InterruptDisable();

    virtual void  AppendToIsrLog(unsigned int isr_log_data);

protected:
    //
    //  Member Data
    KernelIsrType         mKernelIsrType;
    const unsigned int    mIntStatusByteOffset;
    const unsigned int    mIntStatusMask;
    const unsigned int    mIntStatusRegion;
    Innovative::Event     mIsrEvent;   // Interrupt event.
    IsrLog                mIsrInfo;

};

//=============================================================================
//  CLASS BusMasterDeviceDriver  --
//=============================================================================

class BusMasterDeviceDriver : public IsrDeviceDriver
{
public:
    typedef IsrDeviceDriver inherited;

    enum BmRegions { bmIncoming, bmOutgoing };

    //
    //  Member Functions
    BusMasterDeviceDriver(const PciId & id,
                          unsigned int intStatusByteOffset,
                          unsigned int intStatusMask,
                          unsigned int intStatusRegion = 0,
                          KernelIsrType intHandlerType = kIsrStandard);
    virtual ~BusMasterDeviceDriver();

    // Properties
    void    BusMasterSizeWords(unsigned int words);   // Make both the same size
    void    IncomingBusMasterSizeWords(unsigned int words);   // Set size for Board->App flow
    ii32    IncomingBusMasterSizeWords() const;
    void    OutgoingBusMasterSizeWords(unsigned int words);   // Set size for App >Board flow
    ii32    OutgoingBusMasterSizeWords() const;
    void   SetHighMemPoolFlag(bool flag)
        { is64bitCapable = flag; }

protected:
    ii32    FBusMasterSizeWords_Incoming;
    ii32    FBusMasterSizeWords_Outgoing;
    bool   is64bitCapable;
protected:
    //
    //  Member Functions
    //  ...Overridable Configuration methods
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();

    //  default "BM Memory Allocator Functions
    virtual void  AllocateBmRegion(BmRegions region, ii32 region_size_words) = 0;
    virtual void  FreeBmRegion(BmRegions region) = 0;
};

//=============================================================================
//  CLASS  SimpleBusMasterDeviceDriver  --
//=============================================================================

class SimpleBusMasterDeviceDriver : public BusMasterDeviceDriver
{
    typedef BusMasterDeviceDriver inherited;

public:
    //
    //  Member Functions
    SimpleBusMasterDeviceDriver(const PciId & id,
                          unsigned int intStatusByteOffset,
                          unsigned int intStatusMask,
                          unsigned int intStatusRegion = 0,
                          KernelIsrType intHandlerType = kIsrStandard);
    virtual ~SimpleBusMasterDeviceDriver();

    // Methods
    ii32 *  IncomingBusMasterAddress() const;
    ii32 *  IncomingBusMasterPhysicalAddress() const;
    ii32 *  OutgoingBusMasterAddress() const;
    ii32 *  OutgoingBusMasterPhysicalAddress() const;

protected:
    MemoryBlock  BusMasterMemory_Incoming;      // Incoming Device BusMaster memory.
    MemoryBlock  BusMasterMemory_Outgoing;      // Outgoing Device BusMaster memory.

    MemoryBlock &  GetBmMemoryBlock( BmRegions which );

protected:
    //
    //  Member Functions

    //  Implementation of BM Allocation Functions
    virtual void  AllocateBmRegion(BmRegions region, ii32 region_size_words);
    virtual void  FreeBmRegion(BmRegions region);

};

//=============================================================================
//  CLASS  PacketBusMasterDeviceDriver  --
//=============================================================================

class PacketBusMasterDeviceDriver : public BusMasterDeviceDriver
{
    typedef BusMasterDeviceDriver inherited;

public:
    //
    //  Member Functions
    PacketBusMasterDeviceDriver(const PciId & id,
                          unsigned int intStatusByteOffset,
                          unsigned int intStatusMask,
                          unsigned int intStatusRegion = 0 );
    virtual ~PacketBusMasterDeviceDriver();

    // Methods
    IDeviceDriverBmManagerUI &  UI()
            {  return BmMgr->UI();  }

    enum BmManagerModes { bmmLegacy, bmmScatter, bmmCustom };

    BmManagerModes  Mode()
                        {  return BmMgrMode;  }
    void  LegacyMode();
    void  ScatterMode();
    void  CustomMode(IDeviceDriverBmManager * mgr);

protected:
    IDeviceDriverBmManager * BmMgr;

    BmManagerModes   BmMgrMode;

    void  Load_BusmasterManager( IDeviceDriverBmManager * mgr );

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  Supplies_PortBaseAddr() --  "Requirements" class Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class Supplies_PortBaseAddr : public IPmRequires_PortBaseAddr
    {
    public:
        virtual ii32 *  GetPortBaseAddr()
            { return DD->PortMemoryAddress(); }

        Supplies_PortBaseAddr(PacketBusMasterDeviceDriver * dd)
            :  DD(dd)
            {}
        virtual ~Supplies_PortBaseAddr()
            {}
    private:
        PacketBusMasterDeviceDriver * DD;
    };

    Supplies_PortBaseAddr  PortBA_Supplier;

protected:
    //
    //  Member Functions
    virtual void  FinalizeResources();

    //  Implementation of BM Allocation Functions
    virtual void  AllocateBmRegion(BmRegions region, ii32 region_size_words);
    virtual void  FreeBmRegion(BmRegions region);

    IDeviceDriverBmManager::BmRegions  ToMgrBmRegion( BmRegions region );
};

//=============================================================================
//  CLASS DualRegionDeviceDriver  -- 2 Periph Region Busmastering Device
//=============================================================================

class  DualRegionDeviceDriver : public PacketBusMasterDeviceDriver
{
    typedef PacketBusMasterDeviceDriver inherited;

public:
    //
    //  Member Functions
    DualRegionDeviceDriver(const PciId & id,
                           unsigned int intStatusByteOffset,
                           unsigned int intStatusMask);
    DualRegionDeviceDriver(const PciId & id,
                           unsigned int intStatusByteOffset,
                           unsigned int intStatusMask,
                           unsigned int logicDeviceBar);
    virtual ~DualRegionDeviceDriver();
    ii32 *   LogicMemoryAddress();

protected:
    //
    //  Member Data
    MemoryBlock  LogicMemory;          // Device Logic memory region.
    unsigned int LogicDeviceBar;
    //
    //  Member Functions
    //  ...Overridable Configuration methods
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();

};

//=============================================================================
//  CLASS DualRegionIsrDeviceDriver  -- 2 Periph Region Isr only Device
//=============================================================================

class  DualRegionIsrDeviceDriver : public IsrDeviceDriver
{
    typedef IsrDeviceDriver inherited;

public:
    //
    //  Member Functions
    DualRegionIsrDeviceDriver(const PciId & id,
                              unsigned int intStatusByteOffset,
                              unsigned int intStatusMask);
    DualRegionIsrDeviceDriver(const PciId & id,
                              unsigned int intStatusByteOffset,
                              unsigned int intStatusMask,
                              unsigned int logicDeviceBar);
    virtual ~DualRegionIsrDeviceDriver();
    ii32 *   LogicMemoryAddress();

protected:
    //
    //  Member Data
    MemoryBlock  LogicMemory;          // Device Logic memory region.
    unsigned int LogicDeviceBar;
    //
    //  Member Functions
    //  ...Overridable Configuration methods
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();

};

//=============================================================================
//  CLASS TiCpuDeviceDriver  -- TI Device 2 Periph Region Busmastering Cpu
//=============================================================================

class  TiCpuDeviceDriver : public SimpleBusMasterDeviceDriver
{
    typedef SimpleBusMasterDeviceDriver inherited;

public:
    //
    //  Member Functions
    TiCpuDeviceDriver(const PciId & id,
                      unsigned int intStatusByteOffset,
                      unsigned int intStatusMask);
    virtual ~TiCpuDeviceDriver();
    ii32 *   CpuRegisterMemoryAddress();

protected:
    //
    //  Member Data
    MemoryBlock  CpuRegisterMemory;      // Device CpuRegister memory region.
    //
    //  Member Functions
    //  ...Overridable Configuration methods
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();

};

//---------------------------------------------------------------------------
//  RegisterPciDevice() --  Register PCI device in factory table
//---------------------------------------------------------------------------

void RegisterPciDevice(const std::string & name, const std::string & device_base_name,
                       int vendor_id, int device_id, int sub_id,
                       bool check_sub_id);


//=============================================================================
//  CLASS MultiChannelBMDeviceDriver  --
//=============================================================================
//---------------------------------------------------------------------------
//  namespace MultiChanStandardMemoryMap
//---------------------------------------------------------------------------

namespace MultiChanStandardMemoryMap
{

//  MultiChannelBM Memory Map
const unsigned int  mmXuControlStatusRegister         = 0x000;
const unsigned int  mmXuInterruptEnableRegister       = 0x001;
const unsigned int  mmXuInterruptStatusRegister       = 0x002;
const unsigned int  mmXuInterruptQueuesDone           = 0x003;

const unsigned int  mmXuRxChannelSelect               = 0x010;
const unsigned int  mmXuRxBaseAddr_Low                = 0x011;
const unsigned int  mmXuRxBaseAddr_High               = 0x012;
const unsigned int  mmXuRxBufferSize                  = 0x013;
const unsigned int  mmXuRxNotifySize                  = 0x014;

const unsigned int  mmXuRxCreditAdd                   = 0x015;
const unsigned int  mmXuRxInterruptQueue              = 0x016;
const unsigned int  mmXuRxInterruptQueueDepth         = 0x017;
const unsigned int  mmXuRxInterruptTimeout            = 0x018;
const unsigned int  mmXuRxInterruptTimeoutEnable      = 0x019;
//const unsigned int  mmXuRxInterruptTimerClear     = 0x01a;

const unsigned int  mmXuTxChannelSelect               = 0x020;
const unsigned int  mmXuTxBaseAddr_Low                = 0x021;
const unsigned int  mmXuTxBaseAddr_High               = 0x022;
const unsigned int  mmXuTxBufferSize                  = 0x023;
const unsigned int  mmXuTxNotifySize                  = 0x024;

const unsigned int  mmXuTxCreditAdd                   = 0x025;
const unsigned int  mmXuTxInterruptQueue              = 0x026;
const unsigned int  mmXuTxInterruptQueueDepth         = 0x027;
const unsigned int  mmXuTxArbitratorRequestLimit      = 0x028;

const unsigned int  mmXuAlertInterruptQueue           = 0x030;
const unsigned int  mmXuAlertInterruptQueueDepth      = 0x031;
const unsigned int  mmXuAlertPayloadRead              = 0x032;

const unsigned int  mmXuExternalInterruptQueue        = 0x033;
const unsigned int  mmXuExternalInterruptQueueDepth   = 0x034;
const unsigned int  mmXuExternalInterruptEnable       = 0x035;

const unsigned int  mmXuDeviceLinkCSR                 = 0x040;
const unsigned int  mmXuDeviceTestRegister            = 0x041;

}  // end namespace MultiChanStandardMemoryMap

//---------------------------------------------------------------------------
//  declaration of MultiChannelBMDeviceDriver
//---------------------------------------------------------------------------

class MultiChannelBMDeviceDriver : public IsrDeviceDriver, public Supplies_MultiChanDevice
{
public:
    typedef IsrDeviceDriver inherited;
    //
    //  Member Functions
    //
    //  Member Functions
    MultiChannelBMDeviceDriver(IMultiChanOwnerDM & odm,
                               const PciId & id,
                               unsigned int intStatusByteOffset,
                               unsigned int intStatusMask,
                               unsigned int logicDeviceBar);
    virtual ~MultiChannelBMDeviceDriver();

    // Properties
    ii32 *   LogicMemoryAddress();
    ii32 *   PortMemoryAddress();

    Innovative::IMultiChanConfiguration &   BmIn() {  return IncomingCfg; }
    Innovative::IMultiChanConfiguration &   BmOut() {  return OutgoingCfg; }

    Innovative::IMultiChanInitialization &   BmInInit() {  return IncomingCfg; }
    Innovative::IMultiChanInitialization &   BmOutInit() {  return OutgoingCfg; }

    StandardMcbmHardwareManager & Hdw()  {  return HdwMgr;  }

    //
    //  Requirement Supply Implementations
    virtual ii32 *  GetPortBaseAddr()
        { return PortMemoryAddress(); }

    virtual size_t  GetMaxIncomingRegionCount();
    virtual size_t  GetMaxOutgoingRegionCount();
    virtual size_t  GetInUseIncomingRegionCount();
    virtual size_t  GetInUseOutgoingRegionCount();

    virtual StandardMcbm::ChannelRegionInfo *  GetIncomingRegionInfo(size_t chan);
    virtual StandardMcbm::ChannelRegionInfo *  GetOutgoingRegionInfo(size_t chan);
    virtual InnovativeKernel::IMultiChanOwnerDM & IDevMap() {  return ODM;  }

    IMultiChanOwnerDM & ODM;   // device map interface
    std::vector<StandardMcbm::ChannelRegionInfo>    Incoming;
    std::vector<StandardMcbm::ChannelRegionInfo>    Outgoing;

    void   SetHighMemPoolFlag(bool flag)
    { is64bitCapable = flag; }

protected:
    //
    //  Member Data
    MemoryBlock  LogicMemory;          // Device Logic memory region.
    unsigned int LogicDeviceBar;

    StandardMcbmHardwareManager  HdwMgr;

protected:
    //
    //  Member Functions
    //  ...Overridable Configuration methods
    virtual void  InitializeResources();
    virtual void  FinalizeResources();
    virtual void  ReleaseResources();

    //  default BM Memory Allocator Functions
    void  AllocateBmRegion(StandardMcbm::ChannelRegionInfo & CRI);
    void  FreeBmRegion(StandardMcbm::ChannelRegionInfo & CRI);

    void  FreeAllBmRegions();

    //=======================================================================
    //  CLASS MultiChannelBMDeviceDriver::DirectionConfig -- Config Intf
    //=======================================================================
    class DirectionConfig : public Innovative::IMultiChanConfiguration,
                            public Innovative::IMultiChanInitialization
    {
    public:

        //  IMultiChanConfiguration impl
        virtual void  SetAllBusMasterSize(ii32 words);  // set all channels
        virtual void  BusMasterSize(int chan, ii32 words);
        virtual ii32  BusMasterSize(int chan);

        virtual void  SetAllNotifySize(ii32 words);  // set all channels
        virtual void  NotifySize(int chan, ii32 words);
        virtual ii32  NotifySize(int chan);

        virtual ii32  MaxMcChannels();  // absolute system max stream channels
        virtual ii32  McChannels();     // "in use" max stream channels
        virtual bool  McChannelActive(int ch);   // true if channel is "in use" and analog ch enabled

        // Innovative::IMultiChanInitialization  impl
        void          SetMcChannels(ii32 channels);
        virtual void  SetUnusedMcChannelsSize(ii32 words); // Only allocate a (small) size for unused streams
        virtual void  SetMcChannelActive(int chan, bool isActive);  // set to follow analog channel enable

        DirectionConfig(MultiChannelBMDeviceDriver & owner, std::vector<StandardMcbm::ChannelRegionInfo> &info,
                        const std::string & Tag);
        virtual ~DirectionConfig()
            {}

    private:
        MultiChannelBMDeviceDriver & Owner;
        std::vector<StandardMcbm::ChannelRegionInfo> & Info;
        std::string       TagString;
        std::vector<int>  ChannelActive;
        ii32              CurrentMcChannels;
        ii32              UnusedChannelAllocSize;

        std::string MsgText(const std::string& first, const std::string& last);
    };

    DirectionConfig  IncomingCfg;
    DirectionConfig  OutgoingCfg;
    bool                is64bitCapable;


};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace InnovativeKernel

#endif
