// DigitalReceiverDeviceMap.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef DigitalReceiverDeviceMap_MbH
#define DigitalReceiverDeviceMap_MbH


#include <PacketDeviceMap_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PllCalMem_Mb.h>
#include <Ics8402_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <ZbtRam_Mb.h>
#include <ChannelInfo_Mb.h>
#include <GcScript_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class DigitalReceiverDeviceDriver;
class Event;
class PmcBuffer;

namespace DigitalReceiverConstants
{
    const int ChannelCount = 8;
};

class DigitalReceiver;

//===========================================================================
//  CLASS DigitalReceiverDeviceMap  -- Register & low level support for DigitalReceiver
//===========================================================================

class DigitalReceiverDeviceMap : public PacketDeviceImpl,
                                 public PacketDeviceMap,
                                 public IUsesVirtexFpgaLoader,
                                 public IUsesVirtexJtagLoader,
                                 public IUsesPllCalibration,
                                 public IUsesFpgaLogicVersionInfo,
                                 public IUsesPciLogicVersionInfo
{
    friend class DigitalReceiver;

public:
    //
    //  Member Functions
    DigitalReceiverDeviceMap(DigitalReceiver * owner);
    virtual ~DigitalReceiverDeviceMap();

    void  OpenHardware();

    //
    //  Make this property public
    using PacketDeviceImpl::PciLoopbackMode;

    // Zbt Ram
    ZbtRam              Ram;

protected:

    DigitalReceiver *                   Owner;
    DigitalReceiverDeviceDriver *       DDeviceDriver;

    // ???  May not be implemented
    void  LocalLoopbackMode(bool state);

    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnScriptCommand(Innovative::GcScriptCommandEvent & Event);
    void  OnScriptMessage(Innovative::ProcessStatusEvent & Event);

public:
    enum IIDestination { dPci, dJ4, dXmc };

    int  RcvPacketSize;
    int  XmitPacketSize;

    void  DumpIsrStatus(std::vector<int> & vec);

protected:
    //
    //  Class PciTestRegister
    class PciTestRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       TestModeEnable;
        RegisterBit       GenerateRxData;
        RegisterBit       LoopbackEnable;
        RegisterBit       ClearTxCredit;
        RegisterBit       ClearRxCredit;
        RegisterBit       DrainXmitData;
        RegisterBitGroup  GenPacketSize;
        RegisterBit       GenPacketSizeUpdate;

    public:
        PciTestRegister( AddressingSpace &space, int offset );
    };

    class ClockSelectRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        enum IIAdcClkSelect { acA, acB, acPll };

        RegisterBitGroup      Pair0Mux;
        RegisterBitGroup      Pair1Mux;
        RegisterBit           Q0Mux;
        RegisterBit           Q1Mux;
        RegisterBit           FpgaClockSelect;
        RegisterBit           DividerClockSelect;
        RegisterBitGroup      DividerRatio;
        RegisterBit           DividerEnable;
        RegisterBit           DividerReset;

        ClockSelectRegister(AddressingSpace &space,
            int offset)
        :  inherited(space, offset),
           Pair0Mux(*this, 0, 2), Pair1Mux(*this, 2, 2),
           Q0Mux(*this, 4), Q1Mux(*this, 5),
           FpgaClockSelect(*this, 8), DividerClockSelect(*this, 10),
           DividerRatio(*this, 12, 2), DividerEnable(*this, 14),
           DividerReset(*this, 15)
            {
            Divider(1);
            AdcClockSelect(0, acPll);
            AdcClockSelect(1, acPll);
            }

        void AdcClockSelect(int pair, IIAdcClkSelect value)
            {
            RegisterBitGroup & PairMux(pair ? Pair1Mux : Pair0Mux);
            PairMux.Value((value==acPll) ? 0 : 3);
            Q0Mux.Value((value==acPll) ? 0 : 1);
            Q1Mux.Value((value == acB) ? 1 : 0);
            }

        IIAdcClkSelect AdcClockSelect(int pair) const
            {
            RegisterBitGroup PairMux(pair ? Pair1Mux : Pair0Mux);
            if (PairMux.Value()==0)
                return acPll;

            return Q1Mux.Value() ? acB : acA;
            }

        int Divider() const
            {  return FDivider;  }
        void Divider(int value)
            {
            FDivider = value;
            DividerReset.Value(value < 1);
            DividerEnable.Value(value > 1);
            switch (value)
                {
                case 1:     DividerRatio.Value(0);      break;
                case 2:     DividerRatio.Value(1);      break;
                case 3:     DividerRatio.Value(2);      break;
                case 4:    DividerRatio.Value(3);      break;
                }
            }

    private:
        // Data
        int         FDivider;
    };

    //
    //  PacketRegister
    class PacketRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Size;
        RegisterBitGroup      Id;

        //
        //  Member Functions
        PacketRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Size(*this, 0, 24), Id(*this, 24, 8)
            {}
    };

    //
    //  Class UserFpgaCtlRegister
    class UserFpgaCtlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           FpgaReset;
        RegisterBit           Go;
        RegisterBit           ExternalBus;
        RegisterBit           DdcReset;
        RegisterBitGroup      AdcTestMode;
        RegisterBit           FifoReset;
        RegisterBit           Trigger;

        UserFpgaCtlRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               FpgaReset(*this, 0), Go(*this, 1),
               ExternalBus(*this, 3),
               DdcReset(*this, 4),
               AdcTestMode(*this, 5, 4),
               FifoReset(*this, 9), Trigger(*this, 10)
            {
            //  Default setup ????
            }

    private:
    };

    //
    //  Class TriggerRegister
    class TriggerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Count;
		RegisterBit			  Persistent; // used in ext. mode ~ one shot trig.
		RegisterBit           FramedMode;
		RegisterBit           External;

        TriggerRegister(AddressingSpace &space, int offset)
			:  inherited(space, offset),
			   Count(*this, 0, 16),
			   Persistent(*this, 29),
			   FramedMode(*this, 30),
               External(*this, 31)
            {
            //  Default setup ????
            }
    };

    //
    //  Class FifoIntegrityRegister
    class FifoIntegrityRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        FifoIntegrityRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset), Old(8), State(*this, 0, 8)
            {  }

            void    Reset()
            {  std::fill(Old.begin(), Old.end(), false);  }
            bool    Error(int ch) const
            {  return (State.Value() & (1 << ch)) ? true : false;  }
            bool    Change(int ch) const
            {
                bool New = Error(ch);
                bool result = Old[ch] != New;
                Old[ch] = New;

                return result;
            }

    private:
        // Data
        mutable std::vector<bool>   Old;
        ReadOnlyRegisterBitGroup    State;
    };

    UnpairedChannelInfo     FChannels;

	// Calibration
    virtual void		SetCalibrationOffset(int ch, int value);
    virtual void 		SetCalibrationGain(int ch, int value);
    virtual void		SetCalibrationPllCorrection(float value);

    // Communications
    void                ReturnPacketSize(int ch, int size_words);
    int                 ReturnPacketSize(int ch) const;

    //
    //  ...Registers
    PciTestRegister                    PciTestControlReg;
    UserFpgaCtlRegister                UserFpgaControlReg;
    RegisterArray<ShadowRegister>      GainCoeffReg;        // array of 4
    RegisterArray<ShadowRegister>      OffsetCoeffReg;      // array of 4
    ShadowRegister                     ChannelEnableReg;
    ShadowRegister                     ChannelDestinationReg;
    ClockSelectRegister                ClockSelectReg;
    RegisterArray<PacketRegister>      PacketHeaderReg;     // array of 20
    RegisterArray<TriggerRegister>     TriggerReg;          // array of 4
    Ics8402                            Clock;
    RegisterArray<ShadowRegister>      DdcFormatReg;        // array of 4
    RegisterArray<ShadowRegister>      DdcReg;              // array of 4*0x20
    GcScripter                         Script;
    ShadowRegister                     ClockTestReg;
    FifoIntegrityRegister              FifoIntegrityReg;

    // Methods
    void PokeDdcReg(int ch, int offset, int value);
    int  PeekDdcReg(int ch, int offset);
    void InitializeDdc(int device);
    void ResetDdcs()
        {
        UserFpgaControlReg.DdcReset.Set();
        UserFpgaControlReg.DdcReset.Clear();
        Sleep(1);
        }

    void                Trigger(bool state);
    bool                Trigger() const;

    void                ExternalTrigger(int ch, bool state);
    bool                ExternalTrigger(int ch) const;

	void                PersistentExtTrigger(int  ch, bool state);
	bool                PersistentExtTrigger(int ch) const;

    void                Framed(int ch, int size);
    void                Unframed(int ch);

    void                AdcDestination(int ch, IIDestination dest);
    IIDestination       AdcDestination(int ch) const;

    void                AdcTestCounterOn(int ch, bool state);
    bool                AdcTestCounterOn(int ch) const;

    void                DdcDataFrameSize(int device, int words);
    int                 DdcDataFrameSize(int device) const;

    void                ClockTestEnable(bool state);
    bool                ClockTestEnable() const;

    bool                FifoIntegrityError(int ch) const;
    bool                FifoIntegrityChange(int ch) const;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
