//---------------------------------------------------------------------------
// DigitalReceiver_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <DigitalReceiver_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS DigitalReceiver  -- Racal Pmc Board object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DigitalReceiver
//---------------------------------------------------------------------------

DigitalReceiver::DigitalReceiver()
    :  PmcBoard(Device), Ram(Device.Ram), Device(this), FLoopbackMode(lmNone)
{
}

//---------------------------------------------------------------------------
//  destructor for class DigitalReceiver
//---------------------------------------------------------------------------

DigitalReceiver::~DigitalReceiver()
{
    Close();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t DigitalReceiver::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  DigitalReceiver::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  DigitalReceiver::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  DigitalReceiver::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  DigitalReceiver::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  DigitalReceiver::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  DigitalReceiver::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  DigitalReceiver::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & DigitalReceiver::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & DigitalReceiver::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  DigitalReceiver::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  DigitalReceiver::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  DigitalReceiver::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  DigitalReceiver::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  DigitalReceiver::Log( int type, int value )
{
    Device.Log(type, value);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & DigitalReceiver::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & DigitalReceiver::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & DigitalReceiver::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  DigitalReceiver::PciLoopbackMode() --
//------------------------------------------------------------------------------

void DigitalReceiver::PciLoopbackMode(LoopbackMode mode)
{
    FLoopbackMode = mode;

    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:
            mode_value = 0;
            break;
        case lmSerial:
            mode_value = 1;
            break;
        case lmParallel:
            mode_value = 2;
            break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  DigitalReceiver::PciLoopbackMode() --
//------------------------------------------------------------------------------

DigitalReceiver::LoopbackMode DigitalReceiver::PciLoopbackMode()
{
    return FLoopbackMode;
}
 
//---------------------------------------------------------------------------
//  DigitalReceiver::Divider() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::Divider(int value)
{
    Device.ClockSelectReg.Divider(value);
    return *this;
}
 
//---------------------------------------------------------------------------
//  DigitalReceiver::Divider() --
//---------------------------------------------------------------------------

int DigitalReceiver::Divider() const
{
    return Device.ClockSelectReg.Divider();
}

//---------------------------------------------------------------------------
//  Tx::DividerClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::DividerClockSelect(IIExtClkSelect value)
{
	Device.ClockSelectReg.DividerClockSelect.Value( (value == ecA) ? 0 : 1);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::DividerClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver::IIExtClkSelect DigitalReceiver::DividerClockSelect() const
{
    return static_cast<IIExtClkSelect>(Device.ClockSelectReg.DividerClockSelect.Value());
}

//---------------------------------------------------------------------------
//  DigitalReceiver::FpgaClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::FpgaClockSelect(IIExtClkSelect value)
{
	Device.ClockSelectReg.FpgaClockSelect.Value( (value == ecA) ? 0 : 1);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::FpgaClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver::IIExtClkSelect DigitalReceiver::FpgaClockSelect() const
{
    return static_cast<IIExtClkSelect>(Device.ClockSelectReg.FpgaClockSelect.Value());
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Clock() --
//---------------------------------------------------------------------------

Ics8442 & DigitalReceiver::Clock()
{
    return Device.Clock;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::AdcClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::AdcClockSelect(int pair, IIAdcClkSelect value)
{
    DigitalReceiverDeviceMap::ClockSelectRegister::IIAdcClkSelect clk;
    switch (value)
        {
        case acA:       clk = DigitalReceiverDeviceMap::ClockSelectRegister::acA;   break;
        case acB:       clk = DigitalReceiverDeviceMap::ClockSelectRegister::acB;   break;
        default:
        case acPll:     clk = DigitalReceiverDeviceMap::ClockSelectRegister::acPll; break;
        }

    Device.ClockSelectReg.AdcClockSelect(pair, clk);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::AdcClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver::IIAdcClkSelect DigitalReceiver::AdcClockSelect(int pair) const
{
    IIAdcClkSelect clk;
    switch (Device.ClockSelectReg.AdcClockSelect(pair))
        {
        case DigitalReceiverDeviceMap::ClockSelectRegister::acA:       clk = acA;   break;
        case DigitalReceiverDeviceMap::ClockSelectRegister::acB:       clk = acB;   break;
        default:
        case DigitalReceiverDeviceMap::ClockSelectRegister::acPll:     clk = acPll; break;
        }

    return clk;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::PllClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::PllClockSelect(IIPllClkSelect value)
{
    Ics8442::IIReferenceClk clk;
    switch (value)
        {
        case pcDivider:     clk = Ics8442::rcDivider;   break;
        default:
        case pcCrystal:     clk = Ics8442::rcCrystal;   break;
        }

    Device.Clock.ReferenceClock(clk);
    return *this;
}
 
//---------------------------------------------------------------------------
//  DigitalReceiver::PllClockSelect() --
//---------------------------------------------------------------------------

DigitalReceiver::IIPllClkSelect DigitalReceiver::PllClockSelect() const
{
    DigitalReceiver::IIPllClkSelect clk;

    switch (Device.Clock.ReferenceClock())
        {
        case Ics8442::rcDivider:    clk = pcDivider;    break;
        default:
        case Ics8442::rcCrystal:    clk = pcCrystal;    break;
        }

    return clk;
}
 
//---------------------------------------------------------------------------
//  DigitalReceiver::TestMode() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::TestMode(IITestMode output)
{
    Ics8442::IITestMode mode;

    switch (output)
        {
        case toDisabled:            mode = Ics8442::toDisabled;         break;
        case toShiftReg:            mode = Ics8442::toShiftReg;         break;
        case toFeedbackDivider:     mode = Ics8442::toFeedbackDivider;  break;
        default:
        case toNormal:              mode = Ics8442::toNormal;           break;
        }

    Device.Clock.TestMode(mode);
    return *this;
}
 
//---------------------------------------------------------------------------
//  DigitalReceiver::TestMode() --
//---------------------------------------------------------------------------

DigitalReceiver::IITestMode DigitalReceiver::TestMode() const
{
    IITestMode mode;
    switch (Device.Clock.TestMode())
        {
        case toDisabled:            mode = toDisabled;         break;
        case toShiftReg:            mode = toShiftReg;         break;
        case toFeedbackDivider:     mode = toFeedbackDivider;  break;
        default:
        case toNormal:              mode = toNormal;           break;
        }

    return mode;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPllCalibration & DigitalReceiver::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ReturnPacketSize() -- Set size of packet coming in
//---------------------------------------------------------------------------

DigitalReceiver &  DigitalReceiver::ReturnPacketSize(int ch, int size_words)
{
    Device.ReturnPacketSize(ch, size_words);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ReturnPacketSize() -- Set size of packet coming in
//---------------------------------------------------------------------------

int  DigitalReceiver::ReturnPacketSize(int ch) const
{
    return Device.ReturnPacketSize(ch);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::AdcDestination() -- Route channel output
//---------------------------------------------------------------------------

DigitalReceiver &  DigitalReceiver::AdcDestination(int ch, DigitalReceiver::IIDestination dest)
{
    Device.AdcDestination(ch, static_cast<DigitalReceiverDeviceMap::IIDestination>(dest));

    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::AdcDestination() -- Query channel routing
//---------------------------------------------------------------------------

DigitalReceiver::IIDestination  DigitalReceiver::AdcDestination(int ch) const
{
    return static_cast<DigitalReceiver::IIDestination>(Device.AdcDestination(ch));
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ExternalTrigger() -- use external trigger
//---------------------------------------------------------------------------

DigitalReceiver &  DigitalReceiver::ExternalTrigger(int ch, bool state)
{
    Device.ExternalTrigger(ch, state);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ExternalTrigger() -- Route channels
//---------------------------------------------------------------------------

bool  DigitalReceiver::ExternalTrigger(int ch) const
{
	return Device.ExternalTrigger(ch) ? true : false;
}

//---------------------------------------------------------------------------
// DigitalReceiver::PersistentExtTrigger()
//---------------------------------------------------------------------------

DigitalReceiver	& DigitalReceiver::PersistentExtTrigger(int ch, bool state)
{
	Device.PersistentExtTrigger(ch, state);
	return *this;
}

//---------------------------------------------------------------------------
// DigitalReceiver::PersistentExtTrigger()
//---------------------------------------------------------------------------

bool DigitalReceiver::PersistentExtTrigger(int ch) const
{
	return Device.PersistentExtTrigger(ch) ? true : false;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Framed() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::Framed(int ch, int size)
{
    Device.Framed(ch, size);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Unframed() --
//---------------------------------------------------------------------------

DigitalReceiver & DigitalReceiver::Unframed(int ch)
{
    Device.Unframed(ch);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Trigger() -- Set trigger enable
//---------------------------------------------------------------------------

void DigitalReceiver::Trigger(bool state)
{
    Device.Trigger(state);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::Trigger() -- Query trigger enable
//---------------------------------------------------------------------------

int DigitalReceiver::Trigger() const
{
    return Device.Trigger();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::AdcTestCounterOn() -- Enable channels
//---------------------------------------------------------------------------

DigitalReceiver &  DigitalReceiver::AdcTestCounterOn(int ch, bool state)
{
    Device.AdcTestCounterOn(ch, state);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::AdcTestCounterOn() -- Enable channels
//---------------------------------------------------------------------------

bool  DigitalReceiver::AdcTestCounterOn(int ch) const
{
    return Device.AdcTestCounterOn(ch);
}

//------------------------------------------------------------------------------
//  DigitalReceiver::Channels() --  Reference to channel information object
//------------------------------------------------------------------------------

UnpairedChannelInfo & DigitalReceiver::Channels()
{
    return Device.FChannels;
}

//------------------------------------------------------------------------------
//  DigitalReceiver::PokeDdcReg() --
//------------------------------------------------------------------------------

void  DigitalReceiver::PokeDdcReg(int ch, int offset, int value)
{
    Device.PokeDdcReg(ch, offset, value);
}

//------------------------------------------------------------------------------
//  DigitalReceiver::PeekDdcReg() --
//------------------------------------------------------------------------------

int DigitalReceiver::PeekDdcReg(int ch, int offset)
{
    return Device.PeekDdcReg(ch, offset);
}

//------------------------------------------------------------------------------
//  DigitalReceiver::ResetDdcs() --
//------------------------------------------------------------------------------

void DigitalReceiver::ResetDdcs()
{
    Device.ResetDdcs();
}

//------------------------------------------------------------------------------
//  DigitalReceiver::ExecuteDdcScript() --
//------------------------------------------------------------------------------

void DigitalReceiver::ExecuteDdcScript(std::string file, int channel)
{
    Device.Script.Execute(file, channel);
}

//------------------------------------------------------------------------------
//  DigitalReceiver::InitDdcs() --
//------------------------------------------------------------------------------

void DigitalReceiver::DdcDataFrameSize(int device, int words)
{
    Device.DdcDataFrameSize(device, words);
}

//------------------------------------------------------------------------------
//  DigitalReceiver::InitDdcs() --
//------------------------------------------------------------------------------

int  DigitalReceiver::DdcDataFrameSize(int device) const
{
    return Device.DdcDataFrameSize(device);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ClockTestEnable() --
//---------------------------------------------------------------------------

void DigitalReceiver::ClockTestEnable(bool state)
{
    Device.ClockTestEnable(state);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ClockTestEnable() --
//---------------------------------------------------------------------------

bool DigitalReceiver::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  DigitalReceiver::FifoIntegrityError() --
//---------------------------------------------------------------------------

bool DigitalReceiver::FifoIntegrityError(int ch) const
{
    return Device.FifoIntegrityError(ch);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::FifoIntegrityChange() --
//---------------------------------------------------------------------------

bool DigitalReceiver::FifoIntegrityChange(int ch) const
{
    return Device.FifoIntegrityChange(ch);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::BusMasterSize() --
//---------------------------------------------------------------------------

void DigitalReceiver::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  DigitalReceiver::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void DigitalReceiver::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  DigitalReceiver::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void DigitalReceiver::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  DigitalReceiver::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  DigitalReceiver::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  DigitalReceiver::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // namespace Innovative
