// DigitalReceiver_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef DigitalReceiver_MbH
#define DigitalReceiver_MbH

#include <PmcBoard_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <DigitalReceiverDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class DigitalReceiverDeviceMap;

//===========================================================================
//  CLASS DigitalReceiver  -- Digital Receiver Base Board object
//===========================================================================

class DigitalReceiver :  public PmcBoard,
                         public IStreamDeviceEvents
{
    friend class DigitalReceiverDeviceMap;
    
public:
    //
    //  Member Functions
    DigitalReceiver();
    virtual ~DigitalReceiver();

    size_t          BoardCount();
    void            Open();
    void            Close();

    int   Target();
    void  Target( int value );

    bool  Led();
    void  Led( bool value );

    FpgaLogicVersionInfoIntf & UserLogicVersion();
    PciLogicVersionInfoIntf & PciLogicVersion();
    IPciBusDiagnostics & PciDiagnostics();

    ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );

    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );

    int   PeekDdcReg(int ch, int offset);
    void  PokeDdcReg(int ch, int offset, int value);

    // Logic programming
    IUsesVirtexFpgaLoader & Logic();
    // EEprom programming
    IUsesVirtexJtagLoader & Eeprom();

    // Trigger Support
    // Enumerations
    enum LoopbackMode { lmNone, lmParallel, lmSerial };

    // Methods
    void            PciLoopbackMode(LoopbackMode mode);
    LoopbackMode    PciLoopbackMode();

    enum IIExtClkSelect { ecA, ecB };
    enum IIAdcClkSelect { acA, acB, acPll };
    enum IIInternalClkSelect { acNormal, acOther };
    enum IIPllClkSelect { pcDivider, pcCrystal };
    enum IITestMode { toDisabled, toShiftReg, toFeedbackDivider, toNormal };
    enum IIDestination { dPci, dJ4, dXmc };

    // Properties
    DigitalReceiver &   ReturnPacketSize(int ch, int size_words);
    int                 ReturnPacketSize(int ch) const;

    DigitalReceiver &   AdcDestination(int ch, IIDestination dest);
    IIDestination       AdcDestination(int ch) const;

    DigitalReceiver &   AdcTestCounterOn(int ch, bool state);
    bool                AdcTestCounterOn(int ch) const;

    DigitalReceiver &   ExternalTrigger(int ch, bool state);
	bool                ExternalTrigger(int ch) const;

	DigitalReceiver &   PersistentExtTrigger(int ch, bool state);
	bool				PersistentExtTrigger(int ch) const;

    DigitalReceiver &   Framed(int ch, int size);
    DigitalReceiver &   Unframed(int ch);

    void                Trigger(bool state);
    int                 Trigger() const;

    DigitalReceiver &   Divider(int value);
    int					Divider() const;

    DigitalReceiver &   DividerClockSelect(IIExtClkSelect value);
    IIExtClkSelect      DividerClockSelect() const;

    DigitalReceiver &   FpgaClockSelect(IIExtClkSelect value);
    IIExtClkSelect      FpgaClockSelect() const;

    DigitalReceiver &   PllClockSelect(IIPllClkSelect value);
    IIPllClkSelect		PllClockSelect() const;

    DigitalReceiver &   AdcClockSelect(int ch, IIAdcClkSelect value);
    IIAdcClkSelect		AdcClockSelect(int ch) const;

    void                ClockTestEnable(bool state);
    bool                ClockTestEnable() const;

    DigitalReceiver &   TestMode(IITestMode output);
    IITestMode			TestMode() const;

    void                DdcDataFrameSize(int device, int words);
    int                 DdcDataFrameSize(int device) const;

    bool                FifoIntegrityError(int ch) const;
	bool                FifoIntegrityChange(int ch) const;

	void 				BusMasterSize(int bytes);
	void 				IncomingBusMasterSize(int bytes);
	void 				OutgoingBusMasterSize(int bytes);
	int             	IncomingBusMasterSize() const;
	int             	OutgoingBusMasterSize() const;

    // Methods
    Ics8442 &			    Clock();
    IUsesPllCalibration &	IdRom();
    UnpairedChannelInfo &   Channels();

    void                    ResetDdcs();
    void                    ExecuteDdcScript(std::string file, int channel=0);

    // Zbt Ram
    ZbtRam &                Ram;

    OpenWire::SynchronizedEventHandler<Innovative::ProcessStatusEvent> OnScriptMessage;

	//
    //  IPacketStreamDevice interface implementation
//protected:
    virtual void  Reset();
    DigitalReceiverDeviceMap * Debug() { return &Device; };
    virtual void   Log(int type, int data);

protected:
    //
    //  Member Data
    DigitalReceiverDeviceMap    Device;
    LoopbackMode				FLoopbackMode;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

