// DioPortIntf_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef DioPortIntf_Mb_H
#define DioPortIntf_Mb_H

#include <HardwareRegister_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS DioPortIntf -- Interface to DIO port
//===========================================================================

class DioPortIntf
{
public:
    // Dtor
	virtual ~DioPortIntf()
		{}
	//
	//  Interface Functions
	virtual IBitManipulator &   DioPortData() = 0;

	virtual void 			    DioPortConfig(unsigned int) = 0;
	virtual unsigned int 	    DioPortConfig() const = 0;
};

//===========================================================================
//  CLASS K7DioPortIntf -- "Single Word" style with individual bit config
//===========================================================================

class K7DioPortIntf : public DioPortIntf
{
public:
    virtual ~K7DioPortIntf()
        {  }

    virtual void                DioPortOutputEnable(int output_bit, bool output_enabled) = 0;
    virtual bool                DioPortOutputEnable(int output_bit) const = 0;
};

//===========================================================================
//  CLASS XUDioPortIntf -- "Single Word" style with individual bit config
//===========================================================================

typedef K7DioPortIntf  XUDioPortIntf;

//===========================================================================
//  CLASS ExtendedDioPortIntf -- Interface to upper bits of DIO port
//===========================================================================

class ExtendedDioPortIntf : public DioPortIntf
{
public:
	virtual ~ExtendedDioPortIntf()
        {  }

    //
	//  Interface Functions
    virtual IBitManipulator &   DioPortDataHigh() = 0;
};

//===========================================================================
//  CLASS FrontPanelPortIntf -- Interface to front panel port
//===========================================================================

class FrontPanelPortIntf
{
public:
    // Dtor	
	virtual ~FrontPanelPortIntf()
		{}
	//
	//  Interface Functions
	virtual IBitManipulator &   FrontPanelPortData() = 0;

	virtual void 			    FrontPanelPortConfig(unsigned int) = 0;
	virtual unsigned int 	    FrontPanelPortConfig() const = 0;
};

//===========================================================================
//  CLASS AdvancedDioPortIntf -- "X6" style with individual bit config
//===========================================================================

class AdvancedDioPortIntf : public ExtendedDioPortIntf
{
public:
	virtual ~AdvancedDioPortIntf()
        {  }

    //
	//  Interface Functions (upper 32 output configurations)
	virtual void 			    DioPortConfigHigh(unsigned int mask) = 0;
	virtual unsigned int 	    DioPortConfigHigh() const = 0;

	virtual void                DioPortOutputEnable(int output_bit, bool output_enabled) = 0;
	virtual bool                DioPortOutputEnable(int output_bit) const = 0;
};

//===========================================================================
//  CLASS FmcLaDioPortIntf -- FMC LA DIO (low, middle high registers) with individual bit config
//===========================================================================

class FmcLaDioPortIntf
{
public:
        virtual ~FmcLaDioPortIntf()
        {  }

        //  lower 32 bit I/O:
        virtual IBitManipulator &   DioPortDataLow() = 0;
        virtual void                DioPortConfigLow(unsigned int) = 0;
        virtual unsigned int        DioPortConfigLow() const = 0;

        //  middle 32 bit I/O:
        virtual IBitManipulator &   DioPortDataMiddle() = 0;
        virtual void                DioPortConfigMiddle(unsigned int) = 0;
        virtual unsigned int        DioPortConfigMiddle() const = 0;

        //  upper 4 bit I/O:
        virtual IBitManipulator &   DioPortDataHigh() = 0;
        virtual void                DioPortConfigHigh(unsigned int mask) = 0;
        virtual unsigned int        DioPortConfigHigh() const = 0;

        virtual void                DioPortOutputEnable(int output_bit, bool output_enabled) = 0;
        virtual bool                DioPortOutputEnable(int output_bit) const = 0;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // end namespace Innovative


#endif
