// DioPort_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DioPort_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS IUsesNullDioPort -- Mock logic DIO port
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class IUsesDioPort
//---------------------------------------------------------------------------

IUsesNullDioPort::IUsesNullDioPort()
{
}

//---------------------------------------------------------------------------
//  IUsesNullDioPort::DioPortConfig() --  Configure port
//---------------------------------------------------------------------------

void IUsesNullDioPort::DioPortConfig(unsigned int /*value*/)
{
}

//---------------------------------------------------------------------------
//  IUsesNullDioPort::DioPortConfig() --  Query current configuration
//---------------------------------------------------------------------------

unsigned int IUsesNullDioPort::DioPortConfig() const
{
	return 0;
}

//===========================================================================
//  CLASS IUsesDioPort -- Interface to lower bits of DI0 port
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class IUsesDioPort
//---------------------------------------------------------------------------

IUsesDioPort::IUsesDioPort(AddressingSpace & space,
	unsigned int data_addr, unsigned int cfg_addr)
	: DataReg(space, data_addr), ConfigReg(space, cfg_addr)
{
}

//---------------------------------------------------------------------------
//  IUsesDioPort::DioPortConfig() --  Configure port
//---------------------------------------------------------------------------

void IUsesDioPort::DioPortConfig(unsigned int value)
{
	ConfigReg.Value(value);
}

//---------------------------------------------------------------------------
//  IUsesDioPort::DioPortConfig() --  Query current configuration
//---------------------------------------------------------------------------

unsigned int IUsesDioPort::DioPortConfig() const
{
	return ConfigReg.Value();
}

//===========================================================================
//  CLASS IUsesExtendedDioPort -- Interface to upper bits of DI0 port
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class IUsesExtendedDioPort
//---------------------------------------------------------------------------

IUsesExtendedDioPort::IUsesExtendedDioPort(AddressingSpace & space,
	                                       unsigned int data_addr,
                                           unsigned int data_high_addr,
                                           unsigned int cfg_addr)
	: IUsesDioPort(space, data_addr, cfg_addr),
      DataHighReg(space, data_high_addr)
{
}

//===========================================================================
//  CLASS IUsesFrontPanelPort -- Interface to front panel logic IO
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class IUsesFrontPanelPort
//---------------------------------------------------------------------------

IUsesFrontPanelPort::IUsesFrontPanelPort(AddressingSpace & space,
	unsigned int data_addr, unsigned int cfg_addr)
	: DataReg(space, data_addr), ConfigReg(space, cfg_addr)
{
}

//---------------------------------------------------------------------------
//  IUsesDioPort::FrontPanelPortConfig() --  Configure port
//---------------------------------------------------------------------------

void IUsesFrontPanelPort::FrontPanelPortConfig(unsigned int value)
{
	ConfigReg.Value(value);
}

//---------------------------------------------------------------------------
//  IUsesDioPort::FrontPanelPortConfig() --  Query current configuration
//---------------------------------------------------------------------------

unsigned int IUsesFrontPanelPort::FrontPanelPortConfig() const
{
	return ConfigReg.Value();
}


}  // end namespace Innovative



