// DioPort_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef DioPort_MbH
#define DioPort_MbH

#include <DioPortIntf_Mb.h>
#include <HardwareRegister_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IUsesNullDioPort -- Mock logic DIO port
//===========================================================================

class IUsesNullDioPort : public DioPortIntf
{
public:
	IUsesNullDioPort();

	//
	//  Interface Functions
    virtual IBitManipulator &   DioPortData()
        {  return NullReg;  }

	virtual void 			DioPortConfig(unsigned int /*value*/);
	virtual unsigned int 	DioPortConfig() const;

protected:
    class NullRegister : public IBitManipulator
    {
    public:
        NullRegister()
            {  }

    protected:
        // Overrides
        virtual unsigned int  ReadV() const
            {  return 0;  }
        virtual void          WriteV(unsigned int /*value*/)
            {  }
    };

	// Registers
	NullRegister			NullReg;
};

//===========================================================================
//  CLASS IUsesDioPort -- Interface to lower bits of DIO port
//===========================================================================

class IUsesDioPort : public DioPortIntf
{
public:
	IUsesDioPort(AddressingSpace & space, unsigned int data_addr,
		         unsigned int cfg_addr);

	//
	//  Interface Functions
    virtual IBitManipulator &   DioPortData()
        {  return DataReg;  }

	virtual void 			    DioPortConfig(unsigned int value);
	virtual unsigned int 	    DioPortConfig() const;

protected:
    class DataRegister : public IBitManipulator
    {
    public:
        DataRegister(AddressingSpace & space, unsigned int data_addr)
            : DataReg(space, data_addr)
            {  }

    protected:
        // Registers.
    	Register				DataReg;

        // Overrides
        virtual unsigned int  ReadV() const
            {  return DataReg.Value();  }
        virtual void          WriteV(unsigned int value)
            {  DataReg.Value(value);  }
    };

	// Registers
	DataRegister			DataReg;
	ShadowRegister			ConfigReg;
};

//===========================================================================
//  CLASS IUsesExtendedDioPort -- Interface to upper bits of DIO port
//===========================================================================

class IUsesExtendedDioPort : public IUsesDioPort
{
public:
    IUsesExtendedDioPort(AddressingSpace & space, unsigned int data_addr,
		         unsigned int data_high_addr, unsigned int cfg_addr);

    //
	//  Interface Functions
    virtual IBitManipulator &   DioPortDataHigh()
        {  return DataHighReg;  }

protected:
    class DataHighRegister : public IBitManipulator
    {
    public:
        DataHighRegister(AddressingSpace & space, unsigned int data_high_addr)
            : DataReg(space, data_high_addr)
            {  }

    protected:
        // Registers.
    	Register				DataReg;

        // Overrides
        virtual unsigned int  ReadV() const
            {  return DataReg.Value();  }
        virtual void          WriteV(unsigned int value)
            {  DataReg.Value(value);  }
    };

    // Registers
	DataHighRegister        DataHighReg;
};

//===========================================================================
//  CLASS IUsesNullFrontPanelPort -- Interface to front panel DIO port
//===========================================================================

class IUsesNullFrontPanelPort : public FrontPanelPortIntf
{
public:
	IUsesNullFrontPanelPort()
        {  }

	//
	//  Interface Functions
    virtual IBitManipulator &   FrontPanelPortData()
        {  return NullReg;  }

	virtual void 			    FrontPanelPortConfig(unsigned int)
        {  }
	virtual unsigned int 	    FrontPanelPortConfig() const
        {  return 0;  }

protected:
    class NullRegister : public IBitManipulator
    {
    public:
        NullRegister()
            {  }

    protected:
        // Overrides
        virtual unsigned int  ReadV() const
            {  return 0;  }
        virtual void          WriteV(unsigned int /*value*/)
            {  }
    };

	// Registers
	NullRegister			NullReg;
};

//===========================================================================
//  CLASS IUsesFrontPanelPort -- Interface to front panel DIO port
//===========================================================================

class IUsesFrontPanelPort : public FrontPanelPortIntf
{
public:
	IUsesFrontPanelPort(AddressingSpace & space, unsigned int data_addr,
		         unsigned int cfg_addr);

	//
	//  Interface Functions
    virtual IBitManipulator &   FrontPanelPortData()
        {  return DataReg;  }

	virtual void 			    FrontPanelPortConfig(unsigned int value);
	virtual unsigned int 	    FrontPanelPortConfig() const;

protected:
    class DataRegister : public IBitManipulator
    {
    public:
        DataRegister(AddressingSpace & space, unsigned int data_addr)
            : DataReg(space, data_addr)
            {  }

    protected:
        // Registers.
    	Register				DataReg;

        // Overrides
        virtual unsigned int  ReadV() const
            {  return DataReg.Value();  }
        virtual void          WriteV(unsigned int value)
            {  DataReg.Value(value);  }
    };

	// Registers
	DataRegister			DataReg;
	ShadowRegister			ConfigReg;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
