#ifndef __DRIVER_CONTROLLER_H__
#define __DRIVER_CONTROLLER_H__

#include <string>
#include <vector>
#include <KernelPci_Mb.h>
#include <Trace_Mb.h>
#include <Poco/Activity.h>



namespace InnovativeKernel {
	namespace Osal {
		class OsDevice;
	}
}

namespace Innovative
{

using std::string;
using std::vector;
using Poco::Activity;
using InnovativeKernel::MemoryBlock;
using InnovativeKernel::Osal::OsDevice;


#define DMA_MAPPING (1 << 7)

typedef struct {
	unsigned int type;
	unsigned int offset;
	unsigned int mask;
} InterruptInfo;


typedef unsigned int dma_addr_t;

typedef struct
{
	unsigned int *  uaddr;
	unsigned int *  kaddr;
	unsigned int    paddr;
	unsigned int    size;
	unsigned int    index;
	unsigned        mapped;
	dma_addr_t      handle;
} dma_memory_handle_t;


/*
 * Per board private
 */

#define DEVICE_QUEUE_DEPTH 128

typedef struct
{
	unsigned int Depth;
	unsigned int Queue[DEVICE_QUEUE_DEPTH + 1];
} queue_snapshot_t;

#define IIPCI_MAX_MINORS   16
typedef unsigned short u16;

typedef struct
{
	u16 pci_vendor;
	u16 pci_devid;
	u16 pci_subsysid;
} device_id_t;

typedef struct
{
	device_id_t ids[IIPCI_MAX_MINORS];
	u16 count;
} device_ids_t;


/* Innovative Magic Number */
#define II_IOCTL_MAGIC_NUM  'h'

#define  IOCTL_ALLOC_DMA_MEMORY     _IOWR(II_IOCTL_MAGIC_NUM, 10,dma_memory_handle_t*)
#define  IOCTL_FREE_DMA_MEMORY      _IOR (II_IOCTL_MAGIC_NUM, 11,dma_memory_handle_t*)
#define  IOCTL_SET_INTERRUPT_INFO   _IOW (II_IOCTL_MAGIC_NUM, 12,InterruptInfo*)
#define  IOCTL_INTERRUPT_CLEARQ     _IO  (II_IOCTL_MAGIC_NUM, 13)
#define  IOCTL_INTERRUPT_QDEPTH     _IOR (II_IOCTL_MAGIC_NUM, 14,int)
#define  IOCTL_WAIT_INTERRUPT       _IO  (II_IOCTL_MAGIC_NUM, 15)
#define  IOCTL_GET_INTERRUPT_BLOCK  _IOR (II_IOCTL_MAGIC_NUM, 16,queue_snapshot_t*)
#define  IOCTL_GET_MODULE_COUNT     _IOR (II_IOCTL_MAGIC_NUM, 17,int)
#define  IOCTL_GET_BAR_SIZE         _IOWR(II_IOCTL_MAGIC_NUM, 18,int)
#define  IOCTL_INTERRUPT_KICK       _IO  (II_IOCTL_MAGIC_NUM, 19)
#define  IOCTL_GET_IDS              _IOR (II_IOCTL_MAGIC_NUM, 20,device_ids_t*)



class Event;


class DriverController
{
public:
	DriverController(unsigned int id, OsDevice &owner);
	~DriverController();

	bool open(unsigned int target);
	void close();

	int deviceCount();

    vector<string> getDeviceNames();
	int barRegionSize(int bar);

	MemoryBlock mapBarRegion(unsigned int bar);
	MemoryBlock mapDmaMemory(size_t size);
	void unmapBarRegion(const MemoryBlock &mem);
	void unmapDmaMemory(const MemoryBlock &mem);

	void clearInterruptQueue();
	size_t getInterruptQueueDepth();
	void setInterruptInfo(unsigned int itype, unsigned int ioffset, unsigned int imask);
	void enableInterrupt();
	void disableInterrupt();
	bool isInterruptEnabled();
	void installInterruptEventHandler(Event *isrEvent);


    string DriverVersionStr() { return ""; }
    string DriverDateStr() { return ""; }

protected:
	void runActivity();
	void interruptibleWait();

private:
	vector<string> mapIdsToNames(const vector<device_id_t> &ids);
	string deviceName(const Innovative::device_id_t& id);
    vector<device_id_t> deviceIds();

	typedef Activity<DriverController>  ActivityType;

	const size_t     _page_sz;
	unsigned int     _id;
	unsigned         _target;
	int              _fd;
	OsDevice        &_owner;
	ActivityType     _activity;
	Event           *_event;

	Trace            debug;

	vector<string>   deviceNames;
};


} // namespace Innovative

#endif
