
#include "DriverController_Win.h"

#include <sstream>
#include <stdexcept>
#include <iostream>
#include <Event_Mb.h>
#include <Osal_Mb.h>
#include <Trace_Mb.h>
#include <Poco/Format.h>
#include <Exception_Mb.h>
#include <MemDriver_Mb.h>
#include <StringSupport_Mb.h>

namespace   Innovative
{
using InnovativeKernel::Osal::OsDevice;
using Poco::format;

//------------------------------------------------------------------------------
//  DriverController::DriverController() --
//------------------------------------------------------------------------------

DriverController::DriverController(unsigned int id, OsDevice &owner)
   :  _id(id), _target(-1), _fd(-1),
     _owner(owner), _activity(this, &DriverController::runActivity), _event(0)
{
    BBoardCount = 0;
    scanPci();
}

//------------------------------------------------------------------------------
//  DriverController::~DriverController() --
//------------------------------------------------------------------------------

DriverController::~DriverController()
{

}

//------------------------------------------------------------------------------
//  DriverController::open() --
//------------------------------------------------------------------------------

bool DriverController::open(unsigned int target)
{
    ULONG       status;
    ULONG		size;
    TCHAR		*DeviceName = NULL;
    TCHAR		*DeviceLocation = NULL;

    //  Retreive the device information for all II devices.
    hDevInfo = SetupDiGetClassDevs(&GUID_DEVINTERFACE_IIDrvr, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

    //Initialize the appropriate data structures in preparation for the SetupDi calls.
    DeviceInterfaceData.cbSize  = sizeof(SP_DEVICE_INTERFACE_DATA);
    DeviceInfoData.cbSize       = sizeof(SP_DEVINFO_DATA);

    SetupDiEnumDeviceInterfaces(hDevInfo, NULL, (LPGUID)&GUID_DEVINTERFACE_IIDrvr, target, &DeviceInterfaceData);
    // Determine the size required for the DeviceInterfaceData
    SetupDiGetDeviceInterfaceDetail(hDevInfo, &DeviceInterfaceData, NULL, 0, &size, NULL);

    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
        {
        throw MalibuException("SetupDiGetDeviceInterfaceDetail failed");
        }

    pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
    if (!pDeviceInterfaceDetail)
        {
        throw MalibuException("Insufficient memory.\n");
        }

    // Initialize structure and retrieve data.
    pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
    status = SetupDiGetDeviceInterfaceDetail(hDevInfo, &DeviceInterfaceData, pDeviceInterfaceDetail, size, NULL, &DeviceInfoData);
    if (!status)
        {
        throw MalibuException("SetupDiGetDeviceInterfaceDetail failed");
        }

    //Get the Device Name. Calls to SetupDiGetDeviceRegistryProperty require two consecutive calls,
    //first to get required buffer size and second to get the data.
    SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_DEVICEDESC, NULL, (PBYTE)DeviceName, 0, &size);
    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
        {
        throw MalibuException("SetupDiGetDeviceRegistryProperty failed");
        }

    DeviceName = (TCHAR*)malloc(size);
    if (!DeviceName)
        {
        throw MalibuException("Insufficient memory.\n");
        }

    status = SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_DEVICEDESC, NULL, (PBYTE)DeviceName, size, NULL);
    if (!status)
        {
        free(DeviceName);
        throw MalibuException("SetupDiGetDeviceRegistryProperty failed");
        }

    string DevName = DeviceName;
    stringstream InfoMsg;
    InfoMsg << "Trying to open " << DevName;
    Debug.Log(InfoMsg.str().c_str());

    //Now retrieve the Device Location.
    SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_LOCATION_INFORMATION, NULL, (PBYTE)DeviceLocation, 0, &size);

    if (GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
        DeviceLocation = (TCHAR*)malloc(size);

        if (DeviceLocation != NULL)
            {
            status = SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_LOCATION_INFORMATION, NULL, (PBYTE)DeviceLocation, size, NULL);
            if (!status)
                {
                free(DeviceLocation);
                DeviceLocation = NULL;
                }
            }
        }
    else
        {
        DeviceLocation = NULL;
        }

    //Get Driver Version and Date Information
    SP_DRVINFO_DATA_V2_A DrvInfo;
    DrvInfo.cbSize = sizeof(SP_DRVINFO_DATA_V2_A);
    SetupDiBuildDriverInfoList(hDevInfo, &DeviceInfoData, SPDIT_COMPATDRIVER);
    SetupDiEnumDriverInfoA(hDevInfo, &DeviceInfoData, SPDIT_COMPATDRIVER, 0, &DrvInfo);

    //Save Driver Version Information
    ULARGE_INTEGER version;
    version.QuadPart = DrvInfo.DriverVersion;
    int DriverVersionMajor = HIWORD(version.HighPart);
    int DriverVersionMinor = LOWORD(version.HighPart);
    int DriverVersonSubMajor = HIWORD(version.LowPart);
    int DriverVersionSubMinor = LOWORD(version.LowPart);

    stringstream ssDriverVer;
    ssDriverVer << DriverVersionMajor << "." << DriverVersionMinor << "."
    << DriverVersonSubMajor << "." << DriverVersionSubMinor;
    DDriverVersionStr = ssDriverVer.str();

    //Save Driver Date Information
    SYSTEMTIME SysTime;
    FileTimeToSystemTime(&DrvInfo.DriverDate, &SysTime);
    stringstream Date;
    Date << SysTime.wMonth << "-" << SysTime.wDay << "-" << SysTime.wYear;
    DDriverDateStr = Date.str();

    // Open a handle to the device for regular IOCTLS
    Device = CreateFile(pDeviceInterfaceDetail->DevicePath, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                         NULL, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, NULL);
    if (Device == INVALID_HANDLE_VALUE)
        {
        throw MalibuException("Device open failed");
        }

    // open MemDrv
    DeviceMem = CreateFile(DEVICE_NAME, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                                NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    return true;
}

//------------------------------------------------------------------------------
//  DriverController::close() --
//------------------------------------------------------------------------------

void DriverController::close()
{
    Debug.Log("Closing Device");

    if (Device != INVALID_HANDLE_VALUE && Device != NULL)
        {
        CloseHandle(Device);
        Device = INVALID_HANDLE_VALUE;
        }

    if (DeviceMem != INVALID_HANDLE_VALUE)
        CloseHandle(DeviceMem);
}

//------------------------------------------------------------------------------
//  DriverController::deviceCount() --
//------------------------------------------------------------------------------

int DriverController::deviceCount()
{
    return BBoardCount;
}

//------------------------------------------------------------------------------
//  DriverController::scanPci() --
//------------------------------------------------------------------------------

void DriverController::scanPci()
{
    //Add this message to the log to make sure there are no redundant calls to ScanPci()
    Debug.Log("Scanning for II devices.");

    //Retreive the device information for all II devices.
    hDevInfo = SetupDiGetClassDevs(&GUID_DEVINTERFACE_IIDrvr, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

    //Initialize the appropriate data structures in preparation for the SetupDi calls.
    DeviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);

    //Determine how many devices are present.
    BBoardCount = 0;
    while (SetupDiEnumDeviceInterfaces(hDevInfo, NULL, &GUID_DEVINTERFACE_IIDrvr, BBoardCount++, &DeviceInterfaceData));

    // Since the last call fails when all devices have been enumerated, decrement the count to get the true device count.
    BBoardCount--;

    findDeviceNames();
}

//------------------------------------------------------------------------------
//  DriverController::findDeviceNames() --
//------------------------------------------------------------------------------

void DriverController::findDeviceNames()
{
    TCHAR                               *DeviceName = NULL;
    ULONG                               size;
    ULONG                               status;

    DeviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
    DeviceInfoData.cbSize      = sizeof(SP_DEVINFO_DATA);

    //Make sure the name vector is cleared before doing work in case this is run more than once from the app
    DDeviceNames.clear();

    for (unsigned int i = 0; i < BBoardCount; i++)
        {
        SetupDiEnumDeviceInterfaces(hDevInfo, NULL, (LPGUID)&GUID_DEVINTERFACE_IIDrvr, i, &DeviceInterfaceData);

        // Determine the size required for the DeviceInterfaceData
        SetupDiGetDeviceInterfaceDetail(hDevInfo, &DeviceInterfaceData, NULL, 0, &size, NULL);
        if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
            {
            throw MalibuException("SetupDiGetDeviceInterfaceDetail failed!");
            }

        pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
        if (!pDeviceInterfaceDetail)
            {
            throw MalibuException("SetupDiGetDeviceInterfaceDetail failed!");
            }

        // Initialize structure and retrieve data.
        pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
        status = SetupDiGetDeviceInterfaceDetail(hDevInfo, &DeviceInterfaceData, pDeviceInterfaceDetail, size, NULL, &DeviceInfoData);
        if (!status)
            {
            throw MalibuException("SetupDiGetDeviceInterfaceDetail failed!");
            }

        //Get the Device Name. Calls to SetupDiGetDeviceRegistryProperty require two consecutive calls,
        //first to get required buffer size and second to get the data.
        SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_DEVICEDESC, NULL, (PBYTE)DeviceName, 0, &size);

        if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
            {
            throw MalibuException("SetupDiGetDeviceInterfaceDetail failed!");
            }

        DeviceName = (TCHAR*)malloc(size);
        if (!DeviceName)
            {
            throw MalibuException("SetupDiGetDeviceInterfaceDetail failed!");
            }

        status = SetupDiGetDeviceRegistryProperty(hDevInfo, &DeviceInfoData, SPDRP_DEVICEDESC, NULL, (PBYTE)DeviceName, size, NULL);
        if (!status)
            {
            throw MalibuException("SetupDiGetDeviceInterfaceDetail failed!");
            free(DeviceName);
            }

        //Save a vector of names for each target
        DDeviceNames.push_back(DeviceName);

        //Display boards found at each index to the Debug Log
        stringstream msg;
        msg << "Found " << DeviceName << " as target: " << i;
        Debug.Log(msg.str());
        }

}

//------------------------------------------------------------------------------
//  DriverController::barRegionSize() --
//------------------------------------------------------------------------------

int DriverController::barRegionSize(int bar)
{
    static int sizes[] = {256 * 1024, 0, 1 * 1024 * 1024};

    return (bar == 0 || bar == 2) ? sizes[bar] : 0;
}

//------------------------------------------------------------------------------
//  DriverController::mapBarRegion() --
//------------------------------------------------------------------------------

MemoryBlock DriverController::mapBarRegion(unsigned int bar)
{
    MemoryBlock Block;

    DWORD dwReturn;
    if(bar == 0)
        {
        //This IOCTL gets both port and Logic if it exists so it only needs to be done once
        Debug.Log("Getting Board Resource Information.");
        DeviceIoControl(Device, IOCTL_GET_BD_RESOURCES_1303, NULL, 0, &MemInfo, sizeof(MemInfo), &dwReturn, NULL);
        Block.Size = MemInfo.PortSize;
        Block.Bar = 0;
        Block.RawPhysAddr = reinterpret_cast<unsigned int*>(MemInfo.PortPhysAddr.QuadPart);
        Block.PhysAddr = reinterpret_cast<unsigned int*>(MemInfo.PortPhysAddr.QuadPart);
        Block.DriverAddr = reinterpret_cast<unsigned int*>(MemInfo.PortKernelAddr);
        Block.Addr = reinterpret_cast<unsigned int*>(MemInfo.PortUserAddr);
        Block.WasMapped = true;
        }
    else
        {
        Block.Size = MemInfo.LogicSize;
        Block.Bar = 2;
        Block.RawPhysAddr = reinterpret_cast<unsigned int*>(MemInfo.LogicPhysAddr.QuadPart);
        Block.PhysAddr = reinterpret_cast<unsigned int*>(MemInfo.LogicPhysAddr.QuadPart);
        Block.DriverAddr = reinterpret_cast<unsigned int*>(MemInfo.LogicKernelAddr);
        Block.Addr = reinterpret_cast<unsigned int*>(MemInfo.LogicUserAddr);
        Block.WasMapped = true;
        }

    return Block;
}

//------------------------------------------------------------------------------
//  DriverController::unmapBarRegion() --
//------------------------------------------------------------------------------

void DriverController::unmapBarRegion(const MemoryBlock &mem)
{
    DWORD dwReturn;
    Debug.Log(format("BAR %u unmapped.", mem.Bar));

    if (mem.Bar == 0)
        DeviceIoControl(Device, IOCTL_FREE_MDL_1303, NULL, 0, NULL, 0, &dwReturn, NULL);
}

//------------------------------------------------------------------------------
//  DriverController::mapDmaMemory() --
//------------------------------------------------------------------------------

MemoryBlock DriverController::mapDmaMemory(size_t size)
{
    MemoryBlock Block;

    BOOL bRc;
    ULONG bytesReturned;
    DrvInputType memRequest;
    DrvOutputType DmaBuf;

    //  Create a new memory block and add to block list
   // OsDeviceImpl::AllocatedMemoryInfo AllocBlock;
    //  Allocate the block
    memRequest.RequestBytes = size;

    DWORD IOCTL_ALLOC_MEMORY_BLOCK;
//    if(is64BitCapable)
//         IOCTL_ALLOC_MEMORY_BLOCK = High_IOCTL_ALLOC_MEMORY_BLOCK;
//            else
   IOCTL_ALLOC_MEMORY_BLOCK = Low_IOCTL_ALLOC_MEMORY_BLOCK;

   bRc = DeviceIoControl (DeviceMem, IOCTL_ALLOC_MEMORY_BLOCK, &memRequest, sizeof(DrvInputType),
                          &DmaBuf, sizeof(DrvOutputType), &bytesReturned, NULL);

   if ( bRc )
       {
       //  Otherwise push Block onto stack, fill in Memory Block
       Block.PhysAddr    = reinterpret_cast<unsigned int *>(DmaBuf.PhysicalAddr);
       Block.Size        = size;
       Block.DriverAddr  = reinterpret_cast<unsigned int *>(DmaBuf.KernelVirtualAddr);
       Block.Addr        = (unsigned int *)DmaBuf.UserVirtualAddr;
       Block.RawPhysAddr = reinterpret_cast<unsigned int *>(DmaBuf.PhysicalAddr);
       Block.Bar         = 0xffffffff;
       Block.MemType     = InnovativeKernel::mtMemory;  // Memory
       Block.WasMapped   = false;      // Mapped.
       }
   else
       {
       throw MalibuException("MD: MemDriver BM allocate failed.");
       }

    Debug.Log(format("DMA memory [%u] of %u bytes alloced.", Block.Bar, Block.Size));

    return Block;
}

//------------------------------------------------------------------------------
//  DriverController::unmapDmaMemory() --
//------------------------------------------------------------------------------

void DriverController::unmapDmaMemory(const MemoryBlock &Block)
{
    DrvOutputType OutputBuffer;
    ULONG bytesReturned;
    DrvInputType freeMem;

    //test: free mem block - bad handle
    freeMem.UserVirtualAddr = Block.Addr;

    DWORD IOCTL_FREE_MEMORY_BLOCK;
//    if(is64BitCapable)
//    IOCTL_FREE_MEMORY_BLOCK = High_IOCTL_FREE_MEMORY_BLOCK;
//    else
    IOCTL_FREE_MEMORY_BLOCK = Low_IOCTL_FREE_MEMORY_BLOCK;

    DeviceIoControl(DeviceMem, IOCTL_FREE_MEMORY_BLOCK, &freeMem, sizeof(DrvInputType),
                    &OutputBuffer, sizeof(DrvOutputType), &bytesReturned, NULL);

    Debug.Log(format("DMA memory [%u] freed.", Block.Bar));
}

//------------------------------------------------------------------------------
//  DriverController::clearInterruptQueue() --
//------------------------------------------------------------------------------

void DriverController::clearInterruptQueue()
{
    //  Clear this intermediate queue now too
    DWORD dwReturn;
    OVERLAPPED Overlapped;
    memset(&Overlapped, 0, sizeof(Overlapped));
    Overlapped.hEvent = CreateEventA(NULL, FALSE, FALSE, NULL);
    Debug.Log("Clearing Interrupt Queue.");
    DeviceIoControl(Device, IOCTL_ISR_CLEARQ_1303, NULL, 0, NULL, 0, &dwReturn, &Overlapped);
}

//------------------------------------------------------------------------------
//  DriverController::getInterruptQueueDepth() --
//------------------------------------------------------------------------------

size_t DriverController::getInterruptQueueDepth()
{
   DWORD dwReturn;
   DWORD Depth;
   DeviceIoControl(Device, IOCTL_MSG_INTDEPTH_1303, &Depth, sizeof(Depth), &Depth, sizeof(Depth), &dwReturn, NULL);
   return Depth;
}

//------------------------------------------------------------------------------
//  DriverController::setInterruptInfo() --
//------------------------------------------------------------------------------

void DriverController::setInterruptInfo(unsigned int itype, unsigned int ioffset, unsigned int imask)
{
    DWORD dwReturn;
    INTERRUPT_INFO_1303 IntInfo;
    IntInfo.IntStatusMask = imask;
    IntInfo.IntStatusRegisterOffset = ioffset;
    IntInfo.IntHandlerType = itype;

    DeviceIoControl(Device, IOCTL_PREP_INTERRUPT_1303, &IntInfo, sizeof(IntInfo), &IntInfo, sizeof(IntInfo), &dwReturn, NULL);
}

//------------------------------------------------------------------------------
//  DriverController::enableInterrupt() --
//------------------------------------------------------------------------------

void DriverController::enableInterrupt()
{
     Debug.Log("enableInterrupt");
    _activity.start();
}

//------------------------------------------------------------------------------
//  DriverController::disableInterrupt() --
//------------------------------------------------------------------------------

void DriverController::disableInterrupt()
{
    DWORD dwReturn;
    _activity.stop();
    DeviceIoControl(Device, IOCTL_FREE_QUEUE_1303, NULL, 0, NULL, 0, &dwReturn, NULL);
    _activity.wait();
    Debug.Log("disableInterrupt");
}

//------------------------------------------------------------------------------
//  DriverController::isInterruptEnabled() --
//------------------------------------------------------------------------------

bool DriverController::isInterruptEnabled()
{
    return _activity.isRunning();
}

//------------------------------------------------------------------------------
//  DriverController::isInterruptEnabled() --
//------------------------------------------------------------------------------

void DriverController::installInterruptEventHandler(Event *isrEvent)
{
    _event = isrEvent;
}

//------------------------------------------------------------------------------
//  DriverController::getDeviceNames() --
//------------------------------------------------------------------------------

vector<string> DriverController::getDeviceNames()
{
    return DDeviceNames;
}

//------------------------------------------------------------------------------
//  DriverController::runActivity() --
//------------------------------------------------------------------------------

void DriverController::runActivity()
{
    DWORD dwReturn;
    INTSTAT_RESULT  IntStatBlock;

    while (!_activity.isStopped())
    {
        DeviceIoControl(Device, IOCTL_QUEUE_INTERRUPT_1303, NULL, 0, NULL, 0, &dwReturn, NULL);
        if(_activity.isStopped()) continue;
        IntStatBlock.Depth = 0;
        DeviceIoControl(Device, IOCTL_INTSTAT_RESULT_1303, &IntStatBlock, sizeof(INTSTAT_RESULT), &IntStatBlock, sizeof(INTSTAT_RESULT), &dwReturn, NULL);
        for (unsigned int i=0; i < IntStatBlock.Depth; i++)
            {
            //  Push ISR status word onto queue
            _owner.EnqueueIsrStatus(IntStatBlock.Queue[i]);
            _event->Set();
            }
    }
}

//------------------------------------------------------------------------------
//  DriverController::DriverVersionStr() -- Return Driver Version
//------------------------------------------------------------------------------

string DriverController::DriverVersionStr()
{
    return DDriverVersionStr;
}

//------------------------------------------------------------------------------
//  DriverController::DriverDateStr() -- Return Driver Date
//------------------------------------------------------------------------------

string DriverController::DriverDateStr()
{
    return DDriverDateStr;
}

} // namespace Malibu
