#ifndef __DRIVER_CONTROLLER_H__
#define __DRIVER_CONTROLLER_H__

#include <string>
#include <vector>
#include <KernelPci_Mb.h>
#include <Trace_Mb.h>
#include <Poco/Activity.h>
//The order of includes here is IMPORTANT!
#include <Windows.h>
#include <winioctl.h>
#include <initguid.h>
#include <setupapi.h>
#include <IIDrvr_Mb.h>
#include <Poco/Logger.h>


namespace InnovativeKernel {
	namespace Osal {
		class OsDevice;
	}
}

namespace Innovative
{

using std::string;
using std::vector;
using Poco::Activity;
using InnovativeKernel::MemoryBlock;
using InnovativeKernel::MemoryType;
using InnovativeKernel::Osal::OsDevice;


class DriverController
{
public:
	DriverController(unsigned int id, OsDevice &owner);
	~DriverController();

	bool open(unsigned int target);
	void close();

	int deviceCount();
    void findDeviceNames();
	int barRegionSize(int bar);

	MemoryBlock mapBarRegion(unsigned int bar);
	MemoryBlock mapDmaMemory(size_t size);
	void unmapBarRegion(const MemoryBlock &mem);
	void unmapDmaMemory(const MemoryBlock &mem);

	void clearInterruptQueue();
	size_t getInterruptQueueDepth();
	void setInterruptInfo(unsigned int itype, unsigned int ioffset, unsigned int imask);
	void enableInterrupt();
	void disableInterrupt();
	bool isInterruptEnabled();
	void installInterruptEventHandler(Event *isrEvent);

    vector<string> getDeviceNames();
    void scanPci();
    string DriverVersionStr();
    string DriverDateStr();

protected:
	void runActivity();
	void interruptibleWait();

private:
	typedef Activity<DriverController>  ActivityType;

	unsigned int     _id;
	unsigned         _target;
	int              _fd;
	OsDevice        &_owner;
	ActivityType     _activity;
	Event           *_event;

    Trace                                 Debug;
    MemoryInfo                            MemInfo;
    SP_DEVICE_INTERFACE_DATA              DeviceInterfaceData;
    SP_DEVINFO_DATA                       DeviceInfoData;
    HDEVINFO                              hDevInfo;
    PSP_DEVICE_INTERFACE_DETAIL_DATA      pDeviceInterfaceDetail;
    HANDLE                                Device;
    HANDLE                                DeviceMem;

    unsigned int                          BBoardCount;
    std::vector<string>                   DDeviceNames;
    string                                DDriverVersionStr;
    string                                DDriverDateStr;
};


} // namespace Innovative

#endif
