//
// Endian_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Endian_Mb.h>

namespace Innovative
{

//=============================================================================
//  Global Endian Reversal Functions
//=============================================================================

//-----------------------------------------------------------------------------
//  IsBigEndian() -- Returns true if the system is big endian
//-----------------------------------------------------------------------------

bool IsBigEndian()
{
    return !IsLittleEndian();
}

//-----------------------------------------------------------------------------
//  IsLittleEndian() -- Returns true if the system is little endian
//-----------------------------------------------------------------------------
bool IsLittleEndian()
{
    union {
        int  i;
        char c[sizeof(int)];
    } layout;
    layout.i = 1;

    if (layout.c[0] == 1)
        return true;
    else
        return false;
}

//-----------------------------------------------------------------------------
//  Endian() -- Change endian-ness of 2 byte signed integer
//-----------------------------------------------------------------------------

short Endian(short v)
{
    return ((v>>8)&0xff) | ((v&0xff)<<8);
}

//-----------------------------------------------------------------------------
//  Endian() -- Change endian-ness of 4 byte signed integers
//-----------------------------------------------------------------------------

int Endian(int v)
{
    return ((v>>24)&0xff) | ((v&0xff)<<24) | ((v>>8)&0xff00) | ((v&0xff00)<<8);
}

//-----------------------------------------------------------------------------
//  Endian() -- Change endian-ness of 4 byte float
//-----------------------------------------------------------------------------

float Endian(float v)
{
    union
    {
        int 	i;
        float	x;
    } Value;
    Value.x = v;

    Value.i = Endian(Value.i);

    return Value.x;
}

//-----------------------------------------------------------------------------
//  Endian() -- Change endian-ness of short buffer
//-----------------------------------------------------------------------------

void Endian(short * buffer, int size)
{
    for (int i = 0; i < size; ++i)
        buffer[i] = Endian(buffer[i]);
}

//-----------------------------------------------------------------------------
//  Endian() -- Change endian-ness of int buffer
//-----------------------------------------------------------------------------

void Endian(int * buffer, int size)
{
    for (int i = 0; i < size; ++i)
        buffer[i] = Endian(buffer[i]);
}

//-----------------------------------------------------------------------------
//  Endian() -- Change endian-ness of float buffer
//-----------------------------------------------------------------------------

void Endian(float * buffer, int size)
{
    for (int i = 0; i < size; ++i)
        buffer[i] = Endian(buffer[i]);
}

} // namespace Innovative



