// Event_Mb2.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Event_Mb.h>

namespace   Innovative
{

//==============================================================================
//  CLASS  Event
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class Event -- default ctor
//------------------------------------------------------------------------------

Event::Event(bool ManReset, bool InitState)
    : Manual(ManReset),
      Handle(Poco::Event::EVENT_AUTORESET ),
      mSignaled(InitState)
{
	if (InitState)
        Set();
}

//------------------------------------------------------------------------------
//  destructor for class Event
//------------------------------------------------------------------------------

Event::~Event()
{
}

//------------------------------------------------------------------------------
//  Event::Set() -- Set event to state given
//------------------------------------------------------------------------------

bool  Event::Set()
{
    MutexedLock Lock(mLock);

    mSignaled = true;

    try {
      Handle.set();
    } catch (Poco::SystemException &){
      mSignaled = false;
    }

	SignalOwners();

    return mSignaled;
}

//------------------------------------------------------------------------------
//  Event::Clear() -- Clear event
//------------------------------------------------------------------------------

bool  Event::Clear()
{
    MutexedLock Lock(mLock);

    mSignaled = false;

    bool ret = true;
    try {
      Handle.reset();
    } catch (Poco::SystemException &){
      ret = false;
    }

    return ret;
}

//------------------------------------------------------------------------------
//  Event::ShouldReassert() -- Return true if Mult-Object should retrigger
//------------------------------------------------------------------------------

//bool  Event::ShouldReassert()
//{
//    return Manual;
//}

//------------------------------------------------------------------------------
//  Event::WaitFor() -- Wait on event
//------------------------------------------------------------------------------

bool  Event::WaitFor(unsigned int mS)
{
    if (mS == SyncObject::Infinite)
        {
            try {
              Handle.wait();
              mSignaled =  true;
            } catch (Poco::SystemException &) {
              mSignaled =  false;
            }
        }
    else
        {
        // Wait specified interval for signal to fire
        mSignaled = Handle.tryWait(mS-1);
        }

    MutexedLock Lock(mLock);
    if (!Manual)
        {
        bool prevState = mSignaled;
        mSignaled = false; // Maker mSignalled match the current state when AutoReset
        return prevState;
        }
    //
    //  In manual reset case, we have to re-set the event ourselves here
    if (mSignaled)
        {
            try {
              Handle.set();
              mSignaled = true;
            } catch (Poco::SystemException &){
              mSignaled = false;
            }

        }
    return mSignaled;
}


//------------------------------------------------------------------------------
//  Event::IsSignalled() -- Test event non-destructively
//------------------------------------------------------------------------------

bool Event::IsSignalled()
{
    return mSignaled;
}

//------------------------------------------------------------------------------
//  Event::Get() -- Read State non-destructively
//------------------------------------------------------------------------------

bool Event::Get()
{
    WaitFor(1);  // no waiting
    {
    MutexedLock Lock(mLock);
    if (!Manual && mSignaled)
        {
        try {
          Handle.set();
        } catch (Poco::SystemException &){
          mSignaled = false;
        }

        }
    }
    return mSignaled;
}


} // namespace Innovative

