// Events_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Events_MbH
#define Events_MbH

#undef NULL
#define NULL 0

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#include <BaseEvents_Mb.h>
#include <Synchronizer_Mb.h>

namespace OpenWire
{

#ifdef __BORLANDC__
	#pragma option push
	#pragma warn -8027
#endif

//=============================================================================
//  CLASS EventHandler  --
//=============================================================================

template <class T_EVENT>
class EventHandler : public StdEventHandler<T_EVENT>
{
public:
	enum SynchronizationModeValues { syncNone, syncThunker, syncSynchronizer };

	EventHandler() :  mySyncMode( syncNone ), mIntSynchronizer( NULL ) { }
	EventHandler( const EventHandler &other ) :
		StdEventHandler<T_EVENT>( other ),
		mySyncMode( other.mySyncMode )
	{
		if( other.mIntSynchronizer )
			mIntSynchronizer = other.mIntSynchronizer->Clone();

		else
			mIntSynchronizer = NULL;
	}

	virtual ~EventHandler()
		{
		delete mIntSynchronizer;
		mIntSynchronizer = NULL;
		}

	void  Synchronize()
		{
		SynchronizeMode( syncSynchronizer );
		}
	void  Unsynchronize()
		{
		SynchronizeMode( syncNone );
		}
	void  Thunk()
		{
		SynchronizeMode( syncThunker );
		}

public:
	EventHandler &operator = ( const EventHandler & other )
		{
		mySyncMode = other.mySyncMode;
		if( other.mIntSynchronizer )
			mIntSynchronizer = other.mIntSynchronizer->Clone();
		else
			mIntSynchronizer = NULL;

		if( other.mOnDispatch )
            StdEventHandler<T_EVENT>::mOnDispatch = other.mOnDispatch->Clone( this );
		else
            StdEventHandler<T_EVENT>::mOnDispatch = NULL;

		return *this;
		}

protected:
	SynchronizationModeValues			mySyncMode;
	Innovative::BasicEventSynchronizer *mIntSynchronizer;

	void  SynchronizeMode( SynchronizationModeValues mode )
		{
		if( mySyncMode == mode )
			return;

		mySyncMode = mode;

		delete mIntSynchronizer;
		mIntSynchronizer = NULL;

		switch (mode)
			{
			case syncSynchronizer:
				mIntSynchronizer = new Innovative::EventSynchronizer;
				break;

			case syncThunker:
				mIntSynchronizer = new Innovative::EventThunker;
				break;

			default:
				break;
			}

		StdEventHandler<T_EVENT>::mSynchronizer = mIntSynchronizer;
		}

};

//=============================================================================
//  CLASS SynchronizedEventHandler  --
//=============================================================================

template <class T_EVENT>
class SynchronizedEventHandler :  public EventHandler<T_EVENT>
{
public:
    SynchronizedEventHandler()
        {
        EventHandler<T_EVENT>::Synchronize();  // start out using the synchronizer
        }

};

//=============================================================================
//  CLASS ThunkedEventHandler  --
//=============================================================================

template <class T_EVENT>
class ThunkedEventHandler :  public EventHandler<T_EVENT>
{
public:
    ThunkedEventHandler()
        {
        EventHandler<T_EVENT>::Thunk();  // start out using the thunker
        }

};

//=============================================================================
//  CLASS EventSyncPair  --
//=============================================================================
//
//class EventSyncPair
//{
//private:
//	BaseEventHandler * const 	EventHandler;
//	Event * const 				EventItem;
//
//public:
//	EventSyncPair( BaseEventHandler * event_handler, Event * event_item ) :
//		EventHandler( event_handler ),
//		EventItem( event_item )
//	{
//	}
//
//	~EventSyncPair()
//	{
//		delete EventItem;
//	}
//
//public:
//	void Execute()
//	{
//		EventHandler->DirectExecute( EventItem );
//	}
//
//};


#ifdef __CLR_VER
#pragma managed(pop)
#endif

#ifdef __BORLANDC__
    #pragma option pop
#endif



} // OpenWire

#endif
