// FileDataSet_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <IniFile_Mb.h>
#include <FileDataSet_Mb.h>
#include <FileSupport_Mb.h>
#include <cstring>

using namespace std;

namespace Innovative
{

static const int CacheSize = 0x10000;

//===========================================================================
// Class FileDataSet
//===========================================================================

//---------------------------------------------------------------------------
// FileDataSet::FileDataSet() -- Constructor
//---------------------------------------------------------------------------

FileDataSet::FileDataSet()
	: Cache(0), CacheOffset(0), CurrentOffset(0),
	  Dirty(false), FAutoCreate(false), FSize(0)
{
}

//---------------------------------------------------------------------------
// FileDataSet Destructor
//---------------------------------------------------------------------------

FileDataSet::~FileDataSet()
{
	ImageClose();
}

//---------------------------------------------------------------------------
// FileDataSet::ImageOpen() -- Open disk file
//---------------------------------------------------------------------------

bool FileDataSet::ImageOpen()
{
	FSize = 0;

    File.FileName(FFileName);

	// Open the file
    if (!FileExists(FFileName) && FAutoCreate)
        File.Create();
    else if (FReadOnly)
        File.Open();
    else
        File.Append();

	if (!File.Opened())
		return false;
        
	delete [] Cache;
	Cache = new char[CacheSize];

	FSize = File.Size();

	// Fill Cache
	File.Position(0);
	File.Read(Cache, CacheSize);

    Init();

    return true;
}

//---------------------------------------------------------------------------
// FileDataSet::ImageClose() -- Close disk file
//---------------------------------------------------------------------------

void FileDataSet::ImageClose()
{
	if (!File.Opened())
		return;

    // Flush, if necessary
    ImageFlush();

    // Close handle to device
	File.Close();
	FSize = 0;
    CurrentOffset = 0;
    CacheOffset = 0;
    Dirty = false;

    delete [] Cache;
    Cache = 0;
}

//---------------------------------------------------------------------------
// FileDataSet::ImageFetch() -- Read sample from disk file
//---------------------------------------------------------------------------

void FileDataSet::ImageFetch(Subsample & sample)
{
	if (!File.Opened())
        return;

    // Check cache first
	int Index = (int)(CurrentOffset-CacheOffset);
    if ( (Index < 0) || (Index >= CacheSize) )
        {
        // Flush dirty cache, if necessary
        ImageFlush();

        // Pre-read a block on cache miss
		File.Position(CurrentOffset);
		File.Read(Cache, CacheSize);
        CacheOffset = static_cast<int>(CurrentOffset);
        Index = 0;
        }

    CurrentOffset += FSubsampleSize;
	char * pSample = sample.asString();
	memcpy(pSample, &Cache[Index], FSubsampleSize);
	CorrectEndian(pSample);
	sample.Type(static_cast<enum Subsample::SampleType>(FType));
    Progress(CurrentOffset);
}

//---------------------------------------------------------------------------
// FileDataSet::ImageStore() -- Write sample to disk file
//---------------------------------------------------------------------------

void FileDataSet::ImageStore(Subsample & sample)
{
	if (!File.Opened() || FReadOnly)
        return;

    // Insure that sample is cached to allow read-modify-write...
    Subsample temp;
	ImageFetch(temp);
	CurrentOffset -= FSubsampleSize;

	// Write sample into the cache
	unsigned int Index = static_cast<int>(CurrentOffset-CacheOffset);
	char * pSample = sample.asString();
	CorrectEndian(pSample);
	memcpy(&Cache[Index], pSample, FSubsampleSize);
	CurrentOffset += FSubsampleSize;
	FSize = std::max(FSize, CurrentOffset);
	// Mark the cache, so that it'll be written
	Dirty = true;
    Progress(CurrentOffset);
}

//---------------------------------------------------------------------------
// FileDataSet::ImageFlush() -- Write dirty cache to disk
//---------------------------------------------------------------------------

void FileDataSet::ImageFlush()
{
	if (!Dirty || FReadOnly)
		return;

	File.Position(CacheOffset);
	File.Write(Cache, CacheSize);

    Dirty = false;
}

//---------------------------------------------------------------------------
// FileDataSet::ImageSkip() -- Skip forward n bytes into disk file
//---------------------------------------------------------------------------

void FileDataSet::ImageSkip(ii64 offset)
{
    CurrentOffset += offset;
    Progress(CurrentOffset);
}

//---------------------------------------------------------------------------
// FileDataSet::ImageSeek() -- Seek to byte offset in disk file
//---------------------------------------------------------------------------

void FileDataSet::ImageSeek(ii64 offset)
{
	CurrentOffset = offset;
    Progress(CurrentOffset);
}

//---------------------------------------------------------------------------
// FileDataSet::ImageSize() -- Return size of disk file
//---------------------------------------------------------------------------

ii64 FileDataSet::ImageSize() const
{
    return FSize;
}

//---------------------------------------------------------------------------
// FileDataSet::ImageSize() -- Set size of disk file
//---------------------------------------------------------------------------

void FileDataSet::ImageSize(ii64 size)
{
	File.Size(size);
}


} // namespace Innovative

