// FileSupport_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <FileSupport_Mb.h>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <iostream>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <string>
#include <algorithm>
#include <memory>
#include "Poco/Path.h"

#ifndef LINUX
    #include <io.h>
    #include <direct.h>
#else
    #include <unistd.h>
    #include <limits.h>
#ifndef VXWORKS
    #include <libgen.h>
#endif
#endif

using namespace std;

namespace Innovative
{

#ifndef LINUX
    const char slash = '\\';
    const char backslash = '/';
#else
    const char slash = '/';
    const char backslash = '\\';
#endif

//---------------------------------------------------------------------------
//  GetSlash() --
//---------------------------------------------------------------------------
char GetSlash()
{
    return slash;
}

//---------------------------------------------------------------------------
//  GetBackslash() --
//---------------------------------------------------------------------------

char GetBackslash()
{
    return backslash;
}

//---------------------------------------------------------------------------
//  FileExists() --
//---------------------------------------------------------------------------

bool FileExists(const char*  file_name)
{
#ifndef LINUX
	return (_access(file_name, 0) == 0) ? true : false;
#else
	return (access(file_name, F_OK) == 0) ? true : false;
#endif
}

//---------------------------------------------------------------------------
//  FileExists() --
//---------------------------------------------------------------------------

bool FileExists(const string & file_name)
{
    return FileExists(file_name.c_str());
}

//---------------------------------------------------------------------------
//  FileSize() -- Get the size of the data file in bytes
//---------------------------------------------------------------------------

ii64  FileSize(const char*  file_name)
{
    if (!FileExists(file_name))
        return 0;

    ifstream file;
    file.open(file_name, ios::in | ios::ate);
    if (!file.is_open())
        return 0;

    ii64 result = file.tellg();
    file.close();

    return result;
}

//---------------------------------------------------------------------------
//  FileSize() -- Get the size of the data file in bytes
//---------------------------------------------------------------------------

ii64  FileSize(const string & file_name)
{
	return FileSize(file_name.c_str());
}

//---------------------------------------------------------------------------
//  CurrentDirectory() -- Change current directory
//---------------------------------------------------------------------------

void CurrentDirectory(const string & path)
{
#ifdef _MSC_VER
    _chdir(path.c_str());
#else
    chdir(path.c_str());
#endif
}

//---------------------------------------------------------------------------
//  CurrentDirectory() --
//---------------------------------------------------------------------------

string CurrentDirectory()
{
   static const int MALIBU_MAXPATH = 4096;

   char buffer[MALIBU_MAXPATH];

#ifdef _MSC_VER
    _getcwd(&buffer[0], MALIBU_MAXPATH);
#else
    getcwd(&buffer[0], MALIBU_MAXPATH);
#endif

    return string(buffer);
}

//==============================================================================
//  CLASS IncrementalFileName -- construct file name that increments
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class IncrementalFileName
//------------------------------------------------------------------------------

IncrementalFileName::IncrementalFileName(const string & base,
                                         const string & ext)
	: CountNum(0)
    , FBase(base)
	, FExt(ext)
	, FTag("")
	, FDigits(3)
	, FWrap(false)
	, WrapLimit(1)
{
    CalculateWrapLimit();
}

//------------------------------------------------------------------------------
//  constructor for class IncrementalFileName
//------------------------------------------------------------------------------

IncrementalFileName::IncrementalFileName(const string & base,
                                         const string & ext, 
                                         const string & tag)
	: CountNum(0)
    , FBase(base)
	, FExt(ext)
	, FTag(tag)
	, FDigits(3)
	, FWrap(false)
	, WrapLimit(1)
{
    CalculateWrapLimit();
}

//------------------------------------------------------------------------------
//  constructor for class IncrementalFileName
//------------------------------------------------------------------------------

IncrementalFileName::IncrementalFileName(const string & base,
                                         const string & ext,
                                         const string & tag,
                                         unsigned int digits,
                                         bool wrap)
	: CountNum(0)
    , FBase(base)
	, FExt(ext)
	, FTag(tag)
	, FDigits(digits)
	, FWrap(wrap)
	, WrapLimit(1)
{
    CalculateWrapLimit();
}

//------------------------------------------------------------------------------
//  IncrementalFileName::GenerateFileName() - generate file name from parts
//------------------------------------------------------------------------------

string  IncrementalFileName::GenerateFileName()
{
    string  CountStr;
    stringstream  ss;
    //
    //  Set up zero fill N digits
    ss << setw(FDigits);
    ss << setfill('0');
    //
    //  Stream number in -> string out
    ss << CountNum;
    ss >> CountStr;

    string name =  FBase + FTag + CountStr + FExt;
    return name;
}

//------------------------------------------------------------------------------
//  IncrementalFileName::GenerateFileName() - generate file name from parts
//------------------------------------------------------------------------------

void  IncrementalFileName::Increment()
{
    CountNum++;
    if (FWrap && CountNum>=WrapLimit)
    	{
        CountNum = 0;
    	}
}

//------------------------------------------------------------------------------
//  IncrementalFileName::CalculateWrapLimit() - Calculate Wrap Limit
//------------------------------------------------------------------------------

void  IncrementalFileName::CalculateWrapLimit()
{
    WrapLimit = 1;
    for (unsigned int i = 0; i < FDigits; i++)
    	{
        WrapLimit *= 10;
    	}
}


//==============================================================================
//  CLASS PathSpec()  --  Decompose pathspec to constituent parts
//==============================================================================
//---------------------------------------------------------------------------
//  PathSpec Constructor
//---------------------------------------------------------------------------
struct PocoPathImpl
{
    PocoPathImpl(){}
    PocoPathImpl(const string & path) : _path(path) {}

    Poco::Path _path;
};

PathSpec::PathSpec(const string & path) : _pimpl(new PocoPathImpl(path))
{
    Full(path);
}

//---------------------------------------------------------------------------
//  PathSpec Constructor
//---------------------------------------------------------------------------

PathSpec::PathSpec() : _pimpl(new PocoPathImpl)
{
}

//---------------------------------------------------------------------------
//  PathSpec Destructor
//---------------------------------------------------------------------------

PathSpec::~PathSpec()
{
    if (_pimpl) delete _pimpl;
}

//---------------------------------------------------------------------------
//  PathSpec::Path() -- Retrieve path portion of pathspec
//---------------------------------------------------------------------------

string PathSpec::Path() const
{
    return _pimpl->_path.makeParent().toString();
}

//---------------------------------------------------------------------------
//  PathSpec::Name() -- Retrieve name portion of pathspec
//---------------------------------------------------------------------------

string PathSpec::Name() const
{
        return _pimpl->_path.getFileName();
}

//---------------------------------------------------------------------------
//  PathSpec::Full() -- Retrieve full pathspec
//---------------------------------------------------------------------------

string PathSpec::Full() const
{
        return _pimpl->_path.toString();
}

//---------------------------------------------------------------------------
//  PathSpec::Full() -- Set full pathspec
//---------------------------------------------------------------------------

PathSpec & PathSpec::Full(const string & value)
{
    _pimpl->_path.assign(value);
    return *this;
}

//---------------------------------------------------------------------------
//  PathSpec::File() -- Retrieve file portion of pathspec
//---------------------------------------------------------------------------

string PathSpec::File() const
{
    return _pimpl->_path.getFileName();
}

//---------------------------------------------------------------------------
//  PathSpec::File() -- Set file portion of pathspec
//---------------------------------------------------------------------------

PathSpec & PathSpec::File(const string & value)
{
    _pimpl->_path.setFileName(value);
    return *this;
}

//---------------------------------------------------------------------------
//  PathSpec::Drive() -- Retrieve drive portion of pathspec
//---------------------------------------------------------------------------

string PathSpec::Drive() const
{
    Poco::Path temp;
    temp.setNode(_pimpl->_path.getNode());
    temp.setDevice(_pimpl->_path.getDevice());
    return temp.makeDirectory().toString();
}

//---------------------------------------------------------------------------
//  PathSpec::Drive() -- Set drive portion of pathspec
//---------------------------------------------------------------------------

PathSpec & PathSpec::Drive(const string & value)
{
    _pimpl->_path.setDevice(value);
    return *this;
}

//---------------------------------------------------------------------------
//  PathSpec::BareExt() -- Retrieve extension portion of pathspec
//---------------------------------------------------------------------------

string PathSpec::BareExt() const
{
    return _pimpl->_path.getExtension();
}

//---------------------------------------------------------------------------
//  PathSpec::BareExt() -- Set extension portion of pathspec
//---------------------------------------------------------------------------

PathSpec & PathSpec::BareExt(const string & value)
{
    _pimpl->_path.setExtension(value);
    return *this;
}

//---------------------------------------------------------------------------
//  PathSpec::Ext() -- Retrieve extension portion of pathspec with dot
//---------------------------------------------------------------------------

std::string PathSpec::Ext() const
{
    std::string ext = _pimpl->_path.getExtension();
    ext.insert(0,".");

    return ext;
}

//---------------------------------------------------------------------------
//  PathSpec::Ext() -- Set extension portion of pathspec without dot
//---------------------------------------------------------------------------

PathSpec & PathSpec::Ext(const std::string & value)
{
    std::string str = value;

    if(str.substr(0,1) == ".")
        str.erase(0,1);

    _pimpl->_path.setExtension(str);
    return *this;
}

//---------------------------------------------------------------------------
//  PathSpec::Dir() -- Retrieve directory portion of pathspec
//---------------------------------------------------------------------------

string PathSpec::Dir() const
{
    Poco::Path dir;
    for (int i = 0; i < _pimpl->_path.depth(); i++)
        dir.pushDirectory(_pimpl->_path[i]);

    return dir.toString();
}

//---------------------------------------------------------------------------
//  PathSpec::Dir() -- Set directory portion of pathspec
//---------------------------------------------------------------------------

PathSpec & PathSpec::Dir(const string & value)
{
    Poco::Path new_path(value);

    new_path.setNode(_pimpl->_path.getNode());
    new_path.setDevice(_pimpl->_path.getDevice());
    new_path.setFileName(_pimpl->_path.getFileName());

    _pimpl->_path.assign(new_path);

    return *this;
}

//---------------------------------------------------------------------------
//  PathSpec::Parent() -- Retrieve parent directory spec
//---------------------------------------------------------------------------

string PathSpec::Parent() const
{
    return _pimpl->_path.makeParent().toString();
}

//---------------------------------------------------------------------------
//  PathSpec::IsDirectory() -- Set pathspec as dir
//---------------------------------------------------------------------------

void PathSpec::IsDirectory(bool state)
{
    string f = Full();

    if (f.empty()) return;

    if (state)
        {
        if (!IsDirectory())
            Full(f + slash);
        }
    else
        {
        if (IsDirectory())
            Full(f.erase(f.size()-1));
        }
   // _path.makeDirectory()
}

//---------------------------------------------------------------------------
//  PathSpec::IsDirectory() -- Check if pathspec is a directory
//---------------------------------------------------------------------------

bool PathSpec::IsDirectory() const
{
    return _pimpl->_path.isDirectory();
}





#ifdef _MSC_VER
#pragma warning (push)
#pragma warning (disable : 4996)
#endif

} // Innovative
//---------------------------------------------------------------------------
