// FileSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef FileSupport_MbH
#define FileSupport_MbH

#include "DataTypes_Mb.h"
#include <string>


namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//---------------------------------------------------------------------------
bool FileExists( const char *file_name );
bool FileExists(const std::string & file_name);

char GetSlash();
char GetBackslash();

ii64 FileSize( const char *file_name );
ii64 FileSize(const std::string & file_name);

void CurrentDirectory(const std::string & path);
std::string CurrentDirectory();
//---------------------------------------------------------------------------

//==============================================================================
//  CLASS IncrementalFileName -- construct file name that increments
//==============================================================================

class IncrementalFileName
{
public:
    IncrementalFileName(const std::string & base,
                        const std::string & ext);
                        
    IncrementalFileName(const std::string & base,
                        const std::string & ext,
                        const std::string & tag);

    IncrementalFileName(const std::string & base,
                        const std::string & ext,
                        const std::string & tag,
                        unsigned int digits,
                        bool wrap);

    std::string  Base()
            {  return FBase;  };
    void  Base(const std::string & base)
            {  FBase = base;  };
    std::string  Ext()
            {  return FExt;  };
    void  Ext(const std::string & ext)
            {  FExt = ext;  };
    std::string  Tag()
            {  return FTag;  };
    void  Tag(const std::string & tag)
            {  FTag = tag;  };
    int   Digits()
            {  return FDigits;  };
    void  Digits(int digits)
            {  FDigits = digits;  };
    bool  Wrap()
            {  return FWrap;  };
    void  Wrap(bool which)
            {  FWrap = which;  };

    std::string  FileName()
            {  return GenerateFileName();  };
    void         Increment();
    unsigned int Count() const
            {  return CountNum;  };
    void         Reset()
            {  CountNum = 0;  };

private:
    //
    //  Member Data
    unsigned int CountNum;
    std::string  FBase; 
    std::string  FExt;
    std::string  FTag;
    // ...Control Data
    unsigned int  FDigits;
    bool          FWrap;
    unsigned int  WrapLimit;

    std::string  GenerateFileName();
    void         CalculateWrapLimit();

};

//==============================================================================
//  CLASS PathSpec()  --  Decompose pathspec to constituent parts
//==============================================================================

class PathSpec
{
public:
    // Ctor
    PathSpec(const std::string & path);
    PathSpec();
    ~PathSpec();

    // Methods
    std::string     Path() const;
    std::string     Name() const;

    // Properties
    std::string     Full() const;
    PathSpec &      Full(const std::string & value);

    std::string     File() const;
    PathSpec &      File(const std::string & value);

    std::string     Drive() const;
    PathSpec &      Drive(const std::string & value);

    std::string     BareExt() const;
    PathSpec &      BareExt(const std::string & value);

    std::string     Ext() const;
    PathSpec &      Ext(const std::string & value);

    std::string     Dir() const;
    PathSpec &      Dir(const std::string & value);

    void            IsDirectory(bool state);
    bool            IsDirectory() const;

    std::string     Parent() const;

private:
    struct PocoPathImpl *_pimpl;

};

//===========================================================================
//  CLASS CacheDirectory  --  Cache current directory
//===========================================================================

class CacheDirectory
{
public:
    CacheDirectory()
        {  current = Innovative::CurrentDirectory();  }
    ~CacheDirectory()
        {  Innovative::CurrentDirectory(current);  }

private:
    std::string     current;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative

#endif
