//
// FloatingPtSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FloatingPtSupport_MbH
#define FloatingPtSupport_MbH

#include <cmath>

namespace Innovative
{
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//-----------------------------------------------------------------------------
//  template function IsAboutEqual<FT>  -- "about equal" check for float types
//-----------------------------------------------------------------------------

template <class FT>
bool  IsAboutEqual(FT a, FT b, FT epsilon=1e-5)
{
    FT delta = std::abs( a - b );
    if (delta < epsilon)
        return true;       // essentially equal
    else
        return false;
}

//-----------------------------------------------------------------------------
//  template function IntegerPart<FT>  -- drop fraction
//-----------------------------------------------------------------------------

template <class FT>
FT IntegerPart( FT x )
{
    if (x < 0.0)
        return std::ceil(x);
    else
        return std::floor(x);
}

//-----------------------------------------------------------------------------
//  template function FractionalPart<FT>  -- drop integer part
//-----------------------------------------------------------------------------

template <class FT>
FT FractionalPart( FT x )
{
    FT ip = IntegerPart( x );
    return x - ip;
}

//-----------------------------------------------------------------------------
//  template function NearestIntegerPart<FT>  -- closest round integer
//-----------------------------------------------------------------------------

template <class FT>
FT NearestIntegerPart( FT x )
{
    FT div = IntegerPart( x );
    FT fraction = x - div;
    div += static_cast<FT>((fraction > 0.5) ? 1.0 : 0.0 );
    return div;
}

//-----------------------------------------------------------------------------
//  template function NearestInteger<IT, FT>  -- closest  integer to float
//-----------------------------------------------------------------------------

template <class IT, class FT>
IT NearestInteger( FT x )
{
    FT IntPart = NearestIntegerPart(x);
    IT Out = static_cast<IT>(IntPart);
    return Out;
}

//-----------------------------------------------------------------------------
//  template function NearestInteger<FT>  -- closest 32 bit integer to float
//-----------------------------------------------------------------------------

template <class FT>
unsigned int NearestInteger( FT x )
{
    return NearestInteger<unsigned int>(x);
}

//-----------------------------------------------------------------------------
//  template function WithinRange<FT>  -- is value in range given?
//-----------------------------------------------------------------------------

template <class FT>
bool WithinRange(FT value, FT lower, FT upper)
{
	return (value >= lower) && (value <= upper);
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative
//---------------------------------------------------------------------------
#endif
