// FmcBoard_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FmcBoard_MbH
#define FmcBoard_MbH

#include <FmcSupport_Mb.h>
#include <VpxBoard_Mb.h>
#include <FmcSubDevice_Stream_Mb.h>
#include <FmcSubDevice_AnalogIn_Mb.h>
#include <FmcSubDevice_AnalogOut_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS FmcBoard  -- Fmc Daughter card Common baseboard class
//===========================================================================

class FmcBoard : public IFmcDaughterCard, public ISuppliesFmcRequirements
{
public:
    enum FmcSlot { fmcPrimary, fmcPrimaryK7=0, fmcRtm, fmcRtmK7 };

    friend class FmcDacDeviceMixin;

    //=======================================================================
    //  CLASS FmcBoard::MapRegisters  -- Board Wishbone Region Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Wishbone_0_Addr;
        unsigned int   Wishbone_1_Addr;
        unsigned int   Wishbone_2_Addr;
        unsigned int   Wishbone_3_Addr;
        unsigned int   Wishbone_4_Addr;
        unsigned int   Wishbone_5_Addr;
        unsigned int   Wishbone_6_Addr;
        unsigned int   Wishbone_Jesd0_Addr;
        unsigned int   Wishbone_Jesd1_Addr;
        unsigned int   Wishbone_Jesd2_Addr;
        unsigned int   Wishbone_Jesd3_Addr;
    };

    FmcBoard(FmcSlot assigned_slot);
    virtual ~FmcBoard()
        {}

    FmcStandardRomInfo &         RomInfo();
	IFmcInformation &			 FmcInfo();

    IVitaPacketizer &            VitaIn();
    IVitaTimestamp &             VitaIn_TS();
    IVitaDepacketizer &          VitaOut();

    IX6Aurora &     Aurora(unsigned int idx);

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    virtual void    CloseHardware();
    virtual void    RemoveCardPower();

    //  Customization calls
    virtual void    Pex7Cop_PrePowerCustomization();


    //  Internal Configuration Methods
    void  InitializeFmcRomInfo();

    //
    //  Subdevice Customization - implement board specific pieces
    virtual unsigned int GetVitaInStreamCount() = 0;
    virtual unsigned int GetVitaOutStreamCount() = 0;

    virtual IDacCalibrator * GetDacLatencyCalibrator() { return 0; }

    ii32   PeekLogic(ii32 offset);
    void   PokeLogic(ii32 offset, ii32 data);

    ii32   PeekJesdLogic(unsigned int which, ii32 offset);
    void   PokeJesdLogic(unsigned int which, ii32 offset, ii32 data);

    AdvI2CIdrom &   IdRom();   // reserved - use RomInfo instead

protected:
    //
    //  Data
    WishboneBusSpace  WB_FMC_0;
    WishboneBusSpace  WB_FMC_1;
    WishboneBusSpace  WB_FMC_2;
    WishboneBusSpace  WB_FMC_3;
    WishboneBusSpace  WB_FMC_4;
    WishboneBusSpace  WB_FMC_5;
    WishboneBusSpace  WB_FMC_6;
    WishboneBusSpace  WB_JESD_0;
    WishboneBusSpace  WB_JESD_1;
    WishboneBusSpace  WB_JESD_2;
    WishboneBusSpace  WB_JESD_3;
    //
    //  Subdevices
    FmcInformation         FmcInfo_SDev;
    FmcStd_I2C_Bus         FmcI2CBus_SDev;
    X6VitaPacketizerCfg    VitaIn_SDev;
    X6VitaDepacketizerCfg  VitaOut_SDev;
    X6Aurora               Aurora0_SDev;
    X6Aurora               Aurora1_SDev;
    X6Aurora               Aurora2_SDev;

    short           FmcVadj_MinSupportedV;
    short           FmcVadj_MaxSupportedV;

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI) = 0;

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode) = 0;

    void    InitCardPower_Original();
    void    InitCardPower_Pex7Cop();

public:
    Trace   Debug;
    //
    //  Friend Interface Functions
    friend IAddressingSpace & LogicMemorySpace( FmcBoard & board );
    friend IAddressingSpace & PortMemorySpace( FmcBoard & board );

};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------
//  Offset space to location of the base of the FMC card

inline IAddressingSpace & LogicMemorySpace( FmcBoard & board )
{
    return board.WB_FMC_0;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline IAddressingSpace & PortMemorySpace( FmcBoard & board )
{
    return board.PortMemory;
}

//===========================================================================
//  CLASS FmcAnalogBoard  -- Fmc Daughter card with common Analog Subdevices
//===========================================================================

class FmcAnalogBoard : public FmcBoard, public ISuppliesAddedFmcAnalogRequirements
{
public:

    FmcAnalogBoard(FmcSlot assigned_slot);
    virtual ~FmcAnalogBoard()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    //
    //  Subdevice Customization - implement board specific pieces
    virtual class IFrameCalcDevice * MakeFrameCalcDevice(FrameDeviceCode which) = 0;
    virtual void  GetAnalogDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev) = 0;

    FmcAFE_Common & Get_AFECommon_SDev() {  return AFECommon_SDev;  }
    virtual double  Get_DacClockRate() = 0;

protected:
    //
    //  Subdevices
    FmcAFE_Common                    AFECommon_SDev;

};

//===========================================================================
//  CLASS FmcAdcBoard  -- Fmc Daughter card with ADC Input Subdevices
//===========================================================================

class FmcAdcBoard : public FmcAnalogBoard, public ISuppliesAddedFmcAdcRequirements
{
public:
    friend class FmcInputDevice;
    friend class FmcStandardPriClockDevice;

    FmcAdcBoard(FmcSlot assigned_slot);
    virtual ~FmcAdcBoard()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //
    //  Subdevice Customization - implement board specific pieces
    virtual class IPriClockDevice *  MakePriClockDevice(PriDeviceCode which) = 0;

    virtual unsigned int GetCalibrationAdcCount() = 0;

    virtual ICalibrationValueOverride * GetAdcCalibrationOverride() = 0;
protected:
    //
    //  Subdevices
    FmcAFE_AdcCommon                 AFEAdcC_SDev;
    X6AFE_PulseRepetitionInterval    AFEAdcPri_SDev;
    Fmc_AdcCalCoefficientLoad        AFEAdcCal_SDev;

    virtual double  StandardClockFrequency() = 0;

};

//===========================================================================
//  CLASS FmcAdcBoard_T1  -- Fmc Daughter card with ADC Input Subdevices
//===========================================================================
//  Muxed Adc Spi

class FmcAdcBoard_T1 : public FmcAdcBoard, public ISuppliesAddedFmcAdc_T1Requirements
{
public:
    friend class FmcInputDevice_T1;
    //friend class FmcStandardPriClockDevice;

    FmcAdcBoard_T1(FmcSlot assigned_slot);
    virtual ~FmcAdcBoard_T1()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //
    //  Subdevice Customization - implement board specific pieces
    virtual unsigned int GetAdcSpiDeviceCount() = 0;

    virtual double  Get_DacClockRate()  { return 0.0; };

protected:
    //
    //  Subdevices
    FmcAFE_AdcSpi                    AFEAdcSpi_SDev;
    FmcAFE_AdcCalibrationControl     AFEAdcCalCtrl_SDev;
    FmcVga_I2C_Bus                   VgaI2CBus_SDev;

};

//===========================================================================
//  CLASS FmcDacDeviceMixin  -- DAC Mixin Devices for Output and Bidir cards
//===========================================================================

class FmcDacDeviceMixin : public ISuppliesAddedFmcDacDeviceMixinRequirements
{
public:
    friend class FmcOutputDevice;

    FmcDacDeviceMixin(FmcAnalogBoard * owner);
    virtual ~FmcDacDeviceMixin()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    //
    //  Subdevice Customization - implement board specific pieces
    virtual unsigned int GetDacSpiDeviceCount() = 0;

protected:
    //
    //  Subdevices
    FmcAnalogBoard *                 DDMxOwner;
    FmcAFE_DacCommon                 AFEDacC_SDev;
    X6AFE_PulseRepetitionInterval    AFEDacPri_SDev;
    FmcAFE_DacSpi                    AFEDacSpi_SDev;
    Fmc_DacCalCoefficientLoad        AFEDacCal_SDev;

    FmcAFE_Common & Common() {  return DDMxOwner->Get_AFECommon_SDev();  }
    double          DacClockFrequency() { return DDMxOwner->Get_DacClockRate(); }


};



//===========================================================================
//  CLASS FmcBidirBoard  -- Fmc Daughter card with ADC and DACs
//===========================================================================

class FmcBidirBoard : public FmcAdcBoard, public FmcDacDeviceMixin,
                      public ISuppliesAddedFmc_BidirRequirements
{
public:
    friend class FmcInputDevice_Bidir;

    FmcBidirBoard(FmcSlot assigned_slot, bool Fmc500Mode = false);
    virtual ~FmcBidirBoard()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //
    //  Subdevice Customization - implement board specific pieces
    virtual unsigned int GetAdcSpiDeviceCount() = 0;

protected:
    //
    //  Subdevices
    FmcAFE_AdcSpi                    AFEAdcSpi_SDev;

};

//===========================================================================
//  CLASS FmcInputDevice  -- Partially Implemented FmcBoard Input Device
//===========================================================================

class FmcInputDevice : public IFmcIoDevice
{
public:

    FmcInputDevice(FmcAdcBoard * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~FmcInputDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcInputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IFmcIoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(FmcInputDevice * in_dev)
            :  InD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        FmcInputDevice * InD;
    };
    
    friend class FmcInputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual IX6AnalogIoInfo &  Info() = 0;

    virtual unsigned int  Devices();
    virtual unsigned int  ChannelsPerDevice();
    virtual unsigned int  Channels();

    virtual void  DeviceDisableAll() {  ChannelDisableAll();  }
    virtual void  DeviceEnabled(int dev, bool state);
    virtual bool  DeviceEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual ITriggerCfg &                   Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return AdcPri();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double /*frequency_hz*/) {}
    virtual double TestFrequency() {  return 0.0;  }

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void CustomAdcExternalSyncSource(AfeExtSyncOptions /*src*/) {}

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value)
                        {  AdcCal().SetCalibrationOffset(ch, value);  }
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value)
                        {  AdcCal().SetCalibrationGain(ch, value);  }



protected:
    FmcAdcBoard *           Owner;
    TriggerCfgImpl          TriggerCfg;

    //  shortcuts to subdevices
    FmcAFE_AdcCommon &       AdcCommon() {  return Owner->AFEAdcC_SDev;  }
    const FmcAFE_AdcCommon & AdcCommon() const {  return Owner->AFEAdcC_SDev;  }
    FmcAFE_Common &          Common() {  return Owner->AFECommon_SDev;  }
    const FmcAFE_Common &    Common() const {  return Owner->AFECommon_SDev;  }
    X6AFE_PulseRepetitionInterval &        AdcPri() {  return Owner->AFEAdcPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  AdcPri() const {  return Owner->AFEAdcPri_SDev;  }
    Fmc_AdcCalCoefficientLoad &          AdcCal() {  return Owner->AFEAdcCal_SDev;  }
    const Fmc_AdcCalCoefficientLoad &    AdcCal() const {  return Owner->AFEAdcCal_SDev;  }


};

//===========================================================================
//  CLASS FmcInputDevice_T1  -- Partially Implemented FmcBoard Input Device
//===========================================================================

class FmcInputDevice_T1 : public FmcInputDevice
{
public:

    FmcInputDevice_T1(FmcAdcBoard_T1 * owner)
        :  FmcInputDevice(owner), Owner_T1(owner)
        {  }
    virtual ~FmcInputDevice_T1()  {  }

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

    virtual void  Vga(unsigned int ch, float gain_db);

protected:
    FmcAdcBoard_T1 *           Owner_T1;

    FmcAFE_AdcSpi &          AdcSpi() {  return Owner_T1->AFEAdcSpi_SDev;  }
    const FmcAFE_AdcSpi &    AdcSpi() const {  return Owner_T1->AFEAdcSpi_SDev;  }

    FmcVga_I2C_Bus &          VgaBus() {  return Owner_T1->VgaI2CBus_SDev;  }
    const FmcVga_I2C_Bus &    VgaBus() const {  return Owner_T1->VgaI2CBus_SDev;  }

    //  shortcuts to other things
    AdvI2C_16Idrom &     AdcCalIdrom()  {  return Owner_T1->VgaI2CBus_SDev.VgaIdromDevice();  }

};

//===========================================================================
//  CLASS FmcInputDevice_Bidir  -- Partially Implemented Bidir Input Device
//===========================================================================

class FmcInputDevice_Bidir : public FmcInputDevice
{
public:

    FmcInputDevice_Bidir(FmcBidirBoard * owner)
        :  FmcInputDevice(owner), Owner_Bidir(owner)
        {  }
    virtual ~FmcInputDevice_Bidir()  {  }

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

protected:
    FmcBidirBoard *           Owner_Bidir;

    FmcAFE_AdcSpi &          AdcSpi() {  return Owner_Bidir->AFEAdcSpi_SDev;  }
    const FmcAFE_AdcSpi &    AdcSpi() const {  return Owner_Bidir->AFEAdcSpi_SDev;  }

};

//===========================================================================
//  CLASS FmcOutputDevice  -- Base for FMC Carrier Output Devices
//===========================================================================

class FmcOutputDevice : public IFmcIoDevice
{
public:

    FmcOutputDevice( FmcDacDeviceMixin * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~FmcOutputDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxOutputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IFmcIoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(FmcOutputDevice * in_dev)
            :  OutD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        FmcOutputDevice * OutD;
    };
    
    friend class FmcOutputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual unsigned int  Devices();
    virtual unsigned int  ChannelsPerDevice();
    virtual unsigned int  Channels();

    virtual void  DeviceDisableAll() {  ChannelDisableAll();  }
    virtual void  DeviceEnabled(int dev, bool state);
    virtual bool  DeviceEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual ITriggerCfg &  Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return DacPri();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double /*frequency_hz*/);
    virtual double TestFrequency();

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

    virtual void CustomDacExternalSyncSource(AfeExtSyncOptions /*src*/) {}

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value)
                        {  DacCal().SetCalibrationOffset(ch, value);  }
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value)
                        {  DacCal().SetCalibrationGain(ch, value);  }

protected:
    FmcDacDeviceMixin *   Owner;
    TriggerCfgImpl  TriggerCfg;

    //  shortcuts to subdevices
    FmcAFE_DacCommon &        DacCommon() {  return Owner->AFEDacC_SDev;  }
    const FmcAFE_DacCommon &  DacCommon() const {  return Owner->AFEDacC_SDev;  }
    FmcAFE_Common &           Common() {  return Owner->Common();  }
    const FmcAFE_Common &     Common() const {  return Owner->Common();  }
    FmcAFE_DacSpi &          DacSpi() {  return Owner->AFEDacSpi_SDev;  }
    const FmcAFE_DacSpi &    DacSpi() const {  return Owner->AFEDacSpi_SDev;  }
    X6AFE_PulseRepetitionInterval &        DacPri() {  return Owner->AFEDacPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  DacPri() const {  return Owner->AFEDacPri_SDev;  }
    Fmc_DacCalCoefficientLoad &          DacCal() {  return Owner->AFEDacCal_SDev;  }
    const Fmc_DacCalCoefficientLoad &    DacCal() const {  return Owner->AFEDacCal_SDev;  }

};

//===========================================================================
//  CLASS FmcOutputDevice_Bidir  -- Base for FMC Carrier Output Devices
//===========================================================================

class FmcOutputDevice_Bidir : public FmcOutputDevice
{
public:
    FmcOutputDevice_Bidir( FmcDacDeviceMixin * owner)
        :  FmcOutputDevice(owner)
        {  }
    virtual ~FmcOutputDevice_Bidir()  {  }
};

//=============================================================================
//  CLASS FmcStandardPriClockDevice  --  Normal impl to access clock for PRI
//=============================================================================

class FmcStandardPriClockDevice : public IPriClockDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->StandardClockFrequency();  }
    virtual int   CalculatePriDivisor()
        {  return Divisor;  }

    FmcStandardPriClockDevice(FmcAdcBoard * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~FmcStandardPriClockDevice()
        {}

private:
    FmcAdcBoard * Owner;
    int Divisor;
};

//=============================================================================
//  CLASS FmcStandardFrameCalcDevice  --  Supply Divider on Frame ticks
//=============================================================================

class FmcStandardFrameCalcDevice : public IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor()
        {  return Divisor;  }

    FmcStandardFrameCalcDevice(FmcAnalogBoard * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~FmcStandardFrameCalcDevice()
        {}

private:
    FmcAnalogBoard * Owner;
    int Divisor;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif

