//---------------------------------------------------------------------------
// FmcCarrierBoard_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <FmcCarrierBoard_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS FmcDaughterCardSafeLink  -- Safe Linkage to a Fmc Daughter card
//===========================================================================
//---------------------------------------------------------------------------
//  FmcDaughterCardSafeLink::OnAfterLink() - handle post-link stuff
//---------------------------------------------------------------------------

void  FmcDaughterCardSafeLink::OnAfterLink()
{
    size_t link_idx = Owner->GetLinkIndex( Ptr() );
    Owner->FMCCB_DevMap.LinkFmcDaughterTo( Ptr(), link_idx );
    //Owner->FMCCB_DevMap.FmcDaughter.Link( Ptr() );
}

//---------------------------------------------------------------------------
//  FmcDaughterCardSafeLink::OnBeforeUnlink() - handle pre-unlink stuff
//---------------------------------------------------------------------------

void  FmcDaughterCardSafeLink::OnBeforeUnlink()
{
    size_t link_idx = Owner->GetLinkIndex( Ptr() );
    Owner->FMCCB_DevMap.UnlinkFmcDaughterFrom( link_idx );
    //Owner->FMCCB_DevMap.FmcDaughter.Unlink();
}

//===========================================================================
//  CLASS IIsAnFmcCarrierBoard  -- Common Fmc to Carrier link base
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IIsAnFmcCarrierBoard
//---------------------------------------------------------------------------

IIsAnFmcCarrierBoard::IIsAnFmcCarrierBoard(FmcCarrierDeviceMap & dev_map)
    :  FmcSlot_Prime(this), FmcSlot_RTM(this), FMCCB_DevMap(dev_map)
{
    FmcDaughters.push_back(&FmcSlot_Prime);
    FmcDaughters.push_back(&FmcSlot_RTM);
}

//---------------------------------------------------------------------------
//  IIsAnFmcCarrierBoard::ConnectTo() -- connect daughter to carrier
//---------------------------------------------------------------------------

bool  IIsAnFmcCarrierBoard::ConnectTo(IFmcDaughterCard * daughter, unsigned int slot)
{
    if (slot>=FmcDaughters.size())
        return false;

    if (daughter)
        {
        FmcDaughters[slot]->Link(daughter);
        return true;
        }
    else
        return false;
}

//---------------------------------------------------------------------------
//  IIsAnFmcCarrierBoard::DisconnectFrom() -- connect daughter to carrier
//---------------------------------------------------------------------------

void  IIsAnFmcCarrierBoard::DisconnectFrom(IFmcDaughterCard * daughter)
{
    size_t slot = GetLinkIndex(daughter);
    if (slot>=FmcDaughters.size())
        return;

    FmcDaughters[slot]->Unlink();
}

//---------------------------------------------------------------------------
//  IIsAnFmcCarrierBoard::Disconnect() -- disconnect daughters from carrier
//---------------------------------------------------------------------------

void IIsAnFmcCarrierBoard::Disconnect()
{
    for (size_t i=0; i<FmcDaughters.size(); i++)
        FmcDaughters[i]->Unlink();
}

//---------------------------------------------------------------------------
//  IIsAnFmcCarrierBoard::GetLinkIndex() -- Find idx for daughter
//---------------------------------------------------------------------------

size_t  IIsAnFmcCarrierBoard::GetLinkIndex(IFmcDaughterCard * ptr)
{
    for (size_t i=0; i<FmcDaughters.size(); i++)
        if (FmcDaughters[i]->Ptr() == ptr)
            return i;

    return 100;  // return out of range Link Idx
}

//===========================================================================
//  CLASS FmcCarrierBoard  -- Fmc Carrier Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FmcCarrierBoard
//---------------------------------------------------------------------------

FmcCarrierBoard::FmcCarrierBoard(FmcCarrierDeviceMap & deviceMap)
    :  IIsAVitaPacketStreamBoard(deviceMap),
       IIsAnFmcCarrierBoard(deviceMap),
       FmcCDeviceRef(deviceMap)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcCarrierBoard
//---------------------------------------------------------------------------

FmcCarrierBoard::~FmcCarrierBoard()
{
    // Don't call anything using VpxDeviceRef here - put in most derived class
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t FmcCarrierBoard::BoardCount()
{
	return FmcCDeviceRef.BoardCount();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> FmcCarrierBoard::BoardNames()
{
    return FmcCDeviceRef.Driver().ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string FmcCarrierBoard::PrintAllDevices()
{
    return FmcCDeviceRef.Driver().ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

bool  FmcCarrierBoard::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  FmcCarrierBoard::Target()
{
    return FmcCDeviceRef.Target();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Target( int value )
{
    FmcCDeviceRef.Target(value);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  FmcCarrierBoard::Led()
{
    return FmcCDeviceRef.Led();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Led( bool value )
{
    FmcCDeviceRef.Led(value);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  FmcCarrierBoard::GreenLed()
{
    return FmcCDeviceRef.Periph_SDev.GreenLed();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  FmcCarrierBoard::GreenLed( bool value )
{
    FmcCDeviceRef.Periph_SDev.GreenLed(value);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::FrontPanelLed() -- Read the FrontPanel LED state
//---------------------------------------------------------------------------

bool  FmcCarrierBoard::FrontPanelLed()
{
    return FmcCDeviceRef.Periph_SDev.FrontPanelLed();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::FrontPanelLed() -- Set the FrontPanel LED state of the board
//---------------------------------------------------------------------------

void  FmcCarrierBoard::FrontPanelLed( bool value )
{
    FmcCDeviceRef.Periph_SDev.FrontPanelLed(value);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Open()
{
    FmcCDeviceRef.Open();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Close()
{
    FmcCDeviceRef.Close();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Reset()
{
    FmcCDeviceRef.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Internals() -- Internal Packet Stream Interface
//---------------------------------------------------------------------------

IPacketStreamDeviceInternals *  FmcCarrierBoard::Internals()
{
    return &FmcCDeviceRef;
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Log( int type, int value )
{
    FmcCDeviceRef.Log(type, value);
}


//---------------------------------------------------------------------------
//  FmcCarrierBoard::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

IFpaLoader & FmcCarrierBoard::VsProm()
{
	return FmcCDeviceRef.VsProm();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Application() -- Return reference to Application object
//---------------------------------------------------------------------------

IVpxApplication & FmcCarrierBoard::Application()
{
	return FmcCDeviceRef.Application_SDev;
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::VeloIn() -- Return reference to Velocia Packetizer Control
//---------------------------------------------------------------------------

IVeloPacketizer & FmcCarrierBoard::VeloIn()
{
    return FmcCDeviceRef.VeloIn();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::VeloOut() -- Return reference to Velocia Depacketizer 
//---------------------------------------------------------------------------

IVeloDepacketizer & FmcCarrierBoard::VeloOut()
{
    return FmcCDeviceRef.VeloOut();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Pattern() -- Return reference to Pattern engine
//---------------------------------------------------------------------------

IPatternModeSystem & FmcCarrierBoard::Pattern()
{
    return FmcCDeviceRef.Pattern();
}

//------------------------------------------------------------------------------
//  FmcCarrierBoard::PciLoopbackMode() --
//------------------------------------------------------------------------------

void FmcCarrierBoard::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    FmcCDeviceRef.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  FmcCarrierBoard::PciLoopbackMode() --
//------------------------------------------------------------------------------

FmcCarrierBoard::LoopbackMode FmcCarrierBoard::PciLoopbackMode()
{
    FmcCarrierBoard::LoopbackMode result;
    switch (FmcCDeviceRef.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX6Alerts & FmcCarrierBoard::Alerts()
{
    return FmcCDeviceRef.Alerts();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Clock() --
//---------------------------------------------------------------------------
//
//IX6ClockIo & FmcCarrierBoard::Clock()
//{
//    return FmcCDeviceRef.Clock();
//}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Info() --
//---------------------------------------------------------------------------

const IX6LogicVersionInfo & FmcCarrierBoard::Info() const
{
    return FmcCDeviceRef.Info();
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::BusMasterSize() --
//---------------------------------------------------------------------------

void FmcCarrierBoard::BusMasterSize(int bytes)
{
    FmcCDeviceRef.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void FmcCarrierBoard::IncomingBusMasterSize(int bytes)
{
	FmcCDeviceRef.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void FmcCarrierBoard::OutgoingBusMasterSize(int bytes)
{
	FmcCDeviceRef.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  FmcCarrierBoard::IncomingBusMasterSize() const
{
	return FmcCDeviceRef.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  FmcCarrierBoard::OutgoingBusMasterSize() const
{
	return FmcCDeviceRef.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  FmcCarrierBoard::PeekLogic(int offset )
{
    return FmcCDeviceRef.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  FmcCarrierBoard::PokeLogic( int offset, ii32 value )
{
    FmcCDeviceRef.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  FmcCarrierBoard::PeekPort(int offset )
{
    return FmcCDeviceRef.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  FmcCarrierBoard::PokePort( int offset, ii32 value )
{
    FmcCDeviceRef.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::AfterFiclStart() --
//---------------------------------------------------------------------------

void  FmcCarrierBoard::AfterFiclStart(FiclSystem & engine)
{
    //  Replace Callbacks to call FICL too
    FmcCDeviceRef.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  FmcCarrierBoard::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  FmcCarrierBoard::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  FmcCarrierBoard::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  FmcCarrierBoard::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//===========================================================================
//  CLASS FmcCarrierAdvBoard  -- Fmc Carrier Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FmcCarrierAdvBoard
//---------------------------------------------------------------------------

FmcCarrierAdvBoard::FmcCarrierAdvBoard(FmcCarrierAdvDeviceMap & deviceMap)
    :  FmcCarrierBoard(deviceMap),
       FmcCAdvDeviceRef(deviceMap)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcCarrierAdvBoard
//---------------------------------------------------------------------------

FmcCarrierAdvBoard::~FmcCarrierAdvBoard()
{
    // Don't call anything using FmcCAdvDeviceRef here - put in most derived class
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvBoard::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

IX6ThermalDevice & FmcCarrierAdvBoard::Thermal()
{
    return FmcCAdvDeviceRef.Thermal();
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvBoard::Dio() -- DIO port access
//---------------------------------------------------------------------------

AdvancedDioPortIntf & FmcCarrierAdvBoard::Dio()
{
    return FmcCAdvDeviceRef.Dio();
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvBoard::Aurora() -- return ref to Aurora subdevs
//---------------------------------------------------------------------------

IX6Aurora &  FmcCarrierAdvBoard::Aurora()
{
    return FmcCAdvDeviceRef.Aurora();
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvBoard::IdRom() --
//---------------------------------------------------------------------------

IIdRom & FmcCarrierAdvBoard::IdRom()
{
    return FmcCAdvDeviceRef.IdRom();
}



#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
