// FmcCarrierBoard_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FmcCarrierBoard_MbH
#define FmcCarrierBoard_MbH

#include <DataTypes_Mb.h>
#include <VpxSupport_Mb.h>
#include <FmcCarrierDeviceMap_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IFmcCarrierDevice  -- Fmc Carrier card device interface to FMC Dtr
//===========================================================================

class IFmcCarrierDevice
{
public: // Pseudo-protected
    //virtual void  RegisterFmcCarrier( IIsAnFmcCarrierBoard * owner ) = 0;

    virtual ~IFmcCarrierDevice()
        {}

};

//===========================================================================
//  CLASS IFmcDaughterDevice  -- Fmc Daughter card device interface
//===========================================================================

class IFmcDaughterDevice
{
public: // Pseudo-protected
    virtual void  RegisterFmcCarrier( IFmcCarrierDevice * owner ) = 0;

    virtual ~IFmcDaughterDevice()
        {}

};

//===========================================================================
// CLASS FmcDaughterCardSafeLink  --  Safe Linkage to a Fmc Daughter card
//===========================================================================
class IIsAnFmcCarrierBoard;

class FmcDaughterCardSafeLink : public SafeLink<IFmcDaughterCard, SafeFmcDaughterCardImpl>
{
public:
    FmcDaughterCardSafeLink(IIsAnFmcCarrierBoard * owner)
        :  Owner(owner)
        {}

private:
    IIsAnFmcCarrierBoard *  Owner;

    virtual void  OnAfterLink();        // overrides
    virtual void  OnBeforeUnlink();
};

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif
//===========================================================================
//  CLASS IIsAnFmcCarrierBoard  --Fmc daughter card linkage interface class
//===========================================================================

class IIsAnFmcCarrierBoard : public IFmcCarrierDevice
{
    friend class FmcDaughterCardSafeLink;

public:
    IIsAnFmcCarrierBoard(FmcCarrierDeviceMap & dev_map);
    virtual ~IIsAnFmcCarrierBoard()
        {};

    virtual bool ConnectTo(IFmcDaughterCard * daughter, unsigned int slot=0 );
    virtual void DisconnectFrom(IFmcDaughterCard * daughter);
    virtual void Disconnect();

public:  // pseudo-protected

protected:
    FmcDaughterCardSafeLink          FmcSlot_Prime;
    FmcDaughterCardSafeLink          FmcSlot_RTM;
    FmcCarrierDeviceMap &            FMCCB_DevMap;

    FmcDaughterCardSafeLink &        FmcDaughter(size_t slot)
                                        {  return *(FmcDaughters[slot]);  }
    size_t                           FmcSlots()
                                        {  return FmcDaughters.size();  }
    size_t                           GetLinkIndex(IFmcDaughterCard * ptr);

    std::vector<FmcDaughterCardSafeLink *> FmcDaughters;

};
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//===========================================================================
//  CLASS FmcCarrierBoard  -- Vpx Common baseboard class
//===========================================================================

class FmcCarrierBoard : public IIsAVitaPacketStreamBoard,
                        public IStreamDeviceEvents,
                        public IFiclTarget,
                        public IIsAnFmcCarrierBoard
{

public:
    // Trigger Support
    // Enumerations
    enum LoopbackMode { lmNone, lmParallel, lmSerial };    

    //
    //  Member Functions
    FmcCarrierBoard(FmcCarrierDeviceMap & deviceMap);
    virtual ~FmcCarrierBoard();

    //  Informational Methods
	size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
    bool                        LicenseCheck(unsigned int & code);

    std::string                 Name() const
        {  return FmcCDeviceRef.Name();  }
    std::string                 Revision() const
        {  return FmcCDeviceRef.Revision();  }

    int                         Target();
    void                        Target(int value);

    bool                        Led();
    void                        Led(bool value);
    bool                        GreenLed();
    void                        GreenLed(bool value);
    bool                        FrontPanelLed();
    void                        FrontPanelLed(bool value);

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    //
    //  Properties
    void                        PciLoopbackMode(LoopbackMode mode);
    LoopbackMode                PciLoopbackMode();

    void                        BusMasterSize(int bytes);
    void                        IncomingBusMasterSize(int bytes);
    void                        OutgoingBusMasterSize(int bytes);
    int                         IncomingBusMasterSize() const;
    int                         OutgoingBusMasterSize() const;

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
	IFpaLoader &                VsProm();
	IVpxApplication &           Application();

    const IX6LogicVersionInfo &  Info() const;
    IVeloPacketizer &            VeloIn();
    IVeloDepacketizer &          VeloOut();

    IUsesX6Alerts &              Alerts();
    IPatternModeSystem &         Pattern();


public: //Pseudo protected
    //
    //  IPacketStreamDevice interface implementation
    virtual IPacketStreamDeviceInternals *  Internals();

    FmcCarrierDeviceMap * Debug() { return &FmcCDeviceRef; };
    virtual void          Log(int type, int data);

protected:
    //
    //  Member Data
    FmcCarrierDeviceMap &                  FmcCDeviceRef;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( FmcCarrierBoard & board );
    friend AddressingSpace & PortMemorySpace( FmcCarrierBoard & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( FmcCarrierBoard & board )
{
    return board.FmcCDeviceRef.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( FmcCarrierBoard & board )
{
    return board.FmcCDeviceRef.PortMemory;
}

//===========================================================================
//  CLASS FmcCarrierAdvBoard  -- Carrier base for PexCop, VpxCop boards
//===========================================================================

class FmcCarrierAdvBoard : public FmcCarrierBoard
{

public:
    //
    //  Member Functions
    FmcCarrierAdvBoard(FmcCarrierAdvDeviceMap & deviceMap);
    virtual ~FmcCarrierAdvBoard();

    //  SubDevices
    IX6ThermalDevice &           Thermal();
    AdvancedDioPortIntf &        Dio();
    IX6Aurora &                  Aurora();

    IIdRom &                     IdRom();

protected:
    //
    //  Member Data
    FmcCarrierAdvDeviceMap &     FmcCAdvDeviceRef;

};

//===========================================================================
//  CLASS FmcK7CarrierBoard  -- Carrier base for K7 boards
//===========================================================================

//class FmcCarrierK7Board : public FmcCarrierBoard
//{

//public:
//    //
//    //  Member Functions
//    FmcCarrierK7Board(K7DeviceMap & deviceMap);
//    virtual ~FmcCarrierK7Board();

//protected:
//    //
//    //  Member Data
//    K7DeviceMap &     K7DeviceRef;

//};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

