// FmcCarrierDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <FmcCarrierDeviceMap_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceDriver_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS FmcCHdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class FmcCHdwPluginImpl : public IPacketDevicePlugin
{
public:
    FmcCHdwPluginImpl( FmcCarrierDeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        {  PDM->CloseHardware();  }

    FmcCarrierDeviceMap * PDM;
};

//===========================================================================
//  CLASS IIsAnFmcCarrierDeviceMap  --Fmc daughter card linkage interface class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for IIsAnFmcCarrierDeviceMap
//---------------------------------------------------------------------------

IIsAnFmcCarrierDeviceMap::IIsAnFmcCarrierDeviceMap(size_t total_slots)
{
	if (total_slots--)
		FmcDaughters.push_back(&FmcSlot_Prime);
	if (total_slots--)
		FmcDaughters.push_back(&FmcSlot_RTM);
}

//---------------------------------------------------------------------------
//  IIsAnFmcCarrierDeviceMap::LinkFmcDaughterTo() -- connect daughter to map
//---------------------------------------------------------------------------

bool  IIsAnFmcCarrierDeviceMap::LinkFmcDaughterTo(IFmcDaughterCard * daughter, size_t slot)
{
    if (slot>=FmcDaughters.size())
        return false;

    if (daughter)
        {
        FmcDaughters[slot]->Link(daughter);
        return true;
        }
    else
        return false;
}

//---------------------------------------------------------------------------
//  IIsAnFmcCarrierDeviceMap::UnlinkFmcDaughterFrom() -- unlink daughter
//---------------------------------------------------------------------------

void IIsAnFmcCarrierDeviceMap::UnlinkFmcDaughterFrom(size_t slot)
{
    if (slot>=FmcDaughters.size())
        return;
    FmcDaughters[slot]->Unlink();
}

//===========================================================================
//  CLASS FmcCarrierDeviceMap  -- Registers for X6 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcCarrierDeviceMap
//---------------------------------------------------------------------------
//
//  Subdevice Register Structures

using namespace FmcCarrier_MemoryMap;

namespace
{

PacketDeviceRegisters X6DRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
    mmLinkControl,        // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters X6RegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

X6Info::MapRegisters X6InfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmBitstreamDate,      //  BitstreamDate_Addr
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

X6Peripherals::MapRegisters X6PeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

X6PacketizerCfg::MapRegisters X6PacketizerRegAddrs =
    {
    mmPktzer_Enable,          //  PacketizerEnable_Addr
    mmPktzer_AuxHdr,          //  AuxHeader_Addr
    mmPktzer_VeloInHdr,       //  VelociaInHeader_Addr
    mmPktzer_ForceSize,       //  PacketizerForceSize_Addr
    mmPktzer_IntervalPeriod,  //  PacketizerIntervalPeriod_Addr
    };

X6AlertSystem::MapRegisters X6AlertsRegAddrs =
    {
    mmAlert_Enable,       //  AlertEnable_Addr
    mmAlert_Control,      //  AlertControl_Addr
    mmAlert_Software,     //  SoftwareAlert_Addr;
    mmAlert_Clear,        //  ClearAlert_Addr;
    };

X6DepacketizerCfg::MapRegisters X6DepacketizerRegAddrs =
    {
    mmVeloOutTestLB,        //  TestLoopbackPid_Addr;
    mmVeloOutDataPid,       //  OutgoingDataPid_Addr
    };

VpxApplication::MapRegisters X6ApplicationRegAddrs =
    {
    mmApp_ControlStatus,        //  VpxAppControlStatus_Addr
    };

X6_DacPatternModeSystem::MapRegisters X6DacPatternModeRegAddrs =
    {
    mmLPDDR,                 //  PatternModeEnable_Addr;
    };


}  // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class FmcCarrierDeviceMap
//---------------------------------------------------------------------------
const bool  BitTS = true;    // supports bitstream timestamps.

FmcCarrierDeviceMap::FmcCarrierDeviceMap(IStreamDeviceEvents * owner, size_t fmc_slots,
                                         InnovativeKernel::DualRegionDeviceDriver * driver,
                                         MapWishbones & wb_map)
    :  PacketDeviceImpl         (driver, X6DRegAddrs),
       PacketDeviceMap          (this, X6RegAddrs, 0),
	   IIsAnFmcCarrierDeviceMap(fmc_slots),
       IUsesStreamDeviceEventCallbacks(owner),
       BDeviceDriver            (dynamic_cast<InnovativeKernel::DualRegionDeviceDriver*>(DeviceDriver)),
       WB_Periph                (LogicMemory, wbPeripheral),
       WB_Loader                (LogicMemory, wbLogicLoader),
       WB_Packetizer            (LogicMemory, wbPacketizer),
       WB_Alerts                (LogicMemory, wbAlerts),
       WB_FlashRom              (LogicMemory, wbROM),
       WB_Application           (LogicMemory, wb_map.WbApp_Offset),
       Info_SDev                (this, WB_Periph, PortMemory, X6InfoRegAddrs, BitTS),
	   Periph_SDev              (this, WB_Periph, X6PeripheralRegAddrs),
	   Velo_SDev                (this, WB_Packetizer, X6PacketizerRegAddrs),
	   Alerts_SDev              (this, WB_Alerts, X6AlertsRegAddrs),
	   DacPatternMode_SDev      (this, WB_Periph, X6DacPatternModeRegAddrs),
	   VeloOut_SDev             (this, WB_Periph, X6DepacketizerRegAddrs),
	   Application_SDev         (this, WB_Application, X6ApplicationRegAddrs)
{
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new FmcCHdwPluginImpl(this) );

    //  Enable use of dedicated register to signal Kernel Cache ready
    BDeviceDriver->EnableDedicatedKernelCacheRelease(PortMemory, FmcCarrier_MemoryMap::mmKernelCacheAvail);

}

//---------------------------------------------------------------------------
//  destructor for class FmcCarrierDeviceMap
//---------------------------------------------------------------------------

FmcCarrierDeviceMap::~FmcCarrierDeviceMap()
{
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    Velo_SDev.ConstructCustomParts();
    Alerts_SDev.ConstructCustomParts();
    DacPatternMode_SDev.ConstructCustomParts();
	VeloOut_SDev.ConstructCustomParts();
	Application_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::OpenHardware()
{
    MasterReset();

    Periph_SDev.AppReset();

    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    Velo_SDev.OpenHardware();
	Alerts_SDev.OpenHardware();
	DacPatternMode_SDev.OpenHardware();
	//FlashRom_SDev.OpenHardware();
	VeloOut_SDev.OpenHardware();
    Application_SDev.OpenHardware();

    //  Call daughter card
    //  ...make memory spaces on daughter valid
    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        

        FmcDaughter(i)->PortMemory.Link( PortMemory );
        FmcDaughter(i)->LogicMemory.Link( LogicMemory );
        
        PrePowerup_CustomizeDaughter(FmcDaughter(i));

        FmcDaughter(i)->InitCardPower();

        FmcDaughter(i)->OpenHardware();
        }


}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    Velo_SDev.PreconfigureHardware();
    Alerts_SDev.PreconfigureHardware();
    DacPatternMode_SDev.PreconfigureHardware();
    //FlashRom_SDev.PreconfigureHardware();
	VeloOut_SDev.PreconfigureHardware();
	Application_SDev.PreconfigureHardware();

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->PreconfigureHardware();
        }

}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    Velo_SDev.ConfigureHardware();
	Alerts_SDev.ConfigureHardware();
	DacPatternMode_SDev.ConfigureHardware();
	//FlashRom_SDev.ConfigureHardware();
	VeloOut_SDev.ConfigureHardware();
	Application_SDev.ConfigureHardware();

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->ConfigureHardware();
        }
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::StreamStopHardware()
{
    //AFECommon_SDev.AllOff();    // engine off, test off, trigger off

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->StreamStopHardware();
        }
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::CloseHardware()
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::CloseHardware()
{
    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->CloseHardware();
        FmcDaughter(i)->RemoveCardPower();
        }
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::AlertTest() -- Alert Test Processing Hook
//---------------------------------------------------------------------------

bool  FmcCarrierDeviceMap::AlertTest(unsigned int pid)
{
    //  return TRUE if PID is an alert PID.
    //  return FALSE if not
    return Alerts_SDev.AlertTest(pid);
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::AlertCheck() -- Alert Event Processing Hook
//---------------------------------------------------------------------------

bool FmcCarrierDeviceMap::AlertCheck(const Buffer & buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alerts_SDev.CheckAndProcess(buffer);
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::DumpIsrStatus(IntVec & vec)
{
    vec.clear();
    for (int i=0; i<X6DeviceDriver::LOG->Count; i++)
        vec.push_back(X6DeviceDriver::LOG->Status[i]);
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  FmcCarrierDeviceMap::Name() const
{
    static const char * ModuleName[] =
    { "Unk0", "Unk1", "K7Sbc", "Unk3", "Unk4", "Unk5", "Unk6", "Unk7", "Unk8", "Pex7-Cop"};

    unsigned int logicT = static_cast<unsigned int>(Info().PciLogicType());
    unsigned int ModNameElements = (sizeof(ModuleName)/sizeof(ModuleName[0]));
    return (logicT < ModNameElements) ? ModuleName[logicT] : "Bogus Type";
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  FmcCarrierDeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::FmcCarrierRun() -- return ref to Aurora subdevs
//---------------------------------------------------------------------------

void  FmcCarrierDeviceMap::FmcCarrierRun(bool state)
{
    Periph_SDev.FmcCRun(state);
}

//---------------------------------------------------------------------------
//  FmcCarrierDeviceMap::PrePowerup_CustomizeDaughter() --
//---------------------------------------------------------------------------
//  Carrier dependent pre-powerup customization

void  FmcCarrierDeviceMap::PrePowerup_CustomizeDaughter(DevMapSafeLink& /*daughter*/)
{
    // normally no code
}

//===========================================================================
//  CLASS FmcCarrierAdvDeviceMap  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcCarrierAdvDeviceMap
//---------------------------------------------------------------------------
namespace
{
    //
    //  IDROM Page Numbers
    const unsigned int System_IdromPage = 0;
    const unsigned int AdcCal_IdromPage = 1;
    const unsigned int DacCal_IdromPage = 2;
    const unsigned int User_IdromPage   = 3;


FmcCarrierDeviceMap::MapWishbones AdvWishboneDefs =
    {
    wbApplication,            // WbApp_Offset;
    };

X6Thermal::MapRegisters X6ThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

X6DigitalIo::MapRegisters X6DioRegAddrs =
    {
    mmDio_DataLo,          //  DioDataLo_Addr
    mmDio_DataHi,          //  DioDataHi_Addr
    mmDio_OutEnableLo,     //  DioOutputEnableLo_Addr
    mmDio_OutEnableHi,     //  DioOutputEnableHi_Addr
    };

X6Aurora::MapRegisters X6Aurora0RegAddrs =
    {
    mmAurora_TestControl,   //  TestControl_Addr
    mmAurora_ControlStatus, //  ControlStatus_Addr
    mmAurora_WritePort,     //  WritePort_Addr
    mmAurora_ReadPort,      //  ReadPort_Addr
    mmAurora_SfpPortControl, //  SfpPortControl_Addr
    mmAurora_SfpPortI2C,     //  SfpPortI2C_Addr
    };

VpxDdrMemory::MapRegisters X6DdrRegAddrs =
    {
    mmMem_Ddr3_ControlStatus,   //  VpxDdrMemControlStatus_Addr
    mmMem_Ddr3_WriteMask,       //  VpxDdrMemDataMask_Addr
    mmMem_Ddr3_Write0,          //  VpxDdrMemWriteData0_Addr
    mmMem_Ddr3_Write1,          //  VpxDdrMemWriteData1_Addr
    mmMem_Ddr3_Write2,          //  VpxDdrMemWriteData2_Addr
    mmMem_Ddr3_Write3,          //  VpxDdrMemWriteData3_Addr
    mmMem_Ddr3_Read0,           //  VpxDdrMemReadData0_Addr
    mmMem_Ddr3_Read1,           //  VpxDdrMemReadData1_Addr
    mmMem_Ddr3_Read2,           //  VpxDdrMemReadData2_Addr
    mmMem_Ddr3_Read3,           //  VpxDdrMemReadData3_Addr
    };

VpxQdrMemory::MapRegisters X6Qdr0RegAddrs =
    {
    mmMem_Qdr0_ControlStatus,   //  VpxQdrMemControlStatus_Addr
    mmMem_Qdr0_WriteMask,       //  VpxQdrMemDataMask_Addr
    mmMem_Qdr0_Write0,          //  VpxQdrMemWriteData0_Addr
    mmMem_Qdr0_Write1,          //  VpxQdrMemWriteData1_Addr
    mmMem_Qdr0_Write2,          //  VpxQdrMemWriteData2_Addr
    mmMem_Qdr0_ReadCSR,         //  VpxQdrMemReadCSR_Addr
    mmMem_Qdr0_Read0,           //  VpxQdrMemReadData0_Addr
    mmMem_Qdr0_Read1,           //  VpxQdrMemReadData1_Addr
    mmMem_Qdr0_Read2,           //  VpxQdrMemReadData2_Addr
    };

VpxQdrMemory::MapRegisters X6Qdr1RegAddrs =
    {
    mmMem_Qdr1_ControlStatus,   //  VpxQdrMemControlStatus_Addr
    mmMem_Qdr1_WriteMask,       //  VpxQdrMemDataMask_Addr
    mmMem_Qdr1_Write0,          //  VpxQdrMemWriteData0_Addr
    mmMem_Qdr1_Write1,          //  VpxQdrMemWriteData1_Addr
    mmMem_Qdr1_Write2,          //  VpxQdrMemWriteData2_Addr
    mmMem_Qdr1_ReadCSR,         //  VpxQdrMemReadCSR_Addr
    mmMem_Qdr1_Read0,           //  VpxQdrMemReadData0_Addr
    mmMem_Qdr1_Read1,           //  VpxQdrMemReadData1_Addr
    mmMem_Qdr1_Read2,           //  VpxQdrMemReadData2_Addr
    };

//VpxTrigger::MapRegisters X6TriggerRegAddrs =
//    {
//    mmTrigger_Control,          //  VpxTrigger_Control_Addr
//    mmTrigger_ItfOtf,           //  VpxTrigger_ItfOtf_Addr
//    mmTrigger_PlLt,             //  VpxTrigger_PlLt_Addr
//    mmTrigger_PlMd,             //  VpxTrigger_PlMd_Addr
//    mmTrigger_PpsGt,            //  VpxTrigger_PpsGt_Addr
//    };

X6FlashRom::MapRegisters X6FlashRomRegAddrs =
    {
    mmRom_AddrCtl,         //  AddressControl_Addr
    mmRom_Data,            //  Data_Addr
    mmRom_Opcode,          //  Opcode_Addr
    };


}  // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class FmcCarrierAdvDeviceMap
//---------------------------------------------------------------------------

FmcCarrierAdvDeviceMap::FmcCarrierAdvDeviceMap(IStreamDeviceEvents * owner, size_t fmc_slots)
    :  FmcCarrierDeviceMap      (owner, fmc_slots, new X6DeviceDriver, AdvWishboneDefs),
       DDeviceDriver            (dynamic_cast<X6DeviceDriver*>(DeviceDriver)),
       WB_Thermal               (LogicMemory, wbTempControl),
       WB_DigitalIo             (LogicMemory, wbDigitalIo),
       WB_Aurora0               (LogicMemory, wbAurora0),
       WB_Memory                (LogicMemory, wbMemory),
	   Thermal_SDev             (this, WB_Thermal, X6ThermalRegAddrs),
	   Dio_SDev                 (this, WB_DigitalIo, X6DioRegAddrs),
	   Aurora0_SDev             (this, WB_Aurora0, X6Aurora0RegAddrs),
	   Ddr3_SDev                (this, WB_Memory, X6DdrRegAddrs),
	   Qdr_0_SDev               (this, WB_Memory, X6Qdr0RegAddrs),
	   Qdr_1_SDev               (this, WB_Memory, X6Qdr1RegAddrs),
	   FlashRom_SDev            (this, WB_FlashRom, X6FlashRomRegAddrs),
	   System_PG                (FlashRom_SDev, System_IdromPage),
	   AdcCal_PG                (FlashRom_SDev, AdcCal_IdromPage),
	   DacCal_PG                (FlashRom_SDev, DacCal_IdromPage),
	   UserRom_PG               (FlashRom_SDev, User_IdromPage),
	   IdRom_Intf               (this)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcCarrierAdvDeviceMap
//---------------------------------------------------------------------------

FmcCarrierAdvDeviceMap::~FmcCarrierAdvDeviceMap()
{
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcCarrierAdvDeviceMap::ConstructCustomParts()
{
    FmcCarrierDeviceMap::ConstructCustomParts();

    // Subdevice Parts Construction
    Thermal_SDev.ConstructCustomParts();
    Dio_SDev.ConstructCustomParts();
	Aurora0_SDev.ConstructCustomParts();
	Ddr3_SDev.ConstructCustomParts();
	Qdr_0_SDev.ConstructCustomParts();
	Qdr_1_SDev.ConstructCustomParts();
	FlashRom_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  FmcCarrierAdvDeviceMap::OpenHardware()
{
    FmcCarrierDeviceMap::OpenHardware();

    Thermal_SDev.OpenHardware();
    Dio_SDev.OpenHardware();
	Aurora0_SDev.OpenHardware();
	Ddr3_SDev.OpenHardware();
	Qdr_0_SDev.OpenHardware();
	Qdr_1_SDev.OpenHardware();
	FlashRom_SDev.OpenHardware();
	//
	//  Read all ROM pages to Init Cache
	System_PG.LoadFromRom();
	AdcCal_PG.LoadFromRom();
	DacCal_PG.LoadFromRom();
	UserRom_PG.LoadFromRom();

}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcCarrierAdvDeviceMap::PreconfigureHardware()
{
    FmcCarrierDeviceMap::PreconfigureHardware();

    // Subdevice Configuration
    Thermal_SDev.PreconfigureHardware();
    Dio_SDev.PreconfigureHardware();
	Aurora0_SDev.PreconfigureHardware();
	Ddr3_SDev.PreconfigureHardware();
	Qdr_0_SDev.PreconfigureHardware();
	Qdr_1_SDev.PreconfigureHardware();
	FlashRom_SDev.PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcCarrierAdvDeviceMap::ConfigureHardware()
{
    FmcCarrierDeviceMap::ConfigureHardware();

    // Subdevice Configuration
    Thermal_SDev.ConfigureHardware();
    Dio_SDev.ConfigureHardware();
	Aurora0_SDev.ConfigureHardware();
	Ddr3_SDev.ConfigureHardware();
	Qdr_0_SDev.ConfigureHardware();
	Qdr_1_SDev.ConfigureHardware();
	FlashRom_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcCarrierAdvDeviceMap::StreamStopHardware()
{
    FmcCarrierDeviceMap::StreamStopHardware();
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::CloseHardware()
//---------------------------------------------------------------------------

void  FmcCarrierAdvDeviceMap::CloseHardware()
{
    FmcCarrierDeviceMap::CloseHardware();
}

//---------------------------------------------------------------------------
//  FmcCarrierAdvDeviceMap::Aurora() -- return ref to Aurora subdevs
//---------------------------------------------------------------------------

IX6Aurora &  FmcCarrierAdvDeviceMap::Aurora()
{
    return Aurora0_SDev;
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


} // namespace Innovative


