// FmcCarrierDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef FmcCarrierDeviceMap_MbH
#define FmcCarrierDeviceMap_MbH

#include <VpxSupport_Mb.h>
#include <FmcSupport_Mb.h>
#include <X6Support_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <X6SubDevice_AnalogOut_Mb.h>
#include <VpxSubDevice_Misc_Mb.h>

#include <PacketDeviceMap_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <StreamDeviceEvents_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class FmcCarrierDeviceMap;
class X6DeviceDriver;

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif
//===========================================================================
//  CLASS IIsAnFmcCarrierDeviceMap  --Fmc daughter card linkage interface class
//===========================================================================

class IIsAnFmcCarrierDeviceMap 
{

public:
	IIsAnFmcCarrierDeviceMap(size_t total_slots);
	virtual ~IIsAnFmcCarrierDeviceMap()
        {};

    virtual bool LinkFmcDaughterTo(IFmcDaughterCard * daughter, size_t slot);
    virtual void UnlinkFmcDaughterFrom(size_t slot);

protected:
    typedef SafeLink<IFmcDaughterCard, SafeFmcDaughterCardImpl>   DevMapSafeLink;

    DevMapSafeLink  FmcSlot_Prime;
    DevMapSafeLink  FmcSlot_RTM;

    DevMapSafeLink &        FmcDaughter(size_t slot)
                                {  return *(FmcDaughters[slot]);  }
    size_t                  FmcSlots()
                                {  return FmcDaughters.size();  }

    std::vector<DevMapSafeLink *> FmcDaughters;

};
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//===========================================================================
//  CLASS FmcCarrierDeviceMap  -- Register & low level support for VpxBoard
//===========================================================================

class FmcCarrierDeviceMap :  public PacketDeviceImpl,
                             public PacketDeviceMap,
                             public IIsAnFmcCarrierDeviceMap,
                             public IUsesStreamDeviceEventCallbacks,
                             public ISuppliesFmcCarrierRequirements
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcCarrierDeviceMap::MapWishbones  -- Wishbone Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapWishbones
    {
    public:
        unsigned int   WbApp_Offset;
    };


    friend class FmcCarrierBoard;
    friend class FmcCarrierInputDevice;
    friend class FmcCarrierOutputDevice;
    friend class FmcDaughterCardSafeLink;
    //
    //  Member Functions
    FmcCarrierDeviceMap(IStreamDeviceEvents * owner, size_t fmc_slots,
                        InnovativeKernel::DualRegionDeviceDriver * driver,
                        MapWishbones & wb_map );
    virtual ~FmcCarrierDeviceMap();
    //
    //  Make this property public
    using PacketDeviceImpl::PciLoopbackMode;

    // Methods
    // Version
    std::string         Name() const;
    std::string         Revision() const;

    //  Hardware Device Access
    IFpaLoader &              VsProm()  {  return DerivedVsProm();  }

    const IX6LogicVersionInfo &     Info() const   {  return Info_SDev;  }
    IUsesX6Alerts &           Alerts()  {  return Alerts_SDev;  }

    IVeloPacketizer &         VeloIn()    {  return Velo_SDev;  }

    IVeloDepacketizer &       VeloOut()    {  return VeloOut_SDev;  }
    IPatternModeSystem &      Pattern()    {  return DacPatternMode_SDev;  }

public: // pseudo-protected

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    virtual void    CloseHardware();
    //
    //  Subdevice Customization - implement board specific pieces
    //
    //  Busmaster Intf functions
    virtual bool    AlertTest(unsigned int pid);
    virtual bool    AlertCheck(const Buffer & buffer);

    typedef std::vector<int>  IntVec;
    void            DumpIsrStatus(IntVec & vec);

protected:
    InnovativeKernel::DualRegionDeviceDriver *  BDeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Loader;
    WishboneBusSpace       WB_Packetizer;
    WishboneBusSpace       WB_Alerts;
    WishboneBusSpace       WB_FlashRom;
    WishboneBusSpace       WB_Application;
    //
    //  Subdevices
    X6Info                       Info_SDev;
    X6Peripherals                Periph_SDev;
    //X6Thermal                    Thermal_SDev;
    X6PacketizerCfg              Velo_SDev;
    X6AlertSystem                Alerts_SDev;
    X6_DacPatternModeSystem      DacPatternMode_SDev;
    X6DepacketizerCfg            VeloOut_SDev;
    VpxApplication               Application_SDev;

    Trace                  Debug;

    // Data
    // ???  May not be implemented
    void  LocalLoopbackMode(bool state);

    void   FmcCarrierRun(bool state);


    virtual IFpaLoader &  DerivedVsProm() = 0;

    virtual void  PrePowerup_CustomizeDaughter(DevMapSafeLink & daughter);

};

//===========================================================================
//  CLASS FmcCarrierAdvDeviceMap  -- Dev Map Base for Pex, Vpx baseboards
//===========================================================================

class FmcCarrierAdvDeviceMap :  public FmcCarrierDeviceMap
//                             public ISuppliesFmcCarrierAdvRequirements
{
public:
    friend class FmcCarrierAdvBoard;
    //
    //  Member Functions
    FmcCarrierAdvDeviceMap(IStreamDeviceEvents * owner, size_t fmc_slots);
    virtual ~FmcCarrierAdvDeviceMap();

    IX6ThermalDevice &        Thermal() {  return Thermal_SDev;  }
    AdvancedDioPortIntf &     Dio()     {  return Dio_SDev;  }
    IX6Aurora &               Aurora();
    IIdRom &                  IdRom()  {  return IdRom_Intf;   }

    VpxDdrMemory &    Ddr3()  {  return Ddr3_SDev;  }
    VpxQdrMemory &    Qdr(int idx)
                        {
                        if (idx)
                            return Qdr_1_SDev;
                        else
                            return Qdr_0_SDev;
                        }

public: // pseudo-protected

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    virtual void    CloseHardware();

protected:
    X6DeviceDriver *  DDeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Thermal;
    WishboneBusSpace       WB_DigitalIo;
    WishboneBusSpace       WB_Aurora0;
    //WishboneBusSpace       WB_Aurora1;
    WishboneBusSpace       WB_Memory;
    //WishboneBusSpace       WB_Trigger;

    X6Thermal                    Thermal_SDev;
    X6DigitalIo                  Dio_SDev;
    X6Aurora                     Aurora0_SDev;
    VpxDdrMemory                 Ddr3_SDev;
    VpxQdrMemory                 Qdr_0_SDev;
    VpxQdrMemory                 Qdr_1_SDev;
    X6FlashRom                   FlashRom_SDev;
    //VpxTrigger                   Trigger_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    X6IdromPage                  AdcCal_PG;
    X6IdromPage                  DacCal_PG;
    X6IdromPage                  UserRom_PG;


    //=======================================================================
    //  CLASS VpxIdromIo  -- Access to IDROM Objects
    //=======================================================================
    class VpxIdromIo : public IIdRom
    {
    public:

    virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
    //  Page access
    virtual X6SystemIdromPage &  System() { return Owner->System_PG;  }
    virtual X6IdromPage &        AdcCal() { return Owner->AdcCal_PG;  }
    virtual X6IdromPage &        DacCal() { return Owner->DacCal_PG;  }
    virtual X6IdromPage &        User()   { return Owner->UserRom_PG;  }

    VpxIdromIo(FmcCarrierAdvDeviceMap * owner) :  Owner(owner)
        {}
    virtual ~VpxIdromIo()
        {}

    private:
        FmcCarrierAdvDeviceMap *  Owner;
    };

    VpxIdromIo                    IdRom_Intf;
    friend class VpxIdromIo;

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
