//---------------------------------------------------------------------------
// FmcSubDevice_AnalogIn_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <FmcSubDevice_AnalogIn_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS FmcAFE_AdcCommon -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_AdcCommon
//---------------------------------------------------------------------------

const unsigned int MaxFmcAdcChannels = 64;

//---------------------------------------------------------------------------
//  constructor for class FmcAFE_AdcCommon
//---------------------------------------------------------------------------

FmcAFE_AdcCommon::FmcAFE_AdcCommon(IRequires_FmcCommonParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       AdcEnable_Lo_Reg(space, regs.AfeAdcEnable_Lo_Addr),
       AdcEnable_Hi_Reg(space, regs.AfeAdcEnable_Hi_Addr),
       AdcPower_Lo_Reg(space, regs.AfeAdcPower_Lo_Addr),
       AdcPower_Hi_Reg(space, regs.AfeAdcPower_Hi_Addr),
       AdcTriggerCfgReg(space, regs.AfeAdcTriggerCfg_Addr),
       AdcDecimationReg(space, regs.AfeAdcDecimation_Addr),
       FDevices(0), FChannelsPerDevice(0), FPhysDevices(0), FChannelsPerPhysDevice(0)

{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_AdcCommon
//---------------------------------------------------------------------------

FmcAFE_AdcCommon::~FmcAFE_AdcCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_FmcCommonParts::devAdc);
    MDC->GetAnalogDeviceInfo(IRequires_FmcCommonParts::devAdc, FDevices, FChannelsPerDevice);
    MDC->GetAnalogPhysicalDeviceInfo(IRequires_FmcCommonParts::devAdc, FPhysDevices, FChannelsPerPhysDevice);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::ChannelDisableAll()
{
    AdcEnable_Lo_Reg.Value(0);
    AdcEnable_Hi_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  FmcAFE_AdcCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<MaxFmcAdcChannels; i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::ChannelEnabled(int ch, bool state)
{
    if (ch<32)
        AdcEnable_Lo_Reg.Bit(ch, state);
    else
        AdcEnable_Hi_Reg.Bit(ch-32, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::ChannelEnabled(int ch) const
{
    if (ch<32)
        return AdcEnable_Lo_Reg.Bit(ch);
    else
        return AdcEnable_Hi_Reg.Bit(ch-32);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::ChannelPowerDownAll()
{
    AdcPower_Lo_Reg.Value(0);
    AdcPower_Hi_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::ChannelPowered(int ch, bool state)
{
    if (ch<32)
        AdcPower_Lo_Reg.Bit(ch, state);
    else
        AdcPower_Hi_Reg.Bit(ch-32, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::ChannelPowered(int ch) const
{
    if (ch<32)
        return AdcPower_Lo_Reg.Bit(ch);
    else
        return AdcPower_Hi_Reg.Bit(ch-32);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::DeviceIsEnabled() -- Enable an I/O device
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::DeviceIsEnabled(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::DeviceIsEnabled() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::DeviceIsEnabled(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelEnabled(base);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::DevicePowered() -- Enable an I/O device
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::DevicePowered(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::DevicePowered() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::DevicePowered(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelPowered(base);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ActiveDevices() -- return count of active devices
//---------------------------------------------------------------------------

unsigned int  FmcAFE_AdcCommon::ActiveDevices()
{
    unsigned int active_devices(0);
    for (unsigned int i=0; i<Devices(); i++)
        if (DeviceIsEnabled(i)==true)
            active_devices++;

    return active_devices;
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    AdcTriggerCfgReg.AdcFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int FmcAFE_AdcCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = AdcTriggerCfgReg.AdcFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::EdgeTrigger(bool state)
{
    AdcTriggerCfgReg.AdcRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::EdgeTrigger() const
{
    return AdcTriggerCfgReg.AdcRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::ExternalTrigger(bool state)
{
    AdcTriggerCfgReg.AdcExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::ExternalTrigger() const
{
    return AdcTriggerCfgReg.AdcExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::FramedTriggerEnabled(bool state)
{
    AdcTriggerCfgReg.AdcFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool FmcAFE_AdcCommon::FramedTriggerEnabled() const
{
    return AdcTriggerCfgReg.AdcFramedMode.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::Decimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void FmcAFE_AdcCommon::Decimation(unsigned int samples)
{
    AdcDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::Decimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int FmcAFE_AdcCommon::Decimation() const
{
    return AdcDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::PhysicalDeviceEnabled() --
//---------------------------------------------------------------------------
//  Physical device should be enabled if any channel on it is enabled...

bool FmcAFE_AdcCommon::PhysicalDeviceEnabled(unsigned int phys_dev) const
{
    bool enabled(false);
    if (phys_dev>=FPhysDevices)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int end   = start + FChannelsPerPhysDevice;

    for (unsigned int ch=start;  ch<end; ch++)
        enabled |= ChannelEnabled(ch);
    return enabled;
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCommon::PhysicalChannelEnabled() --
//---------------------------------------------------------------------------
//  return enabled state for the Nth channel on device D...

bool FmcAFE_AdcCommon::PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const
{
    if (phys_dev>=FPhysDevices || device_ch>=FChannelsPerPhysDevice)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int ch  = start + device_ch;

    return ChannelEnabled(ch);
}

//===========================================================================
//  CLASS FmcAFE_AdcSpi -- Fmc AFE ADC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_AdcSpi
//---------------------------------------------------------------------------

namespace
{

FmcSpiDevice::RegisterFieldParams kFmc500_RFP(31, 15);
FmcSpiDevice::RegisterFieldParams kDefault_RFP(28, 12);
}
//---------------------------------------------------------------------------
//  constructor for class FmcAFE_AdcSpi
//---------------------------------------------------------------------------

FmcAFE_AdcSpi::FmcAFE_AdcSpi(IRequires_MuxedAdcSpi * mdc, IAddressingSpace & space,
                             FmcSpiDevice::MapRegisters & regs, bool Fmc500Mode)
    :  MDC(mdc),
       SpiDev(space, regs, "Fmc Adc", (Fmc500Mode)? kFmc500_RFP : kDefault_RFP),
       SpiCount(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_AdcSpi
//---------------------------------------------------------------------------

FmcAFE_AdcSpi::~FmcAFE_AdcSpi()
{
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_AdcSpi::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SpiCount = MDC->GetAdcSpiDeviceCount();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcSpi::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_AdcSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_AdcSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  FmcAFE_AdcSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiCount)
        SpiDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  FmcAFE_AdcSpi::Spi(unsigned int device, unsigned int address)
{
    if (device < SpiCount)
        return SpiDev.Spi(device, address);
    else
        return 0;
}


//===========================================================================
//  CLASS FmcAFE_AdcCalibrationControl -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_AdcCalibrationControl
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcAFE_AdcCalibrationControl
//---------------------------------------------------------------------------

FmcAFE_AdcCalibrationControl::FmcAFE_AdcCalibrationControl(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AdcCalibrationReg(space, regs.AfeAdcCal_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_AdcCalibrationControl
//---------------------------------------------------------------------------

FmcAFE_AdcCalibrationControl::~FmcAFE_AdcCalibrationControl()
{
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCalibrationControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCalibrationControl::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCalibrationControl::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCalibrationControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCalibrationControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCalibrationControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCalibrationControl::StartCalibration()
//---------------------------------------------------------------------------

void  FmcAFE_AdcCalibrationControl::StartCalibration()
{
    if (!DeviceEnabled())
        return;

    AdcCalibrationReg.CalStart.Value(false);
    AdcCalibrationReg.CalStart.Value(true);
    AdcCalibrationReg.CalStart.Value(false);

    Debug.Log("Adc Calibration Started");

}

//---------------------------------------------------------------------------
//  FmcAFE_AdcCalibrationControl::CalibrationResults()
//---------------------------------------------------------------------------

unsigned int  FmcAFE_AdcCalibrationControl::CalibrationResults()
{
    if (!DeviceEnabled())
        return 0;

    return AdcCalibrationReg.CalDone.Value();

}


//===========================================================================
//  CLASS Fmc110_AdcCalibrationControl -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc110_AdcCalibrationControl
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class Fmc110_AdcCalibrationControl
//---------------------------------------------------------------------------

Fmc110_AdcCalibrationControl::Fmc110_AdcCalibrationControl(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AdcCalibrationReg(space, regs.AfeAdcCal_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc110_AdcCalibrationControl
//---------------------------------------------------------------------------

Fmc110_AdcCalibrationControl::~Fmc110_AdcCalibrationControl()
{
}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalibrationControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc110_AdcCalibrationControl::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalibrationControl::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc110_AdcCalibrationControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalibrationControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc110_AdcCalibrationControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalibrationControl::StartCalibration()
//---------------------------------------------------------------------------

void  Fmc110_AdcCalibrationControl::StartCalibration(unsigned int ch)
{
    if (!DeviceEnabled())
        return;

	AdcCalibrationReg.CalStatus.Value((ch) ? true : false);

    AdcCalibrationReg.CalStart.Value(0);
    AdcCalibrationReg.CalStart.Value(1);
    AdcCalibrationReg.CalStart.Value(0);

    Debug.Log("Adc Calibration Started, Ch", ch);

}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalibrationControl::WaitForCal()
//---------------------------------------------------------------------------

void  Fmc110_AdcCalibrationControl::WaitForCal()
{
    if (!DeviceEnabled())
        return;

    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        bool done = AdcCalibrationReg.CalDone.Value();
        if (done)
            return;
        else
            Innovative::Sleep(2);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " Adc Cal Done Failure (>10ms)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalibrationControl::CalibrationResults()
//---------------------------------------------------------------------------

unsigned int  Fmc110_AdcCalibrationControl::CalibrationResults(unsigned int /*ch*/)
{
    if (!DeviceEnabled())
        return 0;

    return AdcCalibrationReg.Value();

}

//===========================================================================
//  CLASS FmcVga_I2C_Bus -- Standard Vga I2C Bus "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcVga_I2C_Bus
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcVga_I2C_Bus
//---------------------------------------------------------------------------

FmcVga_I2C_Bus::FmcVga_I2C_Bus(IRequires_Nothing * mdc, IAddressingSpace & space,
                                   MapRegisters & regs, I2C_SubAddr & subaddrs )
    :  MDC(mdc),
       VgaBusI2CReg(space, regs.I2CControl_Addr),
       VgaBusIntf(VgaBusI2CReg),
       Vga0_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga0_SubAddr),
       Vga1_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga1_SubAddr),
       Vga2_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga2_SubAddr),
       Vga3_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga3_SubAddr),
       Vga4_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga4_SubAddr),
       VgaIdrom_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.VgaIdrom_SubAddr),
       VgaIdromDev(VgaIdrom_I2C),
       Vga(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcVga_I2C_Bus
//---------------------------------------------------------------------------

FmcVga_I2C_Bus::~FmcVga_I2C_Bus()
{
}

//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcVga_I2C_Bus::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::OpenHardware()
//---------------------------------------------------------------------------

void  FmcVga_I2C_Bus::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcVga_I2C_Bus::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::Vga()  --  Access Vga I2C
//---------------------------------------------------------------------------

I2CDeviceN &  FmcVga_I2C_Bus::VgaI2C(unsigned int idx)
{
    switch (idx)
        {
        default:
        case 0:     return Vga0_I2C;
        case 1:     return Vga1_I2C;
        case 2:     return Vga2_I2C;
        case 3:     return Vga3_I2C;
        case 4:     return Vga4_I2C;
        }
}


//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::VgaDacLatch()  --  Latch VGA update
//---------------------------------------------------------------------------

void  FmcVga_I2C_Bus::VgaDacLatch()
{
    VgaBusI2CReg.VgaDacLatch.Set();
    VgaBusI2CReg.VgaDacLatch.Clear();
    VgaBusI2CReg.VgaDacLatch.Set();
}

//===========================================================================
//  CLASS FmcVga_I2C_Bus::LTC2635_8  -- DAC controller wrapper
//===========================================================================
//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::LTC2635_8::SetAdcGain()  -- set gain for an Adc Channel
//---------------------------------------------------------------------------

void  FmcVga_I2C_Bus::LTC2635_8::SetAdcGain(unsigned int dev, unsigned int adc, float db_gain)
{
    //  Calculate DAC level for a Gain in DB
    //  Gain_db = 20 * V_gain + 18
    //  .....V_gain is in 256 level steps from -.625 to +.625
    //  V_gain = 2*.625*step/256 - .625
    //   db range is from 6 db to 30 db
    float step = static_cast<float>(10.24*db_gain - 56.32);
    if (step<0.0)
        step = 0;
    if (step>255)
        step = 255;

    // use command code 0 to set DAC for a channel
    SendCommand(dev, 0, adc, (unsigned int)step);
}

//---------------------------------------------------------------------------
//  FmcVga_I2C_Bus::LTC2635_8::SendCommand()  --
//---------------------------------------------------------------------------

void  FmcVga_I2C_Bus::LTC2635_8::SendCommand(unsigned int dev, unsigned int cmd_code,
                                            unsigned int addr_code, unsigned int data)
{
    std::vector<int> Data(2);
    unsigned int addr = (cmd_code<<4)|(addr_code&0xF);
    Data[0] = data;    // 8 bit data is left justified in 2 word cmd...
    Data[1] = 0;
    Owner->VgaI2C(dev).MultiRegW(addr, &Data[0], static_cast<int>(Data.size()) );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//===========================================================================
//  CLASS Fmc_AdcCalCoefficientLoad -- Fmc ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class Fmc_AdcCalCoefficientLoad
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class Fmc_AdcCalCoefficientLoad
//---------------------------------------------------------------------------

Fmc_AdcCalCoefficientLoad::Fmc_AdcCalCoefficientLoad(IRequires_AdcCalParts * mdc,
                IAddressingSpace & space, MapRegisters & regs,
                unsigned int MaxAdcs )
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, MaxAdcs),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, MaxAdcs),
       FIncomingAdcs(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc_AdcCalCoefficientLoad
//---------------------------------------------------------------------------

Fmc_AdcCalCoefficientLoad::~Fmc_AdcCalCoefficientLoad()
{
    if (CalOverride)
        delete CalOverride;
}

//---------------------------------------------------------------------------
//  Fmc_AdcCalCoefficientLoad::ConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc_AdcCalCoefficientLoad::ConstructCustomParts()
{
    FIncomingAdcs = MDC->GetCalibrationAdcCount();
    CalOverride = MDC->GetAdcCalibrationOverride();
}

//---------------------------------------------------------------------------
//  Fmc_AdcCalCoefficientLoad::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc_AdcCalCoefficientLoad::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    for (unsigned int i=0; i<FIncomingAdcs; i++)
        {
        SetCalibrationGain(i, 0x10000);
        SetCalibrationOffset(i, 0x000);
        }
}

//---------------------------------------------------------------------------
//  Fmc_AdcCalCoefficientLoad::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc_AdcCalCoefficientLoad::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  Fmc_AdcCalCoefficientLoad::SetCalibrationOffset()
//---------------------------------------------------------------------------

void  Fmc_AdcCalCoefficientLoad::SetCalibrationOffset(unsigned int ch, unsigned int value)
{
    Debug.Log("Adc Cal Offset=0x", value, 16);
    if (ch<FIncomingAdcs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideOffset(value);
            if (CalOverride->OverrideOffsetUpdate())
                {
                CalOverride->WriteOffset(ch, value);
                return;
                }
            }
        OffsetCoeffArray[ch].Value(value);
        }
}

//---------------------------------------------------------------------------
//  Fmc_AdcCalCoefficientLoad::SetCalibrationGain()
//---------------------------------------------------------------------------

void  Fmc_AdcCalCoefficientLoad::SetCalibrationGain(unsigned int ch, unsigned int value)
{
    Debug.Log("Adc Cal Gain=0x", value, 16);
    if (ch<FIncomingAdcs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideGain(value);
            if (CalOverride->OverrideGainUpdate())
                {
                CalOverride->WriteGain(ch, value);
                return;
                }
            }
        GainCoeffArray[ch].Value(value);
        }
}

//===========================================================================
//  CLASS AD7763_SpiDevice -- Implementation of 'standard' X6 SPI interface
//===========================================================================
//---------------------------------------------------------------------------
//  AD7763_SpiDevice::SetSpiClockPeriod() --  Setup Clock Period
//---------------------------------------------------------------------------
//  Do on frequency changes

void AD7763_SpiDevice::SetSpiClockPeriod(double adc_clock_frequency)
{
    ClockPeriod = static_cast<unsigned int>(1.0e6/adc_clock_frequency);
    if (ClockPeriod==0)
        ClockPeriod=1;

    std::stringstream msg;
    msg << std::hex << Tag << " AD7763_SpiDevice: Period=" << std::dec << ClockPeriod << " uS" ;
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::Spi() --  SPI Read
//---------------------------------------------------------------------------

unsigned short AD7763_SpiDevice::Spi(int device, int addr)
{
    //  convert address to pseudo_address
    int p_address;
    switch (addr)
        {
        default:
            p_address = 0x1;  // status register
            break;
        case 3:
            p_address = 0x2;  // offset register
            break;
        case 4:
            p_address = 0x4;  // gain register
            break;
        case 5:
            p_address = 0x8;  // overrange register
            break;
        }

    //
    // In order to read, we must first write to register 1 what to read
    bool NoDump(true);  // don't print out the "pre-write"
    MemoryBitManipulator CR1(RegisterCache_0x1);   // use cached data for starting point
    CR1.Field(11, 4, p_address);
    Spi(device, 0x1, CR1.Value(), NoDump);

    //SpiReady();   // wait for first command to complete

    // No need to set address

    SpiNotReady();
    SpiReady();
    SpiDataValid();

    unsigned short result = ReadSpiData();

    return result;
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::Spi()  --  SPI Write
//---------------------------------------------------------------------------

void AD7763_SpiDevice::Spi(int device, int addr, unsigned short value, bool no_dump)
{
    // if addr==1, save value into cache
    if (addr==0x1)
        RegisterCache_0x1 = value;

    // Set up a SPI device
    SetupSpiDevice(device);

    SpiReady();

    // Write to register via SPI
    SetupSpiWrite(addr, value);

    SpiNotReady();
    SpiReady();

    if (no_dump)
        return;

    std::stringstream msg;
    msg << std::hex << Tag << " Spi[" << device << "] (0x" << addr <<
             ", 0x" << (short)value << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::SpiReady()
//---------------------------------------------------------------------------

void AD7763_SpiDevice::SpiReady()
{
    // Wait up to 15 clocks for pending SPI to complete
    const unsigned int max_wait = 15;
    for (unsigned int i = 0; i < max_wait*ClockPeriod; ++i)
        {
        if (ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1*ClockPeriod);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << Tag << " Spi Ready Failure (>" << max_wait << " clocks)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::SpiNotReady()
//---------------------------------------------------------------------------

void AD7763_SpiDevice::SpiNotReady()
{
    // Wait up to 15 clocks for pending SPI to complete
    const unsigned int max_wait = 15;
    for (unsigned int i = 0; i < max_wait*ClockPeriod; ++i)
        {
        if (!ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1*ClockPeriod);
        }
    // Gave up
    //std::stringstream msg;
    //msg << std::hex << Tag << " Spi NOT Ready Failure (>" << max_wait << " clocks)";
    //Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::SpiDataValid()
//---------------------------------------------------------------------------

void AD7763_SpiDevice::SpiDataValid()
{
    // Wait up to max_wait clocks for pending SPI to complete
    const unsigned int max_wait = 200;
    const unsigned int step_distance = 10;  // clocks

    for (unsigned int i = 0; i < max_wait*ClockPeriod; i+=step_distance)
        {
        if (SpiReadReg.DataValid.Value())
            return;
        else
            Innovative::uSleep(step_distance*ClockPeriod);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << Tag << " Spi DataValid Failure (>" << max_wait << " clocks)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::SetupSpiDevice() --  SPI Device Enable Setup
//---------------------------------------------------------------------------

void  AD7763_SpiDevice::SetupSpiDevice(int device)
{
    // enable SPI devices, one at a time
    unsigned int data = 1<<device;
    SpiDeviceReg.Value(data);
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::SetupSpiWrite() --  SPI Write Setup
//---------------------------------------------------------------------------

void  AD7763_SpiDevice::SetupSpiWrite(int addr, unsigned short value)
{
    // Set up a SPI write
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.WriteData.Value(value);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::ReadSpiData() --  Read SPI data
//---------------------------------------------------------------------------

unsigned short  AD7763_SpiDevice::ReadSpiData()
{
    return static_cast<unsigned short>(SpiReadReg.ReadData.Value());
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  AD7763_SpiDevice::ReadSpiReadyBit()
{
    return SpiReadReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  AD7763_SpiDevice::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  AD7763_SpiDevice::DumpSpiRegister()
{
    return SpiWriteReg.Value();
}

}  // namespace Innovative

