// FmcSubDevice_AnalogIn_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FmcSubDevice_AnalogIn_MbH
#define FmcSubDevice_AnalogIn_MbH

#include <X6Support_Mb.h>
#include <VpxSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SpiDevice_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS FmcAFE_AdcCommon -- Fmc AFE ADC Common Registers "SubDevice"
//===========================================================================

class FmcAFE_AdcCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_AdcCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcEnable_Lo_Addr;
        unsigned int   AfeAdcEnable_Hi_Addr;
        unsigned int   AfeAdcPower_Lo_Addr;
        unsigned int   AfeAdcPower_Hi_Addr;
        unsigned int   AfeAdcTriggerCfg_Addr;
        unsigned int   AfeAdcDecimation_Addr;
    };

    FmcAFE_AdcCommon(IRequires_FmcCommonParts * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcAFE_AdcCommon();

    //
    //  Methods
    unsigned int  Devices() const  {  return FDevices;  }
    unsigned int  ChannelsPerDevice() const {  return FChannelsPerDevice;  }
    unsigned int  Channels() const {  return FDevices * FChannelsPerDevice;  }

    virtual void  DeviceIsEnabled(int dev, bool state);
    virtual bool  DeviceIsEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    //  Physical Device Enable Mapping
    bool  PhysicalDeviceEnabled(unsigned int phys_dev) const;
    bool  PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_AdcCommon::AdcTriggerCfgRegister -- Configure ADC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  AdcFrameSize;
        RegisterBit       AdcRisingEdge;
        RegisterBit       AdcFramedMode;
        RegisterBit       AdcExternalTrigger;

    public:
        AdcTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcFrameSize(*this, 0, 24),   AdcRisingEdge(*this, 29),
               AdcFramedMode(*this, 30),     AdcExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_AdcCommon::AdcDecimationRegister  -- Adc Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        AdcDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_FmcCommonParts *    MDC;
    IFrameCalcDevice *            Dev;
    Register                      AdcEnable_Lo_Reg;
    Register                      AdcEnable_Hi_Reg;
    Register                      AdcPower_Lo_Reg;
    Register                      AdcPower_Hi_Reg;
    AdcTriggerCfgRegister         AdcTriggerCfgReg;
    AdcDecimationRegister         AdcDecimationReg;

    unsigned int                  FDevices;
    unsigned int                  FChannelsPerDevice;
    unsigned int                  FPhysDevices;
    unsigned int                  FChannelsPerPhysDevice;

};

//===========================================================================
//  CLASS FmcAFE_AdcSpi -- Fmc Muxed ADC Spi Device Manager "SubDevice"
//===========================================================================

class FmcAFE_AdcSpi : public IX6SubDevice
{
public:

    FmcAFE_AdcSpi(IRequires_MuxedAdcSpi * owner, IAddressingSpace & space, FmcSpiDevice::MapRegisters & regs,
                  bool Fmc500Mode = false);
    virtual ~FmcAFE_AdcSpi();

    unsigned int AdcSpiCount() {  return SpiCount;  }
    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_MuxedAdcSpi *  MDC;
    FmcSpiDevice             SpiDev;
    unsigned int             SpiCount;
};

//===========================================================================
//  CLASS FmcAFE_AdcCalibrationControl -- Fmc AFE ADC Cal Control "SubDevice"
//===========================================================================

class FmcAFE_AdcCalibrationControl : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_AdcCalibrationControl::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcCal_Addr;
    };

    FmcAFE_AdcCalibrationControl(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcAFE_AdcCalibrationControl();

    //
    //  Methods
    void          StartCalibration();
    unsigned int  CalibrationResults();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_AdcCalibrationControl::AdcCalibrationRegister -- Configure ADC Cal
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcCalibrationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       CalStart;
        RegisterBitGroup  CalDone;

    public:
        AdcCalibrationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               CalStart(*this, 0),   CalDone(*this, 16, 16)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *     MDC;
    AdcCalibrationRegister  AdcCalibrationReg;

};

//===========================================================================
//  CLASS Fmc110_AdcCalibrationControl -- Fmc AFE ADC Cal Control "SubDevice"
//===========================================================================

class Fmc110_AdcCalibrationControl : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc110_AdcCalibrationControl::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcCal_Addr;
    };

    Fmc110_AdcCalibrationControl(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~Fmc110_AdcCalibrationControl();

    //
    //  Methods
    void          StartCalibration(unsigned int ch);
    void          WaitForCal();
    unsigned int  CalibrationResults(unsigned int ch);
    bool          CalibrationDone()
                      {  return AdcCalibrationReg.CalDone.Value();  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc110_AdcCalibrationControl::AdcCalibrationRegister -- Configure ADC Cal
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcCalibrationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       CalStart;
        RegisterBit       CalStatus;
        RegisterBit       CalSkip;
        RegisterBitGroup  CalEyeAligned;
        RegisterBit       CalPrbsLocked;
        RegisterBit       CalPrbsAligned;
        RegisterBit       CalDone;
        RegisterBit       CalAdcClkStopped;
        RegisterBit       CalAdcClkLocked;

    public:
        AdcCalibrationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               CalStart(*this, 0),  CalStatus(*this, 8), CalSkip(*this, 12),
               CalEyeAligned(*this, 14, 13), CalPrbsLocked(*this, 27),
               CalPrbsAligned(*this, 28), CalDone(*this, 29),
               CalAdcClkStopped(*this, 30), CalAdcClkLocked(*this, 31)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *     MDC;
    AdcCalibrationRegister  AdcCalibrationReg;

};

//===========================================================================
//  CLASS FmcVga_I2C_Bus -- VGA FMC I2C Bus "SubDevice"
//===========================================================================

class FmcVga_I2C_Bus : public IX6SubDevice//,  public IX6_I2C_uDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcVga_I2C_Bus::I2C_SubAddr  -- Device SubAddr on I2C Bus
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class I2C_SubAddr
    {
    public:
        unsigned int   Vga0_SubAddr;
        unsigned int   Vga1_SubAddr;
        unsigned int   Vga2_SubAddr;
        unsigned int   Vga3_SubAddr;
        unsigned int   Vga4_SubAddr;
        unsigned int   VgaIdrom_SubAddr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcVga_I2C_Bus::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   I2CControl_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcVga_I2C_Bus::LTC2635_8  -- DAC controller wrapper
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class LTC2635_8
    {
    public:
        enum CommandCodes { cmdWriteN=0x0,       cmdPowerN=0x1,      cmdWriteN_PowerAll=0x2,
                            cmdWriteNPowerN=0x3, cmdPowerDownN=0x4,  cmdWriteN_PowerDownAll=0x5,
                            cmdInternalRef=0x6,  cmdExtRef=0x1,      cmdNop=0xF,
                          };
        enum AddressCodes { addrDac0, addrDac1, addrDac2, addrDac3, addrAllDacs=0xF };

        LTC2635_8(FmcVga_I2C_Bus * owner): Owner(owner)
            {  }
        virtual ~LTC2635_8()
            {  }

        void  SetAdcGain(unsigned int dev, unsigned int adc, float db_gain);

        void  SendCommand(unsigned int dev, unsigned int cmd_code,
                             unsigned int addr_code,
                             unsigned int data);

    protected:
        FmcVga_I2C_Bus *  Owner;

    };

    friend class LTC2635_8;
    
    FmcVga_I2C_Bus(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs, I2C_SubAddr & subaddrs );
    virtual ~FmcVga_I2C_Bus();

    LTC2635_8 &   VgaDev() {  return Vga;  }
    I2CDeviceN &  VgaIdrom() {  return VgaIdrom_I2C;  }
    AdvI2C_16Idrom & VgaIdromDevice()  {  return VgaIdromDev;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    //virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcVga_I2C_Bus::BusI2CRegister  -- 'Fmc Bus' I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BusI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Data_Out;
        RegisterBit  Clock_Out;
        RegisterBit  Data_In;
        RegisterBit  Clock_In;
        RegisterBit  VgaDacLatch;

    public:
        BusI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data_Out(*this, 0),   Clock_Out(*this, 1),
               Data_In(*this, 2),    Clock_In(*this, 3),
               VgaDacLatch(*this, 8)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VgaBusI2C_Impl  -- FmcBus I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VgaBusI2C_Impl : public I2CDeviceIntf
    {
    public:
        VgaBusI2C_Impl(BusI2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~VgaBusI2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        BusI2CRegister &  Reg;
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    BusI2CRegister       VgaBusI2CReg;
    //
    //  Interface Impls
    VgaBusI2C_Impl       VgaBusIntf;

    //  I2C Bus Devices
    I2CDeviceN           Vga0_I2C;
    I2CDeviceN           Vga1_I2C;
    I2CDeviceN           Vga2_I2C;
    I2CDeviceN           Vga3_I2C;
    I2CDeviceN           Vga4_I2C;
    I2CDevice16          VgaIdrom_I2C;
    AdvI2C_16Idrom       VgaIdromDev;
    LTC2635_8            Vga;

    I2CDeviceN &         VgaI2C(unsigned int idx);
    void                 VgaDacLatch();
};

//===========================================================================
//  CLASS Fmc_AdcCalCoefficientLoad -- X6 ADC Cal Coeff Loader "SubDevice"
//===========================================================================

class Fmc_AdcCalCoefficientLoad : public IX6SubDevice, public IAdvGainCalibrationDevice
{
public:
    //=======================================================================
    //  CLASS Fmc_AdcCalCoefficientLoad::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   GainCoeff_Addr;
        unsigned int   OffsetCoeff_Addr;
    };

    Fmc_AdcCalCoefficientLoad(IRequires_AdcCalParts * mdc, IAddressingSpace & space, MapRegisters & regs, unsigned int MaxAdcs );
    virtual ~Fmc_AdcCalCoefficientLoad();

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value);
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_AdcCalParts *  MDC;
    
    RegisterArray<Register>    GainCoeffArray;
    RegisterArray<Register>    OffsetCoeffArray;

    unsigned int                FIncomingAdcs;
    ICalibrationValueOverride * CalOverride;

};

//===========================================================================
//  CLASS IRequires_AD7763_Support -- call-up needed by AD7763 SPI interface
//===========================================================================

class IRequires_AD7763_Support
{
public:
    virtual void         Reset_AD7763() = 0;
    virtual void         Sync_AD7763() = 0;

    virtual ~IRequires_AD7763_Support()
        {}
};


//===========================================================================
//  CLASS AD7763_SpiDevice -- Implementation of AD7763 SPI interface
//===========================================================================

class AD7763_SpiDevice
{
public:

    enum Read_Reg_pseudoAddresses {  rdStatus, rdOffset, rdGain, rdOverrange };

    //=======================================================================
    //  CLASS AD7763_SpiDevice::MapRegisters -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int  Device_Addr;       // offset of SPI Device Enable register
        unsigned int  Write_Addr;        // offset of SPI Write register
        unsigned int  Read_Addr;         // offset of SPI Read register
    };

    AD7763_SpiDevice(IRequires_AD7763_Support * mdc, IAddressingSpace &space, MapRegisters & regs, std::string tag="")
        :  MDC(mdc), Tag(tag), RegisterCache_0x1(0x0),
           ClockPeriod(4),
           SpiDeviceReg(space, regs.Device_Addr),
           SpiWriteReg(space, regs.Write_Addr),
           SpiReadReg(space,  regs.Read_Addr)
        {}
    virtual ~AD7763_SpiDevice()
        {}

    void  SetSpiClockPeriod(double adc_clock_frequency);

    unsigned short  Spi(int device, int addr);
    void            Spi(int device, int addr, unsigned short value, bool no_dump=false);

protected:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiWriteRegister  -- SPI Write register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SpiWriteRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup    WriteData;
        RegisterBitGroup    Address;

        SpiWriteRegister(IAddressingSpace &space, unsigned int addr)
            : inherited(space, addr), WriteData(*this, 0, 16),
              Address(*this, 16, 12)
        {
        }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiReadRegister  -- SPI Read register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SpiReadRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit         Ready;
        RegisterBit         DataValid;
        RegisterBitGroup    ReadData;

        SpiReadRegister(IAddressingSpace &space, unsigned int addr)
            : inherited(space, addr),
              Ready(*this, 30), DataValid(*this, 31),  ReadData(*this, 0, 16)
        {
        }
    };

protected:
    IRequires_AD7763_Support * MDC;
    std::string  Tag;
    unsigned int RegisterCache_0x1;
    unsigned int ClockPeriod;   // in uS


    Register                    SpiDeviceReg;
    SpiWriteRegister            SpiWriteReg;
    SpiReadRegister             SpiReadReg;

    Trace        Debug;

public:
    void  SpiReady();
protected:
    void  SpiNotReady();
    void  SpiDataValid();

    virtual void            SetupSpiDevice(int device);
    virtual void            SetupSpiWrite(int addr, unsigned short value);
    virtual unsigned short  ReadSpiData();
    virtual bool            ReadSpiReadyBit();
    virtual unsigned int    DumpSpiRegister();

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
