//---------------------------------------------------------------------------
// FmcSubDevice_AnalogOut_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <FmcSubDevice_AnalogOut_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS FmcAFE_DacCommon -- X6 AFE Dac Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_DacCommon
//---------------------------------------------------------------------------

namespace {

    double ii_power_of_two(int exponent)
    {
        const int max_exponent = 30;
        double result = 1.0;
        while (exponent > 0)
            {
            int shift;
            //  we can't shift by more than 31 bits accurately, so break up
            //    into groups of max_exponent
            if (exponent > max_exponent)
                shift = max_exponent;
            else
                shift = exponent;
            unsigned int partial = 1<<shift;
            result *= static_cast<double>(partial);

            exponent -= shift;
            }
        return result;
    }
}

const unsigned int MaxFmcDacChannels = 64;

//---------------------------------------------------------------------------
//  constructor for class FmcAFE_DacCommon
//---------------------------------------------------------------------------

FmcAFE_DacCommon::FmcAFE_DacCommon(IRequires_FmcCommonParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       DacEnable_Lo_Reg(space, regs.AfeDacEnable_Lo_Addr),
       DacEnable_Hi_Reg(space, regs.AfeDacEnable_Hi_Addr),
       DacPower_Lo_Reg(space, regs.AfeDacPower_Lo_Addr),
       DacPower_Hi_Reg(space, regs.AfeDacPower_Hi_Addr),
       DacTriggerCfgReg(space, regs.AfeDacTriggerCfg_Addr),
       DacDecimationReg(space, regs.AfeDacDecimation_Addr),
       DacPhaseIncrReg(space, regs.AfeDacPhaseIncr_Addr),
       DacResetReg(space, regs.AfeDacReset_Addr),
       FDevices(0), FChannelsPerDevice(0), FPhysDevices(0), FChannelsPerPhysDevice(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_DacCommon
//---------------------------------------------------------------------------

FmcAFE_DacCommon::~FmcAFE_DacCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_DacCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_FmcCommonParts::devDac);
    MDC->GetAnalogDeviceInfo(IRequires_FmcCommonParts::devDac, FDevices, FChannelsPerDevice);
    MDC->GetAnalogPhysicalDeviceInfo(IRequires_FmcCommonParts::devDac, FPhysDevices, FChannelsPerPhysDevice);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_DacCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_DacCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::ChannelDisableAll()
{
    DacEnable_Lo_Reg.Value(0);
    DacEnable_Hi_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  FmcAFE_DacCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<MaxFmcDacChannels; i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::ChannelEnabled(int ch, bool state)
{
    if (ch<32)
        DacEnable_Lo_Reg.Bit(ch, state);
    else
        DacEnable_Hi_Reg.Bit(ch-32, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::ChannelEnabled(int ch) const
{
    if (ch<32)
        return DacEnable_Lo_Reg.Bit(ch);
    else
        return DacEnable_Hi_Reg.Bit(ch-32);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::ChannelPowerDownAll()
{
    DacPower_Lo_Reg.Value(0);
    DacPower_Hi_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::ChannelPowered(int ch, bool state)
{
    if (ch<32)
        DacPower_Lo_Reg.Bit(ch, state);
    else
        DacPower_Hi_Reg.Bit(ch-32, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::ChannelPowered(int ch) const
{
    if (ch<32)
        return DacPower_Lo_Reg.Bit(ch);
    else
        return DacPower_Hi_Reg.Bit(ch-32);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DeviceIsEnabled() -- Enable an I/O device
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::DeviceIsEnabled(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DeviceIsEnabled() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::DeviceIsEnabled(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelEnabled(base);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DevicePowered() -- Enable an I/O device
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::DevicePowered(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DevicePowered() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::DevicePowered(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelPowered(base);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ActiveDevices() -- return count of active devices
//---------------------------------------------------------------------------

unsigned int  FmcAFE_DacCommon::ActiveDevices()
{
    unsigned int active_devices(0);
    for (unsigned int i=0; i<Devices(); i++)
        if (DeviceIsEnabled(i)==true)
            active_devices++;

    return active_devices;
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    DacTriggerCfgReg.DacFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int FmcAFE_DacCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = DacTriggerCfgReg.DacFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::EdgeTrigger(bool state)
{
    DacTriggerCfgReg.DacRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::EdgeTrigger() const
{
    return DacTriggerCfgReg.DacRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::ExternalTrigger(bool state)
{
    DacTriggerCfgReg.DacExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::ExternalTrigger() const
{
    return DacTriggerCfgReg.DacExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::FramedTriggerEnabled(bool state)
{
    DacTriggerCfgReg.DacFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::FramedTriggerEnabled() const
{
    return DacTriggerCfgReg.DacFramedMode.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::Decimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::Decimation(unsigned int samples)
{
    DacDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::Decimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int FmcAFE_DacCommon::Decimation() const
{
    return DacDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DacTestToneFrequency() -- Set Reset tone_freq specified I/O channel
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, double tone_freq, int channels)
{
    Debug.Log("Test Frequency", tone_freq);
    Debug.Log("Dac Frequency", dac_clock_freq);
    Debug.Log("Channels", channels);

    double Two32 = ii_power_of_two(32);

    double phase_incr = tone_freq * (Two32) / dac_clock_freq ;
    Debug.Log("Phase Incr", phase_incr);

    DacPhaseIncrReg.Value(static_cast<unsigned int>(phase_incr));
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DacTestToneFrequency() -- Query Reset tone_freq of an I/O channel
//---------------------------------------------------------------------------

double FmcAFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, int /*channels*/) const
{
    double Two32 = ii_power_of_two(32);
    unsigned int raw_phase_incr = DacPhaseIncrReg.Value();

    double tone_freq = static_cast<double>(raw_phase_incr) / (Two32) * dac_clock_freq;
    return tone_freq;
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DacReset() -- Reset DACs
//---------------------------------------------------------------------------

void FmcAFE_DacCommon::DacReset(bool state)
{
    DacResetReg.Bit(0, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DacPowerGood() -- Dac Power Good Bit
//---------------------------------------------------------------------------

bool FmcAFE_DacCommon::DacPowerGood()
{
    return DacResetReg.Bit(8);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DRR_Bit() -- write DRR register bit (board specific)
//---------------------------------------------------------------------------

void  FmcAFE_DacCommon::DRR_Bit(unsigned int bit, bool state)
{
    DacResetReg.Bit(bit, state);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::DRR_Bit() -- read DRR register bit (board specific)
//---------------------------------------------------------------------------

bool  FmcAFE_DacCommon::DRR_Bit(unsigned int bit)
{
    return DacResetReg.Bit(bit);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::PhysicalDeviceEnabled() --
//---------------------------------------------------------------------------
//  Physical device should be enabled if any channel on it is enabled...

bool FmcAFE_DacCommon::PhysicalDeviceEnabled(unsigned int phys_dev) const
{
    bool enabled(false);
    if (phys_dev>=FPhysDevices)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int end   = start + FChannelsPerPhysDevice;

    for (unsigned int ch=start;  ch<end; ch++)
        enabled |= ChannelEnabled(ch);
    return enabled;
}

//---------------------------------------------------------------------------
//  FmcAFE_DacCommon::PhysicalChannelEnabled() --
//---------------------------------------------------------------------------
//  return enabled state for the Nth channel on device D...

bool FmcAFE_DacCommon::PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const
{
    if (phys_dev>=FPhysDevices || device_ch>=FChannelsPerPhysDevice)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int ch  = start + device_ch;

    return ChannelEnabled(ch);
}

//===========================================================================
//  CLASS FmcAFE_DacSpi -- Fmc AFE DAC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_DacSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcAFE_DacSpi
//---------------------------------------------------------------------------

FmcAFE_DacSpi::FmcAFE_DacSpi(IRequires_MuxedDacSpi * mdc, IAddressingSpace & space, FmcSpiDevice::MapRegisters & regs)
    :  MDC(mdc), SpiDev(space, regs, "Fmc Dac"), SpiCount(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_DacSpi
//---------------------------------------------------------------------------

FmcAFE_DacSpi::~FmcAFE_DacSpi()
{
}

//---------------------------------------------------------------------------
//  FmcAFE_DacSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_DacSpi::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SpiCount = MDC->GetDacSpiDeviceCount();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  FmcAFE_DacSpi::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_DacSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_DacSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_DacSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  FmcAFE_DacSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  FmcAFE_DacSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiCount)
        SpiDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  FmcAFE_DacSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  FmcAFE_DacSpi::Spi(unsigned int device, unsigned int address)
{
    if (device < SpiCount)
        return SpiDev.Spi(device, address);
    else
        return 0;
}

//===========================================================================
//  CLASS Fmc_DacCalCoefficientLoad -- Fmc Dac Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class Fmc_DacCalCoefficientLoad
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class Fmc_DacCalCoefficientLoad
//---------------------------------------------------------------------------

Fmc_DacCalCoefficientLoad::Fmc_DacCalCoefficientLoad(IRequires_DacCalParts * mdc,
                IAddressingSpace & space, MapRegisters & regs,
                unsigned int MaxDacs )
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, MaxDacs),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, MaxDacs),
       FDacChannels(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc_DacCalCoefficientLoad
//---------------------------------------------------------------------------

Fmc_DacCalCoefficientLoad::~Fmc_DacCalCoefficientLoad()
{
    if (CalOverride)
        delete CalOverride;
}

//---------------------------------------------------------------------------
//  Fmc_DacCalCoefficientLoad::ConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc_DacCalCoefficientLoad::ConstructCustomParts()
{
    FDacChannels = MDC->GetCalibrationDacCount();
    CalOverride = MDC->GetDacCalibrationOverride();
}

//---------------------------------------------------------------------------
//  Fmc_DacCalCoefficientLoad::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc_DacCalCoefficientLoad::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    for (unsigned int i=0; i<FDacChannels; i++)
        {
        SetCalibrationGain(i, 0x10000);
        SetCalibrationOffset(i, 0x000);
        }
}

//---------------------------------------------------------------------------
//  Fmc_DacCalCoefficientLoad::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc_DacCalCoefficientLoad::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  Fmc_DacCalCoefficientLoad::SetCalibrationOffset()
//---------------------------------------------------------------------------

void  Fmc_DacCalCoefficientLoad::SetCalibrationOffset(unsigned int ch, unsigned int value)
{
    Debug.Log("Dac Cal Offset=", value, 16);
    if (ch<FDacChannels)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideOffset(value);
            if (CalOverride->OverrideOffsetUpdate())
                {
                CalOverride->WriteOffset(ch, value);
                return;
                }
            }
        OffsetCoeffArray[ch].Value(value);
        }
}

//---------------------------------------------------------------------------
//  Fmc_DacCalCoefficientLoad::SetCalibrationGain()
//---------------------------------------------------------------------------

void  Fmc_DacCalCoefficientLoad::SetCalibrationGain(unsigned int ch, unsigned int value)
{
    Debug.Log("Dac Cal Gain=", value, 16);
    if (ch<FDacChannels)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideGain(value);
            if (CalOverride->OverrideGainUpdate())
                {
                CalOverride->WriteGain(ch, value);
                return;
                }
            }
        GainCoeffArray[ch].Value(value);
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative

