// FmcSubDevice_AnalogOut_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FmcSubDevice_AnalogOut_MbH
#define FmcSubDevice_AnalogOut_MbH

#include <X6Support_Mb.h>
#include <VpxSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SpiDevice_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS FmcAFE_DacCommon -- Fmc AFE DAC Common Registers "SubDevice"
//===========================================================================

class FmcAFE_DacCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_DacCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeDacEnable_Lo_Addr;
        unsigned int   AfeDacEnable_Hi_Addr;
        unsigned int   AfeDacPower_Lo_Addr;
        unsigned int   AfeDacPower_Hi_Addr;
        unsigned int   AfeDacTriggerCfg_Addr;
        unsigned int   AfeDacDecimation_Addr;
        unsigned int   AfeDacPhaseIncr_Addr;
        unsigned int   AfeDacReset_Addr;
        };

    FmcAFE_DacCommon(IRequires_FmcCommonParts * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcAFE_DacCommon();

    //
    //  Methods
    unsigned int  Devices() const  {  return FDevices;  }
    unsigned int  ChannelsPerDevice() const {  return FChannelsPerDevice;  }
    unsigned int  Channels() const {  return FDevices * FChannelsPerDevice;  }

    virtual void  DeviceIsEnabled(int dev, bool state);
    virtual bool  DeviceIsEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    //  Physical Device Enable Mapping
    bool  PhysicalDeviceEnabled(unsigned int phys_dev) const;
    bool  PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const;

    void   DacTestToneFrequency(double dac_clock_freq, double tone_freq, int channels); // in Hz
    double DacTestToneFrequency(double dac_clock_freq, int channels) const;

    void   DacReset(bool state);  // bit 0
    bool   DacPowerGood();        // bit 8

    //  Hit (other) bits in DacReset Register
    void   DRR_Bit(unsigned int bit, bool state);
    bool   DRR_Bit(unsigned int bit);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_DacCommon::DacTriggerCfgRegister -- Configure DAC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DacFrameSize;
        RegisterBit       DacRisingEdge;
        RegisterBit       DacFramedMode;
        RegisterBit       DacExternalTrigger;

    public:
        DacTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DacFrameSize(*this, 0, 24),   DacRisingEdge(*this, 29),
               DacFramedMode(*this, 30),     DacExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_DacCommon::DacDecimationRegister  -- DAC Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        DacDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_FmcCommonParts *    MDC;
    IFrameCalcDevice *            Dev;
    Register                      DacEnable_Lo_Reg;
    Register                      DacEnable_Hi_Reg;
    Register                      DacPower_Lo_Reg;
    Register                      DacPower_Hi_Reg;
    DacTriggerCfgRegister         DacTriggerCfgReg;
    DacDecimationRegister         DacDecimationReg;
    Register                      DacPhaseIncrReg;
    Register                      DacResetReg;

    unsigned int                  FDevices;
    unsigned int                  FChannelsPerDevice;
    unsigned int                  FPhysDevices;
    unsigned int                  FChannelsPerPhysDevice;

};

//===========================================================================
//  CLASS FmcAFE_DacSpi -- Fmc Muxed DAC Spi Device Manager "SubDevice"
//===========================================================================

class FmcAFE_DacSpi : public IX6SubDevice
{
public:

    FmcAFE_DacSpi(IRequires_MuxedDacSpi * owner, IAddressingSpace & space, FmcSpiDevice::MapRegisters & regs);
    virtual ~FmcAFE_DacSpi();

    unsigned int DacSpiCount() {  return SpiCount;  }
    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_MuxedDacSpi *  MDC;
    FmcSpiDevice             SpiDev;
    unsigned int             SpiCount;
};


//===========================================================================
//  CLASS Fmc_DacCalCoefficientLoad -- X6 DAC Cal Coeff Loader "SubDevice"
//===========================================================================

class Fmc_DacCalCoefficientLoad : public IX6SubDevice, public IAdvGainCalibrationDevice
{
public:
    //=======================================================================
    //  CLASS Fmc_DacCalCoefficientLoad::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   GainCoeff_Addr;
        unsigned int   OffsetCoeff_Addr;
    };

    Fmc_DacCalCoefficientLoad(IRequires_DacCalParts * mdc, IAddressingSpace & space, MapRegisters & regs, unsigned int MaxDacs );
    virtual ~Fmc_DacCalCoefficientLoad();

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value);
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_DacCalParts *  MDC;

    RegisterArray<Register>    GainCoeffArray;
    RegisterArray<Register>    OffsetCoeffArray;

    unsigned int                FDacChannels;
    ICalibrationValueOverride * CalOverride;

};



#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative


#endif
