//---------------------------------------------------------------------------
// FmcSubDevice_Stream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <FmcSubDevice_Stream_Mb.h>
#include <SystemSupport_Mb.h>
#include <FloatingPtSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS FmcInformation -- FMC device information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcInformation
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcInformation - standard model
//---------------------------------------------------------------------------

FmcInformation::FmcInformation(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs,
                               IAddressingSpace & P7_space, Pex7CopModeMapRegisters& P7_regs)
    :  MDC(mdc),
       ControlStatusReg(space, regs.FmcControlStatus_Addr),
       BoardIdReg(space, regs.FmcId_Addr),
       BidirControlReg(space, regs.FmcBidir_Addr),
       Clk0_M2C_Reg(space, regs.FmcClk0_M2C_Status_Addr),
       Clk1_M2C_Reg(space, regs.FmcClk1_M2C_Status_Addr),
       Clk2_M2C_Reg(space, regs.FmcClk2_M2C_Status_Addr),
       Clk3_M2C_Reg(space, regs.FmcClk3_M2C_Status_Addr),
       Clk2_C2M_Reg(space, regs.FmcClk2_C2M_Status_Addr),
       Clk3_C2M_Reg(space, regs.FmcClk3_C2M_Status_Addr),
       P7Cop_VAdjReg(P7_space,    P7_regs.Pex7_VAdj_Status_Addr),
       P7Cop_ClkCtrlReg(P7_space, P7_regs.Pex7_C2M_Clock_Ctrl_Addr),
       IsPex7Style(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcInformation
//---------------------------------------------------------------------------

FmcInformation::~FmcInformation()
{
}

//---------------------------------------------------------------------------
//  FmcInformation::OpenHardware()
//---------------------------------------------------------------------------

void  FmcInformation::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  FmcInformation::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcInformation::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcPresent() - return FmcPresent bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcPresent()
{
    return ControlStatusReg.PresentN.Value() ? false : true;
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcVadjEnable() - return FmcVadjEnable bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcVadjEnable()
{
    return ControlStatusReg.VADJ_Disable.Value() ? false : true;
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcVadjEnable() - return FmcVadjEnable bit
//---------------------------------------------------------------------------

void  FmcInformation::FmcVadjEnable(bool value)
{
    ControlStatusReg.VADJ_Disable.Value( !value );
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcVadjForced() - return FmcVadjForced bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcVadjForced()
{
    return ControlStatusReg.VADJ_Forced.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcVadjPower() - return VadjPower bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcVadjPower()
{
    return ControlStatusReg.VADJ_PowerGood.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcVadjLevel() - return VadjPower Level
//---------------------------------------------------------------------------

short  FmcInformation::FmcVadjLevel()
{
    return ControlStatusReg.VADJ_Level.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::Pex7_FmcVadjLevel() - return VadjPower Level
//---------------------------------------------------------------------------

short  FmcInformation::Pex7_FmcVadjLevel()
{
    return P7Cop_VAdjReg.VADJ_Level.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcM2CPower() - return FmcM2CPower bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcM2CPower()
{
    return ControlStatusReg.M2CPowerGoodN.Value() ? false : true;
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcC2MPower() - return FmcC2MPower bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcC2MPower()
{
    return ControlStatusReg.C2MPowerGoodN.Value() ? false : true;
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcC2MPower() - set FmcC2MPower bit
//---------------------------------------------------------------------------

void  FmcInformation::FmcC2MPower(bool value)
{
    ControlStatusReg.C2MPowerGoodN.Value( !value );
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcBoardId() - return ID
//---------------------------------------------------------------------------

unsigned int  FmcInformation::FmcBoardId()
{
    return BoardIdReg.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcBidirClockC2M() - return Bidir clock direction bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcBidirClockC2M()
{
    return BidirControlReg.Bidir_C2M.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcClock2_3_Enable() - return FmcClock2_3_Enable bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcClock2_3_Enable()
{
    return BidirControlReg.Fmc_2_3_Enable.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcClock2_3_Enable() - set FmcClock2_3_Enable bit
//---------------------------------------------------------------------------

void  FmcInformation::FmcClock2_3_Enable(bool value)
{
    BidirControlReg.Fmc_2_3_Enable.Value( value );
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcClock2_IsSingleEnded() - return FmcClock2_IsSingleEnded bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcClock2_IsSingleEnded()
{
    return BidirControlReg.Fmc_2_SingleEnded.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcClock2_IsSingleEnded() - set FmcClock2_IsSingleEnded bit
//---------------------------------------------------------------------------

void  FmcInformation::FmcClock2_IsSingleEnded(bool value)
{
    BidirControlReg.Fmc_2_SingleEnded.Value( value );
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcClock3_IsSingleEnded() - return FmcClock2_IsSingleEnded bit
//---------------------------------------------------------------------------

bool  FmcInformation::FmcClock3_IsSingleEnded()
{
    return BidirControlReg.Fmc_3_SingleEnded.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::FmcClock3_IsSingleEnded() - set FmcClock2_IsSingleEnded bit
//---------------------------------------------------------------------------

void  FmcInformation::FmcClock3_IsSingleEnded(bool value)
{
    BidirControlReg.Fmc_3_SingleEnded.Value( value );
}

//---------------------------------------------------------------------------
//  FmcInformation::Pex7_FmcClock2_IsFpga() - return C2M_2_SourceIsFpga bit
//---------------------------------------------------------------------------

bool  FmcInformation::Pex7_FmcClock2_IsFpga()
{
    return P7Cop_ClkCtrlReg.C2M_2_SourceIsFpga.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::Pex7_FmcClock2_IsFpga() - set C2M_2_SourceIsFpga bit
//---------------------------------------------------------------------------

void  FmcInformation::Pex7_FmcClock2_IsFpga(bool value)
{
    P7Cop_ClkCtrlReg.C2M_2_SourceIsFpga.Value( value );
}

//---------------------------------------------------------------------------
//  FmcInformation::Pex7_FmcClock3_IsFpga() - return C2M_3_SourceIsFpga bit
//---------------------------------------------------------------------------

bool  FmcInformation::Pex7_FmcClock3_IsFpga()
{
    return P7Cop_ClkCtrlReg.C2M_3_SourceIsFpga.Value();
}

//---------------------------------------------------------------------------
//  FmcInformation::Pex7_FmcClock3_IsFpga() - set C2M_3_SourceIsFpga bit
//---------------------------------------------------------------------------

void  FmcInformation::Pex7_FmcClock3_IsFpga(bool value)
{
    P7Cop_ClkCtrlReg.C2M_3_SourceIsFpga.Value( value );
}

//---------------------------------------------------------------------------
//  FmcInformation::Fmc_Clk_M2C_Rate() - read M2C rate measurement
//---------------------------------------------------------------------------

unsigned int  FmcInformation::Fmc_Clk_M2C_Rate(int clock)
{
    switch(clock)
        {
        default:
            return 0;
        case 0:
            return Clk0_M2C_Reg.Freq.Value();
        case 1:
            return Clk1_M2C_Reg.Freq.Value();
        case 2:
            return Clk2_M2C_Reg.Freq.Value();
        case 3:
            return Clk3_M2C_Reg.Freq.Value();
        }
}

//---------------------------------------------------------------------------
//  FmcInformation::Fmc_Clk_C2M_Rate() - read C2M rate measurement
//---------------------------------------------------------------------------

unsigned int  FmcInformation::Fmc_Clk_C2M_Rate(int clock)
{
    switch(clock)
        {
        default:
            return 0;
        case 2:
            return Clk2_C2M_Reg.Freq.Value();
        case 3:
            return Clk3_C2M_Reg.Freq.Value();
        }
}

//---------------------------------------------------------------------------
//  FmcInformation::WaitForVadjPowerGood() -- wait for power to come up
//---------------------------------------------------------------------------

void  FmcInformation::WaitForVadjPowerGood()
{
    // Wait up to 1S for power status read to come in
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool bit = FmcVadjPower();
        if (bit==true)
            return;
        else
            {
            std::stringstream msg;
            msg << std::hex << " FmcControl:  " << ControlStatusReg.Value();
            Debug.Log(msg.str());

            Innovative::Sleep(100);
            }
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " VADJ Power Good Status Wait Failure (>1s)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  FmcInformation::WaitForPowerGood() -- wait for power to come up
//---------------------------------------------------------------------------

void  FmcInformation::WaitForPowerGood()
{
    // Wait up to 1S for power status read to come in
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool bit = FmcM2CPower();
        if (bit==true)
            return;
        else
            {
            std::stringstream msg;
            msg << std::hex << " FmcControl:  " << ControlStatusReg.Value();
            Debug.Log(msg.str());

            Innovative::Sleep(100);
            }
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " Power Good Status Wait Failure (>1s)";
    Debug.Log(msg.str());

}

//===========================================================================
//  CLASS FmcStd_I2C_Bus -- Standard FMC I2C Bus "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcStd_I2C_Bus
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcStd_I2C_Bus
//---------------------------------------------------------------------------

FmcStd_I2C_Bus::FmcStd_I2C_Bus(IRequires_Nothing * mdc, IAddressingSpace & space,
                                   MapRegisters & regs, I2C_SubAddr & subaddrs )
    :  MDC(mdc),
       FmcBusI2CReg(space, regs.I2CControl_Addr),
       FmcBusIntf(FmcBusI2CReg),
       TempControl_I2C(FmcBusIntf, I2CDevice::SubAddress, subaddrs.TempControl_SubAddr),
       FmcIdrom_I2C(FmcBusIntf, I2CDevice::SubAddress, subaddrs.FmcIdrom_SubAddr),
       TempControl(this),
       FmcIdromDev(FmcIdrom_I2C),
       FmcRomInfo(FmcIdromDev)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcStd_I2C_Bus
//---------------------------------------------------------------------------

FmcStd_I2C_Bus::~FmcStd_I2C_Bus()
{
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::OpenHardware()
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // Auto-detect device type to configure ROM
    unsigned int dev_id = Temp().GetDeviceId();

    Debug.Log("FMC IDROM DevID=", dev_id, 16);

    // STS2002 = 0x3   STS2004 = 0x22
    switch (dev_id)
        {
        default:
        case 0x3:
            FmcIdrom().DualPageRomMode(false);
            break;
        case 0x22:
            FmcIdrom().DualPageRomMode(true);
            break;
        }

    //  Initialize ROMinfo from ROM
    FmcRomInfo.LoadFromRom();

}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//===========================================================================
//  CLASS FmcStd_I2C_Bus::Tmp175 -- TemperatureController Wrapper
//===========================================================================
//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::GetT() -- read current T
//---------------------------------------------------------------------------

float  FmcStd_I2C_Bus::Tmp175::GetT()
{
    std::vector<int> Data(2);
    Data[0] = 0;
    Data[1] = 0;
    Owner->TempControl_I2C.MultiRegR(0, &Data[0], static_cast<int>(Data.size()) );
    unsigned int rawT = Data[0]<<8 | Data[1];

    return RawToT(rawT);
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::GetHighT() -- read current HighT
//---------------------------------------------------------------------------

float  FmcStd_I2C_Bus::Tmp175::GetHighT()
{
    std::vector<int> Data(2);
    Data[0] = 0;
    Data[1] = 0;
    Owner->TempControl_I2C.MultiRegR(3, &Data[0], static_cast<int>(Data.size()) );
    unsigned int rawT = Data[0]<<8 | Data[1];

    return RawToT(rawT);
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::GetLowT() -- read current LowT
//---------------------------------------------------------------------------

float  FmcStd_I2C_Bus::Tmp175::GetLowT()
{
    std::vector<int> Data(2);
    Data[0] = 0;
    Data[1] = 0;
    Owner->TempControl_I2C.MultiRegR(2, &Data[0], static_cast<int>(Data.size()) );
    unsigned int rawT = Data[0]<<8 | Data[1];

    return RawToT(rawT);
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::SetConfig() -- set Config Register
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::Tmp175::SetConfig(unsigned int bits)
{
    Owner->TempControl_I2C.Reg(1, bits);
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::SetConfig() -- set Config Register
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::Tmp175::SetConfig(bool one_shot, unsigned int resolution, unsigned int faultQ,
                               bool alertActiveHigh, bool interruptMode, bool shutdownMode)
{
    MemoryBitManipulator bits(0);
    bits.Bit(7, one_shot);
    bits.Field(5, 2, resolution);
    bits.Field(3, 2, faultQ);
    bits.Bit(2, alertActiveHigh);
    bits.Bit(1, interruptMode);
    bits.Bit(0, shutdownMode);

    SetConfig( bits.Value() );
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::GetConfig() -- get Config Register
//---------------------------------------------------------------------------

unsigned int  FmcStd_I2C_Bus::Tmp175::GetConfig()
{
    return Owner->TempControl_I2C.Reg(1);
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::GetDeviceId() -- get Device ID part of Full DevId
//---------------------------------------------------------------------------

unsigned int FmcStd_I2C_Bus::Tmp175::GetDeviceId()
{
    return GetFullDevId() >> 8;
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::GetFullDevId() -- get Full Device ID
//---------------------------------------------------------------------------

unsigned int FmcStd_I2C_Bus::Tmp175::GetFullDevId()
{
    std::vector<int> Data(2);

    Data[0] = 0;
    Data[1] = 0;

    Owner->TempControl_I2C.MultiRegR(7, &Data[0], static_cast<int>(Data.size()) );

    unsigned int output;
    output = Data[0]<<8 | Data[1];
    return output;
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::SetHighT() -- set HighT
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::Tmp175::SetHighT(float degreesC)
{
    std::vector<int> Data(2);
    unsigned int rawT = TToRaw(degreesC);

    Data[0] = rawT>>8;
    Data[1] = rawT&0xFF;

    Owner->TempControl_I2C.MultiRegW(3, &Data[0], static_cast<int>(Data.size()) );
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::SetLowT() -- set LowT
//---------------------------------------------------------------------------

void  FmcStd_I2C_Bus::Tmp175::SetLowT(float degreesC)
{
    std::vector<int> Data(2);
    unsigned int rawT = TToRaw(degreesC);

    Data[0] = rawT>>8;
    Data[1] = rawT&0xFF;

    Owner->TempControl_I2C.MultiRegW(2, &Data[0], static_cast<int>(Data.size()) );
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::RawToT() -- convert raw T register to degrees C
//---------------------------------------------------------------------------

float  FmcStd_I2C_Bus::Tmp175::RawToT(short raw)
{
    float out = raw;
    return out/256;
}

//---------------------------------------------------------------------------
//  FmcStd_I2C_Bus::Tmp175::TToRaw() -- convert raw T register to degrees C
//---------------------------------------------------------------------------

short  FmcStd_I2C_Bus::Tmp175::TToRaw(float degreesC)
{
    short out = static_cast<short>(degreesC*256);
    return out;
}

//===========================================================================
//  CLASS FmcAFE_Common -- X6 AFE Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_Common
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcAFE_Common
//---------------------------------------------------------------------------

FmcAFE_Common::FmcAFE_Common(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TestControl(space, regs.AfeTestControl_Addr),
       SwTrigger(space, regs.AfeSoftwareTrigger_Addr),
       ExtSyncSelect(space, regs.AfeExtSyncSelect_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_Common
//---------------------------------------------------------------------------

FmcAFE_Common::~FmcAFE_Common()
{
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_Common::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_Common::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    AllOff();
    TestModeEnabled( tInputOutput, false, 0 );
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_Common::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    AllOff();
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::AllOff()
//---------------------------------------------------------------------------

void  FmcAFE_Common::AllOff()
{
    SoftwareTrigger( tInputOutput, false );
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::TestModeEnabled() --  Set Test Mode state
//---------------------------------------------------------------------------

void FmcAFE_Common::TestModeEnabled(AfeDirection which, bool state, unsigned int mode)
{
    switch (which)
        {
        case tInput:
            TestControl.AdcTestMode.Value(mode);
            TestControl.AdcTestEnable.Value(state);
            break;
        case tOutput:
            TestControl.DacTestMode.Value(mode);
            TestControl.DacTestEnable.Value(state);
            break;
        case tInputOutput:
            TestControl.AdcTestMode.Value(mode);
            TestControl.DacTestMode.Value(mode);
            TestControl.AdcTestEnable.Value(state);
            TestControl.DacTestEnable.Value(state);
            break;
        }
}


//---------------------------------------------------------------------------
//  FmcAFE_Common::SoftwareTrigger() --  Set SW Trigger state
//---------------------------------------------------------------------------

void FmcAFE_Common::SoftwareTrigger(AfeDirection which, bool state)
{
    switch (which)
        {
        case tInput:
            SwTrigger.AdcTrigger.Value(state);
            break;
        case tOutput:
            SwTrigger.DacTrigger.Value(state);
            break;
        case tInputOutput:
            SwTrigger.AdcTrigger.Value(state);
            SwTrigger.DacTrigger.Value(state);
            break;
        }

    SwTrigger.Apply();
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

void FmcAFE_Common::ExternalSyncSelection(AfeExtSyncOptions option)
{
    switch (option)
        {
        case essFrontPanel:
            ExtSyncSelect.ExtSyncOption.Value(0);
            break;
        case essFmc:
            ExtSyncSelect.ExtSyncOption.Value(1);
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcAFE_Common::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

FmcAFE_Common::AfeExtSyncOptions FmcAFE_Common::ExternalSyncSelection() const
{
    bool option =  ExtSyncSelect.ExtSyncOption.Value();
    if (option)
        return essFmc;
    else
        return essFrontPanel;
}

//===========================================================================
//  CLASS FmcAFE_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAFE_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcAFE_Clock
//---------------------------------------------------------------------------

FmcAFE_Clock::FmcAFE_Clock(IRequires_StdFmcClocks * mdc, IAddressingSpace & /*space*/, MapRegisters & /*regs*/)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAFE_Clock
//---------------------------------------------------------------------------

FmcAFE_Clock::~FmcAFE_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAFE_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
        
    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAFE_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    PllDev->Powered(true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAFE_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::Pll()
//---------------------------------------------------------------------------

Cdce62005 & FmcAFE_Clock::Pll()
{
    return *PllDev;
}

//===========================================================================
//  CLASS FmcAFE_Clock::FmcClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  FmcAFE_Clock::FmcClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  FmcAFE_Clock::FmcClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  FmcAFE_Clock::FmcClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcAFE_Clock::FmcClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcAFE_Clock::FmcClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcAFE_Clock::FmcClockIo::Dac()
{
    return Owner->PllDev->Dac();
}


//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IFmcClockIo::IIClockSource FmcAFE_Clock::FmcClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void FmcAFE_Clock::FmcClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IFmcClockIo::IIReferenceSource FmcAFE_Clock::FmcClockIo::Reference() const
{
    if (Owner->PllDev->ReferenceSource() == Cdce62005::refPrimary)
        return rsExternal;
    else if (Owner->PllDev->ReferenceSource() == Cdce62005::refSecondary)
        return rsCarrier;
    else
        return rsInternal;
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void FmcAFE_Clock::FmcClockIo::Reference(IIReferenceSource src)
{
    if (src == rsInternal)
        Owner->PllDev->ReferenceSource(Cdce62005::refAux);
    else if (src == rsExternal)
        Owner->PllDev->ReferenceSource(Cdce62005::refPrimary);
    else
        Owner->PllDev->ReferenceSource(Cdce62005::refSecondary);

}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void FmcAFE_Clock::FmcClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double FmcAFE_Clock::FmcClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  FmcAFE_Clock::FmcClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  FmcAFE_Clock::FmcClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS FmcLMK_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcLMK_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcLMK_Clock
//---------------------------------------------------------------------------

FmcLMK_Clock::FmcLMK_Clock(IRequires_LmkFmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       ExtClkSrcSelectReg(space, regs.AfeExtClkSourceSelect_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcLMK_Clock
//---------------------------------------------------------------------------

FmcLMK_Clock::~FmcLMK_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcLMK_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  FmcLMK_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    PllDev->Powered(true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  FmcLMK_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcLMK_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::Pll()
//---------------------------------------------------------------------------

Lmk04800 & FmcLMK_Clock::Pll()
{
    return *PllDev;
}

//===========================================================================
//  CLASS FmcLMK_Clock::FmcClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  FmcLMK_Clock::FmcClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  FmcLMK_Clock::FmcClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  FmcLMK_Clock::FmcClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcLMK_Clock::FmcClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcLMK_Clock::FmcClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcLMK_Clock::FmcClockIo::Dac()
{
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IFmcClockIo::IIClockSource FmcLMK_Clock::FmcClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void FmcLMK_Clock::FmcClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IFmcClockIo::IIReferenceSource FmcLMK_Clock::FmcClockIo::Reference() const
{
    if (Owner->PllDev->ReferenceSource() == Lmk04800::refPrimary)
        return rsInternal;
    else //if (Owner->PllDev->ReferenceSource() == Lmk04800::refSecondary)
        return rsExternal; //rsCarrier;
    //else
    //    return rsExternal;
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void FmcLMK_Clock::FmcClockIo::Reference(IIReferenceSource src)
{
    if (src == rsInternal)
        Owner->PllDev->ReferenceSource(Lmk04800::refPrimary);
    else // (src == rsExternal) or (src == rsCarrier)
        Owner->PllDev->ReferenceSource(Lmk04800::refSecondary);

}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void  FmcLMK_Clock::FmcClockIo::ExternalClkSelect(IIClockSelect src)
{
    if (src==cslFrontPanel)
        Owner->ExtClkSrcSelectReg.Bit(0,false);
    else
        Owner->ExtClkSrcSelectReg.Bit(0,true);
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

IFmcClockIo::IIClockSelect  FmcLMK_Clock::FmcClockIo::ExternalClkSelect() const
{
    if (Owner->ExtClkSrcSelectReg.Bit(0))
        return cslCarrier;
    else
        return cslFrontPanel;
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void FmcLMK_Clock::FmcClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double FmcLMK_Clock::FmcClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  FmcLMK_Clock::FmcClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  FmcLMK_Clock::FmcClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS FmcLMK20_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcLMK20_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcLMK20_Clock
//---------------------------------------------------------------------------

FmcLMK20_Clock::FmcLMK20_Clock(IRequires_Lmk20FmcClocks * mdc,  IAddressingSpace & /*space*/, MapRegisters & /*regs*/)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcLMK20_Clock
//---------------------------------------------------------------------------

FmcLMK20_Clock::~FmcLMK20_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcLMK20_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  FmcLMK20_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    PllDev->Powered(true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  FmcLMK20_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcLMK20_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::Pll()
//---------------------------------------------------------------------------

Lmk04820 & FmcLMK20_Clock::Pll()
{
    return *PllDev;
}

//===========================================================================
//  CLASS FmcLMK20_Clock::FmcClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  FmcLMK20_Clock::FmcClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  FmcLMK20_Clock::FmcClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  FmcLMK20_Clock::FmcClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcLMK20_Clock::FmcClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcLMK20_Clock::FmcClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcLMK20_Clock::FmcClockIo::Dac()
{
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IFmcClockIo::IIClockSource FmcLMK20_Clock::FmcClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void FmcLMK20_Clock::FmcClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IFmcClockIo::IIReferenceSource FmcLMK20_Clock::FmcClockIo::Reference() const
{
    Lmk04820::LmkReferenceSource current_source = Owner->PllDev->ReferenceSource();
    return static_cast<IFmcClockIo::IIReferenceSource>(Owner->PllDev->GetSourceMapping(current_source));
//    if (Owner->PllDev->ReferenceSource() == Lmk04820::refPrimary)
//        return rsInternal;
//    else if (Owner->PllDev->ReferenceSource() == Lmk04820::refSecondary)
//        return rsExternal;
//    else // (Owner->PllDev->ReferenceSource() == refCLKin2)
//        return rsCarrier;
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void FmcLMK20_Clock::FmcClockIo::Reference(IIReferenceSource src)
{
    Lmk04820::LmkReferenceSource new_source = static_cast<Lmk04820::LmkReferenceSource>(Owner->PllDev->GetSourceMapping_Reverse(src));
    if (Owner->PllDev->SourceInUse(new_source))
        Owner->PllDev->ReferenceSource(new_source);
//    if (src == rsInternal)
//        Owner->PllDev->ReferenceSource(Lmk04820::refPrimary);
//    else if (src == rsExternal)
//        Owner->PllDev->ReferenceSource(Lmk04820::refSecondary);
//    else // (src == rsCarrier)
//        Owner->PllDev->ReferenceSource(Lmk04820::refCLKin2);

}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void FmcLMK20_Clock::FmcClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double FmcLMK20_Clock::FmcClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  FmcLMK20_Clock::FmcClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  FmcLMK20_Clock::FmcClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS SbcDuoLMK20_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuoLMK20_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class SbcDuoLMK20_Clock
//---------------------------------------------------------------------------

SbcDuoLMK20_Clock::SbcDuoLMK20_Clock(IRequires_Lmk20FmcClocks * mdc,  IAddressingSpace & /*space*/, MapRegisters & /*regs*/)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuoLMK20_Clock
//---------------------------------------------------------------------------

SbcDuoLMK20_Clock::~SbcDuoLMK20_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  SbcDuoLMK20_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuoLMK20_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    PllDev->Powered(true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoLMK20_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoLMK20_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::Pll()
//---------------------------------------------------------------------------

Lmk04820 & SbcDuoLMK20_Clock::Pll()
{
    return *PllDev;
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::Pll()  - sub in a new PLL for different config.
//---------------------------------------------------------------------------

void SbcDuoLMK20_Clock::ReplacePll(Lmk04820* newPll)
{
    if (newPll==0)
        return;    // disallow sub-ing in a bad PLL.

    delete PllDev;
    PllDev = newPll;

}

//===========================================================================
//  CLASS SbcDuoLMK20_Clock::SbcDuoClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  SbcDuoLMK20_Clock::SbcDuoClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  SbcDuoLMK20_Clock::SbcDuoClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  SbcDuoLMK20_Clock::SbcDuoClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  SbcDuoLMK20_Clock::SbcDuoClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  SbcDuoLMK20_Clock::SbcDuoClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  SbcDuoLMK20_Clock::SbcDuoClockIo::Dac()
{
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

ISbcDuoClockIo::IIClockSource SbcDuoLMK20_Clock::SbcDuoClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void SbcDuoLMK20_Clock::SbcDuoClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

ISbcDuoClockIo::IIReferenceSource SbcDuoLMK20_Clock::SbcDuoClockIo::Reference() const
{
    if (Owner->PllDev->ReferenceSource() == Lmk04820::refPrimary)
        return rsExternalTrigger;
    else if (Owner->PllDev->ReferenceSource() == Lmk04820::refSecondary)
        return rsExternal;
    else // (Owner->PllDev->ReferenceSource() == refCLKin2)
        return rsInternal;
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void SbcDuoLMK20_Clock::SbcDuoClockIo::Reference(IIReferenceSource src)
{
    if (src == rsInternal)
        Owner->PllDev->ReferenceSource(Lmk04820::refCLKin2);
    else if (src == rsExternal)
        Owner->PllDev->ReferenceSource(Lmk04820::refSecondary);
    else // (src == rsExternalTrigger)
        Owner->PllDev->ReferenceSource(Lmk04820::refPrimary);

}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void SbcDuoLMK20_Clock::SbcDuoClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double SbcDuoLMK20_Clock::SbcDuoClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}


//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::LowLatencyTriggerMode() -- set trig config
//---------------------------------------------------------------------------

void SbcDuoLMK20_Clock::SbcDuoClockIo::LowLatencyTriggerMode(bool enabled)
{
    if (enabled)
        Owner->PllDev->TriggerSyncMode(Lmk04820::tsmLowLatency);
    else
        Owner->PllDev->TriggerSyncMode(Lmk04820::tsmLogic);
}

//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::LowLatencyTriggerMode() -- set trig config
//---------------------------------------------------------------------------

bool SbcDuoLMK20_Clock::SbcDuoClockIo::LowLatencyTriggerMode() const
{
    return (Owner->PllDev->TriggerSyncMode()==Lmk04820::tsmLowLatency) ? true : false;
}
//---------------------------------------------------------------------------
//  SbcDuoLMK20_Clock::SbcDuoClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  SbcDuoLMK20_Clock::SbcDuoClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS Fmc110_Cpld -- FMC CPLD "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc110_Cpld
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class Fmc110_Cpld
//---------------------------------------------------------------------------

Fmc110_Cpld::Fmc110_Cpld(IRequires_Nothing * mdc, IAddressingSpace & space,
                         FmcCpldSpiDevice::Info & regs)
    :  MDC(mdc), SpiDev(space, regs)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc110_Cpld
//---------------------------------------------------------------------------

Fmc110_Cpld::~Fmc110_Cpld()
{
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::ConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Reg0 = 0;
    SetSyncSrc(cpssClockTree);


}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::SetClkSrc()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::SetClkSrc(unsigned int value)
{
    if (!DeviceEnabled())
        return;

    //  write value to reg0
    PtrBitManipulator Reg(&Reg0);
    Reg.Field(0, 3, value);

    // Apply to hardware
    Spi(0, Reg.Value());
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::SetSyncSrc()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::SetSyncSrc(unsigned int value)
{
    if (!DeviceEnabled())
        return;

    //  write value to reg0
    PtrBitManipulator Reg(&Reg0);
    Reg.Field(3, 2, value);

    // Apply to hardware
    Spi(0, Reg.Value());
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::SetPllReset()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::SetPllReset(bool value)
{
    if (!DeviceEnabled())
        return;

    //  write value to reg0
    PtrBitManipulator Reg(&Reg0);
    Reg.Bit(5, value);

    // Apply to hardware
    Spi(0, Reg.Value());
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::SetDacReset()
//---------------------------------------------------------------------------

void  Fmc110_Cpld::SetDacReset(bool value)
{
    if (!DeviceEnabled())
        return;

    //  write value to reg0
    PtrBitManipulator Reg(&Reg0);
    Reg.Bit(6, value);

    // Apply to hardware
    Spi(0, Reg.Value());
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::GetPllLocked()
//---------------------------------------------------------------------------

bool  Fmc110_Cpld::GetPllLocked()
{
    if (!DeviceEnabled())
        return false;

    // Apply to hardware
    unsigned int Reg2 = Spi(2);

    //  write value to reg0
    PtrBitManipulator Reg(&Reg2);
    bool locked = Reg.Bit(1);
    return locked;
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  Fmc110_Cpld::Spi(unsigned int address, unsigned int data)
{
    SpiDev.Spi(address, data);
}

//---------------------------------------------------------------------------
//  Fmc110_Cpld::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  Fmc110_Cpld::Spi(unsigned int address)
{
    return SpiDev.Spi(address);
}

//===========================================================================
//  CLASS AtroposLMK20_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class AtroposLMK20_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class AtroposLMK20_Clock
//---------------------------------------------------------------------------

AtroposLMK20_Clock::AtroposLMK20_Clock(IRequires_Lmk20FmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       ExtClkSrcSelectReg(space, regs.ExtClkSourceSelect_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class AtroposLMK20_Clock
//---------------------------------------------------------------------------

AtroposLMK20_Clock::~AtroposLMK20_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  AtroposLMK20_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  AtroposLMK20_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    PllDev->Powered(true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  AtroposLMK20_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  AtroposLMK20_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::Pll()
//---------------------------------------------------------------------------

Lmk04820 & AtroposLMK20_Clock::Pll()
{
    return *PllDev;
}

//===========================================================================
//  CLASS AtroposLMK20_Clock::AtroposClockIo -- Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  AtroposLMK20_Clock::AtroposClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  AtroposLMK20_Clock::AtroposClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  AtroposLMK20_Clock::AtroposClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  AtroposLMK20_Clock::AtroposClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  AtroposLMK20_Clock::AtroposClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  AtroposLMK20_Clock::AtroposClockIo::Dac()
{
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IAtroposClockIo::IIClockSource AtroposLMK20_Clock::AtroposClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void AtroposLMK20_Clock::AtroposClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IAtroposClockIo::IIClockSelect AtroposLMK20_Clock::AtroposClockIo::ExternalClkSelect() const
{
    if (Owner->ExtClkSrcSelectReg.Bit(0))
        return cslFrontPanel;
    else
        return cslP16;
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::ExternalClkSelect() -- Select external clock source
//---------------------------------------------------------------------------

void AtroposLMK20_Clock::AtroposClockIo::ExternalClkSelect(IIClockSelect src)
{
    if (src==cslFrontPanel)
        Owner->ExtClkSrcSelectReg.Bit(0,false);
    else
        Owner->ExtClkSrcSelectReg.Bit(0,true);
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtrClockIo::AtroposClockIo() -- Retrieve reference source
//---------------------------------------------------------------------------

IAtroposClockIo::IIReferenceSource AtroposLMK20_Clock::AtroposClockIo::Reference() const
{
    if (Owner->PllDev->ReferenceSource() == Lmk04820::refPrimary)
        return rsP2S_Trigger;
    else if (Owner->PllDev->ReferenceSource() == Lmk04820::refSecondary)
        return rsExternal;
    else // (Owner->PllDev->ReferenceSource() == refCLKin2)
        return rsInternal;
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void AtroposLMK20_Clock::AtroposClockIo::Reference(IIReferenceSource src)
{
    if (src == rsInternal)
        Owner->PllDev->ReferenceSource(Lmk04820::refCLKin2);
    else if (src == rsExternal)
        Owner->PllDev->ReferenceSource(Lmk04820::refSecondary);
    else // (src == rsP2S_Trigger)
        Owner->PllDev->ReferenceSource(Lmk04820::refPrimary);

}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void AtroposLMK20_Clock::AtroposClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double AtroposLMK20_Clock::AtroposClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  AtroposLMK20_Clock::AtroposClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}

//---------------------------------------------------------------------------
//  AtroposLMK20_Clock::AtroposClockIo::TriggerSync() -- Query PLL TriggerSync result
//---------------------------------------------------------------------------

bool  AtroposLMK20_Clock::AtroposClockIo::TriggerSync() const
{
    return Owner->PllDev->OutputSyncStatus();
}


//===========================================================================
//  CLASS AtroposTriggerSystem  -- Atropos Trigger Manager
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class AtroposTriggerSystem
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class AtroposTriggerSystem
//---------------------------------------------------------------------------

AtroposTriggerSystem::AtroposTriggerSystem(IRequires_FrameCalcDevice * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       ExtSyncSelectReg(space, regs.ExtSyncSelect_Addr),
       SwTriggerReg(space, regs.SoftwareTrigger_Addr),
       SerDesControlReg(space, regs.SerDesControl_Addr),
       TriggerCfgReg(space, regs.TriggerConfig_Addr),
       DecimationReg(space, regs.TriggerDecimation_Addr),
       TriggerCfgImpl(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class AtroposTriggerSystem
//---------------------------------------------------------------------------

AtroposTriggerSystem::~AtroposTriggerSystem()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::ConstructCustomParts()
//---------------------------------------------------------------------------

void  AtroposTriggerSystem::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    Dev = MDC->MakeFrameCalcDevice(IRequires_FrameCalcDevice::devAdc);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::OpenHardware()
//---------------------------------------------------------------------------

void  AtroposTriggerSystem::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    MasterTrigger( false );
    SoftwareTrigger( false );
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::PreconfigureHardware()
//---------------------------------------------------------------------------

void  AtroposTriggerSystem::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::ConfigureHardware()
//---------------------------------------------------------------------------

void  AtroposTriggerSystem::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    SoftwareTrigger( false );
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::SoftwareTrigger() --  Set SW Trigger state
//---------------------------------------------------------------------------

void AtroposTriggerSystem::SoftwareTrigger(bool state)
{
    SwTriggerReg.SwTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::MasterTrigger() --  Set Master Trigger state
//---------------------------------------------------------------------------

void AtroposTriggerSystem::MasterTrigger(bool state)
{
    SwTriggerReg.MasterTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

void AtroposTriggerSystem::ExternalSyncSelection(TriggerCfg::ExtSyncOptions option)
{
    switch (option)
        {
        case TriggerCfg::essFrontPanel:
            ExtSyncSelectReg.ExtSyncOption.Value(0);
            break;
        case TriggerCfg::essP16:
            ExtSyncSelectReg.ExtSyncOption.Value(1);
            break;
        }
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

AtroposTriggerSystem::TriggerCfg::ExtSyncOptions AtroposTriggerSystem::ExternalSyncSelection() const
{
    bool option =  ExtSyncSelectReg.ExtSyncOption.Value();
    if (option)
        return TriggerCfg::essP16;
    else
        return TriggerCfg::essFrontPanel;
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void AtroposTriggerSystem::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    TriggerCfgReg.FrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int AtroposTriggerSystem::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = TriggerCfgReg.FrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void AtroposTriggerSystem::EdgeTrigger(bool state)
{
    TriggerCfgReg.RisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool AtroposTriggerSystem::EdgeTrigger() const
{
    return TriggerCfgReg.RisingEdge.Value();
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void AtroposTriggerSystem::ExternalTrigger(bool state)
{
    TriggerCfgReg.ExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool AtroposTriggerSystem::ExternalTrigger() const
{
    return TriggerCfgReg.ExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void AtroposTriggerSystem::FramedTriggerEnabled(bool state)
{
    TriggerCfgReg.FramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool AtroposTriggerSystem::FramedTriggerEnabled() const
{
    return TriggerCfgReg.FramedMode.Value();
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::TriggerDecimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void AtroposTriggerSystem::TriggerDecimation(unsigned int samples)
{
    DecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  AtroposTriggerSystem::TriggerDecimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int AtroposTriggerSystem::TriggerDecimation() const
{
    return DecimationReg.Decimation.Value();
}

//===========================================================================
//  CLASS SbcDuoTriggerSystem  -- Atropos Trigger Manager
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuoTriggerSystem
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class SbcDuoTriggerSystem
//---------------------------------------------------------------------------

SbcDuoTriggerSystem::SbcDuoTriggerSystem(IRequires_FrameCalcDevice * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       //ExtSyncSelectReg(space, regs.ExtSyncSelect_Addr),
       SwTriggerReg(space, regs.SoftwareTrigger_Addr),
       TriggerCfgReg(space, regs.TriggerConfig_Addr),
       DecimationReg(space, regs.TriggerDecimation_Addr),
       TriggerCfgImpl(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuoTriggerSystem
//---------------------------------------------------------------------------

SbcDuoTriggerSystem::~SbcDuoTriggerSystem()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::SbcDuoTriggerSystem()
//---------------------------------------------------------------------------

void  SbcDuoTriggerSystem::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    Dev = MDC->MakeFrameCalcDevice(IRequires_FrameCalcDevice::devAdc);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuoTriggerSystem::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    SoftwareTrigger( false );
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::PreconfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoTriggerSystem::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoTriggerSystem::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    SoftwareTrigger( false );
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::SoftwareTrigger() --  Set SW Trigger state
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::SoftwareTrigger(bool state)
{
    SwTriggerReg.SwTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::MasterTrigger() --  Set Master Trigger state
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::MasterTrigger(bool state)
{
    SwTriggerReg.MasterEnable.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::ArmPpsTrigger() --  Arm trigger on next PPS
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::ArmPpsTrigger()
{
    SwTriggerReg.ArmPpsTrigger.Value(true);
    SwTriggerReg.ArmPpsTrigger.Value(false);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

//void SbcDuoTriggerSystem::ExternalSyncSelection(TriggerCfg::ExtSyncOptions option)
//{
//    switch (option)
//        {
//        case TriggerCfg::essFrontPanel:
//            ExtSyncSelectReg.ExtSyncOption.Value(0);
//            break;
//        case TriggerCfg::essP16:
//            ExtSyncSelectReg.ExtSyncOption.Value(1);
//            break;
//        }
//}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

//SbcDuoTriggerSystem::TriggerCfg::ExtSyncOptions AtroposTriggerSystem::ExternalSyncSelection() const
//{
//    bool option =  ExtSyncSelectReg.ExtSyncOption.Value();
//    if (option)
//        return TriggerCfg::essP16;
//    else
//        return TriggerCfg::essFrontPanel;
//}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    TriggerCfgReg.FrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int SbcDuoTriggerSystem::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = TriggerCfgReg.FrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::EdgeTrigger(bool state)
{
    TriggerCfgReg.RisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool SbcDuoTriggerSystem::EdgeTrigger() const
{
    return TriggerCfgReg.RisingEdge.Value();
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::ExternalTrigger(bool state)
{
    TriggerCfgReg.ExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool SbcDuoTriggerSystem::ExternalTrigger() const
{
    return TriggerCfgReg.ExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::FramedTriggerEnabled(bool state)
{
    TriggerCfgReg.FramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool SbcDuoTriggerSystem::FramedTriggerEnabled() const
{
    return TriggerCfgReg.FramedMode.Value();
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::TriggerDecimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void SbcDuoTriggerSystem::TriggerDecimation(unsigned int samples)
{
    DecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  SbcDuoTriggerSystem::TriggerDecimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int SbcDuoTriggerSystem::TriggerDecimation() const
{
    return DecimationReg.Decimation.Value();
}

//=============================================================================
//  CLASS FmcCalibrationIDROMImage
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcCalibrationIDROMImage
//---------------------------------------------------------------------------

const int FmcRomImageReserveSize = 1;

//---------------------------------------------------------------------------
//  constructor for class FmcCalibrationIDROMImage
//---------------------------------------------------------------------------

FmcCalibrationIDROMImage::FmcCalibrationIDROMImage(FmcStandardRomInfo & romInfo, bool isAdcImage, unsigned int idrom_data_size_words)
    :  GeneralIdromImage(idrom_data_size_words+FmcRomImageReserveSize, FmcRomImageReserveSize),  IsAdcImage(isAdcImage),
       RomInfo(romInfo)
{
}


//---------------------------------------------------------------------------
//  FmcCalibrationIDROMImage::RefreshLocalImage() -
//---------------------------------------------------------------------------
//  Refresh local ROM image from std ROM image (called at start)

void  FmcCalibrationIDROMImage::RefreshLocalImage()
{
    ToLocalImage(0, &Packet[0], IdromSizeWords);
}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROMImage::ToLocalImage() - move from FMC rom to our Image
//---------------------------------------------------------------------------

void  FmcCalibrationIDROMImage::ToLocalImage(int /*offset*/, int * data, int words)
{
    FmcStandardRomInfo::CalCoefficientsMR & cal = WhichCal();
    if (cal.Raw().size() < static_cast<size_t>(words))
        cal.Resize(words);
    //
    //  Copy Data from AdcCal to local cache
    for (int i=0; i<words; i++)
        data[i] = cal.AsInt(i);
}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROMImage::ROM_ReadBlock() - read from ROM device
//---------------------------------------------------------------------------

void  FmcCalibrationIDROMImage::ROM_ReadBlock(int offset, int * data, int words)
{
    //
    //  Read From Rom
    RomInfo.LoadFromRom();

    ToLocalImage(offset, data, words);

}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROMImage::FromLocalImage() - move from our Image to FMC rom
//---------------------------------------------------------------------------

void  FmcCalibrationIDROMImage::FromLocalImage(int /*offset*/, const int * data, int words)
{
    FmcStandardRomInfo::CalCoefficientsMR & cal = WhichCal();
    if (cal.Raw().size() < static_cast<size_t>(words))
        cal.Resize(words);
    //
    //  Copy Data to AdcCal from local cache
    for (int i=0; i<words; i++)
        cal.AsInt(i, data[i]);
}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROMImage::ROM_WriteBlock() - write to ROM device
//---------------------------------------------------------------------------

void  FmcCalibrationIDROMImage::ROM_WriteBlock(int offset, const int * data, int words)
{
    FromLocalImage(offset, data, words);
    //
    //  Write To Rom
    RomInfo.StoreToRom();

}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROMImage::WhichCal() - select Cal Record
//---------------------------------------------------------------------------

FmcStandardRomInfo::CalCoefficientsMR &  FmcCalibrationIDROMImage::WhichCal()
{
    if (IsAdcImage)
        return RomInfo.AdcCal();
    else
        return RomInfo.DacCal();
}


//===========================================================================
//  CLASS FmcCalibrationIDROM_Manager -- FMC device information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcCalibrationIDROM_Manager
//---------------------------------------------------------------------------

const int FmcRomDefaultDataSize = 8;

//---------------------------------------------------------------------------
//  constructor for class FmcCalibrationIDROM_Manager
//---------------------------------------------------------------------------

FmcCalibrationIDROM_Manager::FmcCalibrationIDROM_Manager(IRequires_FmcCalIDROM * mdc, FmcStandardRomInfo & romInfo, bool needsAdcImage)
    :  MDC(mdc),
       IDROMImage(romInfo, needsAdcImage, FmcRomDefaultDataSize),
       NeedsAdcImage(needsAdcImage)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcCalibrationIDROM_Manager
//---------------------------------------------------------------------------

FmcCalibrationIDROM_Manager::~FmcCalibrationIDROM_Manager()
{
}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROM_Manager::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcCalibrationIDROM_Manager::ConstructCustomParts()
{
    Debug.Log("FmcCalibrationIDROM_Manager::ConstructCustomParts()");
    if (!DeviceEnabled())
        return;

    IRequires_FmcCalIDROM::FmcCalIdromInfo info = MDC->GetFmcCalIdromInfo(NeedsAdcImage ? IRequires_FmcCalIDROM::fmcCalAdc
                                                                                        : IRequires_FmcCalIDROM::fmcCalDac );
    IDROMImage.Reconfigure(info.IdromSize, info.ReservedSize);
}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROM_Manager::OpenHardware()
//---------------------------------------------------------------------------

void  FmcCalibrationIDROM_Manager::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    if (NeedsAdcImage)
        Debug.Log("Refresh ADC Local Cal Rom Image");
    else
        Debug.Log("Refresh DAC Local Cal Rom Image");

    IDROMImage.RefreshLocalImage();
}

//---------------------------------------------------------------------------
//  FmcCalibrationIDROM_Manager::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcCalibrationIDROM_Manager::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//=============================================================================
//  CLASS FmcCalibrationIDROMImage
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcRangedCalibrationIDROMImage
//---------------------------------------------------------------------------

const int FmcRangedRomImageReserveSize = 1;

//---------------------------------------------------------------------------
//  constructor for class FmcRangedCalibrationIDROMImage
//---------------------------------------------------------------------------

FmcRangedCalibrationIDROMImage::FmcRangedCalibrationIDROMImage(FmcStandardRomInfo & romInfo, unsigned int idrom_data_size_words)
    :  GeneralIdromImage(idrom_data_size_words+FmcRangedRomImageReserveSize, FmcRangedRomImageReserveSize),
       RomInfo(romInfo)
{
}


//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROMImage::RefreshLocalImage() -
//---------------------------------------------------------------------------
//  Refresh local ROM image from std ROM image (called at start)

void  FmcRangedCalibrationIDROMImage::RefreshLocalImage()
{
    ToLocalImage(0, &Packet[0], IdromSizeWords);
}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROMImage::ToLocalImage() - move from FMC rom to our Image
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROMImage::ToLocalImage(int /*offset*/, int * data, int words)
{
    Trace  Debug;

    unsigned int ranges = static_cast<unsigned int>(RomInfo.MultiR().AdcRanges());
    if (!ranges)
        return;

    // we will be copying one word (the calibrated one) to a reserved area so deduct its size
    if (static_cast<size_t>(words) < ranges*(RomInfo.MultiR().AdcRangedCal(0).Raw().size()-1))
        {
        Debug.Log("FmcRangedCalibrationIDROMImage::ToLocalImage() - can't copy Cal, ROM image to large");
        //Debug.Log("NO COPY: words = ", words);
        //Debug.Log("NO COPY: ROM image size = ", RomInfo.MultiR().AdcRangedCal(0).Raw().size());
        return;    // don't copy if IDROM is too small
        }

    for (unsigned int i=0; i<ranges; i++)
        {
        FmcStandardRomInfo::AdcRangedCalCoefficientsMR & cal =  RomInfo.MultiR().AdcRangedCal(i);
        unsigned int RangeSize = words/ranges-1;
        RangeSize >>= 1;  RangeSize <<=1;   // make even
        //
        //  Copy Data from AdcCal to local cache
        unsigned int RangeOfs = i*RangeSize;
        for (unsigned int ii=0; ii<RangeSize; ii++)
            if (ii<cal.Raw().size())
                data[RangeOfs+ii] = cal.AsInt(ii);
        //
        //  Copy Calibrated Bit to Reserved Area
		Reserved(i, cal.AsInt(static_cast<unsigned int>(cal.Raw().size() - 1)));
        }

}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROMImage::ROM_ReadBlock() - read from ROM device
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROMImage::ROM_ReadBlock(int offset, int * data, int words)
{
    //
    //  Read From Rom
    RomInfo.LoadFromRom();

    ToLocalImage(offset, data, words);

}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROMImage::FromLocalImage() - move from our Image to FMC rom
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROMImage::FromLocalImage(int /*offset*/, const int * data, int words)
{
    //Trace  Debug;

    for (unsigned int i=0; i<RomInfo.MultiR().AdcRanges(); i++)
        {
        FmcStandardRomInfo::AdcRangedCalCoefficientsMR & cal =  RomInfo.MultiR().AdcRangedCal(i);

        //Debug.Log("FromLocalImage words: ", words);
        //Debug.Log("FromLocalImage Ranges: " , static_cast<unsigned int>(RomInfo.MultiR().AdcRanges()));

        unsigned int RangeSize = words/static_cast<unsigned int>(RomInfo.MultiR().AdcRanges());
        //Debug.Log("FromLocalImage RangeSize: ", RangeSize);

        cal.Resize(RangeSize+1); // one for calibrated bit
        RangeSize >>= 1;  RangeSize <<=1;   // make even
        //
        //  Copy Data to AdcCal from local cache
		unsigned int RangeOfs = i*RangeSize;
        for (unsigned int ii=0; ii<RangeSize; ii++)
            cal.AsInt(ii, data[RangeOfs+ii]);
        //
        //  Copy Calibrated Bit to Reserved Area
        cal.AsInt(static_cast<unsigned int>(cal.Raw().size())-1, Reserved(i) );
        }

}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROMImage::ROM_WriteBlock() - write to ROM device
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROMImage::ROM_WriteBlock(int offset, const int * data, int words)
{
    FromLocalImage(offset, data, words);
    //
    //  Write To Rom
    RomInfo.StoreToRom();

}

//===========================================================================
//  CLASS FmcRangedCalibrationIDROM_Manager -- FMC device information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcRangedCalibrationIDROM_Manager
//---------------------------------------------------------------------------

const int FmcRangedRomDefaultDataSize = 8;

//---------------------------------------------------------------------------
//  constructor for class FmcRangedCalibrationIDROM_Manager
//---------------------------------------------------------------------------

FmcRangedCalibrationIDROM_Manager::FmcRangedCalibrationIDROM_Manager(IRequires_FmcCalIDROM * mdc, FmcStandardRomInfo & romInfo)
    :  MDC(mdc),
       IDROMImage(romInfo, FmcRangedRomDefaultDataSize)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcRangedCalibrationIDROM_Manager
//---------------------------------------------------------------------------

FmcRangedCalibrationIDROM_Manager::~FmcRangedCalibrationIDROM_Manager()
{
}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROM_Manager::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROM_Manager::ConstructCustomParts()
{
    Debug.Log("FmcCalibrationIDROM_Manager::ConstructCustomParts()");
    if (!DeviceEnabled())
        return;

    IRequires_FmcCalIDROM::FmcCalIdromInfo info = MDC->GetFmcCalIdromInfo(IRequires_FmcCalIDROM::fmcCalAdc );
    IDROMImage.Reconfigure(info.IdromSize, info.ReservedSize);
}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROM_Manager::OpenHardware()
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROM_Manager::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Debug.Log("Refresh ADC Local Cal Rom Image");

    IDROMImage.RefreshLocalImage();
}

//---------------------------------------------------------------------------
//  FmcRangedCalibrationIDROM_Manager::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcRangedCalibrationIDROM_Manager::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//===========================================================================
//  CLASS FmcAd9510Duo_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAd9510Duo_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class FmcAd9510Duo_Clock
//---------------------------------------------------------------------------

FmcAd9510Duo_Clock::FmcAd9510Duo_Clock(IRequires_Ad9510TbClocks * mdc,  IAddressingSpace & /*space*/, MapRegisters & /*regs*/)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       ClockIo(this), ClockDevs(this)
{
    PllDev.AutoReapply(false);  // don't reapply automatically
}

//---------------------------------------------------------------------------
//  destructor for class FmcAd9510Duo_Clock
//---------------------------------------------------------------------------

FmcAd9510Duo_Clock::~FmcAd9510Duo_Clock()
{
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAd9510Duo_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    Ad9510_Tb * master = MDC->MakeMasterPllObject();
    PllDev.LoadMasterPll(master);

    Ad9508_Tb * slave = MDC->MakeSlavePllObject();
    PllDev.LoadSlavePll(slave);
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAd9510Duo_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    //PllDev.Powered(true);
    PllDev.Init();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  FmcAd9510Duo_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev.Reapply();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAd9510Duo_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::Pll()
//---------------------------------------------------------------------------

Ad9510Duo_Tb & FmcAd9510Duo_Clock::Pll()
{
    return PllDev;
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::CustomExternalClkSelect()
//---------------------------------------------------------------------------

void  FmcAd9510Duo_Clock::CustomExternalClkSelect(IX6ClockIo::IIClockSelect src)
{
    Ad9510_Tb::AdExternalSelect recoded_src;
    switch (src)
        {
        default:
        case IX6ClockIo::cslFrontPanel:
            recoded_src = Ad9510_Tb::csFrontPanel;
            break;
        case IX6ClockIo::cslCarrier:
            recoded_src = Ad9510_Tb::csCarrier;
            break;
        }
    PllDev.ExternalClockSelect(recoded_src);
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::CustomExternalClkSelect()
//---------------------------------------------------------------------------

IX6ClockIo::IIClockSelect  FmcAd9510Duo_Clock::CustomExternalClkSelect() const
{
    switch (PllDev.ExternalClockSelect())
        {
        default:
        case Ad9510_Tb::csFrontPanel:
            return IX6ClockIo::cslFrontPanel;
        case Ad9510_Tb::csCarrier:
            return IX6ClockIo::cslCarrier;
        }

}

//===========================================================================
//  CLASS FmcAd9510Duo_Clock::AdClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  FmcAd9510Duo_Clock::AdClockIo::FrequencyActual() const
{
    return Owner->PllDev.FrequencyActual();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  FmcAd9510Duo_Clock::AdClockIo::Frequency(double value)
{
    Owner->PllDev.Frequency(value);
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  FmcAd9510Duo_Clock::AdClockIo::Frequency() const
{
    return Owner->PllDev.Frequency();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcAd9510Duo_Clock::AdClockIo::System()
{
    return Owner->PllDev.System();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcAd9510Duo_Clock::AdClockIo::Adc()
{
    return Owner->PllDev.Adc();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  FmcAd9510Duo_Clock::AdClockIo::Dac()
{
    return Owner->PllDev.Dac();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IX6ClockIo::IIClockSource FmcAd9510Duo_Clock::AdClockIo::Source() const
{
    if (Owner->PllDev.IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void FmcAd9510Duo_Clock::AdClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev.Pll_Mode();
    else
        Owner->PllDev.ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IX6ClockIo::IIReferenceSource FmcAd9510Duo_Clock::AdClockIo::Reference() const
{
    Ad9510_Tb::AdReferenceSource current_source = Owner->PllDev.ReferenceSource();
    return static_cast<AdClockIo::IIReferenceSource>(Owner->PllDev.GetSourceMapping(current_source));
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void FmcAd9510Duo_Clock::AdClockIo::Reference(IIReferenceSource src)
{
    Ad9510_Tb::AdReferenceSource new_source = static_cast<Ad9510_Tb::AdReferenceSource>(Owner->PllDev.GetSourceMapping_Reverse(src));
    if (Owner->PllDev.SourceInUse(new_source))
        Owner->PllDev.ReferenceSource(new_source);

}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void FmcAd9510Duo_Clock::AdClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev.Reference(value);
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double FmcAd9510Duo_Clock::AdClockIo::ReferenceFrequency() const
{
    return Owner->PllDev.Reference();
}

//---------------------------------------------------------------------------
//  FmcAd9510Duo_Clock::AdClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  FmcAd9510Duo_Clock::AdClockIo::Locked() const
{
    return Owner->PllDev.Locked();
}

//===========================================================================
//  CLASS PolledDataFifo  -- Atropos Trigger Manager
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PolledDataFifo
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class PolledDataFifo
//---------------------------------------------------------------------------

PolledDataFifo::PolledDataFifo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs,
                               unsigned int fifo_depth)
    :  MDC(mdc), FFifoDepth(fifo_depth),
       FifoControlReg(space, regs.FifoControl_Addr),
       FifoThresholdReg(space, regs.FifoThreshold_Addr),
       FifoDataReg(space, regs.FifoData_Addr),
       DacDelayReg(space, regs.DacDelay_Addr),
       PolledFifoImpl(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class PolledDataFifo
//---------------------------------------------------------------------------

PolledDataFifo::~PolledDataFifo()
{
}

//---------------------------------------------------------------------------
//  PolledDataFifo::SbcDuoTriggerSystem()
//---------------------------------------------------------------------------

void  PolledDataFifo::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  PolledDataFifo::OpenHardware()
//---------------------------------------------------------------------------

void  PolledDataFifo::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  PolledDataFifo::PreconfigureHardware()
//---------------------------------------------------------------------------

void  PolledDataFifo::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  PolledDataFifo::ConfigureHardware()
//---------------------------------------------------------------------------

void  PolledDataFifo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  PolledDataFifo::LoadFullDelay()
//---------------------------------------------------------------------------
//  decimation register value is used rather than a factor

void PolledDataFifo::LoadFullDelay(unsigned int total_delay_us, double sample_freq_hz, unsigned int decimation_reg_value)
{
    // sanity check
    if (sample_freq_hz==0.0)
        return;

    unsigned int decimation = decimation_reg_value+1 ;

    const unsigned int MaxInPeriodDelay = 1<<10;      // width of field
    const unsigned int MaxSampleClockDelay = 1<<14;   // width of field
    double TotalDelay_us = static_cast<double>(total_delay_us);
    const double  Eps = 1.000001;       // Guard against truncation errors.

    // We can calculate the period using the "long period" only if the value entered in the uS field is in range
    double SamplePeriod_us = 1.0e6 / sample_freq_hz;
    double LongPeriod_us = SamplePeriod_us * decimation;

    unsigned int DelaySamples;
    unsigned int Delay_uS;

    if (LongPeriod_us < static_cast<double>(MaxInPeriodDelay))
        {
        // Use long period, as uS values will fit:
        double DelayPeriods = TotalDelay_us / LongPeriod_us;
        unsigned int  LongPeriods = static_cast<unsigned int>(IntegerPart<double>(DelayPeriods*Eps));

        DelaySamples = LongPeriods * decimation;

        // Convert remainder back to uS:
        double FractionalPeriod =  DelayPeriods - LongPeriods;
        // Adding 1 and truncating => ceil, ensuring required minimum delay is met:
        Delay_uS = static_cast<unsigned int>((FractionalPeriod+ Eps-1.0)*LongPeriod_us + 1.0);
        if (Delay_uS >= static_cast<unsigned int>(Eps*LongPeriod_us))
            Delay_uS--;
        }
    else
        {
        // Use short period to allow uS values to fill the range:
        double DelayPeriods = TotalDelay_us / SamplePeriod_us;
        DelaySamples = static_cast<unsigned int>(IntegerPart<double>(DelayPeriods*Eps));

        // Convert remainder back to uS:
        double FractionalPeriod = DelayPeriods - DelaySamples;
        // Adding 1 and truncating => ceil, ensuring required minimum delay is met:
        Delay_uS = static_cast<unsigned int>((FractionalPeriod+ Eps-1.0)*SamplePeriod_us + 1.0);
        if (Delay_uS >= static_cast<unsigned int>(Eps*SamplePeriod_us))
            Delay_uS--;
        }

    // Do the best possible:
    if (Delay_uS >= MaxInPeriodDelay)
        Delay_uS = MaxInPeriodDelay-1;
    if (DelaySamples >= MaxSampleClockDelay)
        DelaySamples = MaxSampleClockDelay-1;

    // Load the raw registers:
    Fifo().DelayInSamples(DelaySamples);
    Fifo().Delay(Delay_uS);

}



#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative

