// FmcSubDevice_Stream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FmcSubDevice_Stream_MbH
#define FmcSubDevice_Stream_MbH

#include <X6Support_Mb.h>
#include <FmcSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <CDCE62005_Mb.h>
#include <LMK04800_Mb.h>
#include <LMK04820_Mb.h>
#include <PmcIdrom_Mb.h>
#include <SpiDevice_Mb.h>
#include <Ad9510Duo_Tb_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS FmcInformation -- FMC device information "SubDevice"
//===========================================================================

class FmcInformation : public IX6SubDevice, public IFmcInformation
{
public:
    //=======================================================================
    //  CLASS FmcInformation::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   FmcControlStatus_Addr;
        unsigned int   FmcId_Addr;
        unsigned int   FmcBidir_Addr;
        unsigned int   FmcClk0_M2C_Status_Addr;
        unsigned int   FmcClk1_M2C_Status_Addr;
        unsigned int   FmcClk2_M2C_Status_Addr;
        unsigned int   FmcClk3_M2C_Status_Addr;
        unsigned int   FmcClk2_C2M_Status_Addr;
        unsigned int   FmcClk3_C2M_Status_Addr;
    };

    //=======================================================================
    //  CLASS FmcInformation::Pex7CopModeMapRegisters  -- Device Register Offsets
    //=======================================================================
    class Pex7CopModeMapRegisters
    {
    public:
        unsigned int   Pex7_VAdj_Status_Addr;
        unsigned int   Pex7_C2M_Clock_Ctrl_Addr;
    };

    FmcInformation(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs,
                                            IAddressingSpace & P7_space, Pex7CopModeMapRegisters & P7_regs);

    virtual ~FmcInformation();

    bool  Pex7CarrierPowerMode()
        {  return IsPex7Style;  }
    void  SetPex7Cop_PowerMode()
        {  IsPex7Style = true;  }

    //  Methods
    virtual bool         FmcPresent();
    virtual bool         FmcVadjEnable();
    virtual void         FmcVadjEnable(bool enable);
    virtual bool         FmcVadjForced();
    virtual bool         FmcVadjPower();
    //virtual void         FmcVadjLevel(short level);
    virtual short        FmcVadjLevel();
    virtual bool         FmcM2CPower();
    virtual bool         FmcC2MPower();

    short                Pex7_FmcVadjLevel();

    virtual unsigned int FmcBoardId();

    virtual bool         FmcBidirClockC2M();
    virtual void         FmcClock2_3_Enable(bool state);
    virtual bool         FmcClock2_3_Enable();

    virtual void         FmcClock2_IsSingleEnded(bool state);
    virtual bool         FmcClock2_IsSingleEnded();

    virtual void         FmcClock3_IsSingleEnded(bool state);
    virtual bool         FmcClock3_IsSingleEnded();

    virtual void         Pex7_FmcClock2_IsFpga(bool state);
    virtual bool         Pex7_FmcClock2_IsFpga();
    virtual void         Pex7_FmcClock3_IsFpga(bool state);
    virtual bool         Pex7_FmcClock3_IsFpga();

    virtual void         FmcC2MPower(bool enable);

    unsigned int         Fmc_Clk_M2C_Rate(int clock);
    unsigned int         Fmc_Clk_C2M_Rate(int clock);

    void  WaitForVadjPowerGood();

    void  WaitForPowerGood();

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcInformation::ControlStatusRegister  -- FMC Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ControlStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       PresentN;
        RegisterBit       VADJ_Disable;
        RegisterBit       VADJ_Forced;
        RegisterBit       VADJ_PowerGood;
        RegisterBitGroup  VADJ_Level;
        RegisterBit       M2CPowerGoodN;
        RegisterBit       C2MPowerGoodN;

    public:
        ControlStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               PresentN(*this, 0),   VADJ_Disable(*this, 1),
               VADJ_Forced(*this, 2),   VADJ_PowerGood(*this, 3),
               VADJ_Level(*this, 4, 3),
               M2CPowerGoodN(*this, 16), C2MPowerGoodN(*this, 17)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcInformation::BidirControlRegister  -- Bidir Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class BidirControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Bidir_C2M;
        RegisterBit       Fmc_2_3_Enable;
        RegisterBit       Fmc_2_SingleEnded;
        RegisterBit       Fmc_3_SingleEnded;

    public:
        BidirControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Bidir_C2M(*this, 0),   Fmc_2_3_Enable(*this, 1),
               Fmc_2_SingleEnded(*this, 2),   Fmc_3_SingleEnded(*this, 3)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcInformation::FmcClockStatusRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class FmcClockStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup   Freq;

    public:
        FmcClockStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Freq(*this, 0, 10)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcInformation::P7Cop_VAdjStatusRegister  -- P7 Specific Regs
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class P7Cop_VAdjStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  VADJ_Level;

    public:
        P7Cop_VAdjStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               VADJ_Level(*this, 0, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcInformation::P7Cop_FmcClockCtrlRegister  -- P7 Specific Regs
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class P7Cop_FmcClockCtrlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  C2M_2_SourceIsFpga;  // else carrier
        RegisterBit  C2M_3_SourceIsFpga;

    public:
        P7Cop_FmcClockCtrlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               C2M_2_SourceIsFpga(*this, 0),
               C2M_3_SourceIsFpga(*this, 1)
            {
            }
    };

private:
	IRequires_Nothing *     MDC;
public:
    ControlStatusRegister   ControlStatusReg;
    Register                BoardIdReg;
    BidirControlRegister    BidirControlReg;
    FmcClockStatusRegister  Clk0_M2C_Reg;
    FmcClockStatusRegister  Clk1_M2C_Reg;
    FmcClockStatusRegister  Clk2_M2C_Reg;
    FmcClockStatusRegister  Clk3_M2C_Reg;
    FmcClockStatusRegister  Clk2_C2M_Reg;
    FmcClockStatusRegister  Clk3_C2M_Reg;

    P7Cop_VAdjStatusRegister    P7Cop_VAdjReg;
    P7Cop_FmcClockCtrlRegister  P7Cop_ClkCtrlReg;

    bool                    IsPex7Style;

};

//===========================================================================
//  CLASS FmcStd_I2C_Bus -- Standard FMC I2C Bus "SubDevice"
//===========================================================================

class FmcStd_I2C_Bus : public IX6SubDevice//,  public IX6_I2C_uDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStd_I2C_Bus::I2C_SubAddr  -- Device SubAddr on I2C Bus
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class I2C_SubAddr
    {
    public:
        unsigned int   TempControl_SubAddr;
        unsigned int   FmcIdrom_SubAddr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStd_I2C_Bus::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   I2CControl_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStd_I2C_Bus::Tmp175  -- Temperature Controller
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Tmp175
    {
    public:
        Tmp175(FmcStd_I2C_Bus * owner): Owner(owner)
            {  }
        virtual ~Tmp175()
            {  }

        float  GetT();
        float  GetHighT();
        float  GetLowT();

        enum Resolution { res9bits, res10bits, res11bits, res12bits };
        enum FaultQueue { fq1Deep, fq2Deep, fq4Deep, fq6Deep };

        void         SetConfig(unsigned int bits);
        void         SetConfig(bool one_shot, unsigned int resolution, unsigned int faultQ,
                               bool alertActiveHigh, bool interruptMode, bool shutdownMode  );
        unsigned int GetConfig();

        unsigned int GetDeviceId();
        unsigned int GetFullDevId();

        void  SetHighT(float degreesC);
        void  SetLowT(float degreesC);


    protected:
        FmcStd_I2C_Bus *  Owner;

        float RawToT(short raw);
        short TToRaw(float degreesC);
    };

    friend class Tmp175;
    
    FmcStd_I2C_Bus(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs, I2C_SubAddr & subaddrs );
    virtual ~FmcStd_I2C_Bus();

    Tmp175 &       Temp() {  return TempControl;  }

    FmcStandardRomInfo &  RomInfo()  {  return FmcRomInfo;  }
    AdvI2CIdrom &         FmcIdrom() {  return FmcIdromDev;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    //virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStd_I2C_Bus::BusI2CRegister  -- 'Fmc Bus' I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BusI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Data_Out;
        RegisterBit  Clock_Out;
        RegisterBit  Data_In;
        RegisterBit  Clock_In;

    public:
        BusI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data_Out(*this, 0),   Clock_Out(*this, 1),
               Data_In(*this, 2),    Clock_In(*this, 3)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcBusI2C_Impl  -- FmcBus I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcBusI2C_Impl : public I2CDeviceIntf
    {
    public:
        FmcBusI2C_Impl(BusI2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~FmcBusI2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        BusI2CRegister &  Reg;
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    BusI2CRegister       FmcBusI2CReg;
    //
    //  Interface Impls
    FmcBusI2C_Impl       FmcBusIntf;

    //  I2C Bus Devices
    I2CDeviceN           TempControl_I2C;
    I2CDeviceN           FmcIdrom_I2C;
    Tmp175               TempControl;
    AdvI2CIdrom          FmcIdromDev;

    FmcStandardRomInfo   FmcRomInfo;
//public:
//    I2CDeviceN           Test_I2C;
};

//===========================================================================
//  CLASS FmcAFE_Common -- Fmc AFE Common Registers "SubDevice"
//===========================================================================

class FmcAFE_Common : public IX6SubDevice
{
public:

    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essFmc, essCarrier=essFmc };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Common::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeTestControl_Addr;
        unsigned int   AfeSoftwareTrigger_Addr;
        unsigned int   AfeExtSyncSelect_Addr;
    };

    FmcAFE_Common(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcAFE_Common();

    //
    //  Methods
    void TestModeEnabled( AfeDirection which, bool state, unsigned int mode );
    void SoftwareTrigger( AfeDirection which, bool state );
    void AllOff( );

    AfeExtSyncOptions  ExternalSyncSelection() const;
    void               ExternalSyncSelection(AfeExtSyncOptions option);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Common::TestControlRegister  -- TestControl Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TestControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       AdcTestEnable;
        RegisterBitGroup  AdcTestMode;
        RegisterBit       DacTestEnable;
        RegisterBitGroup  DacTestMode;

    public:
        TestControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcTestEnable(*this, 0),   AdcTestMode(*this,  4, 8),
               DacTestEnable(*this, 16),  DacTestMode(*this, 20, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Common::SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit  AdcTrigger;
        RegisterBit  DacTrigger;

    public:
        SoftwareTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcTrigger(*this, 0),   DacTrigger(*this, 16)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Common::ExtSyncSelectRegister  -- Select ext trigger source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ExtSyncSelectRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  ExtSyncOption;

    public:
        ExtSyncSelectRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ExtSyncOption(*this, 0)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    TestControlRegister       TestControl;
    SoftwareTriggerRegister   SwTrigger;
    ExtSyncSelectRegister     ExtSyncSelect;

};

//===========================================================================
//  CLASS FmcAFE_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================

class FmcAFE_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Clock::FmcClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcClockIo : public IFmcClockIo
    {
    public:
        FmcClockIo(FmcAFE_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcClockIo()
            {  }
        //
        //  IFmcClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        FmcAFE_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAFE_Clock::X6RawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcRawClockDevices : public IFmcRawClockDevices
    {
    public:
        FmcRawClockDevices(FmcAFE_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcRawClockDevices()
            {  }

        virtual Cdce62005 &   Pll()      {  return Owner->Pll();  }

    protected:
        FmcAFE_Clock *  Owner;
    };

    friend class FmcClockIo;
    friend class FmcRawClockDevices;

    FmcAFE_Clock(IRequires_StdFmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcAFE_Clock();

    FmcClockIo &           Clock() {  return ClockIo;  }
    FmcRawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Cdce62005 &            Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


private:
    //
    //  Registers
    IRequires_StdFmcClocks *  MDC;
    //Register         PllControl;     // Implemented inside CDC Pll
    //Register         PllSpi;

    Cdce62005 *         PllDev;
    FmcClockIo          ClockIo;
    FmcRawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS FmcLMK_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================

class FmcLMK_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        unsigned int   AfeExtClkSourceSelect_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK_Clock::FmcClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcClockIo : public IFmcClockIo
    {
    public:
        FmcClockIo(FmcLMK_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcClockIo()
            {  }
        //
        //  IFmcClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src);
        virtual IIClockSelect       ExternalClkSelect() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        FmcLMK_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK_Clock::FmcRawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcRawClockDevices : public ILmkRawClockDevices
    {
    public:
        FmcRawClockDevices(FmcLMK_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcRawClockDevices()
            {  }

        virtual Lmk04800 &   Pll()      {  return Owner->Pll();  }

    protected:
        FmcLMK_Clock *  Owner;
    };

    friend class FmcClockIo;
    friend class FmcRawClockDevices;

    FmcLMK_Clock(IRequires_LmkFmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcLMK_Clock();

    FmcClockIo &           Clock() {  return ClockIo;  }
    FmcRawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04800 &             Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


private:
    //
    //  Registers
    IRequires_LmkFmcClocks *  MDC;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;
    Register           ExtClkSrcSelectReg;

    Lmk04800 *          PllDev;
    FmcClockIo          ClockIo;
    FmcRawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS FmcLMK20_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================

class FmcLMK20_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK20_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        //unsigned int   AfeExtClkSourceSelect_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK20_Clock::FmcClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcClockIo : public IFmcClockIo
    {
    public:
        FmcClockIo(FmcLMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcClockIo()
            {  }
        //
        //  IFmcClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src) { Owner->CustomExternalClkSelect(src);  }
        virtual IIClockSelect       ExternalClkSelect() const {  return Owner->CustomExternalClkSelect();  }

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        FmcLMK20_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK_Clock::FmcRawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcRawClockDevices : public ILmk20RawClockDevices
    {
    public:
        FmcRawClockDevices(FmcLMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcRawClockDevices()
            {  }

        virtual Lmk04820 &   Pll()      {  return Owner->Pll();  }

    protected:
        FmcLMK20_Clock *  Owner;
    };

    friend class FmcClockIo;
    friend class FmcRawClockDevices;

    FmcLMK20_Clock(IRequires_Lmk20FmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcLMK20_Clock();

    FmcClockIo &           Clock() {  return ClockIo;  }
    FmcRawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04820 &             Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

protected:
    //  Override these if you need custom external clock src selection
    virtual void                        CustomExternalClkSelect(IFmcClockIo::IIClockSelect /*src*/) {  }
    virtual IFmcClockIo::IIClockSelect  CustomExternalClkSelect() const {  return IFmcClockIo::cslFrontPanel;  }

private:
    //
    //  Registers
    IRequires_Lmk20FmcClocks *  MDC;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;

    Lmk04820 *          PllDev;
    FmcClockIo          ClockIo;
    FmcRawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS SbcDuoLMK20_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================

class SbcDuoLMK20_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcLMK20_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        //unsigned int   AfeExtClkSourceSelect_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoLMK20_Clock::SbcDuoClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SbcDuoClockIo : public ISbcDuoClockIo
    {
    public:
        SbcDuoClockIo(SbcDuoLMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~SbcDuoClockIo()
            {  }
        //
        //  IFmcClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual void                LowLatencyTriggerMode(bool enabled);
        virtual bool                LowLatencyTriggerMode() const;

        virtual bool                Locked() const;

    protected:
        SbcDuoLMK20_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoLMK20_Clock::RawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class RawClockDevices : public ILmk20RawClockDevices
    {
    public:
        RawClockDevices(SbcDuoLMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~RawClockDevices()
            {  }

        virtual Lmk04820 &   Pll()      {  return Owner->Pll();  }
        virtual void         ReplacePll(Lmk04820 *newPll) {  Owner->ReplacePll(newPll);  }

    protected:
        SbcDuoLMK20_Clock *  Owner;
    };

    friend class SbcDuoClockIo;
    friend class RawClockDevices;

    SbcDuoLMK20_Clock(IRequires_Lmk20FmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~SbcDuoLMK20_Clock();

    SbcDuoClockIo &        Clock() {  return ClockIo;  }
    RawClockDevices &      RawClockDevice() {  return ClockDevs;  }

    Lmk04820 &             Pll();
    void                   ReplacePll(Lmk04820 *newPll);
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


private:
    //
    //  Registers
    IRequires_Lmk20FmcClocks *  MDC;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;

    Lmk04820 *          PllDev;
    SbcDuoClockIo       ClockIo;
    RawClockDevices     ClockDevs;

};

//===========================================================================
//  CLASS Fmc110_Cpld -- FMC CPLD "SubDevice"
//===========================================================================

class Fmc110_Cpld : public IX6SubDevice
{
public:

    Fmc110_Cpld(IRequires_Nothing * mdc, IAddressingSpace & space, FmcCpldSpiDevice::Info & regs);
    virtual ~Fmc110_Cpld();

    enum CpldClkSources { cpcsExternal, cpcsExtRef=0x3, cpcsInternal=0x6 };
    enum CpldSyncSources { cpssExternal, cpssCarrier, cpssClockTree, cpssNoSync };

    void  SetClkSrc(unsigned int value);
    void  SetSyncSrc(unsigned int value);
    void  SetPllReset(bool value);
    void  SetDacReset(bool value);

    bool  GetPllLocked();

    //
    //  Methods
    void  Spi(unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int address);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_Nothing *  MDC;
    FmcCpldSpiDevice     SpiDev;
    unsigned int         Reg0;
};

//===========================================================================
//  CLASS IAtroposTriggerCfg -- Interface to Atropos Trigger Cfg
//===========================================================================

class IAtroposTriggerCfg
{
public:
    enum ExtSyncOptions { essFrontPanel, essP16 };

    IAtroposTriggerCfg() {  }
    virtual ~IAtroposTriggerCfg()  {  }

    virtual void          FramedMode(bool state) = 0;
    virtual bool          FramedMode() const = 0;

    virtual void          Edge(bool state) = 0;
    virtual bool          Edge() const = 0;

    virtual void          External(bool state)  = 0;
    virtual bool          External() const  = 0;

    virtual void          FrameSize(unsigned int samples) = 0;
    virtual unsigned int  FrameSize() const  = 0;

    virtual void            ExternalSyncSource(ExtSyncOptions src) = 0;
    virtual ExtSyncOptions  ExternalSyncSource() const = 0;

    virtual unsigned int    TriggerFrameGranularity() = 0;
    virtual void            Decimation(unsigned int samples) = 0;
    virtual unsigned int    Decimation() const = 0;

};

//===========================================================================
//  CLASS AtroposLMK20_Clock -- Atropos AFE Clock "SubDevice"
//===========================================================================

class AtroposLMK20_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  AtroposLMK20_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        unsigned int   ExtClkSourceSelect_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  AtroposLMK20_Clock::AtroposClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AtroposClockIo : public IAtroposClockIo
    {
    public:
        AtroposClockIo(AtroposLMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~AtroposClockIo()
            {  }
        //
        //  IFmcClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src);
        virtual IIClockSelect       ExternalClkSelect() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;        

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;
        virtual bool                TriggerSync() const;

    protected:
        AtroposLMK20_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposLMK20_Clock::FmcRawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AtroposRawClockDevices : public ILmk20RawClockDevices
    {
    public:
        AtroposRawClockDevices(AtroposLMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~AtroposRawClockDevices()
            {  }

        virtual Lmk04820 &   Pll()      {  return Owner->Pll();  }

    protected:
        AtroposLMK20_Clock *  Owner;
    };

    friend class AtroposClockIo;
    friend class AtroposRawClockDevices;

    AtroposLMK20_Clock(IRequires_Lmk20FmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~AtroposLMK20_Clock();

    AtroposClockIo &           Clock() {  return ClockIo;  }
    AtroposRawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04820 &             Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


private:
    //
    //  Registers
    IRequires_Lmk20FmcClocks *  MDC;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;
    Register           ExtClkSrcSelectReg;

    Lmk04820 *              PllDev;
    AtroposClockIo          ClockIo;
    AtroposRawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS AtroposTriggerSystem  -- Atropos Trigger Manager
//===========================================================================

class AtroposTriggerSystem : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  AtroposTriggerSystem::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   ExtSyncSelect_Addr;
        unsigned int   SerDesControl_Addr;
        unsigned int   SoftwareTrigger_Addr;
        unsigned int   TriggerConfig_Addr;
        unsigned int   TriggerDecimation_Addr;
    };

    AtroposTriggerSystem(IRequires_FrameCalcDevice * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~AtroposTriggerSystem();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposTriggerSystem::TriggerCfg  -- Trigger Configuration Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfg : public IAtroposTriggerCfg
    {
    public:
        TriggerCfg(AtroposTriggerSystem * owner) : Owner(owner)  {  }
        virtual ~TriggerCfg()  {  }

        virtual void          FramedMode(bool state) {  Owner->FramedTriggerEnabled(state);  }
        virtual bool          FramedMode() const {  return Owner->FramedTriggerEnabled();  }

        virtual void          Edge(bool state) {  Owner->EdgeTrigger(state);  }
        virtual bool          Edge() const {  return Owner->EdgeTrigger();  }

        virtual void          External(bool state) {  Owner->ExternalTrigger(state);  }
        virtual bool          External() const {  return Owner->ExternalTrigger(); }

        virtual void          FrameSize(unsigned int samples)
                                    {  Owner->FramedTriggerSize(samples);  }
        virtual unsigned int  FrameSize() const
                                    {  return Owner->FramedTriggerSize();  }

        virtual void            ExternalSyncSource(ExtSyncOptions src)
                                        {  Owner->ExternalSyncSelection(src);  }
        virtual ExtSyncOptions  ExternalSyncSource() const
                                        {  return Owner->ExternalSyncSelection();  }

        virtual unsigned int    TriggerFrameGranularity()
                                        {  return 1; }
        virtual void            Decimation(unsigned int samples)
                                        {  Owner->TriggerDecimation(samples);   }
        virtual unsigned int    Decimation() const
                                        {  return Owner->TriggerDecimation();   }

    private:
        AtroposTriggerSystem * Owner;
    };

    IAtroposTriggerCfg &  Trigger()
        {  return TriggerCfgImpl;  }
    //virtual IPulseRepetitionIntervalIntf &  Pulse() = 0;


    virtual void SoftwareTrigger(bool state);
    virtual void MasterTrigger(bool state);

    IAtroposTriggerCfg::ExtSyncOptions  ExternalSyncSelection() const;
    void               ExternalSyncSelection( IAtroposTriggerCfg::ExtSyncOptions option);

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  TriggerDecimation(unsigned int samples);
    unsigned int TriggerDecimation() const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposTriggerSystem::ExtSyncSelectRegister  -- Select ext trigger source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ExtSyncSelectRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  ExtSyncOption;

    public:
        ExtSyncSelectRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ExtSyncOption(*this, 0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposTriggerSystem::SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  SwTrigger;
        RegisterBit  MasterTrigger;

    public:
        SoftwareTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               SwTrigger(*this, 0), MasterTrigger(*this, 1)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposTriggerSystem::TriggerCfgRegister -- Configure Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  FrameSize;
        RegisterBit       RisingEdge;
        RegisterBit       FramedMode;
        RegisterBit       ExternalTrigger;

    public:
        TriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               FrameSize(*this, 0, 24),   RisingEdge(*this, 29),
               FramedMode(*this, 30),     ExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposTriggerSystem::DecimationRegister  -- Sync Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        DecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposTriggerSystem::SerDesControlRegister  -- Serializer/Deserializer
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SerDesControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       S2P_Reset;
        RegisterBit       S2P_Sdr_DdrN_Mode;
        RegisterBit       S2P_SyncPulse;
        RegisterBit       S2P_SelectB;
        RegisterBit       S2P_CalDone;
        RegisterBit       S2P_DataCheckEnable;
        RegisterBit       S2P_DataValid;
        RegisterBit       S2P_DataAligned;
        RegisterBit       P2S_Reset;
        RegisterBit       P2S_Sdr_DdrN_Mode;
        RegisterBit       P2S_UpdateFallingEdge;
        RegisterBit       P2S_DataOverride;
        RegisterBit       P2S_PulserCalEnable;
        RegisterBit       P2S_PulserCalDone;
        RegisterBitGroup  P2S_Samples;

    public:
        SerDesControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
            S2P_Reset(*this, 0),          S2P_Sdr_DdrN_Mode(*this, 1),
            S2P_SyncPulse(*this, 2),      S2P_SelectB(*this, 3),
            S2P_CalDone(*this, 4),        S2P_DataCheckEnable(*this, 5),
            S2P_DataValid(*this, 6),      S2P_DataAligned(*this, 7),
            P2S_Reset(*this, 8),
            P2S_Sdr_DdrN_Mode(*this, 9),  P2S_UpdateFallingEdge(*this, 10),
            P2S_DataOverride(*this, 11),  P2S_PulserCalEnable(*this, 12),
            P2S_PulserCalDone(*this, 13),
            P2S_Samples(*this, 16, 16)
            {
            }
    };

    SerDesControlRegister &  SerDes()  { return SerDesControlReg;  }

private:
    //
    //  Registers
    IRequires_FrameCalcDevice *  MDC;
    IFrameCalcDevice *           Dev;

    ExtSyncSelectRegister     ExtSyncSelectReg;
    SoftwareTriggerRegister   SwTriggerReg;
    SerDesControlRegister     SerDesControlReg;
    TriggerCfgRegister        TriggerCfgReg;
    DecimationRegister        DecimationReg;

    TriggerCfg                TriggerCfgImpl;

};

//===========================================================================
//  CLASS ISbcDuoTriggerCfg -- Interface to SbcDuo Trigger Cfg
//===========================================================================

class ISbcDuoTriggerCfg
{
public:
    ISbcDuoTriggerCfg() {  }
    virtual ~ISbcDuoTriggerCfg()  {  }

    virtual void          FramedMode(bool state) = 0;
    virtual bool          FramedMode() const = 0;

    virtual void          Edge(bool state) = 0;
    virtual bool          Edge() const = 0;

    virtual void          External(bool state)  = 0;
    virtual bool          External() const  = 0;

    virtual void          FrameSize(unsigned int samples) = 0;
    virtual unsigned int  FrameSize() const  = 0;

    virtual unsigned int    TriggerFrameGranularity() = 0;
    virtual void            Decimation(unsigned int samples) = 0;
    virtual unsigned int    Decimation() const = 0;

};

//===========================================================================
//  CLASS SbcDuoTriggerSystem  -- SbcDuo Trigger Manager
//===========================================================================

class SbcDuoTriggerSystem : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  SbcDuoTriggerSystem::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   SoftwareTrigger_Addr;
        unsigned int   TriggerConfig_Addr;
        unsigned int   TriggerDecimation_Addr;
    };

    SbcDuoTriggerSystem(IRequires_FrameCalcDevice * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~SbcDuoTriggerSystem();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoTriggerSystem::TriggerCfg  -- Trigger Configuration Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfg : public ISbcDuoTriggerCfg
    {
    public:
        TriggerCfg(SbcDuoTriggerSystem * owner) : Owner(owner)  {  }
        virtual ~TriggerCfg()  {  }

        virtual void          FramedMode(bool state) {  Owner->FramedTriggerEnabled(state);  }
        virtual bool          FramedMode() const {  return Owner->FramedTriggerEnabled();  }

        virtual void          Edge(bool state) {  Owner->EdgeTrigger(state);  }
        virtual bool          Edge() const {  return Owner->EdgeTrigger();  }

        virtual void          External(bool state) {  Owner->ExternalTrigger(state);  }
        virtual bool          External() const {  return Owner->ExternalTrigger(); }

        virtual void          FrameSize(unsigned int samples)
                                    {  Owner->FramedTriggerSize(samples);  }
        virtual unsigned int  FrameSize() const
                                    {  return Owner->FramedTriggerSize();  }

        virtual unsigned int    TriggerFrameGranularity()
                                        {  return 1; }
        virtual void            Decimation(unsigned int samples)
                                        {  Owner->TriggerDecimation(samples);   }
        virtual unsigned int    Decimation() const
                                        {  return Owner->TriggerDecimation();   }

    private:
        SbcDuoTriggerSystem * Owner;
    };

    ISbcDuoTriggerCfg &  Trigger()
        {  return TriggerCfgImpl;  }

    void  MasterTrigger(bool state);
    void  SoftwareTrigger(bool state);
    void  ArmPpsTrigger();


    //  Trigger Modes IMPL
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;
    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    void  TriggerDecimation(unsigned int samples);
    unsigned int TriggerDecimation() const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoTriggerSystem::SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  SwTrigger;
        RegisterBit  MasterEnable;
        RegisterBit  ArmPpsTrigger;

    public:
        SoftwareTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               SwTrigger(*this, 0), MasterEnable(*this, 1), ArmPpsTrigger(*this, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoTriggerSystem::TriggerCfgRegister -- Configure Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  FrameSize;
        RegisterBit       RisingEdge;
        RegisterBit       FramedMode;
        RegisterBit       ExternalTrigger;

    public:
        TriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               FrameSize(*this, 0, 24),   RisingEdge(*this, 29),
               FramedMode(*this, 30),     ExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoTriggerSystem::DecimationRegister  -- Sync Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        DecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_FrameCalcDevice *  MDC;
    IFrameCalcDevice *           Dev;

    SoftwareTriggerRegister   SwTriggerReg;
    TriggerCfgRegister        TriggerCfgReg;
    DecimationRegister        DecimationReg;

    TriggerCfg                TriggerCfgImpl;

};


//=============================================================================
//  CLASS FmcCalibrationIDROMImage  --  Use section of Info Idrom
//=============================================================================

class FmcCalibrationIDROMImage :  public GeneralIdromImage
{
public:
    //
    //  Member Functions
    FmcCalibrationIDROMImage(FmcStandardRomInfo& romInfo, bool isAdcImage, unsigned int idrom_data_size_words);
    virtual ~FmcCalibrationIDROMImage()
        {}

    void  RefreshLocalImage();

protected:

    void  ToLocalImage(int /*offset*/, int * data, int words);
    void  FromLocalImage(int /*offset*/, const int* data, int words);

    virtual void ROM_ReadBlock(int offset, int * data, int words);
    virtual void ROM_WriteBlock(int offset, const int * data, int words);

    FmcStandardRomInfo::CalCoefficientsMR &  WhichCal();

private:
    bool IsAdcImage;
    FmcStandardRomInfo &  RomInfo;

};

//===========================================================================
//  CLASS FmcCalibrationIDROM_Manager -- Fmc Shadow Cal-Rom "SubDevice"
//===========================================================================
//  The goofy format of the FMC IDROM keeps us from having a known image, so we
//    make a phoney one and use it, then reflect it to the FMC Rom Image at need.

class FmcCalibrationIDROM_Manager : public IX6SubDevice
{
public:
//    //=======================================================================
//    //  CLASS FmcCalibrationIDROM_Manager::MapRegisters  -- Device Register Offsets
//    //=======================================================================
//    class MapRegisters
//    {
//    public:
//        unsigned int   JESD_LineRate_Addr;
//    };

    FmcCalibrationIDROM_Manager(IRequires_FmcCalIDROM * mdc, FmcStandardRomInfo& romInfo, bool needsAdcImage);
    virtual ~FmcCalibrationIDROM_Manager();

    //  Methods
    FmcCalibrationIDROMImage &  Idrom() {  return IDROMImage;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    IRequires_FmcCalIDROM *       MDC;
    FmcCalibrationIDROMImage      IDROMImage;
    bool NeedsAdcImage;

};

//=============================================================================
//  CLASS FmcRangedCalibrationIDROMImage  --  Use section of Info Idrom
//=============================================================================

class FmcRangedCalibrationIDROMImage :  public GeneralIdromImage
{
public:
    //
    //  Member Functions
    FmcRangedCalibrationIDROMImage(FmcStandardRomInfo& romInfo, unsigned int idrom_data_size_words);
    virtual ~FmcRangedCalibrationIDROMImage()
        {}

    void  RefreshLocalImage();

protected:

    void  ToLocalImage(int /*offset*/, int * data, int words);
    void  FromLocalImage(int /*offset*/, const int* data, int words);

    virtual void ROM_ReadBlock(int offset, int * data, int words);
    virtual void ROM_WriteBlock(int offset, const int * data, int words);



private:
    FmcStandardRomInfo &  RomInfo;

};
//===========================================================================
//  CLASS FmcRangedCalibrationIDROM_Manager -- Fmc Shadow Cal-Rom "SubDevice"
//===========================================================================
//  The goofy format of the FMC IDROM keeps us from having a known image, so we
//    make a phoney one and use it, then reflect it to the FMC Rom Image at need.
//  This one expects more than one page, for ADCs with distinct ranges

class FmcRangedCalibrationIDROM_Manager : public IX6SubDevice
{
public:
//    //=======================================================================
//    //  CLASS FmcCalibrationIDROM_Manager::MapRegisters  -- Device Register Offsets
//    //=======================================================================
//    class MapRegisters
//    {
//    public:
//        unsigned int   JESD_LineRate_Addr;
//    };

    FmcRangedCalibrationIDROM_Manager(IRequires_FmcCalIDROM * mdc, FmcStandardRomInfo& romInfo);
    virtual ~FmcRangedCalibrationIDROM_Manager();

    //  Methods
    FmcRangedCalibrationIDROMImage &  Idrom() {  return IDROMImage;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    IRequires_FmcCalIDROM *          MDC;
    FmcRangedCalibrationIDROMImage   IDROMImage;

};

//===========================================================================
//  CLASS FmcAd9510Duo_Clock -- Fmc AFE Clock "SubDevice"
//===========================================================================
//  For Ad9510Duo_Tb

class FmcAd9510Duo_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAd9510Duo_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
    };

    friend class AdClockIo;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAd9510Duo_Clock::AdClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdClockIo : public IX6ClockIo
    {
    public:
        AdClockIo(FmcAd9510Duo_Clock * owner): Owner(owner)
            {  }
        virtual ~AdClockIo()
            {  }
        //
        //  IFmcClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src) { Owner->CustomExternalClkSelect(src);  }
        virtual IIClockSelect       ExternalClkSelect() const {  return Owner->CustomExternalClkSelect();  }

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        FmcAd9510Duo_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcAd9510Duo_Clock::FmcRawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcRawClockDevices : public IAd9510DuoRawClockDevices
    {
    public:
        FmcRawClockDevices(FmcAd9510Duo_Clock * owner): Owner(owner)
            {  }
        virtual ~FmcRawClockDevices()
            {  }

        virtual Ad9510Duo_Tb &   Pll()      {  return Owner->Pll();  }

    protected:
        FmcAd9510Duo_Clock *  Owner;
    };

    friend class FmcClockIo;
    friend class FmcRawClockDevices;

    FmcAd9510Duo_Clock(IRequires_Ad9510TbClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~FmcAd9510Duo_Clock();

    AdClockIo &            Clock() {  return ClockIo;  }
    FmcRawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Ad9510Duo_Tb &         Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

//protected:
    //  Override these if you need custom external clock src selection
    virtual void    CustomExternalClkSelect(IX6ClockIo::IIClockSelect src);
    virtual IX6ClockIo::IIClockSelect  CustomExternalClkSelect() const;

//private:
    //
    //  Registers
    IRequires_Ad9510TbClocks *  MDC;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;

    Ad9510Duo_Tb        PllDev;
    AdClockIo           ClockIo;
    FmcRawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS PolledDataFifo  -- Data FIFO control for polled mode logic
//===========================================================================

class PolledDataFifo : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  PolledDataFifo::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   FifoControl_Addr;
        unsigned int   FifoThreshold_Addr;
        unsigned int   FifoData_Addr;
        unsigned int   DacDelay_Addr;
    };

    PolledDataFifo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs, unsigned int fifo_depth );
    virtual ~PolledDataFifo();

    friend class PolledFifo;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PolledDataFifo::PolledFifo  -- standard FIFO interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PolledFifo : public IPolledFifo
    {
    public:
        PolledFifo(PolledDataFifo * owner) : Owner(owner), FifoSize(1024)
                {  }
        virtual ~PolledFifo()  {  }

        virtual unsigned int  FifoDepth()
            {  return Owner->FFifoDepth;  }
        virtual unsigned int  Count()
            {  return Owner->FifoControlReg.DataCount.Value();  }
        virtual bool          AlmostEmpty()
            {  return Owner->FifoControlReg.AlmostEmpty.Value();  }
        virtual bool          Empty()
            {  return Owner->FifoControlReg.Empty.Value();  }
        virtual bool          AlmostFull()
            {  return Owner->FifoControlReg.AlmostFull.Value();  }
        virtual bool          Full()
            {  return Owner->FifoControlReg.Full.Value();  }

        virtual void          AlmostFullLevel(unsigned int samples)
                                    {  Owner->FifoThresholdReg.AlmostFullThreshold.Value(samples);  }
        virtual unsigned int  AlmostFullLevel() const
                                    {  return Owner->FifoThresholdReg.AlmostFullThreshold.Value();  }
        virtual void          AlmostEmptyLevel(unsigned int samples)
                                    {  Owner->FifoThresholdReg.AlmostEmptyThreshold.Value(samples);  }
        virtual unsigned int  AlmostEmptyLevel() const
                                    {  return Owner->FifoThresholdReg.AlmostEmptyThreshold.Value();  }

        virtual void          Data(unsigned int data)
                                    {  Owner->FifoDataReg.Value(data);  }
        virtual unsigned int  Data() const
                                    {  return Owner->FifoDataReg.Value();  }

        virtual void          DelayInSamples(unsigned int clock_ticks)
                                    {  Owner->DacDelayReg.SampleClockDelay.Value(clock_ticks);  }
        virtual unsigned int  DelayInSamples() const
                                    {  return Owner->DacDelayReg.SampleClockDelay.Value();  }
        virtual void          Delay(unsigned int us_ticks)
                                    {  Owner->DacDelayReg.InPeriodDelay.Value(us_ticks);  }
        virtual unsigned int  Delay() const
                                    {  return Owner->DacDelayReg.InPeriodDelay.Value();  }
        virtual void          LoadFullDelay(unsigned int total_delay_us, double sample_freq_hz, unsigned int decimation)
                                    {  return Owner->LoadFullDelay(total_delay_us, sample_freq_hz, decimation);  }

    private:
        PolledDataFifo * Owner;
        const int FifoSize;
    };

    IPolledFifo &  Fifo()
        {  return PolledFifoImpl;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PolledDataFifo::FifoControlRegister  -- Fifo control register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FifoControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DataCount;
        RegisterBit       AlmostEmpty;
        RegisterBit       Empty;
        RegisterBit       AlmostFull;
        RegisterBit       Full;

    public:
        FifoControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DataCount(*this, 0, 10),
               AlmostEmpty(*this, 28), Empty(*this, 29),
               AlmostFull(*this, 30),  Full(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PolledDataFifo::FifoThresholdRegister -- Set FIFO Thresholds
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FifoThresholdRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  AlmostEmptyThreshold;
        RegisterBitGroup  AlmostFullThreshold;

    public:
        FifoThresholdRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AlmostEmptyThreshold(*this, 0, 10),
               AlmostFullThreshold(*this, 16, 10)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PolledDataFifo::FifoDataRegister  -- Data Access Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FifoDataRegister : public Register
    {
        typedef Register inherited;

    public:

    public:
        FifoDataRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PolledDataFifo::DacDelayRegister  -- Config Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacDelayRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  InPeriodDelay;     // delay within single period (usec)
        RegisterBitGroup  SampleClockDelay;  // full clock periods of delay (added on)

    public:
        DacDelayRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               InPeriodDelay(*this, 0, 10),
               SampleClockDelay(*this, 12, 14)
            {
            }
    };

//private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    const int            FFifoDepth;

    FifoControlRegister       FifoControlReg;
    FifoThresholdRegister     FifoThresholdReg;
    FifoDataRegister          FifoDataReg;
    DacDelayRegister          DacDelayReg;

    PolledFifo                PolledFifoImpl;

    void  LoadFullDelay(unsigned int total_delay_us, double sample_freq_hz, unsigned int decimation_reg_value);


};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
