//---------------------------------------------------------------------------
// FmcSupport_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <FmcSupport_Mb.h>
#include <PmcIdrom_Mb.h>
#include <StringSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
#pragma warning(disable : 4355)
//#pragma warning(disable : 8004)
#endif

//==============================================================================
//  CLASS FmcStandardRomInfo -- Interface to standard Fmc Rom Information
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo
//---------------------------------------------------------------------------

FmcStandardRomInfo::FmcStandardRomInfo(class AdvI2CIdrom & rom)
    :  ROM(rom), CH(rom), BA(rom), MRA(rom), ParseOk(false)
{
    Header().BaseOffset(0);   // Common Header is always at 0
}


//---------------------------------------------------------------------------
//  FmcStandardRomInfo::StoreToRom()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::StoreToRom()
{
    //  Build the image from the Info
    ToRomImage();

//    std::stringstream msg;
//    msg << std::hex << "STORE ROM IMAGE ";
//    Debug.Log(msg.str());
//    for (unsigned int i=0; i<128; i+=8)
//        {
//        std::stringstream msg2;

//        msg2 << std::hex
//             << "    " << ROM.AsInt(i)
//             << "    " << ROM.AsInt(i+1)
//             << "    " << ROM.AsInt(i+2)
//             << "    " << ROM.AsInt(i+3)
//             << "    " << ROM.AsInt(i+4)
//             << "    " << ROM.AsInt(i+5)
//             << "    " << ROM.AsInt(i+6)
//             << "    " << ROM.AsInt(i+7);
//        Debug.Log(msg2.str());
//        }
    //  Save the Image
    ROM.StoreToRom();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::LoadFromRom()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::LoadFromRom()
{
    //  Read the image from ROM
    ROM.LoadFromRom();

//    std::stringstream msg;
//    msg << std::hex << "LOAD ROM IMAGE ";
//    Debug.Log(msg.str());
//    for (unsigned int i=0; i<128; i+=8)
//        {
//        std::stringstream msg2;

//        msg2 << std::hex
//             << "    " << ROM.AsInt(i)
//             << "    " << ROM.AsInt(i+1)
//             << "    " << ROM.AsInt(i+2)
//             << "    " << ROM.AsInt(i+3)
//             << "    " << ROM.AsInt(i+4)
//             << "    " << ROM.AsInt(i+5)
//             << "    " << ROM.AsInt(i+6)
//             << "    " << ROM.AsInt(i+7);
//        Debug.Log(msg2.str());
//        }
    //  Build the Info from the image
    FromRomImage();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::ToRomImage()
{
    //  Board Area Section Positioning
    //  Set offsets
    Board().BaseOffset(Header().End());
    //  ...fill in Area Offset
    Header().BoardAreaOffset(Board().BaseOffset());
    //  ...Change any section data

    //  ...lay out the section
    Board().ToRomImage();

    //  Multi-Record Area Section Positioning
    //  Set offsets
    MultiR().BaseOffset(Board().End());

    //  ...fill in Area Offset
    if (MultiR().IsEmpty())
        Header().MultiRecordOffset(0);  // if no records, then no section
    else
        Header().MultiRecordOffset(MultiR().BaseOffset());
    //  ...Change any section data

    //  ...lay out the section
    MultiR().ToRomImage();

    //  Lay out the Header Section
    Header().ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FromRomImage()
{
    ScopedTraceRef(Debug, "FmcStandardRomInfo::FromRomImage()");

//    Debug.Log("ROM Dump");
//    for (int i=0; i<20; i++)
//        Debug.Log("...", ROM.AsInt(i), 16);

    Header().FromRomImage();
    //
    //  Analyze Header - check checksum
    ParseOk = Header().AreaCheck();
    {
    std::stringstream msg;
    msg << "Header: " << ParseOk << " Result: " << Header().AreaCheckResult();
    Debug.Log(msg.str());
    }
    if (ParseOk==false)
        {
        CheckStr = "FMC Header Parse Failed Checksum - ";
        CheckStr += Header().AreaCheckResult();
        return;
        }

    if (Header().BoardAreaOffset()==0)
        {
        Debug.Log("Board Area: None Found");
        CheckStr = "FMC Board Area not Found";
        return;  // error
        }
    else
        {
        Board().BaseOffset(Header().BoardAreaOffset());
        Board().FromRomImage();

        //
        //  Analyze Header - check checksum
        ParseOk = Board().AreaCheck();

        std::stringstream msg;
        msg << "Board: " << ParseOk << " Result: " << Board().AreaCheckResult();
        Debug.Log(msg.str());

        if (ParseOk==false)
            {
            CheckStr = "FMC Board Area Parse Failed Checksum - ";
            CheckStr += Board().AreaCheckResult();
            return;
            }
        }

    if (Header().MultiRecordOffset()==0)
        {
        Debug.Log("MultiRecord Area: None Found");
        CheckStr = "FMC MultiRecord Area not Found";
        return;  // error
        }
    else
        {
        MultiR().BaseOffset(Header().MultiRecordOffset());
        MultiR().FromRomImage();

        //
        //  Analyze Header - check checksum
        ParseOk = MultiR().AreaCheck();

        std::stringstream msg;
        msg << "MultiRecord: " << ParseOk << " Result: " << MultiR().AreaCheckResult();
        Debug.Log(msg.str());
        if (ParseOk==false)
            {
            CheckStr = "FMC MultiRecord Area Parse Failed Checksum - ";
            CheckStr += MultiR().AreaCheckResult();
            return;
            }
        }

      CheckStr = "Parse Ok.";
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::NewDcOutputRecord() - add record
//---------------------------------------------------------------------------

FmcStandardRomInfo::DcOutputMultiRecord *  FmcStandardRomInfo::NewDcOutputRecord()
{
    DcOutputMultiRecord *  DcOut = new DcOutputMultiRecord(ROM);
    MultiR().AddRecord(DcOut);
    return DcOut;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::NewDcLoadRecord() - add record
//---------------------------------------------------------------------------

FmcStandardRomInfo::DcLoadMultiRecord *  FmcStandardRomInfo::NewDcLoadRecord()
{
    DcLoadMultiRecord *  DcLoad = new DcLoadMultiRecord(ROM);
    MultiR().AddRecord(DcLoad);
    return DcLoad;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::NewAdcCalCoefficientRecord() - add record
//---------------------------------------------------------------------------

FmcStandardRomInfo::AdcCalCoefficientsMR *  FmcStandardRomInfo::NewAdcCalCoefficientRecord()
{
    MultiR().AddAdcCalRecord();
    return &MultiR().AdcCal();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::NewDacCalCoefficientRecord() - add record
//---------------------------------------------------------------------------

FmcStandardRomInfo::DacCalCoefficientsMR *  FmcStandardRomInfo::NewDacCalCoefficientRecord()
{
    MultiR().AddDacCalRecord();
    return &MultiR().DacCal();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::NewAdcRangedCalCoefficientRecord() - add record
//---------------------------------------------------------------------------

FmcStandardRomInfo::AdcRangedCalCoefficientsMR *  FmcStandardRomInfo::NewAdcRangedCalCoefficientRecord(size_t idx, size_t total, size_t channels)
{
    MultiR().AddAdcRangedCalRecord(idx, total, channels);
    return &(MultiR().AdcRangedCal(idx));
}

//==============================================================================
//  CLASS FmcStandardRomInfo::FmcRomAreaBase -- Base class
//==============================================================================
//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::ReadByte() - read a byte from image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::ReadByte(unsigned int offset, unsigned int & byte)
{
    unsigned int idx = Base+offset;
    unsigned char newbyte = ROM.AsChar(idx);
    byte = static_cast<unsigned int>(newbyte);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::WriteByte() - write a byte to image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::WriteByte(unsigned int offset, unsigned int byte)
{
    unsigned int idx = Base+offset;
    ROM.AsChar(idx, byte);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::ReadMultiByte() - read a value from image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::ReadMultiByte(unsigned int offset, unsigned int & value, unsigned int bytes)
{
    value = 0;
    unsigned int shift = 0;
    while (bytes--)
        {
        unsigned int byte = 0;
        ReadByte(offset++, byte);   // data stored LSB first
        value |= (byte<<shift);
        shift += 8;
        }
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::WriteMultiByte() - write a value to image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::WriteMultiByte(unsigned int offset, unsigned int value, unsigned int bytes)
{
    while (bytes--)
        {
        unsigned int byte = value&0xFF;
        value >>= 8;
        WriteByte(offset++, byte);   // data stored LSB first
        }
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::WriteStr() - write a value from image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::WriteStr(unsigned int offset, const std::string & text)
{
    MemoryBitManipulator  type_len(0);
    type_len.Field(0, 6, static_cast<unsigned int>(text.size()));   // set size
    if (text.size()>=2)
        type_len.Field(6, 2, 0x3);       // encode for type 3 encoding
    WriteByte(offset++, type_len.Value());        // if short use type 0 to avoid 0xC1

    for (unsigned int idx = 0; idx<text.size(); idx++)
        {
        unsigned int byte = static_cast<unsigned int>(text[idx]);
        WriteByte(offset++, byte);
        }
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::ReadStr() - read string to image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::ReadStr(unsigned int offset, std::string & text)
{
    unsigned int type_len = 0;
    ReadByte(offset++, type_len);

    unsigned int size = (type_len&0x3F);
    text.resize(size);

    unsigned int idx = 0;
    while (size--)
        {
        unsigned int byte = 0;
        ReadByte(offset++, byte);
        text[idx++] = static_cast<char>(byte);
        }
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcRomAreaBase::CalculateChecksum() - checksum
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcRomAreaBase::CalculateChecksum(unsigned int & cksum, unsigned int start_idx, unsigned int size)
{
    unsigned int idx_0 = Base+start_idx;
    unsigned int sum = 0;
    for (unsigned int idx=idx_0; idx<idx_0+size; idx++)
        sum += ROM.AsChar(idx);

    cksum = 0 - (sum&0xFF);
    cksum &= 0xFF;
}

//==============================================================================
//  CLASS FmcStandardRomInfo::CommonHeader -- Rom Common Header
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::CommonHeader
//---------------------------------------------------------------------------

FmcStandardRomInfo::CommonHeader::CommonHeader(AdvI2CIdrom & rom)
    :   FmcRomAreaBase(rom),
        Version_1(0x2),  InternalUseOfs_1(0x0), ChassisOfs_1(0x0), BoardAreaOfs_1(0x0),
        ProductInfoOfs_1(0x0), MultiRecordOfs_1(0x0)
{
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::CommonHeader::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::CommonHeader::ToRomImage()
{
    //
    //  We don't support some sections right now...
    InternalUseOfs_1 = 0;  ChassisOfs_1 = 0;  ProductInfoOfs_1 = 0;

    WriteByte(0, Version_1);
    WriteByte(1, InternalUseOfs_1);
    WriteByte(2, ChassisOfs_1);
    WriteByte(3, BoardAreaOfs_1);
    WriteByte(4, ProductInfoOfs_1);
    WriteByte(5, MultiRecordOfs_1);
    WriteByte(6, 0);  // PAD

    CalculateChecksum(CheckSum_1, 0, 7);
    WriteByte(7, CheckSum_1);

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::CommonHeader::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::CommonHeader::FromRomImage()
{
    ReadByte(0, Version_1);
    ReadByte(1, InternalUseOfs_1);
    ReadByte(2, ChassisOfs_1);
    ReadByte(3, BoardAreaOfs_1);
    ReadByte(4, ProductInfoOfs_1);
    ReadByte(5, MultiRecordOfs_1);
    //  ReadByte(6, 0);  // PAD
    ReadByte(7, CheckSum_1);

    Debug.Log("Header Dump");
    Debug.Log("...Version_1", Version_1, 16);
    Debug.Log("...InternalUseOfs_1", InternalUseOfs_1, 16);
    Debug.Log("...ChassisOfs_1", ChassisOfs_1, 16);
    Debug.Log("...BoardAreaOfs_1", BoardAreaOfs_1, 16);
    Debug.Log("...ProductInfoOfs_1", ProductInfoOfs_1, 16);
    Debug.Log("...MultiRecordOfs_1", MultiRecordOfs_1, 16);
    unsigned int PAD;  ReadByte(6, PAD);
    Debug.Log("...PAD", PAD, 16);
    Debug.Log("...CheckSum_1", CheckSum_1, 16);

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::CommonHeader::AreaCheck() -
//---------------------------------------------------------------------------

bool  FmcStandardRomInfo::CommonHeader::AreaCheck()
{
    unsigned int check(0);
    CalculateChecksum(check, 0, 7);
    if (CheckSum_1==check)
        {
        CheckStr = "Ok.";
        return true;
        }
    else
        {
        CheckStr = "Failed Checksum.";
        return false;
        }

}

//==============================================================================
//  CLASS FmcStandardRomInfo::BoardArea -- Rom Common Header
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::BoardArea
//---------------------------------------------------------------------------

FmcStandardRomInfo::BoardArea::BoardArea(AdvI2CIdrom & rom)
    :   FmcRomAreaBase(rom),
        Version_1(0x2),  AreaLength_1(0x0), Language_1(25), ManufactureDate_3(0x10203),
        Manufacturer_s("Innovative Integration")
{
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::BoardArea::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::BoardArea::ToRomImage()
{
    unsigned int ofs = 0;
    WriteByte(ofs++, Version_1);
    unsigned int AreaLengthOffset = ofs++;  // save for later
    WriteByte(ofs++, Language_1);
    WriteMultiByte(ofs, ManufactureDate_3, 3);  ofs += 3;
    WriteStr(ofs++, Manufacturer_s);  ofs += static_cast<unsigned int>(Manufacturer_s.size());
    WriteStr(ofs++, Name_s);          ofs += static_cast<unsigned int>(Name_s.size());
    WriteStr(ofs++, SerialNumber_s);  ofs += static_cast<unsigned int>(SerialNumber_s.size());
    WriteStr(ofs++, PartNumber_s);    ofs += static_cast<unsigned int>(PartNumber_s.size());
    WriteStr(ofs++, FruFile_s);       ofs += static_cast<unsigned int>(FruFile_s.size());
    //  Custom Fields
    //  Final Fields
    WriteByte(ofs++, 0xC1);
    while ((ofs%8)!=0)
        {
        WriteByte(ofs++, 0x00);  // Pad
        }
    // calc AreaLength
    AreaLength_1 = ofs>>3;
    WriteByte(AreaLengthOffset, AreaLength_1);

    CalculateChecksum(CheckSum_1, 0, ofs-1);  // don't include checksum word
    WriteByte(ofs-1, CheckSum_1);

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::BoardArea::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::BoardArea::FromRomImage()
{
    unsigned int ofs = 0;
    ReadByte(ofs++, Version_1);
    ReadByte(ofs++, AreaLength_1);
    ReadByte(ofs++, Language_1);
    ReadMultiByte(ofs, ManufactureDate_3, 3);  ofs += 3;
    ReadStr(ofs++, Manufacturer_s);  ofs += static_cast<unsigned int>(Manufacturer_s.size());
    ReadStr(ofs++, Name_s);          ofs += static_cast<unsigned int>(Name_s.size());
    ReadStr(ofs++, SerialNumber_s);  ofs += static_cast<unsigned int>(SerialNumber_s.size());
    ReadStr(ofs++, PartNumber_s);    ofs += static_cast<unsigned int>(PartNumber_s.size());
    ReadStr(ofs++, FruFile_s);       ofs += static_cast<unsigned int>(FruFile_s.size());
    //  Custom Fields - ignored
    //  Final Fields

    ReadByte((AreaLength_1<<3)-1, CheckSum_1);

    Debug.Log("BoardArea Dump");
    Debug.Log("...Version_1", Version_1, 16);
    Debug.Log("...AreaLength_1", AreaLength_1, 16);
    Debug.Log("...Language_1", Language_1, 16);
    Debug.Log("...ManufactureDate_3", ManufactureDate_3);
    Debug.Log("...Manufacturer_s", Manufacturer_s);
    Debug.Log("...Name_s", Name_s);
    Debug.Log("...SerialNumber_s", SerialNumber_s);
    Debug.Log("...PartNumber_s", PartNumber_s);
    Debug.Log("...FruFile_s", FruFile_s);

    Debug.Log("...CheckSum_1", CheckSum_1, 16);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::BoardArea::AreaCheck() -
//---------------------------------------------------------------------------

bool  FmcStandardRomInfo::BoardArea::AreaCheck()
{
    unsigned int check(0);
    CalculateChecksum(check, 0, AreaSizeBytes()-1);
    if (CheckSum_1==check)
        {
        CheckStr = "Ok.";
        return true;
        }
    else
        {
        CheckStr = "Failed Checksum.";
        return false;
        }

}

//==============================================================================
//  CLASS FmcStandardRomInfo::MultiRecordBase -- Rom Common Header
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::MultiRecordBase
//---------------------------------------------------------------------------

FmcStandardRomInfo::MultiRecordBase::MultiRecordBase(AdvI2CIdrom & rom)
    :   FmcRomAreaBase(rom),
        TypeID_1(0), FormatEOL_1(0x02), RecordLength_1(0),
        RecordChecksum_1(0), HeaderChecksum_1(0)
{
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordBase::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::MultiRecordBase::ToRomImage()
{
    unsigned int ofs = 0;
    WriteByte(ofs++, TypeID_1);
    WriteByte(ofs++, FormatEOL_1);
    WriteByte(ofs++, RecordLength_1);
    WriteByte(ofs++, RecordChecksum_1);

    CalculateChecksum(HeaderChecksum_1, 0, 4);  // don't include checksum word
    WriteByte(ofs++, HeaderChecksum_1);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordBase::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::MultiRecordBase::FromRomImage()
{
    unsigned int ofs = 0;
    ReadByte(ofs++, TypeID_1);
    ReadByte(ofs++, FormatEOL_1);
    ReadByte(ofs++, RecordLength_1);
    ReadByte(ofs++, RecordChecksum_1);
    ReadByte(ofs++, HeaderChecksum_1);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordBase::AreaCheck() -
//---------------------------------------------------------------------------

bool  FmcStandardRomInfo::MultiRecordBase::AreaCheck()
{
    unsigned int check(0);
    // Header Checksum
    CalculateChecksum(check, 0, 4);
    if (HeaderChecksum_1!=check)
        {
        CheckStr = "Failed Header Checksum.";
        return false;
        }

    //  Record Checksum
    CalculateChecksum(check, 5, RecordLength_1);
    if (RecordChecksum_1==check)
        {
        CheckStr = "Ok.";
        return true;
        }
    else
        {
        CheckStr = "Failed Record Checksum.";
        return false;
        }

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordBase::EoL_Marker() - set/clear EOL mark
//---------------------------------------------------------------------------

void FmcStandardRomInfo::MultiRecordBase::EoL_Marker(bool state)
{
    PtrBitManipulator  FEOL(&FormatEOL_1);
    FEOL.Bit(7, state);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordBase::EoL_Marker() - read EOL mark
//---------------------------------------------------------------------------

bool FmcStandardRomInfo::MultiRecordBase::EoL_Marker()
{
    PtrBitManipulator  FEOL(&FormatEOL_1);
    return FEOL.Bit(7);
}

//==============================================================================
//  CLASS FmcStandardRomInfo::DcOutputMultiRecord -- Rom Common Header
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::DcOutputMultiRecord
//---------------------------------------------------------------------------

FmcStandardRomInfo::DcOutputMultiRecord::DcOutputMultiRecord(AdvI2CIdrom & rom)
    :   MultiRecordBase(rom),
        OutInfo_1(0), NominalV_2(0), MaxNegVDev_2(0), MaxPosVDev_2(0),
        RipplePkPk_2(0), MinCurrent_2(0), MaxCurrent_2(0)
{
    TypeID_1 = 0x01;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcOutputMultiRecord::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::DcOutputMultiRecord::ToRomImage()
{
    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header
    WriteByte(ofs++, OutInfo_1);
    WriteMultiByte(ofs, NominalV_2,   2);  ofs += 2;
    WriteMultiByte(ofs, MaxNegVDev_2, 2);  ofs += 2;
    WriteMultiByte(ofs, MaxPosVDev_2, 2);  ofs += 2;
    WriteMultiByte(ofs, RipplePkPk_2, 2);  ofs += 2;
    WriteMultiByte(ofs, MinCurrent_2, 2);  ofs += 2;
    WriteMultiByte(ofs, MaxCurrent_2, 2);  ofs += 2;

    RecordLength_1 = ofs-base_ofs;
    CalculateChecksum(RecordChecksum_1, kHeaderSize, RecordLength_1);

    MultiRecordBase::ToRomImage();

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcOutputMultiRecord::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::DcOutputMultiRecord::FromRomImage()
{
    MultiRecordBase::FromRomImage();

    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    ReadByte(ofs++, OutInfo_1);
    ReadMultiByte(ofs, NominalV_2,   2);  ofs += 2;
    ReadMultiByte(ofs, MaxNegVDev_2, 2);  ofs += 2;
    ReadMultiByte(ofs, MaxPosVDev_2, 2);  ofs += 2;
    ReadMultiByte(ofs, RipplePkPk_2, 2);  ofs += 2;
    ReadMultiByte(ofs, MinCurrent_2, 2);  ofs += 2;
    ReadMultiByte(ofs, MaxCurrent_2, 2);  ofs += 2;

    Debug.Log("MultiRecordBase Dump");
    Debug.Log("...TypeID_1", TypeID_1, 16);
    Debug.Log("...FormatEOL_1", FormatEOL_1, 16);
    Debug.Log("...RecordLength_1", RecordLength_1, 16);
    Debug.Log("...RecordChecksum_1", RecordChecksum_1, 16);
    Debug.Log("...HeaderChecksum_1", HeaderChecksum_1, 16);
    Debug.Log("DcOutputMultiRecord Dump");
    Debug.Log("...OutInfo_1", OutInfo_1, 16);
    Debug.Log("...NominalV_2", NominalV_2, 16);
    Debug.Log("...MaxNegVDev_2", MaxNegVDev_2, 16);
    Debug.Log("...MaxPosVDev_2", MaxPosVDev_2, 16);
    Debug.Log("...RipplePkPk_2", RipplePkPk_2, 16);
    Debug.Log("...MinCurrent_2", MinCurrent_2, 16);
    Debug.Log("...MaxCurrent_2", MaxCurrent_2, 16);

    Debug.Log("...CheckSum_1", CheckSum_1, 16);

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcOutputMultiRecord::Standby() - read standby bit
//---------------------------------------------------------------------------

bool  FmcStandardRomInfo::DcOutputMultiRecord::Standby()
{
    PtrBitManipulator  OI(&OutInfo_1);
    return OI.Bit(7);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcOutputMultiRecord::OutputNumber() - read out number
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::DcOutputMultiRecord::OutputNumber()
{
    PtrBitManipulator  OI(&OutInfo_1);
    return OI.Field(0, 4);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcOutputMultiRecord::Standby() - set/clear Standby bit
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::DcOutputMultiRecord::Standby(bool state)
{
    PtrBitManipulator  OI(&OutInfo_1);
    OI.Bit(7, state);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcOutputMultiRecord::OutputNumber() - set Output #
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::DcOutputMultiRecord::OutputNumber(unsigned int number)
{
    PtrBitManipulator  OI(&OutInfo_1);
    OI.Field(0, 4, number);
}

//==============================================================================
//  CLASS FmcStandardRomInfo::DcLoadMultiRecord -- Rom Common Header
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::DcLoadMultiRecord
//---------------------------------------------------------------------------

FmcStandardRomInfo::DcLoadMultiRecord::DcLoadMultiRecord(AdvI2CIdrom & rom)
    :   MultiRecordBase(rom),
        VRequired_1(0), NominalV_2(0), SpecdMinV_2(0), SpecdMaxV_2(0),
        SpecdRipplePkPk_2(0), MinCurrentLoad_2(0), MaxCurrentLoad_2(0)
{
    TypeID_1 = 0x02;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcLoadMultiRecord::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::DcLoadMultiRecord::ToRomImage()
{
    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header
    WriteByte(ofs++, VRequired_1);
    WriteMultiByte(ofs, NominalV_2,        2);  ofs += 2;
    WriteMultiByte(ofs, SpecdMinV_2,       2);  ofs += 2;
    WriteMultiByte(ofs, SpecdMaxV_2,       2);  ofs += 2;
    WriteMultiByte(ofs, SpecdRipplePkPk_2, 2);  ofs += 2;
    WriteMultiByte(ofs, MinCurrentLoad_2,  2);  ofs += 2;
    WriteMultiByte(ofs, MaxCurrentLoad_2,  2);  ofs += 2;

    RecordLength_1 = ofs-base_ofs;
    CalculateChecksum(RecordChecksum_1, kHeaderSize, RecordLength_1);

    MultiRecordBase::ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcLoadMultiRecord::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::DcLoadMultiRecord::FromRomImage()
{
    MultiRecordBase::FromRomImage();

    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    ReadByte(ofs++, VRequired_1);
    ReadMultiByte(ofs, NominalV_2,        2);  ofs += 2;
    ReadMultiByte(ofs, SpecdMinV_2,       2);  ofs += 2;
    ReadMultiByte(ofs, SpecdMaxV_2,       2);  ofs += 2;
    ReadMultiByte(ofs, SpecdRipplePkPk_2, 2);  ofs += 2;
    ReadMultiByte(ofs, MinCurrentLoad_2,  2);  ofs += 2;
    ReadMultiByte(ofs, MaxCurrentLoad_2,  2);  ofs += 2;

    Debug.Log("MultiRecordBase Dump");
    Debug.Log("...TypeID_1", TypeID_1, 16);
    Debug.Log("...FormatEOL_1", FormatEOL_1, 16);
    Debug.Log("...RecordLength_1", RecordLength_1, 16);
    Debug.Log("...RecordChecksum_1", RecordChecksum_1, 16);
    Debug.Log("...HeaderChecksum_1", HeaderChecksum_1, 16);
    Debug.Log("DcLoadMultiRecord Dump");
    Debug.Log("...VRequired_1", VRequired_1, 16);
    Debug.Log("...NominalV_2", NominalV_2, 16);
    Debug.Log("...SpecdMinV_2", SpecdMinV_2, 16);
    Debug.Log("...SpecdMaxV_2", SpecdMaxV_2, 16);
    Debug.Log("...SpecdRipplePkPk_2", SpecdRipplePkPk_2, 16);
    Debug.Log("...MinCurrentLoad_2", MinCurrentLoad_2, 16);
    Debug.Log("...MaxCurrentLoad_2", MaxCurrentLoad_2, 16);

    Debug.Log("...CheckSum_1", CheckSum_1, 16);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcLoadMultiRecord::OutputNumber() - read out number
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::DcLoadMultiRecord::OutputNumber()
{
    PtrBitManipulator  OI(&VRequired_1);
    return OI.Field(0, 4);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::DcLoadMultiRecord::OutputNumber() - set Output #
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::DcLoadMultiRecord::OutputNumber(unsigned int number)
{
    PtrBitManipulator  OI(&VRequired_1);
    OI.Field(0, 4, number);
}

//==============================================================================
//  CLASS FmcStandardRomInfo::ScratchMultiRecord -- OEM FMC DEF
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::ScratchMultiRecord
//---------------------------------------------------------------------------

FmcStandardRomInfo::ScratchMultiRecord::ScratchMultiRecord(AdvI2CIdrom & rom)
    :   MultiRecordBase(rom),
        ManufacturerID_3(0x0012A2), SubT_Ver_1(0)
{
    TypeID_1 = 0xFA;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::ScratchMultiRecord::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::ScratchMultiRecord::ToRomImage()
{
    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header
    WriteMultiByte(ofs, ManufacturerID_3, 3);  ofs += 3;
    WriteByte(ofs++, SubT_Ver_1);

    RecordLength_1 = ofs-base_ofs;
    CalculateChecksum(RecordChecksum_1, kHeaderSize, RecordLength_1);

    MultiRecordBase::ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::ScratchMultiRecord::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::ScratchMultiRecord::FromRomImage()
{
    MultiRecordBase::FromRomImage();

    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    ReadMultiByte(ofs, ManufacturerID_3, 3);  ofs += 3;
    ReadByte(ofs++, SubT_Ver_1);

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::ScratchMultiRecord::SubType() - read param
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::ScratchMultiRecord::SubType()
{
    PtrBitManipulator  OI(&SubT_Ver_1);
    return OI.Field(4, 4);
}

//==============================================================================
//  CLASS FmcStandardRomInfo::FmcDefinitionMultiRecord -- OEM FMC DEF
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::FmcDefinitionMultiRecord
//---------------------------------------------------------------------------

FmcStandardRomInfo::FmcDefinitionMultiRecord::FmcDefinitionMultiRecord(AdvI2CIdrom & rom)
    :   MultiRecordBase(rom),
        ManufacturerID_3(0x0012A2), SubT_Ver_1(0), ModuleInfo_1(0),
        P1a_Signals_1(0), P1b_Signals_1(0),
        P2a_Signals_1(0), P2b_Signals_1(0),
        Transceiver_1(0), MaxClock_1(0)
{
    TypeID_1 = 0xFA;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcDefinitionMultiRecord::ToRomImage()
{
    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header
    WriteMultiByte(ofs, ManufacturerID_3, 3);  ofs += 3;
    WriteByte(ofs++, SubT_Ver_1);
    WriteByte(ofs++, ModuleInfo_1);
    WriteByte(ofs++, P1a_Signals_1);
    WriteByte(ofs++, P1b_Signals_1);
    WriteByte(ofs++, P2a_Signals_1);
    WriteByte(ofs++, P2b_Signals_1);
    WriteByte(ofs++, Transceiver_1);
    WriteByte(ofs++, MaxClock_1);

    RecordLength_1 = ofs-base_ofs;
    CalculateChecksum(RecordChecksum_1, kHeaderSize, RecordLength_1);

    MultiRecordBase::ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::FmcDefinitionMultiRecord::FromRomImage()
{
    MultiRecordBase::FromRomImage();

    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    ReadMultiByte(ofs, ManufacturerID_3, 3);  ofs += 3;
    ReadByte(ofs++, SubT_Ver_1);
    ReadByte(ofs++, ModuleInfo_1);
    ReadByte(ofs++, P1a_Signals_1);
    ReadByte(ofs++, P1b_Signals_1);
    ReadByte(ofs++, P2a_Signals_1);
    ReadByte(ofs++, P2b_Signals_1);
    ReadByte(ofs++, Transceiver_1);
    ReadByte(ofs++, MaxClock_1);

    Debug.Log("MultiRecordBase Dump");
    Debug.Log("...TypeID_1", TypeID_1, 16);
    Debug.Log("...FormatEOL_1", FormatEOL_1, 16);
    Debug.Log("...RecordLength_1", RecordLength_1, 16);
    Debug.Log("...RecordChecksum_1", RecordChecksum_1, 16);
    Debug.Log("...HeaderChecksum_1", HeaderChecksum_1, 16);
    Debug.Log("FmcDefinitionMultiRecord Dump");
    Debug.Log("...ManufacturerID_3", ManufacturerID_3, 16);
    Debug.Log("...SubT_Ver_1", SubT_Ver_1, 16);
    Debug.Log("...P1a_Signals_1", P1a_Signals_1, 16);
    Debug.Log("...P1b_Signals_1", P1b_Signals_1, 16);
    Debug.Log("...P2a_Signals_1", P2a_Signals_1, 16);
    Debug.Log("...P2b_Signals_1", P2b_Signals_1, 16);
    Debug.Log("...Transceiver_1", Transceiver_1, 16);
    Debug.Log("...MaxClock_1", MaxClock_1, 16);

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::SubType() - read param
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::FmcDefinitionMultiRecord::SubType()
{
    PtrBitManipulator  OI(&SubT_Ver_1);
    return OI.Field(4, 4);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::Version() - read param
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::FmcDefinitionMultiRecord::Version()
{
    PtrBitManipulator  OI(&SubT_Ver_1);
    return OI.Field(0, 4);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ModuleSize() - read param
//---------------------------------------------------------------------------

Fmc::FmcModuleSize  FmcStandardRomInfo::FmcDefinitionMultiRecord::ModuleSize()
{
    PtrBitManipulator  OI(&ModuleInfo_1);
    return (OI.Field(6, 2)==0x1) ? Fmc::msDoubleWide : Fmc::msSingleWide ;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ConnectorSize() - read param
//---------------------------------------------------------------------------

Fmc::FmcConnectorSize  FmcStandardRomInfo::FmcDefinitionMultiRecord::ConnectorSize(Fmc::FmcConnector p)
{
    PtrBitManipulator  OI(&ModuleInfo_1);
    unsigned int field_start(0);
    if (p==Fmc::fmcP1)
        field_start = 4;
    else
        field_start = 2;

    return static_cast<Fmc::FmcConnectorSize>(OI.Field(field_start, 2));
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ClockDirection() - read param
//---------------------------------------------------------------------------

Fmc::FmcClockDirection  FmcStandardRomInfo::FmcDefinitionMultiRecord::ClockDirection()
{
    PtrBitManipulator  OI(&ModuleInfo_1);
    return (OI.Bit(1)) ? Fmc::dirCarrierToMezz : Fmc::dirMezzToCarrier ;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::BankSignalCount() - read param
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::FmcDefinitionMultiRecord::BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank)
{
    if (p==Fmc::fmcP1 && bank == Fmc::bankA)
        return P1a_Signals_1;
    if (p==Fmc::fmcP1 && bank == Fmc::bankB)
        return P1b_Signals_1;
    if (p==Fmc::fmcP2 && bank == Fmc::bankA)
        return P2a_Signals_1;
    if (p==Fmc::fmcP2 && bank == Fmc::bankB)
        return P2b_Signals_1;
    else
        return P1a_Signals_1;

}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::TransceiverCount() - read param
//---------------------------------------------------------------------------

unsigned int FmcStandardRomInfo::FmcDefinitionMultiRecord::TransceiverCount(Fmc::FmcConnector p)
{
    PtrBitManipulator  OI(&Transceiver_1);
    unsigned int field_start(0);
    if (p==Fmc::fmcP1)
        field_start = 4;
    else
        field_start = 0;

    return OI.Field(field_start, 4);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ModuleSize() - write param
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcDefinitionMultiRecord::ModuleSize(Fmc::FmcModuleSize size)
{
    PtrBitManipulator  OI(&ModuleInfo_1);
    OI.Field(6, 2, static_cast<unsigned int>(size));
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ConnectorSize() - read param
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcDefinitionMultiRecord::ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size)
{
    PtrBitManipulator  OI(&ModuleInfo_1);
    unsigned int field_start(0);
    if (p==Fmc::fmcP1)
        field_start = 4;
    else
        field_start = 2;

    OI.Field(field_start, 2, static_cast<unsigned int>(size));
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::ClockDirection() - read param
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcDefinitionMultiRecord::ClockDirection(Fmc::FmcClockDirection dir)
{
    PtrBitManipulator  OI(&ModuleInfo_1);
    OI.Bit(1, (dir==Fmc::dirCarrierToMezz) ? true : false );
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::BankSignalCount() - read param
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcDefinitionMultiRecord::BankSignalCount(Fmc::FmcConnector p,
                                    Fmc::FmcConnectorBank bank, unsigned int count)
{
    if (p==Fmc::fmcP1 && bank == Fmc::bankA)
        P1a_Signals_1 = count;
    if (p==Fmc::fmcP1 && bank == Fmc::bankB)
        P1b_Signals_1 = count;
    if (p==Fmc::fmcP2 && bank == Fmc::bankA)
        P2a_Signals_1 = count;
    if (p==Fmc::fmcP2 && bank == Fmc::bankB)
        P2b_Signals_1 = count;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::FmcDefinitionMultiRecord::TransceiverCount() - read param
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::FmcDefinitionMultiRecord::TransceiverCount(Fmc::FmcConnector p, unsigned int count)
{
    PtrBitManipulator  OI(&Transceiver_1);
    unsigned int field_start(0);
    if (p==Fmc::fmcP1)
        field_start = 4;
    else
        field_start = 0;

    OI.Field(field_start, 4, count);
}

//==============================================================================
//  CLASS FmcStandardRomInfo::OEMMultiRecord  -- OEM MultiRecord
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::OEMMultiRecord
//---------------------------------------------------------------------------

FmcStandardRomInfo::OEMMultiRecord::OEMMultiRecord(AdvI2CIdrom & rom)
    :   MultiRecordBase(rom), ManufacturerID_3(0x0012A2)//,
{
    TypeID_1 = 0xC0;  // 0xC0-FF

    Resize(4);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::OEMMultiRecord::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------
//  Pack bytes into full word image...

void  FmcStandardRomInfo::OEMMultiRecord::ToRomImage()
{
    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header
    WriteMultiByte(ofs, ManufacturerID_3, 3);  ofs += 3;

    // Derived Classes can load more parameters here
    ToRawImage_Custom(ofs);

    for (size_t i=0; i<Raw_n.size(); i++)
        {
        for (int byte=0; byte<4; ++byte)
            {
            //
            //  Extract byte value from word
            int byte_value = (Raw_n[i] >> (8*byte)) & 0xff ;
            WriteByte(ofs++, byte_value);
            }
        }

    RecordLength_1 = ofs-base_ofs;
    CalculateChecksum(RecordChecksum_1, kHeaderSize, RecordLength_1);

    // custom parameter trace dumps
    ToRawImage_CustomDebug();

    MultiRecordBase::ToRomImage();

//    Debug.Log("...Raw Words" );
//    for (size_t i=0; i<Raw_n.size(); i++)
//        {
//        Debug.Log(".......", Raw_n[i], 16);
//        }

//    Debug.Log("...size", Raw_n.size(), 16);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::OEMMultiRecord::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------
//  Pack bytes into full word image...

void FmcStandardRomInfo::OEMMultiRecord::FromRomImage()
{
    MultiRecordBase::FromRomImage();

    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    ReadMultiByte(ofs, ManufacturerID_3, 3);  ofs += 3;

    // Derived Classes can load more parameters here
    FromRawImage_Custom(ofs);

    unsigned int total_bytes = RecordLength_1-(ofs-base_ofs);
    Raw_n.resize(total_bytes/sizeof(unsigned int));
    for (size_t i=0; i<Raw_n.size(); i++)
        {
        Raw_n[i] = 0;
        for (int byte=0; byte<4; ++byte)
            {
            unsigned int byte_value(0);
            ReadByte(ofs++, byte_value);
            //
            //  Assemble word from bytes
            Raw_n[i] |= (byte_value << (8*byte));
            }
        }

    Debug.Log("MultiRecordBase Dump");
    Debug.Log("...TypeID_1", TypeID_1, 16);
    Debug.Log("...FormatEOL_1", FormatEOL_1, 16);
    Debug.Log("...RecordLength_1", RecordLength_1, 16);
    Debug.Log("...RecordChecksum_1", RecordChecksum_1, 16);
    Debug.Log("...HeaderChecksum_1", HeaderChecksum_1, 16);
    Debug.Log("OEMMultiRecord Dump");
    Debug.Log("...ManufacturerID_3", ManufacturerID_3, 16);

    // custom parameter trace dumps
    FromRawImage_CustomDebug();

    Debug.Log("...Raw Words" );
    for (size_t i=0; i<Raw_n.size(); i++)
        {
        Debug.Log(".......", Raw_n[i], 16);
        }

    Debug.Log("...total_bytes", total_bytes, 16);
    Debug.Log("...size", Raw_n.size(), 16);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::OEMMultiRecord::ToRawImage_Custom()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::OEMMultiRecord::ToRawImage_Custom(unsigned int & /*ofs*/)
{
    // none yet
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::OEMMultiRecord::ToRawImage_CustomDebug()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::OEMMultiRecord::ToRawImage_CustomDebug()
{
    // none yet
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::OEMMultiRecord::FromRawImage_Custom()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::OEMMultiRecord::FromRawImage_Custom(unsigned int & /*ofs*/)
{
    // none yet
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::OEMMultiRecord::FromRawImage_CustomDebug()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::OEMMultiRecord::FromRawImage_CustomDebug()
{
    // none yet
}

//==============================================================================
//  CLASS FmcStandardRomInfo::CalCoefficientsMR  -- OEM MultiRecord
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::CalCoefficientsMR
//---------------------------------------------------------------------------

FmcStandardRomInfo::CalCoefficientsMR::CalCoefficientsMR(AdvI2CIdrom & rom)
    :   OEMMultiRecord(rom), IdromIoDG(this->IView())
{
    // set TypeID_1 to unique code in most derived class...
}

//==============================================================================
//  CLASS FmcStandardRomInfo::AdcCalCoefficientsMR  -- OEM MultiRecord
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::AdcCalCoefficientsMR
//---------------------------------------------------------------------------

FmcStandardRomInfo::AdcCalCoefficientsMR::AdcCalCoefficientsMR(AdvI2CIdrom & rom)
    :   CalCoefficientsMR(rom)
{
    TypeID_1 = 0xE0;  // 0xC0-FF
}

//==============================================================================
//  CLASS FmcStandardRomInfo::DacCalCoefficientsMR  -- OEM MultiRecord
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::DacCalCoefficientsMR
//---------------------------------------------------------------------------

FmcStandardRomInfo::DacCalCoefficientsMR::DacCalCoefficientsMR(AdvI2CIdrom & rom)
    :   CalCoefficientsMR(rom)
{
    TypeID_1 = 0xE1;  // 0xC0-FF
}

//==============================================================================
//  CLASS FmcStandardRomInfo::AdcRangedCalCoefficientsMR  -- OEM MultiRecord
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::AdcRangedCalCoefficientsMR
//---------------------------------------------------------------------------

FmcStandardRomInfo::AdcRangedCalCoefficientsMR::AdcRangedCalCoefficientsMR(AdvI2CIdrom & rom, unsigned int idx, unsigned int total, unsigned int channels)
    :   OEMMultiRecord(rom), IdromIoDG(this->IView()),
        CalRangeIndex_1(idx), CalRangeTotal_1(total)
{
    TypeID_1 = 0xE2;  // 0xC0-FF

    Resize(channels*2+1);  // 2 words per channel, plus calibrated word
    Zero();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::ToRawImage_Custom()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::ToRawImage_Custom(unsigned int & ofs)
{
    WriteByte(ofs, CalRangeIndex_1);  ofs += 1;
    WriteByte(ofs, CalRangeTotal_1);  ofs += 1;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::ToRawImage_CustomDebug()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::ToRawImage_CustomDebug()
{
    // none yet
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::FromRawImage_Custom()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::FromRawImage_Custom(unsigned int & ofs)
{
    ReadByte(ofs, CalRangeIndex_1);  ofs += 1;
    ReadByte(ofs, CalRangeTotal_1);  ofs += 1;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::FromRawImage_CustomDebug()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::AdcRangedCalCoefficientsMR::FromRawImage_CustomDebug()
{
    Debug.Log("...CalRangeIndex_1", CalRangeIndex_1, 16);
    Debug.Log("...CalRangeTotal_1", CalRangeTotal_1, 16);
}

//==============================================================================
//  CLASS FmcStandardRomInfo::UnknownMultiRecord  -- ??? MultiRecord
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::UnknownMultiRecord
//---------------------------------------------------------------------------

FmcStandardRomInfo::UnknownMultiRecord::UnknownMultiRecord(AdvI2CIdrom & rom)
    :   MultiRecordBase(rom)
{
    TypeID_1 = 0x0;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::UnknownMultiRecord::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::UnknownMultiRecord::ToRomImage()
{
    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    for (size_t i=0; i<Raw_n.size(); i++)
        {
        WriteByte(ofs++, Raw_n[i]);
        }

    RecordLength_1 = ofs-base_ofs;
    CalculateChecksum(RecordChecksum_1, kHeaderSize, RecordLength_1);

    MultiRecordBase::ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::UnknownMultiRecord::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::UnknownMultiRecord::FromRomImage()
{
    MultiRecordBase::FromRomImage();

    unsigned int base_ofs = kHeaderSize;  // skip over common header
    unsigned int ofs = base_ofs;  // skip over common header

    Raw_n.resize(RecordLength_1);
    for (size_t i=0; i<Raw_n.size(); i++)
        {
        ReadByte(ofs++, Raw_n[i]);
        }
}

//==============================================================================
//  CLASS FmcStandardRomInfo::MultiRecordArea -- Rom Common Header
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class  FmcStandardRomInfo::MultiRecordArea
//---------------------------------------------------------------------------

FmcStandardRomInfo::MultiRecordArea::MultiRecordArea(AdvI2CIdrom & rom)
    :   FmcRomAreaBase(rom), FmcDef_ptr(0), AdcCal_ptr(0), DacCal_ptr(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class  FmcStandardRomInfo::MultiRecordArea
//---------------------------------------------------------------------------

FmcStandardRomInfo::MultiRecordArea::~MultiRecordArea()
{
    Clear();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AddAdcCalRecord() - add AdcCal
//---------------------------------------------------------------------------

void FmcStandardRomInfo::MultiRecordArea::AddAdcCalRecord()
{
    AdcCalCoefficientsMR *  AdcCal = new AdcCalCoefficientsMR(ROM);
    AddRecord(AdcCal);
    AdcCal_ptr = AdcCal;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AddDacCalRecord() - add DacCal
//---------------------------------------------------------------------------

void FmcStandardRomInfo::MultiRecordArea::AddDacCalRecord()
{
    DacCalCoefficientsMR *  DacCal = new DacCalCoefficientsMR(ROM);
    AddRecord(DacCal);
    DacCal_ptr = DacCal;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AddAdcRangedCalRecord() - add ranged page
//---------------------------------------------------------------------------

void FmcStandardRomInfo::MultiRecordArea::AddAdcRangedCalRecord(size_t idx, size_t total, size_t channels)
{
    AdcRangedCalCoefficientsMR *  AdcRngCal = new AdcRangedCalCoefficientsMR(ROM, static_cast<unsigned int>(idx),
                                                                                  static_cast<unsigned int>(total),
                                                                                  static_cast<unsigned int>(channels));
    AddRecord(AdcRngCal);
    //  size ranged cal vector, insert element in proper place
    if (AdcRngCal_vec.size()<total)
        AdcRngCal_vec.resize(total);
    AdcRngCal_vec[idx] = AdcRngCal;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::FmcDef()
//---------------------------------------------------------------------------

FmcStandardRomInfo::FmcDefinitionMultiRecord &
                        FmcStandardRomInfo::MultiRecordArea::FmcDef()
{
    if (FmcDef_ptr == 0)
        {
        //  if not created yet, add it
        FmcDef_ptr = new FmcDefinitionMultiRecord(ROM);
        AddRecord(FmcDef_ptr);
        Debug.Log("**** Added a FMC Definition Record  ****");
        }

    return *FmcDef_ptr;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AdcCal()
//---------------------------------------------------------------------------

FmcStandardRomInfo::AdcCalCoefficientsMR &
                        FmcStandardRomInfo::MultiRecordArea::AdcCal()
{
    if (AdcCal_ptr == 0)
        {
        //  if not created yet, add it
        AdcCal_ptr = new AdcCalCoefficientsMR(ROM);
        AddRecord(AdcCal_ptr);
        Debug.Log("**** Added a AdcCal Record  ****");
        }

    return *AdcCal_ptr;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::DacCal()
//---------------------------------------------------------------------------

FmcStandardRomInfo::DacCalCoefficientsMR &
                        FmcStandardRomInfo::MultiRecordArea::DacCal()
{
    if (DacCal_ptr == 0)
        {
        //  if not created yet, add it
        DacCal_ptr = new DacCalCoefficientsMR(ROM);
        AddRecord(DacCal_ptr);
        Debug.Log("**** Added a DacCal Record  ****");
        }

    return *DacCal_ptr;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AdcRangedCal()
//---------------------------------------------------------------------------

FmcStandardRomInfo::AdcRangedCalCoefficientsMR &
                        FmcStandardRomInfo::MultiRecordArea::AdcRangedCal(size_t idx)
{
    if (AdcRanges()<idx)
        {
        size_t newTotal = idx+1;
        size_t current = AdcRanges();
        for (size_t i=0; i<newTotal-current; i++)
            {
            //  if not created yet, add it
            AddAdcRangedCalRecord(current+i, newTotal, 4);   // what is the correct channel count?
            Debug.Log("**** Added a Adc Ranged Cal Record  ****");
            }
        }

    return *(AdcRngCal_vec[idx]);
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::Clear()
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::MultiRecordArea::Clear()
{
    FmcDef_ptr = 0;   AdcCal_ptr =0;  DacCal_ptr = 0;
    for (size_t i=0; i<Records.size(); i++)
        {
        delete Records[i];
        }
    Records.clear();
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::ToRomImage() - pack data to ROM image
//---------------------------------------------------------------------------

void  FmcStandardRomInfo::MultiRecordArea::ToRomImage()
{
    ScopedTraceRef(Debug, "FmcStandardRomInfo::MultiRecordArea::ToRomImage()");
    for (size_t i=0; i<Records.size(); i++)
        {
        Debug.Log("Saved Record = ", Records[i]->TypeID(), 16);
        Records[i]->BaseOffsetBytes(Base+CumulativeSizeBytes(i));
        Records[i]->EoL_Marker(i==(Records.size()-1));  // no EOL mark except on end
        Records[i]->ToRomImage();
        }
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::FromRomImage() - unpack data from ROM image
//---------------------------------------------------------------------------

void FmcStandardRomInfo::MultiRecordArea::FromRomImage()
{
    ScopedTraceRef(Debug, "FmcStandardRomInfo::MultiRecordArea::FromRomImage()");

    //  Clear out current record
    Clear();
    
    //  Create a 'scratch' record for parsing
    FmcStandardRomInfo::ScratchMultiRecord TMR(ROM);
    TMR.BaseOffsetBytes(BaseOffsetBytes());
    TMR.FromRomImage();

    while (1==1)
        {
        switch(TMR.TypeID())
            {
            case 0x1:     // DC Output
                {
                DcOutputMultiRecord *  DcOut = new DcOutputMultiRecord(ROM);
                AddRecord(DcOut);

                DcOut->BaseOffsetBytes(TMR.BaseOffsetBytes());
                DcOut->FromRomImage();

                std::stringstream msg;
                msg << "DCOUT: " << DcOut->OutputNumber() << " V: " << DcOut->NominalV();
                Debug.Log(msg.str());
                }
                break;
            case 0x2:     // DC Load
                {
                DcLoadMultiRecord *  DcLoad = new DcLoadMultiRecord(ROM);
                AddRecord(DcLoad);
                DcLoad->BaseOffsetBytes(TMR.BaseOffsetBytes());
                DcLoad->FromRomImage();

                std::stringstream msg;
                msg << "DCLOAD: " << DcLoad->OutputNumber() << " V: " << DcLoad->NominalV();
                Debug.Log(msg.str());
                }
                break;
            case 0xFA:     // Fmc Definition Record
                {
                if (TMR.ManufacturerID()==0x0012A2 && TMR.SubType()==0)
                    {
                    FmcDefinitionMultiRecord *  FmcDef = new FmcDefinitionMultiRecord(ROM);
                    AddRecord(FmcDef);
                    FmcDef_ptr = FmcDef;
                    FmcDef->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    FmcDef->FromRomImage();
                    }
                else
                    {  // make a dummy record
                    UnknownMultiRecord * Unk = new UnknownMultiRecord(ROM);
                    AddRecord(Unk);
                    Unk->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    Unk->FromRomImage();

                    std::stringstream msg;
                    msg << "UNKNOWN: Type " << Unk->TypeID() ;
                    Debug.Log(msg.str());
                    }
                }
                break;
            case 0xE0:     // Adc Calibration Record
                {
                if (TMR.ManufacturerID()==0x0012A2)
                    {
                    AdcCalCoefficientsMR *  AdcCal = new AdcCalCoefficientsMR(ROM);
                    AddRecord(AdcCal);
                    AdcCal_ptr = AdcCal;
                    AdcCal->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    AdcCal->FromRomImage();
                    }
                else
                    {  // make a dummy record
                    UnknownMultiRecord * Unk = new UnknownMultiRecord(ROM);
                    AddRecord(Unk);
                    Unk->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    Unk->FromRomImage();

                    std::stringstream msg;
                    msg << "UNKNOWN: Type " << Unk->TypeID() ;
                    Debug.Log(msg.str());
                    }
                }
                break;
            case 0xE1:     // Dac Calibration Record
                {
                if (TMR.ManufacturerID()==0x0012A2)
                    {
                    DacCalCoefficientsMR *  DacCal = new DacCalCoefficientsMR(ROM);
                    AddRecord(DacCal);
                    DacCal_ptr = DacCal;
                    DacCal->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    DacCal->FromRomImage();
                    }
                else
                    {  // make a dummy record
                    UnknownMultiRecord * Unk = new UnknownMultiRecord(ROM);
                    AddRecord(Unk);
                    Unk->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    Unk->FromRomImage();

                    std::stringstream msg;
                    msg << "UNKNOWN: Type " << Unk->TypeID() ;
                    Debug.Log(msg.str());
                    }
                }
                break;
            case 0xE2:     // Adc Ranged Calibration Record
                {
                if (TMR.ManufacturerID()==0x0012A2)
                    {
                    AdcRangedCalCoefficientsMR *  RngAdcCal = new AdcRangedCalCoefficientsMR(ROM, 0, 0, 0);
                    RngAdcCal->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    RngAdcCal->FromRomImage();
                    //  vet the arguments
                    if (RngAdcCal->Validate()==false)
                        {
                        Debug.Log("Invalid Ranged Cal Data, skipping");
                        delete RngAdcCal;
                        break;
                        }
                    // If valid, add the record
                    AddRecord(RngAdcCal);
                    AdcRngCal_vec.resize(RngAdcCal->CalRangeTotal());
                    AdcRngCal_vec[RngAdcCal->CalRangeIndex()] = RngAdcCal;
                    }
                else
                    {  // make a dummy record
                    UnknownMultiRecord * Unk = new UnknownMultiRecord(ROM);
                    AddRecord(Unk);
                    Unk->BaseOffsetBytes(TMR.BaseOffsetBytes());
                    Unk->FromRomImage();

                    std::stringstream msg;
                    msg << "UNKNOWN: Type " << Unk->TypeID() ;
                    Debug.Log(msg.str());
                    }
                }
                break;
            default:
                {  // make a dummy record
                if (TMR.AreaSizeBytes() <= 5)
                    {
                    //  If no data, terminate it.
                    TMR.EoL_Marker(true);
                    Record(RecordCount()-1)->EoL_Marker(true);
                    break;
                    }
                //
                //  Record actually has data in it
                FmcStandardRomInfo::UnknownMultiRecord * Unk = new UnknownMultiRecord(ROM);
                AddRecord(Unk);
                Unk->BaseOffsetBytes(TMR.BaseOffsetBytes());
                Unk->FromRomImage();

                std::stringstream msg;
                msg << "UNKNOWN: Type " << Unk->TypeID() ;
                Debug.Log(msg.str());
                }
                break;
            }
        // if last record was the final, break out
        if (TMR.EoL_Marker())
            break;
        //
        //  next record
        unsigned int end = TMR.EndBytes();
        TMR.BaseOffsetBytes(end);
        TMR.FromRomImage();
        }
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AreaCheck() -
//---------------------------------------------------------------------------

bool  FmcStandardRomInfo::MultiRecordArea::AreaCheck()
{
    for (size_t i=0; i<Records.size(); i++)
        if (Records[i]->AreaCheck()==false)
            {
            CheckStr = "Record ";
            CheckStr += IntToString(static_cast<int>(i));
            CheckStr += " ";
            CheckStr += Records[i]->AreaCheckResult();
            return false;
            }

    // no failures
    CheckStr = "Ok.";
    return true;
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::AreaSizeBytes() - Size of MR Area
//---------------------------------------------------------------------------

unsigned int  FmcStandardRomInfo::MultiRecordArea::AreaSizeBytes()
{
    return CumulativeSizeBytes(Records.size());
}

//---------------------------------------------------------------------------
//  FmcStandardRomInfo::MultiRecordArea::CumulativeSizeBytes()
//---------------------------------------------------------------------------
//  Counts size of multi-records before the idx-th one

unsigned int  FmcStandardRomInfo::MultiRecordArea::CumulativeSizeBytes(size_t idx)
{
    unsigned int bytes(0);
    for (size_t i=0; i<idx; i++)
        bytes += Records[i]->AreaSizeBytes();

    return bytes;
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

}  // namespace Innovative

