// FmcSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FmcSupport_MbH
#define FmcSupport_MbH

#include <CDCE62005_Mb.h>
#include <LMK04800_Mb.h>
#include <LMK04820_Mb.h>
#include <Ad9510Duo_Tb_Mb.h>
#include <Ficl/FiclSystem_Mb.h>
#include <Trace_Mb.h>
#include <IdromDatagram_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IFmcDaughterCard  -- FMC Daughter Card Interface
//==============================================================================

class IFmcDaughterCard
{
public:
    enum DaughterCardStatusStates
    {  dsOk, dsCardPoweredDown, dsNoSoftwareObjectPresent, dsNoCardPresent,
         dsCardPowerFailure, dsLogicPowerMismatch, dsPowerForcedOff, dsCarrierVadjPowerFailure,
    };

    virtual DaughterCardStatusStates  BoardStatus()  {  return FBoardStatus;  }

    virtual void    ConstructCustomParts() = 0;    // call in most derived ctor
    virtual void    InitCardPower() = 0;
    virtual void    OpenHardware() = 0;
    virtual void    PreconfigureHardware() = 0;
    virtual void    ConfigureHardware() = 0;
    virtual void    StreamStopHardware() = 0;
    virtual void    CloseHardware() = 0;
    virtual void    RemoveCardPower() = 0;

    //  Customization calls
    virtual void    Pex7Cop_PrePowerCustomization() = 0;

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine) = 0;

    virtual int     Do_DFetch(int dev, int addr) = 0;
    virtual void    Do_DStore(int dev, int addr, int value) = 0;
    virtual void    Do_DFreq(int dev, float freq) = 0;
    virtual float   Do_DFreqActual(int dev) = 0;

    IFmcDaughterCard()  : Owner(0), FBoardStatus(dsCardPoweredDown)
        {}
    virtual ~IFmcDaughterCard()
        {}

    //  Memory Spaces
    AddressingSpace   PortMemory;
    AddressingSpace   LogicMemory;

    ii32   PeekLogic_Carrier(ii32 offset)
                {  return LogicMemory.At(offset);  }
    void   PokeLogic_Carrier(ii32 offset, ii32 data)
                {  LogicMemory.At(offset, data);  }


protected:
    class FmcCarrierBoard *   Owner;
    DaughterCardStatusStates  FBoardStatus;

    void BoardStatus(DaughterCardStatusStates status)
        {  FBoardStatus = status;  }

};

//==============================================================================
//  CLASS SafeFmcDaughterCardImpl  -- "Null" Daughter Card
//==============================================================================

class SafeFmcDaughterCardImpl : public IFmcDaughterCard
{
public:

    virtual DaughterCardStatusStates  BoardStatus()  {  return dsNoSoftwareObjectPresent;  }

    virtual void    ConstructCustomParts() { X(); }
    virtual void    InitCardPower() { X(); }
    virtual void    OpenHardware() { X(); }
    virtual void    PreconfigureHardware()  { X(); }
    virtual void    ConfigureHardware() { X(); }
    virtual void    StreamStopHardware() { X(); }
    virtual void    CloseHardware() { X(); }
    virtual void    RemoveCardPower() { X(); }

    //  Customization calls
    virtual void    Pex7Cop_PrePowerCustomization() { X(); }
    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & /*engine*/) { X(); }

    virtual int     Do_DFetch(int /*dev*/, int /*addr*/) { X(); return 0; }
    virtual void    Do_DStore(int /*dev*/, int /*addr*/, int /*value*/) { X(); }
    virtual void    Do_DFreq(int /*dev*/, float /*freq*/) { X(); }
    virtual float   Do_DFreqActual(int /*dev*/) { X(); return 0.0f; }

private:
    Trace  Debug;

    void X()
        {  Debug.Log("INVALID CALL to unconnected FMC Daughter...: ");  }

};


//==============================================================================
//  CLASS IFmcInformation  -- FMC Daughter Card Information
//==============================================================================

class IFmcInformation
{
public:
    virtual ~IFmcInformation()
        {}

    //  Methods
    virtual bool         FmcPresent() = 0;
    virtual bool         FmcVadjEnable() = 0;
    virtual void         FmcVadjEnable(bool enable) = 0;
    virtual bool         FmcVadjForced() = 0;
    virtual bool         FmcVadjPower() = 0;
    //virtual void         FmcVadjLevel(short level) = 0;
    virtual short        FmcVadjLevel() = 0;
    virtual bool         FmcM2CPower() = 0;
    virtual bool         FmcC2MPower() = 0;

    virtual unsigned int FmcBoardId() = 0;

    virtual bool         FmcBidirClockC2M() = 0;
    virtual void         FmcClock2_3_Enable(bool state) = 0;
    virtual bool         FmcClock2_3_Enable() = 0;

    virtual void         FmcClock2_IsSingleEnded(bool state) = 0;
    virtual bool         FmcClock2_IsSingleEnded() = 0;

    virtual void         FmcClock3_IsSingleEnded(bool state) = 0;
    virtual bool         FmcClock3_IsSingleEnded() = 0;

    virtual void         Pex7_FmcClock2_IsFpga(bool state) = 0;
    virtual bool         Pex7_FmcClock2_IsFpga() = 0;
    virtual void         Pex7_FmcClock3_IsFpga(bool state) = 0;
    virtual bool         Pex7_FmcClock3_IsFpga() = 0;

protected:

};

//===========================================================================
//  CLASS IFmcIoDevice  -- Common FmcBoard Configuration Interface
//===========================================================================

class IFmcIoDevice
{
public:
    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essCarrier, essP16=essCarrier };

    virtual ~IFmcIoDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IFmcIoDevice::ITriggerCfg  -- Trigger Configuration Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ITriggerCfg
    {
    public:
        virtual ~ITriggerCfg()  {  }

        virtual void          FramedMode(bool state) = 0;
        virtual bool          FramedMode() const = 0;

        virtual void          Edge(bool state) = 0;
        virtual bool          Edge() const = 0;

        virtual void          External(bool state) = 0;
        virtual bool          External() const = 0;

        virtual void          FrameSize(unsigned int samples) = 0;
        virtual unsigned int  FrameSize() const = 0;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src) = 0;
        virtual AfeExtSyncOptions  ExternalSyncSource() const = 0;
    };
    //
    //  Interface Functions

    virtual unsigned int  Devices() = 0;
    virtual unsigned int  ChannelsPerDevice() = 0;
    virtual unsigned int  Channels() = 0;

    virtual void  DeviceDisableAll() = 0;
    virtual void  DeviceEnabled(int dev, bool state) = 0;
    virtual bool  DeviceEnabled(int dev) const = 0;

    virtual void  DevicePowered(int dev, bool state) = 0;
    virtual bool  DevicePowered(int dev) const = 0;

    virtual unsigned int  ActiveDevices() = 0;

    virtual void  ChannelDisableAll() = 0;
    virtual void  ChannelEnabled(int ch, bool state) = 0;
    virtual bool  ChannelEnabled(int ch) const = 0;

    virtual void  ChannelPowered(int ch, bool state) = 0;
    virtual bool  ChannelPowered(int ch) const = 0;

    virtual unsigned int  ActiveChannels() = 0;

    virtual ITriggerCfg &  Trigger() = 0;

    virtual void SoftwareTrigger(bool state) = 0;
    virtual void SoftwareTrigger(AfeDirection which, bool state) = 0;

    virtual void TestModeEnabled(bool state, unsigned int mode) = 0;
    virtual void   TestFrequency(double frequency_hz) = 0;
    virtual double TestFrequency() = 0;

    virtual void  Decimation(unsigned int samples) = 0;
    virtual unsigned int Decimation() const = 0;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int Spi(unsigned int device, unsigned int address) = 0;

};

//==============================================================================
//  CLASS IFmcClockIo -- User interface to control Fmc clock functionality
//==============================================================================

class IFmcClockIo
{
public:
    enum IIClockSource { csExternal, csInternal };   // clock routing 
    enum IIReferenceSource { rsExternal, rsCarrier, rsInternal };
    enum IIClockSelect {  cslFrontPanel, cslCarrier };


    virtual ~IFmcClockIo()  {  }

    //
    //  Interface Functions
    // Methods
    virtual double              FrequencyActual() const = 0;

    // Properties
    virtual void                Frequency(double value) = 0;
    virtual double              Frequency() const = 0;

    virtual ITimebaseRate &     Adc() = 0;
    virtual ITimebaseRate &     Dac() = 0;
    virtual ITimebaseRate &     System() = 0;

    virtual void                Source(IIClockSource src) = 0;
    virtual IIClockSource       Source() const = 0;

    virtual void                ExternalClkSelect(IIClockSelect /*src*/) {  }
    virtual IIClockSelect       ExternalClkSelect() const {  return cslFrontPanel;  }

    virtual void                Reference(IIReferenceSource /*src*/)  {  }
    virtual IIReferenceSource   Reference() const  {  return rsInternal;  }

    virtual void                ReferenceFrequency(double /*value*/)  {  }
    virtual double              ReferenceFrequency() const  {  return 0.0;  }

    virtual bool                Locked()  const = 0;

};

//==============================================================================
//  CLASS ISbcDuoClockIo -- User interface to control Fmc clock functionality
//==============================================================================

class ISbcDuoClockIo
{
public:
    enum IIClockSource { csExternal, csInternal };   // clock routing
    enum IIReferenceSource { rsExternalTrigger, rsExternal, rsInternal };
//    enum IIClockSelect {  cslFrontPanel, cslCarrier };


    virtual ~ISbcDuoClockIo()  {  }

    //
    //  Interface Functions
    // Methods
    virtual double              FrequencyActual() const = 0;

    // Properties
    virtual void                Frequency(double value) = 0;
    virtual double              Frequency() const = 0;

    virtual ITimebaseRate &     Adc() = 0;
    virtual ITimebaseRate &     Dac() = 0;
    virtual ITimebaseRate &     System() = 0;

    virtual void                Source(IIClockSource src) = 0;
    virtual IIClockSource       Source() const = 0;

//    virtual void                ExternalClkSelect(IIClockSelect /*src*/) {  }
//    virtual IIClockSelect       ExternalClkSelect() const {  return cslFrontPanel;  }

    virtual void                Reference(IIReferenceSource /*src*/)  {  }
    virtual IIReferenceSource   Reference() const  {  return rsInternal;  }

    virtual void                ReferenceFrequency(double /*value*/)  {  }
    virtual double              ReferenceFrequency() const  {  return 0.0;  }

    virtual void                LowLatencyTriggerMode(bool /*enabled*/) = 0;
    virtual bool                LowLatencyTriggerMode() const = 0;

    virtual bool                Locked()  const = 0;

};

//==============================================================================
//  CLASS IAtroposClockIo -- User interface to control Atropos clock functionality
//==============================================================================

class IAtroposClockIo
{
public:
    enum IIClockSource { csExternal, csInternal };   // clock routing
    enum IIClockSelect { cslFrontPanel, cslP16 };
    enum IIReferenceSource { rsExternal, rsP2S_Trigger, rsInternal };

    virtual ~IAtroposClockIo()  {  }

    //
    //  Interface Functions
    // Methods
    virtual double              FrequencyActual() const = 0;

    // Properties
    virtual void                Frequency(double value) = 0;
    virtual double              Frequency() const = 0;

    virtual ITimebaseRate &     Adc() = 0;
    virtual ITimebaseRate &     Dac() = 0;
    virtual ITimebaseRate &     System() = 0;

    virtual void                Source(IIClockSource src) = 0;
    virtual IIClockSource       Source() const = 0;

    virtual void                ExternalClkSelect(IIClockSelect src) = 0;
    virtual IIClockSelect       ExternalClkSelect() const = 0;

    virtual void                Reference(IIReferenceSource /*src*/)  {  }
    virtual IIReferenceSource   Reference() const  {  return rsInternal;  }

    virtual void                ReferenceFrequency(double /*value*/)  {  }
    virtual double              ReferenceFrequency() const  {  return 0.0;  }

    virtual bool                Locked()  const = 0;
    virtual bool                TriggerSync()  const = 0;


};

//==============================================================================
//  CLASS IFmcRawClockDevices -- Interface to standard Fmc "Raw" Clock Devices
//==============================================================================

class IFmcRawClockDevices
{
public:
    virtual ~IFmcRawClockDevices()  {  }

    virtual Cdce62005 &   Pll() = 0;
};

//==============================================================================
//  CLASS ILmkRawClockDevices -- Interface to standard Fmc "Raw" Clock Devices
//==============================================================================

class ILmkRawClockDevices
{
public:
    virtual ~ILmkRawClockDevices()  {  }

    virtual Lmk04800 &   Pll() = 0;
};

//==============================================================================
//  CLASS ILmk20RawClockDevices -- Interface to standard Fmc "Raw" Clock Devices
//==============================================================================

class ILmk20RawClockDevices
{
public:
    virtual ~ILmk20RawClockDevices()  {  }

    virtual Lmk04820 &   Pll() = 0;
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS IAd9510DuoRawClockDevices  -- Interface to raw clock devices
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

class IAd9510DuoRawClockDevices
{
public:
    virtual ~IAd9510DuoRawClockDevices()
        {  }

    virtual Ad9510Duo_Tb &   Pll() = 0;
};

//===========================================================================
//  CLASS IJESDInformation -- JESD device information interface
//===========================================================================

class IJESDInformation
{
public:
    virtual double  LineRateFrequency() = 0;
    virtual double  RequiredSampleFrequency() = 0;
};

//==============================================================================
//  namespace Fmc  -  holds FMC enumerations and definitions
//==============================================================================

namespace Fmc
{
    enum  FmcConnector { fmcP1, fmcP2 };
    enum  FmcConnectorBank { bankA, bankB };
    enum  FmcModuleSize { msSingleWide, msDoubleWide };
    enum  FmcConnectorSize { csLPC, csHPC, csNotFitted };
    enum  FmcClockDirection { dirMezzToCarrier, dirCarrierToMezz };
}

//==============================================================================
//  CLASS FmcStandardRomInfo -- Interface to standard Fmc Rom Information
//==============================================================================

class FmcStandardRomInfo
{
public:
    FmcStandardRomInfo(class AdvI2CIdrom & rom);
    ~FmcStandardRomInfo()  {  }

    void          Manufacturer(const std::string & str)  {  BA.Manufacturer(str);  }
    std::string   Manufacturer()  {  return BA.Manufacturer();  }

    void          Name(const std::string & str)  {  BA.Name(str);  }
    std::string   Name()  {   return BA.Name();  }

    void          SerialNumber(const std::string & str)  {  BA.SerialNumber(str);  }
    std::string   SerialNumber()  {   return BA.SerialNumber(); }

    void          PartNumber(const std::string & str)  {  BA.PartNumber(str);  }
    std::string   PartNumber()  {   return BA.PartNumber();  }

    void          FruFile(const std::string & str)  {  BA.FruFile(str);  }
    std::string   FruFile()  {   return BA.FruFile();  }

    void  LoadFromRom();
    void  StoreToRom();

    // probably need a clear/reset, if the parse does not work...

    bool          RomParseOk()
                        {  return ParseOk;  }
    std::string   RomParseResult()
                        {  return CheckStr;  }

    //  Reserved...

    void  ToRomImage();
    void  FromRomImage();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::FmcRomAreaBase -- Base class for ROM regions
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcRomAreaBase
    {
    public:
        FmcRomAreaBase(AdvI2CIdrom & rom)
            :  ROM(rom), CheckSum_1(0x0), Base(0)
            {}
        virtual ~FmcRomAreaBase()
            {}

        //  Header offsets are in 8 byte units
        unsigned int BaseOffset()  {  return Base>>3;  }
        void BaseOffset(unsigned int offset)  {  Base = offset<<3;  }

        unsigned int BaseOffsetBytes()  {  return Base;  }
        void BaseOffsetBytes(unsigned int base)  {  Base = base;  }

        //  Methods
        virtual void  ToRomImage() = 0;
        virtual void  FromRomImage() = 0;

        virtual bool  AreaCheck() = 0;
        std::string   AreaCheckResult()
                        {  return CheckStr;  }

        virtual unsigned int  AreaSizeBytes() = 0;
        unsigned int          AreaSize() {  return ToOffset(AreaSizeBytes()); } // in 8 byte units

        unsigned int          EndBytes() {  return Base + AreaSizeBytes(); }
        unsigned int          End()      {  return ToOffset(EndBytes() + 7); }  // rounds up, not down

    protected:
        AdvI2CIdrom &  ROM;
        unsigned int   CheckSum_1;
        unsigned int   Base;

        std::string    CheckStr;
        Trace          Debug;

        void   ReadByte(unsigned int offset, unsigned int & byte);
        void   WriteByte(unsigned int offset, unsigned int byte);

        void   ReadMultiByte(unsigned int offset, unsigned int & value, unsigned int bytes);
        void   WriteMultiByte(unsigned int offset, unsigned int value, unsigned int bytes);

        void   ReadStr(unsigned int offset, std::string & text);
        void   WriteStr(unsigned int offset, const std::string & text);

        void   CalculateChecksum(unsigned int & cksum, unsigned int start_idx, unsigned int size);

        unsigned int ToOffset(unsigned int bytes) {  return bytes>>3;  }
        unsigned int ToBytes(unsigned int offset) {  return offset<<3;  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::CommonHeader  -- Common Header Block
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CommonHeader :  public FmcRomAreaBase
    {
    public:
        CommonHeader(AdvI2CIdrom & rom);
        //  Properties
        unsigned int Version()  {  return Version_1;  }
        void Version(unsigned int version)  {  Version_1 = version;  }

        //  ...Board Area block
        unsigned int BoardAreaOffset()  {  return BoardAreaOfs_1;  }
        void BoardAreaOffset(unsigned int offset)  {  BoardAreaOfs_1 = offset;  }
        unsigned int BoardAreaByteAddr()  {  return ToBytes(BoardAreaOfs_1);  }
        void BoardAreaByteAddr(unsigned int byte_idx)  {  BoardAreaOfs_1 = ToOffset(byte_idx);  }

        //  ...MultiRecord block
        unsigned int MultiRecordOffset()  {  return MultiRecordOfs_1;  }
        void MultiRecordOffset(unsigned int offset)  {  MultiRecordOfs_1 = offset;  }
        unsigned int MultiRecordByteAddr()  {  return ToBytes(MultiRecordOfs_1);  }
        void MultiRecordByteAddr(unsigned int byte_idx)  {  MultiRecordOfs_1 = ToOffset(byte_idx);  }

        //  Methods
        virtual void  ToRomImage();
        virtual void  FromRomImage();
        virtual bool  AreaCheck();
        virtual unsigned int  AreaSizeBytes() {  return 8;  }

    protected:
        unsigned int Version_1;
        unsigned int InternalUseOfs_1;
        unsigned int ChassisOfs_1;
        unsigned int BoardAreaOfs_1;
        unsigned int ProductInfoOfs_1;
        unsigned int MultiRecordOfs_1;

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::BoardArea  -- Board Info Area Block
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BoardArea :  public FmcRomAreaBase
    {
    public:
        BoardArea(AdvI2CIdrom & rom);

        //  Properties
        void          Manufacturer(const std::string & str)  {  Manufacturer_s = str;  }
        std::string   Manufacturer()  {  return Manufacturer_s;  }

        void          Name(const std::string & str)  {  Name_s = str;  }
        std::string   Name()  {  return Name_s;  }

        void          SerialNumber(const std::string & str)  {  SerialNumber_s = str;  }
        std::string   SerialNumber()  {  return SerialNumber_s;  }

        void          PartNumber(const std::string & str)  {  PartNumber_s = str;  }
        std::string   PartNumber()  {  return PartNumber_s;  }

        void          FruFile(const std::string & str)  {  FruFile_s = str;  }
        std::string   FruFile()  {  return FruFile_s;  }

        //  Methods
        virtual void  ToRomImage();
        virtual void  FromRomImage();
        virtual bool  AreaCheck();
        virtual unsigned int  AreaSizeBytes() {  return ToBytes(AreaLength_1);  }

    protected:
        unsigned int  Version_1;
        unsigned int  AreaLength_1;
        unsigned int  Language_1;
        unsigned int  ManufactureDate_3;
        std::string   Manufacturer_s;
        std::string   Name_s;
        std::string   SerialNumber_s;
        std::string   PartNumber_s;
        std::string   FruFile_s;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::MultiRecordBase  -- MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MultiRecordBase :  public FmcRomAreaBase
    {
    public:
        MultiRecordBase(AdvI2CIdrom & rom);

        void          TypeID(unsigned int id)   {  TypeID_1 = id;  }
        unsigned int  TypeID() {  return TypeID_1;  }

        void  EoL_Marker(bool state);
        bool  EoL_Marker();

        virtual void  ToRomImage();
        virtual void  FromRomImage();
        virtual bool  AreaCheck();
        virtual unsigned int  AreaSizeBytes() {  return kHeaderSize + RecordLength_1;  }

    protected:
        enum { kHeaderSize = 5 };  // bytes;
        unsigned int TypeID_1;
        unsigned int FormatEOL_1;
        unsigned int RecordLength_1;
        unsigned int RecordChecksum_1;
        unsigned int HeaderChecksum_1;      // regular Checksum not used?

        //  Conversions
        float         ToFloatV(unsigned int ten_mv_unit_v)
                            {  return static_cast<float>(ten_mv_unit_v)/100.0f;  }
        float         ToFloatV_mv(unsigned int mv_unit_v)
                            {  return static_cast<float>(mv_unit_v)/1000.0f;  }
        float         ToFloatA(unsigned int ma_unit_a)
                            {  return static_cast<float>(ma_unit_a)/1000.0f;  }

        unsigned int  To10mV(float voltage)
                            {  return static_cast<unsigned int>(voltage*100.0+0.5);  }
        unsigned int  To1mV(float voltage)
                            {  return static_cast<unsigned int>(voltage*1000.0+0.5);  }
        unsigned int  To1mA(float amperage)
                            {  return static_cast<unsigned int>(amperage*1000.0+0.5);  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::DcOutputMultiRecord  -- MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DcOutputMultiRecord :  public MultiRecordBase
    {
    public:
        DcOutputMultiRecord(AdvI2CIdrom & rom);

        virtual void  ToRomImage();
        virtual void  FromRomImage();

        bool          Standby();
        unsigned int  OutputNumber();
        float         NominalV()   {  return ToFloatV(NominalV_2);  }
        float         MaxNegVDev() {  return ToFloatV(MaxNegVDev_2);  }
        float         MaxPosVDev() {  return ToFloatV(MaxPosVDev_2);  }
        float         RipplePkPk() {  return ToFloatV_mv(RipplePkPk_2);  }
        float         MinCurrent() {  return ToFloatA(MinCurrent_2);  }
        float         MaxCurrent() {  return ToFloatA(MaxCurrent_2);  }

        void          Standby(bool state);
        void          OutputNumber(unsigned int number);
        void          NominalV(float v)   {  NominalV_2   = To10mV(v);  }
        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
        void          RipplePkPk(float v) {  RipplePkPk_2 = To1mV(v);  }
        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }

    protected:
        unsigned int OutInfo_1;
        unsigned int NominalV_2;
        unsigned int MaxNegVDev_2;
        unsigned int MaxPosVDev_2;
        unsigned int RipplePkPk_2;
        unsigned int MinCurrent_2;
        unsigned int MaxCurrent_2;

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::DcLoadMultiRecord  -- MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DcLoadMultiRecord :  public MultiRecordBase
    {
    public:
        DcLoadMultiRecord(AdvI2CIdrom & rom);

        virtual void  ToRomImage();
        virtual void  FromRomImage();

        unsigned int  OutputNumber();
        float         NominalV()        {  return ToFloatV(NominalV_2);  }
        float         SpecdMinV()       {  return ToFloatV(SpecdMinV_2);  }
        float         SpecdMaxV()       {  return ToFloatV(SpecdMaxV_2);  }
        float         SpecdRipplePkPk() {  return ToFloatV_mv(SpecdRipplePkPk_2);  }
        float         MinCurrentLoad()  {  return ToFloatA(MinCurrentLoad_2);  }
        float         MaxCurrentLoad()  {  return ToFloatA(MaxCurrentLoad_2);  }

        void          OutputNumber(unsigned int number);
        void          NominalV(float v)        {  NominalV_2   = To10mV(v);  }
        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }

    protected:
        unsigned int VRequired_1;
        unsigned int NominalV_2;
        unsigned int SpecdMinV_2;
        unsigned int SpecdMaxV_2;
        unsigned int SpecdRipplePkPk_2;
        unsigned int MinCurrentLoad_2;
        unsigned int MaxCurrentLoad_2;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::ScratchMultiRecord  -- Junk OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ScratchMultiRecord :  public MultiRecordBase
    {
    public:
        ScratchMultiRecord(AdvI2CIdrom & rom);

        virtual void  ToRomImage();
        virtual void  FromRomImage();

        //  getters
        unsigned int            ManufacturerID() {  return ManufacturerID_3;  }
        unsigned int            SubType();

    protected:
        unsigned int ManufacturerID_3;
        unsigned int SubT_Ver_1;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::FmcDefinitionMultiRecord  -- OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class FmcDefinitionMultiRecord :  public MultiRecordBase
    {
    public:
        FmcDefinitionMultiRecord(AdvI2CIdrom & rom);

        virtual void  ToRomImage();
        virtual void  FromRomImage();

        //  getters
        unsigned int            ManufacturerID() {  return ManufacturerID_3;  }
        unsigned int            SubType();
        unsigned int            Version();
        Fmc::FmcModuleSize      ModuleSize();
        Fmc::FmcConnectorSize   ConnectorSize(Fmc::FmcConnector p);
        Fmc::FmcClockDirection  ClockDirection();
        unsigned int            BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank);
        unsigned int            TransceiverCount(Fmc::FmcConnector p);
        unsigned int            MaxClockMHz()   {  return MaxClock_1;  }

        // setters
        void    ModuleSize(Fmc::FmcModuleSize size);
        void    ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size);
        void    ClockDirection(Fmc::FmcClockDirection dir);
        void    BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank, unsigned int count);
        void    TransceiverCount(Fmc::FmcConnector p, unsigned int count);
        void    MaxClockMHz(unsigned int rateMHz )   {  MaxClock_1 = rateMHz;  }

    protected:
        unsigned int ManufacturerID_3;
        unsigned int SubT_Ver_1;
        unsigned int ModuleInfo_1;
        unsigned int P1a_Signals_1;
        unsigned int P1b_Signals_1;
        unsigned int P2a_Signals_1;
        unsigned int P2b_Signals_1;
        unsigned int Transceiver_1;
        unsigned int MaxClock_1;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::OEMMultiRecord  -- OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class OEMMultiRecord :  public MultiRecordBase
    {
    public:
        OEMMultiRecord(AdvI2CIdrom & rom);

        virtual void  ToRomImage();
        virtual void  FromRomImage();

        // ROM load/unload customization
        virtual void  ToRawImage_Custom(unsigned int & ofs);
        virtual void  ToRawImage_CustomDebug();
        virtual void  FromRawImage_Custom(unsigned int & ofs);
        virtual void  FromRawImage_CustomDebug();

        void  Resize(unsigned int size)
            {   Raw_n.resize(size);  }
        void  Clear()
            {   Raw_n.clear();  }
        void  Zero()
            {
            for (size_t i=0;  i< Raw_n.size(); i++)
                Raw_n[i] = 0;
            }

        //  getters
        unsigned int                 ManufacturerID() {  return ManufacturerID_3;  }
        std::vector<unsigned int> &  Raw()  {  return Raw_n;  }

    protected:
        unsigned int ManufacturerID_3;
        std::vector<unsigned int> Raw_n;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::CalCoefficientsMR  -- OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class CalCoefficientsMR :  public OEMMultiRecord, public IIdromViewable, public IdromIoDG
    {
    public:
        CalCoefficientsMR(AdvI2CIdrom & rom);

    protected:
        virtual const int *  const_idrom_base() const {  return reinterpret_cast<const int *>(&Raw_n[0]);  }
        virtual int *        idrom_base() {  return reinterpret_cast<int *>(&Raw_n[0]);  }

        virtual size_t       idrom_size() const {  return Raw_n.size();  }
        virtual size_t       idrom_reserve_size() const {  return 0;  }

        virtual void         after_data_write() { }  // checksum?

        IIdromViewable & IView()
            {  return *this;  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::AdcCalCoefficientsMR  -- OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AdcCalCoefficientsMR :  public CalCoefficientsMR
    {
    public:
        AdcCalCoefficientsMR(AdvI2CIdrom & rom);

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::DacCalCoefficientsMR  -- OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DacCalCoefficientsMR :  public CalCoefficientsMR
    {
    public:
        DacCalCoefficientsMR(AdvI2CIdrom & rom);

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::AdcRangedCalCoefficientsMR  -- OEM MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  customized to hold one of N pages for multirange CAL boards

    class AdcRangedCalCoefficientsMR :  public OEMMultiRecord, public IIdromViewable, public IdromIoDG
    {
    public:
        AdcRangedCalCoefficientsMR(AdvI2CIdrom & rom, unsigned int idx, unsigned int total, unsigned int channels=0);

        unsigned int Channels()  {  return (Raw_n.size()-1)/2;  }  // format is one cal word, one chksum(?) and 2 32-bit coeffs per channel

        // ROM load/unload customization
        virtual void  ToRawImage_Custom(unsigned int & ofs);
        virtual void  ToRawImage_CustomDebug();
        virtual void  FromRawImage_Custom(unsigned int & ofs);
        virtual void  FromRawImage_CustomDebug();

        //  validate object
        bool Validate()
            {
            if (CalRangeTotal_1 > 16)
                return false;
            else if (CalRangeIndex_1 >= CalRangeTotal_1)
                return false;
            else
                return true;
            }

        //  getters
        unsigned int  CalRangeIndex() {  return CalRangeIndex_1;  }
        unsigned int  CalRangeTotal() {  return CalRangeTotal_1;  }

    protected:
        unsigned int  CalRangeIndex_1;   // range number
        unsigned int  CalRangeTotal_1;   // total ranges

        virtual const int *  const_idrom_base() const {  return reinterpret_cast<const int *>(&Raw_n[0]);  }
        virtual int *        idrom_base() {  return reinterpret_cast<int *>(&Raw_n[0]);  }

        virtual size_t       idrom_size() const {  return Raw_n.size();  }
        virtual size_t       idrom_reserve_size() const {  return 0;  }

        virtual void         after_data_write() { }  // checksum?

        IIdromViewable & IView()
            {  return *this;  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::UnknownMultiRecord  -- ??? MultiRecord
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UnknownMultiRecord :  public MultiRecordBase
    {
    public:
        UnknownMultiRecord(AdvI2CIdrom & rom);

        virtual void  ToRomImage();
        virtual void  FromRomImage();

        //  getters
        std::vector<unsigned int> &  Raw()  {  return Raw_n;  }

    protected:
        std::vector<unsigned int> Raw_n;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcStandardRomInfo::MultiRecordArea  -- Multi Record Area Block
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MultiRecordArea :  public FmcRomAreaBase
    {
    public:
        MultiRecordArea(AdvI2CIdrom & rom);
        virtual ~MultiRecordArea();

        void  AddAdcCalRecord();
        void  AddDacCalRecord();
        void  AddAdcRangedCalRecord(size_t idx, size_t total, size_t channels);

        bool  IsEmpty()
                  {  return Records.empty();  }
        void  AddRecord(MultiRecordBase * record)
                  {  Records.push_back(record);  }

        //  Properties
        unsigned int  RecordCount()
                  {  return static_cast<unsigned int>(Records.size());  }
        MultiRecordBase *  Record(unsigned int idx)
                  {  return Records[idx];  }

        FmcDefinitionMultiRecord &   FmcDef();
        AdcCalCoefficientsMR &       AdcCal();
        DacCalCoefficientsMR &       DacCal();
        AdcRangedCalCoefficientsMR & AdcRangedCal(size_t idx);
        size_t  AdcRanges()
                {  return AdcRngCal_vec.size();  }

        //  Methods
        void  Clear();

        virtual void  ToRomImage();
        virtual void  FromRomImage();
        virtual bool  AreaCheck();
        virtual unsigned int  AreaSizeBytes();

    protected:
        std::vector<MultiRecordBase *>  Records;
        FmcDefinitionMultiRecord *      FmcDef_ptr;
        AdcCalCoefficientsMR *          AdcCal_ptr;
        DacCalCoefficientsMR *          DacCal_ptr;
        std::vector<AdcRangedCalCoefficientsMR *> AdcRngCal_vec;

        unsigned int  CumulativeSizeBytes(size_t idx);
    };

    //  Accessors
    CommonHeader &     Header()  {  return CH;  }
    BoardArea &        Board()   {  return BA;  }
    MultiRecordArea &  MultiR()  {  return MRA; }

    FmcDefinitionMultiRecord &   FmcDef()  {  return MRA.FmcDef();  }
    AdcCalCoefficientsMR &       AdcCal()  {  return MRA.AdcCal();  }
    DacCalCoefficientsMR &       DacCal()  {  return MRA.DacCal();  }
    AdcRangedCalCoefficientsMR & AdcRangedCal(size_t idx) {  return MRA.AdcRangedCal(idx);  }

    //  Record Converters  (return casted ptr)
    DcOutputMultiRecord *  AsDcOutput(MultiRecordBase * ptr)
                                {  return dynamic_cast<DcOutputMultiRecord *>(ptr);  }
    DcLoadMultiRecord *    AsDcLoad(MultiRecordBase * ptr)
                                {  return dynamic_cast<DcLoadMultiRecord *>(ptr);  }
    FmcDefinitionMultiRecord *    AsFmcDef(MultiRecordBase * ptr)
                                {  return dynamic_cast<FmcDefinitionMultiRecord *>(ptr);  }
    UnknownMultiRecord *   AsUnknown(MultiRecordBase * ptr)
                                {  return dynamic_cast<UnknownMultiRecord *>(ptr);  }
    AdcCalCoefficientsMR *    AsAdcCal(MultiRecordBase * ptr)
                                {  return dynamic_cast<AdcCalCoefficientsMR *>(ptr);  }
    DacCalCoefficientsMR *    AsDacCal(MultiRecordBase * ptr)
                                {  return dynamic_cast<DacCalCoefficientsMR *>(ptr);  }
    AdcRangedCalCoefficientsMR * AsAdcRangedCal(MultiRecordBase * ptr)
                                {  return dynamic_cast<AdcRangedCalCoefficientsMR *>(ptr);  }


    //  Record Makers  (return ptr to constructed object)
    DcOutputMultiRecord *   NewDcOutputRecord();
    DcLoadMultiRecord *     NewDcLoadRecord();
    AdcCalCoefficientsMR *  NewAdcCalCoefficientRecord();
    DacCalCoefficientsMR *  NewDacCalCoefficientRecord();
    AdcRangedCalCoefficientsMR *  NewAdcRangedCalCoefficientRecord(size_t idx, size_t total, size_t channels);

private:
    AdvI2CIdrom &  ROM;

    CommonHeader      CH;       // Mandatory
    BoardArea         BA;       // Mandatory
    MultiRecordArea  MRA;       // Mandatory

    bool           ParseOk;
    std::string    CheckStr;
    Trace          Debug;

};

//===========================================================================
//  CLASS CrosspointSwitch -- Generic CrossPoint Switch
//===========================================================================

class CrosspointSwitch
{
public:
    CrosspointSwitch(unsigned int size, unsigned int init_value=0)
        :  FieldStartBits(size), FieldSize(size),
           MBM(init_value)
        {
        }

    void  Switch(unsigned int output, unsigned int input)
        {
        if (output>=FieldStartBits.size())
            return;
        MBM.Field(FieldStartBits[output], FieldSize[output], ToField(input));
        }

    unsigned int  Switch(unsigned int output) const
        {
        if (output>=FieldStartBits.size())
            return 0;
        return FromField(MBM.Field(FieldStartBits[output], FieldSize[output]));
        }

    void            Value(unsigned int value) {  MBM.Value(value);  }
    unsigned int    Value() const {  return MBM.Value();  }

protected:
    std::vector<int> FieldStartBits;
    std::vector<int> FieldSize;
    MemoryBitManipulator  MBM;

    virtual unsigned int ToField(unsigned int input) const {  return input;  }
    virtual unsigned int FromField(unsigned int input) const {  return input;  }
};

//===========================================================================
//  CLASS DS90CP04Switch -- CrossPoint Switch
//===========================================================================

class DS90CP04Switch : public CrosspointSwitch
{
public:
    //=======================================================================
    //  CLASS DS90CP04Switch::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int CrosspointSwitch_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DS90CP04Switch::XPointLogicRegister  -- CrossPoint Switch Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XPointLogicRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup Data;
        RegisterBit      Ready;

        XPointLogicRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Data(*this, 0, 12),
              Ready(*this, 31)
        {  }
    };

    XPointLogicRegister &  XPoint() {  return XPointLogicReg;  }

    enum InputCodes {  icTriState, icIn1, icIn2, icIn3, icIn4  };
    enum OutputCodes { ocOut1,  ocOut2,  ocOut3,  ocOut4  };

    DS90CP04Switch(IAddressingSpace & CrosspointSwitch_space, MapRegisters & regs)
        :  CrosspointSwitch(4), XPointLogicReg(CrosspointSwitch_space, regs.CrosspointSwitch_Addr)
        {
        //  Setup Field Definitions
        FieldStartBits[ocOut1] = 9;   FieldSize[ocOut1]=3;
        FieldStartBits[ocOut2] = 6;   FieldSize[ocOut2]=3;
        FieldStartBits[ocOut3] = 3;   FieldSize[ocOut3]=3;
        FieldStartBits[ocOut4] = 0;   FieldSize[ocOut4]=3;
        }

    void Apply()
    {
        if (XPoint().Ready.Value() == 1)
            XPoint().Data.Value(Value());
        else
            Debug.Log("Crosspoint Switch not ready");
    }

    // Don't need to override ToField, FromField?
private:
    //
    //  Registers
    XPointLogicRegister XPointLogicReg;
    Trace               Debug;
};

//===========================================================================
//  CLASS IPolledFifo  -- Interface to a Polled Data Fifo
//===========================================================================

class IPolledFifo
{
public:

    virtual ~IPolledFifo()  {}

    virtual unsigned int  Count() = 0;
    virtual bool          AlmostEmpty() = 0;
    virtual bool          Empty() = 0;
    virtual bool          AlmostFull() = 0;
    virtual bool          Full() = 0;

    virtual void          AlmostFullLevel(unsigned int samples) = 0;
    virtual unsigned int  AlmostFullLevel() const = 0;
    virtual void          AlmostEmptyLevel(unsigned int samples) = 0;
    virtual unsigned int  AlmostEmptyLevel() const = 0;

    virtual void          Data(unsigned int data) = 0;
    virtual unsigned int  Data() const = 0;

    virtual void          DelayInSamples(unsigned int clock_ticks) = 0;
    virtual unsigned int  DelayInSamples() const = 0;
    virtual void          Delay(unsigned int us_ticks) = 0;
    virtual unsigned int  Delay() const = 0;
    virtual void          LoadFullDelay(unsigned int total_delay_us, double sample_freq_hz, unsigned int decimation) = 0;

};



#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative


#endif
