//---------------------------------------------------------------------------
// Fmc_1000_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_1000_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>
#include <math.h>
#include <StringSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 1;
const int             InChannelsPerDevice = 2;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices        = 1;
const int             OutChannelsPerDevice = 2;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;

const unsigned int Fmc_1000_VitaInStreamCount   = MaxInDevices;   // Number of VITA Incoming SIDs
const unsigned int Fmc_1000_VitaOutStreamCount  = MaxOutDevices;  // Number of VITA Outgoing SIDs

const unsigned int Fmc_1000_AdcSpiDeviceCount   = MaxInDevices;   // Number of SPI devices muxed
const unsigned int Fmc_1000_DacSpiDeviceCount   = MaxOutDevices;  // Number of SPI devices muxed

const unsigned int Fmc_1000_AdcChannelsPerDevice = InChannelsPerDevice;
const unsigned int Fmc_1000_AdcChannels          = MaxInChannels;
const unsigned int Fmc_1000_DacChannelsPerDevice = OutChannelsPerDevice;
const unsigned int Fmc_1000_DacChannels          = MaxOutChannels;

const float         InputMaxRate            = 1700.0e6f;
const float         InputMinRate            = 5.0e6f;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 4;

const unsigned int Fmc_1000_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_1000__Adc_One = 0x10000;         // representation of floating point 1.0

const float         OutputMaxRate            = 1700.0e6f;
const float         OutputMinRate            = 5.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 420;
const          int  OutputMin_mV             = -420;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 4;

const unsigned int Fmc_1000_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
const unsigned int Fmc1000_Dac_One = 0x10000;         // representation of floating point 1.0

const unsigned int DacNumConverters = 2;  // constant for DAC38J82

//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
//const int romFmc_1000_SysInfoOffset        = 0;
//const int romFmc_1000_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  VGA BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_1000_InputCoeffOffset       = 0;
const int romFmc_1000_InputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_1000_OutputCoeffOffset       = 0;   // these are in separate "ROMs"   //  romFmc_1000_InputCoeffOffset_avail;
const int romFmc_1000_OutputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//===========================================================================
// Class Fmc1000_AnalogInInfo
//===========================================================================

class Fmc1000_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    Fmc1000_AnalogInInfo(Fmc1000 * owner) : Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~Fmc1000_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return Owner->Adc().Bits(); }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    Fmc1000 *  Owner;
    SpanInfo   FSpan;
};

//===========================================================================
// Class Fmc1000_AnalogOutInfo
//===========================================================================

class Fmc1000_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    Fmc1000_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~Fmc1000_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};


//===========================================================================
//  CLASS Fmc1000InputDevice  -- Fmc1000 Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Fmc1000InputDevice
//---------------------------------------------------------------------------

Fmc1000InputDevice::Fmc1000InputDevice(Fmc1000 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), Owner1000(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_1000_InputCoeffOffset, MaxInChannels,
                       Fmc_1000__Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc1000InputDevice
//---------------------------------------------------------------------------

Fmc1000InputDevice::~Fmc1000InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc1000InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc1000InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc1000InputDevice::ActiveDevices() -- Number of ADC devices in use
//---------------------------------------------------------------------------

unsigned int  Fmc1000InputDevice::ActiveDevices()
{
    return Fmc_1000_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc1000InputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc1000InputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc1000InputDevice::PreconfigureHardware()
//---------------------------------------------------------------------------

void Fmc1000InputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc1000InputDevice::CustomAdcExternalSyncSource() --  Additional Cfg for custom
//---------------------------------------------------------------------------

void  Fmc1000InputDevice::CustomAdcExternalSyncSource(AfeExtSyncOptions src)
{
    if (src==essFrontPanel)
        Owner1000->SyncXPS.Switch(DS90CP04Switch::ocOut1, DS90CP04Switch::icIn2);
    else  // essCarrier,
        Owner1000->SyncXPS.Switch(DS90CP04Switch::ocOut1, DS90CP04Switch::icIn1);
}

//===========================================================================
//  CLASS Fmc1000OutputDevice  -- Fmc1000 Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc1000OutputDevice
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class Fmc1000OutputDevice
//---------------------------------------------------------------------------

Fmc1000OutputDevice::Fmc1000OutputDevice(Fmc1000 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), Owner1000(owner), FInfo(info),
      CalRomSection(DacCalIdrom(), romFmc_1000_OutputCoeffOffset, MaxOutChannels,
                       Fmc1000_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc1000OutputDevice
//---------------------------------------------------------------------------

Fmc1000OutputDevice::~Fmc1000OutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc1000OutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc1000OutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc1000OutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc1000OutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc1000OutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc1000OutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc1000OutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc1000OutputDevice::ConfigureHardware()
{
    Owner1000->Debug.Log("DAC ConfigureHardware()");

    // Toggle DAC RESETB
    Owner1000->Debug.Log("Toggle DAC RESETB");
    DacReset(false);
    DacReset(true);
    DacReset(false);

}

//---------------------------------------------------------------------------
//  Fmc1000OutputDevice::CustomDacExternalSyncSource() --  Additional Cfg for custom
//---------------------------------------------------------------------------

void  Fmc1000OutputDevice::CustomDacExternalSyncSource(AfeExtSyncOptions src)
{
    if (src==essFrontPanel)
        Owner1000->SyncXPS.Switch(DS90CP04Switch::ocOut2, DS90CP04Switch::icIn2);
    else  // essCarrier,
        Owner1000->SyncXPS.Switch(DS90CP04Switch::ocOut2, DS90CP04Switch::icIn1);
}

//=============================================================================
//  CLASS Fmc1000_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc1000_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 25.0e6;

//---------------------------------------------------------------------------
//  constructor for class Fmc1000_Pll
//---------------------------------------------------------------------------

Fmc1000_Pll::Fmc1000_Pll(Fmc1000 * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg), Owner(owner), FDacEffectiveFactor(1)
{
    OutputSyncMode(osmSysRefSync);
    Allow_Pll1_FeedbackAdjustment();
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 0 as guideline output
    AutoReapply(false);
    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 25
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class Fmc1000_Pll
//---------------------------------------------------------------------------

Fmc1000_Pll::~Fmc1000_Pll()
{
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  Fmc1000_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "Fmc1000_Pll::OnInit()");
    //
    //  Set Source Configuration
    SourceInUse(0, false);   SetSourceMapping(0, IFmcClockIo::rsCarrier);    // clkin0 = carrier, not settable
    SourceInUse(1, true);    SetSourceMapping(1, IFmcClockIo::rsInternal);   // clkin1 = internal reference (or external clock)
    SourceInUse(2, true);    SetSourceMapping(2, IFmcClockIo::rsExternal);   // clkin2 = external reference (carrier or front panel)

    //  Device Powered/Enabled Setup
    //PowerDownOutputGroup(8/2, true);
    //
    //  Set Output Configuration
    OutputInUse(0,  true);    SetOutputConfig(0,  ocLvds);      //  FPGA_ADC_CLK (Adc (RX) line rate/40)
    OutputInUse(1,  true);    SetOutputConfig(1,  ocLvds);      //  FPGA_ADC_SYSREF (Adc System Reference)
    OutputInUse(2,  true);    SetOutputConfig(2,  ocLvds);      //  GBTCLK1
    OutputInUse(3,  true);    SetOutputConfig(3,  ocLvds);      //  GBTCLK0
                              SetOutput_AsCopy(3);              //  make 3 a copy of 2
    OutputInUse(4,  true);    SetOutputConfig(4,  ocLvds);      //  FPGA_DAC_CLK (Dac (TX) line rate/40)
    OutputInUse(5,  true);    SetOutputConfig(5,  ocLvds);      //  FMC_DAC_SYSREF (Dac System Reference -actually ADC)
    OutputInUse(6,  true);    SetOutputConfig(6,  ocHsds_08ma); //  DAC_CLK (DAC Sample Clock)
    OutputInUse(7,  true);    SetOutputConfig(7,  ocHsds_08ma); //  DAC_SYSREF (DAC System Reference)
    OutputInUse(8,  true);    SetOutputConfig(8,  ocLvds);      //  Clock Mux IN3 (feedback) and ADC external Sync capture clock
    OutputInUse(9,  false);   SetOutputConfig(9,  ocDisabled);  //  SY Mux in4 (unused)
    OutputInUse(10, true);    SetOutputConfig(10, ocLvds);      //  Clock Mux IN4 (Dac (TX) line rate/40)
    OutputInUse(11, false);   SetOutputConfig(11, ocLvds);      //  SY Mux in3 (Adc System Reference)
    OutputInUse(12, true);    SetOutputConfig(12, ocHsds_08ma); //  ADC_CLK
    OutputInUse(13, true);    SetOutputConfig(13, ocHsds_08ma); //  ADC_SYSREF

    FeedbackOutput(fbm_Dclk8);  // set output to use as PLL feedback in 0-delay modes

    //  Set Reduced Frequency Limits for Outputs (Preset to 3100.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[0] = 1000.0e6;
    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refSecondary);    // Set to use secondary reference
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::OnReapply()
//---------------------------------------------------------------------------

void  Fmc1000_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  Fmc1000_Pll::AdcMasterOutput()
{
    return 12;
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  Fmc1000_Pll::DacMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::DacEffectiveFactor() - Divider factor
//---------------------------------------------------------------------------
//  Ratio of Sample Rate to Dac Input Clock rate

unsigned int  Fmc1000_Pll::DacEffectiveFactor()
{
    return FDacEffectiveFactor;
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool Fmc1000_Pll::IsFeedbackOutputAdc()
{
    return true;
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------

void  Fmc1000_Pll::SetAdcOutputRates(double adc_rate)
{
    double LineRate = Owner->Adc().CalcAdcJesdLineRate(adc_rate);
    double FpgaAdcClk = LineRate/40;
    double SampleFrequency = adc_rate;
    double SysRefClk = SampleFrequency/512;
    double MgtRefClk = Owner->JesdPhy().CalcMgtRefClk(LineRate);

    OutputFrequency(0,    FpgaAdcClk);
    OutputFrequency(1,    SysRefClk);
    OutputFrequency(2,    MgtRefClk);
    //OutputFrequency(3,    MgtRefClk);       // is copy of 2
    OutputFrequency(5,    SysRefClk);         // actually dac
    OutputFrequency(7,    SysRefClk);         // actually dac
    OutputFrequency(8,    SampleFrequency/2); // feedback and adc external sync capture clock
    OutputFrequency(11,   SysRefClk);         // actually dac
    OutputFrequency(12,   SampleFrequency);
    OutputFrequency(13,   SysRefClk);
}

//---------------------------------------------------------------------------
//  Fmc1000_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  Fmc1000_Pll::SetDacOutputRates(double dac_rate)
{
    double LineRate = Owner->Dac().CalcDacJesdLineRate(dac_rate);
    double FpgaDacClk = LineRate/40;
    double DacClkFrequency = Owner->Dac().CalcDacClkFrequency(dac_rate);
    FDacEffectiveFactor = static_cast<unsigned int> (dac_rate/DacClkFrequency);

    OutputFrequency(4,   FpgaDacClk);
    OutputFrequency(6,   DacClkFrequency);
    OutputFrequency(10,  FpgaDacClk);
}

//=============================================================================
//  CLASS Fmc1000_LMK20_Clock  --  FMC1000 customization of LMK20 Clock Subdev
//=============================================================================
//---------------------------------------------------------------------------
//  Fmc1000_LMK20_Clock::CustomExternalClkSelect() -- custom select HDW
//---------------------------------------------------------------------------

void  Fmc1000_LMK20_Clock::CustomExternalClkSelect(IFmcClockIo::IIClockSelect src)
{
    if (src==IFmcClockIo::cslFrontPanel)
        Owner->ClockXPS.Switch(DS90CP04Switch::ocOut4, DS90CP04Switch::icIn2);
    else  // cslCarrier,
        Owner->ClockXPS.Switch(DS90CP04Switch::ocOut4, DS90CP04Switch::icIn1);
}

//---------------------------------------------------------------------------
//  Fmc1000_LMK20_Clock::CustomExternalClkSelect() -- custom select HDW
//---------------------------------------------------------------------------

IFmcClockIo::IIClockSelect  Fmc1000_LMK20_Clock::CustomExternalClkSelect() const
{
    DS90CP04Switch::InputCodes code = static_cast<DS90CP04Switch::InputCodes>(Owner->ClockXPS.Switch(DS90CP04Switch::ocOut4));
    return (code==DS90CP04Switch::icIn2) ? IFmcClockIo::cslFrontPanel : IFmcClockIo::cslCarrier;
}

//===========================================================================
//  CLASS Fmc1000_AdcJesdIo -- Interface to ADC JESD
//===========================================================================

class Fmc1000_AdcJesdIo : public IJESD_IoDevice
{
public:
    Fmc1000_AdcJesdIo( Fmc1000 * owner )
        :  Owner(owner)
            {
            }
    virtual ~Fmc1000_AdcJesdIo()
            {}
    virtual bool  IsDeviceActive()
                    {  return true;  }   // we use input
    virtual double CalcJesdLineRate(double SamplingRate)
                    {  return Owner->Adc().CalcAdcJesdLineRate(SamplingRate);  }

private:
    Fmc1000 * Owner;

};

//===========================================================================
//  CLASS Fmc1000_DacJesdIo -- Interface to DAC JESD
//===========================================================================

class Fmc1000_DacJesdIo : public IJESD_IoDevice
{
public:
    Fmc1000_DacJesdIo( Fmc1000 * owner )
        :  Owner(owner)
            {
            }
    virtual ~Fmc1000_DacJesdIo()
        {}
    virtual bool  IsDeviceActive()
                    {  return true;  }   // we use output
    virtual double CalcJesdLineRate(double SamplingRate)
                    {  return Owner->Dac().CalcDacJesdLineRate(SamplingRate);  }

private:
    Fmc1000 * Owner;

};

//===========================================================================
//  CLASS Fmc1000_JesdPllRates -- Interface to JESD Clock
//===========================================================================

class Fmc1000_JesdPllRates : public IJESD_PllRates
{
public:
    Fmc1000_JesdPllRates( Fmc1000 * owner )
        :  Owner(owner)
            {
            }
    virtual ~Fmc1000_JesdPllRates()
        {}

    virtual double AdcFrequencyActual()
        {  return Owner->Clock().Adc().FrequencyActual();  }
    virtual double DacFrequencyActual()
        {  return Owner->Clock().Dac().FrequencyActual();  }
    virtual double MgtRefFrequencyActual()
        {  return Owner->RawClockDevice().Pll().OutputFrequencyActual(2);  }

private:
    Fmc1000 * Owner;

};

//===========================================================================
//  CLASS Fmc1000  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc1000
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{
FmcLMK20_Clock::MapRegisters FmcLMKClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

Lmk04820Registers  Fmc1000_Lmk04820Registers =
{
    mmFmc_PllControl,    //  Control_Addr;
    mmFmc_PllSpi,        //  SpiIntf_Addr
};

Fmc1000_AdcCalibrationControl::MapRegisters  Fmc1000_AdcCalibrationControl_RegAddrs =
{
    mmFmc1000_AdcPHYCal,     //  PhyCal_Addr
};

DS90CP04Switch::MapRegisters  ExtSyncCrosspointSwitch_RegAddrs =
{
    mmFmc1000_ExtSyncCfg,    //  CrosspointSwitch_Addr
};

DS90CP04Switch::MapRegisters  ExtClkCrosspointSwitch_RegAddrs =
{
    mmFmc1000_ExtClkCfg,     //  CrosspointSwitch_Addr
};

bool IsInFmc500Mode = true;  // uses bit 31 as R/W bit like the 500

}

//---------------------------------------------------------------------------
//  constructor for class Fmc1000
//---------------------------------------------------------------------------

Fmc1000::Fmc1000()
    :  FmcBidirBoard(fmcPrimary, IsInFmc500Mode),
       AFEClock_SDev(this, this, WB_FMC_1, FmcLMKClockRegAddrs),
       AdcCalIDROM_SDev(this, RomInfo(), true),   // true = ADC Rom
       DacCalIDROM_SDev(this, RomInfo(), false),  // true = ADC Rom
       JESD_PHY_SDev(this),
       JESD_ADC_SDev(this),
       JESD_DAC_SDev(this),
       ADC_SDev(this, JESD_ADC_SDev.Device()),
       DAC_SDev(this, JESD_DAC_SDev.Device()),
       AFEAdcCalCtrl_SDev(this, WB_FMC_2, Fmc1000_AdcCalibrationControl_RegAddrs, JESD_ADC_SDev.Device()),
       FInput(new Fmc1000InputDevice(this, new Fmc1000_AnalogInInfo(this))),
       FOutput(new Fmc1000OutputDevice(this, new Fmc1000_AnalogOutInfo)),
       ClockXPS(WB_FMC_1, ExtClkCrosspointSwitch_RegAddrs),
       SyncXPS(WB_FMC_1, ExtSyncCrosspointSwitch_RegAddrs),
       FDacClocksDisabled(false), FTestClocksDisabled(true)
{
    LocalConstructCustomParts();
}

Fmc1000 * an1000()
{
    Fmc1000 * wombat = new Fmc1000;
    return wombat;
}

//---------------------------------------------------------------------------
//  Fmc1000::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  Fmc1000::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  Header Section
    RI.Header().Version(0x2);

    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("Fmc1000");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(1000);
    RI.FmcDef().ModuleSize(Fmc::msSingleWide);
    RI.FmcDef().ClockDirection(Fmc::dirCarrierToMezz);
    RI.FmcDef().ConnectorSize(Fmc::fmcP1, Fmc::csHPC);
    RI.FmcDef().ConnectorSize(Fmc::fmcP2, Fmc::csNotFitted);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankA, 87);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankB, 30);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 0);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankB, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP1, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP2, 0);

    //  ADC CALIBRATION SECTION
    RI.AdcCal().Resize(romFmc_1000_InputCoeffOffset_avail);
    RI.AdcCal().Zero();

    //  DAC CALIBRATION SECTION
    RI.DacCal().Resize(romFmc_1000_OutputCoeffOffset_avail);
    RI.DacCal().Zero();

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(2.5f);
    DcOut->RipplePkPk(0.5f);
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(false);
    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    //        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
    //        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
    //        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(2.5f);
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  Fmc1000::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc1000::LocalConstructCustomParts()
{
    FmcBidirBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    AdcCalIDROM_SDev.ConstructCustomParts();
    DacCalIDROM_SDev.ConstructCustomParts();
    JESD_PHY_SDev.ConstructCustomParts();
    JESD_ADC_SDev.ConstructCustomParts();
    JESD_DAC_SDev.ConstructCustomParts();
    ADC_SDev.ConstructCustomParts();
    DAC_SDev.ConstructCustomParts();
    AFEAdcCalCtrl_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  Fmc1000::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc1000::OpenHardware()
{
    //PokeLogic(0x0,   0x00000);  // offsets relative to FMC

    FmcBidirBoard::OpenHardware();

    Debug.Log("++++++++++++++++++++++++++++++++++++");
    Debug.Log("  Load Adc/Dac Calibration Shadows");
    AdcCalIDROM_SDev.OpenHardware();
    DacCalIDROM_SDev.OpenHardware();
    Debug.Log("++++++++++++++++++++++++++++++++++++");

    //  Turn on VCO Power
    //AFEAdcVcoRegister.Bit(4, true);

    Input().DevicePowered(0,true);
    Input().DevicePowered(1,true);

    unsigned int data = PeekLogic(0x0);
    Debug.Log("Power Register Final: ", data, 16);

    AFEClock_SDev.OpenHardware();
    JESD_PHY_SDev.OpenHardware();
    JESD_ADC_SDev.OpenHardware();
    JESD_DAC_SDev.OpenHardware();
    ADC_SDev.OpenHardware();
    DAC_SDev.OpenHardware();
    AFEAdcCalCtrl_SDev.OpenHardware();
    Input().OpenHardware();
    Output().OpenHardware();
}

//---------------------------------------------------------------------------
//  Fmc1000::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  Fmc1000::PreconfigureHardware()
{
    FmcBidirBoard::PreconfigureHardware();

    // Connect Clock Mux IN4 to OUT1 and IN3 to OUT2
    ClockXPS.Switch(DS90CP04Switch::ocOut1, DS90CP04Switch::icIn4);
    ClockXPS.Switch(DS90CP04Switch::ocOut2, DS90CP04Switch::icIn3);
    ClockXPS.Apply();

    AFEClock_SDev.PreconfigureHardware();

    // Initialize the FPGA MGTs
    JESD_PHY_SDev.PreconfigureHardware();
    JESD_ADC_SDev.PreconfigureHardware();
    JESD_DAC_SDev.PreconfigureHardware();

    //
    //  Special Config for A/D and D/A Devices
    Input().PreconfigureHardware();
    Output().PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc1000::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc1000::ConfigureHardware()
{
    FmcBidirBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    JESD_PHY_SDev.ConfigureHardware();
    JESD_ADC_SDev.ConfigureHardware();
    JESD_DAC_SDev.ConfigureHardware();

    ConfigureAdcs();
    ConfigureDacs();
}

//---------------------------------------------------------------------------
//  Fmc1000::ConfigureAdcs()
//---------------------------------------------------------------------------

void  Fmc1000::ConfigureAdcs()
{
    Debug.Log("ConfigureAdcs()");

    Adc().OverrideChannelEnabled();

    if (Input().ChannelPowered(0)==false && Input().ChannelPowered(1)==false)
        {
        Debug.Log("ADC Power off, config skipped.");
        return;   // no need to go on, power is off.
        }

    //
    //  ADC Setup
    Adc().ConfigureHardware();
    AFEAdcCalCtrl_SDev.ConfigureHardware();

    Debug.Log("end of ConfigureAdcs()");
}

//---------------------------------------------------------------------------
//  Fmc1000::ConfigureDacs()
//---------------------------------------------------------------------------

void  Fmc1000::ConfigureDacs()
{
    Debug.Log("ConfigureDacs()");

    if (Output().ChannelPowered(0)==false && Output().ChannelPowered(1)==false)
        {
        Debug.Log("DAC Power off, config skipped.");
        return;   // no need to go on, power is off.
        }

    //
    //  Special Config for D/A Devices
    Output().ConfigureHardware();  // reset DACs

    // Load Dac actual clock rates
    double DacSamplingRate = Clock().Dac().FrequencyActual();
    double DacClkRate = Clock().Dac().EffectiveFrequencyActual();
    Dac().LoadDacClockRates(DacSamplingRate, DacClkRate);

    //
    //  DAC Setup
    Dac().ConfigureHardware();
    Debug.Log("end of ConfigureDacs()");
}

//---------------------------------------------------------------------------
//  Fmc1000::StreamStopHardware()
//---------------------------------------------------------------------------

void  Fmc1000::StreamStopHardware()
{
    FmcBidirBoard::StreamStopHardware();

    //AFEClock_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  Fmc1000::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc1000::GetVitaInStreamCount()
{
    return Fmc_1000_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc1000::GetVitaOutStreamCount()
{
    return Fmc_1000_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc1000::GetAdcSpiDeviceCount()
{
    return Fmc_1000_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc1000::GetDacSpiDeviceCount()
{
    return Fmc_1000_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc1000::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  Fmc1000::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new FmcStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  Fmc1000::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  Fmc1000::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new FmcStandardPriClockDevice(this, 2);
}


//---------------------------------------------------------------------------
//  Fmc1000::MakeFmcAfePllObject() -- Get Pri Device object
//---------------------------------------------------------------------------

Lmk04820 *  Fmc1000::MakeFmcAfePllObject()
{
    // return same object for both directions
    return new Fmc1000_Pll(this, WB_FMC_1, Fmc1000_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  Fmc1000::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc1000::GetAdcCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  Fmc1000::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc1000::GetCalibrationAdcCount()
{
    return Fmc_1000_AdcChannels;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc1000::GetDacCalibrationOverride()
{
    return 0; //new Fmc1000_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  Fmc1000::GetCalibrationDacCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc1000::GetCalibrationDacCount()
{
    return Fmc_1000_DacChannels;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  Fmc1000::GetAnalogDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = Fmc_1000_AdcChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  Fmc1000::GetAnalogPhysicalDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = InChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc1000::GetFmcCalIdromInfo() -- Make ROM Image Objhect
//---------------------------------------------------------------------------

Fmc1000::FmcCalIdromInfo  Fmc1000::GetFmcCalIdromInfo(FmcCalDeviceCode which)
{
    Debug.Log("Fmc1000::GetFmcCalIdromInfo() which=", which);

    const unsigned int ReservedSize = 1;

    FmcCalIdromInfo  info;
    if (which==fmcCalAdc)
        {
        info.IdromSize = romFmc_1000_InputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }
    else
        {
        info.IdromSize = romFmc_1000_OutputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }

    return info;
}

//---------------------------------------------------------------------------
//  Fmc1000::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void Fmc1000::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = maxFmcVadjCode = 0x4;  // 2.5V only
}

//---------------------------------------------------------------------------
//  Fmc1000::Input() -- Reference to input object
//---------------------------------------------------------------------------

Fmc1000InputDevice & Fmc1000::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  Fmc1000::Output() -- Reference to output object
//---------------------------------------------------------------------------

Fmc1000OutputDevice & Fmc1000::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  Fmc1000::PokeJESDPHY() -- write to the JESD PHY
//---------------------------------------------------------------------------

void  Fmc1000::PokeJESDPHY(unsigned int /*device*/, unsigned int address, unsigned int data)
{
    PokeJesdLogic(2, address, data);

//    std::stringstream msg;
//    msg << std::hex << "JESD204B PHY IP(Dev" << device << " 0x" << address << ", 0x" << data << ")";
//    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  Fmc1000::PeekJESDPHY() -- read from JESD PHY
//---------------------------------------------------------------------------

unsigned int  Fmc1000::PeekJESDPHY(unsigned int /*device*/, unsigned int address)
{
    return PeekJesdLogic(2, address);
}

//---------------------------------------------------------------------------
//  Fmc1000::GetAdcJesd_IoDevice() -- Create interface object to JESD ADC
//---------------------------------------------------------------------------

IJESD_IoDevice *  Fmc1000::GetAdcJesd_IoDevice(unsigned int /*device*/)
{
    return new Fmc1000_AdcJesdIo(this);
}

//---------------------------------------------------------------------------
//  Fmc1000::GetDacJesd_IoDevice() -- Create interface object to JESD DAC
//---------------------------------------------------------------------------

IJESD_IoDevice *  Fmc1000::GetDacJesd_IoDevice(unsigned int /*device*/)
{
    return new Fmc1000_DacJesdIo(this);
}

//---------------------------------------------------------------------------
//  Fmc1000::GetJesd_PllRatesDevice() -- Create interface to clocks for JESD
//---------------------------------------------------------------------------

IJESD_PllRates *  Fmc1000::GetJesd_PllRatesDevice(unsigned int /*device*/)
{
    return new Fmc1000_JesdPllRates(this);
}

//---------------------------------------------------------------------------
//  Fmc1000::PokeJESDADC() -- write to the JESD ADC
//---------------------------------------------------------------------------

void  Fmc1000::PokeJESDADC(unsigned int /*device*/, unsigned int address, unsigned int data)
{
    PokeJesdLogic(0, address, data);
    std::stringstream msg;
    msg << std::hex << "Adc JESD204B IP(0x" << address << ", 0x" << data << ")";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  Fmc1000::PeekJESDADC() -- read from JESD ADC
//---------------------------------------------------------------------------

unsigned int  Fmc1000::PeekJESDADC(unsigned int /*device*/, unsigned int address)
{
    return PeekJesdLogic(0, address);
}

//---------------------------------------------------------------------------
//  Fmc1000::PokeJESDDAC() -- write to the JESD DAC
//---------------------------------------------------------------------------

void  Fmc1000::PokeJESDDAC(unsigned int /*device*/, unsigned int address, unsigned int data)
{
    PokeJesdLogic(1, address, data);
    std::stringstream msg;
    msg << std::hex << "Dac JESD204B IP(0x" << address << ", 0x" << data << ")";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  Fmc1000::PeekJESDDAC() -- read from JESD DAC
//---------------------------------------------------------------------------

unsigned int  Fmc1000::PeekJESDDAC(unsigned int /*device*/, unsigned int address)
{
    return PeekJesdLogic(1, address);
}

//---------------------------------------------------------------------------
//  Fmc1000::SetAdcLaneConfiguration() - pre-set up ADC options
//---------------------------------------------------------------------------
//  This matches FMC-1000 setup

void Fmc1000::SetAdcLaneConfiguration(unsigned int /*device*/)
{
    // Load LanePN_Inverted "bits"
    Adc().LanePN_Inverted[0] = 1;    Adc().LanePN_Inverted[1] = 1;
    Adc().LanePN_Inverted[2] = 1;    Adc().LanePN_Inverted[3] = 1;

    // ADC physical to logical lanes mapping
    Adc().PhysToLogical_LaneMap[0] = 0;    Adc().PhysToLogical_LaneMap[1] = 1;
    Adc().PhysToLogical_LaneMap[2] = 2;    Adc().PhysToLogical_LaneMap[3] = 3;
}

//---------------------------------------------------------------------------
//  Fmc1000::AdcSpi() -- read from ADC SPI
//---------------------------------------------------------------------------

unsigned int Fmc1000::AdcSpi(unsigned int device, unsigned int address)
{
    return AFEAdcSpi_SDev.Spi(device, address);
}

//---------------------------------------------------------------------------
//  Fmc1000::AdcSpi() -- write to ADC SPI
//---------------------------------------------------------------------------

void Fmc1000::AdcSpi(unsigned int device, unsigned int address, unsigned int data)
{
    AFEAdcSpi_SDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  Fmc1000::PulsorModeEnter() -- ADC Clock Interface
//---------------------------------------------------------------------------

void Fmc1000::PulsorModeEnter()
{
    AFEClock_SDev.Pll().PulsorModeEnter();
}

//---------------------------------------------------------------------------
//  Fmc1000::PulsorModeFire() -- ADC Clock Interface
//---------------------------------------------------------------------------

void Fmc1000::PulsorModeFire(unsigned int Log2PulseCnt)
{
    AFEClock_SDev.Pll().PulsorModeFire(Log2PulseCnt);
}

//---------------------------------------------------------------------------
//  Fmc1000::PulsorModeExit() -- ADC Clock Interface
//---------------------------------------------------------------------------

void Fmc1000::PulsorModeExit()
{
    AFEClock_SDev.Pll().PulsorModeExit();
}

//---------------------------------------------------------------------------
//  Fmc1000::AdcChannelEnabled() -- read channel enabled interface
//---------------------------------------------------------------------------

bool  Fmc1000::AdcChannelEnabled(int ch)
{
    return Input().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  Fmc1000::AdcChannelEnabled() -- set channel enabled interface
//---------------------------------------------------------------------------

void  Fmc1000::AdcChannelEnabled(int ch, bool state)
{
    Input().ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  Fmc1000::AdcActiveChannels() -- count enabled ADC channels
//---------------------------------------------------------------------------

unsigned int  Fmc1000::AdcActiveChannels()
{
    return Input().ActiveChannels();
}

//---------------------------------------------------------------------------
//  Fmc1000::SetDacLaneConfiguration() - pre-set up DAC options
//---------------------------------------------------------------------------
//  This matches FMC-1000 setup

void Fmc1000::SetDacLaneConfiguration(unsigned int /*device*/)
{
    // Load LanePN_Inverted "bits"
    Dac().LanePN_Inverted[0] = 0;    Dac().LanePN_Inverted[1] = 1;
    Dac().LanePN_Inverted[2] = 0;    Dac().LanePN_Inverted[3] = 1;
    Dac().LanePN_Inverted[4] = 1;    Dac().LanePN_Inverted[5] = 0;
    Dac().LanePN_Inverted[6] = 1;    Dac().LanePN_Inverted[7] = 0;

    // DAC physical to logical lanes mapping
    Dac().PhysToLogical_LaneMap[0] = 7;    Dac().PhysToLogical_LaneMap[1] = 6;
    Dac().PhysToLogical_LaneMap[2] = 5;    Dac().PhysToLogical_LaneMap[3] = 4;
    Dac().PhysToLogical_LaneMap[4] = 3;    Dac().PhysToLogical_LaneMap[5] = 2;
    Dac().PhysToLogical_LaneMap[6] = 1;    Dac().PhysToLogical_LaneMap[7] = 0;
}

//---------------------------------------------------------------------------
//  Fmc1000::DacSpi() -- read from DAC SPI
//---------------------------------------------------------------------------

unsigned int Fmc1000::DacSpi(unsigned int device, unsigned int address)
{
    return AFEDacSpi_SDev.Spi(device, address);
}

//---------------------------------------------------------------------------
//  Fmc1000::DacSpi() -- write to DAC SPI
//---------------------------------------------------------------------------

void Fmc1000::DacSpi(unsigned int device, unsigned int address, unsigned int data)
{
    AFEDacSpi_SDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  Fmc1000::DacChannelEnabled() -- read channel enabled interface
//---------------------------------------------------------------------------

bool  Fmc1000::DacChannelEnabled(int ch)
{
    return Output().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  Fmc1000::DacChannelEnabled() -- set channel enabled interface
//---------------------------------------------------------------------------

void  Fmc1000::DacChannelEnabled(int ch, bool state)
{
    Output().ChannelEnabled(ch, state);
}

//===========================================================================
//  CLASS Fmc1000_AdcCalibrationControl -- Fmc AFE ADC Cal Control "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Fmc1000_AdcCalibrationControl
//---------------------------------------------------------------------------

Fmc1000_AdcCalibrationControl::Fmc1000_AdcCalibrationControl(IRequires_Fmc1000_AdcCalibration_Io * mdc,
                                                             IAddressingSpace & phy_cal_space,
                                                             MapRegisters & regs,
                                                             unsigned int device_id)
    :  MDC(mdc),
       AdcPhyCalReg(phy_cal_space, regs.PhyCal_Addr),
       DeviceID(device_id)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc1000_AdcCalibrationControl
//---------------------------------------------------------------------------

Fmc1000_AdcCalibrationControl::~Fmc1000_AdcCalibrationControl()
{
}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc1000_AdcCalibrationControl::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc1000_AdcCalibrationControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc1000_AdcCalibrationControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
    AdcLatencyCalibration();
}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::AdcLatencyCalibration()
//---------------------------------------------------------------------------

void Fmc1000_AdcCalibrationControl::AdcLatencyCalibration()
{
    Debug.Log("Fmc1000_AdcCalibrationControl::AdcLatencyCalibration()");

    AdcPhyCal().Value(0);

    // Store the current setting of the Crosspoint switch
    unsigned int CurrentSyncXPSValue = MDC->Sync_XPS().Value();

    // Temporarily connect the System Reference to the testpoint output
    MDC->Sync_XPS().Switch(DS90CP04Switch::ocOut3, DS90CP04Switch::icIn3);

    AdcExtSyncCalibration();

    // Run the 1st stage latency calibration
    AdcPhyCal().LatencyCalStart1.Value(true);
    Sleep(1);
    AdcPhyCal().LatencyCalStart1.Value(false);

    // Check the 1st stage latency calibration done register
    if (AdcPhyCal().LatencyCalDone1.Value() == true)
        Debug.Log("ADC 1st stage latency calibration done");
    else
        Debug.Log("ADC 1st stage latency calibration failed");

    if (!IsInDdcMode())
        {
        // Disable the sysref
        Spi(0x120, 0x0);

        // Enable the pulsar mode
        MDC->PulsorModeEnter();

        // Enable continuous sysref
        Spi(0x120, 0x2);

        // Start the 2nd stage latency calibration
        AdcPhyCal().LatencyCalStart2.Value(true);

        // Generate pulses on the sysref
        MDC->PulsorModeFire(0);
        MDC->PulsorModeFire(0);

        // Disable the sysref
        Spi(0x120, 0x0);

        // Disable the 2nd stage latency calibration
        AdcPhyCal().LatencyCalStart2.Value(false);

        // Exit the pulsar mode
        MDC->PulsorModeExit();

        // Enable continuous sysref
        Spi(0x120, 0x2);

        // Check the 2nd stage latency calibration done register
        if (AdcPhyCal().LatencyCalDone2.Value() == true)
            Debug.Log("ADC 2nd stage latency calibration done");
        else
            Debug.Log("ADC 2nd stage latency calibration failed");
        }

    // Restore the Crosspoint switch value
    MDC->Sync_XPS().Value(CurrentSyncXPSValue);

    // Apply the Sync cross point switch to the logic
    MDC->Sync_XPS().Apply();

    Debug.Log("end Fmc1000_AdcCalibrationControl::AdcLatencyCalibration()");
}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::AdcExtSyncCalibration()
//---------------------------------------------------------------------------

void Fmc1000_AdcCalibrationControl::AdcExtSyncCalibration()
{
    // Connect the System Reference to the AdcExternalSync
    MDC->Sync_XPS().Switch(DS90CP04Switch::ocOut1, DS90CP04Switch::icIn3);

    // Apply the Sync cross point switch to the logic
    MDC->Sync_XPS().Apply();

    // Run the AdcExternalSync calibration
    AdcPhyCal().ExtSyncCalStart.Value(true);
    AdcPhyCal().ExtSyncCalStart.Value(false);

    // Poll the AdcExternalSync calibration done register
    for (int i=0; i<5; i++)
        {
        if (AdcPhyCal().ExtSyncCalDone.Value() == true)
            {
            Debug.Log("ADC external Sync calibration done");
            break;
            }
        else if (i<4)
            Sleep(1);
        else
            Debug.Log("ADC external Sync calibration timeout");
        }
}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  Fmc1000_AdcCalibrationControl::Spi(unsigned int address, unsigned int data)
{
    MDC->AdcSpi(DeviceID, address, data);
}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  Fmc1000_AdcCalibrationControl::Spi(unsigned int address)
{
    return MDC->AdcSpi(DeviceID, address);
}

//---------------------------------------------------------------------------
//  Fmc1000_AdcCalibrationControl::IsInDdcMode() -- User friendly call
//---------------------------------------------------------------------------

bool Fmc1000_AdcCalibrationControl::IsInDdcMode()
{
    unsigned int data = Spi(0x200);

    if ((data & 3) == 0)  // Fullbandwidth
        return false;
    else
        return true;
}

//---------------------------------------------------------------------------
//  Fmc1000::AfterFiclStart() --
//---------------------------------------------------------------------------

void  Fmc1000::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant AdcSpi0  102 constant Pll");
    engine.Evaluate("  103 constant TempI2C  104 constant kIdromI2C 105 constant PllImage");
    engine.Evaluate("  106 constant DacSpi0");
    engine.Evaluate("  108 constant AdcJesd  109 constant DacJesd");
    engine.Evaluate("  110 constant JesdPhyIP");
    engine.Evaluate(": ?Fmc1000 "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   AdcSpi0  Pll  \" "
                    "    cr .\"   TempI2C  IdromI2C  PllImage \" "
                    "    cr .\"   DacSpi0 \" "
                    "    cr .\"   AdcJesd  DacJesd  JesdPhyIP\" "
                    "    cr  "
                    "    cr .\"   ?Fmc1000  : display this message \" "
                    "    cr  ; ");

    engine.Display("Fmc1000-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  Fmc1000::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  Fmc1000::Do_DFetch(int dev, int addr)
{
    const int kFmc       = 0;  // from base board
    const int kAdcSpi0   = 100;
    const int kPll       = 102;
    const int kTempI2C   = 103;
    const int kIdromI2C  = 104;
    const int kPllImage  = 105;
    const int kDacSpi0   = 106;
    const int kAdcJesd   = 108;
    const int kDacJesd   = 109;
    const int kJesdPhyIP = 110;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:
            return AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr);
        case kDacSpi0:
            return AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr);
        case kAdcJesd:
            return JESD_ADC_SDev.Peek(addr);
        case kDacJesd:
            return JESD_DAC_SDev.Peek(addr);
        case kJesdPhyIP:
            return PeekJESDPHY(0, addr);
        case kPll:
            return AFEClock_SDev.Pll().Port(addr);
        case kPllImage:
            return AFEClock_SDev.Pll().PortImage(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  Fmc1000::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  Fmc1000::Do_DStore(int dev, int addr, int value)
{
    const int kFmc       = 0;  // from base board
    const int kAdcSpi0   = 100;
    const int kPll       = 102;
    const int kTempI2C   = 103;
    const int kIdromI2C  = 104;
    const int kPllImage  = 105;
    const int kDacSpi0   = 106;
    const int kAdcJesd   = 108;
    const int kDacJesd   = 109;
    const int kJesdPhyIP = 110;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:
            AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr, value);
            break;
        case kDacSpi0:
            AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr, value);
            break;

        case kAdcJesd:
            JESD_ADC_SDev.Poke(addr, value);
            break;
        case kDacJesd:
            JESD_DAC_SDev.Poke(addr, value);
            break;
        case kJesdPhyIP:
            PokeJESDPHY(0, addr, value);
            break;
        case kPll:
            AFEClock_SDev.Pll().Port(addr, value);
            break;
        case kPllImage:
            AFEClock_SDev.Pll().PortImage(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  Fmc1000::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  Fmc1000::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 102;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  Fmc1000::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  Fmc1000::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 102;

    switch (dev)
        {
        default:
        case kPll:
            return static_cast<float>(AFEClock_SDev.Pll().FrequencyActual());
        case kFmc:      // no frequency setting
            return 0.0;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
