// Fmc_1000_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_1000_MbH
#define Fmc_1000_MbH

#include <FmcBoard_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>
#include <JESDSubDevice_Stream_Mb.h>
#include <AD9680_SubDevice_Mb.h>
#include <DAC3xJ82_SubDevice_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Fmc1000;

//==============================================================================
//  CLASS Fmc1000_Pll -- Fmc PLL Chip (Specialization of Lmk04820)
//==============================================================================

class Fmc1000_Pll : public Lmk04820
{
public:
    Fmc1000_Pll(Fmc1000 * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~Fmc1000_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    bool  IsFeedbackOutputAdc();

protected:
    Fmc1000 *           Owner;
    double              FTrueFrequency;
    double              FPDF;
    unsigned int        FDacEffectiveFactor;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

//    virtual unsigned int AdcEffectiveFactor() {  return 1;  }
    virtual unsigned int DacEffectiveFactor();


};

//=============================================================================
//  CLASS Fmc1000_LMK20_Clock  --  FMC1000 customization of LMK20 Clock Subdev
//=============================================================================

class Fmc1000_LMK20_Clock : public FmcLMK20_Clock
{
public:
    Fmc1000_LMK20_Clock(Fmc1000 * owner, IRequires_Lmk20FmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs )
        :  FmcLMK20_Clock(mdc, space, regs), Owner(owner)
        {}
    virtual ~Fmc1000_LMK20_Clock() {}

protected:
    virtual void                        CustomExternalClkSelect(IFmcClockIo::IIClockSelect /*src*/); //{  }
    virtual IFmcClockIo::IIClockSelect  CustomExternalClkSelect() const; // {  return IFmcClockIo::cslFrontPanel;  }

private:
    Fmc1000 * Owner;
};

//===========================================================================
//  CLASS IRequires_Fmc1000_AdcCalibration_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_Fmc1000_AdcCalibration_Io
{
public:
    // Io Interface
    virtual void         AdcSpi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int AdcSpi(unsigned int device, unsigned int address) = 0;

    virtual DS90CP04Switch &  Sync_XPS() = 0;

    // Calibration
    virtual void PulsorModeEnter() = 0;
    virtual void PulsorModeFire(unsigned int Log2PulseCnt) = 0;
    virtual void PulsorModeExit() = 0;

    // Customization Interface
    virtual ~IRequires_Fmc1000_AdcCalibration_Io()
        {}
};

//===========================================================================
//  CLASS Fmc1000_AdcCalibrationControl -- Fmc AFE ADC Cal Control "SubDevice"
//===========================================================================

class Fmc1000_AdcCalibrationControl : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS Fmc1000_AdcCalibrationControl::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int PhyCal_Addr;
    };

    Fmc1000_AdcCalibrationControl(IRequires_Fmc1000_AdcCalibration_Io * owner,
                                  IAddressingSpace & phy_cal_space,
                                  MapRegisters & regs,
                                  unsigned int device_id);
    virtual ~Fmc1000_AdcCalibrationControl();

    //
    //  Methods
    void         Spi(unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int address);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc1000_AdcCalibrationControl::AdcPhyCalRegister  --  Adc Calibration Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcPhyCalRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        PowerGood;
        RegisterBit        ExtSyncCalStart;
        RegisterBit        ExtSyncCalDone;
        RegisterBit        LatencyCalStart1;
        RegisterBit        LatencyCalStart2;
        RegisterBit        LatencyCalDone1;
        RegisterBit        LatencyCalDone2;

        AdcPhyCalRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              PowerGood(*this, 8),
              ExtSyncCalStart(*this, 12),
              ExtSyncCalDone(*this, 13),
              LatencyCalStart1(*this, 16),
              LatencyCalStart2(*this, 17),
              LatencyCalDone1(*this, 18),
              LatencyCalDone2(*this, 19)
        {  }
    };

    AdcPhyCalRegister &  AdcPhyCal() {  return AdcPhyCalReg;  }

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void ConfigureHardware();

private:
    //
    //  Registers
    IRequires_Fmc1000_AdcCalibration_Io *  MDC;
    AdcPhyCalRegister                      AdcPhyCalReg;
    const unsigned int                     DeviceID;

    bool IsInDdcMode();
    void AdcLatencyCalibration();
    void AdcExtSyncCalibration();

};

//===========================================================================
//  CLASS ISuppliesAdded_1000_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_1000_Requirements : public IRequires_Lmk20FmcClocks,
                                         public IRequires_MuxedAdcSpi,       // or one with wider address lines?
                                         public IRequires_FmcCalIDROM,
                                         public IRequires_JESDPHY_Io,
                                         public IRequires_JESDADC_Io,
                                         public IRequires_JESDDAC_Io,
                                         public IRequires_AD9680_ADC_Io,
                                         public IRequires_DAC3xJ82_Dac_Io,
                                         public IRequires_Fmc1000_AdcCalibration_Io
{
public:
    virtual ~ISuppliesAdded_1000_Requirements()
        {}

};

//===========================================================================
//  CLASS Fmc1000  -- Fmc Daughter card Common baseboard class
//===========================================================================

class Fmc1000 : public FmcBidirBoard, public ISuppliesAdded_1000_Requirements
{
public:
    Fmc1000();
    virtual ~Fmc1000()
        {}

    class Fmc1000InputDevice &    Input();
    class Fmc1000OutputDevice &   Output();

    friend class Fmc1000InputDevice;
    friend class Fmc1000OutputDevice;
    friend class Fmc1000_LMK20_Clock;

    FmcLMK20_Clock::FmcClockIo &          Clock()           {  return AFEClock_SDev.Clock();  }
    FmcLMK20_Clock::FmcRawClockDevices &  RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    JESD_PHY &        JesdPhy() {  return JESD_PHY_SDev;  }
    AD9680_ADC &      Adc()     {  return ADC_SDev;  }
    DAC3xJ82_DAC &    Dac()     {  return DAC_SDev;  }

    virtual double  Get_DacClockRate()  { return Clock().Dac().Frequency(); }

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Supplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    virtual Lmk04820 *                  MakeFmcAfePllObject();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual FmcCalIdromInfo             GetFmcCalIdromInfo(FmcCalDeviceCode which);

    // JESD PHY IP
    virtual void                        PokeJESDPHY(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDPHY(unsigned int device, unsigned int address);
    virtual IJESD_IoDevice *            GetDacJesd_IoDevice(unsigned int device);
    virtual IJESD_IoDevice *            GetAdcJesd_IoDevice(unsigned int device);
    virtual IJESD_PllRates *            GetJesd_PllRatesDevice(unsigned int device);

    // JESD ADC
    virtual void                        PokeJESDADC(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDADC(unsigned int device, unsigned int address);

    // JESD DAC
    virtual void                        PokeJESDDAC(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDDAC(unsigned int device, unsigned int address);

    // AD9680 ADC
    virtual void                        SetAdcLaneConfiguration(unsigned int device);
    virtual void                        AdcSpi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                AdcSpi(unsigned int device, unsigned int address);
    JESD_ADCDevice &                    JesdAdc(unsigned int /*device*/) {  return JESD_ADC_SDev;  }
    virtual bool                        AdcChannelEnabled(int ch);
    virtual void                        AdcChannelEnabled(int ch, bool state);
    virtual unsigned int                AdcActiveChannels();

    // ADC latency calibration
    virtual DS90CP04Switch &            Sync_XPS() {  return SyncXPS;  }
    virtual void                        PulsorModeEnter();
    virtual void                        PulsorModeFire(unsigned int Log2PulseCnt);
    virtual void                        PulsorModeExit();

    // DAC3xJ82 DAC
    virtual void                        SetDacLaneConfiguration(unsigned int device);
    virtual void                        DacSpi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                DacSpi(unsigned int device, unsigned int address);
    virtual bool                        DacChannelEnabled(int ch);
    virtual void                        DacChannelEnabled(int ch, bool state);
    JESD_DACDevice &                    JesdDac(unsigned int /*device_id*/) {  return JESD_DAC_SDev;  }

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    Fmc1000_LMK20_Clock            AFEClock_SDev;
    FmcCalibrationIDROM_Manager    AdcCalIDROM_SDev;    // Cal Rom Image
    FmcCalibrationIDROM_Manager    DacCalIDROM_SDev;    // Cal Rom Image
    JESD_PHY                       JESD_PHY_SDev;       // JESD PHY IP
    JESD_ADCDevice                 JESD_ADC_SDev;
    JESD_DACDevice                 JESD_DAC_SDev;
    AD9680_ADC                     ADC_SDev;            // ADC Chip Management
    DAC3xJ82_DAC                   DAC_SDev;            // DAC Chip Management
    Fmc1000_AdcCalibrationControl  AFEAdcCalCtrl_SDev;

    Fmc1000InputDevice  *    FInput;
    Fmc1000OutputDevice  *   FOutput;

    DS90CP04Switch    ClockXPS;
    DS90CP04Switch    SyncXPS;

    bool              FDacClocksDisabled;
    bool              FTestClocksDisabled;

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

private:

    void ConfigureAdcs();
    void ConfigureDacs();
};

//===========================================================================
//  CLASS Fmc1000InputDevice  -- FMC 1000 Input Devices
//===========================================================================

class Fmc1000InputDevice : public FmcInputDevice_Bidir
{
public:
    typedef FmcInputDevice_Bidir inherited;

    Fmc1000InputDevice(Fmc1000 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc1000InputDevice();

    virtual unsigned int  ActiveDevices();

    virtual IX6AnalogIoInfo &  Info();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    void   OpenHardware();
    void   PreconfigureHardware();

    virtual void    CustomAdcExternalSyncSource(AfeExtSyncOptions src);

    bool   AdcPowerGood()
        {
        return Owner1000->AFEAdcCalCtrl_SDev.AdcPhyCal().PowerGood.Value();
        }

private:
    Fmc1000 *                   Owner1000;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    GeneralIdromImage &  AdcCalIdrom()  {  return Owner1000->AdcCalIDROM_SDev.Idrom();  }

};

//===========================================================================
//  CLASS Fmc1000OutputDevice  -- FMC 1000 Output Devices
//===========================================================================

class Fmc1000OutputDevice : public FmcOutputDevice_Bidir
{
public:
    friend class Fmc110_DacToHardware;

    typedef FmcOutputDevice_Bidir inherited;

    Fmc1000OutputDevice(Fmc1000 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc1000OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }
    bool  DacPowerGood()
        {
        return DacCommon().DacPowerGood();
        }

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    virtual void CustomDacExternalSyncSource(AfeExtSyncOptions src);

private:
    Fmc1000 *                    Owner1000;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    GeneralIdromImage &  DacCalIdrom()  {  return Owner1000->DacCalIDROM_SDev.Idrom();  }

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
