//---------------------------------------------------------------------------
// Fmc_110_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_110_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 2;
const int             InChannelsPerDevice = 1;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices      = 2;
const int             OutChannelsPerDevice = 1;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;


const unsigned int Fmc_110_VitaInStreamCount   = MaxInDevices;    // Number of VITA Incoming SIDs
const unsigned int Fmc_110_VitaOutStreamCount  = MaxOutDevices;   // Number of VITA Outgoing SIDs

const unsigned int Fmc_110_AdcSpiDeviceCount   = MaxInDevices;    // Number of SPI devices muxed
const unsigned int Fmc_110_DacSpiDeviceCount   = MaxOutDevices;   // Number of SPI devices muxed

const unsigned int Fmc_110_AdcChannelsPerDevice   = InChannelsPerDevice;
const unsigned int Fmc_110_AdcChannels   = MaxInChannels;

const unsigned int Fmc_110_DacChannels   = MaxOutChannels;

const float         InputMaxRate            = 1000.0e6f;
const float         InputMinRate            = 5.0e6f;
const unsigned int  InputWidth_Bits         = 12;
const unsigned int  InputShift_Factor       = 4;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int Fmc_110_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc110_Adc_One = 0x10000;         // representation of floating point 1.0

const float         OutputMaxRate            = 1000.0e6f;
const float         OutputMinRate            = 5.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 1000;
const          int  OutputMin_mV             = -1000;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 1;

const unsigned int Fmc_110_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
const unsigned int Fmc110_Dac_One = 0x10000;         // representation of floating point 1.0
//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
//const int romFmc_110_SysInfoOffset        = 0;
//const int romFmc_110_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  DUMMY IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_110_InputCoeffOffset       = 0;
const int romFmc_110_InputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients (on its own page)
const int romFmc_110_OutputCoeffOffset       = 0;
const int romFmc_110_OutputCoeffOffset_avail = 5;  // 2 * 2 coefficients plus 'calibrated' word


//===========================================================================
// Class Fmc110_AnalogInInfo
//===========================================================================

class Fmc110_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    Fmc110_AnalogInInfo(Fmc110 * owner) : Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~Fmc110_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
	virtual unsigned int  Bits() const
        {
        unsigned int bits = InputWidth_Bits;
        if (Owner->AdcAmplitude_SDev.ShiftIsOn())
            bits *= InputShift_Factor;
        return bits;
        }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    Fmc110 *   Owner;
    SpanInfo   FSpan;
};

//===========================================================================
// Class Fmc110_AnalogOutInfo
//===========================================================================

class Fmc110_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    Fmc110_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~Fmc110_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
	virtual float     MinRate() const         {  return OutputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS Fmc110InputDevice  -- Fmc110 Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Fmc110InputDevice
//---------------------------------------------------------------------------

Fmc110InputDevice::Fmc110InputDevice(Fmc110 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner),   FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_110_InputCoeffOffset, MaxInChannels,
                       Fmc110_Adc_One, AdcCal() ),
      AdcDeviceTestEnabled(false), CurrentAdcDeviceTestPattern(0x01)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc110InputDevice
//---------------------------------------------------------------------------

Fmc110InputDevice::~Fmc110InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc110InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc110InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc110InputDevice::Calibration() -- parent Calibration SDEV
//---------------------------------------------------------------------------

Fmc110_AdcCalibrationControl &  Fmc110InputDevice::Calibration()
{
    return DOwner->Calibration();
}

//---------------------------------------------------------------------------
//  Fmc110InputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void Fmc110InputDevice::AdcDeviceTestMode(bool enabled, unsigned int test_pattern)
{
    AdcDeviceTestEnabled = enabled;
    CurrentAdcDeviceTestPattern = test_pattern;
}

//---------------------------------------------------------------------------
//  Fmc110InputDevice::AdcShiftInput() - Shift Input to match output
//---------------------------------------------------------------------------

void Fmc110InputDevice::AdcShiftInput(bool shift)
{
    DOwner->AdcAmplitude_SDev.ShiftInput(shift);
}

//---------------------------------------------------------------------------
//  Fmc110InputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void Fmc110InputDevice::Adc_OnConfigure()
{

    //
    //  ...init ADC devices
    for (unsigned int dev=0; dev<MaxInDevices; dev++)
        {
        if (DeviceEnabled(dev)==false)
            {
            continue;
            }
        DOwner->Debug.Log("Fmc110 Adc Calibration Device: ", dev);
        for (int i=0; i<5; i++)
            {
            //
            //  Adc Calibration
            Calibration().StartCalibration(dev);
            Calibration().WaitForCal();
            unsigned int results = Calibration().CalibrationResults(dev);

            DOwner->Debug.Log("Fmc110 Adc Cal Results: ", results, 16);
            if (Calibration().CalibrationDone()==false)
                {
                DOwner->Debug.Log("...cal failed: ");
                }
            else
                break;
            }
        }
}

//===========================================================================
//  CLASS Fmc110_DacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class Fmc110_DacToHardware
//---------------------------------------------------------------------------

class Fmc110_DacToHardware :  public TiDac5682::IDacToHardware
{
public:
    virtual unsigned short  Spi(int addr)
        {  return Out->Spi(Dev, addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Out->Spi(Dev, addr, value);  }

    virtual double          DCLK(bool /*single_channel*/)
        {  return MyPll().OutputFrequency(6) ;  }
    virtual bool            IsDacDllEnabled(bool /*single_channel*/)
        {
        double FreqMhz = MyPll().Dac().Frequency()/1e6;

        if (FreqMhz>=250.0)
            return true;
        else
            return false;
        }
    virtual unsigned int    FastConfigValue()
        {  return 0xC0;  }

    Fmc110_DacToHardware( Fmc110OutputDevice * out, unsigned int dev=0 )
        : Out(out), Dev(dev)
        {}
    virtual ~Fmc110_DacToHardware()
        {}

    Fmc110_Pll &  MyPll()
        {  return dynamic_cast<Fmc110_Pll &>(Out->DOwner->RawClockDevice().Pll());  }

    Fmc110OutputDevice *   Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS Fmc110OutputDevice  -- FMC 110 Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc110OutputDevice
//---------------------------------------------------------------------------

const bool DevIs5681 = true;

//---------------------------------------------------------------------------
//  constructor for class Fmc110OutputDevice
//---------------------------------------------------------------------------

Fmc110OutputDevice::Fmc110OutputDevice(Fmc110 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner), FInfo(info),
      DacDev_0(new Fmc110_DacToHardware(this, 0), DevIs5681),
      DacDev_1(new Fmc110_DacToHardware(this, 1), DevIs5681),
      CalRomSection(DacCalIdrom(), romFmc_110_OutputCoeffOffset, MaxOutChannels,
                       Fmc110_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc110OutputDevice
//---------------------------------------------------------------------------

Fmc110OutputDevice::~Fmc110OutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc110OutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc110OutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc110OutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc110OutputDevice::PreconfigureHardware()
{
    DOwner->Cpld().SetDacReset(false);
    DOwner->Cpld().SetDacReset(true);
    DOwner->Cpld().SetDacReset(false);

    DacDev_0.PreconfigHardware();  // after FPGA PLL Lock
    DacDev_1.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  Fmc110OutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc110OutputDevice::ConfigureHardware()
{
    DacDev_0.ClearDacTestStatus();
    DacDev_1.ClearDacTestStatus();
}

//---------------------------------------------------------------------------
//  Fmc110OutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  Fmc110OutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    Device(0).Enabled( DacCommon().ChannelEnabled(0) );
    Device(0).ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device(0).ChannelEnabled(1, false );  // single channel only

    Device(1).Enabled( DacCommon().ChannelEnabled(1) );
    Device(1).ChannelEnabled(0, DacCommon().ChannelEnabled(1) );
    Device(1).ChannelEnabled(1, false );  // single channel only
}

//=============================================================================
//  CLASS Fmc110_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc110_Pll
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

const float  Input_ReferenceClk = 100.0e6;

//X6Ad9516_Tb_Hdw::MapRegisters Fmc110ClkRegAddrs =
//    {
//    mmFmc_PllControl,       //  AfePllControl_Addr
//    mmFmc_PllSpi,           //  AfePllSpi_Addr
//    };

//---------------------------------------------------------------------------
//  constructor for class Fmc110_Pll
//---------------------------------------------------------------------------

Fmc110_Pll::Fmc110_Pll(Fmc110 * owner, IAddressingSpace &space, X6Ad9516_Tb_Hdw::MapRegisters & reg)
    : Ad9517_Tb(new Fmc110_Ad9517_Tb_Hdw(owner->Cpld(), space, reg)), Owner(owner), FPDF(10.0e6)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 100
    ReferenceFrequency(1, Input_ReferenceClk);

    InternalVco(false);  // we use external VC)
    VcoRange(950.0e6, 1050.0e6);  // find real values?
}

//---------------------------------------------------------------------------
//  Fmc110_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  Fmc110_Pll::OnInit()
{
    //
    //  Set Output Configuration
    OutputInUse(0, true);   OutputEnabled(0, true);  // ADC1
    OutputInUse(1, true);   OutputEnabled(1, true);  // ADC0
    OutputInUse(2, true);   OutputEnabled(2, true);  // DAC1
    OutputInUse(3, true);   OutputEnabled(3, true);  // DAC0
    OutputInUse(4, true);   OutputEnabled(4, true);  // SYNC
    OutputInUse(5, false);  OutputEnabled(5, false);
    OutputInUse(6, true);   OutputEnabled(6, true);  // FPGA
    OutputInUse(7, false);  OutputEnabled(7, false);

    // make output 4 have 25 pct duty cycle, LVDS
    SetOutputConfig(4, ocLvds);
    SetOutputDutyCycle(4, 0.25);

    SetOutputConfig(6, ocLvds);
}

//---------------------------------------------------------------------------
//  Fmc110_Pll::OnReapply()
//---------------------------------------------------------------------------

void  Fmc110_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  Fmc110_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  Fmc110_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  Fmc110_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  Fmc110_Pll::DacMasterOutput()
{
    return 2;
}

//---------------------------------------------------------------------------
//  Fmc110_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  Fmc110_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(0, adc_rate);
    OutputFrequency(1, adc_rate);
    OutputFrequency(4, adc_rate/4);
}

//---------------------------------------------------------------------------
//  Fmc110_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  Fmc110_Pll::SetDacOutputRates(double dac_rate)
{
    OutputFrequency(2, dac_rate);
    OutputFrequency(3, dac_rate);
    OutputFrequency(6, dac_rate/2);

}

//=============================================================================
//  CLASS Fmc110_Ad9517_Tb_Hdw  --  Hardware Interface for Ad951x_Base_Tb
//=============================================================================
//-----------------------------------------------------------------------------
//  constructor for class Fmc110_Ad9517_Tb_Hdw
//-----------------------------------------------------------------------------

Fmc110_Ad9517_Tb_Hdw::Fmc110_Ad9517_Tb_Hdw(Fmc110_Cpld &cpld, IAddressingSpace & space, MapRegisters & regs)
    :  X6Ad9516_Tb_Hdw(space, regs), CPLD(cpld), FExternalClock(false), FExternalReference(true)
{
}

//---------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::SetSelB() -  Select reference clock source
//---------------------------------------------------------------------------

void Fmc110_Ad9517_Tb_Hdw::SetSelB(bool value)
{
    FExternalReference = value;
    SetPllConfiguration();
}

//---------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::SetSelC() -  Specify external clock or PLL
//---------------------------------------------------------------------------

void Fmc110_Ad9517_Tb_Hdw::SetSelC(bool value)
{
    FExternalClock = value;
    SetPllConfiguration();
}

//---------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::SetPllConfiguration() -  Set Clock/Ref source
//---------------------------------------------------------------------------

void Fmc110_Ad9517_Tb_Hdw::SetPllConfiguration()
{
    Fmc110_Cpld::CpldClkSources setting;

    if (FExternalClock)
        setting = Fmc110_Cpld::cpcsExternal;
    else if (FExternalReference)
        setting = Fmc110_Cpld::cpcsExtRef;
    else
        setting = Fmc110_Cpld::cpcsInternal;

    CPLD.SetClkSrc(setting);
}

//-----------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::IsLocked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  Fmc110_Ad9517_Tb_Hdw::IsLocked()
{
    return CPLD.GetPllLocked();
}

//-----------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::ResetPll()  --  Reset PLL
//-----------------------------------------------------------------------------

void  Fmc110_Ad9517_Tb_Hdw::ResetPll()
{
//  Reset blows away register state?  So don't do it??
//    CPLD.SetPllReset(true);
//    Sleep(1);
//    CPLD.SetPllReset(false);
}

//-----------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::ResetFpgaPll()  --  Reset FPGA PLL
//-----------------------------------------------------------------------------

void  Fmc110_Ad9517_Tb_Hdw::ResetFpgaPll()
{
    PllControlReg.Bit(5, true);
    Sleep(1);
    PllControlReg.Bit(5, false);
}

//-----------------------------------------------------------------------------
//  Fmc110_Ad9517_Tb_Hdw::IsFpgaPllLocked()  --  Check FPGA PLL Locked
//-----------------------------------------------------------------------------

bool  Fmc110_Ad9517_Tb_Hdw::IsFpgaPllLocked()
{
    return PllStatusReg.Locked.Value();  // happens to use the old PLL locked bit
}

//=============================================================================
//  CLASS Fmc110_AdcCalOverride  --  Cal Override for FMC ADCs
//=============================================================================

class Fmc110_AdcCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    Fmc110_AdcCalOverride()
        {}
    virtual ~Fmc110_AdcCalOverride()
        {}

private:
    Trace  Debug;
};

//---------------------------------------------------------------------------
//  Fmc110_AdcCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  Fmc110_AdcCalOverride::OverrideOffset(unsigned int value)
{
    Debug.Log("OverrideOffset: Original=", value, 16);
    int offset = static_cast<int>(value);   // need signed offset
    //  Max offset that can be calibrated by the hardware (120uVsteps)
    int offset_cal_range_counts = static_cast<int>(30.72/((float)InputMax_mV/(1<<(InputWidth_Bits-1))));
    //  Trim range to fit hardware limitations
    if (offset>0 && offset>offset_cal_range_counts)
        offset = offset_cal_range_counts;
    else if (offset<0 && offset>offset_cal_range_counts)
        offset = -offset_cal_range_counts;
    //
    //  Convert 'in range' offset to acutal units of 1/4 LSB, unipolar mode
    offset +=  offset_cal_range_counts;
    offset <<= 2;

    Debug.Log("OverrideOffset: New=", offset, 16);
    return static_cast<unsigned int>(offset);
}

//---------------------------------------------------------------------------
//  Fmc110_AdcCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  Fmc110_AdcCalOverride::OverrideGain(unsigned int value)
{
    Debug.Log("OverrideGain: Original=", value, 16);
    const float HdwStep_mV = 0.120f;

    float gain = static_cast<float>(value)/Fmc110_Adc_One;
    if (gain<=1.0)
        return 0;   // no gain shift

    float full_range_mV =  2.0*InputMax_mV;
    float adjustment_mV =  full_range_mV*(1.0f-1.0f/gain);
    unsigned int hdw_steps = static_cast<unsigned int>(adjustment_mV/HdwStep_mV);
    Debug.Log("ADC OverrideGain: New=", hdw_steps, 16);
    return hdw_steps;
}

//===========================================================================
//  CLASS Fmc110_AdcAmplitudeControl -- X6 ADC Amplitude "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class Fmc110_AdcAmplitudeControl
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class Fmc110_AdcAmplitudeControl
//---------------------------------------------------------------------------

Fmc110_AdcAmplitudeControl::Fmc110_AdcAmplitudeControl(IAddressingSpace & space, MapRegisters & regs)
    :  AmplControlReg(space, regs.AmplitudeControl_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc110_AdcAmplitudeControl
//---------------------------------------------------------------------------

Fmc110_AdcAmplitudeControl::~Fmc110_AdcAmplitudeControl()
{
}

//---------------------------------------------------------------------------
//  Fmc110_AdcAmplitudeControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc110_AdcAmplitudeControl::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  Fmc110_AdcAmplitudeControl::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc110_AdcAmplitudeControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  Fmc110_AdcAmplitudeControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc110_AdcAmplitudeControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    //  No default behavior

}

//---------------------------------------------------------------------------
//  Fmc110_AdcAmplitudeControl::ShiftInput()
//---------------------------------------------------------------------------

void  Fmc110_AdcAmplitudeControl::ShiftInput(bool shift_it)
{
    ScopedTraceRef ccc(Debug, "Fmc110_AdcAmplitudeControl::ShiftInput");

    if (shift_it)
        AmplControlReg.DevShiftOn.Value(0x3);
    else
        AmplControlReg.DevShiftOn.Value(0x0);

    Debug.Log("ADC Shift Val: ", AmplControlReg.DevShiftOn.Value() );
}
//---------------------------------------------------------------------------
//  Fmc110_AdcAmplitudeControl::ShiftIsOn()
//---------------------------------------------------------------------------

bool  Fmc110_AdcAmplitudeControl::ShiftIsOn()
{
    return AmplControlReg.DevShiftOn.Value() != 0;
}

//=============================================================================
//  CLASS Fmc110_DummyIDROM  --  "Fake" IDROM
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc110_DummyIDROM
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  Fmc110_DummyIDROM::ROM_ReadBlock() - read from ROM device
//---------------------------------------------------------------------------

void  Fmc110_DummyIDROM::ROM_ReadBlock(int /*offset*/, int * /*data*/, int /*words*/)
{
    // we don't have a device so do nothing
}

//---------------------------------------------------------------------------
//  Fmc110_DummyIDROM::ROM_WriteBlock() - write to ROM device
//---------------------------------------------------------------------------

void  Fmc110_DummyIDROM::ROM_WriteBlock(int /*offset*/, const int * /*data*/, int /*words*/)
{
    // we don't have a device so do nothing
}

//===========================================================================
//  CLASS Fmc110  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc110
//---------------------------------------------------------------------------

namespace Fmc110_MemoryMap
{
    //
    //  WB_FMC2 - FMC ADC AFE
    const unsigned int  mmAFE_1000M_Adc_Amplitude  = 0x19;

} // end namespace Fmc110_MemoryMap


namespace
{

X6Ad9516_Tb_Hdw::MapRegisters FmcAFEClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

//static Cdce62005Registers Fmc110_PllRegisters =
//    { mmFmc_PllControl,  mmFmc_PllSpi  };

FmcCpldSpiDevice::Info  FmcCpldInfo =
    {
    mmFmc_CpldSpiStatus,  //  Status_Addr;
    mmFmc_CpldSpi,        //  Data_Addr;
    };

Fmc110_AdcCalibrationControl::MapRegisters Fmc110AdcCalRegAddrs =
    {
    mmFmc_AdcCalStatus_T2,      //  AfeAdcCal_Addr;
    };

Fmc110_AdcAmplitudeControl::MapRegisters Fmc110_AmplitudeRegAddrs =
    {
    Fmc110_MemoryMap::mmAFE_1000M_Adc_Amplitude,      //  AmplitudeControl_Addr;
    };

}

//---------------------------------------------------------------------------
//  constructor for class Fmc110
//---------------------------------------------------------------------------

Fmc110::Fmc110()
    :  FmcBidirBoard(fmcPrimary),
       AFEClock_SDev(this, this),
       Cpld_SDev(this, WB_FMC_1, FmcCpldInfo),
       AFEAdcCalCtrl_SDev(this, WB_FMC_2, Fmc110AdcCalRegAddrs),
       AdcAmplitude_SDev( WB_FMC_2, Fmc110_AmplitudeRegAddrs),
       FInput(new Fmc110InputDevice(this, new Fmc110_AnalogInInfo(this))),
       FOutput(new Fmc110OutputDevice(this, new Fmc110_AnalogOutInfo))
{
    LocalConstructCustomParts();
}

Fmc110 * an110()
{
    Fmc110 * wombat = new Fmc110;
    return wombat;
}

//---------------------------------------------------------------------------
//  Fmc110::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  Fmc110::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("Fmc110");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(100);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 20);
//        void    ModuleSize(Fmc::FmcModuleSize size);
//        void    ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size);
//        void    ClockDirection(Fmc::FmcClockDirection dir);
//        void    BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank, unsigned int count);
//        void    TransceiverCount(Fmc::FmcConnector p, unsigned int count);
//        void    MaxClockMHz(unsigned int rateMHz )   {  MaxClock_1 = rateMHz;  }

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(3.3f);
    DcOut->RipplePkPk(0.5f);
//        void          Standby(bool state);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)   {  NominalV_2   = To10mV(v);  }
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          RipplePkPk(float v) {  RipplePkPk_2 = To1mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(3.3f);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)        {  NominalV_2   = To10mV(v);  }
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  Fmc110::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc110::LocalConstructCustomParts()
{
    FmcBidirBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    Cpld_SDev.ConstructCustomParts();
    AFEAdcCalCtrl_SDev.ConstructCustomParts();
    AdcAmplitude_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  Fmc110::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc110::OpenHardware()
{
    FmcBidirBoard::OpenHardware();

    AFEClock_SDev.OpenHardware();
    Cpld_SDev.OpenHardware();
    AFEAdcCalCtrl_SDev.OpenHardware();
    AdcAmplitude_SDev.OpenHardware();

    AdcCalibrationIdrom.Clear();  // no need to read dummy ROMs
    DacCalibrationIdrom.Clear();

}

//---------------------------------------------------------------------------
//  Fmc110::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  Fmc110::PreconfigureHardware()
{
    Output().ConfigureChannelEnables();

    FmcBidirBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();
    Cpld_SDev.PreconfigureHardware();
    AFEAdcCalCtrl_SDev.PreconfigureHardware();
    AdcAmplitude_SDev.PreconfigureHardware();

    //  Configure Devices
    Ad951x_Base_Tb & pll = RawClockDevice().Pll();

    //  ...delay to allow PLL to settle
    Sleep(100);
    //  ...wait for PLL Lock
    if (pll.OutputSource() == Ad951x_Base_Tb::outPll)
        {
        for (unsigned int i=0; i<6; i++)
            {
            bool lock_bit = pll.Locked();
            if (lock_bit)
                {
                Debug.Log("PLL Locked!");
                break;
                }
            Innovative::Sleep(1);
            if (i==5)
                Debug.Log("ERROR: PLL Lock not achieved in 5 ms...");
            }
        }
    //  ...reset FPGA Pll
    pll.HdwIntf()->ResetFpgaPll();
    //  ...wait for FPGA PLL Lock
    for (unsigned int i=0; i<6; i++)
        {
        bool lock_bit = pll.HdwIntf()->IsFpgaPllLocked();
        if (lock_bit)
            {
            Debug.Log("FPGA PLL Locked!");
            break;
            }
        Innovative::Sleep(1);
        if (i==5)
            Debug.Log("ERROR: FPGA PLL Lock not achieved in 5 ms...");
        }

    Output().PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc110::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc110::ConfigureHardware()
{
    FmcBidirBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    Cpld_SDev.ConfigureHardware();
    AFEAdcCalCtrl_SDev.ConfigureHardware();
    AdcAmplitude_SDev.ConfigureHardware();

    Ad951x_Base_Tb & pll = RawClockDevice().Pll();
    //  ...wait for FPGA PLL Loc
	bool lock_bit = pll.HdwIntf()->IsFpgaPllLocked();
    if (lock_bit)
        {
        Debug.Log("At Config FPGA PLL Locked!");
        }
	else
        {
		Debug.Log("ERROR: At Config FPGA PLL Lock not achieved in 5 ms...");
		}
	//
    //  Special Config for A/D Devices
    Input().Adc_OnConfigure();

    Output().ConfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc110::StreamStopHardware()
//---------------------------------------------------------------------------

void  Fmc110::StreamStopHardware()
{
    FmcBidirBoard::StreamStopHardware();

    AFEClock_SDev.StreamStopHardware();
    Cpld_SDev.StreamStopHardware();
    AFEAdcCalCtrl_SDev.StreamStopHardware();
    AdcAmplitude_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  Fmc110::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc110::GetVitaInStreamCount()
{
    return Fmc_110_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc110::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc110::GetVitaOutStreamCount()
{
    return Fmc_110_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc110::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc110::GetAdcSpiDeviceCount()
{
    return Fmc_110_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc110::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc110::GetDacSpiDeviceCount()
{
    return Fmc_110_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc110::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  Fmc110::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new FmcStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  Fmc110::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  Fmc110::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new FmcStandardPriClockDevice(this, 2);
}


//---------------------------------------------------------------------------
//  Fmc110::MakeAfe_Ad951xPllObject() -- Make Ad9516 Pll
//---------------------------------------------------------------------------

Ad9517_Tb *  Fmc110::MakeAfe_Ad951xPllObject()
{
    //  Make our local PLL object (subdev will delete it)
    Ad9517_Tb * pll = new Fmc110_Pll(this, WB_FMC_1, FmcAFEClockRegAddrs );

    return pll;
}

//---------------------------------------------------------------------------
//  Fmc110::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc110::GetAdcCalibrationOverride()
{
    return new Fmc110_AdcCalOverride;
}

//---------------------------------------------------------------------------
//  Fmc110::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc110::GetCalibrationAdcCount()
{
    return Fmc_110_AdcChannels;
}

//---------------------------------------------------------------------------
//  Fmc110::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc110::GetDacCalibrationOverride()
{
    return 0; //new Fmc110_DacCalOverride;
}

//---------------------------------------------------------------------------
//  Fmc110::GetCalibrationDacCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc110::GetCalibrationDacCount()
{
    return Fmc_110_DacChannels;
}

//---------------------------------------------------------------------------
//  Fmc110::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  Fmc110::GetAnalogDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = InChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc110::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  Fmc110::GetAnalogPhysicalDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = InChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc110::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void Fmc110::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = maxFmcVadjCode = 0x4;  // 2.5V only
}

//---------------------------------------------------------------------------
//  Fmc110::Input() -- Reference to input object
//---------------------------------------------------------------------------

Fmc110InputDevice & Fmc110::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  Fmc110::Output() -- Reference to Output object
//---------------------------------------------------------------------------

Fmc110OutputDevice & Fmc110::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  Fmc110::AfterFiclStart() --
//---------------------------------------------------------------------------

void  Fmc110::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant AdcSpi0  101 constant AdcSpi1  ");
    engine.Evaluate("  102 constant DacSpi0  103 constant DacSpi1  104 constant Cpld  105 constant Pll ");
    engine.Evaluate("  106 constant TempI2C  107 constant IdromI2C   ");

    engine.Evaluate(": ?Fmc110 "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   AdcSpi0  AdcSpi1  DacSpi0  DacSpi1  Cpld  Pll  \" "
                    "    cr .\"   TempI2C  IdromI2C   \" "
                    "    cr  "
                    "    cr .\"   ?Fmc110  : display this message \" "
                    "    cr  ; ");

    engine.Display("Fmc110-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  Fmc110::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  Fmc110::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;   const int kDacSpi0 = 102;
    const int kDacSpi1 = 103;     const int kCpld = 104;      const int kPll = 105;
    const int kTempI2C = 106;     const int kIdromI2C = 107;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:    case kAdcSpi1:
            return AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr);
        case kDacSpi0:    case kDacSpi1:
            return AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr);
        case kCpld:
            return Cpld_SDev.Spi(addr);
        case kPll:
            return AFEClock_SDev.Pll().Port(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  Fmc110::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  Fmc110::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;   const int kDacSpi0 = 102;
    const int kDacSpi1 = 103;     const int kCpld = 104;      const int kPll = 105;
    const int kTempI2C = 106;     const int kIdromI2C = 107;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:    case kAdcSpi1:
            AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr, value);
            break;
        case kDacSpi0:    case kDacSpi1:
            AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr, value);
            break;
        case kCpld:
            Cpld_SDev.Spi(addr, value);
            break;
        case kPll:
            AFEClock_SDev.Pll().Port(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  Fmc110::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  Fmc110::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 105;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  Fmc110::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  Fmc110::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 105;

    switch (dev)
        {
        default:
        case kPll:
            return static_cast<float>(AFEClock_SDev.Pll().FrequencyActual());
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
