// Fmc_110_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_110_MbH
#define Fmc_110_MbH

#include <FmcBoard_Mb.h>
#include <X6Support_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <TiDac5682z_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Fmc110;

//===========================================================================
//  CLASS ISuppliesAdded_Fmc110_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_Fmc110_Requirements : public IAd9516Factory//,
                                           //public IX6_Ad9516_RawClockDevices
{
public:
    virtual ~ISuppliesAdded_Fmc110_Requirements()
        {}

};

//==============================================================================
//  CLASS Fmc110_Pll -- Fmc PLL Chip (Specialization of Ad9517_Tb)
//==============================================================================

class Fmc110_Pll : public Ad9517_Tb
{
public:
    Fmc110_Pll(Fmc110 * owner, IAddressingSpace &space, X6Ad9516_Tb_Hdw::MapRegisters & reg);

    // ...timebase interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

protected:
    Fmc110 *            Owner;
    double              FTrueFrequency;
    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//=============================================================================
//  CLASS Fmc110_Ad9517_Tb_Hdw  --  Hardware Interface for Ad9517_Tb
//=============================================================================
//  "Fmc110" version

class Fmc110_Ad9517_Tb_Hdw : public X6Ad9516_Tb_Hdw
{
public:
    //
    //  Interface Implementation
    virtual void  SetSelA(bool /*setting*/) {}
    virtual void  SetSelB(bool setting);
    virtual void  SetSelC(bool setting);
    virtual bool  IsLocked();
    virtual void  ResetPll();

    virtual void  ResetFpgaPll();
    virtual bool  IsFpgaPllLocked();
    //
    //  Member Functions
    Fmc110_Ad9517_Tb_Hdw(Fmc110_Cpld &cpld, IAddressingSpace & space, MapRegisters & regs );

private:
    Fmc110_Cpld &  CPLD;
    bool  FExternalClock;
    bool  FExternalReference;

    void SetPllConfiguration();
};


//===========================================================================
//  CLASS Fmc110_AdcAmplitudeControl -- FMC110 ADC Amp "SubDevice"
//===========================================================================

class Fmc110_AdcAmplitudeControl : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS Fmc110_AdcAmplitudeControl::MapRegisters  -- Dev Reg Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AmplitudeControl_Addr;
    };

    Fmc110_AdcAmplitudeControl(IAddressingSpace & space, MapRegisters & regs);
    virtual ~Fmc110_AdcAmplitudeControl();

    void    ShiftInput(bool shift_it);
    bool    ShiftIsOn();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc110_AdcAmplitudeControl::AmplitudeControlRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AmplitudeControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup    DevShiftOn;

    public:
        AmplitudeControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DevShiftOn(*this, 0, 2)
            {
            }
    };


    AmplitudeControlRegister  AmplControlReg;
};

//=============================================================================
//  CLASS Fmc110_DummyIDROM  --  "Fake" IDROM
//=============================================================================

class Fmc110_DummyIDROM :  public GeneralIdromImage
{
public:
    friend class GeneralRomSection;

    static const int myIdromSizeWords = 2048;   // IDROM size in words
    static const int myReservedSize  = 1;      //  Checksum area
    //
    //  Member Functions
    Fmc110_DummyIDROM() :  GeneralIdromImage(myIdromSizeWords, myReservedSize)
        {}
    virtual ~Fmc110_DummyIDROM()
        {}

protected:

    virtual void ROM_ReadBlock(int offset, int * data, int words);
    virtual void ROM_WriteBlock(int offset, const int * data, int words);

};


//===========================================================================
//  CLASS Fmc110  -- Fmc Daughter card Common baseboard class
//===========================================================================

class Fmc110 : public FmcBidirBoard, public ISuppliesAdded_Fmc110_Requirements
{
public:
    friend class Fmc110InputDevice;
    friend class Fmc110OutputDevice;
    friend class Fmc110_AnalogInInfo;

    Fmc110();
    virtual ~Fmc110()
        {}

    class Fmc110InputDevice  &    Input();
    class Fmc110OutputDevice  &   Output();

    X6AFE_Ad9516Clock::X6ClockIo &          Clock()           {  return AFEClock_SDev.Clock();  }  // -> IFmcClockIo
    X6AFE_Ad9516Clock::X6RawClockDevices &  RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }  //  -> IFmcRawClockDevices

    // pseudo-protected

    Fmc110_Cpld &  Cpld()  {  return Cpld_SDev;  }
    Fmc110_AdcCalibrationControl & Calibration() {  return AFEAdcCalCtrl_SDev;  }

    virtual double  Get_DacClockRate() {  return Clock().Dac().Frequency();  }

    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Suupplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    //virtual Cdce62005 *                 MakeFmcAfePllObject();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual Ad9517_Tb *                 MakeAfe_Ad951xPllObject();


    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    X6AFE_Ad9516Clock                AFEClock_SDev;
    Fmc110_Cpld                      Cpld_SDev;
    Fmc110_AdcCalibrationControl     AFEAdcCalCtrl_SDev;
    Fmc110_AdcAmplitudeControl       AdcAmplitude_SDev;

    Fmc110_DummyIDROM                AdcCalibrationIdrom;
    Fmc110_DummyIDROM                DacCalibrationIdrom;

    Fmc110InputDevice  *    FInput;
    Fmc110OutputDevice  *   FOutput;

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return 100.0e6; /*AFEClock_SDev.Clock().Frequency(); */   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

};

//===========================================================================
//  CLASS Fmc110InputDevice  -- FMC 110 Input Devices
//===========================================================================

class Fmc110InputDevice : public FmcInputDevice_Bidir
{
public:
    typedef FmcInputDevice_Bidir inherited;

    Fmc110InputDevice(Fmc110 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc110InputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    //virtual unsigned int       Channels();

    void   AdcDeviceTestMode(bool enabled, unsigned int test_pattern);
    void   AdcShiftInput(bool shift);

    void   Adc_OnConfigure();

protected:
    Fmc110_AdcCalibrationControl & Calibration();


private:
    Fmc110 *                    DOwner;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    bool          AdcDeviceTestEnabled;
    unsigned int  CurrentAdcDeviceTestPattern;

    GeneralIdromImage &  AdcCalIdrom()  {  return DOwner->AdcCalibrationIdrom;  }
};

//===========================================================================
//  CLASS Fmc110OutputDevice  -- FMC 110 Output Devices
//===========================================================================

class Fmc110OutputDevice : public FmcOutputDevice_Bidir
{
public:
    friend class Fmc110_DacToHardware;

    typedef FmcOutputDevice_Bidir inherited;

    Fmc110OutputDevice(Fmc110 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc110OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    TiDac5682::DacDevice &     Device(unsigned int idx)
        {
        if (idx)
            return DacDev_1;
        else
            return DacDev_0;
        }

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }


private:
    Fmc110 *                DOwner;
    IX6AnalogIoInfo *       FInfo;
    TiDac5682::DacDevice    DacDev_0;
    TiDac5682::DacDevice    DacDev_1;
    GeneralCalibrationSection   CalRomSection;


    GeneralIdromImage &  DacCalIdrom()  {  return DOwner->DacCalibrationIdrom;  }
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
