//---------------------------------------------------------------------------
// Fmc_250_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_250_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 2;
const int             InChannelsPerDevice = 1;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices        = 1;
const int             OutChannelsPerDevice = 2;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;

const unsigned int Fmc_250_VitaInStreamCount   = MaxInDevices;   // Number of VITA Incoming SIDs
const unsigned int Fmc_250_VitaOutStreamCount  = MaxOutDevices;   // Number of VITA Outgoing SIDs

const unsigned int Fmc_250_AdcSpiDeviceCount   = MaxInDevices;   // Number of SPI devices muxed
const unsigned int Fmc_250_DacSpiDeviceCount   = MaxOutDevices;   // Number of SPI devices muxed

const unsigned int Fmc_250_AdcChannelsPerDevice   = InChannelsPerDevice;
const unsigned int Fmc_250_AdcChannels   = MaxInChannels;
const unsigned int Fmc_250_DacChannelsPerDevice   = OutChannelsPerDevice;
const unsigned int Fmc_250_DacChannels   = MaxOutChannels;

const float         InputMaxRate            = 255.0e6f;
const float         InputMinRate            = 5.0e6f;
const unsigned int  InputWidth_Bits         = 16;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int Fmc_250_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_250_Adc_One = 0x10000;         // representation of floating point 1.0

const float         OutputMaxRate            = 615.0e6f;
const float         OutputMinRate            = 5.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 1000;
const          int  OutputMin_mV             = -1000;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 1;

const unsigned int Fmc_250_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
const unsigned int Fmc250_Dac_One = 0x10000;         // representation of floating point 1.0

//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
//const int romFmc_250_SysInfoOffset        = 0;
//const int romFmc_250_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  VGA BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_250_InputCoeffOffset       = 0;
const int romFmc_250_InputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_250_OutputCoeffOffset       = 0;   // these are in separate "ROMs"   //  romFmc_250_InputCoeffOffset_avail;
const int romFmc_250_OutputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//===========================================================================
// Class Fmc250_AnalogInInfo
//===========================================================================

class Fmc250_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    Fmc250_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~Fmc250_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
	virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
// Class Fmc250_AnalogOutInfo
//===========================================================================

class Fmc250_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    Fmc250_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~Fmc250_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};


//===========================================================================
//  CLASS Fmc250InputDevice  -- Fmc250 Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Fmc250InputDevice
//---------------------------------------------------------------------------

Fmc250InputDevice::Fmc250InputDevice(Fmc250 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), Owner250(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_250_InputCoeffOffset, MaxInChannels,
                       Fmc_250_Adc_One, AdcCal() ),
      AdcDeviceTestEnabled(false), CurrentAdcDeviceTestPattern(0x00)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc250InputDevice
//---------------------------------------------------------------------------

Fmc250InputDevice::~Fmc250InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc250InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------
//
//unsigned int  Fmc250InputDevice::Channels()
//{
//    return MaxInChannels;
//}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void Fmc250InputDevice::AdcDeviceTestMode(bool enabled, unsigned int test_mode, std::vector<unsigned int> test_pattern)
{
    AdcDeviceTestEnabled        = enabled;
    CurrentAdcDeviceTestPattern = test_pattern;
    CurrentAdcDeviceTestMode    = test_mode;
}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc250InputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void Fmc250InputDevice::Adc_OnConfigure(double ClockRateMHz)
{
    CalRomSection.ApplyRomCoefficients();

    MemoryBitManipulator e14(0);
    e14.Value( Owner->PeekLogic(0x214));
    if (e14.Bit(21))
        Owner->Debug.Log("ADC Locked on Clock");
    else
        Owner->Debug.Log("ADC NOT LOCKED on Clock");

    Owner->PokeLogic(0x214, 0x3);   // reset ADCs
    Owner->PokeLogic(0x214, 0x0);   // remove reset

    //  Configure Devices
    for (unsigned int dev=0; dev<MaxInDevices; dev++)
        {
        if (DevicePowered(dev) == false)
            continue;

        Spi(dev, 0x0, 0x81);   // return device to 4 wire mode

        // Read ADC Calibration Bit
        const int MAX_ADC_COUNTER = 20;
        int i = 0;
        for (; i < MAX_ADC_COUNTER; i++)
        {
           Sleep(100);
           unsigned int cal_reg = Spi(dev, 0xb6);
           if (cal_reg&0x1)
              {
               Owner->Debug.Log("ADC Cal Status ok.");
               break;
              }
        }
        if (i == MAX_ADC_COUNTER)
            Owner->Debug.Log("ADC Cal Status Failed.");

        //    cr ." gain/offset cal values load here"
        //    cr

        Spi(dev, 0x72, 0x01);
        MemoryBitManipulator spi74(0);
        spi74.Value( Spi(dev, 0x74) );
        if (ClockRateMHz<100.0)
            spi74.Bit(6, true); //slow
        else
            spi74.Bit(6, false); // fast
        Spi(dev, 0x74, spi74.Value());

        if (AdcDeviceTestEnabled)
            {
            for (size_t i = 0; i < CurrentAdcDeviceTestPattern.size(); i++)
                {
                MemoryBitManipulator usrPatternLsb(0);
                usrPatternLsb.Field(0, 8, CurrentAdcDeviceTestPattern[i] & 0xFF);
                Spi(dev, static_cast<unsigned int>(0xc1 + 2 * i), usrPatternLsb.Value());   // pattern LSB

                MemoryBitManipulator usrPatternMsb(0);
                usrPatternMsb.Field(0, 8, (CurrentAdcDeviceTestPattern[i]>>8) & 0xFF);
                Spi(dev, static_cast<unsigned int>(0xc2 + 2 * i), usrPatternMsb.Value());   // pattern MSB
                }

            MemoryBitManipulator testIo(0);
            testIo.Field(4, 4, CurrentAdcDeviceTestMode);
            testIo.Field(0, 3, 0x3); // Cycle Pattern 1,3,5,7
            Spi(dev, 0xc0, testIo.Value());
            }
        }

    //  Check PHY calibration status bit 17,16"
    e14.Value( Owner->PeekLogic(0x214));
    Owner->Debug.Log("PHY Calibration Status =", e14.Field(16, 2));

    // Re-load rom to re-do calibration coefficients
    CalRomSection.ApplyRomCoefficients();

}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::AdcPowerGood() -- test if power is good
//---------------------------------------------------------------------------

bool  Fmc250InputDevice::AdcPowerGood(unsigned int idx)
{
    MemoryBitManipulator e14(0);
    e14.Value(Owner->PeekLogic(0x214));

    if (idx==0)
        return e14.Bit(8);
    else
        return e14.Bit(9);

}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  Fmc250InputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    AdcSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  Fmc250InputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  Fmc250InputDevice::Spi(unsigned int device, unsigned int address)
{
    return AdcSpi().Spi(device, address);
}

//===========================================================================
//  CLASS Fmc250_DacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class Fmc250_DacToHardware
//---------------------------------------------------------------------------

class Fmc250_DacToHardware :  public AdDac9122::IDacToHardware
{
public:
    virtual unsigned short  Spi(int addr)
        {  return Out->Spi(Dev, addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Out->Spi(Dev, addr, value);  }

    virtual void  DacReset(bool state)
        { Out->DacReset(state); }

    Fmc250_DacToHardware( Fmc250OutputDevice * out, unsigned int dev=0 )
        :  Out(out), Dev(dev)
        {}
    virtual ~Fmc250_DacToHardware()
        {}

    Fmc250OutputDevice *   Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS Fmc250OutputDevice  -- Fmc250 Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc250OutputDevice
//---------------------------------------------------------------------------

//const bool DevIs5681 = true;

//---------------------------------------------------------------------------
//  constructor for class Fmc250OutputDevice
//---------------------------------------------------------------------------

Fmc250OutputDevice::Fmc250OutputDevice(Fmc250 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), Owner250(owner), FInfo(info),
      DacDev(new Fmc250_DacToHardware(this, 0)),
      CalRomSection(DacCalIdrom(), romFmc_250_OutputCoeffOffset, MaxOutChannels,
                       Fmc250_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc250OutputDevice
//---------------------------------------------------------------------------

Fmc250OutputDevice::~Fmc250OutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc250OutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc250OutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc250OutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
    DacDev.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc250OutputDevice::ConfigureHardware()
{
    DacDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  Fmc250OutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    //Device().Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1) );
    Device().ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device().ChannelEnabled(1, DacCommon().ChannelEnabled(1) );

}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  Fmc250OutputDevice::SetDacInterpolation(DacInterpolationModeValues values)
{
    SetDacInterpolation(static_cast<unsigned int>(values));
}

//---------------------------------------------------------------------------
//  Fmc250OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  Fmc250OutputDevice::SetDacInterpolation(unsigned int values)
{
    Owner250->FDacInterpolationMode = values&0x7;
}

//=============================================================================
//  CLASS Fmc250_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc250_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class Fmc250_Pll
//---------------------------------------------------------------------------

Fmc250_Pll::Fmc250_Pll(Fmc250 * owner, IAddressingSpace &space, Lmk04800Registers & reg)
    : Lmk04800(space, reg), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 0 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);

}

//---------------------------------------------------------------------------
//  destructor for class Fmc250_Pll
//---------------------------------------------------------------------------

Fmc250_Pll::~Fmc250_Pll()
{
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  Fmc250_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "Fmc250_Pll::OnInit()");

    //  Device Powered/Enabled Setup
//    SetPllEnable(true);

    std::string SN = Owner->RomInfo().SerialNumber();
    Debug.Log(std::string("SN:")+SN);
    SN += '$';  // add a junk character to assure that we have 1 character.
    switch (SN[0])
        {
        case 'a':
        case 'A':
            UseRevAPll = true;
            Debug.Log("PLL Output Config: Rev A");
            break;
        default:
        case 'B':
        case 'b':
        case 'C':
        case 'c':
        case 'D':
        case 'd':
        case 'E':
        case 'e':
            UseRevAPll = false;
            Debug.Log("PLL Output Config: Rev B and later");
            break;
        }
//    Debug.Log("Force To Rev B");
//    UseRevAPll = false;

    if (UseRevAPll)
        RevA_PllOutputInit();
    else
        RevB_PllOutputInit();

    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[0] = 1000.0e6;
//    OutputFrequencyLimit[1] = 1000.0e6;
//    OutputFrequencyLimit[2] = 1000.0e6;
//    OutputFrequencyLimit[3] = 1000.0e6;
//    OutputFrequencyLimit[4] = 1000.0e6;
    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference

}

//---------------------------------------------------------------------------
//  Fmc250_Pll::RevA_PllOutputInit()
//---------------------------------------------------------------------------

void  Fmc250_Pll::RevA_PllOutputInit()
{
    ScopedTraceRef ccc(Debug, "Fmc250_Pll::RevA_PllOutputInit()");
    //
    //  Set Output Configuration
    OutputInUse(0, true);    SetOutputConfig(0,  ocLvPecl_1600);     //  ADC0
    OutputInUse(1, false);   SetOutputConfig(1,  ocDisabled);        //  ADC0 FPGA (unused)
    OutputInUse(2, true);    SetOutputConfig(2,  ocLvPecl_1600);     //  ADC1
    OutputInUse(3, false);   SetOutputConfig(3,  ocDisabled);        //  ADC1 FPGA (unused)
    OutputInUse(4, true);    SetOutputConfig(4,  ocLvds);            //  DAC
    OutputInUse(5, false);   SetOutputConfig(5,  ocDisabled);        //  (unused)
    OutputInUse(6, true);    SetOutputConfig(6,  ocLvds);            //  FMC Global CLK
    OutputInUse(7, false);   SetOutputConfig(7,  ocDisabled);        //  Global CLK 1 (unused)
    OutputInUse(8, true);    SetOutputConfig(8,  ocLvPecl_1600);     //  Test Output
    OutputInUse(9, true);    SetOutputConfig(9,  ocLvds);            //  DAC REF
    OutputInUse(10, false);  SetOutputConfig(10, ocDisabled);        //  (unused)
    OutputInUse(11, true);   SetOutputConfig(11, ocLvds);            //  DAC FPGA

    // Apply Flags to disable types of clocks
    if (Owner->DacClocksDisabled())
        {
        OutputInUse(4, false);    SetOutputConfig(4,  ocDisabled);     //  DAC
        OutputInUse(9, false);    SetOutputConfig(9,  ocDisabled);     //  DAC REF
        OutputInUse(11, false);   SetOutputConfig(11, ocDisabled);     //  DAC FPGA
        }

    if (Owner->TestClocksDisabled())
        {
        OutputInUse(8, false);    SetOutputConfig(8,  ocDisabled);     //  Test Output
        }
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::RevB_PllOutputInit()
//---------------------------------------------------------------------------

void  Fmc250_Pll::RevB_PllOutputInit()
{
    ScopedTraceRef ccc(Debug, "Fmc250_Pll::RevB_PllOutputInit()");
    //
    //  Set Output Configuration
    OutputInUse(0, true);    SetOutputConfig(0,  ocLvPecl_1600);     //  ADC0
    OutputInUse(1, true);    SetOutputConfig(1,  ocLvPecl_1600);     //  ADC1
    OutputInUse(2, true);    SetOutputConfig(2,  ocLvds);            //  DAC REF
    OutputInUse(3, false);   SetOutputConfig(3,  ocDisabled);        //
    OutputInUse(4, true);    SetOutputConfig(4,  ocLvds);            //  DAC
    OutputInUse(5, false);   SetOutputConfig(5,  ocDisabled);        //  (unused)
    OutputInUse(6, true);    SetOutputConfig(6,  ocLvds);            //  FMC Global CLK
    OutputInUse(7, false);   SetOutputConfig(7,  ocDisabled);        //
    OutputInUse(8, true);    SetOutputConfig(8,  ocLvPecl_1600);     //  Test Output
    OutputInUse(9, false);   SetOutputConfig(9,  ocDisabled);        //
    OutputInUse(10, true);   SetOutputConfig(10, ocLvds);            //  FMC CLK 1
    OutputInUse(11, true);   SetOutputConfig(11, ocLvds);            //  DAC FPGA

    // Apply Flags to disable types of clocks
    if (Owner->DacClocksDisabled())
        {
        OutputInUse(2, false);    SetOutputConfig(9,  ocDisabled);     //  DAC REF
        OutputInUse(4, false);    SetOutputConfig(4,  ocDisabled);     //  DAC
        OutputInUse(10, false);   SetOutputConfig(10, ocDisabled);     //  FMC CLK 1
        OutputInUse(11, false);   SetOutputConfig(11, ocDisabled);     //  DAC FPGA
        }

    if (Owner->TestClocksDisabled())
        {
        OutputInUse(8, false);    SetOutputConfig(8,  ocDisabled);     //  Test Output
        }
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::OnReapply()
//---------------------------------------------------------------------------

void  Fmc250_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  Fmc250_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  Fmc250_Pll::DacMasterOutput()
{
    return 4;
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  Fmc250_Pll::SetAdcOutputRates(double adc_rate)
{
    if (UseRevAPll)
        {
        OutputFrequency(0, adc_rate); // only need to do one per pair
        OutputFrequency(2, adc_rate);
        OutputFrequency(6, adc_rate);
        OutputFrequency(8, adc_rate);
        }
    else
        {
        OutputFrequency(0, adc_rate); // only need to do one per pair
        OutputFrequency(1, adc_rate);
        OutputFrequency(6, adc_rate);
        OutputFrequency(8, adc_rate);
        }


}

//---------------------------------------------------------------------------
//  Fmc250_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  Fmc250_Pll::SetDacOutputRates(double dac_rate)
{
    if (UseRevAPll)
        {
        OutputFrequency(4,  dac_rate);
        OutputFrequency(9,  DCLK(dac_rate, DacMode())/8);
        OutputFrequency(11, DCLK(dac_rate, DacMode()));
        }
    else
        {
        OutputFrequency(2,  DCLK(dac_rate, DacMode())/8);
        OutputFrequency(4,  dac_rate);
        OutputFrequency(10, dac_rate);
        OutputFrequency(11, DCLK(dac_rate, DacMode()));
        }

}

//---------------------------------------------------------------------------
//  Fmc250_Pll::DCLK() -- Set DCLK rate by DAC rate
//---------------------------------------------------------------------------

double   Fmc250_Pll::DCLK(double dac_rate, unsigned int mode)
{
    switch (mode)
        {
        case 0:  return dac_rate;      // max rate 615;
        case 1:  return dac_rate/2;    // max rate 1000;
        case 2:  return dac_rate/2;    // max rate 1230;
        case 3:  return dac_rate/4;    // max rate 1230;
        case 6:  return dac_rate/4;    // max rate 1230;
        default:
        case 7:  return dac_rate/8;    // max rate 1230;
        }
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::DacMode() -- Get current Dac Mode
//---------------------------------------------------------------------------

unsigned int   Fmc250_Pll::DacMode()
{
    return Owner->CurrentDacInterpolationMode();
}

//=============================================================================
//  CLASS Fmc250_AdcCalOverride  --  Cal Override for Fmc250 Adcs
//=============================================================================
/*
class Fmc250_AdcCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    Fmc250_AdcCalOverride(Fmc250 *  owner) : Owner(owner)
        {}
    virtual ~Fmc250_AdcCalOverride()
        {}

private:
    Fmc250 *  Owner;

    void  CalcOffset(int value, unsigned short curCoarseAdj,
                        unsigned short curFineAdj,
                        unsigned short &CoarseAdj, unsigned short & FineAdj);
    void  CalcGain(unsigned int value, unsigned short curCoarseAdj,
                        unsigned short curMedAdj, unsigned short curFineAdj,
                        unsigned short &CoarseAdj, unsigned short & MedAdj,
                        unsigned short & FineAdj);

};
*/
/*
//---------------------------------------------------------------------------
//  Fmc250_AdcCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the Adc

unsigned int  Fmc250_AdcCalOverride::OverrideOffset(unsigned int value)
{
    Owner->Debug.Log("OverrideOfs:", value, 16);
    return value;   // we need to delay this calculation for later
}

//---------------------------------------------------------------------------
//  Fmc250_AdcCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  Fmc250_AdcCalOverride::OverrideGain(unsigned int value)
{
    return value;
}

//---------------------------------------------------------------------------
//  Fmc250_AdcCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  Fmc250_AdcCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    // read back current adjusts
    unsigned short curCoarseAdj = Owner->Input().Spi(ch, 0x20);
    unsigned short curFineAdj   = Owner->Input().Spi(ch, 0x21);
    unsigned short CoarseAdj    = 0;
    unsigned short FineAdj      = 0;

    CalcOffset((int)value, curCoarseAdj, curFineAdj, CoarseAdj, FineAdj);
    //  Update
    Owner->Input().Spi(ch, 0xFE, 0);  // enable writing of adjusts
    Owner->Input().Spi(ch, 0x20, CoarseAdj);
    Owner->Input().Spi(ch, 0x21, FineAdj);
    Owner->Input().Spi(ch, 0x26, CoarseAdj);
    Owner->Input().Spi(ch, 0x27, FineAdj);
    Owner->Input().Spi(ch, 0xFE, 1);  // activate
    Owner->Input().Spi(ch, 0xFE, 0);  // ...and done
}

//---------------------------------------------------------------------------
//  Fmc250_AdcCalOverride::CalcOffset() --
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the Adc

void  Fmc250_AdcCalOverride::CalcOffset(int value, unsigned short curCoarseAdj,
                                        unsigned short curFineAdj,
                                        unsigned short &CoarseAdj, unsigned short & FineAdj)
{
    const float HdwStep_Lsb       = 8.25f * 2.0;     //
    const float HdwStepFine_Lsb   = .25f;      //
    const float CoarseRange       = 1050*2;
    const float FineRange         = 32;

    //  convert the current value to an offset
    float CalOffset = HdwStep_Lsb*(curCoarseAdj-0x80) + HdwStepFine_Lsb*(curFineAdj-0x80);
    float Offset = CalOffset + value;
    Owner->Debug.Log("Offset 0", Offset);
    //  trim
    if (Offset>=CoarseRange)
        {
        CoarseAdj = 0xFF; FineAdj = 0x80;
        }
    else if (Offset<=-CoarseRange)
        {
        CoarseAdj = 0x00; FineAdj = 0x80;
        }
    else
        {
        CoarseAdj = static_cast<unsigned short>(Offset/HdwStep_Lsb);
        short CA = CoarseAdj;
        Owner->Debug.Log("CA", CA);
        Offset -= CA*HdwStep_Lsb;  // offset is now fine adjust...
        Owner->Debug.Log("Offset 1", Offset);
        CoarseAdj += 0x80;                // adjust to single ended rep
        CoarseAdj &= 0xFF;

        FineAdj = static_cast<unsigned short>(Offset/HdwStepFine_Lsb);
        if (Offset>=FineRange)
            FineAdj = 0xFF;
        if (Offset<=-FineRange)
            FineAdj = 0x00;
        else
            {
            FineAdj += 0x80;   // adjust to single ended
            FineAdj &= 0xFF;
            }
        }
    Owner->Debug.Log("CalcOffset()");
    Owner->Debug.Log("Value", value, 16);
    Owner->Debug.Log("CalOffset", CalOffset);
    Owner->Debug.Log("curCoarseAdj", curCoarseAdj, 16);
    Owner->Debug.Log("curFineAdj", curFineAdj, 16);
    Owner->Debug.Log("CoarseAdj", CoarseAdj, 16);
    Owner->Debug.Log("FineAdj", FineAdj, 16);
}

//---------------------------------------------------------------------------
//  Fmc250_AdcCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  Fmc250_AdcCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    // read back current adjusts
    unsigned short curCoarseAdj = Owner->Input().Spi(ch, 0x22);
    unsigned short curMedAdj    = Owner->Input().Spi(ch, 0x23);
    unsigned short curFineAdj   = Owner->Input().Spi(ch, 0x24);
    unsigned short CoarseAdj    = 0;
    unsigned short MedAdj       = 0;
    unsigned short FineAdj      = 0;

    CalcGain(value, curCoarseAdj, curMedAdj, curFineAdj, CoarseAdj, MedAdj, FineAdj);
    //Owner->Debug.Log("SKIP WRITING COARSE GAIN");
    //return;
    //  Update
    Owner->Input().Spi(ch, 0xFE, 0);  // enable writing of adjusts
    Owner->Input().Spi(ch, 0x22, CoarseAdj);
    Owner->Input().Spi(ch, 0x23, MedAdj);
    Owner->Input().Spi(ch, 0x24, FineAdj);
    Owner->Input().Spi(ch, 0x28, CoarseAdj);
    Owner->Input().Spi(ch, 0x29, MedAdj);
    Owner->Input().Spi(ch, 0x2A, FineAdj);
    Owner->Input().Spi(ch, 0xFE, 1);  // activate
    Owner->Input().Spi(ch, 0xFE, 0);  // ...and done

}

//---------------------------------------------------------------------------
//  Fmc250_AdcCalOverride::CalcGain() --
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the Adc

void  Fmc250_AdcCalOverride::CalcGain(unsigned int value, unsigned short curCoarseAdj,
                                      unsigned short curMedAdj, unsigned short curFineAdj,
                                      unsigned short &CoarseAdj, unsigned short & MedAdj,
                                      unsigned short & FineAdj)
{
    const float HdwStepMed_Pct    = .0086f;     // in percent gains
    const float HdwStepFine_Pct   = .001f;      // in percent gains

    //  convert the current value to a total gain
    float CalGain = 0;
    switch (curCoarseAdj)
        {
        case 0xC:  CalGain = 4.2f;   break;
        case 0xD:
        case 0x8:  CalGain = 2.8f;   break;
        case 0xE:
        case 0x9:
        case 0x4:  CalGain = 1.4f;   break;
        default:
        case 0xF:
        case 0xA:
        case 0x5:
        case 0x0:  CalGain = 0;      break;
        case 0xB:
        case 0x6:
        case 0x1:  CalGain = -1.4f;  break;
        case 0x7:
        case 0x2:  CalGain = -2.8f;  break;
        case 0x3:  CalGain = -4.2f;  break;
        }

    CalGain += HdwStepMed_Pct*(curMedAdj-0x80) + HdwStepFine_Pct*(curFineAdj-0x80);

    float GainPct = ((static_cast<float>(value)/Fmc_250_Adc_One)-1.0f)*100.0f;
    Owner->Debug.Log("GainPct", GainPct);
    float Gain = CalGain + GainPct;
    Owner->Debug.Log("Gain 0", Gain);
    //
    //  Set Coarse Adjust and Trim...

    if (Gain>=4.2f)
        {
        CoarseAdj = 0xC; MedAdj = 0x80; FineAdj = 0x80;
        }
    else if (Gain<=-4.2f)
        {
        CoarseAdj = 0x3; MedAdj = 0x80; FineAdj = 0x80;
        }
    else
        {
        if (Gain >= 2.8f)       {  CoarseAdj=0xC; Gain -= 4.2f;   }
        else if (Gain >=  2.8f) {  CoarseAdj=0x8; Gain -= 2.8f;   }
        else if (Gain >=  1.4f) {  CoarseAdj=0x4; Gain -= 1.4f;   }
        else if (Gain >=  0.0f) {  CoarseAdj=0x0;                }
        else if (Gain >= -1.4f) {  CoarseAdj=0x1; Gain -= -1.4f;  }
        else if (Gain >= -2.8f) {  CoarseAdj=0x2; Gain -= -2.8f;  }
        else                    {  CoarseAdj=0x3; Gain -= -4.2f;  }


        MedAdj = static_cast<unsigned short>(Gain/HdwStepMed_Pct);
        if (Gain>=1.10)
            {  MedAdj = 0xFF;  FineAdj=0x80;  }
        if (Gain<=-1.10)
            {  MedAdj = 0x00; FineAdj=0x80;  }
        else
            {
            short MA = MedAdj;
            Owner->Debug.Log("MA", MA);
            Gain -= MA*HdwStepMed_Pct;   // offset is now fine adjust...
            Owner->Debug.Log("Gain 1", Gain);
            MedAdj += 0x80;                  // adjust to single ended rep
            MedAdj &= 0xFF;

            FineAdj = static_cast<unsigned short>(Gain/HdwStepFine_Pct);
            if (Gain>=0.15)
                FineAdj = 0xFF;
            if (Gain<=-0.15)
                FineAdj = 0x00;
            else
                {
                FineAdj += 0x80;   // adjust to single ended
                FineAdj &= 0xFF;
                }
            }
        }

    CoarseAdj = 0x0;  MedAdj=0x80;  FineAdj=0x80;
    Owner->Debug.Log("CalcGain()");
    Owner->Debug.Log("Value", value, 16);
    Owner->Debug.Log("CalGain", CalGain);
    Owner->Debug.Log("curCoarseAdj", curCoarseAdj, 16);
    Owner->Debug.Log("curMedAdj", curMedAdj, 16);
    Owner->Debug.Log("curFineAdj", curFineAdj, 16);
    Owner->Debug.Log("CoarseAdj", CoarseAdj, 16);
    Owner->Debug.Log("MedAdj",  MedAdj, 16);
    Owner->Debug.Log("FineAdj", FineAdj, 16);
}
*/

//=============================================================================
//  CLASS Fmc250_DacCalOverride  --  Cal Override for Fmc250 Dacs
//=============================================================================

class Fmc250_DacCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    Fmc250_DacCalOverride(Fmc250 *  owner) : Owner(owner)
        {}
    virtual ~Fmc250_DacCalOverride()
        {}

private:
    Fmc250 *  Owner;
};

//---------------------------------------------------------------------------
//  Fmc250_DacCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  Fmc250_DacCalOverride::OverrideOffset(unsigned int value)
{
    // trim to 16 bit value
    return value&0xFFFF;
}

//---------------------------------------------------------------------------
//  Fmc250_DacCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  Fmc250_DacCalOverride::OverrideGain(unsigned int value)
{
    const float HdwStep_mA    = 0.0225f;    // 22.5 uA steps
    const float Min_mA        = 8.64f;      // 0x000 gain value
    const float Center_mA     = 20.00f;     // 0x1f9 gain value


    float gain = static_cast<float>(value)/Fmc250_Dac_One;

    float adjusted_mA = gain*Center_mA;
    float steps_mA = adjusted_mA - Min_mA;

    float counts = steps_mA / HdwStep_mA;
    unsigned int setting;
    if (counts < 0)
        setting = 0;
    else if (counts > 1023)
        setting = 0x3FF;
    else
        setting = static_cast<unsigned int>(counts);

    return setting;
}

//---------------------------------------------------------------------------
//  Fmc250_DacCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  Fmc250_DacCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Offset(value);
    else
        Owner->Output().Device().Write_Q_Offset(value);

}

//---------------------------------------------------------------------------
//  Fmc250_DacCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  Fmc250_DacCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Gain(value);
    else
        Owner->Output().Device().Write_Q_Gain(value);

}

//===========================================================================
//  CLASS Fmc250  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc250
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{

FmcLMK_Clock::MapRegisters FmcLMKClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    mmFmc_ClockControl,  //  AfeExtClkSourceSelect_Addr
    };

static Lmk04800Registers Fmc250_PllRegisters =
    { mmFmc_PllControl,  mmFmc_PllSpi  };

}

//---------------------------------------------------------------------------
//  constructor for class Fmc250
//---------------------------------------------------------------------------

Fmc250::Fmc250()
    :  FmcBidirBoard(fmcPrimary),
       AFEClock_SDev(this, WB_FMC_1, FmcLMKClockRegAddrs),
       Local_AdcSpi_SDev(this, WB_FMC_2),
       AdcCalIDROM_SDev(this, RomInfo(), true),   // true = ADC Rom
       DacCalIDROM_SDev(this, RomInfo(), false),  // true = ADC Rom
       AFEAdcVcoRegister(WB_FMC_1, mmFmc_VcxoControl),
       FInput(new Fmc250InputDevice(this, new Fmc250_AnalogInInfo)),
       FOutput(new Fmc250OutputDevice(this, new Fmc250_AnalogOutInfo)),
       FDacClocksDisabled(false), FTestClocksDisabled(true),
       FDacInterpolationMode(0x0)
{
    AFEAdcSpi_SDev.DeviceEnabled(false);
    LocalConstructCustomParts();
}

Fmc250 * an250()
{
    Fmc250 * wombat = new Fmc250;
    return wombat;
}

//---------------------------------------------------------------------------
//  Fmc250::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  Fmc250::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  Header Section
    RI.Header().Version(0x2);

    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("Fmc250");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(250);
    RI.FmcDef().ModuleSize(Fmc::msSingleWide);
    RI.FmcDef().ClockDirection(Fmc::dirCarrierToMezz);
    RI.FmcDef().ConnectorSize(Fmc::fmcP1, Fmc::csHPC);
    RI.FmcDef().ConnectorSize(Fmc::fmcP2, Fmc::csNotFitted);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankA, 87);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankB, 30);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 0);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankB, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP1, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP2, 0);

    //  ADC CALIBRATION SECTION
    RI.AdcCal().Resize(romFmc_250_InputCoeffOffset_avail);
    RI.AdcCal().Zero();

    //  DAC CALIBRATION SECTION
    RI.DacCal().Resize(romFmc_250_OutputCoeffOffset_avail);
    RI.DacCal().Zero();

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(2.5f);
    DcOut->RipplePkPk(0.5f);
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(false);
    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    //        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
    //        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
    //        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(2.5f);
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  Fmc250::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc250::LocalConstructCustomParts()
{
    FmcBidirBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    Local_AdcSpi_SDev.ConstructCustomParts();
    AdcCalIDROM_SDev.ConstructCustomParts();
    DacCalIDROM_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  Fmc250::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc250::OpenHardware()
{
    PokeLogic(0x0,   0x00000);  // offsets relative to FMC

    FmcBidirBoard::OpenHardware();

    Debug.Log("++++++++++++++++++++++++++++++++++++");
    Debug.Log("  Load Adc/Dac Calibration Shadows");
    AdcCalIDROM_SDev.OpenHardware();
    DacCalIDROM_SDev.OpenHardware();
    Debug.Log("++++++++++++++++++++++++++++++++++++");

    //  Turn on VCO Power
    AFEAdcVcoRegister.Bit(4, true);

    PokeLogic(0x202, 0x3);
    unsigned int data = PeekLogic(0x0);
    Debug.Log("Power Register Final: ", data, 16);

    AFEClock_SDev.OpenHardware();
    Local_AdcSpi_SDev.OpenHardware();

    Input().OpenHardware();
    Output().OpenHardware();

}

//---------------------------------------------------------------------------
//  Fmc250::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  Fmc250::PreconfigureHardware()
{
    Output().ConfigureChannelEnables();

    FmcBidirBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();
    Local_AdcSpi_SDev.PreconfigureHardware();
    //
    //  Special Config for D/A Devices
    Output().PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc250::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc250::ConfigureHardware()
{
    FmcBidirBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    Local_AdcSpi_SDev.ConfigureHardware();
    //
    //  Special Config for A/D Devices
    double ClockRateMHz = AFEClock_SDev.Clock().Adc().FrequencyActual()/1.0e6;

    Input().Adc_OnConfigure(ClockRateMHz);
    //
    //  Adc Calibration
    //AFEAdcCalCtrl_SDev.StartCalibration();
    //Sleep(1);
    //unsigned int results = AFEAdcCalCtrl_SDev.CalibrationResults();

    //Debug.Log("Fmc250 Adc Cal Results: ", results);
    //
    //  Special Config for D/A Devices
    Output().ConfigureHardware();

}

//---------------------------------------------------------------------------
//  Fmc250::StreamStopHardware()
//---------------------------------------------------------------------------

void  Fmc250::StreamStopHardware()
{
    FmcBidirBoard::StreamStopHardware();

    //AFEClock_SDev.StreamStopHardware();
    Local_AdcSpi_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  Fmc250::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc250::GetVitaInStreamCount()
{
    return Fmc_250_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc250::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc250::GetVitaOutStreamCount()
{
    return Fmc_250_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc250::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc250::GetAdcSpiDeviceCount()
{
    return Fmc_250_AdcSpiDeviceCount; // 0?  base ADC Spi device not used
}

//---------------------------------------------------------------------------
//  Fmc250::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc250::GetDacSpiDeviceCount()
{
    return Fmc_250_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc250::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  Fmc250::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new FmcStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  Fmc250::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  Fmc250::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new FmcStandardPriClockDevice(this, 2);
}


//---------------------------------------------------------------------------
//  Fmc250::MakeFmcAfePllObject() -- Get Pri Device object
//---------------------------------------------------------------------------

Lmk04800 *  Fmc250::MakeFmcAfePllObject()
{
    // return same object for both directions
    return new Fmc250_Pll(this, WB_FMC_1, Fmc250_PllRegisters );
}

//---------------------------------------------------------------------------
//  Fmc250::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc250::GetAdcCalibrationOverride()
{
    return 0;  // new Fmc250_AdcCalOverride(this);
}

//---------------------------------------------------------------------------
//  Fmc250::MakeAfeAdcSpiDevices() -- Build ADC Spi Devices
//---------------------------------------------------------------------------

void  Fmc250::MakeAfeAdcSpiDevices()
{
    //  Make one SPI device for each ADC device
    unsigned int Write_Addr = mmFmc_AdcSpiCtrlBase;
    unsigned int Read_Addr  = mmFmc_AdcSpiStatBase;

    for (unsigned int i=0; i<Fmc_250_AdcSpiDeviceCount; ++i)
        {
        Local_AdcSpi_SDev.MakeSpiDevice("ADC", Write_Addr, Read_Addr);
        Write_Addr += mmFmc_AdcSpiCtrlDelta;
        Read_Addr  += mmFmc_AdcSpiStatDelta;
        }
}


//---------------------------------------------------------------------------
//  Fmc250::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc250::GetCalibrationAdcCount()
{
    return Fmc_250_AdcChannels;
}

//---------------------------------------------------------------------------
//  Fmc250::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc250::GetDacCalibrationOverride()
{
    return new Fmc250_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  Fmc250::GetCalibrationDacCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc250::GetCalibrationDacCount()
{
    return Fmc_250_DacChannels;
}

//---------------------------------------------------------------------------
//  Fmc250::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  Fmc250::GetAnalogDeviceInfo(FrameDeviceCode which,
                                  unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = MaxInDevices;
        ch_per_dev = Fmc_250_AdcChannelsPerDevice;
        }
    else //(which==devDac)
        {
        devices = MaxOutDevices;
        ch_per_dev = Fmc_250_DacChannelsPerDevice;
        }

}

//---------------------------------------------------------------------------
//  Fmc250::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  Fmc250::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = MaxInDevices;
        ch_per_dev = Fmc_250_AdcChannelsPerDevice;
        }
    else //(which==devDac)
        {
        devices = MaxOutDevices;
        ch_per_dev = Fmc_250_DacChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  Fmc250::GetFmcCalIdromInfo() -- Make ROM Image Objhect
//---------------------------------------------------------------------------

Fmc250::FmcCalIdromInfo  Fmc250::GetFmcCalIdromInfo(FmcCalDeviceCode which)
{
    const unsigned int ReservedSize = 1;

    FmcCalIdromInfo  info;
    if (which==fmcCalAdc)
        {
        info.IdromSize = romFmc_250_InputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }
    else
        {
        info.IdromSize = romFmc_250_OutputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }

    return info;
}

//---------------------------------------------------------------------------
//  Fmc250::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void Fmc250::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = 0x3;  // 1.8V min
    maxFmcVadjCode = 0x4;  // 2.5V max
}

//---------------------------------------------------------------------------
//  Fmc250::Input() -- Reference to input object
//---------------------------------------------------------------------------

Fmc250InputDevice & Fmc250::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  Fmc250::Output() -- Reference to output object
//---------------------------------------------------------------------------

Fmc250OutputDevice & Fmc250::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  Fmc250::AfterFiclStart() --
//---------------------------------------------------------------------------

void  Fmc250::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant AdcSpi0  101 constant AdcSpi1  102 constant Pll");
    engine.Evaluate("  103 constant TempI2C  104 constant kIdromI2C 105 constant PllImage");
    engine.Evaluate("  106 constant DacSpi0  ");
//    engine.Evaluate("  108 constant Vga0I2C  109 constant Vga1I2C  110 constant Vga2I2C   ");
//    engine.Evaluate("  111 constant Vga3I2C  112 constant Vga4I2C  113 constant VgaIdromI2C   ");

    engine.Evaluate(": ?Fmc250 "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   AdcSpi0  AdcSpi1  Pll  \" "
                    "    cr .\"   TempI2C  IdromI2C  PllImage \" "
                    "    cr .\"   DacSpi0 \" "
                    "    cr  "
                    "    cr .\"   ?Fmc250  : display this message \" "
                    "    cr  ; ");

    engine.Display("Fmc250-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  Fmc250::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  Fmc250::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;
    const int kPll = 102;
    const int kTempI2C = 103;   const int kIdromI2C = 104;
    const int kPllImage = 105;
    const int kDacSpi0 = 106;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:    case kAdcSpi1:
            return Local_AdcSpi_SDev.Spi(dev-kAdcSpi0, addr);
        case kDacSpi0:
            return AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr);
        case kPll:
            return AFEClock_SDev.Pll().Port(addr);
        case kPllImage:
            return AFEClock_SDev.Pll().PortImage(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  Fmc250::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  Fmc250::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;
    const int kPll = 102;
    const int kTempI2C = 103;  const int kIdromI2C = 104;
    const int kPllImage = 105;
    const int kDacSpi0 = 106;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:    case kAdcSpi1:
            Local_AdcSpi_SDev.Spi(dev-kAdcSpi0, addr, value);
            break;
        case kDacSpi0:
            AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr, value);
            break;
        case kPll:
            AFEClock_SDev.Pll().Port(addr, value);
            break;
        case kPllImage:
            AFEClock_SDev.Pll().PortImage(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  Fmc250::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  Fmc250::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 102;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  Fmc250::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  Fmc250::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 102;

    switch (dev)
        {
        default:
        case kPll:
            return static_cast<float>(AFEClock_SDev.Pll().FrequencyActual());
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
