// Fmc_250_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_250_MbH
#define Fmc_250_MbH

#include <FmcBoard_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>
#include <AdDac9122_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Fmc250;

//===========================================================================
//  CLASS ISuppliesAdded_250_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_250_Requirements : public IRequires_LmkFmcClocks,
                                        public IRequires_AdcSpi,
                                        public IRequires_FmcCalIDROM
{
public:
    virtual ~ISuppliesAdded_250_Requirements()
        {}

};

//==============================================================================
//  CLASS Fmc250_Pll -- Fmc PLL Chip (Specialization of Lmk04800)
//==============================================================================

class Fmc250_Pll : public Lmk04800
{
public:
    Fmc250_Pll(Fmc250 * owner, IAddressingSpace &space, Lmk04800Registers & reg);
    virtual ~Fmc250_Pll();

    // ...Lmk04800 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 1e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

protected:
    Fmc250 *         Owner;
    bool             UseRevAPll;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

    double   DCLK(double dac_rate, unsigned int mode);
    unsigned int DacMode();

    void  RevA_PllOutputInit();
    void  RevB_PllOutputInit();

};

//===========================================================================
//  CLASS Fmc250  -- Fmc Daughter card Common baseboard class
//===========================================================================

class Fmc250 : public FmcBidirBoard, public ISuppliesAdded_250_Requirements
{
public:
    Fmc250();
    virtual ~Fmc250()
        {}

    // Options
    void  DisableDacClocks(bool state)
        {  FDacClocksDisabled = state;  }
    bool  DacClocksDisabled() const
        {  return FDacClocksDisabled;  }
    void  DisableTestClocks(bool state)
        {  FTestClocksDisabled = state;  }
    bool  TestClocksDisabled() const
        {  return FTestClocksDisabled;  }

    class Fmc250InputDevice &    Input();
    class Fmc250OutputDevice &   Output();

    friend class Fmc250InputDevice;
    friend class Fmc250OutputDevice;

    IFmcClockIo &             Clock()           {  return AFEClock_SDev.Clock();  }
    ILmkRawClockDevices &     RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    virtual double  Get_DacClockRate()  { return Clock().Dac().Frequency(); };

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Suupplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    virtual Lmk04800 *                  MakeFmcAfePllObject();
    virtual void                        MakeAfeAdcSpiDevices();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual FmcCalIdromInfo             GetFmcCalIdromInfo(FmcCalDeviceCode which);

    //
    //  Misc
    unsigned int      CurrentDacInterpolationMode()
        {  return FDacInterpolationMode;  }

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    FmcLMK_Clock       AFEClock_SDev;
    X6AFE_AdcSpi       Local_AdcSpi_SDev;
    FmcCalibrationIDROM_Manager    AdcCalIDROM_SDev;    // Cal Rom Image
    FmcCalibrationIDROM_Manager    DacCalIDROM_SDev;    // Cal Rom Image

    Register           AFEAdcVcoRegister;

    Fmc250InputDevice  *    FInput;
    Fmc250OutputDevice  *   FOutput;

    bool              FDacClocksDisabled;
    bool              FTestClocksDisabled;

    unsigned int      FDacInterpolationMode;


    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

};

//===========================================================================
//  CLASS Fmc250InputDevice  -- FMC 250 Input Devices
//===========================================================================

class Fmc250InputDevice : public FmcInputDevice_Bidir
{
public:
    typedef FmcInputDevice_Bidir inherited;

    Fmc250InputDevice(Fmc250 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc250InputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    void   AdcDeviceTestMode(bool enabled, unsigned int test_mode, std::vector<unsigned int> test_pattern);

     void   OpenHardware();
     void   Adc_OnConfigure(double ClockRateMHz);

     bool   AdcPowerGood(unsigned int idx);

private:
    Fmc250 *                    Owner250;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    bool                      AdcDeviceTestEnabled;
    unsigned int              CurrentAdcDeviceTestMode;
    std::vector<unsigned int> CurrentAdcDeviceTestPattern;


    X6AFE_AdcSpi &          AdcSpi() {  return Owner250->Local_AdcSpi_SDev;  }
    const X6AFE_AdcSpi &    AdcSpi() const {  return Owner250->Local_AdcSpi_SDev;  }

    GeneralIdromImage &  AdcCalIdrom()  {  return Owner250->AdcCalIDROM_SDev.Idrom();  }


};

//===========================================================================
//  CLASS Fmc250OutputDevice  -- FMC 250 Output Devices
//===========================================================================

class Fmc250OutputDevice : public FmcOutputDevice_Bidir
{
public:
    friend class Fmc110_DacToHardware;

    typedef FmcOutputDevice_Bidir inherited;

    Fmc250OutputDevice(Fmc250 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc250OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    //  Special Dac Interpolation Mode
    enum  DacInterpolationModeValues { diByOne=0x0, diByTwo=0x1, diByTwo_b=0x02,
                                       diByFour=0x3, diByFour_b=0x6, diByEight=0x7 };
    void  SetDacInterpolation(DacInterpolationModeValues values);
    void  SetDacInterpolation(unsigned int values);


    AdDac9122::DacDevice &     Device()
        {
            return DacDev;
        }
    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }
    bool  DacPowerGood()
        {
        return DacCommon().DacPowerGood();
        }

    // Pseudo-protected
    void  OpenHardware();
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }


private:
    Fmc250 *                Owner250;
    IX6AnalogIoInfo *       FInfo;
    AdDac9122::DacDevice    DacDev;
    GeneralCalibrationSection   CalRomSection;


    GeneralIdromImage &  DacCalIdrom()  {  return Owner250->DacCalIDROM_SDev.Idrom();  }

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
