//---------------------------------------------------------------------------
// Fmc_310_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_310_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 2;
const int             InChannelsPerDevice = 2;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices      = 0;
const int             OutChannelsPerDevice = 1;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;

const unsigned int Fmc_310_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs
const unsigned int Fmc_310_VitaOutStreamCount  = MaxOutDevices;   // Number of VITA Outgoing SIDs

const unsigned int Fmc_310_AdcSpiDeviceCount   = MaxInDevices;    // Number of SPI devices muxed
const unsigned int Fmc_310_DacSpiDeviceCount   = MaxOutDevices;   // Number of SPI devices muxed

const unsigned int Fmc_310_AdcChannelsPerDevice   = 1;
const unsigned int Fmc_310_AdcChannels   = MaxInChannels;

const unsigned int Fmc_310_DacChannels   = MaxOutChannels;

const float         InputMaxRate            = 310.0e6f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 16;
const unsigned int  InputShift_Factor       = 4;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int Fmc_310_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc310_Adc_One = 0x10000;         // representation of floating point 1.0

//const float         OutputMaxRate            = 1000.0e6f;
//const float         OutputMinRate            = 5.0e6f;
//const unsigned int  OutputWidth_Bits         = 16;
//const          int  OutputMax_mV             = 1000;
//const          int  OutputMin_mV             = -1000;
//const unsigned int  OutputSamplesPerWord     = 2;
//const unsigned int  OutputTriggerGranularity = 1;

//const unsigned int Fmc_310_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
//const unsigned int Fmc310_Dac_One = 0x10000;         // representation of floating point 1.0
//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
//const int romFmc_310_SysInfoOffset        = 0;
//const int romFmc_310_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  DUMMY IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_310_InputCoeffOffset       = 0;
const int romFmc_310_InputCoeffOffset_avail = 9;   // 2 * 4 coefficients plus 'checksum' word

//  ...Dac Cal Idrom: DAC coefficients (on its own page)
//const int romFmc_310_OutputCoeffOffset       = 0;
//const int romFmc_310_OutputCoeffOffset_avail = 5;  // 2 * 4 coefficients plus 'checksum' word


//===========================================================================
// Class Fmc310_AnalogInInfo
//===========================================================================

class Fmc310_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    Fmc310_AnalogInInfo(Fmc310 * owner) : Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~Fmc310_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const
        {
        unsigned int bits = InputWidth_Bits;
        //if (Owner->AdcAmplitude_SDev.ShiftIsOn())
         //   bits *= InputShift_Factor;
        return bits;
        }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    Fmc310 *   Owner;
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS Fmc310InputDevice  -- Fmc310 Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Fmc310InputDevice
//---------------------------------------------------------------------------

Fmc310InputDevice::Fmc310InputDevice(Fmc310 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner),   FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_310_InputCoeffOffset, MaxInChannels,
                       Fmc310_Adc_One, AdcCal() ),
      AdcDeviceTestEnabled(false),
      CurrentAdcDeviceTestPattern_A(0x01), CurrentAdcDeviceTestPattern_B(0x01)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc310InputDevice
//---------------------------------------------------------------------------

Fmc310InputDevice::~Fmc310InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc310InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::Calibration() -- parent Calibration SDEV
//---------------------------------------------------------------------------

//Fmc310_AdcCalibrationControl &  Fmc310InputDevice::Calibration()
//{
//    return DOwner->Calibration();
//}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void Fmc310InputDevice::AdcDeviceTestMode(bool enabled, unsigned int test_pattern_a,
                                                        unsigned int test_pattern_b)
{
    AdcDeviceTestEnabled = enabled;
    CurrentAdcDeviceTestPattern_A = test_pattern_a;
    CurrentAdcDeviceTestPattern_B = test_pattern_b;
}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::AdcShiftInput() - Shift Input to match output
//---------------------------------------------------------------------------

//void Fmc310InputDevice::AdcShiftInput(bool shift)
//{
//    DOwner->AdcAmplitude_SDev.ShiftInput(shift);
//}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc310InputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void Fmc310InputDevice::Adc_OnConfigure()
{
    DOwner->Debug.Log("Fmc310InputDevice::Adc_OnConfigure()");

    CalRomSection.ApplyRomCoefficients();

    unsigned int test_pattern_a;
    unsigned int test_pattern_b;
    if (!AdcDeviceTestEnabled)
        {
        test_pattern_a = test_pattern_b = static_cast<unsigned int>(tpOff);
        }
    else
        {
        test_pattern_a = CurrentAdcDeviceTestPattern_A;
        test_pattern_b = CurrentAdcDeviceTestPattern_B;
        }
    //
    //  ...init ADC devices
    for (unsigned int dev=0; dev<MaxInDevices; dev++)
        {
        if (AdcCommon().PhysicalDeviceEnabled(dev)==false)
            {
            continue;
            }
        DOwner->Debug.Log("Fmc310 Adc Calibration Device: ", dev);
        unsigned int dummy = Spi(dev, 0x1);
        DOwner->Debug.Log("Chip ID: ", dummy, 16);
        //
        //  ( Soft reset the ADC)
        Spi(dev, 0x0, 0x3c);
        Sleep(10);

        // Set data format to 2s complement
        Spi(dev, 0x14, 0x01);

        // Enable dither on both Channels
        Spi(dev, 0x30, 0x10);

        // Transfer settings
        Spi(dev, 0xFF, 0x01);

        //  Set up Device A
        Adc_A_Configure(dev, test_pattern_a);

        //  Set up Device B
        Adc_B_Configure(dev, test_pattern_b);

        DOwner->Debug.Log("...Adc setup done ");
        DOwner->Debug.Log(" ");
        }
}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::Adc_A_Configure()
//---------------------------------------------------------------------------

void Fmc310InputDevice::Adc_A_Configure(unsigned int dev, unsigned int test_pattern)
{
    DOwner->Debug.Log("Fmc310InputDevice::Adc_A_Configure()");
    //
    // Select ADC channel A
    Spi(dev, 0x5, 0x01);

    // Select the ADC test pattern
    Spi(dev, 0x0d, 0x00 | (test_pattern & 0x0F));

    // Transfer settings
    Spi(dev, 0xFF,  0x01);
}

//---------------------------------------------------------------------------
//  Fmc310InputDevice::Adc_B_Configure()
//---------------------------------------------------------------------------

void Fmc310InputDevice::Adc_B_Configure(unsigned int dev, unsigned int test_pattern)
{
    DOwner->Debug.Log("Fmc310InputDevice::Adc_B_Configure()");
    //
    // Select ADC channel B
    Spi(dev, 0x5, 0x02);

    // Select the ADC test pattern
    Spi(dev, 0x0d, 0x00 | (test_pattern & 0x0F));

    // Transfer settings
    Spi(dev, 0xFF, 0x01);
}

//=============================================================================
//  CLASS Fmc310_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc310_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 20.0e6;

//---------------------------------------------------------------------------
//  constructor for class Fmc310_Pll
//---------------------------------------------------------------------------

Fmc310_Pll::Fmc310_Pll(Fmc310 * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 0 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 20
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);

}

//---------------------------------------------------------------------------
//  destructor for class Fmc310_Pll
//---------------------------------------------------------------------------

Fmc310_Pll::~Fmc310_Pll()
{
}

//---------------------------------------------------------------------------
//  Fmc310_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  Fmc310_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "Fmc310_Pll::OnInit()");

    //  Device Powered/Enabled Setup
    PowerDownOutputGroup(8/2, true);
    PowerDownOutputGroup(10/2, true);
    //
    //  Set Output Configuration
    OutputInUse(0,  false);   SetOutputConfig(0,  ocDisabled);     //  (unused)
    OutputInUse(1,  true);    SetOutputConfig(1,  ocLvds);         //  ADC12_SYNB
    OutputInUse(2,  true);    SetOutputConfig(2,  ocHsds_08ma);    //  Front Panel J7
    OutputInUse(3,  false);   SetOutputConfig(3,  ocLvds);         //  FMC CLK0
                              SetOutput_AsCopy(3);                 //    - is a copy of output 2
    OutputInUse(4,  false);   SetOutputConfig(4,  ocDisabled);     //  (unused)
    OutputInUse(5,  true);    SetOutputConfig(5,  ocLvds);         //  ADC34_SYNB
    OutputInUse(6,  true);    SetOutputConfig(6,  ocHsds_08ma);    //  ADC34_CLK
    OutputInUse(7,  false);   SetOutputConfig(7,  ocDisabled);     //  ADC34_SYNA (ocLvds)
    OutputInUse(8,  false);   SetOutputConfig(8,  ocDisabled);     //  (unused)
    OutputInUse(9,  false);   SetOutputConfig(9,  ocDisabled);     //  (unused)
    OutputInUse(10, false);   SetOutputConfig(10, ocDisabled);     //  (unused)
    OutputInUse(11, false);   SetOutputConfig(11, ocDisabled);     //  (unused)
    OutputInUse(12, true);    SetOutputConfig(12, ocHsds_08ma);    //  ADC12_CLK
    OutputInUse(13, false);   SetOutputConfig(13, ocDisabled);     //  ADC12_SYNA (ocLvds)

    FeedbackOutput(fbm_Dclk6);  // set output to use as PLL feedback in 0-delay modes

    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[0] = 1000.0e6;
//    OutputFrequencyLimit[1] = 1000.0e6;
//    OutputFrequencyLimit[2] = 1000.0e6;
//    OutputFrequencyLimit[3] = 1000.0e6;
//    OutputFrequencyLimit[4] = 1000.0e6;
    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference

    // Insert 2.5 VCO clock cycle delay to align the trigger with the ADC clock
    SetOutputClockHalfStep(1, true);
    SetOutputClockHalfStep(5, true);
    SetSysrefDigitalDelay(1, 3);
    SetSysrefDigitalDelay(5, 3);
}

//---------------------------------------------------------------------------
//  Fmc310_Pll::OnReapply()
//---------------------------------------------------------------------------

void  Fmc310_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  Fmc250_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  Fmc310_Pll::AdcMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  Fmc310_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  Fmc310_Pll::DacMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  Fmc310_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool Fmc310_Pll::IsFeedbackOutputAdc()
{
    return true;
}

//---------------------------------------------------------------------------
//  Fmc310_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  Fmc310_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(1,   adc_rate);
    OutputFrequency(2,   adc_rate);
    //OutputFrequency(3,   adc_rate);  // copy of 2
    OutputFrequency(5,   adc_rate);
    OutputFrequency(6,   adc_rate);
    OutputFrequency(7,   adc_rate);
    OutputFrequency(12,  adc_rate);
    OutputFrequency(13,  adc_rate);
}

//---------------------------------------------------------------------------
//  Fmc310_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  Fmc310_Pll::SetDacOutputRates(double /*dac_rate*/)
{
    // none
}

/*
//=============================================================================
//  CLASS Fmc310_AdcCalOverride  --  Cal Override for FMC ADCs
//=============================================================================

class Fmc310_AdcCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    Fmc310_AdcCalOverride()
        {}
    virtual ~Fmc310_AdcCalOverride()
        {}

private:
    Trace  Debug;
};

//---------------------------------------------------------------------------
//  Fmc310_AdcCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  Fmc310_AdcCalOverride::OverrideOffset(unsigned int value)
{
    Debug.Log("OverrideOffset: Original=", value, 16);
    int offset = static_cast<int>(value);   // need signed offset
    //  Max offset that can be calibrated by the hardware (120uVsteps)
    int offset_cal_range_counts = static_cast<int>(30.72/((float)InputMax_mV/(1<<(InputWidth_Bits-1))));
    //  Trim range to fit hardware limitations
    if (offset>0 && offset>offset_cal_range_counts)
        offset = offset_cal_range_counts;
    else if (offset<0 && offset>offset_cal_range_counts)
        offset = -offset_cal_range_counts;
    //
    //  Convert 'in range' offset to acutal units of 1/4 LSB, unipolar mode
    offset +=  offset_cal_range_counts;
    offset <<= 2;

    Debug.Log("OverrideOffset: New=", offset, 16);
    return static_cast<unsigned int>(offset);
}

//---------------------------------------------------------------------------
//  Fmc310_AdcCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  Fmc310_AdcCalOverride::OverrideGain(unsigned int value)
{
    Debug.Log("OverrideGain: Original=", value, 16);
    const float HdwStep_mV = 0.120f;

    float gain = static_cast<float>(value)/Fmc310_Adc_One;
    if (gain<=1.0)
        return 0;   // no gain shift

    float full_range_mV =  2.0*InputMax_mV;
    float adjustment_mV =  full_range_mV*(1.0f-1.0f/gain);
    unsigned int hdw_steps = static_cast<unsigned int>(adjustment_mV/HdwStep_mV);
    Debug.Log("ADC OverrideGain: New=", hdw_steps, 16);
    return hdw_steps;
}
*/

//===========================================================================
//  CLASS Fmc310  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc310
//---------------------------------------------------------------------------

namespace Fmc310_MemoryMap
{
    //
    //  WB_FMC2 - FMC ADC AFE

} // end namespace Fmc310_MemoryMap


using namespace FmcCarrier_MemoryMap;
namespace
{

FmcLMK20_Clock::MapRegisters FmcLMKClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

Lmk04820Registers  Fmc310_Lmk04820Registers =
{
    mmFmc_PllControl,    //  Control_Addr;
    mmFmc_PllSpi,        //  SpiIntf_Addr

};

FmcSpiDevice::MapRegisters FmcAfeAdcSpiRegAddrs =
    {
    mmFmc_AdcSpiEnable,      //  Device_Addr;
    mmFmc_AdcSpiCtrl,        //  Write_Addr;
    mmFmc_AdcSpiStatus,      //  Read_Addr;
    };


//Fmc310_AdcCalibrationControl::MapRegisters Fmc310AdcCalRegAddrs =
//    {
//    mmFmc_AdcCalStatus_T2,      //  AfeAdcCal_Addr;
//    };


}

//---------------------------------------------------------------------------
//  constructor for class Fmc310
//---------------------------------------------------------------------------

Fmc310::Fmc310()
    :  FmcAdcBoard(fmcPrimary),
       AFEClock_SDev(this, WB_FMC_1, FmcLMKClockRegAddrs),
       AFEAdcSpi_SDev(this, WB_FMC_2, FmcAfeAdcSpiRegAddrs),
       AdcCalIDROM_SDev(this, RomInfo(), true),   // true = ADC Rom
       //AFEAdcCalCtrl_SDev(this, WB_FMC_2, Fmc310AdcCalRegAddrs),
       FInput(new Fmc310InputDevice(this, new Fmc310_AnalogInInfo(this)))//,

{
    LocalConstructCustomParts();
}

Fmc310 * an310()
{
    Fmc310 * wombat = new Fmc310;
    return wombat;
}

//---------------------------------------------------------------------------
//  Fmc310::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  Fmc310::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("Fmc310");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(100);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 20);
//        void    ModuleSize(Fmc::FmcModuleSize size);
//        void    ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size);
//        void    ClockDirection(Fmc::FmcClockDirection dir);
//        void    BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank, unsigned int count);
//        void    TransceiverCount(Fmc::FmcConnector p, unsigned int count);
//        void    MaxClockMHz(unsigned int rateMHz )   {  MaxClock_1 = rateMHz;  }

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(3.3f);
    DcOut->RipplePkPk(0.5f);
//        void          Standby(bool state);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)   {  NominalV_2   = To10mV(v);  }
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          RipplePkPk(float v) {  RipplePkPk_2 = To1mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(3.3f);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)        {  NominalV_2   = To10mV(v);  }
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  Fmc310::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc310::LocalConstructCustomParts()
{
    FmcAdcBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    AFEAdcSpi_SDev.ConstructCustomParts();
    AdcCalIDROM_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  Fmc310::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc310::OpenHardware()
{
    FmcAdcBoard::OpenHardware();

    Debug.Log("++++++++++++++++++++++++++++++++++++");
    Debug.Log("  Load Adc Calibration Shadows");
    AdcCalIDROM_SDev.OpenHardware();
    Debug.Log("++++++++++++++++++++++++++++++++++++");

    AFEClock_SDev.OpenHardware();
    AFEAdcSpi_SDev.ConstructCustomParts();

    Input().OpenHardware();

}

//---------------------------------------------------------------------------
//  Fmc310::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  Fmc310::PreconfigureHardware()
{
    FmcAdcBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();
    AFEAdcSpi_SDev.ConstructCustomParts();

    //  Configure Devices
//    Ad951x_Base_Tb & pll = RawClockDevice().Pll();

//    //  ...delay to allow PLL to settle
//    Sleep(100);
//    //  ...wait for PLL Lock
//    if (pll.OutputSource() == Ad951x_Base_Tb::outPll)
//        {
//        for (unsigned int i=0; i<6; i++)
//            {
//            bool lock_bit = pll.Locked();
//            if (lock_bit)
//                {
//                Debug.Log("PLL Locked!");
//                break;
//                }
//            Innovative::Sleep(1);
//            if (i==5)
//                Debug.Log("ERROR: PLL Lock not achieved in 5 ms...");
//            }
//        }
//    //  ...reset FPGA Pll
//    pll.HdwIntf()->ResetFpgaPll();
//    //  ...wait for FPGA PLL Lock
//    for (unsigned int i=0; i<6; i++)
//        {
//        bool lock_bit = pll.HdwIntf()->IsFpgaPllLocked();
//        if (lock_bit)
//            {
//            Debug.Log("FPGA PLL Locked!");
//            break;
//            }
//        Innovative::Sleep(1);
//        if (i==5)
//            Debug.Log("ERROR: FPGA PLL Lock not achieved in 5 ms...");
//        }

}

//---------------------------------------------------------------------------
//  Fmc310::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc310::ConfigureHardware()
{
    FmcAdcBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    AFEAdcSpi_SDev.ConfigureHardware();

//    Ad951x_Base_Tb & pll = RawClockDevice().Pll();
//    //  ...wait for FPGA PLL Loc
//	bool lock_bit = pll.HdwIntf()->IsFpgaPllLocked();
//    if (lock_bit)
//        {
//        Debug.Log("At Config FPGA PLL Locked!");
//        }
//	else
//        {
//		Debug.Log("ERROR: At Config FPGA PLL Lock not achieved in 5 ms...");
//		}
	//
    //  Special Config for A/D Devices
    Input().Adc_OnConfigure();

}

//---------------------------------------------------------------------------
//  Fmc310::StreamStopHardware()
//---------------------------------------------------------------------------

void  Fmc310::StreamStopHardware()
{
    FmcAdcBoard::StreamStopHardware();

    AFEClock_SDev.StreamStopHardware();
    AFEAdcSpi_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  Fmc310::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc310::GetVitaInStreamCount()
{
    return Fmc_310_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc310::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc310::GetVitaOutStreamCount()
{
    return Fmc_310_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc310::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc310::GetAdcSpiDeviceCount()
{
    return Fmc_310_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc310::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  Fmc310::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new FmcStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  Fmc310::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  Fmc310::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new FmcStandardPriClockDevice(this, 2);
}

//---------------------------------------------------------------------------
//  Fmc310::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc310::GetAdcCalibrationOverride()
{
    return 0; // new Fmc310_AdcCalOverride;
}

//---------------------------------------------------------------------------
//  Fmc310::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc310::GetCalibrationAdcCount()
{
    return Fmc_310_AdcChannels;
}

//---------------------------------------------------------------------------
//  Fmc310::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  Fmc310::GetAnalogDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = Fmc_310_AdcChannels;
    ch_per_dev = Fmc_310_AdcChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc310::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  Fmc310::GetAnalogPhysicalDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = InChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc310::MakeFmcAfePllObject() -- Get Pri Device object
//---------------------------------------------------------------------------

Lmk04820 *  Fmc310::MakeFmcAfePllObject()
{
    return new Fmc310_Pll(this, WB_FMC_1, Fmc310_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  Fmc310::GetFmcCalIdromInfo() -- Make ROM Image Objhect
//---------------------------------------------------------------------------

Fmc310::FmcCalIdromInfo  Fmc310::GetFmcCalIdromInfo(FmcCalDeviceCode which)
{
    Debug.Log("Fmc310::GetFmcCalIdromInfo() which=", which);

    const unsigned int ReservedSize = 1;

    FmcCalIdromInfo  info;
    // only ADCs
//    if (which==fmcCalAdc)
//        {
        info.IdromSize = romFmc_310_InputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
//        }

    return info;
}

//---------------------------------------------------------------------------
//  Fmc310::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void Fmc310::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = maxFmcVadjCode = 0x4;  // 2.5V only
}

//---------------------------------------------------------------------------
//  Fmc310::Input() -- Reference to input object
//---------------------------------------------------------------------------

Fmc310InputDevice & Fmc310::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  Fmc310::AfterFiclStart() --
//---------------------------------------------------------------------------

void  Fmc310::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant AdcSpi0  101 constant AdcSpi1  ");
    engine.Evaluate("  102 constant AdcSpi2  103 constant AdcSpi3  105 constant Pll ");
    engine.Evaluate("  106 constant TempI2C  107 constant IdromI2C   ");

    engine.Evaluate(": ?Fmc310 "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   AdcSpi0  AdcSpi1  Pll  \" "
                    "    cr .\"   TempI2C  IdromI2C   \" "
                    "    cr  "
                    "    cr .\"   ?Fmc310  : display this message \" "
                    "    cr  ; ");

    engine.Display("Fmc310-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  Fmc310::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  Fmc310::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;   const int kPll = 105;
    const int kTempI2C = 106;     const int kIdromI2C = 107;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:    case kAdcSpi1:
            return AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr);
        case kPll:
            return AFEClock_SDev.Pll().Port(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  Fmc310::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  Fmc310::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;   const int kPll = 105;
    const int kTempI2C = 106;     const int kIdromI2C = 107;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:    case kAdcSpi1:
            AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr, value);
            break;
        case kPll:
            AFEClock_SDev.Pll().Port(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  Fmc310::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  Fmc310::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 105;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  Fmc310::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  Fmc310::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 105;

    switch (dev)
        {
        default:
        case kPll:
            return static_cast<float>(AFEClock_SDev.Pll().FrequencyActual());
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
