// Fmc_310_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_310_MbH
#define Fmc_310_MbH

#include <FmcBoard_Mb.h>
#include <X6Support_Mb.h>
#include <FmcSubDevice_Stream_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Fmc310;

//===========================================================================
//  CLASS ISuppliesAdded_Fmc310_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_Fmc310_Requirements :  public IRequires_Lmk20FmcClocks,
                                            public IRequires_MuxedAdcSpi,       // or one with wider address lines?
                                            public IRequires_FmcCalIDROM
{
public:
    virtual ~ISuppliesAdded_Fmc310_Requirements()
        {}

};

//==============================================================================
//  CLASS Fmc310_Pll -- Fmc PLL Chip (Specialization of Lmk04820)
//==============================================================================

class Fmc310_Pll : public Lmk04820
{
public:
    Fmc310_Pll(Fmc310 * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~Fmc310_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    Fmc310 *            Owner;
    double              FTrueFrequency;
    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//===========================================================================
//  CLASS Fmc310_AdcAmplitudeControl -- FMC310 ADC Amp "SubDevice"
//===========================================================================

class Fmc310_AdcAmplitudeControl : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS Fmc310_AdcAmplitudeControl::MapRegisters  -- Dev Reg Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AmplitudeControl_Addr;
    };

    Fmc310_AdcAmplitudeControl(IAddressingSpace & space, MapRegisters & regs);
    virtual ~Fmc310_AdcAmplitudeControl();

    void    ShiftInput(bool shift_it);
    bool    ShiftIsOn();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc310_AdcAmplitudeControl::AmplitudeControlRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AmplitudeControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup    DevShiftOn;

    public:
        AmplitudeControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DevShiftOn(*this, 0, 2)
            {
            }
    };


    AmplitudeControlRegister  AmplControlReg;
};

//===========================================================================
//  CLASS Fmc310  -- Fmc Daughter card Common baseboard class
//===========================================================================

class Fmc310 : public FmcAdcBoard, public ISuppliesAdded_Fmc310_Requirements
{
public:
    friend class Fmc310InputDevice;
    friend class Fmc310_AnalogInInfo;

    Fmc310();
    virtual ~Fmc310()
        {}

    class Fmc310InputDevice  &    Input();

    FmcLMK20_Clock::FmcClockIo &          Clock()           {  return AFEClock_SDev.Clock();  }
    FmcLMK20_Clock::FmcRawClockDevices &  RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    // pseudo-protected

    //Fmc310_AdcCalibrationControl & Calibration() {  return AFEAdcCalCtrl_SDev;  }

    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Suupplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual Lmk04820 *                  MakeFmcAfePllObject();

    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    FmcCalIdromInfo                     GetFmcCalIdromInfo(FmcCalDeviceCode which);


    virtual double  Get_DacClockRate() {  return Clock().Dac().Frequency();  }


    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    FmcLMK20_Clock                   AFEClock_SDev;
    FmcAFE_AdcSpi                    AFEAdcSpi_SDev;
    FmcCalibrationIDROM_Manager      AdcCalIDROM_SDev;    // Cal Rom Image
    //Fmc310_AdcCalibrationControl     AFEAdcCalCtrl_SDev;

    Fmc310InputDevice  *    FInput;

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency(); }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

};

//===========================================================================
//  CLASS Fmc310InputDevice  -- FMC 310 Input Devices
//===========================================================================

class Fmc310InputDevice : public FmcInputDevice
{
public:
    typedef FmcInputDevice inherited;

    Fmc310InputDevice(Fmc310 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc310InputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data)
        {  AdcSpi().Spi(device, address, data);  }
    virtual unsigned int Spi(unsigned int device, unsigned int address)
        {  return AdcSpi().Spi(device, address);  }


    enum AdcDeviceTestPatterns {  tpOff, tpMidscaleShort, tpPosFD, tpNegFs, tpAltCheckerBd, tpPn23Long, tpPn09Short, tpWordToggle };
    void   AdcDeviceTestMode(bool enabled, unsigned int test_pattern_a, unsigned int test_pattern_b);
    //void   AdcShiftInput(bool shift);

    void   OpenHardware();
    void   Adc_OnConfigure();

protected:

    FmcAFE_AdcSpi &          AdcSpi() {  return DOwner->AFEAdcSpi_SDev;  }
    const FmcAFE_AdcSpi &    AdcSpi() const {  return DOwner->AFEAdcSpi_SDev;  }

protected:
    //Fmc310_AdcCalibrationControl & Calibration();

private:
    Fmc310 *                    DOwner;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    bool          AdcDeviceTestEnabled;
    unsigned int  CurrentAdcDeviceTestPattern_A;
    unsigned int  CurrentAdcDeviceTestPattern_B;

    GeneralIdromImage &  AdcCalIdrom()  {  return DOwner->AdcCalIDROM_SDev.Idrom(); }

    void  Adc_A_Configure(unsigned int dev, unsigned int test_pattern);
    void  Adc_B_Configure(unsigned int dev, unsigned int test_pattern);

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
