//---------------------------------------------------------------------------
// Fmc_500_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_500_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 1;
const int             InChannelsPerDevice = 2;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices        = 1;
const int             OutChannelsPerDevice = 2;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;

const unsigned int Fmc_500_VitaInStreamCount   = MaxInDevices;   // Number of VITA Incoming SIDs
const unsigned int Fmc_500_VitaOutStreamCount  = MaxOutDevices;   // Number of VITA Outgoing SIDs

const unsigned int Fmc_500_AdcSpiDeviceCount   = MaxInDevices;   // Number of SPI devices muxed
const unsigned int Fmc_500_DacSpiDeviceCount   = MaxOutDevices;   // Number of SPI devices muxed

const unsigned int Fmc_500_AdcChannelsPerDevice   = InChannelsPerDevice;
const unsigned int Fmc_500_AdcChannels   = MaxInChannels;
const unsigned int Fmc_500_DacChannelsPerDevice   = OutChannelsPerDevice;
const unsigned int Fmc_500_DacChannels   = MaxOutChannels;

const float         InputMaxRate            = 500.0e6f;
const float         InputMinRate            = 5.0e6f;
const unsigned int  InputWidth_Bits         = 14;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 8;

const unsigned int Fmc_500_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_500__Adc_One = 0x10000;         // representation of floating point 1.0

const float         OutputMaxRate            = 615.0e6f;
const float         OutputMinRate            = 5.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 1000;
const          int  OutputMin_mV             = -1000;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 4;

const unsigned int Fmc_500_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
const unsigned int Fmc500_Dac_One = 0x10000;         // representation of floating point 1.0

//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
//const int romFmc_500_SysInfoOffset        = 0;
//const int romFmc_500_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  VGA BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_500_InputCoeffOffset       = 0;
const int romFmc_500_InputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_500_OutputCoeffOffset       = 0;   // these are in separate "ROMs"   //  romFmc_500_InputCoeffOffset_avail;
const int romFmc_500_OutputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//===========================================================================
// Class Fmc500_AnalogInInfo
//===========================================================================

class Fmc500_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    Fmc500_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~Fmc500_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
	virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
// Class Fmc500_AnalogOutInfo
//===========================================================================

class Fmc500_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    Fmc500_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~Fmc500_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
	virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};


//=============================================================================
//  CLASS Fmc500_AdcPriClockDevice  --  Intf to access info for PRI, Frames
//=============================================================================
//  The calc is the save for both divisors, so lets us the same impl

class Fmc500_AdcPriClockDevice : public IPriClockDevice, public IFrameCalcDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->Clock().Frequency();  }
    virtual int   CalculatePriDivisor()
        {
            int divisor;
            if (IsAdcSingleChannel())
                divisor = 8;
            else
                divisor = 4;

            return divisor;
        }
    virtual  unsigned int  CalculateFrameDivisor()
        {  return static_cast<unsigned int>(CalculatePriDivisor());  }

    virtual bool IsAdcSingleChannel()
        {
        return Owner->Input().ActiveChannels()==1;
        }

    Fmc500_AdcPriClockDevice(Fmc500 * owner)
        :  Owner(owner)
        {}
    virtual ~Fmc500_AdcPriClockDevice()
        {}

private:
    Fmc500 * Owner;

};

//===========================================================================
//  CLASS Fmc500InputDevice  -- Fmc500 Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Fmc500InputDevice
//---------------------------------------------------------------------------

Fmc500InputDevice::Fmc500InputDevice(Fmc500 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), Owner500(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_500_InputCoeffOffset, MaxInChannels,
                       Fmc_500__Adc_One, AdcCal() ),
      AdcDeviceTestEnabled(false), CurrentAdcDeviceTestMode(0x00),
      CurrentAdcDeviceTestPattern1(0x01), CurrentAdcDeviceTestPattern2(0x01),
      CurrentAdcDeviceTestPattern3(0x01), CurrentAdcDeviceTestPattern4(0x01)
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc500InputDevice
//---------------------------------------------------------------------------

Fmc500InputDevice::~Fmc500InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc500InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::ActiveDevices() -- Number of DAC devices in use
//---------------------------------------------------------------------------

unsigned int  Fmc500InputDevice::ActiveDevices()
{
    return Fmc_500_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void Fmc500InputDevice::AdcDeviceTestMode(bool enabled, unsigned int test_mode,
                                           unsigned int pattern1, unsigned int pattern2,
                                           unsigned int pattern3, unsigned int pattern4)
{
    AdcDeviceTestEnabled = enabled;
    CurrentAdcDeviceTestMode = test_mode;
    CurrentAdcDeviceTestPattern1 = pattern1;
    CurrentAdcDeviceTestPattern2 = pattern2;
    CurrentAdcDeviceTestPattern3 = pattern3;
    CurrentAdcDeviceTestPattern4 = pattern4;
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc500InputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void Fmc500InputDevice::Adc_OnConfigure(double /*ClockRateMHz*/)
{
    Owner500->Debug.Log("Adc_OnConfigure()");

    CalRomSection.ApplyRomCoefficients();

    if (ChannelPowered(0)==false && ChannelPowered(1)==false)
        {
        Owner500->Debug.Log("ADC Power off, config skipped.");
        return;   // no need to go on, power is off.
        }

    Adc_Init();

    if (ActiveChannels()==2)
        {
        AdcDualChannelParallel();
        }
    else
        {
        AdcSingleChannelParallel();
        }
    // other options?

    Adc_Phy_Cal();

    AdcCommonRegCfg();

    Adc_Lat_Cal();

    Owner500->Debug.Log("end Adc_OnConfigure()");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Adc_Init()
//---------------------------------------------------------------------------

void Fmc500InputDevice::Adc_Init()
{
    Owner500->Debug.Log("Fmc500 Adc INIT:");

    Owner500->AdcPhyCal().Value(0);

    // Soft reset the ADC
    Spi(0, 0x0, 0x81);
    Sleep(20);

    // Datapath Soft reset
    Spi(0, 0x1, 0x2);
    Sleep(5);

    // Read ADC chip_type
    Owner500->Debug.Log("Adc Chip Type =", Spi(0, 0x3), 16);

    // Read ADC chip_id
    Owner500->Debug.Log("Adc Chip ID (low byte) =", Spi(0, 0x4), 16);
    Owner500->Debug.Log("Adc Chip ID (high byte) =", Spi(0, 0x5), 16);

    // Read ADC chip_grade
    Owner500->Debug.Log("Adc Chip Grade =", Spi(0, 0x6), 16);

    // Read ADC SPI revision
    Owner500->Debug.Log("Adc SPI revision =", Spi(0, 0xB), 16);

    // Read ADC vendor_id
    Owner500->Debug.Log("Adc Vendor ID (low byte) =", Spi(0, 0xC), 16);
    Owner500->Debug.Log("Adc Vendor ID (high byte) =", Spi(0, 0xD), 16);

    // Read clock status
    Owner500->Debug.Log("Adc Clock Status =", Spi(0, 0x11C), 16);

    Owner500->Debug.Log("end ADC INIT");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::AdcSingleChannelParallel() -- Single ADC channel in Parallel mode
//---------------------------------------------------------------------------

void Fmc500InputDevice::AdcSingleChannelParallel()
{
    Owner500->Debug.Log("Fmc500 AdcSingleChannelParallel");

    if (ChannelEnabled(0)) // Channel A
        {
        // Select Channel B and power it down
        Spi(0, 0x8, 0x2);
        Spi(0, 0x2, 0x3);
        // Select Channel A and power it up
        Spi(0, 0x8, 0x1);
        Spi(0, 0x2, 0x0);
        }
    else                   // Channel B
        {
        // Select Channel A and power it down
        Spi(0, 0x8, 0x1);
        Spi(0, 0x2, 0x3);
        // Select Channel B and power it up
        Spi(0, 0x8, 0x2);
        Spi(0, 0x2, 0x0);
        // Swap output channels
        Spi(0, 0x564, 0x1);
        }

    // full bandwidth mode
    Spi(0, 0x200, 0x0);
    // full sample rate (decimation = 1)
    Spi(0, 0x201, 0x0);
    // Enable single ADC channel in Parallel mode
    Spi(0, 0x568, 0x00);

    // Enable the all 1/0 word toggle mode
    Spi(0, 0x550, 0x07);

    Owner500->Debug.Log("end AdcSingleChannelParallel");
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::AdcDualChannelParallel() -- Dual ADC channel in Parallel mode
//---------------------------------------------------------------------------

void Fmc500InputDevice::AdcDualChannelParallel()
{
    Owner500->Debug.Log("Fmc500 AdcDualChannelParallel");

    // Select Channels A and B and power them up
    Spi(0, 0x8, 0x3);
    Spi(0, 0x2, 0x0);

    // full bandwidth mode
    Spi(0, 0x200, 0x0);
    // full sample rate (decimation = 1)
    Spi(0, 0x201, 0x0);
    // Enable Dual ADC channels in Parallel mode
    Spi(0, 0x568, 0x01);

    // Select Adc Channel B
    Spi(0, 0x8, 0x02);
    // Enable the user pattern mode - all zeros
    Spi(0, 0x550, 0x08);
    // Select Adc Channel A
    Spi(0, 0x8, 0x01);
    // Enable the positive full scale mode
    Spi(0, 0x550, 0x02);
    // Enable the offset binary data format
    Spi(0, 0x561, 0x00);

    Owner500->Debug.Log("end AdcDualChannelParallel");
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Adc_Phy_Cal()
//---------------------------------------------------------------------------

void Fmc500InputDevice::Adc_Phy_Cal()
{
    Owner500->Debug.Log("Fmc500 Adc_Phy_Cal");

    // ADC PHY CAL
    Owner500->Debug.Log("Cal Reg =",    Owner500->AdcPhyCal().Value(), 16);
    Owner500->Debug.Log("Cal Status =", Owner500->AdcCalStatus().Value(), 16);

    // Enable the OVR output
    Spi(0, 0x559, 0x1);

    // Dummy read to delay starting the 1st stage calibration till the testmode
    // configuration is done
    Spi(0, 0x550);

    // Run stage 1 ADC PHY calibration
    Owner500->AdcPhyCal().CalStart_1.Value(true);
    Owner500->AdcPhyCal().CalStart_1.Value(false);

    // Poll the stage 1 calibration done register
    for (int i=0; i<5; i++)
    {
        if (Owner500->AdcPhyCal().CalDone_1.Value() == true)
        {
            Owner500->Debug.Log("Fmc500 ADC 1st stage calibration done");
            break;
        }
        else if (i<4)
            Sleep(1);
        else
            Owner500->Debug.Log("Fmc500 ADC 1st stage calibration timeout");
    }

    // ADC PHY CAL
    Owner500->Debug.Log("Cal Reg =",    Owner500->AdcPhyCal().Value(), 16);
    Owner500->Debug.Log("Cal Status =", Owner500->AdcCalStatus().Value(), 16);

    // Run the 2nd stage ADC PHY calibration up to 8 times
    for (int i=0; i<8; i++)
    {
    Owner500->Debug.Log("Stage2 Calibration run index = ", i);
    Adc_Phy_Cal2();
    if (Owner500->AdcCalStatus().Cal2Status.Value() == 0x7fff)
        break;
    }
    if (Owner500->AdcCalStatus().Cal2Status.Value() != 0x7fff)
        Owner500->Debug.Log("Fmc500 Stage2 Calibration Failed");

    Owner500->Debug.Log("Fmc500 end Adc_Phy_Cal");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Adc_Phy_Cal2()
//---------------------------------------------------------------------------

void Fmc500InputDevice::Adc_Phy_Cal2()
{
    Owner500->Debug.Log("Fmc500 Adc_Phy_Cal2");

    // Read and store the ADC test mode register
    unsigned int TestModeReadback = Spi(0, 0x550);

    // Enable the user pattern mode - all zeros
    Spi(0, 0x550, 0x8);

    // Dummy read to delay starting the 2nd stage calibration till the testmode
    // configuration is done
    Spi(0, 0x550);

    // Run stage 2 ADC PHY calibration
    Owner500->AdcPhyCal().CalStart_2.Value(true);
    Owner500->AdcPhyCal().CalStart_2.Value(false);

    // Enable the stored test mode
    Spi(0, 0x550, TestModeReadback);

    // Dummy read to delay checking the 2nd stage calibration status till the testmode
    // configuration is done
    Spi(0, 0x550);

    // Read stage 2 calibration status bit
    if (Owner500->AdcPhyCal().CalDone_2.Value() == true)
        Owner500->Debug.Log("Fmc500 ADC 2nd stage calibration done");
    else
        Owner500->Debug.Log("Fmc500 ADC 2nd stage calibration timeout");

    // ADC PHY CAL
    Owner500->Debug.Log("Cal Reg =",    Owner500->AdcPhyCal().Value(), 16);
    Owner500->Debug.Log("Cal Status =", Owner500->AdcCalStatus().Value(), 16);

    Owner500->Debug.Log("Fmc500 end Adc_Phy_Cal2");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::AdcCommonRegCfg() -- Common ADC Registers Configuration
//---------------------------------------------------------------------------

void Fmc500InputDevice::AdcCommonRegCfg()
{
    Owner500->Debug.Log("Fmc500 AdcCommonRegCfg");

    // Select both ADC channels (okay if in single channel mode)
    Spi(0, 0x8, 0x3);

    if (Owner500->AdcAcCoupled())
        Spi(0, 0x16, 0x2C);
    else
        Spi(0, 0x16, 0x0C);   // DC Coupled

    // Set FastDetect pin functions
    Spi(0, 0x40, 0x00);

    // Enable the FastDetect output
    Spi(0, 0x245, 0x01);

    // Enable the test mode
    if (AdcDeviceTestEnabled)
        {
        Spi(0, 0x550, (CurrentAdcDeviceTestMode & 0x0F));
        }
    else
        {
        Spi(0, 0x550, 0x0);
        }

    // Set the user pattern registers
    Spi(0, 0x551, (CurrentAdcDeviceTestPattern1 & 0xFF));
    Spi(0, 0x552, ((CurrentAdcDeviceTestPattern1 >> 8) & 0xFF));
    Spi(0, 0x553, (CurrentAdcDeviceTestPattern2 & 0xFF));
    Spi(0, 0x554, ((CurrentAdcDeviceTestPattern2 >> 8) & 0xFF));
    Spi(0, 0x555, (CurrentAdcDeviceTestPattern3 & 0xFF));
    Spi(0, 0x556, ((CurrentAdcDeviceTestPattern3 >> 8) & 0xFF));
    Spi(0, 0x557, (CurrentAdcDeviceTestPattern4 & 0xFF));
    Spi(0, 0x558, ((CurrentAdcDeviceTestPattern4 >> 8) & 0xFF));

    // Set the data format to twos complement
    Spi(0, 0x561, 0x1);

    // Enable the data format module
    Owner500->AdcPhyCal().EnableDataFmt.Value(true);

    Owner500->Debug.Log("end AdcCommonRegCfg");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Adc_Lat_Cal()
//---------------------------------------------------------------------------

void Fmc500InputDevice::Adc_Lat_Cal()
{
    Owner500->Debug.Log("Fmc500 Adc_Lat_Cal");

    // Enable continuous Sysref
    Spi(0, 0x120, 0x02);
    // Select Sysref on OVR pin
    Spi(0, 0x559, 0x05);

    // Dummy read to delay starting the latency calibration till the Sysref
    // selection on OVR pin is done
    Spi(0, 0x559);

    // Run the latency calibration
    Owner500->AdcPhyCal().CalStart_3.Value(true);
    Sleep(1);
    Owner500->AdcPhyCal().CalStart_3.Value(false);

    // Disable the Sysref mode
    Spi(0, 0x120, 0x00);
    // Enable the OVR output
    Spi(0, 0x559, 0x1);

    // Read latency calibration status bit
    if (Owner500->AdcPhyCal().CalDone_3.Value() == true)
        Owner500->Debug.Log("Fmc500 ADC latency calibration done");
    else
        Owner500->Debug.Log("Fmc500 ADC latency calibration failed");

    Owner500->Debug.Log("Fmc500 end Adc_Lat_Cal");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Test_1Ch_Mux() -- ADC single channel mux test
//---------------------------------------------------------------------------

void Fmc500InputDevice::Test_1Ch_Mux()
{
    Owner500->Debug.Log("Fmc500 Test_1Ch_Mux");

    AdcDualChannelParallel();
    Adc_Phy_Cal();
    AdcCommonRegCfg();

    // Enable single ADC channel in multiplexed mode
    Spi(0, 0x568, 0x02);

    Owner500->Debug.Log("end Test_1Ch_Mux");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Test_2Ch_Mux() -- ADC dual channel mux test
//---------------------------------------------------------------------------

void Fmc500InputDevice::Test_2Ch_Mux()
{
    Owner500->Debug.Log("Fmc500 Test_2Ch_Mux");

    AdcDualChannelParallel();
    Adc_Phy_Cal();
    AdcCommonRegCfg();

    // Enable dual ADC channel in multiplexed mode
    Spi(0, 0x568, 0x03);

    Owner500->Debug.Log("end Test_2Ch_Mux");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Test_Single_DDC() -- ADC single channel Ddc test
//---------------------------------------------------------------------------

void Fmc500InputDevice::Test_Single_DDC()
{
    Owner500->Debug.Log("Fmc500 Test_Single_DDC");

    AdcSingleChannelParallel();
    Adc_Phy_Cal();
    AdcCommonRegCfg();

    // Enable one converter byte mode
    Spi(0, 0x568, 0x34);
    Spi(0, 0x200, 0x20);
    Spi(0, 0x201, 0x00);

    Owner500->Debug.Log("end Test_Single_DDC");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Test_Dual_DDC() -- ADC dual channel Ddc test
//---------------------------------------------------------------------------

void Fmc500InputDevice::Test_Dual_DDC()
{
    Owner500->Debug.Log("Fmc500 Test_Dual_DDC");

    AdcSingleChannelParallel();
    Adc_Phy_Cal();
    AdcCommonRegCfg();

    // Enable one converter byte mode
    Spi(0, 0x568, 0x15);
    Spi(0, 0x200, 0x20);
    Spi(0, 0x201, 0x01);

    Owner500->Debug.Log("end Test_Dual_DDC");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Test_Quad_DDC() -- ADC quad channel Ddc test
//---------------------------------------------------------------------------

void Fmc500InputDevice::Test_Quad_DDC()
{
    Owner500->Debug.Log("Fmc500 Test_Quad_DDC");

    AdcSingleChannelParallel();
    Adc_Phy_Cal();
    AdcCommonRegCfg();

    // Enable one converter byte mode
    Spi(0, 0x568, 0x16);
    Spi(0, 0x200, 0x20);
    Spi(0, 0x201, 0x02);

    Owner500->Debug.Log("end Test_Quad_DDC");

}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Test_Octal_DDC() -- ADC x8 channel Ddc test
//---------------------------------------------------------------------------

void Fmc500InputDevice::Test_Octal_DDC()
{
    Owner500->Debug.Log("Fmc500 Test_Octal_DDC");

    AdcSingleChannelParallel();
    Adc_Phy_Cal();
    AdcCommonRegCfg();

    // Enable one converter byte mode
    Spi(0, 0x568, 0x17);
    Spi(0, 0x200, 0x20);
    Spi(0, 0x201, 0x03);

    Owner500->Debug.Log("end Test_Octal_DDC");

}


//---------------------------------------------------------------------------
//  Fmc500InputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  Fmc500InputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    AdcSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  Fmc500InputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  Fmc500InputDevice::Spi(unsigned int device, unsigned int address)
{
    return AdcSpi().Spi(device, address);
}

//===========================================================================
//  CLASS Fmc500_DacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class Fmc500_DacToHardware
//---------------------------------------------------------------------------

class Fmc500_DacToHardware :  public AdDac9122::IDacToHardware
{
public:
    virtual unsigned short  Spi(int addr)
        {  return Out->Spi(Dev, addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Out->Spi(Dev, addr, value);  }

    virtual void  DacReset(bool state)
        { Out->DacReset(state); }

    Fmc500_DacToHardware( Fmc500OutputDevice * out, unsigned int dev=0 )
        :  Out(out), Dev(dev)
        {}
    virtual ~Fmc500_DacToHardware()
        {}

    Fmc500OutputDevice *   Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS Fmc500OutputDevice  -- Fmc500 Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc500OutputDevice
//---------------------------------------------------------------------------

//const bool DevIs5681 = true;

//---------------------------------------------------------------------------
//  constructor for class Fmc500OutputDevice
//---------------------------------------------------------------------------

Fmc500OutputDevice::Fmc500OutputDevice(Fmc500 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), Owner500(owner), FInfo(info),
      DacDev(new Fmc500_DacToHardware(this, 0)),
      CalRomSection(DacCalIdrom(), romFmc_500_OutputCoeffOffset, MaxOutChannels,
                       Fmc500_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class Fmc500OutputDevice
//---------------------------------------------------------------------------

Fmc500OutputDevice::~Fmc500OutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  Fmc500OutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  Fmc500OutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc500OutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
    DacDev.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  Fmc500OutputDevice::ConfigureHardware()
{
    DacDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  Fmc500OutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    //Device().Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1) );
    Device().ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device().ChannelEnabled(1, DacCommon().ChannelEnabled(1) );

}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  Fmc500OutputDevice::SetDacInterpolation(DacInterpolationModeValues values)
{
    SetDacInterpolation(static_cast<unsigned int>(values));
}

//---------------------------------------------------------------------------
//  Fmc500OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  Fmc500OutputDevice::SetDacInterpolation(unsigned int values)
{
    Owner500->FDacInterpolationMode = values&0x7;
}

//=============================================================================
//  CLASS Fmc500_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc500_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 100.0e6;

//---------------------------------------------------------------------------
//  constructor for class Fmc500_Pll
//---------------------------------------------------------------------------

Fmc500_Pll::Fmc500_Pll(Fmc500 * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg), Owner(owner)
{
    OutputSyncMode(osmSysRefSync);
    Allow_Pll1_FeedbackAdjustment();
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 0 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 100
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);

}

//---------------------------------------------------------------------------
//  destructor for class Fmc500_Pll
//---------------------------------------------------------------------------

Fmc500_Pll::~Fmc500_Pll()
{
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  Fmc500_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "Fmc500_Pll::OnInit()");
    //
    //  Set Source Configuration
    SourceInUse(0, true);   SetSourceMapping(0, IFmcClockIo::rsInternal);  // clkin0 = internal
    SourceInUse(1, true);   SetSourceMapping(1, IFmcClockIo::rsExternal);  // clkin1 = external
    SourceInUse(2, true);   SetSourceMapping(2, IFmcClockIo::rsCarrier);   // clkin2 = carrier

    //  Device Powered/Enabled Setup
    //PowerDownOutputGroup(2/2, true);
    //
    //  Set Output Configuration
    OutputInUse(0,  true ); SetOutputConfig(0,  ocLvds);        // FMC_CLK1 (DAC)
    OutputInUse(1,  true ); SetOutputConfig(1,  ocHsds_08ma);   // ADC_SYSREF
    OutputInUse(2,  true ); SetOutputConfig(2,  ocLvds);        // DAC_FPGA_CLK
    OutputInUse(3,  true ); SetOutputConfig(3,  ocLvds);        // FMC_ADC_SYSREF
    OutputInUse(4,  true ); SetOutputConfig(4,  ocLvds);        // DAC_CLK
    OutputInUse(5,  false); SetOutputConfig(5,  ocDisabled);    // (unused)
    OutputInUse(6,  true ); SetOutputConfig(6,  ocLvds);        // DAC_REF_CLK
    OutputInUse(7,  false); SetOutputConfig(7,  ocDisabled);    // (unused)
    OutputInUse(8,  true ); SetOutputConfig(8,  ocDisabled);    // feedback
    OutputInUse(9,  false); SetOutputConfig(9,  ocDisabled);    // (unused)
    OutputInUse(10, true ); SetOutputConfig(10, ocHsds_08ma);   // Front Panel J12
    OutputInUse(11, false); SetOutputConfig(11, ocDisabled);    // (unused)
    OutputInUse(12, true ); SetOutputConfig(12, ocLvds);        // FMC_CLK0 (ADC)
    OutputInUse(13, true ); SetOutputConfig(13, ocHsds_08ma);   // ADC_CLK
                            SetOutput_AsCopy(13);               // make 13 a copy of 12

    FeedbackOutput(fbm_Dclk8);  // set output to use as PLL feedback in 0-delay modes

    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6;
    //      if the physical rate limit is 100 MHz
    //    OutputFrequencyLimit[0] = 1000.0e6;
    //    OutputFrequencyLimit[1] = 1000.0e6;
    //    OutputFrequencyLimit[2] = 1000.0e6;
    //    OutputFrequencyLimit[3] = 1000.0e6;
    //    OutputFrequencyLimit[4] = 1000.0e6;
    //
    //  Reference
    //    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference

    //  Compensate for no pullup on PLL lock bits.
    SetLockConfig(stpOutWpush_pull);
    SetPll2LockConfig(stpOutWpush_pull);
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::OnReapply()
//---------------------------------------------------------------------------

void  Fmc500_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  Fmc500_Pll::AdcMasterOutput()
{
    return 12;
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  Fmc500_Pll::DacMasterOutput()
{
    return 4;
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool Fmc500_Pll::IsFeedbackOutputAdc()
{
    return true;
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  Fmc500_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(1,  adc_rate/256);  // disabled
    OutputFrequency(3,  adc_rate/256);  // disabled
    OutputFrequency(8,  adc_rate);      // feedback
    OutputFrequency(10, adc_rate);
    OutputFrequency(12, adc_rate);
    //OutputFrequency(13, adc_rate);      // copy of 12
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  Fmc500_Pll::SetDacOutputRates(double dac_rate)
{
    OutputFrequency(0, dac_rate);
    OutputFrequency(2, DCLK(dac_rate, DacMode()));
    OutputFrequency(4, dac_rate);
    OutputFrequency(6, DCLK(dac_rate, DacMode())/8);
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::DCLK() -- Set DCLK rate by DAC rate
//---------------------------------------------------------------------------

double   Fmc500_Pll::DCLK(double dac_rate, unsigned int mode)
{
    switch (mode)
        {
        case 0:  return dac_rate;      // max rate 615;
        case 1:  return dac_rate/2;    // max rate 1000;
        case 2:  return dac_rate/2;    // max rate 1230;
        case 3:  return dac_rate/4;    // max rate 1230;
        case 6:  return dac_rate/4;    // max rate 1230;
        default:
        case 7:  return dac_rate/8;    // max rate 1230;
        }
}

//---------------------------------------------------------------------------
//  Fmc500_Pll::DacMode() -- Get current Dac Mode
//---------------------------------------------------------------------------

unsigned int   Fmc500_Pll::DacMode()
{
    return Owner->CurrentDacInterpolationMode();
}

//=============================================================================
//  CLASS Fmc500_DacCalOverride  --  Cal Override for Fmc500 Dacs
//=============================================================================

class Fmc500_DacCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    Fmc500_DacCalOverride(Fmc500 *  owner) : Owner(owner)
        {}
    virtual ~Fmc500_DacCalOverride()
        {}

private:
    Fmc500 *  Owner;
};

//---------------------------------------------------------------------------
//  Fmc500_DacCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  Fmc500_DacCalOverride::OverrideOffset(unsigned int value)
{
    // trim to 16 bit value
    return value&0xFFFF;
}

//---------------------------------------------------------------------------
//  Fmc500_DacCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  Fmc500_DacCalOverride::OverrideGain(unsigned int value)
{
    const float HdwStep_mA    = 0.0225f;    // 22.5 uA steps
    const float Min_mA        = 8.64f;      // 0x000 gain value
    const float Center_mA     = 20.00f;     // 0x1f9 gain value


    float gain = static_cast<float>(value)/Fmc500_Dac_One;

    float adjusted_mA = gain*Center_mA;
    float steps_mA = adjusted_mA - Min_mA;

    float counts = steps_mA / HdwStep_mA;
    unsigned int setting;
    if (counts < 0)
        setting = 0;
    else if (counts > 1023)
        setting = 0x3FF;
    else
        setting = static_cast<unsigned int>(counts);

    return setting;
}

//---------------------------------------------------------------------------
//  Fmc500_DacCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  Fmc500_DacCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Offset(value);
    else
        Owner->Output().Device().Write_Q_Offset(value);

}

//---------------------------------------------------------------------------
//  Fmc500_DacCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  Fmc500_DacCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Gain(value);
    else
        Owner->Output().Device().Write_Q_Gain(value);

}

//===========================================================================
//  CLASS Fmc500  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Fmc500
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{

FmcLMK20_Clock::MapRegisters FmcLMKClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

Lmk04820Registers  Fmc500_Lmk04820Registers =
{
    mmFmc_PllControl,    //  Control_Addr;
    mmFmc_PllSpi,        //  SpiIntf_Addr
};

//  FMC500 memory Map
//
//  WB_FMC2 - FMC ADC AFE
const unsigned int  mmFmc500_AdcPHYCal       = 0x14;
const unsigned int  mmFmc500_AdcPHYCalStatus = 0x15;


bool IsInFmc500Mode = true;


}

//---------------------------------------------------------------------------
//  constructor for class Fmc500
//---------------------------------------------------------------------------

Fmc500::Fmc500()
    :  FmcBidirBoard(fmcPrimary, IsInFmc500Mode),
       AFEClock_SDev(this, WB_FMC_1, FmcLMKClockRegAddrs),
       AdcCalIDROM_SDev(this, RomInfo(), true),   // true = ADC Rom
       DacCalIDROM_SDev(this, RomInfo(), false),  // true = ADC Rom
       AFEAdcVcoReg(WB_FMC_1, mmFmc_VcxoControl),
       AdcPhyCalReg(WB_FMC_2, mmFmc500_AdcPHYCal),
       AdcPhyCalStatus(WB_FMC_2, mmFmc500_AdcPHYCalStatus),
       FInput(new Fmc500InputDevice(this, new Fmc500_AnalogInInfo)),
       FOutput(new Fmc500OutputDevice(this, new Fmc500_AnalogOutInfo)),
       FDacClocksDisabled(false), FTestClocksDisabled(true),
       FDacInterpolationMode(0x0), FAdcAcCoupled(false)
{
    LocalConstructCustomParts();
}


Fmc500 * an500()
{
    Fmc500 * wombat = new Fmc500;
    return wombat;
}

//---------------------------------------------------------------------------
//  Fmc500::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  Fmc500::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  Header Section
    RI.Header().Version(0x2);

    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("Fmc500");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(500);
    RI.FmcDef().ModuleSize(Fmc::msSingleWide);
    RI.FmcDef().ClockDirection(Fmc::dirCarrierToMezz);
    RI.FmcDef().ConnectorSize(Fmc::fmcP1, Fmc::csHPC);
    RI.FmcDef().ConnectorSize(Fmc::fmcP2, Fmc::csNotFitted);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankA, 87);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankB, 30);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 0);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankB, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP1, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP2, 0);

    //  ADC CALIBRATION SECTION
    RI.AdcCal().Resize(romFmc_500_InputCoeffOffset_avail);
    RI.AdcCal().Zero();

    //  DAC CALIBRATION SECTION
    RI.DacCal().Resize(romFmc_500_OutputCoeffOffset_avail);
    RI.DacCal().Zero();

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(2.5f);
    DcOut->RipplePkPk(0.5f);
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(false);
    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    //        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
    //        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
    //        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(2.5f);
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  Fmc500::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  Fmc500::LocalConstructCustomParts()
{
    FmcBidirBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    AdcCalIDROM_SDev.ConstructCustomParts();
    DacCalIDROM_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  Fmc500::OpenHardware()
//---------------------------------------------------------------------------

void  Fmc500::OpenHardware()
{
    PokeLogic(0x0,   0x00000);  // offsets relative to FMC

    FmcBidirBoard::OpenHardware();

    Debug.Log("++++++++++++++++++++++++++++++++++++");
    Debug.Log("  Load Adc/Dac Calibration Shadows");
    AdcCalIDROM_SDev.OpenHardware();
    DacCalIDROM_SDev.OpenHardware();
    Debug.Log("++++++++++++++++++++++++++++++++++++");

    //  Turn on VCO Power
    AFEAdcVcoReg.VcoPowerEnable.Value(true);

    PokeLogic(0x202, 0x3);
    unsigned int data = PeekLogic(0x0);
    Debug.Log("Power Register Final: ", data, 16);

    AFEClock_SDev.OpenHardware();

    Input().OpenHardware();
    Output().OpenHardware();

}

//---------------------------------------------------------------------------
//  Fmc500::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  Fmc500::PreconfigureHardware()
{
    Output().ConfigureChannelEnables();

    FmcBidirBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();
    //
    //  Special Config for D/A Devices
    Output().PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  Fmc500::ConfigureHardware()
//---------------------------------------------------------------------------

void  Fmc500::ConfigureHardware()
{
    FmcBidirBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    //
    //  Special Config for A/D Devices
    double ClockRateMHz = AFEClock_SDev.Clock().Adc().FrequencyActual()/1.0e6;
    Input().Adc_OnConfigure(ClockRateMHz);
    //
    //  Adc Calibration
    //AFEAdcCalCtrl_SDev.StartCalibration();
    //Sleep(1);
    //unsigned int results = AFEAdcCalCtrl_SDev.CalibrationResults();

    //Debug.Log("Fmc500 Adc Cal Results: ", results);
    //
    //  Special Config for D/A Devices
    Output().ConfigureHardware();

}

//---------------------------------------------------------------------------
//  Fmc500::StreamStopHardware()
//---------------------------------------------------------------------------

void  Fmc500::StreamStopHardware()
{
    FmcBidirBoard::StreamStopHardware();

    //AFEClock_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  Fmc500::VcoPowerGood() -- Return Power Good Bit on VCO
//---------------------------------------------------------------------------

bool Fmc500::VcoPowerGood()
{
    return AFEAdcVcoReg.VcoPowerGood.Value();
}

//---------------------------------------------------------------------------
//  Fmc500::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc500::GetVitaInStreamCount()
{
    return Fmc_500_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc500::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  Fmc500::GetVitaOutStreamCount()
{
    return Fmc_500_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  Fmc500::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc500::GetAdcSpiDeviceCount()
{
    return Fmc_500_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc500::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  Fmc500::GetDacSpiDeviceCount()
{
    return Fmc_500_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  Fmc500::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  Fmc500::MakeFrameCalcDevice(FrameDeviceCode which)
{
    // return same object for both directions  (we don't need an object)
    if (which==devAdc)
        return new Fmc500_AdcPriClockDevice(this);
    else
        return new FmcStandardFrameCalcDevice(this, OutputTriggerGranularity);
}

//---------------------------------------------------------------------------
//  Fmc500::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  Fmc500::MakePriClockDevice(PriDeviceCode which)
{
    // return same object for both directions  (we don't need an object)
    if (which==priAdc)
        return new Fmc500_AdcPriClockDevice(this);
    else
        return new FmcStandardPriClockDevice(this, OutputTriggerGranularity);
}


//---------------------------------------------------------------------------
//  Fmc500::MakeFmcAfePllObject() -- Get Pri Device object
//---------------------------------------------------------------------------

Lmk04820 *  Fmc500::MakeFmcAfePllObject()
{
    // return same object for both directions
    return new Fmc500_Pll(this, WB_FMC_1, Fmc500_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  Fmc500::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc500::GetAdcCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  Fmc500::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc500::GetCalibrationAdcCount()
{
    return Fmc_500_AdcChannels;
}

//---------------------------------------------------------------------------
//  Fmc500::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  Fmc500::GetDacCalibrationOverride()
{
    return new Fmc500_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  Fmc500::GetCalibrationDacCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  Fmc500::GetCalibrationDacCount()
{
    return Fmc_500_DacChannels;
}

//---------------------------------------------------------------------------
//  Fmc500::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  Fmc500::GetAnalogDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = InChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc500::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  Fmc500::GetAnalogPhysicalDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = InChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  Fmc500::GetFmcCalIdromInfo() -- Make ROM Image Objhect
//---------------------------------------------------------------------------

Fmc500::FmcCalIdromInfo  Fmc500::GetFmcCalIdromInfo(FmcCalDeviceCode which)
{
    const unsigned int ReservedSize = 1;

    FmcCalIdromInfo  info;
    if (which==fmcCalAdc)
        {
        info.IdromSize = romFmc_500_InputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }
    else
        {
        info.IdromSize = romFmc_500_OutputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }

    return info;
}

//---------------------------------------------------------------------------
//  Fmc500::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void Fmc500::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = 0x3;  // 1.8V min
    maxFmcVadjCode = 0x4;  // 2.5V max
}

//---------------------------------------------------------------------------
//  Fmc500::Input() -- Reference to input object
//---------------------------------------------------------------------------

Fmc500InputDevice & Fmc500::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  Fmc500::Output() -- Reference to output object
//---------------------------------------------------------------------------

Fmc500OutputDevice & Fmc500::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  Fmc500::AfterFiclStart() --
//---------------------------------------------------------------------------

void  Fmc500::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant AdcSpi0  102 constant Pll");
    engine.Evaluate("  103 constant TempI2C  104 constant kIdromI2C 105 constant PllImage");
    engine.Evaluate("  106 constant DacSpi0  ");
    engine.Evaluate(": ?Fmc500 "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   AdcSpi0  Pll  \" "
                    "    cr .\"   TempI2C  IdromI2C  PllImage \" "
                    "    cr .\"   DacSpi0 \" "
                    "    cr  "
                    "    cr .\"   ?Fmc500  : display this message \" "
                    "    cr  ; ");

    engine.Display("Fmc500-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  Fmc500::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  Fmc500::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;
    const int kPll = 102;
    const int kTempI2C = 103;   const int kIdromI2C = 104;
    const int kPllImage = 105;
    const int kDacSpi0 = 106;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:    case kAdcSpi1:
            return AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr);
        case kDacSpi0:
            return AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr);
        case kPll:
            return AFEClock_SDev.Pll().Port(addr);
        case kPllImage:
            return AFEClock_SDev.Pll().PortImage(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  Fmc500::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  Fmc500::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;
    const int kPll = 102;
    const int kTempI2C = 103;  const int kIdromI2C = 104;
    const int kPllImage = 105;
    const int kDacSpi0 = 106;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:    case kAdcSpi1:
            AFEAdcSpi_SDev.Spi(dev-kAdcSpi0, addr, value);
            break;
        case kDacSpi0:
            AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr, value);
            break;
        case kPll:
            AFEClock_SDev.Pll().Port(addr, value);
            break;
        case kPllImage:
            AFEClock_SDev.Pll().PortImage(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  Fmc500::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  Fmc500::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 102;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  Fmc500::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  Fmc500::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 102;

    switch (dev)
        {
        default:
        case kPll:
            return static_cast<float>(AFEClock_SDev.Pll().FrequencyActual());
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
