// Fmc_500_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_500_MbH
#define Fmc_500_MbH

#include <FmcBoard_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>
#include <AdDac9122_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Fmc500;

//===========================================================================
//  CLASS ISuppliesAdded_500_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_500_Requirements : public IRequires_Lmk20FmcClocks,
                                        public IRequires_MuxedAdcSpi,       // or one with wider address lines?
                                        public IRequires_FmcCalIDROM
{
public:
    virtual ~ISuppliesAdded_500_Requirements()
        {}

};

//==============================================================================
//  CLASS Fmc500_Pll -- Fmc PLL Chip (Specialization of Lmk04820)
//==============================================================================

class Fmc500_Pll : public Lmk04820
{
public:
    Fmc500_Pll(Fmc500 * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~Fmc500_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    bool IsFeedbackOutputAdc();

protected:
    Fmc500 *            Owner;
    double              FTrueFrequency;
    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

    double   DCLK(double dac_rate, unsigned int mode);
    unsigned int DacMode();


};

//===========================================================================
//  CLASS Fmc500  -- Fmc Daughter card Common baseboard class
//===========================================================================

class Fmc500 : public FmcBidirBoard, public ISuppliesAdded_500_Requirements
{
public:
    Fmc500();
    virtual ~Fmc500()
        {}

    // Options
    void  DisableDacClocks(bool state)
        {  FDacClocksDisabled = state;  }
    bool  DacClocksDisabled() const
        {  return FDacClocksDisabled;  }
    void  DisableTestClocks(bool state)
        {  FTestClocksDisabled = state;  }
    bool  TestClocksDisabled() const
        {  return FTestClocksDisabled;  }

    void AdcAcCoupled(bool state)  {  FAdcAcCoupled = state;  }
    bool AdcAcCoupled()  {  return FAdcAcCoupled;  }

    bool  VcoPowerGood();

    class Fmc500InputDevice &    Input();
    class Fmc500OutputDevice &   Output();

    friend class Fmc500InputDevice;
    friend class Fmc500OutputDevice;

    FmcLMK20_Clock::FmcClockIo &          Clock()           {  return AFEClock_SDev.Clock();  }
    FmcLMK20_Clock::FmcRawClockDevices &  RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    virtual double  Get_DacClockRate()  { return Clock().Dac().Frequency(); }

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Supplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    virtual Lmk04820 *                  MakeFmcAfePllObject();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual FmcCalIdromInfo             GetFmcCalIdromInfo(FmcCalDeviceCode which);

    //
    //  Misc
    unsigned int      CurrentDacInterpolationMode()
        {  return FDacInterpolationMode;  }

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc500::AFEAdcVcoRegister  --  Adc Calibration Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AFEAdcVcoRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   VcoPowerEnable;
        RegisterBit   VcoPowerGood;

        AFEAdcVcoRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              VcoPowerEnable(*this, 4),
              VcoPowerGood(*this, 8)
        {  }
    };

    AFEAdcVcoRegister &  AdcVcoRegister()  { return AFEAdcVcoReg; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc500::AdcPhyCalRegister  --  Adc Calibration Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AdcPhyCalRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        PowerGood;
        RegisterBit        CalStart_1;
        RegisterBit        CalStart_2;
        RegisterBit        CalStart_3;
        RegisterBit        EnableDataFmt;
        RegisterBit        CalDone_1;
        RegisterBit        CalDone_2;
        RegisterBit        CalDone_3;

        AdcPhyCalRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              PowerGood(*this, 8),
              CalStart_1(*this, 12),
              CalStart_2(*this, 13),
              CalStart_3(*this, 14),
              EnableDataFmt(*this, 15),
              CalDone_1(*this, 16),
              CalDone_2(*this, 17),
              CalDone_3(*this, 18)
        {  }
    };

    AdcPhyCalRegister &  AdcPhyCal() {  return AdcPhyCalReg;  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Fmc500::AdcPhyCalStatusRegister  --  Adc Calibration Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AdcPhyCalStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Cal1Status;
        RegisterBitGroup   Cal2Status;

        AdcPhyCalStatusRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Cal1Status(*this, 0, 15),
              Cal2Status(*this, 16, 15)
        {  }
    };

    AdcPhyCalStatusRegister &  AdcCalStatus()  { return AdcPhyCalStatus; }
protected:
    //
    //  Subdevices
    FmcLMK20_Clock     AFEClock_SDev;
    FmcCalibrationIDROM_Manager    AdcCalIDROM_SDev;    // Cal Rom Image
    FmcCalibrationIDROM_Manager    DacCalIDROM_SDev;    // Cal Rom Image

    AFEAdcVcoRegister   AFEAdcVcoReg;
    AdcPhyCalRegister  AdcPhyCalReg;
    AdcPhyCalStatusRegister AdcPhyCalStatus;

    Fmc500InputDevice  *    FInput;
    Fmc500OutputDevice  *   FOutput;

    bool              FDacClocksDisabled;
    bool              FTestClocksDisabled;

    unsigned int      FDacInterpolationMode;

    bool              FAdcAcCoupled;


    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

};

//===========================================================================
//  CLASS Fmc500InputDevice  -- FMC 500 Input Devices
//===========================================================================

class Fmc500InputDevice : public FmcInputDevice_Bidir
{
public:
    typedef FmcInputDevice_Bidir inherited;

    Fmc500InputDevice(Fmc500 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc500InputDevice();

    virtual unsigned int  ActiveDevices();

    virtual IX6AnalogIoInfo &  Info();

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    void   AdcDeviceTestMode(bool enabled, unsigned int test_mode, unsigned int pattern1,
                             unsigned int pattern2, unsigned int pattern3, unsigned int pattern4);

    void   OpenHardware();
    void   Adc_OnConfigure(double ClockRateMHz);

    bool   AdcPowerGood()
        {
        return Owner500->AdcPhyCal().PowerGood.Value();
        }

private:
    Fmc500 *                    Owner500;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    bool          AdcDeviceTestEnabled;
    unsigned int  CurrentAdcDeviceTestMode;
    unsigned int  CurrentAdcDeviceTestPattern1;
    unsigned int  CurrentAdcDeviceTestPattern2;
    unsigned int  CurrentAdcDeviceTestPattern3;
    unsigned int  CurrentAdcDeviceTestPattern4;

    FmcAFE_AdcSpi &          AdcSpi() {  return Owner500->AFEAdcSpi_SDev;  }
    const FmcAFE_AdcSpi &    AdcSpi() const {  return Owner500->AFEAdcSpi_SDev;  }

    GeneralIdromImage &  AdcCalIdrom()  {  return Owner500->AdcCalIDROM_SDev.Idrom();  }

    void Adc_Init();
    void AdcSingleChannelParallel();
    void AdcDualChannelParallel();
    void Adc_Phy_Cal();
    void Adc_Phy_Cal2();
    void Adc_Lat_Cal();
    void AdcCommonRegCfg();

    void Test_1Ch_Mux();
    void Test_2Ch_Mux();
    void Test_Single_DDC();
    void Test_Dual_DDC();
    void Test_Quad_DDC();
    void Test_Octal_DDC();

};

//===========================================================================
//  CLASS Fmc500OutputDevice  -- FMC 500 Output Devices
//===========================================================================

class Fmc500OutputDevice : public FmcOutputDevice_Bidir
{
public:
    friend class Fmc110_DacToHardware;

    typedef FmcOutputDevice_Bidir inherited;

    Fmc500OutputDevice(Fmc500 * owner, IX6AnalogIoInfo * info );
    virtual ~Fmc500OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    //  Special Dac Interpolation Mode
    enum  DacInterpolationModeValues { diByOne=0x0, diByTwo=0x1, diByTwo_b=0x02,
                                       diByFour=0x3, diByFour_b=0x6, diByEight=0x7 };
    void  SetDacInterpolation(DacInterpolationModeValues values);
    void  SetDacInterpolation(unsigned int values);


    AdDac9122::DacDevice &     Device()
        {
            return DacDev;
        }
    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }
    bool  DacPowerGood()
        {
        return DacCommon().DacPowerGood();
        }

    // Pseudo-protected
    void  OpenHardware();
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }


private:
    Fmc500 *                    Owner500;
    IX6AnalogIoInfo *           FInfo;
    AdDac9122::DacDevice        DacDev;
    GeneralCalibrationSection   CalRomSection;


    GeneralIdromImage &  DacCalIdrom()  {  return Owner500->DacCalIDROM_SDev.Idrom();  }
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
