//---------------------------------------------------------------------------
// Fmc_Adc20_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_Adc20_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 5;
const int             ChannelsPerDevice = 4;
const int             MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const unsigned int Fmc_Adc20_VitaInStreamCount   = MaxInDevices;   // Number of VITA Incoming SIDs
const unsigned int Fmc_Adc20_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

const unsigned int Fmc_Adc20_AdcSpiDeviceCount   = MaxInDevices;   // Number of SPI devices muxed

const unsigned int Fmc_Adc20_ChannelsPerDevice   = ChannelsPerDevice;
const unsigned int Fmc_Adc20_Channels   = MaxInChannels;

const float         InputMaxRate            = 25.0e6f;
const float         InputMinRate            = 5.0e6f;
const unsigned int  InputWidth_Bits         = 14;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int Fmc_Adc20_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_Adc20__Adc_One = 0x10000;         // representation of floating point 1.0

//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
const int romFmc_Adc20_SysInfoOffset        = 0;
const int romFmc_Adc20_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  VGA BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_Adc20_InputCoeffOffset       = 0;
const int romFmc_Adc20_InputCoeffOffset_avail = 41;   // 20 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_Adc20_OutputCoeffOffset       = romFmc_Adc20_InputCoeffOffset_avail;
const int romFmc_Adc20_OutputCoeffOffset_avail = 0;  // none used

//===========================================================================
// Class FmcAdc20_AnalogInInfo
//===========================================================================

class FmcAdc20_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    FmcAdc20_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~FmcAdc20_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS FmcAdc20InputDevice  -- FmcAdc20 Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FmcAdc20InputDevice
//---------------------------------------------------------------------------

FmcAdc20InputDevice::FmcAdc20InputDevice(FmcAdc20 * owner, IX6AnalogIoInfo * info)
    : inherited(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_Adc20_InputCoeffOffset, MaxInChannels,
                       Fmc_Adc20__Adc_One, AdcCal() ),
      AdcDeviceTestEnabled(false), CurrentAdcDeviceTestPattern(0x01)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcAdc20InputDevice
//---------------------------------------------------------------------------

FmcAdc20InputDevice::~FmcAdc20InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  FmcAdc20InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  FmcAdc20InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  FmcAdc20InputDevice::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------
//
//unsigned int  FmcAdc20InputDevice::Channels()
//{
//    return MaxInChannels;
//}

//---------------------------------------------------------------------------
//  FmcAdc20InputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void FmcAdc20InputDevice::AdcDeviceTestMode(bool enabled, unsigned int test_pattern)
{
    AdcDeviceTestEnabled = enabled;
    CurrentAdcDeviceTestPattern = test_pattern;
}

//---------------------------------------------------------------------------
//  FmcAdc20InputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void FmcAdc20InputDevice::Adc_OnConfigure()
{
    //  Configure Devices
    for (unsigned int dev=0; dev<MaxInDevices; dev++)
        {
        if (DeviceEnabled(dev)==false)
            {
            Spi(dev, 0x01, 0x10);   // sleep ADC device
            continue;
            }
        //
        //  ...device enabled
        Spi(dev, 0x00, 0x80);   // reset ADC device
        Spi(dev, 0x01, 0x40);
        Spi(dev, 0x02, 0xA0);

        if (AdcDeviceTestEnabled)
            {
            MemoryBitManipulator Reg4(0);
            Reg4.Field(0, 8, CurrentAdcDeviceTestPattern);
            Spi(dev, 0x04, Reg4.Value());   // pattern LSB

            MemoryBitManipulator Reg3(0);
            Reg3.Field(0, 6, CurrentAdcDeviceTestPattern>>8);
            Reg3.Field(6, 2, 0x02);         // activation bit
            Spi(dev, 0x03, Reg3.Value());   // pattern MSB
            }
        }
}


//=============================================================================
//  CLASS FmcAdc20_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAdc20_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class FmcAdc20_Pll
//---------------------------------------------------------------------------

FmcAdc20_Pll::FmcAdc20_Pll(FmcAdc20 * owner, IAddressingSpace &space, Cdce62005Registers & reg)
    : Cdce62005(space, reg), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 0 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class FmcAdc20_Pll
//---------------------------------------------------------------------------

FmcAdc20_Pll::~FmcAdc20_Pll()
{
}

//---------------------------------------------------------------------------
//  FmcAdc20_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  FmcAdc20_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "FmcAdc20_Pll::OnInit()");

    //  Device Powered/Enabled Setup
//    SetPllEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvds);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Primary Input Term NOT disabled
                         false,  // Seconary Input Term NOT disabled
                         false,  // Primary Bias OFF
                         false,  // Secondary Bias OFF
                         false   // NO failsafe
                         );
    //
    //  Set Output Configuration
    OutputInUse(0, true);   SetOutputConfig(0, ocLvds);     // Adc Dev 0
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds);     // Adc Dev 1
    OutputInUse(2, true);   SetOutputConfig(2, ocLvds);     // Adc Dev 2
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds);     // Adc Dev 3
    OutputInUse(4, true);   SetOutputConfig(4, ocLvds);     // Adc Dev 4

    //  Phase = 0
    SetPhaseAdjust(0, 1, 0);  SetPhaseAdjust(1, 1, 0);  // Set all outputs to 0 phase
    SetPhaseAdjust(2, 1, 0);  SetPhaseAdjust(3, 1, 0);
    SetPhaseAdjust(4, 1, 0);

    //  Configure Aux
    SetAuxOutputConfig(true, aoFromDivider2);

    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[0] = 1000.0e6;
//    OutputFrequencyLimit[1] = 1000.0e6;
//    OutputFrequencyLimit[2] = 1000.0e6;
//    OutputFrequencyLimit[3] = 1000.0e6;
//    OutputFrequencyLimit[4] = 1000.0e6;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference

}

//---------------------------------------------------------------------------
//  FmcAdc20_Pll::OnReapply()
//---------------------------------------------------------------------------

void  FmcAdc20_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  FmcAdc20_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  FmcAdc20_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  FmcAdc20_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  FmcAdc20_Pll::DacMasterOutput()
{
    return FMasterOutput;
}

//---------------------------------------------------------------------------
//  FmcAdc20_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  FmcAdc20_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(0, adc_rate);
    OutputFrequency(1, adc_rate);
    OutputFrequency(2, adc_rate);
    OutputFrequency(3, adc_rate);
    OutputFrequency(4, adc_rate);
}

//---------------------------------------------------------------------------
//  FmcAdc20_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  FmcAdc20_Pll::SetDacOutputRates(double /*dac_rate*/)
{
}

//===========================================================================
//  CLASS FmcAdc20  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcAdc20
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{

FmcAFE_Clock::MapRegisters FmcAFEClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

static Cdce62005Registers FmcAdc20_PllRegisters =
    { mmFmc_PllControl,  mmFmc_PllSpi  };


}

//---------------------------------------------------------------------------
//  constructor for class FmcAdc20
//---------------------------------------------------------------------------

FmcAdc20::FmcAdc20()
    :  FmcAdcBoard_T1(fmcPrimary),
       AFEClock_SDev(this, WB_FMC_1, FmcAFEClockRegAddrs),
       FInput(new FmcAdc20InputDevice(this, new FmcAdc20_AnalogInInfo))//,
{
    LocalConstructCustomParts();
}

FmcAdc20 * anAdc20()
{
    FmcAdc20 * wombat = new FmcAdc20;
    return wombat;
}

//---------------------------------------------------------------------------
//  FmcAdc20::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  FmcAdc20::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("FmcAdc20");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(100);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 20);
//        void    ModuleSize(Fmc::FmcModuleSize size);
//        void    ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size);
//        void    ClockDirection(Fmc::FmcClockDirection dir);
//        void    BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank, unsigned int count);
//        void    TransceiverCount(Fmc::FmcConnector p, unsigned int count);
//        void    MaxClockMHz(unsigned int rateMHz )   {  MaxClock_1 = rateMHz;  }

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(3.3f);
    DcOut->RipplePkPk(0.5f);
//        void          Standby(bool state);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)   {  NominalV_2   = To10mV(v);  }
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          RipplePkPk(float v) {  RipplePkPk_2 = To1mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(3.3f);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)        {  NominalV_2   = To10mV(v);  }
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcAdc20::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcAdc20::LocalConstructCustomParts()
{
    FmcAdcBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  FmcAdc20::OpenHardware()
//---------------------------------------------------------------------------

void  FmcAdc20::OpenHardware()
{
    FmcAdcBoard::OpenHardware();

    AFEClock_SDev.OpenHardware();

}

//---------------------------------------------------------------------------
//  FmcAdc20::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcAdc20::PreconfigureHardware()
{
    FmcAdcBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  FmcAdc20::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcAdc20::ConfigureHardware()
{
    FmcAdcBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();

    //
    //  Special Config for A/D Devices
    Input().Adc_OnConfigure();
    //
    //  Adc Calibration
    AFEAdcCalCtrl_SDev.StartCalibration();
    Sleep(1);
    unsigned int results = AFEAdcCalCtrl_SDev.CalibrationResults();

    Debug.Log("FmcAdc20 Adc Cal Results: ", results);

}

//---------------------------------------------------------------------------
//  FmcAdc20::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcAdc20::StreamStopHardware()
{
    FmcAdcBoard::StreamStopHardware();

    AFEClock_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  FmcAdc20::GetVitaInStreamCount()
{
    return Fmc_Adc20_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  FmcAdc20::GetVitaOutStreamCount()
{
    return Fmc_Adc20_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  FmcAdc20::GetAdcSpiDeviceCount()
{
    return Fmc_Adc20_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  FmcAdc20::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  FmcAdc20::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new FmcStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  FmcAdc20::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  FmcAdc20::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new FmcStandardPriClockDevice(this, 2);
}


//---------------------------------------------------------------------------
//  FmcAdc20::MakeFmcAfePllObject() -- Get Pri Device object
//---------------------------------------------------------------------------

Cdce62005 *  FmcAdc20::MakeFmcAfePllObject()
{
    // return same object for both directions
    return new FmcAdc20_Pll(this, WB_FMC_1, FmcAdc20_PllRegisters );
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  FmcAdc20::GetAdcCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  FmcAdc20::GetCalibrationAdcCount()
{
    return Fmc_Adc20_Channels;
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  FmcAdc20::GetAnalogDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = ChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  FmcAdc20::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  FmcAdc20::GetAnalogPhysicalDeviceInfo(FrameDeviceCode /*which*/,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    devices = MaxInDevices;
    ch_per_dev = ChannelsPerDevice;
}

//---------------------------------------------------------------------------
//  FmcAdc20::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void FmcAdc20::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = maxFmcVadjCode = 0x4;  // 2.5V only
}

//---------------------------------------------------------------------------
//  FmcAdc20::Input() -- Reference to input object
//---------------------------------------------------------------------------

FmcAdc20InputDevice & FmcAdc20::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  FmcAdc20::AfterFiclStart() --
//---------------------------------------------------------------------------

void  FmcAdc20::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant AdcSpi0  101 constant AdcSpi1  102 constant AdcSpi2");
    engine.Evaluate("  103 constant AdcSpi3  104 constant AdcSpi4  105 constant Pll ");
    engine.Evaluate("  106 constant TempI2C  107 constant IdromI2C   ");
    engine.Evaluate("  108 constant Vga0I2C  109 constant Vga1I2C  110 constant Vga2I2C   ");
    engine.Evaluate("  111 constant Vga3I2C  112 constant Vga4I2C  113 constant VgaIdromI2C   ");

    engine.Evaluate(": ?FmcAdc20 "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   AdcSpi0  AdcSpi1  AdcSpi2  AdcSpi3  AdcSpi4  Pll  \" "
                    "    cr .\"   TempI2C  IdromI2C   \" "
                    "    cr .\"   Vga0I2C  Vga1I2C  Vga2I2C  Vga3I2C  Vga4I2C  VgaIdromI2C   \" "
                    "    cr  "
                    "    cr .\"   ?FmcAdc20  : display this message \" "
                    "    cr  ; ");

    engine.Display("FmcAdc20-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  FmcAdc20::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  FmcAdc20::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;   const int kAdcSpi2 = 102;
    const int kAdcSpi3 = 103;     const int kAdcSpi4 = 104;   const int kPll = 105;
    const int kTempI2C = 106;     const int kIdromI2C = 107;
    const int kVga0I2C = 108;     const int kVga1I2C = 109;   const int kVga2I2C = 110;
    const int kVga3I2C = 111;     const int kVga4I2C = 112;   const int kVgaIdromI2C = 113;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:    case kAdcSpi1:   case kAdcSpi2:
        case kAdcSpi3:    case kAdcSpi4:
            return AFEAdcSpi_SDev.Spi(dev-100, addr);
        case kPll:
            return AFEClock_SDev.Pll().Port(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kVga0I2C:    case kVga1I2C:   case kVga2I2C:
        case kVga3I2C:    case kVga4I2C:
            return 0;
        case kVgaIdromI2C:
            return VgaI2CBus_SDev.VgaIdrom().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  FmcAdc20::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  FmcAdc20::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kAdcSpi0 = 100;     const int kAdcSpi1 = 101;   const int kAdcSpi2 = 102;
    const int kAdcSpi3 = 103;     const int kAdcSpi4 = 104;   const int kPll = 105;
    const int kTempI2C = 106;     const int kIdromI2C = 107;
    const int kVga0I2C = 108;     const int kVga1I2C = 109;   const int kVga2I2C = 110;
    const int kVga3I2C = 111;     const int kVga4I2C = 112;   const int kVgaIdromI2C = 113;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:    case kAdcSpi1:   case kAdcSpi2:
        case kAdcSpi3:    case kAdcSpi4:
            AFEAdcSpi_SDev.Spi(dev-100, addr, value);
            break;
        case kPll:
            AFEClock_SDev.Pll().Port(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kVga0I2C:    case kVga1I2C:   case kVga2I2C:
        case kVga3I2C:    case kVga4I2C:
            VgaI2CBus_SDev.VgaDev().SendCommand((dev-108), addr>>4, addr&0xF, value);
            break;
        case kVgaIdromI2C:
            VgaI2CBus_SDev.VgaIdrom().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  FmcAdc20::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  FmcAdc20::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 105;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcAdc20::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  FmcAdc20::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 105;

    switch (dev)
        {
        default:
        case kPll:
            return static_cast<float>(AFEClock_SDev.Pll().FrequencyActual());
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
