// Fmc_Adc20_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_Adc20_MbH
#define Fmc_Adc20_MbH

#include <FmcBoard_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FmcAdc20;

//===========================================================================
//  CLASS ISuppliesAdded_Adc20_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_Adc20_Requirements : public IRequires_StdFmcClocks
{
public:
    virtual ~ISuppliesAdded_Adc20_Requirements()
        {}

};

//==============================================================================
//  CLASS FmcAdc20_Pll -- Fmc PLL Chip (Specialization of Cdce62005)
//==============================================================================

class FmcAdc20_Pll : public Cdce62005
{
public:
    FmcAdc20_Pll(FmcAdc20 * owner, IAddressingSpace &space, Cdce62005Registers & reg);
    virtual ~FmcAdc20_Pll();

    // ...Cdce62010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          PhaseDetectorFrequency() const
                                {  return 100e3;   }

protected:
    FmcAdc20 *         Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//===========================================================================
//  CLASS FmcAdc20InputDevice  -- FMC Adc20 Input Devices
//===========================================================================

class FmcAdc20InputDevice : public FmcInputDevice_T1
{
public:
    typedef FmcInputDevice_T1 inherited;

    FmcAdc20InputDevice(FmcAdc20 * owner, IX6AnalogIoInfo * info );
    virtual ~FmcAdc20InputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual AdvCalibrationSection & Cal() {  return CalRomSection;  }

    //virtual unsigned int       Channels();

    void   AdcDeviceTestMode(bool enabled, unsigned int test_pattern);

    void   Adc_OnConfigure();

private:
    IX6AnalogIoInfo *       FInfo;
    AdvCalibrationSection   CalRomSection;

    bool          AdcDeviceTestEnabled;
    unsigned int  CurrentAdcDeviceTestPattern;


};

//===========================================================================
//  CLASS FmcAdc20  -- Fmc Daughter card Common baseboard class
//===========================================================================

class FmcAdc20 : public FmcAdcBoard_T1, public ISuppliesAdded_Adc20_Requirements
{
public:
    FmcAdc20();
    virtual ~FmcAdc20()
        {}

    FmcAdc20InputDevice  &    Input();

    IFmcClockIo &             Clock()           {  return AFEClock_SDev.Clock();  }
    IFmcRawClockDevices &     RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Suupplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual Cdce62005 *                 MakeFmcAfePllObject();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);


    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    FmcAFE_Clock      AFEClock_SDev;

    FmcAdc20InputDevice  *    FInput;

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
