// Fmc_Dac40_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_Dac40_MbH
#define Fmc_Dac40_MbH

#include <FmcBoard_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FmcDac40;

//===========================================================================
//  CLASS ISuppliesAdded_Dac40_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_Dac40_Requirements : public IRequires_StdFmcClocks
{
public:
    virtual ~ISuppliesAdded_Dac40_Requirements()
        {}

};

//==============================================================================
//  CLASS FmcDac40_Pll -- Fmc PLL Chip (Specialization of Cdce62005)
//==============================================================================

class FmcDac40_Pll : public Cdce62005
{
public:
    FmcDac40_Pll(FmcDac40 * owner, IAddressingSpace &space, Cdce62005Registers & reg);
    virtual ~FmcDac40_Pll();

    virtual void    OutputFrequency(unsigned int output, double freq);
    virtual double  OutputFrequency(unsigned int output) const;

    // ...Cdce62010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          PhaseDetectorFrequency() const
                                {  return 100e3;   }

protected:
    FmcDac40 *         Owner;
};

//===========================================================================
//  CLASS FmcDac40OutputDevice  -- FMC Dac40 Output Devices
//===========================================================================

class FmcDac40OutputDevice : public FmcOutputDevice
{
public:
    typedef FmcOutputDevice inherited;

    FmcDac40OutputDevice(FmcDac40 * owner, IX6AnalogIoInfo * info );
    virtual ~FmcDac40OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    //virtual AdvCalibrationSection & Cal() {  return CalRomSection;  }

    //virtual unsigned int       Channels();

    //void   AdcDeviceTestMode(bool enabled, unsigned int test_pattern);

    //void   Adc_OnConfigure();

private:
    IX6AnalogIoInfo *       FInfo;
    //AdvCalibrationSection   CalRomSection;

    //bool          AdcDeviceTestEnabled;
    //unsigned int  CurrentAdcDeviceTestPattern;


};

//===========================================================================
//  CLASS FmcDac40  -- Fmc Daughter card Common baseboard class
//===========================================================================

class FmcDac40 : public FmcDacBoard, public ISuppliesAdded_Dac40_Requirements
{
public:
    FmcDac40();
    virtual ~FmcDac40()
        {}

    FmcDac40OutputDevice  &   Output();

    IFmcClockIo &             Clock()           {  return AFEClock_SDev.Clock();  }
    IFmcRawClockDevices &     RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Supplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
//    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
//    virtual unsigned int                GetCalibrationAdcCount();
//    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual Cdce62005 *                 MakeFmcAfePllObject();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                               unsigned int & devices, unsigned int & ch_per_dev);


    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    FmcAFE_Clock      AFEClock_SDev;

    FmcDac40OutputDevice  *    FOutput;

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
