//---------------------------------------------------------------------------
// Fmc_Dio_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_Dio_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const unsigned int Fmc_Dio_VitaInStreamCount   = 0;    // Number of VITA Incoming SIDs
const unsigned int Fmc_Dio_VitaOutStreamCount   = 0;   // Number of VITA Outgoing SIDs

//===========================================================================
//  CLASS FmcDio  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcDio
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

// FMC LA DIO register (WB Device 13)
const unsigned int  mmLaDio_DataLo          = 0x00; // low 32 bits
const unsigned int  mmLaDio_DataMid         = 0x01; // middle 32 bits
const unsigned int  mmLaDio_DataHi          = 0x02; // upper 2 bits
const unsigned int  mmLaDio_OutEnableLo     = 0x04; // low 32 bits
const unsigned int  mmLaDio_OutEnableMid    = 0x05; // middle 32 bits
const unsigned int  mmLaDio_OutEnableHi     = 0x06; // upper 2 bits

// FMC LA DIO registers (WB Device 13)
FmcLaDigitalIo::FmcLaDioMapRegisters LaDioRegAddrs =
    {
    mmLaDio_DataLo,        //  DioDataLo_Addr
    mmLaDio_DataMid,       //  DioDataMid_Addr
    mmLaDio_DataHi,        //  DioDataHi_Addr
    mmLaDio_OutEnableLo,   //  DioOutputEnableLo_Addr
    mmLaDio_OutEnableMid,  //  DioOutputEnableMid_Addr
    mmLaDio_OutEnableHi,   //  DioOutputEnableHi_Addr
    };

X6DigitalIo::MapRegisters HaDioRegAddrs =
    {
    mmDio_DataLo,          //  DioDataLo_Addr
    mmDio_DataHi,          //  DioDataHi_Addr
    mmDio_OutEnableLo,     //  DioOutputEnableLo_Addr
    mmDio_OutEnableHi,     //  DioOutputEnableHi_Addr
    };

X6DigitalIo::MapRegisters HbDioRegAddrs =
    {
    mmDio_DataLo,          //  DioDataLo_Addr
    mmDio_DataHi,          //  DioDataHi_Addr
    mmDio_OutEnableLo,     //  DioOutputEnableLo_Addr
    mmDio_OutEnableHi,     //  DioOutputEnableHi_Addr
    };

//===========================================================================
//  CLASS FmcLaDigitalIo -- X6 Digital Io "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FmcLaDigitalIo
//---------------------------------------------------------------------------

FmcLaDigitalIo::FmcLaDigitalIo(IRequires_Nothing * mdc, IAddressingSpace & space, FmcLaDioMapRegisters & regs)
    :  MDC(mdc),
       DataLo(space, regs.DioDataLo_Addr),
       DataMid(space, regs.DioDataMid_Addr),
       DataHi(space, regs.DioDataHi_Addr),
       OutEnLo(space, regs.DioOutputEnableLo_Addr),
       OutEnMid(space, regs.DioOutputEnableMid_Addr),
       OutEnHi(space, regs.DioOutputEnableHi_Addr),
       DataLoManip(DataLo),
       DataMidManip(DataMid),
       DataHiManip(DataHi)
{

}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::OpenHardware()
//---------------------------------------------------------------------------

void  FmcLaDigitalIo::OpenHardware()
{
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcLaDigitalIo::ConfigureHardware()
{
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortData() -- access to Lo Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  FmcLaDigitalIo::DioPortDataLow()
{
    return DataLoManip;
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

void  FmcLaDigitalIo::DioPortConfigLow(unsigned int mask)
{
    OutEnLo.Value(mask);
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

unsigned int  FmcLaDigitalIo::DioPortConfigLow() const
{
    return OutEnLo.Value();
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortDataMiddle() -- access to Middle Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  FmcLaDigitalIo::DioPortDataMiddle()
{
    return DataMidManip;
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortConfigMiddle() -- read/change Middle OE config mask
//---------------------------------------------------------------------------

void  FmcLaDigitalIo::DioPortConfigMiddle(unsigned int mask)
{
    OutEnMid.Value(mask);
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortConfigMiddle() -- read/change Middle OE config mask
//---------------------------------------------------------------------------

unsigned int  FmcLaDigitalIo::DioPortConfigMiddle() const
{
    return OutEnMid.Value();
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortDataHigh() -- access to Hi Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  FmcLaDigitalIo::DioPortDataHigh()
{
    return DataHiManip;
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortConfigHigh() -- read/change High OE config mask
//---------------------------------------------------------------------------

void  FmcLaDigitalIo::DioPortConfigHigh(unsigned int mask)
{
    OutEnHi.Value(mask);
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortConfigHigh() -- read/change High OE config mask
//---------------------------------------------------------------------------

unsigned int  FmcLaDigitalIo::DioPortConfigHigh() const
{
    return OutEnHi.Value();
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

void  FmcLaDigitalIo::DioPortOutputEnable(int output_bit, bool output_enabled)
{
    if (output_bit<32)
        OutEnLo.Bit(output_bit, output_enabled);
    else if (output_bit<64)
        OutEnMid.Bit(output_bit-32, output_enabled);
    else
        OutEnHi.Bit(output_bit-64, output_enabled);
}

//---------------------------------------------------------------------------
//  FmcLaDigitalIo::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

bool  FmcLaDigitalIo::DioPortOutputEnable(int output_bit) const
{
    if (output_bit<32)
        return OutEnLo.Bit(output_bit);
    else if (output_bit<64)
        return OutEnMid.Bit(output_bit-32);
    else
        return OutEnHi.Bit(output_bit-64);
}

//---------------------------------------------------------------------------
//  constructor for class FmcDio
//---------------------------------------------------------------------------

FmcDio::FmcDio()
    :   FmcBoard(fmcPrimary),
        LA_Dio_SDev(this, WB_FMC_1, LaDioRegAddrs),
        HA_Dio_SDev(this, WB_FMC_2, HaDioRegAddrs),
        HB_Dio_SDev(this, WB_FMC_3, HbDioRegAddrs)
{
    LocalConstructCustomParts();
}

//int xxx()
//{
//    FmcDio * wombat = new FmcDio;
//}

//---------------------------------------------------------------------------
//  FmcDio::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  FmcDio::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("FmcDio");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(100);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 20);
//        void    ModuleSize(Fmc::FmcModuleSize size);
//        void    ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size);
//        void    ClockDirection(Fmc::FmcClockDirection dir);
//        void    BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank, unsigned int count);
//        void    TransceiverCount(Fmc::FmcConnector p, unsigned int count);
//        void    MaxClockMHz(unsigned int rateMHz )   {  MaxClock_1 = rateMHz;  }

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(3.3f);
    DcOut->RipplePkPk(0.5f);
//        void          Standby(bool state);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)   {  NominalV_2   = To10mV(v);  }
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          RipplePkPk(float v) {  RipplePkPk_2 = To1mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(3.3f);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)        {  NominalV_2   = To10mV(v);  }
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcDio::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcDio::LocalConstructCustomParts()
{
    FmcBoard::ConstructCustomParts();

    LA_Dio_SDev.ConstructCustomParts();
    HA_Dio_SDev.ConstructCustomParts();
    HB_Dio_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  FmcDio::OpenHardware()
//---------------------------------------------------------------------------

void  FmcDio::OpenHardware()
{
    FmcBoard::OpenHardware();
    
    LA_Dio_SDev.OpenHardware();
    HA_Dio_SDev.OpenHardware();
    HB_Dio_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  FmcDio::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcDio::PreconfigureHardware()
{
    FmcBoard::PreconfigureHardware();
    // Subdevice Configuration
    LA_Dio_SDev.PreconfigureHardware();
    HA_Dio_SDev.PreconfigureHardware();
    HB_Dio_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  FmcDio::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcDio::ConfigureHardware()
{
    FmcBoard::ConfigureHardware();
    LA_Dio_SDev.ConfigureHardware();
    HA_Dio_SDev.ConfigureHardware();
    HB_Dio_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  FmcDio::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcDio::StreamStopHardware()
{
    FmcBoard::StreamStopHardware();
}

//---------------------------------------------------------------------------
//  FmcDio::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  FmcDio::GetVitaInStreamCount()
{
    return Fmc_Dio_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  FmcDio::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  FmcDio::GetVitaOutStreamCount()
{
    return Fmc_Dio_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  FmcDio::AfterFiclStart() --
//---------------------------------------------------------------------------

void  FmcDio::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant LA_Dio  101 constant HA_Dio  102 constant HB_Dio");

    engine.Evaluate(": ?FmcDio "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   LA_Dio HA_Dio  HB_Dio   \" "
                    "    cr  "
                    "    cr .\"   ?FmcDio  : display this message \" "
                    "    cr  ; ");

    engine.Display("FmcDio-specific wordset loaded \n" );
}

//---------------------------------------------------------------------------
//  FmcDio::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  FmcDio::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kLA_Dio = 100;     const int kHA_Dio = 101;   const int kHB_Dio = 102;

    switch (dev)
        {
        default:
            return 0;
        case kFmc:
            return PeekLogic(addr);
        case kLA_Dio:
            return DioReadIo(LA_Dio(), addr);
        case kHA_Dio:
            return DioReadIo(HA_Dio(), addr);
        case kHB_Dio:
            return DioReadIo(HB_Dio(), addr);
        }
}

//---------------------------------------------------------------------------
//  FmcDio::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  FmcDio::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kLA_Dio = 100;     const int kHA_Dio = 101;   const int kHB_Dio = 102;

    switch (dev)
        {
        default:
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        case kLA_Dio:
            DioWriteIo(LA_Dio(), addr, value);
            break;
        case kHA_Dio:
            DioWriteIo(HA_Dio(), addr, value);
            break;
        case kHB_Dio:
            DioWriteIo(HB_Dio(), addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcDio::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  FmcDio::Do_DFreq(int dev, float /*freq*/)
{
    const int kFmc = 0;  // from base board
    //const int kLA_Dio = 100;     const int kHA_Dio = 101;   const int kHB_Dio = 102;

    switch (dev)
        {
        default:
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcDio::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  FmcDio::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    //const int LA_Dio = 100;     const int HA_Dio = 101;   const int HB_Dio = 102;

    switch (dev)
        {
        default:
        case kFmc:      // no frequency setting
            return 0.0;
        }
}

//---------------------------------------------------------------------------
//  FmcDio::DioReadIo() --  default implementation
//---------------------------------------------------------------------------

int  FmcDio::DioReadIo(FmcLaDioPortIntf & aDio, int addr)
{
    if (addr==0)
        return aDio.DioPortDataLow().Value();
    else if (addr==1)
        return aDio.DioPortDataMiddle().Value();
    else if (addr==2)
        return aDio.DioPortDataHigh().Value();
    else if (addr==4)
        return aDio.DioPortConfigLow();
    else if (addr==5)
        return aDio.DioPortConfigMiddle();
    else if (addr==6)
        return aDio.DioPortConfigHigh();
    else
        return 0;
}

int  FmcDio::DioReadIo(AdvancedDioPortIntf & aDio, int addr)
{
    if (addr==0)
        return aDio.DioPortData().Value();
    else if (addr==1)
        return aDio.DioPortDataHigh().Value();
    else if (addr==2)
        return aDio.DioPortConfig();
    else if (addr==3)
        return aDio.DioPortConfigHigh();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  FmcDio::DioWriteIo() --  'Dio' write access
//---------------------------------------------------------------------------

void  FmcDio::DioWriteIo(FmcLaDioPortIntf & aDio, int addr, int value)
{
    if (addr==0)
        aDio.DioPortDataLow().Value(value);
    else if (addr==1)
        aDio.DioPortDataMiddle().Value(value);
    else if (addr==2)
        aDio.DioPortDataHigh().Value(value);
    else if (addr==4)
        aDio.DioPortConfigLow(value);
    else if (addr==5)
        aDio.DioPortConfigMiddle(value);
    else if (addr==6)
        aDio.DioPortConfigHigh(value);
}

void  FmcDio::DioWriteIo(AdvancedDioPortIntf & aDio, int addr, int value)
{
    if (addr==0)
        aDio.DioPortData().Value(value);
    else if (addr==1)
        aDio.DioPortDataHigh().Value(value);
    else if (addr==2)
        aDio.DioPortConfig(value);
    else if (addr==3)
        aDio.DioPortConfigHigh(value);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
