// FmcDio_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_Dio_MbH
#define Fmc_Dio_MbH

#include <FmcBoard_Mb.h>
#include <DioPortIntf_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FmcLaDigitalIo : public IX6SubDevice, public FmcLaDioPortIntf
{
public:
    // FMC LA DIO registers (WB Device 13)
    struct FmcLaDioMapRegisters
    {
        unsigned int   DioDataLo_Addr;           // low 32 bits
        unsigned int   DioDataMid_Addr;          // middle 32 bits
        unsigned int   DioDataHi_Addr;           // upper 2 bits
        unsigned int   DioOutputEnableLo_Addr;   // low 32 bits
        unsigned int   DioOutputEnableMid_Addr;  // middle 32 bits
        unsigned int   DioOutputEnableHi_Addr;   // upper 2 bits
    };

    FmcLaDigitalIo(IRequires_Nothing * mdc, IAddressingSpace & space, FmcLaDioMapRegisters & regs);
    virtual ~FmcLaDigitalIo()
    { };

    virtual IBitManipulator &   DioPortDataLow();
    virtual void                DioPortConfigLow(unsigned int);
    virtual unsigned int        DioPortConfigLow() const;

    virtual IBitManipulator &   DioPortDataMiddle();
    virtual void                DioPortConfigMiddle(unsigned int);
    virtual unsigned int        DioPortConfigMiddle() const;

    virtual IBitManipulator &   DioPortDataHigh();
    virtual void                DioPortConfigHigh(unsigned int mask);
    virtual unsigned int        DioPortConfigHigh() const;

    virtual void                DioPortOutputEnable(int output_bit, bool output_enabled);
    virtual bool                DioPortOutputEnable(int output_bit) const;

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_Nothing *  MDC;
    Register  DataLo;
    Register  DataMid;
    Register  DataHi;
    Register  OutEnLo;
    Register  OutEnMid;
    Register  OutEnHi;

    RegisterBitManipulator  DataLoManip;
    RegisterBitManipulator  DataMidManip;
    RegisterBitManipulator  DataHiManip;
};

//===========================================================================
//  CLASS FmcDio  -- Fmc Daughter card Common baseboard class
//===========================================================================

class FmcDio : public FmcBoard
{
public:
    FmcDio();
    virtual ~FmcDio()
        {}

    //===========================================================================
    //  CLASS AdvancedDioPortIntf -- "X6" style with individual bit config
    //===========================================================================

    FmcLaDioPortIntf &      LA_Dio()  {  return LA_Dio_SDev;  }
    AdvancedDioPortIntf &   HA_Dio()  {  return HA_Dio_SDev;  }
    AdvancedDioPortIntf &   HB_Dio()  {  return HB_Dio_SDev;  }

    // pseudo-protected
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Suupplier Methods
    virtual unsigned int GetVitaInStreamCount();
    virtual unsigned int GetVitaOutStreamCount();

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    FmcLaDigitalIo       LA_Dio_SDev;
    X6DigitalIo          HA_Dio_SDev;
    X6DigitalIo          HB_Dio_SDev;

    void LocalConstructCustomParts();
    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    int   DioReadIo(FmcLaDioPortIntf & aDio, int addr);
    void  DioWriteIo(FmcLaDioPortIntf & aDio, int addr, int value);
    int   DioReadIo(AdvancedDioPortIntf & aDio, int addr);
    void  DioWriteIo(AdvancedDioPortIntf & aDio, int addr, int value);
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
