//---------------------------------------------------------------------------
// Fmc_SDF_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_SDF_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition
const unsigned int    MaxInDevices      = 4;
const int             InChannelsPerDevice = 1;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices        = 1;
const int             OutChannelsPerDevice = 2;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;

const unsigned int Fmc_SDF_VitaInStreamCount   = 1;   // Number of VITA Incoming SIDs (actually no VITA packets)
const unsigned int Fmc_SDF_VitaOutStreamCount  = 1;   // Number of VITA Outgoing SIDs (actually no VITA packets)

const unsigned int Fmc_SDF_AdcSpiDeviceCount   = MaxInDevices;   // Number of SPI devices muxed - actually uses custom SPI
const unsigned int Fmc_SDF_DacSpiDeviceCount   = MaxOutDevices;   // Number of SPI devices muxed - actually 0

const unsigned int Fmc_SDF_AdcChannelsPerDevice   = InChannelsPerDevice;
const unsigned int Fmc_SDF_AdcChannels   = MaxInChannels;
const unsigned int Fmc_SDF_DacChannelsPerDevice   = OutChannelsPerDevice;
const unsigned int Fmc_SDF_DacChannels   = MaxOutChannels;

const float         InputMaxRate            = 625.0e3f;
const float         InputMinRate            = 0.0e3f;
const unsigned int  InputWidth_Bits         = 24;
const          int  InputMax_mV             = 5000;
const          int  InputMin_mV             = -5000;
const unsigned int  InputSamplesPerWord     = 1;
const unsigned int  InputTriggerGranularity = 1;
const int           InputClockFactor        = 1;

const unsigned int Fmc_SDF_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_SDF_Adc_One = 0x8000;         // representation of floating point 1.0

const float         OutputMaxRate            = 400.0e3f;
const float         OutputMinRate            = 0.0e3f;
const unsigned int  OutputWidth_Bits         = 24;
const          int  OutputMax_mV             = 5000;
const          int  OutputMin_mV             = -5000;
const unsigned int  OutputSamplesPerWord     = 1;
const unsigned int  OutputTriggerGranularity = 1;
const int           OutputClockFactor        = 1;

const unsigned int Fmc_SDF_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
const unsigned int Fmc_SDF_Dac_One = 0x10000;         // representation of floating point 1.0

//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_SDF_InputCoeffOffset       = 0;
const int romFmc_SDF_InputCoeffOffset_avail = 9;   // 4 * 2 coefficients plus 'checksum' word

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_SDF_OutputCoeffOffset       = 0;   // these are in separate "ROMs"
const int romFmc_SDF_OutputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'checksum' word

//===========================================================================
// Class FmcSDF_AnalogInInfo
//===========================================================================

class FmcSDF_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    FmcSDF_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~FmcSDF_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
	virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
// Class FmcSDF_AnalogOutInfo
//===========================================================================

class FmcSDF_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    FmcSDF_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~FmcSDF_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
	virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS FmcSDFInputDevice  -- FmcSDF Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FmcSDFInputDevice
//---------------------------------------------------------------------------

FmcSDFInputDevice::FmcSDFInputDevice(FmcSDF * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerSDF(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_SDF_InputCoeffOffset, MaxInChannels,
                       Fmc_SDF_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcSDFInputDevice
//---------------------------------------------------------------------------

FmcSDFInputDevice::~FmcSDFInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  FmcSDFInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  FmcSDFInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  FmcSDFInputDevice::ActiveDevices() -- Number of DAC devices in use
//---------------------------------------------------------------------------

unsigned int  FmcSDFInputDevice::ActiveDevices()
{
    return Fmc_SDF_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  FmcSDFInputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  FmcSDFInputDevice::OpenHardware()
{
    //CalRomSection.ApplyRomCoefficients();  // needs cal coefficients
}

//---------------------------------------------------------------------------
//  FmcSDFInputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void FmcSDFInputDevice::ConfigureHardware()
{
    OwnerSDF->Debug.Log("FmcSDFInputDevice::ConfigureHardware()");

    CalRomSection.ApplyRomCoefficients();

//    if (ChannelPowered(0)==false && ChannelPowered(1)==false)
//        {
//        OwnerSDF->Debug.Log("ADC Power off, config skipped.");
//        return;   // no need to go on, power is off.
//        }


    OwnerSDF->Debug.Log("FmcSDFInputDevice::ConfigureHardware()");

}

//---------------------------------------------------------------------------
//  FmcSDFInputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  FmcSDFInputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    AdcSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  FmcSDFInputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  FmcSDFInputDevice::Spi(unsigned int device, unsigned int address)
{
    return AdcSpi().Spi(device, address);
}

//===========================================================================
//  CLASS FmcSDFOutputDevice  -- FmcSDF Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcSDFOutputDevice
//---------------------------------------------------------------------------

//const bool DevIs5681 = true;

//---------------------------------------------------------------------------
//  constructor for class FmcSDFOutputDevice
//---------------------------------------------------------------------------

FmcSDFOutputDevice::FmcSDFOutputDevice(FmcSDF * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerSDF(owner), FInfo(info),
      //DacDev(new FmcSDF_DacToHardware(this, 0)),
      CalRomSection(DacCalIdrom(), romFmc_SDF_OutputCoeffOffset, MaxOutChannels,
                       Fmc_SDF_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcSDFOutputDevice
//---------------------------------------------------------------------------

FmcSDFOutputDevice::~FmcSDFOutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  FmcSDFOutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  FmcSDFOutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  FmcSDFOutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  FmcSDFOutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  FmcSDFOutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  FmcSDFOutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  FmcSDFOutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  FmcSDFOutputDevice::ConfigureHardware()
{
    DacCommon().DacReset(true);
    Sleep(1);
    DacCommon().DacReset(false);
}

//---------------------------------------------------------------------------
//  FmcSDFOutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  FmcSDFOutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    //Device().Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1) );
//    Device().ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
//    Device().ChannelEnabled(1, DacCommon().ChannelEnabled(1) );

}

//=============================================================================
//  FmcSDF Timebase Definitions  --  PLL timebase control
//=============================================================================

const int Ad9510_Tb_Instance = 0;
const int Ad9508_Tb_Instance = 1;

//---------------------------------------------------------------------------
//  housekeeping for class FmcSDF_Ad9510_Tb
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

static const double _VcoLow = 110.0e6;
static const double _VcoHigh = 150.0e6;


//---------------------------------------------------------------------------
//  constructor for class FmcSDF_Ad9510_Tb
//---------------------------------------------------------------------------

FmcSDF_Ad9510_Tb::FmcSDF_Ad9510_Tb(FmcSDF * owner, IAddressingSpace &space, MapRegisters& reg)
    :  Ad9510_Tb(space, reg, Ad9510_Tb_Instance), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);

    VcoRange(_VcoLow, _VcoHigh);  // non-standard VCO range
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::OnInit() --
//---------------------------------------------------------------------------

void  FmcSDF_Ad9510_Tb::OnInit()
{
    //
    //  Set Source Configuration (here same as default)
    SourceInUse(0, true);    SetSourceMapping(0, IX6ClockIo::rsExternal);   // primary = external
    SourceInUse(1, true);    SetSourceMapping(1, IX6ClockIo::rsInternal);   // secondary = internal

    //  Set Output Configuration
    OutputInUse(0,  false);   OutputEnabled(0, false);   // LVPECL
    OutputInUse(1,  false);   OutputEnabled(1, false);   // LVPECL
    OutputInUse(2,  false);   OutputEnabled(2, false);   // LVPECL
    OutputInUse(3,  false);   OutputEnabled(3, false);   // LVPECL
    OutputInUse(4,  true);    OutputEnabled(4, true);   SetOutputConfig(4,  ocLvCmos); // ADC0-4 MCLK
    OutputInUse(5,  false);   OutputEnabled(5, false);  SetOutputConfig(5,  ocLvds); // unused
    OutputInUse(6,  true);    OutputEnabled(6, true);   SetOutputConfig(6,  ocLvds); // input to 9508
    OutputInUse(7,  false);   OutputEnabled(7, false);  SetOutputConfig(7,  ocLvds); // unused

    //  Set Reduced Frequency Limits for MCLK Outputs
    //  - can not exceed 20 MHz
    //OutputFrequencyLimit[4] = 20.0e6;

}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::OnReapply()
//---------------------------------------------------------------------------

void  FmcSDF_Ad9510_Tb::OnReapply()
{

}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::AdcMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcSDF_Ad9510_Tb::AdcMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::DacMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcSDF_Ad9510_Tb::DacMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::AdcMultiplierFactor()
//---------------------------------------------------------------------------

unsigned int FmcSDF_Ad9510_Tb::AdcMultiplierFactor()
{
    return InputClockFactor;
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::DacMultiplierFactor()
//---------------------------------------------------------------------------

unsigned int FmcSDF_Ad9510_Tb::DacMultiplierFactor()
{
    return OutputClockFactor;
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::SetAdcOutputRates()
//---------------------------------------------------------------------------

void FmcSDF_Ad9510_Tb::SetAdcOutputRates(double adc_rate)
{
    double SampleFrequency = adc_rate;

    OutputFrequency(4, 0.0);  // same as master, but capped
    OutputFrequency(6, SampleFrequency);
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9510_Tb::SetDacOutputRates()
//---------------------------------------------------------------------------

void FmcSDF_Ad9510_Tb::SetDacOutputRates(double /*dac_rate*/)
{
    // none
}

//=============================================================================
//  CLASS FmcSDF_Ad9508_Tb --  FmcServo custom Ad9508 (SLAVE)
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcSDF_Ad9508_Tb
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class FmcSDF_Ad9508_Tb
//---------------------------------------------------------------------------

FmcSDF_Ad9508_Tb::FmcSDF_Ad9508_Tb(FmcSDF * owner, IAddressingSpace &space, MapRegisters& reg)
   :  Ad9508_Tb(space, reg, Ad9508_Tb_Instance), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9508_Tb::OnInit()
//---------------------------------------------------------------------------

void  FmcSDF_Ad9508_Tb::OnInit()
{
   //  Set Output Configuration
   OutputInUse(0,  true);   OutputEnabled(0, true);   SetOutputConfig(0,  ocLvds);  //  CLK0_M2C (Adc System Reference)
   OutputInUse(1,  true);   OutputEnabled(1, true);   SetOutputConfig(1,  ocLvds);  //  CLK1_M2C (Dac System Reference)
   OutputInUse(2,  true);   OutputEnabled(2, true);   SetOutputConfig(2,  ocCmos);  //  DAC_LDAC   
   OutputInUse(3,  false);  OutputEnabled(3, false);  SetOutputConfig(3,  ocLvds);  //  Unused

   //  must call for all CMOS outputs because default is garbage.
   SetCmosOutputConfig(2, true, odpNormal, false, odpNormal);   // P normal, N unused

   SetOutputSync(0, true);
   SetOutputSync(1, true);
   SetOutputSync(2, true);
   SetOutputSync(3, true);

}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9508_Tb::OnReapply()
//---------------------------------------------------------------------------

void  FmcSDF_Ad9508_Tb::OnReapply()
{
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9508_Tb::AdcMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcSDF_Ad9508_Tb::AdcMasterOutput()
{
   return 0;
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9508_Tb::DacMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcSDF_Ad9508_Tb::DacMasterOutput()
{
   return 2;
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9508_Tb::SetAdcOutputRates()
//---------------------------------------------------------------------------

void FmcSDF_Ad9508_Tb::SetAdcOutputRates(double adc_rate)
{
   double SampleFrequency = adc_rate;

   OutputFrequency(0, SampleFrequency);
}

//---------------------------------------------------------------------------
//  FmcSDF_Ad9508_Tb::SetDacOutputRates()
//---------------------------------------------------------------------------

void FmcSDF_Ad9508_Tb::SetDacOutputRates(double dac_rate)
{
   double SampleFrequency = dac_rate;
   OutputFrequency(1, SampleFrequency);
   OutputFrequency(2, SampleFrequency);
}

//=============================================================================
//  CLASS FmcSDF_AdcCalOverride  --  Cal Override for FmcSDF Adcs
//=============================================================================

class FmcSDF_AdcCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    FmcSDF_AdcCalOverride(FmcSDF *  owner) : Owner(owner)
        {}
    virtual ~FmcSDF_AdcCalOverride()
        {}

private:
    FmcSDF *  Owner;
};

//---------------------------------------------------------------------------
//  FmcSDF_AdcCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  FmcSDF_AdcCalOverride::OverrideOffset(unsigned int value)
{
    return value;
}

//---------------------------------------------------------------------------
//  FmcSDF_AdcCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  FmcSDF_AdcCalOverride::OverrideGain(unsigned int value)
{
    return value;
}

//---------------------------------------------------------------------------
//  FmcSDF_AdcCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  FmcSDF_AdcCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    //  Output correction range is just 16 bits signed.  First convert data
    //    to 32 bit signed
    int signed_ofs = static_cast<int>(value);

    // trim dac offset to 16 bit range
    int adc_offset = std::max(-65535, std::min(65535, signed_ofs));

    // write to ADC SPI offset register
    Owner->Input().Spi(ch, 0x3, adc_offset);

}

//---------------------------------------------------------------------------
//  FmcSDF_AdcCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  FmcSDF_AdcCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    // data has already been converted to "0x8000 = gain 1" format
    unsigned int adc_gain = value;

    // write to ADC SPI gain register
    Owner->Input().Spi(ch, 0x4, adc_gain);

    //  Device recommends we run at nominal 1.25 gain normally and possibly later we
    //  would want to multiply our calibrated gain with 1.25 to produce the final gain value


    //    // better to do this all in the "write" call...
    //    const float HdwStep_mA    = 0.0225f;    // 22.5 uA steps
    //    const float Min_mA        = 8.64f;      // 0x000 gain value
    //    const float Center_mA     = 20.00f;     // 0x1f9 gain value


    //    float gain = static_cast<float>(value)/Fmc_SDF_Adc_One;

    //    float adjusted_mA = gain*Center_mA;
    //    float steps_mA = adjusted_mA - Min_mA;

    //    float counts = steps_mA / HdwStep_mA;
    //    unsigned int setting;
    //    if (counts < 0)
    //        setting = 0;
    //    else if (counts > 1023)
    //        setting = 0x3FF;
    //    else
    //        setting = static_cast<unsigned int>(counts);

    //    return setting;

}

//===========================================================================
//  CLASS FmcSDF  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcSDF
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{

FmcAd9510Duo_Clock::MapRegisters FmcAd9510DuoClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

Ad9510_Tb::MapRegisters  FmcSDF_MasterPllRegisters =
    {
    mmFmc_PllControl,    //  PllControl_Addr;
    mmFmc_PllSpi,        //  PllSpi_Addr
    };

Ad9508_Tb::MapRegisters  FmcSDF_SlavePllRegisters =
    {
    mmFmc_PllControl,    //  PllControl_Addr;
    mmFmc_PllSpi,        //  PllSpi_Addr
    };

AD7763_SpiDevice::MapRegisters FmcSdf_AdcSpiRegAddrs =
    {
    mmFmc_AdcSpiEnable,      //  Device_Addr;
    mmFmc_AdcSpiCtrl,        //  Write_Addr;
    mmFmc_AdcSpiStatus,      //  Read_Addr;
    };

//  FMCSDF memory Map
//
//  WB_FMC2 - FMC ADC AFE
//const unsigned int  mmFmcSDF_AdcPHYCal       = 0x14;
//const unsigned int  mmFmcSDF_AdcPHYCalStatus = 0x15;


//bool IsInFmcSDFMode = true;


}

//---------------------------------------------------------------------------
//  constructor for class FmcSDF
//---------------------------------------------------------------------------

FmcSDF::FmcSDF()
    :  FmcBidirBoard(fmcPrimary, false),  // not in FMC500 mode for adc SPI
       AFEClock_SDev(this, WB_FMC_1, FmcAd9510DuoClockRegAddrs),
       Local_AdcSpi_SDev(this, WB_FMC_2, FmcSdf_AdcSpiRegAddrs),
       AdcCalIDROM_SDev(this, RomInfo(), true),   // true = ADC Rom
       DacCalIDROM_SDev(this, RomInfo(), false),  // true = ADC Rom
       AFEAdc_ResetReg(WB_FMC_2, mmFmcSDF_Adc_Reset),
       TachometerReg(WB_FMC_1, mmFmcSDF_Tachometer),
       FInput(new FmcSDFInputDevice(this, new FmcSDF_AnalogInInfo)),
       FOutput(new FmcSDFOutputDevice(this, new FmcSDF_AnalogOutInfo))//,
{
    AFEClock_SDev.Pll().PostDividerStyle( Ad951xAutoTune::pdsAd9508_SDF );  // restrict 9508 postdivider
    AFEClock_SDev.Pll().MasterPll_MaxOutput( 20.0e6 );  // Master PLL output no larger than this

    AFEAdcSpi_SDev.DeviceEnabled(false);  // Specialized SPI used
    LocalConstructCustomParts();
    }


FmcSDF * anSDF()
{
    FmcSDF * wombat = new FmcSDF;
    return wombat;
}

//---------------------------------------------------------------------------
//  FmcSDF::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  FmcSDF::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  Header Section
    RI.Header().Version(0x2);

    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("FmcSDF");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(500);
    RI.FmcDef().ModuleSize(Fmc::msSingleWide);
    RI.FmcDef().ClockDirection(Fmc::dirCarrierToMezz);
    RI.FmcDef().ConnectorSize(Fmc::fmcP1, Fmc::csHPC);
    RI.FmcDef().ConnectorSize(Fmc::fmcP2, Fmc::csNotFitted);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankA, 87);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankB, 30);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 0);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankB, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP1, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP2, 0);

    //  ADC CALIBRATION SECTION
    RI.AdcCal().Resize(romFmc_SDF_InputCoeffOffset_avail);
    RI.AdcCal().Zero();

    //  DAC CALIBRATION SECTION
    RI.DacCal().Resize(romFmc_SDF_OutputCoeffOffset_avail);
    RI.DacCal().Zero();

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(2.5f);
    DcOut->RipplePkPk(0.5f);
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(false);
    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    //        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
    //        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
    //        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(2.5f);
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcSDF::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcSDF::LocalConstructCustomParts()
{
    FmcBidirBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    AdcCalIDROM_SDev.ConstructCustomParts();
    DacCalIDROM_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  FmcSDF::OpenHardware()
//---------------------------------------------------------------------------

void  FmcSDF::OpenHardware()
{
//    PokeLogic(0x0,   0x00000);  // offsets relative to FMC

    FmcBidirBoard::OpenHardware();

    Debug.Log("++++++++++++++++++++++++++++++++++++");
    Debug.Log("  Load Adc/Dac Calibration Shadows");
    AdcCalIDROM_SDev.OpenHardware();
    DacCalIDROM_SDev.OpenHardware();
    Debug.Log("++++++++++++++++++++++++++++++++++++");

    //  Turn on VCO Power
    //AFEAdcVcoRegister.Bit(4, true);

//    PokeLogic(0x202, 0x3);
//    unsigned int data = PeekLogic(0x0);
//    Debug.Log("Power Register Final: ", data, 16);

    AFEClock_SDev.OpenHardware();

    Input().OpenHardware();
    Output().OpenHardware();

}

//---------------------------------------------------------------------------
//  FmcSDF::Init_ADC_Devices() - Initialize AD7763 Device
//---------------------------------------------------------------------------

void  FmcSDF::Init_ADC_Devices()
{
    Local_AdcSpi_SDev.SetSpiClockPeriod( Clock().Adc().Frequency() );

    Reset_AD7763();

    for (unsigned int ch=0; ch<Fmc_SDF_AdcSpiDeviceCount; ch++)
        {
        if (Input().ChannelEnabled(ch)==false)
            continue;   // skip disabled channels

        Input().Spi(ch, 0x2, 0x2);
        //  We want to make sure the ADC is ready again
        Local_AdcSpi_SDev.SpiReady();
        }

    Sync_AD7763();
}

//---------------------------------------------------------------------------
//  FmcSDF::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcSDF::PreconfigureHardware()
{
    //Output().ConfigureChannelEnables();

    FmcBidirBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();

    //Sleep(1);  // give time for clock to settle ?

    Init_ADC_Devices();

    Configure_ADC_ClockFactor();

    //
    //  Special Config for D/A Devices
    Output().PreconfigureHardware();
}


//---------------------------------------------------------------------------
//  FmcSDF::Configure_ADC_ClockFactor() - Set up ADC Clock Factor from
//---------------------------------------------------------------------------

void  FmcSDF::Configure_ADC_ClockFactor()
{
    unsigned int divider_idx =  AFEClock_SDev.Pll().Slave()->AdcMasterOutput();
    unsigned int clock_factor = AFEClock_SDev.Pll().Slave()->OutputDivider(divider_idx);

    // AD7763 "Clock Factor" Settings - rate divider
    //   Total   F1    F2    F3
    //     16     4     4     1
    //     32     4     4     2
    //     32     4     8     1
    //     64     4     8     2
    //     64     4    16     1
    //    128     4    16     2
    //    128     4    32     1
    //    256     4    32     2

    enum F2_settings { f2_4x_0, f2_4x_1, f2_4x, f2_8x, f2_16x, f2_32x  };    //  0, 1, 2 = 4x   3 = 8x   4=16x;   5=32x;
    F2_settings F2_setting;
    enum F3_settings { f3_1x, f3_2x  };    //  0 = 1x    1 = 2x
    F3_settings F3_setting;

    switch (clock_factor)
        {
        default:
        case 16:
            F2_setting = f2_4x;    F3_setting = f3_1x;
            break;
        case 32:
            F2_setting = f2_8x;    F3_setting = f3_1x;
            break;
        case 64:
            F2_setting = f2_16x;    F3_setting = f3_1x;
            break;
        case 128:
            F2_setting = f2_32x;    F3_setting = f3_1x;
            break;
        case 256:
            F2_setting = f2_32x;    F3_setting = f3_2x;
            break;
        }

    MemoryBitManipulator Spi_1(0);

    Spi_1.Bit(10, false);
    Spi_1.Bit(4, static_cast<bool>(F3_setting));
    Spi_1.Bit(3, true);
    Spi_1.Field(0, 3, static_cast<int>(F2_setting));

    // write to ADC SPI offset register
    for (unsigned int ch=0;  ch<Fmc_SDF_AdcSpiDeviceCount; ch++)
        {
        if (Input().ChannelEnabled(ch)==false)
            continue;   // skip disabled channels
        Input().Spi(ch, 0x1, Spi_1.Value());
        }

}

//---------------------------------------------------------------------------
//  FmcSDF::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcSDF::ConfigureHardware()
{
    FmcBidirBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    //
    //  Special Config for D/A Devices
    Input().ConfigureHardware();
    Output().ConfigureHardware();

}

//---------------------------------------------------------------------------
//  FmcSDF::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcSDF::StreamStopHardware()
{
    FmcBidirBoard::StreamStopHardware();

    AFEClock_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  FmcSDF::TachometerFrequency() -- return logic tachometer
//---------------------------------------------------------------------------
//  value is frequency in Hz

unsigned int FmcSDF::TachometerFrequency()
{
    return TachometerReg.TachFreq.Value();
}

//---------------------------------------------------------------------------
//  FmcSDF::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  FmcSDF::GetVitaInStreamCount()
{
    return Fmc_SDF_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  FmcSDF::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  FmcSDF::GetVitaOutStreamCount()
{
    return Fmc_SDF_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  FmcSDF::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  FmcSDF::GetAdcSpiDeviceCount()
{
    return Fmc_SDF_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  FmcSDF::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  FmcSDF::GetDacSpiDeviceCount()
{
    return Fmc_SDF_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  FmcSDF::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  FmcSDF::MakeFrameCalcDevice(FrameDeviceCode which)
{
    // return same object for both directions  (we don't need an object)
    if (which==devAdc)
        return new FmcStandardFrameCalcDevice(this, InputTriggerGranularity);
    else
        return new FmcStandardFrameCalcDevice(this, OutputTriggerGranularity);
}

//---------------------------------------------------------------------------
//  FmcSDF::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  FmcSDF::MakePriClockDevice(PriDeviceCode which)
{
    // return same object for both directions  (we don't need an object)
    if (which==priAdc)
        return new FmcStandardPriClockDevice(this, InputTriggerGranularity);
    else
        return new FmcStandardPriClockDevice(this, OutputTriggerGranularity);
}

//---------------------------------------------------------------------------
//  FmcSDF::MakeMasterPllObject() -- Make Master PLL clock (Ad9510_Tb)
//---------------------------------------------------------------------------

Ad9510_Tb *  FmcSDF::MakeMasterPllObject()
{
    // return same object for both directions
    return new FmcSDF_Ad9510_Tb(this, WB_FMC_1, FmcSDF_MasterPllRegisters );
}


//---------------------------------------------------------------------------
//  FmcSDF::MakeSlavePllObject() -- Make Slave PLL clock (Ad9508_Tb)
//---------------------------------------------------------------------------

Ad9508_Tb *  FmcSDF::MakeSlavePllObject()
{
    // return same object for both directions
    return new FmcSDF_Ad9508_Tb(this, WB_FMC_1, FmcSDF_SlavePllRegisters );
}

//---------------------------------------------------------------------------
//  FmcSDF::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  FmcSDF::GetAnalogDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = MaxInDevices;
        ch_per_dev = InChannelsPerDevice;
        }
    else
        {
        devices = MaxOutDevices;
        ch_per_dev = OutChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  FmcSDF::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  FmcSDF::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = MaxInDevices;
        ch_per_dev = InChannelsPerDevice;
        }
    else
        {
        devices = MaxOutDevices;
        ch_per_dev = OutChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  FmcSDF::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  FmcSDF::GetAdcCalibrationOverride()
{
    return new FmcSDF_AdcCalOverride(this);   // is an override
}

//---------------------------------------------------------------------------
//  FmcSDF::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  FmcSDF::GetCalibrationAdcCount()
{
    return Fmc_SDF_AdcChannels;
}

//---------------------------------------------------------------------------
//  FmcSDF::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  FmcSDF::GetDacCalibrationOverride()
{
    return 0;  // no override
}

//---------------------------------------------------------------------------
//  FmcSDF::GetCalibrationDacCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  FmcSDF::GetCalibrationDacCount()
{
    return Fmc_SDF_DacChannels;
}

//---------------------------------------------------------------------------
//  FmcSDF::GetFmcCalIdromInfo() -- Make ROM Image Objhect
//---------------------------------------------------------------------------

FmcSDF::FmcCalIdromInfo  FmcSDF::GetFmcCalIdromInfo(FmcCalDeviceCode which)
{
    const unsigned int ReservedSize = 1;

    FmcCalIdromInfo  info;
    if (which==fmcCalAdc)
        {
        info.IdromSize = romFmc_SDF_InputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }
    else
        {
        info.IdromSize = romFmc_SDF_OutputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }

    return info;
}

//---------------------------------------------------------------------------
//  FmcSDF::Reset_AD7763() -- Reset the ADCs
//---------------------------------------------------------------------------

void FmcSDF::Reset_AD7763()
{
    // Reset all
    AFEAdc_ResetReg.Bit(0, true);
    Sleep(1);
    AFEAdc_ResetReg.Bit(0, false);
}

//---------------------------------------------------------------------------
//  FmcSDF::Sync_AD7763() -- Sync the ADCs
//---------------------------------------------------------------------------

void FmcSDF::Sync_AD7763()
{
    // Reset all
    AFEAdc_ResetReg.Bit(1, true);
    Sleep(1);
    AFEAdc_ResetReg.Bit(1, false);
}

//---------------------------------------------------------------------------
//  FmcSDF::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void FmcSDF::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = 0x3;  // 1.8V min
    maxFmcVadjCode = 0x4;  // 2.5V max
}

//---------------------------------------------------------------------------
//  FmcSDF::Input() -- Reference to input object
//---------------------------------------------------------------------------

FmcSDFInputDevice & FmcSDF::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  FmcSDF::Output() -- Reference to output object
//---------------------------------------------------------------------------

FmcSDFOutputDevice & FmcSDF::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  FmcSDF::AfterFiclStart() --
//---------------------------------------------------------------------------

void  FmcSDF::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant Pll  101 constant SlavePll");
    engine.Evaluate("  102 constant AdcSpi0  103 constant AdcSpi1");
    engine.Evaluate("  104 constant AdcSpi2  105 constant AdcSpi3");
//    engine.Evaluate("  103 constant TempI2C  104 constant kIdromI2C 105 constant PllImage");
//    engine.Evaluate("  106 constant DacSpi0  ");
    engine.Evaluate(": ?FmcSDF "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   Pll  SlavePll  \" "
                    "    cr .\"   AdcSpi0  AdcSpi1  AdcSpi2  AdcSpi3  \" "
                 //   "    cr .\"   TempI2C  IdromI2C  PllImage \" "
                 //   "    cr .\"   DacSpi0 \" "
                    "    cr  "
                    "    cr .\"   ?FmcSDF  : display this message \" "
                    "    cr  ; ");

    engine.Display("FmcSDF-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  FmcSDF::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  FmcSDF::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;  const int kSlavePll = 101;
    const int kAdcSpi0 = 102;  const int kAdcSpi1 = 103;
    const int kAdcSpi2 = 104;  const int kAdcSpi3 = 105;

    //    const int kTempI2C = 103;   const int kIdromI2C = 104;
//    //const int kPllImage = 105;
//    const int kDacSpi0 = 106;

    switch (dev)
        {
        default:
            return 0;
        case kAdcSpi0:    case kAdcSpi1:
        case kAdcSpi2:    case kAdcSpi3:
            return Input().Spi(dev-kAdcSpi0, addr);
//        case kDacSpi0:
//            return AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr);
        case kPll:
            return AFEClock_SDev.Pll().Master()->Port(addr);
        case kSlavePll:
            return AFEClock_SDev.Pll().Slave()->Port(addr);
//        case kTempI2C:
//            {
//            switch (addr)
//                {
//                default:
//                case 0:
//                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
//                case 1:
//                    return FmcI2CBus_SDev.Temp().GetConfig();
//                case 2:
//                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
//                case 3:
//                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
//                }
//            }
//        case kIdromI2C:
//            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  FmcSDF::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  FmcSDF::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;   const int kSlavePll = 101;
    const int kAdcSpi0 = 102;  const int kAdcSpi1 = 103;
    const int kAdcSpi2 = 104;  const int kAdcSpi3 = 105;
//    const int kTempI2C = 103;  const int kIdromI2C = 104;
//    const int kDacSpi0 = 106;

    switch (dev)
        {
        default:
            break;
        case kAdcSpi0:    case kAdcSpi1:
        case kAdcSpi2:    case kAdcSpi3:
            Input().Spi(dev-kAdcSpi0, addr, value);
            break;
//        case kDacSpi0:
//            AFEDacSpi_SDev.Spi(dev-kDacSpi0, addr, value);
//            break;
        case kPll:
            AFEClock_SDev.Pll().Master()->Port(addr, value);
            break;
        case kSlavePll:
            AFEClock_SDev.Pll().Slave()->Port(addr, value);
            break;
//        case kTempI2C:
//            {
//            switch (addr)
//                {
//                default:
//                case 0:
//                    break;
//                case 1:
//                    FmcI2CBus_SDev.Temp().SetConfig(value);
//                    break;
//                case 2:
//                    FmcI2CBus_SDev.Temp().SetLowT(value);
//                    break;
//                case 3:
//                    FmcI2CBus_SDev.Temp().SetHighT(value);
//                    break;
//                }
//            }
//            break;
//        case kIdromI2C:
//            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
//            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  FmcSDF::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  FmcSDF::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;   const int kSlavePll = 101;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Master()->Frequency(freq);
            break;
        case kSlavePll:
            AFEClock_SDev.Pll().Slave()->Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcSDF::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  FmcSDF::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;   const int kSlavePll = 101;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Master()->FrequencyActual();
        case kSlavePll:
            AFEClock_SDev.Pll().Slave()->FrequencyActual();
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
