// Fmc_SDF_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_SDF_MbH
#define Fmc_SDF_MbH

#include <FmcBoard_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FmcSDF;

//===========================================================================
//  CLASS ISuppliesAdded_SDF_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_SDF_Requirements : public IRequires_Ad9510TbClocks,
                                        public IRequires_AD7763_Support,
                                        public IRequires_FmcCalIDROM
{
public:
    virtual ~ISuppliesAdded_SDF_Requirements()
        {}

};

//=============================================================================
//  CLASS FmcSDF_Ad9510_Tb --  FmcSDF custom Ad9510 (MASTER)
//=============================================================================

class FmcSDF_Ad9510_Tb : public Ad9510_Tb
{
public:
    FmcSDF_Ad9510_Tb(FmcSDF * owner, IAddressingSpace &space, Ad9510_Tb::MapRegisters & reg);

protected:
    virtual void  OnInit();
    virtual void  OnReapply();

    virtual unsigned int AdcMasterOutput();  // most derived defines
    virtual unsigned int DacMasterOutput();  // most derived defines

    virtual unsigned int AdcMultiplierFactor();
    virtual unsigned int DacMultiplierFactor();

    virtual double  DefaultPhaseDetectorFrequency() const {  return 100e5;  }

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

private:

    FmcSDF * Owner;

};

//=============================================================================
//  CLASS FmcSDF_Ad9508_Tb --  FmcSDF custom Ad9508 (SLAVE)
//=============================================================================

class FmcSDF_Ad9508_Tb : public Ad9508_Tb
{
public:
    FmcSDF_Ad9508_Tb(FmcSDF * owner, IAddressingSpace &space, Ad9508_Tb::MapRegisters & reg);

protected:
    virtual unsigned int AdcMasterOutput();  // most derived defines
    virtual unsigned int DacMasterOutput();  // most derived defines

    virtual void            OnInit();
    virtual void            OnReapply();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

private:

    FmcSDF *  Owner;

};

//===========================================================================
//  CLASS FmcSDF  -- Fmc Daughter card Common baseboard class
//===========================================================================

class FmcSDF : public FmcBidirBoard, public ISuppliesAdded_SDF_Requirements
{
public:
    FmcSDF();
    virtual ~FmcSDF()
        {}

    unsigned int TachometerFrequency();

    // Options
//    void  DisableDacClocks(bool state)
//        {  FDacClocksDisabled = state;  }
//    bool  DacClocksDisabled() const
//        {  return FDacClocksDisabled;  }
//    void  DisableTestClocks(bool state)
//        {  FTestClocksDisabled = state;  }
//    bool  TestClocksDisabled() const
//        {  return FTestClocksDisabled;  }

//    void AdcAcCoupled(bool state)  {  FAdcAcCoupled = state;  }
//    bool AdcAcCoupled()  {  return FAdcAcCoupled;  }

    class FmcSDFInputDevice &    Input();
    class FmcSDFOutputDevice &   Output();

    friend class FmcSDFInputDevice;
    friend class FmcSDFOutputDevice;

    IX6ClockIo &                    Clock()           {  return AFEClock_SDev.Clock();  }
    IAd9510DuoRawClockDevices &     RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    virtual double  Get_DacClockRate()  { return Clock().Dac().Frequency(); }

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Supplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    virtual Ad9510_Tb *                 MakeMasterPllObject();
    virtual Ad9508_Tb *                 MakeSlavePllObject();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual FmcCalIdromInfo             GetFmcCalIdromInfo(FmcCalDeviceCode which);

    virtual void                        Reset_AD7763();
    virtual void                        Sync_AD7763();


    //
    //  Misc

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcSDF::FmcTachometerRegister  --  Adc Calibration Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FmcTachometerRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   TachFreq;

        FmcTachometerRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              TachFreq(*this, 0, 17)
        {  }
    };

protected:
    //
    //  Subdevices
    FmcAd9510Duo_Clock     AFEClock_SDev;
    AD7763_SpiDevice       Local_AdcSpi_SDev;
    FmcCalibrationIDROM_Manager    AdcCalIDROM_SDev;    // Cal Rom Image
    FmcCalibrationIDROM_Manager    DacCalIDROM_SDev;    // Cal Rom Image

    Register                AFEAdc_ResetReg;
    FmcTachometerRegister   TachometerReg;

    FmcSDFInputDevice  *    FInput;
    FmcSDFOutputDevice  *   FOutput;

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

private:

    void Init_ADC_Devices();
    void Configure_ADC_ClockFactor();
};

//===========================================================================
//  CLASS FmcSDFInputDevice  -- FMC SDF Input Devices
//===========================================================================

class FmcSDFInputDevice : public FmcInputDevice_Bidir
{
public:
    typedef FmcInputDevice_Bidir inherited;

    FmcSDFInputDevice(FmcSDF * owner, IX6AnalogIoInfo * info );
    virtual ~FmcSDFInputDevice();

    virtual unsigned int  ActiveDevices();

    virtual IX6AnalogIoInfo &  Info();

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }

    void   OpenHardware();
    void   ConfigureHardware();

private:
    FmcSDF *                    OwnerSDF;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;

    AD7763_SpiDevice &          AdcSpi() {  return OwnerSDF->Local_AdcSpi_SDev;  }
    const AD7763_SpiDevice &    AdcSpi() const {  return OwnerSDF->Local_AdcSpi_SDev;  }

    GeneralIdromImage &  AdcCalIdrom()  {  return OwnerSDF->AdcCalIDROM_SDev.Idrom();  }

};

//===========================================================================
//  CLASS FmcSDFOutputDevice  -- FMC SDF Output Devices
//===========================================================================

class FmcSDFOutputDevice : public FmcOutputDevice_Bidir
{
public:
    typedef FmcOutputDevice_Bidir inherited;

    FmcSDFOutputDevice(FmcSDF * owner, IX6AnalogIoInfo * info );
    virtual ~FmcSDFOutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    // Pseudo-protected
    void  OpenHardware();
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }


private:
    FmcSDF *                    OwnerSDF;
    IX6AnalogIoInfo *           FInfo;
    GeneralCalibrationSection   CalRomSection;


    GeneralIdromImage &  DacCalIdrom()  {  return OwnerSDF->DacCalIDROM_SDev.Idrom();  }
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
