//---------------------------------------------------------------------------
// Fmc_Servo_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_Servo_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Clock and Module Constants Definition

//const float  Input_VcoMinClk    = 100.0e6;
//const float  Input_VcoMaxClk    = 140.0e6;

//const float  Output_MaxRate      = 2.0e6;
//const float  Output_MinRate      = 0.0;


//const float  Output_VcoMinClk    = 100.0e6;
//const float  Output_VcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 1;
const int             InChannelsPerDevice = 8;
const int             MaxInChannels     = MaxInDevices*InChannelsPerDevice;

const unsigned int    MaxOutDevices        = 1;
const int             OutChannelsPerDevice = 8;
const int             MaxOutChannels     = MaxOutDevices*OutChannelsPerDevice;

const unsigned int Fmc_Servo_VitaInStreamCount   = MaxInDevices;   // Number of VITA Incoming SIDs
const unsigned int Fmc_Servo_VitaOutStreamCount  = MaxOutDevices;   // Number of VITA Outgoing SIDs

const unsigned int Fmc_Servo_AdcSpiDeviceCount   = 0;   // Number of SPI devices muxed
const unsigned int Fmc_Servo_DacSpiDeviceCount   = 0;   // Number of SPI devices muxed

const unsigned int Fmc_Servo_Adc_FifoDepth = 1024;
const unsigned int Fmc_Servo_Dac_FifoDepth = 1024;

const unsigned int Fmc_Servo_AdcChannelsPerDevice   = InChannelsPerDevice;
const unsigned int Fmc_Servo_AdcChannels   = MaxInChannels;
const unsigned int Fmc_Servo_DacChannelsPerDevice   = OutChannelsPerDevice;
const unsigned int Fmc_Servo_DacChannels   = MaxOutChannels;

const float         InputMaxRate            = 500.e3;
const float         InputMinRate            = 0.0;
const unsigned int  InputWidth_Bits         = 16;
const          int  InputMax_mV             = 20000;
const          int  InputMin_mV             = -20000;
const          int  InputGainRanges         = 4;   const int  InputGainRangesHACK = 2;
const int           InputClockFactor        = 1;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int Fmc_Servo_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_Servo_Adc_One = 0x10000;         // representation of floating point 1.0

const float         OutputMaxRate            = 1.0e6f;
const float         OutputMinRate            = 0.0;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 10000;
const          int  OutputMin_mV             = -10000;
const int           OutputClockFactor        = 1;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 1;

const unsigned int Fmc_Servo_CalibrationDacCount = MaxOutChannels;  // Number of Dac Calibrations
const unsigned int FmcServo_Dac_One = 0x10000;         // representation of floating point 1.0

//---------------------------------------------------------
//  VGA BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_Servo_InputCoeffOffset       = 0;
const int romFmc_Servo_InputCoeffOffset_avail = 16*InputGainRanges+1;
const int romFmc_Servo_InputOneRangeSize = 17;
            // 8 * 2 coefficients plus 'calibrated' word for each of 4 gain ranges
const int romFmc_Servo_InputOneRangeDataSize = 16;

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_Servo_OutputCoeffOffset       = 0;   // these are in separate "ROMs" sections
const int romFmc_Servo_OutputCoeffOffset_avail = 17;   // 8 * 2 coefficients plus 'calibrated' word

//===========================================================================
// Class FmcServo_AnalogInInfo
//===========================================================================

class FmcServo_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    FmcServo_AnalogInInfo()
        {
        FSpan.push_back(SpanInfo(10000, -10000));
        FSpan.push_back(SpanInfo(5000,   -5000));
        FSpan.push_back(SpanInfo(2000,   -2000));
        FSpan.push_back(SpanInfo(1000,   -1000));
        }
    virtual ~FmcServo_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan[0];  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

    //  Ranged Device Specific Info (with default impl)
    virtual size_t  GainRanges() const
                {  return InputGainRanges;  }
    virtual SpanInfo  Span(size_t gain_range) const
                {  return FSpan[gain_range];  }

    virtual unsigned int ClockFactor() const
                {  return InputClockFactor; }

private:
    SpanArray					FSpan;
};

//===========================================================================
// Class FmcServo_AnalogOutInfo
//===========================================================================

class FmcServo_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    FmcServo_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~FmcServo_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

    virtual unsigned int ClockFactor() const
                {  return OutputClockFactor; }

private:
    SpanInfo   FSpan;
};


//===========================================================================
//  CLASS FmcServoInputDevice  -- FmcServo Input
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class FmcServoInputDevice
//---------------------------------------------------------------------------

FmcServoInputDevice::FmcServoInputDevice(FmcServo * owner, IX6AnalogIoInfo* info)
    : inherited(owner), OwnerServo(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romFmc_Servo_InputCoeffOffset,
                    GeneralRangedCalibrationSection::GainRanges(InputGainRanges),
                    GeneralRangedCalibrationSection::CalChannels(MaxInChannels),
                    Fmc_Servo_Adc_One, AdcCal() ),
      AdcDeviceTestEnabled(false), CurrentAdcDeviceTestPattern(0x00)
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcServoInputDevice
//---------------------------------------------------------------------------

FmcServoInputDevice::~FmcServoInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  FmcServoInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo& FmcServoInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  FmcServoInputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void FmcServoInputDevice::AdcDeviceTestMode(bool enabled, unsigned int test_mode, std::vector<unsigned int> test_pattern)
{
    AdcDeviceTestEnabled        = enabled;
    CurrentAdcDeviceTestPattern = test_pattern;
    CurrentAdcDeviceTestMode    = test_mode;
}

//---------------------------------------------------------------------------
//  FmcServoInputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  FmcServoInputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  FmcServoInputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void FmcServoInputDevice::Adc_OnConfigure(double /*ClockRateMHz*/)
{
    // Reapply gain ranges too?

    CalRomSection.ApplyRomCoefficients();

}

//---------------------------------------------------------------------------
//  FmcServoInputDevice::GainRange() - read current gain range setting
//---------------------------------------------------------------------------

size_t  FmcServoInputDevice::GainRange(int ch)
{
    return CalRomSection.GainRange(ch);
}

//---------------------------------------------------------------------------
//  FmcServoInputDevice::GainRange() - read current gain range setting
//---------------------------------------------------------------------------

void  FmcServoInputDevice::GainRange(int ch, size_t gain_range)
{
    CalRomSection.GainRange(ch, gain_range);
    OwnerServo->SetGainRange(ch, gain_range);  // apply to hardware too
}

//===========================================================================
//  CLASS FmcServoOutputDevice  -- FmcServo Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcServoOutputDevice
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class FmcServoOutputDevice
//---------------------------------------------------------------------------

FmcServoOutputDevice::FmcServoOutputDevice(FmcServo * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerServo(owner), FInfo(info),
      CalRomSection(DacCalIdrom(), romFmc_Servo_OutputCoeffOffset, MaxOutChannels,
                       FmcServo_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class FmcServoOutputDevice
//---------------------------------------------------------------------------

FmcServoOutputDevice::~FmcServoOutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  FmcServoOutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  FmcServoOutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  FmcServoOutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  FmcServoOutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  FmcServoOutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  FmcServoOutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  FmcServoOutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  FmcServoOutputDevice::ConfigureHardware()
{
}

//---------------------------------------------------------------------------
//  FmcServoOutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  FmcServoOutputDevice::ConfigureChannelEnables()
{
}

//---------------------------------------------------------------------------
//  FmcServoOutputDevice::DacClockMode() --
//---------------------------------------------------------------------------
// change source of Dac Clock depending on how data is taken
//   In Servo mode, clock is direct, in Acquisition mode comes from logic.

void  FmcServoOutputDevice::DacClockMode(DacClockModes state)
{
    bool bit_state = (state==dcmServo) ? false : true ;
    DacCommon().DRR_Bit(1, bit_state);
}

//=============================================================================
//  FmcServo Timebase Definitions  --  PLL timebase control
//=============================================================================

const int Ad9510_Tb_Instance = 0;
const int Ad9508_Tb_Instance = 1;

//---------------------------------------------------------------------------
//  housekeeping for class FmcServo_Ad9510_Tb
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class FmcServo_Ad9510_Tb
//---------------------------------------------------------------------------

FmcServo_Ad9510_Tb::FmcServo_Ad9510_Tb(FmcServo * owner, IAddressingSpace &space, MapRegisters& reg)
    :  Ad9510_Tb(space, reg, Ad9510_Tb_Instance), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::OnInit() --
//---------------------------------------------------------------------------

void  FmcServo_Ad9510_Tb::OnInit()
{
    //
    //  Set Source Configuration (here same as default)
    SourceInUse(0, true);    SetSourceMapping(0, IX6ClockIo::rsExternal);   // primary = external
    SourceInUse(1, true);    SetSourceMapping(1, IX6ClockIo::rsInternal);   // secondary = internal

    //  Set Output Configuration
    OutputInUse(0,  false);    // LVPECL       //  FPGA_ADC_CLK (Adc (RX) line rate/40)
    OutputInUse(1,  false);    // LVPECL       //  FPGA_ADC_SYSREF (Adc System Reference)
    OutputInUse(2,  false);    // LVPECL       //  GBTCLK1
    OutputInUse(3,  false);    // LVPECL       //  GBTCLK0
    OutputInUse(4,  false);    SetOutputConfig(4,  ocLvds); // unused
    OutputInUse(5,  true);     SetOutputConfig(5,  ocLvds); // input to 9508
    OutputInUse(6,  false);    SetOutputConfig(6,  ocLvds); // unused
    OutputInUse(7,  false);    SetOutputConfig(7,  ocLvds); // unused

}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::OnReapply()
//---------------------------------------------------------------------------

void  FmcServo_Ad9510_Tb::OnReapply()
{

}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::AdcMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcServo_Ad9510_Tb::AdcMasterOutput()
{
    return 5;
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::DacMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcServo_Ad9510_Tb::DacMasterOutput()
{
    return 5;
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::AdcMultiplierFactor()
//---------------------------------------------------------------------------

unsigned int FmcServo_Ad9510_Tb::AdcMultiplierFactor()
{
    return InputClockFactor;
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::DacMultiplierFactor()
//---------------------------------------------------------------------------

unsigned int FmcServo_Ad9510_Tb::DacMultiplierFactor()
{
    return OutputClockFactor;
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::SetAdcOutputRates()
//---------------------------------------------------------------------------

void FmcServo_Ad9510_Tb::SetAdcOutputRates(double adc_rate)
{
    double SampleFrequency = adc_rate;

    OutputFrequency(5, SampleFrequency);
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9510_Tb::SetDacOutputRates()
//---------------------------------------------------------------------------

void FmcServo_Ad9510_Tb::SetDacOutputRates(double /*dac_rate*/)
{
    // none
}

//=============================================================================
//  CLASS FmcServo_Ad9508_Tb --  FmcServo custom Ad9508 (SLAVE)
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcServo_Ad9508_Tb
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class FmcServo_Ad9508_Tb
//---------------------------------------------------------------------------

FmcServo_Ad9508_Tb::FmcServo_Ad9508_Tb(FmcServo * owner, IAddressingSpace &space, MapRegisters& reg)
   :  Ad9508_Tb(space, reg, Ad9508_Tb_Instance), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9508_Tb::OnInit()
//---------------------------------------------------------------------------

void  FmcServo_Ad9508_Tb::OnInit()
{
   //  Set Output Configuration
   OutputInUse(0,  true);  SetOutputConfig(0,  ocLvds);  //  FPGA_ADC_CLK (Adc (RX) line rate/40)
   OutputInUse(1,  true);  SetOutputConfig(1,  ocLvds);  //  FPGA_DAC_CLK (Adc System Reference)
   OutputInUse(2,  true);  SetOutputConfig(2,  ocLvds);  //  ADC_CLK
   OutputInUse(3,  true);  SetOutputConfig(3,  ocLvds);  //  DAC_CLK

   SetOutputSync(0, true);
   SetOutputSync(1, true);
   SetOutputSync(2, true);
   SetOutputSync(3, true);

}

//---------------------------------------------------------------------------
//  FmcServo_Ad9508_Tb::OnReapply()
//---------------------------------------------------------------------------

void  FmcServo_Ad9508_Tb::OnReapply()
{
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9508_Tb::AdcMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcServo_Ad9508_Tb::AdcMasterOutput()
{
   return 2;
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9508_Tb::DacMasterOutput()
//---------------------------------------------------------------------------

unsigned int FmcServo_Ad9508_Tb::DacMasterOutput()
{
   return 3;
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9508_Tb::SetAdcOutputRates()
//---------------------------------------------------------------------------

void FmcServo_Ad9508_Tb::SetAdcOutputRates(double adc_rate)
{
   double SampleFrequency = adc_rate;

   OutputFrequency(0, SampleFrequency);
   OutputFrequency(2, SampleFrequency);
}

//---------------------------------------------------------------------------
//  FmcServo_Ad9508_Tb::SetDacOutputRates()
//---------------------------------------------------------------------------

void FmcServo_Ad9508_Tb::SetDacOutputRates(double dac_rate)
{
   double SampleFrequency = dac_rate;
   OutputFrequency(1, SampleFrequency);
   OutputFrequency(3, SampleFrequency);
}

//===========================================================================
//  CLASS FmcServo  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcServo
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{

FmcAd9510Duo_Clock::MapRegisters FmcAd9510DuoClockRegAddrs =
    {
    mmFmc_PllControl,    //  AfePllControl_Addr;
    mmFmc_PllSpi,        //  AfePllSpi_Addr
    };

Ad9510_Tb::MapRegisters  FmcServo_MasterPllRegisters =
    {
    mmFmc_PllControl,    //  PllControl_Addr;
    mmFmc_PllSpi,        //  PllSpi_Addr
    };

Ad9508_Tb::MapRegisters  FmcServo_SlavePllRegisters =
    {
    mmFmc_PllControl,    //  PllControl_Addr;
    mmFmc_PllSpi,        //  PllSpi_Addr
    };

PolledDataFifo::MapRegisters  FmcServoAdcFifoRegAddrs =
    {
    mmFmc_ServoAdcFifoCtl,    //  FifoControl_Addr;
    mmFmc_ServoAdcFifoThrs,   //  FifoThreshold_Addr
    mmFmc_ServoAdcFifoData,   //  FifoData_Addr;
    mmFmc_ServoAdcFifoCtl,    //  DacDelay_Addr - not used
    };

PolledDataFifo::MapRegisters  FmcServoDacFifoRegAddrs =
    {
    mmFmc_ServoDacFifoCtl,    //  FifoControl_Addr;
    mmFmc_ServoDacFifoThrs,   //  FifoThreshold_Addr
    mmFmc_ServoDacFifoData,   //  FifoData_Addr;
    mmFmc_ServoDacDelay,      //  DacDelay_Addr
    };

}  // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class FmcServo
//---------------------------------------------------------------------------

FmcServo::FmcServo()
    :  FmcBidirBoard(fmcPrimary),
       AFEClock_SDev(this, WB_FMC_1, FmcAd9510DuoClockRegAddrs),
       AdcFifo_SDev(this, WB_FMC_2, FmcServoAdcFifoRegAddrs, Fmc_Servo_Adc_FifoDepth),
       DacFifo_SDev(this, WB_FMC_3,FmcServoDacFifoRegAddrs, Fmc_Servo_Dac_FifoDepth),
       AdcCalIDROM_SDev(this, RomInfo() ),        // always = ADC Rom
       DacCalIDROM_SDev(this, RomInfo(), false),  // true = ADC Rom
       AdcControlReg(WB_FMC_2, mmFmc_ServoAdcControl),
       AdcGainControlReg(WB_FMC_2, mmFmc_ServoAdcGain),
       AdcGainReadReg(WB_FMC_2, mmFmc_ServoAdcGain),
       FInput(new FmcServoInputDevice(this, new FmcServo_AnalogInInfo)),
       FOutput(new FmcServoOutputDevice(this, new FmcServo_AnalogOutInfo)),
       FDacClocksDisabled(false), FTestClocksDisabled(true)
{
    //AFEAdcSpi_SDev.DeviceEnabled(false);  enable, but use 0

    //IdRom().DualPageRomMode(true);  // use paged IDROM  - now autodetects

    LocalConstructCustomParts();
}

FmcServo * anServo()
{
    FmcServo * wombat = new FmcServo;
    return wombat;
}

//---------------------------------------------------------------------------
//  FmcServo::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  FmcServo::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  Header Section
    RI.Header().Version(0x2);

    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("FmcServo");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(250);
    RI.FmcDef().ModuleSize(Fmc::msSingleWide);
    RI.FmcDef().ClockDirection(Fmc::dirCarrierToMezz);
    RI.FmcDef().ConnectorSize(Fmc::fmcP1, Fmc::csHPC);
    RI.FmcDef().ConnectorSize(Fmc::fmcP2, Fmc::csNotFitted);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankA, 87);
    RI.FmcDef().BankSignalCount(Fmc::fmcP1, Fmc::bankB, 30);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 0);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankB, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP1, 0);
    RI.FmcDef().TransceiverCount(Fmc::fmcP2, 0);

    //  DAC CALIBRATION SECTION
    RI.DacCal().Resize(romFmc_Servo_OutputCoeffOffset_avail);
    RI.DacCal().Zero();

    //  ...add any section records
//    {
//    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

//    DcOut->Standby(true);
//    DcOut->OutputNumber(0);
//    DcOut->NominalV(2.5f);
//    DcOut->RipplePkPk(0.5f);
////        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
////        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
////        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
////        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
//    }

//    {
//    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

//    DcOut->Standby(false);
//    DcOut->OutputNumber(1);
//    DcOut->NominalV(12.0);
//    DcOut->RipplePkPk(0.1f);
//    DcOut->MaxCurrent(3.5f);
//    //        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//    //        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//    //        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//    }

//    {
//    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

//    DcLoad->OutputNumber(0);
//    DcLoad->NominalV(2.5f);
////        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
////        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
////        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
////        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
////        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
//    }

    //  ADC RANGED CALIBRATION SECTION
    {
    FmcStandardRomInfo::AdcRangedCalCoefficientsMR * Rng0 = RI.NewAdcRangedCalCoefficientRecord(0, InputGainRanges, MaxInChannels);
    //Rng0->Resize(romFmc_Servo_InputOneRangeDataSize);
    Rng0->Zero();
    FmcStandardRomInfo::AdcRangedCalCoefficientsMR * Rng1 = RI.NewAdcRangedCalCoefficientRecord(1, InputGainRanges, MaxInChannels);
    //Rng1->Resize(romFmc_Servo_InputOneRangeDataSize);
    Rng1->Zero();
    FmcStandardRomInfo::AdcRangedCalCoefficientsMR * Rng2 = RI.NewAdcRangedCalCoefficientRecord(2, InputGainRanges, MaxInChannels);
    //Rng2->Resize(romFmc_Servo_InputOneRangeDataSize);
    Rng2->Zero();
    FmcStandardRomInfo::AdcRangedCalCoefficientsMR * Rng3 = RI.NewAdcRangedCalCoefficientRecord(3, InputGainRanges, MaxInChannels);
    //Rng3->Resize(romFmc_Servo_InputOneRangeDataSize);
    Rng3->Zero();
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcServo::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcServo::LocalConstructCustomParts()
{
    FmcBidirBoard::ConstructCustomParts();

    AFEClock_SDev.ConstructCustomParts();
    AdcFifo_SDev.ConstructCustomParts();
    DacFifo_SDev.ConstructCustomParts();
    AdcCalIDROM_SDev.ConstructCustomParts();
    DacCalIDROM_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  FmcServo::OpenHardware()
//---------------------------------------------------------------------------

void  FmcServo::OpenHardware()
{
//    PokeLogic(0x0,   0x00000);  // offsets relative to FMC

    FmcBidirBoard::OpenHardware();

    Debug.Log("++++++++++++++++++++++++++++++++++++");
    Debug.Log("  Load Adc/Dac Calibration Shadows");
    AdcCalIDROM_SDev.OpenHardware();
    DacCalIDROM_SDev.OpenHardware();
    Debug.Log("++++++++++++++++++++++++++++++++++++");

    //  Turn on VCO Power
    //AFEAdcVcoRegister.Bit(4, true);

//    PokeLogic(0x202, 0x3);
//    unsigned int data = PeekLogic(0x0);
//    Debug.Log("Power Register Final: ", data, 16);

    AFEClock_SDev.OpenHardware();
    AdcFifo_SDev.OpenHardware();
    DacFifo_SDev.OpenHardware();

    Input().OpenHardware();
    Output().OpenHardware();

}

//---------------------------------------------------------------------------
//  FmcServo::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcServo::PreconfigureHardware()
{
    Output().ConfigureChannelEnables();

    FmcBidirBoard::PreconfigureHardware();

    AFEClock_SDev.PreconfigureHardware();
    AdcFifo_SDev.PreconfigureHardware();
    DacFifo_SDev.PreconfigureHardware();
    //
    //  Special Config for D/A Devices
    Output().PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  FmcServo::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcServo::ConfigureHardware()
{
    FmcBidirBoard::ConfigureHardware();

    AFEClock_SDev.ConfigureHardware();
    AdcFifo_SDev.ConfigureHardware();
    DacFifo_SDev.ConfigureHardware();
    //
    //  Special Config for A/D Devices
    double ClockRateMHz = AFEClock_SDev.Clock().Adc().FrequencyActual()/1.0e6;

    Input().Adc_OnConfigure(ClockRateMHz);
    //
    //  Adc Calibration
    //AFEAdcCalCtrl_SDev.StartCalibration();
    //Sleep(1);
    //unsigned int results = AFEAdcCalCtrl_SDev.CalibrationResults();

    //Debug.Log("FmcServo Adc Cal Results: ", results);
    //
    //  Special Config for D/A Devices
    Output().ConfigureHardware();

}

//---------------------------------------------------------------------------
//  FmcServo::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcServo::StreamStopHardware()
{
    FmcBidirBoard::StreamStopHardware();

    AFEClock_SDev.StreamStopHardware();
    AdcFifo_SDev.StreamStopHardware();
    DacFifo_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  FmcServo::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  FmcServo::GetVitaInStreamCount()
{
    return Fmc_Servo_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  FmcServo::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  FmcServo::GetVitaOutStreamCount()
{
    return Fmc_Servo_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  FmcServo::GetAdcSpiDeviceCount() -- Return Adc SPI devices
//---------------------------------------------------------------------------

unsigned int  FmcServo::GetAdcSpiDeviceCount()
{
    return Fmc_Servo_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  FmcServo::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  FmcServo::GetDacSpiDeviceCount()
{
    return Fmc_Servo_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  FmcServo::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  FmcServo::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new FmcStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  FmcServo::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  FmcServo::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new FmcStandardPriClockDevice(this, 2);
}

//---------------------------------------------------------------------------
//  FmcServo::MakeMasterPllObject() -- Make Master PLL clock (Ad9510_Tb)
//---------------------------------------------------------------------------

Ad9510_Tb *  FmcServo::MakeMasterPllObject()
{
    // return same object for both directions
    return new FmcServo_Ad9510_Tb(this, WB_FMC_1, FmcServo_MasterPllRegisters );
}


//---------------------------------------------------------------------------
//  FmcServo::MakeSlavePllObject() -- Make Slave PLL clock (Ad9510_Tb)
//---------------------------------------------------------------------------

Ad9508_Tb *  FmcServo::MakeSlavePllObject()
{
    // return same object for both directions
    return new FmcServo_Ad9508_Tb(this, WB_FMC_1, FmcServo_SlavePllRegisters );
}

//---------------------------------------------------------------------------
//  FmcServo::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  FmcServo::GetAdcCalibrationOverride()
{
    return 0;
}

//---------------------------------------------------------------------------
//  FmcServo::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  FmcServo::GetCalibrationAdcCount()
{
    return Fmc_Servo_AdcChannels;
}

//---------------------------------------------------------------------------
//  FmcServo::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  FmcServo::GetDacCalibrationOverride()
{
    return 0;
}

//---------------------------------------------------------------------------
//  FmcServo::GetCalibrationDacCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  FmcServo::GetCalibrationDacCount()
{
    return Fmc_Servo_DacChannels;
}

//---------------------------------------------------------------------------
//  FmcServo::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  FmcServo::GetAnalogDeviceInfo(FrameDeviceCode which,
                                  unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = MaxInDevices;
        ch_per_dev = Fmc_Servo_AdcChannelsPerDevice;
        }
    else //(which==devDac)
        {
        devices = MaxOutDevices;
        ch_per_dev = Fmc_Servo_DacChannelsPerDevice;
        }

}

//---------------------------------------------------------------------------
//  FmcServo::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  FmcServo::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = MaxInDevices;
        ch_per_dev = Fmc_Servo_AdcChannelsPerDevice;
        }
    else //(which==devDac)
        {
        devices = MaxOutDevices;
        ch_per_dev = Fmc_Servo_DacChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  FmcServo::GetFmcCalIdromInfo() -- Make ROM Image Objhect
//---------------------------------------------------------------------------

FmcServo::FmcCalIdromInfo  FmcServo::GetFmcCalIdromInfo(FmcCalDeviceCode which)
{
    const unsigned int ReservedSize = 1;

    FmcCalIdromInfo  info;
    if (which==fmcCalAdc)  // unranged ADC coefficients not used
        {
        info.IdromSize = romFmc_Servo_InputCoeffOffset_avail+InputGainRanges;
        info.ReservedSize = InputGainRanges;
        }
    else
        {
        info.IdromSize = romFmc_Servo_OutputCoeffOffset_avail+ReservedSize;
        info.ReservedSize = ReservedSize;
        }

    return info;
}

//---------------------------------------------------------------------------
//  FmcServo::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void FmcServo::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = 0x3;  // 1.8V min
    maxFmcVadjCode = 0x4;  // 2.5V max
}

//---------------------------------------------------------------------------
//  FmcServo::Input() -- Reference to input object
//---------------------------------------------------------------------------

FmcServoInputDevice & FmcServo::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  FmcServo::Output() -- Reference to output object
//---------------------------------------------------------------------------

FmcServoOutputDevice & FmcServo::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  FmcServo::SetGainRange() -- set gain range setting for a channel
//---------------------------------------------------------------------------

void  FmcServo::SetGainRange(int ch, size_t gain_range)
{
    Debug.Log("FmcServo::SetGainRange()");
    Debug.Log("...Ch", ch);
    Debug.Log("...GainRange", gain_range);

    AdcGainControlReg.AdcChan.Value(ch);
    AdcGainControlReg.Gain.Value(static_cast<ii32>(gain_range));
    AdcGainControlReg.GainWrite.Value(false);
    AdcGainControlReg.Apply();

    Sleep(5);

    AdcGainControlReg.GainWrite.Value(true);
    AdcGainControlReg.Apply();

    Sleep(5);

    AdcGainControlReg.GainWrite.Value(false);
    AdcGainControlReg.Apply();
}

//---------------------------------------------------------------------------
//  FmcServo::AfterFiclStart() --
//---------------------------------------------------------------------------

void  FmcServo::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
    engine.Evaluate("  100 constant Pll  101 constant SlavePll");
    engine.Evaluate("  103 constant TempI2C  104 constant IdromI2C");
//    engine.Evaluate("  ");

    engine.Evaluate(": ?FmcServo "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
                    "    cr .\"   Pll  SlavePll  \" "
                    "    cr .\"   TempI2C  IdromI2C  \" "
                    "    cr  "
                    "    cr .\"   ?FmcServo  : display this message \" "
                    "    cr  ; ");

    engine.Display("FmcServo-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  FmcServo::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  FmcServo::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;  const int kSlavePll = 101;
    const int kTempI2C = 103;   const int kIdromI2C = 104;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return AFEClock_SDev.Pll().Master()->Port(addr);
        case kSlavePll:
            return AFEClock_SDev.Pll().Slave()->Port(addr);
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  FmcServo::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  FmcServo::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;  const int kSlavePll = 101;
    const int kTempI2C = 103;  const int kIdromI2C = 104;

    switch (dev)
        {
        default:
            break;
        case kPll:
            AFEClock_SDev.Pll().Master()->Port(addr, value);
            break;
        case kSlavePll:
            AFEClock_SDev.Pll().Slave()->Port(addr, value);
            break;
        case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
        case kFmc:
            PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  FmcServo::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  FmcServo::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;   const int kSlavePll = 101;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Master()->Frequency(freq);
            break;
        case kSlavePll:
            AFEClock_SDev.Pll().Slave()->Frequency(freq);
            break;
        case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcServo::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  FmcServo::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
    const int kPll = 100;   const int kSlavePll = 101;

    switch (dev)
        {
        default:
        case kPll:
            AFEClock_SDev.Pll().Master()->FrequencyActual();
        case kSlavePll:
            AFEClock_SDev.Pll().Slave()->FrequencyActual();
        case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
