// Fmc_Servo_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Fmc_Servo_MbH
#define Fmc_Servo_MbH

#include <FmcBoard_Mb.h>
#include <Ad9510Duo_Tb_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FmcServo;

//===========================================================================
//  CLASS ISuppliesAdded_Servo_Requirements -- FMC Board Map Requirements
//===========================================================================

class ISuppliesAdded_Servo_Requirements : public IRequires_Ad9510TbClocks,
                                          public IRequires_FmcCalIDROM
{
public:
    virtual ~ISuppliesAdded_Servo_Requirements()
        {}

};

//=============================================================================
//  CLASS FmcServo_Ad9510_Tb --  FmcServo custom Ad9510 (MASTER)
//=============================================================================

class FmcServo_Ad9510_Tb : public Ad9510_Tb
{
public:
    FmcServo_Ad9510_Tb(FmcServo * owner, IAddressingSpace &space, Ad9510_Tb::MapRegisters & reg);

protected:
    virtual void  OnInit();
    virtual void  OnReapply();

    virtual unsigned int AdcMasterOutput();  // most derived defines
    virtual unsigned int DacMasterOutput();  // most derived defines

    virtual unsigned int AdcMultiplierFactor();
    virtual unsigned int DacMultiplierFactor();

    virtual double  DefaultPhaseDetectorFrequency() const {  return 100e5;  }

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

private:

    FmcServo * Owner;

};

//=============================================================================
//  CLASS FmcServo_Ad9508_Tb --  FmcServo custom Ad9508 (SLAVE)
//=============================================================================

class FmcServo_Ad9508_Tb : public Ad9508_Tb
{
public:
    FmcServo_Ad9508_Tb(FmcServo * owner, IAddressingSpace &space, Ad9508_Tb::MapRegisters & reg);

protected:
    virtual unsigned int AdcMasterOutput();  // most derived defines
    virtual unsigned int DacMasterOutput();  // most derived defines

    virtual void            OnInit();
    virtual void            OnReapply();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

private:

    FmcServo *  Owner;

};

//===========================================================================
//  CLASS FmcServo  -- Fmc Daughter card Common baseboard class
//===========================================================================

class FmcServo : public FmcBidirBoard, public ISuppliesAdded_Servo_Requirements
{
public:
    FmcServo();
    virtual ~FmcServo()
        {}

    // Options
    void  DisableDacClocks(bool state)
        {  FDacClocksDisabled = state;  }
    bool  DacClocksDisabled() const
        {  return FDacClocksDisabled;  }
    void  DisableTestClocks(bool state)
        {  FTestClocksDisabled = state;  }
    bool  TestClocksDisabled() const
        {  return FTestClocksDisabled;  }

    class FmcServoInputDevice &    Input();
    class FmcServoOutputDevice &   Output();

    IPolledFifo *   AdcFifo() {  return &AdcFifo_SDev.Fifo();  }
    IPolledFifo *   DacFifo() {  return &DacFifo_SDev.Fifo();  }

    friend class FmcServoInputDevice;
    friend class FmcServoOutputDevice;

    IX6ClockIo &                    Clock()           {  return AFEClock_SDev.Clock();  }
    IAd9510DuoRawClockDevices &     RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }

    virtual double   Get_DacClockRate()  { return Clock().Dac().Frequency(); }

    // pseudo-protected
    //virtual void    InitCardPower();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  FMC Requirements Suupplier Methods
    virtual unsigned int                GetVitaInStreamCount();
    virtual unsigned int                GetVitaOutStreamCount();
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    virtual Ad9510_Tb *                 MakeMasterPllObject();
    virtual Ad9508_Tb *                 MakeSlavePllObject();

//    virtual Lmk04800 *                  MakeFmcAfePllObject();
//    virtual void                        MakeAfeAdcSpiDevices();
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                                                unsigned int & devices, unsigned int & ch_per_dev);
    virtual FmcCalIdromInfo             GetFmcCalIdromInfo(FmcCalDeviceCode which);

    //
    //  Misc
//    unsigned int      CurrentDacInterpolationMode()
//        {  return FDacInterpolationMode;  }

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

protected:
    //
    //  Subdevices
    FmcAd9510Duo_Clock             AFEClock_SDev;
    PolledDataFifo                 AdcFifo_SDev;
    PolledDataFifo                 DacFifo_SDev;

    //X6AFE_AdcSpi       Local_AdcSpi_SDev;
    FmcRangedCalibrationIDROM_Manager    AdcCalIDROM_SDev;    // Cal Rom Image
    FmcCalibrationIDROM_Manager          DacCalIDROM_SDev;    // Cal Rom Image

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcServo::AdcControlRegister  -- Adc Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  AdcReset;
        RegisterBit  AdcStandby;
        RegisterBit  AdcSleep;

    public:
        AdcControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcReset(*this, 0),  AdcStandby(*this, 1), AdcSleep(*this, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcServo::AdcGainControlRegister  -- Adc Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcGainControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup   AdcChan;
        RegisterBit        GainWrite;
        RegisterBitGroup   Gain;

    public:
        AdcGainControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcChan(*this, 0, 3),  GainWrite(*this, 3), Gain(*this, 4, 2)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FmcServo::AdcGainReadRegister  -- Adc Gain Readback
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcGainReadRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup   Gain;

    public:
        AdcGainReadRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Gain(*this, 4, 2)
            {
            }
    };

    AdcControlRegister      AdcControlReg;
    AdcGainControlRegister  AdcGainControlReg;
    AdcGainReadRegister     AdcGainReadReg;

    FmcServoInputDevice  *    FInput;
    FmcServoOutputDevice  *   FOutput;

    bool              FDacClocksDisabled;
    bool              FTestClocksDisabled;

    void    SetGainRange(int ch, size_t gain_range);

    void LocalConstructCustomParts();

    virtual double  StandardClockFrequency()
                        {  return AFEClock_SDev.Clock().Frequency();   }

    virtual void  Configure_FmcRomInfo(FmcStandardRomInfo & RI);

    virtual void  LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode);

};

//===========================================================================
//  CLASS FmcServoInputDevice  -- FMC Servo Input Devices
//===========================================================================

class FmcServoInputDevice : public FmcInputDevice_Bidir
{
public:
    typedef FmcInputDevice_Bidir inherited;

    FmcServoInputDevice(FmcServo * owner, IX6AnalogIoInfo* info );
    virtual ~FmcServoInputDevice();

    virtual IX6AnalogIoInfo &  Info();

    size_t  GainRange(int ch);
    void    GainRange(int ch, size_t gain_range);

    virtual GeneralRangedCalibrationSection & Cal() {  return CalRomSection;  }

    void   AdcDeviceTestMode(bool enabled, unsigned int test_mode, std::vector<unsigned int> test_pattern);

     void   OpenHardware();
     void   Adc_OnConfigure(double ClockRateMHz);

private:
    FmcServo *                        OwnerServo;
    IX6AnalogIoInfo *                 FInfo;
    GeneralRangedCalibrationSection   CalRomSection;

    bool                      AdcDeviceTestEnabled;
    unsigned int              CurrentAdcDeviceTestMode;
    std::vector<unsigned int> CurrentAdcDeviceTestPattern;

    GeneralIdromImage &  AdcCalIdrom()  {  return OwnerServo->AdcCalIDROM_SDev.Idrom();  }

};

//===========================================================================
//  CLASS FmcServoOutputDevice  -- FMC Servo Output Devices
//===========================================================================

class FmcServoOutputDevice : public FmcOutputDevice_Bidir
{
public:
    friend class Fmc110_DacToHardware;

    typedef FmcOutputDevice_Bidir inherited;

    FmcServoOutputDevice(FmcServo * owner, IX6AnalogIoInfo * info );
    virtual ~FmcServoOutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }

    // change source of Dac Clock depending on how data is taken
    enum  DacClockModes { dcmServo, dcmAcquisition };
    void  DacClockMode(DacClockModes state);

    // Pseudo-protected
    void  OpenHardware();
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

    virtual GeneralCalibrationSection & Cal() {  return CalRomSection;  }


private:
    FmcServo *                OwnerServo;
    IX6AnalogIoInfo *       FInfo;
//    AdDac9122::DacDevice    DacDev;
    GeneralCalibrationSection   CalRomSection;


    GeneralIdromImage &  DacCalIdrom()  {  return OwnerServo->DacCalIDROM_SDev.Idrom();  }
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif
