//---------------------------------------------------------------------------
// Fmc_Sfp_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Fmc_Sfp_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
const unsigned int    MaxInDevices      = 1;
const int             ChannelsPerDevice = 1;
const int             MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const unsigned int Fmc_Sfp_VitaInStreamCount   = MaxInDevices;   // Number of VITA Incoming SIDs
const unsigned int Fmc_Sfp_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

//const unsigned int Fmc_Adc20_AdcSpiDeviceCount   = MaxInDevices;   // Number of SPI devices muxed

const unsigned int Fmc_Sfp_ChannelsPerDevice   = ChannelsPerDevice;
const unsigned int Fmc_Sfp_Channels   = MaxInChannels;

const float         InputMaxRate            = 25.0e6f;
const float         InputMinRate            = 5.0e6f;
const unsigned int  InputWidth_Bits         = 14;
const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int Fmc_Sfp_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations
const unsigned int Fmc_Sfp_Adc_One = 0x10000;         // representation of floating point 1.0

//---------------------------------------------------------
//  FMC BUS IDROM Section Information
//  ...System Section: Standard Data only
const int romFmc_Sfp_SysInfoOffset        = 0;
const int romFmc_Sfp_SysInfoOffset_avail  = 18;  // first 'free' word

//---------------------------------------------------------
//  VGA BUS IDROM Section Information
//  ...Adc Cal Idrom: ADC coefficients
const int romFmc_Sfp_InputCoeffOffset       = 0;
const int romFmc_Sfp_InputCoeffOffset_avail = 41;   // 20 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romFmc_Sfp_OutputCoeffOffset       = romFmc_Sfp_InputCoeffOffset_avail;
const int romFmc_Sfp_OutputCoeffOffset_avail = 0;  // none used

//=============================================================================
//  CLASS FmcSfp_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcSfp_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class FmcSfp_SioPll
//---------------------------------------------------------------------------

FmcSfp_SioPll::FmcSfp_SioPll(FmcSfp * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
	ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

	Debug.Enabled(true);  // turn on verbose object output
}

//---------------------------------------------------------------------------
//  FmcSfp_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  FmcSfp_SioPll::OnInit()
{
	DenomForN(1);
	DenomForM(0, 1);     // match settings for utility
	DenomForM(1, 1);
	DenomForM(2, 1);
	DenomForM(3, 1);

	//  PLL enabled by default in SettingsInit()
	//  Device Powered/Enabled Setup
	SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
	SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
	SetClkPowerDown(3, false);    SetMultiSynthPowerDown(3, false);
	//
	//  Set Input Configuration
	SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
	SetFeedbackMuxConfig(fbNoClk);
	SetPllReferenceInput(prRefClk);
	SetFeedbackSource(false, efbNoClk);     // use internal feedback
	//
	//  Set Output Configuration
	//  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
	SetOutDividerSource(2, diNoClk);   // override since not using output
	//SetOutDividerSource(3, diNoClk);   // override since not using output

	OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
	OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
	OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
	OutputInUse(3, true);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, true);

	SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
	SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
	SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
	SetOutputVoltage(3, vo_2p5volts);   // actually off

	SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
	SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);  // follow primary
    OutputFrequency(2, 100.0e6);  // follow primary - actually off
    OutputFrequency(3, 100.0e6);  // follow primary - actually off

}

//---------------------------------------------------------------------------
//  FmcSfp_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  FmcSfp_SioPll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  FmcSfp_SioPll::OutputFrequency() -- Set ALL output frequencies
//---------------------------------------------------------------------------

void  FmcSfp_SioPll::OutputFrequency(unsigned int output, double freq)
{
    //  If this is the master, we need to adjust everyone's output
    //    to follow it
    if (output == FMasterOutput)
        {
        Si5338::OutputFrequency(0, freq);
        Si5338::OutputFrequency(1, freq);
        Si5338::OutputFrequency(2, freq);
        Si5338::OutputFrequency(3, freq);
        }

    //  Apply the frequency
	Si5338::OutputFrequency(output, freq);
}

//---------------------------------------------------------------------------
//  FmcSfp_SioPll::OutputFrequency() -- read an output frequencies
//---------------------------------------------------------------------------

double  FmcSfp_SioPll::OutputFrequency(unsigned int output) const
{
	return Si5338::OutputFrequency(output);
}

//===========================================================================
// Class FmcSfp_AnalogInInfo
//===========================================================================

class FmcSfp_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    FmcSfp_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~FmcSfp_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS FmcSfp  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class FmcSfp
//---------------------------------------------------------------------------
using namespace FmcCarrier_MemoryMap;

namespace
{

X6SioPll::MapRegisters PexSioPllRegAddrs =
    {
	mmAurora_SioXoI2C,   //  SioPll_Addr
    };

}

//---------------------------------------------------------------------------
//  constructor for class FmcSfp
//---------------------------------------------------------------------------

FmcSfp::FmcSfp()
	:  FmcBoard(fmcPrimary),
	   SioPll_SDev(this, WB_FMC_4, PexSioPllRegAddrs)//,     // on Aurora 0
{
    Aurora0_SDev.Sfp_SystemEnabled(true);

    LocalConstructCustomParts();
}

FmcSfp * anSfp()
{
    FmcSfp * wombat = new FmcSfp;
    return wombat;
}

//---------------------------------------------------------------------------
//  FmcSfp::Configure_FmcRomInfo() -- Load Board Specific FMC Rom Info
//---------------------------------------------------------------------------

void  FmcSfp::Configure_FmcRomInfo(FmcStandardRomInfo & RI)
{
    //  BOARD INFO Section
    RI.Board().Manufacturer("Innovative Integration");
    RI.Board().Name("FmcSfp");
    RI.Board().SerialNumber("");
    RI.Board().PartNumber("");
    RI.Board().FruFile("");

    //  FMC DEFINITION SECTION
    RI.FmcDef().MaxClockMHz(100);
    RI.FmcDef().BankSignalCount(Fmc::fmcP2, Fmc::bankA, 20);
//        void    ModuleSize(Fmc::FmcModuleSize size);
//        void    ConnectorSize(Fmc::FmcConnector p, Fmc::FmcConnectorSize size);
//        void    ClockDirection(Fmc::FmcClockDirection dir);
//        void    BankSignalCount(Fmc::FmcConnector p, Fmc::FmcConnectorBank bank, unsigned int count);
//        void    TransceiverCount(Fmc::FmcConnector p, unsigned int count);
//        void    MaxClockMHz(unsigned int rateMHz )   {  MaxClock_1 = rateMHz;  }

    //  ...add any section records
    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->Standby(true);
    DcOut->OutputNumber(0);
    DcOut->NominalV(3.3f);
    DcOut->RipplePkPk(0.5f);
//        void          Standby(bool state);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)   {  NominalV_2   = To10mV(v);  }
//        void          MaxNegVDev(float v) {  MaxNegVDev_2 = To10mV(v);  }
//        void          MaxPosVDev(float v) {  MaxPosVDev_2 = To10mV(v);  }
//        void          RipplePkPk(float v) {  RipplePkPk_2 = To1mV(v);  }
//        void          MinCurrent(float a) {  MinCurrent_2 = To1mA(a);  }
//        void          MaxCurrent(float a) {  MaxCurrent_2 = To1mA(a);  }
    }

    {
    FmcStandardRomInfo::DcOutputMultiRecord *  DcOut = RI.NewDcOutputRecord();

    DcOut->OutputNumber(1);
    DcOut->NominalV(12.0);
    DcOut->RipplePkPk(0.1f);
    DcOut->MaxCurrent(3.5f);
    }

    {
    FmcStandardRomInfo::DcLoadMultiRecord *  DcLoad = RI.NewDcLoadRecord();

    DcLoad->OutputNumber(0);
    DcLoad->NominalV(3.3f);
//        void          OutputNumber(unsigned int number);
//        void          NominalV(float v)        {  NominalV_2   = To10mV(v);  }
//        void          SpecdMinV(float v)       {  SpecdMinV_2 = To10mV(v);  }
//        void          SpecdMaxV(float v)       {  SpecdMaxV_2 = To10mV(v);  }
//        void          SpecdRipplePkPk(float v) {  SpecdRipplePkPk_2 = To1mV(v);  }
//        void          MinCurrentLoad(float a)  {  MinCurrentLoad_2 = To1mA(a);  }
//        void          MaxCurrentLoad(float a)  {  MaxCurrentLoad_2 = To1mA(a);  }
    }

    RI.ToRomImage();
}

//---------------------------------------------------------------------------
//  FmcSfp::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  FmcSfp::LocalConstructCustomParts()
{
	FmcBoard::ConstructCustomParts();

	SioPll_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  FmcSfp::OpenHardware()
//---------------------------------------------------------------------------

void  FmcSfp::OpenHardware()
{
	FmcBoard::OpenHardware();
	SioPll_SDev.OpenHardware();

}

//---------------------------------------------------------------------------
//  FmcSfp::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  FmcSfp::PreconfigureHardware()
{
	FmcBoard::PreconfigureHardware();
	SioPll_SDev.PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  FmcSfp::ConfigureHardware()
//---------------------------------------------------------------------------

void  FmcSfp::ConfigureHardware()
{
	FmcBoard::ConfigureHardware();
	SioPll_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  FmcSfp::StreamStopHardware()
//---------------------------------------------------------------------------

void  FmcSfp::StreamStopHardware()
{
	FmcBoard::StreamStopHardware();
	SioPll_SDev.StreamStopHardware();

}

//---------------------------------------------------------------------------
//  FmcSfp::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  FmcSfp::GetVitaInStreamCount()
{
    return Fmc_Sfp_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  FmcSfp::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  FmcSfp::GetVitaOutStreamCount()
{
    return Fmc_Sfp_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  FmcSfp::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  FmcSfp::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new FmcSfp_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  FmcSfp::LoadSupportedVAdjRange() -- Define allowed VAdj Range
//---------------------------------------------------------------------------

void FmcSfp::LoadSupportedVAdjRange(short & minFmcVadjCode, short & maxFmcVadjCode)
{
    minFmcVadjCode = 0x3;  // 1.8V min
    maxFmcVadjCode = 0x4;  // 2.5V max
}

//---------------------------------------------------------------------------
//  FmcSfp::AfterFiclStart() --
//---------------------------------------------------------------------------

void  FmcSfp::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate("  0 constant Fmc ");   ...in baseboard class
	engine.Evaluate("  105 constant FmcSioPll  106 constant TempI2C  107 constant IdromI2C   ");

    engine.Evaluate(": ?FmcSfp "
                    "    cr .\" --- FMC SubDevice Constants for d@ d! dFreq dFreqActual  --- \" "
                    "    cr .\"     (starting at 100)                                        \" "
					"    cr .\"   FmcSioPll TempI2C  IdromI2C   \" "
					"    cr  "
                    "    cr .\"   ?FmcSfp  : display this message \" "
                    "    cr  ; ");

    engine.Display("FmcSfp-specific wordset loaded \n" );
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
    #pragma warning(disable : 4244)
#endif
//---------------------------------------------------------------------------
//  FmcSfp::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  FmcSfp::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  // from base board
	const int kFmcSioPll = 105;     const int kTempI2C = 106;     const int kIdromI2C = 107;

    switch (dev)
        {
        default:
            return 0;
		case kFmcSioPll:
			return SioPll_SDev.Pll().Port(addr);
		case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetT());
                case 1:
                    return FmcI2CBus_SDev.Temp().GetConfig();
                case 2:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetLowT());
                case 3:
                    return static_cast<int>(FmcI2CBus_SDev.Temp().GetHighT());
                }
            }
        case kIdromI2C:
            return FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr);
        case kFmc:
            return PeekLogic(addr);
        }
}

//---------------------------------------------------------------------------
//  FmcSfp::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  FmcSfp::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;  // from base board
	const int kFmcSioPll = 105;     const int kTempI2C = 106;     const int kIdromI2C = 107;

    switch (dev)
        {
        default:
            break;
		case kFmcSioPll:
			SioPll_SDev.Pll().Port(addr, value);
            break;
		case kTempI2C:
            {
            switch (addr)
                {
                default:
                case 0:
                    break;
                case 1:
                    FmcI2CBus_SDev.Temp().SetConfig(value);
                    break;
                case 2:
                    FmcI2CBus_SDev.Temp().SetLowT(value);
                    break;
                case 3:
                    FmcI2CBus_SDev.Temp().SetHighT(value);
                    break;
                }
            }
            break;
        case kIdromI2C:
            FmcI2CBus_SDev.FmcIdrom().Device().I2C_Reg(addr, value);
            break;
		case kFmc:
			PokeLogic(addr, value);
            break;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  FmcSfp::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  FmcSfp::Do_DFreq(int dev, float freq)
{
	const int kFmc = 0;  // from base board
	const int kFmcSioPll = 105;

    switch (dev)
        {
        default:
		case kFmcSioPll:
			SioPll_SDev.Pll().Frequency(freq);
            break;
		case kFmc:      // no frequency setting
            break;
        }
}

//---------------------------------------------------------------------------
//  FmcSfp::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  FmcSfp::Do_DFreqActual(int dev)
{
    const int kFmc = 0;  // from base board
	const int kFmcSioPll = 105;

    switch (dev)
        {
		default:
		case kFmcSioPll:
			return static_cast<float>(SioPll_SDev.Pll().FrequencyActual());
		case kFmc:      // no frequency setting
            return 0.0;
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
