// IIFourier.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Fourier_MbH
#define Fourier_MbH

#include <WindowType_Mb.h>
#include <CommonFourier_Mb.h>
#include <algorithm>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS Fourier  --
//=============================================================================

class Fourier : public CommonFourier
{
public:
	typedef CommonFourier inherited;
	enum IISpectrumMode { smAmplitude, smPower };

	// Ctors
	Fourier();
	Fourier(const Fourier & other);
	Fourier & operator = (const Fourier & other);

	// Properties
	void          Alpha( float value )
		{  FAlpha = value;  }
	float         Alpha() const
		{  return FAlpha;  }

	void          Window( WindowTypes value )
		{  FWindow = value;  }
	WindowTypes   Window() const
		{  return FWindow;  }

	void          IgnoreDC( bool value )
		{  FIgnoreDC = value;  }
	bool          IgnoreDC() const
		{  return FIgnoreDC;  }

	void          IgnoreFs2( bool value )
		{  FIgnoreFs2 = value;  }
	bool          IgnoreFs2() const
		{  return FIgnoreFs2;  }

	void          SpectrumEnabled(bool state)
        {  FSpectrumEnabled = state;  }
    bool          SpectrumEnabled() const
        {  return FSpectrumEnabled;  }

    void          SpectrumMode(IISpectrumMode value)
        {  FSpectrumMode = value;  }
    IISpectrumMode SpectrumMode() const
        {  return FSpectrumMode;  }

    // Spectral accumulation control
    void          Accumulating(bool state);
    bool          Accumulating() const;

	// Methods
	virtual bool    	  Transform(const IppComplexDG & in, IppComplexDG & out);
	virtual bool          Transform();


	IppFloatDG      Spectrum()
		{  return IppFloatDG(mSpectrum);  }
	IppFloatDG      PhaseAngle()
		{  return IppFloatDG(mPhaseAngle);  }
	IppComplexDG      WindowedTime()
		{  return IppComplexDG(mWindow);  }

	unsigned int    Margin() const
		{  return std::max<unsigned int>(FReach/1000, 1);  }

protected:
	// Fields
	float               FAlpha;
	WindowTypes         FWindow;
    bool                FIgnoreDC;
	bool                FIgnoreFs2;
	bool                FSpectrumEnabled;
	IISpectrumMode      FSpectrumMode;
	unsigned int        FAverages;
	bool                FAveraging;

	// Data
	Buffer              mSpectrum;
	Buffer              mPhaseAngle;
	Buffer              mWindow;
	Buffer              mScratch;
	Buffer              mFrequencySum;
	Buffer              mSpectrumSum;

	// Overloads
	virtual bool        Start();

    // Support
	IppComplexDG        Scratch()
		{  return IppComplexDG(mScratch);  }
  	void ApplyWindow(IppComplexDG & in, IppComplexDG & out);
	void accumulate(IppComplexDG & buffer);
	void average(IppComplexDG & buffer);
	void CalculateSpectrum(IppComplexDG & src);
	IppComplexDG    FrequencySum()
		{  return IppComplexDG(mFrequencySum);  }
	IppFloatDG      SpectrumSum()
		{  return IppFloatDG(mSpectrumSum);  }

	virtual bool transform(const IppComplexDG & in, IppComplexDG & out);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative

#endif
