// FpaLoadThread_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef FpaLoadThread_MbH
#define FpaLoadThread_MbH

#include <Event_Mb.h>
#include <ProcessEvents_Mb.h>
#include <FpgaImage_Mb.h>
#include <Thread_Mb.h>
#include <FpaLoadBase_Mb.h>
#include <StopWatch_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IFpaLoader -- Interface for Flash Loading
//==============================================================================

class IFpaLoader
{
public:
    //
    //  Member Functions
    virtual ~IFpaLoader()  {}

    virtual void  Parse(std::string exo_file) = 0;
    virtual void  Erase() = 0;
    virtual void  Program() = 0;
    virtual void  Verify() = 0;
    virtual void  Read() = 0;
    virtual bool  CheckFpga(std::string BitFileFpgaStr, std::string BoardFpgaStr) = 0;
    virtual std::string BitFileFpgaName(std::string PartStr) = 0;
    //
    //  Golden ROM Image Support
    virtual bool   HasGoldenImage() = 0;
    virtual bool   GoldenImageBooted() = 0;
    virtual bool   GoldenImageActive() = 0;
    virtual void   GoldenImageActive(bool state) = 0;
    virtual int    CpldRevision() = 0;

    virtual int    DeviceId() const = 0;
    virtual int    DeviceSize() const = 0;
    virtual int    Id() const = 0;

    virtual const UniversalFpgaImage &     ExoImage() = 0;

    // Events
    OpenWire::ThunkedEventHandler<ProcessProgressEvent>     OnParseProgress;
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>   OnParseComplete;
    OpenWire::ThunkedEventHandler<ProcessStatusEvent>       OnParseError;

    OpenWire::ThunkedEventHandler<ProcessProgressEvent>     OnLoadProgress;
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>   OnLoadComplete;
    OpenWire::ThunkedEventHandler<ProcessStatusEvent>       OnLoadError;

    OpenWire::ThunkedEventHandler<ProcessStatusEvent>       OnGoldenStatusChanged;
};

//==============================================================================
//  CLASS IFpaPersonality -- Interface to FPA specific operations
//==============================================================================

class IFpaPersonality
{
public:

//    IFpaPersonality(FpaLoadBase & fpa) :  Fpa(fpa)
//        {}
    virtual ~IFpaPersonality()
        {}

    virtual int  FirmwareRevision() const = 0 ;
    //virtual int  PcbRevision() const  = 0  ;
    virtual int  Id() const  = 0  ;

    //---------------------
    //  Personality Methods
    virtual  int       DeviceId() const  = 0;
    virtual  int       DeviceSize() const = 0;
    virtual  int       SectorSize() const = 0;

    virtual  unsigned int       StartSector() const = 0;
    virtual  unsigned int       Sectors() const = 0;
    virtual  unsigned int       GoldenImageStartSector() const = 0;
    virtual  unsigned int       GoldenImageSectors() const = 0;

    virtual  unsigned int DataMask() const = 0;

    virtual  bool      IsProtected() const = 0;

    virtual  bool      Flash(int addr, int value) = 0;
    virtual  int       Flash(int addr) = 0;

    virtual  bool      Authenticate() = 0;
    virtual  bool      Initialize() = 0;
    virtual  bool      EraseSector(int sector) = 0;

    virtual  bool      Program(FpgaImage & Image) = 0;
    virtual  bool      Verify(FpgaImage & Image) = 0;
    virtual  bool      Read(FpgaImage & Image) = 0;

    virtual  bool      HasGoldenImage() = 0;
    virtual  bool      GoldenImageBooted() = 0;
    virtual  bool      GoldenImageActive() = 0;
    virtual  void      GoldenImageActive(bool state) = 0;
    virtual  int       CpldRevision() = 0;
    virtual  int       CpldType() = 0;

    //---------------------
    //  Events
    OpenWire::EventHandler<ProcessStatusEvent>     OnLoadError;
    OpenWire::EventHandler<ProcessProgressEvent>   OnLoadProgress;

};

//==============================================================================
//  CLASS IFpaLoadBasePersonality -- Interface to FPA specific operations
//==============================================================================
//  Uses FpaLoadBase to implement its personality

class IFpaLoadBasePersonality  : public IFpaPersonality
{
public:
    IFpaLoadBasePersonality(FpaLoadBase & fpa) :  Fpa(fpa)
        {}
    virtual ~IFpaLoadBasePersonality()
        {}

    virtual int  FirmwareRevision() const ;
//    virtual int  PcbRevision() const ;
    virtual int  Id() const ;

protected:
    FpaLoadBase &           Fpa;

};

//==============================================================================
//  CLASS IX6FpaCpldPersonality -- Interface to FPA specific operations
//==============================================================================
//  Uses IX6FpaCpld to implement its personality

class IX6FpaCpldPersonality : public IFpaPersonality
{
public:
    enum RomModes { romM29EWtype };

    IX6FpaCpldPersonality(IX6FpaCpld & cpld) :  Cpld(cpld), Chip(0)
        {}
    virtual ~IX6FpaCpldPersonality();

    virtual int  FirmwareRevision() const
        {  return Cpld.CPLD_Version();  }
    virtual int  Id() const
        {  return Cpld.CPLD_FlashType();  }

    //---------------------
    //  Personality Methods
    virtual int   DeviceId() const
        {  return FDeviceId;  }
    virtual int   DeviceSize() const
        {  return FDeviceSize;  }
    virtual int   SectorSize() const
        {  return FSectorSize;  }

    virtual bool  IsProtected() const
       {  return  FProtected;  }
    virtual  unsigned int DataMask() const
       {  return  0xFFFFFFFF;  }

    virtual  unsigned int       StartSector() const;
    virtual  unsigned int       Sectors() const;
    virtual  unsigned int       GoldenImageStartSector() const;
    virtual  unsigned int       GoldenImageSectors() const;

    virtual  bool      Flash(int addr, int value);
    virtual  int       Flash(int addr);

    virtual  bool      Authenticate();
    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);

    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage()
            {  return true;  }
    virtual  bool      GoldenImageBooted()
            {  return Cpld.CPLD_GoldenImageBooted(); }
    virtual  bool      GoldenImageActive()
            {  return Cpld.CPLD_GoldenImageActive();  };
    virtual  void      GoldenImageActive(bool state)
            {  Cpld.CPLD_GoldenImageActive(state);    };
    virtual  int       CpldRevision()
        {  return Cpld.CPLD_Version();  }
    virtual  int       CpldType()
        {  return Id();  }

    //
    //  Implementation Methods
    void                Prom(unsigned int addr, unsigned int value);     // 'native' read/write
    unsigned int        Prom(unsigned int addr);

    void                Prom32(unsigned int addr, unsigned int value);   // 32 bit wide image read/write
    unsigned int        Prom32(unsigned int addr);

protected:
    IX6FpaCpld &           Cpld;
    class IX6FpaChip *     Chip;
    StopWatch              Rtc;

    int                 FDeviceId;
    int                 FDeviceSize;
    int                 FSectorSize;
    bool                FProtected;
    int                 Percent[2];
    RomModes            RomMode;

    virtual void        OnFlashWrite_Start(int addr, int data);

    unsigned int        SectorBaseAddress(unsigned int sector);

    bool       SingleCellProgram(FpgaImage & Image);
    bool       BlockProgram(FpgaImage & Image);
    void       CreateChipImpl();
    void       UncreateChipImpl();
};

//==============================================================================
//  CLASS IX6FpaChip -- X6 Flash Sub-Personality Interface
//==============================================================================

class IX6FpaChip
{
public:
    IX6FpaChip(IX6FpaCpldPersonality * owner)
        :  Owner(owner), FDeviceId(0), FProtected(false), FTotalSectors(0)
        {}
    virtual ~IX6FpaChip()
        {}

    virtual bool  Initialize() = 0;
    virtual void  EnterCFIQueryMode() = 0;
    virtual void  LeaveCFIQueryMode() = 0;

    void  Prom(int addr, int value)
            {  Owner->Prom(addr, value);  }
    void  Prom32(int addr, int value)
            {  Owner->Prom32(addr, value);  }
    int   Prom(int addr)
            {  return Owner->Prom(addr);  }
    int   DeviceId() const
        {  return FDeviceId;  }
    bool  IsProtected() const
       {  return  FProtected;  }

    virtual unsigned int SectorBaseAddress(unsigned int sector) = 0;

    virtual unsigned int ByteOrder(unsigned int data) = 0;
    virtual unsigned int StartSector() const = 0;
    virtual unsigned int Sectors() const = 0;
    virtual unsigned int GoldenImageStartSector() const = 0;
    virtual unsigned int GoldenImageSectors() const = 0;

    virtual void        OnFlashWrite_Start(int addr, int data) = 0;
    virtual void        OnSingleCellPgm_Start() = 0;
    virtual void        OnSingleCellPgm_End() = 0;
    virtual void        OnEraseSector_Start(unsigned int sector) = 0;
    virtual void        OnWriteBufferProgram(FpgaImageDatagram & ImageDg, unsigned int Addr, unsigned int wc)=0;

protected:
    IX6FpaCpldPersonality *  Owner;
    int                 FDeviceId;
    bool                FProtected;
    int                 FTotalSectors;

    unsigned int        ByteSwap(unsigned int data);

};

//------------------------------------------------------------------------------
// IX6FpaChip::ByteSwap() - Byte swap 16 bit word
//------------------------------------------------------------------------------

inline unsigned int  IX6FpaChip::ByteSwap(unsigned int data)
{
    unsigned int LSB = data & 0x00FF;
    unsigned int MSB = (data & 0xFF00) >> 8;
    return (LSB<<8) | MSB;
}

//==============================================================================
//  CLASS IVpxFpaChip -- X6 Flash Sub-Personality Interface
//==============================================================================

class IVpxFpaChip
{
public:
    IVpxFpaChip(/*class VpxFpaPersonality * owner*/)
        :  FDeviceId(0), FProtected(false), FTotalSectors(0)
        {}
    virtual ~IVpxFpaChip()
        {}

    virtual bool  Initialize() = 0;

    virtual unsigned int  Read32(unsigned int addr) = 0;
    virtual void          Write32(unsigned int addr, unsigned int value) = 0;
    //
    //  Information
    int   DeviceId() const
        {  return FDeviceId;  }
    virtual unsigned int SectorSize(unsigned int sector) const = 0;
    virtual unsigned int DeviceSize()
        {  return  SectorSize(0)*Sectors();  }
    virtual unsigned int Sectors() const = 0;
    virtual unsigned int GoldenImageSectors() const = 0;
    virtual unsigned int StartSector() const = 0;
    virtual unsigned int GoldenImageStartSector() const = 0;

    bool  IsProtected() const
       {  return  FProtected;  }

    virtual unsigned int ByteOrder(unsigned int data);

    virtual void         EraseSector(unsigned int sector) = 0;

    virtual unsigned int SectorBaseAddress(unsigned int sector) // normal implementation
                            {  return sector*SectorSize(sector);  }

protected:
    //VpxFpaPersonality *  Owner;
    int                 FDeviceId;
    bool                FProtected;
    int                 FTotalSectors;

    unsigned int        ByteSwap(unsigned int data);

};

//------------------------------------------------------------------------------
// IVpxFpaChip::ByteSwap() - Byte swap 16 bit word
//------------------------------------------------------------------------------

inline unsigned int  IVpxFpaChip::ByteSwap(unsigned int data)
{
    unsigned int LSB = data & 0x00FF;
    unsigned int MSB = (data & 0xFF00) >> 8;
    return (LSB<<8) | MSB;
}

//==============================================================================
//  CLASS VpxFpaPersonality -- Interface to FPA specific operations
//==============================================================================

class VpxFpaPersonality : public IFpaPersonality
{
public:

    VpxFpaPersonality(IVpxFpaChip * chip)
        :  Chip(chip)
        {}
    virtual ~VpxFpaPersonality()
        {}

    virtual int  FirmwareRevision() const
        {  return 0;  }
    virtual int  Id() const
        {  return Chip->DeviceId();  }

    //---------------------
    //  Personality Methods
    //
    //  ...Info
    virtual  int           DeviceId() const
                    {  return Chip->DeviceId();  }
    virtual  int           DeviceSize() const
                    {  return Chip->DeviceSize();  }
    virtual  int           SectorSize() const
                    {  return Chip->SectorSize(Chip->Sectors());  }
    virtual  unsigned int  Sectors() const
                    {  return Chip->Sectors();  }
    virtual  unsigned int  StartSector() const
                    {  return Chip->StartSector();  }
    virtual  unsigned int  GoldenImageStartSector() const
                    {  return Chip->GoldenImageStartSector();  }
    virtual  unsigned int  GoldenImageSectors() const
                    {  return Chip->GoldenImageSectors();  }

    virtual  unsigned int DataMask() const
                    {  return  0xFFFFFFFF;  }

    virtual  bool      IsProtected() const
                    {  return Chip->IsProtected();  }

    //  Are these needed?  X6 rewrites 'Program' 
    virtual  bool      Flash(int /*addr*/, int /*value*/) {  return true;  }
    virtual  int       Flash(int /*addr*/) {  return 0;  }

    virtual  bool      Authenticate();
    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);

    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage() {  return false;  }
    virtual  bool      GoldenImageBooted() {  return false;  }
    virtual  bool      GoldenImageActive()  {  return false;  }
    virtual  void      GoldenImageActive(bool /*state*/) {}
    virtual  int       CpldRevision() {  return DeviceId();  }
    virtual  int       CpldType() {  return 0;  }

private:
    IVpxFpaChip *     Chip;
    StopWatch         Rtc;
    int               Percent[2];

};


//==============================================================================
//  CLASS IVpxCopFpaCpldPersonality -- Interface to FPA specific operations
//==============================================================================
//  Uses IX6FpaCpld to implement its personality

class IVpxCopFpaCpldPersonality : public IFpaPersonality
{
public:
    enum RomModes { romM29EWtype };

    IVpxCopFpaCpldPersonality(IX6FpaCpld & cpld) :  Cpld(cpld), Chip(0)
        {}
    virtual ~IVpxCopFpaCpldPersonality();

    virtual int  FirmwareRevision() const
        {  return Cpld.CPLD_Version();  }
    virtual int  Id() const
        {  return Cpld.CPLD_FlashType();  }

    //---------------------
    //  Personality Methods
    virtual int   DeviceId() const
        {  return FDeviceId;  }
    virtual int   DeviceSize() const
        {  return FDeviceSize;  }
    virtual int   SectorSize() const
        {  return FSectorSize;  }

    virtual bool  IsProtected() const
       {  return  FProtected;  }
    virtual  unsigned int DataMask() const
       {  return  0xFF;  }

    virtual  unsigned int       StartSector() const;
    virtual  unsigned int       Sectors() const;
    virtual  unsigned int       GoldenImageStartSector() const;
    virtual  unsigned int       GoldenImageSectors() const;

    virtual  bool      Flash(int addr, int value);
    virtual  int       Flash(int addr);

    virtual  bool      Authenticate();
    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);

    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage()
            {  return true;  }
    virtual  bool      GoldenImageBooted()
            {  return Cpld.CPLD_GoldenImageBooted(); }
    virtual  bool      GoldenImageActive()
            {  return Cpld.CPLD_GoldenImageActive();  };
    virtual  void      GoldenImageActive(bool state)
            {  Cpld.CPLD_GoldenImageActive(state);    };
    virtual  int       CpldRevision()
        {  return Cpld.CPLD_Version();  }
    virtual  int       CpldType()
        {  return Id();  }

    //
    //  Implementation Methods
    void                Prom(unsigned int addr, unsigned int value);     // 'native' read/write
    unsigned int        Prom(unsigned int addr);

//    void                Prom32(unsigned int addr, unsigned int value);   // 32 bit wide image read/write
//    unsigned int        Prom32(unsigned int addr);

protected:
    IX6FpaCpld &           Cpld;
    class IVpxCopFpaChip *     Chip;
    StopWatch              Rtc;

    int                 FDeviceId;
    int                 FDeviceSize;
    int                 FSectorSize;
    bool                FProtected;
    int                 Percent[2];
    RomModes            RomMode;

    virtual void        OnFlashWrite_Start(int addr, int data);

    unsigned int        SectorBaseAddress(unsigned int sector);

    bool       SingleCellProgram(FpgaImage & Image);
    bool       BlockProgram(FpgaImage & Image);
    void       CreateChipImpl();
    void       UncreateChipImpl();
};

//==============================================================================
//  CLASS IVpxCopFpaChip -- VpxCop Flash Sub-Personality Interface
//==============================================================================

class IVpxCopFpaChip
{
public:
    IVpxCopFpaChip(IVpxCopFpaCpldPersonality * owner)
        :  Owner(owner), FDeviceId(0), FProtected(false), FTotalSectors(0)
        {}
    virtual ~IVpxCopFpaChip()
        {}

    virtual bool  Initialize() = 0;
    virtual void  EnterCFIQueryMode() = 0;
    virtual void  LeaveCFIQueryMode() = 0;

    void  Prom(int addr, int value)
            {  Owner->Prom(addr, value);  }
//    void  Prom32(int addr, int value)
//            {  Owner->Prom32(addr, value);  }
    int   Prom(int addr)
            {  return Owner->Prom(addr);  }
    int   DeviceId() const
        {  return FDeviceId;  }
    bool  IsProtected() const
       {  return  FProtected;  }

    virtual unsigned int SectorBaseAddress(unsigned int sector) = 0;

    virtual unsigned int ByteOrder(unsigned int data) = 0;
    virtual unsigned int StartSector() const = 0;
    virtual unsigned int Sectors() const = 0;
    virtual unsigned int GoldenImageStartSector() const = 0;
    virtual unsigned int GoldenImageSectors() const = 0;

    virtual void        OnFlashWrite_Start(int addr, int data) = 0;
    virtual void        OnSingleCellPgm_Start() = 0;
    virtual void        OnSingleCellPgm_End() = 0;
    virtual void        OnEraseSector_Start(unsigned int sector) = 0;
    virtual void        OnWriteBufferProgram(FpgaImageDatagram & ImageDg, unsigned int Addr, unsigned int wc)=0;
    virtual void        OnBufferProgramAbortAndReset()=0;

protected:
    IVpxCopFpaCpldPersonality *  Owner;
    int                 FDeviceId;
    bool                FProtected;
    int                 FTotalSectors;

    //unsigned int        ByteSwap(unsigned int data);

};

//------------------------------------------------------------------------------
// IVpxCopFpaChip::ByteSwap() - Byte swap 32 bit word
//------------------------------------------------------------------------------
/*
inline unsigned int  IVpxCopFpaChip::ByteSwap(unsigned int data)
{
    // Reverse the order of the four bytes in a 32 bit word
    unsigned char * asChar = reinterpret_cast<unsigned char *>(&data);
    unsigned int LSB = static_cast<unsigned int>(asChar[0]) & 0x00FF;
    unsigned int LMB = static_cast<unsigned int>(asChar[1]) & 0x00FF;
    unsigned int MMB = static_cast<unsigned int>(asChar[2]) & 0x00FF;
    unsigned int MSB = static_cast<unsigned int>(asChar[3]) & 0x00FF;
    
    unsigned int rev_data = (LSB<<24) | (LMB<<16) | (MMB<<8) | MSB;
    return rev_data;
}
*/
//==============================================================================
//  CLASS FpaLoadThread -- Flash load via FPA interface
//==============================================================================

class FpaLoadThread : public IFpaLoader, public Thread
{
    typedef Thread inherited;

public:

    //
    //  Member Functions
    FpaLoadThread(FpaLoadBase & fpa);
    FpaLoadThread(IFpaPersonality * fpaType);
    ~FpaLoadThread();

    // User Methods
    // Foreground
    void                OnOpen();

    void                SetFullEraseMode(bool state)
                            {  FullEraseMode = state;  }
    
    // Background
    void                Initialize();
    void                Parse(std::string exo_file);
    void                Erase();
    void                Program();
    void                Verify();
    void                Read();

    std::string         BitFileFpgaName(std::string PartStr);
    bool                CheckFpga(std::string BitFileFpgaStr, std::string BoardFpgaStr);

    //
    //  Golden ROM Image Support
    bool                HasGoldenImage()
                            { return FpaPersonality->HasGoldenImage();  }
    bool                GoldenImageBooted()
                            { return FpaPersonality->GoldenImageBooted();  }
    bool                GoldenImageActive()
                            { return FpaPersonality->GoldenImageActive();  }
    void                GoldenImageActive(bool state)
                            { 
                            bool StateChanged = GoldenImageActive() != state;
                            FpaPersonality->GoldenImageActive(state);   
                
                            if (StateChanged)
                                {
                                ProcessStatusEvent e("");
                                OnGoldenStatusChanged.Execute(e);
                                }
                            }
    int                 CpldRevision()
                            { return FpaPersonality->CpldRevision();  }
    int                 CpldType()
                            { return FpaPersonality->CpldType();  }

    // User accessible properties

//    void                Device(int value);
//    int                 Device() const;

    // Methods
    int                 DeviceId() const
        {  return FpaPersonality->DeviceId();  }
    int                 DeviceSize() const
        {  return FpaPersonality->DeviceSize();  }
    int                 SectorSize() const
        {  return FpaPersonality->SectorSize();  }
    int                 StartSector() const
        {  return FpaPersonality->StartSector();  }
    int                 Sectors() const
        {  return FpaPersonality->Sectors();  }
    int                 GoldenImageStartSector() const
        {  return FpaPersonality->GoldenImageStartSector();  }
    int                 GoldenImageSectors() const
        {  return FpaPersonality->GoldenImageSectors();  }
    bool                Authentic()
        {  return FpaPersonality->Authenticate();  }
    unsigned int        DataMask() const
        {  return FpaPersonality->DataMask();  }
    int                 FirmwareRevision() const;
    int                 PcbRevision() const   // True Rev code is elsewhere
        {  return 0;  }                       //   on modern cards
    int                 Id() const;

    const UniversalFpgaImage &     ExoImage()
        {  return Image;  }

    bool      Flash(int addr, int value)
        {  return FpaPersonality->Flash(addr, value);  }
    int       Flash(int addr)
        {  return FpaPersonality->Flash(addr);  }

    void  EraseSector(int sector)
        {    FpaPersonality->EraseSector(sector);   }
    void  FpaInit()
        {    FpaPersonality->Initialize();   }

    // Support

    // Events
//    OpenWire::ThunkedEventHandler<ProcessProgressEvent>     OnParseProgress;
//    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>   OnParseComplete;
//    OpenWire::ThunkedEventHandler<ProcessStatusEvent>       OnParseError;
//
//    OpenWire::ThunkedEventHandler<ProcessProgressEvent>     OnLoadProgress;
//    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>   OnLoadComplete;
//    OpenWire::ThunkedEventHandler<ProcessStatusEvent>       OnLoadError;

protected:
    //
    //  Member Functions
    virtual void Init();
    virtual void Execute();

private:
    enum { sParsing=inherited::sLast, sProgramming, sErasing, sVerifying, sReading, sLast};

    // Data
    UniversalFpgaImage      Image;

    IFpaPersonality *       FpaPersonality;
    Event                   Erasing;
    Event                   Parsing;
    Event                   Programming;
    Event                   Verifying;
    Event                   Reading;
    StopWatch               Rtc;
    bool                    Once;
    bool                    FullEraseMode;


    // Methods
    // Event Handlers
    void                    HandleParseProgress(ProcessProgressEvent & event);
    void                    HandleParseComplete(ProcessCompletionEvent & event);
    void                    HandleParseError(ProcessStatusEvent & event);
    void                    HandlePersonalityLoadError(ProcessStatusEvent & event);
    void                    HandlePersonalityLoadProgress(ProcessProgressEvent & event);

    // Member data
    // Percent complete accumulators
    int                 Percent[2];


    // Member functions

    int                 GetRevision();
//    bool                EraseDevice();
    void                DoErase();
    void                DoParse();
    void                DoProgram();
    void                DoVerify();
    void                DoRead();

    // Support Properties



    void                Construct();
};

//==============================================================================
//  CLASS FpaPersonalityBase -- FPA Flash Personality Interface common impl
//==============================================================================

class FpaPersonalityBase : public IFpaLoadBasePersonality
{
public:
    enum FpaMode { Addressing, Quantizing, Fetching, Storing,
                   Authenticating, Incrementing, ReadStatusReg=0x6, SwapImage=0x7 };

    FpaPersonalityBase(FpaLoadBase & fpa) :  IFpaLoadBasePersonality(fpa)
        {  FDataBits = 8;  }

    virtual int   DeviceId() const
        {  return FDeviceId;  }
    virtual int   DeviceSize() const
        {  return FDeviceSize;  }
    virtual int   SectorSize() const
        {  return FSectorSize;  }

    virtual bool  IsProtected() const
       {  return  FProtected;  }
    virtual  unsigned int DataMask() const
       {  return  (1<<FDataBits)-1;  }

    virtual  bool       Authenticate();
    virtual  bool       Flash(int addr, int value);
    virtual  int        Flash(int addr);

    int                 Test(int value);    

    void                Prom(int addr, int value);
    int                 Prom(int addr);
protected:

    int                 ShiftIn();

    void                Clock(int value);
    int                 Clock() const;

    void                Strobe();

    void                Dio(int value);
    int                 Dio() const;

    void                Address(int value);
    int                 Address() const;

    void                Data(int value);
    int                 Data() const;

    void                Mode(FpaMode mode);
    FpaMode             Mode() const;
    
    void                Randomize();
    // Fields
    int                 FSectorSize;
    int                 FDeviceId;
    int                 FAddress;
    int                 FData;
    int                 FClock;
    FpaMode             FMode;
    int                 FDio;
    int                 FSeed;
    int                 FAddressBits;
    bool                FProtected;
    int                 FDevice;
    int                 FDeviceSize;
    int                 Fallow;
    int                 FDataBits;
    StopWatch           Rtc;
    int                 Percent[2];
    //
    //  Additional Internal Personality Functions
    virtual void         OnFlashWrite_Start(int addr, int data) = 0;

};

//==============================================================================
//  CLASS StandardFpaPersonality -- Default FPA Flash Personality Interface
//==============================================================================

class StandardFpaPersonality : public FpaPersonalityBase
{
public:
    StandardFpaPersonality(FpaLoadBase & fpa)
        :  FpaPersonalityBase(fpa), FSectors(0)
        {  FDataBits = 8;  }

    virtual  bool  Initialize();
    virtual  bool  EraseSector(int sector);
    virtual  bool  Program(FpgaImage & Image);
    virtual  bool  Verify(FpgaImage & Image);
    virtual  bool  Read(FpgaImage & Image);

    //  Standard FPA has no Golden Image
    virtual  bool      HasGoldenImage()
                            { return false;  }
    virtual  bool      GoldenImageBooted()
                            { return false;  }
    virtual  bool      GoldenImageActive()
                            { return false;  }
    virtual  void      GoldenImageActive(bool /*state*/)
                            {   }
    virtual  int       CpldRevision()
                            { return 0;  }
    virtual  int       CpldType()
                            { return 0;  }

    virtual  unsigned int       StartSector() const
                            {  return 0;  }
    virtual  unsigned int       Sectors() const
                            {  return FSectors;  }
    virtual  unsigned int       GoldenImageStartSector() const
                            {  return StartSector()+Sectors();  }
    virtual  unsigned int       GoldenImageSectors() const
                            {  return FSectors;  }

protected:
    unsigned int       FSectors;

    virtual void   OnFlashWrite_Start(int addr, int data);
};

//==============================================================================
//  CLASS X5FpaPersonality -- X5 FPA Flash Personality Interface
//==============================================================================

class X5FpaPersonality : public FpaPersonalityBase
{
    enum RomModes { romNtype, romJtype, romXtype };

public:
    X5FpaPersonality(FpaLoadBase & fpa)
        :  FpaPersonalityBase(fpa), FGoldenImageActive(false), Chip(0),
           RomMode(romNtype)
            {  FDataBits = 16;  }
    virtual ~X5FpaPersonality();

    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);
    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage()
                            { return true;  }
    virtual  bool      GoldenImageBooted();
    virtual  bool      GoldenImageActive();
    virtual  void      GoldenImageActive(bool state);
    virtual  int       CpldRevision();
    virtual  int       CpldType();

    virtual  unsigned int       StartSector() const ;
    virtual  unsigned int       Sectors() const ;
    virtual  unsigned int       GoldenImageStartSector() const;
    virtual  unsigned int       GoldenImageSectors() const;

protected:
    bool                FGoldenImageActive;
    bool                FGoldenImageBooted;
    int                 FCpldRevision;
    int                 FCpldType;
    class IX5FpaChip *  Chip;

    RomModes            RomMode;

    virtual void        OnFlashWrite_Start(int addr, int data);

    unsigned int        SectorBaseAddress(unsigned int sector);

    bool       SingleCellProgram(FpgaImage & Image);
    bool       BlockProgram(FpgaImage & Image);
    void       ReadCpldStatus();
    void       CreateChipImpl();

};

//==============================================================================
//  CLASS IX5FpaChip -- X5 FPA Flash Sub-Personality Interface
//==============================================================================

class IX5FpaChip
{
public:
    IX5FpaChip(X5FpaPersonality * owner)
        :  Owner(owner), FDeviceId(0), FProtected(false)
        {}
    virtual ~IX5FpaChip()
        {}

    virtual bool  Initialize() = 0;
    virtual void  EnterCFIQueryMode() = 0;
    virtual void  LeaveCFIQueryMode() = 0;

    void  Prom(int addr, int value)
            {  Owner->Prom(addr, value);  }
    int   Prom(int addr)
            {  return Owner->Prom(addr);  }
    int   DeviceId() const
        {  return FDeviceId;  }
    bool  IsProtected() const
       {  return  FProtected;  }

    virtual unsigned int SectorBaseAddress(unsigned int sector) = 0;

    virtual unsigned int ByteOrder(unsigned int data) = 0;
    virtual unsigned int StartSector() const = 0;
    virtual unsigned int Sectors() const = 0;

    virtual unsigned int GoldenImageStartSector() const = 0;
    virtual unsigned int GoldenImageSectors() const = 0;

    virtual void        OnFlashWrite_Start(int addr, int data) = 0;
    virtual void        OnSingleCellPgm_Start() = 0;
    virtual void        OnSingleCellPgm_End() = 0;
    virtual void        OnEraseSector_Start(unsigned int sector) = 0;
    virtual void        OnWriteBufferProgram(FpgaImageDatagram & ImageDg, unsigned int Addr, unsigned int wc)=0;

protected:
    X5FpaPersonality *  Owner;
    int                 FDeviceId;
    bool                FProtected;

    unsigned int        ByteSwap(unsigned int data);

};

//------------------------------------------------------------------------------
// IX5FpaChip::ByteSwap() - Byte swap 16 bit word
//------------------------------------------------------------------------------

inline unsigned int  IX5FpaChip::ByteSwap(unsigned int data)
{
    unsigned int LSB = data & 0x00FF;
    unsigned int MSB = (data & 0xFF00) >> 8;
    return (LSB<<8) | MSB;
}

//==============================================================================
//  CLASS IK7FpaChip -- K7 Flash Sub-Personality Interface
//==============================================================================

class IK7FpaChip
{
public:
    IK7FpaChip()
        :  FDeviceId(0), FProtected(false), FTotalSectors(0)
        {}
    virtual ~IK7FpaChip()
        {}

    virtual bool  Initialize() = 0;

    //  Page Program Mode
    virtual void  SetProgramPageMode(bool state) = 0;
    virtual void  SetPageAddress(unsigned int addr) = 0;
    virtual void  Enqueue32(unsigned int value) = 0;
    virtual void  WaitForPageComplete() = 0;
    virtual void  PageProgram() = 0;

    virtual unsigned int  Read32(unsigned int addr) = 0;
    virtual void          Write32(unsigned int addr, unsigned int value) = 0;

    virtual void  SetProtection() = 0;
    virtual void  ReleaseProtection() = 0;
    //
    //  Information
    int   DeviceId() const
        {  return FDeviceId;  }
    virtual unsigned int SectorSize(unsigned int sector) const = 0;
    virtual unsigned int DeviceSize()
        {  return  SectorSize(0)*Sectors();  }
    virtual unsigned int Sectors() const = 0;
    virtual unsigned int GoldenImageSectors() const = 0;
    virtual unsigned int StartSector() const = 0;
    virtual unsigned int GoldenImageStartSector() const = 0;

    bool  IsProtected() const
       {  return  FProtected;  }
    bool  IsHWProtected() const
       {  return  FHWProtected;  }

    virtual unsigned int ByteOrder(unsigned int data);

    virtual void         EraseSector(unsigned int sector) = 0;

    virtual unsigned int SectorBaseAddress(unsigned int sector) // normal implementation
                            {  return sector*SectorSize(sector);  }

protected:
    int                 FDeviceId;
    bool                FProtected;
    int                 FTotalSectors;
    bool                FHWProtected;

    unsigned int        ByteSwap(unsigned int data);

};

//------------------------------------------------------------------------------
// IK7FpaChip::ByteSwap() - Byte swap 16 bit word
//------------------------------------------------------------------------------

inline unsigned int  IK7FpaChip::ByteSwap(unsigned int data)
{
    unsigned int LSB = data & 0x00FF;
    unsigned int MSB = (data & 0xFF00) >> 8;
    return (LSB<<8) | MSB;
}

//==============================================================================
//  CLASS K7FpaPersonality -- Interface to FPA specific operations
//==============================================================================

class K7FpaPersonality : public IFpaPersonality
{
public:

    K7FpaPersonality(IK7FpaChip * chip)
        :  Chip(chip)
        {}
    virtual ~K7FpaPersonality()
        {}

    virtual int  FirmwareRevision() const
        {  return 0;  }
    virtual int  Id() const
        {  return Chip->DeviceId();  }

    //---------------------
    //  Personality Methods
    //
    //  ...Info
    virtual  int           DeviceId() const
                    {  return Chip->DeviceId();  }
    virtual  int           DeviceSize() const
                    {  return Chip->DeviceSize();  }
    virtual  int           SectorSize() const
                    {  return Chip->SectorSize(Chip->Sectors());  }
    virtual  unsigned int  Sectors() const
                    {  return Chip->Sectors();  }
    virtual  unsigned int  StartSector() const
                    {  return Chip->StartSector();  }
    virtual  unsigned int  GoldenImageStartSector() const
                    {  return Chip->GoldenImageStartSector();  }
    virtual  unsigned int  GoldenImageSectors() const
                    {  return Chip->GoldenImageSectors();  }

    virtual  unsigned int DataMask() const
                    {  return  0xFFFFFFFF;  }

    virtual  bool      IsProtected() const
                    {  return Chip->IsProtected();  }

    //  Are these needed?  X6 rewrites 'Program'
    virtual  bool      Flash(int /*addr*/, int /*value*/) {  return true;  }
    virtual  int       Flash(int /*addr*/) {  return 0;  }

    virtual  bool      Authenticate();
    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);

    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage() {  return false;  }
    virtual  bool      GoldenImageBooted() {  return false;  }
    virtual  bool      GoldenImageActive()  {  return false;  }
    virtual  void      GoldenImageActive(bool /*state*/) {}
    virtual  int       CpldRevision() {  return DeviceId();  }
    virtual  int       CpldType() {  return 0;  }

private:
    IK7FpaChip *     Chip;
    StopWatch         Rtc;
    int               Percent[2];

};

//==============================================================================
//  CLASS IXUFpaChip -- XU Flash Sub-Personality Interface
//==============================================================================

class IXUFpaChip
{
public:
    IXUFpaChip()
        :  FDeviceId(0), FProtected(false), FTotalSectors(0)
        {}
    virtual ~IXUFpaChip()
        {}

    virtual bool  Initialize() = 0;

    //  Page Program Mode
    virtual void  SetProgramPageMode(bool state) = 0;
    virtual void  SetPageAddress(unsigned int addr) = 0;
    virtual void  Enqueue32(unsigned int value) = 0;
    virtual void  WaitForPageComplete() = 0;
    virtual void  PageProgram() = 0;

    virtual unsigned int  Read32(unsigned int addr) = 0;
    virtual void          Write32(unsigned int addr, unsigned int value) = 0;

    virtual void  SetProtection() = 0;
    virtual void  ReleaseProtection() = 0;
    //
    //  Information
    int   DeviceId() const
        {  return FDeviceId;  }
    virtual unsigned int SectorSize(unsigned int sector) const = 0;
    virtual unsigned int DeviceSize()
        {  return  SectorSize(0)*Sectors();  }
    virtual unsigned int Sectors() const = 0;
    virtual unsigned int GoldenImageSectors() const = 0;
    virtual unsigned int StartSector() const = 0;
    virtual unsigned int GoldenImageStartSector() const = 0;

    bool  IsProtected() const
       {  return  FProtected;  }
    bool  IsHWProtected() const
       {  return  FHWProtected;  }

    virtual unsigned int ByteOrder(unsigned int data);

    virtual void         EraseSector(unsigned int sector) = 0;

    virtual unsigned int SectorBaseAddress(unsigned int sector) // normal implementation
                            {  return sector*SectorSize(sector);  }

protected:
    int                 FDeviceId;
    bool                FProtected;
    int                 FTotalSectors;
    bool                FHWProtected;

    unsigned int        ByteSwap(unsigned int data);

};

//------------------------------------------------------------------------------
// IXUFpaChip::ByteSwap() - Byte swap 16 bit word
//------------------------------------------------------------------------------

inline unsigned int  IXUFpaChip::ByteSwap(unsigned int data)
{
    unsigned int LSB = data & 0x00FF;
    unsigned int MSB = (data & 0xFF00) >> 8;
    return (LSB<<8) | MSB;
}

//==============================================================================
//  CLASS XUFpaPersonality -- Interface to FPA specific operations
//==============================================================================

class XUFpaPersonality : public IFpaPersonality
{
public:

    XUFpaPersonality(IXUFpaChip * chip)
        :  Chip(chip)
        {}
    virtual ~XUFpaPersonality()
        {}

    virtual int  FirmwareRevision() const
        {  return 0;  }
    virtual int  Id() const
        {  return Chip->DeviceId();  }

    //---------------------
    //  Personality Methods
    //
    //  ...Info
    virtual  int           DeviceId() const
                    {  return Chip->DeviceId();  }
    virtual  int           DeviceSize() const
                    {  return Chip->DeviceSize();  }
    virtual  int           SectorSize() const
                    {  return Chip->SectorSize(Chip->Sectors());  }
    virtual  unsigned int  Sectors() const
                    {  return Chip->Sectors();  }
    virtual  unsigned int  StartSector() const
                    {  return Chip->StartSector();  }
    virtual  unsigned int  GoldenImageStartSector() const
                    {  return Chip->GoldenImageStartSector();  }
    virtual  unsigned int  GoldenImageSectors() const
                    {  return Chip->GoldenImageSectors();  }

    virtual  unsigned int DataMask() const
                    {  return  0xFFFFFFFF;  }

    virtual  bool      IsProtected() const
                    {  return Chip->IsProtected();  }

    //  Are these needed?  X6 rewrites 'Program'
    virtual  bool      Flash(int /*addr*/, int /*value*/) {  return true;  }
    virtual  int       Flash(int /*addr*/) {  return 0;  }

    virtual  bool      Authenticate();
    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);

    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage() {  return false;  }
    virtual  bool      GoldenImageBooted() {  return false;  }
    virtual  bool      GoldenImageActive()  {  return false;  }
    virtual  void      GoldenImageActive(bool /*state*/) {}
    virtual  int       CpldRevision() {  return DeviceId();  }
    virtual  int       CpldType() {  return 0;  }

private:
    IXUFpaChip *     Chip;
    StopWatch         Rtc;
    int               Percent[2];

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
