// FpgaImage_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation. 
//  All Rights Reserved.
//


#include <FpgaImage_Mb.h>
#include <SRecord_Mb.h>
#include <BitFile_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

//==============================================================================
// CLASS FpgaImage  --  Fpga Image base class
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class FpgaImage
//------------------------------------------------------------------------------

FpgaImage::FpgaImage(int /*maxsize*/)
    : FFileName("")
{
}

//------------------------------------------------------------------------------
//  destructor for class FpgaImage
//------------------------------------------------------------------------------

FpgaImage::~FpgaImage()
{
}

//------------------------------------------------------------------------------
//  FpgaImage::FileName() - read file name
//------------------------------------------------------------------------------

std::string FpgaImage::FileName() const
{
    return FFileName;
}

//------------------------------------------------------------------------------
//  FpgaImage::FileName() - write file name
//------------------------------------------------------------------------------

void FpgaImage::FileName(const std::string & filename)
{
    FFileName = filename;
}

//------------------------------------------------------------------------------
//  FpgaImage::Size() - read buffer size
//------------------------------------------------------------------------------

size_t FpgaImage::Size() const
{
    return Buffer.size();
}

//------------------------------------------------------------------------------
//  FpgaImage::Size() - write size
//------------------------------------------------------------------------------

void FpgaImage::Size(unsigned int value)
{
    Buffer.resize(value);
}


//===========================================================================
// CLASS UniversalFpgaImage  --  Decode FPGA Image based on extension
//===========================================================================
//------------------------------------------------------------------------------
//  constructor for class UniversalFpgaImage
//------------------------------------------------------------------------------

UniversalFpgaImage::UniversalFpgaImage(int maxsize)
    : FpgaImage(maxsize), Type(fmtSRecord),
      Parser(new SRecord(1000))
{
}

//------------------------------------------------------------------------------
//  destructor for class UniversalFpgaImage
//------------------------------------------------------------------------------

UniversalFpgaImage::~UniversalFpgaImage()
{
    delete Parser;  Parser = 0;
}

//------------------------------------------------------------------------------
//  UniversalFpgaImage::Parse() - write size
//------------------------------------------------------------------------------

void UniversalFpgaImage::Parse()
{
    //
    //  Step 1:  Determine which parser we need to use. If we dont have it
    //           allocate the right one.
    PathSpec Path(FileName());

    if (Path.Ext()==".exo" || Path.Ext()==".EXO")
        {
        if (Type!=fmtSRecord)
            {
            // File is an EXO, parser was not. Switch.
            delete Parser;  Parser = 0;
            Parser = new SRecord;
            }
        }
    else  // File is a bitfile
        {
        if (Type==fmtSRecord)
            {
            // File is NOT an EXO, parser is for an EXO. Switch.
            delete Parser;  Parser = 0;
            Parser = new BitFile;;
            }
        }

    Parser->FileName(FileName());
    //   ...forward parser messages on
    Parser->OnProgress.SetEvent(this, &UniversalFpgaImage::HandleParseProgress);
    Parser->OnComplete.SetEvent(this, &UniversalFpgaImage::HandleParseComplete);
    Parser->OnError.SetEvent(this, &UniversalFpgaImage::HandleParseError);
    //
    //  Step 2: In order reduce memory needed, current parser buffer is small.
    //           Swap in the real buffer.
    //
    Parser->Buffer.swap(Buffer);
    //
    //  Step 3: Parse
    //
    Parser->Parse();
    //
    //  Step 4: Swap the parsed buffer back to Universal Buffer.
    //
    Parser->Buffer.swap(Buffer);

}

//------------------------------------------------------------------------------
//  UniversalFpgaImage::HandleParseProgress() --
//------------------------------------------------------------------------------

void UniversalFpgaImage::HandleParseProgress(ProcessProgressEvent & event)
{
    OnProgress.Execute(event);
}

//------------------------------------------------------------------------------
//  UniversalFpgaImage::HandleParseComplete() --
//------------------------------------------------------------------------------

void UniversalFpgaImage::HandleParseComplete(ProcessCompletionEvent & event)
{
    OnComplete.Execute(event);
}

//------------------------------------------------------------------------------
//  UniversalFpgaImage::HandleParseError() --
//------------------------------------------------------------------------------

void UniversalFpgaImage::HandleParseError(ProcessStatusEvent & event)
{
    OnError.Execute(event);
}

}  // namespace Innovative




