// FpgaImage_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef FpgaImage_MbH
#define FpgaImage_MbH

#include <vector>
#include <string>
#include <ProcessEvents_Mb.h>
#include <Datagram_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS FpgaImage  --  Fpga Image base class
//===========================================================================

class FpgaImage
{
    typedef std::vector<unsigned char> CharArray;

public:
    // CTor
    FpgaImage(int maxsize=(1<<22));
    virtual ~FpgaImage();

    // Methods
    virtual void            Parse() = 0;

    // Properties
    virtual std::string     FileName() const;
    virtual void            FileName(const std::string & filename);
    virtual size_t          Size() const;
    virtual void            Size(unsigned int value);

    //Defualt
    virtual std::string         Tag() const
        {  return std::string("Unknown");  }
    virtual std::string         Part() const
        {  return std::string("Unknown");  }
    virtual std::string         Date() const
        {  return std::string("Unknown");  }
    virtual std::string         Time() const
        {  return std::string("Unknown");  }

    // Events
    OpenWire::EventHandler<ProcessProgressEvent>      OnProgress;
    OpenWire::EventHandler<ProcessStatusEvent>        OnError;
    OpenWire::EventHandler<ProcessCompletionEvent>    OnComplete;

    // Data
    CharArray               Buffer;

protected:
    // Fields
    std::string             FFileName;

    // Methods
    void        Progress(int percent)
    {
        ProcessProgressEvent e(percent);
        OnProgress.Execute(e);
    }
    void        Complete(int result)
    {
        ProcessCompletionEvent e(result);
        OnComplete.Execute(e);
    }
    void        Error(const std::string & msg)
    {
        ProcessStatusEvent e(msg);
        OnError.Execute(e);
    }
};


//===========================================================================
// CLASS UniversalFpgaImage  --  Decode FPGA Image based on extension
//===========================================================================

class UniversalFpgaImage : public FpgaImage
{
public:
    enum ImageType { fmtSRecord, fmtBitFile };

    UniversalFpgaImage(int maxsize=(1<<22));
    virtual ~UniversalFpgaImage();

    // Methods
    virtual void            Parse();

    FpgaImage *             Image() const
                                 { return Parser; }
private:
    ImageType    Type;
    FpgaImage *  Parser;

    void  HandleParseProgress(ProcessProgressEvent & event);
    void  HandleParseComplete(ProcessCompletionEvent & event);
    void  HandleParseError(ProcessStatusEvent & event);

};

//===========================================================================
//  CLASS FpgaImageDatagram  -- Datagram access to Fpga Image
//===========================================================================

class FpgaImageDatagram : public IDatagram
{
public:
    //
    //  Member Functions
    FpgaImageDatagram(FpgaImage & image) :  FImage(image)
        {}

    FpgaImage &  Image()  {  return FImage;  };

protected:
    virtual size_t          DatagramSize() const;
    virtual int *           DatagramBasePtr() const;

private:
    //
    //  Member Data
    FpgaImage &  FImage;

};

//---------------------------------------------------------------------------
//  FpgaImageDatagram::DatagramSize() -- return size of buffer in INTs
//---------------------------------------------------------------------------

inline size_t  FpgaImageDatagram::DatagramSize() const
{
    return FImage.Size()/sizeof(int);
}

//---------------------------------------------------------------------------
//  FpgaImageDatagram::DatagramBasePtr() -- pointer to base of data
//---------------------------------------------------------------------------

inline int *  FpgaImageDatagram::DatagramBasePtr() const
{
    return reinterpret_cast<int *>(&FImage.Buffer[0]);
}

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

#endif


