// GuiXfr_Net.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef GuiXfr_NetH
#define GuiXfr_NetH

#include <vcclr.h>
#include <Framework\Net\ClrSupport_Net.h>
#include <string>
#include <vector>
#include <map>
#include <StringSupport_Mb.h>
#include <FileSupport_Mb.h>
#include <AccessDatagrams_Mb.h>
#include <Units_Mb.h>


//---------------------------------------------------------------------------
namespace Innovative
{
//===========================================================================
//  CLASS GuiColumn -- exact class for Column information
//===========================================================================

class GuiColumn
{
public:
    explicit GuiColumn(unsigned int column) : Column(column)
                {}
    unsigned int ColumnNumber() const {  return Column;  }

private:
    unsigned int Column;
};

//===========================================================================
//  CLASS GuiRow  -- exact class for Row information
//===========================================================================

class GuiRow
{
public:
    explicit GuiRow(unsigned int row) : Row(row)
                {}
    unsigned int RowNumber() const {  return Row;  }

private:
    unsigned int Row;
};


	using namespace System::Security::Cryptography;
	using namespace System::Collections::Generic;
	using namespace System::Text;


//---------------------------------------------------------------------------
//  StdString() -- Convert framework string into std::string
//---------------------------------------------------------------------------

std::string StdString(System::String ^ s)
{
	return OpenWire::MarshalString(s);
}

//---------------------------------------------------------------------------
//  FrameworkString() -- Convert std::string into framework string
//---------------------------------------------------------------------------

System::String ^ FrameworkString(const std::string & src)
{
	return gcnew System::String(src.c_str());
}

//---------------------------------------------------------------------------
//  StdVectorToManagedArray() -- Returns a managed cli::array of a reference type.
//---------------------------------------------------------------------------

cli::array<System::String ^ >^ StdVectorToManagedArray(const std::vector<std::string> &str) 
{
   cli::array< System::String ^ >^ local = gcnew cli::array< System::String ^ >(static_cast<int>(str.size()));

   for (size_t i = 0 ;i < str.size(); i++) 
		local[static_cast<int>(i)]  = FrameworkString(str[i]);

   return local;
}

//---------------------------------------------------------------------------
//  StdMapToManagedDict() -- Returns a managed cli::array of a reference type.
//---------------------------------------------------------------------------

Dictionary<System::String^, cli::array<System::String^ >^ >^ StdMapToManagedDict(const std::map<std::string, std::vector<std::string> > &maps)
{
   Dictionary<System::String^, cli::array<System::String^ >^ >^ local = gcnew Dictionary<System::String^, cli::array<System::String^ >^ >();
   
   std::map<std::string, std::vector<std::string> >::const_iterator it;
   for (it = maps.begin(); it != maps.end(); ++it)
	   local->Add(FrameworkString(it->first), StdVectorToManagedArray(it->second));

   return local;
}

//  This file contains implementations for "Generic" Operations on
//    a window control. A control may have an implementation for:
//
//  ToControl()        --- Move data into a control from variable
//  FromControl()      --- Move data into a variabled from control
//  ControlEnabled()   --- Enable/Disable/Read Enabled State
//  ControlVisible()   --- Enable/Disable/Read Enabled State
//  AddToControl()     --- Add to a "List-Like" Control
//  ClearControl()     --- Clear a control (removes items if a list)
//  ItemCount()        --- Current Items in a List-Like control
//
//  Other more specific helper functions may be defined here too
//
//==============================================================================
//  "Standard" Operation Normal Template Implementations
//==============================================================================
//  We define a template function for these operations, implementations are common.
//---------------------------------------------------------------------------
//  TEMPLATE function ControlEnabled() -- Enable/disable control
//---------------------------------------------------------------------------
template <class ContT>
void  ControlEnabled(ContT ^ control, bool state)
{
	control->Enabled = state;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlEnabled() -- Query enable state
//---------------------------------------------------------------------------
template <class ContT>
bool  ControlEnabled(ContT ^ control)
{
	return control->Enabled;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlVisible() -- Set Visible State of Control
//---------------------------------------------------------------------------
template <class ContT>
void  ControlVisible(ContT ^ control, bool state)
{
    control->Visible = state;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlVisible() -- Return visible state of a Control
//---------------------------------------------------------------------------
template <class ContT>
bool ControlVisible(ContT ^ control)
{
    return control->Visible;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ReadControl() -- Return Control value thru return code
//---------------------------------------------------------------------------
//  Usage:  int rtn = ReadControl<int>(Control)

template <class retT, class ContT>
retT ReadControl(ContT ^ Control)
{
    retT value;
    FromControl(Control, value);
    return value;
}

enum IIColor { IIBlack, IIRed, IIBlue, IIYellow, IIGreen, IIWhite };

//  These functions move between a local store or variable and a control
//==============================================================================
//  TextBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  ToControl() -- Add string to control
//---------------------------------------------------------------------------

void  AddToControl(System::Windows::Forms::TextBox^ control, const std::string & var)
{
    control->AppendText(FrameworkString(var));
    control->AppendText(FrameworkString("\r\n"));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

void  ClearControl(System::Windows::Forms::TextBox^ control)
{
	control->Clear();
}

//---------------------------------------------------------------------------
//  ToControl() -- Copy string to control
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox^ control, const std::string & var)
{
	control->Text = FrameworkString(var); 
}

//---------------------------------------------------------------------------
//  ToControl() -- Copy string to control
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox ^ control, Frequency_Units var, int precision=1)
{
	
	ToControl(control, var.Text(precision));
}


//---------------------------------------------------------------------------
//  FromControl() -- Copy control contents to string
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::TextBox^ control, std::string & var)
{
	var = StdString(control->Text);
}

//---------------------------------------------------------------------------
//  FromControl_X() -- Control >>  (TextBox | Frequency_Units )
//---------------------------------------------------------------------------
//  From Control" option 2...edit consists of number + units as string

void  FromControl(System::Windows::Forms::TextBox ^ control, Frequency_Units & var)
{
	std::string text;
	FromControl(control, text);
	Frequency_Units tmp(text);
	var = tmp;
}


//---------------------------------------------------------------------------
//  ToControl() -- Fill control with text version integer value
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox^ control, const int var, 
    const int base=10)
{
	control->Text = FrameworkString(IntToString(var, base));
}

//---------------------------------------------------------------------------
//  FromControl() -- Query value of control as integer
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::TextBox^ control, int & var)
{
	var = StringToInt(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Query value of control as integer
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::TextBox^ control, ii64 & var)
{
	var = StringToInt64(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Query value of control as integer
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::NumericUpDown^ control, unsigned int & var)
{
	var = StringToInt(StdString(control->Value.ToString()));
}


//---------------------------------------------------------------------------
//  FromControl() -- Query value of control as integer
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::NumericUpDown^ control, int & var)
{
	var = StringToInt(StdString(control->Value.ToString()));
}

//---------------------------------------------------------------------------
//  ToControl() -- Fill control
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::NumericUpDown^ control, unsigned int & var)
{
	control->Value = var;
}

//---------------------------------------------------------------------------
//  ToControl() -- Fill control
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::NumericUpDown^ control, int & var)
{
	control->Value = var;
}

//---------------------------------------------------------------------------
//  ToControl() -- Fill control with text equivalent of unsigned integer value
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox^ control, const unsigned int var, 
    const int base=10)
{
	control->Text = FrameworkString(IntToString(var, base));
}

//---------------------------------------------------------------------------
//  ToControl() -- Fill control with text equivalent of unsigned integer value
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox^ control, ii64 var, 
    const int base=10)
{
	control->Text = FrameworkString(Int64ToString(var, base));
}

//---------------------------------------------------------------------------
//  FromControl() -- Query value of control as unsigned integer
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::TextBox^ control, unsigned int & var)
{
	var = StringToInt(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TextBox | float)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox^ control, const float var, 
    const int precision=1)
{
	control->Text = FrameworkString(FloatToString(var, precision));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >> (TextBox | float)
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::TextBox^ control, float & var)
{
	var = StringToFloat(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TextBox | double)
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::TextBox^ control, double & var)
{
	var = StringToDouble(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TextBox | double)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::TextBox^ control, double var, int precision=1)
{
	control->Text = FrameworkString(FloatToString(var, precision));
}


//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (VScrollBar | int)
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::VScrollBar^ control, int & var)
{
	var = control->Value;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (VScrollBar | int)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::VScrollBar^ control, int var)
{
	control->Value = var;
}

//==============================================================================
//  TabPage / TabControl Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ShowTab() -- Hide/Show Tab on a TabControl
//---------------------------------------------------------------------------

void  ShowTab(System::Windows::Forms::TabPage^ tab, System::Windows::Forms::TabControl^ control, bool show /*, int idx=-1*/)
{
    if (!show)
		{
		control->TabPages->Remove(tab);
        }
    else
        {
        if (control->TabPages->Contains(tab))
            return;
        control->TabPages->Add(tab);
        //  This did not work to insert a tab at a position, even though the function exists.
        //    not important enough to worry about 
 		//if (idx==-1)
        //    control->TabPages->Add(tab);
        //else
        //    control->TabPages->Insert(idx, tab);
        }
}


//-------------------------------------------------------------------------------------
//  TabCompare() -- compare between Tabcontrol current tab index with any tabpage index
//--------------------------------------------------------------------------------------
bool TabCompare(System::Windows::Forms::TabControl ^MainTabControl,System::Windows::Forms::TabPage ^TabPage)
{
		if (MainTabControl->SelectedIndex == TabPage->TabIndex)
			return true;
		else 
			return false;

}
//==============================================================================
//  Label Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- Set control text from string
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::Label^ control, const std::string & var)
{
	control->Text = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabel  | double )
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::Label^ control, double var, int precision=1,
                const std::string & prefix = "",
                const std::string & suffix = "" )
{
	std::string txt(FloatToString(var, precision));
	txt = prefix + txt + suffix;
    ToControl(control, txt);
}

//---------------------------------------------------------------------------
//  FromControl() -- Set string from control text
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::Label^ control, std::string & var)
{
	var = StdString(control->Text);
}

//==============================================================================
//  ToolStripStatusLabel Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled string
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ToolStripItem^ control, 
    const std::string & var, const std::string & label="")
{
	std::string txt = label + var;
	control->Text = FrameworkString(txt);
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled integer
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ToolStripItem^ control, 
    const unsigned int & var, int base=10, const std::string & label="")
{
	std::string txt = label + std::string(": ") + IntToString(var, base);
	control->Text = FrameworkString(txt);
}
	
//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled unsigned integer
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ToolStripItem^ control, 
	const int & var, int base=10, const std::string & label="")
{
	std::string txt = label + std::string(": ") + IntToString(var, base);
	control->Text = FrameworkString(txt);
}	

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled unsigned integer
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ToolStripItem^ control, 
	const ii64 & var, int base=10, const std::string & label="")
{
	std::string txt = label + std::string(": ") + Int64ToString(var, base);
	control->Text = FrameworkString(txt);
}	

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled float
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ToolStripItem^ control, 
	const float & var, int precision=1, const std::string & label="")
{
	std::string txt = label + std::string(": ") + FloatToString(var, precision);
	control->Text = FrameworkString(txt);
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled float
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ToolStripItem^ control, 
	const double & var, int precision=1, const std::string & label="")
{
	std::string txt = label + std::string(": ") + FloatToString(var, precision);
	control->Text = FrameworkString(txt);
}

//==============================================================================
// TStatusStrip
//==============================================================================

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled string
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, int idx, 
    const std::string & var, const std::string & label="")
{
	std::string txt = label + var;
	control->Items[idx]->Text = FrameworkString(txt);
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled integer
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, int idx, 
    const unsigned int & var, int base=10, const std::string & label="")
{
	std::string txt = label + std::string(": ") + IntToString(var, base);
	control->Items[idx]->Text = FrameworkString(txt);
}
	
//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled unsigned integer
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, int idx, 
	const int & var, int base=10, const std::string & label="")
{
	std::string txt = label + std::string(": ") + IntToString(var, base);
	control->Items[idx]->Text = FrameworkString(txt);
}	

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled unsigned integer
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, int idx, 
	const ii64 & var, int base=10, const std::string & label="")
{
	std::string txt = label + std::string(": ") + Int64ToString(var, base);
	control->Items[idx]->Text = FrameworkString(txt);
}	

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled float
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, int idx, 
	const float & var, int precision=1, const std::string & label="")
{
	std::string txt = label + std::string(": ") + FloatToString(var, precision);
	control->Items[idx]->Text = FrameworkString(txt);
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with labeled float
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, int idx,
	double var, int precision=1, const std::string & label="")
{
	std::string txt = label + std::string(": ") + FloatToString(var, precision);
	control->Items[idx]->Text = FrameworkString(txt);
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control text with string vector
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::StatusStrip^ control, const std::vector<std::string>& var)
{
	for(size_t i = 0; i < var.size(); i++)
		ToControl(control, static_cast<int>(i), var[i]); 
}


//==============================================================================
//  ComboBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  ItemCount() -- Returns item count of the control
//---------------------------------------------------------------------------

size_t	ItemCount(System::Windows::Forms::ComboBox^ control)
{
	return control->Items->Count;
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with string text to control
//---------------------------------------------------------------------------

void  AddToControl(System::Windows::Forms::ComboBox^ control, const std::string & var)
{
	control->Items->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

void  AddToControl(System::Windows::Forms::ComboBox^ control, int var)
{
	control->Items->Add(FrameworkString(IntToString(var)));
}

//---------------------------------------------------------------------------
//  ToControl() -- Fill control with text equivalent of integer value
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ComboBox^ control, int var)
{
	if (ItemCount(control) && ItemCount(control) > static_cast<size_t>(var))
		control->SelectedIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Get integer value from control text
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::ComboBox^ control, int & var)
{
	if (control->Items->Count)
		var = control->SelectedIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- Fill control with text equivalent of unsigned integer value
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ComboBox^ control, unsigned int var)
{
	if (ItemCount(control) && ItemCount(control) > var)
		control->SelectedIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Get unsigned integer value from control text
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::ComboBox^ control, unsigned int & var)
{
	if (ItemCount(control) && ItemCount(control) > var)
		control->SelectedIndex = var;
}


//---------------------------------------------------------------------------
//  ToControl() -- Get bool state of the Radio control
//---------------------------------------------------------------------------
void  ToControl(System::Windows::Forms::RadioButton^ control, bool var)
{
	control->Checked = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Set bool state of the Radio control
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::RadioButton^ control, bool & var)
{
	var = control->Checked;
}

//---------------------------------------------------------------------------
//  ToControl() -- Get bool state of the Radio control
//---------------------------------------------------------------------------
void  ToControl(System::Windows::Forms::RadioButton^ control, char var)
{
	control->Checked = (var) ? true : false ;
}

//---------------------------------------------------------------------------
//  FromControl() -- Set bool state of the Radio control
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::RadioButton^ control, char & var)
{
	var = (control->Checked) ? 1 : 0;
}

//---------------------------------------------------------------------------
//  FromControl() -- Get string value from control text
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::ComboBox^ control, std::string & var)
{
	var = StdString(control->Text);
}

//---------------------------------------------------------------------------
//  FromControl() -- Erase contents of combo box
//---------------------------------------------------------------------------

void  ClearControl(System::Windows::Forms::ComboBox^ control)
{
	control->Items->Clear();
}
//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

void FillControl(System::Windows::Forms::ComboBox^ control, unsigned int N)
{
	ClearControl(control);
    for (unsigned int i = 0; i < N; ++i)
        AddToControl(control, IntToString(i));
	ControlEnabled(control, N ? true : false);
	if (N) 
	    ToControl(control, 0);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with List of strings
//---------------------------------------------------------------------------
void FillControl(System::Windows::Forms::ComboBox^ control, const std::vector<std::string>& var)
{
	ClearControl(control);
    for(size_t i = 0; i < var.size(); i++)
		control->Items->Add(FrameworkString(var[i]));
	ControlEnabled(control, true);
	
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

void FillControl(System::Windows::Forms::ComboBox^ control, unsigned int N, const std::string & prefix, const std::string suffix = std::string())
{
	ClearControl(control);
    for (unsigned int i = 0; i < N; ++i)
		{
		std::string text(prefix + IntToString(i) + suffix);
        AddToControl(control, text);
		}
	ControlEnabled(control, N ? true : false);
}

//==============================================================================
//  CheckBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  ToControl() -- Set control state from boolean
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::CheckBox^ control, bool var)
{
	control->Checked = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Get control state from boolean
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::CheckBox^ control, bool & var)
{
	var = control->Checked;
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control state from char
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::CheckBox^ control, char var)
{
    control->Checked = var ? true : false;
}

//---------------------------------------------------------------------------
//  FromControl() -- Set char state from control
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::CheckBox^ control, char & var)
{
	var = control->Checked;
}

//==============================================================================
//  CheckListBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  ItemCount() -- Returns item count of the control
//---------------------------------------------------------------------------

size_t	ItemCount(System::Windows::Forms::CheckedListBox^ control)
{
	return control->Items->Count;
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with specified label to control
//---------------------------------------------------------------------------

void  AddToControl(System::Windows::Forms::CheckedListBox^ control, const std::string & var)
{
    control->Items->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

void  ClearControl(System::Windows::Forms::CheckedListBox^ control)
{
	control->Items->Clear();
}

//---------------------------------------------------------------------------
//  ToControl() -- Set control items to state vector
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::CheckedListBox^ control, const std::vector<char> & var)
{
    if (ItemCount(control) == 0)
        return;

    size_t reps;
    if (ItemCount(control) < var.size())
        reps = ItemCount(control);
    else
        reps = var.size();
        
	for (size_t i = 0; i < reps; ++i)
		control->SetItemChecked(static_cast<int>(i), var[i] ? true : false);
}

//---------------------------------------------------------------------------
//  FromControl() -- Set state vector from control items
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::CheckedListBox^ control, std::vector<char> &  var)
{
    if (ItemCount(control) == 0)
       return;

    size_t reps;
    if (ItemCount(control) < var.size())
        reps = ItemCount(control);
    else
        reps = var.size();

    for (size_t i = 0; i < reps; ++i)
		var[i] = control->GetItemChecked(static_cast<int>(i));	
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

void FillControl(System::Windows::Forms::CheckedListBox^ control, unsigned int N, const std::string & prefix, const std::string suffix = std::string())
{
	ClearControl(control);
    for (unsigned int i = 0; i < N; ++i)
		{
		std::string text(prefix + IntToString(i) + suffix);
        AddToControl(control, text);
		}
	ControlEnabled(control, N ? true : false);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

void FillControl(System::Windows::Forms::CheckedListBox^ control, const std::vector<std::string> & list)
{
	ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, list[i]);
}

//==============================================================================
//  ListBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  ItemCount() -- return count of items
//---------------------------------------------------------------------------

size_t  ItemCount(System::Windows::Forms::ListBox^ control)
{
    return control->Items->Count;
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add string to control
//---------------------------------------------------------------------------

void  AddToControl(System::Windows::Forms::ListBox^ control, const std::string & var)
{
	control->Items->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

void  ClearControl(System::Windows::Forms::ListBox^ control)
{
	control->Items->Clear();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control ( ListBox | vector<string> ) - fill from a list of strings
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::ListBox^ control, std::vector<std::string> list)
{
	ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, list[i]);
}

//---------------------------------------------------------------------------
//  FromControl() -- get the selected item to a variable.
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::ListBox^ control, int & var)
{
    var = control->SelectedIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- set the selected itemto the control.
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::ListBox^ control, int var)
{
	if (ItemCount(control) && ItemCount(control) > static_cast<size_t>(var))
		control->SelectedIndex = var;
}
  
//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

void  AddToControl(System::Windows::Forms::ListBox^ control, int var, int base=10)
{
	AddToControl(control, IntToString(var, base));
}


//---------------------------------------------------------------------------
//  FromControl() -- get the selected item to a variable.
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::ListBox^ control, unsigned int & var)
{
    var = control->SelectedIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- set the selected itemto the control.
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::ListBox^ control, unsigned int var)
{
	if (ItemCount(control) && ItemCount(control) > var)
		control->SelectedIndex = var;
}  
//---------------------------------------------------------------------------
//  ToControl() -- set the selected itemto the control.
//---------------------------------------------------------------------------
void  ToControl(System::Windows::Forms::ListBox^ control, bool var)
{
	if (ItemCount(control))
		control->SelectedIndex = var ? 1 : 0;
}
//---------------------------------------------------------------------------
//  FromControl() -- get the selected item to a variable.
//---------------------------------------------------------------------------
void FromControl(System::Windows::Forms::ListBox^ control, bool & var)
{
	var = control->SelectedIndex ? true : false;
}  

//==============================================================================
// ProgressBar
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- set the selected itemto the control.
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::ProgressBar^ control, int var)
{
    control->Value = var;
}

//==============================================================================
//  DataGridView Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  SetDimensions() -- Set dimensions of grid control
//---------------------------------------------------------------------------

void  SetDimensions(System::Windows::Forms::DataGridView^ control, 
    const int cols, const int rows)
{
    control->ColumnCount = cols;
	control->RowCount = rows;
}

//---------------------------------------------------------------------------
//  ToColumnHeading() -- Set column heading to string text
//---------------------------------------------------------------------------

void  ToColumnHeading(System::Windows::Forms::DataGridView^ control, 
    const int col, const std::string & var)
{
    control->Columns[col]->HeaderCell->Value = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  ToRowHeading() -- Set row heading to string text
//---------------------------------------------------------------------------

void  ToRowHeading(System::Windows::Forms::DataGridView^ control, 
    const int row, const std::string & var)
{
    control->Rows[row]->HeaderCell->Value = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (DataGridCell | string)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, const std::string & var)
{
    control[col, row]->Value = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (DataGridCell | float)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, const float & var, const int precision=1)
{
	std::string msg = FloatToString(var, precision);
	control[col, row]->Value = FrameworkString(msg);
}


//---------------------------------------------------------------------------
//  ToControl() -- >> Control (DataGridCell | ii64)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, const ii64 & var, const int base=10)
{
	std::string msg = Int64ToString(var, base);
	control[col, row]->Value = FrameworkString(msg);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (DataGridCell | int)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, const int & var, const int base=10)
{
	std::string msg = IntToString(var, base);
	control[col, row]->Value = FrameworkString(msg);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (DataGridCell | int)
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, const unsigned int & var, const int base=10)
{
	std::string msg = IntToString(var, base);
	control[col, row]->Value = FrameworkString(msg);
}

//---------------------------------------------------------------------------
//  GetColor() -- >>
//---------------------------------------------------------------------------

inline System::Drawing::Color GetColor(IIColor color)
{
	switch (color)
	{
	case IIBlack:   return System::Drawing::Color::Black;		 break;
	case IIBlue:    return System::Drawing::Color::Blue;		 break;
	case IIWhite:   return System::Drawing::Color::White;		 break;
	case IIRed:     return System::Drawing::Color::Red;			 break;
	case IIGreen:   return System::Drawing::Color::LightGreen;   break;
	case IIYellow:  return System::Drawing::Color::Yellow;		 break;
	default:        return System::Drawing::Color::Black;
	}
}

//---------------------------------------------------------------------------
//  SetCell_BackgroundColor() -- >> Control cell backColor
//---------------------------------------------------------------------------

void  SetCell_BackgroundColor(System::Windows::Forms::DataGridView^ control,
    const int col, const int row, IIColor color)
{
	System::Drawing::Color VcColor = GetColor(color);
	control[col, row]->Style->BackColor = VcColor;
}

//---------------------------------------------------------------------------
//  SetCell_ForeColor() -- >> Control cell ForeColor
//---------------------------------------------------------------------------

void  SetCell_ForegroundColor(System::Windows::Forms::DataGridView^ control,
    const int col, const int row, IIColor color)
{
	System::Drawing::Color VcColor = GetColor(color);
	control[col, row]->Style->ForeColor = VcColor;
}

//---------------------------------------------------------------------------
//  SetCell_FontBold() -- >> Control cell FontBold
//---------------------------------------------------------------------------

void  SetCell_FontBold(System::Windows::Forms::DataGridView^ control,
    const int col, const int row)
{
    control[col, row]->Style->Font = gcnew System::Drawing::Font(control->Font,
                                           System::Drawing::FontStyle::Bold);
}

//---------------------------------------------------------------------------
//  SetCell_FontItalic() -- >> Control cell FontItalic
//---------------------------------------------------------------------------

void  SetCell_FontItalic(System::Windows::Forms::DataGridView^ control,
    const int col, const int row)
{
    control[col, row]->Style->Font = gcnew System::Drawing::Font(control->Font,
                                           System::Drawing::FontStyle::Italic);
}

//---------------------------------------------------------------------------
//  SetCell_FontUnderline() -- >> Control cell FontUnderline
//---------------------------------------------------------------------------

void  SetCell_FontUnderline(System::Windows::Forms::DataGridView^ control,
    const int col, const int row)
{
    control[col, row]->Style->Font = gcnew System::Drawing::Font(control->Font,
                                           System::Drawing::FontStyle::Underline);
}

//---------------------------------------------------------------------------
//  SetCell_Font() -- >> Control cell SetCell_Font
//---------------------------------------------------------------------------

void  SetCell_Font(System::Windows::Forms::DataGridView^ control,
    const int col, const int row, System::Drawing::Font^ font)
{
    control[col, row]->Style->Font = font;
}

//---------------------------------------------------------------------------
//  SetCell_SetAlignment() -- >> 
//---------------------------------------------------------------------------

void  SetCell_Alignment(System::Windows::Forms::DataGridView^ control,
    const int col, const int row, System::Windows::Forms::DataGridViewContentAlignment alignment)
{
    control[col, row]->Style->Alignment = alignment;
}

//---------------------------------------------------------------------------
//  SetCell_CenterAlignment() -- >> 
//---------------------------------------------------------------------------

void  SetCell_CenterAlignment(System::Windows::Forms::DataGridView^ control,
    const int col, const int row)
{
    control[col, row]->Style->Alignment = System::Windows::Forms::DataGridViewContentAlignment::MiddleCenter;
}

//---------------------------------------------------------------------------
//  SetCell_LeftAlignment() -- >> C
//---------------------------------------------------------------------------

void  SetCell_LeftAlignment(System::Windows::Forms::DataGridView^ control,
    const int col, const int row)
{
    control[col, row]->Style->Alignment = System::Windows::Forms::DataGridViewContentAlignment::MiddleLeft;
}

//---------------------------------------------------------------------------
//  SetCell_CenterAlignment() -- >> 
//---------------------------------------------------------------------------

void  SetCell_RightAlignment(System::Windows::Forms::DataGridView^ control,
    const int col, const int row)
{
    control[col, row]->Style->Alignment = System::Windows::Forms::DataGridViewContentAlignment::MiddleRight;
}


//---------------------------------------------------------------------------
//  FromControl() -- >> Control (DataGridCell | string)
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::DataGridView^ control, const int col, 
    const int row, float & var)
{
	System::String ^ name = control[col,row]->Value->ToString();
	var = StringToFloat(StdString(name));
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (DataGridCell | string)
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, int & var)
{
	System::String ^ name = control[col,row]->Value->ToString();
	var = StringToInt(StdString(name));
}

//---------------------------------------------------------------------------
template <class T>
void  ToControl(System::Windows::Forms::DataGridView^ control, int column, const std::vector<T> & var, int arg)
{
	size_t var_size = var.size();  // int used here because Count is an int
	unsigned int rows = DataRows(control);
	for (unsigned int i = 0; i < rows; ++i)
		if (i < var_size)
			ToControl(control, column, i, var[i], arg);
		else
			ToControl(control, column, i, static_cast<T>(0), arg);
}
template <class T>
void FromControl(System::Windows::Forms::DataGridView^ control, int column, std::vector<T> & var)
{
	size_t var_size = var.size();   // int used here because Count is an int
	unsigned int rows = DataRows(control);
	for (unsigned int i = 0; i < rows; ++i)
		if (i < var_size)
			FromControl(control, column, i, var[i]);
}
//  DataRows() -- Get size of grid data region
//---------------------------------------------------------------------------

int  DataRows(System::Windows::Forms::DataGridView^ control)
{
    return control->RowCount;
}

//---------------------------------------------------------------------------
//  DataCols() -- Get size of grid data region
//---------------------------------------------------------------------------

int  DataCols(System::Windows::Forms::DataGridView^ control)
{
    return control->ColumnCount;
}


//-----------------------------------------------------------------------------
//  FillGrid()  --  Fill grid with data from region
//-----------------------------------------------------------------------------

void  FillGrid(System::Windows::Forms::DataGridView^ control, 
               unsigned int * Region, size_t offset, int base)
{
    if (Region==0)
        return;

    for (int row = 0; row<DataRows(control); row++)
        {
        //  Load Row Header (address)
        ToRowHeading(control, row, IntToString(static_cast<int>(offset), base));
        for (int col = 0; col<DataCols(control); col++)
            {
            ToControl(control, col, row, Region[offset], base);
            offset++;
            }
        }
   for (int col = 0; col<DataCols(control); col++)
        {
        //  Load Column Header (offset)
        ToColumnHeading(control, col, IntToString(col, base));
        }

}

//---------------------------------------------------------------------------
//  HideColumn() -- hide the column
//---------------------------------------------------------------------------

void  HideColumn(System::Windows::Forms::DataGridView^ control,const unsigned int Idx)
{
	if(control->ColumnCount > static_cast<int>(Idx))
	  control->Columns[Idx]->Visible = false;
}


//-----------------------------------------------------------------------------
//  SizeGrid()  --  Resize Grid to match control size
//-----------------------------------------------------------------------------
//   Adjust row and column count to fit inside control's dimensions

void  SizeGrid(System::Windows::Forms::DataGridView^ control)
{
    if (!control->RowCount)
        return;

    int CellWidth = control->Rows[0]->Cells[0]->Size.Width;
    int CellHeight = control->Rows[0]->Cells[0]->Size.Height;

    int Columns = control->Width / CellWidth;
    int Rows = control->Height / CellHeight;
    
    SetDimensions(control, Columns, Rows);
}

//-----------------------------------------------------------------------------
//  SizeGridWidth()  --  Resize Grid to match control width & fixed total cells
//-----------------------------------------------------------------------------
//   Adjust column count to fit inside control's dimensions
//   Adjust rows to fit entire region in it (user scrolls down to see all)

void  SizeGridWidth(System::Windows::Forms::DataGridView^ control, const SafeMemoryRange & Region)
{
    if (!control->RowCount)
        return;

    int CellWidth = control->Rows[0]->Cells[0]->Size.Width;
//    int CellHeight = control->DefaultRowHeight + control->GridLineWidth;

    int Columns = control->Width / CellWidth;
    int DataColumns = Columns;
    int Rows = static_cast<int>(Region.size() / (DataColumns));
	if (Rows<1)        // Grid needs at least 1 fixed and 1 data row always
		Rows = 1;
    if (Rows*DataColumns < (int)Region.size())  // If a row is part-filled, grow by 1 to see it
        Rows += 1;

    SetDimensions(control, DataColumns, Rows);
}

//-----------------------------------------------------------------------------
//  FillGrid()  --  Fill grid with data from region
//-----------------------------------------------------------------------------

void FillGrid(System::Windows::Forms::DataGridView^ control,
              const SafeMemoryRange & Region, size_t offset, int base=16, bool skipColHdr=false, bool skipRowHdr=false)
{
    for (int row = 0; row<DataRows(control); row++)
        {
        //  Load Row Header (address)
        if (skipRowHdr==false)
			ToRowHeading(control, row, IntToString(static_cast<int>(offset), base));
        for (int col = 0; col<DataCols(control); col++)
            {
            ToControl(control, col, row, Region[offset], base);
            offset++;
            }
        }
   for (int col = 0; col<DataCols(control); col++)
        {
        //  Load Column Header (offset)
        if (skipColHdr==false)
			ToColumnHeading(control, col, IntToString(col, base));
        }

}

//-----------------------------------------------------------------------------
//  FillGrid_NoCH()  --  Fill grid with data from region - no column header
//-----------------------------------------------------------------------------

void FillGrid_NoCH(System::Windows::Forms::DataGridView^ control,
                   const SafeMemoryRange & Region, size_t offset, int base=16)
{
	FillGrid(control, Region, offset, base, true);
}

//==============================================================================
//  Dialogs
//==============================================================================


//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter (result in 1st arg str)
//---------------------------------------------------------------------------

bool SelectFileDialog(std::string & file, const std::string & filter)
{
    System::Windows::Forms::OpenFileDialog^ Dialog = gcnew System::Windows::Forms::OpenFileDialog;
    if (file.size())
        {
        PathSpec path(file);
        Dialog->InitialDirectory = FrameworkString(path.Dir());
        }
	else
        Dialog->InitialDirectory = ".";

    Dialog->Filter = FrameworkString(filter);
    Dialog->FilterIndex = 0;
    Dialog->RestoreDirectory = true;
	Dialog->CheckFileExists = false;
	Dialog->Title = "Select File";

    if (Dialog->ShowDialog() != System::Windows::Forms::DialogResult::OK)
        return false;

	file = StdString(Dialog->FileName);
    return true;
}

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------
bool SelectFileDialog(System::Windows::Forms::TextBox^ control, const std::string & filter)
{
    std::string file;
    FromControl(control, file);
    bool rtn = SelectFileDialog(file, filter);
    if (rtn)
		ToControl(control, file);
	return rtn;
}

//---------------------------------------------------------------------------
//  WarningDialog() -- Display message dialog
//---------------------------------------------------------------------------

void WarningDialog(const std::string & msg, const std::string & caption="")
{
    // Displays the MessageBox.
    System::Windows::Forms::MessageBox::Show(FrameworkString(msg), FrameworkString(caption));
}

//---------------------------------------------------------------------------
//  SaveDialog() -- Save file
//---------------------------------------------------------------------------

bool SaveDialog(System::Windows::Forms::TextBox^ control, const std::string & filter)
{
	System::Windows::Forms::SaveFileDialog^ Dialog = gcnew System::Windows::Forms::SaveFileDialog;

	std::string file=StdString(control->Text);
	
    if (file.size())
        {
        PathSpec path(file);
        Dialog->InitialDirectory = FrameworkString(path.Dir());
        }
	else
		Dialog->InitialDirectory = ".";

    Dialog->Filter = FrameworkString(filter);
    Dialog->FilterIndex = 0;
    Dialog->RestoreDirectory = true;
	Dialog->CheckFileExists = false;
	Dialog->Title = "Select File";

    if (Dialog->ShowDialog() != System::Windows::Forms::DialogResult::OK)
        return false;

	file = StdString(Dialog->FileName);
    return true;
}

//==============================================================================
//  Buttons
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- Copy string to control
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::Button^ control, const std::string & var)
{
	control->Text = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Copy control contents to string
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::Button^ control, std::string & var)
{
	var = StdString(control->Text);
}

//==============================================================================
//  Forms
//==============================================================================
//---------------------------------------------------------------------------
//  FormTitle() -- Make string the new title of the form
//---------------------------------------------------------------------------

void  FormTitle(System::Windows::Forms::Form^ form, const std::string & var)
{
	form->Text = FrameworkString(var);
}

//==============================================================================
//  GroupBoxes
//==============================================================================

//---------------------------------------------------------------------------
//  ToControl() -- Set control text from String
//---------------------------------------------------------------------------

void  ToControl(System::Windows::Forms::GroupBox^ control, std::string &var)
{
	control->Text=FrameworkString(var); 
}

//==============================================================================
//  System Operations
//==============================================================================
//---------------------------------------------------------------------------
//  Terminate() -- Quit app
//---------------------------------------------------------------------------

void Terminate()
{
    System::Windows::Forms::Application::Exit();
}


//---------------------------------------------------------------------------
//  GetHexMd5Hash(std::string str) --Getting hash number
//---------------------------------------------------------------------------
//  

std::string GetHexMd5Hash(std::string str)
 {
	System::String^ gcStr = FrameworkString(str.c_str());
	// Get MD5 128-bit hash from the password
	MD5CryptoServiceProvider^ md5provider = gcnew MD5CryptoServiceProvider();
	cli::array<System::Byte>^ byteArrayHash = md5provider->ComputeHash(System::Text::Encoding::ASCII->GetBytes(gcStr->ToCharArray()));
	// Convert to Hex 
    System::Text::StringBuilder output;
	for each (System::Byte b in byteArrayHash)
	{
		output.AppendFormat("{0:X}", b);
	}
	return StdString(output.ToString());
 }


//==============================================================================
//  Advanced Control Arrangement
//==============================================================================
//---------------------------------------------------------------------------
//  PlaceControl() -- Dynamically position a control in a row/column arrangement
//---------------------------------------------------------------------------

template <class ContT>
void  PlaceControl(ContT ^ Ctl,
                   int row, int total_rows,
                   int col, int total_cols,
                   int margin=5 )
{
    if (total_cols==0 || total_rows==0)
        return;
    if (col >= total_cols || row >= total_rows)
        return;

    Ctl->Top  = margin + row*Ctl->Parent->Height/total_rows;
    Ctl->Left = margin + col*Ctl->Parent->Width/total_cols;
}

//---------------------------------------------------------------------------
//  FitControl() -- Resize text box to fit in enclosing grid region
//---------------------------------------------------------------------------
//  Fit control in a virtual grid box, subdividing parent into row by col regions

template <class ContT>
void  FitControl(ContT ^ Ctl, int rows=1, int cols=1, int margin=5)
{
    const int MinFontSize = 7;
    bool  Increasing(false);

    while (1==1)
        {
        int col_width = Ctl->Parent->Width/cols;
        int row_height = Ctl->Parent->Height/rows;
        //  find which region control is in
        int col =  Ctl->Left/col_width;
        int row =  Ctl->Top/row_height;
        //  find offset into region
        int col_left = Ctl->Left - col*col_width;
        int row_top = Ctl->Top - row*row_height;

        int width_fit = col_width-col_left-Ctl->Width;
        int height_fit = row_height-row_top-Ctl->Height;
        if (width_fit<=0 || height_fit<=0)
            {
			float current_size = Ctl->Font->Size;
            current_size -= 1;      // reduce to fit
            if (Ctl->Font->Size<MinFontSize)
                break;

            Ctl->Font = gcnew System::Drawing::Font(Ctl->Font->Name, 
                current_size, Ctl->Font->Style, Ctl->Font->Unit);

            if (Increasing)
                break;   // We overshot the size while increasing, so stop
            }
        else if (width_fit>margin && height_fit>margin)
            {
            //  Misfit enought to make correction
			float current_size = Ctl->Font->Size;
            current_size += 1;      // expand to fit
            if (Ctl->Font->Size<MinFontSize)
                break;

            Ctl->Font = gcnew System::Drawing::Font(Ctl->Font->Name, 
                current_size, Ctl->Font->Style, Ctl->Font->Unit);

            Increasing = true;
            }
        else
            break;
        }
}


}  // end namespace Innovative

#endif
