//---------------------------------------------------------------------------

#ifndef GuiXfr_QtH
#define GuiXfr_QtH

#include <string>
#include <vector>
#include <utility>
#include <sstream>
#include <StringList_Mb.h>
#include <StringSupport_Mb.h>
#include <StringList_Mb.h>
#include <FileSupport_Mb.h>
#include <QApplication>
#include <QLineEdit>
#include <QTextEdit>
#include <QTextBrowser>
#include <QComboBox>
#include <QButtonGroup>
#include <QCheckBox>
#include <QListWidget>
#include <QListWidgetItem>
#include <QTableWidget>
#include <QProgressBar>
#include <QFileDialog>
#include <QErrorMessage>
#include <QTabWidget>
#include <QPushButton>
#include <QStatusBar>
#include <QDoubleSpinBox>
#include <QLabel>
#include <memory>
#include <QGroupBox>
#include <QCheckBox>
#include <QCryptographicHash>
#include <QMainWindow>
#include <QMessageBox>
#include <QHeaderView>
#include <QLayout>
#include <qinputdialog.h>
#include <Units_Mb.h>
#include <QStyleFactory>


//---------------------------------------------------------------------------
namespace Innovative
{

enum IIColor { IIBlack, IIRed, IIBlue, IIYellow, IIGreen, IIWhite };

//---------------------------------------------------------------------------
//  GetColor() -- >>
//---------------------------------------------------------------------------

inline QColor GetColor(IIColor color)
{
    switch(color)
    {
        case IIBlack:   return Qt::black;                 break;
        case IIBlue:    return QColor(84, 80, 255);       break;
        case IIWhite:   return Qt::white;                 break;
        case IIRed:     return QColor(227, 67, 67);       break;
        case IIGreen:   return QColor(22, 173, 40);       break;
        case IIYellow:  return QColor(250, 255, 120);     break;
        default:        return Qt::black;
    }
}

//===========================================================================
//  CLASS GuiColumn -- exact class for Column information
//===========================================================================

class GuiColumn
{
public:
    explicit GuiColumn(unsigned int column) : Column(column)
                {}
    unsigned int ColumnNumber() const {  return Column;  }

private:
    unsigned int Column;
};

//===========================================================================
//  CLASS GuiRow  -- exact class for Row information
//===========================================================================

class GuiRow
{
public:
    explicit GuiRow(unsigned int row) : Row(row)
                {}
    unsigned int RowNumber() const {  return Row;  }

private:
    unsigned int Row;
};

//---------------------------------------------------------------------------
//  CompileTimeQtVersion() -- Returns Qt version with which code is compiled
//---------------------------------------------------------------------------

inline std::string CompileTimeQtVersion()
{
    return QT_VERSION_STR;
}

//---------------------------------------------------------------------------
//  RunTimeQtVersion() -- Returns Qt run time version
//---------------------------------------------------------------------------

inline std::string RunTimeQtVersion()
{
    return qVersion();
}

//---------------------------------------------------------------------------
//  qtVersionMatch() -- Do run time and compile time versions match
//---------------------------------------------------------------------------

inline bool QtVersionMatch()
{
  return  CompileTimeQtVersion() == RunTimeQtVersion();
}

//---------------------------------------------------------------------------
//  StdString() -- Convert framework string into std::string
//---------------------------------------------------------------------------

inline std::string StdString(const QString & src)
{
    return std::string(src.toStdString());
}

//---------------------------------------------------------------------------
//  FrameworkString() -- Convert std::string into framework string
//---------------------------------------------------------------------------

inline QString FrameworkString(const std::string & src)
{
    return QString::fromStdString(src);
}

//  This file contains implementations for "Generic" Operations on
//    a window control. A control may have an implementation for:
//
//  ToControl()        --- Move data into a control from variable
//  FromControl()      --- Move data into a variabled from control
//  ControlEnabled()   --- Enable/Disable/Read Enabled State
//  ControlVisible()   --- Enable/Disable/Read Enabled State
//  AddToControl()     --- Add to a "List-Like" Control
//  ClearControl()     --- Clear a control (removes items if a list)
//  ItemCount()        --- Current Items in a List-Like control
//
//  Other more specific helper functions may be defined here too
//
//==============================================================================
//  "Standard" Operation Normal Template Implementations
//==============================================================================
//  We define a template function for these operations, implementations are common.
//---------------------------------------------------------------------------
//  TEMPLATE function ControlEnabled() -- Enable/disable control
//---------------------------------------------------------------------------
template <class ContT>
void  ControlEnabled(ContT * control, bool state)
{
    control->setEnabled(state);
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlEnabled() -- Query enable state
//---------------------------------------------------------------------------
template <class ContT>
bool  ControlEnabled(ContT * control)
{
    return control->enabled();
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlVisible() -- Set Visible State of Control
//---------------------------------------------------------------------------
template <class ContT>
void  ControlVisible(ContT * control, bool state)
{
    control->setVisible(state);
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlVisible() -- Return visible state of a Control
//---------------------------------------------------------------------------
template <class ContT>
bool  ControlVisible(ContT * control)
{
    return control->visible();
}


//---------------------------------------------------------------------------
//  TEMPLATE function ClearControl() -- Erases contents of control
//---------------------------------------------------------------------------
template <class ContT>
bool ControlClear(ContT * control)
{
    return control->Clear();
}

//---------------------------------------------------------------------------
//  TEMPLATE function ToCaption() -- >> sets caption of a control
//---------------------------------------------------------------------------

template <class ContT>
void  ToCaption(ContT * control, const std::string & var)
{
    control->setWindowTitle(FrameworkString(var));
}


//---------------------------------------------------------------------------
//  TEMPLATE Blocked() -- >> Used in conjunction with WhileBlocked function
//---------------------------------------------------------------------------

template<class T>
struct Blocked
{
    Blocked(T *control) : _control(control), _oldstate(control->blockSignals(true))
    {}
    ~Blocked()
    { _control->blockSignals(_oldstate);  }

    T* operator->()
    { return _control; }

    T* Widget()
    { return _control; }

    T    *_control;
    bool  _oldstate;

};

//---------------------------------------------------------------------------
//  TEMPLATE function WhileBlocked() -- >> turns of signals for a given control
//---------------------------------------------------------------------------
template<class T>
inline Blocked<T> WhileBlocked(T *control)
{
    return Blocked<T>(control);
}

//==============================================================================
//  QMainWindow Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  FormTitle() -- >> Form (QMainWindow | string)
//---------------------------------------------------------------------------

inline void  FormTitle(QMainWindow * form, const std::string & var)
{
    ToCaption(form, var);
}


//==============================================================================
//  QLineEdit Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | string)
//---------------------------------------------------------------------------


inline void  ToControl(QLineEdit * control, const std::string & var)
{
    control->setText(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | string)
//---------------------------------------------------------------------------

inline void  FromControl(QLineEdit * control, std::string & var)
{
    var = StdString(control->text());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | int)
//---------------------------------------------------------------------------

inline void  ToControl(QLineEdit * control, int var, int base=10)
{
    control->setText(FrameworkString(IntToString(var, base)));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(QLineEdit * control, unsigned int var, int base=10)
{
    control->setText(FrameworkString(IntToString(var, base)));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | ii64)
//---------------------------------------------------------------------------

inline void  ToControl(QLineEdit * control, ii64 var, int base=10)
{
    control->setText(FrameworkString(Int64ToString(var, base)));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | s_ii64)
//---------------------------------------------------------------------------

inline void  ToControl(QLineEdit * control, s_ii64 var, int base=10)
{
    control->setText(FrameworkString(Int64ToString(var, base)));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | Frequency_Units )
//---------------------------------------------------------------------------
//  "To Control" option 2...edit consists of number + units as string

inline void  ToControl(QLineEdit * control, Frequency_Units var, int precision=1)
{
    ToControl(control, var.Text(precision));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | int)
//---------------------------------------------------------------------------

inline void FromControl(QLineEdit * control, int & var)
{
    var = StringToInt(StdString(control->text()));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(QLineEdit * control, unsigned int & var)
{
    var = StringToInt(StdString(control->text()));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | ii64)
//---------------------------------------------------------------------------

inline void FromControl(QLineEdit * control, ii64 & var)
{
    var = StringToInt64(StdString(control->text()));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | s_ii64)
//---------------------------------------------------------------------------

inline void FromControl(QLineEdit * control, s_ii64 & var)
{
    var = StringToSignedInt64(StdString(control->text()));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | float)
//---------------------------------------------------------------------------

inline void  ToControl(QLineEdit * control, float var, int precision=1)
{
    control->setText(FrameworkString(FloatToString(var, precision)));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | float)
//---------------------------------------------------------------------------

inline void  FromControl(QLineEdit * control, float & var)
{
    var = StringToFloat(StdString(control->text()));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLineEdit | double)
//---------------------------------------------------------------------------

inline void  ToControl(QLineEdit * control, double var, int precision=1)
{
    control->setText(FrameworkString(FloatToString(var, precision)));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | double)
//---------------------------------------------------------------------------

inline void  FromControl(QLineEdit * control, double & var)
{
    var = StringToDouble(StdString(control->text()));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QLineEdit | Frequency_Units )
//---------------------------------------------------------------------------
//  From Control" option 2...edit consists of number + units as string

inline void  FromControl(QLineEdit * control, Frequency_Units & var)
{
    std::string text;
    FromControl(control, text);
    Frequency_Units tmp(text);
    var = tmp;
}

//==============================================================================
//  QGroupBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QGroupBox | bool)
//---------------------------------------------------------------------------

inline void  FromControl(QGroupBox * control, bool & var)
{
    var = control->isChecked();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QGroupBox | const bool &)
//---------------------------------------------------------------------------

inline void ToControl(QGroupBox * control, const bool & var)
{
    control->setChecked(var);
}

//---------------------------------------------------------------------------
//  ClearControl() -- Control >>
//---------------------------------------------------------------------------

inline void  ClearControl(QLineEdit * control)
{
    control->clear();
}

//==============================================================================
// QLabel
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLabel | const string &)
//---------------------------------------------------------------------------

inline void ToControl(QLabel * control, const std::string & msg)
{
    control->setText(FrameworkString(msg));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QLabel  | double )
//---------------------------------------------------------------------------

inline void  ToControl(QLabel * control, double var, int precision=1,
                const std::string & prefix = "",
                const std::string & suffix = "" )
{
    std::string txt(FloatToString(var, precision));
    txt = prefix + txt + suffix;
    ToControl(control, txt);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QLabel  | string )
//---------------------------------------------------------------------------

inline void FromControl(QLabel * control, std::string & msg)
{
    msg = control->text().toStdString();
}

//---------------------------------------------------------------------------
//  SetBackgroundColor() -- >> Control (QLabel  | IIColor )
//---------------------------------------------------------------------------

inline void  SetBackgroundColor(QLabel * control, IIColor col)
{
 if(col == IIRed)
    control->setStyleSheet("background-color: red;");
 else if(col == IIGreen)
    control->setStyleSheet("background-color: green;");
}

//==============================================================================
//  QTabWidget
//==============================================================================

//---------------------------------------------------------------------------
//  RefreshTabs() -- Refresh Tabs to avoid multiple tab re-painting (X3-TIMING)
//---------------------------------------------------------------------------

inline void  RefreshTabs(QTabWidget * control)
{
    for(int i=control->count()-1;i>=0;i--)
    {
        control->setCurrentIndex(i);
    }
}

//---------------------------------------------------------------------------
//  ShowTab() -- Hide/Show Tab on a QTabWidget
//---------------------------------------------------------------------------

inline void  ShowTab(QWidget * tab, QTabWidget * control, bool show)
{
    int idx = control->indexOf(tab);
    control->setCurrentIndex(idx);
    control->setTabEnabled(idx, show);
}

//---------------------------------------------------------------------------
//  FromControl() -- read current tab page index
//---------------------------------------------------------------------------

inline void FromControl(QTabWidget * control, int & var)
{
    var = control->currentIndex();
}

//-------------------------------------------------------------------------------------
//  TabCompare() -- compare between Tabcontrol current tab index with any tabpage index
//--------------------------------------------------------------------------------------
inline bool TabCompare(QTabWidget *MainTab, QWidget *TabPage )
{
                 if (MainTab->widget(MainTab->currentIndex()) == TabPage)
                        return true;
                else
                        return false;
}

//---------------------------------------------------------------------------
//  RemoveTab() -- Remove Tab on a QTabWidget
//---------------------------------------------------------------------------

inline void  RemoveTab(QTabWidget * control, const unsigned int index)
{
    control->removeTab(index);
}


//==============================================================================
//  QComboBox Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(QComboBox * control)
{
    control->clear();
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with string text to control
//---------------------------------------------------------------------------

inline void  AddToControl(QComboBox * control, const std::string & var)
{
    control->addItem(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

inline void  AddToControl(QComboBox * control, int var, int base=10)
{
    control->addItem(FrameworkString(IntToString(var, base)));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QComboBox | int)
//---------------------------------------------------------------------------

inline void  ToControl(QComboBox * control, int var)
{
    control->setCurrentIndex(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QComboBox | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(QComboBox * control, unsigned int var)
{
    control->setCurrentIndex(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QComboBox | int)
//---------------------------------------------------------------------------

inline void FromControl(QComboBox * control, int & var)
{
    var = control->currentIndex();
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QComboBox | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(QComboBox * control, unsigned int & var)
{
    var = control->currentIndex();
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QComboBox | string)
//---------------------------------------------------------------------------

inline void FromControl(QComboBox * control, std::string & var)
{
    int idx;
    FromControl(control, idx);
    var = StdString(control->itemText(idx));
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(QComboBox * control, unsigned int N)
{
    ClearControl(control);
    for (unsigned int i = 0; i < N; ++i)
        AddToControl(control, IntToString(i));
    ControlEnabled(control, N ? true : false);
    if (N)
        ToControl(control, 0);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(QComboBox * control, unsigned int N, const std::string & prefix, const std::string suffix = std::string())
{
    ClearControl(control);
    for (unsigned int i = 0; i < N; ++i)
        {
        std::string text(prefix + IntToString(i) + suffix);
        AddToControl(control, text);
        }
    ControlEnabled(control, N ? true : false);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(QComboBox * control, const std::vector<std::string> & list)
{
    ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, list[i]);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(QComboBox * control, const Innovative::StringList & list)
{
    ClearControl(control);

    for (unsigned int i=0; i<list.Count(); i++)
        AddToControl(control, list.At(i));
}
//---------------------------------------------------------------------------
//  FillControl() -- Fill control with the items in the list
//---------------------------------------------------------------------------

inline void FillControl(QComboBox * control, const std::vector<int> & list, int index = 0)
{
    ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, QString::number(list[i]).toStdString());

    if (control->count()) ToControl(control, index);
}

//---------------------------------------------------------------------------
//  FillControlHex() -- Fill control with the items in the list
//---------------------------------------------------------------------------

inline void FillControlHex(QComboBox * control, const std::vector<int> & list, int index = 0)
{
    ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, list[i], 16);

    if (control->count()) ToControl(control, index);
}

//==============================================================================
//  QDoubleSpinBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QDoubleSpinBox | int)
//---------------------------------------------------------------------------

inline void FromControl(QDoubleSpinBox * control, int & var)
{
    var = control->value();
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QDoubleSpinBox | float)
//---------------------------------------------------------------------------

inline void FromControl(QDoubleSpinBox * control, float & var)
{
    var = control->value();
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QDoubleSpinBox | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(QDoubleSpinBox * control, unsigned int & var)
{
    var = control->value();
}

//---------------------------------------------------------------------------
//  ToControl() -- Control >>  (QDoubleSpinBox | int)
//---------------------------------------------------------------------------

inline void ToControl(QDoubleSpinBox * control, int var)
{
    control->setValue(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- Control >>  (QDoubleSpinBox | unsigned int)
//---------------------------------------------------------------------------

inline void ToControl(QDoubleSpinBox * control, unsigned int var)
{
    control->setValue(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- Control >>  (QDoubleSpinBox | float)
//---------------------------------------------------------------------------

inline void ToControl(QDoubleSpinBox * control, float var)
{
    control->setValue(var);
}

//==============================================================================
//  QSpinBox Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QSpinBox | int)
//---------------------------------------------------------------------------

inline void FromControl(QSpinBox * control, int & var)
{
    var = control->value();
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QSpinBox | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(QSpinBox * control, unsigned int & var)
{
    var = control->value();
}

//---------------------------------------------------------------------------
//  ToControl() -- Control >>  (QSpinBox | int)
//---------------------------------------------------------------------------

inline void ToControl(QSpinBox * control, int var)
{
    control->setValue(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- Control >>  (QSpinBox | unsigned int)
//---------------------------------------------------------------------------

inline void ToControl(QSpinBox * control, unsigned int var)
{
    control->setValue(var);
}

//==============================================================================
//  QCheckBox Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QCheckBox | bool)
//---------------------------------------------------------------------------

inline void  ToControl(QCheckBox * control, bool var)
{
    control->setChecked(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QCheckBox | bool)
//---------------------------------------------------------------------------

inline void  FromControl(QCheckBox * control, bool & var)
{
    var = control->isChecked();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QCheckBox | char)
//---------------------------------------------------------------------------

inline void  ToControl(QCheckBox * control, char var)
{
    control->setChecked(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QCheckBox | char)
//---------------------------------------------------------------------------

inline void FromControl(QCheckBox * control, char & var)
{
    var = control->isChecked();
}

//==============================================================================
//  QListWidget Control I/O
//==============================================================================

//---------------------------------------------------------------------------
//  AddToControl() -- Add string to control
//---------------------------------------------------------------------------

inline void  AddToControl(QListWidget * control, const std::string & var)
{
    control->addItem(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

inline void  AddToControl(QListWidget * control, int var, int base=10)
{
    AddToControl(control, IntToString(var, base));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(QListWidget * control)
{
    control->clear();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control ( QListWidget | vector<string> ) - fill from a list of strings
//---------------------------------------------------------------------------

inline void  ToControl(QListWidget * control, std::vector<std::string> list)
{
    ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, list[i]);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QListWidget | int)
//---------------------------------------------------------------------------

inline void  ToControl(QListWidget * control, int var)
{
    control->setCurrentRow(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QListWidget | int)
//---------------------------------------------------------------------------

inline void FromControl(QListWidget * control, int & var)
{
    var = control->currentRow();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QListWidget | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(QListWidget * control, unsigned int var)
{
    control->setCurrentRow(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QListWidget | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(QListWidget * control, unsigned int & var)
{
    var = control->currentRow();
}


//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QListWidget | bool)
//---------------------------------------------------------------------------

inline void  ToControl(QListWidget * control,  bool var)
{
    control->setCurrentRow(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QListWidget | bool)
//---------------------------------------------------------------------------

inline void FromControl(QListWidget * control, bool & var)
{
    var = control->currentRow();
}


//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QListWidget | vector<T>)
//---------------------------------------------------------------------------

template <class T>
void  ToControl(QListWidget * control, const std::vector<T> & var)
{
    int var_size = var.size();  // int used here because Count is an int

    //  make sure we do not access past either array's bounds
    for (int i = 0; i < control->count(); ++i)
        {
        QListWidgetItem * item = control->item(i);
        if (i < var_size)
            item->setCheckState(var[i] ? Qt::Checked : Qt::Unchecked);
        else
            item->setCheckState(Qt::Unchecked);
        }
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QListWidget | vector<T>)
//---------------------------------------------------------------------------

template <class T>
void FromControl(QListWidget * control, std::vector<T> & var)
{
    int var_size = var.size();   // int used here because Count is an int

    //  make sure we do not access past either array's bounds
    for (int i = 0; i < control->count(); ++i)
        {
        QListWidgetItem * item = control->item(i);
        if (i < var_size)
            var[i] = (item->checkState() == Qt::Checked);
        else
            var[i] = T(0);
        }
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(QListWidget * control, unsigned int N, const std::string & prefix, const std::string suffix = std::string())
{
    ClearControl(control);
    for (unsigned int i = 0; i < N; ++i)
        {
        std::string text(prefix + IntToString(i) + suffix);
        AddToControl(control, text);
        }
    ControlEnabled(control, N ? true : false);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with the items in the list
//---------------------------------------------------------------------------

inline void FillControl(QListWidget * control, const std::vector<std::string> & list)
{
    ClearControl(control);
    for (unsigned int i=0; i<list.size(); i++)
        AddToControl(control, list[i]);

    if(list.size()!=0)
        control->setCurrentRow(0);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(QListWidget * control, const Innovative::StringList & list)
{
    ClearControl(control);

    for (unsigned int i=0; i<list.Count(); i++)
        AddToControl(control, list.At(i));

    if(list.Count()!=0)
        control->setCurrentRow(0);
}
//---------------------------------------------------------------------------
//  ItemCount() -- Return number of items in the list
//---------------------------------------------------------------------------
inline int ItemCount(QListWidget * control)
{
    return control->count();
}

//==============================================================================
//  QTextEdit Control I/O
//==============================================================================
//------------------------------------------------------------------------------
//  ToControl() -- >> Control (QTextEdit | string)
//------------------------------------------------------------------------------
//  Note: this string is 'packed' to remove the line endings that the INI file
//    is sensitive to, so must be translated in each direction

inline void  ToControl(QTextEdit * control, const std::string & var)
{
    StringList list;
    list.Text(var);
    control->setPlainText(FrameworkString(list.AsText(std::string("\n"))));
}

//------------------------------------------------------------------------------
//  FromControl() -- Control >>  (QTextEdit | string)
//------------------------------------------------------------------------------
//  Note: this string is 'packed' to remove the line endings that the INI file
//    is sensitive to, so must be translated in each direction

inline void  FromControl(QTextEdit * control, std::string & var)
{
    StringList list;
    list.Text(StdString(control->toPlainText()), std::string("\n"));
    var = list.Text();
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add to control (QTextEdit | string)
//---------------------------------------------------------------------------
inline void  AddToControl(QTextEdit * control, const std::string & var)
{
    control->append(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add to control (QTextEdit | string)
//---------------------------------------------------------------------------
inline void  WarningLog(QTextEdit * control, const std::string & var)
{
    std::string str = "<body> <p style=color:blue;><br>" + var + "</p>  </body>";
    control->insertHtml(FrameworkString(str));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add to control (QTextEdit | string)
//---------------------------------------------------------------------------
inline void  ErrorLog(QTextEdit * control, const std::string & var)
{
    std::string str = "<body> <p style=color:red;><br>" + var + "</p>  </body>";
    control->insertHtml(FrameworkString(str));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add to control (QTextEdit | string)
//---------------------------------------------------------------------------
inline void  LoudLog(QTextEdit * control, const std::string & var)
{
    std::string str = "<body><p style=font-size:20px><br>" + var + "</p>  </body>";
    control->insertHtml(FrameworkString(str));
}


//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(QTextEdit * control)
{
    control->clear();
}

//---------------------------------------------------------------------------
//  ColorControl() -- set Control text color
//---------------------------------------------------------------------------

inline void  ColorControl(QTextEdit * control, const QColor &color)
{
    control->setTextColor(color);
}
//---------------------------------------------------------------------------
//  ColorControl() -- set Control text color
//---------------------------------------------------------------------------

inline void  ColorControl(QTextBrowser * control, const QColor &color)
{
    control->setTextColor(color);
}
//==============================================================================
// QProgressBar
//==============================================================================

inline void  ToControl(QProgressBar * control, int percent)
{
    control->setValue(percent);
}

//==============================================================================
// QProgressBar
//==============================================================================

inline void  IncrementControl(QProgressBar * control, int num)
{
    control->setValue(control->value()+num);
}

//==============================================================================
//  QStatusBar Control I/O
//==============================================================================
//------------------------------------------------------------------------------
//  MakeStandardStatusBar() -- Create growing StatusBar, attached to Layout
//------------------------------------------------------------------------------

inline QStatusBar *  MakeStandardStatusBar(QWidget * parent, QHBoxLayout * h_layout, bool has_grip=false)
{
    QStatusBar * status_bar = new QStatusBar(parent);
    h_layout->addWidget(status_bar);
    status_bar->setSizeGripEnabled(has_grip);
    return status_bar;
}

//------------------------------------------------------------------------------
//  ItemCount() -- Control >>  (QStatusBar)   Read # of Contained Items
//------------------------------------------------------------------------------

inline int  ItemCount(QStatusBar * control)
{
    int panels = 0;
    const QObjectList & list = control->children();
    for (QObjectList::const_iterator i = list.begin(); i != list.end(); ++i)
        {
        QLabel * panel = dynamic_cast<QLabel*>(*i);
        panels += panel ? 1 : 0;
        }

    return panels;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | string) - Gui Loader version
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, const std::string & var, int idx)
{
    static int StatusBarFieldWidth = 25;

    std::vector<QLabel*> Panels;

    const QObjectList & list = control->children();
    for (QObjectList::const_iterator i = list.begin(); i != list.end(); ++i)
        {
        QLabel * panel = dynamic_cast<QLabel*>(*i);
        if (panel)
            Panels.push_back(panel);
        }

    if (static_cast<size_t>(idx) < Panels.size())
        {
        Panels[idx]->setText(FrameworkString(var));
        }
    else
        {
        std::string s(var);

        int delta = StatusBarFieldWidth - static_cast<int>(var.size());
        if (delta > 0)
            {
            char c = ' ';
             s += std::string(delta, c);
            }

        QLabel * panel = new QLabel(FrameworkString(s));
        control->insertPermanentWidget(idx, panel, 1);
        }
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QStatusBar | string) - Gui Loader version (Ref arg is first)
//---------------------------------------------------------------------------

inline void FromControl(QStatusBar * control, std::string & var, int idx )
{
    std::vector<QLabel*> Panels;

    const QObjectList & list = control->children();
    for (QObjectList::const_iterator i = list.begin(); i != list.end(); ++i)
        {
        QLabel * panel = dynamic_cast<QLabel*>(*i);
        if (panel)
            Panels.push_back(panel);
        }

    if (static_cast<size_t>(idx) < Panels.size())
        var = StdString(Panels[idx]->text());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | const string)
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, int idx, const std::string & var,
    const std::string & label = "")
{
    std::string txt = label + var;
    ToControl(control, txt, idx);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QStatusBar | string)
//---------------------------------------------------------------------------

inline void FromControl(QStatusBar * control, int idx, std::string & var)
{
    FromControl(control, var, idx);
}

//------------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | std::vector<std::string> &)
//------------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, const std::vector<std::string> & items)
{
    for (size_t idx=0; idx < items.size(); idx++)
        ToControl(control, idx, items[idx]);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, int idx,
    unsigned int var, int base=10, const std::string & label="")
{
    std::string txt = label + std::string(": ") + IntToString(var, base);
    ToControl(control, txt, idx);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | int)
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, int idx,
    int var, int base=10, const std::string & label="")
{
    std::string txt = label + std::string(": ") + IntToString(var, base);
    ToControl(control, txt, idx);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | ii64)
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, int idx,
    ii64 var, int base=10, const std::string & label="")
{
    std::string txt = label + std::string(": ") + Int64ToString(var, base);
    ToControl(control, txt, idx);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | float)
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, int idx,
    float var, int precision=1, const std::string & label="")
{
    std::string txt = label + std::string(": ") + FloatToString(var, precision);
    ToControl(control, txt, idx);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QStatusBar | double)
//---------------------------------------------------------------------------

inline void  ToControl(QStatusBar * control, int idx,
    double var, int precision=1, const std::string & label="")
{
    std::string txt = label + std::string(": ") + FloatToString(var, precision);
    ToControl(control, txt, idx);
}

//==============================================================================
//  QTableWidget Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  SetDimensions() -- Set dimensions of grid control
//---------------------------------------------------------------------------

inline void  SetDimensions(QTableWidget * control, int cols, int rows)
{
    control->setColumnCount(cols);
    control->setRowCount(rows);
}

//---------------------------------------------------------------------------
//  ClearControl() -- Control >>
//---------------------------------------------------------------------------

inline void  ClearControl(QTableWidget * control)
{
    control->clear();
}

//---------------------------------------------------------------------------
//  DataRows() -- Get size of grid data region
//---------------------------------------------------------------------------

inline int  DataRows(QTableWidget * control)
{
    return control->rowCount();
}

//---------------------------------------------------------------------------
//  DataCols() -- Get size of grid data region
//---------------------------------------------------------------------------

inline int  DataCols(QTableWidget * control)
{
    return control->columnCount();
}

//---------------------------------------------------------------------------
//  ShowColumnHeader() -- show or hide column headers
//---------------------------------------------------------------------------

inline void  ShowColumnHeader(QTableWidget * control, bool show)
{
    control->horizontalHeader()->setVisible(show);

}

//---------------------------------------------------------------------------
//  ShowRowHeader() -- show or hide row headers
//---------------------------------------------------------------------------

inline void  ShowRowHeader(QTableWidget * control, bool show)
{
    control->verticalHeader()->setVisible(show);

}

//---------------------------------------------------------------------------
//  ResizeGridToFit() -- Fit columns to fill grid window entirely
//---------------------------------------------------------------------------

inline void  ResizeGridToFit(QTableWidget * control, bool fit)
{
    if (fit)
        control->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);
    else
        control->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
}

//---------------------------------------------------------------------------
//  ResizeLastColumnToFit() -- Fit final column to fill grid window entirely
//---------------------------------------------------------------------------

inline void  ResizeLastColumnToFit(QTableWidget * control, bool fit)
{
    control->horizontalHeader()->setStretchLastSection(fit);
}

//---------------------------------------------------------------------------
//  ToColumnHeading() -- Set column heading to string text
//---------------------------------------------------------------------------

inline void  ToColumnHeading(QTableWidget * control, int col, const std::string & var)
{
    QTableWidgetItem * item = new QTableWidgetItem(FrameworkString(var));
    control->setHorizontalHeaderItem(col, item);
}

//---------------------------------------------------------------------------
//  ToRowHeading() -- Set row heading to string text
//---------------------------------------------------------------------------

inline void  ToRowHeading(QTableWidget * control, int row, const std::string & var)
{
    QTableWidgetItem * item = new QTableWidgetItem(FrameworkString(var));
    control->setVerticalHeaderItem(row, item);
}


//---------------------------------------------------------------------------
//  MakeGridItem() -- Assure correct grid item is present in a cell
//---------------------------------------------------------------------------

inline QTableWidgetItem* MakeGridItem(QTableWidget * control, int col, int row)
{
    // First, check current item if any...
    QTableWidgetItem * rawItem = control->item(row, col);
    if (rawItem==0)
        {
        QTableWidgetItem * item = new QTableWidgetItem();  // if no item, make one
        control->setItem(row, col, item);
        return item;
        }

    QTableWidgetItem* casted_item = dynamic_cast<QTableWidgetItem*>(rawItem);
    if (casted_item==0)
        {
        QTableWidgetItem * item = new QTableWidgetItem();  // if not the correct item type, make one
        control->setItem(row, col, item);
        return item;
        }
    // else do nothing, as current item will do fine.
    return casted_item;
}

//---------------------------------------------------------------------------
//  MakeGridItem() -- Assure correct grid item is present in a cell
//---------------------------------------------------------------------------

template <class T>
inline T*  MakeGridWidgetItem(QTableWidget * control, int col, int row)
{
    // First, check current item if any...
    QTableWidgetItem * rawItem = control->item(row, col);
    if (rawItem==0)
        {
        T * item = new T();  // if no item, make one
        control->setCellWidget(row, col, item);
        return item;
        }

    T* casted_item = dynamic_cast<T*>(rawItem);
    if (casted_item==0)
        {
        T * item = new T();  // if not the correct item type, make one
        control->setCellWidget(row, col, item);
        return item;
        }
    // else do nothing, as current item will do fine.
    return casted_item;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | string)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const std::string & var, bool readonly = false)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setText(FrameworkString(var));
    if (readonly)
        item->setFlags(item->flags() ^ Qt::ItemIsEditable);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | bool)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const bool & var, bool readonly = false)
{
   QTableWidgetItem * item = MakeGridItem(control, col, row);
   item->setCheckState(var ? Qt::Checked : Qt::Unchecked);
   if (readonly) item->setFlags(item->flags() ^ Qt::ItemIsUserCheckable);
}



//---------------------------------------------------------------------------
//  SetCell_BackgroundColor() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_BackgroundColor(QTableWidget * control, int col, int row, IIColor color)
{
    QColor qColor = GetColor(color);
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setBackgroundColor(qColor);
}

//---------------------------------------------------------------------------
//  SetCell_ForegroundColor() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_ForegroundColor(QTableWidget * control, int col, int row, IIColor color)
{
    QColor qColor = GetColor(color);
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setForeground(qColor);
}

//---------------------------------------------------------------------------
//  SetCell_FontBold() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_FontBold(QTableWidget * control, int col, int row, bool condition = true)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    QFont font = item->font();
    font.setBold(condition);
    item->setFont(font);
}

//---------------------------------------------------------------------------
//  SetCell_FontItalic() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_FontItalic(QTableWidget * control, int col, int row, bool condition = true)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    QFont font = item->font();
    font.setItalic(condition);
    item->setFont(font);
}

//---------------------------------------------------------------------------
//  SetCell_FontUnderline() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_FontUnderline(QTableWidget * control, int col, int row, bool condition = true)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    QFont font = item->font();
    font.setUnderline(condition);
    item->setFont(font);
}

//---------------------------------------------------------------------------
//  SetCell_Font() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_Font(QTableWidget * control, int col, int row, QFont font)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setFont(font);
}

//---------------------------------------------------------------------------
//  SetCell_Alignment() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_Alignment(QTableWidget * control, int col, int row, int alignment)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setTextAlignment(alignment);
}

//---------------------------------------------------------------------------
//  SetCell_CenterAlignment() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_CenterAlignment(QTableWidget * control, int col, int row)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setTextAlignment(Qt::AlignCenter | Qt::AlignVCenter);
}

//---------------------------------------------------------------------------
//  SetCell_LeftAlignment() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_LeftAlignment(QTableWidget * control, int col, int row)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
}

//---------------------------------------------------------------------------
//  SetCell_LeftAlignment() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_RightAlignment(QTableWidget * control, int col, int row)
{
    QTableWidgetItem * item = MakeGridItem(control, col, row);
    item->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | bool)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const pair<int, vector<string> > &elems)
{
   QComboBox *item = MakeGridWidgetItem<QComboBox>(control, col, row);
   FillControl(item, elems.second);
   ToControl(item, elems.first);

   item->setStyleSheet("border-left-width: 1px;"
                       "border-left-color: darkgray;"
                       "border: 1px solid gray;"
                       "border-radius: 3px;"
                       "padding: 1px 18px 1px 3px;"
                       );
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | float)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const float & var,
   int precision=1, bool readonly = false)
{
    ToControl(control, col, row, FloatToString(var, precision), readonly);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | double)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const double & var,
   int precision=1, bool readonly = false)
{
    ToControl(control, col, row, FloatToString(var, precision), readonly);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | int)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const int & var,
   int base=10, bool readonly = false)
{
    ToControl(control, col, row, IntToString(var, base), readonly);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const unsigned int & var,
   int base=10, bool readonly = false)
{
    ToControl(control, col, row, IntToString(var, base), readonly);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QTableWidget | ii64)
//---------------------------------------------------------------------------

inline void  ToControl(QTableWidget * control, int col, int row, const ii64 & var,
   int base=10, bool readonly = false)
{
    ToControl(control, col, row, Int64ToString(var, base), readonly);
}


//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QTableWidget | string)
//---------------------------------------------------------------------------

inline void FromControl(QTableWidget * control, int col, int row, std::string & var)
{
    QTableWidgetItem * item = control->item(row, col);
    if (item)
        var = StdString(item->text());
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QTableWidget | string)
//---------------------------------------------------------------------------

inline void FromControl(QTableWidget * control, int col, int row, ii64 & var)
{
    std::string s;
    FromControl(control, col, row, s);
    var = StringToInt64(s);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QTableWidget | string)
//---------------------------------------------------------------------------

inline void FromControl(QTableWidget * control, int col, int row, int & var)
{
    std::string s;
    FromControl(control, col, row, s);
    var = StringToInt(s);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QTableWidget | string)
//---------------------------------------------------------------------------

inline void FromControl(QTableWidget * control, int col, int row, unsigned int & var)
{
    std::string s;
    FromControl(control, col, row, s);
    var = StringToInt(s);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QTableWidget | string)
//---------------------------------------------------------------------------

inline void  FromControl(QTableWidget * control, int col, int row, float & var)
{
    std::string s;
    FromControl(control, col, row, s);
    var = StringToFloat(s);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (QTableWidget | bool)
//---------------------------------------------------------------------------

inline void  FromControl(QTableWidget * control, int col, int row, bool & var)
{
    QTableWidgetItem * item = control->item(row, col);
    if (item)
        var = (item->checkState() == Qt::Checked);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStringGrid | vector<T>) - fill a column
//---------------------------------------------------------------------------

template <class T>
void  ToControl(QTableWidget * control, int column, const std::vector<T> & var, int arg=0)
{
    int var_size = var.size();  // int used here because Count is an int
    int rows = DataRows(control);

    //  make sure we do not access past either array's bounds
    for (int i = 0; i < rows; ++i)
        if (i < var_size)
            ToControl(control, column, i, var[i], arg);
        else
            ToControl(control, column, i, static_cast<T>(0), arg);

}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TStringGrid | vector<T>) - read a column
//---------------------------------------------------------------------------

template <class T>
void FromControl(QTableWidget * control, int column, std::vector<T> & var)
{
    int var_size = var.size();   // int used here because Count is an int
    int rows = DataRows(control);

    //  make sure we do not access past either array's bounds
    for (int i = 0; i < rows; ++i)
        if (i < var_size)
            FromControl(control, column, i, var[i]);

}

//==============================================================================
//  Dialogs
//==============================================================================

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(std::string & file, const std::string & filter)
{
    QFileDialog Dialog;

    QString caption("Select File");
    QString dir;
    if (file.size())
        {
        PathSpec path(file);
        dir = FrameworkString(path.Dir());
        }
    QString filt(FrameworkString(filter));
    std::string result = StdString(Dialog.getOpenFileName(0, caption, dir, filt));
    if (!result.size())
        return false;

    file = result;
    return true;
}

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(std::string & file, const StringList & filter)
{
    if (!filter.Count())
        return SelectFileDialog(file, "All Files (*.*)");

    std::ostringstream f;
    StringList::const_iterator i = filter.begin();
    f << *i++;
    for (; i != filter.end(); ++i)
        {
        f << ";;";
        f << *i;
        }

    return SelectFileDialog(file, f.str());
}

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(QLineEdit * control, const std::string & filter)
{
    std::string file=control->text().toStdString();
    if (!SelectFileDialog(file, filter))
        return false;

    ToControl(control, file);
    return true;
}


//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(QTextEdit * control, const std::string & filter)
{
    std::string file=control->toPlainText().toStdString();
    if (!SelectFileDialog(file, filter))
        return false;

    ToControl(control, file);
    return true;
}

//---------------------------------------------------------------------------
//  WarningDialog() -- Display message dialog
//---------------------------------------------------------------------------

inline void WarningDialog(const std::string & msg)
{
    QErrorMessage Dialog;
    Dialog.showMessage(FrameworkString(msg));
}

//---------------------------------------------------------------------------
//  ErrorDialog() -- Display error message dialog
//---------------------------------------------------------------------------

inline void ErrorDialog(const std::string & msg)
{
    QMessageBox Dialog;
    Dialog.setText(FrameworkString(msg));
    Dialog.exec();
}

//==============================================================================
//  QMessage
//==============================================================================
//------------------------------------------------------------------------------
//  ShowMessage() -- >> Display message box with OK button
//------------------------------------------------------------------------------

inline void ShowMessage(QWidget *Widget,const std::string &title, const std::string &msg)
{
    QMessageBox::information(Widget,QString::fromStdString(title),QString::fromStdString(msg));
}

//------------------------------------------------------------------------------
//  ShowMessage() -- >> Display message box with Yes/No button
//------------------------------------------------------------------------------

inline bool ShowYesNoMessage(QWidget *Widget,const std::string &title, const std::string &msg)
{
    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(Widget,QString::fromStdString(title),QString::fromStdString(msg),
                                    QMessageBox::Yes|QMessageBox::No);
    if (reply == QMessageBox::Yes)
        return true;
    else
        return false;
}
//------------------------------------------------------------------------------
//  ShowInputMessage() -- >> Display message box with input option
//------------------------------------------------------------------------------

inline std::string ShowInputMessage(QWidget *Widget,const std::string &title, const std::string &msg)
{
    QString result = QInputDialog::getText(Widget,QString::fromStdString(title),QString::fromStdString(msg));
    return result.toStdString();
}
/*
//==============================================================================
//  TDirectoryListBox Control I/O
//==============================================================================
//------------------------------------------------------------------------------
//  ToControl() -- >> Control (TDirectoryListBox | string)
//------------------------------------------------------------------------------

inline void  ToControl(TDirectoryListBox * control, const std::string & var)
{
    control->Directory = FrameworkString(var);
}

//------------------------------------------------------------------------------
//  FromControl() -- Control >>  (TDirectoryListBox | string)
//------------------------------------------------------------------------------

inline void FromControl(TDirectoryListBox * control, std::string & var)
{
    var = StdString(control->Directory);
}
*/

//==============================================================================
//  QPushButton
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (QPushButton | string)
//---------------------------------------------------------------------------

inline void  ToControl(QPushButton * control, const std::string & var)
{
    control->setText(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (QPushButton | string)
//---------------------------------------------------------------------------

inline void FromControl(QPushButton * control, std::string & var)
{
    var = StdString(control->text());
}


//---------------------------------------------------------------------------
//  SaveDialog() -- Display save dialog
//---------------------------------------------------------------------------

inline bool SaveDialog(std::string & file, const std::string & filter)
{
    QFileDialog Dialog;

    QString caption("Save Document");
    QString dir;
    if (file.size())
        {
        PathSpec path(file);
        dir = FrameworkString(path.Dir());
        }
    QString filt(FrameworkString(filter));
    std::string result = StdString(Dialog.getSaveFileName(0, caption, dir, filt));
    if (!result.size())
        return false;

    file = result;
    return true;
}

//---------------------------------------------------------------------------
//  SaveDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SaveDialog(QTextEdit * control, const std::string & filter)
{
    std::string file;
    if (!SaveDialog(file, filter))
        return false;

    ToControl(control, file);
    return true;
}

//---------------------------------------------------------------------------
//  SaveDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SaveDialog(QLineEdit * control, const std::string & filter)
{
    std::string file;
    if (!SaveDialog(file, filter))
        return false;

    ToControl(control, file);
    return true;
}


//---------------------------------------------------------------------------
//  GetHexMd5Hash() --
//---------------------------------------------------------------------------

inline std::string GetHexMd5Hash(std::string str)
 {

        str = QString(QCryptographicHash::hash(str.c_str(), QCryptographicHash::Md5).toHex()).toStdString();
        return str;
 }

//---------------------------------------------------------------------------
//  TEMPLATE function ReadControl() -- Return Control value thru return code
//---------------------------------------------------------------------------
//  Usage:  int rtn = ReadControl<int>(Control)

template <class retT, class ContT>
retT ReadControl(ContT * Control)
{
    retT value;
    FromControl(Control, value);
    return value;
}


//==============================================================================
//  System Operations
//==============================================================================
//--------------------------------------------------------------------------
//  Terminate() -- Quit app
//---------------------------------------------------------------------------

inline void Terminate()
{
    qApp->exit();
}

}  // end namespace Innovative

#endif
