// Thunker_Qt.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <string>
#include "Thunker_Qt.h"
#include <Events_Mb.h>

namespace   Innovative
{

//=============================================================================
//  CLASS Thunker  --  "Thunk" call into main thread, unlocked with caller
//=============================================================================

#if QT_VERSION >= 0x040400

#else
static int db = QEvent::User;
#endif

//---------------------------------------------------------------------------
//  Dispatcher::Dispatcher() -- constructor for class Dispatcher
//---------------------------------------------------------------------------

Dispatcher::Dispatcher (Thunker & owner)
    : Owner (owner)
{
#if QT_VERSION >=0x040400
    Doorbell = QEvent::registerEventType();
#else
    Doorbell = ++db;  
#endif
    connect (this,SIGNAL ( send(int) ),SLOT ( receive ( int ) ),Qt::QueuedConnection );
}

//---------------------------------------------------------------------------
//  Dispatcher::~Dispatcher() -- Destructor for class Dispatcher
//---------------------------------------------------------------------------

Dispatcher::~Dispatcher()
{
    disconnect();
}

//---------------------------------------------------------------------------
//  Dispatcher::Send() -- Post doorbell message to main window
//---------------------------------------------------------------------------

void Dispatcher::Send()
{
    emit send ( Doorbell );
}

//---------------------------------------------------------------------------
//  Dispatcher::receive() -- Doorbell message handler
//---------------------------------------------------------------------------

void Dispatcher::receive(int e)
{
    if (e==Doorbell)
        Owner.Dispatch();
}


//=============================================================================
//  CLASS Thunker  --  "Thunk" call into main thread, unlocked with caller
//=============================================================================

//---------------------------------------------------------------------------
//  constructor for class Thunker
//---------------------------------------------------------------------------

Thunker::Thunker()
    : Messenger (*this)
{

}

//---------------------------------------------------------------------------
//  destructor for class Thunker
//---------------------------------------------------------------------------

Thunker::~Thunker()
{
}

//---------------------------------------------------------------------------
//  Thunker::Notify() --
//---------------------------------------------------------------------------

bool Thunker::Notify()
{
    if (!OnNotified.Assigned())
        return false;

    if (InnovativeKernel::CurrentThreadId() == OpenWire::GetMainThreadId())
        {
        // We are in the main thread, so execute and exit.
        Dispatch();
        return false;
        }

    Messenger.Send();
    return true;
}

//---------------------------------------------------------------------------
//  Thunker::Dispatch() --
//---------------------------------------------------------------------------

void Thunker::Dispatch()
{
    OpenWire::NotifyEvent e;
    OnNotified.Execute (e);
}

//=============================================================================
//  Thunker Factory Methods
//=============================================================================

//---------------------------------------------------------------------------
//  NewThunker() --
//---------------------------------------------------------------------------

ThunkerIntf * NewThunker()
{
    return new Thunker;
}

//---------------------------------------------------------------------------
//  DeleteThunker() --
//---------------------------------------------------------------------------

void DeleteThunker ( ThunkerIntf * thunker )
{
    delete thunker;
}


}  // namespace
