//---------------------------------------------------------------------------

#ifndef GuiXfr_VclH
#define GuiXfr_VclH

#ifdef __BORLANDC__
	#ifdef __clang__       // 64 bit compiler

	#include <System.Classes.hpp>
	#include <Vcl.StdCtrls.hpp>
	#include <Vcl.ComCtrls.hpp>
	#include <Vcl.ExtCtrls.hpp>
	#include <Vcl.Controls.hpp>
	#include <Vcl.CheckLst.hpp>
	#include <Vcl.Menus.hpp>
	#include <Vcl.ActnList.hpp>
	#include <Vcl.Grids.hpp>
	#include <Vcl.Buttons.hpp>
	#include <Vcl.FileCtrl.hpp>

	#else

	#include <Classes.hpp>
	#include <StdCtrls.hpp>
	#include <ComCtrls.hpp>
	#include <ExtCtrls.hpp>
	#include <Controls.hpp>
	#include <CheckLst.hpp>
	#include <Menus.hpp>
	#include <ActnList.hpp>
	#include <Grids.hpp>
	#include <Buttons.hpp>
	#include <FileCtrl.hpp>

	#endif

#endif

#include <string>
#include <vector>
#include <StringSupport_Mb.h>
#include <StringList_Mb.h>
#include <FileSupport_Mb.h>
#include <AccessDatagrams_Mb.h>
#include <stdlib>
#include <algorithm>

//---------------------------------------------------------------------------
namespace Innovative
{
	
enum IIColor { IIBlack, IIRed, IIBlue, IIYellow, IIGreen, IIWhite };

//===========================================================================
//  CLASS GuiColumn -- exact class for Column information
//===========================================================================

class GuiColumn
{
public:
    explicit GuiColumn(unsigned int column) : Column(column)
                {}
    unsigned int ColumnNumber() const {  return Column;  }

private:
    unsigned int Column;
};

//===========================================================================
//  CLASS GuiRow  -- exact class for Row information
//===========================================================================

class GuiRow
{
public:
    explicit GuiRow(unsigned int row) : Row(row)
                {}
    unsigned int RowNumber() const {  return Row;  }

private:
    unsigned int Row;
};

struct GridRect
{
	int 	Left;
	int		Top;
	int		Bottom;
	int		Right;
};

//---------------------------------------------------------------------------
//  StdString() -- Convert framework string into std::string
//---------------------------------------------------------------------------

inline std::string StdString(const AnsiString & src)
{
	return std::string(UTF8String(src).c_str());
}

//---------------------------------------------------------------------------
//  FrameworkString() -- Convert std::string into framework string
//---------------------------------------------------------------------------

inline AnsiString FrameworkString(const std::string & src)
{
	return AnsiString(src.c_str());
}

//  This file contains implementations for "Generic" Operations on
//    a window control. A control may have an implementation for:
//
//  ToControl()        --- Move data into a control from variable
//  FromControl()      --- Move data into a variabled from control
//  ControlEnabled()   --- Enable/Disable/Read Enabled State
//  ControlVisible()   --- Enable/Disable/Read Enabled State
//  AddToControl()     --- Add to a "List-Like" Control
//  ClearControl()     --- Clear a control (removes items if a list)
//  ItemCount()        --- Current Items in a List-Like control
//
//  Other more specific helper functions may be defined here too
//
//==============================================================================
//  "Standard" Operation Normal Template Implementations
//==============================================================================
//  We define a template function for these operations, implementations are common.
//---------------------------------------------------------------------------
//  TEMPLATE function ControlEnabled() -- Enable/disable control
//---------------------------------------------------------------------------
template <class ContT>
void  ControlEnabled(ContT * control, bool state)
{
	control->Enabled = state;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlEnabled() -- Query enable state
//---------------------------------------------------------------------------
template <class ContT>
bool  ControlEnabled(ContT * control)
{
	return control->Enabled;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlVisible() -- Set Visible State of Control
//---------------------------------------------------------------------------
template <class ContT>
void  ControlVisible(ContT * control, bool state)
{
    control->Visible = state;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ControlVisible() -- Return visible state of a Control
//---------------------------------------------------------------------------
template <class ContT>
bool ControlVisible(ContT * control)
{
    return control->Visible;
}

//---------------------------------------------------------------------------
//  TEMPLATE function ClearControl() -- Erases contents of control
//---------------------------------------------------------------------------
template <class ContT>
bool ControlClear(ContT * control)
{
    return control->Clear();
}

//==============================================================================
//  TForm Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TForm | string)
//---------------------------------------------------------------------------

inline void  ToControl(TForm * control, const std::string & var)
{
	control->Caption = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FormTitle() - set form title
//---------------------------------------------------------------------------

inline void FormTitle(TForm * control, const std::string & var)
{
	ToControl(control, var);
}

//==============================================================================
//  TEdit Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | string)
//---------------------------------------------------------------------------

inline void  ToControl(TEdit * control, const std::string & var)
{
	control->Text = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | string)
//---------------------------------------------------------------------------

inline void  FromControl(TEdit * control, std::string & var)
{
	var = StdString(control->Text);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | int)
//---------------------------------------------------------------------------

inline void  ToControl(TEdit * control, int var, int base=10)
{
	control->Text = FrameworkString(IntToString(var, base));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(TEdit * control, unsigned int var, int base=10)
{
	control->Text = FrameworkString(IntToString(var, base));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | int)
//---------------------------------------------------------------------------

inline void  ToControl(TEdit * control, ii64 var, int base=10)
{
	control->Text = FrameworkString(Int64ToString(var, base));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | ii64)
//---------------------------------------------------------------------------

inline void FromControl(TEdit * control, ii64 & var)
{
	var = StringToInt64(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | int)
//---------------------------------------------------------------------------

inline void FromControl(TEdit * control, int & var)
{
	var = StringToInt(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(TEdit * control, unsigned int & var)
{
	var = StringToInt(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | float)
//---------------------------------------------------------------------------

inline void  ToControl(TEdit * control, float var, int precision=1)
{
	control->Text = FrameworkString(FloatToString(var, precision));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | double)
//---------------------------------------------------------------------------

inline void  ToControl(TEdit * control, double var, int precision=1)
{
	control->Text = FrameworkString(FloatToString(var, precision));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | double)
//---------------------------------------------------------------------------

inline void  FromControl(TEdit * control, double & var)
{
	var = StringToDouble(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | float)
//---------------------------------------------------------------------------

inline void  FromControl(TEdit * control, float & var)
{
	var = StringToFloat(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Control >>
//---------------------------------------------------------------------------

inline void  ClearControl(TEdit * control)
{
	control->Clear();
}

//==============================================================================
//  TLabeledEdit Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabeledEdit | string)
//---------------------------------------------------------------------------

inline void  ToControl(TLabeledEdit * control, const std::string & var)
{
	control->Text = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabeledEdit | string)
//---------------------------------------------------------------------------

inline void FromControl(TLabeledEdit * control, std::string & var)
{
	var = StdString(control->Text);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabeledEdit | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(TLabeledEdit * control, unsigned int var, int base=10)
{
	control->Text = FrameworkString(IntToString(var, base));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabeledEdit | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(TLabeledEdit * control, unsigned int & var)
{
	var = StringToInt(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabeledEdit | int)
//---------------------------------------------------------------------------

inline void  ToControl(TLabeledEdit * control, int var, int base=10)
{
	control->Text = FrameworkString(IntToString(var, base));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabeledEdit | int)
//---------------------------------------------------------------------------

inline void FromControl(TLabeledEdit * control, int & var)
{
	var = StringToInt(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabeledEdit | int64)
//---------------------------------------------------------------------------

inline void  ToControl(TLabeledEdit * control, ii64 var, int base=10)
{
	control->Text = FrameworkString(Int64ToString(var, base));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabeledEdit | ii64)
//---------------------------------------------------------------------------

inline void FromControl(TLabeledEdit * control, ii64 & var)
{
	var = StringToInt64(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabeledEdit | float)
//---------------------------------------------------------------------------

inline void  ToControl(TLabeledEdit * control, float var, int precision=1)
{
	control->Text = FrameworkString(FloatToString(var, precision));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabeledEdit | float)
//---------------------------------------------------------------------------

inline void FromControl(TLabeledEdit * control, float & var)
{
	var = StringToFloat(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabeledEdit | double)
//---------------------------------------------------------------------------

inline void FromControl(TLabeledEdit * control, double & var)
{
	var = StringToDouble(StdString(control->Text));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabeledEdit | double)
//---------------------------------------------------------------------------

inline void  ToControl(TLabeledEdit * control, double var, int precision=1)
{
	control->Text = FrameworkString(FloatToString(var, precision));
}


//==============================================================================
// TLabel
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabel | const string &)
//---------------------------------------------------------------------------

inline void ToControl(TLabel * control, const std::string & msg)
{
	control->Caption = FrameworkString(msg);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TLabel  | double )
//---------------------------------------------------------------------------

inline void  ToControl(TLabel * control, double var, int precision=1,
				const std::string & prefix = "",
				const std::string & suffix = "" )
{
	std::string txt(FloatToString(var, precision));
	txt = prefix + txt + suffix;
	ToControl(control, txt);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TLabel | string)
//---------------------------------------------------------------------------

inline void FromControl(TLabel * control, std::string & var)
{
	var = StdString(control->Caption);
}


//==============================================================================
//  TTabSheet  / TPageControl
//==============================================================================
//---------------------------------------------------------------------------
//  ShowTab() -- Hide/Show Tab on a TPageControl
//---------------------------------------------------------------------------

inline void  ShowTab(TTabSheet * tab, TPageControl * control, bool show)
{
	control->ActivePage = tab;
	tab->TabVisible = show;
}

//---------------------------------------------------------------------------
//  ShowTab() -- Hide/Show Tab on a TPageControl
//---------------------------------------------------------------------------

inline void  ShowTab(TTabSheet * tab, bool show)
{
	tab->TabVisible = show;
}

//==============================================================================
//  TComboBox Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ItemCount() -- return count of items
//---------------------------------------------------------------------------

inline int  ItemCount(TComboBox * control)
{
	return control->Items->Count;
}
//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(TComboBox * control)
{
	control->Items->Clear();
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with string text to control
//---------------------------------------------------------------------------

inline void  AddToControl(TComboBox * control, const std::string & var)
{
	control->Items->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

inline void  AddToControl(TComboBox * control, int var, int base=10)
{
	control->Items->Add(FrameworkString(IntToString(var, base)));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TComboBox | int)
//---------------------------------------------------------------------------

inline void  ToControl(TComboBox * control, int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TComboBox | int)
//---------------------------------------------------------------------------

inline void FromControl(TComboBox * control, int & var)
{
	var = control->ItemIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TComboBox | int)
//---------------------------------------------------------------------------

inline void  ToControl(TComboBox * control, unsigned int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TComboBox | int)
//---------------------------------------------------------------------------

inline void FromControl(TComboBox * control, unsigned int & var)
{
	var = control->ItemIndex;
}
//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TComboBox | string)
//---------------------------------------------------------------------------

inline void FromControl(TComboBox * control, std::string & var)
{
	var = StdString(control->Text);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(TComboBox * control, unsigned int N)
{
	ClearControl(control);
	for (unsigned int i = 0; i < N; ++i)
		AddToControl(control, IntToString(i));
	ControlEnabled(control, N ? true : false);
	if (N)
		ToControl(control, 0);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(TComboBox * control, unsigned int N, const std::string & prefix, const std::string suffix = std::string())
{
	ClearControl(control);
	for (unsigned int i = 0; i < N; ++i)
		{
		std::string text(prefix + IntToString(i) + suffix);
		AddToControl(control, text);
		}
	ControlEnabled(control, N ? true : false);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(TComboBox * control, const std::vector<std::string> & list)
{
	ClearControl(control);
	for (unsigned int i=0; i<list.size(); i++)
		AddToControl(control, list[i]);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(TComboBox * control, const Innovative::StringList & list)
{
	ClearControl(control);

	for (unsigned int i=0; i<list.Count(); i++)
		AddToControl(control, list.At(i));
}

//==============================================================================
//  TRadioGroup Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TRadioGroup | int)
//---------------------------------------------------------------------------

inline void  ToControl(TRadioGroup * control, bool var)
{
	control->ItemIndex = var ? 1 : 0;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TRadioGroup | int)
//---------------------------------------------------------------------------

inline void FromControl(TRadioGroup * control, bool & var)
{
	var = control->ItemIndex ? true : false;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TRadioGroup | int)
//---------------------------------------------------------------------------

inline void  ToControl(TRadioGroup * control, int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TRadioGroup | int)
//---------------------------------------------------------------------------

inline void FromControl(TRadioGroup * control, int & var)
{
	var = control->ItemIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TRadioGroup | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(TRadioGroup * control, unsigned int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TRadioGroup | unsigned int)
//---------------------------------------------------------------------------

inline void FromControl(TRadioGroup * control, unsigned int & var)
{
	var = control->ItemIndex;
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

inline void  AddToControl(TRadioGroup * control, const std::string & var)
{
	control->Items->Add(FrameworkString(var));
}
//==============================================================================
//  TCheckBox Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TCheckBox | bool)
//---------------------------------------------------------------------------

inline void  ToControl(TCheckBox * control, bool var)
{
	control->Checked = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TCheckBox | bool)
//---------------------------------------------------------------------------

inline void  FromControl(TCheckBox * control, bool & var)
{
	var = control->Checked;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TCheckBox | char)
//---------------------------------------------------------------------------

inline void  ToControl(TCheckBox * control, char var)
{
	control->Checked = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TCheckBox | char)
//---------------------------------------------------------------------------

inline void FromControl(TCheckBox * control, char & var)
{
	var = control->Checked;
}

//==============================================================================
//  TCheckListBox Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ItemCount() -- return count of items
//---------------------------------------------------------------------------

inline int  ItemCount(TCheckListBox * control)
{
	return control->Items->Count;
}
//---------------------------------------------------------------------------
//  AddToControl() -- Add string to control
//---------------------------------------------------------------------------

inline void  AddToControl(TCheckListBox * control, const std::string & var)
{
	control->Items->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(TCheckListBox * control)
{
	control->Items->Clear();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TCheckListBox | int)
//---------------------------------------------------------------------------

inline void  ToControl(TCheckListBox * control, int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TCheckListBox | int)
//---------------------------------------------------------------------------

inline void FromControl(TCheckListBox * control, int & var)
{
	var = control->ItemIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TCheckListBox | uint)
//---------------------------------------------------------------------------

inline void  ToControl(TCheckListBox * control, unsigned int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TCheckListBox | uint)
//---------------------------------------------------------------------------

inline void FromControl(TCheckListBox * control, unsigned int & var)
{
	var = control->ItemIndex;
}
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TCheckListBox | vector<T>)
//---------------------------------------------------------------------------

template <class T>
void  ToControl(TCheckListBox * control, const std::vector<T> & var)
{
	int var_size = var.size();  // int used here because Count is an int

	//  make sure we do not access past either array's bounds
	for (int i = 0; i < control->Count; ++i)
		if (i < var_size)
			control->Checked[i] = var[i];
		else
			control->Checked[i] = false;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TCheckListBox | vector<T>)
//---------------------------------------------------------------------------

template <class T>
void FromControl(TCheckListBox * control, std::vector<T> & var)
{
	int var_size = var.size();   // int used here because Count is an int

	//  make sure we do not access past either array's bounds
	for (int i = 0; i < control->Count; ++i)
		if (i < var_size)
			var[i] = control->Checked[i];
		else
			var[i] = T(0);

}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(TCheckListBox * control, unsigned int N, const std::string & prefix, const std::string suffix = std::string())
{
	ClearControl(control);
	for (unsigned int i = 0; i < N; ++i)
		{
		std::string text(prefix + IntToString(i) + suffix);
		AddToControl(control, text);
		}
	ControlEnabled(control, N ? true : false);
}

//---------------------------------------------------------------------------
//  FillControl() -- Fill control with N numbered items
//---------------------------------------------------------------------------

inline void FillControl(TCheckListBox * control, const std::vector<std::string> & list)
{
	ClearControl(control);
	for (unsigned int i=0; i<list.size(); i++)
		AddToControl(control, list[i]);
}

//==============================================================================
//  TListBox Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ItemCount() -- return count of items
//---------------------------------------------------------------------------

inline int  ItemCount(TListBox * control)
{
	return control->Count;
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add string to control
//---------------------------------------------------------------------------

inline void  AddToControl(TListBox * control, const std::string & var)
{
	control->Items->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  AddToControl() -- Append item with text equivalent of integer value to control
//---------------------------------------------------------------------------

inline void  AddToControl(TListBox * control, int var, int base=10)
{
	control->Items->Add(FrameworkString(IntToString(var, base)));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(TListBox * control)
{
	control->Items->Clear();
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control ( TListBox | vector<string> ) - fill from a list of strings
//---------------------------------------------------------------------------

inline void  ToControl(TListBox * control, std::vector<std::string> list)
{
	ClearControl(control);
	for (unsigned int i=0; i<list.size(); i++)
		AddToControl(control, list[i]);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TListBox | int)
//---------------------------------------------------------------------------

inline void  ToControl(TListBox * control, int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TListBox | int)
//---------------------------------------------------------------------------

inline void FromControl(TListBox * control, int & var)
{
	var = control->ItemIndex;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TListBox | uint)
//---------------------------------------------------------------------------

inline void  ToControl(TListBox * control, unsigned int var)
{
	control->ItemIndex = var;
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TListBox | uint)
//---------------------------------------------------------------------------

inline void FromControl(TListBox * control, unsigned int & var)
{
	var = control->ItemIndex;
}

//==============================================================================
//  TMemo Control I/O
//==============================================================================
//------------------------------------------------------------------------------
//  ToControl() -- >> Control (TMemo | string)
//------------------------------------------------------------------------------
//  Note: this string is 'packed' to remove the line endings that the INI file
//    is sensitive to, so must be translated in each direction

inline void  ToControl(TMemo * control, const std::string & var)
{
	StringList list;
	list.Text(var);
	control->Text = FrameworkString(list.AsText(std::string("\r\n")));
}

//------------------------------------------------------------------------------
//  FromControl() -- Control >>  (TMemo | string)
//------------------------------------------------------------------------------
//  Note: this string is 'packed' to remove the line endings that the INI file
//    is sensitive to, so must be translated in each direction

inline void  FromControl(TMemo * control, std::string & var)
{
	StringList list;
	list.Text(StdString(control->Text), std::string("\r\n"));
	var = list.Text();
}

//---------------------------------------------------------------------------
//  AddToControl() -- Add to control (TMemo | string)
//---------------------------------------------------------------------------

inline void  AddToControl(TMemo * control, const std::string & var)
{
	control->Lines->Add(FrameworkString(var));
}

//---------------------------------------------------------------------------
//  ClearControl() -- Clear control
//---------------------------------------------------------------------------

inline void  ClearControl(TMemo * control)
{
	control->Clear();
}

//==============================================================================
// TProgressBar
//==============================================================================

inline void  ToControl(TProgressBar * control, int percent)
{
	control->Position = percent;
}

//==============================================================================
//  TStatusBar Control I/O
//==============================================================================
//------------------------------------------------------------------------------
//  ItemCount() -- Control >>  (TStatusBar)   Read # of Contained Items
//------------------------------------------------------------------------------

inline int  ItemCount(TStatusBar * control)
{
	return control->Panels->Count;
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | string) - Gui Loader version (Ref arg is first)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, const std::string & var, int idx )
{
	control->Panels->Items[idx]->Text = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (TStatusBar | string) - Gui Loader version (Ref arg is first)
//---------------------------------------------------------------------------

inline void FromControl(TStatusBar * control, std::string & var, int idx )
{
	var = StdString(control->Panels->Items[idx]->Text);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | const string)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, int idx, const std::string & var,
	const std::string & label = "" )
{
	std::string txt = label + var;
	ToControl(control, txt, idx );
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (TStatusBar | string)
//---------------------------------------------------------------------------

inline void FromControl(TStatusBar * control, int idx, std::string & var)
{
	FromControl(control, var, idx);
}

//------------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | std::vector<std::string> &)
//------------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, const std::vector<std::string> & items)
{
	//  Fill panels with strings, or blank if too few in string list
	for (int idx=0; idx<ItemCount(control); idx++)
		{
		if (idx < (int)items.size())
			ToControl(control, idx, items[idx]);
		else
			ToControl(control, idx, "");
		}
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, int idx,
	unsigned int var, int base=10, const std::string & label="")
{
	stringstream txt;
	if (label.size())
		txt << label << ": ";
	txt << IntToString(var, base);
	control->Panels->Items[idx]->Text = FrameworkString(txt.str());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | int)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, int idx,
	int var, int base=10, const std::string & label="")
{
	stringstream txt;
	if (label.size())
		txt << label << ": ";
	txt << IntToString(var, base);
	control->Panels->Items[idx]->Text = FrameworkString(txt.str());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | ii64)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, int idx,
	ii64 var, int base=10, const std::string & label="")
{
	stringstream txt;
	if (label.size())
		txt << label << ": ";
	txt << Int64ToString(var, base);
	control->Panels->Items[idx]->Text = FrameworkString(txt.str());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | float)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, int idx,
	float var, int precision=1, const std::string & label="")
{
	stringstream txt;
	if (label.size())
		txt << label << ": ";
	txt << FloatToString(var, precision);
	control->Panels->Items[idx]->Text = FrameworkString(txt.str());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStatusBar | double)
//---------------------------------------------------------------------------

inline void  ToControl(TStatusBar * control, int idx,
	double var, int precision=1, const std::string & label="")
{
	stringstream txt;
	if (label.size())
		txt << label << ": ";
	txt << DoubleToString(var, precision);
	control->Panels->Items[idx]->Text = FrameworkString(txt.str());
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TForm | string)
//---------------------------------------------------------------------------

inline void  ToControl(TPanel * control, const std::string & var)
{
	control->Caption = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FormTitle() - set form title
//---------------------------------------------------------------------------

inline void FormTitle(TPanel * control, const std::string & var)
{
	ToControl(control, var);
}

//==============================================================================
//  TStringGrid Control I/O
//==============================================================================
//---------------------------------------------------------------------------
//  ClearControl() -- Erases contents of control
//---------------------------------------------------------------------------
inline void ClearControl(TStringGrid * control)
{
	// Initialize grids, Disable selection region
	TGridRect myRect;
	myRect.Left = myRect.Top = myRect.Right = myRect.Bottom = -1;
	control->Selection = myRect;

	for (int i = 0; i < control->RowCount; ++i)
		control->Rows[i]->Clear();
}

//------------------------------------------------------------------------------
//  TForm1::AutosizeControl()  --  Autosize grid to contents
//------------------------------------------------------------------------------

inline void AutosizeControl(TStringGrid * control)
{
	for (int col = 0; col < control->ColCount; ++col)
		{
		int WMax = 0;
		for (int row = 0; row < control->RowCount; ++row)
			{
			int W = control->Canvas->TextWidth(control->Cells[col][row]);
			WMax = std::max<int>(WMax, W);
			}
		control->ColWidths[col] = WMax+20;
		}
}

//---------------------------------------------------------------------------
//  DisableSelectionRegion() --  Disable visible selection region
//---------------------------------------------------------------------------

inline void  DisableSelectionRegion(TStringGrid * control)
{
	TGridRect myRect;
	myRect.Left = myRect.Top = myRect.Right = myRect.Bottom = -1;
	control->Selection = myRect;
}

//---------------------------------------------------------------------------
//  FixedRows() --  Get size of grid header region
//---------------------------------------------------------------------------

inline int  FixedRows(TStringGrid * control)
{
	return control->FixedRows;
}

//---------------------------------------------------------------------------
//  DataRows() -- Get size of grid data region
//---------------------------------------------------------------------------

inline int  DataRows(TStringGrid * control)
{
	return control->RowCount - FixedRows(control);
}

//---------------------------------------------------------------------------
//  DataRow() -- Get current data row
//---------------------------------------------------------------------------

inline int  DataRow(TStringGrid * control)
{
	return control->Row + FixedRows(control);
}

//---------------------------------------------------------------------------
//  DataRow() -- Set current data row
//---------------------------------------------------------------------------

inline void DataRow(TStringGrid * control, int row)
{
	control->Row = row + FixedRows(control);
}

//---------------------------------------------------------------------------
//  FixedCols() -- Get size of grid header region
//---------------------------------------------------------------------------

inline int  FixedCols(TStringGrid * control)
{
	return control->FixedCols;
}

//---------------------------------------------------------------------------
//  DataCols() -- Get size of grid data region
//---------------------------------------------------------------------------

inline int  DataCols(TStringGrid * control)
{
	return control->ColCount - FixedCols(control);
}

//---------------------------------------------------------------------------
//  DataCol() -- Get current data column
//---------------------------------------------------------------------------

inline int  DataCol(TStringGrid * control)
{
	return control->Col + FixedCols(control);
}

//---------------------------------------------------------------------------
//  DataCol() -- Set current data column
//---------------------------------------------------------------------------

inline void DataCol(TStringGrid * control, int col)
{
	control->Col = col + FixedCols(control);
}

//---------------------------------------------------------------------------
//  Selected() -- Select grid region
//---------------------------------------------------------------------------

inline void  Selected(TStringGrid * control, int left, int top, int bottom, int right)
{
	TGridRect myRect;
	myRect.Left = left + FixedCols(control);
	myRect.Top = top + FixedRows(control);
	myRect.Right = right + FixedCols(control);
	myRect.Bottom = bottom + FixedRows(control);
	control->Selection = myRect;
}

//---------------------------------------------------------------------------
//  Selected() -- Select grid region
//---------------------------------------------------------------------------

inline GridRect Selected(TStringGrid * control)
{
	TGridRect myRect = control->Selection;

	GridRect result;
	result.Left = myRect.Left - FixedCols(control);
	result.Top = myRect.Top - FixedRows(control);
	result.Right = myRect.Right - FixedCols(control);
	result.Bottom = myRect.Bottom - FixedRows(control);

	return result;
}

//---------------------------------------------------------------------------
//  SetDimensions() -- Set dimensions of grid control
//---------------------------------------------------------------------------

inline void  SetDimensions(TStringGrid * control, int cols, int rows)
{
	control->ColCount = cols+FixedCols(control);
	control->RowCount = rows+FixedRows(control);
}

//---------------------------------------------------------------------------
//  ToColumnHeading() -- Set column heading to string text
//---------------------------------------------------------------------------

inline void  ToColumnHeading(TStringGrid * control, int col, const std::string & var)
{
	control->Cells[col+FixedCols(control)][0] = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromColumnHeading() -- Get column heading to string text
//---------------------------------------------------------------------------

inline void  FromColumnHeading(TStringGrid * control, int col, std::string & var)
{
	 var = StdString(control->Cells[col+FixedCols(control)][0]);
}

//---------------------------------------------------------------------------
//  ToRowHeading() -- Set row heading to string text
//---------------------------------------------------------------------------

inline void  ToRowHeading(TStringGrid * control, int row, const std::string & var)
{
	control->Cells[0][row+FixedRows(control)] = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromRowHeading() -- Get row heading to string text
//---------------------------------------------------------------------------

inline void  FromRowHeading(TStringGrid * control, int row, std::string & var)
{
	var = StdString(control->Cells[0][row+FixedRows(control)]);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStringGrid | string)
//---------------------------------------------------------------------------

inline void  ToControl(TStringGrid * control, int col, int row, const std::string & var)
{
	control->Cells[col+FixedCols(control)][row+FixedRows(control)] = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStringGrid | float)
//---------------------------------------------------------------------------

inline void  ToControl(TStringGrid * control, int col, int row, const float & var,
   int precision=1)
{
	control->Cells[col+FixedCols(control)][row+FixedRows(control)] = FrameworkString( FloatToString(var, precision) );
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStringGrid | int)
//---------------------------------------------------------------------------

inline void  ToControl(TStringGrid * control, int col, int row, const int & var,
   int base=10)
{
	control->Cells[col+FixedCols(control)][row+FixedRows(control)] = FrameworkString( IntToString(var, base) );
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStringGrid | unsigned int)
//---------------------------------------------------------------------------

inline void  ToControl(TStringGrid * control, int col, int row, const unsigned int & var,
   int base=10)
{
	control->Cells[col+FixedCols(control)][row+FixedRows(control)] = FrameworkString( IntToString(var, base) );
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (TStringGrid | string)
//---------------------------------------------------------------------------

inline void FromControl(TStringGrid * control, int col, int row, std::string & var)
{
	String cell = control->Cells[col+FixedCols(control)][row+FixedRows(control)];
	var = StdString(cell);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (TStringGrid | string)
//---------------------------------------------------------------------------

inline void FromControl(TStringGrid * control, int col, int row, int & var)
{
	String cell = control->Cells[col+FixedCols(control)][row+FixedRows(control)];
	var = StringToInt(StdString(cell));
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (TStringGrid | string)
//---------------------------------------------------------------------------

inline void  FromControl(TStringGrid * control, int col, int row, float & var)
{
	String cell = control->Cells[col+FixedCols(control)][row+FixedRows(control)];
	var = StringToFloat(StdString(cell));
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TStringGrid | vector<T>) - fill a column
//---------------------------------------------------------------------------

template <class T>
void  ToControl(TStringGrid * control, int column, const std::vector<T> & var, int arg)
{
	int var_size = var.size();  // int used here because Count is an int
	int rows = DataRows(control);

	//  make sure we do not access past either array's bounds
	for (int i = 0; i < rows; ++i)
		if (i < var_size)
			ToControl(control, column, i, var[i], arg);
		else
			ToControl(control, column, i, static_cast<T>(0), arg);

}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TStringGrid | vector<T>) - read a column
//---------------------------------------------------------------------------

template <class T>
void FromControl(TStringGrid * control, int column, std::vector<T> & var)
{
	int var_size = var.size();   // int used here because Count is an int
	int rows = DataRows(control);

	//  make sure we do not access past either array's bounds
	for (int i = 0; i < rows; ++i)
		if (i < var_size)
			FromControl(control, column, i, var[i]);

}

//-----------------------------------------------------------------------------
//  SizeGrid()  --  Resize Grid to match control size
//-----------------------------------------------------------------------------
//   Adjust row and column count to fit inside control's dimensions

inline void  SizeGrid(TStringGrid * control)
{
	int CellWidth = control->DefaultColWidth + control->GridLineWidth;
	int CellHeight = control->DefaultRowHeight + control->GridLineWidth;

	int Columns = control->Width / CellWidth;
	int Rows = control->Height / CellHeight;

	SetDimensions(control, Columns-FixedCols(control), Rows-FixedRows(control));
}

//-----------------------------------------------------------------------------
//  SizeGridWidth()  --  Resize Grid to match control width & fixed total cells
//-----------------------------------------------------------------------------
//   Adjust column count to fit inside control's dimensions
//   Adjust rows to fit entire region in it (user scrolls down to see all)

inline void  SizeGridWidth(TStringGrid * control, const SafeMemoryRange & Region)
{
	int CellWidth = control->DefaultColWidth + control->GridLineWidth;
//    int CellHeight = control->DefaultRowHeight + control->GridLineWidth;

	int Columns = control->Width / CellWidth;
	int DataColumns = Columns-FixedCols(control);
	int Rows = Region.size() / (DataColumns);
	if (Rows<1)        // Grid needs at least 1 fixed and 1 data row always
		Rows = 1;
	if (Rows*DataColumns < (int)Region.size())  // If a row is part-filled, grow by 1 to see it
		Rows += 1;

	SetDimensions(control, DataColumns, Rows);
}


//-----------------------------------------------------------------------------
//  FillGrid()  --  Fill grid with data from region
//-----------------------------------------------------------------------------

inline void  FillGrid(TStringGrid * control, unsigned int * Region, size_t offset, int base)
{
	if (Region==0)
		return;

	for (int row = 0; row<DataRows(control); row++)
		{
		//  Load Row Header (address)
		ToRowHeading(control, row, IntToString(offset, base));
		for (int col = 0; col<DataCols(control); col++)
			{
			ToControl(control, col, row, Region[offset], base);
			offset++;
			}
		}
   for (int col = 0; col<DataCols(control); col++)
		{
		//  Load Column Header (offset)
		ToColumnHeading(control, col, IntToString(col, base));
		}

}

//-----------------------------------------------------------------------------
//  FillGrid()  --  Fill grid with data from region
//-----------------------------------------------------------------------------

inline void FillGrid(TStringGrid * control, const SafeMemoryRange & Region, size_t offset, int base=16, bool skipColHdr=false, bool skipRowHdr=false)
{
	for (int row = 0; row<DataRows(control); row++)
		{
		//  Load Row Header (address)
		if (skipRowHdr==false)
			ToRowHeading(control, row, IntToString(offset, base));
		for (int col = 0; col<DataCols(control); col++)
			{
			ToControl(control, col, row, Region[offset], base);
			offset++;
			}
		}
   for (int col = 0; col<DataCols(control); col++)
		{
		//  Load Column Header (offset)
		if (skipColHdr==false)
			ToColumnHeading(control, col, IntToString(col, base));
		}

}

//-----------------------------------------------------------------------------
//  FillGrid_NoCH()  --  Fill grid with data from region - no column header
//-----------------------------------------------------------------------------

inline void FillGrid_NoCH(TStringGrid * control, const SafeMemoryRange & Region, size_t offset, int base=16)
{
	FillGrid(control, Region, offset, base, true);
}

//---------------------------------------------------------------------------
//  GetColor() -- >>
//---------------------------------------------------------------------------

inline TColor GetColor(IIColor color)
{
	return clBlack;
}

//---------------------------------------------------------------------------
//  SetCell_BackgroundColorX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_BackgroundColor(TStringGrid * control, int col, int row, IIColor color)
{

}

//---------------------------------------------------------------------------
//  SetCell_ForegroundColorX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_ForegroundColor(TStringGrid * control, int col, int row, IIColor color)
{

}

//---------------------------------------------------------------------------
//  SetCell_FontBoldX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_FontBold(TStringGrid * control, int col, int row, bool condition = true)
{

}

//---------------------------------------------------------------------------
//  SetCell_FontItalic() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_FontItalic(TStringGrid * control, int col, int row, bool condition = true)
{

}

//---------------------------------------------------------------------------
//  SetCell_FontUnderline() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_FontUnderline(TStringGrid * control, int col, int row, bool condition = true)
{

}

//---------------------------------------------------------------------------
//  SetCell_FontX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_Font(TStringGrid * control, int col, int row, TFont *font)
{

}

//---------------------------------------------------------------------------
//  SetCell_Alignment() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_Alignment(TStringGrid * control, int col, int row, int alignment){

}

//---------------------------------------------------------------------------
//  SetCell_CenterAlignmentX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_CenterAlignment(TStringGrid * control, int col, int row)
{

}

//---------------------------------------------------------------------------
//  SetCell_LeftAlignmentX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_LeftAlignment(TStringGrid * control, int col, int row)
{

}

//---------------------------------------------------------------------------
//  SetCell_LeftAlignmentX() -- >>
//---------------------------------------------------------------------------

inline void  SetCell_RightAlignment(TStringGrid * control, int col, int row)
{

}

//==============================================================================
//  Dialogs
//==============================================================================

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(std::string & file, const std::string & filter)
{
	std::auto_ptr<TOpenDialog> Dialog(new TOpenDialog(NULL));
	Dialog->Filter = FrameworkString(filter);
	Dialog->Title = "Select File";
	if (file.size())
		{
		PathSpec path(file);
		Dialog->InitialDir = FrameworkString(path.Dir());
		}

	if (!Dialog->Execute())
		return false;

	file = StdString(Dialog->FileName);

	return true;
}

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(TEdit * control, const std::string & filter)
{
	std::string file;
	if (!SelectFileDialog(file, filter))
		return false;

	ToControl(control, file);
	return true;
}

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(TLabeledEdit * control, const std::string & filter)
{
	std::string file;
	if (!SelectFileDialog(file, filter))
		return false;

	ToControl(control, file);
	return true;
}

//---------------------------------------------------------------------------
//  SelectFileDialog() -- Browse to a file matching specified filter
//---------------------------------------------------------------------------

inline bool SelectFileDialog(TStatusBar * control, const std::string & filter, int idx=0)
{
	std::string file;
	if (!SelectFileDialog(file, filter))
		return false;

	ToControl(control, file, idx);
	return true;
}

//---------------------------------------------------------------------------
//  WarningDialog() -- Display message dialog
//---------------------------------------------------------------------------

inline void WarningDialog(const std::string & msg)
{
	MessageDlg(FrameworkString(msg), mtWarning, TMsgDlgButtons() << mbOK, 0);
}

//==============================================================================
//  PageControl
//==============================================================================

inline void FromControl(TPageControl * control, int & var)
{
	var = control->TabIndex;
}

//==============================================================================
//  TDirectoryListBox Control I/O
//==============================================================================
//------------------------------------------------------------------------------
//  ToControl() -- >> Control (TDirectoryListBox | string)
//------------------------------------------------------------------------------

inline void  ToControl(TDirectoryListBox * control, const std::string & var)
{
	control->Directory = FrameworkString(var);
}

//------------------------------------------------------------------------------
//  FromControl() -- Control >>  (TDirectoryListBox | string)
//------------------------------------------------------------------------------

inline void FromControl(TDirectoryListBox * control, std::string & var)
{
	var = StdString(control->Directory);
}

//==============================================================================
//  TButton
//==============================================================================
//---------------------------------------------------------------------------
//  ToControl() -- >> Control (TEdit | string)
//---------------------------------------------------------------------------

inline void  ToControl(TButton * control, const std::string & var)
{
	control->Caption = FrameworkString(var);
}

//---------------------------------------------------------------------------
//  FromControl() -- Control >>  (TEdit | string)
//---------------------------------------------------------------------------

inline void FromControl(TButton * control, std::string & var)
{
	var = StdString(control->Caption);
}

//---------------------------------------------------------------------------
//  TEMPLATE function ReadControl() -- Return Control value thru return code
//---------------------------------------------------------------------------
//  Usage:  int rtn = ReadControl<int>(Control)

template <class retT, class ContT>
retT ReadControl(ContT * Control)
{
	retT value;
	FromControl(Control, value);
	return value;
};

//==============================================================================
//  TSpeedButton
//==============================================================================
//==============================================================================
//  TToolButton
//==============================================================================
//==============================================================================
//  TGroupBox
//==============================================================================
//==============================================================================
//  System Operations
//==============================================================================
//--------------------------------------------------------------------------
//  Terminate() -- Quit app
//---------------------------------------------------------------------------

inline void Terminate()
{
	Forms::Application->Terminate();
}

//==============================================================================
//  Advanced Control Arrangement
//==============================================================================
//---------------------------------------------------------------------------
//  PlaceControl() -- Dynamically position a control in a row/column arrangement
//---------------------------------------------------------------------------

template <class ContT>
void  PlaceControl(ContT * Ctl,
				   int row, int total_rows,
				   int col, int total_cols,
				   int margin=5 )
{
	if (total_cols==0 || total_rows==0)
		return;
	if (col >= total_cols || row >= total_rows)
		return;

	Ctl->Top  = margin + row*Ctl->Parent->Height/total_rows;
	Ctl->Left = margin + col*Ctl->Parent->Width/total_cols;
}

//---------------------------------------------------------------------------
//  FitControl() -- Resize text box to fit in enclosing grid region
//---------------------------------------------------------------------------
//  Fit control in a virtual grid box, subdividing parent into row by col regions

template <class ContT>
void  FitControl(ContT * Ctl, int rows=1, int cols=1, int margin=5)
{
	const int MinFontSize = 7;
	bool  Increasing(false);

	while (1==1)
		{
		int col_width = Ctl->Parent->Width/cols;
		int row_height = Ctl->Parent->Height/rows;
		//  find which region control is in
		int col =  Ctl->Left/col_width;
		int row =  Ctl->Top/row_height;
		//  find offset into region
		int col_left = Ctl->Left - col*col_width;
		int row_top = Ctl->Top - row*row_height;

		int width_fit = col_width-col_left-Ctl->Width;
		int height_fit = row_height-row_top-Ctl->Height;
		if (width_fit<=0 || height_fit<=0)
			{
			Ctl->Font->Size -= 1;      // reduce to fit
			if (Ctl->Font->Size<MinFontSize)
				break;
			if (Increasing)
				break;   // We overshot the size while increasing, so stop
			}
		else if (width_fit>margin && height_fit>margin)
			{
			//  Misfit enought to make correction
			Ctl->Font->Size += 1;      // expand to fit
			Increasing = true;
			}
		else
			break;
		}
}


}  // end namespace Innovative

#endif
