// Win32CommandLine_Win.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Framework/Windows/CommandLine_Win.h>
#include <windows.h>

namespace   Innovative
{

//==============================================================================
//  CLASS Win32CommandLineArguments  --
//==============================================================================
//---------------------------------------------------------------------------
//  Win32CommandLineArguments::Win32CommandLineArguments() -- Ctor
//---------------------------------------------------------------------------

Win32CommandLineArguments::Win32CommandLineArguments()
    : m_cmdline(0)
{
    // Save local copy of the command line string, because
    // ParseCmdLine() modifies this string while parsing it.
    char * cmdline = GetCommandLine();
    m_cmdline = new char [strlen (cmdline) + 1];
    if (m_cmdline)
        {
        strcpy (m_cmdline, cmdline);
        ParseCmdLine();
        }
}

//---------------------------------------------------------------------------
//  Win32CommandLineArguments::~Win32CommandLineArguments() -- Dtor
//---------------------------------------------------------------------------

Win32CommandLineArguments::~Win32CommandLineArguments()
{
    delete [] m_cmdline;
    m_cmdline = 0;
}

//---------------------------------------------------------------------------
//  Win32CommandLineArguments::ParseCmdLine()
//
// Parse m_cmdline into individual tokens, which are delimited by spaces. If a
// token begins with a quote, then that token is terminated by the next quote
// followed immediately by a space or terminator.  This allows tokens to contain
// spaces.
// This input string:     This "is" a ""test"" "of the parsing" alg"o"rithm.
// Produces these tokens: This, is, a, "test", of the parsing, alg"o"rithm
//---------------------------------------------------------------------------

void Win32CommandLineArguments::ParseCmdLine()
{
	enum { TERM  = '\0', QUOTE = '\"' };

	char * pargs = m_cmdline;

	while (*pargs)
		{
		while (isspace(*pargs))        // skip leading whitespace
			pargs++;

		bool bInQuotes = (*pargs == QUOTE);  // see if this token is quoted

        if (bInQuotes)                  // skip leading quote
            pargs++;

        m_args.Add(std::string(pargs));              // store position of current token

        // Find next token.
        // NOTE: Args are normally terminated by whitespace, unless the
        // arg is quoted.  That's why we handle the two cases separately,
        // even though they are very similar.
        if (bInQuotes)
            {
            // find next quote followed by a space or terminator
            while (*pargs &&
                  !(*pargs == QUOTE && (isspace(pargs[1]) || pargs[1] == TERM)))
                pargs++;
            if (*pargs)
                {
                *pargs = TERM;  // terminate token
                if (pargs[1])   // if quoted token not followed by a terminator
                    pargs += 2; // advance to next token
                }
            }
        else
            {
            // skip to next non-whitespace character
            while (*pargs && !isspace (*pargs))
                pargs++;
            if (*pargs && isspace (*pargs)) // end of token
                {
                *pargs = TERM;    // terminate token
                pargs++;         // advance to next token or terminator
                }
            }
        } // while (*pargs)
}

//=============================================================================
//  Framework-specific retrieval of OS command-line
//=============================================================================

//---------------------------------------------------------------------------
//  GetCommandLine() --
//---------------------------------------------------------------------------

CommandLineArguments GetCommandLineArguments()
{
    Win32CommandLineArguments result;
    return result.Args();
}

}  // namespace
