// DosThread_Win.cpp
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
//
//   Background process manager
//

#include "DosThread_Win.h"
#include "Dos_Win.h"
#include <Exception_Mb.h>
#include <sstream>
#include <memory>
#include <StringSupport_Mb.h>

using namespace std;

namespace Innovative
{

static const int SBufSize = 0x4000;

//===========================================================================
//  CLASS  DosThread   ---  File finder support thread
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class DosThread
//---------------------------------------------------------------------------
// Thread is initially suspended.  Caller performs deletion.

DosThread::DosThread(Dos *owner)
    : Thread("DosThread"), Owner(owner),
      StartupInfo(new STARTUPINFO), ProcessInfo(new PROCESS_INFORMATION),
      StdIn(0), StdInRd(0), StdInWr(0),
      StdOut(0), StdOutRd(0), StdOutWr(0),
      SBuf(new char[SBufSize+1]), Aborting(false), Pausing(false)
{
    OpenHandles();

    // Priorities
    Priority(Owner->FPriority);

    Resume();
}

//---------------------------------------------------------------------------
//  destructor for class DosThread
//---------------------------------------------------------------------------

DosThread::~DosThread()
{
    delete StartupInfo;
    StartupInfo = 0;

    delete ProcessInfo;
    ProcessInfo = 0;

    delete [] SBuf;
    SBuf = 0;

    CloseHandles();
}

//------------------------------------------------------------------------------
//  DosThread::Init() -- Init multi-objects
//------------------------------------------------------------------------------

void DosThread::Init()
{
    //
    //  Initialize Actions with the objects we'll be waiting for:
    inherited::Init();
    AddObject(Spawning, sSpawn);
}

//---------------------------------------------------------------------------
//  DosThread::OpenHandles() -- Open handles for stdio
//---------------------------------------------------------------------------

void DosThread::OpenHandles()
{
    SECURITY_ATTRIBUTES saAttr;
    // Set the bInheritHandle flag so file handles are inherited.
    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
    saAttr.bInheritHandle = TRUE;
    saAttr.lpSecurityDescriptor = NULL;

    // Create a pipe for the child process's STDIN.
    if (!CreatePipe(&StdInRd, &StdInWr, &saAttr, 0))
        throw MalibuException("Stdin pipe creation failed");

    // Duplicate the write handle to the pipe so it is not inherited.
    if (!DuplicateHandle(GetCurrentProcess(), StdInWr,
        GetCurrentProcess(), &StdIn, 0, FALSE, DUPLICATE_SAME_ACCESS))
        {
        CloseHandles();
        throw MalibuException("DuplicateHandle failed");
        }

    if (StdInWr)
        cResult = CloseHandle(StdInWr) ? true : false;
    StdInWr = 0;

    StdOut = CreateFileA(
        Owner->FStdOutFile.c_str(),         // pointer to name of the file
		GENERIC_READ,	                    // access (read-write) mode
		FILE_SHARE_READ | FILE_SHARE_WRITE, // share mode
		NULL,                               // pointer to security attributes
		CREATE_ALWAYS,                      // how to create
		FILE_ATTRIBUTE_TEMPORARY,           // file attributes
		NULL);                              // handle to file with attributes to copy

	if (StdOut == INVALID_HANDLE_VALUE)
		{
    	int ErrorCode = GetLastError();
		CloseHandles();
        stringstream msg;
        msg << "Error: " << ErrorCode;
        throw MalibuException(msg.str());
		}

    StdOutWr = CreateFileA(
        Owner->FStdOutFile.c_str(),         // pointer to name of the file
        GENERIC_WRITE,	                    // access (read-write) mode
        FILE_SHARE_READ | FILE_SHARE_WRITE, // share mode
        &saAttr,                            // pointer to security attributes
        OPEN_EXISTING,                      // how to create
        FILE_ATTRIBUTE_TEMPORARY,           // file attributes
        NULL);                              // handle to file with attributes to copy

    if (StdOutWr == INVALID_HANDLE_VALUE)
        {
        int ErrorCode = GetLastError();
        CloseHandles();
        stringstream msg;
        msg << "Error: " << ErrorCode;
        throw MalibuException(msg.str());
        }
}

//---------------------------------------------------------------------------
//  DosThread::CloseHandles() -- Close any open file handles
//---------------------------------------------------------------------------

void DosThread::CloseHandles()
{
    if (StdIn)
        cResult = CloseHandle(StdIn) ? true : false;
    if (StdInRd)
		cResult = CloseHandle(StdInRd) ? true : false;
    if (StdInWr)
		cResult = CloseHandle(StdInWr) ? true : false;
    if (StdOut)
		cResult = CloseHandle(StdOut) ? true : false;
    if (StdOutRd)
		cResult = CloseHandle(StdOutRd) ? true : false;
    if (StdOutWr)
		cResult = CloseHandle(StdOutWr) ? true : false;

    StdIn = StdInRd = StdInWr = 0;
    StdOut = StdOutRd = StdOutWr = 0;
}

//---------------------------------------------------------------------------
//  DosThread::LaunchProcess() -- Init the command processor
//---------------------------------------------------------------------------

void DosThread::LaunchProcess()
{
    Owner->Response.Clear();

    DWORD ErrorCode = 0;

    // Init the StartupInfo block
    int mode;
    if (Owner->ShowMode() == Dos::wsHide)
        mode = SW_HIDE;
    if (Owner->ShowMode() == Dos::wsNormal)
        mode = SW_SHOW;
    if (Owner->ShowMode() == Dos::wsMaximize)
        mode = SW_MAXIMIZE;
    if (Owner->ShowMode() == Dos::wsMinimize)
        mode = SW_MINIMIZE;

	GetStartupInfo(StartupInfo);
    StartupInfo->lpTitle = const_cast<char*>(string("IIDos").c_str());
    DWORD flags = STARTF_USESHOWWINDOW;
    if (Owner->ShowMode() == Dos::wsHide)
        flags |= STARTF_USESTDHANDLES;
    StartupInfo->dwFlags = flags;
    StartupInfo->wShowWindow = mode;
    StartupInfo->hStdInput = StdInRd;
    StartupInfo->hStdOutput = StdOutWr;
    StartupInfo->hStdError = StdOutWr;

    string Cmd = Owner->CommandLine();
    if (Owner->Parameters().size())
        Cmd += " " + Owner->Parameters();

    DWORD Flags = 0;
    if (Owner->PriorityClass() == Dos::pcNormal)
        Flags |= NORMAL_PRIORITY_CLASS;
    if (Owner->PriorityClass() == Dos::pcIdle)
        Flags |= IDLE_PRIORITY_CLASS;
    if (Owner->PriorityClass() == Dos::pcRealTime)
        Flags |= REALTIME_PRIORITY_CLASS;
    if (Owner->PriorityClass() == Dos::pcHigh)
        Flags |= HIGH_PRIORITY_CLASS;

	char * Env = GetEnvironmentStrings();
    int status = CreateProcessA(NULL, const_cast<char*>(Cmd.c_str()),
		NULL,                       // pointer to process security attributes
		NULL,                       // pointer to thread security attributes
		true,                       // handle inheritance flag
		Flags,                      // creation flags
		Env,                        // pointer to new environment block
		// pointer to current directory name
        Owner->StartDir().size() ? Owner->StartDir().c_str() : NULL,
        StartupInfo,                // pointer to STARTUPINFO
        ProcessInfo);               // pointer to PROCESS_INFORMATION
    FreeEnvironmentStringsA(Env);

    if (!status)
        {
        ErrorCode = GetLastError();

        ProcessCompletionEvent e(ErrorCode);
        Owner->OnFinished.Execute(e);

        return;
        }

    Spawning.Release();
}

//---------------------------------------------------------------------------
// DosThread::Write() -- Write to StdIn
//---------------------------------------------------------------------------

bool DosThread::Write(void *Buffer, int Count)
{
    DWORD written;
	return WriteFile(StdIn, Buffer, Count, &written, NULL) ? true : false;
}

//---------------------------------------------------------------------------
//  DosThread::Eat() -- Gobble text trom std file
//---------------------------------------------------------------------------

bool DosThread::Eat(const HANDLE &Handle)
{
    DWORD read = 0;

    ReadFile(Handle, SBuf, SBufSize, &read, NULL);
    if (!read)
        return false;

    SBuf[read] = 0;
    string Message(SBuf);
    Owner->Response.Add(Message);

    ProcessStatusEvent e(Message);
    Owner->OnProgress.Execute(e);

    return true;
}

//---------------------------------------------------------------------------
//  DosThread::Execute() -- Idle behavior of support thread
//---------------------------------------------------------------------------

void DosThread::Execute(void)
{
    while (!Terminated())
        {
        Owner->FFinished = true;

        // WaitResult is either a) Suicide event, b) Shell event
        int WaitResult = WaitFor();

        switch (WaitResult)
            {
            //  Terminate:  Exit thread
            case sTerminate:
                Shutdown(false);
                break;
            //  Spawn:  Execute subordinate process
            case sSpawn:
                Spawn();
                break;
            }
        }
}

//---------------------------------------------------------------------------
//  DosThread::Spawn() -- Execute subordinate process
//---------------------------------------------------------------------------

void DosThread::Spawn()
{
    int ErrorCode = 0;

    // Wait for launched process to finish
    DWORD ExitCode = STILL_ACTIVE;
    while(!Aborting && !Terminated() && (ExitCode == STILL_ACTIVE))
        {
        while (Pausing)
            {
            Sleep(1);
            continue;
            }

        if (!Eat(StdOut))
            Sleep(10);

        if (!GetExitCodeProcess(ProcessInfo->hProcess, &ExitCode))
            ErrorCode = GetLastError();
        }

    if (Aborting || (ExitCode == STILL_ACTIVE))
        {
        if (Aborting || Owner->MustTerminate())
            {
            if (!TerminateProcess(ProcessInfo->hProcess, ExitCode))
                ErrorCode = GetLastError();
            }
        else
            cResult = CloseHandle(ProcessInfo->hProcess) ? true : false;
        }

    // Gobble residual text
    while (!Aborting && !Terminated() && Eat(StdOut))
        Sleep(1);

    ProcessCompletionEvent e(ErrorCode);
    Owner->OnFinished.Execute(e);
}

} // namespace Innovative
