// Dos_Mb.cpp
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
//
//   Shell to external process
//
//---------------------------------------------------------------------------

#include "Dos_Win.h"
#include "DosThread_Win.h"
#include <FileSupport_Mb.h>
#include <memory>
#include <string>
#include <sys/stat.h>
#include <SystemSupport_Mb.h>
#include <StringSupport_Mb.h>
#include <IniFile_Mb.h>
#include <windows.h>

namespace Innovative
{

static const int MaxEnvVarSize = 512;

//---------------------------------------------------------------------------
//  Constructor for class Dos
//---------------------------------------------------------------------------

Dos::Dos()
    : Interpreter(0), FShowMode(wsHide), FFinished(true),
      FWaitUntilFinished(false), FMustTerminate(false),
      FPriority(Thread::tpNormal)
{
    ReadEnvironment();

    char Buffer[MAX_PATH];
    GetWindowsDirectory(Buffer, MAX_PATH);
    string WinDir = Buffer;
    PathSpec spec(WinDir);
    string TmpDir = spec.Drive()+spec.Dir() + "temp\\";
    CreateDirectoryA(TmpDir.c_str(), NULL);

    char temp[MAX_PATH];
    GetTempFileNameA(TmpDir.c_str(), "Std", 0, temp);
    FStdInFile = string(temp);
    GetTempFileNameA(TmpDir.c_str(), "Std", 0, temp);
    FStdOutFile = string(temp);
    GetTempFileNameA(TmpDir.c_str(), "Std", 0, temp);
    FStdErrFile = string(temp);

    Interpreter = new DosThread(this);
}

//---------------------------------------------------------------------------
//  Destructor for class Dos
//---------------------------------------------------------------------------

Dos::~Dos()
{
    Interpreter->Shutdown(true);
    delete Interpreter;
    Interpreter = 0;

    DeleteFileA(FStdInFile.c_str());
    DeleteFileA(FStdOutFile.c_str());
    DeleteFileA(FStdErrFile.c_str());
}

//---------------------------------------------------------------------------
// Dos::Launch() -- Execute specified command line
//---------------------------------------------------------------------------

bool Dos::Launch()
{
    FFinished = false;
    FErrorCode = 0;

    Interpreter->LaunchProcess();

    while (WaitUntilFinished() && !Finished())
        {
        OpenWire::NotifyEvent e;
        OnProcessMessages.Execute(e);
        Innovative::Sleep(1);
        }

    return (WaitUntilFinished() && ErrorCode()) ? false : true;
}

//---------------------------------------------------------------------------
// Dos::Abort() -- Terminate executing command line
//---------------------------------------------------------------------------

void Dos::Abort(bool state)
{
    Interpreter->Abort(state);
}

//---------------------------------------------------------------------------
// Dos::Abort() -- Fetch abort status
//---------------------------------------------------------------------------

bool Dos::Abort() const
{
    return Interpreter->Abort();
}

//---------------------------------------------------------------------------
// Dos::Pause() -- Pause executing command line
//---------------------------------------------------------------------------

void Dos::Pause(bool state)
{
    Interpreter->Pause(state);
}

//---------------------------------------------------------------------------
// Dos::Pause() -- Fetch pause status
//---------------------------------------------------------------------------

bool Dos::Pause() const
{
    return Interpreter->Pause();
}

//---------------------------------------------------------------------------
// Dos::Write() -- Write to StdIn
//---------------------------------------------------------------------------

bool Dos::Write(void *Buffer, int Count)
{
    return Interpreter->Write(Buffer, Count);
}

//---------------------------------------------------------------------------
// Dos::SetStdInFile() -- Set name of std in file
//---------------------------------------------------------------------------

void Dos::SetStdInFile(string name)
{
    DeleteFileA(StdInFile().c_str());
    FStdInFile = name;
}

//---------------------------------------------------------------------------
// Dos::SetStdOutFile() -- Set name of std file
//---------------------------------------------------------------------------

void Dos::SetStdOutFile(string name)
{
    DeleteFileA(StdOutFile().c_str());
    FStdOutFile = name;
}

//---------------------------------------------------------------------------
// Dos::SetStdErrFile() -- Set name of std file
//---------------------------------------------------------------------------

void Dos::SetStdErrFile(string name)
{
    DeleteFileA(StdErrFile().c_str());
    FStdErrFile = name;
}

//---------------------------------------------------------------------------
// Dos::GetErrorMsg() -- Retreive error string
//---------------------------------------------------------------------------

string Dos::GetErrorMsg(int index) const
{
    LPVOID lpMsgBuf;

    int Size = FormatMessageA(
        FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
        NULL,
        index,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
        (LPTSTR) &lpMsgBuf,
        0,
        NULL
    );

    string Result((LPTSTR)lpMsgBuf, Size);

    // Free the buffer.
    LocalFree( lpMsgBuf );

    return Result;
}

//---------------------------------------------------------------------------
//  Dos::ReadBatchFile() --  Parse the batch file
//---------------------------------------------------------------------------

void Dos::ReadBatchFile(const string & Filename)
{
    StringList Env;
    Env.LoadFromFile(Filename);

    char *EnvVar = new char[MaxEnvVarSize];

    // Update environment
    for (size_t i = 0; i < Env.Count(); i++)
        {
        string line = Env.At(i);
        size_t delim = line.find("=");
        if (delim == string::npos)
            continue;

        string Name = line.substr(0, delim);
        if (!Name.size())
            continue;
        line.erase(0, delim+1);
        string Value = ParseLine(line, EnvVar);

        if (LowerCase(Name).find("set ") != string::npos)
            Name.erase(0, 4);
        SetEnvironmentVariableA(Name.c_str(), Value.c_str());
        }

    delete [] EnvVar;

    ReadEnvironment();
}

//---------------------------------------------------------------------------
//  Dos::ParseLine() --  Parse the batch file line
//---------------------------------------------------------------------------

string Dos::ParseLine(string Line, char *EnvVar)
{
    string Result;

    while (Line.size())
        {
        if (Line[0] != '%')
            {
            Result += Line[0];
            Line.erase(0, 1);
            continue;
            }

        char * Src = new char[Line.size()+1];
        strcpy(Src, Line.c_str());
        string Macro = ExtractQuotedStr(Src, '%');
        Line.erase(0, Macro.size()+2);

        int Size = GetEnvironmentVariableA(Macro.c_str(), EnvVar, MaxEnvVarSize);
        if (Size)
            Result += string(EnvVar);
        }

    return Result;
}

//---------------------------------------------------------------------------
//  Dos::ReadEnvironment() --
//---------------------------------------------------------------------------

void Dos::ReadEnvironment()
{
    FEnvironment.Clear();

    char * Env = GetEnvironmentStrings();
    char * p = Env;

    while (size_t len = strlen(p))
        {
        FEnvironment.Add(string(p));
        p += len+1;
        }

    FreeEnvironmentStringsA(Env);
}

} // namespace Innovative

//---------------------------------------------------------------------------

