// Dos_Win.h
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
//
//   Shell to external process
//

#ifndef Dos_WinH
#define Dos_WinH

#include <string>
#include <StringList_Mb.h>
#include <ProcessEvents_Mb.h>
#include <Thread_Mb.h>
#include <StringList_Mb.h>

using namespace std;
namespace Innovative
{

//---------------------------------------------------------------------------
class DosThread;

class Dos
{
public:
    enum IIShowMode { wsHide, wsNormal, wsMaximize, wsMinimize };
    enum IIPriorityClass { pcNormal, pcIdle, pcRealTime, pcHigh };

    // Ctors
    Dos();
    ~Dos();
    bool                Launch();
    bool                Write(void *Buffer, int Count);
    void                ReadBatchFile(const string &Filename);

    // Properties
    bool  Abort() const;
    void  Abort(bool state);

    bool  Pause() const;
    void  Pause(bool state);

    string & StdInFile()
        {  return FStdInFile;  }
    void StdInFile(const std::string & name)
        {  FStdInFile=name;  }

    string & StdOutFile()
        {  return FStdOutFile;  }
    void StdOutFile(const std::string & name)
        {  FStdOutFile=name;  }

    string & StdErrFile()
        {  return FStdErrFile;  }
    void StdErrFile(const std::string & name)
        {  FStdErrFile=name;  }

    StringList & Environment()
        {  return FEnvironment;  }

    bool Finished() const
        {  return FFinished;  }
    int ErrorCode() const
        {  return FErrorCode;  }
    string ErrorMsg(int index) const
        {  return GetErrorMsg(index);  }

    string & StartDir()
        {  return FStartDir;  }
    void StartDir(const std::string & name)
        {  FStartDir = name;  }

    string & CommandLine()
        {  return FCommandLine;  }
    void CommandLine(const std::string & cmd)
        {  FCommandLine = cmd;  }

    string & Parameters()
        {  return FParameters;  }
    void Parameters(const std::string & parms)
        {  FParameters = parms;  }

    IIShowMode ShowMode() const
        {  return FShowMode;  }
    void ShowMode(IIShowMode mode)
        {  FShowMode = mode;  }

    bool WaitUntilFinished() const
        {  return FWaitUntilFinished;  }
    void WaitUntilFinished(bool state)
        {  FWaitUntilFinished = state;  }

    bool MustTerminate() const
        {  return FMustTerminate;  }
    void MustTerminate(bool state)
        {  FMustTerminate = state;  }

    IIPriorityClass PriorityClass() const
        {  return FPriorityClass;  }
    void PriorityClass(IIPriorityClass value)
        {  FPriorityClass = value;  }

    Thread::ThreadPriority Priority() const
        {  return FPriority;  }
    void Priority(Thread::ThreadPriority priority)
        {  FPriority = priority;  }

    OpenWire::SynchronizedEventHandler<ProcessStatusEvent>      OnProgress;
    OpenWire::SynchronizedEventHandler<ProcessCompletionEvent>  OnFinished;
    OpenWire::EventHandler<OpenWire::NotifyEvent>               OnProcessMessages;

private:
    friend class DosThread;

    // Data
    string                  FCommandLine;
    string                  FParameters;
    string                  FStartDir;
    DosThread             * Interpreter;
    StringList              Response;
    string                  FStdInFile;
    string                  FStdOutFile;
    string                  FStdErrFile;
    StringList              FEnvironment;
    IIShowMode              FShowMode;
    bool                    FFinished;
    bool                    FWaitUntilFinished;
    bool                    FMustTerminate;
    IIPriorityClass         FPriorityClass;
    Thread::ThreadPriority  FPriority;
    int                     FErrorCode;

    // Methods
    void                    SetCommand(string cmd);
    void                    SetEnabled(bool state);
    void                    SetStdInFile(string name);
    void                    SetStdOutFile(string name);
    void                    SetStdErrFile(string name);
    string                  GetErrorMsg(int index) const;

    string                  ParseLine(string Line, char *Scratch);
    void                    ReadEnvironment();
};

} // namespace Innovative

//---------------------------------------------------------------------------
#endif
