// FileFind_Mb.cpp
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
//   Background file search
//
//---------------------------------------------------------------------------

#include "FileFind_Win.h"
#include "FileFindThread_Win.h"
#include <SystemSupport_Mb.h>
#include <memory>

using namespace std;

namespace Innovative
{

//---------------------------------------------------------------------------
//  Constructor for class FileFind
//---------------------------------------------------------------------------

FileFind::FileFind()
    : Finder(new FileFindThread(this)), FRecursive(true), FFinished(true),
      FAllowDuplicates(false)
{
}

//---------------------------------------------------------------------------
//  Destructor for class FileFind
//---------------------------------------------------------------------------

FileFind::~FileFind()
{
    delete Finder;
    Finder = 0;
}

//---------------------------------------------------------------------------
// FileFind::Search() -- Search specified directory tree
//---------------------------------------------------------------------------

void FileFind::Search(const StringList & directories, const StringList & filespecs)
{
    if (!Finished())
        Abort();

    FFinished = false;
    FFiles.Clear();
    FDirectories.Clear();

    Finder->Start(directories, filespecs, Recursive());

    while (WaitUntilFinished() && !Finished())
        HandleProcessMessages();
}

//---------------------------------------------------------------------------
// FileFind::Search() -- Search specified directory tree
//---------------------------------------------------------------------------

void FileFind::Search(const string & directory, const string & filespec)
{
    StringList directories;
    directories.Add(directory);

    StringList files;
    files.Add(filespec);

    Search(directories, files);
}

//---------------------------------------------------------------------------
// FileFind::Abort() -- Abort search in-progress
//---------------------------------------------------------------------------

void FileFind::Abort()
{
    Finder->Stop();
}

//---------------------------------------------------------------------------
// FileFind::HandleFileFound() -- Fire the file find event
//---------------------------------------------------------------------------

void FileFind::HandleFileFound(const string & s)
{
    FFiles.Add(s);

    DiskScanEvent e(s, FFiles.Count());
    OnFileFound.Execute(e);
}

//---------------------------------------------------------------------------
// FileFind::HandleDirectoryFound() -- Fire the (sub)directory find event
//---------------------------------------------------------------------------

void FileFind::HandleDirectoryFound(const string & s)
{
    FDirectories.Add(s);

    DiskScanEvent e(s, FDirectories.Count());
    OnDirectoryFound.Execute(e);
}

//---------------------------------------------------------------------------
// FileFind::HandleProcessMessages() -- Fire the message loop event
//---------------------------------------------------------------------------

void FileFind::HandleProcessMessages()
{
    OpenWire::NotifyEvent e;
    OnProcessMessages.Execute(e);
}

//---------------------------------------------------------------------------
// FileFind::HandleScanFinished() -- Fire the scan finished event
//---------------------------------------------------------------------------

void FileFind::HandleScanFinished(int result)
{
    FFinished = true;

    ProcessCompletionEvent e(result);
    OnFinished.Execute(e);
}


} // namespace Innovative

//---------------------------------------------------------------------------
