//---------------------------------------------------------------------------
#ifndef IIFileFindH
#define IIFileFindH

#include <ProcessEvents_Mb.h>
#include <StringList_Mb.h>

namespace Innovative
{

class FileFindThread;

//===========================================================================
// CLASS DiskScanEvent  --  In process status reporting
//===========================================================================

class DiskScanEvent : public OpenWire::Event
{
public:
    std::string Message;
    int Count;

    DiskScanEvent(const std::string & msg, int count)
        :  Message(msg), Count(count)
        {
        }
};

class FileFindThread;

class FileFind
{
public:
    // Ctors
    FileFind();
    ~FileFind();

    // Methods
    void                Search();
    void                Search(const Innovative::StringList & directories, const Innovative::StringList & filespecs);
    void                Search(const std::string & directory, const std::string & filespec);
    void                Abort();

    // Properties
    Innovative::StringList & Files()
        {  return FFiles;  }
    Innovative::StringList & Directories() 
        {  return FDirectories;  }
    bool Finished() const
        {  return FFinished;  }

    bool AllowDuplicates() const
        {  return FAllowDuplicates;  }
    void AllowDuplicates(bool state)
        {  FAllowDuplicates = state;  }

    bool Recursive() const
        {  return FRecursive;  }
    void Recursive(bool state)
        {  FRecursive = state;  }

    bool WaitUntilFinished() const
        {  return FWaitUntilFinished;  }
    void WaitUntilFinished(bool state)
        {  FWaitUntilFinished = state;  }

    OpenWire::SynchronizedEventHandler<DiskScanEvent>            OnFileFound;
    OpenWire::SynchronizedEventHandler<DiskScanEvent>            OnDirectoryFound;
    OpenWire::SynchronizedEventHandler<OpenWire::NotifyEvent>    OnProcessMessages;
    OpenWire::SynchronizedEventHandler<ProcessCompletionEvent>   OnFinished;

private:
    friend class FileFindThread;

    // Data
    FileFindThread *        Finder;
    Innovative::StringList  FFiles;
    Innovative::StringList  FDirectories;
    bool                    FRecursive;
    bool                    FFinished;
    bool                    FWaitUntilFinished;
    bool                    FAllowDuplicates;

    // Methods
    void                    HandleFileFound(const std::string & s);
    void                    HandleDirectoryFound(const std::string & s);
    void                    HandleProcessMessages();
    void                    HandleScanFinished(int result);
};

} // namespace Innovative

#endif
