// MailSlot_Win.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef MailSlot_WinH
#define MailSlot_WinH

#include <Events_Mb.h>
#include <string>
#include <StopWatch_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS  MailSlotMessage   ---
//===========================================================================

class MailSlotMessage : public OpenWire::NotifyEvent
{
    typedef std::vector<char>   CharArray;

public:
    // Ctor
    MailSlotMessage(CharArray & msg);

	// Data
    CharArray       Body;
    std::string     Computer;
    std::string     User;
    std::string     Id;
};

#ifdef __BORLANDC__
	#pragma option push
	#pragma warn -8026
#endif

//===========================================================================
//  CLASS  MailSlot   ---  Mail slot component
//===========================================================================
class SlotThread;

class MailSlot
{
    friend class SlotThread;

public:
    typedef std::vector<char>   CharArray;

    // Methods
    MailSlot();
    ~MailSlot();

    // Server
    bool        OpenServer();
    bool        CloseServer();
    // Client
    bool        OpenClient(std::string Target, std::string Slot);
    bool        OpenClient(std::string Slot);
    bool        CloseClient();
    bool        Write(CharArray & msg);
    bool        Write(std::string msg);

    // Properties
    HANDLE          ServerMailSlot() const
        {  return FServerMailSlot;  }
    HANDLE          ClientMailSlot() const
        {  return FClientMailSlot;  }
    std::string     ComputerName() const
        {  return FComputerName;  }
    std::string     UserName() const
        {  return FUserName;  }

    void            SlotName(std::string value)
        {  FSlotName=value;  }
    std::string     SlotName() const
        {  return FSlotName;  }

    OpenWire::SynchronizedEventHandler<Innovative::MailSlotMessage> OnMessage;

private:
    // Fields
    std::string             FSlotName;
    std::string             FComputerName;
    std::string             FUserName;
    HANDLE                  FServerMailSlot;
    HANDLE                  FClientMailSlot;

    // Data
    SlotThread *                        Mailer;
    //class Innovative::Windows::Mutex *  Lock;
    Innovative::StopWatch               Rtc;
    CharArray                           Scratch;

    // Methods
    bool                    Read(CharArray & buffer);
};

#ifdef __BORLANDC__
	#pragma option pop
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative

//---------------------------------------------------------------------------
#endif
