// MemDrvCtrl_Win.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <windows.h>
#include <winioctl.h>
#include "MemDrvCtrl_Win.h"
#include <iostream>

using namespace std;

namespace Innovative
{
HANDLE hDevice;

MemDrvCtrl::MemDrvCtrl(void)
{
    //
    // open memory driver
    //
    hDevice = CreateFile(DEVICE_NAME,
                         GENERIC_READ | GENERIC_WRITE,
                         FILE_SHARE_READ | FILE_SHARE_WRITE,
                         NULL,
                         OPEN_ALWAYS,   
                         FILE_ATTRIBUTE_NORMAL,
                         NULL);
}

MemDrvCtrl::~MemDrvCtrl()
{
    CloseHandle ( hDevice );
}

bool MemDrvCtrl::ResetLMemDrv()
{
    bool status = false;
    ULONG bytesReturned;
    DrvOutputType drvOut;
    DrvInputType drvIn;

    if(hDevice != INVALID_HANDLE_VALUE) 
    {
        // reset driver
        status = (DeviceIoControl ( 
                        hDevice,
                        (DWORD)Low_IOCTL_RESET_MEM_POOL,
                        &drvIn,
                        sizeof(DrvInputType),
                        &drvOut,
                        sizeof(DrvOutputType),
                        &bytesReturned,
                        NULL
                        ) == TRUE);
       
    }

    return status;
}

bool MemDrvCtrl::ResetHMemDrv()
{
    bool status = false;
    ULONG bytesReturned;
    DrvOutputType drvOut;
    DrvInputType drvIn;

    if(hDevice != INVALID_HANDLE_VALUE)
    {
        // reset driver
        status = (DeviceIoControl (
                        hDevice,
                        (DWORD)High_IOCTL_RESET_MEM_POOL,
                        &drvIn,
                        sizeof(DrvInputType),
                        &drvOut,
                        sizeof(DrvOutputType),
                        &bytesReturned,
                        NULL
                        ) == TRUE);

    }

    return status;
}

bool MemDrvCtrl::GetLMemDrvStatus(DrvOutputType &drvOut)
{
    bool bRc=false;
    ULONG bytesReturned=0;    
    DrvInputType drvIn;

    drvOut.MaxBytes = 0;
    drvOut.BytesFree = 0;
    drvOut.LargestBlock = 0;

    if(hDevice != INVALID_HANDLE_VALUE) 
    {
        bRc = (DeviceIoControl ( hDevice,
                            (DWORD)Low_IOCTL_DUMP_MEM_POOL,
                            &drvIn,
                            sizeof(DrvInputType),
                            &drvOut,               
                            sizeof(DrvOutputType),
                            &bytesReturned,
                            NULL
                            )== TRUE);
    }

    return (bRc);
}

bool MemDrvCtrl::GetHMemDrvStatus(DrvOutputType &drvOut)
{
    bool bRc=false;
    ULONG bytesReturned=0;
    DrvInputType drvIn;

    drvOut.MaxBytes = 0;
    drvOut.BytesFree = 0;
    drvOut.LargestBlock = 0;

    if(hDevice != INVALID_HANDLE_VALUE)
    {
        bRc = (DeviceIoControl ( hDevice,
                            (DWORD)High_IOCTL_DUMP_MEM_POOL,
                            &drvIn,
                            sizeof(DrvInputType),
                            &drvOut,
                            sizeof(DrvOutputType),
                            &bytesReturned,
                            NULL
                            )== TRUE);
    }

    return (bRc);
}

bool MemDrvCtrl::TestLMemDrv(size_t bytes)
{
    size_t status = 1;
    ULONG bytesReturned;
    DrvInputType memRequest, freeMem;
    DrvOutputType DmaBuf;
    const size_t TESTBLKSIZE = 1024;
    const int bTest = 0xAB;
    PVOID dest1, dest2;

    //
    // test: reserve mem blocks
    //
    memRequest.RequestBytes = bytes;

    if(hDevice != INVALID_HANDLE_VALUE) 
    {
        if (DeviceIoControl (hDevice,
                            (DWORD)Low_IOCTL_ALLOC_MEMORY_BLOCK,
                            &memRequest,
                            sizeof(DrvInputType),
                            &DmaBuf,  //OutputBuffer,
                            sizeof(DrvOutputType),
                            &bytesReturned,
                            NULL
                            ) == TRUE)
        {
            status = status << 1;

            memset(DmaBuf.UserVirtualAddr, bTest, TESTBLKSIZE);            

            dest1 = (PVOID)((LONG_PTR)DmaBuf.UserVirtualAddr+(bytes/2));
            memset(dest1, bTest, TESTBLKSIZE);

            dest2 = (PVOID)((LONG_PTR)DmaBuf.UserVirtualAddr+bytes-TESTBLKSIZE);
            memset(dest2, bTest, TESTBLKSIZE);

            if (!memcmp(DmaBuf.UserVirtualAddr, dest1, TESTBLKSIZE))
                status = status << 1;

            if (!memcmp(DmaBuf.UserVirtualAddr, dest2, TESTBLKSIZE))
                status = status << 1;

            freeMem.UserVirtualAddr = DmaBuf.UserVirtualAddr;

            if (DeviceIoControl ( hDevice,
                                (DWORD)Low_IOCTL_FREE_MEMORY_BLOCK,
                                &freeMem,
                                sizeof(DrvInputType),
                                &DmaBuf,
                                sizeof(DrvOutputType),
                                &bytesReturned,
                                NULL
                                ) == TRUE)                                                
                                    status = status << 1;

        }
    }

    return (status == 0x10);
}

bool MemDrvCtrl::TestHMemDrv(size_t bytes)
{
    size_t status = 1;
    ULONG bytesReturned;
    DrvInputType memRequest, freeMem;
    DrvOutputType DmaBuf;
    const size_t TESTBLKSIZE = 1024;
    const int bTest = 0xAB;
    PVOID dest1, dest2;

    //
    // test: reserve mem blocks
    //
    memRequest.RequestBytes = bytes;

    if(hDevice != INVALID_HANDLE_VALUE)
    {
        if (DeviceIoControl (hDevice,
                            (DWORD)High_IOCTL_ALLOC_MEMORY_BLOCK,
                            &memRequest,
                            sizeof(DrvInputType),
                            &DmaBuf,  //OutputBuffer,
                            sizeof(DrvOutputType),
                            &bytesReturned,
                            NULL
                            ) == TRUE)
        {
            status = status << 1;

            memset(DmaBuf.UserVirtualAddr, bTest, TESTBLKSIZE);

            dest1 = (PVOID)((LONG_PTR)DmaBuf.UserVirtualAddr+(bytes/2));
            memset(dest1, bTest, TESTBLKSIZE);

            dest2 = (PVOID)((LONG_PTR)DmaBuf.UserVirtualAddr+bytes-TESTBLKSIZE);
            memset(dest2, bTest, TESTBLKSIZE);

            if (!memcmp(DmaBuf.UserVirtualAddr, dest1, TESTBLKSIZE))
                status = status << 1;

            if (!memcmp(DmaBuf.UserVirtualAddr, dest2, TESTBLKSIZE))
                status = status << 1;

            freeMem.UserVirtualAddr = DmaBuf.UserVirtualAddr;

            if (DeviceIoControl ( hDevice,
                                (DWORD)High_IOCTL_FREE_MEMORY_BLOCK,
                                &freeMem,
                                sizeof(DrvInputType),
                                &DmaBuf,
                                sizeof(DrvOutputType),
                                &bytesReturned,
                                NULL
                                ) == TRUE)
                                    status = status << 1;

        }
    }

    return (status == 0x10);
}

//---------------------------------------------------------------------------
//  MemDrvCtrl::RegWrite() -- write to register
//      subkey - path to key, excluding major key ex: \system\services\memdrv
//      key - name of key
//---------------------------------------------------------------------------
bool MemDrvCtrl::RegWrite(const std::string & subkey, const std::string & key, const size_t value)
{
    HKEY    hKey = 0;  

    // Create key
    char   type = REG_DWORD;
    DWORD   disposition = 0;

    long ret = ::RegCreateKeyEx(
      HKEY_LOCAL_MACHINE, subkey.c_str(), 0, &type, REG_OPTION_NON_VOLATILE,
      KEY_SET_VALUE, NULL, &hKey, &disposition);

    if (ret == ERROR_SUCCESS)
    {
        ret = ::RegSetValueEx(hKey, key.c_str(), 0, REG_DWORD,
			reinterpret_cast < const BYTE * > (&value), sizeof(ULONG));

        ::RegCloseKey(hKey);
    }

    return (ret == ERROR_SUCCESS);
}

//---------------------------------------------------------------------------
//  MemDrvCtrl::RegRead() --
//  out: value - value from register key
//---------------------------------------------------------------------------
bool MemDrvCtrl::RegRead(const std::string & subkey, const std::string & key, size_t &value)
{
    HKEY    hKey = 0;

    long    ret = ::RegOpenKeyEx(
      HKEY_LOCAL_MACHINE, subkey.c_str(), 0, KEY_READ, &hKey);

    if (ret != ERROR_SUCCESS)
        return false;

    DWORD   type = REG_DWORD;
    DWORD   sz;

    ret = ::RegQueryValueEx(hKey, key.c_str(), NULL, &type,
        reinterpret_cast < LPBYTE > (&value), &sz);

    ::RegCloseKey(hKey);

    return (ret == ERROR_SUCCESS);
}


}
