// MultiMedia_Win.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <iostream>
#include <fstream>
#include <string>
#include <process.h>
#include "MultiMedia_Win.h"

using namespace std;

namespace   Innovative
{

#ifdef __BORLANDC__
    #pragma option push
    #pragma warn -8060
#endif

namespace Windows
{

static const int Bits = 16;
static const float Ceiling = (1 << (Bits-1)) - 1;

//---------------------------------------------------------------------------
// MultiMedia::MultiMedia() --
//---------------------------------------------------------------------------

MultiMedia::MultiMedia()
    : m_pcmRead(NULL), m_pcmWrite(NULL), m_inRead(false), 
      m_initialized(false), m_ir(0), m_iw(0)
{
    Timer.OnElapsed.SetEvent(this, &MultiMedia::HandleTimer);
    Timer.Interval(50);
}

//---------------------------------------------------------------------------
// MultiMedia::~MultiMedia() --
//---------------------------------------------------------------------------

MultiMedia::~MultiMedia()
{
    Finalize();
}

//---------------------------------------------------------------------------
//  ErrorExit() --
//---------------------------------------------------------------------------

void ErrorExit(const MMRESULT err, const char *const str)
{
    char    buffer[256];
    string  s("Error: ");

    if (mciGetErrorString(err, &buffer[0], sizeof(buffer)))
        {
        s += str;
        s += ": ";
        s += buffer;
        s += "\n";
        throw s.c_str();
        }
    else
        {
        s += str;
        s += "\n";
        throw s.c_str();
        }
}

//---------------------------------------------------------------------------
//  MultiMedia::QueueWriteData() --
//---------------------------------------------------------------------------

void MultiMedia::QueueWriteData(WAVEHDR*  whdr)
{
    DWORD   i;
    for (i = 0; i < whdr->dwBufferLength / 2; ++i)
        {
        if (mWriteBuf)
            {
            m_iw %= mSamples;
            short sample = static_cast<short>(mAmplitude * Ceiling * mWriteBuf[m_iw]);
            reinterpret_cast<short *>(whdr->lpData)[i] = sample;
            }
        else
            {
            if (m_iw > DDS_START_DELAY)
                {
                if (mFrequency.size() &&
                    (m_iw - DDS_START_DELAY) > mPositions[0])
                    {
                    mFrequency.erase(mFrequency.begin());
                    mPositions.erase(mPositions.begin());
                    }

                if (mFrequency.size())
                    {
                    short sample = static_cast<short>(mAmplitude * Ceiling * mDds.Sin());
                    reinterpret_cast<short *>(whdr->lpData)[i] = sample;
                    }
                else
                    reinterpret_cast<short *>(whdr->lpData)[i] = 0;
                if (!mStereo || i % 2)
                    mDds.Advance(mFrequency[0], mSampleRate);
                }
            else
                {
                reinterpret_cast<short *>(whdr->lpData)[i] = 0;
                }
            }

        if (mStereo)
            m_iw += i % 2;
        else
            m_iw++;
        }

    whdr->dwFlags &= ~WHDR_DONE;

    MMRESULT    err;

    err = waveOutWrite(m_pcmWrite, whdr, sizeof(WAVEHDR));
    if (err)
        {
        ErrorExit(err, "When writing wave data");
        }
}

//---------------------------------------------------------------------------
//  MultiMedia::WaveThread() --
//---------------------------------------------------------------------------

DWORD WINAPI MultiMedia::WaveThread(LPVOID arg)
{
    MultiMedia*  wmm = (MultiMedia*)arg;
    MSG     msg;

    while (GetMessage(&msg, 0, 0, 0) == 1)
        {
        WAVEHDR*    whdr = (WAVEHDR*)msg.lParam;

        switch (msg.message)
            {

            //Input messages
            case MM_WIM_DATA:
                    {
                    int bytesRecorded = whdr->dwBytesRecorded;

                    if (wmm->m_inRead && whdr->dwBytesRecorded == 0)
                        {
                        //Assume buggy driver and set bytesRecorded to buffer
                        //length
                        //This may cause problems when using long buffers and
                        //short MLS length
                        bytesRecorded = whdr->dwBufferLength;
                        }

                    if (!whdr->dwBytesRecorded &&
                        (wmm->m_inRead || (!wmm->m_inRead && !wmm->m_totalRecorded)))
                        {
                        wmm->mDebug.Log("Warning: Buggy driver detected\n");
                        wmm->mDebug.Log("dwBytesRecorded = ", static_cast<int>(whdr->dwBytesRecorded));
                        wmm->mDebug.Log("dwBufferLength = ", static_cast<int>(whdr->dwBufferLength));
                        }

                    if (bytesRecorded)
                        {
                        int channels = 1;
                        if (wmm->mStereo)
                            channels = 2;

                        for (int i = 0; i < bytesRecorded / 2; ++i)
                            {
                            if (wmm->mWriteBuf)
                                {
                                //Wrap around record buffer if MLS
                                wmm->mReadBuf[wmm->m_ir++] = float(reinterpret_cast<short *>(whdr->lpData)[i]) / Ceiling;
                                wmm->m_ir %= wmm->mSamples * channels;
                                }
                            else if (wmm->m_ir < wmm->mSamples * channels)
                                {
                                //Don't wrap sweep input
                                wmm->mReadBuf[wmm->m_ir++] = float(reinterpret_cast<short *>(whdr->lpData)[i]) / Ceiling;
                                }

                            wmm->Progress(++wmm->m_totalRecorded);
                            }
                        }

                    if (wmm->m_inRead)
                        {
                        whdr->dwFlags &= ~WHDR_DONE;

                        //whdr->dwFlags = 0;
                        waveInAddBuffer(wmm->m_pcmRead, whdr, sizeof(WAVEHDR));
                        }
                    else
                        {
                        if (wmm->m_totalRecorded < wmm->mSamples)
                            {
                            throw "Error: Not enough samples recorded!\n";
                            }

                        wmm->m_readComplete++;
                        }
                    }

                break;

            case MM_WIM_OPEN:
                wmm->m_readComplete = 0;
                wmm->m_totalRecorded = 0;
                break;

            case MM_WIM_CLOSE:
                break;

            //Output messages
            case MM_WOM_DONE:
                if (wmm->m_inWrite)
                    {
                    wmm->QueueWriteData(whdr);
                    }
                else
                    {
                    wmm->m_writeComplete++;
                    }

                break;

            case MM_WOM_OPEN:
                wmm->m_writeComplete = 0;
                break;

            case MM_WOM_CLOSE:
                return 0;
            }
        }

    return 0;
}

//---------------------------------------------------------------------------
//  MultiMedia::Initialize() --
//---------------------------------------------------------------------------

void MultiMedia::Initialize(double nSampleRate, bool bStereo)
{
    Finalize();

    mSampleRate = nSampleRate;
    mStereo = bStereo;

    mDebug.Log("MultiMedia::initialise");

    MMRESULT    err;

    m_waveFormat.wFormatTag = WAVE_FORMAT_PCM;
    if (mStereo)
        m_waveFormat.nChannels = 2;
    else
        m_waveFormat.nChannels = 1;
    m_waveFormat.nSamplesPerSec = int(mSampleRate);
    m_waveFormat.wBitsPerSample = Bits;
    m_waveFormat.nBlockAlign = m_waveFormat.nChannels * (m_waveFormat.wBitsPerSample / 8);
    m_waveFormat.nAvgBytesPerSec = m_waveFormat.nSamplesPerSec * m_waveFormat.nBlockAlign;
    m_waveFormat.cbSize = 0;

    DWORD   tid;
    HANDLE  waveInThread = CreateThread(0,
                                        0,
                                        (LPTHREAD_START_ROUTINE) WaveThread,
                                        this,
                                        0,
                                        &tid);
    if (!waveInThread)
        {
        throw "Error when creating WAVE event thread!\n";
        }

    CloseHandle(waveInThread);

    err = waveOutOpen(&m_pcmWrite,
                      WAVE_MAPPER,
                      &m_waveFormat,
                      (DWORD) tid,
                      0,
                      CALLBACK_THREAD);
    if (err)
        {
        ErrorExit(err, "When opening output device");
        }

    err = waveInOpen(&m_pcmRead,
                     WAVE_MAPPER,
                     &m_waveFormat,
                     (DWORD) tid,
                     0,
                     CALLBACK_THREAD);
    if (err)
        {
        ErrorExit(err, "When opening input device");
        }

    mDebug.Log("MultiMedia::initialize - end");
    m_initialized = true;
}

//---------------------------------------------------------------------------
//  MultiMedia::Start() --
//---------------------------------------------------------------------------

void MultiMedia::Start()
{
    Progress(0);

    mDebug.Log("MultiMedia::start");

    MMRESULT    err;

    //Reset buffer
    m_ir = 0;
    m_iw = 0;

    m_readComplete = 0;
    m_writeComplete = 0;
    m_totalRecorded = 0;

    memset(&m_waveReadHeader[0], 0, sizeof(WAVEHDR));
    memset(&m_waveReadHeader[1], 0, sizeof(WAVEHDR));
    memset(&m_waveWriteHeader[0], 0, sizeof(WAVEHDR));
    memset(&m_waveWriteHeader[1], 0, sizeof(WAVEHDR));

    //Too long buffer makes the Audigy2 driver behave strange
    m_waveWriteHeader[1].dwBufferLength = m_waveWriteHeader[0].dwBufferLength =
            m_waveReadHeader[1].dwBufferLength = m_waveReadHeader[0].
                dwBufferLength = m_waveFormat.nAvgBytesPerSec >>
        1;
        
    m_waveReadHeader[0].lpData =
        (char*)VirtualAlloc(0, m_waveReadHeader[0].dwBufferLength * 4,
                            MEM_COMMIT,
                            PAGE_READWRITE);

    if (!m_waveReadHeader[0].lpData)
        {
        throw "Error allocating wave buffers\n";
        }

    m_waveReadHeader[1].lpData = m_waveReadHeader[0].lpData + m_waveReadHeader[0].dwBufferLength;
    m_waveWriteHeader[0].lpData = m_waveReadHeader[0].lpData +
        m_waveReadHeader[0].dwBufferLength *
        2;
    m_waveWriteHeader[1].lpData = m_waveReadHeader[0].lpData +
        m_waveReadHeader[0].dwBufferLength *
        3;

    //Leave other WAVEHDR fields at 0
    if ((err = waveInPrepareHeader(m_pcmRead,
                                   &m_waveReadHeader[0],
                                   sizeof(WAVEHDR))) ||
        (err = waveInPrepareHeader(m_pcmRead,
                                   &m_waveReadHeader[1],
                                   sizeof(WAVEHDR))) ||
        (err = waveOutPrepareHeader(m_pcmWrite,
                                    &m_waveWriteHeader[0],
                                    sizeof(WAVEHDR))) ||
        (err = waveOutPrepareHeader(m_pcmWrite,
                                    &m_waveWriteHeader[1],
                                    sizeof(WAVEHDR))))
        {
        ErrorExit(err, "When preparing wave headers");
        }

    if ((err = waveInAddBuffer(m_pcmRead,
                               &m_waveReadHeader[0],
                               sizeof(WAVEHDR))) ||
        (err = waveInAddBuffer(m_pcmRead,
                               &m_waveReadHeader[1],
                               sizeof(WAVEHDR))))
        {
        ErrorExit(err, "When queueing wave headers");
        }

    //Start recording
    m_inRead = true;
    if ((err = waveInStart(m_pcmRead)))
        {
        ErrorExit(err, "When starting record");
        }

    m_inWrite = true;
    QueueWriteData(&m_waveWriteHeader[0]);
    QueueWriteData(&m_waveWriteHeader[1]);

    mDebug.Log("MultiMedia::start - end");
}

//---------------------------------------------------------------------------
//  MultiMedia::Stop() --
//---------------------------------------------------------------------------

void MultiMedia::Stop()
{
    mDebug.Log("MultiMedia::stop");

    m_inRead = false;

    waveInReset(m_pcmRead);

    while (m_readComplete < 2)
        {
        Sleep(50);
        }

    m_inWrite = false;
    waveOutReset(m_pcmWrite);

    waveInUnprepareHeader(m_pcmRead, &m_waveReadHeader[1], sizeof(WAVEHDR));
    waveInUnprepareHeader(m_pcmRead, &m_waveReadHeader[0], sizeof(WAVEHDR));

    while (m_writeComplete < 2)
        {
        Sleep(50);
        }

    waveOutUnprepareHeader(m_pcmWrite, &m_waveWriteHeader[1], sizeof(WAVEHDR));
    waveOutUnprepareHeader(m_pcmWrite, &m_waveWriteHeader[0], sizeof(WAVEHDR));

    mDebug.Log("MultiMedia::stop - end");
}

//---------------------------------------------------------------------------
//  MultiMedia::Finalize() --
//---------------------------------------------------------------------------

void MultiMedia::Finalize()
{
    if (!m_initialized)
        return;

    mDebug.Log("MultiMedia::close");

    waveInClose(m_pcmRead);
    waveOutClose(m_pcmWrite);

    if (m_waveReadHeader[0].lpData)
        VirtualFree(m_waveReadHeader[0].lpData, 0, MEM_RELEASE);

    m_initialized = false;
}

//---------------------------------------------------------------------------
//  MultiMedia::Process() --
//---------------------------------------------------------------------------

void MultiMedia::Process(size_t position)
{
    inherited::Process(position);

    Start();
    Timer.Enabled(true);

}

//---------------------------------------------------------------------------
//  MultiMedia::HandleTimer() --
//---------------------------------------------------------------------------

void MultiMedia::HandleTimer(OpenWire::NotifyEvent & event)
{
    if (m_totalRecorded < mPosition)
        return;

    Timer.Enabled(false);
    Stop();

    OnComplete.Execute(event);
}


} // namespace Windows

#ifdef __BORLANDC__
    #pragma option pop
#endif

} // namespace Innovative
