//  MultiObject_Win.h   --  'MultipleObjects'... a Multiple Object Wait class
//
//  This concrete class encapsulates a WaitForMultipleObjects. It contains
//  member functions to specify a case statement (DWORD) for every event to wait
//  upon.
//
//  Currently, the IsValid() function only assures that a handle was
//    loaded for every declared handle in the object.
//  This class does not test for redundant handles!
//

#ifndef MultiObject_WinH
#define MultiObject_WinH

#include <windows.h>

namespace Innovative
{

namespace Windows
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef __BORLANDC__
	#pragma option push
	#pragma warn -8058
#endif

//===========================================================================
//  CLASS  MultipleObjects  --  Wait on multiple events & objects
//===========================================================================

class MultipleObjects
{
public:
    //
    //  Member Functions
    MultipleObjects(DWORD NumHandles);
    MultipleObjects(DWORD NumHandles, int Case, DWORD WakeMask=QS_ALLINPUT);
    virtual ~MultipleObjects();

    bool    Load(HANDLE Handle, int Position, int Case);
    DWORD   WaitFor(DWORD Timeout=INFINITE, bool WaitAll=false);
    bool    Get();

    int     LastPosition();
    DWORD   LastError();
    DWORD   LastCase();
    bool    IsValid();

private:
    bool      isValid;             // True if all handles have been set
    int       Position;            // Position of last Wait operation
    DWORD     lastError;           // Last Error value upon WAIT_FAILED
    int       HandleCount;
    HANDLE *  Handles;
    int *     Cases;
    bool *    ValidHandles;
    DWORD     Mask;
    bool      UseMsg;
    int       MsgCase;

    // Methods
    void      Init(DWORD NumHandles);
};

#ifdef __BORLANDC__
	#pragma option pop
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // Windows

} // Innovative

#endif


