// Mutex_Win.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Mutex_WinH
#define Mutex_WinH

#ifdef _MSC_VER
//#include <afxwin.h>
#else
#include <windows.h>
#endif

namespace Innovative
{

namespace Windows
{




#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//========================================================================
//   CLASS Mutex
//========================================================================
//
//  The Mutex class is designed to ease the use of a Windows Mutex object.
//  Putting it into a class also makes releasing resources less of a
//  concern for the user.

class Mutex
{
public:
//
//  Member Functions
//
    Mutex(LPSECURITY_ATTRIBUTES lpsa = NULL, BOOL InitOwned = FALSE,
                     LPCTSTR Name = NULL)
        {  hMtx = CreateMutexA(lpsa, InitOwned, Name);  }

    Mutex & operator = (const Mutex & aMutex)
    {
        HANDLE temp = hMtx;      // Save original Handle
        Dup(aMutex);             // Clone its Handle
        if (temp!=NULL)
            ::CloseHandle(temp); // Close the old handle
        return *this;
    }

    ~Mutex()
    {
        if (hMtx!=NULL)
            ::CloseHandle(hMtx);
        hMtx = NULL;
    }

    HANDLE GetHandle() const
		{  return hMtx;  }
    bool     Release()
		{  return (::ReleaseMutex(hMtx)) ? true : false;  }
    DWORD    Acquire(DWORD Timeout = INFINITE)
		{  return (::WaitForSingleObject( hMtx, Timeout));  }

private:
    // Data
    HANDLE hMtx;

    //  Member Functions
    void Dup( const Mutex &originalMutex)
    {
        HANDLE thisProcess = GetCurrentProcess();
        if (!::DuplicateHandle(thisProcess,
            originalMutex.GetHandle(),  // orig Mutex
            thisProcess,&hMtx,          // new handle
            STANDARD_RIGHTS_REQUIRED,   // Ignored, since cloning
            false,                      // NOT Inheritable!
            DUPLICATE_SAME_ACCESS))     // clone access
            hMtx = NULL;
        //
        //  If call failed, make sure our handle is NULLed.
    }

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // Windows

} // Innovative


#endif
