// Pipe_Win.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Framework/Windows/Pipe_Win.h>
#include <Framework/Windows/PipeThread_Win.h>
#include <BufferHeader_Mb.h>

namespace   Innovative
{

//---------------------------------------------------------------------------
//  PipeBase::PipeBase() --
//---------------------------------------------------------------------------

 PipeBase::PipeBase(bool server)
    : FServer(server), FRemoteMachineName("."), Monitor(0)
{
}

//---------------------------------------------------------------------------
//  PipeBase::~PipeBase() --
//---------------------------------------------------------------------------

 PipeBase::~PipeBase()
{
    Close();
}

//---------------------------------------------------------------------------
//  PipeBase::Name() --
//---------------------------------------------------------------------------

void PipeBase::Name(const std::string & name)
{
    FPipeName = name;
}

//---------------------------------------------------------------------------
//  PipeBase::Name() --
//---------------------------------------------------------------------------

std::string PipeBase::Name() const
{
    return FPipeName;
}

//---------------------------------------------------------------------------
//  PipeBase::Opened() --
//---------------------------------------------------------------------------

bool PipeBase::Opened() const
{
    return Monitor && Monitor->Opened();
}

//---------------------------------------------------------------------------
//  PipeBase::Connected() --
//---------------------------------------------------------------------------

bool PipeBase::Connected() const
{
    return Monitor && Monitor->Connected();
}

//---------------------------------------------------------------------------
//  PipeBase::Open() --
//---------------------------------------------------------------------------

bool PipeBase::Open()
{
    if (Opened())
        Close();

    Monitor = new PipeSupport(this);

    return Monitor->Opened();
}

//---------------------------------------------------------------------------
//  PipeBase::Close() --
//---------------------------------------------------------------------------

bool PipeBase::Close()
{
    delete Monitor;
    Monitor = 0;

    return true;
}

//---------------------------------------------------------------------------
//  PipeBase::Write() --
//---------------------------------------------------------------------------

int PipeBase::Write(const Buffer & buffer)
{
    //  Make sure packet size is correct...
    PacketBufferHeader headerDG(buffer);
    headerDG.PacketSize(buffer.FullSizeInInts());

    if (!Monitor)
        return 0;

    return Monitor->Write(buffer);
}

//---------------------------------------------------------------------------
//  PipeBase::Write() --
//---------------------------------------------------------------------------

int PipeBase::Write(short PeriphId, const Buffer & buffer)
{
    //
    //  Load Peripheral ID
    PacketBufferHeader headerDG(buffer);
    headerDG.PeripheralId(PeriphId);

    return Write(buffer);
}

//---------------------------------------------------------------------------
//  PipeBase::Write() --
//---------------------------------------------------------------------------

int PipeBase::Write(const std::string & msg)
{
    if (!Monitor)
        return 0;

    return Monitor->Write(msg);
}

//===========================================================================
//  CLASS  ServerPipe   ---  Named pipe component
//===========================================================================

//---------------------------------------------------------------------------
//  PipeBase::ServerPipe() --  Ctor
//---------------------------------------------------------------------------

ServerPipe::ServerPipe()
    : PipeBase(true)
{
}

//===========================================================================
//  CLASS  ClientPipe   ---  Named pipe component
//===========================================================================

//---------------------------------------------------------------------------
//  ClientPipe::ClientPipe() --  Ctor
//---------------------------------------------------------------------------

ClientPipe::ClientPipe()
    : PipeBase(false)
{
}

//---------------------------------------------------------------------------
//  ClientPipe::RemoteMachineName() --
//---------------------------------------------------------------------------

void ClientPipe::RemoteMachineName(const std::string & name)
{
    FRemoteMachineName = name;
}

//---------------------------------------------------------------------------
//  ClientPipe::RemoteMachineName() --
//---------------------------------------------------------------------------

std::string ClientPipe::RemoteMachineName() const
{
    return FRemoteMachineName;
}


}   // Innovative



