// Pipe_Win.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Pipe_WinH
#define Pipe_WinH

#include <Events_Mb.h>
#include <ProcessEvents_Mb.h>
#include <Buffer_Mb.h>
#include <vector>
#include <string>

namespace   Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS  PipeRcvEvent   ---  Named pipe event
//===========================================================================

class PipeRcvEvent : public OpenWire::NotifyEvent
{
public:
    const Buffer Data;

    PipeRcvEvent(const Buffer & data)
        : Data(data)
        {  }
};

//===========================================================================
//  CLASS  PipeBase   ---  Named pipe component
//===========================================================================

class PipeBase
{
    friend class PipeImpl;

public:
    // Ctor
    PipeBase(bool server);
    ~PipeBase();

    // Support Methods
    bool            Open();
    bool            Close();

    bool            Opened() const;
    bool            Connected() const;

    int             Write(short PeriphId, const Buffer & buffer);
    int             Write(const Buffer & buffer);
    int             Write(const std::string & msg);

    // Properties
    void            Name(const std::string & name);
    std::string     Name() const;

    // Properties
    OpenWire::EventHandler<PipeRcvEvent>            OnRead;
    OpenWire::EventHandler<ProcessStatusEvent>      OnMessage;
    OpenWire::EventHandler<ProcessCompletionEvent>  OnConnect;
    OpenWire::EventHandler<ProcessStatusEvent>      OnError;

protected:
    // Fields
    bool                FServer;
    std::string         FPipeName;
    std::string         FRemoteMachineName;

    // Data
    class PipeSupport * Monitor;
};

//===========================================================================
//  CLASS  ServerPipe   ---  Named pipe component
//===========================================================================

class ServerPipe : public PipeBase
{
public:
    // Methods
    ServerPipe();
};

//===========================================================================
//  CLASS  ClientPipe   ---  Named pipe component
//===========================================================================

class ClientPipe : public PipeBase
{
public:
    // Methods
    ClientPipe();

    // Properties
    void            RemoteMachineName(const std::string & name);
    std::string     RemoteMachineName() const;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative

//---------------------------------------------------------------------------
#endif
