// Registry_Win.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
#ifndef Registry_WinH
#define Registry_WinH

#ifdef _MSC_VER
//#include <afxwin.h>
#else
#include <windows.h>
#endif
#include <string>
#include <deque>
#include <StringList_Mb.h>
#include <DataTypes_Mb.h>
#include <ctime>
#include <Trace_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS RegistryKey  --  Registry handle container
//=============================================================================

class RegistryKey
{
public:
	// Ctor
	RegistryKey(const HKEY & root, const std::string & key, bool create=false);
	RegistryKey(const RegistryKey & other);
	virtual ~RegistryKey();

	// Methods
	HKEY  		Handle() const;
	std::string	Key() const;

private:
	HKEY    	FHandle;
	std::string	FKey;
};

//=============================================================================
//  CLASS Registry  --  Registry access base class
//=============================================================================

class Registry
{
public:
	typedef std::vector<char>   CharArray;
	typedef std::vector<int>    IntArray;
	typedef std::vector<float>  FloatArray;

	// Tree Navigation
	bool		Home();
	bool    	Root();
	bool        Descend(const std::string & key, bool create=false);
	bool        Ascend();
	bool        IsKey(const std::string & key);
	bool        IsTag(const std::string & tag);
	std::string Path() const;
	StringList	Keys();
	StringList	Tags();
    DWORD 	   	SizeInBytes(const std::string & tag) const;
	// Tree manipulation
    void  		DeleteKeyTree(const std::string & key);
	bool        DeleteKey(const std::string & key);
	bool        DeleteTag(const std::string & tag);
	bool		Flush();

	// Tag access
	bool        Write(const std::string & tag, const std::string & value);
	bool 		Write(const std::string & tag, const char * value);
	bool        Write(const std::string & tag, const int & value);
	bool        Write(const std::string & tag, const bool & value);
	bool        Write(const std::string & tag, const double & Value);
	bool        Write(const std::string & tag, const ii64 & Value);
	bool        Write(const std::string & tag, const time_t & Value);
	bool        Write(const std::string & tag, const CharArray & data);
	bool        Write(const std::string & tag, const IntArray & data);
	bool        Write(const std::string & tag, const FloatArray & data);
    bool        WriteExpanded(const std::string & tag, const std::string & value);
    bool        Write(const std::string & tag, const vector<std::string> & value);

	std::string Read(const std::string & tag, const std::string & defValue="");
	std::string Read(const std::string & tag, const char * defValue="");
    int 		Read(const std::string & tag, const int defValue=0);
	bool        Read(const std::string & tag, const bool defValue=false);
	double      Read(const std::string & tag, const double defValue=0.0f);
	ii64        Read(const std::string & tag, const ii64 defValue=0);
	time_t      Read(const std::string & tag, const time_t defValue=0);
	bool 		Read(const std::string & tag, CharArray & result);
	bool		Read(const std::string & tag, IntArray & result);
	bool		Read(const std::string & tag, FloatArray & result);
    bool        Read(const std::string & tag, vector<string> &result);

    std::string DefaultOrganization() const;

protected:
	// Data
	typedef std::deque<RegistryKey> KeyDeque;
	KeyDeque        Fifo;

	// Fields
	std::string     FApp;
	std::string     FOrganization;

	// Ctor
	Registry(const HKEY & root, const std::string & key,
		const std::string & app, const std::string & org);
	virtual ~Registry();

	// Methods
	StringList Decompose(const std::string & key);
	HKEY CurrentKey() const;

};

//=============================================================================
//  CLASS SystemRegistry  --
//=============================================================================

class SystemRegistry : public Registry
{
public:
	SystemRegistry(const std::string & app="", const std::string & org="");
};

//=============================================================================
//  CLASS UserRegistry  --
//=============================================================================

class UserRegistry : public Registry
{
public:
	UserRegistry(const std::string & app="", const std::string & org="");
};

//=============================================================================
//  CLASS MachineRegistry  --
//=============================================================================

class MachineRegistry : public Registry
{
public:
	MachineRegistry(const std::string & app="", const std::string & org="");
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}   // namespace
#endif
