// Shutdown_Mb.cpp
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
//   OS shutdown
//
//---------------------------------------------------------------------------

#include "Shutdown_Win.h"
#include "OsDetector_Win.h"
#include "windows.h"

namespace Innovative
{

//---------------------------------------------------------------------------
//  Shutdown::Shutdown() --
//---------------------------------------------------------------------------

Shutdown::Shutdown()
    : FForceTerminate(true)
{
}

//---------------------------------------------------------------------------
//  Shutdown::Execute() --
//---------------------------------------------------------------------------

bool Shutdown::Execute()
{
    OsDetector Os;

    if (Os.IsWin9x())
        return Shutdown9x();
    else
        return ShutdownNT();
}

//---------------------------------------------------------------------------
//  Shutdown::ShutdownNT() --
//---------------------------------------------------------------------------

bool Shutdown::ShutdownNT()
{
    HANDLE hToken;
    TOKEN_PRIVILEGES tkp;

    // Get a token for this process.
    if (!OpenProcessToken(GetCurrentProcess(),
        TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken))
        return false;

    // Get the LUID for the shutdown privilege.
    LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid);
    // one privilege to set
    tkp.PrivilegeCount = 1;
    tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;

    // Get the shutdown privilege for this process.
    AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0);

    // Cannot test the return value of AdjustTokenPrivileges.
    if (GetLastError() != ERROR_SUCCESS)
        return false;

    // Shut down the system and force all applications to close.
    return Shutdown9x();
}

//---------------------------------------------------------------------------
//  Shutdown::Shutdown9x() --
//---------------------------------------------------------------------------

bool Shutdown::Shutdown9x()
{
    UINT Flags;

    switch (FMode)
        {
        default:
        case mLogOff:       Flags = EWX_LOGOFF;     break;
        case mPowerOff:     Flags = EWX_POWEROFF;   break;
        case mReboot:       Flags = EWX_REBOOT;     break;
        case mShutdown:     Flags = EWX_SHUTDOWN;   break;
        }

    if (FForceTerminate)
        Flags |= EWX_FORCE;

    return ExitWindowsEx(Flags, 0) ? true : false;
}

} // namespace Innovative
