// Thunker_Win.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Framework/Windows/Thunker_Win.h>
#include <Events_Mb.h>

namespace   Innovative
{

//=============================================================================
//  CLASS Thunker  --  "Thunk" call into main thread, unlocked with caller
//=============================================================================

// Offset to Id of messages received by window below
static const unsigned int SYNC_MESSAGE = WM_USER + 0x1303;

//---------------------------------------------------------------------------
//  Thunker::WndProc() -- Windows procedure for Thunker
//---------------------------------------------------------------------------

LRESULT CALLBACK Thunker::WndProc(HWND hwnd,     // handle to window
                                  UINT uMsg,     // message identifier
                                  WPARAM wParam, // first message parameter
                                  LPARAM lParam) // second message parameter
{
    if (uMsg == SYNC_MESSAGE)
        {
        ((Thunker*)wParam)->Dispatch();
        }
    else
        return DefWindowProc(hwnd, uMsg, wParam, lParam);

    return 0;
}

//---------------------------------------------------------------------------
//  housekeeping for class Thunker
//---------------------------------------------------------------------------

HWND    Thunker::mHandle = NULL;
int     Thunker::mRefCount = 0;
const char * WindowName = "ThunkWindow";

//---------------------------------------------------------------------------
//  constructor for class Thunker
//---------------------------------------------------------------------------

Thunker::Thunker()
{
    if (!mRefCount)
        {
        WNDCLASS    wc;

        // Register the main window class.
        wc.style = 0;
        wc.lpfnWndProc = WndProc;
        wc.cbClsExtra = 0;
        wc.cbWndExtra = 0;
        wc.hInstance = GetModuleHandle(NULL);
        wc.hIcon = 0;
        wc.hCursor = 0;
        wc.hbrBackground = 0;
        wc.lpszMenuName = NULL;
        wc.lpszClassName = WindowName;

        RegisterClass(&wc);
        mHandle = CreateWindowEx(WS_EX_TOOLWINDOW,
                                 wc.lpszClassName,
                                 "",
                                 WS_POPUP,
                                 0,
                                 0,
                                 0,
                                 0,
                                 0,
                                 0,
                                 wc.hInstance,
                                 NULL);
        }

    mRefCount++;
}

//---------------------------------------------------------------------------
//  destructor for class Thunker
//---------------------------------------------------------------------------

Thunker::~Thunker()
{
    if (!mRefCount)
        return;

    if (!(--mRefCount))
        {
        DestroyWindow(mHandle);
        UnregisterClass(WindowName, GetModuleHandle(NULL));
        }
}

//---------------------------------------------------------------------------
//  Thunker::Notify() --
//---------------------------------------------------------------------------

bool Thunker::Notify()
{
    if (!OnNotified.Assigned())
        return false;

	if(InnovativeKernel::CurrentThreadId() == OpenWire::GetMainThreadId())
		{
		// We are in the main thread, so execute and exit.
        Dispatch();
		return false;
		}

    ::PostMessage(mHandle, SYNC_MESSAGE, (WPARAM) this, 0);
    return true;
}

//---------------------------------------------------------------------------
//  Thunker::Dispatch() --
//---------------------------------------------------------------------------

void Thunker::Dispatch()
{
    OpenWire::Event e;
    OnNotified.Execute(e);
}

//=============================================================================
//  Thunker Factory Methods
//=============================================================================

//---------------------------------------------------------------------------
//  NewThunker() --
//---------------------------------------------------------------------------

ThunkerIntf * NewThunker()
{
    return new Thunker;
}

//---------------------------------------------------------------------------
//  DeleteThunker() --
//---------------------------------------------------------------------------

void DeleteThunker(ThunkerIntf * thunker)
{
    delete thunker;
}


}  // namespace
