// VersionInfo_Win.cpp
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Framework/Windows/VersionInfo_Win.h>
#include <Exception_Mb.h>
#include <cstdlib>
#include <sstream>
#include <iomanip>

using namespace Innovative;
using namespace std;

namespace Innovative
{

#ifdef __BORLANDC__
    #pragma warning(push)
    #pragma warning(disable:4996)
#endif

#ifdef __GNUC__
    #pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif


//===========================================================================
// CLASS VersionData()
//===========================================================================

//---------------------------------------------------------------------------
// VersionData() - Default Ctor for application EXEs
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4996)
#endif

VersionData::VersionData(std::string text)
{
    int radix = 10;
    char *endptr;
    const char *delimiters = " .,()\t\r\n";
    char temp[80];
    memset(temp, 0, 80);
    strcpy(temp, text.c_str());

    FMajor = FMinor = FRelease = FBuild = 0;

    // Decompose the delimited input string
    char *p = strtok(temp, delimiters);
    if (p)
        {
        FMajor = strtol(p, &endptr, radix);
        p = strtok(NULL, delimiters);
        }
    if (p)
        {
        FMinor = strtol(p, &endptr, radix);
        p = strtok(NULL, delimiters);
        }
    if (p)
        {
        FRelease = strtol(p, &endptr, radix);
        p = strtok(NULL, delimiters);
        }
    if (p)
        {
        FBuild = strtol(p, &endptr, radix);
        }
    else
        {
        FBuild = FRelease;
        FRelease = 0;
        }

    BuildStrings();
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// VersionData() - Default Ctor for application EXEs
//---------------------------------------------------------------------------

VersionData::VersionData(unsigned int ls, unsigned int ms)
{
    FMajor = ms >> 16;
    FMinor = ms & 0xffff;
    FRelease = ls >> 16;
    FBuild = ls & 0xffff;

    BuildStrings();
}

//---------------------------------------------------------------------------
// VersionData::BuildStrings() - Builds output revision and strings
//---------------------------------------------------------------------------

void VersionData::BuildStrings()
{
    FRevision = (FMajor << 24) | (FMinor << 16) | (FRelease << 8) | FBuild;

    std::stringstream msg;
    msg << "Version " << FMajor << "." << FMinor
        << ",  Release " << FRelease << ",  Build " << FBuild;
    FText = msg.str();
}

//===========================================================================
// CLASS VersionInfo()
//===========================================================================

//---------------------------------------------------------------------------
// VersionInfo() - Default Ctor for application EXEs
//---------------------------------------------------------------------------

VersionInfo::VersionInfo()
    : FFileVersionInfoData(0), FHasVersionInfo(false),
      FFixed(0), FImage(0), FProduct(0), FFile(0)
{
    // Show the updated form
    char filename[_MAX_DIR];
    GetModuleFileNameA(NULL, filename, _MAX_DIR);
    FFileSpec = filename;
    Read(filename);
}

//---------------------------------------------------------------------------
// VersionInfo(HMODULE) - Constructor for DLLs
//---------------------------------------------------------------------------

VersionInfo::VersionInfo(HMODULE module)
    : FFileVersionInfoData(0), FFixed(0), FImage(0), FProduct(0), FFile(0)
{
    // Show the updated form
    char filename[_MAX_DIR];
    GetModuleFileNameA(module, filename, _MAX_DIR);
    FFileSpec = filename;
    Read(filename);
}

//---------------------------------------------------------------------------
// VersionInfo() - Default destructor
//---------------------------------------------------------------------------

VersionInfo::~VersionInfo()
{
    delete FFile;       FFile = 0;
    delete FFixed;      FFixed = 0;
    delete FProduct;    FProduct = 0;
    delete FImage;      FImage = 0;

    delete [] FFileVersionInfoData;
    FFileVersionInfoData = 0;
}

//---------------------------------------------------------------------------
// Read() - Read version info from fully-qualified filespec
//---------------------------------------------------------------------------

void VersionInfo::Read(const char *filename)
{
    // Init the readable variables to zero
    // File version info
    FFileVersion = 0;
    // Date info
    FFileDate = 0;
    // File type info
    FFileType = 0;

    FFileOS = 0;
    FFileFlagsMask = 0;
    FFileFlags = 0;

    DWORD temp;
    UINT dwBytes;
    LPVOID lpBuffer;

    // Retain full file pathspec
    FFileSpec = filename;

    // Read version info from target file
    DWORD VerSize = GetFileVersionInfoSize(const_cast<char*>(FFileSpec.c_str()), &temp);
    if (!VerSize)
        {
        FFixed = new VersionData(0, 0);
        FProduct = new VersionData(0, 0);
        FImage = new VersionData(0, 0);
        FFile = new VersionData(0, 0);
        return;
//        throw Exception("Couldn't get file version size");
        }

    FFileVersionInfoData = new char[VerSize];
    if (!GetFileVersionInfo(const_cast<char*>(FFileSpec.c_str()), temp, VerSize, FFileVersionInfoData))
        throw MalibuException("Couldn't get file version info");

    VerQueryValue(FFileVersionInfoData,        // address of buffer for version resource
              const_cast<char*>(TEXT("\\")),
              &lpBuffer,        // address of buffer for version pointer
              &dwBytes);        // address of version-value length buffer
    VS_FIXEDFILEINFO *VerInfo = (VS_FIXEDFILEINFO*)lpBuffer;

    // File version info
    FFileVersion = (static_cast<ii64>(VerInfo->dwFileVersionMS)<<32) | VerInfo->dwFileVersionLS;

    // Date info
    FFileDate = (static_cast<ii64>(VerInfo->dwFileDateMS)<<32) | VerInfo->dwFileDateLS;

    // File type info
    FFileType = (static_cast<ii64>(VerInfo->dwFileType)<<32) || VerInfo->dwFileSubtype;
    FFileFlagsMask = VerInfo->dwFileFlagsMask;
    FFileFlags = VerInfo->dwFileFlags;

    FFixed = new VersionData(VerInfo->dwProductVersionLS, VerInfo->dwProductVersionMS);
    FFile = new VersionData(VerInfo->dwFileVersionLS, VerInfo->dwFileVersionMS);

    if(ReadStringInfo())
        {
        FProduct = new VersionData(FProductVersion);
        FImage = new VersionData(FImageVersion);
        }
    else
        {
        FProduct = new VersionData(0, 0);
        FImage = new VersionData(0, 0);
        }
}

//---------------------------------------------------------------------------
// VersionInfo::ReadStringInfo() - Read string version info from filename
//---------------------------------------------------------------------------

bool VersionInfo::ReadStringInfo()
{
    FTranslations.Clear();

    if (!InitializeTranslationTable())
        return false;

    FLanguageCharsetIndex = FTranslations.IndexOf("040904E4");
    if (FLanguageCharsetIndex == 0xffffffff)
        FLanguageCharsetIndex = 0;

    FCompanyName = QueryStringValue("CompanyName");
    FImageDescription = QueryStringValue("FileDescription");
    FImageVersion = QueryStringValue("FileVersion");
    FInternalName = QueryStringValue("InternalName");
    FLegalCopyright = QueryStringValue("LegalCopyright");
    FOriginalFilename = QueryStringValue("OriginalFilename");
    FProductName = QueryStringValue("ProductName");
    FProductVersion = QueryStringValue("ProductVersion");

    return true;
}

//---------------------------------------------------------------------------
// VersionInfo::BuildStringFileInfo() - Build tag for standard key
//---------------------------------------------------------------------------

std::string VersionInfo::BuildStringFileInfo(const std::string& key)
{
    if (FTranslations.Count() <= 0)
        return "";

    if (FLanguageCharsetIndex >= FTranslations.Count())
        return "";

    return "\\StringFileInfo\\" +
        FTranslations.StringAt(FLanguageCharsetIndex) + "\\" + key;
}

#ifdef __GNUC__
//    #pragma GCC diagnostic push
//    #pragma GCC diagnostic ignored "-Wunused-but-set-variable"
#endif

//---------------------------------------------------------------------------
// VersionInfo::InitializeTranslationTable()() -
//---------------------------------------------------------------------------

bool VersionInfo::InitializeTranslationTable()
{
    #pragma pack(push, 1)
    union TTranslationTableEntry
    {
    struct struct_Codes
        {
        WORD LanguageCode;
        WORD CharsetCode;
        } Codes;
    DWORD RawData;
    };
    #pragma pack(pop)

    unsigned int BufferLength;

    LPVOID p;
    QueryValue("\\VarFileInfo\\Translation", p, BufferLength);

    TTranslationTableEntry * ATranslationTableEntry
        = reinterpret_cast<TTranslationTableEntry*>(p);

    // We assume that the translation table is stored correctly;
    // the operating system provides no other means to retrieve
    // the count of items, so dividing table size by
    // sizeof(table entry) is the only means to achieve this.
    if (BufferLength % sizeof(TTranslationTableEntry))
        return false;

    // ...and if we made it down here, we definitely
    // do have VERSIONINFO in FileSpec.
    FHasVersionInfo = true;

    while (BufferLength != 0)
        {
        stringstream msg;
        msg << hex << setfill('0') << uppercase
            << setw(4) << ATranslationTableEntry->Codes.LanguageCode
            << setw(4) << ATranslationTableEntry->Codes.CharsetCode;
        FTranslations.Add(msg.str());

        ATranslationTableEntry++;
        BufferLength = BufferLength - sizeof(TTranslationTableEntry);
        }

    if ((FTranslations.Count() == 0) && FHasVersionInfo)
        {
        // Interestingly, there was *NO* translation
        // data stored inside the buffer - for whatever
        // reason there might be.
        // The operating system mandates a language-charset
        // identifier, though, when it comes to querying
        // for string data.
        // Resolution: create a fake entry which is
        //             language-neutral and which uses
        //             the Unicode charset.

        TTranslationTableEntry TranslationCode;
        TranslationCode.Codes.LanguageCode = MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL);
        TranslationCode.Codes.CharsetCode = 0x04b0; // Unicode

        stringstream msg;
        msg << hex << setfill('0') << uppercase
            << setw(4) << ATranslationTableEntry->Codes.LanguageCode
            << setw(4) << ATranslationTableEntry->Codes.CharsetCode;
        FTranslations.Add(msg.str());
        }

    // By default pick the first language-charset entry
    // for display / analysis.
    FLanguageCharsetIndex = 0;

    return true;
}

#ifdef __GNUC__
//    #pragma GCC diagnostic pop
#endif

//---------------------------------------------------------------------------
// VersionInfo::QueryValue() -
//---------------------------------------------------------------------------

bool VersionInfo::QueryValue(const std::string& SubBlock, LPVOID& Buffer,
  unsigned int& BufferLength) const
{
    if (FFileVersionInfoData == NULL)
        return false;

    return VerQueryValue(FFileVersionInfoData,
        const_cast<char*>(SubBlock.c_str()), &Buffer, &BufferLength)
        ? true : false;
}

//---------------------------------------------------------------------------
// VersionInfo::QueryStringValue() -
//---------------------------------------------------------------------------

std::string VersionInfo::QueryStringValue(const std::string& SubBlock)
{
    std::string ResultString;

    LPVOID StringBuffer;
    unsigned int BufferLength;

    if (FHasVersionInfo)
        {
        std::string s = BuildStringFileInfo(SubBlock);
        if (!s.size())
            return "";

        if (QueryValue(s, StringBuffer, BufferLength))
            {
            if (BufferLength == 0)
                return "";

            ResultString = reinterpret_cast<char*>(StringBuffer);
            }
        else
            {
            ResultString = "";
            }
        }
    else
        {
        ResultString = "";
        }

    return ResultString;
}

#ifdef __BORLANDC__
    #pragma warning(pop)
#endif

} // namespace Innovative

