// VersionInfo_Win.h
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef VersionInfo_WinH
#define VersionInfo_WinH

#include <string>
#include <StringVector_Mb.h>
#include <DataTypes_Mb.h>
#include <windows.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS VersionData()
//===========================================================================

class VersionData
{
public:
    // Ctors
    VersionData(unsigned int ls, unsigned int ms);
    VersionData(std::string text);

    // Properties
    unsigned int Major() const
        {  return FMajor;  }
    unsigned int Minor() const
        {  return FMinor;  }
    unsigned int Release() const
        {  return FRelease;  }
    unsigned int Build() const
        {  return FBuild;  }
    unsigned int Revision() const
        {  return FRevision;  }
    std::string Text() const
        {  return FText;  }

private:
    // Data
    unsigned int                 FMajor;
    unsigned int                 FMinor;
    unsigned int                 FRelease;
    unsigned int                 FBuild;
    unsigned int                 FRevision;
    std::string                  FText;

    // Methods
    void                BuildStrings();
};

//===========================================================================
// CLASS VersionInfo()
//===========================================================================

class VersionInfo
{
    typedef void* LPVOID;
    
public:
    // Ctor fills release info variables
    VersionInfo(const char* filename)
        : FFileVersionInfoData(0), FFixed(0), FImage(0), FProduct(0), FFile(0)
        {  Read(filename);  }
    VersionInfo(const std::string & filename)
        : FFileVersionInfoData(0), FFixed(0), FImage(0), FProduct(0), FFile(0)
        {  Read(filename.c_str());  }
    // No filename: Get current process EXE pathspec
    VersionInfo();
    // Handle to module (DLL):
    VersionInfo(HMODULE module);
    virtual ~VersionInfo();

    // Properties
    VersionData & Fixed() const
        {  return *FFixed;  }
    VersionData & Image() const
        {  return *FImage;  }
    VersionData & Product() const
        {  return *FProduct;  }
    VersionData & File() const
        {  return *FFile;  }

    ii64 FileVersion() const
        {  return FFileVersion;  }
    ii64 FileDate() const
        {  return FFileDate;  }
    ii64 FileType() const
        {  return FFileType;  }
    int FileOS() const
        {  return FFileOS;  }
    int FileFlags()
        {  return FFileFlags;  }
    int FileFlagsMask() const
        {  return FFileFlagsMask;  }
    std::string FileSpec() const
        {  return FFileSpec;  }
    std::string CompanyName() const
        {  return FCompanyName;  }
    std::string ImageDescription() const
        {  return FImageDescription;  }
    std::string ImageVersion() const
        {  return FImageVersion;  }
    std::string InternalName() const
        {  return FInternalName;  }
    std::string LegalCopyright() const
        {  return FLegalCopyright;  }
    std::string OriginalFilename() const
        {  return FOriginalFilename;  }
    std::string ProductName() const
        {  return FProductName;  }
    std::string ProductVersion() const
        {  return FProductVersion;  }

    // Properties
    bool HasVersionInfo() const
        {  return FHasVersionInfo;  }

private:
    // Data
    Innovative::StringVector  FTranslations;
    char *              FFileVersionInfoData;
    size_t              FLanguageCharsetIndex;
    bool                FHasVersionInfo;
    VersionData *       FFixed;
    VersionData *       FImage;
    VersionData *       FProduct;
    VersionData *       FFile;
    ii64                FFileVersion;
    ii64                FFileDate;
    ii64                FFileType;
    int                 FFileOS;
    int                 FFileFlags;
    int                 FFileFlagsMask;
    std::string         FFileSpec;
    std::string         FCompanyName;
    std::string         FImageDescription;
    std::string         FImageVersion;
    std::string         FInternalName;
    std::string         FLegalCopyright;
    std::string         FOriginalFilename;
    std::string         FProductName;
    std::string         FProductVersion;

    // Methods
    void                Read(const char *filename);
    bool                ReadStringInfo();
    std::string         BuildStringFileInfo(const std::string& StringName);
    bool                InitializeTranslationTable();
    bool                QueryValue(const std::string& SubBlock, LPVOID& Buffer,
                                   unsigned int& BufferLength) const;
    std::string         QueryStringValue(const std::string& SubBlock);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
