// GaussGen_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <GaussGen_Mb.h>
#include <IppFloatDG_Mb.h>
#include <ipps.h>   // ipps.h is not redistributable!

namespace   Innovative
{

//===========================================================================
//  CLASS GaussGen  --
//===========================================================================

//---------------------------------------------------------------------------
// GaussGen::GaussGen() --
//---------------------------------------------------------------------------

GaussGen::GaussGen()
    : FMean(0), FStdDev(9000), FSeed(31192)
{
	Seed(FSeed);
	update();
}

//---------------------------------------------------------------------------
// GaussGen::GaussGen() --
//---------------------------------------------------------------------------

GaussGen::GaussGen(const GaussGen & other)
	: inherited(other),
	  FMean(other.FMean), FStdDev(other.FStdDev), FSeed(other.FSeed)
{
	update();
}

//---------------------------------------------------------------------------
//  GaussGen::operator = --
//---------------------------------------------------------------------------

GaussGen & GaussGen::operator = ( const GaussGen & other )
{
	inherited::operator = ( other );

	FMean = other.FMean;
	FStdDev = other.FStdDev;
	FSeed = other.FSeed;

	update();

	return *this;
}

//---------------------------------------------------------------------------
//  GaussGen::Mean() --
//---------------------------------------------------------------------------

float GaussGen::Mean() const
{
	return FMean;
}

//---------------------------------------------------------------------------
//  GaussGen::Mean() --
//---------------------------------------------------------------------------

void  GaussGen::Mean(float value)
{
	FMean = value;
	update();
}

//---------------------------------------------------------------------------
//  GaussGen::StdDev() --
//---------------------------------------------------------------------------

float GaussGen::StdDev() const
{
	return FStdDev;
}

//---------------------------------------------------------------------------
//  GaussGen::StdDev() --
//---------------------------------------------------------------------------

void GaussGen::StdDev( float value )
{
	FStdDev = value;
	update();
}

//---------------------------------------------------------------------------
//  GaussGen::Seed() --
//---------------------------------------------------------------------------

unsigned int GaussGen::Seed() const
{
	return FSeed;
}

//---------------------------------------------------------------------------
//  GaussGen::Seed() --
//---------------------------------------------------------------------------

void GaussGen::Seed(unsigned int value)
{
	FSeed = value;
	update();
}

//---------------------------------------------------------------------------
//  GaussGen::Generate() --
//---------------------------------------------------------------------------

bool GaussGen::Generate(IppFloatDG & buffer)
{
	IppsRandGaussState_32f * p = reinterpret_cast<IppsRandGaussState_32f*>(&mState);

	IppStatus status = ippsRandGauss_32f(buffer.Ptr(), static_cast<int>(buffer.size()), p);

	return (status == ippStsNoErr);
}

//---------------------------------------------------------------------------
//  GaussGen::Generate() --
//---------------------------------------------------------------------------

bool GaussGen::Generate(IppComplexDG & buffer)
{
	return inherited::Generate(buffer);
}

//---------------------------------------------------------------------------
//  GaussGen::update() --  Update internal state array
//---------------------------------------------------------------------------

bool GaussGen::update()
{
	IppsRandGaussState_32f * p = reinterpret_cast<IppsRandGaussState_32f*>(&mState);

	IppStatus status = ippsRandGaussInit_32f(p, FMean, FStdDev, FSeed);

	return (status == ippStsNoErr);
}



} // namespace Innovative
