// GpsMesage_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef GpsMessage_MbH
#define GpsMessage_MbH

#include <vector>
#include <ctime>
#include <string>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS GpsMessage  --  GPS "Partially Parsed" message
//===========================================================================

class GpsMessage
{
public:
    GpsMessage(size_t total_args=0)
        :  Args(total_args)
        {}

    void  AddArgument(const std::string & newArg)
        {  Args.push_back(newArg);  }

    std::string   AsText(const std::string & delim = ",") const;
    unsigned int  TotalArgs() const
        {  return static_cast<unsigned int>(Args.size());  }
    //
    //  Arg Conversion Methods
    std::string   Arg(int Idx) const;
    unsigned int  ArgAsInt(int Idx) const;
    double        ArgAsFloat(int Idx) const;

    typedef std::vector<std::string>  ArgList;
    typedef ArgList::iterator iterator;
    typedef ArgList::const_iterator const_iterator;

    iterator  begin()
		{  return Args.begin();}
	iterator  end()
		{  return Args.end();  }
	const_iterator  cbegin() const
		{  return Args.begin();}
	const_iterator  cend() const
		{  return Args.end();  }

	char AddRawData(unsigned int idx)const;

	void RawData(const std::vector<unsigned char> & data)
		{	FRawData = data;   }
	std::vector<unsigned char> RawData() const
        {  return FRawData;  }
private:
	ArgList  		  Args;
	std::vector<unsigned char> FRawData;		// Used for TSIP.
};



//===========================================================================
// CLASS GpsMessageList  --  Collection of GPS "Partially Parsed" messages
//===========================================================================

class GpsMessageList
{
public:
    GpsMessageList()
        {}

    GpsMessage   MessageAt(int Idx) const
        {  return Messages[Idx];  }

	void  AddMessage(const GpsMessage & newMsg)
		{  Messages.push_back(newMsg);  }

    int FindMessage(const std::string & msg_name);

	std::string  AsText(const std::string & delim= "\r\n",
                        const std::string & msg_delim= "," ) const;
    unsigned int  TotalMessages() const
        {  return static_cast<unsigned int>(Messages.size());  }

    typedef std::vector<GpsMessage>  MessageList;
    typedef MessageList::iterator iterator;
    typedef MessageList::const_iterator const_iterator;

    iterator  begin()
        {  return Messages.begin();  }
    iterator  end()
        {  return Messages.end();  }
    const_iterator  cbegin() const
        {  return Messages.begin();  }
    const_iterator  cend() const
        {  return Messages.end();  }

private:

    MessageList  Messages;
};

//
//   Standard message 'cracker' classes...
//===========================================================================
// CLASS MessageCracker  --  Base Class for Message Crackers
//===========================================================================

class MessageCracker
{
public:
    MessageCracker(const GpsMessage & message);

    //  Common Data
    bool                IsValid;
    std::string         all;            // debug
    unsigned int        argc;
    const GpsMessage &  Message;

    bool            Valid()
        {  return IsValid;  }
    bool            IsCommand(const std::string & shouldBe);
    bool            HasArgs(unsigned int shouldHave);
    std::string     SubString(const std::string & Str, size_t start_idx,
                              size_t len=std::string::npos);

};

//===========================================================================
// CLASS GpZdaCracker  --  Conversions for $GPZDA message
//===========================================================================

class GpZdaCracker : public MessageCracker
{
public:
    GpZdaCracker(const GpsMessage & message);

    std::string   Utc;
    unsigned int  Day;
    unsigned int  Month;
    unsigned int  Year;
    unsigned int  Hour;
    unsigned int  Min;
    unsigned int  Sec;
    std::time_t   Time;
};

//===========================================================================
// CLASS GpGgaCracker  --  Conversions for $GPGGA message
//===========================================================================

class GpGgaCracker  : public MessageCracker
{
public:
    enum  FixStatus { fixNoFix, fixGps, fixDiffGps };

    GpGgaCracker(const GpsMessage & message);

    unsigned int  LatDegrees;
    double        LatMinutes;
    std::string   LatUnits;

    unsigned int  LongDegrees;
    double        LongMinutes;
    std::string   LongUnits;

    FixStatus     Fix;
    unsigned int  Satellites;
};

//===========================================================================
// CLASS GpRmcCracker  --  Conversions for $GPRMC message
//===========================================================================

class GpRmcCracker : public MessageCracker
{
public:
    GpRmcCracker(const GpsMessage & message);

    std::string   Utc;
    unsigned int  Day;
    unsigned int  Month;
    unsigned int  Year;
    unsigned int  Hour;
    unsigned int  Min;
    unsigned int  Sec;
    std::time_t   Time;

    unsigned int  LatDegrees;
    double        LatMinutes;
    std::string   LatUnits;

    unsigned int  LongDegrees;
    double        LongMinutes;
    std::string   LongUnits;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative
//---------------------------------------------------------------------------
#endif
