//---------------------------------------------------------------------------
// Gps_MiniT_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

// Reference: Thunderbolt E GPS Disciplined Clock User Guide

#include <Gps_MiniT_Mb.h>
#include <cmath>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace std;
namespace Innovative
{

//---------------------------------------------------------------------------
// GetDouble() -- Auxiliary extraction routines.
//---------------------------------------------------------------------------

double GetDouble(unsigned char* pucBuf)
{
    unsigned char ucBuf[8];
    ucBuf[0] = pucBuf[7];
    ucBuf[1] = pucBuf[6];
    ucBuf[2] = pucBuf[5];
    ucBuf[3] = pucBuf[4];
    ucBuf[4] = pucBuf[3];
    ucBuf[5] = pucBuf[2];
    ucBuf[6] = pucBuf[1];
    ucBuf[7] = pucBuf[0];
    double * p = reinterpret_cast<double*>(ucBuf);
    return (*p);
}

//---------------------------------------------------------------------------
// GetUShort()
//---------------------------------------------------------------------------

short GetUShort(unsigned char *pucBuf)
{
    unsigned char ucBuf[2];
    ucBuf[0] = pucBuf[1];
    ucBuf[1] = pucBuf[0];
    short * p = reinterpret_cast<short*>(ucBuf);
    return (*p);
}

//---------------------------------------------------------------------------
// GetFloat()
//---------------------------------------------------------------------------

float GetFloat(unsigned char *pucBuf)
{
    unsigned char ucBuf[4];
    ucBuf[0] = pucBuf[3];
    ucBuf[1] = pucBuf[2];
    ucBuf[2] = pucBuf[1];
    ucBuf[3] = pucBuf[0];
    unsigned char * p = reinterpret_cast<unsigned char*>(ucBuf);
    return (*p);
}

//===========================================================================
//  CLASS Gps_MiniT  -- Support for Trimble MiniT GPS Hardware
//===========================================================================

//---------------------------------------------------------------------------
// Gps_MiniT::Gps_MiniT()
//---------------------------------------------------------------------------

Gps_MiniT::Gps_MiniT()
    : IGps(new TsipGpsMessageParser()),
      FAntennaOk(false), FGpsLocked(false),
      FAltitude(0.0f), FTemperature(0.0f)
{
}

//---------------------------------------------------------------------------
//  Gps_MiniT::GpsBaud() --  Set baud-rate -- 9600 only.
//---------------------------------------------------------------------------

void Gps_MiniT::GpsBaud(BaudRates /*rate*/)
{
}

//---------------------------------------------------------------------------
//  Gps_MiniT::OnProcessMessages() --  Process Epoch Message List
//---------------------------------------------------------------------------

void Gps_MiniT::OnProcessMessages(GpsMessageList & Messages)
{
    try
        {
        size_t total = Messages.TotalMessages();
        for (size_t i = 0; i < total; ++i)
            {
            vector<unsigned char> msg = Messages.MessageAt(static_cast<int>(i)).RawData();
            unsigned char id_code = msg[2];
            if ((id_code == idPTime) && (msg.size() == szPTime))
                ProcessPrimaryTP(msg);

            else if((id_code == idSTime) && (msg.size() == szSTime))
                ProcessSuplementalTP(msg);
            }
        }
    catch (...)
        {
        }

}


//---------------------------------------------------------------------------
// Gps_MiniT::ProcessPrimaryTP()
//---------------------------------------------------------------------------

void Gps_MiniT::ProcessPrimaryTP(vector<unsigned char> & pkt)
{
    // Skip IDLE and id
    unsigned char * p = &pkt[2];

    struct tm  unix_time;
    unix_time.tm_hour = p[12];
    unix_time.tm_min  = p[11];
    unix_time.tm_sec  = p[10];

    unix_time.tm_mday = p[13];
    unix_time.tm_mon  = p[14] - 1;

    // This is where I took the "- 1900" from:
    // http://www.cplusplus.com/reference/clibrary/ctime/mktime/
    unix_time.tm_year = GetUShort(&p[15]) - 1900;

    //  Convert time structure to time_t
    FTime = mktime(&unix_time);
}

//---------------------------------------------------------------------------
// Gps_MiniT::ProcessSuplementalTP()
//---------------------------------------------------------------------------

void Gps_MiniT::ProcessSuplementalTP(vector<unsigned char> & pkt)//   , int base
{
    // Skip IDLE and id
    unsigned char * p = &pkt[2];

    //FSatellites   = p[1];
    switch (p[12])
        {
        case 0x00:
			if (p[1]==3)
				{
				FFix = fixGps;
				FSatellites = 4;
				}
			if (p[1]==4 || p[1]==7)
				{
				FFix = fixDiffGps;
				FSatellites = 5;
				}
            break;
        case 0x09:
			FSatellites = 1;
            FFix = fixNoFix;
            break;
        case 0x0a:
			FSatellites = 2;
            FFix = fixNoFix;
            break;
        case 0x0b:
			FSatellites = 3;
            FFix = fixNoFix;
            break;
        case 0x08:
			FSatellites = 0;
            FFix = fixNoFix;
			break;
        default:
			FSatellites = 0;
            FFix = fixNoFix;
            break;
        }

    FGpsLocked    = p[2] ? false : true;
    short alarms  = GetUShort(&p[10]);
    FAntennaOk = (alarms & 0x0006)? false : true;

    CurrentPosition(
        GpsCoordinate(GetDouble(&p[36]), GpsCoordinate::oLatitude),
        GpsCoordinate(GetDouble(&p[44]), GpsCoordinate::oLongitude) );

    FAltitude     = GetDouble(&p[52]);
    FTemperature  = GetFloat(&p[32]);
}


//===========================================================================
// CLASS TsipMessageParser
//===========================================================================

//---------------------------------------------------------------------------
// TsipGpsMsgParser::TsipMessageParser()
//---------------------------------------------------------------------------

TsipGpsMessageParser::TsipGpsMessageParser()
{
    CmdEndToken.push_back(0x10);
    CmdEndToken.push_back(0x03);
}

//---------------------------------------------------------------------------
// TsipGpsMessageParser::Parse()
//---------------------------------------------------------------------------

void TsipGpsMessageParser::Parse(string & UartData, GpsMessageList & msgs)
{
    GpsMessage Message;

    while(true)
        {
        vector<unsigned char> raw;
        raw.resize(UartData.size());
        for (size_t i = 0; i < UartData.size(); ++i)
            raw[i] = UartData[i];

        int pos = FindStartOfPacket(raw);
        if(pos == -1)
            break;

        raw.erase(raw.begin(), raw.begin() + pos);
        int eol = FindEndOfPacket(raw);
        if (eol == -1)
            break;

        // Copy complete message.
        raw.erase(raw.begin() + eol+ EndToken().size(), raw.end());
        Message.RawData(raw);
        msgs.AddMessage(Message);

        // shift contents. Include eol.
        UartData.erase(UartData.begin(), UartData.begin() + eol + EndToken().size());
        }
    return;
}

//---------------------------------------------------------------------------
//  TsipGpsMessageParser::FindStartOfPacket() -- Process a GPS Interrupt
//---------------------------------------------------------------------------

int TsipGpsMessageParser::FindStartOfPacket(vector<unsigned char> & raw)
{
    if (raw.size() == 0)
        return -1;

    int retValue = -1;
    unsigned int idx = 0;
    bool Done = false;

    while(raw.size() && (idx < raw.size() - 1) && (!Done))
        {
        unsigned char myByte    = raw[idx];
        unsigned char myNextByte = raw[idx+1];
        switch(myByte)
            {
            // Idle
            case 0x10:
                if(myNextByte == 0x10)
                    // Byte stuffing.
                    raw.erase(raw.begin() + idx);

                else if(myNextByte == 0x03)
                    {
                    // End of packet.
                    raw.erase(raw.begin(), raw.begin() + idx+2);
                    idx = 0;
                    }
                else
                    {
                    retValue = idx;
                    Done = true;
                    }
                break;
            default:
                ++idx;
                break;
            }
        }

    return retValue;
}

//---------------------------------------------------------------------------
// TsipGpsMessageParser::FindEndOfPacket()
//---------------------------------------------------------------------------

int TsipGpsMessageParser::FindEndOfPacket(vector<unsigned char> & raw)
{
    int retValue = -1;
    unsigned int idx = 0;
    bool Done = false;

    while(raw.size() && (idx < raw.size() - 1) && !Done)
        {
        unsigned char myByte     = raw[idx];
        unsigned char myNextByte = raw[idx+1];
        switch(myByte)
            {
            case 0x10:
                if(myNextByte == 0x10)
                    {
                    // Byte stuffing.
                    raw.erase(raw.begin() + idx);
                    ++idx;		   // otherwise will confuse with end of packet.
                    }
                else if(myNextByte == 0x03)
                    {
                    // Found.
                    retValue = idx;
                    Done = true;
                    }
                else
                    ++idx;
                break;
            default:
                ++idx;
                break;
            }
        }

    return retValue;
}

}  // end namespace Innovative
