//---------------------------------------------------------------------------
//  HardwareRegister_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>

namespace   Innovative
{

//==============================================================================
//  CLASS IBitManipulator  -- Interface applying Bit Manipulations
//==============================================================================
//---------------------------------------------------------------------------
//  Destructor for IBitManipulator
//---------------------------------------------------------------------------

IBitManipulator::~IBitManipulator()
{
}

//---------------------------------------------------------------------------
//  IBitManipulator::Bit() --
//---------------------------------------------------------------------------

bool  IBitManipulator::Bit(unsigned int bit) const
{
    return (Value() & (1 << bit)) != 0;
}

//---------------------------------------------------------------------------
//  IBitManipulator::Bit() --
//---------------------------------------------------------------------------

void  IBitManipulator::Bit(unsigned int bit, bool value)
{
    if (value)
        Value(Value() | (1 << bit));
    else
        Value(Value() & (~(1 << bit)));
}

//---------------------------------------------------------------------------
//  IBitManipulator::Field() --
//---------------------------------------------------------------------------

unsigned int IBitManipulator::Field(unsigned int bit, unsigned int size) const
{
    unsigned int Mask = (1 << size) - 1;
        return ((Value() >> bit) & Mask);
}

//---------------------------------------------------------------------------
//  IBitManipulator::Field() --
//---------------------------------------------------------------------------

void IBitManipulator::Field(unsigned int bit, unsigned int size, unsigned int value)
{
    unsigned int Mask = (1 << size) - 1;
    unsigned int OwnerMask = ~(Mask << bit);
    unsigned int tmp = Value() & OwnerMask;
    value &= Mask;
    value <<= bit;
    tmp |= value;
    Value(tmp);
}

//---------------------------------------------------------------------------
//  IBitManipulator::Mask() --
//---------------------------------------------------------------------------

void IBitManipulator::Mask(unsigned int mask)
{
    Value(Value() & mask);
}

//---------------------------------------------------------------------------
//  IBitManipulator::Value() --
//---------------------------------------------------------------------------

unsigned int IBitManipulator::Value() const
{
    return ReadV();
}

//---------------------------------------------------------------------------
//  IBitManipulator::Value() --
//---------------------------------------------------------------------------

void IBitManipulator::Value(unsigned int value)
{
    WriteV(value);
}

//==============================================================================
//  CLASS MemoryBitManipulator --  applying Bit Manips to an in memory value
//==============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

MemoryBitManipulator::MemoryBitManipulator(unsigned int result)
    : FValue(result)
{
}

//---------------------------------------------------------------------------
// MemoryBitManipulator::~MemoryBitManipulator() --
//---------------------------------------------------------------------------

MemoryBitManipulator::~MemoryBitManipulator()
{
}

//---------------------------------------------------------------------------
//  MemoryBitManipulator::ReadV() --
//---------------------------------------------------------------------------

unsigned int MemoryBitManipulator::ReadV() const
{
    return FValue;
}

//---------------------------------------------------------------------------
//  MemoryBitManipulator::WriteV() --
//---------------------------------------------------------------------------

void MemoryBitManipulator::WriteV(unsigned int value)
{
    FValue = value;
}

//==============================================================================
//  CLASS PtrBitManipulator --  applying Bit Manips to a pointer to memory
//==============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

PtrBitManipulator::PtrBitManipulator(unsigned int * ptr)
    : FValuePtr(ptr)
{
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

PtrBitManipulator::PtrBitManipulator(int * ptr)
    : FValuePtr(reinterpret_cast<unsigned int *>(ptr))
{
}

//---------------------------------------------------------------------------
// PtrBitManipulator::~PtrBitManipulator() --
//---------------------------------------------------------------------------

PtrBitManipulator::~PtrBitManipulator()
{
}

//---------------------------------------------------------------------------
//  PtrBitManipulator::ReadV() --
//---------------------------------------------------------------------------

unsigned int PtrBitManipulator::ReadV() const
{
    return *FValuePtr;
}

//---------------------------------------------------------------------------
//  PtrBitManipulator::WriteV() --
//---------------------------------------------------------------------------

void PtrBitManipulator::WriteV(unsigned int value)
{
    *FValuePtr = value;
}

//==============================================================================
//  CLASS IBitManipulator64  -- Interface applying Bit Manipulations
//==============================================================================
//---------------------------------------------------------------------------
//  Destructor for IBitManipulator64
//---------------------------------------------------------------------------

IBitManipulator64::~IBitManipulator64()
{
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Bit() --
//---------------------------------------------------------------------------

bool  IBitManipulator64::Bit(unsigned int bit) const
{
    ii64 One(1);
    return (Value() & (One << bit)) != 0;
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Bit() --
//---------------------------------------------------------------------------

void  IBitManipulator64::Bit(unsigned int bit, bool value)
{
    ii64 One(1);
    if (value)
        Value(Value() | (One << bit));
    else
        Value(Value() & (~(One << bit)));
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Field() --
//---------------------------------------------------------------------------

ii64 IBitManipulator64::Field(unsigned int bit, unsigned int size) const
{
    ii64 One(1);
    ii64 Mask = (One << size) - 1;
    return ((Value() >> bit) & Mask);
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Field() --
//---------------------------------------------------------------------------

void IBitManipulator64::Field(unsigned int bit, unsigned int size, ii64 value)
{
    ii64 One(1);
    ii64 Mask = (One << size) - 1;
    ii64 OwnerMask = ~(Mask << bit);
    ii64 tmp = Value() & OwnerMask;
    value &= Mask;
    value <<= bit;
    tmp |= value;
    Value(tmp);
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Mask() --
//---------------------------------------------------------------------------

void IBitManipulator64::Mask(ii64 mask)
{
    Value(Value() & mask);
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Value() --
//---------------------------------------------------------------------------

ii64 IBitManipulator64::Value() const
{
    return ReadV();
}

//---------------------------------------------------------------------------
//  IBitManipulator64::Value() --
//---------------------------------------------------------------------------

void IBitManipulator64::Value(ii64 value)
{
    WriteV(value);
}

//==============================================================================
//  CLASS MemoryBitManipulator64 --  applying Bit Manips to an in memory value
//==============================================================================
//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

MemoryBitManipulator64::MemoryBitManipulator64(ii64 result)
    : FValue(result)
{
}

//---------------------------------------------------------------------------
// MemoryBitManipulator64::~MemoryBitManipulator64() --
//---------------------------------------------------------------------------

MemoryBitManipulator64::~MemoryBitManipulator64()
{
}

//---------------------------------------------------------------------------
//  MemoryBitManipulator64::ReadV() --
//---------------------------------------------------------------------------

ii64 MemoryBitManipulator64::ReadV() const
{
    return FValue;
}

//---------------------------------------------------------------------------
//  MemoryBitManipulator64::WriteV() --
//---------------------------------------------------------------------------

void MemoryBitManipulator64::WriteV(ii64 value)
{
    FValue = value;
}

//==============================================================================
//  CLASS PtrBitManipulator64 --  applying Bit Manips to a pointer to memory
//==============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

PtrBitManipulator64::PtrBitManipulator64(ii64 * ptr)
    : FValuePtr(ptr)
{
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

PtrBitManipulator64::PtrBitManipulator64(s_ii64 * ptr)
    : FValuePtr(reinterpret_cast<ii64 *>(ptr))
{
}

//---------------------------------------------------------------------------
// PtrBitManipulator64::~PtrBitManipulator64() --
//---------------------------------------------------------------------------

PtrBitManipulator64::~PtrBitManipulator64()
{
}

//---------------------------------------------------------------------------
//  PtrBitManipulator64::ReadV() --
//---------------------------------------------------------------------------

ii64 PtrBitManipulator64::ReadV() const
{
    return *FValuePtr;
}

//---------------------------------------------------------------------------
//  PtrBitManipulator64::WriteV() --
//---------------------------------------------------------------------------

void PtrBitManipulator64::WriteV(ii64 value)
{
    *FValuePtr = value;
}

//=============================================================================
//  CLASS IAddressingSpace  -- Interface to "AddressingSpace"-like objects
//=============================================================================
//---------------------------------------------------------------------------
//  destructor of class IAddressingSpace
//---------------------------------------------------------------------------


IAddressingSpace::IAddressingSpace()
: Debug(new Trace)
{
}


IAddressingSpace::~IAddressingSpace()
{
    if (Debug) delete Debug;
}

//---------------------------------------------------------------------------
//  IAddressingSpace::Read() -- read from offset in the space
//---------------------------------------------------------------------------

ii32 IAddressingSpace::Read(ii32 offset)
{
        if (IsNull())
                return 0;

    ii32* addr = Base() + offset;
    ii32 data = *addr;

    // Temporarily Reduce Priority below normal
    TracePriority  TP(*Debug, Trace::pNormal+2);

    // If we aren't displaying due to Verb level, skip this...
    if (TP.WillTrace())
        {
        std::stringstream msg;
        msg << std::hex << "0x" << data << "=Reg[0x" << addr <<" : "<< offset << "]";
        Debug->Log(msg.str());
        }

    return data;
}

//---------------------------------------------------------------------------
//  IAddressingSpace::Write() -- write to offset in the space
//---------------------------------------------------------------------------

void IAddressingSpace::Write(ii32 offset, ii32 data)
{
        if (IsNull())
        return;

    ii32 * addr = Base() + offset;
        *addr = data;

    // Temporarily Reduce Priority below normal
    TracePriority  TP(*Debug, Trace::pNormal+1);

    // If we aren't displaying due to Verb level, skip this...
    if (TP.WillTrace())
    {
        std::stringstream msg;
        msg << std::hex
            << "Reg[0x" << addr <<" : " << offset << "]=" << "0x" << data;
        Debug->Log(msg.str());
    }
}

//---------------------------------------------------------------------------
//  IAddressingSpace::At(offset) -- read from offset in the space
//---------------------------------------------------------------------------

ii32 IAddressingSpace::At(ii32 offset)
{
    return Read(offset);
}

//---------------------------------------------------------------------------
//  IAddressingSpace::At(offset, value) -- write to offset in the space
//---------------------------------------------------------------------------

void IAddressingSpace::At(ii32 offset, ii32 data)
{
    Write(offset, data);
}

//=============================================================================
//  CLASS AddressingSpace  --
//=============================================================================
//---------------------------------------------------------------------------
//  Constructor of class AddressingSpace
//---------------------------------------------------------------------------

AddressingSpace::AddressingSpace(ii32 * baseaddr)
    : mBaseAddress(baseaddr)
{
}

//---------------------------------------------------------------------------
//  Destructor of class AddressingSpace
//---------------------------------------------------------------------------

AddressingSpace::~AddressingSpace()
{
        mBaseAddress = NULL;
}

//---------------------------------------------------------------------------
//  AddressingSpace::Link() -- Link space with a region of memory
//---------------------------------------------------------------------------

void AddressingSpace::Link(ii32 * address)
{
        mBaseAddress = address;
}

//---------------------------------------------------------------------------
//  AddressingSpace::Link() -- Link 2 spaces to point to same region
//---------------------------------------------------------------------------

void AddressingSpace::Link(const AddressingSpace & space)
{
        mBaseAddress = space.Base();
}

//---------------------------------------------------------------------------
//  AddressingSpace::Base() -- Return the base of the space
//---------------------------------------------------------------------------

ii32 *  AddressingSpace::Base() const
{
    return mBaseAddress;
}

//=============================================================================
//  CLASS WishboneBusSpace  -- WishboneBus "Section" of an addressing space
//=============================================================================
//---------------------------------------------------------------------------
//  Constructor of class WishboneBusSpace
//---------------------------------------------------------------------------

WishboneBusSpace::WishboneBusSpace(IAddressingSpace & space, int wishbone_offset)
    : Parent(space), WishboneBaseOffset(wishbone_offset)
{
}

//---------------------------------------------------------------------------
//  destructor of class WishboneBusSpace
//---------------------------------------------------------------------------

WishboneBusSpace::~WishboneBusSpace()
{
}

//---------------------------------------------------------------------------
//  WishboneBusSpace::Base() -- Return the base of the wishbone space
//---------------------------------------------------------------------------

ii32 *  WishboneBusSpace::Base() const
{
	return Parent.Base() + WishboneBaseOffset;
}

//=============================================================================
//  CLASS MemoryCell  --
//=============================================================================

MemoryCell::MemoryCell(IAddressingSpace & space, int offset)
    : mAddressingSpace(space), mOffset(offset)
{
}

//---------------------------------------------------------------------------
// MemoryCell::~MemoryCell() --
//---------------------------------------------------------------------------

MemoryCell::~MemoryCell()
{
}

//=============================================================================
//  CLASS ReadOnlyRegister  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ReadOnlyRegister::ReadOnlyRegister(IAddressingSpace & space, int offset)
    : MemoryCell(space, offset)
{
}

//---------------------------------------------------------------------------
// ReadOnlyRegister::ReadOnlyRegister() --
//---------------------------------------------------------------------------

ReadOnlyRegister::ReadOnlyRegister(const std::string & label,
    IAddressingSpace & space, int offset)
    : MemoryCell(space, offset)
{
    if (label.size())
        {
        std::stringstream txt;
        txt << "\t" << label;
        Label = txt.str();
        }
}

//---------------------------------------------------------------------------
//  ReadOnlyRegister::Value() --
//---------------------------------------------------------------------------

ii32 ReadOnlyRegister::Value() const
{
    ii32 value = 0;
    value = mAddressingSpace.At(mOffset);

    return value;
}

//---------------------------------------------------------------------------
//  ReadOnlyRegister::Bit() --
//---------------------------------------------------------------------------

bool ReadOnlyRegister::Bit(int bit) const
{
    return (Value() & (1 << bit)) != 0;
}

//---------------------------------------------------------------------------
//  ReadOnlyRegister::operator ii32() --
//---------------------------------------------------------------------------

ReadOnlyRegister::operator ii32() const
{
    return Value();
}

//---------------------------------------------------------------------------
//  ReadOnlyRegister::Reference() --
//---------------------------------------------------------------------------
//
//ii32 & ReadOnlyRegister::Reference() const
//{
//    return mAddressingSpace.mBaseAddress[mOffset];
//}
//
//=============================================================================
//  CLASS Register  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

Register::Register(IAddressingSpace & space, int offset)
    : ReadOnlyRegister(space, offset)
{
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

Register::Register(const std::string & label, IAddressingSpace & space, int offset)
    : ReadOnlyRegister(label, space, offset)
{
}

//---------------------------------------------------------------------------
//  Register::Value() --
//---------------------------------------------------------------------------

void Register::Value(ii32 value)
{
    mAddressingSpace.At(mOffset, value);
}

//---------------------------------------------------------------------------
//  Register::Value() --
//---------------------------------------------------------------------------

ii32 Register::Value() const
{
    return ReadOnlyRegister::Value();
}

//---------------------------------------------------------------------------
//  Register::Bit() --
//---------------------------------------------------------------------------

bool Register::Bit(int bit) const
{
    return ReadOnlyRegister::Bit(bit);
}

//---------------------------------------------------------------------------
//  Register::Bit() --
//---------------------------------------------------------------------------

void Register::Bit(int bit, bool value)
{
    if (value)
        Value(Value() | (1 << bit));
    else
        Value(Value() & (~(1 << bit)));
}

//---------------------------------------------------------------------------
//  Register::operator =() --
//---------------------------------------------------------------------------

Register & Register::operator =(ii32 value)
{
    Value(value);
    return *this;
}

//=============================================================================
//  CLASS ShadowRegisterBase  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ShadowRegisterBase::ShadowRegisterBase(IAddressingSpace & space, int offset, ii32 value)
    : Register(space, offset), mShadowValue(value)
{
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ShadowRegisterBase::ShadowRegisterBase(const std::string & label,
    IAddressingSpace & space, int offset, ii32 value)
    : Register(label, space, offset), mShadowValue(value)
{
}

//---------------------------------------------------------------------------
//  ShadowRegisterBase::Value() --
//---------------------------------------------------------------------------

void ShadowRegisterBase::Value(ii32 value)
{
    mShadowValue = value;
    ShadowRegisterChanged();
}

//---------------------------------------------------------------------------
//  ShadowRegisterBase::Value() --
//---------------------------------------------------------------------------

ii32 ShadowRegisterBase::Value() const
{
    return mShadowValue;
}

//---------------------------------------------------------------------------
//  ShadowRegisterBase::Apply() --
//---------------------------------------------------------------------------

void ShadowRegisterBase::Apply()
{
    Register::Value(mShadowValue);
}

//---------------------------------------------------------------------------
//  =() --
//---------------------------------------------------------------------------

ShadowRegisterBase & ShadowRegisterBase::operator =(ii32 value)
{
    Value(value);
    return *this;
}

//---------------------------------------------------------------------------
//  ShadowRegisterBase::ShadowValue() --
//---------------------------------------------------------------------------

void ShadowRegisterBase::ShadowValue(ii32 value)
{
    mShadowValue = value;
}

//=============================================================================
//  CLASS ShadowRegister  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ShadowRegister::ShadowRegister(IAddressingSpace & space, int offset, ii32 value)
    : ShadowRegisterBase(space, offset, value)
{
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ShadowRegister::ShadowRegister(const std::string & label,
    IAddressingSpace & space, int offset, ii32 value)
    : ShadowRegisterBase(label, space, offset, value)
{
}

//---------------------------------------------------------------------------
//  ShadowRegister::ShadowRegisterChanged() --
//---------------------------------------------------------------------------

void ShadowRegister::ShadowRegisterChanged()
{
    Apply();
}

//=============================================================================
//  CLASS RefreshableShadowRegister  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

RefreshableShadowRegister::RefreshableShadowRegister(IAddressingSpace & space, int offset, ii32 value)
    : ShadowRegisterBase(space, offset, value)
{
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

RefreshableShadowRegister::RefreshableShadowRegister(const std::string & label,
    IAddressingSpace & space, int offset, ii32 value)
    : ShadowRegisterBase(label, space, offset, value)
{
}

//---------------------------------------------------------------------------
//  RefreshableShadowRegister::ShadowRegisterChanged() --
//---------------------------------------------------------------------------

void RefreshableShadowRegister::ShadowRegisterChanged()
{
    Apply();
}

//---------------------------------------------------------------------------
//  RefreshableShadowRegister::Refresh() --
//---------------------------------------------------------------------------

void RefreshableShadowRegister::Refresh()
{
    mShadowValue = mAddressingSpace.At(mOffset);
}


//=============================================================================
//  CLASS CachedShadowRegister  --
//=============================================================================

CachedShadowRegister::CachedShadowRegister(IAddressingSpace & space, int offset, ii32 value)
    : ShadowRegisterBase(space, offset, value)
{
}

//---------------------------------------------------------------------------
// CachedShadowRegister::CachedShadowRegister() --
//---------------------------------------------------------------------------

CachedShadowRegister::CachedShadowRegister(const std::string & label,
    IAddressingSpace & space, int offset, ii32 value)
    : ShadowRegisterBase(label, space, offset, value)
{
}

//---------------------------------------------------------------------------
//  CachedShadowRegister::ShadowRegisterChanged() --
//---------------------------------------------------------------------------

void CachedShadowRegister::ShadowRegisterChanged()
{
}

//=============================================================================
//  CLASS MultiplexedShadowRegister  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

MultiplexedShadowRegister::MultiplexedShadowRegister(IAddressingSpace & space, int offset,
    ii32 value, int mux)
    : ShadowRegisterBase(space, offset, value), mMux(mux)
{
}

//---------------------------------------------------------------------------
// MultiplexedShadowRegister::MultiplexedShadowRegister() --
//---------------------------------------------------------------------------

MultiplexedShadowRegister::MultiplexedShadowRegister(const std::string & label,
    IAddressingSpace & space, int offset, ii32 value, int mux)
    : ShadowRegisterBase(label, space, offset, value), mMux(mux)
{
}

//---------------------------------------------------------------------------
// MultiplexedShadowRegister::Mux() --
//---------------------------------------------------------------------------

void MultiplexedShadowRegister::Mux(int mux)
{
    mMux=mux;
}

//---------------------------------------------------------------------------
//  MultiplexedShadowRegister::ShadowRegisterChanged() --
//---------------------------------------------------------------------------

void MultiplexedShadowRegister::ShadowRegisterChanged()
{
    Register::Value(mMux);
    Apply();
}

//=============================================================================
//  CLASS RegisterBit  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

RegisterBit::RegisterBit(Register & owner_register, int bit_no)
    : mOwner(owner_register), mBit(bit_no)
{
}

//---------------------------------------------------------------------------
//  RegisterBit::Set() --
//---------------------------------------------------------------------------

void RegisterBit::Set()
{
    mOwner.Bit(mBit, true);
}

//---------------------------------------------------------------------------
//  RegisterBit::Clear() --
//---------------------------------------------------------------------------

void RegisterBit::Clear()
{
    mOwner.Bit(mBit, false);
}

//---------------------------------------------------------------------------
//  RegisterBit::Flip() --
//---------------------------------------------------------------------------

void RegisterBit::Flip()
{
    Value(!Value());
}

//---------------------------------------------------------------------------
//  RegisterBit::Toggle() --
//---------------------------------------------------------------------------

void RegisterBit::Toggle()
{
    Flip();
    Flip();
}

//---------------------------------------------------------------------------
//  RegisterBit::Value() --
//---------------------------------------------------------------------------

void RegisterBit::Value(bool value)
{
    mOwner.Bit(mBit, value);
}

//---------------------------------------------------------------------------
//  RegisterBit::Value() --
//---------------------------------------------------------------------------

bool RegisterBit::Value() const
{
    return mOwner.Bit(mBit);
}

//---------------------------------------------------------------------------
//  RegisterBit::operator bool() --
//---------------------------------------------------------------------------

RegisterBit::operator bool() const
{
    return Value();
}

//---------------------------------------------------------------------------
//  RegisterBit::operator =() --
//---------------------------------------------------------------------------

RegisterBit & RegisterBit::operator =(bool value)
{
    Value(value);
    return *this;
}

//=============================================================================
//  CLASS RegisterBitGroup  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

RegisterBitGroup::RegisterBitGroup(Register & owner_register,
                 int bit_no,
                 int bit_count)
    : mOwner(owner_register), mFirstBit(bit_no), mBitCount(bit_count)
{
    if (bit_count == 32)
    {
        mMask = ~0;
        mOwnerMask = 0;
    }
    else
    {
        mMask = (1 << mBitCount) - 1;
        mOwnerMask = ~(mMask << mFirstBit);
    }
}

//---------------------------------------------------------------------------
// RegisterBitGroup::~RegisterBitGroup() --
//---------------------------------------------------------------------------

RegisterBitGroup::~RegisterBitGroup()
{
}

//---------------------------------------------------------------------------
//  RegisterBitGroup::Value() --
//---------------------------------------------------------------------------

void RegisterBitGroup::Value(ii32 value)
{
    ii32    tmp = mOwner.Value() & mOwnerMask;
    value &= mMask;
    value <<= mFirstBit;
    tmp |= value;
    mOwner.Value(tmp);
}

//---------------------------------------------------------------------------
//  RegisterBitGroup::Value() --
//---------------------------------------------------------------------------

ii32 RegisterBitGroup::Value() const
{
    return (mOwner.Value() >> mFirstBit) & mMask;
}

//---------------------------------------------------------------------------
//  RegisterBitGroup::Bit() --
//---------------------------------------------------------------------------

bool  RegisterBitGroup::Bit(unsigned int bit) const
{
    return (Value() & (1 << bit)) != 0;
}

//---------------------------------------------------------------------------
//  RegisterBitGroup::Bit() --
//---------------------------------------------------------------------------

void  RegisterBitGroup::Bit(unsigned int bit, bool value)
{
    if (value)
        Value(Value() | (1 << bit));
    else
        Value(Value() & (~(1 << bit)));
}

//---------------------------------------------------------------------------
//  RegisterBitGroup::operator=() --
//---------------------------------------------------------------------------

RegisterBitGroup & RegisterBitGroup::operator =(ii32 value)
{
    Value(value); return *this;
}

//---------------------------------------------------------------------------
//  RegisterBitGroup::operator int() --
//---------------------------------------------------------------------------

RegisterBitGroup::operator int() const
{
    return Value();
}

//=============================================================================
//  CLASS ReadOnlyRegisterBit  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ReadOnlyRegisterBit::ReadOnlyRegisterBit(ReadOnlyRegister & owner_register,
    int bit_no)
    : mOwner(owner_register), mBit(bit_no)
{
}

//---------------------------------------------------------------------------
//  ReadOnlyRegisterBit::Value() --
//---------------------------------------------------------------------------

bool ReadOnlyRegisterBit::Value() const
{
    return mOwner.Bit(mBit);
}

//---------------------------------------------------------------------------
//  ReadOnlyRegisterBit::operator bool() --
//---------------------------------------------------------------------------

ReadOnlyRegisterBit::operator bool() const
{
    return Value();
}

//=============================================================================
//  CLASS ReadOnlyRegisterBitGroup  --
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

ReadOnlyRegisterBitGroup::ReadOnlyRegisterBitGroup(ReadOnlyRegister & owner_register,
                         int bit_no,
                         int bit_count)
    : mOwner(owner_register), mFirstBit(bit_no), mBitCount(bit_count)
{
    mMask = (1 << mBitCount) - 1;
    mOwnerMask = ~(mMask << mFirstBit);
}

//---------------------------------------------------------------------------
// ReadOnlyRegisterBitGroup::~ReadOnlyRegisterBitGroup() --
//---------------------------------------------------------------------------

ReadOnlyRegisterBitGroup::~ReadOnlyRegisterBitGroup()
{
}

//---------------------------------------------------------------------------
//  ReadOnlyRegisterBitGroup::Value() --
//---------------------------------------------------------------------------

ii32 ReadOnlyRegisterBitGroup::Value() const
{
    return (mOwner.Value() >> mFirstBit) & mMask;
}

//---------------------------------------------------------------------------
//  ReadOnlyRegisterBitGroup::Bit() --
//---------------------------------------------------------------------------

bool ReadOnlyRegisterBitGroup::Bit(unsigned int bit) const
{
    return (Value() & (1 << bit)) != 0;
}

//---------------------------------------------------------------------------
//  int() --
//---------------------------------------------------------------------------

ReadOnlyRegisterBitGroup::operator int() const
{
    return Value();
}

//===========================================================================
//  CLASS SplitRegister
//===========================================================================
//  This class of register is mapped as halves on two 32 bit registers.

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

SplitRegister::SplitRegister(IAddressingSpace &space,
    int offset_low, int offset_high )
    : Low(space, offset_low), High(space, offset_high)
{
}

//---------------------------------------------------------------------------
// SplitRegister::~SplitRegister() --
//---------------------------------------------------------------------------

SplitRegister::~SplitRegister()
{
}

//---------------------------------------------------------------------------
//  SplitRegister::Value() --
//---------------------------------------------------------------------------

ii32 SplitRegister::Value()
{
    ii32 Result = Low.Value();
    //  Read HPI address high half word
    Result |= High.Value() << 16;

    return Result;
}

//---------------------------------------------------------------------------
//  SplitRegister::Value() --
//---------------------------------------------------------------------------

void SplitRegister::Value( ii32 value )
{
    Low.Value( value );
    High.Value( value >> 16 );
}

//---------------------------------------------------------------------------
//  SplitRegister::operatorii32() --
//---------------------------------------------------------------------------

SplitRegister::operator ii32()
{
    return Value();
}

//---------------------------------------------------------------------------
//  SplitRegister::operator = () --
//---------------------------------------------------------------------------

SplitRegister & SplitRegister::operator = ( ii32 value )
{
    Value( value );
    return *this;
}

//===========================================================================
//  CLASS HpiRegister  -- Special Split register for HPI accesses
//===========================================================================
//  Adds a check of the Ready register between accesses of the HPI registers

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

HpiRegister::HpiRegister( IAddressingSpace &space,
            int offset_low, int offset_high, int offset_hpi_ready )
    : SplitRegister( space, offset_low, offset_high ),
      HpiReady( space, offset_hpi_ready )
{
}

//---------------------------------------------------------------------------
//  HpiRegister::IsReady() --
//---------------------------------------------------------------------------

void HpiRegister::IsReady()
{
    Timer.uS(1);
    while (( HpiReady.Value() & 0x08) != 0x08 )
        ;
}

//---------------------------------------------------------------------------
//  HpiRegister::Value() --
//---------------------------------------------------------------------------

ii32 HpiRegister::Value()
{
    IsReady();
    ii32 Result = Low.Value() & 0xFFFF;
    //  Read HPI address high half word
    IsReady();
    Result |= ( High.Value() & 0xFFFF ) << 16;
    return Result;
}

//---------------------------------------------------------------------------
//  HpiRegister::Value() --
//---------------------------------------------------------------------------

void HpiRegister::Value( ii32 value )
{
    IsReady();
    Low.Value( value & 0xFFFF );

    IsReady();
    High.Value( ( value >> 16 ) & 0xFFFF );
}

//---------------------------------------------------------------------------
//  ii32() --
//---------------------------------------------------------------------------

HpiRegister::operator ii32()
{
    return Value();
}

//---------------------------------------------------------------------------
//  HpiRegister::operator=() --
//---------------------------------------------------------------------------

SplitRegister & HpiRegister::operator =( ii32 value )
{
    Value( value );
    return *this;
}

}  // end namespace Innovative

