// HighPass_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#include <HighPass_Mb.h>


#include <math.h>
#include <ipps.h>               // ipps.h is not redistributable!

namespace   Innovative
{

const float Pi = static_cast<float>(4.0*atan(1.0));

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif 

//---------------------------------------------------------------------------
// HighPass::HighPass() --
//---------------------------------------------------------------------------
 
HighPass::HighPass()
    : FFrequency(45000)
{
}

//---------------------------------------------------------------------------
// HighPass::HighPass() --
//---------------------------------------------------------------------------

HighPass::HighPass(const HighPass & other)
    : inherited( other ), FFrequency(other.FFrequency)
{
}

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

//---------------------------------------------------------------------------
//  DataLogger::operator = --
//---------------------------------------------------------------------------

HighPass & HighPass::operator = ( const HighPass & other )
{
    inherited::operator = ( other );

    FFrequency = other.FFrequency;

    return *this;
}

//---------------------------------------------------------------------------
//  HighPass::Frequency() --
//---------------------------------------------------------------------------

void HighPass::Frequency(float value)
{
    FFrequency = value;
    Start();
}

//---------------------------------------------------------------------------
//  HighPass::Frequency() --
//---------------------------------------------------------------------------

float HighPass::Frequency() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------
//  HighPass::CalculateCoefficients() --
//---------------------------------------------------------------------------

void HighPass::CalculateCoefficients()
{
	float CutLow = (Frequency() / FSampleRate) * 2.0f;

	//---------------------------------------------
	// adjust the number of coefficients to be ODD
	//---------------------------------------------
    size_t count = FCoefficient.size();
	size_t CoefNum = count;
	if (count % 2 == 0)
		++CoefNum;

	size_t HalfLen = (CoefNum - 1) / 2;

	//--------------------------------------------------------
	// Allocate memory for coefficients if length changed
	//--------------------------------------------------------
    FCoefficient.resize(CoefNum);

    //------------------
    // Highpass filter
    //------------------
    FCoefficient[HalfLen] = CutLow;
    for (size_t Cnt = 1; Cnt <= HalfLen; Cnt++)
        {
        float TmpFloat = Pi * Cnt;
        FCoefficient[HalfLen + Cnt] = sin(CutLow * TmpFloat) / TmpFloat;
        FCoefficient[HalfLen - Cnt] = FCoefficient[HalfLen + Cnt];
        }

    //------------------------------
    // multiplying with a window
    //------------------------------
    //  TmpFloat := 2.0 * Pi / (CoefNum - 1.0);
    ApplyWindow(&FCoefficient[0], CoefNum, FWindowType, FAlpha);
    ippsMulC_32f_I(-1, &FCoefficient[0], static_cast<int>(CoefNum));

    //------------------------------
    // Normalize GAIN to 1 ( Step 1 )
    //------------------------------
    float   Sum = 0.0f;
    for (size_t Cnt = 0; Cnt < CoefNum; Cnt++)
        {
        if (Cnt % 2 == 0)
            Sum += FCoefficient[Cnt];   // poly(-1), even +, odd -
        else
            Sum -= FCoefficient[Cnt];

        }

    ++FCoefficient[HalfLen];
    ++Sum;

    //------------------------------
    // Normalize GAIN to 1 ( Step 2 )
    //------------------------------
    if (Sum < 0)
        Sum = -Sum;

    ippsDivC_32f_I(Sum, &FCoefficient[0], static_cast<int>(CoefNum));
}


} // namespace Innovative
