//  HpiEngine_Mb.h    --  HPI hardware interface classes
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

//---------------------------------------------------------------------------

#ifndef HpiEngine_MbH
#define HpiEngine_MbH

#include <HardwareRegister_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  STRUCT HpiEngineAddresses  -- Address Information for Hpi registers
//===========================================================================

struct HpiEngineAddresses
{
    unsigned int AddrLsw;
    unsigned int AddrMsw;
    unsigned int DataLsw;
    unsigned int DataMsw;
    unsigned int DataIncrLsw;
    unsigned int DataIncrMsw;
    unsigned int ControlLsw;
    unsigned int ControlMsw;
};

//===========================================================================
//  STRUCT HpiCeSpaceValues  -- Contents of CE registers
//===========================================================================

struct HpiCeSpaceValues
{
    unsigned int GlobalCtl;
    unsigned int Ce1;
    unsigned int Ce0;
    unsigned int Ce2;
    unsigned int Ce3;
    unsigned int SdramTime;
    unsigned int SdramExt;
    unsigned int SdramCtl;
};

//===========================================================================
//  CLASS HpiEngine  -- HPI hardware control
//===========================================================================

class HpiEngine
{
public:
    //
    //  Member Functions
    HpiEngine(HpiEngineAddresses & Hpi);

    void  Link(ii32 * base_ptr)  {  PortMemory.Link(base_ptr);  };

    void  Reset();
    void  Interrupt();
    void  ConfigureTargetSdram();
    void  LoadCeSpaceValues(HpiCeSpaceValues & values) {  Values = values;  };

    void  WriteBlock(const int*  src, int dst, int size);
    void  ReadBlock(int src, int* dst, int size);
    void  Poke(int address, int value);
    int   Peek(int address);

    void  SetAddress(int address);
    int   GetAddress() const;

    void  SlowHpiMode(bool state) {  mSlowHpi = state;  };

private:
    //
    //  Member Data
    AddressingSpace    PortMemory;

    mutable HpiRegister HpiAddress;
    HpiRegister        HpiData;
    HpiRegister        HpiDataIncr;
    HpiRegister        HpiControl;

    bool               mSlowHpi;
    HpiCeSpaceValues   Values;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
