//
// Ics8402_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Ics8402_Mb.h>
#include <SystemSupport_Mb.h>
#include <algorithm>

//---------------------------------------------------------------------------

namespace Innovative
{

static const double _VcoLow = 250.0e6;
static const double _VcoHigh = 700.0e6;

//=============================================================================
//  CLASS Ics8402  --  Ics8402 Pll Implementation
//=============================================================================

//-----------------------------------------------------------------------------
//  constructor for class Ics8402
//-----------------------------------------------------------------------------

Ics8402::Ics8402(int index, AddressingSpace & space, int offset)
	: inherited(index, space, offset), VcoLow(_VcoLow), VcoHigh(_VcoHigh)
{
}

//-----------------------------------------------------------------------------
//  constructor for class Ics8402
//-----------------------------------------------------------------------------

Ics8402::Ics8402(double freq, double ref, bool enabled, int index,
                 AddressingSpace & space, int offset)
	: inherited(freq, ref, enabled, index, space, offset),
      VcoLow(_VcoLow), VcoHigh(_VcoHigh)
{
	DoSetFrequency(freq);
	DoSetEnabled(enabled);
}

//-----------------------------------------------------------------------------
//  Ics8402::Max()  --  Set maximum output rate
//-----------------------------------------------------------------------------

void Ics8402::Max(double value)
{
	VcoHigh = value;
}

//-----------------------------------------------------------------------------
//  Ics8402::Max()  --  Return maximum output rate
//-----------------------------------------------------------------------------

double Ics8402::Max() const
{
	return VcoHigh;
}

//-----------------------------------------------------------------------------
//  Ics8402::Min()  --  Set minimum output rate
//-----------------------------------------------------------------------------

void Ics8402::Min(double value)
{
	VcoLow = value * MaxOutputDivider();
}

//-----------------------------------------------------------------------------
//  Ics8402::Min()  --  Return minimum output rate
//-----------------------------------------------------------------------------

double Ics8402::Min() const
{
	return VcoLow / MaxOutputDivider();
}

//-----------------------------------------------------------------------------
//  Ics8402::MaxOutputDivider()  --  Return max output post-divider
//-----------------------------------------------------------------------------

int Ics8402::MaxOutputDivider() const
{
	return 16;
}

//-----------------------------------------------------------------------------
//  Ics8402::GetFrequencyActual()  --  Calculate actual clock rate
//-----------------------------------------------------------------------------

double Ics8402::GetFrequencyActual() const
{
	return (CtlReg.M.Value() * Reference()) /
		((2<<CtlReg.N.Value()) * ClockFactor());
}

//-----------------------------------------------------------------------------
//  Ics8402::SetFrequency()  --  Calculate settings for a frequency
//-----------------------------------------------------------------------------

void Ics8402::SetFrequency(double freq)
{
	DoSetFrequency(freq);
}

//-----------------------------------------------------------------------------
//  Ics8402::DoSetFrequency()  --  Calculate settings for a frequency
//-----------------------------------------------------------------------------

void Ics8402::DoSetFrequency(double freq)
{
	if (FReference == 0.0)
		return;

	freq *= ClockFactor();

    //
    //  Determine which frequency divisor to use...
	if (freq < (VcoHigh/16.0e6))        // 1st band to VcoHigh/16 Mhz
		CtlReg.N.Value(3);
	else if (freq < (VcoHigh/8.0e6))    // 2nd band to VcoHigh/8 Mhz
		CtlReg.N.Value(2);
	else if (freq < (VcoHigh/4.0e6))    // 3rd band to VcoHigh/4 Mhz
		CtlReg.N.Value(1);
	else                            // 4th band to VcoHigh/2 Mhz
        CtlReg.N.Value(0);
    //
    //  Calculate M field
    int M_val = static_cast<int>(freq * (2<<CtlReg.N.Value()) / (Reference()));

	//  'M' field limits:  The "M" parameter must be chosen to provide a 'native'
	//      frequency between VcoLow and VcoHigh Mhz. This can then be divided
    //      by the 'N' factor.
	//          M(min) = F(min)/F(xtal)     M(max) = F(max)/F(xtal)
	const int M_max = static_cast<int>(VcoHigh / Reference());
	const int M_min = static_cast<int>(VcoLow / Reference());

    //  ...Only allow M values in our sync band...
	M_val = std::min(M_max, M_val);
	M_val = std::max(M_min, M_val);

	CtlReg.M.Value(M_val);

	CtlReg.Reset = 1;
	Sleep(1);
	CtlReg.Reset = 0;
	Sleep(1);
}

//-----------------------------------------------------------------------------
//  Ics8402::SetEnabled()  --  Activate / Deactivate output
//-----------------------------------------------------------------------------

void  Ics8402::SetEnabled(bool state)
{
	DoSetEnabled(state);
}

//-----------------------------------------------------------------------------
//  Ics8402::DoSetEnabled()  --  Activate / Deactivate output
//-----------------------------------------------------------------------------

void  Ics8402::DoSetEnabled(bool /*state*/)
{
}

} // namespace Innovative
