//
// Ics843002_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Ics843002_MbH
#define Ics843002_MbH

#include <Clockbase_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Magic_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Ics843002  --  PLL timebase control
//=============================================================================

class Ics843002 : public ClockBase
{
	friend class ClockGeneratorDeviceMap;

public:
	enum IIClockSource { rcSyncIn, rcTcxo };
//    enum IIMultiplier { m28, m32 };
    //
	//  Member Functions
    Ics843002(int index, AddressingSpace &space, int offset);
    Ics843002(double freq, double ref, bool enabled, int index,
              AddressingSpace & space, int offset);

	// Methods
	Ics843002 &         Source(IIClockSource value);
	IIClockSource       Source() const;

	Ics843002 &         SourceFrequency(double value);
	double				SourceFrequency() const;

	bool				Locked() const;

	void                Apply();

	// Output frequency range information
	double				Max() const;
	double 				Min() const;

    void                Disable();

	class ControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit           Reset;
		RegisterBitGroup      Mux;
        RegisterBit           OutputEnable;
		RegisterBit           Multiplier;

        ControlRegister(AddressingSpace &space,
            int offset)
        :  inherited(space, offset),
           Reset(*this, 0), Mux(*this, 1, 2),
           OutputEnable(*this, 3), Multiplier(*this, 4)
            {
            }
    };

    //
    //  Class StatusRegister
    class StatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
		ReadOnlyRegisterBit         Reset;
        ReadOnlyRegisterBitGroup    Mux;
        ReadOnlyRegisterBit         OutputEnable;
        ReadOnlyRegisterBit         Multiplier;
		ReadOnlyRegisterBit         Lock;

    public:
        StatusRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               Reset(*this, 0), Mux(*this, 1, 2),
               OutputEnable(*this, 3), Multiplier(*this, 4),
               Lock(*this, 5)
            {
			//  Default setup ????
			}
    };

    //  NP A register
    class QADividerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
		RegisterBitGroup      Reg;

        QADividerRegister(AddressingSpace &space, int offset)
        :  inherited(space, offset),
           Reg(*this, 0, 3)
        {  }

        // Properties
        void    Divisor(int value)
            {
            const int divisor[] = {1, 2, 4, 8, 12, 14, 16};
            int i=0;
            for (; (i < 7) && (value > divisor[i]); ++i)
                ;
            Reg.Value(i);
            }
		int     Divisor() const
            {
            const int divisor[] = {1, 2, 4, 8, 12, 14, 16};
            return divisor[Reg.Value()];
            }

        void    Disable()
            {  Reg.Value(7);  }
    };

    //  NP B register
    class QBDividerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Reg;

        QBDividerRegister(AddressingSpace &space, int offset)
        :  inherited(space, offset),
           Reg(*this, 0, 3)
		{  }

		// Properties
        void    Divisor(int value)
            {  Reg.Value(FloorLog2(value));  }
        int     Divisor() const
            {  return 1 << (Reg.Value() & 3);  }
        void    Xoin()
            {  Reg.Value(4);  }
        void    Xofb()
            {  Reg.Value(5);  }
		void    Mp()
			{  Reg.Value(6);  }
        void    Disable()
			{  Reg.Value(7);  }
    };

    //  VCLK register
    class VclkDividerRegister : public ShadowRegister
    {
		typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Reg;

        VclkDividerRegister(AddressingSpace &space, int offset)
        :  inherited(space, offset),
           Reg(*this, 0, 2)
        {  }

        // Properties
        void    Divisor(int value)
            {
            const int divisor[] = {1, 12, 16};
			int i=0;
			for (; (i < 3) && (value > divisor[i]); ++i)
				;
			Reg.Value(i);
			}
		int     Divisor() const
			{
			const int divisor[] = {1, 12, 16};
			return divisor[Reg.Value()];
			}

		void    Disable()
			{  Reg.Value(3);  }
	};

	//
	//  Member Data
	QADividerRegister   QaReg;
	QBDividerRegister   QbReg;
	VclkDividerRegister VclkReg;
	ShadowRegister      InputReg;
	ShadowRegister      FeedbackReg;
	ControlRegister     ControlReg;
	StatusRegister      StatusReg;

protected:
	// Data
	Trace				Debug;

	// Fields
	double				FSourceFrequency;

	// Overrides
	virtual void    SetFrequency(double freq);
	virtual double  GetFrequencyActual() const;
	virtual void    SetEnabled(bool state);

	// Support Methods
	void 			ClosestQAMatch(double freq);
	void    		DoSetFrequency(double freq);
	void    		DoSetEnabled(bool state);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative

#endif
