// IdromDatagram_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  Datagram for viewing memory like an IDROM page
//---------------------------------------------------------------------------

#include "IdromDatagram_Mb.h"

namespace Innovative
{

//============================================================================
//  CLASS IdromIoDG  --  "Idrom-like" Access Datagram
//=============================================================================
//---------------------------------------------------------------------------
//  IdromIoDG::SizeInInts() -- size of idrom data in INTs
//---------------------------------------------------------------------------

size_t  IdromIoDG::SizeInInts() const
{
    return Intf.idrom_data_size()*sizeof(int)/sizeof(int);
}

//---------------------------------------------------------------------------
//  IdromIoDG::SizeInFloats() -- size of idrom data in Floats
//---------------------------------------------------------------------------

size_t  IdromIoDG::SizeInFloats() const
{
    return Intf.idrom_data_size()*sizeof(int)/sizeof(float);
}

//---------------------------------------------------------------------------
//  IdromIoDG::SizeInDoubles() -- size of idrom data in Doubles
//---------------------------------------------------------------------------

size_t  IdromIoDG::SizeInDoubles() const
{
    return Intf.idrom_data_size()*sizeof(int)/sizeof(double);
}

//---------------------------------------------------------------------------
//  IdromIoDG::SizeInChars() -- size of idrom data in Chars
//---------------------------------------------------------------------------

size_t  IdromIoDG::SizeInChars() const
{
    return Intf.idrom_data_size()*sizeof(int)/sizeof(char);
}

//---------------------------------------------------------------------------
//  IdromIoDG::SizeInShorts() -- size of idrom data in Shorts
//---------------------------------------------------------------------------

size_t  IdromIoDG::SizeInShorts() const
{
    return Intf.idrom_data_size()*sizeof(int)/sizeof(short);
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsInt() const -- Read Data field as int
//-----------------------------------------------------------------------------

int IdromIoDG::AsInt(unsigned int idx) const
{
    const int * pkt = Intf.const_idrom_base();
    //  Range check
    if (idx>=SizeInInts())
        return 0;

    return pkt[idx];
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsInt() -- Write Data field as int
//-----------------------------------------------------------------------------

void IdromIoDG::AsInt(unsigned int idx, int data)
{
    int * pkt = Intf.idrom_base();
    //  Range check
    if (idx>=SizeInInts())
        return;
    
    pkt[idx] = data;
    Intf.after_data_write();
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsFloat() const -- Read Data field as float
//-----------------------------------------------------------------------------

float IdromIoDG::AsFloat(unsigned int idx) const
{
    const float * pkt = reinterpret_cast<const float *>(Intf.const_idrom_base());
    //  Range check
    if (idx>=SizeInFloats())
        return 0;

    return pkt[idx];
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsFloat() -- Write Data field as float
//-----------------------------------------------------------------------------

void IdromIoDG::AsFloat(unsigned int idx, float data)
{
    float * pkt = reinterpret_cast<float *>(Intf.idrom_base());
    //  Range check
    if (idx>=SizeInFloats())
        return;

    pkt[idx] = data;
    Intf.after_data_write();
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsDouble() const -- Read Data field as double
//-----------------------------------------------------------------------------

double IdromIoDG::AsDouble(unsigned int idx) const
{
    const double * pkt = reinterpret_cast<const double *>(Intf.const_idrom_base());
    //  Range check
    if (idx>=SizeInDoubles())
        return 0;

    return pkt[idx];
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsDouble() -- Write Data field as double
//-----------------------------------------------------------------------------

void IdromIoDG::AsDouble(unsigned int idx, double data)
{
    double * pkt = reinterpret_cast<double *>(Intf.idrom_base());
    //  Range check
    if (idx>=SizeInDoubles())
        return;

    pkt[idx] = data;
    Intf.after_data_write();
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsShort() const -- Read Data field as short
//-----------------------------------------------------------------------------

short IdromIoDG::AsShort(unsigned int idx) const
{
    const short * pkt = reinterpret_cast<const short *>(Intf.const_idrom_base());
    //  Range check
    if (idx>=SizeInShorts())
        return 0;

    return pkt[idx];
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsShort() -- Write Data field as short
//-----------------------------------------------------------------------------

void IdromIoDG::AsShort(unsigned int idx, short data)
{
    short * pkt = reinterpret_cast<short *>(Intf.idrom_base());
    //  Range check
    if (idx>=SizeInShorts())
        return;

    pkt[idx] = data;
    Intf.after_data_write();
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsChar() const -- Read Data field as char
//-----------------------------------------------------------------------------

char IdromIoDG::AsChar(unsigned int idx) const
{
    const char * pkt = reinterpret_cast<const char *>(Intf.const_idrom_base());
    //  Range check
    if (idx>=SizeInChars())
        return 0;

    return pkt[idx];
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsChar() -- Write Data field as char
//-----------------------------------------------------------------------------

void IdromIoDG::AsChar(unsigned int idx, char data)
{
    char * pkt = reinterpret_cast<char *>(Intf.idrom_base());
    //  Range check
    if (idx>=SizeInChars())
        return;

    pkt[idx] = data;
    Intf.after_data_write();
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsString() const -- Read Data field(s) as String of Char
//-----------------------------------------------------------------------------

std::string  IdromIoDG::AsString(unsigned int int_idx, unsigned int ints_to_use) const
{
    std::string result;
    //  Range check
    unsigned int char_idx = int_idx*sizeof(int);
    unsigned int chars_to_use = ints_to_use*sizeof(int);

    if (char_idx>=SizeInChars())
        return result;

    while (chars_to_use)
        {
        char next_char = AsChar(char_idx++);
        if (next_char==0)
            break;
        result.push_back(next_char);

        chars_to_use--;
        }
    return result;
}

//-----------------------------------------------------------------------------
//  IdromIoDG::AsString() -- Write Data field(s) as String of Char
//-----------------------------------------------------------------------------

void  IdromIoDG::AsString(unsigned int int_idx, unsigned int ints_to_use, const std::string & text)
{
    //  Range check
    unsigned int char_idx = int_idx*sizeof(int);
    unsigned int chars_to_use = ints_to_use*sizeof(int);

    if (char_idx>=SizeInChars())
        return;

    size_t text_idx = 0;
    while (chars_to_use)
        {
        char next_char;
        if (text_idx>=text.size())
            next_char = 0;
        else
            next_char = text[text_idx];
        text_idx++;
        AsChar(char_idx++, next_char);

        chars_to_use--;
        }
}

}  // end namespace Innovative
