// IdromDatagram_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  Datagram for viewing memory like an IDROM page
//---------------------------------------------------------------------------

#ifndef IdromDatagram_MbH
#define IdromDatagram_MbH

#include <DataTypes_Mb.h>
#include <string>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS IIdromViewable  --  Interface to support "Idrom-like" Access DB
//=============================================================================

class IIdromViewable
{
public:
    friend class IdromIoDG;

    virtual ~IIdromViewable()  {}

protected:
    virtual const int *  const_idrom_base() const = 0;
    virtual int *        idrom_base() = 0;

    virtual size_t       idrom_size() const = 0;
    virtual size_t       idrom_reserve_size() const = 0;
    size_t               idrom_data_size() const
                            {  return idrom_size() - idrom_reserve_size();  }

    virtual void         after_data_write() = 0;

};

//============================================================================
//  CLASS IdromIoDG  --  "Idrom-like" Access Datagram
//============================================================================

class IdromIoDG
{
public:
    IdromIoDG(IIdromViewable & intf)
        :  Intf(intf)
        {}
    IdromIoDG(const IIdromViewable & intf)
        :  Intf(const_cast<IIdromViewable &>(intf))
        {}
    ~IdromIoDG()
        {}

    size_t   SizeInInts() const;
    size_t   SizeInFloats() const;
    size_t   SizeInDoubles() const;
    size_t   SizeInChars() const;
    size_t   SizeInShorts() const;

    int    AsInt(unsigned int idx) const;
    void   AsInt(unsigned int idx, int data);

    float  AsFloat(unsigned int idx) const;
    void   AsFloat(unsigned int idx, float data);

    double AsDouble(unsigned int idx) const;
    void   AsDouble(unsigned int idx, double data);

    short  AsShort(unsigned int idx) const;
    void   AsShort(unsigned int idx, short data);

    char   AsChar(unsigned int idx) const;
    void   AsChar(unsigned int idx, char data);

    std::string  AsString(unsigned int int_idx, unsigned int ints_to_use) const;
    void         AsString(unsigned int int_idx, unsigned int ints_to_use, const std::string & text);

private:
    IIdromViewable &  Intf;
};

//===========================================================================
//  ToOffset<T>()  --  Template function to offset in T's from int offset
//===========================================================================
//  Use to offset in elements of type T given an integer offset
//     offset_shorts = ToOffset<short>( 100 ) );
//     (offset_shorts is 200)

template <class T>
size_t ToOffset(size_t offset_in_ints)
{
    size_t elements = offset_in_ints * sizeof(int) / sizeof(T);
    return elements;
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative
//---------------------------------------------------------------------------
#endif
