// Iir_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#include <Iir_Mb.h>
#include <ipp.h>
#include <Exception_Mb.h>
#include <IppFloatDG_Mb.h>

namespace   Innovative
{

static const int DefaultIIROrder = 12;

//===========================================================================
//  CLASS Iir  --
//===========================================================================

//---------------------------------------------------------------------------
// Iir::Iir() --
//---------------------------------------------------------------------------

Iir::Iir()
{
	Order(DefaultIIROrder);
}

//---------------------------------------------------------------------------
// Iir::Iir() --
//---------------------------------------------------------------------------

Iir::Iir( const Iir & other )
	: inherited(other), FCoefficient(other.FCoefficient)
{
	Order(DefaultIIROrder);
}

//---------------------------------------------------------------------------
// Iir::~Iir() --
//---------------------------------------------------------------------------

Iir::~Iir()
{
	Stop();
}

//---------------------------------------------------------------------------
//  DataLogger::operator = --
//---------------------------------------------------------------------------

Iir & Iir::operator = (const Iir & other)
{
	if (this != &other) // protect against invalid self-assignment
		FCoefficient = other.FCoefficient;

	return *this;
}

//---------------------------------------------------------------------------
//  Iir::Filter() --
//---------------------------------------------------------------------------

bool Iir::Filter(const IppFloatDG & in, IppFloatDG & out)
{
	bool result = false;

	if (!FSpec.size())
		Start();

	IppsIIRState64f_32f ** pSpec = reinterpret_cast<IppsIIRState64f_32f**>(&mState);

	IppStatus Rtn = ippsIIR64f_32f(in.Ptr(), out.Ptr(),
								   static_cast<unsigned int>(out.size()),
								   *pSpec);

	if (Rtn != ippStsNoErr)
		{
		out = in;
		result = true;
		}

	return result;
}

//---------------------------------------------------------------------------
//  Iir::Start() --
//---------------------------------------------------------------------------

bool Iir::Start()
{
	Stop();

	if (!Order())
		return false;

	int sz;
	IppStatus status = ippsIIRGetStateSize64f_32f(static_cast<int>(Order()), &sz);
	if (status != ippStsNoErr)
		throw MalibuException("Unable to get IIR filter state size");
	FSpec.resize(sz);

	IppsIIRState64f_32f ** pSpec = reinterpret_cast<IppsIIRState64f_32f**>(&mState);

	status = ippsIIRInit64f_32f(pSpec,
								&FCoefficient[0],
								static_cast<int>(Order()),
								&FDelay[0],
								reinterpret_cast<Ipp8u*>(&FSpec[0]));

	if (status != ippStsNoErr)
		throw MalibuException("Unable to allocate IIR state structure");

	return true;
}

//---------------------------------------------------------------------------
//  Iir::Stop() --
//---------------------------------------------------------------------------

void Iir::Stop()
{
}

//---------------------------------------------------------------------------
// Iir::SetOrder() --  Specify number of filter taps
//---------------------------------------------------------------------------

void Iir::SetOrder(size_t count)
{
	FCoefficient.resize(2*(count+1), 1.0);
	FDelay.resize(count, 0.0);

	inherited::SetOrder(count);

	Start();
}

//---------------------------------------------------------------------------
// Iir::A() --  Change filter tap A
//---------------------------------------------------------------------------

void Iir::A(unsigned int idx, double value)
{
	FCoefficient[idx+Order()] = value;
}

//---------------------------------------------------------------------------
// Iir::A() -- Query filter tap A
//---------------------------------------------------------------------------

double Iir::A(unsigned int idx) const
{
	return FCoefficient[idx+Order()];
}

//---------------------------------------------------------------------------
// Iir::B() -- Change filter tap B
//---------------------------------------------------------------------------

void Iir::B(unsigned int idx, double value)
{
	FCoefficient[idx] = value;
}

//---------------------------------------------------------------------------
// Iir::B() -- Query filter tap B
//---------------------------------------------------------------------------

double Iir::B(unsigned int idx) const
{
	return FCoefficient[idx];
}

//---------------------------------------------------------------------------
// Iir::Delay() -- Change delay line tap
//---------------------------------------------------------------------------

void Iir::Delay(unsigned int idx, double value)
{
	FDelay[idx] = value;
}

//---------------------------------------------------------------------------
// Iir::Delay() -- Query delay line tap
//---------------------------------------------------------------------------

double Iir::Delay(unsigned int idx) const
{
	return FDelay[idx];
}


//=============================================================================
//  CLASS IirCommonFilter  --
//=============================================================================

//---------------------------------------------------------------------------
// IirCommonFilter::IirCommonFilter() --  Ctor
//---------------------------------------------------------------------------

IirCommonFilter::IirCommonFilter()
	: FFrequency(10000.0), FType(Butterworth), FRipple(0.5)
{
}

//---------------------------------------------------------------------------
// IirCommonFilter::IirCommonFilter() --  Copy ctor
//---------------------------------------------------------------------------

IirCommonFilter::IirCommonFilter(const IirCommonFilter & other)
	: inherited(other)
{
	FFrequency = other.FFrequency;
	FType = other.FType;
	FRipple = other.FRipple;
}

//---------------------------------------------------------------------------
// IirCommonFilter::IirCommonFilter() --  Assignment
//---------------------------------------------------------------------------

IirCommonFilter & IirCommonFilter::operator = (const IirCommonFilter & other)
{
	if (this != &other) // protect against invalid self-assignment
		{
		FFrequency = other.FFrequency;
		FType = other.FType;
		FRipple = other.FRipple;
		}

	return *this;
}

//---------------------------------------------------------------------------
// IirCommonFilter::Type() --  Set filter type
//---------------------------------------------------------------------------

void IirCommonFilter::Type(FilterType value)
{
	FType = value;
	CalculateCoefficients();
	Start();
}

//---------------------------------------------------------------------------
// IirCommonFilter::Type() --  Get filter type
//---------------------------------------------------------------------------

IirCommonFilter::FilterType IirCommonFilter::Type() const
{
	return FType;
}

//---------------------------------------------------------------------------
// IirCommonFilter::Frequency() --  Set filter frequency
//---------------------------------------------------------------------------

void IirCommonFilter::Frequency(float value)
{
	FFrequency = value;
	CalculateCoefficients();
	Start();
}

//---------------------------------------------------------------------------
// IirCommonFilter::Frequency() --  Get filter frequency
//---------------------------------------------------------------------------

float IirCommonFilter::Frequency() const
{
	return FFrequency;
}

//=============================================================================
//  CLASS IirHighPass  --
//=============================================================================

//---------------------------------------------------------------------------
// IirHighPass::IirHighPass() --  Ctor
//---------------------------------------------------------------------------

IirHighPass::IirHighPass()
{
}

//---------------------------------------------------------------------------
// IirHighPass::IirHighPass() --  Copy ctor
//---------------------------------------------------------------------------

IirHighPass::IirHighPass(const IirHighPass & other)
	: inherited(other)
{
}

//---------------------------------------------------------------------------
// IirHighPass::IirHighPass() --  Assignment
//---------------------------------------------------------------------------

IirHighPass & IirHighPass::operator = (const IirHighPass & other)
{
	if (this != &other) // protect against invalid self-assignment
		{
		//
		}

	return *this;
}

//---------------------------------------------------------------------------
// IirHighPass::CalculateCoefficient() --  Design filter
//---------------------------------------------------------------------------

void IirHighPass::CalculateCoefficients()
{
	double f = FFrequency/FSampleRate;

	IppsIIRFilterType type;
	switch (FType)
		{
		default:
		case Butterworth:   type = ippButterworth;      break;
		case Chebyshev1:   	type = ippChebyshev1;      	break;
		}

	int sz;
	ippsIIRGenGetBufferSize(static_cast<int>(Order()), &sz);
	FWork.resize(sz);

	IppStatus status = ippsIIRGenHighpass_64f(f, FRipple, static_cast<int>(Order()), &FCoefficient[0], type, &FWork[0]);
	if (status != ippStsNoErr)
		throw MalibuException("Unable to generate IIR highpass coefficients");
}

//=============================================================================
//  CLASS IirLowPass  --
//=============================================================================

//---------------------------------------------------------------------------
// IirLowPass::IirLowPass() --  Ctor
//---------------------------------------------------------------------------

IirLowPass::IirLowPass()
{
}

//---------------------------------------------------------------------------
// IirLowPass::IirLowPass() --  Copy ctor
//---------------------------------------------------------------------------

IirLowPass::IirLowPass(const IirLowPass & other)
	: inherited(other)
{
}

//---------------------------------------------------------------------------
// IirLowPass::IirLowPass() --  Assignment
//---------------------------------------------------------------------------

IirLowPass & IirLowPass::operator = (const IirLowPass & other)
{
	if (this != &other) // protect against invalid self-assignment
		{
		//
		}

	return *this;
}

//---------------------------------------------------------------------------
// IirLowPass::CalculateCoefficients() --  Design filter
//---------------------------------------------------------------------------

void IirLowPass::CalculateCoefficients()
{
	double f = FFrequency/FSampleRate;

	IppsIIRFilterType type;
	switch (FType)
		{
		default:
		case Butterworth:   type = ippButterworth;      break;
		case Chebyshev1:   	type = ippChebyshev1;      	break;
		}

	int sz;
	ippsIIRGenGetBufferSize(static_cast<int>(Order()), &sz);
	FWork.resize(sz);

	IppStatus status = ippsIIRGenLowpass_64f(f, FRipple, static_cast<int>(Order()), &FCoefficient[0], type, &FWork[0]);
	if (status != ippStsNoErr)
		throw MalibuException("Unable to generate IIR lowpass coefficients");
}

} // namespace Innovative
