// IIIir.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Iir_MbH
#define Iir_MbH

#include <CommonFilter_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS Iir  --  
//=============================================================================

class Iir : public CommonFilter
{
	typedef CommonFilter inherited;

public:
	Iir();
	Iir(const Iir & other);
	~Iir();

	Iir & operator = (const Iir & other);

	// Filter coefficients
	void 			A(unsigned int idx, double value);
	double 			A(unsigned int idx) const;

	void 			B(unsigned int idx, double value);
	double 			B(unsigned int idx) const;

	void 			Delay(unsigned int idx, double value);
	double 			Delay(unsigned int idx) const;

	virtual bool    Filter(const IppFloatDG & in, IppFloatDG & out);

protected:
	virtual void Stop();
	virtual bool Start();

	// Data
	typedef std::vector<double> DoubleBuffer;
	DoubleBuffer FCoefficient;
	DoubleBuffer FDelay;

	void *          mState;

	// Methods
	virtual void    SetOrder(size_t value);
};

//=============================================================================
//  CLASS IirCommonFilter  --
//=============================================================================

class IirCommonFilter : public Iir
{
	typedef Iir inherited;

public:
	enum FilterType { Butterworth, Chebyshev1 };

	// Ctor
	IirCommonFilter();
	IirCommonFilter(const IirCommonFilter & other);

	// Assignment
	IirCommonFilter & operator = (const IirCommonFilter & other);

	// Properties
	float         	Frequency() const;
	void            Frequency(float value);

	void            Type(FilterType value);
	FilterType		Type() const;

	float         	Ripple() const;
	void            Ripple(float value);

protected:
	// Data
	float           FFrequency;
	FilterType      FType;
	float           FRipple;
	UCharArray		FWork;
};

//=============================================================================
//  CLASS IirHighPass  --
//=============================================================================

class IirHighPass : public IirCommonFilter
{
	typedef IirCommonFilter inherited;

public:
	enum FilterType { Butterworth, Chebyshev1 };

	// Ctor
	IirHighPass();
	IirHighPass(const IirHighPass & other);

	// Assignment
	IirHighPass & operator = (const IirHighPass & other);

protected:

	// Methods
	virtual void   	CalculateCoefficients();
};

//=============================================================================
//  CLASS IirLowPass  --
//=============================================================================

class IirLowPass : public IirCommonFilter
{
	typedef IirCommonFilter inherited;

public:
	enum FilterType { Butterworth, Chebyshev1 };

	// Ctor
	IirLowPass();
	IirLowPass(const IirLowPass & other);

	// Assignment
	IirLowPass & operator = (const IirLowPass & other);

protected:

	// Methods
	virtual void   	CalculateCoefficients();
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative

#endif
