// IniFile_Mb.h --  Class to access local configuration files
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "IniFile_Mb.h"
#include <fstream>
#include <sstream>
#include <StringSupport_Mb.h>
#include <cstring>
#include <iomanip>
#include "Poco/Util/JSONConfiguration.h"
#include "Poco/JSON/JSONException.h"

#ifdef _MSC_VER
#pragma warning (push)
#pragma warning (disable : 4786 4503)
#endif

#ifdef __BORLANDC__
	#pragma option push
	#pragma warn -8027
	#pragma warn -8060
#endif

namespace Innovative
{

struct Jimpl
{
    Poco::Util::JSONConfiguration config;
};

//---------------------------------------------------------------------------
//  JsonProtocol::JsonProtocol() --
//---------------------------------------------------------------------------
JsonProtocol::JsonProtocol(int max_len) : IniProtocol(max_len)
{
    pimpl = new Jimpl;
}

//---------------------------------------------------------------------------
//  JsonProtocol::~JsonProtocol() --
//---------------------------------------------------------------------------
JsonProtocol::~JsonProtocol()
{
    delete pimpl;
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- bool
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, bool Value)
{
    pimpl->config.setBool(tag, Value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write() -- int
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, int Value)
{
    pimpl->config.setInt(tag, Value);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- unsigned int
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, unsigned int Value)
{
    pimpl->config.setUInt(tag, Value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write() -- unsigned int 64
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, ii64 Value)
{
    pimpl->config.setUInt64(tag, Value);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- signed int 64
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, s_ii64 Value)
{
    pimpl->config.setInt64(tag, Value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write() -- float
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, float Value)
{
    pimpl->config.setDouble(tag, Value);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- double
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, double Value)
{
    pimpl->config.setDouble(tag, Value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write() -- double
//---------------------------------------------------------------------------
void  JsonProtocol::WriteDP(const std::string & tag, double Value)
{
    pimpl->config.setDouble(tag, Value);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- string
//---------------------------------------------------------------------------
void  JsonProtocol::Write(const std::string & tag, const std::string & Value)
{
    pimpl->config.setString(tag, Value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write() -- StringList
//---------------------------------------------------------------------------
void JsonProtocol::Write(const std::string &tag , StringList &ss, bool /*numbered*/)
{
   pimpl->config.setInt(tag + ".Size", ss.Count());
   for(unsigned int i = 0; i < ss.Count(); i++)
     pimpl->config.setString(tag + ".Data[" + IntToString(i) + "]", ss.At(i));
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- vector<string>
//---------------------------------------------------------------------------
void JsonProtocol::Write(const std::string &tag , const vector<string> &ss)
{
   for(unsigned int i = 0; i < ss.size(); i++)
     pimpl->config.setString(tag + ".Data[" + IntToString(i) + "]", ss[i]);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- vector<char>
//---------------------------------------------------------------------------
void JsonProtocol::Write(const std::string &tag, const CharArray &cs)
{
    for(unsigned int i = 0; i < cs.size(); i++) {
        bool b = cs[i] ? true : false;
        pimpl->config.setBool(tag + "[" + IntToString(i) + "]", b);
    }
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- vector<int>
//---------------------------------------------------------------------------
void JsonProtocol::Write(const std::string &tag, const IntArray &is)
{
    for(unsigned int i = 0; i < is.size(); i++)
         pimpl->config.setInt(tag + "[" + IntToString(i) + "]", is[i]);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write() -- vector<float>
//---------------------------------------------------------------------------
void JsonProtocol::Write(const std::string &tag, const FloatArray &fs)
{
     for(unsigned int i = 0; i < fs.size(); i++)
         pimpl->config.setDouble(tag + "[" + IntToString(i) + "]", fs[i]);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Write() -- vector<double>
//---------------------------------------------------------------------------
void JsonProtocol::Write(const std::string &tag, const DoubleArray &ds)
{
     for(unsigned int i = 0; i < ds.size(); i++)
         pimpl->config.setDouble(tag + "[" + IntToString(i) + "]", ds[i]);
}

//---------------------------------------------------------------------------
//  IniProtocol::Read() -- bool
//---------------------------------------------------------------------------
bool  JsonProtocol::Read(const std::string &tag , bool defVal)
{
     return pimpl->config.getBool(tag, defVal);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- int
//---------------------------------------------------------------------------
int  JsonProtocol::Read(const std::string &tag, int defVal)
{
     return pimpl->config.getInt(tag, defVal);
}

//---------------------------------------------------------------------------
//  IniProtocol::Read() -- unsigned int
//---------------------------------------------------------------------------
unsigned int  JsonProtocol::Read(const std::string &tag, unsigned int defVal)
{
     return pimpl->config.getUInt(tag, defVal);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- unsigned int 64
//---------------------------------------------------------------------------
ii64 JsonProtocol::Read(const std::string &tag, ii64 defVal)
{
     return pimpl->config.getUInt64(tag, defVal);
}

//---------------------------------------------------------------------------
//  IniProtocol::Read() -- signed int 64
//---------------------------------------------------------------------------
s_ii64 JsonProtocol::Read(const std::string &tag, s_ii64 defVal)
{
     return pimpl->config.getInt64(tag, defVal);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- float
//---------------------------------------------------------------------------
float  JsonProtocol::Read(const std::string &tag, float defVal)
{
     return static_cast<float>(pimpl->config.getDouble(tag, defVal));
}

//---------------------------------------------------------------------------
//  IniProtocol::Read() -- double
//---------------------------------------------------------------------------
double  JsonProtocol::Read(const std::string &tag, double defVal)
{
     return pimpl->config.getDouble(tag, defVal);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- double
//---------------------------------------------------------------------------
double  JsonProtocol::ReadDP(const std::string &tag , double defVal)
{
     return pimpl->config.getDouble(tag, defVal);
}

//---------------------------------------------------------------------------
//  IniProtocol::Read() --
//---------------------------------------------------------------------------
std::string  JsonProtocol::Read(const std::string &tag, const std::string & defVal)
{
     return pimpl->config.getString(tag, defVal);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- vector<string>
//---------------------------------------------------------------------------
void JsonProtocol::Read(const std::string &tag , vector<string> &ss)
{
    int i = 0;
    do {
        if(!pimpl->config.has(tag + "[" + IntToString(i) + "]"))
            break;
        ss.push_back(pimpl->config.getString(tag + "[" + IntToString(i++) + "]"));
    } while (true);
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- StringList
//---------------------------------------------------------------------------
int JsonProtocol::Read(const std::string &tag, StringList &ss, bool /*numbered*/)
{
    ss.Clear();

    int sz = 0;
    pimpl->config.getUInt(tag + ".Size", sz);
    int i = 0;
    for (; i < sz; i++)
        if(pimpl->config.has(tag + "[" + IntToString(i) + "]"))
            ss.Add(pimpl->config.getString(tag + ".Data[" + IntToString(i) + "]"));

    return i;
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- <vector<char>
//---------------------------------------------------------------------------
size_t JsonProtocol::Read(const std::string &tag, CharArray &cs)
{
    unsigned int i = 0;
    for (; i < cs.size(); i++)
        if(pimpl->config.has(tag + "[" + IntToString(i) + "]"))
            cs[i] = pimpl->config.getBool(tag + "[" + IntToString(i) + "]");

    return i;
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- vector<int>
//---------------------------------------------------------------------------
size_t JsonProtocol::Read(const std::string &tag, IntArray &is)
{
    unsigned int i = 0;

    for (; i < is.size(); i++)
        if(pimpl->config.has(tag + "[" + IntToString(i) + "]"))
            is[i] = pimpl->config.getInt(tag + "[" + IntToString(i) + "]");

    return i;
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- vector<float>
//---------------------------------------------------------------------------
size_t JsonProtocol::Read(const std::string &tag, FloatArray &fs)
{
    unsigned int i = 0;
    for (; i < fs.size(); i++)
        if(pimpl->config.has(tag + "[" + IntToString(i) + "]"))
            fs[i] = static_cast<float>(pimpl->config.getDouble(tag + "[" + IntToString(i) + "]"));

    return i;
}

//---------------------------------------------------------------------------
//  JsonProtocol::Read() -- vector<double>
//---------------------------------------------------------------------------
size_t JsonProtocol::Read(const std::string &tag, DoubleArray &fs)
{
    unsigned int i = 0;
    for (; i < fs.size(); i++)
        if(pimpl->config.has(tag + "[" + IntToString(i) + "]"))
            fs[i] = pimpl->config.getDouble(tag + "[" + IntToString(i) + "]");

    return i;
}

//---------------------------------------------------------------------------
//  IniProtocol::LoadFromStream() --
//---------------------------------------------------------------------------
void JsonProtocol::LoadFromStream(std::istream & stream)
{
    pimpl->config.load(stream);
}

//---------------------------------------------------------------------------
//  IniProtocol::SaveToStream() --
//---------------------------------------------------------------------------
void JsonProtocol::SaveToStream(std::ostream & stream)
{
    pimpl->config.save(stream);
}

//---------------------------------------------------------------------------
//  IniProtocol::getImpl() --
//---------------------------------------------------------------------------
Poco::Util::JSONConfiguration *JsonProtocol::getImpl()
{
    return &pimpl->config;
}

//===========================================================================
//  IniProtocol  --  Interface Class to local configuration files
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IniProtocol
//---------------------------------------------------------------------------

IniProtocol::IniProtocol(int max_len)
    :  MaxLength(max_len)
{
}

//---------------------------------------------------------------------------
//  destructor for class IniProtocol
//---------------------------------------------------------------------------

IniProtocol::~IniProtocol()
{
}

//---------------------------------------------------------------------------
//  IniProtocol::GetSettings() -- returns false if section does not exist
//---------------------------------------------------------------------------

bool IniProtocol::GetSettings(const std::string & section, StringList & list)
{
	INIFile::iterator iSection = Ini.find(section);
	if(iSection == Ini.end())
        return false;

    INISection::iterator i;
    for (i= iSection->second.begin(); i != iSection->second.end(); ++i)
        {
		std::stringstream  msg;
		msg << i->first << "=" << i->second;
		list.Add(msg.str());
        }

    return true;
}

//---------------------------------------------------------------------------
//  IniProtocol::GetValue() --
//---------------------------------------------------------------------------

std::string IniProtocol::GetValue(const std::string & section, const std::string & key,
                            const std::string & defaultval)
{
	std::string result(defaultval);

	INIFile::iterator iSection = Ini.find(section);
	if(iSection != Ini.end())
        {
        INISection::iterator apair = iSection->second.find(key);
        if(apair != iSection->second.end())
            result = apair->second;
        }

	return result;
}

//---------------------------------------------------------------------------
//  IniProtocol::RemoveSetting() -- Remove key/value pair
//---------------------------------------------------------------------------

bool IniProtocol::RemoveSetting(const std::string & section, const std::string & key)
{
	INIFile::iterator iSection = Ini.find(section);
	if(iSection == Ini.end())
        return false;

    INISection::iterator apair = iSection->second.find(key);
    if(apair != iSection->second.end())
        iSection->second.erase(apair);

    return true;
}

//---------------------------------------------------------------------------
//  IniProtocol::RemoveSection() --
//---------------------------------------------------------------------------

bool IniProtocol::RemoveSection(const std::string & section)
{
	INIFile::iterator iSection = Ini.find(section);
	if(iSection == Ini.end())
        return false;

    Ini.erase(iSection);

	return true;
}

//---------------------------------------------------------------------------
//  IniProtocol::PutSetting() --
//---------------------------------------------------------------------------

void IniProtocol::PutSetting(const std::string & section, const std::string & key,
                       const std::string & value)
{
	INIFile::iterator iniSection;
	INISection::iterator apair;

    iniSection = Ini.find(section);
	if(iniSection == Ini.end())
    	{
		// no such section?  Then add one..
		INISection newsection;
		if(key.size())
			newsection.insert(std::pair<std::string, std::string> (key, value));

		Ini.insert(std::pair<std::string, INISection> (section, newsection));
	    }
	else if(key.size())
    	{
        // found section, make sure key isn't in there already,
		// if it is, just drop and re-add
		apair = iniSection->second.find(key);
		if(apair != iniSection->second.end())
			iniSection->second.erase(apair);

		iniSection->second.insert(std::pair<std::string, std::string> (key, value));
	    }
}

//---------------------------------------------------------------------------
//  IniProtocol::LoadFromStream() -- Load settings from an input stream
//---------------------------------------------------------------------------

void IniProtocol::LoadFromStream(std::istream & stream)
{
	char  *value, *temp;
	std::string section;
 	char * buffer = new char [MaxLength];

	while(stream.good())
	{
		char * p = buffer;

		std::memset(p, 0, MaxLength);
		stream.getline(p, MaxLength);
		temp = strchr(p, '\n');
		if(temp)
			*temp = '\0';		// cut off at newline
		temp = strchr(p, '\r');
		if(temp)
			*temp = '\0';		// cut off at linefeeds
		temp = strrchr(p, ']');
		if((p[0] == '[') && temp)
			{
			// if line is like -->   [section name]
			*temp = '\0';   // chop off the trailing ']';
			section = &p[1];
			PutSetting(section, "", "");	 // start new section
			}
		else if(p[0] && (value = strchr(p, '=')))
			{
			*value++ = '\0'; // assign whatever follows = sign to value, chop at "="
			PutSetting(section.c_str(), p, value); 	// and add both sides to INISection
			}
		else if(p[0])
			PutSetting(section, p, ""); 	// must be a comment or something
	}

	delete [] buffer;
}

//---------------------------------------------------------------------------
//  IniProtocol::SaveToStream() -- Save settings text to an output stream
//---------------------------------------------------------------------------

void IniProtocol::SaveToStream(std::ostream & stream)
{
    //
    //  We can't just write the sections out anyhow since the nameless section
    //     MUST be written out first. If it isn't it will be treated as inside
    //     another section.
    //
    //  Find the nameless section and write it first
	INIFile::iterator namelessSection = Ini.find("");
	if(namelessSection != Ini.end())
        SaveSection(stream, namelessSection);

	//  Write all the other sections out in whatever order
	INIFile::iterator section= Ini.begin();
	while(section != Ini.end())
	    {
		if(section->first > "")
            {
            // if the section is NOT the nameless section, save it with section
            //    name in front
			stream << std::endl << "[" << section->first << "]" << std::endl;
            SaveSection(stream, section);
            }
		section++;
	    }

}

//---------------------------------------------------------------------------
//  IniProtocol::SaveSection() -- Write out section pointed to by iterator
//---------------------------------------------------------------------------

void IniProtocol::SaveSection(std::ostream & file, INIFile::iterator & section)
{
    INISection::iterator pair = section->second.begin();

    while(pair != section->second.end())
        {
        if(pair->second > "")
            file << pair->first << "=" << pair->second << std::endl;
        else
            file << pair->first << std::endl;
        pair++;
        }
}

//---------------------------------------------------------------------------
//  IniProtocol::DeleteKey() -- delete the specified key
//---------------------------------------------------------------------------

void  IniProtocol::DeleteKey(const std::string & key)
{
    // delete the tag(name) associated with the current key
    RemoveSetting(Section, key);
}

//---------------------------------------------------------------------------
//  IniProtocol::DeleteSection() -- delete the specified section
//---------------------------------------------------------------------------

void  IniProtocol::DeleteSection(const std::string & section)
{
    // delete the tag(name) associated with the current key
    RemoveSection(section);
}

//---------------------------------------------------------------------------
//  IniProtocol::DeleteSection() -- delete the current section
//---------------------------------------------------------------------------

void  IniProtocol::DeleteSection()
{
    DeleteSection(Section);
}

//---------------------------------------------------------------------------
//  IniProtocol::ReadValues() -- return a list of keys and values from section
//---------------------------------------------------------------------------

void  IniProtocol::ReadValues(const std::string & section, StringList & list)
{
    GetSettings(section, list);
}

//---------------------------------------------------------------------------
//  IniProtocol::ReadValues() -- return a list of keys and values from current section
//---------------------------------------------------------------------------

void  IniProtocol::ReadValues(StringList & strings)
{
    ReadValues(Section, strings);
}

//---------------------------------------------------------------------------
//  IniProtocol::ReadKeys() -- return a list of keys from specified section
//---------------------------------------------------------------------------

void  IniProtocol::ReadKeys(const std::string & section, StringList & list)
{
	INIFile::iterator iSection = Ini.find(section);
	if(iSection == Ini.end())
        return;

    INISection::iterator i;
    for (i= iSection->second.begin(); i != iSection->second.end(); ++i)
        {
		std::stringstream  msg;
		msg << i->first;
		list.Add(msg.str());
		}
}

//---------------------------------------------------------------------------
//  IniProtocol::ReadKeys() -- return a list of keys in current section
//---------------------------------------------------------------------------

void  IniProtocol::ReadKeys(StringList & strings)
{
    ReadKeys(Section, strings);
}

//---------------------------------------------------------------------------
//  IniProtocol::KeyExists() -- return true if key is present in specified section
//---------------------------------------------------------------------------

bool  IniProtocol::KeyExists(const std::string & section, const std::string & key)
{
	INIFile::iterator iSection = Ini.find(section);
	if(iSection == Ini.end())
        return false;

    INISection::iterator i = iSection->second.find(key);
    return i != iSection->second.end();
}

//---------------------------------------------------------------------------
//  IniProtocol::KeyExists() -- return true if key is present in current section
//---------------------------------------------------------------------------

bool  IniProtocol::KeyExists(const std::string & key)
{
    return KeyExists(Section, key);
}

//---------------------------------------------------------------------------
//  IniProtocol::SectionExists() -- return true if section is present
//---------------------------------------------------------------------------

bool  IniProtocol::SectionExists(const std::string & section)
{
	INIFile::iterator iSection = Ini.find(section);
	return iSection != Ini.end();
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, StringList) -- Read string list to current tag
//---------------------------------------------------------------------------

int IniProtocol::Read(const std::string & key, StringList & list, bool numbered)
{
    list.Clear();

    if (numbered)
        {
        std::string comments = GetValue("Comments", "CommentLines", "0");
        unsigned int count = StringToInt(comments);

        for (unsigned int i = 0; i < count; ++i)
            {
            std::stringstream tag;
            tag << "CommentLine" << i;
            std::string line = Read(tag.str(), std::string(""));
            list.Add(line);
            }
        }
    else
        {
        std::string line = GetValue(Section, key, std::string(""));
        list.Text(line);
        }

    return list.Count();
}

//---------------------------------------------------------------------------
//  IniProtocol::Read() --
//---------------------------------------------------------------------------

bool  IniProtocol::Read(const std::string & key, bool defValue)
{
    std::string txt = GetValue(Section, key, "");

    if (!txt.size())
        return defValue;

    std::istringstream is(txt);
    bool result;
    is >> result;

    return result;
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, int) -- Read integer from current Ini File key
//---------------------------------------------------------------------------

int  IniProtocol::Read(const std::string & key, int defValue)
{
    std::string txt = GetValue(Section, key, "");

    if (!txt.size())
        return defValue;

    // Use StringToInt(), since this will detect C style format text in
    //   conversion
    int result = StringToInt(txt);

    return result;
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, int) -- Read integer from current Ini File key
//---------------------------------------------------------------------------

unsigned int  IniProtocol::Read(const std::string & key, unsigned int defValue)
{
    return Read(key, static_cast<int>(defValue));
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, float) -- Read float from current Ini File key
//---------------------------------------------------------------------------

float  IniProtocol::Read(const std::string & key, float defValue)
{
    std::string txt = GetValue(Section, key, "");

    if (!txt.size())
        return defValue;

	std::istringstream is(txt);
    float result;
    is >> result;

    return result;
}

//---------------------------------------------------------------------------
//  IniProtocol::ReadDP() -- Read double from current Ini File key
//---------------------------------------------------------------------------

double  IniProtocol::ReadDP(const std::string & key, double defValue)
{
    std::string txt = GetValue(Section, key, "");

    if (!txt.size())
        return defValue;

	std::istringstream is(txt);
    double result;
    is >> result;

    return result;
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, double) -- Read double from current Ini File key
//---------------------------------------------------------------------------

double  IniProtocol::Read(const std::string & key, double defValue)
{
	return ReadDP(key, defValue);
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, std::string) -- Read string from current Ini File key
//---------------------------------------------------------------------------

std::string  IniProtocol::Read(const std::string & key, const std::string & defValue)
{
    std::string result = GetValue(Section, key, defValue);
    return result;
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, ii64) -- Read long integer from current Ini File key
//---------------------------------------------------------------------------

ii64 IniProtocol::Read(const std::string & key, ii64 value)
{
    union
    {
        int     i[2];
        ii64    x;
    } Value;
    Value.x = value;

    std::string KeyStr1 = key;
    std::string KeyStr2 = key + std::string("_");

    Value.i[0] = Read(KeyStr1, Value.i[0]);
    Value.i[1] = Read(KeyStr2, Value.i[1]);

    return Value.x;
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, s_ii64) -- Read signed long integer from current Ini File key
//---------------------------------------------------------------------------

s_ii64 IniProtocol::Read(const std::string & key, s_ii64 value)
{
    union
    {
        int     i[2];
        s_ii64    x;
    } Value;
    Value.x = value;

    std::string KeyStr1 = key;
    std::string KeyStr2 = key + std::string("_");

    Value.i[0] = Read(KeyStr1, Value.i[0]);
    Value.i[1] = Read(KeyStr2, Value.i[1]);

    return Value.x;
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, CharArray) -- Read character array from current section
//---------------------------------------------------------------------------

size_t IniProtocol::Read(const std::string & key, CharArray &data)
{
    int ElementSize = sizeof(char);

    std::string Blob = Read(key, std::string(""));
    if (!Blob.size())
        return 0;

	size_t Elements = Blob.size()/(ElementSize*2);
    if (data.size() < Elements)
        data.resize(Elements);
    HexToBin(Blob.data(), reinterpret_cast<char*>(&data[0]), Blob.size()/2);

    return data.size();
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, IntArray) -- Read integer array from current section
//---------------------------------------------------------------------------

size_t IniProtocol::Read(const std::string & tag, IntArray &data)
{
    int ElementSize = sizeof(int);

    std::string Blob = Read(tag, std::string(""));
    if (!Blob.size())
        return 0;

    size_t Elements = Blob.size()/(ElementSize*2);
    if (data.size() < Elements)
        data.resize(Elements);
    HexToBin(Blob.data(), reinterpret_cast<char*>(&data[0]), Blob.size()/2);

    return data.size();
}

//---------------------------------------------------------------------------
//  IniProtocol::Read(, FloatArray) -- Read float array from current section
//---------------------------------------------------------------------------

size_t IniProtocol::Read(const std::string & tag, FloatArray &data)
{
    int ElementSize = sizeof(float);

    std::string Blob = Read(tag, std::string(""));
    if (!Blob.size())
        return 0;

    size_t Elements = Blob.size()/(ElementSize*2);
    if (data.size() < Elements)
        data.resize(Elements);
    HexToBin(Blob.data(), reinterpret_cast<char*>(&data[0]), Blob.size()/2);

    return data.size();
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, std::string) -- Write string to current Ini File key
//---------------------------------------------------------------------------

void IniProtocol::Write(const std::string & key, const std::string & value)
{
    PutSetting(Section, key, value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, StringList) -- Write string list to current tag
//---------------------------------------------------------------------------

void IniProtocol::Write(const std::string & key, StringList & list, bool numbered)
{
    if (numbered)
        {
        unsigned int line = 0;
        StringList::iterator i;
        for (i = list.begin(); i != list.end(); ++i)
            {
            std::stringstream tag;
            tag << key << line++;
            PutSetting(Section, tag.str(), *i);
            }
        }
    else
        {
        PutSetting(Section, key, list.Text());
        }
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, bool) -- Write bool to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, bool value)
{
    std::stringstream  msg;
	msg << value;

	PutSetting(Section, key, msg.str());
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, int) -- Write integer to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, int value)
{
	std::stringstream  msg;
	msg << value;

	PutSetting(Section, key, msg.str());
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, int) -- Write integer to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, unsigned int value)
{
    Write(key, static_cast<int>(value));
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, float) -- Write float to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, float value)
{
    std::stringstream  msg;
	msg << value;

	PutSetting(Section, key, msg.str());
}

//---------------------------------------------------------------------------
//  IniProtocol::WriteDP() -- Write double to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::WriteDP(const std::string & key, double value)
{
    std::stringstream  msg;
	msg << std::setprecision(16) << value;

	PutSetting(Section, key, msg.str());
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, double) -- Write double to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, double value)
{
	WriteDP(key, value);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, ii64) -- Write long integer to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, ii64 value)
{
    union
    {
        int     i[2];
        ii64    x;
    } Value;
    Value.x = value;

    std::string KeyStr1 = key;
    std::string KeyStr2 = key + std::string("_");

    Write(KeyStr1, Value.i[0]);
    Write(KeyStr2, Value.i[1]);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, ii64) -- Write long integer to current Ini File key
//---------------------------------------------------------------------------

void  IniProtocol::Write(const std::string & key, s_ii64 value)
{
    union
    {
        int     i[2];
        s_ii64    x;
    } Value;
    Value.x = value;

    std::string KeyStr1 = key;
    std::string KeyStr2 = key + std::string("_");

    Write(KeyStr1, Value.i[0]);
    Write(KeyStr2, Value.i[1]);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, CharArray) -- Write character array to current section
//---------------------------------------------------------------------------

void IniProtocol::Write(const std::string & key, const CharArray &data)
{
    int ElementSize = sizeof(char);
    size_t SrcBytes = data.size()*ElementSize;
    size_t DstBytes = SrcBytes*2;

    // Dest array
    std::vector<char> array;
    array.resize(DstBytes);

    BinToHex(reinterpret_cast<const unsigned char*>(&data[0]), &array[0], SrcBytes);
    std::string Blob(&array[0], array.size());

    Write(key, Blob);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, IntArray) -- Write integer array to current section
//---------------------------------------------------------------------------

void IniProtocol::Write(const std::string & key, const IntArray &data)
{
    int ElementSize = sizeof(int);
    size_t SrcBytes = data.size()*ElementSize;
    size_t DstBytes = SrcBytes*2;

    // Dest array
    std::vector<char> array;
    array.resize(DstBytes);

    BinToHex(reinterpret_cast<const unsigned char*>(&data[0]), &array[0], SrcBytes);
    std::string Blob(&array[0], array.size());

    Write(key, Blob);
}

//---------------------------------------------------------------------------
//  IniProtocol::Write(, FloatArray) -- Write float array to current section
//---------------------------------------------------------------------------

void IniProtocol::Write(const std::string & key, const FloatArray &data)
{
    int ElementSize = sizeof(float);
    size_t SrcBytes = data.size()*ElementSize;
    size_t DstBytes = SrcBytes*2;

    // Dest array
    std::vector<char> array;
    array.resize(DstBytes);

    BinToHex(reinterpret_cast<const unsigned char*>(&data[0]), &array[0], SrcBytes);
    std::string Blob(&array[0], array.size());

    Write(key, Blob);
}

//---------------------------------------------------------------------------
//  IniProtocol::Clear() -- Clear out database
//---------------------------------------------------------------------------

void  IniProtocol::Clear()
{
    Ini.clear();
    Section = "";
}

//===========================================================================
//  IniFile  --  Read and Write file as INI format text
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IniFile
//---------------------------------------------------------------------------

IniFile::IniFile(const std::string & filename, bool create, int max_len, bool auto_save)
    : IniProtocol(max_len), Filename(filename), AutoSave(auto_save)
{
    if (!create)
		Load();
}

//---------------------------------------------------------------------------
//  destructor for class IniFile
//---------------------------------------------------------------------------

IniFile::~IniFile()
{
    if (AutoSave)
        Save();
}

//---------------------------------------------------------------------------
//  IniFile::Load() -- Load settings from file
//---------------------------------------------------------------------------

void IniFile::Load()
{
	std::fstream file(Filename.c_str(), std::ios::in);

    LoadFromStream(file);

	file.close();
}

//---------------------------------------------------------------------------
//  IniFile::Save() --
//---------------------------------------------------------------------------

void IniFile::Save()
{
	std::fstream file(Filename.c_str(), std::ios::out);
	if(!file.good())
		return;

    SaveToStream(file);

	file.close();
}

//---------------------------------------------------------------------------
//  IniFile::Update() --
//---------------------------------------------------------------------------

void  IniFile::Update()
{
    Save();
}

//===========================================================================
//  JsonFile  --  Read and Write file as INI format text
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class JsonFile
//---------------------------------------------------------------------------

JsonFile::JsonFile(const std::string & filename, bool create, int max_len, bool auto_save)
    : JsonProtocol(max_len), Filename(filename), AutoSave(auto_save)
{
    if (!create)
        Load();
}

//---------------------------------------------------------------------------
//  destructor for class JsonFile
//---------------------------------------------------------------------------

JsonFile::~JsonFile()
{
    if (AutoSave)
        Save();
}

//---------------------------------------------------------------------------
//  JsonFile::Load() -- Load settings from file
//---------------------------------------------------------------------------

void JsonFile::Load()
{
    std::ifstream file(Filename.c_str());
    try {
        if (file.is_open()) {
            LoadFromStream(file);
        }
    } catch(const Poco::JSON::JSONException &e) {
        Innovative::Trace Debug;
        Debug.Log("JSON Syntax is invalid. -- " + e.message());
    }

}

//---------------------------------------------------------------------------
//  JsonFile::Save() --
//---------------------------------------------------------------------------

void JsonFile::Save()
{
    std::ofstream file(Filename.c_str(), std::ios::trunc);
    if(!file.good())
        return;

    SaveToStream(file);
}

//---------------------------------------------------------------------------
//  JsonFile::Update() --
//---------------------------------------------------------------------------

void  JsonFile::Update()
{
    Save();
}

//===========================================================================
//  IniMemory  --  Read and Write to memory region as INI format text
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IniMemory
//---------------------------------------------------------------------------

IniMemory::IniMemory()
    : IniProtocol(100000)
{
}

//---------------------------------------------------------------------------
//  constructor for class IniMemory
//---------------------------------------------------------------------------

IniMemory::IniMemory(const std::string & text, int max_len)
    : IniProtocol(max_len)
{
    Text(text);
}

//---------------------------------------------------------------------------
//  destructor for class IniMemory
//---------------------------------------------------------------------------

IniMemory::~IniMemory()
{
}

//------------------------------------------------------------------------------
//  IniMemory::Text(const std::string &) -- Initialize INI database from string
//------------------------------------------------------------------------------

void  IniMemory::Text(const std::string & text)
{
    Clear();

    std::stringstream  mem;
    mem.str(text);

    LoadFromStream(mem);
}

//---------------------------------------------------------------------------
//  IniMemory::Text() -- return database as string
//---------------------------------------------------------------------------

std::string  IniMemory::Text()
{
    std::stringstream  mem;

    SaveToStream(mem);

    return mem.str();

}

#ifdef _MSC_VER
#pragma warning (pop)
#endif

#ifdef __BORLANDC__
    #pragma option pop
#endif

}  // namespace Innovative
