//---------------------------------------------------------------------------
//  IppCharDG_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "IppCharDG_Mb.h"
#include "IppFloatDG_Mb.h"
#include "IppShortDG_Mb.h"
#include <ipps.h>   // ipps.h is not redistributable!
#include <algorithm>

namespace Innovative
{
//---------------------------------------------------------------------------
//  IppCharDG::Zero() --
//------------------------------------------------------------------------------

void IppCharDG::Zero()
{
    char *  DataPtr =  &(*(begin()));

    ippsZero_8u( reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize() );
}

//---------------------------------------------------------------------------
//  IppCharDG::Set() --
//------------------------------------------------------------------------------

void IppCharDG::Set(char value)
{
    char *  DataPtr =  &(*(begin()));

    ippsSet_8u( value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize() );
}

//---------------------------------------------------------------------------
//  IppCharDG::Add() --
//------------------------------------------------------------------------------

void IppCharDG::Add(char value)
{
    if ( value == 0 )
        return;

    // Add by subtracting a negative
    Sub( -value );
}

//---------------------------------------------------------------------------
//  IppCharDG::Add() --
//------------------------------------------------------------------------------

void IppCharDG::Add(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsAdd_8u_ISfs(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                    reinterpret_cast<unsigned char *>(DataPtr), op_size, 0);
}

//---------------------------------------------------------------------------
//  IppCharDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppCharDG::Multiply(char value)
{
    if (value == 1)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsMulC_8u_ISfs(value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize(), 0);
}

//---------------------------------------------------------------------------
//  IppCharDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppCharDG::Multiply(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsMul_8u_ISfs(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                    reinterpret_cast<unsigned char *>(DataPtr), op_size, 0);
}

//---------------------------------------------------------------------------
//  IppCharDG::Divide() --
//------------------------------------------------------------------------------

void IppCharDG::Divide(char value)
{
    if (value == 1 )
        return;

    if (value)
        {
        //  Convert the value to a vector, then vector divide
        Buffer ValueVector( Holding<char>(size()) );
        IppCharDG ValueVectorDG(ValueVector);

        ValueVectorDG.Set(value);
        Divide(ValueVectorDG);
        }
}

//---------------------------------------------------------------------------
//  IppCharDG::Divide() --
//------------------------------------------------------------------------------
 
void IppCharDG::Divide(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsDiv_8u_ISfs(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                    reinterpret_cast<unsigned char *>(DataPtr), op_size, 0);
}

//---------------------------------------------------------------------------
//  IppCharDG::Sub() --
//------------------------------------------------------------------------------
 
void IppCharDG::Sub(char value)
{
    if (value == 0)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsSubC_8u_ISfs(value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize(), 0);
}

//---------------------------------------------------------------------------
//  IppCharDG::Sub() --
//------------------------------------------------------------------------------
 
void IppCharDG::Sub(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsSub_8u_ISfs(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                    reinterpret_cast<unsigned char *>(DataPtr), op_size, 0);
}

//---------------------------------------------------------------------------
//  IppCharDG::Or() --
//------------------------------------------------------------------------------
 
void IppCharDG::Or(char value)
{
    if (value == 0)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsOrC_8u_I( value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppCharDG::Or() --
//------------------------------------------------------------------------------

void IppCharDG::Or(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsOr_8u_I(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                reinterpret_cast<unsigned char *>(DataPtr), op_size);
}

//---------------------------------------------------------------------------
//  IppCharDG::And() --
//------------------------------------------------------------------------------
 
void IppCharDG::And(char value)
{
    if ((unsigned char)value == 0xFF)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsAndC_8u_I( value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppCharDG::And() --
//------------------------------------------------------------------------------
 
void IppCharDG::And(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsAnd_8u_I(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                 reinterpret_cast<unsigned char *>(DataPtr), op_size);
}

//---------------------------------------------------------------------------
//  IppCharDG::Xor() --
//------------------------------------------------------------------------------
 
void IppCharDG::Xor(char value)
{
    if (value == 0)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsXorC_8u_I( value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppCharDG::Xor() --
//------------------------------------------------------------------------------
 
void IppCharDG::Xor(const IppCharDG & rhs)
{
    char *  DataPtr =  &(*(begin()));
    const char * RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<unsigned int>(std::min(size(), rhs.size()));

    ippsXor_8u_I(reinterpret_cast<const unsigned char *>(RhsDataPtr),
                 reinterpret_cast<unsigned char *>(DataPtr), op_size);
}

//---------------------------------------------------------------------------
//  IppCharDG::LShift() --
//------------------------------------------------------------------------------
 
void IppCharDG::LShift(int Value)
{
    if (Value == 0)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsLShiftC_8u_I(Value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppCharDG::RShift() --
//------------------------------------------------------------------------------
 
void IppCharDG::RShift(int Value)
{
    if (Value == 0)
        return;

    char *  DataPtr =  &(*(begin()));

    ippsRShiftC_8u_I(Value, reinterpret_cast<unsigned char *>(DataPtr), TruncatedSize());
}

} // end namespace Innovative
