//---------------------------------------------------------------------------
//  IppCharDG_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef IppCharDG_MbH
#define IppCharDG_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class IppShortDG;
class IppFloatDG;

//==============================================================================
//  CLASS IppCharDG -- Datagram wrapping a buffer; provides access as 'char'
//==============================================================================
//  Also includes IPP fast analysis methods

class IppCharDG : public AccessDatagram<char>
{
public:
    typedef AccessDatagram<char> BaseClass;
    //
    //  Member Functions
    IppCharDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IppCharDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
    const Buffer & WrappedBuffer() const
        {  return FBuffer;  }

    void Zero();
    void Set( char value );

    void Add( char value );
    void Add( const IppCharDG & rhs);

    void Multiply( char value );
    void Multiply( const IppCharDG & rhs);

    void Divide( char value );
    void Divide( const IppCharDG & rhs);

    void Sub( char value );
    void Sub( const IppCharDG & rhs);

    void Or( char value );
    void Or( const IppCharDG & rhs);

    void And( char value );
    void And( const IppCharDG & rhs );

    void Xor( char value );
    void Xor( const IppCharDG &  Src );

    void LShift( int Value );
    void RShift( int Value );

    IppCharDG & operator += (const IppCharDG & rhs)     { Add( rhs ); return *this; }
    IppCharDG & operator += (char rhs)                  { Add( rhs ); return *this; }
    IppCharDG & operator -= (const IppCharDG & rhs)     { Sub( rhs ); return *this; }
    IppCharDG & operator -= (char rhs)                  { Sub( rhs ); return *this; }
    IppCharDG & operator *= (const IppCharDG & rhs)     { Multiply( rhs ); return *this; }
    IppCharDG & operator *= (char rhs)                  { Multiply( rhs ); return *this; }
    IppCharDG & operator /= (const IppCharDG & rhs)     { Divide( rhs ); return *this; }
    IppCharDG & operator /= (char rhs)                  { Divide( rhs ); return *this; }
    IppCharDG & operator |= (const IppCharDG & rhs)     { Or( rhs ); return *this; }
    IppCharDG & operator |= (char rhs)                  { Or( rhs ); return *this; }
    IppCharDG & operator &= (const IppCharDG & rhs)     { And( rhs ); return *this; }
    IppCharDG & operator &= (char rhs)                  { And( rhs ); return *this; }
    IppCharDG & operator ^= (const IppCharDG & rhs)     { Xor( rhs ); return *this; }
    IppCharDG & operator ^= (char rhs)                  { Xor( rhs ); return *this; }
    IppCharDG & operator <<= (int rhs)                  { LShift( rhs ); return *this; }
    IppCharDG & operator >>= (int rhs)                  { RShift( rhs ); return *this; }

private:
    Buffer & FBuffer;

    unsigned int    TruncatedSize() const
        {  return static_cast<unsigned int>(Size);  }

};

inline Buffer operator + (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG += rhs; return tmp; }
inline Buffer operator + (const char value, const IppCharDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator + (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator - (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG -= rhs; return tmp; }
inline Buffer operator - (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG -= value; return tmp; }
inline Buffer operator - (const IppCharDG & lhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG.Multiply(-1); return tmp; }
inline Buffer operator * (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG *= rhs; return tmp; }
inline Buffer operator * (const char value, const IppCharDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator * (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator / (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG /= rhs; return tmp; }
inline Buffer operator / (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG tmpDG(tmp); tmpDG /= value; return tmp; }
inline Buffer operator | (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG |= rhs; return tmp; }
inline Buffer operator | (const char value, const IppCharDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG |= value; return tmp; }
inline Buffer operator | (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG |= value; return tmp; }
inline Buffer operator & (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG &= rhs; return tmp; }
inline Buffer operator & (const char value, const IppCharDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG &= value; return tmp; }
inline Buffer operator & (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG &= value; return tmp; }
inline Buffer operator ^ (const IppCharDG & lhs, const IppCharDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG ^= rhs;  return tmp; }
inline Buffer operator ^ (const char value, const IppCharDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG ^= value; return tmp; }
inline Buffer operator ^ (const IppCharDG & lhs, const char value)
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG ^= value; return tmp; }
inline Buffer operator << (const IppCharDG & lhs, int value )
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG <<= value;  return tmp; }
inline Buffer operator >> (const IppCharDG & lhs, int value )
    { Buffer tmp(lhs.WrappedBuffer()); IppCharDG  tmpDG(tmp); tmpDG >>= value;  return tmp; }

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif

