//---------------------------------------------------------------------------
//  IppComplexDG_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef IppComplexDG_MbH
#define IppComplexDG_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>
#include <IppFloatDG_Mb.h>
#include <complex>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS Complex  -- alias of real complex class
//==============================================================================
typedef std::complex<float> Complex;

//==============================================================================
//  CLASS IppComplexDG -- Datagram for a Buffer; provides access as 'Complex'
//==============================================================================
//  Also includes IPP fast analysis methods

class IppComplexDG : public AccessDatagram<Complex>
{
public:
    typedef AccessDatagram<Complex> BaseClass;
    //
    //  Member Functions
    IppComplexDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IppComplexDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    IppComplexDG & operator = (const IppComplexDG & other);

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
	const Buffer & WrappedBuffer() const
        {  return FBuffer;  }
    //
    //  Operations
    void       Zero();

	void       Set( const IppFloatDG & RealPart );
	void       Set( const IppFloatDG & RealPart, const IppFloatDG & ImagPart );
    void       Set( float Real, float Imaginary ) { Set(Complex(Real, Imaginary)); }
    void       Set( Complex value );

    void       Add( Complex value );
    void       Add( float Real, float Imaginary ) { Add(Complex(Real, Imaginary)); }
    void       Add( const IppComplexDG & rhs );

    void       Multiply( Complex value );
    void       Multiply( float Real, float Imaginary ) { Multiply(Complex(Real, Imaginary)); }
    void       Multiply( const IppComplexDG & rhs );

    void       Divide( Complex value );
    void       Divide( float Real, float Imaginary ) { Divide(Complex(Real, Imaginary)); }
    void       Divide( const IppComplexDG & rhs );
    void       Conjugate(const IppComplexDG & rhs);
    void       ConjugateFlip(const IppComplexDG & rhs);
    void       Flip(const IppComplexDG & rhs);

    void       Sub( Complex value );
    void       Sub( float Real, float Imaginary ) { Sub(Complex(Real, Imaginary)); }
    void       Sub( const IppComplexDG & rhs );

    bool       Window(IppComplexDG & rhs, WindowTypes type, float alpha=0.1f);

    IppComplexDG & operator += (const IppComplexDG & rhs)   { Add( rhs ); return *this; }
    IppComplexDG & operator += (const Complex & rhs)        { Add( rhs ); return *this; }
    IppComplexDG & operator *= (const IppComplexDG & rhs)   { Multiply( rhs ); return *this; }
    IppComplexDG & operator *= (const Complex &rhs)         { Multiply( rhs ); return *this; }
    IppComplexDG & operator /= (const IppComplexDG & rhs)   { Divide( rhs ); return *this; }
    IppComplexDG & operator /= (const Complex &rhs)         { Divide( rhs ); return *this; }
    IppComplexDG & operator -= (const IppComplexDG & rhs)   { Sub( rhs ); return *this; }
    IppComplexDG & operator -= (const Complex &rhs)         { Sub( rhs ); return *this; }

        
private:
    Buffer & FBuffer;

    unsigned int    TruncatedSize() const
        {  return static_cast<unsigned int>(Size);  }
};


inline Buffer operator + (const IppComplexDG & lhs, const IppComplexDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG += rhs; return tmp; }
inline Buffer operator + (const Complex & value, const IppComplexDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator + (const IppComplexDG & lhs, const Complex & value)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator - (const IppComplexDG & lhs, const IppComplexDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG -= rhs; return tmp; }
inline Buffer operator - (const IppComplexDG & lhs, const Complex & value)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG -= value; return tmp; }
inline Buffer operator - (const IppComplexDG & lhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG.Multiply(-1.0, -1.0); return tmp; }
inline Buffer operator * (const IppComplexDG & lhs, const IppComplexDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG *= rhs; return tmp; }
inline Buffer operator * (const Complex & value, const IppComplexDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator * (const IppComplexDG & lhs, const Complex & value)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator / (const IppComplexDG & lhs, const IppComplexDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG /= rhs; return tmp; }
inline Buffer operator / (const IppComplexDG & lhs, const Complex & value)
    { Buffer tmp(lhs.WrappedBuffer()); IppComplexDG tmpDG(tmp); tmpDG /= value; return tmp; }

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
