//---------------------------------------------------------------------------
//  IppDoubleDG_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "IppDoubleDG_Mb.h"
#include <ipps.h>   // ipps.h is not redistributable!
#include <algorithm>

namespace Innovative
{

//---------------------------------------------------------------------------
//  IppDoubleDG::operator =() --
//------------------------------------------------------------------------------

IppDoubleDG & IppDoubleDG::operator = (const IppDoubleDG & rhs)
{
    if (this != &rhs) // protect against invalid self-assignment
       {
        double *  DataPtr = &(*(begin()));
        const double*    RhsDataPtr = &(*(rhs.begin()));
        ippsCopy_64f(RhsDataPtr, DataPtr, TruncatedSize());
        }

    // by convention, always return *this
    return *this;
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Zero() --
//------------------------------------------------------------------------------

void IppDoubleDG::Zero()
{
    double *  DataPtr = &(*(begin()));

    ippsZero_64f(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Set() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Set(double value)
{
    double *  DataPtr = &(*(begin()));

    ippsSet_64f(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Add() --
//------------------------------------------------------------------------------

void IppDoubleDG::Add(double value)
{
    if (value == 0)
        return;

    double *  DataPtr = &(*(begin()));

    ippsAddC_64f_I(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Add() --
//------------------------------------------------------------------------------

void IppDoubleDG::Add(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             add_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsAdd_64f_I(RhsDataPtr, DataPtr, add_size);
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Multiply(double value)
{
    if (value == 1.0)
        return;

    double*  DataPtr = &(*(begin()));

    ippsMulC_64f_I(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Multiply(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsMul_64f_I(RhsDataPtr, DataPtr, op_size);
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Divide() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Divide(double value)
{
    if (value == 1.0)
        return;

    if (value)
        {
        double*  DataPtr = &(*(begin()));

        ippsDivC_64f_I(value, DataPtr, TruncatedSize());
        }
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Divide() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Divide(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsDiv_64f_I(RhsDataPtr, DataPtr, op_size);
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Flip() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Flip(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsFlip_64f(RhsDataPtr, DataPtr, op_size);
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Sub() --
//------------------------------------------------------------------------------
 
void IppDoubleDG::Sub(double value)
{
    if (value == 0)
        return;

    double*  DataPtr = &(*(begin()));

    ippsSubC_64f_I(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Sub() --
//------------------------------------------------------------------------------

void IppDoubleDG::Sub(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSub_64f_I(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Normalize() --  Block = (Block - Offset) / Factor
//------------------------------------------------------------------------------

void  IppDoubleDG::Normalize(double Offset, double Factor)
{
    double *         DataPtr = &(*(begin()));
    const double *   CDataPtr = &(*(begin()));
    ippsNormalize_64f(CDataPtr, DataPtr, TruncatedSize(), Offset, Factor);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Window() -- Apply window to vector
//------------------------------------------------------------------------------

bool IppDoubleDG::Window(IppDoubleDG rhs, WindowTypes type, double alpha)
{
    double * dst = &(*(begin()));
    const double * src = &(*(rhs.begin()));
	int sz = static_cast<int>(size());

    switch(type)
        {
        default:
        case wtRect:
            return ippsCopy_64f(src, dst, sz) == ippStsNoErr;
        case wtBartlett:
            return ippsWinBartlett_64f(src, dst, sz) == ippStsNoErr;
        case wtBlackman:
            return ippsWinBlackman_64f(src, dst, sz, alpha) == ippStsNoErr;
		case wtBlackmanOpt:
			return ippsWinBlackmanOpt_64f(src, dst, sz) == ippStsNoErr;
		case wtHamming:
			return ippsWinHamming_64f(src, dst, sz) == ippStsNoErr;
		case wtHanning:
			return ippsWinHann_64f(src, dst, sz) == ippStsNoErr;
		case wtKaiser:
			return ippsWinKaiser_64f(src, dst, sz, alpha) == ippStsNoErr;
		}
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Dot() -- Computes the dot product of vector and Src
//------------------------------------------------------------------------------

double  IppDoubleDG::Dot(const IppDoubleDG & rhs)
{
    const double *   DataPtr = &(*(begin()));
    const double *   RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));
    double result;
    ippsDotProd_64f(RhsDataPtr, DataPtr, op_size, &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Threshold() -- Performs the threshold operation on the
//                               elements of vector in-place by
//                               limiting the element values by thresh.
//------------------------------------------------------------------------------

void  IppDoubleDG::Threshold(double Threshold, IIThreshType /*Type*/)
{
    double * DataPtr = &(*(begin()));
    IppCmpOp type = (Threshold == ttGreaterThan) ? ippCmpGreater : ippCmpLess;
    ippsThreshold_64f_I(DataPtr, TruncatedSize(), Threshold, type);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Abs() -- Computes the absolute values of vector
//                         elements in-place.
//------------------------------------------------------------------------------

void  IppDoubleDG::Abs()
{
    double * DataPtr = &(*(begin()));
    ippsAbs_64f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Abs() --  Computes the absolute values of vector
//------------------------------------------------------------------------------

void IppDoubleDG::Abs(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsAbs_64f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Sqr() -- Computes a square of each element of vector
//------------------------------------------------------------------------------

void  IppDoubleDG::Sqr()
{
    double * DataPtr = &(*(begin()));
    ippsSqr_64f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Sqr() --  Computes a square of each element of vector
//------------------------------------------------------------------------------

void IppDoubleDG::Sqr(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSqr_64f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Sqrt() -- Computes a square root of each element of vector
//------------------------------------------------------------------------------

void  IppDoubleDG::Sqrt()
{
    double * DataPtr = &(*(begin()));
    ippsSqrt_64f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Sqrt() --  Computes a square root of each element of vector
//------------------------------------------------------------------------------

void IppDoubleDG::Sqrt(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSqrt_64f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Exp() -- Computes e to the power of each vector element
//------------------------------------------------------------------------------

void  IppDoubleDG::Exp()
{
    double * DataPtr = &(*(begin()));
    ippsExp_64f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Exp() --  Computes e to the power of each vector element
//------------------------------------------------------------------------------

void IppDoubleDG::Exp(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsExp_64f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Ln() -- Computes the ln of each vector element
//------------------------------------------------------------------------------

void  IppDoubleDG::Ln()
{
    double * DataPtr = &(*(begin()));
    ippsLn_64f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppDoubleDG::Ln() --  Computes the ln of each vector element
//------------------------------------------------------------------------------

void IppDoubleDG::Ln(const IppDoubleDG & rhs)
{
    double *         DataPtr = &(*(begin()));
    const double*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsLn_64f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Max() -- Returns the maximum value of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::Max()
{
    double result;
    const double *    CDataPtr = &(*(begin()));
    ippsMax_64f(CDataPtr, TruncatedSize(), &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Max() -- Returns the index to and maximum value of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::Max(int & index)
{
    double result;
    const double *    CDataPtr = &(*(begin()));
    ippsMaxIndx_64f(CDataPtr, TruncatedSize(), &result, &index);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Min() -- Returns the minimum value of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::Min()
{
    double result;
    const double *    CDataPtr = &(*(begin()));
    ippsMin_64f(CDataPtr, TruncatedSize(), &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Min() -- Returns index to and minimum value of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::Min(int & index)
{
    double result;
    const double *    CDataPtr = &(*(begin()));
    ippsMinIndx_64f(CDataPtr, TruncatedSize(), &result, &index);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Mean() -- Computes the mean value of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::Mean()
{
    double result;
    const double *    CDataPtr = &(*(begin()));
    ippsMean_64f(CDataPtr, TruncatedSize(), &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::SDev() -- Computes the standard deviation value of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::SDev()
{
    double result;
    const double *    CDataPtr = &(*(begin()));
    ippsStdDev_64f(CDataPtr, TruncatedSize(), &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppDoubleDG::Norm() -- Computes the L1, L2 or C Norm of vector
//------------------------------------------------------------------------------

double  IppDoubleDG::Norm(IINormType type)
{
    double result;
	const double *    CDataPtr = &(*(begin()));
    switch(type)
        {
        case ntL1Norm:
            ippsNorm_L1_64f(CDataPtr, TruncatedSize(), &result);
        case ntL2Norm:
            ippsNorm_L2_64f(CDataPtr, TruncatedSize(), &result);
        case ntCNorm:
            ippsNorm_Inf_64f(CDataPtr, TruncatedSize(), &result);
        }

    return result;
}




} // end namespace Innovative
