//---------------------------------------------------------------------------
//  IppDoubleDG_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef IppDoubleDG_MbH
#define IppDoubleDG_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>
#include <WindowType_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class IppCharDG;
class IppComplexDG;

//==============================================================================
//  CLASS IppDoubleDG -- Datagram wrapping a buffer; provides access as 'float'
//==============================================================================
//  Also includes IPP fast analysis methods

class IppDoubleDG : public AccessDatagram<double>
{
public:
    enum IIThreshType { ttGreaterThan, ttLessThan };
    enum IINormType { ntL1Norm, ntL2Norm, ntCNorm };

	typedef AccessDatagram<double> BaseClass;
    //
    //  Member Functions
    IppDoubleDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IppDoubleDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    IppDoubleDG & operator = (const IppDoubleDG & other);

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
    const Buffer & WrappedBuffer() const
        {  return FBuffer;  }

    //  Operations
    void    Zero();
    void    Set( double value );
    void    Add( double value );
    void    Add(const IppDoubleDG & rhs );
    void    Multiply( double value );
    void    Multiply(const IppDoubleDG & rhs );
    void    Divide( double value );
    void    Divide(const IppDoubleDG & rhs );
    void    Flip(const IppDoubleDG & rhs);
    void    Sub( double value );
    void    Sub(const IppDoubleDG &rhs );
	bool    Window(IppDoubleDG rhs, WindowTypes type, double alpha);
	void    Normalize(double Offset, double Factor);
	double  Dot(const IppDoubleDG & rhs);
    void    Threshold(double Threshold, IIThreshType Type);
    void    Abs();
    void    Abs(const IppDoubleDG &rhs );
    void    Sqr();
    void    Sqr(const IppDoubleDG &rhs );
    void    Sqrt();
    void    Sqrt(const IppDoubleDG &rhs );
    void    Exp();
    void    Exp(const IppDoubleDG &rhs );
    void    Ln();
    void    Ln(const IppDoubleDG &rhs );
    double   Max();
    double   Max(int & index);
    double   Min();
    double   Min(int & index);
    double   Mean();
    double   SDev();
    double   Norm(IINormType type);

    //  Operators
    IppDoubleDG & operator += (const IppDoubleDG & rhs) { Add( rhs ); return *this; }
    IppDoubleDG & operator += (double rhs)              { Add( rhs ); return *this; }
    IppDoubleDG & operator -= (const IppDoubleDG & rhs) { Sub( rhs ); return *this; }
    IppDoubleDG & operator -= (double rhs)              { Sub( rhs ); return *this; }
    IppDoubleDG & operator *= (const IppDoubleDG & rhs) { Multiply( rhs ); return *this; }
    IppDoubleDG & operator *= (double rhs)              { Multiply( rhs ); return *this; }
    IppDoubleDG & operator /= (const IppDoubleDG & rhs) { Divide( rhs ); return *this; }
    IppDoubleDG & operator /= (double rhs)              { Divide( rhs ); return *this; }

private:
    Buffer & FBuffer;

    unsigned int    TruncatedSize() const
        {  return static_cast<unsigned int>(Size);  }
};

inline Buffer operator + (const IppDoubleDG& lhs, const IppDoubleDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG += rhs; return tmp; }
inline Buffer operator + (const double value, const IppDoubleDG& rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator + (const IppDoubleDG& lhs, const double value)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator - (const IppDoubleDG& lhs, const IppDoubleDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG -= rhs; return tmp; }
inline Buffer operator - (const IppDoubleDG& lhs, const double value)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG -= value; return tmp; }

inline Buffer operator - (const IppDoubleDG& lhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG.Multiply(-1.0); return tmp; }
inline Buffer operator * (const IppDoubleDG& lhs, const IppDoubleDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG *= rhs; return tmp; }
inline Buffer operator * (const double value, const IppDoubleDG& rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator * (const IppDoubleDG& lhs, const double value)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG *= value; return tmp; }

inline Buffer operator / (const IppDoubleDG& lhs, const IppDoubleDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG /= rhs; return tmp; }
inline Buffer operator / (const IppDoubleDG& lhs, const double value)
	{ Buffer tmp(lhs.WrappedBuffer()); IppDoubleDG tmpDG(tmp); tmpDG /= value; return tmp; }


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
