//---------------------------------------------------------------------------
//  IppFloatDG_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "IppFloatDG_Mb.h"
#include "IppComplexDG_Mb.h"
#include <ipps.h>   // ipps.h is not redistributable!
#include <algorithm>

namespace Innovative
{

//---------------------------------------------------------------------------
//  IppFloatDG::operator =() --
//------------------------------------------------------------------------------

IppFloatDG & IppFloatDG::operator = (const IppFloatDG & rhs)
{
	if (this != &rhs) // protect against invalid self-assignment
       {
        float *  DataPtr = &(*(begin()));
        const float*    RhsDataPtr = &(*(rhs.begin()));
        ippsCopy_32f(RhsDataPtr, DataPtr, TruncatedSize());
        }

    // by convention, always return *this
    return *this;
}

//---------------------------------------------------------------------------
//  IppFloatDG::Zero() --
//------------------------------------------------------------------------------

void IppFloatDG::Zero()
{
    float *  DataPtr = &(*(begin()));

    ippsZero_32f(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Set() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Set(float value)
{
    float *  DataPtr = &(*(begin()));

    ippsSet_32f(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Add() --
//------------------------------------------------------------------------------

void IppFloatDG::Add(float value)
{
    if (value == 0)
        return;

    float *  DataPtr = &(*(begin()));

    ippsAddC_32f_I(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Add() --
//------------------------------------------------------------------------------

void IppFloatDG::Add(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             add_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsAdd_32f_I(RhsDataPtr, DataPtr, add_size);
}

//---------------------------------------------------------------------------
//  IppFloatDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Multiply(float value)
{
    if (value == 1.0)
        return;

    float*  DataPtr = &(*(begin()));

    ippsMulC_32f_I(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Multiply(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsMul_32f_I(RhsDataPtr, DataPtr, op_size);
}

//---------------------------------------------------------------------------
//  IppFloatDG::Divide() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Divide(float value)
{
    if (value == 1.0)
        return;

    if (value)
        {
        float*  DataPtr = &(*(begin()));

        ippsDivC_32f_I(value, DataPtr, TruncatedSize());
        }
}

//---------------------------------------------------------------------------
//  IppFloatDG::Divide() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Divide(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsDiv_32f_I(RhsDataPtr, DataPtr, op_size);
}

//---------------------------------------------------------------------------
//  IppFloatDG::Flip() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Flip(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsFlip_32f(RhsDataPtr, DataPtr, op_size);
}

//---------------------------------------------------------------------------
//  IppFloatDG::Sub() --
//------------------------------------------------------------------------------
 
void IppFloatDG::Sub(float value)
{
    if (value == 0)
        return;

    float*  DataPtr = &(*(begin()));

    ippsSubC_32f_I(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Sub() --
//------------------------------------------------------------------------------

void IppFloatDG::Sub(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSub_32f_I(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Normalize() --  Block = (Block - Offset) / Factor
//------------------------------------------------------------------------------

void  IppFloatDG::Normalize(float Offset, float Factor)
{
    float *         DataPtr = &(*(begin()));
    const float *   CDataPtr = &(*(begin()));
    ippsNormalize_32f(CDataPtr, DataPtr, TruncatedSize(), Offset, Factor);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Window() -- Apply window to vector
//------------------------------------------------------------------------------

bool IppFloatDG::Window(IppFloatDG rhs, WindowTypes type, float alpha)
{
    float * dst = &(*(begin()));
    const float * src = &(*(rhs.begin()));
	int sz = static_cast<int>(size());

    switch(type)
        {
        default:
        case wtRect:
            return ippsCopy_32f(src, dst, sz) == ippStsNoErr;
        case wtBartlett:
            return ippsWinBartlett_32f(src, dst, sz) == ippStsNoErr;
        case wtBlackman:
            return ippsWinBlackman_32f(src, dst, sz, alpha) == ippStsNoErr;
		case wtBlackmanOpt:
			return ippsWinBlackmanOpt_32f(src, dst, sz) == ippStsNoErr;
		case wtHamming:
			return ippsWinHamming_32f(src, dst, sz) == ippStsNoErr;
		case wtHanning:
			return ippsWinHann_32f(src, dst, sz) == ippStsNoErr;
		case wtKaiser:
			return ippsWinKaiser_32f(src, dst, sz, alpha) == ippStsNoErr;
		}
}

//------------------------------------------------------------------------------
//  IppFloatDG::Magnitude() -- Computes the magnitude of src vector
//------------------------------------------------------------------------------

bool IppFloatDG::Magnitude(const IppComplexDG & rhs)
{
    float *   dst = &(*(begin()));
    const Ipp32fc * src = reinterpret_cast<const Ipp32fc *>(&(*(rhs.begin())));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

	IppStatus status = ippsMagnitude_32fc(src, dst, op_size);
	return (status == ippStsNoErr);
}

//------------------------------------------------------------------------------
//  IppFloatDG::PowerSpectrum() -- Compute power spectrum of complex vector.
//------------------------------------------------------------------------------

bool IppFloatDG::PowerSpectrum(const IppComplexDG & rhs)
{
	float *   dst = &(*(begin()));
	const Ipp32fc * src = reinterpret_cast<const Ipp32fc*>(&(*(rhs.begin())));
	int op_size = static_cast<int>(std::min(size(), rhs.size()));

	IppStatus status = ippsPowerSpectr_32fc(src, dst, op_size);
	return (status == ippStsNoErr);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Phase() -- Compute phase angles of elements of complex vector.
//------------------------------------------------------------------------------

bool IppFloatDG::Phase(const IppComplexDG & rhs)
{
	float *   dst = &(*(begin()));
	const Ipp32fc * src = reinterpret_cast<const Ipp32fc*>(&(*(rhs.begin())));
	int op_size = static_cast<int>(std::min(size(), rhs.size()));
/*
	// Work-around for apparent bug in 64-bit version of ippsPhase_32fc
	for (int i = 0; i < op_size; ++i)
		dst[i] = src[i].im ? atan2(src[i].re, src[i].im) : 0;
	return true;
*/
	// 32-bit compiler
	IppStatus status = ippsPhase_32fc(src, dst, op_size);
	return (status == ippStsNoErr);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Real() -- Extracts the real component of the src vector
//------------------------------------------------------------------------------

bool IppFloatDG::Real(const IppComplexDG & rhs)
{
    float *   dst = &(*(begin()));
    const Ipp32fc * src = reinterpret_cast<const Ipp32fc *>(&(*(rhs.begin())));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    return ippsReal_32fc(src, dst, op_size) == ippStsNoErr;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Imaginary() -- Extracts the imaginary component of the src vector
//------------------------------------------------------------------------------

bool IppFloatDG::Imaginary(const IppComplexDG & rhs)
{
    float *   dst = &(*(begin()));
    const Ipp32fc * src = reinterpret_cast<const Ipp32fc *>(&(*(rhs.begin())));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    return ippsImag_32fc(src, dst, op_size) == ippStsNoErr;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Dot() -- Computes the dot product of vector and Src
//------------------------------------------------------------------------------

float  IppFloatDG::Dot(const IppFloatDG & rhs)
{
    const float *   DataPtr = &(*(begin()));
    const float *   RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));
    float result;
    ippsDotProd_32f(RhsDataPtr, DataPtr, op_size, &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Threshold() -- Performs the threshold operation on the
//                               elements of vector in-place by
//                               limiting the element values by thresh.
//------------------------------------------------------------------------------

void  IppFloatDG::Threshold(float Threshold, IIThreshType /*Type*/)
{
    float * DataPtr = &(*(begin()));
    IppCmpOp type = (Threshold == ttGreaterThan) ? ippCmpGreater : ippCmpLess;
    ippsThreshold_32f_I(DataPtr, TruncatedSize(), Threshold, type);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Abs() -- Computes the absolute values of vector
//                         elements in-place.
//------------------------------------------------------------------------------

void  IppFloatDG::Abs()
{
    float * DataPtr = &(*(begin()));
    ippsAbs_32f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Abs() --  Computes the absolute values of vector
//------------------------------------------------------------------------------

void IppFloatDG::Abs(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsAbs_32f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Sqr() -- Computes a square of each element of vector
//------------------------------------------------------------------------------

void  IppFloatDG::Sqr()
{
    float * DataPtr = &(*(begin()));
    ippsSqr_32f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Sqr() --  Computes a square of each element of vector
//------------------------------------------------------------------------------

void IppFloatDG::Sqr(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSqr_32f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Sqrt() -- Computes a square root of each element of vector
//------------------------------------------------------------------------------

void  IppFloatDG::Sqrt()
{
    float * DataPtr = &(*(begin()));
    ippsSqrt_32f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Sqrt() --  Computes a square root of each element of vector
//------------------------------------------------------------------------------

void IppFloatDG::Sqrt(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSqrt_32f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Exp() -- Computes e to the power of each vector element
//------------------------------------------------------------------------------

void  IppFloatDG::Exp()
{
    float * DataPtr = &(*(begin()));
    ippsExp_32f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Exp() --  Computes e to the power of each vector element
//------------------------------------------------------------------------------

void IppFloatDG::Exp(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsExp_32f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Ln() -- Computes the ln of each vector element
//------------------------------------------------------------------------------

void  IppFloatDG::Ln()
{
    float * DataPtr = &(*(begin()));
    ippsLn_32f_I(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppFloatDG::Ln() --  Computes the ln of each vector element
//------------------------------------------------------------------------------

void IppFloatDG::Ln(const IppFloatDG & rhs)
{
    float *         DataPtr = &(*(begin()));
    const float*    RhsDataPtr = &(*(rhs.begin()));
    int             op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsLn_32f(RhsDataPtr, DataPtr, op_size);
}

//------------------------------------------------------------------------------
//  IppFloatDG::Max() -- Returns the maximum value of vector
//------------------------------------------------------------------------------

float  IppFloatDG::Max()
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    ippsMax_32f(CDataPtr, TruncatedSize(), &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Max() -- Returns the index to and maximum value of vector
//------------------------------------------------------------------------------

float  IppFloatDG::Max(int & index)
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    ippsMaxIndx_32f(CDataPtr, TruncatedSize(), &result, &index);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Min() -- Returns the minimum value of vector
//------------------------------------------------------------------------------

float  IppFloatDG::Min()
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    ippsMin_32f(CDataPtr, TruncatedSize(), &result);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Min() -- Returns index to and minimum value of vector
//------------------------------------------------------------------------------

float  IppFloatDG::Min(int & index)
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    ippsMinIndx_32f(CDataPtr, TruncatedSize(), &result, &index);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Mean() -- Computes the mean value of vector
//------------------------------------------------------------------------------

float  IppFloatDG::Mean()
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    ippsMean_32f(CDataPtr, TruncatedSize(), &result, ippAlgHintAccurate);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::SDev() -- Computes the standard deviation value of vector
//------------------------------------------------------------------------------

float  IppFloatDG::SDev()
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    ippsStdDev_32f(CDataPtr, TruncatedSize(), &result, ippAlgHintAccurate);

    return result;
}

//------------------------------------------------------------------------------
//  IppFloatDG::Norm() -- Computes the L1, L2 or C Norm of vector
//------------------------------------------------------------------------------

float  IppFloatDG::Norm(IINormType type)
{
    float result;
    const float *    CDataPtr = &(*(begin()));
    switch(type)
        {
        case ntL1Norm:
            ippsNorm_L1_32f(CDataPtr, TruncatedSize(), &result);
        case ntL2Norm:
            ippsNorm_L2_32f(CDataPtr, TruncatedSize(), &result);
        case ntCNorm:
            ippsNorm_Inf_32f(CDataPtr, TruncatedSize(), &result);
        }

    return result;
}




} // end namespace Innovative
