//---------------------------------------------------------------------------
//  IppFloatDG_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef IppFloatDG_MbH
#define IppFloatDG_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>
#include <WindowType_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class IppComplexDG;

//==============================================================================
//  CLASS IppFloatDG -- Datagram wrapping a buffer; provides access as 'float'
//==============================================================================
//  Also includes IPP fast analysis methods

class IppFloatDG : public AccessDatagram<float>
{
public:
    enum IIThreshType { ttGreaterThan, ttLessThan };
    enum IINormType { ntL1Norm, ntL2Norm, ntCNorm };

    typedef AccessDatagram<float> BaseClass;
    //
    //  Member Functions
    IppFloatDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IppFloatDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    IppFloatDG & operator = (const IppFloatDG & other);

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
    const Buffer & WrappedBuffer() const
        {  return FBuffer;  }

    //  Operations
    void    Zero();
    void    Set( float value );
    void    Add( float value );
    void    Add(const IppFloatDG & rhs );
    void    Multiply( float value );
    void    Multiply(const IppFloatDG & rhs );
    void    Divide( float value );
    void    Divide(const IppFloatDG & rhs );
    void    Flip(const IppFloatDG & rhs);
    void    Sub( float value );
    void    Sub(const IppFloatDG &rhs );
    bool    Window(IppFloatDG rhs, WindowTypes type, float alpha);
    bool    Magnitude(const IppComplexDG & Src);
    bool    PowerSpectrum(const IppComplexDG & rhs);
    bool    Phase(const IppComplexDG & rhs);
    bool    Real(const IppComplexDG & Src);
    bool    Imaginary(const IppComplexDG & Src);
    void    Normalize(float Offset, float Factor);
    float   Dot(const IppFloatDG & rhs);
    void    Threshold(float Threshold, IIThreshType Type);
    void    Abs();
    void    Abs(const IppFloatDG &rhs );
    void    Sqr();
    void    Sqr(const IppFloatDG &rhs );
    void    Sqrt();
    void    Sqrt(const IppFloatDG &rhs );
    void    Exp();
    void    Exp(const IppFloatDG &rhs );
    void    Ln();
    void    Ln(const IppFloatDG &rhs );
    float   Max();
    float   Max(int & index);
    float   Min();
    float   Min(int & index);
    float   Mean();
    float   SDev();
    float   Norm(IINormType type);

    //  Operators
    IppFloatDG & operator += (const IppFloatDG & rhs) { Add( rhs ); return *this; }
    IppFloatDG & operator += (float rhs)              { Add( rhs ); return *this; }
    IppFloatDG & operator -= (const IppFloatDG & rhs) { Sub( rhs ); return *this; }
    IppFloatDG & operator -= (float rhs)              { Sub( rhs ); return *this; }
    IppFloatDG & operator *= (const IppFloatDG & rhs) { Multiply( rhs ); return *this; }
    IppFloatDG & operator *= (float rhs)              { Multiply( rhs ); return *this; }
    IppFloatDG & operator /= (const IppFloatDG & rhs) { Divide( rhs ); return *this; }
    IppFloatDG & operator /= (float rhs)              { Divide( rhs ); return *this; }

private:
    Buffer & FBuffer;

    unsigned int    TruncatedSize() const
        {  return static_cast<unsigned int>(Size);  }
};

inline Buffer operator + (const IppFloatDG& lhs, const IppFloatDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG += rhs; return tmp; }
inline Buffer operator + (const float value, const IppFloatDG& rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator + (const IppFloatDG& lhs, const float value)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator - (const IppFloatDG& lhs, const IppFloatDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG -= rhs; return tmp; }
inline Buffer operator - (const IppFloatDG& lhs, const float value)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG -= value; return tmp; }

inline Buffer operator - (const IppFloatDG& lhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG.Multiply(-1.0); return tmp; }
inline Buffer operator * (const IppFloatDG& lhs, const IppFloatDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG *= rhs; return tmp; }
inline Buffer operator * (const float value, const IppFloatDG& rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator * (const IppFloatDG& lhs, const float value)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG *= value; return tmp; }

inline Buffer operator / (const IppFloatDG& lhs, const IppFloatDG& rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG /= rhs; return tmp; }
inline Buffer operator / (const IppFloatDG& lhs, const float value)
    { Buffer tmp(lhs.WrappedBuffer()); IppFloatDG tmpDG(tmp); tmpDG /= value; return tmp; }


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
