//---------------------------------------------------------------------------
//  IppIntegerDG_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "IppIntegerDG_Mb.h"
#include <ipps.h>   // ipps.h is not redistributable!
#include <algorithm>

namespace Innovative
{
//------------------------------------------------------------------------------
//  IppIntegerDG::Zero() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Zero()
{
    int *  DataPtr = &(*(begin()));

    ippsZero_16s(reinterpret_cast<short*>(DataPtr),
                 static_cast<unsigned int>(size() * sizeof(int) / sizeof(short)));
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Set() --
//------------------------------------------------------------------------------

void IppIntegerDG::Set(int value)
{
    int *  DataPtr = &(*(begin()));

    ippsSet_32s(value, DataPtr, TruncatedSize());
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Add() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Add(int value)
{
    if ( value == 0 )
        return;
        
    // Add by subtracting a negative
    Sub( -value );
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Add() --
//------------------------------------------------------------------------------

void IppIntegerDG::Add(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsAdd_32s_ISfs(RhsDataPtr, DataPtr, op_size, 0);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Multiply(int value)
{
    if (value == 1.0)
        return;

    int *  DataPtr = &(*(begin()));

    ippsMulC_32s_ISfs(value, DataPtr, TruncatedSize(), 0);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Multiply(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsMul_32s_ISfs(RhsDataPtr, DataPtr, op_size, 0);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Divide() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Divide(int value)
{
    if (value == 1.0)
        return;

    if (value)
        {
        //  Convert the value to a vector, then vector divide
        Buffer ValueVector( Holding<int>(size()) );
        IppIntegerDG ValueVectorDG(ValueVector);

        ValueVectorDG.Set(value);
        Divide(ValueVectorDG);
        }
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Divide() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Divide(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsDiv_32s_ISfs(RhsDataPtr, DataPtr, op_size, 0);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Sub() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Sub(int value)
{
    if (value == 0)
        return;

    int *  DataPtr = &(*(begin()));

    ippsSubC_32s_ISfs(value, DataPtr, TruncatedSize(), 0);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Sub() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Sub(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsSub_32s_ISfs(RhsDataPtr, DataPtr, op_size, 0);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Or() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Or(int value)
{
    if (value == 0)
        return;

    int *  DataPtr = &(*(begin()));

    ippsOrC_32u_I(value, reinterpret_cast<unsigned int *>(DataPtr), TruncatedSize());
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Or() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Or(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsOr_32u_I(reinterpret_cast<const unsigned int *>(RhsDataPtr),
                 reinterpret_cast<unsigned int *>(DataPtr),
                 op_size);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::And() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::And(int value)
{
    if ((unsigned int)value == 0xFFFFFFFF)
        return;

    int *  DataPtr = &(*(begin()));

    ippsAndC_32u_I(value, reinterpret_cast<unsigned int *>(DataPtr), TruncatedSize());
}

//------------------------------------------------------------------------------
//  IppIntegerDG::And() --
//------------------------------------------------------------------------------

void IppIntegerDG::And(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsAnd_32u_I(reinterpret_cast<const unsigned int *>(RhsDataPtr),
                  reinterpret_cast<unsigned int *>(DataPtr),
                  op_size);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Xor() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::Xor(int value)
{
    if (value == 0)
        return;

    int *  DataPtr = &(*(begin()));

    ippsXorC_32u_I(value, reinterpret_cast<unsigned int *>(DataPtr), TruncatedSize());
}

//------------------------------------------------------------------------------
//  IppIntegerDG::Xor() --
//------------------------------------------------------------------------------

void IppIntegerDG::Xor(const IppIntegerDG & rhs)
{
    int *  DataPtr = &(*(begin()));
    const int *  RhsDataPtr = &(*(rhs.begin()));
    int op_size = static_cast<int>(std::min(size(), rhs.size()));

    ippsXor_32u_I(reinterpret_cast<const unsigned int *>(RhsDataPtr),
                  reinterpret_cast<unsigned int *>(DataPtr),
                  op_size);
}

//------------------------------------------------------------------------------
//  IppIntegerDG::LShift() --
//------------------------------------------------------------------------------
 
void IppIntegerDG::LShift(int Value)
{
    if (Value == 0)
        return;

    int *  DataPtr = &(*(begin()));

    ippsLShiftC_32s_I(Value, DataPtr, TruncatedSize());
}

//------------------------------------------------------------------------------
//  IppIntegerDG::RShift() --
//------------------------------------------------------------------------------

void IppIntegerDG::RShift(int Value)
{
    if (Value == 0)
        return;

    int *  DataPtr = &(*(begin()));

    ippsRShiftC_32s_I(Value, DataPtr, TruncatedSize());
}

} // end namespace Innovative
