//---------------------------------------------------------------------------
//  IppIntegerDG_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef IppIntegerDG_MbH
#define IppIntegerDG_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IppIntegerDG -- Datagram wrapping a buffer; provides access as 'int'
//==============================================================================
//  Also includes IPP fast analysis methods

class IppIntegerDG : public AccessDatagram<int>
{
public:
    typedef AccessDatagram<int> BaseClass;
    //
    //  Member Functions
    IppIntegerDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IppIntegerDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
    const Buffer & WrappedBuffer() const
        {  return FBuffer;  }

	void Zero();
	void Set( int value );

	void Add( int value );
	void Add( const IppIntegerDG & rhs );

	void Multiply( int value );
	void Multiply( const IppIntegerDG & rhs );

	void Divide( int value );
	void Divide( const IppIntegerDG & rhs );

	void Sub( int value );
	void Sub( const IppIntegerDG & rhs );

	void Or( int value );
	void Or( const IppIntegerDG & rhs );

	void And( int value );
	void And( const IppIntegerDG & rhd );

	void Xor( int value );
	void Xor( const IppIntegerDG & rhs );

	void LShift( int Value );
	void RShift( int Value );

	IppIntegerDG & operator += (const IppIntegerDG & rhs)    { Add( rhs ); return *this; }
	IppIntegerDG & operator += (int rhs)                     { Add( rhs ); return *this; }
	IppIntegerDG & operator -= (const IppIntegerDG & rhs)    { Sub( rhs ); return *this; }
	IppIntegerDG & operator -= (int rhs)                     { Sub( rhs ); return *this; }
	IppIntegerDG & operator *= (const IppIntegerDG & rhs)    { Multiply( rhs ); return *this; }
	IppIntegerDG & operator *= (int rhs)                     { Multiply( rhs ); return *this; }
	IppIntegerDG & operator /= (const IppIntegerDG & rhs)    { Divide( rhs ); return *this; }
	IppIntegerDG & operator /= (int rhs)                     { Divide( rhs ); return *this; }
	IppIntegerDG & operator |= (const IppIntegerDG & rhs)    { Or( rhs ); return *this; }
	IppIntegerDG & operator |= (int rhs)                     { Or( rhs ); return *this; }
	IppIntegerDG & operator &= (const IppIntegerDG & rhs)    { And( rhs ); return *this; }
	IppIntegerDG & operator &= (int rhs)                     { And( rhs ); return *this; }
	IppIntegerDG & operator ^= (const IppIntegerDG & rhs)    { Xor( rhs ); return *this; }
	IppIntegerDG & operator ^= (int rhs)                     { Xor( rhs ); return *this; }
	IppIntegerDG & operator <<= (int rhs)                    { LShift( rhs ); return *this; }
	IppIntegerDG & operator >>= (int rhs)                    { RShift( rhs ); return *this; }

private:
    Buffer & FBuffer;

    unsigned int    TruncatedSize() const
        {  return static_cast<unsigned int>(Size);  }
};

inline Buffer operator + (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG += rhs; return tmp; }
inline Buffer operator + (const int value, const IppIntegerDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator + (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator - (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG -= rhs; return tmp; }
inline Buffer operator - (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG -= value; return tmp; }
inline Buffer operator - (const IppIntegerDG & lhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG.Multiply(-1); return tmp; }
inline Buffer operator * (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG *= rhs; return tmp; }
inline Buffer operator * (const int value, const IppIntegerDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator * (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator / (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG /= rhs; return tmp; }
inline Buffer operator / (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG tmpDG(tmp); tmpDG /= value; return tmp; }
inline Buffer operator | (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG |= rhs; return tmp; }
inline Buffer operator | (const int value, const IppIntegerDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG |= value; return tmp; }
inline Buffer operator | (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG |= value; return tmp; }
inline Buffer operator & (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG &= rhs; return tmp; }
inline Buffer operator & (const int value, const IppIntegerDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG &= value; return tmp; }
inline Buffer operator & (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG &= value; return tmp; }
inline Buffer operator ^ (const IppIntegerDG & lhs, const IppIntegerDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG ^= rhs;  return tmp; }
inline Buffer operator ^ (const int value, const IppIntegerDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG ^= value; return tmp; }
inline Buffer operator ^ (const IppIntegerDG & lhs, const int value)
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG ^= value; return tmp; }
inline Buffer operator << (const IppIntegerDG & lhs, int value )
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG <<= value;  return tmp; }
inline Buffer operator >> (const IppIntegerDG & lhs, int value )
    { Buffer tmp(lhs.WrappedBuffer()); IppIntegerDG  tmpDG(tmp); tmpDG >>= value;  return tmp; }

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif

