// IppMemoryUtils_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <IppMemoryUtils_Mb.h>
#include <MemoryUtils_Mb.h>
#include <ipp.h>

namespace Innovative
{
void   IppCopyInt32Buffer( const int *src_buffer, int *dst_buffer, size_t size );
void   IppCopyInt64Buffer( const ii64 *src_buffer, ii64 *dst_buffer, size_t size );
void   IppCopyByteBuffer( const unsigned char *src_buffer, unsigned char *dst_buffer, size_t size );


//===========================================================================
// class IppInitializer -- Pulls in IPP performance library functions.
//===========================================================================

namespace Init
{

//---------------------------------------------------------------------------
// IppInitializer::IppInitializer() 
//---------------------------------------------------------------------------

IppInitializer::IppInitializer() 
{
}

//---------------------------------------------------------------------------
// IppInitializer::IppInitializer(int)
//---------------------------------------------------------------------------

IppInitializer::IppInitializer(int)
{
    CopyInt32Buffer = IppCopyInt32Buffer;
    CopyInt64Buffer = IppCopyInt64Buffer;
    CopyByteBuffer = IppCopyByteBuffer;
}

IppInitializer Instance( 0 );

//---------------------------------------------------------------------------
// InitPerformanceMemoryFunctions() - So that one does not have to instantiate
//                                    an Analysis object to gain access to IPP
//                                    functions.
//---------------------------------------------------------------------------

void UsePerformanceMemoryFunctions()
{
    IppInitializer Init;
}

} // namespace Init

//---------------------------------------------------------------------------
//  CopyInt32Buffer() --
//---------------------------------------------------------------------------

void IppCopyInt32Buffer( const int *src_buffer, int *dst_buffer, size_t size )
{
    ippsCopy_32s( src_buffer,
                 dst_buffer,
                 static_cast<int>(size) );
}

//---------------------------------------------------------------------------
//  CopyInt64Buffer() --
//---------------------------------------------------------------------------

void IppCopyInt64Buffer( const ii64 *src_buffer, ii64 *dst_buffer, size_t size )
{
    ippsCopy_64s( reinterpret_cast<const Ipp64s *>(src_buffer),
                 reinterpret_cast<Ipp64s *>(dst_buffer),
                 static_cast<int>(size) );
}

//---------------------------------------------------------------------------
//  CopyByteBuffer() --
//---------------------------------------------------------------------------

void IppCopyByteBuffer( const unsigned char *src_buffer, unsigned char *dst_buffer, size_t size )
{
    ippsCopy_8u( src_buffer, dst_buffer, static_cast<int>(size) );
}

}  // namespace
