//---------------------------------------------------------------------------
//  IppShortDG_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "IppShortDG_Mb.h"
#include <ipps.h>   // ipps.h is not redistributable!
#include "IppCharDG_Mb.h"
#include <algorithm>

namespace Innovative
{

//---------------------------------------------------------------------------
//  IppShortDG::Zero() --
//------------------------------------------------------------------------------

void IppShortDG::Zero()
{
    short *  DataPtr = &(*(begin()));

    ippsZero_16s(DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppShortDG::Set() --
//------------------------------------------------------------------------------
 
void IppShortDG::Set(short value)
{
    short *  DataPtr = &(*(begin()));

    ippsSet_16s(value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppShortDG::Add() --
//------------------------------------------------------------------------------
 
void IppShortDG::Add(short value)
{
    if (value == 0 )
        return;

    short *  DataPtr = &(*(begin()));

    ippsAddC_16s_ISfs(value, DataPtr, TruncatedSize(), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Add() --
//------------------------------------------------------------------------------
 
void IppShortDG::Add(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsAdd_16s_ISfs(RhsDataPtr, DataPtr, static_cast<unsigned int>(op_size), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppShortDG::Multiply(short value)
{
    if (value == 1.0)
        return;

    short *  DataPtr = &(*(begin()));

    ippsMulC_16s_ISfs(value, DataPtr, TruncatedSize(), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Multiply() --
//------------------------------------------------------------------------------
 
void IppShortDG::Multiply(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsMul_16s_ISfs(RhsDataPtr, DataPtr, static_cast<unsigned int>(op_size), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Divide() --
//------------------------------------------------------------------------------
 
void IppShortDG::Divide(short value)
{
    if (value == 1.0)
        return;

    if (value)
        {
        short *  DataPtr = &(*(begin()));

        ippsDivC_16s_ISfs(value, DataPtr, TruncatedSize(), 0);
        }
}

//---------------------------------------------------------------------------
//  IppShortDG::Divide() --
//------------------------------------------------------------------------------
 
void IppShortDG::Divide(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsDiv_16s_ISfs(RhsDataPtr, DataPtr, static_cast<unsigned int>(op_size), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Sub() --
//------------------------------------------------------------------------------
 
void IppShortDG::Sub(short value)
{
    if (value == 0)
        return;

    short *  DataPtr = &(*(begin()));

    ippsSubC_16s_ISfs(value, DataPtr, TruncatedSize(), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Sub() --
//------------------------------------------------------------------------------
 
void IppShortDG::Sub(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsSub_16s_ISfs(RhsDataPtr, DataPtr, static_cast<unsigned int>(op_size), 0);
}

//---------------------------------------------------------------------------
//  IppShortDG::Or() --
//------------------------------------------------------------------------------
 
void IppShortDG::Or(short value)
{
    if (value == 0)
        return;

    short *  DataPtr = &(*(begin()));

    ippsOrC_16u_I(value, reinterpret_cast<unsigned short *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppShortDG::Or() --
//------------------------------------------------------------------------------
 
void IppShortDG::Or(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsOr_16u_I(reinterpret_cast<const unsigned short *>(RhsDataPtr),
                 reinterpret_cast<unsigned short *>(DataPtr),
                 static_cast<unsigned int>(op_size));

}

//---------------------------------------------------------------------------
//  IppShortDG::And() --
//------------------------------------------------------------------------------
 
void IppShortDG::And(short value)
{
    if (static_cast<unsigned short>(value) == 0xFFFF)
        return;

    short *  DataPtr = &(*(begin()));
    ippsAndC_16u_I(value, reinterpret_cast<unsigned short *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppShortDG::And() --
//------------------------------------------------------------------------------
 
void IppShortDG::And(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsAnd_16u_I(reinterpret_cast<const unsigned short *>(RhsDataPtr),
                  reinterpret_cast<unsigned short *>(DataPtr),
                  static_cast<unsigned int>(op_size));
}

//---------------------------------------------------------------------------
//  IppShortDG::Xor() --
//------------------------------------------------------------------------------
 
void IppShortDG::Xor(short value)
{
    if (value == 0)
        return;

    short *  DataPtr = &(*(begin()));
    ippsXorC_16u_I(value, reinterpret_cast<unsigned short *>(DataPtr), TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppShortDG::Xor() --
//------------------------------------------------------------------------------

void IppShortDG::Xor(const IppShortDG & rhs)
{
    short *  DataPtr = &(*(begin()));
    const short *  RhsDataPtr = &(*(rhs.begin()));
    size_t  op_size = std::min(size(), rhs.size());

    ippsXor_16u_I(reinterpret_cast<const unsigned short *>(RhsDataPtr),
                  reinterpret_cast<unsigned short *>(DataPtr),
                  static_cast<unsigned int>(op_size));
}

//---------------------------------------------------------------------------
//  IppShortDG::LShift() --
//------------------------------------------------------------------------------

void IppShortDG::LShift(int Value)
{
    if (Value == 0)
        return;

    short *  DataPtr = &(*(begin()));

    ippsLShiftC_16s_I(Value, DataPtr, TruncatedSize());
}

//---------------------------------------------------------------------------
//  IppShortDG::RShift() --
//------------------------------------------------------------------------------
 
void IppShortDG::RShift(int Value)
{
    if (Value == 0)
        return;

    short *  DataPtr = &(*(begin()));

    ippsRShiftC_16s_I(Value, DataPtr, TruncatedSize());
}


} // end namespace Innovative
