//---------------------------------------------------------------------------
//  IppShortDG_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
//------------------------------------------------------------------------------

#ifndef IppShortDG_MbH
#define IppShortDG_MbH
//------------------------------------------------------------------------------

#include <AccessDatagrams_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class IppCharDG;
//==============================================================================
//  CLASS IppShortDG -- Datagram wrapping a buffer; provides access as 'short'
//==============================================================================
//  Also includes IPP fast analysis methods

class IppShortDG : public AccessDatagram<short>
{
public:
    typedef AccessDatagram<short> BaseClass;
    //
    //  Member Functions
    IppShortDG(Buffer & buffer)
        :  BaseClass(buffer), FBuffer(buffer)
        {}
    IppShortDG(const Buffer & buffer)
        :  BaseClass(buffer), FBuffer(const_cast<Buffer&>(buffer))
        {}

    Buffer & WrappedBuffer()
        {  return FBuffer;  }
    const Buffer & WrappedBuffer() const
        {  return FBuffer;  }

    void Zero();
    void Set( short value );

    void Add( short value );
    void Add( const IppShortDG & rhs);

    void Multiply( short value );
    void Multiply( const IppShortDG & rhs);

    void Divide( short value );
    void Divide( const IppShortDG & rhs);

    void Sub( short value );
    void Sub( const IppShortDG & rhs);

    void Or( short value );
    void Or( const IppShortDG & rhs);

    void And( short value );
    void And( const IppShortDG & rhs );

    void Xor( short value );
    void Xor( const IppShortDG &  Src );

    void LShift( int Value );
    void RShift( int Value );

    IppShortDG & operator += (const IppShortDG & rhs)     { Add( rhs ); return *this; }
    IppShortDG & operator += (short rhs)                  { Add( rhs ); return *this; }
    IppShortDG & operator -= (const IppShortDG & rhs)     { Sub( rhs ); return *this; }
    IppShortDG & operator -= (short rhs)                  { Sub( rhs ); return *this; }
    IppShortDG & operator *= (const IppShortDG & rhs)     { Multiply( rhs ); return *this; }
    IppShortDG & operator *= (short rhs)                  { Multiply( rhs ); return *this; }
    IppShortDG & operator /= (const IppShortDG & rhs)     { Divide( rhs ); return *this; }
    IppShortDG & operator /= (short rhs)                  { Divide( rhs ); return *this; }
    IppShortDG & operator |= (const IppShortDG & rhs)     { Or( rhs ); return *this; }
    IppShortDG & operator |= (short rhs)                  { Or( rhs ); return *this; }
    IppShortDG & operator &= (const IppShortDG & rhs)     { And( rhs ); return *this; }
    IppShortDG & operator &= (short rhs)                  { And( rhs ); return *this; }
    IppShortDG & operator ^= (const IppShortDG & rhs)     { Xor( rhs ); return *this; }
    IppShortDG & operator ^= (short rhs)                  { Xor( rhs ); return *this; }
    IppShortDG & operator <<= (int rhs)                   { LShift( rhs ); return *this; }
    IppShortDG & operator >>= (int rhs)                   { RShift( rhs ); return *this; }

private:
    Buffer & FBuffer;

    unsigned int    TruncatedSize() const
        {  return static_cast<unsigned int>(Size);  }
};

inline Buffer operator + (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG += rhs; return tmp; }
inline Buffer operator + (const short value, const IppShortDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator + (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG += value; return tmp; }
inline Buffer operator - (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG -= rhs; return tmp; }
inline Buffer operator - (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG -= value; return tmp; }
inline Buffer operator - (const IppShortDG & lhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG.Multiply(-1); return tmp; }
inline Buffer operator * (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG *= rhs; return tmp; }
inline Buffer operator * (const short value, const IppShortDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator * (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG *= value; return tmp; }
inline Buffer operator / (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG /= rhs; return tmp; }
inline Buffer operator / (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG tmpDG(tmp); tmpDG /= value; return tmp; }
inline Buffer operator | (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG |= rhs; return tmp; }
inline Buffer operator | (const short value, const IppShortDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG |= value; return tmp; }
inline Buffer operator | (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG |= value; return tmp; }
inline Buffer operator & (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG &= rhs; return tmp; }
inline Buffer operator & (const short value, const IppShortDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG &= value; return tmp; }
inline Buffer operator & (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG &= value; return tmp; }
inline Buffer operator ^ (const IppShortDG & lhs, const IppShortDG & rhs)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG ^= rhs;  return tmp; }
inline Buffer operator ^ (const short value, const IppShortDG & rhs)
    { Buffer tmp(rhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG ^= value; return tmp; }
inline Buffer operator ^ (const IppShortDG & lhs, const short value)
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG ^= value; return tmp; }
inline Buffer operator << (const IppShortDG & lhs, int value )
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG <<= value;  return tmp; }
inline Buffer operator >> (const IppShortDG & lhs, int value )
    { Buffer tmp(lhs.WrappedBuffer()); IppShortDG  tmpDG(tmp); tmpDG >>= value;  return tmp; }

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif

