// JESDSubDevice_Stream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef JESDSubDevice_Stream_Mb
#define JESDSubDevice_Stream_Mb

#include <X6Support_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IJESD_IoDevice -- Interface to ADC/DAC JESD interfaces
//===========================================================================

class IJESD_IoDevice
{
public:
    virtual bool   IsDeviceActive() = 0;
    virtual double CalcJesdLineRate(double SamplingRate) = 0;

    virtual ~IJESD_IoDevice()
        {}
};

//===========================================================================
//  CLASS IJESD_PllRates -- Interface to ADC/DAC JESD interfaces
//===========================================================================

class IJESD_PllRates
{
public:
    virtual double AdcFrequencyActual() = 0;
    virtual double DacFrequencyActual() = 0;
    virtual double MgtRefFrequencyActual() = 0;

    virtual ~IJESD_PllRates()
        {}
};

//===========================================================================
//  CLASS IRequires_JESDPHY_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_JESDPHY_Io
{
public:

    // Io Interface
    virtual void         PokeJESDPHY(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int PeekJESDPHY(unsigned int device, unsigned int address) = 0;

    virtual IJESD_IoDevice * GetAdcJesd_IoDevice(unsigned int device) = 0;
    virtual IJESD_IoDevice * GetDacJesd_IoDevice(unsigned int device) = 0;
    virtual IJESD_PllRates * GetJesd_PllRatesDevice(unsigned int device) = 0;

    // Customization Interface
    virtual ~IRequires_JESDPHY_Io()
        {}
};

//===========================================================================
//  CLASS JESD_PHY -- JESD PHY IP "SubDevice"
//===========================================================================

class JESD_PHY : public IX6SubDevice
{
public:
    JESD_PHY(IRequires_JESDPHY_Io * mdc, unsigned int device_id=0u);
    virtual ~JESD_PHY();

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void PreconfigureHardware();
    virtual void ConfigureHardware();

    double CalcMgtRefClk(double LineRate);

    struct MgtPllRangesType {
        double       VcoLow;
        double       VcoHigh;
        double       LineRateLow;
        double       LineRateHigh;
        unsigned int MaxOutputDivider;
    };

    struct MgtRefclkRangeType {
        double Low;
        double High;
    };

private:
    IRequires_JESDPHY_Io * MDC;
    const unsigned int     DeviceID;
    IJESD_IoDevice       * AdcJesd;
    IJESD_IoDevice       * DacJesd;
    IJESD_PllRates       * PllRates;

    void         PokeJesdPhy(unsigned int address, unsigned int data) { MDC->PokeJESDPHY(DeviceID, address, data); }
    unsigned int PeekJesdPhy(unsigned int address)                    { return MDC->PeekJESDPHY(DeviceID, address); }

    unsigned int IpVersion;

    enum FpgaTypes { ft7Series, ftUltrascale, ftUltrascalePlus };
    enum FpgaSpeedGrades { fsg1=10, fsg1L, fsg1H, fsg1HV, fsg1LV, fsg2=20, fsg2L, fsg2LV, fsg3=30};
    enum FpgaPackages { fpRF=1, fpFL, fpFF, fpFB, fpHC, fpFH, fpCS, fpCP, fpFT, fpFG, fpSB, fpRB, fpRS, fpCL, fpSF, fpBA, fpFA};
    enum FpgaMgtTypes { fmtGTXE2=2, fmtGTHE2=3, fmtGTHE3=5, fmtGTYE3=6, fmtGTHE4=7 };

    enum LinkDirectionType { ldtRx, ldtTx };
    enum MgtPllTypes { mptCPLL, mptQPLL0, mptQPLL1};
    enum MgtDrpTypes { mdtGTCMN, mdtGTCHAN };

    unsigned int FpgaType;
    unsigned int FpgaSpeedGrade;
    unsigned int FpgaPackage;
    unsigned int FpgaMgtType;

    MgtPllRangesType   CPLLRange;
    MgtPllRangesType   QPLLRange[2];
    MgtRefclkRangeType MgtRefclkRange;

    struct CPLLParamsType {
        unsigned int N1;
        unsigned int N2;
        unsigned int M;

        CPLLParamsType()
            : N1(0), N2(0), M(0)
        {}

        bool operator == (const CPLLParamsType & other);
        bool IsValid() const { return (M!=0 && N1!=0 && N2!=0); }
        void Init() { N1=N2=M=0; }
    };

    struct QPLLParamsType {
        unsigned int N;
        unsigned int M;
        unsigned int QPLLIndex;

        QPLLParamsType()
            : N(0), M(0), QPLLIndex(0)
        {}

        bool operator == (const QPLLParamsType & other);
        bool IsValid() const { return (M!=0 && N!=0); }
        void Init() { N=M=QPLLIndex=0; }
        MgtPllTypes QPLLType()
            {
            MgtPllTypes Types[] = {mptQPLL0, mptQPLL1};
            return Types[QPLLIndex];
            }
    };

    typedef vector<CPLLParamsType> CPLLParamsVec;
    typedef vector<QPLLParamsType> QPLLParamsVec;

    unsigned int NumGtCommon;
    unsigned int NumGtChannel;

    unsigned int   CurrentRxPllType;
    unsigned int   CurrentTxPllType;
    CPLLParamsType CurrentCPLLParams;
    QPLLParamsVec  CurrentQPLLParamsVec;
    unsigned int   CurrentRxOutputDivider;
    unsigned int   CurrentTxOutputDivider;

    CPLLParamsVec RxCPLLParamsVec;
    CPLLParamsVec TxCPLLParamsVec;
    QPLLParamsVec RxQPLLParamsVec;
    QPLLParamsVec TxQPLLParamsVec;

    CPLLParamsType CommonCPLLParams;
    QPLLParamsType CommonQPLLParams;

    unsigned int   NewRxPllType;
    unsigned int   NewTxPllType;
    CPLLParamsType NewCPLLParams;
    QPLLParamsVec  NewQPLLParamsVec;
    unsigned int   RxOutputDivider;
    unsigned int   TxOutputDivider;

    bool         IsGtCommonEnabled() { return (NumGtCommon!=0); }
    void         InitFpgaMgts();
    void         LoadFpgaInfo();
    void         DecodeFpgaSpeedGrade(unsigned int code);
    void         DecodeFpgaPackage(unsigned int code);
    void         LoadFpgaMgtPllRanges();
    void         LoadIpDefaultSettings();
    void         LoadCurrentMgtPllSettings();
    bool         IsFpgaMgtPllUpdateRequired(double RxLineRate, double TxLineRate, double MgtRefClk);
    void         CalcFpgaMgtPllSettings(double RxLineRate, double TxLineRate, double MgtRefClk);
    void         CalcCPLLSettings(double ClkoutFreq, double RefClkFreq, unsigned int LinkDirection);
    void         CalcQPLLSettings(double ClkoutFreq, double RefClkFreq, unsigned int LinkDirection, unsigned int which);
    double       CalcCPLLClkout(double RefClkFreq, CPLLParamsType CPLL);
    double       CalcQPLLClkout(double RefClkFreq, QPLLParamsType QPLL);
    bool         CheckCommonCPLLParams();
    bool         CheckCommonQPLLParams(unsigned int which);
    void         SplitQPLLParamsVec(const QPLLParamsVec & FullQPLLParamsVec, unsigned int which, QPLLParamsVec & PartialQPLLParamsVec);
    void         SelectFpgaMgtPllSettings();
    void         ConfigureFpgaMgts();
    void         ConfigureMgtCPLL();
    void         ConfigureMgtQPLL();
    void         ConfigureQPLL_7S(QPLLParamsType CurrentQPLLParams, QPLLParamsType NewQPLLParams);
    void         ConfigureQPLL_US(QPLLParamsType CurrentQPLLParams, QPLLParamsType NewQPLLParams);
    void         CheckFpgaMgtsPllLock();
    void         SetNumInterfaces();
    void         SetGTCMNSelect(unsigned int which);
    void         SetGTCHANSelect(unsigned int which);
    bool         CheckCPLLLock();
    bool         CheckQPLLLock(unsigned int which);
    void         SetDrpSelect(unsigned int which, unsigned int select);
    void         SetDrpReset(unsigned int which);
    void         SetDrpReg(unsigned int which, unsigned int address, unsigned int data);
    unsigned int GetDrpReg(unsigned int which, unsigned int address);
    void         QPLLPowered(unsigned int QPLLIdx, bool enable);
    void         QPLLPowered_All(unsigned int QPLLIdx, bool enable);
    void         CPLLPowered(bool enable);
    void         CPLLPowered_All(bool enable);
    void         SetTxPllClockSelect();
    void         SetRxPllClockSelect();
    void         GetTxRxPllClockSelect();
    void         GetCPLLParams();
    void         GetQPLLParams();
    unsigned int GetQPLLIndex_7S();
    void         SetOutputDividers();
    void         GetOutputDividers();
    void         InvertTxPolarity(unsigned int which, bool invert);
    void         InvertRxPolarity(unsigned int which, bool invert);
    void         RxPowered(bool enable);
    void         RxPowered_All(bool enable);
    void         TxPowered(bool enable);
    void         TxPowered_All(bool enable);
    unsigned int log2(unsigned int value);
    double       CalcJesdLineRate(unsigned int NumConverters, unsigned int BytePerSample,
                                  double SamplingRate, unsigned int Decimation, unsigned int NumLanes);
};

//===========================================================================
//  CLASS IRequires_JESDADC -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_JESDADC
{
public:
    // Io Interface
    virtual class JESD_ADCDevice & JesdAdc(unsigned int device) = 0;

    // Customization Interface
    virtual ~IRequires_JESDADC()
        {}
};

//===========================================================================
//  CLASS IRequires_JESDADC_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_JESDADC_Io
{
public:
    // Io Interface
    virtual void         PokeJESDADC(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int PeekJESDADC(unsigned int device, unsigned int address) = 0;

    // Customization Interface
    virtual ~IRequires_JESDADC_Io()
        {}
};

//===========================================================================
//  CLASS JESD_ADCDevice -- JESD ADC "SubDevice"
//===========================================================================

class JESD_ADCDevice : public IX6SubDevice
{
public:
    JESD_ADCDevice(IRequires_JESDADC_Io * mdc, unsigned int device_id=0u);
    virtual ~JESD_ADCDevice();

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void PreconfigureHardware();
    virtual void ConfigureHardware();

    unsigned int Device() { return DeviceID; }

    virtual void         Poke(unsigned int address, unsigned int data);
    virtual unsigned int Peek(unsigned int address);

    double CalcJesdLineRate(unsigned int NumConverters, unsigned int BytePerSample,
                            double SamplingRate, unsigned int Decimation, unsigned int NumLanes);

    void  DumpDebugStatus();

private:
    IRequires_JESDADC_Io * MDC;
    const unsigned int     DeviceID;

    unsigned int           IpVersion;
};

//===========================================================================
//  CLASS IRequires_JESDDAC -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_JESDDAC
{
public:
    // Io Interface
    virtual class JESD_DACDevice & JesdDac(unsigned int device_id) = 0;

    // Customization Interface
    virtual ~IRequires_JESDDAC()
        {}
};

//===========================================================================
//  CLASS IRequires_JESDDAC_Io -- X6 "SubDevice" Requirement Interface
//===========================================================================

class IRequires_JESDDAC_Io
{
public:
    // Io Interface
    virtual void         PokeJESDDAC(unsigned int device_id, unsigned int address, unsigned int data) = 0;
    virtual unsigned int PeekJESDDAC(unsigned int device_id, unsigned int address) = 0;

    // Customization Interface
    virtual ~IRequires_JESDDAC_Io()
        {}
};

//===========================================================================
//  CLASS JESD_DACDevice -- JESD DAC "SubDevice"
//===========================================================================

class JESD_DACDevice : public IX6SubDevice
{
public:
    JESD_DACDevice(IRequires_JESDDAC_Io * mdc, unsigned int device_id=0u);
    virtual ~JESD_DACDevice();

    //  Internal Interface
    virtual void ConstructCustomParts();
    virtual void OpenHardware();
    virtual void PreconfigureHardware();
    virtual void ConfigureHardware();

    unsigned int Device() { return DeviceID; }

    virtual void         Poke(unsigned int address, unsigned int data);
    virtual unsigned int Peek(unsigned int address);

    double CalcJesdLineRate(unsigned int NumConverters, unsigned int BytePerSample,
                            double SamplingRate, unsigned int Decimation, unsigned int NumLanes);

    void DumpDebugStatus();

private:
    IRequires_JESDDAC_Io * MDC;
    const unsigned int     DeviceID;

    unsigned int           IpVersion;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
