//---------------------------------------------------------------------------
// K7DeviceDriver_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <K7DeviceDriver_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>

//---------------------------------------------------------------------------

//
//  Nameless namespace for 'static' data
namespace
{

//---------------------------------------------------------------------------
//  K7 Main Device Info
//---------------------------------------------------------------------------
const unsigned int K7_LogicDeviceBar = 2;  // Second BAR is here cause of 64 bit
                                           //   access requires pairing BARs
                                                 
//  Parameters for Interrupt Extension block for use by ISR
const unsigned int   K7_IntStatusMask        = 0xF;
const unsigned int   K7_IntStatusByteAddress =
    Innovative::FmcCarrier_MemoryMap::mmInterruptStatus;
//  Parameters for Driver Open
//  ...Device Driver Name
#define     K7_TARGET_BASE             "K7MANDRV"
#define     K7_TARGET_VENDOR_ID         0x1303
#define     K7_TARGET_DEVICE_ID         0x0070
//  ...Use default Target Number Offset of 0.

//---------------------------------------------------------------------------
//  K7 System Device Info
//---------------------------------------------------------------------------

const unsigned int K7Sys_LogicDeviceBar = 2;  // Second BAR is here cause of 64 bit
                                              //   access requires pairing BARs

//------------------------------------------------
//  Parameters for Interrupt Extension block for use by ISR
const unsigned int   K7Sys_IntStatusMask        = 0x1;
const unsigned int   K7Sys_IntStatusByteAddress =
    Innovative::FmcCarrier_MemoryMap::mmInterruptStatus;
//  Parameters for Driver Open
//  ...Device Driver Name
#define     K7Sys_TARGET_BASE             "K7SYSDRV"
#define     K7Sys_TARGET_VENDOR_ID         0x1303
#define     K7Sys_TARGET_DEVICE_ID         0x0071
//  ...Use default Target Number Offset of 0.

//==============================================================================
//  CLASS RegisterBoard --
//==============================================================================

class RegisterBoard
{

public:
    RegisterBoard()
    {
        InnovativeKernel::RegisterPciDevice(
		// Name      Device name base     Vendor ID    Device ID   Sub ID       CheckSubID
		"K7 Board",  K7_TARGET_BASE,         0x1303,      0x0070,     0x00,        false);
        InnovativeKernel::RegisterPciDevice(
        // Name      Device name base     Vendor ID    Device ID   Sub ID       CheckSubID
		"K7 Sys",    K7Sys_TARGET_BASE,      0x1303,      0x0071,     0x00,        false);
    }
};       

RegisterBoard Registrar;

}  // nameless namespace

namespace Innovative
{
//===========================================================================
//  CLASS K7DeviceDriver  -- Driver level code for K7 Family baseboard
//===========================================================================

InnovativeKernel::IsrLog * K7DeviceDriver::LOG = 0;

//---------------------------------------------------------------------------
//  constructor for class K7DeviceDriver
//---------------------------------------------------------------------------

K7DeviceDriver::K7DeviceDriver()
    :  InnovativeKernel::DualRegionDeviceDriver(
                                    InnovativeKernel::PciId(K7_TARGET_VENDOR_ID, K7_TARGET_DEVICE_ID),
                                    K7_IntStatusByteAddress,
                                    K7_IntStatusMask,
                                    K7_LogicDeviceBar)
{
    //  Default busmaster size is 16 MB
    BusMasterSizeWords(0x1000000/sizeof(int));
    SetHighMemPoolFlag(true);    //use high mempool 64bit addressable

    //  Save static pointer to log info
    LOG = & mIsrInfo;
}

//===========================================================================
//  CLASS K7SysDeviceDriver  -- Driver level code for K7 Family system object
//===========================================================================

InnovativeKernel::IsrLog * K7SysDeviceDriver::LOG = 0;

//---------------------------------------------------------------------------
//  constructor for class K7SysDeviceDriver
//---------------------------------------------------------------------------

K7SysDeviceDriver::K7SysDeviceDriver()
    :  InnovativeKernel::DualRegionIsrDeviceDriver(
                                    InnovativeKernel::PciId(K7Sys_TARGET_VENDOR_ID, K7Sys_TARGET_DEVICE_ID),
                                    K7Sys_IntStatusByteAddress,
                                    K7Sys_IntStatusMask,
                                    K7Sys_LogicDeviceBar)
{
    //  Save static pointer to log info
    LOG = & mIsrInfo;
}

}; // Innovative


