// K7DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <K7DeviceMap_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
//#include <K7MemoryMap_Mb.h>
#include <K7DeviceDriver_Mb.h>

#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS K7DeviceMap  -- Registers for X6 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7DeviceMap
//---------------------------------------------------------------------------
//
//  IDROM Page Numbers

const unsigned int System_IdromPage   = 0;
const unsigned int Assembly_IdromPage = 1;
const unsigned int User_IdromPage     = 3;

//
//  Subdevice Register Structures

using namespace FmcCarrier_MemoryMap;
//using namespace K7_MemoryMap;

namespace
{
const size_t  FMC_Slot_Count = 2;

FmcCarrierDeviceMap::MapWishbones K7_WishboneDefs =
    {
    wbApplication_K7,            // WbApp_Offset;
    };

K7FpaLoader::MapRegisters K7_VsPromRegAddrs =
    {
    mmFpa_Address,           //  AddressControl_Addr;
    mmFpa_Data,              //  Data_Addr
    mmFpa_Opcode,            //  Opcode_Addr;
    };

K7SourceSelectorIo::MapRegisters K7SourceMuxRegAddrs =
    {
    mmK7SourceMux,              //  MuxIo_Addr
    };

K7DualUart::MapRegisters K7DualUartAddrs =
    {
    mmUart_0_Control,       //   Uart0Ctrl_Addr;
    mmUart_0_Tx,            //   Uart0Tx_Addr;
    mmUart_0_Rx,            //   Uart0Rx_Addr;
    mmUart_1_Control,       //   Uart1Ctrl_Addr;
    mmUart_1_Tx,            //   Uart1Tx_Addr;
    mmUart_1_Rx,            //   Uart1Rx_Addr;
    mmUartMode,             //   UartMode_Addr;
    mmInterrupt,            //   UartInterrupt_Addr;
    mmGpioData,             //   GpioData_Addr;
    mmGpioDirection,        //   GpioDir_Addr;
    };

K7FlashRom::MapRegisters K7FlashRomRegAddrs =
    {
    mmRom_AddrCtl,         //  AddressControl_Addr
    mmRom_Data,            //  Data_Addr
    mmRom_Opcode,          //  Opcode_Addr
    };


} // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class K7DeviceMap
//---------------------------------------------------------------------------

K7DeviceMap::K7DeviceMap(IStreamDeviceEvents * owner)
	:   FmcCarrierDeviceMap(owner, FMC_Slot_Count, new K7DeviceDriver, K7_WishboneDefs),
        DDeviceDriver      (dynamic_cast<K7DeviceDriver*>(DeviceDriver)),
        WB_Uart            (LogicMemory, wbUart_K7),
        VsProm_SDev        (this, K7FpaLoader::romS25FL128S, WB_Loader, K7_VsPromRegAddrs),
        SourceMux_SDev     (this, WB_Periph, K7SourceMuxRegAddrs),
        TunerUarts_SDev    (this, WB_Uart, K7DualUartAddrs),
        FlashRom_SDev      (this, K7FlashRom::romS25FL128S, WB_FlashRom, K7FlashRomRegAddrs),
        System_PG          (FlashRom_SDev, System_IdromPage),
        Assembly_PG        (FlashRom_SDev, Assembly_IdromPage),
        UserRom_PG         (FlashRom_SDev, User_IdromPage),
        IdRom_Intf         (this)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7DeviceMap
//---------------------------------------------------------------------------

K7DeviceMap::~K7DeviceMap()
{
}

//---------------------------------------------------------------------------
//  K7DeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  K7DeviceMap::ConstructCustomParts()
{
    FmcCarrierDeviceMap::ConstructCustomParts();

    VsProm_SDev.ConstructCustomParts();
    SourceMux_SDev.ConstructCustomParts();
    TunerUarts_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  K7DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  K7DeviceMap::OpenHardware()
{
    FmcCarrierDeviceMap::OpenHardware();

    //  Rev A cards (only) need Mux selector bits fixed.
    if (Info().PciLogicPcb() == 0)  //FpgaLogicRevision
        SourceMux_SDev.FixReversedSelectorBits(true);

    VsProm_SDev.OpenHardware();
    SourceMux_SDev.OpenHardware();
    TunerUarts_SDev.OpenHardware();

}

//---------------------------------------------------------------------------
//  K7DeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  K7DeviceMap::PreconfigureHardware()
{
    FmcCarrierDeviceMap::PreconfigureHardware();

    VsProm_SDev.PreconfigureHardware();
    SourceMux_SDev.PreconfigureHardware();
    TunerUarts_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  K7DeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7DeviceMap::ConfigureHardware()
{
    FmcCarrierDeviceMap::ConfigureHardware();

    VsProm_SDev.ConfigureHardware();
    SourceMux_SDev.ConfigureHardware();
    TunerUarts_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  K7DeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  K7DeviceMap::StreamStopHardware()
{
    FmcCarrierDeviceMap::StreamStopHardware();
}

//---------------------------------------------------------------------------
//  K7DeviceMap::Uart()
//---------------------------------------------------------------------------

IMulticharUart & K7DeviceMap::Uart(unsigned int ch)
{
    return TunerUarts_SDev.Uart(ch);
}



#ifdef _MSC_VER
    #pragma warning(pop)
#endif


} // namespace Innovative


