// K7DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef K7DeviceMap_MbH
#define K7DeviceMap_MbH

#include <FmcCarrierDeviceMap_Mb.h>
#include <K7SubDevice_Misc_Mb.h>
#include <K7Support_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//class K7DeviceMap;
class K7DeviceDriver;

//===========================================================================
//  CLASS K7DeviceMap  -- Register support for common K7 Boards
//===========================================================================

class K7DeviceMap :  public FmcCarrierDeviceMap//,
                     //public ISuppliesAddedK7Requirements
{
public:

    //
    //  Member Functions
    K7DeviceMap(IStreamDeviceEvents * owner);
    virtual ~K7DeviceMap();

    IMulticharUart &      Uart(unsigned int ch);
    K7SourceSelectorIo &  Selector()   {  return SourceMux_SDev;  }
    IK7_IdRom &           IdRom()      {  return IdRom_Intf;   }
    K7DioPortIntf &       Gpio()       {  return TunerUarts_SDev;  }

public: // pseudo-protected

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:
    K7DeviceDriver *  DDeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace         WB_Uart;
    //
    //  Subdevices
    K7FpaLoader              VsProm_SDev;
    K7SourceSelectorIo       SourceMux_SDev;
    K7DualUart               TunerUarts_SDev;
    K7FlashRom               FlashRom_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    K7AssemblyIdromPage          Assembly_PG;
    X6IdromPage                  UserRom_PG;

    //=======================================================================
    //  CLASS K7IdromIo  -- Access to IDROM Objects
    //=======================================================================
    class K7IdromIo : public IK7_IdRom
    {
    public:

    virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
    //  Page access
    virtual X6SystemIdromPage &   System()   { return Owner->System_PG;  }
    virtual K7AssemblyIdromPage & Assembly() { return Owner->Assembly_PG; }
    virtual X6IdromPage &         User()     { return Owner->UserRom_PG;  }

    K7IdromIo(K7DeviceMap * owner) :  Owner(owner)
        {}
    virtual ~K7IdromIo()
        {}

    private:
        K7DeviceMap *  Owner;
    };

    K7IdromIo                    IdRom_Intf;
    friend class K7IdromIo;

    virtual IFpaLoader &     DerivedVsProm()  {  return VsProm_SDev.VsProm();  }

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
