//---------------------------------------------------------------------------
// K7SubDevice_Misc_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.

#include <K7SubDevice_Misc_Mb.h>
#include <SystemSupport_Mb.h>
#include <cmath>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

static const double UartRefClock = 16.0 * 65536.0/200.0e6;

//===========================================================================
//  CLASS K7SysInformation -- K7 information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysInformation
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class K7SysInformation
//---------------------------------------------------------------------------

K7SysInformation::K7SysInformation(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       FpgaPowerReg(space, regs.FpgaPower_Addr),
       SsdPowerReg(space, regs.SsdPower_Addr),
       K7ForwardReg(space, regs.K7Config_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7SysInformation
//---------------------------------------------------------------------------

K7SysInformation::~K7SysInformation()
{
}

//---------------------------------------------------------------------------
//  K7SysInformation::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysInformation::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SysInformation::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysInformation::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//===========================================================================
//  CLASS K7SysLcdDisplay -- K7 information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysLcdDisplay
//---------------------------------------------------------------------------

enum LcdCommands
{
    lcdClear=0x001, lcdHome=0x002, lcdEntryMode=0x004, lcdOnOff=0x008,
    lcdCursor=0x010, lcdFunction=0x028, lcdCgRamAddr=0x040, lcdDdRamAddr=0x080,
    lcdNextLine=0xc0, lcdBusyAddrCounter=0x100, lcdWriteData=0x200,
    lcdReadData=0x300,
};


//---------------------------------------------------------------------------
//  constructor for class K7SysLcdDisplay
//---------------------------------------------------------------------------

K7SysLcdDisplay::K7SysLcdDisplay(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       WriteDataReg(space, regs.WriteData_Addr),
       ReadDataReg(space, regs.ReadData_Addr),
       LcdType (tlcd_SingleLine)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7SysLcdDisplay
//---------------------------------------------------------------------------

K7SysLcdDisplay::~K7SysLcdDisplay()
{
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysLcdDisplay::OpenHardware()
{
    if (!DeviceEnabled())
        return;
    UpdateLcdType(LcdType);
    Init();
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysLcdDisplay::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::SelectLcdType
//---------------------------------------------------------------------------

void  K7SysLcdDisplay::SelectLcdType(unsigned int Type)
{
    if (!DeviceEnabled())
        return;

    switch (Type)
        {
        case 0:  LcdType = tlcd_MultiLine; break;
        default:  LcdType = tlcd_SingleLine; break;
        }
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::UpdateLcdType()
//---------------------------------------------------------------------------

void  K7SysLcdDisplay::UpdateLcdType(unsigned int Type)
{
    ReadDataReg.LcdTypeSel.Value((Type == 0) ? false : true);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Init     initialize LCD display
//---------------------------------------------------------------------------

void  K7SysLcdDisplay::Init()
{
    Function(true, 0);
    OnOff(false, false, false);
    ClearDisplay();
    EntryMode(true, false);
    Home();
    OnOff(true, false, false);

    Home();
    Type("Hello!");
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::ClearDisplay()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::ClearDisplay()
{
    WriteDataCmd(lcdClear);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Home()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::Home()
{
    WriteDataCmd(lcdHome);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::EntryMode()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::EntryMode(bool inc, bool shift)
{
    WriteDataCmd(lcdEntryMode | (inc ? 2 : 0) | (shift ? 1 : 0));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::OnOff()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::OnOff(bool display, bool cursor, bool blink)
{
    WriteDataCmd(lcdOnOff | (display ? 4 : 0) | (cursor ? 2 : 0) | (blink ? 1 : 0));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Shift()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::Shift(bool displayCursor, bool rightLeft)
{
    WriteDataCmd(lcdCursor | (displayCursor ? 8 : 0) | (rightLeft ? 4 : 0));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Function()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::Function(bool dataLen, unsigned short fontTable)
{
    WriteDataCmd(lcdFunction | (dataLen ? 0x10 : 0) | (fontTable & 3));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::CgRamAddr()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::CgRamAddr(unsigned short addr)
{
    WriteDataCmd(lcdCgRamAddr | (addr & 0x3f));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::DdRamAddr()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::DdRamAddr(unsigned short addr)
{
    WriteDataCmd(lcdDdRamAddr | (addr & 0x7f));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::NextLine()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::NextLine()
{
    WriteDataCmd(lcdNextLine);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::AddressCounter()
//---------------------------------------------------------------------------

unsigned char K7SysLcdDisplay::AddressCounter()
{
    return Key() & 0x7f;
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Ready()
//---------------------------------------------------------------------------

bool K7SysLcdDisplay::Ready()
{
    for (int i = 0; (i < 5) && !WriteDataReg.Ready.Value(); ++i)
        uSleep(500);

    return WriteDataReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::WriteDataCmd()
//---------------------------------------------------------------------------

void   K7SysLcdDisplay::WriteDataCmd(unsigned short data)
{
    Ready();
    WriteDataReg.WriteData.Value(lcdBusyAddrCounter);

    for (int i = 0; (i < 5) && (ReadDataCmd() & 0x80); ++i)
        {
        uSleep(500);

        Ready();
        WriteDataReg.WriteData.Value(lcdBusyAddrCounter);
        }

    Ready();
    WriteDataReg.WriteData.Value(data);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::ReadDataCmd()
//---------------------------------------------------------------------------

unsigned char  K7SysLcdDisplay::ReadDataCmd()
{
    for (int i = 0; (i < 5) && !ReadDataReg.DataValid.Value(); ++i)
        uSleep(500);

    return ReadDataReg.ReadData.Value();
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Key()
//---------------------------------------------------------------------------

unsigned char K7SysLcdDisplay::Key()
{
    WriteDataCmd(lcdReadData);
    return ReadDataCmd();
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Emit()
//---------------------------------------------------------------------------

void K7SysLcdDisplay::Emit(unsigned char c)
{
    WriteDataCmd(lcdWriteData | c);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplay::Type()
//---------------------------------------------------------------------------

void K7SysLcdDisplay::Type(const std::string & txt)
{
    for (size_t i = 0; i < txt.size(); ++i)
        Emit(txt[i]);
}


//===========================================================================
//  CLASS K7SysLcdDisplaySL -- K7 information "SubDevice"
//===========================================================================

//
//---------------------------------------------------------------------------
//  class K7SysLcdDisplaySL is for LCD Display New Haven NHD-02161Z-FSY-YBW-C-S
//---------------------------------------------------------------------------
//

//---------------------------------------------------------------------------
//  housekeeping for class K7SysLcdDisplaySL
//---------------------------------------------------------------------------

enum LcdCommandsSL
{
    lslClear=0x001, lslHome=0x002, lslEntryMode=0x004, lslOnOff=0x008,
    lslCursor=0x010, lslFunction=0x032, lslCgRamAddr=0x040, lslDdRamAddr=0x080, lslWriteData=0x100
};

const unsigned int K7SysLcdDisplaySL_SubAddr = 0x7c;

const unsigned int SLDataWriteOp = 0x01;
const unsigned int SLInstructionWriteOp = 0x00;


//---------------------------------------------------------------------------
//  constructor for class K7SysLcdDisplaySL
//---------------------------------------------------------------------------

K7SysLcdDisplaySL::K7SysLcdDisplaySL(IRequires_Nothing * mdc, IAddressingSpace & space,
                                        MapRegisters & regs)
    :  MDC(mdc),
       I2CReg(space, regs.I2C_Addr),
       SLIntf(I2CReg),
       SL_I2C(SLIntf, (I2CDeviceN::SubAddrTag)0, K7SysLcdDisplaySL_SubAddr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7SysLcdDisplaySL
//---------------------------------------------------------------------------

K7SysLcdDisplaySL::~K7SysLcdDisplaySL()
{
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysLcdDisplaySL::OpenHardware()
{
    if (!DeviceEnabled())
        return;
    Init();

}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysLcdDisplaySL::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::Init     initialize LCD display
//---------------------------------------------------------------------------

void  K7SysLcdDisplaySL::Init()
{
    if (!DeviceEnabled())
        return;

    Function(true, true);
    OnOff(false, false, false);
    ClearDisplay();
    Home();
    OnOff(true, false, false);
    Type("Hello!");
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::ClearDisplay()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::ClearDisplay()
{
    WriteDataCmd(SLInstructionWriteOp, lslClear);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::Home()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::Home()
{
    WriteDataCmd(SLInstructionWriteOp, lslHome);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::EntryMode()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::EntryMode(bool inc, bool shift)
{
    MemoryBitManipulator MBM(lslEntryMode);
    MBM.Bit(0, shift);
    MBM.Bit(1, inc);
    WriteDataCmd(SLInstructionWriteOp, MBM.Value());
    //WriteDataCmd(lcdEntryMode | (inc ? 2 : 0) | (shift ? 1 : 0));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::OnOff()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::OnOff(bool display, bool cursor, bool blink)
{
    MemoryBitManipulator MBM(lslOnOff);
    MBM.Bit(0, blink);
    MBM.Bit(1, cursor);
    MBM.Bit(2, display);
    WriteDataCmd(SLInstructionWriteOp, MBM.Value());
    //WriteDataCmd(lslOnOff | (display ? 4 : 0) | (cursor ? 2 : 0) | (blink ? 1 : 0));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::Shift()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::Shift(bool ShiftCursor, bool RightLeft)
{
    MemoryBitManipulator MBM(lslCursor);
    MBM.Bit(2, RightLeft);
    MBM.Bit(3, ShiftCursor);
    WriteDataCmd(SLInstructionWriteOp, MBM.Value());
    //WriteDataCmd(lcdCursor | (displayCursor ? 8 : 0) | (rightLeft ? 4 : 0));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::Function()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::Function(bool LineNumber, unsigned short FontSize)
{
    MemoryBitManipulator MBM(lslFunction);
    MBM.Bit(2, LineNumber);
	MBM.Bit(3, (FontSize) ? true : false);
    WriteDataCmd(SLInstructionWriteOp, MBM.Value());
    //WriteDataCmd(lcdFunction | (dataLen ? 0x10 : 0) | (fontTable & 3));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::CgRamAddr()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::CgRamAddr(unsigned short addr)
{
    WriteDataCmd(SLInstructionWriteOp, lslCgRamAddr | (addr & 0x3f));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::DdRamAddr()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::DdRamAddr(unsigned short addr)
{
    WriteDataCmd(SLInstructionWriteOp, lslDdRamAddr | (addr & 0x7f));
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::WriteDataCmd()
//---------------------------------------------------------------------------

void   K7SysLcdDisplaySL::WriteDataCmd(unsigned int opcode, unsigned short data)
{
    SL_I2C.LittleReg(opcode, data);
    Sleep(5);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::Emit()
//---------------------------------------------------------------------------

void K7SysLcdDisplaySL::Emit(unsigned char c)
{
    WriteDataCmd(SLDataWriteOp, c);
}

//---------------------------------------------------------------------------
//  K7SysLcdDisplaySL::Type()
//---------------------------------------------------------------------------

void K7SysLcdDisplaySL::Type(const std::string & txt)
{
    size_t totallength = min(txt.size(), size_t(56));
    for (size_t i = 0; i < totallength; ++i)
        {
        if (i == 16)
            {
            for (unsigned int j = 0; j < 24; ++j)
                Emit('i');
            }
        Emit(txt[i]);
        }
}

//===========================================================================
//  CLASS K7SysExtIo -- K7 external io "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysExtIo
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7SysExtIo
//---------------------------------------------------------------------------

/* COMMENTED OUT BECAUSE OF REDUNANCY

K7SysExtIo::K7SysExtIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       ExtIoReg(space, regs.ExtIo_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7SysExtIo
//---------------------------------------------------------------------------

K7SysExtIo::~K7SysExtIo()
{
}

//---------------------------------------------------------------------------
//  K7SysExtIo::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysExtIo::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SysExtIo::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysExtIo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

*/
//===========================================================================
//  CLASS K7ButtonManager -- Button polling manager
//===========================================================================

//---------------------------------------------------------------------------
//  housekeeping for class K7ButtonManager
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7ButtonManager
//---------------------------------------------------------------------------

K7ButtonManager::K7ButtonManager(unsigned int compared)
    : Compared(std::max<unsigned int>(compared, 2))
{
}

//---------------------------------------------------------------------------
//  K7ButtonManager::Reset()
//---------------------------------------------------------------------------

void K7ButtonManager::Reset()
{
    State.clear();
}

//---------------------------------------------------------------------------
//  K7ButtonManager::Monitor()
//---------------------------------------------------------------------------

void K7ButtonManager::Monitor(int state)
{
    State.push_back(state);
    if (State.size() < (Compared+1))
        return;

    int old = State.front();
    State.pop_front();

    bool same = true;
    for (IntList::iterator i = State.begin(); i != State.end(); ++i)
        same &= (*i == state);

    if (same && (old != state) && state)
        {
        ProcessCompletionEvent e(state);
        OnStateChange.Execute(e);
        }
}


//===========================================================================
//  CLASS K7SysLedReset -- K7 LED/Reset "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysLedReset
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7SysLedReset
//---------------------------------------------------------------------------

K7SysLedReset::K7SysLedReset(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), ControlReg(space, regs.LedReset_Addr),
       ExtLedButtonReg(space, regs.ExtLedButton_Addr),
       ExtOutEnableReg(space, regs.ExtLedDirection_Addr),
       ExtIoDataManip(ExtLedButtonReg),
       Buttons(3)
{
    Timer.Interval(1);
    Timer.OnElapsed.SetEvent(this, &K7SysLedReset::HandleTimer);
    Timer.Enabled(false);

    Buttons.OnStateChange.SetEvent(this, &K7SysLedReset::HandleStateChange);
}

//---------------------------------------------------------------------------
//  destructor for class K7SysLedReset
//---------------------------------------------------------------------------

K7SysLedReset::~K7SysLedReset()
{
}

//---------------------------------------------------------------------------
//  K7SysLedReset::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysLedReset::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysLedReset::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    Buttons.Reset();
    Timer.Enabled(OnButtonChange.Assigned());
}

//---------------------------------------------------------------------------
//  K7SysLedReset::StopHardware()
//---------------------------------------------------------------------------

void  K7SysLedReset::StopHardware()
{
    Timer.Enabled(false);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::HandleTimer()
//---------------------------------------------------------------------------

void K7SysLedReset::HandleTimer(OpenWire::NotifyEvent & /*event*/)
{
    Buttons.Monitor(Button());
}

//---------------------------------------------------------------------------
//  K7SysLedReset::HandleStateChange()
//---------------------------------------------------------------------------

void K7SysLedReset::HandleStateChange(ProcessCompletionEvent & event)
{
    OnButtonChange.Execute(event);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::AppReset()
//---------------------------------------------------------------------------

void  K7SysLedReset::AppReset()
{
    Debug.Log("App Reset Applied");
    //
    //  Reset User Logic
    ControlReg.Reset.Set();
    Sleep(1);
    ControlReg.Reset.Clear();
    Sleep(1);

}

//---------------------------------------------------------------------------
//  K7SysLedReset::BlueLed()
//---------------------------------------------------------------------------

void K7SysLedReset::BlueLed(bool state)
{
    ControlReg.BlueLed.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::BlueLed()
//---------------------------------------------------------------------------

bool K7SysLedReset::BlueLed() const
{
    return ControlReg.BlueLed.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  K7SysLedReset::GreenLed()
//---------------------------------------------------------------------------

void K7SysLedReset::GreenLed(bool state)
{
    ControlReg.GreenLed.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::GreenLed()
//---------------------------------------------------------------------------

bool K7SysLedReset::GreenLed() const
{
    return ControlReg.GreenLed.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExternalLed()
//---------------------------------------------------------------------------

void K7SysLedReset::ExternalLed(int mask)
{
    // Led are active-low
    mask ^= 0x0f;
    ExtLedButtonReg.ExternalLed.Value(mask);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExternalLed()
//---------------------------------------------------------------------------

int K7SysLedReset::ExternalLed() const
{
    // Led are active-low
    return ExtLedButtonReg.ExternalLed.Value() ^ 0x0f;
}

//---------------------------------------------------------------------------
//  K7SysLedReset::LedOnOff()
//---------------------------------------------------------------------------

void K7SysLedReset::LedOnOff(bool one, bool two, bool three, bool four)
{
    MemoryBitManipulator MBM(0);
    MBM.Bit(0, one);
    MBM.Bit(1, two);
    MBM.Bit(2, three);
    MBM.Bit(3, four);
    ExternalLed(MBM.Value());
}

//---------------------------------------------------------------------------
//  K7SysLedReset::Button()
//---------------------------------------------------------------------------

int  K7SysLedReset::Button() const
{
    // Buttons are active-low
    return ExtLedButtonReg.Button.Value() ^ 0x3f;
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExtIoData() -- access to Lo Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  K7SysLedReset::ExtIoData()
{
    return ExtIoDataManip;
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExtIoConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

void  K7SysLedReset::ExtIoConfig(unsigned int mask)
{
    ExtOutEnableReg.Value(mask);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExtIoConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

unsigned int  K7SysLedReset::ExtIoConfig() const
{
    return ExtOutEnableReg.Value();
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExtIoOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

void  K7SysLedReset::ExtIoOutputEnable(int output_bit, bool output_enabled)
{
    ExtOutEnableReg.Bit(output_bit, output_enabled);
}

//---------------------------------------------------------------------------
//  K7SysLedReset::ExtIoOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

bool  K7SysLedReset::ExtIoOutputEnable(int output_bit) const
{
    return ExtOutEnableReg.Bit(output_bit);
}

//===========================================================================
//  CLASS K7SourceSelectorIo -- K7 external io "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SourceSelectorIo
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7SourceSelectorIo
//---------------------------------------------------------------------------

K7SourceSelectorIo::K7SourceSelectorIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       MuxIoReg(space, regs.MuxIo_Addr),
       SelectorFixNeeded(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7SourceSelectorIo
//---------------------------------------------------------------------------

K7SourceSelectorIo::~K7SourceSelectorIo()
{
}

//---------------------------------------------------------------------------
//  K7SourceSelectorIo::OpenHardware()
//---------------------------------------------------------------------------

void  K7SourceSelectorIo::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SourceSelectorIo::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SourceSelectorIo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SourceSelectorIo::MasterReferenceSource()
//---------------------------------------------------------------------------

void  K7SourceSelectorIo::MasterReferenceSource(unsigned int src)
{
    if (!DeviceEnabled())
        return;

    MuxIoReg.ClkRefMux.Value(src);
}

//---------------------------------------------------------------------------
//  K7SourceSelectorIo::MasterReferenceSource()
//---------------------------------------------------------------------------

unsigned int  K7SourceSelectorIo::MasterReferenceSource()
{
    if (!DeviceEnabled())
        return 0;

    return MuxIoReg.ClkRefMux.Value();
}


//---------------------------------------------------------------------------
//  K7SourceSelectorIo::MasterPulsePerSecSource()
//---------------------------------------------------------------------------

void  K7SourceSelectorIo::MasterPulsePerSecSource(unsigned int src)
{
    if (!DeviceEnabled())
        return;

    if (SelectorFixNeeded)
        src = SelectorFixup(src);
    MuxIoReg.PpsRefMux.Value(src);
}

//---------------------------------------------------------------------------
//  K7SourceSelectorIo::MasterPulsePerSecSource()
//---------------------------------------------------------------------------

unsigned int  K7SourceSelectorIo::MasterPulsePerSecSource()
{
    if (!DeviceEnabled())
        return 0;

    unsigned int selector_bits = MuxIoReg.PpsRefMux.Value();
    if (SelectorFixNeeded)
        selector_bits = SelectorFixup(selector_bits);
    return selector_bits;
}

//---------------------------------------------------------------------------
//  K7SourceSelectorIo::MasterPulsePerSecSource()
//---------------------------------------------------------------------------

unsigned int  K7SourceSelectorIo::SelectorFixup(unsigned int selector)
{
    // Some hardware flips the bits on the mux improperly, so we flip the bits.
    switch (selector)
        {
        default:
        case 0:
        case 3:  return selector;
        case 2:  return 1;
        case 1:  return 2;
        }
}

//===========================================================================
//  CLASS K7UartPart -- one K7 UART
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7UartPart
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7UartPart
//---------------------------------------------------------------------------

K7UartPart::K7UartPart(IAddressingSpace & space, MapRegisters regs)
    :  UartControlReg(space, regs.UartCtrl_Addr),
       UartTxReg     (space, regs.UartTx_Addr),
       UartRxReg     (space, regs.UartRx_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7UartPart
//---------------------------------------------------------------------------

K7UartPart::~K7UartPart()
{
}

//---------------------------------------------------------------------------
//  K7UartPart::Port(ch) - Write character (polled)
//---------------------------------------------------------------------------

void  K7UartPart::Port(unsigned char ch)
{
    WaitForTxReady();

    UartTxReg.TxData.Value(ch);
}

//---------------------------------------------------------------------------
//  K7UartPart::Port() - Read character (polled)
//---------------------------------------------------------------------------

unsigned char  K7UartPart::Port()
{
    WaitForRxDataReady();

    unsigned char ch = UartRxReg.RxData.Value();
    return ch;
}

//---------------------------------------------------------------------------
//  K7UartPart::PutString() - Write string to serial link
//---------------------------------------------------------------------------

void  K7UartPart::PutString(const std::string & data)
{
    size_t idx=0;
    unsigned int failures(0);

    while (idx < data.size())
        {
        unsigned int room = TxFreeSpace();
        unsigned int to_go = static_cast<unsigned int>(data.size()-idx);
        unsigned int transfer_size = std::min(room, to_go);

        // if sending isn't working, keep track
        if (transfer_size==0)
            failures++;
        else
            failures = 0;
        //  do a max-sized transfer
        for (size_t i=0; i<transfer_size; i++)
            UartTxReg.TxData.Value(data[idx++]);

        if (idx>=data.size())    // should never be greater-than
            break;
        if (failures>=5)
            {
            // Gave up
            std::stringstream msg;
            msg << std::hex << " K7 Uart Tx Send Failure (Full >5 times)";
            Debug.Log(msg.str());
            break;
            }

        WaitForTxReady();   // wait for some characters to come free
        }

}

//---------------------------------------------------------------------------
//  K7UartPart::GetString() - Read accumulated string from serial link
//---------------------------------------------------------------------------

void  K7UartPart::GetString(std::string & data)
{
    data.clear();   // wipe old data

    unsigned int depth = RxDepth();
    data.reserve(depth+1);

    // transfer all you can
    for (size_t i=0; i<depth; i++)
        {
        WaitForRxDataReady();
        data.push_back(UartRxReg.RxData.Value());
        }

}

//---------------------------------------------------------------------------
//  K7UartPart::GetFixedString() - Read fixed sized string
//---------------------------------------------------------------------------

void  K7UartPart::GetFixedString(std::string & data, size_t length)
{
    data.clear();   // wipe old data

    unsigned int depth = static_cast<unsigned int>(length);
    data.reserve(depth+1);

    // transfer all you can
    for (size_t i=0; i<depth; i++)
        {
        WaitForRxDataReady();
        data.push_back(UartRxReg.RxData.Value());
        }

}

//---------------------------------------------------------------------------
//  K7UartPart::WaitForTxReady() -- wait for Tx Ready Bit
//---------------------------------------------------------------------------

void  K7UartPart::WaitForTxReady()
{
    // Wait up to 10ms for Tx Ready
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool bit = UartTxReg.TxReady.Value();
        if (bit==true)
            return;
        else
            {
            Innovative::Sleep(1);
            }
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " K7 Uart Tx Ready Wait Failure (>10ms)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  K7UartPart::WaitForRxDataReady() -- wait for Rx Data Ready Bit
//---------------------------------------------------------------------------

void  K7UartPart::WaitForRxDataReady()
{
    // Wait up to 10ms for Rx Ready
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool bit = UartControlReg.RxReady.Value();
        if (bit==true)
            return;
        else
            {
            Innovative::Sleep(1);
            }
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " K7 Uart Rx Data Ready Wait Failure (>10ms)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  K7UartPart::FrameError() -- return frame error bit
//---------------------------------------------------------------------------

bool  K7UartPart::FrameError()
{
    return UartRxReg.Error.Value();
}

//---------------------------------------------------------------------------
//  K7UartPart::ClearFrameError() -- clear frame error bit
//---------------------------------------------------------------------------

void  K7UartPart::ClearFrameError()
{
    return UartRxReg.ClearFrameError();
}

//---------------------------------------------------------------------------
//  K7UartPart::SetBaud() -- set UART Baud.
//---------------------------------------------------------------------------

void K7UartPart::SetBaud(unsigned int baudrate)
{
    unsigned int reg_val = static_cast<unsigned int>(baudrate*UartRefClock + 0.5f);
    UartControlReg.BaudDivisor.Value(reg_val - 1);
}

//---------------------------------------------------------------------------
//  K7UartPart::GetBaud() -- read UART baud
//---------------------------------------------------------------------------

unsigned int K7UartPart::GetBaud()
{
    unsigned int reg_val = UartControlReg.BaudDivisor.Value() + 1;
    return static_cast<unsigned int>(reg_val/UartRefClock + 0.5f);
}

//---------------------------------------------------------------------------
//  K7UartPart::GetFifoDepth() -- read current FIFO depth
//---------------------------------------------------------------------------

unsigned int K7UartPart::GetFifoDepth()
{
    return UartControlReg.RxCount.Value();
}

//===========================================================================
//  CLASS K7DualUart -- K7 UARTS for wideband tuners "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7DualUart
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7DualUart
//---------------------------------------------------------------------------

K7DualUart::K7DualUart(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       Uart0(space, K7UartPart::MapRegisters(regs.Uart0Ctrl_Addr, regs.Uart0Tx_Addr, regs.Uart0Rx_Addr)),
       Uart1(space, K7UartPart::MapRegisters(regs.Uart1Ctrl_Addr, regs.Uart1Tx_Addr, regs.Uart1Rx_Addr)),
       UartModeReg(space, regs.UartMode_Addr),
       UartInterruptReg(space, regs.UartInterrupt_Addr),
       GpioData(space, regs.GpioData_Addr),
       GpioOutEn(space, regs.GpioDir_Addr),
       GpioDataManip(GpioData)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7DualUart
//---------------------------------------------------------------------------

K7DualUart::~K7DualUart()
{
}

//---------------------------------------------------------------------------
//  K7DualUart::OpenHardware()
//---------------------------------------------------------------------------

void  K7DualUart::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Uart(0).SetBaud(9600);
    Uart(1).SetBaud(9600);
    SetPowerMode(pmNormal);
}

//---------------------------------------------------------------------------
//  K7DualUart::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7DualUart::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7DualUart::SetPowerMode()
//---------------------------------------------------------------------------

void  K7DualUart::SetPowerMode(UartPowerModes  mode)
{
    if (!DeviceEnabled())
        return;

    switch(mode)
        {
        default:
        case pmShutdown:
            UartModeReg.ComMode.Value(false);
            UartModeReg.ComPs.Value(false);
            break;
        case pmRcvActive:
            UartModeReg.ComMode.Value(true);
            UartModeReg.ComPs.Value(false);
            break;
        case pmDriversDisabled:
            UartModeReg.ComMode.Value(false);
            UartModeReg.ComPs.Value(true);
            break;
        case pmNormal:
            UartModeReg.ComMode.Value(true);
            UartModeReg.ComPs.Value(true);
            break;
        }
}

//---------------------------------------------------------------------------
//  K7DualUart::DioPortData() -- access to Lo Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  K7DualUart::DioPortData()
{
    return GpioDataManip;
}

//---------------------------------------------------------------------------
//  K7DualUart::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

void  K7DualUart::DioPortConfig(unsigned int mask)
{
    GpioOutEn.Value(mask);
}

//---------------------------------------------------------------------------
//  K7DualUart::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

unsigned int  K7DualUart::DioPortConfig() const
{
    return GpioOutEn.Value();
}

//---------------------------------------------------------------------------
//  K7DualUart::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

void  K7DualUart::DioPortOutputEnable(int output_bit, bool output_enabled)
{
    GpioOutEn.Bit(output_bit, output_enabled);
}

//---------------------------------------------------------------------------
//  K7DualUart::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

bool  K7DualUart::DioPortOutputEnable(int output_bit) const
{
    return GpioOutEn.Bit(output_bit);
}

//===========================================================================
//  CLASS K7SysTripleUart -- K7 UARTS for wideband tuners "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysTripleUart
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class K7SysTripleUart
//---------------------------------------------------------------------------

K7SysTripleUart::K7SysTripleUart(IRequires_MultiGps * mdc, IGPS_Callback * cb,
                                 IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), CB(cb), GpsIdx(0),//GpsDev(0),
       Uart0(space, K7UartPart::MapRegisters(regs.Uart0Ctrl_Addr, regs.Uart0Tx_Addr, regs.Uart0Rx_Addr)),
       Uart1(space, K7UartPart::MapRegisters(regs.Uart1Ctrl_Addr, regs.Uart1Tx_Addr, regs.Uart1Rx_Addr)),
       Uart2(space, K7UartPart::MapRegisters(regs.Uart2Ctrl_Addr, regs.Uart2Tx_Addr, regs.Uart2Rx_Addr)),
       UartModeReg(space, regs.UartMode_Addr),
       UartInterruptReg(space, regs.UartInterrupt_Addr),
       UartPpsArmReg(space, regs.UartPpsArm_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7SysTripleUart
//---------------------------------------------------------------------------

K7SysTripleUart::~K7SysTripleUart()
{
    delete CB;         CB  = 0;
    for (size_t i=0;  i<GpsDevs.size(); i++)
        {
        delete GpsDevs[i];     GpsDevs[i]  = 0;
        }
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::ConstructCustomParts()
//---------------------------------------------------------------------------

void  K7SysTripleUart::ConstructCustomParts()
{
    size_t GpsCount = MDC->GetGpsCount();
    GpsDevs.resize(GpsCount);

    for (size_t i=0;  i<GpsDevs.size(); i++)
        {
        GpsDevs[i] = MDC->MakeOneGpsDevice(i);
        }
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysTripleUart::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    SetPowerMode(pmNormal);  // turn on power...

    UartModeReg.IspN.Set();
    UartModeReg.LdeKeyN.Set();
    //UartModeReg.ResetN.Set();  // leave in reset

    Uart0.SetBaud(115200);
    Uart1.SetBaud(9600);
    Uart2.SetBaud(9600);
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysTripleUart::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::SetPowerMode()
//---------------------------------------------------------------------------

void  K7SysTripleUart::SetPowerMode(UartPowerModes  mode)
{
    if (!DeviceEnabled())
        return;

    switch(mode)
        {
        default:
        case pmShutdown:
            UartModeReg.ComMode.Value(false);
            UartModeReg.ComPs.Value(false);
            break;
        case pmRcvActive:
            UartModeReg.ComMode.Value(true);
            UartModeReg.ComPs.Value(false);
            break;
        case pmDriversDisabled:
            UartModeReg.ComMode.Value(false);
            UartModeReg.ComPs.Value(true);
            break;
        case pmNormal:
            UartModeReg.ComMode.Value(true);
            UartModeReg.ComPs.Value(true);
            break;
        }
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::Gps()
//---------------------------------------------------------------------------

IGps &  K7SysTripleUart::Gps()
{
    return *(GpsDevs[GpsIdx]);
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::IsLocked()
//---------------------------------------------------------------------------

bool  K7SysTripleUart::IsLocked()
{
    return UartModeReg.Lock.Value();
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::IsAntenna()
//---------------------------------------------------------------------------

bool  K7SysTripleUart::IsAntenna()
{
    return false;  //UartModeReg.Antenna.Value();
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::GpsEnabled() --  Enable/disable GPS subsystem
//---------------------------------------------------------------------------

void K7SysTripleUart::GpsEnabled(bool state)
{
    CB->InterruptEnable(false);

    // Reset receive FIFO
    //UartModeReg.FifoReset.Value(1);
    //UartModeReg.FifoReset.Value(state ? 0 : 1);
    GpsInReset(state ? 0 : 1);
    Sleep(10);

    if (state)
        {
        //Gps->Clear();
        CB->InterruptEnable(true);
        GpsDevs[GpsIdx]->Init();
        }

}

//---------------------------------------------------------------------------
//  K7SysTripleUart::GpsEnabled() --  Query state of GPS subsystem
//---------------------------------------------------------------------------

bool K7SysTripleUart::GpsEnabled() const
{
    return GpsInReset();
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::GpsInReset() --  Query state of GPS reset
//---------------------------------------------------------------------------

bool K7SysTripleUart::GpsInReset() const
{
    return UartModeReg.ResetN.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::GpsInReset() --  Query state of GPS subsystem
//---------------------------------------------------------------------------

void K7SysTripleUart::GpsInReset(bool state)
{
    UartModeReg.ResetN.Value((state) ? false : true);
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::ArmGpsTrigger() --  Fire trigger at next epoch
//---------------------------------------------------------------------------

void K7SysTripleUart::ArmGpsTrigger()
{
    UartPpsArmReg.Bit(0, false);   // Level sensitive...produce + pulse
    UartPpsArmReg.Bit(0, true);    //
    UartPpsArmReg.Bit(0, false);   //
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::UartData() --
//---------------------------------------------------------------------------

void K7SysTripleUart::UartData(char c)
{
    Uart0.Port(c);
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::UartData() --
//---------------------------------------------------------------------------

char K7SysTripleUart::UartData()
{
    return static_cast<char>(Uart0.Port());
}

/*
//---------------------------------------------------------------------------
//  K7SysTripleUart::Baud() -- set UART Baud.
//---------------------------------------------------------------------------

void K7SysTripleUart::Baud(IGps::BaudRates rate)
{
    UartControlReg.Value(rate);
}
*/
//---------------------------------------------------------------------------
//  K7SysTripleUart::Baud() -- read UART baud
//---------------------------------------------------------------------------

IGps::BaudRates K7SysTripleUart::Baud() const
{
    return IGps::b115200;  // don't support changing baud rates
}

//---------------------------------------------------------------------------
//  K7SysTripleUart::UartFifoDepth()
//---------------------------------------------------------------------------

unsigned int  K7SysTripleUart::UartFifoDepth()
{
    ScopedTraceRef ccc(Debug, "K7SysTripleUart::UartFifoDepth()");

	unsigned int depth = Uart0.GetFifoDepth();
    Debug.Log("depth=", depth);
    return depth;
}

//===========================================================================
//  CLASS SbcDuoGpsUart -- SbcDuo UARTS for Gps "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuoGpsUart
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class SbcDuoGpsUart
//---------------------------------------------------------------------------

SbcDuoGpsUart::SbcDuoGpsUart(IRequires_MultiGps * mdc, IGPS_Callback * cb,
                                 IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), CB(cb), GpsIdx(0),
       Uart0(space, K7UartPart::MapRegisters(regs.Uart0Ctrl_Addr, regs.Uart0Tx_Addr, regs.Uart0Rx_Addr)),
       Uart1(space, K7UartPart::MapRegisters(regs.Uart1Ctrl_Addr, regs.Uart1Tx_Addr, regs.Uart1Rx_Addr)),
       UartStatusReg(space, regs.UartStatus_Addr),
       UartInterruptReg(space, regs.UartInterrupt_Addr),
       UartPpsArmReg(space, regs.UartPpsArm_Addr),
       IsGpsEnabled(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuoGpsUart
//---------------------------------------------------------------------------

SbcDuoGpsUart::~SbcDuoGpsUart()
{
    delete CB;         CB  = 0;
    for (size_t i=0;  i<GpsDevs.size(); i++)
        {
        delete GpsDevs[i];     GpsDevs[i]  = 0;
        }
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::ConstructCustomParts()
//---------------------------------------------------------------------------

void  SbcDuoGpsUart::ConstructCustomParts()
{
    size_t GpsCount = MDC->GetGpsCount();
    GpsDevs.resize(GpsCount);

    for (size_t i=0;  i<GpsDevs.size(); i++)
        {
        GpsDevs[i] = MDC->MakeOneGpsDevice(i);
        }
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuoGpsUart::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    IsGpsEnabled = false;
    //SetPowerMode(pmNormal);  // turn on power...

    //UartModeReg.IspN.Set();
    //UartModeReg.LdeKeyN.Set();
    //UartModeReg.ResetN.Set();  // leave in reset

    Uart0.SetBaud(115200);
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoGpsUart::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::SetPowerMode()
//---------------------------------------------------------------------------
/*
void  SbcDuoGpsUart::SetPowerMode(UartPowerModes  mode)
{
    if (!DeviceEnabled())
        return;

    switch(mode)
        {
        default:
        case pmShutdown:
            UartModeReg.ComMode.Value(false);
            UartModeReg.ComPs.Value(false);
            break;
        case pmRcvActive:
            UartModeReg.ComMode.Value(true);
            UartModeReg.ComPs.Value(false);
            break;
        case pmDriversDisabled:
            UartModeReg.ComMode.Value(false);
            UartModeReg.ComPs.Value(true);
            break;
        case pmNormal:
            UartModeReg.ComMode.Value(true);
            UartModeReg.ComPs.Value(true);
            break;
        }
}
*/
//---------------------------------------------------------------------------
//  SbcDuoGpsUart::Gps()
//---------------------------------------------------------------------------

IGps &  SbcDuoGpsUart::Gps()
{
    return *(GpsDevs[GpsIdx]);
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::GpsReset()  - Super-reset of Mezzanine link or Gps Link
//---------------------------------------------------------------------------

void SbcDuoGpsUart::GpsReset(bool state)
{
    state = !state;  // negative logic bit
    UartStatusReg.MezzGpsLinkResetN.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::IsLocked()
//---------------------------------------------------------------------------

bool  SbcDuoGpsUart::IsLocked()
{
    return UartStatusReg.Lock.Value();
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::IsAntenna()
//---------------------------------------------------------------------------

bool  SbcDuoGpsUart::IsAntenna()
{
    return false;  //UartModeReg.Antenna.Value();
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::GpsEnabled() --  Enable/disable GPS subsystem
//---------------------------------------------------------------------------

void SbcDuoGpsUart::GpsEnabled(bool state)
{
    CB->InterruptEnable(false);
    IsGpsEnabled = state;

    // Reset receive FIFO
    //UartModeReg.FifoReset.Value(1);
    //UartModeReg.FifoReset.Value(state ? 0 : 1);
    //GpsInReset(state ? 0 : 1);
    //Sleep(10);

    if (state)
        {
        //Gps->Clear();
        CB->InterruptEnable(true);
        GpsDevs[GpsIdx]->Init();
        }

}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::GpsEnabled() --  Query state of GPS subsystem
//---------------------------------------------------------------------------

bool SbcDuoGpsUart::GpsEnabled() const
{
    return IsGpsEnabled;
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::ArmGpsTrigger() --  Fire trigger at next epoch
//---------------------------------------------------------------------------

void SbcDuoGpsUart::ArmGpsTrigger()
{
    UartPpsArmReg.Bit(0, false);   // Level sensitive...produce + pulse
    UartPpsArmReg.Bit(0, true);    //
    UartPpsArmReg.Bit(0, false);   //
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::UartData() --
//---------------------------------------------------------------------------

void SbcDuoGpsUart::UartData(char c)
{
    Uart0.Port(c);
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::UartData() --
//---------------------------------------------------------------------------

char SbcDuoGpsUart::UartData()
{
    return static_cast<char>(Uart0.Port());
}

/*
//---------------------------------------------------------------------------
//  SbcDuoGpsUart::Baud() -- set UART Baud.
//---------------------------------------------------------------------------

void SbcDuoGpsUart::Baud(IGps::BaudRates rate)
{
    UartControlReg.Value(rate);
}
*/
//---------------------------------------------------------------------------
//  SbcDuoGpsUart::Baud() -- read UART baud
//---------------------------------------------------------------------------

IGps::BaudRates SbcDuoGpsUart::Baud() const
{
    return IGps::b115200;  // don't support changing baud rates
}

//---------------------------------------------------------------------------
//  SbcDuoGpsUart::UartFifoDepth()
//---------------------------------------------------------------------------

unsigned int  SbcDuoGpsUart::UartFifoDepth()
{
    ScopedTraceRef ccc(Debug, "SbcDuoGpsUart::UartFifoDepth()");

    unsigned int depth = Uart0.GetFifoDepth();
    Debug.Log("depth=", depth);
    return depth;
}

//===========================================================================
//  CLASS K7_RomControl -- K7 Rom Programming Shared Code
//===========================================================================
//---------------------------------------------------------------------------
//  K7_RomControl::SetBlockProtection() -- Impose all block protection
//---------------------------------------------------------------------------

void  K7_RomControl::SetBlockProtection()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x01, 0x00, 0x9c);  // WRSR (write status register)
    //    SRWD is 1, using the hardware protect signal.
    Flash_NotBusy_T_ms(1000);  // make sure RDSR busy status is complete
}

//---------------------------------------------------------------------------
//  K7_RomControl::ReleaseBlockProtection() -- Remove all block protection
//---------------------------------------------------------------------------

void  K7_RomControl::ReleaseBlockProtection()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x01, 0x00, 0x80);  // WRSR (write status register)
    // ...the above sets BP0-BP3 to 0, removing all protection
    //    SRWD is 1, using the hardware protect signal.
    Flash_NotBusy_T_ms(1000);  // make sure RDSR busy status is complete
}

//---------------------------------------------------------------------------
//  K7_RomControl::IsBlockProtectionReleased() -- Check block protection
//---------------------------------------------------------------------------

bool  K7_RomControl::IsBlockProtectionReleased()
{   
    unsigned int rdsr = Rdsr();
    if ((rdsr&0xFC)==0x80)
        {
        return true;  // unprotected
        }
    else
        {
        return false;  // protected
        }
}

//---------------------------------------------------------------------------
//  K7_RomControl::ErasePage() -- erase 1 IDROM page
//---------------------------------------------------------------------------

void  K7_RomControl::ErasePage(unsigned int page)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x20, page*Map.Page_Size);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  K7_RomControl::EraseSector() -- erase 1 IDROM sector
//---------------------------------------------------------------------------

void  K7_RomControl::EraseSector(unsigned int sector)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0xd8, sector*Map.Sector_Size);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  K7_RomControl::ReadPage() -- read 1 IDROM page
//---------------------------------------------------------------------------
//  Size argument is in 32 bit words

void  K7_RomControl::ReadPage(unsigned int page, int * pkt, unsigned int size)
{
    size_t rpg_limit = std::min<size_t>(static_cast<size_t>(Map.Page_Size/sizeof(int)), size);

    //  Read one 32 bit word at a time into memory
    for (size_t i=0; i<rpg_limit; i++)
        {
        unsigned int addr = static_cast<unsigned int>(page*Map.Page_Size + i*sizeof(int));
        unsigned int value = Read32(addr);
        *pkt++ = value;
        }
}

//---------------------------------------------------------------------------
//  K7_RomControl::WritePage() -- write 1 IDROM page
//---------------------------------------------------------------------------
//  Size argument is in 32 bit words

void  K7_RomControl::WritePage(unsigned int page, int * pkt, unsigned int size)
{
    size_t wpg_limit = std::min<size_t>(static_cast<size_t>(Map.Page_Size/sizeof(int)), size);

    //  Program one 32 bit word at a time into memory
    for (size_t i=0; i<wpg_limit; i++)
        {
        unsigned int addr =  static_cast<unsigned int>(page*Map.Page_Size + i*sizeof(int));
        Write32(addr, pkt[i]);
        }
}

//---------------------------------------------------------------------------
//  K7_RomControl::Read32() -- read 1 IDROM page
//---------------------------------------------------------------------------

unsigned int  K7_RomControl::Read32(unsigned int addr)
{
    return FlashOp(0x03, addr);
}

//---------------------------------------------------------------------------
//  K7_RomControl::Write32() -- write 1 word
//---------------------------------------------------------------------------

void  K7_RomControl::Write32(unsigned int addr, unsigned int value)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x02, addr, value );

    Flash_QuickNotBusy();
}

//---------------------------------------------------------------------------
//  K7_RomControl::SetProgramPageMode() -- Set Page Mode bit
//---------------------------------------------------------------------------

void  K7_RomControl::SetProgramPageMode(bool state)
{
    WritePageModeBit(state);
}

//---------------------------------------------------------------------------
//  K7_RomControl::SetPageAddress() -- Set Page Mode Address
//---------------------------------------------------------------------------

void  K7_RomControl::SetPageAddress(unsigned int addr)
{
    WriteAddr(addr);
}

//---------------------------------------------------------------------------
//  K7_RomControl::Enqueue32() -- Add data to page mode queue
//---------------------------------------------------------------------------

void  K7_RomControl::Enqueue32(unsigned int value)
{
    WriteData(value);
}

//---------------------------------------------------------------------------
//  K7_RomControl::WaitForPageComplete() -- Wait for write to finish
//---------------------------------------------------------------------------

void  K7_RomControl::WaitForPageComplete()
{
    // Wait up to 2000 uS for pending flash write to be valid
    Flash_NotBusy_T_us(2000);
}

//---------------------------------------------------------------------------
//  K7_RomControl::PageProgram() -- Start a page mode access
//---------------------------------------------------------------------------

void  K7_RomControl::PageProgram()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x2);
}

//---------------------------------------------------------------------------
//  K7_RomControl::Rdsr() -- Flash Operation - Read status register
//---------------------------------------------------------------------------

unsigned int  K7_RomControl::Rdsr()
{
    Flash_Ready_T(1000);

    // Write opcode to trigger the transaction
    WriteOpcode(0x5);

    Flash_ReadValid();

    unsigned int data = ReadData();

//    std::stringstream msg;
//    msg << std::hex << "Rdsr:"  << data;
//    TraceDebug(msg.str());

    return data;
}

//---------------------------------------------------------------------------
//  K7_RomControl::JdecId() -- Flash Instruction - JDEC-ID
//---------------------------------------------------------------------------

unsigned int  K7_RomControl::JdecId()
{
    return FlashOp(0x9F, 0);   // address unused
}

//---------------------------------------------------------------------------
//  K7_RomControl::Rdid() -- Flash Instruction - RDID
//---------------------------------------------------------------------------

unsigned int  K7_RomControl::Rdid()
{
    return FlashOp(0x90, 0);   // address unused
}

//---------------------------------------------------------------------------
//  K7_RomControl::Reset() -- Flash Instruction - Reset
//---------------------------------------------------------------------------

void  K7_RomControl::Reset()
{
    FlashOp(0xF0);
}



//---------------------------------------------------------------------------
//  K7_RomControl::FlashOp() -- primitive Flash Operation - "command-like" ops
//---------------------------------------------------------------------------

void  K7_RomControl::FlashOp(unsigned int opcode)
{
    Flash_Ready();

    // Write opcode to trigger the transaction
    WriteOpcode(opcode);
}

//---------------------------------------------------------------------------
//  K7_RomControl::FlashOp() -- primitive Flash Operation - "read-like" ops
//---------------------------------------------------------------------------

unsigned int  K7_RomControl::FlashOp(unsigned int opcode, unsigned int addr)
{
    Flash_Ready();

    // Write address
    WriteAddr(addr);

    // Write opcode to trigger the transaction
    WriteOpcode(opcode);

    Flash_ReadValid();

    unsigned int data = ReadData();
    return data;
}

//---------------------------------------------------------------------------
//  K7_RomControl::FlashOp() -- primitive Flash Operation - "write-like" ops
//---------------------------------------------------------------------------

void  K7_RomControl::FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
{
    Flash_Ready();

    // Write data
    WriteData(data);

    // Write address
    WriteAddr(addr);

    // Write opcode to trigger the transaction
    WriteOpcode(opcode);

}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_Ready() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_Ready()
{
    Flash_Ready_T(200);
}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_Ready_T() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_Ready_T(unsigned int max_wait_us)
{
    const unsigned int reps = 10;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to "max_wait_us" uS for pending flash to be ready
    for (unsigned int i = 0; i < reps; ++i)
        {
        bool ready = ReadReady();
        if (ready)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Ready Failure (>" << max_wait_us << "us)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_ReadValid() -- wait for flash readvalid indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_ReadValid()
{
    // Wait up to 200 uS for pending flash read to be valid
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool readvalid = ReadDataValid();
        if (readvalid)
            return;
        else
            Innovative::uSleep(20);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Read Valid Failure (>200us)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_NotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_NotBusy()
{
    Flash_NotBusy_T_ms(5000);  // wait up to 5000 ms
}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_QuickNotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_QuickNotBusy()
{
    Flash_NotBusy_T_us(200);  // wait up to 200 us
}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_NotBusy_T_ms() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_NotBusy_T_ms(unsigned int max_wait_ms)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_ms/reps;

    // Wait up to  "max_wait_ms" mS for Rdsr bit 0 to clear
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::Sleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Not Busy Failure (>" << max_wait_ms << "ms)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  K7_RomControl::Flash_NotBusy_T_us() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  K7_RomControl::Flash_NotBusy_T_us(unsigned int max_wait_us)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to  "max_wait_us" uS for Rdsr bit 0 to clear
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Not Busy Failure (>" << max_wait_us << "us)";
    TraceDebug(msg.str());

}
//===========================================================================
//  CLASS K7FpaLoader -- K7 FPA Loader "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7FpaLoader
//---------------------------------------------------------------------------

K7_RomControl::RomMapInfo  RomS25FL128S_MapInfo =
{
    32,            //  Page_Count;
    4096,          //  Page_Size (bytes);
    512,           //  Sector_Count;
    65536,         //  Sector_Size (bytes);
};

//---------------------------------------------------------------------------
//  constructor for class K7FpaLoader
//---------------------------------------------------------------------------

K7FpaLoader::K7FpaLoader(IRequires_Nothing * mdc, RomType which_rom,
                           IAddressingSpace & space, MapRegisters & regs)
    :  K7_RomControl(SelectRomMap(which_rom)),
       MDC(mdc),
       AddressControlReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr),
       Chip(MakeChipObject(which_rom)), RawProm(this),
       Thread( new K7FpaPersonality( Chip ))
{
    Thread.SetFullEraseMode(false);  // only erase Image Size
}

//---------------------------------------------------------------------------
//  destructor for class K7FpaLoader
//---------------------------------------------------------------------------

K7FpaLoader::~K7FpaLoader()
{
    Thread.Shutdown();

    delete Chip;
}

//---------------------------------------------------------------------------
//  K7FpaLoader::OpenHardware()
//---------------------------------------------------------------------------

void  K7FpaLoader::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Thread.OnOpen();
}

//---------------------------------------------------------------------------
//  K7FpaLoader::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7FpaLoader::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  K7FpaLoader::WritePageModeBit() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FpaLoader::WritePageModeBit(bool bit)
{
    AddressControlReg.PageMode.Value(bit);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::WriteOpcode() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FpaLoader::WriteOpcode(unsigned int opcode)
{
    OpcodeReg.Value(opcode);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::WriteAddr() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FpaLoader::WriteAddr(unsigned int addr)
{
    AddressControlReg.Addr.Value(addr);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::WriteData() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FpaLoader::WriteData(unsigned int data)
{
    DataReg.Value(data);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::ReadData() -- primitive Flash Operation
//---------------------------------------------------------------------------

unsigned int  K7FpaLoader::ReadData()
{
    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  K7FpaLoader::ReadReady() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  K7FpaLoader::ReadReady()
{
    bool ready = AddressControlReg.Ready.Value();
    return ready;
}

//---------------------------------------------------------------------------
//  K7FpaLoader::ReadDataValid() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  K7FpaLoader::ReadDataValid()
{
    bool readvalid = AddressControlReg.ReadDataValid.Value();
    return readvalid;
}

//---------------------------------------------------------------------------
//  K7FpaLoader::TraceDebug() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FpaLoader::TraceDebug(const std::string & text)
{
    Debug.Log(text);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::MakeChipObject() -- create proper chip object
//---------------------------------------------------------------------------

IK7FpaChip *   K7FpaLoader::MakeChipObject(RomType which_rom)
{
    if (which_rom==romS25FL128S)
        return new K7FpaChip(this);
    else //(which_rom==romS25FL128S_Sys)
        return new K7SysFpaChip(this);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::SelectRomMap() -- create proper rom map
//---------------------------------------------------------------------------

K7_RomControl::RomMapInfo  K7FpaLoader::SelectRomMap(RomType /*which_rom*/)
{
    //if (which_rom==romS25FL128S || which_rom==romS25FL128S_Sys)
    return RomS25FL128S_MapInfo;
}

//===========================================================================
//  CLASS K7FpaLoader::K7FpaChip  -- CPLD Hardware Interface
//===========================================================================
//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::Initialize() -- init chip object
//---------------------------------------------------------------------------

bool  K7FpaLoader::K7FpaChip::Initialize()
{
    if (!Owner->DeviceEnabled())
        return false;

    Owner->Reset();

    FDeviceId = Owner->Rdid();

    Owner->SetBlockProtection();
    //Owner->ReleaseBlockProtection();
    FHWProtected = true;
    FProtected = false;

    return true;   // or check if RDID is right?
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::Read32() -- Read 32 bit word
//---------------------------------------------------------------------------

unsigned int  K7FpaLoader::K7FpaChip::Read32(unsigned int addr)
{
    if (!Owner->DeviceEnabled())
        return false;

    return Owner->Read32(addr);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::Write32() -- Write 32 bit word
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::Write32(unsigned int addr, unsigned int value)
{
    if (!Owner->DeviceEnabled())
        return;

     Owner->Write32(addr, value);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::SetProtection() -- Set block protection
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::SetProtection()
{
    Owner->SetBlockProtection();
    FHWProtected = true;   // we might as well trust it.
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::ReleaseProtection() -- Remove block protection
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::ReleaseProtection()
{
    Owner->ReleaseBlockProtection();
    if (Owner->IsBlockProtectionReleased())
        FHWProtected = false;
    else
        FHWProtected = true;
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::EraseSector() -- Erase a sector
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::EraseSector(unsigned int sector)
{
    if (!Owner->DeviceEnabled())
        return;

    Owner->EraseSector(sector);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::SetProgramPageMode() -- Prep for Page mode write
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::SetProgramPageMode(bool state)
{
    Owner->SetProgramPageMode(state);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::SetPageAddress() -- load address
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::SetPageAddress(unsigned int addr)
{
    Owner->SetPageAddress(addr);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::Enqueue32() -- load data
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::Enqueue32(unsigned int value)
{
    Owner->Enqueue32(value);
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::WaitForPageComplete() -- check for completion
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::WaitForPageComplete()
{
    Owner->WaitForPageComplete();
}

//---------------------------------------------------------------------------
//  K7FpaLoader::K7FpaChip::PageProgram() -- start page programming process
//---------------------------------------------------------------------------

void  K7FpaLoader::K7FpaChip::PageProgram()
{
    Owner->PageProgram();
}

//===========================================================================
//  CLASS K7FlashRom -- K7 Flash Rom "SubDevice"  - IDROM and other uses
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7FlashRom
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class K7FlashRom
//---------------------------------------------------------------------------

K7FlashRom::K7FlashRom(IRequires_Nothing * mdc, RomType which_rom,
                       IAddressingSpace & space, MapRegisters & regs)
    :  K7_RomControl(SelectRomMap(which_rom)),
       MDC(mdc),
       AddressControlReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7FlashRom
//---------------------------------------------------------------------------

K7FlashRom::~K7FlashRom()
{
}

//---------------------------------------------------------------------------
//  K7FlashRom::OpenHardware()
//---------------------------------------------------------------------------

void  K7FlashRom::OpenHardware()
{
    if (!DeviceEnabled())
        return;
    Reset();
    ReleaseBlockProtection();
}

//---------------------------------------------------------------------------
//  K7FlashRom::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7FlashRom::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  K7FlashRom::WritePageModeBit() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FlashRom::WritePageModeBit(bool bit)
{
    AddressControlReg.PageMode.Value(bit);
}


//---------------------------------------------------------------------------
//  K7FlashRom::WriteOpcode() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FlashRom::WriteOpcode(unsigned int opcode)
{
    OpcodeReg.Value(opcode);
}

//---------------------------------------------------------------------------
//  K7FlashRom::WriteAddr() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FlashRom::WriteAddr(unsigned int addr)
{
    AddressControlReg.Addr.Value(addr);
}

//---------------------------------------------------------------------------
//  K7FlashRom::WriteData() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FlashRom::WriteData(unsigned int data)
{
    DataReg.Value(data);
}

//---------------------------------------------------------------------------
//  K7FlashRom::ReadData() -- primitive Flash Operation
//---------------------------------------------------------------------------

unsigned int  K7FlashRom::ReadData()
{
    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  K7FlashRom::ReadReady() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  K7FlashRom::ReadReady()
{
    bool ready = AddressControlReg.Ready.Value();
    return ready;
}

//---------------------------------------------------------------------------
//  K7FlashRom::ReadDataValid() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  K7FlashRom::ReadDataValid()
{
    bool readvalid = AddressControlReg.ReadDataValid.Value();
    return readvalid;
}

//---------------------------------------------------------------------------
//  K7FlashRom::TraceDebug() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  K7FlashRom::TraceDebug(const std::string & text)
{
    Debug.Log(text);
}

//---------------------------------------------------------------------------
//  K7FlashRom::SelectRomMap() -- create proper rom map
//---------------------------------------------------------------------------

K7_RomControl::RomMapInfo  K7FlashRom::SelectRomMap(RomType /*which_rom*/)
{
    //if (which_rom==romS25FL128S || which_rom==romS25FL128S_Sys)
    return RomS25FL128S_MapInfo;
}

//===========================================================================
//  CLASS K7Ieee1588 -- K7 IEEE1588 "SubDevice"  -
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7Ieee1588
//---------------------------------------------------------------------------

const unsigned int IeeeI2C_SubAddr = 0xA0;

//---------------------------------------------------------------------------
//  constructor for class K7Ieee1588
//---------------------------------------------------------------------------

K7Ieee1588::K7Ieee1588(IRequires_Nothing * mdc, IAddressingSpace & space,
                       MapRegisters & regs)
    :  MDC(mdc),
       Uart0(space, K7UartPart::MapRegisters(regs.Uart_Ctrl_Addr, regs.Uart_Tx_Addr, regs.Uart_Rx_Addr)),
       ControlReg(space, regs.Control_Addr),
       ClockId_Lo_Reg(space, regs.ClockID_Lo_Addr),
       ClockId_Hi_Reg(space, regs.ClockID_Hi_Addr),
       TimeReg(space, regs.Time_Addr),
       I2CReg(space, regs.I2C_Addr),
       Uart_Ctl_Reg(space, regs.Uart_Ctrl_Addr),
       Uart_Tx_Reg(space, regs.Uart_Tx_Addr),
       Uart_Rx_Reg(space, regs.Uart_Rx_Addr),
       Interrupt_Reg(space, regs.Int_Addr),
       TriggerArm_Reg(space, regs.Arm_Addr),
       RomIntf(I2CReg),
       Rom_I2C(RomIntf, I2CDevice::SubAddress, IeeeI2C_SubAddr)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7Ieee1588
//---------------------------------------------------------------------------

K7Ieee1588::~K7Ieee1588()
{
}

//---------------------------------------------------------------------------
//  K7Ieee1588::OpenHardware()
//---------------------------------------------------------------------------

void  K7Ieee1588::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Uart(0).SetBaud(115200);

}

//---------------------------------------------------------------------------
//  K7Ieee1588::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7Ieee1588::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  K7Ieee1588::Enable() - enable IEEE
//---------------------------------------------------------------------------

void  K7Ieee1588::Enable(bool state)
{
    if (!DeviceEnabled())
        return;

    ControlReg.Enable.Value(state);

}

//---------------------------------------------------------------------------
//  K7Ieee1588::Reset() - reset IEEE PLL
//---------------------------------------------------------------------------

void  K7Ieee1588::Reset()
{
    if (!DeviceEnabled())
        return;

    ControlReg.PllReset.Value(false);
    ControlReg.PllReset.Value(true);
    ControlReg.PllReset.Value(false);

}

//---------------------------------------------------------------------------
//  K7Ieee1588::InitIeee1588() -
//---------------------------------------------------------------------------

void K7Ieee1588::InitIeee1588()
{
    Debug.Log("K7Ieee1588::InitIeee1588()");

    bool locked = PllLocked();
    if (!locked)
        {
        Debug.Log("...PLL not locked, reset");
        Reset();

        Innovative::Sleep(5);

        locked = PllLocked();
        if (!locked)
            {
            Debug.Log("ERROR: PLL still not locked");
            return;
            }
        }

    Debug.Log("IEEE Enabled...");
    Enable(true);
}

//---------------------------------------------------------------------------
//  K7Ieee1588::ClockID() - return full ClockID number
//---------------------------------------------------------------------------

ii64  K7Ieee1588::ClockID()
{
    if (!DeviceEnabled())
        return 0;

    ii64 result = ClockId_Hi_Reg.Value();
    result <<= 32;
    result |= ClockId_Lo_Reg.Value();
    return result;
}


//---------------------------------------------------------------------------
//  K7Ieee1588::PllLocked() - return PLL locked bit
//---------------------------------------------------------------------------

bool  K7Ieee1588::PllLocked()
{
    if (!DeviceEnabled())
        return false;

    return ControlReg.PllLock.Value();
}

//---------------------------------------------------------------------------
//  K7Ieee1588::Linked() - return Linked bit
//---------------------------------------------------------------------------

bool  K7Ieee1588::Linked()
{
    if (!DeviceEnabled())
        return false;

    return ControlReg.Linked.Value();
}

//---------------------------------------------------------------------------
//  K7Ieee1588::InSync() - return InSync bit
//---------------------------------------------------------------------------

bool  K7Ieee1588::InSync()
{
    if (!DeviceEnabled())
        return false;

    return ControlReg.InSync.Value();
}

//---------------------------------------------------------------------------
//  K7Ieee1588::Gigabit() - return InSync bit
//---------------------------------------------------------------------------

bool  K7Ieee1588::Gigabit()
{
    if (!DeviceEnabled())
        return false;

    return ControlReg.Gigabit.Value();
}

//---------------------------------------------------------------------------
//  K7Ieee1588::TestReg() - read a register
//---------------------------------------------------------------------------

unsigned int  K7Ieee1588::TestReg(int addr)
{
    if (!DeviceEnabled())
        return 0;

    switch (addr)
        {
        default:
        case 0:
            return ControlReg.Value();
        case 1:
            return ClockId_Lo_Reg.Value();
        case 2:
            return ClockId_Hi_Reg.Value();
        case 3:
            return TimeReg.Value();
        case 4:
            return I2CReg.Value();
        case 5:
            return Uart_Ctl_Reg.Value();
        case 6:
            return Uart_Tx_Reg.Value();
        case 7:
            return Uart_Rx_Reg.Value();
        case 8:
            return Interrupt_Reg.Value();
        case 9:
            return TriggerArm_Reg.Value();
        }
}

//---------------------------------------------------------------------------
//  K7Ieee1588::TestReg() - read a register
//---------------------------------------------------------------------------

void  K7Ieee1588::TestReg(int addr, unsigned int value)
{
    if (!DeviceEnabled())
        return;

    switch (addr)
        {
        default:
        case 0:
            return ControlReg.Value(value);
        case 1:
            return ClockId_Lo_Reg.Value(value);
        case 2:
            return ClockId_Hi_Reg.Value(value);
        case 3:
            return TimeReg.Value(value);
        case 4:
            return I2CReg.Value(value);
        case 5:
            return Uart_Ctl_Reg.Value(value);
        case 6:
            return Uart_Tx_Reg.Value(value);
        case 7:
            return Uart_Rx_Reg.Value(value);
        case 8:
            return Interrupt_Reg.Value(value);
        case 9:
            return TriggerArm_Reg.Value(value);
        }
}

//---------------------------------------------------------------------------
//  K7Ieee1588::Rom() - read the rom for a byte
//---------------------------------------------------------------------------

unsigned char  K7Ieee1588::Rom(int addr)
{
    if (!DeviceEnabled())
        return 0;

    I2CReg.DeviceEnable.Value(true);
    unsigned char rtn = Rom_I2C.Byte(addr);
    I2CReg.DeviceEnable.Value(false);
    return rtn;
}

//---------------------------------------------------------------------------
//  K7Ieee1588::Rom() - write the rom with a byte
//---------------------------------------------------------------------------

void  K7Ieee1588::Rom(int addr, unsigned char value)
{
    if (!DeviceEnabled())
        return;

    I2CReg.DeviceEnable.Value(true);
    Rom_I2C.Byte(addr, value);
    I2CReg.DeviceEnable.Value(false);

}

//===========================================================================
//  CLASS IRIG_RealTimeClock -- IRIG Support "SubDevice"  -
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IRIG_RealTimeClock
//---------------------------------------------------------------------------

IRIG_RealTimeClock::IRIG_RealTimeClock(IRequires_Nothing * mdc, IAddressingSpace & space,
                       MapRegisters & regs)
    :  MDC(mdc),
       DateReg(space, regs.Date_Addr),
       //FracSecReg(space, regs.FractionalSecs_Addr),
       DdsControlReg(space, regs.DdsControl_Addr),
       DdsSpiControlReg(space, regs.DdsSpiControl_Addr),
       DdsSpiStatusReg(space, regs.DdsSpiControl_Addr),
       DdsSpiDataReg(space, regs.DdsSpiData_Addr),
       DesiredCountReg(space, regs.ClocksPerSymbol_Addr),
       DdsTuningReg(space, regs.DdsTuning_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class IRIG_RealTimeClock
//---------------------------------------------------------------------------

IRIG_RealTimeClock::~IRIG_RealTimeClock()
{
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::OpenHardware()
//---------------------------------------------------------------------------

void  IRIG_RealTimeClock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Init();

}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::ConfigureHardware()
//---------------------------------------------------------------------------

void  IRIG_RealTimeClock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::Init() --  Set up IRIG
//---------------------------------------------------------------------------

void  IRIG_RealTimeClock::Init()
{
    // Turn off DDS loop
    DdsControlReg.LoopEnable.Value(false);

    Sleep(1);

    // Pulse DDS Reset
    DdsControlReg.Reset.Value(true);
    Sleep(1);
    DdsControlReg.Reset.Value(false);

    Sleep(1);
    // Spi 0 - addr 0 = 0x200 -> SDIO input only
    Spi(0x0, 0x200);

    Sleep(1);
    // Spi 1 - addr 1 = multiplier | VCORange
    Spi(0x1, 0xa4);

    Sleep(1);
    // Spi 2 - addr 2 = 0)
    Spi(0x2, 0x0);

    Sleep(1);
    // Spi 3 - addr 3 = 0)
    Spi(0x3, 0x0);

    Sleep(1);
    // Spi 4 - addr 4 = FFrequency * MaxIntAsFloat / SysClk
    //                  10e6       * 4294967296    / 400e6)
    Spi(0x4, 0x06666666);

    Sleep(1);
    // Spi 5 - addr 5 = 0)
    Spi(0x5, 0x0);

    Sleep(1);
    // Desired Count Register
    //    100k cycles desired => 10 MHz)
    DesiredCountReg.Count.Value(0x1869f);

    Sleep(1);
    // Pulse IOUPDATE
    DdsControlReg.IoUpdate.Value(true);
    Sleep(1);
    DdsControlReg.IoUpdate.Value(false);

    Sleep(1);
    // Set initial Frequency Tuning Word
    DdsTuningReg.Value(0x06666666);   // same as SPI 4 value always?

    Sleep(50);

    // Turn on DDS loop
    DdsControlReg.LoopEnable.Value(true);
    Sleep(1);

}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::SignalPresent()  --  IRIG signal present
//---------------------------------------------------------------------------

bool IRIG_RealTimeClock::SignalPresent()
{
    return DateReg.WatchdogStatus.Value()==false;
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::SignalPresent()  --  DDS locked on IRIG signal
//---------------------------------------------------------------------------

bool IRIG_RealTimeClock::Locked()
{
    return DdsControlReg.Locked.Value();
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::Spi()  --  SPI Write
//---------------------------------------------------------------------------

void IRIG_RealTimeClock::Spi(int addr, unsigned int value)
{
    SpiReady();

    // Write to D/A register via SPI register
    SetupSpiWrite(addr, value);

    SpiNotReady();
    SpiReady();

    std::stringstream msg;
    msg << std::hex << " IRIG Spi(0x" << addr <<
             ", 0x" << value << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::SpiReady()
//---------------------------------------------------------------------------

void IRIG_RealTimeClock::SpiReady()
{
    // Wait up to 1000 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(200);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " IRIG Spi Ready Failure (>5us)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::SpiNotReady()
//---------------------------------------------------------------------------

void IRIG_RealTimeClock::SpiNotReady()
{
    // Wait up to 1000 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(200);
        }
    // Gave up
    //std::stringstream msg;
    //msg << std::hex << Tag << "IRIG Spi NOT Ready Failure (>5us)";
    //Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::SetupSpiWrite() --  SPI Write Setup
//---------------------------------------------------------------------------

void  IRIG_RealTimeClock::SetupSpiWrite(int addr, unsigned int value)
{
    // Set up a SPI write
    DdsSpiControlReg.Address.Value(addr);
    DdsSpiDataReg.Value(value);
    DdsSpiControlReg.Direction.Value(spiWrite);
    DdsSpiControlReg.Apply();
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  IRIG_RealTimeClock::ReadSpiReadyBit()
{
    return DdsSpiStatusReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  IRIG_RealTimeClock::DumpSpiRegister()
{
    return DdsSpiDataReg.Value();
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::Raw() --  return raw register data
//---------------------------------------------------------------------------

unsigned int  IRIG_RealTimeClock::Raw()
{
    return DateReg.Value();
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::Rtc() --  return RtcData information
//---------------------------------------------------------------------------

IRIG_RealTimeClock::RtcData  IRIG_RealTimeClock::Rtc()
{
    RtcData  result;
    Decode(result);
    return result;
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::Time() --  return time_t information
//---------------------------------------------------------------------------

time_t  IRIG_RealTimeClock::Time(unsigned int year)
{
    RtcData  result;
    Decode(result);

    struct tm  time;
    time.tm_sec = result.seconds;
    time.tm_min = result.minutes;
    time.tm_hour = result.hours;
    time.tm_yday = result.days;
    time.tm_year = year;

    time_t output = mktime(&time);
    return output;
}

//---------------------------------------------------------------------------
//  IRIG_RealTimeClock::Decode() --  Decode and convert IRIG time register
//---------------------------------------------------------------------------

void  IRIG_RealTimeClock::Decode(RtcData & rtc)
{
    // Read once to "Lock In" the time
    MemoryBitManipulator Time(DateReg.Value());
    //
    // Do BCD conversion by unpacking each digit separately...
    unsigned int Seconds_lo = Time.Field(1, 4);
    unsigned int Seconds_hi = Time.Field(5, 3);
    rtc.seconds = Seconds_hi*10 + Seconds_lo;

    unsigned int Minutes_lo = Time.Field(8, 4);
    unsigned int Minutes_hi = Time.Field(12, 3);
    rtc.minutes = Minutes_hi*10 + Minutes_lo;

    unsigned int Hours_lo = Time.Field(15, 4);
    unsigned int Hours_hi = Time.Field(19, 2);
    rtc.hours = Hours_hi*10 + Hours_lo;

    unsigned int Days_lo  = Time.Field(21, 4);
    unsigned int Days_mid = Time.Field(25, 4);
    unsigned int Days_hi  = Time.Field(29, 2);
    rtc.days = Days_hi*100 + Days_mid*10 + Days_lo;

}


//===========================================================================
//  CLASS AtroposLedReset -- Atropos LED/Reset "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class AtroposLedReset
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class AtroposLedReset
//---------------------------------------------------------------------------

AtroposLedReset::AtroposLedReset(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), ControlReg(space, regs.LedReset_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class AtroposLedReset
//---------------------------------------------------------------------------

AtroposLedReset::~AtroposLedReset()
{
}

//---------------------------------------------------------------------------
//  AtroposLedReset::OpenHardware()
//---------------------------------------------------------------------------

void  AtroposLedReset::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  AtroposLedReset::ConfigureHardware()
//---------------------------------------------------------------------------

void  AtroposLedReset::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  AtroposLedReset::StopHardware()
//---------------------------------------------------------------------------

void  AtroposLedReset::StopHardware()
{
}

//---------------------------------------------------------------------------
//  AtroposLedReset::AppReset()
//---------------------------------------------------------------------------

void  AtroposLedReset::AppReset()
{
    Debug.Log("App Reset Applied");
    //
    //  Reset User Logic
    ControlReg.Reset.Set();
    Sleep(1);
    ControlReg.Reset.Clear();
    Sleep(1);

}

//---------------------------------------------------------------------------
//  AtroposLedReset::BlueLed()
//---------------------------------------------------------------------------

void AtroposLedReset::BlueLed(bool state)
{
    ControlReg.BlueLed.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  AtroposLedReset::BlueLed()
//---------------------------------------------------------------------------

bool AtroposLedReset::BlueLed() const
{
    return ControlReg.BlueLed.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  AtroposLedReset::GreenLed()
//---------------------------------------------------------------------------

void AtroposLedReset::GreenLed(bool state)
{
    ControlReg.GreenLed.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  AtroposLedReset::GreenLed()
//---------------------------------------------------------------------------

bool AtroposLedReset::GreenLed() const
{
    return ControlReg.GreenLed.Value() ? true : false;
}

//===========================================================================
//  CLASS SbcDuo_XmcI2C -- Standard bit-banged I2C link "microDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuo_XmcI2C
//---------------------------------------------------------------------------

const int     SbcDuo_XmcI2C_SubAddr  = 0xAA;  // ?

//---------------------------------------------------------------------------
//  constructor for class SbcDuo_XmcI2C
//---------------------------------------------------------------------------

SbcDuo_XmcI2C::SbcDuo_XmcI2C(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       Xmc0Control(space, regs.XMC0_Addr),
       Xmc1Control(space, regs.XMC1_Addr),
       Xmc0Intf(Xmc0Control), Xmc1Intf(Xmc1Control),
       Xmc0_I2C(Xmc0Intf, I2CDevice::SubAddress, SbcDuo_XmcI2C_SubAddr),
       Xmc1_I2C(Xmc1Intf, I2CDevice::SubAddress, SbcDuo_XmcI2C_SubAddr)
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuo_XmcI2C
//---------------------------------------------------------------------------

SbcDuo_XmcI2C::~SbcDuo_XmcI2C()
{
}

//---------------------------------------------------------------------------
//  SbcDuo_XmcI2C::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuo_XmcI2C::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  SbcDuo_XmcI2C::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuo_XmcI2C::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  SbcDuo_XmcI2C::StopHardware()
//---------------------------------------------------------------------------

void  SbcDuo_XmcI2C::StopHardware()
{
}

//---------------------------------------------------------------------------
//  SbcDuo_XmcI2C::XmcPresent() - enable vcxo
//---------------------------------------------------------------------------

bool  SbcDuo_XmcI2C::XmcPresent(unsigned int idx)
{
    bool bit;
    if (idx==0)
        bit = Xmc0Control.Xmc_PresentN.Value();
    else
        bit = Xmc1Control.Xmc_PresentN.Value();

    return (bit==false);  // invert bit
}

//===========================================================================
//  CLASS SbcDuoPeripherals -- Standard bit-banged I2C link "microDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuoPeripherals
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class SbcDuoPeripherals
//---------------------------------------------------------------------------

SbcDuoPeripherals::SbcDuoPeripherals(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), SrcSelector(this),
       TimingControlReg(space, regs.TimingControl_Addr),
       TenGBEthernetCtrlReg(space, regs.X10GbEthernet_Addr),
       CustomIntSourceReg(space, regs.InterruptSourceSel_Addr)//,
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuoPeripherals
//---------------------------------------------------------------------------

SbcDuoPeripherals::~SbcDuoPeripherals()
{
}

//---------------------------------------------------------------------------
//  SbcDuoPeripherals::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuoPeripherals::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Selector().MasterPulsePerSecSource(ISbcDuoSourceSelector::ppsGps);
}

//---------------------------------------------------------------------------
//  SbcDuoPeripherals::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoPeripherals::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  SbcDuoPeripherals::StopHardware()
//---------------------------------------------------------------------------

void  SbcDuoPeripherals::StopHardware()
{
}

//---------------------------------------------------------------------------
//  SbcDuoPeripherals::CustomInterruptSource()
//---------------------------------------------------------------------------

void  SbcDuoPeripherals::CustomInterruptSource(unsigned int src)
{
    CustomIntSourceReg.Source.Value(src);
}

//---------------------------------------------------------------------------
//  SbcDuoPeripherals::CustomInterruptSource()
//---------------------------------------------------------------------------

unsigned int  SbcDuoPeripherals::CustomInterruptSource()
{
    return CustomIntSourceReg.Source.Value();
}

//===========================================================================
//  CLASS QSpi_RomControl -- QSpi Programming Shared Code
//===========================================================================
//---------------------------------------------------------------------------
//  QSpi_RomControl::SetBlockProtection() -- Impose all block protection
//---------------------------------------------------------------------------

void  QSpi_RomControl::SetBlockProtection()
{
    FlashOp(0x06);                              // WREN - enable writing
    FlashOp(0x01, 0x00, Map.Protect_ON_Value);  // WRSR (write status register)
    //    SRWD is 1, using the hardware protect signal.
    Flash_NotBusy_T_ms(1000);  // make sure RDSR busy status is complete
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::ReleaseBlockProtection() -- Remove all block protection
//---------------------------------------------------------------------------

void  QSpi_RomControl::ReleaseBlockProtection()
{
    FlashOp(0x06);                               // WREN - enable writing
    FlashOp(0x01, 0x00, Map.Protect_OFF_Value);  // WRSR (write status register)
    // ...the above sets BP0-BP3 to 0, removing all protection
    //    SRWD is 1, using the hardware protect signal.
    Flash_NotBusy_T_ms(1000);  // make sure RDSR busy status is complete
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::IsBlockProtectionReleased() -- Check block protection
//---------------------------------------------------------------------------

bool  QSpi_RomControl::IsBlockProtectionReleased()
{
    unsigned int rdsr = Rdsr();

    if ((rdsr & Map.Protect_Check_Mask)==0x80)
        {
        return true;  // unprotected
        }
    else
        {
        return false;  // protected
        }
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::ErasePage() -- erase 1 IDROM page
//---------------------------------------------------------------------------

void  QSpi_RomControl::ErasePage(unsigned int page)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x20, page*Map.Page_Size);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::EraseSector() -- erase 1 IDROM sector
//---------------------------------------------------------------------------

void  QSpi_RomControl::EraseSector(unsigned int sector)
{
    std::stringstream msg;
    msg << std::hex << "QSpi_RomControl::EraseSector:" << sector ;
    TraceDebug(msg.str());

    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0xd8, sector*Map.Sector_Size);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::ReadPage() -- read 1 IDROM page
//---------------------------------------------------------------------------
//  Size argument is in 32 bit words

void  QSpi_RomControl::ReadPage(unsigned int page, int * pkt, unsigned int size)
{
    size_t rpg_limit = std::min<size_t>(static_cast<size_t>(Map.Page_Size/sizeof(int)), size);

    //  Read one 32 bit word at a time into memory
    for (size_t i=0; i<rpg_limit; i++)
        {
        unsigned int addr = static_cast<unsigned int>(page*Map.Page_Size + i*sizeof(int));
        unsigned int value = Read32(addr);
        *pkt++ = value;
        }
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::WritePage() -- write 1 IDROM page
//---------------------------------------------------------------------------
//  Size argument is in 32 bit words

void  QSpi_RomControl::WritePage(unsigned int page, int * pkt, unsigned int size)
{
    size_t wpg_limit = std::min<size_t>(static_cast<size_t>(Map.Page_Size/sizeof(int)), size);

    //  Program one 32 bit word at a time into memory
    for (size_t i=0; i<wpg_limit; i++)
        {
        unsigned int addr =  static_cast<unsigned int>(page*Map.Page_Size + i*sizeof(int));
        Write32(addr, pkt[i]);
        }
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Read32() -- read 1 IDROM page
//---------------------------------------------------------------------------

unsigned int  QSpi_RomControl::Read32(unsigned int addr)
{
    return FlashOp(0x03, addr);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Write32() -- write 1 word
//---------------------------------------------------------------------------

void  QSpi_RomControl::Write32(unsigned int addr, unsigned int value)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x02, addr, value );

    Flash_QuickNotBusy();
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::SetProgramPageMode() -- Set Page Mode bit
//---------------------------------------------------------------------------

void  QSpi_RomControl::SetProgramPageMode(bool state)
{
    WritePageModeBit(state);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::SetPageAddress() -- Set Page Mode Address
//---------------------------------------------------------------------------

void  QSpi_RomControl::SetPageAddress(unsigned int addr)
{
    WriteAddr(addr);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Enqueue32() -- Add data to page mode queue
//---------------------------------------------------------------------------

void  QSpi_RomControl::Enqueue32(unsigned int value)
{
    WriteData(value);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::WaitForPageComplete() -- Wait for write to finish
//---------------------------------------------------------------------------

void  QSpi_RomControl::WaitForPageComplete()
{
    std::stringstream msg;
    msg << std::hex << "QSpi_RomControl::WaitForPageComplete:" /*<< sector*/ ;
    TraceDebug(msg.str());

    // Wait up to 5000 uS for pending flash write to be valid
    Flash_NotBusy_T_us(5000);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::PageProgram() -- Start a page mode access
//---------------------------------------------------------------------------

void  QSpi_RomControl::PageProgram()
{
    std::stringstream msg;
    msg << std::hex << "QSpi_RomControl::PageProgram:" /*<< sector*/ ;
    TraceDebug(msg.str());

    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x2);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Rdsr() -- Flash Operation - Read status register
//---------------------------------------------------------------------------

unsigned int  QSpi_RomControl::Rdsr()
{
//    std::stringstream msg;
//    msg << std::hex << "QSpi_RomControl::Rdsr:" /*<< sector */;
//    TraceDebug(msg.str());

    Flash_Ready_T(2000);

    // Write opcode to trigger the transaction
    WriteOpcode(0x5);

    Flash_ReadValid();

    unsigned int data = ReadData();

//    std::stringstream msg2;
//    msg2 << std::hex << " In Rdsr:"  << data;
//    TraceDebug(msg2.str());

    return data;
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::JdecId() -- Flash Instruction - JDEC-ID
//---------------------------------------------------------------------------

unsigned int  QSpi_RomControl::JdecId()
{
    return FlashOp(0x9F, 0);   // address unused
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Rdid() -- Flash Instruction - RDID
//---------------------------------------------------------------------------

unsigned int  QSpi_RomControl::Rdid()
{
    return FlashOp(0x90, 0);   // address unused
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Reset() -- Flash Instruction - Reset
//---------------------------------------------------------------------------

void  QSpi_RomControl::Reset()
{
    FlashOp(0xF0);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::FlashOp() -- primitive Flash Operation - "command-like" ops
//---------------------------------------------------------------------------

void  QSpi_RomControl::FlashOp(unsigned int opcode)
{
//    std::stringstream msg;
//    msg << std::hex << " FlashOp("  << opcode << ")";
//    TraceDebug(msg.str());

    Flash_Ready();

    // Write opcode to trigger the transaction
    WriteOpcode(opcode);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::FlashOp() -- primitive Flash Operation - "read-like" ops
//---------------------------------------------------------------------------

unsigned int  QSpi_RomControl::FlashOp(unsigned int opcode, unsigned int addr)
{
//    std::stringstream msg;
//    msg << std::hex << " FlashOp("  << opcode << "," << addr << ")";
//    TraceDebug(msg.str());

    Flash_Ready();

    // Write address
    WriteAddr(addr);

    // Write opcode to trigger the transaction
    WriteOpcode(opcode);

    Flash_ReadValid();

    unsigned int data = ReadData();
    return data;
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::FlashOp() -- primitive Flash Operation - "write-like" ops
//---------------------------------------------------------------------------

void  QSpi_RomControl::FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
{
//    std::stringstream msg;
//    msg << std::hex << " FlashOp("  << opcode << "," << addr << "," << data << ")";
//    TraceDebug(msg.str());

    Flash_Ready();

    // Write data
    WriteData(data);

    // Write address
    WriteAddr(addr);

    // Write opcode to trigger the transaction
    WriteOpcode(opcode);

}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_Ready() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_Ready()
{
    Flash_Ready_T(200);
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_Ready_T() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_Ready_T(unsigned int max_wait_us)
{
    const unsigned int reps = 10;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to "max_wait_us" uS for pending flash to be ready
    for (unsigned int i = 0; i < reps; ++i)
        {
        bool ready = ReadReady();
        if (ready)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << " FlashRom Ready Failure (>" << max_wait_us << "us)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_ReadValid() -- wait for flash readvalid indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_ReadValid()
{
    // Wait up to 200 uS for pending flash read to be valid
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool readvalid = ReadDataValid();
        if (readvalid)
            return;
        else
            Innovative::uSleep(20);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Read Valid Failure (>200us)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_NotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_NotBusy()
{
    Flash_NotBusy_T_ms(5000);  // wait up to 5000 ms
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_QuickNotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_QuickNotBusy()
{
    Flash_NotBusy_T_us(200);  // wait up to 200 us
}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_NotBusy_T_ms() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_NotBusy_T_ms(unsigned int max_wait_ms)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_ms/reps;

    // Wait up to  "max_wait_ms" mS for Rdsr bit 0 to clear
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::Sleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << " FlashRom Not Busy Failure (>" << max_wait_ms << "ms)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  QSpi_RomControl::Flash_NotBusy_T_us() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  QSpi_RomControl::Flash_NotBusy_T_us(unsigned int max_wait_us)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to  "max_wait_us" uS for Rdsr bit 0 to clear
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << " FlashRom Not Busy Failure (>" << max_wait_us << "us)";
    TraceDebug(msg.str());

}
//===========================================================================
//  CLASS QSpiFpaLoader -- K7 FPA Loader "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class QSpiFpaLoader
//---------------------------------------------------------------------------

QSpi_RomControl::RomMapInfo  RomMicron_N25Q128_MapInfo =
{
    16,            //  Page_Count (per sector?)
    4096,          //  Page_Size (bytes);
    256*2,         //  Sector_Count (doubled because of golden image reserve)
    65536,         //  Sector_Size (bytes);
    0xFC,          //  Protect_ON_Value;
    0xA0,          //  Protect_OFF_Value;
    0xDC,          //  Protect_Check_Mask;
};

//---------------------------------------------------------------------------
//  constructor for class QSpiFpaLoader
//---------------------------------------------------------------------------

QSpiFpaLoader::QSpiFpaLoader(IRequires_Nothing * mdc, RomType which_rom,
                           IAddressingSpace & space, MapRegisters & regs)
    :  QSpi_RomControl(SelectRomMap(which_rom)),
       MDC(mdc),
       AddressControlReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr),
       Chip(MakeChipObject(which_rom)), RawProm(this),
       Thread( new K7FpaPersonality( Chip ))
{
}

//---------------------------------------------------------------------------
//  destructor for class QSpiFpaLoader
//---------------------------------------------------------------------------

QSpiFpaLoader::~QSpiFpaLoader()
{
    Thread.Shutdown();

    delete Chip;
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::OpenHardware()
//---------------------------------------------------------------------------

void  QSpiFpaLoader::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Thread.OnOpen();
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::ConfigureHardware()
//---------------------------------------------------------------------------

void  QSpiFpaLoader::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::WritePageModeBit() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  QSpiFpaLoader::WritePageModeBit(bool bit)
{
    AddressControlReg.PageMode.Value(bit);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::WriteOpcode() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  QSpiFpaLoader::WriteOpcode(unsigned int opcode)
{
    OpcodeReg.Value(opcode);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::WriteAddr() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  QSpiFpaLoader::WriteAddr(unsigned int addr)
{
    AddressControlReg.Addr.Value(addr);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::WriteData() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  QSpiFpaLoader::WriteData(unsigned int data)
{
    DataReg.Value(data);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::ReadData() -- primitive Flash Operation
//---------------------------------------------------------------------------

unsigned int  QSpiFpaLoader::ReadData()
{
    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::ReadReady() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  QSpiFpaLoader::ReadReady()
{
    bool ready = AddressControlReg.Ready.Value();
    return ready;
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::ReadDataValid() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  QSpiFpaLoader::ReadDataValid()
{
    bool readvalid = AddressControlReg.ReadDataValid.Value();
    return readvalid;
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::TraceDebug() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  QSpiFpaLoader::TraceDebug(const std::string & text)
{
    Debug.Log(text);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MakeChipObject() -- create proper chip object
//---------------------------------------------------------------------------

IK7FpaChip *   QSpiFpaLoader::MakeChipObject(RomType /*which_rom*/)
{
    return new MicronFpaChip(this);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::SelectRomMap() -- create proper rom map
//---------------------------------------------------------------------------

QSpi_RomControl::RomMapInfo  QSpiFpaLoader::SelectRomMap(RomType /*which_rom*/)
{
    return RomMicron_N25Q128_MapInfo;
}

//===========================================================================
//  CLASS QSpiFpaLoader::MicronFpaChip  -- CPLD Hardware Interface
//===========================================================================
//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::Initialize() -- init chip object
//---------------------------------------------------------------------------

bool  QSpiFpaLoader::MicronFpaChip::Initialize()
{
    if (!Owner->DeviceEnabled())
        return false;

    Owner->Reset();

    FDeviceId = Owner->Rdid();

    Owner->SetBlockProtection();
    //Owner->ReleaseBlockProtection();
    FHWProtected = true;
    FProtected = false;

    return true;   // or check if RDID is right?
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::Read32() -- Read 32 bit word
//---------------------------------------------------------------------------

unsigned int  QSpiFpaLoader::MicronFpaChip::Read32(unsigned int addr)
{
    if (!Owner->DeviceEnabled())
        return false;

    return Owner->Read32(addr);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::Write32() -- Write 32 bit word
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::Write32(unsigned int addr, unsigned int value)
{
    if (!Owner->DeviceEnabled())
        return;

     Owner->Write32(addr, value);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::SetProtection() -- Set block protection
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::SetProtection()
{
    Owner->SetBlockProtection();
    FHWProtected = true;   // we might as well trust it.
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::ReleaseProtection() -- Remove block protection
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::ReleaseProtection()
{
    Owner->ReleaseBlockProtection();
    if (Owner->IsBlockProtectionReleased())
        FHWProtected = false;
    else
        FHWProtected = true;
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::EraseSector() -- Erase a sector
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::EraseSector(unsigned int sector)
{
    if (!Owner->DeviceEnabled())
        return;

    Owner->EraseSector(sector);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::SetProgramPageMode() -- Prep for Page mode write
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::SetProgramPageMode(bool state)
{
    Owner->SetProgramPageMode(state);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::SetPageAddress() -- load address
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::SetPageAddress(unsigned int addr)
{
    Owner->SetPageAddress(addr);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::Enqueue32() -- load data
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::Enqueue32(unsigned int value)
{
    Owner->Enqueue32(value);
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::WaitForPageComplete() -- check for completion
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::WaitForPageComplete()
{
    Owner->WaitForPageComplete();
}

//---------------------------------------------------------------------------
//  QSpiFpaLoader::MicronFpaChip::PageProgram() -- start page programming process
//---------------------------------------------------------------------------

void  QSpiFpaLoader::MicronFpaChip::PageProgram()
{
    Owner->PageProgram();
}

//===========================================================================
//  CLASS LTC2494 --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class LTC2494
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  LTC2494::LTC2494() -- Constructor
//---------------------------------------------------------------------------

const unsigned int MaxChannels = 16;     // maxixmum channels can be used
const float CountsFS = 65536;            // counts full scale

LTC2494::LTC2494(IAddressingSpace &space,LTC2494SpiDevice::Info &regs, std::string tag)
    :  Spi(space,regs,tag)
{

}

//---------------------------------------------------------------------------
//  LTC2494::~LTC2494() -- Destructor
//---------------------------------------------------------------------------

LTC2494::~LTC2494()
{

}

//---------------------------------------------------------------------------
//  LTC2494::ResizeCaches() -- properly resize measurement caches
//---------------------------------------------------------------------------

void LTC2494::ResizeCaches()
{
    MeasureCache.resize(MeasureCount());
    std::fill(MeasureCache.begin(), MeasureCache.end(), 0.0);
    PowerMeasureCache.resize(PowerMeasureCount());
    std::fill(PowerMeasureCache.begin(), PowerMeasureCache.end(), 0.0);
}

//---------------------------------------------------------------------------
//  LTC2494::GetScaleTable() -- read scale table entry
//---------------------------------------------------------------------------

double  LTC2494::GetScaleTable(unsigned int idx)
{
    return ScaleTable[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetSignalName() -- read Signal name entry
//---------------------------------------------------------------------------

std::string  LTC2494::GetSignalName(unsigned int idx)
{
    return SignalNameStr[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetNominalV() -- read NominalV table entry
//---------------------------------------------------------------------------

double  LTC2494::GetNominalV(unsigned int idx)
{
    return NominalV[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetMaxV() -- read NominalV table entry
//---------------------------------------------------------------------------

double  LTC2494::GetMaxV(unsigned int idx)
{
    return MaxV[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetMinV() -- read NominalV table entry
//---------------------------------------------------------------------------

double  LTC2494::GetMinV(unsigned int idx)
{
    return MinV[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetNominalAdcV() -- read NominalAdcV table entry
//---------------------------------------------------------------------------

double  LTC2494::GetNominalAdcV(unsigned int idx)
{
    return NominalAdcV[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetSignalUnits() -- read Signal Unit entry
//---------------------------------------------------------------------------

std::string  LTC2494::GetSignalUnits(unsigned int idx)
{
    return SignalUnitsStr[idx];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetCurrentPower() -- read PowerMeasureCache entry
//---------------------------------------------------------------------------

double  LTC2494::GetCurrentPower(unsigned int power_ch)
{
    return PowerMeasureCache[power_ch];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::GetCurrentMeasure() -- read  entry
//---------------------------------------------------------------------------

double  LTC2494::GetCurrentMeasure(unsigned int ch)
{
    return MeasureCache[ch];  // range check?
}

//---------------------------------------------------------------------------
//  LTC2494::ReadSpi() -- read data with spi device
//---------------------------------------------------------------------------

int LTC2494::ReadSpi(unsigned int address)
{
    int Data;
    Data = Spi.Spi(address);

    int FinalData;
    FinalData = ExtractData(Data);

    return FinalData;
}

//---------------------------------------------------------------------------
//  LTC2494::ExtractData() -- extract the bits needed to represent data
//---------------------------------------------------------------------------

int LTC2494::ExtractData(int data)
{
    MemoryBitManipulator bitM(data);

    unsigned int NewVal = bitM.Field(4,16);
    if(bitM.Bit(20) == true)
        NewVal = NewVal - 65535; //ground can be either 0 or 0xffff in either cases we need 0
                                 //for that reason we check if the sign bit is true to rescale
    return NewVal;
}


//---------------------------------------------------------------------------
//  LTC2494::ReadTemperature() -- reading the part temperature
//---------------------------------------------------------------------------

double LTC2494::ReadTemperature()
{
    unsigned int addr;
    addr = GenerateAddress(TemperatureConversion, 0/*Any Ch*/);

    int data;
    data = ReadSpi(addr);

    double result;
    result = Measure(data, TemperatureConversion);

    return result;
}

//---------------------------------------------------------------------------
//  LTC2494::ReadChannel() -- reading the channel data
//---------------------------------------------------------------------------

double LTC2494::ReadChannel(unsigned int ch)
{
    unsigned int addr;
    addr = GenerateAddress(ChannelConversion, ch);

    int data;
    data = ReadSpi(addr);

    double result;
    result = Measure(data, ChannelConversion,ch);

    MeasureCache[ch] = result;   // cache result
    return result;
}


//---------------------------------------------------------------------------
//  LTC2494::CalculatePower() -- recalculate all powers based on cache
//---------------------------------------------------------------------------

void LTC2494::CalculatePower()
{
    for (size_t i=0; i<PowerMeasureCache.size(); i++)
        PowerMeasureCache[i] = PowerCalculation(i);
}

//---------------------------------------------------------------------------
//  LTC2494::Measure() -- measure the voltage or the current
//---------------------------------------------------------------------------

double LTC2494::Measure(int data, ConversionType type, unsigned int ch)
{
    double result = 0;

    if(type == ChannelConversion)
    {
        if(ScaleTable[ch] < 0) //Negative voltage calculation
        {
            result = NegativeMeasure(data,ch);
        }
        else                   //Positive voltage calculation
        {
            result = PositiveMeasure(data,ch);
        }
    }
    else if(type == TemperatureConversion)
    {
        result = (0.204 * data) - 273; //Kelvin to celsius
    }

    return result;
}

//---------------------------------------------------------------------------
//  LTC2494::PositiveMeasure() -- measures the positive voltage/current
//---------------------------------------------------------------------------

double LTC2494::PositiveMeasure(int data, unsigned int ch)
{
    double result = 0;
    if(ScaleTable[ch] >= 0)                   //Positive voltage calculation
    {
        result = (ScaleTable[ch] * (data/CountsFS));
    }

    return result;
}

//---------------------------------------------------------------------------
//  LTC2494::Measure() -- measures the negative voltage/current
//---------------------------------------------------------------------------

double LTC2494::NegativeMeasure(int data, unsigned int ch)
{
    double result = 0;

    if(ScaleTable[ch] < 0) //Negative voltage calculation
    {
        result = (ScaleTable[ch] * ((CountsFS - data)/CountsFS));
    }

    return result;
}
//---------------------------------------------------------------------------
//  LTC2494::GenerateAddress() -- generating the outgoing address
//---------------------------------------------------------------------------

unsigned int LTC2494::GenerateAddress(ConversionType type, unsigned int ch)
{

    //Setting up the Measurement bit.
    //rest of the bits must be already init in any derived class
    //or by default it's zero because they are boolean variables

	if (type)
		ConfigBits.IM = true;
	else
		ConfigBits.IM = false;

   // ConfigBits.IM  = type;  //temp = 1 channel = 0

    // initialization of my address based on the bits and channel address
    MemoryBitManipulator Addr(0);

    Addr.Bit(15,1);                                          //bit0: always 1
    Addr.Bit(14,0);                                          //bit1: always 0
    Addr.Bit(13,ConfigBits.EN);                              //bit2: EN
    Addr.Field(8,5,(ChannelMuxAddressTable[ch] & 0x1f));     //bit3-7: channel mux address
    Addr.Bit(7,ConfigBits.EN2);                              //bit8: EN2
    Addr.Bit(6,ConfigBits.IM);                               //bit9: Measurement
    Addr.Bit(5,ConfigBits.FA);                               //bit10: Rejection mode
    Addr.Bit(4,ConfigBits.FB);                               //bit11: Rejection mode
    Addr.Bit(3,ConfigBits.SPD);                              //bit12: Speed mode
    Addr.Bit(2,ConfigBits.GS2);                              //bit13: gain
    Addr.Bit(1,ConfigBits.GS1);                              //bit14: gain
    Addr.Bit(0,ConfigBits.GS0);                              //bit15: gain

    return Addr.Value();
}

//---------------------------------------------------------------------------
//  LTC2494::MapChannels() -- give address to the channels based on
//                            channel type
//---------------------------------------------------------------------------

void LTC2494::MapChannels()
{
   unsigned int idx = 0;
   unsigned int ch  = 0;

    while(idx < MaxChannels)
        {
        MemoryBitManipulator MuxAddr(0);

        switch (ChannelTypeTable[ch])
            {
            case SingleEnded :
                {
                MuxAddr.Bit(4,1);            //single ended bit set to 1

                if (idx % 2 == 0)
                    MuxAddr.Bit(3,0);           //Odd bit set to zero
                else
                    MuxAddr.Bit(3,1);           //odd bit set to one

                MuxAddr.Field(0,3,(idx>>1)); //set the rest of the bits

                idx+=1;
                break;
                }
            case Differential :
                {
                if (idx % 2 != 0)
                    idx+=1;    //Differential always start on positive bit(doc)

                MuxAddr.Bit(4,0);           //set the single ended bit to 0
                MuxAddr.Bit(3,1);           //set the odd/sign bit to 0

                MuxAddr.Field(0,3,idx);     //set the rest bits

                idx+=2;
                break;
                }
            case InvertedDifferential :
                {
                if(idx % 2 != 0)
                    idx+=1;    //Differential always start on positive bit(doc)

                MuxAddr.Bit(4,0);           //set the single ended bit to 0
                MuxAddr.Bit(3,1);           //set the odd/sign bit to 1

                MuxAddr.Field(0,3,idx);     //set the rest of the bits
                idx+=2;
                break;
                }
            case Unused :
                {
                MuxAddr.Field(0,5,0);      //not used = 0
                idx+=1;
                break;
                }
            } //end switch

        ChannelMuxAddressTable.push_back(MuxAddr.Value());
        ch+=1; //moving to the next channel
        } //end the while

}

//===========================================================================
//  CLASS SelfMeasurementDevice -- Internal test point measurement "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SelfMeasurementDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  SelfMeasurementDevice::SelfMeasurementDevice() -- Constructor
//---------------------------------------------------------------------------

SelfMeasurementDevice::SelfMeasurementDevice(IRequires_LTC2494Device * mdc)
    : MDC(mdc)
{

}


//---------------------------------------------------------------------------
//  SelfMeasurementDevice::~SelfMeasurementDevice() -- Destructor
//---------------------------------------------------------------------------

SelfMeasurementDevice::~SelfMeasurementDevice()
{

}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::ConstructCustomParts() --
//---------------------------------------------------------------------------

void SelfMeasurementDevice::ConstructCustomParts()
{
    SelfMeasure = MDC->MakeLTC2494();

    SelfMeasure->ResizeCaches();

    SelfMeasure->ConfigureBoardDefaults();  // set up custom constant arrays
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::OpenHardware() --
//---------------------------------------------------------------------------

void SelfMeasurementDevice::OpenHardware()
{
    SelfMeasure->MapChannels();
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::ConstructCustomParts() -- read data with spi device
//---------------------------------------------------------------------------

void SelfMeasurementDevice::ConfigureHardware()
{

}

/*//---------------------------------------------------------------------------
//  SelfMeasurementDevice::ReadChannel() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::ReadChannel(unsigned int ch)
{
    return SelfMeasure->ReadChannel(ch);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::ReadTemperature() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::ReadTemperature()
{
     return SelfMeasure->ReadTemperature();
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetScaleTable() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::GetScaleTable(unsigned int idx)
{
    return SelfMeasure->GetScaleTable(idx);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetSignalName() --
//---------------------------------------------------------------------------

std::string SelfMeasurementDevice::GetSignalName(unsigned int idx)
{
    return SelfMeasure->GetSignalName(idx);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetNominalV() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::GetNominalV(unsigned int idx)
{
    return SelfMeasure->GetNominalV(idx);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetMinV() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::GetMinV(unsigned int idx)
{
    return SelfMeasure->GetMinV(idx);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetMaxV() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::GetMaxV(unsigned int idx)
{
    return SelfMeasure->GetMaxV(idx);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetNominalAdcV() --
//---------------------------------------------------------------------------

double SelfMeasurementDevice::GetNominalAdcV(unsigned int idx)
{
    return SelfMeasure->GetNominalAdcV(idx);
}

//---------------------------------------------------------------------------
//  SelfMeasurementDevice::GetSignalUnits() --
//---------------------------------------------------------------------------

std::string SelfMeasurementDevice::GetSignalUnits(unsigned int idx)
{
    return SelfMeasure->GetSignalUnits(idx);
}
*/
}  // namespace Innovative



