// K7SubDevice_Misc_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef K7SubDevice_Misc_MbH
#define K7SubDevice_Misc_MbH

#include <X6Support_Mb.h>
#include <K7Support_Mb.h>
#include <FmcSupport_Mb.h>
#include <VpxSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <ProcessEvents_Mb.h>
#include <list>
#include <SoftwareTimer_Mb.h>
#include <LMK04800_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <X6Idrom_Mb.h>
#include <DioPortIntf_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif


//===========================================================================
//  CLASS K7SysInformation -- K7 information "SubDevice"
//===========================================================================

class K7SysInformation : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS K7SysInformation::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   FpgaPower_Addr;
        unsigned int   SsdPower_Addr;
        unsigned int   K7Config_Addr;
    };

    //
    //  Member Functions
    K7SysInformation(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7SysInformation();

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysInformation::FpgaPowerRegister  --  Power status bits
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class FpgaPowerRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        PowerGood_1v0;
        RegisterBit        PowerGood_1v2;
        RegisterBit        PowerGood_1v5;
        RegisterBit        PowerGood_1v8;
        RegisterBit        PowerGood_2v5;
        RegisterBit        PowerGood_Ddr3;
        RegisterBit        PowerGood_MgtVcc;
        RegisterBit        PowerGood_MgtVtt;

        FpgaPowerRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              PowerGood_1v0(*this, 0), PowerGood_1v2(*this, 1),
              PowerGood_1v5(*this, 2), PowerGood_1v8(*this, 3),
              PowerGood_2v5(*this, 4), PowerGood_Ddr3(*this, 5),
              PowerGood_MgtVcc(*this, 7), PowerGood_MgtVtt(*this, 8)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysInformation::SsdPowerRegister  --  SSD status bits
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SsdPowerRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Fault_3v3_N;       // 0 = bad
        RegisterBit        Fault_5v0_N;       // 0 = bad
        RegisterBit        PowerGood_N;       // 1 = bad
        //RegisterBit        Present_N;       // not connected
        RegisterBit        PowerEnable_N;     // 0 = enable

        SsdPowerRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Fault_3v3_N(*this, 0), Fault_5v0_N(*this, 1),
              PowerGood_N(*this, 2), //Present_N(*this, 3),
              PowerEnable_N(*this, 4)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysInformation::K7ForwardRegister  --  K7 status bits
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class K7ForwardRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Done;
        RegisterBit        InitB;
        RegisterBit        ProgB;
        RegisterBit        ResetN;
        RegisterBit        ThermalAlert;
        RegisterBit        K7ThermalAlert;

        K7ForwardRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Done(*this, 0), InitB(*this, 1),
              ProgB(*this, 2), ResetN(*this, 3),
              ThermalAlert(*this, 4), K7ThermalAlert(*this, 5)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    FpgaPowerRegister       FpgaPowerReg;
    SsdPowerRegister        SsdPowerReg;
    K7ForwardRegister       K7ForwardReg;

};

//===========================================================================
//  CLASS K7SysLcdDisplay -- K7 LCD control "SubDevice"
//===========================================================================

class K7SysLcdDisplay : public IX6SubDevice, public IK7LcdDevice
{
public:
    //=======================================================================
    //  CLASS K7SysLcdDisplay::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   WriteData_Addr;
        unsigned int   ReadData_Addr;
    };

    //enum LCD_Types { tlcd_MultiLine, tlcd_SingleLine}; (in IK7LcdDevice)

    //
    //  Member Functions
    K7SysLcdDisplay(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7SysLcdDisplay();

    //  Primitive LCD commands
    virtual void            ClearDisplay();
    virtual void            Home();
    virtual void            EntryMode(bool inc, bool shift);
    virtual void            OnOff(bool display, bool cursor, bool blink);
    virtual void            Shift(bool displayCursor, bool rightLeft);
    virtual void            Function(bool dataLen, unsigned short fontTable);
    virtual void            CgRamAddr(unsigned short addr);
    virtual void            DdRamAddr(unsigned short addr);
    virtual unsigned char   AddressCounter();
    virtual void            WriteDataCmd(unsigned short data);
    virtual unsigned char   ReadDataCmd();
    virtual void            Emit(unsigned char c);
    virtual void            Type(const std::string & txt);
    virtual unsigned char   Key();
    virtual void            NextLine();

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    void            SelectLcdType(unsigned int NewType);
    void            UpdateLcdType(unsigned int Type);
    virtual void    Init();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysLcdDisplay::WriteDataRegister  --  output register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class WriteDataRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   WriteData;
        RegisterBit        Ready;

        WriteDataRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              WriteData(*this, 0, 10), Ready(*this, 31)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysLcdDisplay::ReadDataRegister  --  output register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ReadDataRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   ReadData;
        RegisterBit        LcdTypeSel;
        RegisterBit        DataValid;

        ReadDataRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ReadData(*this, 0, 8), LcdTypeSel(*this, 30), DataValid(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    WriteDataRegister       WriteDataReg;
    ReadDataRegister        ReadDataReg;
    unsigned int            LcdType;

    // Methods
    bool            Ready();
};

//===========================================================================
//  CLASS K7SysLcdDisplaySL -- K7 LCD control "SubDevice"
//===========================================================================

class K7SysLcdDisplaySL : public IX6SubDevice, public IK7LcdDeviceSL
{
public:
    //=======================================================================
    //  CLASS K7SysLcdDisplaySL::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   I2C_Addr;
    };

    //
    //  Member Functions
    K7SysLcdDisplaySL(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7SysLcdDisplaySL();

    //  Primitive LCD commands
    virtual void            ClearDisplay();
    virtual void            Home();
    virtual void            EntryMode(bool inc, bool shift);
    virtual void            OnOff(bool display, bool cursor, bool blink);
    virtual void            Shift(bool displayCursor, bool rightLeft);
    virtual void            Function(bool dataLen, unsigned short fontTable);
    virtual void            CgRamAddr(unsigned short addr);
    virtual void            DdRamAddr(unsigned short addr);
    virtual void            WriteDataCmd(unsigned int opcode, unsigned short data);
    virtual void            Emit(unsigned char c);
    virtual void            Type(const std::string & txt);

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    Init();

private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysLcdDisplaySL::I2CRegisterSL  --  output register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class I2CRegisterSL : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   DataOut;
        RegisterBit   ClockOut;
        RegisterBit   ClockIn;

        I2CRegisterSL(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              DataOut(*this, 0), ClockOut(*this, 1), ClockIn(*this, 3)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysLcdDisplaySL::K7SysLcdDisplaySLI2C_Impl  -- K7SysLcdDisplaySL I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class K7SysLcdDisplaySLI2C_Impl : public I2CDeviceIntf
    {
    public:
       K7SysLcdDisplaySLI2C_Impl(I2CRegisterSL & reg)
            :  Reg(reg)
            {}
        virtual ~K7SysLcdDisplaySLI2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.ClockOut.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.ClockIn.Value();  }
        virtual void    SetData(int state)
            {  Reg.DataOut.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return 0; /*Reg.DataIn.Value(); */  }

    private:
        I2CRegisterSL &  Reg;
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    I2CRegisterSL        I2CReg;
    //
    //  Interface Impls
    K7SysLcdDisplaySLI2C_Impl   SLIntf;

    //  I2C Bus Devices
    I2CDeviceN           SL_I2C;
};

//===========================================================================
//  CLASS K7SysExtIo -- K7 External Io "SubDevice"
//===========================================================================
/* COMMENTED OUT DUE TO REDUNDANCY

class K7SysExtIo : public IX6SubDevice, public IK7LedBtnControl
{
public:
    //=======================================================================
    //  CLASS K7SysExtIo::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   ExtIo_Addr;
    };

    //
    //  Member Functions
    K7SysExtIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7SysExtIo();

    // Properties
    virtual void    LedOnOff(bool one, bool two, bool three, bool four);
    virtual void    ButtonOnOff(bool one, bool two, bool three, bool four, bool five, bool six);

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysExtIo::ExtIoRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ExtIoRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   ExtLED;
        RegisterBitGroup   Button;

        ExtIoRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ExtLED(*this, 0, 4), Button(*this, 4, 6)
        {  }
    };


private:
    IRequires_Nothing *     MDC;
    ExtIoRegister           ExtIoReg;
};
*/
//===========================================================================
//  CLASS K7ButtonManager -- Button polling manager
//===========================================================================

class K7ButtonManager
{
public:
    //
    // Ctor
    K7ButtonManager(unsigned int compared=2);

    //  Member Functions
    void    Reset();
    void    Monitor(int state);

    // Notification events
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>    OnStateChange;

private:
    // Data
    typedef std::list<int>  IntList;
    IntList State;

    unsigned int            Compared;
};

//===========================================================================
//  CLASS K7SysLedReset -- K7 LED/Reset register "SubDevice"
//===========================================================================

class K7SysLedReset : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS K7SysExtIo::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   LedReset_Addr;
        unsigned int   ExtLedButton_Addr;
        unsigned int   ExtLedDirection_Addr;
    };

    //
    //  Member Functions
    K7SysLedReset(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7SysLedReset();

    // Notification events
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent>    OnButtonChange;

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StopHardware();

    // Mathods
    virtual void    AppReset();
    // Buttons
    virtual int     Button() const;

    // Properties
    virtual void    BlueLed(bool state);
    virtual bool    BlueLed() const;

    virtual void    GreenLed(bool state);
    virtual bool    GreenLed() const;

    virtual void    ExternalLed(int mask);
    virtual int     ExternalLed() const;
    virtual void    LedOnOff(bool one, bool two, bool three, bool four);
	
	//  Digital General Purpose Io
    virtual IBitManipulator &   ExtIoData();

    virtual void 			    ExtIoConfig(unsigned int);
    virtual unsigned int 	    ExtIoConfig() const;

    virtual void                ExtIoOutputEnable(int output_bit, bool output_enabled);
    virtual bool                ExtIoOutputEnable(int output_bit) const;


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysExtIo::ExtIoRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class LedResetRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Reset;
        RegisterBit        BlueLed;
        RegisterBit        GreenLed;

        LedResetRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Reset(*this, 0), BlueLed(*this, 1), GreenLed(*this, 2)
        {  }
    };

    class ExtLedButtonRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   ExternalLed;
        RegisterBitGroup   Button;

        ExtLedButtonRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ExternalLed(*this, 0, 4), Button(*this, 4, 6)
        {  }
    };


private:
    IRequires_Nothing *     MDC;
    LedResetRegister        ControlReg;
    ExtLedButtonRegister    ExtLedButtonReg;
    Register                ExtOutEnableReg;

    RegisterBitManipulator  ExtIoDataManip;

    K7ButtonManager         Buttons;
    SoftwareTimer           Timer;

    // Event handlers
    void HandleTimer(OpenWire::NotifyEvent &);
    void HandleStateChange(ProcessCompletionEvent & event);
};

//===========================================================================
//  CLASS K7SourceSelectorIo -- K7 Master Source Mux "SubDevice"
//===========================================================================

class K7SourceSelectorIo : public IX6SubDevice, public IK7SourceSelector
{
public:
    //=======================================================================
    //  CLASS K7SourceSelectorIo::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   MuxIo_Addr;
    };

    //
    //  Member Functions
    K7SourceSelectorIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7SourceSelectorIo();

    //  IK7SourceSelector interface
    virtual void  MasterReferenceSource(unsigned int src);
    virtual unsigned int  MasterReferenceSource();

    virtual void  MasterPulsePerSecSource(unsigned int src);
    virtual unsigned int  MasterPulsePerSecSource();

    //  Bugfix Mode
    void  FixReversedSelectorBits(bool do_fix) {  SelectorFixNeeded = do_fix;  }

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SourceSelectorIo::MuxIoRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class MuxIoRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   ClkRefMux;
        RegisterBitGroup   PpsRefMux;

        MuxIoRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ClkRefMux(*this, 0, 2), PpsRefMux(*this, 4, 2)
        {  }
    };


private:
    IRequires_Nothing *     MDC;
    MuxIoRegister           MuxIoReg;
    bool                    SelectorFixNeeded;

    unsigned int            SelectorFixup(unsigned int selector);
};

//===========================================================================
//  CLASS K7UartPart -- one K7 UART
//===========================================================================

class K7UartPart :  public IMulticharUart
{
public:
	//=======================================================================
	//  CLASS K7UartPart::MapRegisters  -- Device Register Offsets
	//=======================================================================
	class MapRegisters
	{
	public:
		unsigned int   UartCtrl_Addr;
		unsigned int   UartTx_Addr;
		unsigned int   UartRx_Addr;

		MapRegisters(unsigned int ctrl, unsigned int tx, unsigned int rx)
			: UartCtrl_Addr(ctrl), UartTx_Addr(tx), UartRx_Addr(rx)
			{}
	};

	//
	//  Member Functions
	K7UartPart(IAddressingSpace & space, MapRegisters regs);
	virtual ~K7UartPart();

    //virtual void           Gpio(unsigned int bit, bool state);
    //bool                   Gpio(unsigned int bit);

	virtual void           Port(unsigned char ch);
	virtual unsigned char  Port();

	virtual void           PutString(const std::string & data);
	virtual void           GetString(std::string & data);

    virtual void           GetFixedString(std::string & data, size_t length);

    virtual bool           FrameError();
    virtual void           ClearFrameError();

    void         SetBaud(unsigned int baudrate);
	unsigned int GetBaud();

	unsigned int GetFifoDepth();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS K7UartPart::UartControlRegister  --   register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class UartControlRegister : public Register
	{
		typedef Register inherited;

    public:
        RegisterBitGroup   BaudDivisor;
		RegisterBitGroup   RxCount;
        RegisterBit        RxReady;

        UartControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              BaudDivisor(*this, 0, 16),
              RxCount(*this, 20, 10),
              RxReady(*this, 31)
                {  }
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7UartPart::UartTxRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartTxRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   TxData;
        RegisterBitGroup   TxCount;
        RegisterBit        TxReady;

        UartTxRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              TxData(*this, 0, 8),
              TxCount(*this, 8, 4),   // field 8, 4 used.
              TxReady(*this, 31)
                {  }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7UartPart::UartRxRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartRxRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   RxData;
        RegisterBitGroup    InterruptThreshold;
        RegisterBit         Error;

        UartRxRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              RxData(*this, 0, 8),
              InterruptThreshold(*this, 8, 4),
              Error(*this, 31)
                {  }

        void ClearFrameError()
            {  Value(Value());  }
    };

private:
	UartControlRegister  UartControlReg;
	UartTxRegister       UartTxReg;
	UartRxRegister       UartRxReg;
	Trace                Debug;

	void  WaitForTxReady();
	void  WaitForRxDataReady();
	unsigned int TxFreeSpace()  {  return 16 - UartTxReg.TxCount.Value();  }
	unsigned int RxDepth()  {  return UartControlReg.RxCount.Value();  }

};

//===========================================================================
//  CLASS K7DualUart -- K7 UARTS for wideband tuners "SubDevice"
//===========================================================================

class K7DualUart : public IX6SubDevice, public K7DioPortIntf
{
public:
    //=======================================================================
    //  CLASS K7DualUart::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Uart0Ctrl_Addr;
        unsigned int   Uart0Tx_Addr;
        unsigned int   Uart0Rx_Addr;
        unsigned int   Uart1Ctrl_Addr;
        unsigned int   Uart1Tx_Addr;
        unsigned int   Uart1Rx_Addr;
        unsigned int   UartMode_Addr;
        unsigned int   UartInterrupt_Addr;
        unsigned int   GpioData_Addr;
        unsigned int   GpioDir_Addr;
    };

    //
    //  Member Functions
    K7DualUart(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
	virtual ~K7DualUart();

	K7UartPart & Uart(unsigned int idx)
						{  return (idx==1) ? Uart1 : Uart0 ;   }

	enum UartPowerModes { pmShutdown=0x0, pmRcvActive=0x1, pmDriversDisabled=0x2, pmNormal=0x3 };

	void  SetPowerMode(UartPowerModes  mode);

    //  Digital General Purpose Io
    virtual IBitManipulator &   DioPortData();

    virtual void 			    DioPortConfig(unsigned int);
    virtual unsigned int 	    DioPortConfig() const;

    virtual void                DioPortOutputEnable(int output_bit, bool output_enabled);
    virtual bool                DioPortOutputEnable(int output_bit) const;

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7DualUart::UartModeRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartModeRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   ComMode;
        RegisterBit   ComPs;

        UartModeRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ComMode(*this, 0), ComPs(*this, 1)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
	K7UartPart              Uart0;
    K7UartPart              Uart1;
    UartModeRegister        UartModeReg;
    Register                UartInterruptReg;
    Register                GpioData;
    Register                GpioOutEn;

    RegisterBitManipulator  GpioDataManip;
};

//===========================================================================
//  CLASS K7SysTripleUart -- K7Sys UARTS for GPS et al. "SubDevice"
//===========================================================================

class K7SysTripleUart : public IX6SubDevice, public IX6_GpsIo
{
public:
    //=======================================================================
    //  CLASS K7SysTripleUart::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Uart0Ctrl_Addr;
        unsigned int   Uart0Tx_Addr;
        unsigned int   Uart0Rx_Addr;
        unsigned int   Uart1Ctrl_Addr;
        unsigned int   Uart1Tx_Addr;
        unsigned int   Uart1Rx_Addr;
        unsigned int   Uart2Ctrl_Addr;
        unsigned int   Uart2Tx_Addr;
        unsigned int   Uart2Rx_Addr;
        unsigned int   UartMode_Addr;
        unsigned int   UartInterrupt_Addr;
        unsigned int   UartPpsArm_Addr;
    };

    enum UartPowerModes { pmShutdown=0x0, pmRcvActive=0x1, pmDriversDisabled=0x2, pmNormal=0x3 };

    //
    //  Member Functions
    K7SysTripleUart(IRequires_MultiGps * mdc, IGPS_Callback * cb, IAddressingSpace & space,
                                                             MapRegisters & regs);
    virtual ~K7SysTripleUart();

	K7UartPart & Uart(unsigned int idx)
		{
		switch (idx)
			{
			case 0:
			default:
				return Uart0;
			case 1:
				return Uart1;
			case 2:
				return Uart2;
			}
		}

	void  SetPowerMode(UartPowerModes  mode);
	//  Methods
	virtual IGps &   Gps();

    void SelectGps(size_t idx)
        {
        if (idx<GpsDevs.size())
            GpsIdx = idx;
        }


    //  IGps Implementation Methods
    //  ...Status
    virtual bool            IsLocked();
    virtual bool            IsAntenna();  // not implemented?
    //  ...Gps
    virtual void             GpsEnabled(bool state);
    virtual bool             GpsEnabled() const;

    virtual void             ArmGpsTrigger();
    //  ...Uart
    void             UartData(char c);
    char             UartData();

    //void             Baud(IGps::BaudRates rate);
    IGps::BaudRates  Baud() const;

    unsigned int     UartFifoDepth();

    bool        GpsInReset() const;
    void        GpsInReset(bool state);

public:
    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7SysTripleUart::UartModeRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartModeRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   ComMode;
        RegisterBit   ComPs;
        RegisterBit   Alarm;
        RegisterBit   IspN;
        RegisterBit   LdeKeyN;
        RegisterBit   Lock;
        RegisterBit   PpsIn;
        RegisterBit   ResetN;

        UartModeRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ComMode(*this, 0), ComPs(*this, 1),
              Alarm(*this, 2), IspN(*this, 3),
              LdeKeyN(*this, 4), Lock(*this, 5),
              PpsIn(*this, 6), ResetN(*this, 7)
        {  }
    };


private:
    IRequires_MultiGps *    MDC;
    IGPS_Callback *         CB;
    std::vector<IGps *>     GpsDevs;
    size_t                  GpsIdx;
    //IGps *                  GpsDevs;
    K7UartPart              Uart0;
	K7UartPart              Uart1;
	K7UartPart              Uart2;
	UartModeRegister        UartModeReg;
	Register                UartInterruptReg;
	Register                UartPpsArmReg;
};

//===========================================================================
//  CLASS SbcDuoGpsUart -- SbcDuo UART for GPS et al. "SubDevice"
//===========================================================================

class SbcDuoGpsUart : public IX6SubDevice, public IX6_GpsIo
{
public:
    //=======================================================================
    //  CLASS SbcDuoGpsUart::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Uart0Ctrl_Addr;
        unsigned int   Uart0Tx_Addr;
        unsigned int   Uart0Rx_Addr;
        unsigned int   Uart1Ctrl_Addr;
        unsigned int   Uart1Tx_Addr;
        unsigned int   Uart1Rx_Addr;
        unsigned int   UartStatus_Addr;
        unsigned int   UartInterrupt_Addr;
        unsigned int   UartPpsArm_Addr;
    };

    //enum UartPowerModes { pmShutdown=0x0, pmRcvActive=0x1, pmDriversDisabled=0x2, pmNormal=0x3 };

    //
    //  Member Functions
    SbcDuoGpsUart(IRequires_MultiGps* mdc, IGPS_Callback * cb, IAddressingSpace & space,
                                                             MapRegisters & regs);
    virtual ~SbcDuoGpsUart();

    K7UartPart & Uart(unsigned int idx=0)
        {
        switch (idx)
            {
            case 0:
            default:
                return Uart0;
            case 1:
                return Uart1;
            }
        }

    //void  SetPowerMode(UartPowerModes  mode);
    //  Methods
    virtual IGps &   Gps();

    void SelectGps(size_t idx)
        {
        if (idx<GpsDevs.size())
            GpsIdx = idx;
        }

    void  GpsReset(bool state);
    void  Ieee1588_MezzanineReset(bool state)  {  GpsReset(state);  }

    //  IGps Implementation Methods
    //  ...Status
    virtual bool            IsLocked();
    virtual bool            IsAntenna();  // not implemented?
    //  ...Gps
    virtual void             GpsEnabled(bool state);
    virtual bool             GpsEnabled() const;

    virtual void             ArmGpsTrigger();
    //  ...Uart
    void             UartData(char c);
    char             UartData();

    //void             Baud(IGps::BaudRates rate);
    IGps::BaudRates  Baud() const;

    unsigned int     UartFifoDepth();

    //bool        GpsInReset() const;
    //void        GpsInReset(bool state);

public:
    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoGpsUart::UartStatusRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   Lock;
        RegisterBit   PpsIn;
        RegisterBit   MezzGpsLinkResetN;

        UartStatusRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Lock(*this, 5),
              PpsIn(*this, 6),
              MezzGpsLinkResetN(*this, 7)
        {  }
    };


private:
    IRequires_MultiGps *    MDC;
    IGPS_Callback *         CB;
    std::vector<IGps *>     GpsDevs;
    size_t                  GpsIdx;
    K7UartPart              Uart0;
    K7UartPart              Uart1;
    UartStatusRegister      UartStatusReg;
    Register                UartInterruptReg;
    Register                UartPpsArmReg;
    bool                    IsGpsEnabled;
};

//===========================================================================
//  CLASS K7_RomControl -- K7 Rom Programming Shared Code
//===========================================================================

class K7_RomControl : public IIdRomDevice
{
public:
    //=======================================================================
    //  CLASS K7_RomControl::RomMapInfo  -- Device Rom Layout Definitions
    //=======================================================================

    class RomMapInfo
    {
    public:
        unsigned int   Page_Count;
        unsigned int   Page_Size;
        unsigned int   Sector_Count;
        unsigned int   Sector_Size;
    };
    //
    //  Member Functions
    K7_RomControl(RomMapInfo map) :  Map(map)
        {}
    virtual ~K7_RomControl()
        {}

    //
    //  IIdRomDevice implementation:  Idrom Page Operations
    virtual void   ErasePage(unsigned int page);
    virtual void   ReadPage(unsigned int page, int * pkt, unsigned int size);
    virtual void   WritePage(unsigned int page, int * pkt, unsigned int size);

    virtual unsigned int  JdecId();
    virtual unsigned int  Rdid();

    void                  Reset();

    //  ROM Info Commands
    void          EraseSector(unsigned int sector);
    unsigned int  Read32(unsigned int addr);
    void          Write32(unsigned int addr, unsigned int value);
    unsigned int  Rdsr();

    //  Page Program Mode Commands
    virtual void  SetProgramPageMode(bool state);
    virtual void  SetPageAddress(unsigned int addr);
    virtual void  Enqueue32(unsigned int value);
    virtual void  WaitForPageComplete();
    virtual void  PageProgram();

    //---------------------------
    //
    //  Flash Primitives
    void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data);   // write operation
    unsigned int  FlashOp(unsigned int opcode, unsigned int addr);  // read op
    void          FlashOp(unsigned int opcode);  // command op
    //
    //  Internal Flash Primitives
    void  Flash_Ready();
    void  Flash_ReadValid();
    void  Flash_NotBusy();
    void  Flash_QuickNotBusy();

    void  ReleaseBlockProtection();
    void  SetBlockProtection();
    bool  IsBlockProtectionReleased();

    void  Flash_Ready_T(unsigned int max_wait_us);

    void  Flash_NotBusy_T_ms(unsigned int max_wait_ms);
    void  Flash_NotBusy_T_us(unsigned int max_wait_us);

protected:
    //  Data
    RomMapInfo  Map;

    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit) = 0;
    virtual void          WriteOpcode(unsigned int opcode) = 0;
    virtual void          WriteAddr(unsigned int addr) = 0;
    virtual void          WriteData(unsigned int opcode) = 0;
    virtual unsigned int  ReadData() = 0;
    virtual bool          ReadReady() = 0;
    virtual bool          ReadDataValid() = 0;

    virtual void          TraceDebug(const std::string & text) = 0;
};

//===========================================================================
//  CLASS K7FpaLoader -- K7 FPA Loader "SubDevice"
//===========================================================================

class K7FpaLoader : public IX6SubDevice, public K7_RomControl
{
public:

    enum RomType { romS25FL128S, romS25FL128S_Sys };

    //=======================================================================
    //  CLASS K7FpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================

    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    //=======================================================================
    //  CLASS K7FpaLoader::K7FpaChip  -- Chip Interface
    //=======================================================================
    //  romS25FL128S ROM

    class K7FpaChip : public IK7FpaChip
    {
    public:
        K7FpaChip(K7FpaLoader * owner) : Owner(owner)
            {  FTotalSectors = 512;  }
        virtual ~K7FpaChip()
            {  }
        virtual unsigned int  SectorSize(unsigned int /*sector*/) const  {  return 0x10000;  }
        virtual unsigned int  Sectors() const   {  return FTotalSectors >> 1;  }
        virtual unsigned int  StartSector() const {  return 0;  }
        virtual unsigned int  GoldenImageStartSector() const
                    {  return StartSector() + Sectors();  }
        virtual unsigned int  GoldenImageSectors() const
                    {  return Sectors();  }

        virtual bool Initialize();

        //  Page Program Mode
        virtual void  SetProgramPageMode(bool state);
        virtual void  SetPageAddress(unsigned int addr);
        virtual void  Enqueue32(unsigned int value);
        virtual void  WaitForPageComplete();
        virtual void  PageProgram();

        virtual void  SetProtection();
        virtual void  ReleaseProtection();

        virtual unsigned int  Read32(unsigned int addr);
        virtual void          Write32(unsigned int addr, unsigned int value);

        virtual void          EraseSector(unsigned int sector);

    private:
        K7FpaLoader * Owner;
    };

    friend class K7FpaChip;

    //=======================================================================
    //  CLASS K7FpaLoader::K7SysFpaChip  -- Chip Interface
    //=======================================================================
    //  romS25FL128S  ROM  - truncated size to speed up erasure

    class K7SysFpaChip : public K7FpaChip
    {
    public:
        K7SysFpaChip(K7FpaLoader * owner): K7FpaChip(owner)
            {  FTotalSectors = 128;  }
        virtual ~K7SysFpaChip()
            {  }
    };

    //
    //  Member Functions
    K7FpaLoader(IRequires_Nothing * mdc, RomType which_rom, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7FpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    //==============================================================================
    //  CLASS LocalProm  -- "Raw" access to PROM
    //==============================================================================

    class LocalProm : public IVpxProm
    {
    public:
        LocalProm(K7FpaLoader * owner) : Owner(owner)
            {}
        virtual ~LocalProm()
            {}

        virtual void          EraseSector(unsigned int sector)
                                {  Owner->EraseSector(sector);  }
        virtual unsigned int  Read32(unsigned int addr)
                                {  return Owner->Read32(addr);  }
        virtual void          Write32(unsigned int addr, unsigned int value)
                                {  Owner->Write32(addr, value);  }
        virtual unsigned int  Rdsr()
                                {  return Owner->Rdsr();  }
        virtual unsigned int  Rdid()
                                {  return Owner->Rdid();  }

        //---------------------------
        //
        //  Flash Primitives
        //  ...write operation
        virtual void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
                                {  Owner->FlashOp(opcode, addr, data);  }
        //  ...read operation
        virtual unsigned int  FlashOp(unsigned int opcode, unsigned int addr)
                                {  return Owner->FlashOp(opcode, addr);  }
        //  ...command operation
        virtual void          FlashOp(unsigned int opcode)
                                {  Owner->FlashOp(opcode);  }

    private:
        K7FpaLoader * Owner;
    };
    friend class LocalProm;

    LocalProm &  Raw()
            {  return RawProm;  }

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7FpaLoader::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        PageMode;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), PageMode(*this, 26),
              Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    IK7FpaChip *        Chip;
    LocalProm           RawProm;    // debug interface
    FpaLoadThread       Thread;
    StopWatch           Rtc;

    //void          EraseSector(unsigned int sector);

protected:
    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit);
    virtual void          WriteOpcode(unsigned int opcode);
    virtual void          WriteAddr(unsigned int addr);
    virtual void          WriteData(unsigned int opcode);
    virtual unsigned int  ReadData();
    virtual bool          ReadReady();
    virtual bool          ReadDataValid();

    virtual void          TraceDebug(const std::string & text);

    IK7FpaChip *  MakeChipObject(RomType which_rom);
    RomMapInfo    SelectRomMap(RomType which_rom);
};

//===========================================================================
//  CLASS K7FlashRom -- K7 Flash Rom "SubDevice"  - IDROM and other uses
//===========================================================================

class K7FlashRom : public IX6SubDevice, public K7_RomControl
{
public:

    enum RomType { romS25FL128S, romS25FL128S_Sys };

    //=======================================================================
    //  CLASS K7FlashRom::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    K7FlashRom(IRequires_Nothing * mdc, RomType which_rom,
               IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7FlashRom();



    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7FlashRom::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        PageMode;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), PageMode(*this, 26),
              Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *  MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    //
    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit);
    virtual void          WriteOpcode(unsigned int opcode);
    virtual void          WriteAddr(unsigned int addr);
    virtual void          WriteData(unsigned int opcode);
    virtual unsigned int  ReadData();
    virtual bool          ReadReady();
    virtual bool          ReadDataValid();

    virtual void          TraceDebug(const std::string & text);

    RomMapInfo    SelectRomMap(RomType which_rom);
};

//===========================================================================
//  CLASS K7Ieee1588 -- K7 IEEE1588 "SubDevice"  -
//===========================================================================

class K7Ieee1588 : public IX6SubDevice
{
public:

    //=======================================================================
    //  CLASS K7Ieee1588::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Control_Addr;
        unsigned int   ClockID_Lo_Addr;
        unsigned int   ClockID_Hi_Addr;
        unsigned int   Time_Addr;
        unsigned int   I2C_Addr;
        unsigned int   Uart_Ctrl_Addr;
        unsigned int   Uart_Tx_Addr;
        unsigned int   Uart_Rx_Addr;
        unsigned int   Int_Addr;
        unsigned int   Arm_Addr;
    };

    K7Ieee1588(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~K7Ieee1588();

	K7UartPart & Uart(unsigned int /*idx*/)
						{  return Uart0 ;   }

    // status
    ii64           ClockID();
    bool           PllLocked();
    bool           InSync();
    bool           Linked();
    bool           Gigabit();

    // control
    void           Enable(bool state);
    void           Reset();

    void           InitIeee1588();

    unsigned int   TestReg(int addr);
    void           TestReg(int addr, unsigned int value);

    unsigned char  Rom(int addr);
    void           Rom(int addr, unsigned char value);

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7Ieee1588::ControlRegister  --  1588 Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Enable;
        RegisterBit        PllReset;
        RegisterBit        PllLock;
        RegisterBit        Mode;
        RegisterBitGroup   SwMode;
        RegisterBit        InSync;
        RegisterBit        Linked;
        RegisterBit        Gigabit;
        RegisterBit        TestEnable;

        ControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Enable(*this, 0),  PllReset(*this, 1), PllLock(*this, 2),
              Mode(*this, 4), SwMode(*this, 8, 3),
              InSync(*this, 12), Linked(*this, 13), Gigabit(*this, 14),
              TestEnable(*this, 16)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS K7Ieee1588::I2CRegister  --  1588 ROM Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class I2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Data_Out;
        RegisterBit        Clock_Out;
        RegisterBit        Data_In;
        RegisterBit        Clock_In;
        RegisterBit        DeviceEnable;

        I2CRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Data_Out(*this, 0),  Clock_Out(*this, 1), Data_In(*this, 2),
              Clock_In(*this, 4),  DeviceEnable(*this, 5)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ieee_I2C_Impl  -- Vcxo I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ieee_I2C_Impl : public I2CDeviceIntf
    {
    public:
        Ieee_I2C_Impl(I2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~Ieee_I2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        I2CRegister &  Reg;
    };

private:
    IRequires_Nothing *     MDC;
    K7UartPart              Uart0;
    ControlRegister         ControlReg;
    Register                ClockId_Lo_Reg;
    Register                ClockId_Hi_Reg;
    Register                TimeReg;
    I2CRegister             I2CReg;
    Register                Uart_Ctl_Reg;
    Register                Uart_Tx_Reg;
    Register                Uart_Rx_Reg;
    Register                Interrupt_Reg;
    Register                TriggerArm_Reg;
    //
    //  Interface Impls
    Ieee_I2C_Impl           RomIntf;
    I2CDeviceN              Rom_I2C;

};

//===========================================================================
//  CLASS IRIG_RealTimeClock -- IRIG Support "SubDevice"  -
//===========================================================================

class IRIG_RealTimeClock :  public IX6SubDevice, public IRealTimeClock
{
public:
    enum SpiDir { spiWrite, spiRead };

    //=======================================================================
    //  CLASS IRIG_RealTimeClock::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Date_Addr;
        unsigned int   FractionalSecs_Addr;
        unsigned int   DdsControl_Addr;
        unsigned int   DdsSpiControl_Addr;
        unsigned int   DdsSpiData_Addr;
        unsigned int   ClocksPerSymbol_Addr;
        unsigned int   DdsTuning_Addr;
    };

    IRIG_RealTimeClock(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~IRIG_RealTimeClock();

    virtual bool  SignalPresent();
    virtual bool  Locked();

    virtual unsigned int  Raw();
    virtual RtcData       Rtc();
    virtual time_t        Time(unsigned int year);


    void    Init();

    void    Spi(int addr, unsigned int value);

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IRIG_RealTimeClock::DateRegister  --  IRIG date readback
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DateRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        WatchdogStatus;
        RegisterBitGroup   Seconds;    // 0-59, BCD
        RegisterBitGroup   Minutes;    // 0-59, BCD
        RegisterBitGroup   Hours;      // 0-23, BCD
        RegisterBitGroup   Days;       // 1-366, BCD

        DateRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              WatchdogStatus(*this, 0),  Seconds(*this, 1, 7),
              Minutes(*this, 8, 7),
              Hours(*this, 15, 6), Days(*this, 21, 10)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IRIG_RealTimeClock::FracSecRegister  --  Fractional Seconds Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

//    class FracSecRegister : public Register
//    {
//        typedef Register inherited;

//    public:
//        RegisterBitGroup   FracSeconds;

//        FracSecRegister(IAddressingSpace & space, unsigned int offset)
//            : inherited(space, offset),
//              FracSeconds(*this, 0, 4)
//        {  }
//    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IRIG_RealTimeClock::DdsControlRegister  --  IRIG dds control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DdsControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   LoopEnable;
        RegisterBit   Reset;
        RegisterBit   IoUpdate;
        RegisterBit   Clock;
        RegisterBit   Locked;

        DdsControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              LoopEnable(*this, 0),  Reset(*this, 1), IoUpdate(*this, 2),
              Clock(*this, 3), Locked(*this, 5)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IRIG_RealTimeClock::DdsSpiControlRegister  --  IRIG Spi
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DdsSpiControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup   Address;
        RegisterBit        Direction;
        RegisterBit        ReadValid;

        DdsSpiControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Address(*this, 0, 3),  Direction(*this, 4), ReadValid(*this, 30)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IRIG_RealTimeClock::DdsSpiStatusRegister  --  IRIG Spi
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DdsSpiStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Ready;

        DdsSpiStatusRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Ready(*this, 31)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IRIG_RealTimeClock::DesiredCountRegister  --  IRIG Desired Count
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DesiredCountRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Count;

        DesiredCountRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Count(*this, 0, 28)
        {  }
    };


private:
    IRequires_Nothing *     MDC;
    DateRegister            DateReg;
    //FracSecRegister         FracSecReg;    // not used
    DdsControlRegister      DdsControlReg;
    DdsSpiControlRegister   DdsSpiControlReg;
    DdsSpiStatusRegister    DdsSpiStatusReg;
    Register                DdsSpiDataReg;
    DesiredCountRegister    DesiredCountReg;
    Register                DdsTuningReg;

    //  Internal Ops

    void          SpiReady();
    void          SpiNotReady();
    void          SetupSpiWrite(int addr, unsigned int value);
    bool          ReadSpiReadyBit();
    unsigned int  DumpSpiRegister();

    void          Decode(RtcData & rtc);
};

//===========================================================================
//  CLASS AtroposLedReset -- Atropos  LED/Reset register "SubDevice"
//===========================================================================

class AtroposLedReset : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS AtroposLedReset::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   LedReset_Addr;
    };

    //
    //  Member Functions
    AtroposLedReset(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~AtroposLedReset();

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StopHardware();

    // Mathods
    virtual void    AppReset();

    // Properties
    virtual void    BlueLed(bool state);
    virtual bool    BlueLed() const;

    virtual void    GreenLed(bool state);
    virtual bool    GreenLed() const;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AtroposLedReset::LedResetRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class LedResetRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Reset;
        RegisterBit        BlueLed;
        RegisterBit        GreenLed;

        LedResetRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Reset(*this, 0), BlueLed(*this, 1), GreenLed(*this, 2)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    LedResetRegister        ControlReg;
};

//===========================================================================
//  CLASS SbcDuo_XmcI2C -- Standard bit-banged I2C link Subdevice
//===========================================================================

class SbcDuo_XmcI2C : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS SbcDuo_XmcI2C::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   XMC0_Addr;
        unsigned int   XMC1_Addr;
    };

    //
    //  Member Functions
    SbcDuo_XmcI2C(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~SbcDuo_XmcI2C();

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StopHardware();

    virtual I2CDevice &   Device(unsigned int idx)
        {  return (idx)? Xmc1_I2C : Xmc0_I2C;  }
    virtual bool          XmcPresent(unsigned int idx);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuo_XmcI2C::XmcI2CRegister  -- Xmc I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XmcI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Data_Out;
        RegisterBit  Clock_Out;
        RegisterBit  Data_In;
        RegisterBit  Clock_In;
        RegisterBit  Xmc_PresentN;

    public:
        XmcI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data_Out(*this, 0),   Clock_Out(*this, 1),
               Data_In(*this, 2),    Clock_In(*this, 3),
               Xmc_PresentN(*this, 4)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XmcI2C_Impl  -- Vcxo I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XmcI2C_Impl : public I2CDeviceIntf
    {
    public:
        XmcI2C_Impl(XmcI2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~XmcI2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        XmcI2CRegister &  Reg;
    };

private:
    IRequires_Nothing *     MDC;

    XmcI2CRegister    Xmc0Control;
    XmcI2CRegister    Xmc1Control;
    //
    //  Interface Impls
    XmcI2C_Impl       Xmc0Intf;
    XmcI2C_Impl       Xmc1Intf;

    I2CDeviceN        Xmc0_I2C;
    I2CDeviceN        Xmc1_I2C;
};

//==============================================================================
//  CLASS ISbcDuoSourceSelector  -- Routing Control for Duo signals
//==============================================================================

class ISbcDuoSourceSelector
{
public:
    virtual ~ISbcDuoSourceSelector()
        {}

    enum ExtClockRefSignalSources { clkGps, clkExternal };

    enum PulsePerSecSources { ppsInternalFpga, ppsGps };

    void  MasterReferenceSource(ExtClockRefSignalSources src)
        {  MasterReferenceSource(static_cast<unsigned int>(src));  }
    virtual void  MasterReferenceSource(unsigned int src) = 0;
    virtual unsigned int  MasterReferenceSource() = 0;

    void  MasterPulsePerSecSource(PulsePerSecSources src)
        {  MasterPulsePerSecSource(static_cast<unsigned int>(src));  }
    virtual void  MasterPulsePerSecSource(unsigned int src) = 0;
    virtual unsigned int  MasterPulsePerSecSource() = 0;

};

//===========================================================================
//  CLASS SbcDuoPeripherals -- Odds and Ends Periphs for DUO
//===========================================================================

class SbcDuoPeripherals : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS SbcDuo_XmcI2C::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   TimingControl_Addr;
        unsigned int   X10GbEthernet_Addr;
        unsigned int   InterruptSourceSel_Addr;
    };

    //
    //  Member Functions
    SbcDuoPeripherals(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~SbcDuoPeripherals();

    //==============================================================================
    //  CLASS SbcDuoSourceSelector  -- Routing Control for Duo signals
    //==============================================================================

    class SbcDuoSourceSelector : public ISbcDuoSourceSelector
    {
    public:
        SbcDuoSourceSelector(SbcDuoPeripherals * owner) : Owner(owner)
            {}
        virtual ~SbcDuoSourceSelector()
            {}

        virtual void  MasterReferenceSource(unsigned int src)
            {
			bool bit = ((src) ? true : false);
            Owner->TimingControlReg.Pll_ClockSelect.Value(bit);
            }
        virtual unsigned int  MasterReferenceSource()
            {
            return Owner->TimingControlReg.Pll_ClockSelect.Value();
            }

        virtual void  MasterPulsePerSecSource(unsigned int src)
            {
			bool bit = ((src) ? true : false);
            Owner->TimingControlReg.Pps_ClockSelect.Value(bit);
            }
        virtual unsigned int  MasterPulsePerSecSource()
            {
            return Owner->TimingControlReg.Pps_ClockSelect.Value();
            }

    private:
        SbcDuoPeripherals * Owner;
    };

    friend class SbcDuoSourceSelector;
    ISbcDuoSourceSelector &    Selector()
        {  return SrcSelector;  }

    void             CustomInterruptSource(unsigned int src);
    unsigned int     CustomInterruptSource();


public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StopHardware();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoPeripherals::TimingControlRegister  --  Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TimingControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Pll_ClockSelect;
        RegisterBit  Pps_ClockSelect;

    public:
        TimingControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Pll_ClockSelect(*this, 0),
               Pps_ClockSelect(*this, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoPeripherals::TenGBEthernetControl  -- Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TenGBEthernetControl : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup        Enable;
        RegisterBit             Power_3p3v;
        RegisterBit             Power_2p5v;
        RegisterBit             Power_1p2v;
        RegisterBit             Power_0p8v;
        RegisterBit             Power_0p67v;

    public:
        TenGBEthernetControl( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Enable(*this, 0, 2),
               Power_3p3v(*this, 4),   Power_2p5v(*this, 5),
               Power_1p2v(*this, 6),   Power_0p8v(*this, 7),
               Power_0p67v(*this, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoPeripherals::CustomIntSourceRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CustomIntSourceRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup       Source;

    public:
        CustomIntSourceRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Source(*this, 0, 2)
            {
            }
    };


private:
    IRequires_Nothing *     MDC;
    SbcDuoSourceSelector    SrcSelector;

    TimingControlRegister      TimingControlReg;
    TenGBEthernetControl       TenGBEthernetCtrlReg;
    CustomIntSourceRegister    CustomIntSourceReg;

};

//===========================================================================
//  CLASS QSpi_RomControl -- QSPI Rom Programming Shared Code
//===========================================================================

class QSpi_RomControl : public IIdRomDevice
{
public:
    //=======================================================================
    //  CLASS QSpi_RomControl::RomMapInfo  -- Device Rom Layout Definitions
    //=======================================================================

    class RomMapInfo
    {
    public:
        unsigned int   Page_Count;
        unsigned int   Page_Size;
        unsigned int   Sector_Count;
        unsigned int   Sector_Size;
        unsigned int   Protect_ON_Value;
        unsigned int   Protect_OFF_Value;
        unsigned int   Protect_Check_Mask;
    };
    //
    //  Member Functions
    QSpi_RomControl(RomMapInfo map) :  Map(map)
        {}
    virtual ~QSpi_RomControl()
        {}

    //
    //  IIdRomDevice implementation:  Idrom Page Operations
    virtual void   ErasePage(unsigned int page);
    virtual void   ReadPage(unsigned int page, int * pkt, unsigned int size);
    virtual void   WritePage(unsigned int page, int * pkt, unsigned int size);

    virtual unsigned int  JdecId();
    virtual unsigned int  Rdid();

    void                  Reset();

    //  ROM Info Commands
    void          EraseSector(unsigned int sector);
    unsigned int  Read32(unsigned int addr);
    void          Write32(unsigned int addr, unsigned int value);
    unsigned int  Rdsr();

    //  Page Program Mode Commands
    virtual void  SetProgramPageMode(bool state);
    virtual void  SetPageAddress(unsigned int addr);
    virtual void  Enqueue32(unsigned int value);
    virtual void  WaitForPageComplete();
    virtual void  PageProgram();

    //---------------------------
    //
    //  Flash Primitives
    void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data);   // write operation
    unsigned int  FlashOp(unsigned int opcode, unsigned int addr);  // read op
    void          FlashOp(unsigned int opcode);  // command op
    //
    //  Internal Flash Primitives
    void  Flash_Ready();
    void  Flash_ReadValid();
    void  Flash_NotBusy();
    void  Flash_QuickNotBusy();

    void  ReleaseBlockProtection();
    void  SetBlockProtection();
    bool  IsBlockProtectionReleased();

    void  Flash_Ready_T(unsigned int max_wait_us);

    void  Flash_NotBusy_T_ms(unsigned int max_wait_ms);
    void  Flash_NotBusy_T_us(unsigned int max_wait_us);

protected:
    //  Data
    RomMapInfo  Map;

    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit) = 0;
    virtual void          WriteOpcode(unsigned int opcode) = 0;
    virtual void          WriteAddr(unsigned int addr) = 0;
    virtual void          WriteData(unsigned int opcode) = 0;
    virtual unsigned int  ReadData() = 0;
    virtual bool          ReadReady() = 0;
    virtual bool          ReadDataValid() = 0;

    virtual void          TraceDebug(const std::string & text) = 0;
};

//===========================================================================
//  CLASS QSpiFpaLoader -- QSPI supporting FPA Loader "SubDevice"
//===========================================================================

class QSpiFpaLoader : public IX6SubDevice, public QSpi_RomControl
{
public:

    enum RomType { romMicron_N25Q128 };

    //=======================================================================
    //  CLASS QSpiFpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================

    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    //=======================================================================
    //  CLASS QSpiFpaLoader::MicronFpaChip  -- Chip Interface
    //=======================================================================
    //  Micron_N25Q128 ROM

    class MicronFpaChip : public IK7FpaChip
    {
    public:
        MicronFpaChip(QSpiFpaLoader * owner) : Owner(owner)
            {
            FTotalSectors = 64*2; /* actually 256*2 -- includes "fake" golden image reserve */
            // we lie about actual size of Micron chip here to speed erasing.
            }
        virtual ~MicronFpaChip()
            {  }
        virtual unsigned int  SectorSize(unsigned int /*sector*/) const  {  return 0x10000;  }
        virtual unsigned int  Sectors() const   {  return FTotalSectors >> 1;  }
        virtual unsigned int  StartSector() const {  return 0;  }
        virtual unsigned int  GoldenImageStartSector() const
                    {  return StartSector() + Sectors();  }
        virtual unsigned int  GoldenImageSectors() const
                    {  return Sectors();  }

        virtual bool Initialize();

        //  Page Program Mode
        virtual void  SetProgramPageMode(bool state);
        virtual void  SetPageAddress(unsigned int addr);
        virtual void  Enqueue32(unsigned int value);
        virtual void  WaitForPageComplete();
        virtual void  PageProgram();

        virtual void  SetProtection();
        virtual void  ReleaseProtection();

        virtual unsigned int  Read32(unsigned int addr);
        virtual void          Write32(unsigned int addr, unsigned int value);

        virtual void          EraseSector(unsigned int sector);

    private:
        QSpiFpaLoader * Owner;
    };

    friend class MicronFpaChip;

    //
    //  Member Functions
    QSpiFpaLoader(IRequires_Nothing * mdc, RomType which_rom, IAddressingSpace & space, MapRegisters & regs);
    virtual ~QSpiFpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    //==============================================================================
    //  CLASS LocalProm  -- "Raw" access to PROM
    //==============================================================================

    class LocalProm : public IVpxProm
    {
    public:
        LocalProm(QSpiFpaLoader * owner) : Owner(owner)
            {}
        virtual ~LocalProm()
            {}

        virtual void          EraseSector(unsigned int sector)
                                {  Owner->EraseSector(sector);  }
        virtual unsigned int  Read32(unsigned int addr)
                                {  return Owner->Read32(addr);  }
        virtual void          Write32(unsigned int addr, unsigned int value)
                                {  Owner->Write32(addr, value);  }
        virtual unsigned int  Rdsr()
                                {  return Owner->Rdsr();  }
        virtual unsigned int  Rdid()
                                {  return Owner->Rdid();  }

        //---------------------------
        //
        //  Flash Primitives
        //  ...write operation
        virtual void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
                                {  Owner->FlashOp(opcode, addr, data);  }
        //  ...read operation
        virtual unsigned int  FlashOp(unsigned int opcode, unsigned int addr)
                                {  return Owner->FlashOp(opcode, addr);  }
        //  ...command operation
        virtual void          FlashOp(unsigned int opcode)
                                {  Owner->FlashOp(opcode);  }

    private:
        QSpiFpaLoader * Owner;
    };
    friend class LocalProm;

    LocalProm &  Raw()
            {  return RawProm;  }

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS QSpiFpaLoader::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        PageMode;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), PageMode(*this, 26),
              Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    IK7FpaChip *        Chip;
    LocalProm           RawProm;    // debug interface
    FpaLoadThread       Thread;
    StopWatch           Rtc;

    //void          EraseSector(unsigned int sector);

protected:
    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit);
    virtual void          WriteOpcode(unsigned int opcode);
    virtual void          WriteAddr(unsigned int addr);
    virtual void          WriteData(unsigned int opcode);
    virtual unsigned int  ReadData();
    virtual bool          ReadReady();
    virtual bool          ReadDataValid();

    virtual void          TraceDebug(const std::string & text);

    IK7FpaChip *  MakeChipObject(RomType which_rom);
    RomMapInfo    SelectRomMap(RomType which_rom);
};


//===========================================================================
//  CLASS LTC2494 -- 16-Bit 8-/16-Channel ADC chip
//===========================================================================

class LTC2494 : public ISelfMeasurement
{
public:

    //configration bits
    struct ConfigurationBits
    {
        bool EN;
        bool EN2;
        bool IM;
        bool FA;            //Rejection Mode bit
        bool FB;            //Rejection Mode bit
        bool SPD;           //Speed Mode bit
        bool GS0;           //
        bool GS1;           //Gain
        bool GS2;           //
    };


    //ctor
    LTC2494(IAddressingSpace &space, LTC2494SpiDevice::Info &Regs, std::string tag = "");
    ~LTC2494();

    virtual void                ResizeCaches();
    virtual void                ConfigureBoardDefaults() = 0;
    virtual double              PowerCalculation(unsigned int /*power_ch*/)
    { return 0.0; }

    //  External Interface
    virtual double                 ReadTemperature();
    virtual double                 ReadChannel(unsigned int ch);
    virtual void                   CalculatePower();

    // instance specific data
    virtual double          GetScaleTable(unsigned int ch);
    virtual std::string     GetSignalName(unsigned int ch);
    virtual double          GetNominalV(unsigned int ch);
    virtual double          GetMinV(unsigned int ch);
    virtual double          GetMaxV(unsigned int ch);
    virtual double          GetNominalAdcV(unsigned int ch);
    virtual std::string     GetSignalUnits(unsigned int ch);

    virtual double          GetCurrentPower(unsigned int power_ch);
    virtual double          GetCurrentMeasure(unsigned int ch);

protected:

    //  Adc conversion type
    enum ConversionType { ChannelConversion = 0, TemperatureConversion = 1 };

    //  Adc Channel type
    enum ChannelType { SingleEnded, Differential, InvertedDifferential, Unused };

    //------Virtual Members
    virtual int                 ReadSpi(unsigned int Addr);
    virtual unsigned int        GenerateAddress(ConversionType type, unsigned int ch = 0);
    virtual double              Measure(int data,ConversionType type, unsigned int ch = 0);
    virtual double              PositiveMeasure(int data, unsigned int ch);
    virtual double              NegativeMeasure(int data, unsigned int ch);
public:
    virtual void                MapChannels();
protected:
    virtual int                 ExtractData(int data);

    //  measurement caches
    std::vector<double>         MeasureCache;
    std::vector<double>         PowerMeasureCache;

    //  Any derived class must init this members

    std::vector<double>         ScaleTable;
    std::vector<int>            ChannelMuxAddressTable;
    std::vector<ChannelType>    ChannelTypeTable;

    std::vector<std::string>    SignalNameStr;  // schematic table info
    std::vector<double>         NominalV;
    std::vector<double>         MinV;
    std::vector<double>         MaxV;
    std::vector<double>         NominalAdcV;
                                                // ScaleTable is COEF * ReferenceV
    std::vector<std::string>    SignalUnitsStr;

    //
    ConfigurationBits           ConfigBits;


private:
    LTC2494SpiDevice            Spi;
};


//===========================================================================
//  CLASS SelfMeasurementDevice -- Subdevice for LTC2494 self measurement
//===========================================================================

class SelfMeasurementDevice : public IX6SubDevice//, //public ISelfMeasurement  //(remove this?)
{
public:
    //
    //  Member Functions
    SelfMeasurementDevice(IRequires_LTC2494Device * mdc);
    virtual ~SelfMeasurementDevice();

    ISelfMeasurement & SelfM()
        {  return *SelfMeasure;  }

public:
    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

//    virtual double     ReadTemperature();
//    virtual double     ReadChannel(unsigned int ch);

//    // instance specific data
//    virtual double          GetScaleTable(unsigned int idx);
//    virtual std::string     GetSignalName(unsigned int idx);
//    virtual double          GetNominalV(unsigned int idx);
//    virtual double          GetMinV(unsigned int idx);
//    virtual double          GetMaxV(unsigned int idx);
//    virtual double          GetNominalAdcV(unsigned int idx);
//    virtual std::string     GetSignalUnits(unsigned int idx);


private:
    IRequires_LTC2494Device *     MDC;
    LTC2494 *                     SelfMeasure;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
