//---------------------------------------------------------------------------
// K7Support_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <K7Support_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

    //=============================================================================
    //  CLASS K7AssemblyIdromPage  --  System Reserved IDROM page
    //=============================================================================
    //  Layout Info
    const unsigned int  SectionSize = 32;      // leave some room for extensions

    const unsigned int  K7Section_Start       = 0;
    const unsigned int  K7PowerSection_Start  = K7Section_Start + SectionSize;
    const unsigned int  Fmc0Section_Start     = K7PowerSection_Start + SectionSize;
    const unsigned int  Fmc1Section_Start     = Fmc0Section_Start + SectionSize;
    const unsigned int  CpuSection_Start      = Fmc1Section_Start + SectionSize;
    const unsigned int  EpcK7Section_Start    = CpuSection_Start + SectionSize;
    const unsigned int  nextSection_Start     = EpcK7Section_Start + SectionSize;

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_PartNumber()  --  Read Part # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::K7_PartNumber() const
    {
        std::string pn;

        pn = AsString(K7Section_Start+0, 5);   // 20 characters max
        return pn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_PartNumber()  --  Write Part # to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_PartNumber(const std::string & pn)
    {
        AsString(K7Section_Start+0, 5, pn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_SerialNumber()  --  Read Serial # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::K7_SerialNumber() const
    {
        std::string sn;

        sn = AsString(K7Section_Start+5, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_SerialNumber()  --  Write Serial to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_SerialNumber(const std::string & sn)
    {
        AsString(K7Section_Start+5, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_FpgaType()  --  Read Fpga Type Code
    //------------------------------------------------------------------------

    unsigned int  K7AssemblyIdromPage::K7_FpgaType() const
    {
        unsigned int type;

        type = AsInt(K7Section_Start+10);
        return type;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_FpgaType()  --  Write Fpga Type to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_FpgaType(unsigned int type)
    {
        AsInt(K7Section_Start+10, type);
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_FpgaSpeed()  --  Read Fpga Speed Code
    //------------------------------------------------------------------------

    unsigned int  K7AssemblyIdromPage::K7_FpgaSpeed() const
    {
        unsigned int speed;

        speed = AsInt(K7Section_Start+11);
        return speed;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_FpgaSpeed()  --  Write Fpga Speed to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_FpgaSpeed(unsigned int speed)
    {
        AsInt(K7Section_Start+11, speed);
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_FpgaTempGrade()  --  Read Fpga Temp Code
    //------------------------------------------------------------------------

    unsigned int  K7AssemblyIdromPage::K7_FpgaTempGrade() const
    {
        unsigned int tempgrade;

        tempgrade = AsInt(K7Section_Start+12);
        return tempgrade;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_FpgaTempGrade()  --  Write Fpga Temp to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_FpgaTempGrade(unsigned int tempgrade)
    {
        AsInt(K7Section_Start+12, tempgrade);
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_LogicVersion()  --  Read Fpga Logic Ver
    //------------------------------------------------------------------------

    unsigned int  K7AssemblyIdromPage::K7_LogicVersion() const
    {
        unsigned int version;

        version = AsInt(K7Section_Start+13);
        return version;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_LogicVersion()  --  Write Fpga Logic Ver
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_LogicVersion(unsigned int version)
    {
        AsInt(K7Section_Start+13, version);
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_SysLogicVersion()  --  Read Fpga Sys Logic Ver
    //------------------------------------------------------------------------

    unsigned int  K7AssemblyIdromPage::K7_SysLogicVersion() const
    {
        unsigned int version;

        version = AsInt(K7Section_Start+14);
        return version;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7_SysLogicVersion()  --  Write Fpga Sys Logic Ver
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7_SysLogicVersion(unsigned int version)
    {
        AsInt(K7Section_Start+14, version);
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7Power_PartNumber()  --  Read Part # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::K7Power_PartNumber() const
    {
        std::string pn;

        pn = AsString(K7PowerSection_Start+0, 5);   // 20 characters max
        return pn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7Power_PartNumber()  --  Write Part # to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7Power_PartNumber(const std::string & pn)
    {
        AsString(K7PowerSection_Start+0, 5, pn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7Power_SerialNumber()  --  Read Serial # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::K7Power_SerialNumber() const
    {
        std::string sn;

        sn = AsString(K7PowerSection_Start+5, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::K7Power_SerialNumber()  --  Write Serial to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::K7Power_SerialNumber(const std::string & sn)
    {
        AsString(K7PowerSection_Start+5, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc0_PartNumber()  --  Read Part # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Fmc0_PartNumber() const
    {
        std::string pn;

        pn = AsString(Fmc0Section_Start+0, 5);   // 20 characters max
        return pn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc0_PartNumber()  --  Write Part # to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Fmc0_PartNumber(const std::string & pn)
    {
        AsString(Fmc0Section_Start+0, 5, pn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc0_SerialNumber()  --  Read Serial # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Fmc0_SerialNumber() const
    {
        std::string sn;

        sn = AsString(Fmc0Section_Start+5, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc0_SerialNumber()  --  Write Serial to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Fmc0_SerialNumber(const std::string & sn)
    {
        AsString(Fmc0Section_Start+5, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc0_Name()  --  Read Name from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Fmc0_Name() const
    {
        std::string sn;

        sn = AsString(Fmc0Section_Start+10, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc0_Name()  --  Write Name to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Fmc0_Name(const std::string & sn)
    {
        AsString(Fmc0Section_Start+10, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc1_PartNumber()  --  Read Part # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Fmc1_PartNumber() const
    {
        std::string pn;

        pn = AsString(Fmc1Section_Start+0, 5);   // 20 characters max
        return pn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc1_PartNumber()  --  Write Part # to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Fmc1_PartNumber(const std::string & pn)
    {
        AsString(Fmc1Section_Start+0, 5, pn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc1_SerialNumber()  --  Read Serial # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Fmc1_SerialNumber() const
    {
        std::string sn;

        sn = AsString(Fmc1Section_Start+5, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc1_SerialNumber()  --  Write Serial to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Fmc1_SerialNumber(const std::string & sn)
    {
        AsString(Fmc1Section_Start+5, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc1_Name()  --  Read Name from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Fmc1_Name() const
    {
        std::string sn;

        sn = AsString(Fmc1Section_Start+10, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Fmc1_Name()  --  Write Name to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Fmc1_Name(const std::string & sn)
    {
        AsString(Fmc1Section_Start+10, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Cpu_PartNumber()  --  Read Part # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Cpu_PartNumber() const
    {
        std::string pn;

        pn = AsString(CpuSection_Start+0, 5);   // 20 characters max
        return pn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Cpu_PartNumber()  --  Write Part # to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Cpu_PartNumber(const std::string & pn)
    {
        AsString(CpuSection_Start+0, 5, pn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Cpu_SerialNumber()  --  Read Serial # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::Cpu_SerialNumber() const
    {
        std::string sn;

        sn = AsString(CpuSection_Start+5, 10);   // 40 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Cpu_SerialNumber()  --  Write Serial to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Cpu_SerialNumber(const std::string & sn)
    {
        AsString(CpuSection_Start+5, 10, sn);   // 40 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::EpcK7_PartNumber()  --  Read Part # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::EpcK7_PartNumber() const
    {
        std::string pn;

        pn = AsString(EpcK7Section_Start+0, 5);   // 20 characters max
        return pn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::EpcK7_PartNumber()  --  Write Part # to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::EpcK7_PartNumber(const std::string & pn)
    {
        AsString(EpcK7Section_Start+0, 5, pn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::EpcK7_SerialNumber()  --  Read Serial # from Image
    //------------------------------------------------------------------------

    std::string  K7AssemblyIdromPage::EpcK7_SerialNumber() const
    {
        std::string sn;

        sn = AsString(EpcK7Section_Start+5, 5);   // 20 characters max
        return sn;
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::EpcK7_SerialNumber()  --  Write Serial to Image
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::EpcK7_SerialNumber(const std::string & sn)
    {
        AsString(EpcK7Section_Start+5, 5, sn);   // 20 characters max
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Updated()  -- read 'written to' flag for a section
    //------------------------------------------------------------------------

    bool  K7AssemblyIdromPage::Updated() const
    {
        return (AsInt(1023)==1) ? true : false; // make sure junk is 'false'
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::Updated()  -- Mark data area as 'written to'
    //------------------------------------------------------------------------

    void  K7AssemblyIdromPage::Updated(bool state)
    {
        AsInt(1023, (state) ? 1 : 0);
    }

    //------------------------------------------------------------------------
    //   K7AssemblyIdromPage::FirstUnusedWord()  -- First safe int for addons
    //------------------------------------------------------------------------

    unsigned int  K7AssemblyIdromPage::FirstUnusedWord() const
    {
        return nextSection_Start;
    }

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative

