//  K7Support_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef K7Support_MbH
#define K7Support_MbH

#include <string>
#include <X6Idrom_Mb.h>
#include <ctime>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IK7SourceSelector  -- Routing Control for K7 signals
//==============================================================================

class IK7SourceSelector
{
public:
    virtual ~IK7SourceSelector()
        {}

    enum ExtClockRefSignalSources { clkGps, clkExternal, clk1588Ref, clkIrig };

    enum PulsePerSecSources { ppsGps, ppsExternal, pps1588Ref, ppsIrig };

    void  MasterReferenceSource(ExtClockRefSignalSources src)
        {  MasterReferenceSource(static_cast<unsigned int>(src));  }
    virtual void  MasterReferenceSource(unsigned int src) = 0;
    virtual unsigned int  MasterReferenceSource() = 0;

    void  MasterPulsePerSecSource(PulsePerSecSources src)
        {  MasterPulsePerSecSource(static_cast<unsigned int>(src));  }
    virtual void  MasterPulsePerSecSource(unsigned int src) = 0;
    virtual unsigned int  MasterPulsePerSecSource() = 0;

};

//===========================================================================
//  CLASS IMulticharUart -- Interface for "String Based" Uart
//===========================================================================

class IMulticharUart
{
public:
	virtual ~IMulticharUart() {};

    virtual void           SetBaud(unsigned int rate) = 0;
    virtual unsigned int   GetBaud() = 0;

    virtual void           Port(unsigned char ch) = 0;
	virtual unsigned char  Port() = 0;

	virtual void           PutString(const std::string & data) = 0;
	virtual void           GetString(std::string & data) = 0;

    virtual void           GetFixedString(std::string & data, size_t chars) = 0;

    virtual bool           FrameError() = 0;
    virtual void           ClearFrameError() = 0;

};

//===========================================================================
//  CLASS IK7LcdDevice -- User interface to SBC-K7 LCD display
//===========================================================================

class IK7LcdDevice
{
public:
    //
    enum LCD_Types { tlcd_MultiLine, tlcd_SingleLine};

    //  Interface Functions
    virtual void            ClearDisplay() = 0;
    virtual void            Home() = 0;
    virtual void            EntryMode(bool inc, bool shift) = 0;
    virtual void            OnOff(bool display, bool cursor, bool blink) = 0;
    virtual void            Shift(bool displayCursor, bool rightLeft) = 0;
    virtual void            Function(bool dataLen, unsigned short fontTable) = 0;
    virtual void            CgRamAddr(unsigned short addr) = 0;
    virtual void            DdRamAddr(unsigned short addr) = 0;
    virtual unsigned char   AddressCounter() = 0;
    virtual void            Emit(unsigned char c) = 0;
    virtual void            Type(const std::string & txt) = 0;
    virtual unsigned char   Key() = 0;
    virtual void            NextLine() = 0;

    virtual void            WriteDataCmd(unsigned short data) = 0;
    virtual unsigned char   ReadDataCmd() = 0;
    virtual void            UpdateLcdType(unsigned int Type) = 0;

    virtual ~IK7LcdDevice() {}
};

//===========================================================================
//  CLASS IK7LcdDeviceSL -- User interface to SBC-K7 LCD display
//===========================================================================

class IK7LcdDeviceSL
{
public:
    //
    //  Interface Functions
    virtual void            ClearDisplay() = 0;
    virtual void            Home() = 0;
    virtual void            EntryMode(bool inc, bool shift) = 0;
    virtual void            OnOff(bool display, bool cursor, bool blink) = 0;
    virtual void            Shift(bool displayCursor, bool rightLeft) = 0;
    virtual void            Function(bool LineNumber, unsigned short FontSize) = 0;
    virtual void            CgRamAddr(unsigned short addr) = 0;
    virtual void            DdRamAddr(unsigned short addr) = 0;
    virtual void            Emit(unsigned char c) = 0;
    virtual void            Type(const std::string & txt) = 0;

    virtual void            WriteDataCmd(unsigned int opcode, unsigned short data) = 0;

    virtual ~IK7LcdDeviceSL() {}
};

//=============================================================================
//  CLASS K7AssemblyIdromPage  --  System Reserved Assembly Info IDROM page
//=============================================================================
//  Page dedicated to the information about the package assembly
//

class K7AssemblyIdromPage :  public X6IdromPage
{
public:
    //
    //  Member Functions
    K7AssemblyIdromPage(IIdRomDevice & rom, unsigned int page) : X6IdromPage(rom, page)
        {}
    ~K7AssemblyIdromPage()
        {}
    //
    //  Data Access Properties
    //
    //  K7 Board (SBC-K7, Mini-K7) information
    std::string     K7_PartNumber() const;         // Full Part # (include rugged, ECO information
    void            K7_PartNumber(const std::string & name);
    std::string     K7_SerialNumber() const;
    void            K7_SerialNumber(const std::string & revision);
    unsigned int    K7_FpgaType() const;
    void            K7_FpgaType(unsigned int type);
    unsigned int    K7_FpgaSpeed() const;
    void            K7_FpgaSpeed(unsigned int speed);
    unsigned int    K7_FpgaTempGrade() const;
    void            K7_FpgaTempGrade(unsigned int temp_grade);
    unsigned int    K7_LogicVersion() const;
    void            K7_LogicVersion(unsigned int version);
    unsigned int    K7_SysLogicVersion() const;
    void            K7_SysLogicVersion(unsigned int version);
    //
    //  K7 POWER Board information
    std::string     K7Power_PartNumber() const;         // Full Part # (include rugged, ECO information)
    void            K7Power_PartNumber(const std::string & name);
    std::string     K7Power_SerialNumber() const;
    void            K7Power_SerialNumber(const std::string & revision);
    //
    //  FMC 0 Board information
    std::string     Fmc0_PartNumber() const;         // Full Part # (include rugged, ECO information)
    void            Fmc0_PartNumber(const std::string & name);
    std::string     Fmc0_SerialNumber() const;
    void            Fmc0_SerialNumber(const std::string & revision);
    std::string     Fmc0_Name() const;
    void            Fmc0_Name(const std::string & name);
    //
    //  FMC 1 Board information
    std::string     Fmc1_PartNumber() const;         // Full Part # (include rugged, ECO information)
    void            Fmc1_PartNumber(const std::string & name);
    std::string     Fmc1_SerialNumber() const;
    void            Fmc1_SerialNumber(const std::string & revision);
    std::string     Fmc1_Name() const;
    void            Fmc1_Name(const std::string & name);
    //
    //  CPU Mother Board information
    std::string     Cpu_PartNumber() const;         // Full Part #
    void            Cpu_PartNumber(const std::string & name);
    std::string     Cpu_SerialNumber() const;
    void            Cpu_SerialNumber(const std::string & revision);
    //
    //  epc-K7 information
    std::string     EpcK7_PartNumber() const;         // Full Part #
    void            EpcK7_PartNumber(const std::string & name);
    std::string     EpcK7_SerialNumber() const;
    void            EpcK7_SerialNumber(const std::string & revision);

    bool            Updated() const;          // 'Section written' flag
    void            Updated(bool state);

    unsigned int    FirstUnusedWord() const;

};

//=============================================================================
//  CLASS IK7_IdRom  --  Interface to devices supporting IDROM pages
//=============================================================================

class IK7_IdRom
{
public:

    virtual IIdRomDevice &   Device() = 0;

    //  Page access
    virtual X6SystemIdromPage &    System() = 0;
    virtual K7AssemblyIdromPage &  Assembly() = 0;
    virtual X6IdromPage &          User() = 0;

    virtual ~IK7_IdRom()
        {}

};

//=============================================================================
//  CLASS IRealTimeClock --  Interface to devices with Date/Time Info
//=============================================================================

class IRealTimeClock
{
public:
    struct RtcData
        {
        unsigned int   seconds;    // 0-59
        unsigned int   minutes;    // 0-59
        unsigned int   hours;      // 0-23
        unsigned int   days;       // 1-366
        };

    virtual bool  SignalPresent() = 0;
    virtual bool  Locked() = 0;

    virtual unsigned int  Raw() = 0;
    virtual RtcData       Rtc() = 0;
    virtual std::time_t   Time(unsigned int year) = 0;

    virtual void Init() = 0;
    virtual void Spi(int addr, unsigned int value) = 0;

    virtual ~IRealTimeClock()
        {}

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
