// K7SysDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <K7SysDeviceMap_Mb.h>
#include <K7SysMemoryMap_Mb.h>
#include <K7DeviceDriver_Mb.h>
#include <Gps_Csac_Mb.h>
#include <Gps_Ieee1588_Mb.h>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{
//===========================================================================
//  CLASS K7SysCsacPlugin  -- Hardware Plugin impl for Jackson Labs CSAC
//===========================================================================

class K7SysCsacPlugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() {  Owner->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)  // antenna not implemented in hardware
                          {  return gps_value;  }
    virtual bool    PGPS_IsLocked(bool /*gps_value*/)
                          {  return Owner->IsLocked();  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return Owner->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  Owner->UartData(c); }
    virtual char             PGPS_UartData()
                                {  return Owner->UartData();  }

    virtual void             PGPS_UartBaud(IGps::BaudRates /*baud*/)   // don't support changing baud rates?
                                {  /*Owner->Baud(baud);*/  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return Owner->Baud();  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {  Owner->GpsEnabled(state);  }
    virtual bool          PGPS_HdwEnabled()
                                {  return Owner->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/)  {}

    K7SysCsacPlugin(K7SysTripleUart * owner);
    virtual ~K7SysCsacPlugin()
        {}

private:
     K7SysTripleUart * Owner;

};

//---------------------------------------------------------------------------
//  constructor for class K7SysCsacPlugin
//---------------------------------------------------------------------------

K7SysCsacPlugin::K7SysCsacPlugin(K7SysTripleUart * owner)
    : Owner(owner)
{
}

//---------------------------------------------------------------------------
//   K7SysCsacPlugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void K7SysCsacPlugin::PGPS_OnInit()
{
    Owner->Uart(0).PutString("SYNC:OUT:1PPS:RESET ON\r\n");    // PPS from power up
    Owner->Uart(0).PutString("GPS:GPZDA 1\r\n");              // emit GPZDA once per second
    Owner->Uart(0).PutString("GPS:GPGGA 1\r\n");              // emit GPGGA once per second
}

//===========================================================================
//  CLASS K7SysIeee1588Plugin  -- Hardware Plugin impl for IEEE1588 intf
//===========================================================================

class K7SysIeee1588Plugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() {  GpsUart->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)  // antenna not implemented in hardware
                          {  return gps_value;  }
    virtual bool    PGPS_IsLocked(bool gps_value)
                          {  return gps_value;  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return IeeeDev->Uart(0).GetFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  IeeeDev->Uart(0).Port(c); }
    virtual char             PGPS_UartData()
                                {  return IeeeDev->Uart(0).Port();  }

    virtual void             PGPS_UartBaud(IGps::BaudRates /*baud*/)   // don't support changing baud rates?
                                {  /*IeeeDev->Uart(0).SetBaud(baud);*/  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return IGps::b115200;  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {
                                GpsUart->GpsEnabled(state);
                                }
    virtual bool          PGPS_HdwEnabled()
                                {  return GpsUart->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/)  {}

    K7SysIeee1588Plugin(K7SysTripleUart * gps_uart, K7Ieee1588 * ieee_dev);
    virtual ~K7SysIeee1588Plugin()
        {}

private:
    K7SysTripleUart * GpsUart;
    K7Ieee1588 * IeeeDev;
};

//---------------------------------------------------------------------------
//  constructor for class K7SysIeee1588Plugin
//---------------------------------------------------------------------------

K7SysIeee1588Plugin::K7SysIeee1588Plugin(K7SysTripleUart * gps_uart, K7Ieee1588 * ieee_dev)
    : GpsUart(gps_uart), IeeeDev(ieee_dev)
{
}

//---------------------------------------------------------------------------
//   K7SysIeee1588Plugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void K7SysIeee1588Plugin::PGPS_OnInit()
{
    // we can't config thru serial port

    // but can we do the IEEE init here?
    IeeeDev->InitIeee1588();

}


//===========================================================================
//  CLASS K7SysGPS_Callback -- X6 GPS Hardware Callback Interface
//===========================================================================

class K7SysGPS_Callback : public IGPS_Callback
{
public:
    K7SysGPS_Callback(K7SysDeviceMap * owner)
        :  Owner(owner)
        {}
    virtual ~K7SysGPS_Callback()
        {}

    virtual void InterruptEnable(bool state)
                    {  Owner->InterruptEnable(state);  }

private:
    K7SysDeviceMap *  Owner;
};

//===========================================================================
//  CLASS K7SysDeviceMap  -- Register & low level support for Sbc_ComEx
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysDeviceMap
//---------------------------------------------------------------------------
//
//  IDROM Page Numbers

const unsigned int System_IdromPage   = 0;
const unsigned int Assembly_IdromPage = 1;
const unsigned int User_IdromPage     = 3;

using namespace K7Sys_MemoryMap;

namespace  // nameless
{
X6Info::MapRegisters K7SysInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmSubrevision,        //  BitstreamDate_Addr - not used!
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

K7SysLedReset::MapRegisters K7SysPeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    mmExtIo,              //  ExtLedButton_Addr
    mmExtIoDirection,     //  ExtLedDirection_Addr;
    };

K7FpaLoader::MapRegisters K7SysVsPromRegAddrs =
    {
    mmVsProm_ControlAddr,  //  AddressControl_Addr;
    mmVsProm_Data,         //  Data_Addr
    mmVsProm_Opcode,       //  Opcode_Addr;
    };

FmcStd_I2C_Bus::MapRegisters K7SysI2CBusRegAddrs =
    {
    mmTempSensorI2C,       //  I2CControl_Addr
    };

FmcStd_I2C_Bus::I2C_SubAddr K7SysI2CBus_SubAddrs =   //(left justified to byte)
    {
    0x98,                //  TempControl_SubAddr  (left justified)
    0xA0,                //  FmcIdrom_SubAddr  - NOT present
    };

K7FlashRom::MapRegisters K7SysFlashRomRegAddrs =
    {
    mmRom_AddrCtl,         //  AddressControl_Addr
    mmRom_Data,            //  Data_Addr
    mmRom_Opcode,          //  Opcode_Addr
    };

X6Thermal::MapRegisters K7ThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

K7SysInformation::MapRegisters K7Sys_SysInfoRegAddrs =
    {
    mmFpgaPower,           //  FpgaPower_Addr;
    mmSsdPower,            //  SsdPower_Addr;
    mmK7Config,            //  K7Config_Addr;
    };

K7SysLcdDisplay::MapRegisters K7Sys_LcdRegAddrs =
    {
    mmLcdWdata,            //  WriteData_Addr;
    mmLcdRdata,            //  ReadData_Addr;
    };

LTC2494SpiDevice::Info K7SysSpiRegAddrs =
{
    mmLTC_SPI_Control,  //Write Offset
    mmLTC_SPI_Stat,     //Read Offset
};

K7SysLcdDisplaySL::MapRegisters K7Sys_LcdSLRegAddrs =
    {
    mmLcdSLWdata,            //  WriteData_Addr;
    };

//K7SysExtIo::MapRegisters K7SysExtIoRegAddrs =
  //  {
   // mmExtIo,            //  ExtIo_Addr;
   // };

K7SysTripleUart::MapRegisters K7SysUartRegAddrs =
    {
    mmUart0_Ctl,            //  Uart0Ctrl_Addr;
    mmUart0_Tx,             //  Uart0Tx_Addr;
    mmUart0_Rx,             //  Uart0Rx_Addr;
    mmUart1_Ctl,            //  Uart1Ctrl_Addr;
    mmUart1_Tx,             //  Uart1Tx_Addr;
    mmUart1_Rx,             //  Uart1Rx_Addr;
    mmUart2_Ctl,            //  Uart2Ctrl_Addr;
    mmUart2_Tx,             //  Uart2Tx_Addr;
    mmUart2_Rx,             //  Uart2Rx_Addr;
    mmUart_Mode,            //  UartMode_Addr;
    mmUart_IntStatus,       //  UartInterrupt_Addr;
    mmUart_PpsArm,          //  UartPpsArm_Addr
    };

K7Ieee1588::MapRegisters K7Sys_Ieee1588RegAddrs =
    {
    mm1588_Ctrl,            //  Control_Addr;
    mm1588_ClkId_Lo,        //  ClockID_Lo_Addr;
    mm1588_ClkId_Hi,        //  ClockID_Hi_Addr;
    mm1588_Time,            //  Time_Addr;
    mm1588_EEPromI2C,       //  I2C_Addr;
    mm1588_Uart_Ctl,        //  Uart_Ctrl_Addr;
    mm1588_Uart_Tx,         //  Uart_Tx_Addr;
    mm1588_Uart_Rx,         //  Uart_Rx_Addr;
    mm1588_Int,             //  Int_Addr;
    mm1588_Arm,             //  Arm_Addr;
    };

IRIG_RealTimeClock::MapRegisters K7Sys_IrigRegAddrs =
    {
    mmIrig_Date,            //  Date_Addr;
    mmIrig_FracSecs,        //  FractionalSecs_Addr;
    mmIrig_DdsCtrl,         //  DdsControl_Addr;
    mmIrig_SpiCtrl,         //  DdsSpiControl_Addr;
    mmIrig_SpiData,         //  DdsSpiData_Addr;
    mmIrig_DesiredClks,     //  ClocksPerSymbol_Addr;
    mmIrig_FreqTuning,      //  DdsTuning_Addr;
    };

}  // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class K7SysDeviceMap
//---------------------------------------------------------------------------

K7SysDeviceMap::K7SysDeviceMap()
    :  DeviceDriver(new K7SysDeviceDriver),
       WB_Periph             (LogicMemory, wbPeripheral),
       WB_Loader             (LogicMemory, wbLogicLoader),
       WB_Thermal            (LogicMemory, wbTempControl),
       WB_LCD                (LogicMemory, wbLCD),
       WB_FlashRom           (LogicMemory, wbROM),
       WB_Uart               (LogicMemory, wbUart),
       WB_Irig               (LogicMemory, wbIRIG),
       WB_Ieee1588           (LogicMemory, wbIeee1588),
       Info_SDev             (this, WB_Periph, PortMemory, K7SysInfoRegAddrs),
       Periph_SDev           (this, WB_Periph, K7SysPeripheralRegAddrs),
     // ExtIo_SDev            (this, WB_Periph, K7SysExtIoRegAddrs),
       SysInfo_SDev          (this, WB_Periph, K7Sys_SysInfoRegAddrs),
       K7Thermal_SDev        (this, WB_Thermal, K7ThermalRegAddrs),
       BoardTempBus_SDev     (this, WB_Periph, K7SysI2CBusRegAddrs, K7SysI2CBus_SubAddrs),
       VsProm_SDev           (this, K7FpaLoader::romS25FL128S_Sys, WB_Loader, K7SysVsPromRegAddrs),
       K7Lcd_SDev            (this, WB_LCD, K7Sys_LcdRegAddrs),
       K7LcdSL_SDev          (this, WB_LCD, K7Sys_LcdSLRegAddrs),
       FlashRom_SDev         (this, K7FlashRom::romS25FL128S_Sys, WB_FlashRom, K7SysFlashRomRegAddrs),
       Gps_SDev              (this, new K7SysGPS_Callback(this), WB_Uart, K7SysUartRegAddrs),
       Ieee1588_SDev         (this, WB_Ieee1588, K7Sys_Ieee1588RegAddrs),
       IRIG_SDev             (this, WB_Irig, K7Sys_IrigRegAddrs),
       SelfMeasure_SDev      (this),
       System_PG             (FlashRom_SDev, System_IdromPage),
       Assembly_PG           (FlashRom_SDev, Assembly_IdromPage),
       UserRom_PG            (FlashRom_SDev, User_IdromPage),
       PciControlReg         (PortMemory, mmPciControl),
	   InterruptStatusReg    (PortMemory, mmInterruptStatus),
       InterruptEnableReg    (PortMemory, mmInterruptEnable),
       CustomIntSourceReg    (WB_Periph, mmCustomIntSrc),
       IdRom_Intf            (this),
       Isr(*this), Debug(true)
{
    ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  destructor for class K7SysDeviceMap
//---------------------------------------------------------------------------

K7SysDeviceMap::~K7SysDeviceMap()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t K7SysDeviceMap::BoardCount()
{
	return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  K7SysDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  K7SysDeviceMap::Name() const
{
    static const char * ModuleName[] =
        { "K7Sys" };

    return ModuleName[Info().PciLogicType()];
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  K7SysDeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Open()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Open()
{
    Debug.Log("K7SysDeviceMap::Open()");
    if(DeviceDriver->Opened())
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));
    LogicMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->LogicMemoryAddress()));

    InterruptEnable(false);
    Isr.Open();

    //
    //  After Open Behavior
    OpenHardware();

}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Close()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Close()
{
    Debug.Log("K7SysDeviceMap::Close()");
    if(!DeviceDriver->Opened())
        return;

    InterruptEnable(false);

    Isr.Close();
    DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Reset()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Reset()
{
    Debug.Log("+K7SysDeviceMap::Reset()");
    Debug.Log("-K7SysDeviceMap::Reset()");
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::MReset() - Master Reset
//---------------------------------------------------------------------------

void  K7SysDeviceMap::MReset()
{
    Debug.Log("+K7SysDeviceMap::MReset()");
    PciControlReg.BusmasterReset.Value(false);  // apply BM Reset (active low)
    PciControlReg.BusmasterReset.Value(true);   // release BM Reset
    PciControlReg.MasterReset.Value(true);      // Apply "Board" reset (autoclear)
    PciControlReg.MasterReset.Value(false);     // but we need to clear shadow
    Debug.Log("-K7SysDeviceMap::MReset()");
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Led()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Led(bool value)
{
    PciControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Led()
//---------------------------------------------------------------------------

bool  K7SysDeviceMap::Led() const
{
    return PciControlReg.Led;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Start() - turn on clocks & triggers
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Start()
{
    PreconfigureHardware();
    ConfigureHardware();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Stop()  turn off clocks & triggers
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Stop()
{
    StopHardware();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  K7SysDeviceMap::Target() const
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  K7SysDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 K7SysDeviceMap::PeekPort(int offset)
{
    ii32 * PortBasePtr = PortMemory.Base();
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void K7SysDeviceMap::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = PortMemory.Base();
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32 K7SysDeviceMap::PeekLogic(int offset)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    return LogicBasePtr[offset];
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void K7SysDeviceMap::PokeLogic(int offset, ii32 value)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    LogicBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::InterruptEvent() --
//---------------------------------------------------------------------------

Event * K7SysDeviceMap::InterruptEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::InterruptDepth() --
//---------------------------------------------------------------------------

size_t K7SysDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::InterruptAck() --
//---------------------------------------------------------------------------

unsigned int K7SysDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void K7SysDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    //ScopedTraceRef ccc(Debug, "K7SysDeviceMap::InterruptDispatch()");
    // dispatch UART interrupt
    Gps_SDev.Gps().DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void K7SysDeviceMap::InterruptEnable(bool state)
{
    ScopedTraceRef ccc(Debug, "K7SysDeviceMap::InterruptEnable()");
    Debug.Log("state=", (int)state);

    InterruptEnableReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
  //  ExtIo_SDev.ConstructCustomParts();
    SysInfo_SDev.ConstructCustomParts();
    BoardTempBus_SDev.ConstructCustomParts();
    K7Thermal_SDev.ConstructCustomParts();
    Gps_SDev.ConstructCustomParts();
    Ieee1588_SDev.ConstructCustomParts();
    IRIG_SDev.ConstructCustomParts();
    SelfMeasure_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::OpenHardware()
{
    Debug.Log("+K7SysDeviceMap::OpenHardware()");
    //
    //  Reset the device
    MReset();
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
  //  ExtIo_SDev.OpenHardware();
    SysInfo_SDev.OpenHardware();
    BoardTempBus_SDev.OpenHardware();
    K7Thermal_SDev.OpenHardware();
    K7Lcd_SDev.OpenHardware();
    K7LcdSL_SDev.OpenHardware();
    Gps_SDev.OpenHardware();
    Ieee1588_SDev.OpenHardware();
    IRIG_SDev.OpenHardware();
    SelfMeasure_SDev.OpenHardware();
    //
    //  Read all ROM pages to Init Cache
	//System_PG.LoadFromRom();
	//AdcCal_PG.LoadFromRom();
	//DacCal_PG.LoadFromRom();
    //UserRom_PG.LoadFromRom();
    Debug.Log("-K7SysDeviceMap::OpenHardware()");
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  K7SysDeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
  //  ExtIo_SDev.PreconfigureHardware();
    SysInfo_SDev.PreconfigureHardware();
    BoardTempBus_SDev.PreconfigureHardware();
    K7Thermal_SDev.PreconfigureHardware();
    K7Lcd_SDev.PreconfigureHardware();
    K7LcdSL_SDev.PreconfigureHardware();
    Gps_SDev.PreconfigureHardware();
    Ieee1588_SDev.PreconfigureHardware();
    IRIG_SDev.PreconfigureHardware();
    SelfMeasure_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  K7SysDeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
  //  ExtIo_SDev.ConfigureHardware();
    SysInfo_SDev.ConfigureHardware();
    BoardTempBus_SDev.ConfigureHardware();
    K7Thermal_SDev.ConfigureHardware();
    K7Lcd_SDev.ConfigureHardware();
    K7LcdSL_SDev.ConfigureHardware();
    Gps_SDev.ConfigureHardware();
    Ieee1588_SDev.ConfigureHardware();
    IRIG_SDev.ConfigureHardware();
    SelfMeasure_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::StopHardware() -- set hardware for 'off' condition
//---------------------------------------------------------------------------

void  K7SysDeviceMap::StopHardware()
{
    // Subdevice Configuration
    Periph_SDev.StopHardware();
    //Ieee1588_SDev.StopHardware();
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::GetGpsCount() --
//---------------------------------------------------------------------------
//  Count of GPS devices to be created and installed

size_t  K7SysDeviceMap::GetGpsCount()
{
    return 2;
}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::MakeOneGpsDevice() -- Build custom Gps
//---------------------------------------------------------------------------
//  Will be called once per GPS device set up.  Use IDX to distinguish them

IGps *  K7SysDeviceMap::MakeOneGpsDevice(size_t idx)
{
    ScopedTraceRef ccc(Debug, "K7SysDeviceMap::MakeOneGpsDevice()");
    Debug.Log("Idx", idx);
    //  Make our local Gps objects (subdev will delete it)
    if (idx==0)
        return new Gps_CSACPlugin(new K7SysCsacPlugin(&Gps_SDev));
    else
        return new Gps_Ieee1588Plugin(new K7SysIeee1588Plugin(&Gps_SDev, &Ieee1588()) );

}

//---------------------------------------------------------------------------
//  K7SysDeviceMap::MakeLTC2494() -- Get LTC2494 Device object
//---------------------------------------------------------------------------

LTC2494 *  K7SysDeviceMap::MakeLTC2494()
{
    return new K7SysLTC2494(WB_Periph, K7SysSpiRegAddrs, "K7Sys_LTC2494");
}


//===========================================================================
//  CLASS K7SysLTC2494 --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7SysLTC2494
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  K7SysLTC2494::K7SysLTC2494() -- Constructor
//---------------------------------------------------------------------------

K7SysLTC2494::K7SysLTC2494(IAddressingSpace &space,LTC2494SpiDevice::Info &regs, std::string tag)
    : LTC2494(space,regs,tag)
{

}

//---------------------------------------------------------------------------
//  K7SysLTC2494::~K7SysLTC2494() -- Destructor
//---------------------------------------------------------------------------

K7SysLTC2494::~K7SysLTC2494()
{

}

//---------------------------------------------------------------------------
//  K7SysLTC2494::ConfigureBoardDefualts() -- config board defualts
//---------------------------------------------------------------------------

void K7SysLTC2494::ConfigureBoardDefaults()
{
     //Channel Mapping
     ChannelTypeTable.push_back(SingleEnded); //Channel 0
     ChannelTypeTable.push_back(SingleEnded); //Channel 1
     ChannelTypeTable.push_back(SingleEnded); //Channel 2
     ChannelTypeTable.push_back(SingleEnded); //Channel 3
     ChannelTypeTable.push_back(SingleEnded); //Channel 4
     ChannelTypeTable.push_back(SingleEnded); //Channel 5
     ChannelTypeTable.push_back(SingleEnded); //Channel 6
     ChannelTypeTable.push_back(SingleEnded); //Channel 7
     ChannelTypeTable.push_back(SingleEnded); //Channel 8
     ChannelTypeTable.push_back(SingleEnded); //Channel 9
     ChannelTypeTable.push_back(SingleEnded); //Channel 10
     ChannelTypeTable.push_back(SingleEnded); //Channel 11
     ChannelTypeTable.push_back(SingleEnded); //Channel 12
     ChannelTypeTable.push_back(SingleEnded); //Channel 13
     ChannelTypeTable.push_back(SingleEnded); //Channel 14
     ChannelTypeTable.push_back(SingleEnded); //Channel 15
     MapChannels();

     //configuring defualt bits
     ConfigBits.EN  = 1;
     ConfigBits.EN2 = 1;
     ConfigBits.FA  = 0;
     ConfigBits.FB  = 0;
     ConfigBits.SPD = 0; // dont care if temp measurement
     ConfigBits.GS0 = 0; //
     ConfigBits.GS1 = 0; // dont care if temp measurement
     ConfigBits.GS2 = 0; //

    //initialize Scale table
    ScaleTable.push_back(1.25);
    ScaleTable.push_back(1.25);
    ScaleTable.push_back(6.2875);
    ScaleTable.push_back(4.15);
    ScaleTable.push_back(2.25);
    ScaleTable.push_back(15.0625);
    ScaleTable.push_back(6.2875);
    ScaleTable.push_back(4.15);
    ScaleTable.push_back(3.13125);
    ScaleTable.push_back(3.13125);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
}



} // namespace Innovative


