// K7SysDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef K7SysDeviceMap_MbH
#define K7SysDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <FmcSubDevice_Stream_Mb.h>      // Thermal Bus
#include <K7Support_Mb.h>
#include <K7SubDevice_Misc_Mb.h>
#include <SimpleInterrupt_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS K7SysDeviceMap  -- Register support for K7 System Unit
//===========================================================================

class K7SysDeviceMap :  public ISuppliesK7SysRequirements,
                        public ISimpleInterrupt
{
public:
    friend class K7Sys;
    friend class K7SysGPS_Callback;
    //
    //  Member Functions
    K7SysDeviceMap(/*K7Sys * owner*/);
    virtual ~K7SysDeviceMap();

    // Methods
    virtual size_t  InterruptDepth();
	size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);

    void            Open();
    void            Close();
    void            Reset();
    void            Start();
    void            Stop();
    void            SetLcdType();

    void  SelectGps(size_t idx)
        {
        Gps_SDev.SelectGps(idx);
        }

    std::string         Name() const;
    std::string         Revision() const;

    void            Led(bool value);
    bool            Led() const;

    void            CustomInterruptSource(unsigned int src)
        { CustomIntSourceReg.Source.Value(src);  }
    unsigned int    CustomInterruptSource()
        { return CustomIntSourceReg.Source.Value();  }

    //  Hardware Device Access
    const IX6LogicVersionInfo &     Info() const      {  return Info_SDev;  }
    IFpaLoader &                    VsProm()          {  return VsProm_SDev.VsProm();  }
    IK7_IdRom &                     IdRom()           {  return IdRom_Intf;   }
    IX6ThermalDevice &              Thermal()         {  return K7Thermal_SDev;  }
    IK7LcdDevice &                  Lcd()             {  return K7Lcd_SDev;  }
    IK7LcdDeviceSL &                LcdSL()           {  return K7LcdSL_SDev;  }
    ISelfMeasurement &              SelfMeasure()     {  return SelfMeasure_SDev.SelfM();  }


    K7Ieee1588 &                   Ieee1588()             {  return Ieee1588_SDev;  }
    IRealTimeClock &               Irig()                {  return IRIG_SDev;  }

    //  Subdevice Customization Handlers
    virtual size_t             GetGpsCount();
    virtual IGps *             MakeOneGpsDevice(size_t idx);
    virtual LTC2494    *       MakeLTC2494();

    //  Debug Support
    ii32            PeekPort(int offset);
    void            PokePort(int offset, ii32 value);
    ii32            PeekLogic(int offset);
    void            PokeLogic(int offset, ii32 value);

    // Properties
    void            Target(int target);
    int             Target() const;

	AddressingSpace    PortMemory;
    AddressingSpace    LogicMemory;

    // Debug
    K7FpaLoader &                    DVsProm()          {  return VsProm_SDev;  }


   void   MReset();    // Master Reset


protected:
    //
    //  Member Data
    InnovativeKernel::DualRegionIsrDeviceDriver *  DeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Loader;
    WishboneBusSpace       WB_Thermal;
    WishboneBusSpace       WB_LCD;
    WishboneBusSpace       WB_FlashRom;
    WishboneBusSpace       WB_Uart;
    WishboneBusSpace       WB_Irig;
    WishboneBusSpace       WB_Ieee1588;
    //
    //  Subdevices
    X6Info                  Info_SDev;
    K7SysLedReset           Periph_SDev;
//    K7SysExtIo              ExtIo_SDev;
    K7SysInformation        SysInfo_SDev;
    X6Thermal               K7Thermal_SDev;
    FmcStd_I2C_Bus          BoardTempBus_SDev;
    K7FpaLoader             VsProm_SDev;
    K7SysLcdDisplay         K7Lcd_SDev;
    K7SysLcdDisplaySL       K7LcdSL_SDev;
    K7FlashRom              FlashRom_SDev;
    K7SysTripleUart         Gps_SDev;
    K7Ieee1588              Ieee1588_SDev;
    IRIG_RealTimeClock      IRIG_SDev;
    SelfMeasurementDevice   SelfMeasure_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    K7AssemblyIdromPage          Assembly_PG;
    X6IdromPage                  UserRom_PG;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  Class CustomIntSourceRegister
    class CustomIntSourceRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup       Source;

	public:
        CustomIntSourceRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Source(*this, 0, 3)
            {
            }
    };
    //
    //  Registers
    //
	//  Class PciControlRegister
    class PciControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       MasterReset;
        RegisterBit       Led;
        RegisterBit       BusmasterReset;

	public:
        PciControlRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
               MasterReset( *this, 0 ),
               Led( *this, 1 ),
               BusmasterReset( *this, 3 )
            {
            //  Default setup ????
            }
    };

	//  ...Interrupt Registers
    PciControlRegister                 	PciControlReg;
    Register                            InterruptStatusReg;
    ShadowRegister                      InterruptEnableReg;
    CustomIntSourceRegister             CustomIntSourceReg;

private:
    // Interface
    virtual Event *         InterruptEvent();
    virtual unsigned int    InterruptAck();
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);

    void    ConstructCustomParts();
    void    OpenHardware();
    void    PreconfigureHardware();
    void    ConfigureHardware();
    void    StopHardware();

    //=======================================================================
    //  CLASS K7SysIdromIo  -- Access to IDROM Objects
    //=======================================================================
    class K7SysIdromIo : public IK7_IdRom
    {
    public:

    virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
    //  Page access
    virtual X6SystemIdromPage &   System()   { return Owner->System_PG;  }
    virtual K7AssemblyIdromPage & Assembly() { return Owner->Assembly_PG; }
    virtual X6IdromPage &         User()     { return Owner->UserRom_PG;  }

    K7SysIdromIo(K7SysDeviceMap * owner) :  Owner(owner)
        {}
    virtual ~K7SysIdromIo()
        {}

    private:
        K7SysDeviceMap *  Owner;
    };

    K7SysIdromIo  IdRom_Intf;
    friend class  K7SysIdromIo;

    // Data
    SimpleInterrupt         Isr;
    Trace                   Debug;
};


//===========================================================================
//  CLASS K7SysLTC2494 -- 16-Channel ADC chip for K7
//===========================================================================

class K7SysLTC2494 : public LTC2494
{
public:
    //ctor
    //ctor
    K7SysLTC2494(IAddressingSpace &space,LTC2494SpiDevice::Info &Regs,
                  std::string tag="");
    ~K7SysLTC2494();

private:
    //private members
    virtual void                ConfigureBoardDefaults();

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
