//---------------------------------------------------------------------------
// K7Sys_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <K7Sys_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS K7Sys  -- X6 Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class K7Sys
//---------------------------------------------------------------------------

K7Sys::K7Sys()
{
}

//---------------------------------------------------------------------------
//  destructor for class K7Sys
//---------------------------------------------------------------------------

K7Sys::~K7Sys()
{
}

//---------------------------------------------------------------------------
//  K7Sys::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t K7Sys::BoardCount()
{
	return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  K7Sys::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  K7Sys::LicenseCheck(unsigned int & code)
{
	return Device.LicenseCheck(code);
}

//---------------------------------------------------------------------------
//  K7Sys::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  K7Sys::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  K7Sys::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  K7Sys::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  K7Sys::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  K7Sys::GreenLed() const
{
    return Device.Periph_SDev.GreenLed();
}

//---------------------------------------------------------------------------
//  K7Sys::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  K7Sys::GreenLed( bool value )
{
    Device.Periph_SDev.GreenLed(value);
}

//---------------------------------------------------------------------------
//  K7Sys::BlueLed() -- Read the Blue LED state
//---------------------------------------------------------------------------

bool  K7Sys::BlueLed() const
{
    return Device.Periph_SDev.BlueLed();
}

//---------------------------------------------------------------------------
//  K7Sys::BlueLed() -- Set the Blue LED state of the board
//---------------------------------------------------------------------------

void  K7Sys::BlueLed( bool value )
{
    Device.Periph_SDev.BlueLed(value);
}

//---------------------------------------------------------------------------
//  K7Sys::ExternalLed() -- Read the external LED state
//---------------------------------------------------------------------------

int  K7Sys::ExternalLed() const
{
    return Device.Periph_SDev.ExternalLed();
}

//---------------------------------------------------------------------------
//  K7Sys::ExternalLed() -- Set the external LED state of the board
//---------------------------------------------------------------------------

void  K7Sys::ExternalLed( int value )
{
    Device.Periph_SDev.ExternalLed(value);
}

//---------------------------------------------------------------------------
//  K7Sys::ExternalLed() -- Set the external LED state of the board
//---------------------------------------------------------------------------

void  K7Sys::ExternalLed(bool one, bool two, bool three, bool four)
{
    Device.Periph_SDev.LedOnOff(one, two, three, four);
}

//---------------------------------------------------------------------------
//  K7Sys::Button() -- Read the external button state
//---------------------------------------------------------------------------

int  K7Sys::Button() const
{
    return Device.Periph_SDev.Button();
}

//---------------------------------------------------------------------------
//  K7Sys::ExtIoData() -- access to Lo Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  K7Sys::ExtIoData()
{
    return Device.Periph_SDev.ExtIoData();
}

//---------------------------------------------------------------------------
//  K7Sys::ExtIoConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

void  K7Sys::ExtIoConfig(unsigned int mask)
{
    Device.Periph_SDev.ExtIoConfig(mask);
}

//---------------------------------------------------------------------------
//  K7Sys::ExtIoConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

unsigned int  K7Sys::ExtIoConfig() const
{
    return Device.Periph_SDev.ExtIoConfig();
}

//---------------------------------------------------------------------------
//  K7Sys::ExtIoOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

void  K7Sys::ExtIoOutputEnable(int output_bit, bool output_enabled)
{
    Device.Periph_SDev.ExtIoOutputEnable(output_bit, output_enabled);
}

//---------------------------------------------------------------------------
//  K7Sys::ExtIoOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

bool  K7Sys::ExtIoOutputEnable(int output_bit) const
{
    return Device.Periph_SDev.ExtIoOutputEnable(output_bit);
}

//---------------------------------------------------------------------------
//  K7Sys::CustomInterruptSource() -- change interrupt source
//---------------------------------------------------------------------------

void  K7Sys::CustomInterruptSource(K7SysInterruptSources src)
{
    return Device.CustomInterruptSource(static_cast<unsigned int>(src));
}

//---------------------------------------------------------------------------
//  K7Sys::CustomInterruptSource() -- read current interrupt source
//---------------------------------------------------------------------------

K7Sys::K7SysInterruptSources  K7Sys::CustomInterruptSource()
{
    unsigned int src = Device.CustomInterruptSource();
    return static_cast<K7SysInterruptSources>(src);
}

//---------------------------------------------------------------------------
//  K7Sys::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  K7Sys::Open()
{
    Device.Open();
}

//---------------------------------------------------------------------------
//  K7Sys::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  K7Sys::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  K7Sys::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  K7Sys::Reset()
{
    Device.Reset();    // Board Reset
}

////---------------------------------------------------------------------------
////  K7Sys::Start() -- Start the clocks and triggers
////---------------------------------------------------------------------------
//
//void  K7Sys::Start()
//{
//	Device.Start();
//}
//
////---------------------------------------------------------------------------
////  K7Sys::Stop() -- Stop the clocks and triggers
////---------------------------------------------------------------------------
//
//void  K7Sys::Stop()
//{
//	Device.Stop();
//}

//---------------------------------------------------------------------------
//  K7Sys::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------
//
//void  K7Sys::Log( int type, int value )
//{
//    Device.Log(type, value);
//}
//

//---------------------------------------------------------------------------
//  K7Sys::GpsCtrl() -- Return reference to Gps Control
//---------------------------------------------------------------------------

IX6_GpsIo &  K7Sys::GpsCtrl()
{
    return Device.Gps_SDev;
}


//---------------------------------------------------------------------------
//  K7Sys::Info() --
//---------------------------------------------------------------------------

const IX6LogicVersionInfo & K7Sys::Info() const
{
    return Device.Info();
}

//---------------------------------------------------------------------------
//  K7Sys::VsProm() --
//---------------------------------------------------------------------------

IFpaLoader &  K7Sys::VsProm()
{
	return Device.VsProm();
}

//---------------------------------------------------------------------------
//  K7Sys::Thermal() --
//---------------------------------------------------------------------------

IX6ThermalDevice &  K7Sys::Thermal()
{
	return Device.Thermal();
}

//---------------------------------------------------------------------------
//  K7Sys::Lcd() --
//---------------------------------------------------------------------------

IK7LcdDevice &  K7Sys::Lcd()
{
	return Device.Lcd();
}

//---------------------------------------------------------------------------
//  K7Sys::LcdSL() --
//---------------------------------------------------------------------------

IK7LcdDeviceSL &  K7Sys::LcdSL()
{
        return Device.LcdSL();
}

//---------------------------------------------------------------------------
//  K7Sys::Irig() --
//---------------------------------------------------------------------------

IRealTimeClock &  K7Sys::Irig()
{
	return Device.Irig();
}

//---------------------------------------------------------------------------
//  K7Sys::RawProm() --
//---------------------------------------------------------------------------

IVpxProm &  K7Sys::RawProm()
{
    return Device.VsProm_SDev.Raw();
}

//---------------------------------------------------------------------------
//  K7Sys::IdRom() --
//---------------------------------------------------------------------------

IK7_IdRom & K7Sys::IdRom()
{
    return Device.IdRom();
}

//---------------------------------------------------------------------------
//  K7Sys::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  K7Sys::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  K7Sys::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  K7Sys::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  K7Sys::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  K7Sys::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  K7Sys::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  K7Sys::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  K7Sys::AfterFiclStart() --
//---------------------------------------------------------------------------

void  K7Sys::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate(" 0 constant Fan 1 constant I1588Reg 2 constant I1588Rom ");

    engine.Evaluate(": ?K7Sys "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Fan I1588Reg I1588Rom\" "
                    "    cr  "
                    "    cr .\" ---- K7Sys Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?K7Sys  : display this message \" "
                    "    cr  ; ");
    //engine.Evaluate(": ?VpxEpc  ?K7Sys  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("K7Sys-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    //X6Board::AfterFiclStart(engine);
    //  Replace Callbacks to call FICL too
    //Device.Load_StreamDeviceEventCallbacks(
    //                new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  K7Sys::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  K7Sys::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  K7Sys::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  K7Sys::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  K7Sys::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  K7Sys::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  K7Sys::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  K7Sys::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  K7Sys::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  K7Sys::Do_DFetch(int dev, int addr)
{
    const int kFan = 0; const int kI1588Reg = 1; const int I1588Rom = 2;

    switch (dev)
        {
        default:
            return 0;
        case kFan:
            return Device.Thermal().Port(addr);
        case kI1588Reg:
            return Device.Ieee1588().TestReg(addr);
        case I1588Rom:
            return Device.Ieee1588().Rom(addr);
        }
}

//---------------------------------------------------------------------------
//  K7Sys::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  K7Sys::Do_DStore(int dev, int addr, int value)
{
    const int kFan = 0; const int kI1588Reg = 1; const int I1588Rom = 2;


    switch (dev)
        {
        default:
            break;
        case kFan:
            Device.Thermal().Port(addr, value);
            break;
        case kI1588Reg:
            Device.Ieee1588().TestReg(addr, value);
            break;
        case I1588Rom:
            Device.Ieee1588().Rom(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  K7Sys::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  K7Sys::Do_DFreq(int /*dev*/, float /*freq*/)
{
//    const int kPll = 0;  const int kVcxo = 1;  //const int kClkMux = 2;  const int kSmb1 = 3;
//    switch (dev)
//        {
//        default:
//            break;
//        case kPll:
//            Device.Clock_SDev.Pll().Frequency(freq);
//            Device.Clock_SDev.Pll().Reapply();
//            break;
//        case kVcxo:
//            Device.Clock_SDev.Vcxo().Frequency(freq);
//            break;
//        }
}

//---------------------------------------------------------------------------
//  K7Sys::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  K7Sys::Do_DFreqActual(int /*dev*/)
{
//    const int kPll = 0;  const int kVcxo = 1;  //const int kClkMux = 2;  const int kSmb1 = 3;
//    switch (dev)
//        {
//        default:
            return 0.0f;
//        case kPll:
//            return static_cast<float>(Device.Clock_SDev.Pll().FrequencyActual());
//        case kVcxo:
//            return static_cast<float>(Device.Clock_SDev.Vcxo().FrequencyActual());
//        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
