// K7Sys_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef K7Sys_MbH
#define K7Sys_MbH

#include <DataTypes_Mb.h>
#include <K7SysDeviceMap_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS K7Sys  -- K7Sys baseboard class
//===========================================================================

class K7Sys : public IFiclTarget
{

public:
    // Enumerations
    enum K7SysInterruptSources { ksInt_GpsPps, ksInt_Uart, ksInt_Pps1588, ksInt_Int1588,
                                 ksInt_ExtPpsIn };

    enum GpsType { gpsCSAC, gpsIeee1588 };
    //
    //  Member Functions
    K7Sys();
    virtual ~K7Sys();

    //  Informational Methods
	size_t                      BoardCount();
	bool                        LicenseCheck(unsigned int & code);

    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    int                         Target();
    void                        Target(int value);

    bool                        GreenLed() const;
    void                        GreenLed(bool value);

    bool                        BlueLed() const;
    void                        BlueLed(bool value);

    int                         ExternalLed() const;
    void                        ExternalLed(int value);
    void                        ExternalLed(bool one, bool two, bool three, bool four);

    int                         Button() const;

    //  Digital General Purpose Io
    virtual IBitManipulator &   ExtIoData();

    virtual void 			    ExtIoConfig(unsigned int);
    virtual unsigned int 	    ExtIoConfig() const;

    virtual void                ExtIoOutputEnable(int output_bit, bool output_enabled);
    virtual bool                ExtIoOutputEnable(int output_bit) const;

    void            CustomInterruptSource(K7SysInterruptSources src);
    K7SysInterruptSources    CustomInterruptSource();


    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
	//virtual void                Start();
    //virtual void                Stop();
    //
    //  Properties

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    const IX6LogicVersionInfo &  Info() const;
	IX6_GpsIo &                  GpsCtrl();

    void  SelectGps(GpsType gps)
        {
        Device.SelectGps(static_cast<size_t>(gps));
        }
    void  SelectGps(size_t idx)
        {
        Device.SelectGps(idx);
        }


	IFpaLoader &                 VsProm();
	IX6ThermalDevice &           Thermal();
    IK7LcdDevice &               Lcd();
    IK7LcdDeviceSL &             LcdSL();
    IK7_IdRom &                  IdRom();

	IVpxProm &                   RawProm();
	IRealTimeClock &             Irig();

public: //Pseudo protected

	K7SysDeviceMap * Debug() { return &Device; }

	K7SysTripleUart &   GpsT() { return Device.Gps_SDev;  }
	K7UartPart &        IeeeUart() {  return Device.Ieee1588_SDev.Uart(0);  }
    ISelfMeasurement &  SelfMeasure()    {  return Device.SelfMeasure(); }
	
	
	
	//FPGA type name function
        std::string FpgaName(const short Number)
        {
             switch(Number)
             {
                  case 0x00: return "LX75T";
                  default: return "Unknown";
             }
         }

protected:
    //
    //  Member Data
    K7SysDeviceMap       Device;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( K7Sys & board );
    friend AddressingSpace & PortMemorySpace( K7Sys & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( K7Sys & board )
{
    return board.Device.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( K7Sys & board )
{
    return board.Device.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

