// K7_SbcDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <K7_Sbc_Mb.h>
#include <K7_SbcDeviceMap_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <K7_SbcMemoryMap_Mb.h>
//#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace FmcCarrier_MemoryMap;
using namespace K7_SbcMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

//===========================================================================
//  CLASS K7_SbcDeviceMap  -- Register & low level support for X6-400M
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class K7_SbcDeviceMap
//---------------------------------------------------------------------------
const int    MaxInDevices      = 2;
const int    ChannelsPerDevice = 2;
const int    MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const unsigned int K7_Sbc_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int K7_Sbc_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int K7_Sbc_VeloOutStreamCount  = 1;   // Number of Velocia Outgoing PIDs for Data
const unsigned int K7_Sbc_VeloArbStreamCount  = 1;   // Number of Velocia Outgoing PIDs for ARB command channels

const unsigned int K7_Sbc_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

//---------------------------------------------------------------------------
//  constructor for class K7_SbcDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

K7_SbcDeviceMap::K7_SbcDeviceMap(K7_Sbc * owner)
    :  K7DeviceMap(owner),
       Owner(owner)
{
    //
    //  SubDevice Customization
    K7DeviceMap::ConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &K7_SbcDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &K7_SbcDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &K7_SbcDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &K7_SbcDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &K7_SbcDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &K7_SbcDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &K7_SbcDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class K7_SbcDeviceMap
//---------------------------------------------------------------------------

K7_SbcDeviceMap::~K7_SbcDeviceMap()
{
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OpenHardware()
{
    K7DeviceMap::OpenHardware();
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "K7_SbcDeviceMap::OnStreamPreconfigHandler()");

    // Family-common init
    K7DeviceMap::PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "K7_SbcDeviceMap::OnStreamConfigureHandler()");

    // Family-common init
    K7DeviceMap::ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    FmcCarrierRun(true);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  K7_SbcDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    FmcCarrierRun(false);

    //  Base class "on stream stop" behavior
    K7DeviceMap::StreamStopHardware();

	//  Forward notify to baseboard handler
	OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  K7_SbcDeviceMap::GetVeloInStreamCount()
{
    return K7_Sbc_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  K7_SbcDeviceMap::GetVeloOutStreamCount()
{
    return K7_Sbc_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetVeloOutCommandStreamCount() --
//--------------------------------------------------------------------------
//  Configure PIDs for Arb command channels-

unsigned int  K7_SbcDeviceMap::GetVeloOutCommandStreamCount()
{
    return K7_Sbc_VeloArbStreamCount;
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   K7_SbcDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   K7_SbcDeviceMap::GetPatternModeAlignment()
{
    return 16;   // standard K7 (FMC) alignment of 16 words
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   K7_SbcDeviceMap::GetPatternModeScaleFactor()
{
    return 1;   // standard K7 (FMC) scale factor of 1 words
}

//---------------------------------------------------------------------------
//  K7_SbcDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   K7_SbcDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return true;  // single memory
}



} // namespace Innovative


