//---------------------------------------------------------------------------
// K7_Sbc_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <K7_Sbc_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS K7_Sbc  -- K7_Sbc PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class K7_Sbc
//---------------------------------------------------------------------------

K7_Sbc::K7_Sbc()
    :  K7Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class K7_Sbc 
//---------------------------------------------------------------------------

K7_Sbc::~K7_Sbc()
{
    Close();
}

//---------------------------------------------------------------------------
//  K7_Sbc::AfterFiclStart() --
//---------------------------------------------------------------------------

void  K7_Sbc::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Fmc   ");

    engine.Evaluate(": ?K7_Sbc "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Fmc  Vcxo  SioPll  Dac0Spi \" "
                    "    cr  "
                    "    cr .\" ---- K7_Sbc Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?K7_Sbc  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate(": ?K7-Sbc  ?K7_Sbc  ; "  );    // alias
    engine.Evaluate(": ?K7Sbc  ?K7_Sbc  ; "  );    // alias

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->AfterFiclStart(engine);
        }

    engine.Evaluate("marker app");

    engine.Display("K7_Sbc-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    K7Board::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  K7_Sbc::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  K7_Sbc::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;
    switch (dev)
        {
        case kFmc:
        default:
            return FmcDaughter(0)->Do_DFetch(dev, addr);
//        case kPll:
//            return Device.AFEClk_SDev.Pll().Port(addr);
//        case kVcxo:
//            return Device.AFEClk_SDev.Vcxo().Port(addr);
//        case kSioPll:
//            return Device.SioPll_SDev.SioPll_I2C.Reg(addr);
//        case kDac0Spi:
//            return Device.Output().Spi(0, addr);
        }
}

//---------------------------------------------------------------------------
//  K7_Sbc::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  K7_Sbc::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;

    switch (dev)
        {
        case kFmc:
        default:
            FmcDaughter(0)->Do_DStore(dev, addr, value);
            break;
//        case kPll:
//            Device.AFEClk_SDev.Pll().Port(addr, value);
//            break;
//        case kVcxo:
//            Device.AFEClk_SDev.Vcxo().Port(addr, value);
//            break;
//        case kSioPll:
//            Device.SioPll_SDev.SioPll_I2C.Reg(addr, value);
//            break;
//        case kDac0Spi:
//            return Device.Output().Spi(0, addr, value);
        }
}

//---------------------------------------------------------------------------
//  K7_Sbc::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  K7_Sbc::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;
    switch (dev)
        {
        case kFmc:
        default:
            FmcDaughter(0)->Do_DFreq(dev, freq);
            break;
//        case kPll:
//            Device.AFEClk_SDev.Pll().Frequency(freq);
//            Device.AFEClk_SDev.Pll().Reapply();
//            break;
//        case kVcxo:
//            Device.AFEClk_SDev.Vcxo().Frequency(freq);
//            break;
        }
}

//---------------------------------------------------------------------------
//  K7_Sbc::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  K7_Sbc::Do_DFreqActual(int dev)
{
    const int kFmc = 0;
    switch (dev)
        {
        case kFmc:
        default:
            return FmcDaughter(0)->Do_DFreqActual(dev);
//        case kPll:
//            return static_cast<float>(Device.AFEClk_SDev.Pll().FrequencyActual());
//        case kVcxo:
//            return static_cast<float>(Device.AFEClk_SDev.Vcxo().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
