//  KernelPci_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <KernelPci_Mb.h>
#include <string>

namespace InnovativeKernel
{


//==============================================================================
// CLASS PCIConfigSpace.
//==============================================================================


//==============================================================================
//  CLASS PciId -- PCI ID Code
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class PciId
//------------------------------------------------------------------------------

PciId::PciId(unsigned int vendorId, unsigned int deviceId)
    :  VendorId(vendorId), DeviceId(deviceId)
{
}

//------------------------------------------------------------------------------
//  constructor for class PciId
//------------------------------------------------------------------------------

PciId::PciId(const PciId & other)
    :  VendorId(other.VendorId), DeviceId(other.DeviceId)
{
}

//==============================================================================
//  CLASS PciPosition -- PCI Position on the Bus
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class PciPosition
//------------------------------------------------------------------------------

PciPosition::PciPosition(unsigned int bus, unsigned int device, unsigned int function)
    :  Bus(bus), Device(device), Function(function)
{
}

//------------------------------------------------------------------------------
//  constructor for class PciPosition
//------------------------------------------------------------------------------

PciPosition::PciPosition(const PciPosition & other)
    :  Bus(other.Bus), Device(other.Device), Function(other.Function)
{
}

//==============================================================================
//  CLASS PciBoardLocation -- PCI Scan Return Structure
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class PciBoardLocation
//------------------------------------------------------------------------------

PciBoardLocation::PciBoardLocation(const PciId & board_id, const PciPosition & pos)
    :  ID(board_id), Location(pos)
{
}

//------------------------------------------------------------------------------
//  constructor for class PciBoardLocation
//------------------------------------------------------------------------------

PciBoardLocation::PciBoardLocation(const PciBoardLocation & other)
        :  ID(other.ID), Location(other.Location)
{
}

//=============================================================================
//  CLASS DeviceEntry  --  
//=============================================================================
//------------------------------------------------------------------------------
//  constructor for class DeviceEntry
//------------------------------------------------------------------------------

DeviceEntry::DeviceEntry(int bus, int device, int function)
    : Slot(bus, device, function),
	  FVendorId(0), FDeviceId(0), FSubvendorId(0), FSubsystemId(0)
{
}

//------------------------------------------------------------------------------
// DeviceEntry::VendorId() const
//------------------------------------------------------------------------------

unsigned int DeviceEntry::VendorId() const
{
	return FVendorId;
}

//------------------------------------------------------------------------------
// DeviceEntry::VendorId() 
//------------------------------------------------------------------------------

void DeviceEntry::VendorId(unsigned vid)
{
	FVendorId = vid;
}

//------------------------------------------------------------------------------
// DeviceEntry::DeviceId() const
//------------------------------------------------------------------------------

unsigned int DeviceEntry::DeviceId() const
{
	return FDeviceId;
}

//------------------------------------------------------------------------------
// DeviceEntry::DeviceId()
//------------------------------------------------------------------------------

void DeviceEntry::DeviceId(unsigned int did)
{
	FDeviceId = did;
}

//------------------------------------------------------------------------------
// DeviceEntry::SubsystemId() const
//------------------------------------------------------------------------------

unsigned int DeviceEntry::SubsystemId() const
{
	return FSubsystemId;
}

//------------------------------------------------------------------------------
// DeviceEntry::SubsystemId()
//------------------------------------------------------------------------------

void DeviceEntry::SubvendorId(unsigned int svid)
{
	FSubvendorId = svid;
}

//------------------------------------------------------------------------------
// DeviceEntry::SubvendorId() const
//------------------------------------------------------------------------------

unsigned int DeviceEntry::SubvendorId() const
{ 
	return FSubvendorId;
}

//------------------------------------------------------------------------------
// DeviceEntry::SubvendorId()
//------------------------------------------------------------------------------

void DeviceEntry::SubsystemId(unsigned int ssid)
{
	FSubsystemId = ssid;
}

//------------------------------------------------------------------------------
//  DeviceEntry::PositionFrom() - fill entry from config structure
//------------------------------------------------------------------------------

void  DeviceEntry::PositionFrom(const PciPosition & slot)
{
    Slot.Bus = slot.Bus;
    Slot.Device = slot.Device;
    Slot.Function = slot.Function;
}

//==============================================================================
//  CLASS FreeAlignedMemoryPtr -- Pointer to chunk of aligned memory
//==============================================================================
//------------------------------------------------------------------------------
//  housekeeping for class FreeAlignedMemoryPtr
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//  constructor for class FreeAlignedMemoryPtr -- takes a size in bytes
//------------------------------------------------------------------------------

FreeAlignedMemoryPtr::FreeAlignedMemoryPtr(size_t size_in_bytes, size_t align_to)
    : FOrigin(0), FAddress(0), FAlignTo(align_to)
{
    New(size_in_bytes);
}

//------------------------------------------------------------------------------
//  constructor for class FreeAlignedMemoryPtr -- wrap an existing buffer
//------------------------------------------------------------------------------

FreeAlignedMemoryPtr::FreeAlignedMemoryPtr(char * origin, size_t align_to)
    : FOrigin(origin), FAlignTo(align_to)
{
    FAddress = AddressFromOrigin(FOrigin);
}

//---------------------------------------------------------------------------
//  constructor for class FreeAlignedMemoryPtr --  default (uninitialized)
//---------------------------------------------------------------------------

FreeAlignedMemoryPtr::FreeAlignedMemoryPtr(size_t align_to)
    : FOrigin(0), FAddress(0), FAlignTo(align_to)
{
}

//---------------------------------------------------------------------------
//  destructor for class FreeAlignedMemoryPtr --
//---------------------------------------------------------------------------

FreeAlignedMemoryPtr::~FreeAlignedMemoryPtr()
{
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::New() --  Allocate block
//---------------------------------------------------------------------------

FreeAlignedMemoryPtr FreeAlignedMemoryPtr::New(size_t bytes)
{
    FOrigin = new char[bytes+FAlignTo];
    FAddress = AddressFromOrigin(FOrigin);

    return *this;
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::Delete() --  Deallocate block
//---------------------------------------------------------------------------

void FreeAlignedMemoryPtr::Delete()
{
    delete [] FOrigin;
    FOrigin = 0;
    FAddress = 0;
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::IsNull() --  Check pointer state
//---------------------------------------------------------------------------

bool FreeAlignedMemoryPtr::IsNull() const
{
    return FOrigin == 0;
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::Origin() --  Unaligned address
//---------------------------------------------------------------------------

char * FreeAlignedMemoryPtr::Origin() const
{
    return FOrigin;
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::Address() --  Aligned address
//---------------------------------------------------------------------------

unsigned int * FreeAlignedMemoryPtr::Address() const
{
    return FAddress;
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::AlignTo() --  Alignment parameter
//---------------------------------------------------------------------------

size_t FreeAlignedMemoryPtr::AlignTo() const
{
    return FAlignTo;
}

//---------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::AlignTo() --  Alignment parameter
//---------------------------------------------------------------------------

void FreeAlignedMemoryPtr::AlignTo(size_t align_to)
{
    FAlignTo = align_to;
}

//------------------------------------------------------------------------------
//  FreeAlignedMemoryPtr::AddressFromOrigin() --  Aligned from unaligned address
//------------------------------------------------------------------------------

unsigned int *  FreeAlignedMemoryPtr::AddressFromOrigin(char * origin)
{
    size_t original_loc = reinterpret_cast<size_t>(origin);
    size_t offset = FAlignTo - (original_loc%FAlignTo);

    char * addr_ptr = origin + offset;
    return reinterpret_cast<unsigned int *>(addr_ptr);

}

//---------------------------------------------------------------------------
}  // end namespace InnovativeKernel


