//
//  Kernel_Mb.cpp

#include <Kernel_Mb.h>

//---------------------------------------------------------------------------

namespace InnovativeKernel
{
//------------------------------------------------------------------------------
//   OnInterrupt()  --  Kernel ISR Handler
//------------------------------------------------------------------------------

BOOL _stdcall OnInterrupt(HWDEVICE* pDevice)
{
    DEVICE_EXTENSION* pExtension = (DEVICE_EXTENSION*)HwGetExtension(pDevice);
    DWORD   InterruptStatus;
    DWORD   InterruptStatusSave;
    BOOL    Rtn;

    InterruptStatusSave = HwReadLong( pDevice, pExtension->IntStatusRegionIdx,
                                               pExtension->IntStatusOffsetBytes );
    InterruptStatus = InterruptStatusSave & pExtension->IntStatusMask;

    if ( ! InterruptStatus )
        Rtn = FALSE;
    else
        {
        Rtn = TRUE;
        if ( (pExtension->nHeadCount%QUEUE_DEPTH) != ((pExtension->nTailCount-1)%QUEUE_DEPTH) )
            {
            pExtension->Queue[pExtension->nHeadCount] = InterruptStatusSave;
            pExtension->nHeadCount++;
            if ( pExtension->nHeadCount >= QUEUE_DEPTH )
                pExtension->nHeadCount = 0;
            }

        HwRequestDpcForIsr( pDevice );
        }

    return Rtn;
}

//------------------------------------------------------------------------------
//   OnInterruptWriteback()  --  Kernel ISR Handler
//------------------------------------------------------------------------------
//   This handler differs from the first in that the interrupt is cleared
//     by writing back the ISR with the saved value.

BOOL _stdcall OnInterruptWriteback(HWDEVICE* pDevice)
{
    DEVICE_EXTENSION* pExtension = (DEVICE_EXTENSION*)HwGetExtension(pDevice);
    DWORD   InterruptStatus;
    DWORD   InterruptStatusSave;
    BOOL    Rtn;

    InterruptStatusSave = HwReadLong( pDevice, pExtension->IntStatusRegionIdx,
                                               pExtension->IntStatusOffsetBytes );
    InterruptStatus = InterruptStatusSave & pExtension->IntStatusMask;

    if ( ! InterruptStatus )
        Rtn = FALSE;
    else
        {
        Rtn = TRUE;
        if ( (pExtension->nHeadCount%QUEUE_DEPTH) != ((pExtension->nTailCount-1)%QUEUE_DEPTH) )
            {
            pExtension->Queue[pExtension->nHeadCount] = InterruptStatusSave;
            pExtension->nHeadCount++;
            if ( pExtension->nHeadCount >= QUEUE_DEPTH )
                pExtension->nHeadCount = 0;
            }

        //  Write ISR back to Ack the interrupt...
        HwWriteLong( pDevice, pExtension->IntStatusRegionIdx,
                     pExtension->IntStatusOffsetBytes, InterruptStatusSave );

        HwRequestDpcForIsr( pDevice );
        }

    return Rtn;
}

//------------------------------------------------------------------------------
//   DeferredProcedureForIsr()  --  Kernel deferred procedure for the interrupt
//------------------------------------------------------------------------------

void _cdecl DeferredProcedureForIsr(HWDEVICE* pDevice)
{
    HwSetEvent(pDevice, 0);
}

//------------------------------------------------------------------------------
//   InstallKernelRoutines()  --
//------------------------------------------------------------------------------

bool II_DllExport InstallKernelRoutines( HWDEVICE *HwDevice, KernelIsrType type )
{
    HWLINK_OUTPUT linkOutput;

    memset(&linkOutput, 0, sizeof(linkOutput));

    //  Select which ISR handler to install, and install it
    switch (type)
        {
        case kIsrStandard:
        default:
            {
            if ( ! HwSetIsr( HwDevice, OnInterrupt))
                {
                HwGetLinkOutput( HwDevice, &linkOutput );
                HwDeleteDevice( HwDevice );
                return false;
                }
            }
            break;
        case kIsrWriteback:
            {
            if ( ! HwSetIsr( HwDevice, OnInterruptWriteback))
                {
                HwGetLinkOutput( HwDevice, &linkOutput );
                HwDeleteDevice( HwDevice );
                return false;
                }
            }
            break;

        }

    HwSetDpcForIsrC( HwDevice, DeferredProcedureForIsr );
    if (!HwLinkUserFunctions( HwDevice, sizeof(DEVICE_EXTENSION), 0))
        {
        HwGetLinkOutput( HwDevice, &linkOutput );
        HwDeleteDevice( HwDevice );
        return false;
        }
    return true;
}


} // end namespace InnovativeKernel
