//
// Lmk04800_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef LMK04800_MbH
#define LMK04800_MbH

#include <TimebaseDevice_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>
#include <VcxoControl_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS Lmk04800PllTuningEvent  --  Override PLL Tuning Parameters
//===========================================================================

class Lmk04800PllTuningEvent : public OpenWire::Event
{
public:
    unsigned int &CLKin0_PreR1;
    unsigned int &CLKin1_PreR1;
    unsigned int &R1;
    unsigned int &N1;
    unsigned int &R2;
    unsigned int &N2;
    unsigned int &N2_Pre;
    unsigned int &OD;

    Lmk04800PllTuningEvent(unsigned int & clk0_PreR1,  unsigned int & clk1_PreR1,
                            unsigned int & r1,    unsigned int & n1,
                            unsigned int & r2, unsigned int & n2,
                            unsigned int & n2_pre, unsigned int & od)
        : CLKin0_PreR1(clk0_PreR1), CLKin1_PreR1(clk1_PreR1), R1(r1), N1(n1),
          R2(r2), N2(n2), N2_Pre(n2_pre), OD(od)
        {
        }
};

//===========================================================================
//  CLASS Lmk04800Registers  -- Addresses for Device Registers
//===========================================================================

class Lmk04800Registers
{
public:
    unsigned int   Control_Addr;
    unsigned int   SpiIntf_Addr;
};

//=============================================================================
//  CLASS Lmk04800  --  PLL timebase control
//=============================================================================

class Lmk04800 : public PllTimebaseDevice
{
public:
    enum LmkType { lt4803, lt4805, lt4806, lt4808 };
    enum LmkReferenceSource { refPrimary=0x0, refCLKin0=0x0, refSecondary=0x1, refCLKin1=0x1,
                              refPinSelect=0x3, refAuto=0x4, refAutoNext=0x6 };
    enum LmkClockMode { clockPll, clockDistrib };  // run the PLL or Distribute Ext Clock

    enum OperatingModes { DualIVco=0, DualIVcoZD=2, DualEVco=3, DualEVcoZD=5,
                         SingleIVco=6, SingleIVcoZD=8, SingleEVco=0xb, ClkDistro=0x10, };
    enum LmkLockTypes { lltPll_1_Lock=0x1, lltPll_2_Lock=2, lltPll_1_2_Lock=3 };
    enum FbMux_Sources { fbm_Clkout0, fbm_Clkout2, fbm_Clkout4, fbm_Clkout6, fbm_Clkout8, fbm_Clkout10, fbm_FbClkIn };
    //
    //  Member Functions
    Lmk04800(IAddressingSpace &space, Lmk04800Registers & regs, OperatingModes pllMode=DualIVcoZD);
    virtual ~Lmk04800();

    void   Init();
    void   Reapply();

    void   Sync(bool state);
    void   Rewrite();

    virtual void    OutputFrequency(unsigned int output, double freq);
    virtual double  OutputFrequency(unsigned int output) const
        {  return PllTimebaseDevice::OutputFrequency(output);   }

    //  Power state
    void  Powered(bool value);
    bool  Powered()
             {  return FPowered;  }
    //  Select the Input reference
    void  ReferenceSource(LmkReferenceSource value);
    LmkReferenceSource  ReferenceSource()
             {  return FReferenceSource;  }

    //  Select Feedback Output
    void  FeedbackOutput(FbMux_Sources source)
        {  FFbMuxSource = source;  }
    FbMux_Sources  FeedbackOutput()
        {  return FFbMuxSource;  }

    //  Select Operating Mode
    void  OperatingMode(OperatingModes opmode)
        {  FDefaultOpMode = opmode;  }
    OperatingModes  OperatingMode()
        {  return FDefaultOpMode;  }

    //  Select Output Mode
    void  Pll_Mode()
             {  OutputSource(clockPll);  }
    void  ExtClock_Mode()
             {  OutputSource(clockDistrib);  }
    void  OutputSource(LmkClockMode value)
             {
             FClkMode = value;
             UpdateMode = (value==clockPll) ? updFrequency : updDivider;
             SetOutputSource(value);
             }
    LmkClockMode  OutputSource() const
             { return FClkMode; }
    bool  IsIn_Pll_Mode()
             {  return FClkMode == clockPll;  }
    bool  IsIn_ExtClock_Mode()
             {  return IsIn_Pll_Mode() == false;  }

    bool  Locked() const;
    void  LockBitType(LmkLockTypes setting)
             {  SetLockType(static_cast<unsigned int>(setting));  }

    int                 MaxOutputDivider() const;

    //  Raw configuration accesses
    virtual void            Port(unsigned char reg, unsigned int value);
    virtual unsigned int    Port(unsigned char reg);
    void            PortImage(unsigned char reg, unsigned int value);
    unsigned int    PortImage(unsigned char reg);

    OpenWire::EventHandler<Lmk04800PllTuningEvent> OnPllTuning;

    double  FrontEndPhaseDetectorFrequency() const   // Pll_1 PDF
        {  return FFrontEndPhaseDetectorFrequency;  }
    double  FrontEndVcxoFrequency() const            // Pll_1 Vcxo
        {  return FFrontEndVcxoFrequency;  }
    double  PhaseDetectorFrequency() const           // Pll_2 PDF
        {  return FPhaseDetectorFrequency;  }
    double  Modal_FrontEnd_N_InputFrequency();

    void  SetFrontEndPhaseDetectorFrequency(double freq)
        {  FFrontEndPhaseDetectorFrequency = freq;  }
    void  SetFrontEndVcxoFrequency(double freq)
        {  FFrontEndVcxoFrequency = freq;  }
    void  SetPhaseDetectorFrequency(double freq)
        {  FPhaseDetectorFrequency = freq;  }

protected:
    //
    //  Member Data
    LmkType                    FChipType;
    LmkReferenceSource         FReferenceSource;
    LmkClockMode               FClkMode;
    OperatingModes             FDefaultOpMode;
    FbMux_Sources              FFbMuxSource;
    std::vector<int>           RegisterImage;
    std::vector<float>         OutputFrequencyLimit;
    std::vector<unsigned int>  OutputConfigField;

    double  FFrontEndPhaseDetectorFrequency;
    double  FFrontEndVcxoFrequency;
    double  FPhaseDetectorFrequency;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    virtual void    OnSetPower(bool & new_state);

    virtual double  DefaultFrontEndPhaseDetectorFrequency() const = 0;   // Pll_1 PDF
    virtual double  DefaultFrontEndVcxoFrequency() const = 0;            // Pll_1 Vcxo
    virtual double  DefaultPhaseDetectorFrequency() const = 0;           // Pll_2 PDF


    //virtual class IVcxoControl &   Vcxo() {  return LocalVcxo;  }
    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
    enum OutputGroupSources { ogsVco=0, ogsOscIn=0x1, };
    enum OutputGroupDelayModes { dmNone=0x0, dmEven=0x1, dmOdd=0x2, dmBoth=0x3 };

    enum OutputCfgSettings { ocDisabled=0x0, ocLvds=0x1, ocLvPecl_0700=0x2,
                             ocLvPecl_1200=0x3, ocLvPecl_1600=0x4, ocLvPecl_2000=0x5,
                             ocLvCmos_P_Ninv=0x6, ocLvCmos_Pinv_N=0x7,
                             ocLvCmos_P_N=0x8, ocLvCmos_Pinv_Ninv=0x9,
                             ocLvCmos_Plow_N=0x0a, ocLvCmos_Plow_Ninv=0x0b,
                             ocLvCmos_P_Nlow=0x0c, ocLvCmos_Pinv_Nlow=0x0d,
                             ocLvCmos_Plow_Nlow=0x0e,
                           };
    enum OutputOsc1CfgSettings { oocLvPecl_0700=0x0, oocLvPecl_1200=0x1, oocLvPecl_1600=0x2, oocLvPecl_2000=0x3 };

    enum InputCfgSettings { icBipolar=0x0, icCmos=0x1 };
    enum InputPolaritySettings { ipActiveHigh=0x0, ipActiveLow=0x1  };
    enum HoldoverMuxSettings { hmxLow=0x0, hmxP1_dld1=0x1,  hmxP2_dld=0x2, hmxP1P2_dld=0x3, hmxHoldover=0x4,
                               hmxDacLock=0x5, hmxUWireRd=0x7,  hmxDacRail=0x8, hmxDacLow=0x9, hmxDacHigh=0xa,
                               hmxPll1N=0xb, hmxPll1N_2=0xc,  hmxPll2N=0xd, hmxPll2N_2=0xe, hmxPll1R=0xf,
                               hmxPll1R_2=0x10, hmxPll2R=0x11,  hmxPll2R_2=0x12,
                             };
    enum SyncMuxSettings { smxLogicLow, smxUWire=0x3 };

    enum SyncTypes { stpIn, stpInWpullup, stpIntWpulldown, stpOutWpush_pull,
                     stpOutInvWpush_pull, stpOutWopensrc, stpOutWopendrain   };

    void  SetOperatingMode(unsigned int om_setting);
    void  SetReset(bool state);

    void  PowerDownOutputGroup(unsigned int pair, bool state);
    void  SetOutputGroupSource(unsigned int pair, unsigned int ogrp_source);
    void  SetOutputGroupDelayMode(unsigned int pair, unsigned int ogrp_delaymode);
    void  SetOutputGroupDelay(unsigned int pair, unsigned int delay);
    void  SetOutputGroupDelay_HalfShift(unsigned int pair, bool hs);

    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting, bool should_cache_setting=true);
    void  SetOutputOsc0Config(unsigned int cfg_setting);
    void  SetOutputOsc1Config(unsigned int cfg_setting);
    void  EnableOutputOsc0(bool enabled);
    void  EnableOutputOsc1(bool enabled);

    void  SetInputConfig(unsigned int ref_idx, unsigned int cfg_setting);
    void  SetInputPolarity(unsigned int cfg_setting);
    void  SetReferenceSource(LmkReferenceSource source);

    void  SetLockType(unsigned int settings);
    void  SetHoldoverEnable(bool enabled);
    void  SetHoldoverMux(unsigned int setting);
    void  SetVcoMuxEnable(bool enabled);
    void  SetPll2RefDoublerEnable(bool enabled);

    void  SetFeedbackMuxEnable(bool enabled);
    void  SetFeedbackMux(unsigned int mux_state);

    void  SetSyncEnable(bool enabled);
    void  SetNoSyncCLKout(unsigned int pair, bool state);
    void  SetSyncMux(unsigned int cfg_setting);
    void  SetSyncQual(bool enabled);
    void  SetSyncPolarityInverted(bool inverted);
    void  SetSyncAuto(bool auto_sync);
    void  SetSyncType(unsigned int cfg_setting);

    void  SetPll2WindowSize();  // no option...

//----- unconverted
    // CLKoutX_Y_ADLY
    // OSCoutX_MUX, Clock Output Mux
    // PD_OSCin, OSCin Powerdown Control
    // OSCout_DIV, Oscillator Output Divide
    // EN_PLL2_XTAL
    // LD_TYPE // SYNC_PLLX_DLD // EN_TRACK
    // HOLDOVER_TYPE
    // Status_CLKin1_MUX
    // Status_CLKin0_TYPE
    // DISABLE_DLD1_DET  // Status_CLKin0_MUX
    // EN_CLKinX  // LOS_TIMEOUT // EN_LOS //Status_CLKin1_TYPE
    // DAC_HIGH_TRIP  // DAC_LOW_TRIP // EN_VTUNE_RAIL_DET
    // MAN_DAC // EN_MAN_DAC // HOLDOVER_DLD_CNT // FORCE_HOLDOVER
    // XTAL_LVL  // DAC_CNT // PLL2_C4_LF, PLL2 Integrated Loop Filter Component
    // PLL2_C3_LF, PLL2 Integrated Loop Filter Component
    // PLL2_R4_LF, PLL2 Integrated Loop Filter Component
    // PLL2_R3_LF, PLL2 Integrated Loop Filter Component
    // PLL1_N_DLY // PLL1_R_DLY // PLL1_WND_SIZE  // DAC_CLK_DIV
    // PLL1_DLD_CNT
    // PLL2_CP_POL, PLL2 Charge Pump Polarity // PLL2_CP_GAIN, PLL2 Charge Pump Current
    // PLL2_DLD_CNT // PLL2_CP_TRI, PLL2 Charge Pump TRI-STATE
    // PLL1_CP_POL, PLL1 Charge Pump Polarity
    // PLL1_CP_GAIN, PLL1 Charge Pump Current
    // PLL1_CP_TRI, PLL1 Charge Pump TRI-STATE
    // uWire_LOCK

    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);
    void  SetOutputSource(LmkClockMode source);
    //
    //  Internal "Getters"
    unsigned int  GetDivider(unsigned int output);

    void          SetVco2Range(LmkType chip_type);

    //
    //  Pll Programming Registers
    void          SetCLKin0_PreR(unsigned int PreR);
    unsigned int  GetCLKin0_PreR();
    void          SetCLKin1_PreR(unsigned int PreR);
    unsigned int  GetCLKin1_PreR();

    //  PLL1 Registers
    void          SetR1(unsigned int R1);
    unsigned int  GetR1();
    void          SetN1(unsigned int N1);
    unsigned int  GetN1();

    //  PLL2 Registers
    void          SetR2(unsigned int R2);
    unsigned int  GetR2();
    void          SetN2(unsigned int N2);
    unsigned int  GetN2();
    void          SetN2_Cal(unsigned int N2_Cal);
    unsigned int  GetN2_Cal();

    void          SetN2_Pre(unsigned int N2);
    unsigned int  GetN2_Pre();

    void          SetVcoDiv(unsigned int VcoDiv);
    unsigned int  GetVcoDiv();

    void          ConfigurePLL2(double OscInRateMHz, double Pll2_PdfMHz);

    void  ReadDataRegisters();
    void  WriteDataRegisters();

    void  Wait_UWireReady();
    void  Wait_UWireDataAvailable();

    void  CalculateFrontEndPLL(unsigned int & CLKin0_PreR, unsigned int & CLKin1_PreR,
                               unsigned int & R1, unsigned int & N1);
    void  CalculateBackEndPLL(double OscIn, double & Vco2Rate, unsigned int & OD, unsigned int & R2,
                              unsigned int & N2, unsigned int & N2_Pre,
                              double & PdfActual);
    void  AutoselectLoopbackOutput();

    bool    Vco_CanTuneTo(double freq);
    double  Vco_BestTunableFrequency(double freq);
    double  Vco_GetVcoFrequency();
    void    Vco_SetVcoFrequency(double freq);


    //-------------------------------------------------------------
    // CLASS InternalVcxo  --  In-chip VCXO interface
    //-------------------------------------------------------------

    class InternalVcxo : public IVcxoControl
    {
    public:
        virtual double  Min() {  return Pll.FVco2Range_Min;  }
        virtual double  Max() {  return Pll.FVco2Range_Max;  }
        virtual bool    CanTuneTo(double freq) {  return Pll.Vco_CanTuneTo(freq);  }
        virtual double  BestTunableFrequency(double freq)
                              {  return Pll.Vco_BestTunableFrequency(freq);  }

        virtual void    SetVcoFrequency(double freq)
                              {  Pll.Vco_SetVcoFrequency(freq);  }
        virtual void    SetRange(double /*min*/, double /*max*/)
                              {  /* not available  */ }

        InternalVcxo(Lmk04800 & pll)  :  Pll(pll)  {}
        virtual ~InternalVcxo()  {}

        void    Frequency(double freq)
                    {  SetVcoFrequency(freq);   }
        double  Frequency()
                    {  return Pll.Vco_GetVcoFrequency();   }
        double  FrequencyActual()
                    {  return Pll.Vco_GetVcoFrequency();   }

    private:
        Lmk04800 & Pll;
    };
    friend class Lmk04800::InternalVcxo;

    InternalVcxo  LocalVcxo;

    //-------------------------------------------------------------
    // CLASS PllControlRegister  --
    //-------------------------------------------------------------
    class PllControlRegister : public Register
    {
    public:
        typedef Register inherited;
        RegisterBit        Sync;
        RegisterBit        Status_Ho;
        RegisterBit        Status_Ld;
        RegisterBit        Status_ClkIn0;
        RegisterBit        Status_ClkIn1;
        RegisterBit        UWire_Ready;
        RegisterBit        UWire_DataValid;

        PllControlRegister(IAddressingSpace &space, int offset)
           :  inherited(space, offset),
              Sync(*this, 7),
              Status_Ho(*this, 8),
              Status_Ld(*this, 9),
              Status_ClkIn0(*this, 10),
              Status_ClkIn1(*this, 11),
              UWire_Ready(*this, 30),
              UWire_DataValid(*this, 31)
        {  }

    };

    PllControlRegister  ControlReg;
    Register            SpiReg;

    // Fields
    bool                FPowered;
    double              FVco2Range_Min;
    double              FVco2Range_Max;

    double              FVco2Frequency;

    //
    //  PLL Device Implementations
    void            SettingsInit();
    void            FrequencyModeUpdate();
    void            DividerModeUpdate();

    //
    void            SetOutputDividers(double VcoFreq, unsigned int OutDiv);

    virtual bool CheckDivider(unsigned int /*output*/, unsigned int divide_by)
        {  return IsValidDivider(divide_by); }
    virtual void  GetNearestDividers(unsigned int /*output*/, float candidate, int & below, int & above);

    void  InitDividerSupport();

protected:
    typedef std::vector<unsigned int> ValidDividerArray;
    typedef std::map<unsigned int, unsigned int> ValidDividerMap;
    typedef std::map<unsigned int, unsigned int> InverseDividerMap;

    // Mapping for OutputGroup Output Dividers
    bool          IsValidDivider(unsigned int divide_by);
    unsigned int  ToDividerInput(unsigned int divide_by);
    unsigned int  FromDividerInput(unsigned int divider_field);

    // Mapping for Feedback ClkinX_PreR dividers
    ValidDividerArray    ClkinX_PreR_DividerValues;
    ValidDividerMap      ClkinX_PreR_ValidDividers;
    InverseDividerMap    ClkinX_PreR_InverseDividers;

    unsigned int  To_CLKinX_PreR_DividerInput(unsigned int divide_by);
    unsigned int  From_CLKinX_PreR_DividerInput(unsigned int divider_bits);
    bool  IsValid_CLKinX_PreR_Divider(unsigned int divide_by);

    // Mapping for PLL2 Feedback N Prescaler divider
    ValidDividerArray    N2_Pre_DividerValues;
    ValidDividerMap      N2_Pre_ValidDividers;
    InverseDividerMap    N2_Pre_InverseDividers;

    unsigned int  To_N2_Pre_DividerInput(unsigned int divide_by);
    unsigned int  From_N2_Pre_DividerInput(unsigned int divider_bits);
    bool  IsValid_N2_Pre_Divider(unsigned int divide_by);

    // Mapping for PLL2 VcoDiv N Prescaler divider
    ValidDividerArray    VcoDiv_DividerValues;
    ValidDividerMap      VcoDiv_ValidDividers;
    InverseDividerMap    VcoDiv_InverseDividers;

    unsigned int  To_VcoDiv_DividerInput(unsigned int divide_by);
    unsigned int  From_VcoDiv_DividerInput(unsigned int divider_bits);
    bool  IsValid_VcoDiv_Divider(unsigned int divide_by);

    void          SplitN2Factors(unsigned int TotalN2, unsigned int &N2_Pre, unsigned int &N2);
    unsigned int  DivideToBestInt(double number, double divide_by);

public:
    Trace         Debug;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

