//
// Lmk04820_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <LMK04820_Mb.h>
#include <FmcSupport_Mb.h>
#include <SystemSupport_Mb.h>
#include <VcxoControl_Mb.h>
#include <algorithm>
#include <limits>
#include <cmath>
#include <Magic_Mb.h>
#include <Trace_Mb.h>
#include <sstream>

using namespace std;

namespace Innovative
{

//=============================================================================
//  CLASS Lmk04820  --  Lmk04820 Pll Implementation
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class Lmk04820
//-----------------------------------------------------------------------------

const unsigned int  TotalReferences  = 3;    
const unsigned int  TotalOutputs     = 14;   // even = DCLK, odd - SDCLK
const unsigned int  TotalOutPairs    = TotalOutputs>>1;

const int           SpiRegisterCount = 0x200;   //
const int           SpiLockRegisterCount = 3;   //

const double        MaxOutputFrequency = 1536.0e6;   // pg 55, analog delay must be 0 above this...

//  External Vco Parameters

//  Internal Vco Parameters

struct VcoRangesType {
    double Low;
    double High;
};

namespace {

VcoRangesType Lmk04820_Vco0Ranges[] = {
    {1840.0e6, 1970.0e6}, // LMK04826 default values
    {2370.0e6, 2630.0e6}, // LMK04828 default values
    {1930.0e6, 2075.0e6}, // LMK04821 default values

};

VcoRangesType Lmk04820_Vco1Ranges[] = {
    {2440.0e6, 2505.0e6}, // LMK04826 default values
    {2920.0e6, 3080.0e6}, // LMK04828 default values
    {2920.0e6, 3080.0e6}, // LMK04821 default values

};

const unsigned int  TotalVcoTypes     = sizeof(Lmk04820_Vco0Ranges)/sizeof(VcoRangesType) ;


}  // nameless namespace

const double        kPDF1_lower_limit = 1.0e6;    // might go as low as 500.0e3
const double        kPDF1_upper_limit = 40.0e6;   // max PFD

const unsigned int  kR1_Min   =  1;
const unsigned int  kR1_Max   =  16383;
const unsigned int  kN1_Min   =  1;
const unsigned int  kN1_Max   =  16383;

const unsigned int  kR2_Min   =  1;
const unsigned int  kR2_Max   =  4095;
const unsigned int  kN2_Min   =  1;
const unsigned int  kN2_Max   =  262143;

//  Output Dividers
const unsigned int  kOD_Min   =  1;
const unsigned int  kOD_Max   =  32;
//  SYSREF Dividers
const unsigned int  kSYSREF_Min   =  8;
const unsigned int  kSYSREF_Max   =  8191;


const float         DefaultMaxOutFreq        = 3200.0e6;

const double        kPll1_N1_Frequency_Limit = 750.0e6;   // Limit on N1 frequency
const double        kMaxFrontEnd_VcxoRate    = 500.0e6;   // Limit on OscIn frequency

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
//-----------------------------------------------------------------------------
//  constructor for class Lmk04820
//-----------------------------------------------------------------------------

Lmk04820::Lmk04820(IAddressingSpace & space, Lmk04820Registers & regs, OperatingModes pllMode, TriggerSyncModes tsMode)
    : PllTimebaseDevice(TotalReferences, TotalOutputs),
      FChipType(lt4828), FReferenceSource(refPrimary),
      FClkMode(clockPll),
      FDefaultOpMode(pllMode),
      FDefaultSyncMode(osmPulsorSync),
      FDefaultTriggerSyncMode(tsMode),
      FAllow_Pll1_FeedbackAdjustment(false),
      LastOutputSyncStatus(false),
      FFbMuxSource(fbm_Dclk8),
      RegisterImage(SpiRegisterCount),
      LocalVcxo_Standard(*this),
      LocalVcxo_04821(*this),
      WbVco(0),
      UseWidebandVco(false),
      ControlReg(space, regs.Control_Addr),
      SpiReg(space, regs.SpiIntf_Addr),
      SpiStatusReg(space, regs.SpiIntf_Addr),
      FPowered(false),
      Debug(true)
{
    const unsigned int  DefaultOutputCfgBits = ocDisabled;
    //  Fill max frequency checker array
    for (unsigned int i=0; i<TotalOutputs; i++)
        {
        OutputFrequencyLimit.push_back(DefaultMaxOutFreq);
        OutputConfigField.push_back(DefaultOutputCfgBits);
        OutputIsCopy.push_back(0);
        OutputSyncDisable.push_back(0);
        }
    //
    //  Set Default Input Configuration
    SourceInUse(0, true);    SetSourceMapping(0, IFmcClockIo::rsInternal);   // primary = internal
    SourceInUse(1, true);    SetSourceMapping(1, IFmcClockIo::rsExternal);   // secondary = external
    SourceInUse(2, true);    SetSourceMapping(2, IFmcClockIo::rsCarrier);    // clkin2 = carrier

    SetVco2Range(FChipType);

    InitDividerSupport();
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//-----------------------------------------------------------------------------
//  destructor for class Lmk04820
//-----------------------------------------------------------------------------

Lmk04820::~Lmk04820()
{
    // we don't own the VCO passed in, if any
}

//-----------------------------------------------------------------------------
//  Lmk04820::Init()  --  Initialize Chip
//-----------------------------------------------------------------------------

void Lmk04820::Init()
{
    FFrontEndPhaseDetectorFrequency = DefaultFrontEndPhaseDetectorFrequency();
    FFrontEndVcxoFrequency = DefaultFrontEndVcxoFrequency();
    FPhaseDetectorFrequency = DefaultPhaseDetectorFrequency();

    SetReset(true);
//    SetReset(false);  -- auto cleared

    ReadDataRegisters();   // read initial state from chip
    SettingsInit();        // common INIT settings
    OnInit();
    Update();    // Update state of the device based on sources
    WriteDataRegisters();
    OutputSync();
}

//-----------------------------------------------------------------------------
//  Lmk04820::SettingsInit()  --  Initial Settings Loading
//-----------------------------------------------------------------------------
//   Note board specific implementations can override these in OnInit

void Lmk04820::SettingsInit()
{
    SetOscOutputFormat(ocDisabled);
    SetPll1CpGain(1550);   // current level in uA

    // Power on all output chains
    for (unsigned int i=0; i<TotalOutPairs; i++)
        {
        PowerDownOutputGroup(i, false);
        PowerDownOutputDigitalDelay(2*i, true);
//        SetOutputGroupDelay(i, 5);
        }

    SetSysrefEnable(1,  true);  PowerDownSysref(1,  false);
    SetSysrefEnable(3,  true);  PowerDownSysref(3,  false);
    SetSysrefEnable(5,  true);  PowerDownSysref(5,  false);
    SetSysrefEnable(7,  true);  PowerDownSysref(7,  false);
    SetSysrefEnable(9,  true);  PowerDownSysref(9,  false);
    SetSysrefEnable(11, true);  PowerDownSysref(11, false);
    SetSysrefEnable(13, true);  PowerDownSysref(13, false);

    SetPll2WindowSize();
    OptimizeVco1Noise();
    SetHoldoverEnable(false);
    SetHoldoverHitlessSwitch(false);
    SetPll2ExternalCrystalEnable(false);
}

//-----------------------------------------------------------------------------
//  Lmk04820::Reapply()  --  Reapply settings to hardware from saved state
//-----------------------------------------------------------------------------

void Lmk04820::Reapply()
{
    ScopedTraceRef ccc(Debug, "Lmk04820::Reapply()");
    SetReset(true);
    OnReapply();

    Update();    // Update state of the device based on sources

    //Sync(true);
    //SetReset(true);
    WriteDataRegisters();  // first write sets reset false...
    OutputSync();
}

//-----------------------------------------------------------------------------
//  Lmk04820::Rewrite()  --  Rewrite settings without calculations
//-----------------------------------------------------------------------------
//  For use with PortImage() especially...

void Lmk04820::Rewrite()
{
    WriteDataRegisters();
    OutputSync();
}

//-----------------------------------------------------------------------------
//  Lmk04820::OutputSync()  --  Select the Sync mode
//-----------------------------------------------------------------------------

void Lmk04820::OutputSync()
{
    LastOutputSyncStatus = true;

    if (OutputSyncMode()==osmPulsorSync)
        PulsorSync();
    else
        SystemRefSync();
    
    OnAfterOutputSync();

    //...Trigger mode settings
    TriggerSyncSetup();

    // Dummy read to wait for last SPI to finish
    unsigned int data = Port(0x147);
    Debug.Log("Final Dummy Read", data, 16);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PulsorSync()  --  Sync all output dividers
//-----------------------------------------------------------------------------

void Lmk04820::PulsorSync()
{
    ScopedTraceRef ccc(Debug, "Lmk04820::PulsorSync()");

    Sleep(100);  // wait for lock

    const bool ApplyNow = true;
    //
    //  Turn off all Outputs that need it...avoid pulse on trigger
    Debug.Log("...disabling selected outputs");
    for (unsigned int output=0; output<TotalOutputs; output++)
        {
        if (OutputSyncDisable[output])
            SetOutEnabledHdw_autoapply(output, false, ApplyNow);
        }

    Debug.Log("...configure pulsor sync");
    //  ( set SYSREF_MUX)
    SetSysrefGroupSource(srefs_Pulser, ApplyNow);

    // ( clear SYSREF_GBL_PD, SYSREF_PD, SYSREF_DDLY_PD, SYSREF_PLSR_PD)
    PowerDownSysref_GlobalEnable(false);
    PowerDownAllSysref(false);
    PowerDownSysrefDigitalDelay(false);
    PowerDownSysrefPulseGen(false, ApplyNow);  // update register

    //  ( toggle SYSREF_CLR and set SYNC_EN, and SYNC_MODE)
    SetSysrefDelayClear(true);
    SetSyncEnable(true);
    SetSyncMode(smPulsorSpiWrite, ApplyNow);

    SetSysrefDelayClear(false, ApplyNow);

    //  ( clear SYNC_DIS*)
    for (unsigned int pair=0; pair<7; pair++)
        SetNoSyncCLKout(pair, false);
    SetNoSyncSysref(true, ApplyNow);

    //  ( set SYSREF_PULSE_CNT to perform SYNC)
    SetSysrefPulseCount(spc_2, ApplyNow);

    //  ( set SYNC_DIS*)
    for (unsigned int pair=0; pair<7; pair++)
      SetNoSyncCLKout(pair, true);
    SetNoSyncSysref(true, ApplyNow);

    //  ( set SYSREF_MUX)
    SetSysrefGroupSource(srefs_NormalSync, ApplyNow);

    // ( clear SYSREF_CLR, and set pin SYNC_MODE)
//    SetSysrefDelayClear(false);
    SetSyncMode(sm_SyncPin, ApplyNow);

    // ( set SYSREF_PD, SYSREF_DDLY_PD, SYSREF_PLSR_PD)  // go away
    PowerDownAllSysref(true);
    PowerDownSysrefDigitalDelay(true);
    PowerDownSysrefPulseGen(true, ApplyNow);  // update register
    //
    //  Turn on all Outputs that need it...those turned off before
    Debug.Log("...re-enabling selected outputs");
    for (unsigned int output=0; output<TotalOutputs; output++)
        {
        if (OutputSyncDisable[output])
            SetOutEnabledHdw_autoapply(output, true, ApplyNow);
        }

}

//-----------------------------------------------------------------------------
//  Lmk04820::SystemRefSync()  --  Sync all output dividers (SysRefMode)
//-----------------------------------------------------------------------------

void Lmk04820::SystemRefSync()
{
    ScopedTraceRef ccc(Debug, "Lmk04820::SystemRefSync()");
    Sleep(100);  // wait for lock

    const bool ApplyNow = true;

    //  ( set SYSREF_MUX)
    SetSysrefGroupSource(srefs_NormalSync, ApplyNow);

    // ( clear SYSREF_GBL_PD, SYSREF_PD, SYSREF_DDLY_PD, SYSREF_PLSR_PD)
    PowerDownSysref_GlobalEnable(false);
    PowerDownAllSysref(false);
    PowerDownSysrefDigitalDelay(false);
    PowerDownSysrefPulseGen(true, ApplyNow);  // update register

    //  ( toggle SYSREF_CLR and set SYNC_EN, and SYNC_MODE)
    SetSysrefDelayClear(true);
    SetSyncEnable(true);
    SetSyncMode(sm_SyncPin, ApplyNow);

    SetSysrefDelayClear(false, ApplyNow);

    //  ( clear SYNC_DIS*)
    for (unsigned int pair=0; pair<TotalOutPairs; pair++)
        SetNoSyncCLKout(pair, false);
    SetNoSyncSysref(false, ApplyNow);

    //  ( set SYNC_POL to perform SYNC)
    SetSyncPolarityInverted(true, ApplyNow);
    SetSyncPolarityInverted(false, ApplyNow);

    //  ( set SYNC_DIS*)
    for (unsigned int pair=0; pair<TotalOutPairs; pair++)
      SetNoSyncCLKout(pair, true);
    SetNoSyncSysref(true, ApplyNow);

    //  ( set SYSREF_MUX)
    SetSysrefGroupSource(srefsContinuous, ApplyNow);
}

//-----------------------------------------------------------------------------
//  Lmk04820::TriggerSyncSetup()  --  Trigger Sync Setup Modes
//-----------------------------------------------------------------------------

void Lmk04820::TriggerSyncSetup()
{
    ScopedTraceRef ccc(Debug, "Lmk04820::TriggerSyncSetup()");
    const bool ApplyNow = true;

    switch (TriggerSyncMode())
        {
        case tsmLowLatency:
            Debug.Log("...Low Latency Sync");
            SetSysrefClkin0Mux(srck0mux_Clkin0Direct, ApplyNow);
            SetSyncMode(sm_Disabled, ApplyNow);
            SetClkin0_Output(c0_Sysref, ApplyNow);
            SetClkin0_Output(c0_Pll1);  // make sure next cycle starts out in PLL mode
            SetSysrefClkin0Mux(srck0mux_SysrefMux);
            return;
        case tsmLogic:
            Debug.Log("...Sync routed to Logic");
            SetSyncMode(sm_SyncPin, ApplyNow);
            SetClkin0_Output(c0_Pll1, ApplyNow);
            return;
        default:
        case tsmUnused:
            //  Trigger does not use LMK sync, so do nothing
            Debug.Log("...Sync unused, so skip");
            return;
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::OnAfterOutputSync()
//-----------------------------------------------------------------------------
//  Override in derived class

void  Lmk04820::OnAfterOutputSync()
{
    //  Override in derived class
}

//-----------------------------------------------------------------------------
//  Lmk04820::PulsorModeEnter()  --  Enable the pulsor mode
//-----------------------------------------------------------------------------

void  Lmk04820::PulsorModeEnter()
{
    Debug.Log("Enable the Pulsor mode");

    // Save the values of registers that will be modified
    SavedRegImagex139 = RegisterImage[0x139];
    SavedRegImagex140 = RegisterImage[0x140];
    SavedRegImagex143 = RegisterImage[0x143];

    const bool ApplyNow = true;

    // ( clear SYSREF_GBL_PD, SYSREF_PD, SYSREF_DDLY_PD, SYSREF_PLSR_PD)
    PowerDownSysref_GlobalEnable(false);
    PowerDownAllSysref(false);
    PowerDownSysrefDigitalDelay(false);
    PowerDownSysrefPulseGen(false, ApplyNow);  // update register

    //  ( set SYNC_EN, and SYNC_MODE)
    SetSyncEnable(true);
    SetSyncMode(smPulsorSpiWrite, ApplyNow);

    //  ( set SYSREF_MUX)
    SetSysrefGroupSource(srefs_Pulser, ApplyNow);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PulsorModeFire()  --  Fire a pulse
//-----------------------------------------------------------------------------

void  Lmk04820::PulsorModeFire(unsigned int Log2PulseCnt)
{
    //  ( set SYSREF_PULSE_CNT to perform SYNC)
    SetSysrefPulseCount(Log2PulseCnt, true);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PulsorModeExit()  --  Exit the pulsor mode and return to the previous mode
//-----------------------------------------------------------------------------

void  Lmk04820::PulsorModeExit()
{
    Debug.Log("Exit the Pulsor mode");

    // Restore these register values from the saved values
    RegisterImage[0x139] = SavedRegImagex139;
    RegisterImage[0x140] = SavedRegImagex140;
    RegisterImage[0x143] = SavedRegImagex143;

    ApplyImage(0x139);
    ApplyImage(0x140);
    ApplyImage(0x143);
}

//-----------------------------------------------------------------------------
//  Lmk04820::LoadWidebandVco()  --  Load a vco with range for wideband PLL2
//-----------------------------------------------------------------------------

void Lmk04820::LoadWidebandVco(IVcxoControl * aVco)
{
    WbVco = aVco;
    UseWidebandVco = (aVco!=0) ? true : false;
}

//-----------------------------------------------------------------------------
//  Lmk04820::Powered()  --  Change Powered state of chip
//-----------------------------------------------------------------------------

void Lmk04820::Powered(bool value, bool At_Open)
{
    if (value==FPowered)
        return;

    FPowered = value;
    OnSetPower(FPowered);

    if (At_Open)
        return;

    if (FPowered)
        {
        Init();       // Restore complete chip state
        Reapply();
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::ReferenceSource()  --  Change Clock Reference
//-----------------------------------------------------------------------------

void Lmk04820::ReferenceSource(LmkReferenceSource value)
{
    FReferenceSource = value;
    SetReferenceSource(value);
}

//-----------------------------------------------------------------------------
//  Lmk04820::MaxOutputDivider()  --  Return max output post-divider
//-----------------------------------------------------------------------------

int Lmk04820::MaxOutputDivider() const
{
    return kOD_Max;
}

//-----------------------------------------------------------------------------
//  Lmk04820::EffectiveFeedbackFrequency() --
//-----------------------------------------------------------------------------

double  Lmk04820::EffectiveFeedbackFrequency()
{
    if (IsFeedbackOutputAdc())
        return Adc().EffectiveFrequency();
    else
        return Dac().EffectiveFrequency();
}

//-----------------------------------------------------------------------------
//  Lmk04820::EffectiveFeedbackFrequencyActual() --
//-----------------------------------------------------------------------------

double  Lmk04820::EffectiveFeedbackFrequencyActual()
{
    if (IsFeedbackOutputAdc())
        return Adc().EffectiveFrequencyActual();
    else
        return Dac().EffectiveFrequencyActual();
}


//-----------------------------------------------------------------------------
//  Lmk04820::FrequencyModeUpdate()  --  Update using Frequency as baseline
//-----------------------------------------------------------------------------

void Lmk04820::FrequencyModeUpdate()
{
    ScopedTraceRef ccc(Debug, "Lmk04820::FrequencyModeUpdate()");

    if (Reference() == 0 || EffectiveFrequency() == 0)
        return;

    unsigned int R2, N2, N2_Cal, N2_Pre;

    unsigned int CLKin0_R, CLKin1_R, CLKin2_R, N1;
    unsigned int R1;

    //  0)  Calculate OscIn / Front End Vcxo Frequency
    CalculateFrontEndVcxoFrequency();
    double OscIn = FrontEndVcxoFrequency();
    Debug.Log("PLL2 Reference = ", OscIn);
    //
    //  1)  Calculate PLL2 parameters
    unsigned int OD;
    double Vco2Freq;
    double PDF_Actual = PhaseDetectorFrequency();
    CalculateBackEndPLL(OscIn, Vco2Freq, OD, R2, N2, N2_Cal, N2_Pre, PDF_Actual);
    //
    //   Phase Detector Freq = Reference/R1 = Vco/N1
    double PDF = PhaseDetectorFrequency();
    Debug.Log("Target PDF2 = ", PDF);
    Debug.Log("Actual PDF2 = ", PDF_Actual);
    //
    //   Recalculate everything (in case changes were made)
    Vco2Freq = N2_Pre * N2 * OscIn / R2;
    Debug.Log("Mid Vco2Freq = ", Vco2Freq);
    //
    //  Set up the output dividers properly
    SetOutputDividers(Vco2Freq, OD);    
    //
    //  2) Calculate Parameters for Front End PLL
    //       We do this last since N frequency can depend on PLL2
    Debug.Log("N Input Freq = ", Modal_FrontEnd_N_InputFrequency());
    CalculateFrontEndPLL(CLKin0_R, CLKin1_R, CLKin2_R, N1);
    //
    //      ...It seems that there is a limit to the frequency input to N1. Adjust matters
    //          to avoid high rates here
    AdjustPLL1_FeedbackRate(CLKin0_R, CLKin1_R, CLKin2_R, N1, OD);

    switch (GetSourceSelection())
        {
        default:
        case 0:   R1=CLKin0_R;  break;
        case 1:   R1=CLKin1_R;  break;
        case 2:   R1=CLKin2_R;  break;
        }

    double Pfd1_Actual_R = Reference()/R1;
    double Vco1Freq = Pfd1_Actual_R * N1;
    double Pfd1_Actual_N = Modal_FrontEnd_N_InputFrequency()/N1;

    // Get FB index divider
    unsigned int Feedback_OD;
    if (FeedbackOutput()==fbm_Dclk6)
        Feedback_OD = DivideToBestInt(EffectiveFeedbackFrequencyActual(), OutputFrequencyActual(6));
    else if (FeedbackOutput()==fbm_Dclk8)
        Feedback_OD = DivideToBestInt(EffectiveFeedbackFrequencyActual(), OutputFrequencyActual(8));
    else
        Feedback_OD = 1;  // can not adjust

    Debug.Log("Mid Vco1Freq = ", Vco1Freq);
    Debug.Log("N Input Freq = ", Modal_FrontEnd_N_InputFrequency());
    Debug.Log("Vco1 = ", FrontEndVcxoFrequency());
    Debug.Log("Pfd1 (R Leg) = ", Pfd1_Actual_R);
    Debug.Log("Pfd1 (N Leg) = ", Pfd1_Actual_N);

    //  Allow User to Override Settings
    Debug.Log("Lmk04820::CLKin0_R = ", CLKin0_R);
    Debug.Log("Lmk04820::CLKin1_R = ", CLKin1_R);
    Debug.Log("Lmk04820::CLKin2_R = ", CLKin2_R);
    Debug.Log("Lmk04820::N1 = ", N1);
    Debug.Log("Lmk04820::R2 = ", R2);
    Debug.Log("Lmk04820::N2 = ", N2);
    Debug.Log("Lmk04820::N2_Cal = ", N2_Cal);
    Debug.Log("Lmk04820::N2_Pre = ", N2_Pre);
    Debug.Log("Lmk04820::Output Divider = ", OD);
    Debug.Log("Lmk04820::Feedback Output Divider = ", Feedback_OD);
    Debug.Log("Vco2Freq (medium) = ", Vco2Freq);

    Debug.Log("Pll Tuning Event");
    Lmk04820PllTuningEvent e(CLKin0_R, CLKin1_R, CLKin2_R, N1, R2, N2, N2_Cal, N2_Pre, OD, Feedback_OD);
    OnPllTuning.Execute(e);

    Debug.Log("Final Values");
    Debug.Log("Lmk04820::CLKin0_R = ", CLKin0_R);
    Debug.Log("Lmk04820::CLKin1_R = ", CLKin1_R);
    Debug.Log("Lmk04820::CLKin2_R = ", CLKin2_R);
    Debug.Log("Lmk04820::N1 = ", N1);
    Debug.Log("Lmk04820::R2 = ", R2);
    Debug.Log("Lmk04820::N2 = ", N2);
    Debug.Log("Lmk04820::N2_Cal = ", N2_Cal);
    Debug.Log("Lmk04820::N2_Pre = ", N2_Pre);
    Debug.Log("Lmk04820::Output Divider = ", OD);
    Debug.Log("Lmk04820::Feedback Output Divider = ", Feedback_OD);
    //
    //   Recalculate everything (in case changes were made)
    switch (GetSourceSelection())
        {
        default:
        case 0:   R1=CLKin0_R;  break;
        case 1:   R1=CLKin1_R;  break;
        case 2:   R1=CLKin2_R;  break;
        }
    Pfd1_Actual_R = Reference()/R1;
    Vco1Freq = Pfd1_Actual_R * N1;

    Vco2Freq = N2_Pre * N2 * OscIn / R2;

    Debug.Log("Final Vco1Freq = ", Vco1Freq);
    Debug.Log("Final Vco2Freq = ", Vco2Freq);
    //
    //  Update registers & allied hardware
    LocalVcxo().SetVcoFrequency(Vco2Freq);    // reprogram Vcxo

    //  PLL2 Registers
    SetCLKin0_R(CLKin0_R);
    SetCLKin1_R(CLKin1_R);
    SetCLKin2_R(CLKin2_R);
    SetN1(N1);
    SetR2(R2);
    SetN2(N2);
    SetN2_Cal(N2_Cal);
    SetN2_Pre(N2_Pre);

    // Set FB based on overrided value
    if (FeedbackOutput()==fbm_Dclk6)
        OutputFrequency(6, EffectiveFeedbackFrequency()/Feedback_OD);
    else if (FeedbackOutput()==fbm_Dclk8)
        OutputFrequency(8, EffectiveFeedbackFrequency()/Feedback_OD);

/*
    SetVcoMuxEnable(false);           // never use these
*/
    SetPll2RefDoublerEnable(false);
    ConfigurePLL2(OscIn/1.0e6);
    SetPLL1_WindowSize(Pfd1_Actual_R);
    //
    //  Set up the output dividers properly
    SetOutputDividers(Vco2Freq, OD);

}

//-----------------------------------------------------------------------------
//  Lmk04820::CalculateFrontEndVcxoFrequency() --
//-----------------------------------------------------------------------------

void  Lmk04820::CalculateFrontEndVcxoFrequency()
{
    if (UseWidebandVco==false)
        return;
    if (!WbVco)
        return;

    //  if we get here, we have a wide range external Vcxo plugged in
    //    that we can set to provide a varying value of OscIn
    double MinRate = WbVco->Min();
    double MaxRate = std::min(kMaxFrontEnd_VcxoRate, WbVco->Max());

    //  largest divider we need to check is ratio of max to min rates
    unsigned int DivLimit = static_cast<unsigned int>(Frequency()/MinRate);

    //  Set osc in to be the Master Frequency, or a divider of same
    double OscIn(FrontEndVcxoFrequency());
    for (unsigned int divider=1;  divider<DivLimit; divider++)
        {
        double CandidateOscIn = Frequency()/divider;
        if (CandidateOscIn>MaxRate)
            continue;   // still too large
        // accept new value
        OscIn = CandidateOscIn;
        break;
        }

    // Let everyone else use this frequency
    SetFrontEndVcxoFrequency(OscIn);
    // program Vcxo to use this frequency
    WbVco->SetVcoFrequency(OscIn);
}

//-----------------------------------------------------------------------------
//  Lmk04820::CalculateFrontEndPLL() -- Calculate Parameters for FrontEnd PLL
//-----------------------------------------------------------------------------

void  Lmk04820::CalculateFrontEndPLL(unsigned int & CLKin0_PreR,
                                     unsigned int & CLKin1_PreR,
                                     unsigned int & CLKin2_PreR,
                                     unsigned int & N1)
{
    ScopedTraceRef ccc(Debug, "Lmk04820::CalculateFrontEndPLL()");
    //
    //   Phase Detector Freq = Reference/R1 = Vco/N1
    double RefRate = Reference();
    double N_Input_Rate = Modal_FrontEnd_N_InputFrequency();
    //double PDF = FrontEndPhaseDetectorFrequency();
    //Debug.Log("Start PDF1 = ", PDF);
    double PDF_high_limit = kPDF1_upper_limit;
    double PDF_low_limit = kPDF1_lower_limit;

    //
    //  We want to adjust the PFD to match the N1 Input
    //    But we can't go much below 1 MHz (~500K is ok) before we fail to lock
    unsigned int R1 = 0;
    for (unsigned int candidateR1 = 1; candidateR1<static_cast<unsigned int>(RefRate);  candidateR1++)
        {
        double PDF = RefRate/candidateR1;
        if (PDF >= PDF_high_limit)
            continue;
        if (PDF < PDF_low_limit)   // quit if we go too low
            {
            R1 = 1;   // failed
            break;
            }
        //
        //  Check if this PFD evenly divides the N input rate
        double Div = DivideToBestInt(N_Input_Rate, PDF);

        if (Div*PDF == N_Input_Rate)
            {
            R1 = candidateR1;
            Debug.Log("Exact PDF1 Actual = ", PDF);
            Debug.Log("Exact Div*PDF1 = ", Div*PDF);
            break;
            }
        }

    // check R1
    if (R1==0)  R1=1;

    // Find effective PFD
    double Pfd1_Actual_R = Reference()/R1;

    N1 = static_cast<unsigned int>( N_Input_Rate/Pfd1_Actual_R );
    if (N1==0)  N1=1;

    CLKin0_PreR = R1;
    CLKin1_PreR = R1;
    CLKin2_PreR = R1;
}

//-----------------------------------------------------------------------------
//  Lmk04820::AdjustPLL1_FeedbackRate() -- "Fix" PLL1 Feedback
//-----------------------------------------------------------------------------

void  Lmk04820::AdjustPLL1_FeedbackRate(unsigned int & CLKin0_R, unsigned int & CLKin1_R, unsigned int & CLKin2_R,
                                        unsigned int & N1, unsigned int OD)
{
    // Make sure we ony do this when allowed, or need to....
    if (FAllow_Pll1_FeedbackAdjustment==false)
        return;   // not allowed to adjust

    if (FDefaultOpMode != omNestedZeroDelay)
        return;   // can not adjust

    double N_Input_Rate = Modal_FrontEnd_N_InputFrequency();
    if (N_Input_Rate < kPll1_N1_Frequency_Limit)
        return;    // no need to adjust

    // Find the index of the loopback output
    unsigned int LB_OutputIdx;
    if (FeedbackOutput()==fbm_Dclk6)
        LB_OutputIdx = 6;
    else if (FeedbackOutput()==fbm_Dclk8)
        LB_OutputIdx = 8;
    else
        return;  // can not adjust

    //
    //  Find a prime factor of N1, F sucn that
    //    Feedback Rate/F < limit
    //    OD * F < kOD_Max
    unsigned int F(1);
    unsigned int nominalF(1);
    for (unsigned int candidateF = 2; candidateF*OD<kOD_Max;  candidateF++)
        {
        unsigned int remainder = N1%candidateF;
        if (remainder!=0)
            continue;

        if (N_Input_Rate/candidateF < kPll1_N1_Frequency_Limit)
            {
            F=candidateF;
            break;
            }
        else
            nominalF = candidateF;
        }

    //
    //  If we could not factor N1 to get below threshold, we can reduce the PDF and
    //     increase both N1 and R1 by the same factor.  This gives more factors to reduce
    //     the feedback frequency
    if (F==1)
        {
        Debug.Log("Could not totally factor N1. nominalF = ", nominalF);
        unsigned int R1 = CLKin0_R;
        for (unsigned int F2=2; Reference()/(F2*R1)>=kPDF1_lower_limit;  F2++)
            {
            if (N_Input_Rate/(nominalF*F2) < kPll1_N1_Frequency_Limit)
                {
                // we found an extra factor we could apply...
                R1 = R1 * F2;
                N1 = N1 * F2;
                F  = nominalF * F2;
                Debug.Log("PDF reduction F2 = ", F2);
                Debug.Log("New R1 = ", R1);
                Debug.Log("New N1 = ", N1);
                Debug.Log("New  F = ", F);
                break;
                }
            }
        CLKin0_R = CLKin1_R = CLKin2_R = R1;
        }

    N1 /= F;
    OutputFrequency(LB_OutputIdx, OutputFrequency(LB_OutputIdx)/F);
    OutputFreqActualArray[LB_OutputIdx] = OutputFrequencyActual(LB_OutputIdx)/F;

    Debug.Log("Adjust PLL1 Feedback Factor ", F);
    Debug.Log("New N1 = ", N1);
    Debug.Log("New Feedback Frequency = ", OutputFrequency(LB_OutputIdx));
    Debug.Log("New Actual Feedback Frequency = ", OutputFreqActualArray[LB_OutputIdx]);

}

//-----------------------------------------------------------------------------
//  Lmk04820::TuneMatch() -- Calculate Parameters for PLL2
//-----------------------------------------------------------------------------

bool  Lmk04820::TuneMatch(std::vector<TuneMatchResults>& TM_results, unsigned int OD_Max, unsigned int OD_Min)
{
    //ScopedTraceRef ccc(Debug, "Lmk04820::TuneMatch()");
    TM_results.clear();

    bool  TuneMatchFound(false);
    for (unsigned int rep = 0; rep<OD_Max; rep++)
        {
        unsigned int CandidateOD  = OD_Max-rep;   // try ODs in reverse order...
        double       CandidateVco = EffectiveFrequency()*CandidateOD;

        if (CandidateOD < OD_Min)  // skip any out of range values
            continue;

        if (CandidateVco<=LocalVcxo().Max() && CandidateVco>=LocalVcxo().Min())
            {
            //  We have a candidate Divider. Check if we can reset to this freq.
            //     (some Vco's have gaps in the range).
            if (LocalVcxo().CanTuneTo(CandidateOD*EffectiveFrequency()))
                {
                TuneMatchFound = true;
                TuneMatchResults TMR(CandidateVco, CandidateOD);
                TM_results.push_back(TMR);
                // keep looking
                }
            }
        }
    //Debug.Log("OD_Max ", OD_Max);
    Debug.Log("TuneMatchFound ", TuneMatchFound);
    return TuneMatchFound;
}

//bool  Lmk04820::TuneMatch(double & Vco2Rate, unsigned int & OD, unsigned int OD_Max)
//{
//    //ScopedTraceRef ccc(Debug, "Lmk04820::TuneMatch()");

//    bool  TuneMatchFound(false);
//    for (unsigned int rep = 0; rep<OD_Max; rep++)
//        {
//        unsigned int CandidateOD  = OD_Max-rep;   // try ODs in reverse order...
//        double       CandidateVco = EffectiveFrequency()*CandidateOD;

//        if (CandidateVco<=LocalVcxo().Max() && CandidateVco>=LocalVcxo().Min())
//            {
//            //  We have a candidate Divider. Check if we can reset to this freq.
//            //     (some Vco's have gaps in the range).
//            if (LocalVcxo().CanTuneTo(CandidateOD*EffectiveFrequency()))
//                {
//                TuneMatchFound = true;
//                Vco2Rate = CandidateVco;
//                OD = CandidateOD;
//                break;
//                }
//            }
//        }
//    //Debug.Log("OD_Max ", OD_Max);
//    //Debug.Log("TuneMatchFound ", TuneMatchFound);
//    return TuneMatchFound;
//}

//
//  Aux Tune Match - enable three stage more restrictive tune match to
//    perfectly match VCO at lower dividers if possible. Default is off, and returns 1.
//-----------------------------------------------------------------------------
//  Lmk04820::AuxTuneMatchEnabled() -- use three stage tune match?
//-----------------------------------------------------------------------------

bool Lmk04820::AuxTuneMatchEnabled()
{
    return false;  // don't do it!
}

//-----------------------------------------------------------------------------
//  Lmk04820::AuxTuneMatchFactor() -- return factor to limit Tune Match
//-----------------------------------------------------------------------------
//  The new three stage tune match limits the OD range for passes 0 and 1 by dividing
//    by a factor, so lower VCOs are encouraged, thus giving more scope for divided down
//    outputs in the last stage.  Idx 0 should be the largest, Idx 1 the next.

unsigned int Lmk04820::AuxTuneMatchFactor(unsigned int /*idx*/)
{
    return 1;
}


//-----------------------------------------------------------------------------
//  Lmk04820::CalculateBackEndPLL() -- Calculate Parameters for PLL2
//-----------------------------------------------------------------------------

void  Lmk04820::CalculateBackEndPLL(double OscIn, double & Vco2Rate, unsigned int & OD, unsigned int & R2,
                                    unsigned int & N2, unsigned int & N2_Cal, unsigned int & N2_Pre,
                                    double & PdfActual)
{
    Debug.Log("Lmk04820::CalculateBackEndPLL()");

    std::vector<TuneMatchResults>  TM_results;
    bool  TuneMatchFound(false);

    //  If a board has a master rate and some related divided rates that all need to be fit, we
    //     can first try and fit a more restricted Max Divider (and lower VCO).  Need a wide range for this
    //     to work out.
    if (AuxTuneMatchEnabled())
        {
        //  Set up three pass Tune Match process..first pass
        TuneMatchFound = TuneMatch(TM_results, MasterOD_Max()/AuxTuneMatchFactor(0), MasterOD_Min());
        if (!TuneMatchFound)
            {
            //  ..second pass
            TuneMatchFound = TuneMatch(TM_results, MasterOD_Max()/AuxTuneMatchFactor(1), MasterOD_Min());
            if (!TuneMatchFound)
                {
                //  ...third, least restrictive
                TuneMatchFound = TuneMatch(TM_results, MasterOD_Max(), MasterOD_Min());
                }
            }
        }
    else
        {
        //  Standard One Pass Tune Match
        TuneMatchFound = TuneMatch(TM_results, MasterOD_Max(), MasterOD_Min());
        }
    //
    //  Determine Vco Frequency If the scan failed
    if (!TuneMatchFound)
        {
        //  We missed finding a VCO Match. Pick a VCO freq we can use
        Vco2Rate = LocalVcxo().BestTunableFrequency(EffectiveFrequency());  // assume OD=1 ?
        //  have to figure a reasonable FB value too.
        OD =  static_cast<int>(Vco2Rate / EffectiveFrequency());
        if (OD==0)
            OD=1;
        }

    bool ExactMatch(false);
    double OrigPdfActual = PdfActual;
    for (size_t tm_idx=0; tm_idx<TM_results.size(); tm_idx++)
        {
        //  Take Tune Match Results and put them in variables for use
        Vco2Rate = TM_results[tm_idx].CandidateVco;
        OD = TM_results[tm_idx].OD;

        Debug.Log("TM Result Loop Idx=", tm_idx);
        Debug.Log("Vco2Rate=", Vco2Rate);
        Debug.Log("OD=", OD);
        //
        //  We want to adjust the PFD to match the VCO more exactly
        PdfActual = OrigPdfActual;
        unsigned int TestR = static_cast<unsigned int>(OscIn/PdfActual);
        while (TestR < kR2_Max)
            {
            Debug.Log("TestR = ", TestR);

            unsigned int Div;
            Div = DivideToBestInt(Vco2Rate, PdfActual);
    //        Debug.Log("Div = ", Div);
    //        Debug.Log("PDF Actual = ", PdfActual);
    //        Debug.Log("Div*PdfActual = ", Div*PdfActual);

            if (Div*PdfActual == Vco2Rate)
                {
                Debug.Log("Exact PDF Actual = ", PdfActual);
                Debug.Log("Exact Div*PdfActual = ", Div*PdfActual);
                ExactMatch = true;
                break;
                }
            else
                {
                //  Not an exact fit, so reduce PDF
                TestR = static_cast<unsigned int>(OscIn/(PdfActual/10.0));
                if (TestR < kR2_Max)
                    PdfActual /= 10.0;
                }
            }
        //
        //  If we have an exact match, Quit Now
        if (ExactMatch)
            break;
        }

    //
    //  Calculate Candidate N2, N2_Pre Value
    unsigned int TotalN2;
    TotalN2 = DivideToBestInt(Vco2Rate, PdfActual);

    SplitN2Factors(TotalN2, N2_Pre, N2);

    N2_Cal = N2;
    if (FDefaultOpMode==omCascadedZeroDelay)
        {
        // in this mode, the actual N2 avoids PRE, is smaller by OD
        N2 = N2*N2_Pre/OD;
        }

    R2 = DivideToBestInt(OscIn, PdfActual);

    // check R2, N2?
}

//-----------------------------------------------------------------------------
//  Lmk04820::DivideToBestInt() -- Divide number, rounding fraction to nearest
//-----------------------------------------------------------------------------

unsigned int  Lmk04820::DivideToBestInt(double number, double divide_by)
{
    double div = 1.0;
    double fraction = modf(number/divide_by, &div);  // round to best int
    div += (fraction > 0.5) ? 1.0 : 0.0;
    return static_cast<unsigned int>( div );
}

//-----------------------------------------------------------------------------
//  Lmk04820::SplitN2Factors() -- Split Total N2 into N2, N2_Pre
//-----------------------------------------------------------------------------

void  Lmk04820::SplitN2Factors(unsigned int TotalN2, unsigned int &N2_Pre, unsigned int &N2)
{
    unsigned int CandidatePre = 2;
    for (ValidDividerArray::iterator div = N2_Pre_DividerValues.begin();
             div !=  N2_Pre_DividerValues.end();  ++div)
        {
        CandidatePre = *div;
        if (TotalN2%CandidatePre == 0)
            {
            N2_Pre = CandidatePre;
            N2 = TotalN2/N2_Pre;
            return;
            }
        }

    N2_Pre = 2;
    N2 = TotalN2/N2_Pre;
    return;
}

//-----------------------------------------------------------------------------
//  Lmk04820::DividerModeUpdate() -- Update state of dividers
//-----------------------------------------------------------------------------
//  In this mode, the reference is sent to the outputs, with possible division

void  Lmk04820::DividerModeUpdate()
{
    Debug.Log("Lmk04820::DividerModeUpdate()");
    if (Reference() == 0 || EffectiveFrequency() == 0)
        return;

    double div = 1.0;
    double fraction = modf(Reference()/EffectiveFrequency(), &div);
    div += (fraction > 0.5) ? 1.0 : 0.0;
    unsigned int OutDiv = static_cast<unsigned int>(div);
    OutDiv = std::max<unsigned int>(OutDiv, 1);

    SetOutputDividers(Reference(), OutDiv);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputDividers()  --
//-----------------------------------------------------------------------------

void  Lmk04820::SetOutputDividers(double VcoFreq, unsigned int OutDiv)
{
    Debug.Log("Lmk04820::SetOutputDividers()");
    Debug.Log("VcoFreq = ", VcoFreq);
    Debug.Log("OutDiv = ", OutDiv);

    unsigned int BestDivider = OutDiv;
    double       OutputFreq  = VcoFreq/BestDivider;


    for (unsigned int output=0; output<TotalOutputs; output++)
        {
        if (OutputInUse(output)==true)
            {
            float   CandidateDivider = static_cast<float>(BestDivider);
            double  CandidateFreq    = OutputFreq;
            //
            //  If the Output is a copy of its pair, apply the same values directly.
            //  Or if is the Master, use the best values.
            //     if not the master, and Frequency is unspecified, then
            //     the make the output a copy of Master.
            //     Otherwise, figure the best divider possible for each output
            if (OutputIsCopy[output])
                {
                Debug.Log("Output ", output);
                Debug.Log("Copying from Output ", output-1);
                // apply settings from other side, ignore hardware (it is bypassed)
                OutputFreqArray[output]       = OutputFreqArray[output-1];
                OutputDividerArray[output]    = OutputDividerArray[output-1];
                OutputFreqActualArray[output] = OutputFreqActualArray[output-1];
                continue;
                }
            else if ((output!=FMasterOutput) && (OutputFrequency(output)!=0.0))
                {
                Debug.Log("Calculating Optimal Frequency for output ", output);
                CandidateFreq = OutputFrequency(output);
                CandidateDivider = static_cast<float>(VcoFreq / CandidateFreq);
                Debug.Log("CandidateDivider = ", CandidateDivider);
                Debug.Log("CandidateFreq = ", CandidateFreq);

                int above, below, divider;
                GetNearestDividers(output, CandidateDivider, below, above);

                // Find which divider is closest to desired frequency
                double centerFreq = VcoFreq/CandidateDivider;
                double hiFreq     = VcoFreq/below;
                double lowFreq    = VcoFreq/above;
                if (std::abs(centerFreq-lowFreq) > std::abs(hiFreq-centerFreq))
                    divider = below;
                else
                    divider = above;

                CandidateDivider = static_cast<float>(divider);
                }

            //
            //  Check for frequency exceeding applied Limits
            Debug.Log("FC CandidateDivider = ", CandidateDivider);
            Debug.Log("FC CandidateFreq = ", CandidateFreq);
            if (OutputFrequencyLimit[output]>=CandidateFreq)
                OutputDivider(output, static_cast<unsigned int>(CandidateDivider));
            else
                {
                Debug.Log("Out of Frequency Limit Range");
                double MinDivideRatio = VcoFreq/OutputFrequencyLimit[output];

                double div = 1.0;
                double fraction = modf(MinDivideRatio, &div);
                div += (fraction > 0.0) ? 1.0 : 0.0;
                unsigned int OutDiv = static_cast<unsigned int>(div);
                OutDiv = std::max<unsigned int>(OutDiv, 1);

                OutputDivider(output, OutDiv);
                }
            //
            //  Calculate Actual Frequency
            if (OutputDividerArray[output]!=0)  // if illegal divider, skip it
                {
                OutputFreqActualArray[output] = VcoFreq / OutputDividerArray[output];
                }
            //
            //  If Divider 'needed' is not allowed because of rate limitations,
            //     disable the output, otherwise enable it
//            if (VcoFreq > MaxDividerAbove1Freq && OutputDivider(output)!=1)
//                {
//                SetDividerEnable(output, false);
//                OutputFreqActualArray[output] = 0.0;  // no output allowed
//                }
//            else
//                SetDividerEnable(output, true);
            }
        else
            {
            OutputFreqActualArray[output] = 0.0;
            }
        }

    //  Calculate Real Sysref Rate
    int SysRefDiv = GetSysrefDiv();
    FSysRefFrequencyActual = (SysRefDiv) ? VcoFreq/SysRefDiv : 0.0 ;

    Debug.Log("SysRef Freq Acutal = ", FSysRefFrequencyActual);
    Debug.Log("SysRef Div = ", SysRefDiv);

}

//-----------------------------------------------------------------------------
//  Lmk04820::Modal_FrontEnd_N_InputFrequency()  --  N1 input by "operating mode"
//-----------------------------------------------------------------------------

double Lmk04820::Modal_FrontEnd_N_InputFrequency()
{
    switch (FDefaultOpMode)
        {
        default:
            Debug.Log("...unsupported operating mode");
        case omDualLoopMode:
        case omCascadedZeroDelay:
            return FrontEndVcxoFrequency();

        case omNestedZeroDelay:
            if (FeedbackOutput()==fbm_Dclk6)
                return OutputFrequencyActual(6);
            else if (FeedbackOutput()==fbm_Dclk8)
                return OutputFrequencyActual(8);
            else if (FeedbackOutput()==fbm_Sysref)
                return SysRefFrequencyActual();
            else
                return FrontEndVcxoFrequency();

        }
//    return FrontEndVcxoFrequency();   // should never get here...
}

//-----------------------------------------------------------------------------
//  Lmk04820::MasterOD_Max()  --  Return largest OD for active Master
//-----------------------------------------------------------------------------

unsigned int  Lmk04820::MasterOD_Max()
{
    if (FMasterOutput & 1)
        return kSYSREF_Max;  // odd outputs
    else
        return kOD_Max;      // even outputs
}

//-----------------------------------------------------------------------------
//  Lmk04820::MasterOD_Min()  --  Return smallest OD for active Master
//-----------------------------------------------------------------------------

unsigned int  Lmk04820::MasterOD_Min()
{
    if (FMasterOutput & 1)
        return kSYSREF_Min;  // odd outputs
    else
        return kOD_Min;      // even outputs
}


//-----------------------------------------------------------------------------
//  Lmk04820::Locked()  --  Return PLL lock status
//-----------------------------------------------------------------------------

bool  Lmk04820::Locked() const
{
    return (ControlReg.Status_Ld1.Value() && ControlReg.Status_Ld2.Value())? true : false;
}

//-----------------------------------------------------------------------------
//  Lmk04820::Port()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void Lmk04820::Port(unsigned int reg, unsigned int value)
{
    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !ControlReg.Spi_Ready.Value(); ++i)
        Sleep(1);
    if (!ControlReg.Spi_Ready.Value())
        {
        Debug.Log("Lmk04820 PLL SPI ready WR timeout error!");
        return;
        }

    SpiReg.Addr.Value(reg);
    SpiReg.WrData.Value(value);
    SpiReg.Rw.Value(0);
    SpiReg.Apply();

    std::stringstream msg;
    msg << "Lmk04820::Port[0x" << std::hex << static_cast<int>(reg)
        << "]=0x" << static_cast<int>(value);
    Debug.Log(msg.str());
}

//-----------------------------------------------------------------------------
//  Lmk04820::Port()  --  Read from PLL data port
//-----------------------------------------------------------------------------

unsigned int Lmk04820::Port(unsigned int reg)
{
    unsigned char result(0);

    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !ControlReg.Spi_Ready.Value(); ++i)
        Sleep(1);
    if (!ControlReg.Spi_Ready.Value())
        {
        Debug.Log("Lmk04820 PLL SPI ready RD timeout error!");
        return result;
        }

    SpiReg.Addr.Value(reg);
    SpiReg.Rw.Value(1);
    SpiReg.Apply();

    // Wait for previous operation to complete
    for (unsigned int i = 0; (i < 10) && !ControlReg.Spi_DataValid.Value(); ++i)
        Sleep(1);
    if (!ControlReg.Spi_DataValid.Value())
        {
        Debug.Log("Lmk04820 PLL SPI Data Valid RD timeout error!");
        return result;
        }

    result = SpiStatusReg.Data.Value();

/*
    std::stringstream msg;
    msg << "Lmk04820::Read 0x" << std::hex << SpiValue
        << " from reg " << static_cast<int>(reg);
    Debug.Log(msg.str());
*/

    return result;

}

//-----------------------------------------------------------------------------
//  Lmk04820::ApplyImage()  --  Write register image data to physical register
//-----------------------------------------------------------------------------

void Lmk04820::ApplyImage(unsigned int reg)
{
    Port(reg, RegisterImage[reg]);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PortImage()  --  Write to Object Image Data Cache
//-----------------------------------------------------------------------------

void  Lmk04820::PortImage(unsigned int reg, unsigned int value)
{
    if (reg < static_cast<unsigned int>(RegisterImage.size()))
        RegisterImage[reg] = value;
}

//-----------------------------------------------------------------------------
//  Lmk04820::PortImage()  --  Read from Object Image Data Cache
//-----------------------------------------------------------------------------

unsigned int  Lmk04820::PortImage(unsigned int reg)
{
    if (reg < static_cast<unsigned int>(RegisterImage.size()))
        return RegisterImage[reg];
    else
        return 0;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutEnabledHdw()  --  Update the Output Enabled hardware
//-----------------------------------------------------------------------------

void Lmk04820::SetOutEnabledHdw(unsigned int output, bool enabled)
{
    bool auto_apply = false;
    SetOutEnabledHdw_autoapply(output, enabled, auto_apply);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutEnabledHdw_autoapply()  --  Update the Output Enabled
//-----------------------------------------------------------------------------

void Lmk04820::SetOutEnabledHdw_autoapply(unsigned int output, bool enabled, bool auto_apply)
{
    if (output < TotalOutputs)
        {
        if (enabled)
            {
            // restore from cached setting...
            bool should_cache_setting = true;
            SetOutputConfig(output, OutputConfigField[output], should_cache_setting, auto_apply );
            //  the above also disables divider if off
            }
        else
            {
            bool should_cache_setting = false;
            SetOutputConfig(output, ocDisabled, should_cache_setting, auto_apply );
            //  the above also disables divider if off
            }
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOperatingMode()  --  set "operating mode" controls
//-----------------------------------------------------------------------------

void Lmk04820::SetOperatingMode(unsigned int om_setting)
{
    Debug.Log("Lmk04820::SetOperatingMode()");

    switch (om_setting)
        {
        default:
            Debug.Log("...unsupported operating mode");
            break;

        case omDualLoopMode:
            SetPll1_NclkSource(p1i_OscIn);         // PLL1_NCLK_MUX
            SetPll2_NclkSource(p2i_PllPrescaler);  // PLL2_NCLK_MUX
            SetFbMuxEnable(false);                 // FB_MUX_EN
            //SetFbMuxSource(FFbMuxSource);        // FB_MUX  - don't care
            PowerDownOscIn(false);                 // OSCin_PD
            //SetClkin0_Output(c0_Pll1);             // CLKin0_OUT_MUX - done when setting clock mode
            //SetClkin1_Output(c1_Pll1);             // CLKin1_OUT_MUX
            // VCO_MUX   // do this later when configuring clock
            break;

        case omCascadedZeroDelay:
            SetPll1_NclkSource(p1i_OscIn);         // PLL1_NCLK_MUX
            SetPll2_NclkSource(p2i_FbMux);         // PLL2_NCLK_MUX
            SetFbMuxEnable(true);                  // FB_MUX_EN
            SetFbMuxSource(FFbMuxSource);          // FB_MUX
            PowerDownOscIn(false);                 // OSCin_PD
            //SetClkin0_Output(c0_Pll1);             // CLKin0_OUT_MUX - done when setting clock mode
            //SetClkin1_Output(c1_Pll1);             // CLKin1_OUT_MUX
            // VCO_MUX   // do this later when configuring clock
            break;

        case omNestedZeroDelay:
            SetPll1_NclkSource(p1i_FbMux);         // PLL1_NCLK_MUX
            SetPll2_NclkSource(p2i_PllPrescaler);  // PLL2_NCLK_MUX
            SetFbMuxEnable(true);                  // FB_MUX_EN
            SetFbMuxSource(FFbMuxSource);          // FB_MUX
            PowerDownOscIn(false);                 // OSCin_PD
            //SetClkin0_Output(c0_Pll1);             // CLKin0_OUT_MUX - done when setting clock mode
            //SetClkin1_Output(c1_Pll1);             // CLKin1_OUT_MUX
            // VCO_MUX   // do this later when configuring clock
            break;

        }

}

//-----------------------------------------------------------------------------
//  Lmk04820::SetReset()  --  reset the PLL - don't change the image!
//-----------------------------------------------------------------------------

void Lmk04820::SetReset(bool state)
{
    //  RESET  SPI_3WIRE_DIS
    unsigned int R0_Data = Port(0x0);
    PtrBitManipulator Reg(&R0_Data);
    Reg.Bit(7, state);
    Reg.Bit(4, false);  // always use 3 Wire mode

    Port(0x0, R0_Data);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOutputGroup()  --  Power down/up output pair
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOutputGroup(unsigned int pair, bool state)
{
    // CLKoutX_Y_PD
    if (pair < TotalOutPairs)
        {
        // Figure out location of field
        unsigned int reg;
        unsigned int ofs;
        switch (pair)
            {
            default:
            case  0:  reg=0x106;  ofs=3;  break;
            case  1:  reg=0x10e;  ofs=3;  break;
            case  2:  reg=0x116;  ofs=3;  break;
            case  3:  reg=0x11e;  ofs=3;  break;
            case  4:  reg=0x126;  ofs=3;  break;
            case  5:  reg=0x12e;  ofs=3;  break;
            case  6:  reg=0x136;  ofs=3;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Bit(ofs, state);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetInputDriveLevel()  --  Update the Input DL for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetInputDriveLevel(unsigned int pair, bool drive_level)
{
    // CLKout0_1_IDL
    if (pair < TotalOutPairs)
        {
        // Figure out location of field
        unsigned int reg;
        unsigned int ofs;
        switch (pair)
            {
            default:
            case  0:  reg=0x100;  ofs=5;  break;
            case  1:  reg=0x108;  ofs=5;  break;
            case  2:  reg=0x110;  ofs=5;  break;
            case  3:  reg=0x118;  ofs=5;  break;
            case  4:  reg=0x120;  ofs=5;  break;
            case  5:  reg=0x128;  ofs=5;  break;
            case  6:  reg=0x130;  ofs=5;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Bit(ofs, drive_level);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputDriveLevel()  --  Update the Output DL for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputDriveLevel(unsigned int pair, bool drive_level)
{
    // CLKoutX_Y_ODL
    if (pair < TotalOutPairs)
        {
        // Figure out location of field
        unsigned int reg;
        unsigned int ofs;
        switch (pair)
            {
            default:
            case  0:  reg=0x100;  ofs=6;  break;
            case  1:  reg=0x108;  ofs=6;  break;
            case  2:  reg=0x110;  ofs=6;  break;
            case  3:  reg=0x118;  ofs=6;  break;
            case  4:  reg=0x120;  ofs=6;  break;
            case  5:  reg=0x128;  ofs=6;  break;
            case  6:  reg=0x130;  ofs=6;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Bit(ofs, drive_level);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetDigitalDelayCounts()  --  Update the Input DL for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetDigitalDelayCounts(unsigned int pair, int hi_count, int lo_count)
{
    // DCLKoutX_DDLY_CNTH DCLKoutX_DDLY_CNTL

    if (pair < TotalOutPairs)
        {
        // Figure out location of field
        unsigned int reg;
        unsigned int ofsH = 4;
        unsigned int ofsL = 0;
        switch (pair)
            {
            default:
            case  0:  reg=0x101;  break;
            case  1:  reg=0x109;  break;
            case  2:  reg=0x111;  break;
            case  3:  reg=0x119;  break;
            case  4:  reg=0x121;  break;
            case  5:  reg=0x129;  break;
            case  6:  reg=0x131;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Field(ofsH, 4, ToDigitalDelayCount(hi_count));
        Reg.Field(ofsL, 4, ToDigitalDelayCount(lo_count));
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetDividerHdw()  --  Update the Divider setting hardware
//-----------------------------------------------------------------------------

void Lmk04820::SetDividerHdw(unsigned int output, unsigned int divide_by)
{
    ScopedTraceRef ccc(Debug, "Lmk04820::SetDividerHdw()");

    Debug.Log("output = ", output);
    Debug.Log("divide_by = ", divide_by);

    if ((output&1) == 0)  // even X outputs only
        {
        //  Configure normal out divider
        SetDivider(output, divide_by);
        }
    else
        {
        // set other common divider for odd channels
        SetSysrefDiv(divide_by);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetDivider()  --  set divider field
//-----------------------------------------------------------------------------

void  Lmk04820::SetDivider(unsigned int output, unsigned int divide_by)
{
    // DCLKoutX_DIV
    if (!IsValidDivider(divide_by))
        return;

    // Figure out location of field
    unsigned int reg;
    switch (output)
        {
        default:
        case  0:  reg=0x100;  break;
        case  2:  reg=0x108;  break;
        case  4:  reg=0x110;  break;
        case  6:  reg=0x118;  break;
        case  8:  reg=0x120;  break;
        case 10:  reg=0x128;  break;
        case 12:  reg=0x130;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    unsigned int div_field = ToDividerInput(divide_by);
    Reg.Field(0, 5, div_field);

    //  Disable divider if dividing by 1
    if (divide_by==1)
        SetDividerMode(output, odmDividerBypass);
    else
        {
        if (IsOutputAnalogDelay_PoweredDown(output))
            SetDividerMode(output, odmDivider);
        else
            SetDividerMode(output, odmDivider_Delay);  // if delay is in use, use divider with delay
        }

}

//-----------------------------------------------------------------------------
//  Lmk04820::GetDivider()  --  read back the Divider setting
//-----------------------------------------------------------------------------

unsigned int  Lmk04820::GetDivider(unsigned int output)
{
    // DCLKoutX_DIV
    unsigned int divider(1);
    if ((output&1) == 0)  // even X outputs only
        {
        // Figure out location of field
        unsigned int reg;
        switch (output)
            {
            default:
            case  0:  reg=0x100;  break;
            case  2:  reg=0x108;  break;
            case  4:  reg=0x110;  break;
            case  6:  reg=0x118;  break;
            case  8:  reg=0x120;  break;
            case 10:  reg=0x128;  break;
            case 12:  reg=0x130;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        unsigned int div_field = Reg.Field(0, 5);
        divider = FromDividerInput(div_field);
        }
    return divider;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetDividerMode()  --  Update the Output Divider Mode for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetDividerMode(unsigned int output, unsigned int cfg_setting)
{
    // DCLKoutX_MUX
    if ((output&1) == 0)  // even X outputs only
        {
        // Figure out location of field
        unsigned int reg;
        switch (output)
            {
            default:
            case  0:  reg=0x103;  break;
            case  2:  reg=0x10B;  break;
            case  4:  reg=0x113;  break;
            case  6:  reg=0x11B;  break;
            case  8:  reg=0x123;  break;
            case 10:  reg=0x12B;  break;
            case 12:  reg=0x133;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Field(0, 2, cfg_setting);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetAnalogDelayMode()  --  Update the Output Divider Mode for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetAnalogDelayMode(unsigned int output, bool has_duty_cycle_correction)
{
    // DCLKoutX_ADLY_MUX
    if ((output&1) == 0)  // even X outputs only
        {
        // Figure out location of field
        unsigned int reg;
        switch (output)
            {
            default:
            case  0:  reg=0x103;  break;
            case  2:  reg=0x10B;  break;
            case  4:  reg=0x113;  break;
            case  6:  reg=0x11B;  break;
            case  8:  reg=0x123;  break;
            case 10:  reg=0x12B;  break;
            case 12:  reg=0x133;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Bit(2, has_duty_cycle_correction);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetAnalogDelay()  --  Update the Output Divider delay for a channel
//-----------------------------------------------------------------------------
//  Delay 500 ps plus setting value

void Lmk04820::SetAnalogDelay(unsigned int output, unsigned int delay_setting, bool auto_apply)
{
    // DCLKoutX_ADLY
    if ((output&1) == 0)  // even X outputs only
        {
        // Figure out location of field
        unsigned int reg;
        switch (output)
            {
            default:
            case  0:  reg=0x103;  break;
            case  2:  reg=0x10B;  break;
            case  4:  reg=0x113;  break;
            case  6:  reg=0x11B;  break;
            case  8:  reg=0x123;  break;
            case 10:  reg=0x12B;  break;
            case 12:  reg=0x133;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Field(3, 5, delay_setting);

        if (auto_apply)
            ApplyImage(reg);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputClockHalfStep()  --  channel clock halfstep
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputClockHalfStep(unsigned int output, bool has_halfstep)
{
    // DCLKoutX_HS or SDCLKoutY_HS

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x104;  break;
        case  1:  reg=0x10C;  break;
        case  2:  reg=0x114;  break;
        case  3:  reg=0x11C;  break;
        case  4:  reg=0x124;  break;
        case  5:  reg=0x12C;  break;
        case  6:  reg=0x134;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 0)  // even X outputs only
        {
        Reg.Bit(6, has_halfstep);
        }
    else                  // odd Y outputs only
        {
        Reg.Bit(0, has_halfstep);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutput_AsCopy()  --  Make Sysref a copy of CLK output
//-----------------------------------------------------------------------------

void   Lmk04820::SetOutput_AsCopy(unsigned int output)
{
    SetSysrefEnable(output, false);
    PowerDownSysref(output, true);
    OutputIsCopy[output] = 1;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutput_AsNonCopy()  --  Make Sysref its own output
//-----------------------------------------------------------------------------

void   Lmk04820::SetOutput_AsNonCopy(unsigned int output)
{
    SetSysrefEnable(output, true);
    PowerDownSysref(output, false);
    OutputIsCopy[output] = 0;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefClkin0Mux()  --  select sysref output source
//-----------------------------------------------------------------------------

void  Lmk04820::SetSysrefClkin0Mux(unsigned int select_setting, bool auto_apply)
{
    // SYSREF_CLKin0_MUX
    PtrBitManipulator Reg(&RegisterImage[0x139]);
	bool setting = false;
	if (select_setting >= 1)
		setting = true;
    Reg.Bit(2, setting);

    if (auto_apply)
        ApplyImage(0x139);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefGroupSource()  --  enable use of sysref as input
//-----------------------------------------------------------------------------

void  Lmk04820::SetSysrefGroupSource(unsigned int source_setting, bool auto_apply)
{
    // SYSREF_MUX
    PtrBitManipulator Reg(&RegisterImage[0x139]);
    Reg.Field(0, 2, source_setting);

    if (auto_apply)
        ApplyImage(0x139);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefEnable()  --  enable use if sysref as input
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefEnable(unsigned int output, bool source_is_sysref)
{
    // SDCLKoutY_MUX

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x104;  break;
        case  1:  reg=0x10C;  break;
        case  2:  reg=0x114;  break;
        case  3:  reg=0x11C;  break;
        case  4:  reg=0x124;  break;
        case  5:  reg=0x12C;  break;
        case  6:  reg=0x134;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 1)  // odd Y outputs only
        {
        Reg.Bit(5, source_is_sysref);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefDigitalDelay()  --  sysref source mux
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefDigitalDelay(unsigned int output, unsigned int delay_in_vco_clks)
{
    // SDCLKoutY_DDLY

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x104;  break;
        case  1:  reg=0x10C;  break;
        case  2:  reg=0x114;  break;
        case  3:  reg=0x11C;  break;
        case  4:  reg=0x124;  break;
        case  5:  reg=0x12C;  break;
        case  6:  reg=0x134;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 1)  // odd Y outputs only
        {
        Reg.Field(1, 4, ToSysrefDigitalDelayCount(delay_in_vco_clks));
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefAnalogDelayEnable()  --  Enable Sysref Analog Delay
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefAnalogDelayEnable(unsigned int output, bool enabled)
{
    // SDCLKoutY_ADLY_EN
    if ((output&1) == 1)  // odd Y outputs only
        {
        // Figure out location of field
        unsigned int reg;
        switch (output)
            {
            default:
            case  1:  reg=0x105;  break;
            case  3:  reg=0x10D;  break;
            case  5:  reg=0x115;  break;
            case  7:  reg=0x11D;  break;
            case  9:  reg=0x125;  break;
            case 11:  reg=0x12D;  break;
            case 13:  reg=0x135;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Bit(4, enabled);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefAnalogDelay()  --  sysref analog delay
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefAnalogDelay(unsigned int output, unsigned int delay_setting)
{
    // SDCLKoutY_DDLY

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x105;  break;
        case  1:  reg=0x10D;  break;
        case  2:  reg=0x115;  break;
        case  3:  reg=0x11D;  break;
        case  4:  reg=0x125;  break;
        case  5:  reg=0x12D;  break;
        case  6:  reg=0x135;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 1)  // odd Y outputs only
        {
        Reg.Field(0, 4, delay_setting);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefOutputPowerdownMode()  --  sysref output mode
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefOutputPowerdownMode(unsigned int output, unsigned int mode)
{
    // SDCLKoutY_DIS_MODE

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 1)  // odd Y outputs only
        {
        Reg.Field(1, 2, mode);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownSysref()  --  sysref power down
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownSysref(unsigned int output, bool power_down, bool auto_apply)
{
    // SDCLKoutY_PD

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 1)  // odd Y outputs only
        {
        Reg.Bit(0, power_down);
        }

    if (auto_apply)
        ApplyImage(reg);

}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefCommonDigitalDelay()  --  sysref source mux
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefCommonDigitalDelay(unsigned int delay)
{
    // SYSREF_DDLY
    PtrBitManipulator Reg_lsb(&RegisterImage[0x13D]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x13C]);

    unsigned int field = ToSysrefCommonDigitalDelayCount(delay);
    Reg_lsb.Value(field&0xFF);
    Reg_msb.Field(0, 5, field>>8);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefPulseCount()  --  sysref source mux
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefPulseCount(unsigned int setting, bool auto_apply)
{
    // SYSREF_PULSE_CNT
    PtrBitManipulator Reg(&RegisterImage[0x13E]);
    Reg.Field(0, 2, setting);

    if (auto_apply)
        ApplyImage(0x13E);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOutputDigitalDelay()  --  output power down
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOutputDigitalDelay(unsigned int output, bool power_down)
{
    // DCLKoutX_DDLY_PD

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 0)  // even X outputs only
        {
        //Debug.Log("Reg", reg);
        Reg.Bit(7, power_down);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOutputHalfStep()  --  output power down
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOutputHalfStep(unsigned int output, bool power_down)
{
    // DCLKoutX_HSg_PD

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 0)  // even X outputs only
        {
        Reg.Bit(6, power_down);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOutputGlitchlessAnalogDelay()  --  output power down
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOutputGlitchlessAnalogDelay(unsigned int output, bool power_down)
{
    // DCLKoutX_ADLYg_PD

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 0)  // even X outputs only
        {
        Reg.Bit(5, power_down);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOutputAnalogDelay()  --  output power down
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOutputAnalogDelay(unsigned int output, bool power_down)
{
    // DCLKoutX_ADLY_PD

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 0)  // even X outputs only
        {
        Reg.Bit(4, power_down);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::IsOutputAnalogDelay_PoweredDown()  --  output power down?
//-----------------------------------------------------------------------------

bool  Lmk04820::IsOutputAnalogDelay_PoweredDown(unsigned int output)
{
    // DCLKoutX_ADLY_PD (read)

    // Figure out location of field
    unsigned int reg;
    switch (output>>1)
        {
        default:
        case  0:  reg=0x106;  break;
        case  1:  reg=0x10E;  break;
        case  2:  reg=0x116;  break;
        case  3:  reg=0x11E;  break;
        case  4:  reg=0x126;  break;
        case  5:  reg=0x12E;  break;
        case  6:  reg=0x136;  break;
        }

    PtrBitManipulator Reg(&RegisterImage[reg]);
    if ((output&1) == 0)  // even X outputs only
        {
        return Reg.Bit(4);
        }
    else
        {
        return true;  // not present is considered off.
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownPll_1()  --  power down PLL1
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownPll_1(bool power_down)
{
    //  PLL1_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(7, power_down);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownVco()  --  power down Vco
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownVco(bool power_down, bool power_down_ldo)
{
    //  VCO_PD  VCO_LDO_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(5, power_down);
    Reg.Bit(6, power_down_ldo);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOscIn()  --  power down OscIn
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOscIn(bool power_down)
{
    //  OSCin_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(4, power_down);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownSysref_GlobalEnable()  --  use Sysref channel power down
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownSysref_GlobalEnable(bool power_down, bool auto_apply)
{
    //  SYSREF_GBL_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(3, power_down);

    if (auto_apply)
        ApplyImage(0x140);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownAllSysref()  --  power down Sysref
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownAllSysref(bool power_down, bool auto_apply)
{
    //  SYSREF_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(2, power_down);

    if (auto_apply)
        ApplyImage(0x140);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownSysrefDigitalDelay()  --  power down Sysref Digital Dly
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownSysrefDigitalDelay(bool power_down, bool auto_apply)
{
    // SYSREF_DDLY_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(1, power_down);

    if (auto_apply)
        ApplyImage(0x140);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownSysrefPulseGen()  --  power down Sysref Pulse Gen
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownSysrefPulseGen(bool power_down, bool auto_apply)
{
    // SYSREF_PLSR_PD
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(0, power_down);

    if (auto_apply)
        ApplyImage(0x140);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownPll()  --  power down unused PLLs and hardware
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownPll(bool is_powered_down)
{
    PowerDownPll_1(is_powered_down);
    PowerDownVco(is_powered_down, is_powered_down);
    PowerDownOscIn(is_powered_down);
}

//-----------------------------------------------------------------------------
//  Lmk04820::PowerDownOutput()  --  power down/up unused output
//-----------------------------------------------------------------------------

void Lmk04820::PowerDownOutput(unsigned int output, bool state, bool auto_apply)
{
    if ((output&1)==1)
        PowerDownSysref(output, state, auto_apply);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputConfig()  --  Update the Output Config for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputConfig(unsigned int output, unsigned int cfg_setting,
                                bool should_cache_setting, bool auto_apply)
{
    // CLKoutX_FMT  SDCLKoutY_FMT
    if (output < TotalOutputs)
        {
        //  Cache restore-to setting
        if (should_cache_setting)
            OutputConfigField[output] = cfg_setting;
        // Figure out location of field
        unsigned int reg;
        unsigned int ofs;
        switch (output)
            {
            default:
            case  0:  reg=0x107;  ofs=0;  break;
            case  1:  reg=0x107;  ofs=4;  break;
            case  2:  reg=0x10F;  ofs=0;  break;
            case  3:  reg=0x10F;  ofs=4;  break;
            case  4:  reg=0x117;  ofs=0;  break;
            case  5:  reg=0x117;  ofs=4;  break;
            case  6:  reg=0x11F;  ofs=0;  break;
            case  7:  reg=0x11F;  ofs=4;  break;
            case  8:  reg=0x127;  ofs=0;  break;
            case  9:  reg=0x127;  ofs=4;  break;
            case 10:  reg=0x12F;  ofs=0;  break;
            case 11:  reg=0x12F;  ofs=4;  break;
            case 12:  reg=0x137;  ofs=0;  break;
            case 13:  reg=0x137;  ofs=4;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Field(ofs, 3, cfg_setting);

        if (auto_apply)
            ApplyImage(reg);

        // Also set power down if not used
        if (cfg_setting==ocDisabled)
            PowerDownOutput(output, true, auto_apply);
        else
            PowerDownOutput(output, false, auto_apply);


        }
}
//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputPolarity_Inverted()  --  Update the Output Polarity for a channel
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputPolarity_Inverted(unsigned int output, bool inverted)
{
    // CLKoutX_POL
    if (output < TotalOutputs)
        {
        // Figure out location of field
        unsigned int reg;
        unsigned int ofs;
        switch (output)
            {
            default:
            case  0:  reg=0x107;  ofs=3;  break;
            case  1:  reg=0x107;  ofs=7;  break;
            case  2:  reg=0x10F;  ofs=3;  break;
            case  3:  reg=0x10F;  ofs=7;  break;
            case  4:  reg=0x117;  ofs=3;  break;
            case  5:  reg=0x117;  ofs=7;  break;
            case  6:  reg=0x11F;  ofs=3;  break;
            case  7:  reg=0x11F;  ofs=7;  break;
            case  8:  reg=0x127;  ofs=3;  break;
            case  9:  reg=0x127;  ofs=7;  break;
            case 10:  reg=0x12F;  ofs=3;  break;
            case 11:  reg=0x12F;  ofs=7;  break;
            case 12:  reg=0x137;  ofs=3;  break;
            case 13:  reg=0x137;  ofs=7;  break;
            }

        PtrBitManipulator Reg(&RegisterImage[reg]);
        Reg.Bit(ofs, inverted);
        }
}

/*
//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputOsc0Config()  --  Configure Osc0 output config
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputOsc0Config(unsigned int cfg_setting)
{
    PtrBitManipulator Reg(&RegisterImage[10]);
    Reg.Field(24, 4, cfg_setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputOsc1Config()  --  Configure Osc1 Lvpecl level
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputOsc1Config(unsigned int cfg_setting)
{
    PtrBitManipulator Reg(&RegisterImage[10]);
    Reg.Field(30, 2, cfg_setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::EnableOutputOsc0()  --  enable Osc0 output
//-----------------------------------------------------------------------------

void Lmk04820::EnableOutputOsc0(bool state)
{
    PtrBitManipulator Reg(&RegisterImage[10]);
    Reg.Bit(22, state);
}

//-----------------------------------------------------------------------------
//  Lmk04820::EnableOutputOsc1()  --  enable Osc1 output
//-----------------------------------------------------------------------------

void Lmk04820::EnableOutputOsc1(bool state)
{
    PtrBitManipulator Reg(&RegisterImage[10]);
    Reg.Bit(23, state);
}
*/
//-----------------------------------------------------------------------------
//  Lmk04820::SetInputAutoSwitchEnable()  --  auto-switch enable
//-----------------------------------------------------------------------------

void Lmk04820::SetInputAutoSwitchEnable(unsigned int ref_idx, bool auto_switch)
{
    //  CLKin2_EN CLKin1_EN CLKin0_EN
    PtrBitManipulator Reg(&RegisterImage[0x146]);
    unsigned int ofs;
    switch (ref_idx)
        {
        default:
        case 0:   ofs = 3;  break;
        case 1:   ofs = 4;  break;
        case 2:   ofs = 5;  break;
        }

    Reg.Bit(ofs, auto_switch);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetInputConfig()  --  Update the Input Config
//-----------------------------------------------------------------------------

void Lmk04820::SetInputConfig(unsigned int ref_idx, unsigned int cfg_setting)
{
    // CLKin2_TYPE CLKin1_TYPE CLKin0_TYPE
    PtrBitManipulator Reg(&RegisterImage[0x146]);
    unsigned int ofs;
    switch (ref_idx)
        {
        default:
        case 0:   ofs = 0;  break;
        case 1:   ofs = 1;  break;
        case 2:   ofs = 2;  break;
        }

	Reg.Bit(ofs, (cfg_setting) ? true : false);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetLockType()  --  Set
//-----------------------------------------------------------------------------

void Lmk04820::SetLockType(unsigned int setting)
{
    PtrBitManipulator Reg(&RegisterImage[0x15F]);  // PLL1_LD_MUX
    Reg.Field(3, 5, setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetLockConfig()  --  Set
//-----------------------------------------------------------------------------

void Lmk04820::SetLockConfig(unsigned int setting)
{
    PtrBitManipulator Reg(&RegisterImage[0x15F]);  // PLL1_LD_TYPE
    Reg.Field(0, 3, setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetPll2LockType()  --  Set
//-----------------------------------------------------------------------------

void Lmk04820::SetPll2LockType(unsigned int setting)
{
    PtrBitManipulator Reg(&RegisterImage[0x16E]);  // PLL2_LD_MUX
    Reg.Field(3, 5, setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetPll2LockConfig()  --  Set
//-----------------------------------------------------------------------------

void Lmk04820::SetPll2LockConfig(unsigned int setting)
{
    PtrBitManipulator Reg(&RegisterImage[0x16E]);  // PLL2_LD_TYPE
    Reg.Field(0, 3, setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefDigitalDelayEnable()  --  enable sysref digital delay
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefDigitalDelayEnable(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[0x140]);  // DDLYd_SYSREF_EN
    Reg.Bit(7, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetDigitalDelayEnable()  --  enable digital delay
//-----------------------------------------------------------------------------

void Lmk04820::SetDigitalDelayEnable(unsigned int pair, bool enabled)
{
    // DDLYd12_EN DDLYd10_EN DDLYd7_EN DDLYd6_EN DDLYd4_EN DDLYd2_EN DDLYd0_EN
    PtrBitManipulator Reg(&RegisterImage[0x140]);
    Reg.Bit(pair, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetDigitalDelaySteps()  --  set frequency of adjustments
//-----------------------------------------------------------------------------

void Lmk04820::SetDigitalDelaySteps(unsigned int steps)
{
    // DDLYd_STEP_CNT
    PtrBitManipulator Reg(&RegisterImage[0x141]);
    Reg.Field(0, 4, steps);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetHoldoverEnable() - enable holdover
//-----------------------------------------------------------------------------

void Lmk04820::SetHoldoverEnable(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[0x150]);   //HOLDOVER_EN
    Reg.Bit(0, enabled);
}


//-----------------------------------------------------------------------------
//  Lmk04820::SetHoldoverHitlessSwitch() - enable holdover hitless switching
//-----------------------------------------------------------------------------

void Lmk04820::SetHoldoverHitlessSwitch(bool hitless_switching)
{
    PtrBitManipulator Reg(&RegisterImage[0x150]);   //HOLDOVER_HITLESS_SWITCH
    Reg.Bit(1, hitless_switching);
}

/*
//-----------------------------------------------------------------------------
//  Lmk04820::SetHoldoverMux()  --  Set holdover mux bit source
//-----------------------------------------------------------------------------

void Lmk04820::SetHoldoverMux(unsigned int setting)
{
    PtrBitManipulator Reg(&RegisterImage[13]);
    Reg.Field(27, 5, setting);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetVcoMuxEnable()  --  Enable Vco Mux
//-----------------------------------------------------------------------------

void Lmk04820::SetVcoMuxEnable(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[10]);
    Reg.Bit(12, enabled);
}
*/
//-----------------------------------------------------------------------------
//  Lmk04820::SetPll2RefDoublerEnable()  --  Enable Reference Doubler
//-----------------------------------------------------------------------------

void Lmk04820::SetPll2RefDoublerEnable(bool enabled)
{
    //  PLL2_REF_2X_EN
    PtrBitManipulator Reg(&RegisterImage[0x162]);
    Reg.Bit(0, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetPll2ExternalCrystalEnable()  --  Enable Ext Crystal
//-----------------------------------------------------------------------------

void Lmk04820::SetPll2ExternalCrystalEnable(bool enabled)
{
    //  PLL2_XTAL_EN
    PtrBitManipulator Reg(&RegisterImage[0x162]);
    Reg.Bit(1, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefDelayClear()  --  Init Sysref
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefDelayClear(bool cleared, bool auto_apply)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYSREF_CLR
    Reg.Bit(7, cleared);

    if (auto_apply)
        ApplyImage(0x143);

}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncOneshotEnable()  --  edge sensitive SYNC
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncOneshotEnable(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYNC_1SHOT_EN
    Reg.Bit(6, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncPolarityInverted()  --  Enable SYNC polarity inversion
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncPolarityInverted(bool inverted, bool auto_apply)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYNC_POL
    Reg.Bit(5, inverted);

    if (auto_apply)
        ApplyImage(0x143);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncEnable()  --  Enable SYNC
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncEnable(bool enabled, bool auto_apply)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYNC_EN
    Reg.Bit(4, enabled);

    if (auto_apply)
        ApplyImage(0x143);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSync_AssertTo_Pll2_DLD()  --  Assert sync until PLL2 locks
//-----------------------------------------------------------------------------

void Lmk04820::SetSync_AssertTo_Pll2_DLD(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYNC_PLL2_DLD
    Reg.Bit(3, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSync_AssertTo_Pll1_DLD()  --  Assert sync until PLL1 locks
//-----------------------------------------------------------------------------

void Lmk04820::SetSync_AssertTo_Pll1_DLD(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYNC_PLL1_DLD
    Reg.Bit(2, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncMode()  --  Configure SYNC generation Mode
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncMode(unsigned int cfg_setting, bool auto_apply)
{
    PtrBitManipulator Reg(&RegisterImage[0x143]);    // SYNC_MODE
    Reg.Field(0, 2, cfg_setting);

    if (auto_apply)
        ApplyImage(0x143);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetNoSyncCLKout()  --  Disable SYNC on output
//-----------------------------------------------------------------------------

void Lmk04820::SetNoSyncCLKout(unsigned int pair, bool no_sync, bool auto_apply)
{
    // SYNC_DIS12 SYNC_DIS10 SYNC_DIS8 SYNC_DIS6 SYNC_DIS4 SYNC_DIS2 SYNC_DIS0
    PtrBitManipulator Reg(&RegisterImage[0x144]);
    Reg.Bit(pair, no_sync);

    if (auto_apply)
        ApplyImage(0x144);

}

//-----------------------------------------------------------------------------
//  Lmk04820::SetNoSyncSysref()  --  Disable SYNC on output
//-----------------------------------------------------------------------------

void Lmk04820::SetNoSyncSysref(bool no_sync, bool auto_apply)
{
    // SYNC_DISSYSREF
    PtrBitManipulator Reg(&RegisterImage[0x144]);
    Reg.Bit(7, no_sync);

    if (auto_apply)
        ApplyImage(0x144);

}

/*

//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncQual()  --  Enable SYNC QUAL mode
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncQual(bool enabled)
{
    PtrBitManipulator Reg(&RegisterImage[11]);    // SYNC_QUAL
    Reg.Bit(17, enabled);
}

*/
/*
//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncAuto()  --  Enable SYNC auto mode
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncAuto(bool auto_sync)
{
    PtrBitManipulator Reg(&RegisterImage[11]);    // SYNC_EN_AUTO
    Reg.Bit(15, auto_sync);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSyncType()  --  Configure SYNC pin type
//-----------------------------------------------------------------------------

void Lmk04820::SetSyncType(unsigned int cfg_setting)
{
    PtrBitManipulator Reg(&RegisterImage[11]);    // SYNC_TYPE
    Reg.Field(12, 3, cfg_setting);
}
*/
//-----------------------------------------------------------------------------
//  Lmk04820::SetPll2WindowSize()  --  Set Window Size
//-----------------------------------------------------------------------------

void Lmk04820::SetPll2WindowSize( )
{
    // PLL2_WND_SIZE
    PtrBitManipulator Reg(&RegisterImage[0x169]);
    Reg.Field(5, 2, 0x2);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetReferenceSource()  --  Set the Reference Source
//-----------------------------------------------------------------------------

void Lmk04820::SetReferenceSource(LmkReferenceSource source)
{
    Debug.Log("Lmk04820::SetReferenceSource");
    Debug.Log("...source=", source);
    // CLKin_SEL_MODE
    unsigned int source_no;
    switch (source)
        {
        default:
        //case refCLKin0:
        case refPrimary:    source_no = 0;  break;

        //case refCLKin1:
        case refSecondary:  source_no = 1;  break;

        case refCLKin2:     source_no = 2;  break;

        case refAuto:
        case refPinSelect:  source_no = 0;  break;
        }

    Debug.Log("...source_no=", source_no);
    PtrBitManipulator Reg(&RegisterImage[0x147]);
    Reg.Field(4, 3, source_no);

    SetSourceSelection(source_no);   // inform base class too

    SetClkin0_Output(c0_Pll1);
    if (IsIn_Pll_Mode())
        SetClkin1_Output(c1_Pll1);
    else
        SetClkin1_Output(c1_Fin);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetInputPolarity()  --  Update the Input Polarity
//-----------------------------------------------------------------------------

void Lmk04820::SetInputPolarity(unsigned int cfg_setting)
{
    //  CLKin_SEL_POL
    PtrBitManipulator Reg(&RegisterImage[0x147]);
	Reg.Bit(7, (cfg_setting)? true : false);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetClkin0_Output()  --  Set the ClkIn0 output direction
//-----------------------------------------------------------------------------

void Lmk04820::SetClkin0_Output(unsigned int setting, bool auto_apply)
{
    //  CLKin0_OUT_MUX
    PtrBitManipulator Reg(&RegisterImage[0x147]);
    Reg.Field(0, 2, setting);

    if (auto_apply)
        ApplyImage(0x147);

}

//-----------------------------------------------------------------------------
//  Lmk04820::SetClkin1_Output()  --  Set the ClkIn1 output direction
//-----------------------------------------------------------------------------

void Lmk04820::SetClkin1_Output(unsigned int setting, bool auto_apply)
{
    //  CLKin1_OUT_MUX
    PtrBitManipulator Reg(&RegisterImage[0x147]);
    Reg.Field(2, 2, setting);

    if (auto_apply)
        ApplyImage(0x147);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetOutputSource()  --  Sert the Output Mode (and Mux outputs)
//-----------------------------------------------------------------------------

void Lmk04820::SetOutputSource(LmkClockMode source)
{
    //
    //  decide on which internal clock soure to direct
    if (source==clockPll)
        {
        SetOperatingMode(FDefaultOpMode);  // configure for PLL
        PowerDownPll(false);
        }
    else
        {
        // External Mode (use reference 1 always)
        SetVcoMuxSource(vcoin_Clkin1);   // configure for external clock
        SetClkin1_Output(c1_Fin);      // configure output 1 direct to Clock Distribution
        // Power Down PLL
        PowerDownPll(true);
        }

}

//------------------------------------------------------------------------------
//  Lmk04820::SetOscOutputFormat()  --  Set output type of OSC OUT
//------------------------------------------------------------------------------

void Lmk04820::SetOscOutputFormat(unsigned int setting)
{
     // OSCout_FMT
    PtrBitManipulator Reg(&RegisterImage[0x138]);
    Reg.Field(0, 4, setting);
}

//------------------------------------------------------------------------------
//  Lmk04820::SetOscOutputSource()  --  Set source type of OSC OUT
//------------------------------------------------------------------------------

void Lmk04820::SetOscOutputSource(unsigned int setting)
{
    // OSCout_MUX
    PtrBitManipulator Reg(&RegisterImage[0x138]);
	Reg.Bit(4, (setting)? true : false);
}

//------------------------------------------------------------------------------
//  Lmk04820::SetVcoMuxSource()  --  Set source type of Vco
//------------------------------------------------------------------------------

void Lmk04820::SetVcoMuxSource(unsigned int setting)
{
     // VCO_MUX
    PtrBitManipulator Reg(&RegisterImage[0x138]);
    Reg.Field(5, 2, setting);
}

//------------------------------------------------------------------------------
//  Lmk04820::SetPll1_NclkSource()  --  Set source of PLL1 NCLK
//------------------------------------------------------------------------------

void Lmk04820::SetPll1_NclkSource(unsigned int setting)
{
     // PLL1_NCLK_MUX
    PtrBitManipulator Reg(&RegisterImage[0x13F]);
	Reg.Bit(3, (setting) ? true : false);
}

//------------------------------------------------------------------------------
//  Lmk04820::SetPll2_NclkSource()  --  Set source of PLL2 NCLK
//------------------------------------------------------------------------------

void Lmk04820::SetPll2_NclkSource(unsigned int setting)
{
     // PLL2_NCLK_MUX
    PtrBitManipulator Reg(&RegisterImage[0x13F]);
	Reg.Bit(4, (setting) ? true : false);
}


//------------------------------------------------------------------------------
//  Lmk04820::SetFbMuxSource()  --  Set source of FB-Mux
//------------------------------------------------------------------------------

void Lmk04820::SetFbMuxSource(unsigned int setting)
{
     // FB_MUX
    PtrBitManipulator Reg(&RegisterImage[0x13F]);
    Reg.Field(1, 2, setting);
}

//------------------------------------------------------------------------------
//  Lmk04820::SetFbMuxEnable()  --  Enable FB_Mux
//------------------------------------------------------------------------------

void Lmk04820::SetFbMuxEnable(bool enabled)
{
     // FB_MUX_EN
    PtrBitManipulator Reg(&RegisterImage[0x13F]);
    Reg.Bit(0, enabled);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetVco2Range()  --  Load VCO Range based off of Chip Type
//-----------------------------------------------------------------------------

void  Lmk04820::SetVco2Range(LmkType chip_type)
{
    unsigned int ct = chip_type;
    if (ct >= TotalVcoTypes) ct = 0;

    // Load both internal VCO mods, even though we only use 1
    LocalVcxo_Standard.Load(Lmk04820_Vco0Ranges[ct].Low, Lmk04820_Vco0Ranges[ct].High,
                            Lmk04820_Vco1Ranges[ct].Low, Lmk04820_Vco1Ranges[ct].High);

    LocalVcxo_04821.Load(Lmk04820_Vco0Ranges[ct].Low, Lmk04820_Vco0Ranges[ct].High,
                            Lmk04820_Vco1Ranges[ct].Low, Lmk04820_Vco1Ranges[ct].High);

}

//-----------------------------------------------------------------------------
//  Lmk04820::LocalVcxo()  --  Load VCO Range based off of Chip Type
//-----------------------------------------------------------------------------

IVcxoControl &  Lmk04820::LocalVcxo()
{
    if (FChipType==lt4821)
        return LocalVcxo_04821;
    else
        return LocalVcxo_Standard;
}


//-----------------------------------------------------------------------------
//  Lmk04820::OptimizeVco1Noise()  --  Set Rate Dependent Config of PLL2
//-----------------------------------------------------------------------------

void Lmk04820::OptimizeVco1Noise()
{
    // OPT_REG_1  OPT_REG_2

    unsigned int mask = GetMaskRev();
    unsigned int opt_reg1, opt_reg2;
    switch (mask)
        {
        //  LMK04826
        case 37:
        default:  opt_reg1 = 24;  opt_reg2 = 119;  FChipType=lt4826; break;
        //  LMK04828
        case 32:  opt_reg1 = 21;  opt_reg2 = 51;   FChipType=lt4828; break;
        //  LMK04821
        case 36:  opt_reg1 = 21;  opt_reg2 = 51;   FChipType=lt4821; break;
        }

    PtrBitManipulator Opt1_Reg(&RegisterImage[0x17c]);
    Opt1_Reg.Value(opt_reg1);
    PtrBitManipulator Opt2_Reg(&RegisterImage[0x17d]);
    Opt2_Reg.Value(opt_reg2);

    SetVco2Range(FChipType);

}

//-----------------------------------------------------------------------------
//  Lmk04820::ConfigurePLL2()  --  Set Rate Dependent Config of PLL2
//-----------------------------------------------------------------------------

void Lmk04820::ConfigurePLL2(double OscInRateMHz)
{

    //  OSCin_FREQ
    PtrBitManipulator Reg(&RegisterImage[0x162]);
    unsigned int OSCin_FREQ_code;
    if (OscInRateMHz <= 63.0)
        OSCin_FREQ_code = 0x0;
    else if (OscInRateMHz <= 127.0)
        OSCin_FREQ_code = 0x1;
    else if (OscInRateMHz <= 255.0)
        OSCin_FREQ_code = 0x2;
    else if (OscInRateMHz <= 500.0)
        OSCin_FREQ_code = 0x4;
    else
        OSCin_FREQ_code = 0x4;

    Reg.Field(2, 3, OSCin_FREQ_code);  // Set OSCin_FREQ
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetPLL1_WindowSize()  --  Set Rate Dependent Config of PLL1 WND
//-----------------------------------------------------------------------------
//  PFD threshold rate = 1.0/(window_width/.85)

void Lmk04820::SetPLL1_WindowSize(double PLL1_Pfd_Rate)
{

    //  PLL1_WND_SIZE
    PtrBitManipulator Reg(&RegisterImage[0x15B]);
    unsigned int count;
    unsigned int PLL1_WND_SIZE_code;
    if (PLL1_Pfd_Rate <= 19.7e6)
        {
        PLL1_WND_SIZE_code = 0x3;  // 43 ns
        count = 0x2000;   // default
        }
    else if (PLL1_Pfd_Rate <= 44e6)
        {
        PLL1_WND_SIZE_code = 0x2;  // 19 ns
        count = 0x1000;
        }
    else if (PLL1_Pfd_Rate <= 94e6)
        {
        PLL1_WND_SIZE_code = 0x1;  // 9 ns
        count = 0x800;
        }
    else if (PLL1_Pfd_Rate <= 212e6)
        {
        PLL1_WND_SIZE_code = 0x0;  // 4 ns
        count = 0x400;
        }
    else
        {
        PLL1_WND_SIZE_code = 0x0;
        count = 0x400;
        }

    Reg.Field(6, 2, PLL1_WND_SIZE_code);  // Set PLL1_WND_SIZE
    Debug.Log("Lmk04820::PLL1_WND_SIZE = ", PLL1_WND_SIZE_code);
    SetPLL1_DLD_Count(count);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetPLL1_DLD_Count()  --  Set PLL1_DLD_CNT
//-----------------------------------------------------------------------------

void Lmk04820::SetPLL1_DLD_Count(unsigned int count)
{

  //  PLL1_DLD_CNT
  PtrBitManipulator RegMsb(&RegisterImage[0x15C]);
  PtrBitManipulator RegLsb(&RegisterImage[0x15D]);

  RegMsb.Field(0, 6, count>>8);  // upper part
  RegLsb.Value(count&0xFF);      // lower part
  Debug.Log("Lmk04820::PLL1_DLD_CNT = ", count);

}

//-----------------------------------------------------------------------------
//  Lmk04820::SetPll1CpGain()  --
//-----------------------------------------------------------------------------

void Lmk04820::SetPll1CpGain(unsigned int current_in_uA)
{
    // PLL1_CP_GAIN
    if (current_in_uA<50)
        current_in_uA=50;
    unsigned int field = (current_in_uA-50)/100;
    PtrBitManipulator Reg(&RegisterImage[0x15B]);
    Reg.Field(0, 4, field);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetR1()  --  Configure R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetR1(unsigned int R1)
{
    SetCLKin0_R(R1);
    SetCLKin1_R(R1);
    SetCLKin2_R(R1);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetR1()  --  Read R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetR1()
{
    return GetCLKin0_R();  // all three are the same
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetCLKin0_R()  --  Configure Clkin0 R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetCLKin0_R(unsigned int R)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x154]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x153]);
    Reg_lsb.Value(R);
    Reg_msb.Field(0, 6, R>>8);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetCLKin0_R()  --  Read Clkin0 R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetCLKin0_R()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x154]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x153]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Field(0, 6);

    return (msb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetCLKin1_R()  --  Configure Clkin1 R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetCLKin1_R(unsigned int R)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x156]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x155]);
    Reg_lsb.Value(R);
    Reg_msb.Field(0, 6, R>>8);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetCLKin1_R()  --  Read Clkin1 R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetCLKin1_R()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x156]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x155]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Field(0, 6);

    return (msb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetCLKin2_R()  --  Configure Clkin2 R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetCLKin2_R(unsigned int R)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x158]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x157]);
    Reg_lsb.Value(R);
    Reg_msb.Field(0, 6, R>>8);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetCLKin2_R()  --  Read Clkin2 R1  (PLL1 Reference Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetCLKin2_R()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x158]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x157]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Field(0, 6);

    return (msb<<8) | lsb;
}


//-----------------------------------------------------------------------------
//  Lmk04820::SetN1()  --  Configure N1  (PLL1 Feedback Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetN1(unsigned int N1)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x15a]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x159]);
    Reg_lsb.Value(N1);
    Reg_msb.Field(0, 6, N1>>8);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetN1()  --  Read N1  (PLL1 Feedback Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetN1()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x15a]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x159]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Field(0, 6);

    return (msb<<8) | lsb;
}


//-----------------------------------------------------------------------------
//  Lmk04820::SetR2()  --  Configure R2  (PLL2 Reference Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetR2(unsigned int R2)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x161]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x160]);
    Reg_lsb.Value(R2);
    Reg_msb.Field(0, 4, R2>>8);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetR2()  --  Read R2  (PLL2 Reference Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetR2()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x161]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x160]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Field(0, 4);

    return (msb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetN2()  --  Configure N2  (PLL2 Feedback Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetN2(unsigned int N2)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x168]);
    PtrBitManipulator Reg_ctb(&RegisterImage[0x167]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x166]);
    Reg_lsb.Value(N2);
    Reg_ctb.Value(N2>>8);
    Reg_msb.Field(0, 2, N2>>16);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetN2()  --  Read N2  (PLL2 Feedback Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetN2()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x168]);
    PtrBitManipulator Reg_ctb(&RegisterImage[0x167]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x166]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int ctb = Reg_ctb.Value();
    unsigned int msb = Reg_msb.Field(0, 2);

    return (msb<<16) | (ctb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetN2_Cal()  --  Configure SetN2_Cal  (PLL2 Feedback Divider)
//-----------------------------------------------------------------------------

void Lmk04820::SetN2_Cal(unsigned int N2_Cal)
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x165]);
    PtrBitManipulator Reg_ctb(&RegisterImage[0x164]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x163]);

    Reg_lsb.Value(N2_Cal);
    Reg_ctb.Value(N2_Cal>>8);
    Reg_msb.Field(0, 2, N2_Cal>>16);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetN2_Cal()  --  Read SetN2_Cal  (PLL2 Feedback Divider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetN2_Cal()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x165]);
    PtrBitManipulator Reg_ctb(&RegisterImage[0x164]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x163]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int ctb = Reg_ctb.Value();
    unsigned int msb = Reg_msb.Field(0, 2);

    return (msb<<16) | (ctb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetN2_Pre()  --  Configure PLL2 N Prescaler (Feedback PreDivider)
//-----------------------------------------------------------------------------

void Lmk04820::SetN2_Pre(unsigned int N2_Pre)
{
    //  PLL2_P
    PtrBitManipulator Reg(&RegisterImage[0x162]);
    Reg.Field(5, 3, To_N2_Pre_DividerInput(N2_Pre));
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetN2_Pre()  --  Read PLL2 N Prescaler (Feedback PreDivider)
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetN2_Pre()
{
    //  PLL2_P
    PtrBitManipulator Reg(&RegisterImage[0x162]);
    unsigned int N2_Pre_field = Reg.Field(5, 3);
    return From_N2_Pre_DividerInput(N2_Pre_field);
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetVco1Div()  --  Configure VCO1 post divider (LMK048021 only
//-----------------------------------------------------------------------------

void Lmk04820::SetVco1Div(unsigned int Vco1Div)
{
    //  VCO1_DIV
    PtrBitManipulator Reg(&RegisterImage[0x174]);
    Reg.Field(0, 5, To_Vco1Div_DividerInput(Vco1Div));
}

//-----------------------------------------------------------------------------
//  Lmk04820::SetSysrefDiv()  --  Configure Sysref Divider
//-----------------------------------------------------------------------------

void Lmk04820::SetSysrefDiv(unsigned int divide_by)
{
    if (divide_by<kSYSREF_Min)
        divide_by = kSYSREF_Min;
    else if (divide_by>kSYSREF_Max)
        divide_by = kSYSREF_Max;

    PtrBitManipulator Reg_lsb(&RegisterImage[0x13B]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x13A]);
    Reg_lsb.Value(divide_by);
    Reg_msb.Field(0, 5, divide_by>>8);

    SetSysrefCommonDigitalDelay(divide_by);
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetSysrefDiv()  --  Read Sysref Divider
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetSysrefDiv()
{
    PtrBitManipulator Reg_lsb(&RegisterImage[0x13B]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x13A]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Field(0, 5);

    return (msb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetDeviceType()  --  Read device Type
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetDeviceType()
{
    // ID_DEVICE_TYPE
    PtrBitManipulator Reg_lsb(&RegisterImage[0x002]);

    unsigned int lsb = Reg_lsb.Value();

    return lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetProductId()  --  Read ProductId
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetProductId()
{
    // ID_PROD

    PtrBitManipulator Reg_lsb(&RegisterImage[0x005]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x004]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Value();

    return (msb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetMaskRev()  --  Read IC Version ID
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetMaskRev()
{
    //  ID_MASKREV
    PtrBitManipulator Reg_lsb(&RegisterImage[0x006]);

    unsigned int lsb = Reg_lsb.Value();

    return lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetVendorId()  --  Read ProductId
//-----------------------------------------------------------------------------

unsigned int Lmk04820::GetVendorId()
{
    // ID_VNDR

    PtrBitManipulator Reg_lsb(&RegisterImage[0x00D]);
    PtrBitManipulator Reg_msb(&RegisterImage[0x00C]);

    unsigned int lsb = Reg_lsb.Value();
    unsigned int msb = Reg_msb.Value();

    return (msb<<8) | lsb;
}

//-----------------------------------------------------------------------------
//  Lmk04820::ReadDataRegisters()  --  Read all data registers into Image vector
//-----------------------------------------------------------------------------

void Lmk04820::ReadDataRegisters()
{
    // Skip 0-1
    // Read 2-0x0D
    for (unsigned int i=2; i<=0x0D; i++)
        {
        //if (i==9)
        //    continue;
        RegisterImage[i] = Port(i);
        }
    // Read 0x100-0x190
    for (unsigned int i=0x100; i<0x165; i++)
        {
        RegisterImage[i] = Port(i);
        }

    //mirroring WriteDataRegisters()
    RegisterImage[0x171] = Port(0x171);
    RegisterImage[0x172] = Port(0x172);
    RegisterImage[0x174] = Port(0x174);
    RegisterImage[0x17c] = Port(0x17c);
    RegisterImage[0x17d] = Port(0x17d);

    for (unsigned int i=0x166; i<=0x173; i++)
        {
        RegisterImage[i] = Port(i);
        }

    RegisterImage[0x145] = 127;
    RegisterImage[0x171] = 0xAA;
    RegisterImage[0x172] = 0x02;
    // Skip 0x1FFD-0x1FFF
}

//-----------------------------------------------------------------------------
//  Lmk04820::WriteDataRegisters()  --  Write all data registers from Image vector
//-----------------------------------------------------------------------------

void Lmk04820::WriteDataRegisters()
{
    ScopedTraceRef ccc(Debug, "Lmk04820::WriteDataRegisters()");
    // Skip 0-1
    // Write 2
    Port(2, RegisterImage[2]);
    // Skip 0x3-0x0D
    // Write 0x100-0x165
    for (unsigned int i=0x100; i<=0x165; i++)
        {
        if (i==0x102 || i==0x10a || i==0x112 || i==0x11a || i==0x122 || i==0x12a || i==0x132 )
            continue;
        Port(i, RegisterImage[i]);
        }
    // Write 0x171-0x172
    Port(0x171, RegisterImage[0x171]);
    Port(0x172, RegisterImage[0x172]);
    // Write 0x174 only for LMK04821 part
    if (FChipType==lt4821)
        Port(0x174, RegisterImage[0x174]);
    // Write 0x17c-0x17d
    Port(0x17c, RegisterImage[0x17c]);
    Port(0x17d, RegisterImage[0x17d]);
    // Write 0x166-0x173
    for (unsigned int i=0x166; i<=0x173; i++)
        {
        if (i>=0x16f && i<=0x172)
            continue;
        Port(i, RegisterImage[i]);
        }
    // Skip 0x175-0x1FF (except for 0x17c, 0x17d)
    // Skip 0x1FFD-0x1FFF
}

//-----------------------------------------------------------------------------
//  Lmk04820::OnSetPower()  --  Perform Pwr Setting operation
//-----------------------------------------------------------------------------

void  Lmk04820::OnSetPower(bool & new_state)
{
    //  POWER DOWN
    PtrBitManipulator Reg1(&RegisterImage[2]);
    Reg1.Bit(0, new_state);
    //  pause to allow chip to activate
    Sleep(10);
}


//-----------------------------------------------------------------------------
//  Lmk04820::InitDividerSupport()  --  Initialize Special Divider Maps
//-----------------------------------------------------------------------------

void  Lmk04820::InitDividerSupport()
{
    //  N2_Pre are PLL2 Feedback Prescaler dividers
    N2_Pre_DividerValues.push_back(2);     N2_Pre_DividerValues.push_back(3);
    N2_Pre_DividerValues.push_back(4);     N2_Pre_DividerValues.push_back(5);
    N2_Pre_DividerValues.push_back(6);     N2_Pre_DividerValues.push_back(7);
    N2_Pre_DividerValues.push_back(8);

    //  Valid dividers is keyed by divide_by, value is Input Value
    N2_Pre_ValidDividers[2]    = 0x02;       N2_Pre_ValidDividers[3]    = 0x03;
    N2_Pre_ValidDividers[4]    = 0x04;       N2_Pre_ValidDividers[5]    = 0x05;
    N2_Pre_ValidDividers[6]    = 0x06;       N2_Pre_ValidDividers[7]    = 0x07;
    N2_Pre_ValidDividers[8]    = 0x00;

    //     Valid dividers is keyed by Input Value, value is divide_by
    N2_Pre_InverseDividers[0x00]  =    8;      N2_Pre_InverseDividers[0x01]  =    2;
    N2_Pre_InverseDividers[0x02]  =    2;      N2_Pre_InverseDividers[0x03]  =    3;
    N2_Pre_InverseDividers[0x04]  =    4;      N2_Pre_InverseDividers[0x05]  =    5;
    N2_Pre_InverseDividers[0x06]  =    6;      N2_Pre_InverseDividers[0x07]  =    7;

    //  VcoDiv is PLL2 VCO post_scaler divider
    VcoDiv_DividerValues.push_back(2);     VcoDiv_DividerValues.push_back(3);
    VcoDiv_DividerValues.push_back(4);     VcoDiv_DividerValues.push_back(5);
    VcoDiv_DividerValues.push_back(6);     VcoDiv_DividerValues.push_back(7);
    VcoDiv_DividerValues.push_back(8);

    //  Valid dividers is keyed by divide_by, value is Input Value
    VcoDiv_ValidDividers[2]    = 0x02;       VcoDiv_ValidDividers[3]    = 0x03;
    VcoDiv_ValidDividers[4]    = 0x04;       VcoDiv_ValidDividers[5]    = 0x05;
    VcoDiv_ValidDividers[6]    = 0x06;       VcoDiv_ValidDividers[7]    = 0x07;
    VcoDiv_ValidDividers[8]    = 0x00;

    //     Valid dividers is keyed by Input Value, value is divide_by
    VcoDiv_InverseDividers[0x00]  =    8;      VcoDiv_InverseDividers[0x01]  =    2;
    VcoDiv_InverseDividers[0x02]  =    2;      VcoDiv_InverseDividers[0x03]  =    3;
    VcoDiv_InverseDividers[0x04]  =    4;      VcoDiv_InverseDividers[0x05]  =    5;
    VcoDiv_InverseDividers[0x06]  =    6;      VcoDiv_InverseDividers[0x07]  =    7;

    //  Vco1_Div is VCO1 internal post divider
    Vco1Div_DividerValues.push_back(2);     Vco1Div_DividerValues.push_back(3);
    Vco1Div_DividerValues.push_back(4);     Vco1Div_DividerValues.push_back(5);
    Vco1Div_DividerValues.push_back(6);     Vco1Div_DividerValues.push_back(7);
    Vco1Div_DividerValues.push_back(8);

    Vco1Div_DividerValues_Rev.push_back(8);     Vco1Div_DividerValues_Rev.push_back(7);
    Vco1Div_DividerValues_Rev.push_back(6);     Vco1Div_DividerValues_Rev.push_back(5);
    Vco1Div_DividerValues_Rev.push_back(4);     Vco1Div_DividerValues_Rev.push_back(3);
    Vco1Div_DividerValues_Rev.push_back(2);

    //  Valid dividers is keyed by divide_by, value is Input Value
    Vco1Div_ValidDividers[2]    = 0x00;       Vco1Div_ValidDividers[3]    = 0x05;
    Vco1Div_ValidDividers[4]    = 0x14;       Vco1Div_ValidDividers[5]    = 0x17;
    Vco1Div_ValidDividers[6]    = 0x1e;       Vco1Div_ValidDividers[7]    = 0x1b;
    Vco1Div_ValidDividers[8]    = 0x0a;


}

//-----------------------------------------------------------------------------
//  Lmk04820::IsValidDivider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------
//  Output Group Output Dividers

bool  Lmk04820::IsValidDivider(unsigned int divide_by)
{
	if (divide_by !=0 && divide_by<=kOD_Max)
		return true;
	else
		return false;
}

//-----------------------------------------------------------------------------
//  Lmk04820::ToDividerInput() -- Returns input value for divide_by
//-----------------------------------------------------------------------------
//  this maps divider ratios to the field value in the chip hardware

unsigned int  Lmk04820::ToDividerInput(unsigned int divide_by)
{
    if (IsValidDivider(divide_by))
        {
        if (divide_by==32)
            return 0;
        else
            return divide_by;
        }
    else
        return 1;   // if invalid divider, divide by 1.
}

//-----------------------------------------------------------------------------
//  Lmk04820::FromDividerInput() -- Returns divide_by for input value
//-----------------------------------------------------------------------------
//  this maps the field value in the chip hardware to divider ratios

unsigned int  Lmk04820::FromDividerInput(unsigned int divide_field)
{
    if (divide_field==0)
        return 32;
    else
        return divide_field;
}

//-----------------------------------------------------------------------------
//  Lmk04820::IsValidSysrefDivider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------
//  Sysref Divider

bool  Lmk04820::IsValidSysrefDivider(unsigned int divide_by)
{
    if (divide_by>=kSYSREF_Min && divide_by<=kSYSREF_Max)
        return true;
    else
        return false;
}

//-----------------------------------------------------------------------------
//  Lmk04820::To_N2_Pre_DividerInput() -- Returns input value for divide_by
//-----------------------------------------------------------------------------
//  this maps divider ratios to the field value in the chip hardware

unsigned int  Lmk04820::To_N2_Pre_DividerInput(unsigned int divide_by)
{
    if (IsValid_N2_Pre_Divider(divide_by))
        return N2_Pre_ValidDividers[divide_by];
    else
        return N2_Pre_ValidDividers[1];   // if invalid divider, divide by 1.
}

//-----------------------------------------------------------------------------
//  Lmk04820::From_N2_Pre_DividerInput() -- Returns divide_by for input value
//-----------------------------------------------------------------------------
//  this maps divider bits in the field value in the chip hardware to divide_by result

unsigned int  Lmk04820::From_N2_Pre_DividerInput(unsigned int divider_bits)
{
    InverseDividerMap::iterator iter = N2_Pre_InverseDividers.find(divider_bits);

    if (iter != N2_Pre_InverseDividers.end())
        return N2_Pre_InverseDividers[divider_bits];
    else
        return 1;   // if invalid divider_bits, claim it is divide by 1
}

//-----------------------------------------------------------------------------
//  Lmk04820::IsValid_N2_Pre_Divider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  Lmk04820::IsValid_N2_Pre_Divider(unsigned int divide_by)
{
    ValidDividerMap::iterator iter = N2_Pre_ValidDividers.find(divide_by);
    return iter != N2_Pre_ValidDividers.end();
}

//-----------------------------------------------------------------------------
//  Lmk04820::To_VcoDiv_DividerInput() -- Returns input value for divide_by
//-----------------------------------------------------------------------------
//  this maps divider ratios to the field value in the chip hardware

unsigned int  Lmk04820::To_VcoDiv_DividerInput(unsigned int divide_by)
{
    if (IsValid_VcoDiv_Divider(divide_by))
        return VcoDiv_ValidDividers[divide_by];
    else
        return VcoDiv_ValidDividers[1];   // if invalid divider, divide by 1.
}

//-----------------------------------------------------------------------------
//  Lmk04820::From_VcoDiv_DividerInput() -- Returns divide_by for input value
//-----------------------------------------------------------------------------
//  this maps divider bits in the field value in the chip hardware to divide_by result

unsigned int  Lmk04820::From_VcoDiv_DividerInput(unsigned int divider_bits)
{
    InverseDividerMap::iterator iter = VcoDiv_InverseDividers.find(divider_bits);

    if (iter != VcoDiv_InverseDividers.end())
        return VcoDiv_InverseDividers[divider_bits];
    else
        return 1;   // if invalid divider_bits, claim it is divide by 1
}

//-----------------------------------------------------------------------------
//  Lmk04820::IsValid_VcoDiv_Divider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  Lmk04820::IsValid_VcoDiv_Divider(unsigned int divide_by)
{
    ValidDividerMap::iterator iter = VcoDiv_ValidDividers.find(divide_by);
    return iter != VcoDiv_ValidDividers.end();
}

//-----------------------------------------------------------------------------
//  Lmk04820::To_Vco1Div_DividerInput() -- Returns input value for divide_by
//-----------------------------------------------------------------------------
//  this maps divider ratios to the field value in the chip hardware

unsigned int  Lmk04820::To_Vco1Div_DividerInput(unsigned int divide_by)
{
    if (IsValid_Vco1Div_Divider(divide_by))
        return Vco1Div_ValidDividers[divide_by];
    else
        return Vco1Div_ValidDividers[1];   // if invalid divider, divide by lowest divider.
}

//-----------------------------------------------------------------------------
//  Lmk04820::IsValid_Vco1Div_Divider() -- tests if divide_by is a valid divider
//-----------------------------------------------------------------------------

bool  Lmk04820::IsValid_Vco1Div_Divider(unsigned int divide_by)
{
    ValidDividerMap::iterator iter = Vco1Div_ValidDividers.find(divide_by);
    return iter != Vco1Div_ValidDividers.end();
}

//-----------------------------------------------------------------------------
//  Lmk04820::CheckDivider() -- Check for valid divider value
//-----------------------------------------------------------------------------

bool  Lmk04820::CheckDivider(unsigned int output, unsigned int divide_by)
{
    if ((output&1)==0)
        {
        return IsValidDivider(divide_by);
        }
    else
        {
        return IsValidSysrefDivider(divide_by);
        }
}

//-----------------------------------------------------------------------------
//  Lmk04820::GetNearestDividers() -- Find nearest divider to a candidate
//-----------------------------------------------------------------------------

void  Lmk04820::GetNearestDividers(unsigned int output, float candidate, int & below, int & above)
{
    below = static_cast<int>(candidate);
    above = below+1;

    if ((output&1)==0)
        {
        // range check for standard dividers
        if (below<static_cast<int>(kOD_Min))
            below = above = kOD_Min;
        if (below>static_cast<int>(kOD_Max))
            below = above = kOD_Max;
        if (above>static_cast<int>(kOD_Max))
            above = kOD_Max;
        }
    else // if ((output&1)==1)
        {
        // range check for SYSREF divider
        if (below<static_cast<int>(kSYSREF_Min))
            below = above = kSYSREF_Min;
        if (below>static_cast<int>(kSYSREF_Max))
            below = above = kSYSREF_Max;
        if (above>static_cast<int>(kSYSREF_Max))
            above = kSYSREF_Max;
        }

}

//-----------------------------------------------------------------------------
//  Lmk04820::ToDigitalDelayCount() -- Returns field value for input value
//-----------------------------------------------------------------------------
//  this maps digital delay count in clocks to the register field values

unsigned int  Lmk04820::ToDigitalDelayCount(unsigned int count)
{
    if (count>=16)
        return 0;
    else if (count==1)
        return 2;
    else
        return count;
}

//-----------------------------------------------------------------------------
//  Lmk04820::ToSysrefDigitalDelayCount() -- Returns field value for input value
//-----------------------------------------------------------------------------
//  this maps digital delay count in clocks to the register field values

unsigned int  Lmk04820::ToSysrefDigitalDelayCount(unsigned int count)
{
    if (count>11)
        return 10;
    else if (count<2)
        return 0;  // Bypass
    else
        return count-1;
}

//-----------------------------------------------------------------------------
//  Lmk04820::ToSysrefCommonDigitalDelayCount() -- Returns field value for input value
//-----------------------------------------------------------------------------
//  this maps digital delay count in clocks to the register field values

unsigned int  Lmk04820::ToSysrefCommonDigitalDelayCount(unsigned int count)
{
    if (count>=8191)
        return 8191;
    else if (count<8)
        return 8;
    else
        return count;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS Lmk04820::InternalVcxo  --  In-chip VCXO interface
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Lmk04826, Lmk04828
//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo::CanTuneTo() -- return true if VCO can tune here
//-----------------------------------------------------------------------------

bool    Lmk04820::InternalVcxo::CanTuneTo(double freq)
{
    bool retValue;

    if (freq >= Min() && freq <= Gap0Min())
        retValue = true;
    else if (freq >= Gap0Max() && freq <= Max())
        retValue = true;
    else
        retValue = false;

    return retValue;
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo::BestTunableFrequency() -- find closest allowed freq
//-----------------------------------------------------------------------------

double Lmk04820::InternalVcxo::BestTunableFrequency(double freq)
{
    double retValue;

    if (freq < Min())
        retValue = Min();
    else if (freq >= Min() && freq <= Gap0Min())
        retValue = freq;
    else if (freq >= Gap0Min() && freq <= (Gap0Min()+Gap0Max()/2))
        retValue = Gap0Min();
    else if (freq >= (Gap0Min()+Gap0Max()/2) && freq <= Gap0Max())
        retValue = Gap0Max();
    else if (freq >= Gap0Max() && freq <= Max())
        retValue = freq;
    else
        retValue = Max();

    return retValue;
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo::SetVcoFrequency()
//-----------------------------------------------------------------------------

void  Lmk04820::InternalVcxo::SetVcoFrequency(double freq)
{
    ScopedTraceRef ccc(Pll.Debug, "Lmk04820::InternalVcxo::SetVcoFrequency()");

    Pll.FVco2Frequency = freq;

    //  Check which half of VCO to use
    bool usesHighHalf = (Pll.FVco2Frequency > FVco2Range0_Max);

    //  Program Range Control and Prescaler
    if (usesHighHalf)
        Pll.SetVcoMuxSource( vcoin_Vco1 );
    else
        Pll.SetVcoMuxSource( vcoin_Vco0 );

    Pll.Debug.Log("VCO Values");
    Pll.Debug.Log("Lmk04820::Vco Uses High Range = ", (usesHighHalf));
    Pll.Debug.Log("Lmk04820::Output Vco Freq (MHz) =", Pll.FVco2Frequency/1.0e6);
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo::GetVcoFrequency()  --  read Vco Frequenc7y
//-----------------------------------------------------------------------------

double  Lmk04820::InternalVcxo::GetVcoFrequency()
{
    return Pll.FVco2Frequency;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS Lmk04820::InternalVcxo_04821  --  In-chip VCXO interface
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  Lmk04821
//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo_04821::CanTuneTo() -- return true if VCO can tune here
//-----------------------------------------------------------------------------

bool  Lmk04820::InternalVcxo_04821::CanTuneTo(double freq)
{
//    ScopedTraceRef ccc(Pll.Debug, "Lmk04820::InternalVcxo_04821::CanTuneTo()");
//    Pll.Debug.Log("Tune Freq = ", freq);

    bool retValue(false);

    //  Check Range 0
    if (freq >= FVco2Range0_Min && freq <= FVco2Range0_Max)
        retValue = true;

    //  Check Range 1 divided by all VCO1 dividers
    for (unsigned int idx=0; idx<Pll.Vco1Div_DividerValues.size();  idx++)
        {
        unsigned int div = Pll.Vco1Div_DividerValues[idx];
        if (freq >= FVco2Range1_Min/div && freq <= FVco2Range1_Max/div)
            {
            retValue = true;
            break;
            }
        }

    return retValue;
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo_04821::RangeStart() -- start of valid range N
//-----------------------------------------------------------------------------

double Lmk04820::InternalVcxo_04821::RangeStart(size_t idx)
{
    size_t vco1_ranges =  Pll.Vco1Div_DividerValues_Rev.size();
    if (idx<vco1_ranges)
        return FVco2Range1_Min/Pll.Vco1Div_DividerValues_Rev[idx];
    else
        return FVco2Range0_Min;
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo_04821::RangeEnd() -- end of valid range N
//-----------------------------------------------------------------------------

double Lmk04820::InternalVcxo_04821::RangeEnd(size_t idx)
{
    size_t vco1_ranges = Pll.Vco1Div_DividerValues_Rev.size();
    if (idx<vco1_ranges)
        return FVco2Range1_Max/Pll.Vco1Div_DividerValues_Rev[idx];
    else
        return FVco2Range0_Max;
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo_04821::BestTunableFrequency() -- find closest allowed freq
//-----------------------------------------------------------------------------

double Lmk04820::InternalVcxo_04821::BestTunableFrequency(double freq)
{
    double retValue(Min());
    size_t loop_ranges = Pll.Vco1Div_DividerValues_Rev.size();

    if (freq < Min())
        retValue = Min();
    else if (freq >= RangeStart(loop_ranges) && freq <= RangeEnd(loop_ranges)) // check final range
        retValue = freq;
    else if (freq >= Max())
        retValue = Max();
    else  // loop over all lower ranges...
        {
        for (unsigned int rng=0;  rng<loop_ranges;  rng++)
            {
            if (freq >= RangeStart(rng) && freq <= RangeEnd(rng))
                retValue = freq;
            else if (freq >= RangeEnd(rng) && freq <= (RangeEnd(rng)+RangeStart(rng+1)/2))
                retValue = RangeEnd(rng);
            else if (freq >= (RangeEnd(rng)+RangeStart(rng+1)/2) && freq <= RangeStart(rng+1))
                retValue = RangeStart(rng+1);
            }
        }

    return retValue;
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo_04821::SetVcoFrequency()
//-----------------------------------------------------------------------------

void  Lmk04820::InternalVcxo_04821::SetVcoFrequency(double freq)
{
    ScopedTraceRef ccc(Pll.Debug, "Lmk04820::InternalVcxo_04821::SetVcoFrequency()");
//    Pll.Debug.Log("FVco2Range0_Min= ", FVco2Range0_Min);
//    Pll.Debug.Log("FVco2Range0_Max= ", FVco2Range0_Max);
//    Pll.Debug.Log("FVco2Range1_Min= ", FVco2Range1_Min);
//    Pll.Debug.Log("FVco2Range1_Max= ", FVco2Range1_Max);

    Pll.FVco2Frequency = freq;

    //  Check which half of VCO to use
    bool usesHighHalf = (Pll.FVco2Frequency < FVco2Range0_Min);  // looks odd, but "hi" range has lower VCO

    //  Program Range Control and Prescaler
    if (usesHighHalf)
        Pll.SetVcoMuxSource( vcoin_Vco1 );
    else
        Pll.SetVcoMuxSource( vcoin_Vco0 );

    FVco1_Div = Pll.DivideToBestInt(FVco2Range1_Max, freq);
    Pll.SetVco1Div(FVco1_Div);

    Pll.Debug.Log("VCO Values");
    Pll.Debug.Log("Lmk04820::Vco Uses High Range = ", (usesHighHalf));
    Pll.Debug.Log("Lmk04820::Vco1_Div = ", (FVco1_Div));
    Pll.Debug.Log("Lmk04820::Output Vco Freq (MHz) =", Pll.FVco2Frequency/1.0e6);
}

//-----------------------------------------------------------------------------
//  Lmk04820::InternalVcxo_04821::GetVcoFrequency()  --  read Vco Frequenc7y
//-----------------------------------------------------------------------------

double  Lmk04820::InternalVcxo_04821::GetVcoFrequency()
{
    return Pll.FVco2Frequency;
}


} // namespace Innovative
