//
// Lmk04820_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef LMK04820_MbH
#define LMK04820_MbH

#include <TimebaseDevice_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Trace_Mb.h>
#include <Events_Mb.h>
#include <VcxoControl_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS Lmk04820PllTuningEvent  --  Override PLL Tuning Parameters
//===========================================================================

class Lmk04820PllTuningEvent : public OpenWire::Event
{
public:
    unsigned int &CLKin0_R;
    unsigned int &CLKin1_R;
    unsigned int &CLKin2_R;
    unsigned int &N1;
    unsigned int &R2;
    unsigned int &N2;
    unsigned int &N2_Cal;
    unsigned int &N2_Pre;
    unsigned int &OD;
    unsigned int &Feedback_OD;

    Lmk04820PllTuningEvent(unsigned int & clk0_R,  unsigned int & clk1_R,
                            unsigned int & clk2_R,  unsigned int & n1,
                            unsigned int & r2, unsigned int & n2, unsigned int & n2_cal,
                            unsigned int & n2_pre, unsigned int & od, unsigned int & feedback_od)
        : CLKin0_R(clk0_R), CLKin1_R(clk1_R), CLKin2_R(clk2_R), N1(n1),
          R2(r2), N2(n2), N2_Cal(n2_cal), N2_Pre(n2_pre), OD(od), Feedback_OD(feedback_od)
        {
        }
};

//===========================================================================
//  CLASS Lmk04820Registers  -- Addresses for Device Registers
//===========================================================================

class Lmk04820Registers
{
public:
    unsigned int   Control_Addr;
    unsigned int   SpiIntf_Addr;
};

//=============================================================================
//  CLASS Lmk04820  --  PLL timebase control
//=============================================================================

class Lmk04820 : public PllTimebaseDevice
{
public:
    enum LmkType { lt4826, lt4828, lt4821 };
    enum LmkReferenceSource { refPrimary=0x0, refCLKin0=0x0, refSecondary=0x1, refCLKin1=0x1,
                              refCLKin2=0x2, refPinSelect=0x3, refAuto=0x4 };
    enum LmkClockMode { clockPll, clockDistrib };  // run the PLL or Distribute Ext Clock

    enum OperatingModes { omDualLoopMode, omCascadedZeroDelay, omNestedZeroDelay };
    enum OutputSyncModes { osmPulsorSync, osmSysRefSync };
    enum TriggerSyncModes { tsmUnused, tsmLogic, tsmLowLatency };

    enum FbMux_Sources {  fbm_Dclk6, fbm_Dclk8, fbm_Sysref, fbm_External };

    enum LmkLockTypes { lltPll_1_Lock=0x1, lltPll_2_Lock=0x2, lltPll_1_2_Lock=0x3 };
    //
    //  Member Functions
    Lmk04820(IAddressingSpace &space, Lmk04820Registers & regs, OperatingModes pllMode=omNestedZeroDelay,
                  TriggerSyncModes tsMode=tsmUnused);
    virtual ~Lmk04820();

    void   Init();
    void   Reapply();

    void   OutputSync();


    //void   Sync(bool state);
    void   Rewrite();

    //  Power state
    void  Powered(bool value, bool At_Open=false);
    bool  Powered()
             {  return FPowered;  }
    //  Select the Input reference
    void  ReferenceSource(LmkReferenceSource value);
    LmkReferenceSource  ReferenceSource()
             {  return FReferenceSource;  }
    //  Select Feedback Output
    void  FeedbackOutput(FbMux_Sources source)
        {  FFbMuxSource = source;  }
    FbMux_Sources  FeedbackOutput()
        {  return FFbMuxSource;  }

    //  Select Operating Mode
    void  OperatingMode(OperatingModes opmode)
        {  FDefaultOpMode = opmode;  }
    OperatingModes  OperatingMode()
        {  return FDefaultOpMode;  }

    //  Select Output Mode
    void  Pll_Mode()
             {  OutputSource(clockPll);  }
    void  ExtClock_Mode()
             {
             ReferenceSource(refCLKin1);  // must use clock 1
             OutputSource(clockDistrib);
             }
    void  OutputSource(LmkClockMode value)
             {
             FClkMode = value;
             UpdateMode = (value==clockPll) ? updFrequency : updDivider;
             SetOutputSource(value);
             }
    LmkClockMode  OutputSource() const
             { return FClkMode; }
    bool  IsIn_Pll_Mode()
             {  return FClkMode == clockPll;  }
    bool  IsIn_ExtClock_Mode()
             {  return IsIn_Pll_Mode() == false;  }

    void  OutputSyncMode(OutputSyncModes mode)
        {   FDefaultSyncMode = mode;   }
    OutputSyncModes  OutputSyncMode()
        {   return FDefaultSyncMode;  }
    void  TriggerSyncMode(TriggerSyncModes mode)
        {   FDefaultTriggerSyncMode = mode;   }
    TriggerSyncModes  TriggerSyncMode()
        {   return FDefaultTriggerSyncMode;  }

    //
    //  Aux Tune Match - enable three stage more restrictive tune match to
    //    perfectly match VCO at lower dividers if possible. Default is off, and returns 1.
    virtual bool AuxTuneMatchEnabled();
    virtual unsigned int AuxTuneMatchFactor(unsigned int idx);

    void                LoadWidebandVco(IVcxoControl * aVco);
    IVcxoControl*       WB_Vcxo() const  { return WbVco;  }

    bool  Locked() const;
    void  LockBitType(LmkLockTypes setting)
             {  SetLockType(static_cast<unsigned int>(setting));  }
    bool  OutputSyncStatus()
             {  return LastOutputSyncStatus;  }

    int                 MaxOutputDivider() const;

    //  Raw configuration accesses
    virtual void            Port(unsigned int reg, unsigned int value);
    virtual unsigned int    Port(unsigned int reg);
    void            ApplyImage(unsigned int reg);
    void            PortImage(unsigned int reg, unsigned int value);
    unsigned int    PortImage(unsigned int reg);

    OpenWire::EventHandler<Lmk04820PllTuningEvent> OnPllTuning;

    double  FrontEndPhaseDetectorFrequency() const   // Pll_1 PDF
        {  return FFrontEndPhaseDetectorFrequency;  }
    double  FrontEndVcxoFrequency() const            // Pll_1 Vcxo
        {  return FFrontEndVcxoFrequency;  }
    double  PhaseDetectorFrequency() const           // Pll_2 PDF
        {  return FPhaseDetectorFrequency;  }
    double  Modal_FrontEnd_N_InputFrequency();

    void  SetFrontEndPhaseDetectorFrequency(double freq)
        {  FFrontEndPhaseDetectorFrequency = freq;  }
    void  SetFrontEndVcxoFrequency(double freq)
        {  FFrontEndVcxoFrequency = freq;  }
    void  SetPhaseDetectorFrequency(double freq)
        {  FPhaseDetectorFrequency = freq;  }

    virtual bool IsFeedbackOutputAdc() = 0;
    double EffectiveFeedbackFrequency();
    double EffectiveFeedbackFrequencyActual();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  Class TuneMatchResults
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class TuneMatchResults
    {
    public:
        double CandidateVco;
        unsigned int OD;

        TuneMatchResults(double vco, unsigned int od)
            :  CandidateVco(vco), OD(od)
            {}
    };

    void   PulsorModeEnter();
    void   PulsorModeFire(unsigned int Log2PulseCnt);
    void   PulsorModeExit();

protected:
    //
    //  Member Data
    LmkType                    FChipType;
    LmkReferenceSource         FReferenceSource;
    LmkClockMode               FClkMode;
    OperatingModes             FDefaultOpMode;
    OutputSyncModes            FDefaultSyncMode;
    TriggerSyncModes           FDefaultTriggerSyncMode;
    bool                       FAllow_Pll1_FeedbackAdjustment;
    bool                       LastOutputSyncStatus;
    FbMux_Sources              FFbMuxSource;
    std::vector<int>           RegisterImage;
    std::vector<float>         OutputFrequencyLimit;
    std::vector<unsigned int>  OutputConfigField;
    std::vector<unsigned int>  OutputIsCopy;
    std::vector<unsigned int>  OutputSyncDisable;

    double  FFrontEndPhaseDetectorFrequency;
    double  FFrontEndVcxoFrequency;
    double  FPhaseDetectorFrequency;

    unsigned int SavedRegImagex139;
    unsigned int SavedRegImagex140;
    unsigned int SavedRegImagex143;

    virtual void    OnInit() = 0;
    virtual void    OnReapply() = 0;
    virtual void    OnSetPower(bool & new_state);

    virtual double  DefaultFrontEndPhaseDetectorFrequency() const = 0;   // Pll_1 PDF
    virtual double  DefaultFrontEndVcxoFrequency() const = 0;            // Pll_1 Vcxo
    virtual double  DefaultPhaseDetectorFrequency() const = 0;           // Pll_2 PDF

    void  Allow_Pll1_FeedbackAdjustment(bool allowed=true)
                {  FAllow_Pll1_FeedbackAdjustment = allowed;  }

    // Sync Setup Methods
    void   PulsorSync();
    void   SystemRefSync();
    void   TriggerSyncSetup();
    virtual void  OnAfterOutputSync();

    //
    //  RESERVED CONFIG FUNCTIONS (Library Only)
    //----------------------------------------------------------------------------
/*
    enum OutputGroupSources { ogsVco=0, ogsOscIn=0x1, };
    enum OutputGroupDelayModes { dmNone=0x0, dmEven=0x1, dmOdd=0x2, dmBoth=0x3 };
*/
    enum OutputCfgSettings { ocDisabled=0x0, ocLvds=0x1, ocHsds_06ma=0x2,
                             ocHsds_08ma=0x3, ocHsds_10ma=0x4, ocLvPecl_1600=0x5,
                             ocLvPecl_2000=0x6, ocLcPecl=0x7
                           };
    enum OutputOscCfgSettings { oocDisabled=0x0, oocLvds=0x1, oocLvPecl_1600=0x4, oocLvPecl_2000=0x5,
                                 oocLvCmos_P_Ninv=0x6, oocLvCmos_Pinv_N=0x7,
                                 oocLvCmos_P_N=0x8, oocLvCmos_Pinv_Ninv=0x9,
                                 oocLvCmos_Plow_N=0x0a, oocLvCmos_Plow_Ninv=0x0b,
                                 oocLvCmos_P_Nlow=0x0c, oocLvCmos_Pinv_Nlow=0x0d,
                                 oocLvCmos_Plow_Nlow=0x0e,
                               };
    enum OutputDividerModeSettings {  odmDivider=0x0, odmDivider_HalfStep=0x1, odmDividerBypass=0x2,
                                      odmDivider_Delay=0x3
                                   };
    enum AnalogDelayValues { adv_500ps, adv_525ps, adv_550ps, adv_575ps, adv_600ps, adv_625ps, adv_650ps, adv_675ps,
                             adv_700ps, adv_725ps, adv_750ps, adv_775ps, adv_800ps, adv_825ps, adv_850ps, adv_875ps,
                             adv_900ps, adv_925ps, adv_950ps, adv_975ps, adv_1000ps, adv_1025ps, adv_1050ps, adv_1075ps
                           };
    enum SysrefAnalogDelayValues { sadv_700ps, sadv_1300ps, sadv_1450ps, sadv_1600ps, sadv_1750ps, sadv_1900ps, sadv_2050ps, sadv_2200ps,
                                   sadv_2350ps, sadv_2500ps, sadv_2650ps, sadv_2800ps, sadv_2950ps, sadv_3100ps, sadv_3250ps, sadv_3400ps
                           };
    enum SysrefClkIn0MuxSources { srck0mux_SysrefMux, srck0mux_Clkin0Direct };
    enum SysrefSources { srefs_NormalSync, srefs_Reclocked, srefs_Pulser, srefsContinuous };
    enum SysrefOutputModes { som_Normal, som_low_if_off, som_hi_if_off, som_high };

    enum VcoMuxInputs { vcoin_Vco0, vcoin_Vco1, vcoin_ExtVco, vcoin_Clkin1=vcoin_ExtVco };
    enum OscOutSources { oscsrc_OscIn, oscsrc_FbMux };

    enum SysrefPulseCounts { spc_1, spc_2, spc_4, spc_8 };

    enum Pll_1_Nclk_Sources { p1i_OscIn, p1i_FbMux };
    enum Pll_2_Nclk_Sources { p2i_PllPrescaler, p2i_FbMux };

    enum InputCfgSettings { icBipolar=0x0, icCmos=0x1 };
    enum InputClkin0_Outputs { c0_Sysref=0x0, c0_Pll1=0x2 };
    enum InputClkin1_Outputs { c1_Fin=0x0, c1_FbMux=0x1, c1_Pll1=0x2 };
    enum InputPolaritySettings { ipActiveHigh=0x0, ipActiveLow=0x1  };

    enum LockConfigSettings { stpOutWpush_pull=0x3,
                              stpOutInvWpush_pull, stpOutWopendrain=0x6   };

    enum SyncModes { sm_Disabled, sm_SyncPin, sm_PulsorPin, smPulsorSpiWrite };

    void  SetOperatingMode(unsigned int om_setting);

    void  SetReset(bool state);
    void  PowerDownOutputGroup(unsigned int pair, bool state);
    void  PowerDownOutputDigitalDelay(unsigned int output, bool power_down);
    void  PowerDownOutputHalfStep(unsigned int output, bool power_down);
    void  PowerDownOutputGlitchlessAnalogDelay(unsigned int output, bool power_down);
    void  PowerDownOutputAnalogDelay(unsigned int output, bool power_down);
    bool  IsOutputAnalogDelay_PoweredDown(unsigned int output);
    void  PowerDownPll_1(bool power_down);
    void  PowerDownVco(bool power_down, bool power_down_ldo);
    void  PowerDownOscIn(bool power_down);
    void  PowerDownSysref_GlobalEnable(bool power_down, bool auto_apply=false);
    void  PowerDownAllSysref(bool power_down, bool auto_apply=false);
    void  PowerDownSysrefDigitalDelay(bool power_down, bool auto_apply=false);
    void  PowerDownSysrefPulseGen(bool power_down, bool auto_apply=false);
    void  PowerDownOutput(unsigned int output, bool state=true, bool auto_apply=false);
    void  PowerDownPll(bool is_powered_down);

    void  SetOutputConfig(unsigned int output, unsigned int cfg_setting, bool should_cache_setting=true, bool auto_apply=false);
    void  SetOutputPolarity_Inverted(unsigned int output, bool inverted);
    void  SetInputDriveLevel(unsigned int pair, bool drive_level);
    void  SetOutputDriveLevel(unsigned int pair, bool drive_level);
    void  SetDigitalDelayCounts(unsigned int pair, int hi_count, int lo_count);
    void  SetDigitalDelayEnable(unsigned int pair, bool enabled);
    void  SetDigitalDelaySteps(unsigned int steps);
    void  SetAnalogDelayMode(unsigned int output, bool has_duty_cycle_correction);
    void  SetAnalogDelay(unsigned int output, unsigned int delay_setting, bool auto_apply=false);
    void  SetOutputClockHalfStep(unsigned int output, bool has_halfstep);

    void  SetOutput_AsCopy(unsigned int output);    
    void  SetOutput_AsNonCopy(unsigned int output);

    void  SetSysrefClkin0Mux(unsigned int select_setting, bool auto_apply=false);
    void  SetSysrefGroupSource(unsigned int source, bool auto_apply=false);
    void  SetSysrefEnable(unsigned int output, bool source_is_sysref);
    void  SetSysrefDigitalDelay(unsigned int output, unsigned int delay_in_vco_clks);
    void  SetSysrefAnalogDelayEnable(unsigned int output, bool enabled);
    void  SetSysrefAnalogDelay(unsigned int output, unsigned int delay_setting);
    void  SetSysrefOutputPowerdownMode(unsigned int output, unsigned int mode);
    void  PowerDownSysref(unsigned int output, bool power_down, bool auto_apply=false);
    void  SetSysrefCommonDigitalDelay(unsigned int delay);
    void  SetSysrefPulseCount(unsigned int setting, bool auto_apply=false);
    void  SetSysrefDigitalDelayEnable(bool enabled);

    unsigned int  GetDivider(unsigned int output);
    void          SetDivider(unsigned int output, unsigned int divide_by);
    void          SetDividerMode(unsigned int output, unsigned int cfg_setting);

    void  SetInputConfig(unsigned int ref_idx, unsigned int cfg_setting);
    void  SetInputAutoSwitchEnable(unsigned int ref_idx, bool auto_switch);
    void  SetInputPolarity(unsigned int cfg_setting);
    void  SetClkin0_Output(unsigned int setting, bool auto_apply=false);
    void  SetClkin1_Output(unsigned int setting, bool auto_apply=false);
    void  SetReferenceSource(LmkReferenceSource source);

    void  SetLockType(unsigned int settings);
    void  SetLockConfig(unsigned int settings);
    void  SetPll2LockType(unsigned int settings);
    void  SetPll2LockConfig(unsigned int settings);

    void  SetOscOutputFormat(unsigned int setting);
    void  SetOscOutputSource(unsigned int setting);
    void  SetVcoMuxSource(unsigned int setting);
    void  SetPll1_NclkSource(unsigned int setting);
    void  SetPll2_NclkSource(unsigned int setting);
    void  SetFbMuxSource(unsigned int setting);
    void  SetFbMuxEnable(bool enabled);

    void  SetPll1CpGain(unsigned int current_in_uA);

    void  SetHoldoverEnable(bool enabled);
    void  SetHoldoverHitlessSwitch(bool hitless_switching);
/*
    void  SetHoldoverMux(unsigned int setting);
*/
    void  SetVcoMuxEnable(bool enabled);
    void  SetPll2RefDoublerEnable(bool enabled);
    void  SetPll2ExternalCrystalEnable(bool enabled);
    void  SetPll2WindowSize();  // no option...

    void  SetSysrefDelayClear(bool cleared, bool auto_apply=false);
    void  SetSyncEnable(bool enabled, bool auto_apply=false);
    void  SetSync_AssertTo_Pll2_DLD(bool enabled);
    void  SetSync_AssertTo_Pll1_DLD(bool enabled);
    void  SetSyncPolarityInverted(bool inverted, bool auto_apply=false);
    void  SetSyncOneshotEnable(bool enabled);
    void  SetSyncMode(unsigned int mode, bool auto_apply=false);
    void  SetNoSyncCLKout(unsigned int pair, bool no_sync, bool auto_apply=false);
    void  SetNoSyncSysref(bool no_sync, bool auto_apply=false);

    void  SetPLL1_WindowSize(double PLL1_Pfd_Rate);
    void  SetPLL1_DLD_Count(unsigned int count);

    //  Read only fields
    unsigned int GetDeviceType();
    unsigned int GetProductId();
    unsigned int GetMaskRev();
    unsigned int GetVendorId();

        //----- unconverted
/*
    0x148 0 0 CLKin_SEL0_MUX CLKin_SEL0_TYPE
    0x149 0 SDIO_RDBK_TYPE CLKin_SEL1_MUX CLKin_SEL1_TYPE
    0x14A 0 0 RESET_MUX RESET_TYPE

    0x14B LOS_TIMEOUT LOS_EN TRACK_EN HOLDOVER_FORCE MAN_DAC_EN MAN_DAC[9:8]
    0x14C MAN_DAC[7:0]
    0x14D 0 0 DAC_TRIP_LOW
    0x14E DAC_CLK_MULT DAC_TRIP_HIGH
    0x14F DAC_CLK_CNTR
    0x150 0 0 0 HOLDOVER_PLL1_DET HOLDOVER_LOS_DET HOLDOVER_VTUNE_DET
    0x151 0 0 HOLDOVER_DLD_CNT[13:8]
    0x152 HOLDOVER_DLD_CNT[7:0]

    0x15B  PLL1_CP_TRI PLL1_CP_POL
    0x15E 0 0 PLL1_R_DLY PLL1_N_DLY

    0x166 0 0 0 0 0 PLL2_FCAL_DIS
    0x169 0 PLL2_WND_SIZE PLL2_CP_GAIN PLL2_CP_POL PLL2_CP_TRI 1
    0x16A 0 SYSREF_REQ_EN PLL2_DLD_CNT[15:8]
    0x16B PLL2_DLD_CNT[7:0]
    0x16C 0 0 PLL2_LF_R4 PLL2_LF_R3
    0x16D PLL2_LF_C4 PLL2_LF_C3
    0x173 0 PLL2_PRE_PD PLL2_PD 0 0 0 0 0
    0x182 0 0 0 0 0 RB_PLL1_LD_LOST RB_PLL1_LD CLR_PLL1_LD_LOST
    0x183 0 0 0 0 0 RB_PLL2_LD_LOST RB_PLL2_LD CLR_PLL2_LD_LOST
    0x184 RB_DAC_VALUE[9:8] RB_CLKin2_SEL RB_CLKin1_SEL RB_CLKin0_SEL X RB_CLKin1_LOS RB_CLKin0_LOS
    0x185 RB_DAC_VALUE[7:0]
    0x188 0 0 0 RB_HOLDOVER X X X X
    0x1FFD SPI_LOCK[23:16]
    0x1FFE SPI_LOCK[15:8]
    0x1FFF SPI_LOCK[7:0]


*/
    virtual void  SetDividerHdw(unsigned int output, unsigned int divider_setting);
    virtual void  SetOutEnabledHdw(unsigned int output, bool enabled);

    void SetOutEnabledHdw_autoapply(unsigned int output, bool enabled, bool auto_apply);

    void  SetOutputSource(LmkClockMode source);

    //
    //  Internal "Getters"

    void          SetVco2Range(LmkType chip_type);

    //  PLL1 Registers
    void          SetR1(unsigned int R1);
    unsigned int  GetR1();

    void          SetCLKin0_R(unsigned int R);
    unsigned int  GetCLKin0_R();
    void          SetCLKin1_R(unsigned int R);
    unsigned int  GetCLKin1_R();
    void          SetCLKin2_R(unsigned int R);
    unsigned int  GetCLKin2_R();
    void          SetN1(unsigned int N1);
    unsigned int  GetN1();

    //  PLL2 Registers
    void          SetR2(unsigned int R2);
    unsigned int  GetR2();
    void          SetN2(unsigned int N2);
    unsigned int  GetN2();

    void          SetN2_Cal(unsigned int N2_Cal);
    unsigned int  GetN2_Cal();

    void          SetN2_Pre(unsigned int N2);
    unsigned int  GetN2_Pre();

    void          SetVco1Div(unsigned int Vco1Div);
    //unsigned int  GetVcoDiv();

    void          OptimizeVco1Noise();
    void          ConfigurePLL2(double OscInRateMHz/*, double Pll2_PdfMHz*/);

    void          SetSysrefDiv(unsigned int divide_by);
    unsigned int  GetSysrefDiv();

    //  Other Registers
    void  ReadDataRegisters();
    void  WriteDataRegisters();
    void  Wait_UWireReady();
    void  Wait_UWireDataAvailable();

    void    CalculateFrontEndVcxoFrequency();
    void    CalculateFrontEndPLL(unsigned int & CLKin0_R, unsigned int & CLKin1_R, unsigned int & CLKin2_R,
                                 unsigned int & N1);
    void    AdjustPLL1_FeedbackRate(unsigned int& CLKin0_R, unsigned int& CLKin1_R, unsigned int& CLKin2_R,
                                    unsigned int & N1, unsigned int OD);
    bool    TuneMatch(std::vector<TuneMatchResults> &TM_results, unsigned int OD_Max, unsigned int OD_Min);
    void    CalculateBackEndPLL(double OscIn, double & VcoRate, unsigned int & OD,
                                 unsigned int & R2, unsigned int & N2,  unsigned int & N2_Cal, unsigned int & N2_Pre,
                                 double & PdfActual);


    //-------------------------------------------------------------
    // CLASS InternalVcxo  --  In-chip VCXO interface
    //-------------------------------------------------------------

    class InternalVcxo : public IVcxoControl
    {
    public:
        InternalVcxo(Lmk04820 & pll)  :  Pll(pll)  {}
        virtual ~InternalVcxo()  {}

        virtual double  Min() {  return FVco2Range0_Min;  }
        virtual double  Max() {  return FVco2Range1_Max;  }

        double  Gap0Min() { return FVco2Range0_Max;   }
        double  Gap0Max() { return FVco2Range1_Min;   }

        virtual bool    CanTuneTo(double freq);
        virtual double  BestTunableFrequency(double freq);

        virtual void    SetVcoFrequency(double freq);
        virtual double  GetVcoFrequency();
        virtual void    SetRange(double /*min*/, double /*max*/)
                              {  /* not available  */ }


        void    Frequency(double freq)
                    {  SetVcoFrequency(freq);   }
        double  Frequency()
                    {  return GetVcoFrequency();   }
        double  FrequencyActual()
                    {  return GetVcoFrequency();   }

        void    Load(double r0_min, double r0_max, double r1_min, double r1_max)
            {
            FVco2Range0_Min = r0_min;   FVco2Range0_Max = r0_max;
            FVco2Range1_Min = r1_min;   FVco2Range1_Max = r1_max;
            }
    private:
        Lmk04820 & Pll;
        double     FVco2Range0_Min;
        double     FVco2Range0_Max;
        double     FVco2Range1_Min;
        double     FVco2Range1_Max;

    };

    //-------------------------------------------------------------
    // CLASS InternalVcxo_04821  --  In-chip VCXO (Lmk04821 only)
    //-------------------------------------------------------------

    class InternalVcxo_04821 : public IVcxoControl
    {
    public:
        InternalVcxo_04821(Lmk04820 & pll)  :  Pll(pll)  {}
        virtual ~InternalVcxo_04821()  {}

        virtual double  Min() {  return FVco2Range1_Min/Max_Vco1_Div();  }
        virtual double  Max() {  return FVco2Range0_Max;  }

        unsigned int    Max_Vco1_Div() {  return 8;  }

        virtual bool    CanTuneTo(double freq);
        virtual double  BestTunableFrequency(double freq);

        virtual void    SetVcoFrequency(double freq);
        virtual double  GetVcoFrequency();

        virtual void    SetRange(double /*min*/, double /*max*/)
                              {  /* not available  */ }

        double RangeStart(size_t idx);
        double RangeEnd(size_t idx);

        void    Frequency(double freq)
                    {  SetVcoFrequency(freq);   }
        double  Frequency()
                    {  return GetVcoFrequency();   }
        double  FrequencyActual()
                    {  return GetVcoFrequency();   }

        void    Load(double r0_min, double r0_max, double r1_min, double r1_max)
            {
            FVco2Range0_Min = r0_min;   FVco2Range0_Max = r0_max;
            FVco2Range1_Min = r1_min;   FVco2Range1_Max = r1_max;
            }

    private:
        Lmk04820 & Pll;
        double     FVco2Range0_Min;
        double     FVco2Range0_Max;
        double     FVco2Range1_Min;
        double     FVco2Range1_Max;
        unsigned int FVco1_Div;
    };

    friend class Lmk04820::InternalVcxo;
    friend class Lmk04820::InternalVcxo_04821;

    InternalVcxo        LocalVcxo_Standard;
    InternalVcxo_04821  LocalVcxo_04821;
    IVcxoControl &      LocalVcxo();

    IVcxoControl *      WbVco;
    bool                UseWidebandVco;

    //-------------------------------------------------------------
    // CLASS PllControlRegister  --
    //-------------------------------------------------------------
    class PllControlRegister : public Register
    {
    public:
        typedef Register inherited;
        RegisterBit        Reset;
        RegisterBit        Gpo;
        RegisterBit        Status_Ld1;
        RegisterBit        Status_Ld2;
        RegisterBit        Status_ClkIn0;
        RegisterBit        Status_ClkIn1;
        RegisterBit        Status_ClkIn0_Dir;
        RegisterBit        Status_ClkIn1_Dir;
        RegisterBit        Spi_Ready;
        RegisterBit        Spi_DataValid;

        PllControlRegister(IAddressingSpace &space, int offset)
           :  inherited(space, offset),
              Reset(*this, 1),
              Gpo(*this, 12),
              Status_Ld1(*this, 13),
              Status_Ld2(*this, 14),
              Status_ClkIn0(*this, 15),
              Status_ClkIn1(*this, 16),
              Status_ClkIn0_Dir(*this, 17),
              Status_ClkIn1_Dir(*this, 18),
              Spi_Ready(*this, 30),
              Spi_DataValid(*this, 31)
        {  }

    };

    //-------------------------------------------------------------
    // CLASS SpiControlRegister  -- PLL SPI interface
    //-------------------------------------------------------------
    class SpiControlRegister : public CachedShadowRegister
    {
    public:
        typedef CachedShadowRegister inherited;

        RegisterBitGroup   Addr;
        RegisterBit        Rw;
        RegisterBitGroup   WrData;

        SpiControlRegister(IAddressingSpace &space, int offset)
           :  inherited(space, offset),
              Addr(*this, 0, 13),
              Rw(*this, 15),
              WrData(*this, 16, 8)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiStatusRegister  -- PLL SPI interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpiStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Data;

        SpiStatusRegister(IAddressingSpace &space, int offset)
            : inherited("Ad951x::SpiStatusRegister", space, offset),
              Data(*this, 24, 8)
            {
            }
    };

    PllControlRegister  ControlReg;
    SpiControlRegister  SpiReg;
    SpiStatusRegister   SpiStatusReg;

    // Fields
    bool                FPowered;

    double              FVco2Frequency;
    double              FSysRefFrequencyActual;

    double SysRefFrequencyActual()  {  return FSysRefFrequencyActual;  }

    //
    //  PLL Device Implementations
    void            SettingsInit();
    void            FrequencyModeUpdate();
    void            DividerModeUpdate();

    //
    void            SetOutputDividers(double VcoFreq, unsigned int OutDiv);

    virtual bool CheckDivider(unsigned int output, unsigned int divide_by);
//        {  return IsValidDivider(divide_by); }
    virtual void  GetNearestDividers(unsigned int output, float candidate, int & below, int & above);

    void  InitDividerSupport();

protected:
    typedef std::vector<unsigned int> ValidDividerArray;
    typedef std::map<unsigned int, unsigned int> ValidDividerMap;
    typedef std::map<unsigned int, unsigned int> InverseDividerMap;

    // Mapping for  Output Dividers
    bool          IsValidDivider(unsigned int divide_by);
    unsigned int  ToDividerInput(unsigned int divide_by);
    unsigned int  FromDividerInput(unsigned int divider_field);

    // Mapping for  Sysref Dividers
    bool          IsValidSysrefDivider(unsigned int divide_by);

    // Mapping for PLL2 Feedback N Prescaler divider
    ValidDividerArray    N2_Pre_DividerValues;
    ValidDividerMap      N2_Pre_ValidDividers;
    InverseDividerMap    N2_Pre_InverseDividers;

    unsigned int  To_N2_Pre_DividerInput(unsigned int divide_by);
    unsigned int  From_N2_Pre_DividerInput(unsigned int divider_bits);
    bool  IsValid_N2_Pre_Divider(unsigned int divide_by);

    // Mapping for PLL2 VcoDiv N Prescaler divider
    ValidDividerArray    VcoDiv_DividerValues;
    ValidDividerMap      VcoDiv_ValidDividers;
    InverseDividerMap    VcoDiv_InverseDividers;

    unsigned int  To_VcoDiv_DividerInput(unsigned int divide_by);
    unsigned int  From_VcoDiv_DividerInput(unsigned int divider_bits);
    bool  IsValid_VcoDiv_Divider(unsigned int divide_by);

    // Mapping for VCO1 internal post divider
    ValidDividerArray    Vco1Div_DividerValues;
    ValidDividerArray    Vco1Div_DividerValues_Rev;
    ValidDividerMap      Vco1Div_ValidDividers;

    unsigned int  To_Vco1Div_DividerInput(unsigned int divide_by);
    bool  IsValid_Vco1Div_Divider(unsigned int divide_by);


    unsigned int ToDigitalDelayCount(unsigned int count);
    unsigned int ToSysrefDigitalDelayCount(unsigned int count);
    unsigned int ToSysrefCommonDigitalDelayCount(unsigned int count);

    void          SplitN2Factors(unsigned int TotalN2, unsigned int &N2_Pre, unsigned int &N2);
    unsigned int  DivideToBestInt(double number, double divide_by);
private:
    unsigned int MasterOD_Min();
    unsigned int MasterOD_Max();

public:
    Trace         Debug;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative

#endif

