//
// Lmk61E2_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef LMK61E2_MbH
#define LMK61E2_MbH

#include <DataTypes_Mb.h>
#include <I2C_Mb.h>
#include <VcxoControl_Mb.h>
#include <vector>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Lmk61E2 --  Programmable XO/VCXO Chip Support
//=============================================================================

class Lmk61E2 : public IVcxoControl
{
public:
    enum ADD_Mode {  addFloating, addPulledLow, addPulledHigh=3  };

    Lmk61E2(IRequires_I2C_RegIo & device);

    void        Init();
    void        Reapply();

    virtual void    OnInit() = 0;         // derived class customization
    virtual void    OnReapply() = 0;

    void            SetADD_Mode(ADD_Mode newMode)
                        {  FADDMode = newMode;  }

    bool            IsPllLocked()
                        {  return Pll_Lock();  }

    double          Frequency() const
                        { return FFrequency; }
    void            Frequency(double freq);
    double          FrequencyActual() const
                        { return FFrequencyActual; }

    void            Reference(double freq);
    double          Reference() const;

    unsigned int    VendorID();
    unsigned int    RevisionID();
    unsigned int    ProductID();

    //  Raw configuration accesses
    void            Port(unsigned int reg, unsigned int value);
    unsigned char   Port(unsigned int reg);

    IVcxoControl &      VcxoIntf()
        {  return *this;   }
    //
    //  IVcxoControl Interface Impl
    virtual double  Min();
    virtual double  Max();
    virtual void    SetRange(double /*min*/, double /*max*/)
                        {  }        //  Can't set tuning range

    virtual bool    CanTuneTo(double freq);
    virtual double  BestTunableFrequency(double freq);
    virtual void    SetVcoFrequency(double freq)
                        {  Frequency(freq);  }

    void  VcoRange(double low, double high)
            { VcoLow = low;  VcoHigh = high;  }

    //  'Reattach' I2C Io Interface
    void  Relink_I2C(IRequires_I2C_RegIo & device)
            {
            Device = &device;
            }

protected:
    IRequires_I2C_RegIo *    Device;

    double          FFrequency;
    double          FFrequencyActual;
    double          FReference;

    bool            UseClockDoubler;

    double          VcoLow;       // VCO operational frequency range, Hz
    double          VcoHigh;

    ADD_Mode        FADDMode;

    std::vector<unsigned int>  RegisterImage;
    std::vector<unsigned int>  Valid_OD;

    Trace           Debug;
	
    virtual double  GetFrequencyActual() const
                        { return FFrequencyActual; }

    void            ReadStatusRegisters();
    void            ReadDataRegisters();
    void            WriteDataRegisters();

    void            NewFrequency();

protected:
    enum  OutputCfgSettings { ocTriState=0x0, ocLvPecl=0x1, ocLvds=0x2, ocHscl=0x3 };
    void  PllPowerdown(bool state);
    void  OutputBufferPowerdown(bool state);
    void  SetOutputConfig(unsigned int setting);
    bool  Pll_Lock();
    bool  CalActive();

private:
    enum  PllModes { pmInteger, pmFractional };
    enum  MashDitherSettings { mdWeak=0x0, mdDisabled=0x3 };
    enum  MashOrderSettings { moInteger=0x0, moThirdOrder=0x3 };
    enum  ChargePumpSettings { cp6p4mA=0x8, cp1p6mA=0x4 };
    enum  ChargePumpPhaseShift { cppsZero, cpps1p3ns_at100, cpps1ns_at100, cpps0p9ns_at100,
                                cpps1p3ns_at50, cpps1ns_at50, cpps0p9_at50, cpps0p7ns_at50 };
    enum  LoopFilterR2Ohms { lfr2200=1, lfr2500=4, lfr2700=8, lfr21600=32, lfr22400=48, lfr23200=64 };
    enum  LoopFilterR3Ohms { lfr318=0, lfr3205=3, lfr3854=8, lfr31136=9, lfr31535=12, lfr31936=17, lfr32335=20 };
    enum  ClosedLoopWaits { clw150us, clw300us, clw500us, clw2000us };
    enum  VcoWaits { vw20us, vw400us, vw4000us, vw10000us };

    PllModes PllMode;
    void  CalcPllParams(double freq, double pfd, unsigned int &ODiv,
                        unsigned int &NInt, unsigned int &NNum, unsigned int &NDen);
    void  FractionalToRational(double fractional, unsigned int & Num, unsigned int & Den);
    void  LoadPllParams(unsigned int OD, unsigned int N_Int, unsigned int N_Num, unsigned int N_Den);

    void  OutDiv(unsigned int out_div);
    void  NDiv(unsigned int n_div);
    void  PllFracNumerator(unsigned int value);
    void  PllFracDenominator(unsigned int value);

    void  EnableFreqCalibration(bool state);
    void  AutoStart(bool state);
    void  XoCapControl(unsigned int value);

    void  SetMashDitherConfig(unsigned int setting);
    void  SetMashOrderConfig(unsigned int setting);

    void  PllDividerDoublerEnable(bool is_doubled);
    void  SetChargePumpCurrent(unsigned int setting);

    void  SetChargePumpPhaseShift(unsigned int setting);
    void  PllEnableC3(bool state);
    void  SetLoopFilterR2(unsigned int setting);
    void  SetLoopFilterC1(unsigned int setting);
    void  SetLoopFilterR3(unsigned int setting);
    void  SetLoopFilterC3(unsigned int setting);
    void  ClosedLoopWaitPeriod(unsigned int setting);
    void  VcoWaitPeriod(unsigned int setting);

    void  VcoCalibration();
    int   I2C_Address();
    bool  IsWritableDataRegister(unsigned int idx);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative
#endif

