// LockQueue_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef LockQueue_MbH
#define LockQueue_MbH

#include <list>
#include <SyncObjects_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS LockQueue  -- 
//===========================================================================

template <class TItem> class LockQueue
{
protected:
    CriticalSection mListSection;
    std::list<TItem> mBuffersList;

public:
    void Lock()
        {
        mListSection.Enter();
        }

    void Unlock()
        {
        mListSection.Leave();
        }

    void Clear()
        {
        mListSection.Enter();
        mBuffersList.clear();
        mListSection.Leave();
        }
        
    void Add( const TItem &buffer )
        {
        mListSection.Enter();
        mBuffersList.push_back( buffer );
        mListSection.Leave();
        }

    bool Get( TItem &buffer )
        {
        bool Result;
        mListSection.Enter();
        Result = ( mBuffersList.size() > 0 );
        if( Result )
            {
            buffer = mBuffersList.front();
            mBuffersList.pop_front();
            }

        mListSection.Leave();
        return Result;
        }

    bool Delete()
        {
        bool Result;
        mListSection.Enter();
        Result = ( mBuffersList.size() > 0 );
        if( Result )
            mBuffersList.pop_front();

        mListSection.Leave();
        return Result;
        }

    int GetSize()
        {
        mListSection.Enter();
        int Result = mBuffersList.size(); 
        mListSection.Leave();
        return Result;
        }
        
};

//===========================================================================
//  CLASS BufferLockQueue  -- 
//===========================================================================

template <class TItem> class BufferLockQueue
{
protected:
    CriticalSection mListSection;
    std::list<TItem> mBuffersList;
    size_t mCountSamples;

public:
    void Lock()
        {
        mListSection.Enter();
        }

    void Unlock()
        {
        mListSection.Leave();
        }

    void Clear()
        {
        mListSection.Enter();
        mBuffersList.clear();
        mCountSamples = 0; 
        mListSection.Leave();
        }
        
    void AddBuffer( const TItem &buffer )
        {
        mListSection.Enter();
        mCountSamples += buffer.SizeInInts();
        mBuffersList.push_back( buffer );
        mListSection.Leave();
        }

    void AddBack( const TItem &buffer )
        {
        mListSection.Enter();
        mBuffersList.push_front( buffer );
        mCountSamples += buffer.SizeInInts(); 
        mListSection.Leave();
        }

    bool GetBuffer( TItem &buffer )
        {
        bool Result;
        mListSection.Enter();
        Result = ( mBuffersList.size() > 0 );
        if( Result )
            {
            buffer = mBuffersList.front();
            mCountSamples -= buffer.SizeInInts();
            mBuffersList.pop_front();
            }

        mListSection.Leave();
        return Result;
        }

    bool DeleteBuffer()
        {
        bool Result;
        mListSection.Enter();
        Result = ( mBuffersList.size() > 0 );
        if( Result )
            {
            mCountSamples -= mBuffersList.front().SizeInInts();
            mBuffersList.pop_front();
            }

        mListSection.Leave();
        return Result;
        }

    bool PeekBuffer( TItem &buffer )
        {
        bool Result;
        mListSection.Enter();
        Result = ( mBuffersList.size() > 0 );
        if( Result )
            buffer = mBuffersList.front();

        mListSection.Leave();
        return Result;
        }

    size_t GetComulativeSize()
        {
        mListSection.Enter();
        size_t Result = mCountSamples;
        mListSection.Leave();
        return Result;
        }

    size_t GetSize()
        {
        mListSection.Enter();
        size_t Result = mBuffersList.size(); 
        mListSection.Leave();
        return Result;
        }
        
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace

#endif
