// LogicLoadThread_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef LogicLoadThread_MbH
#define LogicLoadThread_MbH

#include <Event_Mb.h>
#include <SRecord_Mb.h>
#include <BitFile_Mb.h>
#include <ProcessEvents_Mb.h>
#include <Thread_Mb.h>
#include <StopWatch_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FpgaLoadBase;

//==============================================================================
//  CLASS LogicLoadThread -- Logic load via select map interface
//==============================================================================

class LogicLoadThread : public Thread
{
    typedef Thread inherited;

public:
    //
    //  Member Functions
    LogicLoadThread(FpgaLoadBase & fpga);
    virtual ~LogicLoadThread();

    // Support
	void                    Configure(std::string file);
	void 					Load(char * buffer, size_t size);

    // Events
    OpenWire::EventHandler<ProcessProgressEvent>     OnFileReadProgress;
    OpenWire::EventHandler<ProcessCompletionEvent>   OnFileReadComplete;

    OpenWire::EventHandler<ProcessProgressEvent>     OnParseProgress;
    OpenWire::EventHandler<ProcessCompletionEvent>   OnParseComplete;
    OpenWire::EventHandler<ProcessStatusEvent>       OnParseError;

    OpenWire::EventHandler<ProcessProgressEvent>     OnLoadProgress;
    OpenWire::EventHandler<ProcessCompletionEvent>   OnLoadComplete;
    OpenWire::EventHandler<ProcessStatusEvent>       OnLoadError;

protected:
    //
    //  Member Functions
    virtual void            Init();
    virtual void            Execute();

private:
    // Data
    FpgaImage *             Image;
    StopWatch               Rtc;

    // FpaEvents
    enum { sConfiguring=inherited::sLast, sLast };
    FpgaLoadBase &          Fpga;
    Event                   Configuring;
    bool                    IsExo;


    // Methods
    void                    Load();
    void                    Progress(int percent);
    void                    Complete(int result);
    void                    Error(const std::string & msg);

    // Event Handlers
    void                    HandleParseProgress(ProcessProgressEvent & event);
    void                    HandleParseComplete(ProcessCompletionEvent & event);
    void                    HandleParseError(ProcessStatusEvent & event);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
