// LogicTemperatureIntf_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef LogicTemperatureIntf_Mb_H
#define LogicTemperatureIntf_Mb_H

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS SingleLogicTemperatureIntf -- User interface to logic temp monitor
//==============================================================================
//  This interface handles a single temperature measuring logic

class SingleLogicTemperatureIntf
{
public:
    //
    //  Interface Functions
	virtual float           LogicTemperature() const = 0;

	virtual float 			LogicWarningTemperature() const = 0;
	virtual void            LogicWarningTemperature(float value) = 0;  // default is 75.0

	virtual float 			LogicFailureTemperature() const = 0;
	virtual void            LogicFailureTemperature(float value) = 0; // default is 85.0

    virtual bool            Failed() const = 0;
    
    virtual ~SingleLogicTemperatureIntf() {}
};

//==============================================================================
//  CLASS DualLogicTemperatureIntf -- User interface to logic temp monitor
//==============================================================================

class DualLogicTemperatureIntf
{
public:
    //
    //  Interface Functions
	virtual float           LogicTemperature(int idx) const = 0;

	virtual float 			LogicWarningTemperature(int idx) const = 0;
	virtual void            LogicWarningTemperature(int idx, float value ) = 0;  // default is 75.0

	virtual float 			LogicFailureTemperature(int idx) const = 0;
	virtual void            LogicFailureTemperature(int idx, float value ) = 0;  // default is 85.0

    virtual bool            Failed(int idx) const = 0;
    
    virtual ~DualLogicTemperatureIntf() {}
};

//==============================================================================
//  CLASS LogicTemperatureIntf -- User interface to logic temp monitor
//==============================================================================

class LogicTemperatureIntf : public SingleLogicTemperatureIntf
{
public:
    virtual ~LogicTemperatureIntf() {}
};

//==============================================================================
//  CLASS ExtendedLogicTemperatureIntf -- User interface to logic temp monitor
//==============================================================================
//  This interface handles a single temperature measuring logic
//   ...Added Fan Temperature

class ExtendedLogicTemperatureIntf : public LogicTemperatureIntf
{
public:
    //
    //  Interface Functions
	virtual float 			LogicFanTemperature() const = 0;
	virtual void            LogicFanTemperature(float value) = 0;  // default is 70.0

    virtual ~ExtendedLogicTemperatureIntf() {}
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // end namespace Innovative


#endif
