// LogicTemperature_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <LogicTemperature_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS IUsesLogicTemperature -- User Interface to user logic ID
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class IUsesLogicTemperature
//---------------------------------------------------------------------------

IUsesLogicTemperature::IUsesLogicTemperature(AddressingSpace & space,
	unsigned int temp_addr, unsigned int warn_addr, unsigned int fail_addr)
	: TempReg(space, temp_addr), WarningReg(space, warn_addr),
	  FailureReg(space, fail_addr), ScaleFactor(.0625)
{
}

//---------------------------------------------------------------------------
//  IUsesLogicTemperature::LogicTemperature() --  Query current temp
//---------------------------------------------------------------------------

float IUsesLogicTemperature::LogicTemperature() const
{
	return TempReg.Degrees.Value() * ScaleFactor;
}

//---------------------------------------------------------------------------
//  IUsesLogicTemperature::LogicWarningTemperature() -- Set warning temp
//---------------------------------------------------------------------------

void IUsesLogicTemperature::LogicWarningTemperature(float value)
{
	WarningReg.Degrees.Value(static_cast<ii32>(value/ScaleFactor));
}

//---------------------------------------------------------------------------
//  IUsesLogicTemperature::LogicWarningTemperature() -- Query warning temp
//---------------------------------------------------------------------------

float IUsesLogicTemperature::LogicWarningTemperature() const
{
	return WarningReg.Degrees.Value() * ScaleFactor;
}

//---------------------------------------------------------------------------
//  IUsesLogicTemperature::LogicFailureTemperature() --  Set failure temp
//---------------------------------------------------------------------------

void IUsesLogicTemperature::LogicFailureTemperature(float value)
{
	FailureReg.Degrees.Value(static_cast<ii32>(value/ScaleFactor));
}

//---------------------------------------------------------------------------
//  IUsesLogicTemperature::LogicFailureTemperature() -- Query failure temp
//---------------------------------------------------------------------------

float IUsesLogicTemperature::LogicFailureTemperature() const
{
	return FailureReg.Degrees.Value() * ScaleFactor;
}

//---------------------------------------------------------------------------
//  IUsesLogicTemperature::Failed() --
//---------------------------------------------------------------------------

bool IUsesLogicTemperature::Failed() const
{
    return false;
}


}  // end namespace Innovative



