// LogicTemperature_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef LogicTemperature_MbH
#define LogicTemperature_MbH

#include <LogicTemperatureIntf_Mb.h>
#include <HardwareRegister_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS IUsesNullLogicTemperature -- User Interface to user logic ID
//===========================================================================

class IUsesNullLogicTemperature : public LogicTemperatureIntf
{
public:
	IUsesNullLogicTemperature()  {  }
	virtual ~IUsesNullLogicTemperature()  {  }

	//
	//  Interface Functions
	virtual float           LogicTemperature() const
		{  return 0.0;  }

	virtual float 			LogicWarningTemperature() const
		{  return 75.0f;  }
	virtual void 			LogicWarningTemperature(float /*value*/)
		{  } 

	virtual float 			LogicFailureTemperature() const
		{  return 85.0f;  }
	virtual void 			LogicFailureTemperature(float /*value*/)
		{  }

    virtual bool            Failed() const
        {  return false;  }
};

//===========================================================================
//  CLASS IUsesLogicTemperature -- User Interface to user logic ID
//===========================================================================

class IUsesLogicTemperature : public LogicTemperatureIntf
{
public:
	IUsesLogicTemperature(AddressingSpace & space, unsigned int temp_addr,
		unsigned int warn_addr, unsigned int fail_addr);
	virtual ~IUsesLogicTemperature()
        {  }

	//
	//  Interface Functions
	virtual float           LogicTemperature() const;

	virtual float 			LogicWarningTemperature() const;
	virtual void 			LogicWarningTemperature(float value);

	virtual float 			LogicFailureTemperature() const;
	virtual void 			LogicFailureTemperature(float value);

    virtual bool            Failed() const;

protected:
	// Registers
	class TempRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBitGroup    Degrees;

		TempRegister(AddressingSpace &space, int offset)
		:  inherited(space, offset),
		   Degrees(*this, 0, 16)
		{  }
	};

	class SetpointRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBitGroup    Degrees;

		SetpointRegister(AddressingSpace &space, int offset)
		:  inherited(space, offset),
		   Degrees(*this, 0, 16)
		{  }
	};

	TempRegister        TempReg;
	SetpointRegister	WarningReg;
    SetpointRegister	FailureReg;

	// Data
	const float 			ScaleFactor;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // end namespace Innovative


#endif
