// LowPass_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <LowPass_Mb.h>
#include <math.h>
#include <ipps.h>   // ipps.h is not redistributable!

namespace   Innovative
{

//---------------------------------------------------------------------------
static const double Pi = 4.0 * atan(1.0);

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif 

//---------------------------------------------------------------------------
// LowPass::LowPass() --
//---------------------------------------------------------------------------
 
LowPass::LowPass() : 
    FFrequency(5000)
{
}

//---------------------------------------------------------------------------
// LowPass::LowPass() --
//---------------------------------------------------------------------------
 
LowPass::LowPass(const LowPass & other)
    : inherited( other ), 
      FFrequency(other.FFrequency)
{
}

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

//---------------------------------------------------------------------------
//  LowPass::operator = --
//---------------------------------------------------------------------------

LowPass & LowPass::operator = (const LowPass & other)
{
    inherited::operator = (other);

    FFrequency = other.FFrequency;

    return *this;
}

//---------------------------------------------------------------------------
//  LowPass::Frequency() --
//---------------------------------------------------------------------------

void LowPass::Frequency(float value)
{
    FFrequency = value;
    Start();
}

//---------------------------------------------------------------------------
//  LowPass::Frequency() --
//---------------------------------------------------------------------------

float LowPass::Frequency() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------
//  LowPass::CalculateCoefficients() --
//---------------------------------------------------------------------------

void LowPass::CalculateCoefficients()
{
	float CutHigh = static_cast<float>((FFrequency / FSampleRate) * 2.0);

	//---------------------------------------------
	// adjust the number of coefficients to be ODD
	//---------------------------------------------
    size_t count = FCoefficient.size();
	size_t CoefNum = count;
	if (count % 2 == 0)
		++CoefNum;

	size_t HalfLen = (CoefNum - 1) / 2;

	//--------------------------------------------------------
	// Allocate memory for coefficients if length changed
	//--------------------------------------------------------
	FCoefficient.resize(CoefNum);

    //------------------
    // Lowpass filter
    //------------------
    FCoefficient[HalfLen] = CutHigh;

    for (size_t Cnt = 1; Cnt <= HalfLen; Cnt++)
        {
        float   TmpFloat = static_cast<float>(Pi * Cnt);
        FCoefficient[HalfLen + Cnt] = sin(CutHigh * TmpFloat) / TmpFloat;
        FCoefficient[HalfLen - Cnt] = FCoefficient[HalfLen + Cnt];
        }

    //------------------------------
    // multiplying with a window
    //------------------------------
    ApplyWindow(&FCoefficient[0], CoefNum, FWindowType, FAlpha);

}

} // namespace Innovative
