// M6713DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <M6713_Mb.h>
#include <M6713DeviceMap_Mb.h>
#include <M6713MemoryMap_Mb.h>
#include <M6713DeviceDriver_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <FpaLoadBase_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <Exception_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

namespace Innovative
{
//===========================================================================
//  CLASS FpaControlRegister
//===========================================================================

class FpaControlRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup            Mode;
    RegisterBit                 Clk;
    RegisterBit                 Out;
    ReadOnlyRegisterBit         In;
    ReadOnlyRegisterBitGroup    FirmwareRevision;
    ReadOnlyRegisterBitGroup    PcbRevision;
    ReadOnlyRegisterBitGroup    Id;

public:
    FpaControlRegister(AddressingSpace &space,
        int offset)
    :  inherited(space, offset),
       Mode(*this, 0, 3), Clk(*this, 3), Out(*this, 4), In(*this, 5),
       FirmwareRevision(*this, 8, 8), PcbRevision(*this, 16, 4),
       Id(*this, 20, 12)
       {  }
};

//==============================================================================
//  CLASS FpaLoader -- Interface to FPA register
//==============================================================================

class FpaLoader : public FpaLoadBase
{
public:
    FpaLoader(AddressingSpace & space)
        : Reg(space, M6713_MemoryMap::mmFpa)
        {  }

    // Methods
    // Read/Write data
    virtual bool            Read()
        {  return Reg.In ? 1 : 0;  }
    virtual void            Write(bool state)
        {  Reg.Out = state;  }
    // Read/write clock state
    virtual void            Clock(bool state)
        {  Reg.Clk = state;  }

    // Mode set
    virtual void            Mode(int mode)
        {  Reg.Mode = mode;  }
    // Device select
    virtual void            Device(int /*device*/)
        {  }
    // Firmware revision readback
    virtual int             FirmwareRevision()
        {  return Reg.FirmwareRevision;  }
    // Pcb revision readback
    virtual int             PcbRevision()
        {  return Reg.PcbRevision;  }
    // Id readback
    virtual int             Id()
        {  return Reg.Id;  }

private:
    // Data
    FpaControlRegister      Reg;
};

//===========================================================================
//  CLASS M6713HdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class M6713HdwPluginImpl : public IPacketDevicePlugin
{
public:
    M6713HdwPluginImpl( M6713DeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    M6713DeviceMap * PDM;
};

//===========================================================================
//  CLASS M6713DeviceMap  -- Registers for M6713 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class M6713DeviceMap
//---------------------------------------------------------------------------

using namespace M6713_MemoryMap;

PacketDeviceRegisters M6713DRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
    0,                    // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters M6713RegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

//  Locations of the HPI registers
static HpiEngineAddresses  M6713HpiAddresses =
    {
    mmHpiAddressLo,  mmHpiAddressHi,  //  HPI Address Register    LO, HI
    mmHpiDataLo,     mmHpiDataHi,     //  HPI Data Register       LO, HI
    mmHpiDataIncrLo, mmHpiDataIncrHi, //  HPI Data Incr Register  LO, HI
    mmHpiControlLo,  mmHpiControlHi   //  HPI Control Register    LO, HI
    };

//  Values of the HPI registers
static HpiCeSpaceValues  M6713HpiCeValues =
    {
    0x00003078,         //  EMIF global control register 
    0x0000C041,         //  CE1
    0x21A28A22,         //  CE0
    0x00000030,         //  CE2
    0x11010420,         //  CE3
    0x00000350,         //  SDRAM Refresh Timing register
    0x000544a7,         //  SDRAM Extension
    0x6B338000          //  SDRAM control register 
    };

//---------------------------------------------------------------------------
//  constructor for class M6713DeviceMap
//---------------------------------------------------------------------------

M6713DeviceMap::M6713DeviceMap(M6713 * owner)
    :  PacketDeviceImpl(new M6713DeviceDriver, M6713DRegAddrs),
       PacketDeviceMap(this, M6713RegAddrs, 0),
       IUsesVirtexFpgaLoader(PortMemory, mmFpgaData, mmFpgaControl),
       IUsesStandardHpiCpu(*this, M6713HpiAddresses, M6713HpiCeValues),
       Owner(owner),
       DDeviceDriver(dynamic_cast<M6713DeviceDriver*>(DeviceDriver)),
       FFpa(new FpaLoader(PortMemory)),
       RcvPacketSize(0x80), XmitPacketSize(0x80),
       PciTestControlReg( PortMemory,  mmPciTestControl)
{
    EepromLoader = new FpaLoadThread(*FFpa);
    //
    //  Hook stream events from base class
    OnBeforeCoffLoad.SetEvent(this, &M6713DeviceMap::OnBeforeCoffLoadHandler);
    OnStreamConfigure.SetEvent(this, &M6713DeviceMap::OnStreamConfigureHandler);
    OnBeforeStreamStart.SetEvent(this, &M6713DeviceMap::OnBeforeStreamStartHandler);
    OnAfterStreamStart.SetEvent(this, &M6713DeviceMap::OnAfterStreamStartHandler);
    OnBeforeStreamStop.SetEvent(this, &M6713DeviceMap::OnBeforeStreamStopHandler);
    OnAfterStreamStop.SetEvent(this, &M6713DeviceMap::OnAfterStreamStopHandler);
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new M6713HdwPluginImpl(this) );

}

//---------------------------------------------------------------------------
//  destructor for class M6713DeviceMap
//---------------------------------------------------------------------------

M6713DeviceMap::~M6713DeviceMap()
{
    EepromLoader->Shutdown();
    delete EepromLoader;
    EepromLoader = 0;

    delete FFpa;
    FFpa = 0;
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OpenHardware()
{
//    if ( DeviceDriver->Opened() )     Need a 'only once' protector??
//        return;

    IUsesStandardHpiCpu::Open(DeviceDriver->PortMemoryAddress());

    EepromLoader->Initialize();
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::ResetBaseBoard()  -- HPI cold initialization
//---------------------------------------------------------------------------

void M6713DeviceMap::ResetBaseBoard()
{
	DspInReset(true);
	Innovative::Sleep(10);
	DspInReset(false);
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OnBeforeCoffLoadHandler()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OnBeforeCoffLoadHandler(ProcessCompletionEvent & /*Event*/)
{
    if (!IsConfigured())
        throw MalibuException("Baseboard logic is not configured!");
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnStreamConfigure.Execute(Event);

}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //  Set loopback mode
    LocalLoopbackMode(Owner->FLocalLoopbackMode);

}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  M6713DeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  M6713DeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  M6713DeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<M6713DeviceDriver::LOG->Count; i++)
        vec.push_back(M6713DeviceDriver::LOG->Status[i]);

}

//---------------------------------------------------------------------------
//  M6713DeviceMap::LocalLoopbackMode()
//---------------------------------------------------------------------------

void  M6713DeviceMap::LocalLoopbackMode(bool state)
{
    if (state)
        {
        PciTestControlReg.LoopbackEnable.Set();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Set();
        }
    else
        {
        PciTestControlReg.LoopbackEnable.Clear();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Clear();
        }
}


//===========================================================================
//  CLASS M6713DeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PmcDeviceMap::M6713DeviceMap
//---------------------------------------------------------------------------

M6713DeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TestModeEnable( *this, 0 ),
      GenerateRxData( *this, 1 ),
      LoopbackEnable( *this, 2 ),
      ClearTxCredit( *this, 3 ),
      ClearRxCredit( *this, 4 ),
      DrainXmitData( *this, 5 ),
      GenPacketSize( *this, 8, 4 ),
      GenPacketSizeUpdate( *this, 12 )
{
}



} // namespace Innovative


