// M6713.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <M6713_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <HpiEngine_Mb.h>
#include <Exception_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS M6713  -- Racal Pmc Board object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class M6713
//---------------------------------------------------------------------------

M6713::M6713()
    :  LegacyBoard(Device), Device(this), FLocalLoopbackMode(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class M6713
//---------------------------------------------------------------------------

M6713::~M6713()
{
}

//---------------------------------------------------------------------------
//  M6713::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t M6713::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  M6713::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  M6713::Open()
{
    static_cast<IPacketDevice &>(Device).Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  M6713::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  M6713::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  M6713::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  M6713::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  M6713::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  M6713::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  M6713::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  M6713::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  M6713::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  M6713::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  M6713::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  M6713::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  M6713::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  M6713::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  M6713::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  M6713::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  M6713::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  M6713::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  M6713::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  M6713::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  M6713::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  M6713::Log( int type, int value )
{
    Device.Log(type, value);
}

//---------------------------------------------------------------------------
//  M6713::Logic() -- Return reference to FPGA Loader Interface
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & M6713::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  M6713::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & M6713::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  M6713::Cpu() -- Return reference to Cpu Interface object
//---------------------------------------------------------------------------

IUsesOnboardHpiCpu & M6713::Cpu()
{
    return Device;
}

//---------------------------------------------------------------------------
//  M6713::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & M6713::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  M6713::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & M6713::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  M6713::BusMasterSize() --
//---------------------------------------------------------------------------

void M6713::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  M6713::BusMasterSize() --
//---------------------------------------------------------------------------

int  M6713::BusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif

}  // namespace Innovative
