// MlsSequence_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "MLSequence_Mb.h"

using namespace std;

namespace   Innovative
{

//---------------------------------------------------------------------------
//  MLSequence::Order() --
//---------------------------------------------------------------------------

unsigned int MLSequence::Order(const unsigned int order)
{
    unsigned int elems = (1 << order) - 1;

    mMls.resize(elems);
    vector < unsigned char > t(order);

    //Initialise shift register to all 1s
    for (unsigned int i = 0; i < order; i++)
        t[i] = 1;

    for (unsigned int i = 0; i < elems; i++)
        {
        //add shifted out bit to sequence
        mMls[i] = t[order - 1];

        //xor
        int add(0);
        switch (order)
            {
            case 3:     add = t[3 - 1] ^ t[2 - 1]; break;
            case 4:     add = t[4 - 1] ^ t[3 - 1]; break;
            case 5:     add = t[5 - 1] ^ t[3 - 1]; break;
            case 6:     add = t[6 - 1] ^ t[5 - 1]; break;
            case 7:     add = t[7 - 1] ^ t[6 - 1]; break;
            case 8:     add = t[8 - 1] ^ t[6 - 1] ^ t[5 - 1] ^ t[4 - 1]; break;
            case 9:     add = t[9 - 1] ^ t[5 - 1]; break;
            case 10:    add = t[10 - 1] ^ t[7 - 1]; break;
            case 11:    add = t[11 - 1] ^ t[9 - 1]; break;
            case 12:    add = t[12 - 1] ^ t[6 - 1] ^ t[4 - 1] ^ t[1 - 1]; break;
            case 13:    add = t[13 - 1] ^ t[4 - 1] ^ t[3 - 1] ^ t[1 - 1]; break;
            case 14:    add = t[14 - 1] ^ t[5 - 1] ^ t[3 - 1] ^ t[1 - 1]; break;
            case 15:    add = t[15 - 1] ^ t[14 - 1]; break;
            case 16:    add = t[16 - 1] ^ t[15 - 1] ^ t[13 - 1] ^ t[4 - 1]; break;
            case 17:    add = t[17 - 1] ^ t[14 - 1]; break;
            case 18:    add = t[18 - 1] ^ t[11 - 1]; break;
            case 19:    add = t[19 - 1] ^ t[6 - 1] ^ t[2 - 1] ^ t[1 - 1]; break;
            case 20:    add = t[20 - 1] ^ t[17 - 1]; break;
            case 21:    add = t[21 - 1] ^ t[19 - 1]; break;
            case 22:    add = t[22 - 1] ^ t[21 - 1]; break;
            case 23:    add = t[23 - 1] ^ t[18 - 1]; break;
            }

        //shift
        for (int j = order - 1; j >= 1; j--)
            t[j] = t[j - 1];

        t[0] = add;
        }

    return elems;
}

//---------------------------------------------------------------------------
//  MLSequence::operator[]() --
//---------------------------------------------------------------------------

unsigned char MLSequence::operator[] (size_t i) const
{
    return mMls[i];
}

//---------------------------------------------------------------------------
//  MLSequence::numElements() --
//---------------------------------------------------------------------------

size_t MLSequence::numElements() const
{
    return mMls.size();
}

} // namespace
