// MappedDataSet_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <IniFile_Mb.h>
#include <MappedDataSet_Mb.h>
#include <FileSupport_Mb.h>
#include <Event_Mb.h>
#include <Windows.h>


using namespace std;

namespace Innovative
{

class Impl
{
    friend class MappedDataSet;

public:
    Impl(MappedDataSet & owner)
        : Owner(owner), hFile(INVALID_HANDLE_VALUE),
          hMapped(INVALID_HANDLE_VALUE)  {  }

    void Close()
        {
        // Close the mapping object and the file
        CloseHandle(hMapped);
        CloseHandle(hFile);

        hMapped = INVALID_HANDLE_VALUE;
        hFile = INVALID_HANDLE_VALUE;

       
       
        }

    void Create(const std::string & /*name*/)
        {
		Lock = new Event();
        }
    bool Acquire()
	{
		return Lock->Set();
	}
    void Release()
	{
		Lock->Clear();
	}

protected:
    // Data
    MappedDataSet &     Owner;
    HANDLE              hFile;
    HANDLE              hMapped;
    Event *             Lock;
};

//===========================================================================
// Class MappedDataSet
//===========================================================================

//---------------------------------------------------------------------------
// MappedDataSet::MappedDataSet() -- Constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

MappedDataSet::MappedDataSet()
    : Cache(0), Position(0), pImpl(new Impl(*this)),
      FAutoCreate(false), FSize(0)
{
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// MappedDataSet::~MappedDataSet() -- Destructor
//---------------------------------------------------------------------------

MappedDataSet::~MappedDataSet()
{
    delete pImpl;
    pImpl = 0;
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageOpen() -- Open disk file
//---------------------------------------------------------------------------

bool MappedDataSet::ImageOpen()
{
    PathSpec Spec;
    Spec.Full(FFileName);

    bool ro = false;//FReadOnly;

    // Try to open existing mapped file
    string name = "Mapped";
    for (size_t i = 0; i < Spec.Full().size(); ++i)
        if (isalnum(Spec.Full()[i]))
            name += tolower(Spec.Full()[i]);
    name += "Mapped";

    pImpl->hMapped = OpenFileMapping(FILE_MAP_ALL_ACCESS, false, name.c_str());
    pImpl->Create(name+string("lock"));

    Debug.Log("MappedDataSet::ImageOpen(): pImpl->hMapped", (size_t)pImpl->hMapped);

    // If that fails, we're the first, so create the map
    if (!pImpl->hMapped)
        {
        int Mode = ro ? FILE_SHARE_READ : FILE_SHARE_READ|FILE_SHARE_WRITE;
        int Access = ro ? GENERIC_READ : GENERIC_READ|GENERIC_WRITE;
        pImpl->hFile = CreateFileA(FFileName.c_str(), Access, Mode, NULL,
                           FAutoCreate ? OPEN_ALWAYS : OPEN_EXISTING,
                           FILE_FLAG_WRITE_THROUGH, NULL);
        if (pImpl->hFile == INVALID_HANDLE_VALUE)
            return false;

        DWORD dummy;
        unsigned int actual = GetFileSize(pImpl->hFile, &dummy);

        if (FAutoCreate || (!ro && (actual < FSize)))
            {
            Int64 Temp;
            Temp.All = FSize;
            Temp.Lo = SetFilePointer(pImpl->hFile, Temp.Lo, (long*)&Temp.Hi, FILE_BEGIN);
            SetEndOfFile(pImpl->hFile);
            Seek(0);
            }
        else
            {
            FSize = actual;
            }

        Debug.Log("MappedDataSet::ImageOpen()1: FSize", FSize);
        int Protect = ro ? PAGE_READONLY : PAGE_READWRITE;
        pImpl->hMapped = CreateFileMapping(pImpl->hFile, NULL, Protect,
            0, static_cast<unsigned int>(FSize), name.c_str());

        if (!pImpl->hMapped)
            {
            CloseHandle(pImpl->hFile);
            return false;
            }
        }

    int Access = ro ? FILE_MAP_READ : FILE_MAP_ALL_ACCESS;
    Cache = (char*)MapViewOfFile(pImpl->hMapped, Access, 0, 0, 0);

    if (!Cache)
        {
        CloseHandle(pImpl->hFile);
        CloseHandle(pImpl->hMapped);
        return false;
        }

    Init();

    ii64 * p = reinterpret_cast<ii64*>(&Cache[0]);
    // If created, zero-fill the mapped file image
    if (FAutoCreate && !ro)
        {
        std::fill (&Cache[0], &Cache[static_cast<int>(FSize)], 0xff);
        *p = FSize;
        }
    else
        {
        FSize = *p;
        pImpl->Release();
        }

    Debug.Log("MappedDataSet::ImageOpen()2: FSize", FSize);
    Seek(0);

    return true;
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageClose() -- Close disk file
//---------------------------------------------------------------------------

void MappedDataSet::ImageClose()
{
    if (!Cache)
        return;

    // Unmap the file (implicit flush to disk)
    UnmapViewOfFile(Cache);
    Cache=0;
    // Close the mapping object and the file
    pImpl->Close();
    Position = 0;
    Debug.Log("MappedDataSet::ImageOpen()3: FSize", FSize);
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageFetch() -- Read sample from disk file
//---------------------------------------------------------------------------

void MappedDataSet::ImageFetch(Subsample & sample)
{
    if (!Cache || (Position >= FSize))
        return;

    char * pSample = sample.asString();
    memcpy(pSample, &Cache[static_cast<unsigned int>(Position)], FSubsampleSize);
    CorrectEndian(pSample);
    sample.Type(static_cast<enum Subsample::SampleType>(FType));
    Position += FSubsampleSize;
    Progress(Position);
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageStore() -- Write sample to disk file
//---------------------------------------------------------------------------

void MappedDataSet::ImageStore(Subsample & sample)
{
    if (!Cache || FReadOnly || !pImpl->Acquire())
        return;

    // Write sample into the cache
    char * pSample = sample.asString();
    CorrectEndian(pSample);
    memcpy(&Cache[static_cast<unsigned int>(Position)], pSample, FSubsampleSize);
    Position += FSubsampleSize;
    Progress(Position);
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageFlush() -- Write dirty cache to disk
//---------------------------------------------------------------------------

void MappedDataSet::ImageFlush()
{
    if (!Cache)
        return;

    FlushViewOfFile(Cache, 0);
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageSkip() -- Skip forward n bytes into disk file
//---------------------------------------------------------------------------

void MappedDataSet::ImageSkip(ii64 offset)
{
    Position += offset;
    Progress(Position);
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageSeek() -- Seek to byte offset in disk file
//---------------------------------------------------------------------------

void MappedDataSet::ImageSeek(ii64 offset)
{
    Position = offset;
    Progress(Position);
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageSize() -- Return size of disk file
//---------------------------------------------------------------------------

ii64 MappedDataSet::ImageSize() const
{
    return FSize;
}

//---------------------------------------------------------------------------
// MappedDataSet::ImageSize() -- Set size of disk file
//---------------------------------------------------------------------------

void MappedDataSet::ImageSize(ii64 size)
{
    Debug.Log("MappedDataSet::ImageSize: ", size);
    FSize = size;
}

} // Namespace Innovative

//---------------------------------------------------------------------------
