// MatadorDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <MatadorDeviceMap_Mb.h>
#include <MatadorDeviceDriver_Mb.h>
#include <MatadorMemoryMap_Mb.h>
#include <StartStopThread_Mb.h>
#include <NotifyThread_Mb.h>
#include <Matador_Mb.h>
#include <SystemSupport_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <FpaLoadBase_Mb.h>
#include <SignalQueue_Mb.h>
#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>
#include <MemoryUtils_Mb.h>
#include <Exception_Mb.h>

#include <deque>
#include <algorithm>

using namespace std;

namespace Innovative
{
//
//   ...Masks for 8 BM Block streaming bus-master cards...
static const int BMUNDERFLOW_OUT_EVENT = 0x080;  // Underflow on Output to Chico
static const int BMOVERFLOW_IN_EVENT = 0x100;    // Overflow on Input from Chico
static const int BM_INCOMING_ACK_SHIFT = 24;     // Amount to shift ACK bits
static const int BM_OUTGOING_ACK_SHIFT = 16;     // Amount to shift ACK bits
static const int BM_NONBMINT_MASK = 0x679F;      // Int Mask w/o BM in and out

//const int   BM_NONBMINT_MASK1       = 0x67FF;   // Int Mask w/o BM in and out
//
//   ...Masks and Definitions common to all streaming bus-master cards...
static const int DSP_EVENT = 0x001;              // DSP INT4 just fired
static const int DSP_MAILBOX_DATA_EVENT = 0x002; // Mailbox In data available
static const int DSP_MAILBOX_ACK_EVENT = 0x004;  // Mailbox data read by DSP
static const int EXTERNAL_EVENT = 0x010;         // External interrupt just fired
static const int BMOUTGOING_EVENT = 0x020;       // BM Outgoing Data interrupt
static const int BMINCOMING_EVENT = 0x040;       // BM Incoming Data interrupt

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

//===========================================================================
//  CLASS FpaControlRegister
//===========================================================================

class FpaControlRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup            Mode;
    RegisterBit                 Clk;
    RegisterBit                 Out;
    ReadOnlyRegisterBit         In;
    ReadOnlyRegisterBitGroup    FirmwareRevision;
    ReadOnlyRegisterBitGroup    PcbRevision;
    ReadOnlyRegisterBitGroup    Id;

public:
    FpaControlRegister(AddressingSpace &space, int offset)
    :  inherited(space, offset),
       Mode(*this, 0, 3), Clk(*this, 3), Out(*this, 4), In(*this, 5),
       FirmwareRevision(*this, 8, 8), PcbRevision(*this, 16, 4),
       Id(*this, 20, 12)
       {  }
};

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

//==============================================================================
//  CLASS FpaLoader -- Interface to FPA register
//==============================================================================

class FpaLoader : public FpaLoadBase
{
public:
    FpaLoader(AddressingSpace & space)
        : Reg(space, Matador_MemoryMap::mmFpaReg)
        {  }

    // Methods
    // Read/Write data
    virtual bool            Read()
        {  return Reg.In ? 1 : 0;  }
    virtual void            Write(bool state)
        {  Reg.Out = state;  }
    // Read/write clock state
    virtual void            Clock(bool state)
        {  Reg.Clk = state;  }

    // Mode set
    virtual void            Mode(int mode)
        {  Reg.Mode = mode;  }
    // Device select
    virtual void            Device(int /*device*/)
        {  }
    // Firmware revision readback
    virtual int             FirmwareRevision()
        {  return Reg.FirmwareRevision;  }
    // Pcb revision readback
    virtual int             PcbRevision()
        {  return Reg.PcbRevision;  }
    // Id readback
    virtual int             Id()
        {  return Reg.Id;  }

private:
    // Data
    FpaControlRegister      Reg;
};

//---------------------------------------------------------------------------
typedef std::list<int>  InterruptStatusArray;

namespace BlockIntScheme
{
//===========================================================================
//  CLASS BmXmitThread  -- Transmission sync handling for PacketInt scheme
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class BmXmitThread
//---------------------------------------------------------------------------

class BmXmitThread : public NotifySemThread
{
    typedef NotifySemThread inherited;

public:
    BmXmitThread(const char * name = "")
        : inherited(name)
        {  Priority(tpHigher);  }
};

//===========================================================================
//  CLASS VirtualIsrThread  -- ISR handling thread for Packet Int scheme
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class VirtualIsrThread
//---------------------------------------------------------------------------

class VirtualIsrThread : public StartStopThread
{
    typedef StartStopThread inherited;
       
public:
    VirtualIsrThread( MatadorDeviceMap * device, Event * isr_event );

    //
    //  Events
    OpenWire::EventHandler<OpenWire::Event> OnXmitData;

protected:
    enum { sDriverEvent=inherited::sLast, sLocalIsrQ, sLast };

    Event &                    DriverEvent;

    MatadorDeviceMap *         BlkStreamDevice;
    SignalQueue<unsigned int>  LocalIsrQ;

    size_t  MaxInterruptDepth;
    size_t  MaxLocalIsrDepth;

    int mLastInBufferIndex;
    int mLastOutBufferIndex;

    virtual void Init();
    virtual void SingleExecution();

    void  DispatchInterrupt( int IntStatus );
    void  DispatchInputEventsFromInterrupt(int IntStatus);
    void  DispatchOutputEventsFromInterrupt(int IntStatus);

    bool  StreamConnected();

};

//---------------------------------------------------------------------------
//  housekeeping for class VirtualIsrThread
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class VirtualIsrThread
//---------------------------------------------------------------------------

VirtualIsrThread::VirtualIsrThread(MatadorDeviceMap * device, Event * isr_event)
    : inherited("BlockStream::VirtualIsrThread"),
      DriverEvent( *( isr_event ) ),
      BlkStreamDevice( device ), 
      MaxInterruptDepth(0), MaxLocalIsrDepth(0),
      mLastInBufferIndex( 0 ),
      mLastOutBufferIndex( 0 )
{
    Priority(tpTimeCritical);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::Init() -- Initialize sync objects
//---------------------------------------------------------------------------

void VirtualIsrThread::Init()
{
    inherited::Init();

    AddObject(DriverEvent, sDriverEvent);
    AddObject(LocalIsrQ.SignalSem(), sLocalIsrQ);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::StreamConnected() -- returns true if the streaming is connected
//---------------------------------------------------------------------------

bool  VirtualIsrThread::StreamConnected()
{
    return BlkStreamDevice->StreamConnected();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::SingleExecution() -- thread function
//---------------------------------------------------------------------------

void VirtualIsrThread::SingleExecution()
{
    int result = WaitFor();

    if ( Terminated() || eStopped.Check() )
        return;
    //
    //  A new interrupt was signalled. Since the low level queue is of
    //     finite size, we drain that queue as fast as we can. Since we
    //     also got at least one interrupt, we process one now.
    if ( result == sDriverEvent )
        {
        size_t depth = BlkStreamDevice->InterruptDepth();
        BlkStreamDevice->Log(0x94, static_cast<int>(depth));
        MaxInterruptDepth = std::max( depth, MaxInterruptDepth );
        //
        //  Drain the interrupt queue into LocalIsrQ
        if( ! StreamConnected() )
            LocalIsrQ.clear();
        
        while( depth-- )
            {
            if( ! StreamConnected() )
                break;

            int irqstatus = BlkStreamDevice->InterruptAck();
            LocalIsrQ.push( irqstatus );
            }
            
        size_t Idepth = LocalIsrQ.depth();
        BlkStreamDevice->Log(0x95, static_cast<int>(Idepth));
        MaxLocalIsrDepth = std::max( Idepth, MaxLocalIsrDepth );
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            //  since we did not wait for Sem before, must decrement sem manually
            //    when popping IRQ register value
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop(SignalQueue<unsigned int>::PopSem);
            DispatchInterrupt( IrqToProcess );
            }
        }
    //
    //  We were signalled because of queued local ISRs.
    if ( result == sLocalIsrQ )
        {
        int Idepth = static_cast<int>(LocalIsrQ.depth());
        BlkStreamDevice->Log(0x9a, Idepth);
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop();  // don't dec sem, already done
            DispatchInterrupt( IrqToProcess );
            }
        }
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::DispatchInterrupt() -- process interrupt
//---------------------------------------------------------------------------

void  VirtualIsrThread::DispatchInterrupt( int IntStatus )
{
    BlkStreamDevice->Log(4, IntStatus);
    if (IntStatus & DSP_MAILBOX_ACK_EVENT)
        {
        OpenWire::Event event;
        BlkStreamDevice->OnMailboxAck.Execute(event);
        }

    if (IntStatus & DSP_MAILBOX_DATA_EVENT)
        {
        OpenWire::Event event;
        BlkStreamDevice->OnMailboxData.Execute(event);
        }

    if (IntStatus & EXTERNAL_EVENT)
        {
        OpenWire::Event event;
        BlkStreamDevice->OnExternal.Execute(event);
        }

    if (IntStatus & DSP_EVENT)
        {
        OpenWire::Event event;
        BlkStreamDevice->OnDsp.Execute(event);
        }

    if (IntStatus & BMUNDERFLOW_OUT_EVENT)
        {
        OpenWire::Event event;
        BlkStreamDevice->OnBmUnderflowOut.Execute(event);
        }

    if (IntStatus & BMOVERFLOW_IN_EVENT)
        {
        OpenWire::Event event;
        BlkStreamDevice->OnBmOverflowIn.Execute(event);
        }

    if (IntStatus & BMOUTGOING_EVENT)
        DispatchOutputEventsFromInterrupt(IntStatus);

    if (IntStatus & BMINCOMING_EVENT)
        DispatchInputEventsFromInterrupt(IntStatus);

}

//---------------------------------------------------------------------------
//  VirtualIsrThread::DispatchInputEventsFromInterrupt() --
//------------------------------------------------------------------------------

void VirtualIsrThread::DispatchInputEventsFromInterrupt(int IntStatus)
{
    //
    //  Isolate Bit Flags
    int bits = (IntStatus >> BM_INCOMING_ACK_SHIFT) & 0xFF;
    bits |= (IntStatus >> (BM_INCOMING_ACK_SHIFT - 8)) & 0xFF00;

    int BufferIndex = mLastInBufferIndex;
    bits >>= BufferIndex;

    // Find the first 0 bit.
    for (int i = 0; i < 8; i++, BufferIndex++, bits >>= 1)
        {
        if (!(bits & 1))
            break;
        }
    //
    //  Find first 1 bit
    for (int i = 0; i < 8; i++, BufferIndex++, bits >>= 1)
        {
        if (bits & 1)
            break;
        }
    //
    //  Process until the last 1 bit
    for (int i = 0; i < 8; i++, BufferIndex++, bits >>= 1)
        {
        if (!(bits & 1))
            break;

        BufferIndex %= 8;
        mLastInBufferIndex = BufferIndex;

        BufferIndexEvent event(mLastInBufferIndex);
        BlkStreamDevice->OnBmIncoming.Execute(event);
        }

}

//---------------------------------------------------------------------------
//  VirtualIsrThread::DispatchOutputEventsFromInterrupt() --
//------------------------------------------------------------------------------
 
void VirtualIsrThread::DispatchOutputEventsFromInterrupt(int IntStatus)
{
    //
    //  Isolate Bit Flags
    int bits = (IntStatus >> BM_OUTGOING_ACK_SHIFT) & 0xFF;
    bits |= (IntStatus >> (BM_OUTGOING_ACK_SHIFT - 8)) & 0xFF00;

    int BufferIndex = mLastOutBufferIndex;
    bits >>= BufferIndex;

    // Find the first 0 bit.
    for (int i = 0; i < 8; i++, BufferIndex++, bits >>= 1)
        {
        if (!(bits & 1))
            break;
        }
    //
    //  Find first 1 bit
    for (int i = 0; i < 8; i++, BufferIndex++, bits >>= 1)
        {
        if (bits & 1)
            break;
        }
    //
    //  Process until the last 1 bit
    for (int i = 0; i < 8; i++, BufferIndex++, bits >>= 1)
        {
        if (!(bits & 1))
            break;

        BufferIndex %= 8;
        mLastOutBufferIndex = BufferIndex;

        BufferIndexEvent event(mLastOutBufferIndex);
        BlkStreamDevice->OnBmOutgoing.Execute(event);
        }

}


}  // end namespace Innovative::BlockIntScheme

//===========================================================================
//  CLASS MatadorDeviceMap  -- Registers for P25 Device
//===========================================================================
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

using namespace Matador_MemoryMap;

//  Locations of the HPI registers
static HpiEngineAddresses  MatadorHpiAddresses =
    {
    mmHpiAddressLswReg,  mmHpiAddressMswReg,  //  HPI Address Register    Lsw, Msw
    mmHpiDataLswReg,     mmHpiDataMswReg,     //  HPI Data Register       Lsw, Msw
    mmHpiDataIncrLswReg, mmHpiDataIncrMswReg, //  HPI Data Incr Register  Lsw, Msw
    mmHpiControlLswReg,  mmHpiControlMswReg   //  HPI Control Register    Lsw, Msw
    };

//  Values of the HPI registers
static HpiCeSpaceValues  MatadorHpiCeValues =
    {
    0x00003078,         //  EMIF global control register
    0x00000040,         //  CE1
    0x10910321,         //  CE0
    0x00000030,         //  CE2
    0x00000040,         //  CE3
    0x00000465,         //  SDRAM Refresh Timing register
    0x000544a7,         //  SDRAM Extension
    0x53225000          //  SDRAM control register
    };

//---------------------------------------------------------------------------
//  constructor for class MatadorDeviceMap
//---------------------------------------------------------------------------

MatadorDeviceMap::MatadorDeviceMap() :
    IUsesRestrictedHpiCpu(*this, MatadorHpiAddresses, MatadorHpiCeValues),
    FFpa(new FpaLoader(mPorts)),
    mDeviceDriver(new MatadorDeviceDriver), 
    Visr(NULL),
    BmX(NULL),
    mMaxInterruptDepth( 0 ),
    mControlRegister( mPorts, mmControlStatusReg ),
    mControlStatusRegister(mPorts, mmControlStatusReg ),
    MessageClearReg( mPorts, mmMailboxClearReg),
    MessageCommandReg( mPorts, mmMailboxCmdReg),
    InterruptStatusReg( mPorts, mmInterruptStatus ),
    LogicConfig( mPorts, mmLogicConfig ),
    MailboxRam( mPorts, mmMailboxRamBase, 16 ),
    BusMasterReadAddressReg( mPorts, mmBmReadAddressReg),
    BusMasterWriteAddressReg( mPorts, mmBmWriteAddressReg),
    BusMasterRegionSizeReg( mPorts, mmBmRegionSizeReg),
    PciInterruptEnableReg( mPorts, mmPciInterruptEnable),
    InputBuffers( 8 ),
    OutputBuffers( 8 ),
    DebugLog(10000)
{
    EepromLoader = new FpaLoadThread(*FFpa);

    mSlowHpi = false;
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif
//---------------------------------------------------------------------------
// MatadorDeviceMap::~MatadorDeviceMap() --
//---------------------------------------------------------------------------

MatadorDeviceMap::~MatadorDeviceMap()
{
    EepromLoader->Shutdown();
    delete EepromLoader;
    EepromLoader = 0;

    delete FFpa;
    FFpa = 0;

    Close();
    delete mDeviceDriver;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::MessageAck() --
//---------------------------------------------------------------------------
 
void MatadorDeviceMap::MessageAck()
{
    MessageClearReg = 1;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::BoardCount() --  Number of enumerated boards
//---------------------------------------------------------------------------

size_t MatadorDeviceMap::BoardCount()
{
    return mDeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Open() --
//---------------------------------------------------------------------------

void MatadorDeviceMap::Open()
{
    if (mDeviceDriver->Opened())
        return;

    mDeviceDriver->Open();
    mPorts.Link(mDeviceDriver->PortMemoryAddress());
    Hpi().Link(mDeviceDriver->PortMemoryAddress());
    RealignBusMasterMemory();

    ClearControlRegister();
    LaunchVISR();
    EepromLoader->Initialize();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::RealignBusMasterMemory() --
//---------------------------------------------------------------------------

void MatadorDeviceMap::RealignBusMasterMemory()
{
    InputBuffers.Address(mDeviceDriver->IncomingBusMasterAddress());
    OutputBuffers.Address(mDeviceDriver->OutgoingBusMasterAddress());
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::BusMasterSizeWords() --
//---------------------------------------------------------------------------

void MatadorDeviceMap::BusMasterSizeWords(unsigned int value)
{
    mDeviceDriver->BusMasterSizeWords(value);
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::IncomingBusMasterSizeWords() --
//---------------------------------------------------------------------------

unsigned int MatadorDeviceMap::IncomingBusMasterSizeWords() const
{
    return mDeviceDriver->IncomingBusMasterSizeWords();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::OutgoingBusMasterSizeWords() --
//---------------------------------------------------------------------------

unsigned int MatadorDeviceMap::OutgoingBusMasterSizeWords() const
{
    return mDeviceDriver->OutgoingBusMasterSizeWords();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::SetBusMasterSize() --
//---------------------------------------------------------------------------

void MatadorDeviceMap::SetBusMasterSize(int size)
{
    InputBuffers.Size(size);    // Matadors only support one size for both
    OutputBuffers.Size(size);

    if ((unsigned int)InputBuffers.FullSize() > IncomingBusMasterSizeWords())
        {
        Error("Block Size too large for Incoming Busmaster Region");
        }
    if ((unsigned int)OutputBuffers.FullSize() > OutgoingBusMasterSizeWords())
        {
        Error("Block Size too large for Outgoing Busmaster Region");
        }

    RealignBusMasterMemory();
    BusMasterInit(mDeviceDriver->OutgoingBusMasterPhysicalAddress(),
                  mDeviceDriver->IncomingBusMasterPhysicalAddress(),
                  InputBuffers.FullSize());
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Close() --
//---------------------------------------------------------------------------
 
void MatadorDeviceMap::Close()
{
    CloseVISR();
    mDeviceDriver->Close();
} 

//---------------------------------------------------------------------------
//  MatadorDeviceMap::ClearControlRegister() --
//---------------------------------------------------------------------------
 
void MatadorDeviceMap::ClearControlRegister()
{

    //  mControlRegister.BaseboardReset.Set();
    mControlRegister.Reset.Set();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Led() --
//------------------------------------------------------------------------------
 
bool MatadorDeviceMap::Led() const
{
    return !mControlRegister.Led;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Led() --
//------------------------------------------------------------------------------

void MatadorDeviceMap::Led(bool value)
{
    mControlRegister.Led = !value;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Opened() --
//------------------------------------------------------------------------------

bool MatadorDeviceMap::Opened() const
{
	return mDeviceDriver->Opened();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Opened() --
//------------------------------------------------------------------------------

void MatadorDeviceMap::Opened( bool value )
{
	if( value )
		Open();
	else
		Close();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Target() --
//------------------------------------------------------------------------------

int  MatadorDeviceMap::Target() const
{
    return mDeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Target() --
//------------------------------------------------------------------------------

void  MatadorDeviceMap::Target(int target)
{
    mDeviceDriver->Target(target);
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Reset() --
//------------------------------------------------------------------------------

void MatadorDeviceMap::Reset()
{
    mControlRegister.BaseboardReset.Clear();
    Innovative::Sleep(20);
    mControlRegister.BaseboardReset.Set();
}

//---------------------------------------------------------------------------
// MatadorDeviceMap::ResetBaseBoard() -- Resets baseboard during HPI load
//------------------------------------------------------------------------------

void MatadorDeviceMap::ResetBaseBoard()
{
	DspInReset(true);
	Innovative::Sleep(10);
	DspInReset(false);

    ClearInterruptStatusQueue();
    IncomingCredit.Reset();
    OutgoingCredit.Reset(true);

    //
    //  Re-apply hardware
    PciInterruptEnableReg.Apply();
}

//------------------------------------------------------------------------------
//  MatadorDeviceMap::DspInReset() --
//------------------------------------------------------------------------------

bool MatadorDeviceMap::DspInReset()
{
    return !mControlRegister.Reset;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::DspInReset() --
//------------------------------------------------------------------------------

void MatadorDeviceMap::DspInReset(bool value)
{
    mControlRegister.Reset = !value;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::DspInterrupt() --
//------------------------------------------------------------------------------
 
void MatadorDeviceMap::DspInterrupt()
{
    mControlRegister.SequencerInterrupt.Set();
    mControlRegister.SequencerInterrupt.Clear();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::InterruptDepth() --
//------------------------------------------------------------------------------

size_t MatadorDeviceMap::InterruptDepth()
{
    return mDeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::InterruptAck() --
//------------------------------------------------------------------------------
 
int MatadorDeviceMap::InterruptAck()
{
    return mDeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  *MatadorDeviceMap::GetEvent() --
//------------------------------------------------------------------------------
 
Event *MatadorDeviceMap::GetEvent()
{
    return mDeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::LaunchVISR() --
//------------------------------------------------------------------------------
 
bool MatadorDeviceMap::LaunchVISR()
{
    //
    //  Create Busmaster Xmit thread.
    BmX = new BlockIntScheme::BmXmitThread("PacketIntScheme::BmXmitThread");
    BmX->OnNotify.SetEvent( this, &MatadorDeviceMap::BmXmitSignalled );
    BmX->Resume();

    //
    //  Create V-ISR thread.
    Visr = new BlockIntScheme::VirtualIsrThread(this, GetEvent());
    Visr->OnXmitData.SetEvent( this, &MatadorDeviceMap::LaunchXmitData );

    //ConfigureBmRegion();
    Visr->Resume();

    //
    //  Start V-ISR thread.
    Visr->Start();
    return true;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::CloseVISR() --
//------------------------------------------------------------------------------

bool MatadorDeviceMap::CloseVISR()
{
     if ( BmX )
        {
        BmX->Stop();
        BmX->Shutdown();
        delete BmX;
        BmX = NULL;
        }
    //
    //  Have VISR Thread shut itself down
    if ( Visr )
        {
        Visr->Stop();
        Visr->Shutdown();
        delete Visr;
        Visr = NULL;
        }

    return true;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::ConfigureBusmastering()
//---------------------------------------------------------------------------

void  MatadorDeviceMap::ConfigureBusmastering()
{
   // SetBusMasterSize(int size)
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::TransmitDataAvailable() -- Signal Xmit thread to process
//---------------------------------------------------------------------------

void  MatadorDeviceMap::TransmitDataAvailable()
{
    Log(0x30,0);
    BmX->Notify();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::LaunchXmitData() -- Signal Xmit thread to process
//---------------------------------------------------------------------------

void  MatadorDeviceMap::LaunchXmitData( OpenWire::Event & )
{
    Log(0x31,0);
    BmX->Notify();
}

//------------------------------------------------------------------------------
//  MatadorDeviceMap::BmXmitSignalled() -- Transmission requested...
//------------------------------------------------------------------------------

void  MatadorDeviceMap::BmXmitSignalled( OpenWire::Event & )
{
    int depth = static_cast<int>(BlkStreamManager->XmitQDepth());
    Log(0x32, depth);

    if (depth == 0)
        return;
        
    Log(0x50, 0);
    Buffer Packet = BlkStreamManager->NextXmitPacket();    
    //
    //  Do we have room to insert the packet?
    //  ...if not, return false
    bool available = OutgoingCredit.IsSectionAvailable(OutgoingCredit.BlockIndex());
    if (!available)
        {
        Log(0x5f, 0xFF);
        return;
        }
        
    int Index = static_cast<int>(OutgoingCredit.BlockIndex());
    //
    //  Copy Data Out
    CopyFromBlockToBusmaster(Index, Packet.ConstData());
    OutgoingCredit.SectionInUse(Index);
    OutgoingCredit.Next();
    //
    //  Grant Credit
    CreditOutgoingStream(Index);
    //
    //  Remove data from queue and grant credit
    BlkStreamManager->DequeueXmitPacket();
    Log(0x5f, 0);
        
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::ProcessIncomingPacket() --
//---------------------------------------------------------------------------

void MatadorDeviceMap::ProcessIncomingPacket(size_t buffer_index)
{
    Log(0x70, static_cast<int>(buffer_index));
    //
    //  Mark Section as "Uncredited"
    IncomingCredit.SectionReleased(buffer_index);
            
    if ( ! StreamConnected() )
        return;
        
    Buffer BlockBuffer(BlkStreamManager->BlockSize());
    //
    //  Copy Data Into Buffer
    CopyToBlockFromBusmaster(static_cast<int>(buffer_index), BlockBuffer.Data());
 
    BufferHeaderDatagram bufferHdr(BlockBuffer);   // save index in buffer
    bufferHdr[0] = static_cast<int>(buffer_index);

    if( BlkStreamManager )
        BlkStreamManager->EnqueueRecvPacket(BlockBuffer);
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::CopyFromBlockToBusmaster() --
//---------------------------------------------------------------------------

void MatadorDeviceMap::CopyFromBlockToBusmaster(int buffer_index, const int * buffer)
{
    // Copy. src, dst, length.
    CopyInt32Buffer(reinterpret_cast<const int *>(buffer),
                    reinterpret_cast<int *>(OutputBuffers[buffer_index].Address()),
                    OutputBuffers[buffer_index].Size()
                    );
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::CopyToBlockFromBusmaster() --
//---------------------------------------------------------------------------
 
void MatadorDeviceMap::CopyToBlockFromBusmaster(int buffer_index, int *  buffer)
{
    // Copy. src, dst, length.
    CopyInt32Buffer(reinterpret_cast<const int *>(InputBuffers[buffer_index].Address()), 
                    reinterpret_cast<int *>(buffer), 
                    InputBuffers[buffer_index].Size()
                    );
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::CreditOutgoingStream() --
//---------------------------------------------------------------------------
 
void MatadorDeviceMap::CreditOutgoingStream(size_t buffer_index)
{
    //
    //  Release by writing the Idx bit to the Interrupt Status Register
    int outval = (1 << buffer_index) << BM_OUTGOING_ACK_SHIFT;
    InterruptStatusReg = outval;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::CreditIncomingStream() --
//------------------------------------------------------------------------------
 
void MatadorDeviceMap::CreditIncomingStream(size_t buffer_index)
{
    //
    //  Release by writing the Idx bit to the Interrupt Status Register
    int outval = (1 << buffer_index) << BM_INCOMING_ACK_SHIFT;
    InterruptStatusReg = outval;
    
    IncomingCredit.SectionInUse(buffer_index);
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::ClearInterruptStatusQueue() --
//------------------------------------------------------------------------------
 
void MatadorDeviceMap::ClearInterruptStatusQueue()
{
    mDeviceDriver->ClearInterruptStatusQueue();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::PreloadXmitBusmasterRegion()
//---------------------------------------------------------------------------

void  MatadorDeviceMap::PreloadXmitBusmasterRegion()
{
    //  Signal 1 buffer
    TransmitDataAvailable();
    //  Start Xfer thread
    BmX->Start();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::GrantCredit()
//---------------------------------------------------------------------------

void  MatadorDeviceMap::GrantCredit(int direction, size_t code)
{
    if (direction)
        CreditIncomingStream(code);  // INPUT
    else
        CreditOutgoingStream(code);  // INPUT
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::BusMasterInit() --
//------------------------------------------------------------------------------
 
void MatadorDeviceMap::BusMasterInit(ii32*  read_address, ii32*  write_address,
                                     int region_size_wds)
{
    size_t read_addr = reinterpret_cast<size_t>(read_address);
    BusMasterReadAddressReg = static_cast<int>(read_addr);
    size_t write_addr = reinterpret_cast<size_t>(write_address);
    BusMasterWriteAddressReg = static_cast<int>(write_addr);

    //
    //  Define Busmaster Region Size and Locations
    int size_code = New_BusMasterRegionCode(region_size_wds);

    BusMasterRegionSizeReg = size_code;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::New_BusMasterRegionCode() --
//------------------------------------------------------------------------------
 
int MatadorDeviceMap::New_BusMasterRegionCode(int region_size_wds)
{
    if (region_size_wds <= 256)
        return 2;
    if (region_size_wds <= 512)
        return 3;
    if (region_size_wds <= 1 * 1024)
        return 4;
    if (region_size_wds <= 2 * 1024)
        return 5;
    if (region_size_wds <= 4 * 1024)
        return 6;
    if (region_size_wds <= 8 * 1024)
        return 7;
    if (region_size_wds <= 16 * 1024)
        return 8;
    if (region_size_wds <= 32 * 1024)
        return 9;
    if (region_size_wds <= 64 * 1024)
        return 10;
    if (region_size_wds <= 128 * 1024)
        return 11;
    if (region_size_wds <= 256 * 1024)
        return 12;
    if (region_size_wds <= 512 * 1024)
        return 13;
    if (region_size_wds <= 1024 * 1024)
        return 14;
    else
        return 15;
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::StartSequencer() --
//------------------------------------------------------------------------------
 
void MatadorDeviceMap::StartSequencer()
{
    //
    //  Begin the Busmastering
    mControlRegister.AllowBusmaster.Set();

    //
    //  Begin the sequencer
    //  Do NOT start sequencer until initial busmaster has
    //     completed! (Started with StartBusmastering)
    for (int i = 0; i < 1000 && mControlStatusRegister.BusMasterInProgress; i++)
        Sleep(1);

    if (mControlStatusRegister.BusMasterInProgress)
        Error("Sequencer start timeout");

    //        throw EChicoSupport("Sequencer start timeout",
    //                            EChicoSupport::SequencerTimeoutErr);
    mControlRegister.SequencerEnable.Set();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::StopSequencer() --
//------------------------------------------------------------------------------

void MatadorDeviceMap::StopSequencer()
{
    //
    //  Shut down the sequencer
    mControlRegister.SequencerEnable.Clear();

    //
    //  Wait for the sequencer to finish
    for (int i = 0; i < 10000 && mControlStatusRegister.BusMasterPending; i++)
        Sleep(1);

    if (mControlStatusRegister.BusMasterPending)
        Error("Sequencer stop timeout");

    //        throw EChicoSupport("Sequencer stop timeout",
    //                            EChicoSupport::SequencerTimeoutErr);
    //  Shut down the Busmastering
    mControlRegister.AllowBusmaster.Clear();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::KpQueueDepth() --  Return depth of KP ISR queue
//------------------------------------------------------------------------------

size_t MatadorDeviceMap::KpQueueDepth()
{
    return mDeviceDriver->KpQueueDepth();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::Error() --
//------------------------------------------------------------------------------

void MatadorDeviceMap::Error(const char*  message)
{
    throw Innovative::MalibuException(message);
}
 
//---------------------------------------------------------------------------
//  MatadorDeviceMap::DeviceInfo() --
//---------------------------------------------------------------------------

InnovativeKernel::DeviceEntry & MatadorDeviceMap::DeviceInfo()
{
    return mDeviceDriver->PciConfig();
}

//---------------------------------------------------------------------------
//  MatadorDeviceMap::DumpLog() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  MatadorDeviceMap::DumpLog(std::vector<int> & vec)
{
	DebugLog.copy_to(vec);
}


//===========================================================================
//  CLASS ControlRegister  --  
//===========================================================================
 
//---------------------------------------------------------------------------
// MatadorDeviceMap::ControlRegister::ControlRegister() --
//------------------------------------------------------------------------------
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)
#endif
MatadorDeviceMap::ControlRegister::ControlRegister(AddressingSpace&  space,
    int offset)
    : ShadowRegister(space, offset), AllowBusmaster(*this, 0), Led(*this, 1), 
      SequencerInterrupt(*this, 2), BaseboardReset(*this, 3), 
      SequencerEnable(*this, 4), Reset(*this, 5)
{
}

//---------------------------------------------------------------------------
// MatadorDeviceMap::InterruptEnableRegister::InterruptEnableRegister() --
//------------------------------------------------------------------------------

MatadorDeviceMap::InterruptEnableRegister::InterruptEnableRegister(AddressingSpace&  space,
    int offset) 
    : ShadowRegister(space, offset), DspMailboxAck(*this, 1), 
      DspMailboxData(*this, 2), External(*this, 3), Read(*this, 4), 
      Write(*this, 5), Dsp(*this, 6)
{
}

//===========================================================================
//  CLASS ControlStatusRegister  --  
//===========================================================================
 
//---------------------------------------------------------------------------
// MatadorDeviceMap::ControlStatusRegister::ControlStatusRegister() --
//------------------------------------------------------------------------------
 
MatadorDeviceMap::ControlStatusRegister::ControlStatusRegister(AddressingSpace&  space,
    int offset) 
    : Register(space, offset), BusMasterInProgress(*this, 0), 
      BusMasterPending(*this, 1), OutBusMasterFlags(*this, BM_OUTGOING_ACK_SHIFT, 8), 
      InBusMasterFlags(*this, BM_INCOMING_ACK_SHIFT, 8)
{
}
#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif
} // Innovative

