// MatadorDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef MatadorDeviceMap_MbH
#define MatadorDeviceMap_MbH

#include <HardwareRegister_Mb.h>
#include <Event_Mb.h>
#include <ContiguousMemoryBlocks_Mb.h>
#include <BaseboardCpuInterfaces_Mb.h>
#include <ThreadSafeLog_Mb.h>
#include <StreamSupport_Mb.h>

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS BufferIndexEvent  --
//===========================================================================

class BufferIndexEvent : public OpenWire::Event
{
public:
  const int Index;
  bool  BufferProcessed;

public:
  BufferIndexEvent( int index ) : Index( index ), BufferProcessed( false )
    {
    }
};
#ifdef __CLR_VER
#pragma managed(pop)
#endif


namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class MatadorDeviceDriver;
class Event;
class NotifyThread;
class MatadorDeviceMap;

namespace BlockIntScheme
{
    class VirtualIsrThread;
    class BmXmitThread;

//===========================================================================
//  CLASS BlockCreditManager -- Manage Busmaster Block Availablity
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class BlockCreditManager
//---------------------------------------------------------------------------

class BlockCreditManager 
{
public:
    BlockCreditManager()
        :  SectionAvailable(8), NextPos(0)
            {}

    void  Reset(bool all_available = false)
        {
        NextPos = 0;
        for (size_t idx=0; idx<SectionAvailable.size(); idx++)
            SectionAvailable[idx] = all_available;  
        } 
               
    bool  IsSectionAvailable(size_t index)
        {  return SectionAvailable[index];  }
    void  SectionInUse(size_t index)
        {  SectionAvailable[index] = false;  }
    void  SectionReleased(size_t index)
        {  SectionAvailable[index] = true;  }
           
    size_t  BlockIndex()
        {   return NextPos;     }
        
    void  Next()
        {  NextPos++; if (NextPos>=8) NextPos = 0;  }
 
    std::vector<bool> SectionAvailable;
    int               NextPos;
};


}


//===========================================================================
//  CLASS HpiRegister  --  
//===========================================================================

class MatadorDeviceMap : public IUsesRestrictedHpiCpu,
						 public IUsesHpiControl

{
    friend class Matador;
    
public:
    // Ctor
    MatadorDeviceMap();
    virtual ~MatadorDeviceMap();

    // Methods
    size_t          BoardCount();
    void            Open();
    void            Close();
    void            DspInReset(bool value);
    bool            DspInReset();
    void Reset();
    void MessageAck();
    void DspInterrupt();
    void BusMasterInit( ii32* read_address, ii32* write_address, int region_size_wds );
    void StartSequencer();
    void StopSequencer();
    size_t    KpQueueDepth();
    InnovativeKernel::DeviceEntry &   DeviceInfo();

    void  RegisterStreamPlugIn(StreamManager*  plug_in)
            {  BlkStreamManager = plug_in;  }
    bool  StreamConnected()
            {  
            if (BlkStreamManager)
                return true;
            else
                return false;
            }              

    // Properties
    void  Led( bool value );
    bool  Led() const;

    void  Opened( bool value );
    bool  Opened() const;

    void  Target( int target );
    int   Target() const;

    // Events
    OpenWire::EventHandler<OpenWire::Event> OnDsp;
    OpenWire::EventHandler<OpenWire::Event> OnExternal;
    OpenWire::EventHandler<BufferIndexEvent> OnBmOutgoing;
    OpenWire::EventHandler<BufferIndexEvent> OnBmIncoming;
    OpenWire::EventHandler<OpenWire::Event> OnMailboxAck;
    OpenWire::EventHandler<OpenWire::Event> OnMailboxData;
    OpenWire::EventHandler<OpenWire::Event> OnBmUnderflowOut;
    OpenWire::EventHandler<OpenWire::Event> OnBmOverflowIn;

protected:
    // Registers
    class ControlRegister : public ShadowRegister
    {
    public:
        RegisterBit       AllowBusmaster;
        RegisterBit       Led;
        RegisterBit       SequencerInterrupt;
        RegisterBit       BaseboardReset;
        RegisterBit       SequencerEnable;
        RegisterBit       Reset;

        ControlRegister( AddressingSpace &space, int offset );
    };

    class ControlStatusRegister : public Register
    {
    public:
        RegisterBit        BusMasterInProgress;
        RegisterBit        BusMasterPending;
        RegisterBitGroup   OutBusMasterFlags;
        RegisterBitGroup   InBusMasterFlags;

        ControlStatusRegister( AddressingSpace &space, int offset );
    };

    class InterruptEnableRegister : public ShadowRegister
    {
    public:
        RegisterBit       DspMailboxAck;
        RegisterBit       DspMailboxData;
        RegisterBit       External;
        RegisterBit       Read;
        RegisterBit       Write;
        RegisterBit       Dsp;

        InterruptEnableRegister( AddressingSpace &space, int offset );
    };

    // Data
    // FPA Eeprom programmer
    class FpaLoadThread *   EepromLoader;
    class FpaLoader *       FFpa;
    MatadorDeviceDriver *   mDeviceDriver;
    StreamManager *	        BlkStreamManager;
    //
    //  Member Data
    BlockIntScheme::VirtualIsrThread *  Visr;
    BlockIntScheme::BmXmitThread *      BmX;
    BlockIntScheme::BlockCreditManager   IncomingCredit;
    BlockIntScheme::BlockCreditManager   OutgoingCredit;
    
    bool mSlowHpi;
    size_t mMaxInterruptDepth;

    AddressingSpace               mPorts;
    ControlRegister               mControlRegister;
    ControlStatusRegister         mControlStatusRegister;

    // Methods
    void ClearControlRegister();
    Event *GetEvent();
    
public:
    size_t  InterruptDepth();
    int     InterruptAck();
private:
    int   New_BusMasterRegionCode( int region_size_wds );
    void  Error(const char *message );
    void  RealignBusMasterMemory();

    void  LaunchXmitData( OpenWire::Event & evt);
    void  BmXmitSignalled( OpenWire::Event & evt);

	virtual void ResetBaseBoard();

public: // Psuedo-public - not for general use
    // Data
    Register           MessageClearReg;
    Register           MessageCommandReg;
    Register           InterruptStatusReg;
    ShadowRegister     LogicConfig;
    RegisterArray<Register>  MailboxRam;
    Register           BusMasterReadAddressReg;
    Register           BusMasterWriteAddressReg;
    Register           BusMasterRegionSizeReg;

    InterruptEnableRegister PciInterruptEnableReg;

    ContiguousMemoryBlocks   InputBuffers;
    ContiguousMemoryBlocks   OutputBuffers;

    // Methods
    bool  LaunchVISR();
    bool  CloseVISR();
    void  ClearInterruptStatusQueue();

    void  CreditOutgoingStream(size_t buffer_index);
    void  CreditIncomingStream(size_t buffer_index);
    void  TransmitDataAvailable();
    void  ProcessIncomingPacket(size_t buffer_index);
    void  CopyFromBlockToBusmaster(int buffer_index, const int * buffer);
    void  CopyToBlockFromBusmaster(int buffer_index, int * buffer);
    void  ConfigureBusmastering();
    void  PreloadXmitBusmasterRegion();
    void  GrantCredit(int direction, size_t code);
        
    // Specify driver-allocated BM region, prior to open
    virtual void            BusMasterSizeWords(unsigned int value);
    virtual unsigned int    IncomingBusMasterSizeWords() const;
    virtual unsigned int    OutgoingBusMasterSizeWords() const;
    // Specify portion of BM region to actually use
    virtual void            SetBusMasterSize( int size );
protected:
    bool            FDownloadVerified;
	ThreadSafeLog   DebugLog;

public:
	//  Debug
	void  Log(int code, int data) {  DebugLog.append(code, data);  };
	void  DumpLog(std::vector<int> & vec);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
