// MatadorMessageStream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//
 
#include <MatadorMessageStream_Mb.h>
#include <MatadorMessage_Mb.h>
#include <Semaphore_Mb.h>
#include <Thread_Mb.h>
#include <string>
#include <list>
#include <Exception_Mb.h>


namespace Innovative
{

const int       MaxMessageQueueSize = 1000;
const int       kDebugMsg = 0xdeb;

//===========================================================================
//  CLASS MatadorMessageThread  --  
//===========================================================================

class MatadorMessageThread : public Thread
{
    typedef Thread inherited;

private:
    static MatadorMessageThread*    mInstance;
    static int                      mReffCount;

private:
    std::list < MatadorMessageStream * > mBoards;
    CriticalSection mBoardsListSection;

    enum { sGo=inherited::sLast, sLast };
    Semaphore		eGo;

private:
    void    IntRegister(MatadorMessageStream *board);
    void    IntUnregister(MatadorMessageStream *board);

private:
    MatadorMessageThread();

public:
    static MatadorMessageThread* Register(MatadorMessageStream*  board)
    {
        if (!mInstance)
            {
			mInstance = new MatadorMessageThread();
            mInstance->Resume();
            }

        mReffCount++;
        mInstance->IntRegister(board);
        return mInstance;
    }

    static void Unregister(MatadorMessageStream*  board)
    {
        mInstance->IntUnregister(board);
        if (!(--mReffCount))
            {
            mInstance->Shutdown();
            delete mInstance;
            mInstance = NULL;
            }
    }

public:
    void    Activate();

protected:
    virtual void    Init();
    virtual void    Execute();

protected:
    bool    HandleProcessMessages();

};

MatadorMessageThread *MatadorMessageThread::mInstance = NULL;
int MatadorMessageThread::mReffCount = 0;

//===========================================================================
//  CLASS MatadorMessageStream  --  
//===========================================================================

MatadorMessageStream::MatadorMessageStream() :
    mMessageDevice(NULL),
    mTransmitBusy( false )
{
}
 
//---------------------------------------------------------------------------
// MatadorMessageStream::~MatadorMessageStream() --
//---------------------------------------------------------------------------
 
MatadorMessageStream::~MatadorMessageStream()
{
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::MessageTransmitted() --
//---------------------------------------------------------------------------
 
bool MatadorMessageStream::MessageTransmitted() // Returns true if the buffer is handled.
{
    mTransmitBusy = false;
    mMessageThread->Activate();
    return true;
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::MessageReceived() --
//---------------------------------------------------------------------------
 
bool MatadorMessageStream::MessageReceived() // Returns true if the buffer is handled.
{
    mMessageThread->Activate();
    return true;
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::AttachTo() --
//---------------------------------------------------------------------------
 
void MatadorMessageStream::AttachToPtr( MatadorMessageDevice *device )
{
	if( device )
		{
		if( mMessageDevice )
			mMessageDevice->RegisterMessagePlugIn(NULL);

		mMessageDevice = device;
		mMessageDevice->RegisterMessagePlugIn(this);

		mTransmitMessageQueue = new MatadorMessageQueue(MaxMessageQueueSize);
		mMessageThread = MatadorMessageThread::Register(this);
		}
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::Detach() --
//---------------------------------------------------------------------------

void MatadorMessageStream::Detach()
{
	MatadorMessageThread::Unregister(this);
	delete mTransmitMessageQueue;
	mTransmitMessageQueue = 0;

	mMessageDevice = NULL;
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::Post() --
//---------------------------------------------------------------------------
 
void MatadorMessageStream::Post(const MatadorMessage&  message)
{
	mTransmitMessageQueue->Push(message);
    mMessageThread->Activate();
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::ProcessMessages() --
//---------------------------------------------------------------------------
 
void MatadorMessageStream::ProcessMessages()
{
    TransmitOutgoingMessage();

    while (ProcessIncomingMessage())
        ;
}
 
//---------------------------------------------------------------------------
//  MatadorMessageStream::TransmitOutgoingMessage() --
//---------------------------------------------------------------------------
 
void MatadorMessageStream::TransmitOutgoingMessage()
{
    if (mTransmitBusy || !mTransmitMessageQueue->Count())
        return;

    //
    //  Extract the next message from outbound queue
	MatadorMessage Message = mTransmitMessageQueue->Pop();

    //
    //  There is a message to transmit....
    mTransmitBusy = true;
    
    mMessageDevice->TransmitMessage(Message);
}

//---------------------------------------------------------------------------
//  MatadorMessageStream::ProcessIncomingMessage() --
//---------------------------------------------------------------------------
 
bool MatadorMessageStream::ProcessIncomingMessage()
{
	MatadorMessage message;

	if( !mMessageDevice->ReceiveMessage( message, 0 ))
		return false;

	if ( message.IsSystemMessage())
		{
		ProcessSystemMessage(message);
		return true;
		}

    MatadorMessageEvent event(message);
	OnMessage.Execute(event);
	return true;
}

//---------------------------------------------------------------------------
//  MatadorMessageStream::ProcessSystemMessage() --
//---------------------------------------------------------------------------
 
void MatadorMessageStream::ProcessSystemMessage( MatadorMessage &Message )
{
    switch (Message.TypeCode())
        {
        case kDebugMsg:
                {

                // String size stored as ID
                int         Size = Message.MessageId();

                //  Eat the received string
                std::string Result;
                for (int i = 0; i < Size; ++i)
                    Result += Message.AsChar(i);

                ParseDebugString(Result);
                }

            break;

        default:
            throw MalibuException("Bad System Message");
        }
}

//---------------------------------------------------------------------------
//  MatadorMessageStream::ParseDebugString() --
//---------------------------------------------------------------------------
 
void MatadorMessageStream::ParseDebugString(std::string /*Msg*/)
{

    /*
        if (Msg[1] == '~')
            {
            Indent = 0;
            Msg = "_Warmstart";
            }
    
        if (Msg[1] == '_')
            {
            Msg.Delete(1, 1);
            OutputDebugString("________________________________________________________");
            OutputDebugString(Msg.c_str());
            return;
            }
    
        if (Msg[1] == '-')
            --Indent;
    
		String Tab;
        for (int i = 0; i < Indent; ++i)
            Tab += (!(i % 5)) ? '.' : ' ';
    
        if (Msg[1] == '+')
            ++Indent;
    
        OutputDebugString((Tab+Msg).c_str());
    */
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
 
//---------------------------------------------------------------------------
// MatadorMessageThread::MatadorMessageThread() --
//---------------------------------------------------------------------------
 
MatadorMessageThread::MatadorMessageThread()
{
}

//---------------------------------------------------------------------------
//  MatadorMessageThread::Init() --
//---------------------------------------------------------------------------

void MatadorMessageThread::Init()
{
    inherited::Init();
	AddObject(eGo, sGo);
}

//---------------------------------------------------------------------------
//  MatadorMessageThread::Execute() --
//---------------------------------------------------------------------------
 
void MatadorMessageThread::Execute()
{
    while (!Terminated())
        {
        int Result = WaitFor();

		if (Result == sGo)
			{
			if (!HandleProcessMessages())
				break;
			}
		}
}

//---------------------------------------------------------------------------
//  MatadorMessageThread::Activate() --
//---------------------------------------------------------------------------
 
void MatadorMessageThread::Activate()
{
	eGo.Release();
}

//---------------------------------------------------------------------------
//  MatadorMessageThread::IntRegister() --
//---------------------------------------------------------------------------
 
void MatadorMessageThread::IntRegister(MatadorMessageStream *board)
{
    CriticalSectionArea Area(mBoardsListSection);
    mBoards.push_back(board);
}

//---------------------------------------------------------------------------
//  MatadorMessageThread::IntUnregister() --
//---------------------------------------------------------------------------
 
void MatadorMessageThread::IntUnregister(MatadorMessageStream *board)
{
    CriticalSectionArea Area(mBoardsListSection);
    mBoards.remove(board);
}

//---------------------------------------------------------------------------
//  MatadorMessageThread::HandleProcessMessages() --
//---------------------------------------------------------------------------
 
bool MatadorMessageThread::HandleProcessMessages()
{
    CriticalSectionArea Area(mBoardsListSection);
    for (std::list<MatadorMessageStream *>::iterator Iter = mBoards.begin();
         Iter != mBoards.end(); Iter++)
        {
        (*Iter)->ProcessMessages();
        if (Terminated())
            return false;
        }

    return true;
}


} // namespace Innovative


