// MatadorMessageSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef MatadorMessageSupport_MbH
#define MatadorMessageSupport_MbH

#include <list>
#include <Semaphore_Mb.h>
#include <SyncObjects_Mb.h>
#include <MatadorMessage_Mb.h>
#include <Thread_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class MatadorMessageDevice;

//===========================================================================
//  CLASS MatadorMessageQueue  --  
//===========================================================================

class   MatadorMessageQueue
{

protected:
	std::list <MatadorMessage> mMessageList;
    Semaphore       mSemaphore;
    CriticalSection mCriticalSection;

public:
    int Count()     { return (int)mMessageList.size(); }

public:
	void        	Push(const MatadorMessage & message);
	MatadorMessage	Pop();

public:
    MatadorMessageQueue(int InitialDepth) :
        mSemaphore(InitialDepth)
            {
            }

	virtual ~MatadorMessageQueue()
	{
		while( Count() )
			Pop();
	}

};

//===========================================================================
//  CLASS MatadorMessageManager  --  
//===========================================================================

class MatadorMessageManager
{
public:
    virtual bool MessageTransmitted() = 0; // Returns true if the buffer is handled.
    virtual bool MessageReceived() = 0; // Returns true if the buffer is handled.

public:
    virtual void AttachToPtr( MatadorMessageDevice *device ) = 0;
    virtual void Detach() = 0;

public:
    void AttachTo( MatadorMessageDevice &device ) { AttachToPtr( &device ); }
    virtual ~MatadorMessageManager() {}
};

//===========================================================================
//  CLASS MatadorMessageDevice  --  
//===========================================================================

class MatadorMessageDevice
{
public:
	virtual bool TransmitMessage( const MatadorMessage &message, int wait_for = Innovative::Thread::Infinite ) = 0;
    virtual bool ReceiveMessage( MatadorMessage &message, int wait_for = Innovative::Thread::Infinite ) = 0;

public:
    virtual void RegisterMessagePlugIn( MatadorMessageManager *plug_in ) = 0;
    virtual ~MatadorMessageDevice() {}
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
