// Matador_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <sstream>
#include <iomanip>

#include <Matador_Mb.h>
#include <CoffFile_Mb.h>
#include <Thread_Mb.h>
#include <MatadorMessage_Mb.h>
#include <MatadorDeviceMap_Mb.h>
#include <Exception_Mb.h>
#include <HpiEngine_Mb.h>

namespace
{
const int       MaxMessageQueueSize = 1000;
const int       kDebugMsg = 0xdeb;

}  // end nameless namespace

namespace   Innovative
{
 
//===========================================================================
//  CLASS Matador  --  Matador DSP baseboard class
//===========================================================================
//---------------------------------------------------------------------------
// Matador::Matador() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif 
Matador::Matador() :
    mStreamManager(NULL),
    mMessageManager(NULL),
    mMessageTransmitEvent( Event::AutoReset, true ),
    FDownloadVerified(false)
{
    mAutoConfigure = true;

    mReceiveMessageQueue = new MatadorMessageQueue(MaxMessageQueueSize);

    mDevice = new MatadorDeviceMap;

    mDevice->OnBmIncoming.SetEvent(this, &Matador::HandleBmIncomingInterrupt);
    mDevice->OnBmOutgoing.SetEvent(this, &Matador::HandleBmOutgoingInterrupt);

    mDevice->OnMailboxAck.SetEvent(this, &Matador::HandlePacketAckInterrupt);
    mDevice->OnMailboxData.SetEvent(this, &Matador::HandlePacketRcvInterrupt);
    mDevice->PciInterruptEnableReg.DspMailboxAck = true;
    mDevice->PciInterruptEnableReg.DspMailboxData = true;
}
#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif
//---------------------------------------------------------------------------
// Matador::~Matador() --
//---------------------------------------------------------------------------
 
Matador::~Matador()
{
//    TMatadorMessageThread::Unregister(this);
    delete mDevice;
    delete mReceiveMessageQueue;
}

//---------------------------------------------------------------------------
//  Matador::RegisterStreamPlugIn() --
//---------------------------------------------------------------------------
 
void Matador::RegisterStreamPlugIn(StreamManager*  plug_in)
{
    if (mStreamManager)
        mStreamManager->Detach();

    mStreamManager = plug_in;
    mDevice->RegisterStreamPlugIn(plug_in);
}

//---------------------------------------------------------------------------
//  Matador::BoardCount() --  Return number of enumerated devices
//---------------------------------------------------------------------------

size_t Matador::BoardCount()
{
    return mDevice->BoardCount();
}

//---------------------------------------------------------------------------
//  Matador::Open() --
//---------------------------------------------------------------------------
 
void Matador::Open()
{
    mDevice->Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  Matador::Close() --
//---------------------------------------------------------------------------

void Matador::Close()
{
    mDevice->Close();
}

//------------------------------------------------------------------------------
//  Matador::BusMasterSize() --
//------------------------------------------------------------------------------

void Matador::BusMasterSize(int bytes)
{
    BusMasterSizeWords(bytes/sizeof(int));
}

//------------------------------------------------------------------------------
//  Matador::IncomingBusMasterSize() --
//------------------------------------------------------------------------------

int Matador::IncomingBusMasterSize() const
{
    return IncomingBusMasterSizeWords() * sizeof(int);
}

//------------------------------------------------------------------------------
//  Matador::OutgoingBusMasterSize() --
//------------------------------------------------------------------------------

int Matador::OutgoingBusMasterSize() const
{
    return OutgoingBusMasterSizeWords() * sizeof(int);
}

//------------------------------------------------------------------------------
//  Matador::BusMasterSizeWords() --
//------------------------------------------------------------------------------

void Matador::BusMasterSizeWords(unsigned int words)
{
    mDevice->BusMasterSizeWords(words);
}

//------------------------------------------------------------------------------
//  Matador::IncomingBusMasterSizeWords() --
//------------------------------------------------------------------------------

ii32 Matador::IncomingBusMasterSizeWords() const
{
    return mDevice->IncomingBusMasterSizeWords();
}

//------------------------------------------------------------------------------
//  Matador::OutgoingBusMasterSizeWords() --
//------------------------------------------------------------------------------

ii32 Matador::OutgoingBusMasterSizeWords() const
{
    return mDevice->OutgoingBusMasterSizeWords();
}

//---------------------------------------------------------------------------
//  Matador::Reset() --
//------------------------------------------------------------------------------

void Matador::Reset()
{
    mDevice->Reset();    // Board Reset

    if (!mDevice->Opened())
        throw Innovative::MalibuException("Error, device not open.");
}

//---------------------------------------------------------------------------
//  Matador::Cpu() -- Return reference to Cpu Interface object
//---------------------------------------------------------------------------

IUsesOnboardHpiCpu & Matador::Cpu()
{
    return *mDevice;
}

//------------------------------------------------------------------------------
//  Matador::GetLogicConfigured() --  Return true if logic is configured
//------------------------------------------------------------------------------

bool Matador::LogicConfigured()
{
    return true;

    //    if (TrueType != mfLobo)
    //        return true;
    // If successful, done bit is true
    //    int Value = mDevice->LogicConfig() & 0x1f;
    //    int Value = mDevice->LogicConfig & 0x1f;
    //    Port[hwmLogicConfig] & 0x1f;
    //    return (Value & 0x02) ? true : false;
}

//------------------------------------------------------------------------------
//  Matador::GetLogicInitializing() --  Return true if logic is initializing
//------------------------------------------------------------------------------

bool Matador::LogicInitializing()
{

    //    if (TrueType != mfLobo)
    //        return false;
    // If low, initialization underway
    int Value = mDevice->LogicConfig & 0x1f;

    //    int Value = Port[hwmLogicConfig] & 0x1f;
    return (Value & 0x04) ? false : true;
}

//---------------------------------------------------------------------------
//  Matador::DspInReset() --
//---------------------------------------------------------------------------

bool Matador::DspInReset() const
{
    return mDevice->DspInReset();
}

//---------------------------------------------------------------------------
//  Matador::DspInReset() --
//---------------------------------------------------------------------------

void Matador::DspInReset(bool value)
{
    //  Apply the Reset Bit to the control register
    mDevice->DspInReset(value);
}

//---------------------------------------------------------------------------
//  Matador::HandlePacketAckInterrupt() --
//---------------------------------------------------------------------------

void Matador::HandlePacketAckInterrupt(OpenWire::Event& )
{
    mMessageTransmitEvent.Set();
    if( mMessageManager )
        mMessageManager->MessageTransmitted();
}

//---------------------------------------------------------------------------
//  Matador::HandleBmIncomingInterrupt() --
//---------------------------------------------------------------------------

void Matador::HandleBmIncomingInterrupt(BufferIndexEvent&  event)
{
    mDevice->ProcessIncomingPacket(event.Index);
}

//---------------------------------------------------------------------------
//  Matador::HandleBmOutgoingInterrupt() --
//---------------------------------------------------------------------------
 
void Matador::HandleBmOutgoingInterrupt(BufferIndexEvent& Event)
{
    //  We have an interrupt ack for a block
    //  ...first keep track in our state (this section is available now)
    mDevice->OutgoingCredit.SectionReleased(Event.Index);
    //
    //  Activate the transmitter thread
    mDevice->TransmitDataAvailable();
}

//---------------------------------------------------------------------------
//  Matador::SetBusmasterSize() --
//---------------------------------------------------------------------------
 
void Matador::SetBusmasterSize(int size)
{
    mDevice->SetBusMasterSize(size);
}

//---------------------------------------------------------------------------
//  Matador::EnableBusMaster() --
//---------------------------------------------------------------------------
 
void Matador::EnableBusMaster(bool enable)
{
    if (!mDevice->Opened())
        throw Innovative::MalibuException("Error, device not open.");

    if (enable)
        {
        mDevice->PciInterruptEnableReg.Read.Set();
        mDevice->PciInterruptEnableReg.Write.Set();
        mDevice->StartSequencer();
        }

    else
        {
        mDevice->StopSequencer();
        mDevice->PciInterruptEnableReg.Read.Clear();
        mDevice->PciInterruptEnableReg.Write.Clear();
        }
}

//---------------------------------------------------------------------------
//  Matador::HandlePacketRcvInterrupt() --
//---------------------------------------------------------------------------
 
void Matador::HandlePacketRcvInterrupt(OpenWire::Event& )
{
    MatadorMessage Message;

    for (int i = 0; i < Message.PacketSize; i++)
        Message.Packet(i, mDevice->MailboxRam[i]);

    mReceiveMessageQueue->Push(Message);

    mDevice->MessageAck();

    if( mMessageManager )
        mMessageManager->MessageReceived();


//    mMessageThread->Activate();
}

//---------------------------------------------------------------------------
//  Matador::RegisterMessagePlugIn() --
//---------------------------------------------------------------------------
 
void Matador::RegisterMessagePlugIn( MatadorMessageManager *plug_in )
{
    if (mMessageManager)
        mMessageManager->Detach();

    mMessageManager = plug_in;
}
 
//---------------------------------------------------------------------------
//  Matador::TransmitMessage() --
//---------------------------------------------------------------------------
 
bool Matador::TransmitMessage( const MatadorMessage &message, int wait_for )
{
	if(!mMessageTransmitEvent.WaitFor( wait_for ))
        return false;
    
    //
    //  Transmit the message
    for (int i = 0; i < message.PacketSize; i++)
        mDevice->MailboxRam[i] = message.Packet(i);

    mDevice->MessageCommandReg = 0xFFFF; // Set the Dirty bits!
    return true;
}
 
//---------------------------------------------------------------------------
//  Matador::ReceiveMessage() --
//---------------------------------------------------------------------------
 
bool Matador::ReceiveMessage(MatadorMessage & message, int /*wait_for*/ )
{
	if (mReceiveMessageQueue->Count())
		{
		message = mReceiveMessageQueue->Pop();
		return true;
		}
	else
		{
		return false;
		}
}

//---------------------------------------------------------------------------
//  Matador::ConfigureBusmastering()
//---------------------------------------------------------------------------

void  Matador::ConfigureBusmastering()
{
    mDevice->ConfigureBusmastering();
}

//---------------------------------------------------------------------------
//  Matador::PreloadXmitBusmasterRegion()
//---------------------------------------------------------------------------

void  Matador::PreloadXmitBusmasterRegion()
{
    mDevice->PreloadXmitBusmasterRegion();
}

//---------------------------------------------------------------------------
//  Matador::TransmitDataAvailable()
//---------------------------------------------------------------------------

void  Matador::TransmitDataAvailable()
{
    mDevice->TransmitDataAvailable();
}

//---------------------------------------------------------------------------
//  Matador::GrantCredit()
//---------------------------------------------------------------------------

void  Matador::GrantCredit(int direction, size_t code)
{
    mDevice->GrantCredit(direction, code);
}

//---------------------------------------------------------------------------
//  Matador::KpQueueDepth() --  Return depth of KP ISR queue
//------------------------------------------------------------------------------

size_t Matador::KpQueueDepth()
{
    return mDevice->KpQueueDepth();
}

//---------------------------------------------------------------------------
//  Matador::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & Matador::VsProm()
{
    return *mDevice->EepromLoader;
}

//---------------------------------------------------------------------------
//  Matador::DeviceInfo() --
//---------------------------------------------------------------------------

InnovativeKernel::DeviceEntry & Matador::DeviceInfo()
{
    return mDevice->DeviceInfo();
}

void Matador::Target(int value)
{
    mDevice->Target(value);
}

int  Matador::Target() const
{
    return mDevice->Target();
}

void Matador::Led(bool state)
{
    mDevice->Led(state);
}

bool Matador::Led() const
{
    return mDevice->Led();
}

void Matador::DownloadVerified(bool state)
{
    FDownloadVerified = state;
}

bool Matador::DownloadVerified() const
{
    return FDownloadVerified;
}


} // namespace Innovative

