// Matador_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Matador_MbH
#define Matador_MbH

#include <ProcessEvents_Mb.h>
#include <StreamSupport_Mb.h>
#include <MatadorMessageSupport_Mb.h>
#include <Event_Mb.h>
#include <MatadorDeviceMap_Mb.h>

class BufferIndexEvent;

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS Matador  --  Matador DSP baseboard class
//===========================================================================

class Matador : public StreamDevice,
                public MatadorMessageDevice
{
public:
    // Ctors
    Matador();
    virtual ~Matador();

    // Methods
    size_t          BoardCount();
    void            Open();
    void            Close();
    void            Reset();
    size_t          KpQueueDepth();
    InnovativeKernel::DeviceEntry & DeviceInfo();

    // Coff loader
    IUsesOnboardHpiCpu &    Cpu();
    FpaLoadThread &         VsProm();

    // Properties
    void                    Target(int value);
    int                     Target() const;

    void                    Led(bool state);
    bool                    Led() const;

    void                    DspInReset(bool state);
    bool                    DspInReset() const;

    void                    DownloadVerified(bool state);
    bool                    DownloadVerified() const;

	void 					BusMasterSize(int bytes);
	int             		IncomingBusMasterSize() const;
	int             		OutgoingBusMasterSize() const;
    
//protected:
    // Pseudo-protected methods
    virtual void   TransmitDataAvailable();
    virtual void   PreloadXmitBusmasterRegion();
    virtual void   ConfigureBusmastering();
    virtual void   GrantCredit(int direction, size_t code);

protected:
    // Data
    MatadorDeviceMap		*	mDevice;
    StreamManager			*	mStreamManager;
    MatadorMessageManager   *	mMessageManager;
    MatadorMessageQueue     *	mReceiveMessageQueue;
	bool mAutoConfigure;
    Event                       mMessageTransmitEvent;

    // Methods
    void HpiReady();

    void HandlePacketAckInterrupt( OpenWire::Event & );
    void HandlePacketRcvInterrupt( OpenWire::Event & );

    void HandleBmIncomingInterrupt( BufferIndexEvent &event );
    void HandleBmOutgoingInterrupt( BufferIndexEvent &event );

    bool LogicConfigured();
    bool LogicInitializing();

    virtual void RegisterStreamPlugIn( StreamManager *plug_in );

    // Specify driver-allocated BM region, prior to open
    virtual void            BusMasterSizeWords(unsigned int value);
    virtual unsigned int    IncomingBusMasterSizeWords() const;
    virtual unsigned int    OutgoingBusMasterSizeWords() const;
    virtual void SetBusmasterSize( int size );
    virtual void EnableBusMaster( bool enable );

    bool GetDspInReset();
    void SetDspInReset( bool value );

public: // pseudo-public -- not for general use
	virtual bool TransmitMessage( const MatadorMessage &message,
        int wait_for = Innovative::Thread::Infinite );
	virtual bool ReceiveMessage( MatadorMessage &message,
        int wait_for = Innovative::Thread::Infinite );

    virtual void RegisterMessagePlugIn( MatadorMessageManager *plug_in );

protected:
    bool            FDownloadVerified;

public:
	//  Debug
	void  Log(int code, int data) {  mDevice->Log(code, data);  };
	void  DumpLog(std::vector<int> & vec) {  mDevice->DumpLog(vec);  };
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
