//---------------------------------------------------------------------------
// MatlabMatrix_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#include "Exception_Mb.h"
#include "MatlabMatrix_Mb.h"
#include <string>
#include <sstream>
#include <algorithm>
#include <Trace_Mb.h>
#include <windows.h>

using namespace std;

namespace Innovative
{

//===========================================================================
//  Class MatlabMatrixDllFunctions  --  Matlab Matrix DLL access
//===========================================================================

class MatlabMatrixDllFunctions
{
    friend class MatlabMatrix;
    friend class MatlabFile;

    //
    //  ...Function Pointer Typedefs
    typedef  mxArray * (__cdecl * DLLPROC_1)(const mxArray *in);
    typedef  void (__cdecl * DLLPROC_2)(mxArray *pa, void  *newdata);
    typedef  void * (__cdecl * DLLPROC_3)(const mxArray *pa);
    typedef  size_t (__cdecl * DLLPROC_4)(const mxArray *pa);
    typedef  bool (__cdecl * DLLPROC_5)(const mxArray *pa);
    typedef  void (__cdecl * DLLPROC_6)(const mxArray *pa);
    typedef  void (__cdecl * DLLPROC_7)(void *ptr);
    typedef  mxArray * (__cdecl * DLLPROC_8)(mwSize  ndim, const mwSize *dims, mxClassID classid, mxComplexity flag);
    typedef  mxArray * (__cdecl * DLLPROC_8a)(mwSize  ndim, const mwSize *dims, int classid, int flag);
    typedef  mxArray * (__cdecl * DLLPROC_9)(mwSize  m, mwSize  n, mxClassID classid, int flag);
    typedef  mxArray * (__cdecl * DLLPROC_9a)(mwSize  m, mwSize  n, int classid, int flag);
    typedef  mxArray * (__cdecl * DLLPROC_10)(mwSize  m, mwSize  n, mxComplexity flag);
    typedef  mxArray * (__cdecl * DLLPROC_10a)(mwSize  m, mwSize  n, int flag);
    typedef  size_t (__cdecl * DLLPROC_11)(const mxArray *pa, mwSize nsubs, mwIndex *subs);

    //
    //  Member Data
    HINSTANCE               Instance;
    bool                    Ok;

    DLLPROC_1  mxDuplicateArray;
    DLLPROC_2  mxSetData;
    DLLPROC_3  mxGetData;
    DLLPROC_3  mxGetImagData;
    DLLPROC_4  mxGetNumberOfElements;
    DLLPROC_5  mxIsDouble;
    DLLPROC_5  mxIsInt16;
    DLLPROC_6  mxDestroyArray;
    DLLPROC_7  mxFree;
    DLLPROC_8a  mxCreateNumericArray;
    DLLPROC_9a  mxCreateNumericMatrix;
    DLLPROC_10a  mxCreateDoubleMatrix;
    DLLPROC_11 mxCalcSingleSubscript;

    //
    //  Member Functions
    MatlabMatrixDllFunctions();
    ~MatlabMatrixDllFunctions();
};

//---------------------------------------------------------------------------
//  constructor for class MatlabMatrixDllFunctions
//---------------------------------------------------------------------------

MatlabMatrixDllFunctions::MatlabMatrixDllFunctions()
    : Instance(0)
    , Ok(true)
    , mxDuplicateArray(0)
    , mxSetData(0)
    , mxGetData(0)
    , mxGetNumberOfElements(0)
    , mxIsDouble(0)
    , mxIsInt16(0)
    , mxDestroyArray(0)
    , mxFree(0)
    , mxCreateNumericArray(0)
{
    const char * lib = "libmx.dll";
    Trace Debug;

    //
    //  Attempt to Open the DLL & get DLL functions
    Instance = LoadLibraryA(lib);
    if (Instance==NULL)
        {
        //
        //  Could not open DLL. Add error to event log
        std::stringstream msg;
        msg << "Could not open " << lib;
        Debug.Log(msg.str());

        Ok = false;
        return;
        }
    //
    //  Get pointers to functions
    mxDuplicateArray = (DLLPROC_1)GetProcAddress(Instance, "mxDuplicateArray");
    if (!mxDuplicateArray)
        Ok = false;

    mxSetData = (DLLPROC_2)GetProcAddress(Instance, "mxSetData");
    if (!mxSetData)
        Ok = false;

    mxGetData = (DLLPROC_3)GetProcAddress(Instance, "mxGetData");
    if (!mxGetData)
        Ok = false;

    mxGetImagData = (DLLPROC_3)GetProcAddress(Instance, "mxGetImagData");
    if (!mxGetImagData)
        Ok = false;

    mxGetNumberOfElements = (DLLPROC_4)GetProcAddress(Instance, "mxGetNumberOfElements");
    if (!mxGetNumberOfElements)
        Ok = false;

    mxIsDouble = (DLLPROC_5)GetProcAddress(Instance, "mxIsDouble");
    if (!mxIsDouble)
        Ok = false;

    mxIsInt16 = (DLLPROC_5)GetProcAddress(Instance, "mxIsInt16");
    if (!mxIsInt16)
        Ok = false;

    mxDestroyArray = (DLLPROC_6)GetProcAddress(Instance, "mxDestroyArray");
    if (!mxDestroyArray)
        Ok = false;

    mxFree = (DLLPROC_7)GetProcAddress(Instance, "mxFree");
    if (!mxFree)
        Ok = false;

    mxCreateNumericArray = (DLLPROC_8a)GetProcAddress(Instance, "mxCreateNumericArray");
    if (!mxCreateNumericArray)
        Ok = false;

    mxCreateNumericMatrix = (DLLPROC_9a)GetProcAddress(Instance, "mxCreateNumericMatrix");
    if (!mxCreateNumericMatrix)
        Ok = false;

    mxCreateDoubleMatrix = (DLLPROC_10a)GetProcAddress(Instance, "mxCreateDoubleMatrix");
    if (!mxCreateDoubleMatrix)
        Ok = false;

    mxCalcSingleSubscript = (DLLPROC_11)GetProcAddress(Instance, "mxCalcSingleSubscript");
    if (!mxCalcSingleSubscript)
        Ok = false;

    std::stringstream msg;
    msg << lib << " open status: " << (Ok ? "ok" : "failed");
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  destructor for class MatlabMatrixDllFunctions
//---------------------------------------------------------------------------

MatlabMatrixDllFunctions::~MatlabMatrixDllFunctions()
{
    //
    //  Free the DLL instance
    if (!Instance)
        return;

    FreeLibrary(Instance);
    Instance = 0;
}

//===========================================================================
//  CLASS MatlabMatrix  --
//===========================================================================

MatlabMatrixDllFunctions * MatlabMatrix::Dll = 0;
unsigned int MatlabMatrix::Instances = 0;

// Ctors
MatlabMatrix::MatlabMatrix()
    : pa(NULL)
{
    if (!Instances++)
        Dll = new MatlabMatrixDllFunctions;

    if (Dll->Ok==false)
        {
        throw MalibuException("Matlab Matrix Support unavailable: No DLLs found");
        }
}

//---------------------------------------------------------------------------
// MatlabMatrix::MatlabMatrix() --
//---------------------------------------------------------------------------

MatlabMatrix::MatlabMatrix(mxArray * p)
    : pa(p)
{
    if (!Instances++)
        Dll = new MatlabMatrixDllFunctions;

    if (Dll->Ok==false)
        {
        throw MalibuException("Matlab Matrix Support unavailable: No DLLs found");
        }
}

//---------------------------------------------------------------------------
// MatlabMatrix::MatlabMatrix() --
//---------------------------------------------------------------------------

MatlabMatrix::MatlabMatrix(const IntArray & n, mxClassID classid, mxComplexity flag)
{
    if (!Instances++)
        Dll = new MatlabMatrixDllFunctions;

    pa = Dll->mxCreateNumericArray(static_cast<int>(n.size()), &n[0], classid, flag);
}

//---------------------------------------------------------------------------
// MatlabMatrix::MatlabMatrix() --
//---------------------------------------------------------------------------

MatlabMatrix::MatlabMatrix(int m, int n, mxClassID classid, mxComplexity flag)
{
    if (!Instances++)
        Dll = new MatlabMatrixDllFunctions;

    pa = Dll->mxCreateNumericMatrix(m, n, classid, flag);
}

//---------------------------------------------------------------------------
// MatlabMatrix::MatlabMatrix() --  Copy ctor
//---------------------------------------------------------------------------

MatlabMatrix::MatlabMatrix(const MatlabMatrix & other)
{
    if (!Instances++)
        Dll = new MatlabMatrixDllFunctions;

    pa = Dll->mxDuplicateArray(other.pa);
}

//---------------------------------------------------------------------------
// MatlabMatrix::~MatlabMatrix() --  Dtor
//---------------------------------------------------------------------------

MatlabMatrix::~MatlabMatrix()
{
    if (Instances)
        {
        --Instances;
        return;
        }

    Destroy();

    delete Dll;
    Dll = 0;
}

//---------------------------------------------------------------------------
// MatlabMatrix::operator = --  Assignment
//---------------------------------------------------------------------------

MatlabMatrix & MatlabMatrix::operator =(const MatlabMatrix & other)
{
    // Avoid self-assignment
    if (pa != other.pa)
        {
        Destroy();
        pa = Dll->mxDuplicateArray(other.pa);
        }

    return *this;
}

//---------------------------------------------------------------------------
// MatlabMatrix::operator Swap --  Exchange
//---------------------------------------------------------------------------

MatlabMatrix & MatlabMatrix::Swap(MatlabMatrix & other)
{
    // Avoid self-assignment
    if (pa != other.pa)
        std::swap(pa, other.pa);

    return *this;
}

//---------------------------------------------------------------------------
//  MatlabMatrix::ReplaceData() -- Replace data with new set
//---------------------------------------------------------------------------
//  Note: must have already freed data ptr!
//  Note: must alloc new data ptr using MATLAB calls!

void MatlabMatrix::ReplaceData(void * newdata)
{
    Dll->mxSetData(pa, newdata);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::DataPtr() -- Return pointer to real array data
//---------------------------------------------------------------------------

void * MatlabMatrix::DataPtr() const
{
    return Dll->mxGetData(pa);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::ImagDataPtr() -- Return pointer to imaginary array data
//---------------------------------------------------------------------------

void * MatlabMatrix::ImagDataPtr() const
{
    return Dll->mxGetImagData(pa);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::ImagDataPtr() -- Return index to specified array element
//---------------------------------------------------------------------------

size_t MatlabMatrix::IndexOf(IntArray & n) const
{
    IntArray m(n);
    for (size_t i = 0; i < m.size(); ++i)
        ++m[i];
    return Dll->mxCalcSingleSubscript(pa, m.size(), &m[0]);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::Destroy() --
//---------------------------------------------------------------------------

void MatlabMatrix::Destroy()
{
    if (pa)
        Dll->mxDestroyArray(pa);
    pa = NULL;
}

//---------------------------------------------------------------------------
//  MatlabMatrix::NumberOfElements() --
//---------------------------------------------------------------------------

size_t MatlabMatrix::NumberOfElements() const
{
    return Dll->mxGetNumberOfElements(pa);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::IsInt16() --
//---------------------------------------------------------------------------

bool MatlabMatrix::IsInt16() const
{
    return Dll->mxIsInt16(pa);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::IsDouble() --
//---------------------------------------------------------------------------

bool MatlabMatrix::IsDouble() const
{
    return Dll->mxIsDouble(pa);
}

//---------------------------------------------------------------------------
//  MatlabMatrix::Free() --
//---------------------------------------------------------------------------

void MatlabMatrix::Free(void *p) const
{
    Dll->mxFree(p);
}

//===========================================================================
//  Class MatlabFileDllFunctions  --  Matlab file DLL access
//===========================================================================

class MatlabFileDllFunctions
{
    friend class MatlabFile;

    //
    //  ...Function Pointer Typedefs

    typedef  MATFile * (__cdecl * DLLPROC_1)(const char *filename, const char * mode);
    typedef  int (__cdecl * DLLPROC_2)(MATFile *pMF);
    typedef  int (__cdecl * DLLPROC_3)(MATFile * pMF, const char * name, const mxArray * pA);
    typedef  mxArray * (__cdecl * DLLPROC_4)(MATFile * pMF, const char * name);
    typedef  char ** (__cdecl * DLLPROC_5)(MATFile * pMF, int *num);

    //
    //  Member Data
    HINSTANCE  Instance;
    bool       Ok;

    DLLPROC_1  matOpen;
    DLLPROC_2  matClose;
    DLLPROC_3  matPutVariable;
    DLLPROC_4  matGetVariable;
    DLLPROC_5  matGetDir;

    //
    //  Member Functions
    MatlabFileDllFunctions();
    ~MatlabFileDllFunctions();
};

//---------------------------------------------------------------------------
//  constructor for class MatlabFileDllFunctions
//---------------------------------------------------------------------------

MatlabFileDllFunctions::MatlabFileDllFunctions()
    : Instance(0)
    ,  Ok(true)
    , matOpen(0)
    , matClose(0)
    , matPutVariable(0)
    , matGetVariable(0)
    , matGetDir(0)
{
    const char * lib = "libmat.dll";
    //"libmat.dll";
    Trace Debug;

    //
    //  Attempt to Open the DLL & get DLL functions
    Instance = LoadLibraryA(lib);
    if (Instance==NULL)
        {
        //
        //  Could not open DLL. Add error to event log
        std::stringstream msg;
        msg << "Could not open " << lib;
        Debug.Log(msg.str());

        Ok = false;
        return;
        }
    //
    //  Get pointers to functions
    matOpen = (DLLPROC_1)GetProcAddress(Instance, "matOpen");
    if (!matOpen)
        Ok = false;

    matClose = (DLLPROC_2)GetProcAddress(Instance, "matClose");
    if (!matClose)
        Ok = false;

    matPutVariable = (DLLPROC_3)GetProcAddress(Instance, "matPutVariable");
    if (!matPutVariable)
        Ok = false;

    matGetVariable = (DLLPROC_4)GetProcAddress(Instance, "matGetVariable");
    if (!matGetVariable)
        Ok = false;

    matGetDir = (DLLPROC_5)GetProcAddress(Instance, "matGetDir");
    if (!matGetDir)
        Ok = false;

    std::stringstream msg;
    msg << lib << " open status: " << (Ok ? "ok" : "failed");
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  destructor for class MatlabFileDllFunctions
//---------------------------------------------------------------------------

MatlabFileDllFunctions::~MatlabFileDllFunctions()
{
    //
    //  Free the DLL instance
    if (!Instance)
        return;

    FreeLibrary(Instance);
    Instance = 0;
}


//===========================================================================
//  Class MatlabFile  --  Matlab file access
//===========================================================================

MatlabFileDllFunctions * MatlabFile::Dll = 0;
unsigned int MatlabFile::Instances = 0;

//---------------------------------------------------------------------------
// MatlabFile::MatlabFile() --
//---------------------------------------------------------------------------

MatlabFile::MatlabFile()
    : file(NULL)
{
    if (!Instances++)
        Dll = new MatlabFileDllFunctions;
}

//---------------------------------------------------------------------------
// MatlabFile::~MatlabFile() --
//---------------------------------------------------------------------------

MatlabFile::~MatlabFile()
{
    if (Instances)
        {
        --Instances;
        return;
        }

    Close();

    delete Dll;
    Dll = 0;
}

//---------------------------------------------------------------------------
// MatlabFile::Open() --
//  * Current valid entries for "mode" are
//  * "r"  == read only.
//  * "w"  == write only (deletes any existing file with name <filename>).
//  * "w4" == as "w", but create a MATLAB 4.0 MAT-file.
//  * "u"  == update.  Read and write allowed, existing file is not deleted.
//---------------------------------------------------------------------------

bool MatlabFile::Open(const std::string & filename, const char * mode)
{
    if (Dll->Ok==false)
        {
        throw Innovative::MalibuException(std::string("Matlab runtime libraries unavailable - get from Mathworks, then retry."));
        }

    file = Dll->matOpen(filename.c_str(), mode);
    return (file) ? true : false;
}

//---------------------------------------------------------------------------
//  MatlabFile::Close() --
//---------------------------------------------------------------------------

bool MatlabFile::Close()
{
    if (!file)
        return true;
    bool result = (Dll->matClose(file)==0) ? true : false;
    file = NULL;

    return result;
}

//---------------------------------------------------------------------------
//  MatlabFile::Write() --
// Result = 0 for success
//---------------------------------------------------------------------------

int MatlabFile::Write(const std::string & name, const MatlabMatrix & mat) const
{
    return Dll->matPutVariable(file, name.c_str(), mat.pa);
}

//---------------------------------------------------------------------------
//  MatlabFile::Read() --
//---------------------------------------------------------------------------

MatlabMatrix MatlabFile::Read(const std::string & name) const
{
    return MatlabMatrix(Dll->matGetVariable(file, name.c_str()));
}

//---------------------------------------------------------------------------
//  MatlabFile::Dir() --
//---------------------------------------------------------------------------

Innovative::StringList MatlabFile::Dir() const
{
    Innovative::StringList result;
    int num;
    char ** p = Dll->matGetDir(file, &num);
    if (p)
        {
        for (int i = 0; i < num; ++i)
            result.Add(p[i]);
        }

    return result;
}

//===========================================================================
//  Class MatlabEngineDllFunctions  --  Matlab Engine DLL access
//===========================================================================

class MatlabEngineDllFunctions
{
    friend class MatlabEngine;
    friend class MatlabFile;

    //
    //  ...Function Pointer Typedefs
    typedef  int (__cdecl * DLLPROC_1)(Engine * ep, const char * string);
    typedef  Engine * (__cdecl * DLLPROC_2)(const char * startcmd, void * rsvd, int * retstatus);
    typedef  int (__cdecl * DLLPROC_3)(Engine * ep, bool value);
    typedef  int (__cdecl * DLLPROC_4)(Engine * ep, bool * value);
    typedef  Engine * (__cdecl * DLLPROC_5)(const char * string);
    typedef  int (__cdecl * DLLPROC_6)(Engine * ep);
    typedef  mxArray * (__cdecl * DLLPROC_7)(Engine * ep, const char * string);
    typedef  int (__cdecl * DLLPROC_8)(Engine * ep, const char * string, const mxArray *ap);
    typedef  int (__cdecl * DLLPROC_9)(Engine * ep, char * buffer, int len);

    //
    //  Member Data
    HINSTANCE  Instance;
    bool       Ok;

    DLLPROC_1  engEvalString;
    DLLPROC_2  engOpenSingleUse;
    DLLPROC_3  engSetVisible;
    DLLPROC_4  engGetVisible;
    DLLPROC_5  engOpen;
    DLLPROC_6  engClose;
    DLLPROC_7  engGetVariable;
    DLLPROC_8  engPutVariable;
    DLLPROC_9  engOutputBuffer;
    //
    //  Member Functions
    MatlabEngineDllFunctions();
    ~MatlabEngineDllFunctions();
};

//---------------------------------------------------------------------------
//  constructor for class MatlabEngineDllFunctions
//---------------------------------------------------------------------------

MatlabEngineDllFunctions::MatlabEngineDllFunctions()
    : Instance(0)
    , Ok(true)
    , engEvalString(0)
    , engOpenSingleUse(0)
    , engSetVisible(0)
    , engGetVisible(0)
    , engOpen(0)
    , engClose(0)
    , engGetVariable(0)
    , engPutVariable(0)
    , engOutputBuffer(0)
{
    const char * lib = "libeng.dll";
    Trace Debug;

    //
    //  Attempt to Open the DLL & get DLL functions
    Instance = LoadLibraryA(lib);
    if (Instance==NULL)
        {
        //
        //  Could not open DLL. Add error to event log
        std::stringstream msg;
        msg << "Could not open " << lib;
        Debug.Log(msg.str());

        Ok = false;
        return;
        }
    //
    //  Get pointers to functions
    engEvalString = (DLLPROC_1)GetProcAddress(Instance, "engEvalString");
    if (!engEvalString)
        Ok = false;

    engOpenSingleUse = (DLLPROC_2)GetProcAddress(Instance, "engOpenSingleUse");
    if (!engOpenSingleUse)
        Ok = false;

    engSetVisible = (DLLPROC_3)GetProcAddress(Instance, "engSetVisible");
    if (!engSetVisible)
        Ok = false;

    engGetVisible = (DLLPROC_4)GetProcAddress(Instance, "engGetVisible");
    if (!engGetVisible)
        Ok = false;

    engOpen = (DLLPROC_5)GetProcAddress(Instance, "engOpen");
    if (!engOpen)
        Ok = false;

    engClose = (DLLPROC_6)GetProcAddress(Instance, "engClose");
    if (!engClose)
        Ok = false;

    engGetVariable = (DLLPROC_7)GetProcAddress(Instance, "engGetVariable");
    if (!engGetVariable)
        Ok = false;

    engPutVariable = (DLLPROC_8)GetProcAddress(Instance, "engPutVariable");
    if (!engPutVariable)
        Ok = false;

    engOutputBuffer = (DLLPROC_9)GetProcAddress(Instance, "engOutputBuffer");
    if (!engOutputBuffer)
        Ok = false;

    std::stringstream msg;
    msg << lib << " open status: " << (Ok ? "ok" : "failed");
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  destructor for class MatlabEngineDllFunctions
//---------------------------------------------------------------------------

MatlabEngineDllFunctions::~MatlabEngineDllFunctions()
{
    //
    //  Free the DLL instance
    if (!Instance)
        return;

    FreeLibrary(Instance);
    Instance = 0;
}

//===========================================================================
//  CLASS MatlabEngine  --  Matlab engine access
//===========================================================================

MatlabEngineDllFunctions * MatlabEngine::Dll = 0;
unsigned int MatlabEngine::Instances = 0;

//---------------------------------------------------------------------------
//  MatlabEngine Ctor
//---------------------------------------------------------------------------

MatlabEngine::MatlabEngine()
    : ep(0)
{
    if (!Instances++)
        Dll = new MatlabEngineDllFunctions;
}

//---------------------------------------------------------------------------
//  MatlabEngine Dtor
//---------------------------------------------------------------------------

MatlabEngine::~MatlabEngine()
{
    if (Instances)
        {
        --Instances;
        return;
        }

    Close();

    delete Dll;
    Dll = 0;
}

//---------------------------------------------------------------------------
//  MatlabEngine::Evaluate() --  Execute matlab statement
//---------------------------------------------------------------------------

bool MatlabEngine::Evaluate(const std::string & cmd)
{
    if (!ep)
        return false;

    return (Dll->engEvalString(ep, cmd.c_str()) == 0);
}

//---------------------------------------------------------------------------
//  MatlabEngine::Open() --  Start matlab process
//---------------------------------------------------------------------------

bool MatlabEngine::Open(const std::string & cmd, bool single_use)
{
    int result = 0;

    if (single_use)
        ep = Dll->engOpenSingleUse(cmd.c_str(), NULL, &result);
    else
        ep = Dll->engOpen(cmd.c_str());

    return (ep != 0) && (result == 0);
}

//---------------------------------------------------------------------------
//  MatlabEngine::Close() --  Close down matlab server
//---------------------------------------------------------------------------

bool MatlabEngine::Close()
{
    if (!ep)
        return false;

    int result = Dll->engClose(ep);
    ep = 0;

    return (result == 0);
}

//---------------------------------------------------------------------------
//  MatlabEngine::Write() --  Put a variable into MATLAB's workspace with the specified name
//---------------------------------------------------------------------------

bool MatlabEngine::Write(const std::string & name, const MatlabMatrix & matrix)
{
    return (Dll->engPutVariable(ep, name.c_str(), matrix.pa) == 0);
}

//---------------------------------------------------------------------------
//  MatlabEngine::Read() --  Get a variable with the specified name from MATLAB's workspace
//---------------------------------------------------------------------------

MatlabMatrix MatlabEngine::Read(const std::string & name)
{
    return MatlabMatrix(Dll->engGetVariable(ep, name.c_str()));
}

//---------------------------------------------------------------------------
//  MatlabEngine::OutputBufferSize() --  Set size of text response buffer
//---------------------------------------------------------------------------

bool MatlabEngine::OutputBufferSize(int len)
{
    text.resize(len);
    if (text.size())
        text[0] = '\0';

    bool result = Dll->engOutputBuffer(ep, &text[0], len) ? false : true;

    return result;
}

//---------------------------------------------------------------------------
//  MatlabEngine::OutputBuffer() --  Return command window text output
//---------------------------------------------------------------------------

std::string MatlabEngine::OutputBuffer() const
{
    string result;
    if (text.size())
        {
        result = text;
        text[0] = '\0';
        }

    return result;
}


} // namespace


