//---------------------------------------------------------------------------
// MatlabMatrix_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef MatlabMatrix_MbH
#define MatlabMatrix_MbH
//---------------------------------------------------------------------------

#include <MatLab/mat.h>
#include <MatLab/engine.h>
#include <StringList_Mb.h>
#include <string>
#include <vector>

namespace Innovative
{

//===========================================================================
//  CLASS MatlabMatrix  --  Matlab matrix access
//===========================================================================

class MatlabMatrix
{
    friend class MatlabFile;
    friend class MatlabEngine;

public:
    typedef std::vector<size_t> IntArray;

    // Ctors
    MatlabMatrix();
    MatlabMatrix(mxArray * p);
    MatlabMatrix(const IntArray & n, mxClassID classid, mxComplexity flag=mxREAL);
    MatlabMatrix(int m, int n, mxClassID classid=mxDOUBLE_CLASS, mxComplexity flag=mxREAL);
    MatlabMatrix(const MatlabMatrix & other);
    // Dtor
    ~MatlabMatrix();
    // Assignment
    MatlabMatrix & operator =(const MatlabMatrix & other);
    MatlabMatrix &  Swap(MatlabMatrix & other);

    // Methods
    void            ReplaceData(void * newdata);
    void *          DataPtr() const;
    void *          ImagDataPtr() const;

    bool            IsDouble() const;
    bool            IsInt16() const;
    size_t          IndexOf(IntArray & n) const;

    size_t          NumberOfElements() const;

protected:
    // Methods
    void            Destroy();
    void            Free(void *p) const;

    // Data
    static class MatlabMatrixDllFunctions * Dll;
    static unsigned int Instances;
    mxArray * pa;
};

//===========================================================================
//  CLASS MatlabFile  --  Matlab file access
//===========================================================================

class MatlabFile
{
public:
    MatlabFile();
    ~MatlabFile();

    bool Open(const std::string & filename, const char * mode);
    bool Close();

    // Result = 0 for success
    int Write(const std::string & name, const MatlabMatrix & mat) const;
    MatlabMatrix Read(const std::string & name) const;
    Innovative::StringList Dir() const;

private:
    static class MatlabFileDllFunctions  * Dll;
    static unsigned int Instances;
    MATFile *   file;
};

//===========================================================================
//  CLASS MatlabEngine  --  Matlab engine access
//===========================================================================

class MatlabEngine
{
public:
    // Ctors
    MatlabEngine();
    ~MatlabEngine();

    // Methods
    bool                Open(const std::string & cmd, bool single_use=false);
    bool                Close();
    // Access
    bool                Evaluate(const std::string & cmd);
    // Debug
    bool                OutputBufferSize(int len=10000);
    std::string         OutputBuffer() const;

    // Properties
    bool                Write(const std::string & name, const MatlabMatrix & matrix);
    MatlabMatrix        Read(const std::string & name);

private:
    // Data
    static class MatlabEngineDllFunctions  * Dll;
    static unsigned int Instances;
    Engine *            ep;
    mutable std::string text;
};



}   // namespace

#endif
