/*++

Module Name:

    MemDriver_Mb.H

OS:

    Windows

--*/
#ifndef MEMDRIVER_MB_H
#define MEMDRIVER_MB_H

//
// Device type           -- in the "User Defined" range."
//
#define FILEIO_TYPE 40001

// The IOCTL function codes from 0x800 to 0xFFF are for customer use.

#define Low_IOCTL_ALLOC_MEMORY_BLOCK \
    CTL_CODE( FILEIO_TYPE, 0x900, METHOD_BUFFERED, FILE_ANY_ACCESS  )
#define High_IOCTL_ALLOC_MEMORY_BLOCK \
    CTL_CODE( FILEIO_TYPE, 0x904, METHOD_BUFFERED, FILE_ANY_ACCESS  )

#define Low_IOCTL_FREE_MEMORY_BLOCK \
    CTL_CODE( FILEIO_TYPE, 0x901, METHOD_BUFFERED, FILE_ANY_ACCESS  )
#define High_IOCTL_FREE_MEMORY_BLOCK \
    CTL_CODE( FILEIO_TYPE, 0x905, METHOD_BUFFERED, FILE_ANY_ACCESS  )

#define Low_IOCTL_DUMP_MEM_POOL \
    CTL_CODE( FILEIO_TYPE, 0x902, METHOD_BUFFERED, FILE_ANY_ACCESS  )
#define High_IOCTL_DUMP_MEM_POOL \
    CTL_CODE( FILEIO_TYPE, 0x906, METHOD_BUFFERED, FILE_ANY_ACCESS  )

#define Low_IOCTL_RESET_MEM_POOL \
    CTL_CODE( FILEIO_TYPE, 0x903, METHOD_BUFFERED, FILE_ANY_ACCESS  )
#define High_IOCTL_RESET_MEM_POOL \
    CTL_CODE( FILEIO_TYPE, 0x907, METHOD_BUFFERED, FILE_ANY_ACCESS  )


#define DRIVER_FUNC_INSTALL     0x01
#define DRIVER_FUNC_REMOVE      0x02

#define DRIVER_NAME       "MEMDRV"
#define DEVICE_NAME       "\\\\.\\MEMDRV\\memdrvsamp.log"

#define STRBUFSIZE 50
#define MEGA_BYTE (1024*1024)
#define MAX_POOL_SIZE ((size_t)(~0)/MEGA_BYTE)
#define MIN_POOL_SIZE 1

typedef enum {POOL_LOADED,
              POOL_UNLOADED,
              POOL_ALLOC_OK,
              POOL_ALLOC_FAILED
} MemPoolState;

typedef struct _DrvVersionType
{
    CHAR VersionStr[STRBUFSIZE];
    CHAR BuildStr[STRBUFSIZE];
} DrvVersionType;

typedef struct _DrvInputType
{
    size_t RequestBytes;
    PVOID UserVirtualAddr;
} DrvInputType;

typedef struct _DrvOutputType
{
    PVOID UserVirtualAddr;
    PVOID KernelVirtualAddr;
    PVOID PhysicalAddr;
    size_t MaxBytes;
    size_t BytesFree;
    size_t LargestBlock;
    DrvVersionType DrvVer;
    MemPoolState Status;
} DrvOutputType;

//memdriver subkeys

#define SUBKEY "System\\CurrentControlSet\\services\\memdrv"
#define HiMemPoolSizeKey "HiMemPoolSize"
#define LoMemPoolSizeKey "LoMemPoolSize"

#define LSUBKEY L"\\Registry\\Machine\\System\\CurrentControlSet\\services\\memdrv"
#define LHiMemPoolSizeKey L"HiMemPoolSize"
#define LLoMemPoolSizeKey L"LoMemPoolSize"

#define CFG_FILE    "MemDrv.cfg"


#endif
