// Merger_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Merger_Mb.h>
#include <list>
#include <ipp.h>
#include <DataTypes_Mb.h>
#include <Buffer_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <MemoryUtils_Mb.h>
#include <algorithm>

namespace   Innovative
{

//===========================================================================
//  CLASS MergerInstruction  --
//===========================================================================

class MergerInstruction
{

public:
    // Ctor
    virtual ~MergerInstruction() {}

    // Overrides
    virtual void    ProcessFirst(const ii32 * Input, ii32 * Output,
                                 int cannel_step, int Size, IppiSize RoiSize) = 0;
    virtual void    Process(ii32* Data, int Size) = 0;

    bool Instruction() const
        { return FInstruction; }
    void Instruction(bool value)
        { FInstruction = value; }

protected:
    // Data
    bool FInstruction;
};

//===========================================================================
//  CLASS MergerAssemblyInstruction  --
//===========================================================================

class MergerAssemblyInstruction : public MergerInstruction
{

public:
    // Ctor
    MergerAssemblyInstruction()
        { Instruction(true); }
    virtual ~MergerAssemblyInstruction() {}

    // Overrides
    virtual void    ProcessFirst(const ii32 * Input, ii32 * Output,
                                 int cannel_step, int Size, IppiSize RoiSize) = 0;
    virtual void    Process(ii32* Data, int Size) = 0;

};

//===========================================================================
//  CLASS MergerMacroInstruction  --
//===========================================================================

class MergerMacroInstruction : public MergerInstruction
{

public:
    // Ctor
    MergerMacroInstruction()
        { Instruction(false); }
    virtual ~MergerMacroInstruction() {}

    // Overrides
    virtual void    ProcessFirst(const ii32 * Input, ii32 * Output,
                                 int cannel_step, int Size, IppiSize RoiSize) = 0;
    virtual void    Process(ii32* Data, int Size) = 0;

};

//===========================================================================
//  CLASS InsertMergerMacroInstruction  --
//===========================================================================

class InsertMergerMacroInstruction : public MergerMacroInstruction
{
public:
    // Ctor
    InsertMergerMacroInstruction(int mask, int shift, int InputWordNo,
            int input_step, int OutputWordNo, int output_step) : mMask(mask),
             mShift(shift), mInputBufferStep(input_step), mInputWordNo(InputWordNo),
             mOutputWordNo(OutputWordNo), mOutputStep(output_step)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int /*channel_step*/,
                              int Size, IppiSize RoiSize)
    {
        Temp = (Ipp32s*)ippMalloc(sizeof(ii32) * Size);

        IppiSize        InputRoiSize;
        InputRoiSize.width = 1;
        InputRoiSize.height = Size;

        ippiAndC_32s_C1R((const Ipp32s*)(Input + mInputWordNo),
                         sizeof(ii32) * mInputBufferStep,
                         mMask,
                         Temp,
                         sizeof(ii32),
                         InputRoiSize);

        if (mShift > 0)
            ippsLShiftC_32s_I(mShift, Temp, Size);
        else
            ippsRShiftC_32s_I(-mShift, Temp, Size);

        int j = 0;
        for (int i = 0; i < RoiSize.height; i += mOutputStep)
            Output[mOutputWordNo + i] |= (ii32)Temp[j++];

        ippFree(Temp);
    }

    virtual void Process(ii32* /*Data*/, int /*Size*/) { }

protected:
    const ii32  mMask;
    const int   mShift;
    const int   mInputBufferStep;
    const int   mInputWordNo;
    const int   mOutputWordNo;
    const int   mOutputStep;
    Ipp32s        *Temp;
};


//===========================================================================
//  CLASS MergerChannelAssembly  --
//===========================================================================

class MergerChannelAssembly
{
public:
    // Ctor
    MergerChannelAssembly(MergerChannel*  channel, int word_no)
        : mChannel(channel), mWordNo(word_no)
    {
    }
    // Dtor
    virtual ~MergerChannelAssembly()
        {
        Clear();
        }

public:
    // Methods
    void    Process(Buffer input_buffer, Buffer&  output_buffer,
                    int channel_step, size_t output_buffer_step);

    void AddSeparationInstruction(MergerInstruction*  instruction)
    {
        mInstructions.push_back(instruction);
    }

    void Clear()
    {
        for (std::list < MergerInstruction * >::iterator Iter =
                 mInstructions.begin(); Iter != mInstructions.end(); Iter++)
            delete(*Iter);

        mInstructions.clear();
    }

//    int mTotalChannels;

protected:
    // Data
    std::list < MergerInstruction * > mInstructions;
    MergerChannel*      mChannel;
    const int           mWordNo;

    // Methods
    virtual void    InternalProcess(Buffer input_data,
                                    Buffer & output_data,
                                    int channel_step,
                                    size_t output_buffer_step);
};

//===========================================================================
//  CLASS MergerChannelAssemblies  --
//===========================================================================

class MergerChannelAssemblies
{
public:
    ~MergerChannelAssemblies();

    // Methods
    void Add(MergerChannelAssembly*  Assembly)
        {  mChannels.push_back(Assembly);  }

    void    Clear();

    size_t Count()
        {  return mChannels.size();  }

    MergerChannelAssembly * Assembly(int index)
        {  return mChannels[index];  }

    // Data
    int mChannelGroupStep;
    int mAllBits;

protected:
    std::vector < MergerChannelAssembly * > mChannels;
};


//===========================================================================
//  CLASS MergerChannelAssembly  --
//===========================================================================

//---------------------------------------------------------------------------
//  MergerChannelAssembly::Process() --
//---------------------------------------------------------------------------

void MergerChannelAssembly::Process(Buffer input_buffer,
                                    Buffer & output_buffer,
                                    int channel_step,
                                    size_t output_buffer_step)
{
    Buffer InputBuffer;
    IppiSize RoiSize;
    RoiSize.width = 1;
    int Bits = (mChannel->FBits < 9)?8:(mChannel->FBits < 17)?16:32;
    switch (Bits)
        {
        case 8:
            {
            UCharDG Temp(input_buffer);
            InputBuffer.Resize(Temp.SizeInElements());
            RoiSize.height = (static_cast<int>(Temp.SizeInElements()));
            ippiCopy_8u_C1R((const Ipp8u*)input_buffer.ConstData(), 1,
                            (Ipp8u*)InputBuffer.Data(), 4, RoiSize);
            }
            break;
        case 16:
            {
            UShortDG Temp(input_buffer);
            InputBuffer.Resize(Temp.SizeInElements());
            RoiSize.height = (static_cast<int>(Temp.SizeInElements()));
            ippiCopy_16u_C1R((const Ipp16u*)input_buffer.ConstData(), 2,
                            (Ipp16u*)InputBuffer.Data(), 4, RoiSize);
            }
            break;
        default:
            InputBuffer = input_buffer;
            break;
        }

    InternalProcess(InputBuffer, output_buffer, channel_step, output_buffer_step);
}

//---------------------------------------------------------------------------
//  MergerChannelAssembly::InternalProcess() --
//---------------------------------------------------------------------------

void MergerChannelAssembly::InternalProcess(Buffer input_data,
                                            Buffer & output_data,
                                            int channel_step,
                                            size_t output_buffer_step)
{
    bool            FirstInstruction = true;
    size_t          ChannelSize = output_data.SizeInInts();
    size_t          InputBufferSize = input_data.SizeInInts();

    const ii32*     InDataPtr = ((const ii32*)input_data.ConstData());

    ii32*           OutDataPtr = ((ii32*)output_data.Data()) + mChannel->FCursor;

    IppiSize        RoiSize;
    RoiSize.width = 1;
    RoiSize.height = static_cast<int>(output_buffer_step);

    mChannel->FCursor += output_buffer_step;
    if (mChannel->FCursor >= ChannelSize)
        mChannel->FCursor = 0;

    for (std::list < MergerInstruction * >::iterator Iter =
             mInstructions.begin(); Iter != mInstructions.end(); Iter++)
        {

        if ((*Iter)->Instruction())
            {
            //    int mParam = Iter->mParam;
            if (FirstInstruction)
                {

                // We have to perform a copy!
                FirstInstruction = false;
                (*Iter)->ProcessFirst(InDataPtr,
                                      OutDataPtr,
                                      channel_step * sizeof(ii32),
                                      static_cast<unsigned int>(ChannelSize),
                                      RoiSize);
                }

            else
                {

                // We already have a copy of the data, so just perform the operation over the output buffer.
                (*Iter)->Process(OutDataPtr, static_cast<unsigned int>(ChannelSize));
                }
            }
        else
            {
            FirstInstruction = false;
            (*Iter)->ProcessFirst(InDataPtr,
                             OutDataPtr,
                             channel_step * sizeof(ii32),
                             static_cast<unsigned int>(InputBufferSize),
                             RoiSize);
            }

        }

    if (FirstInstruction)
        {
        IppiSize    RoiSize;
        RoiSize.width = 1;
        RoiSize.height = static_cast<unsigned int>(ChannelSize);
        ippiLShiftC_32s_C1R((const Ipp32s*)InDataPtr,
                            channel_step * sizeof(ii32),
                            0,
                            (Ipp32s*)OutDataPtr,
                            sizeof(ii32),
                            RoiSize);
        }
}

//===========================================================================
//  CLASS MergerChannelAssemblies  --
//===========================================================================

//---------------------------------------------------------------------------
// MergerChannelAssemblies::~MergerChannelAssemblies() --
//---------------------------------------------------------------------------

MergerChannelAssemblies::~MergerChannelAssemblies()
{
    Clear();
}

//---------------------------------------------------------------------------
//  MergerChannelAssemblies::Clear() --
//---------------------------------------------------------------------------

void MergerChannelAssemblies::Clear()
{
    for (unsigned i = 0; i < mChannels.size(); i++)
        delete mChannels[i];

    mChannels.clear();
}

//===========================================================================
//  CLASS MergerChannels  --
//===========================================================================

//---------------------------------------------------------------------------
// MergerChannels::MergerChannels() --
//---------------------------------------------------------------------------

MergerChannels::MergerChannels(Merger&  owner) : mOwner(owner)
{
}

//---------------------------------------------------------------------------
//  MergerChannels::Clear() --
//---------------------------------------------------------------------------

void MergerChannels::Clear()
{
    for (unsigned i = 0; i < mChannels.size(); i++)
        delete mChannels[i];

    mChannels.clear();
}

//---------------------------------------------------------------------------
//  MergerChannels::AddBinary() --
//---------------------------------------------------------------------------

void MergerChannels::AddBinary(int bits, bool is_signed)
{
    BinaryMergerChannel*  Channel = new BinaryMergerChannel(mOwner, bits, is_signed);
    mChannels.push_back(Channel);
    Renumber();
}

//---------------------------------------------------------------------------
//  MergerChannels::AddNull() --
//---------------------------------------------------------------------------

void MergerChannels::AddNull(int bits)
{
    mChannels.push_back(new MergerChannel(mOwner,
                                          MergerChannel::ctNull,
                                          bits,
                                          0,
                                          0,
                                          false));
    Renumber();
}

//---------------------------------------------------------------------------
//  MergerChannels::InsertBinary() --
//---------------------------------------------------------------------------

void MergerChannels::InsertBinary(int index, int bits, bool is_signed)
{
    MergerChannel*    Channel = new BinaryMergerChannel(mOwner, bits, is_signed);
    mChannels.insert(mChannels.begin() + index, Channel);
    Renumber();
}

//---------------------------------------------------------------------------
//  MergerChannels::InsertNull() --
//---------------------------------------------------------------------------

void MergerChannels::InsertNull(int index, int bits)
{
    mChannels.insert(mChannels.begin() + index,
                     new MergerChannel( mOwner,
                                        MergerChannel::ctNull,
                                        bits,
                                        0,
                                        0,
                                        false));
    Renumber();
}

//---------------------------------------------------------------------------
//  MergerChannels::Delete() --
//---------------------------------------------------------------------------

void MergerChannels::Delete(int index)
{
    mChannels.erase(mChannels.begin() + index);
}

//---------------------------------------------------------------------------
//  MergerChannels::Renumber() --
//---------------------------------------------------------------------------

void MergerChannels::Renumber()
{
    for (size_t i = 0; i < mChannels.size(); ++i)
        mChannels[i]->Index(static_cast<int>(i));
}

//===========================================================================
//  CLASS BinaryMergerChannel  --
//===========================================================================

//---------------------------------------------------------------------------
// BinaryMergerChannel::BinaryMergerChannel() --
//---------------------------------------------------------------------------

BinaryMergerChannel::BinaryMergerChannel(Merger & owner,
    int bits, bool is_signed)
    : MergerChannel(owner, ctBinary, bits, 0, 0, is_signed)
{
}

//---------------------------------------------------------------------------
// BinaryMergerChannel::~BinaryMergerChannel() --
//---------------------------------------------------------------------------

BinaryMergerChannel::~BinaryMergerChannel()
{
}

//===========================================================================
//  CLASS MergerChannel  --
//===========================================================================

//---------------------------------------------------------------------------
//  MergerChannel::MergerChannel()  -- Ctor
//---------------------------------------------------------------------------

MergerChannel::MergerChannel(Merger & owner, ChannelType channel_type,
                        int bits, float offset, float slope, bool is_signed)
    : Owner(owner), FType(channel_type), FBits(bits), FOffset(offset),
    FSlope(slope), FSigned(is_signed), FIndex(0), FMerged(false), FCursor(0)
{
}

//---------------------------------------------------------------------------
//  MergerChannel::Type()  -- Set type of Merger channel
//---------------------------------------------------------------------------

void MergerChannel::Type(ChannelType value)
{
    FType = value;
}

//---------------------------------------------------------------------------
//  MergerChannel::Type()  -- Return type of Merger channel
//---------------------------------------------------------------------------

MergerChannel::ChannelType MergerChannel::Type() const
{
    return FType;
}

//---------------------------------------------------------------------------
//  MergerChannel::Bits()  --  Set bit width of channel
//---------------------------------------------------------------------------

void MergerChannel::Bits(int value)
{
    FBits = value;
}

//---------------------------------------------------------------------------
//  MergerChannel::Bits()  --  Get bit width of channel
//---------------------------------------------------------------------------

int MergerChannel::Bits() const
{
    return FBits;
}

//---------------------------------------------------------------------------
//  MergerChannel::Offset()  --
//---------------------------------------------------------------------------

void MergerChannel::Offset(float value)
{
    FOffset = value;
}

//---------------------------------------------------------------------------
//  MergerChannel::Offset()  --
//---------------------------------------------------------------------------

float MergerChannel::Offset() const
{
    return FOffset;
}

//---------------------------------------------------------------------------
//  MergerChannel::Slope()  --
//---------------------------------------------------------------------------

void MergerChannel::Slope(float value)
{
    FSlope = value;
}

//---------------------------------------------------------------------------
//  MergerChannel::Slope()  --
//---------------------------------------------------------------------------

float MergerChannel::Slope() const
{
    return FSlope;
}

//---------------------------------------------------------------------------
//  MergerChannel::Signed()  --
//---------------------------------------------------------------------------

void MergerChannel::Signed(bool value)
{
    FSigned = value;
}

//---------------------------------------------------------------------------
//  MergerChannel::Signed()  --
//---------------------------------------------------------------------------

bool MergerChannel::Signed() const
{
    return FSigned;
}

//---------------------------------------------------------------------------
//  MergerChannel::Index()  --
//---------------------------------------------------------------------------

int MergerChannel::Index() const
{
    return FIndex;
}

//---------------------------------------------------------------------------
//  MergerChannel::Index()  --
//---------------------------------------------------------------------------

void MergerChannel::Index(int idx)
{
    FIndex = idx;
}

//---------------------------------------------------------------------------
//  MergerChannel::Merged()  --
//---------------------------------------------------------------------------

void MergerChannel::Merged(bool value)
{
    FMerged = value;
}

//---------------------------------------------------------------------------
//  MergerChannel::Merged()  --
//---------------------------------------------------------------------------

bool MergerChannel::Merged() const
{
    return FMerged;
}

//---------------------------------------------------------------------------
// Merger::Merger() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif
Merger::Merger() : Channels(*this), mRunning(false), OutputChannel(*this),
                   TempOutput(NULL), FMergeStarted(false), mOutputBufferStep(0),
                   FCursor(0), mFirstChannel(-1)
{
    mChannelAssemblies = new MergerChannelAssemblies;
}
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// Merger::~Merger() --
//---------------------------------------------------------------------------

Merger::~Merger()
{
    delete mChannelAssemblies;
    mChannelAssemblies = 0;
}

//---------------------------------------------------------------------------
//  Merger::Start() --
//---------------------------------------------------------------------------

void Merger::Start()
{
    if (mRunning)
        return;

    mRunning = true;

    FMergeStarted = false;
    FCursor = 0;
    mFirstChannel = -1;
    if (TempOutput)
        delete TempOutput;
    TempOutput = new Buffer;
    BufferSize(Output.SizeInInts());
    mChannelAssemblies->Clear();
    mChannelAssemblies->mChannelGroupStep = 1;

    const int           BitsPerWord = sizeof(ii32) * 8;
    int                 CurrentChannelOffset = 0;
    int                 AllBits = 0;
    int                 WordNo = 0;
    int                 PatternStepSize = 0;
    int                 EffectiveAllBits = 0;
    std::vector<int>    Shift;
    std::vector<int>    PatternWordNumber;

    for (int i = 0; i < Channels.Count(); i++)
        {
        int Bits = Channels[i].Bits();
        Bits = (Bits < 9)?8:(Bits < 17)?16:32;
        EffectiveAllBits += Bits;
        }

    do
        {
        Shift.push_back(PatternStepSize % 32);
        PatternWordNumber.push_back((PatternStepSize) / (BitsPerWord));
        PatternStepSize += EffectiveAllBits;
        } while (PatternStepSize % 32);

    int OutputOffset = (PatternStepSize + (BitsPerWord - 1)) / (BitsPerWord);

    for (int i = 0; i < Channels.Count(); i++)
        {
        MergerChannel & Channel = Channels[i];

        int                         Bits = Channel.Bits();
        MergerChannelAssembly*    Assembly = NULL;

        int                         EffectiveBits;

        EffectiveBits = (Bits < 9)?8:(Bits < 17)?16:32;

        if (Channel.Type() == MergerChannel::ctBinary)
            Assembly = new MergerChannelAssembly(&Channel, WordNo);

        if (Assembly && Bits)
            {
            for (int j = 0; j < (int)Shift.size(); j++)
                {
                int CurrentWordNumber = WordNo;
                int ChannelOffset = CurrentChannelOffset + Shift[j];

                if (ChannelOffset >= BitsPerWord)
                    {
                    ChannelOffset -= BitsPerWord;
                    CurrentWordNumber++;
                    }

                if (ChannelOffset + Bits > BitsPerWord)
                    {
                    int     FirstWordBits = (BitsPerWord) - ChannelOffset;
                    ii32    LowerMask = ~(0xFFFFFFFF << FirstWordBits);

                    Assembly->AddSeparationInstruction(
                            new InsertMergerMacroInstruction(LowerMask,
                                                             ChannelOffset,
                                                             j,
                                                             static_cast<int>(Shift.size()),
                                                             PatternWordNumber[j] + CurrentWordNumber,
                                                             OutputOffset));

                    ii32    HigherMask = (Bits==32)?0xFFFFFFFF:~(0xFFFFFFFF << Bits);
                    HigherMask = (HigherMask & (~LowerMask));

                    Assembly->AddSeparationInstruction(
                            new InsertMergerMacroInstruction(HigherMask,
                                                             -FirstWordBits,
                                                             j,
                                                             static_cast<int>(Shift.size()),
                                                             PatternWordNumber[j] + CurrentWordNumber,
                                                             OutputOffset));

                }
                else if ((Bits - ChannelOffset) <= BitsPerWord)
                    {
                    ii32    Mask = (Bits==32)?0xFFFFFFFF:~(0xFFFFFFFF << Bits);

                    Assembly->AddSeparationInstruction(
                            new InsertMergerMacroInstruction(Mask,
                                                             ChannelOffset,
                                                             j,
                                                             static_cast<int>(Shift.size()),
                                                             PatternWordNumber[j] + CurrentWordNumber,
                                                             OutputOffset));

                    }
                }
            mChannelAssemblies->Add(Assembly);
            }
        else
            mChannelAssemblies->Add(NULL);

        CurrentChannelOffset += EffectiveBits;
        AllBits += Bits;
        if (CurrentChannelOffset >= BitsPerWord)
            {
            CurrentChannelOffset -= BitsPerWord;
            WordNo++;
            }
        }
    mChannelAssemblies->mAllBits = EffectiveAllBits;
    mChannelAssemblies->mChannelGroupStep = (AllBits + (BitsPerWord - 1)) / (BitsPerWord);
}

//---------------------------------------------------------------------------
//  Merger::Stop() --
//---------------------------------------------------------------------------

void Merger::Stop()
{
    if (!mRunning)
        return;

    mRunning = false;
    mChannelAssemblies->Clear();
}

//---------------------------------------------------------------------------
//  Merger::Merge() --
//---------------------------------------------------------------------------

void Merger::Merge(const Buffer & input, int channel_index)
{
    if (!mRunning)
        Start();

    if (mChannelAssemblies->Assembly(channel_index) == NULL)
        return;

    if (mFirstChannel < 0)
        mFirstChannel = channel_index;

    if (mFirstChannel == channel_index)
        {
        int BytesPerElement = (Channels[channel_index].Bits() < 9)?1:(Channels[channel_index].Bits() < 17)?2:4;
        size_t InputBufferSize = input.SizeInBytes()/BytesPerElement;
        size_t sz = static_cast<size_t>((InputBufferSize * mChannelAssemblies->mAllBits) / 32);

        Buffer Temp;

        if (TempOutput->SizeInInts())
            {
            Temp.Resize(TempOutput->SizeInInts());
            IppiSize RoiSize;
            RoiSize.width = 1;
            RoiSize.height = static_cast<int>(TempOutput->SizeInInts());
            ippiCopy_32s_C1R((const Ipp32s *)TempOutput->ConstData(), 4, (Ipp32s *)Temp.Data(), 4, RoiSize);
            }
        TempOutput->Resize(Temp.SizeInInts() + sz);
        ippsZero_32s((Ipp32s *)TempOutput->Data(), static_cast<int>(TempOutput->SizeInInts()));
        if (Temp.SizeInInts())
            {
            IppiSize RoiSize;
            RoiSize.width = 1;
            RoiSize.height = static_cast<int>(Temp.SizeInInts());
            ippiCopy_32s_C1R((const Ipp32s *)Temp.ConstData(), 4, (Ipp32s *)TempOutput->Data(), 4, RoiSize);
            }

        mOutputBufferStep = sz;
        }
    else if (!FMergeStarted)
        {
        FMergeStarted = true;
        Channels[mFirstChannel].FCursor = 0;
        Channels[mFirstChannel].Merged(true);
        }

    mChannelAssemblies->Assembly(channel_index)->Process(input,
                                                         *TempOutput,
                                                         mChannelAssemblies->mChannelGroupStep,
                                                         mOutputBufferStep);

    if ((mFirstChannel == channel_index) && ((TempOutput->SizeInInts() < Output.SizeInInts()) || (Channels.Count() > 1)))
        Channels[channel_index].FCursor = TempOutput->SizeInInts();

    if (!Channels[channel_index].FCursor)
        Channels[channel_index].Merged(true);

    int mMergedChannels = 0;
    for (int i = 0; i < Channels.Count(); i++)
        mMergedChannels += static_cast<int>(Channels[i].Merged());

    if (mMergedChannels >= Channels.Count())
        {
        mFirstChannel = -1;
        FMergeStarted = false;
        const unsigned char * src = reinterpret_cast<const unsigned char*>(TempOutput->ConstData());
        unsigned char * dst = reinterpret_cast<unsigned char*>(Output.Data());

        // Copy input to output buffer
        size_t available = TempOutput->SizeInBytes();
        size_t room = Output.SizeInBytes();

        // If sizes of I/O buffers equal, avoid copy
        if (room == available)
            {
            SendBufferEvent e(*TempOutput);
            OnMerge.Execute(e);
            }
        else
            {
            while (available)
                {
                size_t residual = room - FCursor;
                size_t chunk = std::min(available, residual);
                CopyByteBuffer (src, &dst[FCursor], chunk);

                FCursor += chunk;
                src += chunk;
                available -= chunk;

                // Signal if output buffer is full
                if (FCursor == room)
                    {
                    SendBufferEvent e(Output);
                    OnMerge.Execute(e);

                    FCursor = 0;
                    ippsZero_32s((Ipp32s *)Output.Data(), static_cast<int>(Output.SizeInInts()));
                    }
                }
            }

        for (int i = 0; i < Channels.Count(); i++)
            Channels[i].Merged(false);
        delete TempOutput;
        TempOutput = new Buffer;
        }
}

//---------------------------------------------------------------------------
//  Merger::BufferSize() --
//---------------------------------------------------------------------------

void Merger::BufferSize(size_t size_in_ints)
{
    Output.Resize(size_in_ints);
    ippsZero_32s((Ipp32s *)Output.Data(), static_cast<int>(size_in_ints));
}

} // namespace Innovative


