// Merger_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Merger_MbH
#define Merger_MbH

#include <IppFloatDG_Mb.h>
#include <IppIntegerDG_Mb.h>
#include <list>
#include <ProcessEvents_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Merger;
class MergerChannel;

//===========================================================================
//  CLASS MergerChannel  --
//===========================================================================

class MergerChannel
{
    friend class MergerChannelAssembly;
    friend class Merger;

public:
    enum ChannelType { ctBinary, ctNull };

    // Ctor
    MergerChannel( Merger &owner, ChannelType channel_type,
        int bits, float offset, float slope, bool is_signed );
    virtual ~MergerChannel(){}

    // Methods
    // Properties
    void        Type(const ChannelType type);
    ChannelType Type() const;

    void        Bits(int value);
    int         Bits() const;

    void        Offset(float value);
    float       Offset() const;

    void        IntOffset(int value);
    int         IntOffset() const;

    void        Slope(float value);
    float       Slope() const;

    void        Signed(bool value);
    bool        Signed() const;

    void        Index(int idx);
    int         Index() const;

    void        Merged(bool value);
    bool        Merged() const;

protected:
    Merger &        Owner;
    ChannelType     FType;
    int             FBits;
    int             FIntOffset;
    float           FOffset;
    float           FSlope;
    bool            FSigned;
    int             FIndex;
    bool            FMerged;
    size_t          FCursor;
};

//===========================================================================
//  CLASS BinaryMergerChannel  --
//===========================================================================

class BinaryMergerChannel : public MergerChannel
{
public:
    BinaryMergerChannel( Merger &owner, int bits, bool is_signed);
    ~BinaryMergerChannel();
};

//===========================================================================
//  CLASS MergerChannels  --
//===========================================================================

class MergerChannels
{
public:
    // Ctor
    MergerChannels( Merger &mOwner );
    ~MergerChannels() { Clear(); }

    // Methods
    void Clear();

    void AddBinary( int bits, bool is_signed );
    void AddNull( int bits );

    void InsertBinary( int index, int bits, bool is_signed );
    void InsertNull( int index, int bits );

    void Delete( int index );

    // Access
    MergerChannel &operator [] (int index)
        {  return *( mChannels[index] );  }
    int Count() { return (int)mChannels.size(); }

protected:
    Merger          &mOwner;

    std::vector<MergerChannel *> mChannels;

    // Methods
    void Renumber();

};

class MergerChannelAssemblies;

//===========================================================================
//  CLASS Merger  --
//===========================================================================

class Merger
{
public:
    Merger();
    virtual ~Merger();

    // Methods
    void Start();
    void Merge(const Buffer & input, int channel_index);

    void        BufferSize(size_t size_in_ints);
    size_t      BufferSize()
        {   return Output.SizeInInts(); }

    MergerChannels  Channels;

    virtual void Stop();

    OpenWire::EventHandler<Innovative::SendBufferEvent> OnMerge;

protected:
    bool            mRunning;
    int             mTotalInputWordNo;
    MergerChannels  OutputChannel;
    Buffer          Output;
    Buffer          *TempOutput;

    bool            FMergeStarted;
    size_t          mOutputBufferStep;
    size_t          FCursor;
    int             mFirstChannel;

    MergerChannelAssemblies *mChannelAssemblies;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
