// MovingAverage_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef MovingAverage_MbH
#define MovingAverage_MbH

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS MovingAverage  -- Simple low-pass filtered rate data
//===========================================================================

class MovingAverage
{
public:
    MovingAverage(unsigned int taps=8)
        : FTaps(taps), History(taps), Cursor(0), Sum(0), Last(0)
    {  Reset();  }

    // Methods
    void   Reset()
    {
        std::fill(History.begin(), History.end(), 0.0);
        Cursor = 0;
        FSettled = false;
        Sum = 0.0;
    }

    double Process(double sample)
    {
        Sum -= History[Cursor];
        Sum += sample;
        History[Cursor++] = sample;
        if (Cursor == FTaps)
            {
            Cursor = 0;
            FSettled = true;
            }

        Last = Sum/FTaps;
        return Last;
    }

    bool Settled() const
        {  return FSettled;  }
    unsigned int Length() const
        {  return FTaps;  }

private:
    // Fields
    unsigned int			FTaps;
    bool                    FSettled;

    // Data
    std::vector<double>		History;
    unsigned int			Cursor;
    double                  Sum;
    double                  Last;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative

#endif

