//---------------------------------------------------------------------------
// MultiChanBoard_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <MultiChanBoard_Mb.h>
#include <DeviceDriverBmManager_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS MultiChanBoard  -- MultiChan Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class MultiChanBoard
//---------------------------------------------------------------------------

MultiChanBoard::MultiChanBoard(MultiChanDeviceMap & deviceMap)
    :  IIsAMultiChanStreamBoard(deviceMap),
       MultiChanDeviceRef(deviceMap)
{
}

//---------------------------------------------------------------------------
//  destructor for class MultiChanBoard
//---------------------------------------------------------------------------

MultiChanBoard::~MultiChanBoard()
{
    // Don't call anything using MultiChanDeviceRef here - put in most derived class
}

//---------------------------------------------------------------------------
//  MultiChanBoard::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t MultiChanBoard::BoardCount()
{
    return MultiChanDeviceRef.BoardCount();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::FpgaName() -- FPGA type name function
//---------------------------------------------------------------------------

std::string  MultiChanBoard::FpgaName(const short number)
{
    switch(static_cast<int>(number))
        {
        case 0:  return "LX5550T";
        case 1:  return "LX240T";
        case 2:  return "SX315T";
        case 3:  return "SX475T";
        default: return "Unknown";
        }
}

//---------------------------------------------------------------------------
//  MultiChanBoard::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  MultiChanBoard::LicenseCheck(unsigned int & /*code*/)
{
    return true;
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  MultiChanBoard::Target()
{
    return MultiChanDeviceRef.Target();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  MultiChanBoard::Target( int value )
{
    MultiChanDeviceRef.Target(value);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  MultiChanBoard::Led()
{
    return MultiChanDeviceRef.Led();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  MultiChanBoard::Led( bool value )
{
    MultiChanDeviceRef.Led(value);
}

////---------------------------------------------------------------------------
////  MultiChanBoard::GreenLed() -- Read the Green LED state
////---------------------------------------------------------------------------

//bool  MultiChanBoard::GreenLed()
//{
//    return MultiChanDeviceRef.Periph_SDev.GreenLed();
//}

////---------------------------------------------------------------------------
////  MultiChanBoard::GreenLed() -- Set the Green LED state of the board
////---------------------------------------------------------------------------

//void  MultiChanBoard::GreenLed( bool value )
//{
//    MultiChanDeviceRef.Periph_SDev.GreenLed(value);
//}

////---------------------------------------------------------------------------
////  MultiChanBoard::FrontPanelLed() -- Read the FrontPanel LED state
////---------------------------------------------------------------------------

//bool  MultiChanBoard::FrontPanelLed()
//{
//    return MultiChanDeviceRef.Periph_SDev.FrontPanelLed();
//}

////---------------------------------------------------------------------------
////  MultiChanBoard::FrontPanelLed() -- Set the FrontPanel LED state of the board
////---------------------------------------------------------------------------

//void  MultiChanBoard::FrontPanelLed( bool value )
//{
//    MultiChanDeviceRef.Periph_SDev.FrontPanelLed(value);
//}

//---------------------------------------------------------------------------
//  MultiChanBoard::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  MultiChanBoard::Open()
{
    MultiChanDeviceRef.Open();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  MultiChanBoard::Close()
{
    MultiChanDeviceRef.Close();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::EnableExtInterrupt() -- enable external INTs notification
//---------------------------------------------------------------------------

void MultiChanBoard::EnableExtInterrupt(int ext_int, MultiChanBoard::ExtIntTypes which, bool state)
{
    MultiChanDeviceRef.BmMgr().EnableExtInterrupt(ext_int,
                                                  static_cast<InnovativeKernel::IMcbmHardwareManager::ExtIntTypes>(which),
                                                  state);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::EnableStreamLoopback() -- XU Stream Loopback
//---------------------------------------------------------------------------

void MultiChanBoard::EnableStreamLoopback(bool state)
{
    MultiChanDeviceRef.BmMgr().EnableChannelLoopback(state);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Internals() -- Internal Packet Stream Interface
//---------------------------------------------------------------------------

IMultiChanStreamDeviceInternals *  MultiChanBoard::Internals()
{
    return &MultiChanDeviceRef;
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  MultiChanBoard::Log( int type, int value )
{
    MultiChanDeviceRef.Log(type, value);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::BmIn() -- Return reference to Incoming BM Channel Config
//---------------------------------------------------------------------------

IMultiChanConfiguration & MultiChanBoard::BmIn()
{
    return MultiChanDeviceRef.BmIn();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::BmOut() -- Return reference to Outgoing BM Channel Config
//---------------------------------------------------------------------------

IMultiChanConfiguration & MultiChanBoard::BmOut()
{
    return MultiChanDeviceRef.BmOut();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  MultiChanBoard::PeekLogic(int offset )
{
    return MultiChanDeviceRef.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  MultiChanBoard::PokeLogic( int offset, ii32 value )
{
    MultiChanDeviceRef.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  MultiChanBoard::PeekPort(int offset )
{
    return MultiChanDeviceRef.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  MultiChanBoard::PokePort( int offset, ii32 value )
{
    MultiChanDeviceRef.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::LinkRate() -- Get Link Rate
//---------------------------------------------------------------------------

int MultiChanBoard::LinkRate()
{
    return MultiChanDeviceRef.LinkRate();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::LinkWidth() -- Get Link Width
//---------------------------------------------------------------------------

int MultiChanBoard::LinkWidth()
{
    return MultiChanDeviceRef.LinkWidth();
}

//---------------------------------------------------------------------------
//  MultiChanBoard::AfterFiclStart() --
//---------------------------------------------------------------------------

void  MultiChanBoard::AfterFiclStart(FiclSystem & engine)
{
    //  Replace Callbacks to call FICL too
    MultiChanDeviceRef.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  MultiChanBoard::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  MultiChanBoard::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  MultiChanBoard::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  MultiChanBoard::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  MultiChanBoard::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
