// MultiChanBoard_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef MultiChanBoard_MbH
#define MultiChanBoard_MbH

//#include <DataTypes_Mb.h>
#include <MultiChanDeviceMap_Mb.h>
#include <MultiChanStreamSupport_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS MultiChanBoard  -- MultiChan Common baseboard class
//===========================================================================

class MultiChanBoard : public IIsAMultiChanStreamBoard,
                public IStreamDeviceEvents,
                public IFiclTarget
{
    enum  ExtIntTypes  { meiRisingEdge, meiFallingEdge, meiBoth };

public:
    //
    //  Member Functions
    MultiChanBoard(MultiChanDeviceMap & deviceMap);
    virtual ~MultiChanBoard();

    //  Informational Methods
    size_t                      BoardCount();
	bool                        LicenseCheck(unsigned int & code);
	
    int                         Target();
    void                        Target(int value);

    bool                        Led();
    void                        Led(bool value);

    std::string                 FpgaName(const short number);

    virtual void                Open();
    virtual void                Close();

    void   EnableExtInterrupt(int ext_int, MultiChanBoard::ExtIntTypes which, bool state);
    void   EnableStreamLoopback(bool state);

    ii32   PeekLogic( int offset );
    void   PokeLogic( int offset, ii32 value );
    ii32   PeekPort( int offset );
    void   PokePort( int offset, ii32 value );

    //  Link Status
    int   LinkRate();   // return X as in Gen1, GenX.
    int   LinkWidth();  // return 0 if unlinked

    //  SubDevices
    IMultiChanConfiguration &   BmIn();
    IMultiChanConfiguration &   BmOut();

public: //Pseudo protected
    //
    //  IMultiChanStreamDevice interface
    virtual IMultiChanStreamDeviceInternals *  Internals();

    MultiChanDeviceMap * Debug() { return &MultiChanDeviceRef; }
    virtual void   Log(int type, int data);


protected:
    //
    //  Member Data
    MultiChanDeviceMap &                  MultiChanDeviceRef;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( MultiChanBoard & board );
    friend AddressingSpace & PortMemorySpace( MultiChanBoard & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);


};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( MultiChanBoard & board )
{
    return board.MultiChanDeviceRef.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( MultiChanBoard & board )
{
    return board.MultiChanDeviceRef.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

