//  MultiChanDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <MultiChanDeviceMap_Mb.h>
#include <MultiChanBoard_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS MultiChanDeviceMap -- General Control of a MultiChan Device
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class MultiChanDeviceMap
//---------------------------------------------------------------------------

MultiChanDeviceMap::MultiChanDeviceMap(MultiChanBoard * owner, InnovativeKernel::MultiChannelBMDeviceDriver * driver )
    :  IUsesStreamDeviceEventCallbacks(owner), DeviceDriver(driver),
       IntScheme(this),
       DebugLog(10000), ShouldPreloadXmit(true),
       SingleTimeAtStartPreconfig(true)
{
}

//---------------------------------------------------------------------------
//  destructor for class MultiChanDeviceMap
//---------------------------------------------------------------------------

MultiChanDeviceMap::~MultiChanDeviceMap()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::IsOpened()
//---------------------------------------------------------------------------

bool  MultiChanDeviceMap::IsOpened()
{
    return DeviceDriver->Opened();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::Open()
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::Open()
{
    Debug.Log(" +++ MultiChanDeviceMap::Open()");

    if ( IsOpened() )
        return;

    AtBeforeOpenTime();

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));
    LogicMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->LogicMemoryAddress()));

    AtAfterOpenTime();
    Debug.Log(" --- MultiChanDeviceMap::Open()");

}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::Close()
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::Close()
{
    Debug.Log(" +++ MultiChanDeviceMap::Close()");
	AtBeforeCloseTime();
    DeviceDriver->Close();
    AtAfterCloseTime();
    Debug.Log(" --- MultiChanDeviceMap::Close()");
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t MultiChanDeviceMap::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::LicenseCheck()  --  Quick check WD License
//---------------------------------------------------------------------------

bool  MultiChanDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
    return true;
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BoardId()  --  Get reference to device driver
//---------------------------------------------------------------------------

InnovativeKernel::MultiChannelBMDeviceDriver & MultiChanDeviceMap::Driver()
{
    return *DeviceDriver;
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BmMgr()  --  Get reference to Manager Class
//---------------------------------------------------------------------------

InnovativeKernel::IMcbmHardwareManager & MultiChanDeviceMap::BmMgr()
{
    return DeviceDriver->Hdw()   ;
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  MultiChanDeviceMap::Target()
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::Led()
//---------------------------------------------------------------------------

bool  MultiChanDeviceMap::Led()
{
    return DeviceDriver->Hdw().Led();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::Led()
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::Led( bool value )
{
    DeviceDriver->Hdw().Led(value);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BusmasteringOn() -- Turn On Busmastering
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::BusmasteringOn()
{
    if (!DeviceDriver->Opened())
        return;

    DeviceDriver->Hdw().BusmasteringOn();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BusmasteringOff() -- Turn Off Busmastering
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::BusmasteringOff()
{
    if (!DeviceDriver->Opened())
        return;

    DeviceDriver->Hdw().BusmasteringOff();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BusmasterReset()
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::BusmasterReset()
{
    //
    //  Make sure BM is off...
    BusmasteringOff();
    Sleep(10);

    DeviceDriver->Hdw().BusmasterReset();
    //
    //  Clear Int Status queue
    DeviceDriver->ClearInterruptStatusQueue();

    DebugLog.clear();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::MasterReset()
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::MasterReset()
{
    DeviceDriver->Hdw().MasterReset();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::EnableBusmasterInterrupt()
//---------------------------------------------------------------------------

void MultiChanDeviceMap::EnableBusmasterInterrupt( bool enable )
{
    DeviceDriver->Hdw().EnableBusmasterInterrupt(enable);
}

//  Subdevices...
//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BmIn() -- Return reference to Incoming BM Channel Config
//---------------------------------------------------------------------------

IMultiChanConfiguration & MultiChanDeviceMap::BmIn()
{
    return DeviceDriver->BmIn();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BmOut() -- Return reference to Outgoing BM Channel Config
//---------------------------------------------------------------------------

IMultiChanConfiguration & MultiChanDeviceMap::BmOut()
{
    return DeviceDriver->BmOut();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BmInInit() -- Return reference to In Write Config
//---------------------------------------------------------------------------

IMultiChanInitialization & MultiChanDeviceMap::BmInInit()
{
    return DeviceDriver->BmInInit();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::BmOutInit() -- Return reference to In Write Config
//---------------------------------------------------------------------------

IMultiChanInitialization & MultiChanDeviceMap::BmOutInit()
{
    return DeviceDriver->BmOutInit();
}

//
//  Normal System "TIMES" -
//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtConstructCustomPartTime() -
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtConstructCustomPartTime()
{
    ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtBeforeOpenTime() -
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtBeforeOpenTime()
{
    PreOpenHardware();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtAfterOpenTime() -
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtAfterOpenTime()
{
    Debug.Log(" +++ MultiChanDeviceMap::AtAfterOpenTime()");

    Debug.Log("....Open Scheme");
    IntScheme.Open();

    Debug.Log("....MasterReset()");
    MasterReset();
    \
    Debug.Log("Calling down OpenHardware()");
    OpenHardware();
    Debug.Log(" --- MultiChanDeviceMap::AtAfterOpenTime()");
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtBeforeCloseTime() - before device driver closes
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtBeforeCloseTime()
{
    //CloseHardware();
    //EnableCustomInterrupt(false);
    IntScheme.Close();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtAfterCloseTime() - after device driver closes
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtAfterCloseTime()
{

}

//
//  Stream/Configuration "Time" Activations
//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtPreconfigureTime() - Stream Callover @ Preconfig
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtPreconfigureTime()
{
    //  Allow Derived Classes to Add Code
    PreconfigureHardware();

    //  Forward notify to baseboard handler
    OpenWire::NotifyEvent e;
    OwnerCB()->CallOnStreamPreconfig(e);

    // "activate" one time at-start preconfiguration
    SingleTimeAtStartPreconfig = true;
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtStreamConfigureTime() - Start of Stream Process
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtStreamConfigureTime()
{
    //
    //  Begin Busmastering Init Sequence with a reset
    BusmasterReset();
    //
    //  Set Notification "Packet Sizes" to Hardware
    DeviceDriver->Hdw().ApplyBusmasterNotifySizes();
    //
    //  Initial Device Configuration
    IntScheme.ConfigureBusmastering();  // AtStreamConfigureTime() now?

    //  Allow Derived Classes to Add Code
    ConfigureHardware();

    // Forward notification to Board Object event
    OpenWire::NotifyEvent e;
    OwnerCB()->CallOnStreamConfigure(e);

    // Run this once, since last Preconfigure time
    if (SingleTimeAtStartPreconfig)
        {
        //  Allow Derived Classes to Add Code
        OnSingleStreamAtStartConfig();

        //  Forward notify to baseboard handler
        OwnerCB()->CallOnStreamSingleTimeAtStartConfig(e);

        SingleTimeAtStartPreconfig = false;
        }

}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtStreamPrefillTime() - Just after Prefill
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtStreamPrefillTime()
{
    if (ShouldPreloadXmit)
        {
        IntScheme.PreloadXmitBusmasterRegion();
        }
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtStreamStartTime() - Starting Streaming
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtStreamStartTime()
{
    Debug.Log(" +++ MultiChanDeviceMap::AtStreamStartTime()");

    //DeviceInternals()->StartBusmastering();
    //  Allow Derived Classes to Add Code
    OnBeforeStreamStart();

    //  Forward notify to baseboard handler
    OpenWire::NotifyEvent e;
    OwnerCB()->CallOnBeforeStreamStart(e);

    EnableBusmasterInterrupt(true);

    // Grant Initial Credit
    DeviceDriver->Hdw().IssueInitialRecvCredit();
    //
    //  Begin full duplex busmastering
    BusmasteringOn();

    //  Allow Derived Classes to Add Code
    OnAfterStreamStart();
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(e);

    Debug.Log(" --- MultiChanDeviceMap::AtStreamStartTime()");

}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtStreamStopTime()
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::AtStreamStopTime()
{
    //  Allow Derived Classes to Add Code
    OnBeforeStreamStop();

    //  Forward notify to baseboard handler
    OpenWire::NotifyEvent e;
    OwnerCB()->CallOnBeforeStreamStop(e);
    //
    //  Begin shutting down the PCI transmissions
    BusmasteringOff();
    //
    //  Stream Event Callback
    StreamStopHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(e);

}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::TxChannelInformation() - get channel info
//---------------------------------------------------------------------------

bool  MultiChanDeviceMap::TxMcChannelActive(int ch)
{
    return BmOut().McChannelActive(ch);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::TxChannelInformation() - get channel info
//---------------------------------------------------------------------------

McImageInformation MultiChanDeviceMap::TxChannelInformation(size_t channel)
{
    return IntScheme.TxChannelInformation(channel);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::TxGetEmptyImage() - get image to fill
//---------------------------------------------------------------------------

McImage MultiChanDeviceMap::TxGetEmptyImage(size_t channel, size_t desired_size)
{
    return IntScheme.TxGetEmptyImage(channel, desired_size);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::TransferFilledImage() - "Send" filled image to board
//---------------------------------------------------------------------------

void MultiChanDeviceMap::TransferFilledImage(McImage& image)
{
    IntScheme.TransferFilledImage(image);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::ForwardPacketTransmitAck() - "Send" filled image to board
//---------------------------------------------------------------------------

void MultiChanDeviceMap::ForwardPacketTransmitAck(McHintPack& OutgoingHints)
{
    IntScheme.ForwardPacketTransmitAck(OutgoingHints);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtDataTransmitTime() - Data Transmission from Stream
//---------------------------------------------------------------------------

void MultiChanDeviceMap::AtDataTransmitTime(unsigned int channel)
{
    IntScheme.TransmitDataAvailable(channel);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::AtDataReceivedTime() - Data Transmission from Stream
//---------------------------------------------------------------------------
//  Packet has been received.  Credit can now be given by VisrThread

void MultiChanDeviceMap::AtDataReceivedTime()
{
    IntScheme.ReceiveDataAcknowledged();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::VisrIsrQDepth() -- return depth of ISR queue
//---------------------------------------------------------------------------

unsigned int  MultiChanDeviceMap::VisrIsrQDepth()
{
    return IntScheme.VisrIsrQDepth();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::TraceLog() -- process custom interrupt
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::TraceLog(int type, int data)
{
    Log(type, data);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::ConstructCustomParts() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::ConstructCustomParts()
{
}
\
//---------------------------------------------------------------------------
//  MultiChanDeviceMap::PreOpenHardware() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::PreOpenHardware()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::OpenHardware() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::OpenHardware()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::PreconfigureHardware()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::ConfigureHardware() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::ConfigureHardware()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::OnSingleStreamAtStartConfig() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::OnSingleStreamAtStartConfig()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::OnBeforeStreamStart() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::OnBeforeStreamStart()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::OnAfterStreamStart() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::OnAfterStreamStart()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::OnBeforeStreamStop() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::OnBeforeStreamStop()
{
}
\
//---------------------------------------------------------------------------
//  MultiChanDeviceMap::StreamStopHardware() --
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::StreamStopHardware()
{
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::IsrEvent() - Get event for the VISR
//---------------------------------------------------------------------------

Event *  MultiChanDeviceMap::IsrEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::InterruptDepth() - Get depth of VISR queue
//---------------------------------------------------------------------------

size_t MultiChanDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::InterruptAck() - Acknowledge interrupt
//---------------------------------------------------------------------------

int MultiChanDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::RegisterStreamPlugIn() -- attach to a streaming object
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::RegisterStreamPlugIn( IMultiChanStreamManagerInternals * plug_in )
{
    if (plug_in)
        {
        //PacketDevice->InstallStreamPlugin( new PacketDevMap::PacketDevicePluginImpl(this) );
        StreamManagerInternals.Link(plug_in);
        }
    else
        StreamManagerInternals.Unlink();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::FillRecvRegion() -- fill busmaster region with pattern
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::FillRecvRegion(size_t chan, unsigned int pattern, bool ramp)
{
    IntScheme.FillRecvRegion(chan, pattern, ramp);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::FillXmitRegion() -- fill busmaster region with pattern
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::FillXmitRegion(size_t chan, unsigned int pattern, bool ramp)
{
    IntScheme.FillXmitRegion(chan, pattern, ramp);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32   MultiChanDeviceMap::PeekPort(int offset)
{
    return PortMemory.At(offset);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::PokePort(int offset, ii32 value)
{
    PortMemory.At(offset, value);
}


//---------------------------------------------------------------------------
//  MultiChanDeviceMap::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32  MultiChanDeviceMap::PeekLogic(int offset)
{
    return LogicMemory.At(offset);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::PokeLogic(int offset, ii32 value)
{
    LogicMemory.At(offset, value);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::LinkRate() - Link "Gen" rating
//---------------------------------------------------------------------------

int MultiChanDeviceMap::LinkRate()
{
    return DeviceDriver->Hdw().LinkRate();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::LinkWidth() - Link width in bits
//---------------------------------------------------------------------------

int MultiChanDeviceMap::LinkWidth()
{
    return DeviceDriver->Hdw().LinkWidth();
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::DumpLog() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::DumpLog(std::vector<int> & vec)
{
    DebugLog.copy_to(vec);
}

//---------------------------------------------------------------------------
//  MultiChanDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  MultiChanDeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<Driver().ILOG().Count; i++)
        vec.push_back(Driver().ILOG().Status[i]);
}

}  // end namespace Innovative

