//  MultiChanDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef MultiChanDeviceMap_MbH
#define MultiChanDeviceMap_MbH

#include <MultiChanStreamSupport_Mb.h>
#include <StreamDeviceEvents_Mb.h>
#include <DeviceDriver_Mb.h>
#include <ThreadSafeLog_Mb.h>
#include <MultiChanInterruptScheme_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS MultiChanDeviceMap -- General Control of a MultiChan Device (sans stream)
//===========================================================================

class MultiChanDeviceMap :  public ISchemeRequiresMultiChanDevice,
                            public IMultiChanStreamDeviceInternals,
                            public InnovativeKernel::IMultiChanOwnerDM,
                            public IUsesStreamDeviceEventCallbacks
{
public:
    MultiChanDeviceMap(class MultiChanBoard * owner, InnovativeKernel::MultiChannelBMDeviceDriver * driver);
    virtual ~MultiChanDeviceMap();
    //
    //  Standard Device Interface
    //  ...Jungo Presence Detect
    size_t  BoardCount();
    bool    LicenseCheck(unsigned int & code);

    virtual bool  IsOpened();
    virtual void  Open();
    virtual void  Close();

    int   Target();
    void  Target(int target);

    bool  Led();
    void  Led( bool value );

    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );

    ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );

    //  Link Status
    int   LinkRate();   // return X as in Gen1, GenX.
    int   LinkWidth();  // return 0 if unlinked

    void  BusmasteringOn();
    void  BusmasteringOff();

    void  BusmasterReset();
    void  MasterReset();

    void  RegisterStreamPlugIn( class IMultiChanStreamManagerInternals *plug_in );



    // Specify driver-allocated BM region, prior to open
//    virtual void            SetBusMasterSizeWords(ii32 value);
//    virtual void            SetIncomingBusMasterSizeWords(ii32 value);
//    virtual void            SetOutgoingBusMasterSizeWords(ii32 value);
//    virtual ii32            GetIncomingBusMasterSizeWords() const;
//    virtual ii32            GetOutgoingBusMasterSizeWords() const;

    //
    //  IntScheme Interface Impls

    virtual InnovativeKernel::MultiChannelBMDeviceDriver & Driver();
    virtual InnovativeKernel::IMcbmHardwareManager &      BmMgr();

    virtual MC_ManagerInternalsSafeLink &  Link()  { return StreamManagerInternals;  }

    //  SubDevices
    IMultiChanConfiguration &   BmIn();
    IMultiChanConfiguration &   BmOut();

    IMultiChanInitialization &   BmInInit();
    IMultiChanInitialization &   BmOutInit();
    //  Debug
    //  ...Busmaster Fill (debug only)
    void  FillRecvRegion(size_t chan, unsigned int pattern, bool ramp);
    void  FillXmitRegion(size_t chan, unsigned int pattern, bool ramp);

    //
    // Memory spaces
    AddressingSpace                    PortMemory;
    AddressingSpace                    LogicMemory;

protected:
    //
    //  Custom System "TIMES" - defined moments for call-up methods. These template methods never overridden
    void  AtConstructCustomPartTime();  // call in most derived class, call base CCP() method and your own subdevices too.

    void  AtBeforeOpenTime();
    void  AtAfterOpenTime();

    void  AtBeforeCloseTime();
    void  AtAfterCloseTime();

public:
    //
    //  MultiChannelDevice Stream/Configuration "Time" Activations
    //    (activated by Stream Object)
    virtual void  AtPreconfigureTime();
    virtual void  AtStreamConfigureTime();
    virtual void  AtStreamPrefillTime();
    virtual void  AtStreamStartTime();
    virtual void  AtStreamStopTime();

    virtual bool                TxMcChannelActive(int ch);  // returns true if McChannel is used this run
    virtual McImageInformation  TxChannelInformation(size_t channel);
    virtual McImage             TxGetEmptyImage(size_t channel, size_t desired_size);
    virtual void                TransferFilledImage(McImage& image);
    virtual void                ForwardPacketTransmitAck(McHintPack & OutgoingHints);

    //  IntScheme interface
    virtual void  AtDataTransmitTime(unsigned int channel);
    virtual void  AtDataReceivedTime();

protected:

    // SELF CONFIG CALLS - board setup call-downs at specific "times" for board specific customization
    virtual void    ConstructCustomParts() = 0;            // board construtor time
    virtual void    PreOpenHardware() = 0;                 // before open time
    virtual void    OpenHardware() = 0;                    // after open time
    virtual void    PreconfigureHardware() = 0;            // preconfigure time
    virtual void    ConfigureHardware() = 0;               // stream configuration
    virtual void    OnSingleStreamAtStartConfig() = 0;     // runs at first stream start (only) after PreconfigureHardware()
    virtual void    OnBeforeStreamStart() = 0;             // before stream start customization
    virtual void    OnAfterStreamStart() = 0;              // after stream start customization
    virtual void    OnBeforeStreamStop() = 0;              // before stream stop customization
    virtual void    StreamStopHardware() = 0;              // after stream stop subdevice configuration
    //virtual void    CloseHardware() = 0;                   // Before Close config??

    //  IntScheme interface
    virtual unsigned int  VisrIsrQDepth();

    // Other Interfaces
    virtual void  TraceLog(int type, int data);
    virtual Trace &  TextLog()  {  return Debug;  }
    //
    //  Member Data
    SafeLink<IMultiChanStreamManagerInternals,
             SafeMultiChanStreamManagerInternalsImpl>  StreamManagerInternals;

private:
	InnovativeKernel::MultiChannelBMDeviceDriver *    DeviceDriver;
	MultiChanInterruptScheme                          IntScheme;
	ThreadSafeLog                                     DebugLog;

	bool  ShouldPreloadXmit;
	bool  SingleTimeAtStartPreconfig;
	bool  ShouldIssueRcvCredit;

	//
	//  Member Functions
	Event *  IsrEvent();
	size_t   InterruptDepth();
	int      InterruptAck();

	void     EnableBusmasterInterrupt( bool enable );

public:
	//  Debug
	Trace                   Debug;

    virtual void  Log(int code, unsigned int data) {  DebugLog.append(code, static_cast<int>(data));  }
	void  DumpLog(std::vector<int> & vec);

    void            DumpIsrStatus(std::vector<int> & vec);

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
