// MultiChanInterruptScheme_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <MultiChanInterruptScheme_Mb.h>
#include <Buffer_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <StartStopThread_Mb.h>
#include <NotifyThread_Mb.h>
#include <sstream>
#include <HardwareRegister_Mb.h>
#include <SignalQueue_Mb.h>
#include <MemoryUtils_Mb.h>
#include <BufferHeader_Mb.h>
#include <Osal_Mb.h>
#include <iostream>
#include <assert.h>

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

namespace Innovative
{

namespace MultiChanIntScheme
{

//===========================================================================
//  CLASS BmMemoryRings  -- Handle BM memory for a direction
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of class BmMemoryRings
//---------------------------------------------------------------------------

class BmMemoryRings
{
public:
    BmMemoryRings();

    void  Clear();

    void  Resize(size_t ring_count)
            {  BMRings.resize(ring_count);  }
    void  Configure(size_t ring,  InnovativeKernel::StandardMcbm::ChannelRegionInfo * info)
            {  BMRings[ring].Configure(info);  }

    void     FillRegion(size_t ring, unsigned int pattern, bool ramp);
    size_t   Depth(size_t ring);
    size_t   Room(size_t ring);
    size_t   Size(size_t ring);

    size_t   InsertionMaxImageSize(size_t ring);


    //  Insertion Calls (adding real data to ring)
    void     AdvanceInsertion(McImage & semiImage);
    McImage  GetInsertionImage(size_t ring, size_t image_size);

    //  Extraction Calls (removing real data from ring)
    void  AdvanceExtraction(McImage & semiImage);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS  BmMemoryRings::Ring
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ring
    {
    public:
        Ring();
        ~Ring();

        ii32 *    InsertionPtr() {  return Base + InsertionIdx;  }
        ii32 *    ExtractionPtr() {  return Base + ExtractionIdx;  }

        size_t    InsertionMaxImageSize()  {  return RingSize - InsertionIdx;  }
        size_t    ExtractionMaxImageSize()  {  return RingSize - ExtractionIdx;  }

        void      Configure(InnovativeKernel::StandardMcbm::ChannelRegionInfo * info);
        void      Clear()
                      {  InsertionIdx = ExtractionIdx = 0;  IsFull = false;  }
        void      FillRegion(unsigned int pattern, bool ramp);

        size_t    Depth();
        size_t    Room();
        size_t    Size();

        void      AdvanceInsertion(size_t words);
        void      AdvanceExtraction(size_t words);

        size_t    InsertionIdx;
        size_t    ExtractionIdx;
        bool      IsFull;   // disambiguate full/empty
        ii32 *    Base;
        size_t    RingSize;

        ii32 *    PhysBase;  // don't dereference!!

    };

private:
    std::vector<Ring>   BMRings;

};

//---------------------------------------------------------------------------
//  constructor of class BmMemoryRings
//---------------------------------------------------------------------------

BmMemoryRings::BmMemoryRings()
{
}


//---------------------------------------------------------------------------
//  BmMemoryRings::Clear() -- Reset position indexes
//---------------------------------------------------------------------------

void  BmMemoryRings::Clear()
{
    for (size_t i=0; i<BMRings.size(); i++)
        BMRings[i].Clear();
}

//---------------------------------------------------------------------------
//  BmMemoryRings::FillRegion() -- Reset position indexes
//---------------------------------------------------------------------------

void  BmMemoryRings::FillRegion(size_t ring, unsigned int pattern, bool ramp)
{
    if (ring<BMRings.size())
        BMRings[ring].FillRegion(pattern, ramp);
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Depth() -- fetch depth of queue (full portion)
//---------------------------------------------------------------------------

size_t BmMemoryRings::Depth(size_t ring)
{
    if (ring<BMRings.size())
        return BMRings[ring].Depth();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Room() --  area remaining in ring for use
//---------------------------------------------------------------------------

size_t BmMemoryRings::Room(size_t ring)
{
    if (ring<BMRings.size())
        return BMRings[ring].Room();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::InsertionMaxImageSize() --  Room for single image in ring
//---------------------------------------------------------------------------

size_t  BmMemoryRings::InsertionMaxImageSize(size_t ring)
{
    if (ring<BMRings.size())
        return BMRings[ring].InsertionMaxImageSize();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Size() -- total size of ring
//---------------------------------------------------------------------------

size_t BmMemoryRings::Size(size_t ring)
{
    if (ring<BMRings.size())
        return BMRings[ring].Size();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::AdvanceInsertion() -- Advance In ptr over data, make pkt
//---------------------------------------------------------------------------
//  "Insertion" adds data is added to a ring, increasing its fullness.
//  Incoming:  Insertion happens when packets are signalled via interrupt, so we advance
//             over actual data.
//  Outgoing:  After Data is filled by App
//

void  BmMemoryRings::AdvanceInsertion(McImage & semiImage)
{
    //  The header already gives the channel, and advance distance.
    McHeaderDatagram ImageH(semiImage);
    size_t ring     = ImageH.ChannelId();
    size_t sizeInts = ImageH.PacketSize();

    if (ring<BMRings.size())
        {
        Ring & MemRing = BMRings[ring];

        //  complete the image object for later use
        ii32 * base = MemRing.InsertionPtr();
        semiImage.SetImage(base, sizeInts);

        //  advance the ring
        MemRing.AdvanceInsertion(sizeInts);
        }
    //else
    //  debug print "bad channel?"
    //  point image to safe "dummy region?"

    }

//---------------------------------------------------------------------------
//  BmMemoryRings::GetInsertionImage() -- Create Image for vacant area
//---------------------------------------------------------------------------
//  "Insertion" adds data is added to a ring, increasing its fullness.
//  Outgoing:  Image refers to space to be filled by app (or Stream)
//               it does not advance the Insertion pointer
//

McImage  BmMemoryRings::GetInsertionImage(size_t ring, size_t image_size)
{
    Ring & MemRing = BMRings[ring];

    McImage outImage(MemRing.InsertionPtr(), image_size);

    McHeaderDatagram ImageH(outImage);
	ImageH.ChannelId(static_cast<unsigned short>(ring));
    ImageH.PacketSize(image_size);
    return outImage;
}


//---------------------------------------------------------------------------
//  BmMemoryRings::AdvanceExtraction() -- Advance In ptr over data, make pkt
//---------------------------------------------------------------------------
//  "Insertion" adds data is added to a ring, increasing its fullness.
//  Incoming:  Insertion happens when packets are signalled via interrupt, so we advance
//             over actual data.
//  Outgoing:  After Data acked via interrupt
//
void BmMemoryRings::AdvanceExtraction(McImage& semiImage)
{
    //  The header already gives the channel, and advance distance.
    McHeaderDatagram ImageH(semiImage);
    size_t ring     = ImageH.ChannelId();
    size_t sizeInts = ImageH.PacketSize();

    if (ring<BMRings.size())
        {
        Ring & MemRing = BMRings[ring];

        //  complete the image object for later use
        ii32 * base = MemRing.ExtractionPtr();
        semiImage.SetImage(base, sizeInts);

        //  advance the ring
        MemRing.AdvanceExtraction(sizeInts);
        }
    //else
    //  debug print "bad channel?"
    //  point image to safe "dummy region?"

}

//===========================================================================
//  CLASS  BmMemoryRings::Ring  -- subclass for a single channel ring rgn
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class BmMemoryRings::Ring --
//---------------------------------------------------------------------------

BmMemoryRings::Ring::Ring()
    :  InsertionIdx(0), ExtractionIdx(0), IsFull(false), Base(0), RingSize(0), PhysBase(0)
{

}

//---------------------------------------------------------------------------
//  destructor for class BmMemoryRings::Ring --
//---------------------------------------------------------------------------

BmMemoryRings::Ring::~Ring()
{

}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::Configure() -- Init Ring from driver information
//---------------------------------------------------------------------------

void  BmMemoryRings::Ring::Configure(InnovativeKernel::StandardMcbm::ChannelRegionInfo * info)
{
    Base = info->Addr();
    PhysBase = info->PhysAddr();
    RingSize = info->SizeInWords();

    InsertionIdx = ExtractionIdx = 0;
    IsFull = false;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::FillRegion() -- Fill region with pattern
//---------------------------------------------------------------------------

void  BmMemoryRings::Ring::FillRegion(unsigned int pattern, bool ramp)
{
    if (Base)
        for (unsigned int i=0; i<RingSize; i++)
            {
            Base[i] = pattern;
            if (ramp)
                ++pattern;
            }
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::AdvanceInsertion() -- Advance Insertion Idx
//---------------------------------------------------------------------------

void  BmMemoryRings::Ring::AdvanceInsertion(size_t words)
{
    // buffer should fit exactly to end of region
    InsertionIdx += words;
    if (InsertionIdx >= RingSize)  // > should never happen, but check anyway
        InsertionIdx = 0;
    if (InsertionIdx==ExtractionIdx)
        IsFull = true;
    else
        IsFull = false;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::AdvanceExtraction() -- Advance Extraction Idx
//---------------------------------------------------------------------------

void  BmMemoryRings::Ring::AdvanceExtraction(size_t words)
{
    // buffer should fit exactly to end of region
    ExtractionIdx += words;
    if (ExtractionIdx >= RingSize)  // > should never happen, but check anyway
        ExtractionIdx = 0;
    if (InsertionIdx==ExtractionIdx)
        IsFull = false;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::Depth() -- Amount of filled space in queue
//---------------------------------------------------------------------------

size_t  BmMemoryRings::Ring::Depth()
{
    size_t Iidx = InsertionIdx;
    size_t Eidx = ExtractionIdx;
    if (Iidx==Eidx)
        {
        if (IsFull)
            return RingSize;
        else
            return 0;
        }
    else if (Iidx>Eidx)
        return InsertionIdx - ExtractionIdx;
    else
        return InsertionIdx - ExtractionIdx + RingSize;
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::Room() -- Amount of empty space in queue
//---------------------------------------------------------------------------

size_t  BmMemoryRings::Ring::Room()
{
    return RingSize - Depth();
}

//---------------------------------------------------------------------------
//  BmMemoryRings::Ring::Size() -- Total allocated space in queue
//---------------------------------------------------------------------------

size_t  BmMemoryRings::Ring::Size()
{
    return RingSize;
}

//===========================================================================
//  CLASS BmXmitThread  -- Transmission sync handling for MultiChanInt scheme
//===========================================================================
//===========================================================================
//  CLASS MultiChanStreamStoppageThread  -- Proper Termination signalling
//===========================================================================
//  We want a callback to happen when it is safe to call Stream:;Start() again
//    this thread watches the condition and calls the handler when ready

class BmXmitThread : public BasicSSThread
{
    typedef BasicSSThread inherited;

public:
    // Ctor
    BmXmitThread(MultiChanInterruptScheme *owner);

    void  Clear();

    void  TransmitDataAvailable(int ch);
    void  ForwardPacketTransmitAck(McHintPack& OutgoingHints);

protected:

    MultiChanInterruptScheme *  Owner;
    Semaphore                   ePushPackets;
    Semaphore                   eNewPacket;

    enum { sPushPackets=inherited::sLast, sNewPacket, sLast };
    virtual void    Init()
        {
        inherited::Init();
        AddObject(eNewPacket, sNewPacket);
        AddObject(ePushPackets, sPushPackets);  // prefill?
        }

    //  do one iteration of main process
    virtual void    AtExecution(int result);

    void  PushOneChannel();
    void  PushAllChannels();

    void  Log(int tag, int value)
                {  Owner->SD()->Log(tag, value);  }
    void  Log(int tag, size_t value)
                {  Log(tag, static_cast<int>(value));  }

private:
    //  Wake Up Optimization
    enum ReadyStates { rsReady, rsNeedData, rsNeedRoom };
    std::vector<unsigned int>  ReadyState;
    CriticalSection  ReadyStateLock;

    unsigned int ReadReadyState(size_t channel)
            {
            CriticalSectionArea guard(ReadyStateLock);
            return ReadyState[channel];
            }
    void WriteReadyState(size_t channel, unsigned int new_state)
            {
            CriticalSectionArea guard(ReadyStateLock);
            ReadyState[channel] = new_state;
            }


    void  SendOnePacket(unsigned int ch, size_t packet_size);

    ThreadSafeQueue<unsigned int>  ChannelQ;

    void  ActivateXmit()
                {  ePushPackets.Release();  }
    void  ActivateXmit(unsigned int ch)
                {
                ChannelQ.push(ch);
                eNewPacket.Release();
                }

};

//---------------------------------------------------------------------------
//  constructor for class BmXmitThread
//---------------------------------------------------------------------------

BmXmitThread::BmXmitThread(MultiChanInterruptScheme* owner)
    : inherited("BmXmitThread"), Owner(owner), ReadyState(Owner->SD()->Driver().GetMaxOutgoingRegionCount())
{
    Priority(tpHigher);

    Clear();
}

//---------------------------------------------------------------------------
//  BmXmitThread::Clear() -- signals new packet sent
//---------------------------------------------------------------------------

void BmXmitThread::Clear()
{
    for (size_t i=0; i<ReadyState.size(); i++)
        ReadyState[i] = rsReady;
}

//---------------------------------------------------------------------------
//  BmXmitThread::TransmitDataAvailable() -- signals new packet sent
//---------------------------------------------------------------------------
//  We optionally wake up the Xmit thread if this channel can proceed.

void BmXmitThread::TransmitDataAvailable(int ch)
{
    unsigned int ready_state = ReadReadyState(ch);
    if (ready_state==rsNeedRoom)  // blocked by BM full, can't do more
        {
        Owner->SD()->Log(0x3F, ch);
        return;
        }

    WriteReadyState(ch, rsReady);  // otherwise, we are ready to go
    ActivateXmit(ch);                // wake thread
}

//---------------------------------------------------------------------------
//  BmXmitThread::ForwardPacketTransmitAck() -- signals packet ack processing
//---------------------------------------------------------------------------
//  We optionally wake up the Xmit thread if any channel can proceed.

void BmXmitThread::ForwardPacketTransmitAck(McHintPack& OutgoingHints)
{
    bool shouldWake(false);
    //
    //  Open up the hint pack
    for (size_t ch=0; ch<OutgoingHints.size(); ch++)
        {
        if (OutgoingHints[ch])  // if we ACKed this channel here
            {
            unsigned int ready_state = ReadReadyState(ch);
            //
            //  We only signal here if this thread is suspended
            //    and acks have been used up.  If it is running or waiting on data, don't wake
            if (ready_state==rsNeedData || ready_state==rsReady)
                continue;

            // We are waiting for room to come free (rsRoom). But if there are still pending signals...
			if (Owner->SMI()->ChannelXmitQ_PendingNotifies(static_cast<int>(ch)) != 0)
                continue;

            // Waiting on Room without any pending signals
            WriteReadyState(ch, rsReady);  // otherwise, we are ready to go
            shouldWake = true;
            }
        }

    if (shouldWake)
        ActivateXmit();                // wake thread if there is work
    else
        Owner->SD()->Log(0x3E, 0);
}

//---------------------------------------------------------------------------
//  BmXmitThread::AtExecution() -- Push packets down into channels...
//---------------------------------------------------------------------------

void  BmXmitThread::AtExecution(int result)
{
    Owner->SD()->Log(0x50, 1);

    switch (result)
        {
        case sNewPacket:
            PushOneChannel();
            break;
        default:
        case sPushPackets:
            PushAllChannels();
            break;
        }
    Owner->SD()->Log(0x50, 0);
}

//---------------------------------------------------------------------------
//  BmXmitThread::PushOneChannel() -- Push one packet down...
//---------------------------------------------------------------------------

void  BmXmitThread::PushOneChannel()
{
    Owner->SD()->Log(0x52, 1);
    //
    // Get the current Image space information
    unsigned int channel(0);
    if (ChannelQ.depth() != 0)
        {
        channel = ChannelQ.front();
        ChannelQ.pop();
        }
    else
        {
        Owner->SD()->Log(0x52, 0xF0);
        // should never happen...channel info should be there
        }

    Owner->SD()->Log(0x51, channel);

    // get channel information
    McImageInformation Info = Owner->TxChannelInformation(channel);

    unsigned int ready_state = ReadReadyState(channel);
    if (ready_state!=rsReady)  // blocked by something...
        {
        Owner->SD()->Log(0x52, 0xF1);  // not ready? shouldn't be!
        return;
        }
    //
    //  There better be a dang packet...we just added it!
    size_t packet_count = Owner->SMI()->ChannelXmitQ_Packets(channel);
    if (packet_count==0)
        {
        //  Hey, we just added one!
        Owner->SMI()->ChannelXmitQ_Signal(channel, 1);   // need to signal?
        Owner->SD()->Log(0x52, 0xF2);
        WriteReadyState(channel, rsNeedData);  // no blocks, can't continue.
        return;
        }

    //  if we have a packet, get the size
    size_t packet_size = Owner->SMI()->ChannelXmitQ_NextPacketSize(channel);

    if (packet_size > Info.TotalFreeSpace)
        {
        // no room for packet
        ChannelQ.push(channel);   // we need to try again later
        Owner->SD()->Log(0x5a, packet_size);
        Owner->SD()->Log(0x5b, Info.TotalFreeSpace);
        Owner->SD()->Log(0x52, 0xF3);
        WriteReadyState(channel, rsNeedRoom);  // no room, can't continue.
        return;
        }
    else
        {
        //  Can send this packet
        SendOnePacket(channel, packet_size);

        Owner->SD()->Log(0x5a, packet_size);
        Owner->SD()->Log(0x5b, Info.TotalFreeSpace);
        Owner->SD()->Log(0x52, 0);
        return;
        }
    // not reached
}

//---------------------------------------------------------------------------
//  BmXmitThread::PushAllChannels() -- Push packets down into channels...
//---------------------------------------------------------------------------

void  BmXmitThread::PushAllChannels()
{
    Owner->SD()->Log(0x53, 1);
    //
    //  Send one packet per channel that can
    for (unsigned int ch=0; ch<Owner->SD()->Driver().GetInUseOutgoingRegionCount(); ch++)
        {
        unsigned int ready_state = ReadReadyState(ch);
        if (ready_state!=rsReady)  // blocked by something...
            {
            //Owner->SD()->Log(0x51, 0xF0+ch);  // channel not ready
            continue;
            }
        Owner->SD()->Log(0x51, ch);

        // get channel information
        McImageInformation Info = Owner->TxChannelInformation(ch);
        //
        //  if there are no packets to push forward, skip
        size_t packet_count = Owner->SMI()->ChannelXmitQ_Packets(ch);
        if (packet_count==0)
            {
            //  We have no blocks, so signal Xmit Q to demand more blocks from application
            Owner->SMI()->ChannelXmitQ_Signal(ch, 1);   // really need 2?
            Owner->SD()->Log(0x53, 0xF0);
            WriteReadyState(ch, rsNeedData);  // no blocks, can't continue.
            continue;
            }

        //  if we have a packet, get the size
        size_t packet_size = Owner->SMI()->ChannelXmitQ_NextPacketSize(ch);

        if (packet_size > Info.TotalFreeSpace)
            {
            // no room for packet
            Owner->SD()->Log(0x5a, packet_size);
            Owner->SD()->Log(0x5b, Info.TotalFreeSpace);
            Owner->SD()->Log(0x53, 0xF1);
            WriteReadyState(ch, rsNeedRoom);  // no room, can't continue.
            continue;
            }
        else
            {
            //  Can send this packet
            SendOnePacket(ch, packet_size);
            //Info[ch].TotalFreeSpace -= packet_size;  // keep track of space deducted

            Owner->SD()->Log(0x5a, packet_size);
            Owner->SD()->Log(0x5b, Info.TotalFreeSpace);
            Owner->SD()->Log(0x53, 2);
            }
        }

    Owner->SD()->Log(0x53, 0);
}

//---------------------------------------------------------------------------
//  BmXmitThread::SendOnePacket() -- Push a Packet to the BM Region
//---------------------------------------------------------------------------

void  BmXmitThread::SendOnePacket(unsigned int ch, size_t packet_size)
{
    size_t offset = 0;
    McBuffer Packet = Owner->SMI()->ChannelXmitQ_DequeueNextPacket(ch);
    // packet might be broken in two
    McImage Img = Owner->TxGetEmptyImage(ch, packet_size);
    Img.CopyFrom(Packet, offset);
    Owner->TransferFilledImage(Img);  // send, credit image data

    offset += Img.SizeInInts();
    packet_size -= Img.SizeInInts();

    if (packet_size)
        {
        //  second half of buffer needs to be sent
        McImage Img2 = Owner->TxGetEmptyImage(ch, packet_size);
        Img2.CopyFrom(Packet, offset);
        Owner->TransferFilledImage(Img2);  // send, credit image data
        }

}



//===========================================================================
//  CLASS VirtualIsrThread  -- ISR handling thread for MultiChan Int scheme
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class VirtualIsrThread
//---------------------------------------------------------------------------

class VirtualIsrThread : public StartStopThread
{
    typedef StartStopThread inherited;
       
    //
    //  Class InterruptStatusRegister
    class InterruptStatusRegister : public Register
    {
    typedef Register inherited;

    public:
        RegisterBit       RxChannelInt;
        RegisterBit       TxChannelInt;
        RegisterBit       AlertInt;
        RegisterBit       ExternalInt;

    public:
        InterruptStatusRegister( AddressingSpace &space, int offset )
          :  inherited( space, offset ),
             RxChannelInt(*this, 0),
             TxChannelInt(*this, 1),
             AlertInt(*this, 2),
             ExternalInt(*this, 3)
        {
        }

    };

public:
    VirtualIsrThread( ISchemeRequiresMultiChanDevice * device );

    void  SizeRecvRings(size_t ring_count)
            {  RecvRing.Resize(ring_count);  }
    void  SizeXmitRings(size_t ring_count)
            {  XmitRing.Resize(ring_count);  }
    void  ConfigureRecvRing(size_t ring,  InnovativeKernel::StandardMcbm::ChannelRegionInfo * info)
            {  RecvRing.Configure(ring, info);  }
    void  ConfigureXmitRing(size_t ring,  InnovativeKernel::StandardMcbm::ChannelRegionInfo * info)
            {  XmitRing.Configure(ring, info);  }

    //  Xmit Packet Parts
    McImageInformation  ChannelInformation(size_t channel);
    McImage             CreateXmitImage(size_t channel, size_t desired_size, bool useMaxSize=false);
    void                TransferFilledImage(McImage& Image);

    void  Clear();
    void  FillRecvRegion(size_t chan, unsigned int pattern, bool ramp)
            {  RecvRing.FillRegion(chan, pattern, ramp);  }
    void  FillXmitRegion(size_t chan, unsigned int pattern, bool ramp)
            {  XmitRing.FillRegion(chan, pattern, ramp);  }

    bool  TransferXmitMultiChan();  // probably going away

    void  GrantIncomingCredit(McImage & Image);
    void  GrantOutgoingCredit(McImage& Image);

    void  ReceiveDataAcknowledged();

    unsigned int  IsrQDepth()
	{
		return static_cast<unsigned int>(LocalIsrQ.depth());
	}

protected:
    enum { sDriverEvent=inherited::sLast, sLocalIsrQ, sLast };

    Event &                    DriverEvent;
    InterruptStatusRegister    InterruptStatusImage;
    AddressingSpace            IsrDecode;    // Fake address space for int decode

    BmMemoryRings              RecvRing;
    BmMemoryRings              XmitRing;

    ThreadSafeQueue<McImage>   RecvCreditQ;


    MC_ManagerInternalsSafeLinkAlias   StreamManagerInternals;

    ISchemeRequiresMultiChanDevice *  StreamDevice;
    SignalQueue<unsigned int>         LocalIsrQ;

    size_t  MaxInterruptDepth;
    size_t  MaxLocalIsrDepth;

    virtual void Init();
    virtual void SingleExecution();

    //
    //  Interrupt Processing Chain
    void  DispatchInterrupt( int IntStatus );
    void  ProcessRecvInterrupt();
    void  ProcessXmitInterrupt();
    void  ProcessAlertInterrupt();
    void  ProcessExternalInterrupt();

    bool  StreamStarted();

private:
    virtual void    XXX_DebugLog(int type, int code);

};

void    VirtualIsrThread::XXX_DebugLog(int type, int code)
    {  StreamDevice->Log(type, code);  }


//---------------------------------------------------------------------------
//  housekeeping for class VirtualIsrThread
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class VirtualIsrThread
//---------------------------------------------------------------------------

VirtualIsrThread::VirtualIsrThread(ISchemeRequiresMultiChanDevice* device)
    :  inherited("VirtualIsrThread"),
       DriverEvent( *( device->IsrEvent() ) ),
       InterruptStatusImage( IsrDecode, 0 ),
       StreamManagerInternals(device->Link()),
       StreamDevice( device ),
       MaxInterruptDepth(0), MaxLocalIsrDepth(0)
{
    Priority(tpTimeCritical);

//    CpuAffinityVec cpus;
//    cpus.push_back(1);
//    SetCpuAffinity(cpus);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::Init() -- Initialize sync objects
//---------------------------------------------------------------------------

void VirtualIsrThread::Init()
{
    inherited::Init();

    AddObject(DriverEvent, sDriverEvent);
    AddObject(LocalIsrQ.SignalSem(), sLocalIsrQ);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::StreamStarted() -- returns true if the streaming is started
//---------------------------------------------------------------------------

bool  VirtualIsrThread::StreamStarted()
{
    return StreamManagerInternals->HasStreamStarted();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::Clear() -- Stream Start Initialization Method
//---------------------------------------------------------------------------

void  VirtualIsrThread::Clear()
{
    RecvRing.Clear();
    XmitRing.Clear();
    RecvCreditQ.clear();
    LocalIsrQ.clear();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::SingleExecution() -- thread function
//---------------------------------------------------------------------------
//  This logic was taken from the PacketStream. The XU system has a register
//    to send a final "ack" to the entire process of interrupts that keeps
//    the number of interrupts here limited to 1 per cycle and they don't
//    pile up.  So parts of this "can't happen" (the local ISRQ signal)
//    and the monitoring is uninteresting.
//

void VirtualIsrThread::SingleExecution()
{
    assert (StreamDevice);
    int result = WaitFor();

    if ( Terminated() || eStopped.Check() )
        return;

    //
    //  A new interrupt was signalled. Since the low level queue is of
    //     finite size, we drain that queue as fast as we can. Since we
    //     also got at least one interrupt, we process one now.
    if ( result == sDriverEvent )
        {
        size_t depth = StreamDevice->InterruptDepth();
        StreamDevice->Log(0x94, static_cast<int>(depth));
        MaxInterruptDepth = std::max( depth, MaxInterruptDepth );

        if (MaxInterruptDepth > 100)
            {
            StreamDevice->Log(0x9F, MaxInterruptDepth);
            }
                   
        while( depth-- )
            {
            if( ! StreamStarted() )
                break;

            LocalIsrQ.push( StreamDevice->InterruptAck());
            }
            
        size_t Idepth = LocalIsrQ.depth();
        //StreamDevice->Log(0x95, Idepth);
        MaxLocalIsrDepth = std::max( Idepth, MaxLocalIsrDepth );
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            //  since we did not wait for Sem before, must decrement sem manually
            //    when popping IRQ register value
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop(SignalQueue<unsigned int>::PopSem);
            DispatchInterrupt( IrqToProcess );
            }
        }
    //
    //  We were signalled because of queued local ISRs.
    if ( result == sLocalIsrQ )
        {
        size_t Idepth = LocalIsrQ.depth();
        StreamDevice->Log(0x9a, Idepth);
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop();  // don't dec sem, already done
            DispatchInterrupt( IrqToProcess );
            }
        }

    //  If we have stopped, clear out the ISR Q now
    if( ! StreamStarted() )
        LocalIsrQ.clear();

    //StreamDevice->Log(0x96, LocalIsrQ.depth());
    //StreamDevice->Log(0x97, (int)LocalIsrQ.SignalSem().Count());
    //
    //  Periodic Update call to BM Manager
    //StreamDevice->BmMgr().PeriodicUpdate();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::DispatchInterrupt() -- process interrupt
//---------------------------------------------------------------------------

void VirtualIsrThread::DispatchInterrupt( int IntStatus )
{
    StreamDevice->Log(0x04, IntStatus);

    //
    //  Overlay Status Register Decode onto IntStatus variable
    IsrDecode.Link( ( ii32 *)&IntStatus );

    if (InterruptStatusImage.RxChannelInt.Value())
        {
        //  Incoming - Board to Host Data
        ProcessRecvInterrupt();
        }
    if (InterruptStatusImage.TxChannelInt.Value())
        {
        //  Outgoing - Host to Board Data
        ProcessXmitInterrupt();
        }
    if (InterruptStatusImage.AlertInt.Value())
        {
        //  Alert Message Processing
        ProcessAlertInterrupt();
        }
    if (InterruptStatusImage.ExternalInt.Value())
        {
        //  External Interrupts
        ProcessExternalInterrupt();
        }
    // Signal Queues Done
    StreamDevice->BmMgr().SignalIntQueuesDone();
    StreamDevice->Log(0x05, 0);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessRecvInterrupt() -- Recv block
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessRecvInterrupt( )
{
    StreamDevice->Log(0x70, 0);

    McImagePack  IncomingImages;  // move to class if too big for stack
    StreamDevice->BmMgr().GetIncomingQueueData(IncomingImages);

    StreamDevice->Log(0x71, IncomingImages.size());   // number of images delivered

    for (size_t img=0; img<IncomingImages.size(); img++)
        {
        //  Complete the image object, advance proper ring
        McImage & Image(IncomingImages[img]);
        RecvRing.AdvanceInsertion(Image);

        //  dispatch image call?
        StreamManagerInternals->ForwardImagePacket(Image);

        //  Mark Image as used up
        RecvRing.AdvanceExtraction(Image);

        //  copy to real buffer??
        if (StreamManagerInternals->CopyIncomingData()==false)
            {
            StreamDevice->Log(0x72, 0);   // Image Copy Mode=OFF
            GrantIncomingCredit(Image);  // release image area for reuse
            }
        else
            {
            StreamDevice->Log(0x72, 1);   // Image Copy Mode=ON
            //  Create a real buffer, enqueue it
            McBuffer Data(Image.SizeInInts());
            Image.CopyTo(Data);

            StreamManagerInternals->ForwardDataPacket(Data);
            //  credit grant - delay credit if recv queue is getting deep?
            if (!StreamManagerInternals->IsIncomingQueueOverThreshold())
                {
                GrantIncomingCredit(Image);  // release image area for reuse
                }
            else
                {
                // push image onto queue (for later crediting)
                RecvCreditQ.push(Image);
                }
            }
        }

}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessXmitInterrupt() -- Ack blocks previously sent
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessXmitInterrupt()
{
    StreamDevice->Log(0x80, 0);

    McImagePack  OutgoingImages;  // move to class if too large ?
    StreamDevice->BmMgr().GetOutgoingQueueData(OutgoingImages);

    StreamDevice->Log(0x81, OutgoingImages.size());   // number of Images delivered

    //
    //  The hint pack is
    McHintPack  OutgoingHints;
    OutgoingHints.Reset();   // all false
    for (size_t img=0; img<OutgoingImages.size(); img++)
        {
        //  Each image marks a section of channel memory now
        //    free to fill again from the application
        McImage & Image(OutgoingImages[img]);
        XmitRing.AdvanceExtraction(Image);

        //  The header already gives the channel
        McHeaderDatagram ImageH(Image);
        OutgoingHints[ImageH.ChannelId()]=true;  // flag hints if present
        }

    //  copy to real buffer??
    if (StreamManagerInternals->CopyOutgoingData()==false)
        {
        //  dispatch notificaton of space opening up to proper handler
        StreamManagerInternals->ForwardImageTransmitAck(OutgoingHints);
        }
    else
        {
        //  Dispatch notification to Malibu
        StreamManagerInternals->ForwardPacketTransmitAck(OutgoingHints);
        }

}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessAlertInterrupt() -- Dispatch Alerts
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessAlertInterrupt()
{
    StreamDevice->Log(0x60, 0);

    McAlertPack  Alerts;  // move to class if too large
    StreamDevice->BmMgr().GetAlertQueueData(Alerts);

    for (size_t i=0; i<Alerts.size(); i++)
        StreamManagerInternals->ForwardAlertPacket(Alerts[i]);

    StreamDevice->Log(0x61, Alerts.size());   // number of Alerts delivered
    AccessDatagram<int> Alert0_DG(Alerts[0]);
    StreamDevice->Log(0x68, Alert0_DG[0] );   // first Alerts data
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessExternalInterrupt() -- Dispatch External Int
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessExternalInterrupt()
{
    StreamDevice->Log(0x40, 0);

    McExternalIntPack  ExtInts;  // move to class if too slow, as we erase in the call?
    StreamDevice->BmMgr().GetExtIntQueueData(ExtInts);

    // dispatch ExtInts to Stream here?
    for (size_t i=0; i<ExtInts.size(); i++)
        StreamManagerInternals->ForwardExternIntPacket(ExtInts[i]);

    StreamDevice->Log(0x41, ExtInts.size());   // number of ExtInts delivered
    StreamDevice->Log(0x48, static_cast<size_t>(ExtInts[0].RisingMask()) );   // first ExtInt data
    StreamDevice->Log(0x49, static_cast<size_t>(ExtInts[0].FallingMask()) );   // first ExtInt data
}

//=============================================================================
//  Packet Image transfer methods
//=============================================================================

//-----------------------------------------------------------------------------
//  VirtualIsrThread::ChannelInformation() -- Get Info about space in channel
//-----------------------------------------------------------------------------

McImageInformation  VirtualIsrThread::ChannelInformation(size_t channel)
{
    McImageInformation Info;
    Info.Channel        = channel;
    Info.TotalFreeSpace = XmitRing.Room(channel);
    Info.MaxImageSize   = XmitRing.InsertionMaxImageSize(channel);

    return Info;
}

//-----------------------------------------------------------------------------
//  VirtualIsrThread::CreateXmitImage() -- Create Valid Image for Filling
//-----------------------------------------------------------------------------

McImage  VirtualIsrThread::CreateXmitImage(size_t channel, size_t desired_size, bool useMaxSize)
{
    // Figure out what size to use...the users, or the Max allowed
    size_t MaxImageSize   = XmitRing.InsertionMaxImageSize(channel);
    size_t Size = (useMaxSize) ? MaxImageSize : desired_size ;
    if (Size>MaxImageSize)
        Size = MaxImageSize;

    McImage outImage = XmitRing.GetInsertionImage(channel, Size);
    return outImage;
}

//-----------------------------------------------------------------------------
//  VirtualIsrThread::TransferFilledImage() -- "Send" a filled Image Packet
//-----------------------------------------------------------------------------
//  When a proper, filled Image is ready, we can notify the board

void  VirtualIsrThread::TransferFilledImage(McImage & Image)
{
    // Adjust Local Ring
    XmitRing.AdvanceInsertion(Image);

    // Grant Credit - Hardware will now process image
    GrantOutgoingCredit(Image);
}


//------------------------------------------------------------------------------
//  VirtualIsrThread::GrantIncomingCredit() -- Grant credit for an image buffer
//------------------------------------------------------------------------------

void VirtualIsrThread::GrantIncomingCredit(McImage & Image)
{
    //  The header already gives the channel, and advance distance.
    McHeaderDatagram ImageH(Image);

    StreamDevice->Log(0x10, ImageH.ChannelId());
    StreamDevice->Log(0x12, ImageH.DataSize());
    StreamDevice->Log(0x13, StreamDevice->BmMgr().GetCurrentIncomingCredit(ImageH.ChannelId()));

	StreamDevice->BmMgr().GrantIncomingCredit(ImageH.ChannelId(), static_cast<unsigned int>(ImageH.DataSize()));
}

//------------------------------------------------------------------------------
//  VirtualIsrThread::GrantOutgoingCredit() -- Grant credit for an image buffer
//------------------------------------------------------------------------------

void VirtualIsrThread::GrantOutgoingCredit(McImage & Image)
{
    //  The header already gives the channel, and advance distance.
    McHeaderDatagram ImageH(Image);

    StreamDevice->Log(0x11, ImageH.ChannelId());
    StreamDevice->Log(0x12, ImageH.DataSize());
    StreamDevice->Log(0x14, StreamDevice->BmMgr().GetCurrentOutgoingCredit(ImageH.ChannelId()));

	StreamDevice->BmMgr().GrantOutgoingCredit(ImageH.ChannelId(), static_cast<unsigned int>(ImageH.DataSize()));
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ReceiveDataAcknowledged() --
//---------------------------------------------------------------------------
//  App has consumed a packet.  We can credit one now, if available

void  VirtualIsrThread::ReceiveDataAcknowledged()
{
    if (RecvCreditQ.empty())
        return;

    McImage next = RecvCreditQ.front();
    GrantIncomingCredit(next);
    RecvCreditQ.pop();
}

}  // end namespace Innovative::MultiChanIntScheme

//===========================================================================
//  CLASS MultiChanInterruptScheme  -- ISR & Busmaster region handling
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class MultiChanInterruptScheme
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class MultiChanInterruptScheme
//---------------------------------------------------------------------------

MultiChanInterruptScheme::MultiChanInterruptScheme(ISchemeRequiresMultiChanDevice * device)
    :  Visr(0), BmX(0), StreamManagerInternals(device->Link()), StreamDevice(device)
{
}

//---------------------------------------------------------------------------
//  destructor for class MultiChanInterruptScheme
//---------------------------------------------------------------------------

MultiChanInterruptScheme::~MultiChanInterruptScheme()
{
    Close();
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::Open()
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::Open()
{
    //
    //  Create Busmaster Xmit thread.
    //  NOTE: not needed in image mode
    BmX = new MultiChanIntScheme::BmXmitThread(this);
    BmX->Resume();

    //
    //  Create V-ISR thread.
    Visr = new MultiChanIntScheme::VirtualIsrThread(StreamDevice);

    ConfigureBusmasterDataStructures();
    Visr->Resume();

    StreamDevice->Log(0x00, 0xEEE);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::Close()
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::Close()
{
    if ( BmX )
        {
        BmX->Stop();
        BmX->Shutdown();
        delete BmX;
        BmX = NULL;
        }
    //
    //  Have VISR Thread shut itself down
    if ( Visr )
        {
        Visr->Stop();
        Visr->Shutdown();
        delete Visr;
        Visr = NULL;
        }
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::ConfigureBusmasterDataStructures()
//---------------------------------------------------------------------------
//  Called after Open() to set BM region sizes

void MultiChanInterruptScheme::ConfigureBusmasterDataStructures()
{
    //  ...input
    size_t in_channels = StreamDevice->Driver().GetMaxIncomingRegionCount();
    Visr->SizeRecvRings(in_channels);
    for (size_t chan=0;  chan<in_channels; chan++)
        {
        InnovativeKernel::StandardMcbm::ChannelRegionInfo * info = StreamDevice->Driver().GetIncomingRegionInfo(chan);
        Visr->ConfigureRecvRing(chan, info);
        }
    //  ...output
    size_t out_channels = StreamDevice->Driver().GetMaxOutgoingRegionCount();
    Visr->SizeXmitRings(out_channels);
    for (size_t chan=0;  chan<out_channels; chan++)
        {
        InnovativeKernel::StandardMcbm::ChannelRegionInfo * info = StreamDevice->Driver().GetOutgoingRegionInfo(chan);
        Visr->ConfigureXmitRing(chan, info);
        }

}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::ConfigureBusmastering()
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::ConfigureBusmastering()
{

	//  ...Clear the VISR storage
	if (Visr)
		{
		Visr->Clear();
		}

    //
	//  Start V-ISR thread.
	if (Visr)
		{
		Visr->Start();
        }
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::FillRecvRegion() -- get TX free memory info
//---------------------------------------------------------------------------

McImageInformation  MultiChanInterruptScheme::TxChannelInformation(size_t channel)
{
    return Visr->ChannelInformation(channel);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::TxGetEmptyImage() -- Get Image to fill
//---------------------------------------------------------------------------

McImage  MultiChanInterruptScheme::TxGetEmptyImage(size_t channel, size_t desired_size)
{
    return Visr->CreateXmitImage(channel, desired_size);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::TransferFilledImage() -- Send Image
//---------------------------------------------------------------------------

void MultiChanInterruptScheme::TransferFilledImage(McImage& image)
{
    Visr->TransferFilledImage(image);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::FillRecvRegion() -- fill bm region with pattern
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::FillRecvRegion(size_t chan, unsigned int pattern, bool ramp)
{
    Visr->FillRecvRegion(chan, pattern, ramp);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::FillXmitRegion() -- fill bm region with pattern
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::FillXmitRegion(size_t chan, unsigned int pattern, bool ramp)
{
    Visr->FillXmitRegion(chan, pattern, ramp);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::PreloadXmitBusmasterRegion()
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::PreloadXmitBusmasterRegion()
{
    // if we are in Image mode, don't do automatic prefill here, or
    //   activate the pusher thread.
    if (StreamManagerInternals->CopyOutgoingData()==false)
        return;

    //  Start Xfer thread
    BmX->Clear();
    BmX->Start();

    // do more than 1 packet?
    for (unsigned int ch=0; ch<SD()->Driver().GetInUseOutgoingRegionCount(); ch++)
        {
        TransmitDataAvailable(ch);
        }
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::TransmitDataAvailable() -- Signal Xmit thread to process
//---------------------------------------------------------------------------

void  MultiChanInterruptScheme::TransmitDataAvailable(unsigned int channel)
{
    StreamDevice->Log(0x30,0);
    BmX->TransmitDataAvailable(channel);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::ReceiveDataAcknowledged() --
//---------------------------------------------------------------------------
//  App has consumed a packet.  We can credit one now, if available

void  MultiChanInterruptScheme::ReceiveDataAcknowledged()
{
    Visr->ReceiveDataAcknowledged();
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::ForwardPacketTransmitAck() --
//---------------------------------------------------------------------------
//  Room has opened up.  Can send more packets

void MultiChanInterruptScheme::ForwardPacketTransmitAck(McHintPack& OutgoingHints)
{
    StreamDevice->Log(0x31,0);
    BmX->ForwardPacketTransmitAck(OutgoingHints);
}

//---------------------------------------------------------------------------
//  MultiChanInterruptScheme::VisrIsrQDepth() -- return depth of VISR to-do Q
//---------------------------------------------------------------------------

unsigned int  MultiChanInterruptScheme::VisrIsrQDepth()
{
    if (Visr)
        return Visr->IsrQDepth();
    else
        return 0;
}

}  // end namespace Innovative

//---------------------------------------------------------------------------


