//  MultiChanInterruptScheme_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef MultiChanInterruptScheme_MbH
#define MultiChanInterruptScheme_MbH

#include <DataTypes_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SafeLink_Mb.h>
#include <MultiChanStreamSupport_Mb.h>
#include <DeviceDriverBmManager_Mb.h>

namespace OpenWire
{
    class Event;
}

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Event;
class Buffer;
typedef  SafeLink<IMultiChanStreamManagerInternals,
                  SafeMultiChanStreamManagerInternalsImpl>   MC_ManagerInternalsSafeLink;

typedef  SafeLinkAlias<IMultiChanStreamManagerInternals,
                  SafeMultiChanStreamManagerInternalsImpl>   MC_ManagerInternalsSafeLinkAlias;

//===========================================================================
//  CLASS ISchemeRequiresMultiChanDevice  -- MC Device callbacks for Scheme
//===========================================================================

class ISchemeRequiresMultiChanDevice
{
public:
    //
    //  Low Level Interrupt Interface
    virtual Event *  IsrEvent() = 0;
    virtual size_t   InterruptDepth() = 0;
    virtual int      InterruptAck() = 0;
    //
    //  TraceLog
    virtual void     Log(int type, unsigned int data) = 0;
    void     Log(int type, int data)
        {  Log(type, static_cast<unsigned int>(data));  }
    void     Log(int type, ii64 data)
        {  Log(type, static_cast<unsigned int>(data));  }

    virtual InnovativeKernel::Supplies_MultiChanDevice &  Driver() = 0;
    virtual InnovativeKernel::IMcbmHardwareManager &      BmMgr() = 0;


    // SafeLink
    virtual MC_ManagerInternalsSafeLink &  Link() = 0;

    virtual ~ISchemeRequiresMultiChanDevice()
        {}

};

//---------------------------------------------------------------------------
namespace MultiChanIntScheme
{
    class VirtualIsrThread;
    class BmXmitThread;
}

//===========================================================================
//  CLASS MultiChanInterruptScheme  -- ISR & Busmaster region handling
//===========================================================================

class MultiChanInterruptScheme
{
public:
    //enum Dir { BmXmit, BmRecv };

    MultiChanInterruptScheme(ISchemeRequiresMultiChanDevice * device);
    virtual ~MultiChanInterruptScheme();

    void  Open();
    void  Close();

    void  ConfigureBusmasterDataStructures();

    void  ConfigureBusmastering();

    McImageInformation  TxChannelInformation(size_t channel);
    McImage             TxGetEmptyImage(size_t channel, size_t desired_size);
    void                TransferFilledImage(McImage& image);

    void  FillRecvRegion(size_t chan, unsigned int pattern, bool ramp);
    void  FillXmitRegion(size_t chan, unsigned int pattern, bool ramp);

    void  PreloadXmitBusmasterRegion();
    void  TransmitDataAvailable(unsigned int channel);
    void  ReceiveDataAcknowledged();
    void  ForwardPacketTransmitAck(McHintPack & OutgoingHints);

    unsigned int VisrIsrQDepth();

    ISchemeRequiresMultiChanDevice *        SD()  {  return StreamDevice;  }
    IMultiChanStreamManagerInternals *      SMI()  { return StreamManagerInternals.Ptr();  }

private:
    //
    //  Member Data
    MultiChanIntScheme::VirtualIsrThread *  Visr;
    MultiChanIntScheme::BmXmitThread *      BmX;
    MC_ManagerInternalsSafeLinkAlias        StreamManagerInternals;
    ISchemeRequiresMultiChanDevice *        StreamDevice;

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#endif
