// MultipleObjects_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <MultipleObjects_Mb.h>

namespace   Innovative
{

//==============================================================================
//  CLASS  MultipleObjects  --  Wait on multiple events & objects
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class MultipleObjects
//---------------------------------------------------------------------------

MultipleObjects::MultipleObjects()
    : isValid(false)
{
    OwnerInfo = new MoSyncObjectOwner(this);
}

//---------------------------------------------------------------------------
//  destructor for class MultipleObjects
//---------------------------------------------------------------------------

MultipleObjects::~MultipleObjects()
{
    for (MoInformationArray::iterator i=Info.begin(); i != Info.end(); ++i)
        {
        if (i->Valid)
            i->Handle->RemoveOwner(OwnerInfo);
        }
        
	delete OwnerInfo;
	OwnerInfo = 0;
}

//---------------------------------------------------------------------------
//  MultipleObjects::AddObject()  --  Append an object handle & tag
//---------------------------------------------------------------------------

bool  MultipleObjects::AddObject(SyncObject * Handle, int Case)
{
    //
    //  Verify load
    if (Handle==0)
         return false;

    // Aggregate signaled state of child objects
    if (Handle->IsSignalled())
        SemGo.Release();

    //
    //  It's OK, Load it
    Info.push_back(MoInformation(Case, Handle));
    Handle->AddOwner( OwnerInfo );
    //
    //  Update validity of entire object
    isValid = true;
    for (MoInformationArray::iterator i=Info.begin(); i != Info.end(); ++i)
        {
        if (i->Valid==false)
            {
            isValid=false;
            break;
            }
         }

    return true;
}

//---------------------------------------------------------------------------
//  MultipleObjects::AddObject() -- Add object pointer and tag
//---------------------------------------------------------------------------

bool MultipleObjects::AddObject(SyncObject& object, int Case)
{
    return AddObject(&object, Case);
}

//---------------------------------------------------------------------------
//  MultipleObjects::WaitFor()  --  wait on the loaded objects
//---------------------------------------------------------------------------
//  This function returns the pre-defined case constant loaded when the
//     object was loaded.
//
//extern void LogQ(int which, int ackidx, int avail, int enq, float resid);

int MultipleObjects::WaitFor(unsigned int mS)
{
    //
    //  Must have loaded all objects before waiting
    if (isValid==false)
        {
        return Invalid;
		}

	while (1)
		{
        //
		//  Wait on our collective semaphore
		if (SemGo.Acquire(mS) == false)
			return Timeout;

		//
		//  Now determine Case for object that fired
		for (MoInformationArray::iterator i=Info.begin(); i != Info.end(); ++i)
			{
			if (!i->Valid)
				continue;

			// Test non-destructively
			if (i->Handle->IsSignalled())
				{
				//
				//  Wait on this object to clear auto-reset objects. Should
				//    return at once.
				i->Handle->WaitFor();
				int rtnval = i->Case;

                return rtnval;
				}
			}
		}
}

//------------------------------------------------------------------------------
//  MultipleObjects::ChildSignal()  --  Child object has been signalled (Wake!)
//------------------------------------------------------------------------------

void  MultipleObjects::ChildSignal()
{
    SemGo.Release();
}

//------------------------------------------------------------------------------
//  MultipleObjects::Clear()  --  Clear internal semaphore
//------------------------------------------------------------------------------

void  MultipleObjects::Clear()
{
    while (SemGo.IsSignalled())
        SemGo.Acquire();
}

//------------------------------------------------------------------------------
//  MultipleObjects::ChildRemove()  --  Child object has been deleted
//------------------------------------------------------------------------------

void  MultipleObjects::ChildRemove(SyncObject * obj)
{
    // Remove handle entry from list
    for (MoInformationArray::iterator i=Info.begin(); i != Info.end(); ++i)
        {
        if (!i->Valid)
            continue;

        if (i->Handle == obj)
            {
            i->Handle = 0;
            i->Valid = false;
            isValid = false;
            return;
            }
        }
}

//==============================================================================
//  CLASS  MultipleObjects::MoInformation
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class MultipleObjects::MoInformation -- default ctor
//------------------------------------------------------------------------------

MultipleObjects::MoInformation::MoInformation()
    :  Valid(false), Handle(0), Case(0)
{
}

//------------------------------------------------------------------------------
//  constructor for class MultipleObjects::MoInformation -- broadside load
//------------------------------------------------------------------------------

MultipleObjects::MoInformation::MoInformation(int aCase, SyncObject * aHandle)
    :  Valid(true), Handle(aHandle), Case(aCase)
{
}

//==============================================================================
//   CLASS MoSyncObjectOwner
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class MoSyncObjectOwner
//---------------------------------------------------------------------------

MoSyncObjectOwner::MoSyncObjectOwner(MultipleObjects * self )
    :  Self(self)
{
}

//---------------------------------------------------------------------------
//  MoSyncObjectOwner::Signal() -- Child object signalling
//---------------------------------------------------------------------------

void  MoSyncObjectOwner::Signal()
{
    Self->ChildSignal();
}

//---------------------------------------------------------------------------
//  MoSyncObjectOwner::Remove() -- Child object being deleted
//---------------------------------------------------------------------------

void  MoSyncObjectOwner::Remove(SyncObject * obj)
{
    Self->ChildRemove(obj);
}

} // namespace Innovative
