// MultipleObjects_Mb2.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  Multiple Object Wait class
//
//  This concrete class emulates a wait-for multiple objects, returning if
//     any one is signalled. It contains member functions to specify a
//     case statement value for every event to wait upon.
//
//  Currently, the IsValid() function only assures that a handle was
//    loaded for every declared handle in the object.
//  This class does not test for redundant handles!
//

#ifndef MultipleObjects_MbH
#define MultipleObjects_MbH

#include <SyncObject_Mb.h>
#include <Semaphore_Mb.h>
#include <DataTypes_Mb.h>
#include <vector>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS  MultipleObjects  --  Wait on multiple events & objects
//==============================================================================
class MoSyncObjectOwner;

class MultipleObjects
{
    friend class MoSyncObjectOwner;

    class MoInformation
    {
    public:
        bool        Valid;
        SyncObject* Handle;
        int         Case;

        MoInformation();
        MoInformation(int aCase, SyncObject * aHandle);
    };
    typedef std::vector<MoInformation>  MoInformationArray;

public:
	enum {Timeout = 0, Invalid = -1};
	 
    //
    //  Member Functions
    MultipleObjects();
    virtual ~MultipleObjects();

    // Methods
    void     Clear();
    bool     AddObject(SyncObject * object, int Case);
    bool     AddObject(SyncObject & object, int Case);

    int      LastPosition()
        {  return static_cast<int>(Info.size()-1);   }

    int      LastCase()
        {  return Info.back().Case;  }
    int      LastTag()
        {  return LastCase();  }

    bool     IsValid()              // Returns true if all handles set
        {  return isValid;   }

    // Interface
	int      WaitFor(unsigned int mS=SyncObject::Infinite);

    unsigned int      GoCount()  {  return SemGo.Count();  }

private:
    bool                    isValid;      // True if all handles have been set

    MoInformationArray      Info;
    MoSyncObjectOwner *     OwnerInfo;
    Semaphore               SemGo;

    void                    ChildSignal();
    void                    ChildRemove(SyncObject * obj);

};

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4311)		
#endif

//==============================================================================
//   CLASS MoSyncObjectOwner
//==============================================================================

class MoSyncObjectOwner :  public SyncObjectOwner
{
public:
    MoSyncObjectOwner( MultipleObjects * self );

    virtual int  Key()
        {
        ii64 value = reinterpret_cast<ii64>(Self);
        return static_cast<int>(value);
        };

    virtual void Signal();
    virtual void Remove(class SyncObject * obj);

private:
    MultipleObjects * Self;
};

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif


#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // namespace Innovative

#endif


