// NetCoffLoadThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#define ACE_NLOGGING

#include <algorithm>
#include <ace/Log_Msg.h>
#include <ace/SOCK_Connector.h>
#include <ace/OS_NS_string.h>
#include <ace/OS_NS_stdio.h>
#include <ace/Time_Value.h>
#include <NetCoffLoadThread_Mb.h>
#include <Sbc6713e_Mb.h>

#ifdef __BORLANDC__
#pragma package(smart_init)
#endif

namespace   Innovative
{

//==============================================================================
//  CLASS CoffDownloadThread  --  Thread to perform actual download
//==============================================================================

//---------------------------------------------------------------------------
//  declaration of class CoffDownloadThread
//---------------------------------------------------------------------------

class CoffDownloadThread : public Thread
{
    typedef Thread inherited;

public:
    //
    //  Member Functions
    CoffDownloadThread(NetCoffLoadThread * parent);
    void    CoffLoad(std::string file);

protected:
    //
    //  Member Functions
    void    Execute();

private:

    //
    //  Member Functions
    Event                       LoadRequested;
    Event                       LoadCompleted;
    std::string                 Filename;

    class NetCoffLoadThread *   PT;
};

//------------------------------------------------------------------------------
//  constructor of class CoffDownloadThread
//------------------------------------------------------------------------------

CoffDownloadThread::CoffDownloadThread(NetCoffLoadThread * pt)
    : inherited("NetCoffDownloadThread"), LoadRequested(Event::ManualReset),
      LoadCompleted(Event::ManualReset), PT(pt)
{
    LoadRequested.Clear();
    LoadCompleted.Set();
    Resume();
}

//------------------------------------------------------------------------------
//  CoffDownloadThread::Execute() -- Thread Function
//------------------------------------------------------------------------------

void CoffDownloadThread::Execute()
{
    LoadRequested.WaitFor();

    PT->Coff.Load(Filename);

    LoadCompleted.Set();
    PT->EndOfFile();
}

//---------------------------------------------------------------------------
//  CoffDownloadThread::CoffLoad() -- Initiate Coff Load in thread
//---------------------------------------------------------------------------

void CoffDownloadThread::CoffLoad(std::string file)
{
    //  Wait for download in progress to finish
    LoadCompleted.WaitFor();
    Filename = file;

    //  Signal thread to start download
    LoadRequested.Set();
}

//==============================================================================
//  CLASS NetCoffLoadThread -- manages COFF load via network
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class NetCoffLoadThread
//------------------------------------------------------------------------------

NetCoffLoadThread::NetCoffLoadThread(Sbc6713e & baseboard, std::string coff_file) 
    : PacketAvailable(Event::ManualReset),
      PacketTransmitted(Event::ManualReset),
      TransmitComplete(Event::ManualReset),
      CoffLoadComplete(Event::ManualReset),
      ID(baseboard.Id()),
      Port(baseboard.DownloadPort()),
      FCoffFile(coff_file)
{
    PacketAvailable.Clear();
    PacketTransmitted.Clear();
    TransmitComplete.Clear();

    Coff.OnMemWrite.SetEvent(this, &NetCoffLoadThread::DoCoffMemWrite);
    Coff.OnProgress.SetEvent(&baseboard,&Sbc6713e::DownloadProgress);
    Coff.OnComplete.SetEvent(&baseboard, &Sbc6713e::DownloadComplete);

    Resume();
}

//------------------------------------------------------------------------------
//  constructor of class NetCoffLoadThread
//------------------------------------------------------------------------------

NetCoffLoadThread::NetCoffLoadThread(std::string id, int port,
                                     std::string coff_file) 
    : PacketAvailable(Event::ManualReset),
      PacketTransmitted(Event::ManualReset),
      TransmitComplete(Event::ManualReset),
      CoffLoadComplete(Event::ManualReset),
      ID(id),
      Port(port),
      FCoffFile(coff_file)
{
    PacketAvailable.Clear();
    PacketTransmitted.Clear();
    TransmitComplete.Clear();

    Coff.OnMemWrite.SetEvent(this,
                             &NetCoffLoadThread::DoCoffMemWrite);

    Resume();
}

//------------------------------------------------------------------------------
//  NetCoffLoadThread::Execute() -- Thread routine
//------------------------------------------------------------------------------

void NetCoffLoadThread::Execute()
{
    //WSADATA      wsaData;
    CoffLoadComplete.Clear();

    //
    //  Attempt Connection to Server
    //  ...Build ourselves a Stream socket. This is a connected socket that
    //     provides reliable end-to-end communications. We will use the
    //     server object to send data to the server we connect to.
    ACE_SOCK_Stream client;

    //  ...And we need a connector object to establish that connection. The
    //     ACE_SOCK_Connector object provides all of the tools we need to
    //     establish a connection once we know the server's network
    //     address...
    ACE_SOCK_Connector connector;

    //  ...Which we create with an ACE_INET_Addr object. This object is
    //     given the TCP/IP port and hostname of the server we want to
    //     connect to.
    ACE_INET_Addr addr(Port, ID.c_str());
    ACE_Time_Value timeout(ACE_DEFAULT_TIMEOUT);

    //unsigned int xxx = addr.get_ip_address();
    //unsigned int xxy = addr.get_port_number();
    //const char * xxz = addr.get_host_addr();
    //  ...So, we feed the Addr object and the Stream object to the
    //     connector's connect() member function. Given this information, it
    //     will establish the network connection to the server and attach
    //     that connection to the server object.
    if (connector.connect(client, addr, &timeout) == -1)
        return;

    //
    //  Send a "Download Request" packet
    enum PakTypes {pkDownloadRequest = 0xA0,
                   pkDownloadRequestAck,
                   pkDownloadDone,
                   pkDownloadDoneAck,
                   pkSectionHeader,
                   pkSectionHeaderAck,
                   pkSectionDataAck};

    int packet[3];
    packet[0] = pkDownloadRequest;
    packet[1] = 0;
    packet[2] = 0;

    if (client.send_n(packet, sizeof(packet), &timeout) == -1)
        return;

    //
    //  Wait for "ack" packet
    if (client.recv_n(packet, sizeof(packet), &timeout) == -1)
        return;

    //
    //  Initiate Download...
    Eof = false;
    DLThread = new CoffDownloadThread(this);
    DLThread->CoffLoad(FCoffFile);

    while (!Eof)
        {
        PacketAvailable.WaitFor();
        if (Eof)
            break;

        //
        //  Send Header
        int Header[3];
        int Reply[3];
        Header[0] = pkSectionHeader;
        Header[1] = PacketAddr;
        Header[2] = PacketSize;

        if (client.send_n(Header, sizeof(Header), &timeout) == -1)
            break;
        if (client.recv_n(Reply, sizeof(Reply), &timeout) == -1)
            break;

        //
        //  Send Data
        int DReply[3];
        int size = (PacketSize > 16384) ? 16384 : PacketSize;

        if (client.send_n(PacketData, size, &timeout) == -1)
            break;

        if (client.recv_n(DReply, sizeof(DReply), &timeout) == -1)
            break;

        PacketAvailable.Clear();
        TransmitComplete.Set();
        }

    //
    //  Send final command to start program...
    packet[0] = pkDownloadDone;
    packet[1] = 0;
    packet[2] = 0;

    if (client.send_n(packet, sizeof(packet), &timeout) == -1)
        return;

    //
    //  Wait for "ack" packet
    if (client.recv_n(packet, sizeof(packet), &timeout) == -1)
        return;

    //  Close the connection to the server.  The servers we've created so
    //  far all are based on the ACE_Reactor.  When we close(), the
    //  server's reactor will see activity for the registered event
    //  handler and invoke handle_input().  That, in turn, will try to
    //  read from the socket but get back zero bytes.  At that point, the
    //  server will know that we've closed from our side.
    client.close();

    //delete DLThread; // ???
    CoffLoadComplete.Set();
    delete DLThread;
    DLThread = 0;
}

//------------------------------------------------------------------------------
//  NetCoffLoadThread::PostBlock() -- Begin transmission of segment block
//------------------------------------------------------------------------------

void NetCoffLoadThread::PostBlock(unsigned int Addr,
                                  unsigned int Size,
                                  const unsigned char * Data)
{
    PacketAddr = Addr;
    PacketSize = Size;
    PacketData = Data;

    TransmitComplete.Clear();
    PacketAvailable.Set();

    TransmitComplete.WaitFor();
}

//------------------------------------------------------------------------------
//  NetCoffLoadThread::AckBlock() -- Acknowlege block transmission
//------------------------------------------------------------------------------

void NetCoffLoadThread::AckBlock()
{
    PacketTransmitted.Set();    // not needed?
}

//------------------------------------------------------------------------------
//  NetCoffLoadThread::DoCoffMemWrite() -- Event Handler
//------------------------------------------------------------------------------
//  This function handles the COFF section write event signal during the COFF
//    load. It is called for each section or portion of section in the file
//    The COFF exectution should not proceed until the block is completely xferred.

void NetCoffLoadThread::DoCoffMemWrite(MemWriteEvent & Event)
{
    PostBlock(Event.Addr, Event.Bytes, Event.Buffer);
}

//------------------------------------------------------------------------------
//  NetCoffLoadThread::EndOfFile() -- Signal end of COFF load
//------------------------------------------------------------------------------
//  Called by download thread when finished with COFF file.

void NetCoffLoadThread::EndOfFile()
{
    Eof = true;
    PacketAvailable.Set();
}

//------------------------------------------------------------------------------
//  NetCoffLoadThread::IsComplete() -- return state of coff download
//------------------------------------------------------------------------------

bool NetCoffLoadThread::IsComplete()
{
    return CoffLoadComplete.Get();
}

}   // namespace Innovative
