// NetInfoCommThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#define ACE_NLOGGING
#include <ace/Log_Msg.h>
#include <ace/SOCK_Connector.h>
#include <ace/OS_NS_string.h>
#include <ace/OS_NS_stdio.h>
#include <ace/Time_Value.h>
#include <NetInfoCommThread_Mb.h>
#include <Sbc6713e_Mb.h>

namespace   Innovative
{

//------------------------------------------------------------------------------
//  constructor of class NetInfoCommThread
//------------------------------------------------------------------------------
NetInfoCommThread::NetInfoCommThread(Sbc6713e&  baseboard,
                                     Sbc6713e::MiscCommId which,
                                     int /*Arg0*/, int /*Arg1*/)
    : Baseboard(baseboard), Port(baseboard.InfoCommPort()), 
      ID(baseboard.Id()),   ReqType(which), 
      InfoRqstComplete(Event::ManualReset),
      client(0)
{
    Resume();
}


//------------------------------------------------------------------------------
//  NetInfoCommThread::Execute() -- Thread routine
//------------------------------------------------------------------------------
void NetInfoCommThread::Execute()
{

    InfoRqstComplete.Clear();

    //  Attempt Connection to Server
    //  ...Build ourselves a Stream socket. This is a connected socket that
    //     provides reliable end-to-end communications. We will use the
    //     server object to send data to the server we connect to.
    client = new ACE_SOCK_Stream;

    //  ...And we need a connector object to establish that connection. The
    //     ACE_SOCK_Connector object provides all of the tools we need to
    //     establish a connection once we know the server's network
    //     address...
    ACE_SOCK_Connector  connector;

    //  ...Which we create with an ACE_INET_Addr object. This object is
    //     given the TCP/IP port and hostname of the server we want to
    //     connect to.
    ACE_INET_Addr       addr(Port, ID.c_str());
    ACE_Time_Value      timeout(3);

    //  ...So, we feed the Addr object and the Stream object to the
    //     connector's connect() member function. Given this information, it
    //     will establish the network connection to the server and attach
    //     that connection to the server object.
    if (connector.connect(*client, addr, &timeout) == -1)
        {
        ProcessStatusEvent evt(std::string("Connection failure"));
        Baseboard.OnInfoRqstError.Execute(evt);
        return;
        }

    //
    //  Send an "Info Request" packet
    enum PakTypes
    {
        pkFirmwareInfoRequest = 0x50,
        pkBootAppInfoRequest,
        pkNetInfoRequest,
        pkSetBootApp,
        pkWriteNetInfo,
        pkInfoRequestAck,
        pkWriteBdRev
    };

    int packet[3];                              // Data to send

    switch(ReqType)
        {
        case Sbc6713e::rdFware:
             packet[0] = pkFirmwareInfoRequest;
             break;
        case Sbc6713e::rdAppBoot:
             packet[0] = pkBootAppInfoRequest;
             break;
        case Sbc6713e::rdNetInfo:
             packet[0] = pkNetInfoRequest;
             break;
        case Sbc6713e::wrAppBoot:
             packet[0] = pkSetBootApp;
             break;
        default:
             ProcessStatusEvent evt(std::string("Invalid request type"));
             Baseboard.OnInfoRqstError.Execute(evt);
             return;
        }

        packet[1] = 0;
        packet[2] = 0;

        // ---- Send command.
        if (client->send_n(packet, sizeof(packet), &timeout) == -1)
            {
            ProcessStatusEvent evt(std::string("Target timeout sending info. command request"));
            Baseboard.OnInfoRqstError.Execute(evt);
            return;
            }

        // --- Receive info.
        int recvPacket[200];
        if (client->recv_n(recvPacket, sizeof(recvPacket), &timeout) == -1)
            {
            ProcessStatusEvent evt(std::string("Target timeout waiting for data"));
            Baseboard.OnInfoRqstError.Execute(evt);
            return;
            }

        // Read/Write firmware revision.
        int intf   = recvPacket[1];
        int lan    = recvPacket[2];
        int client = recvPacket[3];
        Baseboard.TargetFwareInfo(client, intf, lan);

        // Client versions prior to 2.4 do not
        // supply S/N nor Board rev. Read IP info.
        if(Baseboard.ClientFwareVersion() <= 0x0204 )
            {
            // Read board IP settings, each 4 ints except domian = 20 ints.
            char * boardIpAddr = reinterpret_cast<char *>(& recvPacket[4]);
            char * boardMask = reinterpret_cast<char *>(& recvPacket[8]);
            char * gwIpAddr = reinterpret_cast<char *>(& recvPacket[12]);
            char * dns = reinterpret_cast<char *>(& recvPacket[16]);
            char * domain = reinterpret_cast<char *>(& recvPacket[20]);
            Baseboard.StaticIPInfo(boardIpAddr, boardMask, gwIpAddr, dns, domain);
            }
        else if(Baseboard.ClientFwareVersion() <= 0x0209 )
            {
            // Client versions after 2.5 supply S/N and rev.
            // But no info on using Static IP vs. DHCP.
            short *snPtr = reinterpret_cast<short *>(& recvPacket[4]);
            short sn = *snPtr;
            char *revPtr = reinterpret_cast<char*>(& recvPacket[5]);
            char rev = *revPtr;
            Baseboard.SerialNoBoardRev(sn, rev);

            // Read board IP settings
            char * boardIpAddr = reinterpret_cast<char *>(& recvPacket[6]);
            char * boardMask = reinterpret_cast<char *>(& recvPacket[10]);
            char * gwIpAddr = reinterpret_cast<char *>(& recvPacket[14]);
            char * dns = reinterpret_cast<char *>(& recvPacket[18]);
            char * domain = reinterpret_cast<char *>(& recvPacket[22]);
            Baseboard.StaticIPInfo(boardIpAddr, boardMask, gwIpAddr, dns, domain);
            }
        else
            {
            // Client versions after 3.0 supply also
            // info on using Static IP vs. DHCP.
            short *snPtr = reinterpret_cast<short *>(& recvPacket[4]);
            short sn = *snPtr;
            char *revPtr = reinterpret_cast<char*>(& recvPacket[5]);
            char rev = *revPtr;
            Baseboard.SerialNoBoardRev(sn, rev);
            bool useStatic = recvPacket[6] ? true : false;
            Baseboard.UseStaticIp(useStatic);

            // Read board IP settings
            char * boardIpAddr = reinterpret_cast<char *>(& recvPacket[7]);
            char * boardMask = reinterpret_cast<char *>(& recvPacket[11]);
            char * gwIpAddr = reinterpret_cast<char *>(& recvPacket[15]);
            char * dns = reinterpret_cast<char *>(& recvPacket[19]);
            char * domain = reinterpret_cast<char *>(& recvPacket[23]);
            Baseboard.StaticIPInfo(boardIpAddr, boardMask, gwIpAddr, dns, domain);
            }

        ProcessCompletionEvent eventComplete(recvPacket[1]);
        Baseboard.OnInfoRqstComplete.Execute(eventComplete);
}

//------------------------------------------------------------------------------
//  NetImageLoadThread::Done() -- Clean-up
//------------------------------------------------------------------------------

void NetInfoCommThread::Done()
{
    //  Close the connection to the server.  The servers we've created so
    //  far all are based on the ACE_Reactor.  When we close(), the
    //  server's reactor will see activity for the registered event
    //  handler and invoke handle_input().  That, in turn, will try to
    //  read from the socket but get back zero bytes.  At that point, the
    //  server will know that we've closed from our side.
    if (client)
        client->close();
    InfoRqstComplete.Set();
}



}// namespace Innovative.
