// NetInfoCommThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#define ACE_NLOGGING
#include <ace/Log_Msg.h>
#include <ace/SOCK_Connector.h>
#include <ace/OS_NS_string.h>
#include <ace/OS_NS_stdio.h>
#include <ace/Time_Value.h>
#include <NetSystemTestThread_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <Sbc6713e_Mb.h>

namespace   Innovative
{

//------------------------------------------------------------------------------
//  constructor of class NetSystemTestThread
//------------------------------------------------------------------------------
NetSystemTestThread::NetSystemTestThread(Sbc6713e&  baseboard)
    : Baseboard(baseboard), Port(baseboard.SystemTestPort()),
      ID(baseboard.Id()), client(0), RecvBuffer(SystemTestSizeInts)
{
    Resume();   // Init(), then Execute() will be called.
}

//------------------------------------------------------------------------------
// NetSystemTestThread::Init()
//------------------------------------------------------------------------------

void NetSystemTestThread::Init()
{
    inherited::Init();
    AddObject(BeginTransfer, sBeginTransfer);
}

//------------------------------------------------------------------------------
//  NetInfoCommThread::Execute() -- Thread routine
//------------------------------------------------------------------------------
void NetSystemTestThread::Execute()
{
    //  Attempt Connection to Server
    //  ...Build ourselves a Stream socket. This is a connected socket that
    //     provides reliable end-to-end communications. We will use the
    //     server object to send data to the server we connect to.
    client = new ACE_SOCK_Stream;

    //  ...And we need a connector object to establish that connection. The
    //     ACE_SOCK_Connector object provides all of the tools we need to
    //     establish a connection once we know the server's network
    //     address...
    ACE_SOCK_Connector  connector;

    //  ...Which we create with an ACE_INET_Addr object. This object is
    //     given the TCP/IP port and hostname of the server we want to
    //     connect to.
    ACE_INET_Addr       addr(Port, ID.c_str());
    ACE_Time_Value      timeout(3);

    //  ...So, we feed the Addr object and the Stream object to the
    //     connector's connect() member function. Given this information, it
    //     will establish the network connection to the server and attach
    //     that connection to the server object.
    if (connector.connect(*client, addr, &timeout) == -1)
        {
        OpenWire::NotifyEvent e;
        Baseboard.OnSystemTestErrror.Execute(e);
        return;
        }

    //
    // Wait for request.
    while (!Terminated())
        {
        int why = WaitFor(45000);
        switch (why)
            {
            case sTerminate:
                {
                continue;
                }
            case MultipleObjects::Timeout:
                {
                continue;
                }
            case sBeginTransfer:
                break;
            default:
                continue;
            }
        // Resquest test results.
        int packet[4];
        packet[0] = rqstTestInfo;
        packet[1] = 0;
        packet[2] = 0;
        packet[3] = 0;
        
        if (client->send_n(packet, sizeof(packet), &timeout) == -1)
            {
            OpenWire::NotifyEvent e;
            Baseboard.OnSystemTestErrror.Execute(e);
            continue;
            }

        // --- Receive info.
        int dataPtr[6];
        if (client->recv_n(dataPtr, sizeof(dataPtr), &timeout) == -1)
            {
            OpenWire::NotifyEvent e;
            Baseboard.OnSystemTestErrror.Execute(e);
            continue;
            }
        IntegerDG RxDG(RecvBuffer);
        RxDG[0] = dataPtr[0];
        RxDG[1] = dataPtr[1];
        RxDG[2] = dataPtr[2];
        RxDG[3] = dataPtr[3];
        RxDG[4] = dataPtr[4];
        RxDG[5] = dataPtr[5];

        // Done
        TransmitComplete.Set();
        BeginTransfer.Clear();
        }
    client->close();
    delete client;
    client = 0;
}

//------------------------------------------------------------------------------
// NetSystemTestThread::GetSystemTestInfo()
//------------------------------------------------------------------------------

void NetSystemTestThread::GetSystemTestInfo(SystemTestInfo & info)
{
    CriticalSectionArea CSA(CS);    // Lock function...
    TransmitComplete.Clear();
    BeginTransfer.Set();

    TransmitComplete.WaitFor();
    IntegerDG BufDG(RecvBuffer);

    info.DataErrors          = BufDG[0];
    info.AddrErrors          = BufDG[1];
    info.F6713ClockCount     = BufDG[2];
    info.F642ClockCount      = BufDG[3];
    info.BootJumperInstalled = BufDG[4];
    info.Fallow              = BufDG[5];
}


}// namespace Innovative.
