// NetTargetResetThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#define ACE_NLOGGING
#include <ace/Log_Msg.h>
#include <ace/SOCK_Connector.h>
#include <ace/OS_NS_string.h>
#include <ace/OS_NS_stdio.h>
#include <ace/Time_Value.h>
#include <NetTargetResetThread_Mb.h>
#include <Sbc6713e_Mb.h>

namespace   Innovative
{

//------------------------------------------------------------------------------
//  constructor of class NetInfoCommThread
//------------------------------------------------------------------------------
NetTargetResetThread::NetTargetResetThread(Sbc6713e&  baseboard,
                                     Sbc6713e::MiscCommId which,
                                     int /*Arg0*/, int /*Arg1*/)
    : Baseboard(baseboard), Port(baseboard.ResetPort()), ID(baseboard.Id()),
      ReqType(which), TargetResetComplete(Event::ManualReset), client(0)
{
    Resume();
}


//------------------------------------------------------------------------------
//  NetTargetResetThread::Execute() -- Thread routine
//------------------------------------------------------------------------------
void NetTargetResetThread::Execute()
{

    TargetResetComplete.Clear();

    //  Attempt Connection to Server
    //  ...Build ourselves a Stream socket. This is a connected socket that
    //     provides reliable end-to-end communications. We will use the
    //     server object to send data to the server we connect to.
    client = new ACE_SOCK_Stream;

    //  ...And we need a connector object to establish that connection. The
    //     ACE_SOCK_Connector object provides all of the tools we need to
    //     establish a connection once we know the server's network
    //     address...
    ACE_SOCK_Connector  connector;

    //  ...Which we create with an ACE_INET_Addr object. This object is
    //     given the TCP/IP port and hostname of the server we want to
    //     connect to.
    ACE_INET_Addr       addr(Port, ID.c_str());
    ACE_Time_Value      timeout(3);

    //  ...So, we feed the Addr object and the Stream object to the
    //     connector's connect() member function. Given this information, it
    //     will establish the network connection to the server and attach
    //     that connection to the server object.
    if (connector.connect(*client, addr, &timeout) == -1)
        {
        ProcessStatusEvent evt(std::string("Connection failure"));
        Baseboard.OnTargetResetError.Execute(evt);
        return;
        }

    //
    //  Send an "Info Request" packet
    enum PakTypes
    { pkFirmwareInfoRequest = 0x50, pkBootAppInfoRequest, pkNetInfoRequest,
        pkSetBootApp, pkWriteNetInfo, pkInfoRequestAck,  pkWriteBdRev,
		pkMasterReset, pkMasterResetAck, NAK
    };

    int packet[3];                              // Data to send

    //PakTypes PkReqType;
	if(ReqType == Sbc6713e::resetTarget )
		{    
		packet[0] = pkMasterReset;
		}
	else
		{
		ProcessStatusEvent evt(std::string("Invalid request type"));
        Baseboard.OnInfoRqstError.Execute(evt);
        return;
		}
        
    packet[1] = 0;
    packet[2] = 0;

    // ---- Send command.
    if (client->send_n(packet, sizeof(packet), &timeout) == -1)
		{
        ProcessStatusEvent evt(std::string("Target timeout sending command acknowlegement"));
        Baseboard.OnTargetResetError.Execute(evt);
        return;
        }
    ProcessCompletionEvent eventComplete(0);
    Baseboard.OnTargetResetComplete.Execute(eventComplete);
}

//------------------------------------------------------------------------------
//  NetImageLoadThread::Done() -- Clean-up
//------------------------------------------------------------------------------

void NetTargetResetThread::Done()
{
    //  Close the connection to the server.  The servers we've created so
    //  far all are based on the ACE_Reactor.  When we close(), the
    //  server's reactor will see activity for the registered event
    //  handler and invoke handle_input().  That, in turn, will try to
    //  read from the socket but get back zero bytes.  At that point, the
    //  server will know that we've closed from our side.
    if (client)
        client->close();
    TargetResetComplete.Set();
}



}// namespace Innovative.
