// NetTransferThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#define ACE_NLOGGING

#include <algorithm>
#include <ace/Log_Msg.h>
#include <ace/SOCK_Connector.h>
#include <ace/OS_NS_string.h>
#include <ace/OS_NS_stdio.h>
#include <ace/OS_NS_errno.h>
#include <ace/Time_Value.h>
#include <BufferDatagrams_Mb.h>
#include <NetTransferThread_Mb.h>
#include <Sbc6713e_Mb.h>
#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>

namespace   Innovative
{

//==============================================================================
//  CLASS NetworkCommunicator -- Helper class to send and receive data to Sbc6713e
//==============================================================================

class NetworkCommunicator : public INetworkCommunicator
{
public:
    NetworkCommunicator(std::string addr, int port, Sbc6713e & baseboard);
    ~NetworkCommunicator(){};
    bool Connect();
    void Close();
    int  Send(int * packet, int sizeBytes, bool Timed, RequestorID id);
    int  Recv(int * packet, int sizeBytes, bool Timed, RequestorID id);
    void Halt(RequestorID id);

 private:
    ACE_SOCK_Connector  Connector;
    ACE_INET_Addr       IpAddress;
    ACE_SOCK_Stream     Client;
    ACE_Time_Value      Timeout;
    bool                FHaltSend;
    bool                FHaltRecv;
    bool                FConnected;
    Sbc6713e        &   Baseboard;
};

//------------------------------------------------------------------------------
// NetworkCommunicator::NetworkCommunicator()
//------------------------------------------------------------------------------

NetworkCommunicator::NetworkCommunicator(std::string addr, int port, Sbc6713e & baseboard)
    : IpAddress(port, addr.c_str()), Timeout(ACE_DEFAULT_TIMEOUT), FConnected(false),
      Baseboard(baseboard)
{
}

//------------------------------------------------------------------------------
// NetworkCommunicator::Halt(RequestorID id)
//------------------------------------------------------------------------------

void NetworkCommunicator::Halt(RequestorID id)
{
    if( id == Tx)
        FHaltSend = true;
    else
        FHaltRecv = true;
}

//------------------------------------------------------------------------------
// NetworkCommunicator::Connect()
//------------------------------------------------------------------------------

bool NetworkCommunicator::Connect()
{
    if (!FConnected)
        if (Connector.connect(Client, IpAddress, &Timeout) == -1)
            {
            FConnected = false;
            }
        else
            FConnected = true;

    return FConnected;
}

//------------------------------------------------------------------------------
// NetworkCommunicator::Close()
//------------------------------------------------------------------------------

void NetworkCommunicator::Close()
{
    if (FConnected)
        Client.close();
    FConnected = false;
}

//------------------------------------------------------------------------------
// NetworkCommunicator::Send() - Send data.
//------------------------------------------------------------------------------

int NetworkCommunicator::Send(int * srcDataPtr, int sizeBytes, bool Timed, RequestorID id)
{
    FHaltSend = false;
    ACE_OS::last_error(0);

    int sentBytes = 0;
    int bytesLeft = sizeBytes;
    int result;                     // Needs to be a signed number.

    char * localPtr = reinterpret_cast<char *>(srcDataPtr);

    if (!Connect())
        return 0;

    while(sentBytes < sizeBytes)
        {
        result = static_cast<int>(Client.send_n(localPtr, bytesLeft, &Timeout));
        if (result > 0)
            {
            if (result > bytesLeft)
                {
                Close();
                return 0;
                }
            else
                {
                sentBytes   += result;
                bytesLeft   -= result;
                localPtr    += result;
                }
            }
        // The server closed the socket, got
        // 0 bytes, have application handle this.
        else if (result == 0)
            {
            Close();
            return 0;
            }
        // Check for exception conditions.
        else if ((FHaltRecv && (id == Rx)) || (FHaltSend && (id == Tx)))
            {
            Close();
            return 0;
            }
        // Obviously result = -1 here.
        else
            {
            int errorNo;
            errorNo = ACE_OS::last_error();
            if (((errorNo == ETIMEDOUT) || (errorNo == ETIME) || (errorNo == 0)) && Timed)
                {
                Close();
                return 0;
                }
            else if (((errorNo == ETIMEDOUT) || (errorNo == ETIME) || (errorNo == 0)) && !Timed)
                {
                OpenWire::NotifyEvent e;
                if (id == Tx)
                    Baseboard.OnSendTimeout.Execute(e);
                else
                    Baseboard.OnRecvTimeout.Execute(e);
                }
            // Other errors.
            else
                {
                Close();
                return 0;
                }
            }            
        }
        
    return sentBytes;
}

//------------------------------------------------------------------------------
// NetworkCommunicator::Recv() - Receive data.
//------------------------------------------------------------------------------

int NetworkCommunicator::Recv(int * srcDataPtr, int sizeBytes, bool Timed, RequestorID id)
{
    FHaltRecv = false;
    ACE_OS::last_error(0);
    if (!Connect())
        return 0;

    int receivedBytes = 0;
    int bytesLeft = sizeBytes;
    int result;
    char * localPtr = reinterpret_cast<char *>(srcDataPtr);

    while(receivedBytes < sizeBytes)
        {
        result = static_cast<int>(Client.recv_n(localPtr, bytesLeft, &Timeout));
        if ( result > 0 )
            {
            if (result > bytesLeft)
                {
                Close();
                return 0;
                }
            else
                {
                receivedBytes += result;
                bytesLeft     -= result;
                localPtr      += result;
                }
            }
        // This means the server closed the socket
        // and got 0 bytes.
        else if (result == 0)
            {
            Close();
            return 0;
            }
        else if ((FHaltRecv && (id == Rx)) || (FHaltSend && (id == Tx)))
            {
            Close();
            return 0;
            }
        else
            {
            int errorNo;
            errorNo = ACE_OS::last_error();
            if (((errorNo == ETIMEDOUT) || (errorNo == ETIME) || (errorNo == 0)) && Timed)
                {
                Close();
                return 0;
                }
            else if(((errorNo == ETIMEDOUT) || (errorNo == ETIME) || (errorNo == 0)) && !Timed)
                {
                OpenWire::NotifyEvent e;
                if (id == Tx)
                    Baseboard.OnSendTimeout.Execute(e);
                else
                    Baseboard.OnRecvTimeout.Execute(e);
                }
            else
                {
                Close();
                return 0;
                }
            }
        }

    return receivedBytes;
}

//==============================================================================
//  CLASS NetOutboundXfrThread -- Data to Sbc6713e
//==============================================================================
//------------------------------------------------------------------------------
// NetOutboundXfrThread::NetOutboundXfrThread() --  Constructor.
//------------------------------------------------------------------------------

NetOutboundXfrThread::NetOutboundXfrThread(Sbc6713e&  baseboard)
        : BeginTransfer(Event::ManualReset),
          HaltCommunication(Event::ManualReset),
          TransmitComplete(Event::ManualReset), 
          ID(baseboard.Id()), Port(baseboard.OutXfrPort()),
          Baseboard(baseboard),
          Communicator(0)
{
    Communicator = new NetworkCommunicator(ID, Port, baseboard);
    BeginTransfer.Clear();
    HaltCommunication.Clear();
    TransmitComplete.Clear();
    Resume();
}

//------------------------------------------------------------------------------
// NetOutboundXfrThread::~NetOutboundXfrThread() -- Destructor.
//------------------------------------------------------------------------------

NetOutboundXfrThread::~NetOutboundXfrThread()
{
    Shutdown(true);
	FStopped.Acquire();
    delete Communicator;
    Communicator = 0;
}

//------------------------------------------------------------------------------
//  NetOutboundXfrThread::Init() -- Initialize sync objects
//------------------------------------------------------------------------------

void NetOutboundXfrThread::Init()
{
    inherited::Init();
    AddObject(BeginTransfer, sBeginTransfer);
    AddObject(HaltCommunication, sHaltCommunication);
}

//------------------------------------------------------------------------------
//  NetOutboundXfrThread::Execute() -- Thread routine
//------------------------------------------------------------------------------

void NetOutboundXfrThread::Execute()
{
    while (!Terminated())
        {
        //
        //  Wait for something to send
        //
        int why = WaitFor(45000);
        if (why == MultipleObjects::Timeout)
            {
            int ClosePacket[5];
            ClosePacket[0] = xcClose;
            ClosePacket[1] = 0;
            ClosePacket[2] = 0;
            if (Communicator->Send(ClosePacket, sizeof(ClosePacket), true, Tx) <= 0)
                {
                // No response from
                // target within 5 sec.
                continue;
                }

            //
            //  Wait for "ack" packet
            int CloseReply[3];
            if (Communicator->Recv(CloseReply, sizeof(CloseReply), true, Tx) <= 0)
                {
                // No response from
                // target within 5 sec.
                continue;
                }

            Communicator->Close();
            continue;
            }// Timeout

        if (why == sHaltCommunication)
            {
            SendStatus = Sbc6713e::CommAborted;
            TransmitComplete.Set();
            continue;
            }

        if (why == sTerminate)
            continue;   // Will exit on next iteration.

        //-------------------------------
        //  BeginTransfer was signalled.
        //  Begin data transfer.
        //-------------------------------
        BeginTransfer.Clear();
        SendStatus = Sbc6713e::UnknownErr;

        //
        //  Check what kind of transfer
        XfrCmdTypes XfrType = static_cast<XfrCmdTypes>(Packet[0]);

        //---------------------
        //  Send packet type.
        //---------------------
        if (Communicator->Send(Packet, sizeof(Packet), false, Tx) <= 0)
			{
			// No response from
			// target within 5 sec.
            SendStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
        //---------------------

        //---------------------
        //  Wait for "ack" packet
        //---------------------
        int Reply[3];
        if (Communicator->Recv(Reply, sizeof(Reply), false, Tx) <= 0)
            {
            // No response from
            // target within 5 sec.
            SendStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
		if (Reply[0] != Sbc6713e::Success)
			{
			// C6713 timeout.
            Communicator->Close();
            SendStatus = (Sbc6713e::CommStatusCodes)Reply[0];
            TransmitComplete.Set();
            continue;
            }

        //---------------------

        //---------------------
        // If xcClose, just close,
        // we already told target.
        //---------------------
        if (XfrType == xcClose)
            {
            Communicator->Close();
            SendStatus = Sbc6713e::Success;
            TransmitComplete.Set();
            continue;
            }
        //---------------------

        if (XfrType == xcSend)
            {
            //---------------------
            // Data Packet.
            //---------------------
            int      TransferSize = static_cast<int>(xferBuffer->SizeInInts() * sizeof(int));
            int *    BufAddr = xferBuffer->Data();

            //-----------------
            //  Send pkt header
            int Header[3];
            Header[0] = xcSendData;
            Header[1] = static_cast<int>(TransferSize);
            Header[2] = 0;
            if (Communicator->Send(Header, sizeof(Header), false, Tx) <= 0)
                {
                // No response from
                // target within 5 sec.
                SendStatus = Sbc6713e::NetworkTimeoutErr;
                TransmitComplete.Set();
                continue;
                }
            //-----------------

            //-----------------
            // Get Ack
            //-----------------
            if (Communicator->Recv(Reply, sizeof(Reply), false, Tx) <= 0)
                {
                // No response from
                // target within 5 sec.
                SendStatus = Sbc6713e::NetworkTimeoutErr;
                TransmitComplete.Set();
                continue;
                }
            //-----------------

            //-----------------
            //  Send Packet data
            //-----------------
            if (Communicator->Send(BufAddr, TransferSize, false, Tx) <= 0)
                {
                // No response from
                // target within 5 sec.
                SendStatus = Sbc6713e::NetworkTimeoutErr;
                TransmitComplete.Set();
                continue;
                }
            //-----------------

            //-----------------
            // Get Ack
            //-----------------
            if (Communicator->Recv(Reply, sizeof(Reply), false, Tx) <= 0)
                {
				// No response from
                // target within 5 sec.
                SendStatus = Sbc6713e::NetworkTimeoutErr;
                TransmitComplete.Set();
                continue;
                }
			if (Reply[0] != Sbc6713e::Success)
                {
                Communicator->Close();
                SendStatus = (Sbc6713e::CommStatusCodes)Reply[0];
                TransmitComplete.Set();
                continue;
				}
            }

        //
        //  Done!
        SendStatus = Sbc6713e::Success;
        TransmitComplete.Set();
        }

    //  Close the connection to the server.  The servers we've created so
    //  far all are based on the ACE_Reactor.  When we close(), the
    //  server's reactor will see activity for the registered event
    //  handler and invoke handle_input().  That, in turn, will try to
    //  read from the socket but get back zero bytes.  At that point, the
    //  server will know that we've closed from our side.
    Communicator->Close();
	FStopped.Release();
}

//------------------------------------------------------------------------------
//  NetOutboundXfrThread::Send() -- Send data block.
//  Returns:  NetworkTimeoutErr = 0, Success = 1, TargetTimeoutErr = 2, TargetErr = 3.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetOutboundXfrThread::Send(Buffer&  buffer)
{
    CriticalSectionArea CSA(CS);    // Lock function...

    // Save packet information...
    Packet[0] = xcSend;
    Packet[1] = static_cast<int>(buffer.SizeInInts() * sizeof( int ));
    Packet[2] = 0;
    Packet[3] = 0;
    Packet[4] = 0;
    xferBuffer = &buffer;

    TransmitComplete.Clear();
    HaltCommunication.Clear();
    BeginTransfer.Set();

    TransmitComplete.WaitFor();
    return SendStatus;
}

//------------------------------------------------------------------------------
//  NetOutboundXfrThread::SendCommand() -- Send Command.
//  Returns:  NetworkTimeoutErr = 0, Success = 1, TargetTimeoutErr = 2, TargetErr = 3.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetOutboundXfrThread::SendCommand(short CommandId, int Arg0, int Arg1,
                                       int Arg2)
{
    CriticalSectionArea CSA(CS);    // Lock function...

    // Save packet information...
    Packet[0] = xcSendCommand;
    Packet[1] = CommandId;
    Packet[2] = Arg0;
    Packet[3] = Arg1;
    Packet[4] = Arg2;

    TransmitComplete.Clear();
    HaltCommunication.Clear();
    BeginTransfer.Set();

    TransmitComplete.WaitFor();
    return SendStatus;
}

//------------------------------------------------------------------------------
//  NetOutboundXfrThread::SendClose() -- Cancel a pending transfer.
//  Returns:  NetworkTimeoutErr = 0, Success = 1, TargetTimeoutErr = 2, TargetErr = 3.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetOutboundXfrThread::SendClose()
{
    CriticalSectionArea CSA(CSAbort);    // Lock function...
    TransmitComplete.Clear();
    BeginTransfer.Clear();
    HaltCommunication.Set();
    Communicator->Halt(Tx);
    TransmitComplete.WaitFor();
    return SendStatus;
}

//------------------------------------------------------------------------------
//  NetOutboundXfrThread::ClearPendingXfrs()
//  Returns:  NetworkTimeoutErr = 0, Success = 1, TargetTimeoutErr = 2, TargetErr = 3.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetOutboundXfrThread::ClearPendingXfrs()
{
    CriticalSectionArea CSA(CS);
    Packet[0] = xcCleanIpFifo;
    Packet[1] = 0;
    Packet[2] = 0;
    Packet[3] = 0;
    Packet[4] = 0;

    TransmitComplete.Clear();
    HaltCommunication.Clear();
    BeginTransfer.Set();

    TransmitComplete.WaitFor();
    return SendStatus;
}

//==============================================================================
//  CLASS NetInboundXfrThread -- Data from Sbc6713e
//==============================================================================
//------------------------------------------------------------------------------
//  constructor of class NetInboundXfrThread
//------------------------------------------------------------------------------

NetInboundXfrThread::NetInboundXfrThread(Sbc6713e&  baseboard)
        : BeginTransfer(Event::ManualReset),
          HaltCommunication(Event::ManualReset),
          TransmitComplete(Event::ManualReset),
          ID(baseboard.Id()), Port(baseboard.InXfrPort()),
          Baseboard(baseboard),
          Communicator(0)
{
    Communicator = new NetworkCommunicator(ID, Port, baseboard);
    BeginTransfer.Clear();
    HaltCommunication.Clear();
    TransmitComplete.Clear();

    Resume();
}

//------------------------------------------------------------------------------
// NetInboundXfrThread::~NetInboundXfrThread() -- Destructor.
//------------------------------------------------------------------------------

NetInboundXfrThread::~NetInboundXfrThread()
{
    Shutdown(true);
	FStopped.Acquire();
    delete Communicator;
    Communicator = 0;
}

//------------------------------------------------------------------------------
//  NetInboundXfrThread::Init() -- Initialize sync objects
//------------------------------------------------------------------------------

void NetInboundXfrThread::Init()
{
    inherited::Init();
    AddObject(BeginTransfer, sBeginTransfer);
    AddObject(HaltCommunication, sHaltCommunication);
}

//------------------------------------------------------------------------------
//  NetInboundXfrThread::Execute() -- Thread routine
//------------------------------------------------------------------------------

void NetInboundXfrThread::Execute()
{
    // Main loop.
    while (!Terminated())
        {
        // Wait for Application Recv to be posted.
        // OutputDebugString("...Waitfor");
        int why = WaitFor(45000);
        if (why == MultipleObjects::Timeout)
            {
            int ClosePacket[3];
            ClosePacket[0] = xcClose;
            ClosePacket[1] = 0;
            ClosePacket[2] = 0;
            if (Communicator->Send(ClosePacket, sizeof(ClosePacket), true, Rx) <= 0)
                {
                // No response from target.
                continue;
                }

            //
            //  Wait for "ack" packet
            int CloseReply[3];
            if (Communicator->Recv(CloseReply, sizeof(CloseReply), true, Rx) <= 0)
                {
                // No response from target.
                continue;
                }

            // Close socket.
            Communicator->Close();
            continue;
            }// Timeout from application, back to main loop.

        if (why == sHaltCommunication)
            {
            ReceiveStatus = Sbc6713e::CommAborted;
            TransmitComplete.Set();
            continue;
            }

        if (why == sTerminate)
            continue; // Will take us out on
                      // next iteration.

        //-------------------------------
        //  BeginTransfer was signalled.
        //  Begin data transfer.
        //-------------------------------
        BeginTransfer.Clear();
		ReceiveStatus = Sbc6713e::UnknownErr; 	// default return value.;

		//-------------------------
        // Send recevice request.
        // Use timeout.
        //-------------------------
        int xReply[3];
        xReply[0] = xcRecvPosted;
        xReply[1] = 0;
        xReply[2] = 0;
        if (Communicator->Send(xReply, sizeof(xReply), false, Rx) <= 0) // Tiemout
            {
            // No response from target.
            ReceiveStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
        //-------------------------

        //-------------------------
        // Get pkRecvRequest packet with total size.
        //-------------------------
        int Info[5];
        if(Communicator->Recv(Info, sizeof(Info), false, Rx) <= 0)
            {
           // No response from target or Network error.
            ReceiveStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
        //-------------------------

        //-------------------------
        //  Send ack
        //-------------------------
        int Reply[3];
        Reply[0] = pkRecvRequestAck;
        Reply[1] = 0;
        Reply[2] = 0;
        if (Communicator->Send(Reply, sizeof(Reply), false, Rx) <= 0)
            {
            // No response from target.
            ReceiveStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
        //-------------------------

        PakTypes    Type = static_cast<PakTypes>(Info[0]);
        int         TransferSize = 0;
        switch (Type)
            {
            case pkRecvCommandRequest:
                TransferSize = kCommandSize * sizeof(int);
                RT = Sbc6713e::rtCommand;
                break;
            case pkRecvPacketRequest:
                TransferSize = Info[1];
                RT = Sbc6713e::rtPacket;
                 break;
            default:
                break;
            }

        //  Resize buffer
        xferBuffer->Resize(TransferSize/sizeof(int));
        int * BufAddr = xferBuffer->Data();

        //
        //  If this is a command, we have all data.
        if (Type == pkRecvCommandRequest)
            {
            for (int i = 0; i < kCommandSize; i++)
                BufAddr[i] = Info[i + 1];

            // Done.
            ReceiveStatus = Sbc6713e::Success;
            TransmitComplete.Set();
            continue;
            }

        //--------------------
        //  If packet data
        //--------------------
        if (Communicator->Recv(BufAddr, TransferSize, false, Rx) <= 0)
			{
            // No response from target.
            ReceiveStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
        //--------------------

        //--------------------
        // Send ack. Not sure why
        // we can't reuse previous
        // data arrays.
        //--------------------
        int Header[3];
        Header[0] = pkRecvDataAck;
        Header[1] = 0;
        Header[2] = 0;
        if (Communicator->Send(Header, sizeof(Header), false, Rx) <= 0)
            {
            // No response from target.
            ReceiveStatus = Sbc6713e::NetworkTimeoutErr;
            TransmitComplete.Set();
            continue;
            }
        //--------------------

        //
        //  Done!
        ReceiveStatus = Sbc6713e::Success;
        TransmitComplete.Set();
        }

    //  Close the connection to the server.  The servers we've created so
    //  far all are based on the ACE_Reactor.  When we close(), the
    //  server's reactor will see activity for the registered event
    //  handler and invoke handle_input().  That, in turn, will try to
    //  read from the socket but get back zero bytes.  At that point, the
    //  server will know that we've closed from our side.
    Communicator->Close();
	FStopped.Release();
}

//------------------------------------------------------------------------------
//  NetInboundXfrThread::Recv() -- Recv data block. true  = success.
//                                                  false = failure.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetInboundXfrThread::Recv(Sbc6713e::RecvType&  which, Buffer&  buffer)
{
    CriticalSectionArea CSA(CS);    	// Lock function...
    // Save packet information...
    xferBuffer = &buffer;
    TransmitComplete.Clear();
    HaltCommunication.Clear();
    BeginTransfer.Set();

    TransmitComplete.WaitFor();
    which = RT;
    int * ptr = buffer.Header();
    *ptr = which;
    return ReceiveStatus;
}

//------------------------------------------------------------------------------
//  NetInboundXfrThread::Recv() -- Recv data block. true  = success.
//                                                  false = failure.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetInboundXfrThread::Recv(Buffer&  buffer)
{
    CriticalSectionArea CSA(CS);    	// Lock function...
    // Save packet information...
    xferBuffer = &buffer;
    TransmitComplete.Clear();
    HaltCommunication.Clear();
    BeginTransfer.Set();

    TransmitComplete.WaitFor();
    int * ptr = buffer.Header();
    *ptr = RT;
    return ReceiveStatus;
}

//------------------------------------------------------------------------------
// Close socket. Call this function if Recv() is called in a non back to back mode
// If you are calling Recv() continuously within 2 sec from each call , then is fine
// not to call this function, if you receive an error code from Recv() then call
// this function to clean up any past state of the socket. This is a recovery function
//  that sets this socket on to a known state.
//------------------------------------------------------------------------------

Sbc6713e::CommStatusCodes NetInboundXfrThread::RecvClose()
{
    CriticalSectionArea CSA(CSAbort);    	// Lock function...

    TransmitComplete.Clear();
    BeginTransfer.Clear();
    HaltCommunication.Set();
    Communicator->Halt(Rx);
    TransmitComplete.WaitFor();
    return Sbc6713e::Success;
}

} // namespace Innovative
