// NetTransferThread.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef NetTransferThread_MbH
#define NetTransferThread_MbH

#include <Event_Mb.h>
#include <Thread_Mb.h>
#include <Sbc6713e_Mb.h>
#include <SyncObjects_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

const int kCommandSize = 4;   // command size in words...
enum RequestorID{Tx =0, Rx};
class INetworkCommunicator
{
public:
    virtual ~INetworkCommunicator(){};
    virtual bool Connect() = 0;
    virtual void Close() = 0;
    virtual int  Send(int * packet, int sizeBytes, bool Timed, RequestorID id) = 0;
    virtual int  Recv(int * packet, int sizeBytes, bool Timed, RequestorID id) = 0;
    virtual void Halt(RequestorID id) = 0;
    
};

//==============================================================================
//  CLASS NetOutboundXfrThread -- manages Outbound transfers via network
//==============================================================================

class NetOutboundXfrThread : public Thread
{
    typedef Thread inherited;

public:
    //
    //  Member Functions
    NetOutboundXfrThread(class Sbc6713e & baseboard);
    ~NetOutboundXfrThread();
	//  ...Packet Data Transfer
	Sbc6713e::CommStatusCodes  Send(Buffer & buffer);
	Sbc6713e::CommStatusCodes  SendCommand(short CommandId, int Arg0=0, int Arg1=0, int Arg2=0);
    Sbc6713e::CommStatusCodes  SendClose();
    Sbc6713e::CommStatusCodes  ClearPendingXfrs();

protected:
    enum { sBeginTransfer=inherited::sLast, sHaltCommunication, sLast};
    //
    //  Member Functions
    void Init();
    void Execute();

private:
    // Data.
    Sbc6713e::CommStatusCodes   SendStatus;
    Event                       BeginTransfer;
    Event                       HaltCommunication;
    Event                       TransmitComplete;
    std::string                 ID;
    int                         Port;
    int                         Packet[5];
    Buffer               *      xferBuffer;
    CriticalSection             CS;
    CriticalSection             CSAbort;
    Sbc6713e             &      Baseboard;
    INetworkCommunicator *      Communicator;
	Semaphore					FStopped;

    // Methods.
    void  Close();


    enum XfrCmdTypes{ xcSend = 0xB0, xcSendCommand, xcSendData, xcClose,
                       xcSendDataData, xcCleanIpFifo };
};

//==============================================================================
//  CLASS NetInboundXfrThread -- manages Inbound transfers via network
//==============================================================================

class NetInboundXfrThread : public Thread
{
    typedef Thread inherited;

public:
    //
    //  Member Functions
    NetInboundXfrThread(class Sbc6713e & baseboard);
    ~NetInboundXfrThread();

	//  ...Packet Data Transfer
	Sbc6713e::CommStatusCodes  Recv(Sbc6713e::RecvType & which, Buffer & Buffer);
    Sbc6713e::CommStatusCodes  RecvClose();
    Sbc6713e::CommStatusCodes  Recv(Buffer&  buffer);

protected:
    enum { sBeginTransfer=inherited::sLast, sHaltCommunication, sLast};
    //
    //  Member Functions
    void  Init();
    void  Execute();
    int   RecvFromNetwork(ACE_SOCK_Stream & socket, int * srcDataPtr, int sizeBytes);
private:
    // Data.
    Sbc6713e::CommStatusCodes   ReceiveStatus;
    Event                       BeginTransfer;
    Event                       HaltCommunication;
    Event                       TransmitComplete;
    std::string                 ID;
    int                         Port;
    Sbc6713e::RecvType          RT;
    Buffer               *      xferBuffer;
    CriticalSection             CS;
    CriticalSection             CSAbort;    
    Sbc6713e             &      Baseboard;
    INetworkCommunicator *      Communicator;
	Semaphore					FStopped;

    // Methods.

    enum XfrCmdTypes{ xcRecvPosted = 0xE0, xcClose, xcCleanIpFifo };
    enum PakTypes 	{ pkRecvCommandRequest= 0xC0, pkRecvPacketRequest,
                       pkRecvRequestAck, pkRecvData, pkRecvDataAck
                    };	
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif


