// NotifyThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//
 
#include <NotifyThread_Mb.h>

namespace   Innovative
{
 
//===========================================================================
//  CLASS NotifyThread  --  
//===========================================================================
//---------------------------------------------------------------------------
// NotifyThread::NotifyThread() --
//---------------------------------------------------------------------------
 
NotifyThread::NotifyThread(const char * name)
    : inherited(name), mOwnsEvent(true)
{
    eNotifyEvent = new Event(Event::AutoReset);
}

//---------------------------------------------------------------------------
// NotifyThread::NotifyThread() --
//---------------------------------------------------------------------------

NotifyThread::NotifyThread(Event *event, const char * name)
    : inherited(name), mOwnsEvent(false)
{
    eNotifyEvent = event;
}

//---------------------------------------------------------------------------
// NotifyThread::~NotifyThread() --
//---------------------------------------------------------------------------

NotifyThread::~NotifyThread()
{
	if( mOwnsEvent )
		{
		delete eNotifyEvent;
		eNotifyEvent = 0;
		}
}

//------------------------------------------------------------------------------
//  NotifyThread::Init() -- Init Multi-Objects here
//------------------------------------------------------------------------------

void  NotifyThread::Init()
{
    //  Load NotifyObj with our default objects
    inherited::Init();
    AddObject(*eNotifyEvent, sNotifyEvent);
}

//---------------------------------------------------------------------------
//  NotifyThread::SingleExecution() --
//---------------------------------------------------------------------------
 
void NotifyThread::SingleExecution()
{
    //
    //  Wait for any of the Pools to signal data, or one of the
    //     termination events (Suicide or BustOut)
    int result = WaitFor();

    if (result == sTerminate || result == sStopped)
        return;

	OpenWire::Event e;
    OnNotify.Execute(e);
}
 
//---------------------------------------------------------------------------
//  NotifyThread::Notify() --
//------------------------------------------------------------------------------
 
void NotifyThread::Notify()
{
    eNotifyEvent->Set();
}

//===========================================================================
//  CLASS NotifySemThread  --  
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class NotifySemThread -- default
//---------------------------------------------------------------------------
 
NotifySemThread::NotifySemThread(const char * name)
    : inherited(name), mOwnsSem(true)
{
    NotifySem = new Semaphore;//(10000);
}

//---------------------------------------------------------------------------
//  constructor for class NotifySemThread -- load semaphore
//---------------------------------------------------------------------------

NotifySemThread::NotifySemThread(Semaphore *sem, const char * name)
    : inherited(name), mOwnsSem(false)
{
    NotifySem = sem;
}
 
//---------------------------------------------------------------------------
//  destructor for class NotifySemThread
//---------------------------------------------------------------------------
 
NotifySemThread::~NotifySemThread()
{
	if( mOwnsSem )
		{
		delete NotifySem;
		NotifySem = 0;
		}
}
 
//------------------------------------------------------------------------------
//  NotifySemThread::Init() -- Init Multi-Objects here
//------------------------------------------------------------------------------

void  NotifySemThread::Init()
{
    //  Load NotifyObj with our default objects
    inherited::Init();
    AddObject(*NotifySem, sNotifySem);
}

//---------------------------------------------------------------------------
//  NotifySemThread::SingleExecution() --
//---------------------------------------------------------------------------
 
void NotifySemThread::SingleExecution()
{
    //
    //  Wait for any of the Pools to signal data, or one of the
    //     termination events (eTerminate or eStop)
    int result = WaitFor();

    if (result == sTerminate || result == sStopped)
        return;

	OpenWire::Event e;
    OnNotify.Execute(e);
}
 
//---------------------------------------------------------------------------
//  NotifySemThread::Notify() --
//------------------------------------------------------------------------------
 
void NotifySemThread::Notify()
{
    NotifySem->Release();
}



} // namespace Innovative


