// OsalSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef OsalSupport_MbH
#define OsalSupport_MbH

#include <KernelPci_Mb.h>
#include <vector>
#include <string>


namespace Innovative
{
	class Event;
}

namespace InnovativeKernel
{

unsigned int CurrentThreadId();

namespace OsalSupport
{

typedef  std::vector<InnovativeKernel::PciBoardLocation>  PciScanResult;

//==============================================================================
//  ABSTRACT CLASS IOsMutex -- Osal Basic Mutex
//==============================================================================

 class IOsMutex
{
public:
	// Dtor
	virtual ~IOsMutex()  {  }

	virtual bool  Lock() = 0 ;
	virtual bool  Unlock() = 0;
};

//==============================================================================
//  ABSTRACT CLASS IOsEvent -- Osal Basic Event
//==============================================================================

class IOsEvent
{
public:
	enum Timeout { NoWaiting=0 };

	// Dtor
	virtual ~IOsEvent()  {  }

	virtual bool  Wait() = 0;
	virtual bool  Wait(unsigned int Timeout) = 0;
	virtual bool  Set() = 0;
	virtual bool  Reset() = 0;
};

//==============================================================================
//  ABSTRACT CLASS IOsThread -- Osal Basic Thread
//==============================================================================

typedef void (*OSTHREAD_HANDLER_FUNC)(void *pData);

class IOsThread
{
public:
    enum ThreadPriority { tpIdle, tpLowest, tpLower, tpNormal, tpHigher, tpHighest, tpTimeCritical, tpCount };

	// Dtor
	virtual ~IOsThread()  {  }

	virtual bool  Wait() = 0;
	virtual void  Priority(ThreadPriority p) = 0;
};

//==============================================================================
//  ABSTRACT CLASS OsMisc -- general OS utilitiy functions
//==============================================================================

class IOsMisc
{
public:
	// Dtor
	virtual ~IOsMisc()  {  }

	virtual void Error(const std::string & text, unsigned int status) = 0;
    virtual void Trace(const char * msg) const = 0;
};

//==============================================================================
//  ABSTRACT CLASS IOsPciConfig -- access PCI configuration data
//==============================================================================

class IOsPciConfig
{
public:
	// Dtor
	virtual ~IOsPciConfig()  {  }

	virtual size_t        BoardCount(const InnovativeKernel::PciId & ID) = 0;
	virtual bool          LicenseCheck(const InnovativeKernel::PciId & ID, unsigned int & code) = 0;
	virtual bool          FindTarget(InnovativeKernel::PciBoardLocation & Board, unsigned int target) = 0;
	virtual unsigned int  ScanPci(PciScanResult & Result, const InnovativeKernel::PciId & ID) = 0;
	virtual void          ReadPciConfig(InnovativeKernel::DeviceEntry & Result,  const InnovativeKernel::PciPosition & Slot) = 0;
};

//==============================================================================
//  ABSTRACT CLASS IOsPciDriver -- "Per Device" Management
//==============================================================================

class IOsPciDriver
{
public:
	// Dtor
	virtual ~IOsPciDriver()  {  }

    virtual void  DeviceOpen(unsigned int target) = 0;
    virtual void  DeviceClose() = 0;

	virtual size_t BoardCount() = 0;
	virtual InnovativeKernel::PciId BoardId() = 0;
	virtual InnovativeKernel::PciPosition BoardLocation() = 0;

    virtual void  AllocateMemoryBlock(InnovativeKernel::MemoryBlock & Block, unsigned int BufferSizeBytes, bool is64BitCapable = false) = 0;
    virtual void  FreeMemoryBlock(InnovativeKernel::MemoryBlock & Block, bool is64BitCapable = false) = 0;

    virtual bool  IsIntEnabled() = 0;
    virtual bool  InterruptInstall(Innovative::Event * isrEvent) = 0;
    virtual bool  InterruptDeinstall() = 0;
    virtual bool  InterruptEnable() = 0;
    virtual bool  InterruptDisable() = 0;
    virtual void  Error(const std::string & text) = 0;
    virtual  size_t IsrQueueDepth() = 0;
    virtual unsigned int  DequeueIsrStatus() = 0;

private:
    //
    //  Member Data
    InnovativeKernel::PciBoardLocation    Board;


};

}  // end namespace OsalSupport

}  // end namespace Innovative
//---------------------------------------------------------------------------
#endif
