// Osal_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Osal_Mb.h>
#include <sstream>
#include <string.h>
#include <StringSupport_Mb.h>
#include <Exception_Mb.h>
#include <ThreadSafeQueue_Mb.h>
#include <Event_Mb.h>
#include <SystemSupport_Mb.h>

#include <Poco/Thread.h>
#include <Poco/Format.h>
#include <Poco/AutoPtr.h>
#include <Poco/Net/RemoteSyslogChannel.h>
#include <Poco/Net/NetException.h>
#include <Poco/NullChannel.h>

#if LINUX
#include <DriverController_Linux.cpp>
#else
#include <DriverController_Win.cpp>
#endif

using namespace Innovative;

namespace InnovativeKernel
{
#if defined(LINUX)
typedef struct {
    pthread_cond_t cond;
    pthread_mutex_t mutex;
    bool signaled;
} wd_linux_event_t;
#endif

unsigned int CurrentThreadId()
{
#if defined(WIN32) || defined(WIN64)
    return GetCurrentThreadId();
#elif defined(LINUX)
    return pthread_self();
#endif
}

#define BAD_HANDLE -1

namespace Osal
{


//==============================================================================
//  CLASS OsSupport -- Stand-alone Osal support functions
//==============================================================================

using Poco::AutoPtr;
using Poco::Logger;
using Poco::Net::RemoteSyslogChannel;
using Poco::NullChannel;

//------------------------------------------------------------------------------
//  OsSupport::ChannelMaker::ChannelMaker() -
//------------------------------------------------------------------------------

OsSupport::ChannelMaker::ChannelMaker()
{
    try {
        Poco::Net::initializeNetwork();
        AutoPtr<RemoteSyslogChannel> pUdpChannel(new RemoteSyslogChannel("127.0.0.1", "remoteTarget"));
        pUdpChannel->setProperty("host", "127.0.0.1");
        Logger::root().setChannel(pUdpChannel);
    } catch (const Poco::Net::NetException &/*ex*/) {
        //ex.displayText();
        Logger::root().setChannel(new NullChannel());
    }
}

//------------------------------------------------------------------------------
//  OsSupport::OsSupport() -
//------------------------------------------------------------------------------

OsSupport::OsSupport() : logger(Logger::get(""))
{
}



//------------------------------------------------------------------------------
//  OsSupport::~OsSupport() -
//------------------------------------------------------------------------------

OsSupport::~OsSupport()
{
}

//------------------------------------------------------------------------------
//  OsSupport::Error() - throw Win Driver error
//------------------------------------------------------------------------------

void  OsSupport::Error(const std::string & text)
{
    ErrorMsg(text.c_str());
    throw Innovative::MalibuException(text.c_str());
}

//------------------------------------------------------------------------------
//  OsSupport::Error() - throw Win Driver error
//------------------------------------------------------------------------------

void  OsSupport::Error(const std::string & text, unsigned int status)
{
    std::stringstream msg;
    msg << text << " Error " << Innovative::IntToString(status, 16);
    Error(msg.str());
}

//------------------------------------------------------------------------------
//  OsSupport::Sleep() -
//------------------------------------------------------------------------------

void  OsSupport::Sleep(unsigned long mS)
{
    Poco::Thread::sleep(mS);
}

//------------------------------------------------------------------------------
//  OsSupport::uSleep() - Sleeps for the amount of uS passed in.
//------------------------------------------------------------------------------

void  OsSupport::uSleep(unsigned long uS)
{
    static StopWatch wch;
    wch.uS(uS);
}

//------------------------------------------------------------------------------
//  OsSupport::Trace() -
//------------------------------------------------------------------------------

void OsSupport::Trace(const char * msg) const
{
    InfoMsg(msg);
}

//------------------------------------------------------------------------------
//  OsSupport::ErrorMsg() -
//------------------------------------------------------------------------------

void OsSupport::ErrorMsg(const char *msg) const
{
    logger.error(msg);
}

//------------------------------------------------------------------------------
//  OsSupport::Warn() -
//------------------------------------------------------------------------------

void OsSupport::Warn(const char *msg) const
{
    logger.warning(msg);
}

//------------------------------------------------------------------------------
//  OsSupport::InfoMsg() -
//------------------------------------------------------------------------------

void OsSupport::InfoMsg(const char *msg) const
{
    logger.information(msg);
}


//==============================================================================
//  CLASS VersionInfo --
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class VersionInfo
//------------------------------------------------------------------------------

VersionImpl::VersionImpl()
{

}

//------------------------------------------------------------------------------
//  VersionImpl::Driver()
//------------------------------------------------------------------------------

std::string VersionImpl::DriverRaw() const
{
    return mdfullversion;
}

//------------------------------------------------------------------------------
//  VersionImpl::Driver()
//------------------------------------------------------------------------------

std::string VersionImpl::Driver() const
{
    return mdversion;
}

//------------------------------------------------------------------------------
//  VersionImpl::Licence() -- Returns the IILicense string
//------------------------------------------------------------------------------

std::string VersionImpl::Licence() const
{
    return mlicence;
}

//---------------------------------------------------------------------------------------------
//  VersionImpl::IILicenseVersion() -- Returns version number of the current WinDriver license
//---------------------------------------------------------------------------------------------

std::string VersionImpl::IILicenseVersion() const
{
    return mdriverversion;
}

//------------------------------------------------------------------------------------------------
//  VersionImpl::CompiledVersion() -- Returns the version number that WinDriver was compiled with
//------------------------------------------------------------------------------------------------

std::string VersionImpl::CompiledVersion() const
{
    return CompVersion;
}

//==============================================================================
//  CLASS OsDeviceImpl -- "Per Device" Hidden Data
//==============================================================================

//const unsigned int MaxTransferCommands = 5;

//------------------------------------------------------------------------------
//  declaration for class OsDeviceImpl
//------------------------------------------------------------------------------

class OsDeviceImpl
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  Class KernelCacheReleaser
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class KernelCacheReleaser
    {
    public:
        KernelCacheReleaser(Innovative::IAddressingSpace & space, int offset)
            :  KernelCacheReleaseReg(space, offset) {}

        Innovative::Register   KernelCacheReleaseReg;

        void   Release()
            {   KernelCacheReleaseReg.Value(0); }
    };
    //
    //  Member Data
    OsDevice *              WDD;
    int                     Handle;
    KernelCacheReleaser *   KCacheReleaser;
    int                     mdHandle;

    //
    //  Device Context - passed into ISR
    struct DeviceContext
    {
    OsDevice * WDD;
    };

    DeviceContext               Context;

    struct AllocatedMemoryInfo { unsigned int *Dma; };

    std::vector<AllocatedMemoryInfo>           Memory;
    std::vector<AllocatedMemoryInfo>           SG_Memory;
    Innovative::ThreadSafeQueue<unsigned int>  ISRQueue;
    Innovative::Event *                        IsrEventPtr;
    std::vector<unsigned int>                  IsrCountArray;
    unsigned int                               IsrCountIdx;

    //
    //  Member Functions
    OsDeviceImpl(OsDevice * device);
    ~OsDeviceImpl();

    void          EnqueueIsrStatus(unsigned int status);
    unsigned int  DequeueIsrStatus();
    void          SignalKernelCacheRelease();
    void          EnableDedicatedKernelCacheRelease(Innovative::IAddressingSpace & space, int offset);



};
//------------------------------------------------------------------------------
//  constructor for class OsDeviceImpl
//------------------------------------------------------------------------------

OsDeviceImpl::OsDeviceImpl(OsDevice * device)
    :  WDD(device), Handle(0), KCacheReleaser(0),
       mdHandle(BAD_HANDLE), IsrEventPtr(0), IsrCountArray(10000), IsrCountIdx(0)
{
    //
    //  Init Device Context
    Context.WDD = WDD;
}

//------------------------------------------------------------------------------
//  destructor for class OsDeviceImpl
//------------------------------------------------------------------------------

OsDeviceImpl::~OsDeviceImpl()
{
    if (KCacheReleaser != 0)
        {
        delete KCacheReleaser;
        KCacheReleaser = 0;
        }
}

//------------------------------------------------------------------------------
//  OsDeviceImpl::EnqueueIsrStatus() -- Add ISR Status word to queue
//------------------------------------------------------------------------------

void  OsDeviceImpl::EnqueueIsrStatus(unsigned int status)
{
        ISRQueue.push(status);
        WDD->DUI->IsrLog(status);
}

//------------------------------------------------------------------------------
//  OsDeviceImpl::DequeueIsrStatus() - Return queue depth
//------------------------------------------------------------------------------

unsigned int  OsDeviceImpl::DequeueIsrStatus()
{
    unsigned int status = ISRQueue.front();
    ISRQueue.pop();
    return status;
}

//------------------------------------------------------------------------------
//  OsDeviceImpl::SignalKernelCacheRelease() -- Extract Interrupt Data
//------------------------------------------------------------------------------

void  OsDeviceImpl::SignalKernelCacheRelease()
{
    if (KCacheReleaser)
        KCacheReleaser->Release();
}

//------------------------------------------------------------------------------
//  OsDeviceImpl::EnableDedicatedKernelCacheRelease() --
//------------------------------------------------------------------------------

void  OsDeviceImpl::EnableDedicatedKernelCacheRelease(Innovative::IAddressingSpace & space, int offset)
{
    KCacheReleaser = new KernelCacheReleaser(space, offset);
}

//==============================================================================
//  CLASS OsDevice -- "Per Device" Management
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class OsDevice
//------------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


OsDevice::OsDevice(const InnovativeKernel::PciId & id, IOsDeviceDebugUI * dui)
    :  Impl(new OsDeviceImpl(this)), DUI(dui), Ctrl(id.DeviceId, *this)
{
    //
    //  Save the ID
    Board.ID = id;
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//------------------------------------------------------------------------------
//  destructor for class OsDevice
//------------------------------------------------------------------------------

OsDevice::~OsDevice()
{
    delete Impl;
    delete DUI;
}

//------------------------------------------------------------------------------
//  OsDevice::DeviceOpen() - Open Device and Map Resources
//------------------------------------------------------------------------------

void  OsDevice::DeviceOpen(unsigned int target)
{
    Ctrl.open(target);
}

//------------------------------------------------------------------------------
//  OsDevice::DeviceClose() - Close Device, Unmap Resources
//------------------------------------------------------------------------------

void  OsDevice::DeviceClose()
{
    Ctrl.close();
}

//------------------------------------------------------------------------------
//  OsDevice::BoardCount() - Get total number of installed boards
//------------------------------------------------------------------------------

size_t  OsDevice::BoardCount()
{
    return Ctrl.deviceCount();
}

//------------------------------------------------------------------------------
//  OsDevice::LicenseCheck() - Quick check for valid WD License
//------------------------------------------------------------------------------

bool  OsDevice::LicenseCheck(unsigned int &)
{
    return true;
}

//------------------------------------------------------------------------------
//  OsDevice::BoardId() - Return the ID of the device
//------------------------------------------------------------------------------

InnovativeKernel::PciId  OsDevice::BoardId()
{
    return Board.ID;
}

//------------------------------------------------------------------------------
//  OsDevice::BoardLocation() - Return the Slot Location of the device
//------------------------------------------------------------------------------

InnovativeKernel::PciPosition  OsDevice::BoardLocation()
{
    return Board.Location;
}

//------------------------------------------------------------------------------
//  OsDevice::InitializeMappedMemory() -- Init mappable memory block
//------------------------------------------------------------------------------
//
void  OsDevice::InitializeMappedMemory(InnovativeKernel::MemoryBlock & Block, int Region)
{
    InitializeMappedMemory(Block, Region, Region);  // If region and bar are same
}

//------------------------------------------------------------------------------
//  OsDevice::InitializeMappedMemory() -- Init mappable memory block
//------------------------------------------------------------------------------
//
void  OsDevice::InitializeMappedMemory(InnovativeKernel::MemoryBlock & Block, int Bar, int )
{
    Block = Ctrl.mapBarRegion(Bar);
}

//------------------------------------------------------------------------------
//  OsDevice::ReleaseMappedMemory() --
//------------------------------------------------------------------------------

void OsDevice::ReleaseMappedMemory(const InnovativeKernel::MemoryBlock &Block)
{
    Ctrl.unmapBarRegion(Block);
}

//------------------------------------------------------------------------------
//  OsDevice::AllocateMemoryBlock() -- Allocate a memory block
//------------------------------------------------------------------------------

void  OsDevice::AllocateMemoryBlock(InnovativeKernel::MemoryBlock & Block, unsigned int BufferSizeBytes, bool )
{
    Block = Ctrl.mapDmaMemory(BufferSizeBytes);
}

//------------------------------------------------------------------------------
//  OsDevice::FreeMemoryBlock() -- Release a memory block
//------------------------------------------------------------------------------

void  OsDevice::FreeMemoryBlock(InnovativeKernel::MemoryBlock & Block,  bool )
{
    Ctrl.unmapDmaMemory(Block);
}


//------------------------------------------------------------------------------
//  OsDevice::AllocateScatterBlock() -- Alloc a memory block for scatter/gather
//------------------------------------------------------------------------------

void  OsDevice::AllocateScatterBlock(InnovativeKernel::ScatterBlock & , unsigned int )
{
}

//------------------------------------------------------------------------------
//  OsDevice::FreeScatterBlock() -- Release a scatter/gather memory block
//------------------------------------------------------------------------------

void  OsDevice::FreeScatterBlock(InnovativeKernel::ScatterBlock & )
{
}

//------------------------------------------------------------------------------
//  OsDevice::IsIntEnabled() -- Check if interrupt enabled
//------------------------------------------------------------------------------

bool  OsDevice::IsIntEnabled()
{
    return Ctrl.isInterruptEnabled();
}


//------------------------------------------------------------------------------
//  OsDevice::InterruptInstall() -- Install Interrupt
//------------------------------------------------------------------------------

bool  OsDevice::InterruptInstall(Innovative::Event * isrEvent)
{
    Ctrl.installInterruptEventHandler(isrEvent);

    return true;
}

//------------------------------------------------------------------------------
//  OsDevice::InterruptDeinstall() -- Un-Install Interrupt
//------------------------------------------------------------------------------

bool  OsDevice::InterruptDeinstall()
{
    Ctrl.installInterruptEventHandler(0);

    return true;
}

//------------------------------------------------------------------------------
//  OsDevice::InterruptEnable() -- Enable Interrupt Handling
//------------------------------------------------------------------------------

bool  OsDevice::InterruptEnable()
{
    if (IsIntEnabled())
        return true;

    Ctrl.enableInterrupt();
    return true;
}

//------------------------------------------------------------------------------
//  OsDevice::InterruptDisable() -- Disable Interrupt Handling
//------------------------------------------------------------------------------

bool  OsDevice::InterruptDisable()
{
    Ctrl.disableInterrupt();

    return true;
}

//------------------------------------------------------------------------------
//  OsDevice::GetDeviceNames() --
//------------------------------------------------------------------------------

vector<string> OsDevice::GetDeviceNames()
{
    return Ctrl.getDeviceNames();
}
//------------------------------------------------------------------------------
//  OsDevice::DriverVersion() --
//------------------------------------------------------------------------------

string OsDevice::DriverVersion()
{
    return Ctrl.DriverVersionStr();
}
//------------------------------------------------------------------------------
//  OsDevice::DriverDate() --
//------------------------------------------------------------------------------

string OsDevice::DriverDate()
{
    return Ctrl.DriverDateStr();
}
//------------------------------------------------------------------------------
//  OsDevice::GetDeviceNames() --
//------------------------------------------------------------------------------

string OsDevice::PrintAllDevices()
{
    //vector<string> temp = Ctrl.getDeviceNames();

    return string();
}


//------------------------------------------------------------------------------
//  OsDevice::EnqueueIsrStatus() --
//------------------------------------------------------------------------------

void OsDevice::EnqueueIsrStatus(unsigned int stat)
{
    Impl->EnqueueIsrStatus(stat);

    Impl->SignalKernelCacheRelease();
}

//------------------------------------------------------------------------------
//  OsDevice::IsrQueueDepth() - Return queue depth
//------------------------------------------------------------------------------

size_t  OsDevice::IsrQueueDepth()
{
    return Impl->ISRQueue.depth();
}

//------------------------------------------------------------------------------
//  OsDevice::DequeueIsrStatus() - Return status of ISR
//------------------------------------------------------------------------------

unsigned int  OsDevice::DequeueIsrStatus()
{
    return Impl->DequeueIsrStatus();
}

//------------------------------------------------------------------------------
//  OsDevice::ReadPciConfig() -- Get PCI Config for a device
//------------------------------------------------------------------------------
//
void  OsDevice::ReadPciConfig(InnovativeKernel::DeviceEntry & /*Result*/)
{

}

//------------------------------------------------------------------------------
//  OsDevice::KpQueueDepth() -- Retrieve ISR queue depth from KerPlug
//------------------------------------------------------------------------------
//
size_t  OsDevice::KpQueueDepth()
{
    return Ctrl.getInterruptQueueDepth();
}

//------------------------------------------------------------------------------
//  OsDevice::ClearInterruptQueue() -- Clear Interrupt Queue in KerPlug
//------------------------------------------------------------------------------
//
void  OsDevice::ClearInterruptQueue()
{
    Ctrl.clearInterruptQueue();

    //  Clear this intermediate queue now too
    Impl->ISRQueue.clear();
}

//------------------------------------------------------------------------------
//  OsDevice::InitDeviceExtension() - initialize device extension structure
//------------------------------------------------------------------------------
//
void  OsDevice::InitDeviceExtension(unsigned int InterruptType,
                              unsigned int IntStatusByteOffset,
                              unsigned int IntStatusMask )
{
    Ctrl.setInterruptInfo(InterruptType, IntStatusByteOffset, IntStatusMask);
}

//------------------------------------------------------------------------------
//  OsDevice::EnableDedicatedKernelCacheRelease() --
//------------------------------------------------------------------------------

void  OsDevice::EnableDedicatedKernelCacheRelease(Innovative::IAddressingSpace & space, int offset)
{
    Impl->EnableDedicatedKernelCacheRelease(space, offset);
}

}  // end namespace Osal

}  // end namespace Innovative

