// Osal_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//
#ifndef Osal_MbH
#define Osal_MbH

#ifdef __BORLANDC__
#include <mem.h>
#endif
#include <SingletonManager_Mb.h>
#include <KernelPci_Mb.h>
#include <vector>
#include <string>
#include <OsalSupport_Mb.h>
#include <HardwareRegister_Mb.h>

#include <Poco/Logger.h>

#if LINUX
#include <DriverController_Linux.h>
#else
#include <DriverController_Win.h>
#endif

using namespace std;

namespace Innovative
{
	class Event;
}

namespace InnovativeKernel
{

unsigned int CurrentThreadId();

namespace Osal
{
typedef std::vector<InnovativeKernel::PciBoardLocation>  PciScanResult;
typedef std::map<InnovativeKernel::PciId, PciScanResult> PciScanResultMap;
typedef vector<int> CpuVec;

//==============================================================================
//  CLASS OsSupport -- Stand-alone OsalSupport support functions
//==============================================================================

class OsSupport  : OsalSupport::IOsMisc
{

	struct ChannelMaker
	{
		ChannelMaker();
	};

public:
	typedef enum{WD_STATUS, OS_STATUS} STATUS_TYPE;

	OsSupport();
	virtual ~OsSupport();

	void            Error(const std::string & text, unsigned int status);
	void            Error(const std::string & text);

	void            Trace(const char * msg) const;

	void            InfoMsg(const char * msg) const;
	void            ErrorMsg(const char * msg) const;
	void            Warn(const char * msg) const;

	static void     uSleep(unsigned long uS);
	static void     Sleep(unsigned long mS);

	ChannelMaker	channelMake;
	Poco::Logger   &logger;
};

//==============================================================================
//  CLASS VersionImpl -- Manages version info
//==============================================================================

class VersionImpl
{
public:

	VersionImpl();
	virtual ~VersionImpl()
	{}

	std::string Driver() const;
	std::string DriverRaw() const;
	std::string Licence() const;
	std::string IILicenseVersion() const;
	std::string CompiledVersion() const;

private:
	std::string mdversion;
	std::string mdfullversion;
	std::string mlicence;
	std::string mdriverversion;
	std::string CompVersion;
};

//==============================================================================
//  CLASS OsMutex -- OsalSupport Basic Mutex
//==============================================================================

class OsMutex : public OsalSupport::IOsMutex
{
public:
		OsMutex();
		virtual ~OsMutex();

		bool  Lock();
		bool  Unlock();

private:
		class OsMutexImpl *  Impl;
};


//==============================================================================
//  CLASS OsEvent -- OsalSupport Basic Event
//==============================================================================

class OsEvent  : OsalSupport::IOsEvent
{
public:
		enum Timeout { NoWaiting=0 };

		OsEvent();
		virtual ~OsEvent();

		bool  Wait();
		bool  Wait(unsigned int Timeout);
		bool  Set();
		bool  Reset();

private:
		class OsEventImpl *  Impl;
};

//==============================================================================
//  CLASS OsKernelLib --  Kernel and Scheduler Controls
//==============================================================================
class OsKernelLib
{
public:
	bool LockAllMem(const size_t stackSize);
	bool SetProcessAffinity(const CpuVec &v);
	bool SetSchedPolicy(const int policy, const int priority);

private:
	OsSupport Support;
};

//==============================================================================
//  CLASS IOsDeviceDebugUI -- OsDevice Debug Call-up Interface
//==============================================================================

class IOsDeviceDebugUI
{
public:
	virtual ~IOsDeviceDebugUI() {}

	virtual void  IsrLog(unsigned int isr_log_data) = 0;
	virtual void  TraceLog(int code, int data) = 0;

};

//==============================================================================
//  CLASS OsDevice -- "Per Device" Management
//==============================================================================

class OsDevice: OsalSupport::IOsPciDriver
{
	friend class OsDeviceImpl;

public:
	OsDevice(const InnovativeKernel::PciId & id, IOsDeviceDebugUI * dui);
	virtual ~OsDevice();

	void  DeviceOpen(unsigned int target);
	void  DeviceClose();

	size_t                          BoardCount();
	bool                            LicenseCheck(unsigned int & code);
	InnovativeKernel::PciId         BoardId();
	InnovativeKernel::PciPosition   BoardLocation();

	void  ReadPciConfig(InnovativeKernel::DeviceEntry & Result);
	bool  IsAddressRegionActive(int Region);
	void  InitializeMappedMemory(InnovativeKernel::MemoryBlock & Block, int Region);
	void  InitializeMappedMemory(InnovativeKernel::MemoryBlock & Block, int Bar, int Region);
	void  ReleaseMappedMemory(const InnovativeKernel::MemoryBlock &Block);

    void  AllocateMemoryBlock(InnovativeKernel::MemoryBlock & Block, unsigned int BufferSizeBytes, bool is64BitCapable = false);
    void  FreeMemoryBlock(InnovativeKernel::MemoryBlock & Block,  bool is64BitCapable = false);

	void  AllocateScatterBlock(InnovativeKernel::ScatterBlock & Block, unsigned int BufferSizeBytes);
	void  FreeScatterBlock(InnovativeKernel::ScatterBlock & Block);

	void  ClearInterruptQueue();
	void  InitDeviceExtension(unsigned int InterruptType,
							  unsigned int IntStatusByteOffset,
							  unsigned int IntStatusMask );
	void  EnableDedicatedKernelCacheRelease(Innovative::IAddressingSpace & space, int offset);

    vector<string>  GetDeviceNames();
    string          PrintAllDevices();

	bool  IsIntEnabled();
	bool  InterruptInstall(Innovative::Event * isrEvent);
	bool  InterruptDeinstall();
	bool  InterruptEnable();
	bool  InterruptDisable();

	void  Error(const std::string & text )
		{  Support.Error(text);  }

	void  EnqueueIsrStatus(unsigned int stat);


	size_t        KpQueueDepth();
	size_t        IsrQueueDepth();
	unsigned int  DequeueIsrStatus();

    string DriverVersion();
    string DriverDate();
private:
	//
	//  Member Data
	InnovativeKernel::PciBoardLocation    Board;
	class OsDeviceImpl *                  Impl;
	OsSupport                             Support;
	IOsDeviceDebugUI *                    DUI;
	Innovative::DriverController          Ctrl;
};


}  // end namespace OsalSupport

}  // end namespace Innovative
//---------------------------------------------------------------------------

#endif
