// P25DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <P25_Mb.h>
#include <P25DeviceMap_Mb.h>
#include <P25MemoryMap_Mb.h>
#include <P25DeviceDriver_Mb.h>
#include <Exception_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

namespace Innovative
{

//===========================================================================
//  CLASS IUsesP25FpgaLogicVersionInfo -- User Interface to user logic ID
//===========================================================================

//---------------------------------------------------------------------------
//  IUsesP25FpgaLogicVersionInfo::FpgaLogicVersion() --
//---------------------------------------------------------------------------

short IUsesP25FpgaLogicVersionInfo::FpgaLogicVersion() const
{
    return FpgaLogicRawValue() & 0xffff;
}

//---------------------------------------------------------------------------
//  IUsesP25FpgaLogicVersionInfo::FpgaLogicVariant() --
//---------------------------------------------------------------------------

short IUsesP25FpgaLogicVersionInfo::FpgaLogicVariant() const
{
    return (FpgaLogicRawValue() >> 16) & 0x0f;
}

//---------------------------------------------------------------------------
//  IUsesP25FpgaLogicVersionInfo::FpgaLogicRevision() --
//---------------------------------------------------------------------------

short IUsesP25FpgaLogicVersionInfo::FpgaLogicRevision() const
{
    return (FpgaLogicRawValue() >> 20) & 0x0f;
}

//---------------------------------------------------------------------------
//  IUsesP25FpgaLogicVersionInfo::FpgaLogicType() --
//---------------------------------------------------------------------------

short IUsesP25FpgaLogicVersionInfo::FpgaLogicType() const
{
    return (FpgaLogicRawValue() >> 24) & 0xff;
}

//---------------------------------------------------------------------------
//  IUsesP25FpgaLogicVersionInfo::FpgaLogicRawValue() --
//---------------------------------------------------------------------------

unsigned int IUsesP25FpgaLogicVersionInfo::FpgaLogicRawValue() const
{
	if (!Owner.DDeviceDriver->Opened())
		return 0;

    // If processor not running, Peek() will hang
    // Also, must do a bunch of reads in order to flush what appears to
    // be some sort of read pipeline, before proper target memory
    // contents are returned properly.
    volatile int result;
	const unsigned int DeviceStatusReg = 0x80210014;
    for (int i = 0; i < 200; ++i)
    	result = Owner.Hpi().Peek(DeviceStatusReg);

	return result;
}

//===========================================================================
//  CLASS P25HdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class P25HdwPluginImpl : public IPacketDevicePlugin
{
public:
    P25HdwPluginImpl( P25DeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    P25DeviceMap * PDM;
};

//===========================================================================
//  CLASS P25DeviceMap  -- Registers for P25 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class P25DeviceMap
//---------------------------------------------------------------------------

using namespace P25_MemoryMap;

PacketDeviceRegisters P25DRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	0,                    // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters P25RegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

//  Locations of the HPI registers
static HpiEngineAddresses  P25HpiAddresses =
    {
    mmHpiAddressLo,  mmHpiAddressHi,  //  HPI Address Register    LO, HI
    mmHpiDataLo,     mmHpiDataHi,     //  HPI Data Register       LO, HI
    mmHpiDataIncrLo, mmHpiDataIncrHi, //  HPI Data Incr Register  LO, HI
    mmHpiControlLo,  mmHpiControlHi   //  HPI Control Register    LO, HI
    };

//  Values of the HPI registers
static HpiCeSpaceValues  P25HpiCeValues =
    {
    0x00003078,         //  EMIF global control register
    0x0000C041,         //  CE1
    0x21A28A22,         //  CE0
    0x00000030,         //  CE2
    0x11010420,         //  CE3
    0x00000350,         //  SDRAM Refresh Timing register
    0x000544a7,         //  SDRAM Extension
    0x6B338000          //  SDRAM control register 
    };

//---------------------------------------------------------------------------
//  constructor for class P25DeviceMap
//---------------------------------------------------------------------------

P25DeviceMap::P25DeviceMap(P25 * owner)
    :  PacketDeviceImpl(new P25DeviceDriver, P25DRegAddrs),
       PacketDeviceMap(this, P25RegAddrs, 0),
       IUsesVirtexFpgaLoader(PortMemory, mmFpgaData, mmFpgaControl),
       IUsesStandardHpiCpu(*this, P25HpiAddresses, P25HpiCeValues),
       IUsesVirtexJtagLoader(PortMemory, mmJtag),
       IUsesPciLogicVersionInfo(PortMemory, mmPciVersion),
       IUsesP25FpgaLogicVersionInfo(*this),       
       Owner(owner),
       DDeviceDriver(dynamic_cast<P25DeviceDriver*>(DeviceDriver)),
       RcvPacketSize(0x80), XmitPacketSize(0x80),
       PciTestControlReg( PortMemory,  mmPciTestControl)
{
    //
    //  Hook stream events from base class
    OnBeforeCoffLoad.SetEvent(this, &P25DeviceMap::OnBeforeCoffLoadHandler);
    OnStreamConfigure.SetEvent(this, &P25DeviceMap::OnStreamConfigureHandler);
    OnBeforeStreamStart.SetEvent(this, &P25DeviceMap::OnBeforeStreamStartHandler);
    OnAfterStreamStart.SetEvent(this, &P25DeviceMap::OnAfterStreamStartHandler);
    OnBeforeStreamStop.SetEvent(this, &P25DeviceMap::OnBeforeStreamStopHandler);
    OnAfterStreamStop.SetEvent(this, &P25DeviceMap::OnAfterStreamStopHandler);
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new P25HdwPluginImpl(this) );
}

//---------------------------------------------------------------------------
//  destructor for class P25DeviceMap
//---------------------------------------------------------------------------

P25DeviceMap::~P25DeviceMap()
{
}

//---------------------------------------------------------------------------
//  P25DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  P25DeviceMap::OpenHardware()
{
    IUsesStandardHpiCpu::Open(DeviceDriver->PortMemoryAddress());
}

//---------------------------------------------------------------------------
//  P25DeviceMap::ResetBaseBoard() --  Reset baseboard during HPI cold
//---------------------------------------------------------------------------

void P25DeviceMap::ResetBaseBoard()
{
	DspInReset(true);
	Innovative::Sleep(10);
	DspInReset(false);
}

//---------------------------------------------------------------------------
//  P25DeviceMap::OnBeforeCoffLoadHandler()
//---------------------------------------------------------------------------

void  P25DeviceMap::OnBeforeCoffLoadHandler(ProcessCompletionEvent & /*Event*/)
{
    if (!IsConfigured())
        throw MalibuException("Baseboard logic is not configured!");
}

//---------------------------------------------------------------------------
//  P25DeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  P25DeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnStreamConfigure.Execute(Event);

}

//---------------------------------------------------------------------------
//  P25DeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  P25DeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //  Set loopback mode
    LocalLoopbackMode(Owner->FLocalLoopbackMode);

}

//---------------------------------------------------------------------------
//  P25DeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  P25DeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  P25DeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  P25DeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  P25DeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  P25DeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  P25DeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  P25DeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<P25DeviceDriver::LOG->Count; i++)
        vec.push_back(P25DeviceDriver::LOG->Status[i]);

}

//---------------------------------------------------------------------------
//  P25DeviceMap::LocalLoopbackMode()
//---------------------------------------------------------------------------

void  P25DeviceMap::LocalLoopbackMode(bool state)
{
    if (state)
        {
        PciTestControlReg.LoopbackEnable.Set();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Set();
        }
    else
        {
        PciTestControlReg.LoopbackEnable.Clear();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Clear();
        }
}


//===========================================================================
//  CLASS P25DeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PmcDeviceMap::P25DeviceMap
//---------------------------------------------------------------------------

P25DeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TestModeEnable( *this, 0 ),
      GenerateRxData( *this, 1 ),
      LoopbackEnable( *this, 2 ),
      ClearTxCredit( *this, 3 ),
      ClearRxCredit( *this, 4 ),
      DrainXmitData( *this, 5 ),
      GenPacketSize( *this, 8, 4 ),
      GenPacketSizeUpdate( *this, 12 )
{
}



} // namespace Innovative


