// P25DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef P25DeviceMap_MbH
#define P25DeviceMap_MbH


#include <PacketDeviceMap_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <BaseboardCpuInterfaces_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PciLogicVersionInfo_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class P25DeviceMap;
class P25DeviceDriver;
class Event;
class PmcBuffer;

//===========================================================================
//  CLASS IUsesP25FpgaLogicVersionInfo -- User Interface to user logic ID
//===========================================================================

class IUsesP25FpgaLogicVersionInfo : public FpgaLogicVersionInfoIntf
{
public:
    IUsesP25FpgaLogicVersionInfo(P25DeviceMap & owner)
        : Owner(owner)
        {  }

    //
    //  Interface Functions
    virtual short           FpgaLogicVersion() const;
    virtual short           FpgaLogicVariant() const;
    virtual short           FpgaLogicRevision() const;
    virtual short           FpgaLogicType() const;
    virtual unsigned int    FpgaLogicRawValue() const;

private:
    P25DeviceMap &          Owner;
};

//===========================================================================
//  CLASS P25DeviceMap  -- Register & low level support for P25
//===========================================================================
class P25;
class IUsesP25FpgaLogicVersionInfo;

class P25DeviceMap :  public PacketDeviceImpl,
                      public PacketDeviceMap,
                      public IUsesVirtexFpgaLoader,
                      public IUsesStandardHpiCpu,
                      public IUsesVirtexJtagLoader,
                      public IUsesPciLogicVersionInfo,
                      public IUsesP25FpgaLogicVersionInfo,
					  public IUsesHpiControl
{
    friend class P25;
    friend class IUsesP25FpgaLogicVersionInfo;

public:
    //
    //  Member Functions
    P25DeviceMap(P25 * owner);
    virtual ~P25DeviceMap();

    void  OpenHardware();

    void  LocalLoopbackMode(bool state);

protected:

    P25 *               Owner;
    P25DeviceDriver *   DDeviceDriver;

    void  OnBeforeCoffLoadHandler(ProcessCompletionEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
	virtual void ResetBaseBoard();

private:
    //
    //  Member Data
    // FPA Eeprom programmer

public:
    int  RcvPacketSize;
    int  XmitPacketSize;

    void  DumpIsrStatus(std::vector<int> & vec);

protected:
    //
    //  Class PciTestRegister
    class PciTestRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       TestModeEnable;
        RegisterBit       GenerateRxData;
        RegisterBit       LoopbackEnable;
        RegisterBit       ClearTxCredit;
        RegisterBit       ClearRxCredit;
        RegisterBit       DrainXmitData;
        RegisterBitGroup  GenPacketSize;
        RegisterBit       GenPacketSizeUpdate;

    public:
        PciTestRegister( AddressingSpace &space, int offset );
    };
    //
    //  ...Registers
    PciTestRegister                    PciTestControlReg;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}; // Innovative
//---------------------------------------------------------------------------
#endif
