//---------------------------------------------------------------------------
// P25_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <P25_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <HpiEngine_Mb.h>
#include <Exception_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS P25  -- Racal Pmc Board object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class P25
//---------------------------------------------------------------------------

P25::P25()
    :  LegacyBoard(Device), Device(this), FLocalLoopbackMode(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class P25
//---------------------------------------------------------------------------

P25::~P25()
{
}

//---------------------------------------------------------------------------
//  P25::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t P25::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  P25::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  P25::Open()
{
    static_cast<IPacketDevice &>(Device).Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  P25::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  P25::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  P25::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  P25::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  P25::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  P25::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  P25::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  P25::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  P25::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  P25::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  P25::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  P25::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  P25::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  P25::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  P25::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  P25::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  P25::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  P25::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  P25::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  P25::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  P25::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  P25::Log( int type, int value )
{
    Device.Log(type, value);
}

//---------------------------------------------------------------------------
//  P25::Logic() -- Return reference to FPGA Loader Interface
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & P25::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  P25::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & P25::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  P25::Cpu() -- Return reference to Cpu Interface object
//---------------------------------------------------------------------------

IUsesOnboardHpiCpu & P25::Cpu()
{
    return Device;
}

//---------------------------------------------------------------------------
//  P25::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & P25::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  P25::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & P25::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  P25::BusMasterSize() --
//---------------------------------------------------------------------------

void P25::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  P25::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void P25::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  P25::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void P25::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  P25::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  P25::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  P25::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  P25::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

}  // namespace Innovative
