// P25_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef P25_MbH
#define P25_MbH


#include <LegacyBoard_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <P25DeviceMap_Mb.h>
#include <BaseboardInterfaces_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class FpaLoadThread;

//===========================================================================
//  CLASS P25  -- P25 Base Board object
//===========================================================================

class P25 :  public LegacyBoard,
             public IStreamDeviceEvents
{
    friend class P25DeviceMap;
    
public:
    //
    //  Member Functions
    P25();
    virtual ~P25();

    size_t          BoardCount();
    void            Open();
    void            Close();

    void  LocalLoopbackMode(bool state) {  FLocalLoopbackMode = state;  };
    FpgaLogicVersionInfoIntf & UserLogicVersion();
    PciLogicVersionInfoIntf & PciLogicVersion();

    int   Target();
    void  Target( int value );

    bool  Led();
    void  Led( bool value );

    ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );
    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );

	void 							   BusMasterSize(int bytes);
	void 							   IncomingBusMasterSize(int bytes);
	void 							   OutgoingBusMasterSize(int bytes);
	int             				   IncomingBusMasterSize() const;
	int             				   OutgoingBusMasterSize() const;

	//
    //  IPacketStreamDevice interface implementation
    virtual void  Reset();

    P25DeviceMap * Debug() { return &Device; };
    virtual void   Log(int type, int data);

    // Logic programming
    IUsesVirtexFpgaLoader & Logic();
    // EEprom programming
    IUsesVirtexJtagLoader & Eeprom();
    // Coff loader
    IUsesOnboardHpiCpu &  Cpu();

private:
    //
    //  Member Data
    P25DeviceMap          Device;
    bool                  FLocalLoopbackMode;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

