//  PacketDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <PacketDeviceImpl_Mb.h>
#include <Exception_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{
//==============================================================================
//  CLASS StandardDeviceDriverDebugUI -- Impl of Driver Debug Trace Callback
//==============================================================================

class StandardDeviceDriverDebugUI : public InnovativeKernel::IDeviceDriverDebugUI
{
public:
    StandardDeviceDriverDebugUI(BasicPacketDeviceImpl * pd)
        :  PD(pd)
        {}
    virtual ~StandardDeviceDriverDebugUI() {}

    virtual void  TraceLog(int code, int data)
        {  PD->Log(code, data);  }

private:
    BasicPacketDeviceImpl * PD;
};

//===========================================================================
//  CLASS BasicPacketDeviceImpl -- General Control of a Packet Device (sans stream)
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class BasicPacketDeviceImpl
//---------------------------------------------------------------------------

BasicPacketDeviceImpl::BasicPacketDeviceImpl(
                    InnovativeKernel::PacketBusMasterDeviceDriver * driver,
                    PacketDeviceRegisters & regs )
    : DeviceDriver(driver),  
      ControlRegister( PortMemory,  regs.ControlRegister_Addr ),
      StatusRegister ( PortMemory,  regs.StatusRegister_Addr ),
      RxPacketTimer  ( PortMemory,  regs.RxPacketTimerRegister_Addr ),
	  TxPacketTimer  ( PortMemory,  regs.TxPacketTimerRegister_Addr ),
	  LinkControlReg ( PortMemory,  regs.LinkControlRegister_Addr ),
      DebugLog(10000),  
      FTwoPhaseStop(false)
{
    DeviceDriver->LoadDebugUI(new StandardDeviceDriverDebugUI(this));
}

//---------------------------------------------------------------------------
//  destructor for class BasicPacketDeviceImpl
//---------------------------------------------------------------------------

BasicPacketDeviceImpl::~BasicPacketDeviceImpl()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t BasicPacketDeviceImpl::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::BoardId()  --  Get reference to device driver
//---------------------------------------------------------------------------

InnovativeKernel::PacketBusMasterDeviceDriver & BasicPacketDeviceImpl::Driver()
{
    return *DeviceDriver;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::IsOpened()
//---------------------------------------------------------------------------

bool  BasicPacketDeviceImpl::IsOpened()
{
    return DeviceDriver->Opened();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::DoOpen()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::DoOpen()
{
    if( IsOpened() )
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link( ( ii32 *)DeviceDriver->PortMemoryAddress() );

    ClearControlRegister();

    if (DeviceDriver->Mode()==InnovativeKernel::PacketBusMasterDeviceDriver::bmmScatter)
        {
        ControlRegister.ScatterGatherEn.Set();
        }
    else
        {
        ControlRegister.ScatterGatherEn.Clear();
        }
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::DoClose()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::DoClose()
{
    DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  BasicPacketDeviceImpl::Target()
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::Led()
//---------------------------------------------------------------------------

bool  BasicPacketDeviceImpl::Led()
{
    return ControlRegister.Led.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::Led()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::Led( bool value )
{
    ControlRegister.Led = value ? true : false;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::ReadStatus() - Read status register
//---------------------------------------------------------------------------

int  BasicPacketDeviceImpl::ReadStatus()
{
    int result =  StatusRegister.Value();
    laststatus = result;
    return result;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32   BasicPacketDeviceImpl::PeekPort(int offset)
{
    ii32 * PortBasePtr = ( ii32 *)DeviceDriver->PortMemoryAddress();
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = ( ii32 *)DeviceDriver->PortMemoryAddress();
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::DspInReset() -- Applies and Removes DSP reset state
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::DspInReset(bool state)
{
    //  0 puts DSP in reset, 1 removes reset...
    bool bit = (state) ? 0 : 1;
    ControlRegister.DspReset.Value(bit);
    Sleep(1);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::TwoPhaseStop() -- Set Stop Mode
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::TwoPhaseStop(bool state)
{
    FTwoPhaseStop = state;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::BusmasteringOn() -- Turn On Busmastering
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::BusmasteringOn()
{
    if (!DeviceDriver->Opened())
        return;
    //
    //  Begin full duplex busmastering
    ControlRegister.AllowBusmastering.Set();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::BusmasteringOff() -- Turn Off Busmastering
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::BusmasteringOff()
{
    if (!DeviceDriver->Opened())
        return;

    if (FTwoPhaseStop)
        {
        //
        //  End full duplex busmastering
        ControlRegister.BusmasterPauseRequest.Set();

        int loops(50);
        while (--loops)
            {
            if (StatusRegister.BusmasterPauseComplete.Value())
                break;
            Innovative::Sleep(10);
            }

        }
    //
    //  End full duplex busmastering
    ControlRegister.AllowBusmastering.Clear();
    if (FTwoPhaseStop)
        {
        ControlRegister.BusmasterPauseRequest.Clear();
        }
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::ClearControlRegister()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::ClearControlRegister()
{
    ControlRegister.BusmasterReset.Set();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::Reset()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::Reset()
{
    //
    //  Make sure BM is off...
    BusmasteringOff();
    Sleep(10);

    ControlRegister.BusmasterReset.Clear();
    Innovative::Sleep(10);
    ControlRegister.BusmasterReset.Set();
    Innovative::Sleep(10);
    //
    //  Clear Int Status queue 
    DeviceDriver->ClearInterruptStatusQueue();

    DebugLog.clear();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::BusmasterReset()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::BusmasterReset()
{
    Reset();  // alias of Reset();

}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::MasterReset()
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::MasterReset()
{
    ControlRegister.MasterReset.Set();
    Innovative::Sleep(10);
    ControlRegister.MasterReset.Clear();
    Innovative::Sleep(10);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::IsrEvent() - Get event for the VISR
//---------------------------------------------------------------------------

Event *  BasicPacketDeviceImpl::IsrEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::InterruptDepth() - Get depth of VISR queue
//---------------------------------------------------------------------------

size_t BasicPacketDeviceImpl::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::InterruptAck() - Acknowledge interrupt
//---------------------------------------------------------------------------

int BasicPacketDeviceImpl::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//------------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PciLoopbackMode() --
//------------------------------------------------------------------------------

void  BasicPacketDeviceImpl::PciLoopbackMode(int mode)
{
    ControlRegister.PciRocketIoLoopback.Value(mode);
}

//------------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PciLoopbackMode() --
//------------------------------------------------------------------------------

int  BasicPacketDeviceImpl::PciLoopbackMode()
{
    return ControlRegister.PciRocketIoLoopback.Value();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PciClockRate() -- Pci clock rate, in Mhz
//---------------------------------------------------------------------------

float  BasicPacketDeviceImpl::PciClockRate() const
{
    return (StatusRegister.MHz.Value()) ? 66.6f : 33.3f;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PciBusWidth() -- Pci bus width, in bits
//---------------------------------------------------------------------------

int BasicPacketDeviceImpl::PciBusWidth() const
{
    return (StatusRegister.BusWidth.Value()) ? 64 : 32;
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::LaneCount() -- PciExpress lane width, in bits
//---------------------------------------------------------------------------

int BasicPacketDeviceImpl::LaneCount() const
{
    return StatusRegister.LaneCount.Value();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::RocketIoErrors() -- Rocket I/O test failed counter
//---------------------------------------------------------------------------

int BasicPacketDeviceImpl::RocketIoErrors() const
{
    return StatusRegister.RocketIoErrors.Value();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::RxPacketPeriod() -- Receive packet transfer period
//---------------------------------------------------------------------------

int  BasicPacketDeviceImpl::RxPacketPeriod() const
{
    return RxPacketTimer.Period.Value();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::TxPacketPeriod() -- Transmit packet transfer period
//---------------------------------------------------------------------------

int  BasicPacketDeviceImpl::TxPacketPeriod() const
{
    return TxPacketTimer.Period.Value();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::IsGen2Capable() -- Gen 2 Logic linked & capable
//---------------------------------------------------------------------------

bool  BasicPacketDeviceImpl::IsGen2Capable() const
{
	return (LinkControlReg.Gen2Status.Value()==0x7) ? true : false;
}

//------------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PciOutgoingNoCreditModeOn()
//------------------------------------------------------------------------------

void  BasicPacketDeviceImpl::PciOutgoingNoCreditModeOn(bool state)
{
    if (state)
        ControlRegister.PciOutgoingNoCreditModeOn.Set();
    else
        ControlRegister.PciOutgoingNoCreditModeOn.Clear();
}

//------------------------------------------------------------------------------
//  BasicPacketDeviceImpl::PciIncomingNoCreditModeOn()
//------------------------------------------------------------------------------

void  BasicPacketDeviceImpl::PciIncomingNoCreditModeOn(bool state)
{
    if (state)
        ControlRegister.PciIncomingNoCreditModeOn.Set();
    else
        ControlRegister.PciIncomingNoCreditModeOn.Clear();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::DumpLog() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::DumpLog(std::vector<int> & vec)
{
    DebugLog.copy_to(vec);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::SetBusMasterSizeWords() --  Set BM region size
//---------------------------------------------------------------------------

void BasicPacketDeviceImpl::SetBusMasterSizeWords(ii32 value)
{
    DeviceDriver->BusMasterSizeWords(value);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::SetIncomingBusMasterSizeWords() --  Set BM region size
//---------------------------------------------------------------------------

void BasicPacketDeviceImpl::SetIncomingBusMasterSizeWords(ii32 value)
{
    DeviceDriver->IncomingBusMasterSizeWords(value);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::SetOutgoingBusMasterSizeWords() --  Set BM region size
//---------------------------------------------------------------------------

void BasicPacketDeviceImpl::SetOutgoingBusMasterSizeWords(ii32 value)
{
    DeviceDriver->OutgoingBusMasterSizeWords(value);
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::GetIncomingBusMasterSizeWords() --  Get BM region size
//---------------------------------------------------------------------------

ii32 BasicPacketDeviceImpl::GetIncomingBusMasterSizeWords() const
{
    return DeviceDriver->IncomingBusMasterSizeWords();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::GetOutgoingBusMasterSizeWords() --  Get BM region size
//---------------------------------------------------------------------------

ii32 BasicPacketDeviceImpl::GetOutgoingBusMasterSizeWords() const
{
    return DeviceDriver->OutgoingBusMasterSizeWords();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::BmManager() --  BM Mem Mgr
//---------------------------------------------------------------------------

InnovativeKernel::IDeviceDriverBmManagerUI & BasicPacketDeviceImpl::BmManager()
{
    return DeviceDriver->UI();
}

//---------------------------------------------------------------------------
//  BasicPacketDeviceImpl::ScatterGatherMode() --  Set Scatter/Gather mode 
//---------------------------------------------------------------------------

void  BasicPacketDeviceImpl::ScatterGatherMode(bool enabled)
{
    if (enabled)
        {
        DeviceDriver->ScatterMode();
        ControlRegister.ScatterGatherEn.Set();
        }
    else
        {
        DeviceDriver->LegacyMode();
        ControlRegister.ScatterGatherEn.Clear();
        }

}

//===========================================================================
//  CLASS PacketDeviceImpl -- General Control of a Packet Device (sans stream)
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketDeviceImpl
//---------------------------------------------------------------------------

PacketDeviceImpl::PacketDeviceImpl(
                    InnovativeKernel::DualRegionDeviceDriver * driver,
                    PacketDeviceRegisters & regs )
    : BasicPacketDeviceImpl(driver, regs)
{
}

//---------------------------------------------------------------------------
//  destructor for class PacketDeviceImpl
//---------------------------------------------------------------------------

PacketDeviceImpl::~PacketDeviceImpl()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  PacketDeviceImpl::DoOpen()
//---------------------------------------------------------------------------

void  PacketDeviceImpl::DoOpen()
{
    BasicPacketDeviceImpl::DoOpen();

    DRDriver = dynamic_cast<InnovativeKernel::DualRegionDeviceDriver *>(DeviceDriver);
    LogicMemory.Link(reinterpret_cast<ii32 *>(DRDriver->LogicMemoryAddress()));
}

//---------------------------------------------------------------------------
//  PacketDeviceImpl::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32  PacketDeviceImpl::PeekLogic(int offset)
{
    ii32 * LogicBasePtr = reinterpret_cast<ii32 *>(DRDriver->LogicMemoryAddress());
    return LogicBasePtr[offset];
}

//---------------------------------------------------------------------------
//  PacketDeviceImpl::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void  PacketDeviceImpl::PokeLogic(int offset, ii32 value)
{
    ii32 * LogicBasePtr = reinterpret_cast<ii32 *>(DRDriver->LogicMemoryAddress());

    LogicBasePtr[offset] = value;
}


//===========================================================================
//  CLASS PacketDeviceRegMixin::PciControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketDeviceRegMixin::PciControlRegister
//---------------------------------------------------------------------------
#ifdef _MSC_VER		
    #pragma warning(push)	
    #pragma warning(disable : 4355)		
#endif

PacketDeviceRegMixin::PciControlRegister::PciControlRegister(AddressingSpace &space,
    int offset )
    : inherited( space, offset ),
      MasterReset( *this, 0 ),
      Led( *this, 1 ),
      OldLogicReset( *this, 2 ),
      BusmasterReset( *this, 3 ),
      AllowBusmastering( *this, 4 ),
      DspReset( *this, 5 ),
      PciRocketIoLoopback( *this, 6, 2),
      ScatterGatherEn(*this, 8),
      PciOutgoingNoCreditModeOn( *this, 10 ),
      PciIncomingNoCreditModeOn( *this, 9 ),
      BusmasterPauseRequest( *this, 11 )
{
    Led.Set();  // we like this as a default better
}

//===========================================================================
//  CLASS PacketDeviceRegMixin::PciStatusRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketDeviceRegMixin::PciStatusRegister
//---------------------------------------------------------------------------

PacketDeviceRegMixin::PciStatusRegister::PciStatusRegister(AddressingSpace &space,
	int offset )
	: inherited( space, offset ),
	  BusWidth( *this, 0 ),
	  AllClear( *this, 1 ),
	  WriteFifoEmpty( *this, 2 ),
	  WriteFifoFull( *this, 3 ),
	  ReadFifoEmpty( *this, 4 ),
	  ReadFifoFull( *this, 5 ),
	  BusmasterPauseComplete( *this, 7 ),
	  MHz( *this, 6 ),
	  DeviceControl( *this, 8, 7),
	  RocketIoErrors ( *this, 16, 8),
	  LaneCount ( *this, 24, 6)
{
}

//===========================================================================
//  CLASS PacketDeviceRegMixin::PacketTimerRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketDeviceRegMixin::PacketTimerRegister
//---------------------------------------------------------------------------

PacketDeviceRegMixin::PacketTimerRegister::PacketTimerRegister( AddressingSpace &space, int offset )
    : inherited( space, offset ),
      Period( *this, 0, 20 )
{
}

//===========================================================================
//  CLASS PacketDeviceRegMixin::LinkControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketDeviceRegMixin::LinkControlRegister
//---------------------------------------------------------------------------

PacketDeviceRegMixin::LinkControlRegister::LinkControlRegister(AddressingSpace &space,
	int offset )
	: inherited( space, offset ),
	  Gen2Status ( *this, 21, 3),
	  LaneCount ( *this, 24, 6)        
{
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // end namespace Innovative

