//  PacketDeviceImpl_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef PacketDeviceImpl_MbH
#define PacketDeviceImpl_MbH

#include <PacketDeviceSupport_Mb.h>
#include <DeviceDriver_Mb.h>
#include <ThreadSafeLog_Mb.h>
#include <IPciBusDiagnostics_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS PacketDeviceRegisters  -- Addresses for Device Registers
//===========================================================================

class PacketDeviceRegisters
{
public:
    unsigned int   ControlRegister_Addr;
    unsigned int   StatusRegister_Addr;
    unsigned int   RxPacketTimerRegister_Addr;
    unsigned int   TxPacketTimerRegister_Addr;
	unsigned int   LinkControlRegister_Addr;
};

//===========================================================================
//  CLASS PacketDeviceRegMixin  -- Packet Device Register Defs
//===========================================================================

class PacketDeviceRegMixin
{
public:
    //
    //  Class PciControlRegister
    class PciControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       MasterReset;
        RegisterBit       Led;
        RegisterBit       OldLogicReset;
        RegisterBit       BusmasterReset;
        RegisterBit       AllowBusmastering;
        RegisterBit       DspReset;
        RegisterBitGroup  PciRocketIoLoopback;
        RegisterBit       ScatterGatherEn;
        RegisterBit       PciOutgoingNoCreditModeOn;
        RegisterBit       PciIncomingNoCreditModeOn;
        RegisterBit       BusmasterPauseRequest;

    public:
        PciControlRegister( AddressingSpace &space, int offset );

    };
    //
    //  Class PciStatusRegister
    class PciStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit       BusWidth;
        ReadOnlyRegisterBit       AllClear;
        ReadOnlyRegisterBit       WriteFifoEmpty;
        ReadOnlyRegisterBit       WriteFifoFull;
        ReadOnlyRegisterBit       ReadFifoEmpty;
        ReadOnlyRegisterBit       ReadFifoFull;
        ReadOnlyRegisterBit       BusmasterPauseComplete;
        ReadOnlyRegisterBit       MHz;
        ReadOnlyRegisterBitGroup  DeviceControl;
        ReadOnlyRegisterBitGroup  RocketIoErrors;
        ReadOnlyRegisterBitGroup  LaneCount;

    public:
        PciStatusRegister( AddressingSpace &space, int offset );
    };
    //
    //  Class PacketTimerRegister
    class PacketTimerRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup   Period;

    public:
        PacketTimerRegister( AddressingSpace &space, int offset );
    };

	//
	//  Class LinkControlRegister
	class LinkControlRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
		ReadOnlyRegisterBitGroup  Gen2Status;
        ReadOnlyRegisterBitGroup  LaneCount;   // duplicate of the one in Status Register

    public:
        LinkControlRegister( AddressingSpace &space, int offset );
    };

};


//===========================================================================
//  CLASS BasicPacketDeviceImpl -- General Control of a Packet Device (sans stream)
//===========================================================================

class BasicPacketDeviceImpl : public IPacketDevice,
                              public IPciBusDiagnostics,
                              protected PacketDeviceRegMixin   // register defs
{
public:
    BasicPacketDeviceImpl(InnovativeKernel::PacketBusMasterDeviceDriver * driver,
                          PacketDeviceRegisters & regs);
    ~BasicPacketDeviceImpl();

	size_t BoardCount();
    bool   LicenseCheck(unsigned int & code);
    InnovativeKernel::PacketBusMasterDeviceDriver & Driver();

    virtual bool  IsOpened();
    virtual void  DoOpen();
    virtual void  DoClose();

    void  Reset();
    virtual void BusmasterReset();  // alias of Reset();


    int   Target();
    void  Target(int target);

    bool  Led();
    void  Led( bool value );

    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );

    void  DspInReset(bool state);
    virtual void     TwoPhaseStop(bool state);

    void  MasterReset();

    void  ScatterGatherMode(bool enabled);

    // Specify driver-allocated BM region, prior to open
    virtual void            SetBusMasterSizeWords(ii32 value);
    virtual void            SetIncomingBusMasterSizeWords(ii32 value);
    virtual void            SetOutgoingBusMasterSizeWords(ii32 value);
    virtual ii32            GetIncomingBusMasterSizeWords() const;
    virtual ii32            GetOutgoingBusMasterSizeWords() const;

    virtual InnovativeKernel::IDeviceDriverBmManagerUI &  BmManager();

    virtual void     BusmasteringOn();
    virtual void     BusmasteringOff();

    virtual void     PciOutgoingNoCreditModeOn(bool state);
    virtual void     PciIncomingNoCreditModeOn(bool state);

    //  Debug
    virtual int       ReadStatus();
    virtual float     PciClockRate() const;
    virtual int       PciBusWidth() const;
	virtual int       LaneCount() const;
	virtual bool      IsGen2Capable() const;
	virtual int       RocketIoErrors() const;
    virtual int       RxPacketPeriod() const;
    virtual int       TxPacketPeriod() const;

protected:
    //
    //  Member Functions
    void  PciLoopbackMode(int mode);
    int   PciLoopbackMode();
    //
    //  Debug
    int      laststatus;

    //
    //  Member Data
    InnovativeKernel::PacketBusMasterDeviceDriver *   DeviceDriver;
    PciControlRegister                                ControlRegister;
    PciStatusRegister                                 StatusRegister;
    PacketTimerRegister                               RxPacketTimer;
	PacketTimerRegister                               TxPacketTimer;
	LinkControlRegister                               LinkControlReg;
    ThreadSafeLog                                     DebugLog;
    bool                                              FTwoPhaseStop;


    //
    //  Member Functions
    void     ClearControlRegister();
    Event *  IsrEvent();
    size_t   InterruptDepth();
    int      InterruptAck();

public:
    //  Debug
    void  Log(int code, int data) {  DebugLog.append(code, data);  };
    void  DumpLog(std::vector<int> & vec);
};

//===========================================================================
//  CLASS PacketDeviceImpl -- General Control of a Packet Device (sans stream)
//===========================================================================

class PacketDeviceImpl :  public BasicPacketDeviceImpl
{
public:
    PacketDeviceImpl(InnovativeKernel::DualRegionDeviceDriver * driver,
                     PacketDeviceRegisters & regs);
    ~PacketDeviceImpl();

    virtual void  DoOpen();

    ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );

private:
    InnovativeKernel::DualRegionDeviceDriver *      DRDriver;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
