//  PacketDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef PacketDeviceMap_MbH
#define PacketDeviceMap_MbH

#include <PacketInterruptScheme_Mb.h>
#include <PacketDeviceSupport_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <SafeLink_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Event;

//===========================================================================
//  CLASS PacketDeviceMapRegisters  -- Addresses for Device Registers
//===========================================================================

class PacketDeviceMapRegisters
{
public:
    unsigned int   InterruptEnable_Addr;
    unsigned int   CreditQueue_Addr;
    unsigned int   BmRingBaseAddressArray_Addr;
    unsigned int   BmRingSizeArray_Addr;
};

namespace PacketDevMap
{
    class DeviceToScheme;
}

//===========================================================================
//  CLASS PacketDeviceMap  -- Base class for Packet mode BM devices
//===========================================================================

class PacketDeviceMap :  public IPacketStreamDeviceInternals,
                         public IStreamDeviceEvents,
                         protected PacketInterruptRegMixin   // register defs

{
    friend class PacketDevMap::DeviceToScheme;

public:
    //
    //  Member Functions
    PacketDeviceMap(IPacketDevice * pkt_device_intf,
                    PacketDeviceMapRegisters & regs,
                    IPacket_DeviceToScheme * scheme_intf);
    virtual ~PacketDeviceMap();

    PacketDeviceMap *  PDMPtr()
        {  return this;  };

    enum Dir { BmXmit, BmRecv };
    void  FillBmRegion(Dir which, unsigned int pattern, bool ramp=false);

public: // Pseudo-protected
    virtual void  DoOpen();
    virtual void  DoClose();

    //
    //  Busmaster Intf functions
    void  RegisterStreamPlugIn( class IPacketStreamManagerInternals *plug_in );
    void  GrantCredit(int channel, size_t size_wds);
    void  TransmitDataAvailable();
    // Specify driver-allocated BM region, prior to open
    virtual void            BusMasterSizeWords(unsigned int value);
    virtual void            IncomingBusMasterSizeWords(unsigned int value);
    virtual void            OutgoingBusMasterSizeWords(unsigned int value);
    virtual unsigned int    IncomingBusMasterSizeWords() const;
    virtual unsigned int    OutgoingBusMasterSizeWords() const;
    virtual void  PreconfigBusmastering();
    virtual void  ConfigureBusmastering();
    virtual void  PreloadXmitBusmasterRegion();
    virtual void  StartBusmastering();
    virtual void  StopBusmastering();
    virtual bool  AlertTest(unsigned int pid);
    virtual bool  AlertCheck(const Buffer & buffer);
    virtual void  DispatchCustomInterrupt(int CustomFlags);
    virtual void  BusmasterReset();                   
    virtual void  TraceLog(int type, int data);
    virtual unsigned int  VisrIsrQDepth();


    void  EnableCustomInterrupt(bool state);

protected:
    //
    //  Member Data
    IPacketDevice *                    PacketDevice;
    PacketInterruptScheme *            IntScheme;
    SafeLink<IPacketStreamManagerInternals,
             SafePacketStreamManagerInternalsImpl>  StreamManagerInternals;
    //
    //  Member Functions
    void  PciLoopbackMode(int mode);
    int   PciLoopbackMode();

private:
    //
    //  Member Data
    CriticalSection                    CreditLock;
    PacketDeviceMapRegisters           RegAddr;
    //
    //  ...Registers
private:
    CreditQueueRegister                CreditQueue;
    InterruptEnableRegister            InterruptEnable;

    //
    //  Member Functions
    void     EnableBusmasterInterrupt( bool enable );
    void     Error( std::string msg );

public:
    ii32 *          XmitPhysicalAddr() {  return IntScheme->XmitRgnBasePhys;  };
    ii32 *          RecvPhysicalAddr() {  return IntScheme->RecvRgnBasePhys;  };
    ii32 *          XmitAddr() {  return IntScheme->XmitRgnBase;  };
    ii32 *          RecvAddr() {  return IntScheme->RecvRgnBase;  };
    unsigned int    XmitRegionSize() {  return IntScheme->XmitRgnSize;  };
    unsigned int    RecvRegionSize() {  return IntScheme->RecvRgnSize;  };

    //  Debug info words
    int   InitRcvCredits;
    bool  ShouldPreloadXmit;
    bool  ShouldIssueRcvCredit;
    bool  SingleTimeAtStartPreconfig;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
