//  PacketDeviceSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef PacketDeviceSupport_MbH
#define PacketDeviceSupport_MbH

#include <HardwareRegister_Mb.h>
#include <Event_Mb.h>
#include <DeviceDriverBmManager_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IPacketDevicePlugin -- Installable Callbacks for PacketDevice
//===========================================================================

class IPacketDevicePlugin
{
public:
    virtual void  AfterDeviceOpen() = 0;
    virtual void  BeforeDeviceClose() = 0;
    
    virtual ~IPacketDevicePlugin()
        {}

};

//===========================================================================
//  CLASS IPacketDevice -- Interface to a Packet Device (sans stream)
//===========================================================================

class IPacketDevice
{
public:
    IPacketDevice() :  StreamPlugIn(0), HdwPlugIn(0)
        {}
    virtual ~IPacketDevice()
        {
            if (StreamPlugIn)  delete StreamPlugIn;
            if (HdwPlugIn)     delete HdwPlugIn;
        }

    void  InstallStreamPlugin(IPacketDevicePlugin * plugin)
        {
            if (StreamPlugIn)
                delete StreamPlugIn;
            StreamPlugIn = plugin;
            // Must call open once when first connected
            if (StreamPlugIn)
                StreamPlugIn->AfterDeviceOpen();
        }
    void  InstallHdwPlugin(IPacketDevicePlugin * plugin)
        {
            if (HdwPlugIn)
                delete HdwPlugIn;
            HdwPlugIn = plugin;
        }

    void  Open()
        {
            if (IsOpened()) // don't open twice
                return;
            DoOpen();
            if (HdwPlugIn)
                HdwPlugIn->AfterDeviceOpen();
            if (StreamPlugIn)
                StreamPlugIn->AfterDeviceOpen();
        }
    void  Close()
        {
            if (IsOpened()==false)  // don't close twice
                return;
            if (StreamPlugIn)
                StreamPlugIn->BeforeDeviceClose();
            if (HdwPlugIn)
                HdwPlugIn->BeforeDeviceClose();
            DoClose();
        }

    virtual bool     IsOpened() = 0;
    virtual void     DoOpen() = 0;
    virtual void     DoClose() = 0;

    //  ...Driver Busmaster Manager
    virtual InnovativeKernel::IDeviceDriverBmManagerUI &  BmManager() = 0;
    // Specify driver-allocated BM region, prior to open
    virtual void            SetBusMasterSizeWords(ii32 value) = 0;
    virtual void            SetIncomingBusMasterSizeWords(ii32 value) = 0;
    virtual void            SetOutgoingBusMasterSizeWords(ii32 value) = 0;
    virtual ii32            GetIncomingBusMasterSizeWords() const = 0;
    virtual ii32            GetOutgoingBusMasterSizeWords() const = 0;

    virtual Event *  IsrEvent() = 0;
    virtual size_t   InterruptDepth() = 0;
    virtual int      InterruptAck() = 0;
    virtual void     DspInReset(bool state) = 0;
    virtual void     TwoPhaseStop(bool state) = 0;

    virtual void     BusmasteringOn() = 0;
    virtual void     BusmasteringOff() = 0;

    virtual void     PciOutgoingNoCreditModeOn(bool state) = 0;
    virtual void     PciIncomingNoCreditModeOn(bool state) = 0;

    virtual void     BusmasterReset() = 0;
    virtual void     Log(int code, int data) = 0;
    //
    // Memory spaces
    AddressingSpace                    PortMemory;
    AddressingSpace                    LogicMemory;

private:
    IPacketDevicePlugin *              StreamPlugIn;
    IPacketDevicePlugin *              HdwPlugIn;
};



#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
