// PacketFileDataSet_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <IniFile_Mb.h>
#include <PacketFileDataSet_Mb.h>
#include <FileSupport_Mb.h>
#include <algorithm>
#include <limits>
#include <cstring>

using namespace std;

namespace Innovative
{

const size_t MaximumIndex    = std::numeric_limits<size_t>::max();
const ii64   MaximumIndex64  = std::numeric_limits<ii64>::max();
const int kAlertPid = 0xFF;

//===========================================================================
// Class PacketBlock
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketBlock
//---------------------------------------------------------------------------

PacketBlock::PacketBlock()
    : Allocated(false), FSize(ii64(0)), Data(0)
{
}

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4996)
#endif

//---------------------------------------------------------------------------
//  constructor for class PacketBlock -- copy ctor
//---------------------------------------------------------------------------

PacketBlock::PacketBlock(const PacketBlock & other)
    : Allocated(other.Allocated), FSize(other.FSize), Data(0)
{
    if (Allocated)
        {
        Allocate(other.FSize);
        std::copy(other.Data, other.Data+other.FSize, Data);
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class PacketBlock
//---------------------------------------------------------------------------

PacketBlock::~PacketBlock()
{
    Release();
}

//---------------------------------------------------------------------------
//  PacketBlock::Allocate() - Allocate space
//---------------------------------------------------------------------------

void  PacketBlock::Allocate(unsigned int size)
{
    Data = new unsigned int [size];
    FSize = size;
    Allocated = true;
}

//---------------------------------------------------------------------------
//  PacketBlock::Release() - Release space
//---------------------------------------------------------------------------

void  PacketBlock::Release()
{
    delete [] Data;
    Data = 0;
    FSize = 0;
    Allocated = false;
}

//===========================================================================
// Class PacketFile
//===========================================================================
//---------------------------------------------------------------------------
// PacketFile::PacketFile() -- Constructor
//---------------------------------------------------------------------------

PacketFile::PacketFile()
    : FAutoCreate(false), FCursor(ii64(0))
{
}

//---------------------------------------------------------------------------
// PacketFile::Flush() -- Flush cache to disk
//---------------------------------------------------------------------------

void PacketFile::Flush()
{
/*
    if (!Dirty)
        return;

    unsigned int EndOfCache = CacheOffset+CacheSize;
    unsigned int Residual = ((unsigned int)FSize < EndOfCache)
                            ? CacheSize-(EndOfCache-(unsigned int)FSize) : CacheSize;

    File.seekp(CacheOffset, soFromBeginning);
    File.write(Cache, Residual);

    Dirty = false;
*/
}

//---------------------------------------------------------------------------
// PacketFile::Size() -- Return file size
//---------------------------------------------------------------------------

ii64 PacketFile::Size() const
{
    return File.Size();
}

//---------------------------------------------------------------------------
// PacketFile::Open() -- Open disk file
//---------------------------------------------------------------------------

bool PacketFile::Open(std::string filename, bool auto_create, bool read_only)
{
    FFileName = filename;
    File.FileName(FFileName);

    // Open the file
    if (!FileExists(FFileName) && auto_create)
        File.Create();
    else if (read_only)
        File.Open();
    else
        File.Append();

    if (!File.Opened())
        return false;

    Seek(0);
    FCursor = 0;

    return true;
}

//---------------------------------------------------------------------------
// PacketFile::Close() -- Close disk file
//---------------------------------------------------------------------------

void PacketFile::Close()
{
    if (!File.Opened())
        return;

    // Flush, if necessary
    Flush();

    // Close handle to device
    File.Close();
    FCursor = 0;
}

//---------------------------------------------------------------------------
// PacketFile::Seek() -- Move position in file
//---------------------------------------------------------------------------

bool PacketFile::Seek(ii64 offset_from_front)
{
    return Position(offset_from_front);
}

//---------------------------------------------------------------------------
// PacketFile::Read() -- Read from file to buffer
//---------------------------------------------------------------------------

bool PacketFile::Read(unsigned int * buffer, unsigned int size_in_ints)
{
    unsigned int size_in_bytes = size_in_ints*sizeof(int);
    unsigned int read = File.Read(reinterpret_cast<char*>(buffer), size_in_bytes);

    return (read == size_in_bytes);
}

//---------------------------------------------------------------------------
// PacketFile::Write() -- Write to file from buffer
//---------------------------------------------------------------------------

bool PacketFile::Write(unsigned int * buffer, unsigned int size_in_ints)
{
    unsigned int size_in_bytes = size_in_ints*sizeof(int);
    unsigned int wrote = File.Write(reinterpret_cast<char*>(buffer), size_in_bytes);

    return (wrote == size_in_bytes);
}

//---------------------------------------------------------------------------
// PacketFile::Skip() -- Move position in file
//---------------------------------------------------------------------------

bool PacketFile::Skip(int size_in_ints)
{
    return Position(Position() + size_in_ints*sizeof(int));
}

//---------------------------------------------------------------------------
// PacketFile::Position() -- Position file cursor
//---------------------------------------------------------------------------

bool PacketFile::Position(ii64 offset)
{
    bool result = File.Position(offset);
    return result;
}

//---------------------------------------------------------------------------
// PacketFile::Position() -- Retreive file cursor
//---------------------------------------------------------------------------

ii64 PacketFile::Position()
{
    ii64 result = File.Position();
    return result;
}

//===========================================================================
//  Class PacketProxy
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketProxy
//---------------------------------------------------------------------------

PacketProxy::PacketProxy()
    :  PFile(0)
{
}

//---------------------------------------------------------------------------
//  constructor for class PacketProxy - build from sample idx
//---------------------------------------------------------------------------
//  Used for scanning array for proxy containing sample

PacketProxy::PacketProxy(ii64 sample_idx)
    :  BaseSampleNumber(sample_idx), PFile(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class PacketProxy
//---------------------------------------------------------------------------

PacketProxy::~PacketProxy()
{
}

//---------------------------------------------------------------------------
//  PacketProxy::AllocateBody() -- Fill proxy with body data from file
//---------------------------------------------------------------------------

void PacketProxy::AllocateBody()
{
    if (!Body.IsAllocated())
        {
        //  Find
        PFile->Seek(OffsetAfterHeader());
        //  Allocate
        Body.Allocate(DataSize);
        //  Fill
        PFile->Read(Body.Data, DataSize);
        }
}

//---------------------------------------------------------------------------
//  PacketProxy::ReleaseBody() -- empty data from proxy
//---------------------------------------------------------------------------

void PacketProxy::ReleaseBody()
{
    if (Body.IsAllocated())
        {
        Body.Release();
        }
}

//===========================================================================
//  Class PidPacketArray
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PidPacketArray
//---------------------------------------------------------------------------

PidPacketArray::PidPacketArray()
    :  MaxSampleIdx(0), CacheSizeBlocks(500), CurrentProxyIdx(MaximumIndex)
{
}

//---------------------------------------------------------------------------
//  destructor for class PidPacketArray
//---------------------------------------------------------------------------

PidPacketArray::~PidPacketArray()
{
}

//---------------------------------------------------------------------------
//  PidPacketArray::Clear() -- clear all state of array
//---------------------------------------------------------------------------

void  PidPacketArray::Clear()
{
    MaxSampleIdx = 0;
    CurrentProxyIdx= MaximumIndex;

    Packets.clear();
    Cache.clear();
}

//---------------------------------------------------------------------------
//  PidPacketArray::CurrentSample() -- Read sample
//---------------------------------------------------------------------------

Subsample PidPacketArray::CurrentSample(int channel_in_pid, ii64 index)
{
    Subsample value(0);

    FindProxyForSample(index);

    if (CurrentProxyIdx != MaximumIndex)
        {
        PacketProxy & CurrentProxy(Packets[CurrentProxyIdx]);
        //
        //  Manage Proxy Data Allocation
        AllocateProxy();
        //
        //  Fetch the data point
        switch (CurrentProxy.BytesPerSample)
            {
            case 4:
                {
                unsigned int * BodyPtr = CurrentProxy.Body.Data;
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_pid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 4);
                }
                break;
            case 2:
                {
                unsigned short * BodyPtr = reinterpret_cast<unsigned short *>(CurrentProxy.Body.Data);
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_pid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 2);
                }
                break;
            case 1:
                {
                unsigned char * BodyPtr = reinterpret_cast<unsigned char *>(CurrentProxy.Body.Data);
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_pid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 1);
                }
                break;
            }
        }

    return value;
}

//---------------------------------------------------------------------------
//  PidPacketArray::CurrentSample() -- Write sample
//---------------------------------------------------------------------------

void PidPacketArray::CurrentSample(int /*channel_in_pid*/, ii64 /*index*/, Subsample & /*value*/)
{
    // Intentionally blank so far
}

//---------------------------------------------------------------------------
//  PidPacketArray::LastPacket() -- Return last packet in the array
//---------------------------------------------------------------------------

PacketProxy *   PidPacketArray:: LastPacket()
{
    if (Packets.size()==0)
        return 0;
    else
        {
        size_t idx = Packets.size()-1;
        return &(Packets[idx]);
        }
}

//------------------------------------------------------------------------------
//  PidPacketArray::FindProxyForSample() -- Find proxy that holds sample idx
//------------------------------------------------------------------------------

void  PidPacketArray::FindProxyForSample(ii64 index)
{
    if (Packets.size()==0)
        {
        CurrentProxyIdx = MaximumIndex;
        return;
        }
    //  Check if current block has the sample
    if ((CurrentProxyIdx != MaximumIndex ) && Packets[CurrentProxyIdx].PacketContainsSample(index))
        return;

    // Scan our array for the block that does contain the sample...
    PacketProxy candidate(index);
    PacketArray::iterator iter = std::lower_bound(Packets.begin(), Packets.end(), candidate);
    if (iter != Packets.end())
        {
        CurrentProxyIdx = iter - Packets.begin();
        }
    else
        {
        CurrentProxyIdx = MaximumIndex;     // should never happen
		}
}

//------------------------------------------------------------------------------
//  PidPacketArray::AllocateProxy() -- Manage cache allocation of proxy
//------------------------------------------------------------------------------

void  PidPacketArray::AllocateProxy()
{
    PacketProxy & CurrentProxy(Packets[CurrentProxyIdx]);
    //
    //  CurrentProxy may not have been allocated yet.
    if (CurrentProxy.Body.IsAllocated()==false)
        {
        CurrentProxy.AllocateBody();
        //
        //  Push IDX of proxy onto cache
        Cache.push_front(CurrentProxyIdx);
        //  if too many blocks in cache, free one
		if (Cache.size() > CacheSizeBlocks)
            {
            size_t Last = Cache.back();
            //  Release from cache
            Cache.pop_back();
            //  Release data block of packet
            Packets[Last].ReleaseBody();
            }
        }
}

//------------------------------------------------------------------------------
//  PidPacketArray::FoundProxy() -- View found Proxy
//------------------------------------------------------------------------------

PacketProxy &  PidPacketArray::FoundProxy()
{
    return Packets[CurrentProxyIdx];
}

//------------------------------------------------------------------------------
//  PidPacketArray::IsFoundProxyValid() -- is proxy valid?
//------------------------------------------------------------------------------

bool  PidPacketArray::IsFoundProxyValid()
{
    return CurrentProxyIdx != MaximumIndex;
}

//===========================================================================
// Class PacketFileDataSet
//===========================================================================

//---------------------------------------------------------------------------
// PacketFileDataSet::PacketFileDataSet() -- Constructor
//---------------------------------------------------------------------------

PacketFileDataSet::PacketFileDataSet()
    : FSampleIdx(0), FImaginary(false), FLastArray(0), FAutoCreate(false)
{
}

//---------------------------------------------------------------------------
// PacketFileDataSet Destructor
//---------------------------------------------------------------------------

PacketFileDataSet::~PacketFileDataSet()
{
    ImageClose();
}

//---------------------------------------------------------------------------
//  PacketFileDataSet::ImageInit() -- On Image Initialization
//---------------------------------------------------------------------------

void  PacketFileDataSet::ImageInit()
{
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageOpen() -- Open disk file
//---------------------------------------------------------------------------

bool PacketFileDataSet::ImageOpen()
{
    bool ForceDataScan(true);
    File.Open(FFileName, FAutoCreate, FReadOnly);
    int FChannelSave = FChannel;
    int testChan = 0;
    for (int i=0; i<FDevices; i++)
        {
        //  Find sample 0 on ALL devices
        FChannel = testChan;
        ParseDataFile(ForceDataScan);
        testChan += SubchannelsPerDevice();
        }
    FChannel = FChannelSave;  // restore original channel.

    Init();

    return true;
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageClose() -- Close disk file
//---------------------------------------------------------------------------

void PacketFileDataSet::ImageClose()
{
    //  Clean out Alert packet array
    Alert_PPA.Clear();
    //  Clean out Pid parser map
    {
    PidPacketMap  eraser;
    eraser.swap(Pid);
    }
    //  Clean out parse limit array
    {
    std::vector<ii64>  eraser;
    eraser.swap(FParseLimitIdx);
    }
    FLastArray = 0;
    FSampleIdx = 0;
    FImaginary = false;

    File.Close();
}

//---------------------------------------------------------------------------
// PacketFileDataSet::Subchannel() -- Return current subchannel
//---------------------------------------------------------------------------

int PacketFileDataSet::Subchannel() const
{
    int ch;

    if (FComplex)
        ch = FChannel * Subsamples() + (FImaginary ? 1 : 0);
    else
		ch = FChannel;

    return ch;
}

//---------------------------------------------------------------------------
//  PacketFileDataSet::ImageSampleFetch() -- Fetch sample from current channel
//---------------------------------------------------------------------------

void  PacketFileDataSet::ImageSampleFetch(Subsample & sample)
{
    if (!File.IsOpen())
        return;
    //
    //  If we havent parsed this far, we will now!
    ParseDataFile();

    int subchannel = Subchannel()%SubchannelsPerDevice();
    sample = WorkingPid().CurrentSample(subchannel, FSampleIdx);
    char * pSample = sample.asString();
    CorrectEndian(pSample);
    sample.Type(static_cast<enum Subsample::SampleType>(FType));

    FImaginary = !FImaginary;
}

//---------------------------------------------------------------------------
//  PacketFileDataSet::ImageSampleStore() -- Store sample to current channel
//---------------------------------------------------------------------------

void  PacketFileDataSet::ImageSampleStore(Subsample & sample)
{
    if (!File.IsOpen() || FReadOnly)
        return;

    int subchannel = Subchannel()%SubchannelsPerDevice();
    char * pSample = sample.asString();
    CorrectEndian(pSample);
    WorkingPid().CurrentSample(subchannel, FSampleIdx, sample);

    FImaginary = !FImaginary;
}

//---------------------------------------------------------------------------
//  PacketFileDataSet::PreAccess() -- Init prior to sample access
//---------------------------------------------------------------------------

void PacketFileDataSet::PreAccess()
{
    FImaginary = false;
}

//---------------------------------------------------------------------------
//  PacketFileDataSet::PostAccess() -- Finalize after sample access
//---------------------------------------------------------------------------

void PacketFileDataSet::PostAccess()
{
    ++FSampleIdx;
    Progress(FSampleIdx);
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageFlush() -- Write dirty cache to disk
//---------------------------------------------------------------------------

void PacketFileDataSet::ImageFlush()
{
    File.Flush();
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageSeek() -- Seek to byte offset in disk file
//---------------------------------------------------------------------------

void PacketFileDataSet::ImageSeek(ii64 offset)
{
    FSampleIdx = offset/FEventSize;
    Progress(FSampleIdx);
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageSize() -- Return size of disk file
//---------------------------------------------------------------------------

ii64 PacketFileDataSet::ImageSize() const
{
    return File.Size() - PacketCount()*sizeof(PacketHeader);
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageSize() -- Set size of disk file
//---------------------------------------------------------------------------

void PacketFileDataSet::ImageSize(ii64 /*size*/)
{
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ParseDataFile() --
//---------------------------------------------------------------------------

void PacketFileDataSet::ParseDataFile(bool force_scan)
{
    if (!File.IsOpen())
        return;
    //
    //  First lets check if this sample has already been parsed.
    if (!force_scan)
        {
        ii64 idx = WorkingPid().MaxSampleIdx;
        if (idx > FSampleIdx)
            return;
        }

    // Otherwise Parse the file until we find the channel's sample
    //   Position at last parsed position
    ii64  Position = LastParsePosition();       // take outa loop - redundant
    File.Seek(Position);

    while (1==1)
        {
        //
        //  Extract the next packet header
        PacketProxy Proxy;
        //
		//  Read the header
        int PacketHeaderSize = Proxy.Header.SizeInInts();
		FFileReadOk = true;
		bool readOK = File.Read(Proxy.Header.Raw(), PacketHeaderSize);

		if (!readOK)
			{
			FFileReadOk = false;
			break;
			}

		//
        //  Find out which PPA this packet will be added to..
        int PacketPID = Proxy.Header.PeripheralId();

        if (PacketPID != kAlertPid)
            {
            // not an alert
			PidPacketArray & PPA = Pid[ PacketPID ];
            FLastArray = &PPA;

            PidPacketArray::PacketArray & PPV = PPA.Packets;

            //  Fill in the rest
            InitializeProxy(Proxy, Position, PPA.MaxSampleIdx);
            //
            //  Add proxy to our array
            PPA.MaxSampleIdx  += Proxy.Samples;
            PPV.push_back(Proxy);
            }
        else
            {
            //  an alert
            PidPacketArray::PacketArray & PPV = Alert_PPA.Packets;
            //  Fill in the rest
            InitializeProxy(Proxy, Position, Alert_PPA.MaxSampleIdx);
            //
            //  Add proxy to our array
            Alert_PPA.MaxSampleIdx  += Proxy.Samples;
            PPV.push_back(Proxy);
            }

        // Skip the data
        int PacketSize = Proxy.Header.PacketSize();
        int PacketDataSize = PacketSize - PacketHeaderSize;

        File.Skip(PacketDataSize);
        Position = File.Position();
        //
        //  Have we gotten the data on OUR channel?
        if (PacketPID==PidFromSubchannel(Subchannel()))
            if (WorkingPid().MaxSampleIdx > FSampleIdx)
                break;
        }
}

//---------------------------------------------------------------------------
// PacketFileDataSet::InitializeProxy() --
//---------------------------------------------------------------------------

void  PacketFileDataSet::InitializeProxy(PacketProxy & Proxy, ii64 Position, ii64 SampleIdx)
{
    int PacketHeaderSize = Proxy.Header.SizeInInts();
    int PacketSize       = Proxy.Header.PacketSize();
    int PacketDataSize   = PacketSize - PacketHeaderSize;

    Proxy.UseFile(&File);
    Proxy.Offset            = Position;
    Proxy.Size              = PacketSize;
    Proxy.DataSize          = PacketDataSize;
    Proxy.BaseSampleNumber  = SampleIdx;
    Proxy.Channels          = SubchannelsPerDevice();      // Number of channels per PID
    Proxy.Samples           = PacketDataSize * sizeof(int) / FSubsampleSize / Proxy.Channels;
	Proxy.BytesPerSample    = FSubsampleSize;
	Proxy.Index             = PacketCount();

}

//---------------------------------------------------------------------------
// PacketFileDataSet::LastParsePosition() --
//---------------------------------------------------------------------------

ii64  PacketFileDataSet::LastParsePosition()
{
    if (FLastArray==0)
        return 0;
    else
        {
        PacketProxy * LP = FLastArray->LastPacket();
        if (LP)
            return LP->OffsetAfterPacket();
        else
            return 0;
        }
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageReadIniFile() --
//---------------------------------------------------------------------------

void PacketFileDataSet::ImageReadIniFile(IniFile & /*Ini*/)
{
}

//---------------------------------------------------------------------------
// PacketFileDataSet::ImageWriteIniFile() --
//---------------------------------------------------------------------------

void PacketFileDataSet::ImageWriteIniFile(IniFile & /*Ini*/)
{
}

//---------------------------------------------------------------------------
// PacketFileDataSet::PidFromSubchannel() --
//---------------------------------------------------------------------------

int  PacketFileDataSet::PidFromSubchannel(int channel)
{
    //  Channels are assigned to PIDs in order of PID.
    //    if no data is found in a PID its not counted.
    int NthPid = channel/SubchannelsPerDevice();

    PidPacketMap::iterator  iter;
	for (iter = Pid.begin(); iter != Pid.end(); ++iter)
        {
        int pid = (*iter).first;
        PidPacketArray & PPA = (*iter).second;
        if (PPA.MaxSampleIdx!=0)
            {
            if (NthPid==0)
                return pid;
            --NthPid;
            }
        }
    //  We didn't find it, so return -1. (an invalid PID)
    return -1;

}
//---------------------------------------------------------------------------
// PacketFileDataSet::WorkingPid() -- Current PID
//---------------------------------------------------------------------------

PidPacketArray &  PacketFileDataSet::WorkingPid()
{
    return Pid[PidFromSubchannel(Subchannel())];
}

//---------------------------------------------------------------------------
// PacketFileDataSet::SubchannelsPerDevice() -- Return number of ch per device
//---------------------------------------------------------------------------

unsigned int PacketFileDataSet::SubchannelsPerDevice()
{
    return std::max<int>(Subchannels()/FDevices, 1);
}

//---------------------------------------------------------------------------
// PacketFileDataSet::PacketCount() -- Return number of packets in data set
//---------------------------------------------------------------------------

size_t PacketFileDataSet::PacketCount() const
{
    size_t result = 0;

	PidPacketMap::const_iterator  iter;
    for (iter = Pid.begin(); iter != Pid.end(); ++iter)
        result += iter->second.Packets.size();

    return result;
}

//---------------------------------------------------------------------------
// PacketFileDataSet::AlertPacketCount() -- Return number of alert packets
//                                                  in data set
//---------------------------------------------------------------------------

size_t PacketFileDataSet::AlertPacketCount() const
{
    size_t result = Alert_PPA.Packets.size();
    return result;
}

//------------------------------------------------------------------------------
// PacketFileDataSet::FillIncidentList() -- Get list of incidents in range
//------------------------------------------------------------------------------

void  PacketFileDataSet::FillIncidentList(ii64 StartIdx, ii64 EndIdx)
{
    Incidents.clear();

    //  Convert Idx to 'Data Channel Offset'
    ii64 StartOffset = LeastOffsetFromIdx(StartIdx);
    ii64 EndOffset   = MostOffsetFromIdx(EndIdx);

	// Process data packets
	for (PidPacketMap::iterator m = Pid.begin(); m != Pid.end(); ++m)
		{
		for (PidPacketArray::PacketArray::iterator i = m->second.Packets.begin(); i != m->second.Packets.end(); ++i)
			{
			PacketProxy & Packet = *i;
			if (Packet.Offset < StartOffset)
				continue;
			if (Packet.Offset > EndOffset)
				continue;

			//  In Range, make an Incident from it.
//			Packet.AllocateBody();
			DatasetIncident  Incident;

			double Fraction = 0;
			ii64 delta = EndOffset - StartOffset;
			if  (delta)
				Fraction = ((double)(Packet.Offset - StartOffset))/delta;
			ii64 EstimatedSampleIdx = static_cast<ii64>(StartIdx + (EndIdx - StartIdx) * Fraction);

			FillDataIncident(Incident, Packet, EstimatedSampleIdx);
			Incidents.push_back(Incident);
//			Packet.ReleaseBody();
			}
		}

	// Process alert packets
	for (unsigned int i=0; i<Alert_PPA.Packets.size(); ++i)
		{
		PacketProxy & Packet = Alert_PPA.Packets[i];
		if (Packet.Offset < StartOffset)
			continue;
		if (Packet.Offset > EndOffset)
			continue;

		//  In Range, make an Incident from it.
//		Packet.AllocateBody();
		DatasetIncident  Incident;

		double Fraction = ((double)(Packet.Offset - StartOffset))/(EndOffset - StartOffset);
		ii64 EstimatedSampleIdx = static_cast<ii64>(StartIdx + (EndIdx - StartIdx) * Fraction);

		FillAlertIncident(Incident, Packet, EstimatedSampleIdx);
		Incidents.push_back(Incident);
//		Packet.ReleaseBody();
		}

	SortIncidentList();
}

//------------------------------------------------------------------------------
// PacketFileDataSet::LeastOffsetFromIdx() -- Find smallest offset on all data chans
//------------------------------------------------------------------------------

ii64  PacketFileDataSet::LeastOffsetFromIdx(ii64 Idx)
{
	//  Convert Idx to 'File Offset' keeping smallest one
	ii64 FSampleIdxSave = FSampleIdx;

	ii64 FinalOffset = MaximumIndex64;

	for (PidPacketMap::iterator i = Pid.begin(); i != Pid.end(); ++i)
		{
		PidPacketArray & currentPid = i->second;
		FSampleIdx = Idx;

		ParseDataFile();       // make sure we have these packets in memory

		currentPid.FindProxyForSample(Idx);
		if (currentPid.IsFoundProxyValid()==false)    // if we found no proxy, we can't use it
			continue;
		PacketProxy & Proxy(currentPid.FoundProxy());

		if (Proxy.Offset < FinalOffset)
			FinalOffset = Proxy.Offset;
		}

	FSampleIdx = FSampleIdxSave;
	Progress(FSampleIdx);

	return FinalOffset;
}

//------------------------------------------------------------------------------
// PacketFileDataSet::MostOffsetFromIdx() -- Find largest offset on all data chans
//------------------------------------------------------------------------------

ii64  PacketFileDataSet::MostOffsetFromIdx(ii64 Idx)
{
    //  Convert Idx to 'File Offset' keeping largest one
	ii64 FSampleIdxSave = FSampleIdx;

	ii64 FinalOffset = 0;

	for (PidPacketMap::iterator i = Pid.begin(); i != Pid.end(); ++i)
		{
		PidPacketArray & currentPid = i->second;
		FSampleIdx = Idx;

		ParseDataFile();       // make sure we have these packets in memory

		currentPid.FindProxyForSample(Idx);
		if (currentPid.IsFoundProxyValid()==false)    // if we found no proxy, we can't use it
			continue;

		PacketProxy & Proxy(currentPid.FoundProxy());

		if (Proxy.Offset > FinalOffset)
			FinalOffset = Proxy.Offset;
		}

	FSampleIdx = FSampleIdxSave;
	Progress(FSampleIdx);
	return FinalOffset;
}

//------------------------------------------------------------------------------
// PacketFileDataSet::FillAlertIncident() -- Fill incident data
//------------------------------------------------------------------------------

void  PacketFileDataSet::FillAlertIncident(DatasetIncident & incident,
	const PacketProxy & Alert, ii64 EstimatedSampleIdx)
{
	//  Figure out alert #
	int AlertNum(0);
	for (int i=0; i<32; i++)
		if (Alert.Body.Data[0] == (1u<<i))
			{
			AlertNum = i;
			break;
			}

    for (unsigned int i = 0; i < 2; ++i)
        incident.Raw.push_back(Alert.Header.Data[i]);
    for (unsigned int i = 0; i < 28; ++i)
        incident.Raw.push_back(Alert.Body.Data[i]);
    incident.Argument = Alert.Body.Data[2 + AlertNum];
	incident.Seconds = Alert.Body.Data[1];  // Alert timestamp field
	incident.Offset = Alert.Offset;
	incident.Origin = incident.Offset*Devices()/EventSize();
	incident.RangedIncident = false;
	incident.Sample = EstimatedSampleIdx;
	incident.Type = DatasetIncident::Alert;
	incident.Size = Alert.Size;
	incident.Id = AlertNum;
	incident.Index = Alert.Index;
}

//------------------------------------------------------------------------------
// PacketFileDataSet::FillDataIncident() -- Fill incident data
//------------------------------------------------------------------------------

void  PacketFileDataSet::FillDataIncident(DatasetIncident & incident,
	const PacketProxy & Packet, ii64 EstimatedSampleIdx)
{
	incident.Seconds = 0;
	for (int i = 0; i < 2; ++i)
		incident.Raw.push_back(Packet.Header.Data[i]);
	incident.Offset = Packet.Offset;
	incident.Origin = incident.Offset/EventSize();
	incident.RangedIncident = false;
	incident.Sample = EstimatedSampleIdx;
	incident.Size = Packet.Header.PacketSize();
	incident.Id = Packet.Header.PeripheralId() ;
	incident.Type = DatasetIncident::Velocia;
	incident.Index = Packet.Index;
}


} // namespace Innovative


