// PacketInterruptScheme_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <PacketInterruptScheme_Mb.h>
#include <Buffer_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <StartStopThread_Mb.h>
#include <NotifyThread_Mb.h>
#include <sstream>
#include <HardwareRegister_Mb.h>
#include <SignalQueue_Mb.h>
#include <MemoryUtils_Mb.h>
#include <PacketImage_Mb.h>
#include <Osal_Mb.h>
#include <iostream>
#include <assert.h>



#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

namespace Innovative
{

namespace PacketIntScheme
{
//==============================================================================
//  CLASS PacketStreamDatagram  --  Special Copy Ops for BlockStream
//==============================================================================

class PacketStreamDatagram
{
public:
    PacketStreamDatagram(IVelociaBufferManager * mgr, Buffer & op)
        : FBuffer(op), Mgr(mgr)
        {}

	void  CopyBodyFromBmAt( ii32 * buf, size_t my_index, size_t size);
	void  CopyHeaderFromBmAt( ii32 * buf, size_t my_index, size_t size);

private:
    Buffer &  FBuffer;
    IVelociaBufferManager * Mgr;
};

//---------------------------------------------------------------------------
//  PacketStreamDatagram::CopyBodyFromBmAt() --
//---------------------------------------------------------------------------

void  PacketStreamDatagram::CopyBodyFromBmAt( ii32 * buf, size_t my_index, size_t size)
{
    // Body
    ii32 * TargetDataPtr = reinterpret_cast<ii32*>(FBuffer.Data());

    CopyInt64Buffer(reinterpret_cast<const ii64*>(buf),
        reinterpret_cast<ii64*>(TargetDataPtr + my_index),
        static_cast<int>(size * sizeof(ii32) / sizeof(ii64)));
}

//---------------------------------------------------------------------------
//  PacketStreamDatagram::CopyHeaderFromBmAt() --
//---------------------------------------------------------------------------

void  PacketStreamDatagram::CopyHeaderFromBmAt( ii32 * buf, size_t my_index, size_t size)
{
    // Header
    ii32*    TargetDataPtr = (ii32*)FBuffer.Header();

    CopyInt64Buffer(reinterpret_cast<const ii64*>(buf),
        reinterpret_cast<ii64*>(TargetDataPtr + my_index),
        static_cast<int>(size * sizeof(ii32) / sizeof(ii64)));

    size_t sz = Mgr->PacketSizeFromHeaderInts(FBuffer) - Mgr->HeaderSizeInts();
    FBuffer.Resize(sz);
}

//==============================================================================
//  CLASS ConstPacketStreamDatagram  --  Special Copy Ops for BlockStream
//==============================================================================

class ConstPacketStreamDatagram
{
public:
    ConstPacketStreamDatagram(const Buffer & op) : FBuffer(op)
        {}

	void  CopyBodyToBmAt( ii32 * buf, size_t my_index, size_t size) const;
	void  CopyHeaderToBmAt( ii32 * buf, size_t my_index, size_t size) const;

private:
    const Buffer &  FBuffer;
};

//---------------------------------------------------------------------------
//  ConstPacketStreamDatagram::CopyBodyToBmAt() --
//---------------------------------------------------------------------------

void  ConstPacketStreamDatagram::CopyBodyToBmAt(ii32 * buf, size_t my_index, size_t size) const
{
    const ii32 * SourceDataPtr = reinterpret_cast<const ii32*>(FBuffer.ConstData());

    CopyInt64Buffer(reinterpret_cast<const ii64*>(SourceDataPtr + my_index),
        reinterpret_cast<ii64*>(buf),
        static_cast<unsigned int>(size * sizeof(ii32) / sizeof(ii64)));
}

//---------------------------------------------------------------------------
//  ConstPacketStreamDatagram::CopyHeaderToBmAt() --
//---------------------------------------------------------------------------

void  ConstPacketStreamDatagram::CopyHeaderToBmAt( ii32 * buf, size_t my_index, size_t size) const
{
    const ii32*    SourceDataPtr = (const ii32*)FBuffer.ConstHeader();

    CopyInt64Buffer(reinterpret_cast<const ii64*>(SourceDataPtr + my_index),
        reinterpret_cast<ii64*>(buf),
        static_cast<unsigned int>(size * sizeof(ii32) / sizeof(ii64)));

}

//===========================================================================
//  CLASS BmMemoryRing  -- Busmaster Region Overlay class
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of class BmMemoryRing
//---------------------------------------------------------------------------

class BmMemoryRing
{
public:
    BmMemoryRing(ManagerInternalsSafeLink & parentLink);

    void  Configure(ii32 * base_address, int size_words);
    void  Clear();
    void  FillRegion(unsigned int pattern, bool ramp);

    size_t Depth();
    size_t Room();
    size_t Size();
    ii32 * BaseAddr() {  return Base;  }

    void  AdvanceInsertion(size_t packets);
    void  AdvanceExtraction(size_t packets);
    void  AdvanceInsertionByWords(size_t words);
    void  AdvanceExtractionByWords(size_t words);

    size_t NextInsertionPacketSize()
        {  return NextPacketSize(InsertionIdx);  };
    size_t NextExtractionPacketSize()
        {  return NextPacketSize(ExtractionIdx);  };
    size_t NextInsertionPacketPid()
        {  return NextPacketPid(InsertionIdx);  };
    size_t NextExtractionPacketPid()
        {  return NextPacketPid(ExtractionIdx);  };

    void  SetPacketPtr (BmPacket &pkt, size_t s, bool advIndx);
    void  CopyTo(Buffer & buffer);
    void  CopyFrom(const Buffer & buffer);

    void  ImageTo(PacketImage & Image);
    void  ImageFrom(PacketImage & Image);

    size_t      ReadInsertionIdx() { return InsertionIdx; }
    size_t      ReadExtractionIdx() { return ExtractionIdx; }

private:
    ManagerInternalsSafeLinkAlias   SM_Internals;

    size_t          InsertionIdx;
    size_t          ExtractionIdx;
    ii32 *          Base;
    size_t          RingSize;

    size_t          NextPacketSize(size_t packet_start_idx);
    short           NextPacketPid(size_t packet_start_idx);

};

//---------------------------------------------------------------------------
//  constructor of class BmMemoryRing
//---------------------------------------------------------------------------

BmMemoryRing::BmMemoryRing(ManagerInternalsSafeLink & parentLink)
    :  SM_Internals(parentLink), InsertionIdx(0), ExtractionIdx(0),
        Base(0), RingSize(0)
{
}

//---------------------------------------------------------------------------
//  BmMemoryRing::Configure() -- Set base ptr and size
//---------------------------------------------------------------------------

void  BmMemoryRing::Configure(ii32 * base_address, int size_words)
{
    Base = base_address;
    RingSize = size_words;
}

//---------------------------------------------------------------------------
//  BmMemoryRing::Clear() -- Reset position indexes
//---------------------------------------------------------------------------

void  BmMemoryRing::Clear()
{
    InsertionIdx = 0;
    ExtractionIdx = 0;
}

//---------------------------------------------------------------------------
//  BmMemoryRing::FillRegion() -- Fill region with pattern
//---------------------------------------------------------------------------

void  BmMemoryRing::FillRegion(unsigned int pattern, bool ramp)
{
    if (Base)
        for (unsigned int i=0; i<RingSize; i++)
            {
            Base[i] = pattern;
            if (ramp)
                ++pattern;
            }
}

//---------------------------------------------------------------------------
//  BmMemoryRing::AdvanceInsertion() -- Advance Insertion Idx over region
//---------------------------------------------------------------------------

void  BmMemoryRing::AdvanceInsertion(size_t packets)
{
    //  This function will advance the pointer over a data packet already
    //    copied into the memory region, using the header information to
    //    determine the distance to advance.
    for (size_t i=0; i<packets; i++)
        {
        size_t buffersize = NextPacketSize(InsertionIdx);
        AdvanceInsertionByWords(buffersize);
        }
}

//---------------------------------------------------------------------------
//  BmMemoryRing::AdvanceInsertionByWords() -- Advance Insertion Idx
//---------------------------------------------------------------------------

void  BmMemoryRing::AdvanceInsertionByWords(size_t words)
{
    InsertionIdx += words;
    while (InsertionIdx >= RingSize)  // should only wrap once, but lets be sure
        InsertionIdx -= RingSize;
}

//---------------------------------------------------------------------------
//  BmMemoryRing::AdvanceExtraction() -- Advance Extraction Idx over region
//---------------------------------------------------------------------------

void  BmMemoryRing::AdvanceExtraction(size_t packets)
{
    //  This function will advance the pointer over a data packet already
    //    copied into the memory region, using the header information to
    //    determine the distance to advance.
    for (size_t i=0; i<packets; i++)
        {
        size_t buffersize = NextPacketSize(ExtractionIdx);
        AdvanceExtractionByWords(buffersize);
        }
}

//---------------------------------------------------------------------------
//  BmMemoryRing::AdvanceExtractionByWords() -- Advance Extraction Idx
//---------------------------------------------------------------------------

void  BmMemoryRing::AdvanceExtractionByWords(size_t words)
{
    ExtractionIdx += words;
    while (ExtractionIdx >= RingSize)  // should only wrap once, but lets be sure
        ExtractionIdx -= RingSize;
}

//---------------------------------------------------------------------------
//  BmMemoryRing::Depth() -- Amount of filled space in queue
//---------------------------------------------------------------------------

size_t  BmMemoryRing::Depth()
{
    size_t Iidx = InsertionIdx;
    size_t Eidx = ExtractionIdx;
    if (Iidx>=Eidx)
        return InsertionIdx - ExtractionIdx;
    else
        return InsertionIdx - ExtractionIdx + RingSize;
}

//---------------------------------------------------------------------------
//  BmMemoryRing::Room() -- Amount of empty space in queue
//---------------------------------------------------------------------------

size_t  BmMemoryRing::Room()
{
    return RingSize - 1 - Depth();
}

//---------------------------------------------------------------------------
//  BmMemoryRing::Size() -- Total allocated space in queue
//---------------------------------------------------------------------------

size_t  BmMemoryRing::Size()
{
    return RingSize;
}

//---------------------------------------------------------------------------
//  BmMemoryRing::NextPacketSize() -- Read the next header for packet size
//---------------------------------------------------------------------------

size_t  BmMemoryRing::NextPacketSize(size_t packet_start_idx)
{
    return SM_Internals->VelociaManager()->PacketSizeFromMemoryInts(Base + packet_start_idx);
}

//---------------------------------------------------------------------------
//  BmMemoryRing::NextPacketPid() -- Read the next header for packet size
//---------------------------------------------------------------------------

short  BmMemoryRing::NextPacketPid(size_t packet_start_idx)
{
    return SM_Internals->VelociaManager()->PeripheralIdFromMemory(Base + packet_start_idx);
}

//---------------------------------------------------------------------------
//  BmMemoryRing::SetPacketPtr() -- Set pointers to packets in BM ring
//---------------------------------------------------------------------------

void BmMemoryRing::SetPacketPtr (BmPacket & pkt, size_t s, bool advIndx=false)
{
    pkt.TopHalf.Pointer = Base+ExtractionIdx;

    //does packet wraps around?
    size_t words_before_end = RingSize - ExtractionIdx;
    if(s <= words_before_end)
    {
        //no
        pkt.TopHalf.SizeInInts = s;
        pkt.BottomHalf.Pointer = NULL;
        pkt.BottomHalf.SizeInInts = 0;
    }
    else
    {
        //yes
        pkt.TopHalf.SizeInInts = words_before_end;
        pkt.BottomHalf.Pointer = Base;
        pkt.BottomHalf.SizeInInts = s - words_before_end;
    }

    if(advIndx)
        AdvanceExtractionByWords(s);
}

//---------------------------------------------------------------------------
//  BmMemoryRing::CopyTo() -- Copy to data buffer from Ring
//---------------------------------------------------------------------------

void  BmMemoryRing::CopyTo(Buffer & buffer)
{
    // Header
    size_t buffer_idx = 0;
    size_t transfer_size = buffer.HeaderSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = RingSize - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);
        PacketStreamDatagram bufferDG(SM_Internals->VelociaManager(), buffer);
        bufferDG.CopyHeaderFromBmAt(Base+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        AdvanceExtractionByWords(size);
        }

    // Body
    buffer_idx = 0;
    transfer_size = buffer.SizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = RingSize - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);
        PacketStreamDatagram bufferDG(SM_Internals->VelociaManager(), buffer);
        bufferDG.CopyBodyFromBmAt(Base+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        AdvanceExtractionByWords(size);
        }
}

//---------------------------------------------------------------------------
//  BmMemoryRing::CopyFrom() -- Copy to Ring from data buffer
//---------------------------------------------------------------------------

void  BmMemoryRing::CopyFrom(const Buffer & buffer)
{
    // Header
    size_t buffer_idx = 0;
    size_t transfer_size = buffer.HeaderSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = RingSize - InsertionIdx;
        size_t size = std::min(words_before_end, transfer_size);
        ConstPacketStreamDatagram bufferDG(buffer);
        bufferDG.CopyHeaderToBmAt(Base+InsertionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        AdvanceInsertionByWords(size);
        }

    // Body
    buffer_idx = 0;
    transfer_size = buffer.SizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = RingSize - InsertionIdx;
        size_t size = std::min(words_before_end, transfer_size);
        ConstPacketStreamDatagram bufferDG(buffer);
        bufferDG.CopyBodyToBmAt(Base+InsertionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        AdvanceInsertionByWords(size);
        }
}

//---------------------------------------------------------------------------
//  BmMemoryRing::ImageTo() -- Copy location information to PacketImage object
//---------------------------------------------------------------------------

void  BmMemoryRing::ImageTo(PacketImage & Image)
{
    Image.ClearRanges();

    size_t transfer_size = Image.FullSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = RingSize - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);
        //  Create range object for the next block of the packet. May be several due to wrapping
        PacketImage::Range range(Base+ExtractionIdx, size);
        Image.AddRange( range );

        transfer_size -= size;
        AdvanceExtractionByWords(size);
        }

}

//------------------------------------------------------------------------------
//  BmMemoryRing::ImageFrom() -- Copy 'space' information to PacketImage object
//------------------------------------------------------------------------------
//  This information is that of the 'hole' that can be filled with data

void  BmMemoryRing::ImageFrom(PacketImage & Image)
{
    Image.ClearRanges();

    size_t transfer_size = Image.FullSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = RingSize - InsertionIdx;
        size_t size = std::min(words_before_end, transfer_size);
        //  Create range object for the next block of the packet. May be several due to wrapping
        PacketImage::Range range(Base+InsertionIdx, size);
        Image.AddRange( range );

        transfer_size -= size;
        AdvanceInsertionByWords(size);
        }
}

//===========================================================================
//  CLASS BmXmitThread  -- Transmission sync handling for PacketInt scheme
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class BmXmitThread
//---------------------------------------------------------------------------

class BmXmitThread : public NotifySemThread
{
    typedef NotifySemThread inherited;

public:
    BmXmitThread(const char * name = "")
        : inherited(name)
        {  Priority(tpHigher);  }
};

//===========================================================================
//  CLASS VirtualIsrThread  -- ISR handling thread for Packet Int scheme
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class VirtualIsrThread
//---------------------------------------------------------------------------

class VirtualIsrThread : public StartStopThread
{
    typedef StartStopThread inherited;
       
    //
    //  Class InterruptStatusRegister
    class InterruptStatusRegister : public Register
    {
    typedef Register inherited;

    public:
        RegisterBit       BmRecvInt;
        RegisterBit       BmTransmitInt;
        RegisterBit       CustomInt;
        RegisterBitGroup  XmitPacketDepth;
        RegisterBitGroup  RecvPacketDepth;
        RegisterBitGroup  CustomIntFlags;

    public:
        InterruptStatusRegister( AddressingSpace &space, int offset );
    };

public:
    VirtualIsrThread( IPacket_DeviceToScheme * device );

    void  Clear();

    bool  TransferXmitPacket();
    void  ConfigureRecvRing(ii32 * base_address, int size_words)
            {  RecvRing.Configure(base_address, size_words);  }
    void  ConfigureXmitRing(ii32 * base_address, int size_words)
            {  XmitRing.Configure(base_address, size_words);  }
    void  FillRecvRing(unsigned int pattern, bool ramp)
            {  RecvRing.FillRegion(pattern, ramp);  }
    void  FillXmitRing(unsigned int pattern, bool ramp)
            {  XmitRing.FillRegion(pattern, ramp);  }
    void  PipInit()
            {  IncomingPip.LoadManager(StreamManagerInternals->VelociaManager());  }

    size_t  IsrQDepth()
            {  return LocalIsrQ.depth();  }
    //
    //  Events
    OpenWire::EventHandler<OpenWire::Event> OnXmitData;

protected:
    enum { sDriverEvent=inherited::sLast, sLocalIsrQ, sLast };

    Event &                    DriverEvent;
    InterruptStatusRegister    InterruptStatusImage;
    AddressingSpace            IsrDecode;
    BmMemoryRing               RecvRing;
    BmMemoryRing               XmitRing;

    ManagerInternalsSafeLinkAlias   StreamManagerInternals;

	IPacket_DeviceToScheme *        StreamDevice;
    SignalQueue<unsigned int>       LocalIsrQ;
	BasicPacketImageParser          IncomingPip;

    size_t  MaxInterruptDepth;
    size_t  MaxLocalIsrDepth;

    virtual void Init();
    virtual void SingleExecution();

    void  DispatchInterrupt( int IntStatus );
    void  ProcessRecvInterrupt( int packet_depth );
    void  ProcessXmitInterrupt( int packet_depth );
    void  ProcessCustomInterrupt( int custom_flags );
    bool  StreamStarted();
    void  DirectModePipProcessing(PacketImageParserImageAvailable & event);

public:
    virtual void XXX_DebugLog(int type, int code)
        {  StreamDevice->Log(type, code);  }


};

//---------------------------------------------------------------------------
//  constructor for class VirtualIsrThread::InterruptStatusRegister
//---------------------------------------------------------------------------

VirtualIsrThread::InterruptStatusRegister::InterruptStatusRegister( AddressingSpace &space, int offset ) :
  inherited( space, offset ),
  BmRecvInt(                *this, 0 ),
  BmTransmitInt(            *this, 1 ),    
  CustomInt(                *this, 2 ),
  XmitPacketDepth(          *this, 4, 6 ),
  RecvPacketDepth(          *this, 10, 6 ),
  CustomIntFlags(           *this, 16, 8 )
{
}

//---------------------------------------------------------------------------
//  housekeeping for class VirtualIsrThread
//---------------------------------------------------------------------------

const int DOMode_InFractionValue = 4;    // Process input data when this fraction is ready
const int DOMode_OutFractionValue = 4;   // Request more output data when this fraction is available

//---------------------------------------------------------------------------
//  constructor for class VirtualIsrThread
//---------------------------------------------------------------------------

VirtualIsrThread::VirtualIsrThread(IPacket_DeviceToScheme * device)
    : inherited("VirtualIsrThread"),
      DriverEvent( *( device->IsrEvent() ) ),
      InterruptStatusImage( IsrDecode, 0 ),
      RecvRing(device->Link()),
      XmitRing(device->Link()),
      StreamManagerInternals(device->Link()),
      StreamDevice( device ),
      IncomingPip( device->Link()->VelociaManager()),  // 'Safe' object OK.
      MaxInterruptDepth(0), MaxLocalIsrDepth(0)
{
    Priority(tpHigher);

    IncomingPip.OnImageAvailable.SetEvent(this, &VirtualIsrThread::DirectModePipProcessing);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::Init() -- Initialize sync objects
//---------------------------------------------------------------------------

void VirtualIsrThread::Init()
{
    inherited::Init();

    AddObject(DriverEvent, sDriverEvent);
    AddObject(LocalIsrQ.SignalSem(), sLocalIsrQ);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::StreamStarted() -- returns true if the streaming is started
//---------------------------------------------------------------------------

bool  VirtualIsrThread::StreamStarted()
{
    return StreamManagerInternals->HasStreamStarted();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::Clear() -- Stream Start Initialization Method
//---------------------------------------------------------------------------

void  VirtualIsrThread::Clear()
{
    RecvRing.Clear();
    XmitRing.Clear();

    IncomingPip.Clear();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::SingleExecution() -- thread function
//---------------------------------------------------------------------------

void VirtualIsrThread::SingleExecution()
{
    //assert (StreamDevice);
    int result = WaitFor();

    if ( Terminated() || eStopped.Check() )
        {
        return;
        }

    //
    //  A new interrupt was signalled. Since the low level queue is of
    //     finite size, we drain that queue as fast as we can. Since we
    //     also got at least one interrupt, we process one now.
    if ( result == sDriverEvent )
    {
        size_t depth = StreamDevice->InterruptDepth();
        StreamDevice->Log(0x94, static_cast<int>(depth));
        MaxInterruptDepth = std::max( depth, MaxInterruptDepth );

        if (MaxInterruptDepth > 100)
        {
            StreamDevice->Log(0x9F, MaxInterruptDepth);

        }
                   
        if( ! StreamStarted() )
            LocalIsrQ.clear();
        
        while( depth-- )
        {
            if( ! StreamStarted() )
                break;

            LocalIsrQ.push( StreamDevice->InterruptAck());
        }
            
        size_t Idepth = LocalIsrQ.depth();
        StreamDevice->Log(0x95, Idepth);
        MaxLocalIsrDepth = std::max( Idepth, MaxLocalIsrDepth );
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            //  since we did not wait for Sem before, must decrement sem manually
            //    when popping IRQ register value
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop(SignalQueue<unsigned int>::PopSem);
            DispatchInterrupt( IrqToProcess );
            }
        }
    //
    //  We were signalled because of queued local ISRs.
    if ( result == sLocalIsrQ )
        {
        size_t Idepth = LocalIsrQ.depth();
        StreamDevice->Log(0x9a, Idepth);

        //  If we have stopped, clear out the ISR Q now
        if( ! StreamStarted() )
            LocalIsrQ.clear();
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop();  // don't dec sem, already done
            DispatchInterrupt( IrqToProcess );
            }
        }
    StreamDevice->Log(0x96, LocalIsrQ.depth());
    StreamDevice->Log(0x97, (int)LocalIsrQ.SignalSem().Count());
    //
    //  Periodic Update call to BM Manager
    StreamDevice->BmMgr().PeriodicUpdate();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::DispatchInterrupt() -- process interrupt
//---------------------------------------------------------------------------

void VirtualIsrThread::DispatchInterrupt( int IntStatus )
{
    StreamDevice->Log(4, IntStatus);
//    DeltaT_Log(0, 0x02, IntStatus);
    //
    //  Overlay Status Register Decode onto IntStatus variable
    IsrDecode.Link( ( ii32 *)&IntStatus );

    if (InterruptStatusImage.BmRecvInt.Value())
        {
        int PacketDepth = InterruptStatusImage.RecvPacketDepth.Value();
        ProcessRecvInterrupt(PacketDepth);
        }
    if (InterruptStatusImage.BmTransmitInt.Value())
        {
        int PacketDepth = InterruptStatusImage.XmitPacketDepth.Value();
        ProcessXmitInterrupt(PacketDepth);
        }
    if (InterruptStatusImage.CustomInt.Value())
        {
        int CustomFlags = InterruptStatusImage.CustomIntFlags.Value();
        ProcessCustomInterrupt(CustomFlags);
        }
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessRecvInterrupt() -- Recv block
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessRecvInterrupt( int PacketDepth )
{
    //assert(PacketDepth);
    BmPacket Packet;


    //  Mark Queue as having data
    RecvRing.AdvanceInsertion(PacketDepth);

    //  Now we have 3 modes of packet processing:
    //     Normal: copies packets out of BM
    //     Direct Output: sends images of packet in BM buffer out to application for direct logging or processing
    //     Direct Packet: actively manage BM and credit, but no packet copy
    //
    switch (StreamManagerInternals->PacketProcessingMode())
    {
        case ppmDirectPacket:
            //  Process packets in busmaster memory until queue is empty
            while (RecvRing.Depth())
            {
                //set packet pointers
                RecvRing.SetPacketPtr(Packet, RecvRing.NextExtractionPacketSize());

                if (!StreamDevice->AlertTest(Packet.PeripheralID()))
                {
                    //data
                    RecvRing.AdvanceExtractionByWords(RecvRing.NextExtractionPacketSize());

                    //notify application
                    StreamManagerInternals->ForwardDirectDataPacket(Packet);
                }
                else
                {
                    //alert
                    //  create a properly typed buffer (CopyTo will size it later)
                    Buffer buffer(StreamManagerInternals->VelociaManager()->NativeBuffer());

                    //  copy data from ring into packet
                    RecvRing.CopyTo( buffer );

                    //  Perform Alert Processing on Buffer
                    StreamDevice->AlertCheck(buffer);

                    //notify application
                    StreamManagerInternals->ForwardDirectAlertPacket(Packet);
                }
            }
        break;

        case ppmNormal:

            //  Move packets out of busmaster memory until queue is empty
            while (RecvRing.Depth())
            {
                if( ! StreamStarted() )
                    break;
                StreamDevice->Log(0x71, RecvRing.Depth());

                //  get size of next packet (32-bit words)
                size_t PacketSize = RecvRing.NextExtractionPacketSize();
                StreamDevice->Log(0x72, PacketSize);
                short PacketPid = static_cast<short>(RecvRing.NextExtractionPacketPid());
                StreamDevice->Log(0x73, PacketPid);

                if (PacketSize >= 0x20000)
                    {
                    //  Debug..."Leak" a too large packet
                    StreamDevice->Log(0x7F, PacketSize);
                    //  Optional...hang here for debugging...
                    //while (1==1)
                    //    ;
                    //continue;
                    }

                    {
                    // convert pointer to offset
                    size_t offset = StreamDevice->BmMgr().IncomingPhysicalMemOffset( StreamDevice->DebugPortPeek(0x16800) );

                    StreamDevice->Log(0x7a, RecvRing.ReadExtractionIdx());
                    StreamDevice->Log(0x7b, (int)offset);
                    StreamDevice->Log(0x7c, (int)StreamDevice->DebugPortPeek(0x17c00));  // credit
                    }

                //  create a properly typed buffer (CopyTo will size it later)
                Buffer Packet(StreamManagerInternals->VelociaManager()->NativeBuffer());

                //  copy data from ring into packet
                RecvRing.CopyTo( Packet );

                    {
                        //  DEBUG: get size of next packet after next packet
                        size_t nexterPacketSize = RecvRing.NextExtractionPacketSize();
                        StreamDevice->Log(0x7d, nexterPacketSize);
                    }

                //
                //  Enqueue packet and grant credit...
                StreamManagerInternals->EnqueueRecvPacket(Packet);
            }
         break;

        case ppmDirectOutput:

                //  Delay all processing until we have a nice slab ready to go
                if (RecvRing.Depth() > RecvRing.Size()/DOMode_InFractionValue)
                {
                    //  Now that we have a slab, lets notify the application
                    UnformattedDataImage Slab(StreamManagerInternals->VelociaManager(),
                                              RecvRing.Size()/DOMode_OutFractionValue);
                    //  ...copy image information to image
                    RecvRing.ImageTo( Slab.Image );
                    //
                    //  Alert processing..we need to parse for packets
                    //    the handler processes alerts, and no credit is granted.
                    IncomingPip.Append(Slab.Image);
                    IncomingPip.Parse();
                    //
                    //  Process Image and grant credit...
                    StreamManagerInternals->ProcessDataSlab(Slab);
                }
         break;
    }
}

//------------------------------------------------------------------------------
//  VirtualIsrThread::DirectModePipProcessing() -- Recv bock
//------------------------------------------------------------------------------
//  Processes "Packet Image" parsing of input blocks in DirectOut mode
//    we use this to seek out the alerts and process them

void  VirtualIsrThread::DirectModePipProcessing(PacketImageParserImageAvailable & event)
{
    //  Check if we have an alert object. If so we need to make a Packet
    //    from the image for Alert Processing
    if (StreamDevice->AlertTest(event.Image.PeripheralId()))
        {
		Buffer buffer(StreamManagerInternals->VelociaManager()->NativeBuffer());
		//  Make a Buffer from the Image
        event.Image.CopyDataTo(buffer);
        //  Perform Alert Processing on Buffer
        StreamDevice->AlertCheck(buffer);
        }
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessXmitInterrupt() -- Send block
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessXmitInterrupt( int PacketDepth )
{
    //  Mark Queue as having data
    XmitRing.AdvanceExtraction(PacketDepth);
    // convert pointer to offset
    size_t offset = StreamDevice->BmMgr().OutgoingPhysicalMemOffset( StreamDevice->DebugPortPeek(0x16400) );
    StreamDevice->Log(0x56, XmitRing.ReadExtractionIdx());
    StreamDevice->Log(0x57, (int)offset);
    StreamDevice->Log(0x58, (int)StreamDevice->DebugPortPeek(0x17800));  // credit

    OpenWire::Event e;
    OnXmitData.Execute(e);
}


//---------------------------------------------------------------------------
//  VirtualIsrThread::TransferXmitPacket() -- Push a packet to BM Region
//---------------------------------------------------------------------------

bool  VirtualIsrThread::TransferXmitPacket( )
{
    //  Now we have 3 modes of packet processing:
    if (StreamManagerInternals->PacketProcessingMode() == ppmNormal ||
        StreamManagerInternals->PacketProcessingMode() == ppmDirectPacket)
		{
        StreamDevice->Log(0x50, 0);

        //  Do we have room to insert the packet?
        if (StreamManagerInternals->XmitQDepth()!=0)
			{
			size_t NextPacketSize = StreamManagerInternals->NextXmitPacket().FullSizeInInts();
            if (NextPacketSize <= XmitRing.Room())
				{
                //  Copy data into ring from packet
                XmitRing.CopyFrom( StreamManagerInternals->NextXmitPacket() );

                //  Remove data from queue and grant credit
                StreamManagerInternals->DequeueXmitPacket();
				
				{
				// convert pointer to offset
				size_t offset = StreamDevice->BmMgr().OutgoingPhysicalMemOffset( StreamDevice->DebugPortPeek(0x16400) );

				StreamDevice->Log(0x5c, XmitRing.ReadInsertionIdx());
				StreamDevice->Log(0x5d, (int)offset);
				StreamDevice->Log(0x5e, (int)StreamDevice->DebugPortPeek(0x17800));  // credit
				}

                StreamDevice->Log(0x5a, NextPacketSize);
                StreamDevice->Log(0x5b, XmitRing.Room());
                StreamDevice->Log(0x5f, 0);
				}
            else
				{
                //  ...if not, return false

                StreamDevice->Log(0x5a, NextPacketSize);
                StreamDevice->Log(0x5b, XmitRing.Room());
                StreamDevice->Log(0x5f, 0xFF);
                return false;
				}


        }
        else
        {
            //  We have no blocks, so signal Xmit Q to demand more blocks from application
            StreamManagerInternals->XmitQSignal(2);
            StreamDevice->Log(0x5f, 0xFA);
            return false;
        }

    }
    else // if (PacketProcessingMode == ppmDirectOutput)
    {
        StreamDevice->Log(0x50, 1);
        if (XmitRing.Room() > XmitRing.Size()/DOMode_OutFractionValue)
            {
            //  With room available for filling, generate a request to fill 'er up.
            //  ...create Packet Image object to hold data location information
            UnformattedDataImage Slab(StreamManagerInternals->VelociaManager(),
                                      XmitRing.Size()/DOMode_OutFractionValue);
            //  ...copy image information to image
            XmitRing.ImageFrom( Slab.Image );
            //
            //  Process Image and grant credit...
            StreamManagerInternals->FillDataSlab(Slab);
            }
        StreamDevice->Log(0x5f, 1);
    }

    return true;
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::ProcessCustomInterrupt() -- custom interrupt dispatch
//---------------------------------------------------------------------------

void  VirtualIsrThread::ProcessCustomInterrupt( int CustomFlags )
{
    //  Dispatch to device for special cracking in library
    StreamDevice->DispatchCustomInterrupt( CustomFlags );

    //  Dispatch to stream manager for user-defined cracking
    StreamManagerInternals->DispatchCustomInterrupt( CustomFlags );
}


}  // end namespace Innovative::PacketIntScheme

//===========================================================================
//  CLASS PacketInterruptScheme  -- ISR & Busmaster region handling
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PacketInterruptScheme
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class PacketInterruptScheme
//---------------------------------------------------------------------------

PacketInterruptScheme::PacketInterruptScheme(IPacket_DeviceToScheme * device)
    :  Visr(0), BmX(0), StreamManagerInternals(device->Link()), StreamDevice(device)
{
}

//---------------------------------------------------------------------------
//  destructor for class PacketInterruptScheme
//---------------------------------------------------------------------------

PacketInterruptScheme::~PacketInterruptScheme()
{
    Close();

	delete StreamDevice;
	StreamDevice = 0;
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::Open()
//---------------------------------------------------------------------------

void  PacketInterruptScheme::Open()
{
    //
    //  Create Busmaster Xmit thread.
    BmX = new PacketIntScheme::BmXmitThread("PacketIntScheme::BmXmitThread");
    BmX->OnNotify.SetEvent( this, &PacketInterruptScheme::BmXmitSignalled );
    BmX->Resume();

    //
    //  Create V-ISR thread.
    Visr = new PacketIntScheme::VirtualIsrThread(StreamDevice);
    Visr->OnXmitData.SetEvent( this, &PacketInterruptScheme::LaunchXmitData );

    ConfigureBmRegion();
    Visr->Resume();
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::Close()
//---------------------------------------------------------------------------

void  PacketInterruptScheme::Close()
{
    if ( BmX )
        {
        BmX->Stop();
        BmX->Shutdown();
        delete BmX;
        BmX = NULL;
        }
    //
    //  Have VISR Thread shut itself down
    if ( Visr )
        {
        Visr->Stop();
        Visr->Shutdown();

        delete Visr;
        Visr = NULL;
        }
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::ConfigureBmRegion()
//---------------------------------------------------------------------------

void PacketInterruptScheme::ConfigureBmRegion()
{
    //  ...input
    //  Get busmaster position & size
    ii32 * IncBmAddr = StreamDevice->BmMgr().IncomingBusMasterAddress();
    ii32   IncRingSizeWords = StreamDevice->IncomingBusMasterSizeWords();

    //  Config hardware and sw ring image
    StreamDevice->BmMgr().ReapplyDeviceInfo();

    if (Visr)
        {
        Visr->ConfigureRecvRing(IncBmAddr, IncRingSizeWords );
        }

    //  ...output
    //  Get busmaster position & size
    ii32 * OutgBmAddr = StreamDevice->BmMgr().OutgoingBusMasterAddress();
    ii32   OutgRingSizeWords = StreamDevice->OutgoingBusMasterSizeWords();
    if (Visr)
        {
        Visr->ConfigureXmitRing(OutgBmAddr, OutgRingSizeWords );
        }

    RecvRgnBasePhys = StreamDevice->BmMgr().IncomingPhysicalAddress();
    RecvRgnBase     = IncBmAddr;
    XmitRgnBasePhys = StreamDevice->BmMgr().OutgoingPhysicalAddress();
    XmitRgnBase     = OutgBmAddr;

    XmitRgnSize = OutgRingSizeWords;
    RecvRgnSize = IncRingSizeWords;

}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::ConfigureBusmastering()
//---------------------------------------------------------------------------

void  PacketInterruptScheme::ConfigureBusmastering()
{

	//  ...Clear the VISR storage
	if (Visr)
		{
		Visr->Clear();
		}
	//  ...set up memory regions, sizes
	ConfigureBmRegion();
	//
	//  Start V-ISR thread.
	if (Visr)
		{
		Visr->PipInit();
		Visr->Start();
		}
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::FillBmRegion() -- fill bm region with pattern
//---------------------------------------------------------------------------

void  PacketInterruptScheme::FillBmRegion(Dir which, unsigned int pattern,
                                          bool ramp)
{
    switch (which)
        {
        case BmXmit:
            Visr->FillXmitRing(pattern, ramp);
            break;
        case BmRecv:
            Visr->FillRecvRing(pattern, ramp);
            break;
        }
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::PreloadXmitBusmasterRegion()
//---------------------------------------------------------------------------

void  PacketInterruptScheme::PreloadXmitBusmasterRegion()
{
    if (StreamManagerInternals->PacketProcessingMode() == ppmNormal)
        TransmitDataAvailable();        //  Signal 1 buffer
    else // if (StreamManagerInternals->PacketProcessingMode() == ppmDirectOutput)
        {
        for (int i=0; i<PacketIntScheme::DOMode_OutFractionValue-1; i++) // signal N-1 buffers
            TransmitDataAvailable();    
		}
    //  Start Xfer thread
    BmX->Start();
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::TransmitDataAvailable() -- Signal Xmit thread to process
//---------------------------------------------------------------------------

void  PacketInterruptScheme::TransmitDataAvailable()
{
    StreamDevice->Log(0x30,0);
    BmX->Notify();
}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::LaunchXmitData() -- Signal Xmit thread to process
//---------------------------------------------------------------------------

void  PacketInterruptScheme::LaunchXmitData( OpenWire::Event & )
{
    StreamDevice->Log(0x31,0);
    BmX->Notify();
}


//---------------------------------------------------------------------------
//  PacketInterruptScheme::BmXmitSignalled() -- Transmission requested...
//------------------------------------------------------------------------------

void  PacketInterruptScheme::BmXmitSignalled( OpenWire::Event & )
{
    size_t depth = StreamManagerInternals->XmitQDepth();
    StreamDevice->Log(0x32,depth);
        
    Visr->TransferXmitPacket();

    depth = StreamManagerInternals->XmitQDepth();
    StreamDevice->Log(0x33,depth);

}

//---------------------------------------------------------------------------
//  PacketInterruptScheme::VisrIsrQDepth() -- return depth of VISR to-do Q
//---------------------------------------------------------------------------

size_t  PacketInterruptScheme::VisrIsrQDepth()
{
    if (Visr)
        return Visr->IsrQDepth();
    else
        return 0;
}



//===========================================================================
//  CLASS PacketInterruptRegMixin::InterruptEnableRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketInterruptRegMixin::InterruptEnableRegister
//---------------------------------------------------------------------------

PacketInterruptRegMixin::InterruptEnableRegister::InterruptEnableRegister(AddressingSpace &space,
    int offset )
    : inherited( space, offset ),
      BmRecvInt(     *this, 0 ),
      BmTransmitInt( *this, 1 ),
      CustomInt(     *this, 2 )
{
}

//===========================================================================
//  CLASS PacketInterruptRegMixin::CreditQueueRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketInterruptRegMixin::CreditQueueRegister
//---------------------------------------------------------------------------

PacketInterruptRegMixin::CreditQueueRegister::CreditQueueRegister( AddressingSpace &space, int offset )
    : inherited( space, offset ),
      Credit( *this, 0, 28 ),
      ChannelNumber( *this, 28, 4 )
{
}

//===========================================================================
//  CLASS PacketInterruptRegMixin::PacketTimerRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketInterruptRegMixin::PacketTimerRegister
//---------------------------------------------------------------------------

PacketInterruptRegMixin::PacketTimerRegister::PacketTimerRegister( AddressingSpace &space, int offset )
    : inherited( space, offset ),
      Period( *this, 0, 20 )
{
}


}  // end namespace Innovative

//---------------------------------------------------------------------------


