//  PacketInterruptScheme_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef PacketInterruptScheme_MbH
#define PacketInterruptScheme_MbH

#include <DataTypes_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SafeLink_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <DeviceDriverBmManager_Mb.h>

namespace OpenWire
{
    class Event;
}

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Event;
class Buffer;
typedef  SafeLink<IPacketStreamManagerInternals,
                  SafePacketStreamManagerInternalsImpl>   ManagerInternalsSafeLink;

typedef  SafeLinkAlias<IPacketStreamManagerInternals,
                  SafePacketStreamManagerInternalsImpl>   ManagerInternalsSafeLinkAlias;

//===========================================================================
//  CLASS IPacket_DeviceToScheme  -- Packet Device callbacks for Scheme
//===========================================================================

class IPacket_DeviceToScheme
{
public:
    virtual Event *  IsrEvent() = 0;
    virtual size_t   InterruptDepth() = 0;
    virtual int      InterruptAck() = 0;
    virtual void     Log(int type, int data) = 0;
    virtual void     Log(int type, size_t data)
        {  Log(type, static_cast<int>(data));  }

    //  Busmaster Mem Manager UI
    virtual InnovativeKernel::IDeviceDriverBmManagerUI &  BmMgr() = 0;
    virtual ii32     IncomingBusMasterSizeWords() const = 0;
    virtual ii32     OutgoingBusMasterSizeWords() const = 0;

    virtual bool     AlertTest(unsigned int pid) = 0;
    virtual bool     AlertCheck(const Buffer & buffer) = 0;

    virtual void     DispatchCustomInterrupt(int custom_flags) = 0;

    virtual ManagerInternalsSafeLink &  Link() = 0;
    virtual unsigned int  DebugPortPeek(unsigned int port_addr) = 0;

    virtual ~IPacket_DeviceToScheme()
        {}

};

//---------------------------------------------------------------------------
namespace PacketIntScheme
{
    class VirtualIsrThread;
    class BmXmitThread;
}

//===========================================================================
//  CLASS PacketInterruptScheme  -- ISR & Busmaster region handling
//===========================================================================

class PacketInterruptScheme
{
public:
    enum Dir { BmXmit, BmRecv };

    PacketInterruptScheme(IPacket_DeviceToScheme * device);
    virtual ~PacketInterruptScheme();

    void  Open();
    void  Close();
    void  ConfigureBmRegion();
    void  ConfigureBusmastering();
    void  FillBmRegion(Dir which, unsigned int pattern, bool ramp);
    void  PreloadXmitBusmasterRegion();
    void  TransmitDataAvailable();

    size_t VisrIsrQDepth();

private:
    //
    //  Member Data
    PacketIntScheme::VirtualIsrThread *  Visr;
    PacketIntScheme::BmXmitThread *      BmX;
    ManagerInternalsSafeLinkAlias        StreamManagerInternals;
    IPacket_DeviceToScheme *             StreamDevice;

    void  LaunchXmitData( OpenWire::Event & evt);
    void  BmXmitSignalled( OpenWire::Event & evt);

public:
    //
    //  Debug...
    ii32 * XmitRgnBasePhys;
    ii32 * RecvRgnBasePhys;
    ii32 * XmitRgnBase;
    ii32 * RecvRgnBase;
    unsigned int   XmitRgnSize;
    unsigned int   RecvRgnSize;

};

//===========================================================================
//  CLASS PacketInterruptRegMixin  -- Packet Interrupt Register Defs
//===========================================================================

class PacketInterruptRegMixin
{
public:
    //
    //  Class InterruptEnableRegister
    class InterruptEnableRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       BmRecvInt;
        RegisterBit       BmTransmitInt;
        RegisterBit       CustomInt;

    public:
        InterruptEnableRegister( AddressingSpace &space, int offset );
    };
    //
    //  Class CreditQueueRegister
    class CreditQueueRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup   Credit;
        RegisterBitGroup   ChannelNumber;

    public:
        CreditQueueRegister( AddressingSpace &space, int offset );
    };

    //
    //  Class PacketTimerRegister
    class PacketTimerRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup   Period;

    public:
        PacketTimerRegister( AddressingSpace &space, int offset );
    };
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#endif
