//---------------------------------------------------------------------------
// PacketPoll_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef PACKETPOLL_MB_H
#define PACKETPOLL_MB_H

#include <Buffer_Mb.h>
#include <Events_Mb.h>
#include <BaseEvents_Mb.h>
#include <ThreadSafeQueue_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <StartStopThread_Mb.h>
#include <X3FifoExtension_Mb.h>
#include <SystemSupport_Mb.h>
#include <PacketStream_Mb.h>
#include <FifoExtension_Mb.h>
#include <Trace_Mb.h>
#include <VitaPacketStream_Mb.h>

#include <FmcSupport_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

  class PacketPollMgr;

 //===========================================================================
 // CLASS PacketPollQueue  --  Buffering queue for packet polling operation
 //===========================================================================

 class PacketPollQueue
 {
 public:
     PacketPollQueue();
     PacketPollQueue(size_t capacity);
     ~PacketPollQueue() {}

     // Properties
     void            QueueCapacity(size_t cap_in_blocks);
     size_t          QueueCapacity() const;

     // Buffer Methods
     size_t         Depth();
     void           Clear();
     bool           Push(Buffer & packet);
     bool           Pop(Buffer & packet);

 private:
     //
     //  Member Data
     size_t                   CapacityInBlks;
     mutable CriticalSection  lock;
     ThreadSafeQueue<Buffer>  BufferQ;
 };


//===========================================================================
// CLASS PacketPollDataEvent  --  Data I/O Event
//===========================================================================

class PacketPollDataEvent : public OpenWire::Event
{
public:
    PacketPollMgr *  Sender;

    PacketPollDataEvent( PacketPollMgr * sender )
        :  Sender( sender )
    {
    }
};

class PacketPollMgr;
//===========================================================================
//  CLASS PacketPollThread  -- base class I/O thread to poll for packets
//===========================================================================

class PacketPollThread : public StartStopThread
{
public:
    typedef StartStopThread  inherited;

    void Start();
    void Stop();

    //stats
    ii64                loopCount;
    ii64                fullTxFifoCount;
    ii64                emptyRxFifoCount;
    ii64                alertCount;
    ii64                pktTxCount;
    ii64                pktRxCount;
    bool                once;


    Buffer              packet;

protected:
    PacketPollThread(const char * tName);  //,
                     //PacketPollMgr *owner);

    ~PacketPollThread();

    enum { sQueue=inherited::sLast, sLast };

    //  override to load notify events at start
    virtual void Init();
    virtual void SingleExecution() = 0;

    //members
    //PacketPollMgr *     Owner;
    Trace               Debug;
    ii32                pktSize;
};

//===========================================================================
//  CLASS PacketPollDataThread  -- Polling thread to process packets to/from board
//===========================================================================

class PacketPollDataThread : public PacketPollThread
{
public:
    typedef PacketPollThread  inherited;

    PacketPollDataThread(PacketPollMgr *owner)
        : PacketPollThread("PacketPollDataThread" /*, owner*/), Owner(owner), event(owner)
    {}

    void Start();

protected:
    virtual void    SingleExecution();

private:
    bool GetPacketFromFifo(Buffer &);
    bool PutPacketIntoFifo(Buffer &);

    PacketPollMgr *     Owner;
    PacketPollDataEvent event;
};

//===========================================================================
// CLASS PacketPollDeviceSafeLink  --  Safely Link to a device object
//===========================================================================

class PacketPollDeviceSafeLink : public SafeLink<IPacketStreamDevice, SafePacketStreamDeviceImpl>
{
public:
    PacketPollDeviceSafeLink(PacketPollMgr * owner)
        :  Owner(owner)
    {}

private:
    PacketPollMgr *  Owner;

    //TODO:
    //virtual void  OnAfterLink();        // overrides
    //virtual void  OnBeforeUnlink();
};

//===========================================================================
//  CLASS PacketPollMgr  -- This is the main interface class between application layer
//                          and Mallibu's internal polling operations
//===========================================================================
class PacketPollMgr : public IPacketStreamManager
{
    friend class PacketPollDataThread;

public:

    static const ii32 MAXQSIZE;
    static const ii32 MAX_HW_FIFO;
    enum StatesEnum{OPEN, CLOSE, POLLING, STOP, CONNECT, DISCONNECT};

    PacketPollMgr(IFifoExtension &fx);
    ~PacketPollMgr();

    //data events
    OpenWire::EventHandler<PacketPollDataEvent>  OnDataRequired;
    OpenWire::EventHandler<PacketPollDataEvent>  OnDataAvailable;

    //error events
    OpenWire::EventHandler<PacketPollDataEvent>  OnRecvQFull;
    OpenWire::EventHandler<PacketPollDataEvent>  OnSendQEmpty;

    //App interface methods
    void  Start();
    void  Stop();

    void  Send( Buffer & packet );
    void  Recv( Buffer & packet );

    ii32 SetFifoDacDelay(ii32 ns);

    void EnableTsLog(bool s)
    {
        (s) ? tsLogger.Enable() : tsLogger.Disable();
    }

    //stream interface
    virtual void Send( short, Buffer & ) {}  //not implemented
    virtual bool ConnectTo( IPacketStreamDevice *Device );
    virtual void Disconnect();
    virtual bool StreamStarted() {return true;}
    virtual IPacketStreamManagerInternals *  Internals(){return new (SafePacketStreamManagerInternalsImpl);}

    IPacketStreamDeviceInternals *  DeviceInternals()
    {
        return DeviceLink->Internals();
    }

    //data
    PacketPollDataThread        PktPollThread;
    TimeStampLogger             tsLogger;

protected:
    StatesEnum                  State;
    IFifoExtension &            FifoExt;
    PacketPollQueue             RecvQ;
    PacketPollQueue             SendQ;

private:
    PacketPollDeviceSafeLink    DeviceLink;
    Trace                       trace;
};


typedef SingltonTemplate<PacketPollMgr>  PacketPollSingleton;

//===========================================================================
// CLASS FmcPacketPollDataEvent  --  Data I/O Event
//===========================================================================

class FmcPacketPollDataEvent : public OpenWire::Event
{
public:
    class FmcPacketPollManager *  Sender;

    FmcPacketPollDataEvent( FmcPacketPollManager * sender )
        :  Sender( sender )
    {
    }
};

//===========================================================================
// CLASS FmcPacketPollDeviceSafeLink  --  Safely Link to a device object
//===========================================================================

class FmcPacketPollDeviceSafeLink : public SafeLink<IVitaPacketStreamDevice, SafeVitaPacketStreamDeviceImpl>
{
public:
    FmcPacketPollDeviceSafeLink(FmcPacketPollManager * owner)
        :  Owner(owner)
    {}

private:
    FmcPacketPollManager *  Owner;

    //TODO:
    //virtual void  OnAfterLink();        // overrides
    //virtual void  OnBeforeUnlink();
};


//===========================================================================
//  CLASS FmcPacketPollDataThread  -- Polling thread to process packets to/from board
//===========================================================================

class FmcPacketPollDataThread : public PacketPollThread
{
public:
    typedef PacketPollThread  inherited;

    FmcPacketPollDataThread(class FmcPacketPollManager * owner)
        :  PacketPollThread("FmcPacketPollDataThread"), Owner(owner),
           InputPacketSize(4)
        {}

    virtual void Init();
    virtual void Start();

protected:
    virtual void    SingleExecution();

private:
    bool GetPacketFromFifo(Buffer &);
    bool PutPacketIntoFifo(Buffer &);

    FmcPacketPollManager *     Owner;
    unsigned int               InputPacketSize;

};

//===========================================================================
//  CLASS FmcPacketPollManager
//===========================================================================
//  Interface to FMC-Servo Packet Polling

class FmcPacketPollManager : public IVitaPacketStreamManager
{
    friend class FmcPacketPollDataThread;

public:

    static const ii32 MAXQSIZE;
    static const ii32 MAX_HW_FIFO;
    enum StatesEnum {OPEN, CLOSE, POLLING, STOP, CONNECT, DISCONNECT};

    FmcPacketPollManager();
    ~FmcPacketPollManager();

    void  LoadAdc(IPolledFifo * adc)
            {   AdcFifo = adc;   }
    void  LoadDac(IPolledFifo * dac)
            {   DacFifo = dac;   }

    void  Start();
    void  Stop();

    IPolledFifo *  Adc()  {  return AdcFifo;  }
    IPolledFifo *  Dac()  {  return DacFifo;  }

    bool  IsValid()
            {  return (AdcFifo!=0 && DacFifo!=0);  }


    //  Events
    OpenWire::EventHandler<FmcPacketPollDataEvent>  OnDataRequired;
    OpenWire::EventHandler<FmcPacketPollDataEvent>  OnDataAvailable;

    //OpenWire::EventHandler<PacketPollDataEvent>  OnRecvQFull;
    //OpenWire::EventHandler<PacketPollDataEvent>  OnSendQEmpty;


    //ii32 SetFifoDacDelay(ii32 ns);

    void EnableTsLog(bool s)
    {
        (s) ? tsLogger.Enable() : tsLogger.Disable();
    }

    // stream interface
    //  Interface methods
    virtual void  Send( VeloBuffer & packet );
    virtual void  Recv( VeloBuffer & packet );
    virtual void  Send( short, VeloBuffer & packet ) { Send(packet); }

    virtual bool ConnectTo(IVitaPacketStreamDevice* Device );
    virtual void Disconnect();
    virtual bool StreamStarted() { return true; }
    virtual IPacketStreamManagerInternals *  Internals(){ return new SafePacketStreamManagerInternalsImpl;}

    IPacketStreamDeviceInternals *  DeviceInternals()
    {
        return DeviceLink->Internals();
    }


protected:
    //data
    IPolledFifo *               AdcFifo;
    IPolledFifo *               DacFifo;
    FmcPacketPollDataThread     Thread;
    TimeStampLogger             tsLogger;
    StatesEnum                  State;
    PacketPollQueue             RecvQ;
    PacketPollQueue             SendQ;

private:
    FmcPacketPollDeviceSafeLink    DeviceLink;
    Trace                          Debug;


};








//===========================================================================
// CLASS QuickPacketPollDeviceSafeLink  --  Safely Link to a device object
//===========================================================================

class QuickPacketPollDeviceSafeLink : public SafeLink<IPacketStreamDevice, SafePacketStreamDeviceImpl>
{
public:
    QuickPacketPollDeviceSafeLink(class QuickPacketMgr * owner)
        :  Owner(owner)
    {}

private:
    class QuickPacketMgr *  Owner;

};

//===========================================================================
//  CLASS QuickPacketPollMgr  --
//===========================================================================
class QuickPacketMgr : public IPacketStreamManager
{
    friend class QuickPacketData;

public:
    QuickPacketMgr(IFifoExtension &fx);
    ~QuickPacketMgr();


    bool GetPacketFromFifo(Buffer &);
    bool PutPacketIntoFifo(Buffer &);

    ii32 RecvFifoWordCount()
    {
        return FifoExt.RecvFifoWordCount();
    }

    const unsigned char        	A2dPid;
    const unsigned char        	AlertPid;

     bool HasTxFifoRoom(const Buffer & packet);

    virtual bool ConnectTo(IPacketStreamDevice *Device );
    virtual void Disconnect();

    virtual bool StreamStarted()
    {return Running;}

    virtual IPacketStreamManagerInternals *  Internals()
    { return new (SafePacketStreamManagerInternalsImpl); }

    virtual IPacketStreamDeviceInternals *  DeviceInternals()
    { return DeviceLink->Internals(); }


    //OpenWire::EventHandler<PacketStreamCustomInterruptEvent>  OnCustomInterrupt;

    bool                                StreamStopped;
protected:
    IFifoExtension &                    FifoExt;

private:
    // Unimplemented
    virtual void  Send( short /*PeriphId*/, Buffer & /*packet*/ ) { }
    virtual void  Send( Buffer & /*packet*/ ) { }
    virtual void  Recv( Buffer & /*packet*/ ) { }

    bool                                                  Running;
    const size_t                                      MAX_HW_FIFO;
    QuickPacketPollDeviceSafeLink      DeviceLink;
};



#ifdef __CLR_VER
#pragma managed(pop)
#endif

}
#endif // PACKETPOLL_MB_H
