//---------------------------------------------------------------------------
// PacketStream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <PacketStream_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{
//===========================================================================
// CLASS PacketStreamDeviceSafeLink  --  Safe Linkage to a device
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketStream
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  PacketStreamDeviceSafeLink::OnAfterLink() - handle post-link stuff
//---------------------------------------------------------------------------

void  PacketStreamDeviceSafeLink::OnAfterLink()
{
    Ptr()->RegisterStreamPlugIn(Owner);
}

//---------------------------------------------------------------------------
//  PacketStreamDeviceSafeLink::OnBeforeUnlink() - handle pre-unlink stuff
//---------------------------------------------------------------------------

void  PacketStreamDeviceSafeLink::OnBeforeUnlink()
{
    if (Owner->StreamStarted())
        Owner->Stop();

    Ptr()->RegisterStreamPlugIn(0);
}

//===========================================================================
//  CLASS PacketStream  -- Packet Based Stream I/O
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketStream
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
PacketStream::PacketStream()
    : StreamDevice(this)
{
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class PacketStream
//---------------------------------------------------------------------------

PacketStream::~PacketStream()
{
}

//---------------------------------------------------------------------------
//  PacketStream::Preconfigure() -- Non-repeated 'Manual' Configuration
//---------------------------------------------------------------------------

void  PacketStream::Preconfigure()
{
    PacketStreamBase::Preconfigure();
}

//---------------------------------------------------------------------------
//  PacketStream::Start() -- Start busmastering
//---------------------------------------------------------------------------

void  PacketStream::Start()
{
    PacketStreamBase::Start();
}

//---------------------------------------------------------------------------
//  PacketStream::Stop() -- Stop busmastering
//---------------------------------------------------------------------------

void  PacketStream::Stop()
{
    PacketStreamBase::Stop();
}

//---------------------------------------------------------------------------
//  PacketStream::Internals() -- Stream Manager Internal Interface
//---------------------------------------------------------------------------

IPacketStreamManagerInternals *  PacketStream::Internals()
{
    return PacketStreamBase::ManagerInternals();
}

//---------------------------------------------------------------------------
//  PacketStream::DeviceInternals() -- Stream Device Internal Interface
//---------------------------------------------------------------------------

IPacketStreamDeviceInternals *  PacketStream::DeviceInternals()
{
    return StreamDevice->Internals();
}

//---------------------------------------------------------------------------
//  PacketStream::Send() -- Load packet for transmission
//---------------------------------------------------------------------------

void  PacketStream::Send(short PeriphId, Buffer & packet )
{
    //
    //  Load Peripheral ID
    PacketBufferHeader headerDG(packet);
    headerDG.PeripheralId(PeriphId);
    //  ..and finish sending
    Send(packet);
}

//---------------------------------------------------------------------------
//  PacketStream::Send() -- Post packet for transmission
//---------------------------------------------------------------------------

void  PacketStream::Send(Buffer & packet)
{
    //  Make sure packet size is correct...
    PacketBufferHeader headerDG(packet);
    headerDG.PacketSize(packet.FullSizeInInts());
    //  ..and send it out using base class method
    PacketStreamBase::BasicSend(packet);
}

//---------------------------------------------------------------------------
//  PacketStream::Recv() -- Return packet received from host
//---------------------------------------------------------------------------

void  PacketStream::Recv(Buffer & packet)
{
    // Fill using base class method
    PacketStreamBase::BasicRecv(packet);
}

//---------------------------------------------------------------------------
//  PacketStream::SignalDataAvailable() -- Signal data available to app
//---------------------------------------------------------------------------

void  PacketStream::SignalDataAvailable()
{
    PacketStreamDataEvent e(this);
    OnDataAvailable.Execute(e);
}

//---------------------------------------------------------------------------
//  PacketStream::SignalDataRequired() -- Signal data needed to app
//---------------------------------------------------------------------------

void  PacketStream::SignalDataRequired()
{
    PacketStreamDataEvent e(this);
    OnDataRequired.Execute(e);
}

//---------------------------------------------------------------------------
//  PacketStream::StreamStarted() -- returns true if the stream is started
//---------------------------------------------------------------------------

bool PacketStream::StreamStarted()
{
    return DoStreamStarted();
}

//---------------------------------------------------------------------------
//  PacketStream::ConnectTo() -- connect stream to stream device
//---------------------------------------------------------------------------

bool  PacketStream::ConnectTo(IPacketStreamDevice * aStreamDevice)
{
    if (aStreamDevice)
        {
        StreamDevice.Link(aStreamDevice);
        return true;
        }
    else
        return false;
}

//---------------------------------------------------------------------------
//  PacketStream::Disconnect() -- disconnect stream from stream device
//---------------------------------------------------------------------------

void PacketStream::Disconnect()
{
    StreamDevice.Unlink();
}

//---------------------------------------------------------------------------
//  PacketStream::NativeBuffer() -- create native buffer type
//---------------------------------------------------------------------------

Buffer  PacketStream::NativeBuffer(size_t data_size, bool autoinit)
{
    Buffer  buffer(data_size);
    if (autoinit)
        {
        ClearHeader(buffer);
        InitPmcHeader(buffer);
        }
    return buffer;
}

}  // namespace Innovative

