//---------------------------------------------------------------------------
// PacketStream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef PacketStream_MbH
#define PacketStream_MbH

#include <PacketStreamBase_Mb.h>
#include <ThreadSafeQueue_Mb.h>
#include <SafeLink_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class PacketStream;

//===========================================================================
// CLASS PacketStreamDataEvent  --  Data I/O Event for PacketStream
//===========================================================================

class PacketStreamDataEvent : public OpenWire::Event
{
public:
    PacketStream *  Sender;

public:
    PacketStreamDataEvent( PacketStream * sender )
        :  Sender( sender )
        {
        }
};

//===========================================================================
// CLASS PacketStreamDeviceSafeLink  --  Safe Linkage to a device
//===========================================================================

class PacketStreamDeviceSafeLink : public SafeLink<IPacketStreamDevice, SafePacketStreamDeviceImpl>
{
public:
    PacketStreamDeviceSafeLink(PacketStream * owner)
        :  Owner(owner)
        {}

private:
    PacketStream *  Owner;

    virtual void  OnAfterLink();        // overrides
    virtual void  OnBeforeUnlink();
};

//===========================================================================
//  CLASS PacketStream  -- Packet Based Stream I/O
//===========================================================================

class PacketStream : public PacketStreamBase,
                     public IPacketStreamManager
{
public:
    PacketStream();
    virtual ~PacketStream();

    virtual void  Preconfigure();
    virtual void  Start();
    virtual void  Stop();

    void  Send( short PeriphId, Buffer & packet );
    void  Send( Buffer & packet );
    void  Recv( Buffer & packet );

    virtual bool  StreamStarted();

    virtual bool  ConnectTo( IPacketStreamDevice *Device );
    virtual void  Disconnect( );

    Buffer  NativeBuffer(size_t data_size, bool autoinit=true);

    virtual IPacketStreamManagerInternals *  Internals();
    virtual IPacketStreamDeviceInternals *   DeviceInternals();

    OpenWire::EventHandler<PacketStreamDataEvent>  OnDataRequired;
    OpenWire::EventHandler<PacketStreamDataEvent>  OnDataAvailable;

protected:
    //
    //  Member Functions
    //
    //  Velocia-style packet "Interface" between base and derived classes
    virtual void  SignalDataAvailable();
    virtual void  SignalDataRequired();

    virtual IVelociaBufferManager *  ActualVelociaManager()
                                        {  return &BufferMgr;  }

private:
    //
    //  Member Data
    PacketStreamDeviceSafeLink    StreamDevice;
    PmcBufferManager              BufferMgr;
    //
    //  Member Functions

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace

#endif
