//
//  PciTransferStruct_Mb.h
//

#ifndef PciTransferStruct_MbH
#define PciTransferStruct_MbH 

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//
//  In order to have a safe place for shared data between target and host we
//    require a known segment be created and used in the target application.
//    Its address is determined by obtaining the address for "PCIDATA_base"
//    from the COFF symbol table.  The host will access this region
//    using the page memory region of CPU.
//
//  This file is shared between Pismo libraries and Armada libraries
//
//=============================================================================
//  CLASS PciInterruptInfo --  Interrupt Information
//=============================================================================

class PciInterruptInfo
{
public:
    int  Code;    //  Describes the interrupt
    int  Data;    //  Associated Data
    int  Channel;
    int  Flag;
};

//=============================================================================
//  CLASS PciParameters  --  Describes a transfer in either direction
//=============================================================================

class PciParameters
{
public:
    int  Address;
    int  TotalSize;
    int  TransferSize;
};

//=============================================================================
//  CLASS PciMail  --  Describes a transfer in either direction
//=============================================================================

class PciMail
{
public:
    // Record size, in bytes
    static const int Slots = 16;

    int         Command;
    int         Slot[Slots];
};

//=============================================================================
//  CLASS PciTransferInfo  --  Shared Information between Target & Host
//=============================================================================

class PciTransferInfo
{
public:
    PciInterruptInfo  TtoHInterrupt;
    PciInterruptInfo  HtoTInterrupt;
    PciParameters     TtoHXfrParams;
    PciParameters     HtoTXfrParams;
    PciMail           TtoHMail;
    PciMail           HtoTMail;

    enum Codes { icUserInterrupt, icBmReadRequest, icBmReadAck, icBmWriteGetAddr,
                 icBmWriteDataAvail, icBmWriteAck,
                 icMailAvail, icMailAck };
    enum TransferStatus { tsOk, tsErrorTargetAbort, tsErrorMasterAbort };
    enum IntFlagCodes { ifFree, ifBusy = 0xFACE };
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative

#endif

