// Pex7CopDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2018 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Pex7Cop_Mb.h>
#include <Pex7CopDeviceMap_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <Pex7CopMemoryMap_Mb.h>
#include <Pex7CopDeviceDriver_Mb.h>
//#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace FmcCarrier_MemoryMap;
using namespace Pex7CopMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

//===========================================================================
//  CLASS Pex7CopDeviceMap  -- Register & low level support for X6-400M
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Pex7CopDeviceMap
//---------------------------------------------------------------------------

namespace
{
const size_t  FMC_Slot_Count = 1;

//
//  IDROM Page Numbers
const unsigned int System_IdromPage = 0;
const unsigned int AdcCal_IdromPage = 1;
const unsigned int DacCal_IdromPage = 2;
const unsigned int User_IdromPage   = 3;


const int    MaxInDevices      = 2;
const int    ChannelsPerDevice = 2;
const int    MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const unsigned int Pex7Cop_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int Pex7Cop_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int Pex7Cop_VeloOutStreamCount  = 1;   // Number of Velocia Outgoing PIDs for Data
const unsigned int Pex7Cop_VeloArbStreamCount  = 1;   // Number of Velocia Outgoing PIDs for ARB command channels

const unsigned int Pex7Cop_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs


FmcCarrierDeviceMap::MapWishbones Pex7Cop_WishboneDefs =
    {
    wbApplication_K7,            // WbApp_Offset;
    };

X6Thermal::MapRegisters Pex7ThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

XUDigitalIo::MapRegisters  Pex7CopDioRegAddrs =
    {
    mmFpa_Address,        //  DioDataLo_Addr
    mmFpa_Data,           //  DioOutputEnableLo_Addr
    };

XAFpaLoader::MapRegisters Pex7CopFpaRegAddrs =
    {
    mmFpa_Address,        //  AddressControl_Addr
    mmFpa_Data,           //  Data_Addr
    mmFpa_Opcode,         //  Opcode_Addr
    };

K7FlashRom::MapRegisters Pex7FlashRomRegAddrs =
    {
    mmRom_AddrCtl,         //  AddressControl_Addr
    mmRom_Data,            //  Data_Addr
    mmRom_Opcode,          //  Opcode_Addr
    };

LTC2494SpiDevice::Info Pex7CopSpiRegAddrs =
{
    mmPex7Cop_LTC_SPI_Control,  // Write Offset
    mmPex7Cop_LTC_SPI_Stat,     // Read Offset
};


}  // nameless namespace

//---------------------------------------------------------------------------
//  constructor for class Pex7CopDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

Pex7CopDeviceMap::Pex7CopDeviceMap(Pex7Cop * owner)
    :  	FmcCarrierDeviceMap(owner, FMC_Slot_Count, new Pex7CopDeviceDriver, Pex7Cop_WishboneDefs),
        Owner(owner),
        WB_Thermal               (LogicMemory, wbTempControl),
        WB_DigitalIo             (LogicMemory, wbDigitalIo),
        Thermal_SDev             (this, WB_Thermal, Pex7ThermalRegAddrs),
        Dio_SDev                 (this, WB_DigitalIo, Pex7CopDioRegAddrs),
        VsProm_SDev              (this, WB_Loader, Pex7CopFpaRegAddrs),
        FlashRom_SDev            (this, K7FlashRom::romS25FL128S, WB_FlashRom, Pex7FlashRomRegAddrs),
        SelfMeasure_SDev         (this),
        System_PG                (FlashRom_SDev, System_IdromPage),
        AdcCal_PG                (FlashRom_SDev, AdcCal_IdromPage),
        DacCal_PG                (FlashRom_SDev, DacCal_IdromPage),
        UserRom_PG               (FlashRom_SDev, User_IdromPage),
        IdRom_Intf               (this)
{
    //
    //  SubDevice Customization
    LocalConstructCustomParts();

    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &Pex7CopDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &Pex7CopDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &Pex7CopDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &Pex7CopDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &Pex7CopDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &Pex7CopDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &Pex7CopDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class Pex7CopDeviceMap
//---------------------------------------------------------------------------

Pex7CopDeviceMap::~Pex7CopDeviceMap()
{
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::PrePowerup_CustomizeDaughter() --
//---------------------------------------------------------------------------
//  Carrier dependent pre-powerup customization

void  Pex7CopDeviceMap::PrePowerup_CustomizeDaughter(DevMapSafeLink & daughter)
{
    daughter->Pex7Cop_PrePowerCustomization();
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::LocalConstructCustomParts()
{
    FmcCarrierDeviceMap::ConstructCustomParts();

    Thermal_SDev.ConstructCustomParts();
    Dio_SDev.ConstructCustomParts();
    VsProm_SDev.ConstructCustomParts();
    FlashRom_SDev.ConstructCustomParts();
    SelfMeasure_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OpenHardware()
{
    FmcCarrierDeviceMap::OpenHardware();

    Thermal_SDev.OpenHardware();
    Dio_SDev.OpenHardware();
    VsProm_SDev.OpenHardware();
    FlashRom_SDev.OpenHardware();
    SelfMeasure_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "Pex7CopDeviceMap::OnStreamPreconfigHandler()");

    // Family-common init
    FmcCarrierDeviceMap::PreconfigureHardware();

    Thermal_SDev.PreconfigureHardware();
    Dio_SDev.PreconfigureHardware();
    VsProm_SDev.PreconfigureHardware();
    FlashRom_SDev.PreconfigureHardware();
    SelfMeasure_SDev.PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "Pex7CopDeviceMap::OnStreamConfigureHandler()");

    // Family-common init
    FmcCarrierDeviceMap::ConfigureHardware();

    Thermal_SDev.ConfigureHardware();
    Dio_SDev.ConfigureHardware();
    VsProm_SDev.ConfigureHardware();
    FlashRom_SDev.ConfigureHardware();
    SelfMeasure_SDev.ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);

    //
    //  Start execution
    FmcCarrierRun(true);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  Pex7CopDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    FmcCarrierRun(false);

    //  Base class "on stream stop" behavior
    FmcCarrierDeviceMap::StreamStopHardware();

    //  Forward notify to baseboard handler
	OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  Pex7CopDeviceMap::GetVeloInStreamCount()
{
    return Pex7Cop_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  Pex7CopDeviceMap::GetVeloOutStreamCount()
{
    return Pex7Cop_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetVeloOutCommandStreamCount() --
//--------------------------------------------------------------------------
//  Configure PIDs for Arb command channels-

unsigned int  Pex7CopDeviceMap::GetVeloOutCommandStreamCount()
{
    return Pex7Cop_VeloArbStreamCount;
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   Pex7CopDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   Pex7CopDeviceMap::GetPatternModeAlignment()
{
    return 16;   // standard K7 (FMC) alignment of 16 words
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   Pex7CopDeviceMap::GetPatternModeScaleFactor()
{
    return 1;   // standard K7 (FMC) scale factor of 1 words
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   Pex7CopDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return true;  // single memory
}

//---------------------------------------------------------------------------
//  Pex7CopDeviceMap::MakeLTC2494() -- Get LTC2494 Device object
//---------------------------------------------------------------------------

LTC2494 *  Pex7CopDeviceMap::MakeLTC2494()
{
    return new Pex7CopLTC2494(WB_Periph, Pex7CopSpiRegAddrs, "Pex7Cop_LTC2494");
}

//===========================================================================
//  CLASS Pex7CopLTC2494 --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Pex7CopLTC2494
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  Pex7CopLTC2494::Pex7CopLTC2494() -- Constructor
//---------------------------------------------------------------------------

Pex7CopLTC2494::Pex7CopLTC2494(IAddressingSpace &space,  LTC2494SpiDevice::Info &regs, std::string tag)
    :  LTC2494(space, regs, tag)
{

}

//---------------------------------------------------------------------------
//  Pex7CopLTC2494::~Pex7CopLTC2494() -- Destructor
//---------------------------------------------------------------------------

Pex7CopLTC2494::~Pex7CopLTC2494()
{

}

//---------------------------------------------------------------------------
//  Pex7CopLTC2494::ConfigureBoardDefualts() -- config board defualts
//---------------------------------------------------------------------------

void Pex7CopLTC2494::ConfigureBoardDefaults()
{
     // Channel Mapping
     ChannelTypeTable.push_back(SingleEnded); //Channel 0
     ChannelTypeTable.push_back(SingleEnded); //Channel 1
     ChannelTypeTable.push_back(SingleEnded); //Channel 2
     ChannelTypeTable.push_back(SingleEnded); //Channel 3
     ChannelTypeTable.push_back(SingleEnded); //Channel 4
     ChannelTypeTable.push_back(SingleEnded); //Channel 5
     ChannelTypeTable.push_back(SingleEnded); //Channel 6
     ChannelTypeTable.push_back(SingleEnded); //Channel 7
     ChannelTypeTable.push_back(SingleEnded); //Channel 8
     ChannelTypeTable.push_back(SingleEnded); //Channel 9
     ChannelTypeTable.push_back(SingleEnded); //Channel 10
     ChannelTypeTable.push_back(SingleEnded); //Channel 11
     ChannelTypeTable.push_back(SingleEnded); //Channel 12
     ChannelTypeTable.push_back(SingleEnded); //Channel 13
     ChannelTypeTable.push_back(SingleEnded); //Channel 14
     ChannelTypeTable.push_back(SingleEnded); //Channel 15
     MapChannels();

     //configuring defualt bits
     ConfigBits.EN  = 1;
     ConfigBits.EN2 = 1;
     ConfigBits.FA  = 0;
     ConfigBits.FB  = 0;
     ConfigBits.SPD = 0; // dont care if temp measurement
     ConfigBits.GS0 = 0; //
     ConfigBits.GS1 = 0; // dont care if temp measurement
     ConfigBits.GS2 = 0; //

    //  initialize Scale table

    const double ReferenceV = 1.25;

    ScaleTable.push_back(1.0 * ReferenceV);
    ScaleTable.push_back(1.00 * ReferenceV);
    ScaleTable.push_back(12.050 * ReferenceV);
    ScaleTable.push_back(3.32 * ReferenceV);
    ScaleTable.push_back(2.5050 * ReferenceV);
    ScaleTable.push_back(2.5050 * ReferenceV);
    ScaleTable.push_back(2.0000 * ReferenceV);
    ScaleTable.push_back(1.8000 * ReferenceV);
    ScaleTable.push_back(1.5000 * ReferenceV);
    ScaleTable.push_back(1.2015 * ReferenceV);
    ScaleTable.push_back(1.00 * ReferenceV);
    ScaleTable.push_back(1.00 * ReferenceV);
    ScaleTable.push_back(1.00 * ReferenceV);
    ScaleTable.push_back(5.00 * ReferenceV);
    ScaleTable.push_back(5.00 * ReferenceV);
    ScaleTable.push_back(5.00 * ReferenceV);
}




} // namespace Innovative


