// Pex7CopDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2018 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef Pex7CopDeviceMap_MbH
#define Pex7CopDeviceMap_MbH

#include <Pex7Cop_Mb.h>
#include <FmcCarrierDeviceMap_Mb.h>
#include <XASubDevice_Misc_Mb.h>
#include <K7SubDevice_Misc_Mb.h>
#include <XUSubDevice_Misc_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class Pex7CopDeviceMap;
class Pex7CopDeviceDriver;

//===========================================================================
//  CLASS ISuppliesAddedPex7CopRequirements -- Pex7Cop Map Requirement List
//===========================================================================

class ISuppliesAddedPex7CopRequirements :  public IRequires_LTC2494Device
{
public:
    virtual ~ISuppliesAddedPex7CopRequirements()
        {}

};

//===========================================================================
//  CLASS Pex7CopDeviceMap  -- Register & low level support for Pex7Cop module
//===========================================================================
class Pex7Cop;

class Pex7CopDeviceMap :  public FmcCarrierDeviceMap,
                          public ISuppliesAddedPex7CopRequirements
{
    friend class Pex7Cop;
    
public:
    //
    //  Member Functions
    Pex7CopDeviceMap(Pex7Cop * owner);
    virtual ~Pex7CopDeviceMap();

public: // pseudo-protected
    virtual void  OpenHardware();
//    virtual void    ConstructCustomParts();    // call in most derived ctor
//    virtual void    OpenHardware();
//    virtual void    PreconfigureHardware();
//    virtual void    ConfigureHardware();
//    virtual void    StreamStopHardware();

    //  Subdevice Customization Handlers

    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
    virtual unsigned int GetVeloOutCommandStreamCount();

	virtual IIsAVitaPacketStreamBoard *   GetBaseboardPtr();
	virtual unsigned int                  GetPatternModeAlignment();
	virtual unsigned int                  GetPatternModeScaleFactor();
	virtual bool                          GetPatternModeSingleMemoryFlag();
    virtual LTC2494 *                     MakeLTC2494();

protected:
    // Data
    Pex7Cop *           Owner;

    IIdRom &                  IdRom()   {  return IdRom_Intf;   }
    IX6ThermalDevice &        Thermal() {  return Thermal_SDev;  }
    XUDioPortIntf &           Dio()     {  return Dio_SDev;  }

    ISelfMeasurement &        SelfMeasure()     {  return SelfMeasure_SDev.SelfM();  }


protected:
    Pex7CopDeviceDriver *    DeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Thermal;
    WishboneBusSpace       WB_DigitalIo;
    //
    //  Subdevices
    X6Thermal                Thermal_SDev;
    XUDigitalIo              Dio_SDev;
    XAFpaLoader              VsProm_SDev;
    K7FlashRom               FlashRom_SDev;
    SelfMeasurementDevice    SelfMeasure_SDev;

    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    X6IdromPage                  AdcCal_PG;
    X6IdromPage                  DacCal_PG;
    X6IdromPage                  UserRom_PG;


    //=======================================================================
    //  CLASS Pex7IdromIo  -- Access to IDROM Objects
    //=======================================================================
    class Pex7IdromIo : public IIdRom
    {
    public:

        virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
        //  Page access
        virtual X6SystemIdromPage &  System() { return Owner->System_PG;  }
        virtual X6IdromPage &        AdcCal() { return Owner->AdcCal_PG;  }
        virtual X6IdromPage &        DacCal() { return Owner->DacCal_PG;  }
        virtual X6IdromPage &        User()   { return Owner->UserRom_PG;  }

        Pex7IdromIo(Pex7CopDeviceMap * owner) :  Owner(owner)
            {}
        virtual ~Pex7IdromIo()
            {}

    private:
        Pex7CopDeviceMap *  Owner;
    };

    Pex7IdromIo                    IdRom_Intf;
    friend class Pex7IdromIo;

    virtual IFpaLoader &     DerivedVsProm()  {  return VsProm_SDev.VsProm();  }

public:
    Trace               Debug;

protected:
    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void  LocalConstructCustomParts();
    void  PrePowerup_CustomizeDaughter(DevMapSafeLink& daughter);
};


//===========================================================================
//  CLASS Pex7CopLTC2494 -- 16-Channel ADC chip for Pex7Cop
//===========================================================================

class Pex7CopLTC2494 : public LTC2494
{
public:
     Pex7CopLTC2494(IAddressingSpace &space, LTC2494SpiDevice::Info &Regs,
                  std::string tag="");
    ~Pex7CopLTC2494();

private:
    virtual void                ConfigureBoardDefaults();

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
