// PexCopDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <PexCop_Mb.h>
#include <PexCopDeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <PexCopMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X6_MemoryMap;
using namespace PexCopMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxInDevices      = 2;
const int    ChannelsPerDevice = 2;
const int    MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const float         InputMaxRate            = 160.0e6f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 16;
const unsigned int  InputMax_mV             = 1200;
const unsigned int  InputMin_mV             = -1200;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 256;

const unsigned int PexCop_AdcSpiDevices      = MaxInDevices;
const unsigned int PexCop_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int PexCop_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int PexCop_AdcCount            = MaxInChannels;  // Number of Adc Channels
const unsigned int PexCop_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations

const unsigned int PexCop_DacSpiDevices      = 0;   // No DACs with SPIs

const unsigned int PexCop_VeloOutStreamCount  = 1;   // Number of Velocia Outgoing PIDs for Data
const unsigned int PexCop_VeloArbStreamCount  = 1;   // Number of Velocia Outgoing PIDs for ARB command channels

const unsigned int PexCop_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

const unsigned int PexCop_Adc_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romPexCop_SysInfoOffset        = 0;
const int romPexCop_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romPexCop_InputCoeffOffset       = 0;
const int romPexCop_InputCoeffOffset_avail = 9;   // 4 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romPexCop_OutputCoeffOffset       = 0;
const int romPexCop_OutputCoeffOffset_avail = 0;  // none used

//===========================================================================
// Class PexCop_AnalogInInfo
//===========================================================================

class PexCop_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    PexCop_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~PexCop_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

/*
//=============================================================================
//  CLASS PexCop_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PexCop_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class PexCop_Pll
//---------------------------------------------------------------------------

PexCop_Pll::PexCop_Pll(PexCopDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner)
{
    SetMasterOutputSelection(1);  // Use Output 1 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  PexCop_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  PexCop_Pll::OnInit()
{
    //  Device Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         true,   // Secondary Bias ON
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       false,  // Input Term NOT disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, true);   SetOutputConfig(0, ocLvds);     // FPGA
	OutputInUse(1, true);   SetOutputConfig(1, ocLvPecl);   // fudge to get 0's divider turned on...
    //OutputInUse(1, false);  SetOutputConfig(1, ocDisabled);
    OutputInUse(2, false);  SetOutputConfig(2, ocDisabled);
    OutputInUse(3, false);  SetOutputConfig(3, ocDisabled);
    OutputInUse(4, false);  SetOutputConfig(4, ocDisabled);
    OutputInUse(5, true);   SetOutputConfig(5, ocLvCmos_P_Active | ocLvCmos_N_TriStated);   // ADC Dev 0 (Ch0-1)
    OutputInUse(6, false);  SetOutputConfig(6, ocDisabled);
    OutputInUse(7, true);   SetOutputConfig(7, ocLvCmos_P_Active | ocLvCmos_N_TriStated);   // ADC Dev 1 (Ch2-3)
    OutputInUse(8, false);  SetOutputConfig(8, ocDisabled);
    OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);

    //  Phase = 0
    SetPhaseAdjust(0, 1, 0);   // Set all outputs to 0 phase
    SetPhaseAdjust(5, 1, 0);
    SetPhaseAdjust(7, 1, 0);

    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //    (Outputs 0 and 1 match preset value, so leave alone)
    //OutputFrequencyLimit[6] = 100.0e6;
    //OutputFrequencyLimit[8] = 250.0e6;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use external pin
}

//---------------------------------------------------------------------------
//  PexCop_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  PexCop_Pll::Vcxo()
{
    return Owner->AFEClk_SDev.Vcxo();
}

//---------------------------------------------------------------------------
//  PexCop_Pll::OnReapply()
//---------------------------------------------------------------------------

void  PexCop_Pll::OnReapply()
{
}
*/
//=============================================================================
//  CLASS PexCop_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PexCop_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class PexCop_SioPll
//---------------------------------------------------------------------------

PexCop_SioPll::PexCop_SioPll(PexCopDeviceMap * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  PexCop_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  PexCop_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, true);     SetMultiSynthPowerDown(3, true);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(2, diNoClk);   // override since not using output
    SetOutDividerSource(3, diNoClk);   // override since not using output

    OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
    OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, false);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, false);

    SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
    SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
    SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
    SetOutputVoltage(3, vo_2p5volts);   // actually off

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);  // actually off

}

//---------------------------------------------------------------------------
//  PexCop_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  PexCop_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS PexCopDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PexCopDeviceMap
//---------------------------------------------------------------------------

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)

//static Cdce72010Registers Vpx_PllRegisters =
//    { mmAFE_PllControl,  mmAFE_PllSpi  };

//static X6_Std_I2CLinkage::MapRegisters Vpx_VcoI2CLinkRegAddrs =
//    {  mmAFE_VcxoControl,      //  AfeVcxoControl_Addr
//    };

//---------------------------------------------------------------------------
//  constructor for class PexCopDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

PexCopDeviceMap::PexCopDeviceMap(PexCop * owner)
    :  PexDeviceMap(owner),
       //FInput(new PexCopInputDevice(this, new PexCop_AnalogInInfo)),
       Owner(owner)
{
    //
    //  Since this device has no DACs, disable all DAC subdevices
//    AFEDacC_SDev.DeviceEnabled(false);
//    AFEDacSpi_SDev.DeviceEnabled(false);
//    AFEDacLatencyCal_SDev.DeviceEnabled(false);
//    AFEDacCal_SDev.DeviceEnabled(false);
    //
    //  SubDevice Customization
    PexDeviceMap::ConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &PexCopDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &PexCopDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &PexCopDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &PexCopDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &PexCopDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &PexCopDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &PexCopDeviceMap::OnAfterStreamStopHandler );


}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class PexCopDeviceMap
//---------------------------------------------------------------------------

PexCopDeviceMap::~PexCopDeviceMap()
{
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OpenHardware()
{
    PexDeviceMap::OpenHardware();

}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
	// Family-common init
	PexDeviceMap::PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    PexDeviceMap::ConfigureHardware();

    //
    //  Apply Gain/Offset Coefficients
    //	if (FInput->Calibrated())
    //		FInput->ApplyRomCoefficients();
    //	else
    //		FInput->ApplyDefaultCoefficients();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    FmcCarrierRun(true);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  PexCopDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    FmcCarrierRun(false);

    //  Base class "on stream stop" behavior
    PexDeviceMap::StreamStopHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  PexCopDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new PexCop_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  PexCopDeviceMap::GetVeloInStreamCount()
{
    return PexCop_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetVeloOutStreamCount() -- Return Velo Out PIDs
//---------------------------------------------------------------------------

unsigned int  PexCopDeviceMap::GetVeloOutStreamCount()
{
    return PexCop_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetVeloOutCommandStreamCount() --
//--------------------------------------------------------------------------
//  Configure PIDs for Arb command channels-

unsigned int  PexCopDeviceMap::GetVeloOutCommandStreamCount()
{
    return PexCop_VeloArbStreamCount;
}


//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

//unsigned int  PexCopDeviceMap::GetVitaInStreamCount()
//{
//    return PexCop_VitaInStreamCount;
//}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   PexCopDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   PexCopDeviceMap::GetPatternModeAlignment()
{
    return 8;   // standard X6 alignment of 8 words
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   PexCopDeviceMap::GetPatternModeScaleFactor()
{
    return 8;   // standard X6 scale factor of 8 words
}

//---------------------------------------------------------------------------
//  PexCopDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   PexCopDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return false;  // dual memory
}



} // namespace Innovative


