//---------------------------------------------------------------------------
// PexCop_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <PexCop_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS PexCop  -- PexCop PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PexCop
//---------------------------------------------------------------------------

PexCop::PexCop()
    :  PexBoard(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class PexCop 
//---------------------------------------------------------------------------

PexCop::~PexCop()
{
    Close();
}

//---------------------------------------------------------------------------
//  PexCop::AfterFiclStart() --
//---------------------------------------------------------------------------

void  PexCop::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Fmc  1 constant SioPll ");

    engine.Evaluate(": ?PexCop "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Fmc  SioPll\" "
                    "    cr  "
                    "    cr .\" ---- PexCop Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?PexCop  : display this message \" "
                    "    cr  ; ");
    //engine.Evaluate(": ?PexCop  ?PexCop  ; "  );    // alias

    for (size_t i=0; i<FmcSlots(); i++)
        {
        if (FmcDaughter(i).Linked()==false)
            continue;
        FmcDaughter(i)->AfterFiclStart(engine);
        }

    engine.Evaluate("marker app");

    engine.Display("PexCop-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    PexBoard::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  PexCop::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  PexCop::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  const int kSioPll = 1;

    switch (dev)
        {
        case kFmc:
        default:
            return FmcDaughter(0)->Do_DFetch(dev, addr);
        case kSioPll:
            return Device.SioPll_SDev.Pll().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  PexCop::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  PexCop::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;   const int kSioPll = 1;

    switch (dev)
        {
        case kFmc:
        default:
            FmcDaughter(0)->Do_DStore(dev, addr, value);
            break;
        case kSioPll:
            Device.SioPll_SDev.Pll().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  PexCop::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  PexCop::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;   const int kSioPll = 1;
    switch (dev)
        {
        case kFmc:
        default:
            FmcDaughter(0)->Do_DFreq(dev, freq);
            break;
        case kSioPll:
            Device.SioPll_SDev.Pll().Frequency(freq);
            Device.SioPll_SDev.Pll().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  PexCop::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  PexCop::Do_DFreqActual(int dev)
{
    const int kFmc = 0;    const int kSioPll = 1;
    switch (dev)
        {
        case kFmc:
        default:
            return FmcDaughter(0)->Do_DFreqActual(dev);
        case kSioPll:
            return static_cast<float>(Device.SioPll_SDev.Pll().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
