// PexDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <PexDeviceMap_Mb.h>
#include <FmcCarrierMemoryMap_Mb.h>
#include <PexMemoryMap_Mb.h>
#include <X6DeviceDriver_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS PexDeviceMap  -- Registers for X6 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class PexDeviceMap
//---------------------------------------------------------------------------

const size_t  FMC_Slot_Count = 1;
//
//  IDROM Page Numbers

const unsigned int System_IdromPage = 0;
const unsigned int AdcCal_IdromPage = 1;
const unsigned int DacCal_IdromPage = 2;
const unsigned int User_IdromPage   = 3;

//
//  Subdevice Register Structures

using namespace FmcCarrier_MemoryMap;
using namespace Pex_MemoryMap;

namespace
{

X6FpaLoader::MapRegisters X6FpaRegAddrs =
    {
    mmFpa_Address,        //  FpaAddress_Addr
    mmFpa_Data,           //  FpaData_Addr
    mmFpa_Command,        //  FpaCommand_Addr
    };

X6SioPll::MapRegisters PexSioPllRegAddrs =
    {
    mmSioXo,              //  SioPll_Addr
    };


} // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class PexDeviceMap
//---------------------------------------------------------------------------

PexDeviceMap::PexDeviceMap(IStreamDeviceEvents * owner)
	:   FmcCarrierAdvDeviceMap(owner, FMC_Slot_Count),
        Fpa_SDev   (this, WB_Loader, X6FpaRegAddrs),
        SioPll_SDev(this, WB_Periph, PexSioPllRegAddrs)//,
{
}

//---------------------------------------------------------------------------
//  destructor for class PexDeviceMap
//---------------------------------------------------------------------------

PexDeviceMap::~PexDeviceMap()
{
}

//---------------------------------------------------------------------------
//  PexDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  PexDeviceMap::ConstructCustomParts()
{
    FmcCarrierAdvDeviceMap::ConstructCustomParts();

    Fpa_SDev.ConstructCustomParts();
    SioPll_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  PexDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  PexDeviceMap::OpenHardware()
{
    FmcCarrierAdvDeviceMap::OpenHardware();

    Fpa_SDev.OpenHardware();
    SioPll_SDev.OpenHardware();

}

//---------------------------------------------------------------------------
//  PexDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  PexDeviceMap::PreconfigureHardware()
{
    FmcCarrierAdvDeviceMap::PreconfigureHardware();

    Fpa_SDev.PreconfigureHardware();
    SioPll_SDev.PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  PexDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  PexDeviceMap::ConfigureHardware()
{
    FmcCarrierAdvDeviceMap::ConfigureHardware();

    Fpa_SDev.ConfigureHardware();
    SioPll_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  PexDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  PexDeviceMap::StreamStopHardware()
{
    FmcCarrierAdvDeviceMap::StreamStopHardware();
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


} // namespace Innovative


