// PmcEeprom_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifdef __BORLANDC__
#include <mem.h>
#endif
#include <PmcEeprom_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

//===========================================================================
//  CLASS RomI2CDeviceIntf  --  Interface to configuration ROM I2C register
//===========================================================================

class RomI2CDeviceIntf : public I2CDeviceIntf
{
    class I2cReadRegister : public Register
    {
    private:
        typedef Register inherited;

    public:

        RegisterBit         Io;

        I2cReadRegister(IAddressingSpace &space,
            int offset)
        :  inherited(space, offset), Io(*this, 0)
            {
            }

        int Data() const        {   return Io.Value();  }
    };

    class I2cRegister : public ShadowRegister
    {
    private:
        typedef ShadowRegister inherited;

    public:

        RegisterBit         Io;
        RegisterBit         Clk;

        I2cRegister(IAddressingSpace &space,
            int offset)
        :  inherited(space, offset), Io(*this, 0), Clk(*this, 1)
            {
            }

        void Data(int state)    {   Io.Value(state ? true : false);  }
        int Data() const        {   return Io.Value();  }

		void Clock(int state)    {   Clk.Value(state ? true : false);  }
        int Clock() const        {   return Clk.Value();  }
    };

public:
    RomI2CDeviceIntf(IAddressingSpace &space, int offset)
        : ReadReg(space, offset), Reg(space, offset)
        {  }
    virtual ~RomI2CDeviceIntf()  {  }

    // Methods
    virtual void    SetClock(int state)     {  Reg.Clock(state);    }
    virtual int     GetClock() const        {  return Reg.Clock();  }
    virtual void    SetData(int state)      {  Reg.Data(state);     }
    virtual int     GetData() const         {  return ReadReg.Data();   }

private:
    I2cReadRegister ReadReg;
    I2cRegister     Reg;
};

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS ModernRomI2CDeviceIntf  --  Interface to configuration ROM I2C register
//===========================================================================

class ModernRomI2CDeviceIntf : public I2CDeviceIntf
{
    class I2cReadRegister : public Register
    {
    private:
        typedef Register inherited;

    public:

        RegisterBit         Io;
        RegisterBit         Clk;

        I2cReadRegister(IAddressingSpace &space,
            int offset)
        :  inherited(space, offset), Io(*this, 2), Clk(*this, 3)
            {
            }

        int Data() const        {   return Io.Value();  }
        int Clock() const        {   return Clk.Value();  }
    };

    class I2cRegister : public ShadowRegister
    {
    private:
        typedef ShadowRegister inherited;

    public:

        RegisterBit         Io;
        RegisterBit         Clk;

        I2cRegister(IAddressingSpace &space,
            int offset)
        :  inherited(space, offset), Io(*this, 0), Clk(*this, 1)
            {
            }

        void Data(int state)    {   Io.Value(state ? true : false);  }
        int Data() const        {   return Io.Value();  }

        void Clock(int state)    {   Clk.Value(state ? true : false);  }
        int Clock() const        {   return Clk.Value();  }
    };

public:
    ModernRomI2CDeviceIntf(IAddressingSpace &space, int offset)
        : ReadReg(space, offset), Reg(space, offset)
        {  }
    virtual ~ModernRomI2CDeviceIntf()  {  }

    // Methods
    virtual void    SetClock(int state)     {  Reg.Clock(state);    }
    virtual int     GetClock() const        {  return ReadReg.Clock();  }
    virtual void    SetData(int state)      {  Reg.Data(state);     }
    virtual int     GetData() const         {  return ReadReg.Data();   }

private:
    I2cReadRegister ReadReg;
    I2cRegister     Reg;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//===========================================================================
//  CLASS IUsesPmcEeprom -- User Interface to add in PMC config ROM support
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IUsesPmcEeprom
//---------------------------------------------------------------------------

IUsesPmcEeprom::IUsesPmcEeprom(IAddressingSpace & space, unsigned int addr,
                               int size)
    :  Intf(new RomI2CDeviceIntf(space, addr))
{
    Device = new PmcIdrom(*Intf, size, false);
}

//---------------------------------------------------------------------------
//  constructor for class IUsesPmcEeprom
//---------------------------------------------------------------------------

IUsesPmcEeprom::IUsesPmcEeprom(IAddressingSpace & space, unsigned int addr,
                               int size, EEpromIntfStyle /*style*/)
    :  Intf(new ModernRomI2CDeviceIntf(space, addr))
{
    Device = new PmcIdrom(*Intf, size, false);
}

//---------------------------------------------------------------------------
//  destructor for class IUsesPmcEeprom
//---------------------------------------------------------------------------

IUsesPmcEeprom::~IUsesPmcEeprom()
{
    delete Intf;    Intf = 0;
    delete Device;  Device = 0;
}

//------------------------------------------------------------------------------
//  IUsesPmcEeprom::Rom()  --   Return reference to PMC ROM
//------------------------------------------------------------------------------
PmcIdrom & IUsesPmcEeprom::Rom()
{
    return *Device;
}

//-----------------------------------------------------------------------------
//  IUsesPmcEeprom::AfterOpen() -- Behavior after opening board
//-----------------------------------------------------------------------------

void  IUsesPmcEeprom::AfterOpen()
{
	Device->LoadFromRom();
}


}  // end namespace Innovative



