// PmcEeprom_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef PmcEeprom_MbH
#define PmcEeprom_MbH

#include <PmcIdrom_Mb.h>
#include <HardwareRegister_Mb.h>
#include <string>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

enum EEpromIntfStyle { intfModernStyle };

//===========================================================================
//  CLASS IUsesPmcEeprom -- User Interface to add in PMC config ROM support
//===========================================================================

class IUsesPmcEeprom
{
public:
    IUsesPmcEeprom(IAddressingSpace & space, unsigned int addr, int size);
    IUsesPmcEeprom(IAddressingSpace & space, unsigned int addr, int size, EEpromIntfStyle style);
    virtual ~IUsesPmcEeprom();

    // Initialization
    void                AfterOpen();   
    //
    //  ROM load Methods (forwarded to Rom device)
    void   LoadFromRom()  {  Device->LoadFromRom(); }
    void   StoreToRom()  {  Device->StoreToRom(); }

    //
    //  Rom Data Properties
    std::string     Name() const
                        {  return Device->Name(); }
    void            Name(std::string name)
                        {  Device->Name(name); }
    std::string     Revision() const
                        {  return Device->Revision(); }
    void            Revision(std::string revision)
                        {  Device->Revision(revision); }
    unsigned int    SerialNumber() const
						{  return Device->SerialNumber(); }
    void            SerialNumber(unsigned int Num)
						{  Device->SerialNumber(Num); }
    unsigned int    PartNumber() const
                        {  return Device->PartNumber(); }
    void            PartNumber(unsigned int part)
                        {  Device->PartNumber(part); }
    unsigned int    DashNumber() const
                        {  return Device->DashNumber(); }
    void            DashNumber(unsigned int dash)
                        {  Device->DashNumber(dash); }
    std::string     DashVariant() const
                        {  return Device->DashVariant(); }
    void            DashVariant(const std::string & dashV)
                        {  Device->DashVariant(dashV); }
    unsigned int    RuggedizationNumber() const
                        {  return Device->RuggedizationNumber(); }
    void            RuggedizationNumber(unsigned int rugged)
                        {  Device->RuggedizationNumber(rugged); }
    unsigned int    EcoSubRevNumber() const
                        {  return Device->EcoSubRevNumber(); }
    void            EcoSubRevNumber(unsigned int eco_subrev)
                        {  Device->EcoSubRevNumber(eco_subrev); }
    void            ChipErase(int pattern)
                        {  Device->ChipErase(pattern); }
    bool            VerifyChipErased(int pattern)
                        {  return Device->VerifyChipErased(pattern); }
    std::string     PrintChipData()
                        {  return Device->PrintChipData(); }

    //
    //  Interface Functions
    PmcIdrom &                  Rom();

protected:
    // Data
    class I2CDeviceIntf *        Intf;
    class PmcIdrom *             Device;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
