// PmcJtagPort_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef PmcJtagPort_MbH
#define PmcJtagPort_MbH

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS PmcJtagPort  --  Innovative PMC module JTAG port abstraction
//===========================================================================

class PmcJtagPort
{
public:
    // Ctor
    PmcJtagPort()
        {  }
    virtual ~PmcJtagPort()
        {  }

    // Standard Properties
    virtual void            Enabled(bool state) {  Bit(1, state ? 1 : 0);  }
    virtual int             Enabled() const     {  return Bit(1) ? true : false;  }

    virtual void            Tms(int value)      {  Bit(0, value);  }
    virtual int             Tms() const         {  return Bit(0);  }

    virtual void            Tdi(int value)      {  Bit(4, value);  }
    virtual int             Tdi() const         {  return Bit(4);  }

    virtual void            Tck(int value)      {  Bit(3, value);  }
    virtual int             Tck() const         {  return Bit(3);  }

    virtual void            Port(int value) = 0;
    virtual int             Port() const = 0;

    // Standard Methods
    virtual void            Init()
        {  }
    virtual int             Tdo() const
        {  return (Port() & 0x20) ? 1 : 0;  }

    virtual int             Rev() const
        {  return ((Port()>>8) & 0xff);  }
    virtual int             HwRev() const
        {  return ((Port()>>16) & 0x0f);  }
    virtual int             Id() const
        {  return ((Port()>>20) & 0xfff);  }

protected:
    // Properties
    int             Ctl() const
        {  return FCtl;  }
    void            Ctl(int value)
        {  FCtl = value;  Port(value);  }

    int             Bit(int bit) const
        {  return Ctl() & (1 << bit);  }
    void            Bit(int bit, int value)
        {
        int mask = 1 << bit;
        Ctl((Ctl() & ~mask) | (value ? mask : 0));
        }

    // Constants
    mutable int                 FCtl;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
