// ProcessEvents_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef ProcessEvents_MbH
#define ProcessEvents_MbH

#include <DataTypes_Mb.h>
#include <Events_Mb.h>
#include <Buffer_Mb.h>
#include <string>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS ProcessProgressEvent  --  Fractional completion indicator
//===========================================================================

class ProcessProgressEvent : public OpenWire::Event
{
public:
    const int Percent;

    ProcessProgressEvent(int percent)
        :  Percent(percent)
        {
        }
};

//===========================================================================
// CLASS ProcessCompletionEvent  --  Process completion
//===========================================================================

class ProcessCompletionEvent : public OpenWire::Event
{
public:
    const int Result;

    ProcessCompletionEvent(int result=0)
        : Result(result)
        {
        }
};

//===========================================================================
// CLASS ProcessStatusEvent  --  In process status reporting
//===========================================================================

class ProcessStatusEvent : public OpenWire::Event
{
public:
    std::string Message;

    ProcessStatusEvent(const std::string & msg)
        :  Message(msg)
        {
        }
};

//===========================================================================
// CLASS ProcessTypedStatusEvent  --  In process status reporting
//===========================================================================

class ProcessTypedStatusEvent : public OpenWire::Event
{
public:
    enum MsgType { eAlways, eError, eWarning, eInfo };

    MsgType       Type;
    std::string   Message;

    ProcessTypedStatusEvent(const std::string & msg, MsgType type=eError)
        :  Type(type), Message(msg)
        {
        }
};

//===========================================================================
// CLASS ITypedStatusEventSender  --  Interface to send messages
//===========================================================================
//  Interface sends ProcessTypedStatusEvent messages

class ITypedStatusEventSender
{
public:
    ITypedStatusEventSender() : ErrorLevel(ProcessTypedStatusEvent::eError)
        {}
    //
    //  Notification Callback Event
    OpenWire::EventHandler<ProcessTypedStatusEvent>  OnMessage;
    //
    //  Signal message
    void  DeliverMessage(const std::string & msg,
                      ProcessTypedStatusEvent::MsgType type=ProcessTypedStatusEvent::eError);

    void  MessageErrorLevel(ProcessTypedStatusEvent::MsgType level)
            {  ErrorLevel = level; }

private:
    ProcessTypedStatusEvent::MsgType   ErrorLevel;

};

//===========================================================================
// CLASS SenderEvent  --  Basic sender event type
//===========================================================================

template <class T>
class SenderEvent : public OpenWire::Event
{
public:
    T Sender;

    SenderEvent(T sender)
        : Sender(sender)
        {
        }
};

//=============================================================================
//  CLASS StartBufferEvent  --
//=============================================================================

class StartBufferEvent : public OpenWire::NotifyEvent
{
public:
    const ii64 Size;

public:
    StartBufferEvent( ii64 size )
        : Size( size )
        {
        }
};

//===========================================================================
// CLASS SendBufferEvent  --  Buffer notification
//===========================================================================

class SendBufferEvent : public OpenWire::NotifyEvent
{
public:
    Buffer  Data;
    int     Channel;

    SendBufferEvent(const Buffer & data, int ch=0)
        : Data(data), Channel(ch)
        {
        }
};



#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
