// QuadiaDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <QuadiaDeviceMap_Mb.h>
#include <QuadiaMemoryMap_Mb.h>
#include <QuadiaDeviceDriver_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <Exception_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

namespace Innovative
{

//===========================================================================
//  CLASS QuadiaPciLogicVersionReg  --  Quadia PCI Logic ID info register
//===========================================================================

class QuadiaPciLogicVersionReg : public ReadOnlyRegister
{
    typedef ReadOnlyRegister inherited;

public:
    ReadOnlyRegisterBitGroup       Revision;
    ReadOnlyRegisterBitGroup       Pcb;

    QuadiaPciLogicVersionReg(AddressingSpace & space, unsigned int offset)
        : inherited(space, offset), Revision(*this, 16, 8), Pcb(*this, 24, 8)
    {  }
};

//===========================================================================
//  CLASS IUsesQuadiaPciLogicVersionInfo -- User Interface to user logic ID
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class IUsesQuadiaPciLogicVersionInfo
//---------------------------------------------------------------------------

IUsesQuadiaPciLogicVersionInfo::IUsesQuadiaPciLogicVersionInfo(AddressingSpace & space,
    unsigned int addr)
    :  Reg(new QuadiaPciLogicVersionReg(space, addr))
{
}

//---------------------------------------------------------------------------
//  destructor for class IUsesQuadiaPciLogicVersionInfo
//---------------------------------------------------------------------------

IUsesQuadiaPciLogicVersionInfo::~IUsesQuadiaPciLogicVersionInfo()
{
    delete Reg;    Reg = 0;
}

//---------------------------------------------------------------------------
//  IUsesQuadiaPciLogicVersionInfo::PciLogicRevision()
//---------------------------------------------------------------------------

short IUsesQuadiaPciLogicVersionInfo::PciLogicRevision() const
{
    return Reg->Revision;
}

//---------------------------------------------------------------------------
//  IUsesQuadiaPciLogicVersionInfo::PciLogicPcb()
//---------------------------------------------------------------------------

short IUsesQuadiaPciLogicVersionInfo::PciLogicPcb() const
{
    return Reg->Pcb;
}

//---------------------------------------------------------------------------
//  IUsesQuadiaPciLogicVersionInfo::PciLogicRawValue()
//---------------------------------------------------------------------------

unsigned int IUsesQuadiaPciLogicVersionInfo::PciLogicRawValue() const
{
    return Reg->Value();
}



//===========================================================================
//  CLASS QuadiaDeviceMap  -- Registers for Quadia Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class QuadiaDeviceMap
//---------------------------------------------------------------------------

using namespace Quadia_MemoryMap;

const unsigned int  QuadiaVariantSSID = 0x0000;
const unsigned int  DuetVariantSSID   = 0x0001;

//---------------------------------------------------------------------------
//  constructor for class QuadiaDeviceMap
//---------------------------------------------------------------------------

QuadiaDeviceMap::QuadiaDeviceMap()
    : IUsesVirtexJtagLoader(PortMemory, mmJtag),
      IUsesQuadiaPciLogicVersionInfo(PortMemory, hwmStatusReg),
      IUsesQuadiaLogicTemperature(PortMemory, mmThermalSensor),
      Driver(new QuadiaDeviceDriver),
      StatusReg(PortMemory, hwmStatusReg),
      ControlReg(PortMemory, hwmControlReg),
      Pll0ControlReg(PortMemory, hwmPll0Ctl),
      Pll0DataReg(PortMemory, hwmPll0Data),
      Pll1ControlReg(PortMemory, hwmPll1Ctl),
      Pll1DataReg(PortMemory, hwmPll1Data),
      FVariant(QuadiaVariantSSID)
{
    FFpga[0] = new IUsesVirtexFpgaLoader(new VirtexFpgaLoader(PortMemory, hwmFpga0Data, hwmFpga0Ctl, hwmFpga0Status));
    FFpga[1] = new IUsesVirtexFpgaLoader(new VirtexFpgaLoader(PortMemory, hwmFpga1Data, hwmFpga1Ctl, hwmFpga1Status));
}

//---------------------------------------------------------------------------
//  destructor for class QuadiaDeviceMap
//---------------------------------------------------------------------------

QuadiaDeviceMap::~QuadiaDeviceMap()
{
    Close();

    for (int i = 0; i < 2; ++i)
        {
        delete FFpga[i];
        FFpga[i] = 0;
        }

    delete Driver;
    Driver = 0;
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::BoardCount() --  Query # enumerated boards
//---------------------------------------------------------------------------

size_t QuadiaDeviceMap::BoardCount()
{
    return Driver->BoardCount();
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Open()
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::Open()
{
    if( Driver->Opened() )
        return;

    //Devices.clear();
    //InnovativeKernel::DeviceSignature Signature(0x1303, 0x0009);
    //Devices = Pci.Build(Signature);

    Driver->Open();
    //
    //  Set the variant
    DetermineBoardVariant();
    //
    //  Link register definitions to device addresses
    PortMemory.Link( ( ii32 *)Driver->PortMemoryAddress() );
    DualPortMemory.Link( ( ii32 *)Driver->DualPortMemoryAddress() );

    InitializeDdr();
    ConfigurePll();
    Reset();
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Close()
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::Close()
{
    Driver->Close();
}

//------------------------------------------------------------------------------
//  QuadiaDeviceMap::DetermineBoardVariant() -- Load variant of baseboard found
//------------------------------------------------------------------------------

void  QuadiaDeviceMap::DetermineBoardVariant()
{
    //  Variant is encoded in SSID.
    unsigned int SSID = Driver->PciConfig().SubsystemId();
    if (SSID == DuetVariantSSID)
        FVariant = DuetVariantSSID;
    else
        FVariant = QuadiaVariantSSID;
// HACK HACK HACK - everyting a duet
//        FVariant = DuetVariantSSID;

}


//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  QuadiaDeviceMap::Target()
{
    return Driver->Target();
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::Target( int value )
{
    Driver->Target(value);
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Reset()
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::Reset()
{
    // Reset both FPGAs
    ControlReg.LogicInReset(2, true);
    ControlReg.LogicInReset(2, false);
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Led()
//---------------------------------------------------------------------------

bool  QuadiaDeviceMap::Led()
{
    return ControlReg.Led;
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Led()
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::Led( bool value )
{
    ControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::PeekDpram() - read a 32 bit value from dpram-space
//---------------------------------------------------------------------------

ii32   QuadiaDeviceMap::PeekDpram(int offset)
{
    ii32 * LogicBasePtr = ( ii32 *)Driver->DualPortMemoryAddress();
    return LogicBasePtr[offset];
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::PokeDpram() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::PokeDpram(int offset, ii32 value)
{
    ii32 * LogicBasePtr = ( ii32 *)Driver->DualPortMemoryAddress();
    LogicBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32   QuadiaDeviceMap::PeekPort(int offset)
{
    ii32 * PortBasePtr = ( ii32 *)Driver->PortMemoryAddress();
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = ( ii32 *)Driver->PortMemoryAddress();
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::Error() -- throw error string
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::Error( std::string msg )
{
    throw Innovative::MalibuException(msg);
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::ConfigurePll() --
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::ConfigurePll()
{
    // Configure PLL
    const int PllDataRegValue = 0x341b;

    Pll0ControlReg.Value(0);
    Pll0DataReg.Value(PllDataRegValue);
    Pll0ControlReg.Enable.Set();
    Pll0ControlReg.Reset.Set();
    Pll0ControlReg.Reset.Clear();

    //  Only initialize second PLL on non-Duet variants
    if (FVariant != DuetVariantSSID)
        {
        Sleep(1);
        Pll1ControlReg.Value(0);
        Pll1DataReg.Value(PllDataRegValue);
        Pll1ControlReg.Enable.Set();
        Pll1ControlReg.Reset.Set();
        Pll1ControlReg.Reset.Clear();
        }

    Sleep(5);
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::InitializeDdr() --
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::InitializeDdr()
{
    //  Duets do not have the shared DDR Ram
    if (FVariant == DuetVariantSSID)
        return;

    // Initialize DDR
    ControlReg.DllReset.Set();
    ControlReg.DllReset.Clear();
    ControlReg.DdrClkEnable.Set();
    Sleep(1);
    ControlReg.DdrRefreshEnable.Set();
    ControlReg.DdrInit.Set();
}

//---------------------------------------------------------------------------
//  QuadiaDeviceMap::DeviceInfo() --
//---------------------------------------------------------------------------

InnovativeKernel::DeviceEntry & QuadiaDeviceMap::DeviceInfo()
{
    return Driver->PciConfig();
}


//===========================================================================
//  CLASS QuadiaDeviceMap::ControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  QuadiaDeviceMap::ControlRegister::LogicInReset() --
//---------------------------------------------------------------------------

void  QuadiaDeviceMap::ControlRegister::LogicInReset(int which, bool state)
{
    int mask;

    switch (which)
        {
        case 0:     mask = 1;       break;
        case 1:     mask = 2;       break;
        default:    mask = 3;       break;
        }

    int value = FpgaReset.Value() & ~mask;
    value |= state ? 0 : mask;
    FpgaReset.Value(value);
}

} // namespace Innovative


