// QuadiaDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef QuadiaDeviceMap_MbH
#define QuadiaDeviceMap_MbH


#include <vector>
#include <ProcessEvents_Mb.h>
#include <DeviceDriver_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <HardwareRegister_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>
#include <HardwareRegister_Mb.h>
#include <LogicTemperatureIntf_Mb.h>
#include <SystemSupport_Mb.h>

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class QuadiaDeviceDriver;
class Event;
class PmcBuffer;

//===========================================================================
//  CLASS IUsesQuadiaLogicTemperature -- User Interface to user logic ID
//===========================================================================

class IUsesQuadiaLogicTemperature : public DualLogicTemperatureIntf
{
public:
	IUsesQuadiaLogicTemperature(AddressingSpace & space, unsigned int offset)
    	: Status(space, offset), Setpoint(space, offset)
        {  }
	virtual ~IUsesQuadiaLogicTemperature()
        {  }

	//
	//  Interface Functions
	virtual float           LogicTemperature(int idx) const
        {
        int result = ((idx ? Status.Temp1.Value() : Status.Temp0.Value()) << 24) >> 24;
        return static_cast<float>(result);
        }

	virtual void 			LogicFailureTemperature(int idx, float value)
        {
        if (idx)
            Setpoint.Temp1.Value(static_cast<ii32>(value));
        else
            Setpoint.Temp0.Value(static_cast<ii32>(value));
        }
	virtual float 			LogicFailureTemperature(int idx) const
        {  return static_cast<float>((idx ? Setpoint.Temp1.Value() : Setpoint.Temp0.Value()));  }

	virtual bool            Failed(int idx) const
        {  return (idx ? Status.Failed1.Value() : Status.Failed0.Value());  }

protected:
    // Setters
	virtual void 			LogicWarningTemperature(int/*idx*/, float /*value*/)
        {  }
	virtual float 			LogicWarningTemperature(int/*idx*/) const
        {  return 75.0f;  }

    class StatusReg : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup       Temp0;
        ReadOnlyRegisterBitGroup       Temp1;
        ReadOnlyRegisterBit            Failed0;
        ReadOnlyRegisterBit            Failed1;

        StatusReg(AddressingSpace & space, unsigned int offset)
            : inherited(space, offset), Temp0(*this, 0, 8), Temp1(*this, 8, 8),
              Failed0(*this, 16), Failed1(*this, 17)
        {  }
    };

    class SetpointReg : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup       Temp0;
        RegisterBitGroup       Temp1;

        SetpointReg(AddressingSpace & space, unsigned int offset)
            : inherited(space, offset), Temp0(*this, 0, 8), Temp1(*this, 8, 8)
        {  }
    };

	// Registers
	StatusReg           Status;
	SetpointReg         Setpoint;
};

//===========================================================================
//  CLASS IUsesQuadiaPciLogicVersionInfo -- User Interface to user logic ID
//===========================================================================

class IUsesQuadiaPciLogicVersionInfo : public PciLogicVersionInfoIntf
{
public:
    IUsesQuadiaPciLogicVersionInfo(AddressingSpace & space, unsigned int addr);
    virtual ~IUsesQuadiaPciLogicVersionInfo();

    //
    //  Interface Functions
    virtual short           PciLogicRevision() const;
    virtual short           PciLogicFamily() const          {  return 0;  }
    virtual short           PciLogicPcb() const;
    virtual short           PciLogicType() const            {  return 0;  }
    virtual short           PciLogicGeographicAddress() const  {  return 0;  }

    virtual unsigned int    PciLogicRawValue() const;


protected:
    // Data
    class QuadiaPciLogicVersionReg *  Reg;
};

//===========================================================================
//  CLASS QuadiaDeviceMap  -- Register & low level support for Quadia
//===========================================================================

class QuadiaDeviceMap : public IUsesVirtexJtagLoader,
                        public IUsesQuadiaPciLogicVersionInfo,
					    public IUsesQuadiaLogicTemperature
{
    friend class Quadia;

public:
    //
    //  Member Functions
    QuadiaDeviceMap();
    virtual ~QuadiaDeviceMap();

    size_t        BoardCount();
    virtual void  Open();
    virtual void  Close();

    int   Target();
    void  Target(int target);

    bool  Led();
    void  Led(bool value);

    ii32  PeekDpram(int offset);
    void  PokeDpram(int offset, ii32 value);
    ii32  PeekPort(int offset);
    void  PokePort(int offset, ii32 value);

    void  Reset();
    void  ConfigurePll();
    void  InitializeDdr();

    InnovativeKernel::DeviceEntry &   DeviceInfo();

    IUsesVirtexFpgaLoader & Fpga(int index)
        {  return *FFpga[index];  }
    //
    //  Addressing Spaces
    AddressingSpace                    PortMemory;
    AddressingSpace                    DualPortMemory;

protected:
    //===========================================================================
    //  CLASS ControlRegister
    //===========================================================================

    class ControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Led;
        RegisterBit           DllReset;
        RegisterBit           DdrInit;
        RegisterBit           DdrClkEnable;
        RegisterBit           DdrRefreshEnable;
        RegisterBitGroup      FpgaReset;        //  Overlapping
        RegisterBit           GlobalTrigger;

    public:
        ControlRegister(AddressingSpace &space,
            int offset)
        :  inherited( space, offset ), Led(*this, 0), DllReset(*this, 1),
           DdrInit(*this, 2), DdrClkEnable(*this, 3), DdrRefreshEnable(*this, 4),
           FpgaReset(*this, 5, 2), GlobalTrigger(*this, 7)
            {
            }

        void  LogicInReset(int which, bool state);
    };

    //===========================================================================
    //  CLASS PllControlRegister
    //===========================================================================

    class PllControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Reset;
        RegisterBit           Enable;

    public:
        PllControlRegister(AddressingSpace &space,
            int offset)
        :  inherited( space, offset ), Reset(*this, 0), Enable(*this, 1)
            {
            }

    };

    //===========================================================================
    //  CLASS StatusRegister
    //===========================================================================

    class StatusRegister : public Register
    {
        typedef Register inherited;

    public:
        ReadOnlyRegisterBit         DllsLocked;
        ReadOnlyRegisterBit         PllBusy;
        ReadOnlyRegisterBit         PllLoadEnable;
        ReadOnlyRegisterBitGroup    Firmware;
        ReadOnlyRegisterBitGroup    Pcb;

    public:
        StatusRegister(AddressingSpace &space,
            int offset)
        :  inherited( space, offset ),
           DllsLocked(*this, 0), PllBusy(*this, 1), PllLoadEnable(*this, 2),
           Firmware(*this, 16, 8), Pcb(*this, 24, 8)
            {
            }
    };

    //===========================================================================
    //  CLASS FpgaConfigControlRegister
    //===========================================================================

    class FpgaConfigControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Enable;
        RegisterBit           Reset;
        RegisterBit           Prog;

    public:
        FpgaConfigControlRegister(AddressingSpace &space,
            int offset)
        :  inherited( space, offset ), Enable(*this, 0), Reset(*this, 1),
           Prog(*this, 2)
            {
            }
    };

    //===========================================================================
    //  CLASS FpgaConfigStatusRegister
    //===========================================================================

    class FpgaConfigStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        ReadOnlyRegisterBit   Done;
        ReadOnlyRegisterBit   Init;

    public:
        FpgaConfigStatusRegister(AddressingSpace &space,
            int offset)
        :  inherited( space, offset ), Done(*this, 0), Init(*this, 1)
            {
            }
    };

    //===========================================================================
    //  CLASS VirtexFpgaLoader
    //===========================================================================

    class VirtexFpgaLoader : public FpgaLoadBase
    {
    public:
        VirtexFpgaLoader(AddressingSpace & space,
                         unsigned int data_addr,
                         unsigned int control_addr,
                         unsigned int status_addr )
            : Data(space, data_addr),
              Control(space, control_addr),
              Status(space, status_addr)
            {  }

        // Methods
        virtual void Program(bool state)
            {
            // Drive PROG pin
            Control.Prog.Value(state);
            // Enable the logic load interface
            Control.Enable.Set();
            }
        virtual void Reset()
            {
            // Reset the logic
            Sleep(1);
            Control.Reset.Set();
            Control.Reset.Clear();

            // Disable the logic load interface
            Control.Enable.Clear();
            }

        virtual bool IsInit()
            {  return Status.Init.Value() ? false : true;  }

        virtual bool IsDone()
            {  return Status.Done.Value() ? true : false;  }

        virtual void Load(char * buffer, size_t count)
            {
            for (size_t i = 0; i < count; ++i)
                Data = *buffer++;
            }


    private:
        Register                           Data;
        FpgaConfigControlRegister          Control;
        FpgaConfigStatusRegister           Status;
    };

    //
    //  Member Data
    QuadiaDeviceDriver *               Driver;
    IUsesVirtexFpgaLoader *            FFpga[2];
    StatusRegister                     StatusReg;
    ControlRegister                    ControlReg;
    PllControlRegister                 Pll0ControlReg;
    ShadowRegister                     Pll0DataReg;
    PllControlRegister                 Pll1ControlReg;
    ShadowRegister                     Pll1DataReg;

    //InnovativeKernel::DeviceEnum       Pci;
    //int                                CurrentDevice;
    //InnovativeKernel::DeviceEntryArray Devices;

    unsigned int                       FVariant;

    // Methods
    void                     Error( std::string msg );
    void                     DetermineBoardVariant();

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif
//---------------------------------------------------------------------------
#endif
