// Quadia_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Quadia_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <HpiEngine_Mb.h>
#include <Exception_Mb.h>
#include <QuadiaDeviceDriver_Mb.h>


namespace Innovative
{
//===========================================================================
//  CLASS Quadia  -- Racal Pmc Board object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Quadia
//---------------------------------------------------------------------------

Quadia::Quadia()
{
}

//---------------------------------------------------------------------------
//  destructor for class Quadia
//---------------------------------------------------------------------------

Quadia::~Quadia()
{
}

//---------------------------------------------------------------------------
//  Quadia::BoardCount() -- Query # installed baseboards
//---------------------------------------------------------------------------

size_t Quadia::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  Quadia::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  Quadia::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
    //
    //  Remove Logics from Reset
    LogicInReset(lsBothFpga, false);
}

//---------------------------------------------------------------------------
//  Quadia::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  Quadia::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  Quadia::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  Quadia::Reset()
{
    Device.Reset();
}

//---------------------------------------------------------------------------
//  Quadia::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Quadia::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  Quadia::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Quadia::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  Quadia::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  Quadia::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  Quadia::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  Quadia::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  Quadia::GlobalTrigger() -- Set/clear global trigger line
//---------------------------------------------------------------------------

void  Quadia::GlobalTrigger( bool value )
{
    Device.ControlReg.GlobalTrigger.Value(value);
}

//---------------------------------------------------------------------------
//  Quadia::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  Quadia::LogicInReset(LogicSelect which, bool state)
{
    int code = static_cast<int>(which);
    Device.ControlReg.LogicInReset(code, state);
}

//---------------------------------------------------------------------------
//  Quadia::Dpram() -- Read from dual port memory region of board
//---------------------------------------------------------------------------

ii32  Quadia::Dpram(int offset )
{
    //
    //  Duet has no DPRAM.
    if (Variant()==vDuet)
        return 0;
    else
        return Device.PeekDpram(offset);
}

//---------------------------------------------------------------------------
//  Quadia::Dpram() -- Write to dual port memory region of board
//---------------------------------------------------------------------------

void  Quadia::Dpram( int offset, ii32 value )
{
    //
    //  Duet has no DPRAM.
    if (Variant()!=vDuet)
        Device.PokeDpram(offset, value);
}

//---------------------------------------------------------------------------
//  Quadia::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  Quadia::Port(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  Quadia::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  Quadia::Port( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  Quadia::Logic() -- Return reference to FPGA Loader Interface
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & Quadia::Logic(int index)
{
    //
    //  Duet has only 1 logic, so always use logic 0.
    if (Variant()==vDuet)
        return Device.Fpga(0);
    else
        return Device.Fpga(index);
}

//---------------------------------------------------------------------------
//  Quadia::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & Quadia::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Quadia::PortAddress() --
//---------------------------------------------------------------------------

ii32 * Quadia::PortAddress()
{
    return Device.Driver->PortMemoryAddress();
}

//---------------------------------------------------------------------------
//  Quadia::PortPhysicalAddress() --
//---------------------------------------------------------------------------

ii32 * Quadia::PortPhysicalAddress()
{
    return reinterpret_cast<ii32 *>(Device.Driver->PortPhysicalAddress());
}

//---------------------------------------------------------------------------
//  Quadia::PortSizeWords() --
//---------------------------------------------------------------------------

ii32 Quadia::PortSizeWords()
{
    return Device.Driver->PortSizeWords()/sizeof(int);
}
//---------------------------------------------------------------------------
//  Quadia::DpramAddress() --
//---------------------------------------------------------------------------

ii32 * Quadia::DpramAddress()
{
    //
    //  Duet has no DPRAM.
    if (Variant()==vDuet)
        return 0;
    else
        return Device.Driver->DualPortMemoryAddress();
}

//---------------------------------------------------------------------------
//  Quadia::DpramPhysicalAddress() --
//---------------------------------------------------------------------------

ii32 * Quadia::DpramPhysicalAddress()
{
    //
    //  Duet has no DPRAM.
    if (Variant()==vDuet)
        return 0;
    else
        return reinterpret_cast<ii32 *>(Device.Driver->DualPortPhysicalAddress());
}

//---------------------------------------------------------------------------
//  Quadia::DpramSizeWords() --
//---------------------------------------------------------------------------

ii32 Quadia::DpramSizeWords()
{
    //
    //  Duet has no DPRAM.
     if (Variant()==vDuet)
        return 0;
     else
        return Device.Driver->DualPortSizeWords()/sizeof(int);
}

//---------------------------------------------------------------------------
//  Quadia::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

DualLogicTemperatureIntf & Quadia::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Quadia::DeviceInfo() --
//---------------------------------------------------------------------------

InnovativeKernel::DeviceEntry & Quadia::DeviceInfo()
{
    return Device.DeviceInfo();
}

//---------------------------------------------------------------------------
//  Quadia::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & Quadia::PciLogicVersion()
{
    return Device;
}


}  // namespace Innovative
