// QuixoteDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <QuixoteDeviceMap_Mb.h>
#include <QuixoteMemoryMap_Mb.h>
#include <QuixoteDeviceDriver_Mb.h>
#include <Exception_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------

namespace  Innovative
{
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

//==============================================================================
//  CLASS QuixoteFpaLoader -- Interface to FPA register
//==============================================================================

class QuixoteFpaLoader : public FpaLoadBase
{
    //--------------------------------------------------------
    //   CLASS QuixoteFpaLoader::FpaControlRegister
    //
    class FpaControlRegister : public Register
    {
    public:
        RegisterBitGroup            Mode;
        RegisterBit                 Clk;
        RegisterBit                 Out;
        ReadOnlyRegisterBit         In;
        ReadOnlyRegisterBitGroup    FirmwareRevision;
        ReadOnlyRegisterBitGroup    PcbRevision;
        ReadOnlyRegisterBitGroup    Id;

    public:
        FpaControlRegister(AddressingSpace &space, int offset)
            :  Register(space, offset),
               Mode(*this, 0, 3),
               Clk(*this, 3),
               Out(*this, 4),
               In(*this, 5),
               FirmwareRevision(*this, 8, 8),
               PcbRevision(*this, 16, 4),
               Id(*this, 20, 12)
            {  }
    };
public:
    //
    //  Member Functions
    QuixoteFpaLoader(AddressingSpace & space)
        : Reg(space, Quixote_MemoryMap::mmFpaReg)
        {  }

    // Read/Write data
    virtual bool            Read()
        {  return Reg.In ? 1 : 0;  }
    virtual void            Write(bool state)
        {  Reg.Out = state;  }
    // Read/write clock state
    virtual void            Clock(bool state)
        {  Reg.Clk = state;  }

    // Mode set
    virtual void            Mode(int mode)
        {  Reg.Mode = mode;  }
    // Device select
    virtual void            Device(int /*device*/)
        {  }
    // Firmware revision readback
    virtual int             FirmwareRevision()
        {  return Reg.FirmwareRevision;  }
    // Pcb revision readback
    virtual int             PcbRevision()
        {  return Reg.PcbRevision;  }
    // Id readback
    virtual int             Id()
        {  return Reg.Id;  }

private:
    // Data
    FpaControlRegister      Reg;
};
#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

//===========================================================================
//  CLASS QuixoteDeviceMap  --
//===========================================================================

using namespace Quixote_MemoryMap;

//---------------------------------------------------------------------------
//  constructor for class QuixoteDeviceMap
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)		
#endif

QuixoteDeviceMap::QuixoteDeviceMap()
    :  IUsesVirtexFpgaLoader(PortMemory, mmFpgaData, mmFpgaControl),
       Driver(new QuixoteDeviceDriver),
       ControlReg( PortMemory, mmControlStatusReg ),
       FFpa(new QuixoteFpaLoader(PortMemory))
{
    EepromLoader = new FpaLoadThread(*FFpa);
}

#ifdef _MSC_VER		
	#pragma warning(pop)
#endif
//---------------------------------------------------------------------------
//  destructor for class QuixoteDeviceMap
//---------------------------------------------------------------------------

QuixoteDeviceMap::~QuixoteDeviceMap()
{
    EepromLoader->Shutdown();
    delete EepromLoader;
    EepromLoader = 0;

	Close();

    delete Driver;   Driver = 0;
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::BoardCount() --
//---------------------------------------------------------------------------
 
size_t QuixoteDeviceMap::BoardCount()
{
    return Driver->BoardCount();
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::Open() --
//---------------------------------------------------------------------------
 
void  QuixoteDeviceMap::Open()
{
    if (Driver->Opened())
        return;

    Driver->Open();
    PortMemory.Link(Driver->PortMemoryAddress());

    ClearControlRegister();
    EepromLoader->Initialize();
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::Close() --
//---------------------------------------------------------------------------
 
void  QuixoteDeviceMap::Close()
{
    Driver->Close();
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::ClearControlRegister() --
//---------------------------------------------------------------------------
 
void  QuixoteDeviceMap::ClearControlRegister()
{
    ControlReg.BaseboardReset.Set();
    Sleep(1);
    DspInReset(false);
}

//------------------------------------------------------------------------------
//  QuixoteDeviceMap::Led() --
//------------------------------------------------------------------------------
 
bool  QuixoteDeviceMap::Led()
{
    return ControlReg.Led;
}

//------------------------------------------------------------------------------
//  QuixoteDeviceMap::Led() --
//------------------------------------------------------------------------------
 
void  QuixoteDeviceMap::Led(bool value)
{
    ControlReg.Led = value;
}

//------------------------------------------------------------------------------
//  QuixoteDeviceMap::DspInReset() --
//------------------------------------------------------------------------------

bool  QuixoteDeviceMap::DspInReset()
{
    return ! ControlReg.DSPInReset;
}

//------------------------------------------------------------------------------
//  QuixoteDeviceMap::DspInReset() --
//------------------------------------------------------------------------------

void  QuixoteDeviceMap::DspInReset(bool value)
{
    ControlReg.DSPInReset = ! value;
}

//------------------------------------------------------------------------------
//  QuixoteDeviceMap::Target() --
//------------------------------------------------------------------------------

int  QuixoteDeviceMap::Target()
{
    return Driver->Target();
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::Target() --
//------------------------------------------------------------------------------

void  QuixoteDeviceMap::Target(int target)
{
    Driver->Target(target);
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::Reset() --
//------------------------------------------------------------------------------
 
void  QuixoteDeviceMap::Reset()
{
    ControlReg.BaseboardReset.Clear();
    Sleep(20);
    ControlReg.BaseboardReset.Set();
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::Error() --
//------------------------------------------------------------------------------
 
void  QuixoteDeviceMap::Error(std::string & message)
{
    throw Innovative::MalibuException(message);
}

//---------------------------------------------------------------------------
//  QuixoteDeviceMap::DeviceInfo() --
//---------------------------------------------------------------------------

InnovativeKernel::DeviceEntry & QuixoteDeviceMap::DeviceInfo()
{
    return Driver->PciConfig();
}
 
//===========================================================================
//  CLASS ControlRegister  --  
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class QuixoteDeviceMap::ControlRegister
//------------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif 
QuixoteDeviceMap::ControlRegister::ControlRegister(AddressingSpace&  space,
    int offset) 
    :  ShadowRegister(space, offset, 0x28), AllowBusmaster(*this, 0), Led(*this, 1),
       BaseboardReset(*this, 3), HostSystemReady(*this, 4),
       DSPInReset(*this, 5), BusMasterInReset(*this, 6)
{
}

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif
} // Innovative
