// MappedDataSet_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <IniFile_Mb.h>
#include <RamDataSet_Mb.h>
#include <algorithm>
#include <cstring>

namespace Innovative
{

//===========================================================================
// Class RamDataSet
//===========================================================================

//---------------------------------------------------------------------------
// RamDataSet::RamDataSet() -- Constructor
//---------------------------------------------------------------------------

RamDataSet::RamDataSet()
	: Position(ii64(0)), FSize(0), FMemory(0)
{
	  FFileName = "Dsp RAM";
}

//---------------------------------------------------------------------------
// RamDataSet Destructor
//---------------------------------------------------------------------------

RamDataSet::~RamDataSet()
{
	ImageClose();
}

//---------------------------------------------------------------------------
// RamDataSet::Memory() -- Specify memory image location
//---------------------------------------------------------------------------

void RamDataSet::Memory(char *memory)
{
	if (FEnabled)
        return;

    FMemory = memory;
	Position = ii64(0);
    
    Init();        
}


//---------------------------------------------------------------------------
// RamDataSet::ImageReadIniFile() -- Read set characteristics from INI file
//---------------------------------------------------------------------------

void RamDataSet::ImageReadIniFile(IniFile & Ini)
{
	FMemory = reinterpret_cast<char*>(Ini.Read("Memory", static_cast<size_t>(0)));
}

//---------------------------------------------------------------------------
// RamDataSet::ImageWriteIniFile() -- Write set characteristics to INI file
//---------------------------------------------------------------------------

void RamDataSet::ImageWriteIniFile(IniFile & Ini)
{
    size_t value = reinterpret_cast<size_t>(FMemory);
    Ini.Write("Memory", value);
}

//---------------------------------------------------------------------------
// RamDataSet::ImageOpen() -- Open data image 
//---------------------------------------------------------------------------

bool RamDataSet::ImageOpen()
{
	Position = 0;
	return FMemory ? true : false;
}

//---------------------------------------------------------------------------
// RamDataSet::ImageClose() -- Close data image 
//---------------------------------------------------------------------------

void RamDataSet::ImageClose()
{
    Position = ii64(0);
}

//---------------------------------------------------------------------------
// RamDataSet::ImageFetch() -- Read sample from disk file
//---------------------------------------------------------------------------

void RamDataSet::ImageFetch(Subsample &sample)
{
    if (Position < FSize)
        {
        char * pSample = sample.asString();
		memcpy(pSample, &FMemory[(unsigned int)Position], FSubsampleSize);
		CorrectEndian(pSample);
		sample.Type(static_cast<enum Subsample::SampleType>(FType));
		Position += FSubsampleSize;
		}
}

//---------------------------------------------------------------------------
// RamDataSet::ImageStore() -- Write sample to disk file
//---------------------------------------------------------------------------

void RamDataSet::ImageStore(Subsample & sample)
{
	if (FReadOnly)
		return;

	if (Position < FSize)
		{
		char * pSample = sample.asString();
		CorrectEndian(pSample);
		memcpy(&FMemory[static_cast<unsigned int>(Position)], pSample, FSubsampleSize);
		Position += FSubsampleSize;
		}
}

//---------------------------------------------------------------------------
// RamDataSet::ImageFlush() -- Write dirty cache to disk
//---------------------------------------------------------------------------

void RamDataSet::ImageFlush()
{
}

//---------------------------------------------------------------------------
// RamDataSet::ImageSkip() -- Skip forward n bytes into disk file
//---------------------------------------------------------------------------

void RamDataSet::ImageSkip(ii64 offset)
{
    Position += offset;
}

//---------------------------------------------------------------------------
// RamDataSet::ImageSeek() -- Seek to byte offset in disk file
//---------------------------------------------------------------------------

void RamDataSet::ImageSeek(ii64 offset)
{
    Position = offset;
}

//---------------------------------------------------------------------------
// RamDataSet::ImageSize() -- Return size of disk file
//---------------------------------------------------------------------------

ii64 RamDataSet::ImageSize() const
{
    return FSize;
}

//---------------------------------------------------------------------------
// RamDataSet::ImageSize() -- Return size of disk file
//---------------------------------------------------------------------------

void RamDataSet::ImageSize(ii64 size)
{
    FSize = size;
}

} // namespace Innovative

