// RamLogger_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
 
#include <RamLogger_Mb.h>
#include <ProcessEvents_Mb.h>
#include <Thread_Mb.h>
#include <algorithm>
#include <DriveIo_Mb.h>
#include <cstring>

using namespace std;

namespace   Innovative
{

//========================================================================
//  CLASS PostProcessingThread --
//========================================================================

class PostProcessingThread : public Thread
{
    typedef Thread inherited;
    
public:
    // Ctors
    PostProcessingThread(RamLogger * owner)
        : inherited("PostProcessingThread"), Owner(owner)
        {
        Resume();
        }
    virtual ~PostProcessingThread()
        {  }

    // Member Fcns
    void Process()
        {  eWork.Set();  }

protected:
    virtual void    Init()
        {
        inherited::Init();
        AddObject(eWork, sWork);
        }
    virtual void    Execute();

private:
    // Fields
    RamLogger *                 Owner;

    // Data
    enum { sWork=inherited::sLast, sLast };
    Innovative::Event           eWork;
};

//---------------------------------------------------------------------------
//  PostProcessingThread::Execute() --
//---------------------------------------------------------------------------

void PostProcessingThread::Execute()
{
    while (!Terminated())
        {
        int result = WaitFor();
        if (result == sTerminate)
            continue;

        eWork.Clear();

        DriveIo file(Owner->FFileName);
        file.Create();

        ii64 logged = Owner->FLogged;

        if (logged > 100)
            {
            ii64 chunk = logged / 100;
            ii64 residual = logged;
            char * p = &Owner->image[0];
            unsigned int percent = 0;
            while (residual)
                {
                unsigned int portion = static_cast<unsigned int>(std::min(chunk, residual));
                unsigned int wrote = file.Write(p, portion);
                p += wrote;
                residual -= portion;
                ProgressEvent e(percent);
                Owner->OnProgress.Execute(e);
                ++percent;
                percent = std::min(percent, 100u);
                }
            }
        else
            {
            file.Write(&Owner->image[0], static_cast<unsigned int>(logged));
            }

        file.Close();
        OpenWire::NotifyEvent e;
        Owner->OnStop.Execute(e);
        }
}

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS RamLogger  --
//=============================================================================
//---------------------------------------------------------------------------
// RamLogger::RamLogger() --
//---------------------------------------------------------------------------

RamLogger::RamLogger()
    : FLogged(0),
      FCeiling(0),
      FFileName(""),
      FEnabled(true),
      image(0),
      Task(new PostProcessingThread(this))
{
}

//---------------------------------------------------------------------------
// RamLogger::RamLogger() --
//---------------------------------------------------------------------------

RamLogger::RamLogger(const RamLogger & other)
    : FLogged(0),
      FCeiling(other.FCeiling),
      FFileName(other.FFileName),
      FEnabled(other.FEnabled),
      image(0),
      Task(new PostProcessingThread(this))
{
    OnProgress = other.OnProgress;
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// RamLogger::~RamLogger() --
//---------------------------------------------------------------------------

RamLogger::~RamLogger()
{
    Task->Shutdown();
    delete Task;
    Task = 0;

    delete [] image;
    image = 0;
}

//---------------------------------------------------------------------------
//  RamLogger::operator = --
//---------------------------------------------------------------------------

RamLogger & RamLogger::operator = (const RamLogger & other)
{
    FCeiling = other.FCeiling;
    FEnabled = other.FEnabled;
    FFileName = other.FFileName;

    OnProgress = other.OnProgress;

    return *this;
}

//---------------------------------------------------------------------------
//  RamLogger::Log() --
//---------------------------------------------------------------------------

void RamLogger::Log(Buffer &buffer)
{
	Log(buffer.ConstData(), static_cast<unsigned int>(buffer.SizeInInts()));
}

//---------------------------------------------------------------------------
//  RamLogger::LogWithHeader() --
//---------------------------------------------------------------------------

void RamLogger::LogWithHeader( Buffer &buffer )
{
    //  Log Header, then Data
    Log(buffer.ConstHeader(), static_cast<unsigned int>(buffer.HeaderSizeInInts()));
	Log(buffer);
}

//---------------------------------------------------------------------------
//  RamLogger::Log() --
//---------------------------------------------------------------------------

void RamLogger::Log(const int * data, unsigned int size_in_ints)
{
    unsigned int bytes = size_in_ints * sizeof(int);
	DoLogging(data, bytes);
}

//---------------------------------------------------------------------------
//  RamLogger::DoLogging() --
//---------------------------------------------------------------------------

void RamLogger::DoLogging(const void * data, unsigned int bytes)
{
    if (!image || !FEnabled || !bytes)
        return;

    if (FCeiling && ((FLogged + bytes) > FCeiling))
        bytes = static_cast<unsigned int>(FCeiling - FLogged);

    // Avoid vector::push_back(): Too slow under MSVC
    const char * src = static_cast<const char*>(data);
    char * dst = &image[static_cast<unsigned int>(FLogged)];
    memcpy(dst, src, bytes);

	FLogged += bytes;

    if (FCeiling && (FLogged >= FCeiling))
        Stop(); // limit exeeded

    if(FCeiling)
    	{
        int Percent = static_cast<int>(std::min(FLogged, FCeiling) * 100 / FCeiling);
        ProgressEvent e(Percent);
        OnProgress.Execute(e);
	    }
}

//---------------------------------------------------------------------------
//  RamLogger::Start() --
//---------------------------------------------------------------------------

void RamLogger::Start()
{
    if(!FFileName.length())
        return;

    FLogged = 0;
    ProgressEvent e(0);
    OnProgress.Execute(e);
    
    StartBufferEvent event(FFileName.length());
    OnStart.Execute(event);
}

//---------------------------------------------------------------------------
//  RamLogger::Stop() --
//---------------------------------------------------------------------------

void RamLogger::Stop()
{
    if (FLogged)
        Task->Process();
}

//---------------------------------------------------------------------------
//  RamLogger::Logged() --
//---------------------------------------------------------------------------

ii64 RamLogger::Logged() const
{
	return FLogged;
}

//---------------------------------------------------------------------------
//  RamLogger::Ceiling() --
//---------------------------------------------------------------------------

void RamLogger::Ceiling(ii64 value)
{
    FCeiling = value;

    delete [] image;
    image = new char[static_cast<unsigned long>(value)];
}

//---------------------------------------------------------------------------
//  RamLogger::Ceiling() --
//---------------------------------------------------------------------------

ii64 RamLogger::Ceiling() const
{
	return FCeiling;
}

//---------------------------------------------------------------------------
//  RamLogger::FileName() --
//---------------------------------------------------------------------------

void RamLogger::FileName(const string & value)
{
	FFileName = value;
}

//---------------------------------------------------------------------------
//  RamLogger::FileName() --
//---------------------------------------------------------------------------

string RamLogger::FileName() const
{
	return FFileName;
}

//---------------------------------------------------------------------------
//  RamLogger::Enabled() --
//---------------------------------------------------------------------------

void RamLogger::Enabled(bool state)
{
	FEnabled = state;
}

//---------------------------------------------------------------------------
//  RamLogger::Enabled() --
//---------------------------------------------------------------------------

bool RamLogger::Enabled() const
{
	return FEnabled;
}

}   // Innovative

