// RandomGen_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <RandomGen_Mb.h>
#ifndef VXWORKS
#include <ipps.h>   // ipps.h is not redistributable!
#else
#include <stdlib.h>
#include <string.h>
#endif

namespace   Innovative
{

//---------------------------------------------------------------------------
// RandomGen::RandomGen() --  Ctor
//---------------------------------------------------------------------------

RandomGen::RandomGen()
	: FLow(-20000), FHigh(20000), FSeed(31192)
{
	update();
}

//---------------------------------------------------------------------------
// RandomGen::RandomGen() --  Copy ctor
//---------------------------------------------------------------------------

RandomGen::RandomGen(const RandomGen & other)
	: inherited(other), FLow(other.FLow), FHigh(other.FHigh), FSeed(other.FSeed)
{
	update();
}

//---------------------------------------------------------------------------
//  RandomGen::operator = --  Assignment
//---------------------------------------------------------------------------

RandomGen & RandomGen::operator = (const RandomGen & other)
{
	inherited::operator = ( other );

	FLow = other.FLow;
	FHigh = other.FHigh;
	FSeed = other.FSeed;

	update();

	return *this;
}

//---------------------------------------------------------------------------
//  RandomGen::Generate() --  Fill buffer with random data
//---------------------------------------------------------------------------

bool RandomGen::Generate(IppFloatDG & buffer)
{
	IppsRandUniState_32f * p = reinterpret_cast<IppsRandUniState_32f*>(&mState);

	IppStatus status = ippsRandUniform_32f(buffer.Ptr(), static_cast<int>(buffer.size()), p);

	return (status == ippStsNoErr);
}

//---------------------------------------------------------------------------
//  RandomGen::Generate() --
//---------------------------------------------------------------------------

bool RandomGen::Generate(IppComplexDG & buffer)
{
	return inherited::Generate(buffer);
}

//---------------------------------------------------------------------------
//  RandomGen::Low() --  Set lower bound on generated data
//---------------------------------------------------------------------------

void RandomGen::Low(float value)
{
	FLow = value;
	update();
}

//---------------------------------------------------------------------------
//  RandomGen::Low() --  Get lower bound on generated data
//---------------------------------------------------------------------------

float RandomGen::Low() const
{
	return FLow;
}

//---------------------------------------------------------------------------
//  RandomGen::High() --  Set upper bound on generated data
//---------------------------------------------------------------------------

void RandomGen::High(float value)
{
	FHigh = value;
	update();
}

//---------------------------------------------------------------------------
//  RandomGen::High() --  Get upper bound on generated data
//---------------------------------------------------------------------------

float RandomGen::High() const
{
	return FHigh;
}

//---------------------------------------------------------------------------
//  RandomGen::Seed() --  Set seed value on generated data
//---------------------------------------------------------------------------

void RandomGen::Seed(unsigned int value)
{
	FSeed = value;
	update();
}

//---------------------------------------------------------------------------
//  RandomGen::Seed() --  Get seed value on generated data
//---------------------------------------------------------------------------

unsigned int RandomGen::Seed() const
{
	return FSeed;
}

//---------------------------------------------------------------------------
//  RandomGen::update() --  Update internal state array
//---------------------------------------------------------------------------

bool RandomGen::update()
{
	IppsRandUniState_32f * p = reinterpret_cast<IppsRandUniState_32f*>(&mState);

	IppStatus result = ippsRandUniformInit_32f(p, FLow, FHigh, FSeed);

    return (result == ippStsNoErr);
}

} // namespace
