//---------------------------------------------------------------------------
// SafeLink_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef SafeLink_MbH
#define SafeLink_MbH

//===========================================================================
//  TEMPLATE CLASS SafeLink  -- Always safe linkage
//===========================================================================
//  SafeLink creates a safer pointer to an interface, since the link points to
//    a valid link object or a pre-made Null object that avoids any bad pointer
//    accesses if the link is used before or after a true linkage.

template <class T, class SafeT>
class SafeLink
{
public:
    SafeLink()
        :  isLinked(false), LinkPtr(&Safe)
        {}
    virtual ~SafeLink()
        {}

    bool  Linked()  {  return isLinked;  };

    T *  operator -> ()             // smart pointer
        {  return LinkPtr;  }
    T *  Ptr()                      // dumb pointer
        {  return LinkPtr;  }

    void  Link( T * newLink )
        {
        if (Linked())
            Unlink();
        //  safety check
        if  (newLink==0)
            return;
        OnBeforeLink();
        LinkPtr = newLink;  isLinked=true;
        OnAfterLink();
        }
    void  Unlink()
        {
        OnBeforeUnlink();
        LinkPtr = &Safe;  isLinked=false;
        OnAfterUnlink();
        }

private:
    bool    isLinked;
    T *     LinkPtr;
    SafeT   Safe;

    //  Override to add special actions when linking or unlinking
    virtual void  OnBeforeLink()  {};
    virtual void  OnAfterLink()  {};
    virtual void  OnBeforeUnlink()  {};
    virtual void  OnAfterUnlink()  {};
};

//===========================================================================
//  TEMPLATE CLASS SafeLinkAlias  -- Reference to a SafeLink
//===========================================================================

template <class T, class SafeT>
class SafeLinkAlias
{
public:
    SafeLinkAlias(SafeLink<T, SafeT> & s_link)
        :  SLink(s_link)
        {}
    SafeLinkAlias(SafeLinkAlias<T, SafeT> & s_link_alias)  // create from another alias
        :  SLink(s_link_alias.SLink)
        {}

    bool  Linked()  {  return SLink.Linked();  };

    T *  operator -> ()             // smart pointer
        {  return SLink.Ptr();  }
    T *  Ptr()                      // dumb pointer
        {  return SLink.Ptr();  }
	
	
private:
    SafeLink<T, SafeT> &  SLink;
};


#endif

