// Sample_Mb.h
//
//  Acquired data sample container class
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

//---------------------------------------------------------------------------
#ifndef Sample_MbH
#define Sample_MbH

#include <DataTypes_Mb.h>
#include <complex>
#include <algorithm>
#include <limits>

#undef min
#undef max

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//========================================================================
//  CLASS Subsample -- Acquired data sample container class
//========================================================================

class Subsample
{
    friend class Sample;

public:
	enum SampleType { sInt, sFloat, sTiFloat, sALaw, sMuLaw, sUnknown };

	Subsample(char * p, int bytes) : FType(sUnknown)
		{  std::copy(p, &p[bytes], FValue.Byte);  }
	Subsample() : FType(sUnknown)
		{  FValue.Int = 0;  }
	Subsample(int i) : FType(sInt)
		{  FValue.Int = i;  }
	Subsample(double f) : FType(sFloat)
		{  FValue.Float = static_cast<float>(f);  }

	char * asString();

    bool operator < (const Subsample & rhs) const;
    bool operator >= (const Subsample & rhs) const;
    bool operator > (const Subsample & rhs) const;
    Subsample operator- (const Subsample & rhs);
    Subsample operator --();
    bool operator ==(const Subsample & rhs) const;
    bool operator !=(const Subsample & rhs) const;
    operator float() const
        { return Float(); };
    operator double() const
        { return Double(); };
    operator int() const
        { return Int(); };

	// Properties
	void 				Int(int value);
	int 				Int() const;

	void				Float(float value);
	float				Float() const;

	void				Double(double value);
	double				Double() const;
	double				Extended(bool unipolar) const;

	void				Type(SampleType value);
	SampleType          Type() const;

    size_t              SizeInBytes(int bits) const;

private:
	union
	{
		char        Byte[4];
		int         Int;
		float       Float;
	} FValue;
	SampleType 	FType;
};

//========================================================================
//  CLASS Sample -- Complex data sample container class
//========================================================================

class Sample
{
public:
	enum SampleType { sInt, sFloat, sTiFloat, sALaw, sMuLaw, sUnknown };

    // Ctor
    Sample();
    Sample(char * p, int bytes, bool complex=false);
    Sample(int r, int i=0);
    Sample(double r, double i=0.0f);
    Sample(const Sample & sample);

    // Methods
    Sample & operator =(const Sample & other);
    Sample & operator =(const double & rhs);
    Sample & operator =(const int & rhs);
    bool operator < (const Sample & rhs) const;
    bool operator >= (const Sample & rhs) const;
    bool operator > (const Sample & rhs) const;
    Sample operator- (const Sample & rhs);
    Sample operator --();
    bool operator ==(const Sample & rhs) const;
    bool operator !=(const Sample & rhs) const;
    operator float() const
        { return Real.Float(); };
    operator double() const
        { return Real.Double(); };
    operator int() const
        { return Real.Int(); };

	// Properties
	void 				Int(int value);
	int 				Int() const;

	void				Float(float value);
	float				Float() const;

	void				Double(double value);
	double				Double() const;
	double				Extended(bool unipolar) const;

	void				Type(SampleType value);
	SampleType          Type() const;

    std::complex<double> Complex() const;
    double              Phase(double err= std::numeric_limits<double>::min()) const;
    double              Degrees(double err= std::numeric_limits<double>::min()) const;
    double              Magnitude(double err= std::numeric_limits<double>::min()) const;

    inline bool IsNan(double x) const
        {  return x != x;  }

	// Methods
    Subsample operator [] (unsigned int idx) const;
    Subsample & operator [] (unsigned int idx);

    // Data
    Subsample   Real;
    Subsample   Imag;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
