// Sbc6713e_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
// Sbc6713 baseboard functions

#ifndef Sbc6713e_MbH
#define Sbc6713e_MbH


#include <AceLibrary_Mb.h>
#include <string>
#include <Buffer_Mb.h>
#include <ProcessEvents_Mb.h>
#include <HardwareRegister_Mb.h>
#include <TargetInfo_Mb.h>

class UsesAceLibrary
{
public:
	virtual ~UsesAceLibrary() {}
	virtual void Abs() = 0;
private:
    UsesAceLibrary();
};

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//------------------------------------
// SystemTestInfo, only unsigned ints.
//------------------------------------

struct SystemTestInfo
{
unsigned int DataErrors;
unsigned int AddrErrors;
unsigned int F6713ClockCount;
unsigned int F642ClockCount;
unsigned int BootJumperInstalled;
unsigned int Fallow;               // to make it even number of words.
};
const int SystemTestInfoBytes = sizeof(SystemTestInfo);

#if  (SystemTestInfoBytes % 4)
    #error Size must be a 32-bit multiple;
#endif
const int SystemTestSizeInts = sizeof(SystemTestInfo)/sizeof(int);


//=============================================================================
//  CLASS Sbc6713e  --  Baseboard Class for SBC6713e
//=============================================================================

class Sbc6713e : public UsesAceLibrary
{
    friend class NetCoffLoadThread;
    friend class NetImageLoadThread;
    friend class NetInfoCommThread;
    friend class SetBoardInfoThread;
   	friend class NetTargetResetThread;
    friend class NetInboundXfrThread;
    friend class NetOutboundXfrThread;
    friend class NetSystemTestThread;

public:
    enum  RecvType { rtCommand, rtPacket };
    enum  LoadInfoType { ltLANLogic, ltINTFLogic, ltLoaderBin, ltTalkerBin, ltAppBin };
    enum  MiscCommId { rdFware, rdAppBoot, rdNetInfo, wrAppBoot, wrBoardInfo, wrBoardRev, resetTarget };

    //
    //  Member Functions
    Sbc6713e(std::string net_id);
    ~Sbc6713e();

    void  RequestCoffLoad(std::string coff_file);
    void  RequestImageLoad(LoadInfoType which, std::string image_file);
    void  RequestMiscComm(MiscCommId which, int Arg0, int Arg1);
    void  WriteMiscComm(MiscCommId which, TargetInfo Arg0, int Arg1);
	void  RequestTargetReset();
    void  GetSystemTestInfo(SystemTestInfo & info);


	// Packet Data Transfer----
    enum CommStatusCodes{NetworkTimeoutErr = 0x0, Success, TargetTimeoutErr, TargetErr, UnknownErr, CommAborted};

	CommStatusCodes  Send(Buffer & buffer);
	CommStatusCodes  SendCommand(short CommandId, int Arg0=0, int Arg1=0, int Arg2=0);
    CommStatusCodes  SendClose();

	CommStatusCodes  Recv(RecvType & which, Buffer & Buffer);
    CommStatusCodes  Recv(Buffer&  buffer);
    CommStatusCodes  RecvClose();

    CommStatusCodes  ClearPendingXfrs();

    //-------------------------

    //
    //  Methods
    std::string  Id()  {  return NetId;  };
    int          DownloadPort()  { return CoffDownloadPort;  };
    int          ImageLoadPort() { return NetImageLoadPort;  };
    int          OutXfrPort()    { return NetOutboundXfrPort;};
    int          InXfrPort()     { return NetInboundXfrPort; };
    int          InfoCommPort()  { return NetInfoCommPort;   };
    int          SetBdInfoPort() { return SetBoardInfoPort;  };
	int			 ResetPort()     { return TargetResetPort;   };
    int          SystemTestPort(){ return SystemTestInfoPort;};

    // Target info setter methods; used them only
    // when the info is gotten from target.
    // S/N and Board rev..
    void    SerialNoBoardRev( short sn, char rev );
    short   SerialNumber();
    char    BoardRevision();

    // Ip Info.
    void    StaticIPInfo( const char * TargetIpAddr, const char * TargetIpMask,
                          const char * gatewayIPAddr, const char * dnsServerIpAddr,
                          const char * domainName );
    bool    StaticIpInfoLoaded() const;
    void    UseStaticIp( bool state );
    bool    UseStaticIp();
    const char *  TargetIpAddress() const;
    const char *  TargetIpMask() const;
    const char *  GatewayIpAddress() const;
    const char *  DnsServerIPAddress() const;
    const char *  DomainName() const;

    // Boot customer app.
    void    BootApplication( bool state );
    bool    BootApplication();

    // Firmware revision.
    void    TargetFwareInfo( int ClientFwareVer, int IntfLogicVer, int LanLogicVer );
    int     IntfLogicVersion();
    int     LanLogicVersion();
    int     ClientFwareVersion();

    //
    //  Events
    OpenWire::ThunkedEventHandler<ProcessProgressEvent>   OnDownloadProgress;
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent> OnDownloadComplete;

    OpenWire::ThunkedEventHandler<ProcessProgressEvent>   OnLogicParseProgress;
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent> OnLogicParseComplete;

    OpenWire::ThunkedEventHandler<ProcessProgressEvent>   OnLogicLoadProgress;
    OpenWire::ThunkedEventHandler<ProcessCompletionEvent> OnLogicLoadComplete;
    OpenWire::ThunkedEventHandler<ProcessStatusEvent>     OnLogicLoadError;

    OpenWire::EventHandler<ProcessCompletionEvent>        OnInfoRqstComplete;
    OpenWire::EventHandler<ProcessStatusEvent>            OnInfoRqstError;

    OpenWire::EventHandler<ProcessCompletionEvent>        OnWriteRqstComplete;
    OpenWire::EventHandler<ProcessStatusEvent>            OnWriteRqstError;

    OpenWire::EventHandler<ProcessCompletionEvent>        OnInfoWriteComplete;
    OpenWire::EventHandler<ProcessStatusEvent>            OnInfoWriteError;

	OpenWire::EventHandler<ProcessCompletionEvent>	      OnTargetResetComplete;
	OpenWire::EventHandler<ProcessStatusEvent>            OnTargetResetError;

    OpenWire::EventHandler<OpenWire::NotifyEvent>         OnSendTimeout;
    OpenWire::EventHandler<OpenWire::NotifyEvent>         OnRecvTimeout;

    OpenWire::EventHandler<OpenWire::NotifyEvent>         OnSystemTestErrror;

private:
    //
    //  Member Data
    Buffer                          FallowBuff; // Used to bring SingletonManager
                                                // into existance.
    class NetCoffLoadThread    *    NCL;
    class NetOutboundXfrThread *    OXfr;
    class NetInboundXfrThread  *    IXfr;
    class NetImageLoadThread   *    NILT;
    class NetInfoCommThread    *    NICT;
    class SetBoardInfoThread   *    SBIT;
	class NetTargetResetThread *    NTRT;
    class NetSystemTestThread  *    NSTT;

    bool                            FAbortSend;
    bool                            FAbortReceive;
    std::string                     NetId;

    // Member Enumerations
    enum IIPort { CoffDownloadPort   = 1007,
                  NetOutboundXfrPort = 1008,
                  NetInboundXfrPort  = 1009,
                  NetImageLoadPort   = 1020,
                  NetInfoCommPort    = 1021,
                  SetBoardInfoPort   = 1022,
                  TargetResetPort    = 1023,
                  SystemTestInfoPort = 1024};

    void  DownloadProgress(ProcessProgressEvent & Event);
    void  DownloadComplete(ProcessCompletionEvent &  Event);

    void  LogicParseProgress(ProcessProgressEvent & Event);
    void  LogicParseComplete(ProcessCompletionEvent & Event);

    void  LogicLoadProgress(ProcessProgressEvent & Event);
    void  LogicLoadComplete(ProcessCompletionEvent & Event);
    void  LogicLoadError(ProcessStatusEvent & Event);

    void  InfoRqstComplete(ProcessCompletionEvent & Event);
    void  InfoRqstError(ProcessStatusEvent & Event);

    void  InfoWriteComplete(ProcessCompletionEvent & Event);
    void  InfoWriteError(ProcessStatusEvent & Event);

    void  TargetResetComplete(ProcessCompletionEvent & Event);
	void  TargetResetError(ProcessStatusEvent & Event);

    // Target specific information
    TargetInfo  BoardInfo;
};

//=============================================================================
//  CLASS TransferPacketHeader  --  Packet Data Transfer Header
//=============================================================================

class TransferPacketHeader
{
public:
    //
    //  Member Functions
    TransferPacketHeader();
    ~TransferPacketHeader();
    //
    //  Methods
    void  Clear();

    void  CopyTo(Buffer & buffer);
    void  LoadFrom(Buffer & buffer);
    void  CopyTo(int * buffer);
    void  LoadFrom(int * buffer);
    
    //
    //  Properties
    //  ...Channel
    TransferPacketHeader &   ReadChannel(int ch);
    TransferPacketHeader &   WriteChannel(int ch);
    
    bool                     IsReadTransfer() const;
    bool                     IsWriteTransfer() const;
    int                      Channel() const;
    //  ...Type
    TransferPacketHeader &   Type(short type);
    short                    Type() const;
    //  ...Size
    TransferPacketHeader &   Size(int size);
    int                      Size() const;
    //  ...DestinationAddr
    TransferPacketHeader &   DestinationAddr(int addr);
    int                      DestinationAddr() const;
    //  ...User
    TransferPacketHeader &   User(int data);
    int                      User() const;
    //  ...Command Args (Commands only!)
    TransferPacketHeader &   Arg(int idx, int data);
    int                      Arg(int idx) const;
    //  ...Header Word
    TransferPacketHeader &   HeaderWord(int idx, int data);
    int                      HeaderWord(int idx) const;
    
private:
    //
    //  Member Data
    unsigned int              Header[4];
    Register                  HeaderTCWord;
    AddressingSpace           HeaderAS;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
