// SbcDuoDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <SbcDuoDeviceMap_Mb.h>
#include <SbcDuoMemoryMap_Mb.h>
#include <SbcDuoDeviceDriver_Mb.h>
#include <Gps_Csac_Mb.h>
#include <Gps_Ieee1588_Mb.h>
#include <SystemSupport_Mb.h>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{
//===========================================================================
//  Constants for CLASS SbcDuoDeviceMap
//===========================================================================

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Name, P/N, S/N
//       0x40 spare
const int idrom_ParameterSize    = 0x40;

//  ...Second  section: Clock Parameters  ? not used
//       (1 param + 5 calibrated word + 2 spare)
const int AssemblyRegionOffset = idrom_ParameterSize;
const int AssemblyRegionWordSize = 5;

//=============================================================================
//  CLASS SbcDuo_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuo_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class SbcDuo_Pll
//---------------------------------------------------------------------------

SbcDuo_Pll::SbcDuo_Pll(SbcDuoDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg, omNestedZeroDelay, tsmLogic), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 0 as guideline output
    Allow_Pll1_FeedbackAdjustment();

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);

}

//---------------------------------------------------------------------------
//  destructor for class SbcDuo_Pll
//---------------------------------------------------------------------------

SbcDuo_Pll::~SbcDuo_Pll()
{
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  SbcDuo_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "SbcDuo_Pll::OnInit()");
    //
    //  Input Configuration
    SetInputConfig(0, icCmos);
    SetInputConfig(1, icBipolar);
    SetInputConfig(2, icBipolar);

    //  Device Powered/Enabled Setup
    PowerDownOutputGroup(2/2, true);
    //
    //  Set Output Configuration
    OutputInUse(0,  true);    SetOutputConfig(0,  ocLvPecl_1600);  // XMC0_FS  - XMC0 Sampling Clock
    OutputInUse(1,  true);    SetOutputConfig(1,  ocLvPecl_1600);  // XMC0_REF - XMC0 Reference Clock
                                 SetOutput_AsCopy(1);              //    - is a copy of output 0
    OutputInUse(2,  false);   SetOutputConfig(2,  ocDisabled);     //  (unused)
    OutputInUse(3,  false);   SetOutputConfig(3,  ocDisabled);     //  (unused)

    OutputInUse(4,  true);    SetOutputConfig(4,  ocLvPecl_1600);  //  FPGA_FS_CLK
    OutputInUse(5,  true);    SetOutputConfig(5,  ocLvPecl_1600);  //  XMC0_TRIG

    OutputInUse(6,  true);    SetOutputConfig(6,  ocDisabled);     //  (unused)
    OutputInUse(7,  true);    SetOutputConfig(7,  ocLvPecl_1600);  //  XMC1_TRIG
    OutputInUse(8,  true);    SetOutputConfig(8,  ocLvPecl_1600);  //  CLKOUT0
    OutputInUse(9,  true);    SetOutputConfig(9,  ocLvPecl_2000);  //  TRIG_OUT0
    OutputInUse(10, true);    SetOutputConfig(10, ocLvPecl_1600);  //  CLKOUT1
    OutputInUse(11, true);    SetOutputConfig(11, ocLvPecl_2000);  //  TRIG_OUT1
    OutputInUse(12, true);    SetOutputConfig(12, ocLvPecl_1600);  // XMC1_FS  - XMC1 Sampling Clock
    OutputInUse(13, true);    SetOutputConfig(13, ocLvPecl_1600);  // XMC1_REF - XMC1 Reference Clock
                                 SetOutput_AsCopy(13);             //    - is a copy of output 12

    FeedbackOutput(fbm_Dclk6);  // set output to use as PLL feedback in 0-delay modes
//    FeedbackOutput(fbm_Dclk8);  // set output to use as PLL feedback in 0-delay modes

    //  Set Reduced Frequency Limits for Outputs (Preset to 3200.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    OutputFrequencyLimit[4] = 400.0e6;
    //
    //  Specify if Output is to be disabled during SYNC setup (default = 0 (no disable)
    OutputSyncDisable[5]  = 1;
    OutputSyncDisable[7]  = 1;
    OutputSyncDisable[9]  = 1;
    OutputSyncDisable[11] = 1;

    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference

    // Invert SYNC polarity to make trigger have correct state.
    SetSyncPolarityInverted(true);

}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::OnReapply()
//---------------------------------------------------------------------------

void  SbcDuo_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  SbcDuo_Pll::AdcMasterOutput()
{
    return 8;
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  SbcDuo_Pll::DacMasterOutput()
{
    return 8;
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool SbcDuo_Pll::IsFeedbackOutputAdc()
{
    return true;
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  SbcDuo_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(0,    adc_rate);
    //OutputFrequency(1,   adc_rate);   // copy of 0
    OutputFrequency(2,    adc_rate);
    OutputFrequency(3,    adc_rate);
    OutputFrequency(4,    ADC_TRIM(adc_rate));
    OutputFrequency(5,    adc_rate);
    OutputFrequency(6,    adc_rate);
    OutputFrequency(7,    adc_rate);
    OutputFrequency(8,    adc_rate);
    OutputFrequency(9,    adc_rate);
    OutputFrequency(10,   adc_rate);
    OutputFrequency(11,   adc_rate);
    OutputFrequency(12,   adc_rate);
    //OutputFrequency(13,   adc_rate);   // copy of 12
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  SbcDuo_Pll::SetDacOutputRates(double /*dac_rate*/)
{
    // none
}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::EnableAllOutputs() -- enable/disable external outputs
//---------------------------------------------------------------------------

void  SbcDuo_Pll::EnableAllOutputs(bool enabled)
{
    ScopedTraceRef ccc(Debug, "SbcDuo_Pll::EnableAllOutputs()");

    OutputEnabled(0, enabled);
    OutputEnabled(1, enabled);
    //OutputEnabled(2, enabled);  // off
    //OutputEnabled(3, enabled);
    OutputEnabled(4, enabled);
    OutputEnabled(5, enabled);
    OutputEnabled(6, enabled);
    OutputEnabled(7, enabled);
    OutputEnabled(8, enabled);
    OutputEnabled(9, enabled);
    OutputEnabled(10, enabled);
    OutputEnabled(11, enabled);
    OutputEnabled(12, enabled);
    OutputEnabled(13, enabled);

    WriteDataRegisters();

}

//---------------------------------------------------------------------------
//  SbcDuo_Pll::ADC_TRIM() -- Trim ADC rate in a controlled manner
//---------------------------------------------------------------------------

double  SbcDuo_Pll::ADC_TRIM(double rate)
{
    // there is a physical 400 MHz limit.  Get below it by using integer power
    //   of 2 factors rather than trying to match closely as standard trim does.
    if (rate >= 1600.0e6)
        return rate/8.0;
    else if (rate >= 800.0e6)
        return rate/4.0;
    else if (rate >= 400.0e6)
        return rate/2.0;
    else
        return rate;
}


//==============================================================================
//  CLASS SbcDuo_Mezzanine -- Duo Mezzanine
//==============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuo_Mezzanine
//---------------------------------------------------------------------------

const unsigned int OutputMask_Active = 0x7FFFFF00u;  // Bits 0-7, 31 unused

//---------------------------------------------------------------------------
//  constructor for class SbcDuo_Mezzanine
//---------------------------------------------------------------------------

SbcDuo_Mezzanine::SbcDuo_Mezzanine(AdvancedDioPortIntf & owner_dio)
    : ODio(owner_dio)
{

}

//---------------------------------------------------------------------------
//  destructor for class SbcDuo_Mezzanine
//---------------------------------------------------------------------------

SbcDuo_Mezzanine::~SbcDuo_Mezzanine()
{

}

//---------------------------------------------------------------------------
//  SbcDuo_Mezzanine::Active() - master on/off ..turn on to use Mezzanine
//---------------------------------------------------------------------------

void  SbcDuo_Mezzanine::Active(bool state)
{
    if (state)
        ODio.DioPortConfig(OutputMask_Active);
    else
        ODio.DioPortConfig(0);  // all inputs

}

//---------------------------------------------------------------------------
//  SbcDuo_Mezzanine::Xmc_Direction() - Set Xmc Bit Directions
//---------------------------------------------------------------------------

void  SbcDuo_Mezzanine::Xmc_Direction(unsigned int xmc, unsigned int bit, bool isOutput)
{
    // Select Field Base with XMC #
    unsigned int bit_start = (xmc==0) ? 8 : 12 ;
    // Adjust by XMC bit
    if (bit<4)
        bit_start += bit;

    ODio.DioPortData().Bit(bit_start, isOutput);

}

//---------------------------------------------------------------------------
//  SbcDuo_Mezzanine::Xmc_VSel() - Set Xmc VSEL options
//---------------------------------------------------------------------------

void  SbcDuo_Mezzanine::Xmc_VSel(unsigned int xmc, XmcVSel_options opts)
{
    // Select Field Base with XMC #
    unsigned int field_start = (xmc==0) ? 16 : 20 ;

    //  Enum value is field value
	ODio.DioPortData().Field(field_start, 4, static_cast<unsigned int>(opts));

}

//---------------------------------------------------------------------------
//  SbcDuo_Mezzanine::Io_VSel() - Set Io VSEL options
//---------------------------------------------------------------------------

void  SbcDuo_Mezzanine::Io_VSel(IoVSel_options opts)
{
    //  Enum value is field value
	ODio.DioPortData().Field(24, 4, static_cast<unsigned int>(opts));

}

//---------------------------------------------------------------------------
//  SbcDuo_Mezzanine::XmcEnable() - Set Xmc Enable
//---------------------------------------------------------------------------

void  SbcDuo_Mezzanine::XmcEnable(unsigned int xmc, bool state)
{
    // Select Bit Base with XMC #
    unsigned int bit_start = (xmc==0) ? 28 : 29;

    //  Enum value is field value
    ODio.DioPortData().Bit(bit_start, state);

}

//---------------------------------------------------------------------------
//  SbcDuo_Mezzanine::OutputEnable() - Set Io Enable
//---------------------------------------------------------------------------

void  SbcDuo_Mezzanine::OutputEnable(bool state)
{
    //  Enum value is field value
    ODio.DioPortData().Bit(30, state);

}

//===========================================================================
//  CLASS SbcDuoCsacPlugin  -- Hardware Plugin impl for Jackson Labs CSAC
//===========================================================================

class SbcDuoCsacPlugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() {  Owner->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)  // antenna not implemented in hardware
                          {  return gps_value;  }
    virtual bool    PGPS_IsLocked(bool /*gps_value*/)
                          {  return Owner->IsLocked();  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return Owner->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  Owner->UartData(c); }
    virtual char             PGPS_UartData()
                                {  return Owner->UartData();  }

    virtual void             PGPS_UartBaud(IGps::BaudRates /*baud*/)   // don't support changing baud rates?
                                {  /*Owner->Baud(baud);*/  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return Owner->Baud();  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {  Owner->GpsEnabled(state);  }
    virtual bool          PGPS_HdwEnabled()
                                {  return Owner->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/)  {}

    SbcDuoCsacPlugin(SbcDuoGpsUart * owner);
    virtual ~SbcDuoCsacPlugin()
        {}

private:
     SbcDuoGpsUart * Owner;

};

//---------------------------------------------------------------------------
//  constructor for class SbcDuoCsacPlugin
//---------------------------------------------------------------------------

SbcDuoCsacPlugin::SbcDuoCsacPlugin(SbcDuoGpsUart* owner)
    : Owner(owner)
{
}

//---------------------------------------------------------------------------
//   SbcDuoCsacPlugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void SbcDuoCsacPlugin::PGPS_OnInit()
{
    Owner->GpsReset(false);
    Sleep(1);
    Owner->GpsReset(true);
    Sleep(1);
    Owner->GpsReset(false);
    Sleep(1);

    Owner->Uart(0).PutString("SYNC:OUT:1PPS:RESET ON\r\n");    // PPS from power up
    Owner->Uart(0).PutString("GPS:GPZDA 1\r\n");              // emit GPZDA once per second
    Owner->Uart(0).PutString("GPS:GPGGA 1\r\n");              // emit GPGGA once per second
}

//===========================================================================
//  CLASS SbcDuoIeee1588Plugin  -- Hardware Plugin impl for IEEE1588 intf
//===========================================================================

class SbcDuoIeee1588Plugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() {  GpsUart->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)  // antenna not implemented in hardware
                          {  return gps_value;  }
    virtual bool    PGPS_IsLocked(bool gps_value)
                          {  return gps_value;  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return GpsUart->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  GpsUart->Uart(0).Port(c); }  // or leave blank?
    virtual char             PGPS_UartData()
                                {  return GpsUart->Uart(0).Port();  }

    virtual void             PGPS_UartBaud(IGps::BaudRates /*baud*/)   // don't support changing baud rates?
                                {  /*IeeeDev->Uart(0).SetBaud(baud);*/  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return IGps::b115200;  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {
                                GpsUart->GpsEnabled(state);
                                }
    virtual bool          PGPS_HdwEnabled()
                                {  return GpsUart->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/)  {}

    SbcDuoIeee1588Plugin(SbcDuoGpsUart * gps_uart, BabushkaIeee1588 * ieee_dev);
    virtual ~SbcDuoIeee1588Plugin()
        {}

private:
    SbcDuoGpsUart * GpsUart;
    BabushkaIeee1588 * IeeeDev;
};

//---------------------------------------------------------------------------
//  constructor for class SbcDuoIeee1588Plugin
//---------------------------------------------------------------------------

SbcDuoIeee1588Plugin::SbcDuoIeee1588Plugin(SbcDuoGpsUart * gps_uart, BabushkaIeee1588 * ieee_dev)
    : GpsUart(gps_uart), IeeeDev(ieee_dev)
{
}

//---------------------------------------------------------------------------
//   SbcDuoIeee1588Plugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void SbcDuoIeee1588Plugin::PGPS_OnInit()
{
    GpsUart->Ieee1588_MezzanineReset(false);
    Sleep(1);
    GpsUart->Ieee1588_MezzanineReset(true);
    Sleep(1);
    GpsUart->Ieee1588_MezzanineReset(false);
    Sleep(1);

    // we can't config thru serial port

    // but can we do the IEEE init here?
    IeeeDev->InitIeee1588();

}

//===========================================================================
//  CLASS SbcDuoGPS_Callback -- X6 GPS Hardware Callback Interface
//===========================================================================

class SbcDuoGPS_Callback : public IGPS_Callback
{
public:
    SbcDuoGPS_Callback(SbcDuoDeviceMap * owner)
        :  Owner(owner)
        {}
    virtual ~SbcDuoGPS_Callback()
        {}

    virtual void InterruptEnable(bool state)
                    {  Owner->InterruptEnable(state);  }

private:
    SbcDuoDeviceMap *  Owner;
};

//===========================================================================
//  CLASS SbcDuo_SerialRegister -- Use serial link to write to a register bank
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuo_SerialRegister
//---------------------------------------------------------------------------

const unsigned int kProtocolReadSize = 6;

//---------------------------------------------------------------------------
//  SbcDuo_SerialRegister::ReadRegister()  --  Read logic register via serial
//---------------------------------------------------------------------------

ii32   SbcDuo_SerialRegister::ReadRegister(unsigned int address)
{
//    Debug.Log("SbcDuo_SerialRegister::ReadRegister");
//    Debug.Log("address=", address, 16);

    PtrBitManipulator addrB(&address);
    // construct string to send
    TxData[0] = addrB.Field(0, 8);
    TxData[1] = addrB.Field(8, 8) | 0x80;

    Uart_Tx.PutString(TxData);

    // Get string back
    Uart_Rx.GetFixedString(RxData, kProtocolReadSize);

    MemoryBitManipulator outdata(0);
    // RxData[0] and RxData[1] should match TxData[0] and TxData[1]
    outdata.Field( 0, 8, RxData[2]);
    outdata.Field( 8, 8, RxData[3]);
    outdata.Field(16, 8, RxData[4]);
    outdata.Field(24, 8, RxData[5]);

//    Debug.Log("outdata:", outdata.Value(), 16);

    return outdata.Value();

}

//---------------------------------------------------------------------------
//  SbcDuo_SerialRegister::WriteRegister()  --  Write logic register via serial
//---------------------------------------------------------------------------

void  SbcDuo_SerialRegister::WriteRegister(unsigned int address, ii32 data)
{
//    Debug.Log("SbcDuo_SerialRegister::WriteRegister");
//    Debug.Log("address=", address,  16);
//    Debug.Log("data=", data, 16);

    PtrBitManipulator outdata(&data);
    PtrBitManipulator addrB(&address);
    // construct string to send
    //std::string tx;
    TxData_X[0] = addrB.Field(0, 8);
    TxData_X[1] = addrB.Field(8, 7);

    TxData_X[2] = outdata.Field( 0, 8);
    TxData_X[3] = outdata.Field( 8, 8);
    TxData_X[4] = outdata.Field(16, 8);
    TxData_X[5] = outdata.Field(24, 8);
    Uart_Tx.PutString(TxData_X);

}

//===========================================================================
//  CLASS BabushkaIeee1588 -- Access UART using uart link to remote registers
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class BabushkaIeee1588
//---------------------------------------------------------------------------

static const double UartRefClock = 16.0 * 65536.0/200.0e6;

//---------------------------------------------------------------------------
//  constructor for class BabushkaIeee1588
//---------------------------------------------------------------------------

BabushkaIeee1588::BabushkaIeee1588(SbcDuo_SerialRegister & link, MapRegisters regs)
    :  Link(link), RegBaseOfs(regs.UartWishbone_Base),
       UartMem(RegCache),
       ControlReg    (UartMem, regs.Ctrl_Addr),
       ClockId_Lo_Reg(UartMem, regs.ClkId_Lo_Addr),
       ClockId_Hi_Reg(UartMem, regs.ClkId_Hi_Addr),
       TimeReg       (UartMem, regs.Time_Addr),
       I2CReg        (UartMem, regs.EEPromI2C_Addr)
{
    for (size_t i=0; i<10; i++)
        RegCache[i] = 0;
}

//---------------------------------------------------------------------------
//  destructor for class BabushkaIeee1588
//---------------------------------------------------------------------------

BabushkaIeee1588::~BabushkaIeee1588()
{
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::RegisterRead() - update cache from babushka register map
//---------------------------------------------------------------------------

void  BabushkaIeee1588::RegisterRead(MemoryCell & reg)
{
    int register_addr = RegBaseOfs + reg.Offset();
    RegCache[reg.Offset()] = Link.ReadRegister(register_addr);
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::RegisterWrite() - update cache from babushka register map
//---------------------------------------------------------------------------

void  BabushkaIeee1588::RegisterWrite(MemoryCell & reg)
{
    int register_addr = RegBaseOfs + reg.Offset();
    Link.WriteRegister(register_addr, RegCache[reg.Offset()]);
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::Port(ch) - Write character (polled)
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::Port(unsigned char ch)
//{
//    WaitForTxReady();

//    UartTxReg.TxData.Value(ch);
//    RegisterWrite(UartTxReg);
//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::Port() - Read character (polled)
//---------------------------------------------------------------------------

//unsigned char  BabushkaIeee1588::Port()
//{
//    WaitForRxDataReady();

//    RegisterRead(UartRxReg);
//    unsigned char ch = UartRxReg.RxData.Value();
//    return ch;
//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::PutString() - Write string to serial link
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::PutString(const std::string & data)
//{
//    size_t idx=0;
//    unsigned int failures(0);

//    while (idx < data.size())
//        {
//        unsigned int room = TxFreeSpace();
//        unsigned int to_go = static_cast<unsigned int>(data.size()-idx);
//        unsigned int transfer_size = std::min(room, to_go);

//        // if sending isn't working, keep track
//        if (transfer_size==0)
//            failures++;
//        else
//            failures = 0;
//        //  do a max-sized transfer
//        for (size_t i=0; i<transfer_size; i++)
//            {
//            UartTxReg.TxData.Value(data[idx++]);
//            RegisterWrite(UartTxReg);
//            }

//        if (idx>=data.size())    // should never be greater-than
//            break;
//        if (failures>=5)
//            {
//            // Gave up
//            std::stringstream msg;
//            msg << std::hex << " Babushka Uart Tx Send Failure (Full >5 times)";
//            Debug.Log(msg.str());
//            break;
//            }

//        WaitForTxReady();   // wait for some characters to come free
//        }

//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::GetString() - Read accumulated string from serial link
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::GetString(std::string & data)
//{
//    data.clear();   // wipe old data

//    unsigned int depth = RxDepth();
//    data.reserve(depth+1);

//    // transfer all you can
//    for (size_t i=0; i<depth; i++)
//        {
//        WaitForRxDataReady();
//        RegisterRead(UartRxReg);
//        data.push_back(UartRxReg.RxData.Value());
//        }

//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::GetFixedString() - Read fixed sized string
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::GetFixedString(std::string & data, size_t length)
//{
//    data.clear();   // wipe old data

//    unsigned int depth = length;
//    data.reserve(depth+1);

//    // transfer all you can
//    for (size_t i=0; i<depth; i++)
//        {
//        WaitForRxDataReady();
//        RegisterRead(UartRxReg);
//        data.push_back(UartRxReg.RxData.Value());
//        }

//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::WaitForTxReady() -- wait for Tx Ready Bit
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::WaitForTxReady()
//{
//    // Wait up to 10ms for Tx Ready
//    for (unsigned int i = 0; i < 10; ++i)
//        {
//        RegisterRead(UartTxReg);
//        bool bit = UartTxReg.TxReady.Value();
//        if (bit==true)
//            return;
//        else
//            {
//            Innovative::Sleep(1);
//            }
//        }
//    // Gave up
//    std::stringstream msg;
//    msg << std::hex << " Bab Uart Tx Ready Wait Failure (>10ms)";
//    Debug.Log(msg.str());

//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::WaitForRxDataReady() -- wait for Rx Data Ready Bit
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::WaitForRxDataReady()
//{
//    // Wait up to 10ms for Rx Ready
//    for (unsigned int i = 0; i < 10; ++i)
//        {
//        RegisterRead(UartControlReg);
//        bool bit = UartControlReg.RxReady.Value();
//        if (bit==true)
//            {
////            std::stringstream msg;
////            msg << std::hex << " Bab RX Data Ready Waits:" << i;
////            Debug.Log(msg.str());
//            return;
//            }
//        else
//            {
//            Innovative::Sleep(1);
//            }
//        }
//    // Gave up
//    std::stringstream msg;
//    msg << std::hex << " Bab Uart Rx Data Ready Wait Failure (>10ms)";
//    Debug.Log(msg.str());

//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::FrameError() -- return frame error bit
//---------------------------------------------------------------------------

//bool  BabushkaIeee1588::FrameError()
//{
//    RegisterRead(UartRxReg);
//    return UartRxReg.Error.Value();
//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::ClearFrameError() -- clear frame error bit
//---------------------------------------------------------------------------

//void  BabushkaIeee1588::ClearFrameError()
//{
//    UartRxReg.ClearFrameError();
//    RegisterWrite(UartRxReg);
//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::SetBaud() -- set UART Baud.
//---------------------------------------------------------------------------

//void BabushkaIeee1588::SetBaud(unsigned int baudrate)
//{
//    unsigned int reg_val = static_cast<unsigned int>(baudrate*UartRefClock + 0.5f);
//    UartControlReg.BaudDivisor.Value(reg_val - 1);
//    RegisterWrite(UartControlReg);
//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::GetBaud() -- read UART baud
//---------------------------------------------------------------------------

//unsigned int BabushkaIeee1588::GetBaud()
//{
//    RegisterRead(UartControlReg);
//    unsigned int reg_val = UartControlReg.BaudDivisor.Value() + 1;
//    return static_cast<unsigned int>(reg_val/UartRefClock + 0.5f);
//}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::GetFifoDepth() -- read current FIFO depth
//---------------------------------------------------------------------------

//unsigned int BabushkaIeee1588::GetFifoDepth()
//{
//    RegisterRead(UartControlReg);
//    {
//    std::stringstream msg;
//    msg << std::hex << " Bab Fifo Depth:" << UartControlReg.RxCount.Value();
//    Debug.Log(msg.str(),16);
//    }
//    return UartControlReg.RxCount.Value();
//}


//---------------------------------------------------------------------------
//  BabushkaIeee1588::Enable() - enable IEEE
//---------------------------------------------------------------------------

void  BabushkaIeee1588::Enable(bool state)
{
    ControlReg.Enable.Value(state);
    RegisterWrite(ControlReg);
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::Reset() - reset IEEE PLL
//---------------------------------------------------------------------------

void  BabushkaIeee1588::Reset()
{
    ControlReg.PllReset.Value(false);
    RegisterWrite(ControlReg);
    ControlReg.PllReset.Value(true);
    RegisterWrite(ControlReg);
    ControlReg.PllReset.Value(false);
    RegisterWrite(ControlReg);
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::InitIeee1588() -
//---------------------------------------------------------------------------

void BabushkaIeee1588::InitIeee1588()
{
    Debug.Log("BabushkaIeee1588::InitIeee1588()");
    // Hard Code Link baud rate
    Link.SetLinkBaud(115200);  // hardcoded

    //  Reset Link's App Reset
    Link.WriteRegister(0x1, 0x1);
    Sleep(1);
    Link.WriteRegister(0x1, 0x0);
    Sleep(1);


    bool locked = PllLocked();
    if (!locked)
        {
        Debug.Log("...PLL not locked, reset");
        Reset();

        Innovative::Sleep(5);

        locked = PllLocked();
        if (!locked)
            {
            Debug.Log("ERROR: PLL still not locked");
            return;
            }
        }

    Debug.Log("IEEE Enabled...");
    Enable(true);
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::ClockID() - return full ClockID number
//---------------------------------------------------------------------------

ii64  BabushkaIeee1588::ClockID()
{
    RegisterRead(ClockId_Hi_Reg);
    ii64 result = ClockId_Hi_Reg.Value();
    result <<= 32;
    RegisterRead(ClockId_Lo_Reg);
    result |= ClockId_Lo_Reg.Value();
    return result;
}


//---------------------------------------------------------------------------
//  BabushkaIeee1588::PllLocked() - return PLL locked bit
//---------------------------------------------------------------------------

bool  BabushkaIeee1588::PllLocked()
{
    RegisterRead(ControlReg);
    return ControlReg.PllLock.Value();
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::Linked() - return Linked bit
//---------------------------------------------------------------------------

bool  BabushkaIeee1588::Linked()
{
    RegisterRead(ControlReg);
    return ControlReg.Linked.Value();
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::InSync() - return InSync bit
//---------------------------------------------------------------------------

bool  BabushkaIeee1588::InSync()
{
    RegisterRead(ControlReg);
    return ControlReg.InSync.Value();
}

//---------------------------------------------------------------------------
//  BabushkaIeee1588::Gigabit() - return InSync bit
//---------------------------------------------------------------------------

bool  BabushkaIeee1588::Gigabit()
{
    RegisterRead(ControlReg);
    return ControlReg.Gigabit.Value();
}

//===========================================================================
//  CLASS SbcDuoLTC2494 --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuoLTC2494
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  SbcDuoLTC2494::SbcDuoLTC2494() -- Constructor
//---------------------------------------------------------------------------

SbcDuoLTC2494::SbcDuoLTC2494(IAddressingSpace &space,LTC2494SpiDevice::Info &regs, std::string tag)
    : LTC2494(space,regs,tag)
{

}

//---------------------------------------------------------------------------
//  SbcDuoLTC2494::~SbcDuoLTC2494() -- Destructor
//---------------------------------------------------------------------------

SbcDuoLTC2494::~SbcDuoLTC2494()
{

}

//---------------------------------------------------------------------------
//  SbcDuoLTC2494::ConfigureBoardDefualts() -- config board defualts
//---------------------------------------------------------------------------

void SbcDuoLTC2494::ConfigureBoardDefaults()
{
     //Channel Mapping
     ChannelTypeTable.push_back(SingleEnded); //Channel 0
     ChannelTypeTable.push_back(SingleEnded); //Channel 1
     ChannelTypeTable.push_back(SingleEnded); //Channel 2
     ChannelTypeTable.push_back(SingleEnded); //Channel 3
     ChannelTypeTable.push_back(SingleEnded); //Channel 4
     ChannelTypeTable.push_back(SingleEnded); //Channel 5
     ChannelTypeTable.push_back(SingleEnded); //Channel 6
     ChannelTypeTable.push_back(SingleEnded); //Channel 7
     ChannelTypeTable.push_back(SingleEnded); //Channel 8
     ChannelTypeTable.push_back(SingleEnded); //Channel 9
     ChannelTypeTable.push_back(SingleEnded); //Channel 10
     ChannelTypeTable.push_back(SingleEnded); //Channel 11
     ChannelTypeTable.push_back(SingleEnded); //Channel 12
     ChannelTypeTable.push_back(SingleEnded); //Channel 13
     ChannelTypeTable.push_back(SingleEnded); //Channel 14
     ChannelTypeTable.push_back(SingleEnded); //Channel 15
     MapChannels();

     //configuring defualt bits
     ConfigBits.EN  = 1;
     ConfigBits.EN2 = 1;
     ConfigBits.FA  = 0;
     ConfigBits.FB  = 0;
     ConfigBits.SPD = 0; // dont care if temp measurement
     ConfigBits.GS0 = 0; //
     ConfigBits.GS1 = 0; // dont care if temp measurement
     ConfigBits.GS2 = 0; //

    //initialize Scale table
    ScaleTable.push_back(1.25);
    ScaleTable.push_back(1.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(6.25);
    ScaleTable.push_back(1.25);
    ScaleTable.push_back(1.25);
    ScaleTable.push_back(2.5);
    ScaleTable.push_back(5.41);
    ScaleTable.push_back(5.41);
    ScaleTable.push_back(2.5);
    ScaleTable.push_back(5.41);
    ScaleTable.push_back(5.41);
    ScaleTable.push_back(14.37);
    ScaleTable.push_back(-14.37);


}

//===========================================================================
//  CLASS SbcDuoDeviceMap  -- Register & low level support for Sbc_ComEx
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SbcDuoDeviceMap
//---------------------------------------------------------------------------

using namespace SbcDuo_MemoryMap;

namespace  // nameless
{
X6Info::MapRegisters SbcDuoInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmSubrevision,        //  BitstreamDate_Addr
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

X6Peripherals::MapRegisters SbcDuoPeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

K7FpaLoader::MapRegisters SbcDuoVsPromRegAddrs =
    {
    mmVsProm_ControlAddr,  //  AddressControl_Addr;
    mmVsProm_Data,         //  Data_Addr
    mmVsProm_Opcode,       //  Opcode_Addr;
    };

SbcDuoLMK20_Clock::MapRegisters SbcDuoLMKClockRegAddrs =
    {
    mmPllControl,    //  AfePllControl_Addr;
    mmPllSpi,        //  AfePllSpi_Addr
    };

Lmk04820Registers  SbcDuo_Lmk04820Registers =
    {
    mmPllControl,    //  Control_Addr;
    mmPllSpi,        //  SpiIntf_Addr
    };

SbcDuo_XmcI2C::MapRegisters SbcDuoXmcI2CRegAddrs =
    {
    mmXmc0_I2C,       //  XMC0_Addr
    mmXmc1_I2C,       //  XMC1_Addr
    };

LTC2494SpiDevice::Info SbcDuoSpiRegAddrs =
{
    mmLTC_SPI_Control,  //Write Offset
    mmLTC_SPI_Stat,     //Read Offset
};

SbcDuo_QSfp::MapRegisters SbcDuoQSfpRegAddrs =
    {
    mmQSfp0L_Control,         //  QSfpPortControl_0L_Addr;
    mmQSfp0L_I2C,             //  QSfpPortI2C_0L_Addr;
    mmQSfp0H_Control,         //  QSfpPortControl_0H_Addr;
    mmQSfp0H_I2C,             //  QSfpPortI2C_0H_Addr;
    mmQSfp1L_Control,         //  QSfpPortControl_1L_Addr;
    mmQSfp1L_I2C,             //  QSfpPortI2C_1L_Addr;
    mmQSfp1H_Control,         //  QSfpPortControl_1H_Addr;
    mmQSfp1H_I2C,             //  QSfpPortI2C_1H_Addr;
    };

SbcDuoTriggerSystem::MapRegisters SbcDuoTriggerRegAddrs =
    {
    mmTriggerCtrl,            //  SoftwareTrigger_Addr
    mmTriggerMode,            //  TriggerConfig_Addr;
    mmTriggerDecimation,      //  TriggerDecimation_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters SbcDuoPriRegAddrs =
    {
    mmPri_Enable,          //  PriTriggerEnable_Addr;
    mmPri_Interval,        //  PriInterval_Addr;
    mmPri_TrigParam,       //  PriTriggerParam_Addr;
    mmPri_CaptureWin       //  PriCaptureWindow_Addr;
    };

SbcDuoPeripherals::MapRegisters SbcDuoCustomPeriphRegAddrs =
    {
    mmTimingControl,           //  TimingControl_Addr;
    mm10GbEthernet,            //  X10GbEthernet_Addr;
    mmInterruptSourceSel,      //  InterruptSourceSel_Addr;
    };

X6Thermal::MapRegisters K7ThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

SbcDuoGpsUart::MapRegisters SbcDuoGpsUartRegAddrs =
    {
    mmGpsUart0_Ctl,            //  Uart0Ctrl_Addr;
    mmGpsUart0_Tx,             //  Uart0Tx_Addr;
    mmGpsUart0_Rx,             //  Uart0Rx_Addr;
    mmGpsUart1_Ctl,            //  Uart1Ctrl_Addr;
    mmGpsUart1_Tx,             //  Uart1Tx_Addr;
    mmGpsUart1_Rx,             //  Uart1Rx_Addr;
    mmGpsUart_Status,          //  UartStatus_Addr;
    mmGpsUart_IntStatus,       //  UartInterrupt_Addr;
    mmGpsUart_PpsArm,          //  UartPpsArm_Addr
    };

X6DigitalIo::MapRegisters SbcDuoDioRegAddrs =
    {
    mmDio_DataLo,          //  DioDataLo_Addr
    mmDio_DataHi,          //  DioDataHi_Addr
    mmDio_OutEnableLo,     //  DioOutputEnableLo_Addr
    mmDio_OutEnableHi,     //  DioOutputEnableHi_Addr
    };

K7DualUart::MapRegisters SbcDuoDualUartAddrs =
    {
    mmUart_0_Control,       //   Uart0Ctrl_Addr;
    mmUart_0_Tx,            //   Uart0Tx_Addr;
    mmUart_0_Rx,            //   Uart0Rx_Addr;
    mmUart_1_Control,       //   Uart1Ctrl_Addr;
    mmUart_1_Tx,            //   Uart1Tx_Addr;
    mmUart_1_Rx,            //   Uart1Rx_Addr;
    mmUartMode,             //   UartMode_Addr;
    mmInterrupt,            //   UartInterrupt_Addr;
    mmGpioData,             //   GpioData_Addr;
    mmGpioDirection,        //   GpioDir_Addr;
    };

EEpromIntfStyle ModernStyle = intfModernStyle;

BabushkaIeee1588::MapRegisters SbcDuoBabushkaIeee1588Addrs =
    {
    wbBabushkaBase,           //   UartWishbone_Base;
    mmBab1588_Ctrl,           //   Ctrl_Addr
    mmBab1588_ClkId_Lo,       //   ClkId_Lo_Addr;
    mmBab1588_ClkId_Hi,       //   ClkId_Hi_Addr;
    mmBab1588_Time,           //   Time_Addr;
    mmBab1588_EEPromI2C,      //   EEPromI2C_Addr
    };

}  // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class SbcDuoDeviceMap
//---------------------------------------------------------------------------

SbcDuoDeviceMap::SbcDuoDeviceMap()
    :  IUsesPmcEeprom(WB_Periph, mmIDRom, idrom_ParameterSize, ModernStyle),
       DeviceDriver(new SbcDuoDeviceDriver),
       WB_Periph             (LogicMemory, wbPeripheral),
       WB_Loader             (LogicMemory, wbLogicLoader),
       WB_Thermal            (LogicMemory, wbTempControl),
       WB_Dio                (LogicMemory, wbDio),
       WB_GpsUart            (LogicMemory, wbGpsUart),
       WB_RfXcvrUart         (LogicMemory, wbRfXcvrUart),
       Info_SDev             (this, WB_Periph, PortMemory, SbcDuoInfoRegAddrs),
       Periph_SDev           (this, WB_Periph, SbcDuoPeripheralRegAddrs),
       DuoPeriph_SDev        (this, WB_Periph, SbcDuoCustomPeriphRegAddrs),
       AFEClock_SDev         (this, WB_Periph, SbcDuoLMKClockRegAddrs),
       XmcI2C_SDev           (this, WB_Periph, SbcDuoXmcI2CRegAddrs),
       QSfp_SDev             (this, WB_Periph, SbcDuoQSfpRegAddrs),
       Trigger_SDev          (this, WB_Periph, SbcDuoTriggerRegAddrs),
       SelfMeasure_SDev      (this),
       Pri_SDev              (this, true, WB_Periph, SbcDuoPriRegAddrs),
       K7Thermal_SDev        (this, WB_Thermal, K7ThermalRegAddrs),
       VsProm_SDev           (this, K7FpaLoader::romS25FL128S_Sys, WB_Loader, SbcDuoVsPromRegAddrs),
       Dio_SDev              (this, WB_Dio, SbcDuoDioRegAddrs),
       Uart_SDev             (this, WB_RfXcvrUart, SbcDuoDualUartAddrs),
       SerialReg             (GpsUart(0), GpsUart(1)),
       BabIeee1588           (SerialReg,  SbcDuoBabushkaIeee1588Addrs),
       Gps_SDev              (this, new SbcDuoGPS_Callback(this), WB_GpsUart, SbcDuoGpsUartRegAddrs),
       PciControlReg         (PortMemory, mmPciControl),
	   InterruptStatusReg    (PortMemory, mmInterruptStatus),
       InterruptEnableReg    (PortMemory, mmInterruptEnable),
       DuoMezz(Dio_SDev),DuoAssemblyInfo(*this),
       Isr(*this), Debug(true)
{
    ConstructCustomParts();
}

SbcDuoDeviceMap * aDuo()
{
    SbcDuoDeviceMap * wombat = new SbcDuoDeviceMap;
    return wombat;
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuoDeviceMap
//---------------------------------------------------------------------------

SbcDuoDeviceMap::~SbcDuoDeviceMap()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t SbcDuoDeviceMap::BoardCount()
{
	return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  SbcDuoDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  SbcDuoDeviceMap::Name() const
{
    switch (Info().PciLogicType())
        {
        case 5: return "SbcDuo";
        default: return "Unknown";
        }
//    return "Unknown";
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  SbcDuoDeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Open()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Open()
{
    Debug.Log("SbcDuoDeviceMap::Open()");
    if(DeviceDriver->Opened())
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));
    LogicMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->LogicMemoryAddress()));

    InterruptEnable(false);
    Isr.Open();

    //
    //  After Open Behavior
    OpenHardware();

}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Close()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Close()
{
    Debug.Log("SbcDuoDeviceMap::Close()");
    if(!DeviceDriver->Opened())
        return;

    InterruptEnable(false);

    Isr.Close();
    DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Reset()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Reset()
{
    Debug.Log("+SbcDuoDeviceMap::Reset()");
    Debug.Log("-SbcDuoDeviceMap::Reset()");
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::MReset() - Master Reset
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::MReset()
{
    Debug.Log("+SbcDuoDeviceMap::MReset()");
    PciControlReg.BusmasterReset.Value(false);  // apply BM Reset (active low)
    PciControlReg.BusmasterReset.Value(true);   // release BM Reset
    PciControlReg.MasterReset.Value(true);      // Apply "Board" reset (autoclear)
    PciControlReg.MasterReset.Value(false);     // but we need to clear shadow
    Debug.Log("-SbcDuoDeviceMap::MReset()");
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Led()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Led(bool value)
{
    PciControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Led()
//---------------------------------------------------------------------------

bool  SbcDuoDeviceMap::Led() const
{
    return PciControlReg.Led;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Start() - turn on clocks & triggers
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Start()
{
    SbcDuo_Pll & DuoPll = dynamic_cast<SbcDuo_Pll &>(RawClockDevice().Pll());
    DuoPll.EnableAllOutputs(true);

    PreconfigureHardware();
    ConfigureHardware();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Stop()  turn off clocks & triggers
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Stop()
{
    SbcDuo_Pll & DuoPll = dynamic_cast<SbcDuo_Pll &>(RawClockDevice().Pll());
    DuoPll.EnableAllOutputs(false);

    StopHardware();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  SbcDuoDeviceMap::Target() const
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 SbcDuoDeviceMap::PeekPort(int offset)
{
    ii32 * PortBasePtr = PortMemory.Base();
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void SbcDuoDeviceMap::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = PortMemory.Base();
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32 SbcDuoDeviceMap::PeekLogic(int offset)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    return LogicBasePtr[offset];
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void SbcDuoDeviceMap::PokeLogic(int offset, ii32 value)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    LogicBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::InterruptEvent() --
//---------------------------------------------------------------------------

Event * SbcDuoDeviceMap::InterruptEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::InterruptDepth() --
//---------------------------------------------------------------------------

size_t SbcDuoDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::InterruptAck() --
//---------------------------------------------------------------------------

unsigned int SbcDuoDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void SbcDuoDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    ScopedTraceRef ccc(Debug, "SbcDuoDeviceMap::InterruptDispatch()");
    // dispatch UART interrupt
    Gps_SDev.Gps().DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void SbcDuoDeviceMap::InterruptEnable(bool state)
{
    ScopedTraceRef ccc(Debug, "SbcDuoDeviceMap::InterruptEnable()");
    Debug.Log("state=", (int)state);

    InterruptEnableReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    AFEClock_SDev.ConstructCustomParts();
    XmcI2C_SDev.ConstructCustomParts();
    QSfp_SDev.ConstructCustomParts();
    Trigger_SDev.ConstructCustomParts();
    Pri_SDev.ConstructCustomParts();
    K7Thermal_SDev.ConstructCustomParts();
    Gps_SDev.ConstructCustomParts();
    Uart_SDev.ConstructCustomParts();
    SelfMeasure_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::OpenHardware()
{
    Debug.Log("+SbcDuoDeviceMap::OpenHardware()");
    //
    //  Reset the device
    MReset();
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    AFEClock_SDev.OpenHardware();
    XmcI2C_SDev.OpenHardware();
    QSfp_SDev.OpenHardware();
    Trigger_SDev.OpenHardware();
    Pri_SDev.OpenHardware();
    K7Thermal_SDev.OpenHardware();
    Gps_SDev.OpenHardware();
    Uart_SDev.OpenHardware();
    SelfMeasure_SDev.OpenHardware();
    //
    //  Read ROM to Init Cache
    IdRom().LoadFromRom();
    Debug.Log("-SbcDuoDeviceMap::OpenHardware()");
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    AFEClock_SDev.PreconfigureHardware();
    XmcI2C_SDev.PreconfigureHardware();
    QSfp_SDev.PreconfigureHardware();
    Trigger_SDev.PreconfigureHardware();
    Pri_SDev.PreconfigureHardware();
    K7Thermal_SDev.PreconfigureHardware();
    Gps_SDev.PreconfigureHardware();
    Uart_SDev.PreconfigureHardware();
    SelfMeasure_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    AFEClock_SDev.ConfigureHardware();
    XmcI2C_SDev.ConfigureHardware();
    QSfp_SDev.ConfigureHardware();
    Trigger_SDev.ConfigureHardware();
    Pri_SDev.ConfigureHardware();
    K7Thermal_SDev.ConfigureHardware();
    Gps_SDev.ConfigureHardware();
    Uart_SDev.ConfigureHardware();
    SelfMeasure_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::StopHardware() -- set hardware for 'off' condition
//---------------------------------------------------------------------------

void  SbcDuoDeviceMap::StopHardware()
{
    // Subdevice Configuration
    //Periph_SDev.StopHardware();
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::MakeFmcAfePllObject() -- Get PLL Device object
//---------------------------------------------------------------------------

Lmk04820 *  SbcDuoDeviceMap::MakeFmcAfePllObject()
{
    return new SbcDuo_Pll(this, WB_Periph, SbcDuo_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  SbcDuoDeviceMap::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions  (we don't need an object)
    return new SbcDuoFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  AtroposDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  SbcDuoDeviceMap::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new SbcDuoPriClockDevice(this, 1);
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::MakeLTC2494() -- Get LTC2494 Device object
//---------------------------------------------------------------------------

LTC2494 *  SbcDuoDeviceMap::MakeLTC2494()
{
    return new SbcDuoLTC2494(WB_Periph, SbcDuoSpiRegAddrs, "SbcDuo_LTC2494");
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::GetGpsCount() --
//---------------------------------------------------------------------------
//  Count of GPS devices to be created and installed

size_t  SbcDuoDeviceMap::GetGpsCount()
{
    return 2;
}

//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::MakeOneGpsDevice() -- Build custom Gps
//---------------------------------------------------------------------------
//  Will be called once per GPS device set up.  Use IDX to distinguish them

IGps *  SbcDuoDeviceMap::MakeOneGpsDevice(size_t idx)
{
    //  Make our local Gps objects (subdev will delete it)
    if (idx==0)
        return new Gps_CSACPlugin(new SbcDuoCsacPlugin(&Gps_SDev));
    else
        return new Gps_Ieee1588Plugin(new SbcDuoIeee1588Plugin(&Gps_SDev, &Babushka()) );

}
//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::MakeGpsDevice() -- Build custom Gps
//---------------------------------------------------------------------------

//IGps *  SbcDuoDeviceMap::MakeGpsDevice()
//{
//    ScopedTraceRef ccc(Debug, "SbcDuoDeviceMap::MakeGpsDevice()");
//    //  Make our local Gps object (subdev will delete it)
//    return new Gps_CSACPlugin(new SbcDuoCsacPlugin(&Gps_SDev));

//}


//---------------------------------------------------------------------------
//  SbcDuoDeviceMap::Uart()
//---------------------------------------------------------------------------

IMulticharUart & SbcDuoDeviceMap::Uart(unsigned int ch)
{
    return Uart_SDev.Uart(ch);
}



//==============================================================================
//  CLASS AssemblyInfo -- Assembly information
//==============================================================================

//---------------------------------------------------------------------------
//  constructor for class AssemblyInfo
//---------------------------------------------------------------------------

AssemblyInfo::AssemblyInfo(IUsesPmcEeprom &IdRom)
    : Rom(IdRom),AssemblyPartNum_HeaderOffset(0),
    AssemblySerial_NumHeaderOffset(1),
    Module0PartNum_HeaderOffset(2),
    Module0Serial_NumHeaderOffset(3),
    Module1PartNum_HeaderOffset(4),
    Module1SerialNum_HeaderOffset(5),
    CpuPartNum_HeaderOffset(6),
    CpuSerialNum_HeaderOffset(7),
    PowerSupplyPartNum_HeaderOffset(8),
    PowerSupplySerialNum_HeaderOffset(9),
    GpsPartNum_HeaderOffset(10),
    GpsSerialNum_HeaderOffset(11),
    MezzaninePartNum_HeaderOffset(12),
    MezzanineSerialNum_HeaderOffset(13)
{

}

//---------------------------------------------------------------------------
//  destructor for class AssemblyInfo
//---------------------------------------------------------------------------

AssemblyInfo::~AssemblyInfo()
{

}

//---------------------------------------------------------------------------
//  AssemblyInfo::AssemblyPart() --
//---------------------------------------------------------------------------

void AssemblyInfo::AssemblyPartNumber(std::string Num)
{
    int idx =  (AssemblyPartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::AssemblyPart() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::AssemblyPartNumber()
{
    int idx = (AssemblyPartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::AssemblySerial() --
//---------------------------------------------------------------------------

void AssemblyInfo::AssemblySerialNumber(std::string Num)
{
    int idx = (AssemblySerial_NumHeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::AssemblySerial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::AssemblySerialNumber()
{
    int idx = (AssemblySerial_NumHeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module0Part() --
//---------------------------------------------------------------------------

void AssemblyInfo::Module0PartNumber(std::string Num)
{
    int idx = (Module0PartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module0Part() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::Module0PartNumber()
{
    int idx = (Module0PartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module0Serial() --
//---------------------------------------------------------------------------

void AssemblyInfo::Module0SerialNumber(std::string Num)
{
    int idx = (Module0Serial_NumHeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module0Serial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::Module0SerialNumber()
{
    int idx = (Module0Serial_NumHeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module1Part() --
//---------------------------------------------------------------------------

void AssemblyInfo::Module1PartNumber(std::string Num)
{
    int idx = (Module1PartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::ModulePart() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::Module1PartNumber()
{
    int idx = (Module1PartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module1Serial() --
//---------------------------------------------------------------------------

void AssemblyInfo::Module1SerialNumber(std::string Num)
{
    int idx = (Module1SerialNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::Module1Serial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::Module1SerialNumber()
{
    int idx = (Module1SerialNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::CpuPart() --
//---------------------------------------------------------------------------

void AssemblyInfo::CpuPartNumber(std::string Num)
{
    int idx = (CpuPartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::AssemblyInfo() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::CpuPartNumber()
{
    int idx = (CpuPartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::CpuSerial() --
//---------------------------------------------------------------------------

void AssemblyInfo::CpuSerialNumber(std::string Num)
{
    int idx = (CpuSerialNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::CpuSerial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::CpuSerialNumber()
{
    int idx = (CpuSerialNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::PowerSupplyPart() --
//---------------------------------------------------------------------------

void AssemblyInfo::PowerSupplyPartNumber(std::string Num)
{
    int idx = (PowerSupplyPartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::PowerSupplyPart() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::PowerSupplyPartNumber()
{
    int idx = (PowerSupplyPartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::PowerSupplySerial() --
//---------------------------------------------------------------------------

void AssemblyInfo::PowerSupplySerialNumber(std::string Num)
{
    int idx = (PowerSupplySerialNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::PowerSupplySerial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::PowerSupplySerialNumber()
{
    int idx = (PowerSupplySerialNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::GpsPart() --
//---------------------------------------------------------------------------

void AssemblyInfo::GpsPartNumber(std::string Num)
{
    int idx = (GpsPartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::GpsPart() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::GpsPartNumber()
{
    int idx = (GpsPartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::GpsSerial() --
//---------------------------------------------------------------------------

void AssemblyInfo::GpsSerialNumber(std::string Num)
{
    int idx = (GpsSerialNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::GpsSerial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::GpsSerialNumber()
{
    int idx = (GpsSerialNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::MazzaninePart() --
//---------------------------------------------------------------------------

void AssemblyInfo::MezzaninePartNumber(std::string Num)
{
    int idx = (MezzaninePartNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::MazzaninePart() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::MezzaninePartNumber()
{
    int idx = (MezzaninePartNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

//---------------------------------------------------------------------------
//  AssemblyInfo::MazzanineSerial() --
//---------------------------------------------------------------------------

void AssemblyInfo::MezzanineSerialNumber(std::string Num)
{
    int idx = (MezzanineSerialNum_HeaderOffset * AssemblyRegionWordSize);
    Rom.Rom().AsString(idx,AssemblyRegionWordSize,Num);
}

//---------------------------------------------------------------------------
//  AssemblyInfo::MazzanineSerial() --
//---------------------------------------------------------------------------

std::string AssemblyInfo::MezzanineSerialNumber()
{
    int idx = (MezzanineSerialNum_HeaderOffset * AssemblyRegionWordSize);
    std::string Num = Rom.Rom().AsString(idx,AssemblyRegionWordSize);

    return Num;
}

} // namespace Innovative


