// SbcDuoDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef SbcDuoDeviceMap_MbH
#define SbcDuoDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <FmcSubDevice_Stream_Mb.h>      // LMK Clock
//#include <K7Support_Mb.h>
#include <K7SubDevice_Misc_Mb.h>
#include <PmcEeprom_Mb.h>
#include <SimpleInterrupt_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class SbcDuoDeviceMap;

//==============================================================================
//  CLASS SbcDuo_Pll -- Duo PLL Chip (Specialization of Lmk04820)
//==============================================================================

class SbcDuo_Pll : public Lmk04820
{
public:
    SbcDuo_Pll(SbcDuoDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~SbcDuo_Pll();

    void  EnableAllOutputs(bool enabled);

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    SbcDuoDeviceMap *   Owner;
//    double              FTrueFrequency;
//    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);


    virtual double ADC_TRIM(double rate);

};

//==============================================================================
//  CLASS SbcDuo_Mezzanine -- Duo Mezzanine
//==============================================================================

class SbcDuo_Mezzanine
{
public:
    enum XmcVSel_options { xvs_3p3v=0x1, xvs_2p5v=0x2, xvs_1p8v=0x4, xvs_1p5v=0x8 };
    enum IoVSel_options  { ivs_5v=0x1,   ivs_3p3v=0x2, ivs_2p5v=0x4, ivs_1p8v=0x8 };

    SbcDuo_Mezzanine(AdvancedDioPortIntf & owner_dio);
    virtual ~SbcDuo_Mezzanine();

    void  Active(bool state);   // master on/off
    void  Xmc_Direction(unsigned int xmc, unsigned int bit, bool isOutput);

    void  Xmc_VSel(unsigned int xmc, XmcVSel_options opts);
    void  Io_VSel(IoVSel_options opts);

    void  XmcEnable(unsigned int xmc, bool state);
    void  OutputEnable(bool state);

private:
    AdvancedDioPortIntf & ODio;

};

//===========================================================================
//  CLASS SbcDuo_SerialRegister -- Use serial link to write to a register bank
//===========================================================================

class SbcDuo_SerialRegister
{
public:
    SbcDuo_SerialRegister(IMulticharUart & uart_tx, IMulticharUart & uart_rx)
        :  Uart_Tx(uart_tx), Uart_Rx(uart_rx)
        {
        RxData.resize(6);
        TxData.resize(2);
        TxData_X.resize(6);
        }
    ~SbcDuo_SerialRegister()
        {}

    ii32  ReadRegister(unsigned int address);
    void  WriteRegister(unsigned int address, ii32 data);

    void           SetLinkBaud(unsigned int baudrate)
                        {  Uart_Tx.SetBaud(baudrate);  Uart_Rx.SetBaud(baudrate); }
    unsigned int   GetLinkBaud()
                        {  return Uart_Rx.GetBaud();  }

private:
    IMulticharUart & Uart_Tx;
    IMulticharUart & Uart_Rx;
    Trace Debug;

    std::string  RxData;
    std::string  TxData;
    std::string  TxData_X;
};

//===========================================================================
//  CLASS BabushkaIeee1588 -- Access IEEE1588 registers and UART
//===========================================================================
//  using uart link to remote registers

class BabushkaIeee1588
{
public:
    //=======================================================================
    //  CLASS BabushkaIeee1588::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   UartWishbone_Base;
        unsigned int   Ctrl_Addr;
        unsigned int   ClkId_Lo_Addr;
        unsigned int   ClkId_Hi_Addr;
        unsigned int   Time_Addr;
        unsigned int   EEPromI2C_Addr;
    };

    //
    //  Member Functions
    BabushkaIeee1588(SbcDuo_SerialRegister & link, MapRegisters regs);
    virtual ~BabushkaIeee1588();

    //IMulticharUart & Uart()
    //                    {  return BabUart0;  }

    // status
    ii64           ClockID();
    bool           PllLocked();
    bool           InSync();
    bool           Linked();
    bool           Gigabit();

    // control
    void           Enable(bool state);
    void           Reset();

    void           InitIeee1588();

    //
    //  BabushkaUartImpl Methods
//    void                   Init();

//    virtual void           Port(unsigned char ch);
//    virtual unsigned char  Port();

//    virtual void           PutString(const std::string & data);
//    virtual void           GetString(std::string & data);

//    virtual void           GetFixedString(std::string & data, size_t length);

//    virtual bool           FrameError();
//    virtual void           ClearFrameError();

//    virtual void           SetBaud(unsigned int baudrate);
//    virtual unsigned int   GetBaud();

//    unsigned int GetFifoDepth();

    //===========================================================================
    //  CLASS BabushkaUart -- Interface for "String Based" Uart
    //===========================================================================

/*    class BabushkaUart : public IMulticharUart
    {
    public:
        BabushkaUart(BabushkaIeee1588 & owner)
            :  Owner(owner)
              {}
        virtual ~BabushkaUart() {}

        virtual void           SetBaud(unsigned int rate)
                                    {  Owner.SetBaud(rate);  }
        virtual unsigned int   GetBaud()
                                    {  return Owner.GetBaud();  }
        virtual void           Port(unsigned char ch)
                                    {  Owner.Port(ch);  }
        virtual unsigned char  Port()
                                    {  return Owner.Port();  }

        virtual void           PutString(const std::string & data)
                                    {  Owner.PutString(data);  }
        virtual void           GetString(std::string & data)
                                    {  Owner.GetString(data);  }

        virtual void           GetFixedString(std::string & data, size_t chars)
                                    {  Owner.GetFixedString(data, chars);  }

        virtual bool           FrameError()
                                    {  return Owner.FrameError();  }
        virtual void           ClearFrameError()
                                    {  Owner.ClearFrameError();  }

    private:
        BabushkaIeee1588 & Owner;

    };
*/

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BabushkaIeee1588::ControlRegister  --  1588 Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Enable;
        RegisterBit        PllReset;
        RegisterBit        PllLock;
        RegisterBit        Mode;
        RegisterBitGroup   SwMode;
        RegisterBit        InSync;
        RegisterBit        Linked;
        RegisterBit        Gigabit;
        RegisterBit        TestEnable;

        ControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Enable(*this, 0),  PllReset(*this, 1), PllLock(*this, 2),
              Mode(*this, 4), SwMode(*this, 8, 3),
              InSync(*this, 12), Linked(*this, 13), Gigabit(*this, 14),
              TestEnable(*this, 16)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BabushkaIeee1588::I2CRegister  --  1588 ROM Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class I2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Data_Out;
        RegisterBit        Clock_Out;
        RegisterBit        Data_In;
        RegisterBit        Clock_In;
        RegisterBit        DeviceEnable;

        I2CRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Data_Out(*this, 0),  Clock_Out(*this, 1), Data_In(*this, 2),
              Clock_In(*this, 4),  DeviceEnable(*this, 5)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ieee_I2C_Impl  -- Vcxo I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ieee_I2C_Impl : public I2CDeviceIntf
    {
    public:
        Ieee_I2C_Impl(I2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~Ieee_I2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        I2CRegister &  Reg;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BabushkaUart::UartControlRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   BaudDivisor;
        RegisterBitGroup   RxCount;
        RegisterBit        RxReady;

        UartControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              BaudDivisor(*this, 0, 16),
              RxCount(*this, 20, 10),
              RxReady(*this, 31)
                {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BabushkaUart::UartTxRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartTxRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   TxData;
        RegisterBitGroup   TxCount;
        RegisterBit        TxReady;

        UartTxRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              TxData(*this, 0, 8),
              TxCount(*this, 8, 4),   // field 8, 4 used.
              TxReady(*this, 31)
                {  }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BabushkaUart::UartRxRegister  --   register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartRxRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   RxData;
        RegisterBitGroup    InterruptThreshold;
        RegisterBit         Error;

        UartRxRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              RxData(*this, 0, 8),
              InterruptThreshold(*this, 8, 4),
              Error(*this, 31)
                {  }

        void ClearFrameError()
            {  Value(Value());  }
    };

private:
    SbcDuo_SerialRegister & Link;
    //BabushkaUart            BabUart0;

    unsigned int         RegCache[10];
    unsigned int         RegBaseOfs;
    AddressingSpace      UartMem;

    ControlRegister      ControlReg;
    Register             ClockId_Lo_Reg;
    Register             ClockId_Hi_Reg;
    Register             TimeReg;
    I2CRegister          I2CReg;
//    UartControlRegister  UartControlReg;
//    UartTxRegister       UartTxReg;
//    UartRxRegister       UartRxReg;
//    Register             Interrupt_Reg;
//    Register             TriggerArm_Reg;
    Trace                Debug;

//    void  WaitForTxReady();
//    void  WaitForRxDataReady();
//    unsigned int TxFreeSpace()
//        {
//        RegisterRead(UartTxReg);
//        return 16 - UartTxReg.TxCount.Value();
//        }
//    unsigned int RxDepth()
//        {
//        RegisterRead(UartControlReg);
//        return UartControlReg.RxCount.Value();
//        }

    void  RegisterRead(MemoryCell & reg);
    void  RegisterWrite(MemoryCell & reg);




};


//===========================================================================
//  CLASS AssemblyInfo  -- manage the information for assembly
//===========================================================================

class AssemblyInfo
{
public:
    AssemblyInfo(IUsesPmcEeprom &IdRom);
    ~AssemblyInfo();

    void AssemblyPartNumber(std::string Part);
    std::string AssemblyPartNumber();

    void AssemblySerialNumber(std::string Serial);
    std::string AssemblySerialNumber();

    void Module0PartNumber(std::string Part);
    std::string Module0PartNumber();

    void Module0SerialNumber(std::string Serial);
    std::string Module0SerialNumber();

    void Module1PartNumber(std::string Part);
    std::string Module1PartNumber();

    void Module1SerialNumber(std::string Serial);
    std::string Module1SerialNumber();

    void CpuPartNumber(std::string Part);
    std::string CpuPartNumber();

    void CpuSerialNumber(std::string Serial);
    std::string CpuSerialNumber();

    void PowerSupplyPartNumber(std::string Part);
    std::string PowerSupplyPartNumber();

    void PowerSupplySerialNumber(std::string Serial);
    std::string PowerSupplySerialNumber();

    void GpsPartNumber(std::string Part);
    std::string GpsPartNumber();

    void GpsSerialNumber(std::string Serial);
    std::string GpsSerialNumber();

    void MezzaninePartNumber(std::string Part);
    std::string MezzaninePartNumber();

    void MezzanineSerialNumber(std::string Serial);
    std::string MezzanineSerialNumber();

private:
    IUsesPmcEeprom   &Rom;
    const unsigned int AssemblyPartNum_HeaderOffset;
    const unsigned int AssemblySerial_NumHeaderOffset;
    const unsigned int Module0PartNum_HeaderOffset;
    const unsigned int Module0Serial_NumHeaderOffset;
    const unsigned int Module1PartNum_HeaderOffset;
    const unsigned int Module1SerialNum_HeaderOffset;
    const unsigned int CpuPartNum_HeaderOffset;
    const unsigned int CpuSerialNum_HeaderOffset;
    const unsigned int PowerSupplyPartNum_HeaderOffset;
    const unsigned int PowerSupplySerialNum_HeaderOffset;
    const unsigned int GpsPartNum_HeaderOffset;
    const unsigned int GpsSerialNum_HeaderOffset;
    const unsigned int MezzaninePartNum_HeaderOffset;
    const unsigned int MezzanineSerialNum_HeaderOffset;

};

//===========================================================================
//  CLASS SbcDuoDeviceMap  -- Register support for K7 System Unit
//===========================================================================

class SbcDuoDeviceMap :  public ISuppliesSbcDuoRequirements,
                         public IUsesPmcEeprom,
                         public ISimpleInterrupt
{
public:
    friend class SbcDuo;
    friend class SbcDuoGPS_Callback;
        //
    //  Member Functions
    SbcDuoDeviceMap(/*X * owner*/);
    virtual ~SbcDuoDeviceMap();

    // Methods
    virtual size_t  InterruptDepth();
	size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);

    void            Open();
    void            Close();
    void            Reset();
    void            Start();
    void            Stop();

    std::string         Name() const;
    std::string         Revision() const;

    void            Led(bool value);
    bool            Led() const;

    void  SelectGps(size_t idx)
        {
        Gps_SDev.SelectGps(idx);
        }

    //  Hardware Device Access
    const IX6LogicVersionInfo &           Info() const      {  return Info_SDev;  }
    IFpaLoader &                          VsProm()          {  return VsProm_SDev.VsProm();  }
    IUsesPmcEeprom &                      IdRom()           {  return *this;   }
    IX6ThermalDevice &                    Thermal()         {  return K7Thermal_SDev;  }
    SbcDuoLMK20_Clock::SbcDuoClockIo &    Clock()           {  return AFEClock_SDev.Clock();  }
    SbcDuoLMK20_Clock::RawClockDevices &  RawClockDevice()  {  return AFEClock_SDev.RawClockDevice();  }
    AdvancedDioPortIntf &                 Dio()             {  return Dio_SDev;  }
    IMulticharUart &                      Uart(unsigned int ch);
    K7DioPortIntf &                       Gpio()            {  return Uart_SDev;  }
    SbcDuo_Mezzanine &                    Mezz()            {  return DuoMezz;  }
    ISbcDuoSourceSelector &               Selector()        {  return DuoPeriph_SDev.Selector();  }
    ISbcDuoTriggerCfg &                   Trigger()         {  return Trigger_SDev.Trigger();  }
    IPulseRepetitionIntervalIntf &        Pulse()           {  return Pri_SDev;  }
    IMulticharUart &                      GpsUart(int dev)  {  return Gps_SDev.Uart(dev);  }
    AssemblyInfo &                        AsmInfo()         {  return DuoAssemblyInfo;  }
    BabushkaIeee1588 &                    Babushka()        {  return BabIeee1588;  }
    ISelfMeasurement &                    SelfMeasure()     {  return SelfMeasure_SDev.SelfM();  }


    //  Subdevice Customization Handlers
    virtual Lmk04820 *               MakeFmcAfePllObject();
    virtual class IPriClockDevice *  MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *       MakeFrameCalcDevice(FrameDeviceCode which);
    virtual LTC2494          *       MakeLTC2494();

    virtual size_t             GetGpsCount();
    virtual IGps *             MakeOneGpsDevice(size_t idx);

    //  Debug Support
    ii32            PeekPort(int offset);
    void            PokePort(int offset, ii32 value);
    ii32            PeekLogic(int offset);
    void            PokeLogic(int offset, ii32 value);

    // Properties
    void            Target(int target);
    int             Target() const;

	AddressingSpace    PortMemory;
    AddressingSpace    LogicMemory;

   void   MReset();    // Master Reset

   virtual double  StandardClockFrequency()
                       {  return AFEClock_SDev.Clock().Frequency();   }

protected:
    //
    //  Member Data
    InnovativeKernel::DualRegionIsrDeviceDriver *  DeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Loader;
    WishboneBusSpace       WB_Thermal;
    WishboneBusSpace       WB_Dio;
    WishboneBusSpace       WB_GpsUart;
    WishboneBusSpace       WB_RfXcvrUart;
    //
    //  Subdevices
    X6Info                          Info_SDev;
    X6Peripherals                   Periph_SDev;
    SbcDuoPeripherals               DuoPeriph_SDev;
    SbcDuoLMK20_Clock               AFEClock_SDev;
    SbcDuo_XmcI2C                   XmcI2C_SDev;
    SbcDuo_QSfp                     QSfp_SDev;
    SbcDuoTriggerSystem             Trigger_SDev;
    SelfMeasurementDevice           SelfMeasure_SDev;
    X6AFE_PulseRepetitionInterval   Pri_SDev;
    X6Thermal                       K7Thermal_SDev;
    K7FpaLoader                     VsProm_SDev;
    X6DigitalIo                     Dio_SDev;
    K7DualUart                      Uart_SDev;
    SbcDuo_SerialRegister           SerialReg;
    BabushkaIeee1588                BabIeee1588;
    SbcDuoGpsUart                   Gps_SDev;
    //
    //  Registers
    //
	//  Class PciControlRegister
    class PciControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       MasterReset;
        RegisterBit       Led;
        RegisterBit       BusmasterReset;

	public:
        PciControlRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
               MasterReset( *this, 0 ),
               Led( *this, 1 ),
               BusmasterReset( *this, 3 )
            {
            //  Default setup ????
            }
    };

	//  ...Interrupt Registers
    PciControlRegister                 	PciControlReg;
    Register                            InterruptStatusReg;
    ShadowRegister                      InterruptEnableReg;

    SbcDuo_Mezzanine                    DuoMezz;
    AssemblyInfo                        DuoAssemblyInfo;

private:
    // Interface
    virtual Event *         InterruptEvent();
    virtual unsigned int    InterruptAck();
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);

    void    ConstructCustomParts();
    void    OpenHardware();
    void    PreconfigureHardware();
    void    ConfigureHardware();
    void    StopHardware();

    // Data
    SimpleInterrupt         Isr;
    Trace                   Debug;
};

//=============================================================================
//  CLASS SbcDuoFrameCalcDevice  --  Supply Divider on Frame ticks
//=============================================================================

class SbcDuoFrameCalcDevice : public IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor()
        {  return Divisor;  }

    SbcDuoFrameCalcDevice(SbcDuoDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~SbcDuoFrameCalcDevice()
        {}

private:
    SbcDuoDeviceMap * Owner;
    int Divisor;
};

//=============================================================================
//  CLASS SbcDuoDeviceMap  --  Normal impl to access clock for PRI
//=============================================================================

class SbcDuoPriClockDevice : public IPriClockDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->StandardClockFrequency();  }
    virtual int   CalculatePriDivisor()
        {  return Divisor;  }

    SbcDuoPriClockDevice(SbcDuoDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~SbcDuoPriClockDevice()
        {}

private:
    SbcDuoDeviceMap * Owner;
    int Divisor;
};

//===========================================================================
//  CLASS SbcDuoLTC2494 -- 16-Channel ADC chip for Sbc Duo
//===========================================================================

class SbcDuoLTC2494 : public LTC2494
{
public:
    //ctor
    //ctor
    SbcDuoLTC2494(IAddressingSpace &space,LTC2494SpiDevice::Info &Regs,
                  std::string tag="");
    ~SbcDuoLTC2494();

private:
    //private members
    virtual void                ConfigureBoardDefaults();

};



#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
