//---------------------------------------------------------------------------
// SbcDuo_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <SbcDuo_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS SbcDuo  -- X6 Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class SbcDuo
//---------------------------------------------------------------------------

SbcDuo::SbcDuo()
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuo
//---------------------------------------------------------------------------

SbcDuo::~SbcDuo()
{
}

//---------------------------------------------------------------------------
//  SbcDuo::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t SbcDuo::BoardCount()
{
	return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  SbcDuo::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> SbcDuo::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  SbcDuo::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string SbcDuo::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  SbcDuo::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  SbcDuo::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  SbcDuo::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  SbcDuo::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  SbcDuo::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  SbcDuo::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  SbcDuo::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  SbcDuo::GreenLed()
{
    return Device.Periph_SDev.FrontPanelLed();
}

//---------------------------------------------------------------------------
//  SbcDuo::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  SbcDuo::GreenLed( bool value )
{
    Device.Periph_SDev.FrontPanelLed(value);
}

//---------------------------------------------------------------------------
//  SbcDuo::BlueLed() -- Read the Blue LED state
//---------------------------------------------------------------------------

bool  SbcDuo::BlueLed()
{
    return Device.Periph_SDev.GreenLed();  // yes, according to docs they flip
}

//---------------------------------------------------------------------------
//  SbcDuo::BlueLed() -- Set the Blue LED state of the board
//---------------------------------------------------------------------------

void  SbcDuo::BlueLed( bool value )
{
    Device.Periph_SDev.GreenLed(value);  // yes, according to docs they flip
}

//---------------------------------------------------------------------------
//  SbcDuo::MasterTrigger(bool state) -- Set the Master enable for Trigger
//---------------------------------------------------------------------------

void  SbcDuo::MasterTrigger(bool state)
{
    return Device.Trigger_SDev.MasterTrigger(state);
}

//---------------------------------------------------------------------------
//  SbcDuo::SoftwareTrigger() -- Set the software trigger state on/off
//---------------------------------------------------------------------------

void  SbcDuo::SoftwareTrigger(bool state)
{
    Device.Trigger_SDev.SoftwareTrigger(state);
}

//---------------------------------------------------------------------------
//  SbcDuo::ArmPpsTrigger() -- Set the PPS trigger to activate on next GPS
//---------------------------------------------------------------------------

void  SbcDuo::ArmPpsTrigger()
{
    Device.Trigger_SDev.ArmPpsTrigger();
}

//---------------------------------------------------------------------------
//  SbcDuo::CustomInterruptSource() -- change interrupt source
//---------------------------------------------------------------------------

void  SbcDuo::CustomInterruptSource(SbcDuoInterruptSources src)
{
    return Device.DuoPeriph_SDev.CustomInterruptSource(static_cast<unsigned int>(src));
}

//---------------------------------------------------------------------------
//  SbcDuo::CustomInterruptSource() -- read current interrupt source
//---------------------------------------------------------------------------

SbcDuo::SbcDuoInterruptSources  SbcDuo::CustomInterruptSource()
{
    unsigned int src = Device.DuoPeriph_SDev.CustomInterruptSource();
    return static_cast<SbcDuoInterruptSources>(src);
}

//---------------------------------------------------------------------------
//  SbcDuo::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  SbcDuo::Open()
{
    Device.Open();
}

//---------------------------------------------------------------------------
//  SbcDuo::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  SbcDuo::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  SbcDuo::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  SbcDuo::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  SbcDuo::Start() -- Start the clocks and triggers
//---------------------------------------------------------------------------

void  SbcDuo::Start()
{
    Device.Start();
}

//---------------------------------------------------------------------------
//  SbcDuo::Stop() -- Stop the clocks and triggers
//---------------------------------------------------------------------------

void  SbcDuo::Stop()
{
    Device.Stop();
}

//---------------------------------------------------------------------------
//  SbcDuo::GpsCtrl() -- Return reference to Gps Control
//---------------------------------------------------------------------------

IX6_GpsIo &  SbcDuo::GpsCtrl()
{
    return Device.Gps_SDev;
}


//---------------------------------------------------------------------------
//  SbcDuo::Info() --
//---------------------------------------------------------------------------

const IX6LogicVersionInfo & SbcDuo::Info() const
{
    return Device.Info();
}

//---------------------------------------------------------------------------
//  SbcDuo::VsProm() --
//---------------------------------------------------------------------------

IFpaLoader &  SbcDuo::VsProm()
{
	return Device.VsProm();
}

//---------------------------------------------------------------------------
//  SbcDuo::Thermal() --
//---------------------------------------------------------------------------

IX6ThermalDevice &  SbcDuo::Thermal()
{
	return Device.Thermal();
}

//---------------------------------------------------------------------------
//  SbcDuo::Dio() -- DIO port access
//---------------------------------------------------------------------------

AdvancedDioPortIntf & SbcDuo::Dio()
{
    return Device.Dio();
}

//---------------------------------------------------------------------------
//  SbcDuo::Uart() - access UARTs on Duo
//---------------------------------------------------------------------------

IMulticharUart & SbcDuo::Uart(unsigned int device)
{
    return Device.Uart(device);
}

//---------------------------------------------------------------------------
//  SbcDuo::IdRom() --
//---------------------------------------------------------------------------

IUsesPmcEeprom & SbcDuo::IdRom()
{
    return Device.IdRom();
}

//---------------------------------------------------------------------------
//  SbcDuo::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  SbcDuo::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  SbcDuo::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  SbcDuo::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  SbcDuo::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  SbcDuo::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  SbcDuo::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  SbcDuo::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  SbcDuo::AfterFiclStart() --
//---------------------------------------------------------------------------

void  SbcDuo::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate(" 0 constant Pll  1 constant Fan 2 constant XmcI2c0 3 constant XmcI2c1 4 constant SerLink");

    engine.Evaluate(": ?SbcDuo "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll Fan XmcI2c0  XmcI2c1 SerLink\" "
                    "    cr  "
                    "    cr .\" ---- SbcDuo Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?SbcDuo  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("SbcDuo-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    //X6Board::AfterFiclStart(engine);
    //  Replace Callbacks to call FICL too
    //Device.Load_StreamDeviceEventCallbacks(
    //                new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  SbcDuo::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  SbcDuo::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  SbcDuo::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  SbcDuo::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  SbcDuo::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;  const int kFan = 1; const int kXmcI2c0 = 2; const int kXmcI2c1 = 3;
    const int kSerLink = 4;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Device.RawClockDevice().Pll().Port(addr);
        case kFan:
            return Device.Thermal().Port(addr);
        case kXmcI2c0:
            return Device.XmcI2C_SDev.Device(0).Reg(addr);
        case kXmcI2c1:
            return Device.XmcI2C_SDev.Device(1).Reg(addr);
        case kSerLink:
            return SRL().ReadRegister(addr);
//        case I1588Rom:
//            return Device.Ieee1588().Rom(addr);
        }
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  SbcDuo::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;  const int kFan = 1; const int kXmcI2c0 = 2; const int kXmcI2c1 = 3;
    const int kSerLink = 4;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.RawClockDevice().Pll().Port(addr, value);
            break;
        case kFan:
            Device.Thermal().Port(addr, value);
            break;
        case kXmcI2c0:
            Device.XmcI2C_SDev.Device(0).Reg(addr, value);
            break;
        case kXmcI2c1:
            Device.XmcI2C_SDev.Device(1).Reg(addr, value);
            break;
        case kSerLink:
            return SRL().WriteRegister(addr, value);
//        case I1588Rom:
//            Device.Ieee1588().Rom(addr, value);
//            break;
        }
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  SbcDuo::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.RawClockDevice().Pll().Frequency(freq);
            Device.RawClockDevice().Pll().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  SbcDuo::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  SbcDuo::Do_DFreqActual(int dev)
{
    const int kPll = 0;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.RawClockDevice().Pll().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
