// SbcDuo_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2015 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SbcDuo_MbH
#define SbcDuo_MbH

#include <DataTypes_Mb.h>
#include <SbcDuoDeviceMap_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS SbcDuo  -- SbcDuo baseboard class
//===========================================================================

class SbcDuo : public IFiclTarget
{

public:
    // Enumerations
    enum SbcDuoInterruptSources { sdInt_FpgaPps, sdInt_GpsPps, sdInt_GpsUart, sdInt_Uart2,
                                   };
    enum GpsType { gpsCSAC, gpsIeee1588 };

    //
    //  Member Functions
    SbcDuo();
    virtual ~SbcDuo();

    //  Informational Methods
	size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
	bool                        LicenseCheck(unsigned int & code);

    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    int                         Target();
    void                        Target(int value);

    bool                        GreenLed();
    void                        GreenLed(bool value);

    bool                        BlueLed();
    void                        BlueLed(bool value);

    void                        CustomInterruptSource(SbcDuoInterruptSources src);
    SbcDuoInterruptSources      CustomInterruptSource();

    void                        MasterTrigger(bool state);
    void                        SoftwareTrigger(bool state);
    void                        ArmPpsTrigger();

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    virtual void                Start();
    virtual void                Stop();

    void  SelectGps(GpsType gps)
        {
        Device.SelectGps(static_cast<size_t>(gps));
        }
    void  SelectGps(size_t idx)
        {
        Device.SelectGps(idx);
        }

    //
    //  Properties

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    const IX6LogicVersionInfo &           Info() const;
    SbcDuoLMK20_Clock::SbcDuoClockIo &    Clock()           {  return Device.Clock();  }
    SbcDuoLMK20_Clock::RawClockDevices &  RawClockDevice()  {  return Device.RawClockDevice();  }
    IAuroraSfp *                          Sfp(unsigned int idx) { return Device.QSfp_SDev.Sfp(idx);  }
    AdvancedDioPortIntf &                 Dio();
    IX6_GpsIo &                           GpsCtrl();
    IFpaLoader &                          VsProm();
    IX6ThermalDevice &                    Thermal();
    IUsesPmcEeprom &                      IdRom();
    IMulticharUart &                      Uart(unsigned int device);
    K7DioPortIntf &                       Gpio()          {  return Device.Gpio();  }
    SbcDuo_Mezzanine &                    Mezz()          {  return Device.Mezz();  }
    ISbcDuoSourceSelector &               Selector()      {  return Device.Selector();  }
    ISbcDuoTriggerCfg &                   Trigger()       {  return Device.Trigger();  }
    IPulseRepetitionIntervalIntf &        Pulse()         {  return Device.Pulse();  }
    AssemblyInfo &                        AsmbInfo()      {  return Device.AsmInfo(); }

    ISelfMeasurement &                    SelfMeasure()    {  return Device.SelfMeasure(); }

    BabushkaIeee1588 &                    Babushka()        {  return Device.Babushka();  }

    SbcDuo_SerialRegister &               SRL() {  return Device.SerialReg;  }

public: //Pseudo protected

    SbcDuoDeviceMap * Debug() { return &Device; }

    SbcDuoGpsUart &   GpsT() { return Device.Gps_SDev;  }
			
    //  FPGA type name function
    std::string FpgaName(const short Number)
    {
         switch(Number)
         {
              case 0x01: return "LX25T";
              default: return "Unknown";
         }
     }

protected:
    //
    //  Member Data
    SbcDuoDeviceMap       Device;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( SbcDuo & board );
    friend AddressingSpace & PortMemorySpace( SbcDuo & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( SbcDuo & board )
{
    return board.Device.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( SbcDuo & board )
{
    return board.Device.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

