// Sbc_ComExDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DeviceDriver_Mb.h>
#include <Sbc_ComExDeviceMap_Mb.h>
#include <Sbc_ComEx_Mb.h>
#include <Sbc_ComExMemoryMap.h>
#include <Sbc_ComExDeviceDriver_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <DioPort_Mb.h>
#include <SystemSupport_Mb.h>
#include <cmath>
#include <Trace_Mb.h>
#include <time.h>
#include <Ad9511_Mb.h>
#include <TycoGps_Mb.h>
#include <Gps_MiniT_Mb.h>
#include <Gps_CW25_Mb.h>
#include <Gps_Csac_Mb.h>
#include <Mutex_Mb.h>
#include <Tuner_MMS_Mb.h>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

//---------------------------------------------------------
//  Clock and Module Constants Definition

const float  InputReferenceClk = 100.0e6;
const float  InputVcoMinClk    = 100.0e6;
const float  InputVcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Name, P/N, S/N
//       0x40 spare
const int rom_ParameterSize    = 0x40;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romSBC_ComEx_ClockParamOffset = rom_ParameterSize;
const int romSBC_ComEx_ClockParamSize = 4;


//=============================================================================
//  CLASS Sbc_ComExClock  --  PLL timebase control
//=============================================================================

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;    //

class Sbc_ComExClock : public Ad9511Pfd
{
public:
    //
    //  Member Functions
    Sbc_ComExClock(Sbc_ComExDeviceMap * owner, int index, AddressingSpace &space, int ctl)
        : Ad9511Pfd(index, space, ctl), Owner(owner), Debug(true)
        {
        Reference(InputReferenceClk);
        Min(InputVcoMinClk/MaxOutputDivider());
        Max(InputVcoMaxClk);
        PFDPolarity(true);
        }
    Sbc_ComExClock(Sbc_ComExDeviceMap * owner, double freq, double ref,
        bool enabled, int index, AddressingSpace & space, int ctl)
        : Ad9511Pfd(freq, ref, enabled, index, space, ctl), Owner(owner), Debug(true)
        {
        Min(InputVcoMinClk/MaxOutputDivider());
        Max(InputVcoMaxClk);
        PFDPolarity(true);
        }

    //  Set clock output modes and signaling types
        void ConfigureOutputs(int /*outdiv_bits*/, int outdiv_bypass)
    {
        // Customize duty cycle and phase for each output
        for (int i = 0; i < 5; ++i)
            {
            int factor = static_cast<int>(Owner->ClockDivider(i));
            int divisor = static_cast<int>(outdiv*factor);
            std::stringstream msg;
            msg << "PLL divisor[" << i << "] =" << divisor;
            if (divisor > 32)
                msg << ", [overrange, clamped at 32]";
            Debug.Log(msg.str());
            divisor = std::max<int>(std::min<int>(divisor, 32), 1);

            // encode outdiv for register use
            const int odd = (divisor < 32) ? (divisor & 1) : 0;
            const int lo = (divisor >> 1) - 1;
            const int hi = lo + odd;
            int outdiv_bits = ( (hi << 4) | lo );

            Port(0x4a+i*2, outdiv_bits);	    // duty cycle
            Port(0x4a+(i*2+1), (factor==1) ? outdiv_bypass : 0);	// phase
            }

        Port(0x3D, 0x08);       // Clock 0 (FPGA):      LVPECL on @805mV
        Port(0x3E, 0x08);       // Clock 1 (Front0):    LVPECL on @805mV
        Port(0x3F, 0x08);       // Clock 2 (Front1):    LVPECL on @805mV
        Port(0x40, 0x02);       // Clock 3 (Xmc0):      LVCMOS on @3.5 mA
        Port(0x41, 0x02);       // Clock 4 (Xmc1):      LVCMOS on @3.5 mA
    }

    void SetReferenceCalibrationFactor(double value)
    {
    Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
    // set calibrated field
    Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));
    }
    double GetReferenceCalibrationFactor() const
    {
    //  Check Calibrated bit
    if (Owner->ClockParamRomSection.AsInt(1)=='Y')
        {
        return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
        }
    else
        return 1.0;  // default value
    }

        void  InputMuxReady(bool /*state*/) // hold down while programmin clock mux.
    {
    //ControlReg.Bit(3, state);
    }

protected:
    // Data
    Sbc_ComExDeviceMap *    Owner;
    Trace                   Debug;

    virtual	double  PhaseDetectorFrequency() const
    {	return 100.0e3; }

};

//=============================================================================
//  CLASS Sbc_GpsTyco  --  Tyco A1029 GPS interface
//=============================================================================

class Sbc_GpsTyco : public TycoGps
{
public:
    Sbc_GpsTyco(Sbc_ComExDeviceMap * owner)
        : Owner(owner)  {  }

    virtual void  OnInit();
    virtual void  OnReapply()  {  }
    virtual void  OnGpsTrigger()
        {  Owner->ArmGpsTrigger();  }
    virtual unsigned int  UartFifoDepth()
        {  return Owner->UartStatusReg.RxDepth.Value();  }
    virtual void          UartData(char c)
        {  Owner->UartData(c);  }
    virtual char          UartData()
        {  return Owner->UartData();  }
    virtual void          UartBaud(BaudRates baud)
        {
        Debug.Log("Tyco Baud : ", (int)baud);
        Owner->Baud(static_cast<Sbc_ComExDeviceMap::IIBaud>(baud));
        }
    virtual BaudRates     UartBaud()
        {  return static_cast<IGps::BaudRates>(Owner->Baud());  }
    virtual void          HdwEnabled(bool /*state*/)
        {      }
    virtual bool          HdwEnabled()
        {  return Owner->GpsEnabled();  }
    virtual bool          IsAntenna() const
        {  return Owner->GpsStatusReg.Antenna.Value() ? true : false;  }
    virtual bool          IsLocked() const
        {  return Owner->GpsStatusReg.Locked.Value() ? true : false;  }

protected:
    // Data
    Sbc_ComExDeviceMap *     Owner;

    // Overrides
    virtual void    Emit(char c)
        {  UartData(c);  }
};

//---------------------------------------------------------------------------
// Sbc_GpsTyco::OnInit()
//---------------------------------------------------------------------------

void Sbc_GpsTyco::OnInit()
{
    Debug.Log("Tyco On Init");
    TycoGps::OnInit();
    MutexedLock  Lock(mLock);
    FInitializing = true;
}

//=============================================================================
//  CLASS Sbc_GpsMiniT  --  Trimble miniT GPS interface
//=============================================================================

class Sbc_GpsMiniT : public Gps_MiniT
{
public:
    Sbc_GpsMiniT(Sbc_ComExDeviceMap * owner);

    virtual void  OnInit();
    virtual void  OnReapply()
        {  }
    virtual void  OnGpsTrigger()
        {  Owner->ArmGpsTrigger();  }
    virtual unsigned int  UartFifoDepth()
        {  return Owner->UartStatusReg.RxDepth.Value();  }
    virtual void UartData(char c)
        {  Owner->UartData(c);  }
    virtual char UartData()
        {  return Owner->UartData();  }
    virtual void UartBaud(BaudRates baud)
        {  Owner->Baud(static_cast<Sbc_ComExDeviceMap::IIBaud>(baud));  }
    virtual BaudRates UartBaud()
        {  return static_cast<IGps::BaudRates>(Owner->Baud());  }
    virtual void HdwEnabled(bool /*state*/)
        {  }
    virtual bool HdwEnabled()
        {  return Owner->GpsEnabled();  }

protected:
    // Data
    Sbc_ComExDeviceMap *     Owner;

    // Overrides
    virtual void    Emit(char c)
        {  UartData(c);  }

    std::vector<unsigned char>	buffTimeOn;
    std::vector<unsigned char>	buffSuplementalTime;
    std::vector<unsigned char>  buffGpsOn;
};

//---------------------------------------------------------------------------
// Sbc_GpsMiniT::Sbc_GpsMiniT()
//---------------------------------------------------------------------------

Sbc_GpsMiniT::Sbc_GpsMiniT(Sbc_ComExDeviceMap * owner)
    : Owner(owner), buffTimeOn(9), buffSuplementalTime(6),buffGpsOn(20)
{
    buffTimeOn[0] = 0x10; buffTimeOn[1] = 0x8E; buffTimeOn[2] = 0xA5;
    buffTimeOn[3] = 0x05; buffTimeOn[4] = 0x05; buffTimeOn[5] = 0x00;
    buffTimeOn[6] = 0x00; buffTimeOn[7] = 0x10; buffTimeOn[8] = 0x03;

    buffSuplementalTime[0] = 0x10;	buffSuplementalTime[1] = 0x8E;
    buffSuplementalTime[2] = 0xAC;	buffSuplementalTime[3] = 0x00;
    buffSuplementalTime[4] = 0x10;	buffSuplementalTime[5] = 0x03;

    buffGpsOn[0]  = 0x10; buffGpsOn[1]  = 0x8E; buffGpsOn[2]  = 0x4A;
    buffGpsOn[3]  = 0x01; buffGpsOn[4]  = 0x02; buffGpsOn[5]  = 0x00;
    buffGpsOn[6]  = 0x00; buffGpsOn[7]  = 0x00; buffGpsOn[8]  = 0x00;
    buffGpsOn[9]  = 0x00; buffGpsOn[10] = 0x00; buffGpsOn[11] = 0x00;
    buffGpsOn[12] = 0x00; buffGpsOn[13] = 0x00; buffGpsOn[14] = 0x43;
    buffGpsOn[15] = 0x96; buffGpsOn[16] = 0x00; buffGpsOn[17] = 0x00;
    buffGpsOn[18] = 0x10; buffGpsOn[19] = 0x03;
}

//---------------------------------------------------------------------------
// Sbc_GpsMiniT::OnInit()
//---------------------------------------------------------------------------

void Sbc_GpsMiniT::OnInit()
{
    for(size_t i = 0; i < buffTimeOn.size(); ++i)
        UartData(buffTimeOn[i]);
    for(size_t i = 0; i < buffSuplementalTime.size(); ++i)
        UartData(buffSuplementalTime[i]);
    for(size_t i = 0; i < buffGpsOn.size(); ++i)
        UartData(buffGpsOn[i]);
}

//=============================================================================
//  CLASS Sbc_GpsNavSync  --  NavSync CW25 GPS interface
//=============================================================================

class Sbc_GpsNavSync : public Gps_Cw25
{
    typedef Gps_Cw25 inherited;

protected:
    // Data
    Sbc_ComExDeviceMap *     Owner;

    // Overrides
    virtual void    Emit(char c)
        {  UartData(c);  }

public:
    Sbc_GpsNavSync(Sbc_ComExDeviceMap * owner)
        : Owner(owner)
        {  }

    virtual void  OnInit()
        {
        Debug.Log("CW25 On Init");
        UartBaud(b38400);
//		inherited::OnInit();
        }

    virtual void  OnReapply()
        {  }
    virtual void  		  OnGpsTrigger()
        {  Owner->ArmGpsTrigger();  }
    virtual unsigned int  UartFifoDepth()
        {  return Owner->UartStatusReg.RxDepth.Value();  }
    virtual void          UartData(char c)
        {  Owner->UartData(c);  }
    virtual char          UartData()
        {  return Owner->UartData();  }
    virtual void          UartBaud(BaudRates baud)
        {
        Debug.Log("CW25 Baud : ", (int)baud);
        Owner->Baud(static_cast<Sbc_ComExDeviceMap::IIBaud>(baud));
        }
    virtual BaudRates     UartBaud()
        {  return static_cast<IGps::BaudRates>(Owner->Baud());  }
    virtual void          HdwEnabled(bool /*state*/)
        {  }
    virtual bool          HdwEnabled()
        {  return Owner->GpsEnabled();  }
};


//=============================================================================
//  CLASS Sbc_GpsJacksonLabs  --  Sbc_GpsJacksonLabs CSAC GPS interface
//=============================================================================

class Sbc_GpsJacksonLabs : public Gps_CSAC
{
    typedef Gps_CSAC inherited;

protected:
    // Data
    Sbc_ComExDeviceMap *     Owner;

    // Overrides
    virtual void    Emit(char c)
        {  UartData(c);  }

public:
    Sbc_GpsJacksonLabs(Sbc_ComExDeviceMap * owner)
        : Owner(owner)
        {  }

    //void HandleRxData(RxDataEvent &event)

    virtual void  OnInit()
        {
        Debug.Log("CSAC On Init");
        Owner->PutString("SYNC:OUT:1PPS:RESET ON\r\n");    // PPS from power up
        Owner->PutString("GPS:GPZDA 1\r\n");              // emit GPZDA once per second
        Owner->PutString("GPS:GPGGA 1\r\n");              // emit GPGGA once per second
        }

    virtual void  OnReapply() { }

    virtual void  		  OnGpsTrigger()
        {  Owner->ArmGpsTrigger();  }

    virtual unsigned int  UartFifoDepth()
        { return Owner->UartStatusReg.RxDepth.Value();  }

    virtual void          UartData(char c)
        {  Owner->UartData(c); }

    virtual char          UartData()
        {  return Owner->UartData();  }

    virtual void          UartBaud(BaudRates baud)
        {
//        Debug.Log("CSAC Baud : ", (int)baud);
//        uartBaudRate = baud;
        Debug.Log("CSAC Baud : ", (int)baud);
        Owner->Baud(static_cast<Sbc_ComExDeviceMap::IIBaud>(baud));
        }

    virtual BaudRates     UartBaud()
        {  return static_cast<IGps::BaudRates>(Owner->Baud()); }

    virtual void          HdwEnabled(bool /*state*/)
        {  }
    virtual bool          HdwEnabled()
        {  return Owner->GpsEnabled();  }

private:
    BaudRates uartBaudRate;
};

//===========================================================================
//  CLASS Sbc_ComExInterrupt  -- Simple ISR handling
//===========================================================================

class Sbc_ComExInterrupt : public SimpleInterrupt
{
    typedef SimpleInterrupt inherited;

public:
    Sbc_ComExInterrupt(ISimpleInterrupt & device)
        : inherited(device)  {  }
};


//===========================================================================
// CLASS Sbc_ComExTimingInputMux -- Only present on Rev D and as option on Atom
//===========================================================================

using namespace Sbc_ComEx_MemoryMap;

//===========================================================================
//  CLASS Sbc_ComEx_ClockInputMux  -- Configurable Input Mux
//===========================================================================

static Cdce18005Registers Sbc_ComEx_ClockInputMuxRegs =
    { mmClockInputMux  };

//---------------------------------------------------------------------------
// Sbc_ComExTimingInputMux::Sbc_ComExTimingInputMux()
//---------------------------------------------------------------------------

Sbc_ComExTimingInputMux::Sbc_ComExTimingInputMux(Sbc_ComExDeviceMap * owner, AddressingSpace & space)
    : Cdce18005(space, Sbc_ComEx_ClockInputMuxRegs), Owner(owner), Debug(true)
{
}

//---------------------------------------------------------------------------
// Sbc_ComExTimingInputMux::OnInit()
//---------------------------------------------------------------------------

void Sbc_ComExTimingInputMux::OnInit()
{
    Debug.Log("ClockMux: OnInit");
    Owner->ClockInputMuxReady(false);
    Sleep(10);
    //
    //  Set Input Configuration
    SetInputConfig(icLvds);
    SetInputBufferConfig(true,  // DC Terminated
                         false, // Primary Term: internal
                         false, // Secondary Term: external
                         false, // Primary Bias off
                         false, // Secondary Bias off
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    SetOutputConfig(outXmc0Ref, ocLvds);
    SetOutputConfig(outXmc1Ref, ocLvds);
    SetOutputConfig(outFpga, ocLvPecl);
    SetOutputConfig(outPllRef, ocLvPecl);
    SetOutputConfig(outPllClk, ocLvPecl);

    SetOutputDividerSync(osPrimary);
    //  Configure Aux (from ?)
    SetAuxOutputConfig(aeDisabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    Source(outXmc0Ref, inOnBoardOscillator);
    Source(outXmc1Ref, inOnBoardOscillator);
    Source(outFpga, inOnBoardOscillator);
    Source(outPllRef, inOnBoardOscillator);
    Source(outPllClk, inFrontPanel);

    // Set default enable state
    OutputEnabled(outXmc0Ref, false);
    OutputEnabled(outXmc1Ref, false);
    OutputEnabled(outFpga, true);
    OutputEnabled(outPllRef, false);
    OutputEnabled(outPllClk, false);

    Owner->ClockInputMuxReady(true);
}

//---------------------------------------------------------------------------
// Sbc_ComExTimingInputMux::OnReapply()
//---------------------------------------------------------------------------

void Sbc_ComExTimingInputMux::OnReapply()
{
}

//---------------------------------------------------------------------------
// Sbc_ComExTimingInputMux::OnSetPower()
//---------------------------------------------------------------------------

void Sbc_ComExTimingInputMux::OnSetPower(bool & /*new_state*/)
{
//    Owner->InputMuxControlReg.PowerOn.Value(new_state);
    //  pause to allow chip to activate
    Sleep(10);
}

//---------------------------------------------------------------------------
// Sbc_ComExTimingInputMux::UpdateSources()
//---------------------------------------------------------------------------

void Sbc_ComExTimingInputMux::UpdateSources()
{
    SourceFrequency(muxPrimaryIn, InputReferenceClk);           // Onboard xtal
    SourceFrequency(muxSecondaryIn, Owner->Clock->Frequency()); // Front Panel
    SourceFrequency(muxAuxIn, Owner->Clock->Reference());       // Gps clock via PLL reference
}

//===========================================================================
//  CLASS Sbc_ComExDeviceMap  -- Register & low level support for Sbc_ComEx
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class Sbc_ComExDeviceMap
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::Sbc_ComExDeviceMap(Sbc_ComEx * owner)
    : IUsesVirtexJtagLoader(PortMemory, mmJtag),
      IUsesExtendedDioPort(PortMemory, mmDioData, mmDioDataHigh, mmDioConfig),
      IUsesPciLogicVersionInfo(PortMemory, mmPciVersion),
      IUsesPmcEeprom(PortMemory, mmI2c, rom_ParameterSize),
      Owner(owner),
      //Gps(new Sbc_GpsTyco(this)),
      Gps(new Sbc_GpsMiniT(this)),
      Tuner(0),
      //serialPort(new Innovative::SerialPort()),
      FReferenceSource(rsCrystal),
      FClockSource(csPll), FClockDivider(5, 1),
      Clock(new Sbc_ComExClock(this, 0, PortMemory, mmPllClockControl)),
      VcxoI2CReg( PortMemory, mmVcoI2C),
      VcxoIntf(VcxoI2CReg),
      Vcxo_I2C(VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      //Vcxo(0),
      Vcxo(new Si570(Vcxo_I2C, Si570::tcC, kVcxoPowerOnRate)),

      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romSBC_ComEx_ClockParamOffset, romSBC_ComEx_ClockParamSize),
      DeviceDriver(new Sbc_ComExDeviceDriver),
      PciControlReg(PortMemory, mmPciControl),
      PciStatusReg(PortMemory, mmPciStatus),
      ReferenceReg(PortMemory, mmPllReferenceControl),
      SoftwareTriggerReg(PortMemory, mmSoftwareTrigger),
      TriggerReg(PortMemory, mmTriggerControl),
      UartDataReg(PortMemory, mmUartData),
      UartControlReg(PortMemory, mmUartControl),
      UartStatusReg(PortMemory, mmUartStatus),
      GpsStatusReg(PortMemory, mmGpsStatus),
      GpsControlReg(PortMemory, mmGpsControl),
      GpsTriggerReg(PortMemory, mmGpsTrigger),
      InterruptStatusReg(PortMemory, mmInterruptStatus),
      InterruptEnableReg(PortMemory, mmInterruptEnable),
      ClockInputMux(this, PortMemory),
      FGpsInUse(gpsTrimble),
      Isr(*this), Debug(true)
{
}

//---------------------------------------------------------------------------
//  destructor for class Sbc_ComExDeviceMap
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::~Sbc_ComExDeviceMap()
{
    delete Clock;
    Clock = 0;

    delete DeviceDriver;
    DeviceDriver = 0;

    delete Gps;
    Gps = 0;

    delete Vcxo;
    Vcxo = 0;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t Sbc_ComExDeviceMap::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  Sbc_ComExDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Open()
//---------------------------------------------------------------------------

void  Sbc_ComExDeviceMap::Open()
{
    if(DeviceDriver->Opened())
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));

    InterruptEnable(false);
    Isr.Open();

    // Load proper VCO interface.
    if (EnhancedClock())
        {
        Debug.Log("SI570 version detected");
        delete Vcxo;
        if (PciLogicType() == tEnhancedPentiumA)
            {
            Vcxo = new Si570(Vcxo_I2C, Si570::tcA, kVcxoPowerOnRate);
            Clock->Max(1417.5e6);
            }
        else
            {
            Vcxo = new Si570(Vcxo_I2C, Si570::tcC, kVcxoPowerOnRate);
            Clock->Max(280.5e6);
            }
        Clock->LoadVco(new IVcxoControlProxy(Vcxo));
        }

    VcxoI2CReg.Enable.Set();
    Vcxo->Init();
    ClockInputMux.Init();
}

//---------------------------------------------------------------------------
// Sbc_ComeExDeviceMap::ReapplyClock()
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ReapplyClock()
{
    if (EnhancedClock())
        Vcxo->Reapply();
}

//---------------------------------------------------------------------------
// Sbc_ComeExDeviceMap::EnhancedClock()
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::EnhancedClock()
{
    return (PciLogicType() == tAtom) ||
           (PciLogicType() == tEnhancedPentium) ||
           (PciLogicType() == tEnhancedPentiumA);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Close()
//---------------------------------------------------------------------------

void  Sbc_ComExDeviceMap::Close()
{
    if(!DeviceDriver->Opened())
        return;

    InterruptEnable(false);

    Isr.Close();
    DeviceDriver->Close();


}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Reset()
//---------------------------------------------------------------------------

void  Sbc_ComExDeviceMap::Reset()
{
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Sbc_ComExDeviceMap::Target() const
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Sbc_ComExDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Led()
//---------------------------------------------------------------------------

void  Sbc_ComExDeviceMap::Led(bool value)
{
    PciControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Led()
//---------------------------------------------------------------------------

bool  Sbc_ComExDeviceMap::Led() const
{
    return PciControlReg.Led;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 Sbc_ComExDeviceMap::PeekPort(int offset)
{
    ii32 * PortBasePtr = static_cast<ii32 *>(DeviceDriver->PortMemoryAddress());
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = static_cast<ii32 *>(DeviceDriver->PortMemoryAddress());
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::PllReferenceSource() - Select source of PLL reference
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::PllReferenceSource(IIRefSource value)
{
    FReferenceSource = value;

    switch (value)
        {
        default:
        case rsExternal:
            // RevB
            ReferenceReg.Source.Value(0);
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outPllRef,
                                 Sbc_ComExTimingInputMux::inFrontPanel);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outPllRef, true);
            break;
        case rsCrystal:
            // RevB
            ReferenceReg.Source.Value(1);
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outPllRef,
                                 Sbc_ComExTimingInputMux::inOnBoardOscillator);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outPllRef, true);
            break;
        case rsGps:
            // RevB
            ReferenceReg.Source.Value(3);
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outPllRef,
                                 Sbc_ComExTimingInputMux::inGps);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outPllRef, true);
            break;
        case rsNone:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outPllRef,
                                 Sbc_ComExTimingInputMux::inFrontPanel);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outPllRef, false);
            break;
        }

    ClockInputMux.Reapply();

    // RevC-: Always enable;  RevD+: Disable oscillator if fallow
    // PciLogicPcb() for RevA = 0
    bool revC = PciLogicPcb() < 3;
    stringstream msg;
    msg << "Sbc_ComExDeviceMap::ReferenceSource::PCB revision: " << PciLogicPcb();
    Debug.Log(msg.str());

    bool enable = revC || (value==rsCrystal) || (value==rsGps);
    ReferenceReg.Enable.Value(enable);

    // Disable FPGA 10 MHz reference output unless Tyco GPS in use
    ReferenceReg.FpgaRefDisable.Value((FGpsInUse != gpsTyco) ? true : false);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::PllReferenceSource() - Query source of PLL reference
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::IIRefSource Sbc_ComExDeviceMap::PllReferenceSource() const
{
    return FReferenceSource;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Xmc0ReferenceSource() - Select source of Xmc0 reference
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::Xmc0ReferenceSource(IIRefSource value)
{
    FXmc0ReferenceSource = value;

    switch (value)
        {
        default:
        case rsExternal:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc0Ref,
                                 Sbc_ComExTimingInputMux::inFrontPanel);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc0Ref, true);
            break;
        case rsCrystal:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc0Ref,
                                 Sbc_ComExTimingInputMux::inOnBoardOscillator);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc0Ref, true);
            break;
        case rsGps:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc0Ref,
                                 Sbc_ComExTimingInputMux::inGps);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc0Ref, true);
            break;
        case rsNone:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc0Ref,
                                 Sbc_ComExTimingInputMux::inFrontPanel);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc0Ref, false);
            break;
        }

    ClockInputMux.Reapply();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Xmc0ReferenceSource() - Query source of PLL reference
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::IIRefSource Sbc_ComExDeviceMap::Xmc0ReferenceSource() const
{
    return FXmc0ReferenceSource;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Xmc1ReferenceSource() - Select source of Xmc0 reference
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::Xmc1ReferenceSource(IIRefSource value)
{
    FXmc1ReferenceSource = value;

    switch (value)
        {
        default:
        case rsExternal:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc1Ref,
                                 Sbc_ComExTimingInputMux::inFrontPanel);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc1Ref, true);
            break;
        case rsCrystal:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc1Ref,
                                 Sbc_ComExTimingInputMux::inOnBoardOscillator);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc1Ref, true);
            break;
        case rsGps:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc1Ref,
                                 Sbc_ComExTimingInputMux::inGps);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc1Ref, true);
            break;
        case rsNone:
            //Rev D+
            ClockInputMux.Source(Sbc_ComExTimingInputMux::outXmc1Ref,
                                 Sbc_ComExTimingInputMux::inFrontPanel);
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outXmc1Ref, false);
            break;
        }

    ClockInputMux.Reapply();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Xmc0ReferenceSource() - Query source of PLL reference
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::IIRefSource Sbc_ComExDeviceMap::Xmc1ReferenceSource() const
{
    return FXmc1ReferenceSource;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ClockSource() - Select source of PLL clock
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ClockSource(IIClockSource value)
{
    FClockSource = value;

    ClockInputMux.Source(Sbc_ComExTimingInputMux::outPllClk,
                         Sbc_ComExTimingInputMux::inFrontPanel);

    switch (value)
        {
        default:
        case csExternal:
            PllReferenceSource(rsExternal);
            Clock->OutputClock(Ad9511::oExternal);
            // Rev D+
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outPllClk, true);
            break;
        case csPll:
            // RevB
            Clock->OutputClock(Ad9511::oVco);
            // Rev D+
            ClockInputMux.OutputEnabled(Sbc_ComExTimingInputMux::outPllClk, false);
            break;
        }

    ClockInputMux.Reapply();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ClockSource() - Query source of PLL clock
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::IIClockSource Sbc_ComExDeviceMap::ClockSource() const
{
    return FClockSource;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ClockDivider() - Set divider factor for specified output
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ClockDivider(int device, float factor)
{
    FClockDivider[device] = factor;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ClockDivider() - Query divider factor for specified output
//---------------------------------------------------------------------------

float Sbc_ComExDeviceMap::ClockDivider(int device) const
{
    return FClockDivider[device];
}

//---------------------------------------------------------------------------
//
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ClockInputMuxReady(bool state)
{
    ReferenceReg.Sync.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
// Sbc_ComExDeviceMap::ClockMuxRegisters()
//---------------------------------------------------------------------------

std::vector<int> & Sbc_ComExDeviceMap::ClockMuxRegisters()
{
    return ClockInputMux.MuxRegisters();
}

//---------------------------------------------------------------------------
// Sbc_ComExDeviceMap::ClockInMux()
//---------------------------------------------------------------------------

Sbc_ComExTimingInputMux & Sbc_ComExDeviceMap::ClockInMux()
{
    return ClockInputMux;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::SoftwareTrigger() - Set SW trigger register
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::SoftwareTrigger(bool value)
{
    Unframed();
    SoftwareTriggerReg.Value(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::SoftwareTrigger() - Query SW trigger register
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::SoftwareTrigger() const
{
    return SoftwareTriggerReg.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Framed() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::Framed(int size)
{
    TriggerReg.Count.Value(size);
    TriggerReg.FramedMode.Value(true);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Unframed() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::Unframed()
{
    TriggerReg.FramedMode.Value(false);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ExternalTrigger(bool state)
{
    TriggerReg.External.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::ExternalTrigger() const
{
    return TriggerReg.External.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ArmTrigger() --  Enable/disable FPGA generated triggers
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ArmTrigger(bool state)
{
    TriggerReg.Arm.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ArmTrigger() --  Query FPGA generated triggers
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::ArmTrigger() const
{
    return TriggerReg.Arm.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::ArmGpsTrigger() --  Fire trigger at next epoch
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::ArmGpsTrigger()
{
    Framed(0x200);   // use framed mode to make sure trigger is wide enough
    GpsTriggerReg.Arm.Value(0);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::GpsEnabled() --  Enable/disable GPS subsystem
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::GpsEnabled(bool state)
{
    InterruptEnable(false);

    // Reset receive FIFO
    GpsControlReg.FifoReset.Value(1);
    GpsControlReg.FifoReset.Value(state ? 0 : 1);
    Sleep(10);
    GpsControlReg.Enabled.Value(state ? 1 : 0);

    if (state)
    {
        //Gps->Clear();
        InterruptEnable(true);
        Gps->Init();
    }
    else
        Gps->HdwEnabled(false);

}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::GpsEnabled() --  Query state of GPS subsystem
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::GpsEnabled() const
{
    return GpsControlReg.Enabled.Value() ? true : false;
}

//---------------------------------------------------------------------------
// Sbc_ComExDeviceMap::UseTycoGps()
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::UseTycoGps()
{
    if(FGpsInUse == gpsTyco)
        return;

    GpsEnabled(false);
    delete Gps;

    Gps = new Sbc_GpsTyco(this);
    FGpsInUse = gpsTyco;
}

//---------------------------------------------------------------------------
// Sbc_ComExDeviceMap::UseTrimbleGps()
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::UseTrimbleGps()
{
    if(FGpsInUse == gpsTrimble)
        return;

    GpsEnabled(false);
    delete Gps;

    Gps = new Sbc_GpsMiniT(this);
    FGpsInUse = gpsTrimble;
}

//---------------------------------------------------------------------------
// Sbc_ComExDeviceMap::UseNavSyncGps()
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::UseNavSyncGps()
{
    if(FGpsInUse == gpsNavSync)
        return;

    GpsEnabled(false);
    delete Gps;

    Gps = new Sbc_GpsNavSync(this);
    FGpsInUse = gpsNavSync;
}

//---------------------------------------------------------------------------
// Sbc_ComExDeviceMap::UseJacksonLabsGps()
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::UseJacksonLabsGps()
{
    if(FGpsInUse == gpsCSAC)
        return;

    GpsEnabled(false);
    delete Gps;

    Gps = new Sbc_GpsJacksonLabs(this);
    FGpsInUse = gpsCSAC;
}


//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::IsGpsLocked() --  Query if GPS locked
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::IsGpsLocked()
{
    if (!Gps)
        return false;
    else
        return Gps->IsLocked();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::IsGpsAntenna() --  Query if antenna detected
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::IsGpsAntenna()
{
    if (!Gps)
        return false;
    else
        return Gps->IsAntenna();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::InterruptEvent() --
//---------------------------------------------------------------------------

Event * Sbc_ComExDeviceMap::InterruptEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::InterruptDepth() --
//---------------------------------------------------------------------------

size_t Sbc_ComExDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::InterruptAck() --
//---------------------------------------------------------------------------

unsigned int Sbc_ComExDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    Gps->DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::InterruptEnable(bool state)
{
    InterruptEnableReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Baud() -- UART Baud.
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::Baud(IIBaud rate)
{
    UartControlReg.Value(rate);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::Baud() --
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::IIBaud Sbc_ComExDeviceMap::Baud() const
{
    return static_cast<IIBaud>(UartControlReg.Value());
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::UartData() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::UartData(char c)
{
    while (!UartStatusReg.TxReady.Value())
        Innovative::Sleep(1);

    UartDataReg.Value(c);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::UartData() --
//---------------------------------------------------------------------------

char Sbc_ComExDeviceMap::UartData() const
{
    char c = static_cast<char>(UartDataReg.Value());
    return c;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::PutString() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::PutString(const string &data)
{
        for (size_t i=0; i<data.size(); i++)
            UartData(data[i]);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::PutString() --
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::GetString(string &data)
{
    data.clear();
    for(size_t i=0;i<UartFifoDepth();i++)
        data = data + UartData();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::UartFifoDepth() --
//---------------------------------------------------------------------------

unsigned int Sbc_ComExDeviceMap::UartFifoDepth() const
{
    return UartStatusReg.RxDepth.Value();
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::TunerEnabled() -- enable/disable tuner
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::TunerEnabled(bool state)
{
    if (!Tuner)
        return;

    // config and enable Tuner serial port here

    Tuner->Enabled(state);
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::TunerEnabled() -- current state of tuner
//---------------------------------------------------------------------------

bool Sbc_ComExDeviceMap::TunerEnabled() const
{
    if(Tuner)
        return Tuner->Enabled();
    else
        return false;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::TunerReset() -- reset tuner
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::TunerReset()
{
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::UseMMSTuner() -- create MMS tuner object
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::UseMMSTuner(bool /*useSysSerial*/)
{
    if(Tuner)
    {
        if (TunerEnabled())
            TunerEnabled(false);

        delete Tuner;
    }

	Tuner = new Tuner_MMS;
}

//---------------------------------------------------------------------------
//  Sbc_ComExDeviceMap::TunerSetFreq() -- Sets the RF center frequency in MHz.
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::TunerSetFreq(double /*f*/)
{

}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerSetAttn() --  Sets the attenuation in dB
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::TunerSetAttn(double /*a*/)
{

}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerBIT() --  run tuner BIT
//---------------------------------------------------------------------------

void Sbc_ComExDeviceMap::TunerBIT()
{

}




} // namespace Innovative


