// Sbc_ComExDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef Sbc_ComExDeviceMap_MbH
#define Sbc_ComExDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <Magic_Mb.h>
#include <DioPort_Mb.h>
#include <ICS843002_Mb.h>
#include <Ad9511_Mb.h>
#include <SimpleInterrupt_Mb.h>
#include <Gps_Mb.h>
#include <ctime>
#include <PmcEeprom_Mb.h>
#include <Si570_Mb.h>
#include <I2C_Mb.h>
#include <CDCE18005_Mb.h>
#include <Trace_Mb.h>
#include <Tuner_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class Sbc_ComEx;
class SimpleInterrupt;
class Sbc_ComExDeviceMap;

//===========================================================================
//  CLASS Sbc_ComExTimingInputMux  -- Register & low level support for
//                                    ClockInputMux
//===========================================================================

class Sbc_ComExTimingInputMux :  public Cdce18005
{
public:
	enum InMuxSelect { inOnBoardOscillator=muxPrimaryIn, inFrontPanel=muxSecondaryIn, inGps=muxAuxIn };
	enum OutMuxSelect { outXmc0Ref, outXmc1Ref, outFpga, outPllRef, outPllClk };

    Sbc_ComExTimingInputMux(Sbc_ComExDeviceMap * owner, AddressingSpace & space);

    void  Source(unsigned int output, unsigned int source)
        {  SetMuxSelection(output, source);   }
    void  Divider(unsigned int output, unsigned int divide_by)
        {  OutputDivider(output, divide_by);  }
    void  DividerEnabled(unsigned int output, bool enabled)
        {  SetDividerEnable(output, enabled);  }
    void  Phase(unsigned int output, unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(output, divide_by, phase);  }
    void  Boost(unsigned int output, bool enabled)
        {  SetHighSwingLvpecl(output, enabled);  }

    std::vector<int> & MuxRegisters()
        {
        ReadDataRegisters();
        return RegisterImage;
        }
protected:
    virtual void    	 OnInit();
    virtual void    	 OnReapply();
    virtual void    	 OnSetPower(bool & new_state);
    virtual void    	 UpdateSources();

    Sbc_ComExDeviceMap * Owner;
    Trace				 Debug;
};

//===========================================================================
//  CLASS Sbc_ComExDeviceMap  -- Register & low level support for Sbc_ComEx
//===========================================================================

class Sbc_ComExDeviceMap :
    public IUsesVirtexJtagLoader,
    public IUsesExtendedDioPort,
	public IUsesPciLogicVersionInfo,
    public ISimpleInterrupt,
    public IUsesPmcEeprom
{
	friend class Sbc_ComEx;
    friend class Sbc_ComExClock;
	friend class Sbc_GpsTyco;
	friend class Sbc_GpsMiniT;
	friend class Sbc_GpsNavSync;
    friend class Sbc_GpsJacksonLabs;
	friend class Sbc_ComExTimingInputMux;

public:
    enum IIClockSource { csExternal, csPll };
    enum eGpsType {gpsNone = 0, gpsTyco, gpsTrimble, gpsNavSync, gpsCSAC };
    enum TunerTypeEnum {tunerNone, tunerMMS };
	enum IIRefSource { rsExternal, rsCrystal, rsGps, rsNone };
    enum IIBaud { b115200, b57600, b38400, b19200, b9600, b4800 };
    enum ComexType{ tPentium = 0, tAtom = 1, tEnhancedPentium=8, /* SI570C */
                    tEnhancedPentiumA=9 /* SI570A */ };
    //
    //  Member Functions
    Sbc_ComExDeviceMap(Sbc_ComEx * owner);
    virtual ~Sbc_ComExDeviceMap();

    // Methods
    virtual size_t  InterruptDepth();
    size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);

	void            Open();
    void            Close();
    void            Reset();
    void            Start();
    void            ReapplyClock();

    ii32            PeekPort(int offset);
    void            PokePort(int offset, ii32 value);

	// Tuner Support
	ITuner			* GetTunerPtr() const
	{return Tuner;}

    //void			HandleTunerOutput(RxDataEvent &event);
	void			TunerEnabled(bool state);
	bool			TunerEnabled() const;
	void			UseMMSTuner(bool useSysSerial);
	void			TunerSetFreq(double f);
	void			TunerSetAttn(double a);
	void			TunerReset();
	void			TunerBIT();

    // Gps Support
    bool            IsGpsLocked();
    bool            IsGpsAntenna();

    void            GpsEnabled(bool state);
    bool            GpsEnabled() const;
    void 			UseTycoGps();
    void            UseTrimbleGps();
	void            UseNavSyncGps();
    void            UseJacksonLabsGps();

    void            ArmGpsTrigger();

    // GPS Uart
    void            Baud(IIBaud rate);
    IIBaud          Baud() const;

    void            UartData(char c);
    char            UartData() const;
    virtual void           PutString(const std::string & data);
    virtual void           GetString(std::string & data);


    unsigned int    UartFifoDepth() const;

    // Properties
    void            Target(int target);
	int             Target() const;

    void            Led(bool value);
    bool            Led() const;

	void            PllReferenceSource(IIRefSource value);
	IIRefSource     PllReferenceSource() const;

	void 			Xmc0ReferenceSource(IIRefSource value);
	IIRefSource 	Xmc0ReferenceSource() const;

	void 			Xmc1ReferenceSource(IIRefSource value);
	IIRefSource 	Xmc1ReferenceSource() const;

	void            ClockSource(IIClockSource value);
	IIClockSource   ClockSource() const;

	void            ClockDivider(int device, float value);
    float           ClockDivider(int device) const;

    void            SoftwareTrigger(bool value);
    bool            SoftwareTrigger() const;

    void            Framed(int size);
    void            Unframed();

    void            ExternalTrigger(bool state);
    bool            ExternalTrigger() const;

    void            ArmTrigger(bool state);
    bool            ArmTrigger() const;

    Si570 *         VCXO(){return Vcxo;};

    void			ClockInputMuxReady(bool state);
    Sbc_ComExTimingInputMux & ClockInMux();
    std::vector<int> & ClockMuxRegisters();


protected:
    Sbc_ComEx *         Owner;
    IGps *              Gps;
    ITuner              *Tuner;
    //SerialPort          *serialPort;

    // Fields
    IIRefSource         FReferenceSource;
    IIClockSource       FClockSource;
    std::vector<float>  FClockDivider;

    //
    //  Class TriggerControlRegister
    class TriggerControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup    Count;
		RegisterBit         Arm;
        RegisterBit         FramedMode;
        RegisterBit         External;

    public:
		TriggerControlRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Count(*this, 0, 24), Arm(*this, 28),
			  FramedMode(*this, 30), External(*this, 31)
            {
            //  Default setup ????
            }
    };

    //
    //  Class GpsStatusRegister
    class GpsStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Locked;
        RegisterBit       Antenna;

    public:
        GpsStatusRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Locked(*this, 1), Antenna(*this, 2)
            {
            //  Default setup ????
            }
    };
    //
    //  Class GpsControlRegister
    class GpsControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       Enabled;
        RegisterBit       FifoReset;

    public:
        GpsControlRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Enabled(*this, 0), FifoReset(*this, 3)
			{
            //  Default setup ????
            }
    };
    //
	//  Class GpsTriggerRegister
	class GpsTriggerRegister : public ShadowRegister
    {
		typedef ShadowRegister inherited;

    public:
        RegisterBit       Arm;

    public:
        GpsTriggerRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Arm(*this, 0)
            {
            //  Default setup ????
            }
    };

    //
    //  Class PllReferenceControlRegister
    class PllReferenceControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup    Source;
        RegisterBit         Enable;
        RegisterBit         Sync;
        RegisterBit         FpgaRefDisable;

    public:
        PllReferenceControlRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Source(*this, 0, 2), Enable(*this, 2), Sync(*this, 3),
              FpgaRefDisable(*this, 4)
            {
            //  Default setup ????
            }
    };

    //
	//  Class PciControlRegister
    class PciControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
		RegisterBit       Led;

	public:
        PciControlRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
               Led(*this, 1)
            {
            //  Default setup ????
            }
    };

    //
    //  Class PciStatusRegister
    class PciStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit       Slot64;

    public:
        PciStatusRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
               Slot64(*this, 0)
            {
            //  Default setup ????
            }
    };

    //
    //  Class UartStatusRegister
    class UartStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit       TxReady;
        ReadOnlyRegisterBit       RxReady;
        ReadOnlyRegisterBitGroup  RxDepth;

    public:
        UartStatusRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
               TxReady(*this, 3), RxReady(*this, 4),
               RxDepth(*this, 5, 10)
			{
            //  Default setup ????
			}
    };

    //
    //  Class VcxoI2CRegister
    class VcxoI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Enable;
        RegisterBit  Data_In;
        RegisterBit  Data_Out;
        RegisterBit  Clock_In;
        RegisterBit  Clock_Out;

    public:
        VcxoI2CRegister( AddressingSpace &space, int offset )
            :  inherited(space, offset),
               Enable(*this, 7),
               Data_In(*this, 2),
               Data_Out(*this, 0),
               Clock_In(*this, 3),
               Clock_Out(*this, 1)
            {

            }
    };

    //
    // Class VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
    class VcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        VcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)	{ }
        virtual ~VcxoI2C_Impl() {}
		// Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
			{  return Reg.Data_In.Value();  }
    private:
        VcxoI2CRegister &  Reg;
    };


    //
    //  ...Registers
    Ad9511			*				    Clock;
    VcxoI2CRegister     				VcxoI2CReg;
	VcxoI2C_Impl                        VcxoIntf;
    I2CDeviceN          				Vcxo_I2C;
    Si570				*				Vcxo;


    PmcIdromSection                     ClockParamRomSection;
    AddressingSpace                    	PortMemory;
    InnovativeKernel::IsrDeviceDriver * DeviceDriver;
	PciControlRegister                 	PciControlReg;
	PciStatusRegister                  	PciStatusReg;
	PllReferenceControlRegister         ReferenceReg;
	ShadowRegister                      SoftwareTriggerReg;
	TriggerControlRegister              TriggerReg;

	Register                            UartDataReg;
	ShadowRegister                      UartControlReg;
	UartStatusRegister                  UartStatusReg;

	GpsStatusRegister                   GpsStatusReg;
	GpsControlRegister                  GpsControlReg;
	GpsTriggerRegister                  GpsTriggerReg;
	Register                            InterruptStatusReg;
	ShadowRegister                      InterruptEnableReg;

	Sbc_ComExTimingInputMux			    ClockInputMux;
	eGpsType					        FGpsInUse;
	IIRefSource							FXmc0ReferenceSource;
	IIRefSource							FXmc1ReferenceSource;

private:
    // Interface
    virtual Event *         InterruptEvent();
    virtual unsigned int    InterruptAck();
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);
    bool                    EnhancedClock();

	// Data
	SimpleInterrupt         Isr;
	Trace					Debug;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
