// Sbc_ComEx_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Sbc_ComEx_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <SystemSupport_Mb.h>
#include <TycoGps_Mb.h>
using namespace std;


namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS Sbc_ComEx  -- Digital Transmitter PMC module object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Sbc_ComEx
//---------------------------------------------------------------------------

Sbc_ComEx::Sbc_ComEx()
    : Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class Sbc_ComEx
//---------------------------------------------------------------------------

Sbc_ComEx::~Sbc_ComEx()
{
    Close();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t Sbc_ComEx::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> Sbc_ComEx::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string Sbc_ComEx::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  Sbc_ComEx::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  Sbc_ComEx::Open()
{
    Device.Open();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  Sbc_ComEx::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Reset() -- Cold-start initialization
//---------------------------------------------------------------------------

void Sbc_ComEx::Reset()
{
    return Device.Reset();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Sbc_ComEx::Target() const
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Sbc_ComEx::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  Sbc_ComEx::Led() const
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  Sbc_ComEx::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  Sbc_ComEx::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  Sbc_ComEx::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & Sbc_ComEx::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Eeprom() -- Return reference to EEProm object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & Sbc_ComEx::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Clock() --  Return reference to AD9511 PLL
//---------------------------------------------------------------------------

Ad9511 & Sbc_ComEx::Clock()
{
    return *Device.Clock;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ReferenceSource() - Steer reference to PLL
//---------------------------------------------------------------------------

void Sbc_ComEx::ReferenceSource(IIRefSource value)
{
    PllReferenceSource(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PllReferenceSource() - Query reference to PLL
//---------------------------------------------------------------------------

Sbc_ComEx::IIRefSource Sbc_ComEx::ReferenceSource() const
{
    return PllReferenceSource();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PllReferenceSource() - Steer reference to PLL
//---------------------------------------------------------------------------

void Sbc_ComEx::PllReferenceSource(IIRefSource value)
{
    switch (value)
        {
        default:
        case rsExternal:
            Device.PllReferenceSource(Sbc_ComExDeviceMap::rsExternal);
            break;
        case rsCrystal:
            Device.PllReferenceSource(Sbc_ComExDeviceMap::rsCrystal);
            break;
        case rsGps:
            Device.PllReferenceSource(Sbc_ComExDeviceMap::rsGps);
            break;
        case rsNone:
            Device.PllReferenceSource(Sbc_ComExDeviceMap::rsNone);
            break;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::PllReferenceSource() - Query reference to PLL
//---------------------------------------------------------------------------

Sbc_ComEx::IIRefSource Sbc_ComEx::PllReferenceSource() const
{
    switch (Device.PllReferenceSource())
        {
        default:
        case Sbc_ComExDeviceMap::rsExternal:
            return rsExternal;
        case Sbc_ComExDeviceMap::rsCrystal:
            return rsCrystal;
        case Sbc_ComExDeviceMap::rsGps:
            return rsGps;
        case Sbc_ComExDeviceMap::rsNone:
            return rsNone;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Xmc0ReferenceSource() - Set source of XMC site 0 reference pin
//---------------------------------------------------------------------------

void Sbc_ComEx::Xmc0ReferenceSource(IIRefSource value)
{
    switch (value)
        {
        default:
        case rsExternal:
            Device.Xmc0ReferenceSource(Sbc_ComExDeviceMap::rsExternal);
            break;
        case rsCrystal:
            Device.Xmc0ReferenceSource(Sbc_ComExDeviceMap::rsCrystal);
            break;
        case rsGps:
            Device.Xmc0ReferenceSource(Sbc_ComExDeviceMap::rsGps);
            break;
        case rsNone:
            Device.Xmc0ReferenceSource(Sbc_ComExDeviceMap::rsNone);
            break;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Xmc0ReferenceSource() - Query source of XMC site 0 reference pin
//---------------------------------------------------------------------------

Sbc_ComEx::IIRefSource Sbc_ComEx::Xmc0ReferenceSource() const
{
    switch (Device.Xmc0ReferenceSource())
        {
        default:
        case Sbc_ComExDeviceMap::rsExternal:
            return rsExternal;
        case Sbc_ComExDeviceMap::rsCrystal:
            return rsCrystal;
        case Sbc_ComExDeviceMap::rsGps:
            return rsGps;
        case Sbc_ComExDeviceMap::rsNone:
            return rsNone;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Xmc1ReferenceSource() - Set source of XMC site 1 reference pin
//---------------------------------------------------------------------------

void Sbc_ComEx::Xmc1ReferenceSource(IIRefSource value)
{
    switch (value)
        {
        default:
        case rsExternal:
            Device.Xmc1ReferenceSource(Sbc_ComExDeviceMap::rsExternal);
            break;
        case rsCrystal:
            Device.Xmc1ReferenceSource(Sbc_ComExDeviceMap::rsCrystal);
            break;
        case rsGps:
            Device.Xmc1ReferenceSource(Sbc_ComExDeviceMap::rsGps);
            break;
        case rsNone:
            Device.Xmc1ReferenceSource(Sbc_ComExDeviceMap::rsNone);
            break;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Xmc1ReferenceSource() - Query source of XMC site 1 reference pin
//---------------------------------------------------------------------------

Sbc_ComEx::IIRefSource Sbc_ComEx::Xmc1ReferenceSource() const
{
    switch (Device.Xmc1ReferenceSource())
        {
        default:
        case Sbc_ComExDeviceMap::rsExternal:
            return rsExternal;
        case Sbc_ComExDeviceMap::rsCrystal:
            return rsCrystal;
        case Sbc_ComExDeviceMap::rsGps:
            return rsGps;
        case Sbc_ComExDeviceMap::rsNone:
            return rsNone;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ClockSource() - Select source of PLL clock
//---------------------------------------------------------------------------

void Sbc_ComEx::ClockSource(IIClockSource value)
{
    switch (value)
        {
        default:
        case csExternal:
            Device.ClockSource(Sbc_ComExDeviceMap::csExternal);
            break;
        case csPll:
            Device.ClockSource(Sbc_ComExDeviceMap::csPll);
            break;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ClockSource() - Query PLL clock source
//---------------------------------------------------------------------------

Sbc_ComEx::IIClockSource Sbc_ComEx::ClockSource() const
{
    switch (Device.ClockSource())
        {
        default:
        case Sbc_ComExDeviceMap::csExternal:
            return csExternal;
        case Sbc_ComExDeviceMap::csPll:
            return csPll;
        }
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ClockDivider() - Set divider factor for specified output
//---------------------------------------------------------------------------

void Sbc_ComEx::ClockDivider(IIClockOutput device, float factor)
{
    Device.ClockDivider(static_cast<int>(device), factor);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ClockDivider() - Query divider factor for specified output
//---------------------------------------------------------------------------

float Sbc_ComEx::ClockDivider(IIClockOutput device) const
{
    return Device.ClockDivider(static_cast<int>(device));
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::SoftwareTrigger() - Set SW trigger register
//---------------------------------------------------------------------------

void Sbc_ComEx::SoftwareTrigger(bool value)
{
    Device.SoftwareTrigger(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::SoftwareTrigger() - Query SW trigger register
//---------------------------------------------------------------------------

bool Sbc_ComEx::SoftwareTrigger() const
{
    return Device.SoftwareTrigger();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Framed() - Set frame trigger state/size
//---------------------------------------------------------------------------

void Sbc_ComEx::Framed(int size)
{
    Device.Framed(size);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Unframed() - Unframed triggers
//---------------------------------------------------------------------------

void Sbc_ComEx::Unframed()
{
    Device.Unframed();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ExternalTrigger() - Set external trigger state
//---------------------------------------------------------------------------

void Sbc_ComEx::ExternalTrigger(bool value)
{
    Device.ExternalTrigger(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ExternalTrigger() - Query external trigger state
//---------------------------------------------------------------------------

bool Sbc_ComEx::ExternalTrigger() const
{
    return Device.ExternalTrigger();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ArmTrigger() - Set trigger arm state
//---------------------------------------------------------------------------

void Sbc_ComEx::ArmTrigger(bool value)
{
    Device.ArmTrigger(value);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ArmTrigger() - Query trigger arm state
//---------------------------------------------------------------------------

bool Sbc_ComEx::ArmTrigger() const
{
    return Device.ArmTrigger();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ArmGpsTrigger() --  Fire trigger at next epoch
//---------------------------------------------------------------------------

void Sbc_ComEx::ArmGpsTrigger()
{
    Device.ArmGpsTrigger();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & Sbc_ComEx::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Dio() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & Sbc_ComEx::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Gps() -- GPS state
//---------------------------------------------------------------------------

IGps * Sbc_ComEx::Gps()
{
    return Device.Gps;
}


//---------------------------------------------------------------------------
// Sbc_ComEx::UseTycoGps()
//---------------------------------------------------------------------------

void Sbc_ComEx::UseTycoGps()
{
    Device.UseTycoGps();
}

//---------------------------------------------------------------------------
// Sbc_ComEx::UseTrimbleGps()
//---------------------------------------------------------------------------

void Sbc_ComEx::UseTrimbleGps()
{
    Device.UseTrimbleGps();
}

//---------------------------------------------------------------------------
// Sbc_ComEx::UseNavSyncGps()
//---------------------------------------------------------------------------

void Sbc_ComEx::UseNavSyncGps()
{
    Device.UseNavSyncGps();
}

//---------------------------------------------------------------------------
// Sbc_ComEx::UseCSACGps()
//---------------------------------------------------------------------------

void Sbc_ComEx::UseJacksonLabsGps()
{
    Device.UseJacksonLabsGps();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::GpsEnabled() --  Enable/disable GPS subsystem
//---------------------------------------------------------------------------

void Sbc_ComEx::GpsEnabled(bool state)
{
    Queue.erase();
    Device.GpsEnabled(state);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::GpsEnabled() --  query enable state of GPS subsystem
//---------------------------------------------------------------------------

bool Sbc_ComEx::GpsEnabled() const
{
    return Device.GpsEnabled();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Baud() --  Set baud rate of UART
//---------------------------------------------------------------------------

void Sbc_ComEx::Baud(Sbc_ComExDeviceMap::IIBaud rate)
{
    Device.Baud(rate);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::Baud() --  Fetch baud rate of UART
//---------------------------------------------------------------------------

Sbc_ComExDeviceMap::IIBaud Sbc_ComEx::Baud() const
{
    return Device.Baud();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::UartData() --  Write character to UART
//---------------------------------------------------------------------------

void Sbc_ComEx::UartData(char c)
{
    Device.UartData(c);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::UartData() --  Read character from UART
//---------------------------------------------------------------------------

char Sbc_ComEx::UartData() const
{
    return Device.UartData();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::UartFifoDepth() --
//---------------------------------------------------------------------------

unsigned int Sbc_ComEx::UartFifoDepth() const
{
    return Device.UartFifoDepth();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::GpsTriggerTime() --  Trigger on next epoch rising edge
//---------------------------------------------------------------------------

void Sbc_ComEx::GpsTriggerTime(const time_t & then)
{
    if (Gps())
        Gps()->GpsTriggerTime(then);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::IsGpsLocked() --  Query if GPS locked
//---------------------------------------------------------------------------

bool Sbc_ComEx::IsGpsLocked()
{
    return Device.IsGpsLocked();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::IsGpsAntenna() --  Query if antenna detected
//---------------------------------------------------------------------------

bool Sbc_ComEx::IsGpsAntenna()
{
    return Device.IsGpsAntenna();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::HasClockInputMux() --  Query if there is a clock input mux.
//---------------------------------------------------------------------------

bool Sbc_ComEx::HasClockInputMux()
{
    return false;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ClockInputMux() --  Return reference to Clock Input Mux object.
//---------------------------------------------------------------------------

Sbc_ComExTimingInputMux & Sbc_ComEx::ClockInputMux()
{
    return Device.ClockInputMux;
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::ClockDivider() -
//---------------------------------------------------------------------------

void  Sbc_ComEx::ClockInputMuxReady(bool state)
{
    Device.ClockInputMuxReady(state);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerEnabled() --  Enable/disable Tuner subsystem
//---------------------------------------------------------------------------

void Sbc_ComEx::TunerEnabled(bool state)
{
    Device.TunerEnabled(state);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerEnabled() --  query enable state of Tuner subsystem
//---------------------------------------------------------------------------

bool Sbc_ComEx::TunerEnabled() const
{
    return Device.TunerEnabled();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerSetFreq() --  Sets the RF center frequency (MHz) of Tuner subsystem
//---------------------------------------------------------------------------

void Sbc_ComEx::TunerSetFreq(double f)
{
    return Device.TunerSetFreq(f);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerSetAttn() --  Sets the attenuation in dB
//---------------------------------------------------------------------------

void Sbc_ComEx::TunerSetAttn(double a)
{
    return Device.TunerSetAttn(a);
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerReset() --  Resets tuner
//---------------------------------------------------------------------------

void Sbc_ComEx::TunerReset()
{
    return Device.TunerReset();
}

//---------------------------------------------------------------------------
//  Sbc_ComEx::TunerBIT() --  Run tuner BIT
//---------------------------------------------------------------------------

void Sbc_ComEx::TunerBIT()
{
    return Device.TunerBIT();
}

//---------------------------------------------------------------------------
// Sbc_ComEx::UseMMSTuner()
//---------------------------------------------------------------------------

void Sbc_ComEx::UseMMSTuner(bool useSysSerial)
{
    Device.UseMMSTuner(useSysSerial);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

}  // namespace Innovative
