// Sbc_ComEx_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Sbc_ComEx_MbH
#define Sbc_ComEx_MbH

#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <Sbc_ComExDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Sbc_ComExDeviceMap;

//===========================================================================
//  CLASS Sbc_ComEx  -- Digital Transmitter PMC module object
//===========================================================================

class Sbc_ComEx
{
    friend class Sbc_ComExDeviceMap;

public:
    enum IIClockOutput { coFpga, coFront0, coFront1, coXmc0, coXmc1 };
    enum IIClockSource { csExternal, csPll };
    enum IIRefSource { rsExternal, rsCrystal, rsGps, rsNone };


    //
    //  Member Functions
    Sbc_ComEx();
    ~Sbc_ComEx();

    size_t          BoardCount();
    vector<string>  BoardNames();
    string          PrintAllDevices();
	bool            LicenseCheck(unsigned int & code);

	void            Open();
    void            Close();
    void            Reset();
    void            Start();

    void			ClockInputMuxReady(bool state);

    ii32            PeekPort(int offset);
    void            PokePort(int offset, ii32 value);

    // Tuner Support
    void            TunerEnabled(bool state);
    bool            TunerEnabled() const;
    void			UseMMSTuner(bool useSysSerial);

    void			TunerSetFreq(double f);
    void			TunerSetAttn(double a);
    void			TunerReset();
    void			TunerBIT();

    // Gps Support
    void 			UseTycoGps();
    void            UseTrimbleGps();
    void            UseNavSyncGps();
    void            UseJacksonLabsGps();
    void            GpsTriggerTime(const std::time_t & then);
    bool            IsGpsLocked();
    bool            IsGpsAntenna();
    void            ArmGpsTrigger();

    void            GpsEnabled(bool state);
    bool            GpsEnabled() const;

    // GPS Uart
    void            Baud(Sbc_ComExDeviceMap::IIBaud rate);
    Sbc_ComExDeviceMap::IIBaud Baud() const;

    void            UartData(char c);
    char            UartData() const;

    unsigned int    UartFifoDepth() const;

    // Properties
    void  			Led(bool value);
    bool  			Led() const;

    void            Target(int target);
    int             Target() const;

    void            ReferenceSource(IIRefSource value);
    IIRefSource     ReferenceSource() const;

    void            PllReferenceSource(IIRefSource value);
    IIRefSource     PllReferenceSource() const;

    void 			Xmc0ReferenceSource(IIRefSource value);
    IIRefSource 	Xmc0ReferenceSource() const;

    void 			Xmc1ReferenceSource(IIRefSource value);
    IIRefSource 	Xmc1ReferenceSource() const;

    void            ClockSource(IIClockSource value);
    IIClockSource   ClockSource() const;

    void            ClockDivider(IIClockOutput device, float value);
    float           ClockDivider(IIClockOutput device) const;

    void            ExternalTrigger(bool value);
    bool            ExternalTrigger() const;

    void            Framed(int size);
    void            Unframed();

    void            ArmTrigger(bool value);
    bool            ArmTrigger() const;

    void            SoftwareTrigger(bool value);
    bool            SoftwareTrigger() const;

    OpenWire::ThunkedEventHandler<OpenWire::NotifyEvent> OnGpsArm;

    // EEprom programming
    IUsesVirtexJtagLoader & Eeprom();

    // Trigger Support

    // Methods
    Ad9511 					&	Clock();
    IUsesPmcEeprom 			&   IdRom();
    IGps 					*   Gps();
    ITuner				    *   Tuner() const
    { return Device.GetTunerPtr(); }

    IUsesExtendedDioPort 	& 	Dio();

    PciLogicVersionInfoIntf &   PciLogicVersion();
    Sbc_ComExTimingInputMux &	ClockInputMux();
    bool		                HasClockInputMux();

//protected:
    Sbc_ComExDeviceMap * Debug()
        {  return &Device;  }

protected:
    //
    //  Member Data
    Sbc_ComExDeviceMap          Device;
    std::string                 Queue;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

