// Script_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

//  This code implements a small script interpreter.  Commands within the
//  specified script are dispatched when Execute is called.  Numeric values
//  (integer only, C formatted) encountered in the script are pushed onto
//  a stack (ala Forth).  When a non-numeric word is subsequently
//  encountered within the script, it is treated as a command and the
//  OnCommand event is called.  The user code is responsible for calling
//  the Pop method to retreive the appropriate number of stack parameters.

#include "Script_Mb.h"

#include <string>
#include <cstdlib>
#include <memory>
#include <iostream>
#include <sstream>
#include <cctype>
#include <errno.h>
#include "StringSupport_Mb.h"
#include <cstring>

using namespace Innovative;

namespace   Innovative
{

//===========================================================================
//  CLASS Scripter  --  Script command dispatcher
//===========================================================================

//---------------------------------------------------------------------------
//  Scripter::Scripter() --  Ctor
//---------------------------------------------------------------------------

Scripter::Scripter()
{
}

//---------------------------------------------------------------------------
//  Scripter::Execute() --  Execute specified script
//---------------------------------------------------------------------------

void Scripter::Execute(std::string file)
{
    if (!File.LoadFromFile(file))
        {
        ProcessStatusEvent e("Can't open script file");
        OnMessage.Execute(e);
        return;
        }

    for (size_t i = 0; i < File.Count(); ++i)
        {
        std::string line = File.StringAt(i);

        try
            {
            ParseLine(line);
            }
        catch (...)
            {
            std::stringstream msg;
            msg << "Script parse error at line " << i;
            ProcessStatusEvent e(msg.str());
            OnMessage.Execute(e);
            return;
            }
        }
}

//---------------------------------------------------------------------------
//  Scripter::ParseLine() --  Parse single line in script file
//---------------------------------------------------------------------------

void Scripter::ParseLine(std::string line)
{
    // Skip blank lines
    if (!line.length())
        return;

    // Skip comments
    if (line.find("--") != std::string::npos)
        {
        ProcessStatusEvent e(line);
        OnMessage.Execute(e);
        return;
        }

    const char * delimiters = " \n\t\r";

    while (!Stack.empty())
        Pop();

#ifdef __BORLANDC__
    #pragma warning(push)
    #pragma warning(disable:4996)
#endif
    std::vector<char> txt(line.c_str(), line.c_str() + line.size() + 1);
    char *p = strtok(&txt[0], delimiters);
    while (p)
        {
        bool ok = true;
        int value = StringToInt(p, ok);
        if (ok)
            {
            Stack.push(value);
            }
        else
            {
            ProcessStatusEvent e(p);
            OnCommand.Execute(e);
            break;
            }

        p = strtok(NULL, delimiters);
        }

#ifdef __BORLANDC__
    #pragma warning(pop)
#endif
}

//---------------------------------------------------------------------------
//  Scripter::Pop() --  Pop stack and return to caller
//---------------------------------------------------------------------------

int Scripter::Pop()
{
    if (Stack.empty())
        {
        ProcessStatusEvent e("Stack underflow");
        OnMessage.Execute(e);
        return 0;
        }

    int result = Stack.top();
    Stack.pop();

    return result;
}

} // namespace Innovative



