#include "Sema.h"
#include "Trace_Mb.h"

#ifdef __linux__
    const std::string LibPath = "..\\..\\..\\..\\..\\Sema\\libsema.so.2.5";
#elif _WIN32
    const std::string LibPath = "..\\..\\..\\..\\..\\Sema\\libsema.dll";
#endif

//-----------------------------------------------------------------------------
//  constructor for class Sema
//-----------------------------------------------------------------------------
namespace Innovative
{

bool Sema::InitCalled = false;

Sema::Sema() : Lib(new Poco::SharedLibrary(LibPath))
{
    Trace Debug;
    int Loaded = 1; //0 if loaded

    if(Lib->isLoaded())
    {
        Debug.Log("Dll was loaded successfully.", Lib->getPath());

        typedef int (*Sema_Init_Type)();
        Sema_Init_Type Sema_Init = (Sema_Init_Type)Lib->getSymbol("Sema_Init");

        if(Sema_Init || !InitCalled)
        {
            Loaded = Sema_Init();
            InitCalled = true;
        }

        // 0 if Sema initialization was successful
        if(Loaded == 0)
            Debug.Log("Sema Board initialization was successful.");
        else
            Debug.Log("Sema Board initialization was not successful.");
    }
    else
    {
        Debug.Log("Dll was not loaded successfully from ", Lib->getPath());
    }

}

//-----------------------------------------------------------------------------
//  Destructor for class Sema
//-----------------------------------------------------------------------------

Sema::~Sema()
{
    typedef int (*Sema_Close_Type)();
    Sema_Close_Type Sema_Close = (Sema_Close_Type)Lib->getSymbol("Sema_Close");

    if(Sema_Close)
        Sema_Close();

    delete Lib;
}

//-----------------------------------------------------------------------------
//  Sema:: ReadBlock(int addr,int *data,int Words) --
//-----------------------------------------------------------------------------

void  Sema::ReadMemoryBlock(byte *data)
{  
    if(BmcFlag())
    {
      typedef void (*Memory_Read_Type)(byte *,unsigned short,unsigned char);
      Memory_Read_Type Sema_MemoryRead = (Memory_Read_Type)Lib->getSymbol("Sema_MemoryRead");

        if(Sema_MemoryRead)
         {
            for(int idx = 0;idx<IdromSizeInBytes;idx+=JunkSize)
              {
                Sema_MemoryRead(data,idx,(byte)JunkSize);
                data+=JunkSize;
              }
         }
    }
}

//-----------------------------------------------------------------------------
//  Sema:: WriteBlock() --
//-----------------------------------------------------------------------------

void  Sema::WriteMemoryBlock(byte *data)
{
    if(BmcFlag())
    {
     typedef void (*Memory_Write_Type)(byte*,unsigned short,unsigned char);
     Memory_Write_Type Sema_MemoryWrite = (Memory_Write_Type)Lib->getSymbol("Sema_MemoryWrite");

     if(Sema_MemoryWrite)
         {
             for(int idx = 0;idx<IdromSizeInBytes;idx+=JunkSize)
               {
                  Sema_MemoryWrite(data,idx,(byte)JunkSize);
                  data+=JunkSize;
               }
          }
    }
}
//-----------------------------------------------------------------------------
//  Sema:: BMCFlag() --
//-----------------------------------------------------------------------------

bool  Sema::BmcFlag()
{
    typedef unsigned char (*Sema_Bmc_Flags)();
    Sema_Bmc_Flags Sema_BmcFlags = (Sema_Bmc_Flags)Lib->getSymbol("Sema_GetBMCFlags");

    if(Sema_BmcFlags)
        return (bool)Sema_BmcFlags();
    else
        return 0;
}



}  // end namespace Innovative
