//---------------------------------------------------------------------------
// SemaIdrom_Mb.cpp  --  Omnibus Module Idrom Access Object
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include "SemaIdrom_Mb.h"

const unsigned int AssemblyWordSize = 30;
namespace Innovative
{
//=============================================================================
//  CLASS SemaIdrom  --  Omnibus Module Idrom Access Object
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class SemaIdrom
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  constructor for class SemaIdrom
//-----------------------------------------------------------------------------

SemaIdrom::SemaIdrom()
{
    Clear();
}

//-----------------------------------------------------------------------------
//  destructor for class SemaIdrom
//-----------------------------------------------------------------------------

SemaIdrom::~SemaIdrom()
{

}

//-----------------------------------------------------------------------------
//  SemaIdrom::Clear() -- Blank out in-memory image
//-----------------------------------------------------------------------------

void  SemaIdrom:: Clear()
{
    for (int i=0; i<IdromPacketSizeInBytes; i++)
        Packet[i] = 0;
}

//---------------------------------------------------------------------------
//  SemaIdrom::SizeInChars() -- size of data in Chars
//---------------------------------------------------------------------------

unsigned int  SemaIdrom::SizeInChars() const
{
    return IdromPacketSizeInBytes;
}

//------------------------------------------------------------------------
//   SemaIdrom::Name()  --  Read ROM name code from IDROM
//------------------------------------------------------------------------

std::string  SemaIdrom::Name() const
{
    std::string name;
    for (int i=0; i<13; i++)
        {
        byte c = Packet[i];
        if (c=='\0')
            break;
        name += c;
        }
    return name;
}

//------------------------------------------------------------------------
//   SemaIdrom::Name()  --  Write ROM name code to Image
//------------------------------------------------------------------------

void  SemaIdrom::Name(const std::string & name)
{
    for (int i=0; i<13; i++)
        {
        Packet[i] = name[i];
        if (name[i]=='\0')
            break;
        }
}
//------------------------------------------------------------------------
//   SemaIdrom::Revision()  --  Read ROM rev code from IDROM
//------------------------------------------------------------------------

std::string  SemaIdrom::Revision() const
{
    std::string name;

    name += Packet[16];
    return name;
}

//------------------------------------------------------------------------
//   SemaIdrom::Revision()  --  Write ROM rev code to Image
//------------------------------------------------------------------------

void  SemaIdrom::Revision(const std::string & revision)
{
    Packet[16] = revision[0];
}

//-----------------------------------------------------------------------------
//  SemaIdrom::WorkAreaAsChar() const -- Read Data field as char
//-----------------------------------------------------------------------------

byte  SemaIdrom::AsChar(int idx) const
{
    //  Range check
    if (idx>IdromPacketSizeInBytes)
        return 0;
    
    return Packet[idx];
}

//-----------------------------------------------------------------------------
//  SemaIdrom::AsChar() -- Write Data field as char
//-----------------------------------------------------------------------------

void  SemaIdrom::AsChar(int idx, byte data)
{
    if (idx>IdromPacketSizeInBytes)
        return;
    
    Packet[idx] = data;
}

//-----------------------------------------------------------------------------
//  SemaIdrom::AsString() const -- Read Data field(s) as String of Char
//-----------------------------------------------------------------------------

std::string  SemaIdrom::AsString(unsigned int idx, unsigned int chars_to_use) const
{
    std::string result;

    if (idx>=SizeInChars())
        return result;

    while (chars_to_use)
        {
        byte next_char = AsChar(idx);
        if (next_char==0)
            break;
        result.push_back(next_char);

        idx++;
        chars_to_use--;
        }
    return result;
}

//-----------------------------------------------------------------------------
//  SemaIdrom::AsString() -- Write Data field(s) as String of Char
//-----------------------------------------------------------------------------

void  SemaIdrom::AsString(unsigned int idx, unsigned int chars_to_use, const std::string & text)
{

    if (idx>=SizeInChars())
        return;

    size_t text_idx = 0;
    while (chars_to_use)
        {
        byte next_char;
        if (text_idx>=text.size())
            next_char = 0;
        else
            next_char = text[text_idx];
        text_idx++;
        AsChar(idx, next_char);

        idx++;
        chars_to_use--;
        }
}

//-----------------------------------------------------------------------------
//  SemaIdrom::LoadFromRom() -- Load in-memory image from ROM hardware
//-----------------------------------------------------------------------------

void  SemaIdrom::LoadFromRom()
{
    Rom.ReadMemoryBlock(Packet);
}

//-----------------------------------------------------------------------------
//  SemaIdrom::StoreToRom() -- Write in-memory image to ROM hardware
//-----------------------------------------------------------------------------

void  SemaIdrom::StoreToRom()
{
    Rom.WriteMemoryBlock(Packet);
}


//=============================================================================
//  CLASS SemaAssemblyInfo  --
//=============================================================================
//-----------------------------------------------------------------------------
//  housekeeping for class SemaAssemblyInfo
//-----------------------------------------------------------------------------
SemaAssemblyInfo::SemaAssemblyInfo() : CaciDartPartNumber_HeaderOffset(0),
MiniK7ChassisPartNumber_HeaderOffset(1),
MiniK7RFExpansionPartNumber_HeaderOffset(2),
LCDTouchScreenPartNumber_HeaderOffset(3),
DisplayPartNumber_HeaderOffset(4),
DisplaySerialNumber_HeaderOffset(5),
HardDrivePartNumber_HeaderOffset(6),
HardDriveSerialNumber_HeaderOffset(7),
Fmc250PartNumber_HeaderOffset(8),
Fmc250SerialNumber_HeaderOffset(9),
AtomPartNumber_HeaderOffset(10),
XilinxPartNumber_HeaderOffset(11),
XilinxSerialNumber_HeaderOffset(12),
MemoryRAMPartNumber_HeaderOffset(13),
MemoryRAMSerialNumber_HeaderOffset(14),
CompactCOMExpressPartNumber_HeaderOffset(15),
CompactCOMExpressSerialNumber_HeaderOffset(16),
PowerSupplyPartNumber_HeaderOffset(17),
PowerSupplySerialNumber_HeaderOffset(18)
{

}

//---------------------------------------------------------------------------
//  destructor for class SemaAssemblyInfo
//---------------------------------------------------------------------------

SemaAssemblyInfo::~SemaAssemblyInfo()
{

}

//---------------------------------------------------------------------------
//  destructor for class SemaAssemblyInfo
//---------------------------------------------------------------------------

void SemaAssemblyInfo::ReadMemoryBlock()
{
    Rom.LoadFromRom();
}

//---------------------------------------------------------------------------
//  destructor for class SemaAssemblyInfo
//---------------------------------------------------------------------------

void SemaAssemblyInfo::WriteMemoryBlock()
{
    Rom.StoreToRom();
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::CaciDartPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::CaciDartPartNumber(std::string Number)
{
    int idx =  (CaciDartPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::CaciDartPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::CaciDartPartNumber()
{
    int idx = (CaciDartPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MiniK7ChassisPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::MiniK7ChassisPartNumber(std::string Number)
{
    int idx =  (MiniK7ChassisPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MiniK7ChassisPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::MiniK7ChassisPartNumber()
{
    int idx = (MiniK7ChassisPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MiniK7RFExpansionPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::MiniK7RFExpansionPartNumber(std::string Number)
{
    int idx =  (MiniK7RFExpansionPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MiniK7RFExpansionPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::MiniK7RFExpansionPartNumber()
{
    int idx = (MiniK7RFExpansionPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::LCDTouchScreenPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::LCDTouchScreenPartNumber(std::string Number)
{
    int idx =  (LCDTouchScreenPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::LCDTouchScreenPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::LCDTouchScreenPartNumber()
{
    int idx = (LCDTouchScreenPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::DisplayPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::DisplayPartNumber(std::string Number)
{
    int idx =  (DisplayPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::DisplayPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::DisplayPartNumber()
{
    int idx = (DisplayPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::DisplaySerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::DisplaySerialNumber(std::string Number)
{
    int idx =  (DisplayPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::DisplaySerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::DisplaySerialNumber()
{
    int idx = (DisplayPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::HardDrivePartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::HardDrivePartNumber(std::string Number)
{
    int idx =  (HardDrivePartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::HardDrivePartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::HardDrivePartNumber()
{
    int idx = (HardDrivePartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::HardDriveSerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::HardDriveSerialNumber(std::string Number)
{
    int idx =  (HardDriveSerialNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::HardDriveSerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::HardDriveSerialNumber()
{
    int idx = (HardDriveSerialNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::Fmc250PartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::Fmc250PartNumber(std::string Number)
{
    int idx =  (Fmc250PartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::Fmc250PartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::Fmc250PartNumber()
{
    int idx = (Fmc250PartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::Fmc250SerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::Fmc250SerialNumber(std::string Number)
{
    int idx =  (Fmc250SerialNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::Fmc250SerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::Fmc250SerialNumber()
{
    int idx = (Fmc250SerialNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::AtomPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::AtomPartNumber(std::string Number)
{
    int idx =  (AtomPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::AtomPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::AtomPartNumber()
{
    int idx = (AtomPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::XilinxPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::XilinxPartNumber(std::string Number)
{
    int idx =  (XilinxPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::XilinxPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::XilinxPartNumber()
{
    int idx = (XilinxPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::XilinxSerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::XilinxSerialNumber(std::string Number)
{
    int idx =  (XilinxSerialNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::XilinxSerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::XilinxSerialNumber()
{
    int idx = (XilinxSerialNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MemoryRAMPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::MemoryRAMPartNumber(std::string Number)
{
    int idx =  (MemoryRAMPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MemoryRAMPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::MemoryRAMPartNumber()
{
    int idx = (MemoryRAMPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MemoryRAMSerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::MemoryRAMSerialNumber(std::string Number)
{
    int idx =  (MemoryRAMSerialNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::MemoryRAMSerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::MemoryRAMSerialNumber()
{
    int idx = (MemoryRAMSerialNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::CompactCOMExpressPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::CompactCOMExpressPartNumber(std::string Number)
{
    int idx =  (CompactCOMExpressPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::CompactCOMExpressPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::CompactCOMExpressPartNumber()
{
    int idx = (CompactCOMExpressPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::CompactCOMExpressSerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::CompactCOMExpressSerialNumber(std::string Number)
{
    int idx =  (CompactCOMExpressSerialNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::CompactCOMExpressSerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::CompactCOMExpressSerialNumber()
{
    int idx = (CompactCOMExpressSerialNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::PowerSupplyPartNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::PowerSupplyPartNumber(std::string Number)
{
    int idx =  (PowerSupplyPartNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::PowerSupplyPartNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::PowerSupplyPartNumber()
{
    int idx = (PowerSupplyPartNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::PowerSupplySerialNumber() --
//---------------------------------------------------------------------------

void SemaAssemblyInfo::PowerSupplySerialNumber(std::string Number)
{
    int idx =  (PowerSupplySerialNumber_HeaderOffset * AssemblyWordSize);
    Rom.AsString(idx,AssemblyWordSize,Number);
}

//---------------------------------------------------------------------------
//  SemaAssemblyInfo::PowerSupplySerialNumber() --
//---------------------------------------------------------------------------

std::string SemaAssemblyInfo::PowerSupplySerialNumber()
{
    int idx = (PowerSupplySerialNumber_HeaderOffset * AssemblyWordSize);
    std::string Number = Rom.AsString(idx,AssemblyWordSize);
    return Number;
}

}  // end namespace Innovative
