//---------------------------------------------------------------------------
//
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SemaIdrom_MbH
#define PmcIdrom_MbH

#include <IdromDatagram_Mb.h>
#include <Sema\Sema.h>
namespace Innovative
{

//=============================================================================
//  CLASS SemaIdrom  --  Omnibus Module Idrom Access Object
//=============================================================================
//  The Module IDROM contains a header containing a 12 character name field
//     (1 word per character), follwed by a Revision Code character. The
//     final word is a checksum.  This leaves a Working Area of 110 words
//     that are divided up between module-specific data and fallow words
//     that can be used by an applicaton.
//
class SemaIdrom
{
    static const int           IdromPacketSizeInBytes = 1000;   // IDROM size in words

public:
    //
    //  Member Functions
    SemaIdrom();
    ~SemaIdrom();

    void   Clear();
    void   LoadFromRom();
    void   StoreToRom();

    //
    //  Properties
    std::string     Name() const;
    void            Name(const std::string & name);
    std::string     Revision() const;
    void            Revision(const std::string & revision);
    unsigned int    SerialNumber() const;
    void            SerialNumber(unsigned int Num);
    unsigned int    PartNumber() const;
    void            PartNumber(unsigned int part);

    bool            IsEmpty() const;


    // ...User Area Access Functions
    byte   AsChar(int idx) const;
    void   AsChar(int idx, byte data);
    void   AsString(unsigned int int_idx, unsigned int ints_to_use, const std::string & text);
    std::string  AsString(unsigned int int_idx, unsigned int ints_to_use) const;

    bool   BmcFlag()
            { return Rom.BmcFlag(); }

private:
    //
    //  Member Data
    byte   Packet[IdromPacketSizeInBytes];
    //
    //  Support methods
	unsigned int    SizeInChars() const;

private:
    Sema         Rom;
};

//=============================================================================
//  CLASS SemaAssemblyInfo  --
//=============================================================================
class SemaAssemblyInfo
{
public:
    SemaAssemblyInfo();
    ~SemaAssemblyInfo();

    void ReadMemoryBlock();
    void WriteMemoryBlock();

    void CaciDartPartNumber(std::string Number);
    std::string CaciDartPartNumber();

    void MiniK7ChassisPartNumber(std::string Number);
    std::string MiniK7ChassisPartNumber();

    void MiniK7RFExpansionPartNumber(std::string Number);
    std::string MiniK7RFExpansionPartNumber();

    void LCDTouchScreenPartNumber(std::string Number);
    std::string LCDTouchScreenPartNumber();

    void DisplayPartNumber(std::string Number);
    std::string DisplayPartNumber();

    void DisplaySerialNumber(std::string Number);
    std::string DisplaySerialNumber();

    void HardDrivePartNumber(std::string Number);
    std::string HardDrivePartNumber();

    void HardDriveSerialNumber(std::string Number);
    std::string HardDriveSerialNumber();

    void Fmc250PartNumber(std::string Number);
    std::string Fmc250PartNumber();

    void Fmc250SerialNumber(std::string Number);
    std::string Fmc250SerialNumber();

    void AtomPartNumber(std::string Number);
    std::string AtomPartNumber();

    void XilinxPartNumber(std::string Number);
    std::string XilinxPartNumber();

    void XilinxSerialNumber(std::string Number);
    std::string XilinxSerialNumber();

    void MemoryRAMPartNumber(std::string Number);
    std::string MemoryRAMPartNumber();

    void MemoryRAMSerialNumber(std::string Number);
    std::string MemoryRAMSerialNumber();

    void CompactCOMExpressPartNumber(std::string Number);
    std::string CompactCOMExpressPartNumber();

    void CompactCOMExpressSerialNumber(std::string Number);
    std::string CompactCOMExpressSerialNumber();

    void PowerSupplyPartNumber(std::string Number);
    std::string PowerSupplyPartNumber();

    void PowerSupplySerialNumber(std::string Number);
    std::string PowerSupplySerialNumber();



    bool BmcFlag()
        { return Rom.BmcFlag(); }

private:
    SemaIdrom Rom;

    const unsigned int CaciDartPartNumber_HeaderOffset;
    const unsigned int MiniK7ChassisPartNumber_HeaderOffset;
    const unsigned int MiniK7RFExpansionPartNumber_HeaderOffset;
    const unsigned int LCDTouchScreenPartNumber_HeaderOffset;
    const unsigned int DisplayPartNumber_HeaderOffset;
    const unsigned int DisplaySerialNumber_HeaderOffset;
    const unsigned int HardDrivePartNumber_HeaderOffset;
    const unsigned int HardDriveSerialNumber_HeaderOffset;
    const unsigned int Fmc250PartNumber_HeaderOffset;
    const unsigned int Fmc250SerialNumber_HeaderOffset;
    const unsigned int AtomPartNumber_HeaderOffset;
    const unsigned int XilinxPartNumber_HeaderOffset;
    const unsigned int XilinxSerialNumber_HeaderOffset;
    const unsigned int MemoryRAMPartNumber_HeaderOffset;
    const unsigned int MemoryRAMSerialNumber_HeaderOffset;
    const unsigned int CompactCOMExpressPartNumber_HeaderOffset;
    const unsigned int CompactCOMExpressSerialNumber_HeaderOffset;
    const unsigned int PowerSupplyPartNumber_HeaderOffset;
    const unsigned int PowerSupplySerialNumber_HeaderOffset;

};


}  // end namespace Innovative

#endif
