//  Semaphore_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Semaphore_Mb.h>
#include <algorithm>

namespace   Innovative
{

//=============================================================================
//  CLASS Semaphore  --  OS Semaphore object wrapper
//=============================================================================
//  "Original" constructed semaphore as a "guarded integer"

//-------------------------------------------------------------------
// Semaphore::Semaphore() -- Constructor.
//-------------------------------------------------------------------

Semaphore::Semaphore(unsigned int count, unsigned int max_count)
    : mMaxCount(max_count), mCount(std::min(count, max_count)), mSignal(Event::AutoReset)
{
    if (count)
        signal();
}

//-------------------------------------------------------------------
// Semaphore::Acquire() -- Interlocking call if FCount = 0.
//                         Timeout in seconds.
//-------------------------------------------------------------------

bool Semaphore::Acquire(unsigned int mS)
{
    if (!mSignal.WaitFor(mS))
        return false;

    MutexedLock Lock(mLock);
    --mCount;

    bool result = true;
    if (mCount)
        result = signal();

    return result;
}

//-------------------------------------------------------------------
// Semaphore::Release() - release N values
//-------------------------------------------------------------------

bool Semaphore::Release(unsigned int count)
{
    return release(count);
}

//---------------------------------------------------------------------------
// Semaphore::Clear() -- Reduce Semaphore count to 0.
//---------------------------------------------------------------------------

bool Semaphore::Clear()
{
    MutexedLock Lock(mLock);
    mCount = 0;

    return mSignal.Clear();
}

//------------------------------------------------------------------------------
//  Semaphore::Count() -- Return count of Semaphore
//------------------------------------------------------------------------------

unsigned int  Semaphore::Count()
{
    MutexedLock Lock(mLock);
    return mCount;
}

//------------------------------------------------------------------------------
//  Semaphore::IsSignalled() -- Return true if object is signalled
//------------------------------------------------------------------------------

bool Semaphore::IsSignalled()
{
    return Count() ? true : false;
}

//------------------------------------------------------------------------------
//  Semaphore::WaitFor() -- Wait on event
//------------------------------------------------------------------------------

bool  Semaphore::WaitFor(unsigned int mS)
{
    return Acquire(mS);
}

//-------------------------------------------------------------------
// Semaphore::release()
//-------------------------------------------------------------------

bool Semaphore::release(unsigned int count)
{
    MutexedLock Lock(mLock);

    mCount = std::min(mMaxCount, mCount+count);
    return signal();
}

//------------------------------------------------------------------------------
//  Semaphore::signal() -- Return true if object successfully signalled
//------------------------------------------------------------------------------

bool  Semaphore::signal()
{
    bool result = mSignal.Set();
    SignalOwners();

    return result;
}

}  //namespace Innovative
