// OWSet_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef Set_MbH
#define Set_MbH

namespace OpenWire
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//=============================================================================
//  CLASS Set  --
//=============================================================================
template<class T, unsigned char minEl, unsigned char maxEl>
class Set
{
protected:
    unsigned char Data[((((int)(maxEl/8))-((int)(minEl/8))+1) != 3)?
         (((int)(maxEl/8))-((int)(minEl/8))+1): 4];

public:
    Set() { memset(Data, 0, sizeof (Data)); }
    Set(const Set& src)
      {
      for (unsigned int i = 0; i < sizeof (Data); i++)
        Data[i] = src.Data[i];
      }

    Set& operator =(const Set& rhs)
      {
      if (this != &rhs)
        {
        for (int i = 0; i < (int)sizeof (Data); i++)
        Data[i] = rhs.Data[i];
        }
        
      return *this;
      }
      
    Set& operator +=(const Set& rhs)     //Union
      {
      for (int i = 0; i < sizeof (Data); i++)
        Data[i] |= rhs.Data[i];
      return *this;
      }
      
    Set&  operator -=( const Set& rhs )     //Difference
      {
      for (int i = 0; i < sizeof (Data); i++)
        Data[i] ^= (Data[i] & rhs.Data[i]);
        
      return *this;
      }

    Set&  operator *=( const Set& rhs )     //Intersection
      {
      for (int i = 0; i < sizeof (Data); i++)
        Data[i] &= rhs.Data[i];
      return *this;
      }

    Set  operator +( const Set& rhs ) const //Union
      {
      Set<T, minEl, maxEl> s;
      for (int i = 0; i < sizeof (Data); i++)
        s.Data[i] = Data[i] | rhs.Data[i];
        
      return s;
      }

    Set  operator -( const Set& rhs ) const //Difference
      {
      Set<T, minEl, maxEl> s;
      for (int i = 0; i < sizeof (Data); i++)
        s.Data[i] = Data[i] ^ (Data[i] & rhs.Data[i]);
        
      return s;
      }

    Set  operator *( const Set& rhs ) const //Intersection
      {
      Set<T, minEl, maxEl> s;
      for (int i = 0; i < sizeof (Data); i++)
        s.Data[i] = Data[i] & rhs.Data[i];

      return s;
      }

    Set&  operator <<( const T el ) //Add element
      {
      if (el >= minEl && el <= maxEl)
        Data[(int)(el/8) - (int)(minEl/8)] |=
        (unsigned char)((unsigned short)1 << (el % 8));
      return *this;
      }
      
    Set&  operator >>( const T el ) //Remove element
      {
      if (el >= minEl && el <= maxEl)
        Data[(int)(el/8) - (int)(minEl/8)] &=
        (unsigned char)~((unsigned short)1 << (el % 8));
        
      return *this;
      }

    bool  Contains(const T el) const
      {
      return (el >= minEl && el <= maxEl)?
		  ((Data[(int)(el/8) - (int)(minEl/8)] & (unsigned char)((unsigned short)1 << (el % 8))) ? true : false) :
        false;
      }

    Set&  Clear()
      {
      memset(Data, 0, sizeof (Data));
      return *this;
      }

    bool  Empty() const
      {
      for (int i = (sizeof(Data) >> 1)-1; i >= 0; --i)
        if (((short *) Data)[i] != 0)
          return false;
      if (sizeof(Data) & 1)
        return (Data[sizeof(Data)-1] == 0);
        
      return true;
      }

    bool  operator ==(const Set& rhs) const
      {
      for (int i = 0; i < sizeof (Data); i++)
        if (Data[i] != rhs.Data[i])
        // Not so fast, if first or last byte, make sure that
        // the compare does not include unused bits (mask them)
        if (i == 0)
          {
          if (((0xFF >> (8-(minEl%8))) ^ Data[i]) !=
          ((0xFF >> (8-(minEl%8))) ^ rhs.Data[i]))
          return false;
          }

        else if (sizeof (Data) > 1 && i == sizeof (Data) - 1)
          {
          if (((0xFF << (1+(maxEl%8))) ^ Data[i]) !=
          ((0xFF << (1+(maxEl%8))) ^ rhs.Data[i]))
          return false;
          }

        else
          return false;
          
      return true;
      }
      
    bool  operator !=(const Set& rhs) const {return !operator==(rhs);}
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // OpenWire
//---------------------------------------------------------------------------
#endif
