//
// Si570_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Si570_MbH
#define Si570_MbH

#include <DataTypes_Mb.h>
#include <Clockbase_Mb.h>
#include <I2C_Mb.h>
#include <VcxoControl_Mb.h>
#include <vector>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//=============================================================================
//  CLASS Si570 --  Programmable XO/VCXO Chip Support
//=============================================================================

class Si570 :  public ClockBase, public IVcxoControl
{
public:
    enum TypeCode{tcA, tcB, tcC};// see page 24 of Si570 data sheet
    Si570(IRequires_I2C_RegIo & device, TypeCode tCode, double pwr_on_mhz = 125.0);

    void            SetI2C_SubAddr(int sub_addr)
                        {  FOverrideI2C = true; FOverrideI2CAddr = sub_addr;  }

    virtual void        Init();
    virtual void        Reapply();
    virtual void        Calibrate(double power_on_rate_MHz);

    double          PowerOnFrequencyMhz()
                        {  return FPowerOnFrequencyMhz;  };

    void    SyncOnInit(bool value)
        {  FSyncOnInit = value;  }
    bool    SyncOnInit()
        {  return FSyncOnInit;  }

    //  Register Image I/O
    unsigned int    HsDiv();
    void            HsDiv(unsigned int value);
    unsigned int    N1();
    void            N1(unsigned int value);
    ii64            RFreq();
    void            RFreq(ii64 value);

    void LoadFreq(int hsdiv, int n1, ii64 rfreq);

    //  Raw configuration accesses
    void            Port(unsigned char reg, unsigned char value);
    unsigned char   Port(unsigned char reg);

    IVcxoControl &      VcxoIntf()
        {  return *this;   }
    //
    //  IVcxoControl Interface Impl
    virtual double  Min();
    virtual double  Max();
    virtual void    SetRange(double /*min*/, double /*max*/)
                        {  }        //  Can't set tuning range
    virtual void    OutputEnabled(bool state)  // we support this
                        {  FPermaReset = (state==false);  }

    //  "holes" in the Vco

    virtual bool    CanTuneTo(double freq);
    virtual double  BestTunableFrequency(double freq);
    virtual void    SetVcoFrequency(double freq)
        {  Frequency(freq);  }

    void            DcoRange(double low, double high)
                        { DcoLow = low;  DcoHigh = high;  }

    //  'Reattach' I2C Io Interface
    void  Relink_I2C(IRequires_I2C_RegIo & device)
            {
            Device = &device;
            }

protected:
    enum DataRegCodes { rcReg7, rcReg8, rcReg9, rcReg10, rcReg11, rcReg12, rcDataRegCount};
    enum CtrlRegCodes { rcReg135, rcReg137, rcCtrlRegCount};

    TypeCode                 FVcxoType;
    IRequires_I2C_RegIo *    Device;

    double          DcoLow;       // DCO operational frequency range, Hz
    double          DcoHigh;

    double          FPowerOnFrequencyMhz;
    double          FCurrentFrequency;

    bool            FOverrideI2C;
    int             FOverrideI2CAddr;

    bool            FPermaReset;

    std::vector<int>  RegisterImage;
    std::vector<int>  ControlImage;
    std::vector<int>  Valid_HsDiv;
    std::vector<int>  Valid_N1;

    const double    TwoTwentyEightPowerAsFloat;
    bool            FSyncOnInit;
    Trace           Debug;

    double  Gap0Min() { return 945.0e6;   }
    double  Gap0Max() { return 970.0e6;   }
    double  Gap1Min() { return 1134.0e6;  }
    double  Gap1Max() { return 1213.0e6;  }
	
    virtual void    SetEnabled(bool state);
    virtual void    SetFrequency(double freq);
    virtual double  GetFrequencyActual() const;

    void            InitValidDividers();
    void            DividerScan(double & freq, int & HsDiv, int & N1);
    void            ReadDataRegisters();
    void            WriteDataRegisters();
//    void            ReInit();
    void            QuickUpdate(double new_freq);
    void            FullUpdate(double new_freq);
    void            Freeze();
    void            Unfreeze();
    void            NewFrequency();
    void            Reset();
    void            Recall();
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

} // namespace Innovative
#endif
