// SignalGen_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <SignalGen_Mb.h>
#include <IppFloatDG_Mb.h>
#include <ipp.h>
#include <cmath>


namespace   Innovative
{

const float SLPi = 3.141592621f; //DO NOT CHANGE THIS CONSTANT !!!!
// The following will not work with IPP !!!!
//const float SLPi = static_cast<float>(4.0*std::atan(1.0));

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//---------------------------------------------------------------------------
// SignalGen()
//---------------------------------------------------------------------------

SignalGen::SignalGen()
	: FSampleRate(44100), FPhase(0), FAmplitude(1000), FFrequency(100), FAsymmetry(0),
	  FSignalType(stTone), FCalculationOption(saDefault)
{
    Phase(FPhase);
}

//---------------------------------------------------------------------------
// SignalGen::SignalGen() --
//---------------------------------------------------------------------------

SignalGen::SignalGen(const SignalGen & other)
	: inherited(other), FSampleRate(44100),
      FPhase(other.FPhase), FAmplitude(other.FAmplitude),
	  FFrequency(other.FFrequency), FAsymmetry(other.FAsymmetry),
	  FSignalType(other.FSignalType), FCalculationOption(other.FCalculationOption)
{
    Phase(FPhase);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  SignalGen::operator = --
//---------------------------------------------------------------------------

SignalGen & SignalGen::operator = (const SignalGen & other)
{
	inherited::operator = (other);

	FSampleRate = other.FSampleRate;
	FAmplitude = other.FAmplitude;
	FFrequency = other.FFrequency;
	FPhase = other.FPhase;
	FAsymmetry = other.FAsymmetry;
	FSignalType = other.FSignalType;

    Phase(FPhase);

    return *this;
}

//---------------------------------------------------------------------------
//  SignalGen::Generate() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4244)
#endif
bool SignalGen::Generate(IppFloatDG & buffer)
{
	bool result = true;

	float IntSampleRatio = FFrequency / FSampleRate;

	IppFloatDG ToneScratchDG(ToneScratchBuffer);

	switch (FSignalType)
		{
		case stTone:
			{
			IppHintAlgorithm hint;
			switch (FCalculationOption)
				{
				default:
				case saDefault:   hint = ippAlgHintNone;      break;
				case saFast:      hint = ippAlgHintFast;      break;
				case saAccurate:  hint = ippAlgHintAccurate;  break;
				};

			IppStatus status = ippsTone_32f(buffer.Ptr(), static_cast<int>(buffer.size()),
				FAmplitude, IntSampleRatio, &FPhase, hint);
			result &= (status == ippStsNoErr);
			}
			break;

		case stTriangle:
			{
			IppStatus status = ippsTriangle_32f(buffer.Ptr(),
										static_cast<int>(buffer.size()),
										FAmplitude,
										IntSampleRatio,
										FAsymmetry,
										&mPhase);
			result &= (status == ippStsNoErr);
			}
			break;

		case stSquare:
			{
			Buffer TmpBuffer;
			IppFloatDG TmpBufferDG(TmpBuffer);
			TmpBufferDG.Resize(buffer.size());

			IppStatus status = ippsTriangle_32f(buffer.Ptr(),
									static_cast<int>(buffer.size()),
									FAmplitude*1.2,
									IntSampleRatio,
									SLPi,
									&mPhase);
			result &= (status == ippStsNoErr);

			status = ippsTriangle_32f(TmpBufferDG.Ptr(),
									static_cast<int>(TmpBufferDG.size()),
									FAmplitude*1.2,
									IntSampleRatio,
									-SLPi,
									&mPhase1);
			result &= (status == ippStsNoErr);

			buffer += TmpBufferDG;

			status = ippsThreshold_LTValGTVal_32f_I(buffer.Ptr(),
										   static_cast<int>(TmpBufferDG.size()),
										   -FAmplitude,
										   -FAmplitude,
										   FAmplitude,
										   FAmplitude);
			result &= (status == ippStsNoErr);

			break;
			}

		case stDc:
			buffer.Set(FAmplitude);
			break;
		}

	if (!result)
		buffer.Zero();

	return result;
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  SignalGen::Generate() --
//---------------------------------------------------------------------------

bool SignalGen::Generate(IppComplexDG & buffer)
{
	bool result = true;

	float IntSampleRatio = FFrequency / FSampleRate;

	IppComplexDG ToneScratchDG(ToneScratchBuffer);

	switch (FSignalType)
		{
		case stTone:
			{
			IppHintAlgorithm hint;
			switch (FCalculationOption)
				{
				default:
				case saDefault:   hint = ippAlgHintNone;      break;
				case saFast:      hint = ippAlgHintFast;      break;
				case saAccurate:  hint = ippAlgHintAccurate;  break;
				};

			IppStatus status = ippsTone_32fc(reinterpret_cast<Ipp32fc*>(buffer.Ptr()), static_cast<int>(buffer.size()),
				FAmplitude, IntSampleRatio, &FPhase, hint);
			result &= (status == ippStsNoErr);
			}
			break;

		case stTriangle:
			{
			IppStatus status = ippsTriangle_32fc(reinterpret_cast<Ipp32fc*>(buffer.Ptr()),
										static_cast<int>(buffer.size()),
										FAmplitude,
										IntSampleRatio,
										FAsymmetry,
										&mPhase);
			result &= (status == ippStsNoErr);
			}
			break;

		case stSquare:
			{
			Buffer TmpBuffer;
			IppComplexDG TmpBufferDG(TmpBuffer);
			TmpBufferDG.Resize(buffer.size());

			IppStatus status = ippsTriangle_32fc(reinterpret_cast<Ipp32fc*>(buffer.Ptr()),
									static_cast<int>(buffer.size()),
									FAmplitude*1.2f,
									IntSampleRatio,
									SLPi,
									&mPhase);
			result &= (status == ippStsNoErr);

			status = ippsTriangle_32fc(reinterpret_cast<Ipp32fc*>(TmpBufferDG.Ptr()),
									static_cast<int>(TmpBufferDG.size()),
									FAmplitude*1.2f,
									IntSampleRatio,
									-SLPi,
									&mPhase1);
			result &= (status == ippStsNoErr);

			buffer += TmpBufferDG;

			IppFloatDG t2(buffer.WrappedBuffer());

			status = ippsThreshold_LTValGTVal_32f_I(reinterpret_cast<Ipp32f*>(t2.Ptr()),
										   static_cast<int>(TmpBufferDG.size()),
										   -FAmplitude,
										   -FAmplitude,
										   FAmplitude,
										   FAmplitude);
			result &= (status == ippStsNoErr);

			break;
			}

		case stDc:
			buffer.Set(FAmplitude);
			break;
		}

	if (!result)
		buffer.Zero();

	return result;
}

//---------------------------------------------------------------------------//
// Phase()
//---------------------------------------------------------------------------

void SignalGen::Phase(float phase)
{
	FPhase = phase;
	mPhase = static_cast<float>(phase);
	mPhase1 = mPhase + 3.2f;
    if( mPhase1 > 2 * 3.2f )
        mPhase1 -= 2 * 3.2f;
}

//---------------------------------------------------------------------------//
// Phase()
//---------------------------------------------------------------------------

float SignalGen::Phase() const
{
    return FPhase;
}

//---------------------------------------------------------------------------//
// Amplitude()
//---------------------------------------------------------------------------

void SignalGen::Amplitude(float amplitude)
{
    FAmplitude = amplitude;
}

//---------------------------------------------------------------------------//
// Amplitude()
//---------------------------------------------------------------------------

float SignalGen::Amplitude() const
{
    return FAmplitude;
}

//---------------------------------------------------------------------------//
// SampleRate()
//---------------------------------------------------------------------------

void SignalGen::SampleRate(float rate)
{
    FSampleRate = rate;
}

//---------------------------------------------------------------------------//
// SampleRate()
//---------------------------------------------------------------------------

float SignalGen::SampleRate() const
{
    return FSampleRate;
}

//---------------------------------------------------------------------------//
// Frequency()
//---------------------------------------------------------------------------

void SignalGen::Frequency(float frequency)
{
    FFrequency = frequency;
}

//---------------------------------------------------------------------------//
// Frequency()
//---------------------------------------------------------------------------

float SignalGen::Frequency() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------//
// Asymmetry()
//---------------------------------------------------------------------------

void SignalGen::Asymmetry(float asymmetry)
{
    FAsymmetry = asymmetry;
}

//---------------------------------------------------------------------------//
// Asymmetry()
//---------------------------------------------------------------------------

float SignalGen::Asymmetry() const
{
	return FAsymmetry;
}

//---------------------------------------------------------------------------//
// SetSignalType()
//---------------------------------------------------------------------------

void SignalGen::SignalType(SignalTypes value)
{
	FSignalType = value;
}

//---------------------------------------------------------------------------//
// SetSignalType()
//---------------------------------------------------------------------------

SignalGen::SignalTypes SignalGen::SignalType() const
{
	return FSignalType;
}

//---------------------------------------------------------------------------
//  SignalGen::CalculationOption() --
//---------------------------------------------------------------------------

void SignalGen::CalculationOption(SignalCalcOptions option)
{  
    FCalculationOption = option; 
}

//---------------------------------------------------------------------------
//  SignalGen::CalculationOption() --
//---------------------------------------------------------------------------

SignalGen::SignalCalcOptions SignalGen::CalculationOption() const
{
	return FCalculationOption;
}



} // namespace Innovative

