// SignalQueue_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SignalQueue_MbH
#define SignalQueue_MbH

#include <queue>
#include <Semaphore_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS  SignalQueue  --  Queue with semaphore synched with depth
//==============================================================================
//  Note: this version is not thread-safe. Use in a single thread only

template <class TElement> class SignalQueue
{
public:
    enum PopStyle { PopSem, IgnoreSem };
    //
    //  Member Functions
    SignalQueue()  {}

    bool  empty() const
        {
        return Queue.empty();
        }

    size_t  depth() const
        {
        return Queue.size();
        }

    void  clear()
        {
        while (!Queue.empty())
            Queue.pop();
        Sem.Clear();
        }

    void  push(TElement value)
        {
        Queue.push(value);
        Sem.Release();
        }

    TElement  front()
        {
        return Queue.front();
        }

    void  pop(PopStyle style=IgnoreSem)
        {
        //  There are 2 styles here since the Sem may have been pre-popped
        //    by a wait-for before processing. If so, use the default. If
        //    manually extracting w/o waitfors, use the PopSem style.
        if (style==PopSem)
            Sem.Acquire();
        Queue.pop();
        }

    Semaphore & SignalSem()
        {
        return Sem;
        }

private:
    //
    //  Member Data
    mutable Semaphore        Sem;
    std::queue<TElement>     Queue;

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
