// SingleWriteLock_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifdef __BORLANDC__
#include <mem.h>
#endif

#include <SyncObjects_Mb.h>
#include <Osal_Mb.h>
#include <SingleWriteLock_Mb.h>
#include <assert.h>
#include <limits>
#include <memory>
#include <SystemSupport_Mb.h>

namespace   Innovative
{

//===========================================================================
//  CLASS ThreadCounterImpl  --
//===========================================================================

class ThreadCounterImpl
{
public:
	ThreadCounterImpl(){};
	unsigned int CurrentThreadId();
};

//---------------------------------------------------------------------------
// ThreadCounterImpl::CurrentThreadId()
//---------------------------------------------------------------------------

unsigned int ThreadCounterImpl::CurrentThreadId()
{
	return InnovativeKernel::CurrentThreadId();
}

//===========================================================================
//  CLASS ThreadCounter  --
//===========================================================================

//---------------------------------------------------------------------------
// ThreadCounter::ThreadCounter() --
//---------------------------------------------------------------------------

ThreadCounter::ThreadCounter()
	: mHashTable(16)
{
	impl = new ThreadCounterImpl;
	HashTableArray::iterator i;
	for (i = mHashTable.begin(); i != mHashTable.end(); ++i)
		*i = NULL;
}

//---------------------------------------------------------------------------
// ThreadCounter::~ThreadCounter() --
//---------------------------------------------------------------------------

ThreadCounter::~ThreadCounter()
{
	PThreadInfo P, Q;
	for (int i = 0; i < 16; ++i)
		{
		P = mHashTable[i];
		mHashTable[i] = NULL;
		while (P)
			{
            Q = P;
            P = P->Next;
            delete Q;
            }
        }
}

//---------------------------------------------------------------------------
//  ThreadCounter::HashIndex() --
//---------------------------------------------------------------------------
 
unsigned char ThreadCounter::HashIndex()
{
	unsigned short H = static_cast<unsigned short>(impl->CurrentThreadId());
    return ((H & 0xFF) ^ (H >> 8)) & 0xF;
}

//---------------------------------------------------------------------------
//  ThreadCounter::Open() --
//---------------------------------------------------------------------------
 
PThreadInfo ThreadCounter::Open()
{
    unsigned char H = HashIndex();
	unsigned int CurThread = impl->CurrentThreadId();

    PThreadInfo P = mHashTable[H];
    while (P && (P->ThreadID != CurThread))
        P = P->Next;

    if (P == NULL)
        {
        P = Recycle();

        if (P == NULL)
            {
            P = new ThreadInfo;
            P->RecursionCount = 0;
            P->Next = NULL;
            P->ThreadID = CurThread;
			P->Active = std::numeric_limits<int>::max();

            // Another thread could start traversing the list between when we set the
            // head to P and when we assign to P.Next.  Initializing P.Next to point
            // to itself will make others spin until we assign the tail to P.Next.
            P->Next = P;
			AtomicAccess<PThreadInfo> atomic;
            P->Next = PThreadInfo(atomic.Exchange(mHashTable[H], P));
            //P->Next = PThreadInfo(IIInterlockedExchange((long*)&( mHashTable[H] ), (long)(P)));
            }
        }
                                                     
    return P;
}

//---------------------------------------------------------------------------
//  ThreadCounter::Close() --
//---------------------------------------------------------------------------
 
void ThreadCounter::Close(PThreadInfo&  Thread)
{
    Thread = NULL;
}

//---------------------------------------------------------------------------
//  ThreadCounter::Delete() --
//---------------------------------------------------------------------------
 
void ThreadCounter::Delete(PThreadInfo Thread)
{
    Thread->ThreadID = 0;
    Thread->Active = 0;
}

//---------------------------------------------------------------------------
//  ThreadCounter::Recycle() --
//---------------------------------------------------------------------------
 
PThreadInfo ThreadCounter::Recycle()
{
    PThreadInfo Result = mHashTable[HashIndex()];
    while (Result)
        {
        int Gen = Atomic.Exchange(Result->Active, std::numeric_limits<int>::max());
        if (Gen != std::numeric_limits<int>::max())
            {
            Result->ThreadID = impl->CurrentThreadId();
            return Result;
            }

        else
            Result = Result->Next;

        }

    return Result;
}

//===========================================================================
//  CLASS SingleWriteLock  --  
//===========================================================================

const int  mrWriteRequest = 0xFFFF;

//---------------------------------------------------------------------------
// SingleWriteLock::SingleWriteLock() --
//---------------------------------------------------------------------------
 
SingleWriteLock::SingleWriteLock()
    : mReadSignal(Event::ManualReset, true),
      mWriteSignal(Event::AutoReset, false)
{
    mWriteRecursionCount = 0;
    mWriterID = 0;
    mRevisionLevel = 0;
    mSentinel = mrWriteRequest;
}

//---------------------------------------------------------------------------
// SingleWriteLock::~SingleWriteLock() --
//---------------------------------------------------------------------------
 
SingleWriteLock::~SingleWriteLock()
{
    BeginWrite();
}

//---------------------------------------------------------------------------
//  SingleWriteLock::BlockReaders() --
//---------------------------------------------------------------------------
 
void SingleWriteLock::BlockReaders()
{
    mReadSignal.Clear();
}

//---------------------------------------------------------------------------
//  SingleWriteLock::UnblockReaders() --
//---------------------------------------------------------------------------

void SingleWriteLock::UnblockReaders()
{
    mReadSignal.Set();
}

//---------------------------------------------------------------------------
//  SingleWriteLock::UnblockOneWriter() --
//---------------------------------------------------------------------------

void SingleWriteLock::UnblockOneWriter()
{
    mWriteSignal.Set();
}

//---------------------------------------------------------------------------
//  SingleWriteLock::WaitForReadSignal() --
//---------------------------------------------------------------------------

void SingleWriteLock::WaitForReadSignal()
{
    mReadSignal.WaitFor();
}

//---------------------------------------------------------------------------
//  SingleWriteLock::WaitForWriteSignal() --
//---------------------------------------------------------------------------

void SingleWriteLock::WaitForWriteSignal()
{
    mWriteSignal.WaitFor();
}

//---------------------------------------------------------------------------
//  SingleWriteLock::BeginWrite() --
//---------------------------------------------------------------------------
 
bool SingleWriteLock::BeginWrite()
{
    PThreadInfo     Thread;
    bool            HasReadLock;
    unsigned int    ThreadID;
    int             Test;
    unsigned int    OldRevisionLevel;

    bool        Result = true;

	ThreadID = InnovativeKernel::CurrentThreadId();
    if (mWriterID != ThreadID)  // somebody or nobody has a write lock
        {

        // Prevent new readers from entering while we wait for the existing readers
        // to exit.
        BlockReaders();

        OldRevisionLevel = mRevisionLevel;

        Thread = tls.Open();

        // We have another lock already. It must be a read lock, because if it
        // were a write lock, FWriterID would be our threadid.
        HasReadLock = Thread->RecursionCount > 0;

        if (HasReadLock)        // acquiring a write lock requires releasing read locks
            Atomic.Increment(mSentinel);

        // InterlockedExchangeAdd returns prev value
        while( Atomic.Add(mSentinel, -mrWriteRequest) != mrWriteRequest )
            {

            // Undo what we did, since we didn't get the lock
            Test = Atomic.Add(mSentinel, mrWriteRequest);

            // If the old value (in Test) was 0, then we may be able to
            // get the lock (because it will now be mrWriteRequest). So,
            // we continue the loop to find out. Otherwise, we go to sleep,
            // waiting for a reader or writer to signal us.
            if( Test )
                WaitForWriteSignal();

            }

        // At the EndWrite, first Writers are awoken, and then Readers are awoken.
        // If a Writer got the lock, we don't want the readers to do busy
        // waiting. This Block resets the event in case the situation happened.
        BlockReaders();

        // Put our read lock marker back before we lose track of it
        if( HasReadLock )
            Atomic.Decrement(mSentinel);

        mWriterID = ThreadID;

        Result = ( int(OldRevisionLevel) ==
                        (Atomic.Increment(mRevisionLevel) - 1) );
        }

	++mWriteRecursionCount;

    return Result;
}

//---------------------------------------------------------------------------
//  SingleWriteLock::EndWrite() --
//---------------------------------------------------------------------------

void SingleWriteLock::EndWrite()
{
	assert(mWriterID == InnovativeKernel::CurrentThreadId());

    PThreadInfo Thread = tls.Open();
    --mWriteRecursionCount;
    if (mWriteRecursionCount == 0)
        {
        mWriterID = 0;
        Atomic.Add(mSentinel, mrWriteRequest);
        UnblockOneWriter();
        UnblockReaders();
        }

    if (Thread->RecursionCount == 0)
        tls.Delete(Thread);
}

//---------------------------------------------------------------------------
//  SingleWriteLock::BeginRead() --
//---------------------------------------------------------------------------

void SingleWriteLock::BeginRead()
{
    PThreadInfo Thread = tls.Open();
    ++Thread->RecursionCount;

    bool    WasRecursive = Thread->RecursionCount > 1;

	if (mWriterID != InnovativeKernel::CurrentThreadId())
        {

        // In order to prevent recursive Reads from causing deadlock,
        // we need to always WaitForReadSignal if not recursive.
        // This prevents unnecessarily decrementing the FSentinel, and
        // then immediately incrementing it again.
        if (!WasRecursive)
            {

            // Make sure we don't starve writers. A writer will
            // always set the read signal when it is done, and it is initially on.
            WaitForReadSignal();
            while (Atomic.Decrement(mSentinel) <= 0)
                {

                // Because the InterlockedDecrement happened, it is possible that
                // other threads "think" we have the read lock,
                // even though we really don't. If we are the last reader to do this,
                // then SentValue will become mrWriteRequest
                int SentValue = Atomic.Increment(mSentinel);

                // So, if we did inc it to mrWriteRequest at this point,
                // we need to signal the writer.
                if (SentValue == mrWriteRequest)
                    UnblockOneWriter();

                // This sleep below prevents starvation of writers
				Innovative::Sleep(0);

                WaitForReadSignal();
				}
            }
        }

}

//---------------------------------------------------------------------------
//  SingleWriteLock::EndRead() --
//---------------------------------------------------------------------------
 
void SingleWriteLock::EndRead()
{
    PThreadInfo Thread = tls.Open();
    --Thread->RecursionCount;
    if (Thread->RecursionCount == 0)
        {
        tls.Delete(Thread);

		// original code below commented out
		if (mWriterID != InnovativeKernel::CurrentThreadId())
            {
            int Test = Atomic.Increment(mSentinel);

            // It is possible for Test to be mrWriteRequest
            // or, it can be = 0, if the write loops:
            // Test := InterlockedExchangeAdd(FSentinel, mrWriteRequest) + mrWriteRequest;
            // Did not get executed before this has called (the sleep debug makes it happen faster)
            if (Test == mrWriteRequest)
                UnblockOneWriter();

            else if (Test <= 0)         // We may have some writers waiting
                {
                if ((Test % mrWriteRequest) == 0)
                    UnblockOneWriter(); // No more readers left (only writers) so signal one of them
                }
            }
        }

}


} // namespace Innovative
