// SingleWriteLock_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef SingleWriteLock_MbH
#define SingleWriteLock_MbH

#include <Event_Mb.h>
#include <SyncObjects_Mb.h>

namespace   Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

struct ThreadInfo
{
    ThreadInfo*     Next;
    unsigned int    ThreadID;
    int				Active;
    unsigned int    RecursionCount;
};

typedef ThreadInfo*     PThreadInfo;

//=============================================================================
//  CLASS ThreadCounter  --
//=============================================================================

class ThreadCounter
{
public:
    ThreadCounter();
    ~ThreadCounter();

	// Methods
    PThreadInfo     Open();
    void            Delete(PThreadInfo Thread);
    void            Close(PThreadInfo& Thread);

private:
	// Data
	AtomicAccess<int>	Atomic;
	typedef std::vector<PThreadInfo> HashTableArray;
	HashTableArray		mHashTable;

	// Methods
    unsigned char		HashIndex();
    PThreadInfo			Recycle();

	// Implementation
    class ThreadCounterImpl * impl;
};

//=============================================================================
//  CLASS SingleWriteLock  --
//=============================================================================

class SingleWriteLock
{
public:
    SingleWriteLock();
    ~SingleWriteLock();

	// Methods
    void    BeginRead();
    void    EndRead();
    bool    BeginWrite();
    void    EndWrite();

private:
	// Data
    AtomicAccess<int>	Atomic;
    int					mSentinel;
    Event				mReadSignal;
    Event				mWriteSignal;
    unsigned int		mWriteRecursionCount;
    ThreadCounter		tls;
    unsigned int		mWriterID;
    int					mRevisionLevel;

	// Methods
    void    BlockReaders();
    void    UnblockReaders();
    void    UnblockOneWriter();
    void    WaitForReadSignal();
    void    WaitForWriteSignal();

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}   // Innovative

#endif
