// SingletonManager_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <SingletonManager_Mb.h>
#include <list>

#if defined(WIN32) || defined(WIN64)
	#include <Windows.h>
#elif defined(LINUX)
        #include <pthread.h>
#endif

namespace OpenWire
{
static unsigned int __MainThreadId = 0;

//===========================================================================
// CLASS SingletonManager
//===========================================================================

class SingletonManager
{
    friend class SingletonService;
    friend class SingletonManagerKiller;

private:
	void Register( SingletonService *instance )
    {
        Services.push_front( instance );
    }

private:
	SingletonManager()
	{
		__MainThreadId = CurrentThreadId();
	}

    ~SingletonManager()
    {
        for( std::list<SingletonService *>::iterator Iter = Services.begin(); Iter != Services.end(); Iter++ )
        {
            delete (*Iter);
        }
    }

private:
    static SingletonManager * Instance;

private:
    std::list<SingletonService *> Services;

	unsigned int CurrentThreadId() 
	{
		#if defined(WIN32) || defined(WIN64)
			return GetCurrentThreadId();
		#elif defined(LINUX)
			return pthread_self();
		#endif

	    return 0;
	}
};

SingletonManager * SingletonManager::Instance = 0;

//===========================================================================
//  CLASS SingletonManagerKiller -- we live to kill the manager
//===========================================================================

class SingletonManagerKiller
{
public:
	SingletonManagerKiller()
	{
	if (SingletonManager::Instance==0)
		SingletonManager::Instance = new SingletonManager;
	}

	~SingletonManagerKiller()
		{
		if (SingletonManager::Instance!=0)
			{
			delete SingletonManager::Instance;
			SingletonManager::Instance = 0;
			}
		}
};

SingletonManagerKiller  MrPostal;    // will kill the manager, then self

//===========================================================================
//  CLASS SingletonService
//===========================================================================

//---------------------------------------------------------------------------
//  SingletonService::SingletonService() - Ctor
//---------------------------------------------------------------------------

SingletonService::SingletonService()
{
    if (SingletonManager::Instance==0)
        SingletonManager::Instance = new SingletonManager;

    SingletonManager::Instance->Register( this );
}

//---------------------------------------------------------------------------
//  SingletonService::~SingletonService() - Dtor
//---------------------------------------------------------------------------

SingletonService::~SingletonService()
{
}

//===========================================================================
//  FUNCTION GetMainThreadId
//===========================================================================

unsigned int GetMainThreadId()
{
	return __MainThreadId;
}

} // namespace OpenWire

