// IISioDeviceMap.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Sio_Mb.h>
#include <SioMemoryMap_Mb.h>
#include <SioDeviceMap_Mb.h>
#include <SioDeviceDriver_Mb.h>
#include <Buffer_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS SioDeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class SioDeviceMap::PciTestRegister
//---------------------------------------------------------------------------
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif
SioDeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TestModeEnable( *this, 0 ),
      GenerateRxData( *this, 1 ),
      LoopbackEnable( *this, 2 ),
      ClearTxCredit( *this, 3 ),
      ClearRxCredit( *this, 4 ),
      DrainXmitData( *this, 5 ),
      GenPacketSize( *this, 8, 4 ),
      GenPacketSizeUpdate( *this, 12 )
{
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//===========================================================================
//  CLASS SioHdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class SioHdwPluginImpl : public IPacketDevicePlugin
{
public:
    SioHdwPluginImpl( SioDeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        { /* nothing special */   }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    SioDeviceMap * PDM;
};

//===========================================================================
//  CLASS SioDeviceMap  -- Hardware mgmt for High-speed serial I/O Pmc module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SioDeviceMap
//---------------------------------------------------------------------------

using namespace IISio_MemoryMap;

PacketDeviceRegisters SioDRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	0,                    // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters PmcRegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class SioDeviceMap
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

SioDeviceMap::SioDeviceMap(Sio * /*owner*/)
    :  PacketDeviceImpl(new SioDeviceDriver, SioDRegAddrs),
       PacketDeviceMap(this, PmcRegAddrs, 0),
       IUsesVirtexFpgaLoader(PortMemory, mmFpgaData, mmFpgaControl),
       IUsesVirtexJtagLoader(PortMemory, mmJtag),
       DiqlInputControl0Reg          ( LogicMemory, mmDiqlInputControl0 ),
       SystemControlReg              ( LogicMemory, mmSystemControl ),
       DiqlOutputControl0Reg         ( LogicMemory, mmDiqlOutputControl0 ),
       TxFifoControlReg              ( LogicMemory, mmTxFifoControl ),
       DiqlOutBits0Reg               ( LogicMemory, mmDiqlOutBits0 ),
       DiqlOutBits1Reg               ( LogicMemory, mmDiqlOutBits1 ),
       RxFifoControlReg              ( LogicMemory, mmRxFifoControl ),
       RocketIoControlReg            ( LogicMemory, mmRocketIoControl ),
       TxSampleCountReg              ( LogicMemory, mmTxSampleCount ),
       PlaybackStartCountReg         ( LogicMemory, mmPlaybackStartCount ),
       RxSampleCountReg              ( LogicMemory, mmRxSampleCount ),
       CaptureStartCountReg          ( LogicMemory, mmCaptureStartCount ),
       NumberOfSamplesReg            ( LogicMemory, mmNumberOfSamples ),
       TriggerCommandReg             ( LogicMemory, mmTriggerCommand ),
       DiqlInDataBits0Reg            ( LogicMemory, mmDiqlInDataBits0 ),
       DiqlInDataBits1Reg            ( LogicMemory, mmDiqlInDataBits1 ),
       DiqlInBitError0Reg            ( LogicMemory, mmDiqlInBitError0 ),
       DiqlInBitError1Reg            ( LogicMemory, mmDiqlInBitError1 ),
       DiqlIoStatus0Reg              ( LogicMemory, mmDiqlIoStatus0 ),
       DiqlIoStatus1Reg              ( LogicMemory, mmDiqlIoStatus1 ),
       FramerCtlReg                  ( LogicMemory, mmFramerCtl ),
       ExternalFlowCtlReg            ( LogicMemory, mmExternalFlowCtl ),
       PatternGenCtlReg              ( LogicMemory, mmPatternGenCtl ),
       CaptureEventReg               ( LogicMemory, mmCaptureEvent ),
       PciTestControlReg             ( PortMemory,  mmPciTestControl),
       RecvDataFifo                  ( PortMemory,  mmRecvDataFifo),
       XmitDataFifo                  ( PortMemory,  mmXmitDataFifo),
       DDeviceDriver(dynamic_cast<SioDeviceDriver *>(DeviceDriver)),
       RcvPacketSize(0x80), XmitPacketSize(0x80)
{
    //
    //  Hook stream events from base class
    OnStreamConfigure.SetEvent(this,   &SioDeviceMap::OnStreamConfigureHandler );
    OnBeforeStreamStart.SetEvent(this, &SioDeviceMap::OnBeforeStreamStartHandler );
    OnAfterStreamStart.SetEvent(this,   &SioDeviceMap::OnAfterStreamStartHandler );
    OnBeforeStreamStop.SetEvent(this,   &SioDeviceMap::OnBeforeStreamStopHandler );
    OnAfterStreamStop.SetEvent(this,   &SioDeviceMap::OnAfterStreamStopHandler );
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new SioHdwPluginImpl(this) );
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif
//---------------------------------------------------------------------------
//  destructor for class SioDeviceMap
//---------------------------------------------------------------------------

SioDeviceMap::~SioDeviceMap()
{
}

//---------------------------------------------------------------------------
//  SioDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  SioDeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<SioDeviceDriver::PMC->Count; i++)
        vec.push_back(SioDeviceDriver::PMC->Status[i]);

}

//---------------------------------------------------------------------------
//  SioDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  SioDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnStreamConfigure.Execute(Event);

}

//---------------------------------------------------------------------------
//  SioDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  SioDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //
    //  Initialize Analog Logic before streaming. We have an option to
    //    skip this for now so that we can have
    if (Owner->FInitSecondLogic)
        {
        //  Reset FIFOs
        Owner->ResetTxFifo();
        Owner->ResetRxFifo();
        //
        Owner->RioLoopbackMode(Owner->FLoopbackMode);
        Owner->Hijack(Owner->FHijack);
        Owner->FlowCtl(Owner->FFlowCtl);
        //
        Owner->TriggerCommandReg().CaptureStreamData.Clear();
        //
        Owner->PlaybackStartCount(Owner->FPlaybackStartCount);
        Owner->CaptureStartCount(Owner->FCaptureStartCount);
        Owner->NumberOfSamples(Owner->FNumberOfSamples);
        Owner->ReturnFrameSize(Owner->FReturnFrameSize);
        // Load Tx_samples_count , and Rx_sample_count
        //      reg 0x0013   ASSERT  bits (1 downto 0) = "11",
        //      assert bits (1 downto 0) = "00"
        Owner->TriggerCommandReg().ClearSampleCount.Toggle();

        if (Owner->FCaptureStartNow)
            {
            // Assert Capture_start _now reg0x0013 bit(2) = 1;
            Owner->TriggerCommandReg().CaptureStartNow.Set();
            }
        // write to reg 0x0000    => x"00000020" => DIQL_IN_RSTN
        Owner->InputControlReg(Owner->FChannel).ResetN.Set();
        // write to reg 0x0004   => x" 00000020" => DIQL_OUT_RSTN
        Owner->OutputControlReg(Owner->FChannel).ResetN.Set();

        //  Set Pattern generation mode (false=streaming, true=pattern)
        Owner->PatternGenMode(Owner->FPatternGenMode);
        Owner->ExternalFlowCtlMode(Owner->FExternalFlowCtlMode);
        //  Assert System_GO  reg 0x0001 bit(0) = '1';
        Owner->Go(true);
        }

}

//---------------------------------------------------------------------------
//  SioDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  SioDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  SioDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  SioDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);
    //
    //  Initialize Analog Logic before streaming. We have an option to
    //    skip this for now so that we can have
    if (Owner->FInitSecondLogic)
        {
        Owner->Go(false);
        Owner->TriggerCommandReg().CaptureStartNow.Clear();
        Owner->TriggerCommandReg().CaptureStreamData.Set();
        Owner->RioLoopbackMode(Sio::lmNone);
        Owner->OutputControlReg(Owner->FChannel).ResetN.Clear();
        Owner->InputControlReg(Owner->FChannel).ResetN.Clear();

        Owner->TriggerCommandReg().ClearSampleCount.Clear();

        // VFifo
        Owner->ResetTxFifo();
        Owner->ResetRxFifo();
        }


}

//---------------------------------------------------------------------------
//  SioDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  SioDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStop.Execute(Event);
}


} // namespace Innovative


