// IISioDeviceMap.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef SioDeviceMap_MbH
#define SioDeviceMap_MbH

#include <PacketDeviceMap_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SioTrigger_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <PciLogicVersionInfo_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS SioDeviceMap  -- Hardware mgmt for High-speed serial I/O Pmc module
//===========================================================================
class Sio;

class SioDeviceMap :  public PacketDeviceImpl,
                      public PacketDeviceMap,
                      public IUsesVirtexFpgaLoader,
                      public IUsesVirtexJtagLoader,
                      public IUsesNullFpgaLogicVersionInfo,
                      public IUsesNullPciLogicVersionInfo
{
    friend class Sio;

protected:
    //
    //  Class PciTestRegister
    class PciTestRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       TestModeEnable;
        RegisterBit       GenerateRxData;
        RegisterBit       LoopbackEnable;
        RegisterBit       ClearTxCredit;
        RegisterBit       ClearRxCredit;
        RegisterBit       DrainXmitData;
        RegisterBitGroup  GenPacketSize;
        RegisterBit       GenPacketSizeUpdate;

    public:
        PciTestRegister( AddressingSpace &space, int offset );
    };
    
public:
    //
    //  Member Functions
    SioDeviceMap(Sio * owner);
    virtual ~SioDeviceMap();

    // Trigger Registers
    DiqlInputControlRegister           DiqlInputControl0Reg;
    SystemControlRegister              SystemControlReg;
    DiqlOutputControlRegister          DiqlOutputControl0Reg;
    FifoControlRegister                TxFifoControlReg;
    Register                           DiqlOutBits0Reg;
    Register                           DiqlOutBits1Reg;
    FifoControlRegister                RxFifoControlReg;
    RocketIoControlRegister            RocketIoControlReg;
    RegisterPair                       TxSampleCountReg;
    RegisterPair                       PlaybackStartCountReg;
    RegisterPair                       RxSampleCountReg;
    RegisterPair                       CaptureStartCountReg;
    RegisterPair                       NumberOfSamplesReg;
    TriggerCommandRegister             TriggerCommandReg;
    Register                           DiqlInDataBits0Reg;
    Register                           DiqlInDataBits1Reg;
    Register                           DiqlInBitError0Reg;
    Register                           DiqlInBitError1Reg;
    DiqlIoStatusRegister               DiqlIoStatus0Reg;
    DiqlIoStatusRegister               DiqlIoStatus1Reg;
    FramerControlRegister              FramerCtlReg;
    Register                           ExternalFlowCtlReg;
    PatternGeneratorControlRegister    PatternGenCtlReg;
    RegisterPair                       CaptureEventReg;
    PciTestRegister                    PciTestControlReg;

    //
    //  Make this property public
    using PacketDeviceImpl::PciLoopbackMode;
    
private:
    //
    //  Member Data
    ReadOnlyRegister                   RecvDataFifo;
    ShadowRegister                     XmitDataFifo;

    Sio *                              Owner;
    class SioDeviceDriver *            DDeviceDriver;

    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

public:

    int  RcvPacketSize;
    int  XmitPacketSize;

    void  DumpIsrStatus(std::vector<int> & vec);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
