// IIFiberChannelTrigger.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include "SioTrigger_Mb.h"

namespace Innovative
{
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

//===========================================================================
//  CLASS DiqlInputControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  DiqlInputControlRegister::DiqlInputRegister() --  Ctor
//---------------------------------------------------------------------------

DiqlInputControlRegister::DiqlInputControlRegister(AddressingSpace &space,
    int offset)
    : inherited(space, offset),
      ChecksumInDisable(*this, 1),
      ChecksumMismatchMask(*this, 2),
      ChecksumErrorResetMask(*this, 3),
      ResetN(*this, 4),
      InLedControl(*this, 5),
      DisableLedReporting(*this, 6, 2),
      LedGreen(*this, 8),
      LedRed(*this, 9),
      DisparityMask(*this, 10),
      FrameLengthMismatchMask(*this, 11),
      LossOfSyncMask(*this, 12),
      OutOfBandMask(*this, 13),
      SlowClockEnable(*this, 14, 2),
      ErrorReset(*this, 16),
      ControlData(*this, 17),
      Flow(*this, 18)
{
}

//===========================================================================
//  CLASS SystemControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  SystemControlRegister::SystemControlRegister() --  Ctor
//---------------------------------------------------------------------------

SystemControlRegister::SystemControlRegister(AddressingSpace &space,
    int offset)
    : inherited(space, offset),
      Go(*this, 0),
      FlowCtl(*this, 1)
{
}

//===========================================================================
//  CLASS FifoControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  FifoControlRegister::FifoControlRegister() --  Ctor
//---------------------------------------------------------------------------

FifoControlRegister::FifoControlRegister(AddressingSpace &space, int offset)
    : inherited(space, offset),
      Reset(*this, 0),
      Run(*this, 1)
{
}

//===========================================================================
//  CLASS DiqlOutputControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  DiqlOutputControlRegister::DiqlOutputControlRegister() --  Ctor
//---------------------------------------------------------------------------

DiqlOutputControlRegister::DiqlOutputControlRegister(AddressingSpace &space,
    int offset)
    : inherited(space, offset),
      ChecksumDisable(*this, 0),
      BitsMode(*this, 1),
      LedGreen(*this, 2),
      LedRed(*this, 3),
      SoftwareControl(*this, 4),
      ResetN(*this, 5),
      SoftwareFlowControl(*this, 6),
      SlowClockEnable(*this, 7),
      TxOutBitReg(*this, 8),
      Hijack(*this, 9)
{
}

//===========================================================================
//  CLASS RocketIoControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  RocketIoControlRegister::RocketIoControlRegister() --  Ctor
//---------------------------------------------------------------------------

RocketIoControlRegister::RocketIoControlRegister(AddressingSpace &space,
    int offset)
    : inherited(space, offset),
      Mode(*this, 0, 2)
{
}

//===========================================================================
//  CLASS RegisterPair
//===========================================================================
//---------------------------------------------------------------------------
//  RegisterPair::RegisterPair() --  Ctor
//---------------------------------------------------------------------------

RegisterPair::RegisterPair(AddressingSpace &space, int offset)
    : Low(space, offset), High(space, offset+1)
{
}

//---------------------------------------------------------------------------
//  RegisterPair::Count() --  Update paired register value
//---------------------------------------------------------------------------

void RegisterPair::Count(ii64 value)
{
    BigVal Value;
    Value.All = value;

//    volatile int x = Value.Half.Lo;
//    volatile int y = Value.Half.Hi;

    Low = Value.Half.Lo;
    High = Value.Half.Hi;
}

//---------------------------------------------------------------------------
//  RegisterPair::Count() --  Retrieve paired register value
//---------------------------------------------------------------------------

ii64 RegisterPair::Count()
{
    BigVal Value;
    Value.Half.Lo = Low;
    Value.Half.Hi = High;

    return Value.All;
}

//===========================================================================
//  CLASS TriggerCommandRegister
//===========================================================================
//---------------------------------------------------------------------------
//  TriggerCommandRegister::TriggerCommandRegister() --  Ctor
//---------------------------------------------------------------------------

TriggerCommandRegister::TriggerCommandRegister(AddressingSpace &space,
    int offset)
    : inherited(space, offset),
      LoadTxSampleRegister(*this, 0),
      LoadRxSampleRegister(*this, 1),
      CaptureStartNow(*this, 3),
      CaptureStreamData(*this, 4),
      ClearSampleCount(*this, 2),
      LoadPlaybackRegister(*this, 5),
      LoadCaptureStartCountRegister(*this, 6)
{
}

//===========================================================================
//  CLASS DiqlIoStatusRegister
//===========================================================================
//---------------------------------------------------------------------------
//  DiqlIoStatusRegister::DiqlIoStatusRegister() --  Ctor
//---------------------------------------------------------------------------

DiqlIoStatusRegister::DiqlIoStatusRegister(AddressingSpace &space, int offset)
    : inherited(space, offset),
      IsChecksumMismatch(*this, 1),
      IsControl(*this, 2),
      IsInControlInternal(*this, 3),
      IsInDataBitsData(*this, 4),
      IsInDisparity(*this, 7),
      IsInErrorFrame(*this, 8),
      IsInFrameLengthMismatch(*this, 9),
      IsInLossOfSync(*this, 10),
      IsInOutOfBand(*this, 11),
      IsOutFlowStat(*this, 12)
{
}

//===========================================================================
//  CLASS PatternGeneratorControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for PatternGeneratorControlRegister
//---------------------------------------------------------------------------

PatternGeneratorControlRegister::PatternGeneratorControlRegister(AddressingSpace &space, int offset)
    : inherited(space, offset),
      Control(*this, 0, 2)
{
}

//===========================================================================
//  CLASS FramerControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for FramerControlRegister
//---------------------------------------------------------------------------

FramerControlRegister::FramerControlRegister(AddressingSpace &space, int offset)
    : inherited(space, offset),
      ReturnFrameSize(*this, 0, 24)
{
}
#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

}  // namespace Innovative
