// IIFiberChannelTrigger.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SioTrigger_MbH
#define SioTrigger_MbH

#include <HardwareRegister_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS DiqlInputControlRegister
//===========================================================================

class DiqlInputControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    DiqlInputControlRegister(AddressingSpace &space,
        int offset);

    // Data
    RegisterBit             ChecksumInDisable;
    RegisterBit             ChecksumMismatchMask;
    RegisterBit             ChecksumErrorResetMask;
    RegisterBit             ResetN;
    RegisterBit             InLedControl;
    RegisterBitGroup        DisableLedReporting;
    RegisterBit             LedGreen;
    RegisterBit             LedRed;
    RegisterBit             DisparityMask;
    RegisterBit             FrameLengthMismatchMask;
    RegisterBit             LossOfSyncMask;
    RegisterBit             OutOfBandMask;
    RegisterBitGroup        SlowClockEnable;
    RegisterBit             ErrorReset;
    RegisterBit             ControlData;
    RegisterBit             Flow;
};

//===========================================================================
//  CLASS SystemControlRegister
//===========================================================================

class SystemControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    SystemControlRegister(AddressingSpace &space,
        int offset);

    // Data
    RegisterBit             Go;
    RegisterBit             FlowCtl;
};

//===========================================================================
//  CLASS FifoControlRegister
//===========================================================================

class FifoControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    FifoControlRegister(AddressingSpace &space,
        int offset);

    // Data
    RegisterBit             Reset;
    RegisterBit             Run;
};

//===========================================================================
//  CLASS DiqlOutputControlRegister
//===========================================================================

class DiqlOutputControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    DiqlOutputControlRegister(AddressingSpace &space,
        int offset);

    // Data
    RegisterBit             ChecksumDisable;
    RegisterBit             BitsMode;
    RegisterBit             LedGreen;
    RegisterBit             LedRed;
    RegisterBit             SoftwareControl;
    RegisterBit             ResetN;
    RegisterBit             SoftwareFlowControl;
    RegisterBit             SlowClockEnable;
    RegisterBit             TxOutBitReg;
    RegisterBit             Hijack;
};

//===========================================================================
//  CLASS RocketIoControlRegister
//===========================================================================

class RocketIoControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    RocketIoControlRegister(AddressingSpace &space,
        int offset);

    // Data
    RegisterBitGroup        Mode;
};

//===========================================================================
//  CLASS RegisterPair
//===========================================================================

class RegisterPair
{
public:
    // Ctor
    RegisterPair(AddressingSpace &space,
        int offset);

    // Properties
    void                Count(ii64 value);
    ii64                Count();

private:
    typedef struct
    {
        unsigned int        Lo;
        unsigned int        Hi;
    } QuadPair;

    typedef union
    {
        QuadPair            Half;
        ii64                All;
    } BigVal;

    Register            Low;
    Register            High;
};

//===========================================================================
//  CLASS TriggerCommandRegister
//===========================================================================

class TriggerCommandRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    TriggerCommandRegister(AddressingSpace &space,
        int offset);

    // Data
    RegisterBit             LoadTxSampleRegister;
    RegisterBit             LoadRxSampleRegister;
    RegisterBit             CaptureStartNow;
    RegisterBit             CaptureStreamData;
    RegisterBit             ClearSampleCount;
    RegisterBit             LoadPlaybackRegister;
    RegisterBit             LoadCaptureStartCountRegister;
};

//===========================================================================
//  CLASS DiqlIoStatusRegister
//===========================================================================

class DiqlIoStatusRegister : public ReadOnlyRegister
{
    typedef ReadOnlyRegister inherited;

public:
    // Ctor
    DiqlIoStatusRegister(AddressingSpace &space,
        int offset);

    // Data
    ReadOnlyRegisterBit     IsChecksumMismatch;
    ReadOnlyRegisterBit     IsControl;
    ReadOnlyRegisterBit     IsInControlInternal;
    ReadOnlyRegisterBit     IsInDataBitsData;
    ReadOnlyRegisterBit     IsInDisparity;
    ReadOnlyRegisterBit     IsInErrorFrame;
    ReadOnlyRegisterBit     IsInFrameLengthMismatch;
    ReadOnlyRegisterBit     IsInLossOfSync;
    ReadOnlyRegisterBit     IsInOutOfBand;
    ReadOnlyRegisterBit     IsOutFlowStat;
};

//===========================================================================
//  CLASS PatternGeneratorControlRegister
//===========================================================================

class PatternGeneratorControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    PatternGeneratorControlRegister(AddressingSpace &space, int offset);

    enum PGCControl { Streaming = 0, LoopMode = 2 };
    // Data
    RegisterBitGroup        Control;

    void StreamMode( PGCControl mode )
        {  Control.Value( mode );  }
};

//===========================================================================
//  CLASS FramerControlRegister
//===========================================================================

class FramerControlRegister : public Register
{
    typedef Register inherited;

public:
    // Ctor
    FramerControlRegister(AddressingSpace &space, int offset);

    // Data
    RegisterBitGroup        ReturnFrameSize;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative


#endif
