//---------------------------------------------------------------------------
// Sio.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Sio_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>
#include <BufferDatagrams_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS Sio  -- Racal Pmc Board object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Sio
//---------------------------------------------------------------------------

Sio::Sio()
    :  PmcBoard(Device), Device(this), FInitSecondLogic(false), FCaptureStartNow(true),
       FPatternGenMode(false), FExternalFlowCtlMode(false), FChannel(0), FLoopbackMode(lmNone),
       FPlaybackStartCount(0), FCaptureStartCount(0), FTxSampleCount(0),
       FRxSampleCount(0), FNumberOfSamples(0), FReturnFrameSize(0x40),
       FHijack(false), FFlowCtl(false) 
{
}

//---------------------------------------------------------------------------
//  destructor for class Sio
//---------------------------------------------------------------------------

Sio::~Sio()
{
}

//---------------------------------------------------------------------------
//  Sio::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t Sio::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  Sio::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  Sio::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Device.Reset();
}

//---------------------------------------------------------------------------
//  Sio::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  Sio::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  Sio::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  Sio::Reset()
{
    Device.Reset();
}

//---------------------------------------------------------------------------
//  Sio::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Sio::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  Sio::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Sio::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  Sio::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  Sio::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  Sio::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  Sio::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  Sio::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  Sio::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  Sio::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  Sio::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  Sio::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  Sio::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  Sio::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  Sio::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  Sio::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  Sio::Log( int type, int value )
{
    Device.Log(type, value);
}

//------------------------------------------------------------------------------
//  Sio::InputControlReg() --
//------------------------------------------------------------------------------

DiqlInputControlRegister & Sio::InputControlReg(int ch)
{
    FChannel = ch;
    return Device.DiqlInputControl0Reg;
}

//------------------------------------------------------------------------------
//  Sio::OutputControlReg() --
//------------------------------------------------------------------------------

DiqlOutputControlRegister & Sio::OutputControlReg(int ch)
{
    FChannel = ch;
    return Device.DiqlOutputControl0Reg;
}

//------------------------------------------------------------------------------
//  Sio::HoldTransmit() -- Hold transmission
//------------------------------------------------------------------------------

void  Sio::HoldTransmit(bool state)
{
    bool value = (state) ? false : true ;  // invert bit (0 = hold, 1 = go)
    OutputControlReg(FChannel).ResetN.Value(value);
}

//------------------------------------------------------------------------------
//  Sio::ResetTxFifo() --
//------------------------------------------------------------------------------

void Sio::ResetTxFifo()
{
    Device.TxFifoControlReg.Run.Clear();
    Device.TxFifoControlReg.Reset.Set();
    Device.TxFifoControlReg.Reset.Clear();
    Device.TxFifoControlReg.Run.Set();
}

//------------------------------------------------------------------------------
//  Sio::ResetRxFifo() --
//------------------------------------------------------------------------------

void Sio::ResetRxFifo()
{
    Device.RxFifoControlReg.Run.Clear();
    Device.RxFifoControlReg.Reset.Set();
    Device.RxFifoControlReg.Reset.Clear();
    Device.RxFifoControlReg.Run.Set();
}

//------------------------------------------------------------------------------
//  Sio::InDataBits() --
//------------------------------------------------------------------------------

unsigned int Sio::InDataBits(int ch)
{
    FChannel = ch;
    if (ch == 0)
        return Device.DiqlInDataBits0Reg;
    else
        return Device.DiqlInDataBits1Reg;
}

//------------------------------------------------------------------------------
//  Sio::InBitError() --
//------------------------------------------------------------------------------

unsigned int Sio::InBitError(int ch)
{
    FChannel = ch;
    if (ch == 0)
        return Device.DiqlInBitError0Reg;
    else
        return Device.DiqlInBitError1Reg;
}

//------------------------------------------------------------------------------
//  Sio::IoStatusReg() --
//------------------------------------------------------------------------------

DiqlIoStatusRegister & Sio::IoStatusReg(int ch)
{
    FChannel = ch;
    if (ch == 0)
        return Device.DiqlIoStatus0Reg;
    else
        return Device.DiqlIoStatus1Reg;
}

//------------------------------------------------------------------------------
//  Sio::RioLoopbackMode() --
//------------------------------------------------------------------------------

void Sio::RioLoopbackMode(LoopbackMode mode)
{
    FLoopbackMode = mode;  // cache setting...
    Device.RocketIoControlReg.Mode.Value(static_cast<ii32>(mode));
}

//------------------------------------------------------------------------------
//  Sio::RioLoopbackMode() --
//------------------------------------------------------------------------------

Sio::LoopbackMode Sio::RioLoopbackMode() const
{
    return static_cast<Sio::LoopbackMode>(Device.RocketIoControlReg.Mode.Value());
}

//------------------------------------------------------------------------------
//  Sio::PciLoopbackMode() --
//------------------------------------------------------------------------------

void Sio::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:
            mode_value = 0;
            break;
        case lmSerial:
            mode_value = 1;
            break;
        case lmParallel:
            mode_value = 2;
            break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  Sio::PciLoopbackMode() --
//------------------------------------------------------------------------------

Sio::LoopbackMode Sio::PciLoopbackMode()
{
    // This field does not use the same bit pattern, so improvise...
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:
            return lmNone;
        case 1:
            return lmSerial;
        case 2:
            return lmParallel;
        }
}

//------------------------------------------------------------------------------
//  Sio::TriggerCommandReg() --
//------------------------------------------------------------------------------

TriggerCommandRegister & Sio::TriggerCommandReg()
{
    return Device.TriggerCommandReg;
}

//------------------------------------------------------------------------------
//  Sio::Go() --
//------------------------------------------------------------------------------

void Sio::Go(bool state)
{
    if (state)
        Device.SystemControlReg.Go.Set();
    else
        Device.SystemControlReg.Go.Clear();
}

//------------------------------------------------------------------------------
//  Sio::Go() --
//------------------------------------------------------------------------------

bool Sio::Go()
{
    return Device.SystemControlReg.Go ? true : false;
}

//------------------------------------------------------------------------------
//  Sio::FlowCtl() --
//------------------------------------------------------------------------------

bool Sio::FlowCtl()
{
    return Device.SystemControlReg.FlowCtl ? true : false;
}

//------------------------------------------------------------------------------
//  Sio::FlowCtl() --
//------------------------------------------------------------------------------

void Sio::FlowCtl(bool state)
{
    FFlowCtl = state;
    if (state)
        Device.SystemControlReg.FlowCtl.Set();
    else
        Device.SystemControlReg.FlowCtl.Clear();
}

//------------------------------------------------------------------------------
//  Sio::Hijack() --
//------------------------------------------------------------------------------

bool Sio::Hijack()
{
    return Device.DiqlOutputControl0Reg.Hijack ? true : false;
}

//------------------------------------------------------------------------------
//  Sio::Hijack() --
//------------------------------------------------------------------------------

void Sio::Hijack(bool state)
{
    FHijack = state;
    if (state)
        Device.DiqlOutputControl0Reg.Hijack.Set();
    else
        Device.DiqlOutputControl0Reg.Hijack.Clear();
}

//------------------------------------------------------------------------------
//  Sio::OutBits() --
//------------------------------------------------------------------------------

void Sio::OutBits(int ch, unsigned int count)
{
    if (ch == 0)
        Device.DiqlOutBits0Reg = count;
    else
        Device.DiqlOutBits1Reg = count;
}

//------------------------------------------------------------------------------
//  Sio::OutBits() --
//------------------------------------------------------------------------------

unsigned int Sio::OutBits(int ch)
{
    if (ch == 0)
        return Device.DiqlOutBits0Reg;
    else
        return Device.DiqlOutBits1Reg;
}

//------------------------------------------------------------------------------
//  Sio::TxSampleCount() --
//------------------------------------------------------------------------------

void Sio::TxSampleCount(ii64 value)
{
    FTxSampleCount = value;
    Device.TxSampleCountReg.Count(value);
}

//---------------------------------------------------------------------------
//  Sio::TxSampleCount() --
//---------------------------------------------------------------------------

ii64 Sio::TxSampleCount()
{
    return Device.TxSampleCountReg.Count();
}

//---------------------------------------------------------------------------
//  Sio::PlaybackStartCount() --
//---------------------------------------------------------------------------

void Sio::PlaybackStartCount(ii64 value)
{
    FPlaybackStartCount = value;
    Device.PlaybackStartCountReg.Count(value);
    TriggerCommandReg().LoadPlaybackRegister.Toggle();
}

//---------------------------------------------------------------------------
//  Sio::PlaybackStartCount() --
//---------------------------------------------------------------------------

ii64 Sio::PlaybackStartCount()
{
    return Device.PlaybackStartCountReg.Count();
}

//---------------------------------------------------------------------------
//  Sio::RxSampleCount() --
//---------------------------------------------------------------------------

void Sio::RxSampleCount(ii64 value)
{
    FRxSampleCount = value;
    Device.RxSampleCountReg.Count(value);
}

//---------------------------------------------------------------------------
//  Sio::RxSampleCount() --
//---------------------------------------------------------------------------

ii64 Sio::RxSampleCount()
{
    return Device.RxSampleCountReg.Count();
}

//---------------------------------------------------------------------------
//  Sio::CaptureStartCount() --
//---------------------------------------------------------------------------

void Sio::CaptureStartCount(ii64 value)
{
    FCaptureStartCount = value;
    Device.CaptureStartCountReg.Count(value);
    TriggerCommandReg().LoadCaptureStartCountRegister.Toggle();
}

//---------------------------------------------------------------------------
//  Sio::CaptureStartCount() --
//---------------------------------------------------------------------------

ii64 Sio::CaptureStartCount()
{
    return Device.CaptureStartCountReg.Count();
}

//---------------------------------------------------------------------------
//  Sio::NumberOfSamples() --
//---------------------------------------------------------------------------

void Sio::NumberOfSamples(ii64 value)
{
    FNumberOfSamples = value;
    Device.NumberOfSamplesReg.Count(value);
}

//---------------------------------------------------------------------------
//  Sio::NumberOfSamples() --
//---------------------------------------------------------------------------

ii64 Sio::NumberOfSamples()
{
    return Device.NumberOfSamplesReg.Count();
}

//---------------------------------------------------------------------------
//  Sio::CaptureEvent() --
//---------------------------------------------------------------------------

ii64 Sio::CaptureEvent()
{
    return Device.CaptureEventReg.Count();
}

//------------------------------------------------------------------------------
//  Sio::PatternGenMode() --
//------------------------------------------------------------------------------

void Sio::PatternGenMode(bool state)
{
    FPatternGenMode = state;
    PatternGeneratorControlRegister::PGCControl value =
        (state==true) ? PatternGeneratorControlRegister::LoopMode
                      : PatternGeneratorControlRegister::Streaming;
    Device.PatternGenCtlReg.StreamMode(value);
}

//------------------------------------------------------------------------------
//  Sio::PatternGenMode() --
//------------------------------------------------------------------------------

bool  Sio::PatternGenMode()
{
    // value of 2 is LoopMode
    return Device.PatternGenCtlReg.Control.Value()
                == PatternGeneratorControlRegister::LoopMode;
}

//------------------------------------------------------------------------------
//  Sio::ExternalFlowCtlMode() --
//------------------------------------------------------------------------------

void Sio::ExternalFlowCtlMode(int state)
{
	FExternalFlowCtlMode = state ? true : false;
    Device.ExternalFlowCtlReg.Value(state);
}

//------------------------------------------------------------------------------
//  Sio::ExternalFlowCtlMode() --
//------------------------------------------------------------------------------

int  Sio::ExternalFlowCtlMode()
{
    return Device.ExternalFlowCtlReg.Value();
}

//------------------------------------------------------------------------------
//  Sio::SendPatternInfo() --
//------------------------------------------------------------------------------

void  Sio::SendPatternInfo(int start_addr, int buf_size_ints, int count)
{
    Buffer Packet(4);
    PacketBufferHeader Hdr(Packet);
    IntegerDG  PacketDG(Packet);

    Hdr.PeripheralId(128);
    Hdr.PacketSize(Packet.FullSizeInInts());
    Hdr[1] = 0;

    PacketDG[0] = start_addr;
    PacketDG[1] = buf_size_ints/256;
    PacketDG[2] = count;
    PacketDG[3] = 0;
    //
    //  Post to stream
    if (StreamManager.Linked())
        StreamManager->Send(Packet);
}
    
//------------------------------------------------------------------------------
//  Sio::ReturnFrameSize() --
//------------------------------------------------------------------------------

void  Sio::ReturnFrameSize(int size_ints)
{
    FReturnFrameSize = size_ints;
    Device.FramerCtlReg.ReturnFrameSize.Value(size_ints);
}

//------------------------------------------------------------------------------
//  Sio::LoadTxSampleCount() --
//------------------------------------------------------------------------------

void  Sio::LoadTxSampleCount(ii64 value)
{
    TxSampleCount(value);
    TriggerCommandReg().LoadTxSampleRegister.Toggle();
}

//------------------------------------------------------------------------------
//  Sio::LoadRxSampleCount() --
//------------------------------------------------------------------------------

void  Sio::LoadRxSampleCount(ii64 value)
{
    RxSampleCount(value);
    TriggerCommandReg().LoadRxSampleRegister.Toggle();
}

//---------------------------------------------------------------------------
//  Sio::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & Sio::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sio::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & Sio::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sio::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & Sio::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sio::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & Sio::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Sio::BusMasterSize() --
//---------------------------------------------------------------------------

void Sio::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Sio::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void Sio::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Sio::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void Sio::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Sio::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  Sio::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  Sio::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  Sio::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

}  // namespace Innovative
