// Sio.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Sio_MbH
#define Sio_MbH

#include <PmcBoard_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <SioDeviceMap_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS Sio  -- Racal Pmc Board object
//===========================================================================

class Sio : public PmcBoard,
            public IStreamDeviceEvents
{
    friend class SioDeviceMap;
    
public:
    //
    //  Member Functions
    Sio();
    virtual ~Sio();

    size_t          BoardCount();
    void            Open();
    void            Close();

    void  SendPatternInfo(int start_addr, int buf_size_ints, int count);
    void  ReturnFrameSize(int size_ints);
    void  HoldTransmit(bool state);
    
    int   Target();
    void  Target( int value );

    bool  Led();
    void  Led( bool value );

    ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );
    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );
    //
    //  IPacketStreamDevice interface implementation
    virtual void  Reset();

    SioDeviceMap * Debug() { return &Device; };
    virtual void   Log(int type, int data);

    // Logic programming
    IUsesVirtexFpgaLoader & Logic();
    // EEprom programming
    IUsesVirtexJtagLoader & Eeprom();

    // Revision
    FpgaLogicVersionInfoIntf & UserLogicVersion();
    PciLogicVersionInfoIntf & PciLogicVersion();

    // Trigger Support
    // Enumerations
    enum LoopbackMode { lmNone, lmParallel, lmSerial };

    // Registers
    DiqlInputControlRegister &         InputControlReg(int ch);
    DiqlOutputControlRegister &        OutputControlReg(int ch);
    DiqlIoStatusRegister &             IoStatusReg(int ch);

    TriggerCommandRegister &           TriggerCommandReg();

    // Methods
    unsigned int                       InBitError(int ch);
    unsigned int                       InDataBits(int ch);
    void                               ResetTxFifo();
    void                               ResetRxFifo();

    // Properties
    void                               Go(bool state);
    bool                               Go();
    void                               FlowCtl(bool state);
    bool                               FlowCtl();
    void                               Hijack(bool state);
    bool                               Hijack();

    void                               RioLoopbackMode(LoopbackMode mode);
    LoopbackMode                       RioLoopbackMode() const;

    void                               PciLoopbackMode(LoopbackMode mode);
    LoopbackMode                       PciLoopbackMode();

    void                               PatternGenMode(bool state);
    bool                               PatternGenMode();
    void                               ExternalFlowCtlMode(int state);
    int                                ExternalFlowCtlMode();

    void                               OutBits(int ch, unsigned int count);
    unsigned int                       OutBits(int ch);

    void                               LoadTxSampleCount(ii64 value);
    void                               TxSampleCount(ii64 value);
    ii64                               TxSampleCount();

    void                               PlaybackStartCount(ii64 value);
    ii64                               PlaybackStartCount();

    void                               LoadRxSampleCount(ii64 value);
    void                               RxSampleCount(ii64 value);
    ii64                               RxSampleCount();

    void                               CaptureStartCount(ii64 value);
    ii64                               CaptureStartCount();

    void                               NumberOfSamples(ii64 value);
    ii64                               NumberOfSamples();

    ii64                               CaptureEvent();

	void 							   BusMasterSize(int bytes);
	void 							   IncomingBusMasterSize(int bytes);
	void 							   OutgoingBusMasterSize(int bytes);
	int             				   IncomingBusMasterSize() const;
	int             				   OutgoingBusMasterSize() const;

	void        InitSecondLogic(bool value)
                    {  FInitSecondLogic = value;  };
    void        CaptureStartNow(bool value)
                    {  FCaptureStartNow = value;  };

private:
    //
    //  Member Data
    SioDeviceMap            Device;
    bool                    FInitSecondLogic;
    bool                    FCaptureStartNow;
    bool                    FPatternGenMode;
    bool                    FExternalFlowCtlMode;
    //
    //  Cached Register Values
    int             FChannel;
    LoopbackMode    FLoopbackMode;
    ii64            FPlaybackStartCount;
    ii64            FCaptureStartCount;
    ii64            FTxSampleCount;
    ii64            FRxSampleCount;
    ii64            FNumberOfSamples;
    ii32            FReturnFrameSize;
    bool            FHijack;
    bool            FFlowCtl;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

