// SoftwareTimer_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <SoftwareTimer_Mb.h>

namespace   Innovative
{

//========================================================================
//  CLASS SoftwareTimer --  Simple software timer class
//========================================================================

//------------------------------------------------------------------------
//  Constructor
//------------------------------------------------------------------------

SoftwareTimer::SoftwareTimer(bool IgnoreFirstEvent)
	: inherited("SoftwareTimer"), FInterval(1000), FEnabled(false),
	  IgnoreFirstEventFlag(IgnoreFirstEvent)
{
	Resume();
}

//------------------------------------------------------------------------
//  Destructor
//------------------------------------------------------------------------

SoftwareTimer::~SoftwareTimer()
{
	Shutdown();
}

//---------------------------------------------------------------------------
// SoftwareTimer::Interval() --  Set period
//---------------------------------------------------------------------------

void SoftwareTimer::Interval(int msecs)
{
    MutexedLock Lock(FLock);
    FInterval=msecs;
}

//---------------------------------------------------------------------------
// SoftwareTimer::Interval() --  Query period
//---------------------------------------------------------------------------

int  SoftwareTimer::Interval() const
{
    MutexedLock Lock(FLock);
    return FInterval;
}

//---------------------------------------------------------------------------
// SoftwareTimer::Enable() --  Change enable state
//---------------------------------------------------------------------------

void SoftwareTimer::Enabled(bool state)
{
    MutexedLock Lock(FLock);
    FEnabled = state;
}

//---------------------------------------------------------------------------
// SoftwareTimer::Enabled() --  Enabled state query
//---------------------------------------------------------------------------

bool SoftwareTimer::Enabled() const
{
    MutexedLock Lock(FLock);
	return FEnabled;
}

//---------------------------------------------------------------------------
// SoftwareTimer::Execute() --
//---------------------------------------------------------------------------

void SoftwareTimer::Execute()
{
		while (!Terminated()) {
			int cause = WaitFor(FEnabled ? FInterval : 1);
			if ((cause == MultipleObjects::Timeout) && Enabled()) {
				if (IgnoreFirstEventFlag)
					IgnoreFirstEventFlag = false; // ignore the first call only
				else {
					OpenWire::NotifyEvent note;
					OnElapsed.Execute(note);
				}
			}

		}
}



}  // namespace Innovative
