// SpiDevice_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <SpiDevice_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS ISpiDevice -- Interface to SPI controlled device
//===========================================================================
//---------------------------------------------------------------------------
//  ISpiDevice::Spi() --  SPI Read
//---------------------------------------------------------------------------

unsigned char ISpiDevice::Spi(int addr)
{
    SpiReady();

    // Set up a SPI read
    SetupSpiRead(addr);

    SpiNotReady();
    SpiReady();

    unsigned char result = ReadSpiData();

    return result;
}

//---------------------------------------------------------------------------
//  ISpiDevice::Spi()  --  SPI Write
//---------------------------------------------------------------------------

void ISpiDevice::Spi(int addr, unsigned char value)
{
    SpiReady();

    // Write to D/A register via SPI register
    SetupSpiWrite(addr, value);

    SpiNotReady();
    SpiReady();

    std::stringstream msg;
    msg << std::hex << Tag << " Spi(0x" << addr <<
             ", 0x" << (short)value << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  ISpiDevice::SpiReady()
//---------------------------------------------------------------------------

void ISpiDevice::SpiReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << Tag << " Spi Ready Failure (>5us)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  ISpiDevice::SpiNotReady()
//---------------------------------------------------------------------------

void ISpiDevice::SpiNotReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    //std::stringstream msg;
    //msg << std::hex << Tag << " Spi NOT Ready Failure (>5us)";
    //Debug.Log(msg.str());
}

//===========================================================================
//  CLASS ISpiDevice16 -- Interface to SPI controlled device
//===========================================================================
//---------------------------------------------------------------------------
//  ISpiDevice16::Spi() --  SPI Read
//---------------------------------------------------------------------------

unsigned short ISpiDevice16::Spi(int addr)
{
    SpiReady();

    // Set up a SPI read
    SetupSpiRead(addr);

    SpiNotReady();
    SpiReady();

    unsigned short result = ReadSpiData();

    return result;
}

//---------------------------------------------------------------------------
//  ISpiDevice16::Spi()  --  SPI Write
//---------------------------------------------------------------------------

void ISpiDevice16::Spi(int addr, unsigned short value)
{
    SpiReady();

    // Write to D/A register via SPI register
    SetupSpiWrite(addr, value);

    SpiNotReady();
    SpiReady();

    std::stringstream msg;
    msg << std::hex << Tag << " Spi(0x" << addr <<
             ", 0x" << (short)value << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  ISpiDevice16::SpiReady()
//---------------------------------------------------------------------------

void ISpiDevice16::SpiReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << Tag << " Spi Ready Failure (>5us)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  ISpiDevice16::SpiNotReady()
//---------------------------------------------------------------------------

void ISpiDevice16::SpiNotReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    //std::stringstream msg;
    //msg << std::hex << Tag << " Spi NOT Ready Failure (>5us)";
    //Debug.Log(msg.str());
}

//===========================================================================
//  CLASS IMuxedSpiDevice -- Interface to SPI controlled device
//===========================================================================
//---------------------------------------------------------------------------
//  IMuxedSpiDevice::Spi() --  SPI Read
//---------------------------------------------------------------------------

unsigned short IMuxedSpiDevice::Spi(int device, int addr)
{
    // Set up a SPI device
    SetupSpiDevice(device);

    SpiReady();

    // Set up a SPI read
    SetupSpiRead(addr);

    SpiNotReady();
    SpiReady();

    unsigned short result = ReadSpiData();

    return result;
}

//---------------------------------------------------------------------------
//  IMuxedSpiDevice::Spi()  --  SPI Write
//---------------------------------------------------------------------------

void IMuxedSpiDevice::Spi(int device, int addr, unsigned short value)
{
    // Set up a SPI device
    SetupSpiDevice(device);

    SpiReady();

    // Write to D/A register via SPI register
    SetupSpiWrite(addr, value);

    SpiNotReady();
    SpiReady();

    std::stringstream msg;
    msg << std::hex << Tag << " Spi(0x" << addr <<
             ", 0x" << (short)value << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  IMuxedSpiDevice::SpiWriteAll()  --  SPI Broadside Write
//---------------------------------------------------------------------------

void IMuxedSpiDevice::SpiWriteAll(int addr, unsigned short value)
{
    // Set up a SPI device
    SetupSpiAllDevice();

    SpiReady();

    // Write to D/A register via SPI register
    SetupSpiWrite(addr, value);

    SpiNotReady();
    SpiReady();

    std::stringstream msg;
    msg << std::hex << Tag << " SpiAll(0x" << addr <<
             ", 0x" << (short)value << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  IMuxedSpiDevice::SpiReady()
//---------------------------------------------------------------------------

void IMuxedSpiDevice::SpiReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << Tag << " Spi Ready Failure (>5us)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  IMuxedSpiDevice::SpiNotReady()
//---------------------------------------------------------------------------

void IMuxedSpiDevice::SpiNotReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    //std::stringstream msg;
    //msg << std::hex << Tag << " Spi NOT Ready Failure (>5us)";
    //Debug.Log(msg.str());
}


//===========================================================================
//  CLASS ILTC2494SpiDevice -- Interface to SPI controlled device
//===========================================================================
//---------------------------------------------------------------------------
//  ILTC2494SpiDevice::Spi() --  SPI Read
//---------------------------------------------------------------------------

unsigned int ILTC2494SpiDevice::Spi(unsigned int addr)
{
    Innovative::Trace debug;

    //   The ADC returns the result of the
    //     previous conversion with a spi command.
    //     Therfore, 2 spi commands are needed to read
    //     a certain address; the first to load the address
    //     and initiate a conversion; and the second to read
    //     the result of the conversion.
    //       //read 1b read 1c

    std::stringstream ss;
    debug.Log("************LTC2494 Spi Operation************");

    ss << "-- Initial Spi Read register " << std::hex << ReadReg()
    << " Spi Address register " << std::hex << AddrReg();
    debug.Log(ss.str());

    //  Load the spi read address
    SpiReady();
    SetupSpiRead(addr);
    SpiNotReady();
    SpiReady();

    ss.str("");
    ss << "-- After loading the address Spi Read register " << std::hex << ReadReg()
       << " Spi Address register " << std::hex << AddrReg();
    debug.Log(ss.str());

    //  Wait for the conversion to end (Ready Check does this)
    //Sleep(165);

    //  Read the conversion result
    SpiReady();
    SetupSpiRead(addr);
    SpiNotReady();
    SpiReady();

    ss.str("");
    ss << "-- Final Spi Read register " << std::hex << ReadReg()
       << " Spi Address register " << std::hex << AddrReg();
    debug.Log(ss.str());

    unsigned int result = ReadSpiData();

    return result;
}

//---------------------------------------------------------------------------
//  ILTC2494SpiDevice::SpiReady()
//---------------------------------------------------------------------------

void ILTC2494SpiDevice::SpiReady()
{
    //  This device can take up to 165 ms to do a conversion. Wait at least
    //    that long for ready, see what happens.
    for (unsigned int i = 0; i < 20; ++i)  // total of 200 ms
        {
        if (ReadSpiReadyBit())
            return;
        else
            Innovative::Sleep(10);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << Tag << " Spi Ready Failure (>200ms)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  ILTC2494SpiDevice::SpiNotReady()
//---------------------------------------------------------------------------

void ILTC2494SpiDevice::SpiNotReady()
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!ReadSpiReadyBit())
            return;
        else
            Innovative::uSleep(1);
        }

}

//===========================================================================
//  CLASS StdSpiDevice -- Implementation of 'standard' SPI interface
//===========================================================================
//---------------------------------------------------------------------------
//  StdSpiDevice::SetupSpiRead() --  SPI Read Setup
//---------------------------------------------------------------------------

void  StdSpiDevice::SetupSpiRead(int addr)
{
    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Direction.Value(spiRead);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  StdSpiDevice::SetupSpiWrite() --  SPI Write Setup
//---------------------------------------------------------------------------

void  StdSpiDevice::SetupSpiWrite(int addr, unsigned char value)
{
    // Set up a SPI write
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Data.Value(value);
    SpiWriteReg.Direction.Value(spiWrite);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  StdSpiDevice::ReadSpiData() --  Read SPI data
//---------------------------------------------------------------------------

unsigned char  StdSpiDevice::ReadSpiData()
{
    return static_cast<unsigned char>(SpiReadReg.ReadData.Value());
}

//---------------------------------------------------------------------------
//  StdSpiDevice::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  StdSpiDevice::ReadSpiReadyBit()
{
    return SpiReadReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  StdSpiDevice::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  StdSpiDevice::DumpSpiRegister()
{
    return SpiReadReg.Value();
}

//===========================================================================
//  CLASS X6SpiDevice -- Implementation of 'standard' X6 SPI interface
//===========================================================================
//---------------------------------------------------------------------------
//  X6SpiDevice::SetupSpiRead() --  SPI Read Setup
//---------------------------------------------------------------------------

void  X6SpiDevice::SetupSpiRead(int addr)
{
    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Direction.Value(spiRead);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  X6SpiDevice::SetupSpiWrite() --  SPI Write Setup
//---------------------------------------------------------------------------

void  X6SpiDevice::SetupSpiWrite(int addr, unsigned short value)
{
    // Set up a SPI write
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Data.Value(value);
    SpiWriteReg.Direction.Value(spiWrite);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  X6SpiDevice::ReadSpiData() --  Read SPI data
//---------------------------------------------------------------------------

unsigned short  X6SpiDevice::ReadSpiData()
{
    return static_cast<unsigned short>(SpiReadReg.ReadData.Value());
}

//---------------------------------------------------------------------------
//  X6SpiDevice::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  X6SpiDevice::ReadSpiReadyBit()
{
    return SpiReadReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  X6SpiDevice::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  X6SpiDevice::DumpSpiRegister()
{
    return SpiWriteReg.Value();
}

//===========================================================================
//  CLASS FmcSpiDevice -- Implementation of 'standard' X6 SPI interface
//===========================================================================
//---------------------------------------------------------------------------
//  FmcSpiDevice::SetupSpiDevice() --  SPI Device Enable Setup
//---------------------------------------------------------------------------

void  FmcSpiDevice::SetupSpiDevice(int device)
{
    // enable SPI devices, one at a time
    unsigned int data = 1<<device;
    SpiDeviceReg.Value(data);
}

//---------------------------------------------------------------------------
//  FmcSpiDevice::SetupSpiAllDevice() --  SPI Device Enable Setup - broadside
//---------------------------------------------------------------------------

void  FmcSpiDevice::SetupSpiAllDevice()
{
    // enable all SPI devices
    SpiDeviceReg.Value(WriteAllMask);
}

//---------------------------------------------------------------------------
//  FmcSpiDevice::SetupSpiRead() --  SPI Read Setup
//---------------------------------------------------------------------------

void  FmcSpiDevice::SetupSpiRead(int addr)
{
    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Direction.Value(spiRead);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  FmcSpiDevice::SetupSpiWrite() --  SPI Write Setup
//---------------------------------------------------------------------------

void  FmcSpiDevice::SetupSpiWrite(int addr, unsigned short value)
{
    // Set up a SPI write
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Data.Value(value);
    SpiWriteReg.Direction.Value(spiWrite);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  FmcSpiDevice::ReadSpiData() --  Read SPI data
//---------------------------------------------------------------------------

unsigned short  FmcSpiDevice::ReadSpiData()
{
    return static_cast<unsigned short>(SpiReadReg.ReadData.Value());
}

//---------------------------------------------------------------------------
//  FmcSpiDevice::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  FmcSpiDevice::ReadSpiReadyBit()
{
    return SpiReadReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  FmcSpiDevice::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  FmcSpiDevice::DumpSpiRegister()
{
    return SpiWriteReg.Value();
}

//===========================================================================
//  CLASS FmcCpldSpiDevice -- Implementation of 'standard' SPI interface
//===========================================================================
//---------------------------------------------------------------------------
//  FmcCpldSpiDevice::SetupSpiRead() --  SPI Read Setup
//---------------------------------------------------------------------------

void  FmcCpldSpiDevice::SetupSpiRead(int addr)
{
    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Direction.Value(spiRead);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  FmcCpldSpiDevice::SetupSpiWrite() --  SPI Write Setup
//---------------------------------------------------------------------------

void  FmcCpldSpiDevice::SetupSpiWrite(int addr, unsigned char value)
{
    // Set up a SPI write
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Data.Value(value);
    SpiWriteReg.Direction.Value(spiWrite);
    SpiWriteReg.Apply();
}

//---------------------------------------------------------------------------
//  FmcCpldSpiDevice::ReadSpiData() --  Read SPI data
//---------------------------------------------------------------------------

unsigned char  FmcCpldSpiDevice::ReadSpiData()
{
    return static_cast<unsigned char>(SpiReadReg.ReadData.Value());
}

//---------------------------------------------------------------------------
//  FmcCpldSpiDevice::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  FmcCpldSpiDevice::ReadSpiReadyBit()
{
    return SpiStatusReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  FmcCpldSpiDevice::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  FmcCpldSpiDevice::DumpSpiRegister()
{
    return SpiWriteReg.Value();
}

//===========================================================================
//  CLASS LTC2494SpiDevice -- Implementation of 'LTC2494' SPI interface
//===========================================================================
//---------------------------------------------------------------------------
//  LTC2494SpiDevice::SetupSpiRead() --  SPI Read Setup
//---------------------------------------------------------------------------

void  LTC2494SpiDevice::SetupSpiRead(int addr)
{
    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
}

//---------------------------------------------------------------------------
//  LTC2494SpiDevice::ReadSpiData() --  Read SPI data
//---------------------------------------------------------------------------

unsigned int  LTC2494SpiDevice::ReadSpiData()
{
    return static_cast<unsigned int>(SpiReadReg.ReadData.Value());
}

//---------------------------------------------------------------------------
//  LTC2494SpiDevice::ReadSpiReadyBit() --  Read SPI ready bit
//---------------------------------------------------------------------------

bool  LTC2494SpiDevice::ReadSpiReadyBit()
{
    return SpiReadReg.Ready.Value();
}

//---------------------------------------------------------------------------
//  LTC2494SpiDevice::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  LTC2494SpiDevice::DumpSpiRegister()
{
    return SpiReadReg.Value();
}

} // namespace Innovative

