// Splitter_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Splitter_Mb.h>
#include <list>
#include <ipp.h>
#include <DataTypes_Mb.h>
#include <Buffer_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <MemoryUtils_Mb.h>
#include <algorithm>

namespace   Innovative
{

//===========================================================================
//  CLASS SplitterInstruction  --
//===========================================================================

class SplitterInstruction
{

public:
    // Ctor
    virtual ~SplitterInstruction() {}

    // Overrides
    virtual void    ProcessFirst(const ii32 * Input, ii32 * Output,
                                 int cannel_step, int Size, IppiSize RoiSize) = 0;
    virtual void    Process(ii32* Data, int Size) = 0;

    bool Instruction() const
        { return FInstruction; }
    void Instruction(bool value)
        { FInstruction = value; }

    bool First() const
        { return FFirstInstruction; }
    void First(bool value)
        { FFirstInstruction = value; }

protected:
    // Data
    bool FInstruction;
    bool FFirstInstruction;
};

//===========================================================================
//  CLASS SplitterAssemblyInstruction  --
//===========================================================================

class SplitterAssemblyInstruction : public SplitterInstruction
{

public:
    // Ctor
    SplitterAssemblyInstruction()
    {
        Instruction(true);
        First(false);
    }
    virtual ~SplitterAssemblyInstruction() {}

    // Overrides
    virtual void    ProcessFirst(const ii32 * Input, ii32 * Output,
                                 int cannel_step, int Size, IppiSize RoiSize) = 0;
    virtual void    Process(ii32* Data, int Size) = 0;

};

//===========================================================================
//  CLASS SplitterMacroInstruction  --
//===========================================================================

class SplitterMacroInstruction : public SplitterInstruction
{

public:
    // Ctor
    SplitterMacroInstruction()
    {
        Instruction(false);
        First(true);
    }
    virtual ~SplitterMacroInstruction() {}

    // Overrides
    virtual void    ProcessFirst(const ii32 * Input, ii32 * Output,
                                 int cannel_step, int Size, IppiSize RoiSize) = 0;
    virtual void    Process(ii32* Data, int Size) = 0;

};

//===========================================================================
//  CLASS ExtractSplitterMacroInstruction  --
//===========================================================================

class ExtractSplitterMacroInstruction : public SplitterMacroInstruction
{
public:
    // Ctor
    ExtractSplitterMacroInstruction(ii32 mask, int shift, int input_start,
                    int input_offset, int output_start, int output_offset)
        : mMask(mask), mShift(shift), mInputOffset(input_offset), mOutputOffset(output_offset),
          mInputStartLocation(input_start), mOutputStartLocation(output_start)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int /*channel_step*/,
                              int Size, IppiSize RoiSize)
    {
        int TempSize = (RoiSize.height > Size)?RoiSize.height:Size;
        Ipp32s* Temp = new Ipp32s[TempSize];

        RoiSize.height -= mInputStartLocation;

        if (RoiSize.height <= 0)
            return;

        if (mShift > 0)
            {
            ippiLShiftC_32s_C1R((const Ipp32s*)(Input + mInputStartLocation),
                                mInputOffset * sizeof(ii32),
                                mShift,
                                (Ipp32s*)Temp,
                                sizeof(ii32),
                                RoiSize);
            }
        else
            {
            ippiRShiftC_32s_C1R((const Ipp32s*)(Input + mInputStartLocation),
                                mInputOffset * sizeof(ii32),
                                -mShift,
                                (Ipp32s*)Temp,
                                sizeof(ii32),
                                RoiSize);
            }

        RoiSize.height = Size - mOutputStartLocation;

        int j = 0;
        for (int i = 0; i < RoiSize.height; i+= mOutputOffset)
            Output[mOutputStartLocation + i] = (ii32)(Temp[j++] & mMask);

        delete Temp;
    }

    virtual void Process(ii32* /*Data*/, int /*Size*/) { }

protected:
    const ii32      mMask;
    const int       mShift;
    const int       mInputOffset;
    const int       mOutputOffset;
    const int       mInputStartLocation;
    const int       mOutputStartLocation;
};

//===========================================================================
//  CLASS UnWrapSplitterMacroInstruction  --
//===========================================================================

class UnWrapSplitterMacroInstruction : public SplitterMacroInstruction
{
public:
    // Ctor
    UnWrapSplitterMacroInstruction(ii32 mask, int shift, int input_start,
                        int input_offset, int output_start, int output_offset)
       : mMask(mask), mShift(shift), mInputOffset(input_offset), mOutputOffset(output_offset),
         mInputStartLocation(input_start), mOutputStartLocation(output_start)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int /*channel_step*/,
                              int Size, IppiSize RoiSize)
    {
        int TempSize = (RoiSize.height > Size)?RoiSize.height:Size;
        Ipp32s* Temp = new Ipp32s[TempSize];

        RoiSize.height -= (mInputStartLocation + 1);

        if (RoiSize.height <= 0)
            return;

        if (mShift > 0)
            {
            ippiLShiftC_32s_C1R((const Ipp32s*)(Input + mInputStartLocation + 1),
                                mInputOffset * sizeof(ii32),
                                mShift,
                                (Ipp32s*)Temp,
                                sizeof(ii32),
                                RoiSize);

            }
        else
            {
            ippiRShiftC_32s_C1R((const Ipp32s*)(Input + mInputStartLocation + 1),
                                mInputOffset * sizeof(ii32),
                                -mShift,
                                (Ipp32s*)Temp,
                                sizeof(ii32),
                                RoiSize);
            }

        ippsAndC_32u_I(mMask, (Ipp32u*)Temp, Size);

        RoiSize.height = Size - mOutputStartLocation;

        int j = 0;
        for (int i = 0; i < RoiSize.height; i+= mOutputOffset)
            Output[mOutputStartLocation + i] |= (ii32)(Temp[j++]);

        delete Temp;
    }

    virtual void Process(ii32* /*Data*/, int /*Size*/) { }

protected:
    const ii32      mMask;
    const int       mShift;
    const int       mInputOffset;
    const int       mOutputOffset;
    const int       mInputStartLocation;
    const int       mOutputStartLocation;
};

//===========================================================================
//  CLASS AndSplitterAssemblyInstruction  --
//===========================================================================

class AndSplitterAssemblyInstruction : public SplitterAssemblyInstruction
{
public:
    AndSplitterAssemblyInstruction(ii32 mask)
        : mMask(mask)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int channel_step,
                              int /*Size*/, IppiSize RoiSize)
    {
        ippiAndC_32s_C1R((const Ipp32s*)Input,
                         channel_step,
                         mMask,
                         (Ipp32s*)Output,
                         sizeof(ii32),
                         RoiSize);
    }

    virtual void Process(ii32*  Data, int Size)
    {
        ippsAndC_32u_I(mMask, (Ipp32u*)Data, Size);
    }

protected:
    const ii32  mMask;
};

//===========================================================================
//  CLASS OrSplitterAssemblyInstruction  --
//===========================================================================

class OrSplitterAssemblyInstruction : public SplitterAssemblyInstruction
{
public:
    // Ctor
    OrSplitterAssemblyInstruction(ii32 mask)
        : mMask(mask)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int channel_step,
                              int /*Size*/, IppiSize RoiSize)
    {
        ippiOrC_32s_C1R((const Ipp32s*)Input,
                        channel_step,
                        mMask,
                        (Ipp32s*)Output,
                        sizeof(ii32),
                        RoiSize);
    }

    virtual void Process(ii32*  Data, int Size)
    {
        ippsOrC_32u_I(mMask, (Ipp32u*)Data, Size);
    }

protected:
    const ii32  mMask;
};

//===========================================================================
//  CLASS ShiftSplitterAssemblyInstruction  --
//===========================================================================

class ShiftSplitterAssemblyInstruction : public SplitterAssemblyInstruction
{

public:
    // Ctor
    ShiftSplitterAssemblyInstruction(int shift)
        : mShift(shift)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int channel_step,
                              int /*Size*/, IppiSize RoiSize)
    {
        if (mShift > 0)
            {
            ippiLShiftC_32s_C1R((const Ipp32s*)Input,
                                channel_step,
                                mShift,
                                (Ipp32s*)Output,
                                sizeof(ii32),
                                RoiSize);

            }

        else
            {
            ippiRShiftC_32s_C1R((const Ipp32s*)Input,
                                channel_step,
                                -mShift,
                                (Ipp32s*)Output,
                                sizeof(ii32),
                                RoiSize);
            }
    }

    virtual void Process(ii32*  Data, int Size)
    {
        if (mShift > 0)
            ippsLShiftC_32s_I(mShift, (Ipp32s*)Data, Size);

        else
            ippsRShiftC_32s_I(-mShift, (Ipp32s*)Data, Size);
    }

protected:
    const int   mShift;
};

//===========================================================================
//  CLASS MultiplySplitterAssemblyInstruction  --
//===========================================================================

class MultiplySplitterAssemblyInstruction : public SplitterAssemblyInstruction
{

public:
    // Ctor
    MultiplySplitterAssemblyInstruction(int value)
        : mValue(value)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int channel_step,
                              int Size, IppiSize RoiSize)
    {
        ippiLShiftC_32s_C1R((const Ipp32s*)Input,
                            channel_step,
                            0,
                            (Ipp32s*)Output,
                            sizeof(ii32),
                            RoiSize);
        ippsMulC_32s_ISfs(mValue, (Ipp32s*)Output, Size, 0);
    }

    virtual void Process(ii32*  Data, int Size)
    {
        ippsMulC_32s_ISfs(mValue, (Ipp32s*)Data, Size, 0);
    }

protected:
    const int   mValue;
};

//===========================================================================
//  CLASS AddSplitterAssemblyInstruction  --
//===========================================================================

class AddSplitterAssemblyInstruction : public SplitterAssemblyInstruction
{
public:
    // Ctor
    AddSplitterAssemblyInstruction(int value)
        : mValue(value)
    {
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int channel_step,
                              int Size, IppiSize RoiSize)
    {
        ippiLShiftC_32s_C1R((const Ipp32s*)Input,
                            channel_step,
                            0,
                            (Ipp32s*)Output,
                            sizeof(ii32),
                            RoiSize);
        ippsAddC_32s_ISfs(mValue, (Ipp32s*)Output, Size, 0);
    }

    virtual void Process(ii32*  Data, int Size)
    {
        ippsAddC_32s_ISfs(mValue, (Ipp32s*)Data, Size, 0);
    }

protected:
    const int   mValue;

};

//===========================================================================
//  CLASS SignCorSplitterAssemblyInstruction  --
//===========================================================================

class SignCorSplitterAssemblyInstruction : public SplitterAssemblyInstruction
{
public:
    // Ctor
    SignCorSplitterAssemblyInstruction(ii32 bits)
        : mBits(bits)
    {
        mShift = (sizeof(Ipp32s) * 8) - mBits;
    }

    // Overrides
    virtual void ProcessFirst(const ii32*  Input, ii32*  Output, int channel_step,
                              int /*Size*/, IppiSize RoiSize)
    {
        RoiSize.width *= 2;
        ippiCopy_16s_C1R( (const short *)Input, channel_step, (short *)Output, sizeof(ii32), RoiSize );
    }

    virtual void Process(ii32*  Data, int Size)
    {
        ippsLShiftC_32s_I(mShift, (Ipp32s*)Data, Size);
        ippsRShiftC_32s_I(mShift, (Ipp32s*)Data, Size);
    }

protected:
    const ii32  mBits;
    ii32        mShift;
};


//===========================================================================
//  CLASS SplitterChannelAssembly  --
//===========================================================================

class SplitterChannelAssembly
{
public:
    // Ctor
    SplitterChannelAssembly(SplitterChannel*  channel, int word_no)
        : mChannel(channel), mWordNo(word_no)
    {
    }
    // Dtor
    virtual ~SplitterChannelAssembly()
        {
        Clear();
        }

public:
    // Methods
    void    Process(Buffer input_buffer, Buffer&  output_buffer,
                    int channel_step);

    void AddSeparationInstruction(SplitterInstruction*  instruction)
    {
        mInstructions.push_back(instruction);
    }

    void Clear()
    {
        for (std::list < SplitterInstruction * >::iterator Iter =
                 mInstructions.begin(); Iter != mInstructions.end(); Iter++)
            delete(*Iter);

        mInstructions.clear();
    }

protected:
    // Data
    std::list < SplitterInstruction * > mInstructions;
    SplitterChannel*    mChannel;
    const int           mWordNo;

    // Methods
    virtual void    InternalProcess(Buffer input_data,
                                    Buffer & output_data, int channel_step);
};

//===========================================================================
//  CLASS SplitterChannelAssemblies  --
//===========================================================================

class SplitterChannelAssemblies
{
public:
    ~SplitterChannelAssemblies();

    // Methods
    void Add(SplitterChannelAssembly*  Assembly)
        {  mChannels.push_back(Assembly);  }

    void    Clear();

    size_t Count() const
        {  return mChannels.size();  }

    SplitterChannelAssembly * Assembly(int index)
        {  return mChannels[index];  }

    int mStride;
    int AllBits;

protected:
    // Data
    std::vector < SplitterChannelAssembly * > mChannels;
};


//===========================================================================
//  CLASS SplitterChannelAssembly  --
//===========================================================================

//---------------------------------------------------------------------------
//  SplitterChannelAssembly::Process() --
//---------------------------------------------------------------------------

void SplitterChannelAssembly::Process(Buffer input_buffer,
                                      Buffer&  output_buffer,
                                      int channel_step)
{
    InternalProcess(input_buffer, output_buffer, channel_step);

    Buffer OutputBuffer;
    UCharDG Temp(OutputBuffer);
    UShortDG Temp1(OutputBuffer);
    IppiSize RoiSize;
    RoiSize.width = 1;
    RoiSize.height = static_cast<int>(output_buffer.SizeInInts());
    int Bits = (mChannel->Bits() < 9)?8:(mChannel->Bits() < 17)?16:32;
    switch (Bits)
        {
        case 8:
            Temp.Resize(output_buffer.SizeInInts());
            ippiCopy_8u_C1R((const Ipp8u*)output_buffer.ConstData(), 4,
                            (Ipp8u*)OutputBuffer.Data(), 1, RoiSize);
            break;
        case 16:
            Temp1.Resize(output_buffer.SizeInInts());
            ippiCopy_16u_C1R((const Ipp16u*)output_buffer.ConstData(), 4,
                            (Ipp16u*)OutputBuffer.Data(), 2, RoiSize);
            break;
        default:
            OutputBuffer = output_buffer;
            break;
        }

    mChannel->SendBuffer(OutputBuffer);
}

//---------------------------------------------------------------------------
//  SplitterChannelAssembly::InternalProcess() --
//---------------------------------------------------------------------------

void SplitterChannelAssembly::InternalProcess(Buffer input_data,
                                              Buffer & output_data,
                                              int channel_step)
{
    bool            FirstInstruction = false;
    size_t          ChannelSize = output_data.SizeInInts();

    const ii32*     InDataPtr = ((const ii32*)input_data.ConstData()) +  mWordNo;

    ii32*           OutDataPtr = ((ii32*)output_data.Data());

    IppiSize        RoiSize;
    RoiSize.width = 1;
    RoiSize.height = static_cast<int>(input_data.SizeInInts() - mWordNo);//static_cast<int>(ChannelSize);

    for (std::list < SplitterInstruction * >::iterator Iter =
             mInstructions.begin(); Iter != mInstructions.end(); Iter++)
        {

        if ((*Iter)->Instruction() && ((*Iter)->First()))
            {
            // We have to perform a copy!
            (*Iter)->ProcessFirst(InDataPtr,
                                  OutDataPtr,
                                  channel_step * sizeof(ii32),
                                  static_cast<unsigned int>(ChannelSize),
                                  RoiSize);
            }
        else if ((*Iter)->Instruction() && (!(*Iter)->First()))
            {
            // We already have a copy of the data, so just perform the operation over the output buffer.
            (*Iter)->Process(OutDataPtr, static_cast<unsigned int>(ChannelSize));
            }
        else if (!(*Iter)->Instruction())
            {
            (*Iter)->ProcessFirst( InDataPtr,
                                            OutDataPtr,
                                            channel_step * sizeof(ii32),
                                            static_cast<unsigned int>(ChannelSize),
                                            RoiSize);
            }
        }

    if (FirstInstruction)
        {
        IppiSize    RoiSize;
        RoiSize.width = 1;
        RoiSize.height = static_cast<unsigned int>(ChannelSize);
        ippiLShiftC_32s_C1R((const Ipp32s*)InDataPtr,
                            channel_step * sizeof(ii32),
                            0,
                            (Ipp32s*)OutDataPtr,
                            sizeof(ii32),
                            RoiSize);
        }
}

//===========================================================================
//  CLASS SplitterChannelAssemblies  --
//===========================================================================

//---------------------------------------------------------------------------
// SplitterChannelAssemblies::~SplitterChannelAssemblies() --
//---------------------------------------------------------------------------

SplitterChannelAssemblies::~SplitterChannelAssemblies()
{
    Clear();
}

//---------------------------------------------------------------------------
//  SplitterChannelAssemblies::Clear() --
//---------------------------------------------------------------------------

void SplitterChannelAssemblies::Clear()
{
    for (unsigned i = 0; i < mChannels.size(); i++)
        delete mChannels[i];

    mChannels.clear();
}

//===========================================================================
//  CLASS SplitterChannels  --
//===========================================================================

//---------------------------------------------------------------------------
// SplitterChannels::SplitterChannels() --
//---------------------------------------------------------------------------

SplitterChannels::SplitterChannels(Splitter & owner)
    : mOwner(owner)
{
}

//---------------------------------------------------------------------------
//  SplitterChannels::Clear() --
//---------------------------------------------------------------------------

void SplitterChannels::Clear()
{
    for (unsigned i = 0; i < mChannels.size(); i++)
        delete mChannels[i];

    mChannels.clear();
}

//---------------------------------------------------------------------------
//  SplitterChannels::AddBinary() --
//---------------------------------------------------------------------------

void SplitterChannels::AddBinary(int bits)
{
    BinarySplitterChannel * Channel = new BinarySplitterChannel(mOwner, bits);
    mChannels.push_back(Channel);
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::AddBinary() --
//---------------------------------------------------------------------------

void SplitterChannels::AddBinary(int bits, bool is_signed)
{
    BinarySplitterChannel * Channel = new BinarySplitterChannel(mOwner, bits, is_signed);
    mChannels.push_back(Channel);
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::AddNull() --
//---------------------------------------------------------------------------

void SplitterChannels::AddNull(int bits)
{
    mChannels.push_back(new SplitterChannel(mOwner,
                                            SplitterChannel::ctNull,
                                            bits,
                                            0,
                                            0,
                                            false));
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::InsertBinary() --
//---------------------------------------------------------------------------

void SplitterChannels::InsertBinary(int index, int bits)
{
    SplitterChannel * Channel = new BinarySplitterChannel(mOwner, bits);
    mChannels.insert(mChannels.begin() + index, Channel);
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::InsertBinary() --
//---------------------------------------------------------------------------

void SplitterChannels::InsertBinary(int index, int bits, bool is_signed)
{
    SplitterChannel * Channel = new BinarySplitterChannel(mOwner, bits, is_signed);
    mChannels.insert(mChannels.begin() + index, Channel);
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::InsertNull() --
//---------------------------------------------------------------------------

void SplitterChannels::InsertNull(int index, int bits)
{
    mChannels.insert(mChannels.begin() + index,
                     new SplitterChannel(mOwner,
                                         SplitterChannel::ctNull,
                                         bits,
                                         0,
                                         0,
                                         false));
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::Delete() --
//---------------------------------------------------------------------------

void SplitterChannels::Delete(int index)
{
    mChannels.erase(mChannels.begin() + index);
    Renumber();
}

//---------------------------------------------------------------------------
//  SplitterChannels::Renumber() --
//---------------------------------------------------------------------------

void SplitterChannels::Renumber()
{
    for (size_t i = 0; i < mChannels.size(); ++i)
        mChannels[i]->Index(static_cast<int>(i));
}

//===========================================================================
//  CLASS BinarySplitterChannel  --
//===========================================================================

//---------------------------------------------------------------------------
// BinarySplitterChannel::BinarySplitterChannel() --
//---------------------------------------------------------------------------

BinarySplitterChannel::BinarySplitterChannel(Splitter&  owner, int bits)
    : SplitterChannel(owner, ctBinary, bits, 0, 0, false)
{
}

//---------------------------------------------------------------------------
// BinarySplitterChannel::BinarySplitterChannel() --
//---------------------------------------------------------------------------

BinarySplitterChannel::BinarySplitterChannel(Splitter&  owner, int bits, bool is_signed)
    : SplitterChannel(owner, ctBinary, bits, 0, 0, is_signed)
{
}

//=============================================================================
//  CLASS SplitterChannel  --
//=============================================================================

//---------------------------------------------------------------------------
//  SplitterChannel::SplitterChannel()  -- Ctor
//---------------------------------------------------------------------------

SplitterChannel::SplitterChannel(Splitter &owner, ChannelType channel_type,
    int bits, float offset, float slope, bool is_signed)
    : Owner(owner), FType(channel_type), FBits(bits),
      FOffset(offset), FSlope(slope), FSigned(is_signed), FIndex(0),
      FCursor(0), Output(1000)
{
}

//---------------------------------------------------------------------------
//  SplitterChannel::Type()  -- Set type of splitter channel
//---------------------------------------------------------------------------

void SplitterChannel::Type(ChannelType value)
{
    FType = value;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Type()  -- Return type of splitter channel
//---------------------------------------------------------------------------

SplitterChannel::ChannelType SplitterChannel::Type() const
{
    return FType;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Bits()  --  Set bit width of channel
//---------------------------------------------------------------------------

void SplitterChannel::Bits(int value)
{
    FBits = value;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Bits()  --  Get bit width of channel
//---------------------------------------------------------------------------

int SplitterChannel::Bits() const
{
    return FBits;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Offset()  --
//---------------------------------------------------------------------------

void SplitterChannel::Offset(float value)
{
    FOffset = value;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Offset()  --
//---------------------------------------------------------------------------

float SplitterChannel::Offset() const
{
    return FOffset;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Slope()  --
//---------------------------------------------------------------------------

void SplitterChannel::Slope(float value)
{
    FSlope = value;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Slope()  --
//---------------------------------------------------------------------------

float SplitterChannel::Slope() const
{
    return FSlope;
}

//---------------------------------------------------------------------------
//  SplitterChannel::SplitterChannel()  --
//---------------------------------------------------------------------------

void SplitterChannel::Signed(bool value)
{
    FSigned = value;
}

//---------------------------------------------------------------------------
//  SplitterChannel::SplitterChannel()  --
//---------------------------------------------------------------------------

bool SplitterChannel::Signed() const
{
    return FSigned;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Index()  --
//---------------------------------------------------------------------------

int SplitterChannel::Index() const
{
    return FIndex;
}

//---------------------------------------------------------------------------
//  SplitterChannel::Index()  --
//---------------------------------------------------------------------------

void SplitterChannel::Index(int idx)
{
    FIndex = idx;
}

//---------------------------------------------------------------------------
//  SplitterChannel::BufferSize()  --
//---------------------------------------------------------------------------

size_t SplitterChannel::BufferSize() const
{
    return Output.SizeInBytes()/ElementSize();
}

//---------------------------------------------------------------------------
//  SplitterChannel::BufferSize()  --
//---------------------------------------------------------------------------

void SplitterChannel::BufferSize(size_t elements)
{
    Output.Resize((elements * ElementSize()) / sizeof(ii32));
}

//---------------------------------------------------------------------------
//  SplitterChannel::ElementSize()  --  Get byte width of channel
//---------------------------------------------------------------------------

int SplitterChannel::ElementSize() const
{
    if (FBits < 9)
        return 1;
    else if (FBits < 17)
        return 2;
    else
        return 4;
}

//---------------------------------------------------------------------------
//  SplitterChannel::SendBuffer()  --
//---------------------------------------------------------------------------

void SplitterChannel::SendBuffer(const Buffer & buf)
{
    const unsigned char * src = reinterpret_cast<const unsigned char*>(buf.ConstData());
    unsigned char * dst = reinterpret_cast<unsigned char*>(Output.Data());

    // Copy input to output buffer
    size_t available = buf.SizeInBytes();
    size_t room = Output.SizeInBytes();

    // If sizes of I/O buffers equal, avoid copy
    if (room == available)
        {
        SendBufferEvent e(buf, Index());
        Owner.OnSplit.Execute(e);
        return;
        }

    while (available)
        {
        size_t residual = room - FCursor;
        size_t chunk = std::min(available, residual);
        CopyByteBuffer (src, &dst[FCursor], chunk);

        FCursor += chunk;
        src += chunk;
        available -= chunk;

        // Signal if output buffer is full
        if (FCursor == room)
            {
            SendBufferEvent e(Output, Index());
            Owner.OnSplit.Execute(e);

            FCursor = 0;
            }
        }
}

//---------------------------------------------------------------------------
// Splitter::Splitter() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif
Splitter::Splitter()
    : Channels(*this), mRunning(false), FBufferSize(0x40)
{
    mChannelAssemblies = new SplitterChannelAssemblies;
}
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// Splitter::~Splitter() --
//---------------------------------------------------------------------------

Splitter::~Splitter()
{
    delete mChannelAssemblies;
    mChannelAssemblies = 0;
}

//---------------------------------------------------------------------------
//  Splitter::BufferSize()  --
//---------------------------------------------------------------------------

size_t Splitter::BufferSize() const
{
    return FBufferSize;
}

//---------------------------------------------------------------------------
//  Splitter::BufferSize()  --
//---------------------------------------------------------------------------

void Splitter::BufferSize(size_t elements)
{
    FBufferSize = elements;
}

//---------------------------------------------------------------------------
//  Splitter::Start() --
//---------------------------------------------------------------------------

void Splitter::Start()
{
    if (mRunning)
        return;

    mRunning = true;

    mChannelAssemblies->Clear();
    mChannelAssemblies->mStride = 1;

    const int BitsPerWord = sizeof(ii32) * 8;
    int CurrentChannelOffset = 0;
    int AllBits = 0;
    int WordNo = 0;
    int PatternStepSize = 0;
    int EffectiveAllBits = 0;
    std::vector<int>    Shift;
    std::vector<int>    PatternWordNumber;

    for (size_t i = 0; i < Channels.Count(); i++)
        {
        int Bits = static_cast<int>(Channels[static_cast<int>(i)].Bits());
        Bits = (Bits < 9)?8:(Bits < 17)?16:32;
        EffectiveAllBits += Bits;
        }

    do
        {
        Shift.push_back(PatternStepSize % 32);
        PatternWordNumber.push_back((PatternStepSize) / (BitsPerWord));
        PatternStepSize += EffectiveAllBits;
        } while (PatternStepSize % 32);

    int InputOffset = (PatternStepSize + (BitsPerWord - 1)) / (BitsPerWord);

    for (size_t i = 0; i < Channels.Count(); i++)
        {
        SplitterChannel & Channel = Channels[static_cast<int>(i)];
        Channel.BufferSize(FBufferSize);

        int                         Bits = Channel.Bits();
        SplitterChannelAssembly*    Assembly = NULL;
        int                         IntegerOffset = (int)Channel.Offset();

        bool                        Sign = Channel.Signed();

        int                         EffectiveBits;

        EffectiveBits = (Bits < 9)?8:(Bits < 17)?16:32;

        if (Channel.Type() == SplitterChannel::ctBinary)
            Assembly = new SplitterChannelAssembly(&Channel, WordNo);

        if (Assembly && Bits)
            {
            for (int i = 0; i < (int)Shift.size(); i++)
                {
                int CurrentWordNumber = 0;
                int ChannelOffset = CurrentChannelOffset + Shift[i];

                if (ChannelOffset >= BitsPerWord)
                    {
                    ChannelOffset -= BitsPerWord;
                    CurrentWordNumber++;
                    }

                if (ChannelOffset + Bits > BitsPerWord)
                    {
                    int     FirstWordBits = (BitsPerWord) - ChannelOffset;
                    ii32    Mask = ~(0xFFFFFFFF << FirstWordBits);

                    Assembly->AddSeparationInstruction(
                           new ExtractSplitterMacroInstruction(Mask,
                                                               -ChannelOffset,
                                                               PatternWordNumber[i] + CurrentWordNumber,
                                                               InputOffset,
                                                               i,
                                                               static_cast<int>(Shift.size())));

                    Mask = (Bits==32)?0xFFFFFFFF:~(0xFFFFFFFF << Bits);
                    Assembly->AddSeparationInstruction(
                            new UnWrapSplitterMacroInstruction(Mask,
                                                               FirstWordBits,
                                                               PatternWordNumber[i] + CurrentWordNumber,
                                                               InputOffset,
                                                               i,
                                                               static_cast<int>(Shift.size())));
                    }
                else if ((Bits - ChannelOffset) <= BitsPerWord)
                    {
                    ii32    Mask = (Bits==32)?0xFFFFFFFF:~(0xFFFFFFFF << Bits);
                    Assembly->AddSeparationInstruction(
                           new ExtractSplitterMacroInstruction(Mask,
                                                               -ChannelOffset,
                                                               PatternWordNumber[i] + CurrentWordNumber,
                                                               InputOffset,
                                                               i,
                                                               static_cast<int>(Shift.size())));
                    }
                }

            if (IntegerOffset)
                {
                Assembly->AddSeparationInstruction(new AddSplitterAssemblyInstruction(IntegerOffset));
                }

            if (Sign)
                {
                Assembly->AddSeparationInstruction(new SignCorSplitterAssemblyInstruction(Bits));
                }

            mChannelAssemblies->Add(Assembly);
            }
        else
            mChannelAssemblies->Add(NULL);

        CurrentChannelOffset += EffectiveBits;
        AllBits += Bits;
        if (CurrentChannelOffset >= BitsPerWord)
            {
            CurrentChannelOffset -= BitsPerWord;
            WordNo++;
            }
        }
    mChannelAssemblies->AllBits = EffectiveAllBits;
    mChannelAssemblies->mStride = (AllBits + (BitsPerWord - 1)) / (BitsPerWord);
}

//---------------------------------------------------------------------------
//  Splitter::Stop() --
//---------------------------------------------------------------------------

void Splitter::Stop()
{
    if (!mRunning)
        return;

    mRunning = false;
    mChannelAssemblies->Clear();
}

//---------------------------------------------------------------------------
//  Splitter::Split() --
//---------------------------------------------------------------------------

void Splitter::Split(const Buffer & input)
{
    if (!mRunning)
        Start();

    int TotalBits = 0;
    for (size_t i = 0; i < mChannelAssemblies->Count(); i++)
        {
		int ch = static_cast<int>(i);
        if (mChannelAssemblies->Assembly(ch) == NULL)
            continue;
        TotalBits += (Channels[ch].Bits() < 9)?8:(Channels[ch].Bits() < 17)?16:32;
        size_t OutputSize = (input.SizeInInts() * 32) / mChannelAssemblies->AllBits;
        int reminder = (input.SizeInInts() * 32) % mChannelAssemblies->AllBits;
        reminder -= TotalBits;
        if (reminder >= 0)
            OutputSize++;
        Buffer output(OutputSize);
        mChannelAssemblies->Assembly(ch)->Process(input,
                                                 output,
                                                 mChannelAssemblies->mStride);
        }
}

} // namespace Innovative

