// Splitter_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Splitter_MbH
#define Splitter_MbH

#include <IppFloatDG_Mb.h>
#include <Events_Mb.h>
#include <ProcessEvents_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Splitter;
class SplitterChannel;

//=============================================================================
//  CLASS SplitterChannel  --
//=============================================================================

class SplitterChannel
{
    friend class SplitterChannelAssembly;
    friend class Splitter;

public:
    enum ChannelType { ctBinary, ctNull };

    // Ctor
    SplitterChannel(Splitter & owner, ChannelType channel_type,
        int bits, float offset, float slope, bool is_signed);
    virtual ~SplitterChannel(){}

    // Properties
    void        Type(const ChannelType type);
    ChannelType Type() const;

    void        Bits(int value);
    int         Bits() const;

    void        Offset(float value);
    float       Offset() const;

    void        Slope(float value);
    float       Slope() const;

    void        Signed(bool value);
    bool        Signed() const;

    void        Index(int idx);
    int         Index() const;

    void        BufferSize(size_t elements);
    size_t      BufferSize() const;

protected:
    // Data
    Splitter &  Owner;

    // Fields
    ChannelType         FType;
    int                 FBits;
    float               FOffset;
    float               FSlope;
    bool                FSigned;
    int                 FIndex;
    size_t              FCursor;
    Innovative::Buffer  Output;

    // Support
    void    SendBuffer(const Buffer & buf);
    int     ElementSize() const;
};

//=============================================================================
//  CLASS BinarySplitterChannel  --
//=============================================================================

class BinarySplitterChannel : public SplitterChannel
{
public:
    BinarySplitterChannel(Splitter &owner, int bits);
    BinarySplitterChannel(Splitter &owner, int bits, bool is_signed);
};

//=============================================================================
//  CLASS SplitterChannels  --
//=============================================================================

class SplitterChannels
{
public:
    SplitterChannels(Splitter &mOwner);
    ~SplitterChannels()
        {  Clear();  }

    void Clear();

    void AddBinary(int bits);
    void AddBinary( int bits, bool is_signed );
    void AddNull(int bits);

    void InsertBinary(int index, int bits);
    void InsertBinary( int index, int bits, bool is_signed );
    void InsertNull(int index, int bits);

    void Delete(int index);

    SplitterChannel &operator [] (int index)
        { return *( mChannels[index] );}
    size_t Count()
        { return (int)mChannels.size(); }

protected:
    Splitter &  mOwner;

    std::vector<SplitterChannel *> mChannels;

    void Renumber();
};

class SplitterChannelAssemblies;

//=============================================================================
//  CLASS Splitter  --
//=============================================================================

class Splitter
{
    friend class SplitterChannel;

public:
    Splitter();
    ~Splitter();

    void Split(const Buffer & buffer);
    void Start();
    void Stop();

    void        BufferSize(size_t elements);
    size_t      BufferSize() const;

    // Data
    SplitterChannels Channels;

    OpenWire::EventHandler<Innovative::SendBufferEvent> OnSplit;

protected:
    // Data
    SplitterChannelAssemblies * mChannelAssemblies;
    bool        mRunning;
    size_t      FBufferSize;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative

#endif
