// StartStopThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#include <StartStopThread_Mb.h>


namespace   Innovative
{

//===========================================================================
//  CLASS StartStopThread  --  
//===========================================================================
//---------------------------------------------------------------------------
// StartStopThread::StartStopThread() --
//---------------------------------------------------------------------------
 
StartStopThread::StartStopThread(const char * name)
    : inherited(name), eStart(Event::ManualReset),
      eStopped(Event::ManualReset), eJustStopped(Event::ManualReset),
      eAmStopped(Event::ManualReset)
{
    eStart.Clear();
    eStopped.Set();
    eAmStopped.Set();
}

//---------------------------------------------------------------------------
//  StartStopThread::Start() -- Start a stopped thread
//------------------------------------------------------------------------------

void StartStopThread::Start()
{
    eStopped.Clear();
    eStart.Set();
}

//------------------------------------------------------------------------------
//  StartStopThread::Stop() -- Stop the thread (non-blocking)
//------------------------------------------------------------------------------

void StartStopThread::Stop()
{
    XXX_DebugLog(0xD0, 1);
    if (eStart.Check())
        {
        XXX_DebugLog(0xD0, 2);
        // Not stopped yet.
        eJustStopped.Clear();
        XXX_DebugLog(0xD0, 3);
        eStart.Clear();
        XXX_DebugLog(0xD0, 4);
        eStopped.Set();
        XXX_DebugLog(0xD0, 5);
        }
    XXX_DebugLog(0xD0, 0);
}

//------------------------------------------------------------------------------
//  StartStopThread::WaitForStopped() -- Wait for stop to finish
//------------------------------------------------------------------------------
//  Will only block if Stop has already been called

void StartStopThread::WaitForStopped()
{
    eJustStopped.WaitFor();
}

//------------------------------------------------------------------------------
//  StartStopThread::WaitForStopped_External() -- Wait for stop to finish
//------------------------------------------------------------------------------
//  Will block if until Stop is called, and completed

void StartStopThread::WaitForStopped_External()
{
    XXX_DebugLog(0xDF, 1);
    eAmStopped.WaitFor();
    XXX_DebugLog(0xDF, 2);
    eJustStopped.WaitFor();
    XXX_DebugLog(0xDF, 0);
}

//---------------------------------------------------------------------------
//  StartStopThread::LoadDefaultEventsInto() --
//------------------------------------------------------------------------------

void StartStopThread::LoadDefaultEventsInto(MultipleObjects & multi_object)
{
    inherited::LoadDefaultEventsInto(multi_object);
    multi_object.AddObject(eStopped, sStopped);
}

//---------------------------------------------------------------------------
//  StartStopThread::Init() --
//------------------------------------------------------------------------------

void StartStopThread::Init()
{
    inherited::Init();

    inherited::LoadDefaultEventsInto(StartStopEvents);
    StartStopEvents.AddObject(eStart, sStart);

    AddObject(eStopped, sStopped);
}

//---------------------------------------------------------------------------
//  StartStopThread::Execute() --
//------------------------------------------------------------------------------

void StartStopThread::Execute()
{
    bool JustStarted = true;
    while (!Terminated())
        {
        //
        //  Thread is stopped and started by Firing 'eStart'
        //    Make sure it is on to run loop
        //
        //  To stop the thread, clear eStart and set sStopped. This will
        //    eventually stop waiting for eStart to be set again
        //
        //  To kill the thread, terminate it, and fire the eTerminate
        //    event. This skips all other processing if eTerminate has gone
        //    off.
        //
        if (!eStart.Check())
            {
            XXX_DebugLog(0xD3, 1);
            // We will stop !
            eJustStopped.Set();

            if (!JustStarted)
                AtStop();

			eAmStopped.Set();
            int result = StartStopEvents.WaitFor();
            if (result == sTerminate)
            {
                break;
            }
            else
            {
                result = result;
            }

            JustStarted = true;
            eAmStopped.Clear();
            XXX_DebugLog(0xD3, 0);
            }
        //
        //  If we just restarted, call AtStart()
        if (JustStarted)
            {
            XXX_DebugLog(0xD4, 1);
            AtStart();
            JustStarted = false;
            XXX_DebugLog(0xD4, 0);
            }
        //
        //  SingleExecution() processes one iteration 
        SingleExecution();
        XXX_DebugLog(0xD5, eStart.Check());
        XXX_DebugLog(0xD6, eStopped.Check());
        }
}

//------------------------------------------------------------------------------
//  StartStopThread::AtStart() -- Thread start or restart time overridable
//------------------------------------------------------------------------------

void StartStopThread::AtStart()
{
}

//------------------------------------------------------------------------------
//  StartStopThread::AtStop() -- Thread stop time overridable
//------------------------------------------------------------------------------

void StartStopThread::AtStop()
{
}

//===========================================================================
//  CLASS BasicSSThread  -- Basic thread with 1 waitfor & start/stop
//===========================================================================
//------------------------------------------------------------------------------
//  BasicSSThread::Init() -- Init Multi-Objects here
//------------------------------------------------------------------------------

void  BasicSSThread::Init()
{
    inherited::Init();
}

//---------------------------------------------------------------------------
//  BasicSSThread::SingleExecution() -- thread function
//---------------------------------------------------------------------------

void BasicSSThread::SingleExecution()
{
    int result = WaitFor();
    XXX_DebugLog(0xD7, result);
    if ((result == sTerminate) || (result == sStopped))
        return;

    AtExecution(result);
}

//------------------------------------------------------------------------------
//  BasicSSThread::AtExecution() -- User thread processing
//------------------------------------------------------------------------------

void  BasicSSThread::AtExecution(int /*result*/)
{
}


} // namespace Innovative

