// StateMachine_Mb.cpp
//
//  Class to extract version information from DLLs and EXEs
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "StateMachine_Mb.h"
#include <sstream>
#include "Exception_Mb.h"

namespace Innovative
{

using namespace std;

//===========================================================================
//  CLASS  StateMachine  --  Message-based state machine implementation
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class StateMachine
//---------------------------------------------------------------------------

StateMachine::StateMachine()
{
    OnError.SetEvent(this, &StateMachine::HandleError);
}

//---------------------------------------------------------------------------
//  StateMachine::Clear() -- Clear the entire state machine
//---------------------------------------------------------------------------

void  StateMachine::Clear()
{
    Registry.clear();
}

//---------------------------------------------------------------------------
//  StateMachine::Clear() -- Remove specified state from the machine
//---------------------------------------------------------------------------

void  StateMachine::Clear(unsigned int id)
{
    //
    //  Erase the entry in the map
    Registry.erase(id);
}

//---------------------------------------------------------------------------
//  StateMachine::At() -- Access specific machine state
//---------------------------------------------------------------------------

MachineState & StateMachine::At(unsigned int id)
{
    return Registry[id];
}

//---------------------------------------------------------------------------
//  StateMachine::State() -- Change to the specified state
//---------------------------------------------------------------------------

void  StateMachine::State(unsigned int id)
{
    ProcessCompletionEvent e(id);

    //  If not found, dispatch to error handler...
    if (!Registry.count(id))
        {
        OnError.Execute(e);
        return;
        }

    Iterator = Registry.find(id);
    Iterator->second.Execute(e);
    ++Iterator;
}

//---------------------------------------------------------------------------
//  StateMachine::Next() -- Change to the next state
//---------------------------------------------------------------------------

void  StateMachine::Next()
{
    unsigned int id = (*Iterator).first;
    State(id);
}

//---------------------------------------------------------------------------
//  StateMachine::HandleError() -- Default error handler
//---------------------------------------------------------------------------

void  StateMachine::HandleError(ProcessCompletionEvent & Msg)
{
    stringstream msg;
    msg << "Invalid machine state: " << Msg.Result;
    throw MalibuException(msg.str());
}

} // namespace Innovative
