// Stats_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation. 
//  All Rights Reserved.
//

#ifndef __Stats_h__
#define __Stats_h__

#include <limits>
#include <cmath>
#include <Magic_Mb.h>

#ifdef min
	// Eliminate MSVC macro
	#undef min
#endif

#ifdef max
	// Eliminate MSVC macro
	#undef max
#endif

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Statistics
{
public:
//
//  Member Functions
//
    //
    //  Constructor
    Statistics() 
        : maxindex(0), minindex(0), bits(8), range(256)
        { Init(); };
    void Init()
        {
        points = 0;
        sum = sum2 = 0.0;
		minimum = std::numeric_limits<double>::max();
		maximum = -(std::numeric_limits<double>::max());
		}
    // Submit another sample
    void Sample(double sample)
        {
        if (sample < minimum)
            {
            minimum = sample;
            minindex = points;
            }
        if (sample > maximum)
            {
            maximum = sample;
            maxindex = points;
            }
            
        sum += sample;     
        sum2 += sample*sample;
              
        ++points;
        }
    // Return the number of points tallied
	unsigned int Points() const
		{ return points; }
	// Return the integral of the data set
	double Integral() const
		{ return sum; }
	// Return the squared integral of the data set
	double Integral2() const
		{ return sum2; }
	// Return the RMS of the data set
	double Rms() const
		{ return sum2 ? std::sqrt(sum2/Points()) : 0; }
	// Return the equivalent amplitude of the data set
	double Amplitude() const
		{ return Rms() * sqrt(2.0); }
	// Return the maximum value so far
	double Maximum() const
		{ return points ? maximum : 0.0; }
	// Return the index of the maximum value in the set
	int MaxIndex() const
		{ return maxindex; }
	// Return the minimum value so far
	double Minimum() const
		{ return points ? minimum : 0.0; }
	// Return the index of the minimum value in the set
	int MinIndex() const
		{ return minindex; }
	// Return the delta
	double Delta() const
		{ return Maximum() - Minimum(); }
	// Return the arithmetic mean
	double Mean() const
		{ return points ? sum/Points() : 0.0; }
	// Return the standard deviation
	double SDev() const
		{
		if (points>1)
			return std::sqrt(std::fabs((sum2-((sum*sum)/points))/((points-1))));

		return 0.0;
		}
	// Set the bit resolution
	void Bits(unsigned int num_bits)
		{
        bits = num_bits;
        range = std::pow(2.0, static_cast<double>(bits));
        }
	// Get the bit resolution
	int Bits() const
		{ return bits; }
	// Set the voltage range
	void Range(double span)
		{
		range = span;
		bits = static_cast<int>(std::ceil(Innovative::LogN(2.0, span)));
		}
	// Get the voltage range
	double Range() const
		{ return range; }
	// Return the dynamic range
	double dB() const
		{
		return 20.0*std::log10((std::fabs(Maximum()-Minimum())+1.0)/range);
		}
	// Theoretical SNR - pg 42 from P. Embree C Algorithms book
	double SNR() const
		{
		int b = (bits > 1) ? bits : 24;
		return (6.02*b) + 1.76;
		}

private:
	unsigned int    points;
	double	        minimum;
	double	        maximum;
	double	        sum;
	double	        sum2;
	unsigned int	maxindex;
    unsigned int	minindex;
    unsigned int    bits;
    double	        range;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
