// StopWatch_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DataTypes_Mb.h>
#include <StopWatch_Mb.h>
#include <Exception_Mb.h>

#ifdef __BORLANDC__
#include <mem.h>
#endif

#ifndef LINUX
	#include <windows.h>
#else
	#include <sys/time.h>
#endif

namespace   Innovative
{

static const double Meg = 1.0e6;
static const double Gig = 1.0e9;

//------------------------------------------------------------------------
// class StopWatchImpl -- Windows implementation.
//------------------------------------------------------------------------
#ifndef LINUX

class StopWatchImpl
{
public:
    StopWatchImpl();
    
     double          TimerRate() const
        {  return static_cast<double>(FTimerRate);  }
    
    double Now();
private:
	ii64         FTimerRate;
    void QueryPerformanceFrequency(ii64 & timerRate);
    void QueryPerformanceCounter(ii64 & count);
};

//------------------------------------------------------------------------
//  Constructor of class StopWatchImpl
//------------------------------------------------------------------------

StopWatchImpl::StopWatchImpl()
    : FTimerRate(1)
{
    QueryPerformanceFrequency(FTimerRate);

    if (!FTimerRate)
        throw MalibuException("Performance timer unavailable!");
}

//------------------------------------------------------------------------
// StopWatchImpl::Now() - returns a high precision time value
//------------------------------------------------------------------------

double StopWatchImpl::Now()
{
    ii64 Current;
    QueryPerformanceCounter(Current);

    return static_cast<double>(Current)/TimerRate();
}

//------------------------------------------------------------------------
// StopWatchImpl::QueryPerformanceFrequency()
//------------------------------------------------------------------------

void StopWatchImpl::QueryPerformanceFrequency(ii64 & timerRate)
{
    ::QueryPerformanceFrequency(reinterpret_cast<LARGE_INTEGER*>(&timerRate));
}

//------------------------------------------------------------------------
// StopWatchImpl::QueryPerformancCounter()
//------------------------------------------------------------------------

void StopWatchImpl::QueryPerformanceCounter(ii64 & count)
{
    ::QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER*>(&count));
}
#else

class StopWatchImpl
{
public:
    StopWatchImpl()  {  }
    double Now();
    double TimerRate() const
        {  return 0;  }

private:
  
	struct timeval now;	
};

//------------------------------------------------------------------------
// StopWatchImpl::Now() - returns a high precision time value
//------------------------------------------------------------------------

double StopWatchImpl::Now()
{
	
    gettimeofday(&now, NULL);
    double ret_val = now.tv_sec;
    ret_val += now.tv_usec/Meg;
   
    return ret_val;
}

#endif
//========================================================================
//  CLASS StopWatch --  Accurate stopwatch timer class
//========================================================================
//------------------------------------------------------------------------
//  constructor of class StopWatch
//------------------------------------------------------------------------

StopWatch::StopWatch()
    : impl(new StopWatchImpl())
{

}

//------------------------------------------------------------------------
//  Copy constructor
//------------------------------------------------------------------------

StopWatch::StopWatch(const StopWatch & /*other*/)
    : impl(new StopWatchImpl())
{
}

//------------------------------------------------------------------------
//  destructor of class StopWatch
//------------------------------------------------------------------------

StopWatch::~StopWatch()
{
    delete impl;  impl = 0;
}

//------------------------------------------------------------------------
//  Assignment operator
//------------------------------------------------------------------------

StopWatch & StopWatch::operator =(const StopWatch & other)
{
    if (this != &other)
        {
        delete impl;
        impl = new StopWatchImpl();
        }

    return *this;
}

//---------------------------------------------------------------------------
// StopWatch::Start() -- Push a timer reading
//---------------------------------------------------------------------------

void StopWatch::Start()
{
    Timer.push(Now());
}

//---------------------------------------------------------------------------
// StopWatch::Stop() -- Calculate elapsed time (destructive)
//---------------------------------------------------------------------------

double StopWatch::Stop()
{
    if (!Timer.size())
        return 0.0;

    double Delta = Now() - Timer.top();
    Timer.pop();

    return Delta;
}

//---------------------------------------------------------------------------
// StopWatch::Elapsed() -- Calculate elapsed time (non-destructive)
//---------------------------------------------------------------------------

double StopWatch::Elapsed()
{
    if (!Timer.size())
        return 0.0;

    return Now() - Timer.top();
}

//---------------------------------------------------------------------------
// StopWatch::uS() -- Dwell for specified number of uS
//---------------------------------------------------------------------------

void StopWatch::uS(int Delay)
{
	
    double Then = Now() + static_cast<double>(Delay)/Meg;

    while (Now() < Then)
        ;
   
}

//---------------------------------------------------------------------------
// StopWatch::Mhz() -- Timer clock rate
//---------------------------------------------------------------------------

double StopWatch::Mhz() const
{
    return impl->TimerRate()/Meg;
}

//---------------------------------------------------------------------------
// StopWatch::Now() -- Return current time
//---------------------------------------------------------------------------

double StopWatch::Now()
{
    return impl->Now();
}


}  // namespace Innovative
