//---------------------------------------------------------------------------

#ifndef StreamDeviceEvents_MbH
#define StreamDeviceEvents_MbH

#include <Events_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IStreamDeviceEvents  --  Stream Notification Events
//===========================================================================

class IStreamDeviceEvents
{
public:
    // Events
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnStreamPreconfig;
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnStreamSingleTimeAtStartConfig;
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnStreamConfigure;
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnBeforeStreamStart;
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnAfterStreamStart;
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnBeforeStreamStop;
    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnAfterStreamStop;
};

//===========================================================================
//  CLASS StreamDeviceEventCallbacks  --  Stream Notification Callbacks
//===========================================================================

class StreamDeviceEventCallbacks
{
public:
    StreamDeviceEventCallbacks(IStreamDeviceEvents & owner)
        :  Owner(owner)
        {}
    virtual ~StreamDeviceEventCallbacks()
        {}
    // Callbacks
    virtual void  CallOnStreamPreconfig(OpenWire::NotifyEvent & Event)
        { Owner.OnStreamPreconfig.Execute(Event); }
    virtual void  CallOnStreamSingleTimeAtStartConfig(OpenWire::NotifyEvent & Event)
        { Owner.OnStreamSingleTimeAtStartConfig.Execute(Event); }
    virtual void  CallOnStreamConfigure(OpenWire::NotifyEvent & Event)
        { Owner.OnStreamConfigure.Execute(Event); }
    virtual void  CallOnBeforeStreamStart(OpenWire::NotifyEvent & Event)
        { Owner.OnBeforeStreamStart.Execute(Event); }
    virtual void  CallOnAfterStreamStart(OpenWire::NotifyEvent & Event)
        { Owner.OnAfterStreamStart.Execute(Event); }
    virtual void  CallOnBeforeStreamStop(OpenWire::NotifyEvent & Event)
        { Owner.OnBeforeStreamStop.Execute(Event); }
    virtual void  CallOnAfterStreamStop(OpenWire::NotifyEvent & Event)
        { Owner.OnAfterStreamStop.Execute(Event); }

private:
    IStreamDeviceEvents & Owner;
};

//===========================================================================
//  CLASS IUsesStreamDeviceEventCallbacks  --  "Use" of Stream Device CBs
//===========================================================================

class IUsesStreamDeviceEventCallbacks
{
public:
    IUsesStreamDeviceEventCallbacks(IStreamDeviceEvents * owner)
        :  Sde_Cb(new StreamDeviceEventCallbacks(*owner))
        {}
    IUsesStreamDeviceEventCallbacks(IStreamDeviceEvents & owner)
        :  Sde_Cb(new StreamDeviceEventCallbacks(owner))
        {}

    ~IUsesStreamDeviceEventCallbacks()
        {  delete Sde_Cb;  Sde_Cb=0;  }

    void  Load_StreamDeviceEventCallbacks(StreamDeviceEventCallbacks * cb)
        {
        delete Sde_Cb;
        Sde_Cb = cb;
        }

    StreamDeviceEventCallbacks *  OwnerCB()
        {  return Sde_Cb;  }

private:
    StreamDeviceEventCallbacks * Sde_Cb;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative

#endif
