// StringList_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#include <StringList_Mb.h>
#include <fstream>
#include <FileSupport_Mb.h>
#include <StringSupport_Mb.h>
#include <memory>
#include <cstring>

using namespace std;

namespace   Innovative
{

#define Etx "\003"

//=============================================================================
//  CLASS StringList  --  Vector of Strings with file I/O
//=============================================================================
//-----------------------------------------------------------------------------
//  StringList::StringList() -- CTor
//-----------------------------------------------------------------------------

StringList::StringList()
{
}

//-----------------------------------------------------------------------------
//  StringList::Clear() -- empty the vector
//-----------------------------------------------------------------------------

void StringList::Clear()
{
    return FList.clear();
}

//-----------------------------------------------------------------------------
//  StringList::Count() -- return # of strings in the vector
//-----------------------------------------------------------------------------

unsigned int StringList::Count() const
{
    return static_cast<unsigned int>(FList.size());
}

//-----------------------------------------------------------------------------
//  StringList::Add() --
//-----------------------------------------------------------------------------

void StringList::Add(const string & value)
{
    FList.push_back(value);
}

//---------------------------------------------------------------------------
// StringList::Pop() --  Return top of string stack
//---------------------------------------------------------------------------

string StringList::Pop()
{
    string result;

    if (Count())
        {
        result = At(0);
        Delete(0);
        }

    return result;
}

//-----------------------------------------------------------------------------
//  StringList::Text() --  Store list as string
//-----------------------------------------------------------------------------

string StringList::Text() const
{
    return AsText(Etx);
}

//-----------------------------------------------------------------------------
//  StringList::AsText() --  Store list as string (variable etx string)
//-----------------------------------------------------------------------------

string StringList::AsText(const string & etx) const
{
    string result;

    const_iterator i;
    for (i = begin(); i != end(); ++i)
        result += *i + string(etx);

    return result;
}

//-----------------------------------------------------------------------------
//  StringList::Text() --  Retreive list from string
//-----------------------------------------------------------------------------

void StringList::Text(const string & txt, const string & etx)
{
    Clear();

    size_t cursor = 0;
    while (1==1)
        {
        size_t index = txt.find(string(etx), cursor);
        if  (index != string::npos)
            {
            //  string found
            string line = txt.substr(cursor, index-cursor);
            Add(line);
            cursor = index+etx.size();
            }
        else
            {
            //  append entire remaining string
            string line = txt.substr(cursor);
            Add(line);
            break;    // and stop
            }
        }
}

//-----------------------------------------------------------------------------
//  StringList::Text() --  Retreive list from string
//-----------------------------------------------------------------------------

void StringList::Text(const string & txt)
{
    Text(txt, Etx);
}

//-----------------------------------------------------------------------------
//  StringList::FileExists() -- return true if file exists...
//-----------------------------------------------------------------------------

bool StringList::FileExists(const string & filename) const
{
    ifstream in(filename.c_str());

    return in.good();
}

//-----------------------------------------------------------------------------
//  StringList::IndexOf() -- Return index of item in list
//-----------------------------------------------------------------------------

int StringList::IndexOf(const std::string & item) const
{
    int idx = 0;
    for (StringList::const_iterator i = begin(); i != end(); ++i, ++idx)
        {
        if (StartsWith(*i, item) == 0)
            return idx;
        }

    return -1;
}

//-----------------------------------------------------------------------------
//  StringList::LoadFromFile() -- Fill string list from the file
//-----------------------------------------------------------------------------

bool StringList::LoadFromFile(const string & filename)
{
    ifstream in(filename.c_str());

    if (!in.good())
        return in.good();

    ii64  TotalSize(Innovative::FileSize(filename.c_str()));
    bool  DisplayLoadProgress( TotalSize != 0 );
    ii64  SizeRead(0);

    Clear();

    string line;
    int OldPercent = -1;
    while (in.good())
        {
        getline(in, line);
        Add(line);
        //
        SizeRead += line.size();

        if (DisplayLoadProgress)
            {
            int Percent = static_cast<int>((SizeRead * 100) / TotalSize);
            if (Percent != OldPercent)
                {
                OldPercent = Percent;
				ProcessProgressEvent ppe(Percent);
                OnProgress.Execute(ppe);
                }
            }
        }

    if (DisplayLoadProgress)
        {
		ProcessProgressEvent ppe100(100);
        OnProgress.Execute(ppe100);
        }

	ProcessCompletionEvent pce;
    OnComplete.Execute(pce);

    in.close();

    return true;
}

//-----------------------------------------------------------------------------
//  StringList::StoreToFile() -- Write string list to file
//-----------------------------------------------------------------------------

bool StringList::StoreToFile(const string & filename)
{
    ofstream out(filename.c_str());

    if (!out.good())
        return out.good();

    out << Text();
    out.close();

	ProcessCompletionEvent pce;
    OnComplete.Execute(pce);

    return true;
}

//---------------------------------------------------------------------------
//  StringList::Write - Flush contents of string list to file
//---------------------------------------------------------------------------

void StringList::Write(const string & name)
{
    ofstream err;
    err.open(name.c_str());
    for (const_iterator i = FList.begin(); i != FList.end(); ++i)
        err << *i << endl;
    err.close();
}

//-----------------------------------------------------------------------------
//  StringList::At() -- Return  Nth string in list
//-----------------------------------------------------------------------------

std::string StringList::At(size_t idx) const
{
    size_t item = 0;
    for (List::const_iterator i = FList.begin(); i != FList.end(); ++i, ++item)
        {
        if (item == idx)
            return *i;
        }

    return "";
}

//-----------------------------------------------------------------------------
//  StringList::Delete() -- Delete  Nth string in list
//-----------------------------------------------------------------------------

void StringList::Delete(int idx)
{
    int item = 0;
    for (List::iterator i = FList.begin(); i != FList.end(); ++i, ++item)
        {
        if (item == idx)
            {
            FList.erase(i);
            break;
            }
        }
}

//---------------------------------------------------------------------------
//  StringList::Words() --  Parse delimited text, adding to list
//---------------------------------------------------------------------------

size_t StringList::Words(const string & txt, const char * delimiters)
{
    auto_ptr<char> temp(new char[txt.size()+1]);
    char * src = temp.get();
    strcpy(src, txt.c_str());

    char * p = strtok (src, delimiters);
    while(p)
        {
        Add(p);
        p = strtok (NULL, delimiters);
        }

    return Count();
}

//---------------------------------------------------------------------------
//  StringList::NextRevisions() --
//---------------------------------------------------------------------------

void StringList::NextRevisions(const std::string & pn)
{
    if (!pn.size())
        return;

    StringList append;

    stringstream prefix;
    string suffix;
    if (pn.find("-") != string::npos)
        {
        prefix << DelimitedValue(pn, "-");
        suffix = pn.substr(prefix.str().size(), pn.size()-prefix.str().size());
        }
    else
        {
        size_t idx = 0;
        while (isdigit(pn[idx]))
            prefix << pn[idx++];
        for (; idx < pn.size(); ++idx)
            suffix += pn[idx];
        }

    if (!suffix.size())
        {
        append.Add("A");
        append.Add("A1");
        }
    else
        {
        // Suffix processing
        // Forms: -VV, -VVR, -VVRN, -VVLN, -VVLNR, -VVLNRN

        // Eliminate variant
        while (suffix.size() && isdigit(suffix[0]))
            {
            prefix << suffix[0];
            suffix.erase(0, 1);
            }

        // Eliminate ruggedization level
        if (suffix.size() && suffix[0] == 'L')
            for (int i = 0; i < 2; ++i)
                {
                prefix << suffix[0];
                suffix.erase(0, 1);
                }

        if (!suffix.size())
            {
            append.Add("A");
            append.Add("A1");
            }
        else
            {
            if (isalpha(suffix[0]))
                {
                string letter;
                letter.push_back(suffix[0]);
                suffix.erase(0, 1);

                if (!suffix.size())
                    {
                    append.Add(letter + string("1"));
                    ++letter[0];
                    append.Add(letter);
                    append.Add(letter + string("1"));
                    }
                else
                    {
                    int code;
                    if (!(istringstream(suffix) >> code))
                        code = 0;

                    ostringstream msg;
                    msg << letter << code+1;
                    append.Add(msg.str());
                    ++letter[0];
                    append.Add(string(letter));
                    msg.str("");
                    msg << letter << "1";
                    append.Add(msg.str());
                    }
                }
            }
        }

    for (StringList::const_iterator i = append.begin(); i != append.end(); ++i)
        Add(prefix.str() + *i);

    return;
}

} // namespace Innovative

