// StringSupport_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <StringSupport_Mb.h>
#include <cctype>
#include <sstream>
#include <iomanip>
#include <functional>
#include <cstring>
#include <algorithm>
#include <StringList_Mb.h>

using namespace std;

namespace   Innovative
{

//---------------------------------------------------------------------------
//  StartsWith() -- Case insensitive string compare
//  returns: 0=equal, -1=s1<s2, +1=s1>s2
//---------------------------------------------------------------------------

// Case insensitive compare function:
int StartsWith(const char * s1, const char * s2, size_t len)
{
    // Select the first element of each string:
    const char * p1 = s1;
    const char * p2 = s2;

    // Dont run past the end:
    while(*p1 && *p2 && len)
        {
        char c1 = toupper(*p1);
        char c2 = toupper(*p2);
        // Compare upper-cased chars:
        if(c1 != c2)
            {
            // Report which was lexically  greater:
            return (c1<c2)? -1 : 1;
            }

        ++p1;
        ++p2;
        --len;
        }

  return 0;
}

//---------------------------------------------------------------------------
//  StartsWith() -- Case insensitive string compare
//  returns: 0=equal, -1=s1<s2, +1=s1>s2
//---------------------------------------------------------------------------

// Case insensitive compare function:
int StartsWith(const string& s1, const string& s2)
{
    size_t len = std::min(s1.size(), s2.size());
    return StartsWith(s1.c_str(), s2.c_str(), len);
}

//---------------------------------------------------------------------------
//  StringCompare() -- Case insensitive string compare
//  returns: 0=equal, -1=s1<s2, +1=s1>s2
//---------------------------------------------------------------------------

// Case insensitive compare function:
int StringCompare(const char * s1, const char * s2, size_t len)
{
    // Select the first element of each string:
    const char * p1 = s1;
    const char * p2 = s2;

    // Dont run past the end:
    while(*p1 && *p2 && len)
        {
        char c1 = toupper(*p1);
        char c2 = toupper(*p2);
        // Compare upper-cased chars:
        if(c1 != c2)
            {
            // Report which was lexically  greater:
            return (c1<c2)? -1 : 1;
            }

        ++p1;
        ++p2;
        --len;
        }

  // If they match up to the detected eos, say
  // which was longer. Return 0 if the same.
  int difference = static_cast<int>(strlen(s2) - strlen(s1));

  if (difference < 0)
    return -1;

  if (difference > 0)
    return 1;

  return 0;
}

//---------------------------------------------------------------------------
//  StringCompare() -- Case insensitive string compare
//  returns: 0=equal, -1=s1<s2, +1=s1>s2
//---------------------------------------------------------------------------

// Case insensitive compare function:
int StringCompare(const string& s1, const string& s2)
{
    size_t len = std::min(s1.size(), s2.size());
    return StringCompare(s1.c_str(), s2.c_str(), len);
}

namespace
{
// Local Functions
//---------------------------------------------------------------------------
//  IntXXToStringSetup() -- Shared Preliminarys for IntToString functions
//---------------------------------------------------------------------------

void IntXXToStringSetup(std::stringstream & ss, int radix, bool c_prefix_txt)
{
    //
    //  Set up string to be proper radix
    if (radix==16)
        ss << std::hex;
    else if (radix==8)
        ss << std::oct;
    //
    //  Stream in prefix text if enabled
    if (c_prefix_txt && radix==8)
        ss << '0';   // octal has leading 0 prefix
    else if (c_prefix_txt && radix==16)
        ss << '0' << 'x';   // hex has leading 0x prefix

}

}  // end nameless namespace

//---------------------------------------------------------------------------
//  IntToString() -- Convert Integer to a string representation
//---------------------------------------------------------------------------
//  c_prefix_txt will, if true, put the C style base prefixes on the number

std::string IntToString(int value, int radix, bool c_prefix_txt)
{
    std::stringstream ss;
    //
    //  Set up radix, and prefixes
    IntXXToStringSetup(ss, radix, c_prefix_txt);
    //
    //  Stream in actual number
    ss << value;  // do NOT use "; " as this illegally embeds null
                  //   in final output string!!
    //
    //  Stream final number out
    std::string rtn_string;
    ss >> rtn_string;

    return rtn_string;
}

//---------------------------------------------------------------------------
//  IntToString() -- Convert Unsigned Integer to a string representation
//---------------------------------------------------------------------------
//  c_prefix_txt will, if true, put the C style base prefixes on the number

std::string UIntToString(unsigned int value, int radix, bool c_prefix_txt)
{
    std::stringstream ss;
    //
    //  Set up radix, and prefixes
    IntXXToStringSetup(ss, radix, c_prefix_txt);
    //
    //  Stream in actual number
    ss << value;  // do NOT use "; " as this illegally embeds null
                  //   in final output string!!
    //
    //  Stream final number out
    std::string rtn_string;
    ss >> rtn_string;

    return rtn_string;
}

//------------------------------------------------------------------------------
//  Int64ToString() -- Convert 64 bit Integer to a string representation
//------------------------------------------------------------------------------
//  c_prefix_txt will, if true, put the C style base prefixes on the number

std::string Int64ToString(ii64 value, int radix, bool c_prefix_txt)
{
    std::stringstream ss;
    //
    //  Set up radix, and prefixes
    IntXXToStringSetup(ss, radix, c_prefix_txt);
    //
    //  Stream in actual number
    ss << value;  // do NOT use "; " as this illegally embeds null
                  //   in final output string!!
    //
    //  Stream final number out
    std::string rtn_string;
    ss >> rtn_string;

    return rtn_string;
}

//------------------------------------------------------------------------------
//  Int64ToString() -- Convert 64 bit signed Integer to a string representation
//------------------------------------------------------------------------------
//  c_prefix_txt will, if true, put the C style base prefixes on the number

std::string Int64ToString(s_ii64 value, int radix, bool c_prefix_txt)
{
    std::stringstream ss;
    //
    //  Set up radix, and prefixes
    IntXXToStringSetup(ss, radix, c_prefix_txt);
    //
    //  Stream in actual number
    ss << value;  // do NOT use "; " as this illegally embeds null
                  //   in final output string!!
    //
    //  Stream final number out
    std::string rtn_string;
    ss >> rtn_string;

    return rtn_string;
}

//---------------------------------------------------------------------------
//  FloatToString() --
//---------------------------------------------------------------------------
 
std::string FloatToString(double value, int precision, bool fix)
{
    std::stringstream ss;
    //
	//  Stream in
	if (fix)
		ss << std::fixed;
	else
		ss << std::scientific;

	ss << std::setprecision(precision) << value;

    std::string rtn_string;
    ss >> rtn_string;

    return rtn_string;
}

//-----------------------------------------------------------------------------
//  StringToInt() -- Convert number to integer (autodetect base using C style)
//-----------------------------------------------------------------------------
//  '0x' prefix means convert as hex, '0' means convert as octal

int StringToInt(const std::string & value, bool & ok)
{
    if (value.size()>=2 && value[0]=='0' && tolower(value[1])=='x')
        return StringToHex(value, ok);

    if (value.size()>=1 && value[0]=='0')
        return StringToOct(value, ok);

    return StringToDec(value, ok);
}

//-----------------------------------------------------------------------------
//  StringToInt() -- Convert number to integer (autodetect base using C style)
//-----------------------------------------------------------------------------
//  '0x' prefix means convert as hex, '0' means convert as octal

int StringToInt(const std::string & value, int def)
{
    bool ok = true;
    int result = StringToInt(value, ok);
    return ok ? result : def;
}

//-----------------------------------------------------------------------------
//  StringToInt64() -- Convert number to integer (autodetect base using C style)
//-----------------------------------------------------------------------------
//  '0x' prefix means convert as hex, '0' means convert as octal

ii64 StringToInt64(const std::string & value, bool & ok)
{
    if (value.size()>=2 && value[0]=='0' && tolower(value[1])=='x')
        return StringToHex64(value, ok);

    if (value.size()>=1 && value[0]=='0')
        return StringToOct64(value, ok);

    return StringToDec64(value, ok);
}

//-----------------------------------------------------------------------------
//  StringToInt64() -- Convert number to integer (autodetect base using C style)
//-----------------------------------------------------------------------------
//  '0x' prefix means convert as hex, '0' means convert as octal

ii64 StringToInt64(const std::string & value, ii64 def)
{
    bool ok = true;
    ii64 result = StringToInt64(value, ok);
    return ok ? result : def;
}

//-----------------------------------------------------------------------------
//  StringToSignedInt64() -- Convert number to integer (autodetect base using C style)
//-----------------------------------------------------------------------------
//  '0x' prefix means convert as hex, '0' means convert as octal

s_ii64 StringToSignedInt64(const std::string & value, bool & ok)
{
    if (value.size()>=2 && value[0]=='0' && tolower(value[1])=='x')
        return StringToHex64(value, ok);

    if (value.size()>=1 && value[0]=='0')
        return StringToOct64(value, ok);

    return StringToSignedDec64(value, ok);
}

//-----------------------------------------------------------------------------
//  StringToSignedInt64() -- Convert number to integer (autodetect base using C style)
//-----------------------------------------------------------------------------
//  '0x' prefix means convert as hex, '0' means convert as octal

s_ii64 StringToSignedInt64(const std::string & value, s_ii64 def)
{
    bool ok = true;
    s_ii64 result = StringToSignedInt64(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToHex() -- Convert number to integer in base 16
//---------------------------------------------------------------------------

int StringToHex(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::string TmpStr;
    if (value.size() > 1 && value[1] == 'x')
        TmpStr = value.c_str() + 2;
    else
        TmpStr = value;

    std::stringstream strm;
    strm << TmpStr;

    unsigned int Result = 0;
    strm >> std::hex >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToHex() -- Convert number to integer in base 16
//---------------------------------------------------------------------------

int StringToHex(const std::string & value, int def)
{
    bool ok = true;
    int result = StringToHex(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToHex64() -- Convert number to integer in base 16
//---------------------------------------------------------------------------

ii64 StringToHex64(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::string TmpStr;
    if (value.size() > 1 && value[1] == 'x')
        TmpStr = value.c_str() + 2;
    else
        TmpStr = value;

    std::stringstream strm;
    strm << TmpStr;

    ii64 Result = 0;
    strm >> std::hex >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToHex64() -- Convert number to integer in base 16
//---------------------------------------------------------------------------

ii64 StringToHex64(const std::string & value, ii64 def)
{
    bool ok = true;
    ii64 result = StringToHex64(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToDec() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

int StringToDec(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::stringstream  strm;
    strm << value;

    int Result;
    strm >> std::dec >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToDec() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

int StringToDec(const std::string & value, int def)
{
    bool ok = true;
    int result = StringToDec(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToDec() -- Convert number to unsignd integer in base 10
//---------------------------------------------------------------------------

unsigned int StringToUDec(const std::string & value,unsigned int def)
{
    bool ok = true;
    unsigned int result = StringToUDec(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToUDec() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

unsigned int StringToUDec(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::stringstream  strm;
    strm << value;

    unsigned int Result;
    strm >> std::dec >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToDec64() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

ii64 StringToDec64(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::stringstream  strm;
    strm << value;

    ii64 Result;
    strm >> std::dec >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToDec64() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

ii64 StringToDec64(const std::string & value, ii64 def)
{
    bool ok = true;
    ii64 result = StringToDec64(value, ok);
    return ok ? result : def;
}


//---------------------------------------------------------------------------
//  StringToSignedDec64() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

s_ii64 StringToSignedDec64(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::stringstream  strm;
    strm << value;

    s_ii64 Result;
    strm >> std::dec >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToSignedDec64() -- Convert number to integer in base 10
//---------------------------------------------------------------------------

s_ii64 StringToSignedDec64(const std::string & value, s_ii64 def)
{
    bool ok = true;
    s_ii64 result = StringToDec64(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToOct() -- Convert number to integer in base 8
//---------------------------------------------------------------------------

int StringToOct(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::stringstream  strm;
    strm << value;

    int Result;
    strm >> std::oct >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToOct() -- Convert number to integer in base 8
//---------------------------------------------------------------------------

int StringToOct(const std::string & value, int def)
{
    bool ok = true;
    int result = StringToOct(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToOct64() -- Convert number to integer in base 8
//---------------------------------------------------------------------------

ii64 StringToOct64(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0;

    std::stringstream  strm;
    strm << value;

    ii64 Result;
    strm >> std::oct >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}


//---------------------------------------------------------------------------
//  StringToOct64() -- Convert number to integer in base 8
//---------------------------------------------------------------------------

ii64 StringToOct64(const std::string & value, ii64 def)
{
    bool ok = true;
    ii64 result = StringToOct64(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToDouble() -- Convert number to double precistion float
//---------------------------------------------------------------------------

double StringToDouble(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0.0;

    std::stringstream strm(value);

    double Result;
    strm >>  Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToDouble() -- Convert number to double precistion float
//---------------------------------------------------------------------------

double StringToDouble(const std::string & value, double def)
{
    bool ok = true;
    double result = StringToDouble(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  StringToFloat() -- Convert number to float
//---------------------------------------------------------------------------

float StringToFloat(const std::string & value, bool & ok)
{
    if (!value.size())
        return 0.0;

    std::stringstream strm(value);

    float Result;
    strm >> Result;

    if (strm.fail())
        ok = false;

    return Result;
}

//---------------------------------------------------------------------------
//  StringToFloat() -- Convert number to float
//---------------------------------------------------------------------------

float StringToFloat(const std::string & value, float def)
{
    bool ok = true;
    float result = StringToFloat(value, ok);
    return ok ? result : def;
}

//---------------------------------------------------------------------------
//  BinToHex() --
//  Convert the binary value in a buffer into a string that is its
//  hexadecimal representation.
//      buffer is a buffer of bytes that contains the binary value.
//      text returns a null-terminated string that represents the
//          value of buffer as a hexadecimal number.
//      size is the size of buffer. text needs to point to a sequence
//          of characters that has at least 2*size bytes because each
//          hexadecimal character represents two bytes.
//---------------------------------------------------------------------------

void BinToHex(const unsigned char * buffer, char * text, size_t size)
{
    static const char * digits = "0123456789ABCDEF";

    for (size_t i = 0; i < size; ++i)
        {
        *text++ = digits[*buffer >> 4];
        *text++ = digits[*buffer++ & 0xf];
        }
}

//---------------------------------------------------------------------------
//  HexToBin() --
//  Convert the hexadecimal string text to the binary value it represents.
//      text is a string representation of a hexadecimal value.
//      buffer returns the resulting value in binary.
//      size is the size of buffer. text needs to point to at least 2*size
//          hexadecimal characters, because each two hexadecimal characters
//          represent one byte.
//      Returns true of the entire string could be converted, false otherwise
//---------------------------------------------------------------------------

bool HexToBin(const char * text, char * buffer, size_t size)
{
    // Translation table
    static const char translate[] =
    { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,-1,-1,-1,-1,-1,-1,
     -1,10,11,12,13,14,15,-1,-1,-1,-1,-1,-1,-1,-1,-1,
     -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
     -1,10,11,12,13,14,15 };

    for (size_t i = 0; i < size; ++i)
        {
        if (!isxdigit(text[0]) || !isxdigit(text[1]))
            return false;

        char Hi = translate[*text++ - '0'];
        char Lo = translate[*text++ - '0'];
        *buffer++ = (Hi << 4) | Lo;
        }

    return true;
}

//---------------------------------------------------------------------------
//  LTrim() -- Remove leading white space
//---------------------------------------------------------------------------

std::string LTrim(const std::string & txt)
{
    size_t size = txt.size();
    if (!size)
        return txt;

    size_t i = 0;
	for (; i < txt.size(); ++i)
		{
		char c = txt[i];
		if (std::iscntrl(c) || std::isspace(c))
			continue;

		break;
		}

	return txt.substr(i, size-i);
}

//---------------------------------------------------------------------------
//  RTrim() -- Remove trailing white space
//---------------------------------------------------------------------------

std::string RTrim(const std::string & txt)
{
	size_t size = txt.size();
	if (!size)
		return txt;

	size_t i = size-1;
	for (; i != 0; --i)
		{
		char c = txt[i];
		if (std::iscntrl(c) || std::isspace(c))
			continue;

		break;
		}

	return txt.substr(0, i+1);
}

//---------------------------------------------------------------------------
//  Trim() -- Remove leading and trailing white space
//---------------------------------------------------------------------------

std::string Trim(const std::string & txt)
{
	string r(LTrim(RTrim(txt)));

	string result;
	for (string::const_iterator i = r.begin(); i != r.end(); ++i)
		{
//		if (std::iscntrl(*i))
//			continue;
		result += *i;
		}

	return result;
}

//---------------------------------------------------------------------------
//  RemoveTrailingNulls() -- Remove RemoveTrailingNulls white space
//---------------------------------------------------------------------------

std::string RemoveTrailingNulls(const std::string & txt)
{
	size_t size = txt.size();
	if (!size)
		return txt;

	size_t i = size-1;
	for (; i != 0; --i)
		{
		char c = txt[i];
		if (c == 0)
			continue;

		break;
		}

	return txt.substr(0, i+1);
}

//---------------------------------------------------------------------------
// UpperCase() --  Return upper case copy of string
//---------------------------------------------------------------------------

string UpperCase(const string & s)
{
    string result;
    for (size_t i = 0; i < s.size(); ++i)
        result += toupper(s[i]);

    return result;
}

//---------------------------------------------------------------------------
// LowerCase() --  Return lower case copy of string
//---------------------------------------------------------------------------

string LowerCase(const string & s)
{
    string result;
    for (size_t i = 0; i < s.size(); ++i)
        result += tolower(s[i]);

    return result;
}

//---------------------------------------------------------------------------
// DelimitedValue() --  Return text preceding delimiter
//---------------------------------------------------------------------------

string DelimitedValue(const string & s, const string & delim)
{
    if (!s.size())
        return s;

    size_t right = 0;
    do
        {
        right = s.find(delim, right+1);
        if (right == string::npos)
            return "";
        }
    while (!isdigit(s[right-1]));

    size_t left = right-1;
    while (left && isdigit(s[left]))
        --left;
    if (right <= left)
        return "";

    return s.substr(left, right-left+1);
}

//---------------------------------------------------------------------------
// PrefixedValue() --  Return text following delimiter
//---------------------------------------------------------------------------

string PrefixedValue(const string & s, const string & delim)
{
    if (!s.size())
        return s;

    size_t left = s.find_last_of(delim);
    if (left == string::npos)
        return "";

    size_t right = ++left;
	while ((right < s.size()) && isdigit(s[right]))
        ++right;

	return s.substr(left, right-left);
}

//---------------------------------------------------------------------------
// DeleteTrailing() --  Delete trailing characters from string
//---------------------------------------------------------------------------

string DeleteTrailing(const string & s, const string & delim)
{
    if (!s.size())
        return s;

    size_t left = s.find_last_of(delim);
    if (left == string::npos)
        return s;

    return s.substr(0, left-delim.size()+1);
}

//---------------------------------------------------------------------------
// QuotedStr() --  Returns the quoted version of a string
//    A single quote character is inserted at the beginning and end of s,
//    and each single quote character in the string is repeated.
//---------------------------------------------------------------------------

string QuotedStr(const string & s, const char quote)
{
    stringstream result;

    result << quote;

    for (size_t i = 0; i < s.size(); ++i)
        {
        if (s[i] == quote)
            result << quote;
        result << s[i];
        }

    result << quote;

    return result.str();
}

//---------------------------------------------------------------------------
// ExtractQuotedStr() --  Converts a quoted string into an unquoted string.
//    Removes the quote characters from the beginning and end of a quoted
//    string, and reduces pairs of quote characters within the string to a
//    single quote character.
//---------------------------------------------------------------------------

string ExtractQuotedStr(const std::string & s, char quote)
{
    string result(s);

    if (result.size() && (result[0] == quote))
        result.erase(0, 1);

    size_t end = result.size()-1;
    if (result.size() && (result[end] == quote))
        result.erase(end, 1);

    stringstream quotes;
    quotes << quote << quote;

    size_t idx = result.find(quotes.str());
    while (idx != string::npos)
        {
        result.erase(idx, 1);
        idx = result.find(quotes.str());
        }

    return result;
}

//---------------------------------------------------------------------------
// IncludeTrailingBackshash() --  Return string with trailing slash
//---------------------------------------------------------------------------

string IncludeTrailingBackshash(const std::string & s)
{
    stringstream msg;
    msg << s;
    size_t len = s.size();
    if (len && s[len-1] != '\\')
        msg << '\\';

    return msg.str();
}

//---------------------------------------------------------------------------
// Unspace() --  Trim whitespace within
//---------------------------------------------------------------------------

string Unspace(const string & s)
{
    string result;
    for (size_t i = 0; i < s.size(); ++i)
        result.push_back(iswspace(s[i]) ? '+' : s[i]);

    return result;
}

//---------------------------------------------------------------------------
// PortionDigits() --  Return true if string is primarily alphabetic
//---------------------------------------------------------------------------

float PortionDigits(const string & s)
{
    if (s[0] == '(')
        return false;

    int digits = 0;
    for (size_t i = 0; i < s.size(); ++i)
        digits += isdigit(s[i]) ? 1 : 0;

    float size = static_cast<float>(s.size());
    if (size)
        return static_cast<float>(digits) / size;
    else
        return 0.0;
}

//---------------------------------------------------------------------------
//  IsVariantOf() --
//---------------------------------------------------------------------------

bool IsVariantOf(const string & p1, const string & p2)
{
    size_t len = min<size_t>(p1.size(), p2.size());
    bool dash = (p1.find("-") != string::npos) || (p2.find("-") != string::npos);

    return (p1.substr(0, len) == p2.substr(0, len)) && dash;
}

//---------------------------------------------------------------------------
//  IsRugged() --
//---------------------------------------------------------------------------

bool IsRugged(const string & part)
{
    string p = UpperCase(part);
    if (p.find("-L1") !=string::npos
        || p.find("-L2") !=string::npos
        || p.find("-L3") !=string::npos
        || p.find("-L4") !=string::npos)
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
// IsAlpha() --  Return true if string is primarily alphabetic
//---------------------------------------------------------------------------

bool IsAlpha(const string & s)
{
    return  PortionDigits(s) <= 0.5f;
}

//---------------------------------------------------------------------------
// IsNumeric() --  Return true if string is primarily numeric
//---------------------------------------------------------------------------

bool IsNumeric(const string & s)
{
    return  PortionDigits(s) > 0.5f;
}

//---------------------------------------------------------------------------
// Within() --  Return true if value is within specified range
//---------------------------------------------------------------------------

bool Within(int low, int high, int value)
{
    return ((value >= low) && (value <= high)) ? true : false;
}

//---------------------------------------------------------------------------
// Category() --  Return numeric category code from part number
//---------------------------------------------------------------------------

int Category(const string & pn)
{
    string s = BaseNumber(pn);
    int base = atoi(s.c_str());
    return (base / 1000) * 1000;
}

//---------------------------------------------------------------------------
// BroadCategory() --  Return numeric category code from part number
//---------------------------------------------------------------------------

int BroadCategory(const string & pn)
{
    string s = BaseNumber(pn);
    int base = atoi(s.c_str());
    return (base / 10000) * 10000;
}

//---------------------------------------------------------------------------
//  ToDelimited -
//---------------------------------------------------------------------------

string ToDelimited(const string & txt, char delim)
{
    string result;

    StringList list;
    size_t cnt = list.Words(txt);
    if (!cnt)
        return txt;

    for (size_t i = 0; i < cnt-1; ++i)
        result += list.Pop() + delim;
    result += list.Pop();

    return result;
}

//---------------------------------------------------------------------------
//  BaseNumber() --  Return base portion of p/n
//---------------------------------------------------------------------------

string BaseNumber(const string & s)
{
    size_t pos = s.find('-');
    if (pos == string::npos)
        return s;
    else
        return s.substr(0, pos);
}

//---------------------------------------------------------------------------
//  BaseConfiguration() --  Return configuration portion of p/n
//---------------------------------------------------------------------------

string BaseConfiguration(const string & s)
{
    string result;
    size_t pos = s.find('-');
    if (pos == string::npos)
        {
        pos = 0;
        while ((pos < s.size()) && !isalpha(s[pos]))
            ++pos;
        }
    else
        {
        ++pos;
        // Index past configuration digits
        while ((pos < s.size()) && isdigit(s[pos]))
            ++pos;
        // Include configuration letter, if present
        while ( (s.size() > pos) && isalpha(s[pos]) && isalpha(s[pos+1]))
            ++pos;
        }

    if (pos == 0)
        return s;

    result = s.substr(0, pos);

    pos = s.find('-', pos);
    if (pos != string::npos)
        result += s.substr(pos, 3);

    return result;
}

//===========================================================================
//  CLASS Similarity  --  Compare strings via Levenshtein algorithm
//===========================================================================

//---------------------------------------------------------------------------
//  Similarity::GetSimilarity() --
//---------------------------------------------------------------------------

int Similarity::ComputeDistance(const string & s, const string & t)
{
    int n = static_cast<int>(s.length());
	int m = static_cast<int>(t.length());

    if(n == 0)
        return m;
    if(m == 0)
        return n;

    vector<vector<int> > distance(n+1);
    // Size the vectors in the 2.nd dimension. Unfortunately C++ doesn't
    // allow for allocation on declaration of 2.nd dimension of vec of vec
    for (int i = 0; i <= n; ++i)
        distance[i].resize(m+1);

    // Initialize
    for(int i=0; i<=n; ++i)
        distance[i][0]=i;
    for(int j=0; j<=m; ++j)
        distance[0][j]=j;

    // Find min distance
    for(int i=1; i<=n; ++i)
        {
        for(int j=1; j<=m; ++j)
            {
            const char s_i = s[i-1];
            const char t_j = t[j-1];

            int cost = (s_i == t_j ? 0 : 1);
            const int above = distance[i-1][j];
            const int left = distance[i][j-1];
            const int diag = distance[i-1][j-1];
            int cell = Min3(above+1, left+1, diag+cost);

            // Cover transposition, in addition to deletion, insertion and
            // substitution. This step is taken from:
            // Berghel, Hal ; Roach, David : "An Extension of Ukkonen's
            // Enhanced Dynamic Programming ASM Algorithm"
            // (http://www.acm.org/~hlb/publications/asm/asm.html)'
            if (i>2 && j>2)
                {
                int trans=distance[i-2][j-2]+1;

                if (s[i-2]!=t_j)
                    ++trans;
                if (s_i!=t[j-2])
                    ++trans;
                if (cell>trans)
                    cell=trans;
                }

            distance[i][j] = cell;
            }
        }

    return distance[n][m];
}

//---------------------------------------------------------------------------
//  Similarity::Compare() --
//---------------------------------------------------------------------------

float Similarity::Compare(const string & string1, const string & string2)
{
    float result;
    float dis = static_cast<float>(ComputeDistance(string1, string2));
    float maxLen = static_cast<float>(string1.size());

    if (maxLen < static_cast<float>(string2.size()))
		maxLen = static_cast<float>(string2.size());

    if (maxLen == 0.0f)
        result = 1.0f;
    else
        result = 1.0f - dis/maxLen;

    return result;
}

//---------------------------------------------------------------------------
//  Similarity::ICompare() -- Case insensitive compare
//---------------------------------------------------------------------------

float Similarity::ICompare(const string & src, const string & target)
{
    return Compare(LowerCase(src), LowerCase(target));
}

//---------------------------------------------------------------------------
//  Similarity::ICompareNum() -- Case insensitive compare
//---------------------------------------------------------------------------

float Similarity::IMatch(const string & src, const string & target)
{
    return (LowerCase(src) == LowerCase(target)) ? 1.0f : 0.0f;
}

//---------------------------------------------------------------------------
//  Similarity::Min3() --
//---------------------------------------------------------------------------

int Similarity::Min3(int i1, int i2, int i3)
{
    return std::min<int>(std::min<int>(i1, i2), i3);
}



}   // Innovative
