// StringSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef StringSupport_MbH
#define StringSupport_MbH

#include <string>
#include <DataTypes_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355 4996)
#endif

int StartsWith(const std::string& s1, const std::string& s2);
int StartsWith(const char * s1, const char * s2, size_t len);
int StringCompare(const std::string& s1, const std::string& s2);
int StringCompare(const char * s1, const char * s2, size_t len);

std::string IntToString(int value, int radix = 10, bool c_prefix_txt=true);
std::string UIntToString(unsigned int value, int radix = 10, bool c_prefix_txt=true);
std::string Int64ToString(ii64 value, int radix = 10, bool c_prefix_txt=true);
std::string Int64ToString(s_ii64 value, int radix = 10, bool c_prefix_txt=true);
std::string FloatToString(double value, int precision=3, bool fix=true);
inline std::string DoubleToString(double value, int precision=3, bool fix=true)
								{  return  FloatToString(value, precision, fix);  }

int StringToInt(const std::string & value, bool & ok);
int StringToInt(const std::string & value, int def=0);

int StringToHex(const std::string & value, bool & ok);
int StringToHex(const std::string & value, int def=0);

int StringToOct(const std::string & value, bool & ok);
int StringToOct(const std::string & value, int def=0);

int StringToDec(const std::string & value, bool & ok);
int StringToDec(const std::string & value, int def=0);

unsigned int StringToUDec(const std::string & value, bool & ok);
unsigned int StringToUDec(const std::string & value,unsigned int def=0);

ii64 StringToInt64(const std::string & value, bool & ok);
ii64 StringToInt64(const std::string & value, ii64 def=0);

ii64 StringToHex64(const std::string & value, bool & ok);
ii64 StringToHex64(const std::string & value, ii64 def=0);

ii64 StringToOct64(const std::string & value, bool & ok);
ii64 StringToOct64(const std::string & value, ii64 def=0);

ii64 StringToDec64(const std::string & value, bool & ok);
ii64 StringToDec64(const std::string & value, ii64 def=0);

s_ii64 StringToSignedInt64(const std::string & value, bool & ok);
s_ii64 StringToSignedInt64(const std::string & value, s_ii64 def=0);

s_ii64 StringToSignedDec64(const std::string & value, bool & ok);
s_ii64 StringToSignedDec64(const std::string & value, s_ii64 def=0);

float  StringToFloat(const std::string & value, bool & ok);
float  StringToFloat(const std::string & value, float def=0);

double StringToDouble(const std::string & value, bool & ok);
double StringToDouble(const std::string & value, double def=0);

void BinToHex(const unsigned char * buffer, char * text, size_t size);
bool HexToBin(const char * text, char * buffer, size_t size);

std::string LTrim(const std::string & txt);
std::string RTrim(const std::string & txt);
std::string Trim(const std::string & txt);
std::string RemoveTrailingNulls(const std::string & txt);

std::string UpperCase(const std::string & s);
std::string LowerCase(const std::string & s);
std::string DelimitedValue(const std::string & s, const std::string & delim);
std::string PrefixedValue(const std::string & s, const std::string & delim);
std::string DeleteTrailing(const std::string & s, const std::string & delim);
std::string QuotedStr(const std::string & s, const char quote);
std::string ExtractQuotedStr(const std::string & s, char quote);
std::string IncludeTrailingBackslash(const std::string & s);
std::string Unspace(const std::string & s);
float PortionDigits(const std::string & s);
bool IsVariantOf(const std::string & p1, const std::string & p2);
bool IsRugged(const std::string & part);
bool IsAlpha(const std::string & s);
bool IsNumeric(const std::string & s);
bool Within(int low, int high, int value);
std::string ToDelimited(const std::string & txt, char delim);
std::string BaseNumber(const std::string & s);
std::string BaseConfiguration(const std::string & s);
int Category(const std::string & pn);
int BroadCategory(const std::string & pn);

//===========================================================================
//  CLASS Similarity  --  Compare strings via Levenshtein algorithm
//===========================================================================

class Similarity
{
public:
    // Ctor
    Similarity()
        {  }

    // Methods
    float Compare(const std::string & string1, const std::string & string2);
    float ICompare(const std::string & src, const std::string & target);
    float IMatch(const std::string & src, const std::string & target);

private:
    // Methods
    int ComputeDistance(const std::string & s, const std::string & t);
    int Min3(int i1, int i2, int i3);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} //Innovative

#endif
