// StringVector_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
#include "StringVector_Mb.h"
#include <fstream>
#include "FileSupport_Mb.h"


namespace   Innovative
{

//=============================================================================
//  CLASS StringVector  --  Vector of Strings with file I/O
//=============================================================================

//-----------------------------------------------------------------------------
//  StringList::StringList() -- CTor
//-----------------------------------------------------------------------------

StringVector::StringVector()
{
}

//-----------------------------------------------------------------------------
//  StringVector::StringAt() -- return one of the strings in the vector
//-----------------------------------------------------------------------------

std::string StringVector::StringAt(size_t idx)
{
    return FList.at(idx);
}

//-----------------------------------------------------------------------------
//  StringVector::Clear() -- empty the vector
//-----------------------------------------------------------------------------

void StringVector::Clear()
{
    return FList.clear();
}

//-----------------------------------------------------------------------------
//  StringVector::Count() -- return # of strings in the vector
//-----------------------------------------------------------------------------

size_t StringVector::Count() const
{
    return FList.size();
}

//-----------------------------------------------------------------------------
//  StringVector::Add() -- Append string to list
//-----------------------------------------------------------------------------

void StringVector::Add(const std::string & item)
{
    FList.push_back(item);
}

//-----------------------------------------------------------------------------
//  StringVector::[]() --  Random access operator
//-----------------------------------------------------------------------------

std::string StringVector::operator[](int idx) const
{
    return FList[idx];
}

//-----------------------------------------------------------------------------
//  StringVector::IndexOf() --
//-----------------------------------------------------------------------------

int StringVector::IndexOf(const std::string & item)
{
    int result = -1;

    for (List::iterator i = FList.begin(); i != FList.end(); ++i)
        {
        ++result;
        if (*i == item)
            return result;
        }

    return -1;
}

//-----------------------------------------------------------------------------
//  StringVector::FileExists() -- return true if file exists...
//-----------------------------------------------------------------------------

bool StringVector::FileExists(const std::string & filename)
{
    std::ifstream in(filename.c_str());

    return in.good();
}

//-----------------------------------------------------------------------------
//  StringVector::LoadFromFile() -- Fill string list from the file
//-----------------------------------------------------------------------------

bool StringVector::LoadFromFile(const std::string & filename)
{
    std::ifstream in(filename.c_str());

    if (!in.good())
        return in.good();

    ii64  TotalSize( Innovative::FileSize(filename.c_str()));
    bool  DisplayLoadProgress( TotalSize != 0 );
    ii64  SizeRead(0);

    Clear();

    std::string line;
    int OldPercent = -1;
    while (!in.eof())
        {
        getline(in, line);

        FList.push_back(line);
        //
        //  Accumulate # characters and call progress ftn
        SizeRead += line.size();

        if (DisplayLoadProgress)
            {
            int Percent = static_cast<int>((SizeRead * 100) / TotalSize);
            if (Percent != OldPercent)
                {
                OldPercent = Percent;
				ProcessProgressEvent ppe(Percent);
                OnProgress.Execute(ppe);
                }
            }
        }

    if (DisplayLoadProgress)
        {
		ProcessProgressEvent ppe100(100);
        OnProgress.Execute(ppe100);
        }

	ProcessCompletionEvent pce;
    OnComplete.Execute(pce);
    return true;
}

} // namespace Innovative

