// SyncObject_Mb2.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SyncObject_MbH
#define SyncObject_MbH

#include <Osal_Mb.h>
#include <map>
#include <NonCopyable_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//   CLASS SyncObjectOwner
//==============================================================================

class SyncObjectOwner
{
public:
    virtual int  Key() = 0;
    virtual void Signal() = 0;
    virtual void Remove(class SyncObject *) = 0;
    
    virtual ~SyncObjectOwner()
    	{}
    
};

//==============================================================================
//   CLASS SyncObject
//==============================================================================
//
//  This class is a base class for synchronization objects. It allows common
//    functions and an interface to the MultipleObjects class

class SyncObject : public Noncopyable
{
    friend class MultipleObjects;

public:
	static const unsigned int Infinite=0xFFFFFFFF;
	//
    //  Member Functions
    //
    SyncObject();
    virtual ~SyncObject();

    // Multi-object support
    void  SignalOwners();
    void  AddOwner(SyncObjectOwner * owner);
    void  RemoveOwner(SyncObjectOwner * owner);

protected:
    // Generic methods
	virtual bool  WaitFor(unsigned int mS=Infinite) = 0;
    virtual bool  IsSignalled() = 0;

    // Interface functions
    //virtual bool  ShouldReassert() = 0;

private:
    // Data
    std::map<int, SyncObjectOwner *>   MyOwners;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
