// SyncObjects_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SyncObjects_MbH
#define SyncObjects_MbH

#include <Mutex_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS CriticalSection  --
//===========================================================================

class CriticalSection
{
public:
    CriticalSection()	{  }

    void Enter()
		{  lock.Acquire();  }
    void Leave()
		{  lock.Release();  }

protected:
    Mutex       lock;
};

//===========================================================================
//  CLASS CriticalSectionArea  --
//===========================================================================

class CriticalSectionArea
{
public:
    CriticalSectionArea(CriticalSection & criticalSection)
        : mCriticalSection( criticalSection )
        {  mCriticalSection.Enter();  }

    ~CriticalSectionArea()
        {  mCriticalSection.Leave();  }

private:
    CriticalSection &  mCriticalSection;
};

//===========================================================================
//  CLASS AtomicAccess  --
//===========================================================================

template <class T>
class AtomicAccess
{
public:
    AtomicAccess()  {  }

    T Decrement(T &value) const
		{  MutexedLock Lock(lock);  return --value;  }

    T DecrementToZero(T &value) const
		{  MutexedLock Lock(lock);  if (value!=0) --value; return value; }

    T Increment(T &value) const
		{  MutexedLock Lock(lock);  return ++value;  }

    T Exchange(T &value, T new_value) const
	{
    MutexedLock Lock(lock);
    
	T result = value;
    value = new_value;

    return result;
	}

    T Add(T &addend, T value) const
	{
    MutexedLock Lock(lock);

    T result = addend;
    addend += value;

    return result;
	}
    
    Mutex & LockObject() const
        {  return lock;  }

private:
    mutable Mutex lock;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
