// Synchronizer_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  A note on terminology:
//     When using windows programs with background threads, you often run
//     into the problem that UI operations are not safe to call from the
//     background. These operations must be called either from the context
//     of the main thread, or when the main thread is in a safe location.
//
//   Various means can be used to "thunk" into the main thread context. There
//     are two basic kinds, one where the request is 'Posted' and has not been
//     processed when the post call returns, or a 'Send' where the operation
//     thunked to the main thread is synchronized with the background so that the
//     sender waits for the completion of the thunked operation before continuing.
//
//   When sychronizing, it is possible that the sender may have to abort before
//     the response completes. The Synchronizer throws an exception if terminated
//     in this manner.


#ifndef Synchronizer_MbH
#define Synchronizer_MbH

#include <ThunkerIntf_Mb.h>
#include <MultipleObjects_Mb.h>
#include <LockQueue_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class ThunkerIntf;

//=============================================================================
//  CLASS Synchronizer  --  Block for synchronized execution in UI thread
//=============================================================================

class Synchronizer
{
public:
    // Ctor
    Synchronizer();
    virtual ~Synchronizer();

    // Methods
    void            Abort(bool wait=true);
    bool            Notify();
    void            Acknowledge();

// Pseudo-protected
public:
    ThunkerIntf *   mThunker;

protected:
    // Wait for completion, various sources
    enum { sAborted, sSyncCallbackDone, sAbortCompleted, sInSyncronization };
    MultipleObjects WaitOnCompletion;
    Event           Aborted;
    Event           SyncCallbackDone;
    Event           AbortCompleted;
	Event           InSyncronization;
	CriticalSection	Section;
};

//=============================================================================
//  CLASS BasicEventSynchronizer  --
//=============================================================================

class BasicEventSynchronizer : public OpenWire::EventSynchronizerIntf
{
public:
	virtual BasicEventSynchronizer *Clone() = 0;
	virtual ~BasicEventSynchronizer() {}

public:
    //
    //  EventSynchronizerIntf implementation
    virtual void Execute(OpenWire::EventSyncPair * event) = 0;
    //  Executes in UI thread context
    virtual void OnExecuted(OpenWire::Event & event);

protected:
    virtual void ImplementExecute(OpenWire::EventSyncPair * event, ThunkerIntf *thunker);

private:
    //  EventSynchronizerIntf implementation Member Data
    LockQueue<OpenWire::EventSyncPair *> mEventQueue;
};

//=============================================================================
//  CLASS EventSynchronizer  --  Block for synchronized execution in main UI thread
//=============================================================================

class EventSynchronizer : public BasicEventSynchronizer
{
    typedef BasicEventSynchronizer inherited;

public:
    EventSynchronizer();
    virtual ~EventSynchronizer();

// Pseudo-protected
public:
	virtual BasicEventSynchronizer *    Clone();

    //
    //  EventSynchronizerIntf implementation
    virtual void Execute(OpenWire::EventSyncPair * event);
    //
    //  Executes in UI thread context
    void OnExecuted(OpenWire::Event & event);

protected:
    Synchronizer *  mSynchronizer;
};

//=============================================================================
//  CLASS EventThunker  --  "Thunk" call into main thread, unlocked with caller
//=============================================================================

class EventThunker : public BasicEventSynchronizer
{
    typedef BasicEventSynchronizer inherited;

public:
    EventThunker();
    virtual ~EventThunker();

// Pseudo-protected
public:
	virtual BasicEventSynchronizer *    Clone();

    //
    //  EventSynchronizerIntf implementation
    virtual void Execute(OpenWire::EventSyncPair * event);

protected:
    ThunkerIntf *   mThunker;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative

#endif
