// SystemResponse_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SystemResponse_MbH
#define SystemResponse_MbH

#include <vector>
#include <complex>
#include <StimulusResponse_Mb.h>
#include <MLSequence_Mb.h>
#include <IppIntegerDG_Mb.h>
#include <Fourier_Mb.h>
#include <InverseFourier_Mb.h>
#include <Trace_Mb.h>
#include <Stats_Mb.h>
#include <Events_Mb.h>
#include <ProcessEvents_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS SystemMeasurementEvent  --  Fractional completion indicator
//===========================================================================

class SystemMeasurementEvent : public OpenWire::Event
{
public:
    const double Result;

    SystemMeasurementEvent(const double result)
        : Result(result)
        {
        }
};

//===========================================================================
//  CLASS SystemResponse  --
//===========================================================================

class SystemResponse
{
public:
    SystemResponse(StimulusResponse & io);
    virtual ~SystemResponse()  {  }

    // Methods
    void        Smooth(const double octaves);
    void        Gate(const unsigned int start, const unsigned int stop);

    double      SampleRate() const
        {  return mSampleRate;  }

    // Length of input data and impulse response
    size_t      Samples() const
        {  return mSequence.numElements();  }

    // Raw input data
    double      Sample(const unsigned int i) const
		{  return (IppFloatDG(mResponse))[i];  }

    // Impulse impulse response for displaying so Gate can be set
    double      h(const unsigned int i) const
		{  return mImpulse[i];  }

    // Length of output data
    size_t Points() const
        {  return mFrequency.size();  }

    // Frequencies
	double f(const unsigned int i) const
        {  return mFrequency[i];  }

	// Frequency response
	std::complex<double> H(const unsigned int i) const
		{  return mAmplitude[i];  }

	// Statistics of frequency response
	const Statistics & Stats() const
		{  return mStats;  }

    // Events
    OpenWire::ThunkedEventHandler<ProcessProgressEvent>   OnProgress;
    OpenWire::ThunkedEventHandler<ProcessStatusEvent>   OnStatus;
    OpenWire::ThunkedEventHandler<SystemMeasurementEvent> OnComplete;

protected:
    typedef std::vector<double> DoubleArray;

	// Methods
	void                            GenerateStimulusBuffer();
	void                            CalculateImpulseResponse();

	// Data
	unsigned char                       mChannels;
	bool                                mSweepMode;
	double                              mSampleRate;
    bool                                mStereo;
    size_t                              mOutput;

	std::vector<std::complex<double> >  mAmplitude;
	DoubleArray                         mImpulse;
	DoubleArray                         mFrequency;
	MLSequence                          mSequence;
	Statistics							mStats;

	Buffer                              mStimulus;
	Buffer                              mResponse;
	Fourier                             mFft;
	InverseFourier                      mIfft;
	StimulusResponse &                  mIo;
	const double						mPi;

	Trace                               mDebug;
};

//===========================================================================
//  CLASS MaximumLengthSequence  --
//===========================================================================

class MaximumLengthSequence : public SystemResponse
{
public:
    MaximumLengthSequence(StimulusResponse & io);

    void Initialize(const double sampleRate, const bool stereo,
                    const unsigned int order);

    //Mls - make measurement and calculate response
    //returns rms signal for level checking
    void Measure(const double amplitude, const unsigned int averages,
                 const bool reverse = false);

private:
	// Methods
	void Correlate(unsigned int average, const bool reverse);
    void HandleOnStimulusProgress(ProcessProgressEvent & event);
    void HandleOnStimulusComplete(OpenWire::NotifyEvent & event);

    // Data
    unsigned int                        mOrder;
    unsigned int                        mAverages;
    unsigned int                        mAverage;
    bool                                mReverse;
};

//===========================================================================
//  CLASS SineSweep  --
//===========================================================================

class SineSweep : public SystemResponse
{
public:
    SineSweep(StimulusResponse & io);

    void Initialize(const double sampleRate, const bool stereo,
                    const double minStepTime);

    //Sweep - make measurement and calculate response
    //returns rms signal for level checking
    void Measure(const double amplitude, const double start,
                 const double stop, const double pointsPerDecade,
                 const bool reverse = false);

private:
	// Methods
	double SineFit(unsigned int sweepSamples, const bool reverse);
    void GaussEliminate(unsigned int n, DoubleArray my, DoubleArray & x);
    void HandleOnStimulusProgress(ProcessProgressEvent & event);
    void HandleOnStimulusComplete(OpenWire::NotifyEvent & event);

    // Data
	double                              mMinTime;
	std::vector<long>                   mSweepPositions;
    unsigned int                        mSweepSamples;
    bool                                mReverse;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace Innovative

#endif
