// SystemSupportMb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifdef BORLANDC
#include <mem.h>
#endif

#include <SystemSupport_Mb.h>
#include <Osal_Mb.h>
#include <ctype.h>
#include <stdlib.h>
#include <limits.h>
#include <CommandLineIntf_Mb.h>
#include <iostream>
#include <fstream>

using namespace InnovativeKernel::Osal;

namespace   Innovative
{

//==============================================================================
//  CLASS Application  --  Global application information
//==============================================================================

//---------------------------------------------------------------------------
//  Application::Application() -- Ctor
//---------------------------------------------------------------------------

Application::Application()
{
    CommandLineArguments args(GetCommandLineArguments());

    if (args.size())
        Spec.Full(args[0]);
}

//---------------------------------------------------------------------------
//  Application() -- Retreive application executable path
//---------------------------------------------------------------------------

std::string Application::Name() const
{
    return Spec.Name();
}

//---------------------------------------------------------------------------
//  Application() -- Retreive application executable path
//---------------------------------------------------------------------------

std::string Application::Path() const
{
    return Spec.Path();
}

//---------------------------------------------------------------------------
//  Sleep() -- Delay, in mS
//---------------------------------------------------------------------------

void Sleep(unsigned int mS)
{
    OsSupport::Sleep(mS);
}

//---------------------------------------------------------------------------
//  uSleep() -- Delay, in uS
//---------------------------------------------------------------------------

void uSleep(unsigned int uS)
{
    OsSupport::uSleep(uS);
}

//==============================================================================
//  CLASS TimeStampLogger
//==============================================================================

//---------------------------------------------------------------------------
//  TimeStampLogger::Mark() - Capture current timestamp to memory
//---------------------------------------------------------------------------
void TimeStampLogger::Mark()
{
    if(enabled)
    {
        if(curIndex < maxLogEntry)
        {
            curIndex++;
            tsQueue.push(hrClock.Now());
        }
    }
}

//---------------------------------------------------------------------------
//  TimeStampLogger::Save() - Write timestamp logs to file
//---------------------------------------------------------------------------
void TimeStampLogger::Save()
{
    if(enabled)
    {
        cout<<"saving "<<curIndex <<" entries into file: " << logFile<<endl;

        fstream fout(logFile.c_str(), std::fstream::out);
        fout.unsetf ( std::ios::floatfield );
        fout.setf( std::ios::fixed, std::ios::floatfield );
        fout.precision(6);

        //ts
        for(unsigned int i=0; i<curIndex; i++)
        {
            fout << tsQueue.front() << endl;
            tsQueue.pop();
        }

        curIndex = 0;
        fout.close();
    }
}

//---------------------------------------------------------------------------
//  TimeStampLogger::Start() - start the timer
//---------------------------------------------------------------------------
void TimeStampLogger::Start()
{
    if(enabled)
    {
        hrClock.Start();
    }
}

//---------------------------------------------------------------------------
//  TimeStampLogger::Stop() - Stop the timer and log elasped time
//---------------------------------------------------------------------------
void TimeStampLogger::Stop()
{
    if(enabled)
    {
        if(curIndex < maxLogEntry)
        {
            curIndex++;
            tsQueue.push(hrClock.Stop());
        }
    }
}

//---------------------------------------------------------------------------
//  TimeStampLogger::Warn() - Info user of state changes
//---------------------------------------------------------------------------
void TimeStampLogger::Warn()
{
    if(enabled)
    {
         cout<<"Time stamp: Enable"<<endl;
         cout<<"Log file name: "<< logFile <<endl;
    }
    else
    {
        cout<<"Time stamp: Disable"<<endl;
    }
}

}   // Innovative
