// SystemSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef SystemSupport_MbH
#define SystemSupport_MbH

#include <string>
#include <vector>
#include <assert.h>
#include <FileSupport_Mb.h>
#include <CommandLineIntf_Mb.h>
#include <StopWatch_Mb.h>
#include <queue>

using namespace std;

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355 4996)
#endif

//==============================================================================
//  CLASS Application  --  Global application information
//==============================================================================

class Application
{
public:
    // Ctor
	Application();

    // Methods
    std::string     Name() const;
    std::string     Path() const;

private:
    PathSpec        Spec;
};

void        Sleep(unsigned int mS);
void        uSleep(unsigned int uS);

//==============================================================================
//  CLASS SingltonTemplate  --  Make any class into a singlton
//==============================================================================

template<class T>
class SingltonTemplate
{
public:
    static T* Instance()
    {
        if(!m_pInstance)
            m_pInstance = new T;
        assert(m_pInstance != NULL);
        return m_pInstance;
    }

protected:
    SingltonTemplate();
    ~SingltonTemplate();
    static T* m_pInstance;

private:
    SingltonTemplate(SingltonTemplate const&);  //DO NOT IMPLEMENT
    SingltonTemplate& operator=(SingltonTemplate const&);  //DO NOT IMPLEMENT
};

template <class T> T* SingltonTemplate<T>::m_pInstance=NULL;

//==============================================================================
//  CLASS TimeStampLogger  --  Provide timestamping and logging capabilities
//==============================================================================

class TimeStampLogger
{
public:

    static const unsigned int MAX_ENTRIES = 5000;

    TimeStampLogger()
    {
        enabled = false;
        logFile = "tsLog.txt";
        curIndex = 0;
        maxLogEntry = MAX_ENTRIES;
    }

    TimeStampLogger(size_t max, string logFileName )
    {
        enabled = false;
        curIndex = 0;
        logFile = logFileName;
        maxLogEntry = max;
    }

    void Enable()
    {
        enabled = true;
        Warn();
    }

    void Disable()
    {
        enabled = false;
        Warn();
    }

    bool State()
    {
        return enabled;
    }

    void Mark();
    void Save();
    void Start();
    void Stop();
    void Warn();

private:
    ii64 curIndex;
    bool enabled;
    string logFile;
    StopWatch hrClock;
    std::queue <double> tsQueue;
    size_t maxLogEntry;
};

typedef SingltonTemplate<TimeStampLogger> TimeStampLoggerSingleton;


#ifdef _MSC_VER
	#pragma warning(pop)	
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} //Innovative

#endif
