// TargetInfo_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

//
// This class holds target information like firmware revision, S/N, etc.
// A communication thread called from Sbc6713e class needs to be executed to
// fill up the data members of this class.
//
// There is also a method to set this same info on the target and is also taken
// care by Sbc6713e class.

#include <TargetInfo_Mb.h>

namespace   Innovative
{


//----------------------------------------------------------------------
// Constructor
//----------------------------------------------------------------------

TargetInfo::TargetInfo()
    : FSerialNoBoardRevLoaded('N'), FStaticIpInfoLoaded('N')
{
}

//----------------------------------------------------------------------
// SerialNoBoardRev, records Board SN and Rev in the "host data base".
// This info comes as a pair from the target, so we use only one  setter.
//----------------------------------------------------------------------

void TargetInfo::SerialNoBoardRev( short sn, char rev)
{
    FSerialNoBoardRevLoaded = 'Y';
    FSerialNumber[1] = sn >> 8;
    FSerialNumber[0] = sn & 0xFF;

    FBoardRevision = rev;
}

//----------------------------------------------------------------------
// SerialNumber(), returns stored SN.
//----------------------------------------------------------------------

short TargetInfo::SerialNumber()
{
    if (FSerialNoBoardRevLoaded == 'Y')
        {
        short rtn = FSerialNumber[1];
        rtn <<= 8;
        short lo = FSerialNumber[0];
        lo &= 0xFF;
        rtn |= lo;
        return rtn;
        }
    else
        return 0;
}

//----------------------------------------------------------------------
// BoardRevision, returns stored board revision.
//----------------------------------------------------------------------

char TargetInfo::BoardRevision()
{
    return FBoardRevision;
}

//----------------------------------------------------------------------
// StaticIpInfo, records static Ip info from Target, This info comes as a
// block of data from target, so we have only one setter.
// *** We expect properly null terminated strings.
//----------------------------------------------------------------------

void TargetInfo::StaticIPInfo(const char * TargetIpAddr, const char * TargetIpMask,
                              const char * GatewayIPAddr, const char * DnsServerIpAddr,
                              const char * DomainName )
{
    int ApprovedCount(0);

	#ifdef _MSC_VER
        #pragma warning(push)	
        #pragma warning(disable : 4996)
    #endif

	if( strlen(TargetIpAddr) < 20 )
        {
        strcpy(FTargetIPAddr, TargetIpAddr);
        ApprovedCount++;
        }

    if( strlen(TargetIpMask) < 20 )
        {
        strcpy(FTargetIPMask, TargetIpMask);
        ApprovedCount++;
        }

    if( strlen(GatewayIPAddr) < 20 )
        {
        strcpy(FGatewayIpAddress, GatewayIPAddr);
        ApprovedCount++;
        }

    if( strlen(DnsServerIpAddr) < 20 )
        {
        strcpy(FDnsServerIPAddress, DnsServerIpAddr);
        ApprovedCount++;
        }

    if( strlen(DnsServerIpAddr) < 80 )
        {
        strcpy(FDomainName, DomainName);
        ApprovedCount++;
        }
    if (ApprovedCount == 5)
        FStaticIpInfoLoaded = 'Y';
    else
        FStaticIpInfoLoaded = 'N';

	#ifdef _MSC_VER
        #pragma warning(pop)	
    #endif
}

//-----------------------------------------------------------------------------
// StaticIPInfoLoaded(), flag, tells if Target static IP Info is in host.
//-----------------------------------------------------------------------------

bool TargetInfo::StaticIpInfoLoaded() const
{
    if (FStaticIpInfoLoaded == 'Y')
        return true;
    else
        return false;
}

//-----------------------------------------------------------------------------
// UseStaticIP(), set flag, tells if Target uses static IP Info.
//-----------------------------------------------------------------------------

void TargetInfo::UseStaticIp(bool state)
{
    if (state)
        FUseStaticIp = 'Y';
    else
        FUseStaticIp = 'N';
}

//-----------------------------------------------------------------------------
// UseStaticIP(), read flag, tells if Target uses static IP Info.
//-----------------------------------------------------------------------------

bool TargetInfo::UseStaticIp()
{
    if (FUseStaticIp == 'Y')
        return true;
    else
        return false;

}

//----------------------------------------------------------------------
//  TargetIpAddress(), Read stored Target IP Address.
//----------------------------------------------------------------------
const char *  TargetInfo::TargetIpAddress() const
{
    if (StaticIpInfoLoaded())
        return reinterpret_cast<const char*>(FTargetIPAddr);
    else
        return "";
}

//----------------------------------------------------------------------
//  TargetIpMask(), Read stored Target IP mask.
//----------------------------------------------------------------------

const char * TargetInfo::TargetIpMask() const
{
    if (StaticIpInfoLoaded())
        return FTargetIPMask;
    else
        return "";
}

//----------------------------------------------------------------------
//  GatewayIpAddress(), Read stored Target Gateway Ip address.
//----------------------------------------------------------------------

const char * TargetInfo::GatewayIpAddress() const
{
    if (StaticIpInfoLoaded())
        return FGatewayIpAddress;
    else
        return "";
}

//----------------------------------------------------------------------
//  DnsServerIPAddress(), Read stored Target DNS Ip address.
//----------------------------------------------------------------------

const char * TargetInfo::DnsServerIPAddress() const
{
    if (StaticIpInfoLoaded())
        return FDnsServerIPAddress;
    else
        return "";
}

//----------------------------------------------------------------------
//  DomainName(), Read stored Target Domain Name.
//----------------------------------------------------------------------

const char * TargetInfo::DomainName() const
{
    if (StaticIpInfoLoaded())
        return FDomainName;
    else
        return "";
}

//----------------------------------------------------------------------
//  TargetFwareInfo(), stores fware info read from Target.
//----------------------------------------------------------------------

void TargetInfo::TargetFwareInfo(int ClientFwareVer, int IntfLogicVer,
                                 int LanLogicVer)
{
    FClientFwareVersion = ClientFwareVer;
    FIntfLogicVersion = IntfLogicVer;
    FLanLogicVersion = LanLogicVer;
}

//----------------------------------------------------------------------
//  IntfLogicVersion(), reads INTF logic vesion read from Target.
//----------------------------------------------------------------------

int TargetInfo::IntfLogicVersion()
{
    return FIntfLogicVersion;
}

//----------------------------------------------------------------------
//  LanLogicVersion(), reads LAN logic version read from Target.
//----------------------------------------------------------------------

int TargetInfo::LanLogicVersion()
{
    return FLanLogicVersion;
}

//----------------------------------------------------------------------
//  ClientFwareVersion(), reads Client firmware version read from Target.
//----------------------------------------------------------------------

int TargetInfo::ClientFwareVersion()
{
    return FClientFwareVersion;
}

//----------------------------------------------------------------------
//  BootApplication(), sets flag, boot customer target program.
//----------------------------------------------------------------------

void TargetInfo::BootApplication( bool state )
{
    if(state)
        FBootApplication = 'Y';
    else
        FBootApplication = 'N';
}

//----------------------------------------------------------------------
//  BootApplication(), reads flag, boot customer target program.
//----------------------------------------------------------------------
bool TargetInfo::BootApplication()
{
    if(FBootApplication == 'Y')
        return true;
    else
        return false;
}

}// namespace
