// TargetInfo_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

//
// This class holds target information like firmware revision, S/N, etc.
// A communication thread called from Sbc6713e class needs to be executed to
// fill up the data members of this class.
//
// There is also a method to set this same info on the target and is also taken
// care by Sbc6713e class.

#ifndef TargetInfo_MbH
#define TargetInfo_MbH

#include <string>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class TargetInfo
{
    public:
        TargetInfo();

        // S/N and Board rev..
        void    SerialNoBoardRev( short sn, char rev );
        short   SerialNumber();
        char    BoardRevision();

        // Ip Info.
        void    StaticIPInfo( const char * TargetIpAddr, const char * TargetIpMask,
                              const char * gatewayIPAddr, const char * dnsServerIpAddr,
                              const char * domainName );
        bool    StaticIpInfoLoaded() const;
        void    UseStaticIp(bool state);
        bool    UseStaticIp();
        const char *  TargetIpAddress() const;
        const char *  TargetIpMask() const;
        const char *  GatewayIpAddress() const;
        const char *  DnsServerIPAddress() const;
        const char *  DomainName() const;

        // Boot customer app.
        void    BootApplication( bool state );
        bool    BootApplication();

        // Firmware revision.
        void    TargetFwareInfo( int ClientFwareVer, int IntfLogicVer, int LanLogicVer );
        int     IntfLogicVersion();
        int     LanLogicVersion();
        int     ClientFwareVersion();

    private:
        // Data.
        char    FSerialNoBoardRevLoaded;
        char 	FSerialNumber[2];
    	char 	FBoardRevision;

        char    FStaticIpInfoLoaded;
        char 	FUseStaticIp;
        char 	FTargetIPAddr[20];          // properly null terminated strings.
        char 	FTargetIPMask[20];
        char 	FGatewayIpAddress[20];
        char 	FDnsServerIPAddress[20];
        char 	FDomainName[80];            //

        char 	FBootApplication;

        int     FIntfLogicVersion;
        int     FLanLogicVersion;
        int     FClientFwareVersion;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}
#endif
