// ThreadSafeCountingQueue_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef ThreadSafeCountingQueue_MbH
#define ThreadSafeCountingQueue_MbH

#include <queue>
#include <SyncObjects_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS  ThreadSafeCountingQueue  --  Queue with access locking protection
//==============================================================================

template <class TElement> class ThreadSafeCountingQueue
{
    static const int MaxCount = 1000;

public:
    //
    //  Member Functions
    ThreadSafeCountingQueue();
    ThreadSafeCountingQueue(int size);

    // Status events
    Semaphore       Read;
    Semaphore       Write;
    int             Level;

    bool  empty() const
        {
        CriticalSectionArea guard(lock);
        return Queue.empty();
        }

    int  depth() const
        {
        CriticalSectionArea guard(lock);
        return Queue.size();
        }

    int  room() const
        {
        CriticalSectionArea guard(lock);
        return maxsize - Queue.size();
        }

    void  clear()
        {
        CriticalSectionArea guard(lock);
        while (!Queue.empty())
            Queue.pop();
        // All empty, none full
        Read.Clear();
        Write.Clear();
        Write.Release(maxsize);
        }

    void  push(TElement value)
        {
        Write.Acquire();
        CriticalSectionArea guard(lock);
        Queue.push(value);
        Read.Release();
        ++Level;
        }

    TElement  front()
        {
        CriticalSectionArea guard(lock);
        //TElement value = Queue.front();
        //return value;
        return Queue.front();
        }

    TElement  pop(bool interlock=true)
        {
        // if interlock is true, acquire the semaphore before accessing queue
        if (interlock)
            Read.Acquire();
        CriticalSectionArea guard(lock);
        TElement value = Queue.front();
        Queue.pop();
        Write.Release();
        --Level;
        return value;
        }

private:
    //
    //  Member Data
    mutable CriticalSection  lock;
    std::queue<TElement>     Queue;
    int             maxsize;

};

//------------------------------------------------------------------------------
//  constructor for class ThreadSafeCountingQueue
//------------------------------------------------------------------------------

template <class T>
ThreadSafeCountingQueue<T>::ThreadSafeCountingQueue()
    : Read(MaxCount, 0), Write(MaxCount, MaxCount), Level(0), maxsize(MaxCount)
{
}

//------------------------------------------------------------------------------
//  constructor for class ThreadSafeCountingQueue
//------------------------------------------------------------------------------

template <class T>
ThreadSafeCountingQueue<T>::ThreadSafeCountingQueue(int size)
    : Read(0), Write(size), Level(0), maxsize(size)
{
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
