// ThreadSafeLog_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef ThreadSafeLog_MbH
#define ThreadSafeLog_MbH

#include <vector>
#include <SyncObjects_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS  ThreadSafeLog  --  Internal Quick Log Class
//==============================================================================
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4800)		
#endif
class ThreadSafeLog
{
public:
    enum LogMode { KeepFirst, KeepLatest };
    ThreadSafeLog(unsigned int max, LogMode mode=KeepLatest)
        : max_size(max), Mode(mode) {}

    void  clear()
        {
        CriticalSectionArea guard(lock);
        Log.clear();
        }
    void  append(int code, int data)
        {
        CriticalSectionArea guard(lock);
        if (Mode==KeepFirst)
            {
            // keep first mode
            if (Log.size()<max_size)
                Log.push_back((code<<24) | (data&0xFFFFFF));
            }
        else
            {
            // keep latest mode
            if (Log.size()>=max_size)
                Log.clear();
            Log.push_back((code<<24) | (data&0xFFFFFF));
            }
        }
    void  copy_to( std::vector<int> & vec )
        {
        CriticalSectionArea guard(lock);
        vec = Log;
        }
private:
    //
    //  Member Data
    mutable CriticalSection  lock;
    std::vector<int>         Log;
    unsigned int             max_size;
    const bool               Mode;
};
#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
