// ThreadSafeQueue_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef ThreadSafeQueue_MbH
#define ThreadSafeQueue_MbH

#include <queue>
#include <stack>
#include <SyncObjects_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS  ThreadSafeStack  --  Stack with access locking protection
//==============================================================================

template <class TElement> class ThreadSafeStack
{
public:
    //
    //  Member Functions
    ThreadSafeStack()  {};

    bool  empty() const
        {
        CriticalSectionArea guard(lock);
        return Queue.empty();
        }

    size_t size() const
        {
        CriticalSectionArea guard(lock);
        return Queue.size();
        }

    void  clear()
        {
        CriticalSectionArea guard(lock);
        while (!Queue.empty())
            Queue.pop();
        }

    void  push(TElement value)
        {
        CriticalSectionArea guard(lock);
        Queue.push(value);
        }

    TElement  top()
        {
        CriticalSectionArea guard(lock);
        return Queue.top();
        }

    const TElement&  top() const
        {
        CriticalSectionArea guard(lock);
        return Queue.top();
        }

    void  pop()
        {
        CriticalSectionArea guard(lock);
        Queue.pop();
		}

	//
	//  Member Data
	mutable CriticalSection  lock;
	std::stack<TElement>     Queue;
};


//==============================================================================
//  CLASS  ThreadSafeQueue  --  Queue with access locking protection
//==============================================================================

template <class TElement> class ThreadSafeQueue
{
public:
    //
    //  Member Functions
    ThreadSafeQueue()  {};

    bool  empty() const
        {
        CriticalSectionArea guard(lock);
        return Queue.empty();
        }

    size_t depth() const
        {
        CriticalSectionArea guard(lock);
        return Queue.size();
        }

    size_t size() const
        {
        return depth();
        }

    void  clear()
        {
        CriticalSectionArea guard(lock);
        while (!Queue.empty())
            Queue.pop();
        }

    void  push(TElement value)
        {
        CriticalSectionArea guard(lock);
        Queue.push(value);
        }

	TElement  front()
        {
        CriticalSectionArea guard(lock);
        //TElement value = Queue.front();
        //return value;
//        if(Queue.empty())
//            {
//            TElement Junk;
//            return Junk;
//            }
        return Queue.front();
        }

    void  pop()
        {
        CriticalSectionArea guard(lock);
        Queue.pop();
		}

	//
	//  Member Data
	mutable CriticalSection  lock;
	std::queue<TElement>     Queue;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // end namespace Innovative


#endif
