// Thread_Mb2.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2007 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Thread_Mb.h>
#include <Poco/Thread.h>

namespace   Innovative
{

//=============================================================================
//  CLASS Thread  --  OS Thread object wrapper
//=============================================================================

//---------------------------------------------------------------------------
// Thread::Thread() --
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
Thread::Thread(const std::string &name)
    :  FPriority(tpNormal),
       Handle(new Poco::Thread (name)),
       //Handle(ThreadFunc, (void *)this, pName),
       Running(false)
{
    //    Debug.Enabled(false);
    //    Debug.Prefix(name);
    //    Debug.Log("Thread::Thread");

    //  Launch Thread, will block on resume event...
    Handle->start(*this);
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
// Thread::~Thread() --
//---------------------------------------------------------------------------

Thread::~Thread()
{
//    Debug.Log("Thread::~Thread");
}

//---------------------------------------------------------------------------
//  Thread::Run() -- Full-lifetime execute functionality
//---------------------------------------------------------------------------

void Thread::run()
{
    eResume.WaitFor();
    Running = true;

    Init();

    Execute();

    Done();
}

//---------------------------------------------------------------------------
//  Thread::Init() -- Pre-execute functionality
//---------------------------------------------------------------------------

void Thread::Init()
{
    Actions.AddObject(eTerminate, sTerminate);
}

//---------------------------------------------------------------------------
//  Thread::Done() -- Post-execute functionality
//---------------------------------------------------------------------------

void Thread::Done()
{
}

//---------------------------------------------------------------------------
//  Thread::Terminated() --
//---------------------------------------------------------------------------

bool Thread::Terminated()
{
    return eTerminate.Check();
}

//------------------------------------------------------------------------------
//  Thread::Shutdown() -- Terminate the thread, then wait for thread to finish
//------------------------------------------------------------------------------

void Thread::Shutdown(bool wait)
{
    if (!Running)
        return;

    //Debug.Log("Thread::Shutdown");

    //  Terminate Thread Loop
    Terminate();

    if (wait) {
        WaitForCompletion();
    }
}

//---------------------------------------------------------------------------
//  Thread::Terminate() --
//---------------------------------------------------------------------------

void Thread::Terminate()
{
    if (!Running)
        return;

    //Debug.Log("Thread::Terminate");

	eTerminate.Set();
}

//---------------------------------------------------------------------------
//  Thread::WaitForCompletion() --
//---------------------------------------------------------------------------

bool Thread::WaitForCompletion()
{
    if (!Running)
        return true;

    //Debug.Log("Thread::WaitForCompletion");

    Handle->join();

    Running = false;

    return true;
}

//---------------------------------------------------------------------------
//  Thread::LoadDefaultEventsInto() --
//---------------------------------------------------------------------------

void Thread::LoadDefaultEventsInto(MultipleObjects & multi_object)
{
    multi_object.AddObject(eTerminate, sTerminate);
}

//---------------------------------------------------------------------------
//  Thread::Priority() --  Set thread priority
//---------------------------------------------------------------------------

void Thread::Priority(ThreadPriority value)
{
    FPriority = value;

    switch (value)
        {
        case tpLowest:
            Handle->setOSPriority(Poco::Thread::PRIO_LOWEST);
            break;
        case tpLower:
            Handle->setOSPriority(Poco::Thread::PRIO_LOW);
            break;
        default:
        case tpNormal:
            Handle->setOSPriority(Poco::Thread::PRIO_NORMAL);
            break;
        case tpHigher:
            Handle->setOSPriority(Poco::Thread::PRIO_HIGH);
            break;
        case tpHighest:
            Handle->setOSPriority(Poco::Thread::PRIO_HIGHEST);
            break;
        }
}

//---------------------------------------------------------------------------
//  Thread::Priority() --  Get thread priority
//---------------------------------------------------------------------------

Thread::ThreadPriority Thread::Priority() const
{
    return FPriority;
}

//---------------------------------------------------------------------------
//  Thread::Resume() --
//---------------------------------------------------------------------------

void Thread::Resume()
{
    eResume.Set();
}

//---------------------------------------------------------------------------
//  Thread::ShouldReassert() --
//---------------------------------------------------------------------------

bool Thread::ShouldReassert()
{
    return false;
}

//---------------------------------------------------------------------------
//  Thread::AddObject() --  Add to multiple sync object
//---------------------------------------------------------------------------

bool Thread::AddObject(SyncObject * object, int Case)
{
    return Actions.AddObject(object, Case);
}

//---------------------------------------------------------------------------
//  Thread::AddObject() --  Add to multiple sync object
//---------------------------------------------------------------------------

bool Thread::AddObject(SyncObject & object, int Case)
{
    return AddObject(&object, Case);
}

//---------------------------------------------------------------------------
//  Thread::WaitFor() --
//---------------------------------------------------------------------------

int Thread::WaitFor(unsigned int mS)
{
    int result = Actions.WaitFor(mS);
    if (result == MultipleObjects::Invalid)
        return sTerminate;
    else
        return result;
}

//---------------------------------------------------------------------------
//  Thread::GetName() --
//---------------------------------------------------------------------------

const std::string Thread::GetName()
{
    return Handle->name();
}

//---------------------------------------------------------------------------
//  Thread::SetCpuAffinity() --
//---------------------------------------------------------------------------

void Thread::SetCpuAffinity(int cpu)
{
    FAffinity = cpu;
   // Handle->setAffinity(FAffinity);
}

//---------------------------------------------------------------------------
//  Thread::XXX_DebugLog() --
//---------------------------------------------------------------------------

void Thread::XXX_DebugLog(int /*type*/, int /*code*/)
{

}

} // namespace Innovative
