//  TiBmDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <TiBmDeviceMap_Mb.h>
#include <Exception_Mb.h>
#include <StringSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{


//===========================================================================
//  CLASS TiBm_DeviceToScheme  -- Generic Device callbacks for Scheme
//===========================================================================

class TiBm_DeviceToScheme : public ITiBm_DeviceToScheme
{
public:
    TiBm_DeviceToScheme(TiBusmasterDeviceMap * owner) : Owner(owner) {}

    virtual Event * IsrEvent() {  return Owner->IsrEvent(); }
    virtual size_t  InterruptDepth() { return Owner->InterruptDepth(); }
    virtual int     InterruptAck() { return Owner->InterruptAck(); }
    virtual void    Log(int type, int data) {  Owner->DebugLog.append(type, data); }
    virtual void    Log(int type, size_t data) 
        {  Log(type, static_cast<int>(data));  }
    virtual ii32 *  BusMasterAddress() const { return Owner->DeviceDriver->IncomingBusMasterAddress(); }
    virtual ii32 *  BusMasterPhysicalAddress() const { return Owner->DeviceDriver->IncomingBusMasterPhysicalAddress(); }
    virtual ii32    BusMasterSizeWords() const { return Owner->DeviceDriver->IncomingBusMasterSizeWords(); }
    virtual void    BusMasterSizeWords(ii32 value)  { Owner->DeviceDriver->BusMasterSizeWords(value); }
    virtual void    WriteBlockToDspMemory(const ii32 *src, int dst, int size)
                        {  Owner->WriteBlockToDspMemory(src, dst, size); }
    virtual void    ReadBlockFromDspMemory(int src, ii32 *dst, int size)
                        {  Owner->ReadBlockFromDspMemory(src, dst, size); }
    virtual void    DspInt()  {  Owner->DspInt();  }

private:
    TiBusmasterDeviceMap  *  Owner;
};

//===========================================================================
//  CLASS TiBusmasterDeviceMap  -- Base class for Packet mode BM devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class TiBusmasterDeviceMap
//---------------------------------------------------------------------------
//
//  TI Paged Memory Constants
const int   PagedMemoryShiftBits = 22;
const int   PagedMemoryMask = (1 << 10) - 1;
const int   PagedMemoryOffsetMask = (1 << PagedMemoryShiftBits) - 1;

namespace
{
//
//  CPU needs a tiny boot application to be downloaded on Boot. This is the
//     code for this application...
unsigned int    StdMinimalBootApp[] =
{
    0x01800228,  0x0294a0fa,  0x0200002a,  0x0200ca6a,
    0x029002f6,  0x00000028,  0x0000ca68,  0x01800274,
    0x0ffffd10,  0x00008000,  0x00008000,  0x00008000,
    0x00008000,  0x00008000,  0x00008000,  0x00008000
};

const int  StdMinimalBootAppSize = sizeof(StdMinimalBootApp)/sizeof(unsigned int);

} // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class TiBusmasterDeviceMap
//---------------------------------------------------------------------------
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

TiBusmasterDeviceMap::TiBusmasterDeviceMap(
                    InnovativeKernel::TiCpuDeviceDriver * driver,
                    TiBmDeviceMapRegisters & regs )
    :  IUsesStandardC64xCpu(*this),
       DeviceDriver(driver), 
       IntScheme(0), 
       Prom               (CpuRegisterMemory, regs.EepromAddress_Addr,
                                              regs.EepromData_Addr,
                                              regs.EepromCtlStatus_Addr),
       RegAddr(regs),
       FDspPciSymbol("PCIDATA_base"),
       DebugLog(10000),
       HostStatusReg      (CpuRegisterMemory, regs.StatusRegister_Addr),
       HostToDspControlReg(CpuRegisterMemory, regs.HostToDspCtlRegister_Addr),
       DspPageReg         (CpuRegisterMemory, regs.DspPageRegister_Addr),
       CpuEmifAReg        (CpuRegisterMemory, regs.CpuEmifARegister_Addr, 9)
{
    ITiBm_DeviceToScheme * DtS = new TiBm_DeviceToScheme(this);
    IntScheme = new TiBusmasterInterruptScheme(DtS);
}

//---------------------------------------------------------------------------
//  destructor for class TiBusmasterDeviceMap
//---------------------------------------------------------------------------

TiBusmasterDeviceMap::~TiBusmasterDeviceMap()
{
    Close();
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::BoardCount()  --  Query # of enumerated boards
//---------------------------------------------------------------------------

size_t TiBusmasterDeviceMap::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Open()
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::Open()
{
    if( DeviceDriver->Opened() )
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(DeviceDriver->PortMemoryAddress());
    CpuRegisterMemory.Link(DeviceDriver->CpuRegisterMemoryAddress());

    ClearTargetInterrupt();
    //
    //  Clear Int Status queue & set up extension region
    DeviceDriver->ClearInterruptStatusQueue();

    IntScheme->Open();

    HostStatusReg.EnableInterrupt(true);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Close()
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::Close()
{
    HostStatusReg.EnableInterrupt(false);
    IntScheme->Close();
    DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  TiBusmasterDeviceMap::Target()
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Warm()
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::Warm()
{
    //
    //  Clear Int Status queue & set up extension region
    DeviceDriver->ClearInterruptStatusQueue();

    ClearTargetInterrupt();
    HostStatusReg.EnableInterrupt(true);

    DebugLog.clear();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Reset()
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::Reset()
{
    WarmReset();
    Warm();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::WarmReset() -- Warm reset
//---------------------------------------------------------------------------

void TiBusmasterDeviceMap::WarmReset()
{
    //
    //  Set register bits DSPINT to 0, WARMRESET to 1
    HostToDspControlReg.Value(0x01);
    Sleep(1);
}

//------------------------------------------------------------------------------
//  TiBusmasterDeviceMap::StartDspProgram() -- start DSP after download
//------------------------------------------------------------------------------

void TiBusmasterDeviceMap::StartDspProgram()
{
    DspInt();
}

//------------------------------------------------------------------------------
//  TiBusmasterDeviceMap::DspInt() -- signal DSP interrupt
//------------------------------------------------------------------------------

void TiBusmasterDeviceMap::DspInt()
{
    //
    //  Set register bits DSPINT to 1, WARMRESET to 0
    HostToDspControlReg.Value(0x02);
}

//------------------------------------------------------------------------------
//  TiBusmasterDeviceMap::ConfigureTargetSdram() -- MIF settings for download
//------------------------------------------------------------------------------

void TiBusmasterDeviceMap::ConfigureTargetSdram()
{
    const int   emifGblCtl = 0;
    const int   emifCe1 = 1;
    const int   emifCe0 = 2;
    const int   emifCe2 = 4;
    const int   emifCe3 = 5;
    const int   emifSdCtl = 6;
    const int   emifSdTim = 7;
    const int   emifSdExt = 8;

    // EMIFA
    CpuEmifAReg[emifGblCtl] = 0x00012020;  // global control

    CpuEmifAReg[emifCe0] = 0x000000D0;
    CpuEmifAReg[emifCe1] = 0x000000D0;
    CpuEmifAReg[emifCe2] = 0x000000D0;
    CpuEmifAReg[emifCe3] = 0x000000D0;

    CpuEmifAReg[emifSdTim] = 0x000003e8;   // SDRAM timing
    CpuEmifAReg[emifSdExt] = 0x000d8dcb;   // SDRAM extension
    CpuEmifAReg[emifSdCtl] = 0x57338000;   // SDRAM control
    Sleep(5);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::CopyOut() --
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::CopyOut(const MatadorMessage & mail)
{
    IntScheme->CopyOut(mail);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::CopyIn()
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::CopyIn(MatadorMessage & mail)
{
    IntScheme->CopyIn(mail);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::CopyOut() --
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::CopyOut(const Buffer & packet)
{
    IntScheme->CopyOut(packet);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::CopyIn()
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::CopyIn(Buffer & packet)
{
    IntScheme->CopyIn(packet);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::SendStatus() --
//---------------------------------------------------------------------------

PciTransferInfo::TransferStatus  TiBusmasterDeviceMap::SendStatus()
{
    return IntScheme->SendStatus();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::RecvStatus()
//---------------------------------------------------------------------------

PciTransferInfo::TransferStatus  TiBusmasterDeviceMap::RecvStatus()
{
    return IntScheme->RecvStatus();
}

//------------------------------------------------------------------------------
//   TiBusmasterDeviceMap::ClearTargetInterrupt() -- Clear pending interrupt
//------------------------------------------------------------------------------

void TiBusmasterDeviceMap::ClearTargetInterrupt()
{
    //
    //  Clear interrupt leaving enable bit alone
    HostStatusReg.IntSource = true;
}

//------------------------------------------------------------------------------
//  TiBusmasterDeviceMap::WriteBlockToDspMemory() -- Data to Dsp target memory
//------------------------------------------------------------------------------

void TiBusmasterDeviceMap::WriteBlockToDspMemory(const ii32*  src, int dst, int size)
{
    //
    //  Calculate Page and Offset from Dest address
    int Page = (dst >> PagedMemoryShiftBits) & PagedMemoryMask;
    int Offset = dst & PagedMemoryOffsetMask;
    Offset /= sizeof(int);

	const int   NumberOfTries = 100;
	//
	//  Set CPU page
	int errors = 0;
	for (int tries=0; tries<NumberOfTries; ++tries)
		{
		DspPageReg = Page;
		int DspPageRegVal = DspPageReg;
		if (DspPageRegVal == Page)
			break;

		++errors;
		}
	if (errors)
		Ods.Log("TiBusmasterDeviceMap::WriteBlockToDspMemory HPI Page Register Write Failures: ", errors);

	//
	//  Now load the data through Paged access region
	//  ...apparently this is unreliable so retry until successful
	errors = 0;
	for (int i = 0; i < size; ++i)
		{
		for (int tries = 0; tries < NumberOfTries; tries++)
			{
			//
			//  Write the data to memory
			PortMemory.Write(Offset + i, src[i]);

			//  PagedCpuIo.PortAt(Offset+i, src[i]);
			//
			//  Check if write worked, if so stop trying
			if (src[i] == PortMemory.Read(Offset + i))
				//  if (src[i]==PagedCpuIo.PortAt(Offset+i))
				break;

			++errors;
			}
		}
	if (errors)
		Ods.Log("TiBusmasterDeviceMap::WriteBlockToDspMemory HPI Mem Write Failures: ", errors);
}

//------------------------------------------------------------------------------
//  TiBusmasterDeviceMap::ReadBlockFromDspMemory() -- read Dsp target memory
//------------------------------------------------------------------------------

void TiBusmasterDeviceMap::ReadBlockFromDspMemory(int src, ii32*  dst, int size)
{

	//
	//  Calculate Page and Offset from Dest address
	int Page = (src >> PagedMemoryShiftBits) & PagedMemoryMask;
	int Offset = src & PagedMemoryOffsetMask;
	Offset /= sizeof(int);

	//
	//  Set CPU page
	const int   NumberOfTries = 100;
	int errors = 0;
	for (int tries=0; tries<NumberOfTries; tries++)
		{
		DspPageReg = Page;
		int DspPageRegVal = DspPageReg;
		if (DspPageRegVal == Page)
			break;

		++errors;
		}
	if (errors)
		Ods.Log("TiBusmasterDeviceMap::ReadBlockFromDspMemory HPI Page Register Write Failures: ", errors);

	//
	//  Now read the data through Paged access region
	volatile ii32 val = 0;
	errors = 0;
	for (int i = 0; i < size; ++i)
		{
		for (int tries = 0; tries < NumberOfTries; ++ tries)
			{
			dst[i] = PortMemory.Read(Offset + i);
			val = dst[i];
			dst[i] = PortMemory.Read(Offset + i);
			if(val == dst[i])
				break;

			++errors;
			}
		}
	if (errors)
		Ods.Log("TiBusmasterDeviceMap::ReadBlockFromDspMemory HPI Mem Read Failures: ", errors);
}

//-----------------------------------------------------------------------------
//  TiBusmasterDeviceMap::LoadPciMemOffset()
//-----------------------------------------------------------------------------

void TiBusmasterDeviceMap::LoadPciMemOffset(int addr)
{
	IntScheme->LoadPciMemOffset(addr);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Error() -- throw error string
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::Error( std::string msg )
{
	throw Innovative::MalibuException(msg);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::RegisterStreamPlugIn() -- attach to a streaming object
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::RegisterStreamPlugIn( ITiBmStreamManager *plug_in )
{
	StreamManager = plug_in;
	IntScheme->RegisterStreamPlugIn(StreamManager);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::DumpLog() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::DumpLog(std::vector<int> & vec)
{
	DebugLog.copy_to(vec);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::IsrEvent() - Get event for the VISR
//---------------------------------------------------------------------------

Event *  TiBusmasterDeviceMap::IsrEvent()
{
	return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::InterruptDepth() - Get depth of VISR queue
//---------------------------------------------------------------------------

size_t TiBusmasterDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::InterruptDepth() - Acknowledge interrupt
//---------------------------------------------------------------------------

int TiBusmasterDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Eeprom() -  Write to config ROM
//---------------------------------------------------------------------------

void TiBusmasterDeviceMap::Eeprom(int addr, short value)
{
    Prom.Write(addr, value);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::Eeprom() -   Read from config ROM
//---------------------------------------------------------------------------

short TiBusmasterDeviceMap::Eeprom(int addr)
{
    return Prom.Read(addr);
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::BusMasterSizeWords() -
//---------------------------------------------------------------------------

ii32 TiBusmasterDeviceMap::BusMasterSizeWords() const
{
    return DeviceDriver->IncomingBusMasterSizeWords();
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::BusMasterSizeWords() -
//---------------------------------------------------------------------------

void TiBusmasterDeviceMap::BusMasterSizeWords(ii32 value)
{
    DeviceDriver->BusMasterSizeWords(value);
}

//===========================================================================
//  CLASS TiBusmasterDeviceMap::HostToDspControlRegister  --
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for TiBusmasterDeviceMap::HostToDspControlRegister
//---------------------------------------------------------------------------

TiBusmasterDeviceMap::HostToDspControlRegister::HostToDspControlRegister(AddressingSpace&  space,
    int offset) 
    : inherited(space, offset), 
      WarmReset(*this, 0), DspInt(*this, 1)
{
}

//===========================================================================
//  CLASS TiBusmasterDeviceMap::HostStatusRegister  -- 
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for TiBusmasterDeviceMap::HostStatusRegister -
//---------------------------------------------------------------------------

TiBusmasterDeviceMap::HostStatusRegister::HostStatusRegister(AddressingSpace&  space,
    int offset) 
    : inherited(space, offset), 
      IntSource(*this, 0), IntAvailable(*this, 1), 
      IntAM(*this, 2), ConfigError(*this, 3), EERead(*this, 4)
{
}

//---------------------------------------------------------------------------
//  TiBusmasterDeviceMap::HostStatusRegister::EnableInterrupt() -
//---------------------------------------------------------------------------

void  TiBusmasterDeviceMap::HostStatusRegister::EnableInterrupt(bool state)
{
    if (state)
        IntAM.Clear();    // set bit to 0 to enable interrupts
    else
        IntAM.Set();      // set bit to 1 to disable interrupts
}

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

//===========================================================================
//  CLASS IUsesStandardC64xCpu  -- CPU COFF interface
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IUsesStandardC64xCpu
//---------------------------------------------------------------------------

IUsesStandardC64xCpu::IUsesStandardC64xCpu(TiBusmasterDeviceMap & cdm )
    :  CpuCDM(cdm)
{

    Coff.Image.OnMemWrite.SetEvent(this, &IUsesStandardC64xCpu::HandleMemWrite);
    //  ...Forward Coff Image progress events
    Coff.Image.OnProgress.SetEvent(this, &IUsesStandardC64xCpu::HandleOnProgress);
    Coff.Image.OnComplete.SetEvent(this, &IUsesStandardC64xCpu::HandleOnComplete);
    Coff.Image.OnLoadMsg.SetEvent(this, &IUsesStandardC64xCpu::HandleOnLoadMsg);

}

//---------------------------------------------------------------------------
//  destructor for class IUsesStandardC64xCpu
//---------------------------------------------------------------------------

IUsesStandardC64xCpu::~IUsesStandardC64xCpu()
{
    Coff.Shutdown();
}

//---------------------------------------------------------------------------
//  IUsesStandardC64xCpu::Open() -- Open-time behavior...
//---------------------------------------------------------------------------

void  IUsesStandardC64xCpu::Open()
{
}

//---------------------------------------------------------------------------
//  IUsesStandardC64xCpu::Cold() -- Reset the CPU
//---------------------------------------------------------------------------

void  IUsesStandardC64xCpu::Cold()
{
    //  Toggle reset on DSP
    CpuCDM.WarmReset();

    Sleep(100);
    //
    //  Load EMIF settings for SDRAM
    CpuCDM.ConfigureTargetSdram();
}

//---------------------------------------------------------------------------
//  IUsesStandardC64xCpu::Boot() -- Boot the CPU
//---------------------------------------------------------------------------

void  IUsesStandardC64xCpu::Boot()
{
    Cold();
    //
    //  Poke test application into memory
    CpuCDM.WriteBlockToDspMemory(StdMinimalBootApp, 0, StdMinimalBootAppSize);
    //
    //  Trigger DSP to start execution
    CpuCDM.StartDspProgram();
}

//---------------------------------------------------------------------------
//  IUsesStandardC64xCpu::DownloadCoff() -- Perform Coff Download
//---------------------------------------------------------------------------

void  IUsesStandardC64xCpu::DownloadCoff(const std::string & Filename)
{
    Cold();

    Coff.Load(Filename);
}

//---------------------------------------------------------------------------
//  IUsesStandardC64xCpu::CoffImage() -- Return Coff Object
//---------------------------------------------------------------------------

CoffFile &  IUsesStandardC64xCpu::CoffImage()
{
    return Coff.Image;
}

//---------------------------------------------------------------------------
//  IUsesStandardC64xCpu::HandleMemWrite() --
//------------------------------------------------------------------------------

void IUsesStandardC64xCpu::HandleMemWrite(Innovative::MemWriteEvent & event)
{
    const unsigned int * src = reinterpret_cast<const unsigned int*>(event.Buffer);
    int size = event.Bytes/sizeof(int);

    std::vector<unsigned int> dst(size);

    const int MaxIterations = 10;
    int Iteration;
    for (Iteration = 0; Iteration < MaxIterations; ++Iteration)
        {
        CpuCDM.WriteBlockToDspMemory(src, event.Addr, size);
        CpuCDM.ReadBlockFromDspMemory(event.Addr, &dst[0], size);

        int errors = 0;
        for (int i = 0; i < size; ++i)
            errors += src[i] != dst[i];

        if (!errors)
            break;
        }

    if (!Iteration)
        return;

	if (Iteration < MaxIterations)
		Ods.Log("Recoverable HPI write failure!");
	else
        throw MalibuException("Unrecoverable HPI write failure!");
}

//------------------------------------------------------------------------------
//  IUsesStandardC64xCpu::HandleOnProgress() --
//------------------------------------------------------------------------------

void IUsesStandardC64xCpu::HandleOnProgress(ProcessProgressEvent & event)
{
    OnCoffLoadProgress.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesStandardC64xCpu::HandleOnComplete() --
//------------------------------------------------------------------------------

void IUsesStandardC64xCpu::HandleOnComplete(ProcessCompletionEvent & event)
{
    //
    //  In order to have a safe place for shared data between target and host we
    //    require a known segment be created and used in the target application.
    //    Its address is determined by obtaining the address for "PCIDATA_base"
    //    from the COFF symbol table.  The host will access this region
    //    using the page memory region of CPU.
    //  Get the address of PCI memory region, load into DSP Driver
    std::string PciMemRegion = Coff.Image.SymbolList()[CpuCDM.FDspPciSymbol];
    int         PciMemOffset = StringToHex(PciMemRegion); 
    CpuCDM.LoadPciMemOffset(PciMemOffset);

    //
    //  Trigger DSP to start execution
    CpuCDM.StartDspProgram();

    OnCoffLoadComplete.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesStandardC64xCpu::HandleOnLoadMsg() --
//------------------------------------------------------------------------------

void IUsesStandardC64xCpu::HandleOnLoadMsg(ProcessStatusEvent & event)
{
    OnCoffLoadMessage.Execute(event);
}


} // namespace Innovative


