//  TiBmDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef TiBmDeviceMap_MbH
#define TiBmDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <vector>
#include <SyncObjects_Mb.h>
#include <TiBmStreamSupport_Mb.h>
#include <TiBmInterruptScheme_Mb.h>
#include <ThreadSafeLog_Mb.h>
#include <BaseboardInterfaces_Mb.h>
#include <CoffThread_Mb.h>
#include <Trace_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Event;
class TiBusmasterDeviceMap;

//==============================================================================
//  CLASS  C64xEEProm  --  DSP EEProm interface
//==============================================================================

class C64xEEProm
{
public:
    C64xEEProm(AddressingSpace & addr_space, unsigned int addr,
               unsigned int data, unsigned int ctl)
        : AddrReg(addr_space, addr),
          DataReg(addr_space, data),
          CtlReg(addr_space, ctl),
          StatusReg(addr_space, ctl)
    {
    }

    // Methods
    void Write(int addr, short value)
        {
        Address(-1);
        Command(0);
        Address(addr);
        Command(3);
        Data(value);
        Command(1);  }

    short Read(int addr)
        {
        Address(addr);
        Command(2);
        return Data();
        }

private:
#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

	class StatusRegister : public Register
    {
    public:
        StatusRegister(AddressingSpace &space, int offset)
            : Register(space, offset), Ready(*this, 2),
              Size(*this, 3, 3), ExtRom(*this, 6),
              Error(*this, 7), Done(*this, 8)
            {  }

        RegisterBit       Ready;
        RegisterBitGroup  Size;
        RegisterBit       ExtRom;
        RegisterBit       Error;
        RegisterBit       Done;
    };

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

    ShadowRegister  AddrReg;
    Register        DataReg;
    Register        CtlReg;
    StatusRegister  StatusReg;

    // Support methods
    void  Address(int addr)
        {  Ready();  AddrReg.Value(addr);  }

    void Data(short value)
        {  Ready();  DataReg.Value(value);  }
    short Data()
        {  Ready();  return DataReg.Value() & 0xffff;  }

    void Command(int value)
        {  Ready();  CtlReg.Value(value);  }

    bool IsReady()
        {  return StatusReg.Ready.Value() ? true : false;  }
    int Size()
        {
        unsigned int idx = StatusReg.Size.Value();

        int Result[] = {0, 0x400, 0x800, 0x1000, 0x4000, 0, 0, 0};
        return Result[idx];
        }
    bool ExtRom()
        {  return StatusReg.ExtRom.Value() ? true : false;  }
    bool Error()
        {  return StatusReg.Error.Value() ? true : false;  }
    bool Done()
        {  return StatusReg.Done.Value() ? true : false;  }

    void Ready()
        {
        for (unsigned int i = 0; (i < 1000) && !IsReady(); ++i)
            Innovative::uSleep(1);
        }
};

//===========================================================================
//  CLASS IUsesStandardC64xCpu  -- CPU COFF interface
//===========================================================================

class IUsesStandardC64xCpu :  public IUsesOnboardCpu
{
public:
	//
	// Methods
	IUsesStandardC64xCpu(TiBusmasterDeviceMap & cdm);
	~IUsesStandardC64xCpu();

	virtual void  Boot();
	virtual void  DownloadCoff(const std::string & Filename);

	virtual CoffFile & CoffImage();

protected:
	//
	//  Member Data
	TiBusmasterDeviceMap &   CpuCDM;
	CoffThread               Coff;
	Trace					 Ods;

	void  Open();

	// Support Methods
	void  Cold();

	void  HandleMemWrite(MemWriteEvent & event);
	void  HandleOnProgress(ProcessProgressEvent & event);
	void  HandleOnLoadMsg(ProcessStatusEvent & event);
	void  HandleOnComplete(ProcessCompletionEvent & event);

};

//===========================================================================
//  CLASS TiBmDeviceMapRegisters  -- Addresses for Device Registers
//===========================================================================

class TiBmDeviceMapRegisters
{
public:
    unsigned int   StatusRegister_Addr;
    unsigned int   DspPageRegister_Addr;
    unsigned int   HostToDspCtlRegister_Addr;
    unsigned int   CpuEmifARegister_Addr;
    unsigned int   EepromAddress_Addr;
    unsigned int   EepromData_Addr;
    unsigned int   EepromCtlStatus_Addr;
};

//===========================================================================
//  CLASS TiBusmasterDeviceMap  -- Base class for Packet mode BM devices
//===========================================================================

class TiBusmasterDeviceMap :  public ITiBmStreamDeviceInternals,
                              public IUsesStandardC64xCpu
{
    friend class TiBm_DeviceToScheme;
    friend class IUsesStandardC64xCpu;
    
public:
    //
    //  Member Functions
    TiBusmasterDeviceMap(InnovativeKernel::TiCpuDeviceDriver * driver,
                         TiBmDeviceMapRegisters & regs );
    virtual ~TiBusmasterDeviceMap();
    //
    //  Properties
    std::string     DspPciSymbol();

    size_t          BoardCount();
    virtual void    Open();
    virtual void    Close();

    int      Target();
    void     Target(int target);

    void     WriteBlockToDspMemory( const ii32 *src, int dst, int size );
    void     ReadBlockFromDspMemory( int src, ii32 *dst, int size );

    void  Reset();
    void  Warm();
    void  WarmReset();
    void  DspInt();
    void  ConfigureTargetSdram();
    void  ClearTargetInterrupt();

    void  RegisterStreamPlugIn( class ITiBmStreamManager *plug_in );
    void  LoadPciMemOffset(int addr);
	//
    //  Ti Stream Internals Interface
    virtual int    DeviceTargetID() {  return Target();  };
    virtual void   CopyOut(const MatadorMessage & mail);
    virtual void   CopyIn(MatadorMessage & mail);
    virtual void   CopyOut(const Buffer & packet);
    virtual void   CopyIn(Buffer & packet);
    virtual PciTransferInfo::TransferStatus SendStatus();
    virtual PciTransferInfo::TransferStatus RecvStatus();

	virtual ii32    BusMasterSizeWords() const;
	virtual void    BusMasterSizeWords(ii32 value);

    // Properties
    void    Eeprom(int addr, short value);
    short   Eeprom(int addr);

protected:
    //
    //  Member Data
    InnovativeKernel::TiCpuDeviceDriver *  DeviceDriver;

    AddressingSpace            PortMemory;
    AddressingSpace            CpuRegisterMemory;

    TiBusmasterInterruptScheme *      IntScheme;
    ITiBmStreamManager *              StreamManager;
    //
    //  Member Functions
   // void  PciLoopbackMode(int mode);
   // int   PciLoopbackMode();

private:
    //
    //  Member Data
    class C64xEEProm            Prom;    
    TiBmDeviceMapRegisters      RegAddr;
    std::string                 FDspPciSymbol;
    //
    //  Member Functions
    void     StartDspProgram();

    Event *  IsrEvent();
    size_t   InterruptDepth();
    int      InterruptAck();
    void     Error( std::string msg );

    //
    //  Debug
	ThreadSafeLog DebugLog;
	//
    //  Registers
    class HostStatusRegister : public Register
    {
      typedef Register inherited;

    public:
      RegisterBit       IntSource;
      RegisterBit       IntAvailable;
      RegisterBit       IntAM;
      RegisterBit       ConfigError;
      RegisterBit       EERead;

    public:
      HostStatusRegister( AddressingSpace &space, int offset );

      void EnableInterrupt(bool state);
    };

    class HostToDspControlRegister : public Register
    {
      typedef Register inherited;

    public:
      RegisterBit       WarmReset;
      RegisterBit       DspInt;

    public:
      HostToDspControlRegister( AddressingSpace &space, int offset );
    };

    //
    //  Registers
    HostStatusRegister          HostStatusReg;
    HostToDspControlRegister    HostToDspControlReg;
    Register                    DspPageReg;
    RegisterArray<Register>     CpuEmifAReg;

public:
    //  Debug
    int             ReadStatus();

    ii32 *          XmitPhysicalAddr() {  return IntScheme->XmitRgnBasePhys;  };
    ii32 *          RecvPhysicalAddr() {  return IntScheme->RecvRgnBasePhys;  };
    ii32 *          XmitAddr() {  return IntScheme->XmitRgnBase;  };
    ii32 *          RecvAddr() {  return IntScheme->RecvRgnBase;  };
    unsigned int    XmitRegionSize() {  return IntScheme->XmitRgnSize;  };
    unsigned int    RecvRegionSize() {  return IntScheme->RecvRgnSize;  };

    void            Log(int code, int data) {  DebugLog.append(code, data);  };
    //  Debug info words
    void  DumpLog(std::vector<int> & vec);

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
