// TiBmInterruptScheme_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <TiBmInterruptScheme_Mb.h>
#include <Buffer_Mb.h>
#include <StartStopThread_Mb.h>
#include <NotifyThread_Mb.h>
#include <sstream>
#include <HardwareRegister_Mb.h>
#include <TiBmStreamSupport_Mb.h>
#include <SignalQueue_Mb.h>
#include <PciTransferStruct_Mb.h>
#include <Mutex_Mb.h>
#include <MatadorMessage_Mb.h>
#include <MemoryUtils_Mb.h>

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

namespace Innovative
{


namespace TiBusmasterIntScheme
{
//===========================================================================
//  CLASS VirtualIsrThread  -- ISR handling thread for TiBusmaster Int scheme
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of  class VirtualIsrThread
//---------------------------------------------------------------------------

class VirtualIsrThread : public StartStopThread
{
    typedef StartStopThread inherited;

    friend class Innovative::TiBusmasterInterruptScheme;
    
public:
    VirtualIsrThread( ITiBm_DeviceToScheme * device );

    void  Clear()
            {   }
    void  RegisterStreamPlugIn( ITiBmStreamManager *plug_in );
    void  LoadPciMemOffset(int addr);
    void  AckMailbox();
    void  SetMailboxAvailable();

    struct  BmRegionInfo
    {
        ii32 *          BaseAddr;
        ii32 *          PhysAddr;
        unsigned int    SizeInWords;
    };

    BmRegionInfo  Incoming;
    BmRegionInfo  Outgoing;

protected:
    enum { sDriverEvent=inherited::sLast, sLocalIsrQ, sLast };
    Event &                     DriverEvent;
	Mutex                       InterruptMutex;
    ITiBmStreamManager *        StreamManager;
    ITiBm_DeviceToScheme *      StreamDevice;
    SignalQueue<unsigned int>   LocalIsrQ;
    //
    //  Block Transfer System
    PciTransferInfo      Image;
    PciTransferInfo *    TargetImage;

    PciTransferInfo::TransferStatus  SendStatus;
    PciTransferInfo::TransferStatus  RecvStatus;

    int     PciMemOffset;
    size_t  MaxInterruptDepth;
    size_t  MaxLocalIsrDepth;
    //
    // Methods.
    virtual void Init();
    virtual void SingleExecution();

    void  DispatchInterrupt( int IntStatus );
    void  TargetInterrupt(int Code, int Channel, int Data);
    bool  HtoTIntFlagBusy();
    void  ClearTtoHIntFlag();
};

//---------------------------------------------------------------------------
//  housekeeping for class VirtualIsrThread
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class VirtualIsrThread
//---------------------------------------------------------------------------

VirtualIsrThread::VirtualIsrThread( ITiBm_DeviceToScheme * device )
    : inherited("TiBmInterruptScheme::VirtualIsrThread"),
      DriverEvent( *( device->IsrEvent() ) ),
      StreamDevice( device ),
      MaxInterruptDepth(0), MaxLocalIsrDepth(0)
{    
    Priority(tpTimeCritical);

    eStopped.Set();
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::Init() -- Init sync objects
//---------------------------------------------------------------------------

void VirtualIsrThread::Init()
{
    inherited::Init();
    
    AddObject(DriverEvent, sDriverEvent );
    AddObject(LocalIsrQ.SignalSem(), sLocalIsrQ);
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::SingleExecution() -- thread function
//---------------------------------------------------------------------------

void VirtualIsrThread::SingleExecution()
{
    int result = WaitFor();

    if ( Terminated() || eStopped.Check() )
        return;
    //
    //  A new interrupt was signalled. Since the low level queue is of
    //     finite size, we drain that queue as fast as we can. Since we
    //     also got at least one interrupt, we process one now.
    if ( result == sDriverEvent )
        {
        size_t depth = StreamDevice->InterruptDepth();
        StreamDevice->Log(0x94, depth);
        MaxInterruptDepth = std::max(depth, MaxInterruptDepth);
        //
        //  Drain the interrupt queue into LocalIsrQ
        while( depth-- )
            {
            int irqstatus = StreamDevice->InterruptAck();
            LocalIsrQ.push( irqstatus );
            }
        size_t Idepth = LocalIsrQ.depth();
        StreamDevice->Log(0x95, Idepth);
        MaxLocalIsrDepth = std::max( Idepth, MaxLocalIsrDepth );
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            //  since we did not wait for Sem before, must decrement sem manually
            //    when popping IRQ register value
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop(SignalQueue<unsigned int>::PopSem);
            DispatchInterrupt( IrqToProcess );
            }
        }
    //
    //  We were signalled because of queued local ISRs.
    if (result == sLocalIsrQ)
        {
        size_t Idepth = LocalIsrQ.depth();
        StreamDevice->Log(0x9a, Idepth);
        //
        //  Process one element in local Q
        unsigned int IrqToProcess;
        if (LocalIsrQ.depth()!= 0)
            {
            IrqToProcess = LocalIsrQ.front();
            LocalIsrQ.pop();  // don't dec sem, already done
            DispatchInterrupt( IrqToProcess );
            }
        }
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::DispatchInterrupt() -- process interrupt
//---------------------------------------------------------------------------

void VirtualIsrThread::DispatchInterrupt( int IntStatus )
{
    StreamDevice->Log(4, IntStatus);
    //
    //  Only one type of interrupt, so no information is available
    //    in the IntStatus word.  Instead we need to read a block from the DSP
    //    that holds interrupt information.
    {
    size_t src = reinterpret_cast<size_t>(&TargetImage->TtoHInterrupt);
    StreamDevice->ReadBlockFromDspMemory(static_cast<int>(src),
                                         reinterpret_cast<ii32*>(&Image.TtoHInterrupt),
                                         sizeof(PciInterruptInfo)/sizeof(int));
    }
    //
    //  Load Interrupt Data region
    PciTransferInfo::Codes Code = static_cast<PciTransferInfo::Codes>(Image.TtoHInterrupt.Code);
    int Data = Image.TtoHInterrupt.Data;
    int Channel = Image.TtoHInterrupt.Channel;

    switch (Code)
        {
        // Bus mastering support
        case PciTransferInfo::icBmReadAck:
            SendStatus = static_cast<PciTransferInfo::TransferStatus>(Data);
            StreamManager->Internals()->SendXfrComplete();
            break;
        case PciTransferInfo::icBmWriteGetAddr:
            {
            //
            //  Request for BM region to be used for writes. Send Address
            //     back in an interrupt.
            size_t addr = reinterpret_cast<size_t>(Incoming.PhysAddr);
            TargetInterrupt(PciTransferInfo::icBmWriteGetAddr, Channel,
                            static_cast<int>(addr));
            }
            break;
        case PciTransferInfo::icBmWriteDataAvail:
            //
            //  Save status for use by mainline
            RecvStatus = static_cast<PciTransferInfo::TransferStatus>(Data);
            StreamManager->Internals()->RecvDataReady();
            break;

        // Mailbox support
        case PciTransferInfo::icMailAck:
            StreamManager->Internals()->HandleMailAck();
            break;

        case PciTransferInfo::icMailAvail:
            StreamManager->Internals()->HandleMailRcv();
            break;

        case PciTransferInfo::icUserInterrupt:
        case PciTransferInfo::icBmReadRequest:
        case PciTransferInfo::icBmWriteAck:
        default:
            break;
        }
    //
    //  Release target to interrupt again
    ClearTtoHIntFlag();
}

//------------------------------------------------------------------------------
//   VirtualIsrThread::TargetInterrupt() -- Post Interrupt to Target
//------------------------------------------------------------------------------

void  VirtualIsrThread::TargetInterrupt(int Code, int Channel, int Data)
{
    MutexedLock Lock(InterruptMutex);
	
	//
    //  Wait for previous interrupt to have been processed...
	while(HtoTIntFlagBusy())
		;

	//
    //  Load Interrupt Data region
    Image.HtoTInterrupt.Code = Code;
    Image.HtoTInterrupt.Data = Data;
    Image.HtoTInterrupt.Channel = Channel;
    Image.HtoTInterrupt.Flag = PciTransferInfo::ifBusy; // Set Busy flag

    {
    size_t dst = reinterpret_cast<size_t>(&TargetImage->HtoTInterrupt);
    StreamDevice->WriteBlockToDspMemory((const unsigned int *)&Image.HtoTInterrupt,
                             static_cast<int>(dst),
                             sizeof(PciInterruptInfo)/sizeof(int));
    }
    //
    //  Interrupt Target
    StreamDevice->DspInt();
}

//------------------------------------------------------------------------------
//  VirtualIsrThread::AckMailbox() --
//------------------------------------------------------------------------------

void VirtualIsrThread::AckMailbox()
{
    //
    //  Notify target that we have processed previous mailbox msg
	TargetInterrupt(PciTransferInfo::icMailAck, 0, 0);
}

//------------------------------------------------------------------------------
//  VirtualIsrThread::SetMailboxAvailable() --
//------------------------------------------------------------------------------

void VirtualIsrThread::SetMailboxAvailable()
{
    //
    //  Notify target that we have sent a new mailbox msg
	TargetInterrupt(PciTransferInfo::icMailAvail, 0, 0);
}

//-----------------------------------------------------------------------------
//  VirtualIsrThread::HtoTIntFlagBusy() -- Read HtoT flag, get state
//-----------------------------------------------------------------------------

bool  VirtualIsrThread::HtoTIntFlagBusy()
{
    size_t src = reinterpret_cast<size_t>(&TargetImage->HtoTInterrupt);
    StreamDevice->ReadBlockFromDspMemory(static_cast<int>(src),
                                         (unsigned int *)&Image.HtoTInterrupt,
                                         sizeof(PciInterruptInfo)/sizeof(int));
    volatile int result = Image.HtoTInterrupt.Flag;
	if(result == PciTransferInfo::ifFree)
		return false;
	else
		return true;	
}

//-----------------------------------------------------------------------------
//  VirtualIsrThread::ClearTtoHIntFlag()
//-----------------------------------------------------------------------------

void  VirtualIsrThread::ClearTtoHIntFlag()
{
    Image.TtoHInterrupt.Flag = PciTransferInfo::ifFree;

    size_t dst = reinterpret_cast<size_t>(&TargetImage->TtoHInterrupt);
    StreamDevice->WriteBlockToDspMemory((unsigned int *)&Image.TtoHInterrupt,
                                        static_cast<int>(dst),
                                        sizeof(PciInterruptInfo)/sizeof(int));
}

//---------------------------------------------------------------------------
//  VirtualIsrThread::RegisterStreamPlugIn() -- attach to a streaming object
//---------------------------------------------------------------------------

void  VirtualIsrThread::RegisterStreamPlugIn( ITiBmStreamManager *plug_in )
{
    StreamManager = plug_in;
}

//-----------------------------------------------------------------------------
//  VirtualIsrThread::LoadPciMemOffset()
//-----------------------------------------------------------------------------

void VirtualIsrThread::LoadPciMemOffset(int addr)
{
    PciMemOffset = addr;

    //
    //  This pointer used to find relative address of structure members in
    //     ***TARGET MEMORY*** it should never be dereferenced
    TargetImage = reinterpret_cast < PciTransferInfo * > (PciMemOffset);
}


}  // end namespace Innovative::TiBusmasterIntScheme

//===========================================================================
//  CLASS TiBusmasterInterruptScheme  -- ISR & Busmaster region handling
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class TiBusmasterInterruptScheme
//---------------------------------------------------------------------------

TiBusmasterInterruptScheme::TiBusmasterInterruptScheme(ITiBm_DeviceToScheme * device)
    :  Visr(0), StreamManager(0), StreamDevice(device)
{
}

//---------------------------------------------------------------------------
//  destructor for class TiBusmasterInterruptScheme
//---------------------------------------------------------------------------

TiBusmasterInterruptScheme::~TiBusmasterInterruptScheme()
{
    Close();
    delete StreamDevice;
    delete StreamManager;
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::Open()
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::Open()
{
    //
    //  Create V-ISR thread.
    Visr = new TiBusmasterIntScheme::VirtualIsrThread( StreamDevice );

    ConfigureBmRegion();
    Visr->Resume();
    //
    //  Start V-ISR thread.
    Visr->Start();
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::Close()
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::Close()
{
    //  Have VISR Thread shut itself down
    if ( Visr )
        {
        Visr->Stop();
        Visr->Shutdown();
        delete Visr;
        Visr = NULL;
        }
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::ConfigureBmRegion()
//---------------------------------------------------------------------------

void TiBusmasterInterruptScheme::ConfigureBmRegion()
{
    //  Get busmaster position & size
    ii32 * BmAddr = StreamDevice->BusMasterAddress();
    ii32 * BmPhysAddr = StreamDevice->BusMasterPhysicalAddress();
    ii32   RegionSizeWords = StreamDevice->BusMasterSizeWords()/2;

    //  Inform VISR where BM Region is
    //  ...input
    Visr->Incoming.BaseAddr    = BmAddr;
    Visr->Incoming.PhysAddr    = BmPhysAddr;
    Visr->Incoming.SizeInWords = RegionSizeWords;

    RecvRgnBasePhys = BmPhysAddr;
    RecvRgnBase     = BmAddr;

    BmAddr += RegionSizeWords;
    BmPhysAddr += RegionSizeWords;
    //  ...output
    //  Inform VISR where BM Region is
    //  ...input
    Visr->Outgoing.BaseAddr    = BmAddr;
    Visr->Outgoing.PhysAddr    = BmPhysAddr;
    Visr->Outgoing.SizeInWords = RegionSizeWords;

    XmitRgnBasePhys = BmPhysAddr;
    XmitRgnBase     = BmAddr;

    XmitRgnSize = RegionSizeWords;
    RecvRgnSize = RegionSizeWords;
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::CopyOut() --
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::CopyOut(const MatadorMessage & mail)
{
    //
    //  Load mail info into Info block image
    for ( int i = 0; i < MatadorMessage::PacketSize; i ++ )
        Visr->Image.HtoTMail.Slot[i] = mail.Packet(i);
    //
    //  Write image to PCI Info block
    size_t dst = reinterpret_cast<size_t>(&Visr->TargetImage->HtoTMail);
    StreamDevice->WriteBlockToDspMemory((unsigned int *)&Visr->Image.HtoTMail,
                                        static_cast<int>(dst),
                                        sizeof(PciMail)/sizeof(int));

    Visr->SetMailboxAvailable();
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::CopyIn()
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::CopyIn(MatadorMessage & mail)
{
    //
    //  Get mail data from PCI info block
    size_t src = reinterpret_cast<size_t>(&Visr->TargetImage->TtoHMail);
    StreamDevice->ReadBlockFromDspMemory(static_cast<int>(src),
                                         (unsigned int *)&Visr->Image.TtoHMail,
                                         sizeof(PciMail)/sizeof(int));

    for( int i = 0; i < MatadorMessage::PacketSize; i ++ )
        mail.Packet(i,Visr->Image.TtoHMail.Slot[i]);

    Visr->AckMailbox();
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::CopyOut() --
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::CopyOut(const Buffer & packet)
{
    //
    //  Load Data into Busmaster region
    const int * srcDataPtr = packet.ConstData();
    int * destDataPtr = reinterpret_cast<int *>(Visr->Outgoing.BaseAddr);
    
    //  ...transfer data to BM region
    CopyInt32Buffer(srcDataPtr, destDataPtr, packet.SizeInInts());

    //
    //  Load Header into PCI Header
    size_t addr = reinterpret_cast<size_t>(Visr->Outgoing.PhysAddr);
    Visr->Image.HtoTXfrParams.Address = static_cast<int>(addr);
    // Restrict to 1 xfer
    Visr->Image.HtoTXfrParams.TotalSize = static_cast<int>(packet.SizeInInts()*sizeof(int));  
    // Restrict to 1 xfer
    Visr->Image.HtoTXfrParams.TransferSize = static_cast<int>(packet.SizeInInts()*sizeof(int));  

    size_t dst = reinterpret_cast<size_t>(&Visr->TargetImage->HtoTXfrParams);
    StreamDevice->WriteBlockToDspMemory((unsigned int *)&Visr->Image.HtoTXfrParams,
                                        static_cast<int>(dst),
                                        sizeof(PciParameters)/sizeof(int));
    //
    //  Interrupt PC to do transfer
    Visr->TargetInterrupt(PciTransferInfo::icBmReadRequest, 0, 0);

}
//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::CopyIn() --
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::CopyIn(Buffer & packet)
{
    //
    //  Read transfer information from header region
    size_t addr = reinterpret_cast<size_t>(&Visr->TargetImage->TtoHXfrParams);
    StreamDevice->ReadBlockFromDspMemory(static_cast<int>(addr),
                           (unsigned int *)&Visr->Image.TtoHXfrParams,
                           sizeof(PciParameters)/sizeof(int));
    int totalSize = Visr->Image.TtoHXfrParams.TotalSize/sizeof(int);       // Restrict to 1 xfer
    //  ...could skip the transfer if busmaster failed but we won't
    //
    //  Load Data into Busmaster region
    //
    //  Prepare Buffer for transfer
    //  ...Adjust Size to fit
    packet.Resize(totalSize);
    const int * srcDataPtr = reinterpret_cast<const int *>(Visr->Incoming.BaseAddr);
    int * destDataPtr = packet.Data();
    //
    //  ...transfer data from BM region
    CopyInt32Buffer(srcDataPtr, destDataPtr, packet.SizeInInts());
    //
    //  Notify target that we have processed data
    Visr->TargetInterrupt(PciTransferInfo::icBmWriteAck, 0, 0);
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::SendStatus() --
//---------------------------------------------------------------------------

PciTransferInfo::TransferStatus  TiBusmasterInterruptScheme::SendStatus()
{
    return Visr->SendStatus;
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::RecvStatus()
//---------------------------------------------------------------------------

PciTransferInfo::TransferStatus  TiBusmasterInterruptScheme::RecvStatus()
{
    return Visr->RecvStatus;
}

//-----------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::LoadPciMemOffset()
//-----------------------------------------------------------------------------

void TiBusmasterInterruptScheme::LoadPciMemOffset(int addr)
{
    Visr->LoadPciMemOffset(addr);
}

//---------------------------------------------------------------------------
//  TiBusmasterInterruptScheme::RegisterStreamPlugIn() -- attach to a streaming object
//---------------------------------------------------------------------------

void  TiBusmasterInterruptScheme::RegisterStreamPlugIn( ITiBmStreamManager *plug_in )
{
    StreamManager = plug_in;
    Visr->RegisterStreamPlugIn(StreamManager);
}


}  // end namespace Innovative

//---------------------------------------------------------------------------


