//  TiBmInterruptScheme_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef TiBmInterruptScheme_MbH
#define TiBmInterruptScheme_MbH

#include <DataTypes_Mb.h>
#include <HardwareRegister_Mb.h>
#include <PciTransferStruct_Mb.h>

namespace OpenWire
{
    class Event;
}

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class Event;
class MatadorMessage;
class Buffer;

//===========================================================================
//  CLASS ITiBm_DeviceToScheme  -- Ti Busmaster Device callbacks for Scheme
//===========================================================================

class ITiBm_DeviceToScheme
{
public:
    virtual Event *  IsrEvent() = 0;
    virtual size_t   InterruptDepth() = 0;
    virtual int      InterruptAck() = 0;
    virtual void     Log(int type, int data) = 0;
    virtual void     Log(int type, size_t data)
        {  Log(type, static_cast<int>(data));  }
    virtual ii32 *   BusMasterAddress() const = 0;
    virtual ii32 *   BusMasterPhysicalAddress() const = 0;
    virtual ii32     BusMasterSizeWords() const = 0;
    virtual void     BusMasterSizeWords(ii32 value) = 0;
    virtual void     WriteBlockToDspMemory(const ii32 *src, int dst, int size) = 0;
    virtual void     ReadBlockFromDspMemory(int src, ii32 *dst, int size) = 0;
    virtual void     DspInt() = 0;

    virtual ~ITiBm_DeviceToScheme()
    	{}

};

//---------------------------------------------------------------------------
namespace TiBusmasterIntScheme
{
    class VirtualIsrThread;
    class BmXmitThread;
}

//===========================================================================
//  CLASS TiBusmasterInterruptScheme  -- ISR & Busmaster region handling
//===========================================================================

class TiBusmasterInterruptScheme
{
public:
    enum Dir { BmXmit, BmRecv };

    TiBusmasterInterruptScheme(ITiBm_DeviceToScheme * device);
    virtual ~TiBusmasterInterruptScheme();

    void  Open();
    void  Close();

    void  ConfigureBmRegion();
    void  LoadPciMemOffset(int addr);
    void  CopyOut(const MatadorMessage & mail);
    void  CopyIn(MatadorMessage & mail);
    void  CopyOut(const Buffer & packet);
    void  CopyIn(Buffer & packet);
    virtual PciTransferInfo::TransferStatus SendStatus();
    virtual PciTransferInfo::TransferStatus RecvStatus();

    void  RegisterStreamPlugIn( class ITiBmStreamManager *plug_in );
    
private:
    //
    //  Member Data
    TiBusmasterIntScheme::VirtualIsrThread *  Visr;
    ITiBmStreamManager *                      StreamManager;
    ITiBm_DeviceToScheme *                    StreamDevice;

public:
    //
    //  Debug...
    ii32 *          XmitRgnBasePhys;
    ii32 *          RecvRgnBasePhys;
    ii32 *          XmitRgnBase;
    ii32 *          RecvRgnBase;
    unsigned int    XmitRgnSize;
    unsigned int    RecvRgnSize;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
#endif
