//  TiBmStreamSupport.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef TiBmStreamSupportH
#define TiBmStreamSupportH

#include <PciTransferStruct_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class MatadorMessage;
class Buffer;
class ITiBmStreamDevice;

//===========================================================================
//  CLASS ITiBmStreamManagerInternals  -- TI Cpu Stream Internal Interface
//===========================================================================

class ITiBmStreamManagerInternals
{
public:
    virtual void  HandleMailAck() = 0;
    virtual void  HandleMailRcv() = 0;
    virtual void  SendXfrComplete() = 0;
    virtual void  RecvDataReady() = 0;
    
    virtual ~ITiBmStreamManagerInternals()
    	{}

};

//===========================================================================
//  CLASS ITiBmStreamManager  -- TI Cpu Stream Public Interface
//===========================================================================

class ITiBmStreamManager
{
public:
    virtual bool  Send( const Buffer & packet ) = 0;
    virtual bool  Recv( Buffer & packet ) = 0;
    virtual bool  Send( const MatadorMessage & mail ) = 0;
    virtual bool  Recv( MatadorMessage & mail ) = 0;

    virtual void  MessageTerminate() = 0;
//    virtual void  PacketTerminate() = 0;

    virtual bool ConnectTo( ITiBmStreamDevice *Device ) = 0;
    //virtual void DisconnectFrom( ITiBmStreamDevice *Device ) = 0;
    virtual void Disconnect() = 0;

    virtual ITiBmStreamManagerInternals *  Internals() = 0;
    virtual ~ITiBmStreamManager()
    	{}
};

//===========================================================================
//  CLASS ITiBmStreamDeviceInternals  -- TI Cpu Streaming Device Internals
//===========================================================================

class ITiBmStreamDeviceInternals
{
public:

    virtual int    DeviceTargetID() = 0;

    virtual void   CopyOut(const MatadorMessage & mail) = 0;
    virtual void   CopyIn(MatadorMessage & mail) = 0;
    virtual void   CopyOut(const Buffer & packet) = 0;
    virtual void   CopyIn(Buffer & packet) = 0;

    virtual PciTransferInfo::TransferStatus SendStatus() = 0;
    virtual PciTransferInfo::TransferStatus RecvStatus() = 0;
    virtual ~ITiBmStreamDeviceInternals()
    	{}
};

//===========================================================================
//  CLASS ITiBmStreamDevice  -- TI Cpu Streaming Device public interface
//===========================================================================

class ITiBmStreamDevice
{
public:

    virtual void  Reset() = 0;
    virtual void  Log(int type, int data) = 0;

    virtual void  RegisterStreamPlugIn( ITiBmStreamManager *plug_in ) = 0;
	virtual ITiBmStreamDeviceInternals *  Internals() = 0;
    virtual ~ITiBmStreamDevice()
    	{}
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // end namespace Innovative

#endif

