//---------------------------------------------------------------------------
// TiBmStream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#include <TiBmStream_Mb.h>
#include <MatadorMessage_Mb.h>
#include <Mutex_Mb.h>
#include <Buffer_Mb.h>
#include <StartStopThread_Mb.h>
#include <Exception_Mb.h>

namespace Innovative
{
//===========================================================================
//  CLASS TIBSMInternalsImpl  -- Internal Interface Impl
//===========================================================================

class TIBSMInternalsImpl : public ITiBmStreamManagerInternals
{
public:
    TIBSMInternalsImpl(TiBusmasterStream * owner)
        :  Owner(owner) {}

    virtual void  HandleMailAck() {  Owner->HandleMailAck(); }
    virtual void  HandleMailRcv() {  Owner->HandleMailRcv(); }
    virtual void  SendXfrComplete() {  Owner->HandleSendXfrComplete(); }
    virtual void  RecvDataReady()  {  Owner->HandleRecvDataReady(); }

private:
    TiBusmasterStream * Owner;
};

//===========================================================================
//  CLASS IncomingDataNotifierThread  --  Signal new packet arrival
//===========================================================================

class IncomingDataNotifierThread : public StartStopThread
{
    typedef StartStopThread inherited;

public:
    IncomingDataNotifierThread(TiBusmasterStream & stream);
    ~IncomingDataNotifierThread();

    void IncomingMailMessage();
    void IncomingDataPacket();

protected:
    enum  EventTags  { sMailMessage=inherited::sLast, sDataPacket, sLast };
    Event                MailMessage;
    Event                DataPacket;
    TiBusmasterStream &  Stream;

    //  don't override these...
    virtual void    Init();
    virtual void    SingleExecution();

};

//===========================================================================
//  CLASS IncomingDataNotifierThread  --
//===========================================================================
//---------------------------------------------------------------------------
// constructor for class IncomingDataNotifierThread
//---------------------------------------------------------------------------
 
IncomingDataNotifierThread::IncomingDataNotifierThread(TiBusmasterStream & stream)
    : inherited("IncomingDataNotifierThread"),
      MailMessage(Event::AutoReset), DataPacket(Event::AutoReset), Stream(stream)
{
}
 
//---------------------------------------------------------------------------
// destructor for class IncomingDataNotifierThread
//---------------------------------------------------------------------------
 
IncomingDataNotifierThread::~IncomingDataNotifierThread()
{
}
 
//------------------------------------------------------------------------------
//  IncomingDataNotifierThread::Init() -- Init Multi-Objects here
//------------------------------------------------------------------------------

void  IncomingDataNotifierThread::Init()
{
    //  Load NotifyObj with our default objects
    inherited::Init();
    AddObject(DataPacket, sDataPacket);
    AddObject(MailMessage, sMailMessage);
}

//---------------------------------------------------------------------------
//  IncomingDataNotifierThread::SingleExecution() --
//---------------------------------------------------------------------------
 
void IncomingDataNotifierThread::SingleExecution()
{
    //
    //  Wait for any of the Pools to signal data, or one of the
    //     termination events (Suicide or BustOut)
    int result = WaitFor();

    if (result == sTerminate || result == sStopped)
        return;
    //
    //  Its one of the remaining notifiers
    switch (result)
        {
        case sDataPacket:
            //
            //  Signal that packet has arrived...
            {
            TiBusmasterStreamDataEvent TBSEvent(&Stream);
            Stream.OnPacketAvailable.Execute(TBSEvent);
            }
            break;
        case sMailMessage:
            //
            //  Signal that packet has arrived...
            {
            TiBusmasterStreamDataEvent TBSEvent(&Stream);
            Stream.OnMailAvailable.Execute(TBSEvent);
            }
            break;
        }
}
 
//---------------------------------------------------------------------------
//  IncomingDataNotifierThread::IncomingMailMessage() --
//------------------------------------------------------------------------------

void IncomingDataNotifierThread::IncomingMailMessage()
{
    MailMessage.Set();
}

//---------------------------------------------------------------------------
//  IncomingDataNotifierThread::IncomingDataPacket() --
//------------------------------------------------------------------------------

void IncomingDataNotifierThread::IncomingDataPacket()
{
    DataPacket.Set();
}

//===========================================================================
//  CLASS TiBusmasterStream  -- Packet Based Stream I/O
//===========================================================================

//void TestXXX()
//{
//   TiBusmasterStream * yadda = new TiBusmasterStream;
//}

//---------------------------------------------------------------------------
//  constructor for class TiBusmasterStream
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
TiBusmasterStream::TiBusmasterStream()
    :  StreamDevice(0), InternalOps(new TIBSMInternalsImpl(this)),
       FMessageHToTMailEvent(Event::ManualReset),
       FMessageTToHMailEvent(Event::ManualReset),
       FMessagesTerminatedEvent(Event::ManualReset), SendXfrComplete(Event::ManualReset),
       RecvDataReady(Event::ManualReset), IDNThread(0)
{
    FMessageHToTMailEvent.Set();

    FMessageHToTObjects.AddObject( FMessageHToTMailEvent, sMail );
    FMessageHToTObjects.AddObject( FMessagesTerminatedEvent, sMailTerminated );

    FMessageTToHObjects.AddObject( FMessageTToHMailEvent, sMail );
    FMessageTToHObjects.AddObject( FMessagesTerminatedEvent, sMailTerminated );

    IDNThread = new IncomingDataNotifierThread( *this );
    IDNThread->Resume();
    IDNThread->Start();

}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class TiBusmasterStream
//---------------------------------------------------------------------------

TiBusmasterStream::~TiBusmasterStream()
{
    IDNThread->Stop();
    IDNThread->Shutdown();
    delete IDNThread;
    IDNThread = 0;

    delete InternalOps;
    InternalOps = 0;
}

//---------------------------------------------------------------------------
//  TiBusmasterStream::Internals() -- Internal Operations object
//---------------------------------------------------------------------------

ITiBmStreamManagerInternals *  TiBusmasterStream::Internals()
{
    return InternalOps;
}

//---------------------------------------------------------------------------
//  TiBusmasterStream::Send() -- Send mailbox message to target
//---------------------------------------------------------------------------

bool  TiBusmasterStream::Send(const MatadorMessage & Message )
{
    //
    //  Grab 'Send' Mutex
    MutexedLock Lock(SendMutex);

	int result = FMessageHToTObjects.WaitFor();
    if (result == sMailTerminated)
      return false;

    FMessageHToTMailEvent.Clear();
    //
    //  Prepare to send message
    StreamDevice->Internals()->CopyOut(Message);

    FMessageHToTMailEvent.WaitFor();
	return true;
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::Recv() -- Retrieve a message from the target
//------------------------------------------------------------------------------

bool TiBusmasterStream::Recv(MatadorMessage & Message)
{
    MutexedLock  Lock(RecvMutex);
	
	int result = FMessageTToHObjects.WaitFor();
    if(result == sMailTerminated)
        return false;

    FMessageTToHMailEvent.Clear();
    //
    //  Transfer data into message packet
    StreamDevice->Internals()->CopyIn(Message);
	return true;
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::HandleMailAck() -- Signal transmitted message has been received
//------------------------------------------------------------------------------

void  TiBusmasterStream::HandleMailAck()
{
    FMessageHToTMailEvent.Set();
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::HandleMailRcv() -- Signal received message available
//------------------------------------------------------------------------------

void  TiBusmasterStream::HandleMailRcv()
{
    FMessageTToHMailEvent.Set();
    //
    //  Notify that mail message is ready
    IDNThread->IncomingMailMessage();
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::HandleSendXfrComplete() --
//------------------------------------------------------------------------------

void  TiBusmasterStream::HandleSendXfrComplete()
{
    SendXfrComplete.Set();
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::HandleRecvDataReady() --
//------------------------------------------------------------------------------

void  TiBusmasterStream::HandleRecvDataReady()
{
    RecvDataReady.Set();
    //
    //  Notify that new data packet is ready
    IDNThread->IncomingDataPacket();
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::MessageTerminate() -- Terminate message receipt
//------------------------------------------------------------------------------

void TiBusmasterStream::MessageTerminate()
{
    FMessagesTerminatedEvent.Set();
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::Send() -- Transfer Block to Target via Dsp PCI Transfer
//------------------------------------------------------------------------------

bool  TiBusmasterStream::Send(const Buffer & Packet)
{
    //
    //  Grab 'Send' Mutex
    MutexedLock Lock(SendMutex);

    SendXfrComplete.Clear();
    //
    //  Transfer data to target
    StreamDevice->Internals()->CopyOut(Packet);
    //
    //  Wait for Ack to come back
    SendXfrComplete.WaitFor();
    //
    //  'Success' or Failure?
    //  ...If status OK return true
    SendStatus = StreamDevice->Internals()->SendStatus();
    if (SendStatus==PciTransferInfo::tsOk)
        return true;
    else
        return false;
}

//------------------------------------------------------------------------------
//  TiBusmasterStream::Recv() -- Transfer packet from target via PCI Transfer
//------------------------------------------------------------------------------

bool  TiBusmasterStream::Recv(Buffer & Packet)
{
    MutexedLock  Lock(RecvMutex);
    //
    //  Wait for header to be sent from target
    RecvDataReady.WaitFor();
    RecvDataReady.Clear();
    //
    //  Transfer data to target
    StreamDevice->Internals()->CopyIn(Packet);

    RecvStatus = StreamDevice->Internals()->RecvStatus();
    if (RecvStatus==PciTransferInfo::tsOk)
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  TiBusmasterStream::ConnectTo() -- connect stream to stream device
//---------------------------------------------------------------------------

bool  TiBusmasterStream::ConnectTo(ITiBmStreamDevice * aStreamDevice)
{
    if (aStreamDevice)
        {
        if (StreamDevice)
            StreamDevice->RegisterStreamPlugIn(0);

        StreamDevice = aStreamDevice;
        StreamDevice->RegisterStreamPlugIn(this);
        return true;
        }

    return false;
}

//---------------------------------------------------------------------------
//  TiBusmasterStream::Disconnect() -- disconnect stream from stream device
//---------------------------------------------------------------------------

void TiBusmasterStream::Disconnect()
{
    if (StreamDevice)
        StreamDevice->RegisterStreamPlugIn(0);
    StreamDevice = 0;
}

//---------------------------------------------------------------------------
//  TiBusmasterStream::Target() -- Report target stream device instance
//---------------------------------------------------------------------------

int TiBusmasterStream::Target() const
{
    if (!StreamDevice)
        throw Innovative::MalibuException("Stream not connected to Device");
    return StreamDevice->Internals()->DeviceTargetID();
}

}  // namespace Innovative

