//---------------------------------------------------------------------------
// TiBmStream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#ifndef TiBmStream_MbH
#define TiBmStream_MbH

#include <TiBmStreamSupport_Mb.h>
#include <Events_Mb.h>
#include <Event_Mb.h>
#include <MultipleObjects_Mb.h>
#include <Mutex_Mb.h>
#include <MatadorMessage_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class TiBusmasterStream;

//===========================================================================
// CLASS TiBusmasterStreamDataEvent  --  Data I/O Event for TiBusmasterStream
//===========================================================================

class TiBusmasterStreamDataEvent : public OpenWire::Event
{
public:
    TiBusmasterStream *  Sender;

public:
    TiBusmasterStreamDataEvent( TiBusmasterStream * sender )
        :  Sender( sender )
        {
        }
};

//===========================================================================
//  CLASS TiBusmasterStream  -- Packet Based Stream I/O
//===========================================================================

class TiBusmasterStream : public ITiBmStreamManager
{
    friend class TIBSMInternalsImpl;
    
public:
    TiBusmasterStream();
    virtual ~TiBusmasterStream();

    virtual bool  ConnectTo( ITiBmStreamDevice *Device );
    virtual void  Disconnect();

    virtual bool  Send( const Buffer & packet );
    virtual bool  Recv( Buffer & packet );
    virtual bool  Send( const MatadorMessage & mail );
    virtual bool  Recv( MatadorMessage & mail );

    virtual void  MessageTerminate();
    virtual ITiBmStreamManagerInternals *  Internals();
    virtual int   Target() const;

    //
    //  Events
    OpenWire::EventHandler<TiBusmasterStreamDataEvent>     OnPacketAvailable;
    OpenWire::EventHandler<TiBusmasterStreamDataEvent>     OnMailAvailable;

private:
    //
    //  Member Data
    ITiBmStreamDevice *            StreamDevice;
    ITiBmStreamManagerInternals *  InternalOps;
    // Host to target
    enum { sMail, sMailTerminated };
    Event                   FMessageHToTMailEvent;
    Event                   FMessageTToHMailEvent;
    Event                   FMessagesTerminatedEvent;
    
    MultipleObjects         FMessageHToTObjects;
    MultipleObjects         FMessageTToHObjects;

    Mutex                            SendMutex;
    Event                            SendXfrComplete;
    PciTransferInfo::TransferStatus  SendStatus;

    Mutex                            RecvMutex;
    Event                            RecvDataReady;
    PciTransferInfo::TransferStatus  RecvStatus;

    class IncomingDataNotifierThread * IDNThread;
    //
    //  Member Functions
    void  Prefill(unsigned int packets);
    void  HandleMailAck();
    void  HandleMailRcv();
    void  HandleSendXfrComplete();
    void  HandleRecvDataReady();

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // namespace

#endif
