// TiDac5682z_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <TiDac5682z_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{
//---------------------------------------------------------------------------
namespace TiDac5682
{
/*
//===========================================================================
//  CLASS X5-TX IDacToHardware Impl
//===========================================================================

class X5_TXDacToHardware :  public IDacToHardware
{
public:

    virtual unsigned short  Spi(int addr)
        {  return Dac->Spi(addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Dac->Spi(addr, value);  }

    virtual double          DCLK(bool single_channel)
        {  Dac->Owner->Clock.DCLK(SingleActiveChannel()) ;  }
    virtual bool            IsDacDllEnabled(bool single_channel)
        {  Dac->Owner->Clock.IsDacDllEnabled(SingleActiveChannel());  }
    virtual unsigned int    FastConfigValue()
        {
        int fast_cfg_value;
        if (Dac->Owner->PciLogicPcb() >= 2)   // Rev C or later
            fast_cfg_value = 0xC0;
        else
            fast_cfg_value = 0x00;
        }

    X5_TXDacToHardware( X5_TxDacDevice *  dac ) : Dac(dac)
        {}
    virtual ~X5_TXDacToHardware()
        {}

    X5_TxDacDevice *  Dac;
}
*/
//===========================================================================
//  CLASS DacDevice  -- Per-DAC controls for X5-TZ
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DacDevice
//---------------------------------------------------------------------------

DacDevice::DacDevice(IDacToHardware * hdw, bool is5681)
    :  Hdw(hdw), DevIs5681(is5681),
       FEnabled(false), FDacDelay(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class DacDevice
//---------------------------------------------------------------------------

DacDevice::~DacDevice()
{
    delete Hdw;   Hdw = 0;
}

//---------------------------------------------------------------------------
//  DacDevice::SingleActiveChannel() -- True if in single channel mode
//---------------------------------------------------------------------------

bool  DacDevice::SingleActiveChannel()
{
    return (FChannelEnabled[0] ^ FChannelEnabled[1]);
}

//---------------------------------------------------------------------------
//  DacDevice::PreconfigHardware() --
//---------------------------------------------------------------------------

void  DacDevice::PreconfigHardware()
{
    ScopedTraceRef ccc(Debug, "DacDevice::PreconfigHardware()");

    double DCLK = Hdw->DCLK(SingleActiveChannel());
    bool DacDllEnabled = Hdw->IsDacDllEnabled(SingleActiveChannel());

    Debug.Log("Ch 0 En: ", (int)FChannelEnabled[0] );
    Debug.Log("Ch 1 En: ", (int)FChannelEnabled[1] );
    Debug.Log("SingleActiveChannel: ", (int)SingleActiveChannel() );
    Debug.Log("DCLK: ", DCLK );
    Debug.Log("DacDllEnabled: ", DacDllEnabled );
    //
    //  Enable devices and/or triggers
    if (SingleActiveChannel())
        {
        //  Single Channel Mode
        int active_chan = FChannelEnabled[1];
        SingleChannelMode(active_chan);

        if (DacDllEnabled)
            Hdw->Spi(0x06, (DevIs5681) ? 0x0e : 0x2e);
        else
            Hdw->Spi(0x06, (DevIs5681) ? 0x0f : 0x2f);
        }
    else
        {
        //  Dual channel mode
        DualChannelMode();
        if (DacDllEnabled)
            Hdw->Spi(0x06, 0x0e);
        else
            Hdw->Spi(0x06, 0x0f);
        }

    //  Spi Setup
    //  ...Address 0
        // ?
    //  ...Address 1
    //      done later
    //  ...Address 2
    //      2's complement + single/dual channel mode (see above)
    //  ...Address 3
    //      (disable DAC offset, and allow SLFTST FIFO and Pattern errors)

    Hdw->Spi(0x03, 0x00);
    std::stringstream msg;
    msg << std::hex << "TX Dac Spi 0x3 = 0x" << Hdw->Spi(0x03);
    Debug.Log(msg.str());


//    Hdw->Spi(0x03, 0x01);
    //  ...Address 4
    //       clear all errors
    //       ...done later
    //  ...Address 5
    //       (bypass DLL)
    if (DacDllEnabled)
        Hdw->Spi(0x05, 0x02);
    else
        Hdw->Spi(0x05, 0x06);
    //  ...Address 6
    //        put PLL in sleep mode
    //        (see above)
    //  ...Address 7
    //        Gain Adjustment (GAIN_A = GAIN_B = 15)
    Hdw->Spi(0x07, 0xff);
    //  ...Address 9
    //  ???
    //  ...Address 10
    Spi10Setup(DCLK);
    //  ...Address 11
    Hdw->Spi(0x0b, 0x00);

    Hdw->Spi(0x0e, 0x40);

    LoadDacDelay( FDacDelay );
    //  ...Address 8
    //      Toggle DLL Restart
    if (DacDllEnabled)
        {
        Hdw->Spi(0x08, 0x04);
        Hdw->Spi(0x08, 0x00);
        }

    for (int reps=0; reps<5; reps++)
        {
        Sleep(1);

        //  Check for lock and quit if it happens
        int spi0 = Hdw->Spi(0x00);

        std::stringstream msg;
        msg << std::hex << "TX Dac Spi 0x0 = 0x" << spi0;
        Debug.Log(msg.str());
        //if (spi0&0x40)
        //    break;
        }

//    Hdw->Spi(0x03, 0x03);

}

//---------------------------------------------------------------------------
//  DacDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  DacDevice::ConfigureHardware()
{
    //  none
}

//---------------------------------------------------------------------------
//  DacDevice::Spi10Setup() -- Set to single channel mode
//---------------------------------------------------------------------------

void  DacDevice::Spi10Setup(double DCLK)
{
    int fast_cfg_value = Hdw->FastConfigValue();
    DCLK /= 1e6;   // units are MHz

    if (DCLK <= 125.0)
        Hdw->Spi(0x0a, 0x00);
    else if (DCLK <= 150.0)
        Hdw->Spi(0x0a, 0xcd);
    else if (DCLK <= 175.0)
        Hdw->Spi(0x0a, 0xce);
    else if (DCLK <= 200.0)
        Hdw->Spi(0x0a, 0xcf);
    else if (DCLK <= 300.0)
        Hdw->Spi(0x0a, 0xc8);
    else if (DCLK <= 500.0)
        Hdw->Spi(0x0a, fast_cfg_value);
    else
        Hdw->Spi(0x0a, fast_cfg_value);

}

//---------------------------------------------------------------------------
//  DacDevice::SingleChannelMode() -- Set to single channel mode
//---------------------------------------------------------------------------

void  DacDevice::SingleChannelMode(int /*active_chan*/)
{
    //  we actually can't set a single channel to Channel 1...use 0 for
    //    both single channel modes
    //  Set 2's complement (0x80)
    //  NOT dual channel (0x40)
    Hdw->Spi(0x02, 0x80);

}

//---------------------------------------------------------------------------
//  DacDevice::DualChannelMode() -- Set to dual channel mode
//---------------------------------------------------------------------------

void  DacDevice::DualChannelMode()
{
    //  Set 2's complement (0x80)
    //  Set dual channel (0x40)
    Hdw->Spi(0x02, 0xc0);

}
//---------------------------------------------------------------------------
//  DacDevice::Enabled() -- Enable device
//---------------------------------------------------------------------------

void  DacDevice::Enabled(bool value)
{
    FEnabled = value;
}

//---------------------------------------------------------------------------
//  DacDevice::ChannelEnabled() -- Enable channel
//---------------------------------------------------------------------------

void  DacDevice::ChannelEnabled(int idx, bool value)
{
    FChannelEnabled[idx] = value;
}

//---------------------------------------------------------------------------
//  DacDevice::LoadDacDelay() -- Set Dac Delay parameter
//---------------------------------------------------------------------------

void  DacDevice::LoadDacDelay(unsigned int value)
{
    FDacDelay = value;
    Hdw->Spi(0x01, (value<<6));
}

//---------------------------------------------------------------------------
//  DacDevice::ClearDacTestStatus() --  Query dac test status
//---------------------------------------------------------------------------

void  DacDevice::ClearDacTestStatus()
{
    Hdw->Spi(0x04, 0x00);
}

//---------------------------------------------------------------------------
//  DacDevice::DacTestStatus() --  Query dac test status
//---------------------------------------------------------------------------

unsigned int  DacDevice::DacTestStatus()
{
	return Hdw->Spi(0x04);
}
//---------------------------------------------------------------------------
//  DacDevice::IsDacTestStatusOk() --  Query dac test status bitg
//---------------------------------------------------------------------------

bool DacDevice::IsDacTestStatusOk()
{
	return (( Hdw->Spi(0x04) & 0x30 )? true:false);
}


//===========================================================================
//  CLASS DacCalibrator  -- DAC calibration procedure for TiDac6682
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DacCalibrator
//---------------------------------------------------------------------------

DacCalibrator::DacCalibrator(IDacCalibrationRegister * reg, IDacCalibratorToHdw * hdw)
    :  Reg(reg), Hdw(hdw)
{

}

//---------------------------------------------------------------------------
//  destructor for class DacCalibrator
//---------------------------------------------------------------------------

DacCalibrator::~DacCalibrator()
{
    delete Hdw;   Hdw = 0;
    delete Reg;   Reg = 0;
}

//---------------------------------------------------------------------------
//  DacCalibrator::Calibrate() -- Perform complete Cal for Dacs
//---------------------------------------------------------------------------

void  DacCalibrator::Calibrate()
{
    ScopedTraceRef ccc(Debug, "DacCalibrator::Calibrate()");

    if (ShouldCalibrate()==false)
        {
        Debug.Log("...No DACs enabled, no Latency Cal");
        return;
        }

    const unsigned int kMaxTestReps = 10;
    //
    //  Apply Gain/Offset Coefficients - need this so output is on for CAL
    Hdw->ApplyDacGainCoefficients();
    //
    //  Save old dac delays, change to delay of 0.
    unsigned int Dac0Delay_cache(0);
    unsigned int Dac1Delay_cache(0);

    ReadDacDelay(Dac0Delay_cache, Dac1Delay_cache);
    LoadDacDelay(0, 0);

    //
    //  Reset DAC/FPGA
    Hdw->DacReset();
    Hdw->PreconfigureHardware();

    unsigned int should_be( GetShouldBeValue());

    FCalibrationOk = false;
    for (unsigned int i=0; i<kMaxTestReps; i++)
        {
        Debug.Log("...Latency Calibration");

        DoInternalCalibration();

        unsigned int result = GetInternalCalibrationResults();
        Debug.Log("Cal Result ", (int)result);
        if (result==should_be && ShiftCountCheck())
            {
            Debug.Log("Latency Calibration Successful");
            FCalibrationOk = true;
            break;
            }
        Debug.Log("Latency Calibration Failed");
        }

    double output_rate = Hdw->ClockFrequencyActualMhz();
    if ((output_rate>=220.0 && output_rate<=225.0)  ||
        (output_rate>=630.0 && output_rate<=650.0)      )
        {
        Debug.Log("Latency Calibration Failed - false positive");
        }
    //
    //  Restore original dac delays and apply them
    LoadDacDelay(Dac0Delay_cache, Dac1Delay_cache);
}

//------------------------------------------------------------------------------
//  DacCalibrator::ShiftCountCheck() -- require that the shift counts be small
//------------------------------------------------------------------------------

bool  DacCalibrator::ShiftCountCheck()
{
    if ((Reg->GetShiftCount(0) < 0xB) && (Reg->GetShiftCount(1) < 0xB))
        return true;
    else
        return false;
}

//------------------------------------------------------------------------------
//  DacCalibrator::DoInternalCalibration() --
//------------------------------------------------------------------------------

void DacCalibrator::DoInternalCalibration()
{
    // trigger in software, unframed mode
    Hdw->SetSoftwareUnframedTrigger();

    SetDacTestMode();

    Hdw->SetTestEnable(true);
    SetDacRunControl(true);
    Reg->CalEnable(true);

    Sleep(5);    // wait till done

    SetDacRunControl(false);
    Reg->CalEnable(false);
    Hdw->SetTestEnable(false);

}

//------------------------------------------------------------------------------
//  DacCalibrator::GetInternalCalibrationResults() --
//------------------------------------------------------------------------------

unsigned int  DacCalibrator::GetInternalCalibrationResults()
{
    return Reg->GetCalResults();
}

//------------------------------------------------------------------------------
//  DacCalibrator::GetShouldBeValue() --
//------------------------------------------------------------------------------

unsigned int  DacCalibrator::GetShouldBeValue()
{
    unsigned int should_be(0x3);
    return should_be;
}

//------------------------------------------------------------------------------
//  DacCalibrator::ShouldCalibrate() --
//------------------------------------------------------------------------------

bool  DacCalibrator::ShouldCalibrate()
{
    return Hdw->Dac(0).Enabled();
}

//---------------------------------------------------------------------------
//  DacCalibrator::SetDacTestMode()
//---------------------------------------------------------------------------

void  DacCalibrator::SetDacTestMode()
{
    // Single Device Version
    if (Hdw->Dac(0).Enabled())
        {
        if (Hdw->Dac(0).SingleActiveChannel())
            Hdw->SetTestMode_Alternating();
        else
            Hdw->SetTestMode_DoubleAlternating();
        }
}

//---------------------------------------------------------------------------
//  DacCalibrator::SetDacRunControl()
//---------------------------------------------------------------------------

void  DacCalibrator::SetDacRunControl(bool state)
{
    IDacCalibratorToHdw::EngDeviceState which = IDacCalibratorToHdw::engAll;
    if (Hdw->Dac(0).Enabled()==false)
        which = IDacCalibratorToHdw::engDevice1;

    Hdw->DeviceRun(state, which);    // start/stop Cal
}

//---------------------------------------------------------------------------
//  DacCalibrator::ReadDacDelay()
//---------------------------------------------------------------------------

void  DacCalibrator::ReadDacDelay(unsigned int & delay_0, unsigned int & delay_1)
{
    //
    //  Read dac delays
    delay_0 = delay_1 = Hdw->Dac(0).DacDelay();
}

//---------------------------------------------------------------------------
//  DacCalibrator::LoadDacDelay()
//---------------------------------------------------------------------------

void  DacCalibrator::LoadDacDelay(unsigned int delay, unsigned int)
{
    //
    //  Restore original dac delays and apply them
    Hdw->Dac(0).LoadDacDelay(delay);
}

//===========================================================================
//  CLASS DualDacCalibrator  -- DAC calibration procedure for TiDac6682
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class DualDacCalibrator
//---------------------------------------------------------------------------

DualDacCalibrator::DualDacCalibrator(IDacCalibrationRegister * reg, IDacCalibratorToHdw * hdw)
    :  DacCalibrator(reg, hdw)
{
}

//---------------------------------------------------------------------------
//  destructor for class DualDacCalibrator
//---------------------------------------------------------------------------

DualDacCalibrator::~DualDacCalibrator()
{
}

//------------------------------------------------------------------------------
//  DualDacCalibrator::GetShouldBeValue() --
//------------------------------------------------------------------------------

unsigned int  DualDacCalibrator::GetShouldBeValue()
{
    unsigned int should_be(0xF);
    if (Hdw->Dac(0).Enabled()==false)
        should_be = 0xC;
    else if (Hdw->Dac(1).Enabled()==false)
        should_be = 0x3;

    return should_be;
}

//------------------------------------------------------------------------------
//  DualDacCalibrator::ShouldCalibrate() --
//------------------------------------------------------------------------------

bool  DualDacCalibrator::ShouldCalibrate()
{
    return Hdw->Dac(0).Enabled() || Hdw->Dac(1).Enabled() ;
}

//---------------------------------------------------------------------------
//  DualDacCalibrator::SetDacTestMode()
//---------------------------------------------------------------------------

void  DualDacCalibrator::SetDacTestMode()
{
    if (Hdw->Dac(0).Enabled())
        {
        if (Hdw->Dac(0).SingleActiveChannel())
            Hdw->SetTestMode_Alternating();
        else
            Hdw->SetTestMode_DoubleAlternating();
        }
    else  // Use Dac 1
        {
        if (Hdw->Dac(1).SingleActiveChannel())
            Hdw->SetTestMode_Alternating();
        else
            Hdw->SetTestMode_DoubleAlternating();
        }
}

//---------------------------------------------------------------------------
//  DualDacCalibrator::SetDacRunControl()
//---------------------------------------------------------------------------

void  DualDacCalibrator::SetDacRunControl(bool state)
{
    IDacCalibratorToHdw::EngDeviceState which = IDacCalibratorToHdw::engAll;
    if (Hdw->Dac(0).Enabled()==false)
        which = IDacCalibratorToHdw::engDevice1;
    if (Hdw->Dac(1).Enabled()==false)
        which = IDacCalibratorToHdw::engDevice0;

    Hdw->DeviceRun(state, which);    // start/stop Cal

}

//---------------------------------------------------------------------------
//  DualDacCalibrator::ReadDacDelay()
//---------------------------------------------------------------------------

void  DualDacCalibrator::ReadDacDelay(unsigned int & delay_0, unsigned int & delay_1)
{
    //
    //  Read dac delays
    delay_0 = Hdw->Dac(0).DacDelay();
    delay_1 = Hdw->Dac(1).DacDelay();
}

//---------------------------------------------------------------------------
//  DualDacCalibrator::LoadDacDelay()
//---------------------------------------------------------------------------

void  DualDacCalibrator::LoadDacDelay(unsigned int delay_0, unsigned int delay_1)
{
    //
    //  Restore original dac delays and apply them
    Hdw->Dac(0).LoadDacDelay(delay_0);
    Hdw->Dac(1).LoadDacDelay(delay_1);
}

} // namespace TiDac6682

} // namespace Innovative

