// TiDac5682z_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef TiDac5682z_MbH
#define TiDac5682z_MbH

#include <Trace_Mb.h>
#include <X6Support_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

namespace TiDac5682
{
//===========================================================================
//  CLASS IDacToHardware  -- Interface to link device to rest of system
//===========================================================================

class IDacToHardware
{
public:

    virtual unsigned short  Spi(int addr) = 0;
    virtual void            Spi(int addr, unsigned short value) = 0;

    virtual double          DCLK(bool single_channel) = 0;
    virtual bool            IsDacDllEnabled(bool single_channel) = 0;
    virtual unsigned int    FastConfigValue() = 0;

    virtual ~IDacToHardware()
        {}
};

//===========================================================================
//  CLASS DacDevice  -- Per-DAC controls for TiDac6682
//===========================================================================

class DacDevice
{

public:
    DacDevice(IDacToHardware * hdw, bool is5681=false);
    virtual ~DacDevice();

    enum Constants { DeviceChannels = 2 };

    // Methods
    void                Enabled(bool value);
    bool                Enabled()
                            {  return FEnabled;  }
    void                ChannelEnabled(int idx, bool value);
    bool                ChannelEnabled(int idx)
                            {  return FChannelEnabled[idx];  }
    bool                SingleActiveChannel();

    void                ClearDacTestStatus();
	unsigned int        DacTestStatus();
	bool				IsDacTestStatusOk();

    void                DacDelay(unsigned int delay)
                            {  FDacDelay = delay;  }
    unsigned int        DacDelay() const
                            {  return FDacDelay;  }
    void                LoadDacDelay(unsigned int delay);

    void                PreconfigHardware();
    void                ConfigureHardware();

protected:
    IDacToHardware *    Hdw;
    bool                DevIs5681;
    bool                FEnabled;
    unsigned int        FDacDelay;
    bool                FChannelEnabled[DeviceChannels];
    Trace               Debug;

    void            SingleChannelMode(int active_chan);
    void            DualChannelMode();
    void            Spi10Setup(double FreqMhz);


};

//===========================================================================
//  CLASS IDacCalibratorToHdw  -- Interface of Calibrator back to Hardware
//===========================================================================

class IDacCalibratorToHdw
{
public:
    enum EngDeviceState  { engDevice0, engDevice1, engAll };

    virtual void          ApplyDacGainCoefficients() = 0;
    virtual void          DacReset() = 0;
    virtual void          PreconfigureHardware() = 0;
    virtual double        ClockFrequencyActualMhz() = 0;

    virtual void          SetTestMode_Alternating() = 0;
    virtual void          SetTestMode_DoubleAlternating() = 0;
    virtual void          SetTestEnable(bool state) = 0;
    virtual void          SetSoftwareUnframedTrigger() = 0;
    virtual void          DeviceRun(bool state, EngDeviceState which) = 0;

    virtual DacDevice &   Dac(unsigned int idx) = 0;

    virtual ~IDacCalibratorToHdw()
        {}
};

//===========================================================================
//  CLASS DacCalibrator  -- DAC calibration procedure for TiDac6682
//===========================================================================

class DacCalibrator : public IDacCalibrator
{
public:
    DacCalibrator(IDacCalibrationRegister * reg, IDacCalibratorToHdw * hdw);
    virtual ~DacCalibrator();

    virtual void  Calibrate();

protected:
    IDacCalibrationRegister *  Reg;
    IDacCalibratorToHdw *      Hdw;

    Trace         Debug;

    void          DoInternalCalibration();
    unsigned int  GetInternalCalibrationResults();

    //  Single/Dual Device methods
    virtual unsigned int GetShouldBeValue();
    virtual bool         ShouldCalibrate();
            bool         ShiftCountCheck();

    virtual void  SetDacTestMode();
    virtual void  SetDacRunControl(bool state);
    virtual void  ReadDacDelay(unsigned int & delay_0, unsigned int & delay_1);
    virtual void  LoadDacDelay(unsigned int delay_0, unsigned int delay_1);
};

//===========================================================================
//  CLASS DualDacCalibrator  -- DAC calibration procedure for dual TiDac6682
//===========================================================================

class DualDacCalibrator : public DacCalibrator
{
public:
    DualDacCalibrator(IDacCalibrationRegister * reg, IDacCalibratorToHdw * hdw);
    virtual ~DualDacCalibrator();

protected:
    //  Single/Dual Device methods
    virtual unsigned int GetShouldBeValue();
    virtual bool         ShouldCalibrate();

    virtual void  SetDacTestMode();
    virtual void  SetDacRunControl(bool state);
    virtual void  ReadDacDelay(unsigned int & delay_0, unsigned int & delay_1);
    virtual void  LoadDacDelay(unsigned int delay_0, unsigned int delay_1);
};

} // end namespace TiDac6682


//---------------------------------------------------------------------------
#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif


}; // end namespace Innovative
//---------------------------------------------------------------------------
#endif

