#include "TraceLogger_Mb.h"


#include "Poco/FileChannel.h"
#include "Poco/NullChannel.h"
#include "Poco/PatternFormatter.h"
#include "Poco/FormattingChannel.h"
#include "Poco/Message.h"
#include "Poco/Logger.h"

using Poco::FormattingChannel;
using Poco::Logger;
using Poco::FileChannel;
using Poco::NullChannel;
using Poco::PatternFormatter;
using Poco::Message;

namespace Innovative
{

//==============================================================================
//  CLASS YaloImpl --
//==============================================================================
//---------------------------------------------------------------------------
//  YaloImpl::YaloImpl() --
//---------------------------------------------------------------------------
YaloImpl::YaloImpl()
{
   _channel = new FormattingChannel(new PatternFormatter("%s: %p: %t"));

   _channel->setChannel(new NullChannel());
   _channel->open();

   Logger::create("ConsoleLogger", _channel, Message::PRIO_INFORMATION);

}

//---------------------------------------------------------------------------
//  YaloImpl::ChannelToFile() --
//---------------------------------------------------------------------------
void YaloImpl::ChannelToFile()
{
    _channel->close();
    _channel->setChannel(new FileChannel("console.log"));
    _channel->open();
}

//---------------------------------------------------------------------------
//  YaloImpl::Error() --
//---------------------------------------------------------------------------
void YaloImpl::Error(const std::string &s)
{
    Logger::get("ConsoleLogger").error(s);
}

//---------------------------------------------------------------------------
//  YaloImpl::Warning() --
//---------------------------------------------------------------------------
void YaloImpl::Warning(const std::string &s)
{
    Logger::get("ConsoleLogger").warning(s);
}

//---------------------------------------------------------------------------
//  YaloImpl::Information() --
//---------------------------------------------------------------------------
void YaloImpl::Information(const std::string &s)
{
    Logger::get("ConsoleLogger").information(s);
}

//---------------------------------------------------------------------------
//  Yalo() --
//---------------------------------------------------------------------------
YaloImpl &Yalo()
{
    static YaloImpl *yp = new YaloImpl();
    return *yp;
}


}  // namespace
