//---------------------------------------------------------------------------
// Trace_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#include <Trace_Mb.h>
#include <iomanip>
#include <sstream>
#include <memory>
#include <DebugSupport_Mb.h>

#include <Osal_Mb.h>

namespace Innovative
{

unsigned int Trace::FVerbosity = Trace::vSilent;

//---------------------------------------------------------------------------
//  TraceVerbosity() --  Set global trace verbosity
//---------------------------------------------------------------------------

void TraceVerbosity(unsigned int level)
{
    Trace::FVerbosity = level;
}

//---------------------------------------------------------------------------
//  TraceVerbosity() --  read global trace verbosity
//---------------------------------------------------------------------------

unsigned int TraceVerbosity()
{
    return  Trace::FVerbosity;
}

//==============================================================================
//  CLASS Trace -- Persistent trace log
//==============================================================================

//---------------------------------------------------------------------------
//  Trace::Trace() --
//---------------------------------------------------------------------------

Trace::Trace(bool enabled,  enum PriorityLevels default_priority)
	: FEnabled(enabled), FAutoCr(false), FDefaultPriority(default_priority)
{
}

//---------------------------------------------------------------------------
//  Trace::LogOutput() --
//---------------------------------------------------------------------------

const InnovativeKernel::Osal::OsSupport &Trace::LogOutput() const
{
	static const InnovativeKernel::Osal::OsSupport *out = new InnovativeKernel::Osal::OsSupport;

	return *out;
}

//---------------------------------------------------------------------------
// Trace::Enabled() --  Enable output messages
//---------------------------------------------------------------------------

void Trace::Enabled(bool state)
{
    FEnabled=state;
}

//---------------------------------------------------------------------------
// Trace::Enabled() --  Query output enable state
//---------------------------------------------------------------------------

bool Trace::Enabled() const
{
    return FEnabled;
}

//---------------------------------------------------------------------------
// Trace::Prefix() --   Specify prefix text to all output messages
//---------------------------------------------------------------------------

void Trace::Prefix(const std::string & value)
{
    FPrefix=value;
}

//---------------------------------------------------------------------------
// Trace::Prefix() --  Query prefix text to all output messages
//---------------------------------------------------------------------------

std::string Trace::Prefix() const
{
    return FPrefix;
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s) const
{
    Spit(s);    // WillTrace() is checked by Spit() too.
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, std::string y) const
{
    if (!WillTrace())
        return;

    std::string Msg = s + " " + y;
    Spit(Msg.c_str());
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, bool state) const
{
    if (!WillTrace())
        return;

    std::string Msg = s + " " + (state ? "true" : "false");
    Spit(Msg.c_str());
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, int value, int radix) const
{
    if (!WillTrace())
        return;

	std::stringstream msg;
	switch (radix)
		{
		default:
		case 10:
			msg << s << " [" << std::dec << value << "]";
			break;
		case 16:
			msg << s << " [0x" << std::hex << value << "]";
			break;
		case 8:
			msg << s << " [0" << std::oct << value << "]";
			break;
		}

	Spit(msg.str());
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, ii64 value, int radix) const
{
    if (!WillTrace())
        return;

	std::stringstream msg;
	switch (radix)
		{
		default:
		case 10:
			msg << s << " [" << std::dec << value << "]";
			break;
		case 16:
			msg << s << " [0x" << std::hex << value << "]";
			break;
		case 8:
			msg << s << " [0" << std::oct << value << "]";
			break;
		}

	Spit(msg.str());
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, s_ii64 value, int radix) const
{
    if (!WillTrace())
        return;

    std::stringstream msg;
    switch (radix)
        {
        default:
        case 10:
            msg << s << " [" << std::dec << value << "]";
            break;
        case 16:
            msg << s << " [0x" << std::hex << value << "]";
            break;
        case 8:
            msg << s << " [0" << std::oct << value << "]";
            break;
        }

    Spit(msg.str());
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, float value) const
{
    if (!WillTrace())
        return;

	std::stringstream msg;
	msg << s << " [" << std::setprecision(7) << value << "]";
	Spit(msg.str());
}

//---------------------------------------------------------------------------
// Trace::Log() --  Debug message display
//---------------------------------------------------------------------------

void Trace::Log(const std::string & s, double value) const
{
    if (!WillTrace())
        return;

	std::stringstream msg;
	msg << s << " [" << std::setprecision(14) << value << "]";
	Spit(msg.str());
}

//---------------------------------------------------------------------------
//  Trace::PushPriority() -- Change current priority
//---------------------------------------------------------------------------

void  Trace::PushPriority(unsigned int priority)
{
    FPriority.push(priority);
}

//---------------------------------------------------------------------------
//  Trace::PopPriority() --  Remove current priority, restoring one before
//---------------------------------------------------------------------------

void  Trace::PopPriority()
{
    if (!FPriority.empty())
        FPriority.pop();
}

//---------------------------------------------------------------------------
//  Trace::Priority() --  Returns current priority of the Trace
//---------------------------------------------------------------------------

unsigned int  Trace::Priority() const
{
    if (FPriority.empty())
        return FDefaultPriority;
    else
        return FPriority.top();   // return latest priority
}

//---------------------------------------------------------------------------
//  Trace::WillTrace() --  Returns TRUE if a trace message will display
//---------------------------------------------------------------------------

bool  Trace::WillTrace() const
{
    if (!FEnabled)
        return false;
    if (Verbosity() < Priority())
        return false;
    else
        return true;
}

//---------------------------------------------------------------------------
//  Trace::Spit() --  Display debug message
//---------------------------------------------------------------------------

void Trace::Spit(const std::string & s) const
{
    if (!WillTrace())
            return;

        std::stringstream txt;

        if (FPrefix.size())
            txt << FPrefix << " | " << s;
        else
            txt << s;

       LogOutput().Trace( AppendThreadId(s).c_str() );
}

//---------------------------------------------------------------------------
// Trace::Verbosity() --  Retreive debug level
//---------------------------------------------------------------------------

unsigned int Trace::Verbosity() const
{
    return FVerbosity;
}

//==============================================================================
//  CLASS ScopedTrace -- Scope of object reported with Trace log
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for ScopedTrace
//---------------------------------------------------------------------------

ScopedTrace::ScopedTrace(const std::string &text)
    :  Text(text)
{
    Debug.Log(std::string("+") + Text);
}

//---------------------------------------------------------------------------
//  destructor for ScopedTrace
//---------------------------------------------------------------------------

ScopedTrace::~ScopedTrace()
{
    Debug.Log(std::string("-") + Text);
}

//==============================================================================
//  CLASS ScopedTraceRef -- Scope of object reported with Trace log
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for ScopedTraceRef
//---------------------------------------------------------------------------

ScopedTraceRef::ScopedTraceRef(Trace & debug, const std::string &text)
    :  Text(text), Debug(debug)
{
    Debug.Log(std::string("+") + Text);
}

//---------------------------------------------------------------------------
//  destructor for ScopedTraceRef
//---------------------------------------------------------------------------

ScopedTraceRef::~ScopedTraceRef()
{
    Debug.Log(std::string("-") + Text);
}


}  // namespace Innovative

