// Tuner_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef Tuner_MbH
#define Tuner_MbH

#include <string>
#include <Events_Mb.h>
#include <K7Support_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
// CLASS TunerMessageList  --  alias to list of strings
//===========================================================================

typedef std::vector<std::string>  TunerMessageList;

//===========================================================================
// CLASS ITunerMessageParser  --  Interface to string parser
//===========================================================================

class ITunerMessageParser
{
public:
    virtual ~ITunerMessageParser()  {  }
    virtual std::string EndToken() = 0;

	virtual void  Parse(std::string & queue, TunerMessageList & messages) = 0;
};

//===========================================================================
// CLASS TunerOutputEvent  --
//===========================================================================

class TunerOutputEvent : public OpenWire::Event
{
public:
	TunerMessageList list;

	TunerOutputEvent(const TunerMessageList &l)
		:  list(l)
        {
        }
};

//===========================================================================
//  CLASS ITuner  -- Interface to various tuner hardwares
//===========================================================================

class ITuner
{
public:
    enum BaudRates { b115200=115200, b57600=57600, b38400=38400,
                     b19200=19200, b9600=9600, b4800=4800 };

	ITuner();
	ITuner(IMulticharUart * uart);
	ITuner(IMulticharUart * uart, ITunerMessageParser * parser);
	virtual ~ITuner();

	void    Init();
	void    Reapply();
	void    Close();

    //  Events
    OpenWire::ThunkedEventHandler<Innovative::TunerOutputEvent> OnOutput;

	//  Command Interface
	virtual void  Reset() = 0;
	virtual void  SetEcho(bool state) = 0;
	virtual void  SetTuningFrequency(double freq) = 0;
	virtual void  SetAttenuation(double attenuation_db) = 0;

	// Queries
	virtual void  GetId() = 0;
	virtual void  GetStatus() = 0;

    //  Properties
	void  Enabled(bool state)  { FEnabled = state;  Queue.erase();  }
	bool  Enabled()            { return FEnabled; }

/*
    void      Baud(BaudRates r)
    { rate = r; }

    BaudRates Baud()
    { return rate; }

*/

    void            Command(const std::string & txt, string &OutputStr, unsigned int delay = 50);
    void            Command(const std::string & txt);

	void            CustomParser(ITunerMessageParser * newParser);

protected:

	IMulticharUart *       Uart;
	ITunerMessageParser *  Parser;
	std::string            Queue;

	bool                   FEnabled;

	//  Derived Class Interface
	virtual void    OnInit() = 0;
	virtual void    OnReapply() = 0;
	virtual void    OnClose() = 0;
	virtual void    OnProcessMessages(TunerMessageList & Messages) = 0;

public:   // Internals
	void   DispatchTunerInterrupt();

};

//===========================================================================
// CLASS StandardTunerMessageParser  --  Normal Message String Parser
//===========================================================================

class StandardTunerMessageParser : public ITunerMessageParser
{
public:
	StandardTunerMessageParser();
	StandardTunerMessageParser(const std::string & end_token);
	virtual ~StandardTunerMessageParser()  {  }

	virtual std::string EndToken()  { return CmdEndToken; }
	virtual void  Parse(std::string & queue, TunerMessageList & messages);

private:
	std::string  CmdEndToken;

	void  ParseMessage(std::string text, TunerMessageList & messages);
	//Trace Debug;
};


}; //Innovative

#endif
