// TxDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <TxDeviceMap_Mb.h>
#include <Tx_Mb.h>
#include <TxMemoryMap.h>
#include <TxDeviceDriver_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <PmcIdrom_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS TxHdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class TxHdwPluginImpl : public IPacketDevicePlugin
{
public:
    TxHdwPluginImpl( TxDeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    TxDeviceMap * PDM;
};

//===========================================================================
//  CLASS TxDeviceMap  -- Registers for Tx Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class TxDeviceMap
//---------------------------------------------------------------------------

using namespace Tx_MemoryMap;

PacketDeviceRegisters TxDRegAddrs =
    {
	mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	0,                    // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters TxRegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };


const int  TxDeviceChannelCount = 4;

//---------------------------------------------------------------------------
//  constructor for class TxDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

TxDeviceMap::TxDeviceMap(Tx * owner)
    :  PacketDeviceImpl(new TxDeviceDriver, TxDRegAddrs),
       PacketDeviceMap(this, TxRegAddrs, 0),
       IUsesVirtexFpgaLoader(PortMemory, mmFpgaData, mmFpgaControl),
       IUsesVirtexJtagLoader(PortMemory, mmJtag),
       IUsesPllCalibration(PortMemory, mmI2c, 128, 4, 0x4000),
       IUsesFpgaLogicVersionInfo(LogicMemory, mmFpgaVersion),
       IUsesPciLogicVersionInfo(PortMemory, mmPciVersion),
       Owner(owner), 
       DDeviceDriver(dynamic_cast<TxDeviceDriver*>(DeviceDriver)),
       Alert(this),
       RcvPacketSize(0x80), XmitPacketSize(0x80),
       Ram(LogicMemory, mmZbtSramAddress, mmZbtSramReadData, mmZbtSramWriteData),
       FChannels(2, 16, 0, false), FSincFilterEnabled(false),
       PciTestControlReg    (PortMemory,  mmPciTestControl),
	   UserFpgaControlReg   (LogicMemory, mmUserControl),
       GainCoeffReg         (LogicMemory, mmDacGainCoeff0, TxDeviceChannelCount),
       OffsetCoeffReg       (LogicMemory, mmDacOffsetCoeff0, TxDeviceChannelCount),
       ChannelMaskReg       (LogicMemory, mmChannelMask),
       ClockSelectReg       (LogicMemory, mmClockSelect),
       DacReg01             (LogicMemory, mmDacChannels01, 0x20),
       DacReg23             (LogicMemory, mmDacChannels23, 0x20),
       Clock(0, LogicMemory, mmClockControl),
       ClockTestReg         (LogicMemory, mmClockTest)
{
    ChannelMaskReg.Value(0);

	//
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this, &TxDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &TxDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &TxDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &TxDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &TxDeviceMap::OnAfterStreamStopHandler );

    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new TxHdwPluginImpl(this) );
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class TxDeviceMap
//---------------------------------------------------------------------------

TxDeviceMap::~TxDeviceMap()
{
}

//---------------------------------------------------------------------------
//  TxDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  TxDeviceMap::OpenHardware()
{
    //if ( DeviceDriver->Opened() )     ??? Needed?
     //   return;
    //
    //  Init Calibration Interface 
    IUsesCalibration::AfterOpen();
    //
    //  Alert Interface
    Alert.Open();

}

//---------------------------------------------------------------------------
//  TxDeviceMap::AlertTest() -- Alert Test Processing Hook
//---------------------------------------------------------------------------

bool  TxDeviceMap::AlertTest(unsigned int pid)
{
    //  return TRUE if PID is an alert PID.
    //  return FALSE to if not
    return Alert.AlertTest(pid);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::AlertCheck() -- Alert Event Processing Hook
//---------------------------------------------------------------------------

bool  TxDeviceMap::AlertCheck(const Buffer & buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alert.CheckAndProcess(buffer);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  TxDeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<TxDeviceDriver::LOG->Count; i++)
        vec.push_back(TxDeviceDriver::LOG->Status[i]);

}

//---------------------------------------------------------------------------
//  TxDeviceMap::LocalLoopbackMode()
//---------------------------------------------------------------------------

void  TxDeviceMap::LocalLoopbackMode(bool /*state*/)
{
    // not implemented
}

//---------------------------------------------------------------------------
//  TxDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  TxDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    //  ...Reset DAC and FPGA
    UserFpgaControlReg.FpgaReset.Set();
    UserFpgaControlReg.FpgaReset.Clear();

    //
    //  Disable run
    UserFpgaControlReg.Go.Clear();

    UserFpgaControlReg.ResetFifo(UserFpgaCtlRegister::dpBothPairs);
    UserFpgaControlReg.ResetFifo(UserFpgaCtlRegister::dpNoPair);

    ChannelMaskReg.Apply();

    switch (FChannels.ActiveDevices())
        {
        case 0:
            UserFpgaControlReg.Enable(UserFpgaCtlRegister::dpNoPair);
            break;
        case 1:
            UserFpgaControlReg.Enable(UserFpgaCtlRegister::dpPair0Only);
            break;
        default:
        case 2:
            UserFpgaControlReg.Enable(UserFpgaCtlRegister::dpBothPairs);
            break;
        }

    //
    //  Apply Control Register Stuff
    PciTestControlReg.Apply();

    //
    //  Apply Gain/Offset Coefficients
    ApplyRomCoefficients();

    //
    //  Configure Alerts
    Alert.Configure();

    //
    //  Apply Clock Settings from saved state
    Clock.Reapply();
    ClockSelectReg.Apply();
    // Allow PLL to stabilize
    Sleep(40);

    UserFpgaControlReg.ResetDac(UserFpgaCtlRegister::dpBothPairs);
    UserFpgaControlReg.ResetDac(UserFpgaCtlRegister::dpNoPair);

    //
    //  Set loopback mode
    PciLoopbackMode(Owner->FLoopbackMode);

    //  ...Set up DAC configuration for each DAC pair
    InitializeDac(0);
    InitializeDac(1);

    Trigger(false);

    //  Forward notify to baseboard handler
    Owner->OnStreamConfigure.Execute(Event);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  TxDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);

    //  ...Start
    UserFpgaControlReg.Go.Set();
}

//---------------------------------------------------------------------------
//  TxDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  TxDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  TxDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);

    UserFpgaControlReg.Go.Clear();
    UserFpgaControlReg.Enable(UserFpgaCtlRegister::dpNoPair);

    Trigger(false);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  TxDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::InitializeDac() - set up dac pair configuration
//---------------------------------------------------------------------------

void  TxDeviceMap::InitializeDac(int pair)
{
    DacReg(pair, 0x00, 0x20);  //  software reset of DAC SPI register space
    Sleep(1);
    DacReg(pair, 0x00, 0x00);  //  no reset, no power-down, SPI read
		                              //     capability, MSB first
    DacReg(pair, 0x01, 0x00);  //  no zero stuffing, filter interpolation = 1x,
                                      //	 divisor = 1
	DacReg(pair, 0x02, FSincFilterEnabled ? 0x0C : 0x04);
                                //  I first, no dataclk invert, sinc filter,
									  //  	enable Q path, enable both input data ports,
									  //	signed binary data, enable DACCLK_out delay
									  //    data clock delay enabled.
	DacReg(pair, 0x03, 0x84);  //  divider ratio = 1, timing error correction
									  //    disabled
									  //    Data clock delay mode = auto.
									  //    Data timing margin = 800psecs.
	DacReg(pair, 0x04, 0x70);  //  data clock delay = 0, sync_out divisor = 1
    DacReg(pair, 0x05, 0x08);  //  sync out delay = 0, input sync divisor = /2
    DacReg(pair, 0x06, 0x00);  //  sync input delay = 0, don't care
    DacReg(pair, 0x07, 0x00);  //  sync Rx disable, sync Tx disable, don't care
    DacReg(pair, 0x08, 0xcf);  //  Fvco = ~900 Mhz, low VCO AGC gain
    DacReg(pair, 0x09, 0x07);  //  disable VCO, don't care for all divisors
    DacReg(pair, 0x0A, 0x00);  //  PLL disabled: don't care for loop filter &
	                                  // 	control voltage
    DacReg(pair, 0x0B, 0xF9);  //  I-DAC, default gain
    DacReg(pair, 0x0C, 0x01);  //  I-DAC, DAC on
    DacReg(pair, 0x0F, 0xF9);  //  Q-DAC, default gain
    DacReg(pair, 0x10, 0x01);  //  Q-DAC, DAC on
}

//---------------------------------------------------------------------------
//  TxDeviceMap::SetCalibrationOffset()
//---------------------------------------------------------------------------

void TxDeviceMap::SetCalibrationOffset(int ch, int value)
{
    OffsetCoeffReg[ch].Value(value);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::SetCalibrationGain()
//---------------------------------------------------------------------------

void TxDeviceMap::SetCalibrationGain(int ch, int value)
{
	GainCoeffReg[ch].Value(value);
}

//------------------------------------------------------------------------------
//  TxDeviceMap::SetCalibrationPllCorrection() -- apply to hardware
//------------------------------------------------------------------------------

void TxDeviceMap::SetCalibrationPllCorrection(float value)
{
	Clock.ReferenceCalibrationFactor(value);
}

//------------------------------------------------------------------------------
//  TxDeviceMap::PatternGenMode() --  Set pattern generation mode
//------------------------------------------------------------------------------

void TxDeviceMap::PatternGenerator(bool state)
{
    UserFpgaControlReg.PatternGenerator(state);
}

//------------------------------------------------------------------------------
//  TxDeviceMap::PatternGenMode() --  Retreive pattern generation mode
//------------------------------------------------------------------------------

bool TxDeviceMap::PatternGenerator() const
{
    return UserFpgaControlReg.PatternGenerator();
}

//------------------------------------------------------------------------------
//  TxDeviceMap::DacReg() --
//------------------------------------------------------------------------------

void TxDeviceMap::DacReg(int pair, int offset, int value)
{
    if (pair)
        DacReg23[offset].Value(value);
    else
        DacReg01[offset].Value(value);
}

//------------------------------------------------------------------------------
//  TxDeviceMap::DacReg() --
//------------------------------------------------------------------------------

int TxDeviceMap::DacReg(int pair, int offset) const
{
    int result;

    if (pair)
        result = DacReg23[offset].Value();
    else
        result = DacReg01[offset].Value();

    return result;
}

//---------------------------------------------------------------------------
//  TxDeviceMap::ClockTestEnable() --
//---------------------------------------------------------------------------

void TxDeviceMap::ClockTestEnable(bool state)
{
    ClockTestReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::ClockTestEnable() --
//---------------------------------------------------------------------------

bool TxDeviceMap::ClockTestEnable() const
{
	return ClockTestReg.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  TxDeviceMap::UserLoopbackMode() --
//---------------------------------------------------------------------------

void TxDeviceMap::UserLoopbackMode(bool state)
{
	UserFpgaControlReg.RocketIoLoopback.Value(state);
}

//---------------------------------------------------------------------------
//  TxDeviceMap::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool TxDeviceMap::UserLoopbackMode() const
{
	return UserFpgaControlReg.RocketIoLoopback.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  TxDeviceMap::SincFilterEnabled() --
//---------------------------------------------------------------------------

void TxDeviceMap::SincFilterEnabled(bool state)
{
    FSincFilterEnabled = state;
}

//---------------------------------------------------------------------------
//  TxDeviceMap::SincFilterEnabled() --
//---------------------------------------------------------------------------

bool TxDeviceMap::SincFilterEnabled() const
{
    return FSincFilterEnabled;
}


//===========================================================================
//  CLASS TxDeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Tx::TxDeviceMap::PciTestRegister
//---------------------------------------------------------------------------

TxDeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      GenPacketSize( *this, 8, 4 ),
      GenPacketSizeUpdate( *this, 12 )
{
}

//===========================================================================
//  CLASS TxAlertSystem
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class TxAlertSystem
//---------------------------------------------------------------------------

const int TxAlertPid = 0xff;

//---------------------------------------------------------------------------
//  constructor for class TxAlertSystem
//---------------------------------------------------------------------------

TxAlertSystem::TxAlertSystem(TxDeviceMap * device)
    :  AlertSystem( TxAlertPid), Device(device),
       AlertControlReg( device->LogicMemory, mmAlertControl ),
       AlertEnableReg(  device->LogicMemory, mmAlertEnable ),
       SoftwareAlertReg( device->LogicMemory, mmAlertSWTrigger )
{
    // Load Alerts
    LoadAlert(AlertData((int)alertTimeStampRollover, 0, 0x01, false, &OnTimeStampRolloverAlert));
    LoadAlert(AlertData((int)alertSoftware,          1, 0x02, false, &OnSoftwareAlert));
    LoadAlert(AlertData((int)alertPdn0Underflow,     3, 0x08, false, &OnPdn0UnderflowAlert));
    LoadAlert(AlertData((int)alertPdn1Underflow,     4, 0x10, false, &OnPdn1UnderflowAlert));
    LoadAlert(AlertData((int)alertPdn0NeedsData,     5, 0x20, false, &OnPdn0NeedsDataAlert));
    LoadAlert(AlertData((int)alertPdn1NeedsData,     6, 0x40, false, &OnPdn1NeedsDataAlert));
}

//---------------------------------------------------------------------------
//  TxAlertSystem::AlertReset() - Reset Alert Fifo
//---------------------------------------------------------------------------

void  TxAlertSystem::AlertReset()
{
    AlertControlReg.AlertFifoReset.Set();
    AlertControlReg.AlertFifoReset.Clear();
}

//---------------------------------------------------------------------------
//  TxAlertSystem::SoftwareAlert() - Reset Alert Fifo
//---------------------------------------------------------------------------

void  TxAlertSystem::SoftwareAlert(unsigned int tag)
{
    SoftwareAlertReg.Value(tag);
}

//---------------------------------------------------------------------------
//  TxAlertSystem::Open() - Open-Time behavior
//---------------------------------------------------------------------------

void  TxAlertSystem::Open()
{
    // nothing special
}

//---------------------------------------------------------------------------
//  TxAlertSystem::Configure() - Configure-Time behavior
//---------------------------------------------------------------------------

void  TxAlertSystem::Configure()
{
    AlertReset();
    //
    //  Turn on Timestamp Clock
    AlertControlReg.TimestampOn.Set();

    // Enable all configured alerts
    for (unsigned int i=0; i<Alerts.size(); ++i)
        AlertEnableReg.Bit(Alerts[i].EnableBit, Alerts[i].Enabled);

}

//===========================================================================
//  CLASS TxAlertSystem::AlertControlsRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class TxAlertSystem::AlertControlsRegister
//---------------------------------------------------------------------------

TxAlertSystem::AlertControlsRegister::AlertControlsRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      AlertFifoReset( *this, 1 ),
      TimestampOn( *this, 0 )
{
}

//===========================================================================
//  CLASS TxAlertSystem::AlertEnableRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class TxAlertSystem::AlertEnableRegister
//---------------------------------------------------------------------------

TxAlertSystem::AlertEnableRegister::AlertEnableRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TimestampRollover( *this, 0 ),
      Pdn0UnderflowAlert( *this, 3 ),
      Pdn1UnderflowAlert( *this, 4 ),
      Pdn0NeedsDataAlert( *this, 5),
      Pdn1NeedsDataAlert( *this, 6)

{
}

} // namespace Innovative


