// TxDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef TxDeviceMap_MbH
#define TxDeviceMap_MbH

#include <PacketDeviceMap_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PllCalMem_Mb.h>
#include <ICS8442_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <ZbtRam_Mb.h>
#include <ChannelInfo_Mb.h>
#include <TxAlerts_Mb.h>
#include <AlertSystem_Mb.h>
#include <PacketDeviceImpl_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class TxDeviceDriver;
class Event;
class Buffer;

//===========================================================================
//  CLASS TxAlertSystem  -- Tx Alert Processing
//===========================================================================

class TxAlertSystem : public AlertSystem, public IUsesTxAlerts
{
public:
    TxAlertSystem(class TxDeviceMap * device);
    virtual ~TxAlertSystem()
    	{  }

    virtual void  AlertEnable(AlertType alert, bool state)
                       {  DoAlertEnable((int)alert, state);  }
    virtual void  AlertPassThru(AlertType alert, bool state)
                       {  DoAlertPassThru((int)alert, state);  }
    virtual void  AlertReset();

	virtual void  Open();
    virtual void  Configure();
    virtual void  SoftwareAlert(unsigned int tag=0);

private:
    //
    //  Class AlertControlsRegister
	class AlertControlsRegister : public Register
	{
        typedef Register inherited;

    public:
        RegisterBit   AlertFifoReset;
        RegisterBit   TimestampOn;

    public:
        AlertControlsRegister( AddressingSpace &space, int offset );
    };
    //
    //  Class AlertEnableRegister
    class AlertEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   TimestampRollover;
        RegisterBit   Pdn0UnderflowAlert;
        RegisterBit   Pdn1UnderflowAlert;
        RegisterBit   Pdn0NeedsDataAlert;
        RegisterBit   Pdn1NeedsDataAlert;

    public:
        AlertEnableRegister( AddressingSpace &space, int offset );
    };

    TxDeviceMap *          Device;
    AlertControlsRegister  AlertControlReg;
    AlertEnableRegister    AlertEnableReg;
    Register               SoftwareAlertReg;

};

//===========================================================================
//  CLASS TxDeviceMap  -- Register & low level support for Tx
//===========================================================================
class Tx;

class TxDeviceMap :   public PacketDeviceImpl,
                      public PacketDeviceMap,
                      public IUsesVirtexFpgaLoader,
                      public IUsesVirtexJtagLoader,
                      public IUsesPllCalibration,
                      public IUsesFpgaLogicVersionInfo,
                      public IUsesPciLogicVersionInfo
{
    friend class Tx;

public:
    enum IITestMode { tmNormal, tmRamp, tmSine };
	enum IIExtTrigMode{etLevel, etEdge};
	
    static const int NumPairs = 2;
    static const int NumChannels = NumPairs * 2;

    //
    //  Member Functions
	TxDeviceMap(Tx * owner);
    virtual ~TxDeviceMap();

    void  OpenHardware();
    //
    //  Busmaster Intf functions
	virtual bool  AlertCheck(const Buffer & buffer);
    virtual bool  AlertTest(unsigned int pid);

	void 	UserLoopbackMode(bool state);
	bool	UserLoopbackMode() const;

	void 	SincFilterEnabled(bool state);
	bool	SincFilterEnabled() const;

	//
	//  Make this property public
	using PacketDeviceImpl::PciLoopbackMode;

protected:
    Tx *                Owner;
    TxDeviceDriver *    DDeviceDriver;
    TxAlertSystem       Alert;

    // ???  May not be implemented
    void  LocalLoopbackMode(bool state);
    void  InitializeDac(int pair);

    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    // Pseudo-public
public:
    int  RcvPacketSize;
    int  XmitPacketSize;

    void  DumpIsrStatus(std::vector<int> & vec);

protected:
    //
    //  Class PciTestRegister
    class PciTestRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup  GenPacketSize;
        RegisterBit       GenPacketSizeUpdate;

    public:
        PciTestRegister( AddressingSpace &space, int offset );
    };

    //
    //  Class ClockSelectRegister
    class ClockSelectRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        enum IIClockSelect { dcA, dcB, dcPll };

        RegisterBitGroup      Pair0Select;
        RegisterBitGroup      Pair1Select;
        RegisterBit           FpgaClockSelect;
        RegisterBit           DividerClockSelect;
        RegisterBitGroup      DividerRatio;
        RegisterBit           DividerEnable;
        RegisterBit           DividerReset;

        ClockSelectRegister(AddressingSpace &space,
            int offset)
        :  inherited(space, offset),
           Pair0Select(*this, 0, 2), Pair1Select(*this, 2, 2),
           FpgaClockSelect(*this, 8), DividerClockSelect(*this, 10),
           DividerRatio(*this, 12, 2), DividerEnable(*this, 14),
           DividerReset(*this, 15)
            {
            Divider(1);
            ClockSelect(0, dcPll);
            ClockSelect(1, dcPll);
            }

        void ClockSelect(int pair, IIClockSelect value)
            {
            FClockSelect[pair] = value;
            if (pair)
                Pair1Select.Value(value);
            else
                Pair0Select.Value(value);
            }
        IIClockSelect ClockSelect(int pair) const
            {  return FClockSelect[pair];  }

        void Divider(int value)
            {
            FDivider = value;
            DividerReset.Value(value < 1);
            DividerEnable.Value(value > 1);
            switch (value)
                {
                case 2:     DividerRatio.Value(0);      break;
                case 4:     DividerRatio.Value(1);      break;
                case 8:     DividerRatio.Value(2);      break;
                case 16:    DividerRatio.Value(3);      break;
                }
            }                
        int Divider() const
            {  return FDivider;  }
    private:
        // Data
        int                 FDivider;
        IIClockSelect       FClockSelect[NumPairs];
        bool                FSyncClockSelect;
    };

    //
	//  Class UserFpgaCtlRegister
	class UserFpgaCtlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		enum PairSelect { dpNoPair, dpPair0Only, dpPair1Only, dpBothPairs };

		RegisterBit           FpgaReset;
		RegisterBit           Go;
		RegisterBit           ExternalTrigger;
//        RegisterBit           XmcMode;
		RegisterBit           Trigger;
		RegisterBit           RocketIoLoopback;
		RegisterBitGroup      TestMode;
		RegisterBitGroup      PatternGeneratorEnable;
		RegisterBitGroup      FifoReset;
		RegisterBitGroup      DacReset;
		RegisterBitGroup      TxEnable;
		RegisterBit           LowResolutionEnable;
		RegisterBit           ExternalTriggerMode;
		RegisterBitGroup      Frequency;

		UserFpgaCtlRegister(AddressingSpace &space, int offset)
			:  inherited(space, offset),
			   FpgaReset(*this, 0),  Go(*this, 1),
			   ExternalTrigger(*this, 2),  Trigger(*this, 3),
			   RocketIoLoopback(*this, 4),
			   TestMode(*this, 5, 2), PatternGeneratorEnable(*this, 7, 2),
			   FifoReset(*this, 9, 2),  DacReset(*this, 11, 2),
			   TxEnable(*this, 13, 2), LowResolutionEnable(*this, 15),
			   ExternalTriggerMode(*this, 20),
			   Frequency(*this, 24, 8)
			{
			//  Default setup ????
			}

		//  PatternGeneratorOn
        void  PatternGenerator(bool state)
            {  PatternGeneratorEnable.Value(state ? 2 : 0);  }
        bool  PatternGenerator() const
            {  return PatternGeneratorEnable.Value() ? true : false;  }

        //  ResetFifo
        void  ResetFifo(PairSelect value)
            {  FifoReset.Value(value);  }
        PairSelect ResetFifo() const
            {  return Decode(FifoReset);  }

        //  Reset
        void  ResetDac(PairSelect value)
            {  DacReset.Value(value);  }
        PairSelect ResetDac() const
            {  return Decode(DacReset);  }

        //  Enable
        void  Enable(PairSelect value)
            {  TxEnable.Value(value);  }
        PairSelect Enable() const
            {  return Decode(TxEnable);  }

        //  LowResolution Enable
        void  LowResolution(bool state)
            {  LowResolutionEnable.Value(state ? 1 : 0);  }
        bool  LowResolution() const
            {  return LowResolutionEnable.Value() ? true : false;  }

    private:
        PairSelect Decode(const RegisterBitGroup & RBG) const
            {
            PairSelect result;

            switch (RBG.Value())
                {
                case 0:     result = dpNoPair;     break;
                case 1:     result = dpPair0Only;  break;
                case 2:     result = dpPair1Only;  break;
                case 3:     result = dpBothPairs;  break;
                }
            return result;
            }
    };
    // Zbt Ram
    ZbtRam							    Ram;
    PairedBlkChannelInfo                FChannels;
    // D/A control
    bool                                FSincFilterEnabled;

    //
    //  ...Registers
    PciTestRegister                     PciTestControlReg;
    UserFpgaCtlRegister                 UserFpgaControlReg;
    RegisterArray<ShadowRegister>       GainCoeffReg;        // array of 4
    RegisterArray<ShadowRegister>       OffsetCoeffReg;      // array of 4
    ShadowRegister                      ChannelMaskReg;
    ClockSelectRegister                 ClockSelectReg;
    RegisterArray<Register>             DacReg01;
    RegisterArray<Register>             DacReg23;
    Ics8442                             Clock;
    ShadowRegister                      ClockTestReg;

    //
    //  Internal Configuration Methods
    void				TestCounter(UserFpgaCtlRegister::PairSelect pair);
    RegisterBitGroup &	ChannelDacGain(int ch);

    virtual void		SetCalibrationOffset(int ch, int value);
    virtual void 		SetCalibrationGain(int ch, int value);
    virtual void		SetCalibrationPllCorrection(float value);

    //
    //  External Cfg methods
    void                PatternGenerator(bool state);
    bool                PatternGenerator() const;

    void                DacReg(int pair, int offset, int value);
    int                 DacReg(int pair, int offset) const;

    void                ClockTestEnable(bool state);
    bool                ClockTestEnable() const;


    //  Test Mode
    void TestMode(IITestMode mode)
        {  UserFpgaControlReg.TestMode.Value(mode);  }
	IITestMode TestMode() const
        {  return static_cast<IITestMode>(UserFpgaControlReg.TestMode.Value());  }

    void TestFrequency(int value)
        {  UserFpgaControlReg.Frequency.Value(value);  }
    int TestFrequency() const
        {  return UserFpgaControlReg.Frequency.Value();  }

	void ExternalTrigger(bool state)
        {  UserFpgaControlReg.ExternalTrigger.Value(state);  }
    int ExternalTrigger() const
        {  return UserFpgaControlReg.ExternalTrigger.Value();  }

	void ExternalTriggerMode(IIExtTrigMode mode)
		{  UserFpgaControlReg.ExternalTriggerMode.Value((mode==etEdge) ? true : false);  }

	IIExtTrigMode ExternalTriggerMode() const
		{ return static_cast<IIExtTrigMode>(UserFpgaControlReg.ExternalTrigger.Value());}

    void Trigger(bool state)
        {  UserFpgaControlReg.Trigger.Value(state);  }
    int Trigger() const
        {  return UserFpgaControlReg.Trigger.Value();  }

    void LowResolution(bool state)
        {  UserFpgaControlReg.LowResolution(state);  }
    bool LowResolution() const
        {  return UserFpgaControlReg.LowResolution();  }
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
