// Tx_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Tx_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>
#include <BufferDatagrams_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS Tx  -- Digital Transmitter PMC module object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Tx
//---------------------------------------------------------------------------

Tx::Tx()
    :  PmcBoard(Device), Ram(Device.Ram), Device(this), 
       FLoopbackMode(lmNone), FPatternGenMode(false), FReturnFrameSize(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class Tx
//---------------------------------------------------------------------------

Tx::~Tx()
{
    Close();
}

//---------------------------------------------------------------------------
//  Tx::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t Tx::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  Tx::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  Tx::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  Tx::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  Tx::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  Tx::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  Tx::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  Tx::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Tx::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  Tx::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Tx::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  Tx::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  Tx::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  Tx::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  Tx::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  Tx::ClockTestEnable() --
//---------------------------------------------------------------------------

void Tx::ClockTestEnable(bool state)
{
    Device.ClockTestEnable(state);
}

//---------------------------------------------------------------------------
//  Tx::ClockTestEnable() --
//---------------------------------------------------------------------------

bool Tx::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}
//---------------------------------------------------------------------------
//  Tx::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & Tx::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Tx::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & Tx::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Tx::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  Tx::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  Tx::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  Tx::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  Tx::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  Tx::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  Tx::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  Tx::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  Tx::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  Tx::Log( int type, int value )
{
    Device.Log(type, value);
}

//---------------------------------------------------------------------------
//  Tx::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & Tx::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Tx::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & Tx::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Tx::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesTxAlerts & Tx::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  Tx::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & Tx::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  Tx::PciLoopbackMode() --
//------------------------------------------------------------------------------

Tx & Tx::PciLoopbackMode(LoopbackMode mode)
{
    FLoopbackMode = mode;

    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:
            mode_value = 0;
            break;
        case lmSerial:
            mode_value = 1;
            break;
        case lmParallel:
            mode_value = 2;
            break;
        }

    Device.PciLoopbackMode(mode_value);
    return *this;
}

//------------------------------------------------------------------------------
//  Tx::PciLoopbackMode() --
//------------------------------------------------------------------------------

Tx::LoopbackMode Tx::PciLoopbackMode()
{
    return FLoopbackMode;
}

//------------------------------------------------------------------------------
//  Tx::UserLoopbackMode() --
//------------------------------------------------------------------------------

Tx & Tx::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
    return *this;
}

//------------------------------------------------------------------------------
//  Tx::UserLoopbackMode() --
//------------------------------------------------------------------------------

bool Tx::UserLoopbackMode()
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  Tx::SincFilterEnabled() --
//---------------------------------------------------------------------------

void Tx::SincFilterEnabled(bool state)
{
    Device.SincFilterEnabled(state);
}

//---------------------------------------------------------------------------
//  Tx::SincFilterEnabled() --
//---------------------------------------------------------------------------

bool Tx::SincFilterEnabled() const
{
    return Device.SincFilterEnabled();
}

//---------------------------------------------------------------------------
//  Tx::Divider() --
//---------------------------------------------------------------------------

Tx & Tx::Divider(int value)
{
    Device.ClockSelectReg.Divider(value);
    return *this;
}
 
//---------------------------------------------------------------------------
//  Tx::Divider() --
//---------------------------------------------------------------------------

int Tx::Divider() const
{
    return Device.ClockSelectReg.Divider();
}
 
//---------------------------------------------------------------------------
//  Tx::Clock() --
//---------------------------------------------------------------------------

Ics8442 & Tx::Clock()
{
    return Device.Clock;
}
 
//---------------------------------------------------------------------------
//  Tx::DacClockSelect() --
//---------------------------------------------------------------------------

Tx & Tx::DacClockSelect(int pair, IIDacPairClkSelect value)
{
    TxDeviceMap::ClockSelectRegister::IIClockSelect clk;
    switch (value)
        {
        case dcA:       clk = TxDeviceMap::ClockSelectRegister::dcA;   break;
        case dcB:       clk = TxDeviceMap::ClockSelectRegister::dcB;   break;
        default:
        case dcPll:     clk = TxDeviceMap::ClockSelectRegister::dcPll; break;
        }

    Device.ClockSelectReg.ClockSelect(pair, clk);
    return *this;
}

//---------------------------------------------------------------------------
//  Tx::DacClockSelect() --
//---------------------------------------------------------------------------

Tx::IIDacPairClkSelect Tx::DacClockSelect(int pair) const
{
    IIDacPairClkSelect clk;
    switch (Device.ClockSelectReg.ClockSelect(pair))
        {
        case TxDeviceMap::ClockSelectRegister::dcA:       clk = dcA;   break;
        case TxDeviceMap::ClockSelectRegister::dcB:       clk = dcB;   break;
        default:
        case TxDeviceMap::ClockSelectRegister::dcPll:     clk = dcPll; break;
        }

    return clk;
}

//---------------------------------------------------------------------------
//  Tx::FpgaClockSelect() --
//---------------------------------------------------------------------------

Tx & Tx::FpgaClockSelect(IIExtClkSelect value)
{
	Device.ClockSelectReg.FpgaClockSelect.Value( (value == ecA) ? 0 : 1);
    return *this;
}

//---------------------------------------------------------------------------
//  Tx::FpgaClockSelect() --
//---------------------------------------------------------------------------

Tx::IIExtClkSelect Tx::FpgaClockSelect() const
{
    return static_cast<IIExtClkSelect>(Device.ClockSelectReg.FpgaClockSelect.Value());
}

//---------------------------------------------------------------------------
//  Tx::DividerClockSelect() --
//---------------------------------------------------------------------------

Tx & Tx::DividerClockSelect(IIExtClkSelect value)
{
	Device.ClockSelectReg.DividerClockSelect.Value( (value == ecA) ? 0 : 1);
    return *this;
}

//---------------------------------------------------------------------------
//  Tx::DividerClockSelect() --
//---------------------------------------------------------------------------

Tx::IIExtClkSelect Tx::DividerClockSelect() const
{
    return static_cast<IIExtClkSelect>(Device.ClockSelectReg.DividerClockSelect.Value());
}

//---------------------------------------------------------------------------
//  Tx::PllClockSelect() --
//---------------------------------------------------------------------------

Tx & Tx::PllClockSelect(IIPllClkSelect value)
{
    Ics8442::IIReferenceClk clk;
    switch (value)
        {
        case pcDivider:     clk = Ics8442::rcDivider;   break;
        default:
        case pcCrystal:     clk = Ics8442::rcCrystal;   break;
        }

    Device.Clock.ReferenceClock(clk);
    return *this;
}
 
//---------------------------------------------------------------------------
//  Tx::PllClockSelect() --
//---------------------------------------------------------------------------

Tx::IIPllClkSelect Tx::PllClockSelect() const
{
    Tx::IIPllClkSelect clk;

    switch (Device.Clock.ReferenceClock())
        {
        case Ics8442::rcDivider:    clk = pcDivider;    break;
        default:
        case Ics8442::rcCrystal:    clk = pcCrystal;    break;
        }

    return clk;
}
 
//---------------------------------------------------------------------------
//  Tx::TestMode() --
//---------------------------------------------------------------------------

Tx & Tx::TestMode(IITestMode output)
{
    TxDeviceMap::IITestMode mode;

    switch (output)
        {
        case tmNormal:      mode = TxDeviceMap::tmNormal;   break;
        case tmRamp:        mode = TxDeviceMap::tmRamp;     break;
        default:
        case tmSine:        mode = TxDeviceMap::tmSine;     break;
        }

    Device.TestMode(mode);
    return *this;
}

//---------------------------------------------------------------------------
//  Tx::TestMode() --
//---------------------------------------------------------------------------

Tx::IITestMode Tx::TestMode() const
{
    IITestMode mode;
    switch (Device.TestMode())
        {
        case TxDeviceMap::tmNormal:     mode = tmNormal;    break;
        case TxDeviceMap::tmRamp:       mode = tmRamp;      break;
        default:
        case TxDeviceMap::tmSine:       mode = tmSine;      break;
        }

    return mode;
}

//---------------------------------------------------------------------------
//  Tx::TestFrequency() --  Set test frequency
//---------------------------------------------------------------------------

void Tx::TestFrequency(int value)
{
    Device.TestFrequency(value);
}

//---------------------------------------------------------------------------
//  Tx::TestFrequency() --  Retreive test frequency
//---------------------------------------------------------------------------

int Tx::TestFrequency() const
{
    return Device.TestFrequency();
}

//---------------------------------------------------------------------------
//  Tx::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPllCalibration & Tx::IdRom()
{
    return Device;
}

//------------------------------------------------------------------------------
//  Tx::PatternGenerator() --
//------------------------------------------------------------------------------

Tx & Tx::PatternGenerator(bool state)
{
    Device.PatternGenerator(state);

    return *this;
}

//------------------------------------------------------------------------------
//  Tx::PatternGenerator() --
//------------------------------------------------------------------------------

bool Tx::PatternGenerator()
{
    return Device.PatternGenerator();
}

//------------------------------------------------------------------------------
//  Tx::SendPatternInfo() -- Send Pattern Start Information Block
//------------------------------------------------------------------------------

void  Tx::SendPatternInfo(int pair, int start_addr, int buf_size_ints, int count)
{
    Buffer Packet(4);
    PacketBufferHeader Hdr(Packet);
    IntegerDG  PacketDG(Packet);

    Hdr.PeripheralId(128+pair);
    Hdr.PacketSize(Packet.FullSizeInInts());
    Hdr[1] = 0;

    PacketDG[0] = start_addr;
    PacketDG[1] = buf_size_ints/256;
    PacketDG[2] = count;
    PacketDG[3] = 0;
    //
    //  Post to stream
    if (StreamManager.Linked())
        StreamManager->Send(Packet);
}

//------------------------------------------------------------------------------
//  Tx::Channels() --  Reference to channel information object
//------------------------------------------------------------------------------

PairedBlkChannelInfo & Tx::Channels()
{
    return Device.FChannels;
}

//------------------------------------------------------------------------------
//  Tx::DacReg() --  Set D/A configuration register value
//------------------------------------------------------------------------------

void Tx::DacReg(int pair, int offset, int value)
{
    Device.DacReg(pair, offset, value);
}

//------------------------------------------------------------------------------
//  Tx::DacReg() --  Get D/A configuration register value
//------------------------------------------------------------------------------

int Tx::DacReg(int pair, int offset) const
{
    return Device.DacReg(pair, offset);
}

//------------------------------------------------------------------------------
//  Tx::ExternalTrigger() --
//------------------------------------------------------------------------------

void Tx::ExternalTrigger(bool state)
{
    Device.ExternalTrigger(state);
}

//------------------------------------------------------------------------------
//  Tx::ExternalTrigger() --
//------------------------------------------------------------------------------

int Tx::ExternalTrigger() const
{
	return Device.ExternalTrigger();
}

//------------------------------------------------------------------------------
//  Tx::ExternalTriggerMode() --
//------------------------------------------------------------------------------

void Tx::ExternalTriggerMode(TxDeviceMap::IIExtTrigMode mode)
{
	Device.ExternalTriggerMode(mode);
}

//------------------------------------------------------------------------------
//  Tx::ExternalTriggerMode() --
//------------------------------------------------------------------------------

TxDeviceMap::IIExtTrigMode Tx::ExternalTriggerMode() const
{
	return Device.ExternalTriggerMode();
}

//------------------------------------------------------------------------------
//  Tx::Trigger() --
//------------------------------------------------------------------------------

void Tx::Trigger(bool state)
{
    Device.Trigger(state);
}

//------------------------------------------------------------------------------
//  Tx::Trigger() --
//------------------------------------------------------------------------------

int Tx::Trigger() const
{
    return Device.Trigger();
}

//------------------------------------------------------------------------------
//  Tx::LowResolution() --
//------------------------------------------------------------------------------

Tx & Tx::LowResolution(bool state)
{
    Device.LowResolution(state);
    return *this;
}

//------------------------------------------------------------------------------
//  Tx::LowResolution() --
//------------------------------------------------------------------------------

bool Tx::LowResolution() const
{
    return Device.LowResolution();
}

//---------------------------------------------------------------------------
//  Tx::BusMasterSize() --
//---------------------------------------------------------------------------

void Tx::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Tx::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void Tx::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Tx::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void Tx::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Tx::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  Tx::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  Tx::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  Tx::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

}  // namespace Innovative
