// Tx_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Tx_MbH
#define Tx_MbH

#include <PmcBoard_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <TxDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>
#include <TxAlerts_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS Tx  -- Digital Transmitter PMC module object
//===========================================================================

class Tx : public PmcBoard,
           public IStreamDeviceEvents
{
    friend class TxDeviceMap;
    
public:
    //
    //  Member Functions
    Tx();
    virtual ~Tx();

    size_t          BoardCount();
    void            Open();
    void            Close();

    int   Target();
    void  Target( int value );

    bool  Led();
    void  Led( bool value );

    FpgaLogicVersionInfoIntf & UserLogicVersion();
    PciLogicVersionInfoIntf & PciLogicVersion();
    IPciBusDiagnostics & PciDiagnostics();

    ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );
    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );
	//
    //  IPacketStreamDevice interface implementation
    virtual void  Reset();
    virtual void   Log(int type, int data);

    TxDeviceMap * Debug() { return &Device; };

    // Logic programming
    IUsesVirtexFpgaLoader & Logic();
    // EEprom programming
    IUsesVirtexJtagLoader & Eeprom();
    //  Alerts
    IUsesTxAlerts & Alerts();

    // Trigger Support
    // Enumerations
    enum LoopbackMode { lmNone, lmParallel, lmSerial };

    // Methods
    Tx &            PciLoopbackMode(LoopbackMode mode);
    LoopbackMode    PciLoopbackMode();

    Tx &            UserLoopbackMode(bool state);
    bool            UserLoopbackMode();

	void 	        SincFilterEnabled(bool state);
	bool	        SincFilterEnabled() const;

    Tx &            PatternGenerator(bool state);
    bool            PatternGenerator();
    void            SendPatternInfo(int pair, int start_addr,
                                    int buf_size_ints, int count);

    enum IIExtClkSelect { ecA, ecB };
    enum IIDacPairClkSelect { dcA, dcB, dcPll };
    enum IIPllClkSelect { pcDivider, pcCrystal };
    enum IITestMode { tmNormal, tmRamp, tmSine };

    // Properties
    Tx &            Divider(int value);
    int             Divider() const;

    Tx &            PllClockSelect(IIPllClkSelect value);
    IIPllClkSelect  PllClockSelect() const;

    Tx &            DividerClockSelect(IIExtClkSelect value);
    IIExtClkSelect  DividerClockSelect() const;

    Tx &            FpgaClockSelect(IIExtClkSelect value);
    IIExtClkSelect  FpgaClockSelect() const;

    Tx &                DacClockSelect(int pair, IIDacPairClkSelect value);
    IIDacPairClkSelect  DacClockSelect(int pair) const;

    Tx &            TestMode(IITestMode output);
    IITestMode      TestMode() const;

    void            TestFrequency(int value);
    int             TestFrequency() const;

    void            ExternalTrigger(bool state);
    int             ExternalTrigger() const;

	void			ExternalTriggerMode(TxDeviceMap::IIExtTrigMode mode);
	TxDeviceMap::IIExtTrigMode   ExternalTriggerMode() const;

    void            Trigger(bool state);
    int             Trigger() const;

    Tx &            LowResolution(bool state);
    bool            LowResolution() const;

    void            ClockTestEnable(bool state);
    bool            ClockTestEnable() const;

	void 							   BusMasterSize(int bytes);
	void 							   IncomingBusMasterSize(int bytes);
	void 							   OutgoingBusMasterSize(int bytes);
	int             				   IncomingBusMasterSize() const;
	int             				   OutgoingBusMasterSize() const;

	// Methods
    Ics8442 &              Clock();
    IUsesPllCalibration &  IdRom();
    PairedBlkChannelInfo & Channels();

    // Zbt Ram
    ZbtRam &        Ram;

    // Channel resolution mode
    void                    DacReg(int pair, int offset, int value);
    int                     DacReg(int pair, int offset) const;


protected:
    //
    //  Member Data
    TxDeviceMap             Device;
    LoopbackMode            FLoopbackMode;
    bool                    FPatternGenMode;
    int                     FReturnFrameSize;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

