//---------------------------------------------------------------------------
// TycoGps_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.

#include "TycoGps_Mb.h"
#include <sstream>
#include <ctime>
#include <StringSupport_Mb.h>
#include <cmath>
#include <SystemSupport_Mb.h>

using namespace std;

namespace Innovative
{

//===========================================================================
//  CLASS TycoGps  -- Tyco GPS interface
//===========================================================================

//---------------------------------------------------------------------------
//  TycoGps::TycoGps() --
//---------------------------------------------------------------------------

TycoGps::TycoGps()
	: IGps(new StandardGpsMessageParser()), FInitializing(false),
	  RxTallyForInit(0)
{
}

//---------------------------------------------------------------------------
//  TycoGps::~TycoGps()
//---------------------------------------------------------------------------

TycoGps::~TycoGps()
{
}

//---------------------------------------------------------------------------
// TycoGps::OnInit()
//---------------------------------------------------------------------------

void TycoGps::OnInit()
{
    RxTallyForInit = 0;
    UartBaud(b4800);
}


//---------------------------------------------------------------------------
//  TycoGps::Reset() --  Reset settings
//---------------------------------------------------------------------------

void TycoGps::Reset()
{
	Command("$PTYCDEF");
	OnInit();
}

//---------------------------------------------------------------------------
//  TycoGps::Save() --  Save comm settings
//---------------------------------------------------------------------------

void TycoGps::Save()
{
	Command("$PTYCSUPWR");
}

//---------------------------------------------------------------------------
//  TycoGps::GpsBaud() --  Set GPS baud-rate
//---------------------------------------------------------------------------

void TycoGps::GpsBaud(BaudRates /*rate*/)
{
}

//---------------------------------------------------------------------------
//  TycoGps::Sentence() --  Enable/disable sentence state
//---------------------------------------------------------------------------

void TycoGps::Sentence(IISentence type, bool state)
{
	const char * cmd[] = { "GSA", "GSV", "RMC", "VTG", "ZDA" };

	stringstream msg;
	int value = state ? 1 : 0;
	msg << "$PTYC" << cmd[type] << "," << value;
	Command(msg.str());
}

//---------------------------------------------------------------------------
//  TycoGps::Sentence() --  Toggle sentence state
//---------------------------------------------------------------------------

void TycoGps::Sentence(IISentence type)
{
	const char * cmd[] = { "GSA", "GSV", "RMC", "VTG", "ZDA" };

	stringstream msg;
	msg << "$PTYC" << cmd[type];
	Command(msg.str());
}

//---------------------------------------------------------------------------
//  Gps_Cw25::OnProcessMessages() --  Process Epoch Message List
//---------------------------------------------------------------------------

void TycoGps::OnProcessMessages(GpsMessageList & Messages)
{
    //
    //  Need to set "Time()" state from $GPZDA message
    int ZdaIdx = Messages.FindMessage("$GPZDA");
    if (ZdaIdx != -1)
        {
        GpsMessage ZdaMsg = Messages.MessageAt(ZdaIdx);
        GpZdaCracker Zda(ZdaMsg);
        if (Zda.IsValid)
			FTime = Zda.Time;
		}
    //
    //  Need to set Position and other state from $GPGGA message
    int GgaIdx = Messages.FindMessage("$GPGGA");
    if (GgaIdx != -1)
        {
        GpsMessage GgaMsg = Messages.MessageAt(GgaIdx);
        GpGgaCracker Gga(GgaMsg);
        if (Gga.IsValid)
            {
            //  Latitude
            FLatitude.Degrees = Gga.LatDegrees;
            FLatitude.Minutes = Gga.LatMinutes;
            FLatitude.Units   = Gga.LatUnits;
            //  Longitude
            FLongitude.Degrees = Gga.LongDegrees;
            FLongitude.Minutes = Gga.LongMinutes;
            FLongitude.Units   = Gga.LongUnits;
            //  Fix
            FFix = static_cast<FixStatus>(Gga.Fix);
            //  Satellites
            FSatellites = Gga.Satellites;
            }
        }

    if( FInitializing )
        {
        RxTallyForInit++;
        if (RxTallyForInit == 5)
            {
            EnableZdaCommand();
            RxTallyForInit = 0;
            MutexedLock  Lock(mLock);
            FInitializing = false;
            }
        }
}

//---------------------------------------------------------------------------
// TycoGps::EnableZdaCommand()
//---------------------------------------------------------------------------

void TycoGps::EnableZdaCommand()
{
	Sentence(TycoGps::sZDA, true);
}

}  // namespace Innovative

