//---------------------------------------------------------------------------
// UwbCs_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <UwbCs_Mb.h>
#include <UwbCsMemoryMap_Mb.h>

namespace Innovative
{
//===========================================================================
//  CLASS UwbCsRegisters  -- CS project UWB extension registers
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class UwbCsRegisters
//---------------------------------------------------------------------------
using namespace UwbCs_MemoryMap;

//---------------------------------------------------------------------------
//  constructor for class UwbCsRegisters
//---------------------------------------------------------------------------

UwbCsRegisters::UwbCsRegisters(AddressingSpace & LogicMemory)
    : IUsesFpgaLogicVersionInfo(LogicMemory, mmQuadiaLogicRev),
      TuningFrequencyReg(LogicMemory, mmTuningFreq0, 20),
      GainAdjustReg     (LogicMemory, mmGainAdjust0, 20),
      AppStatusReg      (LogicMemory, mmApplicationStatus),
      AdcSelectReg      (LogicMemory, mmAdcSelect),
      //QuadiaLogicRevReg (LogicMemory, mmQuadiaLogicRev),
      TestControlReg    (LogicMemory, mmTestControl),
      SpectralInvertReg (LogicMemory, mmSpectralInvert),
      FpgaStatus        (LogicMemory, mmFpgaStatus)
{
}

//---------------------------------------------------------------------------
//  destructor for class UwbCsRegisters
//---------------------------------------------------------------------------

UwbCsRegisters::~UwbCsRegisters()
{
}

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif
//===========================================================================
//  CLASS UwbCsRegisters::GainAdjustRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class UwbCsRegisters::GainAdjustRegister
//---------------------------------------------------------------------------

UwbCsRegisters::GainAdjustRegister::GainAdjustRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      Gain( *this, 0, 3 )
{
}

//===========================================================================
//  CLASS UwbCsRegisters::FpgaStatusRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class UwbCsRegisters::FpgaStatusRegister
//---------------------------------------------------------------------------

UwbCsRegisters::FpgaStatusRegister::FpgaStatusRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      J4DcmLocked  ( *this, 0),
      Dsp1DcmLocked( *this, 1 ),
      Dsp2DcmLocked( *this, 2 )
{
}

//===========================================================================
//  CLASS UwbCsRegisters::TestControlRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class UwbCsRegisters::TestControlRegister
//---------------------------------------------------------------------------

UwbCsRegisters::TestControlRegister::TestControlRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TestDataSource      ( *this, 0),
      FilterBypassed      ( *this, 1 ),
      TestGeneratorEnabled( *this, 2 )
{
}

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif

//===========================================================================
//  CLASS UwbCs  -- CS UWB object baseboard
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class UwbCs
//---------------------------------------------------------------------------

const double max_rez = 4294967296.0;  //   2^32
const double default_base_freq = 130000000.0;

//---------------------------------------------------------------------------
//  constructor for class UwbCs
//---------------------------------------------------------------------------

UwbCs::UwbCs()
    :  XDevice(Device.LogicMemory), base_frequency(default_base_freq)
{
}

//---------------------------------------------------------------------------
//  destructor for class UwbCs
//---------------------------------------------------------------------------

UwbCs::~UwbCs()
{
}

//---------------------------------------------------------------------------
//  UwbCs::Open() -- Open the card
//---------------------------------------------------------------------------

void  UwbCs::Open()
{
    Uwb::Open();   // perform base class Open()
}

//---------------------------------------------------------------------------
//  UwbCs::BaseFrequency() -- save base frequency for tuning frequency
//---------------------------------------------------------------------------

void  UwbCs::BaseFrequency(double freq)
{
    base_frequency = freq;
}

//-----------------------------------------------------------------------------
//  UwbCs::BaseFrequency() -- read back save base frequency for tuning freq
//-----------------------------------------------------------------------------

double  UwbCs::BaseFrequency() const
{
    return base_frequency;
}

//---------------------------------------------------------------------------
//  UwbCs::TuningFrequency() -- save tuning frequency
//---------------------------------------------------------------------------

void  UwbCs::TuningFrequency(int ch, double freq)
{
    //  scale frequency as fraction of base frequecy normalized to 32 bit rez.
    double scaled_freq = freq/base_frequency*max_rez;

    unsigned int fraction = (unsigned int)scaled_freq;
    XDevice.TuningFrequencyReg[ch].Value(fraction);
}

//---------------------------------------------------------------------------
//  UwbCs::TuningFrequency() -- read back tuning frequency
//---------------------------------------------------------------------------

double  UwbCs::TuningFrequency(int ch) const
{
    return double(TuningFrequencyRaw(ch))/max_rez*base_frequency;
}

//---------------------------------------------------------------------------
//  UwbCs::TuningFrequencyRaw() -- raw register readback
//---------------------------------------------------------------------------

unsigned int  UwbCs::TuningFrequencyRaw(int ch) const
{
    return XDevice.TuningFrequencyReg[ch].Value();
}

//---------------------------------------------------------------------------
//  UwbCs::GainAdjust() -- save Gain Adjust values
//---------------------------------------------------------------------------

void  UwbCs::GainAdjust(int ch, unsigned int gain)
{
    XDevice.GainAdjustReg[ch].Gain = gain;
}

//---------------------------------------------------------------------------
//  UwbCs::GainAdjust() -- read back gain adjust value
//---------------------------------------------------------------------------

unsigned int  UwbCs::GainAdjust(int ch) const
{
    return XDevice.GainAdjustReg[ch].Gain;
}

//---------------------------------------------------------------------------
//  UwbCs::QuadiaLogicVersion() -- logic version of quadia
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & UwbCs::QuadiaLogicVersion() 
{
    return XDevice;
}

//---------------------------------------------------------------------------
//  UwbCs::ChannelSourceRaw() -- raw register readback
//---------------------------------------------------------------------------

unsigned int  UwbCs::ChannelSourceRaw() const
{
    return XDevice.AdcSelectReg.Value();
}

//---------------------------------------------------------------------------
//  UwbCs::ChannelSource() -- set channel source Adc
//---------------------------------------------------------------------------

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4800)		
#endif

void  UwbCs::ChannelSource(int ch, int adc)
{
    XDevice.AdcSelectReg.Bit(ch, adc);
}

#ifdef _MSC_VER		
	#pragma warning(pop)	
#endif
//---------------------------------------------------------------------------
//  UwbCs::ChannelSource() -- read back channel source
//---------------------------------------------------------------------------

int  UwbCs::ChannelSource(int ch) const
{
    return XDevice.AdcSelectReg.Bit(ch);
}


//---------------------------------------------------------------------------
//  UwbCs::SpectralInvertRaw() -- raw register readback
//---------------------------------------------------------------------------

unsigned int  UwbCs::SpectralInvertRaw() const
{
    return XDevice.SpectralInvertReg.Value();
}

//---------------------------------------------------------------------------
//  UwbCs::SpectralInvert() -- set channel source Adc
//---------------------------------------------------------------------------

void  UwbCs::SpectralInvert(int ch, bool should_invert)
{
    XDevice.SpectralInvertReg.Bit(ch, should_invert);
}

//---------------------------------------------------------------------------
//  UwbCs::SpectralInvert() -- read back channel source
//---------------------------------------------------------------------------

bool  UwbCs::SpectralInvert(int ch) const
{
    return XDevice.SpectralInvertReg.Bit(ch);
}

//---------------------------------------------------------------------------
//  UwbCs::TestDataSource() -- set bit
//---------------------------------------------------------------------------

void  UwbCs::TestDataSource(bool state)
{
    XDevice.TestControlReg.TestDataSource.Value(state);
}

//---------------------------------------------------------------------------
//  UwbCs::TestDataSource() -- read back bit
//---------------------------------------------------------------------------

bool  UwbCs::TestDataSource() const
{
    return XDevice.TestControlReg.TestDataSource;
}

//---------------------------------------------------------------------------
//  UwbCs::FilterBypassed() -- set bit
//---------------------------------------------------------------------------

void  UwbCs::FilterBypassed(bool state)
{
    XDevice.TestControlReg.FilterBypassed.Value(state);
}

//---------------------------------------------------------------------------
//  UwbCs::FilterBypassed() -- read back bit
//---------------------------------------------------------------------------

bool  UwbCs::FilterBypassed() const
{
    return XDevice.TestControlReg.FilterBypassed;
}

//---------------------------------------------------------------------------
//  UwbCs::TestGeneratorEnabled() -- set bit
//---------------------------------------------------------------------------

void  UwbCs::TestGeneratorEnabled(bool state)
{
    XDevice.TestControlReg.TestGeneratorEnabled.Value(state);
}

//---------------------------------------------------------------------------
//  UwbCs::TestGeneratorEnabled() -- read back bit
//---------------------------------------------------------------------------

bool  UwbCs::TestGeneratorEnabled() const
{
    return XDevice.TestControlReg.TestGeneratorEnabled;
}


}  // end namespace Innovative;
