//  UwbCs_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef UwbCs_MbH
#define UwbCs_MbH

#include <Uwb_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS UwbCsRegisters  -- CS UWB object registers
//===========================================================================

class UwbCsRegisters : public IUsesFpgaLogicVersionInfo
{
    friend class UwbCs;

public:
    //
    //  Member Functions
    UwbCsRegisters(AddressingSpace & space);
    ~UwbCsRegisters();

private:
    //
    //  Class GainAdjustRegister
    class GainAdjustRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Gain;

    public:
        GainAdjustRegister( AddressingSpace &space, int offset );
    };
    //
    //  Class FpgaStatusRegister
    class FpgaStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit  J4DcmLocked;
        ReadOnlyRegisterBit  Dsp1DcmLocked;
        ReadOnlyRegisterBit  Dsp2DcmLocked;

    public:
        FpgaStatusRegister( AddressingSpace &space, int offset );
    };
    //
    //  Class TestControlRegister
    class TestControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  TestDataSource;
        RegisterBit  FilterBypassed;
        RegisterBit  TestGeneratorEnabled;

    public:
        TestControlRegister( AddressingSpace &space, int offset );
    };

    RegisterArray<Register>           TuningFrequencyReg;  // array of 20
    RegisterArray<GainAdjustRegister> GainAdjustReg;       // array of 20

    ReadOnlyRegister    AppStatusReg;
    Register            AdcSelectReg;

    //ReadOnlyRegister    QuadiaLogicRevReg;
    TestControlRegister TestControlReg;
    Register            SpectralInvertReg;
    FpgaStatusRegister  FpgaStatus;
};

//===========================================================================
//  CLASS UwbCs  -- CS UWB object baseboard
//===========================================================================

class UwbCs : public Uwb
{
public:
    //
    //  Member Functions
    UwbCs();
    ~UwbCs();

    virtual void  Open();

    void    BaseFrequency(double freq);
    double  BaseFrequency() const;

    void    TuningFrequency(int ch, double freq);
    double  TuningFrequency(int ch) const;
    unsigned int TuningFrequencyRaw(int ch)  const;

    void         GainAdjust(int ch, unsigned int gain);
    unsigned int GainAdjust(int ch) const;

    FpgaLogicVersionInfoIntf &  QuadiaLogicVersion();

    void         TestDataSource(bool state);
    bool         TestDataSource() const;
    void         FilterBypassed(bool state);
    bool         FilterBypassed() const;
    void         TestGeneratorEnabled(bool state);
    bool         TestGeneratorEnabled() const;

    void         ChannelSource(int ch, int adc);
    int          ChannelSource(int ch) const;
    unsigned int ChannelSourceRaw()  const;

    void         SpectralInvert(int ch, bool should_invert);
    bool         SpectralInvert(int ch) const;
    unsigned int SpectralInvertRaw()  const;

protected:
    UwbCsRegisters  XDevice;
    double          base_frequency;
};

}; // namespace Innovative

//---------------------------------------------------------------------------
#endif
