// UwbDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <Uwb_Mb.h>
#include <UwbDeviceMap_Mb.h>
#include <UwbMemoryMap_Mb.h>
#include <UwbDeviceDriver_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <PmcIdrom_Mb.h>
#include <SystemSupport_Mb.h>

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS UwbHdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class UwbHdwPluginImpl : public IPacketDevicePlugin
{
public:
    UwbHdwPluginImpl( UwbDeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual ~UwbHdwPluginImpl()
    	{}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    UwbDeviceMap * PDM;
};

//===========================================================================
//  CLASS UwbDeviceMap  -- Registers for Uwb Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class UwbDeviceMap
//---------------------------------------------------------------------------

using namespace Uwb_MemoryMap;

PacketDeviceRegisters UwbDRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	0,                    // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters UwbRegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class UwbDeviceMap
//---------------------------------------------------------------------------

UwbDeviceMap::UwbDeviceMap(Uwb * owner)
    :  PacketDeviceImpl(new UwbDeviceDriver, UwbDRegAddrs),
       PacketDeviceMap(this, UwbRegAddrs, 0),
       IUsesVirtexFpgaLoader(PortMemory, mmFpgaData, mmFpgaControl),
       IUsesVirtexJtagLoader(PortMemory, mmJtag),
       IUsesPllCalibration(PortMemory, mmI2c, 128, 2, 0x4000),
       IUsesFpgaLogicVersionInfo(LogicMemory, mmFpgaVersion),
       IUsesPciLogicVersionInfo(PortMemory, mmPciVersion),
       Ram(LogicMemory, mmZbtSramAddress, mmZbtSramReadData, mmZbtSramWriteData),
       //        devices, bits,     muxChannelCount,    unipolar
       FChannels(2,       12,       0,                  false),
       Owner(owner),
       DDeviceDriver(dynamic_cast<UwbDeviceDriver*>(DeviceDriver)),
       RcvPacketSize(0x80), XmitPacketSize(0x80),
       PciTestControlReg    (PortMemory,  mmPciTestControl),
       UserFpgaControlReg   (LogicMemory, mmUserControl),
       GainCoeffReg         (LogicMemory, mmAdcGainCoeff0, UwbConstants::ChannelCount),
       OffsetCoeffReg       (LogicMemory, mmAdcOffsetCoeff0, UwbConstants::ChannelCount),
       ChannelEnableReg     (LogicMemory, mmChannelEnable),
       ChannelDestinationReg(LogicMemory, mmChannelDestination),
       AdcGainControlReg    (LogicMemory, mmAdcShutdown),
       ClockSelectReg       (LogicMemory, mmClockSelect),
       PacketHeaderReg      (LogicMemory, mmPacketHeader0, UwbConstants::ChannelCount),
       Clock(0, LogicMemory, mmClockControl),
       TriggerReg           (LogicMemory, mmTrigger, UwbConstants::ChannelCount),
       ClockTestReg         (LogicMemory, mmClockTest)

{
    for (int ch = 0; ch < UwbConstants::ChannelCount; ++ch)
        {
        GainCoeffReg[ch].Val(1.0);
        OffsetCoeffReg[ch] = 0;
        }
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &UwbDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &UwbDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &UwbDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &UwbDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &UwbDeviceMap::OnAfterStreamStopHandler );

    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new UwbHdwPluginImpl(this) );

}

//---------------------------------------------------------------------------
//  destructor for class UwbDeviceMap
//---------------------------------------------------------------------------

UwbDeviceMap::~UwbDeviceMap()
{
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  UwbDeviceMap::OpenHardware()
{
//    if ( DeviceDriver->Opened() )      ??? Need this guard?
//        return;
    //
    //  Init Calibration Interface 
    IUsesCalibration::AfterOpen();

    // Configure packet headers for default flow through, safe size
    for (int ch = 0; ch < FChannels.Channels(); ++ch)
        ReturnPacketSize(ch, 0x1f0);

    // Configure A/Ds
    for (int ch = 0; ch < 2; ++ch)
        {
        AdcTestCounterOn(ch, false);
        Unframed(ch);
        }

}

//---------------------------------------------------------------------------
//  UwbDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  UwbDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Configure triggers
    for (int ch = 0; ch < 2; ++ch)
        TriggerReg[ch].Apply();
    Trigger(false);

    //
    //  Reset User Logic
    UserFpgaControlReg.Go.Clear();
    UserFpgaControlReg.FpgaReset.Set();
    Sleep(1);
    UserFpgaControlReg.FpgaReset.Clear();
    //
    //  Apply Control Register Stuff
    ExternalRouting(ExternalRouting());
    PciTestControlReg.Apply();

    //
    //  Apply Clock Settings from saved state
    Clock.Reapply();
    ClockSelectReg.Reapply();

    //
    //  Apply Gain/Offset Coefficients
    ApplyRomCoefficients();
    //
    //  Apply settings
    for (int ch=0; ch<FChannels.Channels(); ch++)
        {
        ChannelEnableBit(ch).Value(FChannels.Enabled(ch));  //Apply Channel Enable
        ChannelShutdownBit(ch).Value(FChannels.Enabled(ch)==false);  // if enabled, turn on
        ReturnPacketSize(ch, ReturnPacketSize(ch)); //Apply Return Packet Size info
        AdcDestination(ch, AdcDestination(ch)); //Apply Channel Destination
        }

    //
    //  Set loopback mode
    PciLoopbackMode(Owner->FLoopbackMode);
    //
    //  Allow 200 ms for ADCs to 'wake up'?????
    Sleep(200);
    //  Forward notify to baseboard handler
    Owner->OnStreamConfigure.Execute(Event);

}

//---------------------------------------------------------------------------
//  UwbDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  UwbDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //
    //  Start execution
    UserFpgaControlReg.Go.Set();

}

//---------------------------------------------------------------------------
//  UwbDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  UwbDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  UwbDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  UwbDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
    UserFpgaControlReg.Go.Clear();

    Trigger(false);

    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  UwbDeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<UwbDeviceDriver::LOG->Count; i++)
        vec.push_back(UwbDeviceDriver::LOG->Status[i]);

}

//---------------------------------------------------------------------------
//  UwbDeviceMap::LocalLoopbackMode()
//---------------------------------------------------------------------------

void  UwbDeviceMap::LocalLoopbackMode(bool state)
{
    if (state)
        {
        PciTestControlReg.LoopbackEnable.Set();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Set();
        }
    else
        {
        PciTestControlReg.LoopbackEnable.Clear();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Clear();
        }
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::AdcTestCounterOn() -- Set test mode state
//---------------------------------------------------------------------------

void UwbDeviceMap::AdcTestCounterOn(int ch, bool state)
{
   ChannelTestCounterOnBit(ch).Value(state);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::AdcTestCounterOn() --  Query test mode state
//---------------------------------------------------------------------------

bool UwbDeviceMap::AdcTestCounterOn(int ch)
{
    return (ChannelTestCounterOnBit(ch).Value()) ? true : false;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::AdcDestination() --  Set A/D data destination
//---------------------------------------------------------------------------

void UwbDeviceMap::AdcDestination(int ch, IIDestination dest)
{
    ChannelDestinationBit(ch).Value(dest==dstExternal);
}  

//---------------------------------------------------------------------------
//  UwbDeviceMap::AdcDestination() --  Get A/D data destination
//---------------------------------------------------------------------------

UwbDeviceMap::IIDestination UwbDeviceMap::AdcDestination(int ch) 
{
    if (ChannelDestinationBit(ch).Value())
        return dstExternal;
    else
        return dstNormal;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ExternalRouting() -- Route external data
//---------------------------------------------------------------------------

void UwbDeviceMap::ExternalRouting(IIExternalRouting routing)
{
    UserFpgaControlReg.ExternalBus.Value(routing==extXmc);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ExternalRouting() -- Route channels
//---------------------------------------------------------------------------

UwbDeviceMap::IIExternalRouting  UwbDeviceMap::ExternalRouting() const
{
    if (UserFpgaControlReg.ExternalBus.Value())
        return extXmc;
    else
        return extJ4;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::Framed() --
//---------------------------------------------------------------------------

void UwbDeviceMap::Framed(int ch, int size)
{
    TriggerReg[ch].Count.Value(size);
    TriggerReg[ch].FramedMode.Value(true);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::Unframed() --
//---------------------------------------------------------------------------

void UwbDeviceMap::Unframed(int ch)
{
    TriggerReg[ch].FramedMode.Value(false);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void UwbDeviceMap::ExternalTrigger(int ch, bool state)
{
    TriggerReg[ch].External.Value(state);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool UwbDeviceMap::ExternalTrigger(int ch) const
{
    return TriggerReg[ch].External.Value()? true : false;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::Trigger() --  Set trigger state
//---------------------------------------------------------------------------

void UwbDeviceMap::Trigger(bool state)
{
    UserFpgaControlReg.Trigger.Value(state);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::Trigger() --  Query trigger state
//---------------------------------------------------------------------------

bool UwbDeviceMap::Trigger() const
{
    return UserFpgaControlReg.Trigger.Value()? true : false;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::SetCalibrationOffset()
//---------------------------------------------------------------------------

void UwbDeviceMap::SetCalibrationOffset(int ch, int value)
{
    OffsetCoeffReg[ch].Value(value);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::SetCalibrationGain()
//---------------------------------------------------------------------------

void UwbDeviceMap::SetCalibrationGain(int ch, int value)
{
	GainCoeffReg[ch].Value(value);
}

//------------------------------------------------------------------------------
//  UwbDeviceMap::SetCalibrationPllCorrection() -- apply to hardware
//------------------------------------------------------------------------------

void UwbDeviceMap::SetCalibrationPllCorrection(float value)
{
	Clock.ReferenceCalibrationFactor(value);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ReturnPacketSize() -- Set size of packet coming in
//---------------------------------------------------------------------------

void  UwbDeviceMap::ReturnPacketSize(int ch, int size_words)
{
   PacketHeaderReg[ch].Size.Value(size_words);
   PacketHeaderReg[ch].Id.Value(ch);
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ReturnPacketSize() -- Set size of packet coming in
//---------------------------------------------------------------------------

int  UwbDeviceMap::ReturnPacketSize(int ch) const
{
    return PacketHeaderReg[ch].Size.Value();
}


//---------------------------------------------------------------------------
//  UwbDeviceMap::ClockTestEnable() --
//---------------------------------------------------------------------------

void UwbDeviceMap::ClockTestEnable(bool state)
{
    ClockTestReg.Bit(0,(state ? 1 : 0));
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ClockTestEnable() --
//---------------------------------------------------------------------------

bool UwbDeviceMap::ClockTestEnable() const
{
    return ClockTestReg.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ChannelEnableBit()
//---------------------------------------------------------------------------

RegisterBit &  UwbDeviceMap::ChannelEnableBit(int ch)
{
     if (ch==0)
        return ChannelEnableReg.Adc0Enable;
     else
        return ChannelEnableReg.Adc1Enable;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ChannelShutdownBit()
//---------------------------------------------------------------------------

RegisterBit &  UwbDeviceMap::ChannelShutdownBit(int ch)
{
     if (ch==0)
        return AdcGainControlReg.Adc0Shutdown;
     else
        return AdcGainControlReg.Adc1Shutdown;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ChannelDestinationBit()
//---------------------------------------------------------------------------

RegisterBit &  UwbDeviceMap::ChannelDestinationBit(int ch)
{
     if (ch==0)
        return ChannelDestinationReg.Adc0Dest;
     else
        return ChannelDestinationReg.Adc1Dest;
}

//---------------------------------------------------------------------------
//  UwbDeviceMap::ChannelTestCounterOnBit()
//---------------------------------------------------------------------------

RegisterBit &  UwbDeviceMap::ChannelTestCounterOnBit(int ch)
{
     if (ch==0)
        return UserFpgaControlReg.Adc0TestCounter;
     else
        return UserFpgaControlReg.Adc1TestCounter;
}

//===========================================================================
//  CLASS UwbDeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Uwb::UwbDeviceMap
//---------------------------------------------------------------------------

UwbDeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TestModeEnable( *this, 0 ),
      GenerateRxData( *this, 1 ),
      LoopbackEnable( *this, 2 ),
      ClearTxCredit( *this, 3 ),
      ClearRxCredit( *this, 4 ),
      DrainXmitData( *this, 5 ),
      GenPacketSize( *this, 8, 4 ),
      GenPacketSizeUpdate( *this, 12 )
{
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif



} // namespace Innovative


