// UwbDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef UwbDeviceMap_MbH
#define UwbDeviceMap_MbH


#include <PacketDeviceMap_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PllCalMem_Mb.h>
#include <ICS8442_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <ZbtRam_Mb.h>
#include <ChannelInfo_Mb.h>
#include <PacketDeviceImpl_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class UwbDeviceDriver;
class Event;
class PmcBuffer;

namespace UwbConstants
{
    const int ChannelCount = 2;
};

//===========================================================================
//  CLASS UwbDeviceMap  -- Register & low level support for Uwb
//===========================================================================
class Uwb;

class UwbDeviceMap :  public PacketDeviceImpl,
                      public PacketDeviceMap,
                      public IUsesVirtexFpgaLoader,
                      public IUsesVirtexJtagLoader,
                      public IUsesPllCalibration,
                      public IUsesFpgaLogicVersionInfo,
                      public IUsesPciLogicVersionInfo
{
    friend class Uwb;

public:
    //
    //  Member Functions
    UwbDeviceMap(Uwb * owner);
    virtual ~UwbDeviceMap();

    virtual void  OpenHardware();

    //
    //  Make this property public
    using PacketDeviceImpl::PciLoopbackMode;

    // Zbt Ram
    ZbtRam               Ram;
    UnpairedChannelInfo  FChannels;

protected:

    Uwb *                   Owner;
    UwbDeviceDriver *       DDeviceDriver;

    // ???  May not be implemented
    void  LocalLoopbackMode(bool state);

    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

public:
    enum IIDestination { dstNormal, dstExternal };
    enum IIExternalRouting { extJ4, extXmc };

    int  RcvPacketSize;
    int  XmitPacketSize;

    void  DumpIsrStatus(std::vector<int> & vec);

protected:
    //
    //  Class PciTestRegister
    class PciTestRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       TestModeEnable;
        RegisterBit       GenerateRxData;
        RegisterBit       LoopbackEnable;
        RegisterBit       ClearTxCredit;
        RegisterBit       ClearRxCredit;
        RegisterBit       DrainXmitData;
        RegisterBitGroup  GenPacketSize;
        RegisterBit       GenPacketSizeUpdate;

    public:
        PciTestRegister( AddressingSpace &space, int offset );
    };

    class ClockSelectRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        enum IIAdcClkSelect { acA, acB, acPll };

        RegisterBitGroup      Ch0Select;
        RegisterBitGroup      Ch1Select;
        RegisterBit           FpgaClkSelect;
        RegisterBit           DividerClockSelect;
        RegisterBitGroup      DividerRatio;
        RegisterBit           DividerEnable;
        RegisterBit           DividerReset;

        ClockSelectRegister(AddressingSpace &space,
            int offset)
        :  inherited(space, offset),
           Ch0Select(*this, 0, 2), Ch1Select(*this, 2, 2),
           FpgaClkSelect(*this, 8), DividerClockSelect(*this, 10),
           DividerRatio(*this, 12, 2), DividerEnable(*this, 14),
           DividerReset(*this, 15)
            {
            Divider(1);
            AdcClockSelect(0, acPll);
            AdcClockSelect(1, acPll);
            FpgaClockSelect(true);
            }

        void FpgaClockSelect(bool state)
            {
            FFpgaClockSelect = state;
            FpgaClkSelect.Value(state);
            }
        bool FpgaClockSelect() const
            {
            return FpgaClkSelect.Value();
            }

        void AdcClockSelect(int ch, IIAdcClkSelect value)
            {
            FAdcClockSelect[ch] = value;
            if (ch)
                Ch1Select.Value(value);
            else
                Ch0Select.Value(value);
            }
        IIAdcClkSelect AdcClockSelect(int ch) const
            {
            IIAdcClkSelect result;

            switch (ch ? Ch1Select.Value() : Ch0Select.Value())
                {
                case 0:     result = acA;       break;
                case 1:     result = acB;       break;
                default:
                case 2:     result = acPll;     break;
                }
            return result;
            }

        int Divider() const
            {  return FDivider;  }
        void Divider(int value)
            {
            FDivider = value;
            DividerReset.Value(value < 1);
            DividerEnable.Value(value > 1);
            switch (value)
                {
                case 2:     DividerRatio.Value(0);      break;
                case 4:     DividerRatio.Value(1);      break;
                case 8:     DividerRatio.Value(2);      break;
                case 16:    DividerRatio.Value(3);      break;
                }
            }
        void  Reapply()
            {
            Divider(FDivider);
            for (int i=0; i<UwbConstants::ChannelCount; i++)
                AdcClockSelect(FAdcClockSelect[i]);
            }
    private:
        // Data
        int              FDivider;
        IIAdcClkSelect   FAdcClockSelect[UwbConstants::ChannelCount];
        bool             FFpgaClockSelect;
    };

    class GainRegister : public ShadowRegister
    {
    private:
        typedef ShadowRegister inherited;
        static const int Bits = 14;
        static const int CoefMask = (1 << (Bits+1)) - 1;

        const float One;

    public:

        RegisterBitGroup      Gain;

        GainRegister(AddressingSpace &space,
            int offset)
        :  inherited(space, offset), One(1 << Bits), Gain(*this, 0, Bits+1)
            {
            }

        float Val() const
            {
            return static_cast<float>(Gain.Value())/One;
            }
        void Val(float value)
            {
            Gain.Value(static_cast<int>(value*One));
            }

    };
    
    //
    //  PacketRegister
    class PacketRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Size;
        RegisterBitGroup      Id;

        //
        //  Member Functions
        PacketRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Size(*this, 0, 24), Id(*this, 24, 7)
            {}
    };
    

    //
    //  Class UserFpgaCtlRegister
    class UserFpgaCtlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           FpgaReset;
        RegisterBit           Go;
        RegisterBit           ExternalBus;
        RegisterBit           Adc0TestCounter;
        RegisterBit           Adc1TestCounter;
        RegisterBit           Trigger;

        UserFpgaCtlRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               FpgaReset(*this, 0),  Go(*this, 1),
               ExternalBus(*this, 3),
               Adc0TestCounter(*this, 5),
               Adc1TestCounter(*this, 6), Trigger(*this, 10)
            {
            //  Default setup ????
            }

    };

    //
    //  Class TriggerRegister
    class TriggerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Count;
        RegisterBit           FramedMode;
        RegisterBit           External;

        TriggerRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               Count(*this, 0, 16), FramedMode(*this, 30),
               External(*this, 31)
            {
            //  Default setup ????
            }
    };

    //
    //  Class ChannelEnableRegister
    class ChannelEnableRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Adc0Enable;
        RegisterBit           Adc1Enable;

        ChannelEnableRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               Adc0Enable(*this, 0),
               Adc1Enable(*this, 1)
            {
            //  Default setup ????
            }

    };
    //
    //  Class ChannelDestinationRegister
    class ChannelDestinationRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Adc0Dest;
        RegisterBit           Adc1Dest;

        ChannelDestinationRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               Adc0Dest(*this, 0),
               Adc1Dest(*this, 1)
            {
            //  Default setup ????
            }

    };
    //
    //  Class AdcGainControlRegister
    class AdcGainControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Adc0Shutdown;
        RegisterBit           Adc1Shutdown;

        AdcGainControlRegister(AddressingSpace &space, int offset)
            :  inherited(space, offset),
               Adc0Shutdown(*this, 16),
               Adc1Shutdown(*this, 20)
            {
            //  Default setup ????
            }

    };
    //
    //  ...Registers
    PciTestRegister                    PciTestControlReg;
    UserFpgaCtlRegister                UserFpgaControlReg;
    RegisterArray<GainRegister>        GainCoeffReg;        // array of 2
    RegisterArray<Register>            OffsetCoeffReg;      // array of 2
    ChannelEnableRegister              ChannelEnableReg;
    ChannelDestinationRegister         ChannelDestinationReg;
    AdcGainControlRegister             AdcGainControlReg;
    ClockSelectRegister                ClockSelectReg;
    RegisterArray<PacketRegister>      PacketHeaderReg;     // array of 2
    Ics8442                            Clock;
    RegisterArray<TriggerRegister>     TriggerReg;
    ShadowRegister                     ClockTestReg;


    //
    //  Internal Configuration Methods
    RegisterBit & ChannelEnableBit(int ch);
    RegisterBit & ChannelShutdownBit(int ch);
    RegisterBit & ChannelDestinationBit(int ch);
    RegisterBit & ChannelTestCounterOnBit(int ch);

    
    //
    //  External Cfg methods
    //void               Gain(int ch, float value);
    //void               Offset(int ch, int value);
    void                Trigger(bool state);
    bool                Trigger() const;

    void                ExternalTrigger(int ch, bool state);
    bool                ExternalTrigger(int ch) const;

    void                Framed(int ch, int size);
    void                Unframed(int ch);

    void                AdcDestination(int ch, IIDestination dest);
    IIDestination       AdcDestination(int ch);

    void                AdcTestCounterOn(int ch, bool state);
    bool                AdcTestCounterOn(int ch);

    void                ClockTestEnable(bool state);
    bool                ClockTestEnable() const;

    void                ExternalRouting(IIExternalRouting routing);
    IIExternalRouting   ExternalRouting() const;

    void                ReturnPacketSize(int ch, int size_words);
    int                 ReturnPacketSize(int ch) const;
    
    virtual void    	SetCalibrationOffset(int ch, int value);
    virtual void 	    SetCalibrationGain(int ch, int value);
    virtual void		SetCalibrationPllCorrection(float value);

};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
