//---------------------------------------------------------------------------
// Uwb_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <Uwb_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS Uwb  -- Racal Pmc Board object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class Uwb
//---------------------------------------------------------------------------

Uwb::Uwb()
    :  PmcBoard(Device), Ram(Device.Ram), Device(this), FLoopbackMode(lmNone)
{
}

//---------------------------------------------------------------------------
//  destructor for class Uwb
//---------------------------------------------------------------------------

Uwb::~Uwb()
{
    Close();
}

//---------------------------------------------------------------------------
//  Uwb::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t Uwb::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  Uwb::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  Uwb::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  Uwb::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  Uwb::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  Uwb::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  Uwb::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  Uwb::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  Uwb::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  Uwb::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  Uwb::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  Uwb::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  Uwb::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  Uwb::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  Uwb::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  Uwb::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & Uwb::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Uwb::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & Uwb::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Uwb::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  Uwb::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  Uwb::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  Uwb::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  Uwb::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  Uwb::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  Uwb::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  Uwb::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  Uwb::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  Uwb::Log( int type, int value )
{
    Device.Log(type, value);
}

//---------------------------------------------------------------------------
//  Uwb::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & Uwb::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Uwb::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & Uwb::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Uwb::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & Uwb::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  Uwb::PciLoopbackMode() --
//------------------------------------------------------------------------------

void Uwb::PciLoopbackMode(LoopbackMode mode)
{
    FLoopbackMode = mode;

    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:
            mode_value = 0;
            break;
        case lmSerial:
            mode_value = 1;
            break;
        case lmParallel:
            mode_value = 2;
            break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  Uwb::PciLoopbackMode() --
//------------------------------------------------------------------------------

Uwb::LoopbackMode Uwb::PciLoopbackMode()
{
    return FLoopbackMode;
}
 
//---------------------------------------------------------------------------
//  Uwb::Divider() --
//---------------------------------------------------------------------------

Uwb & Uwb::Divider(int value)
{
    Device.ClockSelectReg.Divider(value);
    return *this;
}
 
//---------------------------------------------------------------------------
//  Uwb::Divider() --
//---------------------------------------------------------------------------

int Uwb::Divider() const
{
    return Device.ClockSelectReg.Divider();
}

//---------------------------------------------------------------------------
//  Uwb::DividerClockSelect() --
//---------------------------------------------------------------------------

Uwb & Uwb::DividerClockSelect(IIExtClkSelect value)
{
	Device.ClockSelectReg.DividerClockSelect.Value( (value == ecA) ? 0 : 1);
    return *this;
}
//---------------------------------------------------------------------------
//  Uwb::DividerClockSelect() --
//---------------------------------------------------------------------------

Uwb::IIExtClkSelect Uwb::DividerClockSelect() const
{
    return static_cast<IIExtClkSelect>(Device.ClockSelectReg.DividerClockSelect.Value());
}

//---------------------------------------------------------------------------
//  Uwb::FpgaClockSelect() --
//---------------------------------------------------------------------------

Uwb & Uwb::FpgaClockSelect(IIExtClkSelect value)
{
    bool bit = (value == ecA) ? false : true;
 	Device.ClockSelectReg.FpgaClockSelect( bit );
    return *this;
}
 
//---------------------------------------------------------------------------
//  Uwb::FpgaClockSelect() --
//---------------------------------------------------------------------------

Uwb::IIExtClkSelect Uwb::FpgaClockSelect() const
{
    return static_cast<IIExtClkSelect>(Device.ClockSelectReg.FpgaClockSelect());
}

//---------------------------------------------------------------------------
//  Uwb::Clock() --
//---------------------------------------------------------------------------

Ics8442 & Uwb::Clock()
{
    return Device.Clock;
}
 
//---------------------------------------------------------------------------
//  Uwb::AdcClockSelect() --
//---------------------------------------------------------------------------

Uwb & Uwb::AdcClockSelect(int ch, IIAdcClkSelect value)
{
    UwbDeviceMap::ClockSelectRegister::IIAdcClkSelect clk;
    switch (value)
        {
        case acA:       clk = UwbDeviceMap::ClockSelectRegister::acA;   break;
        case acB:       clk = UwbDeviceMap::ClockSelectRegister::acB;   break;
        default:
        case acPll:     clk = UwbDeviceMap::ClockSelectRegister::acPll; break;
        }

    Device.ClockSelectReg.AdcClockSelect(ch, clk);
    return *this;
}
 
//---------------------------------------------------------------------------
//  Uwb::AdcClockSelect() --
//---------------------------------------------------------------------------

Uwb::IIAdcClkSelect Uwb::AdcClockSelect(int ch) const
{
    IIAdcClkSelect clk;
    switch (Device.ClockSelectReg.AdcClockSelect(ch))
        {
        case UwbDeviceMap::ClockSelectRegister::acA:       clk = acA;   break;
        case UwbDeviceMap::ClockSelectRegister::acB:       clk = acB;   break;
        default:
        case UwbDeviceMap::ClockSelectRegister::acPll:     clk = acPll; break;
        }

    return clk;
}

//---------------------------------------------------------------------------
//  Uwb::PllClockSelect() --
//---------------------------------------------------------------------------

Uwb & Uwb::PllClockSelect(IIPllClkSelect value)
{
    Ics8442::IIReferenceClk clk;
    switch (value)
        {
        case pcDivider:     clk = Ics8442::rcDivider;   break;
        default:
        case pcCrystal:     clk = Ics8442::rcCrystal;   break;
        }

    Device.Clock.ReferenceClock(clk);
    return *this;
}
 
//---------------------------------------------------------------------------
//  Uwb::PllClockSelect() --
//---------------------------------------------------------------------------

Uwb::IIPllClkSelect Uwb::PllClockSelect() const
{
    Uwb::IIPllClkSelect clk;

    switch (Device.Clock.ReferenceClock())
        {
        case Ics8442::rcDivider:    clk = pcDivider;    break;
        default:
        case Ics8442::rcCrystal:    clk = pcCrystal;    break;
        }

    return clk;
}
 
//---------------------------------------------------------------------------
//  Uwb::TestMode() --
//---------------------------------------------------------------------------

Uwb & Uwb::TestMode(IITestMode output)
{
    Ics8442::IITestMode mode;

    switch (output)
        {
        case toDisabled:            mode = Ics8442::toDisabled;         break;
        case toShiftReg:            mode = Ics8442::toShiftReg;         break;
        case toFeedbackDivider:     mode = Ics8442::toFeedbackDivider;  break;
        default:
        case toNormal:              mode = Ics8442::toNormal;           break;
        }

    Device.Clock.TestMode(mode);
    return *this;
}
 
//---------------------------------------------------------------------------
//  Uwb::TestMode() --
//---------------------------------------------------------------------------

Uwb::IITestMode Uwb::TestMode() const
{
    IITestMode mode;
    switch (Device.Clock.TestMode())
        {
        case toDisabled:            mode = toDisabled;         break;
        case toShiftReg:            mode = toShiftReg;         break;
        case toFeedbackDivider:     mode = toFeedbackDivider;  break;
        default:
        case toNormal:              mode = toNormal;           break;
        }

    return mode;
}


//---------------------------------------------------------------------------
//  Uwb::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPllCalibration & Uwb::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  Uwb::ReturnPacketSize() -- Set size of packet coming in
//---------------------------------------------------------------------------

Uwb &  Uwb::ReturnPacketSize(int ch, int size_words)
{
    Device.ReturnPacketSize(ch, size_words);
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::ReturnPacketSize() -- Set size of packet coming in
//---------------------------------------------------------------------------

int  Uwb::ReturnPacketSize(int ch) const
{
    return Device.ReturnPacketSize(ch);
}

//---------------------------------------------------------------------------
//  Uwb::AdcDestination() -- Route channels
//---------------------------------------------------------------------------

Uwb &  Uwb::AdcDestination(int ch, Uwb::IIDestination dest)
{
    Device.AdcDestination(ch, static_cast<UwbDeviceMap::IIDestination>(dest));
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::AdcDestination() -- Route channels
//---------------------------------------------------------------------------

Uwb::IIDestination  Uwb::AdcDestination(int ch) 
{
    return static_cast<Uwb::IIDestination>(Device.AdcDestination(ch));
}

//---------------------------------------------------------------------------
//  Uwb::AdcEnabled() -- Enable channels
//---------------------------------------------------------------------------

Uwb &  Uwb::AdcEnabled(int ch, bool state)
{
    Channels().Enabled(ch, state);
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::AdcEnabled() -- Enable channels
//---------------------------------------------------------------------------

bool  Uwb::AdcEnabled(int ch) 
{
    return Channels().Enabled(ch);
}

//---------------------------------------------------------------------------
//  Uwb::ExternalRouting() -- Route external data
//---------------------------------------------------------------------------

Uwb &  Uwb::ExternalRouting(IIExternalRouting routing)
{
    Device.ExternalRouting(static_cast<UwbDeviceMap::IIExternalRouting>(routing));
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::ExternalRouting() -- Route channels
//---------------------------------------------------------------------------

Uwb::IIExternalRouting  Uwb::ExternalRouting() const
{
    return static_cast<Uwb::IIExternalRouting>(Device.ExternalRouting());
}

//---------------------------------------------------------------------------
//  Uwb::ExternalTrigger() -- use external trigger
//---------------------------------------------------------------------------

Uwb &  Uwb::ExternalTrigger(int ch, bool state)
{
    Device.ExternalTrigger(ch, state);
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::ExternalTrigger() -- Route channels
//---------------------------------------------------------------------------

bool  Uwb::ExternalTrigger(int ch) const
{
    return Device.ExternalTrigger(ch) ? true : false;
}

//---------------------------------------------------------------------------
//  Uwb::Framed() --
//---------------------------------------------------------------------------

Uwb & Uwb::Framed(int ch, int size)
{
    Device.Framed(ch, size);
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::Unframed() --
//---------------------------------------------------------------------------

Uwb & Uwb::Unframed(int ch)
{
    Device.Unframed(ch);
    return *this;
}

//---------------------------------------------------------------------------
//  Uwb::Trigger() -- Set trigger enable
//---------------------------------------------------------------------------

void Uwb::Trigger(bool state)
{
    Device.Trigger(state);
}

//---------------------------------------------------------------------------
//  Uwb::Trigger() -- Query trigger enable
//---------------------------------------------------------------------------

bool Uwb::Trigger() const
{
     return Device.Trigger();
}

//---------------------------------------------------------------------------
//  Uwb::AdcTestCounterOn() -- Enable channels
//---------------------------------------------------------------------------

Uwb &  Uwb::AdcTestCounterOn(int ch, bool state)
{
    Device.AdcTestCounterOn(ch, state);
    return *this;
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ClockTestEnable() --
//---------------------------------------------------------------------------

void Uwb::ClockTestEnable(bool state)
{
    Device.ClockTestEnable(state);
}

//---------------------------------------------------------------------------
//  DigitalReceiver::ClockTestEnable() --
//---------------------------------------------------------------------------

bool Uwb::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  Uwb::AdcTestCounterOn() -- Enable channels
//---------------------------------------------------------------------------

bool  Uwb::AdcTestCounterOn(int ch) 
{
    return Device.AdcTestCounterOn(ch);   
}

//------------------------------------------------------------------------------
//  Uwb::Channels() --  Reference to channel information object
//------------------------------------------------------------------------------

UnpairedChannelInfo & Uwb::Channels()
{
    return Device.FChannels;
}

//---------------------------------------------------------------------------
//  Uwb::BusMasterSize() --
//---------------------------------------------------------------------------

void Uwb::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Uwb::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void Uwb::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Uwb::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void Uwb::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  Uwb::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  Uwb::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  Uwb::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  Uwb::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // namespace Innovative
