// Uwb_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef Uwb_MbH
#define Uwb_MbH

#include <PmcBoard_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <UwbDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS Uwb  -- Uwb Base Board object
//===========================================================================

class Uwb :  public PmcBoard,
             public IStreamDeviceEvents
{
    friend class UwbDeviceMap;
    
public:
    //
    //  Member Functions
    Uwb();
    virtual ~Uwb();

    size_t          BoardCount();
    void            Open();
    void            Close();

    int   Target();
    void  Target( int value );

    bool  Led();
    void  Led( bool value );

    FpgaLogicVersionInfoIntf & UserLogicVersion();
    PciLogicVersionInfoIntf & PciLogicVersion();
    IPciBusDiagnostics & PciDiagnostics();

	ii32  PeekLogic( int offset );
    void  PokeLogic( int offset, ii32 value );
    ii32  PeekPort( int offset );
    void  PokePort( int offset, ii32 value );
	//
	//  IPacketStreamDevice interface implementation
    virtual void  Reset();
    UwbDeviceMap * Debug() { return &Device; };
    virtual void   Log(int type, int data);

    // Logic programming
    IUsesVirtexFpgaLoader & Logic();
    // EEprom programming
    IUsesVirtexJtagLoader & Eeprom();

    // Trigger Support
    // Enumerations
    enum LoopbackMode { lmNone, lmParallel, lmSerial };

    // Methods
    void            PciLoopbackMode(LoopbackMode mode);
    LoopbackMode    PciLoopbackMode();

    enum IIExtClkSelect { ecA, ecB };
    enum IIAdcClkSelect { acA, acB, acPll };
    enum IIPllClkSelect { pcDivider, pcCrystal };
    enum IITestMode { toDisabled, toShiftReg, toFeedbackDivider, toNormal };
    enum IIDestination { dstNormal, dstExternal };
    enum IIExternalRouting { extJ4, extXmc };

    // Properties
    Uwb &               ReturnPacketSize(int ch, int size_words);
    int                 ReturnPacketSize(int ch) const;

    Uwb &               AdcEnabled(int ch, bool state);
    bool                AdcEnabled(int ch);

    Uwb &               AdcDestination(int ch, IIDestination dest);
    IIDestination       AdcDestination(int ch);

    Uwb &               AdcTestCounterOn(int ch, bool state);
    bool                AdcTestCounterOn(int ch);

    Uwb &               ExternalRouting(IIExternalRouting routing);
    IIExternalRouting   ExternalRouting() const;
    
    Uwb &               ExternalTrigger(int ch, bool state);
    bool                ExternalTrigger(int ch) const;

    Uwb &               Divider(int value);
    int                 Divider() const;

    Uwb &               Framed(int ch, int size);
    Uwb &               Unframed(int ch);

    void                Trigger(bool state);
    bool                Trigger() const;

    void                ClockTestEnable(bool state);
    bool                ClockTestEnable() const;

    Uwb &               PllClockSelect(IIPllClkSelect value);
    IIPllClkSelect      PllClockSelect() const;

    Uwb &               DividerClockSelect(IIExtClkSelect value);
    IIExtClkSelect      DividerClockSelect() const;

    Uwb &               AdcClockSelect(int ch, IIAdcClkSelect value);
    IIAdcClkSelect      AdcClockSelect(int ch) const;

    Uwb &               FpgaClockSelect(IIExtClkSelect value);
    IIExtClkSelect      FpgaClockSelect() const;

    Uwb &               TestMode(IITestMode output);
    IITestMode          TestMode() const;

	void 							   BusMasterSize(int bytes);
	void 							   IncomingBusMasterSize(int bytes);
	void 							   OutgoingBusMasterSize(int bytes);
	int             				   IncomingBusMasterSize() const;
	int             				   OutgoingBusMasterSize() const;

	// Methods
    Ics8442 &               Clock();
    IUsesPllCalibration &	IdRom();
    UnpairedChannelInfo &   Channels();

    // Zbt Ram
    ZbtRam &                Ram;

protected:
    //
    //  Member Data
    UwbDeviceMap                Device;
    LoopbackMode                FLoopbackMode;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

