//
// VcxoControl_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef VcxoControl_MbH
#define VcxoControl_MbH

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
// CLASS IVcxoControl  --  Base Interface for VCXO devices
//===========================================================================

class IVcxoControl
{
public:
    virtual double  Min() = 0;         //  Tuning Range
    virtual double  Max() = 0;
    virtual bool    CanTuneTo(double freq) = 0;
    virtual double  BestTunableFrequency(double freq) = 0;

    //  General programming features (may be unimplemented)
    virtual void    SetVcoFrequency(double freq) = 0;
    virtual void    SetRange(double min, double max) = 0;
    virtual void    OutputEnabled(bool /*state*/) {}

    virtual ~IVcxoControl()  {}
};

//===========================================================================
// CLASS IVcxoControlProxy  --  Proxy to allow 'owning' any Vcxo Controller
//===========================================================================

class IVcxoControlProxy : public IVcxoControl
{
public:
    IVcxoControlProxy(IVcxoControl * ptr)
        :  Ptr(ptr)
        {}
    virtual ~IVcxoControlProxy()
        {}
    virtual double  Min()         //  Tuning Range
        {  return Ptr->Min();   }
    virtual double  Max()
        {  return Ptr->Max();   }
    virtual bool    CanTuneTo(double freq)
        {  return Ptr->CanTuneTo(freq);   }
    virtual double  BestTunableFrequency(double freq)
        {  return Ptr->BestTunableFrequency(freq);   }

    //  General programming features (may be unimplemented)
    virtual void    SetVcoFrequency(double freq)
        {  return Ptr->SetVcoFrequency(freq);   }
    virtual void    SetRange(double min, double max)
        {  return Ptr->SetRange(min, max);   }

private:
    IVcxoControl * Ptr;
};

//===========================================================================
// CLASS RangedVcxoControl  --  Vcxo device with settable range
//===========================================================================

class RangedVcxoControl : public IVcxoControl
{
public:
    RangedVcxoControl(double min, double max)
        :  VcxoMin(min), VcxoMax(max)
        {}
    virtual ~RangedVcxoControl()
        {}

    virtual double  Min()
        {  return VcxoMin;  }
    virtual double  Max()
        {  return VcxoMax;  }

    virtual bool    CanTuneTo(double freq)
        {
        if (freq >= Min() && freq <= Max())
            return true;
        else
            return false;
        }
    virtual double  BestTunableFrequency(double freq)
        {
        if (freq <= Min())
            return Min();
        else if (freq >= Max())
            return Max();
        else
            return freq;
        }
    virtual void    SetVcoFrequency(double /*freq*/)
        { }

    virtual void    SetRange(double min, double max)
        {  VcxoMin = min;  VcxoMax = max;  }

private:
    double  VcxoMin;
    double  VcxoMax;
};


#ifdef _MSC_VER
	#pragma warning(pop)
#endif

} // namespace Innovative

#endif

