// VirtexFpgaLoader_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <VirtexFpgaLoader_Mb.h>
#include <LogicLoadThread_Mb.h>

#ifdef _MSC_VER		
	#pragma warning(push)	
	#pragma warning(disable : 4355)		
#endif

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS FpgaConfigControlRegister
//===========================================================================

class FpgaConfigControlRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBit           Prog;
    ReadOnlyRegisterBit   Busy;
    ReadOnlyRegisterBit   Done;
    ReadOnlyRegisterBit   Init;

public:
    FpgaConfigControlRegister(AddressingSpace &space, int offset)
    :  inherited( space, offset ),
          Prog(*this, 4), Busy(*this, 0), Done(*this, 1), Init(*this, 2)
        {
        }
};

//===========================================================================
//  CLASS VirtexFpgaLoader
//===========================================================================

class VirtexFpgaLoader : public FpgaLoadBase
{
public:
    VirtexFpgaLoader(AddressingSpace & space,
                     unsigned int data_addr,
                     unsigned int control_addr )
        : Data(space, data_addr),
          Control(space, control_addr)
        {  }

    // Methods
    virtual void Program(bool state)
        {
        if (state)
            Control.Prog.Clear();
        else
            Control.Prog.Set();
        }

    virtual bool IsInit()
        {  return Control.Init.Value() ? false : true;  }

    virtual bool IsDone()
        {  return Control.Done.Value() ? true : false;  }

    virtual void Load(char * buffer, size_t count)
        {
        for (size_t i = 0; i < count; ++i)
            Data = *buffer++;
        }


private:
    Register                           Data;
    FpgaConfigControlRegister          Control;
};

//===========================================================================
//  CLASS IUsesVirtexFpgaLoader -- User Interface to add in Virtex Loader
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IUsesVirtexFpgaLoader
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader::IUsesVirtexFpgaLoader(AddressingSpace & space,
                                             unsigned int data_addr,
                                             unsigned int control_addr)
    :  Fpga(new VirtexFpgaLoader(space, data_addr, control_addr))
{
    Construct();
}

//---------------------------------------------------------------------------
//  constructor for class IUsesVirtexFpgaLoader
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader::IUsesVirtexFpgaLoader(FpgaLoadBase *fpga)
    :  Fpga(fpga)
{
    Construct();
}

//---------------------------------------------------------------------------
//  destructor for class IUsesVirtexFpgaLoader
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader::~IUsesVirtexFpgaLoader()
{
    LogicLoader->Shutdown();
    delete LogicLoader;
    LogicLoader = 0;

    delete Fpga;
    Fpga = 0;
}

//---------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::Construct() -- Common ctor body
//---------------------------------------------------------------------------

void  IUsesVirtexFpgaLoader::Construct()
{
    LogicLoader = new LogicLoadThread(*Fpga);

    LogicLoader->OnFileReadProgress.SetEvent(this, &IUsesVirtexFpgaLoader::HandleFileReadProgress);
    LogicLoader->OnFileReadComplete.SetEvent(this, &IUsesVirtexFpgaLoader::HandleFileReadComplete);

    LogicLoader->OnParseProgress.SetEvent(this, &IUsesVirtexFpgaLoader::HandleParseProgress);
    LogicLoader->OnParseComplete.SetEvent(this, &IUsesVirtexFpgaLoader::HandleParseComplete);
    LogicLoader->OnParseError.SetEvent(this, &IUsesVirtexFpgaLoader::HandleParseMessage);

    LogicLoader->OnLoadProgress.SetEvent(this, &IUsesVirtexFpgaLoader::HandleLoadProgress);
    LogicLoader->OnLoadComplete.SetEvent(this, &IUsesVirtexFpgaLoader::HandleLoadComplete);
    LogicLoader->OnLoadError.SetEvent(this, &IUsesVirtexFpgaLoader::HandleLoadMessage);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::IsConfigured()  --   Return true if logic loaded
//------------------------------------------------------------------------------

bool IUsesVirtexFpgaLoader::IsConfigured() const
{
    return Fpga->IsDone();
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::ConfigureFpga()  --   Configure FPGA
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::ConfigureFpga(std::string exofile)
{
	LogicLoader->Configure(exofile);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::ConfigureFpga()  --   Configure FPGA
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::ConfigureFpga(char * buffer, size_t size)
{
	LogicLoader->Load(buffer, size);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleFileReadProgress() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleFileReadProgress(ProcessProgressEvent & event)
{
    OnFpgaFileReadProgress.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleFileReadComplete() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleFileReadComplete(ProcessCompletionEvent & event)
{
    OnFpgaFileReadComplete.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleParseProgress() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleParseProgress(ProcessProgressEvent & event)
{
    OnFpgaParseProgress.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleParseMessage() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleParseMessage(ProcessStatusEvent & event)
{
    OnFpgaParseMessage.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleParseComplete() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleParseComplete(ProcessCompletionEvent & event)
{
    OnFpgaParseComplete.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleLoadProgress() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleLoadProgress(ProcessProgressEvent & event)
{
    OnFpgaLoadProgress.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleLoadMessage() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleLoadMessage(ProcessStatusEvent & event)
{
    OnFpgaLoadMessage.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexFpgaLoader::HandleLoadComplete() --
//------------------------------------------------------------------------------

void IUsesVirtexFpgaLoader::HandleLoadComplete(ProcessCompletionEvent & event)
{
    OnFpgaLoadComplete.Execute(event);
}


}  // end namespace Innovative



