// VirtexJtagLoader_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <HardwareRegister_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <string>

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS JtagPort  --
//===========================================================================

class JtagPort : public PmcJtagPort
{
public:
    // Ctor
    JtagPort(Innovative::Register & reg)
        : Reg(reg)
        {
        }

    // Standard Properties
    virtual void            Init()
        {
        Tms(0);
        Tdi(1);
        Tck(1);
        }
    virtual void            Port(int value)
        {  Reg.Value(value);  }
    virtual int             Port() const
        {  return Reg.Value();  }

private:
    // Data
    Innovative::Register &      Reg;
};

//===========================================================================
//  CLASS IUsesVirtexJtagLoader -- User Interface to mix-in JTAG Loader
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IUsesVirtexJtagLoader
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader::IUsesVirtexJtagLoader(AddressingSpace & space,
                                             unsigned int addr)
    : Reg(new Register(space, addr))
{
    Port = new JtagPort(*Reg);
    Loader = new XsvfLoadThread(Port);

    Loader->OnLoadProgress.SetEvent(this, &IUsesVirtexJtagLoader::HandleProgress);
    Loader->OnLoadComplete.SetEvent(this, &IUsesVirtexJtagLoader::HandleComplete);
    Loader->OnLoadMessage.SetEvent(this, &IUsesVirtexJtagLoader::HandleMessage);
}

//---------------------------------------------------------------------------
//  constructor for class IUsesVirtexJtagLoader
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader::IUsesVirtexJtagLoader(PmcJtagPort *port)
    : Port(port)
{
    Loader = new XsvfLoadThread(Port);

    Loader->OnLoadProgress.SetEvent(this, &IUsesVirtexJtagLoader::HandleProgress);
    Loader->OnLoadComplete.SetEvent(this, &IUsesVirtexJtagLoader::HandleComplete);
    Loader->OnLoadMessage.SetEvent(this, &IUsesVirtexJtagLoader::HandleMessage);
}

//---------------------------------------------------------------------------
//  destructor for class IUsesVirtexJtagLoader
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader::~IUsesVirtexJtagLoader()
{
    Loader->Shutdown();
    delete Loader;
    Loader = 0;

    delete Port;
    Port = 0;

    delete Reg;
    Reg = 0;
}

//------------------------------------------------------------------------------
//  IUsesVirtexJtagLoader::LoadJtag()  --   Load EEProm via Jtag
//------------------------------------------------------------------------------

void IUsesVirtexJtagLoader::LoadJtag(std::string file, int debug_level)
{
    Port->Init();
    Loader->Load(file, debug_level);
}

//------------------------------------------------------------------------------
//  IUsesVirtexJtagLoader::HandleProgress() --
//------------------------------------------------------------------------------

void IUsesVirtexJtagLoader::HandleProgress(ProcessProgressEvent & event)
{
    OnJtagLoadProgress.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexJtagLoader::HandleMessage() --
//------------------------------------------------------------------------------

void IUsesVirtexJtagLoader::HandleMessage(ProcessStatusEvent & event)
{
    OnJtagLoadMessage.Execute(event);
}

//------------------------------------------------------------------------------
//  IUsesVirtexJtagLoader::HandleComplete() --
//------------------------------------------------------------------------------

void IUsesVirtexJtagLoader::HandleComplete(ProcessCompletionEvent & event)
{
    OnJtagLoadComplete.Execute(event);
}



}  // end namespace Innovative



