// VitaPacketFileDataSet_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include <IniFile_Mb.h>
#include <VitaPacketFileDataSet_Mb.h>
#include <FileSupport_Mb.h>
#include <algorithm>
#include <limits>
#include <cstring>

using namespace std;

namespace Innovative
{

namespace VitaPktFileDS
{

const size_t MaximumIndex    = std::numeric_limits<size_t>::max();
const ii64   MaximumIndex64  = std::numeric_limits<ii64>::max();

//===========================================================================
//  Class PacketProxy
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PacketProxy
//---------------------------------------------------------------------------

PacketProxy::PacketProxy()
    :  PFile(0)
{
}

//---------------------------------------------------------------------------
//  constructor for class PacketProxy - build from sample idx
//---------------------------------------------------------------------------
//  Used for scanning array for proxy containing sample

PacketProxy::PacketProxy(ii64 sample_idx)
    :  BaseSampleNumber(sample_idx), PFile(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class PacketProxy
//---------------------------------------------------------------------------

PacketProxy::~PacketProxy()
{
}

//---------------------------------------------------------------------------
//  PacketProxy::AllocateBody() -- Fill proxy with body data from file
//---------------------------------------------------------------------------

void PacketProxy::AllocateBody()
{
    if (!Body.IsAllocated())
        {
        //  Find
        PFile->Seek(OffsetAfterHeader());
        //  Allocate
        Body.Allocate(DataSize);
        //  Fill
		PFile->Read(Body.Data, DataSize);
        }
}

//---------------------------------------------------------------------------
//  PacketProxy::ReleaseBody() -- empty data from proxy
//---------------------------------------------------------------------------

void PacketProxy::ReleaseBody()
{
    if (Body.IsAllocated())
        {
        Body.Release();
		}
}

//===========================================================================
//  Class PidPacketArray
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class PidPacketArray
//---------------------------------------------------------------------------

PidPacketArray::PidPacketArray()
	: MaxSampleIdx(0), CacheSizeBlocks(500), CurrentProxyIdx(MaximumIndex)
{
}

//---------------------------------------------------------------------------
//  destructor for class PidPacketArray
//---------------------------------------------------------------------------

PidPacketArray::~PidPacketArray()
{
}

//---------------------------------------------------------------------------
//  PidPacketArray::Clear() -- clear all state of array
//---------------------------------------------------------------------------

void  PidPacketArray::Clear()
{
    MaxSampleIdx = 0;
    CurrentProxyIdx= MaximumIndex;

    Packets.clear();
	Cache.clear();
}

//---------------------------------------------------------------------------
//  PidPacketArray::CurrentSample() -- Read sample
//---------------------------------------------------------------------------

Subsample PidPacketArray::CurrentSample(int channel_in_pid, ii64 index)
{
	Subsample value(0);

	FindProxyForSample(index);

	if (CurrentProxyIdx != MaximumIndex)
		{
		PacketProxy & CurrentProxy(Packets[CurrentProxyIdx]);

        //
        //  Manage Proxy Data Allocation
        AllocateProxy();
        //
        //  Fetch the data point
        switch (CurrentProxy.BytesPerSample)
            {
            case 4:
                {
                unsigned int * BodyPtr = CurrentProxy.Body.Data;
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_pid;
				memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 4);
                }
                break;
            case 2:
                {
                unsigned short * BodyPtr = reinterpret_cast<unsigned short *>(CurrentProxy.Body.Data);
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_pid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 2);
                }
                break;
            case 1:
				{
                unsigned char * BodyPtr = reinterpret_cast<unsigned char *>(CurrentProxy.Body.Data);
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_pid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 1);
                }
                break;
            }
		}

    return value;
}

//---------------------------------------------------------------------------
//  PidPacketArray::CurrentSample() -- Write sample
//---------------------------------------------------------------------------

void PidPacketArray::CurrentSample(int /*channel_in_pid*/, ii64 /*index*/, Subsample & /*value*/)
{
    // Intentionally blank so far
}

//---------------------------------------------------------------------------
//  PidPacketArray::LastPacket() -- Return last packet in the array
//---------------------------------------------------------------------------

PacketProxy *   PidPacketArray::LastPacket()
{
    if (Packets.size()==0)
        return 0;
    else
        {
        size_t idx = Packets.size()-1;
        return &(Packets[idx]);
        }
}

//------------------------------------------------------------------------------
//  PidPacketArray::FindProxyForSample() -- Find proxy that holds sample idx
//------------------------------------------------------------------------------

void  PidPacketArray::FindProxyForSample(ii64 index)
{
    if (Packets.size()==0)
        {
		CurrentProxyIdx = MaximumIndex;
        return;
        }
    //  Check if current block has the sample
    if ((CurrentProxyIdx != MaximumIndex ) && Packets[CurrentProxyIdx].PacketContainsSample(index))
        return;

    // Scan our array for the block that does contain the sample...
    PacketProxy candidate(index);
    PacketArray::iterator iter = std::lower_bound(Packets.begin(), Packets.end(), candidate);
    if (iter != Packets.end())
        {
        CurrentProxyIdx = iter - Packets.begin();
        }
    else
        {
		CurrentProxyIdx = Packets.end() - Packets.begin() - 1;
//		CurrentProxyIdx = MaximumIndex;		// should never happen
        }
}

//------------------------------------------------------------------------------
//  PidPacketArray::AllocateProxy() -- Manage cache allocation of proxy
//------------------------------------------------------------------------------

void  PidPacketArray::AllocateProxy()
{
    PacketProxy & CurrentProxy(Packets[CurrentProxyIdx]);
	//
	//  CurrentProxy may not have been allocated yet.
    if (CurrentProxy.Body.IsAllocated()==false)
        {
		CurrentProxy.AllocateBody();
        //
        //  Push IDX of proxy onto cache
		Cache.push_front(CurrentProxyIdx);
        //  if too many blocks in cache, free one
        if (Cache.size() > CacheSizeBlocks)
            {
            size_t Last = Cache.back();
            //  Release from cache
            Cache.pop_back();
			//  Release data block of packet
            Packets[Last].ReleaseBody();
            }
        }

}

//------------------------------------------------------------------------------
//  PidPacketArray::FoundProxy() -- View found Proxy
//------------------------------------------------------------------------------

PacketProxy &  PidPacketArray::FoundProxy()
{
    return Packets[CurrentProxyIdx];
}

//------------------------------------------------------------------------------
//  PidPacketArray::IsFoundProxyValid() -- is proxy valid?
//------------------------------------------------------------------------------

bool  PidPacketArray::IsFoundProxyValid()
{
    return CurrentProxyIdx != MaximumIndex;
}

}  // namespace VitaPktFileDS


//===========================================================================
// Class VeloPacketFileDataSet
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for VeloPacketFileDataSet
//---------------------------------------------------------------------------

const int kVeloPacketHeaderSize    =    4;
const int kVeloAlertPacketFullSize = 0x28;
const int kAlertPid = 0xFF;

//---------------------------------------------------------------------------
//  constructor for VeloPacketFileDataSet
//---------------------------------------------------------------------------

VeloPacketFileDataSet::VeloPacketFileDataSet()
	: FOwner(0), FSampleIdx(0), FImaginary(false),
	  FFileReadOk(true), FLastArray(0), FAutoCreate(false)
{
}

//---------------------------------------------------------------------------
//  destructor for VeloPacketFileDataSet
//---------------------------------------------------------------------------

VeloPacketFileDataSet::~VeloPacketFileDataSet()
{
    ImageClose();
}

//---------------------------------------------------------------------------
//  VeloPacketFileDataSet::ImageInit() -- On Image Initialization
//---------------------------------------------------------------------------

void  VeloPacketFileDataSet::ImageInit()
{
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageOpen() -- Open disk file
//---------------------------------------------------------------------------

bool VeloPacketFileDataSet::ImageOpen()
{
    bool ForceDataScan(true);
    File.Open(FFileName, FAutoCreate, FReadOnly);
    int FChannelSave = FChannel;
    int testChan = 0;
    for (int i=0; i<FDevices; i++)
		{
        //  Find sample 0 on ALL devices
        FChannel = testChan;
        ParseDataFile(ForceDataScan);
        testChan += SubchannelsPerDevice();
        }
    FChannel = FChannelSave;  // restore original channel.

    FFileReadOk = true;
	Init();

    return true;
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageClose() -- Close disk file
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::ImageClose()
{
    //  Clean out Alert packet array
	Alert_PPA.Clear();
    //  Clean out Pid parser map
    {
    PidPacketMap  eraser;
    eraser.swap(Pid);
    }
    //  Clean out parse limit array
	{
    std::vector<ii64>  eraser;
    eraser.swap(FParseLimitIdx);
    }
    FLastArray = 0;
    FSampleIdx = 0;
    FImaginary = false;


    File.Close();
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::Subchannel() -- Return current subchannel
//---------------------------------------------------------------------------

int VeloPacketFileDataSet::Subchannel() const
{
    int ch;

    if (FComplex)
        ch = FChannel * Subsamples() + (FImaginary ? 1 : 0);
    else
        ch = FChannel;

    return ch;
}

//---------------------------------------------------------------------------
//  VeloPacketFileDataSet::ImageSampleFetch() -- Fetch sample from current channel
//---------------------------------------------------------------------------

void  VeloPacketFileDataSet::ImageSampleFetch(Subsample & sample)
{
    if (!File.IsOpen())
        return;
    //
    //  If we havent parsed this far, we will now!
    ParseDataFile();

    int subchannel = Subchannel()%SubchannelsPerDevice();
    sample = WorkingPid().CurrentSample(subchannel, FSampleIdx);
	char * pSample = sample.asString();
    CorrectEndian(pSample);
    sample.Type(static_cast<enum Subsample::SampleType>(FType));

    FImaginary = !FImaginary;
}

//---------------------------------------------------------------------------
//  VeloPacketFileDataSet::ImageSampleStore() -- Store sample to current channel
//---------------------------------------------------------------------------

void  VeloPacketFileDataSet::ImageSampleStore(Subsample & sample)
{
    if (!File.IsOpen() || FReadOnly)
        return;

    int subchannel = Subchannel()%SubchannelsPerDevice();
    char * pSample = sample.asString();
    CorrectEndian(pSample);
    WorkingPid().CurrentSample(subchannel, FSampleIdx, sample);

    FImaginary = !FImaginary;
}

//---------------------------------------------------------------------------
//  VeloPacketFileDataSet::PreAccess() -- Init prior to sample access
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::PreAccess()
{
    FImaginary = false;
}

//---------------------------------------------------------------------------
//  VeloPacketFileDataSet::PostAccess() -- Finalize after sample access
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::PostAccess()
{
    ++FSampleIdx;
    Progress(FSampleIdx);
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageFlush() -- Write dirty cache to disk
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::ImageFlush()
{
	File.Flush();
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageSeek() -- Seek to byte offset in disk file
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::ImageSeek(ii64 offset)
{
	FSampleIdx = offset/FEventSize;
    Progress(FSampleIdx);
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageSize() -- Return size of disk file
//---------------------------------------------------------------------------
//  1) Subtract off Velo Headers from total data size
//  2) Subtract off Alert Header and Data from total data size

ii64 VeloPacketFileDataSet::ImageSize() const
{
	size_t size = File.Size();

	ii64 result = size - (PacketCount()*kVeloPacketHeaderSize)*sizeof(int)
					   - (AlertPacketCount()*kVeloAlertPacketFullSize)*sizeof(int);

	return result;
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageSize() -- Set size of disk file
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::ImageSize(ii64 /*size*/)
{
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ParseDataFile() --
//---------------------------------------------------------------------------

bool VeloPacketFileDataSet::ParseDataFile(bool force_scan)
{
	if (!File.IsOpen())
		return false;
	//
	//  First lets check if this sample has already been parsed.
	if (!force_scan)
		{
		ii64 idx = WorkingPid().MaxSampleIdx;
		if (idx > FSampleIdx)
			return true;
		}

	// Otherwise Parse the file until we find the channel's sample
	//   Position at last parsed position
	ii64  Position = LastParsePosition();       // take outa loop - redundant
	File.Seek(Position);

	while (!Abort())
		{
		//
		//  Extract the next packet header
		VitaPktFileDS::PacketProxy Proxy;
		//
		//  Read the header
		FFileReadOk = true;
		int PacketHeaderSize = Proxy.Header.SizeInInts();
		bool readOK = File.Read(Proxy.Header.Raw(), PacketHeaderSize) &&
			(Position <= File.Size());

		if (!readOK)
			{
			FFileReadOk = false;
			break;
			}

        if (readOK && BigEndian())
            {
            unsigned int * p = Proxy.Header.Raw();
            for (int i = 0; i < PacketHeaderSize; ++i)
                CorrectEndian(reinterpret_cast<char*>(p++));
            }

        //
        //  Find out which PPA this packet will be added to..
        int PacketPID = Proxy.Header.PeripheralId();

        if (PacketPID != kAlertPid)
            {
            // not an alert
            VitaPktFileDS::PidPacketArray & PPA = Pid[ PacketPID ];
            FLastArray = &PPA;

            VitaPktFileDS::PidPacketArray::PacketArray & PPV = PPA.Packets;

            //  Fill in the rest
			InitializeProxy(Proxy, Position, PPA.MaxSampleIdx);
            //
            //  Add proxy to our array
            PPA.MaxSampleIdx  += Proxy.Samples;
            PPV.push_back(Proxy);
            }
        else
            {
            //  an alert
            VitaPktFileDS::PidPacketArray::PacketArray & PPV = Alert_PPA.Packets;
            //  Fill in the rest
			InitializeProxy(Proxy, Position, Alert_PPA.MaxSampleIdx);
            //
            //  Add proxy to our array
            Alert_PPA.MaxSampleIdx  += Proxy.Samples;
            PPV.push_back(Proxy);
            }

        // Skip the data
        int PacketSize = Proxy.Header.PacketSize();
        int PacketDataSize = PacketSize - PacketHeaderSize;

        File.Skip(PacketDataSize);
        Position = File.Position();
        //
        //  Have we gotten the data on OUR channel?
        if (PacketPID==PidFromSubchannel(Subchannel()))
            if (WorkingPid().MaxSampleIdx > FSampleIdx)
                break;
        }

	return FFileReadOk;
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::InitializeProxy() --
//---------------------------------------------------------------------------

void  VeloPacketFileDataSet::InitializeProxy(VitaPktFileDS::PacketProxy & Proxy, ii64 Position, ii64 SampleIdx)
{
	int PacketHeaderSize = Proxy.Header.SizeInInts();
	int PacketSize       = Proxy.Header.PacketSize();
	int PacketDataSize   = PacketSize - PacketHeaderSize;

	Proxy.UseFile(&File);
	Proxy.Offset            = Position;
	Proxy.Size              = PacketSize;
	Proxy.DataSize          = PacketDataSize;
	Proxy.BaseSampleNumber  = SampleIdx;
	Proxy.Channels          = SubchannelsPerDevice();      // Number of channels per PID
	Proxy.Samples           = PacketDataSize * sizeof(int) / FSubsampleSize / Proxy.Channels;
	Proxy.BytesPerSample    = FSubsampleSize;
	Proxy.Index             = PacketCount();
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::LastParsePosition() --
//---------------------------------------------------------------------------

ii64  VeloPacketFileDataSet::LastParsePosition()
{
	if (FLastArray==0)
        return 0;
    else
        {
        VitaPktFileDS::PacketProxy * LP = FLastArray->LastPacket();
		if (LP)
            return LP->OffsetAfterPacket();
        else
            return 0;
        }
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::LastBodyPosition() --
//---------------------------------------------------------------------------

ii64  VeloPacketFileDataSet::LastBodyPosition()
{
	if (FLastArray==0)
        return 0;
    else
        {
        VitaPktFileDS::PacketProxy * LP = FLastArray->LastPacket();
		if (LP)
			return LP->OffsetAfterHeader();
        else
            return 0;
        }
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageReadIniFile() --
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::ImageReadIniFile(IniFile & /*Ini*/)
{
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::ImageWriteIniFile() --
//---------------------------------------------------------------------------

void VeloPacketFileDataSet::ImageWriteIniFile(IniFile & /*Ini*/)
{
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::PidFromSubchannel() --
//---------------------------------------------------------------------------

int  VeloPacketFileDataSet::PidFromSubchannel(int channel)
{
    //  Channels are assigned to PIDs in order of PID.
    //    if no data is found in a PID its not counted.
    int NthPid = channel/SubchannelsPerDevice();

    PidPacketMap::iterator  iter;
    for (iter = Pid.begin(); iter != Pid.end(); ++iter)
        {
        int pid = (*iter).first;
        VitaPktFileDS::PidPacketArray & PPA = (*iter).second;
        if (PPA.MaxSampleIdx!=0)
            {
            if (NthPid==0)
                return pid;
            --NthPid;
            }
        }
    //  We didn't find it, so return 0.
    return 0;

}
//---------------------------------------------------------------------------
// VeloPacketFileDataSet::WorkingPid() -- Current PID
//---------------------------------------------------------------------------

VitaPktFileDS::PidPacketArray &  VeloPacketFileDataSet::WorkingPid()
{
    return Pid[PidFromSubchannel(Subchannel())];
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::SubchannelsPerDevice() -- Return number of ch per device
//---------------------------------------------------------------------------

unsigned int VeloPacketFileDataSet::SubchannelsPerDevice()
{
    return std::max<int>(Subchannels()/FDevices, 1);
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::PacketCount() -- Return number of packets in data set
//---------------------------------------------------------------------------

size_t VeloPacketFileDataSet::PacketCount() const
{
	size_t result = 0;

	PidPacketMap::const_iterator  iter;
	for (iter = Pid.begin(); iter != Pid.end(); ++iter)
		result += iter->second.Packets.size();

	return result;
}

//---------------------------------------------------------------------------
// VeloPacketFileDataSet::AlertPacketCount() -- Return number of alert packets
//                                                  in data set
//---------------------------------------------------------------------------

size_t VeloPacketFileDataSet::AlertPacketCount() const
{
	size_t result = Alert_PPA.Packets.size();
    return result;
}

//------------------------------------------------------------------------------
// VeloPacketFileDataSet::FillIncidentList() -- Get list of incidents in range
//------------------------------------------------------------------------------

void  VeloPacketFileDataSet::FillIncidentList(ii64 StartIdx, ii64 EndIdx)
{
	Incidents.clear();

	//  Convert Idx to 'Data Channel Offset'
	ii64 StartOffset = LeastOffsetFromIdx(StartIdx);
	ii64 EndOffset   = MostOffsetFromIdx(EndIdx);

	// Process data packets
	for (PidPacketMap::iterator m = Pid.begin(); m != Pid.end(); ++m)
		{
		for (VitaPktFileDS::PidPacketArray::PacketArray::iterator i = m->second.Packets.begin(); i != m->second.Packets.end(); ++i)
			{
			VitaPktFileDS::PacketProxy & Packet = *i;

			if (Packet.Offset < StartOffset)
				continue;
			if (Packet.Offset > EndOffset)
				continue;

			//  In Range, make an Incident from it.
//			Packet.AllocateBody();
			DatasetIncident  Incident;
			Incident.Id = m->first;// % 0x100;

			ii64 divisor = (EndOffset - StartOffset);
			double Fraction = 0.0f;
			if (divisor)
				Fraction = ((double)(Packet.Offset - StartOffset))/divisor;
			ii64 EstimatedSampleIdx = static_cast<ii64>(StartIdx + (EndIdx - StartIdx) * Fraction);

			FillDataIncident(Incident, Packet, EstimatedSampleIdx);
			Incidents.push_back(Incident);
//			Packet.ReleaseBody();
			}
		}

	for (unsigned int i=0; i<Alert_PPA.Packets.size(); i++)
		{
		VitaPktFileDS::PacketProxy & Packet = Alert_PPA.Packets[i];

		if (Packet.Offset < StartOffset)
			continue;
		if (Packet.Offset > EndOffset)
			break;

		//  In Range, make an Incident from it.
//		Packet.AllocateBody();
		DatasetIncident  Incident;

		ii64 divisor = (EndOffset - StartOffset);
		double Fraction = 0.0f;
		if (divisor)
			Fraction = ((double)(Packet.Offset - StartOffset))/divisor;

		ii64 EstimatedSampleIdx = static_cast<ii64>(StartIdx + (EndIdx - StartIdx) * Fraction);

		FillAlertIncident(Incident, Packet, EstimatedSampleIdx);
		Incidents.push_back(Incident);
//		Packet.ReleaseBody();
		}

	SortIncidentList();
}

//------------------------------------------------------------------------------
// VeloPacketFileDataSet::LeastOffsetFromIdx() -- Find smallest offset on all data chans
//------------------------------------------------------------------------------

ii64  VeloPacketFileDataSet::LeastOffsetFromIdx(ii64 Idx)
{
	//  Convert Idx to 'File Offset' keeping smallest one
	ii64 FSampleIdxSave = FSampleIdx;

	ii64 FinalOffset = VitaPktFileDS::MaximumIndex64;

	for (PidPacketMap::iterator i = Pid.begin(); i != Pid.end(); ++i)
		{
		VitaPktFileDS::PidPacketArray & currentPid = i->second;
		FSampleIdx = Idx;

		// make sure we have these packets in memory
		ParseDataFile();

		currentPid.FindProxyForSample(Idx);
		if (currentPid.IsFoundProxyValid()==false)    // if we found no proxy, we can't use it
			continue;

		VitaPktFileDS::PacketProxy & Proxy(currentPid.FoundProxy());

		if (Proxy.Offset < FinalOffset)
			FinalOffset = Proxy.Offset;
		}

	FSampleIdx = FSampleIdxSave;
	return FinalOffset;
}

//------------------------------------------------------------------------------
// VeloPacketFileDataSet::MostOffsetFromIdx() -- Find largest offset on all data chans
//------------------------------------------------------------------------------

ii64  VeloPacketFileDataSet::MostOffsetFromIdx(ii64 Idx)
{
	//  Convert Idx to 'File Offset' keeping largest one
	ii64 FSampleIdxSave = FSampleIdx;

	ii64 FinalOffset = 0;

	for (PidPacketMap::iterator i = Pid.begin(); i != Pid.end(); ++i)
		{
		VitaPktFileDS::PidPacketArray & currentPid = i->second;
		FSampleIdx = Idx;

		ParseDataFile();       // make sure we have these packets in memory

		currentPid.FindProxyForSample(Idx);
		if (currentPid.IsFoundProxyValid()==false)    // if we found no proxy, we can't use it
			continue;

		VitaPktFileDS::PacketProxy & Proxy(currentPid.FoundProxy());

		if (Proxy.Offset > FinalOffset)
			FinalOffset = Proxy.Offset;
		}

	FSampleIdx = FSampleIdxSave;
	return FinalOffset;
}

//------------------------------------------------------------------------------
// VeloPacketFileDataSet::FillAlertIncident() -- Fill incident data
//------------------------------------------------------------------------------

void  VeloPacketFileDataSet::FillAlertIncident(DatasetIncident & incident, const VitaPktFileDS::PacketProxy & Alert, ii64 EstimatedSampleIdx)
{
	//  Figure out alert #
	int AlertNum(0);
	for (int i=0; i<32; i++)
		if (Alert.Body.Data[0] == (1u<<i))
			{
			AlertNum = i;
			break;
			}

	incident.Id = AlertNum;
	for (int i = 0; i < 30; ++i)
		incident.Raw.push_back(Alert.Body.Data[2 + i]);
	incident.Argument = Alert.Body.Data[2 + AlertNum];
	incident.Seconds = Alert.Body.Data[1];  // Alert timestamp field
	incident.Offset = Alert.Offset;
	int devices = Devices();
	int es = EventSize();
	if (FOwner)
		es = FOwner->EventSize();//NominalPacketSize();
	incident.Origin = incident.Offset*devices/es;
	incident.RangedIncident = false;
	incident.Sample = EstimatedSampleIdx;
	incident.Type = DatasetIncident::Alert;
	incident.Index = Alert.Index;
}

//------------------------------------------------------------------------------
// VeloPacketFileDataSet::FillDataIncident() -- Fill incident data
//------------------------------------------------------------------------------

void  VeloPacketFileDataSet::FillDataIncident(DatasetIncident & incident,
	const VitaPktFileDS::PacketProxy & Packet, ii64 EstimatedSampleIdx)
{
	incident.Seconds = 0;
    for (size_t i = 0; i < Packet.Header.Data.size(); ++i)
		incident.Raw.push_back(Packet.Header.Data[i]);
	incident.Offset = Packet.Offset;
	int slip = Packet.Header.SizeInBytes() * static_cast<unsigned int>(Packet.Index);
	incident.Origin = ((incident.Offset-slip)*sizeof(int))/EventSize();
	if (FOwner)
		{
		incident.Padding = FOwner->PaddingSize(incident.Offset);
		int padding = static_cast<unsigned int>(incident.Padding)*sizeof(int);
		incident.Offset -= slip + padding;
		ii64 offset = incident.Offset/sizeof(int);
		int subpackets = static_cast<int>(offset) / FOwner->NominalPacketSize();
		int overhead = subpackets*FOwner->NominalHeaderSize();
		incident.Origin = (offset-overhead) *sizeof(int)/FOwner->EventSize();;
		}
	incident.RangedIncident = false;
	incident.Sample = EstimatedSampleIdx;
	incident.Type = DatasetIncident::Velo;
	incident.Size = Packet.Header.PacketSize();
	incident.Id = Packet.Header.PeripheralId() ;
	incident.Index = Packet.Index;
}


namespace VitaPktFileDS
{
//===========================================================================
//  Class VitaPacketProxy
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VitaPacketProxy
//---------------------------------------------------------------------------

VitaPacketProxy::VitaPacketProxy()
	:  Offset(0), Size(0), DataSize(0), BaseSampleNumber(0),
       Samples(0), Channels(0), BytesPerSample(0), Index(0), PVeloDS(0)
{
}

//---------------------------------------------------------------------------
//  constructor for class VitaPacketProxy - build from sample idx
//---------------------------------------------------------------------------
//  Used for scanning array for proxy containing sample

VitaPacketProxy::VitaPacketProxy(ii64 sample_idx)
	:  Offset(0), Size(0), DataSize(0), BaseSampleNumber(sample_idx),
	   Samples(0), Channels(0), BytesPerSample(0), Index(0), PVeloDS(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class VitaPacketProxy
//---------------------------------------------------------------------------

VitaPacketProxy::~VitaPacketProxy()
{
}

//---------------------------------------------------------------------------
//  VitaPacketProxy::FillHeaderAndTrailer()
//---------------------------------------------------------------------------

bool  VitaPacketProxy::FillHeaderAndTrailer(VeloPacketFileDataSet & VeloDS, ii64 offset)
{
	//  Save Velo pointer & state data
	PVeloDS = &VeloDS;
	Offset = offset;

	//  Copy Header
	for (unsigned int i=0; i<Header.HeaderSize(); i++)
		Header.Data[i] = VeloDS.Element(Offset+i).Int();

	if (VeloDS.IsFileReadOk() == false)
        return false;

    Size       = Header.PacketSize();
	DataSize   = Header.PacketDataSize();

    if (!Size || !DataSize)
        return false;

    //  Copy Trailer
    Header.Data[Header.HeaderSize()] = VeloDS.Element(OffsetAfterPacket()-1).Int();

	return true;
}

//---------------------------------------------------------------------------
//  VitaPacketProxy::AllocateBody() -- Fill proxy with body data from file
//---------------------------------------------------------------------------

void VitaPacketProxy::AllocateBody()
{
    if (!Body.IsAllocated())
        {
		//  Allocate
        Body.Allocate(DataSize);
        //  Fill
        bool state = PVeloDS->BigEndian();
        PVeloDS->BigEndian(false);
        for (size_t i=0; i<DataSize; i++)
			{
			Body.Data[i] = PVeloDS->Element(OffsetAfterHeader() + i).Int();
            }
        PVeloDS->BigEndian(state);
        }
}

//---------------------------------------------------------------------------
//  VitaPacketProxy::ReleaseBody() -- empty data from proxy
//---------------------------------------------------------------------------

void VitaPacketProxy::ReleaseBody()
{
    if (Body.IsAllocated())
        {
        Body.Release();
        }
}


//===========================================================================
//  Class SidPacketArray
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class SidPacketArray
//---------------------------------------------------------------------------

SidPacketArray::SidPacketArray()
	:  MaxSampleIdx(0), CacheSizeBlocks(500), CurrentProxyIdx(MaximumIndex)
{
}

//---------------------------------------------------------------------------
//  destructor for class SidPacketArray
//---------------------------------------------------------------------------

SidPacketArray::~SidPacketArray()
{
}

//---------------------------------------------------------------------------
//  SidPacketArray::LastPacket() -- Return last packet in the array
//---------------------------------------------------------------------------

VitaPacketProxy *   SidPacketArray::LastPacket()
{
    if (Packets.size()==0)
        return 0;
    else
        {
        size_t idx = Packets.size()-1;
        return &(Packets[idx]);
		}
}

//---------------------------------------------------------------------------
//  SidPacketArray::CurrentSample() -- Read sample
//---------------------------------------------------------------------------

Subsample SidPacketArray::CurrentSample(int channel_in_sid, ii64 index)
{
    Subsample value(0);

    FindProxyForSample(index);

    if (CurrentProxyIdx != MaximumIndex)
        {
        VitaPacketProxy & CurrentProxy(Packets[CurrentProxyIdx]);
        //
        //  Manage Proxy Data Allocation
		AllocateProxy();
        //
        //  Fetch the data point
		switch (CurrentProxy.BytesPerSample)
            {
            case 4:
				{
                unsigned int * BodyPtr = CurrentProxy.Body.Data;
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_sid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 4);
                }
                break;
            case 2:
				{
                unsigned short * BodyPtr = reinterpret_cast<unsigned short *>(CurrentProxy.Body.Data);
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_sid;
				memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 2);
                }
                break;
            case 1:
                {
                unsigned char * BodyPtr = reinterpret_cast<unsigned char *>(CurrentProxy.Body.Data);
                unsigned int Idx = (unsigned int)(index - CurrentProxy.BaseSampleNumber);
                unsigned int Chan = channel_in_sid;
                memcpy(value.asString(), &BodyPtr[CurrentProxy.Channels*Idx + Chan], 1);
                }
                break;
            }
		}

	return value;
}

//------------------------------------------------------------------------------
//  SidPacketArray::FindProxyForSample() -- Find proxy that holds sample idx
//------------------------------------------------------------------------------

void  SidPacketArray::FindProxyForSample(ii64 index)
{
	if (Packets.size()==0)
		{
		CurrentProxyIdx = MaximumIndex;
		return;
		}
	//  Check if current block has the sample
	if ((CurrentProxyIdx != MaximumIndex ) && Packets[CurrentProxyIdx].PacketContainsSample(index))
		return;

	// Scan our array for the block that does contain the sample...
	VitaPacketProxy candidate(index);
	PacketArray::iterator iter = std::lower_bound(Packets.begin(), Packets.end(), candidate);
	if (iter != Packets.end())
        {
        CurrentProxyIdx = iter - Packets.begin();
        }
    else
        {
		CurrentProxyIdx = Packets.end() - Packets.begin() - 1;
//		CurrentProxyIdx = MaximumIndex;     // should never happen
        }
}

//------------------------------------------------------------------------------
//  SidPacketArray::AllocateProxy() -- Manage cache allocation of proxy
//------------------------------------------------------------------------------

void  SidPacketArray::AllocateProxy()
{
    VitaPacketProxy & CurrentProxy(Packets[CurrentProxyIdx]);
    //
    //  CurrentProxy may not have been allocated yet.
    if (CurrentProxy.Body.IsAllocated()==false)
        {
        CurrentProxy.AllocateBody();
        //
        //  Push IDX of proxy onto cache
        Cache.push_front(CurrentProxyIdx);
        //  if too many blocks in cache, free one
		if (Cache.size() > CacheSizeBlocks)
            {
            size_t Last = Cache.back();
			//  Release from cache
            Cache.pop_back();
            //  Release data block of packet
            Packets[Last].ReleaseBody();
            }
        }

}

//------------------------------------------------------------------------------
//  SidPacketArray::FoundProxy() -- View found Proxy
//------------------------------------------------------------------------------

VitaPacketProxy &  SidPacketArray::FoundProxy()
{
    return Packets[CurrentProxyIdx];
}

//------------------------------------------------------------------------------
//  SidPacketArray::IsFoundProxyValid() -- is proxy valid?
//------------------------------------------------------------------------------

bool  SidPacketArray::IsFoundProxyValid()
{
    return CurrentProxyIdx != MaximumIndex;
}

} //  end namespace VitaPktFileDS

//===========================================================================
//  Class VitaPacketFileDataSet
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VitaPacketFileDataSet
//---------------------------------------------------------------------------

const int kVitaPacketHeaderTrailerSize = 8;
const int kTrailerId = 0xFFFF;

//---------------------------------------------------------------------------
//  constructor for class VitaPacketFileDataSet
//---------------------------------------------------------------------------

VitaPacketFileDataSet::VitaPacketFileDataSet()
	: FSampleIdx(0), FImaginary(false), FLastArray(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class VitaPacketFileDataSet
//---------------------------------------------------------------------------

VitaPacketFileDataSet::~VitaPacketFileDataSet()
{
    ImageClose();
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageInit() -- On Image Initialization
//---------------------------------------------------------------------------

void  VitaPacketFileDataSet::ImageInit()
{
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageOpen()
//---------------------------------------------------------------------------

bool  VitaPacketFileDataSet::ImageOpen()
{
	ScopedTrace("VitaPacketFileDataSet::ImageOpen");

	//  Transfer settings to VeloDS
	VeloDS.Owner(this);
	VeloDS.FileName(FFileName);
	VeloDS.Devices(Devices());
	VeloDS.Bits(32);
	VeloDS.Type(VeloPacketFileDataSet::Int);
	VeloDS.BigEndian(BigEndian());
	VeloDS.OnProgress = OnProgress;

	VeloDS.Enabled(true);

	// use this to pre-fetch packets for each SID
	bool ForceDataScan(true);

	int FChannelSave = FChannel;
	int testChan = 0;

	for (int i=0; i<FDevices; i++)
		{
		//  Find sample 0 on ALL devices
		FChannel = testChan;
		ParseDataFile(ForceDataScan);
		testChan += SubchannelsPerDevice();
		}
	FChannel = FChannelSave;  // restore original channel.

	Init();

	return true;
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageClose()
//---------------------------------------------------------------------------

void  VitaPacketFileDataSet::ImageClose()
{
//    ScopedTrace("VitaPacketFileDataSet::ImageClose");

    //  Clean out Sid parser map
    {
    SidPacketMap  eraser;
    eraser.swap(SID);
    }

	//  Clean out parse limit array
    {
    std::vector<ii64>  eraser;
    eraser.swap(FParseLimitIdx);
    }

    FLastArray = 0;
    FImaginary = false;
    FSampleIdx = 0;

    VeloDS.Enabled(false);
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::PreAccess() -- Init prior to sample access
//---------------------------------------------------------------------------

void VitaPacketFileDataSet::PreAccess()
{
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::PostAccess() -- Finalize after sample access
//---------------------------------------------------------------------------

void VitaPacketFileDataSet::PostAccess()
{
	++FSampleIdx;
	Progress(FSampleIdx);
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageSize() -- Set size of disk file
//---------------------------------------------------------------------------

void VitaPacketFileDataSet::ImageSize(ii64 /*size*/ )
{
    // not supported
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageSampleFetch() -- Fetch a sample
//---------------------------------------------------------------------------

void  VitaPacketFileDataSet::ImageSampleFetch(Subsample &sample)
{
    if (!VeloDS.Enabled())
        return;
	//
	//  If we havent parsed this far, we will now!
    ParseDataFile();

    int subchannel = Subchannel()%SubchannelsPerDevice();
    sample = WorkingSid().CurrentSample(subchannel, FSampleIdx);
    char * pSample = sample.asString();
    CorrectEndian(pSample);
    sample.Type(static_cast<enum Subsample::SampleType>(FType));

    FImaginary = !FImaginary;
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageSampleStore() -- Store a sample
//---------------------------------------------------------------------------

void VitaPacketFileDataSet::ImageSampleStore(Subsample & /*sample*/)
{
	// not supported
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageFlush() -- Flush to disk
//---------------------------------------------------------------------------

void VitaPacketFileDataSet::ImageFlush()
{
	// not supported
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageSeek() -- Find a vita sample offset
//---------------------------------------------------------------------------

void VitaPacketFileDataSet::ImageSeek(ii64 offset)
{
	FSampleIdx = offset/FEventSize;
	Progress(FSampleIdx);
}

//---------------------------------------------------------------------------
//  VitaPacketFileDataSet::ImageSize() -- return the size of the image data
//---------------------------------------------------------------------------

ii64 VitaPacketFileDataSet::ImageSize() const
{
//    ScopedTrace("VitaPacketFileDataSet::ImageSize");

	return VeloDS.ImageSize() - (PacketCount()*kVitaPacketHeaderTrailerSize*sizeof(int));
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::ParseDataFile() --
//---------------------------------------------------------------------------

bool  VitaPacketFileDataSet::ParseDataFile(bool force_scan)
{
	bool result = true;

//    ScopedTrace("VitaPacketFileDataSet::ParseDataFile");

    if (!VeloDS.Enabled())
        return false;
    //
    //  First lets check if this sample has already been parsed.
    if (!force_scan)
        {
        ii64 idx = WorkingSid().MaxSampleIdx;
        if (idx > FSampleIdx)
			return result;
		}

    // Otherwise Parse the file until we find the channel's sample
    //   Position at last parsed position
	ii64  Position = LastParsePosition();       // take outa loop - redundant
	VeloDS.Seek(Position);

	while (!Abort())
		{
		//
		//  Extract the next packet header
		VitaPktFileDS::VitaPacketProxy Proxy;
		//
		//  Read the header & trailer
		bool readOK = Proxy.FillHeaderAndTrailer(VeloDS, Position) &&
			(Position <= VeloDS.ImageSize());

		if (!readOK)
			{
			result = false;
			break;
			}
		//
		//  Find out which SPA this packet will be added to..
		int PacketSID = Proxy.Header.StreamId();

		VitaPktFileDS::SidPacketArray & SPA = SID[ PacketSID ];
		if (PacketSID != kTrailerId)
			FLastArray = &SPA;

        VitaPktFileDS::SidPacketArray::PacketArray & SPV = SPA.Packets;
        //  Fill in the rest
		InitializeProxy(Proxy, Position, SPA.MaxSampleIdx);
		//
		//  Add proxy to our array
		SPA.MaxSampleIdx  += Proxy.Samples;
		SPV.push_back(Proxy);

		// Skip to the next packet
		Position += Proxy.Header.PacketSize();
		//
		//  Have we gotten the data on OUR channel?
		if (PacketSID==SidFromSubchannel(Subchannel()))
			if (WorkingSid().MaxSampleIdx > FSampleIdx)
				break;
		}

	return result;
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::InitializeProxy() --
//---------------------------------------------------------------------------

void  VitaPacketFileDataSet::InitializeProxy(VitaPktFileDS::VitaPacketProxy & Proxy,
    ii64 /*Position*/, ii64 SampleIdx)
{
//	Proxy.Offset            = Position;
//	Proxy.Size              = Proxy.Header.PacketSize();
//	Proxy.DataSize   		= Proxy.Header.PacketDataSize();
	Proxy.BaseSampleNumber  = SampleIdx;
	Proxy.Channels          = SubchannelsPerDevice();      // Number of channels per PID
	Proxy.Samples           = Proxy.Header.ActualDataSizeInBytes() / FSubsampleSize / Proxy.Channels ;
	Proxy.BytesPerSample    = FSubsampleSize;
	Proxy.Index             = PacketCount();
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::WorkingSid() -- Current SID
//---------------------------------------------------------------------------

VitaPktFileDS::SidPacketArray &  VitaPacketFileDataSet::WorkingSid()
{
	return SID[SidFromSubchannel(Subchannel())];
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::LastParsePosition() --
//---------------------------------------------------------------------------

ii64  VitaPacketFileDataSet::LastParsePosition()
{
	if (FLastArray==0)
		return 0;
	else
		{
		VitaPktFileDS::VitaPacketProxy * LP = FLastArray->LastPacket();
		if (LP)
			{
			return LP->OffsetAfterPacket();
			}
		else
			return 0;
		}
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::NominalPacketSize() --
//---------------------------------------------------------------------------

int  VitaPacketFileDataSet::NominalPacketSize() const
{
	if (FLastArray==0)
		return 0;
	else
		{
		VitaPktFileDS::VitaPacketProxy * LP = FLastArray->LastPacket();
		return LP->Size;
		}
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::NominalHeaderSize() --
//---------------------------------------------------------------------------

int  VitaPacketFileDataSet::NominalHeaderSize() const
{
	if (FLastArray==0)
		return 0;
	else
		{
		VitaPktFileDS::VitaPacketProxy * LP = FLastArray->LastPacket();
		return LP->Header.PacketSize()-LP->Header.PacketDataSize();
		}
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::PaddingSize() --
//---------------------------------------------------------------------------

ii64 VitaPacketFileDataSet::PaddingSize(ii64 idx) const
{
	if (!SID.count(VitaPacketHeader::kPaddingSid))
		return 0;

	VitaPktFileDS::SidPacketArray & pa = SID[VitaPacketHeader::kPaddingSid];

	size_t result = 0;

	for (VitaPktFileDS::SidPacketArray::PacketArray::iterator i = pa.Packets.begin();
		i != pa.Packets.end(); ++i)
		{
		if (i->Offset > idx)
			break;

		result += i->Size;
		}

	return result;
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::SidFromSubchannel() --
//---------------------------------------------------------------------------

int  VitaPacketFileDataSet::SidFromSubchannel(int channel)
{
	//  Channels are assigned to SIDs in order of SID.
	//    if no data is found in a SID its not counted.
	int NthSid = channel/SubchannelsPerDevice();

	SidPacketMap::iterator  iter;
	for (iter = SID.begin(); iter != SID.end(); ++iter)
		{
        int SID = (*iter).first;
        VitaPktFileDS::SidPacketArray & SPA = (*iter).second;
		if (SPA.MaxSampleIdx!=0)
            {
            if (NthSid==0)
                return SID;
            --NthSid;
			}
        }
    //  We didn't find it, so return 0.
	return 0;

}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::Subchannel() -- Return current subchannel
//---------------------------------------------------------------------------

int VitaPacketFileDataSet::Subchannel() const
{
	int ch;

    if (FComplex)
        ch = FChannel * Subsamples() + (FImaginary ? 1 : 0);
	else
        ch = FChannel;

    return ch;
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::SubchannelsPerDevice() -- Return number of ch per device
//---------------------------------------------------------------------------

unsigned int VitaPacketFileDataSet::SubchannelsPerDevice()
{
    return std::max<int>(Subchannels()/FDevices, 1);
}

//---------------------------------------------------------------------------
// VitaPacketFileDataSet::PacketCount() -- Return number of packets in data set
//---------------------------------------------------------------------------

size_t VitaPacketFileDataSet::PacketCount() const
{
	size_t result = 0;

	SidPacketMap::const_iterator  iter;
	for (iter = SID.begin(); iter != SID.end(); ++iter)
		result += iter->second.Packets.size();

	return result;
}

//------------------------------------------------------------------------------
// VitaPacketFileDataSet::FillIncidentList() -- Get list of incidents in range
//------------------------------------------------------------------------------

void  VitaPacketFileDataSet::FillIncidentList(ii64 StartIdx, ii64 EndIdx)
{
//    ScopedTrace("VitaPacketFileDataSet::FillIncidentList");
	Incidents.clear();

	//  Convert Idx to 'Velo Dataset Offset'
	ii64 StartOffset = LeastOffsetFromIdx(StartIdx);
	ii64 EndOffset   = MostOffsetFromIdx(EndIdx);
	if ((EndOffset-StartOffset) == 0)
		++EndOffset;

	VeloDS.FillIncidentList(StartOffset, EndOffset);

	// Process Velo data packets
	for (DatasetIncidentList::iterator ii = VeloDS.Incidents.begin();
		ii != VeloDS.Incidents.end(); ++ii)
		{
		Incidents.push_back(*ii);

		//  Parse Vita Incidents from within Velo Incidents
		for (SidPacketMap::iterator m = SID.begin(); m != SID.end(); ++m)
			{
			for (VitaPktFileDS::SidPacketArray::PacketArray::iterator i = m->second.Packets.begin();
				i != m->second.Packets.end(); ++i)
				{
				VitaPktFileDS::VitaPacketProxy & Packet = *i;
				if (Packet.Offset < StartOffset)
					continue;
				if (Packet.Offset > EndOffset)
					continue;

				//  In Range, make an Incident from it.
//				Packet.AllocateBody();
				DatasetIncident  Incident;
				Incident.Id = m->first;

				ii64 divisor = (EndOffset - StartOffset);
				double Fraction = 0.0f;
				if (divisor)
					Fraction = ((double)(Packet.Offset - StartOffset))/divisor;
				ii64 EstimatedSampleIdx = static_cast<ii64>(StartIdx + (EndIdx - StartIdx) * Fraction);

				FillDataIncident(Incident, Packet, EstimatedSampleIdx);
//				Packet.ReleaseBody();

				DatasetIncidentList::const_iterator j = find(Incidents.begin(), Incidents.end(), Incident);
				if (j == Incidents.end())
					Incidents.push_back(Incident);
				}
			}
		}

	SortIncidentList();
}

//------------------------------------------------------------------------------
// VitaPacketFileDataSet::LeastOffsetFromIdx() -- Find smallest offset on all data chans
//------------------------------------------------------------------------------

ii64  VitaPacketFileDataSet::LeastOffsetFromIdx(ii64 Idx)
{
	//  Convert Idx to 'Velo Offset' keeping smallest one
	ii64 FSampleIdxSave = FSampleIdx;

	ii64 FinalOffset = VitaPktFileDS::MaximumIndex64;

	for (SidPacketMap::iterator i = SID.begin(); i != SID.end(); ++i)
		{
		VitaPktFileDS::SidPacketArray & currentSid = i->second;
		FSampleIdx = Idx;

		// make sure we have these packets in memory
		if (!ParseDataFile())
            {
            ;
            }

		currentSid.FindProxyForSample(Idx);
		if (currentSid.IsFoundProxyValid()==false)    // if we found no proxy, we can't use it
			continue;

		VitaPktFileDS::VitaPacketProxy & Proxy(currentSid.FoundProxy());

		if (Proxy.Offset < FinalOffset)
			FinalOffset = Proxy.Offset;
		}

	FSampleIdx = FSampleIdxSave;
	Progress(FSampleIdx);

	return FinalOffset;
}

//------------------------------------------------------------------------------
// VitaPacketFileDataSet::MostOffsetFromIdx() -- Find largest offset on all data chans
//------------------------------------------------------------------------------

ii64  VitaPacketFileDataSet::MostOffsetFromIdx(ii64 Idx)
{
	//  Convert Idx to 'Velo Offset' keeping largest one
	ii64 FSampleIdxSave = FSampleIdx;

	ii64 FinalOffset = 0;

	for (SidPacketMap::iterator i = SID.begin(); i != SID.end(); ++i)
		{
		int id = i->first;
		if (id == kTrailerId)
			continue;

		VitaPktFileDS::SidPacketArray & currentSid = i->second;
		FSampleIdx = Idx;

		// make sure we have these packets in memory
		ParseDataFile();       // make sure we have these packets in memory

		currentSid.FindProxyForSample(Idx);
		if (currentSid.IsFoundProxyValid()==false)    // if we found no proxy, we can't use it
			continue;

		VitaPktFileDS::VitaPacketProxy & Proxy(currentSid.FoundProxy());

		if (Proxy.Offset > FinalOffset)
			FinalOffset = Proxy.Offset;
		}

	FSampleIdx = FSampleIdxSave;
	Progress(FSampleIdx);

	return FinalOffset;
}

//------------------------------------------------------------------------------
// VitaPacketFileDataSet::FillDataIncident() -- Fill incident data
//------------------------------------------------------------------------------

void  VitaPacketFileDataSet::FillDataIncident(DatasetIncident & incident,
	const VitaPktFileDS::VitaPacketProxy & Packet, ii64 EstimatedSampleIdx)
{
	incident.Seconds = 0;
	incident.Offset = Packet.Offset;
	incident.Origin = Packet.BaseSampleNumber;
    for (size_t i = 0; i < Packet.Header.Data.size(); ++i)
		incident.Raw.push_back(Packet.Header.Data[i]);
	incident.Seconds = Packet.Header.Seconds();
	incident.Subseconds = Packet.Header.SubSeconds();
	incident.RangedIncident = false;
	incident.Sample = EstimatedSampleIdx;
	incident.Type = DatasetIncident::Vita;
	incident.Size = Packet.Header.PacketSize();
	incident.Id = Packet.Header.StreamId() ;
	incident.Index = Packet.Index;
}

} // namespace Innovative


