// VitaPacketFileDataSet_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#ifndef VitaPacketFileDataSet_MbH
#define VitaPacketFileDataSet_MbH

#include <PacketFileDataSet_Mb.h>
#include <DriveIo_Mb.h>
#include <map>
#include <vector>
#include <list>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class VeloPacketFileDataSet;

namespace VitaPktFileDS
{
//===========================================================================
// Class VeloPacketHeader
//===========================================================================

class VeloPacketHeader
{
public:
	// Ctor
	VeloPacketHeader()
		: Data(4, 0)
		{  }

	// Methods
	unsigned int        PacketSize() const
		{  return Data[0] & 0xffffff;  }
	unsigned int        PeripheralId() const
		{  return Data[0] >> 24;  }
	unsigned int *      Raw()
		{  return &Data[0];  }

	unsigned int        SizeInBytes() const
		{  return SizeInInts()*sizeof(int);  }
	unsigned int        SizeInInts() const
	{
		return static_cast<unsigned int>(Data.size());
	}

	// Data
	std::vector<unsigned int>	Data;
};

//===========================================================================
// Class PacketProxy -- Information
//===========================================================================

class PacketProxy
{
public:
	PacketProxy();
	PacketProxy(ii64 sample_idx);
	~PacketProxy();

	ii64   OffsetAfterHeader()
		{  return Offset + Header.SizeInBytes();  };
	ii64   OffsetAfterPacket()
		{  return Offset + Size*sizeof(int);  };
	void      UseFile(PacketFile * file)
		{  PFile = file;  };

	bool  operator < (const PacketProxy & rhs) const
		{  return LessThanEnd(rhs);  }
	//
	//  Search predicates
	bool PacketContainsSample(ii64 Sample) const
		{  return Sample >= BaseSampleNumber && BaseSampleNumber+Samples > Sample;  }
	bool LessThanStart(const PacketProxy & rhs) const
		{  return BaseSampleNumber < rhs.BaseSampleNumber;  }
	bool LessThanEnd(const PacketProxy & rhs) const
		{  return BaseSampleNumber+Samples-1 < rhs.BaseSampleNumber;  }

public:
	ii64             	Offset;            // Offset within image to start of packet
	unsigned int        Size;              // Number of ints in packet
	unsigned int        DataSize;          // Number of ints in data portion of packet
	ii64             	BaseSampleNumber;  // Sample # of first sample in packet
	ii64             	Samples;           // # of samples PER CHANNEL
	int                 Channels;          // # of data channels in packet
	int                 BytesPerSample;    // Width of samples
	VeloPacketHeader    Header;
	PacketBlock         Body;
	size_t              Index;

	void                AllocateBody();
	void                ReleaseBody();

private:
	PacketFile *   PFile;
};

//===========================================================================
// Class PidPacketArray
//===========================================================================

class PidPacketArray
{
public:
	PidPacketArray();
	~PidPacketArray();

	Subsample   CurrentSample(int channel_in_pid, ii64 index);
	void       	CurrentSample(int channel_in_pid, ii64 index, Subsample & value);

	void  Clear();

	PacketProxy *  LastPacket();

public:
	ii64             			MaxSampleIdx;
	const size_t				CacheSizeBlocks;
	size_t						CurrentProxyIdx;

	typedef std::vector<PacketProxy>  PacketArray;
	typedef std::list<size_t>  CacheList;

	PacketArray    Packets;
	CacheList      Cache;

	void  FindProxyForSample(ii64 index);
	void  AllocateProxy();

	PacketProxy &  FoundProxy();
	bool  IsFoundProxyValid();
};

}  // namespace VitaPktFileDS

//===========================================================================
// Class VeloPacketFileDataSet
//===========================================================================

class VeloPacketFileDataSet : public DataSet
{
public:
	friend class VitaPacketFileDataSet;
	friend class VitaPacketProxy;

	VeloPacketFileDataSet();
	virtual ~VeloPacketFileDataSet();

	// Properties
	void  AutoCreate(bool value)
		{  FAutoCreate=value;  }
	bool  AutoCreate() const
		{  return FAutoCreate;  }

	bool  IsFileReadOk() const
		{  return FFileReadOk;  }

	void  FillIncidentList(ii64 StartIdx, ii64 EndIdx);

	void    Owner(class VitaPacketFileDataSet * owner)
		{  FOwner = owner;   }

	ii64              LastBodyPosition();

protected:
	// Overridden Methods
	virtual void        ImageInit();
	virtual void        ImageSampleFetch(Subsample &sample);
	virtual void        ImageSampleStore(Subsample &sample);
	virtual void        PreAccess();
	virtual void        PostAccess();
	// Override Methods
	virtual bool        ImageOpen();
	virtual void        ImageClose();
	virtual void        ImageSeek(ii64 offset);
	virtual ii64        ImageSize() const;
	virtual void        ImageSize(ii64 size);
	virtual void        ImageFlush();

private:
	typedef std::map<int, VitaPktFileDS::PidPacketArray> PidPacketMap;

	// Data
	class VitaPacketFileDataSet *     FOwner;
	PacketFile                        File;
	PidPacketMap                      Pid;
	VitaPktFileDS::PidPacketArray     Alert_PPA;
	ii64                              FSampleIdx;
	bool                              FImaginary;
	bool                              FFileReadOk;
	std::vector<ii64>                 FParseLimitIdx;
	VitaPktFileDS::PidPacketArray *   FLastArray;

	// Fields
	bool                    FAutoCreate;        // Auto-create files on open fail

	// Methods
	virtual void        ImageReadIniFile(IniFile &Ini);
	virtual void        ImageWriteIniFile(IniFile &Ini);

	VitaPktFileDS::PidPacketArray &  WorkingPid();
	int               PidFromSubchannel(int channel);
	bool              ParseDataFile(bool force_scan = false);
	void              InitializeProxy(VitaPktFileDS::PacketProxy & Proxy, ii64 Position, ii64 SampleIdx);
	ii64              LastParsePosition();
	unsigned int      SubchannelsPerDevice();
	size_t            PacketCount() const;
	size_t            AlertPacketCount() const;
	int               Subchannel() const;

	ii64  LeastOffsetFromIdx(ii64 idx);
	ii64  MostOffsetFromIdx(ii64 idx);
	void  FillAlertIncident(DatasetIncident & incident, const VitaPktFileDS::PacketProxy & Alert, ii64 EstimatedSampleIdx);
	void  FillDataIncident(DatasetIncident & incident, const VitaPktFileDS::PacketProxy & Packet, ii64 EstimatedSampleIdx);
};

//===========================================================================
// Class VitaPacketHeader
//===========================================================================

class VitaPacketHeader
{
public:
	enum SidType { kPaddingSid = 0xffff };

	// Ctor
	VitaPacketHeader()
		: Data(8, 0)
		{   }

	// Methods
	unsigned int        PacketSize() const
		{  return Data[0] & 0xffff;  }
	unsigned int        PacketDataSize() const
		{  return PacketSize() -  SizeInInts();  }
	unsigned int        HeaderSize() const
		{  return SizeInInts() -  TrailerSize();  }
	unsigned int        TrailerSize() const
		{  return 1;  }
	unsigned int        StreamId() const
		{  return Data[1] & 0xffff;  }
	unsigned int        PaddingBytes() const
		{  return (Data[7] & 0x0f00)>>8;  }
	unsigned int *      Raw()
		{  return &Data[0];  }

	unsigned int        ActualDataSizeInBytes() const
		{  return PacketDataSize()*sizeof(int) - PaddingBytes();  }
	unsigned int        SizeInBytes() const
		{  return SizeInInts()*sizeof(int);  }
	unsigned int        SizeInInts() const
		{  return static_cast<unsigned int>(Data.size());  }
	unsigned int        Seconds() const
		{  return Data[4];  }
	ii64				SubSeconds() const
		{
		ii64 result = Data[5];
		result <<= 32;
		result |= Data[6];
		return result;
		}
	bool                IsPadding() const
		{  return StreamId() == kPaddingSid;  }

	// Data
	std::vector<unsigned int>	Data;
};

namespace VitaPktFileDS
{
//===========================================================================
// Class VitaPacketProxy -- Information
//===========================================================================

class VitaPacketProxy
{
public:
	VitaPacketProxy();
    VitaPacketProxy(ii64 sample_idx);
    ~VitaPacketProxy();

    ii64   OffsetAfterHeader()
        {  return Offset + Header.HeaderSize();  };
    ii64   OffsetAfterPacket()
		{  return Offset + Size;  };

	bool  operator < (const VitaPacketProxy & rhs) const
		{  return LessThanEnd(rhs);  }
    //
	//  Search predicates
    bool PacketContainsSample(ii64 Sample) const
		{  return (Sample >= BaseSampleNumber) && (BaseSampleNumber+Samples > Sample);  }
    bool LessThanStart(const VitaPacketProxy & rhs) const
        {  return BaseSampleNumber < rhs.BaseSampleNumber;  }
    bool LessThanEnd(const VitaPacketProxy & rhs) const
        {  return BaseSampleNumber+Samples-1 < rhs.BaseSampleNumber;  }

	bool  FillHeaderAndTrailer(VeloPacketFileDataSet & VeloDS, ii64 offset);

public:
	ii64             	Offset;            // Offset within image to start of packet
	unsigned int        Size;              // Number of ints in packet
    unsigned int        DataSize;          // Number of ints in data portion of packet
	ii64             	BaseSampleNumber;  // Sample # of first sample in packet
	ii64             	Samples;           // # of samples PER CHANNEL
    int                 Channels;          // # of data channels in packet
    int                 BytesPerSample;    // Width of samples
    VitaPacketHeader    Header;
	PacketBlock         Body;
	size_t              Index;

    void                AllocateBody();
    void                ReleaseBody();

private:
    VeloPacketFileDataSet *   PVeloDS;
};

//===========================================================================
// Class SidPacketArray
//===========================================================================

class SidPacketArray
{
public:
    SidPacketArray();
    ~SidPacketArray();

	Subsample   CurrentSample(int channel_in_sid, ii64 index);
	//void       	CurrentSample(int channel_in_sid, ii64 index, Subsample & value);

	VitaPacketProxy *  LastPacket();

public:
	ii64             	MaxSampleIdx;
	const size_t		CacheSizeBlocks;
	size_t				CurrentProxyIdx;

	typedef std::vector<VitaPacketProxy>  PacketArray;
	typedef std::list<size_t>  CacheList;

	PacketArray    Packets;
	CacheList      Cache;

	void  FindProxyForSample(ii64 index);
    void  AllocateProxy();

    VitaPacketProxy &  FoundProxy();
    bool  IsFoundProxyValid();

};

}  // namespace VitaPktFileDS

//===========================================================================
// Class VitaPacketFileDataSet
//===========================================================================

class VitaPacketFileDataSet : public DataSet
{
public:
	VitaPacketFileDataSet();
	virtual ~VitaPacketFileDataSet();

	virtual void  FillIncidentList(ii64 StartIdx, ii64 EndIdx);

	// Pseudo-protected
	int			  NominalPacketSize() const;
	int			  NominalHeaderSize() const;
	ii64  	  	  PaddingSize(ii64 idx) const;

protected:
	// Overridden Methods
	virtual void        ImageInit();
	virtual void        ImageSampleFetch(Subsample &sample);
	virtual void        ImageSampleStore(Subsample &sample);
	virtual void        PreAccess();
	virtual void        PostAccess();
	// Override Methods
	virtual bool        ImageOpen();
	virtual void        ImageClose();
	virtual void        ImageSeek(ii64 offset);
	virtual ii64        ImageSize() const;
	virtual void        ImageSize(ii64 size);
	virtual void        ImageFlush();

private:
	typedef std::map<int, VitaPktFileDS::SidPacketArray> SidPacketMap;

	VeloPacketFileDataSet   VeloDS;
	mutable SidPacketMap    SID;
	std::vector<ii64>    	FParseLimitIdx;
	ii64                 	FSampleIdx;
	bool                    FImaginary;
	VitaPktFileDS::SidPacketArray *        FLastArray;

	bool            ParseDataFile(bool force_scan = false);
	VitaPktFileDS::SidPacketArray &  WorkingSid();
	ii64           	                 LastParsePosition();
	void          InitializeProxy(VitaPktFileDS::VitaPacketProxy & Proxy, ii64 Position, ii64 SampleIdx);
	int           SidFromSubchannel(int channel);
	int           Subchannel() const;
	unsigned int  SubchannelsPerDevice();
	size_t        PacketCount() const;
	ii64          LeastOffsetFromIdx(ii64 idx);
	ii64          MostOffsetFromIdx(ii64 idx);
	void  		  FillDataIncident(DatasetIncident & incident,
		const VitaPktFileDS::VitaPacketProxy & Packet, ii64 EstimatedSampleIdx);
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif
