//---------------------------------------------------------------------------
// VitaPacketStream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <VitaPacketStream_Mb.h>
#include <SystemSupport_Mb.h>
#include <MemoryUtils_Mb.h>
#include <BufferTrailer_Mb.h>
#include <algorithm>
#include <functional>

namespace Innovative
{
//===========================================================================
//  CLASS VitaStreamRcvrThread  -- Recieve VITA packets to app
//===========================================================================
//---------------------------------------------------------------------------
//  VitaStreamRcvrThread::ProcessPacket() -- Recv data packet
//---------------------------------------------------------------------------

void  VitaStreamRcvrThread::ProcessPacket()
{
//    int depth_check = Queue.Depth();
//    Log(b8, depth_check);
    //
    //  Signal packet available downstream...
    Signal.Release();

    Owner->SignalVitaDataAvailable();

//    depth_check = Queue.Depth();
//    Log(b9, depth_check);
}

//---------------------------------------------------------------------------
//  VitaStreamRcvrThread::Recv() -- Return packet in data queue
//---------------------------------------------------------------------------

void  VitaStreamRcvrThread::Recv( VitaBuffer & packet )
{
    int depth_check = static_cast<int>(Queue.Depth());
    Log(0xc8, depth_check);
    //
    //  Wait for packet to arrive....
    Signal.Acquire();
    //
    //  Extract packet from queue
    packet = Queue.Front();
    //
    //  Pop the queue
    Queue.Pop();

    depth_check = static_cast<int>(Queue.Depth());
    Log(0xc9, depth_check);
}

//---------------------------------------------------------------------------
//  VitaStreamRcvrThread::Log() -- Trace Log Data
//---------------------------------------------------------------------------

void  VitaStreamRcvrThread::Log(int tag, int value)
{
    Owner->DeviceInternals()->TraceLog(tag, value);
}

//---------------------------------------------------------------------------
//  VitaStreamRcvrThread::Log() -- Trace Log Data
//---------------------------------------------------------------------------

void  VitaStreamRcvrThread::Log(int tag, size_t value)
{
    Log(tag, static_cast<int>(value));
}

//===========================================================================
//  CLASS VitaStreamXmitThread  --  Signal VITA packets needed to app
//===========================================================================
//---------------------------------------------------------------------------
//  VitaStreamXmitThread::ProcessPacket() -- Recv data packet
//---------------------------------------------------------------------------

void  VitaStreamXmitThread::ProcessPacket()
{
    Log(0xb8, 0);
    //
    //  Request actual packet
    Owner->SignalVitaDataRequired();
}

//---------------------------------------------------------------------------
//  VitaStreamXmitThread::SignalDataRequired() -- More Data Needed
//---------------------------------------------------------------------------

void  VitaStreamXmitThread::SignalDataRequired()
{
    Log(0xb9, 0);
    //
    //  Signal for more data
    Signal.Release();
}

//---------------------------------------------------------------------------
//  VitaStreamXmitThread::Log() -- Trace Log Data
//---------------------------------------------------------------------------

void  VitaStreamXmitThread::Log(int tag, int value)
{
    Owner->DeviceInternals()->TraceLog(tag, value);
}

//---------------------------------------------------------------------------
//  VitaStreamXmitThread::Log() -- Trace Log Data
//---------------------------------------------------------------------------

void  VitaStreamXmitThread::Log(int tag, size_t value)
{
    Log(tag, static_cast<int>(value));
}

//===========================================================================
// CLASS VitaPacketStreamDeviceSafeLink  --  Safe Linkage to a device
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VitaPacketStream
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  VitaPacketStreamDeviceSafeLink::OnAfterLink() - handle post-link stuff
//---------------------------------------------------------------------------

void  VitaPacketStreamDeviceSafeLink::OnAfterLink()
{
    Ptr()->RegisterStreamPlugIn(Owner);
}

//---------------------------------------------------------------------------
//  VitaPacketStreamDeviceSafeLink::OnBeforeUnlink() - handle pre-unlink stuff
//---------------------------------------------------------------------------

void  VitaPacketStreamDeviceSafeLink::OnBeforeUnlink()
{
    if (Owner->StreamStarted())
        Owner->Stop();

    Ptr()->RegisterStreamPlugIn(0);
}

//===========================================================================
//  CLASS VitaPacketStream  -- Packet Based Stream I/O
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VitaPacketStream
//---------------------------------------------------------------------------
#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif
VitaPacketStream::VitaPacketStream()
    :  StreamDevice(this), FVitaMode(vsmVitaMode),
       VitaOut(this), VitaIn(this, VitaInQ)
{
    //  TEMPORARY!!! until vita streaming works out
    FVitaMode = vsmVelociaMode;
//    Packer.OnDataAvailable.SetEvent(this, &VitaPacketStream::HandlePackerDataAvailable);
//    Packer.OnDataRequired.SetEvent(this, &VitaPacketStream::HandlePackerDataRequired);
//    Parser.OnImageAvailable.SetEvent(this, &VitaPacketStream::HandleParserImageAvailable);

    VitaOut.Resume();
    VitaIn.Resume();
}
#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class VitaPacketStream
//---------------------------------------------------------------------------

VitaPacketStream::~VitaPacketStream()
{
    VitaOut.Shutdown();
    VitaIn.Shutdown();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Preconfigure() -- Non-repeated 'Manual' Configuration
//---------------------------------------------------------------------------

void  VitaPacketStream::Preconfigure()
{
    PacketStreamBase::Preconfigure();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Start() -- Start busmastering
//---------------------------------------------------------------------------

void  VitaPacketStream::Start()
{
    VitaInQ.Clear();
    VitaIn.Clear();
    VitaOut.Clear();
    Parser.Clear();
//    Packer.Clear();

    VitaIn.Start();
    VitaOut.Start();

    PacketStreamBase::Start();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Stop() -- Stop busmastering
//---------------------------------------------------------------------------

void  VitaPacketStream::Stop()
{
    VitaIn.Stop();
    VitaOut.Stop();
    PacketStreamBase::Stop();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Internals() -- Stream Manager Internal Interface
//---------------------------------------------------------------------------

IPacketStreamManagerInternals *  VitaPacketStream::Internals()
{
    return PacketStreamBase::ManagerInternals();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::DeviceInternals() -- Stream Device Internal Interface
//---------------------------------------------------------------------------

IPacketStreamDeviceInternals *  VitaPacketStream::DeviceInternals()
{
    return StreamDevice->Internals();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Send() -- Post Vita packet for transmission
//---------------------------------------------------------------------------

void  VitaPacketStream::Send(VitaBuffer & packet)
{
    DeviceInternals()->TraceLog(0xb0, (int)packet.SizeInInts());

    //  Make sure packet size is correct...
    InitHeader(packet);
    //  ..and pack it into Velo buffer for transmission
//    Packer.Pack(packet);
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Send() -- Load Velo packet for transmission
//---------------------------------------------------------------------------

void  VitaPacketStream::Send(short PeriphId, VeloBuffer & packet )
{
    //
    //  Load Peripheral ID
    VeloHeaderDatagram headerDG(packet);
    headerDG.PeripheralId(PeriphId);
    //  ..and finish sending
    Send(packet);
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Send() -- Post Velo packet for transmission
//---------------------------------------------------------------------------

void  VitaPacketStream::Send(VeloBuffer & packet)
{
    //  Make sure header size & packet size is correct...
    ConvertHeader(packet);
    //  ..and send it out using base class method
    PacketStreamBase::BasicSend(packet);
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Recv() -- Return Velo packet received from host
//---------------------------------------------------------------------------

void  VitaPacketStream::Recv(VeloBuffer & packet)
{
    // Fill using base class method
    PacketStreamBase::BasicRecv(packet);
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Recv() -- Return Velo packet received from host
//---------------------------------------------------------------------------

void  VitaPacketStream::Recv(VitaBuffer & packet)
{
    VitaIn.Recv(packet);
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Pokey() --  DEBUG
//---------------------------------------------------------------------------

void  VitaPacketStream::Pokey(VitaBuffer & packet)
{
    VitaInQ.Push(packet);
}
//---------------------------------------------------------------------------
//  VitaPacketStream::Pokeyinit() --  DEBUG
//---------------------------------------------------------------------------

void  VitaPacketStream::Pokeyinit()
{
    VitaInQ.Clear();
    VitaIn.Clear();
    VitaIn.Start();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::VDepth() --  DEBUG
//---------------------------------------------------------------------------

unsigned int  VitaPacketStream::VDepth()
{
    return static_cast<unsigned int>(VitaInQ.Depth());
}

//---------------------------------------------------------------------------
//  VitaPacketStream::SignalDataAvailable() -- Signal data available to app
//---------------------------------------------------------------------------

void  VitaPacketStream::SignalDataAvailable()
{
    if (VelociaStreamMode())
        {
        VitaPacketStreamDataEvent e(this);
        OnVeloDataAvailable.Execute(e);
        return;
        }
    //
    //  Vita Mode Behavior...
    VeloBuffer  Velo;
    Recv(Velo);

    //  Push into parser & Parse
    Parser.Append(Velo);
    Parser.Parse();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::SignalDataRequired() -- Signal data needed to app
//---------------------------------------------------------------------------

void  VitaPacketStream::SignalDataRequired()
{
    if (VelociaStreamMode())
        {
        VitaPacketStreamDataEvent e(this);
        OnVeloDataRequired.Execute(e);
        return;
        }
    DeviceInternals()->TraceLog(0xbf, 0);
    //  Vita mode behavior
    VitaOut.SignalDataRequired();
}

//------------------------------------------------------------------------------
//  VitaPacketStream::SignalVitaDataAvailable() -- VITA data available to app
//------------------------------------------------------------------------------

void  VitaPacketStream::SignalVitaDataAvailable()
{
    if (VelociaStreamMode())
        return;

    VitaPacketStreamDataEvent e(this);
    OnDataAvailable.Execute(e);
}

//------------------------------------------------------------------------------
//  VitaPacketStream::SignalVitaDataRequired() -- VITA data needed from app
//------------------------------------------------------------------------------

void  VitaPacketStream::SignalVitaDataRequired()
{
    if (VelociaStreamMode())
        return;

    DeviceInternals()->TraceLog(0xbe, 0);
    VitaPacketStreamDataEvent e(this);
    OnDataRequired.Execute(e);
}

//---------------------------------------------------------------------------
//  VitaPacketStream::StreamStarted() -- returns true if the stream is started
//---------------------------------------------------------------------------

bool VitaPacketStream::StreamStarted()
{
    return DoStreamStarted();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::ConnectTo() -- connect stream to stream device
//---------------------------------------------------------------------------

bool  VitaPacketStream::ConnectTo(IVitaPacketStreamDevice * aStreamDevice)
{
    if (aStreamDevice)
        {
        StreamDevice.Link(aStreamDevice);
        return true;
        }
    else
        return false;
}

//---------------------------------------------------------------------------
//  VitaPacketStream::Disconnect() -- disconnect stream from stream device
//---------------------------------------------------------------------------

void VitaPacketStream::Disconnect()
{
    StreamDevice.Unlink();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::NativeBuffer() -- create native Velocia buffer type
//---------------------------------------------------------------------------

VeloBuffer  VitaPacketStream::NativeBuffer(size_t data_size, bool autoinit)
{
    VeloBuffer  buffer(data_size);
    if (autoinit)
        {
        ClearHeader(buffer);
        InitHeader(buffer);
        }
    return buffer;
}

//---------------------------------------------------------------------------
//  VitaPacketStream::NativeVitaBuffer() -- create native Vita buffer type
//---------------------------------------------------------------------------

VitaBuffer  VitaPacketStream::NativeVitaBuffer(size_t data_size, bool autoinit)
{
    VitaBuffer  buffer(data_size);
    if (autoinit)
        {
        ClearHeader(buffer);
        ClearTrailer(buffer);
        InitHeader(buffer);
        InitTrailer(buffer);
        }
    return buffer;
}

//------------------------------------------------------------------------------
//  VitaPacketStream::HandlePackerDataAvailable() -- VITA mode packet ready
//------------------------------------------------------------------------------

void  VitaPacketStream::HandlePackerDataAvailable(VitaPacketPackerDataAvailable & event)
{
    DeviceInternals()->TraceLog(0xb1, (int)event.Data.SizeInInts());

    //  At this point, we have a VELO packet ready to be sent on. So send it.
    Send(event.Data);
}

//------------------------------------------------------------------------------
//  VitaPacketStream::HandlePackerDataRequired() -- Packer needs more data
//------------------------------------------------------------------------------

void  VitaPacketStream::HandlePackerDataRequired(VitaPacketPackerDataRequired & /*event*/)
{
    DeviceInternals()->TraceLog(0xbb, 0);
    VitaOut.SignalDataRequired();
    //SignalVitaDataRequired();
}

//---------------------------------------------------------------------------
//  VitaPacketStream::HandleParserImageAvailable() -- Parser callback
//---------------------------------------------------------------------------

void  VitaPacketStream::HandleParserImageAvailable(VitaPacketParserImageAvailable & event)
{
    //
    //  Hoist Image Into
    //  Create Vita Packet
    VitaBuffer  Vita;
    //  Fill with Image Data
    event.Image.CopyDataTo( Vita );

    VitaInQ.Push(Vita);
}

//===========================================================================
//  CLASS VitaRcvPacketQueue -- Manage VITA packets to application
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VitaRcvPacketQueue
//---------------------------------------------------------------------------

VitaRcvPacketQueue::VitaRcvPacketQueue()
    : CapacityInBlks(10)
{
    IsNotEmpty.Clear();   // We are empty (not not empty)
    IsNotFull.Set();      // We are not full
}

//---------------------------------------------------------------------------
//  destructor for class VitaRcvPacketQueue
//---------------------------------------------------------------------------

VitaRcvPacketQueue::~VitaRcvPacketQueue()
{
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::Clear() -- clear queue
//---------------------------------------------------------------------------

void  VitaRcvPacketQueue::Clear()
{
    //  clear buffer queue
    VitaQ.clear();
    //  reduce signal to 0
    Signal.Clear();
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::Front() -- Front element in queue
//---------------------------------------------------------------------------

VitaBuffer  VitaRcvPacketQueue::Front()
{
    return VitaQ.front();
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::Depth() -- depth of queue
//---------------------------------------------------------------------------

size_t VitaRcvPacketQueue::Depth()
{
    return VitaQ.depth();
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::QueueCapacity() -- Set queue capacity
//---------------------------------------------------------------------------

void  VitaRcvPacketQueue::QueueCapacity(size_t cap)
{
    CriticalSectionArea guard(lock);
    CapacityInBlks = cap;
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::QueueCapacity() -- Retreive capacity
//---------------------------------------------------------------------------

size_t VitaRcvPacketQueue::QueueCapacity() const
{
    return CapacityInBlks;
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::SignalSem() -- Return reference to semaphore
//---------------------------------------------------------------------------

Semaphore & VitaRcvPacketQueue::SignalSem()
{
    return Signal;
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::BumpSignal() -- Release Signal Sem
//---------------------------------------------------------------------------

void VitaRcvPacketQueue::BumpSignal(unsigned int count)
{
    for (unsigned int i = 0; i < count; ++i)
        Signal.Release();
}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::Push() -- push buffer onto queue
//---------------------------------------------------------------------------

void  VitaRcvPacketQueue::Push( VitaBuffer & buffer )
{
    //  Wait for space
    IsNotFull.WaitFor();

    // ...otherwise
    CriticalSectionArea guard(lock);

    VitaQ.push(buffer);

    //  We are no longer empty, if someone is waiting
    IsNotEmpty.Set();

    //  We may be full now
    if (Depth()>CapacityInBlks)
        {
        //  Now full
        IsNotFull.Clear();
        }
    //
    //  For every buffer added, signal once
    Signal.Release();

}

//---------------------------------------------------------------------------
//  VitaRcvPacketQueue::Pop() --
//---------------------------------------------------------------------------

void  VitaRcvPacketQueue::Pop()
{
    //  Wait for data
    IsNotEmpty.WaitFor();

    CriticalSectionArea guard(lock);

    VitaQ.pop();

    //  We are no longer full, if someone is waiting
    IsNotFull.Set();

    //  We may be empty now
    if (Depth()==0)
        {
        //  Now empty
        IsNotEmpty.Clear();
        }

}

namespace VitaPacketImageNS
{
//==============================================================================
//  CLASS VitaPacketImageDatagram  --  Special Copy Ops for VitaPacketImage
//==============================================================================
//------------------------------------------------------------------------------
//  declaration for class VitaPacketImageDatagram
//------------------------------------------------------------------------------

class VitaPacketImageDatagram
{
public:
    VitaPacketImageDatagram(VitaBuffer & op, VitaPacketImage & image)
        : FBuffer(op), FImage(image), CurrentRange(0,0)
        {}
    void  FillBufferFromImage();
    void  FillBufferDataSectionFromImage();

    void  CopyBodyFromImageAt( ii32 * buf, size_t my_index, size_t size);
    void  CopyHeaderFromImageAt( ii32 * buf, size_t my_index, size_t size);
    void  CopyTrailerFromImageAt( ii32 * buf, size_t my_index, size_t size);
    void  AdvanceExtractionByWords(size_t words);

private:
    VitaBuffer &            FBuffer;
    VitaPacketImage &       FImage;
    VitaPacketImage::Range  CurrentRange;
    size_t                  RangeIdx;
    size_t                  ExtractionIdx;
};

//------------------------------------------------------------------------------
//  VitaPacketImageDatagram::FillBufferFromImage() -- Fill VitaBuffer
//------------------------------------------------------------------------------
//  This fill command assumes the image is larger or equal to the VitaBuffer size

void  VitaPacketImageDatagram::FillBufferFromImage()
{
    // Resize packet to equal packet size
    size_t buf_size = FImage.NextPacketDataSize();
    FBuffer.Resize(buf_size);

    //  Get the first image range
    RangeIdx = 0;
    CurrentRange = FImage.ImageRange(RangeIdx);
    ExtractionIdx = 0;

    // Header
    size_t buffer_idx = 0;
    size_t transfer_size = FBuffer.HeaderSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

        CopyHeaderFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        AdvanceExtractionByWords(size);
        }

    // Body
    buffer_idx = 0;
    transfer_size = FBuffer.SizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

        CopyBodyFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        
        //if (transfer_size)
            AdvanceExtractionByWords(size);
        }

    // Trailer
    buffer_idx = 0;
    transfer_size = FBuffer.TrailerSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

        CopyTrailerFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        
        if (transfer_size)
            AdvanceExtractionByWords(size);
        }
}

//------------------------------------------------------------------------------
//  VitaPacketImageDatagram::FillBufferDataSectionFromImage() -- Fill VitaBuffer
//------------------------------------------------------------------------------
//  This fill command assumes the image is larger or equal to the VitaBuffer size

void  VitaPacketImageDatagram::FillBufferDataSectionFromImage()
{
    // Resize packet to equal packet size
    size_t buf_size = FImage.NextPacketDataSize();
    FBuffer.Resize(buf_size);

    //  Get the first image range
    RangeIdx = 0;
    CurrentRange = FImage.ImageRange(RangeIdx);
    ExtractionIdx = 0;

    // Header
    size_t buffer_idx = 0;
    size_t transfer_size = FBuffer.HeaderSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

        //CopyHeaderFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        AdvanceExtractionByWords(size);
        }

    // Body
    buffer_idx = 0;
    transfer_size = FBuffer.SizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

        CopyBodyFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        
        //if (transfer_size)
            AdvanceExtractionByWords(size);
        }

    // Trailer
    buffer_idx = 0;
    transfer_size = FBuffer.TrailerSizeInInts();
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

//        CopyTrailerFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;

        if (transfer_size)
            AdvanceExtractionByWords(size);
        }
}

//---------------------------------------------------------------------------
//  VitaPacketImageDatagram::AdvanceExtractionByWords() --
//---------------------------------------------------------------------------

void  VitaPacketImageDatagram::AdvanceExtractionByWords(size_t words)
{
    ExtractionIdx += words;
    if (ExtractionIdx==CurrentRange.SizeInInts)
        {
        // Advance to next range
        RangeIdx++;
        if (RangeIdx<FImage.ImageRangeCount())
            CurrentRange = FImage.ImageRange(RangeIdx);
        ExtractionIdx = 0;
        }
}

//---------------------------------------------------------------------------
//  VitaPacketImageDatagram::CopyBodyFromImageAt() --
//---------------------------------------------------------------------------

void  VitaPacketImageDatagram::CopyBodyFromImageAt( ii32 * buf, size_t my_index, size_t size)
{
    // Body
    ii32 * TargetDataPtr = reinterpret_cast<ii32*>(FBuffer.Data());

    CopyInt32Buffer(reinterpret_cast<const int*>(buf),
                 reinterpret_cast<int*>(TargetDataPtr + my_index),
                 static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));
}

//---------------------------------------------------------------------------
//  VitaPacketImageDatagram::CopyHeaderFromImageAt() --
//---------------------------------------------------------------------------

void  VitaPacketImageDatagram::CopyHeaderFromImageAt( ii32 * buf, size_t my_index, size_t size)
{
    // Header
    ii32*    TargetDataPtr = (ii32*)FBuffer.Header();

    CopyInt32Buffer(reinterpret_cast<const int*>(buf),
                 reinterpret_cast<int*>(TargetDataPtr + my_index),
                 static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));

}

//---------------------------------------------------------------------------
//  VitaPacketImageDatagram::CopyTrailerFromImageAt() --
//---------------------------------------------------------------------------

void  VitaPacketImageDatagram::CopyTrailerFromImageAt( ii32 * buf, size_t my_index, size_t size)
{
    // Trailer
    ii32*    TargetDataPtr = (ii32*)FBuffer.Trailer();

    CopyInt32Buffer(reinterpret_cast<const int*>(buf),
                 reinterpret_cast<int*>(TargetDataPtr + my_index),
                 static_cast<int>(size * sizeof(int) / sizeof(int)));

}

}  // end namespace VitaPacketImageNS

//==============================================================================
//  CLASS VitaPacketImage  -- Vita Packet Position Information Store Object
//==============================================================================
//------------------------------------------------------------------------------
//  housekeeping for class VitaPacketImage
//------------------------------------------------------------------------------

const size_t MinimumVitaPacketSize = 12;  // eight word header, 4 word body

//------------------------------------------------------------------------------
//  VitaPacketImage::DataDepth() -- Total Data in Image in words
//------------------------------------------------------------------------------

size_t  VitaPacketImage::DataDepth() const
{
    size_t sum = 0;
    for (RangePack::const_iterator iter = Ranges.begin(); iter!=Ranges.end(); ++iter)
        {
        sum += iter->SizeInInts;
        }
    return sum;
}

//------------------------------------------------------------------------------
//  VitaPacketImage::NextPacketFullSize() -- read VITA packet size from image
//------------------------------------------------------------------------------

size_t  VitaPacketImage::NextPacketFullSize()
{
    if (!empty())
        {
        unsigned int * pos = Ranges[0].begin();
        VitaHeaderDatagram vh_dg(pos);
        return vh_dg.PacketSize();
        }
    else
        return 0;
}

//------------------------------------------------------------------------------
//  VitaPacketImage::NextPacketDataSize() -- read VITA data size from image
//------------------------------------------------------------------------------

size_t  VitaPacketImage::NextPacketDataSize()
{
    if (!empty())
        {
        unsigned int * pos = Ranges[0].begin();
        VitaHeaderDatagram vh_dg(pos);
        return vh_dg.DataSize();
        }
    else
        return 0;
}

//------------------------------------------------------------------------------
//  VitaPacketImage::ImageWord() -- Read Nth word from Image
//------------------------------------------------------------------------------

unsigned int  VitaPacketImage::ImageWord(size_t image_idx)
{
    if (image_idx >= DataDepth())
        return 0;

    for (unsigned int img=0; img<ImageRangeCount(); img++)
        {
        size_t RangeSize = Ranges[img].SizeInInts;
        if (image_idx < RangeSize )
            return *(Ranges[img].cbegin() + image_idx);
        else
            image_idx -= RangeSize;
        }

    return 0;
}

//------------------------------------------------------------------------------
//  VitaPacketImage::BodyWord() -- Read Nth word from Body
//------------------------------------------------------------------------------

unsigned int  VitaPacketImage::BodyWord(size_t body_idx)
{
    VitaBuffer  scratch;
    size_t header_size = scratch.HeaderSizeInInts();

    return ImageWord(body_idx + header_size);
}

//------------------------------------------------------------------------------
//  VitaPacketImage::TrailerWord() -- Read Nth word from Trailer
//------------------------------------------------------------------------------

unsigned int  VitaPacketImage::TrailerWord(size_t trailer_idx)
{
    VitaBuffer  scratch;
    size_t trailer_size = scratch.TrailerSizeInInts();
    size_t packet_size = NextPacketFullSize();
    size_t trailer_start = packet_size - trailer_size;

    return ImageWord(trailer_idx + trailer_start);
}

//------------------------------------------------------------------------------
//  VitaPacketImage::ExtractNextImage() -- 'pop' single VITA image from data
//------------------------------------------------------------------------------

VitaPacketImage  VitaPacketImage::ExtractNextImage()
{
    if (empty())
        return *this;

    size_t  depth = DataDepth();
    size_t  packet_remaining = NextPacketFullSize();
    //  Check for pathological packet sizes and fix
    if (packet_remaining < MinimumVitaPacketSize)
        packet_remaining = MinimumVitaPacketSize;

    if (depth < packet_remaining)
        return VitaPacketImage(0);   // not enough to make a packet

    VitaPacketImage out_image;
    while (packet_remaining)
        {
        if (packet_remaining >= Ranges[0].SizeInInts)
            {
            // The packet spans the entire first range, so output it
            out_image.AddRange(Ranges[0]);
            packet_remaining -= Ranges[0].SizeInInts;
            Ranges.pop_front();  // we have processed this range now
            }
        else
            {
            // Split the front Range and send out just our packet part
            // ...create Range covering first packet
            VitaPacketImage::Range first_vita = Ranges.front();
            first_vita.SizeInInts = packet_remaining;
            out_image.AddRange(first_vita);
            //
            // ...now deduct the data extracted from the range
            //    for next time.
            Ranges.front().Position += packet_remaining;
            Ranges.front().SizeInInts -= packet_remaining;
            packet_remaining = 0;
            }
        }
   return out_image;   // valid image of next packet
}

//------------------------------------------------------------------------------
//  VitaPacketImage::CopyDataTo() -- copy data in packet image into a buffer
//------------------------------------------------------------------------------

void  VitaPacketImage::CopyDataTo(VitaBuffer & buffer)
{
    VitaPacketImageNS::VitaPacketImageDatagram VPID( buffer, *this );
    VPID.FillBufferFromImage();
}

//------------------------------------------------------------------------------
//  VitaPacketImage::CopyOnlyDataSectionTo() -- copy data in packet image into a buffer
//------------------------------------------------------------------------------

void  VitaPacketImage::CopyOnlyDataSectionTo( VitaBuffer & buffer )
{
    VitaPacketImageNS::VitaPacketImageDatagram VPID( buffer, *this );
    VPID.FillBufferDataSectionFromImage();
}

//------------------------------------------------------------------------------
//  VitaPacketImage::AreaDump() --
//------------------------------------------------------------------------------

void  VitaPacketImage::AreaDump(std::vector<unsigned int> & vec)
{
    size_t Start = (Ranges[0].Position<vec.size()/2) ? 0 :  Ranges[0].Position-vec.size()/2;
    unsigned int * From = reinterpret_cast<unsigned int *>(Ranges[0].Velo.Data()) + Start;
    for (size_t idx = 0; idx<vec.size(); idx++)
        {
        if (idx<Ranges[0].Position-Start+Ranges[0].SizeInInts)
            vec[idx] = From[idx];
        else
            vec[idx] = 0;
        }
}

//==============================================================================
// CLASS VitaPacketParser  --  Parse VITA packets from a VELO buffer
//==============================================================================
//------------------------------------------------------------------------------
//  VitaPacketParser::Clear() -- Clear state of Parser of any old data
//------------------------------------------------------------------------------

void  VitaPacketParser::Clear()
{
    VeloQ.ClearRanges();
}

//------------------------------------------------------------------------------
//  VitaPacketParser::Append() -- Append buffer to Parser memory
//------------------------------------------------------------------------------

void  VitaPacketParser::Append(const VeloBuffer & buffer)
{
    //Log(0x01, buffer.SizeInInts());
    VeloQ.AddRange(buffer);
    //Log(0x02, VeloQ.ImageRangeCount());
    //Log(0x03, VeloQ.NextPacketFullSize());
    //Log(0x04, Depth());
}

//------------------------------------------------------------------------------
//  VitaPacketParser::Depth() -- Total Data in Parser in words
//------------------------------------------------------------------------------

size_t  VitaPacketParser::Depth() const
{
    return VeloQ.DataDepth();
}

//------------------------------------------------------------------------------
//  VitaPacketParser::Parse() -- Parse Images for embedded packets
//------------------------------------------------------------------------------

void  VitaPacketParser::Parse()
{
    if (VeloQ.empty())
        return;
    //Log(0x60, VeloQ.ImageRangeCount());
    //Log(0x61, Depth());

    while (1==1)
        {
        VitaPacketImage image(ParsePacketImage());
        if (image.empty())
            {
            Log(0x6F, 0);
            return;
            }
        //  call handler
        //Log(0x80, image.DataDepth());
        Log(0x81, Depth());
        VitaPacketParserImageAvailable evt(this, image);
        OnImageAvailable.Execute(evt);
        }
}

//------------------------------------------------------------------------------
//  VitaPacketParser::ParsePacketImage() -- Extract image of next packet
//------------------------------------------------------------------------------

VitaPacketImage  VitaPacketParser::ParsePacketImage()
{
    return VeloQ.ExtractNextImage();
}

//---------------------------------------------------------------------------
//  VitaPacketParser::DumpLog() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  VitaPacketParser::DumpLog(std::vector<int> & vec)
{
	DebugLog.copy_to(vec);
}


namespace VitaPacketImageNS
{
//==============================================================================
//  CLASS VitaIntoVeloPacketDatagram  --  Special Copy Ops for loading Vitas
//==============================================================================
//------------------------------------------------------------------------------
//  declaration for class VitaIntoVeloPacketDatagram
//------------------------------------------------------------------------------

class VitaIntoVeloPacketDatagram
{
public:
    VitaIntoVeloPacketDatagram(VeloBuffer & out, size_t & pos,
                               const VitaBuffer & in, size_t & from_pos)
        : OutBuffer(out), OutPos(pos), InBuffer(in), FromPos(from_pos)
        {}
    void  PackVitaIntoVelo();

	void  PackHeaderFrom(size_t vita_offset, size_t size);
	void  PackBodyFrom(size_t vita_offset, size_t size);
	void  PackTrailerFrom(size_t vita_offset, size_t size);

private:
    VeloBuffer &       OutBuffer;
    size_t &           OutPos;       // where to start filling Velo
    const VitaBuffer &       InBuffer;
    size_t &           FromPos;      // where to start copying VITA from
};

//------------------------------------------------------------------------------
//  VitaIntoVeloPacketDatagram::PackVitaIntoVelo() -- Fill VeloBuffer with VITA
//------------------------------------------------------------------------------

void  VitaIntoVeloPacketDatagram::PackVitaIntoVelo()
{
    // Bad Offset
    if (OutPos >= OutBuffer.SizeInInts())
        return;

    //  Max words that CAN be transferred to this Velo buffer
    size_t words_before_end = OutBuffer.SizeInInts() - OutPos;

    // Header
    //   Q: have we copied this last time?
    size_t hdr_size = InBuffer.HeaderSizeInInts();
    size_t size_to_skip = FromPos;
    if (size_to_skip >= hdr_size)
        {
        //  no need to load header at all
        size_to_skip -= hdr_size;
        }
    else
        {
        // copy some or all of the header
        size_t header_words_left = hdr_size - size_to_skip;
        size_t size = std::min(words_before_end, header_words_left);
        // perform copy from header region
        PackHeaderFrom( size_to_skip, size );
        // adjust offsets
        OutPos += size;
        FromPos += size;
        words_before_end -= size;
        size_to_skip = 0;      // start at front on other sections
        }

    //  No room in out buffer?
    if (words_before_end==0)
        {
        OutPos = 0;
        return;
        }

    // Body
    size_t body_size = InBuffer.SizeInInts();
    if (size_to_skip >= body_size)
        {
        //  no need to load body at all
        size_to_skip -= body_size;
        }
    else
        {
        // copy some or all of the body
        size_t body_words_left = body_size - size_to_skip;
        size_t size = std::min(words_before_end, body_words_left);
        // perform copy from body region
        PackBodyFrom( size_to_skip, size );
        // adjust offsets
        OutPos += size;
        FromPos += size;
        words_before_end -= size;
        size_to_skip = 0;      // start at front on other sections
        }

    if (words_before_end==0)
        {
        OutPos = 0;
        return;
        }

    // Trailer
    size_t trailer_size = InBuffer.TrailerSizeInInts();
    if (size_to_skip >= trailer_size)
        {
        //  no need to load trailer at all (this should never happen)
        size_to_skip -= trailer_size;
        }
    else
        {
        // copy some or all of the trailer
        size_t trailer_words_left = trailer_size - size_to_skip;
        size_t size = std::min(words_before_end, trailer_words_left);
        // perform copy from trailer region
        PackTrailerFrom( size_to_skip, size );
        // adjust offsets
        OutPos += size;
        FromPos += size;
        words_before_end -= size;
        }

    if (FromPos==InBuffer.FullSizeInInts())
        {
        FromPos = 0;
        }
    if (words_before_end==0)
        {
        OutPos = 0;
        }
    //  If OutPos is 0 on exit, the VELO packet is full.
    //  If FromPos is 0 on exit, the VITA packet is fully copied
    //  Both conditions can be true if you are lucky
}

//---------------------------------------------------------------------------
//  VitaIntoVeloPacketDatagram::PackHeaderFrom() --
//---------------------------------------------------------------------------

void  VitaIntoVeloPacketDatagram::PackHeaderFrom(size_t vita_offset, size_t size)
{
    // Header
    const int *   SrcDataPtr  = InBuffer.ConstHeader() + vita_offset;
          int *   DestDataPtr = OutBuffer.Data() + OutPos;

    CopyInt32Buffer(reinterpret_cast<const int*>(SrcDataPtr),
                    reinterpret_cast<int*>(DestDataPtr),
                    static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));

}

//---------------------------------------------------------------------------
//  VitaIntoVeloPacketDatagram::PackBodyFrom() --
//---------------------------------------------------------------------------

void  VitaIntoVeloPacketDatagram::PackBodyFrom(size_t vita_offset, size_t size)
{
    // Header
    const int *   SrcDataPtr  = InBuffer.ConstData() + vita_offset;
          int *   DestDataPtr = OutBuffer.Data() + OutPos;

    CopyInt32Buffer(reinterpret_cast<const int*>(SrcDataPtr),
                    reinterpret_cast<int*>(DestDataPtr),
                    static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));

}

//---------------------------------------------------------------------------
//  VitaIntoVeloPacketDatagram::PackTrailerFrom() --
//---------------------------------------------------------------------------

void  VitaIntoVeloPacketDatagram::PackTrailerFrom(size_t vita_offset, size_t size)
{
    // Header
    const int *   SrcDataPtr  = InBuffer.ConstTrailer() + vita_offset;
          int *   DestDataPtr = OutBuffer.Data() + OutPos;

    CopyInt32Buffer(reinterpret_cast<const int*>(SrcDataPtr),
                    reinterpret_cast<int*>(DestDataPtr),
                    static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));

}
}  // end namespace VitaPacketImageNS

//==============================================================================
// CLASS VitaPacketPacker  --  Pack Velo packets with VITA packets
//==============================================================================
//------------------------------------------------------------------------------
//  VitaPacketPacker::Clear() --  Clear state of packer...
//------------------------------------------------------------------------------

void  VitaPacketPacker::Clear()
{
    OutPos = 0;
    Out.Resize(OutputSize);
}

//------------------------------------------------------------------------------
//  VitaPacketPacker::Pack() --  Pack VITA into buffer
//------------------------------------------------------------------------------

void  VitaPacketPacker::Pack(const VitaBuffer & In)
{
    size_t FromPos(0);
    bool   wasVeloOutput(false);

    //  Multi-pass packing, to make sure we fully process 'In'
    while (1==1)
        {
        VitaPacketImageNS::VitaIntoVeloPacketDatagram Packer(Out, OutPos, In, FromPos);
        Packer.PackVitaIntoVelo();
        //
        //  If OutPos is 0 on exit, the VELO packet is full. Call event to dispatch
        if (OutPos==0)
            {
            VitaPacketPackerDataAvailable e(this, Out);
            OnDataAvailable.Execute(e);
            // Just in case we have changed output packet size....
            Out.Resize(OutputSize);
            wasVeloOutput = true;
            }
        //
        //  If FromPos is 0 on exit, the VITA packet is fully copied, so we are done
        if (FromPos==0)
            break;
        }

    if (wasVeloOutput==false)
        {
        //  If no packet was output, we need to allow a resignal to request more input
        VitaPacketPackerDataRequired e(this);
        OnDataRequired.Execute(e);
        }

}

//------------------------------------------------------------------------------
//  VitaPacketPacker::Flush() --  Output 'Remainder' as complete data packet
//------------------------------------------------------------------------------

void  VitaPacketPacker::Flush()
{
    //
    //  If OutPos is not 0, we have a partial packet
    if (OutPos!=0)
        {
        //  Truncate the output packet (this does not wipe data)
        Out.Resize(OutPos);
        OutPos = 0;
        InitHeader(Out);

        VitaPacketPackerDataAvailable e(this, Out);
        OnDataAvailable.Execute(e);
		
		Out.Resize(OutputSize);   // resize Out to normal.
		}

}

namespace VitaPacketImageNS
{
//==============================================================================
//  CLASS VitaPacketImageAppendDatagram  --  Special Copy
//==============================================================================
//------------------------------------------------------------------------------
//  declaration for class VitaPacketImageAppendDatagram
//------------------------------------------------------------------------------

class VitaPacketImageAppendDatagram
{
public:
    VitaPacketImageAppendDatagram(VitaBuffer & op, size_t & pos,
                                  VitaPacketImage & image, size_t & from_pos)
        : FBuffer(op), OutPos(pos), FImage(image), FromPos(from_pos),
          CurrentRange(0,0)
        {}
    void  FillHeaderFromImage();
    void  AppendBufferFromImage();
    void  FillTrailerFromImage();

	void  CopyBodyFromImageAt( ii32 * buf, size_t my_index, size_t size);
	void  CopyHeaderFromImageAt( ii32 * buf, size_t my_index, size_t size);
	void  CopyTrailerFromImageAt( ii32 * buf, size_t my_index, size_t size);
    void  AdvanceExtractionByWords(size_t words);

private:
    VitaBuffer &            FBuffer;
    size_t &                OutPos;       // where to start filling Vita
    VitaPacketImage &       FImage;
    size_t &                FromPos;       // where to start emptying Image
    VitaPacketImage::Range  CurrentRange;
    size_t                  RangeIdx;
    size_t                  ExtractionIdx;
};

//------------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::FillHeaderFromImage() -- Fill Vita Header
//------------------------------------------------------------------------------
//  This fill command assumes the image is larger or equal to the VitaBuffer size

void  VitaPacketImageAppendDatagram::FillHeaderFromImage()
{
    ii32*    TargetDataPtr = (ii32*)FBuffer.Header();

    for (unsigned int i=0; i<FBuffer.HeaderSizeInInts(); i++)
        {
        *TargetDataPtr = FImage.HeaderWord(i);
        }

    InitHeader(FBuffer);
}

//------------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::AppendBufferFromImage() -- Fill VitaBuffer
//------------------------------------------------------------------------------
//  This fill command assumes the image is larger or equal to the VitaBuffer size

void  VitaPacketImageAppendDatagram::AppendBufferFromImage()
{
    //  Get the first image range
    RangeIdx = 0;
    CurrentRange = FImage.ImageRange(RangeIdx);
    ExtractionIdx = 0;

    //
    //  Initial Skip in Image
    size_t buffer_idx = 0;
    size_t transfer_size = FBuffer.HeaderSizeInInts() + FromPos;

    // SKIP Header and already copied area
    while (transfer_size>0)
        {
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;
        size_t size = std::min(words_before_end, transfer_size);

        //  no copying intentionally
        transfer_size -= size;
        buffer_idx += size;
        AdvanceExtractionByWords(size);
        }

    // Body
    buffer_idx = OutPos;
    transfer_size = FBuffer.SizeInInts() - OutPos;
    while (transfer_size>0)
        {
        if (FromPos==FImage.NextPacketDataSize())
            {
            break;
            }

        size_t data_size = FImage.NextPacketDataSize() - FromPos;
        size_t words_before_end = CurrentRange.SizeInInts - ExtractionIdx;

        //  Min of all three - transfer, range, and remaining data sizes
        size_t size = std::min(words_before_end, transfer_size);
               size = std::min(size, data_size);

        CopyBodyFromImageAt(CurrentRange.begin()+ExtractionIdx, buffer_idx, size);
        transfer_size -= size;
        buffer_idx += size;
        OutPos += size;
        FromPos += size;

        if (transfer_size)
            AdvanceExtractionByWords(size);
        }

    if (FromPos==FImage.NextPacketDataSize())
        {
        FromPos = 0;
        }
    if (OutPos==FBuffer.SizeInInts())
        {
        OutPos = 0;
        }
    //  If OutPos is 0 on exit, the VELO packet is full.
    //  If FromPos is 0 on exit, the VITA Image is fully copied
    //  Both conditions can be true if you are lucky

}

//------------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::FillTrailerFromImage() -- Fill Vita Trailer
//------------------------------------------------------------------------------

void  VitaPacketImageAppendDatagram::FillTrailerFromImage()
{
    // Trailer
    ii32*    TargetDataPtr = (ii32*)FBuffer.Trailer();

    *TargetDataPtr = FImage.TrailerWord(0);
}

//---------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::AdvanceExtractionByWords() --
//---------------------------------------------------------------------------

void  VitaPacketImageAppendDatagram::AdvanceExtractionByWords(size_t words)
{
    ExtractionIdx += words;
    if (ExtractionIdx==CurrentRange.SizeInInts)
        {
        // Advance to next range
        RangeIdx++;
        CurrentRange = FImage.ImageRange(RangeIdx);
        ExtractionIdx = 0;
        }
}

//---------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::CopyBodyFromImageAt() --
//---------------------------------------------------------------------------

void  VitaPacketImageAppendDatagram::CopyBodyFromImageAt( ii32 * buf, size_t my_index, size_t size)
{
    // Body
    ii32 * TargetDataPtr = reinterpret_cast<ii32*>(FBuffer.Data());

    CopyInt32Buffer(reinterpret_cast<const int*>(buf),
                 reinterpret_cast<int*>(TargetDataPtr + my_index),
                 static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));
}

//---------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::CopyHeaderFromImageAt() --
//---------------------------------------------------------------------------

void  VitaPacketImageAppendDatagram::CopyHeaderFromImageAt( ii32 * buf, size_t my_index, size_t size)
{
    // Header
    ii32*    TargetDataPtr = (ii32*)FBuffer.Header();

    CopyInt32Buffer(reinterpret_cast<const int*>(buf),
                 reinterpret_cast<int*>(TargetDataPtr + my_index),
                 static_cast<int>(size * sizeof(ii32) / sizeof(ii32)));

}

//---------------------------------------------------------------------------
//  VitaPacketImageAppendDatagram::CopyTrailerFromImageAt() --
//---------------------------------------------------------------------------

void  VitaPacketImageAppendDatagram::CopyTrailerFromImageAt( ii32 * buf, size_t my_index, size_t size)
{
    // Trailer
    ii32*    TargetDataPtr = (ii32*)FBuffer.Trailer();

    CopyInt32Buffer(reinterpret_cast<const int*>(buf),
                 reinterpret_cast<int*>(TargetDataPtr + my_index),
                 static_cast<int>(size * sizeof(int) / sizeof(int)));

}

}  // end namespace VitaPacketImageNS

//==============================================================================
// CLASS VitaPacketCombiner  --  Pack Velo packets with VITA packets
//==============================================================================
//------------------------------------------------------------------------------
//  VitaPacketCombiner::Clear() --  Clear state of packer...
//------------------------------------------------------------------------------

void  VitaPacketCombiner::Clear()
{
    OutPos = 0;
    Out.Resize(OutputSize);
}

//------------------------------------------------------------------------------
//  VitaPacketCombiner::Append() --  Append new VITA into OUT buffer
//------------------------------------------------------------------------------

void  VitaPacketCombiner::Append(VitaPacketImage & In)
{
    size_t FromPos(0);

    if (OutPos==0)   // If this is the first time in, copy the header
        {
        VitaPacketImageNS::VitaPacketImageAppendDatagram Appender(Out, OutPos, In, FromPos);
        Appender.FillHeaderFromImage();
        }

    //  Multi-pass packing, to make sure we fully process 'In'
    while (1==1)
        {
        VitaPacketImageNS::VitaPacketImageAppendDatagram Appender(Out, OutPos, In, FromPos);
        Appender.AppendBufferFromImage();
        //
        //  If OutPos is 0 on exit, the VITA packet is full. Call event to dispatch
        if (OutPos==0)
            {
            VitaPacketCombinerDataAvailable e(this, Out);
            OnDataAvailable.Execute(e);
            // Just in case we have changed output packet size....
            Out.Resize(OutputSize);

            //  And copy the header too
            VitaPacketImageNS::VitaPacketImageAppendDatagram Appender(Out, OutPos, In, FromPos);
            Appender.FillHeaderFromImage();
            }
        //
        //  If FromPos is 0 on exit, the VITA image is fully copied...
        if (FromPos==0)
            {
            //  Copy trailer from image to output packet
            VitaPacketImageNS::VitaPacketImageAppendDatagram Appender(Out, OutPos, In, FromPos);
            Appender.FillTrailerFromImage();

            //  Truncate if non-zero padding
            if (Out.PadBytes() != 0)
                {
                Flush();   // emit truncated Out buffer
                }
            return;
            }
        }

}

//------------------------------------------------------------------------------
//  VitaPacketCombiner::Flush() --  Output 'Remainder' as complete data packet
//------------------------------------------------------------------------------

void  VitaPacketCombiner::Flush()
{
    //
    //  If OutPos is not 0, we have a partial packet
    if (OutPos!=0)
        {
        //  Truncate the output packet (this does not wipe data)
        Out.Resize(OutPos);
        OutPos = 0;
        InitHeader(Out);

        VitaPacketCombinerDataAvailable e(this, Out);
        OnDataAvailable.Execute(e);

        Out.Resize(OutputSize);   // resize Out to normal.
        }

}

//==============================================================================
//  CLASS ASidComparePredicate  -- Compare OutInfo by matching SIDs
//==============================================================================

class ASidComparePredicate : public std::unary_function<VeloMergeParser::OutInfo, bool>
{
public:
    unsigned int SID;

    bool operator () (VeloMergeParser::OutInfo value)
    {
        return SID == value.Sid;
    }

    ASidComparePredicate(unsigned int sid) : SID(sid)
        {}
};

//==============================================================================
//  CLASS AlignedVeloPacketExQ  -- Vita Packet Position Information Store Object
//==============================================================================
//------------------------------------------------------------------------------
//  housekeeping for class AlignedVeloPacketExQ
//------------------------------------------------------------------------------

//const size_t MinimumVitaPacketSize = 12;  // eight word header, 4 word body

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::Init() -- Init the output sid list
//------------------------------------------------------------------------------

void  AlignedVeloPacketExQ:: Init(std::vector<int> & sids)
{
    Sids = sids;
}

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::ClearRanges() -- Total Data in Image in words
//------------------------------------------------------------------------------

void  AlignedVeloPacketExQ::ClearRanges()
{
    OutVitaRanges.clear();
    UnknownSidCount = 0;
}

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::NextPacketFullSizeNC() -- not checked packet full size
//------------------------------------------------------------------------------

size_t  AlignedVeloPacketExQ::NextPacketFullSizeNC(AlignedVeloPacketExQ::Range & InVelo)
{
    unsigned int * pos = InVelo.begin();
    VitaHeaderDatagram vh_dg(pos);
    return vh_dg.PacketSize();
}

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::NextPacketSID() -- not checked packet Stream Id
//------------------------------------------------------------------------------

int  AlignedVeloPacketExQ::NextPacketSID(AlignedVeloPacketExQ::Range & InVelo)
{
    unsigned int * pos = InVelo.begin();
    VitaHeaderDatagram vh_dg(pos);
    return vh_dg.StreamId();
}

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::vita_trailer_ptr() -- return pointer to trailer word
//------------------------------------------------------------------------------

unsigned int *  AlignedVeloPacketExQ::vita_trailer_ptr()
{
    // the trailer is the last word
    unsigned int * pos = OutVitaRanges[0].end() - 1;
    return pos;
}

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::vita_header_ptr() -- return pointer to header word
//------------------------------------------------------------------------------

unsigned int *  AlignedVeloPacketExQ::vita_header_ptr()
{
    // the header is the first word
    unsigned int * pos = OutVitaRanges[0].begin();
    return pos;
}

//------------------------------------------------------------------------------
//  AlignedVeloPacketExQ::UnpackVitas() -- Unpack all Vitas into OutQ
//------------------------------------------------------------------------------

void  AlignedVeloPacketExQ::UnpackVitas(const VeloBuffer & buffer)
{
    Range InVelo(buffer);
    while (InVelo.SizeInInts!=0)
        {
        size_t  velo_depth = InVelo.SizeInInts;
        size_t  packet_size = NextPacketFullSizeNC(InVelo);
        if (packet_size<MinimumVitaPacketSize)       // sanity check
            packet_size = MinimumVitaPacketSize;

        size_t  extraction_size = std::min(velo_depth, packet_size);

        if (std::find(Sids.begin(), Sids.end(), NextPacketSID(InVelo)) != Sids.end())
            {
            AlignedVeloPacketExQ::Range first_vita = InVelo;    // Copy first vita
            first_vita.SizeInInts = extraction_size;
            AddOutRange(first_vita);
            }
        else
            {
            UnknownSidCount++;
            }

        InVelo.Position += packet_size;
        InVelo.SizeInInts -= packet_size;
        }
}

//==============================================================================
// CLASS VeloMergeParser  --  Merge contained VITA packets into large Velos
//==============================================================================
//------------------------------------------------------------------------------
//  VeloMergeParser::Init() -- Init the output sid list
//------------------------------------------------------------------------------

void  VeloMergeParser:: Init(std::vector<int> & sids)
{
    Sids = sids;
    VeloQ.Init(sids);
    To.resize(sids.size());
    for (size_t i=0; i<To.size(); i++)
        {
        To[i].Sid = sids[i];
        ClearHeader(To[i].OutData);
        //  Set pid for outdata
        VeloHeaderDatagram vh(To[i].OutData);
        //
        //  Since SID is larger than pid, use index instead
        vh.PeripheralId(static_cast<unsigned short>(i));
        }
}

//------------------------------------------------------------------------------
//  VeloMergeParser::Resize() -- Resize the output buffers
//------------------------------------------------------------------------------

void  VeloMergeParser:: Resize(size_t out_data_size)
{
    OutDataSize = out_data_size;
    for (size_t i=0; i<To.size(); i++)
        {
        To[i].OutData.Resize(out_data_size);
        InitHeader(To[i].OutData);
        }
}

//------------------------------------------------------------------------------
//  VeloMergeParser::Clear() -- Clear state of Parser of any old data
//------------------------------------------------------------------------------

void  VeloMergeParser::Clear()
{
    VeloQ.ClearRanges();
    for (size_t i=0; i<To.size(); i++)
        {
        To[i].Pos = 0;
        }
}

//------------------------------------------------------------------------------
//  VeloMergeParser::Append() -- Append buffer to Parser memory
//------------------------------------------------------------------------------

void  VeloMergeParser::Append(const VeloBuffer & buffer)
{
    VeloQ.UnpackVitas(buffer);
}

//------------------------------------------------------------------------------
//  VeloMergeParser::Parse() -- Parse Images for embedded packets
//------------------------------------------------------------------------------

void  VeloMergeParser::Parse()
{
    //
    //  Parse the unpacked vitas
    while (1==1)
        {
        //  check if we are done parsing
        if (VeloQ.vita_empty()) // no more work
            {
            return;
            }

        PackVitaIntoVelo();
        }
}

//------------------------------------------------------------------------------
//  VeloMergeParser::PackVitaIntoVelo() -- Fill VeloBuffer with VITA
//------------------------------------------------------------------------------

void  VeloMergeParser::PackVitaIntoVelo()
{
    //
    //  Step 1: Get SID, To Buffer
    unsigned int CurrentSid = GetCurrentSid();

    OutInfo * to(GetOutInfo(CurrentSid));
    if (to == 0 || to->OutData.SizeInBytes()==0)
        {
        //  Unknown Sid or zero sized output buffer 
        VeloQ.vita_pop();  //  done with current image
        return;
        }

    size_t       Pos = VitaHeaderBytes();  // Skip Vita Header
    size_t       RearSkip = VitaTrailerBytes() + VitaPadding();

    while (1==1)
        {
        //
        //  Max words that CAN be transferred to this Velo buffer
        size_t bytes_before_end = to->OutData.SizeInBytes() - to->Pos;
        size_t image_data_bytes = VeloQ.vita_image_bytes() - Pos - RearSkip;

        size_t size = std::min(bytes_before_end, image_data_bytes);
        //
        //  Copy what you can to output buffer..
        CopyToVelo(VeloQ.vita_image(0), Pos, to->OutData, to->Pos, size);
        Pos += size;
        to->Pos += size;
        bytes_before_end -= size;
        //
        //  "Back Skip Processing"
        if (bytes_before_end==0)
            {
            to->Pos = 0;
            //
            //  Output VELO packet for processing
            VeloMergeParserDataAvailable evt(this, to->OutData);
            OnDataAvailable.Execute(evt);
            }
        else
            {
            //  Completed Vita packet
            VeloQ.vita_pop();  //  done with current image
            return;
            }
        }
}

//---------------------------------------------------------------------------
//  VeloMergeParser::CopyToVelo() --
//---------------------------------------------------------------------------

void  VeloMergeParser::CopyToVelo(AlignedVeloPacketExQ::Range & inRange, size_t InPos,
                                  VeloBuffer & outBuffer, size_t OutPos, size_t bytes)
{
    // Body only
    const char *   SrcDataPtr  = reinterpret_cast<char *>(inRange.begin())+ InPos;
          char *   DestDataPtr = reinterpret_cast<char *>(outBuffer.Data()) + OutPos;

    CopyByteBuffer( reinterpret_cast<const unsigned char*>(SrcDataPtr),
                    reinterpret_cast<unsigned char*>(DestDataPtr),
                    static_cast<unsigned int>(bytes) );

}

//---------------------------------------------------------------------------
//  VeloMergeParser::VitaPadding() -- return padding value of current range
//---------------------------------------------------------------------------

unsigned int  VeloMergeParser::VitaPadding()
{
    VitaTrailerDatagram vTrailer(VeloQ.vita_trailer_ptr());
    return static_cast<unsigned int>(vTrailer.Padding());
}

//---------------------------------------------------------------------------
//  VeloMergeParser::GetCurrentSid() -- return sid value of current pkt
//---------------------------------------------------------------------------

unsigned int  VeloMergeParser::GetCurrentSid()
{
    VitaHeaderDatagram vHdr(VeloQ.vita_header_ptr());
    return vHdr.StreamId();
}

//---------------------------------------------------------------------------
//  VeloMergeParser::GetOutInfo() -- return sid value of current pkt
//---------------------------------------------------------------------------

VeloMergeParser::OutInfo *   VeloMergeParser::GetOutInfo(unsigned int sid)
{
    ASidComparePredicate pred(sid);

    std::vector<OutInfo>::iterator iter;

    iter = std::find_if(To.begin(), To.end(), pred);
    if (iter == To.end())
        {
        //  not found
        return 0;
        }
    else
        return &*iter;
}

//------------------------------------------------------------------------------
//  VeloMergeParser::Flush() --  Output 'Remainder' as complete data packet
//------------------------------------------------------------------------------

void  VeloMergeParser::Flush()
{
    for (size_t i=0; i<To.size(); i++)
        {
        if (To[i].Pos!=0)
            {
            //  Truncate the output packet (this does not wipe data)
            To[i].OutData.Resize(To[i].Pos / sizeof(int));
            To[i].Pos = 0;
            InitHeader(To[i].OutData);

            VeloMergeParserDataAvailable evt(this, To[i].OutData);
            OnDataAvailable.Execute(evt);

            To[i].OutData.Resize(OutDataSize);   // resize Out to normal.
            }
        }
}

}  // namespace Innovative

