// Vpx_ComexDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <VpxComexDeviceMap_Mb.h>
#include <VpxComex_Mb.h>
#include <VpxComexMemoryMap_Mb.h>
#include <VpxComexDeviceDriver_Mb.h>
#include <Gps_MiniT_Mb.h>
#include <Gps_CW25_Mb.h>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

//=============================================================================
//  CLASS VpxComex_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxComex_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class VpxComex_Pll
//---------------------------------------------------------------------------

VpxComex_Pll::VpxComex_Pll(VpxComexDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner)
{
    SetMasterOutputSelection(5);  // Use Output 5 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class VpxComex_Pll
//---------------------------------------------------------------------------

VpxComex_Pll::~VpxComex_Pll()
{
}

//---------------------------------------------------------------------------
//  VpxComex_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  VpxComex_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "VpxEpc_Pll::OnInit()");

    //  Device Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvds);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         true,   // Secondary Bias ON
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       false,  // Input Term NOT disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, true);   SetOutputConfig(0, ocLvPecl);   // FPGA
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds);     // GCLK 0
    OutputInUse(2, true);   SetOutputConfig(2, ocLvds);     // GCLK 1
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds);     // GCLK 2
    OutputInUse(4, true);   SetOutputConfig(4, ocLvds);     // GCLK 3
    OutputInUse(5, true);   SetOutputConfig(5, ocLvPecl);   // FP 0
    OutputInUse(6, true);   SetOutputConfig(6, ocLvPecl);   // FP 1
    OutputInUse(7, true);   SetOutputConfig(7, ocLvPecl);   // FP 2
    OutputInUse(8, true);   SetOutputConfig(8, ocLvPecl);   // FP 3
    OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);

    // Set default enable state
    OutputEnabled(0, true);            //  FPGA
    OutputEnabled(1, false);           //  GCLK 0
    OutputEnabled(2, false);           //  GCLK 1
    OutputEnabled(3, false);           //  GCLK 2
    OutputEnabled(4, false);           //  GCLK 3
    OutputEnabled(5, false);           //  FP 0
    OutputEnabled(6, false);           //  FP 1
    OutputEnabled(7, false);           //  FP 2
    OutputEnabled(8, false);           //  FP 3

    //  Phase = 0
    SetPhaseAdjust(0, 1, 0);  SetPhaseAdjust(1, 1, 0);  // Set all outputs to 0 phase
    SetPhaseAdjust(2, 1, 0);  SetPhaseAdjust(3, 1, 0);
    SetPhaseAdjust(4, 1, 0);  SetPhaseAdjust(5, 1, 0);
    SetPhaseAdjust(6, 1, 0);  SetPhaseAdjust(7, 1, 0);
    SetPhaseAdjust(8, 1, 0);  SetPhaseAdjust(9, 1, 0);

    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    OutputFrequencyLimit[0] = 1000.0e6;     // FPGA
    OutputFrequencyLimit[1] = 1000.0e6;     // GCLK 0
    OutputFrequencyLimit[2] = 1000.0e6;     // GCLK 1
    OutputFrequencyLimit[3] = 1000.0e6;     // GCLK 2
    OutputFrequencyLimit[4] = 1000.0e6;     // GCLK 3
    OutputFrequencyLimit[5] = 1000.0e6;     // FP 0
    OutputFrequencyLimit[6] = 1000.0e6;     // FP 1
    OutputFrequencyLimit[7] = 1000.0e6;     // FP 2
    OutputFrequencyLimit[8] = 1000.0e6;     // FP 3
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use external pin
    ReferenceSource(refSecondary);    // Set to use secondary reference

}

//---------------------------------------------------------------------------
//  VpxComex_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  VpxComex_Pll::Vcxo()
{
    return Owner->Clock_SDev.Vcxo();
}

//---------------------------------------------------------------------------
//  VpxComex_Pll::OnReapply()
//---------------------------------------------------------------------------

void  VpxComex_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  VpxComex_Pll::OutputFrequency() -- Set ALL output frequencies
//---------------------------------------------------------------------------
//
//void  VpxComex_Pll::OutputFrequency(unsigned int output, double freq)
//{
//    //  If this is the master, we need to adjust everyone's output
//    //    to follow it
//    if (output == FMasterOutput)
//        {
//        bool  single_channel = DOps->IsDacInSingleChannelMode();
//
//        Cdce72010::OutputFrequency(2, ADCCLK(freq)/2);
//        Cdce72010::OutputFrequency(3, DCLK(single_channel, freq));
//        Cdce72010::OutputFrequency(4, DCLK(single_channel, freq));
//        Cdce72010::OutputFrequency(5, freq);
//        Cdce72010::OutputFrequency(6, ADCCLK(freq));
//        Cdce72010::OutputFrequency(7, ADCCLK(freq));
//        }
//
//    //  Apply the frequency
//    Cdce72010::OutputFrequency(output, freq);
//}
//
//---------------------------------------------------------------------------
//  VpxComex_Pll::OutputFrequency() -- Set ALL output frequencies
//---------------------------------------------------------------------------
//
//double  VpxComex_Pll::OutputFrequency(unsigned int output) const
//{
//    return Cdce72010::OutputFrequency(output);
//}
//
//---------------------------------------------------------------------------
//  VpxComex_Pll::UpdateSources() -- Set reference frequencies from IMUX
//---------------------------------------------------------------------------
void  VpxComex_Pll::UpdateSources()
{
    if (Src==0)
        return;
    ReferenceFrequency(0, Src->OutputFrequencyActual(3));

    if (OutputSource() == outPll)
        ReferenceFrequency(1, Src->OutputFrequencyActual(1));
    else
        ReferenceFrequency(1, Src->OutputFrequencyActual(4));
}

//===========================================================================
//  CLASS VpxComex_ClockMux  -- Configurable Input Mux
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VpxComex_ClockMux
//---------------------------------------------------------------------------

VpxComex_ClockMux::VpxComex_ClockMux(VpxComexDeviceMap * owner, IAddressingSpace & space,
                             Cdce18005Registers & regs)
    :  Cdce18005(space, regs), Owner(owner), Debug(true)
{
}

//---------------------------------------------------------------------------
// VpxComex_ClockMux::OnInit()
//---------------------------------------------------------------------------

void VpxComex_ClockMux::OnInit()
{
    Debug.Log("VpxEpc ClockMux: OnInit");
    //Owner->ClockInputMuxReady(false);
    //Sleep(10);
    //
    //  Set Input Configuration
    SetInputConfig(icLvds);
    SetInputBufferConfig(false, // AC Terminated
                         false, // Primary Term NOT disabled
                         false, // Primary Term NOT disabled
                         false, // Primary Bias off
                         false, // Secondary Bias off
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    SetOutputConfig(0, ocLvds);     //  Vpx
    SetOutputConfig(1, ocLvds);     //  Pll Secondary
    SetOutputConfig(2, ocLvds);     //  FPGA
    SetOutputConfig(3, ocLvds);     //  Pll Primary
    SetOutputConfig(4, ocLvds);     //  Pll Aux

//    SetOutputDividerSync(osPrimary);
    SetOutputDividerSync(osAux);
    //  Configure Aux (from ?)
    SetAuxOutputConfig(aeEnabled, aoFromDivider2);   // test!
    //SetAuxOutputConfig(aeDisabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    InputSource(0, 2);  OutputDivider(0, 1);   //  Vpx
    InputSource(1, 2);  OutputDivider(1, 1);   //  Pll Secondary (use 10 MHz Ref)
    InputSource(2, 2);  OutputDivider(2, 1);   //  FPGA
    InputSource(3, 0);  OutputDivider(3, 1);   //  Pll Primary   (use LClk)
    InputSource(4, 0);  OutputDivider(4, 1);   //  Pll Aux       (use LClk)

    // Set default enable state
    OutputEnabled(0, false);           //  Vpx
    OutputEnabled(1, true);            //  Pll Secondary
    OutputEnabled(2, true);            //  FPGA
    OutputEnabled(3, true);            //  Pll Primary
    OutputEnabled(4, true);            //  Pll Aux

    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    //OutputFrequencyLimit[0] = 100.0e6;      // Vpx Bus
    //OutputFrequencyLimit[2] = 250.0e6;      // FPGA


    //Owner->ClockInputMuxReady(true);
}

//---------------------------------------------------------------------------
// VpxComex_ClockMux::OnReapply()
//---------------------------------------------------------------------------

void VpxComex_ClockMux::OnReapply()
{
}

//---------------------------------------------------------------------------
// VpxComex_ClockMux::OnSetPower()
//---------------------------------------------------------------------------

void VpxComex_ClockMux::OnSetPower(bool & /*new_state*/)
{
    //  pause to allow chip to activate
    //Sleep(10);
}

//---------------------------------------------------------------------------
// VpxComex_ClockMux::UpdateSources()
//---------------------------------------------------------------------------

void VpxComex_ClockMux::UpdateSources()
{
    SourceFrequency(0, Owner->MasterT_SDev.LClkFrequency());   // LCLK In
    SourceFrequency(1, Input_ReferenceClk);        // Not used
    SourceFrequency(2, Input_ReferenceClk);        // "Crystal"
}


//===========================================================================
//  CLASS VpxComex_TriggerMux  -- Configurable Trigger Mux
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VpxComex_TriggerMux
//---------------------------------------------------------------------------

VpxComex_TriggerMux::VpxComex_TriggerMux(VpxComexDeviceMap * owner, IAddressingSpace & space,
                             Cdce18005Registers & regs)
    :  Cdce18005(space, regs), Owner(owner), Debug(true)
{
}

//---------------------------------------------------------------------------
// VpxComex_TriggerMux::OnInit()
//---------------------------------------------------------------------------

void VpxComex_TriggerMux::OnInit()
{
    Debug.Log("VpxEpc TriggerMux: OnInit");
    //Owner->ClockInputMuxReady(false);
    //Sleep(10);
    //
    //  Set Input Configuration
    SetInputConfig(icLvds);
    SetInputBufferConfig(false, // AC Terminated
                         false, // Primary Term NOT disabled
                         false, // Secondary Term NOT disabled
                         false, // Primary Bias off
                         false, // Secondary Bias off
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    SetOutputConfig(0, ocLvds);     //
    SetOutputConfig(1, ocLvds);     //
    SetOutputConfig(2, ocLvds);     //
    SetOutputConfig(3, ocLvds);     //
    SetOutputConfig(4, ocLvds);     //

    SetOutputDividerSync(osPrimary);
    //  Configure Aux (from ?)
    //SetAuxOutputConfig(aeEnabled, aoFromDivider2);   // test!
    SetAuxOutputConfig(aeDisabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    InputSource(0, 0);  OutputDivider(0, 1);   //
    InputSource(1, 0);  OutputDivider(1, 1);   //
    InputSource(2, 0);  OutputDivider(2, 1);   //
    InputSource(3, 0);  OutputDivider(3, 1);   //
    InputSource(4, 0);  OutputDivider(4, 1);   //

    // Set default enable state
    OutputEnabled(0, true);           //
    OutputEnabled(1, true);           //
    OutputEnabled(2, true);           //
    OutputEnabled(3, true);           //
    OutputEnabled(4, false);          //

    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    //OutputFrequencyLimit[0] = 100.0e6;      // Vpx Bus
    //OutputFrequencyLimit[2] = 250.0e6;      // FPGA
}

//---------------------------------------------------------------------------
// VpxComex_TriggerMux::OnReapply()
//---------------------------------------------------------------------------

void VpxComex_TriggerMux::OnReapply()
{
}

//---------------------------------------------------------------------------
// VpxComex_TriggerMux::OnSetPower()
//---------------------------------------------------------------------------

void VpxComex_TriggerMux::OnSetPower(bool & /*new_state*/)
{
    //  pause to allow chip to activate
    //Sleep(10);
}

//---------------------------------------------------------------------------
// VpxComex_TriggerMux::UpdateSources()
//---------------------------------------------------------------------------

void VpxComex_TriggerMux::UpdateSources()
{
}

//===========================================================================
//  CLASS VpxComex_MasterTimer  -- Master Time Config Controller
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxComex_MasterTimer
//---------------------------------------------------------------------------

const float  defaultLClkFreq = 100.0e6;

//---------------------------------------------------------------------------
//  constructor for class VpxComex_MasterTimer
//---------------------------------------------------------------------------

VpxComex_MasterTimer::VpxComex_MasterTimer(VpxComexDeviceMap * owner)
    :  Owner(owner), LClk_Frequency(defaultLClkFreq)
{
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::OpenHardware() -- on Open Configuration
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::OpenHardware()
{
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::ConfigureHardware() -- on Start Configuration
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::ConfigureHardware()
{
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::StreamStopHardware() -- on Stop Configuration
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::StreamStopHardware()
{
    //  turn off clocks, disable outputs?
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::LClkSource(() -- set LClk source
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::LClkSource(LClkSourceSelect source)
{
    if (source==srcJ1)
        {
        Smb1Ctl().ExternalSourceSelect(VpxComex_SmbOneControl::inJ1);
        }
    else   // srcC_Clk_I
        {
        Smb1Ctl().ExternalSourceSelect(VpxComex_SmbOneControl::inC_Clk_I);
        }

}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::PllReference() -- Retrieve reference source
//---------------------------------------------------------------------------

IX6ClockIo::IIReferenceSource VpxComex_MasterTimer::PllReference()
{
    if (Pll().ReferenceSource() == Cdce72010::refPrimary)
        return IX6ClockIo::rsExternal;
    else
        return IX6ClockIo::rsInternal;
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::PllReference() -- Select reference source
//---------------------------------------------------------------------------

void VpxComex_MasterTimer::PllReference(IX6ClockIo::IIReferenceSource src)
{
    if (src == IX6ClockIo::rsInternal)
        Pll().ReferenceSource(Cdce72010::refSecondary);
    else
        Pll().ReferenceSource(Cdce72010::refPrimary);
    //
    //  Turn off Ref VCO if not using it
    //Owner->EnableRefVco( src == rsInternal );
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::PllSource(() -- set pll source
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::PllSource(IX6ClockIo::IIClockSource source)
{
    if (source==IX6ClockIo::csInternal)
        {
        // Turn ON Pll power
        Pll().SetPllEnable(true);
        }
    else
        {
        // Turn OFF Pll power
        Pll().SetPllEnable(false);
        }
    Owner->Clock().Source(source);

}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::SetPllMasterOutputSelection(() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::SetPllMasterOutputSelection(int idx)
{
    Pll().SetMasterOutputSelection(idx);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::ToDivider() -- convert divider list idx to divider
//---------------------------------------------------------------------------

unsigned int  VpxComex_MasterTimer::ToDivider(unsigned int divider_idx)
{
    return InMux().ToDivider(divider_idx);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::VpxOutSource() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::VpxOutSource(unsigned int mux_source_select)
{
    int SrcRemap[] = { 0, 2 };
    InMux().InputSource(0, SrcRemap[mux_source_select]);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::VpxOutDivider() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::VpxOutDivider(unsigned int divide_by)
{
    InMux().OutputDivider(0, divide_by);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::VpxOutOutputEnabled() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::VpxOutOutputEnabled(bool enabled)
{
    InMux().OutputEnabled(0, enabled);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::VpxOutOutputFrequencyActual() --
//---------------------------------------------------------------------------

double  VpxComex_MasterTimer::VpxOutOutputFrequencyActual()
{
    return InMux().OutputFrequencyActual(0);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaRefSource() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::FpgaRefSource(unsigned int mux_source_select)
{
    int SrcRemap[] = { 0, 2 };
    InMux().InputSource(2, SrcRemap[mux_source_select]);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaRefDivider() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::FpgaRefDivider(unsigned int divide_by)
{
    InMux().OutputDivider(2, divide_by);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaRefOutputEnabled() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::FpgaRefOutputEnabled(bool enabled)
{
    InMux().OutputEnabled(2, enabled);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaRefOutputFrequencyActual() --
//---------------------------------------------------------------------------

double  VpxComex_MasterTimer::FpgaRefOutputFrequencyActual()
{
    return InMux().OutputFrequencyActual(2);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::GClkOutputEnabled() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer:: GClkOutputEnabled(unsigned int idx, bool enabled)
{
    Pll().OutputEnabled(1+idx, enabled);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::GClkOutputFrequency() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::GClkOutputFrequency(unsigned int idx, double frequency)
{
    Pll().OutputFrequency(1+idx, frequency);
    if (idx==0)
        FpgaFsOutputFrequency(frequency);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::GClkOutputFrequencyActual() --
//---------------------------------------------------------------------------

double  VpxComex_MasterTimer::GClkOutputFrequencyActual(unsigned int idx)
{
    return Pll().OutputFrequencyActual(1+idx);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpOutputEnabled() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer:: FpOutputEnabled(unsigned int idx, bool enabled)
{
    Pll().OutputEnabled(5+idx, enabled);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpOutputFrequency() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::FpOutputFrequency(unsigned int idx, double frequency)
{
    Pll().OutputFrequency(5+idx, frequency);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpOutputFrequencyActual() --
//---------------------------------------------------------------------------

double  VpxComex_MasterTimer::FpOutputFrequencyActual(unsigned int idx)
{
    return Pll().OutputFrequencyActual(5+idx);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaFsOutputEnabled() --
//---------------------------------------------------------------------------
//
//void  VpxComex_MasterTimer:: FpgaFsOutputEnabled(bool enabled)
//{
//    Pll().OutputEnabled(0, enabled);
//}
//
//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaFsOutputFrequency() --
//---------------------------------------------------------------------------

void  VpxComex_MasterTimer::FpgaFsOutputFrequency(double frequency)
{
    Pll().OutputFrequency(0, frequency);
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::FpgaFsOutputFrequencyActual() --
//---------------------------------------------------------------------------
//
//double  VpxComex_MasterTimer::FpgaFsOutputFrequencyActual()
//{
//    return Pll().OutputFrequencyActual(0);
//}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::InMux() --
//---------------------------------------------------------------------------

Cdce18005 & VpxComex_MasterTimer::InMux()
{
    return Owner->ClkMux();
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::Pll() --
//---------------------------------------------------------------------------

VpxComex_Pll & VpxComex_MasterTimer::Pll()
{
    return dynamic_cast<VpxComex_Pll &>(Owner->RawClockDevice().Pll());
}

//---------------------------------------------------------------------------
//  VpxComex_MasterTimer::Smb1Ctl() --
//---------------------------------------------------------------------------

VpxComex_SmbOneControl & VpxComex_MasterTimer::Smb1Ctl()
{
    return Owner->Smb1Ctl();
}

/*
//===========================================================================
//  CLASS VpxComexCw25Plugin  -- Hardware Plugin impl for NavSync CW25
//===========================================================================

class VpxComexCw25Plugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply() {}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & / *Messages* /){}
    virtual void  PGPS_OnGpsTrigger() {  Owner->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool / *gps_value* /)
                          {  / *return gps_value;* /  return Owner->IsAntenna();  }
    virtual bool    PGPS_IsLocked(bool / *gps_value* /)
                          {  / *return gps_value;* /  return Owner->IsLocked();  }
    //  (note: here we read from status register...
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return Owner->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  Owner->UartData(c);  }
    virtual char             PGPS_UartData()
                                {  return Owner->UartData();  }
    virtual void             PGPS_UartBaud(IGps::BaudRates baud)
                                {  Owner->Baud(baud);  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return Owner->Baud();  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {  Owner->GpsEnabled(state);  }
    virtual bool          PGPS_HdwEnabled()
                                {  return Owner->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates / *baud* /){}

    VpxComexCw25Plugin(X6_GPSIo * owner);
    virtual ~VpxComexCw25Plugin()
        {}

private:
     X6_GPSIo * Owner;

};

//---------------------------------------------------------------------------
// VpxComexCw25Plugin::VpxComexCw25Plugin()
//---------------------------------------------------------------------------

VpxComexCw25Plugin::VpxComexCw25Plugin(X6_GPSIo * owner)
    : Owner(owner)
{
}

//---------------------------------------------------------------------------
// VpxComexCw25Plugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void VpxComexCw25Plugin::PGPS_OnInit()
{
    //Debug.Log("CW25 On Init");
    PGPS_UartBaud(IGps::b38400);
}
*/

//===========================================================================
//  CLASS VpxComexMiniTPlugin  -- Hardware Plugin implementation for MiniT
//===========================================================================

class VpxComexMiniTPlugin : public IGps_Plugin
{
public:
    virtual void  PGPS_OnInit();
    virtual void  PGPS_OnReapply(){}
    virtual void  PGPS_OnProcessMessages(GpsMessageList & /*Messages*/){}
    virtual void  PGPS_OnGpsTrigger() {  Owner->ArmGpsTrigger();  }
    //
    //  Baseboard Config
    //   ...status
    virtual bool    PGPS_IsAntenna(bool gps_value)
                          {  return gps_value;  /*return Owner->IsAntenna(); */  }
    virtual bool    PGPS_IsLocked(bool gps_value)
                          {  return gps_value;  /* return Owner->IsLocked(); */  }
    //  (note: on the MinT we accept the gps object's value for these flags)
    //   ...uart
    virtual unsigned int     PGPS_UartFifoDepth()
                                {  return Owner->UartFifoDepth();  }
    virtual void             PGPS_UartData(char c)
                                {  Owner->UartData(c);  }
    virtual char             PGPS_UartData()
                                {  return Owner->UartData();  }
    virtual void             PGPS_UartBaud(IGps::BaudRates baud)
                                {  Owner->Baud(baud);  }
    virtual IGps::BaudRates  PGPS_UartBaud()
                                {  return Owner->Baud();  }

    //   ...gps control
    virtual void          PGPS_HdwEnabled(bool state)
                                {  Owner->GpsEnabled(state);  }
    virtual bool          PGPS_HdwEnabled()
                                {  return Owner->GpsEnabled();  }
    //
    //  Derived Gps Config
    virtual void          PGPS_GpsBaud(IGps::BaudRates /*baud*/){}

    VpxComexMiniTPlugin(X6_GPSIo * owner);
    virtual ~VpxComexMiniTPlugin()
        {}

private:
     X6_GPSIo * Owner;

    std::vector<unsigned char>  buffTimeOn;
    std::vector<unsigned char>  buffSuplementalTime;
    std::vector<unsigned char>  buffGpsOn;

};

//---------------------------------------------------------------------------
// VpxComexMiniTPlugin::VpxComexMiniTPlugin()
//---------------------------------------------------------------------------

VpxComexMiniTPlugin::VpxComexMiniTPlugin(X6_GPSIo * owner)
    : Owner(owner), buffTimeOn(9), buffSuplementalTime(6), buffGpsOn(20)
{
    buffTimeOn[0] = 0x10; buffTimeOn[1] = 0x8E; buffTimeOn[2] = 0xA5;
    buffTimeOn[3] = 0x05; buffTimeOn[4] = 0x05; buffTimeOn[5] = 0x00;
    buffTimeOn[6] = 0x00; buffTimeOn[7] = 0x10; buffTimeOn[8] = 0x03;

    buffSuplementalTime[0] = 0x10;	buffSuplementalTime[1] = 0x8E;
    buffSuplementalTime[2] = 0xAC;	buffSuplementalTime[3] = 0x00;
    buffSuplementalTime[4] = 0x10;	buffSuplementalTime[5] = 0x03;

    buffGpsOn[0]  = 0x10; buffGpsOn[1]  = 0x8E; buffGpsOn[2]  = 0x4A;
    buffGpsOn[3]  = 0x01; buffGpsOn[4]  = 0x02; buffGpsOn[5]  = 0x00;
    buffGpsOn[6]  = 0x00; buffGpsOn[7]  = 0x00; buffGpsOn[8]  = 0x00;
    buffGpsOn[9]  = 0x00; buffGpsOn[10] = 0x00; buffGpsOn[11] = 0x00;
    buffGpsOn[12] = 0x00; buffGpsOn[13] = 0x00; buffGpsOn[14] = 0x43;
    buffGpsOn[15] = 0x96; buffGpsOn[16] = 0x00; buffGpsOn[17] = 0x00;
    buffGpsOn[18] = 0x10; buffGpsOn[19] = 0x03;
}

//---------------------------------------------------------------------------
// VpxComexMiniTPlugin::PGPS_OnInit()
//---------------------------------------------------------------------------

void VpxComexMiniTPlugin::PGPS_OnInit()
{
    for(size_t i = 0; i < buffTimeOn.size(); ++i)
        PGPS_UartData(buffTimeOn[i]);
    for(size_t i = 0; i < buffSuplementalTime.size(); ++i)
        PGPS_UartData(buffSuplementalTime[i]);
    for(size_t i = 0; i < buffGpsOn.size(); ++i)
        PGPS_UartData(buffGpsOn[i]);
}


//===========================================================================
//  CLASS VpxComexGPS_Callback -- X6 GPS Hardware Callback Interface
//===========================================================================

class VpxComexGPS_Callback : public IGPS_Callback
{
public:
    VpxComexGPS_Callback(VpxComexDeviceMap * owner)
        :  Owner(owner)
        {}
    virtual ~VpxComexGPS_Callback()
        {}

    virtual void InterruptEnable(bool state)
                    {  Owner->InterruptEnable(state);  }

private:
    VpxComexDeviceMap *  Owner;
};

//===========================================================================
//  CLASS VpxComexDeviceMap  -- Register & low level support for Sbc_ComEx
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxComexDeviceMap
//---------------------------------------------------------------------------

using namespace VpxComex_MemoryMap;

Comex_ControlStatus::MapRegisters VCmxCsrRegAddrs =
    {
    mmEpcControl,        //  Bar0Control_Addr
    mmEpcStatus,         //  Bar0Status_Addr
    };

X6Info::MapRegisters VCmxInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmSubrevision,        //  not used!
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

X6Peripherals::MapRegisters VCmxPeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

X6AFE_Clock::MapRegisters VCmxClkRegAddrs =
    {
    mmPllControl,       //  AfePllControl_Addr
    mmPllSpi,           //  AfePllSpi_Addr
    mmClkControl,       //  AfeClkControl_Addr
    };

X6_ClockMux::MapRegisters VCmxClockMuxRegAddrs =
    {
    mmClockMuxSpi,       //  ClockMuxSpi_Addr;
    };

X6_ClockMux::MapRegisters VCmxTriggerMuxRegAddrs =
    {
    mmTriggerMuxSpi,       //  ClockMuxSpi_Addr;
    };

X6_GPSIo::MapRegisters VCmxGpsRegAddrs =
    {
    mmGpsUartData,         //  GpsUartData_Addr;
    mmGpsUartCtrlStatus,   //  GpsUartControl_Addr;
    mmGpsUartCtrlStatus,   //  GpsUartStatus_Addr;
    mmGpsCtrlStatus,       //  GpsStatus_Addr;
    mmGpsCtrlStatus,       //  GpsControl_Addr;
    };

VpxComex_Trigger::MapRegisters VCmxTriggerRegAddrs =
    {
    mmTriggerMode,         //  TriggerModeCfg_Addr;
    mmTriggerFrameSize,    //  FrameSizeCfg_Addr;
    mmTriggerDecimation,   //  Decimation_Addr;
    mmTriggerSwTrigger,    //  SoftwareTrigger_Addr;
    mmTriggerSyncSource,   //  ExtSyncSelect_Addr;
    };

VpxComex_Utility::MapRegisters VCmxUtilityRegAddrs =
    {
    mmVoltageStatus,       //  VoltageStatus_Addr;
    mmGpioDirection,       //  GpioDirection_Addr;
    mmGpioData,            //  GpioData_Addr;
    };

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)

static Cdce72010Registers VpxCmx_PllRegisters =
    { mmPllControl,  mmPllSpi  };

static Cdce18005Registers VpxCmx_ClockMuxRegs =
    { mmClockMuxSpi  };

static Cdce18005Registers VpxCmx_TriggerMuxRegs =
    { mmTriggerMuxSpi  };

static X6_MasterCore_I2CLinkage::MapRegisters Vpx_CmxI2CLinkRegAddrs =
    {  mmVcxoControl,      //  AfeVcxoControl_Addr
    };

static I2C_MasterCore::MapRegisters Vpx_CmxVcxoMCRegAddrs =
    {  mmVcxoMC_PrescaleLo,    //  PrescaleLo_Addr
       mmVcxoMC_PrescaleHi,    //  PrescaleHi_Addr
       mmVcxoMC_Control,       //  Control_Addr;
       mmVcxoMC_DataIo,        //  DataIo_Addr;
       mmVcxoMC_CmdStatus,     //  CmdStatus_Addr;
    };

static I2C_MasterCore::MapRegisters Vpx_CmxSmb1MCRegAddrs =
    {  mmSmb1MC_PrescaleLo,  //  PrescaleLo_Addr
       mmSmb1MC_PrescaleHi,  //  PrescaleHi_Addr
       mmSmb1MC_Control,     //  Control_Addr;
       mmSmb1MC_DataIo,      //  DataIo_Addr;
       mmSmb1MC_CmdStatus,   //  CmdStatus_Addr;
    };

static I2C_MasterCore::MapRegisters Vpx_CmxPexMCRegAddrs =
    {  mmPexMC_PrescaleLo,  //  PrescaleLo_Addr
       mmPexMC_PrescaleHi,  //  PrescaleHi_Addr
       mmPexMC_Control,     //  Control_Addr;
       mmPexMC_DataIo,      //  DataIo_Addr;
       mmPexMC_CmdStatus,   //  CmdStatus_Addr;
    };

//  Yes, this device intentionally shares the MC Addresses with the PEX device
static I2C_MasterCore::MapRegisters Vpx_CmxRep0MCRegAddrs =
    {  mmPexMC_PrescaleLo,  //  PrescaleLo_Addr
       mmPexMC_PrescaleHi,  //  PrescaleHi_Addr
       mmPexMC_Control,     //  Control_Addr;
       mmPexMC_DataIo,      //  DataIo_Addr;
       mmPexMC_CmdStatus,   //  CmdStatus_Addr;
    };

//  Yes, this device intentionally shares the MC Addresses with the PEX device
static I2C_MasterCore::MapRegisters Vpx_CmxRep1MCRegAddrs =
    {  mmPexMC_PrescaleLo,  //  PrescaleLo_Addr
       mmPexMC_PrescaleHi,  //  PrescaleHi_Addr
       mmPexMC_Control,     //  Control_Addr;
       mmPexMC_DataIo,      //  DataIo_Addr;
       mmPexMC_CmdStatus,   //  CmdStatus_Addr;
    };

static VpxFpaLoader::MapRegisters Vpx_Cmx_VsPromRegAddrs =
    {  mmVsProm_ControlAddr,  //  AddressControl_Addr;
       mmVsProm_Data,         //  Data_Addr
       mmVsProm_Opcode,       //  Opcode_Addr;
    };

const unsigned int Rep0_Subaddr = 0xE0;
const unsigned int Rep1_Subaddr = 0xE4;

//
//  Mux Device Tags
const int ClockMuxTag   = 0x0;
const int TriggerMuxTag = 0x1;

//---------------------------------------------------------------------------
//  constructor for class VpxComexDeviceMap
//---------------------------------------------------------------------------

VpxComexDeviceMap::VpxComexDeviceMap(/*VpxComex * owner*/)
    :  DeviceDriver(new VpxComexDeviceDriver),
       WB_Comex(LogicMemory, wbComex),
       Csr_SDev    (this, PortMemory, VCmxCsrRegAddrs),
       Info_SDev   (this, WB_Comex, PortMemory, VCmxInfoRegAddrs),
       Periph_SDev (this, WB_Comex, VCmxPeripheralRegAddrs),
       ClkMux_SDev (this, WB_Comex, VCmxClockMuxRegAddrs, ClockMuxTag),
       Clock_SDev  (this, WB_Comex, VCmxClkRegAddrs),
       Gps_SDev    (this, new VpxComexGPS_Callback(this), WB_Comex, VCmxGpsRegAddrs),
       Trigger_SDev(this, WB_Comex, VCmxTriggerRegAddrs),
       Util_SDev   (this, WB_Comex, VCmxUtilityRegAddrs),
       MasterT_SDev(this),
       Smb1_SDev(this, WB_Comex, Vpx_CmxSmb1MCRegAddrs),
       Pex_SDev(this, WB_Comex, Vpx_CmxPexMCRegAddrs),
       Rep_0_SDev(this, Rep0_Subaddr, WB_Comex, Vpx_CmxRep0MCRegAddrs),
       Rep_1_SDev(this, Rep1_Subaddr, WB_Comex, Vpx_CmxRep1MCRegAddrs),
       TriggerMux_SDev(this, WB_Comex, VCmxTriggerMuxRegAddrs, TriggerMuxTag),
       VsProm_SDev(this, VpxFpaLoader::romM25P64, WB_Comex, Vpx_Cmx_VsPromRegAddrs),
       InterruptStatusReg(PortMemory, mmInterruptStatus),
       InterruptEnableReg(PortMemory, mmInterruptEnable),
       //Owner(owner),
       Isr(*this), Debug(true)
{
    ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  destructor for class VpxComexDeviceMap
//---------------------------------------------------------------------------

VpxComexDeviceMap::~VpxComexDeviceMap()
{
    delete DeviceDriver;
    DeviceDriver = 0;
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t VpxComexDeviceMap::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  VpxComexDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  VpxComexDeviceMap::Name() const
{
    static const char * ModuleName[] =
        { "VpxComex" };

    return ModuleName[Info().PciLogicType()];
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  VpxComexDeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Open()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Open()
{
    if(DeviceDriver->Opened())
        return;

    DeviceDriver->Open();
    //
    //  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));
    LogicMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->LogicMemoryAddress()));

    InterruptEnable(false);
    Isr.Open();

    //
    //  After Open Behavior
    OpenHardware();

}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Close()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Close()
{
    if(!DeviceDriver->Opened())
        return;

    InterruptEnable(false);

    Isr.Close();
    DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Reset()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Reset()
{
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Start() - turn on clocks & triggers
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Start()
{
    PreconfigureHardware();
    ConfigureHardware();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Stop()  turn off clocks & triggers
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Stop()
{
    StopHardware();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  VpxComexDeviceMap::Target() const
{
    return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Led()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::Led(bool value)
{
    Csr_SDev.Led(value);
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::Led()
//---------------------------------------------------------------------------

bool  VpxComexDeviceMap::Led() const
{
    return Csr_SDev.Led();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 VpxComexDeviceMap::PeekPort(int offset)
{
    ii32 * PortBasePtr = PortMemory.Base();
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void VpxComexDeviceMap::PokePort(int offset, ii32 value)
{
    ii32 * PortBasePtr = PortMemory.Base();
    PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::PeekLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

ii32 VpxComexDeviceMap::PeekLogic(int offset)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    return LogicBasePtr[offset];
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::PokeLogic() - read a 32 bit value from logic-space
//---------------------------------------------------------------------------

void VpxComexDeviceMap::PokeLogic(int offset, ii32 value)
{
    ii32 * LogicBasePtr = LogicMemory.Base();
    LogicBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::InterruptEvent() --
//---------------------------------------------------------------------------

Event * VpxComexDeviceMap::InterruptEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::InterruptDepth() --
//---------------------------------------------------------------------------

size_t VpxComexDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::InterruptAck() --
//---------------------------------------------------------------------------

unsigned int VpxComexDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void VpxComexDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    Gps_SDev.Gps().DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void VpxComexDeviceMap::InterruptEnable(bool state)
{
    InterruptEnableReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Csr_SDev.ConstructCustomParts();
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    ClkMux_SDev.ConstructCustomParts();   // before clock
    Clock_SDev.ConstructCustomParts();
    Gps_SDev.ConstructCustomParts();
    Trigger_SDev.ConstructCustomParts();
    Util_SDev.ConstructCustomParts();
    MasterT_SDev.ConstructCustomParts();
    Smb1_SDev.ConstructCustomParts();
    Pex_SDev.ConstructCustomParts();
    TriggerMux_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::OpenHardware()
{
    //
    //  Reset the device
    //MasterReset();

    Periph_SDev.AppReset();

    // Subdevice Configuration
    Csr_SDev.OpenHardware();
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    ClkMux_SDev.OpenHardware();   // before clock
    Clock_SDev.OpenHardware();
    Gps_SDev.OpenHardware();
    Trigger_SDev.OpenHardware();
    Util_SDev.OpenHardware();
    MasterT_SDev.OpenHardware();
    Smb1_SDev.OpenHardware();
    Pex_SDev.OpenHardware();
    TriggerMux_SDev.OpenHardware();

    PokeLogic(0x8,  0x4);   // temp setting of Mux bit
    PokeLogic(0x18, 0x4);   // temp setting of Mux bit
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Csr_SDev.PreconfigureHardware();
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    ClkMux_SDev.PreconfigureHardware();   // before clock
    Clock_SDev.PreconfigureHardware();
    Gps_SDev.PreconfigureHardware();
    Trigger_SDev.PreconfigureHardware();
    Util_SDev.PreconfigureHardware();
    MasterT_SDev.PreconfigureHardware();
    Smb1_SDev.PreconfigureHardware();
    Pex_SDev.PreconfigureHardware();
    TriggerMux_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Csr_SDev.ConfigureHardware();
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    ClkMux_SDev.ConfigureHardware();   // before clock
    Clock_SDev.ConfigureHardware();
    Gps_SDev.ConfigureHardware();
    Trigger_SDev.ConfigureHardware();
    Util_SDev.ConfigureHardware();
    MasterT_SDev.ConfigureHardware();
    Smb1_SDev.ConfigureHardware();
    Pex_SDev.ConfigureHardware();
    TriggerMux_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::StopHardware() -- set hardware for 'off' condition
//---------------------------------------------------------------------------

void  VpxComexDeviceMap::StopHardware()
{
    Trigger_SDev.AllOff();    //  trigger off

    MasterT_SDev.StreamStopHardware();
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::MakeAfePllObject() --Build custom PLL object
//---------------------------------------------------------------------------

Cdce72010 *  VpxComexDeviceMap::MakeAfePllObject()
{
    //  Make our local PLL object (subdev will delete it)
    VpxComex_Pll * thePll = new VpxComex_Pll(this, WB_Comex, VpxCmx_PllRegisters );

    // Bind it to the Input Mux
    thePll->SetSourceTimer(&ClkMux_SDev.Mux());

    return thePll;
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::MakeAfeVcxoObject() --  Build custom VCO object
//---------------------------------------------------------------------------

Si570 *  VpxComexDeviceMap::MakeAfeVcxoObject(IRequires_I2C_RegIo & device)
{
    //  Make our local Vcxo object (subdev will delete it)
    return new Si570(device, Si570::tcA, kVcxoPowerOnRate );
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::MakeAfeVcxoI2CLink() --  Build custom Vco I2C Link
//---------------------------------------------------------------------------

IX6_I2C_uDevice *  VpxComexDeviceMap::MakeAfeVcxoI2CLink()
{
    //  Make our local Vcxo I2C Link (subdev will delete it)
    return new X6_MasterCore_I2CLinkage(WB_Comex, Vpx_CmxI2CLinkRegAddrs, Vpx_CmxVcxoMCRegAddrs);
}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::MakeClockMuxDevice() -- Build custom Clock Mux
//---------------------------------------------------------------------------

Cdce18005 *  VpxComexDeviceMap::MakeClockMuxDevice(int device_tag)
{
    if (device_tag==ClockMuxTag)
        {
        //  Make our local Clock Mux object (subdev will delete it)
        return new VpxComex_ClockMux(this, WB_Comex, VpxCmx_ClockMuxRegs );
        }
    else if (device_tag==TriggerMuxTag)
        {
        //  Make our local Clock Mux object (subdev will delete it)
        return new VpxComex_TriggerMux(this, WB_Comex, VpxCmx_TriggerMuxRegs );
        }
    else
        return 0;  // should never happen

}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::MakeGpsDevice() -- Build custom Gps
//---------------------------------------------------------------------------

IGps *  VpxComexDeviceMap::MakeGpsDevice()
{
    //  Make our local Gps object (subdev will delete it)
    return new Gps_MiniTPlugin(new VpxComexMiniTPlugin(&Gps_SDev));
    //return new Gps_Cw25Plugin(new VpxComexCw25Plugin(&Gps_SDev));

}

//---------------------------------------------------------------------------
//  VpxComexDeviceMap::MakeComexSignalMuxPtr() -- return ptr to Signal Mux
//---------------------------------------------------------------------------

IComexSignalMux *  VpxComexDeviceMap::MakeComexSignalMuxPtr()
{
    return & Smb1_SDev;
}

} // namespace Innovative


