// VpxComexDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef VpxComexDeviceMap_MbH
#define VpxComexDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <VpxSubDevice_Misc_Mb.h>
#include <SimpleInterrupt_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class VpxComex;
class SimpleInterrupt;
class VpxComexDeviceMap;

//==============================================================================
//  CLASS VpxComex_Pll -- Vpx Comex PLL Chip (Specialization of Cdce72010)
//==============================================================================

class VpxComex_Pll : public Cdce72010
{
public:
    VpxComex_Pll(VpxComexDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg);
    virtual ~VpxComex_Pll();

    using Cdce72010::SetPllEnable;
    
    //virtual void    OutputFrequency(unsigned int output, double freq);
    //virtual double  OutputFrequency(unsigned int output) const;

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return 100e3;   }

    virtual void  SetSourceTimer(Cdce18005 * src)
        { Src = src;  }

    using Cdce72010::SetMasterOutputSelection;

protected:
    VpxComexDeviceMap *       Owner;
    Cdce18005 *               Src;     // IMux

    virtual void  UpdateSources();

    virtual unsigned int AdcMasterOutput()
        {  return FMasterOutput;  }
    virtual unsigned int DacMasterOutput()
        {  return FMasterOutput;  }

    virtual void  SetAdcOutputRates(double adc_rate)
        {  OutputFrequency(AdcMasterOutput(), adc_rate);  }
    virtual void  SetDacOutputRates(double /*dac_rate*/)
        {}
};

//===========================================================================
//  CLASS VpxComex_ClockMux  -- Vpx Comex Clock Mux
//===========================================================================

class VpxComex_ClockMux :  public Cdce18005
{
public:
    //  Need a different place for these as we use interface...
    // enum InMuxSelect { inOnBoardOscillator=muxPrimaryIn, inFrontPanel=muxSecondaryIn, inGps=muxAuxIn };
    // enum OutMuxSelect { outXmc0Ref, outXmc1Ref, outFpga, outPllRef, outPllClk };

    VpxComex_ClockMux(VpxComexDeviceMap * owner, IAddressingSpace & space,
                             Cdce18005Registers & regs);

protected:
    virtual void        OnInit();
    virtual void        OnReapply();
    virtual void        OnSetPower(bool & new_state);
    virtual void        UpdateSources();

    VpxComexDeviceMap *   Owner;
    Trace               Debug;
};

//===========================================================================
//  CLASS VpxComex_TriggerMux  -- Vpx Comex Trigger Mux
//===========================================================================

class VpxComex_TriggerMux :  public Cdce18005
{
public:
    //  Need a different place for these as we use interface...
    // enum InMuxSelect { inOnBoardOscillator=muxPrimaryIn, inFrontPanel=muxSecondaryIn, inGps=muxAuxIn };
    // enum OutMuxSelect { outXmc0Ref, outXmc1Ref, outFpga, outPllRef, outPllClk };

    VpxComex_TriggerMux(VpxComexDeviceMap * owner, IAddressingSpace & space,
                             Cdce18005Registers & regs);

protected:
    virtual void        OnInit();
    virtual void        OnReapply();
    virtual void        OnSetPower(bool & new_state);
    virtual void        UpdateSources();

    VpxComexDeviceMap *   Owner;
    Trace               Debug;
};

//===========================================================================
//  CLASS VpxComex_MasterTimer  -- Vpx Comex Master Timer Subsystem
//===========================================================================

class VpxComex_MasterTimer : public IX6SubDevice   //,  public IComexMasterTimer
{
public:
    enum LClkSourceSelect { srcJ1, srcC_Clk_I };
    enum MuxSourceSelect { inLclk, inCrystal };

    VpxComex_MasterTimer(VpxComexDeviceMap * owner);

    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    double  ReferenceInFrequency()  {  return InMux().SourceFrequency(2); }
    double  LClkFrequency()         {  return InMux().SourceFrequency(0); }

    void    SetLClkFrequency(double frequency)
                {
                LClk_Frequency = frequency;
                InMux().SourceFrequency(0, LClk_Frequency);
                }

    IX6ClockIo::IIReferenceSource    PllReference();

    void    LClkSource(LClkSourceSelect source);
    void    PllReference(IX6ClockIo::IIReferenceSource source);
    void    PllSource(IX6ClockIo::IIClockSource source);
    void    SetPllMasterOutputSelection(int idx);

    unsigned int ToDivider(unsigned int divider_idx);

    //  InMux Signals Configuration
    void     VpxOutSource(unsigned int mux_source_select);
    void     VpxOutDivider(unsigned int divide_by);
    void     VpxOutOutputEnabled(bool enabled);
    double   VpxOutOutputFrequencyActual();

    void     FpgaRefSource(unsigned int mux_source_select);
    void     FpgaRefDivider(unsigned int divide_by);
    void     FpgaRefOutputEnabled(bool enabled);
    double   FpgaRefOutputFrequencyActual();

    //  PLL Signals Configuration
    void     GClkOutputEnabled(unsigned int idx, bool enabled);
    void     GClkOutputFrequency(unsigned int idx, double frequency);
    double   GClkOutputFrequencyActual(unsigned int idx);

    void     FpOutputEnabled(unsigned int idx, bool enabled);
    void     FpOutputFrequency(unsigned int idx, double frequency);
    double   FpOutputFrequencyActual(unsigned int idx);

protected:
    VpxComexDeviceMap *   Owner;
    double              LClk_Frequency;

    Cdce18005 &    InMux();
    VpxComex_Pll & Pll();
    class VpxComex_SmbOneControl & Smb1Ctl();
	
    //void     FpgaFsOutputEnabled(bool enabled);
    void     FpgaFsOutputFrequency(double frequency);
    //double   FpgaFsOutputFrequencyActual();
};

//===========================================================================
//  CLASS VpxComexDeviceMap  -- Register & low level support for VpxComex
//===========================================================================

class VpxComexDeviceMap :  public ISuppliesVpxComexRequirements,
                           public ISimpleInterrupt
{
    friend class VpxComex;
    friend class VpxComex_Pll;
    friend class VpxComex_ClockMux;
    friend class VpxComexGPS_Callback;


public:
    //
    //  Member Functions
    VpxComexDeviceMap(/*VpxComex * owner*/);
    virtual ~VpxComexDeviceMap();

    // Methods
    virtual size_t  InterruptDepth();
    size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);

    void            Open();
    void            Close();
    void            Reset();
    void            Start();
    void            Stop();

    std::string         Name() const;
    std::string         Revision() const;

    //  Hardware Device Access
    const IX6LogicVersionInfo &     Info() const      {  return Info_SDev;  }
    X6AFE_Clock::X6ClockIo &        Clock()           {  return Clock_SDev.Clock();  }
    IX6RawClockDevices &            RawClockDevice()  {  return Clock_SDev.RawClockDevice();  }
    IComexUtility &                 Utility()         {  return Util_SDev;  }
    VpxComex_MasterTimer &          MasterT()         {  return MasterT_SDev;  }
    Cdce18005 &                     ClkMux()          {  return ClkMux_SDev.Mux();  }
    VpxComex_SmbOneControl &        Smb1Ctl()         {  return Smb1_SDev;  }
    VpxComex_Repeater &             Repeater(int idx)
                                        {
                                        if (idx)
                                            return Rep_1_SDev;
                                        else
                                            return Rep_0_SDev;
                                        }
    IFpaLoader &                    VsProm()          {  return VsProm_SDev.VsProm();  }

    //  Subdevice Customization Handlers
    virtual Cdce72010 *        MakeAfePllObject();
    virtual Si570 *            MakeAfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *  MakeAfeVcxoI2CLink();
    virtual Cdce18005 *        MakeClockMuxDevice(int device_tag);
    virtual IGps *             MakeGpsDevice();
    virtual IComexSignalMux *  MakeComexSignalMuxPtr();

    //  Debug Support
    ii32            PeekPort(int offset);
    void            PokePort(int offset, ii32 value);
    ii32            PeekLogic(int offset);
    void            PokeLogic(int offset, ii32 value);

    // Properties
    void            Target(int target);
    int             Target() const;

    void            Led(bool value);
    bool            Led() const;

    AddressingSpace    PortMemory;
    AddressingSpace    LogicMemory;

protected:
    //
    //  Member Data
    InnovativeKernel::DualRegionIsrDeviceDriver *  DeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Comex;
    //
    //  Subdevices
    Comex_ControlStatus     Csr_SDev;
    X6Info                  Info_SDev;
    X6Peripherals           Periph_SDev;
    X6_ClockMux             ClkMux_SDev;
    X6AFE_Clock             Clock_SDev;
    X6_GPSIo                Gps_SDev;
    VpxComex_Trigger        Trigger_SDev;
    VpxComex_Utility        Util_SDev;
    VpxComex_MasterTimer    MasterT_SDev;
    VpxComex_SmbOneControl  Smb1_SDev;
    VpxComex_PexSwitch      Pex_SDev;
    VpxComex_Repeater       Rep_0_SDev;
    VpxComex_Repeater       Rep_1_SDev;
    X6_ClockMux             TriggerMux_SDev;
    VpxFpaLoader            VsProm_SDev;

    //
    //  Registers
    //  ...Interrupt Registers
    Register                            InterruptStatusReg;
    ShadowRegister                      InterruptEnableReg;

private:
    // Interface
    virtual Event *         InterruptEvent();
    virtual unsigned int    InterruptAck();
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);

    void    ConstructCustomParts();
    void    OpenHardware();
    void    PreconfigureHardware();
    void    ConfigureHardware();
    void    StopHardware();

    // Data
    SimpleInterrupt         Isr;
    Trace                   Debug;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
