// VpxComex_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef VpxComex_MbH
#define VpxComex_MbH

#include <DataTypes_Mb.h>
#include <VpxComexDeviceMap_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS VpxComex  -- VpxComex baseboard class
//===========================================================================

class VpxComex : public IFiclTarget
{

public:
    // Enumerations

    //
    //  Member Functions
    VpxComex();
    virtual ~VpxComex();

    //  Informational Methods
    size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
	bool                        LicenseCheck(unsigned int & code);

    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    int                         Target();
    void                        Target(int value);

    bool                        Led();
    void                        Led(bool value);
    bool                        GreenLed();
    void                        GreenLed(bool value);
    bool                        FrontPanelLed();
    void                        FrontPanelLed(bool value);

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    virtual void                Start();
    virtual void                Stop();
    //
    //  Properties

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    const IX6LogicVersionInfo &  Info() const;
    IComexTrigger &              Trigger();
    IComexUtility &              Utility();
    IX6_GpsIo &                  GpsCtrl();
    VpxComex_MasterTimer &       MasterT();
    VpxComex_Repeater &          Repeater(unsigned int idx);
    IFpaLoader &                 VsProm(); 

    IVpxProm &                   RawProm();

public: //Pseudo protected

    VpxComexDeviceMap * Debug() { return &Device; };
    //virtual void   Log(int type, int data);

    IX6RawClockDevices &         RawClockDevice();
    IX6ClockIo &                 Clock();
    Cdce18005 &                  ClkMux();

protected:
    //
    //  Member Data
    VpxComexDeviceMap       Device;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( VpxComex & board );
    friend AddressingSpace & PortMemorySpace( VpxComex & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( VpxComex & board )
{
    return board.Device.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( VpxComex & board )
{
    return board.Device.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

