// VpxCopDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <VpxCop_Mb.h>
#include <VpxCopDeviceMap_Mb.h>
#include <VpxCopMemoryMap_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <VpxMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace Vpx_MemoryMap;
using namespace VpxCopMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxInDevices      = 2;
const int    ChannelsPerDevice = 2;
const int    MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const float         InputMaxRate            = 160.0e6f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 16;
const unsigned int  InputMax_mV             = 1200;
const unsigned int  InputMin_mV             = -1200;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 256;

const unsigned int VpxCop_AdcSpiDevices      = MaxInDevices;
const unsigned int VpxCop_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int VpxCop_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int VpxCop_AdcCount            = MaxInChannels;  // Number of Adc Channels
const unsigned int VpxCop_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations

const unsigned int VpxCop_DacSpiDevices      = 0;   // No DACs with SPIs

const unsigned int VpxCop_VeloOutStreamCount  = 1;   // Number of Velocia Outgoing PIDs
const unsigned int VpxCop_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

const unsigned int VpxCop_Adc_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romVpxCarrier_SysInfoOffset        = 0;
const int romVpxCarrier_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romVpxCarrier_InputCoeffOffset       = 0;
const int romVpxCarrier_InputCoeffOffset_avail = 9;   // 4 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romVpxCarrier_OutputCoeffOffset       = 0;
const int romVpxCarrier_OutputCoeffOffset_avail = 0;  // none used

//===========================================================================
// Class VpxCop_AnalogInInfo
//===========================================================================

class VpxCop_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    VpxCop_AnalogInInfo() : FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~VpxCop_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//=============================================================================
//  CLASS VpxCop_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxCop_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class VpxCop_Pll
//---------------------------------------------------------------------------

VpxCop_Pll::VpxCop_Pll(VpxCopDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner), Src(0)
{
    SetMasterOutputSelection(1);  // Use Output 1 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}


//---------------------------------------------------------------------------
//  destructor for class VpxCop_Pll
//---------------------------------------------------------------------------

VpxCop_Pll::~VpxCop_Pll()
{
}

//---------------------------------------------------------------------------
//  VpxCop_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  VpxCop_Pll::OnInit()
{
    //  Device Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         true,   // Secondary Bias ON
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       false,  // Input Term NOT disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, false);  SetOutputConfig(0, ocDisabled);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds);         // FPGA CLK0
    OutputInUse(2, true);   SetOutputConfig(2, ocLvds);         // GC0
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds);         // GC1
    OutputInUse(4, true);   SetOutputConfig(4, ocLvds);         // GC2
    OutputInUse(5, true);   SetOutputConfig(5, ocLvds);         // GC3
    OutputInUse(6, true);   SetOutputConfig(6, ocLvds);         // PL Clk
    OutputInUse(7, true);   SetOutputConfig(7, ocLvds);         // FMC2 Clk
    OutputInUse(8, true);   SetOutputConfig(8, ocLvds);         // FMC3 Clk
    OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);

    //  Phase = 0
    SetPhaseAdjust(1, 1, 0);   SetPhaseAdjust(2, 1, 0);  // Set all outputs to 0 phase
    SetPhaseAdjust(3, 1, 0);   SetPhaseAdjust(4, 1, 0);
    SetPhaseAdjust(5, 1, 0);   SetPhaseAdjust(6, 1, 0);
    SetPhaseAdjust(7, 1, 0);   SetPhaseAdjust(8, 1, 0);

    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //    (Outputs 0 and 1 match preset value, so leave alone)
    //OutputFrequencyLimit[6] = 100.0e6;
    //OutputFrequencyLimit[8] = 250.0e6;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use external pin
}

//---------------------------------------------------------------------------
//  VpxCop_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  VpxCop_Pll::Vcxo()
{
    return Owner->VpxCG_SDev.Vcxo();
}

//---------------------------------------------------------------------------
//  VpxCop_Pll::OnReapply()
//---------------------------------------------------------------------------

void  VpxCop_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  VpxCop_Pll::UpdateSources() -- Set reference frequencies from IMUX
//---------------------------------------------------------------------------
void  VpxCop_Pll::UpdateSources()
{
    if (Src==0)
        return;

    ReferenceFrequency(0, Src->OutputFrequencyActual(3));
    ReferenceFrequency(1, Src->OutputFrequencyActual(1));  // same as 4-IMux Out 4
}

//===========================================================================
//  CLASS VpxCop_ClockMux  -- Configurable Input Mux
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxCop_Pll
//---------------------------------------------------------------------------

const float  LClk_DefaultFreq = 100.0e6;
const float  FmcClk1_DefaultFreq = 90.0e6;

//---------------------------------------------------------------------------
//  constructor for class VpxCop_ClockMux
//---------------------------------------------------------------------------

VpxCop_ClockMux::VpxCop_ClockMux(VpxCopDeviceMap * owner, IAddressingSpace & space,
                             Cdce18005Registers & regs)
    :  Cdce18005(space, regs), Owner(owner)
{
    SourceFrequency(0, LClk_DefaultFreq);      
    SourceFrequency(1, FmcClk1_DefaultFreq);
}

//---------------------------------------------------------------------------
// VpxCop_ClockMux::OnInit()
//---------------------------------------------------------------------------

void VpxCop_ClockMux::OnInit()
{
    Debug.Log("VpxCop ClockMux: OnInit");
    //Owner->ClockInputMuxReady(false);
    //Sleep(10);
    //
    //  Set Input Configuration
    SetInputConfig(icLvds);
    SetInputBufferConfig(true,  // DC Terminated
                         false, // Primary Term NOT disabled
                         false, // Primary Term NOT disabled
                         false, // Primary Bias off
                         false, // Secondary Bias off
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    SetOutputConfig(0, ocDisabled);   //  not used
    SetOutputConfig(1, ocLvPecl);     //  Pll Secondary
    SetOutputConfig(2, ocLvPecl);     //  FPGA
    SetOutputConfig(3, ocLvPecl);     //  Pll Primary
    SetOutputConfig(4, ocLvPecl);     //  Pll Aux

    SetOutputDividerSync(osAux);
    //  Configure Aux (from ?)
    SetAuxOutputConfig(aeDisabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    InputSource(0, 0);  OutputDivider(0, 1);   //  unused
    InputSource(1, 0);  OutputDivider(1, 1);   //  Pll Secondary (External Ref Input)
    InputSource(2, 2);  OutputDivider(2, 1);   //  FPGA          (use 10 MHz Ref)
    InputSource(3, 2);  OutputDivider(3, 1);   //  Pll Primary   (use 10 MHz Ref)
    InputSource(4, 0);  OutputDivider(4, 1);   //  Pll Aux       (External Clock Input)

    // Set default enable state
    OutputEnabled(0, false);           //  unused
    OutputEnabled(1, true);            //  Pll Secondary
    OutputEnabled(2, true);            //  FPGA
    OutputEnabled(3, true);            //  Pll Primary
    OutputEnabled(4, true);            //  Pll Aux

    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    //OutputFrequencyLimit[0] = 100.0e6;      // Vpx Bus
    //OutputFrequencyLimit[2] = 250.0e6;      // FPGA


    //Owner->ClockInputMuxReady(true);
}

//---------------------------------------------------------------------------
// VpxCop_ClockMux::OnReapply()
//---------------------------------------------------------------------------

void VpxCop_ClockMux::OnReapply()
{
}

//---------------------------------------------------------------------------
// VpxCop_ClockMux::OnSetPower()
//---------------------------------------------------------------------------

void VpxCop_ClockMux::OnSetPower(bool & /*new_state*/)
{
    //  pause to allow chip to activate
    //Sleep(10);
}

//---------------------------------------------------------------------------
// VpxCop_ClockMux::UpdateSources()
//---------------------------------------------------------------------------

void VpxCop_ClockMux::UpdateSources()
{
    SourceFrequency(0, Owner->VpxCG_SDev.LClkFrequency());     // LCLK In
    SourceFrequency(1, Owner->VpxCG_SDev.FmcClk1Frequency());  // FMC CLK1 In
    SourceFrequency(2, Input_ReferenceClk);                    // "Crystal"
}

/*
//=============================================================================
//  CLASS VpxCop_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxCop_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class VpxCop_SioPll
//---------------------------------------------------------------------------

VpxCop_SioPll::VpxCop_SioPll(VpxCopDeviceMap * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  VpxCop_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  VpxCop_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, false);    SetMultiSynthPowerDown(3, false);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);    
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(2, diNoClk);   // override since not using output

    OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
    OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, true);

    SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
    SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
    SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
    SetOutputVoltage(3, vo_2p5volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  VpxCop_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  VpxCop_SioPll::OnReapply()
{
}
*/
//===========================================================================
//  CLASS VpxCopDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxCopDeviceMap
//---------------------------------------------------------------------------

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)

static Cdce72010Registers Vpx_PllRegisters =
    { mmVPX_PllControl,  mmVPX_PllSpi  };

static Cdce18005Registers Vpx_ClockMuxRegisters =
    { mmVPX_InMuxSpi  };

static X6_Std_I2CLinkage::MapRegisters Vpx_VcoI2CLinkRegAddrs =
    {  mmVPX_VcxoControl,      //  AfeVcxoControl_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class VpxCopDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

VpxCopDeviceMap::VpxCopDeviceMap(VpxCop * owner)
    :  VpxDeviceMap(owner),
       Owner(owner)
{
    //
    //  SubDevice Customization
    VpxDeviceMap::ConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &VpxCopDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &VpxCopDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &VpxCopDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &VpxCopDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &VpxCopDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &VpxCopDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &VpxCopDeviceMap::OnAfterStreamStopHandler );


}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class VpxCopDeviceMap
//---------------------------------------------------------------------------

VpxCopDeviceMap::~VpxCopDeviceMap()
{
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OpenHardware()
{
    VpxDeviceMap::OpenHardware();
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    VpxDeviceMap::PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    VpxDeviceMap::ConfigureHardware();
    //
    //  Special Config

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    FmcCarrierRun(true);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  VpxCopDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    FmcCarrierRun(false);

    //  Base class "on stream stop" behavior
    VpxDeviceMap::StreamStopHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::MakeVCG_PllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Cdce72010 *  VpxCopDeviceMap::MakeVCG_PllObject()
{
    //  Make our local PLL object (subdev will delete it)
    VpxCop_Pll * thePll = new VpxCop_Pll(this, WB_VpxClk, Vpx_PllRegisters );

    // Bind it to the Input Mux
    thePll->SetSourceTimer(&VpxCG_SDev.InMux());

    return thePll;
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::MakeVCG_VcxoObject() --  Build custom VCO object
//---------------------------------------------------------------------------

Si570 *  VpxCopDeviceMap::MakeVCG_VcxoObject(IRequires_I2C_RegIo & device)
{
    //  Make our local Vcxo object (subdev will delete it)
    return new Si570(device, Si570::tcC, kVcxoPowerOnRate );
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::MakeVCG_VcxoI2CLink() --  Build custom Vco I2C Link
//---------------------------------------------------------------------------

IX6_I2C_uDevice *  VpxCopDeviceMap::MakeVCG_VcxoI2CLink()
{
    //  Make our local Vcxo I2C Link (subdev will delete it)
    return new X6_Std_I2CLinkage(WB_VpxClk, Vpx_VcoI2CLinkRegAddrs);
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::MakeVCG_ClkMuxObject() --  Build custom ClkMux I2C Link
//---------------------------------------------------------------------------

Cdce18005 *  VpxCopDeviceMap::MakeVCG_ClkMuxObject()
{
    //  Make our local Clock Mux link (subdev will delete it)
    return new VpxCop_ClockMux(this, WB_VpxClk, Vpx_ClockMuxRegisters );

}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------
//
//Si5338 *  VpxCopDeviceMap::MakeSioPllObject(I2CDevice & device)
//{
//    //  Make our local SioPll object (subdev will delete it)
//    return new VpxCop_SioPll(this, device);
//}
//
//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  VpxCopDeviceMap::GetVeloInStreamCount()
{
    return VpxCop_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetVeloOutStreamCount() -- Return Velo Out PIDs
//---------------------------------------------------------------------------

unsigned int  VpxCopDeviceMap::GetVeloOutStreamCount()
{
    return VpxCop_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

//unsigned int  VpxCopDeviceMap::GetVitaInStreamCount()
//{
//    return VpxCop_VitaInStreamCount;
//}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   VpxCopDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   VpxCopDeviceMap::GetPatternModeAlignment()
{
    return 8;   // standard X6 alignment of 8 words
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   VpxCopDeviceMap::GetPatternModeScaleFactor()
{
    return 8;   // standard X6 scale factor of 8 words
}

//---------------------------------------------------------------------------
//  VpxCopDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   VpxCopDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return false;  // dual memory
}


} // namespace Innovative


