// VpxCopDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef VpxCopDeviceMap_MbH
#define VpxCopDeviceMap_MbH

#include <VpxCop_Mb.h>
#include <VpxDeviceMap_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class VpxCopDeviceMap;

//==============================================================================
//  CLASS VpxCop_Pll -- RX PLL Chip (Specialization of Cdce72010)
//==============================================================================

class VpxCop_Pll : public Cdce72010
{
public:
    VpxCop_Pll(VpxCopDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg);
    virtual ~VpxCop_Pll();

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return 100e3;   }

    void  SetSourceTimer(Cdce18005 * src)
        { Src = src;  }

protected:
    VpxCopDeviceMap *   Owner;
    Cdce18005 *         Src;     // IMux

    void  UpdateSources();
    virtual unsigned int AdcMasterOutput()
        {  return FMasterOutput;  }
    virtual unsigned int DacMasterOutput()
        {  return FMasterOutput;  }

    virtual void  SetAdcOutputRates(double adc_rate)
        {  OutputFrequency(AdcMasterOutput(), adc_rate);  }
    virtual void  SetDacOutputRates(double /*dac_rate*/)
        {}

};

//===========================================================================
//  CLASS VpxCop_ClockMux  -- Vpx Comex Clock Mux
//===========================================================================

class VpxCop_ClockMux :  public Cdce18005
{
public:
    //  Need a different place for these as we use interface...
    enum InMuxSelect { inVpxLClk=muxPrimaryIn, inFmcClk1=muxSecondaryIn, inOnBoardOscillator=muxAuxIn };
    enum OutMuxSelect { outUnused, outPllSecondaryRef, outFpga, outPllPrimaryRef, outPllAux };

    VpxCop_ClockMux(VpxCopDeviceMap * owner, IAddressingSpace & space,
                             Cdce18005Registers & regs);

protected:
    virtual void        OnInit();
    virtual void        OnReapply();
    virtual void        OnSetPower(bool & new_state);
    virtual void        UpdateSources();

    VpxCopDeviceMap *   Owner;

};

//==============================================================================
//  CLASS VpxCop_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================
//
//class VpxCop_SioPll : public Si5338
//{
//public:
//    VpxCop_SioPll(VpxCopDeviceMap * owner, I2CDevice & device);
//
//    // ...Si5338 interface impl
//    virtual void            OnInit();
//    virtual void            OnReapply();
//
//protected:
//    VpxCopDeviceMap * Owner;
//};
//
//===========================================================================
//  CLASS VpxCopDeviceMap  -- Register & low level support for VpxCarrier module
//===========================================================================
class VpxCop;

class VpxCopDeviceMap :  public VpxDeviceMap

{
    friend class VpxCop;
    friend class VpxCop_Pll;
    friend class VpxCop_ClockMux;
    friend class VpxCopInputDevice;

public:
    //
    //  Member Functions
    VpxCopDeviceMap(VpxCop * owner);
    virtual ~VpxCopDeviceMap();

public: // pseudo-protected
    virtual void  OpenHardware();

    //  Subdevice Customization Handlers
    virtual Cdce72010 *         MakeVCG_PllObject();
    virtual Si570 *             MakeVCG_VcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *   MakeVCG_VcxoI2CLink();
    virtual Cdce18005 *         MakeVCG_ClkMuxObject();

    virtual IIsAVitaPacketStreamBoard *  GetBaseboardPtr();
    virtual unsigned int                 GetPatternModeAlignment();
    virtual unsigned int                 GetPatternModeScaleFactor();
    virtual bool                         GetPatternModeSingleMemoryFlag();

    //virtual Si5338 *     MakeSioPllObject(I2CDevice & device);
    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
//    virtual unsigned int GetVitaInStreamCount();

protected:
    // Fields

    // Data
    VpxCop *      Owner;
    Trace         Debug;

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
