//---------------------------------------------------------------------------
// VpxCop_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <VpxCop_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS VpxCop  -- VpxCop PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class VpxCop
//---------------------------------------------------------------------------

VpxCop::VpxCop()
    :  VpxBoard(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class VpxCop 
//---------------------------------------------------------------------------

VpxCop::~VpxCop()
{
    Close();
}

//---------------------------------------------------------------------------
//  VpxCop::AfterFiclStart() --
//---------------------------------------------------------------------------

void  VpxCop::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Fmc  1 constant VpxPll "
                    "  2 constant VpxVcxo 3 constant VpxClkMux 5  constant RtmFmc");

    engine.Evaluate(": ?VpxCop "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Fmc  VpxPll  VpxVcxo VpxClkMux RtmFmc\" "
                    "    cr  "
                    "    cr .\" ---- VpxCop Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?VpxCop  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    for (size_t i=0; i<FmcSlots(); i++)
        {
        unsigned int bd = static_cast<unsigned int>(i);
        if (FmcDaughter(bd).Linked()==false)
            continue;
        FmcDaughter(bd)->AfterFiclStart(engine);
        }

    engine.Display("VpxCop-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    VpxBoard::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  VpxCop::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  VpxCop::Do_DFetch(int dev, int addr)
{
    const int kFmc = 0;  const int kVpxPll = 1;   const int kVpxVcxo = 2;
    const int kVpxClkMux = 3;  const int kRtmFmc = 4;

    switch (dev)
        {
        case kFmc:
        default:
            return FmcDaughter(0)->Do_DFetch(dev, addr);
        case kRtmFmc:
            return FmcDaughter(1)->Do_DFetch(dev, addr);
        case kVpxPll:
            return Device.VpxCG_SDev.Pll().Port(addr);
        case kVpxVcxo:
            return Device.VpxCG_SDev.Vcxo().Port(addr);
        case kVpxClkMux:
            return Device.VpxCG_SDev.InMux().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  VpxCop::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  VpxCop::Do_DStore(int dev, int addr, int value)
{
    const int kFmc = 0;   const int kVpxPll = 1;   const int kVpxVcxo = 2;
    const int kVpxClkMux = 3;   const int kRtmFmc = 4;

    switch (dev)
        {
        case kFmc:
        default:
            FmcDaughter(0)->Do_DStore(dev, addr, value);
            break;
        case kRtmFmc:
            FmcDaughter(1)->Do_DStore(dev, addr, value);
            break;
        case kVpxPll:
            Device.VpxCG_SDev.Pll().Port(addr, value);
            break;
        case kVpxVcxo:
            Device.VpxCG_SDev.Vcxo().Port(addr, value);
            break;
        case kVpxClkMux:
            Device.VpxCG_SDev.InMux().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  VpxCop::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  VpxCop::Do_DFreq(int dev, float freq)
{
    const int kFmc = 0;   const int kVpxPll = 1;   const int kVpxVcxo = 2;
    const int kVpxClkMux = 3;  const int kRtmFmc = 4;

    switch (dev)
        {
        case kFmc:
        default:
            FmcDaughter(0)->Do_DFreq(dev, freq);
            break;
        case kRtmFmc:
            FmcDaughter(1)->Do_DFreq(dev, freq);
            break;
        case kVpxPll:
            Device.VpxCG_SDev.Pll().Frequency(freq);
            Device.VpxCG_SDev.Pll().Reapply();
            break;
        case kVpxVcxo:
            Device.VpxCG_SDev.Vcxo().Frequency(freq);
            break;
        case kVpxClkMux:
            break;
        }
}

//---------------------------------------------------------------------------
//  VpxCop::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  VpxCop::Do_DFreqActual(int dev)
{
    const int kFmc = 0;   const int kVpxPll = 1;   const int kVpxVcxo = 2;
    const int kVpxClkMux = 3;  const int kRtmFmc = 4;

    switch (dev)
        {
        case kFmc:
        default:
            return FmcDaughter(0)->Do_DFreqActual(dev);
        case kRtmFmc:
            return FmcDaughter(1)->Do_DFreqActual(dev);
        case kVpxPll:
            return static_cast<float>(Device.VpxCG_SDev.Pll().FrequencyActual());
        case kVpxVcxo:
            return static_cast<float>(Device.VpxCG_SDev.Vcxo().FrequencyActual());
        case kVpxClkMux:
            return 0.0;
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
