// VpxDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <VpxDeviceMap_Mb.h>
#include <VpxMemoryMap_Mb.h>
#include <X6DeviceDriver_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS VpxDeviceMap  -- Registers for X6 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class VpxDeviceMap
//---------------------------------------------------------------------------

const size_t  FMC_Slot_Count = 2;

//
//  IDROM Page Numbers

const unsigned int System_IdromPage = 0;
const unsigned int AdcCal_IdromPage = 1;
const unsigned int DacCal_IdromPage = 2;
const unsigned int User_IdromPage   = 3;

//
//  Subdevice Register Structures

using namespace FmcCarrier_MemoryMap;
using namespace Vpx_MemoryMap;

namespace
{

VpxCopFpaLoader::MapRegisters VpxFpaRegAddrs =
    {
    mmFpa_Address,        //  FpaAddress_Addr
    mmFpa_Data,           //  FpaData_Addr
    mmFpa_Command,        //  FpaCommand_Addr
    };

Vpx_ClockGen::MapRegisters VpxClkGenRegAddrs =
    {
    mmVPX_PllControl,       //  VpxPllControl_Addr
    mmVPX_PllSpi,           //  VpxPllSpi_Addr
//    mmVpx_VcxoControl,    //  AfeVcxoControl_Addr
    mmVPX_InMuxControl,     //  AfInMuxControl_Addr
    mmVPX_InMuxSpi,         //  VpxInMuxSpi_Addr
    };

} // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class VpxDeviceMap
//---------------------------------------------------------------------------

VpxDeviceMap::VpxDeviceMap(IStreamDeviceEvents * owner)
    :  FmcCarrierAdvDeviceMap(owner, FMC_Slot_Count),
       WB_VpxClk          (LogicMemory, wbVpxClk),
       VFpa_SDev          (this, WB_Loader, VpxFpaRegAddrs),
       VpxCG_SDev         (this, WB_VpxClk, VpxClkGenRegAddrs)
{
}

//---------------------------------------------------------------------------
//  destructor for class VpxDeviceMap
//---------------------------------------------------------------------------

VpxDeviceMap::~VpxDeviceMap()
{
}

//---------------------------------------------------------------------------
//  VpxDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  VpxDeviceMap::ConstructCustomParts()
{
    FmcCarrierAdvDeviceMap::ConstructCustomParts();

    VFpa_SDev.ConstructCustomParts();
    VpxCG_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  VpxDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  VpxDeviceMap::OpenHardware()
{
    FmcCarrierAdvDeviceMap::OpenHardware();

    VFpa_SDev.OpenHardware();
    VpxCG_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  VpxDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  VpxDeviceMap::PreconfigureHardware()
{
    FmcCarrierAdvDeviceMap::PreconfigureHardware();

    VFpa_SDev.PreconfigureHardware();
    VpxCG_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  VpxDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  VpxDeviceMap::ConfigureHardware()
{
    FmcCarrierAdvDeviceMap::ConfigureHardware();

    VFpa_SDev.ConfigureHardware();
    VpxCG_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  VpxDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  VpxDeviceMap::StreamStopHardware()
{
    FmcCarrierAdvDeviceMap::StreamStopHardware();
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


} // namespace Innovative


