// VpxSubDevice_Misc_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef VpxSubDevice_Misc_MbH
#define VpxSubDevice_Misc_MbH

#include <X6Support_Mb.h>
#include <VpxSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <FpaLoadThread_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS VpxFpaLoader -- X6 FPA Loader "SubDevice"
//===========================================================================

class VpxFpaLoader : public IX6SubDevice
{
public:

    enum RomType { romM25P64, romS25FL128S };

    //=======================================================================
    //  CLASS VpxFpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================

    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    //=======================================================================
    //  CLASS VpxFpaLoader::VpxFpaChip  -- Chip Interface
    //=======================================================================
    //  M25P64  ROM

    class VpxFpaChip : public IVpxFpaChip
    {
    public:
        VpxFpaChip(VpxFpaLoader * owner): Owner(owner)
            {  FTotalSectors = 128;  }
        virtual ~VpxFpaChip()
            {  }
        virtual unsigned int  SectorSize(unsigned int /*sector*/) const  {  return 0x10000;  }
        virtual unsigned int  Sectors() const   {  return FTotalSectors >> 1;  }
        virtual unsigned int  StartSector() const {  return 0;  }
        virtual unsigned int  GoldenImageStartSector() const
                    {  return StartSector() + Sectors();  }
        virtual unsigned int  GoldenImageSectors() const
                    {  return Sectors();  }

        virtual bool Initialize();

        virtual unsigned int  Read32(unsigned int addr);
        virtual void          Write32(unsigned int addr, unsigned int value);

        virtual void          EraseSector(unsigned int sector);

    private:
        VpxFpaLoader * Owner;
    };

    friend class VpxFpaChip;

    //=======================================================================
    //  CLASS VpxFpaLoader::K7FpaChip  -- Chip Interface
    //=======================================================================
    //  romS25FL128S  ROM  - so far identical to M25P64 ROM aside from size

    class K7FpaChip : public VpxFpaChip
    {
    public:
        K7FpaChip(VpxFpaLoader * owner): VpxFpaChip(owner)
            {  FTotalSectors = 512;  }
        virtual ~K7FpaChip()
            {  }
    };

    //
    //  Member Functions
    VpxFpaLoader(IRequires_Nothing * mdc, RomType which_rom, IAddressingSpace & space, MapRegisters & regs);
    virtual ~VpxFpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    //==============================================================================
    //  CLASS M25P64Prom  -- "Raw" access to PROM
    //==============================================================================

    class M25P64Prom : public IVpxProm
    {
    public:
        M25P64Prom(VpxFpaLoader * owner) : Owner(owner)
            {}
        virtual ~M25P64Prom()
            {}

        virtual void          EraseSector(unsigned int sector)
                                {  Owner->EraseSector(sector);  }
        virtual unsigned int  Read32(unsigned int addr)
                                {  return Owner->Read32(addr);  }
        virtual void          Write32(unsigned int addr, unsigned int value)
                                {  Owner->Write32(addr, value);  }
        virtual unsigned int  Rdsr()
                                {  return Owner->Rdsr();  }
        virtual unsigned int  Rdid()
                                {  return Owner->Rdid();  }

        //---------------------------
        //
        //  Flash Primitives
        //  ...write operation
        virtual void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
                                {  Owner->FlashOp(opcode, addr, data);  }
        //  ...read operation
        virtual unsigned int  FlashOp(unsigned int opcode, unsigned int addr)
                                {  return Owner->FlashOp(opcode, addr);  }
        //  ...command operation
        virtual void          FlashOp(unsigned int opcode)
                                {  Owner->FlashOp(opcode);  }

    private:
        VpxFpaLoader * Owner;
    };
    friend class M25P64Prom;

    M25P64Prom &  Raw()
            {  return RawProm;  }

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxFpaLoader::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    IVpxFpaChip *       Chip;
    M25P64Prom          RawProm;    // debug interface
    FpaLoadThread       Thread;
    StopWatch           Rtc;


    //  ROM Info Commands
    void          EraseSector(unsigned int sector);
    unsigned int  Read32(unsigned int addr);
    void          Write32(unsigned int addr, unsigned int value);
    unsigned int  Rdsr();
    unsigned int  Rdid();

    //---------------------------
    //
    //  Flash Primitives
    void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data);   // write operation
    unsigned int  FlashOp(unsigned int opcode, unsigned int addr);  // read op
    void          FlashOp(unsigned int opcode);  // command op
    //
    //  Internal Flash Primitives
    void  Flash_Ready();
    void  Flash_ReadValid();
    void  Flash_NotBusy();
    void  Flash_QuickNotBusy();

    void  ReleaseBlockProtection();
    void  ByteProgram(unsigned int addr, unsigned int byte);

    IVpxFpaChip * MakeChipObject(RomType which_rom);

};

//===========================================================================
//  CLASS VpxApplication -- Vpx device information "SubDevice"
//===========================================================================

class VpxApplication : public IX6SubDevice, public IVpxApplication
{
public:
    //=======================================================================
    //  CLASS VpxApplication::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   VpxAppControlStatus_Addr;
    };

    VpxApplication(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~VpxApplication();

    //  Methods
	virtual bool  LoopbackEnabled();
	virtual void  LoopbackEnabled(bool state);

	virtual bool  TestGenEnabled();
	virtual void  TestGenEnabled(bool state);

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxApplication::ControlStatusRegister  -- Vpx Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ControlStatusRegister : public Register
    {
        typedef Register inherited;

    public:
		RegisterBit  Loopback;
		RegisterBit  TestGen;

    public:
		ControlStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
			   Loopback(*this, 0), TestGen(*this, 1)
            {
            }
    };


private:
	IRequires_Nothing *     MDC;
    ControlStatusRegister   ControlStatusReg;

};

//===========================================================================
//  CLASS VpxDdrMemory -- Vpx DDR3 memory "SubDevice"
//===========================================================================

class VpxDdrMemory : public IX6SubDevice
{
public:
    struct DdrData
    {
		unsigned int Word[4];   //  128 bit memory

		DdrData(unsigned int w0=0, unsigned int w1=0, unsigned int w2=0, unsigned int w3=0)
			{  Word[0]=w0; Word[1]=w1;  Word[2]=w2; Word[3]=w3;  }
	};

    //=======================================================================
    //  CLASS VpxDdrMemory::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   VpxDdrMemControlStatus_Addr;
        unsigned int   VpxDdrMemDataMask_Addr;
        unsigned int   VpxDdrMemWriteData0_Addr;
        unsigned int   VpxDdrMemWriteData1_Addr;
        unsigned int   VpxDdrMemWriteData2_Addr;
        unsigned int   VpxDdrMemWriteData3_Addr;
        unsigned int   VpxDdrMemReadData0_Addr;
        unsigned int   VpxDdrMemReadData1_Addr;
        unsigned int   VpxDdrMemReadData2_Addr;
        unsigned int   VpxDdrMemReadData3_Addr;
    };

    VpxDdrMemory(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
	virtual ~VpxDdrMemory();

	unsigned int AddressSpaceRange() {  return 1<<25;  }

    //  Methods
    void  Read(unsigned int address, DdrData & data);
    void  Write(unsigned int address, const DdrData & data);

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VpxMemory::DdrControlRegister  -- Vpx Control/Status reg
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class DdrControlRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup  Address;
		RegisterBit       RWSelect;

	public:
		DdrControlRegister( IAddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Address(*this, 0, 25),  RWSelect(*this, 28)
			{
			}
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VpxMemory::DdrStatusRegister  -- Vpx Control/Status reg
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class DdrStatusRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBit       Ready;
		ReadOnlyRegisterBit       DataValid;
		ReadOnlyRegisterBit       InitDone;

	public:
		DdrStatusRegister( IAddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Ready(*this, 29),  DataValid(*this, 30),  InitDone(*this, 31)
			{
			}
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxDdrMemory::DdrMaskRegister  -- Vpx Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class DdrMaskRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Mask;

    public:
        DdrMaskRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Mask(*this, 0, 16)
            {
            }
	};


private:
    IRequires_Nothing *  MDC;
	DdrControlRegister   DdrControlReg;
	DdrStatusRegister    DdrStatusReg;
    DdrMaskRegister      DdrMaskReg;
    Register             DdrWrite0_Reg;
    Register             DdrWrite1_Reg;
    Register             DdrWrite2_Reg;
    Register             DdrWrite3_Reg;
	Register             DdrRead0_Reg;
    Register             DdrRead1_Reg;
    Register             DdrRead2_Reg;
    Register             DdrRead3_Reg;

	void  WriteReady();
	void  ReadReady();
	void  ReadDataAvailable();

};

//===========================================================================
//  CLASS VpxQdrMemory -- Vpx QDR memory "SubDevice"
//===========================================================================

class VpxQdrMemory : public IX6SubDevice
{
public:
    struct QdrData
    {
        unsigned int Word[3];   //  72 bit memory

		QdrData(unsigned int w0=0, unsigned int w1=0, unsigned int w2=0)
			{  Word[0]=w0; Word[1]=w1;  Word[2]=w2;  Word[2] &= 0xFF; }
	};

    //=======================================================================
    //  CLASS VpxQdrMemory::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   VpxQdrMemControlStatus_Addr;
        unsigned int   VpxQdrMemDataMask_Addr;
        unsigned int   VpxQdrMemWriteData0_Addr;
        unsigned int   VpxQdrMemWriteData1_Addr;
        unsigned int   VpxQdrMemWriteData2_Addr;
        unsigned int   VpxQdrMemReadCSR_Addr;
		unsigned int   VpxQdrMemReadData0_Addr;
        unsigned int   VpxQdrMemReadData1_Addr;
        unsigned int   VpxQdrMemReadData2_Addr;
    };

    VpxQdrMemory(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~VpxQdrMemory();

	unsigned int AddressSpaceRange() {  return 1<<19;  }

	//  Methods
    void  Read(unsigned int address, QdrData & data);
    void  Write(unsigned int address, const QdrData & data);

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxMemory::QdrControlRegister  -- Vpx Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QdrControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Address;
        RegisterBit       Ready;
        RegisterBit       InitDone;

    public:
        QdrControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Address(*this, 0, 19),
               Ready(*this, 29),  InitDone(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxQdrMemory::QdrMaskRegister  -- Vpx Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QdrMaskRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Mask;

    public:
        QdrMaskRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Mask(*this, 0, 8)
            {
            }
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxMemory::QdrReadCsrRegister  -- Vpx Read Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QdrReadCsrRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Address;
        RegisterBit       Ready;
        RegisterBit       DataValid;

    public:
        QdrReadCsrRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Address(*this, 0, 19),
			   Ready(*this, 29),  DataValid(*this, 30)
            {
            }
    };

private:
    IRequires_Nothing *  MDC;
    QdrControlRegister   QdrControlReg;
    QdrMaskRegister      QdrMaskReg;
    Register             QdrWrite0_Reg;
    Register             QdrWrite1_Reg;
    Register             QdrWrite2_Reg;
	QdrReadCsrRegister   QdrReadCSR_Reg;
	Register             QdrRead0_Reg;
    Register             QdrRead1_Reg;
	Register             QdrRead2_Reg;

	void  WriteReady();
	void  ReadReady();
	void  ReadDataAvailable();

};

//===========================================================================
//  CLASS VpxTrigger -- Vpx  trigger "SubDevice"
//===========================================================================

class VpxTrigger : public IX6SubDevice
{
public:

    //=======================================================================
    //  CLASS VpxTrigger::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   VpxTrigger_Control_Addr;
        unsigned int   VpxTrigger_ItfOtf_Addr;
        unsigned int   VpxTrigger_PlLt_Addr;
        unsigned int   VpxTrigger_PlMd_Addr;
        unsigned int   VpxTrigger_PpsGt_Addr;
    };

    VpxTrigger(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~VpxTrigger();

    //  Methods

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxMemory::TriggerControlRegister  -- Vpx Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class TriggerControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       LdlOutputEnable;
        RegisterBit       LdrOutputEnable;

    public:
        TriggerControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               LdlOutputEnable(*this, 4),  LdrOutputEnable(*this, 12)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxTrigger::ItfOtfRegister  -- ITF/OTF reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ItfOtfRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Itf;
        RegisterBitGroup  Otf;

    public:
        ItfOtfRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Itf(*this, 0, 2),   Otf(*this, 4, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxTrigger::PlLtRegister  -- PL/OT reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class PlLtRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  LocalTrigger;
        RegisterBit       LocalTriggerEnable;
        RegisterBit       PayloadLocalTrigger;
        RegisterBit       PayloadLocalTriggerEnable;

    public:
        PlLtRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               LocalTrigger(*this, 0, 2),   LocalTriggerEnable(*this, 4),
               PayloadLocalTrigger(*this, 8),   PayloadLocalTriggerEnable(*this, 12)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxTrigger::PlMdRegister  -- PL/MD reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class PlMdRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  ModeTrigger;
        RegisterBit       ModeTriggerEnable;
        RegisterBitGroup  PayloadModeTrigger;
        RegisterBit       PayloadModeTriggerEnable;

    public:
        PlMdRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ModeTrigger(*this, 0, 2),   ModeTriggerEnable(*this, 4),
               PayloadModeTrigger(*this, 8, 2),   PayloadModeTriggerEnable(*this, 12)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxMemory::GlobalTriggerRegister  -- Global Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class GlobalTriggerRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Pps;
        RegisterBitGroup  GlobalTriggerRx;
        RegisterBitGroup  GlobalTriggerTx;

    public:
        GlobalTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Pps(*this, 0),
               GlobalTriggerRx(*this, 4, 2),  GlobalTriggerTx(*this, 8, 4)
            {
            }
    };

private:
    IRequires_Nothing *  MDC;
    TriggerControlRegister   TriggerControlReg;
    ItfOtfRegister           ItfOtfReg;
    PlLtRegister             PlLtReg;
    PlMdRegister             PlMdReg;
    GlobalTriggerRegister    GlobalReg;

};

//===========================================================================
//  CLASS Comex_ControlStatus -- Comex CSR "SubDevice"
//===========================================================================

class Comex_ControlStatus : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS Comex_ControlStatus::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Bar0Control_Addr;
        unsigned int   Bar0Status_Addr;
    };

    Comex_ControlStatus(IRequires_Nothing * mdc, IAddressingSpace & bar_0space, MapRegisters & regs);
    virtual ~Comex_ControlStatus();

    virtual bool            Led() const;
    virtual void            Led(bool state);

    //  Status
    virtual bool            Slot64() const;

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ComexControlRegister  --  EPC Control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ComexControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       Led;

    public:
        ComexControlRegister( IAddressingSpace & space, unsigned int offset )
            :  inherited(space, offset),
               Led(*this, 1)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ComexStatusRegister  --  EPC Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ComexStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit       Slot64;

    public:
        ComexStatusRegister( IAddressingSpace & space, unsigned int offset )
            :  inherited(space, offset),
               Slot64(*this, 0)
            {
            }
    };

private:
    IRequires_Nothing *  MDC;
    ComexControlRegister   ControlReg;
    ComexStatusRegister    StatusReg;

};

//===========================================================================
//  CLASS VpxComex_SmbOneControl  -- Vpx SMB 1 Subsystem
//===========================================================================

class VpxComex_SmbOneControl : public IX6SubDevice, public IComexSignalMux
{
public:
    VpxComex_SmbOneControl(IRequires_Nothing * mdc,
                           IAddressingSpace & space, I2C_MasterCore::MapRegisters & regs);

    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    void  ExternalSourceSelect( Mux0SourceSelect which );
    void  ExternalPpsSourceSelect( Mux3SourceSelect which );
    void  ExternalTriggerSelect( Mux7SourceSelect which );

    unsigned int Port(unsigned int addr)
            {  return Smb1.I2C_Reg(addr);  }
    void  Port(unsigned int addr, unsigned int data)
            {  Smb1.I2C_Reg(addr, data);  }

protected:
    IRequires_Nothing *  MDC;

    I2C_MasterCore        Smb1;

    MemoryBitManipulator  Smb1Mux;

};

//===========================================================================
//  CLASS VpxComex_Trigger -- Vpx Comex Trigger "SubDevice"
//===========================================================================

class VpxComex_Trigger : public IX6SubDevice, public IComexTrigger
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Trigger::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   TriggerModeCfg_Addr;
        unsigned int   FrameSizeCfg_Addr;
        unsigned int   Decimation_Addr;
        unsigned int   SoftwareTrigger_Addr;
        unsigned int   ExtSyncSelect_Addr;
    };

    VpxComex_Trigger(IRequires_ComexSignalMux * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~VpxComex_Trigger();

    //
    //  Methods
    void AllOff( );

    //  IComexTrigger Implementation
    virtual void          SoftwareTrigger( bool state );
    virtual bool          SoftwareTrigger() const;

    virtual void          FramedTriggerSize(unsigned int samples);
    virtual unsigned int  FramedTriggerSize() const;

    virtual void           Decimation(unsigned int samples);
    virtual unsigned int   Decimation() const;

    //  ...Trigger Modes
    virtual void  FramedTriggerEnabled(bool state);
    virtual bool  FramedTriggerEnabled() const;

    virtual void  EdgeTrigger(bool state);
    virtual bool  EdgeTrigger() const;

    virtual void  ExternalTrigger(bool state);
    virtual bool  ExternalTrigger() const;

    //  ...Trigger Routing
    virtual ExtSyncOptions  ExternalSyncSelection() const;
    virtual void            ExternalSyncSelection(ExtSyncOptions option);

    virtual void  LTrigSource(LTrigSourceSelect source);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Trigger::SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit  SwTrigger;

    public:
        SoftwareTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               SwTrigger(*this, 0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Trigger::ExtSyncSelectRegister  -- Select ext trigger source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ExtSyncSelectRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  ExtSyncOption;

    public:
        ExtSyncSelectRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ExtSyncOption(*this, 0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Trigger::TriggerModeRegister -- Configure ADC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerModeRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       RisingEdge;
        RegisterBit       FramedMode;
        RegisterBit       ExternalTrigger;

    public:
        TriggerModeRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               RisingEdge(*this, 0),
               FramedMode(*this, 1),     ExternalTrigger(*this, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Trigger::TriggerFrameRegister -- Configure  Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerFrameRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  FrameSize;

    public:
        TriggerFrameRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               FrameSize(*this, 0, 24)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Trigger::DecimationRegister  --  Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        DecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_ComexSignalMux *  MDC;
    IComexSignalMux *           SigMux;  // do NOT delete

    TriggerModeRegister       TriggerModeReg;
    TriggerFrameRegister      TriggerFrameReg;
    DecimationRegister        DecimationReg;
    SoftwareTriggerRegister   SwTriggerReg;
    ExtSyncSelectRegister     ExtSyncSelectReg;

};

//===========================================================================
//  CLASS VpxComex_Utility -- Vpx Comex Utility "SubDevice"
//===========================================================================

class VpxComex_Utility : public IX6SubDevice, public IComexUtility
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Utility::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   VoltageStatus_Addr;
        unsigned int   GpioDirection_Addr;
        unsigned int   GpioData_Addr;
    };

    VpxComex_Utility(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~VpxComex_Utility();

    //
    //  Methods

    //  IComexUtility Implementation
    //  ...VoltageStatus
    virtual bool          V2_5Status() const;
    virtual bool          V1_0Status() const;
    virtual bool          V1_2Status() const;
    virtual bool          V1_5Status() const;
    virtual bool          Ddr3Status() const;
    //   ...Gpio
    virtual void          GpioDirection(bool isOutput);
    virtual bool          Gpio() const;
    virtual void          Gpio(bool data);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxComex_Utility::VoltageStatusRegister  -- SW Utility Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VoltageStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit  V2_5Status;
        ReadOnlyRegisterBit  V1_0Status;
        ReadOnlyRegisterBit  V1_2Status;
        ReadOnlyRegisterBit  V1_5Status;
        ReadOnlyRegisterBit  Ddr3Status;

    public:
        VoltageStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               V2_5Status(*this, 0),  V1_0Status(*this, 1),
               V1_2Status(*this, 2),  V1_5Status(*this, 3),
               Ddr3Status(*this, 4)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;

    VoltageStatusRegister     VoltageStatusReg;
    Register                  GpioDirectionReg;
    Register                  GpioDataReg;

};

//===========================================================================
//  CLASS VpxComex_PexSwitch  -- Vpx Comex PEX Switch Subsystem
//===========================================================================

class VpxComex_PexSwitch : public IX6SubDevice
{
public:
    enum PexCCODE { kCsrBlockIo = 0x43 };
    enum CsrCMD { kDWordWrite = 0x0F, kDWordRead=0x1F };

    VpxComex_PexSwitch(IRequires_Nothing * mdc,
                           IAddressingSpace & space, I2C_MasterCore::MapRegisters & regs);

    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    unsigned int CsrPort(unsigned int addr);
    void  CsrPort(unsigned int addr, unsigned int data);

protected:
    IRequires_Nothing *  MDC;
    I2C_MasterCore       Pex;

};

//===========================================================================
//  CLASS VpxComex_Repeater  -- Vpx Comex PCIE Repeater Subsystem
//===========================================================================

class VpxComex_Repeater : public IX6SubDevice
{
public:
    VpxComex_Repeater(IRequires_Nothing * mdc, unsigned int SubAddr,
                      IAddressingSpace & space,
                      I2C_MasterCore::MapRegisters & regs);

    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    bool  Write(std::vector<int> & data);
    bool  Read(std::vector<int> & data);

protected:
    IRequires_Nothing *  MDC;
    I2C_MasterCore       Repeater;

};

//===========================================================================
//  CLASS Vpx_ClockGen  -- Vpx Baseboard ClockGen Subsystem
//===========================================================================

class Vpx_ClockGen : public IX6SubDevice   //,  public IVpx_ClockGen
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Vpx_ClockGen::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   VpxPllControl_Addr;
        unsigned int   VpxPllSpi_Addr;
        unsigned int   VpxInMuxControl_Addr;
        unsigned int   VpxInMuxSpi_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Vpx_ClockGen::VpxClkGenIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VpxClkGenIo : public IVpxClkGenIo
    {
    public:
        VpxClkGenIo(Vpx_ClockGen * owner): Owner(owner)
            {  }
        virtual ~VpxClkGenIo()
            {  }
        //
        //  IVpxClkGenIo Interface Functions
        virtual void                Source(VpxPllClockSource src);
        virtual VpxPllClockSource   Source() const;

        virtual void                ExternalClkSelect(VpxExtClockSource src);
        virtual VpxExtClockSource   ExternalClkSelect() const;

        virtual void                Reference(VpxPllRefSource src);
        virtual VpxPllRefSource     Reference() const;

        virtual void                ReferenceFrequency(VpxExtClockSource clk, double value);
        virtual double              ReferenceFrequency(VpxExtClockSource clk) const;

        virtual bool                Locked()  const;

        virtual void                SetPllMasterOutputSelection(int idx);

        //  InMux Signals Configuration
        virtual void     FpgaClk1Source(unsigned int mux_source_select);
        virtual void     FpgaClk1Divider(unsigned int divide_by);
        virtual void     FpgaClk1OutputEnabled(bool enabled);
        virtual double   FpgaClk1OutputFrequencyActual();

        virtual unsigned int ToDivider(unsigned int divider_idx);

        //  PLL Output Signals Configuration
        virtual void     FpgaClk0OutputEnabled(bool enabled);
        virtual void     FpgaClk0OutputFrequency(double frequency);
        virtual double   FpgaClk0OutputFrequencyActual();

        virtual void     GClkOutputEnabled(unsigned int idx, bool enabled);
        virtual void     GClkOutputFrequency(unsigned int idx, double frequency);
        virtual double   GClkOutputFrequencyActual(unsigned int idx);

        virtual void     PlClkOutputEnabled(bool enabled);
        virtual void     PlClkOutputFrequency(double frequency);
        virtual double   PlClkOutputFrequencyActual();

        virtual void     FmcClk2OutputEnabled(bool enabled);
        virtual void     FmcClk2OutputFrequency(double frequency);
        virtual double   FmcClk2OutputFrequencyActual();

        virtual void     FmcClk3OutputEnabled(bool enabled);
        virtual void     FmcClk3OutputFrequency(double frequency);
        virtual double   FmcClk3OutputFrequencyActual();

    protected:
        Vpx_ClockGen *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Vpx_ClockGen::VpxRawClkGenDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VpxRawClkGenDevices : public IVpxRawClkGenDevices
    {
    public:
        VpxRawClkGenDevices(Vpx_ClockGen * owner): Owner(owner)
            {  }
        virtual ~VpxRawClkGenDevices()
            {  }

        virtual Si570 &       Vcxo()     {  return Owner->Vcxo();  }
        virtual Cdce72010 &   Pll()      {  return Owner->Pll();   }
        virtual Cdce18005 &   InMux()    {  return Owner->InMux(); }

    protected:
        Vpx_ClockGen *  Owner;
    };

    friend class VpxClkGenIo;
    friend class VpxRawClkGenDevices;

    VpxClkGenIo &            ClockGen() {  return ClockGenIo;  }
    VpxRawClkGenDevices &    RawClockGenDevice() {  return ClockGenDevs;  }

    Si570 &      Vcxo();
    Cdce18005 &  InMux();
    Cdce72010 &  Pll();


    Vpx_ClockGen(IRequires_VpxClkGenParts * mdc, IAddressingSpace & space, MapRegisters & regs );
    ~Vpx_ClockGen();

    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    void  EnableVcxo(bool state);

    double  LClkFrequency()         {  return InMux().SourceFrequency(0); }
    double  FmcClk1Frequency()      {  return InMux().SourceFrequency(1); }
    double  ReferenceInFrequency()  {  return InMux().SourceFrequency(2); }

    void    SetLClkFrequency(double frequency)
                {
                LClk_Frequency = frequency;
                InMux().SourceFrequency(0, LClk_Frequency);
                }
    void    SetFmcClk1Frequency(double frequency)
                {
                FmcClk1_Frequency = frequency;
                InMux().SourceFrequency(1, FmcClk1_Frequency);
                }


protected:
    IRequires_VpxClkGenParts *  MDC;
    //Register         PllControl;     // Implemented inside CDC Pll
    //Register         PllSpi;
    //ClockCtrlRegister  ClockCtrlReg;
    //Register         InMuxControl;
    //Register         InMuxSpi;      // Implemented inside CDC Mux

    IX6_I2C_uDevice *  I2C_uDev;
    Si570 *            VcxoDev;

    Cdce72010 *        PllDev;
    Cdce18005 *        InMuxDev;

    double               LClk_Frequency;
    double               FmcClk1_Frequency;

    VpxClkGenIo          ClockGenIo;
    VpxRawClkGenDevices  ClockGenDevs;

};

//===========================================================================
//  CLASS VpxCopFpaLoader -- VpxCop FPA Loader "SubDevice"
//===========================================================================

class VpxCopFpaLoader : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS VpxCopFpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   FpaAddress_Addr;
        unsigned int   FpaData_Addr;
        unsigned int   FpaCommand_Addr;
    };

    //=======================================================================
    //  CLASS VpxCopFpaLoader::VpxFpaCpld  -- CPLD Hardware Interface
    //=======================================================================
    class VpxFpaCpld : public IX6FpaCpld
    {
    public:
        VpxFpaCpld(VpxCopFpaLoader * owner): Owner(owner)
            {  }
        virtual ~VpxFpaCpld()
            {  }

        // Methods
        virtual void  CPLD_Write(unsigned int addr, unsigned int data)
                {  Owner->CPLD_Write(addr, data);  }
        virtual unsigned int  CPLD_Read(unsigned int addr)
                {  return  Owner->CPLD_Read(addr);  }
        virtual void  CPLD_GoldenImageActive(bool state)
                {  Owner->CPLD_GoldenImageActive(state);  }
        virtual void  CPLD_UpdateStatus()
                {  Owner->CPLD_UpdateStatus();  }
        virtual unsigned int  CPLD_ReadPowerStatus()
                {  return Owner->CPLD_ReadPowerStatus();  }

        virtual void  CPLD_Address(unsigned int addr)
                {  Owner->CPLD_Address(addr);  }
        virtual void  CPLD_Data(unsigned int data)
                {  Owner->CPLD_Data(data);  }
        virtual unsigned int  CPLD_Data()
                {  return Owner->CPLD_Data();  }
        virtual void  CPLD_Command(unsigned int code)
                {  Owner->CPLD_Command(code);  }

        virtual unsigned int  CPLD_Version()
                {  return Owner->CPLD_Version();  }
        virtual unsigned int  CPLD_FlashType()
                {  return Owner->CPLD_FlashType();  }
        virtual bool  CPLD_GoldenImageActive()
                {  return Owner->CPLD_GoldenImageActive();  }
        virtual bool  CPLD_GoldenImageBooted()
                {  return Owner->CPLD_GoldenImageBooted();  }
        virtual bool  CPLD_Busy()
                {  return Owner->CPLD_Busy();  }
        virtual bool  CPLD_ReadValid()
                {  return Owner->CPLD_ReadValid();  }
        virtual bool  CPLD_FlashReady()
                {  return Owner->CPLD_FlashReady();  }
        virtual bool  CPLD_OverTemperature()
                {  return Owner->CPLD_OverTemperature();  }
    private:
        VpxCopFpaLoader *  Owner;
    };

    //
    //  Member Functions
    VpxCopFpaLoader(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~VpxCopFpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    //  CPLD methods
    //  Operations
    void          CPLD_Write(unsigned int addr, unsigned int data);
    unsigned int  CPLD_Read(unsigned int addr);
    void          CPLD_GoldenImageActive(bool state);
    void          CPLD_RestartConfig(bool restart_golden_image=false);
    void          CPLD_UpdateStatus();
    unsigned int  CPLD_ReadPowerStatus();

    //  Primitives
    void          CPLD_Address(unsigned int addr);
    void          CPLD_Data(unsigned int data);
    unsigned int  CPLD_Data();
    void          CPLD_Command(unsigned int code);

    unsigned int  CPLD_Version();
    unsigned int  CPLD_FlashType();
    bool          CPLD_GoldenImageActive();
    bool          CPLD_GoldenImageBooted();
    bool          CPLD_Busy();
    bool          CPLD_ReadValid();
    bool          CPLD_FlashReady();
    bool          CPLD_OverTemperature();

private:
    //  CPLD Private Impl Methods
    enum  CpldCommands { cmdFlashWrite=0x01, cmdFlashRead=0x02,  cmdSwapImage=0x04,
                         cmdAddrIncr=0x08,   cmdReadStatus=0x10, cmdRestartCfg=0x20,
                         cmdPowerStatus=0x40 };

    void  CPLD_BusyWait();
    void  CPLD_ReadValidWait();
    void  CPLD_FlashReadyWait();

public:

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxCopFpaLoader::FpaAddressRegister  -- Address Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FpaAddressRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Addr;

    public:
        FpaAddressRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Addr(*this, 0, 27)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VpxCopFpaLoader::FpaCommandRegister  -- Command Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FpaCommandRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup            Command;
        ReadOnlyRegisterBit         GoldenImageInUse;
        ReadOnlyRegisterBit         Busy;
        ReadOnlyRegisterBit         ReadValid;
        ReadOnlyRegisterBit         FlashReady;
        ReadOnlyRegisterBit         OverTemp;
        ReadOnlyRegisterBit         GoldenImageBooted;
        ReadOnlyRegisterBitGroup    FlashType;
        ReadOnlyRegisterBitGroup    CPLDVersion;

    public:
        FpaCommandRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Command(*this, 0, 7),   GoldenImageInUse(*this, 15),
               Busy(*this, 16),        ReadValid(*this, 17),
               FlashReady(*this, 18),  OverTemp(*this, 19),
               GoldenImageBooted(*this, 23),  FlashType(*this, 24, 4),
               CPLDVersion(*this, 28, 4)
            {
            }
    };

private:
    IRequires_Nothing *  MDC;
    FpaAddressRegister  FpaAddress;
    Register            FpaData;
    FpaCommandRegister  FpaCommand;

    bool                FGoldenImageMode;

    VpxFpaCpld           CpldIntf;
    FpaLoadThread       Thread;
    StopWatch           Rtc;

    void  RefreshStatus();
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
