// VpxSupport_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef VpxSupport_MbH
#define VpxSupport_MbH

//#include <HardwareRegister_Mb.h>
//#include <Ficl/FiclSystem_Mb.h>
//#include <Trace_Mb.h>
#include <Si570_Mb.h>
#include <CDCE72010_Mb.h>
#include <CDCE18005_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IVpxApplication  -- Interface to Application Subdevice
//==============================================================================

class IVpxApplication
{
public:
	virtual ~IVpxApplication()
		{}

	virtual bool  LoopbackEnabled() = 0;
	virtual void  LoopbackEnabled(bool state) = 0;

	virtual bool  TestGenEnabled() = 0;
	virtual void  TestGenEnabled(bool state) = 0;

};

//==============================================================================
//  CLASS IVpxProm  -- Vpx primitive interface to ROM chip
//==============================================================================

class IVpxProm
{
public:
	virtual ~IVpxProm()
        {}

    virtual void          EraseSector(unsigned int sector) = 0;
    virtual unsigned int  Read32(unsigned int addr) = 0;
    virtual void          Write32(unsigned int addr, unsigned int value) = 0;
    virtual unsigned int  Rdsr() = 0;
    virtual unsigned int  Rdid() = 0;

    //---------------------------
    //
    //  Flash Primitives
    //  ...write operation
    virtual void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data) = 0;
    //  ...read operation
    virtual unsigned int  FlashOp(unsigned int opcode, unsigned int addr) = 0;
    //  ...command operation
    virtual void          FlashOp(unsigned int opcode) = 0;

};

//==============================================================================
//  CLASS IComexTrigger  -- Vpx Comex Trigger User Interface
//==============================================================================

class IComexTrigger
{
public:
    enum ExtSyncOptions { essFrontPanel, essPps };
    enum LTrigSourceSelect { srcJ2, srcC_Trig_I };

    virtual ~IComexTrigger()
        {}

    //  Methods
    virtual void          SoftwareTrigger( bool state ) = 0;
    virtual bool          SoftwareTrigger() const = 0;

    virtual void          FramedTriggerSize(unsigned int samples) = 0;
    virtual unsigned int  FramedTriggerSize() const = 0;

    virtual void           Decimation(unsigned int samples) = 0;
    virtual unsigned int   Decimation() const = 0;

    //  ...Trigger Modes
    virtual void  FramedTriggerEnabled(bool state) = 0;
    virtual bool  FramedTriggerEnabled() const = 0;

    virtual void  EdgeTrigger(bool state) = 0;
    virtual bool  EdgeTrigger() const = 0;

    virtual void  ExternalTrigger(bool state) = 0;
    virtual bool  ExternalTrigger() const = 0;

    virtual void  LTrigSource(LTrigSourceSelect source) = 0;

    //  ...Trigger Routing
    virtual ExtSyncOptions  ExternalSyncSelection() const = 0;
    virtual void            ExternalSyncSelection(ExtSyncOptions option) = 0;

protected:

};

//==============================================================================
//  CLASS IComexSignalMux  -- Vpx Comex SMB 1 Mux Interface
//==============================================================================

class IComexSignalMux
{
public:
    enum Mux0SourceSelect { inJ1, inC_Clk_I };
    enum Mux3SourceSelect { inJ3, inC_PPS_I };
    enum Mux7SourceSelect { inJ2, inC_Trig_I };


    virtual ~IComexSignalMux()
        {}

    //  Methods
    virtual void  ExternalSourceSelect( Mux0SourceSelect which ) = 0;
    virtual void  ExternalPpsSourceSelect( Mux3SourceSelect which ) = 0;
    virtual void  ExternalTriggerSelect( Mux7SourceSelect which ) = 0;
};

//==============================================================================
//  CLASS IComexUtility  -- Vpx Epc Utility Ops User Interface
//==============================================================================

class IComexUtility
{
public:
    //  ...VoltageStatus
    virtual bool          V2_5Status() const = 0;
    virtual bool          V1_0Status() const = 0;
    virtual bool          V1_2Status() const = 0;
    virtual bool          V1_5Status() const = 0;
    virtual bool          Ddr3Status() const = 0;

    virtual void          GpioDirection(bool isOutput) = 0;
    virtual bool          Gpio() const = 0;
    virtual void          Gpio(bool data) = 0;

    virtual ~IComexUtility()
        {}

};

//==============================================================================
//  CLASS IVpxClkGenIo  -- Vpx Clock Gen User Interface
//==============================================================================

class IVpxClkGenIo
{
public:
    enum VpxPllClockSource { csExternal, csInternal };  // PLL used if internal
    enum VpxPllRefSource { rsExternal, rsInternal };    // onboard ref if internal
    enum VpxExtClockSource { cslLClk, cslFmcClk1 };     // which Ext Source used
                                                        //   for ext clock/ref
    virtual ~IVpxClkGenIo()
        {}
    //
    //  Interface Functions
    virtual void         SetPllMasterOutputSelection(int idx)= 0;
    virtual unsigned int ToDivider(unsigned int divider_idx)= 0;

    //  InMux Signals Configuration
    virtual void     FpgaClk1Source(unsigned int mux_source_select) = 0;
    virtual void     FpgaClk1Divider(unsigned int divide_by) = 0;
    virtual void     FpgaClk1OutputEnabled(bool enabled) = 0;
    virtual double   FpgaClk1OutputFrequencyActual() = 0;

    //  PLL Output Signals Configuration
    virtual  void     FpgaClk0OutputEnabled(bool enabled) = 0;
    virtual  void     FpgaClk0OutputFrequency(double frequency) = 0;
    virtual  double   FpgaClk0OutputFrequencyActual() = 0;

    virtual void      GClkOutputEnabled(unsigned int idx, bool enabled) = 0;
    virtual void      GClkOutputFrequency(unsigned int idx, double frequency) = 0;
    virtual double    GClkOutputFrequencyActual(unsigned int idx) = 0;

    virtual void      PlClkOutputEnabled(bool enabled) = 0;
    virtual void      PlClkOutputFrequency(double frequency) = 0;
    virtual double    PlClkOutputFrequencyActual() = 0;

    virtual void      FmcClk2OutputEnabled(bool enabled) = 0;
    virtual void      FmcClk2OutputFrequency(double frequency) = 0;
    virtual double    FmcClk2OutputFrequencyActual() = 0;

    virtual void      FmcClk3OutputEnabled(bool enabled) = 0;
    virtual void      FmcClk3OutputFrequency(double frequency) = 0;
    virtual double    FmcClk3OutputFrequencyActual() = 0;

    virtual void                Source(VpxPllClockSource src) = 0;
    virtual VpxPllClockSource   Source() const = 0;

    virtual void                ExternalClkSelect(VpxExtClockSource src) = 0;
    virtual VpxExtClockSource   ExternalClkSelect() const = 0;

    virtual void                Reference(VpxPllRefSource src) = 0;
    virtual VpxPllRefSource     Reference() const = 0;

    virtual void                ReferenceFrequency(VpxExtClockSource clk, double value) = 0;
    virtual double              ReferenceFrequency(VpxExtClockSource clk) const = 0;

    virtual bool                Locked()  const = 0;

};

//==============================================================================
//  CLASS IVpxRawClkGenDevices -- Interface to standard "Raw" Clock Gen Devices
//==============================================================================

class IVpxRawClkGenDevices
{
public:
    virtual ~IVpxRawClkGenDevices()  {  }

    virtual Si570 &       Vcxo() = 0;
    virtual Cdce72010 &   Pll() = 0;
    virtual Cdce18005 &   InMux() = 0;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
