// X3AlertSystem.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3AlertSystem_MbH
#define X3AlertSystem_MbH

#include <HardwareRegister_Mb.h>
#include <X3Alerts_Mb.h>
#include <AlertSystem_Mb.h>
#include <queue>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X3DeviceMap;

//===========================================================================
//  CLASS X3AlertSystem  -- X3 Alert Processing
//===========================================================================

class X3AlertSystem : public AlertSystem, public IUsesX3Alerts
{
public:
	X3AlertSystem(class X3DeviceMap * device);

	virtual void  AlertEnable(AlertType alert, bool state)
					   {  DoAlertEnable((int)alert, state);  }
    virtual void  AlertPassThru(AlertType alert, bool state)
                       {  DoAlertPassThru((int)alert, state);  }
    virtual void  AlertReset();
    virtual void  Append(const AlertData & alert);

	virtual void            Open();
    virtual void            Configure();
    virtual void            SoftwareAlert(unsigned int tag=0);

private:
    // Methods
    virtual int     ExtractAlertArgument(const Buffer & alert, int bit, int field);

    //
	//  Class AlertControlsRegister
	class AlertControlsRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

    public:
        RegisterBit         AlertFifoReset;
        RegisterBit         TimestampOn;
        RegisterBitGroup    Pdn;

    public:
        AlertControlsRegister( AddressingSpace &space, int offset );
    };
    //
    //  Class AlertEnableRegister
    class AlertEnableRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit   TimestampRollover;
        RegisterBit   Software;
        RegisterBit   WarningTemperature;
        RegisterBit   PllLost;
        RegisterBit   InputFifoOverrun;
        RegisterBit   InputTrigger;
        RegisterBit   InputPretriggerError;
        RegisterBit   InputStopTrigger;
        RegisterBit   InputOverrange;
        RegisterBit   OutputFifoUnderrun;
        RegisterBit   OutputTrigger;

    public:
        AlertEnableRegister( AddressingSpace &space, int offset );
    };

    X3DeviceMap *          Device;
    AlertControlsRegister  AlertControlReg;
    AlertEnableRegister    AlertEnableReg;
    Register               SoftwareAlertReg;
    unsigned int           FRolloverCount;
};


#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------

#endif
