// X3ClockDivider_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3ClockDivider_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS X3ClockDivider  -- X3 PLD Clock Divider Interface
//===========================================================================

//---------------------------------------------------------------------------
//  constructor for class X3IoDevice
//---------------------------------------------------------------------------

X3ClockDivider::X3ClockDivider(AddressingSpace &space, int offset)
	: TimerControlPldReg(space, offset)
{
    FDivisor.resize(16);
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Divisor() -- Enable output clock
//---------------------------------------------------------------------------

void X3ClockDivider::Divisor(int ch, unsigned short value)
{
    FDivisor[ch] = value;

	Port(0, value);
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Divisor() -- Query conversion clock
//---------------------------------------------------------------------------

unsigned short X3ClockDivider::Divisor(int ch) const
{
	return FDivisor[ch];
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Source() -- Select clock source
//---------------------------------------------------------------------------

void X3ClockDivider::Source(int ch, IIClockSource value)
{
	int state = value==csExternal ? 1 : 0;
    int mask = 1 << ch;
    FSource = ~mask;
    FSource |= state << ch;

    Port(1, FSource);
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Source() -- Query clock source
//---------------------------------------------------------------------------

X3ClockDivider::IIClockSource X3ClockDivider::Source(int ch) const
{
    int state = FSource & (1 << ch);
	return state ? csExternal : csPll;
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Enable() -- Enable output clock
//---------------------------------------------------------------------------

void X3ClockDivider::Enable(int ch, bool value)
{
	int state = value ? 1 : 0;
    int mask = 1 << ch;
    FEnable = ~mask;
    FEnable |= state << ch;

    Port(2, FEnable);
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Enable() -- Query conversion clock
//---------------------------------------------------------------------------

bool X3ClockDivider::Enable(int ch) const
{
    int state = FEnable & (1 << ch);
	return state ? true : false;
}

//---------------------------------------------------------------------------
//  X3ClockDivider::Port() -- Write value to specified addr
//---------------------------------------------------------------------------

void X3ClockDivider::Port(int data, int addr)
{
	TimerControlPldReg.Address.Value(addr);
    TimerControlPldReg.Data.Value(data);
    TimerControlPldReg.Apply();
}


} // Innovative

