// X3DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3DeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <X3DeviceDriver_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <PmcIdrom_Mb.h>
#include <SystemSupport_Mb.h>
#include <sstream>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//---------------------------------------------------------------------------
namespace Innovative
{

//===========================================================================
//  CLASS X3HdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class X3HdwPluginImpl : public IPacketDevicePlugin
{
public:
    X3HdwPluginImpl( X3DeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    X3DeviceMap * PDM;
};

//===========================================================================
//  CLASS X3DeviceMap  -- Registers for X3 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X3DeviceMap
//---------------------------------------------------------------------------
using namespace X3_MemoryMap;


PacketDeviceRegisters X3DRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	0,                    // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters X3RegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };


//---------------------------------------------------------------------------
//  constructor for class X3DeviceMap
//---------------------------------------------------------------------------

X3DeviceMap::X3DeviceMap()
    :  PacketDeviceImpl         (new X3DeviceDriver, X3DRegAddrs),
       PacketDeviceMap          (this, X3RegAddrs, 0),
       IUsesVirtexFpgaLoader	(PortMemory, mmFpgaData, mmFpgaControl),
       IUsesVirtexJtagLoader	(PortMemory, mmJtag),
       IUsesFpgaLogicVersionInfo(LogicMemory, mmFpgaVersion),
       IUsesPciLogicVersionInfo	(PortMemory, mmPciVersion),
       IUsesLogicTemperature	(LogicMemory, mmTempCurrent, mmTempWarning, mmTempFail),
       IUsesExtendedDioPort  	(LogicMemory, mmDioData, mmDioDataHigh, mmDioConfig),
       IUsesPmcEeprom           (PortMemory, mmI2c, 128),
       RcvPacketSize            (0x80),
       XmitPacketSize           (0x80),
       FLoopbackMode            (lmNone),
       DDeviceDriver            (dynamic_cast<X3DeviceDriver*>(DeviceDriver)),
       Alert                    (this),
       PciTestControlReg    	(PortMemory, mmPciTestControl),
       UserFpgaControlReg   	(LogicMemory, mmUserControl),
       //PacketHeaderReg   		(LogicMemory, mmInPacketHeader),
       TestReg					(LogicMemory, mmTest),
       PllClockControlReg		(LogicMemory, mmPldClockControl),
       VcxoIntf                 (PllClockControlReg),
       Ram						(LogicMemory, mmZbtAddress, mmZbtData)
{
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new X3HdwPluginImpl(this) );
}

//---------------------------------------------------------------------------
//  destructor for class X3DeviceMap
//---------------------------------------------------------------------------

X3DeviceMap::~X3DeviceMap()
{
}

//---------------------------------------------------------------------------
//  X3DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3DeviceMap::OpenHardware()
{
    IUsesPmcEeprom::AfterOpen();
    //
    //  Alert Interface
    Alert.Open();
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  X3DeviceMap::ConfigureHardware()
{
    //  NOTE:  Starting with X3-SDF firmware v3, all user-registers are
    //         cleared via the FpgaReset bit.  So, all reg initialization
    //         must follow
    //
    //  Reset User Logic
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.OutGo.Clear();
    UserFpgaControlReg.InTrigger.Value(false);
    UserFpgaControlReg.OutTrigger.Value(false);
    UserFpgaControlReg.FpgaReset.Set();
    UserFpgaControlReg.Apply();

    Sleep(1);
    UserFpgaControlReg.FpgaReset.Clear();
    UserFpgaControlReg.Apply();

    //
    //  Configure Alerts
    Alert.Configure();

    //
    //  Apply Control Register Stuff
    PciTestControlReg.Apply();

    // Config temp monitor level
    LogicWarningTemperature(80.0);
    LogicFailureTemperature(85.0);

    DioPortConfig(DioPortConfig());

    // Steer reference and sample clock muxes
    PllClockControlReg.Apply();

    //
    //  Set loopback mode
    PciLoopbackMode(PciLoopbackMode());
}

//---------------------------------------------------------------------------
//  X3DeviceMap::AlertTest() -- Alert Test Processing Hook
//---------------------------------------------------------------------------

bool  X3DeviceMap::AlertTest(unsigned int pid)
{
    //  return TRUE if PID is an alert PID.
    //  return FALSE to if not
    return Alert.AlertTest(pid);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::AlertCheck() -- Alert Event Processing Hook
//---------------------------------------------------------------------------

bool X3DeviceMap::AlertCheck(const Buffer & buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alert.CheckAndProcess(buffer);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  X3DeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<X3DeviceDriver::LOG->Count; i++)
        vec.push_back(X3DeviceDriver::LOG->Status[i]);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::LocalLoopbackMode()
//---------------------------------------------------------------------------

void  X3DeviceMap::LocalLoopbackMode(bool state)
{
    if (state)
        {
        PciTestControlReg.LoopbackEnable.Set();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Set();
        }
    else
        {
        PciTestControlReg.LoopbackEnable.Clear();
        PciTestControlReg.GenerateRxData.Clear();
        PciTestControlReg.DrainXmitData.Clear();
        PciTestControlReg.TestModeEnable.Clear();
        }
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Trigger() --  Set trigger state
//---------------------------------------------------------------------------

void X3DeviceMap::Trigger(IIDirection dir, bool state)
{
    switch (dir)
        {
        case tInput:
            UserFpgaControlReg.InTrigger.Value(state);
            break;
        case tOutput:
            UserFpgaControlReg.OutTrigger.Value(state);
            break;
        case tInputOutput:
            UserFpgaControlReg.InTrigger.Value(state);
            UserFpgaControlReg.OutTrigger.Value(state);
            break;
        }

    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  X3DeviceMap::Name() const
{
    static const char * ModuleName[] =
        { "X3-SD", "X3-SDF", "X3-A4D4", "X3-DIO", "X3-25M",
          "X3-SERVO", "X3-10M", "X3-2M", "X3-TIMING", "X3-SD16" };

    return ModuleName[FpgaLogicType()];
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Revision() --  Return revision of PCB as character
//---------------------------------------------------------------------------

std::string  X3DeviceMap::Revision() const
{
    std::stringstream msg;
    char c = 'A' + static_cast<char>(PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  X3DeviceMap::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3DeviceMap::UserLoopbackMode(bool state)
{
    UserFpgaControlReg.Loopback.Value(state);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3DeviceMap::UserLoopbackMode() const
{
    return UserFpgaControlReg.Loopback.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ClockTestEnable() -- Set sync pin cal mode status
//---------------------------------------------------------------------------

void X3DeviceMap::ClockTestEnable(bool state)
{
    TestReg.Bit(0, state);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ClockTestEnable() -- Query sync pin cal mode status
//---------------------------------------------------------------------------

bool X3DeviceMap::ClockTestEnable() const
{
    return TestReg.Bit(0) ? true : false;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::TestCounterEnable() -- Set test mode status
//---------------------------------------------------------------------------

void X3DeviceMap::TestCounterEnable(bool state)
{
    AdcTestEnable(state);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::TestCounterEnable() -- Query test mode status
//---------------------------------------------------------------------------

bool X3DeviceMap::TestCounterEnable() const
{
    return AdcTestEnable();
}

//---------------------------------------------------------------------------
//  X3DeviceMap::AdcTestEnable() -- Set A/D test mode status
//---------------------------------------------------------------------------

void X3DeviceMap::AdcTestEnable(bool state)
{
    UserFpgaControlReg.AdcTest.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::AdcTestEnable() -- Query A/D test mode status
//---------------------------------------------------------------------------

bool X3DeviceMap::AdcTestEnable() const
{
    return UserFpgaControlReg.AdcTest.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::DacTestEnable() -- Set D/A test mode status
//---------------------------------------------------------------------------

void X3DeviceMap::DacTestEnable(bool state)
{
    UserFpgaControlReg.DacTest.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::DacTestEnable() -- Query D/A test mode status
//---------------------------------------------------------------------------

bool X3DeviceMap::DacTestEnable() const
{
    return UserFpgaControlReg.DacTest.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Sync() -- Set sync state
//---------------------------------------------------------------------------

void X3DeviceMap::Sync(bool state)
{
    UserFpgaControlReg.Sync.Value(state ? 0 : 1);
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Sync() -- Get sync state
//---------------------------------------------------------------------------

bool X3DeviceMap::Sync() const
{
    return UserFpgaControlReg.Sync.Value() ? false : true;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Master() -- Set master sync status
//---------------------------------------------------------------------------

void X3DeviceMap::Master(bool state)
{
    UserFpgaControlReg.Master.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::Master() -- Get master sync status
//---------------------------------------------------------------------------

bool X3DeviceMap::Master() const
{
    return UserFpgaControlReg.Master.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3DeviceMap::ExternalClockSource(IIExtClockSource src)
{
    PllClockControlReg.ClockSource(src == esPxi);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3DeviceMap::IIExtClockSource X3DeviceMap::ExternalClockSource() const
{
    return PllClockControlReg.ClockSource() ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3DeviceMap::ReferenceSource(IIReferenceSource src)
{
    PllClockControlReg.ReferenceSource(src == rsPxi);
}

//---------------------------------------------------------------------------
//  X3DeviceMap::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3DeviceMap::IIReferenceSource X3DeviceMap::ReferenceSource() const
{
    return PllClockControlReg.ReferenceSource() ? rsPxi : rsCrystal;
}

//===========================================================================
//  CLASS X3DeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X3::X3DeviceMap
//---------------------------------------------------------------------------

X3DeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
    : inherited( space, offset ),
      TestModeEnable( *this, 0 ),
      GenerateRxData( *this, 1 ),
      LoopbackEnable( *this, 2 ),
      ClearTxCredit( *this, 3 ),
      ClearRxCredit( *this, 4 ),
      DrainXmitData( *this, 5 ),
      GenPacketSize( *this, 8, 4 ),
      GenPacketSizeUpdate( *this, 12 )
{
}


} // namespace Innovative


