// X3DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3DeviceMap_MbH
#define X3DeviceMap_MbH


#include <PacketDeviceMap_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <VirtexFpgaLoader_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <ZbtRam_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <HardwareInfoIntf_Mb.h>
#include <LogicTemperature_Mb.h>
#include <DioPort_Mb.h>
#include <X3AlertSystem_Mb.h>
#include <PmcEeprom_Mb.h>
#include <I2C_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X3DeviceDriver;

//===========================================================================
//  CLASS X3DeviceMap  -- Register & low level support for X3
//===========================================================================

class X3DeviceMap :  public PacketDeviceImpl,
					 public PacketDeviceMap,
					 public IUsesVirtexFpgaLoader,
					 public IUsesVirtexJtagLoader,
					 public IUsesFpgaLogicVersionInfo,
					 public IUsesPciLogicVersionInfo,
					 public IUsesLogicTemperature,
					 public IUsesExtendedDioPort,
                     public IUsesPmcEeprom
{
	friend class X3;
    friend class X3AlertSystem;

public:
	enum IIDirection { tInput, tOutput, tInputOutput };
	enum LoopbackMode { lmNone, lmParallel, lmSerial };
	enum IIExtClockSource { esPanel, esPxi };
	enum IIReferenceSource { rsCrystal, rsPxi };

	//
	//  Member Functions
	X3DeviceMap();
	virtual ~X3DeviceMap();

	void 	            UserLoopbackMode(bool state);
	bool	            UserLoopbackMode() const;

	//
	//  Make this property public
	using PacketDeviceImpl::PciLoopbackMode;

	// Methods
	void                Trigger(IIDirection dir, bool state);
    // Version
    std::string         Name() const;
    std::string         Revision() const;

    // Properties
	void               	ClockTestEnable(bool state);
	bool          		ClockTestEnable() const;

	void               	TestCounterEnable(bool state);
	bool          		TestCounterEnable() const;

	void               	ExternalClockSource(IIExtClockSource src);
	IIExtClockSource    ExternalClockSource() const;

	void               	ReferenceSource(IIReferenceSource src);
	IIReferenceSource   ReferenceSource() const;

	void               	AdcTestEnable(bool state);
	bool          		AdcTestEnable() const;

	void               	DacTestEnable(bool state);
	bool          		DacTestEnable() const;

	void               	Sync(bool state);
	bool          		Sync() const;

	void               	Master(bool state);
	bool          		Master() const;

public: // pseudo-protected
	int  RcvPacketSize;
	int  XmitPacketSize;

	//
	//  Busmaster Intf functions
    virtual bool    AlertTest(unsigned int pid);
	virtual bool    AlertCheck(const Buffer & buffer);
	virtual void    OpenHardware();
	virtual void    ConfigureHardware();
	void            DumpIsrStatus(std::vector<int> & vec);

protected:
	// Fields
	LoopbackMode           FLoopbackMode;

	// Data
	X3DeviceDriver *       DDeviceDriver;
	X3AlertSystem          Alert;


	// ???  May not be implemented
	void  LocalLoopbackMode(bool state);

	//
	//  Class PciTestRegister
	class PciTestRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit       TestModeEnable;
		RegisterBit       GenerateRxData;
		RegisterBit       LoopbackEnable;
		RegisterBit       ClearTxCredit;
		RegisterBit       ClearRxCredit;
		RegisterBit       DrainXmitData;
		RegisterBitGroup  GenPacketSize;
		RegisterBit       GenPacketSizeUpdate;

	public:
		PciTestRegister( AddressingSpace &space, int offset );
	};

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS PllClockControlRegister  -- Sample and reference clock routing
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class PllClockControlRegister : public Register
	{
		typedef Register inherited;

	public:
   		//
		//  Member Data
		RegisterBit     Clock_Source;
		RegisterBit     Reference_Source;
		RegisterBit     Data_Out;
		RegisterBit     Clock_Out;
		RegisterBit     Data_In;
		RegisterBit     Clock_In;
		RegisterBit     Vcxo_Enable;

		//
		//  Member Functions
		PllClockControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Clock_Source(*this, 0), Reference_Source(*this, 1),
              Data_Out(*this, 2), Clock_Out(*this, 3), Data_In(*this, 4),
              Clock_In(*this, 5), Vcxo_Enable(*this, 6)
			{}

        void        ClockSource(bool state)
            {  FClockSource=state;  Clock_Source.Value(state ? 1 : 0);  }
        bool        ClockSource() const
            {  return FClockSource;  }

        void        ReferenceSource(bool state)
            {  FReferenceSource=state;  Reference_Source.Value(state ? 1 : 0);  }
        bool        ReferenceSource() const
            {  return FReferenceSource;  }

        void        VcxoEnable(bool state)
            {  FVcxoEnable=state;  Vcxo_Enable.Value(state ? 1 : 0);  }
        bool        VcxoEnable() const
            {  return FVcxoEnable;  }

        void Apply()
            {
                ClockSource(FClockSource);
                ReferenceSource(FReferenceSource);
                VcxoEnable(FVcxoEnable);
            }

    private:
        // Fields
        bool            FClockSource;
        bool            FReferenceSource;
        bool            FVcxoEnable;
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        VcxoI2C_Impl(PllClockControlRegister & reg)
            :  Reg(reg)
            {  Reg.VcxoEnable(true);  }
	    virtual ~VcxoI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_Out.Value();  }
        virtual void    SetData(int state)
			{  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        PllClockControlRegister &  Reg;
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS UserFpgaCtlRegister  -- User FPGA control
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class UserFpgaCtlRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBit           FpgaReset;
		RegisterBit           InGo;
		RegisterBit           Loopback;
		RegisterBit           AdcTest;
		RegisterBit           DioOut;
		RegisterBit           InMqReset;
		RegisterBit           OutMqReset;
		RegisterBit           InTrigger;
		RegisterBit           OutTrigger;
		RegisterBit           Sync;
		RegisterBit           Master;
		RegisterBit           OutGo;
		RegisterBit           DacTest;

		UserFpgaCtlRegister(AddressingSpace &space, int offset)
			:  inherited(space, offset),
			   FpgaReset(*this, 0),
			   InGo(*this, 1),
			   Loopback(*this, 4),
			   AdcTest(*this, 5),
			   DioOut(*this, 6),
			   InMqReset(*this, 8),
			   OutMqReset(*this, 9),
			   InTrigger(*this, 10),
			   OutTrigger(*this, 11),
			   Sync(*this, 12),
			   Master(*this, 13),
			   OutGo(*this, 16),
               DacTest(*this, 17)
			{
			//  Default setup ????
			}
	};

	//
	//  ...Registers
	PciTestRegister                    PciTestControlReg;
	UserFpgaCtlRegister                UserFpgaControlReg;
	ShadowRegister					   TestReg;
    PllClockControlRegister            PllClockControlReg;
    VcxoI2C_Impl                       VcxoIntf;
	ZbtRam              			   Ram;
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
