//  X3FifoExtension_Mb.h
//
//  Definition of FIFO extension registers and functions to support data polling.
//---------------------------------------------------------------------------

#ifndef X3FifoExtensionsH
#define X3FifoExtensionsH

#include <X3Servo_Mb.h>
#include <HardwareRegister_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <Events_Mb.h>
#include <AlertSystem_Mb.h>
#include <FifoExtension_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{
using namespace X3_MemoryMap;

//==============================================================================
//  CLASS X3FifoExtensionRegisters  --  Fifo related device registers
//==============================================================================

class X3FifoExtensionRegisters :  public BaseboardExtension<X3Servo>,
                                  public IFifoExtension
{

public:
    //
    //  Member Functions
    X3FifoExtensionRegisters(X3Servo & board);

    //register access functions
    ii32 RecvFifoWordCount()
    {
        return RecvWordCountRegister.Value();
    }

    ii32 ReadRecvFifoWord()
    {
        return RecvFifoRegister.Value();
    }

    ii32 SendFifoWordCount()
    {
        return SendWordCountRegister.Value();
    }

    void WriteSendFifoWord(ii32 sendWord)
    {
        SendFifoRegister.Value(sendWord);
    }

    //ticks: 0 to 2^26-1
    void SetDacDelay(ii32 ticks)
    {        
        DacDelayTimerRegister.Value(ticks-1);
    }

    ii32 GetDacDelay()
    {
        ii32 reg=DacDelayTimerRegister.Value();
        return (reg+1);
    }

private:

    class RecvWordCountReg : public Register
    {
        typedef Register inherited;

    public:
        RecvWordCountReg( IAddressingSpace &space, const unsigned int & offset )
            :  inherited(space, offset)
            {
            }
    };

    class RecvFifoReg : public Register
    {
        typedef Register inherited;

    public:
        RecvFifoReg( IAddressingSpace &space, const unsigned int & offset )
            :  inherited(space, offset)
            {
            }
    };

    class SendWordCountReg : public Register
    {
        typedef Register inherited;

    public:
        SendWordCountReg( IAddressingSpace &space, const unsigned int & offset )
            :  inherited(space, offset)
            {
            }
    };

    class SendFifoReg : public Register
    {
        typedef Register inherited;

    public:
        SendFifoReg( IAddressingSpace &space, const unsigned int & offset )
            :  inherited(space, offset)            {
            }
    };

    class DacDelayTimerReg : public Register
    {
        typedef Register inherited;

    public:
        DacDelayTimerReg( IAddressingSpace &space, const unsigned int  & offset )
            :  inherited(space, offset)            {
            }
    };

    //  Register Objects
    RecvWordCountReg    RecvWordCountRegister;
    RecvFifoReg         RecvFifoRegister;
    SendWordCountReg    SendWordCountRegister;
    SendFifoReg         SendFifoRegister;
    DacDelayTimerReg    DacDelayTimerRegister;
};

}  // end namespace Innovative
#endif
