// X3IoDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3IoDevice_Mb.h>
#include <X3DeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS X3IoDevice_Base  -- Register & low level support for X3
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X3IoDevice_Base
//---------------------------------------------------------------------------

X3IoDevice_Base::X3IoDevice_Base(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
	                             IoInfoIntf * info)
	: FInfo(info),
	  GainCoeffReg(owner->LogicMemory, reg.GainCoefRegister_Addr, info->Channels().Channels()),
	  OffsetCoeffReg(owner->LogicMemory, reg.OffsetCoefRegister_Addr, info->Channels().Channels()),
	  ChannelEnableReg(owner->LogicMemory, reg.ChannelEnableRegister_Addr),
	  AmpEnableReg(owner->LogicMemory, reg.AmpEnableRegister_Addr),
	  TriggerReg(owner->LogicMemory, reg.TriggerControlRegister_Addr),
	  DecimationPrefillReg(owner->LogicMemory, reg.DecimationRegister_Addr),
      PacketHeaderReg(owner->LogicMemory, reg.PacketHeaderRegister_Addr),
      TriggerDelayReg(owner->LogicMemory, reg.TriggerDelayRegister_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X3IoDevice_Base
//---------------------------------------------------------------------------

X3IoDevice_Base::~X3IoDevice_Base()
{
    delete FInfo; FInfo = 0;     // owned object
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::Framed() --
//---------------------------------------------------------------------------

void X3IoDevice_Base::Framed(int size)
{
	TriggerReg.Count.Value(size-1);
	TriggerReg.FramedMode.Value(true);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::Unframed() --
//---------------------------------------------------------------------------

void X3IoDevice_Base::Unframed()
{
	TriggerReg.FramedMode.Value(false);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::ConfigureHardware() -- Apply hardware settings after reset
//---------------------------------------------------------------------------

void X3IoDevice_Base::ConfigureHardware()
{
    // Reapply Decimation Factor (and pretrigger count too)
	Decimation( Decimation() );
    // Reapply Trigger Delay
	TriggerDelayReg.Apply();
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void X3IoDevice_Base::ExternalTrigger(bool state)
{
	TriggerReg.External.Value(state);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool X3IoDevice_Base::ExternalTrigger() const
{
	return TriggerReg.External.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::FrontPanelTrigger() --  Set external trigger source
//---------------------------------------------------------------------------

void X3IoDevice_Base::FrontPanelTrigger(bool state)
{
	TriggerReg.FrontPanelTrigger.Value(state ? 0 : 1);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::FrontPanelTrigger() --  Query ext trigger source
//---------------------------------------------------------------------------

bool X3IoDevice_Base::FrontPanelTrigger() const
{
	return TriggerReg.FrontPanelTrigger.Value() ? false : true;
}


//---------------------------------------------------------------------------
//  X3IoDevice_Base::PretriggerMode() --  Set PretriggerMode trigger enable
//---------------------------------------------------------------------------

void X3IoDevice_Base::PretriggerMode(bool state)
{
	TriggerReg.PretriggerMode.Value(state);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PretriggerMode() --  Query pre trigger enable
//---------------------------------------------------------------------------

bool X3IoDevice_Base::PretriggerMode() const
{
	return TriggerReg.PretriggerMode.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PretriggerCount()
//---------------------------------------------------------------------------

void X3IoDevice_Base::PretriggerCount(unsigned int count)
{
	DecimationPrefillReg.PretriggerCount.Value(count);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PretriggerCount()
//---------------------------------------------------------------------------

unsigned int X3IoDevice_Base::PretriggerCount() const
{
	return DecimationPrefillReg.PretriggerCount.Value();
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::Info() --  Query I/O inteface
//---------------------------------------------------------------------------

IoInfoIntf & X3IoDevice_Base::Info()
{
	return *FInfo;
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::Decimation()
//---------------------------------------------------------------------------

void X3IoDevice_Base::Decimation(unsigned int factor)
{
	DecimationPrefillReg.DecimationFactor.Value(factor);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::Decimation()
//---------------------------------------------------------------------------

unsigned int X3IoDevice_Base::Decimation() const
{
	return DecimationPrefillReg.DecimationFactor.Value();
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PacketId() -- Set ID tage of received data packets
//---------------------------------------------------------------------------

void  X3IoDevice_Base::PacketId(unsigned char id)
{
    PacketHeaderReg.Id.Value(id);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PacketId() -- Query ID tage of received data packets
//---------------------------------------------------------------------------

unsigned char X3IoDevice_Base::PacketId() const
{
	return PacketHeaderReg.Id.Value();
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

void  X3IoDevice_Base::PacketSize(int size_words)
{
   PacketHeaderReg.Size.Value(size_words);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::PacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

int  X3IoDevice_Base::PacketSize() const
{
	return PacketHeaderReg.Size.Value();
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::TriggerDelay() -- Set Delay of trigger start
//---------------------------------------------------------------------------

void  X3IoDevice_Base::TriggerDelay(unsigned int delay)
{
   TriggerDelayReg.Delay.Value(delay);
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::TriggerDelay() -- Set Delay of trigger start
//---------------------------------------------------------------------------

unsigned int  X3IoDevice_Base::TriggerDelay() const
{
	return TriggerDelayReg.Delay.Value();
}



//===========================================================================
//  CLASS InDeviceBase  -- Support for X3 Ranged Devices
//===========================================================================
InDeviceBase::InDeviceBase(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
	IoInfoIntf * info)
	: X3IoDevice_Base(owner, reg, info),
	  AnalogControlReg(owner->LogicMemory, reg.ControlRegister_Addr)
{
	//
}

//---------------------------------------------------------------------------
//  X3IoDevice_Base::TestEnable() --  Set analog test enable
//---------------------------------------------------------------------------

void InDeviceBase::TestEnable(bool state)
{
	AnalogControlReg.TestEnable.Value(state);
}

//---------------------------------------------------------------------------
//  InDeviceBase::TestEnable() --  Query analog test enable
//---------------------------------------------------------------------------

bool InDeviceBase::TestEnable() const
{
	return AnalogControlReg.TestEnable.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  constructor for class InDevice
//---------------------------------------------------------------------------

InDevice::InDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
    IoInfoIntf * info, const PmcIdromSection & section, unsigned int one)
	: InDeviceBase(owner, reg, info),
      IUsesSectionCalibration(section, info->Channels().Channels(), one)
{

}

//---------------------------------------------------------------------------
//  InDevice::SetCalibrationOffset()
//---------------------------------------------------------------------------

void InDevice::SetCalibrationOffset(int ch, int value)
{
    OffsetCoeffReg[ch] = value;
}

//---------------------------------------------------------------------------
//  InDevice::SetCalibrationGain()
//---------------------------------------------------------------------------

void InDevice::SetCalibrationGain(int ch, int value)
{
    GainCoeffReg[ch].Value(value);
}

//---------------------------------------------------------------------------
//  constructor for class X3RangedIoDevice
//---------------------------------------------------------------------------

X3RangedIoDevice::X3RangedIoDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
	                                IoInfoIntf * info, unsigned int one)
	:  InDeviceBase(owner, reg, info),
       IUsesRangedSectionCalibration(info->Channels().Channels(), one),
	   GainControlReg(owner->LogicMemory, reg.GainModeRegister_Addr, info->Channels().Channels())
{
}
//---------------------------------------------------------------------------
//  X3RangedIoDevice::AsRangeIndex() --  Convert range to range index
//---------------------------------------------------------------------------

int X3RangedIoDevice::AsRangeIndex(IIRange value) const
{
    switch (value)
        {
        default:
        case x1:     return 0;
        case x2:     return 1;
        case x5:     return 2;
        case x10:    return 3;
        }
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::AsRange() --  Convert range index to range
//---------------------------------------------------------------------------

X3RangedIoDevice::IIRange  X3RangedIoDevice::AsRange(int idx) const
{
    switch (idx)
        {
        default:
        case 0:     return x1;
        case 1:     return x2;
        case 2:     return x5;
        case 3:     return x10;
        }
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::Range() --  Set analog range
//---------------------------------------------------------------------------

void X3RangedIoDevice::Range(int ch, IIRange value)
{
    SetRange(ch, value);
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::Range() --  Query analog range
//---------------------------------------------------------------------------

X3RangedIoDevice::IIRange X3RangedIoDevice::Range(int ch) const
{
    return GetRange(ch);
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::SetRange() --  Set analog range
//---------------------------------------------------------------------------

void X3RangedIoDevice::SetRange(int ch, IIRange value)
{
    GainControlReg[ch].Value( AsRangeIndex(value) );
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::GetRange() --  Query analog range
//---------------------------------------------------------------------------

X3RangedIoDevice::IIRange X3RangedIoDevice::GetRange(int ch) const
{
    return AsRange( GainControlReg[ch].Value() );
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::SetCalibrationOffset()
//---------------------------------------------------------------------------

void X3RangedIoDevice::SetCalibrationOffset(size_t gain_range, int ch, int value)
{
    if (gain_range == GainControlReg[ch].Value())
    	OffsetCoeffReg[ch] = value;
}

//---------------------------------------------------------------------------
//  X3RangedIoDevice::SetCalibrationGain()
//---------------------------------------------------------------------------

void X3RangedIoDevice::SetCalibrationGain(size_t gain_range, int ch, int value)
{
    if (gain_range == GainControlReg[ch].Value())
    	GainCoeffReg[ch].Value(value);
}


//===========================================================================
//  CLASS OutDeviceBase  -- Register & low level support for X3
//===========================================================================
OutDeviceBase::OutDeviceBase(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
	IoInfoIntf * info)
	: X3IoDevice_Base(owner, reg, info),
    AnalogControlReg(owner->LogicMemory, reg.ControlRegister_Addr),
    DacDelayTimerReg(owner->LogicMemory, X3_MemoryMap::mmOutDacDelay)
{
	//
}

/*
//---------------------------------------------------------------------------
//  OutDeviceBase::TestEnable() --  Set analog test enable
//---------------------------------------------------------------------------

void OutDeviceBase::TestEnable(bool state)
{
	AnalogControlReg.TestEnable.Value(state);
}

//---------------------------------------------------------------------------
//  OutDeviceBase::TestEnable() --  Query analog test enable
//---------------------------------------------------------------------------

bool OutDeviceBase::TestEnable() const
{
	return AnalogControlReg.TestEnable.Value() ? true : false;
}
*/
//---------------------------------------------------------------------------
//  OutDeviceBase::Loopback() --  Set  AnalogControlReg.Loopback
//---------------------------------------------------------------------------

void OutDeviceBase::Loopback(bool state)
{
	AnalogControlReg.Loopback.Value(state);
}

//---------------------------------------------------------------------------
//  OutDeviceBase::Loopback() --  Get AnalogControlReg.Loopback
//---------------------------------------------------------------------------

bool OutDeviceBase::Loopback() const
{
	return AnalogControlReg.Loopback.Value() ? true : false;
}
//---------------------------------------------------------------------------
//  OutDeviceBase::InterpolationFactor() --  Set  AnalogControlReg.InterpolationFactor
//---------------------------------------------------------------------------

void OutDeviceBase::InterpolationFactor(unsigned int state)
{
	AnalogControlReg.InterpolationFactor.Value(state == 0 ? 0 : state - 1);
}

//---------------------------------------------------------------------------
//  OutDeviceBase::InterpolationFactor() --  Get AnalogControlReg.InterpolationFactor
//---------------------------------------------------------------------------

unsigned int OutDeviceBase::InterpolationFactor() const
{
	return AnalogControlReg.InterpolationFactor.Value() == 0 ? 0 : AnalogControlReg.InterpolationFactor.Value() + 1;
}

//---------------------------------------------------------------------------
//  OutDeviceBase::DacDelay() -- ticks: 0 to 2^26-1
//---------------------------------------------------------------------------
void OutDeviceBase::DacDelay(unsigned int ticks)
{
    DacDelayTimerReg.Value(ticks - 1);
}

//---------------------------------------------------------------------------
//  OutDeviceBase::DacDelay() --
//---------------------------------------------------------------------------
unsigned int OutDeviceBase::DacDelay() const
{
    ii32 reg = DacDelayTimerReg.Value();
    return (reg + 1);
}

//---------------------------------------------------------------------------
//  OutDeviceBase::ConfigureHardware() -- Apply hardware settings after reset
//---------------------------------------------------------------------------

void OutDeviceBase::ConfigureHardware()
{
	X3IoDevice_Base::ConfigureHardware();
	// Reapply Trigger Delay
	AnalogControlReg.Apply();
}

//---------------------------------------------------------------------------
//  constructor for class OutDevice
//---------------------------------------------------------------------------

OutDevice::OutDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
    IoInfoIntf * info, const PmcIdromSection & section, unsigned int one)
	: OutDeviceBase(owner, reg, info),
	IUsesSectionCalibration(section, info->Channels().Channels(), one)
{
	//
}

//---------------------------------------------------------------------------
//  OutDevice::SetCalibrationOffset()
//---------------------------------------------------------------------------

void OutDevice::SetCalibrationOffset(int ch, int value)
{
    OffsetCoeffReg[ch] = value;
}

//---------------------------------------------------------------------------
//  OutDevice::SetCalibrationGain()
//---------------------------------------------------------------------------

void OutDevice::SetCalibrationGain(int ch, int value)
{
    GainCoeffReg[ch].Value(value);
}



} // Innovative


