// X3IoDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3IoDevice_MbH
#define X3IoDevice_MbH

#include <HardwareRegister_Mb.h>
#include <HardwareInfoIntf_Mb.h>
#include <CalMemBase_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X3DeviceMap;

//===========================================================================
//  CLASS X3IoDeviceRegisters  -- Addresses for Device Registers
//===========================================================================

class X3IoDeviceRegisters
{
public:
	unsigned int   I2CRegister_Addr;
	unsigned int   Rom_Origin;
	unsigned int   ControlRegister_Addr;
	unsigned int   GainCoefRegister_Addr;
	unsigned int   OffsetCoefRegister_Addr;
	unsigned int   GainModeRegister_Addr;
	unsigned int   ChannelEnableRegister_Addr;
	unsigned int   AmpEnableRegister_Addr;
	unsigned int   TriggerControlRegister_Addr;
	unsigned int   DecimationRegister_Addr;
    unsigned int   PacketHeaderRegister_Addr;
    unsigned int   TriggerDelayRegister_Addr;
};

//===========================================================================
//  CLASS X3IoDevice_Base  -- Register & low level support for X3
//===========================================================================

class X3IoDevice_Base 
{
	friend class X3SDDeviceMap;
	friend class X3SDFDeviceMap;
	friend class X3ServoDeviceMap;
	friend class X3_25MDeviceMap;
	friend class X3_10MDeviceMap;
	friend class X3_DigIODeviceMap;
	friend class X3_A4D4DeviceMap;
	friend class X3_SD16DeviceMap;
	friend class X3_2MDeviceMap;

public:
    virtual ~X3IoDevice_Base();

	// Methods
	void                Framed(int size);
	void                Unframed();
	IoInfoIntf & 	    Info();

	// Properties
	void                ExternalTrigger(bool state);
	bool                ExternalTrigger() const;

	void                FrontPanelTrigger(bool state);
	bool                FrontPanelTrigger() const;

	void                PretriggerMode(bool state);
	bool                PretriggerMode() const;

	void                PretriggerCount(unsigned int count);
	unsigned int        PretriggerCount() const;
    unsigned int        MaxPretriggerCountWords() const
							{ return (1<<19) - 32;  }  // logic requires small pad..we add a bit too

    void                Decimation(unsigned int factor);
    unsigned int        Decimation() const;
    
    void                PacketId(unsigned char id);
	unsigned char       PacketId() const;

    void                PacketSize(int size);
    int                 PacketSize() const;

	void                TriggerDelay(unsigned int delay_clocks);
	unsigned int        TriggerDelay() const;



    virtual void        ConfigureHardware();

protected:
	// Ctor
	X3IoDevice_Base(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
		IoInfoIntf * info);

	// Fields
	IoInfoIntf *                FInfo;

	// Data
	X3DeviceMap * 		        Owner;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TriggerRegister  -- Trigger control
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class TriggerRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBitGroup      Count;
		RegisterBit           FrontPanelTrigger;
		RegisterBit           PretriggerMode;
		RegisterBit           FramedMode;
		RegisterBit           External;

		TriggerRegister(AddressingSpace &space, int offset)
			:  inherited(space, offset),
			   Count(*this, 0, 24), FrontPanelTrigger(*this, 27),
			   PretriggerMode(*this, 29), FramedMode(*this, 30),
			   External(*this, 31)
			{
			//  Default setup ????
			}
	};


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PacketRegister  -- Rx Packet size management
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PacketRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Size;
        RegisterBitGroup      Id;

        //
        //  Member Functions
        PacketRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Size(*this, 0, 24), Id(*this, 24, 8)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DecimationPrefillRegister  -- Rx Decimation & Prefill
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DecimationPrefillRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      DecimationFactor;
        RegisterBitGroup      PretriggerCount;

        //
        //  Member Functions
        DecimationPrefillRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              DecimationFactor(*this, 0, 12), PretriggerCount(*this, 13, 19)
            {
            DecimationFactor = 1;
            PretriggerCount = 0;
            }

    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TriggerDelayRegister  -- Trigger Delay
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerDelayRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Delay;

        //
        //  Member Functions
        TriggerDelayRegister(AddressingSpace &space, int offset)
            : inherited(space, offset),
              Delay(*this, 0, 4)
            {
            Delay = 0;
            }

    };

	// Registers
	RegisterArray<Register>			   GainCoeffReg;
	RegisterArray<Register>            OffsetCoeffReg;
	ShadowRegister              	   ChannelEnableReg;
	ShadowRegister					   AmpEnableReg;
	TriggerRegister					   TriggerReg;
	DecimationPrefillRegister		   DecimationPrefillReg;
    PacketRegister                     PacketHeaderReg;
    TriggerDelayRegister               TriggerDelayReg;
};



//===========================================================================
//  CLASS InDeviceBase  -- Input Device
//===========================================================================

class InDeviceBase : public X3IoDevice_Base
{

	friend class X3_DigIODeviceMap;

public:
	void                TestEnable(bool state);
	bool                TestEnable() const;
protected:
	InDeviceBase(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
		IoInfoIntf * info);
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS AnalogControlRegister  -- Misc analog control register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class AnalogControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBitGroup      Fso;
		RegisterBit           Reset;
		RegisterBit           FailedReset;
		RegisterBit           AmpHighPassDisable0;
		RegisterBit           AmpHighPassDisable1;
		RegisterBit           AmpHighPassDisable2;
		RegisterBit           AmpHighPassDisable3;
		RegisterBit           FilterDisable;
		RegisterBit           Sync;
		RegisterBit           TestEnable;

		AnalogControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			Fso(*this, 0, 4),
			Reset(*this, 4),
			FailedReset(*this, 5),
			AmpHighPassDisable0(*this, 8),
			AmpHighPassDisable1(*this, 9),
			AmpHighPassDisable2(*this, 10),
			AmpHighPassDisable3(*this, 11),
			FilterDisable(*this, 12),
			Sync(*this, 13),
			TestEnable(*this, 15)
		{
			//  Default setup ????
		}
	};

	AnalogControlRegister			   AnalogControlReg;
}; //InDeviceBase

//===========================================================================
//  CLASS InDevice  -- Register & low level support for X3
//===========================================================================

class InDevice : public InDeviceBase,
			   	 public IUsesSectionCalibration
{
protected:
    InDevice(X3DeviceMap *   owner, X3IoDeviceRegisters & reg,
                             IoInfoIntf * info, const PmcIdromSection & section,
                             unsigned int one);

    // Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(int ch, int value);
    virtual void 		SetCalibrationGain(int ch, int value);

};

//===========================================================================
//  CLASS X3RangedIoDevice  -- Support for X3 Ranged Devices
//===========================================================================
class X3RangedIoDevice : public InDeviceBase,
						 public IUsesRangedSectionCalibration
{
public:
    enum IIRange { x1, x2, x5, x10 };

    // Range to index conversion
    int  				AsRangeIndex(IIRange value) const;
    IIRange             AsRange(int ch) const;
    // Properties
    void				Range(int ch, IIRange value);
    IIRange             Range(int ch) const;

protected:
    // Ctor
    X3RangedIoDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
        IoInfoIntf * info, unsigned int one);

    // Methods
    virtual void		SetRange(int ch, IIRange value);
    virtual IIRange     GetRange(int ch) const;

    // Registers
    RegisterArray<ShadowRegister>   GainControlReg;

    // Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(size_t gain_range, int ch, int value);
    virtual void 		SetCalibrationGain(size_t gain_range, int ch, int value);


};









//===========================================================================
//  CLASS OutDeviceBase  -- Out Device
//===========================================================================

class OutDeviceBase : public X3IoDevice_Base
{
	friend class X3_DigIODeviceMap;

public:

	void                Loopback(bool state);
	bool                Loopback() const;

	void                InterpolationFactor(unsigned int state);
	unsigned int        InterpolationFactor() const;
	
    void                DacDelay(unsigned int ticks);
    unsigned int        DacDelay() const;

	virtual void        ConfigureHardware();

protected:
	OutDeviceBase(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
		IoInfoIntf * info);
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS AnalogControlRegister  -- Misc analog control register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class AnalogControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		//  RegisterBitGroup      Fso;
		RegisterBit           Reset;
		RegisterBit           FailedReset;
		RegisterBit           Loopback;
		RegisterBitGroup      InterpolationFactor;

		AnalogControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			//     Fso(*this, 0, 4),
			Reset(*this, 4),
			FailedReset(*this, 5),
			Loopback(*this, 6),
			InterpolationFactor(*this, 7, 8)
		    //	TestEnable(*this, 15)
			//     AmpHighPassDisable0(*this, 8),
			//     AmpHighPassDisable1(*this, 9),
			//     AmpHighPassDisable2(*this, 10),
			//     AmpHighPassDisable3(*this, 11),
			//     FilterDisable(*this, 12),
			//     Sync(*this, 13),
		{
			Loopback = 0;
            InterpolationFactor = 0;
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DacDelayTimerRegister  -- DAC Delay
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacDelayTimerRegister : public Register
    {
        typedef Register inherited;

    public:
        DacDelayTimerRegister( IAddressingSpace &space, const unsigned int  & offset )
            :  inherited(space, offset)            {
            }
    };

	AnalogControlRegister			   AnalogControlReg;
    DacDelayTimerRegister              DacDelayTimerReg;


}; //OutDeviceBase

//===========================================================================
//  CLASS OutDevice  -- Register & low level support for X3
//===========================================================================

class OutDevice :   public OutDeviceBase,
	                public IUsesSectionCalibration
{
protected:
    OutDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg,
                                   IoInfoIntf * info, const PmcIdromSection & section,
                                   unsigned int one);

    // Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(int ch, int value);
    virtual void 		SetCalibrationGain(int ch, int value);

};





#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
