// X3SDDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3SD_Mb.h>
#include <X3SDDeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    InputMaxChannels  = 16;
const int    InputWidth_Bits   = 24;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 128;
const float  InputMaxRate      = 216.e3;
const float  InputMinRate      = 0.0;
const int    InputGangSize     = 1;
const float  InputReferenceClk = 24.576e6;
const float  InputVcoMinClk    = 79.0e6;
const float  InputVcoMaxClk    = 111.0e6;

const float  MaxSingleRate_Rate = 54.0e3;
const float  MaxDualRate_Rate   = 108.0e3;
const float  MaxQuadRate_Rate   = 216.0e3;
const float  MaxRangeRate       = 38.4e6;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (# channels x 2 values/ch + 1 calibration flag + 1 spare) x # ranges
const int romX3_SD_InputCoeffOffset = 34;   // Note:  Workaround for previous X3SDInputDevice ctor IDROM origin bug, now fixed.
const int romX3_SD_InChannels = InputMaxChannels;
const int romX3_SD_Ranges = 1;
const int romX3_SD_RangeSize = romX3_SD_InChannels * 2 + 1 + 1;
const int romX3_SD_InputCoeffSize = romX3_SD_RangeSize * romX3_SD_Ranges;
const int romX3_SD_InputCoeff_END = romX3_SD_InputCoeffOffset + romX3_SD_InputCoeffSize;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_SD_ClockParamOffset = romX3_SD_InputCoeff_END;
const int romX3_SD_ClockParamSize = 4;
const int romX3_SD_ClockParam_END = romX3_SD_ClockParamOffset + romX3_SD_ClockParamSize;


class X3SDInfoIntf : public IoInfoIntf
{
public:
    X3SDInfoIntf()
        : FClockFactor(InputClockFactor),
               //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(InputMaxChannels, InputWidth_Bits, InputMuxCount,  InputIsUnipolar)
        {  FSpan.push_back(SpanInfo(10000, -10000));  }
    virtual ~X3SDInfoIntf()  {  }

    virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
    virtual float               		MaxRate() const			{  return InputMaxRate;  }
    virtual float               		MinRate() const			{  return InputMinRate;  }
    virtual const SpanArray &   		Span() const			{  return FSpan;  }
    virtual int                 		GangSize() const		{  return InputGangSize;  }
    virtual bool                		IsStackable() const		{  return false;  }
    virtual bool                		IsInverted() const		{  return true;  }
    virtual bool                		IsMuxed() const			{  return false;  }
    virtual bool                		MuxDepth() const		{  return InputMuxCount;  }
    virtual int                 		BufferSize() const		{  return 65536;  }
    virtual int                 		ClockFactor() const		{  return FClockFactor;  }
    virtual float               		ReferenceClock() const	{  return InputReferenceClk;  }
    virtual int               	        SamplesPerFrame() const {  return 1;  }

    void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
    UnpairedChannelInfo	        FChannels;
    SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3SDInputDevice  -- Module-specific input device features
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3SDInputDevice::X3SDInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
    : X3RangedIoDevice(owner, reg, new X3SDInfoIntf, 0x10000)  /* 'one' value for gain registers */
{
    AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
        romX3_SD_InputCoeffOffset, romX3_SD_RangeSize));
}

//---------------------------------------------------------------------------
//  X3SDInputDevice::AmpHighPassDisable() --  Set filter state
//---------------------------------------------------------------------------

void X3SDInputDevice::AmpHighPassDisable(int bank, bool state)
{
    switch (bank)
        {
        default:
            break;
        case 0:
            AnalogControlReg.AmpHighPassDisable0.Value(state);
            break;
        case 1:
            AnalogControlReg.AmpHighPassDisable1.Value(state);
            break;
        case 2:
            AnalogControlReg.AmpHighPassDisable2.Value(state);
            break;
        case 3:
            AnalogControlReg.AmpHighPassDisable3.Value(state);
            break;
        }
}

//---------------------------------------------------------------------------
//  X3SDInputDevice::AmpHighPassDisable() --  Get filter state
//---------------------------------------------------------------------------

bool X3SDInputDevice::AmpHighPassDisable(int bank) const
{
    int result = 0;

    switch (bank)
        {
        default:
            break;
        case 0:
            result = AnalogControlReg.AmpHighPassDisable0.Value();
            break;
        case 1:
            result = AnalogControlReg.AmpHighPassDisable1.Value();
            break;
        case 2:
            result = AnalogControlReg.AmpHighPassDisable2.Value();
            break;
        case 3:
            result = AnalogControlReg.AmpHighPassDisable3.Value();
            break;
        }

    return result ? true : false;
}

//---------------------------------------------------------------------------
//  X3SDInputDevice::AmpHighPassDisable() --  Get filter state
// Not really a ranged device, so this is a no-op
//---------------------------------------------------------------------------

void X3SDInputDevice::SetRange(int /*ch*/, IIRange /*value*/)
{
}

//---------------------------------------------------------------------------
//  X3SDInputDevice::AmpHighPassDisable() --  Get filter state
// Not really a ranged device, so this is a no-op
//---------------------------------------------------------------------------

X3SDInputDevice::IIRange X3SDInputDevice::GetRange(int /*ch*/) const
{
    return X3RangedIoDevice::x1;
}


//---------------------------------------------------------------------------
//  X3IoDeviceRegisters X3SDInputDeviceRegisters --  Register address collection
//---------------------------------------------------------------------------

static X3IoDeviceRegisters X3SDInputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, 0,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };

//=============================================================================
//  CLASS X3SDClock  --  PLL timebase control
//=============================================================================

class X3SDClock : public Ad9511Pfd
{
    typedef Ad9511Pfd inherited;

public:
    enum SamplingMode { smSingle, smDual, smQuad };
    enum ClockFactors { cf128, cf256, cf512 };
    //
    //  Member Functions
    X3SDClock(X3SDDeviceMap * owner, int index, AddressingSpace &space, int ctl)
        : inherited(index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
        Reference(InputReferenceClk);
        Min(InputVcoMinClk/MaxOutputDivider());
        Max(InputVcoMaxClk);
        PFDPolarity(true);
        Tuner.PfdRate(PhaseDetectorFrequency());
        }
    X3SDClock(X3SDDeviceMap * owner, double freq, double ref,
        bool enabled, int index, AddressingSpace & space, int ctl)
        : inherited(freq, ref, enabled, index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
        Min(InputVcoMinClk/MaxOutputDivider());
        Max(InputVcoMaxClk);
        PFDPolarity(true);
        Tuner.PfdRate(PhaseDetectorFrequency());
        }

    //  Set clock output modes and signalling types
    void ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
    {
        PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
        OnPllConfigureOutput.Execute(e);

        // Identical duty cycle and phase for all outputs
        for (int i = 0; i < 8; ++i)
            {
            Port(0x48+i*2, outdiv_bits);	    // duty cycle
            Port(0x48+(i*2+1), outdiv_bypass);	// phase
            }

        Port(0x3C, 0x08);       // Clock 0 LVPECL on @810mV
        Port(0x3D, 0x03);       // Clock 1 disabled
        Port(0x3E, 0x03);       // Clock 2 disabled
        Port(0x3F, 0x03);       // Clock 3 disabled

        Port(0x40, 0x0a);       // Clocks 4..7 LVCMOS on @3.5 mA
        Port(0x41, 0x0a);
        Port(0x42, 0x0a);
        Port(0x43, 0x0a);

        PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
        OnAfterPllConfigureOutput.Execute(e1);
    }

    void SetReferenceCalibrationFactor(double value)
    {
    Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
    Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));    // set calibrated field
    }
    double GetReferenceCalibrationFactor() const
    {
    //  Check Calibrated bit
    if (Owner->ClockParamRomSection.AsInt(1)=='Y')
        {
        return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
        }
    else
        return 1.0;  // default value
    }

protected:
    // Data
    X3SDDeviceMap *         Owner;
    double                  FPDF;

    SamplingMode   SelectSamplingMode(double freq)
    {
        if (freq<MaxSingleRate_Rate)     return smSingle;
        else if (freq<MaxDualRate_Rate)  return smDual;
        else if (freq<MaxQuadRate_Rate)  return smQuad;
        else
            return smQuad;
    }

    ClockFactors  SelectClockFactor(double freq)
    {
        SamplingMode mode = SelectSamplingMode(freq);
        switch (mode)
            {
            case smQuad:  return cf128;
            case smDual:  return cf256;
            default:
            case smSingle: return cf512;
            }
    }

    int            ToClockFactor(ClockFactors cf)
    {
        switch (cf)
            {
            default:
            case cf128:  return 128;
            case cf256:  return 256;
            case cf512:  return 512;
            }
    }

    void            ConfigureFSORegister(SamplingMode mode, ClockFactors /*cf*/)
    {
        int value;
        if (mode==smSingle)       value=0;
        else if (mode==smDual)    value=1;
        else if (mode==smQuad)    value=2;
        else
            value = 0;

        Owner->SetFSO(value);

    }

    virtual void    SetFrequency(double freq)
    {
        SamplingMode mode = SelectSamplingMode(freq);
        ClockFactors cf   = SelectClockFactor(freq);

        ClockFactor(ToClockFactor(cf));

        X3SDInfoIntf & info = dynamic_cast<X3SDInfoIntf&>(Owner->Input().Info());
        info.SetClockFactor( ToClockFactor(cf) );

        ConfigureFSORegister(mode, cf);
        Tuner.PfdRate(PhaseDetectorFrequency());
        inherited::SetFrequency(freq);

        Owner->ResetAnalog();
    }

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }
};

//===========================================================================
//  CLASS X3SDDeviceMap  -- Register & low level support for SD module
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

X3SDDeviceMap::X3SDDeviceMap(X3SD * owner)
    : IUsesStreamDeviceEventCallbacks(owner),
      FClockSelect(csInternal),
      FInput(new X3SDInputDevice(this, X3SDInputDeviceRegisters)),
      Owner(owner),
      Clock(new X3SDClock(this, 0, LogicMemory, mmPll)),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romX3_SD_ClockParamOffset, romX3_SD_ClockParamSize)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3SDDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3SDDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3SDDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3SDDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3SDDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X3SDDeviceMap::~X3SDDeviceMap()
{
    delete Clock;
    Clock = 0;

    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3SDDeviceMap::OpenHardware()
{
    X3DeviceMap::OpenHardware();

    // Configure A/Ds
    const size_t range = 0;
    for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
        {
        Input().PacketId(0x0);
        Input().PacketSize(0x4000);
        FInput->Gain(range, ch, 1.0);
        FInput->Offset(range, ch, 0.0);
        Input().Unframed();
        }
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3SDDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    X3DeviceMap::ConfigureHardware();

    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

    //
    //  Apply Clock Settings from saved state
    Clock->Reapply();

    //
    //  Apply Gain/Offset Coefficients
    if (FInput->Calibrated(0))
        FInput->ApplyRomCoefficients(0);
    else
        FInput->ApplyDefaultCoefficients(0);

    //
    //  Apply input settings
    for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
        {
        FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
        Input().PacketSize(Input().PacketSize());
        }
    //
    //  If all 4 channels in a range are disabled, disable the AMP to save power
    for (int ch = 0; ch < InputMaxChannels; ch+=4)
        {
        if (Input().Info().Channels().Enabled(ch+0) || Input().Info().Channels().Enabled(ch+1) ||
            Input().Info().Channels().Enabled(ch+2) || Input().Info().Channels().Enabled(ch+3)    )
            {
            Input().AmpEnableReg.Bit(ch/4, true);
            }
        else
            {
            Input().AmpEnableReg.Bit(ch/4, false);
            }
        }
    // Configure triggers
    FInput->TriggerReg.Apply();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3SDDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    UserFpgaControlReg.InGo.Set();
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3SDDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3SDDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3SDDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.Apply();

    Trigger(tInput, false);

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3SDInputDevice & X3SDDeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::ConfigureFilters() -- Configure A/D filters and modes
//---------------------------------------------------------------------------

int X3SDDeviceMap::ConfigureFilters(float /*rate*/)
{
    int result = 1;
    return result;  // clock factor
}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::SetFSO() -- Set operation mode bits
//---------------------------------------------------------------------------

void  X3SDDeviceMap::SetFSO(unsigned int value)
{
    FInput->AnalogControlReg.Fso.Value(value);

}

//---------------------------------------------------------------------------
//  X3SDDeviceMap::ResetAnalog() -- Reset Analog Chip
//---------------------------------------------------------------------------

void  X3SDDeviceMap::ResetAnalog()
{
    //  Reset analog
    FInput->AnalogControlReg.Reset.Set();
    FInput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms

}


} // namespace Innovative


