// X3SDDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3SDDeviceMap_MbH
#define X3SDDeviceMap_MbH

#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Ad9511_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3SDInputDevice  -- Module-specific input device features
//===========================================================================

class X3SDInputDevice : public X3RangedIoDevice
{
	friend class X3SDDeviceMap;

public:
	// Ctor
	X3SDInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

    // Properties
    void        AmpHighPassDisable(int bank, bool state);
    bool        AmpHighPassDisable(int bank) const;

protected:
    // Methods
	virtual void		SetRange(int ch, IIRange value);
	virtual IIRange     GetRange(int ch) const;
};

//===========================================================================
//  CLASS X3SDDeviceMap  -- Register & low level support for SD module
//===========================================================================
class X3SD;
class X3SDClock;

class X3SDDeviceMap : public X3DeviceMap,
					  public IUsesStreamDeviceEventCallbacks,
					  public IUsesNullFrontPanelPort
{
	friend class X3SD;
    friend class X3SDInputDevice;
    friend class X3SDClock;

public:
	enum IIClkSelect { csInternal, csExternal };

	//
	//  Member Functions
	X3SDDeviceMap(X3SD * owner);
	virtual ~X3SDDeviceMap();

	X3SDInputDevice & 	Input();

public: // pseudo-protected
	virtual void  OpenHardware();
    void          SetFSO(unsigned int value);
    void          ResetAnalog();

protected:
	// Fields
	IIClkSelect		    FClockSelect;
	X3SDInputDevice * 	FInput;

	// Data
	X3SD *				Owner;
    Trace				Debug;
	Ad9511Pfd *         Clock;
    PmcIdromSection     ClockParamRomSection;

	// Methods
	void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

	//
	//  Support Methods
    int                 ConfigureFilters(float rate);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
