// X3SDFDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3SDF_Mb.h>
#include <X3SDFDeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <Magic_Mb.h>
#include <algorithm>
#include <SystemSupport_Mb.h>
#include <sstream>
#include <limits>
#include <math.h>

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    MaxChannels       = 4;
const int    InputWidth_Bits   = 24;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 5.0e6;
const float  InputMinRate      = 0.0;
const int    InputGangSize     = 1;
const float  InputReferenceClk = 24.576e6;
const float  InputVcoMinClk    = 100.0e6;
const float  InputVcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (# channels x 2 values/ch + 1 calibration flag + 1 spare) x # ranges
const int romX3_SDF_InputCoeffOffset = 10;  // Note:  Workaround for previous X3SDFInputDevice ctor IDROM origin bug, now fixed.
const int romX3_SDF_InChannels = MaxChannels;
const int romX3_SDF_Ranges = 1;
const int romX3_SDF_RangeSize = romX3_SDF_InChannels * 2 + 1 + 1;
const int romX3_SDF_InputCoeffSize = romX3_SDF_RangeSize * romX3_SDF_Ranges;
const int romX3_SDF_InputCoeff_END = romX3_SDF_InputCoeffOffset + romX3_SDF_InputCoeffSize;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_SDF_ClockParamOffset = romX3_SDF_InputCoeff_END;
const int romX3_SDF_ClockParamSize = 4;
const int romX3_SDF_ClockParam_END = romX3_SDF_ClockParamOffset + romX3_SDF_ClockParamSize;

//---------------------------------------------------------

class X3SDFInfoIntf : public IoInfoIntf
{
    friend class X3SDFDeviceMap;
    friend class X3SDFClock;

public:
    X3SDFInfoIntf()
          //        devices,      bits,              muxChannelCount,      unipolar
        : FChannels(MaxChannels,  InputWidth_Bits,    InputMuxCount,    InputIsUnipolar),
          FClockFactor(InputClockFactor)
        {  FSpan.push_back(SpanInfo(10000, -10000));  }
    virtual ~X3SDFInfoIntf()  {  }

    virtual UnpairedChannelInfo &	    Channels()		        {  return FChannels;  }
    virtual float               		MaxRate() const			{  return InputMaxRate;  }
    virtual float               		MinRate() const			{  return InputMinRate;  }
    virtual const SpanArray &   		Span() const			{  return FSpan;  }
    virtual int                 		GangSize() const		{  return InputGangSize;  }
    virtual bool                		IsStackable() const		{  return false;  }
    virtual bool                		IsInverted() const		{  return false;  }
    virtual bool                		IsMuxed() const			{  return false;  }
    virtual bool                		MuxDepth() const		{  return InputMuxCount;  }
    virtual int                 		BufferSize() const		{  return 65536;  }
    virtual int                 		ClockFactor() const		{  return FClockFactor;  }
    virtual float               		ReferenceClock() const	{  return InputReferenceClk;  }
    virtual int               	        SamplesPerFrame() const {  return 1;  }

private:
    UnpairedChannelInfo	        FChannels;
    int                         FClockFactor;
    SpanArray					FSpan;
};

//=============================================================================
//  CLASS X3SDFClock  --  PLL timebase control
//=============================================================================

class X3SDFClock : public Ad9511Pfd
{
    typedef Ad9511Pfd inherited;

public:
    //
    //  Member Functions
    X3SDFClock(X3SDFDeviceMap * owner, int index, AddressingSpace &space, int ctl)
        : inherited(index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
        Reference(InputReferenceClk);
        Min(InputVcoMinClk/MaxOutputDivider());
        Max(InputVcoMaxClk);
        PFDPolarity(true);
        Tuner.PfdRate(PhaseDetectorFrequency());
        }
    X3SDFClock(X3SDFDeviceMap * owner, double freq, double ref,
        bool enabled, int index, AddressingSpace & space, int ctl)
        : inherited(freq, ref, enabled, index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
        Min(InputVcoMinClk/MaxOutputDivider());
        Max(InputVcoMaxClk);
        PFDPolarity(true);
        Tuner.PfdRate(PhaseDetectorFrequency());
        }

    //  Set clock output modes and signalling types
    void ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
    {
        PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
        OnPllConfigureOutput.Execute(e);

        // Identical duty cycle and phase for all outputs
        for (int i = 0; i < 8; ++i)
            {
            Port(0x48+i*2, outdiv_bits);	    // duty cycle
            Port(0x48+(i*2+1), outdiv_bypass);	// phase
            }

        Port(0x3C, 0x08);       // Clock 0 LVPECL on @810mV
        Port(0x3D, 0x03);       // Clock 1 disabled
        Port(0x3E, 0x03);       // Clock 2 disabled
        Port(0x3F, 0x03);       // Clock 3 disabled

        Port(0x40, 0x0a);       // Clocks 4..7 LVCMOS on @3.5 mA
        Port(0x41, 0x0a);
        Port(0x42, 0x0a);
        Port(0x43, 0x0a);

        PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
        OnAfterPllConfigureOutput.Execute(e1);
    }

    void SetReferenceCalibrationFactor(double value)
    {
    Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
    Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));    // set calibrated field
    }
    double GetReferenceCalibrationFactor() const
    {
    //  Check Calibrated bit
    if (Owner->ClockParamRomSection.AsInt(1)=='Y')
        {
        return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
        }
    else
        return 1.0;  // default value
    }

protected:
    // Data
    X3SDFDeviceMap *        Owner;
    double                  FPDF;

    virtual void    SetFrequency(double freq)
    {
    Tuner.PfdRate(PhaseDetectorFrequency());
    int factor = Owner->DecimationFactor(static_cast<float>(freq));
    ClockFactor(factor);
    X3SDFInfoIntf & info = dynamic_cast<X3SDFInfoIntf&>(Owner->Input().Info());
    info.FClockFactor = factor;

    inherited::SetFrequency(freq);

    Owner->ConfigureFilters(static_cast<float>(freq));
    }

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

};

//===========================================================================
//  CLASS X3SDFShadowRegister  --  Debug version of ShadowRegister, which
//                               echoes value written to log window
//===========================================================================

class X3SDFRegister : public Register
{
    typedef Register inherited;

public:
    X3SDFRegister(AddressingSpace &space, int offset)
        : inherited(space, offset)
        {  }

    virtual void Value(ii32 value)
    {
        std::stringstream msg;
        msg << "Reg[0x" << std::hex << mOffset << "] = 0x" << value;
        Debug.Log(msg.str().c_str());

        Sleep(1);
        inherited::Value(value);
        Sleep(1);
    }
    virtual ii32 Value() const
    {
        return inherited::Value();
    }

protected:
    Trace           Debug;
};

//=============================================================================
//  CLASS X3SDFShadowRegister  --
//=============================================================================

class X3SDFShadowRegister : public MultiplexedShadowRegister
{
public:
    X3SDFShadowRegister(AddressingSpace & space, int offset,
        ii32 value=0, int mux=0)
        : MultiplexedShadowRegister(space, offset, value, mux), Debug(true)
    {
    }

protected:
    Trace           Debug;

    virtual void ShadowRegisterChanged()
    {
        std::stringstream msg;
        msg << "Mux: " << mMux << ", Reg[0x" << std::hex << mOffset << "] = 0x" << mShadowValue;
        Debug.Log(msg.str().c_str());

        Register::Value(mMux);
        Sleep(1);
        Apply();
        Sleep(1);
    }
};

//=============================================================================
//  CLASS X3SDFFilter  --  Custom filter coefficient loading support
//=============================================================================

class X3SDFFilter
{
    typedef std::vector<double> DoubleArray;
    typedef std::vector<int> IntArray;
    typedef std::vector<unsigned short> ShortArray;
public:

    X3SDFFilter(AddressingSpace & space, int offset)
        : Reg(space, offset)
        {  }

    void Load(DoubleArray & coef)
    {
        // Data
        IntArray        Coef;
        ShortArray      Code;

        // Calculate scale factor needed to produce Q27 format
        double sum = 0.0;
        for (DoubleArray::iterator i = coef.begin(); i != coef.end(); ++i)
            sum += *i;

        const int unity = 67108863;
        double scale = unity/sum;

        // Scale into Q27 format, stored in intermediate array Coef
        int tally = 0;
        for (DoubleArray::iterator i = coef.begin(); i != coef.end(); ++i)
            {
            int value = static_cast<int>(*i * scale);
            tally += value;
            Coef.push_back(value);
            }

        // Accumulate rounding-induced error into largest coefficient
        int delta = unity - tally;
        *std::max_element(Coef.begin(), Coef.end()) += delta;

        for (IntArray::iterator i = Coef.begin(); i != Coef.end(); ++i)
            {
            unsigned int sign = 1 << 26;
            unsigned int value = abs(*i) & (sign - 1);
            value |= (*i < 0) ? sign : 0;

            Code.push_back(value >> 16);
            Code.push_back(value & 0xffff);
            }

        // Determine checksum
        unsigned short FChecksum = 0;
        for (size_t i = 0; i < Code.size(); ++i)
            {
            FChecksum += Code[i] & 0xff;
            FChecksum += (Code[i] >> 8) & 0xff;
            }

        // Upload coefficients and checksum into register
        for (ShortArray::iterator i = Code.begin(); i != Code.end(); ++i)
            Reg.Value(*i);

        Reg.Value(FChecksum);
    }

private:
    // Data
    X3SDFRegister   Reg;
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS X3FilteredInputDevice  --  Ctor
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

X3FilteredInputDevice:: X3FilteredInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
    /* 'one' value for gain registers */
    : inherited(owner, reg, new X3SDFInfoIntf, 0x8000)
{
}

//---------------------------------------------------------------------------
//  X3FilteredInputDevice::AmpHighPassDisable() --  Get filter state
// Not really a ranged device, so this is a no-op
//---------------------------------------------------------------------------

void X3FilteredInputDevice::SetRange(int /*ch*/, IIRange /*value*/)
{
}

//---------------------------------------------------------------------------
//  X3FilteredInputDevice::AmpHighPassDisable() --  Get filter state
// Not really a ranged device, so this is a no-op
//---------------------------------------------------------------------------

X3FilteredInputDevice::IIRange X3FilteredInputDevice::GetRange(int /*ch*/) const
{
    return X3RangedIoDevice::x1;
}

//===========================================================================
//  CLASS X3SDFInputDevice  -- Register & low level support for X3
//===========================================================================

class X3SDFInputDevice : public X3FilteredInputDevice
{
    typedef X3FilteredInputDevice inherited;
    friend class X3SDFDeviceMap;

public:
    // Ctor
    X3SDFInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
        : inherited(owner, reg),
          FilterReg(owner->LogicMemory, mmSDFData, MaxChannels),
          ControlReg1(owner->LogicMemory, mmSDFData, MaxChannels),
          ControlReg2(owner->LogicMemory, mmSDFData, MaxChannels),
          OffsetReg(owner->LogicMemory, mmSDFData, MaxChannels),
          GainReg(owner->LogicMemory, mmSDFData, MaxChannels),
          OverrangeReg(owner->LogicMemory, mmSDFData, MaxChannels),
          DataReg(owner->LogicMemory, mmSDFData, MaxChannels),
          Debug(true)
        {
        AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
            romX3_SDF_InputCoeffOffset, romX3_SDF_RangeSize));
        }
    // Dtor
    ~X3SDFInputDevice()
        {
        }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad7760ControlRegister  --  A/D control #1
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ad7760ControlRegister1 : public X3SDFShadowRegister
    {
        typedef X3SDFShadowRegister inherited;

    public:
        RegisterBitGroup      Decimation;
        RegisterBit           Filter1Enable;
        RegisterBit           Filter3Enable;
        RegisterBitGroup      FilterLength;
        RegisterBit           Sync;
        RegisterBit           ReadStatus;
        RegisterBit           ReadOffset;
        RegisterBit           ReadGain;
        RegisterBit           ReadOverrange;
        RegisterBit           DownloadFilter;

        Ad7760ControlRegister1(AddressingSpace &space,
            int offset)
            :  inherited(space, offset, 0x1a, 1),
               Decimation(*this, 0, 3), Filter1Enable(*this, 3),
               Filter3Enable(*this, 4), FilterLength(*this, 5, 4),
               Sync(*this, 9), ReadStatus(*this, 11),
               ReadOffset(*this, 12), ReadGain(*this, 13),
               ReadOverrange(*this, 14), DownloadFilter(*this, 15),
               Debug(true)
            {
            //  Default setup ????
            }

        virtual void ShadowRegisterChanged()
            {
            inherited::ShadowRegisterChanged();
            // Bits 15..9 are self-clearing
            mShadowValue &= 0x01ff;
            Debug.Log("Ad7760ControlRegister1: ",  (int)mShadowValue, 16);
            }
    private:
        Trace           Debug;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad7760ControlRegister  --  A/D control #2
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ad7760ControlRegister2 : public X3SDFShadowRegister
    {
        typedef X3SDFShadowRegister inherited;

    public:
        RegisterBit           DiffAmpPowerDown;
        RegisterBit           LowPower;
        RegisterBit           PowerDown;
        RegisterBit           ClockDivider;

        Ad7760ControlRegister2(AddressingSpace &space,
            int offset)
            :  inherited(space, offset, 0x02, 2),
               DiffAmpPowerDown(*this, 0), LowPower(*this, 2),
               PowerDown(*this, 3), ClockDivider(*this, 5),
               Debug(true)
            {
            //  Default setup ????
            }

        virtual void ShadowRegisterChanged()
            {
            inherited::ShadowRegisterChanged();
            Debug.Log("Ad7760ControlRegister2: ",  (int)mShadowValue, 16);
            }
    private:
        Trace           Debug;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad7760StatusRegister  --  A/D status register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ad7760StatusRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Decimation;
        RegisterBit           Filter1Enable;
        RegisterBit           Filter3Enable;
        RegisterBit           UserDefinedFilter;
        RegisterBit           FilterChecksumOk;
        RegisterBit           FilterDownloadOk;
        RegisterBit           Overrange;
        RegisterBit           LowPower;
        RegisterBitGroup      DieNumber;
        RegisterBitGroup      PartNumber;

        Ad7760StatusRegister(AddressingSpace & space,
            int offset, ii32 value)
            :  inherited(space, offset, value),
               Decimation(*this, 0, 3), Filter1Enable(*this, 3),
               Filter3Enable(*this, 4), UserDefinedFilter(*this, 5),
               FilterChecksumOk(*this, 6), FilterDownloadOk(*this, 7),
               Overrange(*this, 8), LowPower(*this, 9),
               DieNumber(*this, 11, 3), PartNumber(*this, 14, 2)
            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad7760GainRegister  -- Gain coef access
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ad7760GainRegister : public X3SDFShadowRegister
    {
        typedef X3SDFShadowRegister inherited;

    public:
        Ad7760GainRegister(AddressingSpace &space,
            int offset)
            :  inherited(space, offset, 0x8000, 4)
            {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad7760OffsetRegister  --  A/D coef offset register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ad7760OffsetRegister : public X3SDFShadowRegister
    {
        typedef X3SDFShadowRegister inherited;

    public:
        Ad7760OffsetRegister(AddressingSpace &space,
            int offset)
            :  inherited(space, offset, 0, 3)
            {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS Ad7760OverrangeRegister  --  A/D overrange register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class Ad7760OverrangeRegister : public X3SDFShadowRegister
    {
        typedef X3SDFShadowRegister inherited;

    public:
        Ad7760OverrangeRegister(AddressingSpace &space,
            int offset)
            :  inherited(space, offset, 0xcccc, 5)
            {  }
    };

    // Programmable filter support
    bool CustomCoefficients(int ch)
        {
            int flen = static_cast<int>(((Coefficient[ch].size()) / 3) - 1);
            ControlReg1[ch].FilterLength.Value(flen);
            ControlReg1[ch].DownloadFilter.Value(1);
            FilterReg[ch].Load(Coefficient[ch]);

            bool result = Status(ch).FilterDownloadOk.Value() ? true : false;

            if (!result)
                Debug.Log("Invalid custom filter coefficients");

            return result;
        }
    bool DefaultCoefficients(int ch)
        {
            ControlReg1[ch].FilterLength.Value(0);
//            ControlReg1[ch].DownloadFilter.Value(0);

            bool result = Status(ch).FilterDownloadOk.Value() ? false : true;

            if (!result)
                Debug.Log("Invalid default filter coefficients");

            return result;
        }

    // Overrides: Apply to ROM and hardware
    virtual void		SetCalibrationOffset(size_t /*range*/, int ch, int value)
        {
        // Each count in the offset reg is .78125% of reference
        // Counts are then placed in a 16 bit register

        const float correction = (((100.0 / 0.78125) * (1<<15))  / (1<<23));
        value = static_cast<int>(value * correction);

        OffsetReg[ch].Value(value);
        }
    virtual void 		SetCalibrationGain(size_t /*range*/, int ch, int value)
        {
        // Gain reg affects input gain at 1:1 ratio.  No correction required
        GainReg[ch].Value(value);
        }

    Ad7760StatusRegister    Status(int ch)
        {
        ControlReg1[ch].ReadStatus.Value(1);
        int result = DataReg[ch].Value();

        std::stringstream msg;
        msg << "Ad7760StatusRegister: " << std::hex << result;
        Debug.Log(msg.str().c_str());

        return Ad7760StatusRegister(Fallow, 0, result);
        }

    // Registers
    RegisterArray<X3SDFFilter>                  FilterReg;
    RegisterArray<Ad7760ControlRegister1>       ControlReg1;
    RegisterArray<Ad7760ControlRegister2>       ControlReg2;
    RegisterArray<Ad7760OffsetRegister>         OffsetReg;
    RegisterArray<Ad7760GainRegister>           GainReg;
    RegisterArray<Ad7760OverrangeRegister>      OverrangeReg;
    RegisterArray<Register>                     DataReg;
    AddressingSpace                             Fallow;
    Trace                                       Debug;
};

static X3IoDeviceRegisters X3SdfInputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, 0,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };

//===========================================================================
//  CLASS X3SDFDeviceMap  -- Register & low level support for SDF module
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X3SDFDeviceMap
//---------------------------------------------------------------------------

X3SDFDeviceMap::X3SDFDeviceMap(X3SDF * owner)
    : IUsesStreamDeviceEventCallbacks(owner),
      FClockSelect(csInternal),
      FInput(new X3SDFInputDevice(this, X3SdfInputDeviceRegisters)),
      FExternalSyncEnable(false), FLowPowerEnable(false),
      Owner(owner),
      Debug(true),
      Clock(new X3SDFClock(this, 0, LogicMemory, mmPll)),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romX3_SDF_ClockParamOffset, romX3_SDF_ClockParamSize
                            ),
      F1(false), F2(1), F3(false), ClkDiv(2)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3SDFDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3SDFDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3SDFDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3SDFDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3SDFDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X3SDFDeviceMap
//---------------------------------------------------------------------------

X3SDFDeviceMap::~X3SDFDeviceMap()
{
    delete Clock;
    Clock = 0;

    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3SDFDeviceMap::OpenHardware()
{
    X3DeviceMap::OpenHardware();
    // Configure A/Ds
    const size_t range = 0;
    for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
        {
        Input().PacketId(0x0);
        Input().PacketSize(0x4000);
        FInput->Gain(range, ch, 1.0);
        FInput->Offset(range, ch, 0.0);
        FInput->Unframed();
        }
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3SDFDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    X3DeviceMap::ConfigureHardware();

    // Reapply Hardware Settings for  X3 IO Devices
//    FInput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

    //
    //  Apply Clock Settings from saved state
    Clock->Reapply();

    //
    //  Apply Gain/Offset Coefficients
    if (FInput->Calibrated(0))
        FInput->ApplyRomCoefficients(0);
    else
        FInput->ApplyDefaultCoefficients(0);

    if (!FExternalSyncEnable)
        Sync();

    //
    //  Apply input settings
    for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
        {
        FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
        Input().PacketSize(Input().PacketSize());
        }

    // Configure triggers
    FInput->TriggerReg.Apply();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3SDFDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //
    //  Start execution
    UserFpgaControlReg.InGo.Set();
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3SDFDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3SDFDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3SDFDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.Apply();

    Trigger(tInput, false);

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::CheckValidFVCO() -- Check if freq is valid Fvco.
//---------------------------------------------------------------------------

bool X3SDFDeviceMap::CheckValidFVCO(float freq)
{
    if ((freq >= 100.0e6f) && (freq <= 140.0e6f))
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::TryTuningPLL() -- Try tuning the PLL Fvco to match freq.
//---------------------------------------------------------------------------

bool X3SDFDeviceMap::TryTuningPLL(float freq)
{
    for (int curr_outdiv = 32; curr_outdiv >= 1; curr_outdiv -= 2)
        {
        if (CheckValidFVCO(curr_outdiv * freq))
            return true;
        }
    return false;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::CheckValidMClk() -- Check if freq is valid MClk.
//---------------------------------------------------------------------------

bool X3SDFDeviceMap::CheckValidMClk(float freq)
{
    if ((freq >= 1.0e6f) && (freq <= 40.0e6f))
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::CheckValidIClk() -- Check if freq is valid IClk.
//---------------------------------------------------------------------------

bool X3SDFDeviceMap::CheckValidIClk(float freq)
{
    if ((freq >= 500.0e3f) && (freq <= 20.0e6f))
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::DecimationFactor() -- Calculate A/D decimation factor
//---------------------------------------------------------------------------

int X3SDFDeviceMap::DecimationFactor(float rate)
{
    if (rate==0)
        return 1;

    double error = numeric_limits<double>::max();
    int result = 1;

    // Check all possibilities of ClkDiv, F1, F2 and F3.
    // Possible values for;
    // ClkDiv = 1, 2
    // F1 = 1, 4
    // F2 = 1, 2, 4, 8, 16, 32
    // F3 = 1, 2
    for (int curr_clkdiv = 2; curr_clkdiv >= 1; curr_clkdiv--)
        for (int curr_F1 = 4; curr_F1 >= 1; curr_F1 -= 3)
            for (int curr_F2 = 32; curr_F2 >= 1; curr_F2 >>= 1)
                for (int curr_F3 = 2; curr_F3 >= 1; curr_F3--)
                    {
                    float IClk = curr_F1 * curr_F2 * curr_F3 * rate;
                    float MClk = curr_clkdiv * curr_F1 * curr_F2 * curr_F3 * rate;
                    if (CheckValidMClk(MClk) && CheckValidIClk(IClk))
                        {
                        // If PLL can be tuned then set all the values and return decimation factor.
                        if (TryTuningPLL(MClk))
                            {
                            float rem = fmod(IClk, rate);
                            if (rem < error)
                                {
                                error = rem;
                                ClkDiv = curr_clkdiv;
                                F1 = (curr_F1 == 4)?true:false;
                                F2 = curr_F2;
                                F3 = (curr_F3 == 2)?true:false;
                                result = curr_F1 * curr_F2 * curr_F3 * curr_clkdiv;
                                }
                            }
                        }
                    }

    // Return factor or unity, if no match
    return result;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::ConfigureFilters() -- Configure A/D filters and modes
//---------------------------------------------------------------------------

void X3SDFDeviceMap::ConfigureFilters(float rate)
{
    // Insure A/D out of reset after clock is running
    ResetAdcs();

    Debug.Log("X3SDFDeviceMap::ConfigureFilters");

    int result = DecimationFactor(rate);
    const int F2_log = FloorLog2(F2);

    Debug.Log("ClkDiv: ", ClkDiv);
    Debug.Log("F1: ", F1);
    Debug.Log("F2: ", F2);
    Debug.Log("F2_log: ", F2_log);
    Debug.Log("F3: ", F3);
    Debug.Log("Decimation factor: ", result);

    for (int i = 0; i < FInput->Info().Channels().Channels(); ++i)
        {
        bool enabled = Input().Info().Channels().Enabled(i);

        // 0=Powered-up
        FInput->ControlReg2[i].PowerDown.Value(enabled ? false : true);
        FInput->ControlReg2[i].DiffAmpPowerDown.Value(enabled ? false : true);

        if (!enabled)
            continue;

        // true = /1, false = /2
        FInput->ControlReg2[i].ClockDivider.Value((ClkDiv==1) ? true : false);
        // false=Better performance, more power
        FInput->ControlReg2[i].LowPower.Value(FLowPowerEnable);

        FInput->ControlReg1[i].Filter1Enable.Value(F1);
        FInput->ControlReg1[i].Filter3Enable.Value(F3);
        FInput->ControlReg1[i].Decimation.Value(F2_log);

        // Apply custom coefficients, if loaded
        if (FInput->UsingCustomCoefficients(i))
            FInput->CustomCoefficients(i);
        else
            FInput->DefaultCoefficients(i);

        //
        //  Overrange Regsiter represents Full Scale as 0xFFFF
        //  ...set to 95 percent of Full Scale
        FInput->OverrangeReg[i].Value(static_cast<int>((1 << 16) * .95));
        }
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::ResetAdcs() -- Reset A/D devices
//---------------------------------------------------------------------------

void X3SDFDeviceMap::ResetAdcs()
{
    Sleep(10);
    FInput->AnalogControlReg.Reset.Toggle();
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::Sync() -- Issue software sync
//---------------------------------------------------------------------------

void X3SDFDeviceMap::Sync()
{
    FInput->AnalogControlReg.Sync.Toggle();
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3FilteredInputDevice & X3SDFDeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::ExternalSyncEnable() --
//---------------------------------------------------------------------------

void X3SDFDeviceMap::ExternalSyncEnable(bool state)
{
    FExternalSyncEnable=state;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::ExternalSyncEnable() --
//---------------------------------------------------------------------------

bool X3SDFDeviceMap::ExternalSyncEnable() const
{
    return FExternalSyncEnable;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::LowPowerEnable() --
//---------------------------------------------------------------------------

void X3SDFDeviceMap::LowPowerEnable(bool state)
{
    FLowPowerEnable=state;
}

//---------------------------------------------------------------------------
//  X3SDFDeviceMap::LowPowerEnable() --
//---------------------------------------------------------------------------

bool X3SDFDeviceMap::LowPowerEnable() const
{
    return FLowPowerEnable;
}


} // namespace Innovative


