// X3SDFDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3SDFDeviceMap_MbH
#define X3SDFDeviceMap_MbH

#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <Ad9511_Mb.h>
#include <vector>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X3FilteredInputDevice : public X3RangedIoDevice
{
    friend class X3SDFDeviceMap;
    friend class X3SDFInputDevice;
    typedef X3RangedIoDevice inherited;
public:
    typedef std::vector<double> DoubleArray;

    // Ctor
    X3FilteredInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

    // Programmable filter support
    // Coefficients to be specified from center of symmetry outwards,
    // as illustrated in Table 12 of AD7760 data sheet.  Thus,
    // coefficient array should be 1/2 length of filter
    void ResetCustomCoefficients(int ch)
        {  Coefficient[ch].clear();  }
    void LoadCustomCoefficients(int ch, const DoubleArray & coef)
        {  Coefficient[ch] = coef;  }
    bool UsingCustomCoefficients(int ch) const
        {  return Coefficient[ch].size() ? true : false;  }

protected:
    // Methods
    virtual void		SetRange(int ch, IIRange value);
    virtual IIRange     GetRange(int ch) const;

private:
    DoubleArray     Coefficient[4];
};

//===========================================================================
//  CLASS X3SDFDeviceMap  -- Register & low level support for SDF module
//===========================================================================
class X3SDF;
class X3SDFClock;
class X3SDFInputDevice;

class X3SDFDeviceMap : public X3DeviceMap,
                       public IUsesStreamDeviceEventCallbacks,
                       public IUsesNullFrontPanelPort
{
    friend class X3SDF;
    friend class X3SDFInputDevice;
    friend class X3SDFClock;

public:
    enum IIClkSelect { csInternal, csExternal };

    //
    //  Member Functions
    X3SDFDeviceMap(X3SDF * owner);
    virtual ~X3SDFDeviceMap();

    // Properties
    void                ExternalSyncEnable(bool state);
    bool                ExternalSyncEnable() const;

    void                LowPowerEnable(bool state);
    bool                LowPowerEnable() const;

    int                 DecimationFactor(float rate);

    // Data
    X3FilteredInputDevice &   Input();

public: // pseudo-protected
    virtual void  OpenHardware();

protected:
    // Fields
    IIClkSelect		    FClockSelect;
    X3SDFInputDevice *  FInput;
    bool                FExternalSyncEnable;
    bool                FLowPowerEnable;

    // Data
    X3SDF *				Owner;
    Trace				Debug;
    Ad9511Pfd *         Clock;
    PmcIdromSection     ClockParamRomSection;
    // A/D filter settings
    bool                F1;
    int                 F2;
    bool                F3;
    int                 ClkDiv;

    // Event Handler Methods
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    //
    //  Support Methods
    void                ConfigureFilters(float rate);
    bool                TryTuningPLL(float freq);
    bool                CheckValidFVCO(float freq);
    bool                CheckValidMClk(float freq);
    bool                CheckValidIClk(float freq);
    void                ResetAdcs();
    void                Sync();
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
