//---------------------------------------------------------------------------
// X3SDF_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3SDF_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3SDF  -- SDF PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3SDF::X3SDF()
    :  X3Board(Device), Ram(Device.Ram), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X3SDF::~X3SDF()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3SDF::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X3SDF::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3SDF::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X3SDF::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3SDF::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X3SDF::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3SDF::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3SDF::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X3SDF::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3SDF::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3SDF::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X3SDF::Reset()
{
    Device.Reset();
}

//---------------------------------------------------------------------------
//  X3SDF::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3SDF::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3SDF::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3SDF::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3SDF::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3SDF::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X3SDF::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3SDF::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3SDF::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X3SDF::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3SDF::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X3SDF::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X3SDF::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X3SDF::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X3SDF::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3SDF::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3SDF::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3SDF::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3SDF::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & X3SDF::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3SDF::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X3SDF::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X3SDF::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X3SDF::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:		mode_value = 0;		break;
        case lmSerial:		mode_value = 1;		break;
        case lmParallel:	mode_value = 2;		break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X3SDF::PciLoopbackMode() --
//------------------------------------------------------------------------------

X3SDF::LoopbackMode X3SDF::PciLoopbackMode()
{
    X3SDF::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:		result = lmNone;		break;
        case 1:		result = lmSerial;		break;
        case 2:		result = lmParallel;	break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X3SDF::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3SDF::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X3SDF::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3SDF::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X3SDF::Clock() --
//---------------------------------------------------------------------------

Ad9511Pfd & X3SDF::Clock()
{
    return *Device.Clock;
}

//---------------------------------------------------------------------------
//  X3SDF::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X3FilteredInputDevice & X3SDF::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  X3SDF::IdRom() -- Rom info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3SDF::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::Dio() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & X3SDF::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::FrontPanel() -- Front panel port access
//---------------------------------------------------------------------------

IUsesNullFrontPanelPort & X3SDF::FrontPanel()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X3SDF::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X3DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X3SDF::ExternalSyncEnable() -- Set external sync enable state
//---------------------------------------------------------------------------

void  X3SDF::ExternalSyncEnable(bool state)
{
    Device.ExternalSyncEnable(state);
}

//---------------------------------------------------------------------------
//  X3SDF::ExternalSyncEnable() -- Return external sync enable state
//---------------------------------------------------------------------------

bool  X3SDF::ExternalSyncEnable() const
{
    return Device.ExternalSyncEnable();
}

//---------------------------------------------------------------------------
//  X3SDF::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX3Alerts & X3SDF::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X3SDF::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X3SDF::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SDF::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X3SDF::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X3SDF::ClockTestEnable() --
//---------------------------------------------------------------------------

void X3SDF::ClockTestEnable(bool state)
{
    return Device.ClockTestEnable(state);
}

//---------------------------------------------------------------------------
//  X3SDF::TestCounterEnable() --
//---------------------------------------------------------------------------

bool X3SDF::TestCounterEnable() const
{
    return Device.TestCounterEnable();
}

//---------------------------------------------------------------------------
//  X3SDF::TestCounterEnable() --
//---------------------------------------------------------------------------

void X3SDF::TestCounterEnable(bool state)
{
    Device.TestCounterEnable(state);
}

//---------------------------------------------------------------------------
//  X3SDF::LowPowerEnable() --
//---------------------------------------------------------------------------

void X3SDF::LowPowerEnable(bool state)
{
    Device.LowPowerEnable(state);
}

//---------------------------------------------------------------------------
//  X3SDF::LowPowerEnable() --
//---------------------------------------------------------------------------

bool X3SDF::LowPowerEnable() const
{
    return Device.LowPowerEnable();
}

//---------------------------------------------------------------------------
//  X3SDF::DecimationFactor() --  Retreive ext clock needed for spec'd sample rate
//---------------------------------------------------------------------------

int X3SDF::DecimationFactor(float freq)
{
    return Device.DecimationFactor(freq);
}

//---------------------------------------------------------------------------
//  X3SDF::BusMasterSize() --
//---------------------------------------------------------------------------

void X3SDF::BusMasterSize(int bytes)
{
    Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3SDF::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X3SDF::IncomingBusMasterSize(int bytes)
{
    Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3SDF::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X3SDF::OutgoingBusMasterSize(int bytes)
{
    Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3SDF::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3SDF::IncomingBusMasterSize() const
{
    return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3SDF::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3SDF::OutgoingBusMasterSize() const
{
    return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3SDF::Master() --  Set master status
//---------------------------------------------------------------------------

void X3SDF::Master(bool state)
{
    Device.Master(state);
}

//---------------------------------------------------------------------------
//  X3SDF::Master() --  Query master status
//---------------------------------------------------------------------------

bool X3SDF::Master() const
{
    return Device.Master();
}

//---------------------------------------------------------------------------
//  X3SDF::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3SDF::ExternalClockSource(IIExtClockSource src)
{
    Device.ExternalClockSource((src == esPxi) ? X3DeviceMap::esPxi : X3DeviceMap::esPanel);
}

//---------------------------------------------------------------------------
//  X3SDF::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3SDF::IIExtClockSource X3SDF::ExternalClockSource() const
{
    return (Device.ExternalClockSource() == X3DeviceMap::esPxi) ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3SDF::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3SDF::ReferenceSource(IIReferenceSource src)
{
    Device.ReferenceSource((src == rsPxi) ? X3DeviceMap::rsPxi : X3DeviceMap::rsCrystal);
}

//---------------------------------------------------------------------------
//  X3SDF::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3SDF::IIReferenceSource X3SDF::ReferenceSource() const
{
    return (Device.ReferenceSource() == X3DeviceMap::rsPxi) ? rsPxi : rsCrystal;
}

//---------------------------------------------------------------------------
//  X3SDF::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X3SDF::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll    ");

    engine.Evaluate(": ?X3-SDF "
                    "    cr .\"   ?X3-SDF  : display this message \" "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll   \" "
                    "    cr  "
                    "    cr .\" ---- X3-SDF Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr  ; ");

    engine.Display("X3-SDF -specific wordset loaded \n" );

    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X3SDF::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X3SDF::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X3SDF::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X3SDF::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X3SDF::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X3SDF::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X3SDF::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X3SDF::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X3SDF::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X3SDF::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Clock().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  X3SDF::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X3SDF::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Clock().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X3SDF::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X3SDF::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Clock().Frequency(freq);
            Clock().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  X3SDF::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X3SDF::Do_DFreqActual(int dev)
{
    const int kPll = 0;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Clock().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
