//---------------------------------------------------------------------------
// X3SD_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3SD_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3SD  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3SD::X3SD()
    :  X3Board(Device), Ram(Device.Ram), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X3SD::~X3SD()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3SD::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3SD::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X3SD::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X3SD::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3SD::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X3SD::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3SD::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X3SD::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3SD::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3SD::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3SD::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X3SD::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X3SD::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3SD::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3SD::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3SD::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3SD::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3SD::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X3SD::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3SD::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3SD::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X3SD::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3SD::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X3SD::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X3SD::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X3SD::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X3SD::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3SD::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3SD::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3SD::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3SD::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & X3SD::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3SD::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X3SD::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X3SD::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X3SD::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:		mode_value = 0;		break;
        case lmSerial:		mode_value = 1;		break;
        case lmParallel:	mode_value = 2;		break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X3SD::PciLoopbackMode() --
//------------------------------------------------------------------------------

X3SD::LoopbackMode X3SD::PciLoopbackMode()
{
    X3SD::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:		result = lmNone;		break;
        case 1:		result = lmSerial;		break;
        case 2:		result = lmParallel;	break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X3SD::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3SD::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X3SD::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3SD::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X3SD::Clock() --
//---------------------------------------------------------------------------

Ad9511Pfd & X3SD::Clock()
{
    return *Device.Clock;
}

//---------------------------------------------------------------------------
//  X3SD::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X3SDInputDevice & X3SD::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  X3SD::Dio() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & X3SD::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::FrontPanel() -- Front panel port access
//---------------------------------------------------------------------------

IUsesNullFrontPanelPort & X3SD::FrontPanel()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3SD::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X3SD::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X3DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X3SD::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX3Alerts & X3SD::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X3SD::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

const LogicTemperatureIntf & X3SD::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3SD::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X3SD::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X3SD::TestCounterEnable() --
//---------------------------------------------------------------------------

bool X3SD::TestCounterEnable() const
{
    return Device.TestCounterEnable();
}

//---------------------------------------------------------------------------
//  X3SD::TestCounterEnable() --
//---------------------------------------------------------------------------

void X3SD::TestCounterEnable(bool state)
{
    Device.TestCounterEnable(state);
}

//---------------------------------------------------------------------------
//  X3SD::BusMasterSize() --
//---------------------------------------------------------------------------

void X3SD::BusMasterSize(int bytes)
{
    Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3SD::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X3SD::IncomingBusMasterSize(int bytes)
{
    Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3SD::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X3SD::OutgoingBusMasterSize(int bytes)
{
    Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3SD::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3SD::IncomingBusMasterSize() const
{
    return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3SD::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3SD::OutgoingBusMasterSize() const
{
    return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3SD::Master() --  Set master status
//---------------------------------------------------------------------------

void X3SD::Master(bool state)
{
    Device.Master(state);
}

//---------------------------------------------------------------------------
//  X3SD::Master() --  Query master status
//---------------------------------------------------------------------------

bool X3SD::Master() const
{
    return Device.Master();
}

//---------------------------------------------------------------------------
//  X3SD::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3SD::ExternalClockSource(IIExtClockSource src)
{
    Device.ExternalClockSource((src == esPxi) ? X3DeviceMap::esPxi : X3DeviceMap::esPanel);
}

//---------------------------------------------------------------------------
//  X3SD::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3SD::IIExtClockSource X3SD::ExternalClockSource() const
{
    return (Device.ExternalClockSource() == X3DeviceMap::esPxi) ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3SD::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3SD::ReferenceSource(IIReferenceSource src)
{
    Device.ReferenceSource((src == rsPxi) ? X3DeviceMap::rsPxi : X3DeviceMap::rsCrystal);
}

//---------------------------------------------------------------------------
//  X3SD::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3SD::IIReferenceSource X3SD::ReferenceSource() const
{
    return (Device.ReferenceSource() == X3DeviceMap::rsPxi) ? rsPxi : rsCrystal;
}

//---------------------------------------------------------------------------
//  X3SD::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X3SD::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll    ");

    engine.Evaluate(": ?X3-SD "
                    "    cr .\"   ?X3-SD  : display this message \" "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  \" "
                    "    cr  "
                    "    cr .\" ---- X3-SD Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr  ; ");

    engine.Display("X3-SD -specific wordset loaded \n" );

    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X3SD::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X3SD::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X3SD::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X3SD::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X3SD::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X3SD::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X3SD::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X3SD::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X3SD::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X3SD::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Clock().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  X3SD::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X3SD::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Clock().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X3SD::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X3SD::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Clock().Frequency(freq);
            Clock().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  X3SD::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X3SD::Do_DFreqActual(int dev)
{
    const int kPll = 0;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Clock().FrequencyActual());
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
