// X3ServoDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2008 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3Servo_Mb.h>
#include <X3ServoDeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>
#include <math.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    Input_MaxChannels  = 12;
const int    Input_Width_Bits   = 16;
const int    Input_MuxCount     = 0;
const bool   Input_IsUnipolar   = false;
const int    Input_ClockFactor  = 20;
const float  Input_MaxRate      = 250.e3;
const float  Input_MinRate      = 0.0;
const int    Input_GangSize     = 1;
const float  Input_ReferenceClk = 100.0e6;
const float  Input_VcoMinClk    = 100.0e6;
const float  Input_VcoMaxClk    = 140.0e6;

const int    Output_MaxChannels  = 12;
const int    Output_Width_Bits   = 16;
const int    Output_MuxCount     = 0;
const bool   Output_IsUnipolar   = false;
const int    Output_ClockFactor  = 20;
const float  Output_MaxRate      = 2.0e6;
const float  Output_MinRate      = 0.0;
const int    Output_GangSize     = 1;
const float  Output_ReferenceClk = 100.0e6;
const float  Output_VcoMinClk    = 100.0e6;
const float  Output_VcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (12 channels x 2 values + 1 calibration flag + 1 spare) x 4 ranges
const int romX3Servo_InputCoeffOffset = 0;
const int romX3Servo_InChannels = Input_MaxChannels;
const int romX3Servo_Ranges = 4;
const int romX3Servo_RangeSize = romX3Servo_InChannels * 2 + 1 + 1;
const int romX3Servo_InputCoeffSize = romX3Servo_RangeSize * romX3Servo_Ranges + 4;
const int romX3Servo_InputCoeff_END = romX3Servo_InputCoeffOffset + romX3Servo_InputCoeffSize;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3Servo_ClockParamOffset = romX3Servo_InputCoeff_END;
const int romX3Servo_ClockParamSize = 4;
const int romX3Servo_ClockParam_END = romX3Servo_ClockParamOffset + romX3Servo_ClockParamSize;

//  ...Third  section: Output Coefficient Section Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3Servo_OutputCoeffOffset  = romX3Servo_ClockParam_END;
const int romX3Servo_OutChannels        = Output_MaxChannels;
const int romX3Servo_OutputCoeffSize    = romX3Servo_OutChannels * 2 + 1 + 1;
const int romX3Servo_OutputCoeff_END    = romX3Servo_OutputCoeffOffset + romX3Servo_OutputCoeffSize;

//===========================================================================
//  CLASS X3ServoInputInfoIntf  -- Information about Inputs
//===========================================================================

class X3ServoInputInfoIntf : public IoInfoIntf
{
public:
	X3ServoInputInfoIntf()
		: FClockFactor(Input_ClockFactor),
		       //    devices,           bits,            muxChannelCount,   unipolar
          FChannels(Input_MaxChannels, Input_Width_Bits, Input_MuxCount,  Input_IsUnipolar)
		{
            FSpan.push_back(SpanInfo(10000, -10000));
		    FSpan.push_back(SpanInfo(5000, -5000));
		    FSpan.push_back(SpanInfo(2000, -2000));
		    FSpan.push_back(SpanInfo(1000, -1000));
        }
	virtual ~X3ServoInputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return Input_MaxRate;  }
	virtual float               		MinRate() const			{  return Input_MinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return Input_GangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return Input_MuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return Input_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

	void                                SetClockFactor(unsigned int factor)
											{  FClockFactor = factor;  }
private:
	unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3ServoInputInfoIntf  -- Information about Outputs
//===========================================================================

class X3ServoOutputInfoIntf : public IoInfoIntf
{
public:
	X3ServoOutputInfoIntf()
		: FClockFactor(Output_ClockFactor),
			   //    devices,                 bits,        muxChannelCount,      unipolar
		  FChannels(Output_MaxChannels, Output_Width_Bits, Output_MuxCount,  Output_IsUnipolar)
		{  FSpan.push_back(SpanInfo(10000, -10000));  }
	virtual ~X3ServoOutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return Output_MaxRate;  }
	virtual float               		MinRate() const			{  return Output_MinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return Output_GangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return Output_MuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return Output_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

	void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
	unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3ServoInputDevice  -- Register & low level support for X3
//===========================================================================

class X3ServoInputDevice : public X3RangedIoDevice
{
	friend class X3ServoDeviceMap;

public:
    // Ctor
	X3ServoInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
		: X3RangedIoDevice(owner, reg, new X3ServoInputInfoIntf, 0x10000)  /* 'one' value for gain registers */
        {
        for (int i = 0; i < romX3Servo_Ranges; ++i)
            {
            AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
                i * romX3Servo_RangeSize, romX3Servo_RangeSize));
            }
        }
	// Dtor
	~X3ServoInputDevice()
        {
        }
};

//===========================================================================
//  CLASS X3ServoOutputDevice  -- X3Servo Output Device Definitions
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for X3ServoOutputDevice
//---------------------------------------------------------------------------

class X3ServoOutputDevice : public OutDevice
{
	typedef OutDevice inherited;
	friend class X3ServoDeviceMap;

public:

protected:
	X3ServoOutputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

};

//---------------------------------------------------------------------------
//  constructor for X3ServoOutputDevice
//---------------------------------------------------------------------------

X3ServoOutputDevice::X3ServoOutputDevice(X3DeviceMap * owner,
										 X3IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X3ServoOutputInfoIntf,
				PmcIdromSection(owner->Rom(),
							    PmcIdrom::waModule,
								romX3Servo_OutputCoeffOffset,
								romX3Servo_OutputCoeffSize),
				0x10000)	// 'one' value for gain registers
{
}

//===========================================================================

static X3IoDeviceRegisters X3ServoInputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, mmInGainMode,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };

//===========================================================================

static X3IoDeviceRegisters X3ServoOutputDeviceRegisters =
	{ mmI2c, 0,
      mmOutControl, mmOutGainCoef, mmOutOffsetCoef, 0,
      mmOutChannelEnable, /*mmOutAmpEnable*/ 0, mmOutTriggerControl,
      mmOutDecimation, mmOutPacketHeader, 0
	};


//=============================================================================
//  CLASS X3ServoClock  --  PLL timebase control
//=============================================================================

class X3ServoClock : public Ad9511Duo
{
public:
	//
	//  Member Functions
    X3ServoClock(X3ServoDeviceMap * owner, int index, AddressingSpace &space, int ctl_master, int ctl_slave)
        : Ad9511Duo(index, space, ctl_master, ctl_slave), Owner(owner), FPDF(100.0e3)
        {
        Reference(Input_ReferenceClk);
        PfdRate(PhaseDetectorFrequency());
        }
	X3ServoClock(X3ServoDeviceMap * owner, double freq, double ref,
		bool enabled, int index, AddressingSpace & space, int ctl_master, int ctl_slave)
		: Ad9511Duo(freq, ref, enabled, index, space, ctl_master, ctl_slave), Owner(owner), FPDF(100.0e3)
		{
        PfdRate(PhaseDetectorFrequency());
		}

	void  ConfigureSlaveOutputs(Ad9511 & clk, int outdiv_bits, int outdiv_bypass)
		{
		// Identical duty cycle and phase for all outputs
		for (int i = 0; i < 8; ++i)
			{
			clk.Port(0x48+i*2, outdiv_bits);	        // duty cycle
			clk.Port(0x48+(i*2+1), outdiv_bypass);	    // phase
			}

		clk.Port(0x3C, 0x08);   // Clock 0 LVPECL on @810mV
		clk.Port(0x3D, 0x03);   // ...powered off
		clk.Port(0x3E, 0x03);   // ...powered off
		clk.Port(0x3F, 0x03);   // ...powered off

		clk.Port(0x40, 0x1a);    // inverted ouput LVCMOS on @3.5 mA
		clk.Port(0x41, 0x1a);    // inverted ouput LVCMOS on @3.5 mA
		clk.Port(0x42, 0x0a);    // output LVCMOS on @3.5 mA
		clk.Port(0x43, 0x0a);    // output LVCMOS on @3.5 mA
        }
        
    void  SetReferenceCalibrationFactor(double value)
        {
        Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
        Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));    // set calibrated field
        }
    double GetReferenceCalibrationFactor() const
		{
		//  Check Calibrated bit
		if (Owner->ClockParamRomSection.AsInt(1)=='Y')
			{
			return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
			}
		else
			return 1.0;  // default value
		}

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }
protected:
	// Data
	X3ServoDeviceMap *         Owner;
    double                     FPDF;

	virtual void    SetFrequency(double freq)
	{
        PfdRate(PhaseDetectorFrequency());
		// Adjust clock factor
		Ad9511Duo::ClockFactor(Owner->ClockFactor());

		Ad9511Duo::SetFrequency(freq);

		Owner->ResetAnalog();
	}

};

//===========================================================================
//  CLASS X3ServoDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for CLASS X3ServoDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X3ServoDeviceMap::X3ServoDeviceMap(X3Servo * owner)
	: IUsesStreamDeviceEventCallbacks(owner),
	  IUsesFrontPanelPort(LogicMemory, mmFrontPanelData, mmFrontPanelConfig),
	  FClockSelect(csInternal),
	  FInput(new X3ServoInputDevice(this, X3ServoInputDeviceRegisters)),
	  FOutput(new X3ServoOutputDevice(this, X3ServoOutputDeviceRegisters)),
	  Owner(owner),
	  Clock(new X3ServoClock(this, 0, LogicMemory, mmPll, mmSampleClockDivider)),
	  ClockParamRomSection(Rom(), PmcIdrom::waModule, romX3Servo_ClockParamOffset,
						   romX3Servo_ClockParamSize),
	  FFastClock(false)
{
	//
	//  Hook stream events from base class
	PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3ServoDeviceMap::OnStreamConfigureHandler );
	PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3ServoDeviceMap::OnBeforeStreamStartHandler );
	PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3ServoDeviceMap::OnAfterStreamStartHandler );
	PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3ServoDeviceMap::OnBeforeStreamStopHandler );
	PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3ServoDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for CLASS X3ServoDeviceMap
//---------------------------------------------------------------------------

X3ServoDeviceMap::~X3ServoDeviceMap()
{
	delete Clock;
	Clock = 0;

	delete FInput;
	FInput = 0;

	delete FOutput;
	FOutput = 0;
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::OpenHardware()
{
    // Base class initialization
    X3DeviceMap::OpenHardware();

    // Configure A/Ds
    Input().PacketId(0x0);
    Input().PacketSize(0x4000);
    Input().Unframed();
    for (size_t range = 0; range < Input().Ranges(); ++range)
        {
        for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
            {
            Input().Gain(range, ch, 1.0);
            Input().Offset(range, ch, 0.0);
            }
        }
    // Configure D/As
    Output().PacketSize(0x4000);
    Output().PacketId(0x2);
    Output().Unframed();
    for (int ch = 0; ch < Output().Info().Channels().Channels(); ++ch)
        {
        FOutput->Gain(ch, 1.0);
        FOutput->Offset(ch, 0.0);
        }

    FFastClock = HasFastClock();  // will activate FC if rom says it can
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
	X3DeviceMap::ConfigureHardware();
    
    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();
	FOutput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

	//
	//  Apply Clock Settings from saved state
	Clock->Reapply();

	//
	//  Apply Front config settings
    FrontPanelPortConfig(FrontPanelPortConfig());

	//
	//  Apply Gain/Offset Coefficients
    for (size_t i = 0; i < FInput->Ranges(); ++i)
        {
        if (FInput->Calibrated(i))
            FInput->ApplyRomCoefficients(i);
        else
            FInput->ApplyDefaultCoefficients(i);
        }

	if (FOutput->Calibrated())
		FOutput->ApplyRomCoefficients();
	else
		FOutput->ApplyDefaultCoefficients();

	//
	//  Apply input settings
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
		FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
		Input().PacketSize(Input().PacketSize());
        FInput->Range(ch, FInput->Range(ch));
		}
	//
	//  Apply output settings
	for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
		{
		FOutput->ChannelEnableReg.Bit(ch, FOutput->Info().Channels().Enabled(ch));
        FOutput->PacketId(FOutput->PacketId());
		FOutput->PacketSize(FOutput->PacketSize());
		}

	// Configure triggers
	FInput->TriggerReg.Apply();
	FOutput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
	//
	//  Start execution
	UserFpgaControlReg.InGo.Set();
	UserFpgaControlReg.OutGo.Set();
	UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Disable User Logic Streaming
	UserFpgaControlReg.InGo.Clear();
	UserFpgaControlReg.OutGo.Clear();
	UserFpgaControlReg.Apply();

	Trigger(tInput, false);

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3RangedIoDevice & X3ServoDeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

OutDevice & X3ServoDeviceMap::Output()
{
	return *FOutput;
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::ConfigureFilters() -- Configure A/D filters and modes
//---------------------------------------------------------------------------

int X3ServoDeviceMap::ConfigureFilters(float /*rate*/)
{
    int result = 1;
    return result;  // clock factor
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::ResetAnalog() -- Reset Analog Chip
//---------------------------------------------------------------------------

void  X3ServoDeviceMap::ResetAnalog()
{
	//  Reset ADC analog
	FInput->AnalogControlReg.Reset.Set();
	FInput->AnalogControlReg.Reset.Clear();
	Sleep(1);   // delay 1 ms

	//  Reset DAC analog
	FOutput->AnalogControlReg.Reset.Set();
	FOutput->AnalogControlReg.Reset.Clear();
	Sleep(1);   // delay 1 ms
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::ClockFactor() -- Query active clock factor
//---------------------------------------------------------------------------

int X3ServoDeviceMap::ClockFactor() const
{
    return FastClock() ? 20 : 1;
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::HasFastClock() -- Decode if this board has Fastclock
//---------------------------------------------------------------------------

bool  X3ServoDeviceMap::HasFastClock()
{
    unsigned int part = PartNumber();
    if (part != 80179)
        return false;       // not identifying as X3-Servo, so no FastClock

    //  We are a servo, so move on
    int rev = PciLogicPcb();
    switch (rev)
        {
        case 0:   // Rev A
            {
            if (EcoSubRevNumber() >= 5)
                return true;
            else
                return false;
            }
        case 1:   // Rev B
            {
            if (EcoSubRevNumber() >= 8)
                return true;
            else
                return false;
            }
        case 2:   // Rev C
            {
            if (DashNumber() == 0)
                if (EcoSubRevNumber() >= 5)
                    return true;
                else
                    return false;
            else
                if (EcoSubRevNumber() >= 6)
                    return true;
                else
                    return false;
            }
        case 3:   // Rev D
            {
            if (EcoSubRevNumber() >= 1)
                return true;
            else
                return false;
            }
        case 4:   // Rev E
        default:
            return true;
        }
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::FastClock() -- Query fast clock patch state
//---------------------------------------------------------------------------

void X3ServoDeviceMap::FastClock(bool state)
{
    bool hasFastClock = HasFastClock();
    if (hasFastClock)
        FFastClock = true;
    else
        FFastClock = state;

    // Determine clock patch status
    X3ServoInputInfoIntf & InInfo = dynamic_cast<X3ServoInputInfoIntf&>(Owner->Input().Info());
    InInfo.SetClockFactor(ClockFactor());
    X3ServoOutputInfoIntf & OutInfo = dynamic_cast<X3ServoOutputInfoIntf&>(Owner->Output().Info());
    OutInfo.SetClockFactor(ClockFactor());
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::FastClock() -- Set fast clock patch state
//---------------------------------------------------------------------------

bool X3ServoDeviceMap::FastClock() const
{
    return FFastClock;
}

//---------------------------------------------------------------------------
//  X3ServoDeviceMap::SetFifoDacDelay() --
//---------------------------------------------------------------------------

unsigned int X3ServoDeviceMap::SetFifoDacDelay(unsigned int ns)
{
    const ii32 TICK2NS = 15;

    //max=15*(2^26-1)
    const ii32 MAX = TICK2NS*static_cast<const ii32>(pow(2.0, 26)-1);
    if(ns >= 15 && ns <= MAX)
        {
        double t = 1.0 / (Clock->GetFrequencyActual() * Clock->Decimation() * ClockFactor()); // in secs

        double idelay = (6 * t * 1e9); // in nano secs

        Debug.Log("Dac delay factor ", idelay);

        double rawdelay = ns - idelay;

		Owner->Output().DacDelay(static_cast<unsigned int>(rawdelay / static_cast<double>(TICK2NS)));

        return (Owner->Output().DacDelay()*TICK2NS);
        }
    else
        return 0;
}

} // namespace Innovative


