// X3ServoDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2008 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3ServoDeviceMap_MbH
#define X3ServoDeviceMap_MbH

#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Ad9511Duo_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3ServoDeviceMap  -- Register & low level support for SD module
//===========================================================================
class X3Servo;
class X3ServoInputDevice;
class X3ServoClock;
class X3ServoOutputDevice;
class X3ServoDeviceMap : public X3DeviceMap,
						 public IUsesStreamDeviceEventCallbacks,
						 public IUsesFrontPanelPort

{
	friend class X3Servo;
	friend class X3ServoInputDevice;
	friend class X3ServoOutputDevice;
	friend class X3ServoClock;

public:
	enum IIClkSelect { csInternal, csExternal };

	//
	//  Member Functions
	X3ServoDeviceMap(X3Servo * owner);
	virtual ~X3ServoDeviceMap();

	X3RangedIoDevice & 	Input();
	OutDevice & 	    Output();

	// Properties
    void 		 FastClock(bool state);
    bool		 FastClock() const;

    unsigned int SetFifoDacDelay(unsigned int ns);

public: // pseudo-protected
	virtual void  OpenHardware();
	void          ResetAnalog();

protected:
	// Fields
	IIClkSelect		    FClockSelect;
	X3ServoInputDevice * 	FInput;
	X3ServoOutputDevice * 	FOutput;

	// Data
	X3Servo *			Owner;
	Trace				Debug;
	Ad9511Duo *         Clock;
	PmcIdromSection     ClockParamRomSection;
	bool				FFastClock;

	// Methods
	void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

	//
	//  Support Methods
	int                 ConfigureFilters(float rate);
	int					ClockFactor() const;

    bool  HasFastClock();

};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
