//---------------------------------------------------------------------------
// X3Servo_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3Servo_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3Servo  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3Servo::X3Servo()
	:  X3Board(Device), Ram(Device.Ram), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X3Servo::~X3Servo()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3Servo::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3Servo::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X3Servo::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X3Servo::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3Servo::BoardNames() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

vector<string> X3Servo::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3Servo::PrintAllDevices() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

string X3Servo::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3Servo::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3Servo::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3Servo::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X3Servo::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X3Servo::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3Servo::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3Servo::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3Servo::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3Servo::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3Servo::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X3Servo::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3Servo::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3Servo::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X3Servo::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3Servo::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X3Servo::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X3Servo::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X3Servo::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X3Servo::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3Servo::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3Servo::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3Servo::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3Servo::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & X3Servo::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3Servo::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X3Servo::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X3Servo::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X3Servo::PciLoopbackMode(LoopbackMode mode)
{
	// This field does not use the same bit pattern, so improvise...
	int mode_value;
	switch (mode)
		{
		default:
		case lmNone:		mode_value = 0;		break;
		case lmSerial:		mode_value = 1;		break;
		case lmParallel:	mode_value = 2;		break;
		}

	Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X3Servo::PciLoopbackMode() --
//------------------------------------------------------------------------------

X3Servo::LoopbackMode X3Servo::PciLoopbackMode()
{
	X3Servo::LoopbackMode result;
	switch (Device.PciLoopbackMode())
		{
		default:
		case 0:		result = lmNone;		break;
		case 1:		result = lmSerial;		break;
		case 2:		result = lmParallel;	break;
		}

	return result;
}

//---------------------------------------------------------------------------
//  X3Servo::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3Servo::UserLoopbackMode(bool state)
{
	Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X3Servo::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3Servo::UserLoopbackMode() const
{
	return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X3Servo::Clock() --
//---------------------------------------------------------------------------

Ad9511Duo & X3Servo::Clock()
{
	return *Device.Clock;
}

//---------------------------------------------------------------------------
//  X3Servo::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X3RangedIoDevice & X3Servo::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X3Servo::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

OutDevice & X3Servo::Output()
{
	return Device.Output();
}

//---------------------------------------------------------------------------
//  X3Servo::Dio() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & X3Servo::Dio()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::FrontPanel() -- Front panel port access
//---------------------------------------------------------------------------

IUsesFrontPanelPort & X3Servo::FrontPanel()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3Servo::IdRom()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X3Servo::Trigger(IIDirection dir, bool state)
{
	Device.Trigger(static_cast<X3DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X3Servo::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX3Alerts & X3Servo::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X3Servo::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X3Servo::Thermal() 
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3Servo::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X3Servo::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X3Servo::TestCounterEnable() --
//---------------------------------------------------------------------------

bool X3Servo::TestCounterEnable() const
{
    return Device.TestCounterEnable();
}

//---------------------------------------------------------------------------
//  X3Servo::TestCounterEnable() --
//---------------------------------------------------------------------------

void X3Servo::TestCounterEnable(bool state)
{
    Device.TestCounterEnable(state);
}

//---------------------------------------------------------------------------
//  X3Servo::BusMasterSize() --
//---------------------------------------------------------------------------

void X3Servo::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3Servo::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X3Servo::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3Servo::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X3Servo::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3Servo::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3Servo::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3Servo::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3Servo::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3Servo::Master() --  Set master status
//---------------------------------------------------------------------------

void X3Servo::Master(bool state)
{
    Device.Master(state);
}

//---------------------------------------------------------------------------
//  X3Servo::Master() --  Query master status
//---------------------------------------------------------------------------

bool X3Servo::Master() const
{
	return Device.Master();
}

//---------------------------------------------------------------------------
//  X3Servo::FastClock() --  Set fast clock status status
//---------------------------------------------------------------------------

void X3Servo::FastClock(bool state)
{
	Device.FastClock(state);
}

//---------------------------------------------------------------------------
//  X3Servo::FastClock() --  Query fast clock status status
//---------------------------------------------------------------------------

bool X3Servo::FastClock() const
{
	return Device.FastClock();
}

//---------------------------------------------------------------------------
//  X3Servo::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3Servo::ExternalClockSource(IIExtClockSource src)
{
    Device.ExternalClockSource((src == esPxi) ? X3DeviceMap::esPxi : X3DeviceMap::esPanel);
}

//---------------------------------------------------------------------------
//  X3Servo::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3Servo::IIExtClockSource X3Servo::ExternalClockSource() const
{
    return (Device.ExternalClockSource() == X3DeviceMap::esPxi) ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3Servo::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3Servo::ReferenceSource(IIReferenceSource src)
{
    Device.ReferenceSource((src == rsPxi) ? X3DeviceMap::rsPxi : X3DeviceMap::rsCrystal);
}

//---------------------------------------------------------------------------
//  X3Servo::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3Servo::IIReferenceSource X3Servo::ReferenceSource() const
{
    return (Device.ReferenceSource() == X3DeviceMap::rsPxi) ? rsPxi : rsCrystal;
}

//---------------------------------------------------------------------------
//  X3Servo::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X3Servo::AfterFiclStart(FiclSystem & engine)
{
	engine.Evaluate("  0 constant Pll    ");

	engine.Evaluate(": ?X3-Servo "
					"    cr .\"   ?X3-Servo  : display this message \" "
					"    cr .\" --- SubDevice Constants --- \" "
					"    cr .\"   Pll  Vcxo  \" "
					"    cr  "
					"    cr .\" ---- X3-Servo Definitions ---- \" "
					"    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
					"    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
					"    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
					"    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
					"    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
					"    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
					"    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
					"    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
					"    cr  ; ");

	engine.Display("X3-Servo -specific wordset loaded \n" );

	//  Replace Callbacks to call FICL too
	Device.Load_StreamDeviceEventCallbacks(
					new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X3Servo::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X3Servo::Do_LFetch(int addr)
{
	return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X3Servo::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X3Servo::Do_LStore(int addr, int value)
{
	PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X3Servo::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X3Servo::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X3Servo::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X3Servo::Do_PStore(int addr, int value)
{
	PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X3Servo::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X3Servo::Do_DFetch(int dev, int addr)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			return 0;
		case kPll:
			return Clock().Port(dev, addr);
		}
}

//---------------------------------------------------------------------------
//  X3Servo::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X3Servo::Do_DStore(int dev, int addr, int value)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Port(dev, addr, value);
			break;
		}
}

//---------------------------------------------------------------------------
//  X3Servo::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X3Servo::Do_DFreq(int dev, float freq)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Frequency(freq);
			Clock().Reapply();
			break;
		}
}

//---------------------------------------------------------------------------
//  X3Servo::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X3Servo::Do_DFreqActual(int dev)
{
	const int kPll = 0;
	switch (dev)
		{
		default:
			return 0.0f;
		case kPll:
			return static_cast<float>(Clock().FrequencyActual());
		}
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // namespace Innovative
