// X3Servo_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef X3Servo_MbH
#define X3Servo_MbH

#include <X3Board_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <X3ServoDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>
#include <Ficl/FiclSystem_Mb.h>


namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS X3  -- X3  PCI Express baseboard object
//===========================================================================

class X3Servo : public X3Board,
				public IStreamDeviceEvents,
				public IFiclTarget

{
	friend class X3ServoDeviceMap;
	friend class X3Io;

public:
    typedef Ad9511Duo Timebase;

	// Trigger Support
	// Enumerations
	enum IIDirection { tInput, tOutput, tInputOutput };
	enum LoopbackMode { lmNone, lmParallel, lmSerial };
	enum IIExtClockSource { esPanel, esPxi };
	enum IIReferenceSource { rsCrystal, rsPxi };

	//
	//  Member Functions
	X3Servo();
	virtual ~X3Servo();

	// Methods
    size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
	void  						Open();
	void  						Close();
	virtual void  				Reset();
    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

	FpgaLogicVersionInfoIntf & 	UserLogicVersion();
	PciLogicVersionInfoIntf & 	PciLogicVersion();
	IPciBusDiagnostics & 		PciDiagnostics();

	ii32  						PeekLogic( int offset );
	void  						PokeLogic( int offset, ii32 value );
	ii32  						PeekPort( int offset );
	void  						PokePort( int offset, ii32 value );

	// Properties
	int   						Target();
	void  						Target(int value);

	bool  						Led();
	void  						Led(bool value);

	void						UserLoopbackMode(bool state);
	bool                        UserLoopbackMode() const;

	void            			PciLoopbackMode(LoopbackMode mode);
	LoopbackMode    			PciLoopbackMode();

	void                		Trigger(IIDirection dir, bool state);

	void                		ClockTestEnable(bool state);
	bool                		ClockTestEnable() const;

	void               	        ExternalClockSource(IIExtClockSource src);
	IIExtClockSource            ExternalClockSource() const;

	void               	        ReferenceSource(IIReferenceSource src);
	IIReferenceSource           ReferenceSource() const;

	void               	        TestCounterEnable(bool state);
	bool          		        TestCounterEnable() const;

	void 						BusMasterSize(int bytes);
	void 						IncomingBusMasterSize(int bytes);
	void 						OutgoingBusMasterSize(int bytes);
	int             			IncomingBusMasterSize() const;
	int             			OutgoingBusMasterSize() const;

	void               	        Master(bool state);
	bool          		        Master() const;

	void 						FastClock(bool state);
	bool						FastClock() const;

	// Device access methods
	// Logic programming
	IUsesVirtexFpgaLoader &           Logic();
	// EEprom programming
	IUsesVirtexJtagLoader & 	      Eeprom();
	IUsesExtendedDioPort & 		      Dio();
	IUsesFrontPanelPort & 		      FrontPanel();

	X3RangedIoDevice & 	     	      Input();
	OutDevice & 	     		      Output();

	Ad9511Duo &               	      Clock();
	IUsesPmcEeprom &			      IdRom();
    //  Alerts
    IUsesX3Alerts &                   Alerts();
    LogicTemperatureIntf &            Thermal();
    X3ServoDeviceMap &                X3ServoDevice()
    {return Device;}

	ZbtRam &                          Ram;

public: //Pseudo protected

protected:
	//
	//  Member Data
	X3ServoDeviceMap               Device;

	//  Ficl Interface
	virtual void    AfterFiclStart(FiclSystem & engine);

	virtual int     Do_LFetch(int addr);
	virtual void    Do_LStore(int addr, int value);
	virtual int     Do_PFetch(int addr);
	virtual void    Do_PStore(int addr, int value);
	virtual int     Do_DFetch(int dev, int addr);
	virtual void    Do_DStore(int dev, int addr, int value);
	virtual void    Do_DFreq(int dev, float freq);
	virtual float   Do_DFreqActual(int dev);
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

