//---------------------------------------------------------------------------
// X3_10M_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3_10M_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3_10M  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3_10M::X3_10M()
	:  X3Board(Device), Ram(Device.Ram), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X3_10M::~X3_10M()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3_10M::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3_10M::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X3_10M::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X3_10M::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3_10M::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X3_10M::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3_10M::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X3_10M::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3_10M::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3_10M::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3_10M::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X3_10M::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X3_10M::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3_10M::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3_10M::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3_10M::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3_10M::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3_10M::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X3_10M::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3_10M::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3_10M::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X3_10M::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3_10M::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X3_10M::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X3_10M::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X3_10M::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X3_10M::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3_10M::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3_10M::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3_10M::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3_10M::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & X3_10M::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3_10M::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X3_10M::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X3_10M::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X3_10M::PciLoopbackMode(LoopbackMode mode)
{
	// This field does not use the same bit pattern, so improvise...
	int mode_value;
	switch (mode)
		{
		default:
		case lmNone:		mode_value = 0;		break;
		case lmSerial:		mode_value = 1;		break;
		case lmParallel:	mode_value = 2;		break;
		}

	Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X3_10M::PciLoopbackMode() --
//------------------------------------------------------------------------------

X3_10M::LoopbackMode X3_10M::PciLoopbackMode()
{
	X3_10M::LoopbackMode result;
	switch (Device.PciLoopbackMode())
		{
		default:
		case 0:		result = lmNone;		break;
		case 1:		result = lmSerial;		break;
		case 2:		result = lmParallel;	break;
		}

	return result;
}

//---------------------------------------------------------------------------
//  X3_10M::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3_10M::UserLoopbackMode(bool state)
{
	Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X3_10M::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3_10M::UserLoopbackMode() const
{
	return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X3_10M::Clock() --
//---------------------------------------------------------------------------

Ad9511Pfd & X3_10M::Clock()
{
	return *Device.Clock;
}

//---------------------------------------------------------------------------
//  X3_10M::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X3RangedIoDevice & X3_10M::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X3_10M::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & X3_10M::Dio()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::FrontPanel() -- Front panel port access
//---------------------------------------------------------------------------

IUsesNullFrontPanelPort & X3_10M::FrontPanel()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3_10M::IdRom()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X3_10M::Trigger(IIDirection dir, bool state)
{
	Device.Trigger(static_cast<X3DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X3_10M::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX3Alerts & X3_10M::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X3_10M::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X3_10M::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_10M::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X3_10M::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X3_10M::TestCounterEnable() --
//---------------------------------------------------------------------------

bool X3_10M::TestCounterEnable() const
{
    return Device.TestCounterEnable();
}

//---------------------------------------------------------------------------
//  X3_10M::TestCounterEnable() --
//---------------------------------------------------------------------------

void X3_10M::TestCounterEnable(bool state)
{
    Device.TestCounterEnable(state);
}

//---------------------------------------------------------------------------
//  X3_10M::BusMasterSize() --
//---------------------------------------------------------------------------

void X3_10M::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_10M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X3_10M::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_10M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X3_10M::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_10M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3_10M::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3_10M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3_10M::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3_10M::Master() --  Set master status
//---------------------------------------------------------------------------

void X3_10M::Master(bool state)
{
    Device.Master(state);
}

//---------------------------------------------------------------------------
//  X3_10M::Master() --  Query master status
//---------------------------------------------------------------------------

bool X3_10M::Master() const
{
    return Device.Master();
}

//---------------------------------------------------------------------------
//  X3_10M::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3_10M::ExternalClockSource(IIExtClockSource src)
{
    Device.ExternalClockSource((src == esPxi) ? X3DeviceMap::esPxi : X3DeviceMap::esPanel);
}

//---------------------------------------------------------------------------
//  X3_10M::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3_10M::IIExtClockSource X3_10M::ExternalClockSource() const
{
    return (Device.ExternalClockSource() == X3DeviceMap::esPxi) ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3_10M::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3_10M::ReferenceSource(IIReferenceSource src)
{
    Device.ReferenceSource((src == rsPxi) ? X3DeviceMap::rsPxi : X3DeviceMap::rsCrystal);
}

//---------------------------------------------------------------------------
//  X3_10M::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3_10M::IIReferenceSource X3_10M::ReferenceSource() const
{
    return (Device.ReferenceSource() == X3DeviceMap::rsPxi) ? rsPxi : rsCrystal;
}

//---------------------------------------------------------------------------
//  X3_10M::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X3_10M::AfterFiclStart(FiclSystem & engine)
{
	engine.Evaluate("  0 constant Pll    ");

	engine.Evaluate(": ?X3-10M "
					"    cr .\"   ?X3-10M  : display this message \" "
					"    cr .\" --- SubDevice Constants --- \" "
					"    cr .\"   Pll   \" "
					"    cr  "
					"    cr .\" ---- X3-10M Definitions ---- \" "
					"    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
					"    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
					"    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
					"    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
					"    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
					"    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
					"    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
					"    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
					"    cr  ; ");

	engine.Display("X3-10M -specific wordset loaded \n" );

	//  Replace Callbacks to call FICL too
	Device.Load_StreamDeviceEventCallbacks(
					new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X3_10M::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X3_10M::Do_LFetch(int addr)
{
	return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X3_10M::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X3_10M::Do_LStore(int addr, int value)
{
	PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X3_10M::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X3_10M::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X3_10M::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X3_10M::Do_PStore(int addr, int value)
{
	PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X3_10M::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X3_10M::Do_DFetch(int dev, int addr)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			return 0;
		case kPll:
			return Clock().Port(addr);
		}
}

//---------------------------------------------------------------------------
//  X3_10M::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X3_10M::Do_DStore(int dev, int addr, int value)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Port(addr, value);
			break;
		}
}

//---------------------------------------------------------------------------
//  X3_10M::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X3_10M::Do_DFreq(int dev, float freq)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Frequency(freq);
			Clock().Reapply();
			break;
		}
}

//---------------------------------------------------------------------------
//  X3_10M::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X3_10M::Do_DFreqActual(int dev)
{
	const int kPll = 0;
	switch (dev)
		{
		default:
			return 0.0f;
		case kPll:
			return static_cast<float>(Clock().FrequencyActual());
		}
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // namespace Innovative
