// X3_25MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2008 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3_25M_Mb.h>
#include <X3_25MDeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    Input_MaxChannels  = 2;
const int    Input_Width_Bits   = 16;
const int    Input_MuxCount     = 0;
const bool   Input_IsUnipolar   = false;
const int    Input_ClockFactor  = 1;
const float  Input_MaxRate      = 105.e6;
const float  Input_MinRate      = 0.0;
const int    Input_GangSize     = 1;
const float  Input_ReferenceClk = 100.0e6;
const float  Input_VcoMinClk    = 100.0e6;
const float  Input_VcoMaxClk    = 140.0e6;

const int    Output_MaxChannels  = 2;
const int    Output_Width_Bits   = 16;
const int    Output_MuxCount     = 0;
const bool   Output_IsUnipolar   = false;
const int    Output_ClockFactor  = 1;
const float  Output_MaxRate      = 50.0e6;
const float  Output_MinRate      = 0.0;
const int    Output_GangSize     = 1;
const float  Output_ReferenceClk = 100.0e6;
const float  Output_VcoMinClk    = 100.0e6;
const float  Output_VcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (12 channels x 2 values + 1 calibration flag + 1 spare) x 4 ranges
const int romX3_25M_InputCoeffOffset = 0;
const int romX3_25M_InChannels = Input_MaxChannels;
const int romX3_25M_Ranges = 3;
const int romX3_25M_RangeSize = romX3_25M_InChannels * 2 + 1 + 1;
const int romX3_25M_InputCoeffSize = romX3_25M_RangeSize * romX3_25M_Ranges + 4;
const int romX3_25M_InputCoeff_END = romX3_25M_InputCoeffOffset + romX3_25M_InputCoeffSize;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_25M_ClockParamOffset = romX3_25M_InputCoeff_END;
const int romX3_25M_ClockParamSize = 4;
const int romX3_25M_ClockParam_END = romX3_25M_ClockParamOffset + romX3_25M_ClockParamSize;

//  ...Third  section: Output Coefficient Section Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_25M_OutputCoeffOffset  = romX3_25M_ClockParam_END;
const int romX3_25M_OutChannels        = Output_MaxChannels;
const int romX3_25M_OutputCoeffSize    = romX3_25M_OutChannels * 2 + 1 + 1;
const int romX3_25M_OutputCoeff_END    = romX3_25M_OutputCoeffOffset + romX3_25M_OutputCoeffSize;

//===========================================================================
//  CLASS X3_25MInputInfoIntf  -- Information about Inputs
//===========================================================================

class X3_25MInputInfoIntf : public IoInfoIntf
{
public:
	X3_25MInputInfoIntf()
		: FClockFactor(Input_ClockFactor),
		       //    devices,           bits,            muxChannelCount,   unipolar
          FChannels(Input_MaxChannels, Input_Width_Bits, Input_MuxCount,  Input_IsUnipolar)
		{
        FSpan.push_back(SpanInfo(2000, -2000));
        FSpan.push_back(SpanInfo(1000, -1000));
        FSpan.push_back(SpanInfo(200, -200));
        }
	virtual ~X3_25MInputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return Input_MaxRate;  }
	virtual float               		MinRate() const			{  return Input_MinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return Input_GangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return Input_MuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return Input_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

    void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3_25MOutputInfoIntf  -- Information about Outputs
//===========================================================================

class X3_25MOutputInfoIntf : public IoInfoIntf
{
public:
	X3_25MOutputInfoIntf()
		: FClockFactor(Output_ClockFactor),
		       //    devices,                 bits,        muxChannelCount,      unipolar
          FChannels(Output_MaxChannels, Output_Width_Bits, Output_MuxCount,  Output_IsUnipolar)
		{  FSpan.push_back(SpanInfo(2000, -2000));  }
	virtual ~X3_25MOutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return Output_MaxRate;  }
	virtual float               		MinRate() const			{  return Output_MinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return Output_GangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return Output_MuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return Output_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

    void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3_25MInputDevice  -- Register & low level support for X3
//===========================================================================

class X3_25MInputDevice : public X3RangedIoDevice
{
	friend class X3_25MDeviceMap;

public:
    // Ctor
	X3_25MInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
        : X3RangedIoDevice(owner, reg, new X3_25MInputInfoIntf, 0x10000)  /* 'one' value for gain registers */
        {
        for (int i = 0; i < romX3_25M_Ranges; ++i)
            {
            AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
                i * romX3_25M_RangeSize, romX3_25M_RangeSize));
            }
        }
	// Dtor
	~X3_25MInputDevice()
        {
        }
};

//===========================================================================
//  CLASS X3_25MOutputDevice  -- X3_25M Output Device Definitions
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for X3_25MOutputDevice
//---------------------------------------------------------------------------

class X3_25MOutputDevice : public OutDevice
{
	typedef OutDevice inherited;
	friend class X3_25MDeviceMap;

public:

protected:
	X3_25MOutputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

};

//---------------------------------------------------------------------------
//  constructor for X3_25MOutputDevice
//---------------------------------------------------------------------------

X3_25MOutputDevice::X3_25MOutputDevice(X3DeviceMap * owner,
									   X3IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X3_25MOutputInfoIntf,
				PmcIdromSection(owner->Rom(),
							    PmcIdrom::waModule,
								romX3_25M_OutputCoeffOffset,
								romX3_25M_OutputCoeffSize),
				0x10000)	// 'one' value for gain registers
{
}

//===========================================================================

static X3IoDeviceRegisters X3_25MInputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, mmInGainMode,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };

//===========================================================================

static X3IoDeviceRegisters X3_25MOutputDeviceRegisters =
	{ mmI2c, 0,
      mmOutControl, mmOutGainCoef, mmOutOffsetCoef, 0,
      mmOutChannelEnable, /*mmOutAmpEnable*/ 0, mmOutTriggerControl,
      mmOutDecimation, mmOutPacketHeader, 0
	};


//=============================================================================
//  CLASS X3_25MClock  --  PLL timebase control
//=============================================================================

class X3_25MClock : public Ad9511Pfd
{
    typedef Ad9511Pfd inherited;

public:
	//
	//  Member Functions
	X3_25MClock(X3_25MDeviceMap * owner, int index, AddressingSpace &space, int ctl)
        : inherited(index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
		Tuner.PfdRate(PhaseDetectorFrequency());
        Reference(Input_ReferenceClk);
		Min(Input_VcoMinClk/MaxOutputDivider());
		Max(Input_VcoMaxClk);
        }
	X3_25MClock(X3_25MDeviceMap * owner, double freq, double ref,
        bool enabled, int index, AddressingSpace & space, int ctl)
        : inherited(freq, ref, enabled, index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
        Tuner.PfdRate(PhaseDetectorFrequency());
		Reference(Input_ReferenceClk);
		Min(Input_VcoMinClk/MaxOutputDivider());
		Max(Input_VcoMaxClk);
        }

    void  ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
        {
        PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
        OnPllConfigureOutput.Execute(e);

        // Identical duty cycle and phase for all outputs
        for (int i = 0; i < 8; ++i)
            {
            Port(0x48+i*2, outdiv_bits);	        // duty cycle
            Port(0x48+(i*2+1), outdiv_bypass);	    // phase
            }

        Port(0x3C, 0x08);   // Clock 0 LVPECL on @810mV
        Port(0x3D, 0x03);   // ...powered off
        Port(0x3E, 0x03);   // ...powered off
        Port(0x3F, 0x03);   // ...powered off

        Port(0x40, 0x02);    // LVDS
        Port(0x41, 0x02);    // LVDS
        Port(0x42, 0x0a);    // output LVCMOS on @3.5 mA
        Port(0x43, 0x0a);    // output LVCMOS on @3.5 mA

        PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
        OnAfterPllConfigureOutput.Execute(e1);
        }

    void  SetReferenceCalibrationFactor(double value)
        {
        Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
        Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));    // set calibrated field
        }
    double GetReferenceCalibrationFactor() const
        {
        //  Check Calibrated bit
        if (Owner->ClockParamRomSection.AsInt(1)=='Y')
            {
            return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
            }
        else
            return 1.0;  // default value
        }


protected:
    // Data
    X3_25MDeviceMap *         Owner;
    double                    FPDF;

    virtual void    SetFrequency(double freq)
    {
        Tuner.PfdRate(PhaseDetectorFrequency());
        inherited::SetFrequency(freq);

//        Owner->ResetAnalog();
    }
    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }


};

//===========================================================================
//  CLASS X3_25MDeviceMap  -- Register & low level support for 25M module
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for CLASS X3_25MDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X3_25MDeviceMap::X3_25MDeviceMap(X3_25M * owner)
	: IUsesStreamDeviceEventCallbacks(owner),
	  IUsesFrontPanelPort(LogicMemory, mmFrontPanelData, mmFrontPanelConfig),
	  FClockSelect(csInternal),
   	  FInput(new X3_25MInputDevice(this, X3_25MInputDeviceRegisters)),
   	  FOutput(new X3_25MOutputDevice(this, X3_25MOutputDeviceRegisters)),
	  Owner(owner),
      Clock(new X3_25MClock(this, 0, LogicMemory, mmPll)),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romX3_25M_ClockParamOffset, romX3_25M_ClockParamSize
                           )
{
	//
	//  Hook stream events from base class
	PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3_25MDeviceMap::OnStreamConfigureHandler );
	PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3_25MDeviceMap::OnBeforeStreamStartHandler );
	PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3_25MDeviceMap::OnAfterStreamStartHandler );
	PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3_25MDeviceMap::OnBeforeStreamStopHandler );
	PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3_25MDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for CLASS X3_25MDeviceMap
//---------------------------------------------------------------------------

X3_25MDeviceMap::~X3_25MDeviceMap()
{
    delete Clock;
    Clock = 0;

	delete FInput;
	FInput = 0;

	delete FOutput;
	FOutput = 0;
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::OpenHardware()
{
	X3DeviceMap::OpenHardware();

	// Configure A/Ds
    Input().PacketId(0x0);
	Input().PacketSize(0x4000);
	Input().Unframed();
	for (size_t range = 0; range < Input().Ranges(); ++range)
		{
		for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
			{
			Input().Gain(range, ch, 1.0);
			Input().Offset(range, ch, 0.0);
			}
		}
    // Configure D/As
    Output().PacketSize(0x4000);
    Output().PacketId(0x2);
	Output().Unframed();
    for (int ch = 0; ch < Output().Info().Channels().Channels(); ++ch)
        {
        FOutput->Gain(ch, 1.0);
        FOutput->Offset(ch, 0.0);
        }
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
	X3DeviceMap::ConfigureHardware();
    
    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();
	FOutput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

	//
	//  Apply Clock Settings from saved state
	Clock->Reapply();

	//
	//  Apply Front config settings
    FrontPanelPortConfig(FrontPanelPortConfig());

	//
	//  Apply Gain/Offset Coefficients
    for (size_t i = 0; i < FInput->Ranges(); ++i)
        {
        if (FInput->Calibrated(i))
            FInput->ApplyRomCoefficients(i);
        else
            FInput->ApplyDefaultCoefficients(i);
        }

	if (FOutput->Calibrated())
		FOutput->ApplyRomCoefficients();
	else
		FOutput->ApplyDefaultCoefficients();

	//
	//  Apply input settings
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
		FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
		Input().PacketSize(Input().PacketSize());
        FInput->Range(ch, FInput->Range(ch));
		}
	//
	//  Apply output settings
	for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
		{
		FOutput->ChannelEnableReg.Bit(ch, FOutput->Info().Channels().Enabled(ch));
        FOutput->PacketId(FOutput->PacketId());
		FOutput->PacketSize(FOutput->PacketSize());
		}

	// Configure triggers
	FInput->TriggerReg.Apply();
	FOutput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
	//
	//  Start execution
	UserFpgaControlReg.InGo.Set();
	UserFpgaControlReg.OutGo.Set();
	UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Disable User Logic Streaming
	UserFpgaControlReg.InGo.Clear();
	UserFpgaControlReg.OutGo.Clear();
	UserFpgaControlReg.Apply();

	Trigger(tInput, false);

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3RangedIoDevice & X3_25MDeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

OutDevice & X3_25MDeviceMap::Output()
{
	return *FOutput;
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::ConfigureFilters() -- Configure A/D filters and modes
//---------------------------------------------------------------------------

int X3_25MDeviceMap::ConfigureFilters(float /*rate*/)
{
    int result = 1;
    return result;  // clock factor
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::ResetAnalog() -- Reset Analog Chip
//---------------------------------------------------------------------------

void  X3_25MDeviceMap::ResetAnalog()
{
    //  Reset ADC analog
    FInput->AnalogControlReg.Reset.Set();
    FInput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms

    //  Reset DAC analog
    FOutput->AnalogControlReg.Reset.Set();
    FOutput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms
}


} // namespace Innovative


