// X3_2MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3_2M_Mb.h>
#include <X3_2MDeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition
const double  kVcxoPowerOnRate = 10.0f;       // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;        //

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    InputMaxChannels  = 12;
const int    InputWidth_Bits   = 16;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 10.0e6f;
const float  InputMinRate      = 0.0;
const int    InputGangSize     = 1;
const float  InputReferenceClk = 100.0e6f;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (# channels x 2 values/ch + 1 calibration flag + 1 spare) x # ranges
const int romX3_2M_InputCoeffOffset = 26;   // Note:  Workaround for previous X3_2MInputDevice ctor IDROM origin bug, now fixed.
const int romX3_2M_InChannels = InputMaxChannels;
const int romX3_2M_Ranges = 1;
const int romX3_2M_RangeSize = romX3_2M_InChannels * 2 + 1 + 1;
const int romX3_2M_InputCoeffSize = romX3_2M_RangeSize * romX3_2M_Ranges;
const int romX3_2M_InputCoeff_END = romX3_2M_InputCoeffOffset + romX3_2M_InputCoeffSize;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_2M_ClockParamOffset = romX3_2M_InputCoeff_END;
const int romX3_2M_ClockParamSize = 4;
const int romX3_2M_ClockParam_END = romX3_2M_ClockParamOffset + romX3_2M_ClockParamSize;


class X3_2MInfoIntf : public IoInfoIntf
{
public:
	X3_2MInfoIntf()
		: FClockFactor(InputClockFactor), 
		       //    devices,         bits,            muxChannelCount,   unipolar
          FChannels(InputMaxChannels, InputWidth_Bits, InputMuxCount,     InputIsUnipolar)
		{  FSpan.push_back(SpanInfo(10000, -10000));  }
	virtual ~X3_2MInfoIntf()  {  }


	virtual UnpairedChannelInfo & Channels() 		    {  return FChannels;  }
	virtual float               MaxRate() const			{  return InputMaxRate;  }
	virtual float               MinRate() const			{  return InputMinRate;  }
	virtual const SpanArray &   Span() const			{  return FSpan;  }
	virtual int                 GangSize() const		{  return InputGangSize;  }
	virtual bool                IsStackable() const		{  return false;  }
	virtual bool                IsInverted() const		{  return true;  }
	virtual bool                IsMuxed() const			{  return false;  }
	virtual bool                MuxDepth() const		{  return InputMuxCount;  }
	virtual int                 BufferSize() const		{  return 65536;  }
	virtual int                 ClockFactor() const		{  return FClockFactor;  }
	virtual float               ReferenceClock() const	{  return InputReferenceClk;  }
	virtual int               	SamplesPerFrame() const {  return 1;  }

    void                        SetClockFactor(unsigned int factor)
                                    {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//==============================================================================
//  CLASS X3_2MClock -- X3_2M specific 9511 implementation
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class X3_2MClock
//---------------------------------------------------------------------------

X3_2MClock::X3_2MClock(X3_2MDeviceMap * owner, int index,
    AddressingSpace &space, int ctl)
    : Ad9511Pfd(index, space, ctl), Owner(owner),
      Vcxo_I2C(Owner->VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      Vcxo(Vcxo_I2C, Si570::tcC, kVcxoPowerOnRate), FPDF(100.0e3)
{
    LoadVco(new IVcxoControlProxy(&Vcxo));
    UseAdvancedPBACalculation(true);
    PfdRate(PhaseDetectorFrequency());
}

//---------------------------------------------------------------------------
//  constructor for class X3_2MClock
//---------------------------------------------------------------------------

X3_2MClock::X3_2MClock(X3_2MDeviceMap * owner, double freq, double ref,
    bool enabled, int index, AddressingSpace & space, int ctl)
    : Ad9511Pfd(freq, ref, enabled, index, space, ctl), Owner(owner),
      Vcxo_I2C(Owner->VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      Vcxo(Vcxo_I2C, Si570::tcC, kVcxoPowerOnRate), FPDF(100.0e3)
{
    LoadVco(new IVcxoControlProxy(&Vcxo));
    UseAdvancedPBACalculation(true);
    PfdRate(PhaseDetectorFrequency());
}

//---------------------------------------------------------------------------
//  X3_2MClock::ConfigureOutputs()  -- Tx specific output config
//---------------------------------------------------------------------------

void X3_2MClock::ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
{
    PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
    OnPllConfigureOutput.Execute(e);

    // Identical duty cycle and phase for all outputs
    for (int i = 0; i < 8; ++i)
        {
        Port(0x48+i*2, outdiv_bits);	    // duty cycle
        Port(0x48+(i*2+1), outdiv_bypass);	// phase
        }

    Port(0x3C, 0x08);       // Clock 0 LVPECL on @810mV

    Port(0x3D, 0x03);       // ...powered off
    Port(0x3E, 0x03);       // ...powered off
    Port(0x3F, 0x03);       // ...powered off

    Port(0x40, 0x0a);       // Clocks 4..7 LVCMOS on @3.5 mA
    Port(0x41, 0x0a);
    Port(0x42, 0x0a);
    Port(0x43, 0x0a);

    PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
    OnAfterPllConfigureOutput.Execute(e1);
}


//===========================================================================
// class X3_2MInputInfoIntf
//===========================================================================

class X3_2MInputInfoIntf : public IoInfoIntf
{
public:
    X3_2MInputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(InputMaxChannels, InputWidth_Bits, InputMuxCount,  InputIsUnipolar)
        {  FSpan.push_back(SpanInfo(1000, -1000));  }
    virtual ~X3_2MInputInfoIntf()  {  }

    virtual UnpairedChannelInfo &       Channels()              {  return FChannels;}
    virtual float                       MaxRate() const         {  return InputMaxRate;  }
    virtual float                       MinRate() const         {  return InputMinRate;  }
    virtual const SpanArray &           Span() const            {  return FSpan;  }
    virtual int                         GangSize() const        {  return InputGangSize;  }
    virtual bool                        IsStackable() const     {  return true;  }
	virtual bool                        IsInverted() const      {  return true;  }
    virtual bool                        IsMuxed() const         {  return false;  }
    virtual bool                        MuxDepth() const        {  return InputMuxCount;  }
    virtual int                         BufferSize() const      {  return 65536;  }
    virtual int                         ClockFactor() const     {  return InputClockFactor;  }
    virtual float                       ReferenceClock() const  {  return InputReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 32;  }

private:
    UnpairedChannelInfo         FChannels;
    SpanArray                   FSpan;
};

//===========================================================================
//  CLASS X3_2MInputDevice  -- Module-specific input device features
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3_2MInputDevice::X3_2MInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
    : X3RangedIoDevice(owner, reg, new X3_2MInfoIntf, 0x10000)  /* 'one' value for gain registers */
{
    AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
        romX3_2M_InputCoeffOffset, romX3_2M_RangeSize));
}


//---------------------------------------------------------------------------
//  X3_2MInputDevice::SetRange() --
// Not really a ranged device, so this is a no-op
//---------------------------------------------------------------------------

void X3_2MInputDevice::SetRange(int /*ch*/, IIRange /*value*/)
{
}

//---------------------------------------------------------------------------
//  X3_2MInputDevice::GetRange() --
// Not really a ranged device, so this is a no-op
//---------------------------------------------------------------------------

X3_2MInputDevice::IIRange X3_2MInputDevice::GetRange(int /*ch*/) const
{
    return X3RangedIoDevice::x1;
}

//---------------------------------------------------------------------------
//  X3IoDeviceRegisters X3_2MInputDeviceRegisters --  Register address collection
//---------------------------------------------------------------------------

static X3IoDeviceRegisters X3_2MInputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, 0,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };


//===========================================================================
//  CLASS X3_2MDeviceMap  -- Register & low level support for SD module
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X3_2MDeviceMap::X3_2MDeviceMap(X3_2M * owner)
	: IUsesStreamDeviceEventCallbacks(owner),
      IUsesFrontPanelPort(LogicMemory, mmFrontPanelData, mmFrontPanelConfig),
      FClockSelect(csInternal),
   	  FInput(new X3_2MInputDevice(this, X3_2MInputDeviceRegisters)),
	  Owner(owner),
      Clock(this, 0, LogicMemory, mmPll),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                            romX3_2M_ClockParamOffset, romX3_2M_ClockParamSize)
{
	//
	//  Hook stream events from base class
	PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3_2MDeviceMap::OnStreamConfigureHandler );
	PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3_2MDeviceMap::OnBeforeStreamStartHandler );
	PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3_2MDeviceMap::OnAfterStreamStartHandler );
	PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3_2MDeviceMap::OnBeforeStreamStopHandler );
	PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3_2MDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X3_2MDeviceMap::~X3_2MDeviceMap()
{
	delete FInput;
	FInput = 0;
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3_2MDeviceMap::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "X3_2MDeviceMap::OpenHardware()");
	X3DeviceMap::OpenHardware();

    // Re-apply all Vcxo settings
    Debug.Log("At Vcxo Init()");
    Clock.Init();
    Debug.Log("After Vcxo Init()");

	// Configure A/Ds
	const size_t range = 0;
	for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
		{
        Input().PacketId(0x0);
		Input().PacketSize(0x4000);
		FInput->Gain(range, ch, 1.0);
		FInput->Offset(range, ch, 0.0);
		Input().Unframed();
		}
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3_2MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
	X3DeviceMap::ConfigureHardware();
    
    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

	//
	//  Apply Clock Settings from saved state
	Clock.Reapply();

        //  Apply Front config settings
    FrontPanelPortConfig(FrontPanelPortConfig());

	//
	//  Apply Gain/Offset Coefficients
    if (FInput->Calibrated(0))
    	FInput->ApplyRomCoefficients(0);
    else
    	FInput->ApplyDefaultCoefficients(0);

	//
	//  Apply input settings
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
		FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
		Input().PacketSize(Input().PacketSize());
		}

	// Configure triggers
	FInput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
	OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_2MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);

	//
	//  Start execution
	UserFpgaControlReg.InGo.Set();
	UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_2MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_2MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_2MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Disable User Logic Streaming
	UserFpgaControlReg.InGo.Clear();
	UserFpgaControlReg.Apply();

	Trigger(tInput, false);

	//  Forward notify to baseboard handler
	OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_2MDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3_2MInputDevice & X3_2MDeviceMap::Input()
{
	return *FInput;
}


} // namespace Innovative


