// X3_2MDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3_2MDeviceMap_MbH
#define X3_2MDeviceMap_MbH

#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Ad9511_Mb.h>
#include <Si570_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X3_2MDeviceMap;

//==============================================================================
//  CLASS X3_2MClock -- X3_2M specific 9511 implementation
//==============================================================================

class X3_2MClock : public Ad9511Pfd
{
    friend class X3_2MDeviceMap;
    
public:
    //
    //  Member Functions
    X3_2MClock(X3_2MDeviceMap * owner, int index, AddressingSpace &space,
           int ctl);
    X3_2MClock(X3_2MDeviceMap * owner, double freq, double ref, bool enabled,
           int index, AddressingSpace & space, int ctl);

    // Overrides
    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                                {  FPDF = freq;   }

protected:
    virtual void  ConfigureOutputs(int outdiv_bits, int outdiv_bypass);

private:
    // Data
    X3_2MDeviceMap * Owner;

    // Registers
    I2CDeviceN          Vcxo_I2C;
    Si570               Vcxo;
    double              FPDF;

    virtual void    SetFrequency(double freq)
    {
        Tuner.PfdRate(PhaseDetectorFrequency());
        Ad9511Pfd::SetFrequency(freq);
    }

    // Open-time initialization
    void                Init()
        {  Vcxo.Init();  }
};

//===========================================================================
//  CLASS X3_2MInputDevice  -- Module-specific input device features
//===========================================================================

class X3_2MInputDevice : public X3RangedIoDevice
{
	friend class X3_2MDeviceMap;

public:
	// Ctor
	X3_2MInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

protected:
    // Data
    Trace               Debug;

    // Methods
	// Overrides: Apply to ROM and hardware
	virtual void		SetRange(int ch, IIRange value);
	virtual IIRange     GetRange(int ch) const;
};

//===========================================================================
//  CLASS X3_2MDeviceMap  -- Register & low level support for SD module
//===========================================================================
class X3_2M;

class X3_2MDeviceMap : public X3DeviceMap,
                       public IUsesStreamDeviceEventCallbacks,
				       public IUsesFrontPanelPort
{
	friend class X3_2M;
    friend class X3_2MInputDevice;
    friend class X3_2MClock;
    friend class X3_2M_Pll;

public:
	enum IIClkSelect { csInternal, csExternal };

	//
	//  Member Functions
	X3_2MDeviceMap(X3_2M * owner);
	virtual ~X3_2MDeviceMap();

	X3_2MInputDevice & 	Input();

public: // pseudo-protected
	virtual void  OpenHardware();

protected:
	// Fields
	IIClkSelect		    FClockSelect;
	X3_2MInputDevice * 	FInput;

	// Data
	X3_2M *				Owner;

    Ad9511 &            Pll()
        {  return Clock;  }
    Si570 &             Vcxo()
        {  return Clock.Vcxo;  }
    X3_2MClock          Clock;
    Trace				Debug;
    PmcIdromSection     ClockParamRomSection;

	// Methods
	void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
