//---------------------------------------------------------------------------
// X3_2M_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3_2M_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3_2M  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3_2M::X3_2M()
	:  X3Board(Device), Ram(Device.Ram), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X3_2M::~X3_2M()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3_2M::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3_2M::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X3_2M::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X3_2M::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3_2M::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X3_2M::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3_2M::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X3_2M::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3_2M::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3_2M::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3_2M::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X3_2M::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X3_2M::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3_2M::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3_2M::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3_2M::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3_2M::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3_2M::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X3_2M::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3_2M::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3_2M::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X3_2M::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3_2M::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X3_2M::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X3_2M::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X3_2M::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X3_2M::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3_2M::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3_2M::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3_2M::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3_2M::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & X3_2M::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3_2M::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X3_2M::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X3_2M::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X3_2M::PciLoopbackMode(LoopbackMode mode)
{
	// This field does not use the same bit pattern, so improvise...
	int mode_value;
	switch (mode)
		{
		default:
		case lmNone:		mode_value = 0;		break;
		case lmSerial:		mode_value = 1;		break;
		case lmParallel:	mode_value = 2;		break;
		}

	Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X3_2M::PciLoopbackMode() --
//------------------------------------------------------------------------------

X3_2M::LoopbackMode X3_2M::PciLoopbackMode()
{
	X3_2M::LoopbackMode result;
	switch (Device.PciLoopbackMode())
		{
		default:
		case 0:		result = lmNone;		break;
		case 1:		result = lmSerial;		break;
		case 2:		result = lmParallel;	break;
		}

	return result;
}

//---------------------------------------------------------------------------
//  X3_2M::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3_2M::UserLoopbackMode(bool state)
{
	Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X3_2M::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3_2M::UserLoopbackMode() const
{
	return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X3_2M::Clock() --
//---------------------------------------------------------------------------

X3_2MClock & X3_2M::Clock()
{
	return Device.Clock;
}

//---------------------------------------------------------------------------
//  X3_2M::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X3_2MInputDevice & X3_2M::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X3_2M::Dio() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & X3_2M::Dio()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::FrontPanel() -- Front panel port access
//---------------------------------------------------------------------------

IUsesFrontPanelPort & X3_2M::FrontPanel()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3_2M::IdRom()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X3_2M::Trigger(IIDirection dir, bool state)
{
	Device.Trigger(static_cast<X3DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X3_2M::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX3Alerts & X3_2M::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X3_2M::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X3_2M::Thermal() 
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_2M::Pll() --  Return reference to Onboard PLL
//---------------------------------------------------------------------------

Ad9511 & X3_2M::Pll()
{
	return Device.Pll();
}

//---------------------------------------------------------------------------
//  X3_2M::Vcxo() --  Return reference to Vcxo
//---------------------------------------------------------------------------

Si570 & X3_2M::Vcxo()
{
	return Device.Vcxo();
}

//---------------------------------------------------------------------------
//  X3_2M::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X3_2M::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X3_2M::TestCounterEnable() --
//---------------------------------------------------------------------------

bool X3_2M::TestCounterEnable() const
{
    return Device.TestCounterEnable();
}

//---------------------------------------------------------------------------
//  X3_2M::TestCounterEnable() --
//---------------------------------------------------------------------------

void X3_2M::TestCounterEnable(bool state)
{
    Device.TestCounterEnable(state);
}

//---------------------------------------------------------------------------
//  X3_2M::BusMasterSize() --
//---------------------------------------------------------------------------

void X3_2M::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_2M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X3_2M::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_2M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X3_2M::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_2M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3_2M::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3_2M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3_2M::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3_2M::Master() --  Set master status
//---------------------------------------------------------------------------

void X3_2M::Master(bool state)
{
    Device.Master(state);
}

//---------------------------------------------------------------------------
//  X3_2M::Master() --  Query master status
//---------------------------------------------------------------------------

bool X3_2M::Master() const
{
    return Device.Master();
}

//---------------------------------------------------------------------------
//  X3_2M::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3_2M::ExternalClockSource(IIExtClockSource src)
{
    Device.ExternalClockSource((src == esPxi) ? X3DeviceMap::esPxi : X3DeviceMap::esPanel);
}

//---------------------------------------------------------------------------
//  X3_2M::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3_2M::IIExtClockSource X3_2M::ExternalClockSource() const
{
    return (Device.ExternalClockSource() == X3DeviceMap::esPxi) ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3_2M::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3_2M::ReferenceSource(IIReferenceSource src)
{
    Device.ReferenceSource((src == rsPxi) ? X3DeviceMap::rsPxi : X3DeviceMap::rsCrystal);
}

//---------------------------------------------------------------------------
//  X3_2M::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3_2M::IIReferenceSource X3_2M::ReferenceSource() const
{
    return (Device.ReferenceSource() == X3DeviceMap::rsPxi) ? rsPxi : rsCrystal;
}

//---------------------------------------------------------------------------
//  X3_2M::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X3_2M::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll    1 constant Vcxo  ");

    engine.Evaluate(": ?X3-2M "
                    "    cr .\"   ?X3-2M  : display this message \" "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Vcxo  \" "
                    "    cr  "
                    "    cr .\" ---- X3-2M Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr  ; ");

    engine.Display("X3-2M -specific wordset loaded \n" );

    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X3_2M::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X3_2M::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X3_2M::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X3_2M::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X3_2M::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X3_2M::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X3_2M::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X3_2M::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X3_2M::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X3_2M::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Pll().Port(addr);
        case kVcxo:
            return Vcxo().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  X3_2M::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X3_2M::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Pll().Port(addr, value);
            break;
        case kVcxo:
            Vcxo().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X3_2M::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X3_2M::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Pll().Frequency(freq);
            Pll().Reapply();
            break;
        case kVcxo:
            Vcxo().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X3_2M::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X3_2M::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;   
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(Vcxo().FrequencyActual());
        }
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // namespace Innovative
