// X3_A4D4DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2008 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3_A4D4_Mb.h>
#include <X3_A4D4DeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    Input_MaxChannels  = 4;
const int    Input_Width_Bits   = 16;
const int    Input_MuxCount     = 0;
const bool   Input_IsUnipolar   = false;
const int    Input_ClockFactor  = 1;
const float  Input_MaxRate      = 4.e6;
const float  Input_MinRate      = 0.0;
const int    Input_GangSize     = 1;
const float  Input_ReferenceClk = 100.0e6;
const float  Input_VcoMinClk    = 100.0e6;
const float  Input_VcoMaxClk    = 140.0e6;

const int    Output_MaxChannels  = 4;
const int    Output_Width_Bits   = 16;
const int    Output_MuxCount     = 0;
const bool   Output_IsUnipolar   = false;
const int    Output_ClockFactor  = 1;
const float  Output_MaxRate      = 50.0e6;
const float  Output_MinRate      = 0.0;
const int    Output_GangSize     = 1;
const float  Output_ReferenceClk = 100.0e6;
const float  Output_VcoMinClk    = 100.0e6;
const float  Output_VcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input Coefficient section
//       (12 channels x 2 values + 1 calibration flag + 1 spare) x 4 ranges
const int romX3_A4D4_InputCoeffOffset = 0;
const int romX3_A4D4_InChannels = Input_MaxChannels;
const int romX3_A4D4_Ranges = 4;
const int romX3_A4D4_RangeSize = romX3_A4D4_InChannels * 2 + 1 + 1;
const int romX3_A4D4_InputCoeffSize = romX3_A4D4_RangeSize * romX3_A4D4_Ranges + 4;
const int romX3_A4D4_InputCoeff_END = romX3_A4D4_InputCoeffOffset + romX3_A4D4_InputCoeffSize;

//  ...Second  section: Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_A4D4_ClockParamOffset = romX3_A4D4_InputCoeff_END;
const int romX3_A4D4_ClockParamSize = 4;
const int romX3_A4D4_ClockParam_END = romX3_A4D4_ClockParamOffset + romX3_A4D4_ClockParamSize;

//  ...Third  section: Output Coefficient Section Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_A4D4_OutputCoeffOffset  = romX3_A4D4_ClockParam_END;
const int romX3_A4D4_OutChannels        = Output_MaxChannels;
const int romX3_A4D4_OutputCoeffSize    = romX3_A4D4_OutChannels * 2 + 1 + 1;
const int romX3_A4D4_OutputCoeff_END    = romX3_A4D4_OutputCoeffOffset + romX3_A4D4_OutputCoeffSize;

//===========================================================================
//  CLASS X3_A4D4InputInfoIntf  -- Information about Inputs
//===========================================================================

class X3_A4D4InputInfoIntf : public IoInfoIntf
{
public:
	X3_A4D4InputInfoIntf()
		: FClockFactor(Input_ClockFactor),
		       //    devices,           bits,            muxChannelCount,   unipolar
          FChannels(Input_MaxChannels, Input_Width_Bits, Input_MuxCount,  Input_IsUnipolar)
		{
        FSpan.push_back(SpanInfo(10000, -10000));
        FSpan.push_back(SpanInfo(5000, -5000));
        FSpan.push_back(SpanInfo(2500, -2500));
        FSpan.push_back(SpanInfo(1250, -1250));
        }
	virtual ~X3_A4D4InputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return Input_MaxRate;  }
	virtual float               		MinRate() const			{  return Input_MinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return Input_GangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return Input_MuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return Input_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

    void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3_A4D4InputInfoIntf  -- Information about Outputs
//===========================================================================

class X3_A4D4OutputInfoIntf : public IoInfoIntf
{
public:
	X3_A4D4OutputInfoIntf()
		: FClockFactor(Output_ClockFactor),
		       //    devices,                 bits,        muxChannelCount,      unipolar
          FChannels(Output_MaxChannels, Output_Width_Bits, Output_MuxCount,  Output_IsUnipolar)
		{  FSpan.push_back(SpanInfo(2000, -2000));  }
	virtual ~X3_A4D4OutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return Output_MaxRate;  }
	virtual float               		MinRate() const			{  return Output_MinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return Output_GangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return Output_MuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return Output_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

    void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================
//  CLASS X3_A4D4InputDevice  -- Register & low level support for X3
//===========================================================================

class X3_A4D4InputDevice : public X3RangedIoDevice
{
	friend class X3_A4D4DeviceMap;

public:
    // Ctor
	X3_A4D4InputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
        : X3RangedIoDevice(owner, reg, new X3_A4D4InputInfoIntf, 0x10000)  /* 'one' value for gain registers */
        {
        for (int i = 0; i < romX3_A4D4_Ranges; ++i)
            {
            AppendRange(PmcIdromSection(owner->Rom(), PmcIdrom::waModule,
                i * romX3_A4D4_RangeSize, romX3_A4D4_RangeSize));
            }
        }
	// Dtor
	~X3_A4D4InputDevice()
        {
        }
};

//===========================================================================
//  CLASS X3_A4D4OutputDevice  -- X3_A4D4 Output Device Definitions
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for X3_A4D4OutputDevice
//---------------------------------------------------------------------------

class X3_A4D4OutputDevice : public OutDevice
{
	typedef OutDevice inherited;
	friend class X3_A4D4DeviceMap;

public:

protected:
	X3_A4D4OutputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

};

//---------------------------------------------------------------------------
//  constructor for X3_A4D4OutputDevice
//---------------------------------------------------------------------------

X3_A4D4OutputDevice::X3_A4D4OutputDevice(X3DeviceMap * owner,
									   X3IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X3_A4D4OutputInfoIntf,
				PmcIdromSection(owner->Rom(),
							    PmcIdrom::waModule,
								romX3_A4D4_OutputCoeffOffset,
								romX3_A4D4_OutputCoeffSize),
				0x10000)	// 'one' value for gain registers
{
}

//===========================================================================

static X3IoDeviceRegisters X3_A4D4InputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, mmInGainCoef, mmInOffsetCoef, mmInGainMode,
      mmInChannelEnable, mmInAmpEnable, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };

//===========================================================================

static X3IoDeviceRegisters X3_A4D4OutputDeviceRegisters =
	{ mmI2c, 0,
      mmOutControl, mmOutGainCoef, mmOutOffsetCoef, 0,
      mmOutChannelEnable, /*mmOutAmpEnable*/ 0, mmOutTriggerControl,
      mmOutDecimation, mmOutPacketHeader, 0
	};


//=============================================================================
//  CLASS X3_A4D4Clock  --  PLL timebase control
//=============================================================================

class X3_A4D4Clock : public Ad9511Duo
{
    typedef Ad9511Duo inherited;

public:
	//
	//  Member Functions
	X3_A4D4Clock(X3_A4D4DeviceMap * owner, int index, AddressingSpace &space, int ctl_master, int ctl_slave)
        : inherited(index, space, ctl_master, ctl_slave), Owner(owner), FPDF(100.0e3)
        {
        PfdRate(PhaseDetectorFrequency());
		Reference(Input_ReferenceClk);
        }
	X3_A4D4Clock(X3_A4D4DeviceMap * owner, double freq, double ref,
        bool enabled, int index, AddressingSpace & space, int ctl_master, int ctl_slave)
        : inherited(freq, ref, enabled, index, space, ctl_master, ctl_slave), Owner(owner), FPDF(100.0e3)
        {
        PfdRate(PhaseDetectorFrequency());
        }

    void  ConfigureSlaveOutputs(Ad9511 & clk, int outdiv_bits, int outdiv_bypass)
        {
        // Identical duty cycle and phase for all outputs
        for (int i = 0; i < 8; ++i)
            {
            clk.Port(0x48+i*2, outdiv_bits);	        // duty cycle
            clk.Port(0x48+(i*2+1), outdiv_bypass);	    // phase
            }

        clk.Port(0x3C, 0x08);   // Clock 0 LVPECL on @810mV
        clk.Port(0x3D, 0x03);   // ...powered off
        clk.Port(0x3E, 0x03);   // ...powered off
        clk.Port(0x3F, 0x03);   // ...powered off

        clk.Port(0x40, 0x1a);    // inverted ouput LVCMOS on @3.5 mA
        clk.Port(0x41, 0x1a);    // inverted ouput LVCMOS on @3.5 mA
        clk.Port(0x42, 0x0a);    // output LVCMOS on @3.5 mA
        clk.Port(0x43, 0x0a);    // output LVCMOS on @3.5 mA
        }

    void  SetReferenceCalibrationFactor(double value)
        {
        Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
        Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));    // set calibrated field
        }
    double GetReferenceCalibrationFactor() const
        {
        //  Check Calibrated bit
        if (Owner->ClockParamRomSection.AsInt(1)=='Y')
            {
            return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
            }
        else
            return 1.0;  // default value
        }

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }
protected:
    // Data
    X3_A4D4DeviceMap *         Owner;
    double                     FPDF;

    virtual void    SetFrequency(double freq)
    {
        PfdRate(PhaseDetectorFrequency());
        Ad9511Duo::SetFrequency(freq);

        Owner->ResetAnalog();
    }

};

//===========================================================================
//  CLASS X3_A4D4DeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for CLASS X3_A4D4DeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X3_A4D4DeviceMap::X3_A4D4DeviceMap(X3_A4D4 * owner)
	: IUsesStreamDeviceEventCallbacks(owner),
	  IUsesFrontPanelPort(LogicMemory, mmFrontPanelData, mmFrontPanelConfig),
	  FClockSelect(csInternal),
   	  FInput(new X3_A4D4InputDevice(this, X3_A4D4InputDeviceRegisters)),
   	  FOutput(new X3_A4D4OutputDevice(this, X3_A4D4OutputDeviceRegisters)),
	  Owner(owner),
      Clock(new X3_A4D4Clock(this, 0, LogicMemory, mmPll, mmSampleClockDivider)),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romX3_A4D4_ClockParamOffset, romX3_A4D4_ClockParamSize
                           )
{
	//
	//  Hook stream events from base class
	PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3_A4D4DeviceMap::OnStreamConfigureHandler );
	PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3_A4D4DeviceMap::OnBeforeStreamStartHandler );
	PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X3_A4D4DeviceMap::OnAfterStreamStartHandler );
	PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X3_A4D4DeviceMap::OnBeforeStreamStopHandler );
	PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3_A4D4DeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for CLASS X3_A4D4DeviceMap
//---------------------------------------------------------------------------

X3_A4D4DeviceMap::~X3_A4D4DeviceMap()
{
    delete Clock;
    Clock = 0;

	delete FInput;
	FInput = 0;

	delete FOutput;
	FOutput = 0;
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::OpenHardware()
{
	X3DeviceMap::OpenHardware();

	// Configure A/Ds
    Input().PacketId(0x0);
	Input().PacketSize(0x4000);
	Input().Unframed();
	for (size_t range = 0; range < Input().Ranges(); ++range)
		{
		for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
			{
			Input().Gain(range, ch, 1.0);
			Input().Offset(range, ch, 0.0);
			}
		}
    // Configure D/As
    Output().PacketSize(0x4000);
    Output().PacketId(0x2);
	Output().Unframed();
    for (int ch = 0; ch < Output().Info().Channels().Channels(); ++ch)
        {
        FOutput->Gain(ch, 1.0);
        FOutput->Offset(ch, 0.0);
        }
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
	X3DeviceMap::ConfigureHardware();
    
    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();
	FOutput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

	//
	//  Apply Clock Settings from saved state
	Clock->Reapply();

	//
	//  Apply Front config settings
    FrontPanelPortConfig(FrontPanelPortConfig());

	//
	//  Apply Gain/Offset Coefficients
    for (size_t i = 0; i < FInput->Ranges(); ++i)
        {
        if (FInput->Calibrated(i))
            FInput->ApplyRomCoefficients(i);
        else
            FInput->ApplyDefaultCoefficients(i);
        }

	if (FOutput->Calibrated())
		FOutput->ApplyRomCoefficients();
	else
		FOutput->ApplyDefaultCoefficients();

	//
	//  Apply input settings
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
		FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
		Input().PacketSize(Input().PacketSize());
        FInput->Range(ch, FInput->Range(ch));
		}
	//
	//  Apply output settings
	for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
		{
		FOutput->ChannelEnableReg.Bit(ch, FOutput->Info().Channels().Enabled(ch));
        FOutput->PacketId(FOutput->PacketId());
		FOutput->PacketSize(FOutput->PacketSize());
		}

	// Configure triggers
	FInput->TriggerReg.Apply();
	FOutput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
	Owner->OnBeforeStreamStart.Execute(Event);
	//
	//  Start execution
	UserFpgaControlReg.InGo.Set();
	UserFpgaControlReg.OutGo.Set();
	UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Disable User Logic Streaming
	UserFpgaControlReg.InGo.Clear();
	UserFpgaControlReg.OutGo.Clear();
	UserFpgaControlReg.Apply();

	Trigger(tInput, false);

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3RangedIoDevice & X3_A4D4DeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

OutDevice & X3_A4D4DeviceMap::Output()
{
	return *FOutput;
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::ConfigureFilters() -- Configure A/D filters and modes
//---------------------------------------------------------------------------

int X3_A4D4DeviceMap::ConfigureFilters(float /*rate*/)
{
    int result = 1;
    return result;  // clock factor
}

//---------------------------------------------------------------------------
//  X3_A4D4DeviceMap::ResetAnalog() -- Reset Analog Chip
//---------------------------------------------------------------------------

void  X3_A4D4DeviceMap::ResetAnalog()
{
    //  Reset ADC analog
    FInput->AnalogControlReg.Reset.Set();
    FInput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms

    //  Reset DAC analog
    FOutput->AnalogControlReg.Reset.Set();
    FOutput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms
}


} // namespace Innovative


