// X3_DigIODeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X3_DigIO_Mb.h>
#include <X3_DigIODeviceMap_Mb.h>
#include <X3MemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X3_MemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition

const int    InputMaxChannels  = 8;
const int    InputWidthBits    = 8;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 100.0e6;
const float  InputMinRate      = 0.0;
const int    InputGangSize     = 1;
const float  InputReferenceClk = 100.0e6;
const float  InputVcoMinClk    = 100.0e6;
const float  InputVcoMaxClk    = 140.0e6;

const int    OutputMaxChannels  = 8;
const int    OutputWidthBits    = 8;
const int    OutputMuxCount     = 0;
const bool   OutputIsUnipolar   = false;
const int    OutputClockFactor  = 1;
const float  OutputMaxRate      = 100.0e6;
const float  OutputMinRate      = 0.0;
const int    OutputGangSize     = 1;
const float  OutputReferenceClk = 100.0e6;
const float  OutputVcoMinClk    = 100.0e6;
const float  OutputVcoMaxClk    = 140.0e6;

//---------------------------------------------------------
//  IDROM Section Map:

//  ...Clock Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_DigIO_ClockParamOffset = 0;
const int romX3_DigIO_ClockParamSize = 4;
const int romX3_DigIO_ClockParam_END = romX3_DigIO_ClockParamOffset + romX3_DigIO_ClockParamSize;

//===========================================================================
//  CLASS X3_DigIOInputInfoIntf  -- Information about Inputs
//===========================================================================

class X3_DigIOInputInfoIntf : public IoInfoIntf
{
	typedef std::vector<UnpairedBlkChannelInfo> UBChannelInfoArray;
    
public:
	X3_DigIOInputInfoIntf()
		: FClockFactor(InputClockFactor),
			   //    devices,           bits,            muxChannelCount,   unipolar
		  FChannels(InputMaxChannels, InputWidthBits, InputMuxCount,  InputIsUnipolar)
		{
        FSpan.push_back(SpanInfo(3300, 0));
		}
	virtual ~X3_DigIOInputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return InputMaxRate;  }
	virtual float               		MinRate() const			{  return InputMinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return InputGangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return false;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return InputMuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return InputReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

	void                                SetClockFactor(unsigned int factor)
											{  FClockFactor = factor;  }
private:
	unsigned int            FClockFactor;
	UnpairedChannelInfo  FChannels;
	SpanArray				FSpan;
};

//===========================================================================
//  CLASS X3_DigIOOutputInfoIntf  -- Information about Outputs
//===========================================================================

class X3_DigIOOutputInfoIntf : public IoInfoIntf
{
public:
	X3_DigIOOutputInfoIntf()
		: FClockFactor(OutputClockFactor),
			   //    devices,                 bits,        muxChannelCount,      unipolar
		  FChannels(OutputMaxChannels, OutputWidthBits, OutputMuxCount,  OutputIsUnipolar)
		{  FSpan.push_back(SpanInfo(2000, -2000));  }
	virtual ~X3_DigIOOutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &	    Channels() 		        {  return FChannels;  }
	virtual float               		MaxRate() const			{  return OutputMaxRate;  }
	virtual float               		MinRate() const			{  return OutputMinRate;  }
	virtual const SpanArray &   		Span() const			{  return FSpan;  }
	virtual int                 		GangSize() const		{  return OutputGangSize;  }
	virtual bool                		IsStackable() const		{  return false;  }
	virtual bool                		IsInverted() const		{  return true;  }
	virtual bool                		IsMuxed() const			{  return false;  }
	virtual bool                		MuxDepth() const		{  return OutputMuxCount;  }
	virtual int                 		BufferSize() const		{  return 65536;  }
	virtual int                 		ClockFactor() const		{  return FClockFactor;  }
	virtual float               		ReferenceClock() const	{  return OutputReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 1;  }

	void                                SetClockFactor(unsigned int factor)
                                            {  FClockFactor = factor;  }
private:
    unsigned int                FClockFactor;
	UnpairedChannelInfo	        FChannels;
	SpanArray					FSpan;
};

//===========================================================================

static X3IoDeviceRegisters X3_DigitalInputDeviceRegisters =
    { mmI2c, 0,
      mmInControl, /*mmInGainCoef*/ 0, /*mmInOffsetCoef*/ 0, /*mmInGainMode*/ 0,
      mmInChannelEnable, /*mmInAmpEnable*/ 0, mmInTriggerControl,
      mmInDecimation, mmInPacketHeader, 0
    };

//===========================================================================

static X3IoDeviceRegisters X3_DigitalOutputDeviceRegisters =
	{ mmI2c, 0,
      mmOutControl, /*mmOutGainCoef*/ 0, /*mmOutOffsetCoef*/ 0, /*mmOutGainMode*/ 0,
      mmOutChannelEnable, /*mmOutAmpEnable*/ 0, mmOutTriggerControl,
      mmOutDecimation, mmOutPacketHeader, 0
	};

//=============================================================================
//  CLASS X3_DigIOClock  --  PLL timebase control
//=============================================================================

class X3_DigIOClock : public Ad9511Pfd
{
    typedef Ad9511Pfd inherited;

public:
	enum SamplingMode { smSingle, smDual, smQuad };
	enum ClockFactors { cf128, cf256, cf512 };
	//
	//  Member Functions
	X3_DigIOClock(X3_DigIODeviceMap * owner, int index, AddressingSpace &space, int ctl)
        : inherited(index, space, ctl), Owner(owner), FPDF(100.0e3)
        {
		Reference(InputReferenceClk);
		Min(InputVcoMinClk/MaxOutputDivider());
		Max(InputVcoMaxClk);
		PFDPolarity(true);
        Tuner.PfdRate(PhaseDetectorFrequency());
		}
	X3_DigIOClock(X3_DigIODeviceMap * owner, double freq, double ref,
		bool enabled, int index, AddressingSpace & space, int ctl)
		: inherited(freq, ref, enabled, index, space, ctl), Owner(owner), FPDF(100.0e3)
		{
		Min(InputVcoMinClk/MaxOutputDivider());
		Max(InputVcoMaxClk);
		PFDPolarity(true);
        Tuner.PfdRate(PhaseDetectorFrequency());
        }

	//  Set clock output modes and signalling types		
    void  ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
        {
        PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
        OnPllConfigureOutput.Execute(e);

        // Identical duty cycle and phase for all outputs
        for (int i = 0; i < 8; ++i)
            {
			Port(0x48+i*2, outdiv_bits);	 	// duty cycle
            Port(0x48+(i*2+1), outdiv_bypass);	// phase
            }

		Port(0x3C, 0x08);   	// Clock 0 LVPECL on @810mV
		Port(0x3D, 0x03);       // Clock 1 disabled
		Port(0x3E, 0x03);       // Clock 2 disabled
        Port(0x3F, 0x03);       // Clock 3 disabled

		Port(0x40, 0x0a);    	// Clocks 4..7 LVCMOS on @3.5 mA
		Port(0x41, 0x0a);
		Port(0x42, 0x0a);
        Port(0x43, 0x0a);

        PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
        OnAfterPllConfigureOutput.Execute(e1);
        }

    void  SetReferenceCalibrationFactor(double value)
        {
		Owner->ClockParamRomSection.AsFloat(0, static_cast<float>(value));
		// set calibrated field
		Owner->ClockParamRomSection.AsInt(1, static_cast<int>('Y'));
		}
    double GetReferenceCalibrationFactor() const
        {
        //  Check Calibrated bit
        if (Owner->ClockParamRomSection.AsInt(1)=='Y')
            {
            return static_cast<double>(Owner->ClockParamRomSection.AsFloat(0));
            }
        else
            return 1.0;  // default value
        }

protected:
    // Data
    X3_DigIODeviceMap *         Owner;
    double                      FPDF;

    virtual void    SetFrequency(double freq)
    {
        Tuner.PfdRate(PhaseDetectorFrequency());
        inherited::SetFrequency(freq);
		Owner->ResetIo();
    }

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }
};

//===========================================================================
//  CLASS X3_DigitalInputDevice  -- 
//===========================================================================

X3_DigitalInputDevice::X3_DigitalInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
	: X3RangedIoDevice(owner, reg, new X3_DigIOInputInfoIntf, 0x10000)
{
}

//===========================================================================
//  CLASS X3_DigitalOutputDevice  -- 
//===========================================================================

X3_DigitalOutputDevice::X3_DigitalOutputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg)
	: OutDevice(owner, reg, new X3_DigIOOutputInfoIntf,
				 PmcIdromSection(owner->Rom(),	PmcIdrom::waModule,
								 romX3_DigIO_ClockParam_END,
								 romX3_DigIO_ClockParam_END),
				 0x10000)	// 'one' value for gain registers
{

}

//===========================================================================
//  CLASS X3_DigIODeviceMap  -- Register & low level support for DigIO module
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for CLASS X3_DigIODeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X3_DigIODeviceMap::X3_DigIODeviceMap(X3_DigIO * owner)
	: IUsesStreamDeviceEventCallbacks(owner),
	  FClockSelect(csInternal),
	  FInput(new X3_DigitalInputDevice(this, X3_DigitalInputDeviceRegisters)),
	  FOutput(new X3_DigitalOutputDevice(this, X3_DigitalOutputDeviceRegisters)),
	  Owner(owner),
      Clock(new X3_DigIOClock(this, 0, LogicMemory, mmPll)),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romX3_DigIO_ClockParamOffset, romX3_DigIO_ClockParamSize
                           )
{
	//
	//  Hook stream events from base class
	PDMPtr()->OnStreamConfigure.SetEvent(this,   &X3_DigIODeviceMap::OnStreamConfigureHandler );
	PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X3_DigIODeviceMap::OnBeforeStreamStartHandler );
	PDMPtr()->OnAfterStreamStart.SetEvent(this,  &X3_DigIODeviceMap::OnAfterStreamStartHandler );
	PDMPtr()->OnBeforeStreamStop.SetEvent(this,  &X3_DigIODeviceMap::OnBeforeStreamStopHandler );
	PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X3_DigIODeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for CLASS X3_DigIODeviceMap
//---------------------------------------------------------------------------

X3_DigIODeviceMap::~X3_DigIODeviceMap()
{
	delete FInput;
	FInput = 0;

	delete FOutput;
	FOutput = 0;

    delete Clock;
    Clock = 0;
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::OpenHardware()
{
	X3DeviceMap::OpenHardware();

	// Configure Digital Input Stream
    Input().PacketId(0x0);
	Input().PacketSize(0x4000);
	Input().Unframed();

    // Configure Digital Output Stream
    Output().PacketSize(0x4000);
    Output().PacketId(0x2);
	Output().Unframed();
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
	X3DeviceMap::ConfigureHardware();
    
    // Reapply Hardware Settings for  X3 IO Devices
    FInput->ConfigureHardware();
	FOutput->ConfigureHardware();

    // Clear any FIFO overrun condition
    FInput->AnalogControlReg.FailedReset.Toggle();

	//
	//  Apply Clock Settings from saved state
	Clock->Reapply();


	//
	//  Apply input settings
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
		FInput->ChannelEnableReg.Bit(ch, FInput->Info().Channels().Enabled(ch));
        Input().PacketId(Input().PacketId());
		Input().PacketSize(Input().PacketSize());
        FInput->Range(ch, FInput->Range(ch));
		}
	
	//
	//  Apply output settings
	for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
		{
		FOutput->ChannelEnableReg.Bit(ch, FOutput->Info().Channels().Enabled(ch));
		Output().PacketId(Output().PacketId());
		Output().PacketSize(Output().PacketSize());
		}

	// Configure triggers
	FInput->TriggerReg.Apply();
	FOutput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
	//
	//  Start execution
	UserFpgaControlReg.InGo.Set();
	UserFpgaControlReg.OutGo.Set();
	UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_25MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Disable User Logic Streaming
	UserFpgaControlReg.InGo.Clear();
	UserFpgaControlReg.OutGo.Clear();
	UserFpgaControlReg.Apply();

	Trigger(tInput, false);

	//  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X3_DigitalInputDevice & X3_DigIODeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

X3_DigitalOutputDevice & X3_DigIODeviceMap::Output()
{
	return *FOutput;
}

//---------------------------------------------------------------------------
//  X3_DigIODeviceMap::ResetIo() -- Reset FIFOs
//---------------------------------------------------------------------------

void  X3_DigIODeviceMap::ResetIo()
{
    //  Reset input FIFO
    FInput->AnalogControlReg.Reset.Set();
    FInput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms

    //  Reset output FIFO
    FOutput->AnalogControlReg.Reset.Set();
    FOutput->AnalogControlReg.Reset.Clear();
    Sleep(1);   // delay 1 ms
}


} // namespace Innovative
