// X3_DigIODeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2008 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3_DigIODeviceMap_MbH
#define X3_DigIODeviceMap_MbH

#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Ad9511_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS X3_DigitalInputDevice  --
//===========================================================================

class X3_DigitalInputDevice : public X3RangedIoDevice
{
public:
	X3_DigitalInputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);
	~X3_DigitalInputDevice(){};

protected:
    // Methods
	virtual void		SetRange(int /*ch*/, IIRange /*value*/)
		{  }
	virtual IIRange     GetRange(int /*ch*/) const
		{  return X3RangedIoDevice::x1;  }

	virtual void		SetCalibrationOffset(size_t /*gain_range*/, int /*ch*/, int /*value*/)
		{  }
	virtual void 		SetCalibrationGain(size_t /*gain_range*/, int /*ch*/, int /*value*/)
		{  }
};

//===========================================================================
//  CLASS X3_DigitalOutputDevice  -- Register & low level support for X3
//===========================================================================

class X3_DigitalOutputDevice : public OutDevice
{
public:
	// Ctor
	X3_DigitalOutputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);
	// Dtor
	~X3_DigitalOutputDevice(){};

	//  Loading functions
	void		ApplyRomCoefficients()
		{  }
	void        ApplyDefaultCoefficients()
		{  }

	void		SetCalibrationOffset(int /*ch*/, int /*value*/)
		{  }
	void 		SetCalibrationGain(int /*ch*/, int /*value*/)
		{  }
};


//===========================================================================
//  CLASS X3_DigIODeviceMap  -- Register & low level support for Digital IO module
//===========================================================================

class X3_DigIO;
class X3_DigIOClock;

class X3_DigIODeviceMap : public X3DeviceMap,
						  public IUsesStreamDeviceEventCallbacks,
						  public IUsesNullFrontPanelPort
{
	friend class X3_DigIO;
    friend class X3_DigitalInputDevice;
	friend class X3_DigitalOutputDevice;
    friend class X3_DigIOClock;

public:
	enum IIClkSelect { csInternal, csExternal };

	//
	//  Member Functions
	X3_DigIODeviceMap(X3_DigIO * owner);
	virtual ~X3_DigIODeviceMap();

	X3_DigitalInputDevice & 	Input();
	X3_DigitalOutputDevice &	Output();
    
public: // pseudo-protected
	virtual void  OpenHardware();
    void          ResetIo();

protected:
	// Fields
	IIClkSelect		        FClockSelect;
	X3_DigitalInputDevice  * 	FInput;
	X3_DigitalOutputDevice * 	FOutput;

	// Data
	X3_DigIO *			Owner;
	Trace				Debug;
	Ad9511Pfd *         Clock;
	PmcIdromSection     ClockParamRomSection;

	// Methods
	void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
	
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; //namespace Innovative

#endif

