//---------------------------------------------------------------------------
// X3_DigIO_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3_DigIO_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3_DigIO  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X3_DigIO::X3_DigIO()
	:  X3Board(Device), Ram(Device.Ram), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X3_DigIO::~X3_DigIO()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3_DigIO::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X3_DigIO::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X3_DigIO::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3_DigIO::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X3_DigIO::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3_DigIO::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X3_DigIO::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3_DigIO::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X3_DigIO::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X3_DigIO::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3_DigIO::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3_DigIO::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3_DigIO::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3_DigIO::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3_DigIO::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X3_DigIO::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3_DigIO::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X3_DigIO::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X3_DigIO::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X3_DigIO::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X3_DigIO::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3_DigIO::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3_DigIO::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3_DigIO::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Logic() -- Return reference to FPGA object
//---------------------------------------------------------------------------

IUsesVirtexFpgaLoader & X3_DigIO::Logic()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::Eeprom() -- Return reference to Jtag loader object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3_DigIO::Eeprom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X3_DigIO::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X3_DigIO::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X3_DigIO::PciLoopbackMode(LoopbackMode mode)
{
	// This field does not use the same bit pattern, so improvise...
	int mode_value;
	switch (mode)
		{
		default:
		case lmNone:		mode_value = 0;		break;
		case lmSerial:		mode_value = 1;		break;
		case lmParallel:	mode_value = 2;		break;
		}

	Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X3_DigIO::PciLoopbackMode() --
//------------------------------------------------------------------------------

X3_DigIO::LoopbackMode X3_DigIO::PciLoopbackMode()
{
	X3_DigIO::LoopbackMode result;
	switch (Device.PciLoopbackMode())
		{
		default:
		case 0:		result = lmNone;		break;
		case 1:		result = lmSerial;		break;
		case 2:		result = lmParallel;	break;
		}

	return result;
}

//---------------------------------------------------------------------------
//  X3_DigIO::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X3_DigIO::UserLoopbackMode(bool state)
{
	Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X3_DigIO::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X3_DigIO::UserLoopbackMode() const
{
	return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Clock() --
//---------------------------------------------------------------------------

Ad9511Pfd & X3_DigIO::Clock()
{
	return *Device.Clock;
}

//---------------------------------------------------------------------------
//  X3_DigIO::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X3_DigitalInputDevice & X3_DigIO::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X3_DigIO::Output() --  Return reference to output object
//---------------------------------------------------------------------------

X3_DigitalOutputDevice & X3_DigIO::Output()
{
	return Device.Output();
}

//---------------------------------------------------------------------------
//  X3_DigIO::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesExtendedDioPort & X3_DigIO::Dio()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::FrontPanel() -- Front panel port access
//---------------------------------------------------------------------------

IUsesNullFrontPanelPort & X3_DigIO::FrontPanel()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3_DigIO::IdRom()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X3_DigIO::Trigger(IIDirection dir, bool state)
{
	Device.Trigger(static_cast<X3DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX3Alerts & X3_DigIO::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X3_DigIO::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X3_DigIO::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_DigIO::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X3_DigIO::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X3_DigIO::TestCounterEnable() --
//---------------------------------------------------------------------------

bool X3_DigIO::TestCounterEnable() const
{
    return Device.TestCounterEnable();
}

//---------------------------------------------------------------------------
//  X3_DigIO::TestCounterEnable() --
//---------------------------------------------------------------------------

void X3_DigIO::TestCounterEnable(bool state)
{
    Device.TestCounterEnable(state);
}
//---------------------------------------------------------------------------
//  X3_DigIO::BusMasterSize() --
//---------------------------------------------------------------------------

void X3_DigIO::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_DigIO::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X3_DigIO::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_DigIO::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X3_DigIO::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X3_DigIO::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3_DigIO::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3_DigIO::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X3_DigIO::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Master() --  Set master status
//---------------------------------------------------------------------------

void X3_DigIO::Master(bool state)
{
    Device.Master(state);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Master() --  Query master status
//---------------------------------------------------------------------------

bool X3_DigIO::Master() const
{
    return Device.Master();
}

//---------------------------------------------------------------------------
//  X3_DigIO::ExternalClockSource() -- Set external clock source
//---------------------------------------------------------------------------

void X3_DigIO::ExternalClockSource(IIExtClockSource src)
{
    Device.ExternalClockSource((src == esPxi) ? X3DeviceMap::esPxi : X3DeviceMap::esPanel);
}

//---------------------------------------------------------------------------
//  X3_DigIO::ExternalClockSource() --  Query external clock source
//---------------------------------------------------------------------------

X3_DigIO::IIExtClockSource X3_DigIO::ExternalClockSource() const
{
    return (Device.ExternalClockSource() == X3DeviceMap::esPxi) ? esPxi : esPanel;
}

//---------------------------------------------------------------------------
//  X3_DigIO::ReferenceSource() --  Set PLL reference source
//---------------------------------------------------------------------------

void X3_DigIO::ReferenceSource(IIReferenceSource src)
{
    Device.ReferenceSource((src == rsPxi) ? X3DeviceMap::rsPxi : X3DeviceMap::rsCrystal);
}

//---------------------------------------------------------------------------
//  X3_DigIO::ReferenceSource() --  Query PLL reference source
//---------------------------------------------------------------------------

X3_DigIO::IIReferenceSource X3_DigIO::ReferenceSource() const
{
    return (Device.ReferenceSource() == X3DeviceMap::rsPxi) ? rsPxi : rsCrystal;
}

//---------------------------------------------------------------------------
//  X3_DigIO::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X3_DigIO::AfterFiclStart(FiclSystem & engine)
{
	engine.Evaluate("  0 constant Pll    ");

	engine.Evaluate(": ?X3-DigIO "
					"    cr .\"   ?X3-DigIO  : display this message \" "
					"    cr .\" --- SubDevice Constants --- \" "
					"    cr .\"   Pll   \" "
					"    cr  "
					"    cr .\" ---- X3-DigIO Definitions ---- \" "
					"    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
					"    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
					"    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
					"    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
					"    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
					"    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
					"    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
					"    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
					"    cr  ; ");

	engine.Display("X3-DigIO -specific wordset loaded \n" );

	//  Replace Callbacks to call FICL too
	Device.Load_StreamDeviceEventCallbacks(
					new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X3_DigIO::Do_LFetch(int addr)
{
	return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X3_DigIO::Do_LStore(int addr, int value)
{
	PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X3_DigIO::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X3_DigIO::Do_PStore(int addr, int value)
{
	PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X3_DigIO::Do_DFetch(int dev, int addr)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			return 0;
		case kPll:
			return Clock().Port(addr);
		}
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X3_DigIO::Do_DStore(int dev, int addr, int value)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Port(addr, value);
			break;
		}
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X3_DigIO::Do_DFreq(int dev, float freq)
{
	const int kPll = 0;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Frequency(freq);
			Clock().Reapply();
			break;
		}
}

//---------------------------------------------------------------------------
//  X3_DigIO::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X3_DigIO::Do_DFreqActual(int dev)
{
	const int kPll = 0;
	switch (dev)
		{
		default:
			return 0.0f;
		case kPll:
			return static_cast<float>(Clock().FrequencyActual());
		}
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}  // namespace Innovative
