// X3_SD16DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3_SD16DeviceMap_MbH
#define X3_SD16DeviceMap_MbH

#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Ad9511_Mb.h>
#include <Si570_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X3_SD16DeviceMap;

//==============================================================================
//  CLASS X3_SD16Clock -- X3_SD16 specific 9511 implementation
//==============================================================================

class X3_SD16Clock : public Ad9511Pfd
{
    friend class X3_SD16DeviceMap;
    typedef Ad9511Pfd inherited;

public:
    enum SamplingMode { smHighSpeed=4, smHiRes=5,
                        smLowPower=6, smLowSpeed=7 };

    //
    //  Member Functions
    X3_SD16Clock(X3_SD16DeviceMap * owner, int index, AddressingSpace &space,
               int ctl);
    X3_SD16Clock(X3_SD16DeviceMap * owner, double freq, double ref, bool enabled,
               int index, AddressingSpace & space, int ctl);

    // Overrides
    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

protected:
    virtual void  ConfigureOutputs(int outdiv_bits, int outdiv_bypass);
    virtual void  SetFrequency(double freq);

private:
    // Properties
    void        LowPower(bool state)
        {  FLowPower = state;  }
    bool        LowPower() const
        {  return FLowPower;  }

    // Data
    X3_SD16DeviceMap * Owner;

    // Registers
    I2CDeviceN          Vcxo_I2C;
    Si570               Vcxo;
    bool                FLowPower;
    double              FPDF;
    // Methods
    void    SetReferenceCalibrationFactor(double value);
    double  GetReferenceCalibrationFactor() const;

    SamplingMode SelectSamplingMode(double freq);
    int SelectClockFactor(double freq);

    // Open-time initialization
    void                Init()
        {  Vcxo.Init();  }
};

//===========================================================================
//  CLASS X3_SD16InputDevice  -- Module-specific input device features
//===========================================================================

class X3_SD16InputDevice : public X3RangedIoDevice
{
    friend class X3_SD16DeviceMap;
    typedef X3RangedIoDevice inherited;

public:
    // Properties
    void        LowPower(bool state)
        {  FLowPower = state;  }
    bool        LowPower() const
        {  return FLowPower;  }

protected:
    // Ctor
    X3_SD16InputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);
    // Overrides

    virtual void SetCalibrationOffset(size_t range, int ch, int value);
    virtual void SetCalibrationGain(size_t range, int ch, int value);

    // Fields
    bool                FLowPower;

    // Data
    Trace               Debug;
};

//===========================================================================
//  CLASS X3_SD16OutputDevice  -- X3_SD16 Output Device Definitions
//===========================================================================

class X3_SD16OutputDevice : public OutDevice
{
    friend class X3_SD16DeviceMap;
	typedef OutDevice inherited;

public:

protected:
    X3_SD16OutputDevice(X3DeviceMap * owner, X3IoDeviceRegisters & reg);

    // Overrides
    virtual void SetCalibrationOffset(int ch, int value);
    virtual void SetCalibrationGain(int ch, int value);
};

//===========================================================================
//  CLASS X3_SD16DeviceMap  -- Register & low level support for SD module
//===========================================================================
class X3_SD16;

class X3_SD16DeviceMap : public X3DeviceMap,
                       public IUsesStreamDeviceEventCallbacks,
                       public IUsesFrontPanelPort
{
    friend class X3_SD16;
    friend class X3_SD16InputDevice;
    friend class X3_SD16Clock;
    friend class X3_SD16_Pll;

public:
    enum IIClkSelect { csInternal, csExternal };

    //
    //  Member Functions
    X3_SD16DeviceMap(X3_SD16 * owner);
    virtual ~X3_SD16DeviceMap();

    X3RangedIoDevice &  Input();
	OutDevice & 	    Output();


public: // pseudo-protected
    virtual void  OpenHardware();

protected:
    // Fields
    IIClkSelect		        FClockSelect;
    X3_SD16InputDevice * 	FInput;
    X3_SD16OutputDevice * 	FOutput;


    // Data
    X3_SD16 *				Owner;

    Ad9511Pfd &         Pll()
        {  return Clock;  }
    Si570 &             Vcxo()
        {  return Clock.Vcxo;  }
    X3_SD16Clock        Clock;
    Trace				Debug;
    PmcIdromSection     ClockParamRomSection;

    // Event handlers
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    // Methods
    void          SetFSO(unsigned int value);
    void          ResetAnalog();
    int           ConfigureFilters(float rate);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
