// X3_TimingDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <DeviceDriver_Mb.h>
#include <X3_TimingDeviceMap_Mb.h>
#include <X3_Timing_Mb.h>
#include <X3_TimingMemoryMap.h>
#include <X3_TimingDeviceDriver_Mb.h>
#include <XsvfLoadThread_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <DioPort_Mb.h>
#include <CDCE72010_Mb.h>
#include <VcxoControl_Mb.h>
#include <SystemSupport_Mb.h>
#include <cmath>
#include <Trace_Mb.h>
#include <time.h>


#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

//---------------------------------------------------------
//  Clock and Module Constants Definition
const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;    //

const int     ReferenceI2C_SubAddr  = 0x82;  //  DS4026 Address

const float  InputReferenceClk = 10.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Name, P/N, S/N
//       0x40 spare
const int rom_ParameterSize    = 0x40;

//  ...Second  section: Clock Parameters
//       (2 param (1 double) + 1 calibrated word + 1 spare)
const int romX3_Timing_ClockParamOffset = rom_ParameterSize;
const int romX3_Timing_ClockParamSize = 4;
//  ...Third  section: Variant Parameters
//       (1 param + 1 calibrated word + 2 spare)
const int romX3_Timing_VariantParamOffset = romX3_Timing_ClockParamOffset + romX3_Timing_ClockParamSize;
const int romX3_Timing_VariantParamSize = 4;


using namespace X3_Timing_MemoryMap;

static Cdce72010Registers X3_Timing_ClockRegisters =
	{ mmPllControl,  mmPllSpiIntf  };


//===========================================================================
//  CLASS X3_TimingVcxo  -- Extended Interface to X3 Timing's Vcxo
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X3_TimingVcxo
//---------------------------------------------------------------------------

X3_TimingVcxo::X3_TimingVcxo(X3_TimingDeviceMap * owner, I2CDevice & device)
	    :  Si570(device, Si570::tcA, kVcxoPowerOnRate), Owner(owner)
{
}

//---------------------------------------------------------------------------
//  X3_TimingVcxo::Init()
//---------------------------------------------------------------------------

void  X3_TimingVcxo::Init()
{
    if (IsCalibrated())
        {
        double pwrOnRate = GetVcxoCalibrationRate();
        Calibrate(pwrOnRate);
        }
    Si570::Init();
}

//---------------------------------------------------------------------------
//  X3_TimingVcxo::Reapply()
//---------------------------------------------------------------------------

void  X3_TimingVcxo::Reapply()
{
    Si570::Reapply();
}

//---------------------------------------------------------------------------
//  X3_TimingVcxo::Calibrate(double power_on_rate_MHz)
//---------------------------------------------------------------------------

void  X3_TimingVcxo::Calibrate(double power_on_rate_MHz)
{
    Si570::Calibrate(power_on_rate_MHz);
    SetVcxoCalibrationRate(power_on_rate_MHz);  // write to ROM
}

//---------------------------------------------------------------------------
//  X3_TimingVcxo::SetVcxoCalibrationRate()
//---------------------------------------------------------------------------

void  X3_TimingVcxo::SetVcxoCalibrationRate(double value)
{
    Owner->ClockParamRomSection.AsDouble(0, value);
    // set calibrated field
    Owner->ClockParamRomSection.AsInt(3, static_cast<int>('Y'));
}

//---------------------------------------------------------------------------
//  X3_TimingVcxo::GetVcxoCalibrationRate()
//---------------------------------------------------------------------------

double  X3_TimingVcxo::GetVcxoCalibrationRate() const
{
    //  Check Calibrated bit
    if (IsCalibrated())
        {
        return Owner->ClockParamRomSection.AsDouble(0);
        }
    else
        return kVcxoPowerOnRate;  // default value - assumed power on rate
}

//---------------------------------------------------------------------------
//  X3_TimingVcxo::IsCalibrated()
//---------------------------------------------------------------------------

bool  X3_TimingVcxo::IsCalibrated() const
{
    if (Owner->ClockParamRomSection.AsInt(3)=='Y')
        return true;
    else
        return false;
}

//=============================================================================
//  CLASS X3_TimingClock  --  PLL timebase control
//=============================================================================

class X3_TimingClock : public IX3_TimingClock
{
public:
	//
	//  Member Functions
	X3_TimingClock(X3_TimingDeviceMap * owner, AddressingSpace &space)
		: IX3_TimingClock(space, X3_Timing_ClockRegisters),
          Owner(owner),
          FVco(0),
          Debug(true), FPDF(10.0e3), Src(0)
		{
        SetMasterOutputSelection(1);  // Use Output 1 as guideline output

		ReferenceFrequency(0, InputReferenceClk);    // actually read from Input Mux
		ReferenceFrequency(1, InputReferenceClk);
		}
    //
    //  Properties
    // ...IX3_TimingClock interface impl
    virtual void  SetActiveVco(int idx)
        {
        FVco = idx;
        Owner->SetActiveVco(idx);
        }
    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();
//    virtual double          PhaseDetectorFrequency() const
//                                {  return (10.0e3;   }
    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void  SetPDF(double freq)  // DEBUG!!
                               {  FPDF = freq;   }

    virtual IVcxoControl &  Vcxo()
                    {
                    if (FVco==0)
                        return Owner->Vcxo;
                    else
                        return Vco_1;
                    }
                    
    virtual unsigned int PllOutputDivider()
        {  return GetDivider(1);  }
    virtual void  PllOutputDivider(unsigned int divide_by)
        {  OutputDivider(1, divide_by);  }
    virtual void  PllOutputPhase(unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(1, divide_by, phase);  }
    virtual void  PllOutputBoost(bool enabled)
        {  SetHighSwingLvpecl(1, enabled);  }
    virtual unsigned int DebugDivider()
        {  return GetDivider(8);  }
    virtual void  DebugDivider(unsigned int divide_by)
        {  OutputDivider(8, divide_by);  }
    virtual void  DebugPhase(unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(8, divide_by, phase);  }
    virtual void  DebugBoost(bool enabled)
        {  SetHighSwingLvpecl(8, enabled);  }


protected:
	// Data
	X3_TimingDeviceMap *    Owner;
    unsigned int            FVco;
    Trace                   Debug;
    double                  FPDF;

    //==============================================
    //  Vcxo Test Objects
    class VcxoIntf_1 :  public RangedVcxoControl
    {
    public:
        VcxoIntf_1() : RangedVcxoControl( 800.0e6, 1200.0e6 )  // Tuning Range
            {}
    };

    VcxoIntf_1   Vco_1;

public:
    virtual void  SetSourceTimer(IMultiTimebaseDevice * src)
        { Src = src;  }

protected:
    IMultiTimebaseDevice *  Src;     // IMux outputs
    //
    //  Update source frequency to match current setup
    virtual void  UpdateSources()
        {
        if (Src==0)
            return;
		ReferenceFrequency(0, Src->OutputFrequencyActual(0));
		ReferenceFrequency(1, Src->OutputFrequencyActual(1));
        }

};

//---------------------------------------------------------------------------
//  X3_TimingClock::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X3_TimingClock::OnInit()
{
    //  Devcie Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(true,   // DC Terminated 
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         false,  // Secondary Bias OFF
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       true,   // Input Term disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration and InUse fields in base class
    OutputInUse(0, true);     SetOutputConfig(0, ocLvPecl);
    OutputInUse(1, true);     SetOutputConfig(1, ocLvPecl);
    OutputInUse(2, false);    SetOutputConfig(2, ocDisabled);
    OutputInUse(3, false);    SetOutputConfig(3, ocDisabled);
    OutputInUse(4, false);    SetOutputConfig(4, ocDisabled);
    OutputInUse(5, false);    SetOutputConfig(5, ocDisabled);
    OutputInUse(6, true);     SetOutputConfig(6, ocLvCmos_P_Active | ocLvCmos_N_TriStated);
    OutputInUse(7, false);    SetOutputConfig(7, ocDisabled);
    OutputInUse(8, true);     SetOutputConfig(8, ocLvCmos_P_Active | ocLvCmos_N_TriStated);
    OutputInUse(9, false);    SetOutputConfig(9, ocDisabled);   // used as input
    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //    (Outputs 0 and 1 match preset value, so leave alone)
    OutputFrequencyLimit[6] = 100.0e6;
    OutputFrequencyLimit[8] = 250.0e6;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use internal bits, not external pin
    SetActiveVco(0);
    ReferenceSource(refPrimary);    // Set to use primary reference
}

//---------------------------------------------------------------------------
//  X3_TimingClock::OnReapply()
//---------------------------------------------------------------------------

void  X3_TimingClock::OnReapply()
{
}

//=============================================================================
//  CLASS X3_Timing_Gps  --  Impl of  X3_Timing GPS
//=============================================================================

class X3_Timing_Gps : public IX3_TimingGps
{
public:
    X3_Timing_Gps(X3_TimingDeviceMap * owner)
        : Owner(owner),
          UartStatusReg(owner->PortMemory, mmUartStatus),
          UartDataReg(owner->PortMemory, mmUartData),
          GpsControlReg(owner->PortMemory, mmGpsControl),
          GpsTriggerReg(owner->PortMemory, mmGpsTrigger)
            {  }

protected:
	// Data
	X3_TimingDeviceMap *     Owner;

	//
	//  Class UartStatusRegister
	class UartStatusRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBitGroup  Baud;
		RegisterBit       TxReady;
		RegisterBit       RxReady;
		RegisterBitGroup  RxDepth;

	public:
		UartStatusRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               Baud(*this, 0, 3),
			   TxReady(*this, 3), RxReady(*this, 4),
               RxDepth(*this, 5, 10)
			{
			}
	};
	//
	//  Class GpsControlRegister
	class GpsControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit       Enabled;
		RegisterBit       FifoReset;
		RegisterBit       Event;

	public:
		GpsControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Enabled(*this, 0), FifoReset(*this, 3),
              Event(*this, 4)
			{
			//  Default setup ????
			}
	};
	//
	//  Class GpsTriggerRegister
	class GpsTriggerRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit       Arm;

	public:
		GpsTriggerRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Arm(*this, 0)
			{
			//  Default setup ????
			}
	};


    UartStatusRegister                  UartStatusReg;
    Register                            UartDataReg;
    GpsControlRegister                  GpsControlReg;
    GpsTriggerRegister                  GpsTriggerReg;

    // Overrides
    virtual void  OnInit()
        {  Enabled(true);   }
    virtual void  OnReapply()
        {}
    virtual void  OnGpsTrigger();
    //
    //  Hardware Impl functions
    virtual unsigned int  UartFifoDepth();
    virtual void          UartData(char c);
    virtual char          UartData();
    virtual void          UartBaud(BaudRates baud);
    virtual BaudRates     UartBaud();
    virtual void          HdwEnabled(bool state);
    virtual bool          HdwEnabled();

    void  ArmGpsTrigger();
};

//---------------------------------------------------------------------------
//  X3_Timing_Gps::OnGpsTrigger() --
//---------------------------------------------------------------------------

void X3_Timing_Gps::OnGpsTrigger()
{
    ArmGpsTrigger();
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::UartFifoDepth() --
//---------------------------------------------------------------------------

unsigned int X3_Timing_Gps::UartFifoDepth()
{
    return UartStatusReg.RxDepth.Value();
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::UartData() --
//---------------------------------------------------------------------------

void X3_Timing_Gps::UartData(char c)
{
    while (!UartStatusReg.TxReady.Value())
        Innovative::Sleep(1);

    UartDataReg.Value(c);
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::UartData() --
//---------------------------------------------------------------------------

char X3_Timing_Gps::UartData()
{
    char c = static_cast<char>(UartDataReg.Value());
    return c;
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::UartBaud() --
//---------------------------------------------------------------------------

void X3_Timing_Gps::UartBaud(BaudRates rate)
{
    UartStatusReg.Baud.Value(rate);
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::UartBaud() --
//---------------------------------------------------------------------------

IGps::BaudRates X3_Timing_Gps::UartBaud()
{
    return static_cast<BaudRates>(UartStatusReg.Baud.Value());
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::ArmGpsTrigger() --  Fire trigger at next epoch
//---------------------------------------------------------------------------

void X3_Timing_Gps::ArmGpsTrigger()
{
    GpsTriggerReg.Arm.Value(0);
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::HdwEnabled() --  Enable/disable GPS subsystem
//---------------------------------------------------------------------------

void X3_Timing_Gps::HdwEnabled(bool state)
{
    Owner->InterruptEnable(false);

    // Reset receive FIFO
    GpsControlReg.FifoReset.Value(1);
    GpsControlReg.FifoReset.Value(state ? 0 : 1);
    Sleep(10);
    GpsControlReg.Enabled.Value(state ? 1 : 0);

    if (state)
        {
        //  Turning on...
        Owner->InterruptEnable(true);
        }
}

//---------------------------------------------------------------------------
//  X3_Timing_Gps::HdwEnabled() --  Query state of GPS subsystem
//---------------------------------------------------------------------------

bool X3_Timing_Gps::HdwEnabled()
{
    return GpsControlReg.Enabled.Value() ? true : false;
}

//===========================================================================
//  CLASS X3_TimingInterrupt  -- Simple ISR handling
//===========================================================================

class X3_TimingInterrupt : public SimpleInterrupt
{
    typedef SimpleInterrupt inherited;

public:
    X3_TimingInterrupt(ISimpleInterrupt & device)
        : inherited(device)  {  }
};

//===========================================================================
//  CLASS X3_TimingInputMux  -- Configurable Input Mux
//===========================================================================

static Cdce18005Registers X3_Timing_InputMuxRegisters =
	{ mmInputMuxSpiIntf  };

//---------------------------------------------------------------------------
//  constructor for class X3_TimingInputMux
//---------------------------------------------------------------------------

X3_TimingInputMux::X3_TimingInputMux(X3_TimingDeviceMap * owner, AddressingSpace & space)
    : Cdce18005(space, X3_Timing_InputMuxRegisters), Owner(owner)
{
}

//---------------------------------------------------------------------------
//  X3_TimingInputMux::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X3_TimingInputMux::OnInit()
{
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(false, // NOT DC Terminated (AC)
                         false, // Primary Term NOT disabled
                         false, // Secondary Term NOT disabled
                         true,  // Primary Bias ON
                         true,  // Secondary Bias ON
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    SetOutputConfig(0, ocLvPecl);
    SetOutputConfig(1, ocLvPecl);
    SetOutputConfig(2, ocLvPecl);
    SetOutputConfig(3, ocLvPecl);
    SetOutputConfig(4, ocLvCmos_P_Active | ocLvCmos_N_Inverted);

    SetOutputDividerSync(osPrimary);
    //  Configure Aux (from ?)
    SetAuxOutputConfig(aeEnabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    Source(0, inReference);
    Source(1, inGps);
    Source(2, inFrontPanel);
    Source(3, inFrontPanel);
    Source(4, inGps);
}

//---------------------------------------------------------------------------
//  X3_TimingInputMux::OnReapply()
//---------------------------------------------------------------------------

void  X3_TimingInputMux::OnReapply()
{
    // nothing special
}

//---------------------------------------------------------------------------
//  X3_TimingInputMux::OnSetPower()
//---------------------------------------------------------------------------

void  X3_TimingInputMux::OnSetPower(bool & new_state)
{
    Owner->InputMuxControlReg.PowerOn.Value(new_state);
    //  pause to allow chip to activate
    Sleep(10);
}

//---------------------------------------------------------------------------
//  X3_TimingInputMux::UpdateSources()
//---------------------------------------------------------------------------

void  X3_TimingInputMux::UpdateSources()
{
    SourceFrequency(0, InputReferenceClk);               // 10 Mhz reference
    SourceFrequency(1, Owner->FrontPanelRateMhz()*1e6);  // Front Panel Input
    SourceFrequency(2, InputReferenceClk);               // Gps 10 Mhz reference
}

//===========================================================================
//  CLASS X3_TimingOutputMux  -- Configurable Output Mux
//===========================================================================

static Cdce18005Registers X3_Timing_OutputMuxRegisters =
	{ mmOutputMuxSpiIntf  };

//---------------------------------------------------------------------------
//  constructor for class X3_TimingOutputMux
//---------------------------------------------------------------------------

X3_TimingOutputMux::X3_TimingOutputMux(X3_TimingDeviceMap * owner, AddressingSpace & space)
    : Cdce18005(space, X3_Timing_OutputMuxRegisters), Owner(owner), Src(0), SrcPll(0)
{
}

//---------------------------------------------------------------------------
//  X3_TimingOutputMux::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X3_TimingOutputMux::OnInit()
{
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(true,  // DC Terminated 
                         false, // Primary Term NOT disabled
                         false, // Secondary Term NOT disabled
                         false,  // Primary Bias OFF
                         false,  // Secondary Bias OFF
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    unsigned int outputConfig = ocLvPecl;
    if (Owner->HasCmosOutputs())
    //if (Owner->GetVariant() == x3tCmosOutput)     // Change OutputCfg if CMOS variant
       outputConfig = ocLvCmos_P_Active | ocLvCmos_N_TriStated;

    SetOutputConfig(0, outputConfig);
    SetOutputConfig(1, outputConfig);
    SetOutputConfig(2, outputConfig);
    SetOutputConfig(3, outputConfig);
    SetOutputConfig(4, ocLvCmos_P_Inverted | ocLvCmos_N_Active);

    SetOutputDividerSync(osPrimary);
    //  Configure Aux
    SetAuxOutputConfig(aeEnabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    Source(0, inPll);
    Source(1, inPll);
    Source(2, inFrontPanel);
    Source(3, inFrontPanel);
    Source(4, inGps);
}

//---------------------------------------------------------------------------
//  X3_TimingOutputMux::OnReapply()
//---------------------------------------------------------------------------

void  X3_TimingOutputMux::OnReapply()
{
}

//---------------------------------------------------------------------------
//  X3_TimingOutputMux::OnSetPower()
//---------------------------------------------------------------------------

void  X3_TimingOutputMux::OnSetPower(bool & new_state)
{
    Owner->OutputMuxControlReg.PowerOn.Value(new_state);
    //  pause to allow chip to activate
    Sleep(10);
}

//---------------------------------------------------------------------------
//  X3_TimingOutputMux::UpdateSources()
//---------------------------------------------------------------------------

void  X3_TimingOutputMux::UpdateSources()
{
    if (SrcPll!=0)
        {
        SourceFrequency(0, SrcPll->OutputFrequencyActual(0));
        }
    if (Src!=0)
        {
        SourceFrequency(1, Src->OutputFrequencyActual(2));
        SourceFrequency(2, Src->OutputFrequencyActual(4));
        }
}

//---------------------------------------------------------------------------
//  X3_TimingOutputMux::EnableAllOutputs() -- enable/disable external outputs
//---------------------------------------------------------------------------

void  X3_TimingOutputMux::EnableAllOutputs(bool enabled)
{
    OutputEnabled(0, enabled);
    OutputEnabled(1, enabled);
    OutputEnabled(2, enabled);
    OutputEnabled(3, enabled);
    WriteDataRegisters();
}

//===========================================================================
//  CLASS X3_TimingPxiMux  -- Configurable Pxi Mux
//===========================================================================

static Cdce18005Registers X3_Timing_PxiMuxRegisters =
	{ mmPxiMuxSpiIntf  };

//---------------------------------------------------------------------------
//  constructor for class X3_TimingPxiMux
//---------------------------------------------------------------------------

X3_TimingPxiMux::X3_TimingPxiMux(X3_TimingDeviceMap * owner, AddressingSpace & space)
    :  Cdce18005(space, X3_Timing_PxiMuxRegisters), Owner(owner), Src(0), SrcPll(0),
       DriveDSTAROut(false), Drive10MHzOut(false)

{
}

//---------------------------------------------------------------------------
//  X3_TimingPxiMux::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X3_TimingPxiMux::OnInit()
{
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(true,  // DC Terminated
                         false, // Primary Term NOT disabled
                         false, // Secondary Term NOT disabled
                         false, // Primary Bias OFF
                         false, // Secondary Bias OFF
                         false  // NO failsafe
                         );
    //
    //  Set Output Configuration
    if (DriveDSTAROut)
        {
        SetOutputConfig(0, ocLvPecl);
        SetOutputConfig(1, ocLvds);
        SetOutputConfig(2, ocLvds);
        }
    else
        {
        SetOutputConfig(0, ocDisabled);
        SetOutputConfig(1, ocDisabled);
        SetOutputConfig(2, ocDisabled);
        }
    if (Drive10MHzOut)
        {
        SetOutputConfig(3, ocLvCmos_P_Active | ocLvCmos_N_TriStated);
        }
    else
        {
        SetOutputConfig(3, ocDisabled);
        }

    SetOutputConfig(4, ocLvCmos_P_Active | ocLvCmos_N_TriStated);   // Trigger

    SetOutputDividerSync(osPrimary);
    //  Configure Aux (enabled, LVCmos, from ?)
    SetAuxOutputConfig(aeEnabled, aoFromDivider3);
    //
    //  Set Default Mux Settings
    Source(0, inPll);
    Source(1, inPll);
    Source(2, inFrontPanel);
    Source(3, inFrontPanel);
    Source(4, inGps);
}

//---------------------------------------------------------------------------
//  X3_TimingPxiMux::OnReapply()
//---------------------------------------------------------------------------

void  X3_TimingPxiMux::OnReapply()
{
}

//---------------------------------------------------------------------------
//  X3_TimingPxiMux::OnSetPower()
//---------------------------------------------------------------------------

void  X3_TimingPxiMux::OnSetPower(bool & new_state)
{
    Owner->PxiMuxControlReg.PowerOn.Value(new_state);
    //  pause to allow chip to activate
    Sleep(10);
}

//---------------------------------------------------------------------------
//  X3_TimingPxiMux::TriggerOutputEnable()
//---------------------------------------------------------------------------

void  X3_TimingPxiMux::TriggerOutputEnable(bool state)
{
    Owner->PxiMuxControlReg.OutputEnable.Value(state);
}

//---------------------------------------------------------------------------
//  X3_TimingPxiMux::UpdateSources()
//---------------------------------------------------------------------------

void  X3_TimingPxiMux::UpdateSources()
{
    if (SrcPll!=0)
        {
        SourceFrequency(0, SrcPll->OutputFrequencyActual(1));
        }
    if (Src!=0)
        {
        SourceFrequency(1, Src->OutputFrequencyActual(3));
        SourceFrequency(2, Src->OutputFrequencyActual(4));
        }
}

//---------------------------------------------------------------------------
//  X3_TimingPxiMux::EnableAllOutputs() -- enable/disable external outputs
//---------------------------------------------------------------------------

void  X3_TimingPxiMux::EnableAllOutputs(bool enabled)
{
    OutputEnabled(0, enabled);
    OutputEnabled(1, enabled);
    OutputEnabled(2, enabled);
    OutputEnabled(3, enabled);
    WriteDataRegisters();
}

//===========================================================================
//  CLASS X3_TimingDeviceMap  -- Device Map
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X3_TimingDeviceMap
//---------------------------------------------------------------------------

X3_TimingDeviceMap::X3_TimingDeviceMap(X3_Timing * owner)
	: IUsesVirtexJtagLoader(PortMemory, mmJtag),
	  IUsesPciLogicVersionInfo(PortMemory, mmPciVersion),
      IUsesLogicTemperature(PortMemory, mmTempCurrent, mmTempWarning, mmTempFail),
	  IUsesDioPort(PortMemory, mmDioData, mmDioConfig),
      IUsesPmcEeprom(PortMemory, mmI2c, rom_ParameterSize),
	  Owner(owner),
      Gps(new X3_Timing_Gps(this)),
      FFrontPanelRateMhz(20.0),
      FTriggerSampleClock(0),
      Pll(new X3_TimingClock(this, PortMemory)),
      InputMux(this, PortMemory),
      OutputMux(this, PortMemory),
      PxiMux(this, PortMemory),
      ClockParamRomSection(Rom(), PmcIdrom::waModule,
                           romX3_Timing_ClockParamOffset, romX3_Timing_ClockParamSize),
      BoardVariantRomSection(Rom(), PmcIdrom::waModule,
                           romX3_Timing_VariantParamOffset, romX3_Timing_VariantParamSize),
	  DeviceDriver(new X3_TimingDeviceDriver),
	  PciControlReg(PortMemory, mmControl),
	  PciStatusReg(PortMemory, mmStatus),
      SoftwareTriggerReg(PortMemory, mmSoftwareTrigger),
      TriggerReg(PortMemory, mmTriggerControl),
      ClockTestCtlReg(PortMemory, mmClockTest),
      IMuxTestCountReg(PortMemory, mmIMuxTestCount),
      PllTestCountReg(PortMemory, mmPllTestCount),
      OMuxTestCountReg(PortMemory, mmOMuxTestCount),
      PxiTestCountReg(PortMemory, mmPxiTestCount),
      InterruptStatusReg(PortMemory, mmInterruptStatus),
      InterruptEnableReg(PortMemory, mmInterruptEnable),
      InputMuxControlReg(PortMemory, mmInputMuxControl),
      OutputMuxControlReg(PortMemory, mmOutputMuxControl),
      PxiMuxControlReg(PortMemory, mmPxiMuxControl),
      OutputSyncControlReg(PortMemory, mmOutputSync),
      VcxoI2CReg(PortMemory, mmVcxoI2C),
      VcxoIntf(VcxoI2CReg),
      Vcxo_I2C(VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      Vcxo(this, Vcxo_I2C),
      ReferenceIntf(InputMuxControlReg),
      Reference_I2C(ReferenceIntf, I2CDevice::SubAddress, ReferenceI2C_SubAddr),
      Reference_Tcxo(Reference_I2C),
      Isr(*this)
{
    // Link together timing devices to match the hardware
    Pll->SetSourceTimer(&InputMux);   // Pll sources from IMUX

    OutputMux.SetPllSourceTimer(Pll);
    OutputMux.SetSourceTimer(&InputMux);

    PxiMux.SetPllSourceTimer(Pll);
    PxiMux.SetSourceTimer(&InputMux);
}

//---------------------------------------------------------------------------
//  destructor for class X3_TimingDeviceMap
//---------------------------------------------------------------------------

X3_TimingDeviceMap::~X3_TimingDeviceMap()
{
	delete Pll;
	Pll = 0;

	delete DeviceDriver;
	DeviceDriver = 0;

    delete Gps;
    Gps = 0;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::BoardCount()  --  Query # installed boards
//---------------------------------------------------------------------------

size_t X3_TimingDeviceMap::BoardCount()
{
    return DeviceDriver->BoardCount();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  X3_TimingDeviceMap::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Open()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Open()
{
	if(DeviceDriver->Opened())
		return;

    DeviceDriver->Open();
    //
	//  Link register definitions to device addresses
    PortMemory.Link(reinterpret_cast<ii32 *>(DeviceDriver->PortMemoryAddress()));

    InterruptEnable(false);
    Isr.Open();

    IUsesPmcEeprom::AfterOpen();
    //
    //  Initialize Subdevices
    Pll->Powered(true);
    InputMux.Powered(true);
    OutputMux.Powered(true);
    PxiMux.Powered(true);

    Reference_Tcxo.Init();
    Vcxo.Init();
    InputMux.Init();
    Pll->Init();
    OutputMux.Init();
    PxiMux.Init();
    Gps->Init();

    TriggerSampleClock(0);   //  Trigger Sample Clock = PLL

    OutputMux.EnableAllOutputs(false);
    PxiMux.EnableAllOutputs(false);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Close()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Close()
{
	if(!DeviceDriver->Opened())
		return;

    OutputMux.EnableAllOutputs(false);
    PxiMux.EnableAllOutputs(false);
    InterruptEnable(false);

    Isr.Close();
	DeviceDriver->Close();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Start()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Start()
{
    //  Hold all clock outputs OFF
    OutputSyncControlReg.SyncHold.Value(true);
    //
    //  Reapply Subdevices
    Reference_Tcxo.Reapply();
    Vcxo.Reapply();
    InputMux.Reapply();
    Pll->Powered(true);
    Pll->Reapply();
    OutputMux.Reapply();
    PxiMux.Reapply();
    Gps->Reapply();

    OutputMux.EnableAllOutputs(true);
    PxiMux.EnableAllOutputs(true);

    //  Release clock outputs ON
    OutputSyncControlReg.SyncHold.Value(false);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Stop()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Stop()
{
    //  Hold all clock outputs OFF
    OutputSyncControlReg.SyncHold.Value(true);

    OutputMux.EnableAllOutputs(false);
    PxiMux.EnableAllOutputs(false);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Reset()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Reset()
{
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3_TimingDeviceMap::Target() const
{
	return DeviceDriver->Target();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Target( int value )
{
    DeviceDriver->Target(value);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Led()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::Led(bool value)
{
    PciControlReg.Led = value;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Led()
//---------------------------------------------------------------------------

bool  X3_TimingDeviceMap::Led() const
{
	return PciControlReg.Led;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::PeekPort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

ii32 X3_TimingDeviceMap::PeekPort(int offset)
{
	ii32 * PortBasePtr = static_cast<ii32 *>(DeviceDriver->PortMemoryAddress());
    return PortBasePtr[offset];
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::PokePort() - read a 32 bit value from port-space
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::PokePort(int offset, ii32 value)
{
	ii32 * PortBasePtr = static_cast<ii32 *>(DeviceDriver->PortMemoryAddress());
	PortBasePtr[offset] = value;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::SoftwareTrigger() - Set SW trigger register
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::SoftwareTrigger(bool value)
{
    SoftwareTriggerReg.Value(value);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::SoftwareTrigger() - Query SW trigger register
//---------------------------------------------------------------------------

bool X3_TimingDeviceMap::SoftwareTrigger() const
{
	return SoftwareTriggerReg.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Framed() --
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::Framed(int size)
{
	TriggerReg.Count.Value(size);
	TriggerReg.FramedMode.Value(true);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::Unframed() --
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::Unframed()
{
	TriggerReg.FramedMode.Value(false);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::ExternalTrigger(bool state)
{
	TriggerReg.External.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool X3_TimingDeviceMap::ExternalTrigger() const
{
	return TriggerReg.External.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::ArmTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::ArmTrigger(bool state)
{
	TriggerReg.Arm.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::ArmTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool X3_TimingDeviceMap::ArmTrigger() const
{
	return TriggerReg.Arm.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::TriggerSampleClock() - Set trigger sample clock source
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::TriggerSampleClock(int source_idx)
{
    //  Set the sources of the Trigger Clocks to match
    FTriggerSampleClock = source_idx;
    OutputMux.Source(4, source_idx);
    PxiMux.Source(4, source_idx);
    //  Make sure Mux output frequency < 250MHz
    double InFrequency = PxiMux.SourceFrequency(source_idx);
    int divider = 1;
    while (InFrequency > 250e6)
        {
        divider *= 2;
        InFrequency /= 2.0;
        }

    //  Also set the dividers to match
    OutputMux.Divider(4, 2*divider);
    PxiMux.Divider(4, 1*divider);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::TriggerSampleClock() - Query trigger sample clock source
//---------------------------------------------------------------------------

int X3_TimingDeviceMap::TriggerSampleClock() const
{
    return FTriggerSampleClock;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::InterruptEvent() --
//---------------------------------------------------------------------------

Event * X3_TimingDeviceMap::InterruptEvent()
{
    return DeviceDriver->IsrEvent();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::InterruptDepth() --
//---------------------------------------------------------------------------

size_t X3_TimingDeviceMap::InterruptDepth()
{
    return DeviceDriver->InterruptDepth();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::InterruptAck() --
//---------------------------------------------------------------------------

unsigned int X3_TimingDeviceMap::InterruptAck()
{
    return DeviceDriver->InterruptAck();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::InterruptDispatch() --
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::InterruptDispatch(unsigned int /*value*/)
{
    Gps->DispatchGpsInterrupt();
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::InterruptEnable() --
//---------------------------------------------------------------------------

void X3_TimingDeviceMap::InterruptEnable(bool state)
{
    InterruptEnableReg.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::SetActiveVco() --
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::SetActiveVco(int idx)
{
    VcxoI2CReg.Vco0Enable.Value(false);
    VcxoI2CReg.Vco1Enable.Value(false);

    if (idx==1)
        {
        VcxoI2CReg.VcoSelect.Value(true);
        VcxoI2CReg.Vco0Enable.Value(false);
        VcxoI2CReg.Vco1Enable.Value(true);
        }
    else
        {
        VcxoI2CReg.VcoSelect.Value(false);
        VcxoI2CReg.Vco0Enable.Value(true);
        VcxoI2CReg.Vco1Enable.Value(false);
        }
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::BeginCalibrationTest() -- start internal self test
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::BeginCalibrationTest()
{
    ClockTestCtlReg.Value(0);   // any write triggers the Cal Test
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::CalibrationTestResults() -- get results of test
//---------------------------------------------------------------------------

bool  X3_TimingDeviceMap::CalibrationTestResults(CalTestResults & results)
{
    if (ClockTestCtlReg.Bit(0) == true)
        {
        // test complete, counters valid
        results.InMuxCount  =  IMuxTestCountReg.Value();
        results.PllCount    =  PllTestCountReg.Value();
        results.OutMuxCount =  OMuxTestCountReg.Value();
        results.PxiMuxCount =  PxiTestCountReg.Value();

        return true;
        }
    else
        {
        // test not complete -- zero counters out
        results.InMuxCount  =  0;
        results.PllCount    =  0;
        results.OutMuxCount =  0;
        results.PxiMuxCount =  0;

        return false;
        }
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::SetVariant()
//---------------------------------------------------------------------------

void  X3_TimingDeviceMap::SetVariant(unsigned int value)
{
    BoardVariantRomSection.AsInt(0, value);
    // set calibrated field
    BoardVariantRomSection.AsInt(3, static_cast<int>('Y'));
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::GetVariant()
//---------------------------------------------------------------------------

unsigned int  X3_TimingDeviceMap::GetVariant() const
{
    //  Check Calibrated bit
    if (IsVariantCalibrated())
        {
        return BoardVariantRomSection.AsInt(0);
        }
    else
        return x3tNormal;  // default value - assumed power on Variant
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::IsVariantCalibrated()
//---------------------------------------------------------------------------

bool  X3_TimingDeviceMap::IsVariantCalibrated() const
{
    if (BoardVariantRomSection.AsInt(3)=='Y')
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  X3_TimingDeviceMap::HasCmosOutputs()
//---------------------------------------------------------------------------

bool  X3_TimingDeviceMap::HasCmosOutputs() const
{
    if (DashNumber() == 4 || DashNumber() == 5)  // '-4' or '-5' means CMOS outputs
        return true;
    if (GetVariant() == x3tCmosOutput) // pseudo Variant code also works
        return true;
    return false;
}


} // namespace Innovative


