// X3_TimingDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X3_TimingDeviceMap_MbH
#define X3_TimingDeviceMap_MbH

#include <DeviceDriver_Mb.h>
#include <VirtexJtagLoader_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <Magic_Mb.h>
#include <DioPort_Mb.h>
#include <SimpleInterrupt_Mb.h>
#include <ctime>
#include <PmcEeprom_Mb.h>
#include <X3DeviceMap_Mb.h>
#include <X3IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Si570_Mb.h>
#include <Ds4026_Mb.h>
#include <CDCE18005_Mb.h>
#include <CDCE72010_Mb.h>
#include <Gps_CW25_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X3_Timing;
class SimpleInterrupt;

//===========================================================================
//  CLASS IX3_TimingGps  -- Extended Interface to X3 Timing's Gps
//===========================================================================

class IX3_TimingGps : public Gps_Cw25
{
public:
    IX3_TimingGps()
        {}
};

//===========================================================================
//  CLASS X3_TimingVcxo  -- Extended Interface to X3 Timing's Vcxo
//===========================================================================

class X3_TimingVcxo : public Si570
{
public:
    X3_TimingVcxo(class X3_TimingDeviceMap * owner, I2CDevice & device);

	virtual void        Init();
	virtual void        Reapply();
	virtual void        Calibrate(double power_on_rate_MHz);

private:
    X3_TimingDeviceMap * Owner;

    void SetVcxoCalibrationRate(double value);
	double GetVcxoCalibrationRate() const;
    bool   IsCalibrated() const;

};

//===========================================================================
//  CLASS IX3_TimingClock  -- Extended Interface to X3 Timing's Clock
//===========================================================================

class IX3_TimingClock : public Cdce72010
{
public:
    IX3_TimingClock(AddressingSpace &space, Cdce72010Registers & reg)
		: Cdce72010(space, reg)
        {}
    virtual void  SetActiveVco(int idx) = 0;
    virtual unsigned int PllOutputDivider() = 0;
    virtual void         PllOutputDivider(unsigned int divide_by) = 0;
    virtual void  PllOutputPhase(unsigned int divide_by, unsigned int phase) = 0;
    virtual void  PllOutputBoost(bool enabled) = 0;

    virtual unsigned int DebugDivider() = 0;
    virtual void         DebugDivider(unsigned int divide_by) = 0;
    virtual void  DebugPhase(unsigned int divide_by, unsigned int phase) = 0;
    virtual void  DebugBoost(bool enabled) = 0;

    virtual void  SetSourceTimer(IMultiTimebaseDevice * src) = 0;
    virtual void  SetPDF(double freq) = 0;  // DEBUG!!

protected:
    //  Don't support ADC/DAC Groups on this clock
    virtual unsigned int AdcMasterOutput()
        {  return FMasterOutput;  }
    virtual unsigned int DacMasterOutput()
        {  return FMasterOutput;  }

    virtual void  SetAdcOutputRates(double adc_rate)
        {  OutputFrequency(AdcMasterOutput(), adc_rate);  }
    virtual void  SetDacOutputRates(double /*dac_rate*/)
        {}

};

//===========================================================================
//  CLASS X3_TimingInputMux  -- Configurable Input Mux
//===========================================================================

class X3_TimingInputMux :  public Cdce18005
{
public:
    enum InMuxSelect { inReference=muxPrimaryIn, inFrontPanel=muxSecondaryIn, inGps=muxAuxIn };

    X3_TimingInputMux(class X3_TimingDeviceMap * owner, AddressingSpace & space);

    void  Source(unsigned int output, unsigned int source)
        {  SetMuxSelection(output, source);   }
    void  Divider(unsigned int output, unsigned int divide_by)
        {  OutputDivider(output, divide_by);  }
    void  DividerEnabled(unsigned int output, bool enabled)
        {  SetDividerEnable(output, enabled);  }
    void  Phase(unsigned int output, unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(output, divide_by, phase);  }
    void  Boost(unsigned int output, bool enabled)
        {  SetHighSwingLvpecl(output, enabled);  }

protected:
    virtual void    OnInit();
    virtual void    OnReapply();
    virtual void    OnSetPower(bool & new_state);

    virtual void    UpdateSources();

    X3_TimingDeviceMap * Owner;
};

//===========================================================================
//  CLASS X3_TimingOutputMux  -- Configurable Output Mux
//===========================================================================

class X3_TimingOutputMux :  public Cdce18005
{
public:
    enum OutMuxSelect { inPll=muxPrimaryIn, inFrontPanel=muxSecondaryIn, inGps=muxAuxIn };
    X3_TimingOutputMux(class X3_TimingDeviceMap * owner, AddressingSpace & space);

    void  Source(unsigned int output, unsigned int source)
        {  SetMuxSelection(output, source);  }
    unsigned int   Divider(unsigned int output)
        {  return OutputDivider(output);  }
    void  Divider(unsigned int output, unsigned int divide_by)
        {  OutputDivider(output, divide_by);  }
    void  DividerEnabled(unsigned int output, bool enabled)
        {  SetDividerEnable(output, enabled);  }
    void  Phase(unsigned int output, unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(output, divide_by, phase);  }
    void  Boost(unsigned int output, bool enabled)
        {  SetHighSwingLvpecl(output, enabled);  }

    void  EnableAllOutputs(bool enabled);

protected:
    virtual void    OnInit();
    virtual void    OnReapply();
    virtual void    OnSetPower(bool & new_state);

    X3_TimingDeviceMap * Owner;

public:
    virtual void  SetPllSourceTimer(IMultiTimebaseDevice * src)
        {  SrcPll = src;  }
    virtual void  SetSourceTimer(IMultiTimebaseDevice * src)
        {  Src = src;  }

protected:
    IMultiTimebaseDevice *  Src;        // Pll outputs
    IMultiTimebaseDevice *  SrcPll;     // IMux outputs

    virtual void  UpdateSources();
};

//===========================================================================
//  CLASS X3_TimingPxiMux  -- Configurable Pxi Mux
//===========================================================================

class X3_TimingPxiMux :  public Cdce18005
{
public:
    enum PxiMuxSelect { inPll=muxPrimaryIn, inFrontPanel=muxSecondaryIn, inGps=muxAuxIn };
    X3_TimingPxiMux(class X3_TimingDeviceMap * owner, AddressingSpace & space);

    void  Source(unsigned int output, unsigned int source)
        {  SetMuxSelection(output, source);  }
    unsigned int   Divider(unsigned int output)
        {  return OutputDivider(output);  }
    void  Divider(unsigned int output, unsigned int divide_by)
        {  OutputDivider(output, divide_by);  }
    void  DividerEnabled(unsigned int output, bool enabled)
        {  SetDividerEnable(output, enabled);  }
    void  Phase(unsigned int output, unsigned int divide_by, unsigned int phase)
        {  SetPhaseAdjust(output, divide_by, phase);  }
    void  Boost(unsigned int output, bool enabled)
        {  SetHighSwingLvpecl(output, enabled);  }

    void  TriggerOutputEnable(bool state);

    void  EnableAllOutputs(bool enabled);

    void  DriveDSTAROutputs(bool enabled) { DriveDSTAROut=enabled; }
    void  Drive10MHzOutput(bool enabled) { Drive10MHzOut=enabled; }

protected:
    virtual void    OnInit();
    virtual void    OnReapply();
    virtual void    OnSetPower(bool & new_state);

    X3_TimingDeviceMap * Owner;

public:
    virtual void  SetPllSourceTimer(IMultiTimebaseDevice * src)
        {  SrcPll = src;  }
    virtual void  SetSourceTimer(IMultiTimebaseDevice * src)
        {  Src = src;  }

protected:
    IMultiTimebaseDevice *  Src;        // Pll outputs
    IMultiTimebaseDevice *  SrcPll;     // IMux outputs
    bool                    DriveDSTAROut;
    bool                    Drive10MHzOut;

    virtual void  UpdateSources();
};

//===========================================================================
//  CLASS X3_TimingDeviceMap  -- Register & low level support for X3_Timing
//===========================================================================

class X3_TimingDeviceMap :
    public IUsesVirtexJtagLoader,
    public IUsesPciLogicVersionInfo,
    public IUsesLogicTemperature,
    public IUsesDioPort,
    public IUsesPmcEeprom,
    public ISimpleInterrupt,
    public IUsesNullFrontPanelPort
{
	friend class X3_Timing;
    friend class X3_TimingClock;
    friend class X3_Timing_Gps;
    friend class X3_TimingVcxo;
    friend class X3_TimingInputMux;
    friend class X3_TimingOutputMux;
    friend class X3_TimingPxiMux;

public:
	//
	//  Member Functions
	X3_TimingDeviceMap(X3_Timing * owner);
	virtual ~X3_TimingDeviceMap();

    // Methods
    virtual size_t    InterruptDepth();
    size_t          BoardCount();
    bool            LicenseCheck(unsigned int & code);

	void            Open();
	void            Close();
	void            Reset();
    void            Start();
    void            Stop();

	ii32            PeekPort(int offset);
	void            PokePort(int offset, ii32 value);

	// Properties
	void            Target(int target);
	int             Target() const;

	void            Led(bool value);
	bool            Led() const;

	void            SoftwareTrigger(bool value);
	bool            SoftwareTrigger() const;

	void            Framed(int size);
	void            Unframed();

	void            ExternalTrigger(bool state);
	bool            ExternalTrigger() const;

	void            ArmTrigger(bool state);
	bool            ArmTrigger() const;

	void            FrontPanelRateMhz(double rate)
                        {  FFrontPanelRateMhz = rate;  }
	double          FrontPanelRateMhz() const
                        {  return FFrontPanelRateMhz;  }

	void            TriggerSampleClock(int source_idx);
	int             TriggerSampleClock() const ;

    //
    //  Variant Code "Calibration" Feature
    bool          HasCmosOutputs() const;

    void          SetVariant(unsigned int value);
    unsigned int  GetVariant() const;
    bool          IsVariantCalibrated() const;
    
protected:
	X3_Timing *         Owner;
    IX3_TimingGps *     Gps;
    double              FFrontPanelRateMhz;
    int                 FTriggerSampleClock;
    //
    //  Methods
    void  SetActiveVco(int idx);
	//
	//  Class TriggerControlRegister
	class TriggerControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBitGroup    Count;
		RegisterBit         Arm;
		RegisterBit         FramedMode;
		RegisterBit         External;

	public:
		TriggerControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Count(*this, 0, 24), Arm(*this, 28),
              FramedMode(*this, 30), External(*this, 31)
			{
			//  Default setup ????
			}
	};
	//
	//  Class PciControlRegister
	class PciControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit       Led;

	public:
		PciControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Led(*this, 1)
			{
			//  Default setup ????
			}
	};

	//
	//  Class PciStatusRegister
	class PciStatusRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBit       Slot64;

	public:
		PciStatusRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Slot64(*this, 0)
			{
			//  Default setup ????
			}
	};

	//
	//  Class ClockTestCounterRegister
	class ClockTestCounterRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBitGroup  Counter;

	public:
		ClockTestCounterRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               Counter(*this, 0, 23)
			{
			}
	};
	//
	//  Class VcxoI2CRegister
	class VcxoI2CRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit  Data_Out;
		RegisterBit  Clock_Out;
		RegisterBit  Data_In;
		RegisterBit  Clock_In;
		RegisterBit  VcoSelect;
		RegisterBit  Vco0Enable;
		RegisterBit  Vco1Enable;

	public:
		VcxoI2CRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               Data_Out(*this, 0),
               Clock_Out(*this, 1),
               Data_In(*this, 2),
               Clock_In(*this, 3),
               VcoSelect(*this, 6),
               Vco0Enable(*this, 7),
               Vco1Enable(*this, 8)
			{
			}
	};
	//
	//  Class InputMuxControlRegister
	class InputMuxControlRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit  PowerOn;
		RegisterBit  RefI2C_Clock_Out;
		RegisterBit  RefI2C_Data_Out;
		RegisterBit  RefI2C_Clock_In;
		RegisterBit  RefI2C_Data_In;

	public:
		InputMuxControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               PowerOn(*this, 0),
               RefI2C_Clock_Out(*this, 7),
               RefI2C_Data_Out(*this, 8),
               RefI2C_Clock_In(*this, 9),
               RefI2C_Data_In(*this, 10)
			{
			}
	};
	//
	//  Class OutputMuxControlRegister
	class OutputMuxControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit  PowerOn;

	public:
		OutputMuxControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               PowerOn(*this, 0)
 			{
			}
	};
	//
	//  Class PxiMuxControlRegister
	class PxiMuxControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit  PowerOn;
		RegisterBit  OutputEnable;

	public:
		PxiMuxControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               PowerOn(*this, 0),
               OutputEnable(*this, 1)
 			{
			}
	};
	//
	//  Class OutputSyncControlRegister
	class OutputSyncControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit  SyncHold;
		RegisterBit  ExternalSyncEnable;

	public:
		OutputSyncControlRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               SyncHold(*this, 0),
               ExternalSyncEnable(*this, 1)
 			{
			}
	};
	//
	//  ...Registers
	IX3_TimingClock *  					Pll;
    X3_TimingInputMux                   InputMux;
    X3_TimingOutputMux                  OutputMux;
    X3_TimingPxiMux                     PxiMux;
	PmcIdromSection                     ClockParamRomSection;
	PmcIdromSection                     BoardVariantRomSection;
	AddressingSpace                    	PortMemory;
	InnovativeKernel::IsrDeviceDriver * DeviceDriver;
	PciControlRegister                 	PciControlReg;
	PciStatusRegister                  	PciStatusReg;
//    PllReferenceControlRegister         ReferenceReg;
    ShadowRegister                      SoftwareTriggerReg;
    TriggerControlRegister              TriggerReg;

    Register    	                    ClockTestCtlReg;
    ClockTestCounterRegister    	    IMuxTestCountReg;
    ClockTestCounterRegister   	        PllTestCountReg;
    ClockTestCounterRegister   	        OMuxTestCountReg;
    ClockTestCounterRegister   	        PxiTestCountReg;

    Register                            InterruptStatusReg;
    ShadowRegister                      InterruptEnableReg;
    InputMuxControlRegister             InputMuxControlReg;
    OutputMuxControlRegister            OutputMuxControlReg;
    PxiMuxControlRegister               PxiMuxControlReg;

    OutputSyncControlRegister           OutputSyncControlReg;

    VcxoI2CRegister                     VcxoI2CReg;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        VcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~VcxoI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        VcxoI2CRegister &  Reg;
    };

    VcxoI2C_Impl        VcxoIntf;
    I2CDeviceN          Vcxo_I2C;
    X3_TimingVcxo       Vcxo;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS RI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ReferenceI2C_Impl : public I2CDeviceIntf
    {
    public:
        ReferenceI2C_Impl(InputMuxControlRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~ReferenceI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.RefI2C_Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.RefI2C_Clock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.RefI2C_Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.RefI2C_Data_In.Value();  }

    private:
        InputMuxControlRegister &  Reg;
    };

    ReferenceI2C_Impl        ReferenceIntf;
    I2CDeviceN               Reference_I2C;
    Ds4026                   Reference_Tcxo;

private:
    // Interface
    virtual Event *         InterruptEvent();
    virtual unsigned int    InterruptAck();
    virtual void            InterruptEnable(bool state);
    virtual void            InterruptDispatch(unsigned int value);

	// Data
    SimpleInterrupt         Isr;

public:
    //
    //  Clock Test Functions
    struct CalTestResults
    {
        unsigned int InMuxCount;
        unsigned int PllCount;
        unsigned int OutMuxCount;
        unsigned int PxiMuxCount;
    };

    void  BeginCalibrationTest();
    bool  CalibrationTestResults(CalTestResults & results);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
