// X3_Timing_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X3_Timing_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <SystemSupport_Mb.h>

using namespace std;


namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X3_Timing  -- Low-jitter timebase XMC module object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X3_Timing
//---------------------------------------------------------------------------

X3_Timing::X3_Timing()
    : Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X3_Timing
//---------------------------------------------------------------------------

X3_Timing::~X3_Timing()
{
    Close();
}

//---------------------------------------------------------------------------
//  X3_Timing::BoardCount() -- Query # installed boards
//---------------------------------------------------------------------------

size_t X3_Timing::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X3_Timing::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X3_Timing::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X3_Timing::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X3_Timing::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X3_Timing::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  X3_Timing::LicenseCheck(unsigned int & /*code*/)
{
	return true;
}

//---------------------------------------------------------------------------
//  X3_Timing::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X3_Timing::Open()
{
    Device.Open();
}

//---------------------------------------------------------------------------
//  X3_Timing::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X3_Timing::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X3_Timing::Start() -- Reapply timer settings, start timers
//---------------------------------------------------------------------------

void X3_Timing::Start()
{
    Device.Start();
}

//---------------------------------------------------------------------------
//  X3_Timing::Stop() -- Stop timers
//---------------------------------------------------------------------------

void X3_Timing::Stop()
{
    Device.Stop();
}

//---------------------------------------------------------------------------
//  X3_Timing::Reset() -- Cold-start initialization
//---------------------------------------------------------------------------

void X3_Timing::Reset()
{
    return Device.Reset();
}

//---------------------------------------------------------------------------
//  X3_Timing::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X3_Timing::Target() const
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X3_Timing::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X3_Timing::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X3_Timing::Name() -- Board name string
//---------------------------------------------------------------------------

std::string  X3_Timing::Name() const
{
    return std::string("X3_Timing");
}

//---------------------------------------------------------------------------
//  X3_Timing::Variant() -- Board variant code
//---------------------------------------------------------------------------

X3Timing_Variants  X3_Timing::Variant() const
{
    if (Device.IsVariantCalibrated() == false)
        return x3tUnknown;
    else
        return static_cast<X3Timing_Variants>(Device.GetVariant());
}

//---------------------------------------------------------------------------
//  X3_Timing::HasCmosOutputs() -- Have CMOS Outputs?
//---------------------------------------------------------------------------

bool  X3_Timing::HasCmosOutputs() const
{
    return Device.HasCmosOutputs();
}

//---------------------------------------------------------------------------
//  X3_Timing::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X3_Timing::Led() const
{
	return Device.Led();
}

//---------------------------------------------------------------------------
//  X3_Timing::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X3_Timing::Led( bool value )
{
	Device.Led(value);
}

//---------------------------------------------------------------------------
//  X3_Timing::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X3_Timing::PeekPort(int offset )
{
	return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X3_Timing::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X3_Timing::PokePort( int offset, ii32 value )
{
	Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X3_Timing::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X3_Timing::PciLogicVersion()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_Timing::Eeprom() -- Return reference to EEProm object
//---------------------------------------------------------------------------

IUsesVirtexJtagLoader & X3_Timing::Eeprom()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_Timing::Pll() --  Return reference to Onboard PLL
//---------------------------------------------------------------------------

IX3_TimingClock & X3_Timing::Pll()
{
	return *Device.Pll;
}

//---------------------------------------------------------------------------
//  X3_Timing::InMux() --  Return reference to Input Mux
//---------------------------------------------------------------------------

X3_TimingInputMux & X3_Timing::InMux()
{
	return Device.InputMux;
}

//---------------------------------------------------------------------------
//  X3_Timing::OutMux() --  Return reference to Input Mux
//---------------------------------------------------------------------------

X3_TimingOutputMux & X3_Timing::OutMux()
{
	return Device.OutputMux;
}

//---------------------------------------------------------------------------
//  X3_Timing::PxiMux() --  Return reference to Input Mux
//---------------------------------------------------------------------------

X3_TimingPxiMux & X3_Timing::PxiMux()
{
	return Device.PxiMux;
}

//---------------------------------------------------------------------------
//  X3_Timing::Vcxo() --  Return reference to Vcxo
//---------------------------------------------------------------------------

Si570 & X3_Timing::Vcxo()
{
	return Device.Vcxo;
}

//---------------------------------------------------------------------------
//  X3_Timing::Ref() --  Return reference to Input Mux
//---------------------------------------------------------------------------

Ds4026 & X3_Timing::Ref()
{
	return Device.Reference_Tcxo;
}

//---------------------------------------------------------------------------
//  X3_Timing::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X3_Timing::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_Timing::Dio() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X3_Timing::Dio()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X3_Timing::Gps() -- GPS state
//---------------------------------------------------------------------------

IX3_TimingGps & X3_Timing::Gps()
{
	return *Device.Gps;
}

//---------------------------------------------------------------------------
//  X3_Timing::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X3_Timing::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X3_Timing::SoftwareTrigger() - Set SW trigger register
//---------------------------------------------------------------------------

void X3_Timing::SoftwareTrigger(bool value)
{
    Device.SoftwareTrigger(value);
}

//---------------------------------------------------------------------------
//  X3_Timing::SoftwareTrigger() - Query SW trigger register
//---------------------------------------------------------------------------

bool X3_Timing::SoftwareTrigger() const
{
    return Device.SoftwareTrigger();
}

//---------------------------------------------------------------------------
//  X3_Timing::ExternalTrigger() - Set external trigger state
//---------------------------------------------------------------------------

void X3_Timing::ExternalTrigger(bool value)
{
    Device.ExternalTrigger(value);
}

//---------------------------------------------------------------------------
//  X3_Timing::ExternalTrigger() - Query external trigger state
//---------------------------------------------------------------------------

bool X3_Timing::ExternalTrigger() const
{
    return Device.ExternalTrigger();
}

//---------------------------------------------------------------------------
//  X3_Timing::ArmTrigger() - Set trigger arm state
//---------------------------------------------------------------------------

void X3_Timing::ArmTrigger(bool value)
{
    Device.ArmTrigger(value);
}

//---------------------------------------------------------------------------
//  X3_Timing::ArmTrigger() - Query trigger arm state
//---------------------------------------------------------------------------

bool X3_Timing::ArmTrigger() const
{
    return Device.ArmTrigger();
}

//---------------------------------------------------------------------------
//  X3_Timing::TriggerSampleClock() - Set trigger sample clock source
//---------------------------------------------------------------------------

void X3_Timing::TriggerSampleClock(int source_idx)
{
    Device.TriggerSampleClock(source_idx);
}

//---------------------------------------------------------------------------
//  X3_Timing::TriggerSampleClock() - Query trigger sample clock source
//---------------------------------------------------------------------------

int X3_Timing::TriggerSampleClock() const
{
    return Device.TriggerSampleClock();
}

//---------------------------------------------------------------------------
//  X3_Timing::Framed() --
//---------------------------------------------------------------------------

void X3_Timing::Framed(int size)
{
	Device.Framed(size);
}

//---------------------------------------------------------------------------
//  X3_Timing::Unframed() --
//---------------------------------------------------------------------------

void X3_Timing::Unframed()
{
	Device.Unframed();
}

//---------------------------------------------------------------------------
//  X3_Timing::FrontPanelRateMhz() --
//---------------------------------------------------------------------------

void X3_Timing::FrontPanelRateMhz(double rate)
{
	Device.FrontPanelRateMhz(rate);
}

//---------------------------------------------------------------------------
//  X3_Timing::FrontPanelRateMhz() --
//---------------------------------------------------------------------------

double X3_Timing::FrontPanelRateMhz() const
{
	return Device.FrontPanelRateMhz();
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

}  // namespace Innovative
