// X3_Timing_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2009 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef X3_Timing_MbH
#define X3_Timing_MbH

#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <X3_TimingDeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

class X3_TimingDeviceMap;

//-------------------------------------------------
//  X3Timing Variant Codes
//    Variant codes change configuration based on board type.
//       These must be burned into IDROM by Innovative
//
//  x3tNormal      = normal configuration
//  x3tCmosOutput  = Output Mux uses CMOS outputs
//  x3tUnknown     = Variant not programmed, treated as x3tNormal
//
enum X3Timing_Variants { x3tNormal, x3tCmosOutput = 0x100, x3tUnknown = 0x666 };

//===========================================================================
//  CLASS X3_Timing  -- Low-jitter timebase XMC module object
//===========================================================================

class X3_Timing
{
    friend class X3_TimingDeviceMap;

    enum TriggerSampleClockSources { tscPll, tscFrontPanel, tscGps };

public:
	//
	//  Member Functions
	X3_Timing();
	~X3_Timing();

    size_t          BoardCount();
    vector<string>  BoardNames();
    string          PrintAllDevices();
	bool            LicenseCheck(unsigned int & code);
	
	void            Open();
    void            Close();
	void            Reset();
    void            Start();
    void            Stop();

	ii32            PeekPort(int offset);
	void            PokePort(int offset, ii32 value);

	// Properties
	void  			Led(bool value);
	bool  			Led() const;

	void            Target(int target);
	int             Target() const;

    std::string     Name() const;

    bool               HasCmosOutputs() const;
    X3Timing_Variants  Variant() const;

	// Trigger Support
	void            ExternalTrigger(bool value);
	bool            ExternalTrigger() const;

	void            ArmTrigger(bool value);
	bool            ArmTrigger() const;

	void            SoftwareTrigger(bool value);
	bool            SoftwareTrigger() const;

	void            Framed(int size);
	void            Unframed();

	void            FrontPanelRateMhz(double rate);
	double          FrontPanelRateMhz() const ;

    void            TriggerSampleClock(TriggerSampleClockSources src)
                        {  TriggerSampleClock(static_cast<int>(src));  }
	void            TriggerSampleClock(int source_idx);
	int             TriggerSampleClock() const ;

	X3_TimingDeviceMap * Debug() { return &Device; };

	// EEprom programming
	IUsesVirtexJtagLoader & Eeprom();


	// Methods
    //
    //  Sub-Device Interfaces
    //   Vcxo_1, Reference_1
	IX3_TimingClock &           Pll();
    X3_TimingInputMux &         InMux();
    X3_TimingOutputMux &        OutMux();
    X3_TimingPxiMux &           PxiMux();
    Si570 &                     Vcxo();
    Ds4026 &                    Ref();

	IUsesPmcEeprom &            IdRom();
	IX3_TimingGps &             Gps();
	IUsesDioPort & 	            Dio();

    PciLogicVersionInfoIntf &     PciLogicVersion();
    LogicTemperatureIntf &  Thermal();


protected:
    //
    //  Member Data
    X3_TimingDeviceMap          Device;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

