// X5DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5DeviceMap_Mb.h>
#include <X5MemoryMap_Mb.h>
#include <X5DeviceDriver_Mb.h>
#include <PmcIdrom_Mb.h>
#include <SystemSupport_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5FpaControlRegister
//===========================================================================

class X5FpaControlRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup            Mode;
    RegisterBit                 Clk;
    RegisterBit                 Out;
    ReadOnlyRegisterBit         In;
    ReadOnlyRegisterBitGroup    FirmwareRevision;
    ReadOnlyRegisterBitGroup    PcbRevision;
    ReadOnlyRegisterBitGroup    Id;

public:
    X5FpaControlRegister(AddressingSpace &space,
        int offset)
    :  inherited(space, offset),
       Mode(*this, 0, 3), Clk(*this, 3), Out(*this, 4), In(*this, 5),
       FirmwareRevision(*this, 8, 8), PcbRevision(*this, 16, 4),
       Id(*this, 20, 12)
       {  }
};

//==============================================================================
//  CLASS X5FpaLoader -- Interface to FPA register
//==============================================================================

class X5FpaLoader : public FpaLoadBase
{
public:
    X5FpaLoader(AddressingSpace & space)
        : Reg(space, X5_MemoryMap::mmFpa)
        {  }

    // Methods
    // Read/Write data
    virtual bool            Read()
        {  return Reg.In ? 1 : 0;  }
    virtual void            Write(bool state)
        {  Reg.Out = state;  }
    // Read/write clock state
    virtual void            Clock(bool state)
        {  Reg.Clk = state;  }

    // Mode set
    virtual void            Mode(int mode)
        {  Reg.Mode = mode;  }
    // Device select
    virtual void            Device(int /*device*/)
        {  }
    // Firmware revision readback
    virtual int             FirmwareRevision()
        {  return Reg.FirmwareRevision;  }
    // Pcb revision readback
    virtual int             PcbRevision()
        {  return Reg.PcbRevision;  }
    // Id readback
    virtual int             Id()
        {  return Reg.Id;  }

private:
    // Data
    X5FpaControlRegister      Reg;
};

//===========================================================================
//  CLASS X5HdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class X5HdwPluginImpl : public IPacketDevicePlugin
{
public:
	X5HdwPluginImpl( X5DeviceMap * pdm )
		:  PDM(pdm)
		{}
	virtual void  AfterDeviceOpen()
		{  PDM->OpenHardware();  }
	virtual void  BeforeDeviceClose()
		{ /* nothing special */  }

	X5DeviceMap * PDM;
};

//===========================================================================
//  CLASS IUsesX5LogicTemperature -- User Interface to user logic ID
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class IUsesX5LogicTemperature
//---------------------------------------------------------------------------

IUsesX5LogicTemperature::IUsesX5LogicTemperature(AddressingSpace & space,
	unsigned int temp_addr, unsigned int warn_addr, unsigned int fail_addr,
    unsigned int fan_addr)
	:  TempReg(space, temp_addr),
       WarningReg(space, warn_addr),
	   FailureReg(space, fail_addr),
	   FanReg(space, fan_addr),
	   FDefaultFailTemp(85)
{
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicTemperature() --  Query current temp
//---------------------------------------------------------------------------

float IUsesX5LogicTemperature::LogicTemperature() const
{
	return ToDegrees(TempReg.Degrees.Value());
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicWarningTemperature() -- Set warning temp
//---------------------------------------------------------------------------

void IUsesX5LogicTemperature::LogicWarningTemperature(float value)
{
	WarningReg.Degrees.Value(ToCounts(value));
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicWarningTemperature() -- Query warning temp
//---------------------------------------------------------------------------

float IUsesX5LogicTemperature::LogicWarningTemperature() const
{
	return ToDegrees(WarningReg.Degrees.Value());
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicFailureTemperature() --  Set failure temp
//---------------------------------------------------------------------------

void IUsesX5LogicTemperature::LogicFailureTemperature(float value)
{
	FailureReg.Degrees.Value(ToCounts(value));
	FDefaultFailTemp = value;
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicFailureTemperature() -- Query failure temp
//---------------------------------------------------------------------------

float IUsesX5LogicTemperature::LogicFailureTemperature() const
{
	return ToDegrees(FailureReg.Degrees.Value());
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicFanTemperature() --  Set fan temp
//---------------------------------------------------------------------------

void IUsesX5LogicTemperature::LogicFanTemperature(float value)
{
	FanReg.Degrees.Value(ToCounts(value));
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::LogicFanTemperature() -- Query fan temp
//---------------------------------------------------------------------------

float IUsesX5LogicTemperature::LogicFanTemperature() const
{
	return ToDegrees(FanReg.Degrees.Value());
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::Failed() --
//---------------------------------------------------------------------------

bool IUsesX5LogicTemperature::Failed() const
{
    return false;
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::ToCounts() --  Convert from degrees C to counts
//---------------------------------------------------------------------------

unsigned int IUsesX5LogicTemperature::ToCounts(float value) const
{
    return static_cast<unsigned int>((value + 273.15) * 1024.0/503.975);
}

//---------------------------------------------------------------------------
//  IUsesX5LogicTemperature::ToDegrees() --  Convert counts to degrees C
//---------------------------------------------------------------------------

float IUsesX5LogicTemperature::ToDegrees(unsigned int value) const
{
    return static_cast<float>((value * 503.975/1024.0) - 273.15);
}


//===========================================================================
//  CLASS X5DeviceMap  -- Registers for X5 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X5DeviceMap
//---------------------------------------------------------------------------
using namespace X5_MemoryMap;


PacketDeviceRegisters X5DRegAddrs =
	{
	mmControl,            // ControlRegister_Addr;
	mmStatus,             // StatusRegister_Addr;
	mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
	mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	0,                    // LinkControlRegister_Addr;
	};

PacketDeviceMapRegisters X5RegAddrs =
	{
	mmInterruptEnable,    // InterruptEnable_Addr;
	mmCreditQueue,        // CreditQueue_Addr;
	mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
	mmBmRingSize,         // BmRingSizeArray_Addr;
	};

//---------------------------------------------------------------------------
//  constructor for class X5DeviceMap
//---------------------------------------------------------------------------

X5DeviceMap::X5DeviceMap()
	:  PacketDeviceImpl			(new X5DeviceDriver, X5DRegAddrs),
	   PacketDeviceMap			(this, X5RegAddrs, 0),
	   IUsesFpgaLogicVersionInfo(LogicMemory, mmFpgaVersion),
	   IUsesPciLogicVersionInfo	(PortMemory, mmPciVersion),
	   IUsesX5LogicTemperature	(LogicMemory, mmTempCurrent, mmTempWarning, mmTempFail, mmTempFan),
	   IUsesDioPort				(LogicMemory, mmDioData, mmDioConfig),
	   IUsesPmcEeprom           (PortMemory, mmI2c, 128),
	   RcvPacketSize            (0x80),
	   XmitPacketSize           (0x80),
	   FLoopbackMode            (lmNone),
	   DDeviceDriver            (dynamic_cast<X5DeviceDriver*>(DeviceDriver)),
	   Alert                    (this),
	   PciTestControlReg    	(PortMemory, mmPciTestControl),
	   UserFpgaControlReg   	(LogicMemory, mmUserControl),
	   UserFpgaStatusReg   	    (LogicMemory, mmUserControl),
	   TestReg					(LogicMemory, mmTest),
	   Drp0						(PortMemory, mmDrp0),
	   Drp1						(PortMemory, mmDrp1),
	   FFpa                     (new X5FpaLoader(PortMemory))
{
	EepromLoader = new FpaLoadThread(new X5FpaPersonality(*FFpa));
	//
	//  Install Open Loader for Hardware
	InstallHdwPlugin( new X5HdwPluginImpl(this) );
}

//---------------------------------------------------------------------------
//  destructor for class X5DeviceMap
//---------------------------------------------------------------------------

X5DeviceMap::~X5DeviceMap()
{
	EepromLoader->Shutdown();
	delete EepromLoader;
	EepromLoader = 0;

	delete FFpa;
	FFpa = 0;
}

//---------------------------------------------------------------------------
//  X5DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X5DeviceMap::OpenHardware()
{
	IUsesPmcEeprom::AfterOpen();
	//
	//  Alert Interface
	Alert.Open();
	//
	//  Eeprom Interface
	EepromLoader->OnOpen();

	// Cal temp readback
	CalibrateSystemMonitor();
}

//---------------------------------------------------------------------------
//  X5DeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  X5DeviceMap::ConfigureHardware()
{
	//
	//  Reset User Logic
	UserFpgaControlReg.InGo.Clear();
	UserFpgaControlReg.OutGo.Clear();
    UserFpgaControlReg.InTrigger.Value(false);
    UserFpgaControlReg.OutTrigger.Value(false);
	UserFpgaControlReg.FpgaReset.Set();
	UserFpgaControlReg.Apply();

	Sleep(1);
	UserFpgaControlReg.FpgaReset.Clear();
	UserFpgaControlReg.Apply();
	Sleep(1);

	//
    //  Configure Alerts
    Alert.Configure();

	//
	//  Apply Control Register Stuff
	PciTestControlReg.Apply();

    // Config temp monitor level
	LogicWarningTemperature(80.0);

// this was removed to allow fail temp to be set and not re-set at stream start
//	LogicFailureTemperature(85.0);   

	LogicFailureTemperature(FDefaultFailTemp);
    LogicFanTemperature(80.0);
	//
	//  Set loopback mode
	PciLoopbackMode(PciLoopbackMode());
}

//---------------------------------------------------------------------------
//  X5DeviceMap::AlertTest() -- Alert Test Processing Hook
//---------------------------------------------------------------------------

bool  X5DeviceMap::AlertTest(unsigned int pid)
{
	//  return TRUE if PID is an alert PID.
    //  return FALSE to if not
    return Alert.AlertTest(pid);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::AlertCheck() -- Alert Event Processing Hook
//---------------------------------------------------------------------------

bool X5DeviceMap::AlertCheck(const Buffer & buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alert.CheckAndProcess(buffer);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  X5DeviceMap::DumpIsrStatus(std::vector<int> & vec)
{
    vec.clear();
    for (int i=0; i<X5DeviceDriver::LOG->Count; i++)
        vec.push_back(X5DeviceDriver::LOG->Status[i]);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::LocalLoopbackMode()
//---------------------------------------------------------------------------

void  X5DeviceMap::LocalLoopbackMode(bool state)
{
    if (state)
        {
		PciTestControlReg.LoopbackEnable.Set();
		PciTestControlReg.GenerateRxData.Clear();
		PciTestControlReg.DrainXmitData.Clear();
		PciTestControlReg.TestModeEnable.Set();
		}
	else
		{
		PciTestControlReg.LoopbackEnable.Clear();
		PciTestControlReg.GenerateRxData.Clear();
		PciTestControlReg.DrainXmitData.Clear();
		PciTestControlReg.TestModeEnable.Clear();
		}
}

//---------------------------------------------------------------------------
//  X5DeviceMap::CalibrateSystemMonitor() --  Cal temp measurement via SM
//
//  Enables System Monitor calibration by poking the registers via the
//  DRP interface as mapped to PCIe registers
//---------------------------------------------------------------------------

void  X5DeviceMap::CalibrateSystemMonitor()
{
	// Recalibration needed only for older PCIe cores...
	if (PciLogicRevision() > 74)
		return;

	Drp1.Value(0x20a1);
	Sleep(1);
	Drp0.Value(0x141);
	Sleep(1);
	Drp1.Value(0x0101);
	Sleep(1);
	Drp0.Value(0x148);
	Sleep(1);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::Trigger() --  Set trigger state
//---------------------------------------------------------------------------

void X5DeviceMap::Trigger(IIDirection dir, bool state)
{
	switch (dir)
		{
		case tInput:
			UserFpgaControlReg.InTrigger.Value(state);
			break;
		case tOutput:
			UserFpgaControlReg.OutTrigger.Value(state);
			break;
		case tInputOutput:
			UserFpgaControlReg.InTrigger.Value(state);
			UserFpgaControlReg.OutTrigger.Value(state);
			break;
		}

	UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X5DeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  X5DeviceMap::Name() const
{
    static const char * ModuleName[] =
    	{ "X5-400M", "X5-210M", "X5-GSPS", "X5-COM", "X5-RF" };

    return ModuleName[PciLogicType()];
}

//---------------------------------------------------------------------------
//  X5DeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  X5DeviceMap::Revision() const
{
	std::stringstream msg;
    char c = 'A' + static_cast<char>(PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  X5DeviceMap::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5DeviceMap::UserLoopbackMode(bool state)
{
	UserFpgaControlReg.Loopback.Value(state);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5DeviceMap::UserLoopbackMode() const
{
	return UserFpgaControlReg.Loopback.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5DeviceMap::ClockTestEnable() -- Set sync pin cal mode status
//---------------------------------------------------------------------------

void X5DeviceMap::ClockTestEnable(bool /*state*/)
{
	//TestReg.Bit(0, state);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::ClockTestEnable() -- Query sync pin cal mode status
//---------------------------------------------------------------------------

bool X5DeviceMap::ClockTestEnable() const
{
	return TestReg.Bit(0) ? true : false;
}

//---------------------------------------------------------------------------
//  X5DeviceMap::TestCounterEnable() -- Set test mode status (A/Ds generate ramp on true).
//---------------------------------------------------------------------------

void X5DeviceMap::TestCounterEnable(bool state)
{
    UserFpgaControlReg.TestCounter.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X5DeviceMap::TestCounterEnable() -- Query test mode status
//---------------------------------------------------------------------------

bool X5DeviceMap::TestCounterEnable() const
{
    return UserFpgaControlReg.TestCounter.Value() ? true : false;
}

//===========================================================================
//  CLASS X5DeviceMap::PciTestRegister
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X5::X5DeviceMap::PciTestRegister
//---------------------------------------------------------------------------

X5DeviceMap::PciTestRegister::PciTestRegister(AddressingSpace &space, int offset )
	: inherited( space, offset ),
	  TestModeEnable( *this, 0 ),
	  GenerateRxData( *this, 1 ),
	  LoopbackEnable( *this, 2 ),
	  ClearTxCredit ( *this, 3 ),
	  ClearRxCredit ( *this, 4 ),
	  DrainXmitData ( *this, 5 ),
	  GenPacketSize ( *this, 8, 4 ),
	  GenPacketSizeUpdate( *this, 12 )
{
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


} // namespace Innovative


