// X5DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5DeviceMap_MbH
#define X5DeviceMap_MbH

#include <PacketDeviceMap_Mb.h>
#include <vector>
#include <ProcessEvents_Mb.h>
#include <FpgaLogicVersionInfo_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <HardwareInfoIntf_Mb.h>
#include <LogicTemperature_Mb.h>
#include <DioPort_Mb.h>
#include <X5AlertSystem_Mb.h>
#include <PmcEeprom_Mb.h>
#include <FpaLoadThread_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X5DeviceDriver;

//===========================================================================
//  CLASS IX5RawClockDevices -- User Interface to clock device
//===========================================================================
class Si570;
class X5_Tx_Ad9511;

class IX5RawClockDevices
{
public:
	virtual ~IX5RawClockDevices()  {  }

    virtual Si570 &         Vcxo() = 0;
    virtual X5_Tx_Ad9511 &   Pll() = 0;
};


//===========================================================================
//  CLASS IUsesX5LogicTemperature -- User Interface to user logic ID
//===========================================================================

class IUsesX5LogicTemperature : public ExtendedLogicTemperatureIntf
{
public:
    IUsesX5LogicTemperature(AddressingSpace & space, unsigned int temp_addr,
        unsigned int warn_addr, unsigned int fail_addr, unsigned int fan_addr);
    virtual ~IUsesX5LogicTemperature()
        {  }

    //
    //  Interface Functions
    virtual float           LogicTemperature() const;

    virtual float           LogicWarningTemperature() const;
    virtual void            LogicWarningTemperature(float value);

    virtual float           LogicFailureTemperature() const;
    virtual void            LogicFailureTemperature(float value);

	virtual float 			LogicFanTemperature() const;
	virtual void            LogicFanTemperature(float value);

    virtual bool            Failed() const;

protected:
    // Registers
	class TempRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBitGroup    Degrees;

		TempRegister(AddressingSpace &space, int offset)
		: inherited(space, offset), Degrees(*this, 0, 10)
		{  }
	};

	class SetpointRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBitGroup    Degrees;

		SetpointRegister(AddressingSpace &space, int offset)
		: inherited(space, offset), Degrees(*this, 0, 16)
        {  }
	};

    TempRegister        TempReg;
    SetpointRegister    WarningReg;
    SetpointRegister    FailureReg;
    SetpointRegister    FanReg;
    float               FDefaultFailTemp;

    // Support Methods
    unsigned int    ToCounts(float value) const;
    float           ToDegrees(unsigned int value) const;
};


//===========================================================================
//  CLASS X5DeviceMap  -- Register & low level support for X5
//===========================================================================

class X5DeviceMap :  public PacketDeviceImpl,
                     public PacketDeviceMap,
                     public IUsesFpgaLogicVersionInfo,
                     public IUsesPciLogicVersionInfo,
                     public IUsesX5LogicTemperature,
                     public IUsesDioPort,
                     public IUsesPmcEeprom
{
    friend class X5AlertSystem;

public:
    enum IIDirection { tInput, tOutput, tInputOutput };
    enum LoopbackMode { lmNone, lmParallel, lmSerial };

    //
    //  Member Functions
    X5DeviceMap();
    virtual ~X5DeviceMap();

	//
	//  Make this property public
	using PacketDeviceImpl::PciLoopbackMode;

	// Methods
	void                Trigger(IIDirection dir, bool state);
	void  				CalibrateSystemMonitor();

	// Version
	std::string         Name() const;
	std::string         Revision() const;

	// Properties
	void                UserLoopbackMode(bool state);
	bool                UserLoopbackMode() const;

    void                ClockTestEnable(bool state);
    bool                ClockTestEnable() const;

    virtual void        TestCounterEnable(bool state);
    virtual bool        TestCounterEnable() const;

public: // pseudo-protected
    int  RcvPacketSize;
    int  XmitPacketSize;

    //
    //  Busmaster Intf functions
    virtual bool    AlertTest(unsigned int pid);
    virtual bool    AlertCheck(const Buffer & buffer);
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    void            DumpIsrStatus(std::vector<int> & vec);

protected:
    // Fields
    LoopbackMode           FLoopbackMode;

    // Data
    X5DeviceDriver *       DDeviceDriver;
    X5AlertSystem          Alert;


    // ???  May not be implemented
    void  LocalLoopbackMode(bool state);

    //
    //  Class PciTestRegister
    class PciTestRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       TestModeEnable;
        RegisterBit       GenerateRxData;
        RegisterBit       LoopbackEnable;
        RegisterBit       ClearTxCredit;
        RegisterBit       ClearRxCredit;
        RegisterBit       DrainXmitData;
        RegisterBitGroup  GenPacketSize;
        RegisterBit       GenPacketSizeUpdate;

    public:
        PciTestRegister( AddressingSpace &space, int offset );
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS UserFpgaCtlRegister  -- User FPGA control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class UserFpgaCtlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit           FpgaReset;
        RegisterBit           InGo;
        RegisterBit           InPowerCtl;
        RegisterBit           OutSleep;
        RegisterBit           Loopback;     // Pcie.
        RegisterBit           TestCounter;  // Adc (In)
        RegisterBit           DdrReset;
        RegisterBit           InTrigger;
        RegisterBit           OutTrigger;
        RegisterBit           OutGo;
        RegisterBit           QdrReset;  
        RegisterBit           RampCheckEnable;  
        RegisterBit           RampCheckStatus;  
        RegisterBit           AdcReady;     // Read only.     

        UserFpgaCtlRegister(AddressingSpace &space, int offset)
            :  inherited("X5DeviceMap::UserFpgaCtlRegister", space, offset),
               FpgaReset      (*this,  0),   
               InGo           (*this,  1),
               InPowerCtl     (*this,  2),
               OutSleep       (*this,  3),
               Loopback       (*this,  4),
               TestCounter    (*this,  5),
               DdrReset       (*this,  9),
               InTrigger      (*this, 10),
               OutTrigger     (*this, 11),
               OutGo          (*this, 16),
               QdrReset       (*this, 18),
               RampCheckEnable(*this, 20),
               RampCheckStatus(*this, 30),
               AdcReady       (*this, 31)
               
            {
            //  Default setup ????
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS UserFpgaCtlRegister  -- User FPGA status
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class UserFpgaStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           Qdr1DcmLocked;
        RegisterBit           Qdr0DcmLocked;
        RegisterBit           Qdr1CalibrationDone;
        RegisterBit           Qdr0CalibrationDone;
        RegisterBit           Ddr2InitializationDone;

        UserFpgaStatusRegister(AddressingSpace &space, int offset)
            :  inherited("X5DeviceMap::UserFpgaStatusRegister", space, offset),
               Qdr1DcmLocked            (*this, 24),
               Qdr0DcmLocked            (*this, 25),
               Qdr1CalibrationDone      (*this, 26),
               Qdr0CalibrationDone      (*this, 27),
               Ddr2InitializationDone   (*this, 28)
            {
            //  Default setup ????
			}
    };

    //
    //  ...Registers
    PciTestRegister         PciTestControlReg;
    UserFpgaCtlRegister     UserFpgaControlReg;
    UserFpgaStatusRegister  UserFpgaStatusReg;
    ShadowRegister          TestReg;
	ShadowRegister          Drp0;
	ShadowRegister          Drp1;

	// FPA Eeprom programmer
    class FpaLoadThread *   EepromLoader;
    FpaLoadBase *           FFpa;

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
