// X5IoDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5IoDevice_Mb.h>
#include <X5DeviceMap_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS X5IoDevice_Base  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X5IoDevice_Base
//---------------------------------------------------------------------------

X5IoDevice_Base::X5IoDevice_Base(X5DeviceMap * owner, X5IoDeviceRegisters & reg,
                                 IoInfoIntf * info)
    : FInfo(info), Owner(owner), 
      GainCoeffReg(owner->LogicMemory, reg.GainCoefRegister_Addr, info->Channels().Channels()),
      OffsetCoeffReg(owner->LogicMemory, reg.OffsetCoefRegister_Addr, info->Channels().Channels()),
      ChannelControlReg(owner->LogicMemory, reg.ChannelControlRegister_Addr),
	  TriggerReg(owner->LogicMemory, reg.TriggerControlRegister_Addr),
      DecimationReg(owner->LogicMemory, reg.DecimationRegister_Addr, 1),
	  PacketHeaderReg(owner->LogicMemory, reg.PacketHeaderRegister_Addr),
	  PacketControlReg(owner->LogicMemory, reg.PacketControlRegister_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X5IoDevice_Base
//---------------------------------------------------------------------------

X5IoDevice_Base::~X5IoDevice_Base()
{
    delete FInfo; FInfo = 0;     // owned object
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Framed() --
//---------------------------------------------------------------------------

void X5IoDevice_Base::Framed(int samples)
{
    const int SamplesPerFrame = 256;
    TriggerReg.Count.Value(samples/SamplesPerFrame);
    TriggerReg.FramedMode.Value(true);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Unframed() --
//---------------------------------------------------------------------------

void X5IoDevice_Base::Unframed()
{
    TriggerReg.FramedMode.Value(false);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::AcknowledgeAlert() -- Clear I/O specific alert condition
//---------------------------------------------------------------------------

void X5IoDevice_Base::AcknowledgeAlert()
{
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::ConfigureHardware() -- Apply hardware settings after reset
//---------------------------------------------------------------------------

void X5IoDevice_Base::ConfigureHardware()
{
    // Reapply Decimation Factor
    Decimation( Decimation() );
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void X5IoDevice_Base::ExternalTrigger(bool state)
{
    TriggerReg.External.Value(state);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool X5IoDevice_Base::ExternalTrigger() const
{
    return TriggerReg.External.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::PersistentTrigger() --  Set edge-triggered unframed mode
//---------------------------------------------------------------------------

void X5IoDevice_Base::PersistentTrigger(bool state)
{
    TriggerReg.Persistent.Value(state);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::PersistentTrigger() --  Query edge-triggered unframed mode
//---------------------------------------------------------------------------

bool X5IoDevice_Base::PersistentTrigger() const
{
    return TriggerReg.Persistent.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Info() --  Query I/O inteface
//---------------------------------------------------------------------------

IoInfoIntf & X5IoDevice_Base::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Decimation()
//---------------------------------------------------------------------------

void X5IoDevice_Base::Decimation(unsigned int factor)
{
    DecimationReg.Value(factor);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Decimation()
//---------------------------------------------------------------------------

unsigned int X5IoDevice_Base::Decimation() const
{
    return DecimationReg.Value();
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::PacketId() -- Set ID tage of received data packets
//---------------------------------------------------------------------------

void  X5IoDevice_Base::PacketId(unsigned char id)
{
    PacketHeaderReg.Id.Value(id);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::PacketId() -- Query ID tage of received data packets
//---------------------------------------------------------------------------

unsigned char X5IoDevice_Base::PacketId() const
{
	return PacketHeaderReg.Id.Value();
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::PacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

void  X5IoDevice_Base::PacketSize(int size_words)
{
   PacketHeaderReg.Size.Value(size_words);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::PacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

int  X5IoDevice_Base::PacketSize() const
{
	return PacketHeaderReg.Size.Value();
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::VariablePacketSize() -- Allow variable sized packets to
//                                       eliminate pipeline stalls
//---------------------------------------------------------------------------

void X5IoDevice_Base::VariablePacketSize(bool state)
{
    PacketControlReg.VariableSizeEnable.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::VariablePacketSize() -- Allow variable sized packets to
//                                       eliminate pipeline stalls
//---------------------------------------------------------------------------

bool X5IoDevice_Base::VariablePacketSize() const
{
    return PacketControlReg.VariableSizeEnable.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void X5IoDevice_Base::ChannelEnabled(int ch, bool state)
{
    ChannelControlReg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool X5IoDevice_Base::ChannelEnabled(int ch) const
{
    return ChannelControlReg.Bit(ch);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Pipeline() --  Set pipeline delay, in samples
//---------------------------------------------------------------------------

void X5IoDevice_Base::Pipeline(int value)
{
    ChannelControlReg.Pipeline.Value(value);
}

//---------------------------------------------------------------------------
//  X5IoDevice_Base::Pipeline() --  Query pipeline delay
//---------------------------------------------------------------------------

int X5IoDevice_Base::Pipeline() const
{
    return ChannelControlReg.Pipeline.Value();
}

//===========================================================================
//  CLASS X5IoDevice  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X5IoDevice
//---------------------------------------------------------------------------

X5IoDevice::X5IoDevice(X5DeviceMap * owner, X5IoDeviceRegisters & reg,
    IoInfoIntf * info, const PmcIdromSection & section, unsigned int one)
    : X5IoDevice_Base(owner, reg, info),
      IUsesSectionCalibration(section, info->Channels().Channels(), one)
{
}

//---------------------------------------------------------------------------
//  X5IoDevice::SetCalibrationOffset()
//---------------------------------------------------------------------------

void X5IoDevice::SetCalibrationOffset(int ch, int value)
{
    OffsetCoeffReg[ch] = value;
}

//---------------------------------------------------------------------------
//  X5IoDevice::SetCalibrationGain()
//---------------------------------------------------------------------------

void X5IoDevice::SetCalibrationGain(int ch, int value)
{
    GainCoeffReg[ch].Value(value);
}


} // Innovative


