// X5IoDevice_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5IoDevice_MbH
#define X5IoDevice_MbH

#include <HardwareRegister_Mb.h>
#include <HardwareInfoIntf_Mb.h>
#include <CalMemBase_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X5DeviceMap;

//===========================================================================
//  CLASS X5IoDeviceRegisters  -- Addresses for Device Registers
//===========================================================================

class X5IoDeviceRegisters
{
public:
    unsigned int   I2CRegister_Addr;
    unsigned int   Rom_Origin;
    
    unsigned int   GainCoefRegister_Addr;
    unsigned int   OffsetCoefRegister_Addr;
    unsigned int   ChannelControlRegister_Addr;
    unsigned int   TriggerControlRegister_Addr;
	unsigned int   DecimationRegister_Addr;
	unsigned int   PacketHeaderRegister_Addr;
	unsigned int   PacketControlRegister_Addr;
};

//===========================================================================
//  CLASS X5IoDevice_Base  -- Register & low level support for X5
//===========================================================================

class X5IoDevice_Base 
{
public:
    virtual ~X5IoDevice_Base();

    // Methods
    virtual void        Framed(int samples);
    virtual void        Unframed();
    IoInfoIntf &        Info();

    virtual void        AcknowledgeAlert();
    virtual void        ConfigureHardware();

    // Properties
    virtual void        ExternalTrigger(bool state);
    virtual bool        ExternalTrigger() const;

    virtual void        PersistentTrigger(bool state);
    virtual bool        PersistentTrigger() const;

    virtual void         Decimation(unsigned int factor);
    virtual unsigned int Decimation() const;

    void                PacketId(unsigned char id);
    unsigned char       PacketId() const;

    void                PacketSize(int size);
	int                 PacketSize() const;

	virtual void        VariablePacketSize(bool enable);
	virtual bool        VariablePacketSize() const;

	virtual void        ChannelEnabled(int ch, bool state);
    virtual bool        ChannelEnabled(int ch) const;

    virtual void        Pipeline(int value);
	virtual int         Pipeline() const;

protected:
    // Ctor
    X5IoDevice_Base(X5DeviceMap * owner, X5IoDeviceRegisters & reg,
                    IoInfoIntf * info);

    // Fields
    IoInfoIntf *            FInfo;

    // Data
    X5DeviceMap *           Owner;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PacketRegister  -- Rx Packet size management
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PacketRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Size;
        RegisterBitGroup      Id;

        //
        //  Member Functions
        PacketRegister(AddressingSpace &space, int offset)
            : inherited("X5IoDevice_Base::PacketRegister", space, offset),
              Size(*this, 0, 24), Id(*this, 24, 8)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TriggerRegister  -- Trigger control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Count;
		RegisterBit			  Persistent;		// latches on external posedge.
		RegisterBit           FramedMode;
		RegisterBit           External;

        TriggerRegister(AddressingSpace &space, int offset)
            :  inherited("X5IoDevice_Base::TriggerRegister", space, offset),
			   Count(*this, 0, 16),   Persistent(*this, 29),
			   FramedMode(*this, 30), External(*this, 31)

            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ChannelControlRegister  -- Channel control control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ChannelControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Enable;
        RegisterBitGroup      Pipeline;
		RegisterBit			  AdcDataTimesFour;
		
        ChannelControlRegister(AddressingSpace &space, int offset)
            :  inherited("X5IoDevice_Base::ChannelControlRegister", space, offset),
			   Enable(*this, 0, 4), Pipeline(*this, 4, 4),
			   AdcDataTimesFour(*this, 8)

            {
            //  Default setup ????
            }
	};

    //
    //  Class PacketControlRegister
    class PacketControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       VariableSizeEnable;

    public:
        PacketControlRegister( AddressingSpace &space, int offset )
            : inherited("X5DeviceMap::PacketControlRegister", space, offset),
            VariableSizeEnable      (*this, 0)
            {  }
    };

	// Registers
    RegisterArray<Register>     GainCoeffReg;
    RegisterArray<Register>     OffsetCoeffReg;
	ChannelControlRegister      ChannelControlReg;
    TriggerRegister             TriggerReg;
    ShadowRegister              DecimationReg;
	PacketRegister              PacketHeaderReg;
	PacketControlRegister   	PacketControlReg;
};

//===========================================================================
//  CLASS X5IoDevice  -- Register & low level support for X5
//===========================================================================

class X5IoDevice : public X5IoDevice_Base,
                   public IUsesSectionCalibration
{
public:
    virtual void    TestEnable(bool state) = 0;
    virtual bool    TestEnable() const = 0;

protected:
    X5IoDevice(X5DeviceMap * owner, X5IoDeviceRegisters & reg,
               IoInfoIntf * info, const PmcIdromSection & section,
			   unsigned int one);
                                
    // Overrides: Apply to ROM and hardware
    virtual void    SetCalibrationOffset(int ch, int value);
    virtual void    SetCalibrationGain(int ch, int value);
};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
