// X5_210MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5_210M_Mb.h>
#include <X5_210MDeviceMap_Mb.h>
#include <X5MemoryMap_Mb.h>
#include <X5_210MMemoryMap_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X5_MemoryMap;
using namespace X5_210MMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxInDevices      = 4;
const int    InputWidth_Bits   = 14;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 250.0e6f;
const float  InputMinRate      = 20.0e6f;
const int    InputGangSize     = 2;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input and Output Coefficient section
//       2channels x 2 values + 1 calibration flag + 1 spare
const int romX5_210M_InputCoeffOffset   = 0;
const int romX5_210M_InChannels         = MaxInDevices;
const int romX5_210M_InputCoeffSize     = romX5_210M_InChannels * 2 + 1 + 1;
const int romX5_210M_CoeffSize          = romX5_210M_InputCoeffSize;


//===========================================================================
// class X5_210MInputInfoIntf
//===========================================================================

class X5_210MInputInfoIntf : public IoInfoIntf
{
public:
    X5_210MInputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxInDevices, InputWidth_Bits, InputMuxCount, InputIsUnipolar)
        {  FSpan.push_back(SpanInfo(1000, -1000));  }
    virtual ~X5_210MInputInfoIntf()  {  }

    virtual UnpairedChannelInfo &       Channels()              {  return FChannels;}
    virtual float                       MaxRate() const         {  return InputMaxRate;  }
    virtual float                       MinRate() const         {  return InputMinRate;  }
    virtual const SpanArray &           Span() const            {  return FSpan;  }
    virtual int                         GangSize() const        {  return InputGangSize;  }
    virtual bool                        IsStackable() const     {  return false;  }
    virtual bool                        IsInverted() const      {  return true;  }
    virtual bool                        IsMuxed() const         {  return false;  }
    virtual bool                        MuxDepth() const        {  return InputMuxCount;  }
    virtual int                         BufferSize() const      {  return 65536;  }
    virtual int                         ClockFactor() const     {  return InputClockFactor;  }
    virtual float                       ReferenceClock() const  {  return 0.0;  }
    virtual int               	        SamplesPerFrame() const {  return 256;  }

private:
    UnpairedChannelInfo         FChannels;
    SpanArray                   FSpan;
};

//===========================================================================
//  CLASS X5_210MInputDevice  -- Register & low level support for X5
//===========================================================================

//---------------------------------------------------------------------------
//  X5_210MInputDevice Ctor
//---------------------------------------------------------------------------

X5_210MInputDevice::X5_210MInputDevice(X5DeviceMap * owner,
                                       X5IoDeviceRegisters & reg,
                                       AddressingSpace & space, int ctl)
    : inherited(owner,
                reg,
                new X5_210MInputInfoIntf,
                PmcIdromSection(owner->Rom(),
                                PmcIdrom::waModule,
                                romX5_210M_InputCoeffOffset,
                                romX5_210M_InputCoeffSize),
                0x10000),	// 'one' value for gain registers
      FPatternEnable(false),
      ConfigReg(space, ctl)
{
}

//---------------------------------------------------------------------------
//  X5_210MInputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_210MInputDevice::TestEnable(bool enable)
{
    Owner->TestCounterEnable(enable);
}

//---------------------------------------------------------------------------
//  X5_210MInputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_210MInputDevice::TestEnable() const
{
    return Owner->TestCounterEnable() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_210MInputDevice::AdcConfig()
//---------------------------------------------------------------------------

void  X5_210MInputDevice::AdcConfig(int addr, unsigned char value, int mask)
{
    ConfigReg.Port(addr, value, mask);
}

//---------------------------------------------------------------------------
//  X5_210MInputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void X5_210MInputDevice::ConfigureHardware()
{
    inherited::ConfigureHardware();

    if (Owner->FpgaLogicRevision() >= 2)     // Rev C or above
        {
        // X5-210M rev C A/D serial register initialization
        // initialize all four ADS6149 devices simultaneously
        ConfigReg.Port(0x20, 0x00);
        ConfigReg.Port(0x3f, 0x20);
        ConfigReg.Port(0x41, 0x80);
        // clock aligned with data.  Used for rev C test plan test data
        ConfigReg.Port(0x44, 0xd8);
         // two's complement output format (for unsigned, use 0x06)
        ConfigReg.Port(0x50, 0x04);
        ConfigReg.Port(0x51, 0x00);
        ConfigReg.Port(0x52, 0x00);
        ConfigReg.Port(0x53, 0x00);
        ConfigReg.Port(0x55, 0x00);
        // test pattern off
        ConfigReg.Port(0x62, FPatternEnable ? 0x04 : 0x00);

        ConfigReg.Port(0x63, 0x00);
        }

    OpenWire::NotifyEvent e;
    OnAfterConfigure.Execute(e);
}

//=============================================================================
//  CLASS X5_210MClock  --  PLL timebase control
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

X5_210MClock::X5_210MClock(X5_210MDeviceMap * owner,
    AddressingSpace & space, int reg)
    : Owner(owner), ClockReg(space, reg)
{
}

//---------------------------------------------------------------------------
//  X5_210MClock::Frequency() -- Change output frequency
//---------------------------------------------------------------------------

void X5_210MClock::Frequency(double value)
{
    FFrequency = value;
}

//---------------------------------------------------------------------------
//  X5_210MClock::Frequency() -- Query output frequency
//---------------------------------------------------------------------------

double X5_210MClock::Frequency() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------
//  X5_210MClock::FrequencyActual() -- Query actual output frequency
//---------------------------------------------------------------------------

double X5_210MClock::FrequencyActual() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------
//  X5_210MClock::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

X5_210MClock::IIClockSource X5_210MClock::Source() const
{
    return ClockReg.Mode.Value() ? csInternal : csExternal;
}

//---------------------------------------------------------------------------
//  X5_210MClock::Source() -- Select clock source
//---------------------------------------------------------------------------

void X5_210MClock::Source(IIClockSource src)
{
    ClockReg.Mode.Value(src == csInternal ? true : false);
}

//---------------------------------------------------------------------------
//  X5_210MClock::Reapply() -- Reapply all clock settings
//---------------------------------------------------------------------------

void X5_210MClock::Reapply()
{
    Source(Source());
}

//---------------------------------------------------------------------------
//  X5_210MClock::Locked() -- Return PLL lock status
//---------------------------------------------------------------------------

bool X5_210MClock::Locked() const
{
    return true;
}

//===========================================================================

static X5IoDeviceRegisters X5_210MInputDeviceRegisters =
    { mmI2c, 0,  mmInGainCoef, mmInOffsetCoef, mmInChannelEnable,
      mmInTriggerControl, mmInDecimation, mmInPacketHeader, mmRxPacketControl
    };

//===========================================================================
//  CLASS X5_210MDeviceMap  -- Register & low level support for SD module
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

X5_210MDeviceMap::X5_210MDeviceMap(X5_210M * owner)
    :  IUsesStreamDeviceEventCallbacks(owner),
       FInput (new X5_210MInputDevice (this, X5_210MInputDeviceRegisters, LogicMemory, mmAdcControl)),
       Owner(owner), Clock(this, LogicMemory, mmClockSource)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_210MDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_210MDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_210MDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_210MDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_210MDeviceMap::OnAfterStreamStopHandler );

//    DDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_210MDeviceMap::OnStreamConfigureHandler );
//    DDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_210MDeviceMap::OnBeforeStreamStartHandler );
//    DDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_210MDeviceMap::OnAfterStreamStartHandler );
//    DDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_210MDeviceMap::OnBeforeStreamStopHandler );
//    DDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_210MDeviceMap::OnAfterStreamStopHandler );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X5_210MDeviceMap::~X5_210MDeviceMap()
{
    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X5_210MDeviceMap::OpenHardware()
{
    X5DeviceMap::OpenHardware();

    // Configure A/Ds
    for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
        {
        FInput->PacketSize(0x4000);
        FInput->PacketId(0x0);
        FInput->Gain(ch, 1.0);
        FInput->Offset(ch, 0.0);
        Input().Unframed();
        }
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X5_210MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    X5DeviceMap::ConfigureHardware();

    // Reapply Hardware Settings for  X5 IO Devices
    FInput->ConfigureHardware();

    // Re-apply all clock settings
    Clock.Reapply();

    //
    //  Apply Gain/Offset Coefficients
    if (FInput->Calibrated())
        FInput->ApplyRomCoefficients();
    else
        FInput->ApplyDefaultCoefficients();

    //
    //  Apply input settings
    FInput->PacketId(FInput->PacketId());
    FInput->PacketSize(FInput->PacketSize());
    for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
        FInput->ChannelEnabled(ch, FInput->Info().Channels().Enabled(ch));

    // Configure triggers
    FInput->TriggerReg.Apply();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_210MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    UserFpgaControlReg.InGo.Set();
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_210MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_210MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_210MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.Apply();

    Trigger(tInput, false);

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X5_210MDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X5_210MInputDevice & X5_210MDeviceMap::Input()
{
    return *FInput;
}



} // namespace Innovative


