// X5_210MDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_210MDeviceMap_MbH
#define X5_210MDeviceMap_MbH

#include <X5_210M_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <SystemSupport_Mb.h>
#include <Events_Mb.h>


namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X5_210MDeviceMap;

//==============================================================================
//  CLASS X5_210MClock -- GSPS clock control
//==============================================================================

class X5_210MClock : public X5ClockIntf
{
public:
    X5_210MClock(X5_210MDeviceMap * owner, AddressingSpace & space, int reg);

    //
    //  Interface Functions
    // Methods
    virtual double			FrequencyActual() const;

    // Properties
    virtual void    		Frequency(double value);
    virtual double			Frequency() const;

    virtual IIClockSource   Source() const;
    virtual void            Source(IIClockSource src);

    virtual void 			Reapply();
    virtual bool            Locked() const;

protected:
    // Data
    X5_210MDeviceMap *      Owner;
    double                  FFrequency;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ClockRegister  -- External vs. Internal clock source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ClockRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit Mode;

        ClockRegister(AddressingSpace &space, int offset)
            : inherited("X5DeviceMap::ClockRegister", space, offset),
              Mode(*this, 0)
        {
            Mode.Value(true); // default to internal.
        }
    };

    ClockRegister       ClockReg;


//    virtual void            SetFrequency(double freq);
};

//===========================================================================
//  CLASS X5_210MInputDevice  -- Register & low level support for X5 ADCs
//===========================================================================

class X5_210MInputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
    friend class X5_210MDeviceMap;

public:
    // Methods
    // A/D configuration
    void            AdcConfig(int addr, unsigned char value, int mask=0xf);

    // Properties
    virtual void    TestEnable(bool state);
    virtual bool    TestEnable() const;

    void            PatternEnable(bool state);
    bool            PatternEnable() const;

    OpenWire::EventHandler<OpenWire::NotifyEvent>   OnAfterConfigure;

protected:
    // Fields
    bool        FPatternEnable;

    // Ctor
    X5_210MInputDevice(X5DeviceMap * owner, X5IoDeviceRegisters & reg,
        AddressingSpace & space, int ctl);

    // Method
    virtual void        ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ConfigRegister  -- A/D serial config register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ConfigRegister : public Register
    {
        typedef Register inherited;

    public:
        ConfigRegister(AddressingSpace &space, int offset)
            : inherited("X5_210MInputDevice::ConfigRegister", space, offset)
        {
        }

    void    Port(int address, int value, int mask=0xf)
        {
        // Wait for interface ready
        while (!Ready())
            Innovative::Sleep(1);

        mask &= 0xf;
        Value((mask << 16) | (address << 8) | value);
        }

    protected:
    bool    Ready() const
        {  return (Value() & (1 << 31)) ? true : false;  }
    };

    ConfigRegister      ConfigReg;
};

//===========================================================================
//  CLASS X5_210MDeviceMap  -- Register & low level support for X5-210M module
//===========================================================================
class X5_210M;

class X5_210MDeviceMap :  public X5DeviceMap,
                          public IUsesStreamDeviceEventCallbacks

{
    friend class X5_210M;
    friend class X5_210MInputDevice;

public:
    //
    //  Member Functions
    X5_210MDeviceMap(X5_210M * owner);
    virtual ~X5_210MDeviceMap();

    X5_210MInputDevice  &    Input();

public: // pseudo-protected
    virtual void  OpenHardware();

protected:
    // Fields
    X5_210MInputDevice  *   FInput;

    // Data
    X5_210M *           Owner;
    X5_210MClock        Clock;
    Trace               Debug;

    // Methods
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
