//---------------------------------------------------------------------------
// X5_210M_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X5_210M_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5_210M  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X5_210M::X5_210M()
    :  X5Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X5_210M::~X5_210M()
{
    Close();
}

//---------------------------------------------------------------------------
//  X5_210M::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X5_210M::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X5_210M::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X5_210M::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X5_210M::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X5_210M::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X5_210M::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X5_210M::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X5_210M::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X5_210M::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X5_210M::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X5_210M::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X5_210M::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X5_210M::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X5_210M::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X5_210M::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X5_210M::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X5_210M::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X5_210M::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X5_210M::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X5_210M::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X5_210M::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_210M::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X5_210M::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_210M::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X5_210M::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X5_210M::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X5_210M::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X5_210M::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X5_210M::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X5_210M::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X5_210M::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X5_210M::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & X5_210M::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  X5_210M::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X5_210M::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X5_210M::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X5_210M::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X5_210M::PciLoopbackMode() --
//------------------------------------------------------------------------------

X5_210M::LoopbackMode X5_210M::PciLoopbackMode()
{
    X5_210M::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X5_210M::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5_210M::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X5_210M::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5_210M::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X5_210M::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X5_210MInputDevice & X5_210M::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X5_210M::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X5_210M::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_210M::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X5_210M::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_210M::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X5_210M::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X5DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X5_210M::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX5Alerts & X5_210M::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X5_210M::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X5_210M::Thermal() 
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_210M::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X5_210M::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X5_210M::Clock() --
//---------------------------------------------------------------------------

X5_210MClock & X5_210M::Clock()
{
    return Device.Clock;
}

//---------------------------------------------------------------------------
//  X5_210M::BusMasterSize() --
//---------------------------------------------------------------------------

void X5_210M::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_210M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_210M::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_210M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_210M::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_210M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_210M::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_210M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_210M::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_210M::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X5_210M::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate(": ?X5-210M "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   PLL  Vcxo  RefVcxo Adc0  Adc1 \" "
                    "    cr  "
                    "    cr .\" ---- X5-210M Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   ?X5-210M  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("X5-210M-specific wordset loaded \n" );

    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X5_210M::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X5_210M::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X5_210M::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X5_210M::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X5_210M::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X5_210M::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X5_210M::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X5_210M::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
