// X5_400MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5_400M_Mb.h>
#include <X5_400MDeviceMap_Mb.h>
#include <X5MemoryMap_Mb.h>
#include <X5_400MMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>
#include <Magic_Mb.h>
#include <sstream>

using namespace std;

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X5_MemoryMap;
using namespace X5_400MMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxInDevices      = 2;
const int    InputWidth_Bits   = 14;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 400.0e6;
const float  InputMinRate      = 20.0e6;
const int    InputGangSize     = 2;

const int    MaxOutDevices     = 2;
const int    OutputWidth_Bits  = 16;
const int    OutputMuxCount    = 0;
const bool   OutputIsUnipolar  = false;
const int    OutputClockFactor = 1;
const float  OutputMaxRate     = 500.0e6;
const float  OutputMinRate     = 16.0e6;
const int    OutputGangSize    = 2;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input and Output Coefficient section
//       2channels x 2 values + 1 calibration flag + 1 spare
const int romX5_400M_InputCoeffOffset   = 0;
const int romX5_400M_InChannels         = 2;
const int romX5_400M_InputCoeffSize     = romX5_400M_InChannels * 2 + 1 + 1;
const int romX5_400M_OutputCoeffOffset  = romX5_400M_InputCoeffSize;
const int romX5_400M_OutChannels        = 2;
const int romX5_400M_OutputCoeffSize    = romX5_400M_OutChannels * 2 + 1 + 1;
const int romX5_400M_CoeffSize          = romX5_400M_InputCoeffSize + romX5_400M_OutputCoeffSize;


//===========================================================================
// class X5_400MInputInfoIntf
//===========================================================================

class X5_400MInputInfoIntf : public IoInfoIntf
{
public:
    X5_400MInputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxInDevices, InputWidth_Bits, InputMuxCount, InputIsUnipolar)
        {  FSpan.push_back(SpanInfo(1000, -1000));  }
    virtual ~X5_400MInputInfoIntf()  {  }

    virtual UnpairedChannelInfo &       Channels()              {  return FChannels;}
    virtual float                       MaxRate() const         {  return InputMaxRate;  }
    virtual float                       MinRate() const         {  return InputMinRate;  }
    virtual const SpanArray &           Span() const            {  return FSpan;  }
    virtual int                         GangSize() const        {  return InputGangSize;  }
    virtual bool                        IsStackable() const     {  return false;  }
	virtual bool                        IsInverted() const      {  return true;  }
    virtual bool                        IsMuxed() const         {  return false;  }
    virtual bool                        MuxDepth() const        {  return InputMuxCount;  }
    virtual int                         BufferSize() const      {  return 65536;  }
    virtual int                         ClockFactor() const     {  return InputClockFactor;  }
    virtual float                       ReferenceClock() const  {  return 0.0;  }
	virtual int               	        SamplesPerFrame() const {  return 256;  }

private:
    UnpairedChannelInfo         FChannels;
	SpanArray                   FSpan;
};

//===========================================================================
// class X5_400MOutputInfoIntf
//===========================================================================

class X5_400MOutputInfoIntf : public IoInfoIntf
{
public:
	X5_400MOutputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxOutDevices, OutputWidth_Bits, OutputMuxCount, OutputIsUnipolar)
		{  FSpan.push_back(SpanInfo(1000, -1000));  }
	virtual ~X5_400MOutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &       Channels()              {  return FChannels; }
	virtual float                       MaxRate() const         {  return OutputMaxRate; }
	virtual float                       MinRate() const         {  return OutputMinRate; }
	virtual const SpanArray &           Span() const            {  return FSpan;  }
	virtual int                         GangSize() const        {  return OutputGangSize; }
	virtual bool                        IsStackable() const     {  return true;  }
	virtual bool                        IsInverted() const      {  return true;  }
	virtual bool                        IsMuxed() const         {  return false;  }
	virtual bool                        MuxDepth() const        {  return OutputMuxCount;  }
	virtual int                         BufferSize() const      {  return 65536;  }
	virtual int                         ClockFactor() const     {  return OutputClockFactor;  }
	virtual float                       ReferenceClock() const  {  return 0.0;  }
	virtual int               	        SamplesPerFrame() const {  return 256;  }

private:
	UnpairedChannelInfo         FChannels;
	SpanArray                   FSpan;
};


//===========================================================================
//  CLASS X5_400MInputDevice  -- Register & low level support for X5
//===========================================================================

//---------------------------------------------------------------------------
//  X5_400MInputDevice Ctor
//---------------------------------------------------------------------------

X5_400MInputDevice::X5_400MInputDevice(X5DeviceMap * owner,
									   X5IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X5_400MInputInfoIntf,
				PmcIdromSection(owner->Rom(),
								PmcIdrom::waModule,
								romX5_400M_InputCoeffOffset,
								romX5_400M_InputCoeffSize),
				0x10000)	// 'one' value for gain registers
{
}

//---------------------------------------------------------------------------
//  X5_400MInputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_400MInputDevice::TestEnable(bool enable)
{
	Owner->TestCounterEnable(enable);
}

//---------------------------------------------------------------------------
//  X5_400MInputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_400MInputDevice::TestEnable() const
{
	return Owner->TestCounterEnable() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_400MInputDevice::ScaleTimesFour()
//---------------------------------------------------------------------------

void X5_400MInputDevice::ScaleTimesFour(bool enable)
{
	ChannelControlReg.AdcDataTimesFour.Value(enable);

}

//---------------------------------------------------------------------------
//  X5_400MInputDevice::ScaleTimesFour()
//---------------------------------------------------------------------------

bool X5_400MInputDevice::ScaleTimesFour()
{
	return ChannelControlReg.AdcDataTimesFour.Value();
}

//===========================================================================
//  CLASS X5_400MOutputDevice  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X5_400MOutputDevice
//---------------------------------------------------------------------------

//  Fill in the Info argument for the SPI device constructors
//      -address-  -direction bit-  -ready bit-  -data valid bit-
StdSpiDevice::Info  Dac0Info = { mmOutSpiCtl, 16, 23, 22 };

//---------------------------------------------------------------------------
//  X5_400MOutputDevice Ctor
//---------------------------------------------------------------------------

X5_400MOutputDevice::X5_400MOutputDevice(X5DeviceMap * owner,
										 X5IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X5_400MOutputInfoIntf,
				PmcIdromSection(owner->Rom(),
								PmcIdrom::waModule,
								romX5_400M_OutputCoeffOffset,
								romX5_400M_OutputCoeffSize),
				0x10000),	// 'one' value for gain registers
	  AnalogControlReg(owner->LogicMemory, mmOutControl),
	  AnalogStatusReg(owner->LogicMemory, mmOutControl),
	  TestPhaseReg(owner->LogicMemory, mmGeneratorPhase, 2),
	  TestControlReg(owner->LogicMemory, mmGeneratorControl),
	  SpiStatusReg(owner->LogicMemory, mmOutSpiCtl),
	  SpiReg(owner->LogicMemory, mmOutSpiCtl),
      Dac0Spi(owner->LogicMemory, Dac0Info, "Dac0"),

	  FTestMode(tmRamp),
	  FEnabled(false),
	  FInterpolate(1),
	  FSincFilterEnabled(false),
	  FTransitionFrequency(200.0e6)
{
	for (unsigned int i = 0; i < 2; ++i)
	  FTestFrequency[i] = 10.0e6;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TestMode()
//---------------------------------------------------------------------------

unsigned int X5_400MOutputDevice::TestMode()const
{
	return FTestMode;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TestMode()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::TestMode(unsigned int mode)
{
	TestControlReg.Mode.Value(mode);
	FTestMode = (IITestMode)mode;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::TestEnable(bool enable)
{
	TestControlReg.Enable.Value(enable);
	FEnabled = enable;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::TestEnable() const
{
	return FEnabled;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TestFrequency()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::TestFrequency(int ch, float freq)
{
    // Sample clock, in Mhz
    X5_400MDeviceMap * owner = dynamic_cast<X5_400MDeviceMap*>(Owner);
	const double FClk = owner->Clock.FrequencyActual()/owner->Clock.DacDivider();

	unsigned int phase = FClk ? static_cast<unsigned int>(freq * ((1 << 25) / FClk)) : 1;
	TestPhaseReg[ch].PhaseIncrement.Value(phase);
	FTestFrequency[ch] = freq;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TestFrequency()
//---------------------------------------------------------------------------

float X5_400MOutputDevice::TestFrequency(int ch) const
{
	return FTestFrequency[ch];
}

//---------------------------------------------------------------------------
// X5_400MOutputDevice::SincFilterEnabled()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::SincFilterEnabled(bool value)
{
	FSincFilterEnabled = value;
}

//---------------------------------------------------------------------------
// X5_400MOutputDevice::SincFilterEnabled()
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::SincFilterEnabled()
{
    return FSincFilterEnabled;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::Spi()
//---------------------------------------------------------------------------

unsigned char X5_400MOutputDevice::Spi(int addr)
{
    if ( Owner->FpgaLogicVersion() < 0x708 )
        return OldSpi(addr);
    else
        return Dac0Spi.Spi(addr);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::Spi()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::Spi(int addr, unsigned char value)
{
    if ( Owner->FpgaLogicVersion() < 0x708  )
        OldSpi(addr, value);
    else
        Dac0Spi.Spi(addr, value);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::OldSpi()
//---------------------------------------------------------------------------

unsigned char X5_400MOutputDevice::OldSpi(int addr)
{
    SpiReady();

    // Write to D/A register via Spi register
    SpiReg.Address.Value(addr);
    SpiReg.ReadStrobe.Set();
    SpiReg.Apply();

    SpiNotReady();
    SpiReady();

    unsigned char result = static_cast<unsigned char>(SpiReg.Data.Value());

	SpiReg.ReadStrobe.Clear();
    SpiReg.Apply();

    return result;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::OldSpi()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::OldSpi(int addr, unsigned char value)
{
    SpiReady();

    // Write to D/A register via OldSpi register
    SpiReg.Address.Value(addr);
    SpiReg.Data.Value(value);
    SpiReg.WriteStrobe.Set();
    SpiReg.Apply();

    SpiNotReady();
    SpiReady();

    SpiReg.WriteStrobe.Clear();
    SpiReg.Apply();

    std::stringstream msg;
    msg << std::hex << "Spi(0x" << addr << ", 0x" << (short)value << ") = [0x" << SpiReg.Value() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::SpiReady()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::SpiReady()
{
	Sleep(2);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::SpiNotReady()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::SpiNotReady()
{
	Sleep(2);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TransitionFrequency()
//---------------------------------------------------------------------------

float X5_400MOutputDevice::TransitionFrequency() const
{
	return FTransitionFrequency;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::TransitionFrequency()
//---------------------------------------------------------------------------

void  X5_400MOutputDevice::TransitionFrequency(float value)
{
	FTransitionFrequency = value;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::Interpolate()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::Interpolate(unsigned int value)
{
    if (!IsPowerOfTwo(value) || (value < 1) || (value > 8))
        return;

	FInterpolate = value;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::Interpolate()
//---------------------------------------------------------------------------

unsigned int X5_400MOutputDevice::Interpolate() const
{
    return FInterpolate;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::OutputStack()
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::OutputStack() const
{
	return AnalogControlReg.Stack.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::OutputStack()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::OutputStack(bool value)
{
    AnalogControlReg.Stack.Value(value);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::Configure()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::ConfigureHardware()
{
    inherited::ConfigureHardware();

    int pll_freq = 0;
	int pll_kv = 0;
    int pll_div = 0;

	X5_400MDeviceMap * owner = dynamic_cast<X5_400MDeviceMap*>(Owner);
	float rate = static_cast<float>(owner->Clock.FrequencyActual()/owner->Clock.DacDivider());
	float FDac = static_cast<float>(rate * FInterpolate / 1.e6);

	if (FDac < 25.0)
		Debug.Log("Error: FDac < 25 MHz");
	else if (FDac < 28.125)
		{  pll_freq = 0;   pll_kv = 1;     pll_div = 3;  }
	else if (FDac < 46.25)
		{  pll_freq = 0;   pll_kv = 0;     pll_div = 3;  }
	else if (FDac < 60.)
		{  pll_freq = 0;   pll_kv = 1;     pll_div = 3;  }
	else if (FDac < 61.875)
		{  pll_freq = 1;   pll_kv = 0;     pll_div = 3;  }
	else if (FDac < 65.)
		{  pll_freq = 1;   pll_kv = 1;     pll_div = 3;  }
	else if (FDac < 92.5)
		{  pll_freq = 0;   pll_kv = 0;     pll_div = 2;  }
	else if (FDac < 120.)
		{  pll_freq = 0;   pll_kv = 1;     pll_div = 2;  }
	else if (FDac < 123.75)
		{  pll_freq = 1;   pll_kv = 0;     pll_div = 2;  }
	else if (FDac < 130.0)
		{  pll_freq = 1;   pll_kv = 1;     pll_div = 2;  }
	else if (FDac < 185.0)
		{  pll_freq = 0;   pll_kv = 0;     pll_div = 1;  }
    else if (FDac < 240.0)
        {  pll_freq = 0;   pll_kv = 1;     pll_div = 1;  }
	else if (FDac < 247.5)
		{  pll_freq = 1;   pll_kv = 0;     pll_div = 1;  }
	else if (FDac < 260.0)
		{  pll_freq = 1;   pll_kv = 1;     pll_div = 1;  }
	else if (FDac < 370.0)
		{  pll_freq = 0;   pll_kv = 0;     pll_div = 0;  }
	else if (FDac < 480.0)
		{  pll_freq = 0;   pll_kv = 1;     pll_div = 0;  }
	else if (FDac < 495.0)
		{  pll_freq = 1;   pll_kv = 0;     pll_div = 0;  }
	else if (FDac < 520.0)
		{  pll_freq = 1;   pll_kv = 1;     pll_div = 0;  }
	else
		Debug.Log("Error: FDac > 520 MHz");

//    int pll_lock = 1 << pll_div;
	int log_interpolate = FloorLog2(FInterpolate);

	// Awaken dacs, low-pass first FIR filter
	Spi(0x00, 0x00);
	int interp[] = { 0, 0, 1, 3 };
	int inv_clk = InvClock() ? 1 : 0;
	Spi(0x01, (pll_div << 6) | (pll_freq << 5) | (pll_kv << 4) |
			  (interp[log_interpolate] << 2) | (inv_clk << 1) | 1);
	// Two's complement, FIR bypass
	Spi(0x02, 0x10 | (log_interpolate ? 0x00 : 0x02));
	Spi(0x03, FSincFilterEnabled ? 0x01 : 0x00);
	//  Turn on Stack Mode if only channel 0 is active
	if ((FInfo->Channels().ActiveChannels() == 1) && FInfo->Channels().Enabled(0))
		{
		OutputStack(true);
		Spi(0x04, 0x20);
		}
	else
		{
		OutputStack(false);
		Spi(0x04, 0x00);
		}
	Spi(0x05, 0x00);
	Spi(0x06, 0x00);
	Spi(0x07, 0x00);
    Spi(0x08, 0x00);
    Spi(0x09, 0x00);
    Spi(0x0A, 0x00);
    Spi(0x0B, 0x00);
	Spi(0x0C, 0x0C);
    Spi(0x0D, 0x00);
    Spi(0x0E, 0x00);
    Spi(0x0F, 0x00);
    Spi(0x10, 0x00);
    Spi(0x11, 0x00);
    Spi(0x12, 0x00);
    Spi(0x13, 0x00);
    Spi(0x14, 0x00);
    Spi(0x15, 0x00);
    Spi(0x16, 0x00);
    Spi(0x17, 0x00);
    Spi(0x18, 0x00);
    Spi(0x19, 0xFF);
    Spi(0x1A, 0x00);

    //              x1    x2    x4    x4 (x8 not supported)
	int atest[] = { 0x1d, 0x1e, 0x1f, 0x1f };
	Spi(0x1B, atest[log_interpolate] << 3);

	Spi(0x1C, 0x00);
	Spi(0x1D, 0x00);
	Spi(0x1E, 0x00);
	Spi(0x1F, 0x00);

	TestMode(FTestMode);
	for (unsigned int i = 0; i < 2; ++i)
		TestFrequency(i, FTestFrequency[i]);
	TestEnable(FEnabled);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::InvMode()
//  Return state of INV D/A bit based on factors below:  
// Channels      Interpolation      Range(s) where INV_PLLMODE=1
//	1              1               230-350MHz; 480-500MHz (1)
//	1              2               230-330MHz
//	1              4               none
//	1              8               none (2)
//	2              1               270-330MHz (3)
//	2              2               220-350MHz
//	2              4               none
//	2              8               none
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::InvClock() const
{
	bool result = false;

	X5_400MDeviceMap * owner = dynamic_cast<X5_400MDeviceMap*>(Owner);
	int channels = FInfo->Channels().ActiveChannels();
	int interpolation = FInterpolate;
	float rate = static_cast<float>(owner->Clock.FrequencyActual()/owner->Clock.DacDivider());

	if ((channels==1) && (interpolation==1))
		if (Within(rate, 230e6f, 350e6f) || Within(rate, 480e6f, 500e6f))
			result = true;

	if ((channels==1) && (interpolation==2))
		if (Within(rate, 230e6f, 330e6f))
			result = true;

	if ((channels==2) && (interpolation==1))
		if (Within(rate, 270e6f, 330e6f))
			result = true;

	if ((channels==2) && (interpolation==2))
		if (Within(rate, 220e6f, 350e6f))
			result = true;

	return result;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::CheckEnable()
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::CheckEnable(int ch) const
{
	if (ch)
		return AnalogControlReg.Ch1CheckEnable.Value() ? true : false;
	else
		return AnalogControlReg.Ch0CheckEnable.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::CheckEnable()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::CheckEnable(int ch, bool state)
{
    if (ch)
        AnalogControlReg.Ch1CheckEnable.Value(state ? 1 : 0);
    else
        AnalogControlReg.Ch0CheckEnable.Value(state ? 1 : 0);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::CheckType()
//---------------------------------------------------------------------------

X5_400MOutputDevice::IICheckType X5_400MOutputDevice::CheckType(int ch) const
{
    bool value;
    if (ch)
        value = AnalogControlReg.Ch1Incrementing.Value();
    else
        value = AnalogControlReg.Ch0Incrementing.Value();

    return value ? ctIncrementing : ctDecrementing;
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::CheckType()
//---------------------------------------------------------------------------

void X5_400MOutputDevice::CheckType(int ch, IICheckType type)
{
    bool value = (type==ctIncrementing) ? true : false;

	if (ch)
        AnalogControlReg.Ch1Incrementing.Value(value);
    else
        AnalogControlReg.Ch0Incrementing.Value(value);
}

//---------------------------------------------------------------------------
//  X5_400MOutputDevice::CheckFailed()
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::CheckFailed(int ch) const
{
    if (ch)
        return AnalogStatusReg.Ch1Status.Value() ? true : false;
    else
        return AnalogStatusReg.Ch0Status.Value() ? true : false;
}

//---------------------------------------------------------------------------
// X5_400MOutputDevice::Within() -
//---------------------------------------------------------------------------

bool X5_400MOutputDevice::Within(double value, double lower, double upper) const
{
    return (value > lower) && (value < upper);
}


//=============================================================================
//  CLASS X5_400MClock  --  PLL timebase control
//=============================================================================

const double  kVcxoPowerOnRate = 400.0;   // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;    //

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

X5_400MClock::X5_400MClock(X5_400MDeviceMap * owner,
	AddressingSpace & space, int reg)
	: Owner(owner), HasSi570(false),
	  FDacDivider(1), FAutoDacDivider(true), ClockReg(space, reg),
	  VcxoI2CReg( space, mmVcoI2C),
	  VcxoIntf(VcxoI2CReg),
	  Vcxo_I2C(VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
	  Vcxo(new Si570(Vcxo_I2C, Si570::tcA, kVcxoPowerOnRate))
{
}

//---------------------------------------------------------------------------
//  Destructor
//---------------------------------------------------------------------------

X5_400MClock::~X5_400MClock()
{
	delete Vcxo;
	Vcxo = 0;
}

//---------------------------------------------------------------------------
//  X5_400MClock::Open() --
//---------------------------------------------------------------------------

void X5_400MClock::Open()
{
    // Has SI570 if revF or later
    HasSi570 = Owner->PciLogicPcb() >= 5;

	if (!HasSi570)
		return;

	Vcxo->Init();
}

//---------------------------------------------------------------------------
//  X5_400MClock::Frequency() -- Change output frequency
//---------------------------------------------------------------------------

void X5_400MClock::Frequency(double value)
{
	if ((Source() == csInternal) && HasSi570)
        Vcxo->Frequency(value);

	FFrequency = value;
}

//---------------------------------------------------------------------------
//  X5_400MClock::Frequency() -- Query output frequency
//---------------------------------------------------------------------------

double X5_400MClock::Frequency() const
{
	return FFrequency;
}

//---------------------------------------------------------------------------
//  X5_400MClock::FrequencyActual() -- Query actual output frequency
//---------------------------------------------------------------------------

double X5_400MClock::FrequencyActual() const
{
	if ((Source() == csInternal) && HasSi570)
		return Vcxo->FrequencyActual();

	return FFrequency;
}

//---------------------------------------------------------------------------
//  X5_400MClock::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

X5_400MClock::IIClockSource X5_400MClock::Source() const
{
	return ClockReg.Mode.Value() ? csInternal : csExternal;
}

//---------------------------------------------------------------------------
//  X5_400MClock::Source() -- Select clock source
//---------------------------------------------------------------------------

void X5_400MClock::Source(IIClockSource src)
{
	ClockReg.Mode.Value(src == csInternal ? true : false);
}

//---------------------------------------------------------------------------
//  X5_400MClock::Reapply() -- Reapply all clock settings
//---------------------------------------------------------------------------

void X5_400MClock::Reapply()
{
	Source(Source());
	DacDivider(DacDivider());
	Frequency(Frequency());
}

//---------------------------------------------------------------------------
//  X5_400MClock::Locked() -- Return PLL lock status
//---------------------------------------------------------------------------

bool X5_400MClock::Locked() const
{
	return true;
}

//---------------------------------------------------------------------------
//  X5_400MClock::DacDivider() -- Set D/A clock divider
//---------------------------------------------------------------------------

void X5_400MClock::DacDivider(int divisor)
{
	FDacDivider = std::max<int>(std::min<int>(divisor, 4), 1);
	if (FDacDivider == 3)
		FDacDivider = 4;

	switch (divisor)
		{
		default:
		case 1:     ClockReg.Divider.Value(0);      break;
		case 2:     ClockReg.Divider.Value(2);      break;
		case 4:     ClockReg.Divider.Value(3);      break;
		}
}

//---------------------------------------------------------------------------
//  X5_400MClock::DacDivider() -- Query D/A clock divider
//---------------------------------------------------------------------------

int X5_400MClock::DacDivider() const
{
	return FDacDivider;
}

//===========================================================================

static X5IoDeviceRegisters X5_400MInputDeviceRegisters =
	{ mmI2c, 0,  mmInGainCoef, mmInOffsetCoef, mmInChannelEnable,
	  mmInTriggerControl, mmInDecimation, mmInPacketHeader, mmRxPacketControl
	};

//===========================================================================

static X5IoDeviceRegisters X5_400MOutputDeviceRegisters =
	{ mmI2c, 0,  mmOutGainCoef, mmOutOffsetCoef, mmOutChannelEnable,
	  mmOutTriggerControl, mmOutDecimation, mmOutPacketHeader, 0
	};


//===========================================================================
//  CLASS X5_400MDeviceMap  -- Register & low level support for SD module
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X5_400MDeviceMap::X5_400MDeviceMap(X5_400M * owner)
    :  IUsesStreamDeviceEventCallbacks(owner),
	   FInput (new X5_400MInputDevice (this, X5_400MInputDeviceRegisters)),
	   FOutput(new X5_400MOutputDevice(this, X5_400MOutputDeviceRegisters)),
       Owner(owner), Clock(this, LogicMemory, mmClockSource),
	   DataHighReg(LogicMemory, mmDioExtraData),
	   TriggerDelayReg(LogicMemory, mmTriggerDelay)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_400MDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_400MDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_400MDeviceMap::OnAfterStreamStartHandler );
	PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_400MDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_400MDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X5_400MDeviceMap::~X5_400MDeviceMap()
{
	delete FInput;
	FInput = 0;

	delete FOutput;
	FOutput = 0;
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X5_400MDeviceMap::OpenHardware()
{
    X5DeviceMap::OpenHardware();

    // Configure A/Ds
    for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
        {
        FInput->PacketSize(0x4000);
        FInput->PacketId(0x0);
        FInput->Gain(ch, 1.0);
        FInput->Offset(ch, 0.0);
        Input().Unframed();
        }
    // Configure D/As
    for (int ch = 0; ch < Output().Info().Channels().Channels(); ++ch)
		{
        FOutput->PacketSize(0x4000);
		FOutput->PacketId(0x2);
		FOutput->Gain(ch, 1.0);
		FOutput->Offset(ch, 0.0);
		Output().Unframed();
		}

	Clock.Open();
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X5_400MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
	// Family-common init
	X5DeviceMap::ConfigureHardware();

    // Re-apply all clock settings
    Clock.Reapply();

	// Reapply Hardware Settings for  X5 IO Devices
	FInput->ConfigureHardware();
	FOutput->ConfigureHardware();

	//
	//  Apply Gain/Offset Coefficients
	if (FInput->Calibrated())
		FInput->ApplyRomCoefficients();
	else
		FInput->ApplyDefaultCoefficients();


	if (FOutput->Calibrated())
		FOutput->ApplyRomCoefficients();
	else
		FOutput->ApplyDefaultCoefficients();

	//
	//  Apply input settings
    int adcs = 0;
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
        bool enable = FInput->Info().Channels().Enabled(ch);
        adcs += enable ? 1 : 0;
		FInput->ChannelEnabled(ch, enable);
        FInput->PacketId(FInput->PacketId());
		FInput->PacketSize(FInput->PacketSize());
		}

	//
	//  Apply output settings
    int dacs = 0;
	for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
		{
        bool enable = FOutput->Info().Channels().Enabled(ch);
        dacs += enable ? 1 : 0;
		FOutput->ChannelEnabled(ch, enable);
        FOutput->PacketId(FOutput->PacketId());
		FOutput->PacketSize(FOutput->PacketSize());
		}

    // Work-around for DAC trigger dependency on ADC forwarded clock
    if (dacs && !adcs)
		FInput->ChannelEnabled(0, true);

	// Configure triggers
	FInput->TriggerReg.Apply();
	FOutput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
	OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_400MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    UserFpgaControlReg.InGo.Set();
    UserFpgaControlReg.OutGo.Set();
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_400MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_400MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
	OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_400MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.OutGo.Clear();	
    UserFpgaControlReg.Apply();

	Trigger(tInputOutput, false);

	//  Forward notify to baseboard handler
	OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X5_400MInputDevice & X5_400MDeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

X5_400MOutputDevice & X5_400MDeviceMap::Output()
{
	return *FOutput;
}

//---------------------------------------------------------------------------
//  X5_400MDeviceMap::TriggerDelay() -- Set trigger idelay
//---------------------------------------------------------------------------

void X5_400MDeviceMap::TriggerDelay(unsigned int pS)
{

	TriggerDelayReg.Reset.Value(true);
	TriggerDelayReg.Apply();

	TriggerDelayReg.Reset.Value(false);
	TriggerDelayReg.Taps.Value(pS/72);
	TriggerDelayReg.Apply();
}

} // namespace Innovative


