// X5_400MDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_400MDeviceMap_MbH
#define X5_400MDeviceMap_MbH

#include <X5_400M_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <Si570_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X5_400MDeviceMap;

//==============================================================================
//  CLASS X5_400MClock -- GSPS clock control
//==============================================================================

class X5_400MClock : public X5ClockIntf
{
public:
    X5_400MClock(X5_400MDeviceMap * owner, AddressingSpace & space, int reg);
	~X5_400MClock();
    
	//
    //  Interface Functions
	// Methods
	virtual void 			Open();
	virtual void 			Reapply();
	virtual bool            Locked() const;
	virtual double			FrequencyActual() const;

	// Properties
	virtual void    		Frequency(double value);
	virtual double			Frequency() const;

	virtual IIClockSource   Source() const;
	virtual void            Source(IIClockSource src);

	virtual void 			DacDivider(int divisor);
    virtual int             DacDivider() const;

protected:
    // Data
    X5_400MDeviceMap *      Owner;
    bool                    HasSi570;

    // Fields
    double                  FFrequency;
    double                  FExternal;
    int                     FDacDivider;
	bool                    FAutoDacDivider;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ClockRegister  -- External vs. Internal clock source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ClockRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit         Mode;
        RegisterBitGroup    Divider;

        ClockRegister(AddressingSpace &space, int offset)
            : inherited("X5DeviceMap::ClockRegister", space, offset),
              Mode(*this, 0), Divider(*this, 1, 2)
        {
            Mode.Value(true);
            Divider.Value(0);
        }
	};

    //
    //  Class VcxoI2CRegister
    class VcxoI2CRegister : public Register
    {
		typedef Register inherited;

	public:
		RegisterBit  Clock;
		RegisterBit  DataOut;
		RegisterBit  DataIn;

	public:

		VcxoI2CRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Clock(*this, 0),
			   DataOut(*this, 1),
			   DataIn(*this, 2)
			{
			}
	};

	//
	// Class VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	class VcxoI2C_Impl : public I2CDeviceIntf
	{
	public:
		VcxoI2C_Impl(VcxoI2CRegister & reg)
			:  Reg(reg)	{ }
		virtual ~VcxoI2C_Impl() {}
		// Methods
		virtual void    SetClock(int state)
			{  Reg.Clock.Value(state ? true : false);  }
		virtual int     GetClock() const
			{  return Reg.Clock.Value();  }
		virtual void    SetData(int state)
			{  Reg.DataOut.Value(state ? true : false);  }
		virtual int     GetData() const
			{  return Reg.DataIn.Value();  }
			
	private:
		VcxoI2CRegister &  Reg;
	};

	ClockRegister       ClockReg;
	VcxoI2CRegister     VcxoI2CReg;
	VcxoI2C_Impl        VcxoIntf;
	I2CDeviceN          Vcxo_I2C;
	Si570*				Vcxo;
};

//===========================================================================
//  CLASS X5_400MInputDevice  -- Register & low level support for X5 ADCs
//===========================================================================

class X5_400MInputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_400MDeviceMap;

public:
    virtual void    TestEnable(bool state);
	virtual bool    TestEnable() const;
	void			ScaleTimesFour(bool enable);
	bool            ScaleTimesFour();
	
protected:
	X5_400MInputDevice(X5DeviceMap * owner, X5IoDeviceRegisters & reg);
};

//===========================================================================
//  CLASS X5_400MOutputDevice  -- Register & low level support for X5 DACs
//===========================================================================

class X5_400MOutputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_400MDeviceMap;

public:
	enum IICheckType{ctDecrementing, ctIncrementing};
	enum IITestMode{tmRamp = 0x0, tmSine};

	// Properties
	bool    		CheckEnable(int ch) const;
	void    		CheckEnable(int ch, bool state);

	IICheckType     CheckType(int ch) const;
	void    		CheckType(int ch, IICheckType type);

	bool            CheckFailed(int ch) const;

	bool    		TestEnable() const;
	void    		TestEnable(bool state);

	unsigned int 	TestMode() const;
	void 			TestMode(unsigned int mode);

	float           TestFrequency(int ch) const;
	void 			TestFrequency(int ch, float value);

	unsigned int    Interpolate() const;
	void            Interpolate(unsigned int value);

	void 			SincFilterEnabled(bool value);
	bool 			SincFilterEnabled();

	void            Spi(int addr, unsigned char value);
	unsigned char   Spi(int addr);

	float           TransitionFrequency() const;
	void 			TransitionFrequency(float value);

	virtual void            ConfigureHardware();

protected:
	X5_400MOutputDevice(X5DeviceMap * owner, X5IoDeviceRegisters & reg);

	class AnalogControlRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBit     Ch0CheckEnable;
		RegisterBit     Ch1CheckEnable;
		RegisterBit     Ch0Incrementing;
		RegisterBit     Ch1Incrementing;
		RegisterBit     Stack;

		AnalogControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Ch0CheckEnable(*this, 0), Ch1CheckEnable(*this, 1),
              Ch0Incrementing(*this, 2), Ch1Incrementing(*this, 3),
			  Stack(*this, 29)
		{
		}
	};

	class AnalogStatusRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit     Ch0Status;
		RegisterBit     Ch1Status;

		AnalogStatusRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
              Ch0Status(*this, 30), Ch1Status(*this, 31)
		{
		}
	};

	class TestPhaseRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBitGroup    PhaseIncrement;

		TestPhaseRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), PhaseIncrement(*this, 0, 23)
		{
		}
	};

	class TestControlRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         Enable;
		RegisterBitGroup    Mode;

		TestControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Enable(*this, 0),
			  Mode(*this, 1, 2)
		{
		}
	};

	class SpiDataRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup    Data;
		RegisterBitGroup    Address;
		RegisterBit         ReadStrobe;
		RegisterBit         WriteStrobe;

		SpiDataRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Data(*this, 0, 8),
			  Address(*this, 8, 5), ReadStrobe(*this, 13),
              WriteStrobe(*this, 14)
		{
		}
	};

	class SpiStatusRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         Ready;

		SpiStatusRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Ready(*this, 23)
		{
		}
	};

	// Registers
    AnalogControlRegister               AnalogControlReg;
    AnalogStatusRegister                AnalogStatusReg;
	RegisterArray<TestPhaseRegister>    TestPhaseReg;
	TestControlRegister                 TestControlReg;
	SpiStatusRegister                   SpiStatusReg;
	SpiDataRegister                     SpiReg;
    StdSpiDevice                        Dac0Spi;

    Trace                               Debug;

	// Fields
	IITestMode      FTestMode;
	bool            FEnabled;
    unsigned int    FInterpolate;
	float           FTestFrequency[2];
	bool			FSincFilterEnabled;
	float           FTransitionFrequency;

	// Methods
	bool            OutputStack() const;
	void            OutputStack(bool value);

	void            SpiReady();
	void            SpiNotReady();
	void            OldSpi(int addr, unsigned char value);
	unsigned char   OldSpi(int addr);
	bool 			InvClock() const;
	bool 			Within(double value, double lower, double upper) const;
};

//===========================================================================
//  CLASS X5_400MDeviceMap  -- Register & low level support for SD module
//===========================================================================
class X5_400M;

class X5_400MDeviceMap :  public X5DeviceMap,
						  public IUsesStreamDeviceEventCallbacks
{
	friend class X5_400M;
	friend class X5_400MOutputDevice;
	friend class X5_400MInputDevice;
	friend class X5_400MClock;

public:
	//
	//  Member Functions
	X5_400MDeviceMap(X5_400M * owner);
	virtual ~X5_400MDeviceMap();

	X5_400MInputDevice  &    Input();
	X5_400MOutputDevice &    Output();

    IBitManipulator &   DioPortDataHigh()
		{  return DataHighReg;  }

	void TriggerDelay(unsigned int uS);

public: // pseudo-protected
    virtual void  OpenHardware();

protected:
    // Fields
    X5_400MInputDevice  *   FInput;
    X5_400MOutputDevice *   FOutput;

    // Data
    X5_400M *           Owner;
    X5_400MClock        Clock;
    Trace               Debug;

    class DataHighRegister : public IBitManipulator
    {
    public:
        DataHighRegister(AddressingSpace & space, unsigned int data_high_addr)
            : DataReg(space, data_high_addr)
            {  }

    protected:
        // Registers.
    	Register				DataReg;

        // Overrides
        virtual unsigned int  ReadV() const
            {  return DataReg.Value();  }
        virtual void          WriteV(unsigned int value)
            {  DataReg.Value(value);  }
    };

	class TriggerDelayRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup    Taps;
		RegisterBit         Increment;
		RegisterBit         Reset;

		TriggerDelayRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
			  Taps(*this, 0, 6), Increment(*this, 8), Reset(*this, 9)
		{
		}
	};

	// Registers
	DataHighRegister        DataHighReg;
	TriggerDelayRegister	TriggerDelayReg;

    // Methods
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
