//---------------------------------------------------------------------------
// X5_400M_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X5_400M_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5_400M  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X5_400M::X5_400M()
    :   X5Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X5_400M::~X5_400M()
{
    Close();
}

//---------------------------------------------------------------------------
//  X5_400M::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X5_400M::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X5_400M::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X5_400M::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X5_400M::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X5_400M::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X5_400M::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X5_400M::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X5_400M::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X5_400M::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X5_400M::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X5_400M::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X5_400M::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X5_400M::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X5_400M::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X5_400M::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X5_400M::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X5_400M::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X5_400M::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X5_400M::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X5_400M::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X5_400M::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_400M::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X5_400M::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_400M::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X5_400M::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X5_400M::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X5_400M::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X5_400M::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X5_400M::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X5_400M::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X5_400M::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X5_400M::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & X5_400M::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  X5_400M::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X5_400M::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X5_400M::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X5_400M::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X5_400M::PciLoopbackMode() --
//------------------------------------------------------------------------------

X5_400M::LoopbackMode X5_400M::PciLoopbackMode()
{
    X5_400M::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X5_400M::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5_400M::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X5_400M::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5_400M::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X5_400M::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X5_400MInputDevice & X5_400M::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X5_400M::Output() --  Return reference to input object
//---------------------------------------------------------------------------

X5_400MOutputDevice & X5_400M::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  X5_400M::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X5_400M::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_400M::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X5_400M::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_400M::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X5_400M::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X5DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X5_400M::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX5Alerts & X5_400M::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X5_400M::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X5_400M::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_400M::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X5_400M::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X5_400M::Clock() --   Access to clock control object
//---------------------------------------------------------------------------

X5_400MClock & X5_400M::Clock()
{
    return Device.Clock;
}

//---------------------------------------------------------------------------
//  X5_400M::BusMasterSize() --
//---------------------------------------------------------------------------

void X5_400M::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_400M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_400M::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_400M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_400M::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_400M::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_400M::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_400M::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_400M::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_400M::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X5_400M::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate(" 0 constant Dac" );

    engine.Evaluate(": ?X5-400M "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Dac \" "
                    "    cr  "
                    "    cr .\" ---- X5-400M Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X5-400M  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("X5-400M-specific wordset loaded \n" );
    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X5_400M::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X5_400M::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X5_400M::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X5_400M::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X5_400M::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X5_400M::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X5_400M::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X5_400M::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X5_400M::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X5_400M::Do_DFetch(int dev, int addr)
{
	const int kDac = 0;

    switch (dev)
        {
        default:
            return 0;
        case kDac:
            return Output().Spi(addr);
        }
}

//---------------------------------------------------------------------------
//  X5_400M::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X5_400M::Do_DStore(int dev, int addr, int value)
{
    const int kDac = 0;

    switch (dev)
        {
        default:
            break;
        case kDac:
            Output().Spi(addr, value);
			break;
        }
}

//---------------------------------------------------------------------------
//  X5_400M::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X5_400M::Do_DFreq(int /*dev*/, float /*freq*/)
{
    // no frequency devices
    /*
	switch (dev)
        {
        default:
            break;
        }
	*/
}

//---------------------------------------------------------------------------
//  X5_400M::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X5_400M::Do_DFreqActual(int /*dev*/)
{
    // no frequency devices
	return 0.0f;
	/*
    switch (dev)
        {
        default:
            return 0.0f;
        }
	*/

}
#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
