// X5_COMDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5_COM_Mb.h>
#include <X5_COMDeviceMap_Mb.h>
#include <X5MemoryMap_Mb.h>
#include <X5_COMMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>
#include <Magic_Mb.h>
#include <sstream>
#include <Exception_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X5_MemoryMap;
using namespace X5_COMMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxInDevices      = 1;
const int    InputWidth_Bits   = 32;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 1000.0e6;
const float  InputMinRate      = 0.0;
const int    InputGangSize     = 1;

const int    MaxOutDevices     = 1;
const int    OutputWidth_Bits  = 32;
const int    OutputMuxCount    = 0;
const bool   OutputIsUnipolar  = false;
const int    OutputClockFactor = 1;
const float  OutputMaxRate     = 1000.0e6;
const float  OutputMinRate     = 0.0;
const int    OutputGangSize    = 1;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input and Output Coefficient section
//       2channels x 2 values + 1 calibration flag + 1 spare
const int romX5_COM_InputCoeffOffset   = 0;
const int romX5_COM_InChannels         = 1;
const int romX5_COM_InputCoeffSize     = romX5_COM_InChannels * 2 + 1 + 1;
const int romX5_COM_OutputCoeffOffset  = romX5_COM_InputCoeffSize;
const int romX5_COM_OutChannels        = 1;
const int romX5_COM_OutputCoeffSize    = romX5_COM_OutChannels * 2 + 1 + 1;
const int romX5_COM_CoeffSize          = romX5_COM_InputCoeffSize + romX5_COM_OutputCoeffSize;

//---------------------------------------------------------
//  I2C Codes

const int OscillatorI2C_SubAddr = 0xAA;
const int TransI2C_Subaddr_A = 0xA0;
const int TransI2C_Subaddr_B = 0xA2;

//===========================================================================
// class X5_COMInputInfoIntf
//===========================================================================

class X5_COMInputInfoIntf : public IoInfoIntf
{
public:
    X5_COMInputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxInDevices, InputWidth_Bits, InputMuxCount, InputIsUnipolar)
        {  FSpan.push_back(SpanInfo(1000, -1000));  }
    virtual ~X5_COMInputInfoIntf()  {  }

    virtual UnpairedChannelInfo &       Channels()              {  return FChannels;}
    virtual float                       MaxRate() const         {  return InputMaxRate;  }
    virtual float                       MinRate() const         {  return InputMinRate;  }
    virtual const SpanArray &           Span() const            {  return FSpan;  }
    virtual int                         GangSize() const        {  return InputGangSize;  }
    virtual bool                        IsStackable() const     {  return false;  }
	virtual bool                        IsInverted() const      {  return true;  }
    virtual bool                        IsMuxed() const         {  return false;  }
    virtual bool                        MuxDepth() const        {  return InputMuxCount;  }
    virtual int                         BufferSize() const      {  return 65536;  }
    virtual int                         ClockFactor() const     {  return InputClockFactor;  }
    virtual float                       ReferenceClock() const  {  return 0.0;  }
	virtual int               	        SamplesPerFrame() const {  return 256;  }

private:
    UnpairedChannelInfo         FChannels;
    SpanArray                   FSpan;
};

//===========================================================================
// class X5_COMOutputInfoIntf
//===========================================================================

class X5_COMOutputInfoIntf : public IoInfoIntf
{
public:
	X5_COMOutputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxOutDevices, OutputWidth_Bits, OutputMuxCount, OutputIsUnipolar)
		{  FSpan.push_back(SpanInfo(1000, -1000));  }
	virtual ~X5_COMOutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &       Channels()              {  return FChannels; }
	virtual float                       MaxRate() const         {  return OutputMaxRate; }
	virtual float                       MinRate() const         {  return OutputMinRate; }
	virtual const SpanArray &           Span() const            {  return FSpan;  }
	virtual int                         GangSize() const        {  return OutputGangSize; }
	virtual bool                        IsStackable() const     {  return true;  }
	virtual bool                        IsInverted() const      {  return true;  }
	virtual bool                        IsMuxed() const         {  return false;  }
	virtual bool                        MuxDepth() const        {  return OutputMuxCount;  }
	virtual int                         BufferSize() const      {  return 65536;  }
	virtual int                         ClockFactor() const     {  return OutputClockFactor;  }
	virtual float                       ReferenceClock() const  {  return 0.0;  }
	virtual int               	        SamplesPerFrame() const {  return 256;  }

private:
	UnpairedChannelInfo         FChannels;
	SpanArray                   FSpan;
};

//==============================================================================
//  CLASS X5_Rio_Registers -- X5-COM RocketIo Register Definitions
//==============================================================================
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS X5_Rio_Registers::TranceiverRegister  -- RIO tranceiver register
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::Enabled()  -- Enabled
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::Enabled(int dev, bool state)
{
    if (dev==0)
        {
        Enable0.Value(state);
        }
    else
        {
        Enable1.Value(state);
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::Enabled()  -- Read Enabled
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::Enabled(int dev)
{
    if (dev==0)
        {
        return Enable0.Value();
        }
    else
        {
        return Enable1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::HighSpeed()  -- Read Highspeed
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::HighSpeed(int dev)
{
    if (dev==0)
        {
        return HighSpeed0.Value();
        }
    else
        {
        return HighSpeed1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::HighSpeed()  -- Write Highspeed
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::HighSpeed(int dev, bool state)
{
    if (dev==0)
        {
        HighSpeed0.Value(state);
        }
    else
        {
        HighSpeed1.Value(state);
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::SerialClock()  -- Write Clock
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::SerialClock(int dev, bool state)
{
    if (dev==0)
        {
        SerialClock0.Value(state);
        }
    else
        {
        SerialClock1.Value(state);
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::SerialClock()  -- Read SerialClock
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::SerialClock(int dev)
{
    if (dev==0)
        {
        return SerialClock0.Value();
        }
    else
        {
        return SerialClock1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::SerialDataOut()  -- Read SerialDataOut
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::SerialDataOut(int dev)
{
    if (dev==0)
        {
        return SerialDataOut0.Value();
        }
    else
        {
        return SerialDataOut1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::SerialDataOut()  -- Write SerialDataOut
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::SerialDataOut(int dev, bool state)
{
    if (dev==0)
        {
        SerialDataOut0.Value(state);
        }
    else
        {
        SerialDataOut1.Value(state);
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::SerialDataIn()  -- Read SerialDataIn
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::SerialDataIn(int dev)
{
    if (dev==0)
        {
        return SerialDataIn0.Value();
        }
    else
        {
        return SerialDataIn1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::SerialDataIn()  -- Write SerialDataIn
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::SerialDataIn(int dev, bool state)
{
    if (dev==0)
        {
        SerialDataIn0.Value(state);
        }
    else
        {
        SerialDataIn1.Value(state);
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::NoSignal()  -- Read NoSignal
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::NoSignal(int dev)
{
    if (dev==0)
        {
        return NoSignal0.Value();
        }
    else
        {
        return NoSignal1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::Fault()  -- Read Fault
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::Fault(int dev)
{
    if (dev==0)
        {
        return Fault0.Value();
        }
    else
        {
        return Fault1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::NoTranceiver()  -- Read NoTranceiver
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::NoTranceiver(int dev)
{
    if (dev==0)
        {
        return NoTranceiver0.Value();
        }
    else
        {
        return NoTranceiver1.Value();
        }
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::OscillatorClock()  -- Read Clock
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::OscillatorClock()
{
    return OscillatorClockBit.Value();
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::OscillatorClock()  -- Write Clock
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::OscillatorClock(bool state)
{
    OscillatorClockBit.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::OscillatorDataOut()  -- Read OscillatorDataOut
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::OscillatorDataOut()
{
    return OscillatorDataOutBit.Value();
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::OscillatorDataOut()  -- Write OscillatorDataOut
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::OscillatorDataOut(bool state)
{
    OscillatorDataOutBit.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::OscillatorDataIn()  -- Read OscillatorDataIn
//---------------------------------------------------------------------------

bool  X5_Rio_Registers::TranceiverRegister::OscillatorDataIn()
{
    return OscillatorDataInBit.Value();
}

//---------------------------------------------------------------------------
//  X5_Rio_Registers::TranceiverRegister::OscillatorDataIn()  -- Write OscillatorDataIn
//---------------------------------------------------------------------------

void  X5_Rio_Registers::TranceiverRegister::OscillatorDataIn(bool state)
{
    OscillatorDataInBit.Value(state);
}

//===========================================================================
//  CLASS X5_Sio -- X5-COM base Rocket I/O control
//===========================================================================
//---------------------------------------------------------------------------
//  X5_Sio::Reset()  -- Reset
//---------------------------------------------------------------------------

void  X5_Sio::Reset()
{
    GtpReset(true);
    ClearErrorFlags();
    GtpReset(false);
}

//---------------------------------------------------------------------------
//  X5_Sio::LinkEnable()  -- Enable one or both link directions
//---------------------------------------------------------------------------

void  X5_Sio::LinkEnable(X5_Sio::IIDirection dir, bool state)
{
    switch (dir)
        {
        case tInput:
            RxEnabled(state);
            if (state)
                PowerDown(false);
            break;
        case tOutput:
            TxEnabled(state);
            if (state)
                PowerDown(false);
            break;
        case tInputOutput:
            LinkEnable(tInput, state);
            LinkEnable(tOutput, state);
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_Sio::WaitForDcmLock()  -- Poll for DCM Lock (20 ms timeout)
//---------------------------------------------------------------------------

void  X5_Sio::WaitForDcmLock()
{
    for (int i=0; i<20; i++)
        {
        if (DcmLocked()==true)
            break;
        Sleep(1);
        }
}

//---------------------------------------------------------------------------
//  X5_Sio::WaitForChannelUp()  -- Poll for Channel up (no timeout yet)
//---------------------------------------------------------------------------

void  X5_Sio::WaitForChannelUp()
{
    while(1==1)
        {
        if (ChannelUp()==true)
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_Sio::ClearErrorFlags()  -- Clear flags
//---------------------------------------------------------------------------

void  X5_Sio::ClearErrorFlags()
{
    ControlReg.ClearErrorFlags.Value(1);
    ControlReg.ClearErrorFlags.Value(0);
}

//---------------------------------------------------------------------------
//  X5_Sio::PowerDown()  -- Read Power down state
//---------------------------------------------------------------------------

bool  X5_Sio::PowerDown()
{
    return ControlReg.PowerDown.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::PowerDown()  -- Set Power down state
//---------------------------------------------------------------------------

void  X5_Sio::PowerDown(bool state)
{
    GtpReset(state);
    ControlReg.PowerDown.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Sio::GtpReset()  -- Read GtpReset down state
//---------------------------------------------------------------------------

bool  X5_Sio::GtpReset()
{
    return ControlReg.Reset.Value()==0;   // negative logic
}

//---------------------------------------------------------------------------
//  X5_Sio::GtpReset()  -- Set GtpReset down state
//---------------------------------------------------------------------------

void  X5_Sio::GtpReset(bool state)
{
    ControlReg.Reset.Value(state==0);    // negative logic
}

//---------------------------------------------------------------------------
//  X5_Sio::Run()  -- Read Run active state
//---------------------------------------------------------------------------

bool  X5_Sio::Run()
{
    return ControlReg.Run.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::Run()  -- Set Run active state
//---------------------------------------------------------------------------

void  X5_Sio::Run(bool state)
{
    ControlReg.Run.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Sio::LoopbackMode()  -- Read Tx enabled state
//---------------------------------------------------------------------------

int  X5_Sio::LoopbackMode()
{
    return ControlReg.Loopback.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::LoopbackMode()  -- Set Tx enabled state
//---------------------------------------------------------------------------

void  X5_Sio::LoopbackMode(int state)
{
    FLoopbackMode = state;
    ControlReg.Loopback.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Sio::TxEnabled()  -- Read Tx enabled state
//---------------------------------------------------------------------------

bool  X5_Sio::TxEnabled()
{
    return ControlReg.TxEnable.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::TxEnabled()  -- Set Tx enabled state
//---------------------------------------------------------------------------

void  X5_Sio::TxEnabled(bool state)
{
    ControlReg.TxEnable.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Sio::RxEnabled()  -- Read Rx enabled state
//---------------------------------------------------------------------------

bool  X5_Sio::RxEnabled()
{
    return ControlReg.RxEnable.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::RxEnabled()  -- Set Rx enabled state
//---------------------------------------------------------------------------

void  X5_Sio::RxEnabled(bool state)
{
    ControlReg.RxEnable.Value(state);
}

//---------------------------------------------------------------------------
//  X5_Sio::FrameError()  -- Read error state bit
//---------------------------------------------------------------------------

bool  X5_Sio::FrameError()
{
    return ControlReg.FrameError.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::SoftError()  -- Read error state bit
//---------------------------------------------------------------------------

bool  X5_Sio::SoftError()
{
    return ControlReg.SoftError.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::HardError()  -- Read error state bit
//---------------------------------------------------------------------------

bool  X5_Sio::HardError()
{
    return ControlReg.HardError.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::LaneUp()  -- Read Lane up field
//---------------------------------------------------------------------------

int  X5_Sio::LaneUp()
{
    return ControlReg.LaneUp.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::ChannelUp()  -- Read channel up bit
//---------------------------------------------------------------------------

bool  X5_Sio::ChannelUp()
{
    return ControlReg.ChannelUp.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::DcmLocked()  -- Read Dcm lock bit
//---------------------------------------------------------------------------

bool  X5_Sio::DcmLocked()
{
    return ControlReg.DcmLocked.Value();
}

//---------------------------------------------------------------------------
//  X5_Sio::WriteCmd() -- write over command channel
//---------------------------------------------------------------------------

void  X5_Sio::WriteCmd(int addr, int data)
{
    //  Wait for outbound channel to come available
    while (CommandWriteRb.Ready.Value() == 0)
        ;
    //  Command and Argument
    CommandWriteReg.Address.Value(addr);
    CommandWriteReg.Data.Value(data);
    CommandWriteReg.Write.Value(true);
    //  Apply
    CommandWriteReg.Apply();

}

//---------------------------------------------------------------------------
//  X5_Sio::ReadCmd() -- Write command, and Read Reply data
//---------------------------------------------------------------------------

int  X5_Sio::ReadCmd(int addr, int data)
{
    //  Wait for outbound channel to come available
    while (CommandWriteRb.Ready.Value() == 0)
        ;
    //  Command and Argument
    CommandWriteReg.Address.Value(addr);
    CommandWriteReg.Data.Value(data);
    CommandWriteReg.Write.Value(false);
    //  Apply
    CommandWriteReg.Apply();
    //
    //  Wait for data available
    while (CommandReadRb.Ready.Value() == 0)
        ;
    //  Read data
    return CommandReadRb.Data.Value();

}

//---------------------------------------------------------------------------
//  X5_Sio::Reapply() -- Reapply Sio State
//---------------------------------------------------------------------------

void  X5_Sio::Reapply()
{
    LoopbackMode(FLoopbackMode);
}

//==============================================================================
//  CLASS X5_Sfp -- X5-COM SFP Rocket I/O control
//==============================================================================
//---------------------------------------------------------------------------
//  Constructor for class X5_Sfp
//---------------------------------------------------------------------------

X5_Sfp::X5_Sfp(AddressingSpace & space, int control_offset, int write_offset,
                                        int read_offset, int trans_offset)
    :  X5_Sio(space, control_offset, write_offset, read_offset),
       TranceiverReg(space, trans_offset),
       OscillatorIntf(TranceiverReg),
       Trans0Intf(Trans[0]),
       Trans1Intf(Trans[1]),
       Oscillator_I2C(OscillatorIntf, I2CDevice::SubAddress, OscillatorI2C_SubAddr),
       Trans0_I2C_A(Trans0Intf, I2CDevice::SubAddress, TransI2C_Subaddr_A),
       Trans0_I2C_B(Trans0Intf, I2CDevice::SubAddress, TransI2C_Subaddr_B),
       Trans1_I2C_A(Trans1Intf, I2CDevice::SubAddress, TransI2C_Subaddr_A),
       Trans1_I2C_B(Trans1Intf, I2CDevice::SubAddress, TransI2C_Subaddr_B),
       Oscillator(Oscillator_I2C, Si570::tcA)
{
    Trans[0].Init(&TranceiverReg, 0);   Trans[1].Init(&TranceiverReg, 1);
}

//------------------------------------------------------------------------------
//  X5_Sfp::TransI2C() -- access I2C for transcievers
//------------------------------------------------------------------------------

I2CDevice &  X5_Sfp::TransI2C(int trans, int dev)
{
    if (trans==0 && dev==0)
        return Trans0_I2C_A;
    else if (trans==0 && dev==1)
        return Trans0_I2C_B;
    else if (trans==0 && dev==1)
        return Trans1_I2C_A;
    else if (trans==0 && dev==1)
        return Trans1_I2C_B;
    else
        return Trans0_I2C_A;    // default to the first one
}

//===========================================================================
//  CLASS X5_COM_Io  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X5_COM_Io 
//---------------------------------------------------------------------------

X5_COM_Io::X5_COM_Io(X5_COMDeviceMap * owner)
	: Owner(owner),
      Sfp_01(Owner->LogicMemory, mmSfp01GeneralControl, mmSfp01CommandChWrite,
                                 mmSfp01CommandChRead,  mmSfp01Tranceivers),
      Sfp_23(Owner->LogicMemory, mmSfp23GeneralControl, mmSfp23CommandChWrite,
                                 mmSfp23CommandChRead,  mmSfp23Tranceivers),
      Sio_01(Owner->LogicMemory, mmSio01GeneralControl, mmSio01CommandChWrite, mmSio01CommandChRead),
      Sio_23(Owner->LogicMemory, mmSio23GeneralControl, mmSio23CommandChWrite, mmSio23CommandChRead)
{
    //  SFP style images
    Sfp[0] = &Sfp_01;    Sfp[1] = &Sfp_23;
    //  SIO style images
    Sio[0] = &Sio_01;    Sio[1] = &Sio_23;
}

//------------------------------------------------------------------------------
//  X5_COM_Io::Link() -- "Generic" Serial Link Access
//------------------------------------------------------------------------------

X5_Sio * X5_COM_Io::Link(X5_COM_Io::SerialLinkId link)
{
    switch (link)
        {
        case lnkSfp01:
            return Sfp[0];
        case lnkSfp23:
            return Sfp[1];
        case lnkSio01:
            return Sio[0];
        case lnkSio23:
            return Sio[1];
        case lnkTotalLinks:
            return 0;
        }
    return 0;   // never gets here
}

//------------------------------------------------------------------------------
//  X5_COM_Io::SfpLink() -- "Generic" Serial SfpLink Access
//------------------------------------------------------------------------------

X5_Sfp * X5_COM_Io::SfpLink(X5_COM_Io::SerialLinkId SfpLink)
{
    switch (SfpLink)
        {
        case lnkSfp01:
            return Sfp[0];
        case lnkSfp23:
            return Sfp[1];
        case lnkSio01:
            return 0;
        case lnkSio23:
            return 0;
        case lnkTotalLinks:
            return 0;
        }
    return 0;   // never gets here
}
//===========================================================================
//  CLASS X5_COMInputDevice  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  X5_COMInputDevice Ctor
//---------------------------------------------------------------------------

X5_COMInputDevice::X5_COMInputDevice(X5_COMDeviceMap * owner,
									 X5IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X5_COMInputInfoIntf,
				PmcIdromSection(owner->Rom(),
								PmcIdrom::waModule,
								romX5_COM_InputCoeffOffset,
								romX5_COM_InputCoeffSize),
				0x10000),	// 'one' value for gain registers
      COM_Owner(owner),
      FLink(X5_COM_Io::lnkSfp01), FTestEnabled(false)
{
}

//---------------------------------------------------------------------------
//  X5_COMInputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void X5_COMInputDevice::ConfigureHardware()
{
    inherited::ConfigureHardware();

    Link()->Reset();
    //  enable hardware and power on
    if (Info().Channels().Enabled(0))
        {
        Link()->LinkEnable(X5_Sio::tInput, true);
        if (SfpLink())
            {
            SfpLink()->Trans[0].Enabled(true);
            SfpLink()->Trans[1].Enabled(true);
            }
        }
    //  wait for DCM Lock
    Link()->WaitForDcmLock();

    TestEnable(FTestEnabled);

}

//---------------------------------------------------------------------------
//  X5_COMInputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_COMInputDevice::TestEnable(bool enable)
{
    FTestEnabled = enable;
	COM_Owner->RxTestEnable(enable);
}

//---------------------------------------------------------------------------
//  X5_COMInputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_COMInputDevice::TestEnable() const
{
	return COM_Owner->RxTestEnable() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_COMInputDevice::TestErrorCount()
//---------------------------------------------------------------------------

int  X5_COMInputDevice::TestErrorCount() const
{
	return COM_Owner->ErrorCount();
}

//---------------------------------------------------------------------------
//  X5_COMInputDevice::Io()
//---------------------------------------------------------------------------

X5_COM_Io &  X5_COMInputDevice::Io()
{
    return COM_Owner->Io();
}

//===========================================================================
//  CLASS X5_COMOutputDevice  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X5_COMOutputDevice
//---------------------------------------------------------------------------

X5_COMOutputDevice::X5_COMOutputDevice(X5_COMDeviceMap * owner,
										 X5IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X5_COMOutputInfoIntf,
				PmcIdromSection(owner->Rom(),
							    PmcIdrom::waModule,
								romX5_COM_OutputCoeffOffset,
								romX5_COM_OutputCoeffSize),
				0x10000),	// 'one' value for gain registers
      COM_Owner(owner),
      FLink(X5_COM_Io::lnkSfp01), FTestEnabled(false)
{
}

//---------------------------------------------------------------------------
//  X5_COMOutputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void X5_COMOutputDevice::ConfigureHardware()
{
    inherited::ConfigureHardware();

    //  Reapply Link Loopback State
    Link()->Reapply();

    Link()->Reset();
    //  enable hardware and power on
    if (Info().Channels().Enabled(0))
        {
        Link()->LinkEnable(X5_Sio::tOutput, true);
        if (SfpLink())
            {
            SfpLink()->Trans[0].Enabled(true);
            SfpLink()->Trans[1].Enabled(true);
            }
        }
    //  wait for DCM Lock
    Link()->WaitForDcmLock();

    TestEnable(FTestEnabled);

}

//---------------------------------------------------------------------------
//  X5_COMOutputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_COMOutputDevice::TestEnable(bool enable)
{
    FTestEnabled = enable;
	COM_Owner->TxTestEnable(enable);
}

//---------------------------------------------------------------------------
//  X5_COMOutputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_COMOutputDevice::TestEnable() const
{
	return COM_Owner->TxTestEnable() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_COMOutputDevice::TestErrorCount()
//---------------------------------------------------------------------------

int  X5_COMOutputDevice::TestErrorCount() const
{
	return COM_Owner->ErrorCount();
}

//---------------------------------------------------------------------------
//  X5_COMOutputDevice::Io()
//---------------------------------------------------------------------------

X5_COM_Io &  X5_COMOutputDevice::Io()
{
    return COM_Owner->Io();
}

//=============================================================================
//  CLASS X5_COMClock  --  PLL timebase control (a NULL clock here)
//=============================================================================
//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

X5_COMClock::X5_COMClock(X5_COMDeviceMap * owner)
    : Owner(owner)
{
}

//---------------------------------------------------------------------------
//  X5_COMClock::Frequency() -- Change output frequency
//---------------------------------------------------------------------------

void X5_COMClock::Frequency(double value)
{
    FFrequency = value;
}

//---------------------------------------------------------------------------
//  X5_COMClock::Frequency() -- Query output frequency
//---------------------------------------------------------------------------

double X5_COMClock::Frequency() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------
//  X5_COMClock::FrequencyActual() -- Query actual output frequency
//---------------------------------------------------------------------------

double X5_COMClock::FrequencyActual() const
{
    return FFrequency;
}

//---------------------------------------------------------------------------
//  X5_COMClock::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

X5_COMClock::IIClockSource X5_COMClock::Source() const
{
    return FSource;
}

//---------------------------------------------------------------------------
//  X5_COMClock::Source() -- Select clock source
//---------------------------------------------------------------------------

void X5_COMClock::Source(IIClockSource src)
{
    FSource = src;
}

//---------------------------------------------------------------------------
//  X5_COMClock::Reapply() -- Reapply all clock settings
//---------------------------------------------------------------------------

void X5_COMClock::Reapply()
{
    Source(Source());
}

//---------------------------------------------------------------------------
//  X5_COMClock::Locked() -- Return PLL lock status
//---------------------------------------------------------------------------

bool X5_COMClock::Locked() const
{
    return true;
}

//===========================================================================

static X5IoDeviceRegisters X5_COMInputDeviceRegisters =
	{ mmI2c, 0,  0 /*mmInGainCoef*/, 0 /*mmInOffsetCoef*/, 0 /*mmInChannelEnable*/,
	  0 /*mmInTriggerControl*/, 0 /*mmInDecimation*/, mmInPacketHeader, mmRxPacketControl
	};

//===========================================================================

static X5IoDeviceRegisters X5_COMOutputDeviceRegisters =
	{ mmI2c, 0, 0 /*mmOutGainCoef*/, 0 /*mmOutOffsetCoef*/, 0 /*mmOutChannelEnable*/,
	  0 /*mmOutTriggerControl*/, 0 /*mmOutDecimation*/, mmOutPacketHeader, 0
	};


//===========================================================================
//  CLASS X5_COMDeviceMap  -- Register & low level support for SD module
//===========================================================================

//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X5_COMDeviceMap::X5_COMDeviceMap(X5_COM * owner)
	:  IUsesStreamDeviceEventCallbacks(owner),
	   FInput (new X5_COMInputDevice (this, X5_COMInputDeviceRegisters)),
	   FOutput(new X5_COMOutputDevice(this, X5_COMOutputDeviceRegisters)),
       FIo(this),
       Owner(owner), Clock(this),
       TestCtlReg(LogicMemory, mmSfpSioChannelEnable)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_COMDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_COMDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_COMDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_COMDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_COMDeviceMap::OnAfterStreamStopHandler );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X5_COMDeviceMap::~X5_COMDeviceMap()
{
	delete FInput;
	FInput = 0;

	delete FOutput;
	FOutput = 0;
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::OpenHardware()
{
    X5DeviceMap::OpenHardware();

    // Configure A/Ds
    for (int ch = 0; ch < Input().Info().Channels().Channels(); ++ch)
        {
        FInput->PacketSize(0x4000);
        FInput->PacketId(0x0);
        FInput->Gain(ch, 1.0);
        FInput->Offset(ch, 0.0);
        Input().Unframed();
        }
    // Configure D/As
    for (int ch = 0; ch < Output().Info().Channels().Channels(); ++ch)
        {
        FOutput->PacketSize(0x4000);
        FOutput->PacketId(0x2);
        FOutput->Gain(ch, 1.0);
        FOutput->Offset(ch, 0.0);
		Output().Unframed();
        }

    AllLinksOff();
    //
    //  Init SFP channel oscillators
    Io().Sfp[0]->Oscillator.Init();
    Io().Sfp[1]->Oscillator.Init();
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
	// Family-common init
	X5DeviceMap::ConfigureHardware();
    AllLinksOff();

    // Re-apply all clock settings
    Clock.Reapply();
	//
	//  Apply Gain/Offset Coefficients
	if (FOutput->Calibrated())
		FOutput->ApplyRomCoefficients();
	else
		FOutput->ApplyDefaultCoefficients();

	if (FInput->Calibrated())
		FInput->ApplyRomCoefficients();
	else
		FInput->ApplyDefaultCoefficients();

	//
	//  Apply output settings
	for (int ch = 0; ch < FOutput->Info().Channels().Channels(); ++ch)
		{
		FOutput->ChannelEnabled(ch, FOutput->Info().Channels().Enabled(ch));
        FOutput->PacketId(FOutput->PacketId());
		FOutput->PacketSize(FOutput->PacketSize());
		}

	//
	//  Apply input settings
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		{
		FInput->ChannelEnabled(ch, FInput->Info().Channels().Enabled(ch));
        FInput->PacketId(FInput->PacketId());
		FInput->PacketSize(FInput->PacketSize());
		}

	// Reapply Hardware Settings for  X5 IO Devices
	FOutput->ConfigureHardware();
	FInput->ConfigureHardware();

	// Configure triggers
	FOutput->TriggerReg.Apply();
	FInput->TriggerReg.Apply();

	//  Forward notify to baseboard handler
	Owner->OnStreamConfigure.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);

    //  Set Run bit for the enabled links
    if (Input().Info().Channels().Enabled(0))
        Input().Link()->Run(true);
    if (Output().Info().Channels().Enabled(0))
        Output().Link()->Run(true);

    //  Enable User Logic Streaming
    UserFpgaControlReg.InGo.Set();
    UserFpgaControlReg.OutGo.Set();	
    UserFpgaControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    AllLinksOff();

    //  Disable User Logic Streaming
    UserFpgaControlReg.InGo.Clear();
    UserFpgaControlReg.OutGo.Clear();	
    UserFpgaControlReg.Apply();

	Trigger(tInputOutput, false);

	//  Forward notify to baseboard handler
	Owner->OnAfterStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X5_COMInputDevice & X5_COMDeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

X5_COMOutputDevice & X5_COMDeviceMap::Output()
{
	return *FOutput;
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::TxTestEnable() -- Enable Tx Test Mode
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::TxTestEnable(bool enable)
{
	TestCtlReg.TxTestGenEnable.Value(enable);
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::TxTestEnable() -- Read Tx Test state
//---------------------------------------------------------------------------

bool  X5_COMDeviceMap::TxTestEnable()
{
	return TestCtlReg.TxTestGenEnable.Value();
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::RxTestEnable() -- Enable Tx Test Mode
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::RxTestEnable(bool enable)
{
	TestCtlReg.RxTestChkEnable.Value(enable);
}
//---------------------------------------------------------------------------
//  X5_COMDeviceMap::RxTestEnable() -- Read Rx Test state
//---------------------------------------------------------------------------

bool  X5_COMDeviceMap::RxTestEnable()
{
	return TestCtlReg.RxTestChkEnable.Value();
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::ErrorCount() -- Read error count
//---------------------------------------------------------------------------

int  X5_COMDeviceMap::ErrorCount()
{
	return TestCtlReg.ErrorCount.Value();
}

//---------------------------------------------------------------------------
//  X5_COMDeviceMap::AllLinksOff() -- Turn off all links
//---------------------------------------------------------------------------

void  X5_COMDeviceMap::AllLinksOff()
{
    // Power Down & Disable all Links
    for (int i=0; i<X5_COM_Io::lnkTotalLinks; i++)
        {
        Io().Link(static_cast<X5_COM_Io::SerialLinkId>(i))->Run(false);
        Io().Link(static_cast<X5_COM_Io::SerialLinkId>(i))->PowerDown(true);
        Io().Link(static_cast<X5_COM_Io::SerialLinkId>(i))->LinkEnable(X5_Sio::tInputOutput, false);
        X5_Sfp * Sfp = Io().SfpLink(static_cast<X5_COM_Io::SerialLinkId>(i));
        if (Sfp)
            {
            Sfp->Trans[0].Enabled(false);
            Sfp->Trans[1].Enabled(false);
            }
        }
}

} // namespace Innovative


