// X5_COMDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_COMDeviceMap_MbH
#define X5_COMDeviceMap_MbH

#include <X5_COM_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <I2C_Mb.h>
#include <Si570_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X5_COMDeviceMap;

//==============================================================================
//  CLASS X5_Rio_Registers -- X5-COM RocketIo Register Definitions
//==============================================================================

class X5_Rio_Registers
{
public:
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS ControlRegister  -- RIO control/status register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class ControlRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit 		Reset;
		RegisterBit 		PowerDown;
		RegisterBit 		Run;
		RegisterBitGroup	Loopback;
		RegisterBit 		ClearErrorFlags;
		RegisterBit 		TxEnable;
		RegisterBit 		RxEnable;
		RegisterBit 		FrameError;
		RegisterBit 		SoftError;
		RegisterBit 		HardError;
		RegisterBitGroup	LaneUp;
		RegisterBit 		ChannelUp;
		RegisterBit 		DcmLocked;

		ControlRegister(AddressingSpace &space, int offset)
			: inherited("X5_Rio_Registers::ControlRegister", space, offset),
			  Reset(*this, 0), PowerDown(*this, 1),
			  Run(*this, 2), Loopback(*this, 3, 2),
              ClearErrorFlags(*this, 6), TxEnable(*this, 7), RxEnable(*this, 8),
			  FrameError(*this, 25), SoftError(*this, 26),
			  HardError(*this, 27), LaneUp(*this, 28, 2),
			  ChannelUp(*this, 30), DcmLocked(*this, 31)
		{
		}
	};
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS CommandRegister  -- RIO command channel I/O register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class CommandRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup	Data;
		RegisterBitGroup	Address;
		RegisterBit 		Write;

		CommandRegister(AddressingSpace &space, int offset)
			: inherited("X5_Rio_Registers::CommandRegister", space, offset),
			  Data(*this, 0, 24), Address(*this, 24, 6),
			  Write(*this, 30)
		{
		}
    };
    
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS CommandRegisterReadback  -- RIO command channel I/O register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class CommandRegisterReadback : public Register
	{
		typedef Register inherited;

	public:
		RegisterBitGroup	Data;
		RegisterBit 		Ready;

		CommandRegisterReadback(AddressingSpace &space, int offset)
			: inherited("X5_Rio_Registers::CommandRegisterReadback", space, offset),
			  Data(*this, 0, 24), Ready(*this, 31)
		{
		}


	};
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TranceiverRegister  -- RIO tranceiver register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class TranceiverRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         Enable0;
		RegisterBit         Enable1;
		RegisterBit         HighSpeed0;
		RegisterBit         HighSpeed1;
		RegisterBit         SerialClock0;
		RegisterBit         SerialClock1;
		RegisterBit         SerialDataOut0;
		RegisterBit         SerialDataOut1;
		RegisterBit         SerialDataIn0;
		RegisterBit         SerialDataIn1;
		RegisterBit         NoSignal0;
		RegisterBit         NoSignal1;
		RegisterBit         Fault0;
		RegisterBit         Fault1;
		RegisterBit         NoTranceiver0;
		RegisterBit         NoTranceiver1;
        RegisterBit         OscillatorClockBit;
        RegisterBit         OscillatorDataOutBit;
        RegisterBit         OscillatorDataInBit;

		TranceiverRegister(AddressingSpace &space, int offset)
			: inherited("X5_Rio_Registers::TranceiverRegister", space, offset),
			  Enable0(*this, 0), Enable1(*this, 1),
              HighSpeed0(*this, 2), HighSpeed1(*this, 3),
			  SerialClock0(*this, 4), SerialClock1(*this, 5),
              SerialDataOut0(*this, 6), SerialDataOut1(*this, 7),
              SerialDataIn0(*this, 8), SerialDataIn1(*this, 9),
			  NoSignal0(*this, 10), NoSignal1(*this, 11),
              Fault0(*this, 12), Fault1(*this, 13),
			  NoTranceiver0(*this, 14),  NoTranceiver1(*this, 15),
			  OscillatorClockBit(*this, 16),
              OscillatorDataOutBit(*this, 17), OscillatorDataInBit(*this, 18)
		{
		}

        void  Enabled(int dev, bool state);
        bool  Enabled(int dev);

        void  HighSpeed(int dev, bool state);
        bool  HighSpeed(int dev);

        bool  SerialClock(int dev);
        void  SerialClock(int dev, bool state);

        bool  SerialDataOut(int dev);
        void  SerialDataOut(int dev, bool state);
        bool  SerialDataIn(int dev);
        void  SerialDataIn(int dev, bool state);

        bool  NoSignal(int dev);
        bool  Fault(int dev);
        bool  NoTranceiver(int dev);

        bool  OscillatorClock();
        void  OscillatorClock(bool state);

        bool  OscillatorDataOut();
        void  OscillatorDataOut(bool state);
        bool  OscillatorDataIn();
        void  OscillatorDataIn(bool state);
	};
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TranceiverIntf  -- RIO tranceiver interface to "subdevices"
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class TranceiverIntf
	{
	public:
        TranceiverIntf()
            :  TR(0), SubDev(0)
            {}

        void  Init(TranceiverRegister * tr, int subdev)
            {  TR = tr;  SubDev=subdev;  }

        void  Enabled(bool state)
            {  TR->Enabled(SubDev, state);  }
        bool  Enabled()
            {  return TR->Enabled(SubDev);  }

        void  HighSpeed(bool state)
            {  TR->HighSpeed(SubDev, state);  }
        bool  HighSpeed()
            {  return TR->HighSpeed(SubDev);  }

        bool  SerialClock()
            {  return TR->SerialClock(SubDev);  }
        void  SerialClock(bool state)
            {  TR->SerialClock(SubDev, state);  }

        bool  SerialDataOut()
            {  return TR->SerialDataOut(SubDev);  }
        void  SerialDataOut(bool state)
            {  TR->SerialDataOut(SubDev, state);  }
        bool  SerialDataIn()
            {  return TR->SerialDataIn(SubDev);  }
        void  SerialDataIn(bool state)
            {  TR->SerialDataIn(SubDev, state);  }

        bool  NoSignal()
            {  return TR->NoSignal(SubDev);  }
        bool  Fault()
            {  return TR->Fault(SubDev);  }
        bool  NoTranceiver()
            {  return TR->NoTranceiver(SubDev);  }

    private:
        TranceiverRegister * TR;
        int                  SubDev;
    };

};

//==============================================================================
//  CLASS X5_Sio -- X5-COM base Rocket I/O control
//==============================================================================

class X5_Sio : public X5_Rio_Registers
{
public:
    enum IIDirection { tInput, tOutput, tInputOutput };
    enum LoopbackModes { lbNone, lbParallel, lbSerial };

    X5_Sio(AddressingSpace & space, int control_offset, int write_offset, int read_offset)
        :  ControlReg(space, control_offset),
           CommandWriteReg(space, write_offset), CommandWriteRb(space, write_offset),
           CommandReadRb(space, read_offset),
           FLoopbackMode(0)
        {}

    void  Reapply();

    void  Reset();
    void  LinkEnable(IIDirection dir, bool state);

    void  ClearErrorFlags();

    void  WaitForDcmLock();
    void  WaitForChannelUp();

    void  GtpReset(bool state);
    bool  GtpReset();

    void  PowerDown(bool state);
    bool  PowerDown();

    void  Run(bool state);
    bool  Run();

    void  LoopbackMode(LoopbackModes mode)
            {  LoopbackMode(static_cast<int>(mode));  }
    void  LoopbackMode(int value);
    int   LoopbackMode();

    void  TxEnabled(bool value);
    bool  TxEnabled();
    void  RxEnabled(bool value);
    bool  RxEnabled();

    bool  FrameError();
    bool  SoftError();
    bool  HardError();
    int   LaneUp();
    bool  ChannelUp();
    bool  DcmLocked();
    //
    //  Command Channel I/O
    void  WriteCmd(int addr, int data);      // Send command, no reply
    int   ReadCmd(int addr, int data);       // Send command, reply

protected:
  	ControlRegister		     ControlReg;
	CommandRegister          CommandWriteReg;
	CommandRegisterReadback  CommandWriteRb;
	CommandRegisterReadback  CommandReadRb;

    int                      FLoopbackMode;


};

//==============================================================================
//  CLASS X5_Sfp -- X5-COM SFP Rocket I/O control
//==============================================================================

class X5_Sfp : public X5_Sio
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS OscI2C_Impl  -- RIO tranceiver interface to "subdevices"
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class OscI2C_Impl : public I2CDeviceIntf
    {
    public:
        OscI2C_Impl(TranceiverRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~OscI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.OscillatorClock(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.OscillatorClock();  }
        virtual void    SetData(int state)
			{  Reg.OscillatorDataOut(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.OscillatorDataIn();  }

    private:
        TranceiverRegister &  Reg;
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TransI2C_Impl  -- RIO tranceiver interface to "subdevices"
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TransI2C_Impl : public I2CDeviceIntf
    {
    public:
        TransI2C_Impl(TranceiverIntf & intf)
            :  Trans(intf)
            {}
        virtual ~TransI2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
			{  Trans.SerialClock(state ? true : false);  }
        virtual int     GetClock() const
            {  return Trans.SerialClock();  }
        virtual void    SetData(int state)
			{  Trans.SerialDataOut(state ? true : false);  }
        virtual int     GetData() const
            {  return Trans.SerialDataIn();  }

    private:
        TranceiverIntf & Trans;
    };

protected:
	TranceiverRegister	TranceiverReg;

    OscI2C_Impl         OscillatorIntf;
    TransI2C_Impl       Trans0Intf;
    TransI2C_Impl       Trans1Intf;

    I2CDeviceN          Oscillator_I2C;
    I2CDeviceN          Trans0_I2C_A;
    I2CDeviceN          Trans0_I2C_B;
    I2CDeviceN          Trans1_I2C_A;
    I2CDeviceN          Trans1_I2C_B;

public:

    TranceiverIntf  Trans[2];
    Si570           Oscillator;

    X5_Sfp(AddressingSpace & space, int control_offset, int write_offset,
                                    int read_offset, int trans_offset);


    I2CDevice &     TransI2C(int trans, int dev);

};


//==============================================================================
//  CLASS X5_COMClock -- X5 COM "Null" clock control
//==============================================================================

class X5_COMClock : public X5ClockIntf
{
public:
    X5_COMClock(X5_COMDeviceMap * owner);

    //
    //  Interface Functions
    // Methods
    virtual double			FrequencyActual() const;

    // Properties
    virtual void    		Frequency(double value);
    virtual double			Frequency() const;

    virtual IIClockSource   Source() const;
    virtual void            Source(IIClockSource src);

    virtual void 			Reapply();
    virtual bool            Locked() const;

protected:
    // Data
    X5_COMDeviceMap *       Owner;
    double                  FFrequency;
    double                  FExternal;
    IIClockSource           FSource;

};

//===========================================================================
//  CLASS X5_COM_Io  -- SIO/SPF Register and Interface
//===========================================================================

class X5_COM_Io
{
public:
    enum SerialLinkId { lnkSfp01, lnkSfp23, lnkSio01, lnkSio23, lnkTotalLinks };

    X5_Sio *        Sio[2];
    X5_Sfp *        Sfp[2];

    X5_Sio *  Link(SerialLinkId link);
    X5_Sfp *  SfpLink(SerialLinkId link);

	X5_COM_Io(X5_COMDeviceMap * owner);

private:

    X5_COMDeviceMap *                   Owner;
    X5_Sfp                              Sfp_01;
    X5_Sfp                              Sfp_23;
    X5_Sio                              Sio_01;
    X5_Sio                              Sio_23;
};

//===========================================================================
//  CLASS X5_COMInputDevice  -- Register & low level support for X5 ADCs
//===========================================================================

class X5_COMInputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_COMDeviceMap;

public:
	X5_COM_Io &             Io();
    X5_Sio *                Link()
        {  return Io().Link(SerialLink());  };
    X5_Sfp *                SfpLink()
        {  return Io().SfpLink(SerialLink());  };

	X5_COM_Io::SerialLinkId   SerialLink() const
        {  return FLink;  }
 	void    		     SerialLink(X5_COM_Io::SerialLinkId link)
        {  FLink = link;  }
 	void    		     SerialLink(unsigned int link)
        {  FLink = static_cast<X5_COM_Io::SerialLinkId>(link);  }

	virtual void    TestEnable(bool state);
    virtual bool    TestEnable() const;

    int             TestErrorCount() const;

    virtual void    ConfigureHardware();

protected:
	X5_COMInputDevice(X5_COMDeviceMap * owner, X5IoDeviceRegisters & reg);
    X5_COMDeviceMap *                   COM_Owner;
    X5_COM_Io::SerialLinkId             FLink;
    bool                                FTestEnabled;

};

//===========================================================================
//  CLASS X5_COMOutputDevice  -- Register & low level support for X5 DACs
//===========================================================================

class X5_COMOutputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_COMDeviceMap;

public:

	// Properties
	X5_COM_Io &             Io();
    X5_Sio *                Link()
        {  return Io().Link(SerialLink());  };
    X5_Sfp *                SfpLink()
        {  return Io().SfpLink(SerialLink());  };

	X5_COM_Io::SerialLinkId   SerialLink() const
        {  return FLink;  }
 	void    		     SerialLink(X5_COM_Io::SerialLinkId link)
        {  FLink = link;  }
 	void    		     SerialLink(unsigned int link)
        {  FLink = static_cast<X5_COM_Io::SerialLinkId>(link);  }

	bool    		TestEnable() const;
 	void    		TestEnable(bool state);

    int             TestErrorCount() const;

    virtual void    ConfigureHardware();


protected:
	X5_COMOutputDevice(X5_COMDeviceMap * owner, X5IoDeviceRegisters & reg);


	// Registers
    X5_COMDeviceMap *                   COM_Owner;
    Trace                               Debug;
    X5_COM_Io::SerialLinkId             FLink;
    bool                                FTestEnabled;

	// Fields
};

//===========================================================================
//  CLASS X5_COMDeviceMap  -- Register & low level support for SD module
//===========================================================================
class X5_COM;

class X5_COMDeviceMap :  public X5DeviceMap,
						 public IUsesStreamDeviceEventCallbacks

{
	friend class X5_COM;
    friend class X5_COMOutputDevice;
    friend class X5_COMInputDevice;
    friend class X5_COMClock;

public:
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TestGeneratorControlRegister  -- SFP/SIO test register
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class TestGeneratorControlRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit 		TxTestGenEnable;
		RegisterBit 		RxTestChkEnable;
		RegisterBitGroup	ErrorCount;

		TestGeneratorControlRegister(AddressingSpace &space, int offset)
			: inherited("X5_Rio_Registers::ControlRegister", space, offset),
			  TxTestGenEnable(*this, 0), RxTestChkEnable(*this, 1),
			  ErrorCount(*this, 16, 16)
		{
		}
	};
	//
	//  Member Functions
	X5_COMDeviceMap(X5_COM * owner);
	virtual ~X5_COMDeviceMap();

	X5_COMInputDevice  &    Input();
	X5_COMOutputDevice &    Output();
	X5_COM_Io &             Io()
        {  return FIo;   }

public: // pseudo-protected
    virtual void  OpenHardware();

protected:
    // Fields
    X5_COMInputDevice  *   FInput;
    X5_COMOutputDevice *   FOutput;
    X5_COM_Io              FIo;
    
    // Data
    X5_COM *           Owner;
    X5_COMClock        Clock;
    //
    // Registers
    TestGeneratorControlRegister  TestCtlReg;
    
    Trace               Debug;

    // Methods
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void  TxTestEnable(bool enable);
    bool  TxTestEnable();
    void  RxTestEnable(bool enable);
    bool  RxTestEnable();
    int   ErrorCount();
    void  AllLinksOff();
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
