//---------------------------------------------------------------------------
// X5_COM_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X5_COM_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5_COM  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X5_COM::X5_COM()
    :   X5Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X5_COM::~X5_COM()
{
    Close();
}

//---------------------------------------------------------------------------
//  X5_COM::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X5_COM::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X5_COM::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X5_COM::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X5_COM::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X5_COM::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X5_COM::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X5_COM::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X5_COM::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X5_COM::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X5_COM::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X5_COM::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X5_COM::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X5_COM::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X5_COM::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X5_COM::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X5_COM::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X5_COM::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X5_COM::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X5_COM::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X5_COM::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X5_COM::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_COM::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X5_COM::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_COM::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X5_COM::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X5_COM::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X5_COM::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X5_COM::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X5_COM::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X5_COM::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X5_COM::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X5_COM::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & X5_COM::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  X5_COM::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X5_COM::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X5_COM::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X5_COM::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X5_COM::PciLoopbackMode() --
//------------------------------------------------------------------------------

X5_COM::LoopbackMode X5_COM::PciLoopbackMode()
{
    X5_COM::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X5_COM::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5_COM::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X5_COM::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5_COM::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X5_COM::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X5_COMInputDevice & X5_COM::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X5_COM::Output() --  Return reference to input object
//---------------------------------------------------------------------------

X5_COMOutputDevice & X5_COM::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  X5_COM::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X5_COM::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_COM::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X5_COM::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_COM::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X5_COM::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X5DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X5_COM::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX5Alerts & X5_COM::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X5_COM::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X5_COM::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_COM::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X5_COM::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X5_COM::Clock() --   Access to clock control object
//---------------------------------------------------------------------------

X5_COMClock & X5_COM::Clock()
{
    return Device.Clock;
}

//---------------------------------------------------------------------------
//  X5_COM::BusMasterSize() --
//---------------------------------------------------------------------------

void X5_COM::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_COM::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_COM::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_COM::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_COM::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_COM::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_COM::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_COM::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_COM::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_COM::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X5_COM::AfterFiclStart(FiclSystem & engine)
{
//    engine.Evaluate(" " );

    engine.Evaluate(": ?X5-COM "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Dac \" "
                    "    cr  "
                    "    cr .\" ---- X5-COM Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X5-COM  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

	engine.Display("X5-COM-specific wordset loaded \n" );
    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X5_COM::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X5_COM::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X5_COM::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X5_COM::Do_LStore(int addr, int value)
{
	PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X5_COM::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X5_COM::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X5_COM::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X5_COM::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X5_COM::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X5_COM::Do_DFetch(int /*dev*/, int /*addr*/)
{
    return 0;
}

//---------------------------------------------------------------------------
//  X5_COM::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X5_COM::Do_DStore(int /*dev*/, int /*addr*/, int /*value*/)
{
}

//---------------------------------------------------------------------------
//  X5_COM::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X5_COM::Do_DFreq(int /*dev*/, float /*freq*/)
{
	// no frequency devices
	/*
	switch (dev)
        {
        default:
            break;
        }
	*/
}

//---------------------------------------------------------------------------
//  X5_COM::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X5_COM::Do_DFreqActual(int /*dev*/)
{
    // no frequency devices
	return 0.0f;
	/*
    switch (dev)
        {
        default:
            return 0.0f;
        }
	*/

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
