// X5_G12DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_G12DeviceMap_MbH
#define X5_G12DeviceMap_MbH

#include <X5_G12_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <CDCE72010_Mb.h>
#include <Si570_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X5_G12DeviceMap;

//==============================================================================
//  CLASS X5_G12_Pll -- G12 PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X5_G12_Pll : public Cdce72010
{
public:
    X5_G12_Pll(X5_G12DeviceMap * owner,  AddressingSpace &space, Cdce72010Registers & reg);

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return 1.0e6;   }
protected:
    X5_G12DeviceMap * Owner;

    //  Don't support ADC/DAC Groups on this clock
    virtual unsigned int AdcMasterOutput()
        {  return FMasterOutput;  }
    virtual unsigned int DacMasterOutput()
        {  return FMasterOutput;  }

    virtual void  SetAdcOutputRates(double adc_rate)
        {  OutputFrequency(AdcMasterOutput(), adc_rate);  }
    virtual void  SetDacOutputRates(double /*dac_rate*/)
        {}

};

//==============================================================================
//  CLASS X5_G12Clock -- G12 clock control
//==============================================================================

class X5_G12Clock : public X5ClockIntf
{
public:
    X5_G12Clock(X5_G12DeviceMap * owner);

    //
    //  Interface Functions
    // Methods
    virtual double			    FrequencyActual() const;

    // Properties
    virtual void    		    Frequency(double value);
    virtual double			    Frequency() const;

    virtual IIClockSource       Source() const;
    virtual void                Source(IIClockSource src);

    virtual void                Reference(IIReferenceSource src);
    virtual IIReferenceSource   Reference() const;

    virtual void    		    ReferenceFrequency(double value);
    virtual double			    ReferenceFrequency() const;

    void 			            Reapply();
    bool                        Locked() const;

    void                        EnableRefVco(bool enabled);

protected:
    // Data
    X5_G12DeviceMap *      Owner;
    float                  FClockFactor;
};


//===========================================================================
//  CLASS X5_G12InputDevice  -- Register & low level support for X5 ADCs
//===========================================================================

class X5_G12InputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_G12DeviceMap;

public:
    enum  TestModes {  tmSawtooth, tmSine, tmWalking, tmAlternating };
    enum  FFTSizes  {  fft4k=6, fft16k=7, fft64k=8, fft256k=9 };

    // Methods
    virtual void            Framed(int samples);
    virtual void            Unframed();

    // Properties
    virtual void            ExternalTrigger(bool state);
    virtual bool            ExternalTrigger() const;

    virtual void            EdgeTriggered(bool state);
    virtual bool            EdgeTriggered() const;

    virtual void            ChannelEnabled(int ch, bool state);
    virtual bool            ChannelEnabled(int ch) const;

    virtual void            TestEnable(bool state);
    virtual bool            TestEnable() const;

    void                    TestMode(TestModes mode);
    TestModes               TestMode() const;

    void                    TestFrequency(float value);
    float                   TestFrequency() const;

    void                    SoftwareTrigger(bool state);
    bool                    SoftwareTrigger() const;

    void                    LowSpeedSampleRate(float value);
    float                   LowSpeedSampleRate() const;

    void                    LowSpeedPacketId(unsigned char id);
    unsigned char           LowSpeedPacketId() const;

    void                    LowSpeedPacketSize(unsigned int size);
    unsigned int            LowSpeedPacketSize() const;

    void                    Spi(int adc, int addr, unsigned char data);
    unsigned int            Spi(int adc, int addr );

    void                    FFTMode(bool mode);

    void                    FFTPacketId(unsigned char id);
    unsigned char           FFTPacketId() const;

    void                    FFTSize(FFTSizes size);
    FFTSizes                FFTSize() const;

    void                    DESMode(bool state);
    bool                    DESMode() const;
    bool                    DESModeCapable() const;

    void                    DESPhase(int ch, unsigned int phase);
    unsigned int            DESPhase(int ch) const;
    void                    DESPhaseCalibrated(bool flag);
    bool                    DESPhaseCalibrated() const;

    void                    HsRun(bool state);
    void                    LsRun(bool state);

    void                    FFTScaling(int div0, int div1, int div2, int div3, int div4);
    //  Operations
    void                    WaitForReady();

protected:
    // Methods
    virtual void            ConfigureHardware();
    // Overrides: Apply to ROM and hardware
    virtual void    SetCalibrationOffset(int ch, int value);
    virtual void    SetCalibrationGain(int ch, int value);


//    unsigned int    LogicVariant() const;
    unsigned int            EnabledLsChannels();

    // Data
    X5_G12DeviceMap *       Owner;
    StdSpiDevice            Adc0;
    StdSpiDevice            Adc1;
    Trace                   Debug;

	X5_G12InputDevice(X5_G12DeviceMap * owner, X5IoDeviceRegisters & reg);
	~X5_G12InputDevice();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcTestGeneratorRegister  -- ADC Test Mode interface control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcTestGeneratorRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           TestEnable;
        RegisterBitGroup      TestMode;

        AdcTestGeneratorRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::AdcTestGeneratorRegister", space, offset),
               TestEnable(*this,  0),
               TestMode(*this, 1, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcControlRegister  -- ADC interface control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           HsRun;
        RegisterBit           LsRun;
        RegisterBit           DesModeEnable;
        RegisterBitGroup      ChannelEnable;
        RegisterBitGroup      ChannelPHYReady;
        RegisterBit           LowSpeedReady;

        AdcControlRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::AdcControlRegister", space, offset),
               HsRun(*this, 1),
               LsRun(*this, 2),
               DesModeEnable(*this, 3),
               ChannelEnable(*this, 4, 4),
               ChannelPHYReady(*this, 28, 2),
               LowSpeedReady(*this, 30)
            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS LsAdcRateRegister  -- Low-speed ADC rate control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class LsAdcRateRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Divider;
        RegisterBitGroup      HighPeriod;

        LsAdcRateRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::LsAdcRateRegister", space, offset),
               Divider(*this, 0, 20),
               HighPeriod(*this, 20, 10)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TriggerRegister  -- Trigger control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Count;
        RegisterBit           EdgeTriggered;
        RegisterBit           FramedMode;
        RegisterBit           External;

        TriggerRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::TriggerRegister", space, offset),
               Count(*this, 0, 24), EdgeTriggered(*this, 29),
               FramedMode(*this, 30), External(*this, 31)
            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ExternalTriggerIDelayRegister  -- Trigger IDelay Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//    class ExternalTriggerIDelayRegister : public Register
//    {
//        typedef Register inherited;
//
//    public:
//        RegisterBitGroup      IDelayTapSetting;
//
//        ExternalTriggerIDelayRegister(AddressingSpace &space, int offset)
//            :  inherited("X5_G12DeviceMap::ExternalTriggerIDelayRegister", space, offset),
//               IDelayTapSetting(*this, 0, 6)
//            {
//            }
//    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit      Trigger;

        SoftwareTriggerRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::SoftwareTriggerRegister", space, offset),
               Trigger(*this, 0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS LowSpeedPacketHeaderRegister  -- Low Speed Packet Header Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class LowSpeedPacketHeaderRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup  Id;
        RegisterBitGroup  PacketSize;

        LowSpeedPacketHeaderRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::LowSpeedPacketHeaderRegister", space, offset),
               Id(*this, 24, 8), PacketSize(*this, 0, 24)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DclkSkewRegister  -- Dclk Skew Header Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//    class DclkSkewRegister : public ShadowRegister
//    {
//        typedef ShadowRegister inherited;
//
//    public:
//        RegisterBitGroup  Skew;
//
//        DclkSkewRegister(AddressingSpace &space, int offset)
//            :  inherited("X5_G12DeviceMap::DclkSkewRegister", space, offset),
//               Skew(*this, 0, 6)
//            {
//            }
//    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FFTControlRegister  -- FFT Control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FFTControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       ForwardFFT;
        RegisterBit       Is_2DFFT;
        RegisterBitGroup  Scaling;
        RegisterBitGroup  FFTSize;
        RegisterBitGroup  FFTWindow;
        RegisterBit       RawDataStreaming;
        RegisterBit       FFTDataStreaming;
        RegisterBit       Dithering;

        FFTControlRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::FFTControlRegister", space, offset),
               ForwardFFT(*this, 0),   Is_2DFFT(*this, 1),
               Scaling(*this, 2, 10),  FFTSize(*this, 12, 5),
               FFTWindow(*this, 17, 2),
               RawDataStreaming(*this, 19),  FFTDataStreaming(*this, 20),
               Dithering(*this, 21)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FFTWindowRegister  -- Period between FFT calculations
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FFTWindowRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup  Period;

        FFTWindowRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::FFTWindowRegister", space, offset),
               Period(*this, 0, 24)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FFTPacketHeaderRegister  -- FFT Packet Header Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FFTPacketHeaderRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup  Id;
        RegisterBitGroup  PacketSize;

        FFTPacketHeaderRegister(AddressingSpace &space, int offset)
            :  inherited("X5_G12DeviceMap::FFTPacketHeaderRegister", space, offset),
               Id(*this, 24, 8), PacketSize(*this, 0, 24)
            {
            }
    };


    // Registers
    AdcTestGeneratorRegister         AdcTestGenReg;
    ShadowRegister                   TestGenRateReg;
    AdcControlRegister               AdcControlReg;
    LsAdcRateRegister                LsAdcRateReg;
    TriggerRegister                  TriggerReg;
    //ExternalTriggerIDelayRegister    TriggerDelayReg;
    SoftwareTriggerRegister          SoftwareTriggerReg;
    LowSpeedPacketHeaderRegister     LowSpeedPacketHeaderReg;
    //DclkSkewRegister                 DclkSkewReg;
    FFTControlRegister               FFTControlReg;
    FFTWindowRegister                FFTWindowReg;
    FFTPacketHeaderRegister          FFTPacketHeaderReg;
    ShadowRegister                   Adc0PhaseReg;
    ShadowRegister                   Adc1PhaseReg;


    int                     FFrameSize;
    bool                    FFrameMode;
    bool                    FEdgeTriggered;
    float                   FTestGenRate;
    float                   FLowSpeedSampleRate;
    bool                    FIsBoardDESCapable;

    void  DESCapabilityCheck();
};

//===========================================================================
//  CLASS X5_G12DeviceMap  -- Register & low level support for X5-210M module
//===========================================================================
class X5_G12;

class X5_G12DeviceMap :  public X5DeviceMap,
                         public IUsesStreamDeviceEventCallbacks
{
	friend class X5_G12;
    friend class X5_G12InputDevice;
    friend class X5_G12Clock;
    friend class X5_G12_Pll;

public:
	//
	//  Member Functions
	X5_G12DeviceMap(X5_G12 * owner);
	virtual ~X5_G12DeviceMap();

	X5_G12InputDevice  &    Input();

public: // pseudo-protected
    virtual void  OpenHardware();

    //  DES Phase ROM I/O
    void                    SetDESPhase(int ch, unsigned int phase);
    unsigned int            GetDESPhase(int ch) const;
    void                    SetDESPhaseCalibrated(bool state);
    bool                    GetDESPhaseCalibrated() const;
    
protected:
    // Fields
    X5_G12InputDevice  *   FInput;

    // Data
    X5_G12 *           Owner;
    Trace              Debug;
    X5_G12Clock        Clock;
    X5_G12_Pll         Pll;
    PmcIdromSection    DESParamRomSection;

	//---------------------------------------------------
	//  Class VcxoI2CRegister
	class VcxoI2CRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit  Data_Out;
		RegisterBit  Clock_Out;
		RegisterBit  Data_In;
		RegisterBit  Clock_In;
		RegisterBit  RefData_Out;
		RegisterBit  RefClock_Out;
		RegisterBit  RefData_In;
		RegisterBit  RefClock_In;
		RegisterBit  RefVco_En;

	public:
		VcxoI2CRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
               Data_Out(*this, 0),
               Clock_Out(*this, 1),
               Data_In(*this, 2),
               Clock_In(*this, 3),
               RefData_Out(*this, 16),
               RefClock_Out(*this, 17),
               RefData_In(*this, 18),
               RefClock_In(*this, 19),
               RefVco_En(*this, 23)
			{
			}
	};

    VcxoI2CRegister                     VcxoI2CReg;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        VcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~VcxoI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        VcxoI2CRegister &  Reg;
    };

    VcxoI2C_Impl        VcxoIntf;
    I2CDeviceN          Vcxo_I2C;
    Si570               Vcxo;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS RefVcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class RefVcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        RefVcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~RefVcxoI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.RefClock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.RefClock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.RefData_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.RefData_In.Value();  }

    private:
        VcxoI2CRegister &  Reg;
    };

    RefVcxoI2C_Impl     RefVcxoIntf;
    I2CDeviceN          RefVcxo_I2C;
    Si570               RefVcxo;

    // Methods
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void            EnableRefVco(bool enabled);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
