// X5_GSPSDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_GSPSDeviceMap_MbH
#define X5_GSPSDeviceMap_MbH

#include <X5_GSPS_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <Ad9516_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <PulseRepetitionIntervalIntf_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X5_GSPSDeviceMap;

//==============================================================================
//  CLASS X5_GSPSClock -- GSPS clock control
//==============================================================================

class X5_GSPSClock : public X5ClockIntf
{
public:
	X5_GSPSClock(X5_GSPSDeviceMap * owner, AddressingSpace &space,
		int ctl, int spi);

	//
	//  Interface Functions
    // Methods
	virtual double			    FrequencyActual() const;

	// Properties
	virtual void    		    Frequency(double value);
	virtual double			    Frequency() const;

	virtual IIClockSource       Source() const;
	virtual void                Source(IIClockSource src);

	virtual void                Reference(IIReferenceSource src);
	virtual IIReferenceSource   Reference() const;

	virtual void    		    ReferenceFrequency(double value);
	virtual double			    ReferenceFrequency() const;

	void 			            Reapply();
	bool                        Locked() const;

	Ad9516 &   Pll()
		{  return Clock;  }

protected:
	// Data
	X5_GSPSDeviceMap *      Owner;
	Ad9516                  Clock;
};


//===========================================================================
//  CLASS X5_GSPSInputDevice  -- Register & low level support for X5 ADCs
//===========================================================================

class X5_GSPSInputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_GSPSDeviceMap;
    friend class ADC08D1500;

public:
    // Methods
    virtual void            Framed(int samples);
    virtual void            Unframed();

    // Properties
    virtual void            ExternalTrigger(bool state);
    virtual bool            ExternalTrigger() const;

    virtual void            Decimation(unsigned int factor);
    virtual unsigned int    Decimation() const;

    virtual void            ChannelEnabled(int ch, bool state);
    virtual bool            ChannelEnabled(int ch) const;

    virtual void            TestEnable(bool state);
	virtual bool            TestEnable() const;

    void                    PatternEnable(bool state);
    bool                    PatternEnable() const;

    virtual void            AcknowledgeAlert();

    virtual void            AdcClockPhase(unsigned int value);
    virtual unsigned int    AdcClockPhase() const;

	void             Spi(int addr, unsigned short value);
	unsigned short   Spi(int addr);

protected:
    // Methods
    virtual void            ConfigureHardware();

    // Data
    X5_GSPSDeviceMap *      Owner;
    Trace                   Debug;

	X5_GSPSInputDevice(X5_GSPSDeviceMap * owner, X5IoDeviceRegisters & reg);
	~X5_GSPSInputDevice();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcControlRegister  -- ADC interface control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Reset;
        RegisterBit           PowerDown;
        RegisterBit           Calibrate;
        RegisterBit           TestMode;
        RegisterBitGroup      ChannelEnable;
        RegisterBit           AcknowledgeAlert;
        RegisterBit           DclkReset;
        RegisterBit           SyncPipelineReset;
        RegisterBitGroup      Decimation;

        AdcControlRegister(AddressingSpace &space, int offset)
            :  inherited("X5_GSPSDeviceMap::AdcControlRegister", space, offset),
               Reset(*this,  0),
			   PowerDown(*this, 1),
               Calibrate(*this, 2),
               TestMode(*this, 3),
               ChannelEnable(*this, 4, 2),
               AcknowledgeAlert(*this, 6),
               DclkReset(*this, 7),
               SyncPipelineReset(*this, 8),
               Decimation(*this, 9, 2)
            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcStatusRegister  -- ADC interface status
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit           AdcCalibrationDone;
        RegisterBit           ClockCalibrationFailed;
        RegisterBit           ClockCalibrationDone;
        RegisterBit           PhyDcmLocked;

        AdcStatusRegister(AddressingSpace &space, int offset)
            :  inherited("X5_GSPSDeviceMap::AdcStatusRegister", space, offset),
               AdcCalibrationDone(*this, 28),
               ClockCalibrationFailed(*this, 29),
               ClockCalibrationDone(*this, 30),
               PhyDcmLocked(*this, 31)
            {
            //  Default setup ????
            }
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TriggerRegister  -- Trigger control
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class TriggerRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBitGroup      Count;
		RegisterBit           FramedMode;
		RegisterBit           External;

		TriggerRegister(AddressingSpace &space, int offset)
			:  inherited("X5_GSPSDeviceMap::TriggerRegister", space, offset),
			   Count(*this, 0, 24), FramedMode(*this, 30),
			   External(*this, 31)
			{
			//  Default setup ????
			}
	};

	// Overrides: Apply to ROM and hardware
	virtual void    SetCalibrationOffset(int ch, int value);
    virtual void    SetCalibrationGain(int ch, int value);

    void            TriggerEnable(bool state);
    unsigned int    LogicVariant() const;
    bool            IsADC08D1500() const;

	bool            DualEdgeSampling() const;
	void            DualEdgeSampling(bool value);

    // Registers
    AdcControlRegister      AdcControlReg;
    AdcStatusRegister       AdcStatusReg;
    TriggerRegister         TriggerReg;
	Register                AdcClockPhaseReg;
	class ADC08D1500 *      AdcConfigReg;

	int                     FDecimation;
	int                     FFrameSize;
	bool                    FFrameMode;
	unsigned int            FAdcClockPhase;
	bool                    FPatternEnable;
};

//===========================================================================
//  CLASS X5_GSPSDeviceMap  -- Register & low level support for X5-210M module
//===========================================================================
class X5_GSPS;

class X5_GSPSDeviceMap :  public X5DeviceMap,
						  public IUsesStreamDeviceEventCallbacks
{
	friend class X5_GSPS;
	friend class X5_GSPSInputDevice;
	friend class X5_GSPSClock;
	friend class X5_GSPSClockIntf;
	friend class X5_GSPS_PRI_Intf;

public:
	//
	//  Member Functions
	X5_GSPSDeviceMap(X5_GSPS * owner);
	virtual ~X5_GSPSDeviceMap();

	X5_GSPSInputDevice &    		Input();
	IPulseRepetitionIntervalIntf & 	Pulse();

public: // pseudo-protected
	virtual void  OpenHardware();

protected:

	// Fields
	X5_GSPSInputDevice *   			FInput;
	IPulseRepetitionIntervalIntf * 	FPulse;

	// Data
	X5_GSPS *           	Owner;
	Trace               	Debug;
	X5_GSPSClock        	Clock;

	// Methods
	void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

	// Overrides: Apply to ROM and hardware
	virtual void    SetCalibrationOffset(int ch, int value);
	virtual void    SetCalibrationGain(int ch, int value);
};

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
