//---------------------------------------------------------------------------
// X5_GSPS_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X5_GSPS_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5_GSPS  -- GSPS PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X5_GSPS::X5_GSPS()
    :  X5Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X5_GSPS::~X5_GSPS()
{
    Close();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X5_GSPS::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X5_GSPS::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X5_GSPS::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X5_GSPS::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X5_GSPS::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X5_GSPS::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X5_GSPS::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X5_GSPS::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X5_GSPS::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X5_GSPS::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X5_GSPS::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X5_GSPS::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X5_GSPS::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X5_GSPS::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X5_GSPS::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X5_GSPS::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X5_GSPS::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_GSPS::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X5_GSPS::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_GSPS::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X5_GSPS::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X5_GSPS::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X5_GSPS::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X5_GSPS::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X5_GSPS::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X5_GSPS::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X5_GSPS::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X5_GSPS::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & X5_GSPS::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  X5_GSPS::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X5_GSPS::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X5_GSPS::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X5_GSPS::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X5_GSPS::PciLoopbackMode() --
//------------------------------------------------------------------------------

X5_GSPS::LoopbackMode X5_GSPS::PciLoopbackMode()
{
    X5_GSPS::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X5_GSPS::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5_GSPS::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X5_GSPS::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5_GSPS::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Clock() --
//---------------------------------------------------------------------------

X5ClockIntf & X5_GSPS::Clock()
{
	return Device.Clock;
}

//---------------------------------------------------------------------------
//  X5_GSPS::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X5_GSPSInputDevice & X5_GSPS::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X5_GSPS::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X5_GSPS::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_GSPS::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X5_GSPS::IdRom()
{
	return Device;
}

//---------------------------------------------------------------------------
//  X5_GSPS::Pulse() -- Return reference to pulse config object
//---------------------------------------------------------------------------

IPulseRepetitionIntervalIntf & X5_GSPS::Pulse()
{
	return Device.Pulse();
}

//---------------------------------------------------------------------------
//  X5_GSPS::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X5_GSPS::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X5DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X5_GSPS::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX5Alerts & X5_GSPS::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X5_GSPS::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X5_GSPS::Thermal() 
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_GSPS::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X5_GSPS::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X5_GSPS::BusMasterSize() --
//---------------------------------------------------------------------------

void X5_GSPS::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_GSPS::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_GSPS::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_GSPS::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_GSPS::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_GSPS::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_GSPS::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_GSPS::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_GSPS::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_GSPS::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X5_GSPS::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate(" 0 constant Adc   1 constant Pll" );

    engine.Evaluate(": ?X5-GSPS "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Dac   Pll \" "
                    "    cr  "
                    "    cr .\" ---- ?X5-GSPS Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X5-GSPS  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("?X5-GSPS-specific wordset loaded \n" );
    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X5_GSPS::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X5_GSPS::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X5_GSPS::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X5_GSPS::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X5_GSPS::Do_DFetch(int dev, int addr)
{
	const int kAdc = 0;     const int kPll = 1;

    switch (dev)
        {
        default:
            return 0;
        case kAdc:
            return Input().Spi(addr);
        case kPll:
            return Device.Clock.Pll().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X5_GSPS::Do_DStore(int dev, int addr, int value)
{
	const int kAdc = 0;     const int kPll = 1;

    switch (dev)
        {
        default:
            break;
        case kAdc:
            Input().Spi(addr, value);
            break;
        case kPll:
            Device.Clock.Pll().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X5_GSPS::Do_DFreq(int dev, float freq)
{
    const int kPll = 1;

	switch (dev)
        {
        default:
            break;
        case kPll:
            Device.Clock.Pll().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_GSPS::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X5_GSPS::Do_DFreqActual(int dev)
{
    const int kPll = 1;

    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.Clock.Pll().FrequencyActual());
        }

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
