// X5_GSPS_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef X5_GSPS_MbH
#define X5_GSPS_MbH

#include <X5Board_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <X5_GSPSDeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS X5_GSPS  -- X5_GSPS PCI Express baseboard object
//===========================================================================

class X5_GSPS : public X5Board,
                public IStreamDeviceEvents,
                public IFiclTarget
{
    friend class X5_GSPSDeviceMap;

public:
    typedef Ad9516 Timebase;

    // Trigger Support
    // Enumerations
    enum IIDirection { tInput, tOutput, tInputOutput };
    enum LoopbackMode { lmNone, lmParallel, lmSerial };

    //
    //  Member Functions
    X5_GSPS();
    virtual ~X5_GSPS();

    // Methods
    size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
    void                        Open();
    void                        Close();
    virtual void                Reset();
    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    FpgaLogicVersionInfoIntf &  UserLogicVersion();
    PciLogicVersionInfoIntf &   PciLogicVersion();
    IPciBusDiagnostics &        PciDiagnostics();

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    // Properties
    int                         Target();
    void                        Target(int value);

    bool                        Led();
    void                        Led(bool value);

    void                        UserLoopbackMode(bool state);
    bool                        UserLoopbackMode() const;

    void                        PciLoopbackMode(LoopbackMode mode);
	LoopbackMode                PciLoopbackMode();

    void                        Trigger(IIDirection dir, bool state);

    void                        ClockTestEnable(bool state);
    bool                        ClockTestEnable() const;

	void 						BusMasterSize(int bytes);
	void 						IncomingBusMasterSize(int bytes);
	void 						OutgoingBusMasterSize(int bytes);
	int             			IncomingBusMasterSize() const;
	int             			OutgoingBusMasterSize() const;

    // Device access methods
    // EEprom programming
    FpaLoadThread &                   VsProm();

    IUsesDioPort &                    Dio();
	X5_GSPSInputDevice &              Input();
	X5ClockIntf &               	  Clock();
	IUsesPmcEeprom &                  IdRom();
	IPulseRepetitionIntervalIntf &	  Pulse();
	//  Alerts
    IUsesX5Alerts &                   Alerts();
    LogicTemperatureIntf &            Thermal();

public: //Pseudo protected

protected:
    //
    //  Member Data
    X5_GSPSDeviceMap             Device;

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
    virtual int     Do_DFetch(int dev, int addr);
    virtual void    Do_DStore(int dev, int addr, int value);
    virtual void    Do_DFreq(int dev, float freq);
    virtual float   Do_DFreqActual(int dev);

};



#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

