// X5_RXDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5_RX_Mb.h>
#include <X5_RXDeviceMap_Mb.h>
#include <X5MemoryMap_Mb.h>
#include <X5_RXMemoryMap_Mb.h>
#include <Magic_Mb.h>
#include <sstream>
#include <cmath>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X5_MemoryMap;
using namespace X5_RXMemoryMap;

//---------------------------------------------------------
//  Clock and Module Constants Definition
const double  kRefVcxoPowerOnRate = 100.0;    // Ref Si570 power on rate (MHz)
const double  kVcxoPowerOnRate = 10.0;        // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;        //

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxHsDevices      = 4;
const int    MaxLsDevices      = 4;
const int    MaxInDevices      = MaxHsDevices + MaxLsDevices;
const int    InputWidth_Bits   = 16;
const int    InputMuxCount     = 0;
const bool   InputIsUnipolar   = false;
const int    InputClockFactor  = 1;
const float  InputMaxRate      = 200.0e6f;
const float  DesMinRate        = InputMaxRate / 2.0f;
const float  InputMinRate      = 1.0e6f;
const int    InputGangSize     = 1;
const float  Input_ReferenceClk = 100.0e6;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input and Output Coefficient section
//       2channels x 2 values + 1 calibration flag + 1 spare
const int romX5_RX_InputCoeffOffset   = 0;
const int romX5_RX_InChannels         = MaxInDevices;
const int romX5_RX_InputCoeffSize     = romX5_RX_InChannels * 2 + 1 + 1;
const int romX5_RX_CoeffSize          = romX5_RX_InputCoeffSize;

//=============================================================================
//  CLASS X5_RX_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X5_RX_Pll
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X5_RX_Pll
//---------------------------------------------------------------------------

X5_RX_Pll::X5_RX_Pll(X5_RXDeviceMap * owner, AddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner)
{
	SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // Claim Front Panel Ref is 100
   	ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  X5_RX_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  X5_RX_Pll::Vcxo()
{
    return Owner->Vcxo;
}

//---------------------------------------------------------------------------
//  X5_RX_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X5_RX_Pll::OnInit()
{
    //  Devcie Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(true,   // DC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         false,  // Secondary Bias OFF
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       true,   // Input Term disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, true);   SetOutputConfig(0, ocLvds);         // FPGA
	SetDividerEnable(0, true);

	OutputInUse(1, true);  SetOutputConfig(1, ocDisabled);
	SetDividerEnable(1, true);

    OutputInUse(2, false);  SetOutputConfig(2, ocDisabled);
    OutputInUse(3, false);  SetOutputConfig(3, ocDisabled);
    OutputInUse(4, false);  SetOutputConfig(4, ocDisabled);
	OutputInUse(5, true);   SetOutputConfig(5, ocLvPecl);       // A/D 0
	OutputInUse(6, true);   SetOutputConfig(6, ocLvPecl);       // A/D 1
	OutputInUse(7, true);   SetOutputConfig(7, ocLvPecl);       // A/D 2
	OutputInUse(8, true);   SetOutputConfig(8, ocLvPecl);       // A/D 3
	OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);     // used as input

	for (unsigned int pin = 5; pin < 10; ++pin)
		SetHighSwingLvpecl(pin, true);

	//  Configure Aux
	SetAuxInputConfig(aeEnabled, aiNormal);
	SetLockConfig(false);   // NOT analog lock

	//
	//  Set Frequency Limits for Outputs (Preset to 1500.0 MHz)
	//    (Outputs 0 and 1 match preset value, so leave alone)
	for (unsigned int pin = 5; pin < 9; ++pin)
		OutputFrequencyLimit[pin] = static_cast<float>(InputMaxRate + 1.0e3);   // Non-inclusive range
	OutputFrequencyLimit[0] = static_cast<float>(InputMaxRate + 1.0e3);

    //
    //  Reference
    SetExternalReferenceControl(true);  // Use internal bits, not external pin
}

//---------------------------------------------------------------------------
//  X5_RX_Pll::OnReapply()
//---------------------------------------------------------------------------

void  X5_RX_Pll::OnReapply()
{
}

//===========================================================================
// class X5_RXInputInfoIntf
//===========================================================================

class X5_RXInputInfoIntf : public IoInfoIntf
{
public:
    X5_RXInputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxInDevices, InputWidth_Bits, InputMuxCount, InputIsUnipolar)
        {  FSpan.push_back(SpanInfo(1500, -1500));  }
    virtual ~X5_RXInputInfoIntf()  {  }

    virtual UnpairedChannelInfo &       Channels()              {  return FChannels;}
    virtual float                       MaxRate() const         {  return InputMaxRate;  }
    virtual float                       MinRate() const         {  return InputMinRate;  }
    virtual const SpanArray &           Span() const            {  return FSpan;  }
    virtual int                         GangSize() const        {  return InputGangSize;  }
    virtual bool                        IsStackable() const     {  return true;  }
	virtual bool                        IsInverted() const      {  return true;  }
    virtual bool                        IsMuxed() const         {  return false;  }
    virtual bool                        MuxDepth() const        {  return InputMuxCount;  }
    virtual int                         BufferSize() const      {  return 65536;  }
    virtual int                         ClockFactor() const     {  return InputClockFactor;  }
    virtual float                       ReferenceClock() const  {  return Input_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 64;  }

private:
    UnpairedChannelInfo         FChannels;
    SpanArray                   FSpan;
};

//===========================================================================
//  CLASS X5_RXInputDevice  -- Register & low level support for X5
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X5_RXInputDevice
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  constructor for class X5_RXInputDevice
//---------------------------------------------------------------------------

X5_RXInputDevice::X5_RXInputDevice(X5_RXDeviceMap * owner,
								   X5IoDeviceRegisters & reg)
	: inherited(owner,
				reg,
				new X5_RXInputInfoIntf,
				PmcIdromSection(owner->Rom(),
								PmcIdrom::waModule,
								romX5_RX_InputCoeffOffset,
								romX5_RX_InputCoeffSize),
				0x10000),	// 'one' value for gain registers
      Owner(owner),
      Debug(true),
      AdcDitherReg(owner->LogicMemory, mmDither),
      AdcControlReg(owner->LogicMemory, mmAdcControl),
      AdcStatusReg(owner->LogicMemory, mmAdcControl),
      LsAdcRateReg(owner->LogicMemory, mmLsRateConfig),
      TriggerReg(owner->LogicMemory, reg.TriggerControlRegister_Addr),
      LowSpeedPacketHeaderReg(owner->LogicMemory, mmLowSpeedPacketHeader),
      FFrameSize(0x1000), FFrameMode(false),
      FEdgeTriggered(true),
      FLowSpeedSampleRate(50000.0f),
      FTestEnable(false), FDitherEnable(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class X5_RXInputDevice
//---------------------------------------------------------------------------

X5_RXInputDevice::~X5_RXInputDevice()
{
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::Framed() --
//---------------------------------------------------------------------------

void X5_RXInputDevice::Framed(int samples)
{
    FFrameSize = samples/64;
    FFrameMode = true;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::Unframed() --
//---------------------------------------------------------------------------

void X5_RXInputDevice::Unframed()
{
    FFrameMode = false;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void X5_RXInputDevice::ExternalTrigger(bool state)
{
    TriggerReg.External.Value(state);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool X5_RXInputDevice::ExternalTrigger() const
{
    return TriggerReg.External.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::EdgeTriggered() --  Set EdgeTrigger mode
//---------------------------------------------------------------------------

void X5_RXInputDevice::EdgeTriggered(bool state)
{
    TriggerReg.EdgeTriggered.Value(state);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::EdgeTriggered() --  Query EdgeTrigger mode
//---------------------------------------------------------------------------

bool X5_RXInputDevice::EdgeTriggered() const
{
    return TriggerReg.EdgeTriggered.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::HsRun() --  Set HS A/D run state mode
//---------------------------------------------------------------------------

void X5_RXInputDevice::Run(bool state)
{
    AdcControlReg.Run.Value(state);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::ConfigureHardware() -- Apply hardware settings after reset
//---------------------------------------------------------------------------

void X5_RXInputDevice::ConfigureHardware()
{
    double Fs = Owner->Clock.Frequency();
    AdcControlReg.DcmMode.Value((Fs > 120.0e6) ? 1 : 0);

    //
    //  Reset the DCM
    AdcControlReg.DcmReset.Set();
    AdcControlReg.DcmReset.Clear();
    WaitForReady();

    // Apply frame settings
    TriggerReg.Count.Value(FFrameSize);
    TriggerReg.FramedMode.Value(FFrameMode);
    TriggerReg.EdgeTriggered.Value(FEdgeTriggered);

    // LS A/D
    LowSpeedPacketHeaderReg.Apply();
    LowSpeedSampleRate(FLowSpeedSampleRate);

    // Test
    AdcControlReg.TestEnable.Value(FTestEnable ? 1 : 0);
    AdcDitherReg.Enable.Value(FDitherEnable ? 1 : 0);

    // Reapply trigger settings
    TriggerReg.Apply();

    if (EnabledLsChannels() && (AdcStatusReg.LowSpeedReady.Value() != 3))
        Debug.Log("Warning: LS A/Ds not ready - continuing...");
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void X5_RXInputDevice::ChannelEnabled(int ch, bool state)
{
    AdcControlReg.ChannelEnable.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool X5_RXInputDevice::ChannelEnabled(int ch) const
{
    return AdcControlReg.ChannelEnable.Bit(ch);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::WaitForReady() -- Test PHY Ready registers
//---------------------------------------------------------------------------

void X5_RXInputDevice::WaitForReady()
{
    const int Reps = 10;
    //
    //  We try a few times, but eventually give up
    for (int i=0; i<Reps; i++)
        {
        bool Ready =  AdcStatusReg.DcmLocked.Value();
        //  If an enable bit is 0, we don't care about its state. Otherwise, wait
        //    for them all to be 1. Bit 'and' with Enable does this
        if (Ready)
            return;

        Innovative::Sleep(1);
        }
    //
    // if we get here we timed out. Log error
    Debug.Log("Warning: DCM lock did not occur - continuing...");
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_RXInputDevice::TestEnable(bool enable)
{
    FTestEnable = enable;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_RXInputDevice::TestEnable() const
{
	return FTestEnable;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::DitherEnable()
//---------------------------------------------------------------------------

void X5_RXInputDevice::DitherEnable(bool enable)
{
    FDitherEnable = enable;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::DitherEnable()
//---------------------------------------------------------------------------

bool X5_RXInputDevice::DitherEnable() const
{
	return FDitherEnable;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::SoftwareTrigger() -- set software trigger
//---------------------------------------------------------------------------

void X5_RXInputDevice::SoftwareTrigger(bool state)
{
    AdcControlReg.SoftwareTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::SoftwareTrigger()  -- read software trigger
//---------------------------------------------------------------------------

bool X5_RXInputDevice::SoftwareTrigger() const
{
    return AdcControlReg.SoftwareTrigger.Value() ? true : false ;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedSampleRate() --
//---------------------------------------------------------------------------

void X5_RXInputDevice::LowSpeedSampleRate(float rate)
{
    FLowSpeedSampleRate = rate;

    double Fs = Owner->Clock.Frequency();

    int mode = 1;
    if ( (ChannelEnabled(4) && ChannelEnabled(5)) ||
         (ChannelEnabled(6) && ChannelEnabled(7)) )
         mode = 2;

    unsigned int channels = EnabledLsChannels();
    if (!channels)
        return;

    unsigned int divider = static_cast<unsigned int>(Fs / (mode * rate));
    LsAdcRateReg.Divider.Value(divider);

    unsigned int period = static_cast<unsigned int>(std::ceil(3.2e-6f * Fs));
    LsAdcRateReg.HighPeriod.Value(period);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedSampleRate() --
//---------------------------------------------------------------------------

float X5_RXInputDevice::LowSpeedSampleRate() const
{
    return FLowSpeedSampleRate;
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedPacketId() -- Set ID tage of received data packets
//---------------------------------------------------------------------------

void  X5_RXInputDevice::LowSpeedPacketId(unsigned char id)
{
    LowSpeedPacketHeaderReg.Id.Value(id);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedPacketId() -- Query ID tage of received data packets
//---------------------------------------------------------------------------

unsigned char X5_RXInputDevice::LowSpeedPacketId() const
{
	return LowSpeedPacketHeaderReg.Id.Value();
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedPacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

void  X5_RXInputDevice::LowSpeedPacketSize(unsigned int size_words)
{
   LowSpeedPacketHeaderReg.PacketSize.Value(size_words);
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedPacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

unsigned int  X5_RXInputDevice::LowSpeedPacketSize() const
{
	return LowSpeedPacketHeaderReg.PacketSize.Value();
}

//---------------------------------------------------------------------------
//  X5_RXInputDevice::LowSpeedPacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

unsigned int X5_RXInputDevice::EnabledLsChannels()
{
    unsigned int channels = 0;
    for (int ch = 4; ch < Info().Channels().Channels(); ++ch)
        channels += ChannelEnabled(ch) ? 1 : 0;

    return channels;        
}



//==============================================================================
//  CLASS X5_RXClock -- RX clock control interface
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class X5_RXClock
//---------------------------------------------------------------------------

X5_RXClock::X5_RXClock(X5_RXDeviceMap * owner)
    : Owner(owner)
{
}

//---------------------------------------------------------------------------
//  X5_RXClock::Frequency() -- Change output frequency
//---------------------------------------------------------------------------

void X5_RXClock::Frequency(double value)
{
    Owner->Pll.Frequency(value);
}

//---------------------------------------------------------------------------
//  X5_RXClock::Frequency() -- Query output frequency
//---------------------------------------------------------------------------

double X5_RXClock::Frequency() const
{
    return Owner->Pll.Frequency();
}

//---------------------------------------------------------------------------
//  X5_RXClock::FrequencyActual() -- Query actual output frequency
//---------------------------------------------------------------------------

double X5_RXClock::FrequencyActual() const
{
    return Owner->Pll.FrequencyActual();
}

//---------------------------------------------------------------------------
//  X5_RXClock::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

X5_RXClock::IIClockSource X5_RXClock::Source() const
{
    if (Owner->Pll.OutputSource() == X5_RX_Pll::outPll) 
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  X5_RXClock::Source() -- Select clock source
//---------------------------------------------------------------------------

void X5_RXClock::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->Pll.OutputSource(X5_RX_Pll::outPll);
    else
        Owner->Pll.OutputSource(X5_RX_Pll::outAux);

    //
    //  Turn off Output VCO if not using it
    Owner->EnableOutputVco(src == csInternal);
}

//---------------------------------------------------------------------------
//  X5_RXClock::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

X5_RXClock::IIReferenceSource X5_RXClock::Reference() const
{
	if (Owner->Pll.ReferenceSource() == X5_RX_Pll::refPrimary)
		return rsExternal;
	else
        return rsInternal;
}

//---------------------------------------------------------------------------
//  X5_RXClock::Reference() -- Select reference source
//---------------------------------------------------------------------------

void X5_RXClock::Reference(IIReferenceSource src)
{
    if (src == rsInternal)
        Owner->Pll.ReferenceSource(X5_RX_Pll::refAutoSelect);
    else
		Owner->Pll.ReferenceSource(X5_RX_Pll::refPrimary);

    //
    //  Turn off Ref VCO if not using it
    Owner->EnableRefVco( src == rsInternal );
}

//---------------------------------------------------------------------------
//  X5_RXClock::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void X5_RXClock::ReferenceFrequency(double value)
{
	Owner->Pll.Reference(value);
}

//---------------------------------------------------------------------------
//  X5_RXClock::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double X5_RXClock::ReferenceFrequency() const
{
	return Owner->Pll.Reference();
}

//---------------------------------------------------------------------------
//  X5_RXClock::Reapply() -- Reapply all clock settings
//---------------------------------------------------------------------------

void X5_RXClock::Reapply()
{
	// Route clock to active analog devices
	Source(Source());

	// Select reference clock
	Reference(Reference());

	// Select reference rate
	ReferenceFrequency(ReferenceFrequency());

	// Select clock rate
	Frequency(Frequency());
}

//---------------------------------------------------------------------------
//  X5_RXClock::Locked() -- Return PLL lock status
//---------------------------------------------------------------------------

bool X5_RXClock::Locked() const
{
   return Owner->Pll.Locked();
}

//---------------------------------------------------------------------------
//  X5_RXClock::EnableRefVco() -- enable/disable Reference Vco
//---------------------------------------------------------------------------

void  X5_RXClock::EnableRefVco(bool enabled)
{
    Owner->EnableRefVco(enabled);
}

//---------------------------------------------------------------------------
//  X5_RXClock::EnableOutputVco() -- enable/disable Output Vco
//---------------------------------------------------------------------------

void  X5_RXClock::EnableOutputVco(bool enabled)
{
    Owner->EnableOutputVco(enabled);
}

//===========================================================================
//  CLASS X5_RXDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X5_RXDeviceMap
//---------------------------------------------------------------------------

static Cdce72010Registers X5_RX_PllRegisters =
	{ mmPllControl,  mmPllSpiIntf  };

static X5IoDeviceRegisters X5RXIoDeviceRegisters =
	{ mmI2c, 0,  mmInGainCoef, mmInOffsetCoef, mmInChannelEnable,
	  mmInTriggerControl, mmInDecimation, mmInPacketHeader, mmRxPacketControl
	};

//---------------------------------------------------------------------------
//  constructor for class X5_RXDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X5_RXDeviceMap::X5_RXDeviceMap(X5_RX * owner)
    : IUsesStreamDeviceEventCallbacks(owner),
	  FInput (new X5_RXInputDevice (this, X5RXIoDeviceRegisters)),
      Owner(owner), Debug(true),
      Clock(this),
      Pll(this, LogicMemory, X5_RX_PllRegisters),
	  VcxoI2CReg(LogicMemory, mmVcxoI2C),
      VcxoIntf(VcxoI2CReg),
      Vcxo_I2C(VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      Vcxo(Vcxo_I2C, Si570::tcA, kVcxoPowerOnRate),
      RefVcxoIntf(VcxoI2CReg),
      RefVcxo_I2C(RefVcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
      RefVcxo(RefVcxo_I2C, Si570::tcC, kRefVcxoPowerOnRate)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_RXDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_RXDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_RXDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_RXDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_RXDeviceMap::OnAfterStreamStopHandler );

//    DDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_RXDeviceMap::OnStreamConfigureHandler );
//    DDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_RXDeviceMap::OnBeforeStreamStartHandler );
//    DDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_RXDeviceMap::OnAfterStreamStartHandler );
//    DDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_RXDeviceMap::OnBeforeStreamStopHandler );
//    DDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_RXDeviceMap::OnAfterStreamStopHandler );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X5_RXDeviceMap
//---------------------------------------------------------------------------

X5_RXDeviceMap::~X5_RXDeviceMap()
{
	delete FInput;
	FInput = 0;
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::OpenHardware()
{
    X5DeviceMap::OpenHardware();

    RefVcxo.Init();
    Vcxo.Init();
    Pll.Powered(true);
    Pll.Init();

    // Configure A/Ds
    FInput->PacketSize(0x4000);
    FInput->PacketId(0x0);
    FInput->Unframed();
    // Gains and offsets for HS channels only
    for (int ch = 0; ch < MaxHsDevices; ++ch)
        {
        FInput->Gain(ch, 1.0);
        FInput->Offset(ch, 0.0);
        }

    //  Initial Defaults for Low Speed Data
    Input().LowSpeedPacketId(0x08);
    Input().LowSpeedPacketSize(0x3f0);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
	// Family-common init
	X5DeviceMap::ConfigureHardware();

	// Reapply PLL settings
    EnableRefVco(true);
    EnableOutputVco(true);

    RefVcxo.Reapply();
    Vcxo.Reapply();

    Pll.Reapply();
    Clock.Reapply();
    Innovative::Sleep(100);
    if (!Clock.Locked())
        Debug.Log("ERROR: Pll not locked!");

	//
	//  Apply input settings
	//  NOTE: Channel enables MUST PRECEDE input configuration for
	//        proper RX initialization!
	for (int ch = 0; ch < FInput->Info().Channels().Channels(); ++ch)
		FInput->ChannelEnabled(ch, FInput->Info().Channels().Enabled(ch));

    FInput->PacketId(FInput->PacketId());
    FInput->PacketSize(FInput->PacketSize());

	// Reapply Hardware Settings for Input Device
	FInput->ConfigureHardware();

	//
	//  Apply Gain/Offset Coefficients
	if (FInput->Calibrated())
		FInput->ApplyRomCoefficients();
	else
		FInput->ApplyDefaultCoefficients();

	//  Forward notify to baseboard handler
   	OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
   	OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start streaming
    Input().Run(true);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
   	OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
   	OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Disable User Logic Streaming
	Input().SoftwareTrigger(false);
	Input().Run(false);

	//  Forward notify to baseboard handler
   	OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X5_RXInputDevice & X5_RXDeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::EnableRefVco() -- enable/disable Ref Vco
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::EnableRefVco(bool enabled)
{
    VcxoI2CReg.RefVco_En.Value(enabled);
}

//---------------------------------------------------------------------------
//  X5_RXDeviceMap::EnableOutputVco() -- enable/disable output Vco
//---------------------------------------------------------------------------

void  X5_RXDeviceMap::EnableOutputVco(bool enabled)
{
    VcxoI2CReg.Vcxo_En.Value(enabled);
}

} // namespace Innovative


