// X5_RXDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_RXDeviceMap_MbH
#define X5_RXDeviceMap_MbH

#include <X5_RX_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <CDCE72010_Mb.h>
#include <Si570_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X5_RXDeviceMap;

//==============================================================================
//  CLASS X5_RX_Pll -- RX PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X5_RX_Pll : public Cdce72010
{
public:
    X5_RX_Pll(X5_RXDeviceMap * owner,  AddressingSpace &space, Cdce72010Registers & reg);

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return 1.0e6;   }
protected:
    X5_RXDeviceMap * Owner;

    virtual unsigned int AdcMasterOutput()
        {  return FMasterOutput;  }
    virtual unsigned int DacMasterOutput()
        {  return FMasterOutput;  }

    virtual void  SetAdcOutputRates(double adc_rate)
        {  OutputFrequency(AdcMasterOutput(), adc_rate);  }
    virtual void  SetDacOutputRates(double /*dac_rate*/)
        {}

};

//==============================================================================
//  CLASS X5_RXClock -- RX clock control
//==============================================================================

class X5_RXClock : public X5ClockIntf
{
public:
    X5_RXClock(X5_RXDeviceMap * owner);

    //
    //  Interface Functions
    // Methods
    virtual double			    FrequencyActual() const;

    // Properties
    virtual void    		    Frequency(double value);
    virtual double			    Frequency() const;

    virtual IIClockSource       Source() const;
    virtual void                Source(IIClockSource src);

    virtual void                Reference(IIReferenceSource src);
    virtual IIReferenceSource   Reference() const;

    virtual void    		    ReferenceFrequency(double value);
    virtual double			    ReferenceFrequency() const;

    void 			            Reapply();
    bool                        Locked() const;

    void                        EnableRefVco(bool enabled);
    void                        EnableOutputVco(bool enabled);

protected:
    // Data
    X5_RXDeviceMap *      Owner;
};


//===========================================================================
//  CLASS X5_RXInputDevice  -- Register & low level support for X5 ADCs
//===========================================================================

class X5_RXInputDevice : public X5IoDevice
{
    typedef X5IoDevice inherited;
	friend class X5_RXDeviceMap;

public:
    // Methods
    virtual void            Framed(int samples);
    virtual void            Unframed();

    // Properties
    virtual void            ExternalTrigger(bool state);
    virtual bool            ExternalTrigger() const;

    virtual void            EdgeTriggered(bool state);
    virtual bool            EdgeTriggered() const;

    virtual void            ChannelEnabled(int ch, bool state);
    virtual bool            ChannelEnabled(int ch) const;

    virtual void            DitherEnable(bool state);
    virtual bool            DitherEnable() const;

    virtual void            TestEnable(bool state);
    virtual bool            TestEnable() const;

    void                    SoftwareTrigger(bool state);
    bool                    SoftwareTrigger() const;

    void                    LowSpeedSampleRate(float value);
    float                   LowSpeedSampleRate() const;

    void                    LowSpeedPacketId(unsigned char id);
    unsigned char           LowSpeedPacketId() const;

    void                    LowSpeedPacketSize(unsigned int size);
    unsigned int            LowSpeedPacketSize() const;

    void                    Run(bool state);

    //  Operations
    void                    WaitForReady();

protected:
    // Methods
    virtual void            ConfigureHardware();

//    unsigned int    LogicVariant() const;
    unsigned int            EnabledLsChannels();

    // Data
    X5_RXDeviceMap *        Owner;
    Trace                   Debug;

	X5_RXInputDevice(X5_RXDeviceMap * owner, X5IoDeviceRegisters & reg);
	~X5_RXInputDevice();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcControlRegister  -- ADC interface control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           DcmReset;
        RegisterBit           Run;
        RegisterBit           TestEnable;
        RegisterBit           SoftwareTrigger;
        RegisterBitGroup      ChannelEnable;
        RegisterBit           DcmMode;

        AdcControlRegister(AddressingSpace &space, int offset)
            :  inherited("X5_RXDeviceMap::AdcControlRegister", space, offset),
               DcmReset(*this, 0),
               Run(*this, 1),
               TestEnable(*this, 2),
               SoftwareTrigger(*this, 3),
               ChannelEnable(*this, 4, 8),
               DcmMode(*this, 12)
            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcStatusRegister  -- ADC interface state
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      LowSpeedReady;
        RegisterBit           DcmLocked;

        AdcStatusRegister(AddressingSpace &space, int offset)
            :  inherited("X5_RXDeviceMap::AdcStatusRegister", space, offset),
               LowSpeedReady(*this, 29, 2),
               DcmLocked(*this, 31)
            {
            //  Default setup ????
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcDitherRegister  -- ADC dither control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcDitherRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit           Enable;

        AdcDitherRegister(AddressingSpace &space, int offset)
            :  inherited("X5_RXDeviceMap::AdcDitherRegister", space, offset),
               Enable(*this,  0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS LsAdcRateRegister  -- Low-speed ADC rate control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class LsAdcRateRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      Divider;
        RegisterBitGroup      HighPeriod;

        LsAdcRateRegister(AddressingSpace &space, int offset)
            :  inherited("X5_RXDeviceMap::LsAdcRateRegister", space, offset),
               Divider(*this, 0, 20),
               HighPeriod(*this, 20, 10)
            {
            }
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS TriggerRegister  -- Trigger control
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class TriggerRegister : public ShadowRegister
	{
		typedef ShadowRegister inherited;

	public:
		RegisterBitGroup      Count;
		RegisterBit           EdgeTriggered;
		RegisterBit           FramedMode;
		RegisterBit           External;

		TriggerRegister(AddressingSpace &space, int offset)
			:  inherited("X5_RXDeviceMap::TriggerRegister", space, offset),
			   Count(*this, 0, 24), EdgeTriggered(*this, 29),
			   FramedMode(*this, 30), External(*this, 31)
			{
			//  Default setup ????
			}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS LowSpeedPacketHeaderRegister  -- Low Speed Packet Header Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class LowSpeedPacketHeaderRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup  Id;
        RegisterBitGroup  PacketSize;

        LowSpeedPacketHeaderRegister(AddressingSpace &space, int offset)
            :  inherited("X5_RXDeviceMap::LowSpeedPacketHeaderRegister", space, offset),
               Id(*this, 24, 8), PacketSize(*this, 0, 24)
            {
            }
    };


    // Registers
    AdcDitherRegister                AdcDitherReg;
    AdcControlRegister               AdcControlReg;
    AdcStatusRegister                AdcStatusReg;
    LsAdcRateRegister                LsAdcRateReg;
    TriggerRegister                  TriggerReg;
    LowSpeedPacketHeaderRegister     LowSpeedPacketHeaderReg;


    int                     FFrameSize;
    bool                    FFrameMode;
    bool                    FEdgeTriggered;
    float                   FLowSpeedSampleRate;
    bool                    FTestEnable;
    bool                    FDitherEnable;
};

//===========================================================================
//  CLASS X5_RXDeviceMap  -- Register & low level support for X5-210M module
//===========================================================================
class X5_RX;

class X5_RXDeviceMap :  public X5DeviceMap,
                        public IUsesStreamDeviceEventCallbacks
{
	friend class X5_RX;
    friend class X5_RXInputDevice;
    friend class X5_RXClock;
    friend class X5_RX_Pll;

public:
	//
	//  Member Functions
	X5_RXDeviceMap(X5_RX * owner);
	virtual ~X5_RXDeviceMap();

	X5_RXInputDevice  &    Input();

public: // pseudo-protected
    virtual void  OpenHardware();

protected:
    // Fields
    X5_RXInputDevice  *   FInput;

    // Data
    X5_RX *           Owner;
    Trace             Debug;
    X5_RXClock        Clock;
    X5_RX_Pll         Pll;

	//---------------------------------------------------
	//  Class VcxoI2CRegister
	class VcxoI2CRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit  Data_Out;
		RegisterBit  Clock_Out;
		RegisterBit  Data_In;
		RegisterBit  Clock_In;
		RegisterBit  Vcxo_En;
		RegisterBit  RefData_Out;
		RegisterBit  RefClock_Out;
		RegisterBit  RefData_In;
		RegisterBit  RefClock_In;
		RegisterBit  RefVco_En;

	public:
		VcxoI2CRegister( AddressingSpace &space, int offset )
//            :  inherited("X5_RXDeviceMap::VcxoI2CRegister", space, offset),
            :  inherited(space, offset),
               Data_Out(*this, 0),
               Clock_Out(*this, 1),
               Data_In(*this, 2),
               Clock_In(*this, 3),
               Vcxo_En(*this, 7),
               RefData_Out(*this, 16),
               RefClock_Out(*this, 17),
               RefData_In(*this, 18),
               RefClock_In(*this, 19),
               RefVco_En(*this, 23)
			{
			}
	};

    VcxoI2CRegister                     VcxoI2CReg;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        VcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~VcxoI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        VcxoI2CRegister &  Reg;
    };

    VcxoI2C_Impl        VcxoIntf;
    I2CDeviceN          Vcxo_I2C;
    Si570               Vcxo;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS RefVcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class RefVcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        RefVcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)
            {}
	    virtual ~RefVcxoI2C_Impl()
		    {}
        // Methods
        virtual void    SetClock(int state)
			{  Reg.RefClock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.RefClock_In.Value();  }
        virtual void    SetData(int state)
			{  Reg.RefData_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.RefData_In.Value();  }

    private:
        VcxoI2CRegister &  Reg;
    };

    RefVcxoI2C_Impl     RefVcxoIntf;
    I2CDeviceN          RefVcxo_I2C;
    Si570               RefVcxo;

    // Methods
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    // Overrides: Apply to ROM and hardware
    void            EnableRefVco(bool enabled);
    void            EnableOutputVco(bool enabled);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
