//---------------------------------------------------------------------------
// X5_RX_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X5_RX_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5_RX  -- PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X5_RX::X5_RX()
    :  X5Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X5_RX::~X5_RX()
{
    Close();
}

//---------------------------------------------------------------------------
//  X5_RX::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X5_RX::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X5_RX::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X5_RX::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X5_RX::PrintAllDevices() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X5_RX::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X5_RX::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X5_RX::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X5_RX::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X5_RX::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X5_RX::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X5_RX::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X5_RX::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X5_RX::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X5_RX::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X5_RX::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X5_RX::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X5_RX::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X5_RX::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X5_RX::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X5_RX::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X5_RX::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_RX::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X5_RX::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_RX::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X5_RX::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X5_RX::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X5_RX::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X5_RX::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X5_RX::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X5_RX::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X5_RX::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X5_RX::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & X5_RX::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  X5_RX::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X5_RX::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X5_RX::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X5_RX::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X5_RX::PciLoopbackMode() --
//------------------------------------------------------------------------------

X5_RX::LoopbackMode X5_RX::PciLoopbackMode()
{
    X5_RX::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X5_RX::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5_RX::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X5_RX::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5_RX::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X5_RX::Clock() --
//---------------------------------------------------------------------------

X5ClockIntf & X5_RX::Clock()
{
	return Device.Clock;
}

//---------------------------------------------------------------------------
//  X5_RX::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X5_RXInputDevice & X5_RX::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X5_RX::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X5_RX::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_RX::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X5_RX::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_RX::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X5_RX::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X5DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X5_RX::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX5Alerts & X5_RX::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X5_RX::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

LogicTemperatureIntf & X5_RX::Thermal() 
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_RX::Pll() --  Return reference to Onboard PLL
//---------------------------------------------------------------------------

X5_RX_Pll & X5_RX::Pll()
{
	return Device.Pll;
}

//---------------------------------------------------------------------------
//  X5_RX::Vcxo() --  Return reference to Vcxo
//---------------------------------------------------------------------------

Si570 & X5_RX::Vcxo()
{
	return Device.Vcxo;
}

//---------------------------------------------------------------------------
//  X5_RX::RefVcxo() --  Return reference to RefVcxo
//---------------------------------------------------------------------------

Si570 & X5_RX::RefVcxo()
{
	return Device.RefVcxo;
}

//---------------------------------------------------------------------------
//  X5_RX::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X5_RX::ClockTestEnable() const
{
    return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X5_RX::BusMasterSize() --
//---------------------------------------------------------------------------

void X5_RX::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_RX::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_RX::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_RX::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_RX::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_RX::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_RX::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_RX::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_RX::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_RX::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X5_RX::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo   2 constant RefVcxo");

    engine.Evaluate(": ?X5-RX "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   PLL  Vcxo  RefVcxo \" "
                    "    cr  "
                    "    cr .\" ---- X5-RX Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X5-RX  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("X5-RX-specific wordset loaded \n" );

    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X5_RX::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X5_RX::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X5_RX::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X5_RX::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X5_RX::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X5_RX::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X5_RX::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X5_RX::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X5_RX::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X5_RX::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;   const int kRefVcxo = 2;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Pll().Port(addr);
        case kVcxo:
            return Vcxo().Port(addr);
        case kRefVcxo:
            return RefVcxo().Port(addr);
        }
}

//---------------------------------------------------------------------------
//  X5_RX::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X5_RX::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;   const int kRefVcxo = 2;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Pll().Port(addr, value);
            break;
        case kVcxo:
            Vcxo().Port(addr, value);
            break;
        case kRefVcxo:
            RefVcxo().Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_RX::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X5_RX::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;   const int kRefVcxo = 2;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Pll().Frequency(freq);
            Pll().Reapply();
            break;
        case kVcxo:
            Vcxo().Frequency(freq);
            break;
        case kRefVcxo:
            RefVcxo().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_RX::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X5_RX::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;   const int kRefVcxo = 2;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(Vcxo().FrequencyActual());
        case kRefVcxo:
            return static_cast<float>(RefVcxo().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
