// X5_TXDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X5_TX_Mb.h>
#include <X5_TXDeviceMap_Mb.h>
#include <X5MemoryMap_Mb.h>
#include <X5_TXMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>
#include <Magic_Mb.h>
#include <sstream>
#include <cmath>
#include <Exception_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X5_MemoryMap;
using namespace X5_TxMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxOutDevices     = 4;
const int    OutputWidth_Bits  = 16;
const int    OutputMuxCount    = 0;
const bool   OutputIsUnipolar  = false;
const int    OutputClockFactor = 1;
const float  OutputMaxRate     = 1000.0e6;
const float  OutputMinRate     = 16.0e6;
const int    OutputGangSize    = 2;

const float  Input_ReferenceClk = 100.0e6;
const float  Output_VcoMinClk    = 979.0e6f;
const float  Output_VcoMaxClk    = 1005.0e6f;

//---------------------------------------------------------
//  IDROM Section Map
//  ...First section: Input and Output Coefficient section
//       4 channels x 2 values + 1 calibration flag + 1 spare
const int romX5_Tx_OutputCoeffOffset  = 0;
const int romX5_Tx_OutChannels        = 4;
const int romX5_Tx_OutputCoeffSize    = romX5_Tx_OutChannels * 2 + 1 + 1;
const int romX5_Tx_CoeffSize          = romX5_Tx_OutputCoeffSize;


//===========================================================================
// class X5_TxOutputInfoIntf
//===========================================================================

class X5_TxOutputInfoIntf : public IoInfoIntf
{
public:
	X5_TxOutputInfoIntf()
        : //    devices,           bits,    muxChannelCount,   unipolar
          FChannels(MaxOutDevices, OutputWidth_Bits, OutputMuxCount, OutputIsUnipolar)
		{  FSpan.push_back(SpanInfo(1000, -1000));  }
	virtual ~X5_TxOutputInfoIntf()  {  }

	virtual UnpairedChannelInfo &       Channels()              {  return FChannels; }
	virtual float                       MaxRate() const         {  return OutputMaxRate; }
	virtual float                       MinRate() const         {  return OutputMinRate; }
	virtual const SpanArray &           Span() const            {  return FSpan;  }
	virtual int                         GangSize() const        {  return OutputGangSize; }
	virtual bool                        IsStackable() const     {  return true;  }
	virtual bool                        IsInverted() const      {  return true;  }
	virtual bool                        IsMuxed() const         {  return false;  }
	virtual bool                        MuxDepth() const        {  return OutputMuxCount;  }
	virtual int                         BufferSize() const      {  return 65536;  }
	virtual int                         ClockFactor() const     {  return OutputClockFactor;  }
	virtual float                       ReferenceClock() const  {  return Input_ReferenceClk;  }
	virtual int               	        SamplesPerFrame() const {  return 256;  }

private:
	UnpairedChannelInfo         FChannels;
	SpanArray                   FSpan;
};

//==============================================================================
//  CLASS X5_Tx_Ad9511 -- X5_Tx specific 9511 implementation
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class X5_Tx_Ad9511
//---------------------------------------------------------------------------

X5_Tx_Ad9511::X5_Tx_Ad9511(X5_TxDeviceMap * owner, int index, AddressingSpace &space, int ctl)
    :  Ad9511Pfd(index, space, ctl), Owner(owner)
{
    UseAdvancedPBACalculation(true);
}

//---------------------------------------------------------------------------
//  constructor for class X5_Tx_Ad9511
//---------------------------------------------------------------------------

X5_Tx_Ad9511::X5_Tx_Ad9511(X5_TxDeviceMap * owner, double freq, double ref, bool enabled, int index,
                             AddressingSpace & space, int ctl)
    :  Ad9511Pfd(freq, ref, enabled, index, space, ctl), Owner(owner)
{
    UseAdvancedPBACalculation(true);
}

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::Port()  -- Port Write
//---------------------------------------------------------------------------

void  X5_Tx_Ad9511::Port(unsigned char reg, unsigned char value)
{
    Owner->Output().PllSpi(reg, value);
}

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::Port()  -- Port Write
//---------------------------------------------------------------------------

unsigned char X5_Tx_Ad9511::Port(unsigned char reg)
{
    return Owner->Output().PllSpi(reg);
}

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::ConfigureOutputs()  -- Tx specific output config
//---------------------------------------------------------------------------

void X5_Tx_Ad9511::ConfigureOutputs(int outdiv_bits, int outdiv_bypass)
{
    PllConfigureOutputEvent e(*this, outdiv_bits, outdiv_bypass);
    OnPllConfigureOutput.Execute(e);

    // Override base class ClkOut write (we power off unused items)
    unsigned char ClkOut = (FOutputClock==oVco)? 0x2 : 0x1d;
    Port(0x45, ClkOut);

    Port(0x07, 0x04);  // LOR enable

    Port(0x08, 0x7b);  // TEST! (put in base class later)

    //  Fine Delays
    //  -- Delay Bypass 5
    Port(0x34, 0x01);
    //  -- Delay Full-Scale 5
    Port(0x35, 0x00);
    //  -- Delay Fine Adjust 5
    Port(0x36, 0x00);
    //  -- Delay Bypass 6
    Port(0x38, 0x01);
    //  -- Delay Full-Scale 6
    Port(0x39, 0x00);
    //  -- Delay Fine Adjust 6
    Port(0x3a, 0x00);

    // Identical duty cycle and phase for all outputs
    Port(0x48, outdiv_bits);	// duty cycle 0
    Port(0x49, outdiv_bypass);	// phase

    Port(0x4a, outdiv_bits);	// duty cycle 1
    Port(0x4b, outdiv_bypass);	// phase

    Port(0x4d, 0x80);  // (off)
    Port(0x4f, 0x80);  // (off)

    int outdiv_50;
    int outdiv_52;
    int outdiv_54;

    if (FOutputClock==oVco)
        {  // internal
        outdiv_50 = PllOutputDividers(Owner->Output().Device(0).SingleActiveChannel(),
                                                  outdiv );
        outdiv_52 = PllOutputDividers(Owner->Output().Device(1).SingleActiveChannel(),
                                                  outdiv );
        }
    else
        {  // external clock
		double dclk0 = DCLK(Owner->Output().Device(0).SingleActiveChannel());
		if (std::fabs(dclk0-Frequency()/1e6) < 1.0)   // "equal freqs"
            outdiv_50 = 1;  // divide by 1 (bypass)
        else if (dclk0 < Frequency()/1e6)
            outdiv_50 = 2;  // divide by 2
        else
            outdiv_50 = 1;  // divide by 1 (bypass)

		double dclk1 = DCLK(Owner->Output().Device(1).SingleActiveChannel());
		if (std::fabs(dclk1-Frequency()/1e6) < 1.0)  // "equal freqs"
            outdiv_52 = 1;  // divide by 1 (bypass)
        else if (dclk1 < Frequency()/1e6)
            outdiv_52 = 2;  // divide by 2
        else
            outdiv_52 = 1;  // divide by 1 (bypass)
        }

    //  For Rev C, slave clock 6 to clock 4 or 5
    if (Owner->Output().Device(0).Enabled())
        outdiv_54 = outdiv_50;      // slave to Device 0
    else
        outdiv_54 = outdiv_52;      // slave to Device 1

    //  For Rev C Calibration, make sure ~inactive~ device clock matches active
    if (Owner->Output().Device(0).Enabled()==false)
        outdiv_50 = outdiv_52;
    if (Owner->Output().Device(1).Enabled()==false)
        outdiv_52 = outdiv_50;

    Port(0x50, ToDividerValue(outdiv_50));	// duty cycle 4
    Port(0x51, ToBypassValue(outdiv_50));	// phase

    Port(0x52, ToDividerValue(outdiv_52));	// duty cycle 5
    Port(0x53, ToBypassValue(outdiv_52));	// phase

    if (Owner->PciLogicPcb() >= 2)   // Rev C or later
	    {
        Port(0x54, ToDividerValue(outdiv_54));	// duty cycle 6
        Port(0x55, ToBypassValue(outdiv_54));	// phase
        }
    else
        LogicDividerSetting(0x54, 0x55);  // duty cycle 6  (125-400)

    Port(0x57, 0x80);	// (off)

    Port(0x3C, 0x08);       // Clock 0 LVPECL on @810mV
    Port(0x3D, 0x08);       // Clock 1 LVPECL on @805mV
    Port(0x3E, 0x0b);       // Clock 2 off
    Port(0x3F, 0x0b);       // Clock 3 off

    Port(0x40, 0x02);       // Clock 4 LVDS
    Port(0x41, 0x02);       // Clock 5 LVDS
    Port(0x42, 0x02);       // Clock 6 LVDS
    Port(0x43, 0x03);       // Clock 7 disabled

    PllConfigureOutputEvent e1(*this, outdiv_bits, outdiv_bypass);
    OnAfterPllConfigureOutput.Execute(e1);
}

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::LogicDividerSetting()  -- Logic Clock Divider Programming
//---------------------------------------------------------------------------

void X5_Tx_Ad9511::LogicDividerSetting(int div_register, int bypass_register)
{
	double VcoFrequency = FrequencyActual()*outdiv;
	const double LogicOutputFrequency = 125.0e6;
	
	int candidateDiv = static_cast<int>(VcoFrequency/LogicOutputFrequency);
	if (candidateDiv<=1)
		candidateDiv = 1;
	else if (candidateDiv>32)
		candidateDiv = 32;
	else
		{
		candidateDiv /= 2;
		candidateDiv *= 2;   // Must be even divider between 2 and 32
		}
			
    Port(div_register,    ToDividerValue(candidateDiv));  // duty cycle 6  (125-400)
    Port(bypass_register, ToBypassValue(candidateDiv));	  // phase
}    

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::DCLK()  -- DCLK Output Rate (Mhz)
//---------------------------------------------------------------------------

double X5_Tx_Ad9511::DCLK(bool single_channel)
{
    double FreqMhz = Frequency()/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return FreqMhz/2;
        else
            return FreqMhz;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return FreqMhz;
        else
            return FreqMhz*2;
        }
}

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::PllOutputDividers()  -- Output Divider Settings
//---------------------------------------------------------------------------

int X5_Tx_Ad9511::PllOutputDividers(bool single_channel, int outdiv)
{
    double FreqMhz = Frequency()/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return outdiv * 2;
        else
            return outdiv;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return outdiv;
        else
            return outdiv / 2;
        }
}

//---------------------------------------------------------------------------
//  X5_Tx_Ad9511::IsDacDllEnabled()  -- Output Divider Settings
//---------------------------------------------------------------------------

bool  X5_Tx_Ad9511::IsDacDllEnabled(bool single_channel)
{
    double FreqMhz = Frequency()/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return true;
        else
            return false;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return true;
        else
            return false;
        }
}

//===========================================================================
//  CLASS X5_TxDacDevice  -- Per-DAC controls for X5-TZ
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X5_TxDacDevice
//---------------------------------------------------------------------------

X5_TxDacDevice::X5_TxDacDevice(X5_TxDeviceMap * owner,
							   X5_TxDacDeviceRegisters & reg)
    :  TriggerReg(owner->LogicMemory, reg.TriggerControlRegister_Addr),
       SpiWriteReg(owner->LogicMemory, reg.SpiInterfaceRegister_Addr),
       SpiReadReg(owner->LogicMemory, reg.SpiInterfaceRegister_Addr),
       Owner(owner), FEnabled(false), FDacDelay(0)

{
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::SingleActiveChannel() -- True if in single channel mode
//---------------------------------------------------------------------------

bool  X5_TxDacDevice::SingleActiveChannel()
{
    return (FChannelEnabled[0] ^ FChannelEnabled[1]);
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::PreconfigHardware() --
//---------------------------------------------------------------------------

void  X5_TxDacDevice::PreconfigHardware()
{
    double DCLK = Owner->Clock.DCLK(SingleActiveChannel());
    bool DacDllEnabled = Owner->Clock.IsDacDllEnabled(SingleActiveChannel());
    //
    //  Enable devices and/or triggers
    if (SingleActiveChannel())
        {
        //  Single Channel Mode
        int active_chan = FChannelEnabled[1];
        SingleChannelMode(active_chan);

        if (DacDllEnabled)
            Spi(0x06, 0x2e);
        else
            Spi(0x06, 0x2f);
        }
    else
        {
        //  Dual channel mode
        DualChannelMode();
        if (DacDllEnabled)
            Spi(0x06, 0x0e);
        else
            Spi(0x06, 0x0f);
        }

    //  Spi Setup
    //  ...Address 0
        // ?
    //  ...Address 1
    //Spi(0x01, (FDacDelay<<6) | 0x01);   // done later
    //  ...Address 2
    //      2's complement + single/dual channel mode (see above)
    //  ...Address 3
    //      (disable DAC offset, and allow SLFTST FIFO and Pattern errors)
	//   Spi(0x03, 0x00);
//    std::stringstream msg;
//    msg << std::hex << "TX Dac Spi 0x3 = 0x" << Spi(0x03);
//    Debug.Log(msg.str());

    Spi(0x03, 0x01);
    //  ...Address 4
    //       clear all errors
    //Spi(0x04, 0x00);          // done later
    //  ...Address 5
    //       (bypass DLL)
    if (DacDllEnabled)
        Spi(0x05, 0x02);
    else
        Spi(0x05, 0x06);
    //  ...Address 6
    //        put PLL in sleep mode
    //        (see above)
    //  ...Address 7
    //        Gain Adjustment (GAIN_A = GAIN_B = 15)
    Spi(0x07, 0xff);
    //  ...Address 9
    //  ???
    //  ...Address 10
    Spi10Setup(DCLK);
    //  ...Address 11
    Spi(0x0b, 0x00);

    Spi(0x0e, 0x40);

    LoadDacDelay( FDacDelay );
    //  ...Address 8
    //      Toggle DLL Restart
    if (DacDllEnabled)
        {
        Spi(0x08, 0x04);
        Spi(0x08, 0x00);
        }
    Sleep(1);

    Spi(0x03, 0x03);

}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  X5_TxDacDevice::ConfigureHardware()
{
    ExternalTriggerSource(extSync0);
    //  Pre streaming configuration
	//
    //  Configure triggers
	TriggerReg.Apply();

}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::Spi10Setup() -- Set to single channel mode
//---------------------------------------------------------------------------

void  X5_TxDacDevice::Spi10Setup(double DCLK)
{
    int fast_cfg_value;
    if (Owner->PciLogicPcb() >= 2)   // Rev C or later
        fast_cfg_value = 0xC0;
    else
        fast_cfg_value = 0x00;

    if (DCLK <= 125.0)
        Spi(0x0a, 0x00);
    else if (DCLK <= 150.0)
        Spi(0x0a, 0xcd);
    else if (DCLK <= 175.0)
        Spi(0x0a, 0xce);
    else if (DCLK <= 200.0)
        Spi(0x0a, 0xcf);
    else if (DCLK <= 300.0)
        Spi(0x0a, 0xc8);
    else if (DCLK <= 500.0)
        Spi(0x0a, fast_cfg_value);
    else
        Spi(0x0a, fast_cfg_value);

}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::SingleChannelMode() -- Set to single channel mode
//---------------------------------------------------------------------------

void  X5_TxDacDevice::SingleChannelMode(int /*active_chan*/)
{
    //  we actually can't set a single channel to Channel 1...use 0 for
    //    both single channel modes
    //  Set 2's complement (0x80)
    //  NOT dual channel (0x40)
    Spi(0x02, 0x80);

}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::DualChannelMode() -- Set to dual channel mode
//---------------------------------------------------------------------------

void  X5_TxDacDevice::DualChannelMode()
{
    //  Set 2's complement (0x80)
    //  Set dual channel (0x40)
    Spi(0x02, 0xc0);

}
//---------------------------------------------------------------------------
//  X5_TxDacDevice::Enabled() -- Enable device
//---------------------------------------------------------------------------

void  X5_TxDacDevice::Enabled(bool value)
{
    FEnabled = value;
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::ChannelEnabled() -- Enable channel
//---------------------------------------------------------------------------

void  X5_TxDacDevice::ChannelEnabled(int idx, bool value)
{
    FChannelEnabled[idx] = value;
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::LoadDacDelay() -- Set Dac Delay parameter
//---------------------------------------------------------------------------

void  X5_TxDacDevice::LoadDacDelay(unsigned int value)
{
    FDacDelay = value;
    Spi(0x01, (value<<6));
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::Spi() --  SPI Read
//---------------------------------------------------------------------------

unsigned char X5_TxDacDevice::Spi(int addr)
{
    SpiReady();

    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Direction.Value(spiRead);
    SpiWriteReg.Apply();

    SpiNotReady();
    SpiReady();

    unsigned char result = static_cast<unsigned char>(SpiReadReg.ReadData.Value());

    return result;
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::Spi()  --  SPI Write
//---------------------------------------------------------------------------

void X5_TxDacDevice::Spi(int addr, unsigned char value)
{
//  DEBUG DEBUG give time to settle before all SPI accesses
//    Sleep(10);

    SpiReady();

    // Write to D/A register via SPI register
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Data.Value(value);
    SpiWriteReg.Direction.Value(spiWrite);
    SpiWriteReg.Apply();

    SpiNotReady();
    SpiReady();

    std::stringstream msg;
    msg << std::hex << "Spi(0x" << addr << ", 0x" << (short)value << ") = [0x" << SpiReadReg.Value() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::SpiReady()
//---------------------------------------------------------------------------

void X5_TxDacDevice::SpiReady()
{
    //Sleep(2);

    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (SpiReadReg.Ready.Value())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << "TX Dac Spi Ready Failure (>5us)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::SpiNotReady()
//---------------------------------------------------------------------------

void X5_TxDacDevice::SpiNotReady()
{
    //Sleep(2);
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!SpiReadReg.Ready.Value())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << "TX Dac Spi NOT Ready Failure (>5us)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::DacStatus() --  Return DAC status bit
//---------------------------------------------------------------------------

bool X5_TxDacDevice::DacStatus() const
{
    return SpiReadReg.DacStatus.Value();
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::ExternalTrigger() --  Set external trigger enable
//---------------------------------------------------------------------------

void X5_TxDacDevice::ExternalTrigger(bool state)
{
    TriggerReg.ExtTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::ExternalTrigger() --  Query ext trigger enable
//---------------------------------------------------------------------------

bool X5_TxDacDevice::ExternalTrigger() const
{
    return TriggerReg.ExtTrigger.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::ExternalTriggerSource() --  Set external trigger enable
//---------------------------------------------------------------------------

void X5_TxDacDevice::ExternalTriggerSource(ExtTriggerSources state)
{
    TriggerReg.ExtTriggerSource.Value((state==extSync1)? true : false);
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::ExternalTriggerSource() --  Query ext trigger enable
//---------------------------------------------------------------------------

X5_TxDacDevice::ExtTriggerSources X5_TxDacDevice::ExternalTriggerSource() const
{
    return ExtTriggerSources(TriggerReg.ExtTriggerSource.Value());
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::FrameMode() --  Set frame mode enable
//---------------------------------------------------------------------------

void X5_TxDacDevice::FrameMode(bool state)
{
    TriggerReg.FramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::FrameMode() --  Query frame mode enable
//---------------------------------------------------------------------------

bool X5_TxDacDevice::FrameMode() const
{
    return TriggerReg.FramedMode.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::FrameSize() --  Set frame size
//---------------------------------------------------------------------------

void X5_TxDacDevice::FrameSize(unsigned int size)
{
    size >>=2;
    size <<=2;  //  size must be multiple of 4
    TriggerReg.FrameCount.Value(size);
}

//---------------------------------------------------------------------------
//  X5_TxDacDevice::FrameSize() --  Query frame mode enable
//---------------------------------------------------------------------------

unsigned int  X5_TxDacDevice::FrameSize() const
{
    return TriggerReg.FrameCount.Value();
}

//===========================================================================
//  CLASS X5_TxOutputDevice  -- Register & low level support for X5
//===========================================================================
//===========================================================================

static X5_TxDacDeviceRegisters X5_TxDac0Registers =
	{ mmDac0Spi, mmDac0TriggerControl };

//===========================================================================

static X5_TxDacDeviceRegisters X5_TxDac1Registers =
	{ mmDac1Spi, mmDac1TriggerControl  };

//---------------------------------------------------------------------------
//  constructor for class X5_TxOutputDevice
//---------------------------------------------------------------------------

X5_TxOutputDevice::X5_TxOutputDevice(X5_TxDeviceMap * owner)
	:  IUsesSectionCalibration(	PmcIdromSection(owner->Rom(),
								                PmcIdrom::waModule,
								                romX5_Tx_OutputCoeffOffset,
								                romX5_Tx_OutputCoeffSize),
                                romX5_Tx_OutChannels,
                                0x10000),	// 'one' value for gain registers
       Owner(owner),
       Dac0(owner, X5_TxDac0Registers),
       Dac1(owner, X5_TxDac1Registers),
       FInfo(new X5_TxOutputInfoIntf),
	   TestPhaseReg(owner->LogicMemory, mmGeneratorCoeff),
	   TestControlReg(owner->LogicMemory, mmGeneratorControl),
       SpiWriteReg(owner->LogicMemory, mmPllSpi),
       SpiReadReg(owner->LogicMemory, mmPllSpi),
       SoftTriggerReg(owner->LogicMemory, mmSoftwareTrigger),
       DacControlReg(owner->LogicMemory, mmOutControl),
       DacStatusReg(owner->LogicMemory, mmOutControl),
       GainCoeffReg(owner->LogicMemory, mmOutGainCoef, MaxOutDevices),
       OffsetCoeffReg(owner->LogicMemory, mmOutOffsetCoef, MaxOutDevices),
       PacketHeaderReg(owner->LogicMemory, mmDacPacketHeader),
       PatternModeCtlReg(owner->LogicMemory, mmPatternModeCtl),
       LoopbackPacketHeaderReg(owner->LogicMemory, mmLoopbackPacketHeader),
       OutputCalibrationReg(owner->LogicMemory, mmDacCalCtrl),
       FTestMode(tmRamp),
       FTestEnabled(false),
       FTestFrequency(10.0e6)
{
}

//---------------------------------------------------------------------------
//  destructor for class X5_TxOutputDevice
//---------------------------------------------------------------------------

X5_TxOutputDevice::~X5_TxOutputDevice()
{
    delete FInfo; FInfo = 0;     // owned object
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::TestMode()
//---------------------------------------------------------------------------

unsigned int X5_TxOutputDevice::TestMode()const
{
	return FTestMode;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::TestMode()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::TestMode(unsigned int mode)
{
	TestControlReg.Mode.Value(mode);
	FTestMode = (IITestMode)mode;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::TestEnable()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::TestEnable(bool enable)
{
	TestControlReg.Enable.Value(enable);
	FTestEnabled = enable;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::TestEnable()
//---------------------------------------------------------------------------

bool X5_TxOutputDevice::TestEnable() const
{
	return FTestEnabled;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::TestFrequency()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::TestFrequency(float freq)
{
    // Sample clock, in Mhz
	const double FClk = Owner->Clock.Frequency();

	FTestFrequency = freq;
    //  If a 2 channel run, and both on same pair we generate points effectively
    //     twice as fast
    unsigned int divider = 1;
    if (Device(0).Enabled() && !Device(1).Enabled())
        if (Device(0).SingleActiveChannel()==false)
            divider = 2;
    if (Device(1).Enabled() && !Device(0).Enabled())
        if (Device(1).SingleActiveChannel()==false)
            divider = 2;
    if (Device(1).Enabled() && Device(0).Enabled())
        if (Device(0).SingleActiveChannel()==false)
            divider = 2;

	unsigned int phase = static_cast<unsigned int>(freq * ((1 << 24) / FClk));
    //  Adjust phase
    phase /= divider;
	TestPhaseReg.PhaseIncrement.Value(phase);
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::TestFrequency()
//---------------------------------------------------------------------------

float X5_TxOutputDevice::TestFrequency() const
{
	return FTestFrequency;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PllSpi()  -- Read PLL Spi
//---------------------------------------------------------------------------

unsigned char X5_TxOutputDevice::PllSpi(int addr)
{
    PllSpiReady();

    // Set up a SPI read
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Direction.Value(spiRead);
    SpiWriteReg.Apply();

    PllSpiNotReady();
    PllSpiReady();

    unsigned char result = static_cast<unsigned char>(SpiReadReg.ReadData.Value());

    return result;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PllSpi()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::PllSpi(int addr, unsigned char value)
{
//  DEBUG DEBUG give time to settle before all SPI accesses
//    Sleep(10);

    PllSpiReady();

    // Write to D/A register via SPI register
    SpiWriteReg.Address.Value(addr);
    SpiWriteReg.Data.Value(value);
    SpiWriteReg.Direction.Value(spiWrite);
    SpiWriteReg.Apply();

    PllSpiNotReady();
    PllSpiReady();

    std::stringstream msg;
    msg << std::hex << "Pll Spi(0x" << addr << ", 0x" << (short)value << ") = [0x" << SpiReadReg.Value() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PllSpiReady()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::PllSpiReady()
{
    //Sleep(2);

    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (SpiReadReg.Ready.Value())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << "TX PLL Spi Ready Failure (>5us)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PllSpiNotReady()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::PllSpiNotReady()
{
    //Sleep(2);
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (!SpiReadReg.Ready.Value())
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << "TX PLL Spi NOT Ready Failure (>5us)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PacketId() -- Set ID tag of received data packets
//---------------------------------------------------------------------------

void  X5_TxOutputDevice::PacketId(unsigned char id)
{
    PacketHeaderReg.Id.Value(id);
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PacketId() -- Query ID tag of received data packets
//---------------------------------------------------------------------------

unsigned char X5_TxOutputDevice::PacketId() const
{
	return PacketHeaderReg.Id.Value();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

void  X5_TxOutputDevice::PacketSize(int size_words)
{
   PacketHeaderReg.Size.Value(size_words);
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

int  X5_TxOutputDevice::PacketSize() const
{
	return PacketHeaderReg.Size.Value();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::LoopbackPacketId() -- Set ID tag of received data packets
//---------------------------------------------------------------------------

void  X5_TxOutputDevice::LoopbackPacketId(unsigned char id)
{
    LoopbackPacketHeaderReg.Id.Value(id);
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::LoopbackPacketId() -- Query ID tag of received data packets
//---------------------------------------------------------------------------

unsigned char X5_TxOutputDevice::LoopbackPacketId() const
{
	return LoopbackPacketHeaderReg.Id.Value();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::LoopbackPacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

void  X5_TxOutputDevice::LoopbackPacketSize(int size_words)
{
    LoopbackPacketHeaderReg.Size.Value(size_words);
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::LoopbackPacketSize() -- Set size of received data packets
//---------------------------------------------------------------------------

int  X5_TxOutputDevice::LoopbackPacketSize() const
{
	return LoopbackPacketHeaderReg.Size.Value();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::ConfigureChannelEnables()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::ConfigureChannelEnables()
{
	//
	//  Enable Devices and Channels
    Device(0).Enabled( Info().Channels().Enabled(0) || Info().Channels().Enabled(1));
    Device(0).ChannelEnabled(0, Info().Channels().Enabled(0));
    Device(0).ChannelEnabled(1, Info().Channels().Enabled(1));

    Device(1).Enabled( Info().Channels().Enabled(2) || Info().Channels().Enabled(3));
    Device(1).ChannelEnabled(0, Info().Channels().Enabled(2));
    Device(1).ChannelEnabled(1, Info().Channels().Enabled(3));
    // special case to avoid 3 channel case?

}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::PreconfigHardware()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::PreconfigHardware()
{
    // Set Logic Channel Enables
    unsigned int ChEnableField(0);
    ChEnableField |= (Info().Channels().Enabled(0)) ? 0x1 : 0x00 ;
    ChEnableField |= (Info().Channels().Enabled(1)) ? 0x2 : 0x00 ;
    ChEnableField |= (Info().Channels().Enabled(2)) ? 0x4 : 0x00 ;
    ChEnableField |= (Info().Channels().Enabled(3)) ? 0x8 : 0x00 ;

    DacControlReg.ChannelEnable.Value(ChEnableField);

    //  Call config for "SubDevices"
    Dac0.PreconfigHardware();
    Dac1.PreconfigHardware();

}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void X5_TxOutputDevice::ConfigureHardware()
{
    PacketSize(PacketSize());
    PacketId(PacketId());

    //  Call config for "SubDevices"
    Dac0.ConfigureHardware();
    Dac1.ConfigureHardware();

    TestMode(FTestMode);
    TestFrequency(FTestFrequency);
    TestEnable(FTestEnabled);

    Dac0.ClearDacTestStatus();
    Dac1.ClearDacTestStatus();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::SoftwareTrigger() -- Set Sw Trigger Bits
//---------------------------------------------------------------------------

void X5_TxOutputDevice::SoftwareTrigger(bool state, SwTriggerState which)
{
    switch (which)
        {
        case swDevice0:
            SoftTriggerReg.Dev0_Trigger.Value(state);
            break;
        case swDevice1:
            SoftTriggerReg.Dev1_Trigger.Value(state);
            break;
        case swAll:
            SoftTriggerReg.Dev0_Trigger.Value(state);
            SoftTriggerReg.Dev1_Trigger.Value(state);
            break;
        }

    SoftTriggerReg.Apply();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::DeviceRun() -- Set Device Engine Run Bits
//---------------------------------------------------------------------------

void X5_TxOutputDevice::DeviceRun(bool state, EngDeviceState which)
{
    switch (which)
        {
        case engDevice0:
            DacControlReg.Dev0_Run.Value(state);
            break;
        case engDevice1:
            DacControlReg.Dev1_Run.Value(state);
            break;
        case engAll:
            DacControlReg.Dev0_Run.Value(state);
            DacControlReg.Dev1_Run.Value(state);
            break;
        }

    DacControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::Reset() -- Set Device Engine Reset bit
//---------------------------------------------------------------------------

void X5_TxOutputDevice::Reset(bool state)
{
    DacControlReg.Reset.Value(state);
    DacControlReg.Apply();
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::SetCalibrationOffset()  --
//---------------------------------------------------------------------------

void X5_TxOutputDevice::SetCalibrationOffset(int ch, int value)
{
    OffsetCoeffReg[ch] = value;
}

//---------------------------------------------------------------------------
//  X5_TxOutputDevice::SetCalibrationGain() --
//---------------------------------------------------------------------------

void X5_TxOutputDevice::SetCalibrationGain(int ch, int value)
{
    GainCoeffReg[ch].Value(value);
}

//------------------------------------------------------------------------------
//  X5_TxOutputDevice::PatternModeLoopback() --
//------------------------------------------------------------------------------

void X5_TxOutputDevice::PatternModeLoopback(bool state)
{
    PatternModeCtlReg.PatternLoopbackEnable.Value(state);
}

//------------------------------------------------------------------------------
//  X5_TxOutputDevice::PatternModeLoopback() --
//------------------------------------------------------------------------------

bool X5_TxOutputDevice::PatternModeLoopback() const
{
    return PatternModeCtlReg.PatternLoopbackEnable.Value();
}

//------------------------------------------------------------------------------
//  X5_TxOutputDevice::DoInternalCalibration() --
//------------------------------------------------------------------------------

void X5_TxOutputDevice::DoInternalCalibration()
{

    // trigger in software, unframed mode
    Dac0.ExternalTrigger(false);     Dac1.ExternalTrigger(false);
    Dac0.FrameMode(false);           Dac1.FrameMode(false);

    if (Dac0.Enabled())
        {
        if (Dac0.SingleActiveChannel())
            TestMode(0x6);
        else
            TestMode(0x7);
        }
    else  // Use Dac 1
        {
        if (Dac1.SingleActiveChannel())
            TestMode(0x6);
        else
            TestMode(0x7);
        }
    EngDeviceState which = engAll;
    if (Dac0.Enabled()==false)
        which = engDevice1;
    if (Dac1.Enabled()==false)
        which = engDevice0;

    TestEnable(true);
    DeviceRun(true, which);                             // start Cal
    OutputCalibrationReg.CalEnable.Value(true);  // enter CAL mode

    Sleep(5);    // wait till done

    DeviceRun(false, which);                             // stop Cal
    OutputCalibrationReg.CalEnable.Value(false);  // leave CAL mode
    TestEnable(false);

}

//------------------------------------------------------------------------------
//  X5_TxOutputDevice::GetInternalCalibrationResults() --
//------------------------------------------------------------------------------

unsigned int  X5_TxOutputDevice::GetInternalCalibrationResults()
{
    return OutputCalibrationReg.CalResults.Value();
}

//------------------------------------------------------------------------------
//  X5_TxOutputDevice::InternalDacCalibration() --
//------------------------------------------------------------------------------

bool  X5_TxOutputDevice::InternalDacCalibration()
{
    return Owner->FInternalCal;
}

//=============================================================================
//  CLASS X5_TxClock  --  PLL timebase control
//=============================================================================

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

X5_TxClock::X5_TxClock(X5_TxDeviceMap * owner, int index, AddressingSpace &space,
    int ctl)
    : Owner(owner), Clock(owner, index, space, ctl), ExternalRefFreq(100e6)
{
    Clock.Reference(Input_ReferenceClk);
    Clock.Min(Output_VcoMinClk/Clock.MaxOutputDivider());
    Clock.Max(Output_VcoMaxClk);
    Clock.Frequency(200e6);
}

//---------------------------------------------------------------------------
//  Constructor
//---------------------------------------------------------------------------

X5_TxClock::X5_TxClock(X5_TxDeviceMap * owner, double freq, double ref,
    bool enabled, int index, AddressingSpace & space, int ctl)
    : Owner(owner), Clock(owner, freq, ref, enabled, index, space, ctl)
{
    Clock.Reference(Input_ReferenceClk);
    Clock.Min(Output_VcoMinClk/Clock.MaxOutputDivider());
    Clock.Max(Output_VcoMaxClk);
}

//---------------------------------------------------------------------------
//  X5_TxClock::Frequency() -- Change output frequency
//---------------------------------------------------------------------------

void X5_TxClock::Frequency(double value)
{
    Clock.ClockFactor(1.0);
    Clock.Frequency(value);
}

//---------------------------------------------------------------------------
//  X5_TxClock::Frequency() -- Query output frequency
//---------------------------------------------------------------------------

double X5_TxClock::Frequency() const
{
    return Clock.Frequency();
}

//---------------------------------------------------------------------------
//  X5_TxClock::FrequencyActual() -- Query actual output frequency
//---------------------------------------------------------------------------

double X5_TxClock::FrequencyActual() const
{
    return Clock.FrequencyActual();
}

//---------------------------------------------------------------------------
//  X5_TxClock::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

X5_TxClock::IIClockSource X5_TxClock::Source() const
{
    if (Clock.OutputClock() == Ad9511::oVco)
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  X5_TxClock::Source() -- Select clock source
//---------------------------------------------------------------------------

void X5_TxClock::Source(IIClockSource src)
{
    if (src == csInternal)
        Clock.OutputClock(Ad9511::oVco);
    else
        Clock.OutputClock(Ad9511::oExternal);
}

//---------------------------------------------------------------------------
//  X5_TxClock::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

X5_TxClock::IIReferenceSource X5_TxClock::Reference() const
{
    if (Clock.ReferenceClock() == Ad9511::rInternal)
        return rsInternal;
    else
        return rsExternal;
}

//---------------------------------------------------------------------------
//  X5_TxClock::Reference() -- Select reference source
//---------------------------------------------------------------------------

void X5_TxClock::Reference(IIReferenceSource src)
{
    if (src == rsInternal)
        Clock.ReferenceClock(Ad9511::rInternal);
    else
        Clock.ReferenceClock(Ad9511::rExternal);
}

//---------------------------------------------------------------------------
//  X5_TxClock::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void X5_TxClock::ReferenceFrequency(double value)
{
    ExternalRefFreq = value;
	if (Reference() == rsInternal)
        Clock.Reference(Owner->Output().Info().ReferenceClock());
    else
        Clock.Reference(value);
}

//---------------------------------------------------------------------------
//  X5_TxClock::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double X5_TxClock::ReferenceFrequency() const
{
	if (Reference() == rsInternal)
        return Clock.Reference();
    else
        return ExternalRefFreq;
}

//---------------------------------------------------------------------------
//  X5_TxClock::Reapply() -- Reapply all clock settings
//---------------------------------------------------------------------------

void X5_TxClock::Reapply()
{
	// Route clocks to active analog devices
    Source(Source());
    Reference(Reference());

	// Set reference based on clock source to obtain correct FrequencyActual
	if (Reference() == rsInternal)
        if (Source() == csExternal)
    	    {
            Clock.Reference(Frequency());
            Owner->VcoOutputEnable(false);
            }
	    else
		    {
            Clock.Reference(Owner->Output().Info().ReferenceClock());
            Owner->VcoOutputEnable(true);
            }
    else
        ReferenceFrequency(ReferenceFrequency());

    Clock.Reapply();
}

//---------------------------------------------------------------------------
//  X5_TxClock::Locked() -- Return PLL lock status
//---------------------------------------------------------------------------

bool X5_TxClock::Locked() const
{
    return Clock.Locked();
}

//---------------------------------------------------------------------------
//  X5_TxClock::Vcxo() -- Return Vcxo
//---------------------------------------------------------------------------
 Si570 &X5RawClockDevices::Vcxo()     
 {  
	 return Owner->Vcxo;  
 }

//---------------------------------------------------------------------------
//  X5_TxClock::Pll() -- Return PLL 
//---------------------------------------------------------------------------
 X5_Tx_Ad9511 &X5RawClockDevices::Pll()      
 {  
	 return Owner->Clock.Clock;  
 }

//===========================================================================
//  CLASS X5_TxDeviceMap  -- Register & low level support for X5-Tx module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X5_TxDeviceMap
//---------------------------------------------------------------------------

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)
const int     VcxoI2C_SubAddr  = 0xAA;    //

//---------------------------------------------------------------------------
//  constructor for class X5_TxDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X5_TxDeviceMap::X5_TxDeviceMap(X5_Tx * owner)
	:  IUsesStreamDeviceEventCallbacks(owner),
	   FOutput(new X5_TxOutputDevice(this)),
	   FLowResolution(false),
       Owner(owner), FInternalCal(true),
       Clock(this, 0, LogicMemory, mmPllControl),
	   VcxoI2CReg( LogicMemory, mmVcoI2C),
	   VcxoIntf(VcxoI2CReg),
	   Vcxo_I2C(VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr),
	   Vcxo(Vcxo_I2C, Si570::tcA, kVcxoPowerOnRate),
	   LowResolutionReg(LogicMemory, mmResolution),
	   RawClockDev(this)
{
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &X5_TxDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &X5_TxDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X5_TxDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X5_TxDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X5_TxDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X5_TxDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X5_TxDeviceMap::OnAfterStreamStopHandler );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor
//---------------------------------------------------------------------------

X5_TxDeviceMap::~X5_TxDeviceMap()
{
	delete FOutput;
	FOutput = 0;
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "X5_TxDeviceMap::OpenHardware()");
    X5DeviceMap::OpenHardware();

    if (PciLogicPcb() >= 1)   // Rev B or later
        {
		//  Substitue the Si570 as Vco for the '9511 Clock
        //    (need proxy since 9511 owns its controller)
        Clock.LoadVco(new IVcxoControlProxy(&Vcxo));
        }

    // Re-apply all Vcxo settings
    Debug.Log("At Vcxo Init()");
	Vcxo.Init();
    VcoOutputEnable(true);
    Debug.Log("After Vcxo Init()");

    Output().PacketSize(0x4000);
    Output().PacketId(0);
    
    // Configure D/As
    for (int dev = 0; dev < 2; ++dev)
        {
        Output().Device(dev).FrameMode(false);
        }
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnStreamPreconfigHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & /*event*/)
{
    ScopedTraceRef ccc(Debug, "X5_TxDeviceMap::OnStreamPreconfigHandler()");
    // Clock depends on Channel Enables being set
    Output().ConfigureChannelEnables();
    // Re-apply all clock settings
    Clock.Reapply();
	//
    //  Different Versions have different preconfigs
    if (PciLogicPcb() < 2)   // NOT Rev C or later
        OnStreamPreconfig_RevAB();
    else
        OnStreamPreconfig_RevC();

	LowResolutionReg.Enable.Value(FLowResolution);
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnStreamPreconfig_RevAB() -- Rev A/B Preconfigure
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnStreamPreconfig_RevAB()
{
	Debug.Log("REV A/B Preconfigure");
    //
	//  Reset DAC/FPGA
    Output().Reset(true);
    Output().Reset(false);
    Sleep(1);

    Output().Reset(true);     // Reset two extra times
    Output().Reset(false);
    Sleep(1);

    Output().Reset(true);
    Output().Reset(false);
    Sleep(1);

    bool dac_locked = Output().DacStatusReg.DacClockLocked.Value();
    if (dac_locked)
        Debug.Log("DAC Clock Locked Bit -- true");
    else
        Debug.Log("DAC Clock Locked Bit -- false");

	// Reapply Hardware Settings for  X5 IO Devices
	Output().PreconfigHardware();

}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnStreamPreconfig_RevC() -- Rev C Preconfigure
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnStreamPreconfig_RevC()
{
	const unsigned int kMaxTestReps = 10;
	//
	//  Apply Gain/Offset Coefficients - need this so output is on for CAL
	if (Output().Calibrated())
		Output().ApplyRomCoefficients();
	else
		Output().ApplyDefaultCoefficients();

	//
	//  Save old dac delays, change to delay of 0.
	unsigned int Dac0Delay_cache = Output().Device(0).DacDelay();
	unsigned int Dac1Delay_cache = Output().Device(1).DacDelay();
	Output().Device(0).DacDelay(0);
	Output().Device(1).DacDelay(0);

	Debug.Log("REV C+ Preconfigure");
	FInternalCal = false;
	for (unsigned int i=0; i<kMaxTestReps; i++)
		{
		Debug.Log("...Latency Calibration");
		//
		//  Reset DAC/FPGA
		Output().Reset(true);
		Output().Reset(false);
		Sleep(1);

		// Reapply Hardware Settings for  X5 IO Devices
		Output().PreconfigHardware();

		unsigned int should_be(0xF);
		if (Output().Device(0).Enabled()==false)
			should_be = 0xC;
		else if (Output().Device(1).Enabled()==false)
			should_be = 0x3;

		Output().DoInternalCalibration();

		unsigned int result = Output().GetInternalCalibrationResults();
		Debug.Log("Cal Result ", (int)result);
		if (result==should_be)
			{
			Debug.Log("Latency Calibration Successful");
			FInternalCal = true;
			break;
			}
		Debug.Log("Latency Calibration Failed");
		}

	double output_rate = Clock.FrequencyActual()/1e6;
	if ((output_rate>=220.0 && output_rate<=225.0)  ||
		(output_rate>=630.0 && output_rate<=650.0)      )
		{
		Debug.Log("Latency Calibration Failed - false positive");
		}
    //
    //  Restore original dac delays and apply them
    Output().Device(0).LoadDacDelay(Dac0Delay_cache);
	Output().Device(1).LoadDacDelay(Dac1Delay_cache);
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & /*event*/)
{
	// nothing as of yet
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X5_TxDeviceMap::OnStreamConfigureHandler()");

	// Family-common init
	X5DeviceMap::ConfigureHardware();

    //  Clear all run bits
	Output().DeviceRun(false);
    Output().SoftwareTrigger(false);

    Output().TestControlReg.Enable.Value(false);

    // Re-apply all Vcxo settings
    //
    //  There is a theory that this reappply might disturb the DAC clock.
    //    since this is a "quick update" it shouldn't, but lets remove it
    //    just in case  (ccas 1-18-10)
    //Vcxo.Reapply();

	// Reapply Hardware Settings for  X5 IO Devices
	Output().ConfigureHardware();
	//
	//  Apply Gain/Offset Coefficients
	if (Output().Calibrated())
		Output().ApplyRomCoefficients();
	else
		Output().ApplyDefaultCoefficients();

	//  Forward notify to baseboard handler
	Owner->OnStreamConfigure.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X5_TxDeviceMap::OnBeforeStreamStartHandler()");
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStart.Execute(Event);
    //
    //  Start execution
    if (Output().Device(0).Enabled() && Output().Device(1).Enabled())
        Output().DeviceRun(true);
    else if (Output().Device(0).Enabled())
        Output().DeviceRun(true, X5_TxOutputDevice::engDevice0);
	else if (Output().Device(1).Enabled())
        Output().DeviceRun(true, X5_TxOutputDevice::engDevice1);

}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X5_TxDeviceMap::OnAfterStreamStartHandler()");
    //  Forward notify to baseboard handler
    Owner->OnAfterStreamStart.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X5_TxDeviceMap::OnBeforeStreamStopHandler()");
    //  Forward notify to baseboard handler
    Owner->OnBeforeStreamStop.Execute(Event);
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X5_TxDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
	//  Disable Devices
    Output().DeviceRun(false);

    //  Disable Trigger
    Output().SoftwareTrigger(false);

	//  Forward notify to baseboard handler
	Owner->OnAfterStreamStop.Execute(Event);
}

//------------------------------------------------------------------------------
//  X5_TxDeviceMap::PatternGenerator() --
//------------------------------------------------------------------------------

void X5_TxDeviceMap::PatternGenerator(bool state)
{
    Output().PatternModeCtlReg.PatternModeEnable.Value(state);
}

//------------------------------------------------------------------------------
//  X5_TxDeviceMap::PatternGenerator() --
//------------------------------------------------------------------------------

bool X5_TxDeviceMap::PatternGenerator()
{
    return Output().PatternModeCtlReg.PatternModeEnable.Value();
}

//------------------------------------------------------------------------------
//  X5_TxDeviceMap::VcoOutputEnable() --
//------------------------------------------------------------------------------

void X5_TxDeviceMap::VcoOutputEnable(bool state)
{
	VcxoI2CReg.VcoOutputEnable.Value(state);
}

//------------------------------------------------------------------------------
//  X5_TxDeviceMap::VcoOutputEnable() --
//------------------------------------------------------------------------------

bool X5_TxDeviceMap::VcoOutputEnable()
{
	return VcxoI2CReg.VcoOutputEnable.Value();
}

//------------------------------------------------------------------------------
//  X5_TxDeviceMap::LowResolution() --
//------------------------------------------------------------------------------

void X5_TxDeviceMap::LowResolution(bool state)
{
	FLowResolution = state;
}

//------------------------------------------------------------------------------
//  X5_TxDeviceMap::LowResolution() --
//------------------------------------------------------------------------------

bool X5_TxDeviceMap::LowResolution() const
{
	return FLowResolution;
}

//---------------------------------------------------------------------------
//  X5_TxDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

X5_TxOutputDevice & X5_TxDeviceMap::Output()
{
	return *FOutput;
}


} // namespace Innovative


