// X5_TxDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X5_TxDeviceMap_MbH
#define X5_TxDeviceMap_MbH

#include <X5_TX_Mb.h>
#include <X5DeviceMap_Mb.h>
#include <Events_Mb.h>
#include <PmcEeprom_Mb.h>
#include <Trace_Mb.h>
#include <X5ClockIntf_Mb.h>
#include <CalMemBase_Mb.h>
#include <Ad9511_Mb.h>
#include <Si570_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X5_TxDeviceMap;

//==============================================================================
//  CLASS X5_Tx_Ad9511 -- X5_Tx specific 9511 implementation
//==============================================================================

class X5_Tx_Ad9511 : public Ad9511Pfd
{
public:
	//
	//  Member Functions
	X5_Tx_Ad9511(X5_TxDeviceMap * owner, int index, AddressingSpace &space, int ctl);
	X5_Tx_Ad9511(X5_TxDeviceMap * owner, double freq, double ref, bool enabled, int index,
		   AddressingSpace & space, int ctl);

    //  Raw configuration accesses
	virtual void 			Port(unsigned char reg, unsigned char value);
	virtual unsigned char 	Port(unsigned char reg);

    double                  DCLK(bool single_channel);
    bool                    IsDacDllEnabled(bool single_channel);

protected:
    virtual void  ConfigureOutputs(int outdiv_bits, int outdiv_bypass);

private:
    X5_TxDeviceMap * Owner;

    int  PllOutputDividers(bool single_channel, int outdiv);

    void LogicDividerSetting(int div_register, int bypass_register);

   	virtual	double  PhaseDetectorFrequency() const
	        {	return 1.0e6; }

};

//==============================================================================
//  CLASS X5RawClockDevices  -- Interface to raw clock devices
//==============================================================================
class X5RawClockDevices : public IX5RawClockDevices
{
	public:
		X5RawClockDevices(X5_TxDeviceMap * owner): Owner(owner)
			{  }
		virtual ~X5RawClockDevices()
			{  }

		virtual Si570 &          Vcxo();
		virtual X5_Tx_Ad9511 &   Pll();

	protected:
	  X5_TxDeviceMap *  Owner;
};

//==============================================================================
//  CLASS X5_TxClock -- Tx clock control
//==============================================================================

class X5_TxClock : public X5ClockIntf
{
	
public:
    X5_TxClock(X5_TxDeviceMap * owner, int index, AddressingSpace &space,
        int ctl);
	X5_TxClock(X5_TxDeviceMap * owner, double freq, double ref,
        bool enabled, int index, AddressingSpace & space, int ctl);

	friend class X5RawClockDevices;
	
    //
    //  Interface Functions
    // Methods
    virtual double			FrequencyActual() const;

    // Properties
    virtual void    		Frequency(double value);
    virtual double			Frequency() const;


    virtual IIClockSource   Source() const;
    virtual void            Source(IIClockSource src);

    virtual void                Reference(IIReferenceSource src);
    virtual IIReferenceSource   Reference() const;

    virtual void    		    ReferenceFrequency(double value);
    virtual double			    ReferenceFrequency() const;

    void 			        Reapply();
    bool                    Locked() const;

    double                  DCLK(bool single_channel)
                                {  return Clock.DCLK(single_channel);  }
    bool                    IsDacDllEnabled(bool single_channel)
                                {  return Clock.IsDacDllEnabled(single_channel);  }

    void                    LoadVco(IVcxoControl * vcxo)
								{  Clock.LoadVco(vcxo);  }

	void 					Port(unsigned char reg, unsigned char value)
		{  Clock.Port(reg, value);  }
	unsigned char 			Port(unsigned char reg)
		{  return Clock.Port(reg);  }

protected:
    // Data
    X5_TxDeviceMap *      Owner;
    X5_Tx_Ad9511          Clock;
    double                ExternalRefFreq;

//    virtual void            SetFrequency(double freq);
};

//===========================================================================
//  CLASS X5_TxDacDeviceRegisters  -- Addresses for Dac Device Registers
//===========================================================================

class X5_TxDacDeviceRegisters
{
public:
//    unsigned int   PacketHeaderRegister_Addr;
    unsigned int   SpiInterfaceRegister_Addr;
    unsigned int   TriggerControlRegister_Addr;
};

//===========================================================================
//  CLASS X5_TxDacDevice  -- Per-DAC controls for X5-TZ
//===========================================================================

class X5_TxDacDevice
{
	friend class X5_TxOutputDevice;

public:
	enum ExtTriggerSources {extSync0 = 0x0, extSync1};

    enum Constants { DeviceChannels = 2 };

    // Methods
	void                Enabled(bool value);
	bool                Enabled()
                            {  return FEnabled;  }
	void                ChannelEnabled(int idx, bool value);
	bool                ChannelEnabled(int idx)
                            {  return FChannelEnabled[idx];  }
    bool                SingleActiveChannel();

    void                ClearDacTestStatus()
                            {  Spi(0x04, 0x00);   }
    unsigned int        DacTestStatus()
                            {  return Spi(0x04);  }

	void                Spi(int addr, unsigned char value);
	unsigned char       Spi(int addr);

    bool                DacStatus() const;

    void                ExternalTrigger(bool state);
    bool                ExternalTrigger() const;

    void                DacDelay(unsigned int delay)
                            {  FDacDelay = delay;  }
    unsigned int        DacDelay() const
                            {  return FDacDelay;  }
    void                LoadDacDelay(unsigned int delay);

    void                FrameMode(bool state);
    bool                FrameMode() const;

    void                FrameSize(unsigned int size);
    unsigned int        FrameSize() const;

    void                PreconfigHardware();
    void                ConfigureHardware();

protected:
    void                ExternalTriggerSource(ExtTriggerSources state);
    ExtTriggerSources   ExternalTriggerSource() const;

	X5_TxDacDevice(X5_TxDeviceMap * owner, X5_TxDacDeviceRegisters & reg);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TriggerRegister  -- Trigger control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBitGroup      FrameCount;
		RegisterBit           FramedMode;
		RegisterBit           ExtTrigger;
		RegisterBit           ExtTriggerSource;

        TriggerRegister(AddressingSpace &space, int offset)
            :  inherited("X5_TxDacDevice::TriggerRegister", space, offset),
			   FrameCount(*this, 0, 24), FramedMode(*this, 29),
               ExtTrigger(*this, 30),  ExtTriggerSource(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiWriteRegister  -- DAC SPI Write register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class SpiWriteRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup    Data;
		RegisterBitGroup    Address;
		RegisterBit         Direction;

		SpiWriteRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Data(*this, 0, 8),
			  Address(*this, 8, 5), Direction(*this, 16)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiReadRegister  -- DAC SPI Read register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class SpiReadRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         DacStatus;
		RegisterBit         Ready;
		RegisterBit         DataValid;
		RegisterBitGroup    ReadData;

		SpiReadRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), DacStatus(*this, 21),
              Ready(*this, 22), DataValid(*this, 23),  ReadData(*this, 24, 8)
		{
		}
	};
    enum SpiDir { spiWrite, spiRead };
    //
    //  Registers
    TriggerRegister             TriggerReg;
    SpiWriteRegister            SpiWriteReg;
    SpiReadRegister             SpiReadReg;

    X5_TxDeviceMap *            Owner;
    bool                        FEnabled;
    unsigned int                FDacDelay;
    bool                        FChannelEnabled[DeviceChannels];
    Trace               Debug;

    void            SpiReady();
    void            SpiNotReady();
    void            SingleChannelMode(int active_chan);
    void            DualChannelMode();
    void            Spi10Setup(double FreqMhz);


};

//===========================================================================
//  CLASS X5_TxOutputDevice  -- Register & low level support for X5_Tx DACs
//===========================================================================

class X5_TxOutputDevice  :  public IUsesSectionCalibration
{
	friend class X5_TxDeviceMap;

public:
	enum EngDeviceState { engDevice0, engDevice1, engAll };
	enum SwTriggerState { swDevice0, swDevice1, swAll };
	//enum IICheckType{ctDecrementing, ctIncrementing};
	enum IITestMode{tmRamp = 0x0, tmSine, tmBitPattern};

	~X5_TxOutputDevice();

	// Properties
    //  ...  Subdevice
    X5_TxDacDevice &  Device(int idx)
        {  return (idx==0) ? Dac0 : Dac1;  }

    IoInfoIntf &    Info()
                        {  return *FInfo;  }

    void            Reset(bool state);

    void            DeviceRun(bool state, EngDeviceState which = engAll);

    void            SoftwareTrigger(bool state, SwTriggerState which = swAll);

    bool            InternalDacCalibration();

	bool    		PatternModeLoopback() const;
	void    		PatternModeLoopback(bool state);

	bool    		TestEnable() const;
	void    		TestEnable(bool state);

	unsigned int 	TestMode() const;
	void 			TestMode(unsigned int mode);

	float           TestFrequency() const;
	void 			TestFrequency(float value);

	void            PllSpi(int addr, unsigned char value);
	unsigned char   PllSpi(int addr);

    void                PacketId(unsigned char id);
    unsigned char       PacketId() const;

    void                PacketSize(int size);
    int                 PacketSize() const;

    void                LoopbackPacketId(unsigned char id);
    unsigned char       LoopbackPacketId() const;

    void                LoopbackPacketSize(int size);
    int                 LoopbackPacketSize() const;

    void    ConfigureChannelEnables();
    
    void            PreconfigHardware();
    virtual void    ConfigureHardware();

protected:
	X5_TxOutputDevice(X5_TxDeviceMap * owner);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PacketRegister  -- Rx Packet size management
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PacketRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Size;
        RegisterBitGroup      Id;

        //
        //  Member Functions
        PacketRegister(AddressingSpace &space, int offset)
            : inherited("X5_TxDacDevice::PacketRegister", space, offset),
              Size(*this, 0, 24), Id(*this, 24, 8)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DacControlRegister  -- Pll Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacControlRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBit         Reset;
		RegisterBit         Dev0_Run;
		RegisterBit         Dev1_Run;
		RegisterBitGroup    ChannelEnable;

		DacControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Reset(*this, 0),
              Dev0_Run(*this, 1), Dev1_Run(*this, 2), ChannelEnable(*this, 4, 4)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DacStatusRegister  -- Dac Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacStatusRegister : public ReadOnlyRegister
	{
		typedef ReadOnlyRegister inherited;

	public:
		ReadOnlyRegisterBit         DacClockLocked;

		DacStatusRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), DacClockLocked(*this, 31)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DacSWTriggerRegister  -- Pll Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacSWTriggerRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBit    Dev0_Trigger;
		RegisterBit    Dev1_Trigger;

		DacSWTriggerRegister(AddressingSpace &space, int offset)
			: inherited(space, offset),
              Dev0_Trigger(*this, 0), Dev1_Trigger(*this, 1)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TestPhaseRegister  -- Test Phase Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TestPhaseRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBitGroup    PhaseIncrement;

		TestPhaseRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), PhaseIncrement(*this, 0, 23)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TestControlRegister  -- DAC SPI Write register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class TestControlRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         Enable;
		RegisterBitGroup    Mode;

		TestControlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Enable(*this, 0),
			  Mode(*this, 1, 3)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiWriteRegister  -- PLL SPI Write register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class SpiWriteRegister : public CachedShadowRegister
	{
		typedef CachedShadowRegister inherited;

	public:
		RegisterBitGroup    Data;
		RegisterBitGroup    Address;
		RegisterBit         Direction;

		SpiWriteRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), Data(*this, 0, 8),
			  Address(*this, 8, 7), Direction(*this, 15)
		{
		}
	};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpiReadRegister  -- PLL SPI Read register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class SpiReadRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         Ready;
		RegisterBit         DataValid;
		RegisterBitGroup    ReadData;

		SpiReadRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), 
              Ready(*this, 22), DataValid(*this, 23),  ReadData(*this, 24, 8)
		{
		}
	};
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PatternModeCtlRegister  -- Controls for Pattern Playback Mode
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class PatternModeCtlRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         PatternModeEnable;
		RegisterBit         PatternLoopbackEnable;

		PatternModeCtlRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), PatternModeEnable(*this, 0),
			  PatternLoopbackEnable(*this, 1)
		{
		}
	};
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS LatencyCalRegister  -- Controls for Latency Calibration
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class LatencyCalRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit         CalEnable;
		RegisterBitGroup    CalResults;

		LatencyCalRegister(AddressingSpace &space, int offset)
			: inherited(space, offset), CalEnable(*this, 0),
			  CalResults(*this, 4, 4)
		{
		}
	};

    enum SpiDir { spiWrite, spiRead };

    // Data
    X5_TxDeviceMap *        Owner;
    //  Subdevices
    X5_TxDacDevice          Dac0;
    X5_TxDacDevice          Dac1;

    // Fields
    IoInfoIntf *            FInfo;
    
	// Registers
	TestPhaseRegister           TestPhaseReg;
	TestControlRegister         TestControlReg;
    SpiWriteRegister            SpiWriteReg;
    SpiReadRegister             SpiReadReg;
    DacSWTriggerRegister        SoftTriggerReg;
    DacControlRegister          DacControlReg;
    DacStatusRegister           DacStatusReg;
    RegisterArray<Register>     GainCoeffReg;
    RegisterArray<Register>     OffsetCoeffReg;
    PacketRegister              PacketHeaderReg;
    PatternModeCtlRegister      PatternModeCtlReg;
    PacketRegister              LoopbackPacketHeaderReg;
    LatencyCalRegister          OutputCalibrationReg;
    Trace                       Debug;

	// Fields
	IITestMode      FTestMode;
	bool            FTestEnabled;
	float           FTestFrequency;


    void            PllSpiReady();
    void            PllSpiNotReady();

    // Overrides: Apply to ROM and hardware
    virtual void    SetCalibrationOffset(int ch, int value);
    virtual void    SetCalibrationGain(int ch, int value);

    void            DoInternalCalibration();

public:
    unsigned int    GetInternalCalibrationResults();

};

//===========================================================================
//  CLASS X5_TxDeviceMap  -- Register & low level support for SD module
//===========================================================================
class X5_Tx;

class X5_TxDeviceMap :  public X5DeviceMap,
						public IUsesStreamDeviceEventCallbacks
{
	friend class X5_Tx;
	friend class X5_TxOutputDevice;
	friend class X5_TxClock;
	friend class X5_TxDacDevice;
	friend class X5RawClockDevices;
public:
	//
	//  Member Functions
	X5_TxDeviceMap(X5_Tx * owner);
	virtual ~X5_TxDeviceMap();

	X5_TxOutputDevice &    Output();

	void            PatternGenerator(bool state);
	bool            PatternGenerator();

	void            VcoOutputEnable(bool state);
	bool            VcoOutputEnable();

	void            LowResolution(bool state);
	bool            LowResolution() const;

public: // pseudo-protected
	virtual void  OpenHardware();

protected:
	// Fields
	X5_TxOutputDevice *   FOutput;
	bool				FLowResolution;

	// Data
	X5_Tx *             Owner;
	bool                FInternalCal;
	X5_TxClock          Clock;
	Trace               Debug;


	//
	//  Class ResolutionRegister
	class ResolutionRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit  Enable;

	public:
		ResolutionRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   Enable(*this, 0)
			{
			}
	};

	//
	//  Class VcxoI2CRegister
	class VcxoI2CRegister : public Register
	{
		typedef Register inherited;

	public:
		RegisterBit  VcoOutputEnable;
		RegisterBit  Data_Out;
		RegisterBit  Clock_Out;
		RegisterBit  Data_In;

	public:
		VcxoI2CRegister( AddressingSpace &space, int offset )
			:  inherited(space, offset),
			   VcoOutputEnable(*this, 0),
			   Data_Out(*this, 2),
			   Clock_Out(*this, 1),
			   Data_In(*this, 3)
			{
			}
	};

	VcxoI2CRegister                     VcxoI2CReg;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//  CLASS VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class VcxoI2C_Impl : public I2CDeviceIntf
	{
	public:
		VcxoI2C_Impl(VcxoI2CRegister & reg)
			:  Reg(reg)
			{}
		virtual ~VcxoI2C_Impl()
			{}
		// Methods
		virtual void    SetClock(int state)
			{  Reg.Clock_Out.Value(state ? true : false);  }
		virtual int     GetClock() const
			{  return Reg.Clock_Out.Value();  }
		virtual void    SetData(int state)
			{  Reg.Data_Out.Value(state ? true : false);  }
		virtual int     GetData() const
			{  return Reg.Data_In.Value();  }

	private:
		VcxoI2CRegister &  Reg;
	};

	VcxoI2C_Impl        VcxoIntf;
	I2CDeviceN          Vcxo_I2C;
	Si570               Vcxo;
	ResolutionRegister	LowResolutionReg;


	X5RawClockDevices RawClockDev;

	// Methods
	void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
	void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
	void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
	void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
	void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void  OnStreamPreconfig_RevAB();
    void  OnStreamPreconfig_RevC();

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
