//---------------------------------------------------------------------------
// X5_TX_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2006 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X5_TX_Mb.h>
#include <LogicLoadThread_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <Buffer_Mb.h>
#include <BufferHeader_Mb.h>
#include <BufferDatagrams_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X5_Tx  -- SD PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor
//---------------------------------------------------------------------------

X5_Tx::X5_Tx()
    :  X5Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor 
//---------------------------------------------------------------------------

X5_Tx::~X5_Tx()
{
    Close();
}

//---------------------------------------------------------------------------
//  X5_Tx::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X5_Tx::Open()
{
    Device.Open();
    //
    //  Reset the device too
    Reset();
}

//---------------------------------------------------------------------------
//  X5_Tx::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X5_Tx::BoardCount()
{
    return Device.BoardCount();
}

//---------------------------------------------------------------------------
//  X5_Tx::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X5_Tx::BoardNames()
{
    return Device.DeviceDriver->ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X5_Tx::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X5_Tx::PrintAllDevices()
{
    return Device.DeviceDriver->ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X5_Tx::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X5_Tx::Close()
{
    Device.Close();
}

//---------------------------------------------------------------------------
//  X5_Tx::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X5_Tx::Reset()
{
    Device.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X5_Tx::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X5_Tx::Target()
{
    return Device.Target();
}

//---------------------------------------------------------------------------
//  X5_Tx::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X5_Tx::Target( int value )
{
    Device.Target(value);
}

//---------------------------------------------------------------------------
//  X5_Tx::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X5_Tx::Led()
{
    return Device.Led();
}

//---------------------------------------------------------------------------
//  X5_Tx::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X5_Tx::Led( bool value )
{
    Device.Led(value);
}

//---------------------------------------------------------------------------
//  X5_Tx::UserLogicVersion() -- Get User Logic version info
//---------------------------------------------------------------------------

FpgaLogicVersionInfoIntf & X5_Tx::UserLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_Tx::PciLogicVersion() -- Get PCI Logic version info
//---------------------------------------------------------------------------

PciLogicVersionInfoIntf & X5_Tx::PciLogicVersion()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_Tx::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X5_Tx::PeekLogic(int offset )
{
    return Device.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X5_Tx::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X5_Tx::PokeLogic( int offset, ii32 value )
{
    Device.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X5_Tx::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X5_Tx::PeekPort(int offset )
{
    return Device.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X5_Tx::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X5_Tx::PokePort( int offset, ii32 value )
{
    Device.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X5_Tx::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

FpaLoadThread & X5_Tx::VsProm()
{
    return *Device.EepromLoader;
}

//---------------------------------------------------------------------------
//  X5_Tx::PciDiagnostics() -- Return reference to Pci Diagnostics object
//---------------------------------------------------------------------------

IPciBusDiagnostics & X5_Tx::PciDiagnostics()
{
    return Device;
}

//------------------------------------------------------------------------------
//  X5_Tx::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X5_Tx::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    Device.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X5_Tx::PciLoopbackMode() --
//------------------------------------------------------------------------------

X5_Tx::LoopbackMode X5_Tx::PciLoopbackMode()
{
    X5_Tx::LoopbackMode result;
    switch (Device.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X5_Tx::UserLoopbackMode() --
//---------------------------------------------------------------------------

void X5_Tx::UserLoopbackMode(bool state)
{
    Device.UserLoopbackMode(state);
}

//---------------------------------------------------------------------------
//  X5_Tx::UserLoopbackMode() --
//---------------------------------------------------------------------------

bool X5_Tx::UserLoopbackMode() const
{
    return Device.UserLoopbackMode();
}

//---------------------------------------------------------------------------
//  X5_Tx::Input() --  Return reference to input object
//---------------------------------------------------------------------------
//
//X5_TxInputDevice & X5_Tx::Input()
//{
//	return Device.Input();
//}
//
//---------------------------------------------------------------------------
//  X5_Tx::Output() --  Return reference to input object
//---------------------------------------------------------------------------

X5_TxOutputDevice & X5_Tx::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  X5_Tx::IdRom() -- DIO port access
//---------------------------------------------------------------------------

IUsesDioPort & X5_Tx::Dio()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_Tx::IdRom() -- Calibration Info access
//---------------------------------------------------------------------------

IUsesPmcEeprom & X5_Tx::IdRom()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_Tx::Trigger() -- Issue software trigger
//---------------------------------------------------------------------------

void X5_Tx::Trigger(IIDirection dir, bool state)
{
    Device.Trigger(static_cast<X5DeviceMap::IIDirection>(dir), state);
}

//---------------------------------------------------------------------------
//  X5_Tx::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX5Alerts & X5_Tx::Alerts()
{
    return Device.Alert;
}

//---------------------------------------------------------------------------
//  X5_Tx::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

ExtendedLogicTemperatureIntf & X5_Tx::Thermal()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X5_Tx::ClockTestEnable() --
//---------------------------------------------------------------------------

bool X5_Tx::ClockTestEnable() const
{
	return Device.ClockTestEnable();
}

//---------------------------------------------------------------------------
//  X5_Tx::LowResolution() --
//---------------------------------------------------------------------------

void X5_Tx::LowResolution(bool state)
{
	Device.LowResolution(state);
}

//---------------------------------------------------------------------------
//  X5_Tx::LowResolution() --
//---------------------------------------------------------------------------

bool X5_Tx::LowResolution() const
{
	return Device.LowResolution();
}

//---------------------------------------------------------------------------
//  X5_Tx::Clock() --   Access to clock control object
//---------------------------------------------------------------------------

X5_TxClock & X5_Tx::Clock()
{
    return Device.Clock;
}

//---------------------------------------------------------------------------
//  X5_Timing::Vcxo() --  Return reference to Vcxo
//---------------------------------------------------------------------------

Si570 & X5_Tx::Vcxo()
{
	return Device.Vcxo;
}

//---------------------------------------------------------------------------
//  X5_Tx::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX5RawClockDevices & X5_Tx::RawClockDevice()
{
    return Device.RawClockDev;
}

//---------------------------------------------------------------------------
//  X5_Tx::BusMasterSize() --
//---------------------------------------------------------------------------

void X5_Tx::BusMasterSize(int bytes)
{
	Device.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_Tx::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_Tx::IncomingBusMasterSize(int bytes)
{
	Device.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_Tx::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X5_Tx::OutgoingBusMasterSize(int bytes)
{
	Device.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X5_Tx::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_Tx::IncomingBusMasterSize() const
{
	return Device.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X5_Tx::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X5_Tx::OutgoingBusMasterSize() const
{
	return Device.OutgoingBusMasterSizeWords() * sizeof(int);
}

//------------------------------------------------------------------------------
//  X5_Tx::PatternGenerator() --
//------------------------------------------------------------------------------

void X5_Tx::PatternGenerator(bool state)
{
    Device.PatternGenerator(state);
}

//------------------------------------------------------------------------------
//  X5_Tx::PatternGenerator() --
//------------------------------------------------------------------------------

bool X5_Tx::PatternGenerator()
{
    return Device.PatternGenerator();
}

//------------------------------------------------------------------------------
//  X5_Tx::SendPatternInfo() -- Send Pattern Start Information Block
//------------------------------------------------------------------------------

const unsigned int TagFieldStartBit     =  0;
const unsigned int TagFieldSize         =  8;

const unsigned int WriteCommandBit      =  16;
const unsigned int ReadCommandBit       =  17;
const unsigned int IdlePacketBit        =  18;

void  X5_Tx::SendPatternInfo(char tag_value, int start_addr,
                             int buf_size_ints, int count,
                             PatternInfoType mode)
{
    Buffer Packet(4);
    PacketBufferHeader Hdr(Packet);
    IntegerDG  PacketDG(Packet);

    Hdr.PeripheralId(0x80);
    Hdr.PacketSize(Packet.FullSizeInInts());
    Hdr[1] = 0;

    PacketDG[0] = start_addr/8;
    PacketDG[1] = buf_size_ints/8;
    PacketDG[2] = count;
    PacketDG[3] = 0;

    bool write_bit_value;
    bool read_bit_value;
    switch (mode)
        {
        case piPlay:
        default:
            write_bit_value = true;   // data follows, and play it
            read_bit_value = true;
            break;
        case piLoad:
            write_bit_value = true;
            read_bit_value = false;   // data follows, don't play it
            break;
        case piReplay:
            write_bit_value = false;
            read_bit_value = true;   // no data follows, play loaded pattern
            break;
        case piHalt:
            write_bit_value = false;
            read_bit_value = false;   // no data follows, halt playing
            break;
        }

    PtrBitManipulator InfoWord( &PacketDG[3] );
    //  Fill Info Word
    InfoWord.Field(TagFieldStartBit, TagFieldSize, tag_value);
    InfoWord.Bit(WriteCommandBit, write_bit_value);
    InfoWord.Bit(ReadCommandBit, read_bit_value);
    InfoWord.Bit(IdlePacketBit, 0);
    //
    //  Post to stream
    if (StreamManager.Linked())
        StreamManager->Send(Packet);
}

//---------------------------------------------------------------------------
//  X5_Tx::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X5_Tx::AfterFiclStart(FiclSystem & engine)
{
	engine.Evaluate("  0 constant Pll    1 constant Vcxo  2 constant Dac0   3 constant Dac1 ");

	engine.Evaluate(": ?X5-TX "
					"    cr .\" --- SubDevice Constants --- \" "
					"    cr .\"   PLL  Vcxo  Dac0  Dac1 \" "
					"    cr  "
					"    cr .\" ---- X5-TX Definitions ---- \" "
					"    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
					"    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X5-TX  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("X5-TX-specific wordset loaded \n" );

    //  Replace Callbacks to call FICL too
    Device.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X5_Tx::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X5_Tx::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X5_Tx::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X5_Tx::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X5_Tx::Do_DFetch(int dev, int addr)
{
	const int kPll = 0;   const int kVcxo = 1;   const int kDac0 = 2;  const int kDac1 = 3;
	
	switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Clock().Port(addr);
        case kVcxo:
			return Vcxo().Port(addr);
		case kDac0:
			return Output().Device(0).Spi(addr);
		case kDac1:
			return Output().Device(1).Spi(addr);
		}
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X5_Tx::Do_DStore(int dev, int addr, int value)
{
	const int kPll = 0;   const int kVcxo = 1;   const int kDac0 = 2;  const int kDac1 = 3;

	switch (dev)
		{
		default:
			break;
		case kPll:
			Clock().Port(addr, value);
			break;
		case kVcxo:
			Vcxo().Port(addr, value);
			break;
		case kDac0:
			Output().Device(0).Spi(addr, value);
			break;
		case kDac1:
			Output().Device(1).Spi(addr, value);
            break;
		}
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X5_Tx::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;   
    switch (dev)
        {
        default:
            break;
        case kPll:
            Clock().Frequency(freq);
            Clock().Reapply();
            break;
        case kVcxo:
            Vcxo().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X5_Tx::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X5_Tx::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;   
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Clock().FrequencyActual());
        case kVcxo:
            return static_cast<float>(Vcxo().FrequencyActual());
		}
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
