// X5_Tx_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef X5_Tx_MbH
#define X5_Tx_MbH

#include <X5Board_Mb.h>
#include <DataTypes_Mb.h>
#include <ProcessEvents_Mb.h>
#include <X5_TXDeviceMap_Mb.h>
#include <X5IoDevice_Mb.h>
#include <Events_Mb.h>
#include <PmcIdrom_Mb.h>
#include <FpgaLogicVersionInfoIntf_Mb.h>
#include <PciLogicVersionInfoIntf_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS X5_Tx  -- X5_Tx PCI Express baseboard object
//===========================================================================

class X5_Tx : public X5Board,
			  public IStreamDeviceEvents,
              public IFiclTarget
{
    friend class X5_TxDeviceMap;

public:
    // Trigger Support
    // Enumerations
    enum IIDirection { tInput, tOutput, tInputOutput };
    enum LoopbackMode { lmNone, lmParallel, lmSerial };
    enum PatternInfoType { piPlay, piLoad, piReplay,/* piLoadIdle, */ piHalt };

    //
    //  Member Functions
    X5_Tx();
    virtual ~X5_Tx();

    // Methods
    size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
    void                        Open();
    void                        Close();
    virtual void                Reset();
    std::string                 Name() const
        {  return Device.Name();  }
    std::string                 Revision() const
        {  return Device.Revision();  }

    FpgaLogicVersionInfoIntf &  UserLogicVersion();
    PciLogicVersionInfoIntf &   PciLogicVersion();
    IPciBusDiagnostics &        PciDiagnostics();

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    // Properties
    int                         Target();
    void                        Target(int value);

    bool                        Led();
    void                        Led(bool value);

    void                        UserLoopbackMode(bool state);
    bool                        UserLoopbackMode() const;

    void                        PciLoopbackMode(LoopbackMode mode);
    LoopbackMode                PciLoopbackMode();

    void                        Trigger(IIDirection dir, bool state);

    void                        ClockFrequency(float value);
    float                       ClockFrequency() const;
    float                       ActualClockFrequency() const;

    void                        ReferenceFrequency(float value);
    float                       ReferenceFrequency() const;

    void                        ClockTestEnable(bool state);
    bool                        ClockTestEnable() const;

	void                        LowResolution(bool state);
	bool                        LowResolution() const;

	void 							   BusMasterSize(int bytes);
	void 							   IncomingBusMasterSize(int bytes);
	void 							   OutgoingBusMasterSize(int bytes);
	int             				   IncomingBusMasterSize() const;
	int             				   OutgoingBusMasterSize() const;

    void            PatternGenerator(bool state);
    bool            PatternGenerator();
    void            SendPatternInfo(char tag_value, int start_addr,
                                    int buf_size_ints, int count,
                                    PatternInfoType mode = piPlay);

    // Device access methods
    // EEprom programming
    FpaLoadThread &                   VsProm();

    IUsesDioPort &                    Dio();
	X5_TxOutputDevice  &              Output();
	X5_TxClock &                      Clock();
	IX5RawClockDevices &			  RawClockDevice();
    Si570 &                           Vcxo();
    IUsesPmcEeprom &                  IdRom();
    //  Alerts
	IUsesX5Alerts &                   Alerts();
    ExtendedLogicTemperatureIntf &  Thermal();

public: //Pseudo protected
	class Trace &       Tracer() { return Device.Debug;  }

protected:
    //
    //  Member Data
    X5_TxDeviceMap             Device;

	//  Ficl Interface
	virtual void    AfterFiclStart(FiclSystem & engine);

	virtual int     Do_LFetch(int addr);
	virtual void    Do_LStore(int addr, int value);
	virtual int     Do_PFetch(int addr);
	virtual void    Do_PStore(int addr, int value);
	virtual int     Do_DFetch(int dev, int addr);
	virtual void    Do_DStore(int dev, int addr, int value);
	virtual void    Do_DFreq(int dev, float freq);
	virtual float   Do_DFreqActual(int dev);
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

