//---------------------------------------------------------------------------
// X6Board_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X6Board_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6Board  -- X6 Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6Board
//---------------------------------------------------------------------------

X6Board::X6Board(X6DeviceMap & deviceMap)
    :  IIsAVitaPacketStreamBoard(deviceMap), X6DeviceRef(deviceMap)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6Board
//---------------------------------------------------------------------------

X6Board::~X6Board()
{
    // Don't call anything using X6DeviceRef here - put in most derived class
}

//---------------------------------------------------------------------------
//  X6Board::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t X6Board::BoardCount()
{
	return X6DeviceRef.BoardCount();
}

//---------------------------------------------------------------------------
//  X6Board::BoardNames() -- Get Device Names
//---------------------------------------------------------------------------

vector<string> X6Board::BoardNames()
{
    return X6DeviceRef.Driver().ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  X6Board::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string X6Board::PrintAllDevices()
{
    return X6DeviceRef.Driver().ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  X6Board::DriverDate() --
//---------------------------------------------------------------------------

string X6Board::DriverDate()
{
    return X6DeviceRef.Driver().ODevice().DriverDate();
}

//---------------------------------------------------------------------------
//  X6Board::DriverVersion() --
//---------------------------------------------------------------------------

string X6Board::DriverVersion()
{
    return X6DeviceRef.Driver().ODevice().DriverVersion();
}
//---------------------------------------------------------------------------
//  X6Board::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  X6Board::LicenseCheck(unsigned int & /*code*/)
{
    return true;
}

//---------------------------------------------------------------------------
//  X6Board::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  X6Board::Target()
{
    return X6DeviceRef.Target();
}

//---------------------------------------------------------------------------
//  X6Board::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  X6Board::Target( int value )
{
    X6DeviceRef.Target(value);
}

//---------------------------------------------------------------------------
//  X6Board::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  X6Board::Led()
{
    return X6DeviceRef.Led();
}

//---------------------------------------------------------------------------
//  X6Board::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  X6Board::Led( bool value )
{
    X6DeviceRef.Led(value);
}

//---------------------------------------------------------------------------
//  X6Board::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  X6Board::GreenLed()
{
    return X6DeviceRef.Periph_SDev.GreenLed();
}

//---------------------------------------------------------------------------
//  X6Board::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  X6Board::GreenLed( bool value )
{
    X6DeviceRef.Periph_SDev.GreenLed(value);
}

//---------------------------------------------------------------------------
//  X6Board::FrontPanelLed() -- Read the FrontPanel LED state
//---------------------------------------------------------------------------

bool  X6Board::FrontPanelLed()
{
    return X6DeviceRef.Periph_SDev.FrontPanelLed();
}

//---------------------------------------------------------------------------
//  X6Board::FrontPanelLed() -- Set the FrontPanel LED state of the board
//---------------------------------------------------------------------------

void  X6Board::FrontPanelLed( bool value )
{
    X6DeviceRef.Periph_SDev.FrontPanelLed(value);
}

//---------------------------------------------------------------------------
//  X6Board::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  X6Board::Open()
{
    X6DeviceRef.Open();
}

//---------------------------------------------------------------------------
//  X6Board::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  X6Board::Close()
{
    X6DeviceRef.Close();
}

//---------------------------------------------------------------------------
//  X6Board::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

void  X6Board::Reset()
{
    X6DeviceRef.Reset();    // Board Reset
}

//---------------------------------------------------------------------------
//  X6Board::Internals() -- Internal Packet Stream Interface
//---------------------------------------------------------------------------

IPacketStreamDeviceInternals *  X6Board::Internals()
{
    return &X6DeviceRef;
}

//---------------------------------------------------------------------------
//  X6Board::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  X6Board::Log( int type, int value )
{
    X6DeviceRef.Log(type, value);
}


//---------------------------------------------------------------------------
//  X6Board::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

IFpaLoader & X6Board::VsProm()
{
    return X6DeviceRef.VsProm();
}

//---------------------------------------------------------------------------
//  X6Board::Velo() -- Return reference to Velocia Packetizer Control
//---------------------------------------------------------------------------

IVeloPacketizer & X6Board::Velo()
{
    return X6DeviceRef.Velo();
}

//---------------------------------------------------------------------------
//  X6Board::VitaIn() -- Return reference to Vita Packetizer Control
//---------------------------------------------------------------------------

IVitaPacketizer & X6Board::VitaIn()
{
    return X6DeviceRef.VitaIn();
}

//---------------------------------------------------------------------------
//  X6Board::VitaIn_TS() -- Return reference to Vita Timestamp Control
//---------------------------------------------------------------------------

IVitaTimestamp & X6Board::VitaIn_TS()
{
    return X6DeviceRef.VitaIn_TS();
}

//---------------------------------------------------------------------------
//  X6Board::VeloOut() -- Return reference to Velocia Depacketizer Control
//---------------------------------------------------------------------------

IVeloDepacketizer & X6Board::VeloOut()
{
    return X6DeviceRef.VeloOut();
}

//---------------------------------------------------------------------------
//  X6Board::VitaOut() -- Return reference to Vita Depacketizer Control
//---------------------------------------------------------------------------

IVitaDepacketizer & X6Board::VitaOut()
{
    return X6DeviceRef.VitaOut();
}

//------------------------------------------------------------------------------
//  X6Board::PciLoopbackMode() --
//------------------------------------------------------------------------------

void X6Board::PciLoopbackMode(LoopbackMode mode)
{
    // This field does not use the same bit pattern, so improvise...
    int mode_value;
    switch (mode)
        {
        default:
        case lmNone:        mode_value = 0;     break;
        case lmSerial:      mode_value = 1;     break;
        case lmParallel:    mode_value = 2;     break;
        }

    X6DeviceRef.PciLoopbackMode(mode_value);
}

//------------------------------------------------------------------------------
//  X6Board::PciLoopbackMode() --
//------------------------------------------------------------------------------

X6Board::LoopbackMode X6Board::PciLoopbackMode()
{
    X6Board::LoopbackMode result;
    switch (X6DeviceRef.PciLoopbackMode())
        {
        default:
        case 0:     result = lmNone;        break;
        case 1:     result = lmSerial;      break;
        case 2:     result = lmParallel;    break;
        }

    return result;
}

//---------------------------------------------------------------------------
//  X6Board::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX6Alerts & X6Board::Alerts()
{
    return X6DeviceRef.Alerts();
}

//---------------------------------------------------------------------------
//  X6Board::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

IX6ThermalDevice & X6Board::Thermal()
{
    return X6DeviceRef.Thermal();
}

//---------------------------------------------------------------------------
//  X6Board::Dio() -- DIO port access
//---------------------------------------------------------------------------

AdvancedDioPortIntf & X6Board::Dio()
{
    return X6DeviceRef.Dio();
}

//---------------------------------------------------------------------------
//  X6Board::Clock() --
//---------------------------------------------------------------------------

IX6ClockIo & X6Board::Clock()
{
    return X6DeviceRef.Clock();
}

//---------------------------------------------------------------------------
//  X6Board::Info() --
//---------------------------------------------------------------------------

const IX6LogicVersionInfo & X6Board::Info() const
{
    return X6DeviceRef.Info();
}

//---------------------------------------------------------------------------
//  X6Board::Aurora() -- return ref to Aurora subdevs
//---------------------------------------------------------------------------

IX6Aurora &  X6Board::Aurora(unsigned int idx)
{
    return X6DeviceRef.Aurora(idx);
}

//---------------------------------------------------------------------------
//  X6Board::IdRom() --
//---------------------------------------------------------------------------

IIdRom & X6Board::IdRom()
{
    return X6DeviceRef.IdRom();
}

//---------------------------------------------------------------------------
//  X6Board::BusMasterSize() --
//---------------------------------------------------------------------------

void X6Board::BusMasterSize(int bytes)
{
    X6DeviceRef.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X6Board::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void X6Board::IncomingBusMasterSize(int bytes)
{
    X6DeviceRef.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X6Board::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void X6Board::OutgoingBusMasterSize(int bytes)
{
    X6DeviceRef.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  X6Board::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  X6Board::IncomingBusMasterSize() const
{
    return X6DeviceRef.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X6Board::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  X6Board::OutgoingBusMasterSize() const
{
    return X6DeviceRef.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  X6Board::ScatterGatherMode() --
//---------------------------------------------------------------------------

void  X6Board::ScatterGatherMode(bool state)
{
    return X6DeviceRef.ScatterGatherMode(state);   
}

//---------------------------------------------------------------------------
//  X6Board::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  X6Board::PeekLogic(int offset )
{
    return X6DeviceRef.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  X6Board::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  X6Board::PokeLogic( int offset, ii32 value )
{
    X6DeviceRef.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  X6Board::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  X6Board::PeekPort(int offset )
{
    return X6DeviceRef.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  X6Board::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  X6Board::PokePort( int offset, ii32 value )
{
    X6DeviceRef.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  X6Board::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6Board::AfterFiclStart(FiclSystem & engine)
{
    //  Replace Callbacks to call FICL too
    X6DeviceRef.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  X6Board::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  X6Board::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  X6Board::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  X6Board::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  X6Board::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  X6Board::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  X6Board::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  X6Board::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
