// X6Board_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef X6Board_MbH
#define X6Board_MbH

#include <DataTypes_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS X6Board  -- X6 Common baseboard class
//===========================================================================

class X6Board : public IIsAVitaPacketStreamBoard,   
                public IStreamDeviceEvents,
                public IFiclTarget
{

public:
    // Trigger Support
    // Enumerations
    enum LoopbackMode { lmNone, lmParallel, lmSerial };    

    //
    //  Member Functions
    X6Board(X6DeviceMap & deviceMap);
    virtual ~X6Board();

    //  Informational Methods
    size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      DriverDate();
    string                      DriverVersion();
    string                      PrintAllDevices();
	bool                        LicenseCheck(unsigned int & code);
	
	std::string                 Name() const
        {  return X6DeviceRef.Name();  }
    std::string                 Revision() const
        {  return X6DeviceRef.Revision();  }

    int                         Target();
    void                        Target(int value);

    bool                        Led();
    void                        Led(bool value);
    bool                        GreenLed();
    void                        GreenLed(bool value);
    bool                        FrontPanelLed();
    void                        FrontPanelLed(bool value);

    virtual void                Open();
    virtual void                Close();
    virtual void                Reset();
    //
    //  Properties
    void                        PciLoopbackMode(LoopbackMode mode);
    LoopbackMode                PciLoopbackMode();

    void                        ScatterGatherMode(bool state);

    void                        BusMasterSize(int bytes);
    void                        IncomingBusMasterSize(int bytes);
    void                        OutgoingBusMasterSize(int bytes);
    int                         IncomingBusMasterSize() const;
    int                         OutgoingBusMasterSize() const;

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    IFpaLoader &           VsProm();
    virtual IX6ClockIo &   Clock();
    AdvancedDioPortIntf &  Dio();
    
    const IX6LogicVersionInfo &  Info() const;
    IVeloPacketizer &            Velo();
	IVeloPacketizer &            VeloIn()  {  return Velo();   }
    IVitaPacketizer &            VitaIn();
    IVitaTimestamp &             VitaIn_TS();
    IUsesX6Alerts &              Alerts();
    IX6ThermalDevice &           Thermal();

    IVeloDepacketizer &          VeloOut();
    IVitaDepacketizer &          VitaOut();
    
    IX6Aurora &                  Aurora(unsigned int idx);

    IIdRom &                     IdRom();

public: //Pseudo protected
    //
    //  IPacketStreamDevice interface implementation
    virtual IPacketStreamDeviceInternals *  Internals();

    X6DeviceMap * Debug() { return &X6DeviceRef; };
    virtual void   Log(int type, int data);





    //FPGA type name function
std::string FpgaName(const short Number)
{
    switch(static_cast<int>(Number))
    {
     case 0: return "LX5550T";
     case 1: return "LX240T";
     case 2: return "SX315T";
     case 3: return "SX475T";
     default: return "Unknown";
    }
}

//FPGA type name function
std::string FamilyName(const short Number)
{
	switch (static_cast<int>(Number))
	{
	case 0: return "X3";
	case 1: return "X5";
	case 2: return "X6";
	case 3: return "K7";
	default: return "Unknown";
	}
}

//FPGA type name function
std::string BoardName(const short Number)
{
	switch (static_cast<int>(Number))
	{
	case 0: return "No Board";
	case 1: return "X6-RX";
	case 2: return "X6-400M";
	case 3: return "X6-GSPS";
	case 4: return "X6-250M";
	case 5: return "X6-1000M";
	default: return "Unknown";
	}
}


protected:
    //
    //  Member Data
    X6DeviceMap &                  X6DeviceRef;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( X6Board & board );
    friend AddressingSpace & PortMemorySpace( X6Board & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( X6Board & board )
{
    return board.X6DeviceRef.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( X6Board & board )
{
    return board.X6DeviceRef.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

