// X6DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X6DeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <X6DeviceDriver_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{
#ifdef __GNUC__
    #pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6HdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class X6HdwPluginImpl : public IPacketDevicePlugin
{
public:
    X6HdwPluginImpl( X6DeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    X6DeviceMap * PDM;
};

//===========================================================================
//  CLASS X6DeviceMap  -- Registers for X6 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6DeviceMap
//---------------------------------------------------------------------------
//
//  IDROM Page Numbers

const unsigned int System_IdromPage = 0;
const unsigned int AdcCal_IdromPage = 1;
const unsigned int DacCal_IdromPage = 2;
const unsigned int User_IdromPage   = 3;

//
//  Subdevice Register Structures

using namespace X6_MemoryMap;


PacketDeviceRegisters X6DRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	mmLinkControl,        // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters X6RegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

X6Info::MapRegisters X6InfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmBitstreamDate,      //  BitstreamDate_Addr
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

X6Peripherals::MapRegisters X6PeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

X6SioPll::MapRegisters X6SioPllRegAddrs =
    {
    mmSioXo,              //  SioPll_Addr
    };

X6FpaLoader::MapRegisters X6FpaRegAddrs =
    {
    mmFpa_Address,        //  FpaAddress_Addr
    mmFpa_Data,           //  FpaData_Addr
    mmFpa_Command,        //  FpaCommand_Addr
    };

X6Thermal::MapRegisters X6ThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

X6PacketizerCfg::MapRegisters X6PacketizerRegAddrs =
    {
    mmPktzer_Enable,          //  PacketizerEnable_Addr
    mmPktzer_AuxHdr,          //  AuxHeader_Addr
    mmPktzer_VeloInHdr,       //  VelociaInHeader_Addr
    mmPktzer_ForceSize,       //  PacketizerForceSize_Addr
    mmPktzer_IntervalPeriod,  //  PacketizerIntervalPeriod_Addr
    };

X6AlertSystem::MapRegisters X6AlertsRegAddrs =
    {
    mmAlert_Enable,       //  AlertEnable_Addr
    mmAlert_Control,      //  AlertControl_Addr
    mmAlert_Software,     //  SoftwareAlert_Addr;
    mmAlert_Clear,        //  ClearAlert_Addr;
    };

X6DigitalIo::MapRegisters X6DioRegAddrs =
    {
    mmDio_DataLo,          //  DioDataLo_Addr
    mmDio_DataHi,          //  DioDataHi_Addr
    mmDio_OutEnableLo,     //  DioOutputEnableLo_Addr
    mmDio_OutEnableHi,     //  DioOutputEnableHi_Addr
    };

X6FlashRom::MapRegisters X6FlashRomRegAddrs =
    {
    mmRom_AddrCtl,         //  AddressControl_Addr
    mmRom_Data,            //  Data_Addr
    mmRom_Opcode,          //  Opcode_Addr
    };

X6AFE_Clock::MapRegisters X6AfeClkRegAddrs =
    {
    mmAFE_PllControl,       //  AfePllControl_Addr
    mmAFE_PllSpi,           //  AfePllSpi_Addr
    mmAFE_ClkControl,       //  AfeClkControl_Addr
    };

X6AFE_Common::MapRegisters X6AfeCommonRegAddrs =
    {
    mmAFE_TestControl,       //  AfeTestControl_Addr
    mmAFE_FlowControl,       //  AfeFlowControl_Addr
    mmAFE_SoftwareTrigger,   //  AfeSoftwareTrigger_Addr
    mmAFE_ExtSyncSelect,     //  AfeExtSyncSelect_Addr
    };

X6AFE_AdcCommon::MapRegisters X6AfeAdcCommonRegAddrs =
    {
    mmAFE_AdcEnable,         //  AfeAdcEnable_Addr
    mmAFE_AdcTrigger,        //  AfeAdcTriggerCfg_Addr
    mmAFE_AdcDecimate,       //  AfeAdcDecimation_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters X6AfeAdcPriRegAddrs =
    {
    mmAFE_AdcPriEnable,      //  PriTriggerEnable_Addr;
    mmAFE_AdcPriInterval,    //  PriInterval_Addr;
    mmAFE_AdcPriTrigParam,   //  PriTriggerParam_Addr;
    mmAFE_AdcCaptureWin      //  PriCaptureWindow_Addr;
    };

X6VitaPacketizerCfg::MapRegisters X6VitaPacketizerRegAddrs =
    {
    mmAFE_AdcTStampInit,     //  TimestampInit_Addr
    mmAFE_AdcTStampCtl,      //  TimestampCtrl_Addr
    mmAFE_AdcVitaSize,       //  VitaAdcSize_Addr
    mmAFE_AdcVitaStreamId,   //  VitaAdcStreamId_Addr
    };

X6AFE_AdcCalibration::MapRegisters X6AfeAdcCalibrationRegAddrs =
    {
    mmAFE_AdcGainCoeff,      //  GainCoeff_Addr;
    mmAFE_AdcOffsetCoeff,    //  OffsetCoeff_Addr
    };

X6Aurora::MapRegisters X6Aurora0RegAddrs =
    {
    mmAurora_TestControl,    //  TestControl_Addr
    mmAurora_ControlStatus,  //  ControlStatus_Addr
    mmAurora_WritePort,      //  WritePort_Addr
    mmAurora_ReadPort,       //  ReadPort_Addr
    mmAurora_SfpPortControl, //  SfpPortControl_Addr
	mmAurora_SfpPortI2C,     //  SfpPortI2C_Addr
	};

X6Aurora::MapRegisters X6Aurora1RegAddrs =
    {
    mmAurora_TestControl,   //  TestControl_Addr
    mmAurora_ControlStatus, //  ControlStatus_Addr
    mmAurora_WritePort,     //  WritePort_Addr
    mmAurora_ReadPort,      //  ReadPort_Addr
    mmAurora_SfpPortControl, //  SfpPortControl_Addr
    mmAurora_SfpPortI2C,     //  SfpPortI2C_Addr
    };

X6Aurora::MapRegisters X6Aurora2RegAddrs =
    {
    mmAurora_TestControl_1,   //  TestControl_Addr
    mmAurora_ControlStatus_1, //  ControlStatus_Addr
    mmAurora_WritePort_1,     //  WritePort_Addr
    mmAurora_ReadPort_1,      //  ReadPort_Addr
    };

X6Aurora::MapRegisters X6Aurora3RegAddrs =
    {
    mmAurora_TestControl_1,   //  TestControl_Addr
    mmAurora_ControlStatus_1, //  ControlStatus_Addr
    mmAurora_WritePort_1,     //  WritePort_Addr
    mmAurora_ReadPort_1,      //  ReadPort_Addr
    };


X6DepacketizerCfg::MapRegisters X6DepacketizerRegAddrs =
    {
    mmVeloOutTestLB,        //  TestLoopbackPid_Addr;
    mmVeloOutDataPid,       //  OutgoingDataPid_Addr
    };

X6AFE_DacCommon::MapRegisters X6AfeDacCommonRegAddrs =
    {
    mmAFE_DacEnable,         //  AfeDacEnable_Addr
    mmAFE_DacTrigger,        //  AfeDacTriggerCfg_Addr
    mmAFE_DacDecimate,       //  AfeDacDecimation_Addr
    mmAFE_DacReset,          //  AfeDacReset_Addr
    mmAFE_DacPhaseIncr,      //  AfeDacPhaseIncrement_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters X6AfeDacPriRegAddrs =
    {
    mmAFE_DacPriEnable,      //  PriTriggerEnable_Addr;
    mmAFE_DacPriInterval,    //  PriInterval_Addr;
    mmAFE_DacPriTrigParam,   //  PriTriggerParam_Addr;
    mmAFE_DacCaptureWin      //  PriCaptureWindow_Addr;
    };

X6AFE_DacLatencyCalibration::MapRegisters X6AfeLatencyCalRegAddrs =
    {
    mmAFE_DacLatencyCal,      //  LatencyCal_Addr;
    };

X6AFE_DacCalibration::MapRegisters X6AfeDacCalibrationRegAddrs =
    {
    mmAFE_DacGainCoeff,      //  GainCoeff_Addr;
    mmAFE_DacOffsetCoeff,    //  OffsetCoeff_Addr
    };

X6_DacPatternModeSystem::MapRegisters X6DacPatternModeRegAddrs =
    {
    mmLPDDR,                 //  PatternModeEnable_Addr;
    };

X6VitaDepacketizerCfg::MapRegisters X6VitaDepacketizerRegAddrs =
    {
    mmAFE_DacVitaStreamId,   //  VitaDacStreamId_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class X6DeviceMap
//---------------------------------------------------------------------------
const bool  BitTS = true;    // supports bitstream timestamps.


X6DeviceMap::X6DeviceMap(IStreamDeviceEvents * owner)
    :  PacketDeviceImpl         (new X6DeviceDriver, X6DRegAddrs),
       PacketDeviceMap          (this, X6RegAddrs, 0),
       IUsesStreamDeviceEventCallbacks(owner),
       DDeviceDriver            (dynamic_cast<X6DeviceDriver*>(DeviceDriver)),
       WB_Periph                (LogicMemory, wbPeripheral),
       WB_Loader                (LogicMemory, wbLogicLoader),
       WB_Thermal               (LogicMemory, wbTempControl),
       WB_Packetizer            (LogicMemory, wbPacketizer),
       WB_Alerts                (LogicMemory, wbAlerts),
       WB_DigitalIo             (LogicMemory, wbDigitalIo),
       WB_FlashRom              (LogicMemory, wbROM),
       WB_AFE                   (LogicMemory, wbAFE),
       WB_Aurora0               (LogicMemory, wbAurora0),
       WB_Aurora1               (LogicMemory, wbAurora1),
       Info_SDev                (this, WB_Periph, PortMemory, X6InfoRegAddrs, BitTS),
       Periph_SDev              (this, WB_Periph, X6PeripheralRegAddrs),
       SioPll_SDev              (this, WB_Periph, X6SioPllRegAddrs),
       Fpa_SDev                 (this, WB_Loader, X6FpaRegAddrs),
       Thermal_SDev             (this, WB_Thermal, X6ThermalRegAddrs),
       Velo_SDev                (this, WB_Packetizer, X6PacketizerRegAddrs),
       Alerts_SDev              (this, WB_Alerts, X6AlertsRegAddrs),
       Dio_SDev                 (this, WB_DigitalIo, X6DioRegAddrs),
       FlashRom_SDev            (this, WB_FlashRom, X6FlashRomRegAddrs),
       AFEClk_SDev              (this, WB_AFE, X6AfeClkRegAddrs),
       AFECommon_SDev           (this, WB_AFE, X6AfeCommonRegAddrs),
       AFEAdcC_SDev             (this, WB_AFE, X6AfeAdcCommonRegAddrs),
       AFEAdcPri_SDev           (this, true, WB_AFE, X6AfeAdcPriRegAddrs),
       AFEAdcSpi_SDev           (this, WB_AFE),
       VitaIn_SDev              (this, WB_AFE, X6VitaPacketizerRegAddrs),
       AFEAdcCal_SDev           (this, WB_AFE, X6AfeAdcCalibrationRegAddrs, mmAFE_TotalAdcs),
       Aurora0_SDev             (this, WB_Aurora0, X6Aurora0RegAddrs),
       Aurora1_SDev             (this, WB_Aurora1, X6Aurora1RegAddrs),
       Aurora2_SDev             (this, WB_Aurora0, X6Aurora2RegAddrs),
       Aurora3_SDev             (this, WB_Aurora1, X6Aurora3RegAddrs),
       VeloOut_SDev             (this, WB_Periph, X6DepacketizerRegAddrs),
       AFEDacC_SDev             (this, WB_AFE, X6AfeDacCommonRegAddrs),
       AFEDacPri_SDev           (this, false, WB_AFE, X6AfeDacPriRegAddrs),
       AFEDacSpi_SDev           (this, WB_AFE),
       AFEDacLatencyCal_SDev    (this, WB_AFE, X6AfeLatencyCalRegAddrs),
       AFEDacCal_SDev           (this, WB_AFE, X6AfeDacCalibrationRegAddrs),
       DacPatternMode_SDev      (this, WB_Periph, X6DacPatternModeRegAddrs),
       VitaOut_SDev             (this, WB_AFE, X6VitaDepacketizerRegAddrs),
       System_PG                (FlashRom_SDev, System_IdromPage),
       AdcCal_PG                (FlashRom_SDev, AdcCal_IdromPage),
       DacCal_PG                (FlashRom_SDev, DacCal_IdromPage),
       UserRom_PG               (FlashRom_SDev, User_IdromPage),
       IdRom_Intf               (this)
{
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new X6HdwPluginImpl(this) );

    //  Enable use of dedicated register to signal Kernel Cache ready
    DDeviceDriver->EnableDedicatedKernelCacheRelease(PortMemory, X6_MemoryMap::mmKernelCacheAvail);

}

//---------------------------------------------------------------------------
//  destructor for class X6DeviceMap
//---------------------------------------------------------------------------

X6DeviceMap::~X6DeviceMap()
{
}

//---------------------------------------------------------------------------
//  X6DeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6DeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    SioPll_SDev.ConstructCustomParts();
    Velo_SDev.ConstructCustomParts();
    Alerts_SDev.ConstructCustomParts();
    Fpa_SDev.ConstructCustomParts();
    Thermal_SDev.ConstructCustomParts();
    Dio_SDev.ConstructCustomParts();
    FlashRom_SDev.ConstructCustomParts();
    AFEClk_SDev.ConstructCustomParts();
    AFECommon_SDev.ConstructCustomParts();
    AFEAdcC_SDev.ConstructCustomParts();
    AFEAdcPri_SDev.ConstructCustomParts();
    AFEAdcSpi_SDev.ConstructCustomParts();
    VitaIn_SDev.ConstructCustomParts();
    AFEAdcCal_SDev.ConstructCustomParts();
    Aurora0_SDev.ConstructCustomParts();
    Aurora1_SDev.ConstructCustomParts();
    Aurora2_SDev.ConstructCustomParts();
    Aurora3_SDev.ConstructCustomParts();
    VeloOut_SDev.ConstructCustomParts();
    AFEDacC_SDev.ConstructCustomParts();
    AFEDacPri_SDev.ConstructCustomParts();
    AFEDacSpi_SDev.ConstructCustomParts();
    AFEDacLatencyCal_SDev.ConstructCustomParts();
    AFEDacCal_SDev.ConstructCustomParts();
    DacPatternMode_SDev.ConstructCustomParts();
    VitaOut_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  X6DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X6DeviceMap::OpenHardware()
{
    //
    //  Reset the device
    MasterReset();

    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    SioPll_SDev.OpenHardware();
    Velo_SDev.OpenHardware();
    Alerts_SDev.OpenHardware();
    Fpa_SDev.OpenHardware();
    Thermal_SDev.OpenHardware();
    Dio_SDev.OpenHardware();
    FlashRom_SDev.OpenHardware();
    AFEClk_SDev.OpenHardware();
    AFECommon_SDev.OpenHardware();
    AFEAdcC_SDev.OpenHardware();
    AFEAdcPri_SDev.OpenHardware();
    AFEAdcSpi_SDev.OpenHardware();
    VitaIn_SDev.OpenHardware();
    AFEAdcCal_SDev.OpenHardware();
    Aurora0_SDev.OpenHardware();
    Aurora1_SDev.OpenHardware();
    Aurora2_SDev.OpenHardware();
    Aurora3_SDev.OpenHardware();

    VeloOut_SDev.OpenHardware();

    AFEDacC_SDev.OpenHardware();
    AFEDacPri_SDev.OpenHardware();
    AFEDacSpi_SDev.OpenHardware();
    AFEDacLatencyCal_SDev.OpenHardware();
    AFEDacCal_SDev.OpenHardware();
    DacPatternMode_SDev.OpenHardware();
    VitaOut_SDev.OpenHardware();
    //
    //  Read all ROM pages to Init Cache
    System_PG.LoadFromRom();
    AdcCal_PG.LoadFromRom();
    DacCal_PG.LoadFromRom();
    UserRom_PG.LoadFromRom();

}

//---------------------------------------------------------------------------
//  X6DeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  X6DeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    SioPll_SDev.PreconfigureHardware();
    Velo_SDev.PreconfigureHardware();
    Alerts_SDev.PreconfigureHardware();
    Fpa_SDev.PreconfigureHardware();
    Thermal_SDev.PreconfigureHardware();
    Dio_SDev.PreconfigureHardware();
    FlashRom_SDev.PreconfigureHardware();
    AFEClk_SDev.PreconfigureHardware();
    AFECommon_SDev.PreconfigureHardware();
    AFEAdcC_SDev.PreconfigureHardware();
    AFEAdcPri_SDev.PreconfigureHardware();
    AFEAdcSpi_SDev.PreconfigureHardware();
    VitaIn_SDev.PreconfigureHardware();
    AFEAdcCal_SDev.PreconfigureHardware();
    Aurora0_SDev.PreconfigureHardware();
    Aurora1_SDev.PreconfigureHardware();
    Aurora2_SDev.PreconfigureHardware();
    Aurora3_SDev.PreconfigureHardware();

    VeloOut_SDev.PreconfigureHardware();
    AFEDacC_SDev.PreconfigureHardware();
    AFEDacPri_SDev.PreconfigureHardware();
    AFEDacSpi_SDev.PreconfigureHardware();
    AFEDacLatencyCal_SDev.PreconfigureHardware();
    AFEDacCal_SDev.PreconfigureHardware();
    DacPatternMode_SDev.PreconfigureHardware();
    VitaOut_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  X6DeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6DeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();

    // Subdevice Configuration
    AFECommon_SDev.ConfigureHardware();  // do early to disable triggers/engine

    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    SioPll_SDev.ConfigureHardware();
    Velo_SDev.ConfigureHardware();
    Alerts_SDev.ConfigureHardware();
    Fpa_SDev.ConfigureHardware();
    Thermal_SDev.ConfigureHardware();
    Dio_SDev.ConfigureHardware();
    FlashRom_SDev.ConfigureHardware();
    AFEClk_SDev.ConfigureHardware();
    AFEAdcC_SDev.ConfigureHardware();
    AFEAdcPri_SDev.ConfigureHardware();
    AFEAdcSpi_SDev.ConfigureHardware();
    VitaIn_SDev.ConfigureHardware();
    AFEAdcCal_SDev.ConfigureHardware();
    Aurora0_SDev.ConfigureHardware();
    Aurora1_SDev.ConfigureHardware();
    Aurora2_SDev.ConfigureHardware();
    Aurora3_SDev.ConfigureHardware();
    VeloOut_SDev.ConfigureHardware();
    AFEDacC_SDev.ConfigureHardware();
    AFEDacPri_SDev.ConfigureHardware();
    AFEDacSpi_SDev.ConfigureHardware();
    AFEDacLatencyCal_SDev.ConfigureHardware();
    AFEDacCal_SDev.ConfigureHardware();
    DacPatternMode_SDev.ConfigureHardware();
    VitaOut_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  X6DeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  X6DeviceMap::StreamStopHardware()
{
    AFECommon_SDev.AllOff();    // engine off, test off, trigger off

    // Subdevice Finalization
    VitaIn_SDev.StreamStopHardware();
    AFEAdcPri_SDev.StreamStopHardware();
    AFEDacPri_SDev.StreamStopHardware();
    VitaOut_SDev.StreamStopHardware();

}

//---------------------------------------------------------------------------
//  X6DeviceMap::EnableDacDevices() -- turn on DAC specific subdevices
//---------------------------------------------------------------------------

void  X6DeviceMap::EnableDacDevices()
{
    AFEDacC_SDev.DeviceEnabled(true);
    AFEDacPri_SDev.DeviceEnabled(true);
    AFEDacSpi_SDev.DeviceEnabled(true);
    AFEDacLatencyCal_SDev.DeviceEnabled(true);
    AFEDacCal_SDev.DeviceEnabled(true);
    DacPatternMode_SDev.DeviceEnabled(true);
}

//---------------------------------------------------------------------------
//  X6DeviceMap::DisableDacDevices() -- turn off DAC specific subdevices
//---------------------------------------------------------------------------

void  X6DeviceMap::DisableDacDevices()
{
    AFEDacC_SDev.DeviceEnabled(false);
    AFEDacPri_SDev.DeviceEnabled(false);
    AFEDacSpi_SDev.DeviceEnabled(false);
    AFEDacLatencyCal_SDev.DeviceEnabled(false);
    AFEDacCal_SDev.DeviceEnabled(false);
    DacPatternMode_SDev.DeviceEnabled(false);
}

//---------------------------------------------------------------------------
//  X6DeviceMap::DisablePriDevices() -- turn off ADC/DAC PRI specific subdevices
//---------------------------------------------------------------------------

void  X6DeviceMap::DisablePriDevices()
{
    AFEAdcPri_SDev.DeviceEnabled(false);
    AFEDacPri_SDev.DeviceEnabled(false);
}

//---------------------------------------------------------------------------
//  X6DeviceMap::AlertTest() -- Alert Test Processing Hook
//---------------------------------------------------------------------------

bool  X6DeviceMap::AlertTest(unsigned int pid)
{
    //  return TRUE if PID is an alert PID.
    //  return FALSE if not
    return Alerts_SDev.AlertTest(pid);
}

//---------------------------------------------------------------------------
//  X6DeviceMap::AlertCheck() -- Alert Event Processing Hook
//---------------------------------------------------------------------------

bool X6DeviceMap::AlertCheck(const Buffer & buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alerts_SDev.CheckAndProcess(buffer);
}

//---------------------------------------------------------------------------
//  X6DeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

//void  X6DeviceMap::DumpIsrStatus(std::vector<int> & vec)
void  X6DeviceMap::DumpIsrStatus(IntVec & vec)
{
    vec.clear();
    for (int i=0; i<X6DeviceDriver::LOG->Count; i++)
        vec.push_back(X6DeviceDriver::LOG->Status[i]);
}

//---------------------------------------------------------------------------
//  X6DeviceMap::Name() --  Return name of module as text
//---------------------------------------------------------------------------

std::string  X6DeviceMap::Name() const
{
    switch(static_cast<int>(Info().PciLogicType()))
    {
		case 1: return "X6-RX";
		case 2: return "X6-400M";
		case 3: return "X6-GSPS";
		case 4: return "X6-250M";
		case 5: return "X6-1000M";
		default: return "Unknown";
    }
}

//---------------------------------------------------------------------------
//  X6DeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  X6DeviceMap::Revision() const
{
	std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  X6DeviceMap::Aurora() -- return ref to Aurora subdevs
//---------------------------------------------------------------------------

IX6Aurora &  X6DeviceMap::Aurora(unsigned int idx)
{
    switch (idx)
    {
    case 0:
    default:
        return Aurora0_SDev;
    case 1:
        return Aurora1_SDev;
    case 2:
        return Aurora2_SDev;
    case 3:
        return Aurora3_SDev;
    }
}

//===========================================================================
//  CLASS X6InputDevice  -- Partially Implemented X6 Input Device
//===========================================================================
//---------------------------------------------------------------------------
//  X6InputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void  X6InputDevice::ChannelDisableAll()
{
     AdcCommon().ChannelDisableAll();
     AdcCommon().ChannelPowerDownAll();
}

//---------------------------------------------------------------------------
//  X6InputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void  X6InputDevice::ChannelEnabled(int ch, bool state)
{
     AdcCommon().ChannelEnabled(ch, state);
     ChannelPowered(ch, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  X6InputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool X6InputDevice::ChannelEnabled(int ch) const
{
    return AdcCommon().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  X6InputDevice::ActiveChannels() -- Return number of active channels
//---------------------------------------------------------------------------

unsigned int X6InputDevice::ActiveChannels()
{
    return AdcCommon().ActiveChannels();
}

//---------------------------------------------------------------------------
//  X6InputDevice::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  X6InputDevice::ChannelPowered(int ch, bool state)
{
     AdcCommon().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  X6InputDevice::ChannelPowered() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

bool X6InputDevice::ChannelPowered(int ch) const
{
    return AdcCommon().ChannelPowered(ch);
}

//---------------------------------------------------------------------------
//  X6InputDevice::SoftwareTrigger() -- Enable Test Mode on input
//---------------------------------------------------------------------------

void  X6InputDevice::SoftwareTrigger(bool state)
{
    Common().SoftwareTrigger(X6AFE_Common::tInput, state);
}

//---------------------------------------------------------------------------
//  X6InputDevice::SoftwareTrigger() -- Enable Test Mode on input
//---------------------------------------------------------------------------

void  X6InputDevice::SoftwareTrigger(AfeDirection which, bool state)
{
    X6AFE_Common::AfeDirection which2 = static_cast<X6AFE_Common::AfeDirection>(which);
    Common().SoftwareTrigger(which2, state);
}

//---------------------------------------------------------------------------
//  X6InputDevice::TestModeEnabled() -- Enable Test Mode on input
//---------------------------------------------------------------------------

void  X6InputDevice::TestModeEnabled(bool state, unsigned int mode)
{
    Common().TestModeEnabled(X6AFE_Common::tInput, state, mode);
}

//---------------------------------------------------------------------------
//  X6InputDevice::Decimation() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  X6InputDevice::Decimation(unsigned int samples)
{
     AdcCommon().Decimation(samples);
}

//---------------------------------------------------------------------------
//  X6InputDevice::Decimation() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

unsigned int X6InputDevice::Decimation() const
{
    return AdcCommon().Decimation();
}

//---------------------------------------------------------------------------
//  X6InputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  X6InputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    AdcSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  X6InputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  X6InputDevice::Spi(unsigned int device, unsigned int address) const
{
    return AdcSpi().Spi(device, address);
}

//===========================================================================
//  CLASS X6InputDevice::TriggerCfgImpl  --
//===========================================================================
//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::FramedMode() -- Setter
//---------------------------------------------------------------------------

void  X6InputDevice::TriggerCfgImpl::FramedMode(bool state)
{
    InD->AdcCommon().FramedTriggerEnabled(state);
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::FramedMode() -- Getter
//---------------------------------------------------------------------------

bool X6InputDevice::TriggerCfgImpl::FramedMode() const
{
    return InD->AdcCommon().FramedTriggerEnabled();
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::Edge() -- Setter
//---------------------------------------------------------------------------

void  X6InputDevice::TriggerCfgImpl::Edge(bool state)
{
    InD->AdcCommon().EdgeTrigger(state);
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::Edge() -- Getter
//---------------------------------------------------------------------------

bool X6InputDevice::TriggerCfgImpl::Edge() const
{
    return InD->AdcCommon().EdgeTrigger();
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::External() -- Setter
//---------------------------------------------------------------------------

void  X6InputDevice::TriggerCfgImpl::External(bool state)
{
    InD->AdcCommon().ExternalTrigger(state);
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::External() -- Getter
//---------------------------------------------------------------------------

bool X6InputDevice::TriggerCfgImpl::External() const
{
    return InD->AdcCommon().ExternalTrigger();
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::FrameSize() -- Setter
//---------------------------------------------------------------------------

void  X6InputDevice::TriggerCfgImpl::FrameSize(unsigned int samples)
{
    InD->AdcCommon().FramedTriggerSize(samples);
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::FrameSize() -- Getter
//---------------------------------------------------------------------------

unsigned int X6InputDevice::TriggerCfgImpl::FrameSize() const
{
    return InD->AdcCommon().FramedTriggerSize();
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::ExternalSyncSource() -- Setter
//---------------------------------------------------------------------------

void  X6InputDevice::TriggerCfgImpl::ExternalSyncSource(AfeExtSyncOptions src)
{
    InD->Common().ExternalSyncSelection(
                static_cast<X6AFE_Common::AfeExtSyncOptions>(src) );
}

//---------------------------------------------------------------------------
//  X6InputDevice::TriggerCfgImpl::ExternalSyncSource() -- Getter
//---------------------------------------------------------------------------

X6InputDevice::AfeExtSyncOptions X6InputDevice::TriggerCfgImpl::ExternalSyncSource() const
{
    X6AFE_Common::AfeExtSyncOptions opt = InD->Common().ExternalSyncSelection();
    return static_cast<X6InputDevice::AfeExtSyncOptions>( opt );
}

//===========================================================================
//  CLASS X6OutputDevice  -- Partially Implemented X6 Output Device
//===========================================================================
//---------------------------------------------------------------------------
//  X6OutputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void  X6OutputDevice::ChannelDisableAll()
{
     DacCommon().ChannelDisableAll();
     DacCommon().ChannelPowerDownAll();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void  X6OutputDevice::ChannelEnabled(int ch, bool state)
{
     DacCommon().ChannelEnabled(ch, state);
     ChannelPowered(ch, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  X6OutputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool X6OutputDevice::ChannelEnabled(int ch) const
{
    return DacCommon().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::ActiveChannels() -- Return number of active channels
//---------------------------------------------------------------------------

unsigned int X6OutputDevice::ActiveChannels()
{
    return DacCommon().ActiveChannels();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  X6OutputDevice::ChannelPowered(int ch, bool state)
{
     DacCommon().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::ChannelPowered() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

bool X6OutputDevice::ChannelPowered(int ch) const
{
    return DacCommon().ChannelPowered(ch);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::SoftwareTrigger() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  X6OutputDevice::SoftwareTrigger(bool state)
{
    Common().SoftwareTrigger(X6AFE_Common::tOutput, state);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::SoftwareTrigger() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  X6OutputDevice::SoftwareTrigger(AfeDirection which, bool state)
{
    X6AFE_Common::AfeDirection which2 = static_cast<X6AFE_Common::AfeDirection>(which);
    Common().SoftwareTrigger(which2, state);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TestModeEnabled() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  X6OutputDevice::TestModeEnabled(bool state, unsigned int mode)
{
    Common().TestModeEnabled(X6AFE_Common::tOutput, state, mode);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TestFrequency() -- Set frequency for Test Mode on Output
//---------------------------------------------------------------------------

void  X6OutputDevice::TestFrequency(double frequency_hz)
{
    unsigned int channels = DacCommon().ActiveChannels();
    unsigned int dacdevs = ActiveDevices();
    unsigned int factor;
    if (channels==0 || dacdevs==0)
        factor = 1;
    else
        factor = channels/dacdevs;
    DacCommon().DacTestToneFrequency(Clock().Clock().Dac().Frequency(), frequency_hz, factor);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TestFrequency() -- Get frequency for Test Mode on Output
//---------------------------------------------------------------------------

double  X6OutputDevice::TestFrequency()
{
    unsigned int channels = DacCommon().ActiveChannels();
    if (channels==0)
         channels = 1;
    return DacCommon().DacTestToneFrequency(Clock().Clock().Dac().Frequency(), channels);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::Decimation() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  X6OutputDevice::Decimation(unsigned int samples)
{
     DacCommon().Decimation(samples);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::Decimation() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

unsigned int X6OutputDevice::Decimation() const
{
    return DacCommon().Decimation();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  X6OutputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    DacSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  X6OutputDevice::Spi(unsigned int device, unsigned int address) const
{
    return DacSpi().Spi(device, address);
}

//===========================================================================
//  CLASS X6OutputDevice::TriggerCfgImpl  --
//===========================================================================
//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::FramedMode() -- Setter
//---------------------------------------------------------------------------

void  X6OutputDevice::TriggerCfgImpl::FramedMode(bool state)
{
    OutD->DacCommon().FramedTriggerEnabled(state);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::FramedMode() -- Getter
//---------------------------------------------------------------------------

bool X6OutputDevice::TriggerCfgImpl::FramedMode() const
{
    return OutD->DacCommon().FramedTriggerEnabled();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::Edge() -- Setter
//---------------------------------------------------------------------------

void  X6OutputDevice::TriggerCfgImpl::Edge(bool state)
{
    OutD->DacCommon().EdgeTrigger(state);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::Edge() -- Getter
//---------------------------------------------------------------------------

bool X6OutputDevice::TriggerCfgImpl::Edge() const
{
    return OutD->DacCommon().EdgeTrigger();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::External() -- Setter
//---------------------------------------------------------------------------

void  X6OutputDevice::TriggerCfgImpl::External(bool state)
{
    OutD->DacCommon().ExternalTrigger(state);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::External() -- Getter
//---------------------------------------------------------------------------

bool X6OutputDevice::TriggerCfgImpl::External() const
{
    return OutD->DacCommon().ExternalTrigger();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::FrameSize() -- Setter
//---------------------------------------------------------------------------

void  X6OutputDevice::TriggerCfgImpl::FrameSize(unsigned int samples)
{
    OutD->DacCommon().FramedTriggerSize(samples);
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::FrameSize() -- Getter
//---------------------------------------------------------------------------

unsigned int X6OutputDevice::TriggerCfgImpl::FrameSize() const
{
    return OutD->DacCommon().FramedTriggerSize();
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::ExternalSyncSource() -- Setter
//---------------------------------------------------------------------------

void  X6OutputDevice::TriggerCfgImpl::ExternalSyncSource(AfeExtSyncOptions src)
{
    OutD->Common().ExternalSyncSelection(
                static_cast<X6AFE_Common::AfeExtSyncOptions>(src) );
}

//---------------------------------------------------------------------------
//  X6OutputDevice::TriggerCfgImpl::ExternalSyncSource() -- Getter
//---------------------------------------------------------------------------

X6OutputDevice::AfeExtSyncOptions X6OutputDevice::TriggerCfgImpl::ExternalSyncSource() const
{
    X6AFE_Common::AfeExtSyncOptions opt = OutD->Common().ExternalSyncSelection();
    return static_cast<X6OutputDevice::AfeExtSyncOptions>( opt );
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


} // namespace Innovative


