// X6DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X6DeviceMap_MbH
#define X6DeviceMap_MbH

#include <X6Support_Mb.h>
#include <X6Idrom_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>
#include <X6SubDevice_AnalogOut_Mb.h>

#include <PacketDeviceMap_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <StreamDeviceEvents_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X6DeviceMap;
class X6DeviceDriver;

//===========================================================================
//  CLASS X6DeviceMap  -- Register & low level support for X6
//===========================================================================

class X6DeviceMap :  public PacketDeviceImpl,
                     public PacketDeviceMap,
                     public IUsesStreamDeviceEventCallbacks,
                     public ISuppliesX6Requirements
{
public:
    friend class X6Board;
    friend class X6InputDevice;
    friend class X6OutputDevice;
    friend class X6StandardPriClockDevice;
    //
    //  Member Functions
    X6DeviceMap(IStreamDeviceEvents * owner);
    virtual ~X6DeviceMap();
    //
    //  Make this property public
    using PacketDeviceImpl::PciLoopbackMode;

    // Methods
    // Version
    std::string         Name() const;
    std::string         Revision() const;

    //  Hardware Device Access
    X6AFE_Clock::X6ClockIo &  Clock()   {  return AFEClk_SDev.Clock();  }
    IFpaLoader &              VsProm()  {  return Fpa_SDev.VsProm();  }
    AdvancedDioPortIntf &     Dio()     {  return Dio_SDev;  }
    const IX6LogicVersionInfo &     Info() const   {  return Info_SDev;  }
    IUsesX6Alerts &           Alerts()  {  return Alerts_SDev;  }
    IX6ThermalDevice &        Thermal() {  return Thermal_SDev;  }
    IX6Aurora &               Aurora(unsigned int idx);

    IVeloPacketizer &         Velo()    {  return Velo_SDev;  }
    IVeloPacketizer &         VeloIn()    {  return Velo();  }
    IVitaPacketizer &         VitaIn()   {  return VitaIn_SDev;  }
    IVitaTimestamp &          VitaIn_TS() {  return VitaIn_SDev;  }

    IVeloDepacketizer &       VeloOut()   {  return VeloOut_SDev;  }
    IVitaDepacketizer &       VitaOut()   {  return VitaOut_SDev;  }

    IIdRom &                  IdRom()  {  return IdRom_Intf;   }

    IX6SubDevice &            AuroraPll()  {  return SioPll_SDev;   }

public: // pseudo-protected

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    //
    //  Busmaster Intf functions
    virtual bool    AlertTest(unsigned int pid);
    virtual bool    AlertCheck(const Buffer & buffer);
    //void            DumpIsrStatus(std::vector<int> & vec);
    typedef std::vector<int>  IntVec;
    void            DumpIsrStatus(IntVec & vec);

    Trace                  Debug;

    virtual void ChipErase()
    {
        FlashRom_SDev.ChipErase();
    }

protected:
    X6DeviceDriver *       DDeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Loader;
    WishboneBusSpace       WB_Thermal;
    WishboneBusSpace       WB_Packetizer;
    WishboneBusSpace       WB_Alerts;
    WishboneBusSpace       WB_DigitalIo;
    WishboneBusSpace       WB_FlashRom;
    WishboneBusSpace       WB_AFE;
    WishboneBusSpace       WB_Aurora0;
    WishboneBusSpace       WB_Aurora1;
    //
    //  Subdevices
    X6Info                         Info_SDev;
    X6Peripherals                  Periph_SDev;
    X6SioPll                       SioPll_SDev;
    X6FpaLoader                    Fpa_SDev;
    X6Thermal                      Thermal_SDev;
    X6PacketizerCfg                Velo_SDev;
    X6AlertSystem                  Alerts_SDev;
    X6DigitalIo                    Dio_SDev;
    X6FlashRom                     FlashRom_SDev;
    X6AFE_Clock                    AFEClk_SDev;
    X6AFE_Common                   AFECommon_SDev;
    X6AFE_AdcCommon                AFEAdcC_SDev;
    X6AFE_PulseRepetitionInterval  AFEAdcPri_SDev;
    X6AFE_AdcSpi                   AFEAdcSpi_SDev;
    X6VitaPacketizerCfg            VitaIn_SDev;
    X6AFE_AdcCalibration           AFEAdcCal_SDev;
    X6Aurora                       Aurora0_SDev;
    X6Aurora                       Aurora1_SDev;
    X6Aurora                       Aurora2_SDev;
    X6Aurora                       Aurora3_SDev;
    X6DepacketizerCfg              VeloOut_SDev;
    X6AFE_DacCommon                AFEDacC_SDev;
    X6AFE_PulseRepetitionInterval  AFEDacPri_SDev;
    X6AFE_DacSpi                   AFEDacSpi_SDev;
    X6AFE_DacLatencyCalibration    AFEDacLatencyCal_SDev;
    X6AFE_DacCalibration           AFEDacCal_SDev;
    X6_DacPatternModeSystem        DacPatternMode_SDev;
    X6VitaDepacketizerCfg          VitaOut_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    X6IdromPage                  AdcCal_PG;
    X6IdromPage                  DacCal_PG;
    X6IdromPage                  UserRom_PG;

    //=======================================================================
    //  CLASS X6DeviceMap::X6IdromIo  -- Access to IDROM Objects
    //=======================================================================
    class X6IdromIo : public IIdRom
    {
    public:

    virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
    //  Page access
    virtual X6SystemIdromPage &  System() { return Owner->System_PG;  }
    virtual X6IdromPage &        AdcCal() { return Owner->AdcCal_PG;  }
    virtual X6IdromPage &        DacCal() { return Owner->DacCal_PG;  }
    virtual X6IdromPage &        User()   { return Owner->UserRom_PG;  }

    X6IdromIo(X6DeviceMap * owner) :  Owner(owner)
        {}
    virtual ~X6IdromIo()
        {}

    private:
        X6DeviceMap *  Owner;
    };

    X6IdromIo                    IdRom_Intf;
    friend class X6IdromIo;
    // Data
    // ???  May not be implemented
    void  LocalLoopbackMode(bool state);

    void  DisableDacDevices();
    void  EnableDacDevices();

public:
    void  DisablePriDevices();

};

//===========================================================================
//  CLASS X6InputDevice  -- Partially Implemented X6 Input Device
//===========================================================================

class X6InputDevice : public IX6IoDevice
{
public:

    X6InputDevice( X6DeviceMap * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~X6InputDevice()  {  }

    virtual X6CalibrationSection & Cal() = 0;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6InputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IX6IoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(X6InputDevice * in_dev)
            :  InD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        X6InputDevice * InD;
    };
    
    friend class X6InputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual unsigned int ActiveDevices()
    {
        return ActiveChannels();
    }

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual ITriggerCfg &  Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return AdcPri();  }


    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double /*frequency_hz*/ ) {}
    virtual double TestFrequency() {  return 0.0;  }

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address) const;

protected:
    X6DeviceMap *   Owner;
    TriggerCfgImpl  TriggerCfg;

    //  shortcuts to subdevices
    X6Info &                FpgaInfo() {  return Owner->Info_SDev;  }
    const X6Info &          FpgaInfo() const {  return Owner->Info_SDev;  }
    X6Peripherals &         Periph() {  return Owner->Periph_SDev;  }
    const X6Peripherals &   Periph() const {  return Owner->Periph_SDev;  }
    X6AFE_AdcCommon &       AdcCommon() {  return Owner->AFEAdcC_SDev;  }
    const X6AFE_AdcCommon & AdcCommon() const {  return Owner->AFEAdcC_SDev;  }
    X6AFE_Common &          Common() {  return Owner->AFECommon_SDev;  }
    const X6AFE_Common &    Common() const {  return Owner->AFECommon_SDev;  }
    X6AFE_AdcSpi &          AdcSpi() {  return Owner->AFEAdcSpi_SDev;  }
    const X6AFE_AdcSpi &    AdcSpi() const {  return Owner->AFEAdcSpi_SDev;  }
    X6AFE_AdcCalibration &          AdcCal() {  return Owner->AFEAdcCal_SDev;  }
    const X6AFE_AdcCalibration &    AdcCal() const {  return Owner->AFEAdcCal_SDev;  }
    X6AFE_PulseRepetitionInterval &        AdcPri() {  return Owner->AFEAdcPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  AdcPri() const {  return Owner->AFEAdcPri_SDev;  }

    //  shortcuts to other things
    X6IdromPage &     AdcCalIdrom()  {  return Owner->AdcCal_PG;  }

};

//===========================================================================
//  CLASS X6OutputDevice  -- Partially Implemented X6 Output Device
//===========================================================================

class X6OutputDevice : public IX6IoDevice
{
public:

    X6OutputDevice( X6DeviceMap * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~X6OutputDevice()  {  }

    virtual X6CalibrationSection &  Cal() = 0;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6OutputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IX6IoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(X6OutputDevice * in_dev)
            :  OutD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        X6OutputDevice * OutD;
    };
    
    friend class X6OutputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual unsigned int  ActiveChannels();
    virtual unsigned int  ActiveDevices() = 0;

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;
	void          DramPowerDown(bool state) {  DacPattern().DramPowerDown(state);  }

    virtual ITriggerCfg &  Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return DacPri();  }
    virtual IPatternModeSystem & Pattern() {  return DacPattern();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double frequency_hz);
    virtual double TestFrequency();

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address) const;

    virtual bool  DacInternalCalibrationOk()
                    {  return DacLatencyCal().CalibrationOk();  }

protected:
    X6DeviceMap *   Owner;
    TriggerCfgImpl  TriggerCfg;

    //  shortcuts to subdevices
    X6Peripherals &         Periph() {  return Owner->Periph_SDev;  }
    const X6Peripherals &   Periph() const {  return Owner->Periph_SDev;  }
    X6AFE_Clock &           Clock() {  return Owner->AFEClk_SDev;  }
    const X6AFE_Clock &     Clock() const {  return Owner->AFEClk_SDev;  }
    X6AFE_DacCommon &       DacCommon() {  return Owner->AFEDacC_SDev;  }
    const X6AFE_DacCommon & DacCommon() const {  return Owner->AFEDacC_SDev;  }
    X6AFE_Common &          Common() {  return Owner->AFECommon_SDev;  }
    const X6AFE_Common &    Common() const {  return Owner->AFECommon_SDev;  }
    X6AFE_DacSpi &          DacSpi() {  return Owner->AFEDacSpi_SDev;  }
    const X6AFE_DacSpi &    DacSpi() const {  return Owner->AFEDacSpi_SDev;  }
    X6AFE_DacCalibration &          DacCal() {  return Owner->AFEDacCal_SDev;  }
    const X6AFE_DacCalibration &    DacCal() const {  return Owner->AFEDacCal_SDev;  }
    X6AFE_PulseRepetitionInterval &        DacPri() {  return Owner->AFEDacPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  DacPri() const {  return Owner->AFEDacPri_SDev;  }
    X6_DacPatternModeSystem &        DacPattern() {  return Owner->DacPatternMode_SDev;  }
    const X6_DacPatternModeSystem &  DacPattern() const {  return Owner->DacPatternMode_SDev;  }
    X6AFE_DacLatencyCalibration &  DacLatencyCal() {  return Owner->AFEDacLatencyCal_SDev;  }
    //  shortcuts to other things
    X6IdromPage &     DacCalIdrom()  {  return Owner->DacCal_PG;  }
};

//=============================================================================
//  CLASS X6StandardPriClockDevice  --  Normal impl to access clock for PRI
//=============================================================================

class X6StandardPriClockDevice : public IPriClockDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->AFEClk_SDev.Clock().Frequency();  }
    virtual int   CalculatePriDivisor()
        {  return Divisor;  }

    X6StandardPriClockDevice(X6DeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~X6StandardPriClockDevice()
        {}

private:
    X6DeviceMap * Owner;
    int Divisor;
};

//=============================================================================
//  CLASS X6StandardFrameCalcDevice  --  Supply Divider on Frame ticks
//=============================================================================

class X6StandardFrameCalcDevice : public IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor()
        {  return Divisor;  }

    X6StandardFrameCalcDevice(X6DeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~X6StandardFrameCalcDevice()
        {}

private:
    X6DeviceMap * Owner;
    int Divisor;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
