//---------------------------------------------------------------------------
// X6SubDevice_AnalogIn_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "AnalogIn" Subdevices
//---------------------------------------------------------------------------

#include <X6SubDevice_AnalogIn_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <StringSupport_Mb.h>

//---------------------------------------------------------------------------

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6AFE_AdcCommon -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_AdcCommon
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_AdcCommon
//---------------------------------------------------------------------------

X6AFE_AdcCommon::X6AFE_AdcCommon(IRequires_FrameCalcDevice * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       AdcEnableReg(space, regs.AfeAdcEnable_Addr),
       AdcTriggerCfgReg(space, regs.AfeAdcTriggerCfg_Addr),
       AdcDecimationReg(space, regs.AfeAdcDecimation_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_AdcCommon
//---------------------------------------------------------------------------

X6AFE_AdcCommon::~X6AFE_AdcCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_AdcCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_FrameCalcDevice::devAdc);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_AdcCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_AdcCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::ChannelDisableAll()
{
    AdcEnableReg.AdcChannelEnables.Value(0);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  X6AFE_AdcCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<AdcEnableReg.MaxAdcChannels(); i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::ChannelEnabled(int ch, bool state)
{
    AdcEnableReg.AdcChannelEnables.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool X6AFE_AdcCommon::ChannelEnabled(int ch) const
{
    return AdcEnableReg.AdcChannelEnables.Bit(ch);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::ChannelPowerDownAll()
{
    AdcEnableReg.AdcChannelPower.Value(0);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::ChannelPowered(int ch, bool state)
{
    AdcEnableReg.AdcChannelPower.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool X6AFE_AdcCommon::ChannelPowered(int ch) const
{
    return AdcEnableReg.AdcChannelPower.Bit(ch);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    AdcTriggerCfgReg.AdcFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int X6AFE_AdcCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = AdcTriggerCfgReg.AdcFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::EdgeTrigger(bool state)
{
    AdcTriggerCfgReg.AdcRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool X6AFE_AdcCommon::EdgeTrigger() const
{
    return AdcTriggerCfgReg.AdcRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::ExternalTrigger(bool state)
{
    AdcTriggerCfgReg.AdcExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool X6AFE_AdcCommon::ExternalTrigger() const
{
    return AdcTriggerCfgReg.AdcExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::FramedTriggerEnabled(bool state)
{
    AdcTriggerCfgReg.AdcFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool X6AFE_AdcCommon::FramedTriggerEnabled() const
{
    return AdcTriggerCfgReg.AdcFramedMode.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::Decimation() -- Set Decimation Count  
//---------------------------------------------------------------------------

void X6AFE_AdcCommon::Decimation(unsigned int samples)
{
    AdcDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCommon::Decimation() -- Read Decimation Count  
//---------------------------------------------------------------------------

unsigned int X6AFE_AdcCommon::Decimation() const
{
    return AdcDecimationReg.Decimation.Value();
}

//===========================================================================
//  CLASS X6AFE_AdcSpi -- X6 AFE ADC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_AdcSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_AdcSpi
//---------------------------------------------------------------------------

X6AFE_AdcSpi::X6AFE_AdcSpi(IRequires_AdcSpi * mdc, IAddressingSpace & space)
    :  MDC(mdc), AFE_Space(space)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_AdcSpi
//---------------------------------------------------------------------------

X6AFE_AdcSpi::~X6AFE_AdcSpi()
{
    unsigned int count = static_cast<unsigned int>(SpiDev.size());
    for (unsigned int i=0; i<count; i++)
        {
        X6SpiDevice * dev = SpiDev.back();
        delete dev;
        SpiDev.pop_back();
        }
}

//---------------------------------------------------------------------------
//  X6AFE_AdcSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_AdcSpi::ConstructCustomParts()
{
    MDC->MakeAfeAdcSpiDevices();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  X6AFE_AdcSpi::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_AdcSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_AdcSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_AdcSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_AdcSpi::MakeSpiDevice() --  Make SPI device
//---------------------------------------------------------------------------

void  X6AFE_AdcSpi::MakeSpiDevice(const std::string & Tag, unsigned int Wr_Addr,
                                                           unsigned int Rd_Addr)
{
    // Set up device unique tag string
    unsigned int nextDeviceID = static_cast<unsigned int>(SpiDev.size());
    std::string DevTag = Tag + IntToString(nextDeviceID);
    // Set up info structure
    X6SpiDevice::Info info;
    info.WriteOffset = Wr_Addr;
    info.ReadOffset = Rd_Addr;
    // Make Spi Device
    X6SpiDevice * dev = new X6SpiDevice(AFE_Space, info, DevTag);
    // Save in Device Array
    SpiDev.push_back(dev);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  X6AFE_AdcSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiDev.size())
        SpiDev[device]->Spi(address, data);
}

//---------------------------------------------------------------------------
//  X6AFE_AdcSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  X6AFE_AdcSpi::Spi(unsigned int device, unsigned int address) const
{
    if (device < SpiDev.size())
        return SpiDev[device]->Spi(address);
    else
        return 0;
}


//===========================================================================
//  CLASS X6AFE_AdcCalibration -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AFE_AdcCalibration
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_AdcCalibration
//---------------------------------------------------------------------------

X6AFE_AdcCalibration::X6AFE_AdcCalibration(IRequires_AdcCalParts * mdc,
                IAddressingSpace & space, MapRegisters & regs,
                unsigned int MaxAdcs )
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, MaxAdcs),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, MaxAdcs),
       FVitaInAdcs(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_AdcCalibration
//---------------------------------------------------------------------------

X6AFE_AdcCalibration::~X6AFE_AdcCalibration()
{
    if (CalOverride)
        delete CalOverride;
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCalibration::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_AdcCalibration::ConstructCustomParts()
{
    FVitaInAdcs = MDC->GetCalibrationAdcCount();
    CalOverride = MDC->GetAdcCalibrationOverride();
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCalibration::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_AdcCalibration::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    for (unsigned int i=0; i<FVitaInAdcs; i++)
        {
        SetCalibrationGain(i, 0x10000);
        SetCalibrationOffset(i, 0x000);
        }
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCalibration::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_AdcCalibration::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCalibration::SetCalibrationOffset()
//---------------------------------------------------------------------------

void  X6AFE_AdcCalibration::SetCalibrationOffset(unsigned int ch, unsigned int value)
{
    if (ch<FVitaInAdcs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideOffset(value);
            if (CalOverride->OverrideOffsetUpdate())
                {
                CalOverride->WriteOffset(ch, value);
                return;
                }
            }
        OffsetCoeffArray[ch].Value(value);
        }
}

//---------------------------------------------------------------------------
//  X6AFE_AdcCalibration::SetCalibrationGain()
//---------------------------------------------------------------------------

void  X6AFE_AdcCalibration::SetCalibrationGain(unsigned int ch, unsigned int value)
{
    if (ch<FVitaInAdcs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideGain(value);
            if (CalOverride->OverrideGainUpdate())
                {
                CalOverride->WriteGain(ch, value);
                return;
                }
            }
        GainCoeffArray[ch].Value(value);
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}   // end namespace Innovative
