//---------------------------------------------------------------------------
// X6SubDevice_AnalogIn_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "AnalogIn" Subdevices
//---------------------------------------------------------------------------

#ifndef X6SubDevice_AnalogIn_MbH
#define X6SubDevice_AnalogIn_MbH

#include <X6Support_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS X6AFE_AdcCommon -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================

class X6AFE_AdcCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_AdcCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcEnable_Addr;
        unsigned int   AfeAdcTriggerCfg_Addr;
        unsigned int   AfeAdcDecimation_Addr;
    };

    X6AFE_AdcCommon(IRequires_FrameCalcDevice * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~X6AFE_AdcCommon();

    //
    //  Methods
    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_AdcCommon::AdcEnableRegister  -- Enable Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  AdcChannelEnables;
        RegisterBitGroup  AdcChannelPower;

        unsigned int MaxAdcChannels() {  return 16;  }

    public:
        AdcEnableRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcChannelEnables(*this, 0, 16),   AdcChannelPower(*this, 16, 16)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_AdcCommon::AdcTriggerCfgRegister -- Configure ADC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  AdcFrameSize;
        RegisterBit       AdcRisingEdge;
        RegisterBit       AdcFramedMode;
        RegisterBit       AdcExternalTrigger;

    public:
        AdcTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcFrameSize(*this, 0, 24),   AdcRisingEdge(*this, 29),
               AdcFramedMode(*this, 30),     AdcExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_AdcCommon::AdcDecimationRegister  -- Adc Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        AdcDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_FrameCalcDevice *   MDC;
    IFrameCalcDevice *            Dev;
    AdcEnableRegister             AdcEnableReg;
    AdcTriggerCfgRegister         AdcTriggerCfgReg;
    AdcDecimationRegister         AdcDecimationReg;

};

//===========================================================================
//  CLASS X6AFE_AdcSpi -- X6 AFE ADC Spi Device Manager "SubDevice"
//===========================================================================

class X6AFE_AdcSpi : public IX6SubDevice
{
public:

    X6AFE_AdcSpi(IRequires_AdcSpi * owner, IAddressingSpace & space);
    virtual ~X6AFE_AdcSpi();

    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address) const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    void  MakeSpiDevice(const std::string & Tag, unsigned int Wr_Addr,
                                                 unsigned int Rd_Addr);

    X6SpiDevice *  SpiDevice(unsigned int idx)
         {
         if (idx<SpiDev.size())
            return SpiDev[idx];
         else
            return 0;
         }

private:
    IRequires_AdcSpi *  MDC;
    IAddressingSpace &          AFE_Space;
    std::vector<X6SpiDevice *>  SpiDev;
};


//===========================================================================
//  CLASS X6AFE_AdcCalibration -- X6 ADC Calibration "SubDevice"
//===========================================================================

class X6AFE_AdcCalibration : public IX6SubDevice, public IGainCalibrationDevice
{
public:
    //=======================================================================
    //  CLASS X6AFE_AdcCalibration::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   GainCoeff_Addr;
        unsigned int   OffsetCoeff_Addr;
    };

    X6AFE_AdcCalibration(IRequires_AdcCalParts * mdc, IAddressingSpace & space, MapRegisters & regs, unsigned int MaxAdcs );
    virtual ~X6AFE_AdcCalibration();

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value);
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    IRequires_AdcCalParts *  MDC;
    
    RegisterArray<Register>    GainCoeffArray;
    RegisterArray<Register>    OffsetCoeffArray;

    unsigned int                FVitaInAdcs;
    ICalibrationValueOverride * CalOverride;

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}   // end namespace Innovative
//---------------------------------------------------------------------------
#endif
