//---------------------------------------------------------------------------
// X6SubDevice_AnalogOut_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "AnalogOut" Subdevices
//---------------------------------------------------------------------------

#include <X6SubDevice_AnalogOut_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <StringSupport_Mb.h>
#include <BufferHeader_Mb.h>
#include <BufferTrailer_Mb.h>
#include <BufferDatagrams_Mb.h>
#include <X6Board_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

namespace {

double ii_power_of_two(int exponent)
{
    const int max_exponent = 30;
    double result = 1.0;
    while (exponent > 0)
        {
        int shift;
        //  we can't shift by more than 31 bits accurately, so break up
        //    into groups of max_exponent
        if (exponent > max_exponent)
            shift = max_exponent;
        else
            shift = exponent;
        unsigned int partial = 1<<shift;
        result *= static_cast<double>(partial);

        exponent -= shift;
        }
    return result;
}

}  // end nameless namespace

//===========================================================================
//  CLASS X6AFE_DacCommon -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_DacCommon
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_DacCommon
//---------------------------------------------------------------------------

X6AFE_DacCommon::X6AFE_DacCommon(IRequires_FrameCalcDevice * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       DacEnableReg(space, regs.AfeDacEnable_Addr),
       DacTriggerCfgReg(space, regs.AfeDacTriggerCfg_Addr),
       DacDecimationReg(space, regs.AfeDacDecimation_Addr),
       DacResetReg(space, regs.AfeDacReset_Addr ),
       DacPhaseIncrReg(space, regs.AfeDacPhaseIncrement_Addr )
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_DacCommon
//---------------------------------------------------------------------------

X6AFE_DacCommon::~X6AFE_DacCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_DacCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_FrameCalcDevice::devDac);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ChannelDisableAll()
{
    DacEnableReg.DacChannelEnables.Value(0);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  X6AFE_DacCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<DacEnableReg.MaxDacChannels(); i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ChannelEnabled(int ch, bool state)
{
    DacEnableReg.DacChannelEnables.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool X6AFE_DacCommon::ChannelEnabled(int ch) const
{
    return DacEnableReg.DacChannelEnables.Bit(ch);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ChannelPowerDownAll()
{
    DacEnableReg.DacChannelPower.Value(0);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ChannelPowered(int ch, bool state)
{
    DacEnableReg.DacChannelPower.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool X6AFE_DacCommon::ChannelPowered(int ch) const
{
    return DacEnableReg.DacChannelPower.Bit(ch);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void X6AFE_DacCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    DacTriggerCfgReg.DacFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int X6AFE_DacCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = DacTriggerCfgReg.DacFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void X6AFE_DacCommon::EdgeTrigger(bool state)
{
    DacTriggerCfgReg.DacRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool X6AFE_DacCommon::EdgeTrigger() const
{
    return DacTriggerCfgReg.DacRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ExternalTrigger(bool state)
{
    DacTriggerCfgReg.DacExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool X6AFE_DacCommon::ExternalTrigger() const
{
    return DacTriggerCfgReg.DacExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void X6AFE_DacCommon::FramedTriggerEnabled(bool state)
{
    DacTriggerCfgReg.DacFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool X6AFE_DacCommon::FramedTriggerEnabled() const
{
    return DacTriggerCfgReg.DacFramedMode.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::Decimation() -- Set Decimation Count  
//---------------------------------------------------------------------------

void X6AFE_DacCommon::Decimation(unsigned int samples)
{
    DacDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::Decimation() -- Read Decimation Count  
//---------------------------------------------------------------------------

unsigned int X6AFE_DacCommon::Decimation() const
{
    return DacDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelResetAll() -- Reset all channels
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ChannelResetAll()
{
    DacResetReg.DacChannelResets.Value(0xFFFF);
    DacResetReg.DacChannelResets.Value(0x0);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelReset() -- Set Reset state specified I/O channel
//---------------------------------------------------------------------------

void X6AFE_DacCommon::ChannelReset(int ch, bool state)
{
    DacResetReg.DacChannelResets.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::ChannelReset() -- Query Reset state of an I/O channel
//---------------------------------------------------------------------------

bool X6AFE_DacCommon::ChannelReset(int ch) const
{
    return DacResetReg.DacChannelResets.Bit(ch);
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::DacTestToneFrequency() -- Set Reset tone_freq specified I/O channel
//---------------------------------------------------------------------------

void X6AFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, double tone_freq, int channels)
{
    double Two32 = ii_power_of_two(32);

    double phase_incr = tone_freq * (Two32) / dac_clock_freq / channels;

    DacPhaseIncrReg.Value(static_cast<unsigned int>(phase_incr));
}

//---------------------------------------------------------------------------
//  X6AFE_DacCommon::DacTestToneFrequency() -- Query Reset tone_freq of an I/O channel
//---------------------------------------------------------------------------

double X6AFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, int channels) const
{
    double Two32 = ii_power_of_two(32);
    unsigned int raw_phase_incr = DacPhaseIncrReg.Value();

    double tone_freq = static_cast<double>(raw_phase_incr) / (Two32) * dac_clock_freq * channels;
    return tone_freq;
}

//===========================================================================
//  CLASS X6AFE_DacSpi -- X6 AFE DAC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_DacSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_DacSpi
//---------------------------------------------------------------------------

X6AFE_DacSpi::X6AFE_DacSpi(IRequires_DacSpi * mdc, IAddressingSpace & space)
    :  MDC(mdc), AFE_Space(space)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_DacSpi
//---------------------------------------------------------------------------

X6AFE_DacSpi::~X6AFE_DacSpi()
{
    unsigned int count = static_cast<unsigned int>(SpiDev.size());
    for (unsigned int i=0; i<count; i++)
        {
        X6SpiDevice * dev = SpiDev.back();
        delete dev;
        SpiDev.pop_back();
        }
}

//---------------------------------------------------------------------------
//  X6AFE_DacSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_DacSpi::ConstructCustomParts()
{
    MDC->MakeAfeDacSpiDevices();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  X6AFE_DacSpi::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_DacSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_DacSpi::MakeSpiDevice() --  Make SPI device
//---------------------------------------------------------------------------

void  X6AFE_DacSpi::MakeSpiDevice(const std::string & Tag, unsigned int Wr_Addr,
                                                           unsigned int Rd_Addr)
{
    // Set up device unique tag string
    unsigned int nextDeviceID = static_cast<unsigned int>(SpiDev.size());
    std::string DevTag = Tag + IntToString(nextDeviceID);
    // Set up info structure
    X6SpiDevice::Info info;
    info.WriteOffset = Wr_Addr;
    info.ReadOffset = Rd_Addr;
    // Make Spi Device
    X6SpiDevice * dev = new X6SpiDevice(AFE_Space, info, DevTag);
    // Save in Device Array
    SpiDev.push_back(dev);
}

//---------------------------------------------------------------------------
//  X6AFE_DacSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  X6AFE_DacSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiDev.size())
        SpiDev[device]->Spi(address, data);
}

//---------------------------------------------------------------------------
//  X6AFE_DacSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  X6AFE_DacSpi::Spi(unsigned int device, unsigned int address) const
{
    if (device < SpiDev.size())
        return SpiDev[device]->Spi(address);
    else
        return 0;
}

//===========================================================================
//  CLASS X6AFE_DacLatencyCalibration -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AFE_DacLatencyCalibration
//---------------------------------------------------------------------------

const int X6_MaxCalDevices = X6_MemoryMap::mmAFE_TotalDacs/2;  // Max Devices allowed

//---------------------------------------------------------------------------
//  constructor for class X6AFE_DacLatencyCalibration
//---------------------------------------------------------------------------

X6AFE_DacLatencyCalibration::X6AFE_DacLatencyCalibration(IRequires_DacLatencyCal * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       LatencyCalArray(space, regs.LatencyCal_Addr, X6_MaxCalDevices),
       Cal(0), FDacDevices(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_DacLatencyCalibration
//---------------------------------------------------------------------------

X6AFE_DacLatencyCalibration::~X6AFE_DacLatencyCalibration()
{
    delete Cal;  Cal = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_DacLatencyCalibration::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_DacLatencyCalibration::ConstructCustomParts()
{
    FDacDevices = MDC->GetCalibrationDacCount();
    if (FDacDevices == 0)
        DeviceEnabled(false);  // no need for this if no DACs

    Cal = MDC->GetDacLatencyCalibrator();
}

//---------------------------------------------------------------------------
//  X6AFE_DacLatencyCalibration::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacLatencyCalibration::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6AFE_DacLatencyCalibration::PreconfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacLatencyCalibration::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    CalibrateDacLatency();

}

//---------------------------------------------------------------------------
//  X6AFE_DacLatencyCalibration::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacLatencyCalibration::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6AFE_DacLatencyCalibration::CalibrateDacLatency()
//---------------------------------------------------------------------------

void  X6AFE_DacLatencyCalibration::CalibrateDacLatency()
{
    // check if any devices enabled (?) return if not

    //  Clock must be running at this time...
    Cal->Calibrate();

}
//---------------------------------------------------------------------------
//  X6AFE_DacLatencyCalibration::GetLatencyRegisterIntf()
//---------------------------------------------------------------------------

X6AFE_DacLatencyCalibration::ILatencyRegIntf *
        X6AFE_DacLatencyCalibration::GetLatencyRegisterIntf()
{
    return new ILatencyRegIntf(this);
}

//===========================================================================
//  CLASS X6AFE_DacCalibration -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AFE_DacCalibration
//---------------------------------------------------------------------------

const int X6_MaxOutputChannels = X6_MemoryMap::mmAFE_TotalDacs;  // Max DACs allowed

//---------------------------------------------------------------------------
//  constructor for class X6AFE_DacCalibration
//---------------------------------------------------------------------------

X6AFE_DacCalibration::X6AFE_DacCalibration(IRequires_DacCalParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, X6_MaxOutputChannels),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, X6_MaxOutputChannels),
       FVitaOutDacs(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  constructor for class X6AFE_DacCalibration
//---------------------------------------------------------------------------

X6AFE_DacCalibration::X6AFE_DacCalibration(IRequires_DacCalParts* mdc, IAddressingSpace& space, X6AFE_DacCalibration::MapRegisters& regs,
                                           unsigned int MaxDacCount)
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, MaxDacCount),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, MaxDacCount),
       FVitaOutDacs(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_DacCalibration
//---------------------------------------------------------------------------

X6AFE_DacCalibration::~X6AFE_DacCalibration()
{
    if (CalOverride)
        delete CalOverride;
}

//---------------------------------------------------------------------------
//  X6AFE_DacCalibration::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_DacCalibration::ConstructCustomParts()
{
    FVitaOutDacs = MDC->GetCalibrationDacCount();
    CalOverride = MDC->GetDacCalibrationOverride();
}

//---------------------------------------------------------------------------
//  X6AFE_DacCalibration::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacCalibration::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    for (unsigned int i=0; i<FVitaOutDacs; i++)
        {
        SetCalibrationGain(i, 0x10000);
        SetCalibrationOffset(i, 0x000);
        }
}

//---------------------------------------------------------------------------
//  X6AFE_DacCalibration::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_DacCalibration::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6AFE_DacCalibration::SetCalibrationOffset()
//---------------------------------------------------------------------------

void  X6AFE_DacCalibration::SetCalibrationOffset(unsigned int ch, unsigned int value)
{
    if (ch<FVitaOutDacs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideOffset(value);
            if (CalOverride->OverrideOffsetUpdate())
                {
                CalOverride->WriteOffset(ch, value);
                return;
                }
            }
        OffsetCoeffArray[ch].Value(value);
        }
}

//---------------------------------------------------------------------------
//  X6AFE_DacCalibration::SetCalibrationGain()
//---------------------------------------------------------------------------

void  X6AFE_DacCalibration::SetCalibrationGain(unsigned int ch, unsigned int value)
{
    if (ch<FVitaOutDacs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideGain(value);
            if (CalOverride->OverrideGainUpdate())
                {
                CalOverride->WriteGain(ch, value);
                return;
                }
            }
        GainCoeffArray[ch].Value(value);
        }
}

//===========================================================================
//  CLASS X6_DacPatternModeSystem -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6_DacPatternModeSystem
//---------------------------------------------------------------------------

const int DefaultPatternModeOnValue =  0x2;
const int DefaultPatternModeOffValue = 0x0;

//---------------------------------------------------------------------------
//  constructor for class X6_DacPatternModeSystem
//---------------------------------------------------------------------------

X6_DacPatternModeSystem::X6_DacPatternModeSystem(IRequires_PatternMode * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       ModeEnableReg(space, regs.PatternModeEnable_Addr),
       FModeEnabled(false),
       FModeOnValue(DefaultPatternModeOnValue), FModeOffValue(DefaultPatternModeOffValue),
       PatternModeAlignment(1), PatternModeScaleFactor(1)

{
}

//---------------------------------------------------------------------------
//  destructor for class X6_DacPatternModeSystem
//---------------------------------------------------------------------------

X6_DacPatternModeSystem::~X6_DacPatternModeSystem()
{
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6_DacPatternModeSystem::ConstructCustomParts()
{
    Board = MDC->GetBaseboardPtr();
    PatternModeAlignment = MDC->GetPatternModeAlignment();
    PatternModeScaleFactor = MDC->GetPatternModeScaleFactor();
    SingleMemoryFlag = MDC->GetPatternModeSingleMemoryFlag();
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::OpenHardware()
//---------------------------------------------------------------------------

void  X6_DacPatternModeSystem::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6_DacPatternModeSystem::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::PatternModeEnable() - enable pattern mode
//---------------------------------------------------------------------------

void  X6_DacPatternModeSystem::PatternModeEnable(bool state)
{
    int output_val = (state) ? FModeOnValue : FModeOffValue;
    ModeEnableReg.PlaybackEnable.Value(output_val);
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::PatternModeEnable() - return pattern mode state
//---------------------------------------------------------------------------

bool  X6_DacPatternModeSystem::PatternModeEnable()
{
    if (ModeEnableReg.PlaybackEnable.Value() == (unsigned int)FModeOnValue)
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::CalcVitaPatternSize() -
//---------------------------------------------------------------------------
//  Calculates total pattern data size for a raw data pattern by including
//     the Vita Header/Trailer words

unsigned int  X6_DacPatternModeSystem::CalcVitaPatternSize(unsigned int RawDataSizeWords,
                                                           unsigned int VitaDataSize,
                                                           unsigned int sid_count)
{
    VitaBufferManager  VBM;

    //unsigned int VitaDataSize = VitaPacketSize - VBM.FullSizeDelta();
    //
    //  The data will be evenly divided across the SIDs...so divide it first
    unsigned int PerDeviceRawDataSizeWords = RawDataSizeWords / sid_count;
    //
    //  Find number of packets per SID in the dataset
    unsigned int VitaPkts = PerDeviceRawDataSizeWords / VitaDataSize;
    if (PerDeviceRawDataSizeWords%VitaDataSize != 0)
        VitaPkts++;         // if we have a fractional packet, add one

    //  One packet for each SID
    unsigned int TotalVPackets = VitaPkts*sid_count;
    //  Total size is orignal size, plus "Delta" times packet count
    return RawDataSizeWords + TotalVPackets*VBM.FullSizeDelta();

}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::SendPatternInfo()
//---------------------------------------------------------------------------
const unsigned int TagFieldStartBit     =  0;
const unsigned int TagFieldSize         =  8;

const unsigned int WriteCommandBit      =  16;
const unsigned int ReadCommandBit       =  17;
const unsigned int RepeatPacketBit      =  18;

void  X6_DacPatternModeSystem::SendPatternInfo(unsigned int data_pid,
                      char tag_value, unsigned int start_addr,
                      unsigned int buffer_size_ints, unsigned int rep_count,
                      PatternCommands cmd, PatternRepeatType repeat)
{
    VeloBuffer Packet(4);
    VeloHeaderDatagram Hdr(Packet);
    IntegerDG  PacketDG(Packet);

    Hdr.PeripheralId(0x80 | data_pid);
    Hdr.PacketSize(Packet.FullSizeInInts());
    Hdr[1] = 0;

    // Align the start address
    start_addr = (start_addr/PatternModeAlignment)*PatternModeAlignment;

    PacketDG[0] = buffer_size_ints/PatternModeScaleFactor;
    PacketDG[1] = start_addr/PatternModeScaleFactor;
    PacketDG[2] = 0;
    PacketDG[3] = rep_count;

    bool write_bit_value;
    bool read_bit_value;
    switch (cmd)
        {
        case piPlay:
        default:
            write_bit_value = true;   // data follows, and play it
            read_bit_value = true;
            break;
        case piLoad:
            write_bit_value = true;
            read_bit_value = false;   // data follows, don't play it
            break;
        case piReplay:
            write_bit_value = false;
            read_bit_value = true;   // no data follows, play loaded pattern
            break;
        case piHalt:
            write_bit_value = false;
            read_bit_value = false;   // no data follows, halt playing
            break;
        }

    bool repeat_bit_value;
    switch (repeat)     // set repeat bit based off of mode
        {
        default:
        case prPlayAgain:
            repeat_bit_value = true;
            break;
        case prFlatline:
            repeat_bit_value = false;
            break;
        }

    PtrBitManipulator InfoWord( &PacketDG[2] );
    //  Fill Info Word
    InfoWord.Field(TagFieldStartBit, TagFieldSize, tag_value);
    InfoWord.Bit(WriteCommandBit, write_bit_value);
    InfoWord.Bit(ReadCommandBit, read_bit_value);
    InfoWord.Bit(RepeatPacketBit, repeat_bit_value);
    //
    //  Post to stream
    if (Board->StreamManager.Linked())
        Board->StreamManager->Send(Packet);
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::SendPatternInfo()
//---------------------------------------------------------------------------

void  X6_DacPatternModeSystem::SendPatternInfo(unsigned int data_pid,
                                  std::vector<unsigned int> & sids,
                                  std::vector<char> & tags, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int rep_count,
                                  IPatternModeSystem::PatternCommands cmd,  IPatternModeSystem::PatternRepeatType repeat )
{
    VitaBuffer Command(4);

    size_t command_packets;
    unsigned int per_command_size;
    if (SingleMemoryFlag)
        {
        command_packets = 1;
        per_command_size = buffer_size_ints;
        }
    else
        {
        //  size of image in each memory region
        command_packets = sids.size();
        unsigned int per_device_size = static_cast<unsigned int>(buffer_size_ints / sids.size());
        per_command_size = per_device_size;
        }

    //  calculate final buffer data size...
    unsigned int velo_data_size = static_cast<unsigned int>(command_packets * Command.FullSizeInInts());
    VeloBuffer Packet(velo_data_size);
    InitHeader(Packet);
    VeloHeaderDatagram Hdr(Packet);
    Hdr.PeripheralId(data_pid);

    size_t pos = 0;
    for (unsigned int i=0; i<command_packets; i++)
        {
        // Fill the command buffer with data
        FillVitaCommandBuffer(Command, sids[i], tags[i], start_addr, per_command_size,
                          rep_count, cmd, repeat );
        CopyVitaToDataBuffer( Packet, Command, static_cast<unsigned int>(pos) );
        pos += Command.FullSizeInInts();
        }

    //
    //  Post to stream
    if (Board->StreamManager.Linked())
        Board->StreamManager->Send(Packet);
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::FillVitaCommandBuffer()
//---------------------------------------------------------------------------
//const unsigned int TagFieldStartBit     =  0;
//const unsigned int TagFieldSize         =  8;
//
//const unsigned int WriteCommandBit      =  16;
//const unsigned int ReadCommandBit       =  17;
//const unsigned int RepeatPacketBit      =  18;

void  X6_DacPatternModeSystem::FillVitaCommandBuffer(VitaBuffer & Command,
            unsigned int  sid, char tag,
            unsigned int start_addr, unsigned int per_device_size_ints,
            unsigned int rep_count, PatternCommands cmd, PatternRepeatType repeat )
{
    ClearHeader(Command);  ClearTrailer(Command);
    InitHeader(Command);   InitTrailer(Command);
    IntegerDG  CmdDG(Command);

    //  Write first sid
    VitaHeaderDatagram CmdHdr(Command);
    CmdHdr.StreamId(0x8000 | sid);

    start_addr = (start_addr/PatternModeAlignment)*PatternModeAlignment;

    //  Initialize VITA data (for first packet)
    CmdDG[0] = per_device_size_ints/PatternModeScaleFactor;
    CmdDG[1] = start_addr/PatternModeScaleFactor;
    CmdDG[2] = 0;
    CmdDG[3] = rep_count;

    bool write_bit_value;
    bool read_bit_value;
    switch (cmd)
        {
        case piPlay:
        default:
            write_bit_value = true;   // data follows, and play it
            read_bit_value = true;
            break;
        case piLoad:
            write_bit_value = true;
            read_bit_value = false;   // data follows, don't play it
            break;
        case piReplay:
            write_bit_value = false;
            read_bit_value = true;   // no data follows, play loaded pattern
            break;
        case piHalt:
            write_bit_value = false;
            read_bit_value = false;   // no data follows, halt playing
            break;
        }

    bool repeat_bit_value;
    switch (repeat)     // set repeat bit based off of mode
        {
        default:
        case prPlayAgain:
            repeat_bit_value = true;
            break;
        case prFlatline:
            repeat_bit_value = false;
            break;
        }

    PtrBitManipulator InfoWord( &CmdDG[2] );
    //  Fill Info Word
    InfoWord.Field(TagFieldStartBit, TagFieldSize, tag);
    InfoWord.Bit(WriteCommandBit, write_bit_value);
    InfoWord.Bit(ReadCommandBit, read_bit_value);
    InfoWord.Bit(RepeatPacketBit, repeat_bit_value);

}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::CopyVitaToDataBuffer()
//---------------------------------------------------------------------------

void   X6_DacPatternModeSystem::CopyVitaToDataBuffer(VeloBuffer & Packet, VitaBuffer & Command, unsigned int pos )
{
    //
    //  Set up for Copies
    IntegerDG  PacketDG(Packet);
    //
    //  Copy header
    VitaHeaderDatagram CmdHdrDG(Command);
    Debug.Log("Command Vita Header");
    for (unsigned int i=0; i<CmdHdrDG.size(); i++)
        {
        PacketDG[pos+i] = CmdHdrDG[i];
        Debug.Log("...0x", CmdHdrDG[i], 16);
        }
    pos += static_cast<unsigned int>(CmdHdrDG.size());
    //
    //  Copy data
    IntegerDG  CmdDataDG(Command);
    Debug.Log("Command Vita Body");
    for (unsigned int i=0; i<CmdDataDG.size(); i++)
        {
        PacketDG[pos+i] = CmdDataDG[i];
        Debug.Log("...0x", CmdDataDG[i], 16);
        }
    pos += static_cast<unsigned int>(CmdDataDG.size());
    //
    //  Copy trailer
    VitaTrailerDatagram  CmdTrailerDG(Command);
    Debug.Log("Command Vita Trailer");
    for (unsigned int i=0; i<CmdTrailerDG.size(); i++)
        {
        PacketDG[pos+i] = CmdTrailerDG[i];
        Debug.Log("...0x", CmdTrailerDG[i], 16);
        }

}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::DramPowerDown()
//---------------------------------------------------------------------------

void   X6_DacPatternModeSystem::DramPowerDown(bool state)
{
	ModeEnableReg.DramPowerDown.Value((state)?0xF:0x0);
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::MemoryInit()
//---------------------------------------------------------------------------

unsigned int X6_DacPatternModeSystem::MemoryInit()
{
    return ModeEnableReg.DramPhyInitDone.Value();
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::GetMemoryStatus()
//---------------------------------------------------------------------------

unsigned int X6_DacPatternModeSystem::GetMemoryStatus()
{
    return ModeEnableReg.MemoryTestError.Value();
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::MemoryTestEnable()
//---------------------------------------------------------------------------

void X6_DacPatternModeSystem::MemoryTestEnable(unsigned int val)
{
    ModeEnableReg.MemoryTestEnable.Value(val);
}

//---------------------------------------------------------------------------
//  X6_DacPatternModeSystem::DoMemoryTest() -- Runs firmware mem test for 60 seconds
//---------------------------------------------------------------------------

bool X6_DacPatternModeSystem::DoMemoryTest()
{
    bool Test = false;
    ModeEnableReg.MemoryTestEnable.Value(1);
    Innovative::Sleep(60000);
    unsigned int status = GetMemoryStatus();
    if (status == 0)
        Test = true;
    else
        Test = false;
    ModeEnableReg.MemoryTestEnable.Value(0);
    return Test;
}



#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}   // end namespace Innovative
