//---------------------------------------------------------------------------
// X6SubDevice_AnalogOut_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "AnalogOut" Subdevices
//---------------------------------------------------------------------------

#ifndef X6SubDevice_AnalogOut_MbH
#define X6SubDevice_AnalogOut_MbH

#include <X6Support_Mb.h>
#include <SpiDevice_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6AFE_DacCommon -- X6 AFE DAC Common Registers "SubDevice"
//===========================================================================

class X6AFE_DacCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeDacEnable_Addr;
        unsigned int   AfeDacTriggerCfg_Addr;
        unsigned int   AfeDacDecimation_Addr;
        unsigned int   AfeDacReset_Addr;
        unsigned int   AfeDacPhaseIncrement_Addr;
    };

    X6AFE_DacCommon(IRequires_FrameCalcDevice * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~X6AFE_DacCommon();

    //
    //  Methods
    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  ChannelResetAll();
    void  ChannelReset(int ch, bool state);
    bool  ChannelReset(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    void   DacTestToneFrequency(double dac_clock_freq, double tone_freq, int channels); // in Hz
    double DacTestToneFrequency(double dac_clock_freq, int channels) const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacCommon::DacEnableRegister  -- Enable Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DacChannelEnables;
        RegisterBitGroup  DacChannelPower;

        unsigned int MaxDacChannels() {  return 16;  }

    public:
        DacEnableRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DacChannelEnables(*this, 0, 16),   DacChannelPower(*this, 16, 16)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacCommon::DacTriggerCfgRegister -- Configure DAC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DacFrameSize;
        RegisterBit       DacRisingEdge;
        RegisterBit       DacFramedMode;
        RegisterBit       DacExternalTrigger;

    public:
        DacTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DacFrameSize(*this, 0, 24),   DacRisingEdge(*this, 29),
               DacFramedMode(*this, 30),     DacExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacCommon::DacDecimationRegister  -- Dac Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        DacDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacCommon::DacResetRegister  -- Dac Reset Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacResetRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DacChannelResets;

        unsigned int MaxDacChannels() {  return 16;  }

    public:
        DacResetRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DacChannelResets(*this, 0, 16)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_FrameCalcDevice *  MDC;
    IFrameCalcDevice *        Dev;
    DacEnableRegister         DacEnableReg;
    DacTriggerCfgRegister     DacTriggerCfgReg;
    DacDecimationRegister     DacDecimationReg;
    DacResetRegister          DacResetReg;
    Register                  DacPhaseIncrReg;

};

//===========================================================================
//  CLASS X6AFE_DacSpi -- X6 AFE DAC Spi Device Manager "SubDevice"
//===========================================================================

class X6AFE_DacSpi : public IX6SubDevice
{
public:

    X6AFE_DacSpi(IRequires_DacSpi * mdc, IAddressingSpace & space);
    virtual ~X6AFE_DacSpi();

    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address) const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    void  MakeSpiDevice(const std::string & Tag, unsigned int Wr_Addr,
                                                 unsigned int Rd_Addr);

private:
    IRequires_DacSpi *  MDC;
    IAddressingSpace &          AFE_Space;
    std::vector<X6SpiDevice *>  SpiDev;
};

//===========================================================================
//  CLASS X6AFE_DacLatencyCalibration -- X6 DAC Latency Cal "SubDevice"
//===========================================================================

class X6AFE_DacLatencyCalibration : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS X6AFE_DacLatencyCalibration::MapRegisters  -- Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   LatencyCal_Addr;
    };

    X6AFE_DacLatencyCalibration(IRequires_DacLatencyCal * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6AFE_DacLatencyCalibration();

    void    CalibrateDacLatency();

    bool    CalibrationOk()
                {  return Cal->CalibrationOk();  }


    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacLatencyCalibration::DacLatencyCalRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacLatencyCalRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       CalEnable;
        RegisterBitGroup  CalResults;
        RegisterBitGroup  IoDelayTapCount;
        RegisterBitGroup  Output0ShiftCount;
        RegisterBitGroup  Output1ShiftCount;

    public:
        DacLatencyCalRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               CalEnable(*this, 0),
               CalResults(*this, 4, 4),
               IoDelayTapCount(*this, 8, 4),
               Output0ShiftCount(*this, 24, 4),
               Output1ShiftCount(*this, 28, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_DacLatencyCalibration::ILatencyRegIntf  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ILatencyRegIntf : public IDacCalibrationRegister
    {
    public:
        virtual unsigned int  GetCalResults()
            {  return DLC->LatencyCalArray[Idx].CalResults.Value();  }
        virtual void          CalEnable(bool enabled)
            {  DLC->LatencyCalArray[Idx].CalEnable.Value(enabled);  }
        virtual unsigned int  GetShiftCount(int idx)
            {
            if (idx==1)
                return DLC->LatencyCalArray[Idx].Output1ShiftCount.Value();
            else
                return DLC->LatencyCalArray[Idx].Output0ShiftCount.Value();
            }

        ILatencyRegIntf(X6AFE_DacLatencyCalibration * dlc, unsigned int idx=0)
            : DLC(dlc), Idx(idx)
            {}
        virtual ~ILatencyRegIntf()
            {}

    private:
        X6AFE_DacLatencyCalibration *  DLC;
        unsigned int                   Idx;
    };

    friend class ILatencyRegIntf;
    
    ILatencyRegIntf * GetLatencyRegisterIntf();

private:
    IRequires_DacLatencyCal *  MDC;
    RegisterArray<DacLatencyCalRegister>  LatencyCalArray;

    IDacCalibrator * Cal;

    unsigned int FDacDevices;

};

//===========================================================================
//  CLASS X6AFE_DacCalibration -- X6 DAC Calibration "SubDevice"
//===========================================================================

class X6AFE_DacCalibration : public IX6SubDevice, public IGainCalibrationDevice
{
public:
    //=======================================================================
    //  CLASS X6AFE_DacCalibration::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   GainCoeff_Addr;
        unsigned int   OffsetCoeff_Addr;
    };

    X6AFE_DacCalibration(IRequires_DacCalParts * mdc, IAddressingSpace & space, MapRegisters & regs);
    X6AFE_DacCalibration(IRequires_DacCalParts * mdc, IAddressingSpace & space, MapRegisters & regs, unsigned int MaxDacCount);
    virtual ~X6AFE_DacCalibration();

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value);
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    IRequires_DacCalParts *  MDC;
    RegisterArray<Register>    GainCoeffArray;
    RegisterArray<Register>    OffsetCoeffArray;

    unsigned int                FVitaOutDacs;
    ICalibrationValueOverride * CalOverride;

};

//===========================================================================
//  CLASS X6_DacPatternModeSystem -- X6 DAC Pattern Mode "SubDevice"
//===========================================================================

class X6_DacPatternModeSystem : public IX6SubDevice, public IPatternModeSystem
{
public:
    //=======================================================================
    //  CLASS X6_DacPatternModeSystem::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   PatternModeEnable_Addr;
    };

    X6_DacPatternModeSystem(IRequires_PatternMode * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6_DacPatternModeSystem();

    virtual unsigned int  CalcVitaPatternSize(unsigned int RawDataSizeWords, 
                                              unsigned int VitaDataSize,
                                              unsigned int sid_count = 1);

    virtual void  PatternModeEnable(bool state);
    virtual bool  PatternModeEnable();

    // "old" VELO packet command
    virtual void  SendPatternInfo(unsigned int data_pid,
                                  char tag_value, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int count,
                                  IPatternModeSystem::PatternCommands cmd = piPlay,
                                  IPatternModeSystem::PatternRepeatType repeat = prPlayAgain );
    // "new" VITA packet(s) command
    virtual void  SendPatternInfo(unsigned int data_pid, std::vector<unsigned int> & sids,
                                  std::vector<char> & tags, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int count,
                                  IPatternModeSystem::PatternCommands cmd,
                                  IPatternModeSystem::PatternRepeatType repeat );

    virtual unsigned int  Alignment() {  return PatternModeAlignment;  }
    virtual unsigned int  ScaleFactor() {  return PatternModeScaleFactor;  }

    //  Memory test
    virtual void            MemoryTestEnable(unsigned int val);
    virtual unsigned int    GetMemoryStatus();
    virtual unsigned int    MemoryInit();
    virtual bool            DoMemoryTest(); // pass is true fail is false


	//  Other Methods
	void            DramPowerDown(bool state);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    void            DefineModeEnableValues(int on_bits, int off_bits)
                        { FModeOnValue = on_bits,  FModeOffValue = off_bits;  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_DacPatternModeSystem::ModeEnableRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ModeEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DramPowerDown;
        RegisterBitGroup  DramPhyInitDone;
        RegisterBitGroup  PlaybackEnable;
        RegisterBitGroup  MemoryTestEnable;
        RegisterBitGroup  MemoryTestError;

    public:
        ModeEnableRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DramPowerDown(*this, 0, 4),
               DramPhyInitDone(*this, 4, 4),
               PlaybackEnable(*this, 8, 4),
               MemoryTestEnable(*this, 12, 4),
               MemoryTestError(*this, 16, 4)
            {
            }
    };

private:
    IRequires_PatternMode *  MDC;
    ModeEnableRegister   ModeEnableReg;
    class IIsAVitaPacketStreamBoard *      Board;
    
    bool  FModeEnabled;
    int   FModeOnValue;
    int   FModeOffValue;

    unsigned int  PatternModeAlignment;
    unsigned int  PatternModeScaleFactor;
    bool          SingleMemoryFlag;

    void  FillVitaCommandBuffer(VitaBuffer & Command, unsigned int  sid, char tag,
            unsigned int start_addr, unsigned int per_device_size_ints,
            unsigned int rep_count, PatternCommands cmd, PatternRepeatType repeat );
    void  CopyVitaToDataBuffer(VeloBuffer & Packet, VitaBuffer & Command, unsigned int pos );
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}   // end namespace Innovative
//---------------------------------------------------------------------------
#endif
