//---------------------------------------------------------------------------
// X6SubDevice_Misc_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "Miscellaneous" Subdevices
//---------------------------------------------------------------------------

#include <X6SubDevice_Misc_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <SystemSupport_Mb.h>

#include <Poco/Foundation/include/Poco/DateTime.h>
#include <Poco/Foundation/include/Poco/DateTimeFormat.h>
#include <Poco/Foundation/include/Poco/DateTimeFormatter.h>

//---------------------------------------------------------------------------

namespace Innovative
{

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6Info -- X6 Information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6Info
//---------------------------------------------------------------------------

X6Info::X6Info(IRequires_Nothing * mdc, IAddressingSpace & space,
                        IAddressingSpace & bar_0space, MapRegisters & regs, bool bitstream_date_valid)
    :  MDC(mdc),
       PciLogicVersionReg(bar_0space, regs.Bar0Version_Addr),
       FpgaLogicVersionReg(space, regs.Info_Addr),
       K7FpgaLogicVersionReg(space, regs.Info_Addr, FpgaLogicVersionRegister::styleK7),
       FpgaLogicSubRevReg(space, regs.Subrev_Addr),
       BitStreamDateReg(space, regs.BitstreamDate_Addr),
       BitStreamDateValid(bitstream_date_valid)
{
    // date valid for K7 and later boards
}

//---------------------------------------------------------------------------
//  constructor for class X6Info
//---------------------------------------------------------------------------

X6Info::X6Info(IRequires_Nothing * mdc, IAddressingSpace & space,
                        IAddressingSpace & bar_0space, MapRegisters & regs, PciVersionRegStyle /*style*/)
    :  MDC(mdc),
       PciLogicVersionReg(bar_0space, regs.Bar0Version_Addr, PciLogicVersionRegister::styleXU),
       FpgaLogicVersionReg(space, regs.Info_Addr, FpgaLogicVersionRegister::styleXU),
       /*needed*/ K7FpgaLogicVersionReg(space, regs.Info_Addr, FpgaLogicVersionRegister::styleK7),
       FpgaLogicSubRevReg(space, regs.Subrev_Addr),
       BitStreamDateReg(space, regs.BitstreamDate_Addr),
       BitStreamDateValid(true)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6Info
//---------------------------------------------------------------------------

X6Info::~X6Info()
{
}

//---------------------------------------------------------------------------
//  X6Info::OpenHardware()
//---------------------------------------------------------------------------

void  X6Info::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6Info::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6Info::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6Info::PciLogicRevision() -- PCI Rev Code
//---------------------------------------------------------------------------

short X6Info::PciLogicRevision() const
{
    return PciLogicVersionReg.Revision.Value();
}

//---------------------------------------------------------------------------
//  X6Info::PciLogicFamily() -- PCI Family Code
//---------------------------------------------------------------------------

short X6Info::PciLogicFamily() const
{
    return PciLogicVersionReg.Family.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaLogicVersion() -- FPGA Version Code
//---------------------------------------------------------------------------

short X6Info::FpgaLogicVersion() const
{
    return FpgaLogicVersionReg.Version.Value();
}

//---------------------------------------------------------------------------
//  X6Info::K7FpgaLogicVersion() -- K7 FPGA Version Code
//---------------------------------------------------------------------------

short X6Info::K7FpgaLogicVersion() const
{
    return K7FpgaLogicVersionReg.Version.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaHardwareVariant() -- FPGA Hardware Variant Code
//---------------------------------------------------------------------------

short X6Info::FpgaHardwareVariant() const
{
    return FpgaLogicVersionReg.HdwVariant.Value();
}

//---------------------------------------------------------------------------
//  X6Info::K7FpgaHardwareVariant() -- K7 FPGA Hardware Variant Code
//---------------------------------------------------------------------------

short X6Info::K7FpgaHardwareVariant() const
{
    return K7FpgaLogicVersionReg.HdwVariant.Value();
}

//---------------------------------------------------------------------------
//  X6Info::PciLogicPcb() -- Hardware Board Rev Code
//---------------------------------------------------------------------------

short X6Info::PciLogicPcb() const
{
    return FpgaLogicVersionReg.BoardHwRev.Value();
}

//---------------------------------------------------------------------------
//  X6Info::PciLogicType() -- Hardware Board Type Code
//---------------------------------------------------------------------------

short X6Info::PciLogicType() const
{
    return FpgaLogicVersionReg.BoardHwType.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaChipType() -- FPGA Chip Type Code
//---------------------------------------------------------------------------

short X6Info::FpgaChipType() const
{
    return FpgaLogicVersionReg.FpgaType.Value();
}

//---------------------------------------------------------------------------
//  X6Info::K7BoardHwType() -- K7 Board Hw Type
//---------------------------------------------------------------------------

short X6Info::K7BoardHwType() const
{
    return K7FpgaLogicVersionReg.BoardHwType.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaChipType() -- FPGA Chip Type Code
//---------------------------------------------------------------------------

short X6Info::K7FpgaChipType() const
{
    return FpgaLogicVersionReg.FpgaType.Value();
}

//---------------------------------------------------------------------------
//  X6Info::BoardRevisionBuiltFor() -- Which revision is this logic for?
//---------------------------------------------------------------------------
//  0xF = all versions...

short X6Info::BoardRevisionBuiltFor() const
{
    return FpgaLogicSubRevReg.BoardRevBuiltFor.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaLogicSubrevision() -- FPGA Logic Subrev Code
//---------------------------------------------------------------------------

short X6Info::FpgaLogicSubrevision() const
{
    return FpgaLogicSubRevReg.SubRevision.Value();
}

//---------------------------------------------------------------------------
//  X6Info::PciLogicRawValue() -- read entire register
//---------------------------------------------------------------------------

unsigned int  X6Info::PciLogicRawValue() const
{
    return PciLogicVersionReg.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaInformationRawValue() -- read entire register
//---------------------------------------------------------------------------

unsigned int  X6Info::FpgaInformationRawValue() const
{
    return FpgaLogicVersionReg.Value();
}

//---------------------------------------------------------------------------
//  X6Info::FpgaSubRevRawValue() -- read entire register
//---------------------------------------------------------------------------

unsigned int  X6Info::FpgaSubRevRawValue() const
{
    return FpgaLogicSubRevReg.Value();
}

//---------------------------------------------------------------------------
//  X6Info::BitStreamDate() -- Get elements of BitStream creation date
//---------------------------------------------------------------------------

struct tm  X6Info::BitStreamDate() const
{
    struct tm TMS;
    if (BitStreamDateValid)
        {
        TMS.tm_hour  = BitStreamDateReg.Hour.Value();
        TMS.tm_min   = BitStreamDateReg.Minute.Value();
        TMS.tm_sec   = BitStreamDateReg.Second.Value();
        TMS.tm_isdst = 0;
        TMS.tm_mday  = BitStreamDateReg.Day.Value();
        TMS.tm_mon   = BitStreamDateReg.Month.Value() - 1;    // Jan = 0
        TMS.tm_year  = BitStreamDateReg.Year2K.Value() + 100; //  convert to year 1900 base
        }
    else
        {
        // return a reasonable, but wrong, timestamp
        TMS.tm_hour = 12;  TMS.tm_min = 0;  TMS.tm_sec=0;  TMS.tm_isdst = 0;
        TMS.tm_mday = 1;   TMS.tm_mon = 0;  TMS.tm_year=100;  //  year 2000
        }

    return TMS;
}

//---------------------------------------------------------------------------
//  X6Info::BitStreamDateString() -- Human readable date string
//---------------------------------------------------------------------------

std::string  X6Info::BitStreamDateString() const
{
    try
        {
        Poco::DateTime  PDT;

        if (BitStreamDateValid)
            {
            PDT.assign(BitStreamDateReg.Year2K.Value() + 2000, //  convert to year 0 base
                       BitStreamDateReg.Month.Value(),
                       BitStreamDateReg.Day.Value(),
                       BitStreamDateReg.Hour.Value(),
                       BitStreamDateReg.Minute.Value(),
                       BitStreamDateReg.Second.Value()
                       );
            }

        std::string str = Poco::DateTimeFormatter::format(PDT, Poco::DateTimeFormat::ASCTIME_FORMAT, 0);
        return str;
        }
    catch (...)
        {
        return "BAD DATE";
        }
}

//---------------------------------------------------------------------------
//  X6Info::BitStreamDateString() -- Human readable date string
//---------------------------------------------------------------------------

std::string  X6Info::BitStreamDateString_YMD() const
{
    try
        {
        Poco::DateTime  PDT;

        if (BitStreamDateValid)
            {
            PDT.assign(BitStreamDateReg.Year2K.Value() + 2000, //  convert to year 0 base
                       BitStreamDateReg.Month.Value(),
                       BitStreamDateReg.Day.Value(),
                       BitStreamDateReg.Hour.Value(),
                       BitStreamDateReg.Minute.Value(),
                       BitStreamDateReg.Second.Value()
                       );
            }

        std::string FormatType = "%b %e %Y";
        std::string str = Poco::DateTimeFormatter::format(PDT, FormatType, 0);
        return str;
        }
    catch (...)
        {
        return "BAD Y:M:D";
        }
}

//---------------------------------------------------------------------------
//  X6Info::BitStreamDateString() -- Human readable date string
//---------------------------------------------------------------------------

std::string  X6Info::BitStreamDateString_HMS() const
{
    try
        {
        Poco::DateTime  PDT;

        if (BitStreamDateValid)
            {
            PDT.assign(BitStreamDateReg.Year2K.Value() + 2000, //  convert to year 0 base
                       BitStreamDateReg.Month.Value(),
                       BitStreamDateReg.Day.Value(),
                       BitStreamDateReg.Hour.Value(),
                       BitStreamDateReg.Minute.Value(),
                       BitStreamDateReg.Second.Value()
                       );
            }

        std::string FormatType = "%H:%M:%S";
        std::string str = Poco::DateTimeFormatter::format(PDT, FormatType, 0);
        return str;
        }
    catch (...)
        {
        return "BAD H:M:S";
        }
}
//===========================================================================
//  CLASS X6Peripherals -- X6 Information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6Peripherals
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6Peripherals
//---------------------------------------------------------------------------

X6Peripherals::X6Peripherals(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       ResetNLedReg(space, regs.ResetReg_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6Peripherals
//---------------------------------------------------------------------------

X6Peripherals::~X6Peripherals()
{
}

//---------------------------------------------------------------------------
//  X6Peripherals::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6Peripherals::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6Peripherals::OpenHardware()
//---------------------------------------------------------------------------

void  X6Peripherals::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  X6Peripherals::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6Peripherals::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  X6Peripherals::GreenLed() -- Read the Green Led state
//---------------------------------------------------------------------------

bool  X6Peripherals::GreenLed()
{
    return ResetNLedReg.GreenLed.Value();
}

//---------------------------------------------------------------------------
//  X6Peripherals::GreenLed() -- Set the Green Led state
//---------------------------------------------------------------------------

void  X6Peripherals::GreenLed( bool value )
{
    ResetNLedReg.GreenLed.Value(value);
}

//---------------------------------------------------------------------------
//  X6Peripherals::FrontPanelLed() -- Read the FrontPanel Led state
//---------------------------------------------------------------------------

bool  X6Peripherals::FrontPanelLed()
{
    return ResetNLedReg.FrontPanelLed.Value();
}

//---------------------------------------------------------------------------
//  X6Peripherals::FrontPanelLed() -- Set the FrontPanel Led state
//---------------------------------------------------------------------------

void  X6Peripherals::FrontPanelLed( bool value )
{
    ResetNLedReg.FrontPanelLed.Value(value);
}

//---------------------------------------------------------------------------
//  X6Peripherals::AppReset() -- Apply "Application" Reset
//---------------------------------------------------------------------------

void  X6Peripherals::AppReset()
{
    Debug.Log("App Reset Applied");
    //
    //  Reset User Logic
    ResetNLedReg.FpgaReset.Set();
    Sleep(1);
    ResetNLedReg.FpgaReset.Clear();
    Sleep(1);

}

//---------------------------------------------------------------------------
//  X6Peripherals::FmcCRun() --  (FMC only)
//---------------------------------------------------------------------------

void  X6Peripherals::FmcCRun(bool state)
{
    if (state)
        Debug.Log("FMC Carrier Run ON");
    else
        Debug.Log("FMC Carrier Run OFF");

    ResetNLedReg.FmcCarrierRun.Value(state);
}

//---------------------------------------------------------------------------
//  X6Peripherals::XURun() --  (XU only)
//---------------------------------------------------------------------------

void  X6Peripherals::XURun(bool state)
{
    if (state)
        Debug.Log("XU Run ON");
    else
        Debug.Log("XU Run OFF");

    ResetNLedReg.FmcCarrierRun.Value(state);
}

//---------------------------------------------------------------------------
//  X6Peripherals::XARun() --  (XU only)
//---------------------------------------------------------------------------

void  X6Peripherals::XARun(bool state)
{
    if (state)
        Debug.Log("XA Run ON");
    else
        Debug.Log("XA Run OFF");

    ResetNLedReg.FmcCarrierRun.Value(state);
}

//===========================================================================
//  CLASS X6SioPll -- X6 Information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6SioPll
//---------------------------------------------------------------------------

const int     SioPllI2C_SubAddr  = 0xE0;     // 0x70 left shifted one

//---------------------------------------------------------------------------
//  constructor for class X6SioPll
//---------------------------------------------------------------------------

X6SioPll::X6SioPll(IRequires_SioPll * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       SioPll_I2CReg(space, regs.SioPll_Addr),
       SioPllIntf(SioPll_I2CReg),
       SioPll_I2C(SioPllIntf, I2CDevice::SubAddress, SioPllI2C_SubAddr),
       SioPllDev(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6SioPll
//---------------------------------------------------------------------------

X6SioPll::~X6SioPll()
{
    delete SioPllDev;  SioPllDev=0;
}

//---------------------------------------------------------------------------
//  X6SioPll::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6SioPll::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SioPllDev = MDC->MakeSioPllObject(SioPll_I2C);
}

//---------------------------------------------------------------------------
//  X6SioPll::OpenHardware()
//---------------------------------------------------------------------------

void  X6SioPll::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    SioPllDev->Init();
}

//---------------------------------------------------------------------------
//  X6SioPll::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6SioPll::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    SioPllDev->Reapply();
}

//---------------------------------------------------------------------------
//  X6SioPll::ClockOutputEnable() - Set clock output
//---------------------------------------------------------------------------

void  X6SioPll::ClockOutputEnable(bool state)
{
    SioPll_I2CReg.SioPll_OutputEnable.Value(state);
}

//---------------------------------------------------------------------------
//  X6SioPll::ClockOutputEnable() - Return clock output state
//---------------------------------------------------------------------------

bool  X6SioPll::ClockOutputEnable()
{
    return SioPll_I2CReg.SioPll_OutputEnable.Value();
}

//---------------------------------------------------------------------------
//  X6SioPll::ExternalReference()
//---------------------------------------------------------------------------

void  X6SioPll::ExternalReference(bool state)
{
    SioPll_I2CReg.SioPll_ExtRefSelect.Value(!state);
}

//---------------------------------------------------------------------------
//  X6SioPll::ExternalReference()
//---------------------------------------------------------------------------

bool  X6SioPll::ExternalReference()
{
    return SioPll_I2CReg.SioPll_ExtRefSelect.Value()==false;
}

//==============================================================================
//  CLASS X6FpaCpldPersonality -- X6 FPA Personality
//==============================================================================
//  Uses IX6FpaCpld to implement its personality

class X6FpaCpldPersonality : public IX6FpaCpldPersonality
{
public:
    X6FpaCpldPersonality(IX6FpaCpld & cpld)
        :  IX6FpaCpldPersonality(cpld)
        {}
    virtual ~X6FpaCpldPersonality()
        {}
};

//===========================================================================
//  CLASS X6FpaLoader -- X6 FPA Loader "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6FpaLoader
//---------------------------------------------------------------------------

const double cpldBusyWaitTimeout    = 0.001;  // in seconds
const double cpldReadValidTimeout   = 0.5;    // in seconds
const double cpldFlashReadyTimeout  = 0.5;    // in seconds

//---------------------------------------------------------------------------
//  constructor for class X6FpaLoader
//---------------------------------------------------------------------------

X6FpaLoader::X6FpaLoader(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       FpaAddress(space, regs.FpaAddress_Addr),
       FpaData(space, regs.FpaData_Addr),
       FpaCommand(space, regs.FpaCommand_Addr),
       FGoldenImageMode(false),
       CpldIntf(this),
       Thread( new X6FpaCpldPersonality( CpldIntf ))
{
}

//---------------------------------------------------------------------------
//  destructor for class X6FpaLoader
//---------------------------------------------------------------------------

X6FpaLoader::~X6FpaLoader()
{
    Thread.Shutdown();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::OpenHardware()
//---------------------------------------------------------------------------

void  X6FpaLoader::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Thread.OnOpen();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6FpaLoader::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Address()
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_Address(unsigned int addr)
{
    FpaAddress.Addr.Value(addr);
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Data() -- write data
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_Data(unsigned int data)
{
    FpaData.Value(data);
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Data()  -- read data
//---------------------------------------------------------------------------

unsigned int  X6FpaLoader::CPLD_Data()
{
    return FpaData.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Command()
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_Command(unsigned int code)
{
    FpaCommand.Command.Value(code);
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Version()  -- read data
//---------------------------------------------------------------------------

unsigned int  X6FpaLoader::CPLD_Version()
{
    return FpaCommand.CPLDVersion.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_FlashType()  -- read data
//---------------------------------------------------------------------------

unsigned int  X6FpaLoader::CPLD_FlashType()
{
    return FpaCommand.FlashType.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_GoldenImageBooted()  -- read state of golden jumper
//---------------------------------------------------------------------------

bool  X6FpaLoader::CPLD_GoldenImageBooted()
{
    return FpaCommand.GoldenImageBooted.Value()==false;  // 0 = Golden Boot
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_GoldenImageActive()  -- read golden image state
//---------------------------------------------------------------------------

bool  X6FpaLoader::CPLD_GoldenImageActive()
{
    return FGoldenImageMode;
    //return FpaCommand.GoldenImageInUse.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Busy()  -- read bit
//---------------------------------------------------------------------------

bool  X6FpaLoader::CPLD_Busy()
{
    return FpaCommand.Busy.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_ReadValid()  -- read bit
//---------------------------------------------------------------------------

bool  X6FpaLoader::CPLD_ReadValid()
{
    return FpaCommand.ReadValid.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_FlashReady()  -- read bit
//---------------------------------------------------------------------------

bool  X6FpaLoader::CPLD_FlashReady()
{
    return FpaCommand.FlashReady.Value();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_OverTemperature()  -- read bit (inverted)
//---------------------------------------------------------------------------

bool  X6FpaLoader::CPLD_OverTemperature()
{
    return FpaCommand.OverTemp.Value()==0;
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Write()  -- Write To Flash
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_Write(unsigned int addr, unsigned int data)
{
    CPLD_BusyWait();
    CPLD_Address(addr);

    CPLD_BusyWait();
    CPLD_Data(data);

    CPLD_BusyWait();
    CPLD_Command(cmdFlashWrite);

    CPLD_FlashReadyWait();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_Read()  -- Read Flash
//---------------------------------------------------------------------------

unsigned int  X6FpaLoader::CPLD_Read(unsigned int addr)
{
    CPLD_BusyWait();
    CPLD_Address(addr);

    CPLD_BusyWait();
    CPLD_Command(cmdFlashRead);

    CPLD_ReadValidWait();
    return CPLD_Data();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_ReadPowerStatus()  -- Read Pwr Status
//---------------------------------------------------------------------------

unsigned int  X6FpaLoader::CPLD_ReadPowerStatus()
{
    CPLD_BusyWait();
    CPLD_Command(cmdPowerStatus);

    CPLD_ReadValidWait();
    return CPLD_Data();
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_GoldenImageActive()  -- Make selected image current
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_GoldenImageActive(bool state)
{
    FGoldenImageMode = state;      // no longer read hardware for state

    //  Toggle the Golden/normal image mode
    // If we are already in the state, do nothing
    //if (CPLD_GoldenImageActive()==state)
    //    return;
    //CPLD_BusyWait();
    //CPLD_Command(cmdSwapImage);

    // Update status - automatic after command
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_RestartConfig()  -- Restart ROM confiugration
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_RestartConfig(bool restart_golden_image)
{
    // write a number not in the pattern to reset state machine
    CPLD_BusyWait();
    CPLD_Address(0x00000000);

    CPLD_BusyWait();
    CPLD_Address(0xAAAAAAAA);
    CPLD_BusyWait();
    CPLD_Address(0xBBBBBBBB);
    CPLD_BusyWait();
    CPLD_Address(0xCCCCCCCC);

    unsigned int image_code(0);
    if (restart_golden_image)
        image_code = 0x07000000;

    CPLD_BusyWait();
    CPLD_Address(image_code);    // tell which image to boot
    CPLD_BusyWait();
    CPLD_Command(cmdRestartCfg);
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_UpdateStatus()  -- Update status register
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_UpdateStatus()
{
    CPLD_BusyWait();
    CPLD_Command(cmdReadStatus);
}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_BusyWait()  -- Wait For BUSY flag
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_BusyWait()
{
    // Update status - automatic after command
    const double Timeout = cpldBusyWaitTimeout;   // in sec
    double Then = Rtc.Now() + Timeout;
	while (CPLD_Busy() == true)
        {
        if (Rtc.Now() > Then)
            {
            Debug.Log("Timeout waiting for CPLD Busy OFF");
            break;
            }
        }

}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_ReadValidWait()  -- Wait For Read Valid flag
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_ReadValidWait()
{
    RefreshStatus();
    // Update status - automatic after command
    const double Timeout = cpldReadValidTimeout;   // in sec
    double Then = Rtc.Now() + Timeout;
	while (CPLD_ReadValid() == false)
        {
        RefreshStatus();
        if (Rtc.Now() > Then)
            {
            Debug.Log("Timeout waiting for CPLD Read Valid ON");
            break;
            }
        }

}

//---------------------------------------------------------------------------
//  X6FpaLoader::CPLD_FlashReadyWait()  -- Wait For Flash Ready flag
//---------------------------------------------------------------------------

void  X6FpaLoader::CPLD_FlashReadyWait()
{
    RefreshStatus();
    // Update status - automatic after command
    const double Timeout = cpldFlashReadyTimeout;   // in sec
    double Then = Rtc.Now() + Timeout;
	while (CPLD_FlashReady() == false)
        {
        RefreshStatus();
        if (Rtc.Now() > Then)
            {
            Debug.Log("Timeout waiting for CPLD Flash Ready ON");
            break;
            }
        }

}

//---------------------------------------------------------------------------
//  X6FpaLoader::RefreshStatus()  -- Update Status Register Bits
//---------------------------------------------------------------------------

void  X6FpaLoader::RefreshStatus()
{
    CPLD_UpdateStatus();
}

//===========================================================================
//  CLASS X6DigitalIo -- X6 Digital Io "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6DigitalIo
//---------------------------------------------------------------------------

X6DigitalIo::X6DigitalIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       DataLo(space, regs.DioDataLo_Addr),
       DataHi(space, regs.DioDataHi_Addr),
       OutEnLo(space, regs.DioOutputEnableLo_Addr),
       OutEnHi(space, regs.DioOutputEnableHi_Addr),
       DataLoManip(DataLo),
       DataHiManip(DataHi)
{

}

//---------------------------------------------------------------------------
//  destructor for class X6DigitalIo
//---------------------------------------------------------------------------

X6DigitalIo::~X6DigitalIo()
{
}

//---------------------------------------------------------------------------
//  X6DigitalIo::OpenHardware()
//---------------------------------------------------------------------------

void  X6DigitalIo::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6DigitalIo::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6DigitalIo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortData() -- access to Lo Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  X6DigitalIo::DioPortData()
{
    return DataLoManip;
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

void  X6DigitalIo::DioPortConfig(unsigned int mask)
{
    OutEnLo.Value(mask);
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

unsigned int  X6DigitalIo::DioPortConfig() const
{
    return OutEnLo.Value();
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortDataHigh() -- access to Hi Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  X6DigitalIo::DioPortDataHigh()
{
    return DataHiManip;
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortConfigHigh() -- read/change High OE config mask
//---------------------------------------------------------------------------

void  X6DigitalIo::DioPortConfigHigh(unsigned int mask)
{
    OutEnHi.Value(mask);
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortConfigHigh() -- read/change High OE config mask
//---------------------------------------------------------------------------

unsigned int  X6DigitalIo::DioPortConfigHigh() const
{
    return OutEnHi.Value();
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

void  X6DigitalIo::DioPortOutputEnable(int output_bit, bool output_enabled)
{
    if (output_bit<32)
        OutEnLo.Bit(output_bit, output_enabled);
    else
        OutEnHi.Bit(output_bit-32, output_enabled);
}

//---------------------------------------------------------------------------
//  X6DigitalIo::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

bool  X6DigitalIo::DioPortOutputEnable(int output_bit) const
{
    if (output_bit<32)
        return OutEnLo.Bit(output_bit);
    else
        return OutEnHi.Bit(output_bit-32);
}

//===========================================================================
//  CLASS X6Thermal -- X6 Tempearature Control "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6Thermal
//---------------------------------------------------------------------------

X6Thermal::X6Thermal(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TempControlReg(space, regs.TempControl_Addr),
       FanThresholdTempReg(space, regs.FanThresholdTemp_Addr),
       AlertThresholdTempReg(space, regs.AlertThresholdTemp_Addr),
       TempCommandReg(space, regs.CommandChannel_Addr),
       TempCommandReadReg(space, regs.CommandChannel_Addr),
       ManualTemperatureMode(false)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6Thermal
//---------------------------------------------------------------------------

X6Thermal::~X6Thermal()
{
}

//---------------------------------------------------------------------------
//  X6Thermal::OpenHardware()
//---------------------------------------------------------------------------

void  X6Thermal::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // Post Open Sanity Check
    if (TempControlReg.SensorFault.Value())
        {
        //  Disable monitor to allow talking to Fan Port
        TempControlReg.MonitorEnable.Value(false);
        ManualTemperatureMode = true;
        Sleep(1);
        }
    else
        ManualTemperatureMode = false;

    // Wait for power on reset to clear
    for (int i = 0; (Port(0x33) & 0x80) && i < 10; ++i)
        Sleep(1);
    if (Port(0x33) & 0x80)
        {  }     // Error

    // Enable Remote diode TruTherm mode
    Port(0x30, 2);

    //
    Port(0x4a, 0x20); // 0V=off, 360 kHz clock, Tach mode 0, PWPGM=1
//    Port(0x45, 0x30); // 8-bit temp res, 22.5 kHz PWM resolution
    Port(0x45, 0x00); // 8-bit temp res, 22.5 kHz PWM resolution
    Port(0x4b, 0x3f); // spin-up
    Port(0x4d, 0x17); // PWM frequency
#if 0
    // Run at max PWM frequency
    Port(0x4c, 0x3f);
#else
    // Run at PWM frequency specified by table entries below
    short temp[12] = {35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90};
    short pwm[12] = {5, 10, 15, 20, 25, 30, 35, 40, 45, 55, 60, 63};
    for (int i = 0; i < 12; ++i)
        {
        Port(0x50 + 2*i, ToDeviceTemp(temp[i], 8));
        Port(0x51 + 2*i, pwm[i]);
        }
#endif
    Port(0x4a, 0x00); // Apply LUT, Enable PWPGM

    // Set Critical threshold
    //LogicCriticalSetpoint(110);  -- use logic default, as changing here is too late

    MonitorEnable(true);

    // Enable tach readback and setting critical setpoint
    MemoryBitManipulator Reg03(Port(0x03));
    Reg03.Bit(1, true);
    Reg03.Bit(2, true);   // tach readback

    Port(0x03, Reg03.Value());
}

//---------------------------------------------------------------------------
//  X6Thermal::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6Thermal::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//------------------------------------------------------------------------------
//  X6Thermal::LogicTemperature() -- Read current logic temperature (Celsius)
//------------------------------------------------------------------------------

int  X6Thermal::LogicTemperature() const
{
    if (!DeviceEnabled())
        return 0;

    X6Thermal * This = const_cast<X6Thermal *>(this);

    if (ManualTemperatureMode)
        return TempSignExtend(This->Port(0x00));   // return local temperature
    else
        return TempSignExtend(TempControlReg.Temperature.Value());
}

//------------------------------------------------------------------------------
//  X6Thermal::LogicFanSetpoint() -- Read current fan setpoint
//------------------------------------------------------------------------------

int  X6Thermal::LogicFanSetpoint() const
{
    if (!DeviceEnabled())
        return 0;

    return TempSignExtend(FanThresholdTempReg.Temperature.Value());
}

//------------------------------------------------------------------------------
//  X6Thermal::LogicFanSetpoint() -- Set current fan setpoint
//------------------------------------------------------------------------------

void  X6Thermal::LogicFanSetpoint(int temp)
{
    if (!DeviceEnabled())
        return;

    FanThresholdTempReg.Temperature.Value(temp);
}

//------------------------------------------------------------------------------
//  X6Thermal::LogicAlertSetpoint() -- Read current alert setpoint (Celsius)
//------------------------------------------------------------------------------

int  X6Thermal::LogicAlertSetpoint() const
{
    if (!DeviceEnabled())
        return 0;

    return TempSignExtend(AlertThresholdTempReg.Temperature.Value());
}

//------------------------------------------------------------------------------
//  X6Thermal::LogicAlertSetpoint() -- Set current Alert setpoint
//------------------------------------------------------------------------------

void  X6Thermal::LogicAlertSetpoint(int temp)
{
    if (!DeviceEnabled())
        return;

    AlertThresholdTempReg.Temperature.Value(temp);
}

//------------------------------------------------------------------------------
//  X6Thermal::MonitorEnable() -- Read Monitor Enable state
//------------------------------------------------------------------------------

bool  X6Thermal::MonitorEnable() const
{
    if (!DeviceEnabled())
        return false;

    return TempControlReg.MonitorEnable.Value();
}

//------------------------------------------------------------------------------
//  X6Thermal::MonitorEnable() -- Write Monitor Enable state
//------------------------------------------------------------------------------

void  X6Thermal::MonitorEnable(bool state)
{
    if (!DeviceEnabled())
        return;
    TempControlReg.MonitorEnable.Value(state);
}

//------------------------------------------------------------------------------
//  X6Thermal::CommFault() -- Read CommFault state
//------------------------------------------------------------------------------

bool  X6Thermal::CommFault() const
{
    if (!DeviceEnabled())
        return false;
    return TempControlReg.CommFault.Value();
}

//------------------------------------------------------------------------------
//  X6Thermal::SensorInReset() -- Read SensorInReset state
//------------------------------------------------------------------------------

bool  X6Thermal::SensorInReset() const
{
    if (!DeviceEnabled())
        return false;
    return TempControlReg.SensorInReset.Value();
}

//------------------------------------------------------------------------------
//  X6Thermal::SensorFault() -- Read SensorFault state
//------------------------------------------------------------------------------

bool  X6Thermal::SensorFault() const
{
    if (!DeviceEnabled())
        return false;

    return TempControlReg.SensorFault.Value();
}

//------------------------------------------------------------------------------
//  X6Thermal::LogicCriticalSetpoint() -- Change critical setpoint
//------------------------------------------------------------------------------

void  X6Thermal::LogicCriticalSetpoint(int temp_deg_C)
{
    if (!DeviceEnabled())
        return;

    Port(0x19, ToDeviceTemp(static_cast<float>(temp_deg_C), 8));
}

//------------------------------------------------------------------------------
//  X6Thermal::ToDeviceTemp() -- Convert to local temperature
//------------------------------------------------------------------------------

int X6Thermal::ToDeviceTemp(float temp_deg_C, int width) const
{
    if (!DeviceEnabled())
        return 0;

    float scalefactor = TempScalefactor(width);
    int shift = (width > 8) ? (16-width) : 0;

    return static_cast<int>(temp_deg_C / scalefactor) << shift;
}

//------------------------------------------------------------------------------
//  X6Thermal::FromDeviceTemp() -- Convert from local temperature
//------------------------------------------------------------------------------

float X6Thermal::FromDeviceTemp(int temp_deg_C, int width) const
{
    if (!DeviceEnabled())
        return 0.0f;

    float scalefactor = TempScalefactor(width);
    int shift = (width > 8) ? (16-width) : 0;

    return (static_cast<int>(temp_deg_C) >> shift) * scalefactor;
}

//------------------------------------------------------------------------------
//  X6Thermal::TempScalefactor() -- Return temperature scalefactor
//------------------------------------------------------------------------------

float X6Thermal::TempScalefactor(int width) const
{
    if (width < 11)
        return 1.0f;
    else if (width > 11)
        return 0.03125f;
    else
        return 0.125f;
}

//------------------------------------------------------------------------------
//  X6Thermal::TempSignExtend() -- properly sign extend 8 bit temperature #
//------------------------------------------------------------------------------

int  X6Thermal::TempSignExtend(unsigned int temp_8bit) const
{
    int signed_temp = temp_8bit;
    return ((signed_temp<<24)>>24);   // propogate bit 8 across all bits
}

//------------------------------------------------------------------------------
//  X6Thermal::Port() -- Write to internal register
//------------------------------------------------------------------------------

void X6Thermal::Port(int addr, int value)
{
    if (!DeviceEnabled())
        return;

    for (int i = 0; !TempCommandReadReg.Ready && (i < 10); ++i)
        Sleep(1);
    if (!TempCommandReadReg.Ready)
        return;

    TempCommandReg.Addr.Value(addr);
    TempCommandReg.WriteData.Value(value);
    TempCommandReg.RWDirection.Value(0x0);
    TempCommandReg.Apply();
}

//------------------------------------------------------------------------------
//  X6Thermal::Port() -- Read from internal register
//------------------------------------------------------------------------------

int X6Thermal::Port(int addr)
{
    if (!DeviceEnabled())
        return -1;

    for (int i = 0; !TempCommandReadReg.Ready && (i < 10); ++i)
        Sleep(1);

    if (!TempCommandReadReg.Ready)
        return -2;

    TempCommandReg.Addr.Value(addr);
    TempCommandReg.RWDirection.Value(0x1);
    TempCommandReg.Apply();

    for (int i = 0; !TempCommandReadReg.ReadValid && (i < 10); ++i)
        Sleep(1);

    if (!TempCommandReadReg.ReadValid)
        return -3;

    return TempCommandReadReg.ReadData;
}

//------------------------------------------------------------------------------
//  X6Thermal::Rpm() -- Read fan speed
//------------------------------------------------------------------------------

int X6Thermal::Rpm()
{
    int lo = Port(0x46) & 0xfc;
    int hi = Port(0x47) << 8;
    int tach = hi+lo;
    if (tach)
        return 5400000/tach;
    else
        return 0;
}

//===========================================================================
//  CLASS X6Aurora -- X6 Aurora Device section "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6Aurora
//---------------------------------------------------------------------------

X6Aurora::X6Aurora(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs, SfpType type)
    :  MDC(mdc),
       TestControlReg(space, regs.TestControl_Addr),
       ControlStatusReg(space, regs.ControlStatus_Addr),
       WritePortReg(space, regs.WritePort_Addr),
       ReadPortRegReg(space, regs.ReadPort_Addr),
       SfpEnabled(false),
       SfpPortControlReg(space, regs.SfpPortControl_Addr),
       SfpPortI2CReg(space, regs.SfpPortI2C_Addr),
       QSfpPortControlReg(space, regs.SfpPortControl_Addr),
       FSfpPtr(0), FSfpType(type) //FSfp(this), FQSfp(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6Aurora
//---------------------------------------------------------------------------

X6Aurora::~X6Aurora()
{
    delete FSfpPtr;  FSfpPtr=0;
}

//---------------------------------------------------------------------------
//  X6Aurora::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6Aurora::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    if (FSfpType==sfpQSfpPlus)
        {
        QSfpIntf *  QSfp = new QSfpIntf(this);
        QSfp->Init();
        FSfpPtr = QSfp;  // save for later
        }
    else
        {
        SfpIntf *  Sfp = new SfpIntf(this);
        Sfp->Init();
        FSfpPtr = Sfp;
        }

}

//---------------------------------------------------------------------------
//  X6Aurora::OpenHardware()
//---------------------------------------------------------------------------

void  X6Aurora::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing

    if (!SfpEnabled)
        return;
    // SFP setup
}

//---------------------------------------------------------------------------
//  X6Aurora::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6Aurora::ConfigureHardware()
{
	if (!DeviceEnabled())
		return;

	// nothing
}

//---------------------------------------------------------------------------
//  X6Aurora::LogicPresent()
//---------------------------------------------------------------------------

bool  X6Aurora::LogicPresent()
{
	if (!DeviceEnabled())
		return false;

	unsigned int reg = TestControlReg.Value();
	if (reg == 0x0bada55)
		return false;
	else
    	return true;
}

//---------------------------------------------------------------------------
//  X6Aurora::Preset()
//---------------------------------------------------------------------------

void  X6Aurora::Reset()
{
    ControlStatusReg.Value(0);
}

//---------------------------------------------------------------------------
//  X6Aurora::Init()
//---------------------------------------------------------------------------

void  X6Aurora::Init(LoopbackTypes lb)
{
	ControlStatusReg.GtxResetN.Value(false);
	//  Power Up
	ControlStatusReg.PowerDown.Value(true);
	ControlStatusReg.PowerDown.Value(false);
	Sleep(1);
	//  Clear Errors
	ControlStatusReg.ClearError.Value(true);
	ControlStatusReg.ClearError.Value(false);

	//  Set Loopback State
	ControlStatusReg.LoopbackType.Value(static_cast<unsigned int>(lb));
	//  Out of Reset
	ControlStatusReg.GtxResetN.Value(true);
	Sleep(5);
}
//---------------------------------------------------------------------------
//  X6Aurora::StartTestTxMB()
//---------------------------------------------------------------------------

void  X6Aurora::StartTestTxMB()
{
	Debug.Log("Starting Multi-Board TX");
	TestControlReg.EnableTxTest.Value(true);
}

//---------------------------------------------------------------------------
//  X6Aurora::EnableChannels()
//---------------------------------------------------------------------------

void  X6Aurora::EnableChannels()
{
	Debug.Log("Aurora Enable Channels");
	//  Clear Errors
	ControlStatusReg.ClearError.Value(true);
	ControlStatusReg.ClearError.Value(false);

	// enable tx/rx channel
	ControlStatusReg.EnableRx.Value(true);
	ControlStatusReg.EnableTx.Value(true);
}

//---------------------------------------------------------------------------
//  X6Aurora::EnableChannelsRxMB()
//---------------------------------------------------------------------------

void  X6Aurora::EnableChannelsRxMB()
{
	Debug.Log("Starting Multi-Board Enable Channels");
	//  Clear Errors
	ControlStatusReg.ClearError.Value(true);
	ControlStatusReg.ClearError.Value(false);

	// enable tx/rx channel
	ControlStatusReg.EnableRx.Value(true);
//	ControlStatusReg.EnableTx.Value(true);
}

//---------------------------------------------------------------------------
//  X6Aurora::EnableChannelsTxMB()
//---------------------------------------------------------------------------

void  X6Aurora::EnableChannelsTxMB()
{
	Debug.Log("Starting Multi-Board Enable Channels");
	//  Clear Errors
	ControlStatusReg.ClearError.Value(true);
	ControlStatusReg.ClearError.Value(false);

	// enable tx/rx channel
//	ControlStatusReg.EnableRx.Value(true);
	ControlStatusReg.EnableTx.Value(true);
}

//---------------------------------------------------------------------------
//  X6Aurora::Start()
//---------------------------------------------------------------------------

void  X6Aurora::Start()
{
	Debug.Log("Aurora Start");
	// Start Run
	ControlStatusReg.Run.Value(false);

	//  Clear Errors
	ControlStatusReg.ClearError.Value(true);
	ControlStatusReg.ClearError.Value(false);

	// Start Run
	ControlStatusReg.Run.Value(true);

}

//---------------------------------------------------------------------------
//  X6Aurora::End()
//---------------------------------------------------------------------------

void  X6Aurora::End()
{
	//  End Run
	ControlStatusReg.Run.Value(false);
	//  Into Reset
	ControlStatusReg.GtxResetN.Value(false);
	//  Power Down
	ControlStatusReg.PowerDown.Value(true);
}

//---------------------------------------------------------------------------
//  X6Aurora::InitTest()
//---------------------------------------------------------------------------

void  X6Aurora::InitTest(LoopbackTypes lb)
{
	Debug.Log("Init Aurora Test");
	Init(lb);
	// poll pll lock
	Debug.Log("Pll Lock State", PllLocked());
}

//---------------------------------------------------------------------------
//  X6Aurora::InitTestMB()
//---------------------------------------------------------------------------

void  X6Aurora::InitTestMB(LoopbackTypes lb)
{
	Debug.Log("Init Multi-Board Aurora Test");
	Init(lb);
	// poll pll lock
	Debug.Log("Pll Lock State", PllLocked());
}

//---------------------------------------------------------------------------
//  X6Aurora::StartTest()
//---------------------------------------------------------------------------

void  X6Aurora::StartTest()
{
	Debug.Log("Starting Aurora Test");
	// poll channel up
	Debug.Log("Channel Up State", ChannelUp());
	// enable test generator/checker

	TestControlReg.EnableTxTest.Value(true);

	//  Turn on channels
	EnableChannels();

	//  Start Run
	Start();

	TestControlReg.EnableRxTest.Value(true);

	// monitor error counts...by calling separate functions
	Debug.Log("Hard Error", HardError());
	Debug.Log("Soft Error", SoftError());
	Debug.Log("Frame Error", FrameError());
	Debug.Log("ErrorCount", (int)ErrorCount());
}
//---------------------------------------------------------------------------
//  X6Aurora::StartTest()
//---------------------------------------------------------------------------

void  X6Aurora::StartTestSingleBoard()
{
	Debug.Log("Starting Single board Aurora Test");
	// poll channel up
	Debug.Log("Channel Up State", ChannelUp());
	// enable test generator/checker

	TestControlReg.EnableTxTest.Value(true);
	TestControlReg.EnableRxTest.Value(true);

	//  Turn on channels
	EnableChannels();

	//  Start Run
	Start();

	// monitor error counts...by calling separate functions
	Debug.Log("Hard Error", HardError());
	Debug.Log("Soft Error", SoftError());
	Debug.Log("Frame Error", FrameError());
	Debug.Log("ErrorCount", (int)ErrorCount());
}

//---------------------------------------------------------------------------
//  X6Aurora::StartTestMB()    - Multi board start differs from single board loopback
//---------------------------------------------------------------------------

void  X6Aurora::StartTestMB()
{
	Debug.Log("Starting Multi-Board Aurora Test");
	// poll channel up
	Debug.Log("Channel Up State", ChannelUp());
	// enable test generator/checker

	// The starting of transmission (Tx) must be AFTER all boards have initialized
//	TestControlReg.EnableTxTest.Value(true);

	//  Turn on channels
//	EnableChannelsMB();

	//  Start Run
	Start();

//	TestControlReg.EnableRxTest.Value(true);

	// monitor error counts...by calling separate functions
	Debug.Log("Hard Error", HardError());
	Debug.Log("Soft Error", SoftError());
	Debug.Log("Frame Error", FrameError());
	Debug.Log("ErrorCount", (int)ErrorCount());
}

//---------------------------------------------------------------------------
//  X6Aurora::EndTest()
//---------------------------------------------------------------------------

void  X6Aurora::EndTest()
{
	//  Disable test
	TestControlReg.EnableTxTest.Value(false);
	TestControlReg.EnableRxTest.Value(false);
	// Stop Aurora
	End();

	Debug.Log("Ending Aurora Test");
}

//---------------------------------------------------------------------------
//  X6Aurora::EndTestMB()
//---------------------------------------------------------------------------

void  X6Aurora::EndTestMB()
{
	//  Disable test
	TestControlReg.EnableTxTest.Value(false);
	TestControlReg.EnableRxTest.Value(false);
	// Stop Aurora
	End();

	Debug.Log("Ending Multi-Board Aurora Test");
}

//---------------------------------------------------------------------------
//  X6Aurora::StartTestRxMB()
//---------------------------------------------------------------------------

void  X6Aurora::StartTestRxMB()
{
	Debug.Log("Starting Multi-Board RX");
	TestControlReg.EnableRxTest.Value(true);
}

//---------------------------------------------------------------------------
//  X6Aurora::PllLocked() - return lock bit
//---------------------------------------------------------------------------

bool  X6Aurora::PllLocked()
{
    ControlStatusReg.Refresh();
    return ControlStatusReg.PllLocked.Value();
}

//---------------------------------------------------------------------------
//  X6Aurora::ChannelUp() - return ChannelUp bit
//---------------------------------------------------------------------------

bool  X6Aurora::ChannelUp()
{
    ControlStatusReg.Refresh();
	return ControlStatusReg.ChannelUp.Value();
}

//---------------------------------------------------------------------------
//  X6Aurora::HardError() - return HardError bit
//---------------------------------------------------------------------------

bool  X6Aurora::HardError()
{
    ControlStatusReg.Refresh();
    return ControlStatusReg.HardError.Value();
}

//---------------------------------------------------------------------------
//  X6Aurora::SoftError() - return SoftError bit
//---------------------------------------------------------------------------

bool  X6Aurora::SoftError()
{
    ControlStatusReg.Refresh();
    return ControlStatusReg.SoftError.Value();
}

//---------------------------------------------------------------------------
//  X6Aurora::FrameError() - return FrameError bit
//---------------------------------------------------------------------------

bool  X6Aurora::FrameError()
{
    ControlStatusReg.Refresh();
    return ControlStatusReg.FrameError.Value();
}

//---------------------------------------------------------------------------
//  X6Aurora::ErrorCount() - return ErrorCount bit
//---------------------------------------------------------------------------

unsigned int  X6Aurora::ErrorCount()
{
    ControlStatusReg.Refresh();
    return TestControlReg.TestErrorCount.Value();
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpDisable()  -- disable SPF+ device
//---------------------------------------------------------------------------

void  X6Aurora::SfpDisable(unsigned int dev, bool state)
{
    switch(dev)
        {
        default:
        case 0:  SfpPortControlReg.Disable_0.Value(state);  break;
        case 1:  SfpPortControlReg.Disable_1.Value(state);  break;
        case 2:  SfpPortControlReg.Disable_2.Value(state);  break;
        case 3:  SfpPortControlReg.Disable_3.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpDisable()  -- disable SPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::SfpDisable(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortControlReg.Disable_0.Value();
		case 1:  return SfpPortControlReg.Disable_1.Value();
		case 2:  return SfpPortControlReg.Disable_2.Value();
		case 3:  return SfpPortControlReg.Disable_3.Value();
		}

}

//---------------------------------------------------------------------------
//  X6Aurora::SfpHiSpeedRx()  -- set high speed rx on  SPF+ device
//---------------------------------------------------------------------------

void  X6Aurora::SfpHiSpeedRx(unsigned int dev, bool state)
{
    switch(dev)
        {
        default:
        case 0:  SfpPortControlReg.HiSpeedRx_0.Value(state);  break;
        case 1:  SfpPortControlReg.HiSpeedRx_1.Value(state);  break;
        case 2:  SfpPortControlReg.HiSpeedRx_2.Value(state);  break;
        case 3:  SfpPortControlReg.HiSpeedRx_3.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpHiSpeedRx()  -- read hi speed rx on SPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::SfpHiSpeedRx(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortControlReg.HiSpeedRx_0.Value();
		case 1:  return SfpPortControlReg.HiSpeedRx_1.Value();
		case 2:  return SfpPortControlReg.HiSpeedRx_2.Value();
        case 3:  return SfpPortControlReg.HiSpeedRx_3.Value();  
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpHiSpeedTx()  -- set high speed tx on  SPF+ device
//---------------------------------------------------------------------------

void  X6Aurora::SfpHiSpeedTx(unsigned int dev, bool state)
{
    switch(dev)
        {
        default:
        case 0:  SfpPortControlReg.HiSpeedTx_0.Value(state);  break;
        case 1:  SfpPortControlReg.HiSpeedTx_1.Value(state);  break;
        case 2:  SfpPortControlReg.HiSpeedTx_2.Value(state);  break;
        case 3:  SfpPortControlReg.HiSpeedTx_3.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpHiSpeedTx()  -- read hi speed tx on SPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::SfpHiSpeedTx(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortControlReg.HiSpeedTx_0.Value();
		case 1:  return SfpPortControlReg.HiSpeedTx_1.Value();
		case 2:  return SfpPortControlReg.HiSpeedTx_2.Value();
		case 3:  return SfpPortControlReg.HiSpeedTx_3.Value();
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpSignalLoss()  -- SignalLoss detected on SPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::SfpSignalLoss(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortControlReg.SignalLoss_0.Value();
		case 1:  return SfpPortControlReg.SignalLoss_1.Value();
		case 2:  return SfpPortControlReg.SignalLoss_2.Value();
        case 3:  return SfpPortControlReg.SignalLoss_3.Value();  
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpFault()  -- Fault detected on SPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::SfpFault(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortControlReg.Fault_0.Value();
		case 1:  return SfpPortControlReg.Fault_1.Value();
		case 2:  return SfpPortControlReg.Fault_2.Value();
		case 3:  return SfpPortControlReg.Fault_3.Value();
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpNotDetect()  -- transceiver not detected on SPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::SfpNotDetect(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortControlReg.NotDetect_0.Value();
		case 1:  return SfpPortControlReg.NotDetect_1.Value();
		case 2:  return SfpPortControlReg.NotDetect_2.Value();
        case 3:  return SfpPortControlReg.NotDetect_3.Value();  
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpDisable()  -- disable QSPF+ device
//---------------------------------------------------------------------------

void  X6Aurora::QSfpDisable(unsigned int dev, bool state)
{
    switch(dev)
        {
        default:
        case 0:  QSfpPortControlReg.Disable_0.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpDisable()  -- disable QSPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::QSfpDisable(unsigned int dev)
{
    switch(dev)
        {
        default:
        case 0:  return QSfpPortControlReg.Disable_0.Value();
        }

}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpReset()  -- reset mode on QSPF+ device
//---------------------------------------------------------------------------

void  X6Aurora::QSfpReset(unsigned int dev, bool state)
{
    bool state_n = !state;
    switch(dev)
        {
        default:
        case 0:  QSfpPortControlReg.NotReset_0.Value(state_n);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpReset()  -- reset mode on QSPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::QSfpReset(unsigned int dev)
{
    switch(dev)
        {
        default:
        case 0:  return ! QSfpPortControlReg.NotReset_0.Value();  // negative logic
        }

}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpLowPower()  -- power mode on QSPF+ device
//---------------------------------------------------------------------------

void  X6Aurora::QSfpLowPower(unsigned int dev, bool state)
{
    bool state_n = state;
    switch(dev)
        {
        default:
        case 0:  QSfpPortControlReg.HighPower_0.Value(state_n);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpLowPower()  -- power mode on QSPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::QSfpLowPower(unsigned int dev)
{
    switch(dev)
        {
        default:
        case 0:  return QSfpPortControlReg.HighPower_0.Value();  // negative logic
        }

}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpFault()  -- Fault detected on QSPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::QSfpFault(unsigned int dev)
{
    switch(dev)
        {
        default:
        case 0:  return QSfpPortControlReg.NotFault_0.Value()==false;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpNotDetect()  -- transceiver not detected on QSPF+ device
//---------------------------------------------------------------------------

bool  X6Aurora::QSfpNotDetect(unsigned int dev)
{
    switch(dev)
        {
        default:
        case 0:  return QSfpPortControlReg.NotDetect_0.Value();
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpDataOut()  -- QSFP I2C bit access
//--------------------------------------------------------------------------

void  X6Aurora::SfpDataOut(unsigned int dev, bool state)
{
    switch(dev)
        {
        default:
        case 0:  SfpPortI2CReg.DataOut_0.Value(state);  break;
        case 1:  SfpPortI2CReg.DataOut_1.Value(state);  break;
        case 2:  SfpPortI2CReg.DataOut_2.Value(state);  break;
        case 3:  SfpPortI2CReg.DataOut_3.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpDataIn()  -- SFP I2C bit access
//---------------------------------------------------------------------------

bool  X6Aurora::SfpDataIn(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortI2CReg.DataIn_0.Value();
		case 1:  return SfpPortI2CReg.DataIn_1.Value();
		case 2:  return SfpPortI2CReg.DataIn_2.Value();
		case 3:  return SfpPortI2CReg.DataIn_3.Value();
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpClockOut()  -- SFP I2C bit access
//--------------------------------------------------------------------------

void  X6Aurora::SfpClockOut(unsigned int dev, bool state)
{
    switch(dev)
        {
        default:
        case 0:  SfpPortI2CReg.ClockOut_0.Value(state);  break;
        case 1:  SfpPortI2CReg.ClockOut_1.Value(state);  break;
        case 2:  SfpPortI2CReg.ClockOut_2.Value(state);  break;
        case 3:  SfpPortI2CReg.ClockOut_3.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  X6Aurora::SfpClockIn()  -- SFP I2C bit access
//---------------------------------------------------------------------------

bool  X6Aurora::SfpClockIn(unsigned int dev)
{
    switch(dev)
        {
        default:
		case 0:  return SfpPortI2CReg.ClockIn_0.Value();
		case 1:  return SfpPortI2CReg.ClockIn_1.Value();
		case 2:  return SfpPortI2CReg.ClockIn_2.Value();
        case 3:  return SfpPortI2CReg.ClockIn_3.Value();  
        }
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS X6Aurora::SfpIntf  -- SPF interface
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

const int TransI2C_Subaddr = 0xA0;

//---------------------------------------------------------------------------
//  constructor for class X6Aurora::SfpIntf::
//---------------------------------------------------------------------------

X6Aurora::SfpIntf::SfpIntf(X6Aurora * aurora)
	:  Aurora(aurora),
	   Trans_I2C_0(I2CImpl[0], I2CDevice::SubAddress, TransI2C_Subaddr),
	   Trans_I2C_1(I2CImpl[1], I2CDevice::SubAddress, TransI2C_Subaddr),
	   Trans_I2C_2(I2CImpl[2], I2CDevice::SubAddress, TransI2C_Subaddr),
	   Trans_I2C_3(I2CImpl[3], I2CDevice::SubAddress, TransI2C_Subaddr)
	{}

//---------------------------------------------------------------------------
//  X6Aurora::SfpIntf::Init()  -- Set up SFP Transceivers
//---------------------------------------------------------------------------

void  X6Aurora::SfpIntf::Init()
{
	FTrans[0].Init(Aurora, 0);
    FTrans[1].Init(Aurora, 1);
    FTrans[2].Init(Aurora, 2);
    FTrans[3].Init(Aurora, 3);

    I2CImpl[0].Init(Aurora, 0);
    I2CImpl[1].Init(Aurora, 1);
    I2CImpl[2].Init(Aurora, 2);
    I2CImpl[3].Init(Aurora, 3);

}

//---------------------------------------------------------------------------
//  X6Aurora::SfpIntf::Init()  -- retrun SFP Transceiver
//---------------------------------------------------------------------------

IAuroraSfpTransceiver * X6Aurora::SfpIntf::Trans(unsigned int idx)
{
	if (idx>=4)
		idx = 0;

	return &FTrans[idx];

}

//---------------------------------------------------------------------------
//  X6Aurora::SfpIntf::I2CDev()  -- Return I2C Device
//---------------------------------------------------------------------------

I2CDeviceN & X6Aurora::SfpIntf::I2CDev(unsigned int idx)
{
	switch (idx)
		{
		default:
		case 0:   return Trans_I2C_0;
		case 1:   return Trans_I2C_1;
		case 2:   return Trans_I2C_2;
		case 3:   return Trans_I2C_3;
		}
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS X6Aurora::QSfpIntf  -- QSPF interface
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

const int QTransI2C_Subaddr = 0xA0;

//---------------------------------------------------------------------------
//  constructor for class X6Aurora::QSfpIntf
//---------------------------------------------------------------------------

X6Aurora::QSfpIntf::QSfpIntf(X6Aurora * aurora)
	:  Aurora(aurora),
	   Trans_I2C_0(I2CImpl, I2CDevice::SubAddress, TransI2C_Subaddr)
	{}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpIntf::Init()  -- Set up QSFP Transceivers
//---------------------------------------------------------------------------

void  X6Aurora::QSfpIntf::Init()
{
    FTrans.Init(Aurora, 0);
    I2CImpl.Init(Aurora, 0);
}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpIntf::Init()  -- retrun SFP Transceiver
//---------------------------------------------------------------------------

IAuroraSfpTransceiver * X6Aurora::QSfpIntf::Trans(unsigned int /*idx*/)
{

    return &FTrans;

}

//---------------------------------------------------------------------------
//  X6Aurora::QSfpIntf::I2CDev()  -- Return I2C Device
//---------------------------------------------------------------------------

I2CDeviceN & X6Aurora::QSfpIntf::I2CDev(unsigned int idx)
{
	switch (idx)
		{
		default:
		case 0:   return Trans_I2C_0;
		}
}

//===========================================================================
//  CLASS SbcDuo_QSfp -- Separate QSFP Device section "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class SbcDuo_QSfp
//---------------------------------------------------------------------------

SbcDuo_QSfp::SbcDuo_QSfp(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       QSfp_0L_PortControlReg(space, regs.QSfpPortControl_0L_Addr),
       QSfp_0L_PortI2CReg(space, regs.QSfpPortI2C_0L_Addr),
       QSfp_0H_PortControlReg(space, regs.QSfpPortControl_0H_Addr),
       QSfp_0H_PortI2CReg(space, regs.QSfpPortI2C_0H_Addr),
       QSfp_1L_PortControlReg(space, regs.QSfpPortControl_1L_Addr),
       QSfp_1L_PortI2CReg(space, regs.QSfpPortI2C_1L_Addr),
       QSfp_1H_PortControlReg(space, regs.QSfpPortControl_1H_Addr),
       QSfp_1H_PortI2CReg(space, regs.QSfpPortI2C_1H_Addr),
       QSfp_0L(this, 0), QSfp_0H(this, 1), QSfp_1L(this,2), QSfp_1H(this, 3)
{
}

//---------------------------------------------------------------------------
//  destructor for class SbcDuo_QSfp
//---------------------------------------------------------------------------

SbcDuo_QSfp::~SbcDuo_QSfp()
{
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::Sfp() - return SFP device
//---------------------------------------------------------------------------

IAuroraSfp * SbcDuo_QSfp::Sfp(unsigned int device)
{
    switch(device)
        {
        default:
        case 0:  return &QSfp_0L;
        case 1:  return &QSfp_0H;
        case 2:  return &QSfp_1L;
        case 3:  return &QSfp_1H;
        }
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::ConstructCustomParts()
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    QSfp_0L.Init();
    QSfp_0H.Init();
    QSfp_1L.Init();
    QSfp_1H.Init();
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::OpenHardware()
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing

    // SFP setup
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::ConfigureHardware()
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::SfpDataOut()  -- QSFP I2C bit access
//--------------------------------------------------------------------------

void  SbcDuo_QSfp::SfpDataOut(unsigned int dev, unsigned int /*subdev*/, bool state)
{
    switch(dev)
        {
        default:
        case 0:  QSfp_0L_PortI2CReg.DataOut_0.Value(state);  break;
        case 1:  QSfp_0H_PortI2CReg.DataOut_0.Value(state);  break;
        case 2:  QSfp_1L_PortI2CReg.DataOut_0.Value(state);  break;
        case 3:  QSfp_1H_PortI2CReg.DataOut_0.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::SfpDataIn()  -- SFP I2C bit access
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::SfpDataIn(unsigned int dev, unsigned int /*subdev*/)
{
    switch(dev)
        {
        default:
        case 0:  return QSfp_0L_PortI2CReg.DataIn_0.Value();
        case 1:  return QSfp_0H_PortI2CReg.DataIn_0.Value();
        case 2:  return QSfp_1L_PortI2CReg.DataIn_0.Value();
        case 3:  return QSfp_1H_PortI2CReg.DataIn_0.Value();
        }
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::SfpClockOut()  -- SFP I2C bit access
//--------------------------------------------------------------------------

void  SbcDuo_QSfp::SfpClockOut(unsigned int dev, unsigned int /*subdev*/, bool state)
{
    switch(dev)
        {
        default:
        case 0:  QSfp_0L_PortI2CReg.ClockOut_0.Value(state);  break;
        case 1:  QSfp_0H_PortI2CReg.ClockOut_0.Value(state);  break;
        case 2:  QSfp_1L_PortI2CReg.ClockOut_0.Value(state);  break;
        case 3:  QSfp_1H_PortI2CReg.ClockOut_0.Value(state);  break;
        }
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::SfpClockIn()  -- SFP I2C bit access
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::SfpClockIn(unsigned int dev, unsigned int /*subdev*/)
{
    switch(dev)
        {
        default:
        case 0:  return QSfp_0L_PortI2CReg.ClockIn_0.Value();
        case 1:  return QSfp_0H_PortI2CReg.ClockIn_0.Value();
        case 2:  return QSfp_1L_PortI2CReg.ClockIn_0.Value();
        case 3:  return QSfp_1H_PortI2CReg.ClockIn_0.Value();
        }
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfp_PCR()  -- return Port Control Register for a device
//---------------------------------------------------------------------------

SbcDuo_QSfp::QSfpPortControlRegister&  SbcDuo_QSfp::QSfp_PCR(unsigned int dev)
{
    switch(dev)
        {
        default:
        case 0:  return QSfp_0L_PortControlReg;
        case 1:  return QSfp_0H_PortControlReg;
        case 2:  return QSfp_1L_PortControlReg;
        case 3:  return QSfp_1H_PortControlReg;
        }
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpDisable()  -- disable QSPF+ device
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::QSfpDisable(unsigned int dev, unsigned int /*subdev*/, bool state)
{
    QSfp_PCR(dev).Disable_0.Value(state);
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpDisable()  -- disable QSPF+ device
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::QSfpDisable(unsigned int dev, unsigned int /*subdev*/)
{
    return QSfp_PCR(dev).Disable_0.Value();
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpReset()  -- reset mode on QSPF+ device
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::QSfpReset(unsigned int dev, unsigned int /*subdev*/, bool state)
{
    bool state_n = !state;
    QSfp_PCR(dev).NotReset_0.Value(state_n);
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpReset()  -- reset mode on QSPF+ device
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::QSfpReset(unsigned int dev, unsigned int /*subdev*/)
{
    return !  QSfp_PCR(dev).NotReset_0.Value();  // negative logic
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpLowPower()  -- power mode on QSPF+ device
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::QSfpLowPower(unsigned int dev, unsigned int /*subdev*/, bool state)
{
    bool state_n = !state;
    QSfp_PCR(dev).HighPower_0.Value(state_n);
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpLowPower()  -- power mode on QSPF+ device
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::QSfpLowPower(unsigned int dev, unsigned int /*subdev*/)
{
    return ! QSfp_PCR(dev).HighPower_0.Value();  // negative logic
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpFault()  -- Fault detected on QSPF+ device
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::QSfpFault(unsigned int dev, unsigned int /*subdev*/)
{
    return QSfp_PCR(dev).NotFault_0.Value()==false;
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpNotDetect()  -- transceiver not detected on QSPF+ device
//---------------------------------------------------------------------------

bool  SbcDuo_QSfp::QSfpNotDetect(unsigned int dev, unsigned int /*subdev*/)
{
    return QSfp_PCR(dev).NotDetect_0.Value();
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS SbcDuo_QSfp::QSfpIntf  -- QSPF interface
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

const int Duo_QTransI2C_Subaddr = 0xA0;

//---------------------------------------------------------------------------
//  constructor for class SbcDuo_QSfp::QSfpIntf
//---------------------------------------------------------------------------

SbcDuo_QSfp::QSfpIntf::QSfpIntf(SbcDuo_QSfp * owner, int device)
    :  Owner(owner), Device(device),
       Trans_I2C_0(I2CImpl, I2CDevice::SubAddress, Duo_QTransI2C_Subaddr)
    {}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpIntf::Init()  -- Set up QSFP Transceivers
//---------------------------------------------------------------------------

void  SbcDuo_QSfp::QSfpIntf::Init()
{
    FTrans.Init(Owner, Device, 0);
    I2CImpl.Init(Owner, Device, 0);
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpIntf::Init()  -- retrun SFP Transceiver
//---------------------------------------------------------------------------

IAuroraSfpTransceiver * SbcDuo_QSfp::QSfpIntf::Trans(unsigned int /*idx*/)
{
    return &FTrans;
}

//---------------------------------------------------------------------------
//  SbcDuo_QSfp::QSfpIntf::I2CDev()  -- Return I2C Device
//---------------------------------------------------------------------------

I2CDeviceN & SbcDuo_QSfp::QSfpIntf::I2CDev(unsigned int idx)
{
    switch (idx)
        {
        default:
        case 0:   return Trans_I2C_0;
        }
}

//===========================================================================
//  CLASS X6FlashRom -- X6 Flash Rom "SubDevice"  - IDROM and other uses
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6FlashRom
//---------------------------------------------------------------------------

const size_t PageSizeBytes = 4096;

//---------------------------------------------------------------------------
//  constructor for class X6FlashRom
//---------------------------------------------------------------------------

X6FlashRom::X6FlashRom(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AddressControlReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6FlashRom
//---------------------------------------------------------------------------

X6FlashRom::~X6FlashRom()
{
}

//---------------------------------------------------------------------------
//  X6FlashRom::OpenHardware()
//---------------------------------------------------------------------------

void  X6FlashRom::OpenHardware()
{
    if (!DeviceEnabled())
        return;
		
	unsigned int DeviceId = JdecId();
	bool 		 NewChip;
	if (DeviceId == 0xBF2641BF || DeviceId == 0xBF2642BF)
	{
		NewChip = true;
	}
    else 
	{
		NewChip = false;
	}
	
    ReleaseBlockProtection(NewChip);
}

//---------------------------------------------------------------------------
//  X6FlashRom::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6FlashRom::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6FlashRom::ReleaseBlockProtection() -- Remove all block protection
//---------------------------------------------------------------------------

void  X6FlashRom::ReleaseBlockProtection(bool NewChip)
{
    if (!DeviceEnabled())
        return;
		
	
	
	if(NewChip)
	{
        FlashOp(0x06);   // WREN - enable writing
		FlashOp(0x98);  // ULBPR - Global Block Protection Unlock
	}
    else 
	{
		FlashOp(0x50);   // EWSR - enable writing status register
		FlashOp(0x01, 0x00, 0x00);  // WRSR (write status register)
		// ...the above sets BP0-BP3 to 0, removing all protection
	}
	
	
}

//---------------------------------------------------------------------------
//  X6FlashRom::ChipErase() --
//---------------------------------------------------------------------------

void  X6FlashRom::ChipErase()
{
    if (!DeviceEnabled())
        return;

    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0xC7);

    Flash_NotBusy(true);   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  X6FlashRom::ErasePage() -- erase 1 IDROM page
//---------------------------------------------------------------------------

void  X6FlashRom::ErasePage(unsigned int page)
{
    if (!DeviceEnabled())
        return;

    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x20, page*PageSizeBytes);
    
    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  X6FlashRom::ReadPage() -- read 1 IDROM page
//---------------------------------------------------------------------------

void  X6FlashRom::ReadPage(unsigned int page, int * pkt, unsigned int size)
{
    if (!DeviceEnabled())
        return;
    size_t rpg_limit = std::min(PageSizeBytes, size*sizeof(int));

    //  Read one 32 bit word at a time into memory
    for (size_t i=0; i<rpg_limit; i+=4)
        {
        unsigned int addr = static_cast<unsigned int>(page*PageSizeBytes + i);
        unsigned int value = FlashOp(0x03, addr);
        *pkt++ = value;
        }
}

#ifdef _MSC_VER
#pragma warning(disable : 4309)
#endif
//---------------------------------------------------------------------------
//  X6FlashRom::WritePage() -- write 1 IDROM page
//---------------------------------------------------------------------------

void  X6FlashRom::WritePage(unsigned int page, int * pkt, unsigned int size)
{
    if (!DeviceEnabled())
        return;

    size_t wpg_limit = std::min(PageSizeBytes, size*sizeof(int));


    //  Program one 32 bit word at a time into memory (4 accesses needed)
    char * pkt_asChar = reinterpret_cast<char *>(pkt);

	for (size_t i = 0; i < wpg_limit; i += 4)
	{
		unsigned int addr = static_cast<unsigned int>(page*PageSizeBytes + i);

		char byte1 = *pkt_asChar++;
		char byte2 = *pkt_asChar++;  
		char byte3 = *pkt_asChar++;
		char byte4 = *pkt_asChar++;

		const char NotWriteChar = 0xff;

		if (byte1 != NotWriteChar) 
			ByteProgram(addr + 3, byte1);


		if (byte2 != NotWriteChar)
			ByteProgram(addr + 2, byte2);


		if (byte3 != NotWriteChar)
			ByteProgram(addr + 1, byte3);


		if (byte4 != NotWriteChar)
			ByteProgram(addr + 0, byte4);
	}

}

//---------------------------------------------------------------------------
//  X6FlashRom::ByteProgram() -- Flash Operation - Byte Program
//---------------------------------------------------------------------------

void  X6FlashRom::ByteProgram(unsigned int addr, unsigned int byte)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x02, addr, byte );
	Flash_NotBusy();   // wait for chip to report completed Program
}

//---------------------------------------------------------------------------
//  X6FlashRom::Rdsr() -- Flash Operation - Read status register
//---------------------------------------------------------------------------

unsigned int  X6FlashRom::Rdsr()
{
    return FlashOp(0x05, 0);    // address unused
}

//---------------------------------------------------------------------------
//  X6FlashRom::JdecId() -- Flash Instruction - JDEC-ID
//---------------------------------------------------------------------------

unsigned int  X6FlashRom::JdecId()
{
    return FlashOp(0x9F, 0);   // address unused
}

//---------------------------------------------------------------------------
//  X6FlashRom::Rdid() -- Flash Instruction - RDID
//---------------------------------------------------------------------------

unsigned int  X6FlashRom::Rdid()
{
    return FlashOp(0x90, 0);   // address unused
}

//---------------------------------------------------------------------------
//  X6FlashRom::FlashOp() -- primitive Flash Operation - "command-like" ops
//---------------------------------------------------------------------------

void  X6FlashRom::FlashOp(unsigned int opcode)
{
    if (!DeviceEnabled())
        return;

    Flash_Ready();

    // Write opcode to trigger the transaction
    OpcodeReg.Value(opcode);
}

//---------------------------------------------------------------------------
//  X6FlashRom::FlashOp() -- primitive Flash Operation - "read-like" ops
//---------------------------------------------------------------------------

unsigned int  X6FlashRom::FlashOp(unsigned int opcode, unsigned int addr)
{
    if (!DeviceEnabled())
        return 0;

    Flash_Ready();

    // Write address
    AddressControlReg.Addr.Value(addr);

    // Write opcode to trigger the transaction
    OpcodeReg.Value(opcode);

    Flash_ReadValid();

    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  X6FlashRom::FlashOp() -- primitive Flash Operation - "write-like" ops
//---------------------------------------------------------------------------

void  X6FlashRom::FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
{
    if (!DeviceEnabled())
        return;

    Flash_Ready();

    // Write data
    DataReg.Value(data);

    // Write address
    AddressControlReg.Addr.Value(addr);

    // Write opcode to trigger the transaction
    OpcodeReg.Value(opcode);

}

//---------------------------------------------------------------------------
//  X6FlashRom::Flash_Ready() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  X6FlashRom::Flash_Ready()
{
    // Wait up to 5 uS for pending flash to be ready
    for (unsigned int i = 0; i < 5; ++i)
        {
        bool ready = AddressControlReg.Ready.Value();
        if (ready)
            return;
        else
            Innovative::uSleep(10);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Ready Failure (>50us)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  X6FlashRom::Flash_ReadValid() -- wait for flash readvalid indicator
//---------------------------------------------------------------------------

void  X6FlashRom::Flash_ReadValid()
{
    // Wait up to 5 uS for pending flash read to be valid
    for (unsigned int i = 0; i < 5; ++i)
        {
        bool readvalid = AddressControlReg.ReadDataValid.Value();
        if (readvalid)
            return;
        else
            Innovative::uSleep(10);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Read Valid Failure (>50us)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  X6FlashRom::Flash_NotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  X6FlashRom::Flash_NotBusy(bool WaitLonger)
{
    unsigned int WaitPeriod = 5; //MS
    if(WaitLonger)
        WaitPeriod = 8;

    // Wait up to 10*WaitPeriod mS for pending flash read to be validRdsr
    for (unsigned int i = 0; i < 10; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::Sleep(WaitPeriod);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Not Busy Failure  (>" <<  WaitPeriod*10 << "ms)";
    Debug.Log(msg.str());

}

//===========================================================================
//  CLASS X6_GPSIo -- X6 GPS Device "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_GPSIo
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6_GPSIo
//---------------------------------------------------------------------------

X6_GPSIo::X6_GPSIo(IRequires_Gps * mdc, IGPS_Callback * cb,
                        IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), CB(cb), GpsDev(0),
       UartDataReg(space, regs.GpsUartData_Addr),
       UartControlReg(space, regs.GpsUartControl_Addr),
       UartStatusReg(space, regs.GpsUartStatus_Addr),
       GpsStatusReg(space, regs.GpsStatus_Addr),
       GpsControlReg(space, regs.GpsControl_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_GPSIo
//---------------------------------------------------------------------------

X6_GPSIo::~X6_GPSIo()
{
    delete CB;         CB  = 0;
    delete GpsDev;     GpsDev  = 0;
}

//---------------------------------------------------------------------------
//  X6_GPSIo::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6_GPSIo::ConstructCustomParts()
{
    GpsDev = MDC->MakeGpsDevice();
}

//---------------------------------------------------------------------------
//  X6_GPSIo::OpenHardware()
//---------------------------------------------------------------------------

void  X6_GPSIo::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6_GPSIo::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6_GPSIo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  X6_GPSIo::Gps()
//---------------------------------------------------------------------------

IGps &  X6_GPSIo::Gps()
{
    return *GpsDev;
}

//---------------------------------------------------------------------------
//  X6_GPSIo::IsLocked()
//---------------------------------------------------------------------------

bool  X6_GPSIo::IsLocked()
{
    return GpsStatusReg.Locked.Value();
}

//---------------------------------------------------------------------------
//  X6_GPSIo::IsAntenna()
//---------------------------------------------------------------------------

bool  X6_GPSIo::IsAntenna()
{
    return GpsStatusReg.Antenna.Value();
}

//---------------------------------------------------------------------------
//  X6_GPSIo::GpsEnabled() --  Enable/disable GPS subsystem
//---------------------------------------------------------------------------

void X6_GPSIo::GpsEnabled(bool state)
{
    CB->InterruptEnable(false);

    // Reset receive FIFO
    GpsControlReg.FifoReset.Value(1);
    GpsControlReg.FifoReset.Value(state ? 0 : 1);
    Sleep(10);
    GpsControlReg.Enabled.Value(state ? 1 : 0);

    if (state)
        {
        //Gps->Clear();
        CB->InterruptEnable(true);
        GpsDev->Init();
        }
}

//---------------------------------------------------------------------------
//  X6_GPSIo::GpsEnabled() --  Query state of GPS subsystem
//---------------------------------------------------------------------------

bool X6_GPSIo::GpsEnabled() const
{
    return GpsControlReg.Enabled.Value() ? true : false;
}

//---------------------------------------------------------------------------
//  X6_GPSIo::ArmGpsTrigger() --  Fire trigger at next epoch
//---------------------------------------------------------------------------

void X6_GPSIo::ArmGpsTrigger()
{
    GpsControlReg.Arm.Value(1);   // Produce an edge, then turn it off
    GpsControlReg.Arm.Value(0);   //   needed since this bit is in shared reg.
}

//---------------------------------------------------------------------------
//  X6_GPSIo::UartData() --
//---------------------------------------------------------------------------

void X6_GPSIo::UartData(char c)
{
    while (!UartStatusReg.TxReady.Value())
        Innovative::Sleep(1);

    UartDataReg.Value(c);
}

//---------------------------------------------------------------------------
//  X6_GPSIo::UartData() --
//---------------------------------------------------------------------------

char X6_GPSIo::UartData() const
{
    char c = static_cast<char>(UartDataReg.Value());
    return c;
}

//---------------------------------------------------------------------------
//  X6_GPSIo::Baud() -- set UART Baud.
//---------------------------------------------------------------------------

void X6_GPSIo::Baud(IGps::BaudRates rate)
{
    UartControlReg.Value(rate);
}

//---------------------------------------------------------------------------
//  X6_GPSIo::Baud() -- read UART baud
//---------------------------------------------------------------------------

IGps::BaudRates X6_GPSIo::Baud() const
{
    return static_cast<IGps::BaudRates>(UartControlReg.Value());
}

//---------------------------------------------------------------------------
//  X6_GPSIo::UartFifoDepth()
//---------------------------------------------------------------------------

unsigned int  X6_GPSIo::UartFifoDepth()
{
    return UartStatusReg.RxDepth.Value();
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


}   // end namespace Innovative


