//---------------------------------------------------------------------------
// X6SubDevice_Misc_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "Miscellaneous" Subdevices
//---------------------------------------------------------------------------

#ifndef X6SubDevice_Misc_MbH
#define X6SubDevice_Misc_MbH

#include <X6Support_Mb.h>
#include <PciLogicVersionInfo_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <DioPortIntf_Mb.h>
#include <Si5338_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6Info -- X6 Information "SubDevice"
//===========================================================================

class X6Info : public IX6SubDevice, public IX6LogicVersionInfo
{
public:
    //=======================================================================
    //  CLASS X6Info::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Info_Addr;
        unsigned int   BitstreamDate_Addr;
        unsigned int   Subrev_Addr;
        unsigned int   Bar0Version_Addr;
    };

    enum PciVersionRegStyle { styleXU };

    //  Normal (X6, K7) configuration
    X6Info(IRequires_Nothing * mdc, IAddressingSpace & space,
                                    IAddressingSpace & bar_0space, MapRegisters & regs, bool bitstream_date_valid=false);
    //  XU stream configuration
    X6Info(IRequires_Nothing * mdc, IAddressingSpace & space,
                                    IAddressingSpace & bar_0space, MapRegisters & regs, PciVersionRegStyle style);

    virtual ~X6Info();

    virtual short           PciLogicRevision() const;
    virtual short           PciLogicFamily() const;    // Board Family (X6, K7, XU)

    //X6 and K7sys
    virtual short           FpgaLogicVersion() const;
    virtual short           FpgaHardwareVariant() const;    
    virtual short           PciLogicPcb() const;       // Board Revision
    virtual short           PciLogicType() const;      // Board Type
    virtual short           FpgaChipType() const;
    virtual short           FpgaLogicSubrevision() const;
    virtual short           BoardRevisionBuiltFor() const;
    virtual unsigned int    PciLogicRawValue() const;
    virtual unsigned int    FpgaInformationRawValue() const;
    virtual unsigned int    FpgaSubRevRawValue() const;

    //k7 special
    virtual short           K7FpgaLogicVersion() const;
    virtual short           K7FpgaHardwareVariant() const;
    virtual short           K7FpgaChipType() const;
    virtual short           K7BoardHwType() const;


    virtual struct tm       BitStreamDate() const;
    virtual std::string     BitStreamDateString() const;
    virtual std::string     BitStreamDateString_YMD() const;
    virtual std::string     BitStreamDateString_HMS() const;

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PciLogicVersionRegister  --  X6 Module User Logic ID info reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class PciLogicVersionRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        enum RegStyle { styleXU };

        ReadOnlyRegisterBitGroup       Revision;
        ReadOnlyRegisterBitGroup       Family;

        //  Normal (X6, K7) configuration
        PciLogicVersionRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Revision(*this, 0, 16), Family(*this, 16, 4)
                    {  }
        //  XU stream configuration
        PciLogicVersionRegister(IAddressingSpace & space, unsigned int offset, RegStyle)
            : inherited(space, offset),
              Revision(*this, 12, 4), Family(*this, 16, 4)
                    {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FpgaLogicVersionRegister  --  X6 Module User Logic ID info reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class FpgaLogicVersionRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        enum XURegStyle { styleXU };
        enum K7RegStyle { styleK7 };

        ReadOnlyRegisterBitGroup       Version;
        ReadOnlyRegisterBitGroup       HdwVariant;
        ReadOnlyRegisterBitGroup       BoardHwRev;
        ReadOnlyRegisterBitGroup       BoardHwType;
        ReadOnlyRegisterBitGroup       FpgaType;

        //X6 and K7Sys mapping
        FpgaLogicVersionRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Version(*this, 0, 16), HdwVariant(*this, 16, 4),
              BoardHwRev(*this, 20, 4), BoardHwType(*this, 24, 4),
              FpgaType(*this, 28, 4)
                    {  }
        //K7
        FpgaLogicVersionRegister(IAddressingSpace & space, unsigned int offset, K7RegStyle)
            : inherited(space, offset),
             /*revision*/ Version(*this, 0, 16), /*cfg*/ HdwVariant(*this, 16, 8),
              BoardHwRev(*this, 0, 0), BoardHwType(*this, 24, 4),
              FpgaType(*this, 28, 2)
                    {  }
        //  XU stream configuration
        FpgaLogicVersionRegister(IAddressingSpace & space, unsigned int offset, XURegStyle)
            : inherited(space, offset),
              Version(*this, 0, 12), HdwVariant(*this, 16, 8),
              BoardHwRev(*this, 12, 4), BoardHwType(*this, 24, 4),
              FpgaType(*this, 28, 2)
                    {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS FpgaLogicSubRevRegister  --  X6 Module User Logic ID info reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class FpgaLogicSubRevRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup       BoardRevBuiltFor;
        ReadOnlyRegisterBitGroup       SubRevision;

        FpgaLogicSubRevRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset), BoardRevBuiltFor(*this, 0, 4),
              SubRevision(*this, 24, 8)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS BitStreamDateRegister  --  Logic creation date register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class BitStreamDateRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBitGroup       Second;
        ReadOnlyRegisterBitGroup       Minute;
        ReadOnlyRegisterBitGroup       Hour;
        ReadOnlyRegisterBitGroup       Year2K;
        ReadOnlyRegisterBitGroup       Month;
        ReadOnlyRegisterBitGroup       Day;

        BitStreamDateRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Second(*this, 0, 6),   Minute(*this, 6, 6),
              Hour  (*this, 12, 5),  Year2K(*this, 17, 6),
              Month (*this, 23, 4),  Day(*this, 27, 5)
        {  }
    };

private:
    IRequires_Nothing *  MDC;
    PciLogicVersionRegister    PciLogicVersionReg;
    FpgaLogicVersionRegister   FpgaLogicVersionReg;
    FpgaLogicVersionRegister   K7FpgaLogicVersionReg;
    FpgaLogicSubRevRegister    FpgaLogicSubRevReg;
    BitStreamDateRegister      BitStreamDateReg;

    bool  BitStreamDateValid;

};

//===========================================================================
//  CLASS X6Peripherals -- X6 Peripheral section "SubDevice"
//===========================================================================

class X6Peripherals : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS X6Peripherals::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   ResetReg_Addr;
    };

    X6Peripherals(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6Peripherals();

    //  Methods
    bool                        GreenLed();
    void                        GreenLed(bool value);

    bool                        FrontPanelLed();
    void                        FrontPanelLed(bool value);

    void                        AppReset();
    void                        FmcCRun(bool state);
    void                        XURun(bool state);
    void                        XARun(bool state);


    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Peripherals::ResetNLedRegister  --  X6 Module User Logic ID info reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ResetNLedRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        FpgaReset;
        RegisterBit        GreenLed;
        RegisterBit        FrontPanelLed;
        RegisterBit        FmcCarrierRun;

        ResetNLedRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              FpgaReset(*this, 0), GreenLed(*this, 1), FrontPanelLed(*this, 2),
              FmcCarrierRun(*this, 4)
        {  }
    };

private:
    IRequires_Nothing *   MDC;
    ResetNLedRegister     ResetNLedReg;

};

//===========================================================================
//  CLASS X6SioPll -- X6 Sio Pll Periph section "SubDevice"
//===========================================================================

class X6SioPll : public IX6SubDevice, public IX6SioPllControl
{
public:
    //=======================================================================
    //  CLASS X6SioPll::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   SioPll_Addr;
    };

    X6SioPll(IRequires_SioPll * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6SioPll();

    //  Methods
    virtual Si5338 &  Pll()  {  return *SioPllDev;  }

    virtual void  ClockOutputEnable(bool state);
    virtual bool  ClockOutputEnable();

    virtual void  ExternalReference(bool state);
    virtual bool  ExternalReference();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6SioPll::SioPllI2CRegister  -- Sio Pll I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SioPll_I2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Data_Out;
        RegisterBit  Clock_Out;
        RegisterBit  Data_In;
        RegisterBit  Clock_In;
        RegisterBit  SioPll_Int;
        RegisterBit  SioPll_OutputEnable;
        RegisterBit  SioPll_ExtRefSelect;

    public:
        SioPll_I2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data_Out(*this, 0),   Clock_Out(*this, 1),
               Data_In(*this, 2),    Clock_In(*this, 3),
               SioPll_Int(*this, 4), SioPll_OutputEnable(*this, 8),
               SioPll_ExtRefSelect(*this, 9)
            {
            }
    };

private:
    IRequires_SioPll *   MDC;
    SioPll_I2CRegister   SioPll_I2CReg;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SioPll_I2C_Impl  -- SioPll I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SioPll_I2C_Impl : public I2CDeviceIntf
    {
    public:
        SioPll_I2C_Impl(SioPll_I2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~SioPll_I2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        SioPll_I2CRegister &  Reg;
    };
    //
    //  Interface Impls
    SioPll_I2C_Impl     SioPllIntf;
public:
    I2CDeviceN          SioPll_I2C;
private:
    Si5338 *            SioPllDev;

};

//===========================================================================
//  CLASS X6FpaLoader -- X6 FPA Loader "SubDevice"
//===========================================================================

class X6FpaLoader : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS X6FpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   FpaAddress_Addr;
        unsigned int   FpaData_Addr;
        unsigned int   FpaCommand_Addr;
    };

    //=======================================================================
    //  CLASS X6FpaLoader::X6FpaCpld  -- CPLD Hardware Interface
    //=======================================================================
    class X6FpaCpld : public IX6FpaCpld
    {
    public:
        X6FpaCpld(X6FpaLoader * owner): Owner(owner)
            {  }
        virtual ~X6FpaCpld()
            {  }

        // Methods
        virtual void  CPLD_Write(unsigned int addr, unsigned int data)
                {  Owner->CPLD_Write(addr, data);  }
        virtual unsigned int  CPLD_Read(unsigned int addr)
                {  return  Owner->CPLD_Read(addr);  }
        virtual void  CPLD_GoldenImageActive(bool state)
                {  Owner->CPLD_GoldenImageActive(state);  }
        virtual void  CPLD_UpdateStatus()
                {  Owner->CPLD_UpdateStatus();  }
        virtual unsigned int  CPLD_ReadPowerStatus()
                {  return Owner->CPLD_ReadPowerStatus();  }

        virtual void  CPLD_Address(unsigned int addr)
                {  Owner->CPLD_Address(addr);  }
        virtual void  CPLD_Data(unsigned int data)
                {  Owner->CPLD_Data(data);  }
        virtual unsigned int  CPLD_Data()
                {  return Owner->CPLD_Data();  }
        virtual void  CPLD_Command(unsigned int code)
                {  Owner->CPLD_Command(code);  }

        virtual unsigned int  CPLD_Version()
                {  return Owner->CPLD_Version();  }
        virtual unsigned int  CPLD_FlashType()
                {  return Owner->CPLD_FlashType();  }
        virtual bool  CPLD_GoldenImageActive()
                {  return Owner->CPLD_GoldenImageActive();  }
        virtual bool  CPLD_GoldenImageBooted()
                {  return Owner->CPLD_GoldenImageBooted();  }
        virtual bool  CPLD_Busy()
                {  return Owner->CPLD_Busy();  }
        virtual bool  CPLD_ReadValid()
                {  return Owner->CPLD_ReadValid();  }
        virtual bool  CPLD_FlashReady()
                {  return Owner->CPLD_FlashReady();  }
        virtual bool  CPLD_OverTemperature()
                {  return Owner->CPLD_OverTemperature();  }
    private:
        X6FpaLoader *  Owner;
    };

    //
    //  Member Functions
    X6FpaLoader(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6FpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    //  CPLD methods
    //  Operations
    void          CPLD_Write(unsigned int addr, unsigned int data);
    unsigned int  CPLD_Read(unsigned int addr);
    void          CPLD_GoldenImageActive(bool state);
    void          CPLD_RestartConfig(bool restart_golden_image=false);
    void          CPLD_UpdateStatus();
    unsigned int  CPLD_ReadPowerStatus();

    //  Primitives
    void          CPLD_Address(unsigned int addr);
    void          CPLD_Data(unsigned int data);
    unsigned int  CPLD_Data();
    void          CPLD_Command(unsigned int code);

    unsigned int  CPLD_Version();
    unsigned int  CPLD_FlashType();
    bool          CPLD_GoldenImageActive();
    bool          CPLD_GoldenImageBooted();
    bool          CPLD_Busy();
    bool          CPLD_ReadValid();
    bool          CPLD_FlashReady();
    bool          CPLD_OverTemperature();

private:
    //  CPLD Private Impl Methods
    enum  CpldCommands { cmdFlashWrite=0x01, cmdFlashRead=0x02,  cmdSwapImage=0x04,
                         cmdAddrIncr=0x08,   cmdReadStatus=0x10, cmdRestartCfg=0x20,
                         cmdPowerStatus=0x40 };

    void  CPLD_BusyWait();
    void  CPLD_ReadValidWait();
    void  CPLD_FlashReadyWait();

public:

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6FpaLoader::FpaAddressRegister  -- Address Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FpaAddressRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Addr;

    public:
        FpaAddressRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Addr(*this, 0, 27)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6FpaLoader::FpaCommandRegister  -- Command Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FpaCommandRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup            Command;
        ReadOnlyRegisterBit         GoldenImageInUse;
        ReadOnlyRegisterBit         Busy;
        ReadOnlyRegisterBit         ReadValid;
        ReadOnlyRegisterBit         FlashReady;
        ReadOnlyRegisterBit         OverTemp;
        ReadOnlyRegisterBit         GoldenImageBooted;
        ReadOnlyRegisterBitGroup    FlashType;
        ReadOnlyRegisterBitGroup    CPLDVersion;

    public:
        FpaCommandRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Command(*this, 0, 7),   GoldenImageInUse(*this, 15),
               Busy(*this, 16),        ReadValid(*this, 17),
               FlashReady(*this, 18),  OverTemp(*this, 19),
               GoldenImageBooted(*this, 23),  FlashType(*this, 24, 4),
               CPLDVersion(*this, 28, 4)
            {
            }
    };

private:
    IRequires_Nothing *  MDC;
    FpaAddressRegister  FpaAddress;
    Register            FpaData;
    FpaCommandRegister  FpaCommand;

    bool                FGoldenImageMode;

    X6FpaCpld           CpldIntf;
    FpaLoadThread       Thread;
    StopWatch           Rtc;

    void  RefreshStatus();
};

//===========================================================================
//  CLASS X6DigitalIo -- X6 Digital Io "SubDevice"
//===========================================================================

class X6DigitalIo : public IX6SubDevice, public AdvancedDioPortIntf
{
public:
    //=======================================================================
    //  CLASS X6DigitalIo::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   DioDataLo_Addr;
        unsigned int   DioDataHi_Addr;
        unsigned int   DioOutputEnableLo_Addr;
        unsigned int   DioOutputEnableHi_Addr;
    };

    X6DigitalIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6DigitalIo();

    virtual IBitManipulator &   DioPortData();

    virtual void 			    DioPortConfig(unsigned int);
    virtual unsigned int 	    DioPortConfig() const;

    virtual IBitManipulator &   DioPortDataHigh();

    virtual void 			    DioPortConfigHigh(unsigned int mask);
    virtual unsigned int 	    DioPortConfigHigh() const;

    virtual void                DioPortOutputEnable(int output_bit, bool output_enabled);
    virtual bool                DioPortOutputEnable(int output_bit) const;

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_Nothing *  MDC;
    Register  DataLo;
    Register  DataHi;
    Register  OutEnLo;
    Register  OutEnHi;

    RegisterBitManipulator  DataLoManip;
    RegisterBitManipulator  DataHiManip;

};

//===========================================================================
//  CLASS X6Thermal -- X6 Tempearature Control "SubDevice"
//===========================================================================

class X6Thermal : public IX6SubDevice, public IX6ThermalDevice
{
public:
    //=======================================================================
    //  CLASS X6Thermal::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   TempControl_Addr;
        unsigned int   FanThresholdTemp_Addr;
        unsigned int   AlertThresholdTemp_Addr;
        unsigned int   CommandChannel_Addr;
    };

    X6Thermal(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6Thermal();

    virtual int             LogicTemperature() const;

    virtual int 			LogicFanSetpoint() const;
    virtual void 			LogicFanSetpoint(int temp_deg_C);

    virtual int 			LogicAlertSetpoint() const;
    virtual void 			LogicAlertSetpoint(int temp_deg_C);

    //  Control & Status methods
    virtual bool            MonitorEnable() const;
    virtual void            MonitorEnable(bool state);

    virtual bool            CommFault() const;
    virtual bool            SensorInReset() const;
    virtual bool            SensorFault() const;

    virtual int             Rpm();

    virtual void            Port(int addr, int value);
    virtual int             Port(int addr);

    //  Reserved UI
    virtual void            LogicCriticalSetpoint(int temp_deg_C);

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Thermal::TemperatureControlRegister  -- Temp CSR Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TemperatureControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit                 MonitorEnable;
        RegisterBit                 CommFault;
        RegisterBit                 SensorInReset;
        RegisterBit                 SensorFault;
        RegisterBitGroup            Temperature;

    public:
        TemperatureControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               MonitorEnable(*this, 0),
               CommFault(*this, 5),
               SensorInReset(*this, 6),
               SensorFault(*this, 7),
               Temperature(*this, 8, 8)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Thermal::TemperatureCfgRegister  -- Temp Cfg Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TemperatureCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup            Temperature;

    public:
        TemperatureCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Temperature(*this, 0, 8)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Thermal::TemperatureIoRegisterW  -- Command Io Register (out)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TemperatureIoRegisterW : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup            Addr;
        RegisterBitGroup            WriteData;
        RegisterBit                 RWDirection;

    public:
        TemperatureIoRegisterW( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Addr(*this, 0, 8),        WriteData(*this, 8, 8),
               RWDirection(*this, 16)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Thermal::TemperatureIoRegisterR  -- Command Io Register (in)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TemperatureIoRegisterR : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit                 Ready;
        ReadOnlyRegisterBit                 ReadValid;
        ReadOnlyRegisterBitGroup            ReadData;

    public:
        TemperatureIoRegisterR( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Ready(*this, 22),
               ReadValid(*this, 23),     ReadData(*this, 24, 8)
            {
            }
    };

private:
    // Data
    IRequires_Nothing *  MDC;
    TemperatureControlRegister  TempControlReg;
    TemperatureCfgRegister      FanThresholdTempReg;
    TemperatureCfgRegister      AlertThresholdTempReg;
    TemperatureIoRegisterW      TempCommandReg;
    TemperatureIoRegisterR      TempCommandReadReg;

    bool  ManualTemperatureMode;

public:
    // Methods
    int             ToDeviceTemp(float temp_deg_C, int width=8) const;
    float           FromDeviceTemp(int temp_deg_C, int width=8) const;
    float           TempScalefactor(int width) const;

    int             TempSignExtend(unsigned int temp_8bit) const;

//    unsigned int DumpTCR()
//        { return TempControlReg.Value();  }

};

//===========================================================================
//  CLASS X6Aurora -- X6 Aurora Device section "SubDevice"
//===========================================================================

class X6Aurora : public IX6SubDevice, public IX6Aurora
{
public:

    enum SfpType { sfpSfpPlus, sfpQSfpPlus };

    //=======================================================================
    //  CLASS X6Aurora::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   TestControl_Addr;
        unsigned int   ControlStatus_Addr;
        unsigned int   WritePort_Addr;
        unsigned int   ReadPort_Addr;
        unsigned int   SfpPortControl_Addr;
        unsigned int   SfpPortI2C_Addr;
    };

    X6Aurora(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs, SfpType=sfpSfpPlus);
    virtual ~X6Aurora();

    //  Methods
    void  SetSfpType(SfpType type)
        {  FSfpType = type;  }

    virtual bool  LogicPresent();

    virtual void  Reset();
    virtual void  Init(LoopbackTypes lb);
    virtual void  Start();
    virtual void  End();

    virtual void  InitTest(LoopbackTypes lb);
    virtual void  StartTest();
    virtual void  StartTestSingleBoard();
    virtual void  EndTest();
    virtual void  EnableChannels();

    virtual void  InitTestMB(LoopbackTypes lb);
    virtual void  StartTestMB();
    virtual void  EndTestMB();
	virtual void  EnableChannelsRxMB();
	virtual void  EnableChannelsTxMB();
	virtual void  StartTestTxMB();
	virtual void  StartTestRxMB();

    virtual bool         PllLocked();
    virtual bool         ChannelUp();
    virtual bool         HardError();
    virtual bool         SoftError();
    virtual bool         FrameError();
    virtual unsigned int ErrorCount();

    virtual IAuroraSfp * Sfp() {  return FSfpPtr;  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::TransceiverIntf  -- SFP Transceiver Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class TransceiverIntf : public IAuroraSfpTransceiver
    {
    public:
        TransceiverIntf()
            :  Aurora(0), SubDev(0)
            {}

        void  Init(X6Aurora * tr, int subdev)
            {  Aurora = tr;  SubDev=subdev;  }

        virtual void  Enabled(bool state)
            {  bool inv = !state; Aurora->SfpDisable(SubDev, inv);  }
        virtual bool  Enabled()
            {  return Aurora->SfpDisable(SubDev)==false;  }
        virtual void  Reset(bool /*state*/)
            {  /* not supported */  }
        virtual bool  Reset()
            {  return false;  }

        virtual void  HiSpeedRx(bool state)
            {  Aurora->SfpHiSpeedRx(SubDev, state);  }
        virtual bool  HiSpeedRx()
            {  return Aurora->SfpHiSpeedRx(SubDev);  }
        virtual void  HiSpeedTx(bool state)
            {  Aurora->SfpHiSpeedTx(SubDev, state);  }
        virtual bool  HiSpeedTx()
            {  return Aurora->SfpHiSpeedTx(SubDev);  }

        virtual void  LowPower(bool /*state*/)
            {  /* not supported */    }
        virtual bool  LowPower()
            {  return false;  }

        bool  NoSignal()
            {  return Aurora->SfpSignalLoss(SubDev);  }
        bool  Fault()
            {  return Aurora->SfpFault(SubDev);  }
        bool  NoTransceiver()
            {  return Aurora->SfpNotDetect(SubDev);  }

    private:
        X6Aurora * Aurora;
        int        SubDev;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS TransI2C_Impl  -- SPF tranceiver interface to "subdevices"
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TransI2C_Impl : public I2CDeviceIntf
    {
    public:
        TransI2C_Impl()
            :  Aurora(0), SubDev(0)
            {}
        virtual ~TransI2C_Impl()
            {}
        // Methods
        void  Init(X6Aurora * tr, int subdev)
            {  Aurora = tr;  SubDev=subdev;  }

        virtual void    SetClock(int state)
            {  Aurora->SfpClockOut(SubDev, state ? true : false);  }
        virtual int     GetClock() const
            {  return Aurora->SfpClockIn(SubDev);  }
        virtual void    SetData(int state)
            {  Aurora->SfpDataOut(SubDev, state ? true : false);  }
        virtual int     GetData() const
            {  return Aurora->SfpDataIn(SubDev);  }

    private:
        X6Aurora * Aurora;
        int        SubDev;
    };


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::SfpIntf  -- SPF interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SfpIntf : public IAuroraSfp
    {
    public:
        SfpIntf(X6Aurora * aurora);
        void  Init();

        // interface
        virtual bool Enabled()
            {  return Aurora->Sfp_SystemEnabled();  }
        virtual unsigned int TransCount()
            {  return 4;  }

        IAuroraSfpTransceiver * Trans(unsigned int idx);
        I2CDeviceN &            I2CDev(unsigned int idx);

    private:
        X6Aurora *       Aurora;
        TransceiverIntf  FTrans[4];
        TransI2C_Impl    I2CImpl[4];
        I2CDeviceN       Trans_I2C_0;
        I2CDeviceN       Trans_I2C_1;
        I2CDeviceN       Trans_I2C_2;
        I2CDeviceN       Trans_I2C_3;
    };

    //  Sfp Primitives
    bool         Sfp_SystemEnabled()
                        {  return SfpEnabled;  }
    void         Sfp_SystemEnabled(bool state)
                        {  SfpEnabled = state;  }

    bool         SfpDisable(unsigned int dev);
    void         SfpDisable(unsigned int dev, bool state);

    bool         SfpHiSpeedRx(unsigned int dev);
    void         SfpHiSpeedRx(unsigned int dev, bool state);

    bool         SfpHiSpeedTx(unsigned int dev);
    void         SfpHiSpeedTx(unsigned int dev, bool state);

    bool         SfpSignalLoss(unsigned int dev);

    bool         SfpFault(unsigned int dev);

    bool         SfpNotDetect(unsigned int dev);
    //
    //  Spf I2C
    bool         SfpDataIn(unsigned int dev);
    void         SfpDataOut(unsigned int dev, bool state);
    bool         SfpClockIn(unsigned int dev);
    void         SfpClockOut(unsigned int dev, bool state);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::QTransceiverIntf  -- QSFP Transceiver Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QTransceiverIntf : public IAuroraSfpTransceiver
    {
    public:
        QTransceiverIntf()
            :  Aurora(0), SubDev(0)
            {}

        void  Init(X6Aurora * tr, int subdev)
            {  Aurora = tr;  SubDev=subdev;  }

        virtual void  Enabled(bool state)
            {  bool inv = !state; Aurora->QSfpDisable(SubDev, inv);  }
        virtual bool  Enabled()
            {  return Aurora->QSfpDisable(SubDev)==false;  }
        virtual void  Reset(bool state)
            {  Aurora->QSfpReset(SubDev, state);  }
        virtual bool  Reset()
            {  return Aurora->QSfpReset(SubDev);  }

        virtual void  HiSpeedRx(bool /*state*/)
            {  /* not supported */  }
        virtual bool  HiSpeedRx()
            {  return false;  }
        virtual void  HiSpeedTx(bool /*state*/)
            {  /* not supported */  }
        virtual bool  HiSpeedTx()
            {  return false;  }

        virtual void  LowPower(bool state)
            {   Aurora->QSfpLowPower(SubDev, state);    }
        virtual bool  LowPower()
            {  return Aurora->QSfpLowPower(SubDev);  }

        bool  NoSignal()
            {  return false;  }
        bool  Fault()
            {  return Aurora->QSfpFault(SubDev);  }
        bool  NoTransceiver()
            {  return Aurora->QSfpNotDetect(SubDev);  }

    private:
        X6Aurora * Aurora;
        int        SubDev;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::QSfpIntf  -- SPF interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QSfpIntf : public IAuroraSfp
    {
    public:
        QSfpIntf(X6Aurora * aurora);
        void  Init();

        // interface
        virtual bool Enabled()
            {  return Aurora->Sfp_SystemEnabled();  }
        virtual unsigned int TransCount()
            {  return 1;  }

        IAuroraSfpTransceiver * Trans(unsigned int idx=0);
        I2CDeviceN &            I2CDev(unsigned int idx=0);

    private:
        X6Aurora *        Aurora;
        QTransceiverIntf  FTrans;
        TransI2C_Impl     I2CImpl;
        I2CDeviceN        Trans_I2C_0;
    };

    //  QSfp Primitives
    bool         QSfpDisable(unsigned int dev);
    void         QSfpDisable(unsigned int dev, bool state);

    bool         QSfpReset(unsigned int dev);
    void         QSfpReset(unsigned int dev, bool state);

    bool         QSfpLowPower(unsigned int dev);
    void         QSfpLowPower(unsigned int dev, bool state);

    bool         QSfpFault(unsigned int dev);

    bool         QSfpNotDetect(unsigned int dev);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::TestControlRegister  --  Aurora Test Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class TestControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        EnableTxTest;
        RegisterBit        EnableRxTest;
        RegisterBitGroup   TestErrorCount;

        TestControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              EnableTxTest(*this, 0), EnableRxTest(*this, 1),
              TestErrorCount(*this, 16, 16)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::ControlStatusRegister  -- Aurora Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ControlStatusRegister : public RefreshableShadowRegister
    {
        typedef RefreshableShadowRegister inherited;

    public:
        RegisterBit  GtxResetN;
        RegisterBit  PowerDown;
        RegisterBit  Run;
        RegisterBitGroup  LoopbackType;
        RegisterBit  ClearError;
        RegisterBit  EnableTx;
        RegisterBit  EnableRx;
        RegisterBit  HardError;
        RegisterBit  SoftError;
        RegisterBit  FrameError;
        RegisterBitGroup  LaneUp;
        RegisterBit  ChannelUp;
        RegisterBit  PllLocked;

    public:
        ControlStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               GtxResetN(*this, 0),   PowerDown(*this, 1),
               Run(*this, 2), LoopbackType(*this, 3, 3),
               ClearError(*this, 6),
               EnableTx(*this, 7), EnableRx(*this, 8),
               HardError(*this, 23),  SoftError(*this, 24),
               FrameError(*this, 25),
               LaneUp(*this, 26, 4),  ChannelUp(*this, 30),
               PllLocked(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::PortRegister  -- Aurora Read/Write port reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class PortRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Data;
        RegisterBitGroup  Addr;
        RegisterBit       ReadWrite;
        RegisterBit       ReadyValid;

    public:
        PortRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data(*this, 0, 24),
               Addr(*this, 24, 6),
               ReadWrite(*this, 30),
               ReadyValid(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::SfpPortControlRegister  -- SFP Port CSR (SFP+)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SfpPortControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Disable_0;
        RegisterBit       Disable_1;
        RegisterBit       Disable_2;
        RegisterBit       Disable_3;
        RegisterBit       HiSpeedRx_0;
        RegisterBit       HiSpeedTx_0;
        RegisterBit       HiSpeedRx_1;
        RegisterBit       HiSpeedTx_1;
        RegisterBit       HiSpeedRx_2;
        RegisterBit       HiSpeedTx_2;
        RegisterBit       HiSpeedRx_3;
        RegisterBit       HiSpeedTx_3;
        RegisterBit       SignalLoss_0;
        RegisterBit       SignalLoss_1;
        RegisterBit       SignalLoss_2;
        RegisterBit       SignalLoss_3;
        RegisterBit       Fault_0;
        RegisterBit       Fault_1;
        RegisterBit       Fault_2;
        RegisterBit       Fault_3;
        RegisterBit       NotDetect_0;
        RegisterBit       NotDetect_1;
        RegisterBit       NotDetect_2;
        RegisterBit       NotDetect_3;

    public:
        SfpPortControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Disable_0(*this, 0),     Disable_1(*this, 1),     Disable_2(*this, 2),     Disable_3(*this, 3),
               HiSpeedRx_0(*this, 4),   HiSpeedTx_0(*this, 5),   HiSpeedRx_1(*this, 6),   HiSpeedTx_1(*this, 7),
               HiSpeedRx_2(*this, 8),   HiSpeedTx_2(*this, 9),   HiSpeedRx_3(*this, 10),  HiSpeedTx_3(*this, 11),
               SignalLoss_0(*this, 12), SignalLoss_1(*this, 13), SignalLoss_2(*this, 14), SignalLoss_3(*this, 15),
               Fault_0(*this, 16),      Fault_1(*this, 17),      Fault_2(*this, 18),      Fault_3(*this, 19),
               NotDetect_0(*this, 20),  NotDetect_1(*this, 21),  NotDetect_2(*this, 22),  NotDetect_3(*this, 23)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::QSfpPortControlRegister  -- SFP Port CSR (QSFP+)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QSfpPortControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Disable_0;
        RegisterBit       NotReset_0;
        RegisterBit       HighPower_0;
        RegisterBit       NotFault_0;
        RegisterBit       NotDetect_0;

    public:
        QSfpPortControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Disable_0(*this, 0),  NotReset_0(*this, 4),   HighPower_0(*this, 12),
               NotFault_0(*this, 16),  NotDetect_0(*this, 20)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6Aurora::SfpPortI2CRegister  -- SFP I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SfpPortI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   DataOut_0;
        RegisterBit   DataOut_1;
        RegisterBit   DataOut_2;
        RegisterBit   DataOut_3;
        RegisterBit   ClockOut_0;
        RegisterBit   ClockOut_1;
        RegisterBit   ClockOut_2;
        RegisterBit   ClockOut_3;
        RegisterBit   DataIn_0;
        RegisterBit   DataIn_1;
        RegisterBit   DataIn_2;
        RegisterBit   DataIn_3;
        RegisterBit   ClockIn_0;
        RegisterBit   ClockIn_1;
        RegisterBit   ClockIn_2;
        RegisterBit   ClockIn_3;

    public:
        SfpPortI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DataOut_0(*this, 0),  DataOut_1(*this, 1),  DataOut_2(*this, 2),  DataOut_3(*this, 3),
               ClockOut_0(*this, 4), ClockOut_1(*this, 5), ClockOut_2(*this, 6), ClockOut_3(*this, 7),
               DataIn_0(*this, 8),   DataIn_1(*this, 9),   DataIn_2(*this, 10),  DataIn_3(*this, 11),
               ClockIn_0(*this, 12), ClockIn_1(*this, 13), ClockIn_2(*this, 14), ClockIn_3(*this, 15)
            {
            }
    };


private:
    IRequires_Nothing *      MDC;
    TestControlRegister      TestControlReg;
    ControlStatusRegister    ControlStatusReg;
    PortRegister             WritePortReg;
    PortRegister             ReadPortRegReg;
    bool                     SfpEnabled;
    SfpPortControlRegister   SfpPortControlReg;
    SfpPortI2CRegister       SfpPortI2CReg;
    QSfpPortControlRegister  QSfpPortControlReg;

    //SfpIntf                  FSfp;
    //QSfpIntf                 FQSfp;
    IAuroraSfp *             FSfpPtr;
    SfpType                  FSfpType;
};

//===========================================================================
//  CLASS SbcDuo_QSfp -- Separate QSFP Device section "SubDevice"
//===========================================================================

class SbcDuo_QSfp : public IX6SubDevice
{
public:

    //=======================================================================
    //  CLASS SbcDuo_QSfp::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   QSfpPortControl_0L_Addr;
        unsigned int   QSfpPortI2C_0L_Addr;
        unsigned int   QSfpPortControl_0H_Addr;
        unsigned int   QSfpPortI2C_0H_Addr;
        unsigned int   QSfpPortControl_1L_Addr;
        unsigned int   QSfpPortI2C_1L_Addr;
        unsigned int   QSfpPortControl_1H_Addr;
        unsigned int   QSfpPortI2C_1H_Addr;
    };

    SbcDuo_QSfp(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~SbcDuo_QSfp();

	//  Methods
    virtual IAuroraSfp * Sfp(unsigned int idx=0);

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuo_QSfp::TransI2C_Impl  -- SPF tranceiver interface to "subdevices"
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	class TransI2C_Impl : public I2CDeviceIntf
	{
	public:
		TransI2C_Impl()
            :  Owner(0), Dev(0), SubDev(0)
			{}
		virtual ~TransI2C_Impl()
			{}
		// Methods
        void  Init(SbcDuo_QSfp * tr, int dev, int subdev)
            {  Owner = tr;  Dev=dev; SubDev=subdev;  }

		virtual void    SetClock(int state)
            {  Owner->SfpClockOut(Dev, SubDev, state ? true : false);  }
		virtual int     GetClock() const
            {  return Owner->SfpClockIn(Dev, SubDev);  }
		virtual void    SetData(int state)
            {  Owner->SfpDataOut(Dev, SubDev, state ? true : false);  }
		virtual int     GetData() const
            {  return Owner->SfpDataIn(Dev, SubDev);  }

    private:
        SbcDuo_QSfp * Owner;
        int           Dev;
        int           SubDev;
    };

    //
    //  Spf I2C
    bool         SfpDataIn(unsigned int dev, unsigned int subdev);
    void         SfpDataOut(unsigned int dev, unsigned int subdev, bool state);
    bool         SfpClockIn(unsigned int dev, unsigned int subdev);
    void         SfpClockOut(unsigned int dev, unsigned int subdev, bool state);

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuo_QSfp::QTransceiverIntf  -- QSFP Transceiver Interface
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class QTransceiverIntf : public IAuroraSfpTransceiver
	{
	public:
		QTransceiverIntf()
            :  Owner(0), Dev(0), SubDev(0)
			{}

        void  Init(SbcDuo_QSfp * tr, int dev, int subdev)
            {  Owner = tr; Dev=dev; SubDev=subdev;  }

        virtual void  Enabled(bool state)
            {  bool inv = !state; Owner->QSfpDisable(Dev, SubDev, inv);  }
        virtual bool  Enabled()
            {  return Owner->QSfpDisable(Dev, SubDev)==false;  }
        virtual void  Reset(bool state)
            {  Owner->QSfpReset(Dev, SubDev, state);  }
        virtual bool  Reset()
            {  return Owner->QSfpReset(Dev, SubDev);  }

        virtual void  HiSpeedRx(bool /*state*/)
            {  /* not supported */  }
        virtual bool  HiSpeedRx()
            {  return false;  }
        virtual void  HiSpeedTx(bool /*state*/)
            {  /* not supported */  }
        virtual bool  HiSpeedTx()
            {  return false;  }

        virtual void  LowPower(bool state)
            {   Owner->QSfpLowPower(Dev, SubDev, state);    }
        virtual bool  LowPower()
            {  return Owner->QSfpLowPower(Dev, SubDev);  }

        bool  NoSignal()
            {  return false;  }
        bool  Fault()
            {  return Owner->QSfpFault(Dev, SubDev);  }
        bool  NoTransceiver()
            {  return Owner->QSfpNotDetect(Dev, SubDev);  }

    private:
        SbcDuo_QSfp * Owner;
        int           Dev;
        int           SubDev;
    };

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuo_QSfp::QSfpIntf  -- SPF interface
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	class QSfpIntf : public IAuroraSfp
	{
	public:
        QSfpIntf(SbcDuo_QSfp * owner, int device);
		void  Init();

        // interface
        virtual bool Enabled()
            {  return true;  }
        virtual unsigned int TransCount()
            {  return 1;  }

        IAuroraSfpTransceiver * Trans(unsigned int idx=0);
        I2CDeviceN &            I2CDev(unsigned int idx=0);

    private:
        SbcDuo_QSfp *     Owner;
        int               Device;
        QTransceiverIntf  FTrans;
        TransI2C_Impl     I2CImpl;
        I2CDeviceN        Trans_I2C_0;
    };

    //  QSfp Primitives
    bool         QSfpDisable(unsigned int dev, unsigned int subdev);
    void         QSfpDisable(unsigned int dev, unsigned int subdev, bool state);

    bool         QSfpReset(unsigned int dev, unsigned int subdev);
    void         QSfpReset(unsigned int dev, unsigned int subdev, bool state);

    bool         QSfpLowPower(unsigned int dev, unsigned int subdev);
    void         QSfpLowPower(unsigned int dev, unsigned int subdev, bool state);

    bool         QSfpFault(unsigned int dev, unsigned int subdev);

    bool         QSfpNotDetect(unsigned int dev, unsigned int subdev);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuo_QSfp::QSfpPortControlRegister  -- SFP Port CSR (QSFP+)
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class QSfpPortControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Disable_0;
        RegisterBit       NotReset_0;
        RegisterBit       HighPower_0;
        RegisterBit       NotFault_0;
        RegisterBit       NotDetect_0;

    public:
        QSfpPortControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Disable_0(*this, 0),  NotReset_0(*this, 4),   HighPower_0(*this, 12),
               NotFault_0(*this, 16),  NotDetect_0(*this, 20)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuo_QSfp::SfpPortI2CRegister  -- SFP I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SfpPortI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit   DataOut_0;
        RegisterBit   ClockOut_0;
        RegisterBit   DataIn_0;
        RegisterBit   ClockIn_0;

    public:
        SfpPortI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DataOut_0(*this, 0),
               ClockOut_0(*this, 4),
               DataIn_0(*this, 8),
               ClockIn_0(*this, 12)
            {
            }
    };


private:
    IRequires_Nothing *      MDC;
    QSfpPortControlRegister  QSfp_0L_PortControlReg;
    SfpPortI2CRegister       QSfp_0L_PortI2CReg;
    QSfpPortControlRegister  QSfp_0H_PortControlReg;
    SfpPortI2CRegister       QSfp_0H_PortI2CReg;
    QSfpPortControlRegister  QSfp_1L_PortControlReg;
    SfpPortI2CRegister       QSfp_1L_PortI2CReg;
    QSfpPortControlRegister  QSfp_1H_PortControlReg;
    SfpPortI2CRegister       QSfp_1H_PortI2CReg;

    QSfpIntf                 QSfp_0L;
    QSfpIntf                 QSfp_0H;
    QSfpIntf                 QSfp_1L;
    QSfpIntf                 QSfp_1H;

    //  Utility Ftns
    QSfpPortControlRegister&  QSfp_PCR(unsigned int dev);

};

//===========================================================================
//  CLASS X6FlashRom -- X6 Flash Rom "SubDevice"  - IDROM and other uses
//===========================================================================

class X6FlashRom : public IX6SubDevice, public IIdRomDevice
{
public:
    //=======================================================================
    //  CLASS X6FlashRom::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    X6FlashRom(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6FlashRom();

    //  Methods
    //
    //  IIdRomDevice implementation:  Idrom Page Operations
    virtual void   ChipErase();
    virtual void   ErasePage(unsigned int page);
    virtual void   ReadPage(unsigned int page, int * pkt, unsigned int size);
    virtual void   WritePage(unsigned int page, int * pkt, unsigned int size);

    unsigned int   Rdsr();

    //  ROM Info Commands
    virtual unsigned int  JdecId();
    virtual unsigned int  Rdid();

    //---------------------------
    //
    //  Flash Primitives
    void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data);   // write operation
    unsigned int  FlashOp(unsigned int opcode, unsigned int addr);  // read op
    void          FlashOp(unsigned int opcode);  // command op

    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6FlashRom::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *  MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    //
    //  Internal Flash Primitives
    void  Flash_Ready();
    void  Flash_ReadValid();
    void  Flash_NotBusy(bool WaitLonger = false);

    void  ReleaseBlockProtection(bool NewChip);
public:
    void  ByteProgram(unsigned int addr, unsigned int byte);
};

//===========================================================================
//  CLASS X6_GPSIo -- X6 GPS Device "SubDevice"
//===========================================================================

class X6_GPSIo : public IX6SubDevice, public IX6_GpsIo
{
public:
    //=======================================================================
    //  CLASS X6_GPSIo::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   GpsUartData_Addr;
        unsigned int   GpsUartControl_Addr;
        unsigned int   GpsUartStatus_Addr;
        unsigned int   GpsStatus_Addr;
        unsigned int   GpsControl_Addr;
    };

    X6_GPSIo(IRequires_Gps * mdc, IGPS_Callback * cb,
                    IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6_GPSIo();

    //  Methods
    virtual IGps &   Gps();

    //  Implementation Methods
    //  ...Status
    virtual bool            IsLocked();
    virtual bool            IsAntenna();
    //  ...Gps
    virtual void             GpsEnabled(bool state);
    virtual bool             GpsEnabled() const;

    virtual void             ArmGpsTrigger();
    //  ...Uart
    void             UartData(char c);
    char             UartData() const;

    void             Baud(IGps::BaudRates rate);
    IGps::BaudRates  Baud() const;

    unsigned int     UartFifoDepth();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_GPSIo::GpsStatusRegister  --  GPS Status
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class GpsStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       Locked;
        RegisterBit       Antenna;

    public:
        GpsStatusRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Locked(*this, 1), Antenna(*this, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_GPSIo::GpsControlRegister  -- GPS Control reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class GpsControlRegister : public ShadowRegister
    {
        typedef ShadowRegister inherited;

    public:
        RegisterBit       Enabled;
        RegisterBit       FifoReset;
        RegisterBit       Arm;

    public:
        GpsControlRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Enabled(*this, 0), FifoReset(*this, 3), Arm(*this, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_GPSIo::UartStatusRegister  -- GPS UART status
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class UartStatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit       TxReady;
        ReadOnlyRegisterBit       RxReady;
        ReadOnlyRegisterBitGroup  RxDepth;

    public:
        UartStatusRegister(IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               TxReady(*this, 3), RxReady(*this, 4),
               RxDepth(*this, 5, 11)
            {
            }
    };

private:
    IRequires_Gps *  MDC;
    IGPS_Callback *  CB;
    IGps *           GpsDev;
    Register                            UartDataReg;
    ShadowRegister                      UartControlReg;
    UartStatusRegister                  UartStatusReg;
    GpsStatusRegister                   GpsStatusReg;
    GpsControlRegister                  GpsControlReg;

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

};    // end namespace Innovative

//---------------------------------------------------------------------------
#endif
