//---------------------------------------------------------------------------
// X6SubDevice_Stream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "Stream" Subdevices
//---------------------------------------------------------------------------

#include <X6SubDevice_Stream_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <BufferDatagrams_Mb.h>

//---------------------------------------------------------------------------

namespace Innovative {

#ifdef _MSC_VER
#pragma warning(push)
#pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS X6PacketizerCfg -- X6 Packetizer "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6PacketizerCfg
//---------------------------------------------------------------------------

const int X6_MaxInputStreams = X6_MemoryMap::mmPktzer_TotalVeloHdr;  // Max PIDs allowed

//---------------------------------------------------------------------------
//  constructor for class X6PacketizerCfg
//---------------------------------------------------------------------------

X6PacketizerCfg::X6PacketizerCfg(IRequires_VeloInCount * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       PacketizerEnableReg(space, regs.PacketizerEnable_Addr),
       AuxHeaderReg(space, regs.AuxHeader_Addr),
       VelociaInHeaderArray(space, regs.VelociaInHeader_Addr, X6_MaxInputStreams),
       PacketizerForceSizeReg(space, regs.PacketizerForceSize_Addr),
       PacketIntervalReg(space, regs.PacketizerIntervalPeriod_Addr),
       FInStreamCount(0) {
}

//---------------------------------------------------------------------------
//  destructor for class X6PacketizerCfg
//---------------------------------------------------------------------------

X6PacketizerCfg::~X6PacketizerCfg() {
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6PacketizerCfg::ConstructCustomParts() {
    FInStreamCount = MDC->GetVeloInStreamCount();
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::OpenHardware()
//---------------------------------------------------------------------------

void  X6PacketizerCfg::OpenHardware() {
    if (!DeviceEnabled())
        return;

    // Auto-enable packetizers
    for (unsigned int ch=0; ch<FInStreamCount; ch++) {
        Enable(ch, true);           // all streams on
        VeloPacketId(ch, ch);
        VeloDataSize(ch, 0x2000);
    }

    // Set other parameters
    LoadAuxHeaderValue(0x0);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6PacketizerCfg::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::Custom_Fft_VeloPacketConfig() -
//---------------------------------------------------------------------------

void X6PacketizerCfg::Custom_Fft_VeloPacketConfig(unsigned int size) {
    // Add more channels
    Resize(VeloChannels() + size);

    for (unsigned int i = 0; i < size; i++) {
        // Set pids
        VeloPacketId(2*i + 2, 0x40 + 2*i + 0);
        // Set packet size
        VeloDataSize(2*i + 2, 16448);
        // Force packet size
        ForceVeloPacketSize(2*i + 2, true);
        // Enable
        Enable(2*i + 2, true);
    }
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::Custom_Tls_VeloPacketConfig() -
//---------------------------------------------------------------------------

void X6PacketizerCfg::Custom_Tls_VeloPacketConfig(unsigned int size) {
    // Add more channels
    Resize(VeloChannels() + size);

    for (unsigned int i = 0; i < size; i++) {
        // Set pids
        VeloPacketId(2*i + 3, 0x40 + 2*i + 1);
        // Set packet size
        VeloDataSize(2*i + 3, 520);
        // Force packet size
        ForceVeloPacketSize(2*i + 3, true);
        // Enable
        Enable(2*i + 3, true);
    }
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::Enable() - enable Packetizer Streams
//---------------------------------------------------------------------------

void  X6PacketizerCfg::Enable(unsigned int velo_ch, bool state) {
    if (velo_ch < FInStreamCount)
        PacketizerEnableReg.Bit(velo_ch, state);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::VeloPacketId() - Set Packet ID
//---------------------------------------------------------------------------

void  X6PacketizerCfg::VeloPacketId(unsigned int velo_ch, unsigned char pid) {
    if (velo_ch < FInStreamCount)
        VelociaInHeaderArray[velo_ch].Id.Value(pid);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::VeloPacketId() - Read Packet ID
//---------------------------------------------------------------------------

unsigned char  X6PacketizerCfg::VeloPacketId(unsigned int velo_ch) {
    if (velo_ch < FInStreamCount)
        return VelociaInHeaderArray[velo_ch].Id.Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::ForceVeloPacketSize() - Enable Forced Packet Size for all
//---------------------------------------------------------------------------

void  X6PacketizerCfg::ForceVeloPacketSize(bool enable) {
    PacketizerForceSizeReg.Enable.Value(enable ? 0xff : 0x0);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::ForceVeloPacketSize() - Read to see Forced Size is enabled
//---------------------------------------------------------------------------

bool  X6PacketizerCfg::ForceVeloPacketSize() {
    return (PacketizerForceSizeReg.Enable.Value() == 0xff);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::ForceVeloPacketSize() - Enable Forced Packet Size for all
//---------------------------------------------------------------------------

void  X6PacketizerCfg::ForceVeloPacketSize(unsigned int velo_ch, bool enable) {
    PacketizerForceSizeReg.Enable.Bit(velo_ch, enable);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::ForceVeloPacketSize() - Read to see Forced Size is enabled
//---------------------------------------------------------------------------

bool  X6PacketizerCfg::ForceVeloPacketSize(unsigned int velo_ch) {
    return PacketizerForceSizeReg.Enable.Bit(velo_ch);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::VeloPacketSize() - Set Packet Size
//---------------------------------------------------------------------------

void  X6PacketizerCfg::VeloPacketSize(unsigned int velo_ch, unsigned int full_size_ints) {
    if (velo_ch < FInStreamCount)
        VelociaInHeaderArray[velo_ch].Size.Value(full_size_ints);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::VeloPacketSize() - Read Packet Size
//---------------------------------------------------------------------------

unsigned int  X6PacketizerCfg::VeloPacketSize(unsigned int velo_ch) {
    if (velo_ch < FInStreamCount)
        return VelociaInHeaderArray[velo_ch].Size.Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::VeloDataSize() - Set Packet Data Size
//---------------------------------------------------------------------------

void  X6PacketizerCfg::VeloDataSize(unsigned int velo_ch, unsigned int data_size_ints) {
    VeloBufferManager  VBM;
    unsigned int packet_size = data_size_ints + VBM.HeaderSizeInts();
    VeloPacketSize(velo_ch, packet_size);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::VeloDataSize() - Read Packet Data Size
//---------------------------------------------------------------------------

unsigned int  X6PacketizerCfg::VeloDataSize(unsigned int velo_ch) {
    VeloBufferManager  VBM;
    unsigned int packet_size =  VeloPacketSize(velo_ch);
    //  Convert to data size and return...
    return packet_size - VBM.HeaderSizeInts();
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::LoadAll_VeloPacketSize() - Load all velo packet sizes
//---------------------------------------------------------------------------

void  X6PacketizerCfg::LoadAll_VeloPacketSize(unsigned int full_size_ints) {
    for (unsigned int ch=0; ch<FInStreamCount; ch++)
        VeloPacketSize(ch, full_size_ints);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::LoadAll_VeloDataSize() - Load all velo data sizes
//---------------------------------------------------------------------------

void  X6PacketizerCfg:: LoadAll_VeloDataSize(unsigned int data_size_ints) {
    for (unsigned int ch=0; ch<FInStreamCount; ch++)
        VeloDataSize(ch, data_size_ints);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::PacketIntervalPeriod() - Set interval period
//---------------------------------------------------------------------------

void  X6PacketizerCfg::PacketIntervalPeriod(float period_ms) {
    const unsigned int ticks_per_ms = 250000;
    unsigned int ticks = static_cast<unsigned int>(period_ms * ticks_per_ms);
    PacketIntervalReg.Period.Value(ticks);
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::PacketIntervalPeriod() - Return interval period
//---------------------------------------------------------------------------

float  X6PacketizerCfg::PacketIntervalPeriod() {
    unsigned int ticks =  PacketIntervalReg.Period.Value();
    const float ticks_per_ms = 250000;
    return static_cast<float>(ticks) / ticks_per_ms;
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::SetStreamCount() - Change stream count
//---------------------------------------------------------------------------

void  X6PacketizerCfg::SetStreamCount(unsigned int new_count) {
    if (new_count <= static_cast<unsigned int>(X6_MaxInputStreams)) {
        FInStreamCount = new_count;
        for (unsigned int ch=0; ch<FInStreamCount; ch++)
            Enable(ch, true);           // all streams on
    }
}

//---------------------------------------------------------------------------
//  X6PacketizerCfg::Resize() - Change stream count
//---------------------------------------------------------------------------

void  X6PacketizerCfg::Resize(unsigned int new_count) {
    if (new_count <= static_cast<unsigned int>(X6_MaxInputStreams)) {
        FInStreamCount = new_count;
    }
}

//===========================================================================
//  CLASS X6DepacketizerCfg -- X6 Depacketizer "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6DepacketizerCfg
//---------------------------------------------------------------------------

const int X6_MaxOutputStreams = X6_MemoryMap::mmDepktzer_TotalVeloHdr;  // Max PIDs allowed

//---------------------------------------------------------------------------
//  constructor for class X6DepacketizerCfg
//---------------------------------------------------------------------------

X6DepacketizerCfg::X6DepacketizerCfg(IRequires_VeloOutCount * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TestLoopbackPidReg(space, regs.TestLoopbackPid_Addr),
       VelociaOutHeaderArray(space, regs.OutgoingDataPid_Addr, X6_MaxOutputStreams),
       FOutStreamCount(0) {
}

//---------------------------------------------------------------------------
//  destructor for class X6DepacketizerCfg
//---------------------------------------------------------------------------

X6DepacketizerCfg::~X6DepacketizerCfg() {
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::ConstructCustomParts() {
    FOutDataCount = MDC->GetVeloOutStreamCount();
    FOutCmdCount = MDC->GetVeloOutCommandStreamCount();

    FOutStreamCount = FOutDataCount + FOutCmdCount;
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::OpenHardware()
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::OpenHardware() {
    if (!DeviceEnabled())
        return;

    TestLoopbackVeloPacketId(0x1b);   // Ha ha get it! LB!! ...

    // Auto-enable packetizers
    for (unsigned int ch=0; ch<FOutDataCount; ch++)
        VeloPacketId(ch, ch);
    for (unsigned int ch=0; ch<FOutCmdCount; ch++) {
        unsigned int velo_ch = FOutDataCount + ch;
        VeloPacketId(velo_ch, 0x80 + ch);
    }
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::TestLoopbackVeloPacketId() - Set Packet ID
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::TestLoopbackVeloPacketId(unsigned char pid) {
    TestLoopbackPidReg.Id.Value(pid);
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::TestLoopbackVeloPacketId() - Read Packet ID
//---------------------------------------------------------------------------

unsigned char  X6DepacketizerCfg::TestLoopbackVeloPacketId() {
    return TestLoopbackPidReg.Id.Value();
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::VeloPacketId() - Set Packet ID
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::VeloPacketId(unsigned int velo_ch, unsigned char pid) {
    if (velo_ch < FOutStreamCount)
        VelociaOutHeaderArray[velo_ch].Id.Value(pid);
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::VeloPacketId() - Read Packet ID
//---------------------------------------------------------------------------

unsigned char  X6DepacketizerCfg::VeloPacketId(unsigned int velo_ch) {
    if (velo_ch < FOutStreamCount)
        return VelociaOutHeaderArray[velo_ch].Id.Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::SetStreamCount() - Change stream count
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::SetStreamCount(unsigned int new_count) {
    if (new_count <= static_cast<unsigned int>(X6_MaxOutputStreams)) {
        FOutStreamCount = new_count;
        FOutDataCount = FOutStreamCount - FOutCmdCount;
    }
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::SetDataStreamCount() - Change stream count
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::SetDataStreamCount(unsigned int new_count) {
    if (new_count <= static_cast<unsigned int>(X6_MaxOutputStreams)) {
        FOutDataCount = new_count;
        FOutStreamCount = FOutDataCount + FOutCmdCount;
    }
}

//---------------------------------------------------------------------------
//  X6DepacketizerCfg::SetCommandStreamCount() - Change stream count
//---------------------------------------------------------------------------

void  X6DepacketizerCfg::SetCommandStreamCount(unsigned int new_count) {
    if (new_count <= static_cast<unsigned int>(X6_MaxOutputStreams)) {
        FOutCmdCount = new_count;
        FOutStreamCount = FOutDataCount + FOutCmdCount;
    }
}

//===========================================================================
//  CLASS X6AlertSystem -- X6 Alert "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AlertSystem
//---------------------------------------------------------------------------

const int X6AlertPid = 0xff;

//---------------------------------------------------------------------------
//  constructor for class X6AlertSystem
//---------------------------------------------------------------------------

X6AlertSystem::X6AlertSystem(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  AlertSystem(X6AlertPid), MDC(mdc),
       AlertEnableReg(space, regs.AlertEnable_Addr),
       AlertControlReg(space, regs.AlertControl_Addr),
       SoftwareAlertReg(space, regs.SoftwareAlert_Addr),
       AlertClearReg(space, regs.AlertClear_Addr),
       FRolloverCount(0) {
    InitAlerts();
}

//---------------------------------------------------------------------------
//  destructor for class X6AlertSystem
//---------------------------------------------------------------------------

X6AlertSystem::~X6AlertSystem() {
}

//---------------------------------------------------------------------------
//  X6AlertSystem::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AlertSystem::ConstructCustomParts() {
}

//---------------------------------------------------------------------------
//  X6AlertSystem::OpenHardware()
//---------------------------------------------------------------------------

void  X6AlertSystem::OpenHardware() {
    if (!DeviceEnabled())
        return;

    Open();
}

//---------------------------------------------------------------------------
//  X6AlertSystem::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AlertSystem::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    Configure();
}

//---------------------------------------------------------------------------
//  X6AlertSystem::InitAlerts() - Initialize Alert Database
//---------------------------------------------------------------------------

void  X6AlertSystem::InitAlerts() {
    SenderTag(reinterpret_cast<size_t>(this));

    // Load Alerts
    LoadAlert(AlertData((int)alertTimeStampRollover,    0, 1 << 0,  false, &OnTimeStampRolloverAlert));
    LoadAlert(AlertData((int)alertSoftware,             1, 1 << 1,  false, &OnSoftwareAlert));
    LoadAlert(AlertData((int)alertWarningTemperature,   3, 1 << 3, false, &OnWarningTemperature));

    LoadAlert(AlertData((int)alertPatternDone,         19, 1 << 19, false, &OnPatternDoneAlert));
    LoadAlert(AlertData((int)alertInputOverrange,      20, 1 << 20, false, &OnInputOverrange));
    LoadAlert(AlertData((int)alertInputOverflow,       21, 1 << 21, false, &OnInputOverflow));

    LoadAlert(AlertData((int)alertNumericError,       23, 1 << 23, false, &OnNumericError));
    LoadAlert(AlertData((int)alertTrigger,             24, 1 << 24, false, &OnTrigger));
    LoadAlert(AlertData((int)alertOutputOverrange,     25, 1 << 25, false, &OnOutputOverrange));
    LoadAlert(AlertData((int)alertOutputUnderflow,     26, 1 << 26, false, &OnOutputUnderflow));
    LoadAlert(AlertData((int)alertVFifoDiagnostic,     27, 1 << 27, false, &OnVFifoDiagnostic));
}

//---------------------------------------------------------------------------
//  X6AlertSystem::Open() - Open-Time behavior
//---------------------------------------------------------------------------

void  X6AlertSystem::Open() {
    // nothing special
}

//---------------------------------------------------------------------------
//  X6AlertSystem::Configure() - Configure-Time behavior
//---------------------------------------------------------------------------

void  X6AlertSystem::Configure() {
    AlertReset();
    //
    //  Turn on Timestamp Clock
    AlertControlReg.TimestampOn.Set();

    // Apply enabled state for all supported alerts
    for (unsigned int i=0; i<Alerts.size(); ++i)
        AlertEnableReg.Bit(Alerts[i].EnableBit, Alerts[i].Enabled);
}

//---------------------------------------------------------------------------
//  X6AlertSystem::AlertReset() - Reset Alert Fifo
//---------------------------------------------------------------------------

void  X6AlertSystem::AlertReset() {
    AlertControlReg.AlertFifoReset.Set();
    AlertControlReg.AlertFifoReset.Clear();
    FRolloverCount = 0;
}

//---------------------------------------------------------------------------
//  X6AlertSystem::Append() - Add a custom user alert
//---------------------------------------------------------------------------

void X6AlertSystem::Append(const AlertData & alert) {
    LoadAlert(alert);
}

//------------------------------------------------------------------------------
//   X6AlertSystem::ExtractAlertArgument() -- Return alert argument
//------------------------------------------------------------------------------
//   Crack the argument out of the raw alert

int  X6AlertSystem::ExtractAlertArgument(const Buffer & alert, int bit, int field) {
    const int alert_arg_offset = 4;
    int result;
    const IntegerDG  alertDG(alert);

    switch (bit) {
    case 0:     // TimeStampRollover
        result = ++FRolloverCount;
        break;
    case 1:     // Software
        result = alertDG[bit+alert_arg_offset];
        break;
    case 3:     //TemperatureWarning
        result = alertDG[bit+alert_arg_offset];
        break;
    case 19:     // PatternDone
        result = alertDG[bit+alert_arg_offset];
        break;
    case 20:     // InputOverrange
        result = alertDG[bit+alert_arg_offset];
        break;
    case 21:     // InputOverflow
        result = alertDG[bit+alert_arg_offset];
        break;
    case 23:    // NumericError
        result = alertDG[bit+alert_arg_offset];
        break;
    case 24:     // Trigger
        result = alertDG[bit+alert_arg_offset];
        break;
    case 25:    // OutputOverrange
        result = alertDG[bit+alert_arg_offset];
        break;
    case 26:    // OutputUnderflow
        result = alertDG[bit+alert_arg_offset];
        break;
    case 27:     // VFifoDiagnostic
        result = alertDG[bit+alert_arg_offset];
        break;
    default:    // Spurious, unknown
        result = alertDG[field+alert_arg_offset];
        break;
    }

    return result;
}

//------------------------------------------------------------------------------
//   X6AlertSystem::SoftwareAlert() -- trigger software alert
//------------------------------------------------------------------------------

void  X6AlertSystem::SoftwareAlert(unsigned int tag) {
    SoftwareAlertReg.Value(tag);
}

//------------------------------------------------------------------------------
//   X6AlertSystem::ApplyAlertClearHdw() -- Set Alert Clear bit in software
//------------------------------------------------------------------------------

void  X6AlertSystem::ApplyAlertClearHdw(unsigned int bit) {
    AlertClearReg.Bit(bit, true);
    AlertClearReg.Bit(bit, false);
}

//===========================================================================
//  CLASS X6_Std_I2CLinkage -- Standard bit-banged I2C link "microDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_Std_I2CLinkage
//---------------------------------------------------------------------------

const int     VcxoI2C_SubAddr  = 0xAA;

//---------------------------------------------------------------------------
//  constructor for class X6_Std_I2CLinkage
//---------------------------------------------------------------------------

X6_Std_I2CLinkage::X6_Std_I2CLinkage(IAddressingSpace & space, MapRegisters & regs)
    :   VcxoControl(space, regs.AfeVcxoControl_Addr),
        VcxoIntf(VcxoControl),
        Vcxo_I2C(VcxoIntf, I2CDevice::SubAddress, VcxoI2C_SubAddr) {
}

//---------------------------------------------------------------------------
//  destructor for class X6_Std_I2CLinkage
//---------------------------------------------------------------------------

X6_Std_I2CLinkage::~X6_Std_I2CLinkage() {
}

//---------------------------------------------------------------------------
//  X6_Std_I2CLinkage::DeviceEnable() - enable vcxo
//---------------------------------------------------------------------------

void  X6_Std_I2CLinkage::DeviceEnable(bool enabled) {
    VcxoControl.Vco_Enable.Value(enabled);
}

//===========================================================================
//  CLASS X6_MasterCore_I2CLinkage -- Mastercore I2C link Vcxo "microDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_MasterCore_I2CLinkage
//---------------------------------------------------------------------------

const int     MCVcxoI2C_SubAddr  = 0xAA;

//---------------------------------------------------------------------------
//  constructor for class X6_MasterCore_I2CLinkage
//---------------------------------------------------------------------------

X6_MasterCore_I2CLinkage::X6_MasterCore_I2CLinkage(IAddressingSpace & space, MapRegisters & regs,
        I2C_MasterCore::MapRegisters & mc_regs)
    :   VcxoControl(space, regs.AfeVcxoControl_Addr),
        Vcxo_I2C(space, mc_regs, MCVcxoI2C_SubAddr) {
}

//---------------------------------------------------------------------------
//  destructor for class X6_MasterCore_I2CLinkage
//---------------------------------------------------------------------------

X6_MasterCore_I2CLinkage::~X6_MasterCore_I2CLinkage() {
}

//---------------------------------------------------------------------------
//  X6_MasterCore_I2CLinkage::Init() - One time initialization
//---------------------------------------------------------------------------

void  X6_MasterCore_I2CLinkage::Init() {
    Vcxo_I2C.Init();
}

//---------------------------------------------------------------------------
//  X6_MasterCore_I2CLinkage::DeviceEnable() - enable vcxo
//---------------------------------------------------------------------------

void  X6_MasterCore_I2CLinkage::DeviceEnable(bool enabled) {
    VcxoControl.Vco_Enable.Value(enabled);
}

//===========================================================================
//  CLASS X6AFE_Clock -- X6 AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_Clock
//---------------------------------------------------------------------------

X6AFE_Clock::X6AFE_Clock(IRequires_StdClocks * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       ClockCtrlReg(space, regs.AfeClkControl_Addr),
       I2C_uDev(0), VcxoDev(0), PllDev(0),
       ClockIo(this), ClockDevs(this) {
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_Clock
//---------------------------------------------------------------------------

X6AFE_Clock::~X6AFE_Clock() {
    delete PllDev;
    PllDev  = 0;
    delete VcxoDev;
    VcxoDev = 0;
    delete I2C_uDev;
    I2C_uDev = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_Clock::ConstructCustomParts() {
    if (!DeviceEnabled())
        return;

    I2C_uDev = MDC->MakeAfeVcxoI2CLink();
    VcxoDev  = MDC->MakeAfeVcxoObject(I2C_uDev->Device());
    PllDev   = MDC->MakeAfePllObject();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_Clock::OpenHardware() {
    if (!DeviceEnabled())
        return;

    I2C_uDev->Init();
    EnableVcxo(true);
    VcxoDev->Init();
    PllDev->Powered(true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_Clock::PreconfigureHardware() {
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    VcxoDev->Reapply();
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_Clock::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //VcxoDev->Reapply();
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::EnableVcxo()
//---------------------------------------------------------------------------

void  X6AFE_Clock::EnableVcxo(bool state) {
    I2C_uDev->DeviceEnable(state);
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::Vcxo()
//---------------------------------------------------------------------------

Si570 & X6AFE_Clock::Vcxo() {
    return *VcxoDev;
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::Pll()
//---------------------------------------------------------------------------

Cdce72010 & X6AFE_Clock::Pll() {
    return *PllDev;
}

//===========================================================================
//  CLASS X6AFE_Clock::X6ClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  X6AFE_Clock::X6ClockIo::FrequencyActual() const {
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  X6AFE_Clock::X6ClockIo::Frequency(double value) {
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  X6AFE_Clock::X6ClockIo::Frequency() const {
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  X6AFE_Clock::X6ClockIo::System() {
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  X6AFE_Clock::X6ClockIo::Adc() {
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  X6AFE_Clock::X6ClockIo::Dac() {
    return Owner->PllDev->Dac();
}


//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IX6ClockIo::IIClockSource X6AFE_Clock::X6ClockIo::Source() const {
    if (Owner->PllDev->OutputSource() == Cdce72010::outPll)
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void X6AFE_Clock::X6ClockIo::Source(IIClockSource src) {
    if (src == csInternal)
        Owner->PllDev->OutputSource(Cdce72010::outPll);
    else
        Owner->PllDev->OutputSource(Cdce72010::outAux);
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::ExternalClkSelect() -- Retrieve selected clock
//---------------------------------------------------------------------------

IX6ClockIo::IIClockSelect X6AFE_Clock::X6ClockIo::ExternalClkSelect() const {
    if (Owner->ClockCtrlReg.Pll_ExternalClkSelect.Value() == 0)
        return cslFrontPanel;
    else
        return cslP16;
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::ExternalClkSelect() -- Select clock for external IN
//---------------------------------------------------------------------------

void X6AFE_Clock::X6ClockIo::ExternalClkSelect(IIClockSelect src) {
    if (src == cslFrontPanel)
        Owner->ClockCtrlReg.Pll_ExternalClkSelect.Value(0);
    else
        Owner->ClockCtrlReg.Pll_ExternalClkSelect.Value(1);
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IX6ClockIo::IIReferenceSource X6AFE_Clock::X6ClockIo::Reference() const {
    if (Owner->PllDev->ReferenceSource() == Cdce72010::refPrimary)
        return rsExternal;
    else
        return rsInternal;
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void X6AFE_Clock::X6ClockIo::Reference(IIReferenceSource src) {
    if (src == rsInternal)
        Owner->PllDev->ReferenceSource(Cdce72010::refSecondary);
    else
        Owner->PllDev->ReferenceSource(Cdce72010::refPrimary);
    //
    //  Turn off Ref VCO if not using it
    //Owner->EnableRefVco( src == rsInternal );
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void X6AFE_Clock::X6ClockIo::ReferenceFrequency(double value) {
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double X6AFE_Clock::X6ClockIo::ReferenceFrequency() const {
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  X6AFE_Clock::X6ClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  X6AFE_Clock::X6ClockIo::Locked() const {
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS X6AFE_Ad9516Clock -- X6 AFE Clock "SubDevice" (uses AD9516)
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_Ad9516Clock
//---------------------------------------------------------------------------

const double AD9516_DES_Limit = 1.8e9;

//---------------------------------------------------------------------------
//  constructor for class X6AFE_Ad9516Clock
//---------------------------------------------------------------------------

X6AFE_Ad9516Clock::X6AFE_Ad9516Clock(IRequires_Nothing * mdc, IAd9516Factory * factory)
    :  MDC(mdc), Factory(factory),
       PllDev(0),
       ClockIo(this), ClockDevs(this), ClockDivider(1) {
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_Ad9516Clock
//---------------------------------------------------------------------------

X6AFE_Ad9516Clock::~X6AFE_Ad9516Clock() {
    delete PllDev;
    PllDev  = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_Ad9516Clock::ConstructCustomParts() {
    PllDev  = Factory->MakeAfe_Ad951xPllObject();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_Ad9516Clock::OpenHardware() {
    if (!DeviceEnabled())
        return;

    PllDev->Init();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_Ad9516Clock::PreconfigureHardware() {
    if (!DeviceEnabled())
        return;

    //  PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_Ad9516Clock::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::Pll()
//---------------------------------------------------------------------------

Ad951x_Base_Tb & X6AFE_Ad9516Clock::Pll() {
    return *PllDev;
}

//===========================================================================
//  CLASS X6AFE_Ad9516Clock::X6ClockIo -- X6 AFE Clock "SubDevice" UI
//===========================================================================
//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  X6AFE_Ad9516Clock::X6ClockIo::FrequencyActual() const {
    return Owner->PllDev->FrequencyActual()*Owner->ClockDivider;
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  X6AFE_Ad9516Clock::X6ClockIo::Frequency(double value) {
    if (value > AD9516_DES_Limit)
        Owner->ClockDivider = 2;  // Run Clock at Half Rate
    else
        Owner->ClockDivider = 1;

    Owner->PllDev->Frequency(value/Owner->ClockDivider);
    //Owner->PllDev->Reapply();    // apply this change at once. - now auto reapply
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  X6AFE_Ad9516Clock::X6ClockIo::Frequency() const {
    return Owner->PllDev->Frequency()*Owner->ClockDivider;
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  X6AFE_Ad9516Clock::X6ClockIo::System() {
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  X6AFE_Ad9516Clock::X6ClockIo::Adc() {
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  X6AFE_Ad9516Clock::X6ClockIo::Dac() {
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IX6ClockIo::IIClockSource X6AFE_Ad9516Clock::X6ClockIo::Source() const {
    if (Owner->PllDev->OutputSource() == Ad9516_Tb::outPll)
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void X6AFE_Ad9516Clock::X6ClockIo::Source(IIClockSource src) {
    if (src == csInternal)
        Owner->PllDev->OutputSource(Ad9516_Tb::outPll);
    else
        Owner->PllDev->OutputSource(Ad9516_Tb::outExtClk);
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::ExternalClkSelect() -- Retrieve selected clock
//---------------------------------------------------------------------------

IX6ClockIo::IIClockSelect X6AFE_Ad9516Clock::X6ClockIo::ExternalClkSelect() const {
    if (Owner->Pll().ExternalClockSelect() == Ad9516_Tb::csFrontPanel)
        return cslFrontPanel;
    else
        return cslP16;
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::ExternalClkSelect() -- Select clock for external IN
//---------------------------------------------------------------------------

void X6AFE_Ad9516Clock::X6ClockIo::ExternalClkSelect(IIClockSelect src) {
    if (src == cslFrontPanel)
        Owner->Pll().ExternalClockSelect(Ad9516_Tb::csFrontPanel);
    else
        Owner->Pll().ExternalClockSelect(Ad9516_Tb::csP16);
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IX6ClockIo::IIReferenceSource X6AFE_Ad9516Clock::X6ClockIo::Reference() const {
    if (Owner->PllDev->ReferenceSource() == Ad9516_Tb::refExternal)
        return rsExternal;
    else
        return rsInternal;
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void X6AFE_Ad9516Clock::X6ClockIo::Reference(IIReferenceSource src) {
    if (src == rsInternal)
        Owner->PllDev->ReferenceSource(Ad9516_Tb::refInternal);
    else
        Owner->PllDev->ReferenceSource(Ad9516_Tb::refExternal);
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void X6AFE_Ad9516Clock::X6ClockIo::ReferenceFrequency(double value) {
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double X6AFE_Ad9516Clock::X6ClockIo::ReferenceFrequency() const {
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  X6AFE_Ad9516Clock::X6ClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  X6AFE_Ad9516Clock::X6ClockIo::Locked() const {
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS X6AFE_Common -- X6 AFE Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_Common
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_Common
//---------------------------------------------------------------------------

X6AFE_Common::X6AFE_Common(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TestControl(space, regs.AfeTestControl_Addr),
       EngineReg(space, regs.AfeFlowControl_Addr),
       SwTrigger(space, regs.AfeSoftwareTrigger_Addr),
       ExtSyncSelect(space, regs.AfeExtSyncSelect_Addr) {
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_Common
//---------------------------------------------------------------------------

X6AFE_Common::~X6AFE_Common() {
}

//---------------------------------------------------------------------------
//  X6AFE_Common::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_Common::ConstructCustomParts() {
}

//---------------------------------------------------------------------------
//  X6AFE_Common::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_Common::OpenHardware() {
    if (!DeviceEnabled())
        return;

    AllOff();
    TestModeEnabled( tInputOutput, false, 0 );
}

//---------------------------------------------------------------------------
//  X6AFE_Common::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_Common::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    AllOff();
}

//---------------------------------------------------------------------------
//  X6AFE_Common::AllOff()
//---------------------------------------------------------------------------

void  X6AFE_Common::AllOff() {
    Engine( tInputOutput, false );
//    TestModeEnabled( tInputOutput, false );
    SoftwareTrigger( tInputOutput, false );
}

//---------------------------------------------------------------------------
//  X6AFE_Common::TestModeEnabled() --  Set Test Mode state
//---------------------------------------------------------------------------

void X6AFE_Common::TestModeEnabled(AfeDirection which, bool state, unsigned int mode) {
    switch (which) {
    case tInput:
        TestControl.AdcTestMode.Value(mode);
        TestControl.AdcTestEnable.Value(state);
        break;
    case tOutput:
        TestControl.DacTestMode.Value(mode);
        TestControl.DacTestEnable.Value(state);
        break;
    case tInputOutput:
        TestControl.AdcTestMode.Value(mode);
        TestControl.DacTestMode.Value(mode);
        TestControl.AdcTestEnable.Value(state);
        TestControl.DacTestEnable.Value(state);
        break;
    }
}

//---------------------------------------------------------------------------
//  X6AFE_Common::Engine() --  Set SW Trigger state
//---------------------------------------------------------------------------

void X6AFE_Common::Engine(AfeDirection which, bool state) {
    switch (which) {
    case tInput:
        EngineReg.AdcGo.Value(state);
        break;
    case tOutput:
        EngineReg.DacGo.Value(state);
        break;
    case tInputOutput:
        EngineReg.AdcGo.Value(state);
        EngineReg.DacGo.Value(state);
        break;
    }

    EngineReg.Apply();
}

//---------------------------------------------------------------------------
//  X6AFE_Common::SoftwareTrigger() --  Set SW Trigger state
//---------------------------------------------------------------------------

void X6AFE_Common::SoftwareTrigger(AfeDirection which, bool state) {
    switch (which) {
    case tInput:
        SwTrigger.AdcTrigger.Value(state);
        break;
    case tOutput:
        SwTrigger.DacTrigger.Value(state);
        break;
    case tInputOutput:
        SwTrigger.AdcTrigger.Value(state);
        SwTrigger.DacTrigger.Value(state);
        break;
    }

    SwTrigger.Apply();
}

//---------------------------------------------------------------------------
//  X6AFE_Common::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

void X6AFE_Common::ExternalSyncSelection(AfeExtSyncOptions option) {
    switch (option) {
    case essFrontPanel:
        ExtSyncSelect.ExtSyncOption.Value(0);
        break;
    case essP16:
        ExtSyncSelect.ExtSyncOption.Value(1);
        break;
    }
}

//---------------------------------------------------------------------------
//  X6AFE_Common::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

X6AFE_Common::AfeExtSyncOptions X6AFE_Common::ExternalSyncSelection() const {
    bool option =  ExtSyncSelect.ExtSyncOption.Value();
    if (option)
        return essP16;
    else
        return essFrontPanel;
}

//==============================================================================
//  CLASS X6AFE_PulseRepetitionInterval --  X6 Pulse Repetiton stuff
//==============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6AFE_PulseRepetitionInterval
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_PulseRepetitionInterval
//---------------------------------------------------------------------------

X6AFE_PulseRepetitionInterval::X6AFE_PulseRepetitionInterval(IRequires_PriClockDevice * mdc,
        bool isAdcPri, IAddressingSpace & space, MapRegisters & regs)
    :  IsAdcPri(isAdcPri), ClkDev(0), FEnabled(true),
       MDC(mdc),
       PriTriggerEnableReg(space, regs.PriTriggerEnable_Addr),
       PriIntervalReg(space, regs.PriInterval_Addr),
       PriTriggerParametersReg(space, regs.PriTriggerParam_Addr),
       PriCaptureWindowReg(space, regs.PriCaptureWindow_Addr) {
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_PulseRepetitionInterval
//---------------------------------------------------------------------------

X6AFE_PulseRepetitionInterval::~X6AFE_PulseRepetitionInterval() {
    delete ClkDev;
    ClkDev = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::ConstructCustomParts() {
    ClkDev = MDC->MakePriClockDevice(IsAdcPri ? IRequires_PriClockDevice::priAdc
                                     : IRequires_PriClockDevice::priDac );
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::OpenHardware() {
    if (!DeviceEnabled())
        return;

    Reset();

}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    Apply();
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::StreamStopHardware()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::StreamStopHardware() {
    if (!DeviceEnabled())
        return;

    Reset();
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::Reset()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::Reset() {
    Run(false);
    SetEnable(false);

    CaptureEvents.clear();
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::AddEvent()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::AddEvent(unsigned int period,
        unsigned int delay,
        unsigned int width) {
    CaptureEvents.push_back(CaptureEvent(period, delay, width));
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::Apply()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::Apply() {
    SetEnable(FEnabled);
    if (!FEnabled)
        return;

    int divisor = ClkDev->CalculatePriDivisor();

    for (CaptureEventArray::iterator i = CaptureEvents.begin(); i != CaptureEvents.end(); ++i) {
        // Period register must be loaded with N-1
        PriIntervalReg.Value((i->Period/divisor) - 1);
        PriTriggerParametersReg.Delay.Value(i->Delay/divisor);
        PriCaptureWindowReg.Width.Value(i->Width/divisor);
    }

    SetFiniteFrames(FFiniteFrames);
    SetFiniteFrameRearm(FFiniteFrameRearm);
    SetFiniteFrameCount(FFiniteFrameCount);
    Run(true);

}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::Enabled()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::Enabled(bool state) {
    FEnabled = state;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::Enabled()
//---------------------------------------------------------------------------

bool  X6AFE_PulseRepetitionInterval::Enabled() const {
    return FEnabled;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::Run()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::Run(bool state) {
    PriTriggerEnableReg.RunN.Value(state ? 0 : 1);   // negative logic
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::Run()
//---------------------------------------------------------------------------

bool  X6AFE_PulseRepetitionInterval::Run() const {
    return PriTriggerEnableReg.RunN.Value() ? false : true; // negative logic
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::FiniteFrames()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::FiniteFrames(bool state) {
    FFiniteFrames = state;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::FiniteFrames()
//---------------------------------------------------------------------------

bool  X6AFE_PulseRepetitionInterval::FiniteFrames() const {
    return FFiniteFrames;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::FiniteFrameRearm()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::FiniteFrameRearm(bool state) {
    FFiniteFrameRearm = state;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::FiniteFrameRearm()
//---------------------------------------------------------------------------

bool  X6AFE_PulseRepetitionInterval::FiniteFrameRearm() const {
    return FFiniteFrameRearm;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::FiniteFrameCount()
//---------------------------------------------------------------------------

void  X6AFE_PulseRepetitionInterval::FiniteFrameCount(unsigned short state) {
    FFiniteFrameCount = state;
}

//---------------------------------------------------------------------------
//  X6AFE_PulseRepetitionInterval::FiniteFrameCount()
//---------------------------------------------------------------------------

unsigned short  X6AFE_PulseRepetitionInterval::FiniteFrameCount() const {
    return FFiniteFrameCount;
}

//===========================================================================
//  CLASS X6VitaPacketizerCfg -- X6 Packetizer "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6VitaPacketizerCfg
//---------------------------------------------------------------------------

const int X6_MaxInputVitas = X6_MemoryMap::mmAFE_TotalAdcVitas;  // Max SIDs allowed

//---------------------------------------------------------------------------
//  constructor for class X6VitaPacketizerCfg
//---------------------------------------------------------------------------

X6VitaPacketizerCfg::X6VitaPacketizerCfg(IRequires_VitaInCount * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TimestampLoadReg(space, regs.TimestampInit_Addr),
       TimestampCtrlReg(space, regs.TimestampCtrl_Addr),
       VitaInSizeArray(space, regs.VitaAdcSize_Addr, X6_MaxInputVitas),
       VitaInStreamIdArray(space, regs.VitaAdcStreamId_Addr, X6_MaxInputVitas),
       FVitaInStreamCount(0) {
}

//---------------------------------------------------------------------------
//  destructor for class X6VitaPacketizerCfg
//---------------------------------------------------------------------------

X6VitaPacketizerCfg::~X6VitaPacketizerCfg() {
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::ConstructCustomParts() {
    FVitaInStreamCount = MDC->GetVitaInStreamCount();
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::OpenHardware()
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::OpenHardware() {
    if (!DeviceEnabled())
        return;

    // Auto-enable packetizers
    for (unsigned int ch=0; ch<FVitaInStreamCount; ch++) {
        VitaStreamId(ch, 0x100+ch);
        //VitaPacketSize(ch, 0x100);      // DEBUG - should accept default size when
        //                                //   they have one...
    }


}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    ArmTimestamp(true);  // Temporary? (replace with logic using 'Run'
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::StreamStopHardware()
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::StreamStopHardware() {
    if (!DeviceEnabled())
        return;

    ArmTimestamp(false);  // Temporary? (replace with logic using 'Run'
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampPpsMode() - Set PPS Mode
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::TimestampPpsMode(IVitaTimestamp::TimestampPpsModes mode) {
    bool bitval = (mode==tsInternalTimer) ? false : true ;
    TimestampCtrlReg.PpsMode.Value(bitval);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampPpsMode() - Get PPS Mode
//---------------------------------------------------------------------------

IVitaTimestamp::TimestampPpsModes  X6VitaPacketizerCfg::TimestampPpsMode() {
    unsigned int mode = (TimestampCtrlReg.PpsMode.Value()) ? 0 : 1;
    return static_cast<TimestampPpsModes>(mode);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampIsMode() - Set IS Mode
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::TimestampIsMode(IVitaTimestamp::TimestampIsModes mode) {
    TimestampCtrlReg.IntSecMode.Value(mode);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampIsMode() - Get IS Mode
//---------------------------------------------------------------------------

IVitaTimestamp::TimestampIsModes  X6VitaPacketizerCfg::TimestampIsMode() {
    unsigned int mode = TimestampCtrlReg.IntSecMode.Value();
    return static_cast<TimestampIsModes>(mode);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampFsMode() - Set FS Mode
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::TimestampFsMode(IVitaTimestamp::TimestampFsModes mode) {
    TimestampCtrlReg.FracSecMode.Value(mode);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampFsMode() - Get FS Mode
//---------------------------------------------------------------------------

IVitaTimestamp::TimestampFsModes  X6VitaPacketizerCfg::TimestampFsMode() {
    unsigned int mode = TimestampCtrlReg.FracSecMode.Value();
    return static_cast<TimestampFsModes>(mode);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::TimestampCtrl() -
//---------------------------------------------------------------------------

void X6VitaPacketizerCfg::TimestampCtrl(IVitaTimestamp::TimestampPpsModes ppsmode, IVitaTimestamp::TimestampIsModes ismode, IVitaTimestamp::TimestampFsModes modefs) {
    TimestampCtrlReg.PpsMode.Value(ppsmode);
    TimestampCtrlReg.IntSecMode.Value(ismode);
    TimestampCtrlReg.FracSecMode.Value(modefs);
    TimestampCtrlReg.Arm.Value(1);
    TimestampCtrlReg.Apply();
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::ArmTimestamp() -
//---------------------------------------------------------------------------

void X6VitaPacketizerCfg::ArmTimestamp(bool state) {
    TimestampCtrlReg.Arm.Value(state);
    TimestampCtrlReg.Apply();
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaStreamId() - Set Stream Id
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::VitaStreamId(unsigned int velo_ch, unsigned int stream_id) {
    if (velo_ch < FVitaInStreamCount)
        VitaInStreamIdArray[velo_ch].StreamId.Value(stream_id);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaStreamId() - Read Stream Id
//---------------------------------------------------------------------------

unsigned int  X6VitaPacketizerCfg::VitaStreamId(unsigned int velo_ch) {
    if (velo_ch < FVitaInStreamCount)
        return VitaInStreamIdArray[velo_ch].StreamId.Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaDestinationId() - Set Destination Id
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::VitaDestinationId(unsigned int velo_ch, unsigned int stream_id) {
    if (velo_ch < FVitaInStreamCount)
        VitaInStreamIdArray[velo_ch].DestId.Value(stream_id);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaDestinationId() - Read Destination Id
//---------------------------------------------------------------------------

unsigned int  X6VitaPacketizerCfg::VitaDestinationId(unsigned int velo_ch) {
    if (velo_ch < FVitaInStreamCount)
        return VitaInStreamIdArray[velo_ch].DestId.Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaPacketSize() - Set Packet Size
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::VitaPacketSize(unsigned int velo_ch, unsigned int full_size_ints) {
    if (velo_ch < FVitaInStreamCount)
        VitaInSizeArray[velo_ch].Value(full_size_ints);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaPacketSize() - Read Packet Size
//---------------------------------------------------------------------------

unsigned int  X6VitaPacketizerCfg::VitaPacketSize(unsigned int velo_ch) {
    if (velo_ch < FVitaInStreamCount)
        return VitaInSizeArray[velo_ch].Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaDataSize() - Set Packet Data Size
//---------------------------------------------------------------------------

void  X6VitaPacketizerCfg::VitaDataSize(unsigned int velo_ch, unsigned int data_size_ints) {
    VitaBufferManager  VBM;
    unsigned int packet_size = data_size_ints + VBM.FullSizeDelta();
    VitaPacketSize(velo_ch, packet_size);
}

//---------------------------------------------------------------------------
//  X6VitaPacketizerCfg::VitaDataSize() - Read Packet Data Size
//---------------------------------------------------------------------------

unsigned int  X6VitaPacketizerCfg::VitaDataSize(unsigned int velo_ch) {
    VitaBufferManager  VBM;
    unsigned int packet_size =  VitaPacketSize(velo_ch);
    //  Convert to data size and return...
    return packet_size - VBM.FullSizeDelta();
}

//===========================================================================
//  CLASS X6VitaDepacketizerCfg -- X6 Packetizer "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6VitaDepacketizerCfg
//---------------------------------------------------------------------------

const int X6_MaxOutputVitas = X6_MemoryMap::mmAFE_TotalDacVitas;  // Max SIDs allowed

//---------------------------------------------------------------------------
//  constructor for class X6VitaDepacketizerCfg
//---------------------------------------------------------------------------

X6VitaDepacketizerCfg::X6VitaDepacketizerCfg(IRequires_VitaOutCount * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       VitaOutStreamIdArray(space, regs.VitaDacStreamId_Addr, X6_MaxOutputVitas),
       FVitaOutStreamCount(0) {
}

//---------------------------------------------------------------------------
//  destructor for class X6VitaDepacketizerCfg
//---------------------------------------------------------------------------

X6VitaDepacketizerCfg::~X6VitaDepacketizerCfg() {
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6VitaDepacketizerCfg::ConstructCustomParts() {
    FVitaOutStreamCount = MDC->GetVitaOutStreamCount();
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::OpenHardware()
//---------------------------------------------------------------------------

void  X6VitaDepacketizerCfg::OpenHardware() {
    if (!DeviceEnabled())
        return;

    // Auto-enable packetizers
    for (unsigned int ch=0; ch<FVitaOutStreamCount; ch++) {
        VitaStreamId(ch, 0x100+ch);
    }

}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6VitaDepacketizerCfg::ConfigureHardware() {
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::StreamStopHardware()
//---------------------------------------------------------------------------

void  X6VitaDepacketizerCfg::StreamStopHardware() {
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::VitaStreamId() - Set Stream Id
//---------------------------------------------------------------------------

void  X6VitaDepacketizerCfg::VitaStreamId(unsigned int velo_ch, unsigned int stream_id) {
    if (velo_ch < FVitaOutStreamCount)
        VitaOutStreamIdArray[velo_ch].StreamId.Value(stream_id);
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::VitaStreamId() - Read Stream Id
//---------------------------------------------------------------------------

unsigned int  X6VitaDepacketizerCfg::VitaStreamId(unsigned int velo_ch) {
    if (velo_ch < FVitaOutStreamCount)
        return VitaOutStreamIdArray[velo_ch].StreamId.Value();
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::VitaDestinationId() - Set Destination Id
//---------------------------------------------------------------------------

void  X6VitaDepacketizerCfg::VitaDestinationId(unsigned int velo_ch, unsigned int stream_id) {
    if (velo_ch < FVitaOutStreamCount)
        VitaOutStreamIdArray[velo_ch].DestId.Value(stream_id);
}

//---------------------------------------------------------------------------
//  X6VitaDepacketizerCfg::VitaDestinationId() - Read Destination Id
//---------------------------------------------------------------------------

unsigned int  X6VitaDepacketizerCfg::VitaDestinationId(unsigned int velo_ch) {
    if (velo_ch < FVitaOutStreamCount)
        return VitaOutStreamIdArray[velo_ch].DestId.Value();
    else
        return 0;
}

//===========================================================================
//  CLASS X6_ClockMux -- X6 Clock Mux "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_ClockMux
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6_ClockMux
//---------------------------------------------------------------------------

X6_ClockMux::X6_ClockMux(IRequires_ClockMux * mdc, IAddressingSpace & /*space*/,
                         MapRegisters & /*regs */, int device_tag)
    :  MDC(mdc),
       //  MuxSpi(space, regs.ClockMuxSpi_Addr),    // in module
       ClkMux(0), DeviceTag(device_tag) {
}

//---------------------------------------------------------------------------
//  destructor for class X6_ClockMux
//---------------------------------------------------------------------------

X6_ClockMux::~X6_ClockMux() {
    delete ClkMux;
    ClkMux  = 0;
}

//---------------------------------------------------------------------------
//  X6_ClockMux::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6_ClockMux::ConstructCustomParts() {
    ClkMux = MDC->MakeClockMuxDevice(DeviceTag);
}

//---------------------------------------------------------------------------
//  X6_ClockMux::OpenHardware()
//---------------------------------------------------------------------------

void  X6_ClockMux::OpenHardware() {
    if (!DeviceEnabled())
        return;

//    ClkMux->Powered(true);
    ClkMux->Init();
}

//---------------------------------------------------------------------------
//  X6_ClockMux::PreconfigureHardware()
//---------------------------------------------------------------------------

void  X6_ClockMux::PreconfigureHardware() {
    if (!DeviceEnabled())
        return;

    // Reapply Mux settings
    ClkMux->Reapply();
}

//---------------------------------------------------------------------------
//  X6_ClockMux::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6_ClockMux::ConfigureHardware() {
    if (!DeviceEnabled())
        return;

    // Reapply Mux settings
    //ClkMux->Reapply();
}

//---------------------------------------------------------------------------
//  X6_ClockMux::Mux()
//---------------------------------------------------------------------------

Cdce18005 & X6_ClockMux::Mux() {
    return *ClkMux;
}

#ifdef _MSC_VER
#pragma warning(pop)
#endif

}   // end namespace Innovative

