//---------------------------------------------------------------------------
// X6SubDevice_Stream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  "Stream" Subdevices
//---------------------------------------------------------------------------

#ifndef X6SubDevice_Stream_MbH
#define X6SubDevice_Stream_MbH

#include <X6Support_Mb.h>
#include <HardwareRegister_Mb.h>
#include <CDCE18005_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6PacketizerCfg -- X6 Packetizer "SubDevice"
//===========================================================================

class X6PacketizerCfg : public IX6SubDevice, public IVeloPacketizer
{
public:
    //=======================================================================
    //  CLASS X6PacketizerCfg::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   PacketizerEnable_Addr;
        unsigned int   AuxHeader_Addr;
        unsigned int   VelociaInHeader_Addr;
        unsigned int   PacketizerForceSize_Addr;
        unsigned int   PacketizerIntervalPeriod_Addr;
    };

    X6PacketizerCfg(IRequires_VeloInCount * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6PacketizerCfg();

    void  Enable(unsigned int velo_ch, bool state);
    void  Resize(unsigned int new_count);

    virtual unsigned int  VeloChannels()  {  return FInStreamCount;  }

    virtual void  LoadAuxHeaderValue(unsigned int value)
            {   AuxHeaderReg.Value(value);   }

    virtual void  VeloPacketId(unsigned int velo_ch, unsigned char pid);
    virtual unsigned char VeloPacketId(unsigned int velo_ch);

    virtual void  VeloPacketSize(unsigned int velo_ch, unsigned int full_size_ints);
    virtual unsigned int  VeloPacketSize(unsigned int velo_ch);

    virtual void  VeloDataSize(unsigned int velo_ch, unsigned int data_size_ints);
    virtual unsigned int  VeloDataSize(unsigned int velo_ch);

    virtual void  ForceVeloPacketSize(bool enable);
    virtual bool  ForceVeloPacketSize();

    virtual void  ForceVeloPacketSize(unsigned int velo_ch, bool enable);
    virtual bool  ForceVeloPacketSize(unsigned int velo_ch);

    virtual void  LoadAll_VeloPacketSize(unsigned int full_size_ints);
    virtual void  LoadAll_VeloDataSize(unsigned int data_size_ints);

    virtual void   PacketIntervalPeriod(float period_ms);
    virtual float  PacketIntervalPeriod();

    virtual void  SetStreamCount(unsigned int new_count);

    // Custom
    virtual void Custom_Tls_VeloPacketConfig(unsigned int size);
    virtual void Custom_Fft_VeloPacketConfig(unsigned int size);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VeloPacketRegister  -- Rx Packet size management
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VeloPacketRegister : public Register
    {
        typedef Register inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Size;
        RegisterBitGroup      Id;

        //
        //  Member Functions
        VeloPacketRegister(IAddressingSpace &space, int offset)
            : inherited("X6PacketizerCfg::VeloPacketRegister", space, offset),
              Size(*this, 0, 24), Id(*this, 24, 8)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS ForcedPacketSizeEnableRegister  -- Force the packet size
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ForcedPacketSizeEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Enable;

        //
        //  Member Functions
        ForcedPacketSizeEnableRegister(IAddressingSpace &space, int offset)
            : inherited("X6PacketizerCfg::ForcedPacketSizeEnableRegister", space, offset),
              Enable(*this, 0, 8)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PacketIntervalRegister  -- Interval between normal packets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PacketIntervalRegister : public Register
    {
        typedef Register inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Period;

        //
        //  Member Functions
        PacketIntervalRegister(IAddressingSpace &space, int offset)
            : inherited("X6PacketizerCfg::PacketIntervalRegister", space, offset),
              Period(*this, 0, 22)
            {}
    };
private:
    IRequires_VeloInCount *  MDC;
    Register  PacketizerEnableReg;
    Register  AuxHeaderReg;
    RegisterArray<VeloPacketRegister>  VelociaInHeaderArray;
    ForcedPacketSizeEnableRegister     PacketizerForceSizeReg;
    PacketIntervalRegister             PacketIntervalReg;

    unsigned int FInStreamCount;

};

//===========================================================================
//  CLASS X6DepacketizerCfg -- X6 Depacketizer "SubDevice"
//===========================================================================

class X6DepacketizerCfg : public IX6SubDevice, public IVeloDepacketizer
{
public:
    //=======================================================================
    //  CLASS X6DepacketizerCfg::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   TestLoopbackPid_Addr;
        unsigned int   OutgoingDataPid_Addr;
    };

    X6DepacketizerCfg(IRequires_VeloOutCount * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6DepacketizerCfg();

    virtual void  TestLoopbackVeloPacketId(unsigned char pid);
    virtual unsigned char TestLoopbackVeloPacketId();

    virtual void  VeloPacketId(unsigned int velo_ch, unsigned char pid);
    virtual unsigned char  VeloPacketId(unsigned int velo_ch);

	virtual void SetStreamCount(unsigned int new_count);
    virtual void SetCommandStreamCount(unsigned int new_count);
    virtual void SetDataStreamCount(unsigned int new_count);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VeloPacketRegister  -- Tx Packet size management
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VeloPacketRegister : public Register
    {
        typedef Register inherited;

    public:
        //
        //  Member Data
        RegisterBitGroup      Id;

        //
        //  Member Functions
        VeloPacketRegister(IAddressingSpace &space, int offset)
            : inherited("X6DepacketizerCfg::VeloPacketRegister", space, offset),
              Id(*this, 24, 8)
            {}
    };


private:
    IRequires_VeloOutCount *  MDC;
    VeloPacketRegister  TestLoopbackPidReg;
    RegisterArray<VeloPacketRegister>  VelociaOutHeaderArray;

    unsigned int FOutStreamCount;
    unsigned int FOutDataCount;
    unsigned int FOutCmdCount;

};

//===========================================================================
//  CLASS X6AlertSystem -- X6 Alert "SubDevice"
//===========================================================================

class X6AlertSystem : public IX6SubDevice, public AlertSystem, public IUsesX6Alerts
{
public:
    //=======================================================================
    //  CLASS X6Alerts::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AlertEnable_Addr;
        unsigned int   AlertControl_Addr;
        unsigned int   SoftwareAlert_Addr;
        unsigned int   AlertClear_Addr;
    };

    X6AlertSystem(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6AlertSystem();

    virtual void  AlertEnable(AlertType alert, bool state)
                       {  DoAlertEnable((int)alert, state);  }
    virtual void  AlertPassThru(AlertType alert, bool state)
                       {  DoAlertPassThru((int)alert, state);  }
    virtual void  AlertClear(AlertType alert)
                       {  DoAlertClear((int)alert);  }
    virtual void  AlertReset();

    virtual void  SoftwareAlert(unsigned int tag=0);

    virtual void  Append(const AlertData & alert);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    virtual void    Open();       // don't call these
    virtual void    Configure();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AlertControlRegister  --  X6 Module User Logic ID info reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AlertControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit         TimestampOn;
        RegisterBit         AlertFifoReset;

    public:
        AlertControlRegister( IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               TimestampOn( *this, 0 ),  AlertFifoReset( *this, 1 )
            {   }
    };

protected:
    virtual void  ApplyAlertClearHdw(unsigned int bit);

private:
    IRequires_Nothing *   MDC;
    Register              AlertEnableReg;
    AlertControlRegister  AlertControlReg;
    Register              SoftwareAlertReg;
    Register              AlertClearReg;
    unsigned int          FRolloverCount;

    void  InitAlerts();
    virtual int  ExtractAlertArgument(const Buffer & alert, int bit, int field);

};

//===========================================================================
//  CLASS X6_Std_I2CLinkage -- Standard bit-banged I2C link "microDevice"
//===========================================================================

class X6_Std_I2CLinkage : public IX6_I2C_uDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_Std_I2CLinkage::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeVcxoControl_Addr;
    };

    X6_Std_I2CLinkage( IAddressingSpace & space, MapRegisters & regs );
    virtual ~X6_Std_I2CLinkage();

    virtual void          Init()
                {  }

    virtual I2CDevice &   Device()
                {  return Vcxo_I2C;  }
    virtual void          DeviceEnable(bool enabled);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_Std_I2CLinkage::VcxoI2CRegister  -- Vcxo I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Data_Out;
        RegisterBit  Clock_Out;
        RegisterBit  Data_In;
        RegisterBit  Clock_In;
        RegisterBit  Vco_Enable;

    public:
        VcxoI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data_Out(*this, 0),   Clock_Out(*this, 1),
               Data_In(*this, 2),    Clock_In(*this, 3),
               Vco_Enable(*this, 7)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS VcxoI2C_Impl  -- Vcxo I2C Interface Implementation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2C_Impl : public I2CDeviceIntf
    {
    public:
        VcxoI2C_Impl(VcxoI2CRegister & reg)
            :  Reg(reg)
            {}
        virtual ~VcxoI2C_Impl()
            {}
        // Methods
        virtual void    SetClock(int state)
            {  Reg.Clock_Out.Value(state ? true : false);  }
        virtual int     GetClock() const
            {  return Reg.Clock_In.Value();  }
        virtual void    SetData(int state)
            {  Reg.Data_Out.Value(state ? true : false);  }
        virtual int     GetData() const
            {  return Reg.Data_In.Value();  }

    private:
        VcxoI2CRegister &  Reg;
    };

private:
    VcxoI2CRegister    VcxoControl;
    //
    //  Interface Impls
    VcxoI2C_Impl       VcxoIntf;
    I2CDeviceN         Vcxo_I2C;
};

//===========================================================================
//  CLASS X6_MasterCore_I2CLinkage -- Mastercore I2C link Vcxo "microDevice"
//===========================================================================

class X6_MasterCore_I2CLinkage : public IX6_I2C_uDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_MasterCore_I2CLinkage::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeVcxoControl_Addr;
    };

    X6_MasterCore_I2CLinkage( IAddressingSpace & space, MapRegisters & regs,
                                I2C_MasterCore::MapRegisters & mc_regs );
    virtual ~X6_MasterCore_I2CLinkage();

    virtual void               Init();
    virtual I2C_MasterCore &   Device()
                {  return Vcxo_I2C;  }
    virtual void               DeviceEnable(bool enabled);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_Std_I2CLinkage::VcxoI2CRegister  -- Vcxo I2C Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoI2CRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Vco_Enable;

    public:
        VcxoI2CRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Vco_Enable(*this, 7)
            {
            }
    };

private:
    VcxoI2CRegister    VcxoControl;
    //
    //  Interface Impls
    I2C_MasterCore     Vcxo_I2C;
};

//===========================================================================
//  CLASS X6AFE_Clock -- X6 AFE Clock "SubDevice"
//===========================================================================

class X6AFE_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
//        unsigned int   AfeVcxoControl_Addr;
        unsigned int   AfeClkControl_Addr;    // Misc Clock Control Stuff
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Clock::X6ClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class X6ClockIo : public IX6ClockIo
    {
    public:
        X6ClockIo(X6AFE_Clock * owner): Owner(owner)
            {  }
        virtual ~X6ClockIo()
            {  }
        //
        //  IX6ClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src);
        virtual IIClockSelect       ExternalClkSelect() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        X6AFE_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Clock::X6RawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class X6RawClockDevices : public IX6RawClockDevices
    {
    public:
        X6RawClockDevices(X6AFE_Clock * owner): Owner(owner)
            {  }
        virtual ~X6RawClockDevices()
            {  }

        virtual Si570 &       Vcxo()     {  return Owner->Vcxo();  }
        virtual Cdce72010 &   Pll()      {  return Owner->Pll();  }

    protected:
        X6AFE_Clock *  Owner;
    };

    friend class X6ClockIo;
    friend class X6RawClockDevices;

    X6AFE_Clock(IRequires_StdClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~X6AFE_Clock();

    X6ClockIo &            Clock() {  return ClockIo;  }
    X6RawClockDevices &    RawClockDevice() {  return ClockDevs;  }

    Si570 &                Vcxo();
    Cdce72010 &            Pll();
    //
    //  Methods
    void  EnableVcxo(bool state);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Clock::ClockCtrlRegister  -- Assorted Clock Control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ClockCtrlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  Pll_ExternalClkSelect;

    public:
        ClockCtrlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Pll_ExternalClkSelect(*this, 0)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_StdClocks *  MDC;
    //Register         PllControl;     // Implemented inside CDC Pll
    //Register         PllSpi;
    ClockCtrlRegister  ClockCtrlReg;

    IX6_I2C_uDevice *  I2C_uDev;
    Si570 *            VcxoDev;

    Cdce72010 *        PllDev;
    X6ClockIo          ClockIo;
    X6RawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS X6AFE_Ad9516Clock -- X6 AFE Clock "SubDevice" (uses AD9516)
//===========================================================================

class X6AFE_Ad9516Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6ClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class X6ClockIo : public IX6ClockIo
    {
    public:
        X6ClockIo(X6AFE_Ad9516Clock * owner): Owner(owner)
            {  }
        virtual ~X6ClockIo()
            {  }
        //
        //  IX6ClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src);
        virtual IIClockSelect       ExternalClkSelect() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        X6AFE_Ad9516Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Ad9516Clock::X6RawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class X6RawClockDevices : public IX6_Ad951x_RawClockDevices
    {
    public:
        X6RawClockDevices(X6AFE_Ad9516Clock * owner): Owner(owner)
            {  }
        virtual ~X6RawClockDevices()
            {  }

        virtual Ad951x_Base_Tb &   Pll()      {  return Owner->Pll();  }

    protected:
        X6AFE_Ad9516Clock *  Owner;
    };

    friend class X6ClockIo;
    friend class X6RawClockDevices;

    X6AFE_Ad9516Clock(IRequires_Nothing * mdc, IAd9516Factory * factory);
    virtual ~X6AFE_Ad9516Clock();

    X6ClockIo &            Clock() {  return ClockIo;  }
    X6RawClockDevices &    RawClockDevice() {  return ClockDevs;  }

    Ad951x_Base_Tb &       Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    IAd951xFactory *   Factory;

    Ad951x_Base_Tb *   PllDev;
    X6ClockIo          ClockIo;
    X6RawClockDevices  ClockDevs;
    unsigned int       ClockDivider;

};

//===========================================================================
//  CLASS X6AFE_Common -- X6 AFE Common Registers "SubDevice"
//===========================================================================

class X6AFE_Common : public IX6SubDevice
{
public:

    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essP16 };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Common::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeTestControl_Addr;
        unsigned int   AfeFlowControl_Addr;
        unsigned int   AfeSoftwareTrigger_Addr;
        unsigned int   AfeExtSyncSelect_Addr;
    };

    X6AFE_Common(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~X6AFE_Common();

    //
    //  Methods
    void TestModeEnabled( AfeDirection which, bool state, unsigned int mode );
    void Engine( AfeDirection which, bool state );
    void SoftwareTrigger( AfeDirection which, bool state );
    void AllOff( );

    AfeExtSyncOptions  ExternalSyncSelection() const;
    void               ExternalSyncSelection(AfeExtSyncOptions option);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Common::TestControlRegister  -- TestControl Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TestControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       AdcTestEnable;
        RegisterBitGroup  AdcTestMode;
        RegisterBit       DacTestEnable;
        RegisterBitGroup  DacTestMode;

    public:
        TestControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcTestEnable(*this, 0),   AdcTestMode(*this,  4, 8),
               DacTestEnable(*this, 16),  DacTestMode(*this, 20, 8)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Common::EngineRegister  -- 'Engine' flow control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class EngineRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit  AdcGo;
        RegisterBit  DacGo;

    public:
        EngineRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcGo(*this, 0),   DacGo(*this, 16)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Common::SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit  AdcTrigger;
        RegisterBit  DacTrigger;

    public:
        SoftwareTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcTrigger(*this, 0),   DacTrigger(*this, 16)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_Common::ExtSyncSelectRegister  -- Select ext trigger source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ExtSyncSelectRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit  ExtSyncOption;

    public:
        ExtSyncSelectRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ExtSyncOption(*this, 0)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    TestControlRegister       TestControl;
    EngineRegister            EngineReg;
    SoftwareTriggerRegister   SwTrigger;
    ExtSyncSelectRegister     ExtSyncSelect;

};

//==============================================================================
//  CLASS X6AFE_PulseRepetitionInterval --  X6 Pulse Repetiton stuff
//==============================================================================

class X6AFE_PulseRepetitionInterval : public IX6SubDevice, public IPulseRepetitionIntervalIntf
{
public:
    //=======================================================================
    //  CLASS X6AFE_PulseRepetitionInterval::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   PriTriggerEnable_Addr;
        unsigned int   PriInterval_Addr;
        unsigned int   PriTriggerParam_Addr;
        unsigned int   PriCaptureWindow_Addr;
    };

    X6AFE_PulseRepetitionInterval(IRequires_PriClockDevice * mdc, bool isAdcPri,
                                  IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6AFE_PulseRepetitionInterval();

    //  Internal Interface
    //  ....IX6SubDevice
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //  ....IPulseRepetitionIntervalIntf
    virtual void    Reset();
    virtual void    AddEvent(unsigned int period, unsigned int delay,
                                                  unsigned int width);
    virtual void    Apply();

    virtual void    Enabled(bool state);
    virtual bool    Enabled() const;

    virtual void    Run(bool state);
    virtual bool    Run() const;

  	virtual void    		    FiniteFrames(bool state);
	virtual bool	    		FiniteFrames() const;

  	virtual void    		    FiniteFrameRearm(bool state);
	virtual bool	    		FiniteFrameRearm() const;

   	virtual void    		    FiniteFrameCount(unsigned short count);
	virtual unsigned short 		FiniteFrameCount() const;

protected:
    class CaptureEvent
    {
    public:
        CaptureEvent(unsigned int period,
                     unsigned int delay,
                     unsigned int width)
            : Period(period), Delay(delay), Width(width)
            {  }
        // Data
        unsigned int Period;
        unsigned int Delay;
        unsigned int Width;
    };
    // Data
    bool               IsAdcPri;
    IPriClockDevice *  ClkDev;

    typedef std::vector<CaptureEvent> CaptureEventArray;
    CaptureEventArray   CaptureEvents;

    // Fields
    bool   FEnabled;
    bool   FFiniteFrames;
    bool   FFiniteFrameRearm;
    unsigned short   FFiniteFrameCount;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PriTriggerEnableRegister  -- Pri Enable and Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PriTriggerEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit      Enable;
        RegisterBit      RunN;
        RegisterBit      FiniteFrames;
        RegisterBit      FiniteFrameRearm;
        RegisterBit      Busy;
        RegisterBitGroup FiniteFrameCount;

        PriTriggerEnableRegister(IAddressingSpace &space, int offset)
            :  inherited(space, offset),
               Enable(*this, 0), RunN(*this, 1),
               FiniteFrames(*this, 2), FiniteFrameRearm(*this, 3),
               Busy(*this, 8), FiniteFrameCount(*this, 16, 16)
            {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PriIntervalRegister  -- Pri configure
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PriIntervalRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Count;

        PriIntervalRegister(IAddressingSpace &space, int offset)
            :  inherited(space, offset),
               Count(*this, 0, 32)
            {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PriTriggerParametersRegister  -- Pri configure
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PriTriggerParametersRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Delay;

        PriTriggerParametersRegister(IAddressingSpace &space, int offset)
            :  inherited(space, offset),
               Delay(*this, 0, 24)
            {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS PriCaptureWindowRegister  --  Pri configure
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PriCaptureWindowRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Width;

        PriCaptureWindowRegister(IAddressingSpace &space, int offset)
            :  inherited(space, offset),
               Width(*this, 0, 24)
            {  }
    };

    // Registers
    IRequires_PriClockDevice *  MDC;
    PriTriggerEnableRegister      PriTriggerEnableReg;
    PriIntervalRegister           PriIntervalReg;
    PriTriggerParametersRegister  PriTriggerParametersReg;
    PriCaptureWindowRegister      PriCaptureWindowReg;

    // Support methods
    void       SetEnable(bool state)
        {  PriTriggerEnableReg.Enable.Value(state ? 1 : 0);  }
    void       SetFiniteFrames(bool state)
        {  PriTriggerEnableReg.FiniteFrames.Value(state ? 1 : 0);  }
    void       SetFiniteFrameRearm(bool state)
        {  PriTriggerEnableReg.FiniteFrameRearm.Value(state ? 1 : 0);  }
    void       SetFiniteFrameCount(unsigned short count)
        {  PriTriggerEnableReg.FiniteFrameCount.Value(count);  }

    bool       GetEnable() const
        {  return PriTriggerEnableReg.Enable.Value() ? true : false;  }
};

//===========================================================================
//  CLASS X6VitaPacketizerCfg -- X6 VITA Packetizer "SubDevice"
//===========================================================================

class X6VitaPacketizerCfg : public IX6SubDevice, public IVitaPacketizer, public IVitaTimestamp
{
public:
    //=======================================================================
    //  CLASS X6VitaPacketizerCfg::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   TimestampInit_Addr;
        unsigned int   TimestampCtrl_Addr;
        unsigned int   VitaAdcSize_Addr;
        unsigned int   VitaAdcStreamId_Addr;
    };

    X6VitaPacketizerCfg(IRequires_VitaInCount * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6VitaPacketizerCfg();

    virtual void  TimestampPpsMode(TimestampPpsModes mode);
    virtual TimestampPpsModes  TimestampPpsMode();

    virtual void  TimestampIsMode(TimestampIsModes mode);
    virtual TimestampIsModes  TimestampIsMode();

    virtual void  TimestampFsMode(TimestampFsModes mode);
    virtual TimestampFsModes  TimestampFsMode();

    virtual void  TimestampCtrl(TimestampPpsModes ppsmode, TimestampIsModes ismode, TimestampFsModes modefs);

    virtual void  LoadTimestamp(unsigned int value)
            {   TimestampLoadReg.Value(value);   }
    void  ArmTimestamp(bool state);

    virtual unsigned int  VitaStreams()
            {   return FVitaInStreamCount;  }

    virtual void  VitaStreamId(unsigned int vita_ch, unsigned int vitaStreamId);
    virtual unsigned int  VitaStreamId(unsigned int vita_ch);

    virtual void  VitaDestinationId(unsigned int vita_ch, unsigned int vitaDestId);
    virtual unsigned int  VitaDestinationId(unsigned int vita_ch);

    virtual void  VitaPacketSize(unsigned int vita_ch, unsigned int full_size_ints);
    virtual unsigned int  VitaPacketSize(unsigned int vita_ch);

    virtual void  VitaDataSize(unsigned int vita_ch, unsigned int data_size_ints);
    virtual unsigned int  VitaDataSize(unsigned int vita_ch);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6VitaPacketizerCfg::TimestampControlRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TimestampControlRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit       Arm;
        RegisterBit       PpsMode;
        RegisterBitGroup  IntSecMode;
        RegisterBitGroup  FracSecMode;

    public:
        TimestampControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Arm(*this, 0),  PpsMode(*this, 1),
               IntSecMode(*this, 2, 2),
               FracSecMode(*this, 4, 2)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6VitaPacketizerCfg::StreamIdControlRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class StreamIdControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  StreamId;
        RegisterBitGroup  DestId;

    public:
        StreamIdControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               StreamId(*this, 0, 16),
               DestId(*this, 16, 16)
            {
            }
    };

private:
    IRequires_VitaInCount *  MDC;
    Register                                  TimestampLoadReg;
    TimestampControlRegister                  TimestampCtrlReg;
    RegisterArray<Register>                   VitaInSizeArray;
    RegisterArray<StreamIdControlRegister>    VitaInStreamIdArray;

    unsigned int FVitaInStreamCount;

};

//===========================================================================
//  CLASS X6VitaDepacketizerCfg -- X6 VITA Depacketizer "SubDevice"
//===========================================================================

class X6VitaDepacketizerCfg : public IX6SubDevice, public IVitaDepacketizer
{
public:
    //=======================================================================
    //  CLASS X6VitaDepacketizerCfg::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   VitaDacStreamId_Addr;
    };

    X6VitaDepacketizerCfg(IRequires_VitaOutCount * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6VitaDepacketizerCfg();

    virtual unsigned int  VitaStreams()
            {   return FVitaOutStreamCount;  }

    virtual void  VitaStreamId(unsigned int vita_ch, unsigned int vitaStreamId);
    virtual unsigned int  VitaStreamId(unsigned int vita_ch);

    virtual void  VitaDestinationId(unsigned int vita_ch, unsigned int vitaStreamId);
    virtual unsigned int  VitaDestinationId(unsigned int vita_ch);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6VitaDepacketizerCfg::StreamIdControlRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class StreamIdControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  StreamId;
        RegisterBitGroup  DestId;

    public:
        StreamIdControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               StreamId(*this, 0, 16),
               DestId(*this, 16, 16)
            {
            }
    };

private:
    IRequires_VitaOutCount *  MDC;
    RegisterArray<StreamIdControlRegister>    VitaOutStreamIdArray;

    unsigned int FVitaOutStreamCount;

};

//===========================================================================
//  CLASS X6_ClockMux -- X6 Clock Mux "SubDevice"
//===========================================================================

class X6_ClockMux : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_ClockMux::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   ClockMuxSpi_Addr;
    };

    X6_ClockMux(IRequires_ClockMux * mdc, IAddressingSpace & space, MapRegisters & regs, int device_tag );
    virtual ~X6_ClockMux();

    Cdce18005 &  Mux();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

private:
    //
    //  Registers
    IRequires_ClockMux *   MDC;
    Cdce18005 *            ClkMux;
    int                    DeviceTag;   // allows baseboard to support multiple muxes

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}   // end namespace Innovative
//---------------------------------------------------------------------------
#endif
