// X6Support_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//
//  Interfaces and Support Classes for X6 Boards
//---------------------------------------------------------------------------

#ifndef X6Support_MbH
#define X6Support_MbH

#include <AlertSystem_Mb.h>
#include <Si570_Mb.h>
#include <CDCE72010_Mb.h>
#include <CDCE18005_Mb.h>
#include <SpanInfo_Mb.h>
#include <Ad9516_Tb_Mb.h>
#include <Si5338_Mb.h>
#include <X6Idrom_Mb.h>
#include <Gps_Mb.h>
#include <PulseRepetitionIntervalIntf_Mb.h>
#include <CDCE62005_Mb.h>
#include <LMK04800_Mb.h>
#include <LMK04820_Mb.h>
#include <Ad9510Duo_Tb_Mb.h>

//---------------------------------------------------------------------------

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X6DeviceMap;
class IDacCalibrator;
class IX6_I2C_uDevice;
class IComexSignalMux;

//
//  "Subdevice Requirements" Interfaces - Implement "callbacks" to get info
//     or devices from the derived class.
//===========================================================================
//  CLASS IRequires_Nothing -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs nothing from anyone

class IRequires_Nothing
{
public:
    // no functions
    virtual ~IRequires_Nothing()
        {}
};

//===========================================================================
//  CLASS IRequires_SioPll -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs an SIO PLL object

class IRequires_SioPll
{
public:
    virtual Si5338 *     MakeSioPllObject(I2CDevice & device) = 0;

    virtual ~IRequires_SioPll()
        {}
};

//===========================================================================
//  CLASS IRequires_VeloInCount -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the velocia in stream count

class IRequires_VeloInCount
{
public:
    virtual unsigned int GetVeloInStreamCount() = 0;

    virtual ~IRequires_VeloInCount()
        {}
};

//===========================================================================
//  CLASS IRequires_VeloOutCount -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the velocia in stream count

class IRequires_VeloOutCount
{
public:
    virtual unsigned int GetVeloOutStreamCount() = 0;
    virtual unsigned int GetVeloOutCommandStreamCount()
                            {  return 0;  }

    virtual ~IRequires_VeloOutCount()
        {}
};

//===========================================================================
//  CLASS IRequires_VitaInCount -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the vita in stream count

class IRequires_VitaInCount
{
public:
    virtual unsigned int GetVitaInStreamCount() = 0;

    virtual ~IRequires_VitaInCount()
        {}
};

//===========================================================================
//  CLASS IRequires_VitaOutCount -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the vita in stream count

class IRequires_VitaOutCount
{
public:
    virtual unsigned int GetVitaOutStreamCount() = 0;

    virtual ~IRequires_VitaOutCount()
        {}
};

//===========================================================================
//  CLASS IRequires_CalAdcCount -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the Adc channel calibration count

class IRequires_CalAdcCount
{
public:
    virtual unsigned int GetCalibrationAdcCount() = 0;

    virtual ~IRequires_CalAdcCount()
        {}
};

//===========================================================================
//  CLASS IRequires_AdcCalParts -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  Parts needed for Adc Calibration

class IRequires_AdcCalParts : public IRequires_CalAdcCount
{
public:
    virtual class ICalibrationValueOverride * GetAdcCalibrationOverride() = 0;

    virtual ~IRequires_AdcCalParts()
        {}
};

//===========================================================================
//  CLASS IRequires_CalDacCount -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the Dac channel calibration count

class IRequires_CalDacCount
{
public:
    virtual unsigned int GetCalibrationDacCount() = 0;

    virtual ~IRequires_CalDacCount()
        {}
};

//===========================================================================
//  CLASS IRequires_DacCalParts -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  Parts needed for Dac Calibration

class IRequires_DacCalParts : public IRequires_CalDacCount
{
public:
    virtual class ICalibrationValueOverride * GetDacCalibrationOverride() = 0;

    virtual ~IRequires_DacCalParts()
        {}
};

//===========================================================================
//  CLASS IRequires_StdClocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard clock and vcxo devices

class IRequires_StdClocks
{
public:
    virtual IX6_I2C_uDevice *  MakeAfeVcxoI2CLink() = 0;
    virtual Si570 *            MakeAfeVcxoObject(IRequires_I2C_RegIo & device) = 0;
    virtual Cdce72010 *        MakeAfePllObject() = 0;

    virtual ~IRequires_StdClocks()
        {}
};

//===========================================================================
//  CLASS IRequires_AdcSpi -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the ADC Spi Devices

class IRequires_AdcSpi
{
public:
    virtual void         MakeAfeAdcSpiDevices() = 0;

    virtual ~IRequires_AdcSpi()
        {}
};

//===========================================================================
//  CLASS IRequires_MuxedAdcSpi -- Fmc "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the count for a Muxed ADC Spi Device

class IRequires_MuxedAdcSpi
{
public:
    virtual unsigned int GetAdcSpiDeviceCount() = 0;

    virtual ~IRequires_MuxedAdcSpi()
        {}
};

//===========================================================================
//  CLASS IRequires_DacSpi -- Fmc "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the Fmc DAC Spi Devices

class IRequires_DacSpi
{
public:
    virtual void         MakeAfeDacSpiDevices() = 0;

    virtual ~IRequires_DacSpi()
        {}
};

//===========================================================================
//  CLASS IRequires_MuxedDacSpi -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the count for a Muxed ADC Spi Device

class IRequires_MuxedDacSpi
{
public:
    virtual unsigned int GetDacSpiDeviceCount() = 0;

    virtual ~IRequires_MuxedDacSpi()
        {}
};

//===========================================================================
//  CLASS IRequires_DacLatencyCal -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the DAC Spi Devices

class IRequires_DacLatencyCal  :  public IRequires_CalDacCount
{
public:
    virtual IDacCalibrator * GetDacLatencyCalibrator() = 0;

    virtual ~IRequires_DacLatencyCal()
        {}
};

//===========================================================================
//  CLASS IRequires_ClockMux -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a clock mux device

class IRequires_ClockMux
{
public:
    virtual Cdce18005 * MakeClockMuxDevice(int device_tag) = 0;

    virtual ~IRequires_ClockMux()
        {}
};

//===========================================================================
//  CLASS IRequires_Gps -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a gps device

class IRequires_Gps
{
public:
    virtual IGps * MakeGpsDevice() = 0;

    virtual ~IRequires_Gps()
        {}
};

//===========================================================================
//  CLASS IRequires_MultiGps -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a more than 1 gps device

class IRequires_MultiGps
{
public:
    virtual IGps *   MakeOneGpsDevice(size_t idx) = 0;
    virtual size_t   GetGpsCount() = 0;

    virtual ~IRequires_MultiGps()
        {}
};

//===========================================================================
//  CLASS IRequires_PriClockDevice -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a PRI Clock device

class IRequires_PriClockDevice
{
public:
    enum  PriDeviceCode { priAdc, priDac };
    virtual class IPriClockDevice * MakePriClockDevice(PriDeviceCode which) = 0;

    virtual ~IRequires_PriClockDevice()
        {}
};

//===========================================================================
//  CLASS IRequires_FrameCalcDevice -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a Frame divider calculator device

class IRequires_FrameCalcDevice
{
public:
    enum  FrameDeviceCode { devAdc, devDac };
    virtual class IFrameCalcDevice * MakeFrameCalcDevice(FrameDeviceCode which) = 0;

    virtual ~IRequires_FrameCalcDevice()
        {}
};

//===========================================================================
//  CLASS IRequires_LTC2494Device -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a LTC2494 device

class IRequires_LTC2494Device
{
public:
    virtual class LTC2494 * MakeLTC2494() = 0;

    virtual ~IRequires_LTC2494Device()
        {}
};

//===========================================================================
//  CLASS IRequires_Baseboard -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a Vita Packet capable Baseboard Ptr

class IRequires_Baseboard
{
public:
    virtual class IIsAVitaPacketStreamBoard * GetBaseboardPtr() = 0;

    virtual ~IRequires_Baseboard()
        {}
};

//===========================================================================
//  CLASS IRequires_PatternMode -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs pattern mode data plus a baseboard ptr

class IRequires_PatternMode :  public IRequires_Baseboard
{
public:
    virtual unsigned int GetPatternModeAlignment() = 0;
    virtual unsigned int GetPatternModeScaleFactor() = 0;
    virtual bool         GetPatternModeSingleMemoryFlag() = 0;

    virtual ~IRequires_PatternMode()
        {}
};

//===========================================================================
//  CLASS IRequires_VpxClkGenParts -- Vpx "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard clock and vcxo devices

class IRequires_VpxClkGenParts
{
public:
    virtual IX6_I2C_uDevice *  MakeVCG_VcxoI2CLink() = 0;
    virtual Si570 *            MakeVCG_VcxoObject(IRequires_I2C_RegIo & device) = 0;
    virtual Cdce72010 *        MakeVCG_PllObject() = 0;
    virtual Cdce18005 *        MakeVCG_ClkMuxObject() = 0;

    virtual ~IRequires_VpxClkGenParts()
        {}
};

//===========================================================================
//  CLASS IRequires_FmcCommonParts -- Fmc "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a Frame divider calculator device AND some info

class IRequires_FmcCommonParts : public IRequires_FrameCalcDevice
{
public:
    virtual void GetAnalogDeviceInfo(FrameDeviceCode which, unsigned int & devices, unsigned int & ch_per_dev) = 0;
    virtual void GetAnalogPhysicalDeviceInfo(FrameDeviceCode which, unsigned int & phys_devices, unsigned int & ch_per_physdev) = 0;

    virtual ~IRequires_FmcCommonParts()
        {}
};

//===========================================================================
//  CLASS IRequires_XUCommonParts -- Fmc "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice all of the above, AND needs to set/clear XU stream active flag

class IRequires_XUCommonParts : public IRequires_FmcCommonParts
{
public:
    virtual void GetAnalogDeviceInfo(FrameDeviceCode which, unsigned int & devices, unsigned int & ch_per_dev) = 0;
    virtual void GetAnalogPhysicalDeviceInfo(FrameDeviceCode which, unsigned int & phys_devices, unsigned int & ch_per_physdev) = 0;
    virtual void SetXuStreamChannelActive(FrameDeviceCode which, int chan, bool isActive) = 0;

    virtual ~IRequires_XUCommonParts()
        {}
};

//===========================================================================
//  CLASS IRequires_ComexSignalMux -- Vpx "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs a pointer to Comex Signal Devce (not owned)

class IRequires_ComexSignalMux
{
public:
    virtual IComexSignalMux *  MakeComexSignalMuxPtr() = 0;

    virtual ~IRequires_ComexSignalMux()
        {}
};

//===========================================================================
//  CLASS IRequires_StdFmcClocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard FMC clock and vcxo devices

class IRequires_StdFmcClocks
{
public:
    virtual Cdce62005 *        MakeFmcAfePllObject() = 0;

    virtual ~IRequires_StdFmcClocks()
        {}
};

//===========================================================================
//  CLASS IRequires_LmkFmcClocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard FMC clock and vcxo devices

class IRequires_LmkFmcClocks
{
public:
    virtual Lmk04800 *        MakeFmcAfePllObject() = 0;

    virtual ~IRequires_LmkFmcClocks()
        {}
};

typedef IRequires_LmkFmcClocks IRequires_LmkXAClocks;

//===========================================================================
//  CLASS IRequires_Lmk20FmcClocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard FMC clock and vcxo devices

class IRequires_Lmk20FmcClocks
{
public:
    virtual Lmk04820 *        MakeFmcAfePllObject() = 0;

    virtual ~IRequires_Lmk20FmcClocks()
        {}
};

typedef IRequires_Lmk20FmcClocks IRequires_Lmk20XAClocks;

//===========================================================================
//  CLASS IRequires_Ad9510TbClocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice the master and slave clocks for a Ad9510Duo_Tb PLL

class IRequires_Ad9510TbClocks
{
public:
    virtual Ad9510_Tb *        MakeMasterPllObject() = 0;
    virtual Ad9508_Tb *        MakeSlavePllObject() = 0;

    virtual ~IRequires_Ad9510TbClocks()
        {}
};

//===========================================================================
//  CLASS IRequires_FmcCalIDROM-- Fmc "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard FMC clock and vcxo devices

class IRequires_FmcCalIDROM
{
public:
    enum  FmcCalDeviceCode { fmcCalAdc, fmcCalDac };

    class FmcCalIdromInfo
    {
    public:
        unsigned int IdromSize;
        unsigned int ReservedSize;
    };

    virtual class FmcCalIdromInfo GetFmcCalIdromInfo(FmcCalDeviceCode which) = 0;

    virtual ~IRequires_FmcCalIDROM()
        {}
};

//
//  "Subdevice Supplier" Interfaces - Define the set of suppliers needed
//===========================================================================
//  CLASS ISuppliesX6Requirements -- X6 Device Map Requirement List
//===========================================================================

class ISuppliesX6Requirements : public IRequires_SioPll,
                                public IRequires_VeloInCount, public IRequires_VeloOutCount,
                                public IRequires_VitaInCount, public IRequires_VitaOutCount,
                                public IRequires_StdClocks,
                                public IRequires_AdcSpi,      public IRequires_DacSpi,
                                public IRequires_AdcCalParts, public IRequires_DacCalParts,
                                public IRequires_DacLatencyCal,
                                public IRequires_PriClockDevice,  public IRequires_PatternMode,
                                public IRequires_FrameCalcDevice, public IRequires_Nothing
{
public:
    virtual ~ISuppliesX6Requirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesFmcCarrierRequirements -- Vpx Device Map Requirement List
//===========================================================================

class ISuppliesFmcCarrierRequirements : public IRequires_VeloInCount,
                                 public IRequires_VeloOutCount,     //public IRequires_VitaInCount,
                                 public IRequires_PatternMode,      public IRequires_Nothing
{
public:
    virtual ~ISuppliesFmcCarrierRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedVpxRequirements -- Vpx Device Map Requirement List
//===========================================================================

class ISuppliesAddedVpxRequirements : public IRequires_VpxClkGenParts
{
public:
    virtual ~ISuppliesAddedVpxRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedPexRequirements -- Pex Device Map Requirement List
//===========================================================================

class ISuppliesAddedPexRequirements :  public IRequires_SioPll//,
{
public:
    virtual ~ISuppliesAddedPexRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesFmcRequirements -- FMC Board Map Requirement List
//===========================================================================

class ISuppliesFmcRequirements : public IRequires_VitaInCount,  public IRequires_VitaOutCount,
                                 public IRequires_Nothing
{
public:
    virtual ~ISuppliesFmcRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedFmcAnalogRequirements -- Fmc Analog SDev Requirements
//===========================================================================

class ISuppliesAddedFmcAnalogRequirements : public IRequires_FmcCommonParts//,
{
public:
    virtual ~ISuppliesAddedFmcAnalogRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedFmcAdcRequirements --FmcAdcSubdev Requirements
//===========================================================================

class ISuppliesAddedFmcAdcRequirements : public IRequires_PriClockDevice,
                                         //public IRequires_MuxedAdcSpi,
                                         public IRequires_AdcCalParts
{
public:
    virtual ~ISuppliesAddedFmcAdcRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedFmcAdc_T1Requirements --FmcAdcSubdev Requirements
//===========================================================================

class ISuppliesAddedFmcAdc_T1Requirements : public IRequires_MuxedAdcSpi//,
{
public:
    virtual ~ISuppliesAddedFmcAdc_T1Requirements()
        {}

};


//===========================================================================
//  CLASS ISuppliesAddedFmcAdc_BidirRequirements --FmcAdcSubdev Requirements
//===========================================================================
//  Fmc_Bidir requirements

class ISuppliesAddedFmc_BidirRequirements : public IRequires_MuxedAdcSpi//,
{
public:
    virtual ~ISuppliesAddedFmc_BidirRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedFmcDacDeviceMixinRequirements
//===========================================================================
//  FmcDacDeviceMixin requirements

class ISuppliesAddedFmcDacDeviceMixinRequirements : public IRequires_FmcCommonParts,
                                         public IRequires_PriClockDevice,
                                         public IRequires_MuxedDacSpi,
                                         public IRequires_DacCalParts
{
public:
    virtual ~ISuppliesAddedFmcDacDeviceMixinRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesAddedFmcDacRequirements --FmcAdcSubdev Requirements
//===========================================================================

class ISuppliesAddedFmcDacRequirements //: //public IRequires_PriClockDevice,
                                       //  public IRequires_MuxedDacSpi//,
                                         //public IRequires_AdcCalParts
{
public:
    virtual ~ISuppliesAddedFmcDacRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesVpxComexRequirements -- VpxComex Board Map Requirement List
//===========================================================================

class ISuppliesVpxComexRequirements : public IRequires_StdClocks,
                                      public IRequires_ClockMux,
                                      public IRequires_Gps, public IRequires_ComexSignalMux,
                                      public IRequires_Nothing
{
public:
    virtual ~ISuppliesVpxComexRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesK7SysRequirements -- K7Sys Unit Map Requirement List
//===========================================================================

class ISuppliesK7SysRequirements :  public IRequires_MultiGps,
                                    public IRequires_LTC2494Device,
                                    public IRequires_Nothing
{
public:
    virtual ~ISuppliesK7SysRequirements()
        {}

};

//===========================================================================
//  CLASS ISuppliesSbcDuoRequirements -- SbcDuo Unit Map Requirement List
//===========================================================================

class ISuppliesSbcDuoRequirements :  public IRequires_MultiGps,
                                     public IRequires_Lmk20FmcClocks,
                                     public IRequires_PriClockDevice,
                                     public IRequires_FrameCalcDevice,
                                     public IRequires_LTC2494Device,
                                     public IRequires_Nothing
{
public:
    virtual ~ISuppliesSbcDuoRequirements()
        {}
};

//===========================================================================
//  CLASS IX6SubDevice -- X6 "SubDevice" Common Interface
//===========================================================================

class IX6SubDevice
{
public:
    IX6SubDevice()
        :  SD_Enabled(true)
        {}
    virtual ~IX6SubDevice()
        {}
    virtual void    ConstructCustomParts() {}
    virtual void    OpenHardware() = 0;
    virtual void    PreconfigureHardware() {}
    virtual void    ConfigureHardware() = 0;
    virtual void    StreamStopHardware() {}

    void  DeviceEnabled(bool state)
              { SD_Enabled = state;  }
    bool  DeviceEnabled() const
              { return SD_Enabled;  }
protected:
    bool           SD_Enabled;
    Trace          Debug;
};

//===========================================================================
//  CLASS IX6LogicVersionInfo -- User interface to X6 Version and Info
//===========================================================================

class IX6LogicVersionInfo
{
public:
    //
    //  Interface Functions
    virtual short           PciLogicRevision() const = 0;
    virtual short           PciLogicFamily() const = 0;    // Board Family (X6)

    virtual short           FpgaLogicVersion() const = 0;
    virtual short           FpgaHardwareVariant() const = 0;
    virtual short           PciLogicPcb() const = 0;       // Board Revision
    virtual short           PciLogicType() const = 0;      // Board Type
    virtual short           FpgaChipType() const = 0;

    virtual short           FpgaLogicSubrevision() const = 0;
    virtual short           BoardRevisionBuiltFor() const = 0;

    virtual unsigned int    PciLogicRawValue() const = 0;
    virtual unsigned int    FpgaInformationRawValue() const = 0;
    virtual unsigned int    FpgaSubRevRawValue() const = 0;
	
    //k7 special
    virtual short           K7FpgaLogicVersion() const = 0;
    virtual short           K7FpgaHardwareVariant() const = 0;
    virtual short           K7FpgaChipType() const = 0;
    virtual short           K7BoardHwType() const = 0;

    virtual struct tm       BitStreamDate() const = 0;
    virtual std::string     BitStreamDateString() const = 0;
    virtual std::string     BitStreamDateString_YMD() const = 0;
    virtual std::string     BitStreamDateString_HMS() const = 0;

    unsigned int            K7_FpgaTempGrade() const
								{  return PciLogicPcb() & 0x3;  }
	unsigned int            K7_FpgaSpeedGrade() const
								{  return (PciLogicPcb() & 0xc) >> 2;  }

    virtual ~IX6LogicVersionInfo() {}
};

//===========================================================================
//  CLASS IX6SioPllControl -- User interface to Sio Pll Control Functions
//===========================================================================

class IX6SioPllControl
{
public:
    //
    //  Interface Functions
    virtual Si5338 &  Pll() = 0;

    virtual void  ClockOutputEnable(bool state) = 0;
    virtual bool  ClockOutputEnable() = 0;

    virtual void  ExternalReference(bool state) = 0;
    virtual bool  ExternalReference() = 0;

    virtual ~IX6SioPllControl() {}
};

//===========================================================================
//  CLASS IX6ThermalDevice -- User interface to X6 Version and Info
//===========================================================================

class IX6ThermalDevice
{
public:
    //
    //  Interface Functions
    virtual int             LogicTemperature() const = 0;

    virtual int             LogicFanSetpoint() const = 0;
    virtual void            LogicFanSetpoint(int temp_deg_C) = 0;

    virtual int             LogicAlertSetpoint() const = 0;
    virtual void            LogicAlertSetpoint(int temp_deg_C) = 0;

    //  Control & Status methods
    virtual bool            MonitorEnable() const = 0;
    virtual void            MonitorEnable(bool state) = 0;

    virtual bool            CommFault() const = 0;
    virtual bool            SensorInReset() const = 0;
    virtual bool            SensorFault() const = 0;

    virtual void            Port(int /*addr*/, int /*value*/)
        {  }
    virtual int             Port(int /*addr*/)
        {  return 0;  }

    virtual int             Rpm()
        {  return 0;  }

    //  Reserved UI methods
    virtual void            LogicCriticalSetpoint(int temp_deg_C) = 0;

    virtual ~IX6ThermalDevice() {}
};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS IAuroraSfpTransceiver -- Interface to Aurora SPF Transceiver
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

class IAuroraSfpTransceiver
{
public:
    virtual ~IAuroraSfpTransceiver() {}

    virtual void  Enabled(bool state) = 0;
    virtual bool  Enabled() = 0;

    virtual void  Reset(bool state) = 0;
    virtual bool  Reset() = 0;

    virtual void  HiSpeedRx(bool state) = 0;
    virtual bool  HiSpeedRx() = 0;
    virtual void  HiSpeedTx(bool state) = 0;
    virtual bool  HiSpeedTx() = 0;

    virtual void  LowPower(bool state) = 0;
    virtual bool  LowPower() = 0;

    virtual bool  NoSignal() = 0;
    virtual bool  Fault() = 0;
    virtual bool  NoTransceiver() = 0;

};

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//  CLASS IAuroraSfp -- Interface to Aurora SPF
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

class IAuroraSfp
{
public:
    virtual ~IAuroraSfp() {}

    virtual bool  Enabled() = 0;
    virtual unsigned int TransCount() = 0;

    virtual IAuroraSfpTransceiver *  Trans(unsigned int idx) = 0;
    virtual I2CDeviceN &             I2CDev(unsigned int idx) = 0;

};


//===========================================================================
//  CLASS IX6Aurora -- User interface to Aurora Device
//===========================================================================

class IX6Aurora
{
public:
    enum LoopbackTypes { lbDisabled=0x0, lbParallel=0x1, lbSerial=0x2 };
    //
	//  Interface Functions
	virtual bool  LogicPresent() = 0;

	virtual void  InitTest(LoopbackTypes lb) = 0;
	virtual void  StartTest() = 0;
	virtual void  StartTestSingleBoard() = 0;
	virtual void  EndTest() = 0;
	virtual void  EnableChannels() = 0;

	// Multi-board functions are different from single board loopback
	virtual void  InitTestMB(LoopbackTypes lb) = 0;
	virtual void  StartTestMB() = 0;
	virtual void  EndTestMB() = 0;
	virtual void  StartTestTxMB() = 0;
	virtual void  StartTestRxMB() = 0;
	virtual void  EnableChannelsRxMB() = 0;
	virtual void  EnableChannelsTxMB() = 0;

	virtual void  Reset()=0;
	virtual void  Init(LoopbackTypes lb) = 0;
	virtual void  Start() = 0;
	virtual void  End() = 0;

	virtual bool         PllLocked() = 0;
    virtual bool         ChannelUp() = 0;
    virtual bool         HardError() = 0;
    virtual bool         SoftError() = 0;
    virtual bool         FrameError() = 0;
    virtual unsigned int ErrorCount() = 0;

    virtual IAuroraSfp * Sfp() = 0;

    virtual ~IX6Aurora() {}
};


//===========================================================================
//  CLASS IUsesX6Alerts -- X6 Alert User Interface
//===========================================================================

class IUsesX6Alerts
{
public:
    virtual ~IUsesX6Alerts()
        {  }

    enum AlertType { alertTimeStampRollover,  alertSoftware,
                     alertWarningTemperature,
                     alertPatternDone,
                     alertInputOverrange,     alertInputOverflow,
                     alertNumericError,
                     alertTrigger,
                     alertOutputOverrange,    alertOutputUnderflow,
                     alertVFifoDiagnostic
                     };

    virtual void            AlertEnable(AlertType alert, bool state) = 0;
    virtual void            AlertPassThru(AlertType alert, bool state) = 0;
    virtual void            AlertClear(AlertType alert) = 0;

    virtual void            AlertReset() = 0;
    virtual void            SoftwareAlert(unsigned int tag=0) = 0;
    virtual void            Append(const AlertData & alert) = 0;
    //
    // Events
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnTimeStampRolloverAlert;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnSoftwareAlert;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnWarningTemperature;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnPatternDoneAlert;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnInputOverrange;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnInputOverflow;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnNumericError;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnTrigger;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnOutputOverrange;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnOutputUnderflow;
    OpenWire::ThunkedEventHandler<AlertSignalEvent>   OnVFifoDiagnostic;
};

//==============================================================================
//  CLASS IX6ClockIo -- User interface to control X6 clock functionality
//==============================================================================

class IX6ClockIo
{
public:
    enum IIClockSource { csExternal, csInternal };
    enum IIReferenceSource { rsExternal, rsInternal };
    enum IIClockSelect { cslFrontPanel, cslP16, cslCarrier=cslP16 };

    virtual ~IX6ClockIo()  {  }

    //
    //  Interface Functions
    // Methods
    virtual double			    FrequencyActual() const = 0;

    // Properties
    virtual void    		    Frequency(double value) = 0;
    virtual double			    Frequency() const = 0;

    virtual ITimebaseRate & Adc() = 0;
    virtual ITimebaseRate & Dac() = 0;
    virtual ITimebaseRate & System() = 0;

    virtual void                Source(IIClockSource src) = 0;
    virtual IIClockSource       Source() const = 0;

    virtual void                ExternalClkSelect(IIClockSelect src) = 0;
    virtual IIClockSelect       ExternalClkSelect() const = 0;

    virtual void                Reference(IIReferenceSource /*src*/)  {  }
    virtual IIReferenceSource   Reference() const  {  return rsInternal;  }

    virtual void    		    ReferenceFrequency(double /*value*/)  {  }
    virtual double			    ReferenceFrequency() const  {  return 0.0;  }

    virtual bool                Locked()  const = 0;

};

//==============================================================================
//  CLASS IX6_I2C_uDevice -- Interface to I2C link support "microDevice"
//==============================================================================

class IX6_I2C_uDevice
{
public:
    virtual ~IX6_I2C_uDevice()  {  }

    virtual void                    Init() = 0;   // one time initialization

    virtual IRequires_I2C_RegIo &   Device() = 0;
    virtual void                    DeviceEnable(bool enabled) = 0;
};

//==============================================================================
//  CLASS IX6RawClockDevices -- Interface to standard "Raw" Clock Devices
//==============================================================================

class IX6RawClockDevices
{
public:
    virtual ~IX6RawClockDevices()  {  }

    virtual Si570 &       Vcxo() = 0;
    virtual Cdce72010 &   Pll() = 0;
};

//==============================================================================
//  CLASS IAd951xFactory -- Interface to builder of Ad651x timebase
//==============================================================================

class IAd951xFactory
{
public:
    virtual ~IAd951xFactory()  {  }

    virtual Ad951x_Base_Tb *   MakeAfe_Ad951xPllObject() = 0;
};

typedef IAd951xFactory IAd9516Factory;

//==============================================================================
//  CLASS IX6_Ad951x_RawClockDevices -- Interface to Ad951x "Raw" Clock Devices
//==============================================================================

class IX6_Ad951x_RawClockDevices
{
public:
    virtual ~IX6_Ad951x_RawClockDevices()  {  }

    virtual Ad951x_Base_Tb &   Pll() = 0;
};

//===========================================================================
//  CLASS IVeloPacketizer -- Velocia mode packetizer functions
//===========================================================================

class IVeloPacketizer
{
public:
    virtual ~IVeloPacketizer() {}

    virtual unsigned int  VeloChannels() = 0;

    virtual void  LoadAuxHeaderValue(unsigned int value) = 0;

    virtual void  VeloPacketId(unsigned int velo_ch, unsigned char pid) = 0;
    virtual unsigned char VeloPacketId(unsigned int velo_ch) = 0;

    virtual void  VeloPacketSize(unsigned int velo_ch, unsigned int full_size_ints) = 0;
    virtual unsigned int  VeloPacketSize(unsigned int velo_ch) = 0;

    virtual void  VeloDataSize(unsigned int velo_ch, unsigned int data_size_ints) = 0;
    virtual unsigned int  VeloDataSize(unsigned int velo_ch) = 0;

    virtual void  ForceVeloPacketSize(bool enable) = 0;
    virtual bool  ForceVeloPacketSize() = 0;

    virtual void  ForceVeloPacketSize(unsigned int velo_ch, bool enable) = 0;
    virtual bool  ForceVeloPacketSize(unsigned int velo_ch) = 0;

    virtual void   PacketIntervalPeriod(float period_ms) = 0;
    virtual float  PacketIntervalPeriod() = 0;

    virtual void  LoadAll_VeloPacketSize(unsigned int full_size_ints) = 0;
    virtual void  LoadAll_VeloDataSize(unsigned int data_size_ints) = 0;

    virtual void  SetStreamCount(unsigned int new_count) = 0;

    virtual void Custom_Tls_VeloPacketConfig(unsigned int size) = 0;
    virtual void Custom_Fft_VeloPacketConfig(unsigned int size) = 0;

    virtual void Enable(unsigned int velo_ch, bool state) = 0;

};

//===========================================================================
//  CLASS IVeloDepacketizer -- Velocia mode depacketizer functions
//===========================================================================

class IVeloDepacketizer
{
public:
    virtual ~IVeloDepacketizer() {}

    virtual void  TestLoopbackVeloPacketId(unsigned char pid) = 0;
    virtual unsigned char  TestLoopbackVeloPacketId() = 0;

    virtual void  VeloPacketId(unsigned int velo_ch, unsigned char pid) = 0;
    virtual unsigned char VeloPacketId(unsigned int velo_ch) = 0;

    virtual void  SetStreamCount(unsigned int new_count) = 0;
    virtual void  SetCommandStreamCount(unsigned int new_count) = 0;
    virtual void  SetDataStreamCount(unsigned int new_count) = 0;

    //  Support 'one pid' format for compatability
    virtual void  DataVeloPacketId(unsigned char pid)
                    {  VeloPacketId(0, pid);  }
    virtual unsigned char  DataVeloPacketId()
                    {  return VeloPacketId(0);  }

};

//===========================================================================
//  CLASS IVitaPacketizer -- Vita mode packetizer functions
//===========================================================================

class IVitaPacketizer
{
public:
    virtual ~IVitaPacketizer() {}

    virtual unsigned int  VitaStreams() = 0;

    virtual void  VitaStreamId(unsigned int vita_ch, unsigned int vitaStreamId) = 0;
    virtual unsigned int  VitaStreamId(unsigned int vita_ch) = 0;

    virtual void  VitaDestinationId(unsigned int vita_ch, unsigned int vitaDestId) = 0;
    virtual unsigned int  VitaDestinationId(unsigned int vita_ch) = 0;

    virtual void  VitaPacketSize(unsigned int vita_ch, unsigned int full_size_ints) = 0;
    virtual unsigned int  VitaPacketSize(unsigned int vita_ch) = 0;

    virtual void  VitaDataSize(unsigned int vita_ch, unsigned int data_size_ints) = 0;
    virtual unsigned int  VitaDataSize(unsigned int vita_ch) = 0;

};

//===========================================================================
//  CLASS IVitaDepacketizer -- Vita mode Depacketizer functions
//===========================================================================

class IVitaDepacketizer
{
public:
    virtual ~IVitaDepacketizer() {};

    virtual unsigned int  VitaStreams() = 0;

    virtual void  VitaStreamId(unsigned int vita_ch, unsigned int vitaStreamId) = 0;
    virtual unsigned int  VitaStreamId(unsigned int vita_ch) = 0;

    virtual void  VitaDestinationId(unsigned int vita_ch, unsigned int vitaStreamId) = 0;
    virtual unsigned int  VitaDestinationId(unsigned int vita_ch) = 0;

};


//===========================================================================
//  CLASS IVitaTimestamp -- VITA Timestamp Interface
//===========================================================================

class IVitaTimestamp
{
public:
    enum TimestampPpsModes { tsInternalTimer, tsPpsMode };
    enum TimestampIsModes { tsmUtc = 1, tsmGps, tsmElapsed };
    enum TimestampFsModes { tsfSamples = 1, tsfRealtime, tsfFreeRun };

    virtual ~IVitaTimestamp() {}

    virtual void  TimestampPpsMode(TimestampPpsModes mode) = 0;
    virtual TimestampPpsModes  TimestampPpsMode() = 0;

    virtual void  TimestampIsMode(TimestampIsModes mode) = 0;
    virtual TimestampIsModes  TimestampIsMode() = 0;

    virtual void  TimestampFsMode(TimestampFsModes mode) = 0;
    virtual TimestampFsModes  TimestampFsMode() = 0;

    virtual void  LoadTimestamp(unsigned int value) = 0;
    virtual void  TimestampCtrl(TimestampPpsModes, TimestampIsModes, TimestampFsModes) {};
};

//===========================================================================
//  CLASS IX6AnalogIoInfo  -- Common Configuration Interface
//===========================================================================

class IX6AnalogIoInfo
{
public:
    virtual ~IX6AnalogIoInfo()  {  }

    virtual float   MaxRate() const = 0;
    virtual float   MinRate() const = 0;

    virtual SpanInfo  Span() const  = 0;
    virtual unsigned int  Bits() const = 0;

    virtual unsigned int  SamplesPerWord() const = 0;
    virtual unsigned int  TriggerFrameGranularity() const = 0;

    //  Ranged Device Specific Info (with default impl)
    virtual size_t  GainRanges() const
                {  return 1;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }

    //  Clock Multiplier Factor (with default impl)
    virtual unsigned int ClockFactor() const
                {  return 1; }
};

//===========================================================================
//  CLASS IX6IoDevice  -- Common Configuration Interface
//===========================================================================

class IX6IoDevice
{
public:
    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essP16 };

    virtual ~IX6IoDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IX6IoDevice::ITriggerCfg  -- Trigger Configuration Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ITriggerCfg
    {
    public:
        virtual ~ITriggerCfg()  {  }

        virtual void          FramedMode(bool state) = 0;
        virtual bool          FramedMode() const = 0;

        virtual void          Edge(bool state) = 0;
        virtual bool          Edge() const = 0;

        virtual void          External(bool state) = 0;
        virtual bool          External() const = 0;

        virtual void          FrameSize(unsigned int samples) = 0;
        virtual unsigned int  FrameSize() const = 0;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src) = 0;
        virtual AfeExtSyncOptions  ExternalSyncSource() const = 0;
    };
    //
    //  Interface Functions
    virtual IX6AnalogIoInfo &  Info() = 0;

    virtual unsigned int  Channels() = 0;
    // virtual unsigned int  ActiveChannels() = 0;    // may not be needed

    virtual void  ChannelDisableAll() = 0;
    virtual void  ChannelEnabled(int ch, bool state) = 0;
    virtual bool  ChannelEnabled(int ch) const = 0;

    virtual void  ChannelPowered(int ch, bool state) = 0;
    virtual bool  ChannelPowered(int ch) const = 0;

    virtual ITriggerCfg &                   Trigger() = 0;
    virtual IPulseRepetitionIntervalIntf &  Pulse() = 0;

    virtual void SoftwareTrigger(bool state) = 0;
    virtual void SoftwareTrigger(AfeDirection which, bool state) = 0;

    virtual void TestModeEnabled(bool state, unsigned int mode) = 0;
    virtual void   TestFrequency(double frequency_hz) = 0;
    virtual double TestFrequency() = 0;

    virtual void  Decimation(unsigned int samples) = 0;
    virtual unsigned int Decimation() const = 0;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int Spi(unsigned int device, unsigned int address) const = 0;

};

//===========================================================================
//  CLASS IDacCalibrationRegister  -- Cal Register Interface
//===========================================================================

class IDacCalibrationRegister
{
public:

    virtual unsigned int  GetCalResults() = 0;
    virtual void          CalEnable(bool enabled) = 0;
    virtual unsigned int  GetShiftCount(int idx) = 0;

    virtual ~IDacCalibrationRegister()
        {}
};

//===========================================================================
//  CLASS IDacCalibrator  -- Interface to Dac Device Calibration Procedure
//===========================================================================

class IDacCalibrator
{
public:
    IDacCalibrator() : FCalibrationOk(false)
        {}
    virtual ~IDacCalibrator()
        {}

    virtual void  Calibrate() = 0;

    bool  CalibrationOk()
            {  return FCalibrationOk;  }

protected:
    bool  FCalibrationOk;

};

//=============================================================================
//  CLASS IIdRom  --  Interface to devices supporting IDROM pages
//=============================================================================

class IIdRom
{
public:

    virtual IIdRomDevice &   Device() = 0;

    //  Page access
    virtual X6SystemIdromPage &  System() = 0;
    virtual X6IdromPage &        AdcCal() = 0;
    virtual X6IdromPage &        DacCal() = 0;
    virtual X6IdromPage &        User() = 0;

    virtual ~IIdRom()
        {}

};

//=============================================================================
//  CLASS IFrameCalcDevice  --  Interface to Frame conversion calculator
//=============================================================================

class IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor() = 0;

    virtual ~IFrameCalcDevice()
        {}

};

//=============================================================================
//  CLASS ISelfMeasurement  --  Interface to ISelfMeasurement (LTC2494) chip
//=============================================================================

class ISelfMeasurement
{
public:
    virtual double         ReadTemperature() = 0;
    virtual double         ReadChannel(unsigned int ch) = 0;
    virtual void           CalculatePower() = 0;

    // instance specific data
    virtual unsigned int    MeasureCount()       {  return 16;  }   // number of defined measurements
    virtual unsigned int    PowerMeasureCount()  {  return  0;  }   // number of power measurements

    virtual double          GetScaleTable(unsigned int /*idx*/)  {  return 0; }
    virtual std::string     GetSignalName(unsigned int /*idx*/) { return ""; }
    virtual double          GetNominalV(unsigned int /*idx*/) {   return 0; }
    virtual double          GetMinV(unsigned int /*idx*/) {   return 0; }
    virtual double          GetMaxV(unsigned int /*idx*/) {   return 0; }
    virtual double          GetNominalAdcV(unsigned int /*idx*/) { return 0; }
    virtual std::string     GetSignalUnits(unsigned int /*idx*/) { return ""; }

    virtual double          GetCurrentPower(unsigned int power_ch) = 0;
    virtual double          GetCurrentMeasure(unsigned int ch) = 0;

    virtual ~ISelfMeasurement()
        {}

};

//=============================================================================
//  CLASS IPriClockDevice  --  Interface to access board clock for PRI
//=============================================================================

class IPriClockDevice
{
public:
    virtual double   ClockFrequency() = 0;
    virtual int      CalculatePriDivisor() = 0;

    virtual ~IPriClockDevice()
        {}

};

//===========================================================================
//  CLASS IX6_GpsIo -- X6 GPS User Interface
//===========================================================================

class IX6_GpsIo
{
public:
    virtual ~IX6_GpsIo()
        {}

    virtual IGps &   Gps() = 0;
    virtual bool     IsLocked() = 0;
    virtual bool     IsAntenna() = 0;
    //  ...Gps
    virtual void     GpsEnabled(bool state) = 0;
    virtual bool     GpsEnabled() const = 0;

    virtual void     ArmGpsTrigger() = 0;

};

//===========================================================================
//  CLASS IGPS_Callback -- X6 GPS Hardware Callback Interface
//===========================================================================

class IGPS_Callback
{
public:
    virtual ~IGPS_Callback()
        {}

    virtual void InterruptEnable(bool state) = 0;

};

//===========================================================================
//  CLASS IPatternModeSystem -- X6 Pattern Mode Control Interface
//===========================================================================

class IPatternModeSystem
{
public:
    enum PatternCommands { piPlay, piLoad, piReplay, piHalt, piMemoryReplay };
    enum PatternRepeatType { prFlatline, prPlayAgain };

    virtual ~IPatternModeSystem()
        {}

    virtual unsigned int  CalcVitaPatternSize(unsigned int RawDataSizeWords,
                                              unsigned int VitaPacketSize,
                                              unsigned int sid_count = 1) = 0;

    virtual void  PatternModeEnable(bool state) = 0;
    virtual bool  PatternModeEnable() = 0;

    virtual void  SendPatternInfo(unsigned int data_pid,
                                  char tag_value, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int count,
                                  PatternCommands cmd = piPlay,
                                  PatternRepeatType repeat = prPlayAgain ) = 0;
    virtual void  SendPatternInfo(unsigned int data_pid, std::vector<unsigned int> & sids,
                                  std::vector<char> & tags, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int count,
                                  PatternCommands cmd,
                                  PatternRepeatType repeat ) = 0;

    virtual unsigned int  Alignment() = 0;
    virtual unsigned int  ScaleFactor() = 0;

    // Memory test
    virtual void            MemoryTestEnable(unsigned int /*val*/) {}
    virtual unsigned int    GetMemoryStatus() {  return 0;  }
    virtual unsigned int    MemoryInit() {  return 0;  }
    virtual bool            DoMemoryTest() {  return false;  }


};

//===========================================================================
//  CLASS ICalibrationValueOverride -- X6 Calibration Val Overrides
//===========================================================================
//  This class provides functions to replace the Cal Gain and Offset values
//    with different ones for non-standard devices

class ICalibrationValueOverride
{
public:
    virtual ~ICalibrationValueOverride()
        {}
	virtual unsigned int OverrideOffset(unsigned int value) = 0;
	virtual unsigned int OverrideGain(unsigned int value) = 0;

	//  optionally customize the device update
	virtual bool OverrideOffsetUpdate() {  return false;  }
	virtual bool OverrideGainUpdate() {  return false;  }

	virtual void WriteOffset(unsigned int /*ch*/, unsigned int /*value*/) {    }
	virtual void WriteGain(unsigned int /*ch*/, unsigned int /*value*/) {    }
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
