// X6_1000MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X6_1000M_Mb.h>
#include <X6_1000MDeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <X6_1000MMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X6_MemoryMap;
using namespace X6_1000MMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

//
//  Rev 1 & 2 Logic Differences
//
//  Rev 1
const unsigned int  InputWidth_Bits         = 12;

//
//  Rev 2
const unsigned int  InputWidth_Bits_R2      = 16;

//
//  Common Device Count and Properties

const int    MaxInDevices      = 2;
const int    InChanPerDevice   = 1;
const int    MaxInChannels     = MaxInDevices*InChanPerDevice;

const int    MaxOutDevices     = 2;
const int    OutChanPerDevice  = 2;
const int    MaxOutChannels    = MaxOutDevices*OutChanPerDevice;

//  Rev 2 Bidir In/Out changes 
const int    R2b_MaxInDevices   = 1;
const int    R2b_MaxInChannels  = R2b_MaxInDevices*InChanPerDevice;

const int    R2b_MaxOutDevices  = 1;
const int    R2b_MaxOutChannels = R2b_MaxOutDevices*OutChanPerDevice;

//
//  Common 

const float         InputMaxRate            = 1000.0e6f;
const float         InputMinRate            = 100.0e6f;

const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 16;          // ????

const unsigned int X6_1000M_AdcSpiDevices       = 2;
const unsigned int X6_1000M_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int X6_1000M_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int X6_1000M_AdcCount            = MaxInChannels;  // Number of Adc Channels
const unsigned int X6_1000M_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations

const float         OutputMaxRate            = 1000.0e6f;
const float         OutputMinRate            = 0.0e6f;
const unsigned int  OutputWidth_Bits         = 16;

const          int  OutputMax_mV             = 1000;
const          int  OutputMin_mV             = -1000;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 16;

const unsigned int X6_1000M_DacSpiDevices       = MaxOutDevices;

const unsigned int X6_1000M_VeloOutStreamCount  = 1;    // one PID
const unsigned int X6_1000M_VitaOutStreamCount  = 2;    // one SID per Device

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romX6_1000M_SysInfoOffset        = 0;
const int romX6_1000M_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romX6_1000M_InputCoeffOffset       = 0;
const int romX6_1000M_InputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romX6_1000M_OutputCoeffOffset       = 0;
const int romX6_1000M_OutputCoeffOffset_avail = 9;  // 4 * 2 coefficients plus 'calibrated' word

const unsigned int X6_1000M_Adc_One = 0x10000;    // representation of floating point 1.0
const unsigned int X6_1000M_Dac_One = 0x10000;    // representation of floating point 1.0

//===========================================================================
// Class X6_1000M_AnalogInInfo
//===========================================================================

class X6_1000M_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    X6_1000M_AnalogInInfo(X6_1000MDeviceMap * owner)
         : Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~X6_1000M_AnalogInInfo()  {  }

    virtual float     MaxRate() const
        {
            return InputMaxRate;
        }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
	virtual unsigned int  Bits() const
        {
            return InputWidth_Bits;
        }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    X6_1000MDeviceMap * Owner;
    SpanInfo   FSpan;
};

//===========================================================================
// Class X6_1000M_R2_AnalogInInfo
//===========================================================================

class X6_1000M_R2_AnalogInInfo : public X6_1000M_AnalogInInfo
{
public:
    X6_1000M_R2_AnalogInInfo(X6_1000MDeviceMap * owner)
         : X6_1000M_AnalogInInfo(owner)
        {   }
    virtual ~X6_1000M_R2_AnalogInInfo()  {  }

    virtual unsigned int  Bits() const
        {
            return InputWidth_Bits_R2;
        }

};

//===========================================================================
// Class X6_1000M_AnalogOutInfo
//===========================================================================

class X6_1000M_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    X6_1000M_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~X6_1000M_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS X6AFE_X6_1000MAdcCalibrator -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AFE_X6_1000MAdcCalibrator
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_X6_1000MAdcCalibrator
//---------------------------------------------------------------------------

X6AFE_X6_1000MAdcCalibrator::X6AFE_X6_1000MAdcCalibrator(IAddressingSpace & space, MapRegisters & regs)
    :  CalControlReg(space, regs.CalControl_Addr),
       Adc0_Result_Reg(space, regs.Adc0_Status_Addr),
       Adc1_Result_Reg(space, regs.Adc1_Status_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_X6_1000MAdcCalibrator
//---------------------------------------------------------------------------

X6AFE_X6_1000MAdcCalibrator::~X6AFE_X6_1000MAdcCalibrator()
{
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcCalibrator::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcCalibrator::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcCalibrator::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcCalibrator::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcCalibrator::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcCalibrator::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Calibrate
    Calibrate();
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcCalibrator::Calibrate()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcCalibrator::Calibrate(bool skip_cal)
{
    ScopedTraceRef ccc(Debug, "X6AFE_X6_1000MAdcCalibrator::Calibrate");
    //Debug.Log("Cal Done: ", CalControlReg.CalDone.Value() );

    CalControlReg.SkipPhyCal.Value( skip_cal );
    CalControlReg.PhyInit.Value( false );
    CalControlReg.PhyInit.Value( true );
    CalControlReg.PhyInit.Value( false );

    Sleep(1);  //  1 ms  wait for calibration
}

//===========================================================================
//  CLASS X6AFE_X6_1000MAdcAmplitudeControl -- X6 ADC Amplitude "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AFE_X6_1000MAdcAmplitudeControl
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_X6_1000MAdcAmplitudeControl
//---------------------------------------------------------------------------

X6AFE_X6_1000MAdcAmplitudeControl::X6AFE_X6_1000MAdcAmplitudeControl(IAddressingSpace & space, MapRegisters & regs)
    :  AmplControlReg(space, regs.AmplitudeControl_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_X6_1000MAdcAmplitudeControl
//---------------------------------------------------------------------------

X6AFE_X6_1000MAdcAmplitudeControl::~X6AFE_X6_1000MAdcAmplitudeControl()
{
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcAmplitudeControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcAmplitudeControl::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcAmplitudeControl::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcAmplitudeControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcAmplitudeControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcAmplitudeControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    //  No default behavior

}

//---------------------------------------------------------------------------
//  X6AFE_X6_1000MAdcAmplitudeControl::ShiftInput()
//---------------------------------------------------------------------------

void  X6AFE_X6_1000MAdcAmplitudeControl::ShiftInput(bool shift_it)
{
    ScopedTraceRef ccc(Debug, "X6AFE_X6_1000MAdcCalibrator::ShiftInput");

    if (shift_it)
        AmplControlReg.DevShiftOn.Value(0x3);
    else
        AmplControlReg.DevShiftOn.Value(0x0);

    Debug.Log("ADC Shift Val: ", AmplControlReg.DevShiftOn.Value() );
}

//===========================================================================
//  CLASS X6_1000MInputDevice  -- User Interface for X6-1000M ADCs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000MInputDevice
//---------------------------------------------------------------------------

X6_1000MInputDevice::X6_1000MInputDevice(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romX6_1000M_InputCoeffOffset, MaxInChannels,
                       X6_1000M_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000MInputDevice
//---------------------------------------------------------------------------

X6_1000MInputDevice::~X6_1000MInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  X6_1000MInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  X6_1000MInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X6_1000MInputDevice::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------

unsigned int  X6_1000MInputDevice::Channels()
{
    return MaxInChannels;
}

//===========================================================================
//  CLASS X6_1000MInputDeviceR2bidir  -- User Interface for X6-1000M ADCs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000MInputDeviceR2bidir
//---------------------------------------------------------------------------

X6_1000MInputDeviceR2bidir::X6_1000MInputDeviceR2bidir(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner, info)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000MInputDeviceR2bidir
//---------------------------------------------------------------------------

X6_1000MInputDeviceR2bidir::~X6_1000MInputDeviceR2bidir()
{
}

//---------------------------------------------------------------------------
//  X6_1000MInputDeviceR2bidir::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------

unsigned int  X6_1000MInputDeviceR2bidir::Channels()
{
    return R2b_MaxInChannels;   // only one device usable at full rate
}

//===========================================================================
//  CLASS X6_1000MDacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class X6_1000MDacToHardware
//---------------------------------------------------------------------------

class X6_1000MDacToHardware :  public TiDac5682::IDacToHardware
{
public:
    virtual unsigned short  Spi(int addr)
        {  return Out->Spi(Dev, addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Out->Spi(Dev, addr, value);  }

    virtual double          DCLK(bool single_channel)
        {  return MyPll().DCLK(single_channel, MyPll().Frequency()) ;  }
    virtual bool            IsDacDllEnabled(bool single_channel)
        {  return MyPll().IsDacDllEnabled(single_channel);  }
    virtual unsigned int    FastConfigValue()
        {  return 0xC0;  }

    X6_1000MDacToHardware( X6_1000MOutputDevice * out, unsigned int dev=0 )
        : Out(out), Dev(dev)
        {}
    virtual ~X6_1000MDacToHardware()
        {}

    X6_1000M_Pll &  MyPll()
        {  return dynamic_cast<X6_1000M_Pll &>(Out->DOwner->RawClockDevice().Pll());  }

    X6_1000MOutputDevice *  Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS X6_1000MOutputDevice  -- User Interface for X6-1000M DACs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000MOutputDevice
//---------------------------------------------------------------------------

X6_1000MOutputDevice::X6_1000MOutputDevice(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner),
      FInfo(info),
      DacDev_0(new X6_1000MDacToHardware(this, 0)),
      DacDev_1(new X6_1000MDacToHardware(this, 1)),
      CalRomSection(DacCalIdrom(), romX6_1000M_OutputCoeffOffset, MaxOutChannels,
                       X6_1000M_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000MOutputDevice
//---------------------------------------------------------------------------

X6_1000MOutputDevice::~X6_1000MOutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  X6_1000MOutputDevice::PreconfigureHardware()
{
    // Set Dac Control Reg back?

    DacDev_0.PreconfigHardware();
    DacDev_1.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  X6_1000MOutputDevice::ConfigureHardware()
{
    DacDev_0.ClearDacTestStatus();
    DacDev_1.ClearDacTestStatus();
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  X6_1000MOutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDevice::Channels() -- Number of enable-able DAC channels
//---------------------------------------------------------------------------

unsigned int  X6_1000MOutputDevice::Channels()
{
    return MaxOutChannels;
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDevice::ActiveDevices() -- Number of DAC devices in use
//---------------------------------------------------------------------------

unsigned int  X6_1000MOutputDevice::ActiveDevices()
{
    unsigned int active_channels = ActiveChannels();
    if (active_channels==4)
        return 2;
    else if (active_channels==1)
        return 1;
    else if (active_channels==2)
        {
        // two active channels...
        if ( (ChannelEnabled(0) ^ ChannelEnabled(1)) &&
             (ChannelEnabled(2) ^ ChannelEnabled(3)) )
            return 2;
        else
            return 1;
        }
    else //  if (active_channels==0)   (3 is illegal)
        return 0;
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  X6_1000MOutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    Device(0).Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1));
    Device(0).ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device(0).ChannelEnabled(1, DacCommon().ChannelEnabled(1) );

    Device(1).Enabled( DacCommon().ChannelEnabled(2) || DacCommon().ChannelEnabled(3));
    Device(1).ChannelEnabled(0, DacCommon().ChannelEnabled(2) );
    Device(1).ChannelEnabled(1, DacCommon().ChannelEnabled(3) );
}


//===========================================================================
//  CLASS X6_1000MOutputDeviceR2bidir  -- User Interface for X6-1000M DACs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000MOutputDeviceR2bidir
//---------------------------------------------------------------------------

X6_1000MOutputDeviceR2bidir::X6_1000MOutputDeviceR2bidir(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner, info)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000MOutputDeviceR2bidir
//---------------------------------------------------------------------------

X6_1000MOutputDeviceR2bidir::~X6_1000MOutputDeviceR2bidir()
{
}

//---------------------------------------------------------------------------
//  X6_1000MOutputDeviceR2bidir::Channels() -- Number of enable-able DAC channels
//---------------------------------------------------------------------------

unsigned int  X6_1000MOutputDeviceR2bidir::Channels()
{
    return R2b_MaxOutChannels;   // only one device usable at full rate
}

//=============================================================================
//  CLASS X6_1000M_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_1000M_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_1000M_Pll
//---------------------------------------------------------------------------

X6_1000M_Pll::X6_1000M_Pll(X6_1000MDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner), DOps( owner->GetDacOperations() ), FPDF(100.0e3)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 1 as guideline output
    //SetMasterOutputSelection(3);  // Use Output 3 as guideline output  DACONLY TEST

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000M_Pll
//---------------------------------------------------------------------------

X6_1000M_Pll::~X6_1000M_Pll()
{
    delete DOps;  DOps = 0;
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_1000M_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "X6_1000M_Pll::OnInit()");

    //  Device Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         true,   // Secondary Bias ON
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       false,  // Input Term NOT disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, true);   SetOutputConfig(0, ocLvPecl);   // Adc 0
    OutputInUse(1, true);   SetOutputConfig(1, ocLvPecl);   // Adc 1
    OutputInUse(2, true);   SetOutputConfig(2, ocLvds);     // FPGA
    OutputInUse(3, true);   SetOutputConfig(3, ocLvPecl);   // Dac Dev0
    OutputInUse(4, true);   SetOutputConfig(4, ocLvPecl);   // Dac Dev1
    OutputInUse(5, true);   SetOutputConfig(5, ocLvds);     // FPGA
    OutputInUse(6, true);   SetOutputConfig(6, ocLvds);     // FPGA
    OutputInUse(7, true);   SetOutputConfig(7, ocLvds);     // FPGA
    OutputInUse(8, false);  SetOutputConfig(8, ocDisabled);
    OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);


    SetHighSwingLvpecl(0, true);    // Make Pecls Hi Swing
    SetHighSwingLvpecl(1, true);
    SetHighSwingLvds  (2, true);
    SetHighSwingLvpecl(3, true);
    SetHighSwingLvpecl(4, true);
    SetHighSwing      (5, true);
    SetHighSwing      (6, true);
    SetHighSwingLvds  (7, true);

    //  Phase = 0
    SetPhaseAdjust(0, 1, 0);  SetPhaseAdjust(1, 1, 0);  // Set all outputs to 0 phase
    SetPhaseAdjust(2, 1, 0);  SetPhaseAdjust(3, 1, 0);
    SetPhaseAdjust(4, 1, 0);  SetPhaseAdjust(5, 1, 0);
    SetPhaseAdjust(6, 1, 0);  SetPhaseAdjust(7, 1, 0);

    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    float input_max_rate = static_cast<float>(InputMaxRate());
    float output_max_rate = static_cast<float>(OutputMaxRate());
    OutputFrequencyLimit[0] = input_max_rate;  // as in ADCs, but halved (can vary)
    OutputFrequencyLimit[1] = input_max_rate;
    OutputFrequencyLimit[2] = input_max_rate/4.0f;
    OutputFrequencyLimit[3] = output_max_rate;
    OutputFrequencyLimit[4] = output_max_rate;       
    OutputFrequencyLimit[5] = output_max_rate/2.0f;  
    OutputFrequencyLimit[6] = output_max_rate/2.0f; 
    OutputFrequencyLimit[7] = output_max_rate/2.0f;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use external pin
    ReferenceSource(refPrimary);    // Set to use primary reference

}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  X6_1000M_Pll::Vcxo()
{
    return Owner->AFEClk_SDev.Vcxo();
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::OnReapply()
//---------------------------------------------------------------------------

void  X6_1000M_Pll::OnReapply()
{
    if (Owner->Input().ActiveChannels()==0)
        {
        // DAC ONLY TEST
        OutputEnabled(0, false);
        OutputEnabled(1, false);
        OutputEnabled(2, false);
        }
    else
        {
        OutputEnabled(0, true);
        OutputEnabled(1, true);
        OutputEnabled(2, true);
        }
    if (Owner->Output().ActiveChannels()==0)
        {
        // ADC ONLY TEST
        OutputEnabled(3, false);
        OutputEnabled(4, false);
        OutputEnabled(5, false);
        OutputEnabled(6, false);
        OutputEnabled(7, false);
        }
    else
        {
        OutputEnabled(3, true);
        OutputEnabled(4, true);
        OutputEnabled(5, true);
        OutputEnabled(6, true);
        OutputEnabled(7, true);
        }

}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  X6_1000M_Pll::AdcMasterOutput()
{
    return 1;
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  X6_1000M_Pll::DacMasterOutput()
{
    return 3;
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  X6_1000M_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(0, ADCCLK(adc_rate));
    OutputFrequency(1, ADCCLK(adc_rate));
    OutputFrequency(2, ADCCLK(adc_rate)/4);
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  X6_1000M_Pll::SetDacOutputRates(double dac_rate)
{
    bool  single_channel = DOps->IsDacInSingleChannelMode();

    OutputFrequency(3, dac_rate);
    OutputFrequency(4, dac_rate);
    OutputFrequency(5, DCLK(single_channel, dac_rate)/2);
    OutputFrequency(6, DCLK(single_channel, dac_rate));
    OutputFrequency(7, DCLK(single_channel, dac_rate));

}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::DCLK()  -- DCLK Output Rate
//---------------------------------------------------------------------------

double X6_1000M_Pll::DCLK(bool single_channel, double freq)
{
    double FreqMhz = freq/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return FreqMhz/2 * 1.0e6;
        else
            return FreqMhz * 1.0e6;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return FreqMhz * 1.0e6;
        else
            return FreqMhz*2 * 1.0e6;
        }
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::ADCCLK()  -- ADCCLK Output Rate
//---------------------------------------------------------------------------

double X6_1000M_Pll::ADCCLK(double freq)
{
    if (freq>InputMaxRate())
        return InputMaxRate();
    else
        return freq;
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::IsDacDllEnabled()  -- Output Divider Settings
//---------------------------------------------------------------------------

bool  X6_1000M_Pll::IsDacDllEnabled(bool single_channel)
{
    double FreqMhz = Dac().Frequency()/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return true;
        else
            return false;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return true;
        else
            return false;
        }
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::InputMaxRate()  -- Return Max Input Rate
//---------------------------------------------------------------------------

double  X6_1000M_Pll::InputMaxRate()
{
    return Owner->Input().Info().MaxRate();
}

//---------------------------------------------------------------------------
//  X6_1000M_Pll::OutputMaxRate()  -- Return Max Output Rate
//---------------------------------------------------------------------------

double  X6_1000M_Pll::OutputMaxRate()
{
    return Owner->Output().Info().MaxRate();
}

//=============================================================================
//  CLASS X6_1000M_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_1000M_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_1000M_SioPll
//---------------------------------------------------------------------------

X6_1000M_SioPll::X6_1000M_SioPll(X6_1000MDeviceMap * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  X6_1000M_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_1000M_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, false);    SetMultiSynthPowerDown(3, false);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);    
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(2, diNoClk);   // override since not using output

    OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
    OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, true);

    SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
    SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
    SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
    SetOutputVoltage(3, vo_2p5volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  X6_1000M_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  X6_1000M_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS X6_1000M_DacOperations  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of class X6_1000M_DacOperations
//---------------------------------------------------------------------------

class X6_1000M_DacOperations :  public X6_1000M_Pll::IDacOperations
{
public:
    X6_1000M_DacOperations( X6_1000MDeviceMap * owner )
        :  Owner(owner)
        {}

    virtual bool  IsDacInSingleChannelMode();
    
private:
    X6_1000MDeviceMap * Owner;
};

//---------------------------------------------------------------------------
//  X6_1000M_DacOperations::IsDacInSingleChannelMode()
//---------------------------------------------------------------------------

bool  X6_1000M_DacOperations::IsDacInSingleChannelMode()
{
    bool dac0en = Owner->Output().ChannelEnabled(0);
    bool dac1en = Owner->Output().ChannelEnabled(1);

    bool dac2en = Owner->Output().ChannelEnabled(2);
    bool dac3en = Owner->Output().ChannelEnabled(3);

    // this works because mixed modes are only allowed if a device is off..
    //   that is, no 3 channel modes?

    return (dac0en ^ dac1en) || (dac2en ^ dac3en);
}

//===========================================================================
//  CLASS X6_1000M_DacCalToHdw -- Cal Hdw Interface
//===========================================================================

class X6_1000M_DacCalToHdw : public TiDac5682::IDacCalibratorToHdw
{
public:

    virtual void  ApplyDacGainCoefficients()
        {
        Output().Cal().ApplyRomCoefficients();  // will apply default if uncalibrated
        }
    virtual void  DacReset()
        {
        Output().DacCommon().ChannelResetAll();
        }
    virtual void  PreconfigureHardware()
        {
        // Reapply settings for DAC Devices for test
        Output().PreconfigureHardware();
        }
    virtual double        ClockFrequencyActualMhz()
        {  return Owner->Clock().Dac().FrequencyActual()/1e6; }
    virtual void          SetTestMode_Alternating()
        {  PatternCache = 0x6; }
    virtual void          SetTestMode_DoubleAlternating()
        {  PatternCache = 0x7; }
    virtual void          SetTestEnable(bool state)
        {  Output().Common().TestModeEnabled(X6AFE_Common::tOutput, state, PatternCache);  }
    virtual void          SetSoftwareUnframedTrigger()
        {
        Output().DacCommon().EdgeTrigger(false);
        Output().DacCommon().ExternalTrigger(false);
        Output().DacCommon().FramedTriggerEnabled(false);
        }
    virtual void  DeviceRun(bool state, EngDeviceState /*which*/)
        {   Output().Common().Engine(X6AFE_Common::tOutput, state);  }

    virtual TiDac5682::DacDevice &   Dac(unsigned int idx)
        {  return Output().Device(idx);  }


    X6_1000M_DacCalToHdw( X6_1000MDeviceMap * owner)
        :  Owner(owner), PatternCache(0)
        {}
    virtual ~X6_1000M_DacCalToHdw()
        {}

private:
    X6_1000MDeviceMap * Owner;
    unsigned int       PatternCache;

    X6_1000MOutputDevice & Output()  {  return Owner->Output(); }
};

//=============================================================================
//  CLASS X6_1000MDacPriClockDevice  --  Interface to access board clock for PRI
//=============================================================================
//  The calc is the save for both divisors, so lets us the same impl

class X6_1000MDacPriClockDevice : public IPriClockDevice, public IFrameCalcDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->AFEClk_SDev.Clock().Frequency();  }
    virtual int   CalculatePriDivisor()
        {
            // Register divisor is a Single Channel and DacDll Mode
            int divisor;
            if (IsDacSingleChannel())
                if (IsDacDllEnabled())
                    divisor = 4;
                else
                    divisor = 2;
            else
                if (IsDacDllEnabled())
                    divisor = 2;
                else
                    divisor = 1;

            return divisor;
        }
    virtual  unsigned int  CalculateFrameDivisor()
        {  return static_cast<unsigned int>(CalculatePriDivisor());  }

    virtual bool IsDacSingleChannel()
        {
        bool dac0en = Owner->Output().ChannelEnabled(0);
        bool dac1en = Owner->Output().ChannelEnabled(1);

        bool dac2en = Owner->Output().ChannelEnabled(2);
        bool dac3en = Owner->Output().ChannelEnabled(3);

        // this works because mixed modes are only allowed if a device is off..
        //   that is, no 3 channel modes?

        return (dac0en ^ dac1en) || (dac2en ^ dac3en);
        }

    virtual bool IsDacDllEnabled()
        {
        return MyPll().IsDacDllEnabled(IsDacSingleChannel());
        }

    X6_1000MDacPriClockDevice(X6_1000MDeviceMap * owner)
        :  Owner(owner)
        {}
    virtual ~X6_1000MDacPriClockDevice()
        {}

private:
    X6_1000MDeviceMap * Owner;

    X6_1000M_Pll &  MyPll()
        {  return dynamic_cast<X6_1000M_Pll &>(Owner->RawClockDevice().Pll());  }

};

//=============================================================================
//  CLASS X6_1000MAdcCalOverride  --  Cal Override for X61000 ADCs
//=============================================================================

class X6_1000MAdcCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    X6_1000MAdcCalOverride()
        {}
    virtual ~X6_1000MAdcCalOverride()
        {}

private:

};

//---------------------------------------------------------------------------
//  X6_1000MAdcCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  X6_1000MAdcCalOverride::OverrideOffset(unsigned int value)
{
    int offset = static_cast<int>(value);   // need signed offset
    //  Max offset that can be calibrated by the hardware (120uVsteps)
    int offset_cal_range_counts = static_cast<int>(30.72/((float)InputMax_mV/(1<<(InputWidth_Bits-1))));
    //  Trim range to fit hardware limitations
    if (offset>0 && offset>offset_cal_range_counts)
        offset = offset_cal_range_counts;
    else if (offset<0 && offset>offset_cal_range_counts)
        offset = -offset_cal_range_counts;
    //
    //  Convert 'in range' offset to acutal units of 1/4 LSB, unipolar mode
    offset +=  offset_cal_range_counts;
    offset <<= 2;

    return static_cast<unsigned int>(offset);
}

//---------------------------------------------------------------------------
//  X6_1000MAdcCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  X6_1000MAdcCalOverride::OverrideGain(unsigned int value)
{
    const float HdwStep_mV = 0.120f;

    float gain = static_cast<float>(value)/X6_1000M_Adc_One;
    if (gain<=1.0)
        return 0;   // no gain shift

    float full_range_mV =  2.0*InputMax_mV;
    float adjustment_mV =  full_range_mV*(1.0f-1.0f/gain);
    unsigned int hdw_steps = static_cast<unsigned int>(adjustment_mV/HdwStep_mV);
    return hdw_steps;
}

//===========================================================================
//  CLASS X6_1000MDeviceMap  -- Register & low level support for X6-1000M
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_1000MDeviceMap
//---------------------------------------------------------------------------

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)

static Cdce72010Registers X6_1000M_PllRegisters =
    { mmAFE_PllControl,  mmAFE_PllSpi  };

static X6_Std_I2CLinkage::MapRegisters X6_1000M_VcoI2CLinkRegAddrs =
    {  mmAFE_VcxoControl,      //  AfeVcxoControl_Addr
    };

static X6AFE_X6_1000MAdcCalibrator::MapRegisters X6_1000M_AdcCalRegAddrs =
    { mmAFE_1000M_AdcCalControl,    mmAFE_1000M_Adc0_Status,
      mmAFE_1000M_Adc1_Status   };

static X6AFE_X6_1000MAdcAmplitudeControl::MapRegisters X6_1000M_AmplitudeRegAddrs =
    {  mmAFE_1000M_Adc_Amplitude,      //  AmplitudeControl_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class X6_1000MDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X6_1000MDeviceMap::X6_1000MDeviceMap(X6Board * owner)
    :  X6DeviceMap(owner),
       AFE1000MAdcCal_SDev( WB_AFE, X6_1000M_AdcCalRegAddrs),
       AFE1000MAmplitude_SDev( WB_AFE, X6_1000M_AmplitudeRegAddrs),
       FInput(new X6_1000MInputDevice(this, new X6_1000M_AnalogInInfo(this))),
       FOutput(new X6_1000MOutputDevice(this, new X6_1000M_AnalogOutInfo)),
       Owner(owner)
{
    //
    //  SubDevice Customization
    LocalConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &X6_1000MDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &X6_1000MDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X6_1000MDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X6_1000MDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X6_1000MDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X6_1000MDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X6_1000MDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_1000MDeviceMap
//---------------------------------------------------------------------------

X6_1000MDeviceMap::~X6_1000MDeviceMap()
{
    delete FInput;
    FInput = 0;

    delete FOutput;
    FOutput = 0;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------
//  We can't make this part of virtual tree for ConstructCustomParts since
//   its being called in the constructor and the tree is not built yet.

void  X6_1000MDeviceMap::LocalConstructCustomParts()
{
    X6DeviceMap::ConstructCustomParts();

    AFE1000MAdcCal_SDev.ConstructCustomParts();
    AFE1000MAmplitude_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OpenHardware()
{
    X6DeviceMap::OpenHardware();

    AFE1000MAdcCal_SDev.OpenHardware();
    AFE1000MAmplitude_SDev.OpenHardware();

    FInput->Cal().ApplyRomCoefficients();
    FOutput->Cal().ApplyRomCoefficients();

    const int  X6_1000M_PatternModeOnValue =  0x3;
    const int  X6_1000M_PatternModeOffValue = 0x0;

    DacPatternMode_SDev.DefineModeEnableValues(X6_1000M_PatternModeOnValue,
                                               X6_1000M_PatternModeOffValue);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X6_1000MDeviceMap::OnStreamPreconfigHandler()");
    Output().ConfigureChannelEnables();

    FInput->Cal().ApplyRomCoefficients();
    FOutput->Cal().ApplyRomCoefficients();

    // Family-common init
    X6DeviceMap::PreconfigureHardware();
    AFE1000MAdcCal_SDev.PreconfigureHardware();
    AFE1000MAmplitude_SDev.PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X6_1000MDeviceMap::OnStreamConfigureHandler()");
    Output().ConfigureChannelEnables();   // remove when preconfig readied...

    Output().ConfigureHardware();

    // Family-common init
    X6DeviceMap::ConfigureHardware();
    AFE1000MAdcCal_SDev.ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    if (Input().ActiveChannels()==0)
        AFECommon_SDev.Engine(X6AFE_Common::tOutput, true);
    else if (Output().ActiveChannels()==0)
        AFECommon_SDev.Engine(X6AFE_Common::tInput, true);
    else
        AFECommon_SDev.Engine(X6AFE_Common::tInputOutput, true);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Base class "on stream stop" behavior
    X6DeviceMap::StreamStopHardware();
    AFE1000MAdcCal_SDev.StreamStopHardware();
    AFE1000MAmplitude_SDev.StreamStopHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeAfePllObject() -- Reference to input object
//---------------------------------------------------------------------------

Cdce72010 *  X6_1000MDeviceMap::MakeAfePllObject()
{
    //  Make our local PLL object (subdev will delete it)
    return new X6_1000M_Pll(this, WB_AFE, X6_1000M_PllRegisters );
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeAfeVcxoObject() --  Build custom VCO object
//---------------------------------------------------------------------------

Si570 *  X6_1000MDeviceMap::MakeAfeVcxoObject(IRequires_I2C_RegIo & device)
{
    //  Make our local Vcxo object (subdev will delete it)
    return new Si570(device, Si570::tcA, kVcxoPowerOnRate );
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeAfeVcxoI2CLink() --  Build custom Vco I2C Link
//---------------------------------------------------------------------------

IX6_I2C_uDevice *  X6_1000MDeviceMap::MakeAfeVcxoI2CLink()
{
    //  Make our local Vcxo I2C Link (subdev will delete it)
    return new X6_Std_I2CLinkage(WB_AFE, X6_1000M_VcoI2CLinkRegAddrs);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeAfeAdcSpiDevices() -- Build ADC Spi Devices
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::MakeAfeAdcSpiDevices()
{
    //  Make one SPI device for each ADC device
    unsigned int Write_Addr = mmAFE_AdcSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_AdcSpiStatBase;

    for (unsigned int i=0; i<X6_1000M_AdcSpiDevices; ++i)
        {
        AFEAdcSpi_SDev.MakeSpiDevice("ADC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_AdcSpiCtrlDelta;
        Read_Addr  += mmAFE_AdcSpiStatDelta;
        }
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeAfeDacSpiDevices() -- Build DAC Spi Devices
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap::MakeAfeDacSpiDevices()
{
    //  Make one SPI device for each DAC device
    unsigned int Write_Addr = mmAFE_DacSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_DacSpiStatBase;

    for (unsigned int i=0; i<X6_1000M_DacSpiDevices; ++i)
        {
        AFEDacSpi_SDev.MakeSpiDevice("DAC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_DacSpiCtrlDelta;
        Read_Addr  += mmAFE_DacSpiStatDelta;
        }
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  X6_1000MDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new X6_1000M_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_1000MDeviceMap::GetVeloInStreamCount()
{
    return X6_1000M_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_1000MDeviceMap::GetVeloOutStreamCount()
{
    return X6_1000M_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  X6_1000MDeviceMap::GetVitaInStreamCount()
{
    return X6_1000M_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetVitaOutStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  X6_1000MDeviceMap::GetVitaOutStreamCount()
{
    return X6_1000M_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_1000MDeviceMap::GetCalibrationAdcCount()
{
    return X6_1000M_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_1000MDeviceMap::GetCalibrationDacCount()
{
    return MaxOutChannels;  //X6_1000M_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_1000MDeviceMap::GetAdcCalibrationOverride()
{
    return new X6_1000MAdcCalOverride;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_1000MDeviceMap::GetDacCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetDacLatencyCalibrator() -- Get Latency Cal object
//---------------------------------------------------------------------------

IDacCalibrator *  X6_1000MDeviceMap::GetDacLatencyCalibrator()
{
    IDacCalibrationRegister *  CalReg = AFEDacLatencyCal_SDev.GetLatencyRegisterIntf();
    TiDac5682::IDacCalibratorToHdw * CtoH = new X6_1000M_DacCalToHdw(this);

    return new TiDac5682::DualDacCalibrator(CalReg, CtoH);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  X6_1000MDeviceMap::MakePriClockDevice(PriDeviceCode which)
{
    if (which==priAdc)
        return new X6StandardPriClockDevice(this, 4);
    else
        return new X6_1000MDacPriClockDevice(this);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  X6_1000MDeviceMap::MakeFrameCalcDevice(FrameDeviceCode which)
{
    if (which==devAdc)
        return new X6StandardFrameCalcDevice(this, 4);
    else
        return new X6_1000MDacPriClockDevice(this);  // does double duty!
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   X6_1000MDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   X6_1000MDeviceMap::GetPatternModeAlignment()
{
    return 8;   // standard X6 alignment of 8 words
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   X6_1000MDeviceMap::GetPatternModeScaleFactor()
{
    return 8;   // standard X6 scale factor of 8 words
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   X6_1000MDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return false;  // dual memory
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::GetDacOperations() -- Return Interface for PLLs to DACs
//---------------------------------------------------------------------------

X6_1000M_Pll::IDacOperations *  X6_1000MDeviceMap::GetDacOperations()
{
    return new X6_1000M_DacOperations(this);
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::Input() -- Reference to Input object
//---------------------------------------------------------------------------

X6_1000MInputDevice & X6_1000MDeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap::Output() -- Reference to Output object
//---------------------------------------------------------------------------

X6_1000MOutputDevice & X6_1000MDeviceMap::Output()
{
    return *FOutput;
}

//===========================================================================
//  CLASS X6_1000MDeviceMap_R1  -- Revision 1 Logic
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_1000MDeviceMap_R1
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6_1000MDeviceMap_R1
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X6_1000MDeviceMap_R1::X6_1000MDeviceMap_R1(X6_1000M * owner)
    :  X6_1000MDeviceMap(owner)
{
    FInput = new X6_1000MInputDevice(this, new X6_1000M_AnalogInInfo(this));
    FOutput= new X6_1000MOutputDevice(this, new X6_1000M_AnalogOutInfo);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_1000MDeviceMap_R1
//---------------------------------------------------------------------------

X6_1000MDeviceMap_R1::~X6_1000MDeviceMap_R1()
{
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap_R1::OpenHardware()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap_R1::OpenHardware()
{
    X6_1000MDeviceMap::OpenHardware();

    AFE1000MAmplitude_SDev.ShiftInput(false);
}

//===========================================================================
//  CLASS X6_1000MDeviceMap_R2  -- Revision 2 Logic
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_1000MDeviceMap_R2
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6_1000MDeviceMap_R2
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X6_1000MDeviceMap_R2::X6_1000MDeviceMap_R2(X6Board * owner)
    :  X6_1000MDeviceMap(owner)
{
    FInput = new X6_1000MInputDevice(this, new X6_1000M_R2_AnalogInInfo(this));
    FOutput= new X6_1000MOutputDevice(this, new X6_1000M_AnalogOutInfo);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_1000MDeviceMap_R2
//---------------------------------------------------------------------------

X6_1000MDeviceMap_R2::~X6_1000MDeviceMap_R2()
{
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap_R2::OpenHardware()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap_R2::OpenHardware()
{
    X6_1000MDeviceMap::OpenHardware();

    AFE1000MAmplitude_SDev.ShiftInput(true);    // shift to 16 bits
}

//===========================================================================
//  CLASS X6_1000MDeviceMap_R2_bidir  -- Revision 2 Logic
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_1000MDeviceMap_R2_bidir
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6_1000MDeviceMap_R2_bidir
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

X6_1000MDeviceMap_R2_bidir::X6_1000MDeviceMap_R2_bidir(X6Board * owner)
    :  X6_1000MDeviceMap(owner)
{
    FInput = new X6_1000MInputDeviceR2bidir(this, new X6_1000M_R2_AnalogInInfo(this));
    FOutput= new X6_1000MOutputDeviceR2bidir(this, new X6_1000M_AnalogOutInfo);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_1000MDeviceMap_R2_bidir
//---------------------------------------------------------------------------

X6_1000MDeviceMap_R2_bidir::~X6_1000MDeviceMap_R2_bidir()
{
}

//---------------------------------------------------------------------------
//  X6_1000MDeviceMap_R2_bidir::OpenHardware()
//---------------------------------------------------------------------------

void  X6_1000MDeviceMap_R2_bidir::OpenHardware()
{
    X6_1000MDeviceMap::OpenHardware();

    AFE1000MAmplitude_SDev.ShiftInput(true);    // shift to 16 bits

    //  Adjust Dest IDs for
    this->VitaIn().VitaDestinationId(0, 0x2);
    this->VitaOut().VitaDestinationId(0, 0x1);
}

} // namespace Innovative


