// X6_1000MDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X6_1000MDeviceMap_MbH
#define X6_1000MDeviceMap_MbH

#include <X6_1000M_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <TiDac5682z_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X6_1000MDeviceMap;

//==============================================================================
//  CLASS X6_1000M_Pll -- RX PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X6_1000M_Pll : public Cdce72010
{
public:
    X6_1000M_Pll(X6_1000MDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg);
    virtual ~X6_1000M_Pll();

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

    double                  DCLK(bool single_channel, double freq);
    double                  ADCCLK(double freq);
    bool                    IsDacDllEnabled(bool single_channel);
    double                  InputMaxRate();
    double                  OutputMaxRate();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_1000M_Pll::IDacOperations -- Interface for DAC operations
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class IDacOperations
    {
    public:
        virtual ~IDacOperations()
            {}

        virtual bool  IsDacInSingleChannelMode() = 0;
    };

protected:
    X6_1000MDeviceMap * Owner;
    IDacOperations *   DOps;
    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//==============================================================================
//  CLASS X6_1000M_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class X6_1000M_SioPll : public Si5338
{
public:
    X6_1000M_SioPll(X6_1000MDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    X6_1000MDeviceMap * Owner;
};

//===========================================================================
//  CLASS X6AFE_X6_1000MAdcCalibrator -- X6_1000M ADC Calibration "SubDevice"
//===========================================================================

class X6AFE_X6_1000MAdcCalibrator : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS X6AFE_X6_1000MAdcCalibrator::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   CalControl_Addr;
        unsigned int   Adc0_Status_Addr;
        unsigned int   Adc1_Status_Addr;
    };

    X6AFE_X6_1000MAdcCalibrator(IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6AFE_X6_1000MAdcCalibrator();

    void    Calibrate(bool skip_cal=false);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_X6_1000MAdcCalibrator::CalControlRegister  -- Control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CalControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       PhyInit;
        RegisterBit       SkipPhyCal;

    public:
        CalControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               PhyInit(*this, 0),   SkipPhyCal(*this, 1)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_X6_1000MAdcCalibrator::CalResultRegister  -- Result Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CalResultRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  EyeAligned_Result;
        RegisterBit       PrbsLocked_Result;
        RegisterBit       PrbsAligned_Result;
        RegisterBit       PhyReady_Result;

    public:
        CalResultRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               EyeAligned_Result(*this, 0, 13),   PrbsLocked_Result(*this, 13),
               PrbsAligned_Result(*this, 14),  PhyReady_Result(*this, 15)
            {
            }
    };


    CalControlRegister  CalControlReg;
    CalResultRegister   Adc0_Result_Reg;
    CalResultRegister   Adc1_Result_Reg;
};

//===========================================================================
//  CLASS X6AFE_X6_1000MAdcAmplitudeControl -- X6_1000M ADC Amp "SubDevice"
//===========================================================================

class X6AFE_X6_1000MAdcAmplitudeControl : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS X6AFE_X6_1000MAdcAmplitudeControl::MapRegisters  -- Dev Reg Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AmplitudeControl_Addr;
    };

    X6AFE_X6_1000MAdcAmplitudeControl(IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6AFE_X6_1000MAdcAmplitudeControl();

    void    ShiftInput(bool shift_it);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_X6_1000MAdcAmplitudeControl::CalControlRegister  -- Control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AmplitudeControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup    DevShiftOn;

    public:
        AmplitudeControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DevShiftOn(*this, 0, 2)
            {
            }
    };


    AmplitudeControlRegister  AmplControlReg;
};

//===========================================================================
//  CLASS X6_1000MInputDevice  -- User Interface for X6-1000M ADCs
//===========================================================================

class X6_1000MInputDevice : public X6InputDevice
{
public:
    typedef X6InputDevice inherited;

	X6_1000MInputDevice(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_1000MInputDevice();

    virtual IX6AnalogIoInfo &  Info();
    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int       Channels();

private:
    IX6AnalogIoInfo *      FInfo;
    X6CalibrationSection   CalRomSection;
};

//===========================================================================
//  CLASS X6_1000MInputDeviceR2bidir  -- User Interface for X6-1000M ADCs
//===========================================================================
//  Logic R2, bidirectional

class X6_1000MInputDeviceR2bidir : public X6_1000MInputDevice
{
public:
    typedef X6_1000MInputDevice inherited;

    X6_1000MInputDeviceR2bidir(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_1000MInputDeviceR2bidir();

    virtual unsigned int       Channels();

private:

};

//===========================================================================
//  CLASS X6_1000MOutputDevice  -- User Interface for X6-1000M DACs
//===========================================================================

class X6_1000MOutputDevice : public X6OutputDevice
{
public:
    typedef X6OutputDevice inherited;

    friend class X6_1000M_DacCalToHdw;
    friend class X6_1000MDacToHardware;

    X6_1000MOutputDevice(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_1000MOutputDevice();

    virtual IX6AnalogIoInfo &  Info();
    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int       Channels();
    virtual unsigned int       ActiveDevices();

    TiDac5682::DacDevice &     Device(unsigned int idx)
        {
        if (idx)
            return DacDev_1;
        else
            return DacDev_0;
        }

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

private:
    X6_1000MDeviceMap *     DOwner;
    IX6AnalogIoInfo *       FInfo;
    TiDac5682::DacDevice    DacDev_0;
    TiDac5682::DacDevice    DacDev_1;
    X6CalibrationSection    CalRomSection;

};

//===========================================================================
//  CLASS X6_1000MOutputDeviceR2bidir  -- User Interface for X6-1000M DACs
//===========================================================================
//  Logic R2, bidirectional

class X6_1000MOutputDeviceR2bidir : public X6_1000MOutputDevice
{
public:
    typedef X6_1000MOutputDevice inherited;

    X6_1000MOutputDeviceR2bidir(X6_1000MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_1000MOutputDeviceR2bidir();

    virtual unsigned int       Channels();
    //virtual unsigned int       ActiveDevices();

private:

};

//===========================================================================
//  CLASS X6_1000MDeviceMap  -- Register & low level support for X6_1000M module
//===========================================================================
//  This is now a base class

class X6_1000M;

class X6_1000MDeviceMap :  public X6DeviceMap
{
    //friend class X6_1000M;
    friend class X6_1000M_R2_mixin;
    friend class X6_1000M_Pll;
    friend class X6_1000MInputDevice;
    friend class X6_1000MDacPriClockDevice;
    friend class X6_1000M_AnalogInInfo;

public:
    //
    //  Member Functions
    X6_1000MDeviceMap(X6Board * owner);
    virtual ~X6_1000MDeviceMap();

    X6_1000MInputDevice  &    Input();
    X6_1000MOutputDevice &    Output();

public: // pseudo-protected
    virtual void  OpenHardware();

    //  Subdevice Customization Handlers
    virtual Cdce72010 *         MakeAfePllObject();
    virtual Si570 *             MakeAfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *   MakeAfeVcxoI2CLink();
    virtual void                MakeAfeAdcSpiDevices();
    virtual Si5338 *            MakeSioPllObject(I2CDevice & device);
    virtual void                MakeAfeDacSpiDevices();
    virtual IPriClockDevice *   MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *  MakeFrameCalcDevice(FrameDeviceCode which);
    virtual IIsAVitaPacketStreamBoard *  GetBaseboardPtr();
    virtual unsigned int                 GetPatternModeAlignment();
    virtual unsigned int                 GetPatternModeScaleFactor();
    virtual bool                         GetPatternModeSingleMemoryFlag();

    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
    virtual unsigned int GetVitaInStreamCount();
    virtual unsigned int GetVitaOutStreamCount();

    virtual unsigned int GetCalibrationAdcCount();
    virtual unsigned int GetCalibrationDacCount();

    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    virtual IDacCalibrator * GetDacLatencyCalibrator();
    virtual X6_1000M_Pll::IDacOperations *  GetDacOperations();

    IX6RawClockDevices & RawClockDevice()  {  return AFEClk_SDev.RawClockDevice();  }

protected:
    //  Subdevices
    X6AFE_X6_1000MAdcCalibrator         AFE1000MAdcCal_SDev;
    X6AFE_X6_1000MAdcAmplitudeControl   AFE1000MAmplitude_SDev;

    // Fields
    X6_1000MInputDevice  *   FInput;
    X6_1000MOutputDevice *   FOutput;

    // Data
    X6Board *           Owner;
    Trace               Debug;

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void  LocalConstructCustomParts();
};

//===========================================================================
//  CLASS X6_1000MDeviceMap_R1  -- Register & low level support for X6_1000M module
//===========================================================================
//  For Revision 1 Boards

class X6_1000MDeviceMap_R1 :  public X6_1000MDeviceMap
{
    friend class X6_1000M;

public:
    //
    //  Member Functions
    X6_1000MDeviceMap_R1(X6_1000M * owner);
    virtual ~X6_1000MDeviceMap_R1();

    virtual void  OpenHardware();

protected:
};

//===========================================================================
//  CLASS X6_1000MDeviceMap_R2  -- Register & low level support for X6_1000M module
//===========================================================================
//  For Revision 2 Boards

class X6_1000MDeviceMap_R2 :  public X6_1000MDeviceMap
{
    friend class X6_1000M_R2;

public:
    //
    //  Member Functions
    X6_1000MDeviceMap_R2(X6Board * owner);
    virtual ~X6_1000MDeviceMap_R2();

    virtual void  OpenHardware();

protected:
};

//===========================================================================
//  CLASS X6_1000MDeviceMap_R2_bidir  -- Register & low level support for X6_1000M module
//===========================================================================
//  For Revision 2 Boards
//
class X6_1000MDeviceMap_R2_bidir :  public X6_1000MDeviceMap
{
    friend class X6_1000M_R2;

public:
    //
    //  Member Functions
    X6_1000MDeviceMap_R2_bidir(X6Board * owner);
    virtual ~X6_1000MDeviceMap_R2_bidir();

    virtual void  OpenHardware();

protected:
};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
