//---------------------------------------------------------------------------
// X6_1000M_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X6_1000M_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6_1000M  -- X6_1000M PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000M
//---------------------------------------------------------------------------

X6_1000M::X6_1000M()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000M 
//---------------------------------------------------------------------------

X6_1000M::~X6_1000M()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_1000M::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X6_1000MInputDevice & X6_1000M::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  X6_1000M::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

X6_1000MOutputDevice & X6_1000M::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  X6_1000M::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6RawClockDevices & X6_1000M::RawClockDevice()
{
    return Device.AFEClk_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_1000M::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_1000M::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo  2 constant SioPll "
                    "  3 constant Dac0Spi  4 constant Dac1Spi"
                    "  5 constant Adc0Spi  6 constant Adc1Spi");
    engine.Evaluate(": ?X6_1000M "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Vcxo  SioPll  Dac0Spi Dac1Spi Adc0Spi Adc1Spi \" "
                    "    cr  "
                    "    cr .\" ---- X6_1000M Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X6_1000M  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate(": ?X6-1000M  ?X6_1000M  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("X6_1000M-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  X6_1000M::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X6_1000M::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kDac0Spi = 3;  const int kDac1Spi = 4;
    const int kAdc0Spi = 5;  const int kAdc1Spi = 6;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Device.AFEClk_SDev.Pll().Port(addr);
        case kVcxo:
            return Device.AFEClk_SDev.Vcxo().Port(addr);
        case kSioPll:
            return Device.SioPll_SDev.SioPll_I2C.Reg(addr);
        case kDac0Spi:
            return Device.Output().Spi(0, addr);
        case kDac1Spi:
            return Device.Output().Spi(1, addr);
        case kAdc0Spi:
            return Device.Input().Spi(0, addr);
        case kAdc1Spi:
            return Device.Input().Spi(1, addr);
        }
}

//---------------------------------------------------------------------------
//  X6_1000M::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X6_1000M::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kDac0Spi = 3;  const int kDac1Spi = 4;
    const int kAdc0Spi = 5;  const int kAdc1Spi = 6;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEClk_SDev.Pll().Port(addr, value);
            break;
        case kVcxo:
            Device.AFEClk_SDev.Vcxo().Port(addr, value);
            break;
        case kSioPll:
            Device.SioPll_SDev.SioPll_I2C.Reg(addr, value);
            break;
        case kDac0Spi:
            Device.Output().Spi(0, addr, value);
            break;
        case kDac1Spi:
            Device.Output().Spi(1, addr, value);
            break;
        case kAdc0Spi:
            Device.Input().Spi(0, addr, value);
            break;
        case kAdc1Spi:
            Device.Input().Spi(1, addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_1000M::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X6_1000M::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEClk_SDev.Pll().Frequency(freq);
            Device.AFEClk_SDev.Pll().Reapply();
            break;
        case kVcxo:
            Device.AFEClk_SDev.Vcxo().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_1000M::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X6_1000M::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.AFEClk_SDev.Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(Device.AFEClk_SDev.Vcxo().FrequencyActual());
        }
}


//===========================================================================
//  CLASS X6_1000M_R2_mixin  -- X6_1000M PCI baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  X6_1000M_R2_mixin::Do_AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_1000M_R2_mixin::Do_AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo  2 constant SioPll "
                    "  3 constant Dac0Spi  4 constant Dac1Spi"
                    "  5 constant Adc0Spi  6 constant Adc1Spi");
    engine.Evaluate(": ?X6_1000M "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Vcxo  SioPll  Dac0Spi Dac1Spi Adc0Spi Adc1Spi \" "
                    "    cr  "
                    "    cr .\" ---- X6_1000M R2 Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X6_1000M  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate(": ?X6-1000M  ?X6_1000M  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("X6_1000M R2-specific wordset loaded \n" );
}

//---------------------------------------------------------------------------
//  X6_1000M_R2_mixin::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X6_1000M_R2_mixin::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kDac0Spi = 3;  const int kDac1Spi = 4;
    const int kAdc0Spi = 5;  const int kAdc1Spi = 6;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return R2Device().AFEClk_SDev.Pll().Port(addr);
        case kVcxo:
            return R2Device().AFEClk_SDev.Vcxo().Port(addr);
        case kSioPll:
            return R2Device().SioPll_SDev.SioPll_I2C.Reg(addr);
        case kDac0Spi:
            return R2Device().Output().Spi(0, addr);
        case kDac1Spi:
            return R2Device().Output().Spi(1, addr);
        case kAdc0Spi:
            return R2Device().Input().Spi(0, addr);
        case kAdc1Spi:
            return R2Device().Input().Spi(1, addr);
        }
}

//---------------------------------------------------------------------------
//  X6_1000M_R2_mixin::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X6_1000M_R2_mixin::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kDac0Spi = 3;  const int kDac1Spi = 4;
    const int kAdc0Spi = 5;  const int kAdc1Spi = 6;

    switch (dev)
        {
        default:
            break;
        case kPll:
            R2Device().AFEClk_SDev.Pll().Port(addr, value);
            break;
        case kVcxo:
            R2Device().AFEClk_SDev.Vcxo().Port(addr, value);
            break;
        case kSioPll:
            R2Device().SioPll_SDev.SioPll_I2C.Reg(addr, value);
            break;
        case kDac0Spi:
            R2Device().Output().Spi(0, addr, value);
            break;
        case kDac1Spi:
            R2Device().Output().Spi(1, addr, value);
            break;
        case kAdc0Spi:
            R2Device().Input().Spi(0, addr, value);
            break;
        case kAdc1Spi:
            R2Device().Input().Spi(1, addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_1000M_R2_mixin::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X6_1000M_R2_mixin::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            break;
        case kPll:
            R2Device().AFEClk_SDev.Pll().Frequency(freq);
            R2Device().AFEClk_SDev.Pll().Reapply();
            break;
        case kVcxo:
            R2Device().AFEClk_SDev.Vcxo().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_1000M_R2_mixin::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X6_1000M_R2_mixin::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(R2Device().AFEClk_SDev.Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(R2Device().AFEClk_SDev.Vcxo().FrequencyActual());
        }
}

//===========================================================================
//  CLASS X6_1000M_R2in  -- X6_1000M_R2in PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000M_R2in
//---------------------------------------------------------------------------

X6_1000M_R2in::X6_1000M_R2in()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000M_R2in
//---------------------------------------------------------------------------

X6_1000M_R2in::~X6_1000M_R2in()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2in::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6RawClockDevices & X6_1000M_R2in::RawClockDevice()
{
    return Device.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2in::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X6_1000MInputDevice & X6_1000M_R2in::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2in::R2Device() --  Return reference to device map
//---------------------------------------------------------------------------

X6_1000MDeviceMap & X6_1000M_R2in::R2Device()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X6_1000M_R2in::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_1000M_R2in::AfterFiclStart(FiclSystem & engine)
{
    //  do common mixin code
    Do_AfterFiclStart(engine);
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

//===========================================================================
//  CLASS X6_1000M_R2out  -- X6_1000M_R2out PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000M_R2out
//---------------------------------------------------------------------------

X6_1000M_R2out::X6_1000M_R2out()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000M_R2out
//---------------------------------------------------------------------------

X6_1000M_R2out::~X6_1000M_R2out()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2out::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6RawClockDevices & X6_1000M_R2out::RawClockDevice()
{
    return Device.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2out::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

X6_1000MOutputDevice & X6_1000M_R2out::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2out::R2Device() --  Return reference to device map
//---------------------------------------------------------------------------

X6_1000MDeviceMap & X6_1000M_R2out::R2Device()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X6_1000M_R2out::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_1000M_R2out::AfterFiclStart(FiclSystem & engine)
{
    //  do common mixin code
    Do_AfterFiclStart(engine);
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

//===========================================================================
//  CLASS X6_1000M_R2bidir  -- X6_1000M_R2bidir PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_1000M_R2bidir
//---------------------------------------------------------------------------

X6_1000M_R2bidir::X6_1000M_R2bidir()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_1000M_R2bidir
//---------------------------------------------------------------------------

X6_1000M_R2bidir::~X6_1000M_R2bidir()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2bidir::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6RawClockDevices & X6_1000M_R2bidir::RawClockDevice()
{
    return Device.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2bidir::Input() --  Return reference to Input object
//---------------------------------------------------------------------------

X6_1000MInputDevice & X6_1000M_R2bidir::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2bidir::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

X6_1000MOutputDevice & X6_1000M_R2bidir::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  X6_1000M_R2bidir::R2Device() --  Return reference to device map
//---------------------------------------------------------------------------

X6_1000MDeviceMap & X6_1000M_R2bidir::R2Device()
{
    return Device;
}

//---------------------------------------------------------------------------
//  X6_1000M_R2bidir::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_1000M_R2bidir::AfterFiclStart(FiclSystem & engine)
{
    //  do common mixin code
    Do_AfterFiclStart(engine);
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
