// X6_250MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X6_250M_Mb.h>
#include <X6_250MDeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <X6_250MMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X6_MemoryMap;
using namespace X6_250MMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const unsigned int    MaxInDevices      = 4;
const int             ChannelsPerDevice = 2;
const int             MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const float         InputMaxRate            = 250.0e6f;

//  Hardware Variant code 1 - 310 MHz Adc
const float         InputMaxRate_Var1       = 310.0e6f;

const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 14;
const          int  InputMax_mV             = 750;
const          int  InputMin_mV             = -750;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 16;

const unsigned int X6_250M_AdcSpiDevices      = MaxInDevices;
const unsigned int X6250M_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int X6250M_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int X6250M_AdcCount            = MaxInChannels;  // Number of Adc Channels
const unsigned int X6250M_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations

const unsigned int X6_250M_DacSpiDevices      = 0;   // No DACs with SPIs

const unsigned int X6250M_VeloOutStreamCount  = 1;   // Number of Velo PIDs
const unsigned int X6250M_VitaOutStreamCount  = 1;   // Number of VITA Outgoing SIDs

const unsigned int X6_250M_Adc_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romX6_250M_SysInfoOffset        = 0;
const int romX6_250M_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romX6_250M_InputCoeffOffset       = 0;
const int romX6_250M_InputCoeffOffset_avail = 17;   // 8 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romX6_250M_OutputCoeffOffset       = 0;
const int romX6_250M_OutputCoeffOffset_avail = 0;  // none used

//===========================================================================
// Class X6_250M_AnalogInInfo
//===========================================================================

class X6_250M_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    X6_250M_AnalogInInfo(X6_250MDeviceMap * owner)
         :  Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~X6_250M_AnalogInInfo()  {  }

    virtual float     MaxRate() const
        {
        if (Owner->Is310MhzVariant())
            return InputMaxRate_Var1;
        else
            return InputMaxRate;
        }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    X6_250MDeviceMap * Owner;
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS X6_250MInputDevice  -- Register & low level support for X6 RX
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_250MInputDevice
//---------------------------------------------------------------------------

X6_250MInputDevice::X6_250MInputDevice(X6_250MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romX6_250M_InputCoeffOffset, MaxInChannels,
                       X6_250M_Adc_One, AdcCal() ),
      CurrentAdcDeviceTestMode(0x01)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_250MInputDevice
//---------------------------------------------------------------------------

X6_250MInputDevice::~X6_250MInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  X6_250MInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::StartAdcPhyCal() --
//---------------------------------------------------------------------------

void X6_250MInputDevice::StartAdcPhyCal()
{
    DOwner->AdcPhyCalibrationReg.adc_cal_start.Value(false);
    DOwner->AdcPhyCalibrationReg.adc_cal_start.Value(true);
    DOwner->AdcPhyCalibrationReg.adc_cal_start.Value(false);
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::Enable16BitRange() --
//---------------------------------------------------------------------------

void X6_250MInputDevice::Enable16BitRange(bool value)
{
    DOwner->AdcAmplitudeControlReg.adc_multx16_en.Value(value ? 0xff : 0x0);
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::Enable16BitRange() --
//---------------------------------------------------------------------------

bool X6_250MInputDevice::Enable16BitRange()
{
    return DOwner->AdcAmplitudeControlReg.adc_multx16_en.Value() == 0xff;
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------

unsigned int  X6_250MInputDevice::Channels()
{
    return MaxInChannels;
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::ConfigureHardware()
//---------------------------------------------------------------------------
//
//void X6_250MInputDevice::ConfigureHardware()
//{
//    inherited::ConfigureHardware();
//}
//
//---------------------------------------------------------------------------
//  X6_250MInputDevice::AdcDeviceTestMode() - set device test mode
//---------------------------------------------------------------------------

void X6_250MInputDevice::AdcDeviceTestMode(bool enabled, AdcDevTestModes test_mode)
{
    if (enabled)
        {
        if (test_mode == tmAllZeros)
            CurrentAdcDeviceTestMode = 0x05;
        else if (test_mode == tmAllOnes)
            CurrentAdcDeviceTestMode = 0x25;
        else if (test_mode == tmAlternating1_0)
            CurrentAdcDeviceTestMode = 0x45;
        else // (test_mode == tmCheckerboard)
            CurrentAdcDeviceTestMode = 0x85;
        }
    else
        CurrentAdcDeviceTestMode = 0x01;
}

//---------------------------------------------------------------------------
//  X6_250MInputDevice::Adc_OnConfigure()
//---------------------------------------------------------------------------

void X6_250MInputDevice::Adc_OnConfigure()
{
    for (unsigned int dev=0; dev<MaxInDevices; dev++)
        {
        Spi(dev, 0x00, 0x80);   // reset ADC device

        unsigned int CurrentChanSetting;
        if (ChannelEnabled(2*dev)==false && ChannelEnabled(2*dev+1)==false)
            CurrentChanSetting = 0x08;
        else if (ChannelEnabled(2*dev)==true && ChannelEnabled(2*dev+1)==false)
            CurrentChanSetting = 0x02;
        else   // both enabled
            CurrentChanSetting = 0x00;
        Spi(dev, 0x01, CurrentChanSetting);

        Spi(dev, 0x04, CurrentAdcDeviceTestMode);   // dev mode

        if (Owner->Info().FpgaLogicVersion() < 0x106)
            Spi(dev, 0x02, 0x8);   // clock invert
        }
    if (Owner->Info().FpgaLogicVersion() < 0x106)
        return;   // no need to go on, logic doesn't support calibration

//    Owner->Debug.Log("Run the ADC PHY calibration");
    StartAdcPhyCal();
//    Owner->PokeLogic(0x818, 0);
//    Owner->PokeLogic(0x818, 1);
//    Owner->PokeLogic(0x818, 0);
}

//=============================================================================
//  CLASS X6_250M_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_250M_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_250M_Pll
//---------------------------------------------------------------------------

X6_250M_Pll::X6_250M_Pll(X6_250MDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner), FPDF(100.0e3) 
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Output 3 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_250M_Pll::OnInit()
{
    //  Device Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         true,   // Secondary Bias ON
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       false,  // Input Term NOT disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, false);  SetOutputConfig(0, ocDisabled);
    OutputInUse(1, false);  SetOutputConfig(1, ocDisabled);
    OutputInUse(2, true);   SetOutputConfig(2, ocLvds);        // FPGA

    if (Owner->IsLvdsHiSwingVariant())
        {
        // Set ADC outputs to LVDS
        OutputInUse(3, true);   SetOutputConfig(3, ocLvds);        // ADC Dev 0
        OutputInUse(4, true);   SetOutputConfig(4, ocLvds);        // ADC Dev 1
        OutputInUse(5, true);   SetOutputConfig(5, ocLvds);        // ADC Dev 2
        OutputInUse(6, true);   SetOutputConfig(6, ocLvds);        // ADC Dev 3
        }
    else
        {
        // Set ADC outputs to LVPECL
        OutputInUse(3, true);   SetOutputConfig(3, ocLvPecl);        // ADC Dev 0
        OutputInUse(4, true);   SetOutputConfig(4, ocLvPecl);        // ADC Dev 1
        OutputInUse(5, true);   SetOutputConfig(5, ocLvPecl);        // ADC Dev 2
        OutputInUse(6, true);   SetOutputConfig(6, ocLvPecl);        // ADC Dev 3
        }

    OutputInUse(7, false);  SetOutputConfig(7, ocDisabled);
    OutputInUse(8, false);  SetOutputConfig(8, ocDisabled);
    OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);

    //  High Swing Output
    SetHighSwingLvds(2, true);
    if (Owner->IsLvdsHiSwingVariant())
        {
        SetHighSwing(3, true);
        SetHighSwing(4, true);
        SetHighSwing(5, true);
        SetHighSwing(6, true);
        }
    else
        {
        SetHighSwing(3, false);
        SetHighSwing(4, false);
        SetHighSwing(5, false);
        SetHighSwing(6, false);
        }

    //  Phase = 0
    SetPhaseAdjust(2, 1, 0);   SetPhaseAdjust(3, 1, 0);  // Set all outputs to 0 phase
    SetPhaseAdjust(4, 1, 0);   SetPhaseAdjust(5, 1, 0);
    SetPhaseAdjust(6, 1, 0);

    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    float input_max_rate = static_cast<float>(InputMaxRate());

    OutputFrequencyLimit[2] = input_max_rate/2.0f;
    OutputFrequencyLimit[3] = input_max_rate;
    OutputFrequencyLimit[4] = input_max_rate;
    OutputFrequencyLimit[5] = input_max_rate;
    OutputFrequencyLimit[6] = input_max_rate;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use external pin
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  X6_250M_Pll::AdcMasterOutput()
{
    return 3;
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  X6_250M_Pll::DacMasterOutput()
{
    return FMasterOutput;
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  X6_250M_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(2, adc_rate/2.0);
    OutputFrequency(3, adc_rate);
    OutputFrequency(4, adc_rate);
    OutputFrequency(5, adc_rate);
    OutputFrequency(6, adc_rate);
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  X6_250M_Pll::SetDacOutputRates(double /*dac_rate*/)
{
    // no dacs
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  X6_250M_Pll::Vcxo()
{
    return Owner->AFEClk_SDev.Vcxo();
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::OnReapply()
//---------------------------------------------------------------------------

void  X6_250M_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  X6_250M_Pll::InputMaxRate()  -- Return Max Input Rate
//---------------------------------------------------------------------------

double  X6_250M_Pll::InputMaxRate()
{
    return Owner->Input().Info().MaxRate();
}

//=============================================================================
//  CLASS X6_250M_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_250M_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_250M_SioPll
//---------------------------------------------------------------------------

X6_250M_SioPll::X6_250M_SioPll(X6_250MDeviceMap * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  X6_250M_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_250M_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, false);    SetMultiSynthPowerDown(3, false);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);    
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(2, diNoClk);   // override since not using output

    OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
    OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, true);

    SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
    SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
    SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
    SetOutputVoltage(3, vo_2p5volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  X6_250M_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  X6_250M_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS X6_250MDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_250MDeviceMap
//---------------------------------------------------------------------------

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)

static Cdce72010Registers X6_RX_PllRegisters =
    { mmAFE_PllControl,  mmAFE_PllSpi  };

static X6_Std_I2CLinkage::MapRegisters X6_RX_VcoI2CLinkRegAddrs =
    {  mmAFE_VcxoControl,      //  AfeVcxoControl_Addr
    };
    
//---------------------------------------------------------------------------
//  constructor for class X6_250MDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

X6_250MDeviceMap::X6_250MDeviceMap(X6_250M * owner)
    :  X6DeviceMap(owner),
       FInput(new X6_250MInputDevice(this, new X6_250M_AnalogInInfo(this))),
       Owner(owner),
       AdcPhyCalibrationReg(WB_AFE, mmAFE_AdcPhyCal),
       AdcAmplitudeControlReg(WB_AFE, mmAFE_AdcIQ_ModeEnable)
{
    //
    //  Since this device has no DACs, disable all DAC subdevices
    DisableDacDevices();
    //
    //  SubDevice Customization
    X6DeviceMap::ConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &X6_250MDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &X6_250MDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X6_250MDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X6_250MDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X6_250MDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X6_250MDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X6_250MDeviceMap::OnAfterStreamStopHandler );


}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_250MDeviceMap
//---------------------------------------------------------------------------

X6_250MDeviceMap::~X6_250MDeviceMap()
{
    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OpenHardware()
{
    X6DeviceMap::OpenHardware();
    FInput->Cal().ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    FInput->Cal().ApplyRomCoefficients();

    // Family-common init
    X6DeviceMap::PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    X6DeviceMap::ConfigureHardware();
    //
    //  Special Config for A/D Devices
    Input().Adc_OnConfigure();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    AFECommon_SDev.Engine(X6AFE_Common::tInput, true);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Base class "on stream stop" behavior
    X6DeviceMap::StreamStopHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeAfePllObject() -- Reference to input object
//---------------------------------------------------------------------------

Cdce72010 *  X6_250MDeviceMap::MakeAfePllObject()
{
    //  Make our local PLL object (subdev will delete it)
    return new X6_250M_Pll(this, WB_AFE, X6_RX_PllRegisters );
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeAfeVcxoObject() --  Build custom VCO object
//---------------------------------------------------------------------------

Si570 *  X6_250MDeviceMap::MakeAfeVcxoObject(IRequires_I2C_RegIo & device)
{
    //  Make our local Vcxo object (subdev will delete it)
    return new Si570(device, Si570::tcA, kVcxoPowerOnRate );
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeAfeVcxoI2CLink() --  Build custom Vco I2C Link
//---------------------------------------------------------------------------

IX6_I2C_uDevice *  X6_250MDeviceMap::MakeAfeVcxoI2CLink()
{
    //  Make our local Vcxo I2C Link (subdev will delete it)
    return new X6_Std_I2CLinkage(WB_AFE, X6_RX_VcoI2CLinkRegAddrs);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeAfeAdcSpiDevices() -- Build ADC Spi Devices
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::MakeAfeAdcSpiDevices()
{
    //  Make one SPI device for each ADC device
    unsigned int Write_Addr = mmAFE_AdcSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_AdcSpiStatBase;

    for (unsigned int i=0; i<X6_250M_AdcSpiDevices; ++i)
        {
        AFEAdcSpi_SDev.MakeSpiDevice("ADC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_AdcSpiCtrlDelta;
        Read_Addr  += mmAFE_AdcSpiStatDelta;
        }
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeAfeDacSpiDevices() -- Build DAC Spi Devices
//---------------------------------------------------------------------------

void  X6_250MDeviceMap::MakeAfeDacSpiDevices()
{
/*  ...No DAC SPI for X6_RX
    //  Make one SPI device for each DAC device
    unsigned int Write_Addr = mmAFE_DacSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_DacSpiStatBase;

    for (unsigned int i=0; i<X6_250M_DacSpiDevices; ++i)
        {
        AFEDacSpi_SDev.MakeSpiDevice("DAC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_DacSpiCtrlDelta;
        Read_Addr  += mmAFE_DacSpiStatDelta;
        }
*/
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  X6_250MDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new X6_250M_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_250MDeviceMap::GetVeloInStreamCount()
{
    return X6250M_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetVeloOutStreamCount() -- Return Velo Out PIDs
//---------------------------------------------------------------------------

unsigned int  X6_250MDeviceMap::GetVeloOutStreamCount()
{
    return X6250M_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  X6_250MDeviceMap::GetVitaInStreamCount()
{
    return X6250M_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  X6_250MDeviceMap::GetVitaOutStreamCount()
{
    return X6250M_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_250MDeviceMap::GetCalibrationAdcCount()
{
    return X6250M_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_250MDeviceMap::GetCalibrationDacCount()
{
    return X6250M_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_250MDeviceMap::GetAdcCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_250MDeviceMap::GetDacCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetDacLatencyCalibrator() -- Get Latency Cal object
//---------------------------------------------------------------------------

IDacCalibrator *  X6_250MDeviceMap::GetDacLatencyCalibrator()
{
    // No dacs, no calibrator needed
    return 0;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  X6_250MDeviceMap::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new X6StandardPriClockDevice(this, 2);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  X6_250MDeviceMap::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions
    return new X6StandardFrameCalcDevice(this, 2);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   X6_250MDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   X6_250MDeviceMap::GetPatternModeAlignment()
{
    return 8;   // standard X6 alignment of 8 words
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   X6_250MDeviceMap::GetPatternModeScaleFactor()
{
    return 8;   // standard X6 scale factor of 8 words
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   X6_250MDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return false;  // dual memory
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X6_250MInputDevice & X6_250MDeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::Is310MhzVariant() -- Reference to input object
//---------------------------------------------------------------------------

bool  X6_250MDeviceMap::Is310MhzVariant()
{
    MemoryBitManipulator Variant(Info_SDev.FpgaHardwareVariant());

    return Variant.Bit(0);
}

//---------------------------------------------------------------------------
//  X6_250MDeviceMap::IsLvdsHiSwingVariant() -- Reference to input object
//---------------------------------------------------------------------------

bool X6_250MDeviceMap::IsLvdsHiSwingVariant()
{
    MemoryBitManipulator Variant(Info_SDev.FpgaHardwareVariant());

    return Variant.Bit(1);
}


} // namespace Innovative


