// X6_250MDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X6_250MDeviceMap_MbH
#define X6_250MDeviceMap_MbH

#include <X6_250M_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X6_250MDeviceMap;

//==============================================================================
//  CLASS X6_250M_Pll -- 250M PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X6_250M_Pll : public Cdce72010
{
public:
    X6_250M_Pll(X6_250MDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg);

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

protected:
    X6_250MDeviceMap *  Owner;
    double              FPDF;

    double  InputMaxRate();

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//==============================================================================
//  CLASS X6_250M_SioPll -- 250M SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class X6_250M_SioPll : public Si5338
{
public:
    X6_250M_SioPll(X6_250MDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    X6_250MDeviceMap * Owner;
};

//===========================================================================
//  CLASS X6_250MInputDevice  -- Register & low level support for X6-250M ADCs
//===========================================================================

class X6_250MInputDevice : public X6InputDevice
{
public:
    typedef X6InputDevice inherited;

    X6_250MInputDevice(X6_250MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_250MInputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int       Channels();

    virtual void StartAdcPhyCal();
    virtual void Enable16BitRange(bool value);
    virtual bool Enable16BitRange();


    enum   AdcDevTestModes { tmAllZeros, tmAllOnes, tmAlternating1_0, tmCheckerboard=4 };
    void   AdcDeviceTestMode(bool enabled, AdcDevTestModes test_mode);

    void   Adc_OnConfigure();

private:
    X6_250MDeviceMap *     DOwner;
    IX6AnalogIoInfo *      FInfo;
    X6CalibrationSection   CalRomSection;

    int  CurrentAdcDeviceTestMode;


};

//===========================================================================
//  CLASS X6_250MDeviceMap  -- Register & low level support for X6_250M module
//===========================================================================
class X6_250M;

class X6_250MDeviceMap :  public X6DeviceMap

{
    friend class X6_250M;
    friend class X6_250M_Pll;
    friend class X6_250MInputDevice;
    friend class X6_250M_AnalogInInfo;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcPhyCalibrationRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcPhyCalibrationRegister : public Innovative::Register
    {
        typedef Register inherited;

    public:
        Innovative::RegisterBit         adc_cal_start;

        AdcPhyCalibrationRegister(Innovative::IAddressingSpace &space, int offset)
            : inherited(space, offset),
              adc_cal_start(*this, 0)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcAmplitudeControlRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcAmplitudeControlRegister : public Innovative::Register
    {
        typedef Register inherited;

    public:
        Innovative::RegisterBitGroup    adc_multx16_en;

        AdcAmplitudeControlRegister(Innovative::IAddressingSpace &space, int offset)
            : inherited(space, offset),
              adc_multx16_en(*this, 0, 8)
            {
            }
    };

public:
    //
    //  Member Functions
    X6_250MDeviceMap(X6_250M * owner);
    virtual ~X6_250MDeviceMap();

    X6_250MInputDevice  &    Input();

    bool  Is310MhzVariant();
    bool  IsLvdsHiSwingVariant();


public: // pseudo-protected
    virtual void  OpenHardware();

    //  Subdevice Customization Handlers
    virtual Cdce72010 *         MakeAfePllObject();
    virtual Si570 *             MakeAfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *   MakeAfeVcxoI2CLink();
    virtual void                MakeAfeAdcSpiDevices();
    virtual Si5338 *            MakeSioPllObject(I2CDevice & device);
    virtual void                MakeAfeDacSpiDevices();
    virtual IPriClockDevice *   MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *  MakeFrameCalcDevice(FrameDeviceCode which);
    virtual IIsAVitaPacketStreamBoard *           GetBaseboardPtr();
    virtual unsigned int                          GetPatternModeAlignment();
    virtual unsigned int                          GetPatternModeScaleFactor();
    virtual bool                                  GetPatternModeSingleMemoryFlag();

    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
    virtual unsigned int GetVitaInStreamCount();
    virtual unsigned int GetVitaOutStreamCount();

    virtual unsigned int GetCalibrationAdcCount();
    virtual unsigned int GetCalibrationDacCount();

    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    virtual IDacCalibrator * GetDacLatencyCalibrator();

protected:
    // Fields
    X6_250MInputDevice  *    FInput;

    // Data
    X6_250M *         Owner;
    AdcPhyCalibrationRegister   AdcPhyCalibrationReg;
    AdcAmplitudeControlRegister AdcAmplitudeControlReg;
    Trace             Debug;

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
