//---------------------------------------------------------------------------
// X6_250M_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X6_250M_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6_250M  -- X6_250M PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_250M
//---------------------------------------------------------------------------

X6_250M::X6_250M()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_250M 
//---------------------------------------------------------------------------

X6_250M::~X6_250M()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_250M::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X6_250MInputDevice & X6_250M::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X6_250M::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6RawClockDevices & X6_250M::RawClockDevice()
{
    return Device.AFEClk_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_250M::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_250M::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo  2 constant SioPll "
                    "  3 constant Adc0Spi   4 constant Adc1Spi  5 constant Adc2Spi  6 constant Adc3Spi");

    engine.Evaluate(": ?X6_250M "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Vcxo  SioPll Adc0Spi Adc1Spi Adc2Spi Adc3Spi\" "
                    "    cr  "
                    "    cr .\" ---- X6_250M Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X6_250M  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate(": ?X6-250M  ?X6_250M  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("X6_250M-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  X6_250M::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X6_250M::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kAdc0Spi = 3;  const int kAdc1Spi = 4;  const int kAdc2Spi = 5;  const int kAdc3Spi = 6;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Device.AFEClk_SDev.Pll().Port(addr);
        case kVcxo:
            return Device.AFEClk_SDev.Vcxo().Port(addr);
        case kSioPll:
            return Device.SioPll_SDev.SioPll_I2C.Reg(addr);
        case kAdc0Spi:
            return Device.Input().Spi(0, addr);
        case kAdc1Spi:
            return Device.Input().Spi(1, addr);
        case kAdc2Spi:
            return Device.Input().Spi(2, addr);
        case kAdc3Spi:
            return Device.Input().Spi(3, addr);
        }
}

//---------------------------------------------------------------------------
//  X6_250M::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X6_250M::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kAdc0Spi = 3;  const int kAdc1Spi = 4;  const int kAdc2Spi = 5;  const int kAdc3Spi = 6;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEClk_SDev.Pll().Port(addr, value);
            break;
        case kVcxo:
            Device.AFEClk_SDev.Vcxo().Port(addr, value);
            break;
        case kSioPll:
            Device.SioPll_SDev.SioPll_I2C.Reg(addr, value);
            break;
        case kAdc0Spi:
            Device.Input().Spi(0, addr, value);
            break;
        case kAdc1Spi:
            Device.Input().Spi(1, addr, value);
            break;
        case kAdc2Spi:
            Device.Input().Spi(2, addr, value);
            break;
        case kAdc3Spi:
            Device.Input().Spi(3, addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_250M::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X6_250M::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEClk_SDev.Pll().Frequency(freq);
            Device.AFEClk_SDev.Pll().Reapply();
            break;
        case kVcxo:
            Device.AFEClk_SDev.Vcxo().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_250M::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X6_250M::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.AFEClk_SDev.Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(Device.AFEClk_SDev.Vcxo().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
