// X6_400MDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X6_400M_Mb.h>
#include <X6_400MDeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <X6_400MMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X6_MemoryMap;
using namespace X6_400MMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.

const int    MaxInDevices      = 2;
const int    InChanPerDevice   = 1;
const int    MaxInChannels     = MaxInDevices*InChanPerDevice;

const int    MaxOutDevices     = 1;
const int    OutChanPerDevice  = 2;
const int    MaxOutChannels    = MaxOutDevices*OutChanPerDevice;

const float         InputMaxRate            = 400.0e6f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 14;

//  Hardware Variant code 1 - 500 MHz Adc
const float         InputMaxRate_Var1       = 500.0e6f;
const unsigned int  InputWidth_Bits_Var1    = 12;

const          int  InputMax_mV             = 1000;
const          int  InputMin_mV             = -1000;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 16;

const unsigned int X6_400M_AdcSpiDevices       = 0;
const unsigned int X6_400M_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int X6_400M_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int X6_400M_AdcCount            = MaxInChannels;  // Number of Adc Channels
const unsigned int X6_400M_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations

const float         OutputMaxRate            = 1000.0e6f;
const float         OutputMinRate            = 0.0e6f;
const unsigned int  OutputWidth_Bits         = 16;

const          int  OutputMax_mV             = 1000;
const          int  OutputMin_mV             = -1000;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 16;

const unsigned int X6_400M_DacSpiDevices       = MaxOutDevices;

const unsigned int X6_400M_VeloOutStreamCount  = 1;
const unsigned int X6_400M_VitaOutStreamCount  = MaxOutChannels;

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romX6_400M_SysInfoOffset        = 0;
const int romX6_400M_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romX6_400M_InputCoeffOffset       = 0;
const int romX6_400M_InputCoeffOffset_avail = 5;   // 2 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romX6_400M_OutputCoeffOffset       = 0;
const int romX6_400M_OutputCoeffOffset_avail = 5;  // 2 * 2 coefficients plus 'calibrated' word

const unsigned int X6_400M_Adc_One = 0x10000;    // representation of floating point 1.0
const unsigned int X6_400M_Dac_One = 0x10000;    // representation of floating point 1.0

//===========================================================================
// Class X6_400M_AnalogInInfo
//===========================================================================

class X6_400M_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    X6_400M_AnalogInInfo(X6_400MDeviceMap * owner)
         : Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~X6_400M_AnalogInInfo()  {  }

    virtual float     MaxRate() const
        {
        if (Owner->Info_SDev.FpgaHardwareVariant()==1)
            return InputMaxRate_Var1;
        else
            return InputMaxRate;
        }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
	virtual unsigned int  Bits() const
        {
        if (Owner->Info_SDev.FpgaHardwareVariant()==1)
            return InputWidth_Bits_Var1;
        else
            return InputWidth_Bits;
        }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    X6_400MDeviceMap * Owner;
    SpanInfo   FSpan;
};

//===========================================================================
// Class X6_400M_AnalogOutInfo
//===========================================================================

class X6_400M_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    X6_400M_AnalogOutInfo() : FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~X6_400M_AnalogOutInfo()  {  }

    virtual float     MaxRate() const         {  return OutputMaxRate;  }
    virtual float     MinRate() const         {  return OutputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS X6_400MInputDevice  -- User Interface for X6-400M ADCs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_400MInputDevice
//---------------------------------------------------------------------------

X6_400MInputDevice::X6_400MInputDevice(X6_400MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romX6_400M_InputCoeffOffset, MaxInChannels,
                       X6_400M_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_400MInputDevice
//---------------------------------------------------------------------------

X6_400MInputDevice::~X6_400MInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  X6_400MInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------

unsigned int  X6_400MInputDevice::Channels()
{
    return MaxInChannels;
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::EnableIQMode() --
//---------------------------------------------------------------------------

void  X6_400MInputDevice::EnableIQMode(bool value)
{
    DOwner->AdcAmplitudeControlReg.adc_intrlvr_en.Value(value);
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::EnableIQMode() --
//---------------------------------------------------------------------------

bool X6_400MInputDevice::EnableIQMode()
{
    return DOwner->AdcAmplitudeControlReg.adc_intrlvr_en.Value();
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::Enable16BitRange() --
//---------------------------------------------------------------------------

void X6_400MInputDevice::Enable16BitRange(bool value)
{
    DOwner->AdcAmplitudeControlReg.adc_multx16_en.Value(value ? 0x3 : 0x0);
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::Enable16BitRange() --
//---------------------------------------------------------------------------

bool X6_400MInputDevice::Enable16BitRange()
{
    return DOwner->AdcAmplitudeControlReg.adc_multx16_en.Value() == 0x3;
}

//---------------------------------------------------------------------------
//  X6_400MInputDevice::ActiveDevices() -- Number of active devices
//---------------------------------------------------------------------------

unsigned int  X6_400MInputDevice::ActiveDevices()
{
    return EnableIQMode() ? 1 : ActiveChannels();
}

//===========================================================================
//  CLASS X6_400MDacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class X6_400MDacToHardware
//---------------------------------------------------------------------------

class X6_400MDacToHardware :  public TiDac5682::IDacToHardware
{
public:
    virtual unsigned short  Spi(int addr)
        {  return Out->Spi(Dev, addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Out->Spi(Dev, addr, value);  }

    virtual double          DCLK(bool single_channel)
        {  return MyPll().DCLK(single_channel, MyPll().Frequency()) ;  }
    virtual bool            IsDacDllEnabled(bool single_channel)
        {  return MyPll().IsDacDllEnabled(single_channel);  }
    virtual unsigned int    FastConfigValue()
        {  return 0xC0;  }

    X6_400MDacToHardware( X6_400MOutputDevice * out, unsigned int dev=0 )
        : Out(out), Dev(dev)
        {}
    virtual ~X6_400MDacToHardware()
        {}

    X6_400M_Pll &  MyPll()
        {  return dynamic_cast<X6_400M_Pll &>(Out->DOwner->RawClockDevice().Pll());  }

    X6_400MOutputDevice *  Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS X6_400MOutputDevice  -- User Interface for X6-400M DACs
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_400MOutputDevice
//---------------------------------------------------------------------------

X6_400MOutputDevice::X6_400MOutputDevice(X6_400MDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner),
      FInfo(info), DacDev(new X6_400MDacToHardware(this)),
      CalRomSection(DacCalIdrom(), romX6_400M_OutputCoeffOffset, MaxOutChannels,
                       X6_400M_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_400MOutputDevice
//---------------------------------------------------------------------------

X6_400MOutputDevice::~X6_400MOutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  X6_400MOutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  X6_400MOutputDevice::PreconfigureHardware()
{
    // Set Dac Control Reg back?

    DacDev.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  X6_400MOutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  X6_400MOutputDevice::ConfigureHardware()
{
    //PreconfigureHardware();    // hack for now

	//DacDev.ConfigHardware();

    DacDev.ClearDacTestStatus();
}

//---------------------------------------------------------------------------
//  X6_400MOutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  X6_400MOutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X6_400MOutputDevice::Channels() -- Number of enable-able DAC channels
//---------------------------------------------------------------------------

unsigned int  X6_400MOutputDevice::Channels()
{
    return MaxOutChannels;
}

//---------------------------------------------------------------------------
//  X6_400MOutputDevice::ActiveDevices() -- Number of DAC devices in use
//---------------------------------------------------------------------------

unsigned int  X6_400MOutputDevice::ActiveDevices()
{
    if (ActiveChannels())
        return 1;
    else
        return 0;
}

//---------------------------------------------------------------------------
//  X6_400MOutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  X6_400MOutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    Device(0).Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1));
    Device(0).ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device(0).ChannelEnabled(1, DacCommon().ChannelEnabled(1) );
}


//=============================================================================
//  CLASS X6_400M_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_400M_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//const double  RvcxoFrequency = 400e6;

//---------------------------------------------------------------------------
//  constructor for class X6_400M_Pll
//---------------------------------------------------------------------------

X6_400M_Pll::X6_400M_Pll(X6_400MDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg)
    : Cdce72010(space, reg), Owner(owner), DOps( owner->GetDacOperations() ), FPDF(100.0e3)//,
      //RangedVcxo(RvcxoFrequency, RvcxoFrequency)
{
    SetMasterOutputSelection(DacMasterOutput());  // Use Output 5 as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class X6_400M_Pll
//---------------------------------------------------------------------------

X6_400M_Pll::~X6_400M_Pll()
{
    delete DOps;  DOps = 0;
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_400M_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "X6_400M_Pll::OnInit()");
	
    //  Device Powered/Enabled Setup
    SetPllEnable(true);
    SetFBDividerEnable(true);
    //
    //  Set Input Configuration
    SetInputConfig(icLvPecl);
    SetInputBufferConfig(false,  // AC Terminated
                         false,  // Input Term NOT disabled
                         false,  // Primary Bias OFF
                         true,   // Secondary Bias ON
                         false   // NO failsafe
                         );
    //
    //  Set Vco/Aux Configuration
    SetVcoConfig(vcLvPecl);
    SetVcoBufferConfig(true,   // DC Terminated
                       false,  // Hysteresis
                       false,  // Input Term NOT disabled
                       false   // Vco Bias OFF
                       );
    //
    //  Set Output Configuration
    OutputInUse(0, false);  SetOutputConfig(0, ocDisabled);
	OutputInUse(1, false);  SetOutputConfig(1, ocDisabled);
	OutputInUse(2, true);   SetOutputConfig(2, ocLvds);     // Ref ADC (FPGA)
	OutputInUse(3, true);   SetOutputConfig(3, ocLvds);     // Ref DAC (FPGA)
    OutputInUse(4, true);   SetOutputConfig(4, ocLvds);     // FPGA
    OutputInUse(5, true);   SetOutputConfig(5, ocLvPecl );  // DACs
    OutputInUse(6, true);   SetOutputConfig(6, ocLvPecl);   // ADC Dev 0 (Ch0-1)
    OutputInUse(7, true);   SetOutputConfig(7, ocLvPecl);   // ADC Dev 1 (Ch2-3)
    OutputInUse(8, false);  SetOutputConfig(8, ocDisabled);
    OutputInUse(9, false);  SetOutputConfig(9, ocDisabled);

    //  Phase = 0
    SetPhaseAdjust(2, 1, 0);  SetPhaseAdjust(3, 1, 0);  // Set all outputs to 0 phase
    SetPhaseAdjust(4, 1, 0);  SetPhaseAdjust(5, 1, 0);
    SetPhaseAdjust(6, 1, 0);  SetPhaseAdjust(7, 1, 0);

    //  Configure Aux
    SetAuxInputConfig(aeEnabled, aiNormal);
    SetLockConfig(false);   // NOT analog lock
    //
    //  Set Reduced Frequency Limits for Outputs (Preset to 1500.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6; '
    //      if the physical rate limit is 100 MHz
    float input_max_rate = static_cast<float>(InputMaxRate());
    OutputFrequencyLimit[2] = input_max_rate;  // as in ADCs, but halved (can vary)
    OutputFrequencyLimit[3] = 1000.0e6;
    OutputFrequencyLimit[4] = 1000.0e6;
    OutputFrequencyLimit[5] = 1000.0e6;        // DAC max rate = 1 GHz
    OutputFrequencyLimit[6] = input_max_rate;  // ADC Max Rate = 400 kHz (can vary)
    OutputFrequencyLimit[7] = input_max_rate;
    //
    //  Reference
    SetExternalReferenceControl(true);  // Use external pin
    ReferenceSource(refPrimary);    // Set to use primary reference

}

//---------------------------------------------------------------------------
//  X6_400M_Pll::Vcxo() -- return Vcxo
//---------------------------------------------------------------------------

IVcxoControl &  X6_400M_Pll::Vcxo()
{
    return Owner->AFEClk_SDev.Vcxo();
    //return RangedVcxo;
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::OnReapply()
//---------------------------------------------------------------------------

void  X6_400M_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  X6_400M_Pll::AdcMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  X6_400M_Pll::DacMasterOutput()
{
    return 5;
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  X6_400M_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(2, ADCCLK(adc_rate)/2);
    OutputFrequency(6, ADCCLK(adc_rate));
    OutputFrequency(7, ADCCLK(adc_rate));
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  X6_400M_Pll::SetDacOutputRates(double dac_rate)
{
    bool  single_channel = DOps->IsDacInSingleChannelMode();

    OutputFrequency(3, DCLK(single_channel, dac_rate));
    OutputFrequency(4, DCLK(single_channel, dac_rate));
    OutputFrequency(5, dac_rate);

}

//---------------------------------------------------------------------------
//  X6_400M_Pll::DCLK()  -- DCLK Output Rate
//---------------------------------------------------------------------------

double X6_400M_Pll::DCLK(bool single_channel, double freq)
{
    double FreqMhz = freq/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return FreqMhz/2 * 1.0e6;
        else
            return FreqMhz * 1.0e6;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return FreqMhz * 1.0e6;
        else
            return FreqMhz*2 * 1.0e6;
        }
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::ADCCLK()  -- ADCCLK Output Rate
//---------------------------------------------------------------------------

double X6_400M_Pll::ADCCLK(double freq)
{
    if (freq>InputMaxRate())
        return InputMaxRate();
    else
        return freq;
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::IsDacDllEnabled()  -- Output Divider Settings
//---------------------------------------------------------------------------

bool  X6_400M_Pll::IsDacDllEnabled(bool single_channel)
{
    double FreqMhz = Dac().Frequency()/1e6;

    if (single_channel)
        {
        if (FreqMhz>=250.0)
            return true;
        else
            return false;
        }
    else   // dual dac mode
        {
        if (FreqMhz>=125.0)
            return true;
        else
            return false;
        }
}

//---------------------------------------------------------------------------
//  X6_400M_Pll::InputMaxRate()  -- Return Max Input Rate
//---------------------------------------------------------------------------

double  X6_400M_Pll::InputMaxRate()
{
    return Owner->Input().Info().MaxRate();
}

//=============================================================================
//  CLASS X6_400M_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_400M_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_400M_SioPll
//---------------------------------------------------------------------------

X6_400M_SioPll::X6_400M_SioPll(X6_400MDeviceMap * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  X6_400M_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_400M_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, false);    SetMultiSynthPowerDown(3, false);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);    
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(2, diNoClk);   // override since not using output

    OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
    OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, true);

    SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
    SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
    SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
    SetOutputVoltage(3, vo_2p5volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  X6_400M_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  X6_400M_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS X6_400M_DacOperations  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  declaration of class X6_400M_DacOperations
//---------------------------------------------------------------------------

class X6_400M_DacOperations :  public X6_400M_Pll::IDacOperations
{
public:
    X6_400M_DacOperations( X6_400MDeviceMap * owner )
        :  Owner(owner)
        {}

    virtual bool  IsDacInSingleChannelMode();
    
private:
    X6_400MDeviceMap * Owner;
};

//---------------------------------------------------------------------------
//  X6_400M_DacOperations::IsDacInSingleChannelMode()
//---------------------------------------------------------------------------

bool  X6_400M_DacOperations::IsDacInSingleChannelMode()
{
    bool dac0en = Owner->Output().ChannelEnabled(0);
    bool dac1en = Owner->Output().ChannelEnabled(1);

    return dac0en ^ dac1en;
}

//===========================================================================
//  CLASS X6_400M_DacCalToHdw -- Cal Hdw Interface
//===========================================================================

class X6_400M_DacCalToHdw : public TiDac5682::IDacCalibratorToHdw
{
public:

    virtual void  ApplyDacGainCoefficients()
        {
        Output().Cal().ApplyRomCoefficients();  // will apply default if uncalibrated
        }
    virtual void  DacReset()
        {
        //  DEBUG PRINTS
        unsigned int at805 = Owner->PeekLogic(0x805);
        unsigned int at806 = Owner->PeekLogic(0x806);
        Owner->Debug.Log("Reg 805: ", at805, 16);
        Owner->Debug.Log("Reg 806: ", at806, 16);

        Output().DacCommon().ChannelResetAll();
        }
    virtual void  PreconfigureHardware()
        {
        // Reapply settings for DAC Devices for test
        Output().PreconfigureHardware();
        }
    virtual double        ClockFrequencyActualMhz()
        {  return Owner->Clock().Dac().FrequencyActual()/1e6; }
    virtual void          SetTestMode_Alternating()
        {  PatternCache = 0x6; }
    virtual void          SetTestMode_DoubleAlternating()
        {  PatternCache = 0x7; }
    virtual void          SetTestEnable(bool state)
        {  Output().Common().TestModeEnabled(X6AFE_Common::tOutput, state, PatternCache);  }
    virtual void          SetSoftwareUnframedTrigger()
        {
        Output().DacCommon().EdgeTrigger(false);
        Output().DacCommon().ExternalTrigger(false);
        Output().DacCommon().FramedTriggerEnabled(false);
        }
    virtual void  DeviceRun(bool state, EngDeviceState /*which*/)
        {   Output().Common().Engine(X6AFE_Common::tOutput, state);  }

    virtual TiDac5682::DacDevice &   Dac(unsigned int idx)
        {  return Output().Device(idx);  }


    X6_400M_DacCalToHdw( X6_400MDeviceMap * owner)
        :  Owner(owner), PatternCache(0)
        {}
    virtual ~X6_400M_DacCalToHdw()
        {}

private:
    X6_400MDeviceMap * Owner;
    unsigned int       PatternCache;

    X6_400MOutputDevice & Output()  {  return Owner->Output(); }
};

//=============================================================================
//  CLASS X6_400MDacPriClockDevice  --  Interface to access board clock for PRI
//=============================================================================
//  The calc is the save for both divisors, so lets us the same impl

class X6_400MDacPriClockDevice : public IPriClockDevice, public IFrameCalcDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->AFEClk_SDev.Clock().Frequency();  }
    virtual int   CalculatePriDivisor()
        {
            // Register divisor is a Single Channel and DacDll Mode
            int divisor;
            if (IsDacSingleChannel())
                if (IsDacDllEnabled())
                    divisor = 4;
                else
                    divisor = 2;
            else
                if (IsDacDllEnabled())
                    divisor = 2;
                else
                    divisor = 1;

            return divisor;
        }
    virtual  unsigned int  CalculateFrameDivisor()
        {  return static_cast<unsigned int>(CalculatePriDivisor());  }

    virtual bool IsDacSingleChannel()
        {
        bool dac0en = Owner->Output().ChannelEnabled(0);
        bool dac1en = Owner->Output().ChannelEnabled(1);

        return dac0en ^ dac1en;
        }

    virtual bool IsDacDllEnabled()
        {
        return MyPll().IsDacDllEnabled(IsDacSingleChannel());
        }

    X6_400MDacPriClockDevice(X6_400MDeviceMap * owner)
        :  Owner(owner)
        {}
    virtual ~X6_400MDacPriClockDevice()
        {}

private:
    X6_400MDeviceMap * Owner;

    X6_400M_Pll &  MyPll()
        {  return dynamic_cast<X6_400M_Pll &>(Owner->RawClockDevice().Pll());  }

};

//===========================================================================
//  CLASS X6_400MDeviceMap  -- Register & low level support for X6-400M
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_400MDeviceMap
//---------------------------------------------------------------------------

const double  kVcxoPowerOnRate = 10.0;    // Si570 power on rate (MHz)

static Cdce72010Registers X6_400M_PllRegisters =
    { mmAFE_PllControl,  mmAFE_PllSpi  };

static X6_Std_I2CLinkage::MapRegisters X6_400M_VcoI2CLinkRegAddrs =
    {  mmAFE_VcxoControl,      //  AfeVcxoControl_Addr
    };
    
//---------------------------------------------------------------------------
//  constructor for class X6_400MDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
	#pragma warning(disable : 4355)
#endif

X6_400MDeviceMap::X6_400MDeviceMap(X6_400M * owner)
    :  X6DeviceMap(owner),
       FInput(new X6_400MInputDevice(this, new X6_400M_AnalogInInfo(this))),
       FOutput(new X6_400MOutputDevice(this, new X6_400M_AnalogOutInfo)),
       Owner(owner),
       AdcAmplitudeControlReg(WB_AFE, mmAFE_AdcIQ_ModeEnable)

{
    //
    //  SubDevice Customization
    X6DeviceMap::ConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &X6_400MDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &X6_400MDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X6_400MDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X6_400MDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X6_400MDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X6_400MDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X6_400MDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_400MDeviceMap
//---------------------------------------------------------------------------

X6_400MDeviceMap::~X6_400MDeviceMap()
{
    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OpenHardware()
{
    X6DeviceMap::OpenHardware();

    FInput->Cal().ApplyRomCoefficients();
    FOutput->Cal().ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X6_400MDeviceMap::OnStreamPreconfigHandler()");
    Output().ConfigureChannelEnables();

    FInput->Cal().ApplyRomCoefficients();
    FOutput->Cal().ApplyRomCoefficients();

    // Family-common init
    X6DeviceMap::PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X6_400MDeviceMap::OnStreamConfigureHandler()");
    Output().ConfigureChannelEnables();   // remove when preconfig readied...

    Output().ConfigureHardware();

    // Family-common init
    X6DeviceMap::ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    if (Input().ActiveChannels()==0)
        AFECommon_SDev.Engine(X6AFE_Common::tOutput, true);
    else if (Output().ActiveChannels()==0)
        AFECommon_SDev.Engine(X6AFE_Common::tInput, true);
    else
        AFECommon_SDev.Engine(X6AFE_Common::tInputOutput, true);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Base class "on stream stop" behavior
    X6DeviceMap::StreamStopHardware();

	//  Forward notify to baseboard handler
	OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeAfePllObject() -- Reference to input object
//---------------------------------------------------------------------------

Cdce72010 *  X6_400MDeviceMap::MakeAfePllObject()
{
    //  Make our local PLL object (subdev will delete it)
    return new X6_400M_Pll(this, WB_AFE, X6_400M_PllRegisters );
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeAfeVcxoObject() --  Build custom VCO object
//---------------------------------------------------------------------------

Si570 *  X6_400MDeviceMap::MakeAfeVcxoObject(IRequires_I2C_RegIo & device)
{
    //  Make our local Vcxo object (subdev will delete it)
    return new Si570(device, Si570::tcA, kVcxoPowerOnRate );
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeAfeVcxoI2CLink() --  Build custom Vco I2C Link
//---------------------------------------------------------------------------

IX6_I2C_uDevice *  X6_400MDeviceMap::MakeAfeVcxoI2CLink()
{
    //  Make our local Vcxo I2C Link (subdev will delete it)
    return new X6_Std_I2CLinkage(WB_AFE, X6_400M_VcoI2CLinkRegAddrs);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeAfeAdcSpiDevices() -- Build ADC Spi Devices
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::MakeAfeAdcSpiDevices()
{
    //  Make one SPI device for each ADC device
    /*
    unsigned int Write_Addr = mmAFE_AdcSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_AdcSpiStatBase;

    for (unsigned int i=0; i<X6_400M_AdcSpiDevices; ++i)
        {
        AFEAdcSpi_SDev.MakeSpiDevice("ADC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_AdcSpiCtrlDelta;
        Read_Addr  += mmAFE_AdcSpiStatDelta;
        }
    */
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeAfeDacSpiDevices() -- Build DAC Spi Devices
//---------------------------------------------------------------------------

void  X6_400MDeviceMap::MakeAfeDacSpiDevices()
{
    //  Make one SPI device for each DAC device
    unsigned int Write_Addr = mmAFE_DacSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_DacSpiStatBase;

    for (unsigned int i=0; i<X6_400M_DacSpiDevices; ++i)
        {
        AFEDacSpi_SDev.MakeSpiDevice("DAC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_DacSpiCtrlDelta;
        Read_Addr  += mmAFE_DacSpiStatDelta;
        }
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  X6_400MDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new X6_400M_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_400MDeviceMap::GetVeloInStreamCount()
{
    return X6_400M_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_400MDeviceMap::GetVeloOutStreamCount()
{
    return X6_400M_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  X6_400MDeviceMap::GetVitaInStreamCount()
{
    return X6_400M_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  X6_400MDeviceMap::GetVitaOutStreamCount()
{
    return X6_400M_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_400MDeviceMap::GetCalibrationAdcCount()
{
    return X6_400M_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_400MDeviceMap::GetCalibrationDacCount()
{
    return X6_400M_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_400MDeviceMap::GetAdcCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_400MDeviceMap::GetDacCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetDacLatencyCalibrator() -- Get Latency Cal object
//---------------------------------------------------------------------------

IDacCalibrator *  X6_400MDeviceMap::GetDacLatencyCalibrator()
{
    IDacCalibrationRegister *  CalReg = AFEDacLatencyCal_SDev.GetLatencyRegisterIntf();
    TiDac5682::IDacCalibratorToHdw * CtoH = new X6_400M_DacCalToHdw(this);

    return new TiDac5682::DacCalibrator(CalReg, CtoH);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  X6_400MDeviceMap::MakePriClockDevice(PriDeviceCode which)
{
    if (which==priAdc)
        return new X6StandardPriClockDevice(this, 2);
    else
        return new X6_400MDacPriClockDevice(this);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  X6_400MDeviceMap::MakeFrameCalcDevice(FrameDeviceCode which)
{
    if (which==devAdc)
        return new X6StandardFrameCalcDevice(this, 2);
    else
        return new X6_400MDacPriClockDevice(this);  // does double duty!
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   X6_400MDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   X6_400MDeviceMap::GetPatternModeAlignment()
{
    return 8;   // standard X6 alignment of 8 words
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   X6_400MDeviceMap::GetPatternModeScaleFactor()
{
    return 8;   // standard X6 scale factor of 8 words
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   X6_400MDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return false;  // dual memory
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::GetDacOperations() -- Return Interface for PLLs to DACs
//---------------------------------------------------------------------------

X6_400M_Pll::IDacOperations *  X6_400MDeviceMap::GetDacOperations()
{
    return new X6_400M_DacOperations(this);
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::Input() -- Reference to Input object
//---------------------------------------------------------------------------

X6_400MInputDevice & X6_400MDeviceMap::Input()
{
	return *FInput;
}

//---------------------------------------------------------------------------
//  X6_400MDeviceMap::Output() -- Reference to Output object
//---------------------------------------------------------------------------

X6_400MOutputDevice & X6_400MDeviceMap::Output()
{
	return *FOutput;
}

} // namespace Innovative


