// X6_400MDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X6_400MDeviceMap_MbH
#define X6_400MDeviceMap_MbH

#include <X6_400M_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <TiDac5682z_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X6_400MDeviceMap;

//==============================================================================
//  CLASS X6_400M_Pll -- RX PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X6_400M_Pll : public Cdce72010
{
public:
    X6_400M_Pll(X6_400MDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg);
    virtual ~X6_400M_Pll();

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

    double                  DCLK(bool single_channel, double freq);
    double                  ADCCLK(double freq);
    bool                    IsDacDllEnabled(bool single_channel);
    double                  InputMaxRate();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6_400M_Pll::IDacOperations -- Interface for DAC operations
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class IDacOperations
    {
    public:
        virtual ~IDacOperations()
            {}

        virtual bool  IsDacInSingleChannelMode() = 0;
    };

protected:
    X6_400MDeviceMap * Owner;
    IDacOperations *   DOps;
    //RangedVcxoControl   RangedVcxo;
    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//==============================================================================
//  CLASS X6_400M_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class X6_400M_SioPll : public Si5338
{
public:
    X6_400M_SioPll(X6_400MDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    X6_400MDeviceMap * Owner;
};

//===========================================================================
//  CLASS X6_400MInputDevice  -- User Interface for X6-400M ADCs
//===========================================================================

class X6_400MInputDevice : public X6InputDevice
{
public:
    typedef X6InputDevice inherited;

    X6_400MInputDevice(X6_400MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_400MInputDevice();

    virtual IX6AnalogIoInfo &  Info();
    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int Channels();
    virtual unsigned int ActiveDevices();

    virtual void EnableIQMode(bool value);
    virtual bool EnableIQMode();
    virtual void Enable16BitRange(bool value);
    virtual bool Enable16BitRange();

private:
    X6_400MDeviceMap *     DOwner;
    IX6AnalogIoInfo *      FInfo;
    X6CalibrationSection   CalRomSection;
};

//===========================================================================
//  CLASS X6_400MOutputDevice  -- User Interface for X6-400M DACs
//===========================================================================

class X6_400MOutputDevice : public X6OutputDevice
{
public:
    typedef X6OutputDevice inherited;

    friend class X6_400M_DacCalToHdw;
    friend class X6_400MDacToHardware;

    X6_400MOutputDevice(X6_400MDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_400MOutputDevice();

    virtual IX6AnalogIoInfo &  Info();
    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int       Channels();

    virtual unsigned int  ActiveDevices();

    TiDac5682::DacDevice &     Device(unsigned int /*idx*/)
                                  {  return DacDev;   }

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

private:
    X6_400MDeviceMap *     DOwner;
    IX6AnalogIoInfo *      FInfo;
    TiDac5682::DacDevice   DacDev;
    X6CalibrationSection   CalRomSection;

};

//===========================================================================
//  CLASS X6_400MDeviceMap  -- Register & low level support for X6_400M module
//===========================================================================
class X6_400M;

class X6_400MDeviceMap :  public X6DeviceMap

{
    friend class X6_400M;
    friend class X6_400M_Pll;
    friend class X6_400MInputDevice;
    friend class X6_400MDacPriClockDevice;
    friend class X6_400M_AnalogInInfo;
    
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AdcAmplitudeControlRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcAmplitudeControlRegister : public Innovative::Register
    {
        typedef Register inherited;

    public:
        Innovative::RegisterBitGroup    adc_multx16_en;
        Innovative::RegisterBit         adc_intrlvr_en;

        AdcAmplitudeControlRegister(Innovative::IAddressingSpace &space, int offset)
            : inherited(space, offset),
              adc_multx16_en(*this, 0, 2), adc_intrlvr_en(*this, 2)
            {
            }
    };

public:
    //
    //  Member Functions
    X6_400MDeviceMap(X6_400M * owner);
    virtual ~X6_400MDeviceMap();

    X6_400MInputDevice  &    Input();
    X6_400MOutputDevice &    Output();

public: // pseudo-protected
    virtual void  OpenHardware();

    //  Subdevice Customization Handlers
    virtual Cdce72010 *         MakeAfePllObject();
    virtual Si570 *             MakeAfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *   MakeAfeVcxoI2CLink();
    virtual void                MakeAfeAdcSpiDevices();
    virtual Si5338 *            MakeSioPllObject(I2CDevice & device);
    virtual void                MakeAfeDacSpiDevices();
    virtual IPriClockDevice *   MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *  MakeFrameCalcDevice(FrameDeviceCode which);
    virtual IIsAVitaPacketStreamBoard *           GetBaseboardPtr();
    virtual unsigned int                          GetPatternModeAlignment();
    virtual unsigned int                          GetPatternModeScaleFactor();
    virtual bool                                  GetPatternModeSingleMemoryFlag();

    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
    virtual unsigned int GetVitaInStreamCount();
    virtual unsigned int GetVitaOutStreamCount();

    virtual unsigned int GetCalibrationAdcCount();
    virtual unsigned int GetCalibrationDacCount();

    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    virtual IDacCalibrator * GetDacLatencyCalibrator();
    virtual X6_400M_Pll::IDacOperations *  GetDacOperations();

    IX6RawClockDevices & RawClockDevice()  {  return AFEClk_SDev.RawClockDevice();  } 

protected:
    // Fields
    X6_400MInputDevice  *   FInput;
    X6_400MOutputDevice *   FOutput;

    // Data
    X6_400M *           Owner;
    AdcAmplitudeControlRegister AdcAmplitudeControlReg;
public:
    Trace               Debug;

protected:
    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
