// X6_GSPSDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <X6_GSPS_Mb.h>
#include <X6_GSPSDeviceMap_Mb.h>
#include <X6MemoryMap_Mb.h>
#include <X6_GSPSMemoryMap_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace X6_MemoryMap;
using namespace X6_GSPSMemoryMap;

//---------------------------------------------------------
//  Module Constants Definition, Clock is either external or internal fixed crystal.
//
//  Rev 1 & 2 Logic Differences
//
//  Rev 1
const unsigned int  InputWidth_Bits         = 12;

//
//  Rev 2
const unsigned int  InputWidth_Bits_R2      = 8;

const int    MaxInDevices      = 1;
const int    ChannelsPerDevice = 2;
const int    MaxInChannels     = MaxInDevices*ChannelsPerDevice;

const float         InputMaxRate            = 3600.0e6f;
const float         InputMinRate            = 0.0e6f;

const          int  InputMax_mV             = 500;
const          int  InputMin_mV             = -500;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 8;

const unsigned int X6_GSPS_AdcSpiDevices      = MaxInDevices;
const unsigned int X6GSPS_VeloInStreamCount   = 1;   // Number of Velocia Incoming PIDs
const unsigned int X6GSPS_VitaInStreamCount   = MaxInChannels;   // Number of VITA Incoming SIDs

const unsigned int X6GSPS_AdcCount            = MaxInChannels;  // Number of Adc Channels
const unsigned int X6GSPS_CalibrationAdcCount = MaxInChannels;  // Number of Adc Calibrations

const unsigned int X6_GSPS_DacSpiDevices      = 0;   // No DACs with SPIs

const unsigned int X6GSPS_VeloOutStreamCount  = 1;   // Number of Velo Outgoing PIDs
const unsigned int X6GSPS_VitaOutStreamCount  = 0;   // Number of VITA Outgoing SIDs

const unsigned int X6_GSPS_Adc_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romX6_GSPS_SysInfoOffset        = 0;
const int romX6_GSPS_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romX6_GSPS_InputCoeffOffset       = 0;
const int romX6_GSPS_InputCoeffOffset_avail = MaxInChannels * 2 + 1;   // 2 * 2 coefficients plus 'calibrated' word

//  ...Dac Cal Idrom: DAC coefficients
const int romX6_GSPS_OutputCoeffOffset       = 0;
const int romX6_GSPS_OutputCoeffOffset_avail = 0;  // none used

//==============================================================================
//  CLASS ADC12D1800  -- Register & low level support for GSPS A/D
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class ADC12D1800
//---------------------------------------------------------------------------

ADC12D1800::ADC12D1800(X6_GSPSDeviceMap * owner, X6SpiDevice * spi)
        : Owner(owner), SpiDev(spi)
{
}


//-----------------------------------------------------------------------------
//  ADC12D1800::Port()  --  Write to PLL data port
//-----------------------------------------------------------------------------

void  ADC12D1800::Port(unsigned short reg, unsigned short value)
{
    SpiDev->Spi(reg, value);
}

//-----------------------------------------------------------------------------
//  ADC12D1800::Port()  --  Write to PLL data port
//-----------------------------------------------------------------------------

unsigned short  ADC12D1800::Port(unsigned short reg)
{
    return SpiDev->Spi(reg);
}

//===========================================================================
//  Class X6_GSPS_AnalogInInfo
//===========================================================================

class X6_GSPS_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    X6_GSPS_AnalogInInfo(X6_GSPSDeviceMap * owner) : Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~X6_GSPS_AnalogInInfo()  {  }

    virtual float     MaxRate() const         {  return InputMaxRate;  }
    virtual float     MinRate() const         {  return InputMinRate;  }
    virtual SpanInfo  Span() const            {  return FSpan;  }
    virtual SpanInfo  Span(size_t /*gain_range*/) const
                {  return Span();  }
    virtual unsigned int  Bits() const
        {
        if ((Owner->Info_SDev.FpgaLogicVersion()&0xFF00)==0x200)
            return InputWidth_Bits_R2;
        else
            return InputWidth_Bits;
        }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    X6_GSPSDeviceMap * Owner;
    SpanInfo   FSpan;
};

//===========================================================================
//  CLASS X6_GSPSInputDevice  -- Register & low level support for X6 GSPS
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_GSPSInputDevice
//---------------------------------------------------------------------------

X6_GSPSInputDevice::X6_GSPSInputDevice(X6_GSPSDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), DOwner(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romX6_GSPS_InputCoeffOffset, MaxInChannels,
                       X6_GSPS_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_GSPSInputDevice
//---------------------------------------------------------------------------

X6_GSPSInputDevice::~X6_GSPSInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  X6_GSPSInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::Channels() -- Number of enable-able ADC channels
//---------------------------------------------------------------------------

unsigned int  X6_GSPSInputDevice::Channels()
{
    return MaxInChannels;
}

//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::Pulse() -- Pulse Repetition Interface
//---------------------------------------------------------------------------
//
//IPulseRepetitionIntervalIntf &   X6_GSPSInputDevice::Pulse()
//{
//    return DOwner->AFEGspsPri_SDev;
//}
//
//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::ConfigureHardware()
//---------------------------------------------------------------------------
//
//void X6_GSPSInputDevice::ConfigureHardware()
//{
//    inherited::ConfigureHardware();
//}
//
//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------
// unlike base class, don't power down unused input devices

void  X6_GSPSInputDevice::ChannelDisableAll()
{
     AdcCommon().ChannelDisableAll();
}

//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------
// unlike base class, don't power down unused input devices

void  X6_GSPSInputDevice::ChannelEnabled(int ch, bool state)
{
     AdcCommon().ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  X6_GSPSInputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------
//  Use default behavior

bool X6_GSPSInputDevice::ChannelEnabled(int ch) const
{
    return X6InputDevice::ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  constructor for class X6_GSPSOutputDevice
//---------------------------------------------------------------------------

X6_GSPSOutputDevice::X6_GSPSOutputDevice(X6_GSPSDeviceMap * devMap )
    : inherited(devMap), DOwner(devMap),
      FNullCalRomSection(DacCalIdrom(), 0, 0, 0, DacCal() )
{
    DOwner->EnableDacDevices();
}

//---------------------------------------------------------------------------
//  destructor for class X6_GSPSOutputDevice
//---------------------------------------------------------------------------
X6_GSPSOutputDevice::~X6_GSPSOutputDevice()
{
    }

//---------------------------------------------------------------------------
//  X6_GSPSOutputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void X6_GSPSOutputDevice::PreconfigureHardware()
{
}

//---------------------------------------------------------------------------
//  X6_GSPSOutputDevice::ConfigureHardware()
//---------------------------------------------------------------------------

void X6_GSPSOutputDevice::ConfigureHardware()
{
}

//---------------------------------------------------------------------------
//  X6_GSPSOutputDevice::ConfigureChannelEnables()
//---------------------------------------------------------------------------

void X6_GSPSOutputDevice::ConfigureChannelEnables()
{
}

//=============================================================================
//  CLASS X6_GSPS_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_GSPS_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 10.0e6;

//X6Ad9516_Tb_Hdw::MapRegisters X6GspsClkRegAddrs =
//    {
//    mmAFE_PllControl,       //  AfePllControl_Addr
//    mmAFE_PllSpi,           //  AfePllSpi_Addr
//    };

//---------------------------------------------------------------------------
//  constructor for class X6_GSPS_Pll
//---------------------------------------------------------------------------

X6_GSPS_Pll::X6_GSPS_Pll(X6_GSPSDeviceMap * owner, IAddressingSpace &space, X6Ad9516_Tb_Hdw::MapRegisters & reg)
    : Ad9516_Tb(new X6Ad9516_Tb_Hdw(space, reg)), Owner(owner), FPDF(100.0e3)
{
    SetMasterOutputSelection(AdcMasterOutput());

    ReferenceFrequency(0, Input_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  X6_GSPS_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_GSPS_Pll::OnInit()
{
    //
    //  Set Output Configuration
    OutputInUse(0, true);   OutputEnabled(0, true);  // FPGA
    OutputInUse(1, false);  OutputEnabled(1, false);
    OutputInUse(2, true);   OutputEnabled(2, true);
    OutputInUse(3, false);  OutputEnabled(3, false);
    OutputInUse(4, true);   OutputEnabled(4, true);
    OutputInUse(5, false);  OutputEnabled(5, false);

}

//---------------------------------------------------------------------------
//  X6_GSPS_Pll::OnReapply()
//---------------------------------------------------------------------------

void  X6_GSPS_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  X6_GSPS_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  X6_GSPS_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  X6_GSPS_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  X6_GSPS_Pll::DacMasterOutput()
{
    return FMasterOutput;
}

//---------------------------------------------------------------------------
//  X6_GSPS_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  X6_GSPS_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(0, adc_rate);
    OutputFrequency(2, adc_rate);
    OutputFrequency(4, adc_rate/4);
}

//---------------------------------------------------------------------------
//  X6_GSPS_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  X6_GSPS_Pll::SetDacOutputRates(double /*dac_rate*/)
{
    // no dacs
}

//=============================================================================
//  CLASS X6_GSPS_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_GSPS_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_GSPS_SioPll
//---------------------------------------------------------------------------

X6_GSPS_SioPll::X6_GSPS_SioPll(X6_GSPSDeviceMap * owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  X6_GSPS_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  X6_GSPS_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, false);    SetMultiSynthPowerDown(1, false);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, false);    SetMultiSynthPowerDown(3, false);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(2, diNoClk);   // override since not using output

    OutputInUse(0, true);   SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, true);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, true);
    OutputInUse(2, false);  SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, true);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, true);

    SetOutputTrim(0, ot_2p5lvds);
    SetOutputTrim(1, ot_2p5lvds);
    SetOutputTrim(2, ot_2p5lvds);
    SetOutputTrim(3, ot_2p5lvds);

    SetOutputVoltage(0, vo_2p5volts);
    SetOutputVoltage(1, vo_2p5volts);
    SetOutputVoltage(2, vo_2p5volts);   // actually off
    SetOutputVoltage(3, vo_2p5volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  X6_GSPS_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  X6_GSPS_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS X6AFE_GspsAdcCalibrator -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class X6AFE_GspsAdcCalibrator
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class X6AFE_GspsAdcCalibrator
//---------------------------------------------------------------------------

X6AFE_GspsAdcCalibrator::X6AFE_GspsAdcCalibrator(IPriClockDevice * clk,
                                                 IAddressingSpace & space, MapRegisters & regs)
    :  ClkDev(clk),
       CalControlReg(space, regs.CalControl_Addr),
       Adc0_I_Reg(space, regs.Adc0_I_Addr),
       Adc0_Q_Reg(space, regs.Adc0_Q_Addr),
       Adc1_I_Reg(space, regs.Adc1_I_Addr),
       Adc1_Q_Reg(space, regs.Adc1_Q_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6AFE_GspsAdcCalibrator
//---------------------------------------------------------------------------

X6AFE_GspsAdcCalibrator::~X6AFE_GspsAdcCalibrator()
{
    delete ClkDev;  ClkDev = 0;
}

//---------------------------------------------------------------------------
//  X6AFE_GspsAdcCalibrator::ConstructCustomParts()
//---------------------------------------------------------------------------

void  X6AFE_GspsAdcCalibrator::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  X6AFE_GspsAdcCalibrator::OpenHardware()
//---------------------------------------------------------------------------

void  X6AFE_GspsAdcCalibrator::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  X6AFE_GspsAdcCalibrator::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6AFE_GspsAdcCalibrator::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Calibrate
    Calibrate( static_cast<unsigned int>(ClkDev->ClockFrequency() / 1.0e6) );  // Freq in MHz
}

//---------------------------------------------------------------------------
//  X6AFE_GspsAdcCalibrator::Calibrate()
//---------------------------------------------------------------------------

void  X6AFE_GspsAdcCalibrator::Calibrate(unsigned int sample_freq_mhz)
{
    ScopedTraceRef ccc(Debug, "X6AFE_GspsAdcCalibrator::Calibrate");

    CalControlReg.SampleFreqMHz.Value( sample_freq_mhz );
    CalControlReg.PhyInit.Value( false );
    CalControlReg.PhyInit.Value( true );
    CalControlReg.PhyInit.Value( false );

    Sleep(5);  //  5 ms  wait for calibration

    Debug.Log("ADC Cal/Init Done: ", CalControlReg.InitDone.Value() );
    Debug.Log("CalControlReg: ", CalControlReg.Value(), 16 );

}

//=============================================================================
//  CLASS X6GspsPriClockDevice  --  Interface to access board clock for PRI
//=============================================================================

class X6GspsPriClockDevice : public IPriClockDevice, public IFrameCalcDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->AFEGspsClk_SDev.Clock().Frequency();  }
    virtual int   CalculatePriDivisor()
        {
            // Register divisor is a function of sample rate, since data format
            // from A/D changes with data rate
            int divisor = 8;
            if (ClockFrequency() > DesMinRate)
                divisor *= 2;
            return divisor;
        }
    virtual  unsigned int  CalculateFrameDivisor()
        {  return static_cast<unsigned int>(CalculatePriDivisor());  }

    X6GspsPriClockDevice(X6_GSPSDeviceMap * owner)
        :  Owner(owner), DesMinRate(1800.0e6)
        {}
    virtual ~X6GspsPriClockDevice()
        {}

private:
    X6_GSPSDeviceMap * Owner;
    float  DesMinRate;
};

//=============================================================================
//  CLASS X6_GSPSAdcCalOverride  --  Cal Override for X6-GSPS ADCs
//=============================================================================

class X6_GSPSAdcCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    X6_GSPSAdcCalOverride()
        {}
    virtual ~X6_GSPSAdcCalOverride()
        {}

private:

};

//---------------------------------------------------------------------------
//  X6_GSPSAdcCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  X6_GSPSAdcCalOverride::OverrideOffset(unsigned int value)
{
    int offset = static_cast<int>(value);   // need signed offset
    //  Max offset that can be calibrated by the hardware (120uVsteps)
    int offset_cal_range_counts = static_cast<int>(45.0/((float)InputMax_mV/(1<<(InputWidth_Bits-1))));
    //  Trim range to fit hardware limitations
    if (offset>0 && offset>offset_cal_range_counts)
        offset = offset_cal_range_counts;
    else if (offset<0 && offset>offset_cal_range_counts)
        offset = -offset_cal_range_counts;

    //  Offset is sign magnitude...
    bool sign_is_negative(false);
    if (offset<0)
        {
        sign_is_negative = true;
        offset = -offset;
        }

    //
    //  Convert to a 12 bit range
    float magnitude_value = static_cast<float>(offset) / offset_cal_range_counts *4095;

    MemoryBitManipulator OffsetReg(0);
    OffsetReg.Bit(12, sign_is_negative);
    OffsetReg.Field(0, 12, static_cast<unsigned int>(magnitude_value));

    return OffsetReg.Value();
}

//---------------------------------------------------------------------------
//  X6_GSPSAdcCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  X6_GSPSAdcCalOverride::OverrideGain(unsigned int value)
{
    //const float HdwStep_mV = 0.0122f;  // 16384 steps over 200 mV range
    const float adjustment_range_mv = 100.0;
    const float adjustment_base_mv = 0.8*InputMax_mV;

    float gain = static_cast<float>(value)/X6_GSPS_Adc_One;
    if (gain<=1.0)
        return 0x7FFF;   // no gain shift
    if (gain>=1.25)
        return 0x4000;   // max gain shift

    float full_range_mV =  InputMax_mV;
    float adjustment_mV =  full_range_mV/gain - adjustment_base_mv;

    unsigned int hdw_steps = static_cast<unsigned int>((16384.0*adjustment_mV)/adjustment_range_mv);

    MemoryBitManipulator GainReg(0);
    GainReg.Bit(14, true);
    GainReg.Field(0, 14, hdw_steps);

    return GainReg.Value();
}

//===========================================================================
//  CLASS X6_GSPSDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class X6_GSPSDeviceMap
//---------------------------------------------------------------------------

static X6Ad9516_Tb_Hdw::MapRegisters X6_GSPS_PllRegisters =
    { mmAFE_PllControl,  mmAFE_PllSpi  };

static X6AFE_GspsAdcCalibrator::MapRegisters X6_GSPS_AdcCalRegAddrs =
    { mmAFE_GSPS_AdcCalControl,    mmAFE_GSPS_Adc0_I,
      mmAFE_GSPS_Adc0_Q, mmAFE_GSPS_Adc1_I, mmAFE_GSPS_Adc1_Q   };

//  Default VCO Range
static const double Dash_Four_VcoLow  = 1450.0e6;     // AD9516-4  values
static const double Dash_Four_VcoHigh = 1800.0e6;

//  Variant 1 VCO Range
static const double Dash_One_VcoLow  = 2300.0e6;      // AD9516-1  values
static const double Dash_One_VcoHigh = 2650.0e6;

//---------------------------------------------------------------------------
//  constructor for class X6_GSPSDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

X6_GSPSDeviceMap::X6_GSPSDeviceMap(X6_GSPS * owner)
    :  X6DeviceMap(owner),
       AFEGspsClk_SDev(this, this),
       AFEGspsAdcCal_SDev(new X6GspsPriClockDevice(this), WB_AFE, X6_GSPS_AdcCalRegAddrs),
       FInput(new X6_GSPSInputDevice(this, new X6_GSPS_AnalogInInfo(this))),
       FOutput(new X6_GSPSOutputDevice(this)),
       AdcDev_0(this, 0),    // Spi Device not made yet
       AdcDev_1(this, 0),
       Owner(owner)
{
    //
    //  Since this device has no DACs, disable all DAC subdevices
    DisableDacDevices();

    AFEClk_SDev.DeviceEnabled(false);   // don't use standard clock
    //
    //  SubDevice Customization
    LocalConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &X6_GSPSDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &X6_GSPSDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &X6_GSPSDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &X6_GSPSDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &X6_GSPSDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &X6_GSPSDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &X6_GSPSDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class X6_GSPSDeviceMap
//---------------------------------------------------------------------------

X6_GSPSDeviceMap::~X6_GSPSDeviceMap()
{
    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------
//  We can't make this part of virtual tree for ConstructCustomParts since
//   its being called in the constructor and the tree is not built yet.

void  X6_GSPSDeviceMap::LocalConstructCustomParts()
{
    X6DeviceMap::ConstructCustomParts();

    AFEGspsClk_SDev.ConstructCustomParts();
    AFEGspsAdcCal_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OpenHardware()
{
    X6DeviceMap::OpenHardware();
    FInput->Cal().ApplyRomCoefficients();

    AFEGspsClk_SDev.OpenHardware();
    AFEGspsAdcCal_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::PreconfigureHardware()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::PreconfigureHardware()
{
    AFEGspsClk_SDev.PreconfigureHardware();

    FInput->Cal().ApplyRomCoefficients();

    X6DeviceMap::PreconfigureHardware();

    AFEGspsAdcCal_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::ConfigureHardware()
{
    X6DeviceMap::ConfigureHardware();

    AFEGspsClk_SDev.ConfigureHardware();
    AFEGspsAdcCal_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    // Family-common init
    ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    AFECommon_SDev.Engine(X6AFE_Common::tInput, true);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Base class "on stream stop" behavior
    StreamStopHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeAfePllObject() -- Reference to input object
//---------------------------------------------------------------------------

Cdce72010 *  X6_GSPSDeviceMap::MakeAfePllObject()
{
    //  Make our local PLL object (subdev will delete it)
    return 0;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeAfeVcxoObject() --  Build custom VCO object
//---------------------------------------------------------------------------

Si570 *  X6_GSPSDeviceMap::MakeAfeVcxoObject(IRequires_I2C_RegIo & /*device*/)
{
    //  We don't use this kind of Vcxo object (subdev will delete it)
    return 0;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeAfeVcxoI2CLink() --  Build custom Vco I2C Link
//---------------------------------------------------------------------------

IX6_I2C_uDevice *  X6_GSPSDeviceMap::MakeAfeVcxoI2CLink()
{
    //  We don't use this kind of Vcxo Link (subdev will delete it)
    //  Make our local Vcxo I2C Link (subdev will delete it)
    return 0;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeAfe_Ad951xPllObject() -- Make Ad9516 Pll
//---------------------------------------------------------------------------

Ad9516_Tb *  X6_GSPSDeviceMap::MakeAfe_Ad951xPllObject()
{
    //  Make our local PLL object (subdev will delete it)
    Ad9516_Tb * pll = new X6_GSPS_Pll(this, WB_AFE, X6_GSPS_PllRegisters );

    //  If we have a non-default variant, change the Vcxo Range
    if (Info().FpgaHardwareVariant()==0x1)
        {
        pll->VcoRange(Dash_One_VcoLow, Dash_One_VcoHigh);
        }
    return pll;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeAfeAdcSpiDevices() -- Build ADC Spi Devices
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::MakeAfeAdcSpiDevices()
{
    //  Make one SPI device for each ADC device
    unsigned int Write_Addr = mmAFE_AdcSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_AdcSpiStatBase;

    for (unsigned int i=0; i<X6_GSPS_AdcSpiDevices; ++i)
        {
        AFEAdcSpi_SDev.MakeSpiDevice("ADC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_AdcSpiCtrlDelta;
        Read_Addr  += mmAFE_AdcSpiStatDelta;
        }

    // initialize the Adc Devices
    AdcDev(0).LoadSpiDevice(AFEAdcSpi_SDev.SpiDevice(0));
    AdcDev(1).LoadSpiDevice(AFEAdcSpi_SDev.SpiDevice(1));
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeAfeDacSpiDevices() -- Build DAC Spi Devices
//---------------------------------------------------------------------------

void  X6_GSPSDeviceMap::MakeAfeDacSpiDevices()
{
/*  ...No DAC SPI for X6_GSPS
    //  Make one SPI device for each DAC device
    unsigned int Write_Addr = mmAFE_DacSpiCtrlBase;
    unsigned int Read_Addr  = mmAFE_DacSpiStatBase;

    for (unsigned int i=0; i<X6_GSPS_DacSpiDevices; ++i)
        {
        AFEDacSpi_SDev.MakeSpiDevice("DAC", Write_Addr, Read_Addr);
        Write_Addr += mmAFE_DacSpiCtrlDelta;
        Read_Addr  += mmAFE_DacSpiStatDelta;
        }
*/
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  X6_GSPSDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new X6_GSPS_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_GSPSDeviceMap::GetVeloInStreamCount()
{
    return X6GSPS_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  X6_GSPSDeviceMap::GetVeloOutStreamCount()
{
    return X6GSPS_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetVitaInStreamCount() -- Return Vita In SIDs
//---------------------------------------------------------------------------

unsigned int  X6_GSPSDeviceMap::GetVitaInStreamCount()
{
    return X6GSPS_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetVitaOutStreamCount() -- Return Vita Out SIDs
//---------------------------------------------------------------------------

unsigned int  X6_GSPSDeviceMap::GetVitaOutStreamCount()
{
    return X6GSPS_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetCalibrationAdcCount() -- Return ADCs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_GSPSDeviceMap::GetCalibrationAdcCount()
{
    return X6GSPS_VitaInStreamCount;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  X6_GSPSDeviceMap::GetCalibrationDacCount()
{
    return X6GSPS_VitaOutStreamCount;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_GSPSDeviceMap::GetAdcCalibrationOverride()
{
    return new X6_GSPSAdcCalOverride;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  X6_GSPSDeviceMap::GetDacCalibrationOverride()
{
    return 0;   // no override
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetDacLatencyCalibrator() -- Get Latency Cal object
//---------------------------------------------------------------------------

IDacCalibrator *  X6_GSPSDeviceMap::GetDacLatencyCalibrator()
{
    // No dacs, no calibrator needed
    return 0;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  X6_GSPSDeviceMap::MakePriClockDevice(PriDeviceCode /*which*/)
{
    // return same object for both directions
    return new X6GspsPriClockDevice(this);
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  X6_GSPSDeviceMap::MakeFrameCalcDevice(FrameDeviceCode /*which*/)
{
    // return same object for both directions
    return new X6GspsPriClockDevice(this);   // does double duty!
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetBaseboardPtr() -- Get Baseboard Device object ptr
//---------------------------------------------------------------------------

IIsAVitaPacketStreamBoard *   X6_GSPSDeviceMap::GetBaseboardPtr()
{
    return Owner;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   X6_GSPSDeviceMap::GetPatternModeAlignment()
{
    return 8;   // standard X6 alignment of 8 words
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   X6_GSPSDeviceMap::GetPatternModeScaleFactor()
{
    return 8;   // standard X6 scale factor of 8 words
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::GetPatternModeSingleMemoryFlag() --
//---------------------------------------------------------------------------
//  return true if we use a single memory for pattern mode

bool   X6_GSPSDeviceMap::GetPatternModeSingleMemoryFlag()
{
    return false;  // dual memory
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::Input() -- Reference to input object
//---------------------------------------------------------------------------

X6_GSPSInputDevice & X6_GSPSDeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  X6_GSPSDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

X6_GSPSOutputDevice & X6_GSPSDeviceMap::Output()
{
    return *FOutput;
}

} // namespace Innovative


