// X6_GSPSDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X6_GSPSDeviceMap_MbH
#define X6_GSPSDeviceMap_MbH

#include <X6_GSPS_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <Trace_Mb.h>


namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class X6_GSPSDeviceMap;

//==============================================================================
//  CLASS ADC12D1800  -- Register & low level support for GSPS A/D
//==============================================================================

class ADC12D1800
{
public:
    ADC12D1800(X6_GSPSDeviceMap * owner, X6SpiDevice * spi);

    // Properties

    // Methods

    void  LoadSpiDevice(X6SpiDevice * spi)
            {  SpiDev = spi;  }

protected:

    // Data
    X6_GSPSDeviceMap *      Owner;
    X6SpiDevice *           SpiDev;
    Trace                   Debug;

public:
    //  Raw configuration accesses
    void            Port(unsigned short reg, unsigned short value);
    unsigned short  Port(unsigned short reg);
};

//==============================================================================
//  CLASS X6_GSPS_Pll -- GSPS PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X6_GSPS_Pll : public Ad9516_Tb
{
public:
    X6_GSPS_Pll(X6_GSPSDeviceMap * owner, IAddressingSpace &space, X6Ad9516_Tb_Hdw::MapRegisters & reg);

    // ...timebase interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

//    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

protected:
    X6_GSPSDeviceMap * Owner;
    double  FTrueFrequency;
    double              FPDF;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//==============================================================================
//  CLASS X6_GSPS_SioPll -- GSPS SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class X6_GSPS_SioPll : public Si5338
{
public:
    X6_GSPS_SioPll(X6_GSPSDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    X6_GSPSDeviceMap * Owner;
};

//===========================================================================
//  CLASS X6AFE_GspsAdcCalibrator -- X6 Gsps ADC Calibration "SubDevice"
//===========================================================================

class X6AFE_GspsAdcCalibrator : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS X6AFE_GspsAdcCalibrator::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   CalControl_Addr;
        unsigned int   Adc0_I_Addr;
        unsigned int   Adc0_Q_Addr;
        unsigned int   Adc1_I_Addr;
        unsigned int   Adc1_Q_Addr;
    };

    X6AFE_GspsAdcCalibrator(IPriClockDevice * clk,
                            IAddressingSpace & space, MapRegisters & regs);
    virtual ~X6AFE_GspsAdcCalibrator();

    void    Calibrate(unsigned int sample_freq_mhz);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    IPriClockDevice *  ClkDev;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_GspsAdcCalibrator::CalControlRegister  -- Control Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CalControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       PhyInit;
        RegisterBit       CalDone;
        RegisterBit       PhyDone;
        RegisterBit       InitDone;
        RegisterBitGroup  SampleFreqMHz;

    public:
        CalControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               PhyInit(*this, 0),   CalDone(*this, 4),
               PhyDone(*this, 5),   InitDone(*this, 6),
               SampleFreqMHz(*this, 8, 12)
            {
            }
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS X6AFE_GspsAdcCalibrator::CalResultRegister  -- Result Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CalResultRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  D_D_Result;
        RegisterBitGroup  D_Result;
        RegisterBit       OR_Result;

    public:
        CalResultRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               D_D_Result(*this, 0, 12),   D_Result(*this, 12, 12),
               OR_Result(*this, 24)
            {
            }
    };


    CalControlRegister  CalControlReg;
    CalResultRegister   Adc0_I_Reg;
    CalResultRegister   Adc0_Q_Reg;
    CalResultRegister   Adc1_I_Reg;
    CalResultRegister   Adc1_Q_Reg;
};

//===========================================================================
//  CLASS X6_GSPSInputDevice  -- Register & low level support for X6-GSPS ADCs
//===========================================================================

class X6_GSPSInputDevice : public X6InputDevice
{
public:
    typedef X6InputDevice inherited;

    X6_GSPSInputDevice(X6_GSPSDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_GSPSInputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int       Channels();

    //IPulseRepetitionIntervalIntf &  Pulse();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;


private:
    X6_GSPSDeviceMap *     DOwner;
    IX6AnalogIoInfo *      FInfo;
    X6CalibrationSection   CalRomSection;
};

//===========================================================================
//  CLASS X6_GSPSOutputDevice  --
//===========================================================================

class X6_GSPSOutputDevice : public X6OutputDevice
{
public:
    typedef X6OutputDevice inherited;

    X6_GSPSOutputDevice(X6_GSPSDeviceMap * owner );
    virtual ~X6_GSPSOutputDevice();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    void  ConfigureChannelEnables();

    virtual IX6AnalogIoInfo &  Info()
        {   return FNullInfo; }

    virtual X6CalibrationSection & Cal()
        {  return FNullCalRomSection;  }

    virtual unsigned int       Channels()
        { return 1; }

    virtual unsigned int  ActiveDevices()
        { return 1; }

protected:
    class NullInfo : public IX6AnalogIoInfo
        {
        public:

            virtual ~NullInfo()
            {  }
            virtual float   MaxRate() const
            { return 0.0f; }
            virtual float   MinRate() const
            { return 0.0f; }
            virtual SpanInfo  Span() const
            { return mSpan; }
			virtual SpanInfo  Span(size_t /*gain_range*/) const
						{  return Span();  }
			virtual unsigned int  Bits() const
			{ return 32; }
            virtual unsigned int  SamplesPerWord() const
            { return 0; }
            virtual unsigned int  TriggerFrameGranularity() const
            { return 0; }

        private:
            SpanInfo mSpan;
		};



private:
    X6_GSPSDeviceMap *     DOwner;
    X6CalibrationSection   FNullCalRomSection;
    NullInfo               FNullInfo;
};

//===========================================================================
//  CLASS X6_GSPSDeviceMap  -- Register & low level support for X6_GSPS module
//===========================================================================
class X6_GSPS;

class X6_GSPSDeviceMap :  public X6DeviceMap, public IAd9516Factory
{
    friend class X6_GSPS;
    friend class X6_GSPS_Pll;
    friend class X6_GSPSInputDevice;
    friend class X6_GSPSOutputDevice;
    friend class X6GspsPriClockDevice;
    friend class X6_GSPS_AnalogInInfo;

public:
    //
    //  Member Functions
    X6_GSPSDeviceMap(X6_GSPS * owner);
    virtual ~X6_GSPSDeviceMap();

    X6_GSPSInputDevice  &    Input();
    X6_GSPSOutputDevice &    Output();

    ADC12D1800 &  AdcDev(unsigned int idx)
        {
        if (idx)
           return AdcDev_1;
        else
           return AdcDev_0;
        }

public: // pseudo-protected
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //  Subdevice Customization Handlers
    virtual Cdce72010 *         MakeAfePllObject();
    virtual Si570 *             MakeAfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *   MakeAfeVcxoI2CLink();
    virtual void                MakeAfeAdcSpiDevices();
    virtual Si5338 *            MakeSioPllObject(I2CDevice & device);
    virtual void                MakeAfeDacSpiDevices();
    virtual IPriClockDevice *   MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *  MakeFrameCalcDevice(FrameDeviceCode which);
    virtual IIsAVitaPacketStreamBoard *           GetBaseboardPtr();
    virtual unsigned int                          GetPatternModeAlignment();
    virtual unsigned int                          GetPatternModeScaleFactor();
    virtual bool                                  GetPatternModeSingleMemoryFlag();

    virtual Ad9516_Tb *  MakeAfe_Ad951xPllObject();

    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
    virtual unsigned int GetVitaInStreamCount();
    virtual unsigned int GetVitaOutStreamCount();

    virtual unsigned int GetCalibrationAdcCount();
    virtual unsigned int GetCalibrationDacCount();

    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    virtual IDacCalibrator * GetDacLatencyCalibrator();

protected:
    //  Subdevices
    X6AFE_Ad9516Clock               AFEGspsClk_SDev;
    X6AFE_GspsAdcCalibrator         AFEGspsAdcCal_SDev;
    // Fields
    X6_GSPSInputDevice  *    FInput;
    X6_GSPSOutputDevice *    FOutput;
    ADC12D1800               AdcDev_0;
    ADC12D1800               AdcDev_1;

    // Data
    X6_GSPS *           Owner;
    Trace               Debug;

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void  LocalConstructCustomParts();
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
