//---------------------------------------------------------------------------
// X6_GSPS_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2011 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X6_GSPS_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6_GSPS  -- X6_GSPS PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_GSPS
//---------------------------------------------------------------------------

X6_GSPS::X6_GSPS()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_GSPS 
//---------------------------------------------------------------------------

X6_GSPS::~X6_GSPS()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_GSPS::Clock() -- Override default clock to use special one
//---------------------------------------------------------------------------

IX6ClockIo & X6_GSPS::Clock()
{
    return Device.AFEGspsClk_SDev.Clock();
}

//---------------------------------------------------------------------------
//  X6_GSPS::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6_Ad951x_RawClockDevices & X6_GSPS::RawClockDevice()
{
    return Device.AFEGspsClk_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_GSPS::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X6_GSPSInputDevice & X6_GSPS::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X6_GSPS::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_GSPS::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo  2 constant SioPll");
    engine.Evaluate("  3 constant Adc0Spi ");

    engine.Evaluate(": ?X6_GSPS "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Vcxo  SioPll Adc0Spi  Adc1Spi\" "
                    "    cr  "
                    "    cr .\" ---- X6_GSPS Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X6_GSPS  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate(": ?X6-GSPS  ?X6_GSPS  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("X6_GSPS-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  X6_GSPS::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X6_GSPS::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kAdc0Spi = 3;

    switch (dev)
        {
        case kVcxo:   // can't be programmed
        default:
            return 0;
        case kPll:
            return Device.AFEGspsClk_SDev.Pll().Port(addr);
        case kSioPll:
            return Device.SioPll_SDev.SioPll_I2C.Reg(addr);
        case kAdc0Spi:
            return Device.AdcDev(0).Port(addr);
        }
}

//---------------------------------------------------------------------------
//  X6_GSPS::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X6_GSPS::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    const int kAdc0Spi = 3;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEGspsClk_SDev.Pll().Port(addr, value);
            break;
        case kVcxo:    // can't be programmed
            break;
        case kSioPll:
            Device.SioPll_SDev.SioPll_I2C.Reg(addr, value);
            break;
        case kAdc0Spi:
            Device.AdcDev(0).Port(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_GSPS::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X6_GSPS::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEGspsClk_SDev.Pll().Frequency(freq);
            Device.AFEGspsClk_SDev.Pll().Reapply();
            break;
        case kVcxo:    // Ranged VCOs can't be tuned
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_GSPS::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X6_GSPS::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;
    switch (dev)
        {
        case kVcxo:         // can't be read back
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.AFEGspsClk_SDev.Pll().FrequencyActual());
        }
}

//---------------------------------------------------------------------------
//  constructor for class X6_GSPSPatternPlayer
//---------------------------------------------------------------------------
X6_GSPSPatternPlayer::X6_GSPSPatternPlayer()
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_GSPSPatternPlayer
//---------------------------------------------------------------------------
X6_GSPSPatternPlayer::~X6_GSPSPatternPlayer()
{
}


//---------------------------------------------------------------------------
//  X6_GSPSPatternPlayer::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

X6_GSPSOutputDevice & X6_GSPSPatternPlayer::Output()
{
    return Device.Output();
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
