// X6_RXDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef X6_RXDeviceMap_MbH
#define X6_RXDeviceMap_MbH

#include <X6_RX_Mb.h>
#include <X6DeviceMap_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class X6_RXDeviceMap;

//==============================================================================
//  CLASS X6_RX_Pll -- RX PLL Chip (Specialization of Cdce72010)
//==============================================================================

class X6_RX_Pll : public Cdce72010
{
public:
    X6_RX_Pll(X6_RXDeviceMap * owner, IAddressingSpace &space, Cdce72010Registers & reg);

    // ...Cdce72010 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual IVcxoControl &  Vcxo();

    virtual double          PhaseDetectorFrequency() const
                                {  return FPDF;   }
    virtual void            SetPhaseDetectorFrequency(double freq)
                               {  FPDF = freq;   }

protected:
    X6_RXDeviceMap * Owner;
    double              FPDF;

    virtual unsigned int AdcMasterOutput()
        {  return FMasterOutput;  }
    virtual unsigned int DacMasterOutput()
        {  return FMasterOutput;  }

    virtual void  SetAdcOutputRates(double adc_rate)
        {  OutputFrequency(AdcMasterOutput(), adc_rate);  }
    virtual void  SetDacOutputRates(double /*dac_rate*/)
        {}

};

//==============================================================================
//  CLASS X6_RX_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class X6_RX_SioPll : public Si5338
{
public:
    X6_RX_SioPll(X6_RXDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    X6_RXDeviceMap * Owner;
};

//===========================================================================
//  CLASS X6_RXInputDevice  -- Register & low level support for X6-RX ADCs
//===========================================================================

class X6_RXInputDevice : public X6InputDevice
{
public:
    typedef X6InputDevice inherited;

    X6_RXInputDevice(X6_RXDeviceMap * owner, IX6AnalogIoInfo * info );
    virtual ~X6_RXInputDevice();

    virtual IX6AnalogIoInfo &  Info();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual unsigned int       Channels();

private:
    IX6AnalogIoInfo *      FInfo;
    X6CalibrationSection   CalRomSection;
};

//===========================================================================
//  CLASS X6_RXDeviceMap  -- Register & low level support for X6_RX module
//===========================================================================
class X6_RX;

class X6_RXDeviceMap :  public X6DeviceMap

{
    friend class X6_RX;
    friend class X6_RX_Pll;
    friend class X6_RXInputDevice;

public:
    //
    //  Member Functions
    X6_RXDeviceMap(X6_RX * owner);
    virtual ~X6_RXDeviceMap();

    X6_RXInputDevice  &    Input();

public: // pseudo-protected
    virtual void  OpenHardware();

    //  Subdevice Customization Handlers
    virtual Cdce72010 *         MakeAfePllObject();
    virtual Si570 *             MakeAfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *   MakeAfeVcxoI2CLink();
    virtual void                MakeAfeAdcSpiDevices();
    virtual Si5338 *            MakeSioPllObject(I2CDevice & device);
    virtual void                MakeAfeDacSpiDevices();
    virtual IPriClockDevice *   MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *  MakeFrameCalcDevice(FrameDeviceCode which);
    virtual IIsAVitaPacketStreamBoard *           GetBaseboardPtr();
    virtual unsigned int                          GetPatternModeAlignment();
    virtual unsigned int                          GetPatternModeScaleFactor();
    virtual bool                                  GetPatternModeSingleMemoryFlag();

    virtual unsigned int GetVeloInStreamCount();
    virtual unsigned int GetVeloOutStreamCount();
    virtual unsigned int GetVitaInStreamCount();
    virtual unsigned int GetVitaOutStreamCount();

    virtual unsigned int GetCalibrationAdcCount();
    virtual unsigned int GetCalibrationDacCount();

    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    virtual IDacCalibrator * GetDacLatencyCalibrator();

protected:
    // Fields
    X6_RXInputDevice  *    FInput;

    // Data
    X6_RX *           Owner;
    Trace             Debug;

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // Innovative
//---------------------------------------------------------------------------
#endif
