//---------------------------------------------------------------------------
// X6_RX_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2010 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <X6_RX_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS X6_RX  -- X6_RX PCI Express baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6_RX
//---------------------------------------------------------------------------

X6_RX::X6_RX()
    :  X6Board(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class X6_RX 
//---------------------------------------------------------------------------

X6_RX::~X6_RX()
{
    Close();
}

//---------------------------------------------------------------------------
//  X6_RX::Input() --  Return reference to input object
//---------------------------------------------------------------------------

X6_RXInputDevice & X6_RX::Input()
{
	return Device.Input();
}

//---------------------------------------------------------------------------
//  X6_RX::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

IX6RawClockDevices & X6_RX::RawClockDevice()
{
    return Device.AFEClk_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  X6_RX::AfterFiclStart() --
//---------------------------------------------------------------------------

void  X6_RX::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo  2 constant SioPll");

    engine.Evaluate(": ?X6_RX "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Vcxo  SioPll\" "
                    "    cr  "
                    "    cr .\" ---- X6_RX Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?X6_RX  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate(": ?X6-RX  ?X6_RX  ; "  );    // alias
    engine.Evaluate("marker app");

    engine.Display("X6_RX-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    X6Board::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  X6_RX::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  X6_RX::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return Device.AFEClk_SDev.Pll().Port(addr);
        case kVcxo:
            return Device.AFEClk_SDev.Vcxo().Port(addr);
        case kSioPll:
            return Device.SioPll_SDev.SioPll_I2C.Reg(addr);
        }
}

//---------------------------------------------------------------------------
//  X6_RX::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  X6_RX::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;

    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEClk_SDev.Pll().Port(addr, value);
            break;
        case kVcxo:
            Device.AFEClk_SDev.Vcxo().Port(addr, value);
            break;
        case kSioPll:
            Device.SioPll_SDev.SioPll_I2C.Reg(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_RX::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  X6_RX::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    switch (dev)
        {
        default:
            break;
        case kPll:
            Device.AFEClk_SDev.Pll().Frequency(freq);
            Device.AFEClk_SDev.Pll().Reapply();
            break;
        case kVcxo:
            Device.AFEClk_SDev.Vcxo().Frequency(freq);
            break;
        case kSioPll:
            Device.SioPll_SDev.Pll().Frequency(freq);
            Device.SioPll_SDev.Pll().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  X6_RX::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  X6_RX::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kSioPll = 2;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(Device.AFEClk_SDev.Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(Device.AFEClk_SDev.Vcxo().FrequencyActual());
        case kSioPll:
            return static_cast<float>(Device.SioPll_SDev.Pll().FrequencyActual());
        }
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
