//---------------------------------------------------------------------------
// XAFamilyBoard_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#include <XAFamilyBoard_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XAFamilyBoard  -- XAFamily Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XAFamilyBoard
//---------------------------------------------------------------------------

XAFamilyBoard::XAFamilyBoard(XAFamilyDeviceMap & deviceMap)
    :  IIsAVitaPacketStreamBoard(deviceMap), XADeviceRef(deviceMap)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAFamilyBoard
//---------------------------------------------------------------------------

XAFamilyBoard::~XAFamilyBoard()
{
    // Don't call anything using XADeviceRef here - put in most derived class
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::BoardCount() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

size_t XAFamilyBoard::BoardCount()
{
    return XADeviceRef.BoardCount();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::BoardNames() -- Number of enumerated baseboards
//---------------------------------------------------------------------------

vector<string> XAFamilyBoard::BoardNames()
{
    return XADeviceRef.Driver().ODevice().GetDeviceNames();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::PrintAllDevices() -- Print Device Names and Target #
//---------------------------------------------------------------------------

string XAFamilyBoard::PrintAllDevices()
{
    return XADeviceRef.Driver().ODevice().PrintAllDevices();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::DriverDate() --
//---------------------------------------------------------------------------

string XAFamilyBoard::DriverDate()
{
    return XADeviceRef.Driver().ODevice().DriverDate();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::DriverVersion() --
//---------------------------------------------------------------------------

string XAFamilyBoard::DriverVersion()
{
    return XADeviceRef.Driver().ODevice().DriverVersion();
}
//---------------------------------------------------------------------------
//  XAFamilyBoard::LicenseCheck() -- Check WD License
//---------------------------------------------------------------------------

bool  XAFamilyBoard::LicenseCheck(unsigned int & /*code*/)
{
    return true;
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Target() -- Get the target number of the board
//---------------------------------------------------------------------------

int  XAFamilyBoard::Target()
{
    return XADeviceRef.Target();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Target() -- Set the target number of the board
//---------------------------------------------------------------------------

void  XAFamilyBoard::Target( int value )
{
    XADeviceRef.Target(value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  XAFamilyBoard::Led()
{
    return XADeviceRef.Led();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  XAFamilyBoard::Led( bool value )
{
    XADeviceRef.Led(value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  XAFamilyBoard::GreenLed()
{
    return XADeviceRef.Periph_SDev.GreenLed();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  XAFamilyBoard::GreenLed( bool value )
{
    XADeviceRef.Periph_SDev.GreenLed(value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::FrontPanelLed() -- Read the FrontPanel LED state
//---------------------------------------------------------------------------

bool  XAFamilyBoard::FrontPanelLed()
{
    return XADeviceRef.Periph_SDev.FrontPanelLed();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::FrontPanelLed() -- Set the FrontPanel LED state of the board
//---------------------------------------------------------------------------

void  XAFamilyBoard::FrontPanelLed( bool value )
{
    XADeviceRef.Periph_SDev.FrontPanelLed(value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Open() -- Open the baseboard
//---------------------------------------------------------------------------

void  XAFamilyBoard::Open()
{
    XADeviceRef.Open();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Close() -- Close the baseboard
//---------------------------------------------------------------------------

void  XAFamilyBoard::Close()
{
    XADeviceRef.Close();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Reset() -- Reset the baseboard
//---------------------------------------------------------------------------

//void  XAFamilyBoard::Reset()
//{
//    XADeviceRef.Reset();    // Board Reset
//}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Internals() -- Internal Packet Stream Interface
//---------------------------------------------------------------------------

IPacketStreamDeviceInternals *  XAFamilyBoard::Internals()
{
    return &XADeviceRef;
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Log() -- Log an entry to debug log
//---------------------------------------------------------------------------

void  XAFamilyBoard::Log( int type, int value )
{
    XADeviceRef.Log(type, value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

IFpaLoader & XAFamilyBoard::VsProm()
{
    return XADeviceRef.VsProm();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::VeloIn() -- Return reference to Velocia Packetizer Control
//---------------------------------------------------------------------------

IVeloPacketizer & XAFamilyBoard::VeloIn()
{
    return XADeviceRef.VeloIn();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::VitaIn_TS() -- Return reference to Vita Timestamp Control
//---------------------------------------------------------------------------

//IVitaTimestamp & XAFamilyBoard::VitaIn_TS()
//{
//    return XADeviceRef.VitaIn_TS();
//}

//---------------------------------------------------------------------------
//  XAFamilyBoard::VeloOut() -- Return reference to Velocia Depacketizer Control
//---------------------------------------------------------------------------

IVeloDepacketizer & XAFamilyBoard::VeloOut()
{
    return XADeviceRef.VeloOut();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Alerts() -- Return reference to Alert object
//---------------------------------------------------------------------------

IUsesX6Alerts & XAFamilyBoard::Alerts()
{
    return XADeviceRef.Alerts();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Thermal() --  Return reference to logic temperature object
//---------------------------------------------------------------------------

IX6ThermalDevice & XAFamilyBoard::Thermal()
{
    return XADeviceRef.Thermal();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::VFifo() --  VFifo Information object
//---------------------------------------------------------------------------

XU_VFifoInfo & XAFamilyBoard::VFifo()
{
    return XADeviceRef.VFifo();
}


//---------------------------------------------------------------------------
//  XAFamilyBoard::Uart() --  Return dual UART
//---------------------------------------------------------------------------

IMulticharUart&XAFamilyBoard::Uart(unsigned int ch)
{
    return XADeviceRef.Uart(ch);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::FrontPanelDio() -- FP DIO port access
//---------------------------------------------------------------------------

XUDioPortIntf & XAFamilyBoard::FrontPanelDio()
{
    return XADeviceRef.FrontPanelDio();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Dio() -- DIO port access
//---------------------------------------------------------------------------

AdvancedDioPortIntf & XAFamilyBoard::Dio()
{
    return XADeviceRef.Dio();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Info() --
//---------------------------------------------------------------------------

const IX6LogicVersionInfo & XAFamilyBoard::Info() const
{
    return XADeviceRef.Info();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Aurora() -- return ref to Aurora subdevs
//---------------------------------------------------------------------------

IX6Aurora &  XAFamilyBoard::Aurora(unsigned int idx)
{
    return XADeviceRef.Aurora(idx);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::IdRom() --
//---------------------------------------------------------------------------

IIdRom & XAFamilyBoard::IdRom()
{
    return XADeviceRef.IdRom();
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::BusMasterSize() --
//---------------------------------------------------------------------------

void XAFamilyBoard::BusMasterSize(int bytes)
{
    XADeviceRef.BusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

void XAFamilyBoard::IncomingBusMasterSize(int bytes)
{
    XADeviceRef.IncomingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

void XAFamilyBoard::OutgoingBusMasterSize(int bytes)
{
    XADeviceRef.OutgoingBusMasterSizeWords(bytes/sizeof(int));
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::IncomingBusMasterSize() --
//---------------------------------------------------------------------------

int  XAFamilyBoard::IncomingBusMasterSize() const
{
    return XADeviceRef.IncomingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::OutgoingBusMasterSize() --
//---------------------------------------------------------------------------

int  XAFamilyBoard::OutgoingBusMasterSize() const
{
    return XADeviceRef.OutgoingBusMasterSizeWords() * sizeof(int);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::PeekLogic() -- Read from Logic memory region of board
//---------------------------------------------------------------------------

ii32  XAFamilyBoard::PeekLogic(int offset )
{
    return XADeviceRef.PeekLogic(offset);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::PokeLogic() -- Write to Logic memory region of board
//---------------------------------------------------------------------------

void  XAFamilyBoard::PokeLogic( int offset, ii32 value )
{
    XADeviceRef.PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::PeekPort() -- Read from Port memory region of board
//---------------------------------------------------------------------------

ii32  XAFamilyBoard::PeekPort(int offset )
{
    return XADeviceRef.PeekPort(offset);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::PokePort() -- Write to Port memory region of board
//---------------------------------------------------------------------------

void  XAFamilyBoard::PokePort( int offset, ii32 value )
{
    XADeviceRef.PokePort(offset, value);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::AfterFiclStart() --
//---------------------------------------------------------------------------

void  XAFamilyBoard::AfterFiclStart(FiclSystem & engine)
{
    //  Replace Callbacks to call FICL too
    XADeviceRef.Load_StreamDeviceEventCallbacks(
                    new FiclStreamDeviceEventCallbacks(*this, engine) );
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Do_LFetch() --  l@ implementation
//---------------------------------------------------------------------------

int  XAFamilyBoard::Do_LFetch(int addr)
{
    return PeekLogic(addr);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Do_LStore() --  l! implementation
//---------------------------------------------------------------------------

void  XAFamilyBoard::Do_LStore(int addr, int value)
{
    PokeLogic(addr, value );
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Do_PFetch() --  p@ implementation
//---------------------------------------------------------------------------

int  XAFamilyBoard::Do_PFetch(int addr)
{
    return PeekPort(addr);
}

//---------------------------------------------------------------------------
//  XAFamilyBoard::Do_PStore() --  p! implementation
//---------------------------------------------------------------------------

void  XAFamilyBoard::Do_PStore(int addr, int value)
{
    PokePort(addr, value);
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
