// XAFamilyBoard_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef XAFamilyBoard_MbH
#define XAFamilyBoard_MbH

#include <DataTypes_Mb.h>
#include <XAFamilyDeviceMap_Mb.h>
#include <PacketStreamSupport_Mb.h>
#include <Ficl/FiclSystem_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS XAFamilyBoard  -- XAFamily Common baseboard class
//===========================================================================

class XAFamilyBoard : public IIsAVitaPacketStreamBoard,
                public IStreamDeviceEvents,
                public IFiclTarget
{

public:
    // Trigger Support
    // Enumerations

    //
    //  Member Functions
    XAFamilyBoard(XAFamilyDeviceMap & deviceMap);
    virtual ~XAFamilyBoard();

    //  Informational Methods
    size_t                      BoardCount();
    vector<string>              BoardNames();
    string                      PrintAllDevices();
    string                      DriverDate();
    string                      DriverVersion();
	bool                        LicenseCheck(unsigned int & code);
	
    std::string                 Name()
        {  return BoardName();  }
    std::string                 Revision()
        {  return XADeviceRef.Revision();  }

    int                         Target();
    void                        Target(int value);


    void                        Reset()  {}

    bool                        Led();
    void                        Led(bool value);
    bool                        GreenLed();
    void                        GreenLed(bool value);
    bool                        FrontPanelLed();
    void                        FrontPanelLed(bool value);

    virtual void                Open();
    virtual void                Close();
    //
    //  Properties
    void                        BusMasterSize(int bytes);
    void                        IncomingBusMasterSize(int bytes);
    void                        OutgoingBusMasterSize(int bytes);
    int                         IncomingBusMasterSize() const;
    int                         OutgoingBusMasterSize() const;

    ii32                        PeekLogic( int offset );
    void                        PokeLogic( int offset, ii32 value );
    ii32                        PeekPort( int offset );
    void                        PokePort( int offset, ii32 value );

    //  SubDevices
    const IX6LogicVersionInfo &   Info() const;
    XUDioPortIntf &               FrontPanelDio();
    AdvancedDioPortIntf &         Dio();

    IFpaLoader &                 VsProm();

    IVeloDepacketizer &          VeloOut();
    IVeloPacketizer &            VeloIn();

    IUsesX6Alerts &              Alerts();
    IX6ThermalDevice &           Thermal();
    
    IX6Aurora &                  Aurora(unsigned int idx=0);
    X6SioPll  &                  AuroraPll()  {  return XADeviceRef.AuroraPll();   }

    IIdRom &                     IdRom();
    XU_VFifoInfo &               VFifo();
    IMulticharUart &             Uart(unsigned int ch);

public: //Pseudo protected
    //
    //  IPacketStreamDevice interface implementation
    virtual IPacketStreamDeviceInternals *  Internals();

    XAFamilyDeviceMap * Debug() { return &XADeviceRef; }
    virtual void   Log(int type, int data);

    //  FPGA type name function
    std::string FpgaName() {  return FpgaName(Info().FpgaChipType());  }
    std::string FpgaName(const short Number)
        {
            switch(static_cast<int>(Number))
                {
                case 0:   return "No Logic";
                case 1:   return "200T";
                default:  return "Unknown";
                }
        }

    std::string FamilyName()  {   return FamilyName(Info().PciLogicFamily());  }
    std::string FamilyName(const short Number)
    {
        switch (static_cast<int>(Number))
        {
            case 0: return "X3";
            case 1: return "X5";
            case 2: return "X6";
            case 3: return "XA / K7";
            default: return "Unknown";
        }
    }

    std::string BoardName()  {   return BoardName(Info().PciLogicType());  }
    std::string BoardName(const short Number)
    {
        switch (static_cast<int>(Number))
        {
            case 0: return "No Board";
            case 1: return "XA-160M";
            case 2: return "XA-RX";
            case 3: return "XA-TX";
            case 9: return "XA-402";
            default: return "Unknown";
        }
    }

protected:
    //
    //  Member Data
    XAFamilyDeviceMap &                  XADeviceRef;
    //
    //  Friend Interface Functions
    friend AddressingSpace & LogicMemorySpace( XAFamilyBoard & board );
    friend AddressingSpace & PortMemorySpace( XAFamilyBoard & board );

    //  Ficl Interface
    virtual void    AfterFiclStart(FiclSystem & engine);

    virtual int     Do_LFetch(int addr);
    virtual void    Do_LStore(int addr, int value);
    virtual int     Do_PFetch(int addr);
    virtual void    Do_PStore(int addr, int value);
};

//---------------------------------------------------------------------------
//  LogicMemorySpace() - Return reference to LogicMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & LogicMemorySpace( XAFamilyBoard & board )
{
    return board.XADeviceRef.LogicMemory;
}

//---------------------------------------------------------------------------
//  PortMemorySpace() - Return reference to PortMemory addressing space
//---------------------------------------------------------------------------

inline AddressingSpace & PortMemorySpace( XAFamilyBoard & board )
{
    return board.XADeviceRef.PortMemory;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // namespace Innovative

#endif

