// XAFamilyDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XAFamilyDeviceMap_Mb.h>
#include <XAFamilyMemoryMap_Mb.h>
#include <XAFamilyDeviceDriver_Mb.h>
#include <sstream>

//---------------------------------------------------------------------------
namespace Innovative
{
#ifdef __GNUC__
    #pragma GCC diagnostic ignored "-Wmissing-field-initializers"
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XAFamilyHdwPluginImpl -- Installable Callbacks for PacketDevice
//===========================================================================

class XAFamilyHdwPluginImpl : public IPacketDevicePlugin
{
public:
    XAFamilyHdwPluginImpl( XAFamilyDeviceMap * pdm )
        :  PDM(pdm)
        {}
    virtual void  AfterDeviceOpen()
        {  PDM->OpenHardware();  }
    virtual void  BeforeDeviceClose()
        { /* nothing special */  }

    XAFamilyDeviceMap * PDM;
};

//===========================================================================
//  CLASS XAFamilyDeviceMap  -- Registers for XAFamily Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAFamilyDeviceMap
//---------------------------------------------------------------------------
//
//  IDROM Page Numbers

const unsigned int System_IdromPage = 0;
const unsigned int AdcCal_IdromPage = 1;
const unsigned int DacCal_IdromPage = 2;
const unsigned int User_IdromPage   = 3;

//
//  Subdevice Register Structures

using namespace XA_MemoryMap;

PacketDeviceRegisters XAFamilyDRegAddrs =
    {
    mmControl,            // ControlRegister_Addr;
    mmStatus,             // StatusRegister_Addr;
    mmRxPacketTimer,      // RxPacketTimerRegister_Addr;
    mmTxPacketTimer,      // TxPacketTimerRegister_Addr;
	mmLinkControl,        // LinkControlRegister_Addr;
    };

PacketDeviceMapRegisters XAFamilyRegAddrs =
    {
    mmInterruptEnable,    // InterruptEnable_Addr;
    mmCreditQueue,        // CreditQueue_Addr;
    mmBmRingBaseAddress,  // BmRingBaseAddressArray_Addr;
    mmBmRingSize,         // BmRingSizeArray_Addr;
    };

X6Info::MapRegisters XAFamilyInfoRegAddrs =
    {
    mmInformation,        //  Info_Addr
    mmBitstreamDate,      //  BitstreamDate_Addr
    mmSubrevision,        //  Subrev_Addr
    mmPciVersion,         //  Bar0Version_Addr
    };

X6Peripherals::MapRegisters XAFamilyPeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

X6SioPll::MapRegisters XAFamilySioPllRegAddrs =
    {
    mmSioXo,              //  SioPll_Addr
    };

XU_VFifoInfo::MapRegisters XA_VFifoRegAddrs =
    {
    mmVFifo_Status,           //  VFifoStatus_Addr;
    mmVFifo_Depth,            //  VFifoWordCount_Addr
    };

X6DepacketizerCfg::MapRegisters XAFamilyDepacketizerRegAddrs =
    {
    0,                      //  TestLoopbackPid_Addr;
    mmVeloOutDataPid,       //  OutgoingDataPid_Addr
    };

XUDigitalIo::MapRegisters XAFamilyFrontPanelDioRegAddrs =
    {
    mmFrontPanelDioData,         //  DioDataLo_Addr
    mmFrontPanelDioOutEn,        //  DioOutputEnableLo_Addr
    };

XAFpaLoader::MapRegisters XAFamilyFpaRegAddrs =
    {
    mmFpa_AddressCtrl,    //  AddressControl_Addr
    mmFpa_Data,           //  Data_Addr
    mmFpa_Opcode,         //  Opcode_Addr
    };

X6Thermal::MapRegisters XAFamilyThermalRegAddrs =
    {
    mmTemp_TempControl,        //  TempControl_Addr
    mmTemp_FanThreshold,       //  FanThresholdTemp_Addr
    mmTemp_AlertThreshold,     //  AlertThresholdTemp_Addr
    mmTemp_CommandChannel,     //  CommandChannel_Addr
    };

X6PacketizerCfg::MapRegisters XAFamilyPacketizerRegAddrs =
    {
    mmPktzer_Enable,          //  PacketizerEnable_Addr
    mmPktzer_AuxHdr,          //  AuxHeader_Addr
    mmPktzer_VeloInHdr,       //  VelociaInHeader_Addr
    mmPktzer_ForceSize,       //  PacketizerForceSize_Addr
    mmPktzer_IntervalPeriod,  //  PacketizerIntervalPeriod_Addr
    };

X6AlertSystem::MapRegisters XAFamilyAlertsRegAddrs =
    {
    mmAlert_Enable,       //  AlertEnable_Addr
    mmAlert_Control,      //  AlertControl_Addr
    mmAlert_Software,     //  SoftwareAlert_Addr;
    mmAlert_Clear,        //  ClearAlert_Addr;
    };

X6DigitalIo::MapRegisters XAFamilyDioRegAddrs =
    {
    mmDio_DataLo,          //  DioDataLo_Addr
    mmDio_DataHi,          //  DioDataHi_Addr
    mmDio_OutEnableLo,     //  DioOutputEnableLo_Addr
    mmDio_OutEnableHi,     //  DioOutputEnableHi_Addr
    };

X6FlashRom::MapRegisters XAFamilyFlashRomRegAddrs =
    {
    mmRom_AddrCtl,         //  AddressControl_Addr
    mmRom_Data,            //  Data_Addr
    mmRom_Opcode,          //  Opcode_Addr
    };

FmcAFE_Common::MapRegisters XA_AFECommonRegAddrs =
    {
    mmAFE_TestControl,       //  AfeTestControl_Addr
    mmAFE_SoftwareTrigger,   //  AfeSoftwareTrigger_Addr
    mmAFE_ExtSyncSelect,     //  AfeExtSyncSelect_Addr
    };

X6Aurora::MapRegisters XAAurora0RegAddrs =
    {
    mmAurora_TestControl,    //  TestControl_Addr
    mmAurora_ControlStatus,  //  ControlStatus_Addr
    mmAurora_WritePort,      //  WritePort_Addr
    mmAurora_ReadPort,       //  ReadPort_Addr
    mmAurora_SfpPortControl, //  SfpPortControl_Addr
	mmAurora_SfpPortI2C,     //  SfpPortI2C_Addr
	};

/*
XAFamilyAFE_DacLatencyCalibration::MapRegisters XAFamilyAfeLatencyCalRegAddrs =
    {
    mmAFE_DacLatencyCal,      //  LatencyCal_Addr;
    };

*/

K7DualUart::MapRegisters XADualUartAddrs =
    {
    mmUart_0_Control,       //   Uart0Ctrl_Addr;
    mmUart_0_Tx,            //   Uart0Tx_Addr;
    mmUart_0_Rx,            //   Uart0Rx_Addr;
    mmUart_1_Control,       //   Uart1Ctrl_Addr;
    mmUart_1_Tx,            //   Uart1Tx_Addr;
    mmUart_1_Rx,            //   Uart1Rx_Addr;
    mmUartMode,             //   UartMode_Addr;
    mmInterrupt,            //   UartInterrupt_Addr;
    mmGpioData,             //   GpioData_Addr;
    mmGpioDirection,        //   GpioDir_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class XAFamilyDeviceMap
//---------------------------------------------------------------------------
const bool  BitTS = true;    // supports bitstream timestamps.


XAFamilyDeviceMap::XAFamilyDeviceMap(IStreamDeviceEvents * owner)
    :  PacketDeviceImpl         (new XAFamilyDeviceDriver, XAFamilyDRegAddrs),
       PacketDeviceMap          (this, XAFamilyRegAddrs, 0),
       IUsesStreamDeviceEventCallbacks(owner),
       DDeviceDriver            (dynamic_cast<XAFamilyDeviceDriver*>(DeviceDriver)),
       WB_Periph                (LogicMemory, wbPeripheral),
       WB_Loader                (LogicMemory, wbLogicLoader),
       WB_Thermal               (LogicMemory, wbTempControl),
       WB_Packetizer            (LogicMemory, wbPacketizer),
       WB_Alerts                (LogicMemory, wbAlerts),
       WB_DigitalIo             (LogicMemory, wbDigitalIo),
       WB_FlashRom              (LogicMemory, wbROM),
       WB_AFE                   (LogicMemory, wbAFE),
       WB_Uart                  (LogicMemory, wbUart),
       WB_Aurora0               (LogicMemory, wbAurora0),
       Info_SDev                (this, WB_Periph, PortMemory, XAFamilyInfoRegAddrs, BitTS),
       Periph_SDev              (this, WB_Periph, XAFamilyPeripheralRegAddrs),
       SioPll_SDev              (this, WB_Periph, XAFamilySioPllRegAddrs),
       VFifo_SDev               (this, WB_Periph, XA_VFifoRegAddrs),
       VeloIn_SDev              (this, WB_Packetizer, XAFamilyPacketizerRegAddrs),
       VeloOut_SDev             (this, WB_Periph, XAFamilyDepacketizerRegAddrs),
       FrontPanelDio_SDev       (this, WB_Periph, XAFamilyFrontPanelDioRegAddrs),
       Dio_SDev                 (this, WB_DigitalIo, XAFamilyDioRegAddrs),
       Thermal_SDev             (this, WB_Thermal, XAFamilyThermalRegAddrs),
       Alerts_SDev              (this, WB_Alerts, XAFamilyAlertsRegAddrs),
       Uart_SDev                (this, WB_Uart, XADualUartAddrs),
       Aurora0_SDev             (this, WB_Aurora0, XAAurora0RegAddrs),
       AFECommon_SDev           (this, WB_AFE, XA_AFECommonRegAddrs),
       Fpa_SDev                 (this, WB_Loader, XAFamilyFpaRegAddrs),
       FlashRom_SDev            (this, WB_FlashRom, XAFamilyFlashRomRegAddrs),
//       AFEDacLatencyCal_SDev    (this, WB_AFE, XAFamilyAfeLatencyCalRegAddrs),
       System_PG                (FlashRom_SDev, System_IdromPage),
       AdcCal_PG                (FlashRom_SDev, AdcCal_IdromPage),
       DacCal_PG                (FlashRom_SDev, DacCal_IdromPage),
       UserRom_PG               (FlashRom_SDev, User_IdromPage),
       IdRom_Intf               (this)
{
    //
    //  Install Open Loader for Hardware
    InstallHdwPlugin( new XAFamilyHdwPluginImpl(this) );

    //  Enable use of dedicated register to signal Kernel Cache ready
    DDeviceDriver->EnableDedicatedKernelCacheRelease(PortMemory, XA_MemoryMap::mmKernelCacheAvail);

}

//---------------------------------------------------------------------------
//  destructor for class XAFamilyDeviceMap
//---------------------------------------------------------------------------

XAFamilyDeviceMap::~XAFamilyDeviceMap()
{
}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAFamilyDeviceMap::ConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    SioPll_SDev.ConstructCustomParts();
    VFifo_SDev.ConstructCustomParts();
    FrontPanelDio_SDev.ConstructCustomParts();
    VeloIn_SDev.ConstructCustomParts();
    VeloOut_SDev.ConstructCustomParts();
    Alerts_SDev.ConstructCustomParts();
    Fpa_SDev.ConstructCustomParts();
    Thermal_SDev.ConstructCustomParts();
    Dio_SDev.ConstructCustomParts();
    Uart_SDev.ConstructCustomParts();
    FlashRom_SDev.ConstructCustomParts();
    AFECommon_SDev.ConstructCustomParts();
    Aurora0_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XAFamilyDeviceMap::OpenHardware()
{
    //
    //  Reset the device
    MasterReset();

    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    SioPll_SDev.OpenHardware();
    VFifo_SDev.OpenHardware();
    VeloOut_SDev.OpenHardware();
    FrontPanelDio_SDev.OpenHardware();
    VeloIn_SDev.OpenHardware();
    Alerts_SDev.OpenHardware();
    Fpa_SDev.OpenHardware();
    Thermal_SDev.OpenHardware();
    Dio_SDev.OpenHardware();
    Uart_SDev.OpenHardware();
    AFECommon_SDev.OpenHardware();
    FlashRom_SDev.OpenHardware();
    Aurora0_SDev.OpenHardware();
    //
    //  Read all ROM pages to Init Cache
    System_PG.LoadFromRom();
    AdcCal_PG.LoadFromRom();
    DacCal_PG.LoadFromRom();
    UserRom_PG.LoadFromRom();

}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XAFamilyDeviceMap::PreconfigureHardware()
{
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    SioPll_SDev.PreconfigureHardware();
    VFifo_SDev.PreconfigureHardware();
    FrontPanelDio_SDev.PreconfigureHardware();
    VeloIn_SDev.PreconfigureHardware();
    VeloOut_SDev.PreconfigureHardware();
    Alerts_SDev.PreconfigureHardware();
    Fpa_SDev.PreconfigureHardware();
    Thermal_SDev.PreconfigureHardware();
    Dio_SDev.PreconfigureHardware();
    Uart_SDev.PreconfigureHardware();
    AFECommon_SDev.PreconfigureHardware();
    FlashRom_SDev.PreconfigureHardware();
    Aurora0_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAFamilyDeviceMap::ConfigureHardware()
{
    Periph_SDev.AppReset();

    // Subdevice Configuration

    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    SioPll_SDev.ConfigureHardware();
    VFifo_SDev.ConfigureHardware();
    FrontPanelDio_SDev.ConfigureHardware();
    VeloIn_SDev.ConfigureHardware();
    VeloOut_SDev.ConfigureHardware();
    Alerts_SDev.ConfigureHardware();
    Fpa_SDev.ConfigureHardware();
    Thermal_SDev.ConfigureHardware();
    Dio_SDev.ConfigureHardware();
    Uart_SDev.ConfigureHardware();
    AFECommon_SDev.PreconfigureHardware();
    FlashRom_SDev.ConfigureHardware();
    Aurora0_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XAFamilyDeviceMap::StreamStopHardware()
{
    // Subdevice Finalization
}


//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::AlertTest() -- Alert Test Processing Hook
//---------------------------------------------------------------------------

bool  XAFamilyDeviceMap::AlertTest(unsigned int pid)
{
    //  return TRUE if PID is an alert PID.
    //  return FALSE if not
    return Alerts_SDev.AlertTest(pid);
}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::AlertCheck() -- Alert Event Processing Hook
//---------------------------------------------------------------------------

bool XAFamilyDeviceMap::AlertCheck(const Buffer & buffer)
{
    //  return TRUE if buffer is to be passed on to application.
    //  return FALSE to discard buffer now.
    return Alerts_SDev.CheckAndProcess(buffer);
}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::DumpIsrStatus() -- Transfer contents of log out
//---------------------------------------------------------------------------

void  XAFamilyDeviceMap::DumpIsrStatus(IntVec & vec)
{
    vec.clear();
    for (int i=0; i<XAFamilyDeviceDriver::LOG->Count; i++)
        vec.push_back(XAFamilyDeviceDriver::LOG->Status[i]);
}


//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::Revision() --  Return revision of module PCB as character
//---------------------------------------------------------------------------

std::string  XAFamilyDeviceMap::Revision() const
{
	std::stringstream msg;
    char c = 'A' + static_cast<char>(Info().PciLogicPcb());
    msg << c;
    return msg.str();
}

//---------------------------------------------------------------------------
//  XAFamilyDeviceMap::Uart()
//---------------------------------------------------------------------------

IMulticharUart & XAFamilyDeviceMap::Uart(unsigned int ch)
{
    return Uart_SDev.Uart(ch);
}

//===========================================================================
//  CLASS XAAdcDeviceMap  -- input only card device map
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAAdcDeviceMap
//---------------------------------------------------------------------------

using namespace XA_MemoryMap;

XAAFE_AdcCommon::MapRegisters XA_AfeAdcCommonRegAddrs =
    {
    mmAFE_AdcEnable,        //  AfeAdcEnable_Addr
    mmAFE_AdcPowerDown,     //  AfeAdcPower_Addr
    mmAFE_AdcTrigger,       //  AfeAdcTriggerCfg_Addr
    mmAFE_AdcDecimation,    //  AfeAdcDecimation_Addr
    mmAFE_AdcReset,         //  AfeAdcReset_Addr
    mmAFE_AdcSync,          //  AfeAdcSync_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters XA_AfeAdcPriRegAddrs =
    {
    mmAFE_AdcPriEnable,      //  PriTriggerEnable_Addr;
    mmAFE_AdcPriInterval,    //  PriInterval_Addr;
    mmAFE_AdcPriTrigParam,   //  PriTriggerParam_Addr;
    mmAFE_AdcCaptureWin      //  PriCaptureWindow_Addr;
    };

XASpiDevice::MapRegisters XAAfeAdcSpiRegAddrs =
    {
    mmAFE_AdcSpiEnable,      //  Device_Addr;
    mmAFE_AdcSpiCtrl,        //  Write_Addr;
    mmAFE_AdcSpiStatus,      //  Read_Addr;
    };

XA_AdcCalCoefficientLoad::MapRegisters XAAfeAdcCalCoeffRegAddrs =
    {
    mmAFE_AdcGainCoeff,      //  GainCoeff_Addr;
    mmAFE_AdcOffsetCoeff,    //  OffsetCoeff_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class XAAdcDeviceMap
//---------------------------------------------------------------------------

XAAdcDeviceMap::XAAdcDeviceMap(IStreamDeviceEvents * owner)
    :  XAFamilyDeviceMap(owner),
       AFEAdcC_SDev  (this, WB_AFE, XA_AfeAdcCommonRegAddrs),
       AFEAdcPri_SDev(this, true, WB_AFE, XA_AfeAdcPriRegAddrs),  // true=AdcPri
       AFEAdcSpi_SDev(this, WB_AFE, XAAfeAdcSpiRegAddrs),
       AFEAdcCal_SDev(this, WB_AFE, XAAfeAdcCalCoeffRegAddrs, mmAFE_TotalAdcs)//,
{
}

//---------------------------------------------------------------------------
//  XAAdcDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAAdcDeviceMap::ConstructCustomParts()
{
    XAFamilyDeviceMap::ConstructCustomParts();
    // Subdevice Parts Construction
    AFEAdcC_SDev.ConstructCustomParts();
    AFEAdcPri_SDev.ConstructCustomParts();
    AFEAdcSpi_SDev.ConstructCustomParts();
    AFEAdcCal_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  XAAdcDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XAAdcDeviceMap::OpenHardware()
{
    XAFamilyDeviceMap::OpenHardware();

    AFEAdcC_SDev.OpenHardware();
    AFEAdcPri_SDev.OpenHardware();
    AFEAdcSpi_SDev.OpenHardware();
    AFEAdcCal_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  XAAdcDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XAAdcDeviceMap::PreconfigureHardware()
{
    XAFamilyDeviceMap::PreconfigureHardware();

    // Subdevice Configuration
    AFEAdcC_SDev.PreconfigureHardware();
    AFEAdcPri_SDev.PreconfigureHardware();
    AFEAdcSpi_SDev.PreconfigureHardware();
    AFEAdcCal_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XAAdcDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAAdcDeviceMap::ConfigureHardware()
{
    XAFamilyDeviceMap::ConfigureHardware();

    AFEAdcC_SDev.ConfigureHardware();
    AFEAdcPri_SDev.ConfigureHardware();
    AFEAdcSpi_SDev.ConfigureHardware();
    AFEAdcCal_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  XAAdcDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XAAdcDeviceMap::StreamStopHardware()
{
    XAFamilyDeviceMap::StreamStopHardware();
    AFEAdcPri_SDev.StreamStopHardware();
    AFEAdcCal_SDev.StreamStopHardware();

}

//===========================================================================
//  CLASS XADac_DeviceMixin  -- DAC Mixin Devices for Output and Bidir cards
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XADac_DeviceMixin
//---------------------------------------------------------------------------

XAAFE_DacCommon::MapRegisters XAAfeDacCommonRegAddrs =
    {
    mmAFE_DacEnable,        //  AfeDacEnable_Addr
    mmAFE_DacPowerDown,     //  AfeDacPower_Addr
    mmAFE_DacTrigger,       //  AfeDacTriggerCfg_Addr
    mmAFE_DacDecimate,      //  AfeDacDecimation_Addr
    mmAFE_DacTestPhaseIncr, //  AfeDacPhaseIncr_Addr
    mmAFE_DacReset,         //  AfeDacReset_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters XAAfeDacPriRegAddrs =
    {
    mmAFE_DacPriEnable,      //  PriTriggerEnable_Addr;
    mmAFE_DacPriInterval,    //  PriInterval_Addr;
    mmAFE_DacPriTrigParam,   //  PriTriggerParam_Addr;
    mmAFE_DacCaptureWin      //  PriCaptureWindow_Addr;
    };

XUSpiDevice::MapRegisters XAAfeDacSpiRegAddrs =
    {
    mmAFE_DacSpiEnable,      //  Device_Addr;
    mmAFE_DacSpiCtrl,        //  Write_Addr;
    mmAFE_DacSpiStatus,      //  Read_Addr;
    };

XU_DacCalCoefficientLoad::MapRegisters XAAfeDacCalCoeffRegAddrs =
    {
    mmAFE_DacGainCoeff,      //  GainCoeff_Addr
    mmAFE_DacOffsetCoeff,    //  OffsetCoeff_Addr
    };

XU_DacPatternModeSystem::MapRegisters XADacPatternRegAddrs =
    {
    mmDdr3Status,       //   PatternModeEnable_Addr
    mmArbPB_Control,    //   PlaybackCommand_Addr
    mmArbPB_Data0,      //   CmdFifo0_Addr
    mmArbPB_Data1,      //   CmdFifo1_Addr
    mmArbPB_Data2,      //   CmdFifo2_Addr
    mmArbPB_Data3,      //   CmdFifo3_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class XADac_DeviceMixin
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  Housekeeping for class XU_DacPatternModeSystem
//---------------------------------------------------------------------------

const int DefaultXAPatternModeOnValue =  0x2;
const int DefaultXAPatternModeOffValue = 0x0;

XADac_DeviceMixin::XADac_DeviceMixin(XAFamilyDeviceMap * owner)
    :   DDMxOwner(owner),
        AFEDacC_SDev  (this, DDMxOwner->WB_AFE, XAAfeDacCommonRegAddrs),
        AFEDacPri_SDev(this, false, DDMxOwner->WB_AFE, XAAfeDacPriRegAddrs),  // false=DacPri
        AFEDacSpi_SDev(this, DDMxOwner->WB_AFE, XAAfeDacSpiRegAddrs),
        AFEDacCal_SDev(this, DDMxOwner->WB_AFE, XAAfeDacCalCoeffRegAddrs, mmAFE_TotalDacs),
        DacPattern_SDev(this, DDMxOwner->WB_Periph, XADacPatternRegAddrs)
{
    DacPattern_SDev.DefineModeEnableValues(DefaultXAPatternModeOnValue, DefaultXAPatternModeOffValue);
}

//---------------------------------------------------------------------------
//  XADac_DeviceMixin::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XADac_DeviceMixin::ConstructCustomParts()
{
    // Subdevice Parts Construction
    AFEDacC_SDev.ConstructCustomParts();
    AFEDacPri_SDev.ConstructCustomParts();
    AFEDacSpi_SDev.ConstructCustomParts();
    AFEDacCal_SDev.ConstructCustomParts();
    DacPattern_SDev.ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  XADac_DeviceMixin::OpenHardware()
//---------------------------------------------------------------------------

void  XADac_DeviceMixin::OpenHardware()
{
    AFEDacC_SDev.OpenHardware();
    AFEDacPri_SDev.OpenHardware();
    AFEDacSpi_SDev.OpenHardware();
    AFEDacCal_SDev.OpenHardware();
    DacPattern_SDev.OpenHardware();

}

//---------------------------------------------------------------------------
//  XADac_DeviceMixin::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XADac_DeviceMixin::PreconfigureHardware()
{
    // Subdevice Configuration
    AFEDacC_SDev.PreconfigureHardware();
    AFEDacPri_SDev.PreconfigureHardware();
    AFEDacSpi_SDev.PreconfigureHardware();
    AFEDacCal_SDev.PreconfigureHardware();
    DacPattern_SDev.PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  XADac_DeviceMixin::ConfigureHardware()
//---------------------------------------------------------------------------

void  XADac_DeviceMixin::ConfigureHardware()
{
    AFEDacC_SDev.ConfigureHardware();
    AFEDacPri_SDev.ConfigureHardware();
    AFEDacSpi_SDev.ConfigureHardware();
    AFEDacCal_SDev.ConfigureHardware();
    DacPattern_SDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  XADac_DeviceMixin::StreamStopHardware()
//---------------------------------------------------------------------------

void  XADac_DeviceMixin::StreamStopHardware()
{
    AFEDacC_SDev.StreamStopHardware();
    AFEDacPri_SDev.StreamStopHardware();
    AFEDacSpi_SDev.StreamStopHardware();
    AFEDacCal_SDev.StreamStopHardware();
    DacPattern_SDev.StreamStopHardware();

}


//===========================================================================
//  CLASS XADacDeviceMap  -- Dac Only Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XADacDeviceMap
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XADacDeviceMap
//---------------------------------------------------------------------------

XADacDeviceMap::XADacDeviceMap(IStreamDeviceEvents * owner)
    :   XAFamilyDeviceMap(owner),
        XADac_DeviceMixin(this)
{
}

//---------------------------------------------------------------------------
//  XADacDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XADacDeviceMap::ConstructCustomParts()
{
    XAFamilyDeviceMap::ConstructCustomParts();
    XADac_DeviceMixin::ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  XADacDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XADacDeviceMap::OpenHardware()
{
    XAFamilyDeviceMap::OpenHardware();
    XADac_DeviceMixin::OpenHardware();
}

//---------------------------------------------------------------------------
//  XADacDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XADacDeviceMap::PreconfigureHardware()
{
    XAFamilyDeviceMap::PreconfigureHardware();
    XADac_DeviceMixin::PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XADacDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XADacDeviceMap::ConfigureHardware()
{
    XAFamilyDeviceMap::ConfigureHardware();
    XADac_DeviceMixin::ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XADacDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XADacDeviceMap::StreamStopHardware()
{
    XAFamilyDeviceMap::StreamStopHardware();
    XADac_DeviceMixin::StreamStopHardware();
}

//===========================================================================
//  CLASS XABidirDeviceMap  -- Fmc Daughter card Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XABidirDeviceMap
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XABidirDeviceMap
//---------------------------------------------------------------------------

XABidirDeviceMap::XABidirDeviceMap(IStreamDeviceEvents * owner)
    :   XAAdcDeviceMap(owner),
        XADac_DeviceMixin(this)
{
}

//---------------------------------------------------------------------------
//  XABidirDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XABidirDeviceMap::ConstructCustomParts()
{
    XAAdcDeviceMap::ConstructCustomParts();
    XADac_DeviceMixin::ConstructCustomParts();

}

//---------------------------------------------------------------------------
//  XABidirDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XABidirDeviceMap::OpenHardware()
{
    XAAdcDeviceMap::OpenHardware();
    XADac_DeviceMixin::OpenHardware();
}

//---------------------------------------------------------------------------
//  XABidirDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XABidirDeviceMap::PreconfigureHardware()
{
    XAAdcDeviceMap::PreconfigureHardware();
    XADac_DeviceMixin::PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XABidirDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XABidirDeviceMap::ConfigureHardware()
{
    XAAdcDeviceMap::ConfigureHardware();
    XADac_DeviceMixin::ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XABidirDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XABidirDeviceMap::StreamStopHardware()
{
    XAAdcDeviceMap::StreamStopHardware();
    XADac_DeviceMixin::StreamStopHardware();
}

//===========================================================================
//  CLASS XAInputDevice  -- Partially Implemented XA Input Device
//===========================================================================
//---------------------------------------------------------------------------
//  XAInputDevice::Devices() -- return device count
//---------------------------------------------------------------------------

unsigned int  XAInputDevice::Devices()
{
     return AdcCommon().Devices();
}

//---------------------------------------------------------------------------
//  XAInputDevice::ChannelsPerDevice() -- return channels per device
//---------------------------------------------------------------------------

unsigned int  XAInputDevice::ChannelsPerDevice()
{
     return AdcCommon().ChannelsPerDevice();
}

//---------------------------------------------------------------------------
//  XAInputDevice::Channels() -- return channels
//---------------------------------------------------------------------------

unsigned int  XAInputDevice::Channels()
{
     return AdcCommon().Channels();
}

//---------------------------------------------------------------------------
//  XAInputDevice::DeviceEnabled() -- Enable specified I/O devices
//---------------------------------------------------------------------------

void  XAInputDevice::DeviceEnabled(int dev, bool state)
{
     AdcCommon().DeviceIsEnabled(dev, state);
     AdcCommon().DevicePowered(dev, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XAInputDevice::DeviceEnabled() -- Query state of specified I/O devices
//---------------------------------------------------------------------------

bool XAInputDevice::DeviceEnabled(int dev) const
{
    return AdcCommon().DeviceIsEnabled(dev);
}

//---------------------------------------------------------------------------
//  XAInputDevice::ActiveDevices() -- Return number of active devices
//---------------------------------------------------------------------------

unsigned int XAInputDevice::ActiveDevices()
{
    return AdcCommon().ActiveDevices();
}

//---------------------------------------------------------------------------
//  XAInputDevice::DevicePowered() -- Power specified I/O devices
//---------------------------------------------------------------------------

void  XAInputDevice::DevicePowered(int ch, bool state)
{
     AdcCommon().DevicePowered(ch, state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::DevicePowered() -- Query power state of an I/O devices
//---------------------------------------------------------------------------

bool XAInputDevice::DevicePowered(int ch) const
{
    return AdcCommon().DevicePowered(ch);
}

//---------------------------------------------------------------------------
//  XAInputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void  XAInputDevice::ChannelDisableAll()
{
     AdcCommon().ChannelDisableAll();
     AdcCommon().ChannelPowerDownAll();
}

//---------------------------------------------------------------------------
//  XAInputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void  XAInputDevice::ChannelEnabled(int ch, bool state)
{
     AdcCommon().ChannelEnabled(ch, state);
     ChannelPowered(ch, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XAInputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XAInputDevice::ChannelEnabled(int ch) const
{
    return AdcCommon().ChannelEnabled(ch);
}


//---------------------------------------------------------------------------
//  XAInputDevice::ActiveChannels() -- Return number of active channels
//---------------------------------------------------------------------------

unsigned int XAInputDevice::ActiveChannels()
{
    return AdcCommon().ActiveChannels();
}

//---------------------------------------------------------------------------
//  XAInputDevice::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XAInputDevice::ChannelPowered(int ch, bool state)
{
     AdcCommon().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::ChannelPowered() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

bool XAInputDevice::ChannelPowered(int ch) const
{
    return AdcCommon().ChannelPowered(ch);
}


//---------------------------------------------------------------------------
//  XAInputDevice::SoftwareTrigger() -- Enable Test Mode on input
//---------------------------------------------------------------------------

void  XAInputDevice::SoftwareTrigger(bool state)
{
    Common().SoftwareTrigger(FmcAFE_Common::tInput, state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::SoftwareTrigger() -- Enable Test Mode on input
//---------------------------------------------------------------------------

void  XAInputDevice::SoftwareTrigger(AfeDirection which, bool state)
{
    FmcAFE_Common::AfeDirection which2 = static_cast<FmcAFE_Common::AfeDirection>(which);
    Common().SoftwareTrigger(which2, state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::TestModeEnabled() -- Enable Test Mode on input
//---------------------------------------------------------------------------

void  XAInputDevice::TestModeEnabled(bool state, unsigned int mode)
{
    Common().TestModeEnabled(FmcAFE_Common::tInput, state, mode);
}

//---------------------------------------------------------------------------
//  XAInputDevice::Decimation() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XAInputDevice::Decimation(unsigned int samples)
{
     AdcCommon().Decimation(samples);
}

//---------------------------------------------------------------------------
//  XAInputDevice::Decimation() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

unsigned int XAInputDevice::Decimation() const
{
    return AdcCommon().Decimation();
}

//---------------------------------------------------------------------------
//  XAInputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XAInputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    AdcSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XAInputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XAInputDevice::Spi(unsigned int device, unsigned int address)
{
    return AdcSpi().Spi(device, address);
}

//===========================================================================
//  CLASS XAInputDevice::TriggerCfgImpl  --
//===========================================================================
//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::FramedMode() -- Setter
//---------------------------------------------------------------------------

void  XAInputDevice::TriggerCfgImpl::FramedMode(bool state)
{
    InD->AdcCommon().FramedTriggerEnabled(state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::FramedMode() -- Getter
//---------------------------------------------------------------------------

bool XAInputDevice::TriggerCfgImpl::FramedMode() const
{
    return InD->AdcCommon().FramedTriggerEnabled();
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::Edge() -- Setter
//---------------------------------------------------------------------------

void  XAInputDevice::TriggerCfgImpl::Edge(bool state)
{
    InD->AdcCommon().EdgeTrigger(state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::Edge() -- Getter
//---------------------------------------------------------------------------

bool XAInputDevice::TriggerCfgImpl::Edge() const
{
    return InD->AdcCommon().EdgeTrigger();
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::External() -- Setter
//---------------------------------------------------------------------------

void  XAInputDevice::TriggerCfgImpl::External(bool state)
{
    InD->AdcCommon().ExternalTrigger(state);
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::External() -- Getter
//---------------------------------------------------------------------------

bool XAInputDevice::TriggerCfgImpl::External() const
{
    return InD->AdcCommon().ExternalTrigger();
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::FrameSize() -- Setter
//---------------------------------------------------------------------------

void  XAInputDevice::TriggerCfgImpl::FrameSize(unsigned int samples)
{
    InD->AdcCommon().FramedTriggerSize(samples);
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::FrameSize() -- Getter
//---------------------------------------------------------------------------

unsigned int XAInputDevice::TriggerCfgImpl::FrameSize() const
{
    return InD->AdcCommon().FramedTriggerSize();
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::ExternalSyncSource() -- Setter
//---------------------------------------------------------------------------

void  XAInputDevice::TriggerCfgImpl::ExternalSyncSource(AfeExtSyncOptions src)
{
    InD->Common().ExternalSyncSelection(
                static_cast<FmcAFE_Common::AfeExtSyncOptions>(src) );
}

//---------------------------------------------------------------------------
//  XAInputDevice::TriggerCfgImpl::ExternalSyncSource() -- Getter
//---------------------------------------------------------------------------

XAInputDevice::AfeExtSyncOptions XAInputDevice::TriggerCfgImpl::ExternalSyncSource() const
{
    FmcAFE_Common::AfeExtSyncOptions opt = InD->Common().ExternalSyncSelection();
    return static_cast<XAInputDevice::AfeExtSyncOptions>( opt );
}

//===========================================================================
//  CLASS XAOutputDevice  -- Partially Implemented XA Output Device
//===========================================================================
//---------------------------------------------------------------------------
//  XAOutputDevice::Devices() -- return device count
//---------------------------------------------------------------------------

unsigned int  XAOutputDevice::Devices()
{
     return DacCommon().Devices();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::DeviceEnabled() -- Enable specified I/O devices
//---------------------------------------------------------------------------

void  XAOutputDevice::DeviceEnabled(int dev, bool state)
{
     DacCommon().DeviceIsEnabled(dev, state);
     DacCommon().DevicePowered(dev, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XAOutputDevice::DeviceEnabled() -- Query state of specified I/O devices
//---------------------------------------------------------------------------

bool XAOutputDevice::DeviceEnabled(int dev) const
{
    return DacCommon().DeviceIsEnabled(dev);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::DevicePowered() -- Power specified I/O devices
//---------------------------------------------------------------------------

void  XAOutputDevice::DevicePowered(int ch, bool state)
{
     DacCommon().DevicePowered(ch, state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::DevicePowered() -- Query power state of an I/O devices
//---------------------------------------------------------------------------

bool XAOutputDevice::DevicePowered(int ch) const
{
    return DacCommon().DevicePowered(ch);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ActiveDevices() -- Return number of active devices
//---------------------------------------------------------------------------

unsigned int XAOutputDevice::ActiveDevices()
{
    return DacCommon().ActiveDevices();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ChannelsPerDevice() -- return channels per device
//---------------------------------------------------------------------------

unsigned int  XAOutputDevice::ChannelsPerDevice()
{
     return DacCommon().ChannelsPerDevice();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::Channels() -- return channels
//---------------------------------------------------------------------------

unsigned int  XAOutputDevice::Channels()
{
     return DacCommon().Channels();
}
//---------------------------------------------------------------------------
//  XAOutputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void  XAOutputDevice::ChannelDisableAll()
{
     DacCommon().ChannelDisableAll();
     DacCommon().ChannelPowerDownAll();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void  XAOutputDevice::ChannelEnabled(int ch, bool state)
{
     DacCommon().ChannelEnabled(ch, state);
     ChannelPowered(ch, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XAOutputDevice::ChannelEnabled(int ch) const
{
    return DacCommon().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ActiveChannels() -- Return number of active channels
//---------------------------------------------------------------------------

unsigned int XAOutputDevice::ActiveChannels()
{
    return DacCommon().ActiveChannels();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XAOutputDevice::ChannelPowered(int ch, bool state)
{
     DacCommon().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::ChannelPowered() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

bool XAOutputDevice::ChannelPowered(int ch) const
{
    return DacCommon().ChannelPowered(ch);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::SoftwareTrigger() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  XAOutputDevice::SoftwareTrigger(bool state)
{
    Common().SoftwareTrigger(FmcAFE_Common::tOutput, state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::SoftwareTrigger() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  XAOutputDevice::SoftwareTrigger(AfeDirection which, bool state)
{
    FmcAFE_Common::AfeDirection which2 = static_cast<FmcAFE_Common::AfeDirection>(which);
    Common().SoftwareTrigger(which2, state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TestModeEnabled() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  XAOutputDevice::TestModeEnabled(bool state, unsigned int mode)
{
    Common().TestModeEnabled(FmcAFE_Common::tOutput, state, mode);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TestFrequency() -- Set frequency for Test Mode on Output
//---------------------------------------------------------------------------

void  XAOutputDevice::TestFrequency(double frequency_hz)
{
//    unsigned int channels = DacCommon().ActiveChannels();
//    unsigned int dacdevs = ActiveDevices();
//    unsigned int factor;
//    if (channels==0 || dacdevs==0)
//        factor = 1;
//    else
//        factor = channels/dacdevs;
    //  On XA family, in general factor is always 1. Override this if custom factor needed
    unsigned int factor = 1;
    DacCommon().DacTestToneFrequency(Owner->DacClockFrequency(), frequency_hz, factor);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TestFrequency() -- Get frequency for Test Mode on Output
//---------------------------------------------------------------------------

double  XAOutputDevice::TestFrequency()
{
//    unsigned int factor = DacCommon().ActiveChannels();
//    if (channels==0)
//         channels = 1;
    //  On XA family, in general factor is always 1. Override this if custom factor needed
    unsigned int factor = 1;
    return DacCommon().DacTestToneFrequency(Owner->DacClockFrequency(), factor);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::Decimation() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XAOutputDevice::Decimation(unsigned int samples)
{
     DacCommon().Decimation(samples);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::Decimation() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

unsigned int XAOutputDevice::Decimation() const
{
    return DacCommon().Decimation();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XAOutputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    DacSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XAOutputDevice::Spi(unsigned int device, unsigned int address)
{
    return DacSpi().Spi(device, address);
}

//===========================================================================
//  CLASS XAOutputDevice::TriggerCfgImpl  --
//===========================================================================
//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::FramedMode() -- Setter
//---------------------------------------------------------------------------

void  XAOutputDevice::TriggerCfgImpl::FramedMode(bool state)
{
    OutD->DacCommon().FramedTriggerEnabled(state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::FramedMode() -- Getter
//---------------------------------------------------------------------------

bool XAOutputDevice::TriggerCfgImpl::FramedMode() const
{
    return OutD->DacCommon().FramedTriggerEnabled();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::Edge() -- Setter
//---------------------------------------------------------------------------

void  XAOutputDevice::TriggerCfgImpl::Edge(bool state)
{
    OutD->DacCommon().EdgeTrigger(state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::Edge() -- Getter
//---------------------------------------------------------------------------

bool XAOutputDevice::TriggerCfgImpl::Edge() const
{
    return OutD->DacCommon().EdgeTrigger();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::External() -- Setter
//---------------------------------------------------------------------------

void  XAOutputDevice::TriggerCfgImpl::External(bool state)
{
    OutD->DacCommon().ExternalTrigger(state);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::External() -- Getter
//---------------------------------------------------------------------------

bool XAOutputDevice::TriggerCfgImpl::External() const
{
    return OutD->DacCommon().ExternalTrigger();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::FrameSize() -- Setter
//---------------------------------------------------------------------------

void  XAOutputDevice::TriggerCfgImpl::FrameSize(unsigned int samples)
{
    OutD->DacCommon().FramedTriggerSize(samples);
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::FrameSize() -- Getter
//---------------------------------------------------------------------------

unsigned int XAOutputDevice::TriggerCfgImpl::FrameSize() const
{
    return OutD->DacCommon().FramedTriggerSize();
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::ExternalSyncSource() -- Setter
//---------------------------------------------------------------------------

void  XAOutputDevice::TriggerCfgImpl::ExternalSyncSource(AfeExtSyncOptions src)
{
    OutD->Common().ExternalSyncSelection(
                static_cast<FmcAFE_Common::AfeExtSyncOptions>(src) );
}

//---------------------------------------------------------------------------
//  XAOutputDevice::TriggerCfgImpl::ExternalSyncSource() -- Getter
//---------------------------------------------------------------------------

XAOutputDevice::AfeExtSyncOptions XAOutputDevice::TriggerCfgImpl::ExternalSyncSource() const
{
    FmcAFE_Common::AfeExtSyncOptions opt = OutD->Common().ExternalSyncSelection();
    return static_cast<XAOutputDevice::AfeExtSyncOptions>( opt );
}


#ifdef _MSC_VER
	#pragma warning(pop)
#endif


} // namespace Innovative


