// XAFamilyDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XAFamilyDeviceMap_MbH
#define XAFamilyDeviceMap_MbH

#include <X6Support_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>

#include <K7SubDevice_Misc_Mb.h>

#include <FmcSubDevice_Stream_Mb.h>

#include <XASubDevice_Misc_Mb.h>
#include <XASubDevice_AnalogIn_Mb.h>
#include <XASubDevice_AnalogOut_Mb.h>

#include <XUSubDevice_AnalogOut_Mb.h>
#include <XUSubDevice_Stream_Mb.h>
#include <XUSubDevice_Misc_Mb.h>

#include <PacketDeviceMap_Mb.h>
#include <PacketDeviceImpl_Mb.h>
#include <StreamDeviceEvents_Mb.h>
//#include <XAFamilyIdrom_Mb.h>
//#include <XAFamilySubDevice_Stream_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class XAFamilyDeviceMap;
class XAFamilyDeviceDriver;

//===========================================================================
//  CLASS IXA_IoDevice  -- Common XA Board Configuration Interface
//===========================================================================

class IXA_IoDevice
{
public:
    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essCarrier };

    virtual ~IXA_IoDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IXA_IoDevice::ITriggerCfg  -- Trigger Configuration Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ITriggerCfg
    {
    public:
        virtual ~ITriggerCfg()  {  }

        virtual void          FramedMode(bool state) = 0;
        virtual bool          FramedMode() const = 0;

        virtual void          Edge(bool state) = 0;
        virtual bool          Edge() const = 0;

        virtual void          External(bool state) = 0;
        virtual bool          External() const = 0;

        virtual void          FrameSize(unsigned int samples) = 0;
        virtual unsigned int  FrameSize() const = 0;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src) = 0;
        virtual AfeExtSyncOptions  ExternalSyncSource() const = 0;
    };
    //
    //  Interface Functions
    virtual IX6AnalogIoInfo &  Info() = 0;

    virtual unsigned int  Devices() = 0;
    virtual unsigned int  ChannelsPerDevice() = 0;
    virtual unsigned int  Channels() = 0;

    virtual void  DeviceDisableAll() = 0;
    virtual void  DeviceEnabled(int dev, bool state) = 0;
    virtual bool  DeviceEnabled(int dev) const = 0;

    virtual void  DevicePowered(int dev, bool state) = 0;
    virtual bool  DevicePowered(int dev) const = 0;

    virtual unsigned int  ActiveDevices() = 0;

    virtual void  ChannelDisableAll() = 0;
    virtual void  ChannelEnabled(int ch, bool state) = 0;
    virtual bool  ChannelEnabled(int ch) const = 0;

    virtual void  ChannelPowered(int ch, bool state) = 0;
    virtual bool  ChannelPowered(int ch) const = 0;

    virtual unsigned int  ActiveChannels() = 0;

    virtual ITriggerCfg &  Trigger() = 0;

    virtual void SoftwareTrigger(bool state) = 0;
    virtual void SoftwareTrigger(AfeDirection which, bool state) = 0;

    virtual void TestModeEnabled(bool state, unsigned int mode) = 0;
//    virtual void   TestFrequency(double frequency_hz) = 0;
//    virtual double TestFrequency() = 0;

    virtual void  Decimation(unsigned int samples) = 0;
    virtual unsigned int Decimation() const = 0;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int Spi(unsigned int device, unsigned int address) = 0;

};

//===========================================================================
//  CLASS ISuppliesXADeviceMapRequirements -- interfaces needed by XA DMap
//===========================================================================

class ISuppliesXADeviceMapRequirements : public IRequires_SioPll,
                                         public IRequires_VeloInCount, public IRequires_VeloOutCount,
                                         public IRequires_Nothing
{
public:
    virtual ~ISuppliesXADeviceMapRequirements()
        {}

};

//===========================================================================
//  CLASS XAFamilyDeviceMap  -- Register & low level support for XAFamily
//===========================================================================

class XAFamilyDeviceMap :  public PacketDeviceImpl,
                     public PacketDeviceMap,
                     public IUsesStreamDeviceEventCallbacks,
                     public ISuppliesXADeviceMapRequirements
{
public:
    friend class XADac_DeviceMixin;
    friend class XAFamilyBoard;
//    friend class XAInputDevice;
//    friend class XAOutputDevice;
//    friend class XAStandardPriClockDevice;
    //
    //  Member Functions
    XAFamilyDeviceMap(IStreamDeviceEvents * owner);
    virtual ~XAFamilyDeviceMap();

    // Methods
    // Version
    std::string         Revision() const;

    //  Hardware Device Access
//    XAFamilyAFE_Clock::XAFamilyClockIo &  Clock()   {  return AFEClk_SDev.Clock();  }
    IFpaLoader &                    VsProm()  {  return Fpa_SDev.VsProm();  }
    XUDioPortIntf &                 FrontPanelDio() {  return FrontPanelDio_SDev;  }
    AdvancedDioPortIntf &           Dio()           {  return Dio_SDev;  }
    const IX6LogicVersionInfo &     Info() const    {  return Info_SDev;  }

    IUsesX6Alerts &             Alerts()  {  return Alerts_SDev;  }
    IX6ThermalDevice &          Thermal() {  return Thermal_SDev;  }

    IVeloPacketizer &           VeloIn()    {  return VeloIn_SDev;  }
    IVeloDepacketizer &         VeloOut()   {  return VeloOut_SDev;  }

    IIdRom &                    IdRom()  {  return IdRom_Intf;   }

    XU_VFifoInfo &              VFifo()   {  return VFifo_SDev;    }
    IMulticharUart &            Uart(unsigned int ch);

    IX6Aurora &               Aurora(unsigned int /*idx*/) {  return Aurora0_SDev;    }
    X6SioPll  &               AuroraPll()  {  return SioPll_SDev;   }

    XAProm &  RawFpaRom()    {  return Fpa_SDev.Raw();   }

    X6FlashRom & RawIdRom()  {  return FlashRom_SDev;   }


public: // pseudo-protected

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    //
    //  Busmaster Intf functions
    virtual bool    AlertTest(unsigned int pid);
    virtual bool    AlertCheck(const Buffer & buffer);

    //void            DumpIsrStatus(std::vector<int> & vec);
    typedef std::vector<int>  IntVec;
    void            DumpIsrStatus(IntVec & vec);


    virtual double  StandardClockFrequency() = 0;   // PRI Frequency
    virtual double  Get_DacClockFrequency() = 0;

    Trace                  Debug;

protected:
    XAFamilyDeviceDriver *       DDeviceDriver;
    //
    //  Wishbone Bus Spaces
    WishboneBusSpace       WB_Periph;
    WishboneBusSpace       WB_Loader;
    WishboneBusSpace       WB_Thermal;
    WishboneBusSpace       WB_Packetizer;
    WishboneBusSpace       WB_Alerts;
    WishboneBusSpace       WB_DigitalIo;
    WishboneBusSpace       WB_FlashRom;
    WishboneBusSpace       WB_AFE;
    WishboneBusSpace       WB_Uart;
    WishboneBusSpace       WB_Aurora0;
    //
    //  Subdevices
    X6Info                         Info_SDev;
    X6Peripherals                  Periph_SDev;
    X6SioPll                       SioPll_SDev;
    XU_VFifoInfo                   VFifo_SDev;
    X6PacketizerCfg                VeloIn_SDev;
    X6DepacketizerCfg              VeloOut_SDev;
    XUDigitalIo                    FrontPanelDio_SDev;
    X6DigitalIo                    Dio_SDev;
    X6Thermal                      Thermal_SDev;
    X6AlertSystem                  Alerts_SDev;
    K7DualUart                     Uart_SDev;
    X6Aurora                       Aurora0_SDev;    
    FmcAFE_Common                  AFECommon_SDev;

    XAFpaLoader                    Fpa_SDev;      // VSPROM
    X6FlashRom                     FlashRom_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    X6IdromPage                  AdcCal_PG;
    X6IdromPage                  DacCal_PG;
    X6IdromPage                  UserRom_PG;

    //=======================================================================
    //  CLASS XADeviceMap::XAIdromIo  -- Access to IDROM Objects
    //=======================================================================
    class XAIdromIo : public IIdRom
    {
    public:

    virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
    //  Page access
    virtual X6SystemIdromPage &  System() { return Owner->System_PG;  }
    virtual X6IdromPage &        AdcCal() { return Owner->AdcCal_PG;  }
    virtual X6IdromPage &        DacCal() { return Owner->DacCal_PG;  }
    virtual X6IdromPage &        User()   { return Owner->UserRom_PG;  }

    XAIdromIo(XAFamilyDeviceMap * owner) :  Owner(owner)
        {}
    virtual ~XAIdromIo()
        {}

    private:
        XAFamilyDeviceMap *  Owner;
    };

    XAIdromIo                    IdRom_Intf;
    friend class XAIdromIo;

public:
    FmcAFE_Common & Get_AFECommon_SDev() {  return AFECommon_SDev;  }

};
//===========================================================================
//  CLASS ISuppliesAddedXAAdcDeviceMapRequirements -- intfs for XAAdc DMap
//===========================================================================
//class ISuppliesX6Requirements :
//                                public IRequires_DacSpi,
//                                public IRequires_DacLatencyCal,

class ISuppliesAddedXAAdcDeviceMapRequirements : public IRequires_XACommonParts,
                            public IRequires_PriClockDevice,
                            public IRequires_MuxedAdcSpi, public IRequires_AdcCalParts
{
public:
    virtual ~ISuppliesAddedXAAdcDeviceMapRequirements()
        {}

};

//===========================================================================
//  CLASS XAAdcDeviceMap  -- input only card device map
//===========================================================================

class XAAdcDeviceMap : public XAFamilyDeviceMap,
                       public ISuppliesAddedXAAdcDeviceMapRequirements
{
public:
    friend class XAInputDevice;

    XAAdcDeviceMap(IStreamDeviceEvents * owner);
    virtual ~XAAdcDeviceMap()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:
    XAAFE_AdcCommon                AFEAdcC_SDev;
    X6AFE_PulseRepetitionInterval  AFEAdcPri_SDev;
    XAAFE_AdcSpi                   AFEAdcSpi_SDev;
    X6AFE_AdcCalibration           AFEAdcCal_SDev;
};

//===========================================================================
//  CLASS ISuppliesAdded_XADac_DeviceMixin_Requirements
//===========================================================================
//  XADac_DeviceMixin requirements

class ISuppliesAdded_XADac_DeviceMixin_Requirements : public IRequires_XACommonParts,
                                                      public IRequires_PriClockDevice,
                                                      public IRequires_MuxedDacSpi,
                                                      public IRequires_DacCalParts,
                                                      public IRequires_XUPatternMode
{
public:
    virtual ~ISuppliesAdded_XADac_DeviceMixin_Requirements()
        {}

};

//===========================================================================
//  CLASS XADac_DeviceMixin - Dac standard hardware device pack
//===========================================================================

class XADac_DeviceMixin : public ISuppliesAdded_XADac_DeviceMixin_Requirements
{
public:
    friend class XAOutputDevice;

    XADac_DeviceMixin(XAFamilyDeviceMap * owner);
    virtual ~XADac_DeviceMixin()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    //
    //  Subdevice Customization - implement board specific pieces
    //virtual unsigned int GetDacSpiDeviceCount() = 0;

protected:
    //
    //  Subdevices
    XAFamilyDeviceMap *              DDMxOwner;
    XAAFE_DacCommon                  AFEDacC_SDev;
    X6AFE_PulseRepetitionInterval    AFEDacPri_SDev;
    XAAFE_DacSpi                     AFEDacSpi_SDev;
    X6AFE_DacCalibration             AFEDacCal_SDev;
    XU_DacPatternModeSystem          DacPattern_SDev;
    //    XAFamilyAFE_DacLatencyCalibration    AFEDacLatencyCal_SDev;

    FmcAFE_Common &  Common() {  return DDMxOwner->Get_AFECommon_SDev();  }
    double          DacClockFrequency() { return DDMxOwner->Get_DacClockFrequency(); }

};

//===========================================================================
//  CLASS XADacDeviceMap  -- input/output card device map
//===========================================================================

class XADacDeviceMap : public XAFamilyDeviceMap, public XADac_DeviceMixin//,
                      // public ISuppliesAdded_XADac_DeviceMap_Requirements
{
public:

    XADacDeviceMap(IStreamDeviceEvents * owner);
    virtual ~XADacDeviceMap()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:

};

//===========================================================================
//  CLASS XABidirDeviceMap  -- input/output card device map
//===========================================================================

class XABidirDeviceMap : public XAAdcDeviceMap, public XADac_DeviceMixin//,
                      // public ISuppliesAdded_XABidir_DeviceMap_Requirements
{
public:

    XABidirDeviceMap(IStreamDeviceEvents * owner);
    virtual ~XABidirDeviceMap()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:

};

//===========================================================================
//  CLASS XAInputDevice  -- Partially Implemented XA Family Input Device
//===========================================================================

class XAInputDevice : public IXA_IoDevice
{
public:

    XAInputDevice( XAAdcDeviceMap * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~XAInputDevice()  {  }


    //virtual XAFamilyCalibrationSection & Cal() = 0;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAInputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IXA_IoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(XAInputDevice * in_dev)
            :  InD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        XAInputDevice * InD;
    };
    
    friend class XAInputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual unsigned int  Devices();
    virtual unsigned int  ChannelsPerDevice();
    virtual unsigned int  Channels();

    virtual void  DeviceDisableAll() {  ChannelDisableAll();  }
    virtual void  DeviceEnabled(int dev, bool state);
    virtual bool  DeviceEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual ITriggerCfg &  Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return AdcPri();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double /*frequency_hz*/ ) {}
    virtual double TestFrequency() {  return 0.0;  }

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

protected:
    XAAdcDeviceMap *   Owner;
    TriggerCfgImpl  TriggerCfg;

    //  shortcuts to subdevices
    X6Info &                FpgaInfo() {  return Owner->Info_SDev;  }
    const X6Info &          FpgaInfo() const {  return Owner->Info_SDev;  }
    X6Peripherals &         Periph() {  return Owner->Periph_SDev;  }
    const X6Peripherals &   Periph() const {  return Owner->Periph_SDev;  }
    XAAFE_AdcCommon &       AdcCommon() {  return Owner->AFEAdcC_SDev;  }
    const XAAFE_AdcCommon & AdcCommon() const {  return Owner->AFEAdcC_SDev;  }
    FmcAFE_Common &          Common() {  return Owner->AFECommon_SDev;  }
    const FmcAFE_Common &    Common() const {  return Owner->AFECommon_SDev;  }
    XAAFE_AdcSpi &          AdcSpi() {  return Owner->AFEAdcSpi_SDev;  }
    const XAAFE_AdcSpi &    AdcSpi() const {  return Owner->AFEAdcSpi_SDev;  }
    X6AFE_AdcCalibration &          AdcCal() {  return Owner->AFEAdcCal_SDev;  }
    const X6AFE_AdcCalibration &    AdcCal() const {  return Owner->AFEAdcCal_SDev;  }
    X6AFE_PulseRepetitionInterval &        AdcPri() {  return Owner->AFEAdcPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  AdcPri() const {  return Owner->AFEAdcPri_SDev;  }


};

//===========================================================================
//  CLASS XAOutputDevice  -- Partially Impl XA Family Output Device
//===========================================================================

class XAOutputDevice : public IXA_IoDevice
{
public:

    XAOutputDevice( XADac_DeviceMixin * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~XAOutputDevice()  {  }

    //virtual XAFamilyCalibrationSection &  Cal() = 0;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAFamilyOutputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IXA_IoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(XAOutputDevice * in_dev)
            :  OutD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        XAOutputDevice * OutD;
    };
    
    friend class XAOutputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual unsigned int  Devices();
    virtual unsigned int  ChannelsPerDevice();
    virtual unsigned int  Channels();

    virtual void  DeviceDisableAll() {  ChannelDisableAll();  }
    virtual void  DeviceEnabled(int dev, bool state);
    virtual bool  DeviceEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual ITriggerCfg &  Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return DacPri();  }
    virtual IXUPatternModeSystem & Pattern() {  return DacPattern();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double frequency_hz);
    virtual double TestFrequency();

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

//    virtual bool  DacInternalCalibrationOk()
//                    {  return DacLatencyCal().CalibrationOk();  }

protected:
    XADac_DeviceMixin *   Owner;
    TriggerCfgImpl  TriggerCfg;

    //  shortcuts to subdevices
    XAAFE_DacCommon &       DacCommon() {  return Owner->AFEDacC_SDev;  }
    const XAAFE_DacCommon & DacCommon() const {  return Owner->AFEDacC_SDev;  }
    FmcAFE_Common &          Common() {  return Owner->Common();  }
    const FmcAFE_Common &    Common() const {  return Owner->Common();  }
    XAAFE_DacSpi &          DacSpi() {  return Owner->AFEDacSpi_SDev;  }
    const XAAFE_DacSpi &    DacSpi() const {  return Owner->AFEDacSpi_SDev;  }
    X6AFE_DacCalibration &          DacCal() {  return Owner->AFEDacCal_SDev;  }
    const X6AFE_DacCalibration &    DacCal() const {  return Owner->AFEDacCal_SDev;  }
    X6AFE_PulseRepetitionInterval &        DacPri() {  return Owner->AFEDacPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  DacPri() const {  return Owner->AFEDacPri_SDev;  }
    XU_DacPatternModeSystem &        DacPattern() {  return Owner->DacPattern_SDev;  }
    const XU_DacPatternModeSystem &  DacPattern() const {  return Owner->DacPattern_SDev;  }
    //XAFamilyAFE_DacLatencyCalibration &  DacLatencyCal() {  return Owner->AFEDacLatencyCal_SDev;  }
    //  shortcuts to other things
};

//=============================================================================
//  CLASS XAFamilyStandardPriClockDevice  --  Normal impl to access clock for PRI
//=============================================================================

class XAStandardPriClockDevice : public IPriClockDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->StandardClockFrequency();  }
    virtual int   CalculatePriDivisor()
        {  return Divisor;  }

    XAStandardPriClockDevice(XAFamilyDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~XAStandardPriClockDevice()
        {}

private:
    XAFamilyDeviceMap * Owner;
    int Divisor;
};

//=============================================================================
//  CLASS XAStandardFrameCalcDevice  --  Supply Divider on Frame ticks
//=============================================================================

class XAStandardFrameCalcDevice : public IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor()
        {  return Divisor;  }

    XAStandardFrameCalcDevice(XAFamilyDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~XAStandardFrameCalcDevice()
        {}

private:
    XAFamilyDeviceMap * Owner;
    int Divisor;
};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}  // Innovative
//---------------------------------------------------------------------------
#endif
