//---------------------------------------------------------------------------
// XASubDevice_AnalogIn_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XASubDevice_AnalogIn_Mb.h>
#include <SystemSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XAAFE_AdcCommon -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAAFE_AdcCommon
//---------------------------------------------------------------------------

const unsigned int MaxXAAdcChannels = 32;

//---------------------------------------------------------------------------
//  constructor for class XAAFE_AdcCommon
//---------------------------------------------------------------------------

XAAFE_AdcCommon::XAAFE_AdcCommon(IRequires_XACommonParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       AdcEnable_Reg(space, regs.AfeAdcEnable_Addr),
       AdcPower_Reg(space, regs.AfeAdcPower_Addr),
       AdcTriggerCfgReg(space, regs.AfeAdcTriggerCfg_Addr),
       AdcDecimationReg(space, regs.AfeAdcDecimation_Addr),
       AdcReset_Reg(space, regs.AfeAdcReset_Addr),
       AdcSync_Reg(space, regs.AfeAdcSync_Addr),
       FDevices(0), FChannelsPerDevice(0), FPhysDevices(0), FChannelsPerPhysDevice(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAAFE_AdcCommon
//---------------------------------------------------------------------------

XAAFE_AdcCommon::~XAAFE_AdcCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAAFE_AdcCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_XACommonParts::devAdc);
    MDC->GetAnalogDeviceInfo(IRequires_XACommonParts::devAdc, FDevices, FChannelsPerDevice);
    MDC->GetAnalogPhysicalDeviceInfo(IRequires_XACommonParts::devAdc, FPhysDevices, FChannelsPerPhysDevice);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::OpenHardware()
//---------------------------------------------------------------------------

void  XAAFE_AdcCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAAFE_AdcCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::ChannelDisableAll()
{
    AdcEnable_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  XAAFE_AdcCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<MaxXAAdcChannels; i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::ChannelEnabled(int ch, bool state)
{
    AdcEnable_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::ChannelEnabled(int ch) const
{
    return AdcEnable_Reg.Bit(ch);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::ChannelPowerDownAll()
{
    AdcPower_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::ChannelPowered(int ch, bool state)
{
     AdcPower_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::ChannelPowered(int ch) const
{
    return AdcPower_Reg.Bit(ch);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::DeviceIsEnabled() -- Enable an I/O device
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::DeviceIsEnabled(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::DeviceIsEnabled() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::DeviceIsEnabled(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelEnabled(base);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::DevicePowered() -- Enable an I/O device
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::DevicePowered(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::DevicePowered() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::DevicePowered(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelPowered(base);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::Reset() --
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::Reset()
{
    AdcReset_Reg.Value(1);
    Sleep(1);
    AdcReset_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::Sync() --
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::Sync()
{
    AdcSync_Reg.Value(1);
    Sleep(1);
    AdcSync_Reg.Value(0);
}


//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ActiveDevices() -- return count of active devices
//---------------------------------------------------------------------------

unsigned int  XAAFE_AdcCommon::ActiveDevices()
{
    unsigned int active_devices(0);
    for (unsigned int i=0; i<Devices(); i++)
        if (DeviceIsEnabled(i)==true)
            active_devices++;

    return active_devices;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    AdcTriggerCfgReg.AdcFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int XAAFE_AdcCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = AdcTriggerCfgReg.AdcFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::EdgeTrigger(bool state)
{
    AdcTriggerCfgReg.AdcRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::EdgeTrigger() const
{
    return AdcTriggerCfgReg.AdcRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::ExternalTrigger(bool state)
{
    AdcTriggerCfgReg.AdcExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::ExternalTrigger() const
{
    return AdcTriggerCfgReg.AdcExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::FramedTriggerEnabled(bool state)
{
    AdcTriggerCfgReg.AdcFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool XAAFE_AdcCommon::FramedTriggerEnabled() const
{
    return AdcTriggerCfgReg.AdcFramedMode.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::Decimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void XAAFE_AdcCommon::Decimation(unsigned int samples)
{
    AdcDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::Decimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int XAAFE_AdcCommon::Decimation() const
{
    return AdcDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::PhysicalDeviceEnabled() --
//---------------------------------------------------------------------------
//  Physical device should be enabled if any channel on it is enabled...

bool XAAFE_AdcCommon::PhysicalDeviceEnabled(unsigned int phys_dev) const
{
    bool enabled(false);
    if (phys_dev>=FPhysDevices)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int end   = start + FChannelsPerPhysDevice;

    for (unsigned int ch=start;  ch<end; ch++)
        enabled |= ChannelEnabled(ch);
    return enabled;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCommon::PhysicalChannelEnabled() --
//---------------------------------------------------------------------------
//  return enabled state for the Nth channel on device D...

bool XAAFE_AdcCommon::PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const
{
    if (phys_dev>=FPhysDevices || device_ch>=FChannelsPerPhysDevice)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int ch  = start + device_ch;

    return ChannelEnabled(ch);
}

//===========================================================================
//  CLASS XAAFE_AdcSpi -- XA AFE ADC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAAFE_AdcSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XAAFE_AdcSpi
//---------------------------------------------------------------------------

XAAFE_AdcSpi::XAAFE_AdcSpi(IRequires_MuxedAdcSpi * mdc, IAddressingSpace & space, XASpiDevice::MapRegisters & regs)
    :  MDC(mdc), SpiDev(space, regs, "XA Adc"), SpiCount(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAAFE_AdcSpi
//---------------------------------------------------------------------------

XAAFE_AdcSpi::~XAAFE_AdcSpi()
{
}

//---------------------------------------------------------------------------
//  XAAFE_AdcSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAAFE_AdcSpi::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SpiCount = MDC->GetAdcSpiDeviceCount();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  XAAFE_AdcSpi::OpenHardware()
//---------------------------------------------------------------------------

void  XAAFE_AdcSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_AdcSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAAFE_AdcSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_AdcSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XAAFE_AdcSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiCount)
        SpiDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XAAFE_AdcSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XAAFE_AdcSpi::Spi(unsigned int device, unsigned int address)
{
    if (device < SpiCount)
        return SpiDev.Spi(device, address);
    else
        return 0;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcSpi::SpiWriteAll() -- write to all SPI device
//---------------------------------------------------------------------------

void  XAAFE_AdcSpi::SpiWriteAll(unsigned int address, unsigned int data)
{
    SpiDev.SpiWriteAll(address, data);
}

/*
//===========================================================================
//  CLASS XAAFE_AdcCalibrationControl -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAAFE_AdcCalibrationControl
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XAAFE_AdcCalibrationControl
//---------------------------------------------------------------------------

XAAFE_AdcCalibrationControl::XAAFE_AdcCalibrationControl(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AdcCalibrationReg(space, regs.AfeAdcCal_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAAFE_AdcCalibrationControl
//---------------------------------------------------------------------------

XAAFE_AdcCalibrationControl::~XAAFE_AdcCalibrationControl()
{
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCalibrationControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAAFE_AdcCalibrationControl::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XAAFE_AdcCalibrationControl::OpenHardware()
//---------------------------------------------------------------------------

void  XAAFE_AdcCalibrationControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_AdcCalibrationControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAAFE_AdcCalibrationControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_AdcCalibrationControl::StartCalibration()
//---------------------------------------------------------------------------

void  XAAFE_AdcCalibrationControl::StartCalibration()
{
    if (!DeviceEnabled())
        return;

    AdcCalibrationReg.CalStart.Value(false);
    AdcCalibrationReg.CalStart.Value(true);
    AdcCalibrationReg.CalStart.Value(false);

    Debug.Log("Adc Calibration Started");

}

//---------------------------------------------------------------------------
//  XAAFE_AdcCalibrationControl::CalibrationResults()
//---------------------------------------------------------------------------

unsigned int  XAAFE_AdcCalibrationControl::CalibrationResults()
{
    if (!DeviceEnabled())
        return 0;

    return AdcCalibrationReg.CalDone.Value();

}


//===========================================================================
//  CLASS XA110_AdcCalibrationControl -- X6 AFE ADC Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA110_AdcCalibrationControl
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA110_AdcCalibrationControl
//---------------------------------------------------------------------------

XA110_AdcCalibrationControl::XA110_AdcCalibrationControl(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AdcCalibrationReg(space, regs.AfeAdcCal_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA110_AdcCalibrationControl
//---------------------------------------------------------------------------

XA110_AdcCalibrationControl::~XA110_AdcCalibrationControl()
{
}

//---------------------------------------------------------------------------
//  XA110_AdcCalibrationControl::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XA110_AdcCalibrationControl::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XA110_AdcCalibrationControl::OpenHardware()
//---------------------------------------------------------------------------

void  XA110_AdcCalibrationControl::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XA110_AdcCalibrationControl::ConfigureHardware()
//---------------------------------------------------------------------------

void  XA110_AdcCalibrationControl::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XA110_AdcCalibrationControl::StartCalibration()
//---------------------------------------------------------------------------

void  XA110_AdcCalibrationControl::StartCalibration(unsigned int ch)
{
    if (!DeviceEnabled())
        return;

	AdcCalibrationReg.CalStatus.Value((ch) ? true : false);

    AdcCalibrationReg.CalStart.Value(0);
    AdcCalibrationReg.CalStart.Value(1);
    AdcCalibrationReg.CalStart.Value(0);

    Debug.Log("Adc Calibration Started, Ch", ch);

}

//---------------------------------------------------------------------------
//  XA110_AdcCalibrationControl::WaitForCal()
//---------------------------------------------------------------------------

void  XA110_AdcCalibrationControl::WaitForCal()
{
    if (!DeviceEnabled())
        return;

    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        bool done = AdcCalibrationReg.CalDone.Value();
        if (done)
            return;
        else
            Innovative::Sleep(2);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " Adc Cal Done Failure (>10ms)";
    Debug.Log(msg.str());

}

//---------------------------------------------------------------------------
//  XA110_AdcCalibrationControl::CalibrationResults()
//---------------------------------------------------------------------------

unsigned int  XA110_AdcCalibrationControl::CalibrationResults(unsigned int / *ch* /)
{
    if (!DeviceEnabled())
        return 0;

    return AdcCalibrationReg.Value();

}

//===========================================================================
//  CLASS XAVga_I2C_Bus -- Standard Vga I2C Bus "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAVga_I2C_Bus
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XAVga_I2C_Bus
//---------------------------------------------------------------------------

XAVga_I2C_Bus::XAVga_I2C_Bus(IRequires_Nothing * mdc, IAddressingSpace & space,
                                   MapRegisters & regs, I2C_SubAddr & subaddrs )
    :  MDC(mdc),
       VgaBusI2CReg(space, regs.I2CControl_Addr),
       VgaBusIntf(VgaBusI2CReg),
       Vga0_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga0_SubAddr),
       Vga1_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga1_SubAddr),
       Vga2_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga2_SubAddr),
       Vga3_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga3_SubAddr),
       Vga4_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.Vga4_SubAddr),
       VgaIdrom_I2C(VgaBusIntf, I2CDevice::SubAddress, subaddrs.VgaIdrom_SubAddr),
       VgaIdromDev(VgaIdrom_I2C),
       Vga(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAVga_I2C_Bus
//---------------------------------------------------------------------------

XAVga_I2C_Bus::~XAVga_I2C_Bus()
{
}

//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAVga_I2C_Bus::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::OpenHardware()
//---------------------------------------------------------------------------

void  XAVga_I2C_Bus::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAVga_I2C_Bus::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::Vga()  --  Access Vga I2C
//---------------------------------------------------------------------------

I2CDeviceN &  XAVga_I2C_Bus::VgaI2C(unsigned int idx)
{
    switch (idx)
        {
        default:
        case 0:     return Vga0_I2C;
        case 1:     return Vga1_I2C;
        case 2:     return Vga2_I2C;
        case 3:     return Vga3_I2C;
        case 4:     return Vga4_I2C;
        }
}


//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::VgaDacLatch()  --  Latch VGA update
//---------------------------------------------------------------------------

void  XAVga_I2C_Bus::VgaDacLatch()
{
    VgaBusI2CReg.VgaDacLatch.Set();
    VgaBusI2CReg.VgaDacLatch.Clear();
    VgaBusI2CReg.VgaDacLatch.Set();
}

//===========================================================================
//  CLASS XAVga_I2C_Bus::LTC2635_8  -- DAC controller wrapper
//===========================================================================
//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::LTC2635_8::SetAdcGain()  -- set gain for an Adc Channel
//---------------------------------------------------------------------------

void  XAVga_I2C_Bus::LTC2635_8::SetAdcGain(unsigned int dev, unsigned int adc, float db_gain)
{
    //  Calculate DAC level for a Gain in DB
    //  Gain_db = 20 * V_gain + 18
    //  .....V_gain is in 256 level steps from -.625 to +.625
    //  V_gain = 2*.625*step/256 - .625
    //   db range is from 6 db to 30 db
    float step = static_cast<float>(10.24*db_gain - 56.32);
    if (step<0.0)
        step = 0;
    if (step>255)
        step = 255;

    // use command code 0 to set DAC for a channel
    SendCommand(dev, 0, adc, (unsigned int)step);
}

//---------------------------------------------------------------------------
//  XAVga_I2C_Bus::LTC2635_8::SendCommand()  --
//---------------------------------------------------------------------------

void  XAVga_I2C_Bus::LTC2635_8::SendCommand(unsigned int dev, unsigned int cmd_code,
                                            unsigned int addr_code, unsigned int data)
{
    std::vector<int> Data(2);
    unsigned int addr = (cmd_code<<4)|(addr_code&0xF);
    Data[0] = data;    // 8 bit data is left justified in 2 word cmd...
    Data[1] = 0;
    Owner->VgaI2C(dev).MultiRegW(addr, &Data[0], static_cast<int>(Data.size()) );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//===========================================================================
//  CLASS XA_AdcCalCoefficientLoad -- XA ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class XA_AdcCalCoefficientLoad
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_AdcCalCoefficientLoad
//---------------------------------------------------------------------------

XA_AdcCalCoefficientLoad::XA_AdcCalCoefficientLoad(IRequires_AdcCalParts * mdc,
                IAddressingSpace & space, MapRegisters & regs,
                unsigned int MaxAdcs )
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, MaxAdcs),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, MaxAdcs),
       FIncomingAdcs(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_AdcCalCoefficientLoad
//---------------------------------------------------------------------------

XA_AdcCalCoefficientLoad::~XA_AdcCalCoefficientLoad()
{
    if (CalOverride)
        delete CalOverride;
}

//---------------------------------------------------------------------------
//  XA_AdcCalCoefficientLoad::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XA_AdcCalCoefficientLoad::ConstructCustomParts()
{
    FIncomingAdcs = MDC->GetCalibrationAdcCount();
    CalOverride = MDC->GetAdcCalibrationOverride();
}

//---------------------------------------------------------------------------
//  XA_AdcCalCoefficientLoad::OpenHardware()
//---------------------------------------------------------------------------

void  XA_AdcCalCoefficientLoad::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    for (unsigned int i=0; i<FIncomingAdcs; i++)
        {
        SetCalibrationGain(i, 0x10000);
        SetCalibrationOffset(i, 0x000);
        }
}

//---------------------------------------------------------------------------
//  XA_AdcCalCoefficientLoad::ConfigureHardware()
//---------------------------------------------------------------------------

void  XA_AdcCalCoefficientLoad::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XA_AdcCalCoefficientLoad::SetCalibrationOffset()
//---------------------------------------------------------------------------

void  XA_AdcCalCoefficientLoad::SetCalibrationOffset(unsigned int ch, unsigned int value)
{
    Debug.Log("Adc Cal Offset=0x", value, 16);
    if (ch<FIncomingAdcs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideOffset(value);
            if (CalOverride->OverrideOffsetUpdate())
                {
                CalOverride->WriteOffset(ch, value);
                return;
                }
            }
        OffsetCoeffArray[ch].Value(value);
        }
}

//---------------------------------------------------------------------------
//  XA_AdcCalCoefficientLoad::SetCalibrationGain()
//---------------------------------------------------------------------------

void  XA_AdcCalCoefficientLoad::SetCalibrationGain(unsigned int ch, unsigned int value)
{
    Debug.Log("Adc Cal Gain=0x", value, 16);
    if (ch<FIncomingAdcs)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideGain(value);
            if (CalOverride->OverrideGainUpdate())
                {
                CalOverride->WriteGain(ch, value);
                return;
                }
            }
        GainCoeffArray[ch].Value(value);
        }
}

*/
}  // namespace Innovative

