// XASubDevice_AnalogIn_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef XASubDevice_AnalogIn_MbH
#define XASubDevice_AnalogIn_MbH

#include <X6Support_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>
#include <SpiDevice_Mb.h>
//#include <PmcIdrom_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS XAAFE_AdcCommon -- XA AFE ADC Common Registers "SubDevice"
//===========================================================================

typedef IRequires_FmcCommonParts  IRequires_XACommonParts;

class XAAFE_AdcCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAAFE_AdcCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcEnable_Addr;
        unsigned int   AfeAdcPower_Addr;
        unsigned int   AfeAdcTriggerCfg_Addr;
        unsigned int   AfeAdcDecimation_Addr;
        unsigned int   AfeAdcReset_Addr;
        unsigned int   AfeAdcSync_Addr;
    };

    XAAFE_AdcCommon(IRequires_XACommonParts * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XAAFE_AdcCommon();

    //
    //  Methods
    unsigned int  Devices() const  {  return FDevices;  }
    unsigned int  ChannelsPerDevice() const {  return FChannelsPerDevice;  }
    unsigned int  Channels() const {  return FDevices * FChannelsPerDevice;  }

    virtual void  DeviceIsEnabled(int dev, bool state);
    virtual bool  DeviceIsEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual void  Reset();
    virtual void  Sync();

    virtual unsigned int  ActiveDevices();

    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    //  Physical Device Enable Mapping
    bool  PhysicalDeviceEnabled(unsigned int phys_dev) const;
    bool  PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const;

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAAFE_AdcCommon::AdcTriggerCfgRegister -- Configure ADC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  AdcFrameSize;
        RegisterBit       AdcRisingEdge;
        RegisterBit       AdcFramedMode;
        RegisterBit       AdcExternalTrigger;

    public:
        AdcTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcFrameSize(*this, 0, 24),   AdcRisingEdge(*this, 29),
               AdcFramedMode(*this, 30),     AdcExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAAFE_AdcCommon::AdcDecimationRegister  -- Adc Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        AdcDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_XACommonParts *    MDC;
    IFrameCalcDevice *            Dev;
    Register                      AdcEnable_Reg;
    Register                      AdcPower_Reg;
    AdcTriggerCfgRegister         AdcTriggerCfgReg;
    AdcDecimationRegister         AdcDecimationReg;
    Register                      AdcReset_Reg;
    Register                      AdcSync_Reg;


    unsigned int                  FDevices;
    unsigned int                  FChannelsPerDevice;
    unsigned int                  FPhysDevices;
    unsigned int                  FChannelsPerPhysDevice;

};

//===========================================================================
//  CLASS XAAFE_AdcSpi -- XA Muxed ADC Spi Device Manager "SubDevice"
//===========================================================================

class XAAFE_AdcSpi : public IX6SubDevice
{
public:

    XAAFE_AdcSpi(IRequires_MuxedAdcSpi * owner, IAddressingSpace & space, XASpiDevice::MapRegisters & regs);
    virtual ~XAAFE_AdcSpi();

    unsigned int AdcSpiCount() {  return SpiCount;  }
    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address);

    void  SpiWriteAll(unsigned int address, unsigned int data);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_MuxedAdcSpi *  MDC;
    XASpiDevice              SpiDev;
    unsigned int             SpiCount;
};

typedef X6AFE_AdcCalibration  XA_AdcCalCoefficientLoad;

/*
//===========================================================================
//  CLASS XAAFE_AdcCalibrationControl -- XA AFE ADC Cal Control "SubDevice"
//===========================================================================

class XAAFE_AdcCalibrationControl : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAAFE_AdcCalibrationControl::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcCal_Addr;
    };

    XAAFE_AdcCalibrationControl(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XAAFE_AdcCalibrationControl();

    //
    //  Methods
    void          StartCalibration();
    unsigned int  CalibrationResults();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAAFE_AdcCalibrationControl::AdcCalibrationRegister -- Configure ADC Cal
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcCalibrationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       CalStart;
        RegisterBitGroup  CalDone;

    public:
        AdcCalibrationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               CalStart(*this, 0),   CalDone(*this, 16, 16)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *     MDC;
    AdcCalibrationRegister  AdcCalibrationReg;

};

//===========================================================================
//  CLASS XA110_AdcCalibrationControl -- XA AFE ADC Cal Control "SubDevice"
//===========================================================================

class XA110_AdcCalibrationControl : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA110_AdcCalibrationControl::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcCal_Addr;
    };

    XA110_AdcCalibrationControl(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XA110_AdcCalibrationControl();

    //
    //  Methods
    void          StartCalibration(unsigned int ch);
    void          WaitForCal();
    unsigned int  CalibrationResults(unsigned int ch);
    bool          CalibrationDone()
                      {  return AdcCalibrationReg.CalDone.Value();  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA110_AdcCalibrationControl::AdcCalibrationRegister -- Configure ADC Cal
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcCalibrationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       CalStart;
        RegisterBit       CalStatus;
        RegisterBit       CalSkip;
        RegisterBitGroup  CalEyeAligned;
        RegisterBit       CalPrbsLocked;
        RegisterBit       CalPrbsAligned;
        RegisterBit       CalDone;
        RegisterBit       CalAdcClkStopped;
        RegisterBit       CalAdcClkLocked;

    public:
        AdcCalibrationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               CalStart(*this, 0),  CalStatus(*this, 8), CalSkip(*this, 12),
               CalEyeAligned(*this, 14, 13), CalPrbsLocked(*this, 27),
               CalPrbsAligned(*this, 28), CalDone(*this, 29),
               CalAdcClkStopped(*this, 30), CalAdcClkLocked(*this, 31)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *     MDC;
    AdcCalibrationRegister  AdcCalibrationReg;

};


//===========================================================================
//  CLASS XA_AdcCalCoefficientLoad -- X6 ADC Cal Coeff Loader "SubDevice"
//===========================================================================

class XA_AdcCalCoefficientLoad : public IX6SubDevice, public IAdvGainCalibrationDevice
{
public:
    //=======================================================================
    //  CLASS XA_AdcCalCoefficientLoad::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   GainCoeff_Addr;
        unsigned int   OffsetCoeff_Addr;
    };

    XA_AdcCalCoefficientLoad(IRequires_AdcCalParts * mdc, IAddressingSpace & space, MapRegisters & regs, unsigned int MaxAdcs );
    virtual ~XA_AdcCalCoefficientLoad();

    virtual void    SetCalibrationOffset(unsigned int ch, unsigned int value);
    virtual void    SetCalibrationGain(unsigned int ch, unsigned int value);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_AdcCalParts *  MDC;
    
    RegisterArray<Register>    GainCoeffArray;
    RegisterArray<Register>    OffsetCoeffArray;

    unsigned int                FIncomingAdcs;
    ICalibrationValueOverride * CalOverride;

};
*/

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
