//---------------------------------------------------------------------------
// XASubDevice_AnalogOut_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XASubDevice_AnalogOut_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XAAFE_DacCommon -- X6 AFE Dac Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAAFE_DacCommon
//---------------------------------------------------------------------------

namespace {

    double ii_power_of_two(int exponent)
    {
        const int max_exponent = 30;
        double result = 1.0;
        while (exponent > 0)
            {
            int shift;
            //  we can't shift by more than 31 bits accurately, so break up
            //    into groups of max_exponent
            if (exponent > max_exponent)
                shift = max_exponent;
            else
                shift = exponent;
            unsigned int partial = 1<<shift;
            result *= static_cast<double>(partial);

            exponent -= shift;
            }
        return result;
    }
}

const unsigned int MaxXADacChannels = 32;

//---------------------------------------------------------------------------
//  constructor for class XAAFE_DacCommon
//---------------------------------------------------------------------------

XAAFE_DacCommon::XAAFE_DacCommon(IRequires_XACommonParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       DacEnable_Reg(space, regs.AfeDacEnable_Addr),
       DacPower_Reg(space, regs.AfeDacPower_Addr),
       DacTriggerCfgReg(space, regs.AfeDacTriggerCfg_Addr),
       DacDecimationReg(space, regs.AfeDacDecimation_Addr),
       DacPhaseIncrReg(space, regs.AfeDacPhaseIncr_Addr),
       DacResetReg(space, regs.AfeDacReset_Addr),
       FDevices(0), FChannelsPerDevice(0), FPhysDevices(0), FChannelsPerPhysDevice(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAAFE_DacCommon
//---------------------------------------------------------------------------

XAAFE_DacCommon::~XAAFE_DacCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAAFE_DacCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_XACommonParts::devDac);
    MDC->GetAnalogDeviceInfo(IRequires_XACommonParts::devDac, FDevices, FChannelsPerDevice);
    MDC->GetAnalogPhysicalDeviceInfo(IRequires_XACommonParts::devDac, FPhysDevices, FChannelsPerPhysDevice);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::OpenHardware()
//---------------------------------------------------------------------------

void  XAAFE_DacCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAAFE_DacCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void XAAFE_DacCommon::ChannelDisableAll()
{
    DacEnable_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  XAAFE_DacCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<MaxXADacChannels; i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void XAAFE_DacCommon::ChannelEnabled(int ch, bool state)
{
    if (ch<32)
        DacEnable_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::ChannelEnabled(int ch) const
{
    if (ch<32)
        return DacEnable_Reg.Bit(ch);
    else
        return false;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void XAAFE_DacCommon::ChannelPowerDownAll()
{
    DacPower_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void XAAFE_DacCommon::ChannelPowered(int ch, bool state)
{
    if (ch<32)
        DacPower_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::ChannelPowered(int ch) const
{
    if (ch<32)
        return DacPower_Reg.Bit(ch);
    else
        return false;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DeviceIsEnabled() -- Enable an I/O device
//---------------------------------------------------------------------------

void XAAFE_DacCommon::DeviceIsEnabled(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DeviceIsEnabled() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::DeviceIsEnabled(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelEnabled(base);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DevicePowered() -- Enable an I/O device
//---------------------------------------------------------------------------

void XAAFE_DacCommon::DevicePowered(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DevicePowered() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::DevicePowered(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelPowered(base);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ActiveDevices() -- return count of active devices
//---------------------------------------------------------------------------

unsigned int  XAAFE_DacCommon::ActiveDevices()
{
    unsigned int active_devices(0);
    for (unsigned int i=0; i<Devices(); i++)
        if (DeviceIsEnabled(i)==true)
            active_devices++;

    return active_devices;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void XAAFE_DacCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    DacTriggerCfgReg.DacFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int XAAFE_DacCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = DacTriggerCfgReg.DacFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XAAFE_DacCommon::EdgeTrigger(bool state)
{
    DacTriggerCfgReg.DacRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::EdgeTrigger() const
{
    return DacTriggerCfgReg.DacRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XAAFE_DacCommon::ExternalTrigger(bool state)
{
    DacTriggerCfgReg.DacExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::ExternalTrigger() const
{
    return DacTriggerCfgReg.DacExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void XAAFE_DacCommon::FramedTriggerEnabled(bool state)
{
    DacTriggerCfgReg.DacFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool XAAFE_DacCommon::FramedTriggerEnabled() const
{
    return DacTriggerCfgReg.DacFramedMode.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::Decimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void XAAFE_DacCommon::Decimation(unsigned int samples)
{
    DacDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::Decimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int XAAFE_DacCommon::Decimation() const
{
    return DacDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DacTestToneFrequency() -- Set Reset tone_freq specified I/O channel
//---------------------------------------------------------------------------

void XAAFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, double tone_freq, int factor)
{
    Debug.Log(" XAAFE_DacCommon::DacTestToneFrequency");
    Debug.Log("Test Frequency", tone_freq);
    Debug.Log("Dac Frequency", dac_clock_freq);
    Debug.Log("Factor", factor);

    double Two32 = ii_power_of_two(32);

    double phase_incr = tone_freq * (Two32) / dac_clock_freq / factor ;
    Debug.Log("Phase Incr", phase_incr);

    DacPhaseIncrReg.Value(static_cast<unsigned int>(phase_incr));
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DacTestToneFrequency() -- Query Reset tone_freq of an I/O channel
//---------------------------------------------------------------------------

double XAAFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, int factor) const
{
    double Two32 = ii_power_of_two(32);
    unsigned int raw_phase_incr = DacPhaseIncrReg.Value();

    double tone_freq = static_cast<double>(raw_phase_incr) / (Two32) * dac_clock_freq * factor;
    return tone_freq;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DacReset() -- Reset DACs
//---------------------------------------------------------------------------

void XAAFE_DacCommon::DacReset(bool state)
{
    DacResetReg.Bit(0, state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DRR_Bit() -- write DRR register bit (board specific)
//---------------------------------------------------------------------------

void  XAAFE_DacCommon::DRR_Bit(unsigned int bit, bool state)
{
    DacResetReg.Bit(bit, state);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::DRR_Bit() -- read DRR register bit (board specific)
//---------------------------------------------------------------------------

bool  XAAFE_DacCommon::DRR_Bit(unsigned int bit)
{
    return DacResetReg.Bit(bit);
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::PhysicalDeviceEnabled() --
//---------------------------------------------------------------------------
//  Physical device should be enabled if any channel on it is enabled...

bool XAAFE_DacCommon::PhysicalDeviceEnabled(unsigned int phys_dev) const
{
    bool enabled(false);
    if (phys_dev>=FPhysDevices)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int end   = start + FChannelsPerPhysDevice;

    for (unsigned int ch=start;  ch<end; ch++)
        enabled |= ChannelEnabled(ch);
    return enabled;
}

//---------------------------------------------------------------------------
//  XAAFE_DacCommon::PhysicalChannelEnabled() --
//---------------------------------------------------------------------------
//  return enabled state for the Nth channel on device D...

bool XAAFE_DacCommon::PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const
{
    if (phys_dev>=FPhysDevices || device_ch>=FChannelsPerPhysDevice)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int ch  = start + device_ch;

    return ChannelEnabled(ch);
}

//===========================================================================
//  CLASS XAAFE_DacSpi -- XA AFE DAC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAAFE_DacSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XAAFE_DacSpi
//---------------------------------------------------------------------------

XAAFE_DacSpi::XAAFE_DacSpi(IRequires_MuxedDacSpi * mdc, IAddressingSpace & space, XASpiDevice::MapRegisters & regs)
    :  MDC(mdc), SpiDev(space, regs, "XA Dac"), SpiCount(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XAAFE_DacSpi
//---------------------------------------------------------------------------

XAAFE_DacSpi::~XAAFE_DacSpi()
{
}

//---------------------------------------------------------------------------
//  XAAFE_DacSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAAFE_DacSpi::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SpiCount = MDC->GetDacSpiDeviceCount();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  XAAFE_DacSpi::OpenHardware()
//---------------------------------------------------------------------------

void  XAAFE_DacSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_DacSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAAFE_DacSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAAFE_DacSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XAAFE_DacSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiCount)
        SpiDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XAAFE_DacSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XAAFE_DacSpi::Spi(unsigned int device, unsigned int address)
{
    if (device < SpiCount)
        return SpiDev.Spi(device, address);
    else
        return 0;
}

//---------------------------------------------------------------------------
//  XAAFE_DacSpi::SpiWriteAll() -- write to all SPI device
//---------------------------------------------------------------------------

void  XAAFE_DacSpi::SpiWriteAll(unsigned int address, unsigned int data)
{
    SpiDev.SpiWriteAll(address, data);
}


/*
//===========================================================================
//  CLASS XA_DacCalCoefficientLoad -- XA Dac Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class XA_DacCalCoefficientLoad
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_DacCalCoefficientLoad
//---------------------------------------------------------------------------

XA_DacCalCoefficientLoad::XA_DacCalCoefficientLoad(IRequires_DacCalParts * mdc,
                IAddressingSpace & space, MapRegisters & regs,
                unsigned int MaxDacs )
    :  MDC(mdc),
       GainCoeffArray(space, regs.GainCoeff_Addr, MaxDacs),
       OffsetCoeffArray(space, regs.OffsetCoeff_Addr, MaxDacs),
       FDacChannels(0), CalOverride(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_DacCalCoefficientLoad
//---------------------------------------------------------------------------

XA_DacCalCoefficientLoad::~XA_DacCalCoefficientLoad()
{
    if (CalOverride)
        delete CalOverride;
}

//---------------------------------------------------------------------------
//  XA_DacCalCoefficientLoad::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XA_DacCalCoefficientLoad::ConstructCustomParts()
{
    FDacChannels = MDC->GetCalibrationDacCount();
    CalOverride = MDC->GetDacCalibrationOverride();
}

//---------------------------------------------------------------------------
//  XA_DacCalCoefficientLoad::OpenHardware()
//---------------------------------------------------------------------------

void  XA_DacCalCoefficientLoad::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    for (unsigned int i=0; i<FDacChannels; i++)
        {
        SetCalibrationGain(i, 0x10000);
        SetCalibrationOffset(i, 0x000);
        }
}

//---------------------------------------------------------------------------
//  XA_DacCalCoefficientLoad::ConfigureHardware()
//---------------------------------------------------------------------------

void  XA_DacCalCoefficientLoad::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XA_DacCalCoefficientLoad::SetCalibrationOffset()
//---------------------------------------------------------------------------

void  XA_DacCalCoefficientLoad::SetCalibrationOffset(unsigned int ch, unsigned int value)
{
    Debug.Log("Dac Cal Offset=", value, 16);
    if (ch<FDacChannels)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideOffset(value);
            if (CalOverride->OverrideOffsetUpdate())
                {
                CalOverride->WriteOffset(ch, value);
                return;
                }
            }
        OffsetCoeffArray[ch].Value(value);
        }
}

//---------------------------------------------------------------------------
//  XA_DacCalCoefficientLoad::SetCalibrationGain()
//---------------------------------------------------------------------------

void  XA_DacCalCoefficientLoad::SetCalibrationGain(unsigned int ch, unsigned int value)
{
    Debug.Log("Dac Cal Gain=", value, 16);
    if (ch<FDacChannels)
        {
        if (CalOverride)
            {
            value = CalOverride->OverrideGain(value);
            if (CalOverride->OverrideGainUpdate())
                {
                CalOverride->WriteGain(ch, value);
                return;
                }
            }
        GainCoeffArray[ch].Value(value);
        }
}

*/
#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative

