//---------------------------------------------------------------------------
// XASubDevice_Misc_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.

#include <XASubDevice_Misc_Mb.h>
#include <SystemSupport_Mb.h>
#include <Magic_Mb.h>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XA_AFEPower -- XUDigital Io "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6DigitalIo
//---------------------------------------------------------------------------

XA_AFEPower::XA_AFEPower(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       Power(space, regs.PowerCtrl_Addr)
{

}

//---------------------------------------------------------------------------
//  destructor for class XA_AFEPower
//---------------------------------------------------------------------------

XA_AFEPower::~XA_AFEPower()
{
}

//---------------------------------------------------------------------------
//  XA_AFEPower::OpenHardware()
//---------------------------------------------------------------------------

void  XA_AFEPower::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XA_AFEPower::ConfigureHardware()
//---------------------------------------------------------------------------

void  XA_AFEPower::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}


//---------------------------------------------------------------------------
//  XA_AFEPower::Power_3p3vEnable() -- set power bit
//---------------------------------------------------------------------------

void  XA_AFEPower::Power_3p3vEnable(bool state)
{
    return Power.Pwr3_3v_Enable.Value(state);
}

//---------------------------------------------------------------------------
//  XA_AFEPower::Power_3p3vGood() -- read power good bit
//---------------------------------------------------------------------------

bool  XA_AFEPower::Power_3p3vGood()
{
    return Power.Pwr3_3v_Good.Value();
}

//---------------------------------------------------------------------------
//  XA_AFEPower::Power_PllGood() -- read pll power good bit
//---------------------------------------------------------------------------

bool  XA_AFEPower::Power_PllGood()
{
    return Power.PwrPll_Good.Value();
}

//==============================================================================
//  CLASS XAPromLayouts --
//==============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAPromLayouts
//---------------------------------------------------------------------------

XAPromLayouts  MicronMT28GU256_Layout = { 8, 16, 0x20000, 256}; //  Micron MT28GU256AAA1EGC-0SIT

//---------------------------------------------------------------------------
//  XAPromLayouts::LoadLayout() - load predefined ROM
//---------------------------------------------------------------------------

void  XAPromLayouts::LoadLayout(PredefinedLayouts layout)
{
    switch (layout)
        {
        default:
        case pdl_MicronMT28GU256:
            *this = MicronMT28GU256_Layout;
        }

}

//==============================================================================
//  CLASS XAProm -- XA ROM basic control
//==============================================================================
//---------------------------------------------------------------------------
//  constructor for class XAProm
//---------------------------------------------------------------------------
//  init-ed to values for Micron MT28GU256AAA1EGC-0SIT

XAProm::XAProm(IXAProm_HardwareIntf* owner)
    :  Owner(owner), FDeviceId(0),
      FPartitions(8),  FBlocksPerPartition(16),
      FBlockSize(0x20000), FProgramCacheSize(256),
      FHWProtected(false)
{

}

//---------------------------------------------------------------------------
//  XAProm::Initialize() -- init chip object
//---------------------------------------------------------------------------

void XAProm::Initialize()
{
    Trace  Debug;

    Debug.Log("enter XAProm::Initialize()"  );
    // Write to the ReadConfigRegister to enter Async mode
    RomCommand_Confirm(0x60, 0x8000, 0x3, 0x8000);
    //Flash_NotBusy_T_us(5000);

    // Cache ROM device ID
    FDeviceId = Read_RomDeviceID();
    // Read other stuff?

    SetBlockProtection();

    //FHWProtected = true;
    Debug.Log("leave XAProm::Initialize()"  );
}

//---------------------------------------------------------------------------
//  XAProm::SetBlockProtection() -- Turn on write protection for ROM
//---------------------------------------------------------------------------

void XAProm::SetBlockProtection()
{
    for (int i=0; i<TotalBlocks(); i++)
        {
        unsigned int addr = i*BlockSize();
        BlockLock(addr);
        }
}

//---------------------------------------------------------------------------
//  XAProm::ReleaseBlockProtection() -- Turn off write protection for ROM
//---------------------------------------------------------------------------

void XAProm::ReleaseBlockProtection()
{
    //FHWProtected = false;
    for (int i=0; i<TotalBlocks(); i++)
        {
        unsigned int addr = i*BlockSize();
        BlockUnlock(addr);

        if (IsBlockLocked(addr))
            {
            //FHWProtected = true;
            std::stringstream msg;
            msg << std::hex << " block still locked: " << addr;
            Owner->TraceDebug(msg.str());
            }
        }
}

//---------------------------------------------------------------------------
//  XAProm::ReadMode_Array() -- switch to read mode
//---------------------------------------------------------------------------

void XAProm::ReadMode_Array(unsigned int partition_addr)
{
    RomCommand(0xFF, partition_addr);
}

//---------------------------------------------------------------------------
//  XAProm::ReadMode_StatusReg() -- switch to status register read mode
//---------------------------------------------------------------------------

void XAProm::ReadMode_StatusReg()
{
    RomCommand(0x70, 0);
}

//---------------------------------------------------------------------------
//  XAProm::ReadMode_ID() -- switch to chip ID read mode
//---------------------------------------------------------------------------

void XAProm::ReadMode_ID()
{
    RomCommand(0x90, 0);
}

//---------------------------------------------------------------------------
//  XAProm::ReadMode_CFI() -- switch to chip ID read mode
//---------------------------------------------------------------------------

void XAProm::ReadMode_CFI()
{
    RomCommand(0x98, 0);
}

//---------------------------------------------------------------------------
//  XAProm::Read_RomDeviceID() -- Read ROM device ID
//---------------------------------------------------------------------------

unsigned int XAProm::Read_RomDeviceID()
{
    // Enter READ_ID mode
    ReadMode_ID();
    unsigned short device_id = RomCommand_Read(0x1);  // Read Manufacturer ID & Chip ID;
    // Enter normal mode
    ReadMode_Array(0);

    return device_id;
}

//---------------------------------------------------------------------------
//  XAProm::Read_StatusRegister() -- Read Status Register Information
//---------------------------------------------------------------------------
//  If not already in status register mode, use ReadMode_StatusReg() to enable it first

unsigned int XAProm::Read_StatusRegister()
{
    unsigned int rdsr = RomCommand_Read(0x0);

//    if (rdsr != 0 && rdsr != 0x80)  // print only bad status registers?
//        {
//        std::stringstream msg;
//        msg << std::hex << " Read_StatusRegister(): " << rdsr;
//        Owner->TraceDebug(msg.str());
//        }

    return rdsr;
}

//---------------------------------------------------------------------------
//  XAProm::Clear_StatusRegister() -- Clear ROM Status Register
//---------------------------------------------------------------------------

void  XAProm::Clear_StatusRegister()
{
    RomCommand(0x50, 0x0);
}

//---------------------------------------------------------------------------
//  XAProm::BlockLock() -- Software Protect block from change
//---------------------------------------------------------------------------

void XAProm::BlockLock(unsigned int byte_address, unsigned int max_wait_us)
{
//    std::stringstream msg;
//    msg << std::hex << " Block Lock @ " << byte_address ;
//    Owner->TraceDebug(msg.str());

    RomCommand_Confirm(0x60, byte_address, 0x1, byte_address, max_wait_us);
    unsigned int rdsr = Read_StatusRegister();
    if (rdsr&0x20)
        {
        Clear_StatusRegister();
//        std::stringstream msg;
//        msg << std::hex << " Block Lock Error @ " << byte_address << " : sr = " << rdsr;
//        Owner->TraceDebug(msg.str());
        }

}

//---------------------------------------------------------------------------
//  XAProm::BlockUnlock() -- Software release protection
//---------------------------------------------------------------------------

void XAProm::BlockUnlock(unsigned int byte_address, unsigned int max_wait_us)
{
//    std::stringstream msg;
//    msg << std::hex << " Block Unlock @ " << byte_address ;
//    Owner->TraceDebug(msg.str());
    RomCommand_Confirm(0x60, byte_address, 0xD0, byte_address, max_wait_us);
    ReadMode_StatusReg();
    unsigned int rdsr = Read_StatusRegister();
    if (rdsr&0x20)
        {
        Clear_StatusRegister();
        std::stringstream msg;
        msg << std::hex << " Block Unlock Error @ " << byte_address << " : sr = " << rdsr;
        Owner->TraceDebug(msg.str());
        }
}

//---------------------------------------------------------------------------
//  XAProm::IsBlockLocked() -- Hardware Protection Checker
//---------------------------------------------------------------------------

bool XAProm::IsBlockLocked(unsigned int byte_address)
{
    // Enter READ_ID mode
    ReadMode_ID();
    unsigned short lock_info  = RomCommand_Read(byte_address + 0x2);
    // Enter normal mode
    ReadMode_Array(byte_address);

    bool locked = (lock_info & 0x3) != 0;
    return locked;
}

//---------------------------------------------------------------------------
//  XAProm::IsBlockLockedDown() -- Hardware Protection Checker
//---------------------------------------------------------------------------

bool XAProm::IsBlockLockedDown(unsigned int byte_address)
{
    // Enter READ_ID mode
    ReadMode_ID();
    unsigned short lock_info  = RomCommand_Read(byte_address + 0x2);
    // Enter normal mode
    ReadMode_Array(byte_address);

    bool locked = (lock_info & 0x2) != 0;
    return locked;
}

//---------------------------------------------------------------------------
//  XAProm::EraseBlock() -- erase ROM block
//---------------------------------------------------------------------------

bool XAProm::EraseBlock(unsigned int block, unsigned int max_wait_us, unsigned int max_quick_wait_us)
{

    unsigned int sector_addr = SectorAddress(block);
    RomCommand_Confirm(0x20, sector_addr, 0xD0, sector_addr, max_quick_wait_us);
    Flash_NotBusy_T_us(max_wait_us);

    unsigned int rdsr = Read_StatusRegister();
    Clear_StatusRegister();
    if (rdsr&0x20)
        return false;  // erase failed
    else
        return true;   // erase succeeded
}

//---------------------------------------------------------------------------
//  XAProm::BlankCheck() -- verify erase
//---------------------------------------------------------------------------

bool  XAProm::BlankCheck(unsigned int block, unsigned int max_wait_us)
{
//    std::stringstream msg;
//    msg << std::hex << " Block Blank Check @ " << block ;
//    Owner->TraceDebug(msg.str());

    unsigned int sector_addr = SectorAddress(block);
    RomCommand_Confirm(0xBC, sector_addr, 0xD0, sector_addr, max_wait_us);
    Flash_NotBusy_T_us(max_wait_us);

    unsigned int rdsr = Read_StatusRegister();
    Clear_StatusRegister();
    if (rdsr&0x20)
        return false;  // not blank
    else
        return true;   // blank

}

//---------------------------------------------------------------------------
//  XAProm::RomCommand() -- Simple ROM command
//---------------------------------------------------------------------------

void XAProm::RomCommand(unsigned short opcode, unsigned int setup_data, unsigned int max_wait_us)
{
    // Wait for ready
    Flash_Ready_T(max_wait_us);

    Owner->SetAddress(setup_data);
    Owner->SetCommand(opcode, IXAProm_HardwareIntf::cmOpWrite);

    // flash now busy...
}

//---------------------------------------------------------------------------
//  XAProm::RomCommand_Read() -- Simple ROM command, reads data
//---------------------------------------------------------------------------

unsigned int  XAProm::RomCommand_Read(unsigned int setup_data, unsigned int max_wait_us)
{
    // Wait for ready
    Flash_Ready_T(max_wait_us);

    Owner->SetAddress(setup_data);
    Owner->SetCommand(0, IXAProm_HardwareIntf::cmRead);  // opcode not used

    // flash now busy...wait for data

    Flash_ReadValid();

    unsigned int data = Owner->ReadData();
    return data;
}

//---------------------------------------------------------------------------
//  XAProm::RomCommand_Confirm() -- Two-command command rules
//---------------------------------------------------------------------------

void XAProm::RomCommand_Confirm(unsigned short setup_opcode, unsigned int setup_data,
                                unsigned short confirm_opcode, unsigned int confirm_data,
                                unsigned int max_wait_us)
{
    RomCommand(setup_opcode, setup_data, max_wait_us);
    RomCommand(confirm_opcode, confirm_data, max_wait_us);
}

//---------------------------------------------------------------------------
//  XAProm::Rom_BufferedProgram() -- Program a Buffer
//---------------------------------------------------------------------------

void XAProm::Rom_BufferedProgram(unsigned int rom_addr, IDatagram& Image_DG, unsigned int image_ofs,
                                 unsigned int word_count,  bool reorder_data, unsigned int max_wait_us)
{

    // SETUP
    RomCommand(0xE9, rom_addr, max_wait_us);

    unsigned int short_count = word_count<<1;
    RomCommand(short_count-1, rom_addr, max_wait_us);

    // Load Data
    for (unsigned int ofs=0;  ofs<word_count; ofs++)
        {
        unsigned int data_word = static_cast<unsigned int>(Image_DG.AsInt(image_ofs + ofs));
        if (reorder_data)
            data_word = ByteOrder(data_word);
        Owner->WriteData(data_word);
        }

    // PROGRAM
    Owner->SetCommand(0, IXAProm_HardwareIntf::cmProgram);  // opcode not used

    // CONFIRM
    RomCommand(0xD0, rom_addr, max_wait_us);

    // Wait for completion
    Flash_NotBusy_T_us(max_wait_us);
}

//---------------------------------------------------------------------------
//  XAProm::Rom_Read32() -- Read full 32 bit word starting at chip_address
//---------------------------------------------------------------------------
//  Chip address is "short int" address, so two accesses needed.

unsigned int XAProm::Rom_Read32(unsigned int chip_address)
{
    unsigned int value_lo = RomCommand_Read(chip_address);
    unsigned int value_hi = RomCommand_Read(chip_address+1);
    unsigned int value = value_hi<<16 | value_lo;
    return value;
}


//---------------------------------------------------------------------------
//  XAProm::Flash_Ready_T() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  XAProm::Flash_Ready_T(unsigned int max_wait_us)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to "max_wait_us" uS for pending flash to be ready
    for (unsigned int i = 0; i < reps; ++i)
        {
        bool ready = Owner->ReadReady();
        if (ready)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << " XAProm Ready Failure (>" << max_wait_us << "us)";
    Owner->TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  XAProm::Flash_ReadValid() -- wait for flash readvalid indicator
//---------------------------------------------------------------------------

void  XAProm::Flash_ReadValid()
{
    const unsigned int max_wait_us = 5000;  // Wait up to 5 uS

    const unsigned int reps = 20;
    const unsigned int delay = max_wait_us/reps;

    for (unsigned int i = 0; i < reps; ++i)
        {
        bool readvalid = Owner->ReadDataValid();
        if (readvalid)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << " XAProm Read Valid Failure (>" << max_wait_us << " us)";
    Owner->TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  XAProm::Flash_NotBusy_T_us() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  XAProm::Flash_NotBusy_T_us(unsigned int max_wait_us)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to  "max_wait_us" uS for Rdsr bit 7 to get set
    ReadMode_StatusReg();
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Read_StatusRegister();
        if ((rdsr&0x80)==0x80)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << " FlashRom Not Busy Failure (>" << max_wait_us << "us)";
    Owner->TraceDebug(msg.str());

}

//------------------------------------------------------------------------------
//  XAProm::ByteOrder() -- Reorder Bytes between image and chip
//------------------------------------------------------------------------------

unsigned int  XAProm::ByteOrder(unsigned int data)
{
////     this is a 32 bit image. Assume Hi and Lo short are correct, swap
////       the rest
    unsigned short * asShort = reinterpret_cast<unsigned short *>(&data);
    asShort[0] = ByteSwap(asShort[0]);
    asShort[1] = ByteSwap(asShort[1]);
//    //  then bit reverse what is left
//    //return BitReverse(data);
    return data;
}

//==============================================================================
//  CLASS XAFpaPersonality -- Interface to FPA specific operations
//==============================================================================
//---------------------------------------------------------------------------
//  XAFpaPersonality::Initialize() -- Initialize Personality
//---------------------------------------------------------------------------

bool  XAFpaPersonality::Initialize()
{
    Chip->Initialize();
    return true;
}

//------------------------------------------------------------------------------
//  XAFpaPersonality::EraseSector() - Erase specified flash sector contents
//------------------------------------------------------------------------------

bool XAFpaPersonality::EraseSector(int sector)
{
    Chip->BlockUnlock(Chip->SectorAddress(sector));
    if (Chip->IsBlockLocked(Chip->SectorAddress(sector))==true)
        {
        std::stringstream msg;
        msg << "ROM is write protected @ sector " << sector;
        ProcessStatusEvent e(msg.str());
        OnLoadError.Execute(e);
        return false;
        }

    bool erased = Chip->EraseBlock(sector);

    //bool erased_chked = Chip->BlankCheck(sector);

    Chip->BlockLock(Chip->SectorAddress(sector));

    return erased;
    //return erased && erased_chked;
}

//------------------------------------------------------------------------------
//  XAFpaPersonality::Program() -- Write buffer to flash
//------------------------------------------------------------------------------

bool  XAFpaPersonality::Program(FpgaImage & Image)
{
    Percent[0] = Percent[1] = 0;
    const unsigned int BlockWriteSize = Chip->ProgramCacheSize();

    FpgaImageDatagram Image_DG(Image);

    Chip->ReleaseBlockProtection();

    Chip->ReadMode_Array(0);  // begin at offset 0
    unsigned int curPartition = 0;

    // Program device
    for (unsigned int i = 0; i < Image_DG.SizeInInts(); i+=BlockWriteSize)
        {
        bool ReorderData(true);
        unsigned int chip_address = i<<1;  // chip address is in "16 bit" elements

        unsigned newPartition = Chip->PartitionID(chip_address);
        if (newPartition != curPartition)
            {
            Chip->ReadMode_Array(chip_address);  // must re-do RM Array for each partition
            curPartition = newPartition;
            }

        Chip->Rom_BufferedProgram(chip_address, Image_DG, i, BlockWriteSize, ReorderData /* , max_wait_us */);
        //
        //
        //  Progress Update
        Percent[1] = static_cast<int>((i*100)/Image_DG.SizeInInts());
        if (Percent[1] != Percent[0])
            {
            ProcessProgressEvent e(Percent[1]);
            OnLoadProgress.Execute(e);
            Percent[0] = Percent[1];
            }
        }

    ProcessProgressEvent e(100);
    OnLoadProgress.Execute(e);

    Chip->SetBlockProtection();

    return true;
}

//------------------------------------------------------------------------------
//  XAFpaPersonality::Verify() -- Verify buffer against flash contents
//------------------------------------------------------------------------------

bool XAFpaPersonality::Verify(FpgaImage & Image)
{
    Percent[0] = Percent[1] = 0;

    Chip->ReadMode_Array(0);  // begin at offset 0
    unsigned int curPartition = 0;

    FpgaImageDatagram Image_DG(Image);
    // Program device
    for (unsigned int i = 0; i < Image_DG.SizeInInts(); ++i)
        {
        unsigned int chip_address = i<<1;  // chip in 16 bit elements
        unsigned newPartition = Chip->PartitionID(chip_address);
        if (newPartition != curPartition)
            {
            Chip->ReadMode_Array(chip_address);  // must re-do RM Array for each partition
            curPartition = newPartition;
            }

        unsigned int dst = Chip->Rom_Read32(chip_address);
        unsigned int src = static_cast<unsigned int>(Image_DG.AsInt(i));
        if (Chip->ByteOrder(dst) != src)
            {
            std::ostringstream msg;

            msg << "Prom read 0x" << std::hex << dst << ", " << Chip->ByteOrder(dst)
                << " instead of 0x" << src
                << " at address 0x" << i;
            ProcessStatusEvent e(msg.str());
            OnLoadError.Execute(e);
            return false;
            }

        Percent[1] = static_cast<int>((i*100)/Image_DG.SizeInInts());
        if (Percent[1] != Percent[0])
            {
            ProcessProgressEvent e(Percent[1]);
            OnLoadProgress.Execute(e);
            Percent[0] = Percent[1];
            }
        }

    ProcessProgressEvent p(100);
    OnLoadProgress.Execute(p);

    return true;
}

//---------------------------------------------------------------------------
//  XAFpaPersonality::Read() -- Fill buffer from flash
//---------------------------------------------------------------------------

bool XAFpaPersonality::Read(FpgaImage & Image)
{
    Percent[0] = Percent[1] = 0;

    Chip->ReadMode_Array(0);  // begin at offset 0
    unsigned int curPartition = 0;

    FpgaImageDatagram Image_DG(Image);
    // Read device
    for (unsigned int i = 0; i < Image_DG.SizeInInts(); ++i)
        {
        unsigned int chip_address = i<<1;  // chip in 16 bit elements
        unsigned newPartition = Chip->PartitionID(chip_address);
        if (newPartition != curPartition)
            {
            Chip->ReadMode_Array(chip_address);  // must re-do RM Array for each partition
            curPartition = newPartition;
            }

        Image_DG.AsInt(i, Chip->ByteOrder(Chip->Rom_Read32(chip_address)));

        Percent[1] = static_cast<int>((i*100)/Image_DG.SizeInInts());
        if (Percent[1] != Percent[0])
            {
            ProcessProgressEvent e(Percent[1]);
            OnLoadProgress.Execute(e);
            Percent[0] = Percent[1];
            }
        }

    ProcessProgressEvent p(100);
    OnLoadProgress.Execute(p);
    return true;

}

//-------------------------------------------------------------
// XAFpaPersonality::Authenticate() - Verify board integrity
//-------------------------------------------------------------

bool XAFpaPersonality::Authenticate()
{
    return true;
}

//===========================================================================
//  CLASS XAFpaLoader -- XA FPA Loader "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XAFpaLoader
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XAFpaLoader
//---------------------------------------------------------------------------

XAFpaLoader::XAFpaLoader(IRequires_XARom * mdc,
                           IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AddressReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr),
       StatusReg(space, regs.Opcode_Addr),
       RawProm(this),
       Thread( new XAFpaPersonality( &RawProm ))
{
    Thread.SetFullEraseMode(false);  // only erase Image Size
}

//---------------------------------------------------------------------------
//  destructor for class XAFpaLoader
//---------------------------------------------------------------------------

XAFpaLoader::~XAFpaLoader()
{
    Thread.Shutdown();

    //delete Chip;
}

//---------------------------------------------------------------------------
//  XAFpaLoader::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XAFpaLoader::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XAFpaLoader::OpenHardware()
//---------------------------------------------------------------------------

void  XAFpaLoader::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Thread.OnOpen();

    Raw().Initialize();
}

//---------------------------------------------------------------------------
//  XAFpaLoader::ConfigureHardware()
//---------------------------------------------------------------------------

void  XAFpaLoader::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XAFpaLoader::ReadReady() -- Return ready bit from hardware
//---------------------------------------------------------------------------

bool XAFpaLoader::ReadReady()
{
    bool ready = StatusReg.Ready.Value();
    return ready;
}

//---------------------------------------------------------------------------
//  XAFpaLoader::ReadDataValid() -- logic signals data valid
//---------------------------------------------------------------------------

bool  XAFpaLoader::ReadDataValid()
{
    bool readvalid = StatusReg.ReadDataValid.Value();
    return readvalid;
}

//---------------------------------------------------------------------------
//  XAFpaLoader::SetAddress() -- Load address register
//---------------------------------------------------------------------------

void XAFpaLoader::SetAddress(unsigned int address)
{
    AddressReg.Addr.Value(address);
}

//---------------------------------------------------------------------------
//  XAFpaLoader::SetCommand() -- Load opcode and command type
//---------------------------------------------------------------------------

void XAFpaLoader::SetCommand(unsigned short opcode, IXAProm_HardwareIntf::CommandModes mode)
{
    OpcodeReg.Opcode.Value(opcode);
    OpcodeReg.CommandClass.Value(mode);
    OpcodeReg.Apply();
}

//---------------------------------------------------------------------------
//  XAFpaLoader::ReadData() -- read data register
//---------------------------------------------------------------------------

unsigned int XAFpaLoader::ReadData()
{
    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  XAFpaLoader::WriteData() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XAFpaLoader::WriteData(unsigned int data)
{
    DataReg.Value(data);
}

//---------------------------------------------------------------------------
//  XAFpaLoader::TraceDebug() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XAFpaLoader::TraceDebug(const std::string & text)
{
    Debug.Log(text);
}

}  // namespace Innovative



