// XASubDevice_Misc_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef XASubDevice_Misc_MbH
#define XASubDevice_Misc_MbH

#include <X6Support_Mb.h>
//#include <VpxSupport_Mb.h>
//#include <HardwareRegister_Mb.h>
#include <FpaLoadThread_Mb.h>
//#include <DioPortIntf_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif


//===========================================================================
//  CLASS XA_AFEPower -- XA AFE Power "SubDevice"
//===========================================================================

class XA_AFEPower : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS XA_AFEPower::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   PowerCtrl_Addr;
    };

    XA_AFEPower(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XA_AFEPower();

    void  Power_3p3vEnable(bool state);

    bool  Power_3p3vGood();
    bool  Power_PllGood();


    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_AFEPower::PowerCtlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class PowerCtlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        Pwr3_3v_Enable;
        RegisterBit        Pwr3_3v_Good;
        RegisterBit        PwrPll_Good;

        PowerCtlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Pwr3_3v_Enable(*this, 0), Pwr3_3v_Good(*this, 8),
              PwrPll_Good(*this, 9)
        {  }
    };

private:
    IRequires_Nothing *  MDC;
    PowerCtlRegister  Power;

};


//==============================================================================
//  CLASS IXAProm_HardwareIntf -- Hardware interface for XA_Prom
//==============================================================================

class IXAProm_HardwareIntf
{
public:
    enum CommandModes { cmOpWrite=0x1, cmProgram=0x2, cmRead=0x4 };
    virtual ~IXAProm_HardwareIntf()
        {}

    virtual bool  ReadReady() = 0;
    virtual bool  ReadDataValid() = 0;

    virtual void  SetAddress(unsigned int address) = 0;
    virtual void  SetCommand(unsigned short opcode, CommandModes mode) = 0;

    virtual unsigned int ReadData() = 0;
    virtual void         WriteData(unsigned int data) = 0;

    virtual void  TraceDebug(const std::string & text) = 0;
};

//==============================================================================
//  CLASS XAPromLayouts --
//==============================================================================

struct XAPromLayouts
{
    int Partitions;
    int BlocksPerPartition;
    int BlockSize;
    int ProgramCacheSize;

    enum PredefinedLayouts { pdl_MicronMT28GU256 };

    void  LoadLayout(PredefinedLayouts layout);
};

//==============================================================================
//  CLASS XAProm -- XA ROM basic control
//==============================================================================

class XAProm
{
public:
    XAProm(IXAProm_HardwareIntf * owner);
    virtual ~XAProm()
        {}

    virtual void  Initialize();

    void  SetPromLayoutInfo(int partitions, int blocks_per_partition, int block_size, int program_cache_size)
        {
        FPartitions = partitions;  FBlocksPerPartition = blocks_per_partition;
        FBlockSize=block_size;     FProgramCacheSize = program_cache_size;
        }

    //
    //  Information
    int   DeviceId()
        {  return FDeviceId;  }
    int   Partitions()
        {  return FPartitions;  }
    int   BlocksPerPartition()
        {  return FBlocksPerPartition;  }
    int   BlockSize()
        {  return FBlockSize;  }
    int   ProgramCacheSize()
        {  return FProgramCacheSize;  }
    int   TotalBlocks()
        {  return FPartitions*FBlocksPerPartition;  }
//    int   BlocksInUse()
//        {  return (FLimitedRomSize) ?  FBlocksInUse :  TotalBlocks();  }

    // Rom Access
    void  SetBlockProtection();
    void  ReleaseBlockProtection();
    bool  IsHWProtected() const
       {  return  FHWProtected;  }

    void  ReadMode_Array(unsigned int partition_addr);
    void  ReadMode_StatusReg();
    void  ReadMode_ID();
    void  ReadMode_CFI();

    unsigned int  Read_RomDeviceID();
    unsigned int  Read_StatusRegister();
    void Clear_StatusRegister();

    void  BlockLock(unsigned int byte_address, unsigned int max_wait_us=5000);
    void  BlockUnlock(unsigned int byte_address, unsigned int max_wait_us=5000);
    bool  IsBlockLocked(unsigned int byte_address);
    bool  IsBlockLockedDown(unsigned int byte_address);

    bool  EraseBlock(unsigned int block, unsigned int max_wait_us=5000000, unsigned int max_quick_wait_us=5000);
    bool  BlankCheck(unsigned int block, unsigned int max_wait_us=10000);

    // Primitives
    void          RomCommand(unsigned short opcode, unsigned int setup_data, unsigned int max_wait_us=5000);
    unsigned int  RomCommand_Read(unsigned int setup_data, unsigned int max_wait_us=5000);
    void          RomCommand_Confirm(unsigned short setup_opcode, unsigned int setup_data,
                                     unsigned short confirm_opcode, unsigned int confirm_data, unsigned int max_wait_us=5000);
    void          Rom_BufferedProgram(unsigned int rom_addr, IDatagram& Image_DG, unsigned int image_ofs, unsigned int word_count,
                                      bool reorder_data=false, unsigned int max_wait_us=5000);

    unsigned int  Rom_Read32(unsigned int chip_address);

    unsigned int  SectorAddress(unsigned int sector)
        {  return sector*BlockSize();   }
    unsigned int  PartitionID(unsigned int chip_address)
        {  return chip_address/FBlockSize/FBlocksPerPartition;  }

    unsigned int ByteOrder(unsigned int data);

protected:

    IXAProm_HardwareIntf * Owner;
    int                    FDeviceId;  // cached

    int   FPartitions;
    int   FBlocksPerPartition;
    int   FBlockSize;
    int   FProgramCacheSize;   // 32 bit words that can be programmed at once

    bool  FHWProtected;

private:
    void Flash_Ready_T(unsigned int max_wait_us);
    void Flash_ReadValid();
    void Flash_NotBusy_T_us(unsigned int max_wait_us);

    unsigned int ByteSwap(unsigned int data);
};

//------------------------------------------------------------------------------
// XAProm::ByteSwap() - Byte swap 16 bit word
//------------------------------------------------------------------------------

inline unsigned int  XAProm::ByteSwap(unsigned int data)
{
    unsigned int LSB = data & 0x00FF;
    unsigned int MSB = (data & 0xFF00) >> 8;
    return (LSB<<8) | MSB;
}

//==============================================================================
//  CLASS XAFpaPersonality -- Interface to FPA specific operations
//==============================================================================

class XAFpaPersonality : public IFpaPersonality
{
public:

    XAFpaPersonality(XAProm * chip)
        :  Chip(chip)
        {}
    virtual ~XAFpaPersonality()
        {}

    virtual int  FirmwareRevision() const
        {  return 0;  }
    virtual int  Id() const
        {  return Chip->DeviceId();  }

    //---------------------
    //  Personality Methods
    //
    //  ...Info
    virtual  int           DeviceId() const
                    {  return Chip->DeviceId();  }
    virtual  int           DeviceSize() const
                    {  return Sectors()*SectorSize();  }
    virtual  int           SectorSize() const
                    {  return Chip->BlockSize();  }
    virtual  unsigned int  Sectors() const
                    {  return Chip->Partitions()*Chip->BlocksPerPartition();  }
    virtual  unsigned int  StartSector() const
                    {  return 0;  }
    virtual  unsigned int  GoldenImageStartSector() const
                    {  return 0;  }
    virtual  unsigned int  GoldenImageSectors() const
                    {  return Sectors();  }

    virtual  unsigned int DataMask() const
                    {  return  0xFFFFFFFF;  }

    virtual  bool      IsProtected() const
                    {  return Chip->IsHWProtected();  }

    //  Are these needed?  X6 rewrites 'Program'
    virtual  bool      Flash(int /*addr*/, int /*value*/) {  return true;  }
    virtual  int       Flash(int /*addr*/) {  return 0;  }

    virtual  bool      Authenticate();
    virtual  bool      Initialize();
    virtual  bool      EraseSector(int sector);

    virtual  bool      Program(FpgaImage & Image);
    virtual  bool      Verify(FpgaImage & Image);
    virtual  bool      Read(FpgaImage & Image);

    virtual  bool      HasGoldenImage() {  return false;  }
    virtual  bool      GoldenImageBooted() {  return false;  }
    virtual  bool      GoldenImageActive()  {  return false;  }
    virtual  void      GoldenImageActive(bool /*state*/) {}
    virtual  int       CpldRevision() {  return DeviceId();  }
    virtual  int       CpldType() {  return 0;  }

private:
    XAProm *          Chip;
    StopWatch         Rtc;
    int               Percent[2];

};

//===========================================================================
//  CLASS XAFpaLoader -- XA FPA Loader "SubDevice"
//===========================================================================

typedef IRequires_Nothing IRequires_XARom;

class XAFpaLoader : public IX6SubDevice, public IXAProm_HardwareIntf
{
public:

    //enum RomType { romS25FL128S, romS25FL128S_Sys };

    //=======================================================================
    //  CLASS XAFpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================

    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    //
    //  Member Functions
    XAFpaLoader(IRequires_XARom * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XAFpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    XAProm &  Raw()
            {  return RawProm;  }

public:
    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAFpaLoader::AddressRegister  --  Flash Address
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;

        AddressRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAFpaLoader::OpcodeCmdRegister  --  Command Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class OpcodeCmdRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBitGroup   Opcode;
        RegisterBitGroup   CommandClass;

        OpcodeCmdRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Opcode(*this, 0, 16), CommandClass(*this, 16, 3)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XAFpaLoader::StatusRegister  --  Status Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class StatusRegister : public ReadOnlyRegister
    {
        typedef ReadOnlyRegister inherited;

    public:
        ReadOnlyRegisterBit        ReadDataValid;
        ReadOnlyRegisterBit        Ready;

        StatusRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };


private:
    IRequires_Nothing *     MDC;
    AddressRegister         AddressReg;
    Register                DataReg;
    OpcodeCmdRegister       OpcodeReg;
    StatusRegister          StatusReg;

    XAProm              RawProm;
    FpaLoadThread       Thread;
    StopWatch           Rtc;

protected:
    virtual bool          ReadReady();
    virtual bool          ReadDataValid();
    virtual void          SetAddress(unsigned int address);
    virtual void          SetCommand(unsigned short opcode, CommandModes mode);

    virtual unsigned int  ReadData();
    virtual void          WriteData(unsigned int data);
    virtual void          TraceDebug(const std::string & text);

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
