//---------------------------------------------------------------------------
// XASubDevice_Stream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2012 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XASubDevice_Stream_Mb.h>
#include <SystemSupport_Mb.h>
#include <FloatingPtSupport_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS XA_LMK20_Clock -- XA AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_LMK20_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_LMK20_Clock
//---------------------------------------------------------------------------

XA_LMK20_Clock::XA_LMK20_Clock(IRequires_Lmk20XAClocks * mdc,  IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       VcxoControlReg(space, regs.AfeVcxoControl_Addr),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_LMK20_Clock
//---------------------------------------------------------------------------

XA_LMK20_Clock::~XA_LMK20_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XA_LMK20_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  XA_LMK20_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // Enable power outputs
    VcxoControlReg.PowerEn_3p3.Value(true);
    VcxoControlReg.PowerEn_Vcxo.Value(true);

    PllDev->Powered(true, true);
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  XA_LMK20_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  XA_LMK20_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::Pll()
//---------------------------------------------------------------------------

Lmk04820 & XA_LMK20_Clock::Pll()
{
    return *PllDev;
}

//===========================================================================
//  CLASS XA_LMK20_Clock::XAClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  XA_LMK20_Clock::XAClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  XA_LMK20_Clock::XAClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  XA_LMK20_Clock::XAClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XA_LMK20_Clock::XAClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XA_LMK20_Clock::XAClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XA_LMK20_Clock::XAClockIo::Dac()
{
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IXAClockIo::IIClockSource XA_LMK20_Clock::XAClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void XA_LMK20_Clock::XAClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IXAClockIo::IIReferenceSource XA_LMK20_Clock::XAClockIo::Reference() const
{
    Lmk04820::LmkReferenceSource current_source = Owner->PllDev->ReferenceSource();
    return static_cast<IXAClockIo::IIReferenceSource>(Owner->PllDev->GetSourceMapping(current_source));
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void XA_LMK20_Clock::XAClockIo::Reference(IIReferenceSource src)
{
    Lmk04820::LmkReferenceSource new_source = static_cast<Lmk04820::LmkReferenceSource>(Owner->PllDev->GetSourceMapping_Reverse(src));
    if (Owner->PllDev->SourceInUse(new_source))
        Owner->PllDev->ReferenceSource(new_source);

}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void XA_LMK20_Clock::XAClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double XA_LMK20_Clock::XAClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  XA_LMK20_Clock::XAClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  XA_LMK20_Clock::XAClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}

//===========================================================================
//  CLASS XA_LMK_Clock -- XA AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_LMK_Clock
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_LMK_Clock
//---------------------------------------------------------------------------

XA_LMK_Clock::XA_LMK_Clock(IRequires_LmkXAClocks * mdc,  IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       VcxoControlReg(space, regs.AfeVcxoControl_Addr),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  ClockIo(this), ClockDevs(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_LMK_Clock
//---------------------------------------------------------------------------

XA_LMK_Clock::~XA_LMK_Clock()
{
    delete PllDev;     PllDev  = 0;
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XA_LMK_Clock::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev   = MDC->MakeFmcAfePllObject();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::OpenHardware()
//---------------------------------------------------------------------------

void  XA_LMK_Clock::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // Enable power outputs
    VcxoControlReg.PowerEn_3p3.Value(true);
    VcxoControlReg.PowerEn_Vcxo.Value(true);
    VcxoControlReg.PowerEn_Pll.Value(true);

    PllDev->Powered(true);  // add "at open" flag like 020?
    PllDev->Init();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::PreconfigureHardware()
//---------------------------------------------------------------------------

void  XA_LMK_Clock::PreconfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::ConfigureHardware()
//---------------------------------------------------------------------------

void  XA_LMK_Clock::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // Reapply PLL settings
    //PllDev->Reapply();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::Pll()
//---------------------------------------------------------------------------

Lmk04800 & XA_LMK_Clock::Pll()
{
    return *PllDev;
}

//===========================================================================
//  CLASS XA_LMK_Clock::XAClockIo -- X6 AFE Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  XA_LMK_Clock::XAClockIo::FrequencyActual() const
{
    return Owner->PllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  XA_LMK_Clock::XAClockIo::Frequency(double value)
{
    Owner->PllDev->Frequency(value);
    //Owner->PllDev->Reapply();    // apply this change at once.
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  XA_LMK_Clock::XAClockIo::Frequency() const
{
    return Owner->PllDev->Frequency();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::X6ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XA_LMK_Clock::XAClockIo::System()
{
    return Owner->PllDev->System();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::X6ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XA_LMK_Clock::XAClockIo::Adc()
{
    return Owner->PllDev->Adc();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::X6ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XA_LMK_Clock::XAClockIo::Dac()
{
    return Owner->PllDev->Dac();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IXAClockIo::IIClockSource XA_LMK_Clock::XAClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void XA_LMK_Clock::XAClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IXAClockIo::IIReferenceSource XA_LMK_Clock::XAClockIo::Reference() const
{
    Lmk04800::LmkReferenceSource current_source = Owner->PllDev->ReferenceSource();
    return static_cast<IXAClockIo::IIReferenceSource>(Owner->PllDev->GetSourceMapping(current_source));
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void XA_LMK_Clock::XAClockIo::Reference(IIReferenceSource src)
{
    Lmk04800::LmkReferenceSource new_source = static_cast<Lmk04800::LmkReferenceSource>(Owner->PllDev->GetSourceMapping_Reverse(src));
    if (Owner->PllDev->SourceInUse(new_source))
        Owner->PllDev->ReferenceSource(new_source);
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void XA_LMK_Clock::XAClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double XA_LMK_Clock::XAClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  XA_LMK_Clock::XAClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  XA_LMK_Clock::XAClockIo::Locked() const
{
    return Owner->PllDev->Locked();
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative

