// XASubDevice_Stream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef XASubDevice_Stream_MbH
#define XASubDevice_Stream_MbH

#include <X6Support_Mb.h>
#include <FmcSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <CDCE62005_Mb.h>
#include <LMK04800_Mb.h>
#include <LMK04820_Mb.h>
#include <PmcIdrom_Mb.h>
#include <SpiDevice_Mb.h>
#include <Ad9510Duo_Tb_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS IXAClockIo -- User interface to control XA clock functionality
//==============================================================================

class IXAClockIo
{
public:
    enum IIClockSource { csExternal, csInternal };   // clock routing
    enum IIReferenceSource { rsExternal, rsP16, rsInternal };
    enum IIClockSelect {  cslFrontPanel, cslCarrier };


    virtual ~IXAClockIo()  {  }

    //
    //  Interface Functions
    // Methods
    virtual double              FrequencyActual() const = 0;

    // Properties
    virtual void                Frequency(double value) = 0;
    virtual double              Frequency() const = 0;

    virtual ITimebaseRate &     Adc() = 0;
    virtual ITimebaseRate &     Dac() = 0;
    virtual ITimebaseRate &     System() = 0;

    virtual void                Source(IIClockSource src) = 0;
    virtual IIClockSource       Source() const = 0;

    virtual void                ExternalClkSelect(IIClockSelect /*src*/) {  }
    virtual IIClockSelect       ExternalClkSelect() const {  return cslFrontPanel;  }

    virtual void                Reference(IIReferenceSource /*src*/)  {  }
    virtual IIReferenceSource   Reference() const  {  return rsInternal;  }

    virtual void                ReferenceFrequency(double /*value*/)  {  }
    virtual double              ReferenceFrequency() const  {  return 0.0;  }

    virtual bool                Locked()  const = 0;

};

//===========================================================================
//  CLASS XA_LMK20_Clock -- XA AFE Clock "SubDevice"
//===========================================================================

class XA_LMK20_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_LMK20_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        unsigned int   AfeVcxoControl_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_LMK20_Clock::XAClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XAClockIo : public IXAClockIo
    {
    public:
        XAClockIo(XA_LMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~XAClockIo()
            {  }
        //
        //  IXAClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src) { Owner->CustomExternalClkSelect(src);  }
        virtual IIClockSelect       ExternalClkSelect() const {  return Owner->CustomExternalClkSelect();  }

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        XA_LMK20_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XALMK_Clock::XARawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XARawClockDevices : public ILmk20RawClockDevices
    {
    public:
        XARawClockDevices(XA_LMK20_Clock * owner): Owner(owner)
            {  }
        virtual ~XARawClockDevices()
            {  }

        virtual Lmk04820 &   Pll()      {  return Owner->Pll();  }

    protected:
        XA_LMK20_Clock *  Owner;
    };

    friend class XAClockIo;
    friend class XARawClockDevices;

    XA_LMK20_Clock(IRequires_Lmk20FmcClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XA_LMK20_Clock();

    XAClockIo &           Clock() {  return ClockIo;  }
    XARawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04820 &             Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_LMK20_Clock::VcxoControlRegister -- Vcxo and power stuff
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       PowerEn_3p3;
        RegisterBit       PowerEn_Vcxo;
        RegisterBit       PowerGood_3p3;
        RegisterBit       PowerGood_Vcxo;
        RegisterBit       PowerGood_Pll;

    public:
        VcxoControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               PowerEn_3p3(*this, 0),   PowerEn_Vcxo(*this, 4),
               PowerGood_3p3(*this, 8),   PowerGood_Vcxo(*this, 12),   PowerGood_Pll(*this, 15)
            {
            }
    };

protected:
    //  Override these if you need custom external clock src selection
    virtual void                       CustomExternalClkSelect(IXAClockIo::IIClockSelect /*src*/) {  }
    virtual IXAClockIo::IIClockSelect  CustomExternalClkSelect() const {  return IXAClockIo::cslFrontPanel;  }

private:
    //
    //  Registers
    IRequires_Lmk20XAClocks *  MDC;
    VcxoControlRegister       VcxoControlReg;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;

    Lmk04820 *         PllDev;
    XAClockIo          ClockIo;
    XARawClockDevices  ClockDevs;

};

//===========================================================================
//  CLASS XA_LMK_Clock -- XA AFE Clock "SubDevice" for LMK4800 series
//===========================================================================

class XA_LMK_Clock : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_LMK_Clock::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        unsigned int   AfeVcxoControl_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_LMK_Clock::XAClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XAClockIo : public IXAClockIo
    {
    public:
        XAClockIo(XA_LMK_Clock * owner): Owner(owner)
            {  }
        virtual ~XAClockIo()
            {  }
        //
        //  IXAClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                ExternalClkSelect(IIClockSelect src) { Owner->CustomExternalClkSelect(src);  }
        virtual IIClockSelect       ExternalClkSelect() const {  return Owner->CustomExternalClkSelect();  }

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        XA_LMK_Clock *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XALMK_Clock::XARawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class XARawClockDevices : public ILmkRawClockDevices
    {
    public:
        XARawClockDevices(XA_LMK_Clock * owner): Owner(owner)
            {  }
        virtual ~XARawClockDevices()
            {  }

        virtual Lmk04800 &   Pll()      {  return Owner->Pll();  }

    protected:
        XA_LMK_Clock *  Owner;
    };

    friend class XAClockIo;
    friend class XARawClockDevices;

    XA_LMK_Clock(IRequires_LmkXAClocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XA_LMK_Clock();

    XAClockIo &           Clock() {  return ClockIo;  }
    XARawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04800 &             Pll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_LMK_Clock::VcxoControlRegister -- Vcxo and power stuff
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VcxoControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       PowerEn_3p3;
        RegisterBit       PowerEn_Vcxo;
        RegisterBit       PowerEn_Pll;
        RegisterBit       PowerGood_3p3;
        RegisterBit       PowerGood_Vcxo;
        RegisterBit       PowerGood_Pll;

    public:
        VcxoControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               PowerEn_3p3(*this, 0),   PowerEn_Vcxo(*this, 4),
               PowerEn_Pll(*this, 7), PowerGood_3p3(*this, 8),
               PowerGood_Vcxo(*this, 12),   PowerGood_Pll(*this, 15)
            {
            }
    };

protected:
    //  Override these if you need custom external clock src selection
    virtual void                       CustomExternalClkSelect(IXAClockIo::IIClockSelect /*src*/) {  }
    virtual IXAClockIo::IIClockSelect  CustomExternalClkSelect() const {  return IXAClockIo::cslFrontPanel;  }

private:
    //
    //  Registers
    IRequires_LmkXAClocks *  MDC;
    VcxoControlRegister       VcxoControlReg;
    //Register         PllControl;     // Implemented inside LMK Pll
    //Register         PllSpi;

    Lmk04800 *         PllDev;
    XAClockIo          ClockIo;
    XARawClockDevices  ClockDevs;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative


#endif
