// XA_160DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XA_160DeviceMap_MbH
#define XA_160DeviceMap_MbH

#include <XA_160_Mb.h>
#include <XASubDevice_Stream_Mb.h>
#include <AdDac9122_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class XA_160DeviceMap;

//==============================================================================
//  CLASS XA_160_Pll -- XA_160 PLL Chip (Specialization of Lmk04820)
//==============================================================================

class XA_160_Pll : public Lmk04820
{
public:
    XA_160_Pll(XA_160DeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~XA_160_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 40e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XA_160DeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double /*dac_rate*/);

private:
    unsigned int  DacMode();
    double        DCLK(double dac_rate, unsigned int mode);
};


//==============================================================================
//  CLASS XA_160_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class XA_160_SioPll : public Si5338
{
public:
    XA_160_SioPll(XA_160DeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    XA_160DeviceMap * Owner;
};

//===========================================================================
//  CLASS ISuppliesXA_160Requirements -- Device Map Requirement List
//===========================================================================

class ISuppliesXA_160Requirements : public IRequires_Nothing,
                                    public IRequires_Lmk20FmcClocks
{
public:
    virtual ~ISuppliesXA_160Requirements()
        {}

};

//===========================================================================
//  CLASS XA_160DeviceMap  -- Register & low level support for XA_160 module
//===========================================================================
class XA_160;

class XA_160DeviceMap :  public XABidirDeviceMap, public ISuppliesXA_160Requirements

{
    friend class XA_160;
    friend class XA_160InputDevice;
    friend class XA_160OutputDevice;
//    friend class XA_160_Pll;

public:
    //
    //  Member Functions
    XA_160DeviceMap(XA_160 * owner);
    virtual ~XA_160DeviceMap();

    XA_LMK20_Clock::XAClockIo & Clock()  { return Clock_SDev.Clock(); }

    class XA_160InputDevice &  Input();
    class XA_160OutputDevice &  Output();



public: // pseudo-protected

    unsigned int      CurrentDacInterpolationMode()
        {  return FDacInterpolationMode;  }

//    virtual double   Get_DacClockRate()  { return Clock().Dac().Frequency(); }

protected:
    // Data
    XA_160 *         Owner;
    //
    //  Subdevices
    XA_LMK20_Clock        Clock_SDev;

    XA_160InputDevice *     FInput;
    XA_160OutputDevice *    FOutput;

    Trace             Debug;

    // Methods
//    virtual void    ConstructCustomParts();            // board construtor time
    virtual void    OpenHardware();                    // after open time
//    virtual void    StreamStopHardware();              // after stream stop subdevice configuration

    virtual double  StandardClockFrequency()
                        {  return Clock().Frequency();   }
    virtual double  Get_DacClockFrequency()
                        {  return Clock().Dac().Frequency();   }

    //  Subdevice Customization Handlers
    //  Type1 Chrono
    virtual Lmk04820 *                  MakeFmcAfePllObject();

    //  XA Adc Device Mixin
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();

    //  XA Dac Device Mixin
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    //  Stream Configuration
    virtual unsigned int                GetVeloInStreamCount();
    virtual unsigned int                GetVeloOutStreamCount();

    //  Pattern Mode
    virtual unsigned int                GetPatternModeScaleFactor();
    virtual unsigned int                GetPatternModeAlignment();

    // Aurora
    virtual Si5338 *                    MakeSioPllObject(I2CDevice & device);

private:
    unsigned int  FDacInterpolationMode;

    void     LocalConstructCustomParts();

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void     ConfigureDacs();
};

//=============================================================================
//  CLASS XA160_DacCalOverride  --  Cal Override for XA160 Dacs
//=============================================================================

class XA160_DacCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    XA160_DacCalOverride(XA_160DeviceMap *  owner) : Owner(owner)
        {}
    virtual ~XA160_DacCalOverride()
        {}

private:
    XA_160DeviceMap *  Owner;
};

//===========================================================================
//  CLASS XA_160InputDevice  -- XA_160 Input Devices
//===========================================================================

class XA_160InputDevice : public XAInputDevice
{
public:
    typedef XAInputDevice inherited;

    XA_160InputDevice(XA_160DeviceMap* owner , IX6AnalogIoInfo* info);
    virtual ~XA_160InputDevice();

    virtual IX6AnalogIoInfo &  Info();

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

private:
    XA_160DeviceMap *           OwnerXA;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     AdcCalIdrom()  {  return OwnerXA->AdcCal_PG;  }

};

//===========================================================================
//  CLASS XA_160OutputDevice  -- XA_160 Output Devices
//===========================================================================

class XA_160OutputDevice : public XAOutputDevice
{
public:
    typedef XAOutputDevice inherited;

    XA_160OutputDevice(XA_160DeviceMap* owner, IX6AnalogIoInfo * info );
    virtual ~XA_160OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    //  Special Dac Interpolation Mode
    enum  DacInterpolationModeValues { diByOne=0x0, diByTwo=0x1, diByTwo_b=0x02,
                                       diByFour=0x3, diByFour_b=0x6, diByEight=0x7 };
    void  SetDacInterpolation(DacInterpolationModeValues values);
    void  SetDacInterpolation(unsigned int values);

    AdDac9122::DacDevice &     Device()
        {
            return DacDev;
        }
    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    void ConfigureChannelEnables();

private:
    XA_160DeviceMap *           OwnerXA;
    IX6AnalogIoInfo *           FInfo;
    AdDac9122::DacDevice        DacDev;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     DacCalIdrom()  {  return OwnerXA->DacCal_PG;  }

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
