//---------------------------------------------------------------------------
// XA_160_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XA_160_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XA_160  -- XA_160 baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XA_160
//---------------------------------------------------------------------------

XA_160::XA_160()
    :  XAFamilyBoard(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_160
//---------------------------------------------------------------------------

XA_160::~XA_160()
{
    Close();
}

//---------------------------------------------------------------------------
//  XA_160::Input() --  Return reference to Output object
//---------------------------------------------------------------------------

XA_160InputDevice & XA_160::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  XA_160::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

XA_160OutputDevice & XA_160::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  XA_160::Clock() --  Return reference to timebase manager
//---------------------------------------------------------------------------

XA_LMK20_Clock::XAClockIo& XA_160::Clock()
{
    return Device.Clock();
}

//---------------------------------------------------------------------------
//  XA_160::RawClockDevice() --  Return reference to raw clock classes
//---------------------------------------------------------------------------

XA_LMK20_Clock::XARawClockDevices& XA_160::RawClockDevice()
{
    return Device.Clock_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  XA_160::AfterFiclStart() --
//---------------------------------------------------------------------------

void  XA_160::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant SioPll  "
                    "  2 constant Adc0Spi  3 constant Dac0Spi "
                    );

    engine.Evaluate(": ?XA-160 "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll   SioPll   Adc0Spi   Dac0Spi  \" "
                    "    cr  "
                    "    cr .\" ---- XA-160 Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?XA-160  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("XA-160-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    XAFamilyBoard::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  XA_160::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  XA_160::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;        const int kSioPll = 1;
    const int kAdc0Spi = 2;    const int kDac0Spi = 3;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return RawClockDevice().Pll().Port(addr);
        case kSioPll:
            return AuroraPll().Pll().Port(addr);
        case kAdc0Spi:
            return Device.Input().Spi(0, addr);
        case kDac0Spi:
            return Device.Output().Spi(0, addr);
    }
}

//---------------------------------------------------------------------------
//  XA_160::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  XA_160::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;        const int kSioPll = 1;
    const int kAdc0Spi = 2;    const int kDac0Spi = 3;

    switch (dev)
        {
        default:
            break;
        case kPll:
            RawClockDevice().Pll().Port(addr, value);
            break;
        case kSioPll:
            AuroraPll().Pll().Port(addr, value);
            break;
        case kAdc0Spi:
            Device.Input().Spi(0, addr, value);
            break;
        case kDac0Spi:
            Device.Output().Spi(0, addr, value);
            break;
    }
}

//---------------------------------------------------------------------------
//  XA_160::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  XA_160::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kSioPll = 1;

    switch (dev)
        {
        default:
            break;
        case kPll:
            RawClockDevice().Pll().Frequency(freq);
            RawClockDevice().Pll().Reapply();
            break;
        case kSioPll:
            AuroraPll().Pll().Frequency(freq);
            break;
        }
}

//---------------------------------------------------------------------------
//  XA_160::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  XA_160::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kSioPll = 1;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(RawClockDevice().Pll().FrequencyActual());
        case kSioPll:
            return static_cast<float>(AuroraPll().Pll().FrequencyActual());
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
