// XA_402DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2018 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XA_402_Mb.h>
#include <XA_402DeviceMap_Mb.h>
#include <XAFamilyMemoryMap_Mb.h>
#include <XA_402MemoryMap_Mb.h>
#include <FloatingPtSupport_Mb.h>
#include <SystemSupport_Mb.h>
#include <AD7768_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace XA_402MemoryMap;


//---------------------------------------------------------
//  Subdevice parameters Definition

//  Parameters for SubDevices

// ADC
using cint_t = const int;
using uint_t = unsigned int;

cint_t  XA_402_AdcLogicalDevices           = 1;
cint_t  XA_402_AdcLogicalChannelsPerDevice = 16;
cint_t  XA_402_AdcChannels = XA_402_AdcLogicalDevices*XA_402_AdcLogicalChannelsPerDevice;

cint_t  XA_402_AdcCalibrationDeviceCount    = XA_402_AdcChannels;   // number of cal correction channels

cint_t  XA_402_AdcPhysicalDevices           = 2;
cint_t  XA_402_AdcPhysicalChannelsPerDevice = 8;
uint_t  XA_402_AdcSpiDeviceCount            = XA_402_AdcPhysicalDevices;

// DAC
cint_t  XA_402_DacLogicalDevices            = 1;
cint_t  XA_402_DacLogicalChannelsPerDevice  = 8;
cint_t  XA_402_DacChannels = XA_402_DacLogicalDevices*XA_402_DacLogicalChannelsPerDevice;

cint_t  XA_402_DacPhysicalDevices           = 8;
cint_t  XA_402_DacPhysicalChannelsPerDevice = 1;
uint_t  XA_402_DacSpiDeviceCount            = 0;

uint_t  XA_402_DacCalibrationDeviceCount = XA_402_DacChannels;  // number of cal correction channels

cint_t  XA_402_VeloInStreamCount            = 1;    // Number of Velocia Incoming PIDs
cint_t  XA_402_VeloOutStreamCount           = 1;    // Number of Velocia Outgoing PIDs

//  Input Info Data
const float         InputMaxRate            = 265.0e3f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 24;
const          int  InputMax_mV             = 1200;
const          int  InputMin_mV             = -1200;
const unsigned int  InputSamplesPerWord     = 1;
const unsigned int  InputTriggerGranularity = 1;

const unsigned int XA_402_Adc_One = 0x10000;    // representation of floating point 1.0

//  Output Info Data
const float         OutputMaxRate            = 50.0e6f;
const float         OutputMinRate            = 5.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 2000;
const          int  OutputMin_mV             = -2000;
const unsigned int  OutputSamplesPerWord     = 1;
const unsigned int  OutputTriggerGranularity = 1;


const unsigned int XA_402_Dac_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romXA_402_SysInfoOffset        = 0;
const int romXA_402_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romXA_402_InputCoeffOffset       = 0;                             // XA_402_AdcChannels * 2 coefficients plus 'calibrated' word
const int romXA_402_InputCoeffOffset_avail =  XA_402_AdcChannels * 2 + 1;

//  ...Dac Cal Idrom: DAC coefficients
const int romXA_402_OutputCoeffOffset       = 0;
const int romXA_402_OutputCoeffOffset_avail = XA_402_DacChannels * 2 + 1;   // XA_402_DacChannels * 2 coefficients plus 'calibrated' word

//===========================================================================
// Class XA_402_AnalogInInfo
//===========================================================================

class XA_402_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    XA_402_AnalogInInfo(XA_402DeviceMap * owner)
        :  Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~XA_402_AnalogInInfo()  {  }

    virtual float           MaxRate() const  {  return InputMaxRate;     }
    virtual float           MinRate() const  {  return InputMinRate;     }
    virtual SpanInfo        Span()    const  {  return FSpan;            }
    virtual unsigned int    Bits()    const  {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    XA_402DeviceMap * Owner;
    SpanInfo   FSpan;
};

//===========================================================================
// Class XA_402_AnalogOutInfo
//===========================================================================

class XA_402_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    XA_402_AnalogOutInfo(XA_402DeviceMap * owner)
         :  Owner(owner), FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~XA_402_AnalogOutInfo()  {  }

    virtual float     MaxRate() const
        {
            return OutputMaxRate;
        }
    virtual float         MinRate() const     {  return OutputMinRate;  }
    virtual SpanInfo      Span() const        {  return FSpan;  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    XA_402DeviceMap * Owner;
    SpanInfo   FSpan;
};

//=============================================================================
//  CLASS XA_402_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_402_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 122.88e6;

//---------------------------------------------------------------------------
//  constructor for class XA_402_Pll
//---------------------------------------------------------------------------

XA_402_Pll::XA_402_Pll(XA_402DeviceMap * owner, IAddressingSpace &space, Lmk04800Registers & reg)
    : Lmk04800(space, reg, DualIVco), Owner(owner)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Adc Master as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);    // All Refs are 122.88MHz
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class XA_402_Pll
//---------------------------------------------------------------------------

XA_402_Pll::~XA_402_Pll()
{
}

//---------------------------------------------------------------------------
//  XA_402_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_402_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "XA_402_Pll::OnInit()");
    //
    //  Input Configuration
    SetInputConfig(0, icBipolar);
    SetInputConfig(1, icBipolar);

    //
    //  Set Source Configuration
    SourceInUse(0, true);   SetSourceMapping(0, IXAClockIo::rsInternal);     // clkin0 = internal
    SourceInUse(1, true);   SetSourceMapping(1, IXAClockIo::rsExternal);     // clkin1 = front panel external

    //  Set Output Configuration
    OutputInUse(0,  true);     SetOutputConfig(0,  ocLvCmos_P_N);    // DAC0_CLK / DAC1_CLK
    OutputInUse(1,  true);     SetOutputConfig(1,  ocLvCmos_P_N);    // DAC2_CLK / DAC3_CLK
    OutputInUse(2,  true);     SetOutputConfig(2,  ocLvCmos_P_N);    // DAC4_CLK / DAC5_CLK
    OutputInUse(3,  true);     SetOutputConfig(3,  ocLvCmos_P_N);    // DAC6_CLK / DAC7_CLK
    OutputInUse(4,  true);     SetOutputConfig(4,  ocLvds);          // DAC_FPGA_CLK
    OutputInUse(5,  false);    SetOutputConfig(5,  ocDisabled);      //
    OutputInUse(6,  true);     SetOutputConfig(6,  ocLvds);          // ADC0_CLK
    OutputInUse(7,  true);     SetOutputConfig(7,  ocLvds);          // ADC1_CLK
    OutputInUse(8,  true );    SetOutputConfig(8,  ocLvds);          // ADC_FPGA_MCLK
    OutputInUse(9,  false);    SetOutputConfig(9,  ocDisabled);      //
    OutputInUse(10, true);     SetOutputConfig(10, ocLvPecl_1600);   // feedback, CLK_OUTPUT
    OutputInUse(11, false);    SetOutputConfig(11, ocDisabled);      //

    FeedbackOutput(fbm_Clkout10);  // set output to use as PLL feedback in 0-delay modes

    //
    //  Reference
    ReferenceSource(refPrimary);    // Set to use primary reference
}

//---------------------------------------------------------------------------
//  XA_402_Pll::OnReapply()
//---------------------------------------------------------------------------

void  XA_402_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  XA_402_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  XA_402_Pll::AdcMasterOutput()
{
    return 6;
}

//---------------------------------------------------------------------------
//  XA_402_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  XA_402_Pll::DacMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  XA_402_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool XA_402_Pll::IsFeedbackOutputAdc()
{
    return false;
}

//---------------------------------------------------------------------------
//  XA_402_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  XA_402_Pll::SetAdcOutputRates(double adc_rate)
{
    double mClk0 = Owner->Adc(0).CalculateDivisors(adc_rate);
    double mClk1 = Owner->Adc(1).CalculateDivisors(adc_rate);

    // assert (mClk0 == mClk1)
    OutputFrequency(6,   mClk0);  // ADC0_CLK
    OutputFrequency(7,   mClk1);  // ADC1_CLK
    OutputFrequency(8,   mClk0);  // ADC_FPGA_MCLK
}

//---------------------------------------------------------------------------
//  XA_402_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  XA_402_Pll::SetDacOutputRates(double dac_rate)
{
    OutputFrequency(0,   dac_rate);    // DAC0_CLK / DAC1_CLK
    OutputFrequency(1,   dac_rate);    // DAC2_CLK / DAC3_CLK
    OutputFrequency(2,   dac_rate);    // DAC4_CLK / DAC5_CLK
    OutputFrequency(3,   dac_rate);    // DAC6_CLK / DAC7_CLK
    OutputFrequency(4,   dac_rate);    // DAC_FPGA_CLK
    OutputFrequency(10,  dac_rate);    // CLK_OUTPUT
}

//===========================================================================
//  CLASS XA_402DeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_402DeviceMap
//---------------------------------------------------------------------------

using namespace XA_MemoryMap;
using namespace XA_402MemoryMap;

namespace
{

K7FpaLoader::MapRegisters XA_VsPromRegAddrs =
    {
    mmFpa_AddressCtrl,       //  AddressControl_Addr;
    mmFpa_Data,              //  Data_Addr
    mmFpa_Opcode,            //  Opcode_Addr;
    };

K7FlashRom::MapRegisters XA_IdRomRegAddrs =
    {
    mmRom_AddrCtl,           //  AddressControl_Addr;
    mmRom_Data,              //  Data_Addr
    mmRom_Opcode,            //  Opcode_Addr;
    };

XA_LMK_Clock::MapRegisters  LMK_ClockMapAddrs =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    mmAFE_VcxoControl,   //  AfeVcxoControl_Addr;
    };

Lmk04800Registers  XA_402_Lmk04800Registers =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    };

LTC2494SpiDevice::Info XA_402SpiRegAddrs =
    {
    mmPowerMon_LTC_SPI_Control,  //
    mmPowerMon_LTC_SPI_Stat,     //
    };
}

const unsigned int System_IdromPage = 0;
const unsigned int AdcCal_IdromPage = 1;
const unsigned int DacCal_IdromPage = 2;
const unsigned int User_IdromPage   = 3;


//---------------------------------------------------------------------------
//  constructor for class XA_402DeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

XA_402DeviceMap::XA_402DeviceMap(XA_402 * owner)
    :  XABidirDeviceMap(owner),
       Owner(owner),
       VsProm_SDev(0, K7FpaLoader::romS25FL128S, WB_Loader, XA_VsPromRegAddrs),
       IdRom_SDev(0, K7FlashRom::romS25FL128S, WB_FlashRom, XA_IdRomRegAddrs),
       Clock_SDev(this, WB_AFE, LMK_ClockMapAddrs),
       SelfMeasure_SDev(this),
       FInput(new XA_402InputDevice(this, new XA_402_AnalogInInfo(this))),
       FOutput(new XA_402OutputDevice(this, new XA_402_AnalogOutInfo(this))),
       ADC0_SDev(this, 0),
       ADC1_SDev(this, 1),
       System_Page(IdRom_SDev,  System_IdromPage),
       AdcCal_Page(IdRom_SDev,  AdcCal_IdromPage),
       DacCal_Page(IdRom_SDev,  DacCal_IdromPage),
       UserRom_Page(IdRom_SDev, User_IdromPage),
       IdRom_Tunnel(this),
       FDacInterpolationMode(0x0)
{
    //
    //  SubDevice Customization
    LocalConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,               &XA_402DeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &XA_402DeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,               &XA_402DeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this,             &XA_402DeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,              &XA_402DeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,              &XA_402DeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,               &XA_402DeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class XA_402DeviceMap
//---------------------------------------------------------------------------

XA_402DeviceMap::~XA_402DeviceMap()
{
    delete FInput;
    FInput = 0;
    delete FOutput;
    FOutput = 0;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------
//  We can't make this part of virtual tree for ConstructCustomParts since
//   its being called in the constructor and the tree is not built yet.

void  XA_402DeviceMap::LocalConstructCustomParts()
{
    // Subdevice Parts Construction
    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    VFifo_SDev.ConstructCustomParts();
    FrontPanelDio_SDev.ConstructCustomParts();
    VeloIn_SDev.ConstructCustomParts();
    VeloOut_SDev.ConstructCustomParts();
    Alerts_SDev.ConstructCustomParts();
    Thermal_SDev.ConstructCustomParts();
    Dio_SDev.ConstructCustomParts();
    Uart_SDev.ConstructCustomParts();
    IdRom_SDev.ConstructCustomParts();
    AFECommon_SDev.ConstructCustomParts();
    Aurora0_SDev.ConstructCustomParts();
    // Subdevice Parts Construction
    AFEAdcC_SDev.ConstructCustomParts();
    AFEAdcPri_SDev.ConstructCustomParts();
    AFEAdcSpi_SDev.ConstructCustomParts();
    AFEAdcCal_SDev.ConstructCustomParts();
    XADac_DeviceMixin::ConstructCustomParts();
    //
    VsProm_SDev.ConstructCustomParts();
    Clock_SDev.ConstructCustomParts();
    SelfMeasure_SDev.ConstructCustomParts();
    ADC0_SDev.ConstructCustomParts();
    ADC1_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OpenHardware()
{
    //  Reset the device
    MasterReset();
    Periph_SDev.AppReset();
    // Subdevice Configuration
    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    VFifo_SDev.OpenHardware();
    VeloOut_SDev.OpenHardware();
    FrontPanelDio_SDev.OpenHardware();
    VeloIn_SDev.OpenHardware();
    Alerts_SDev.OpenHardware();
    Thermal_SDev.OpenHardware();
    Dio_SDev.OpenHardware();
    Uart_SDev.OpenHardware();
    AFECommon_SDev.OpenHardware();
    IdRom_SDev.OpenHardware();
    Aurora0_SDev.OpenHardware();

    //  Read all ROM pages to Init Cache
    System_Page.LoadFromRom();
    AdcCal_Page.LoadFromRom();
    DacCal_Page.LoadFromRom();
    UserRom_Page.LoadFromRom();

    AFEAdcC_SDev.OpenHardware();
    AFEAdcPri_SDev.OpenHardware();
    AFEAdcSpi_SDev.OpenHardware();
    AFEAdcCal_SDev.OpenHardware();

    XADac_DeviceMixin::OpenHardware();
    //
    VsProm_SDev.OpenHardware();
    Clock_SDev.OpenHardware();
    SelfMeasure_SDev.OpenHardware();
    ADC0_SDev.OpenHardware();
    ADC1_SDev.OpenHardware();
    Input().OpenHardware();
    Output().OpenHardware();
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "XA_402DeviceMap::OnStreamPreconfigHandler()");
//    Output().ConfigureChannelEnables();

    FInput->Cal().ApplyRomCoefficients();
    FOutput->Cal().ApplyRomCoefficients();

    // Family-common init
    // Subdevice Configuration
    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    VFifo_SDev.PreconfigureHardware();
    FrontPanelDio_SDev.PreconfigureHardware();
    VeloIn_SDev.PreconfigureHardware();
    VeloOut_SDev.PreconfigureHardware();
    Alerts_SDev.PreconfigureHardware();
    Thermal_SDev.PreconfigureHardware();
    Dio_SDev.PreconfigureHardware();
    Uart_SDev.PreconfigureHardware();
    AFECommon_SDev.PreconfigureHardware();
    //FlashRom_SDev.PreconfigureHardware();
    Aurora0_SDev.PreconfigureHardware();

    // Subdevice Configuration
    AFEAdcC_SDev.PreconfigureHardware();
    AFEAdcPri_SDev.PreconfigureHardware();
    AFEAdcSpi_SDev.PreconfigureHardware();
    AFEAdcCal_SDev.PreconfigureHardware();

    XADac_DeviceMixin::PreconfigureHardware();

    VsProm_SDev.PreconfigureHardware();

    SelfMeasure_SDev.PreconfigureHardware();

    Output().ConfigureChannelEnables();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);

    ADC0_SDev.PreconfigureHardware();
    ADC1_SDev.PreconfigureHardware();

    Input().PreconfigureHardware();
    Output().PreconfigureHardware();


}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "XA_402DeviceMap::OnStreamConfigureHandler()");

    // Family-common init
    Periph_SDev.AppReset();

    // Subdevice Configuration
    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    VFifo_SDev.ConfigureHardware();
    FrontPanelDio_SDev.ConfigureHardware();
    VeloIn_SDev.ConfigureHardware();
    VeloOut_SDev.ConfigureHardware();
    Alerts_SDev.ConfigureHardware();
    Thermal_SDev.ConfigureHardware();
    Dio_SDev.ConfigureHardware();
    Uart_SDev.ConfigureHardware();
    AFECommon_SDev.PreconfigureHardware();
    IdRom_SDev.ConfigureHardware();
    Aurora0_SDev.ConfigureHardware();

    AFEAdcC_SDev.ConfigureHardware();
    AFEAdcPri_SDev.ConfigureHardware();
    AFEAdcSpi_SDev.ConfigureHardware();
    AFEAdcCal_SDev.ConfigureHardware();

    XADac_DeviceMixin::ConfigureHardware();

    VsProm_SDev.ConfigureHardware();

    SelfMeasure_SDev.ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);

    Input().AdcCommon().Reset();
    ADC0_SDev.ConfigureHardware();
    ADC1_SDev.ConfigureHardware();
    Input().AdcCommon().Sync();

    Input().ConfigureHardware();
    Output().ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    Periph_SDev.XARun(true);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_402DeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    Periph_SDev.XARun(false);

    //  Base class "on stream stop" behavior
    XABidirDeviceMap::StreamStopHardware();
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}


//---------------------------------------------------------------------------
//  XA_402DeviceMap::Input() -- Reference to output object
//---------------------------------------------------------------------------

XA_402InputDevice & XA_402DeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

XA_402OutputDevice & XA_402DeviceMap::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::AdcSpi()
//---------------------------------------------------------------------------

void XA_402DeviceMap::AdcSpi(unsigned int dev, unsigned int addr, unsigned int value)
{
    Input().Spi(dev, addr, value);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::AdcSpi()
//---------------------------------------------------------------------------

unsigned int XA_402DeviceMap::AdcSpi(unsigned int dev, unsigned int addr)
{
    return Input().Spi(dev, addr);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::AdcChannelEnabled()
//---------------------------------------------------------------------------

bool XA_402DeviceMap::AdcChannelEnabled(int ch)
{
    return Input().AdcCommon().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::AdcChannelEnabled()
//---------------------------------------------------------------------------

void XA_402DeviceMap::AdcChannelEnabled(int ch, bool state)
{
    Input().AdcCommon().ChannelEnabled(ch, state);
    Input().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::Make_AfePllObject()
//---------------------------------------------------------------------------

Lmk04800 *  XA_402DeviceMap::MakeFmcAfePllObject()
{
    return new XA_402_Pll(this, WB_AFE, XA_402_Lmk04800Registers );
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  XA_402DeviceMap::MakePriClockDevice(PriDeviceCode which)
{
    if (which==priAdc)
        return new XAStandardPriClockDevice(this, 1);
    else
        return new XAStandardPriClockDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetAdcSpiDeviceCount() -- Get # of ADC Spi Devices
//---------------------------------------------------------------------------

unsigned int XA_402DeviceMap::GetAdcSpiDeviceCount()
{
    return XA_402_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetCalibrationAdcCount() -- Get # of ADC cal channels
//---------------------------------------------------------------------------

unsigned int XA_402DeviceMap::GetCalibrationAdcCount()
{
    return XA_402_AdcCalibrationDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  XA_402DeviceMap::GetAdcCalibrationOverride()
{
    return 0;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  XA_402DeviceMap::MakeFrameCalcDevice(FrameDeviceCode which)
{
    if (which==devAdc)
        return new XAStandardFrameCalcDevice(this, 1);
    else
        return new XAStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  XA_402DeviceMap::GetAnalogDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = XA_402_AdcLogicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_402_AdcLogicalChannelsPerDevice;
        }
    else
        {
        devices = XA_402_DacLogicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_402_DacLogicalChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  XA_402DeviceMap::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices    = XA_402_AdcPhysicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_402_AdcPhysicalChannelsPerDevice;
        }
    else
        {
        devices    = XA_402_DacPhysicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_402_DacPhysicalChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  XA_402DeviceMap::GetDacSpiDeviceCount()
{
    return XA_402_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  XA_402DeviceMap::GetDacCalibrationOverride()
{
    return 0;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  XA_402DeviceMap::GetCalibrationDacCount()
{
    return XA_402_DacCalibrationDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  XA_402DeviceMap::MakeSioPllObject(I2CDevice & /*device*/)
{
    //  Make our local SioPll object (subdev will delete it)
    return 0;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_402DeviceMap::GetVeloInStreamCount()
{
    return XA_402_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_402DeviceMap::GetVeloOutStreamCount()
{
    return XA_402_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   XA_402DeviceMap::GetPatternModeAlignment()
{
    return 16;   // standard XA alignment of 16 words
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   XA_402DeviceMap::GetPatternModeScaleFactor()
{
    return 1;   // standard XA scale factor of 1 words
}

//---------------------------------------------------------------------------
//  XA_402DeviceMap::MakeLTC2494() -- Get LTC2494 Device object
//---------------------------------------------------------------------------

LTC2494 *  XA_402DeviceMap::MakeLTC2494()
{
    return new XA_402LTC2494(WB_Periph, XA_402SpiRegAddrs, "XA_402_LTC2494");
}

//=============================================================================
//  CLASS XA402_DacCalOverride  --  Cal Override for XA402 Dacs
//=============================================================================
//---------------------------------------------------------------------------
//  XA402_DacCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  XA402_DacCalOverride::OverrideOffset(unsigned int value)
{
    // trim to 16 bit value
    return value&0xFFFF;
}

//---------------------------------------------------------------------------
//  XA402_DacCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  XA402_DacCalOverride::OverrideGain(unsigned int value)
{
    const float HdwStep_mA    = 0.0225f;    // 22.5 uA steps
    const float Min_mA        = 8.64f;      // 0x000 gain value
    const float Center_mA     = 20.00f;     // 0x1f9 gain value


    float gain = static_cast<float>(value)/XA_402_Dac_One;

    float adjusted_mA = gain*Center_mA;
    float steps_mA = adjusted_mA - Min_mA;

    float counts = steps_mA / HdwStep_mA;
    unsigned int setting;
    if (counts < 0)
        setting = 0;
    else if (counts > 1023)
        setting = 0x3FF;
    else
        setting = static_cast<unsigned int>(counts);

    return setting;
}

//---------------------------------------------------------------------------
//  XA402_DacCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  XA402_DacCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    Owner->DacCalibration().SetCalibrationOffset(ch, value);
}

//---------------------------------------------------------------------------
//  XA402_DacCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  XA402_DacCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    Owner->DacCalibration().SetCalibrationGain(ch, value);
}

//===========================================================================
//  CLASS XA_402InputDevice  -- XA_402 Input Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_402InputDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_402InputDevice
//---------------------------------------------------------------------------

XA_402InputDevice::XA_402InputDevice(XA_402DeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerXA(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romXA_402_InputCoeffOffset, XA_402_AdcChannels,
                       XA_402_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_402InputDevice
//---------------------------------------------------------------------------

XA_402InputDevice::~XA_402InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  XA_402InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  XA_402InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  XA_402InputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  XA_402InputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  XA_402InputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  XA_402InputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  XA_402InputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  XA_402InputDevice::ConfigureHardware()
{
}

void XA_402InputDevice::ChannelEnabled(int ch, bool state)
{
    OwnerXA->AdcChannelEnabled(ch, state);
}

bool XA_402InputDevice::ChannelEnabled(int ch) const
{
    return OwnerXA->AdcChannelEnabled(ch);
}


//===========================================================================
//  CLASS XA_402OutputDevice  -- XA_402 Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_402OutputDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_402OutputDevice
//---------------------------------------------------------------------------

XA_402OutputDevice::XA_402OutputDevice(XA_402DeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerXA(owner), FInfo(info),
      CalRomSection(DacCalIdrom(), romXA_402_OutputCoeffOffset, XA_402_DacChannels,
                       XA_402_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_402OutputDevice
//---------------------------------------------------------------------------

XA_402OutputDevice::~XA_402OutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  XA_402OutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  XA_402OutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();

}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  XA_402OutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();

}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  XA_402OutputDevice::ConfigureHardware()
{
    OwnerXA->Debug.Log("DAC ConfigureHardware()");

    // Toggle the DacReset
    DacCommon().DacReset(false);
    DacCommon().DacReset(true);
    Sleep(1);
    DacCommon().DacReset(false);
}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  XA_402OutputDevice::ConfigureChannelEnables()
{
    //  Enable Devices and Channels
    //Device().Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1) );
    DacCommon().ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    DacCommon().ChannelEnabled(1, DacCommon().ChannelEnabled(1) );
}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  XA_402OutputDevice::SetDacInterpolation(DacInterpolationModeValues values)
{
    SetDacInterpolation(static_cast<unsigned int>(values));
}

//---------------------------------------------------------------------------
//  XA_402OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  XA_402OutputDevice::SetDacInterpolation(unsigned int values)
{
    OwnerXA->FDacInterpolationMode = values&0x7;
}

//===========================================================================
//  CLASS XA_402LTC2494 --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_402LTC2494
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  XA_402LTC2494::XA_402LTC2494() -- Constructor
//---------------------------------------------------------------------------

XA_402LTC2494::XA_402LTC2494(IAddressingSpace &space,  LTC2494SpiDevice::Info &regs, std::string tag)
    :  LTC2494(space, regs, tag)
{

}

//---------------------------------------------------------------------------
//  XA_402LTC2494::~XA_402LTC2494() -- Destructor
//---------------------------------------------------------------------------

XA_402LTC2494::~XA_402LTC2494()
{

}

//---------------------------------------------------------------------------
//  XA_402LTC2494::ConfigureBoardDefaults() -- config board defualts
//---------------------------------------------------------------------------

void XA_402LTC2494::ConfigureBoardDefaults()
{
    // Channel Mapping
    ChannelTypeTable.resize(16);
    fill(ChannelTypeTable.begin(), ChannelTypeTable.end(), SingleEnded);

    MapChannels();

    //configuring defualt bits
    ConfigBits.EN  = 1;
    ConfigBits.EN2 = 1;
    ConfigBits.FA  = 0;
    ConfigBits.FB  = 0;
    ConfigBits.SPD = 0; // dont care if temp measurement
    ConfigBits.GS0 = 0; //
    ConfigBits.GS1 = 0; // dont care if temp measurement
    ConfigBits.GS2 = 0; //

    const double ReferenceV = 1.25;
    double scales[] = { 1.0000 * ReferenceV, 1.0000 * ReferenceV,
                        12.050 * ReferenceV, 5.0300 * ReferenceV,
                        3.3200 * ReferenceV, 5.0300 * ReferenceV,
                        6.0000 * ReferenceV, 5.0300 * ReferenceV,
                        3.3200 * ReferenceV, 2.5050 * ReferenceV,
                        1.8000 * ReferenceV, 1.5000 * ReferenceV,
                        1.2256 * ReferenceV, 1.0000 * ReferenceV,
                        1.0000 * ReferenceV, 1.0000 * ReferenceV
    };
    // initialize Scale table (COEF column * ReferenceV)
    copy(std::begin(scales), std::end(scales), std::back_inserter(ScaleTable));

    string signalNames[] = {
        "GROUND" , "1.0VREF", "12V_S0", "5V_S0",
        "3.3V_S0", "5V_S5"  , "P_VAMP", "5P0V",
        "3P3V"   , "2P5V"   , "1P8V"  ,  "1P5V_DDR3",
        "1P2V_VMGTAVTT", "1P0V", "1P0V_VMGTAVCC", "DDR3_VTT"
    };
    //  initialize SignalNameStr table
    copy(std::begin(signalNames), std::end(signalNames), std::back_inserter(SignalNameStr));

    double nominalVs[] = {
        0.000, 1.000, 12.000, 5.000,
        3.300, 5.000,  5.800, 5.000,
        3.300, 2.500,  1.800, 1.500,
        1.200, 1.000,  1.000, 0.750
    };
    //  initialize NominalV table
    copy(std::begin(nominalVs), std::end(nominalVs), std::back_inserter(NominalV));

    double minVs[] = {
        -0.01, 0.99, 11.40, 4.80,
         3.15, 4.80,  5.60, 4.80,
         3.15, 2.40,  1.70, 1.40,
         1.10, 0.90,  0.90, 0.70
    };
    //  initialize MinV table
    copy(std::begin(minVs), std::end(minVs), std::back_inserter(MinV));

    double maxVs[] = {
        0.01, 1.01, 12.60, 5.20,
        3.45, 5.20,  6.00, 5.20,
        3.45, 2.60,  1.90, 1.60,
        1.30, 1.10,  1.10, 0.80
    };
    //  initialize MaxV table
    copy(std::begin(maxVs), std::end(maxVs), std::back_inserter(MaxV));

    double nominalAdcs[] = {
        0.000000000, 1.000000000, 0.995850622, 0.994035785,
        0.985074627, 0.994035785, 0.966666667, 0.994035785,
        0.985074627, 0.968992248, 1.000000000, 1.000000000,
        0.978393804, 1.000000000, 1.000000000, 0.750000000
    };
    //  initialize NominalAdcV table
    copy(std::begin(nominalAdcs), std::end(nominalAdcs), std::back_inserter(NominalAdcV));

    //  initialize SignalUnitsStr table
    fill(SignalUnitsStr.begin(), SignalUnitsStr.begin(), " V");
}

//---------------------------------------------------------------------------
//  XA_402LTC2494::PowerCalculation() -- calc power numbers using cache
//---------------------------------------------------------------------------

double XA_402LTC2494::PowerCalculation(unsigned int power_ch)
{
    switch (power_ch)
        {
        default:
            return 0.0;
        }
}


} // namespace Innovative


