// XA_402DeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2018 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XA_402DeviceMap_MbH
#define XA_402DeviceMap_MbH


#include <XASubDevice_Stream_Mb.h>
#include <XAFamilyBoard_Mb.h>
#include <AdDac9122_Mb.h>
#include <AD7768_Mb.h>

#include <Trace_Mb.h>


using Innovative::ad7768::AD7768_ADC;
using Innovative::ad7768::IRequires_AD7768_ADC_Io;

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class XA_402DeviceMap;

//==============================================================================
//  CLASS XA_402_Pll -- XA_402 PLL Chip (Specialization of Lmk04820)
//==============================================================================

class XA_402_Pll : public Lmk04800
{
public:
    enum Ad7768_Modes { mFastMode, mMedianMode, mEcoMode };

    XA_402_Pll(XA_402DeviceMap * owner, IAddressingSpace &space, Lmk04800Registers & reg);
    virtual ~XA_402_Pll();

    // ...Lmk04800 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 40e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 122.88e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XA_402DeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

private:
    Ad7768_Modes  AdcMode;
    unsigned int  Decimation;
    unsigned int  MCLKDiv;
    double        CurrentMCLK;
    double        CurrentDCLK;

    double  MCLK(double adc_rate);
};


//===========================================================================
//  CLASS ISuppliesXA_402Requirements -- Device Map Requirement List
//===========================================================================

class ISuppliesXA_402Requirements : public IRequires_Nothing,
                                    public IRequires_LmkFmcClocks,
                                    public IRequires_LTC2494Device,
                                    public IRequires_AD7768_ADC_Io
{
public:
    virtual ~ISuppliesXA_402Requirements()
        {}

};

//===========================================================================
//  CLASS XA_402DeviceMap  -- Register & low level support for XA_402 module
//===========================================================================
class XA_402;

class XA_402DeviceMap :  public XABidirDeviceMap, public ISuppliesXA_402Requirements

{
    friend class XA_402;
    friend class XA_402InputDevice;
    friend class XA_402OutputDevice;

public:
    //
    //  Member Functions
    XA_402DeviceMap(XA_402 * owner);
    virtual ~XA_402DeviceMap();

    XA_LMK_Clock::XAClockIo & Clock()           { return Clock_SDev.Clock();        }
    ISelfMeasurement        & SelfMeasure()     { return SelfMeasure_SDev.SelfM();  }
    IFpaLoader              & VsProm()          { return VsProm_SDev.VsProm();      }
    IIdRom                  & IdRom()           { return IdRom_Tunnel;              }

    AD7768_ADC&               Adc(unsigned int which)
    {  return (which == 0) ? ADC0_SDev : ADC1_SDev;  }

    class XA_402InputDevice  &  Input();
    class XA_402OutputDevice &  Output();


    X6AFE_DacCalibration &DacCalibration() {  return AFEDacCal_SDev;  }

private:
    // Prevent access to the parent
    using XAFamilyDeviceMap::RawFpaRom;
    using XAFamilyDeviceMap::RawIdRom;
    using XAFamilyDeviceMap::IdRom;
    using XAFamilyDeviceMap::VsProm;

public: // pseudo-protected

    unsigned int      CurrentDacInterpolationMode()
        {  return FDacInterpolationMode;  }

    virtual void AdcSpi(unsigned int dev, unsigned int addr, unsigned int value);
    virtual unsigned int AdcSpi(unsigned int dev, unsigned int addr);

    virtual bool         AdcChannelEnabled(int ch);
    virtual void         AdcChannelEnabled(int ch, bool state);


protected:
    // Data
    XA_402 *                Owner;
    //  Subdevices
    K7FpaLoader             VsProm_SDev;
    K7FlashRom              IdRom_SDev;
    XA_LMK_Clock            Clock_SDev;
    SelfMeasurementDevice   SelfMeasure_SDev;
    XA_402InputDevice *     FInput;
    XA_402OutputDevice *    FOutput;
    AD7768_ADC              ADC0_SDev;
    AD7768_ADC              ADC1_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage       System_Page;
    X6IdromPage             AdcCal_Page;
    X6IdromPage             DacCal_Page;
    X6IdromPage             UserRom_Page;


    //=======================================================================
    //  CLASS XA_402IdromIo::XAIdromIo  -- Access to IDROM Objects
    //=======================================================================
    class XA_402IdromIo : public IIdRom
    {
    public:

        XA_402IdromIo(XA_402DeviceMap * owner) :  Owner(owner) {}
        virtual ~XA_402IdromIo() {}

        virtual IIdRomDevice      &Device() { return Owner->IdRom_SDev;  }
        virtual X6SystemIdromPage &System() { return Owner->System_Page; }
        virtual X6IdromPage       &AdcCal() { return Owner->AdcCal_Page; }
        virtual X6IdromPage       &DacCal() { return Owner->DacCal_Page; }
        virtual X6IdromPage       &User()   { return Owner->UserRom_Page;}

    private:
        XA_402DeviceMap *  Owner;
    };

    friend class XA_402IdromIo;
    XA_402IdromIo                    IdRom_Tunnel;

    //
    Trace                   Debug;

    virtual void    OpenHardware();                    // after open time

    virtual double  StandardClockFrequency()
                        {  return Clock().Frequency();   }
    virtual double  Get_DacClockFrequency()
                        {  return Clock().Dac().Frequency();   }

    //  Subdevice Customization Handlers
    //  Type1 Chrono
    virtual Lmk04800 *                  MakeFmcAfePllObject();
    //  XA Adc Device Mixin
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();

    //  XA Dac Device Mixin
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    //  Stream Configuration
    virtual unsigned int                GetVeloInStreamCount();
    virtual unsigned int                GetVeloOutStreamCount();

    //  Pattern Mode
    virtual unsigned int                GetPatternModeScaleFactor();
    virtual unsigned int                GetPatternModeAlignment();

    // Aurora
    virtual Si5338 *                    MakeSioPllObject(I2CDevice & device);

    virtual LTC2494 *                   MakeLTC2494();

private:
    unsigned int  FDacInterpolationMode;

    void     LocalConstructCustomParts();

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void     ConfigureDacs();
};

//=============================================================================
//  CLASS XA402_DacCalOverride  --  Cal Override for XA402 Dacs
//=============================================================================

class XA402_DacCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    XA402_DacCalOverride(XA_402DeviceMap *  owner) : Owner(owner)
        {}
    virtual ~XA402_DacCalOverride()
        {}

private:
    XA_402DeviceMap *  Owner;
};

//===========================================================================
//  CLASS XA_402InputDevice  -- XA_402 Input Devices
//===========================================================================

class XA_402InputDevice : public XAInputDevice
{
public:
    typedef XAInputDevice inherited;

    XA_402InputDevice(XA_402DeviceMap* owner , IX6AnalogIoInfo* info);
    virtual ~XA_402InputDevice();

    virtual IX6AnalogIoInfo &  Info();

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    using XAInputDevice::AdcCommon;

private:
    XA_402DeviceMap *           OwnerXA;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     AdcCalIdrom()  {  return OwnerXA->AdcCal_Page;  }

};

//===========================================================================
//  CLASS XA_402OutputDevice  -- XA_402 Output Devices
//===========================================================================

class XA_402OutputDevice : public XAOutputDevice
{
public:
    typedef XAOutputDevice inherited;

    XA_402OutputDevice(XA_402DeviceMap* owner, IX6AnalogIoInfo * info );
    virtual ~XA_402OutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    //  Special Dac Interpolation Mode
    enum  DacInterpolationModeValues { diByOne=0x0, diByTwo=0x1, diByTwo_b=0x02,
                                       diByFour=0x3, diByFour_b=0x6, diByEight=0x7 };
    void  SetDacInterpolation(DacInterpolationModeValues values);
    void  SetDacInterpolation(unsigned int values);

    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    void ConfigureChannelEnables();

private:
    XA_402DeviceMap *           OwnerXA;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     DacCalIdrom()  {  return OwnerXA->DacCal_Page;  }

};

//===========================================================================
//  CLASS XA_402LTC2494 -- 16-Channel ADC chip for Pex7Cop
//===========================================================================

class XA_402LTC2494 : public LTC2494
{
public:
     XA_402LTC2494(IAddressingSpace &space, LTC2494SpiDevice::Info &Regs,
                  std::string tag="");
    ~XA_402LTC2494();

     virtual void      ConfigureBoardDefaults();
     virtual double    PowerCalculation(unsigned int power_ch);

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
