//---------------------------------------------------------------------------
// XA_402_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2018 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XA_402_Mb.h>

namespace Innovative
{

//===========================================================================
//  CLASS XA_402  -- XA_402 baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XA_402
//---------------------------------------------------------------------------

XA_402::XA_402()
    :  XAFamilyBoard(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_402
//---------------------------------------------------------------------------

XA_402::~XA_402()
{
    Close();
}

//---------------------------------------------------------------------------
//  XA_402::Input() --  Return reference to Output object
//---------------------------------------------------------------------------

XA_402InputDevice & XA_402::Input()
{
    return Device.Input();
}

//---------------------------------------------------------------------------
//  XA_402::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

XA_402OutputDevice & XA_402::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  XA_402::Clock() --  Return reference to timebase manager
//---------------------------------------------------------------------------

XA_LMK_Clock::XAClockIo& XA_402::Clock()
{
    return Device.Clock();
}

//---------------------------------------------------------------------------
//  XA_402::RawClockDevice() --  Return reference to raw clock classes
//---------------------------------------------------------------------------

XA_LMK_Clock::XARawClockDevices& XA_402::RawClockDevice()
{
    return Device.Clock_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  XA_402::SelfMeasure() --
//---------------------------------------------------------------------------

ISelfMeasurement &XA_402::SelfMeasure()
{
    return Device.SelfMeasure();
}

//---------------------------------------------------------------------------
//  XA_402::VsProm() --
//---------------------------------------------------------------------------

IFpaLoader &XA_402::VsProm()
{
    return Device.VsProm();
}

//---------------------------------------------------------------------------
//  XA_402::VsProm() --
//---------------------------------------------------------------------------

IIdRom & XA_402::IdRom()
{
    return Device.IdRom();
}

//---------------------------------------------------------------------------
//  XA_402::AfterFiclStart() --
//---------------------------------------------------------------------------

void  XA_402::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll "
                    "  1 constant Adc0Spi  2 constant Adc1Spi "
                    );

    engine.Evaluate(": ?XA-402 "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll  Adc0Spi   Adc1Spi  \" "
                    "    cr  "
                    "    cr .\" ---- XA-402 Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?XA-402  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("XA-402-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    XAFamilyBoard::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  XA_402::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  XA_402::Do_DFetch(int dev, int addr)
{
    const int kPll     = 0;
    const int kAdc0Spi = 1;
    const int kAdc1Spi = 2;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return RawClockDevice().Pll().Port(addr);
        case kAdc0Spi:
            return Device.Adc(0).Spi(addr);
        case kAdc1Spi:
            return Device.Adc(1).Spi(addr);
        }
}

//---------------------------------------------------------------------------
//  XA_402::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  XA_402::Do_DStore(int dev, int addr, int value)
{
    const int kPll     = 0;
    const int kAdc0Spi = 1;
    const int kAdc1Spi = 2;

    switch (dev)
        {
        default:
            break;
        case kPll:
            RawClockDevice().Pll().Port(addr, value);
            break;
        case kAdc0Spi:
            Device.Adc(0).Spi(addr, value);
            break;
        case kAdc1Spi:
            Device.Adc(1).Spi(addr, value);
            break;
        }
}

//---------------------------------------------------------------------------
//  XA_402::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  XA_402::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;

    switch (dev)
        {
        default:
            break;
        case kPll:
            RawClockDevice().Pll().Frequency(freq);
            RawClockDevice().Pll().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  XA_402::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  XA_402::Do_DFreqActual(int dev)
{
    const int kPll = 0;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(RawClockDevice().Pll().FrequencyActual());
        }
}

}  // namespace Innovative
