// XA_500DeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XA_500_Mb.h>
#include <XA_500DeviceMap_Mb.h>
#include <XAFamilyMemoryMap_Mb.h>
#include <XA_500MemoryMap_Mb.h>
#include <FloatingPtSupport_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace XA_500MemoryMap;

//---------------------------------------------------------
//  Subdevice parameters Definition

//  Parameters for SubDevices
const int  XA_500_AdcLogicalDevices = 1;     // PIDs ?
const int  XA_500_AdcLogicalChannelsPerDevice = 2;
const int  XA_500_AdcChannels = XA_500_AdcLogicalDevices*XA_500_AdcLogicalChannelsPerDevice;

unsigned int  XA_500_AdcCalibrationDeviceCount = XA_500_AdcChannels;   // number of cal correction channels

const int     XA_500_AdcPhysicalDevices = 1;
const int     XA_500_AdcPhysicalChannelsPerDevice = 2;
unsigned int  XA_500_AdcSpiDeviceCount = XA_500_AdcPhysicalDevices;

const int  XA_500_DacLogicalDevices = 1;
const int  XA_500_DacLogicalChannelsPerDevice = 2;
const int  XA_500_DacChannels = XA_500_DacLogicalDevices*XA_500_DacLogicalChannelsPerDevice;

const int     XA_500_DacPhysicalDevices = 1;
const int     XA_500_DacPhysicalChannelsPerDevice = 2;
unsigned int  XA_500_DacSpiDeviceCount = XA_500_DacPhysicalDevices;

unsigned int  XA_500_DacCalibrationDeviceCount = XA_500_DacChannels;  // number of cal correction channels

const unsigned int XA_500_VeloInStreamCount   = 1;    // Number of Velocia Incoming PIDs
const unsigned int XA_500_VeloOutStreamCount   = 1;   // Number of Velocia Outgoing PIDs

//  Input Info Data
const float         InputMaxRate            = 500.0e6f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 16;
const          int  InputMax_mV             = 1030;
const          int  InputMin_mV             = -1030;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 256;

const unsigned int XA_500_Adc_One = 0x10000;    // representation of floating point 1.0

//  Output Info Data
const float         OutputMaxRate            = 615.0e6f;
const float         OutputMinRate            = 5.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 1000;
const          int  OutputMin_mV             = -1000;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 4;   // ?


const unsigned int XA_500_Dac_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romXA_500_SysInfoOffset        = 0;
const int romXA_500_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romXA_500_InputCoeffOffset       = 0;  // XA_500_AdcChannels * 2 coefficients plus 'calibrated' word
const int romXA_500_InputCoeffOffset_avail =  XA_500_AdcChannels * 2 + 1;

//  ...Dac Cal Idrom: DAC coefficients
const int romXA_500_OutputCoeffOffset       = 0;
const int romXA_500_OutputCoeffOffset_avail = XA_500_DacChannels * 2 + 1;   // XA_500_DacChannels * 2 coefficients plus 'calibrated' word

//===========================================================================
// Class XA_500_AnalogInInfo
//===========================================================================

class XA_500_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    XA_500_AnalogInInfo(XA_500DeviceMap * owner)
        :  Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~XA_500_AnalogInInfo()  {  }

    virtual float     MaxRate() const
        {
        return InputMaxRate;
        }
    virtual float         MinRate() const     {  return InputMinRate;  }
    virtual SpanInfo      Span() const        {  return FSpan;  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    XA_500DeviceMap * Owner;
    SpanInfo   FSpan;
};

//===========================================================================
// Class XA_500_AnalogOutInfo
//===========================================================================

class XA_500_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    XA_500_AnalogOutInfo(XA_500DeviceMap * owner)
         :  Owner(owner), FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~XA_500_AnalogOutInfo()  {  }

    virtual float     MaxRate() const
        {
            return OutputMaxRate;
        }
    virtual float         MinRate() const     {  return OutputMinRate;  }
    virtual SpanInfo      Span() const        {  return FSpan;  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    XA_500DeviceMap * Owner;
    SpanInfo   FSpan;
};

//=============================================================================
//  CLASS XA_500_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_500_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 100.0e6;

//---------------------------------------------------------------------------
//  constructor for class XA_500_Pll
//---------------------------------------------------------------------------

XA_500_Pll::XA_500_Pll(XA_500DeviceMap * owner, IAddressingSpace &space, Lmk04800Registers & reg)
    : Lmk04800(space, reg, DualIVco), Owner(owner)//, FMasterOutIdx(0)
{
    SetMasterOutputSelection(AdcMasterOutput());  // Use Adc Master as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);    // All Refs are 100.0MHz
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class XA_500_Pll
//---------------------------------------------------------------------------

XA_500_Pll::~XA_500_Pll()
{
}

//---------------------------------------------------------------------------
//  XA_500_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_500_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "XA_500_Pll::OnInit()");
    //
    //  Input Configuration
    SetInputConfig(0, icBipolar);
    SetInputConfig(1, icBipolar);

    //
    //  Set Source Configuration
    SourceInUse(0, true);   SetSourceMapping(0, IXAClockIo::rsInternal);     // clkin0 = internal
    SourceInUse(1, true);   SetSourceMapping(1, IXAClockIo::rsExternal);     // clkin1 = front panel external

    //
    //  Set Output Configuration
    OutputInUse(0,  true);     SetOutputConfig(0,  ocLvds);        // ADC_CLK
    OutputInUse(1,  false);    SetOutputConfig(1,  ocDisabled);    // (unused)
    OutputInUse(2,  true);     SetOutputConfig(2,  ocLvds);        // ADC_SYSREF
    OutputInUse(3,  false);    SetOutputConfig(3,  ocDisabled);    // (unused)
    OutputInUse(4,  true);     SetOutputConfig(4,  ocLvds);        // DAC_CLK
    OutputInUse(5,  false);    SetOutputConfig(5,  ocDisabled);    // (unused)
    OutputInUse(6,  false);    SetOutputConfig(6,  ocDisabled);    // (unused)
    OutputInUse(7,  true);     SetOutputConfig(7,  ocLvds);    	   // DAC_REFCLK
    OutputInUse(8,  true );    SetOutputConfig(8,  ocLvds);    	   // DAC_FPGA_CLK
    OutputInUse(9,  false);    SetOutputConfig(9,  ocDisabled);    // (unused)
    OutputInUse(10, true);     SetOutputConfig(10, ocLvds);        // ADC_FPGA_CLK
    OutputInUse(11, false);    SetOutputConfig(11, ocDisabled);    // (unused)

    FeedbackOutput(fbm_Clkout6);  // set output to use as PLL feedback in 0-delay modes

    //  Compensate for no pullup on PLL lock bits.
//    SetLockConfig(stpOutWpush_pull);
//    SetPll2LockConfig(stpOutWpush_pull);

    //  Set Reduced Frequency Limits for Outputs (Preset to 3200.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6;
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[8]  = 600.0e6;
//    OutputFrequencyLimit[12] = 600.0e6;

    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference
}

//---------------------------------------------------------------------------
//  XA_500_Pll::OnReapply()
//---------------------------------------------------------------------------

void  XA_500_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  XA_500_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  XA_500_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  XA_500_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  XA_500_Pll::DacMasterOutput()
{
    return 4;
}

//---------------------------------------------------------------------------
//  XA_500_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool XA_500_Pll::IsFeedbackOutputAdc()
{
    return false;
}

//---------------------------------------------------------------------------
//  XA_500_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  XA_500_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency(0,   adc_rate);  // ADC_CLK
    OutputFrequency(2,   adc_rate);  // ADC_SYSREF
}

//---------------------------------------------------------------------------
//  XA_500_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  XA_500_Pll::SetDacOutputRates(double dac_rate)
{
    OutputFrequency(4,  dac_rate);                     // DAC_CLK
    OutputFrequency(7,  DCLK(dac_rate, DacMode())/8);  // DAC_REFCLK
	OutputFrequency(8,  DCLK(dac_rate, DacMode()));    // DAC_FPGA_CLK
}

//---------------------------------------------------------------------------
//  XA_500_Pll::DCLK() -- Set DCLK rate by DAC rate
//---------------------------------------------------------------------------

double   XA_500_Pll::DCLK(double dac_rate, unsigned int mode)
{
    switch (mode)
        {
        case 0:  return dac_rate;      // max rate 615;
        case 1:  return dac_rate/2;    // max rate 1000;
        case 2:  return dac_rate/2;    // max rate 1230;
        case 3:  return dac_rate/4;    // max rate 1230;
        case 6:  return dac_rate/4;    // max rate 1230;
        default:
        case 7:  return dac_rate/8;    // max rate 1230;
        }
}

//---------------------------------------------------------------------------
//  XA_500_Pll::DacMode() -- Get current Dac Mode
//---------------------------------------------------------------------------

unsigned int   XA_500_Pll::DacMode()
{
    return Owner->CurrentDacInterpolationMode();
}

//=============================================================================
//  CLASS XA_500_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_500_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class XA_500_SioPll
//---------------------------------------------------------------------------

XA_500_SioPll::XA_500_SioPll(XA_500DeviceMap* owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  XA_500_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_500_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, true);     SetMultiSynthPowerDown(1, true);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, true);     SetMultiSynthPowerDown(3, true);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(1, diNoClk);   // override since not using output
    SetOutDividerSource(2, diNoClk);   // override since not using output
    SetOutDividerSource(3, diNoClk);   // override since not using output

    OutputInUse(0, true);    SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, false);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, false);
    OutputInUse(2, false);   SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, false);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, false);

    SetOutputTrim(0, ot_3p3lvds);
    SetOutputTrim(1, ot_3p3lvds);
    SetOutputTrim(2, ot_3p3lvds);
    SetOutputTrim(3, ot_3p3lvds);

    SetOutputVoltage(0, vo_3p3volts);
    SetOutputVoltage(1, vo_3p3volts);
    SetOutputVoltage(2, vo_3p3volts);   // actually off
    SetOutputVoltage(3, vo_3p3volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  XA_500_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  XA_500_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS XA_500DeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_500DeviceMap
//---------------------------------------------------------------------------

using namespace XA_MemoryMap;
using namespace XA_500MemoryMap;

namespace
{

XA_LMK_Clock::MapRegisters  LMK_ClockMapAddrs =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    mmAFE_VcxoControl,   //  AfeVcxoControl_Addr;
    };

Lmk04800Registers  XA_500_Lmk04800Registers =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    };

}

//---------------------------------------------------------------------------
//  constructor for class XA_500DeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

XA_500DeviceMap::XA_500DeviceMap(XA_500 * owner)
    :  XABidirDeviceMap(owner),
       Owner(owner),
       Clock_SDev(this, WB_AFE, LMK_ClockMapAddrs),
       FInput(new XA_500InputDevice(this, new XA_500_AnalogInInfo(this))),
       FOutput(new XA_500OutputDevice(this, new XA_500_AnalogOutInfo(this))),
       FDacInterpolationMode(0x0)
{
    //
    //  SubDevice Customization
    LocalConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &XA_500DeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &XA_500DeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &XA_500DeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &XA_500DeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &XA_500DeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &XA_500DeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &XA_500DeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class XA_500DeviceMap
//---------------------------------------------------------------------------

XA_500DeviceMap::~XA_500DeviceMap()
{
    delete FInput;
    FInput = 0;
    delete FOutput;
    FOutput = 0;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------
//  We can't make this part of virtual tree for ConstructCustomParts since
//   its being called in the constructor and the tree is not built yet.

void  XA_500DeviceMap::LocalConstructCustomParts()
{
    XABidirDeviceMap::ConstructCustomParts();
    Clock_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

//void  XA_500DeviceMap::ConstructCustomParts()
//{
//    XABidirDeviceMap::ConstructCustomParts();
//}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OpenHardware()
{
    XABidirDeviceMap::OpenHardware();
    Clock_SDev.OpenHardware();

    Input().OpenHardware();
    Output().OpenHardware();

}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "XA_500DeviceMap::OnStreamPreconfigHandler()");
//    Output().ConfigureChannelEnables();

    FInput->Cal().ApplyRomCoefficients();
    FOutput->Cal().ApplyRomCoefficients();

    // Family-common init
    XABidirDeviceMap::PreconfigureHardware();

    Output().ConfigureChannelEnables();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);

    Input().PreconfigureHardware();
    Output().PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X6_1000MDeviceMap::OnStreamConfigureHandler()");

    // Family-common init
    XABidirDeviceMap::ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);

    Input().ConfigureHardware();
    Output().ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    Periph_SDev.XARun(true);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_500DeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    Periph_SDev.XARun(false);

    //  Base class "on stream stop" behavior
    XABidirDeviceMap::StreamStopHardware();
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}


//---------------------------------------------------------------------------
//  XA_500DeviceMap::Input() -- Reference to output object
//---------------------------------------------------------------------------

XA_500InputDevice & XA_500DeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

XA_500OutputDevice & XA_500DeviceMap::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::Make_AfePllObject()
//---------------------------------------------------------------------------

Lmk04800 *  XA_500DeviceMap::MakeFmcAfePllObject()
{
    return new XA_500_Pll(this, WB_AFE, XA_500_Lmk04800Registers );
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  XA_500DeviceMap::MakePriClockDevice(PriDeviceCode which)
{
    if (which==priAdc)
        return new XAStandardPriClockDevice(this, 1);
    else
        return new XAStandardPriClockDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetAdcSpiDeviceCount() -- Get # of ADC Spi Devices
//---------------------------------------------------------------------------

unsigned int XA_500DeviceMap::GetAdcSpiDeviceCount()
{
    return XA_500_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetCalibrationAdcCount() -- Get # of ADC cal channels
//---------------------------------------------------------------------------

unsigned int XA_500DeviceMap::GetCalibrationAdcCount()
{
    return XA_500_AdcCalibrationDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  XA_500DeviceMap::GetAdcCalibrationOverride()
{
    return 0; //new XA500_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  XA_500DeviceMap::MakeFrameCalcDevice(FrameDeviceCode which)
{
    if (which==devAdc)
        return new XAStandardFrameCalcDevice(this, 1);
    else
        return new XAStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  XA_500DeviceMap::GetAnalogDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = XA_500_AdcLogicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_500_AdcLogicalChannelsPerDevice;
        }
    else
        {
        devices = XA_500_DacLogicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_500_DacLogicalChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  XA_500DeviceMap::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = XA_500_AdcPhysicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_500_AdcPhysicalChannelsPerDevice;
        }
    else
        {
        devices = XA_500_DacPhysicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_500_DacPhysicalChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  XA_500DeviceMap::GetDacSpiDeviceCount()
{
    return XA_500_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  XA_500DeviceMap::GetDacCalibrationOverride()
{
    return new XA500_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  XA_500DeviceMap::GetCalibrationDacCount()
{
    return XA_500_DacCalibrationDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  XA_500DeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new XA_500_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_500DeviceMap::GetVeloInStreamCount()
{
    return XA_500_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_500DeviceMap::GetVeloOutStreamCount()
{
    return XA_500_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   XA_500DeviceMap::GetPatternModeAlignment()
{
    return 16;   // standard XA alignment of 16 words
}

//---------------------------------------------------------------------------
//  XA_500DeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   XA_500DeviceMap::GetPatternModeScaleFactor()
{
    return 1;   // standard XA scale factor of 1 words
}

//=============================================================================
//  CLASS XA500_DacCalOverride  --  Cal Override for XA500 Dacs
//=============================================================================
//---------------------------------------------------------------------------
//  XA500_DacCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  XA500_DacCalOverride::OverrideOffset(unsigned int value)
{
    // trim to 16 bit value
    return value&0xFFFF;
}

//---------------------------------------------------------------------------
//  XA500_DacCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  XA500_DacCalOverride::OverrideGain(unsigned int value)
{
    const float HdwStep_mA    = 0.0225f;    // 22.5 uA steps
    const float Min_mA        = 8.64f;      // 0x000 gain value
    const float Center_mA     = 20.00f;     // 0x1f9 gain value


    float gain = static_cast<float>(value)/XA_500_Dac_One;

    float adjusted_mA = gain*Center_mA;
    float steps_mA = adjusted_mA - Min_mA;

    float counts = steps_mA / HdwStep_mA;
    unsigned int setting;
    if (counts < 0)
        setting = 0;
    else if (counts > 1023)
        setting = 0x3FF;
    else
        setting = static_cast<unsigned int>(counts);

    return setting;
}

//---------------------------------------------------------------------------
//  XA500_DacCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  XA500_DacCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Offset(value);
    else
        Owner->Output().Device().Write_Q_Offset(value);

}

//---------------------------------------------------------------------------
//  XA500_DacCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  XA500_DacCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Gain(value);
    else
        Owner->Output().Device().Write_Q_Gain(value);

}

//===========================================================================
//  CLASS XA_500InputDevice  -- XA_500 Input Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_500InputDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_500InputDevice
//---------------------------------------------------------------------------

XA_500InputDevice::XA_500InputDevice(XA_500DeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerXA(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romXA_500_InputCoeffOffset, XA_500_AdcChannels,
                       XA_500_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_500InputDevice
//---------------------------------------------------------------------------

XA_500InputDevice::~XA_500InputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  XA_500InputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  XA_500InputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  XA_500InputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  XA_500InputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  XA_500InputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  XA_500InputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  XA_500InputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  XA_500InputDevice::ConfigureHardware()
{
}

//===========================================================================
//  CLASS XA500_DacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class XA500_DacToHardware
//---------------------------------------------------------------------------

class XA500_DacToHardware :  public AdDac9122::IDacToHardware
{
public:
    virtual unsigned short  Spi(int addr)
        {  return Out->Spi(Dev, addr);  }
    virtual void            Spi(int addr, unsigned short value)
        {  Out->Spi(Dev, addr, value);  }

    virtual void  DacReset(bool state)
        {  Out->DacReset(state); }

    XA500_DacToHardware( XA_500OutputDevice * out, unsigned int dev=0 )
        :  Out(out), Dev(dev)
        {}
    virtual ~XA500_DacToHardware()
        {}

    XA_500OutputDevice *   Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS XA_500OutputDevice  -- XA_500 Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_500OutputDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_500OutputDevice
//---------------------------------------------------------------------------

XA_500OutputDevice::XA_500OutputDevice(XA_500DeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerXA(owner), FInfo(info),
      DacDev(new XA500_DacToHardware(this, 0)),
      CalRomSection(DacCalIdrom(), romXA_500_OutputCoeffOffset, XA_500_DacChannels,
                       XA_500_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_500OutputDevice
//---------------------------------------------------------------------------

XA_500OutputDevice::~XA_500OutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  XA_500OutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  XA_500OutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();

}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  XA_500OutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
    DacDev.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  XA_500OutputDevice::ConfigureHardware()
{
    OwnerXA->Debug.Log("DAC ConfigureHardware()");

    // Toggle the DacReset
    DacCommon().DacReset(false);
    DacCommon().DacReset(true);
    Sleep(1);
    DacCommon().DacReset(false);

    DacDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  XA_500OutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    //Device().Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1) );
    Device().ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device().ChannelEnabled(1, DacCommon().ChannelEnabled(1) );

}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  XA_500OutputDevice::SetDacInterpolation(DacInterpolationModeValues values)
{
    SetDacInterpolation(static_cast<unsigned int>(values));
}

//---------------------------------------------------------------------------
//  XA_500OutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  XA_500OutputDevice::SetDacInterpolation(unsigned int values)
{
    OwnerXA->FDacInterpolationMode = values&0x7;
}


} // namespace Innovative


