// XA_RXDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XA_RX_Mb.h>
#include <XA_RXDeviceMap_Mb.h>
#include <XAFamilyMemoryMap_Mb.h>
#include <XA_RXMemoryMap_Mb.h>
#include <FloatingPtSupport_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace XA_RXMemoryMap;

//---------------------------------------------------------
//  Subdevice parameters Definition

//  Parameters for SubDevices
const int  XA_RX_AdcLogicalDevices = 1;     // PIDs
const int  XA_RX_AdcLogicalChannelsPerDevice = 8;
const int  XA_RX_AdcChannels = XA_RX_AdcLogicalDevices*XA_RX_AdcLogicalChannelsPerDevice;

unsigned int  XA_RX_AdcCalibrationDeviceCount = XA_RX_AdcChannels;   // number of cal correction channels

const int     XA_RX_AdcPhysicalDevices = 2;
const int     XA_RX_AdcPhysicalChannelsPerDevice = 4;
unsigned int  XA_RX_AdcSpiDeviceCount = XA_RX_AdcPhysicalDevices;

const unsigned int XA_RX_VeloInStreamCount   = 1;    // Number of Velocia Incoming PIDs
const unsigned int XA_RX_VeloOutStreamCount   = 1;   // Number of Velocia Outgoing PIDs

//  Input Info Data
const float         InputMaxRate            = 125.0e6f;
const float         InputMinRate            = 0.0e6f;
const unsigned int  InputWidth_Bits         = 16;
const          int  InputMax_mV             = 1300;
const          int  InputMin_mV             = -1300;
const unsigned int  InputSamplesPerWord     = 2;
const unsigned int  InputTriggerGranularity = 256;

const unsigned int XA_RX_Adc_One = 0x10000;    // representation of floating point 1.0


//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romXA_RX_SysInfoOffset        = 0;
const int romXA_RX_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Adc Cal Idrom: ADC coefficients
const int romXA_RX_InputCoeffOffset       = 0;  // XA_RX_AdcChannels * 2 coefficients plus 'calibrated' word
const int romXA_RX_InputCoeffOffset_avail =  XA_RX_AdcChannels * 2 + 1;

//  ...Dac Cal Idrom: DAC coefficients
//const int romXA_RX_OutputCoeffOffset       = 0;
//const int romXA_RX_OutputCoeffOffset_avail = XA_RX_DacChannels * 2 + 1;   // XA_RX_DacChannels * 2 coefficients plus 'calibrated' word

//===========================================================================
// Class XA_RX_AnalogInInfo
//===========================================================================

class XA_RX_AnalogInInfo : public IX6AnalogIoInfo
{
public:
    XA_RX_AnalogInInfo(XA_RXDeviceMap * owner)
         :  Owner(owner), FSpan(InputMax_mV, InputMin_mV)
        {   }
    virtual ~XA_RX_AnalogInInfo()  {  }

    virtual float     MaxRate() const
        {
            return InputMaxRate;
        }
    virtual float         MinRate() const     {  return InputMinRate;  }
    virtual SpanInfo      Span() const        {  return FSpan;  }
    virtual unsigned int  Bits() const        {  return InputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return InputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return InputTriggerGranularity;  }

private:
    XA_RXDeviceMap * Owner;
    SpanInfo   FSpan;
};

//=============================================================================
//  CLASS XA_RX_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_RX_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 100.0e6;

//---------------------------------------------------------------------------
//  constructor for class XA_RX_Pll
//---------------------------------------------------------------------------

XA_RX_Pll::XA_RX_Pll(XA_RXDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg, omNestedZeroDelay, tsmLogic), Owner(owner)//, FMasterOutIdx(0)
{
    OutputSyncMode(osmSysRefSync);
    Allow_Pll1_FeedbackAdjustment();
    SetMasterOutputSelection(AdcMasterOutput());  // Use Adc Master as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);    // All Refs are 100.0MHz
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class XA_RX_Pll
//---------------------------------------------------------------------------

XA_RX_Pll::~XA_RX_Pll()
{
}

//---------------------------------------------------------------------------
//  XA_RX_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_RX_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "XA_RX_Pll::OnInit()");
    //
    //  Input Configuration
    SetInputConfig(0, icBipolar);
    SetInputConfig(1, icBipolar);
    SetInputConfig(2, icBipolar);

    //
    //  Set Source Configuration
    SourceInUse(0, true);   SetSourceMapping(0, IXAClockIo::rsInternal);     // clkin0 = internal
    SourceInUse(1, true);   SetSourceMapping(1, IXAClockIo::rsExternal);     // clkin1 = front panel external
    SourceInUse(2, true);   SetSourceMapping(2, IXAClockIo::rsP16);          // clkin2 = carrier

    //  Device Powered/Enabled Setup
    PowerDownOutputGroup( 2/2, true);
    PowerDownOutputGroup( 4/2, true);
    PowerDownOutputGroup( 6/2, true);
    //
    //  Set Output Configuration (assu
    OutputInUse(0,  true );    SetOutputConfig(0,  ocLvds);        //
    OutputInUse(1,  true );    SetOutputConfig(1,  ocLvds);        // FPGA_CLK
    OutputInUse(2,  false);    SetOutputConfig(2,  ocDisabled);    //
    OutputInUse(3,  false);    SetOutputConfig(3,  ocDisabled);    //
    OutputInUse(4,  false);    SetOutputConfig(4,  ocDisabled);    //
    OutputInUse(5,  false);    SetOutputConfig(5,  ocDisabled);    //
    OutputInUse(6,  false);    SetOutputConfig(6,  ocDisabled);    //
    OutputInUse(7,  false);    SetOutputConfig(7,  ocDisabled);    //
    OutputInUse(8,  false);    SetOutputConfig(8,  ocDisabled);    //
    OutputInUse(9,  false);    SetOutputConfig(9,  ocDisabled);    //
    OutputInUse(10, true );    SetOutputConfig(10, ocLvds);        //
    OutputInUse(11, true );    SetOutputConfig(11, ocLvds);        // ADC0 Sampling Clock
    OutputInUse(12, true );    SetOutputConfig(12, ocLvds);        //
    OutputInUse(13, true );    SetOutputConfig(13, ocLvds);        // ADC1 Sampling Clock

    FeedbackOutput(fbm_Sysref);  // set output to use sysref as feedback clock

    //  Compensate for no pullup on PLL lock bits.
//    SetLockConfig(stpOutWpush_pull);
//    SetPll2LockConfig(stpOutWpush_pull);

    //  Set Reduced Frequency Limits for Outputs (Preset to 3200.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6;
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[8]  = 600.0e6;
//    OutputFrequencyLimit[12] = 600.0e6;

    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference
}

//---------------------------------------------------------------------------
//  XA_RX_Pll::OnReapply()
//---------------------------------------------------------------------------

void  XA_RX_Pll::OnReapply()
{
    if (IsIn_ExtClock_Mode())
        {
        //
        //  Set Output Configuration for External Clock Mode
        SetOutput_AsCopy(1);            // same as 0
        SetOutput_AsCopy(11);           // same as 10
        SetOutput_AsCopy(13);           // same as 12
        }
    else
        {
        //
        //  Set Output Configuration for PLL Mode
        SetOutput_AsNonCopy(1);            // same as 10
        SetOutput_AsNonCopy(11);           // same as 10
        SetOutput_AsNonCopy(13);           // same as 12
        }

}

//---------------------------------------------------------------------------
//  XA_RX_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  XA_RX_Pll::AdcMasterOutput()
{
    if (IsIn_ExtClock_Mode())
        {
        return 10;   // calc on even channel to allow divide by 1
        }
    else
        {
        return 11;   // calc on SYSREF divider
        }
}

//---------------------------------------------------------------------------
//  XA_RX_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  XA_RX_Pll::DacMasterOutput()
{
    if (IsIn_ExtClock_Mode())
        {
        return 10;   // calc on even channel to allow divide by 1
        }
    else
        {
        return 11;   // calc on SYSREF divider
        }
}

//---------------------------------------------------------------------------
//  XA_RX_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool XA_RX_Pll::IsFeedbackOutputAdc()
{
    return true;
}

//---------------------------------------------------------------------------
//  XA_RX_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  XA_RX_Pll::SetAdcOutputRates(double adc_rate)
{
    OutputFrequency( 0,  adc_rate);   // copy source for 1 in EXT mode
    OutputFrequency(10,  adc_rate);   // copy source for 11
    OutputFrequency(12,  adc_rate);   // copy source for 13
    OutputFrequency( 1,  adc_rate);   // FPGA_CLK
    OutputFrequency(11,  adc_rate);  //  ADC0 Sampling Clock
    OutputFrequency(13,  adc_rate);  //  ADC1 Sampling Clock
}

//---------------------------------------------------------------------------
//  XA_RX_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  XA_RX_Pll::SetDacOutputRates(double /*dac_rate*/)
{
}


//=============================================================================
//  CLASS XA_RX_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_RX_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class XA_RX_SioPll
//---------------------------------------------------------------------------

XA_RX_SioPll::XA_RX_SioPll(XA_RXDeviceMap* owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  XA_RX_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_RX_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, true);     SetMultiSynthPowerDown(1, true);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, true);     SetMultiSynthPowerDown(3, true);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(1, diNoClk);   // override since not using output
    SetOutDividerSource(2, diNoClk);   // override since not using output
    SetOutDividerSource(3, diNoClk);   // override since not using output

    OutputInUse(0, true);    SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, false);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, false);
    OutputInUse(2, false);   SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, false);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, false);

    SetOutputTrim(0, ot_3p3lvds);
    SetOutputTrim(1, ot_3p3lvds);
    SetOutputTrim(2, ot_3p3lvds);
    SetOutputTrim(3, ot_3p3lvds);

    SetOutputVoltage(0, vo_3p3volts);
    SetOutputVoltage(1, vo_3p3volts);
    SetOutputVoltage(2, vo_3p3volts);   // actually off
    SetOutputVoltage(3, vo_3p3volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  XA_RX_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  XA_RX_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS XA_RXDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_RXDeviceMap
//---------------------------------------------------------------------------

using namespace XA_MemoryMap;
using namespace XA_RXMemoryMap;

namespace
{

XA_LMK20_Clock::MapRegisters  LMK20_ClockMapAddrs =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    mmAFE_VcxoControl,   //  AfeVcxoControl_Addr;
    };

Lmk04820Registers  XA_RX_Lmk04820Registers =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    };

XU_DacPatternModeSystem::MapRegisters XADacPatternRegAddrs =
    {
    mmDdr3Status,       //   PatternModeEnable_Addr
    mmArbPB_Control,    //   PlaybackCommand_Addr
    mmArbPB_Data0,      //   CmdFifo0_Addr
    mmArbPB_Data1,      //   CmdFifo1_Addr
    mmArbPB_Data2,      //   CmdFifo2_Addr
    mmArbPB_Data3,      //   CmdFifo3_Addr
    };

}

//---------------------------------------------------------------------------
//  constructor for class XA_RXDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


XA_RXDeviceMap::XA_RXDeviceMap(XA_RX * owner)
    :  XAAdcDeviceMap(owner),
       Owner(owner),
       Clock_SDev(this, WB_AFE, LMK20_ClockMapAddrs),
       Memory(this, WB_Periph, XADacPatternRegAddrs),
       FInput(new XA_RXInputDevice(this, new XA_RX_AnalogInInfo(this))),
       FDacInterpolationMode(0x0),
       AdcPHYReg(WB_AFE, mmAFE_PHYCalibration),
       AdcPHYStatus1(WB_AFE, mmAFE_PHYCalibration1),
       AdcPHYStatus2(WB_AFE, mmAFE_PHYCalibration2)//,
{
    //
    //  SubDevice Customization
    LocalConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &XA_RXDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &XA_RXDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &XA_RXDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &XA_RXDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &XA_RXDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &XA_RXDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &XA_RXDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class XA_RXDeviceMap
//---------------------------------------------------------------------------

XA_RXDeviceMap::~XA_RXDeviceMap()
{
    delete FInput;
    FInput = 0;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------
//  We can't make this part of virtual tree for ConstructCustomParts since
//   its being called in the constructor and the tree is not built yet.

void  XA_RXDeviceMap::LocalConstructCustomParts()
{
    XAAdcDeviceMap::ConstructCustomParts();
    Clock_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

//void  XA_RXDeviceMap::ConstructCustomParts()
//{
//    XAAdcDeviceMap::ConstructCustomParts();
//}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OpenHardware()
{
    XAAdcDeviceMap::OpenHardware();
    Clock_SDev.OpenHardware();

    Input().OpenHardware();

}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "XA_RXDeviceMap::OnStreamPreconfigHandler()");

    FInput->Cal().ApplyRomCoefficients();

    // Family-common init
    XAAdcDeviceMap::PreconfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);

    Input().PreconfigureHardware();
    //Output().PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "XA_RXDeviceMap::OnStreamConfigureHandler()");

    // Family-common init
    XAAdcDeviceMap::ConfigureHardware();

    Ad9653_ConfigureHardware(0);
    Ad9653_ConfigureHardware(1);


    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);

    Input().ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    Periph_SDev.XARun(true);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    Periph_SDev.XARun(false);

    //  Base class "on stream stop" behavior
    XAAdcDeviceMap::StreamStopHardware();
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}


//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Input() -- Reference to output object
//---------------------------------------------------------------------------

XA_RXInputDevice & XA_RXDeviceMap::Input()
{
    return *FInput;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

//XA_RXOutputDevice & XA_RXDeviceMap::Output()
//{
//    return *FOutput;
//}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Make_AfePllObject()
//---------------------------------------------------------------------------

Lmk04820 *  XA_RXDeviceMap::MakeFmcAfePllObject()
{
    return new XA_RX_Pll(this, WB_AFE, XA_RX_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  XA_RXDeviceMap::MakePriClockDevice(PriDeviceCode which)
{
    if (which==priAdc)
        return new XAStandardPriClockDevice(this, 1);
    else
        return new XAStandardPriClockDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetAdcSpiDeviceCount() -- Get # of ADC Spi Devices
//---------------------------------------------------------------------------

unsigned int XA_RXDeviceMap::GetAdcSpiDeviceCount()
{
    return XA_RX_AdcSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetCalibrationAdcCount() -- Get # of ADC cal channels
//---------------------------------------------------------------------------

unsigned int XA_RXDeviceMap::GetCalibrationAdcCount()
{
    return XA_RX_AdcCalibrationDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  XA_RXDeviceMap::GetAdcCalibrationOverride()
{
    return 0; //new XA160_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  XA_RXDeviceMap::MakeFrameCalcDevice(FrameDeviceCode which)
{
    if (which==devAdc)
        return new XAStandardFrameCalcDevice(this, 1);
    else
        return new XAStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::GetAnalogDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = XA_RX_AdcLogicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_RX_AdcLogicalChannelsPerDevice;
        }
    else
        {
        devices = 0;   // SID devices and channels INFO
        ch_per_dev = 0;
        }
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  XA_RXDeviceMap::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        devices = XA_RX_AdcPhysicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_RX_AdcPhysicalChannelsPerDevice;
        }
    else
        {
        devices = 0;   // SID devices and channels INFO
        ch_per_dev = 0;
        }
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  XA_RXDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new XA_RX_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Ad9653_ConfigureHardware() -- Configure Ad9653 device
//---------------------------------------------------------------------------
\
void XA_RXDeviceMap::Ad9653_ConfigureHardware(unsigned int device)
{
    Debug.Log("====================================================");
    Debug.Log("Ad9653_ConfigureHardware(), Device: ", device);

    unsigned int device_base = 4*device;
    bool should_cal_device = Input().ChannelEnabled(device_base+0) || Input().ChannelEnabled(device_base+1)
                            || Input().ChannelEnabled(device_base+2) || Input().ChannelEnabled(device_base+3);
    if (should_cal_device)
        for (unsigned int reps=0; reps<3; reps++)
            {
            Ad9653_Init(device);
            bool cal1_result = Ad9653_Cal_Stage1(device);
            if (cal1_result)
                if (Ad9653_Cal_Stage2(device))
                    break;
            Debug.Log("Restarting Configuration for Device: ", device);
            }
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Ad9653_Init() -- INIT ADC chip
//---------------------------------------------------------------------------

void XA_RXDeviceMap::Ad9653_Init(unsigned int device)
{
    Debug.Log("Ad9653_Init()");

    Debug.Log("Reset ADC");
    Input().Spi(device, 0x8, 0x3);
    Sleep(10);
    Input().Spi(device, 0x8, 0x0);

    Debug.Log("Soft reset ADC");
    Input().Spi(device, 0x0, 0x3C);

    Sleep(10);  // wait for completion

    unsigned int chip_id = Input().Spi(device, 0x1);
    unsigned int chip_grade = Input().Spi(device, 0x2);

    Debug.Log("ADC Chip ID: =0xB5?", chip_id, 16);
    Debug.Log("ADC Chip Grade: =0x6x?", chip_grade, 16);
    Debug.Log("Ad9653_Init() DONE");
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Ad9653_Cal_Stage1() -- CAL ADC chip
//---------------------------------------------------------------------------

bool XA_RXDeviceMap::Ad9653_Cal_Stage1(unsigned int device)
{
    Debug.Log("Ad9653_Cal_Stage1()");

    Debug.Log("Enable 1/0 bit toggle test...");
    Input().Spi(device, 0xD, 0x9);
    Sleep(1);

    Debug.Log("Enable Cal1..");
    const unsigned int phases = 2;
    const unsigned int phase = 1;
    unsigned int start_bit = (device) ? phases : 0;
    unsigned int bit = start_bit + (phase-1);   // device bits are together

    //Debug.Log("Bit select..", bit);
    AdcPHYReg.CalStart.Bit(bit, true);
    AdcPHYReg.CalStart.Bit(bit, false);

    // poll for done-ness
    CalDoneReady(bit);

    //unsigned int phy_reg = AdcPHYReg.Value();
    MemoryBitManipulator  MbmStatus(AdcPHYStatus1.Value());
    unsigned int status_start = (device) ? 9 : 0;
    unsigned int status = MbmStatus.Field(status_start, 9);

    //Debug.Log("AdcPHYReg (0x819): ", phy_reg, 16);
    Debug.Log("AdcPHY Cal1 Status : ", status, 16);
    bool retval(true);
    if (status != 0x1FF)
        {
        Debug.Log("Cal1 FAILED: ", status, 16);
        retval = false;
        }

    Debug.Log("Ad9653_Cal_Stage1() DONE");
    return retval;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Ad9653_Cal_Stage2() -- CAL ADC chip
//---------------------------------------------------------------------------

bool XA_RXDeviceMap::Ad9653_Cal_Stage2(unsigned int device)
{
    Debug.Log("Ad9653_Cal_Stage2()");

    Input().Spi(device, 0x19, 0xF0);
    Input().Spi(device, 0x1A, 0xF0);
    Input().Spi(device, 0x1B, 0xF0);
    Input().Spi(device, 0x1C, 0xF0);

    const unsigned int phases = 2;
    const unsigned int phase = 2;
    unsigned int start_bit = (device) ? phases : 0;
    unsigned int bit = start_bit + (phase-1);   // device bits are together
    //Debug.Log("Bit select..", bit);

    for (unsigned int i=0; i<9; i++)
        {
        Input().Spi(device, 0xD, 0x1);

        Debug.Log("Enable Cal2..");
        AdcPHYReg.CalStart.Bit(bit, true);
        AdcPHYReg.CalStart.Bit(bit, false);

        Input().Spi(device, 0xD, 0x88);
        if (AdcPHYReg.CalDone.Bit(bit)==false)
            Debug.Log("Failed Step ", i);

        //unsigned int phy_reg = AdcPHYReg.Value();
        MemoryBitManipulator  MbmStatus(AdcPHYStatus2.Value());
        unsigned int status_start = (device) ? 9 : 0;
        unsigned int status = MbmStatus.Field(status_start, 9);

        //Debug.Log("AdcPHYReg (0x819): ", phy_reg, 16);
        Debug.Log("AdcPHY Cal 2 Status : ", status, 16);
        if (status == 0x1FF)
            {
            Debug.Log("All bits aligned, exiting Cal2...");
            break;
            }
        }

    MemoryBitManipulator  MbmFinalStatus(AdcPHYStatus2.Value());
    unsigned int fs_start = (device) ? 9 : 0;
    unsigned int final_status = MbmFinalStatus.Field(fs_start, 9);
    bool  cal_failed(false);

    if (final_status != 0x1FF)
        {
        Debug.Log("Cal2 FAILED: ", final_status, 16);
        cal_failed = true;
        }

    // Enable normal data mode
    Input().Spi(device, 0xD, 0x0);

    Debug.Log("Ad9653_Cal_Stage2() DONE");
    return !cal_failed;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::CalDoneReady()
//---------------------------------------------------------------------------

void XA_RXDeviceMap::CalDoneReady(unsigned int bit)
{
    // Wait up to 10ms for pending Cal to complete

    for (unsigned int i = 0; i < 10; ++i)
        {
        if (AdcPHYReg.CalDone.Bit(bit)==true)
            return;
        else
            Innovative::Sleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " Ad9653_Cal Ready Failure (>10ms)";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::Ad9653_EndCal() -- End ADC chip calibration
//---------------------------------------------------------------------------

void XA_RXDeviceMap::Ad9653_EndCal(unsigned int device)
{
    Input().Spi(device, 0xD, 0x0);
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_RXDeviceMap::GetVeloInStreamCount()
{
    return XA_RX_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_RXDeviceMap::GetVeloOutStreamCount()
{
    return XA_RX_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   XA_RXDeviceMap::GetPatternModeAlignment()
{
    return 16;   // standard XA alignment of 16 words
}

//---------------------------------------------------------------------------
//  XA_RXDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   XA_RXDeviceMap::GetPatternModeScaleFactor()
{
    return 1;   // standard XA scale factor of 1 words
}

//===========================================================================
//  CLASS XA_RXInputDevice  -- XA_RX Input Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_RXInputDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_RXInputDevice
//---------------------------------------------------------------------------

XA_RXInputDevice::XA_RXInputDevice(XA_RXDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerXA(owner), FInfo(info),
      CalRomSection(AdcCalIdrom(), romXA_RX_InputCoeffOffset, XA_RX_AdcChannels,
                       XA_RX_Adc_One, AdcCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_RXInputDevice
//---------------------------------------------------------------------------

XA_RXInputDevice::~XA_RXInputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  XA_RXInputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  XA_RXInputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  XA_RXInputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  XA_RXInputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  XA_RXInputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  XA_RXInputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
}

//---------------------------------------------------------------------------
//  XA_RXInputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  XA_RXInputDevice::ConfigureHardware()
{
}

} // namespace Innovative


