// XA_RXDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XA_RXDeviceMap_MbH
#define XA_RXDeviceMap_MbH

#include <XA_RX_Mb.h>
#include <XASubDevice_Stream_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class XA_RXDeviceMap;

//==============================================================================
//  CLASS XA_RX_Pll -- XA_RX PLL Chip (Specialization of Lmk04820)
//==============================================================================

class XA_RX_Pll : public Lmk04820
{
public:
    XA_RX_Pll(XA_RXDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~XA_RX_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 40e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XA_RXDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

private:
    //unsigned int  DacMode();
    //double        DCLK(double dac_rate, unsigned int mode);
};


//==============================================================================
//  CLASS XA_RX_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class XA_RX_SioPll : public Si5338
{
public:
    XA_RX_SioPll(XA_RXDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    XA_RXDeviceMap * Owner;
};

//===========================================================================
//  CLASS ISuppliesXA_RXRequirements -- Device Map Requirement List
//===========================================================================

class ISuppliesXA_RXRequirements : public IRequires_Nothing,
                                    public IRequires_Lmk20FmcClocks,
                                    public IRequires_XUPatternMode
{
public:
    virtual ~ISuppliesXA_RXRequirements()
        {}

};

//===========================================================================
//  CLASS XA_RXDeviceMap  -- Register & low level support for XA_RX module
//===========================================================================
class XA_RX;

class XA_RXDeviceMap :  public XAAdcDeviceMap, public ISuppliesXA_RXRequirements

{
    friend class XA_RX;
    friend class XA_RXInputDevice;
    //friend class XA_RXOutputDevice;
//    friend class XA_RX_Pll;

public:
    //
    //  Member Functions
    XA_RXDeviceMap(XA_RX * owner);
    virtual ~XA_RXDeviceMap();

    XA_LMK20_Clock::XAClockIo & Clock()  { return Clock_SDev.Clock(); }

    class XA_RXInputDevice &  Input();



public: // pseudo-protected

    unsigned int      CurrentDacInterpolationMode()
        {  return FDacInterpolationMode;  }

//    virtual double   Get_DacClockRate()  { return Clock().Dac().Frequency(); }

protected:
    // Data
    XA_RX *         Owner;
    //
    //  Subdevices
    XA_LMK20_Clock        Clock_SDev;

    XU_DacPatternModeSystem  Memory;

    XA_RXInputDevice *     FInput;
    //XA_RXOutputDevice *    FOutput;

    Trace             Debug;

    // Methods
//    virtual void    ConstructCustomParts();            // board construtor time
    virtual void    OpenHardware();                    // after open time
//    virtual void    StreamStopHardware();              // after stream stop subdevice configuration

    virtual double  StandardClockFrequency()
                        {  return Clock().Frequency();   }
    virtual double  Get_DacClockFrequency()
                        {  return Clock().Dac().Frequency();   }

    //  Subdevice Customization Handlers
    //  Type1 Chrono
    virtual Lmk04820 *                  MakeFmcAfePllObject();

    //  XA Adc Device Mixin
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();

    //  XA Dac Device Mixin
//    virtual unsigned int                GetDacSpiDeviceCount();
//    virtual unsigned int                GetCalibrationDacCount();
//    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    //  Stream Configuration
    virtual unsigned int                GetVeloInStreamCount();
    virtual unsigned int                GetVeloOutStreamCount();

    //  Pattern Mode
    virtual unsigned int                GetPatternModeScaleFactor();
    virtual unsigned int                GetPatternModeAlignment();

    // Aurora
    virtual Si5338 *                    MakeSioPllObject(I2CDevice & device);


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_RXDeviceMap::AdcPHYCalibrationRegister -- Cal ADC chip
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcPHYCalibrationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  CalStart;
        RegisterBitGroup  CalDone;

    public:
        AdcPHYCalibrationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               CalStart(*this, 0, 4),   CalDone(*this, 16, 4)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XA_RXDeviceMap::AdcPHYResultsRegister -- ADC Cal Results
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcPHYResultsRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Status;

    public:
        AdcPHYResultsRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Status(*this, 0, 18)
            {
            }
    };

    void  Ad9653_ConfigureHardware(unsigned int device);
    void  Ad9653_Init(unsigned int device);
    bool  Ad9653_Cal_Stage1(unsigned int device);
    bool  Ad9653_Cal_Stage2(unsigned int device);

    void  CalDoneReady(unsigned int bit);
    void  Ad9653_EndCal(unsigned int device);

private:
    unsigned int  FDacInterpolationMode;

    AdcPHYCalibrationRegister  AdcPHYReg;
    AdcPHYResultsRegister      AdcPHYStatus1;
    AdcPHYResultsRegister      AdcPHYStatus2;

    void     LocalConstructCustomParts();

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void     ConfigureDacs();
};

//===========================================================================
//  CLASS XA_RXInputDevice  -- XA_RX Input Devices
//===========================================================================

class XA_RXInputDevice : public XAInputDevice
{
public:
    typedef XAInputDevice inherited;

    XA_RXInputDevice(XA_RXDeviceMap* owner , IX6AnalogIoInfo* info);
    virtual ~XA_RXInputDevice();

    virtual IX6AnalogIoInfo &  Info();

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

private:
    XA_RXDeviceMap *           OwnerXA;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     AdcCalIdrom()  {  return OwnerXA->AdcCal_PG;  }

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
