// XA_TXDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XA_TX_Mb.h>
#include <XA_TXDeviceMap_Mb.h>
#include <XAFamilyMemoryMap_Mb.h>
#include <XA_TXMemoryMap_Mb.h>
#include <FloatingPtSupport_Mb.h>
#include <SystemSupport_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

using namespace XA_TXMemoryMap;

//---------------------------------------------------------
//  Subdevice parameters Definition

//  Parameters for SubDevices
const int  XA_TX_DacLogicalDevices = 2;
const int  XA_TX_DacLogicalChannelsPerDevice = 4;
const int  XA_TX_DacChannels = XA_TX_DacLogicalDevices*XA_TX_DacLogicalChannelsPerDevice;

const int     XA_TX_DacPhysicalDevices = 2;
const int     XA_TX_DacPhysicalChannelsPerDevice = 4;
unsigned int  XA_TX_DacSpiDeviceCount = XA_TX_DacPhysicalDevices;

unsigned int  XA_TX_DacCalibrationDeviceCount = XA_TX_DacChannels;  // number of cal correction channels

const unsigned int XA_TX_VeloInStreamCount   = 1;    // Number of Velocia Incoming PIDs
const unsigned int XA_TX_VeloOutStreamCount   = 1;   // Number of Velocia Outgoing PIDs

//  Output Info Data
const float         OutputMaxRate            = 312.5e6f;
const float         OutputMinRate            = 1.0e6f;
const unsigned int  OutputWidth_Bits         = 16;
const          int  OutputMax_mV             = 500;
const          int  OutputMin_mV             = -500;
const unsigned int  OutputSamplesPerWord     = 2;
const unsigned int  OutputTriggerGranularity = 4;   // ?


const unsigned int XA_TX_Dac_One = 0x10000;    // representation of floating point 1.0

//---------------------------------------------------------
//  IDROM Section Information
//  ...System Section: Standard Data only
const int romXA_TX_SysInfoOffset        = 0;
const int romXA_TX_SysInfoOffset_avail  = 18;  // first 'free' word

//  ...Dac Cal Idrom: DAC coefficients
const int romXA_TX_OutputCoeffOffset       = 0;
const int romXA_TX_OutputCoeffOffset_avail = XA_TX_DacChannels * 2 + 1;   // XA_TX_DacChannels * 2 coefficients plus 'calibrated' word

//===========================================================================
// Class XA_TX_AnalogOutInfo
//===========================================================================

class XA_TX_AnalogOutInfo : public IX6AnalogIoInfo
{
public:
    XA_TX_AnalogOutInfo(XA_TXDeviceMap * owner)
         :  Owner(owner), FSpan(OutputMax_mV, OutputMin_mV)
        {   }
    virtual ~XA_TX_AnalogOutInfo()  {  }

    virtual float     MaxRate() const
        {
            return OutputMaxRate;
        }
    virtual float         MinRate() const     {  return OutputMinRate;  }
    virtual SpanInfo      Span() const        {  return FSpan;  }
    virtual unsigned int  Bits() const        {  return OutputWidth_Bits;  }

    virtual unsigned int  SamplesPerWord() const {  return OutputSamplesPerWord;  }
    virtual unsigned int  TriggerFrameGranularity() const   {  return OutputTriggerGranularity;  }

private:
    XA_TXDeviceMap * Owner;
    SpanInfo   FSpan;
};

//=============================================================================
//  CLASS XA_TX_Pll  --  PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_TX_Pll
//---------------------------------------------------------------------------

const float  Input_ReferenceClk = 100.0e6;

//---------------------------------------------------------------------------
//  constructor for class XA_TX_Pll
//---------------------------------------------------------------------------

XA_TX_Pll::XA_TX_Pll(XA_TXDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg)
    : Lmk04820(space, reg, omNestedZeroDelay, tsmLogic), Owner(owner)//, FMasterOutIdx(0)
{
    OutputSyncMode(osmSysRefSync);
    Allow_Pll1_FeedbackAdjustment();
    SetMasterOutputSelection(AdcMasterOutput());  // Use Adc Master as guideline output

    ReferenceFrequency(0, Input_ReferenceClk);    // All Refs are 100.0MHz
    ReferenceFrequency(1, Input_ReferenceClk);
    ReferenceFrequency(2, Input_ReferenceClk);
}

//---------------------------------------------------------------------------
//  destructor for class XA_TX_Pll
//---------------------------------------------------------------------------

XA_TX_Pll::~XA_TX_Pll()
{
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_TX_Pll::OnInit()
{
    ScopedTraceRef ccc(Debug, "XA_TX_Pll::OnInit()");
    //
    //  Input Configuration
    SetInputConfig(0, icBipolar);
    SetInputConfig(1, icBipolar);
    SetInputConfig(2, icBipolar);

    //
    //  Set Source Configuration
    SourceInUse(0, true);   SetSourceMapping(0, IXAClockIo::rsInternal);     // clkin0 = internal
    SourceInUse(1, true);   SetSourceMapping(1, IXAClockIo::rsExternal);     // clkin1 = front panel external
    SourceInUse(2, true);   SetSourceMapping(2, IXAClockIo::rsP16);          // clkin2 = rsP16

    //
    //  Set Output Configuration
    OutputInUse(0,  true);    SetOutputConfig(0,  ocLvPecl_1600);   // DAC0_CLK
    OutputInUse(1,  true);    SetOutputConfig(1,  ocLvPecl_1600);   // DAC0_OSTR_CLK
    OutputInUse(2,  true);    SetOutputConfig(2,  ocLvds);          // DAC0_FPGA_CLK
    OutputInUse(3,  true);    SetOutputConfig(3,  ocLvds);          // DAC0_FPGA_OSTR_CLK
    OutputInUse(4,  true);    SetOutputConfig(4,  ocLvds);          // DAC1_FPGA_CLK
    OutputInUse(5,  true);    SetOutputConfig(5,  ocLvds);          // DAC1_FPGA_OSTR_CLK
    OutputInUse(6,  true);    SetOutputConfig(6,  ocDisabled);      // feedback
    OutputInUse(7,  false);   SetOutputConfig(7,  ocDisabled);      // (unused)
    OutputInUse(8,  true );   SetOutputConfig(8,  ocLvPecl_1600);   // DAC1_CLK
    OutputInUse(9,  true);    SetOutputConfig(9,  ocLvPecl_1600);   // DAC1_OSTR_CLK
    OutputInUse(10, false);   SetOutputConfig(10, ocDisabled);      // (unused)
    OutputInUse(11, false);   SetOutputConfig(11, ocDisabled);      // (unused)
    OutputInUse(12, false);   SetOutputConfig(12, ocDisabled);      // (unused)
    OutputInUse(13, false);   SetOutputConfig(13, ocDisabled);      // (unused)

    FeedbackOutput(fbm_Dclk6);  // set output to use as PLL feedback in 0-delay modes

    //  Compensate for no pullup on PLL lock bits.
//    SetLockConfig(stpOutWpush_pull);
//    SetPll2LockConfig(stpOutWpush_pull);

    //  Set Reduced Frequency Limits for Outputs (Preset to 3200.0 MHz)
    //  to set a limit, use a line like "OutputFrequencyLimit[6] = 100.0e6;
    //      if the physical rate limit is 100 MHz
//    OutputFrequencyLimit[8]  = 600.0e6;
//    OutputFrequencyLimit[12] = 600.0e6;

    //
    //  Reference
//    SetExternalReferenceControl(true);  // Use Internal Select
    ReferenceSource(refPrimary);    // Set to use primary reference
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::OnReapply()
//---------------------------------------------------------------------------

void  XA_TX_Pll::OnReapply()
{
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::AdcMasterOutput() - choose master if ADC is Master
//---------------------------------------------------------------------------

unsigned int  XA_TX_Pll::AdcMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::DacMasterOutput() - choose master if Dac is Master
//---------------------------------------------------------------------------

unsigned int  XA_TX_Pll::DacMasterOutput()
{
    return 0;
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::IsFeedbackOutputAdc() - which group is Feedback Out in
//---------------------------------------------------------------------------

bool XA_TX_Pll::IsFeedbackOutputAdc()
{
    return false;
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::SetAdcOutputRates() -- Set ADC group frequencies
//---------------------------------------------------------------------------
//  ADC/DAC clock outputs must be set in unison, at fixed ratios

void  XA_TX_Pll::SetAdcOutputRates(double adc_rate)
{
    //OutputFrequency(4,   adc_rate);  // ADC_CLKIN
    OutputFrequency(6,   adc_rate);  // feedback
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::SetDacOutputRates() -- Set DAC group frequencies
//---------------------------------------------------------------------------

void  XA_TX_Pll::SetDacOutputRates(double dac_rate)
{
    OutputFrequency(0,   dac_rate);     // DAC0_CLK
    OutputFrequency(1,   dac_rate);     // DAC0_OSTR_CLK
    OutputFrequency(2,   dac_rate);     // DAC0_FGPA_CLK
    OutputFrequency(3,   dac_rate);     // DAC0_FGPA_OSTR_CLK
    OutputFrequency(4,   dac_rate);     // DAC1_FPGA_CLK
    OutputFrequency(5,   dac_rate);     // DAC1_FPGA_OSTR_CLK
    OutputFrequency(8,   dac_rate);     // DAC1_CLK
    OutputFrequency(9,   dac_rate);     // DAC1_OSTR_CLK
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::DCLK() -- Set DCLK rate by DAC rate
//---------------------------------------------------------------------------

double   XA_TX_Pll::DCLK(double dac_rate, unsigned int mode)
{
    switch (mode)
        {
        case 0:  return dac_rate;      // max rate 615;
        case 1:  return dac_rate/2;    // max rate 1000;
        case 2:  return dac_rate/2;    // max rate 1230;
        case 3:  return dac_rate/4;    // max rate 1230;
        case 6:  return dac_rate/4;    // max rate 1230;
        default:
        case 7:  return dac_rate/8;    // max rate 1230;
        }
}

//---------------------------------------------------------------------------
//  XA_TX_Pll::DacMode() -- Get current Dac Mode
//---------------------------------------------------------------------------

unsigned int   XA_TX_Pll::DacMode()
{
    return Owner->CurrentDacInterpolationMode();
}

//=============================================================================
//  CLASS XA_TX_SioPll  --  SIO PLL timebase control
//=============================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_TX_SioPll
//---------------------------------------------------------------------------

const float  SioPll_ReferenceClk = 10.0e6;

//---------------------------------------------------------------------------
//  constructor for class XA_TX_SioPll
//---------------------------------------------------------------------------

XA_TX_SioPll::XA_TX_SioPll(XA_TXDeviceMap* owner, I2CDevice & device)
    : Si5338(device), Owner(owner)
{
    SetMasterOutputSelection(0);  // Use Output 0 as guideline output

    ReferenceFrequency(0, SioPll_ReferenceClk);  // All Refs are 10
    ReferenceFrequency(1, SioPll_ReferenceClk);
    ReferenceFrequency(2, SioPll_ReferenceClk);
    ReferenceFrequency(3, SioPll_ReferenceClk);

    ReferenceSource(ref3SingleEnded);  // Use IN3 as input clock

    Debug.Enabled(false);  // turn off verbose object output
}

//---------------------------------------------------------------------------
//  XA_TX_SioPll::OnInit() -- inital configuration
//---------------------------------------------------------------------------

void  XA_TX_SioPll::OnInit()
{
    DenomForN(1);
    DenomForM(0, 1);     // match settings for utility
    DenomForM(1, 1);
    DenomForM(2, 1);
    DenomForM(3, 1);

    //  PLL enabled by default in SettingsInit()
    //  Device Powered/Enabled Setup
    SetClkPowerDown(0, false);    SetMultiSynthPowerDown(0, false);
    SetClkPowerDown(1, true);     SetMultiSynthPowerDown(1, true);
    SetClkPowerDown(2, true);     SetMultiSynthPowerDown(2, true);
    SetClkPowerDown(3, true);     SetMultiSynthPowerDown(3, true);
    //
    //  Set Input Configuration
    SetInputMuxConfig(ra3SingleEnded, SioPll_ReferenceClk);  // rate needed if ra12Xtal used
    SetFeedbackMuxConfig(fbNoClk);
    SetPllReferenceInput(prRefClk);
    SetFeedbackSource(false, efbNoClk);     // use internal feedback
    //
    //  Set Output Configuration
    //  SetOutDividerSource(0, N) (Use standard setting of from SettingsInit()
    SetOutDividerSource(1, diNoClk);   // override since not using output
    SetOutDividerSource(2, diNoClk);   // override since not using output
    SetOutDividerSource(3, diNoClk);   // override since not using output

    OutputInUse(0, true);    SetOutputConfig(0, ocLvds, oiNormal);  OutputEnabled(0, true);
    OutputInUse(1, false);   SetOutputConfig(1, ocLvds, oiNormal);  OutputEnabled(1, false);
    OutputInUse(2, false);   SetOutputConfig(2, ocLvds, oiNormal);  OutputEnabled(2, false);
    OutputInUse(3, false);   SetOutputConfig(3, ocLvds, oiNormal);  OutputEnabled(3, false);

    SetOutputTrim(0, ot_3p3lvds);
    SetOutputTrim(1, ot_3p3lvds);
    SetOutputTrim(2, ot_3p3lvds);
    SetOutputTrim(3, ot_3p3lvds);

    SetOutputVoltage(0, vo_3p3volts);
    SetOutputVoltage(1, vo_3p3volts);
    SetOutputVoltage(2, vo_3p3volts);   // actually off
    SetOutputVoltage(3, vo_3p3volts);

    SetOutputDisableState(0, odHighZ);
    SetOutputDisableState(1, odHighZ);
    SetOutputDisableState(2, odHighZ);
    SetOutputDisableState(3, odHighZ);

    OutputFrequency(0, 100.0e6);
    OutputFrequency(1, 100.0e6);
    OutputFrequency(2, 100.0e6);  // actually off
    OutputFrequency(3, 100.0e6);

}

//---------------------------------------------------------------------------
//  XA_TX_SioPll::OnReapply()
//---------------------------------------------------------------------------

void  XA_TX_SioPll::OnReapply()
{
}

//===========================================================================
//  CLASS XA_TXDeviceMap  -- Register & low level support for SD module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_TXDeviceMap
//---------------------------------------------------------------------------

using namespace XA_MemoryMap;
using namespace XA_TXMemoryMap;

namespace
{

XA_LMK20_Clock::MapRegisters  LMK20_ClockMapAddrs =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    mmAFE_VcxoControl,   //  AfeVcxoControl_Addr;
    };

Lmk04820Registers  XA_TX_Lmk04820Registers =
    {
    mmAFE_PllControl,    //  AfePllControl_Addr
    mmAFE_PllSpi,        //  AfePllSpi_Addr
    };

}

//---------------------------------------------------------------------------
//  constructor for class XA_TXDeviceMap
//---------------------------------------------------------------------------

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

XA_TXDeviceMap::XA_TXDeviceMap(XA_TX * owner)
    :  XADacDeviceMap(owner),
       Owner(owner),
       Clock_SDev(this, WB_AFE, LMK20_ClockMapAddrs),
       FOutput(new XA_TXOutputDevice(this, new XA_TX_AnalogOutInfo(this))),
       FDacInterpolationMode(0x0)
{
    //
    //  SubDevice Customization
    LocalConstructCustomParts();
    //
    //  Hook stream events from base class
    PDMPtr()->OnStreamPreconfig.SetEvent(this,   &XA_TXDeviceMap::OnStreamPreconfigHandler );
    PDMPtr()->OnStreamSingleTimeAtStartConfig.SetEvent(this, &XA_TXDeviceMap::OnStreamSingleTimeAtStartConfigHandler );
    PDMPtr()->OnStreamConfigure.SetEvent(this,   &XA_TXDeviceMap::OnStreamConfigureHandler );
    PDMPtr()->OnBeforeStreamStart.SetEvent(this, &XA_TXDeviceMap::OnBeforeStreamStartHandler );
    PDMPtr()->OnAfterStreamStart.SetEvent(this,   &XA_TXDeviceMap::OnAfterStreamStartHandler );
    PDMPtr()->OnBeforeStreamStop.SetEvent(this,   &XA_TXDeviceMap::OnBeforeStreamStopHandler );
    PDMPtr()->OnAfterStreamStop.SetEvent(this,   &XA_TXDeviceMap::OnAfterStreamStopHandler );

}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

//---------------------------------------------------------------------------
//  destructor for class XA_TXDeviceMap
//---------------------------------------------------------------------------

XA_TXDeviceMap::~XA_TXDeviceMap()
{
    delete FOutput;
    FOutput = 0;
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::LocalConstructCustomParts()
//---------------------------------------------------------------------------
//  We can't make this part of virtual tree for ConstructCustomParts since
//   its being called in the constructor and the tree is not built yet.

void  XA_TXDeviceMap::LocalConstructCustomParts()
{
    XADacDeviceMap::ConstructCustomParts();
    Clock_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

//void  XA_TXDeviceMap::ConstructCustomParts()
//{
//    XADacDeviceMap::ConstructCustomParts();
//}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OpenHardware()
{
    XADacDeviceMap::OpenHardware();
    Clock_SDev.OpenHardware();

    Output().OpenHardware();

}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnStreamPreconfigHandler() - preconfigure operations
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "XA_TXDeviceMap::OnStreamPreconfigHandler()");
//    Output().ConfigureChannelEnables();

    FOutput->Cal().ApplyRomCoefficients();

    // Family-common init
    XADacDeviceMap::PreconfigureHardware();

    Output().ConfigureChannelEnables();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamPreconfig(Event);

    Output().PreconfigureHardware();

}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnStreamSingleTimeAtStartConfigHandler()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamSingleTimeAtStartConfig(Event);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnStreamConfigureHandler()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnStreamConfigureHandler(OpenWire::NotifyEvent & Event)
{
    ScopedTraceRef ccc(Debug, "X6_1000MDeviceMap::OnStreamConfigureHandler()");

    // Family-common init
    XADacDeviceMap::ConfigureHardware();

    //  Forward notify to baseboard handler
    OwnerCB()->CallOnStreamConfigure(Event);

    Output().ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnBeforeStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStart(Event);
    //
    //  Start execution
    Periph_SDev.XARun(true);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnAfterStreamStartHandler()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStart(Event);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnBeforeStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnBeforeStreamStop(Event);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::OnAfterStreamStopHandler()
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event)
{
    Periph_SDev.XARun(false);

    //  Base class "on stream stop" behavior
    XADacDeviceMap::StreamStopHardware();
    //  Forward notify to baseboard handler
    OwnerCB()->CallOnAfterStreamStop(Event);
}


//---------------------------------------------------------------------------
//  XA_TXDeviceMap::Output() -- Reference to output object
//---------------------------------------------------------------------------

XA_TXOutputDevice & XA_TXDeviceMap::Output()
{
    return *FOutput;
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::Make_AfePllObject()
//---------------------------------------------------------------------------

Lmk04820 *  XA_TXDeviceMap::MakeFmcAfePllObject()
{
    return new XA_TX_Pll(this, WB_AFE, XA_TX_Lmk04820Registers );
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::MakePriClockDevice() -- Get Pri Device object
//---------------------------------------------------------------------------

IPriClockDevice *  XA_TXDeviceMap::MakePriClockDevice(PriDeviceCode which)
{
    if (which==priAdc)
        return new XAStandardPriClockDevice(this, 1);
    else
        return new XAStandardPriClockDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetAdcSpiDeviceCount() -- Get # of ADC Spi Devices
//---------------------------------------------------------------------------

//unsigned int XA_TXDeviceMap::GetAdcSpiDeviceCount()
//{
//    return XA_TX_AdcSpiDeviceCount;
//}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetCalibrationAdcCount() -- Get # of ADC cal channels
//---------------------------------------------------------------------------

//unsigned int XA_TXDeviceMap::GetCalibrationAdcCount()
//{
//    return XA_TX_AdcCalibrationDeviceCount;
//}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetAdcCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

//ICalibrationValueOverride *  XA_TXDeviceMap::GetAdcCalibrationOverride()
//{
//    return 0; //new XA_TX_DacCalOverride(this);
//}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::MakeFrameCalcDevice() -- Get Frame Calc Device object
//---------------------------------------------------------------------------

IFrameCalcDevice *  XA_TXDeviceMap::MakeFrameCalcDevice(FrameDeviceCode which)
{
    if (which==devAdc)
        return new XAStandardFrameCalcDevice(this, 1);
    else
        return new XAStandardFrameCalcDevice(this, 1);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetAnalogDeviceInfo() -- Return devices, channels on device
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::GetAnalogDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        //devices = XA_TX_AdcLogicalDevices;   // SID devices and channels INFO
        //ch_per_dev = XA_TX_AdcLogicalChannelsPerDevice;
        }
    else
        {
        devices = XA_TX_DacLogicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_TX_DacLogicalChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetAnalogPhysicalDeviceInfo() -- Return devices, channels on PHYSICAL device
//---------------------------------------------------------------------------

void  XA_TXDeviceMap::GetAnalogPhysicalDeviceInfo(FrameDeviceCode which,
                            unsigned int & devices, unsigned int & ch_per_dev)
{
    if (which==devAdc)
        {
        //devices = XA_TX_AdcPhysicalDevices;   // SID devices and channels INFO
        //ch_per_dev = XA_TX_AdcPhysicalChannelsPerDevice;
        }
    else
        {
        devices = XA_TX_DacPhysicalDevices;   // SID devices and channels INFO
        ch_per_dev = XA_TX_DacPhysicalChannelsPerDevice;
        }
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetDacSpiDeviceCount() -- Return Dac SPI devices
//---------------------------------------------------------------------------

unsigned int  XA_TXDeviceMap::GetDacSpiDeviceCount()
{
    return XA_TX_DacSpiDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetDacCalibrationOverride() -- Return override object or 0
//---------------------------------------------------------------------------

ICalibrationValueOverride *  XA_TXDeviceMap::GetDacCalibrationOverride()
{
    return new XA_TX_DacCalOverride(this);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetCalibrationDacCount() -- Return DACs needing Cal
//---------------------------------------------------------------------------

unsigned int  XA_TXDeviceMap::GetCalibrationDacCount()
{
    return XA_TX_DacCalibrationDeviceCount;
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::MakeSioPllObject() -- Build custom PLL object
//---------------------------------------------------------------------------

Si5338 *  XA_TXDeviceMap::MakeSioPllObject(I2CDevice & device)
{
    //  Make our local SioPll object (subdev will delete it)
    return new XA_TX_SioPll(this, device);
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetVeloInStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_TXDeviceMap::GetVeloInStreamCount()
{
    return XA_TX_VeloInStreamCount;
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetVeloOutStreamCount() -- Return Velo In PIDs
//---------------------------------------------------------------------------

unsigned int  XA_TXDeviceMap::GetVeloOutStreamCount()
{
    return XA_TX_VeloOutStreamCount;
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetPatternModeAlignment() --
//---------------------------------------------------------------------------
//  Patterns must start on an even multiple of N (32-bit words)

unsigned int   XA_TXDeviceMap::GetPatternModeAlignment()
{
    return 16;   // standard XA alignment of 16 words
}

//---------------------------------------------------------------------------
//  XA_TXDeviceMap::GetPatternModeScaleFactor() --
//---------------------------------------------------------------------------
//  Divide 32 bit address and size by this number when writing to command packet

unsigned int   XA_TXDeviceMap::GetPatternModeScaleFactor()
{
    return 1;   // standard XA scale factor of 1 words
}

//=============================================================================
//  CLASS XA_TX_DacCalOverride  --  Cal Override for XA_TX Dacs
//=============================================================================
//---------------------------------------------------------------------------
//  XA_TX_DacCalOverride::OverrideOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------
//  Adapt the 'offset' value for the

unsigned int  XA_TX_DacCalOverride::OverrideOffset(unsigned int value)
{
    // trim to 16 bit value
    return value&0xFFFF;
}

//---------------------------------------------------------------------------
//  XA_TX_DacCalOverride::OverrideGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

unsigned int  XA_TX_DacCalOverride::OverrideGain(unsigned int value)
{
    const float HdwStep_mA    = 0.0225f;    // 22.5 uA steps
    const float Min_mA        = 8.64f;      // 0x000 gain value
    const float Center_mA     = 20.00f;     // 0x1f9 gain value


    float gain = static_cast<float>(value)/XA_TX_Dac_One;

    float adjusted_mA = gain*Center_mA;
    float steps_mA = adjusted_mA - Min_mA;

    float counts = steps_mA / HdwStep_mA;
    unsigned int setting;
    if (counts < 0)
        setting = 0;
    else if (counts > 1023)
        setting = 0x3FF;
    else
        setting = static_cast<unsigned int>(counts);

    return setting;
}

//---------------------------------------------------------------------------
//  XA_TX_DacCalOverride::WriteOffset() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  XA_TX_DacCalOverride::WriteOffset(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Offset(value);
    else
        Owner->Output().Device().Write_Q_Offset(value);

}

//---------------------------------------------------------------------------
//  XA_TX_DacCalOverride::WriteGain() -- adapt for nonstandard cal
//---------------------------------------------------------------------------

void  XA_TX_DacCalOverride::WriteGain(unsigned int ch, unsigned int value)
{
    if (ch==0)
        Owner->Output().Device().Write_I_Gain(value);
    else
        Owner->Output().Device().Write_Q_Gain(value);

}

//===========================================================================
//  CLASS XA_TX_DacToHardware  - link Dac Device to hardware
//===========================================================================
//---------------------------------------------------------------------------
//  declaration for class XA_TX_DacToHardware
//---------------------------------------------------------------------------

class XA_TX_DacToHardware :  public IRequires_DAC3484_DAC_Io
{
public:
    unsigned int  DacSpi(unsigned int addr)
        {  return Out->Spi(Dev, addr);  }
    void          DacSpi(unsigned int addr, unsigned int value)
        {  Out->Spi(Dev, addr, value);  }

    bool DacChannelEnabled(int ch)
        {  return Out->ChannelEnabled(ch);  }
    void DacChannelEnabled(int ch, bool state)
        {  Out->ChannelEnabled(ch, state);  }

    void  DacReset(bool state)
        {  Out->DacReset(state); }

    XA_TX_DacToHardware( XA_TXOutputDevice * out, unsigned int dev=0 )
        :  Out(out), Dev(dev)
        {}
    virtual ~XA_TX_DacToHardware()
        {}

    XA_TXOutputDevice *    Out;
    unsigned int           Dev;
};

//===========================================================================
//  CLASS XA_TXOutputDevice  -- XA_TX Output Devices
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XA_TXOutputDevice
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XA_TXOutputDevice
//---------------------------------------------------------------------------

XA_TXOutputDevice::XA_TXOutputDevice(XA_TXDeviceMap * owner, IX6AnalogIoInfo * info)
    : inherited(owner), OwnerXA(owner), FInfo(info),
      DacDev(new XA_TX_DacToHardware(this, 0)),
      CalRomSection(DacCalIdrom(), romXA_TX_OutputCoeffOffset, XA_TX_DacChannels,
                       XA_TX_Dac_One, DacCal() )
{
}

//---------------------------------------------------------------------------
//  destructor for class XA_TXOutputDevice
//---------------------------------------------------------------------------

XA_TXOutputDevice::~XA_TXOutputDevice()
{
    delete FInfo;
    FInfo = 0;
}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::Info() -- Analog Information
//---------------------------------------------------------------------------

IX6AnalogIoInfo &  XA_TXOutputDevice::Info()
{
    return *FInfo;
}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::OpenHardware() -- on open
//---------------------------------------------------------------------------

void  XA_TXOutputDevice::OpenHardware()
{
    CalRomSection.ApplyRomCoefficients();

}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::PreconfigureHardware() --
//---------------------------------------------------------------------------

void  XA_TXOutputDevice::PreconfigureHardware()
{
    CalRomSection.ApplyRomCoefficients();
    //DacDev.PreconfigHardware();
}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::ConfigureHardware() --
//---------------------------------------------------------------------------

void  XA_TXOutputDevice::ConfigureHardware()
{
    OwnerXA->Debug.Log("DAC ConfigureHardware()");

    // Toggle the DacReset
    DacCommon().DacReset(false);
    DacCommon().DacReset(true);
    Sleep(1);
    DacCommon().DacReset(false);

    DacDev.ConfigureHardware();

}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::ConfigureChannelEnables() --
//---------------------------------------------------------------------------

void  XA_TXOutputDevice::ConfigureChannelEnables()
{
    //
    //  Enable Devices and Channels
    //Device().Enabled( DacCommon().ChannelEnabled(0) || DacCommon().ChannelEnabled(1) );
    Device().ChannelEnabled(0, DacCommon().ChannelEnabled(0) );
    Device().ChannelEnabled(1, DacCommon().ChannelEnabled(1) );

}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  XA_TXOutputDevice::SetDacInterpolation(DacInterpolationModeValues values)
{
    SetDacInterpolation(static_cast<unsigned int>(values));
}

//---------------------------------------------------------------------------
//  XA_TXOutputDevice::SetDacInterpolation() --
//---------------------------------------------------------------------------

void  XA_TXOutputDevice::SetDacInterpolation(unsigned int values)
{
    OwnerXA->FDacInterpolationMode = values&0x7;
}


} // namespace Innovative


