// XA_TXDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XA_TXDeviceMap_MbH
#define XA_TXDeviceMap_MbH

#include <XA_TX_Mb.h>
#include <XASubDevice_Stream_Mb.h>
#include <DAC3484_SubDevice_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class XA_TXDeviceMap;

//==============================================================================
//  CLASS XA_TX_Pll -- XA_TX PLL Chip (Specialization of Lmk04820)
//==============================================================================

class XA_TX_Pll : public Lmk04820
{
public:
    XA_TX_Pll(XA_TXDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~XA_TX_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 40e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XA_TXDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double /*dac_rate*/);

private:
    unsigned int  DacMode();
    double        DCLK(double dac_rate, unsigned int mode);
};


//==============================================================================
//  CLASS XA_TX_SioPll -- RX SIO PLL Chip (Specialization of Si5338)
//==============================================================================

class XA_TX_SioPll : public Si5338
{
public:
    XA_TX_SioPll(XA_TXDeviceMap * owner, I2CDevice & device);

    // ...Si5338 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

protected:
    XA_TXDeviceMap * Owner;
};

//===========================================================================
//  CLASS ISuppliesXA_TXRequirements -- Device Map Requirement List
//===========================================================================

class ISuppliesXA_TXRequirements : public IRequires_Nothing,
                                    public IRequires_Lmk20FmcClocks
{
public:
    virtual ~ISuppliesXA_TXRequirements()
        {}

};

//===========================================================================
//  CLASS XA_TXDeviceMap  -- Register & low level support for XA_TX module
//===========================================================================
class XA_TX;

class XA_TXDeviceMap :  public XADacDeviceMap, public ISuppliesXA_TXRequirements

{
    friend class XA_TX;
    friend class XA_TXOutputDevice;

public:
    //
    //  Member Functions
    XA_TXDeviceMap(XA_TX * owner);
    virtual ~XA_TXDeviceMap();

    XA_LMK20_Clock::XAClockIo & Clock()  { return Clock_SDev.Clock(); }

    class XA_TXOutputDevice &  Output();



public: // pseudo-protected

    unsigned int      CurrentDacInterpolationMode()
        {  return FDacInterpolationMode;  }

//    virtual double   Get_DacClockRate()  { return Clock().Dac().Frequency(); }

protected:
    // Data
    XA_TX *         Owner;
    //
    //  Subdevices
    XA_LMK20_Clock        Clock_SDev;

    XA_TXOutputDevice *    FOutput;

    Trace             Debug;

    // Methods
//    virtual void    ConstructCustomParts();            // board construtor time
    virtual void    OpenHardware();                    // after open time
//    virtual void    StreamStopHardware();              // after stream stop subdevice configuration

    virtual double  StandardClockFrequency()
                        {  return Clock().Frequency();   }
    virtual double  Get_DacClockFrequency()
                        {  return Clock().Dac().Frequency();   }

    //  Subdevice Customization Handlers
    //  Type1 Chrono
    virtual Lmk04820 *                  MakeFmcAfePllObject();

    //  XA Adc Device Mixin
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode which, unsigned int& devices, unsigned int& ch_per_dev);
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
//    virtual unsigned int                GetAdcSpiDeviceCount();
//    virtual unsigned int                GetCalibrationAdcCount();
//    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();

    //  XA Dac Device Mixin
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();
    //  Stream Configuration
    virtual unsigned int                GetVeloInStreamCount();
    virtual unsigned int                GetVeloOutStreamCount();

    //  Pattern Mode
    virtual unsigned int                GetPatternModeScaleFactor();
    virtual unsigned int                GetPatternModeAlignment();

    // Aurora
    virtual Si5338 *                    MakeSioPllObject(I2CDevice & device);

private:
    unsigned int  FDacInterpolationMode;

    void     LocalConstructCustomParts();

    // Methods
    void  OnStreamPreconfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamSingleTimeAtStartConfigHandler(OpenWire::NotifyEvent & Event);
    void  OnStreamConfigureHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStartHandler(OpenWire::NotifyEvent & Event);
    void  OnBeforeStreamStopHandler(OpenWire::NotifyEvent & Event);
    void  OnAfterStreamStopHandler(OpenWire::NotifyEvent & Event);

    void     ConfigureDacs();
};

//=============================================================================
//  CLASS XA_TX_DacCalOverride  --  Cal Override for XA_TX Dacs
//=============================================================================

class XA_TX_DacCalOverride : public ICalibrationValueOverride
{
public:
    virtual unsigned int   OverrideOffset(unsigned int value) ;
    virtual unsigned int   OverrideGain(unsigned int value) ;

    virtual bool OverrideOffsetUpdate() {  return true;  }
    virtual bool OverrideGainUpdate() {  return true;  }

    virtual void WriteOffset(unsigned int ch, unsigned int value);
    virtual void WriteGain(unsigned int ch, unsigned int value);

    XA_TX_DacCalOverride(XA_TXDeviceMap *  owner) : Owner(owner)
        {}
    virtual ~XA_TX_DacCalOverride()
        {}

private:
    XA_TXDeviceMap *  Owner;
};

//===========================================================================
//  CLASS XA_TXOutputDevice  -- XA_TX Output Devices
//===========================================================================

class XA_TXOutputDevice : public XAOutputDevice
{
public:
    typedef XAOutputDevice inherited;

    XA_TXOutputDevice(XA_TXDeviceMap* owner, IX6AnalogIoInfo * info );
    virtual ~XA_TXOutputDevice();

    virtual IX6AnalogIoInfo &  Info();

    //  Special Dac Interpolation Mode
    enum  DacInterpolationModeValues { diByOne=0x0, diByTwo=0x1, diByTwo_b=0x02,
                                       diByFour=0x3, diByFour_b=0x6, diByEight=0x7 };
    void  SetDacInterpolation(DacInterpolationModeValues values);
    void  SetDacInterpolation(unsigned int values);

    DAC3484::DAC3484_DAC &     Device()
        {
            return DacDev;
        }
    void  DacReset(bool state)
        {
        DacCommon().DacReset(state);
        }

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

    void ConfigureChannelEnables();

private:
    XA_TXDeviceMap *           OwnerXA;
    IX6AnalogIoInfo *          FInfo;
    DAC3484::DAC3484_DAC       DacDev;
    X6CalibrationSection       CalRomSection;

    X6IdromPage &     DacCalIdrom()  {  return OwnerXA->DacCal_PG;  }

};

#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
