//---------------------------------------------------------------------------
// XUFamilyBoard_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XUFamilyBoard_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XuBoard  -- Xu Common baseboard class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XuBoard
//---------------------------------------------------------------------------

XuBoard::XuBoard(XUDeviceMap & deviceMap)
    :  MultiChanBoard(deviceMap), XuDeviceRef(deviceMap)
{
}

//---------------------------------------------------------------------------
//  destructor for class XuBoard
//---------------------------------------------------------------------------

XuBoard::~XuBoard()
{
    // Don't call anything using XUDeviceRef here - put in most derived class
}

//---------------------------------------------------------------------------
//  XuBoard::Led() -- Read the LED state of the board
//---------------------------------------------------------------------------

bool  XuBoard::Led()
{
    return XuDeviceRef.Led();
}

//---------------------------------------------------------------------------
//  XuBoard::Led() -- Set the LED state of the board
//---------------------------------------------------------------------------

void  XuBoard::Led( bool value )
{
    XuDeviceRef.Led(value);
}

//---------------------------------------------------------------------------
//  XuBoard::GreenLed() -- Read the Green LED state
//---------------------------------------------------------------------------

bool  XuBoard::GreenLed()
{
    return XuDeviceRef.Periph().GreenLed();
}

//---------------------------------------------------------------------------
//  XuBoard::GreenLed() -- Set the Green LED state of the board
//---------------------------------------------------------------------------

void  XuBoard::GreenLed( bool value )
{
    XuDeviceRef.Periph().GreenLed(value);
}

//---------------------------------------------------------------------------
//  XuBoard::FrontPanelLed() -- Read the FrontPanel LED state
//---------------------------------------------------------------------------

bool  XuBoard::FrontPanelLed()
{
    return XuDeviceRef.Periph().FrontPanelLed();
}

//---------------------------------------------------------------------------
//  XuBoard::FrontPanelLed() -- Set the FrontPanel LED state of the board
//---------------------------------------------------------------------------

void  XuBoard::FrontPanelLed( bool value )
{
    XuDeviceRef.Periph().FrontPanelLed(value);
}

//---------------------------------------------------------------------------
//  XuBoard::Info() --  Return reference to Info interface
//---------------------------------------------------------------------------

const IX6LogicVersionInfo&  XuBoard::Info()
{
    return XuDeviceRef.Info();
}

//---------------------------------------------------------------------------
//  XuBoard::VsProm() -- Return reference to FPA object
//---------------------------------------------------------------------------

IFpaLoader & XuBoard::VsProm()
{
    return XuDeviceRef.VsProm();
}

//---------------------------------------------------------------------------
//  XuBoard::IdRom() --
//---------------------------------------------------------------------------

IIdRom & XuBoard::IdRom()
{
    return XuDeviceRef.IdRom();
}

//---------------------------------------------------------------------------
//  XuBoard::Alert() --  Alert Config Interface
//---------------------------------------------------------------------------

IUsesXUAlerts & XuBoard::Alert()
{
    return XuDeviceRef.Alert();
}

//---------------------------------------------------------------------------
//  XuBoard::DioP() --  Dio Config Interface
//---------------------------------------------------------------------------

XUDioPortIntf & XuBoard::DioP()
{
    return XuDeviceRef.DioP();
}

//---------------------------------------------------------------------------
//  XuBoard::DioN() --  Dio Config Interface
//---------------------------------------------------------------------------

XUDioPortIntf & XuBoard::DioN()
{
    return XuDeviceRef.DioN();
}

//---------------------------------------------------------------------------
//  XuBoard::VFifo() --  VFifo Info Interface
//---------------------------------------------------------------------------

XU_VFifoInfo & XuBoard::VFifo()
{
    return XuDeviceRef.VFifo();
}

//---------------------------------------------------------------------------
//  XuBoard::StreamTester() --  Stream Tester Interface
//---------------------------------------------------------------------------

XU_StreamTester & XuBoard::StreamTester()
{
    return XuDeviceRef.StreamTester();
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
