// XUFamilyBoard_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef XUFamilyBoard_MbH
#define XUFamilyBoard_MbH

#include <MultiChanBoard_Mb.h>
#include <XUFamilyDeviceMap_Mb.h>


namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS XuBoard  -- Xu Common baseboard class
//===========================================================================

class XuBoard  : public MultiChanBoard
{
public:
    //
    //  Member Functions
    XuBoard(XUDeviceMap & deviceMap);
    virtual ~XuBoard();

    bool                        Led();
    void                        Led(bool value);
    bool                        GreenLed();
    void                        GreenLed(bool value);
    bool                        FrontPanelLed();
    void                        FrontPanelLed(bool value);

    //  Informational Methods
    const IX6LogicVersionInfo&   Info();
    IFpaLoader &                 VsProm();
    IIdRom &                     IdRom();

    IUsesXUAlerts &              Alert();
    XUDioPortIntf &              DioP();
    XUDioPortIntf &              DioN();
    XU_VFifoInfo &               VFifo();
    XU_StreamTester &            StreamTester();
    XU_LogicSystemMonitor &      LogicSysMon()    {  return XuDeviceRef.LogicSysMon();    }
    IMuxedAurora &               Aurora()         {  return XuDeviceRef.Aurora();    }
    IXUSioPllControl &           SioPll()         {  return XuDeviceRef.SioPll();   }

public: //Pseudo protected

    // FPGA type name function
    std::string FpgaName(const short Number)
    {
        switch(Number)
            {
            case 0x01: return "KU060";
            case 0x02: return "KU085";
            case 0x03: return "KU115";
            default: return "Unknown";
            }
    }

    // Board type name function
    std::string BoardName(const short Number)
    {
        switch(Number)
            {
            case 0x01: return "XU-TX";
            case 0x02: return "XU-RT";
            case 0x03: return "XU-RX";
            default: return "Unknown";
            }
    }

protected:
    //
    //  Member Data
    XUDeviceMap &                  XuDeviceRef;

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif
}; // namespace Innovative

#endif

