// XUFamilyDeviceMap_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XUFamilyBoard_Mb.h>
#include <XUFamilyDeviceMap_Mb.h>
#include <XUFamilyMemoryMap_Mb.h>
#include <XUDeviceDriver_Mb.h>

//---------------------------------------------------------------------------
namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XUDeviceMap  -- Registers for X6 Device
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUDeviceMap
//---------------------------------------------------------------------------
//
//  IDROM Page Numbers
const unsigned int System_IdromPage   = 0;
const unsigned int AdcCal_IdromPage   = 1;
const unsigned int DacCal_IdromPage   = 2;
const unsigned int User_IdromPage     = 3;

//
//  Subdevice Register Structures

using namespace XUFamily_MemoryMap;
using namespace InnovativeKernel::MultiChanStandardMemoryMap;

namespace
{

X6Info::MapRegisters X6InfoRegAddrs =
    {
    mmInformation,                 //  Info_Addr
    mmBitstreamDate,               //  BitstreamDate_Addr
    mmSubrevision,                 //  Subrev_Addr
    mmXuControlStatusRegister,     //  Bar0Version_Addr
    };

X6Peripherals::MapRegisters XU_PeripheralRegAddrs =
    {
    mmReset,              //  ResetReg_Addr
    };

XUAFE_Common::MapRegisters XU_AfeCommonRegAddrs =
    {
    mmAFE_TestControl,       // AfeTestControl_Addr
    mmAFE_SoftwareTrig,      // AfeSoftwareTrigger_Addr
    mmAFE_ExtSyncSelect,     // AfeExtSyncSelect_Addr
    };

XU_FpaLoader::MapRegisters XU_VsPromRegAddrs =
    {
    mmFpa_Address,           //  AddressControl_Addr;
    mmFpa_Data,              //  Data_Addr
    mmFpa_Command,           //  Opcode_Addr;
    };

XU_FlashRom::MapRegisters XUFlashRomRegAddrs =
    {
    mmFlash2_Address,         //  AddressControl_Addr
    mmFlash2_Data,            //  Data_Addr
    mmFlash2_Command,         //  Opcode_Addr
    };

XUAlertSystem::MapRegisters XU_AlertRegAddrs =
    {
    mmAlert_Enable,           //  AlertEnable_Addr
    mmAlert_Control,          //  AlertControl_Addr
    mmAlert_Software,         //  SoftwareAlert_Addr
    mmAlert_Clear,            //  AlertClear_Addr
    };

XUDigitalIo::MapRegisters XU_DioPRegAddrs =
    {
    mmDioP_DataLo,             //  DioDataLo_Addr
    mmDioP_OutEnableLo,        //  DioOutputEnableLo_Addr
    };

XUDigitalIo::MapRegisters XU_DioNRegAddrs =
    {
    mmDioN_DataLo,             //  DioDataLo_Addr
    mmDioN_OutEnableLo,        //  DioOutputEnableLo_Addr
    };

XU_VFifoInfo::MapRegisters XU_VFifoRegAddrs =
    {
    mmVFifo_Status,           //  VFifoStatus_Addr;
    mmVFifo_Depth,            //  VFifoWordCount_Addr
    };

XU_StreamTester::MapRegisters XU_StreamTesterRegAddrs =
    {
    mmXuCheck_TestEnable,     //  XuTestEnable_Addr;
    mmXuCheck_PktSize,        //  XuTestPacketSize_Addr
    mmXuCheck_ChkDecimate,    //  XuTestOutDecimation_Addr;
    mmXuCheck_ChkError,       //  XuTestChkErrors_Addr
    mmXuCheck_ExtIntTrig,     //  XuTestSignalExtInt_Addr;
    };

XUSystemMonitor::MapRegisters XU_SysmonRegAddrs =
    {
    mmSysmon_Control,     //  SysMonControl_Addr;
    mmSysmon_Data,        //  SysMonData_Addr
    };

XUAurora::MapRegisters XU_Aurora_RegAddrs =
    {
    mmAurora_Select,          //  Select_Addr;
    mmAurora_TestControl,     //  TestControl_Addr;
    mmAurora_ControlStatus,   //  ControlStatus_Addr;
    mmAurora_WritePort,       //  WritePort_Addr;
    mmAurora_ReadPort,        //  ReadPort_Addr;
    };

XU_LogicSystemMonitor::MapRegisters XU_LogicSysmonRegAddrs =
    {
    mmLogicSM_CurTemperature,     //  CurTemperature_Addr;
    mmLogicSM_CurVccInt,          //  CurVccInt_Addr
    mmLogicSM_CurVccAux,          //  CurVccAux_Addr;
    mmLogicSM_CurVbram,           //  CurVbram_Addr
    mmLogicSM_MaxTemperature,     //  MaxTemperature_Addr;
    mmLogicSM_MaxVccInt,          //  MaxVccInt_Addr
    mmLogicSM_MaxVccAux,          //  MaxVccAux_Addr;
    mmLogicSM_MaxVbram,           //  MaxVbram_Addr
    mmLogicSM_MinTemperature,     //  MinTemperature_Addr;
    mmLogicSM_MinVccInt,          //  MinVccInt_Addr
    mmLogicSM_MinVccAux,          //  MinVccAux_Addr;
    mmLogicSM_MinVbram,           //  MinVbram_Addr
    mmLogicSM_WarnThresh_U,       //  WarnThresh_U_Addr;
    mmLogicSM_FatalThresh_U,      //  FatalThresh_U_Addr
    mmLogicSM_WarnThresh_L,       //  WarnThresh_L_Addr;
    mmLogicSM_FatalThresh_L,      //  FatalThresh_L_Addr
    };

} // end nameless namespace

//---------------------------------------------------------------------------
//  constructor for class XUDeviceMap
//---------------------------------------------------------------------------

XUDeviceMap::XUDeviceMap(XuBoard * owner)
    :   MultiChanDeviceMap(owner, new Innovative::XUDeviceDriver(*this)),
        Owner(owner),
        //DDeviceDriver     (dynamic_cast<XUDeviceDriver*>(DeviceDriver)),
        AXI_Periph        (LogicMemory, axiPeripheral),
        AXI_SysMgmt       (LogicMemory, axiSysMgmtWizard),
        AXI_AFE           (LogicMemory, axiAFE),
        AXI_JESD_0        (LogicMemory, axiJESD_0),
        AXI_JESD_1        (LogicMemory, axiJESD_1),
        AXI_JESD_2        (LogicMemory, axiJESD_2),
        AXI_JESD_3        (LogicMemory, axiJESD_3),
        AXI_Custom        (LogicMemory, axiCustom),
        Info_SDev         (this, AXI_Periph, PortMemory, X6InfoRegAddrs, X6Info::styleXU),
        Periph_SDev       (this, AXI_Periph, XU_PeripheralRegAddrs),
        AFECommon_SDev    (this, AXI_AFE, XU_AfeCommonRegAddrs),
        VsProm_SDev       (this, XU_FpaLoader::romMT25Q_512, AXI_Periph, XU_VsPromRegAddrs),
        FlashRom_SDev     (this, XU_FlashRom::romMT25Q_512, AXI_Periph, XUFlashRomRegAddrs),
        Alerts_SDev       (this, AXI_Periph, XU_AlertRegAddrs),
        DioP_SDev         (this, AXI_Periph, XU_DioPRegAddrs),
        DioN_SDev         (this, AXI_Periph, XU_DioNRegAddrs),
        VFifo_SDev        (this, AXI_Periph, XU_VFifoRegAddrs),
        StreamTester_SDev (this, AXI_Periph, XU_StreamTesterRegAddrs),
        SysMon_SDev       (this, AXI_Periph, XU_SysmonRegAddrs),
        LogicSysMon_SDev  (this, AXI_SysMgmt, XU_LogicSysmonRegAddrs),
        Aurora_SDev       (this, AXI_Periph, XU_Aurora_RegAddrs),
        SioPll_SDev       (this),
        System_PG         (FlashRom_SDev, System_IdromPage),
        AdcCal_PG         (FlashRom_SDev, AdcCal_IdromPage),
        DacCal_PG         (FlashRom_SDev, DacCal_IdromPage),
        UserRom_PG        (FlashRom_SDev, User_IdromPage),
        IdRom_Intf        (this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUDeviceMap
//---------------------------------------------------------------------------

XUDeviceMap::~XUDeviceMap()
{
}

//---------------------------------------------------------------------------
//  XUDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUDeviceMap::ConstructCustomParts()
{
    MultiChanDeviceMap::ConstructCustomParts();

    Info_SDev.ConstructCustomParts();
    Periph_SDev.ConstructCustomParts();
    AFECommon_SDev.ConstructCustomParts();
    VsProm_SDev.ConstructCustomParts();
    FlashRom_SDev.ConstructCustomParts();
    Alerts_SDev.ConstructCustomParts();
    DioP_SDev.ConstructCustomParts();
    DioN_SDev.ConstructCustomParts();
    VFifo_SDev.ConstructCustomParts();
    StreamTester_SDev.ConstructCustomParts();
    SysMon_SDev.ConstructCustomParts();
    LogicSysMon_SDev.ConstructCustomParts();
    Aurora_SDev.ConstructCustomParts();
    SioPll_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::PreOpenHardware()
//---------------------------------------------------------------------------

void  XUDeviceMap::PreOpenHardware()
{
    MultiChanDeviceMap::PreOpenHardware();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XUDeviceMap::OpenHardware()
{
    MultiChanDeviceMap::OpenHardware();

    Periph_SDev.AppReset();

    Info_SDev.OpenHardware();
    Periph_SDev.OpenHardware();
    AFECommon_SDev.OpenHardware();
    VsProm_SDev.OpenHardware();
    FlashRom_SDev.OpenHardware();
    Alerts_SDev.OpenHardware();
    DioP_SDev.OpenHardware();
    DioN_SDev.OpenHardware();
    VFifo_SDev.OpenHardware();
    StreamTester_SDev.OpenHardware();
    SysMon_SDev.OpenHardware();
    LogicSysMon_SDev.OpenHardware();
    Aurora_SDev.OpenHardware();
    SioPll_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XUDeviceMap::PreconfigureHardware()
{
    MultiChanDeviceMap::PreconfigureHardware();

    Info_SDev.PreconfigureHardware();
    Periph_SDev.PreconfigureHardware();
    AFECommon_SDev.PreconfigureHardware();
    VsProm_SDev.PreconfigureHardware();
    FlashRom_SDev.PreconfigureHardware();
    Alerts_SDev.PreconfigureHardware();
    DioP_SDev.PreconfigureHardware();
    DioN_SDev.PreconfigureHardware();
    VFifo_SDev.PreconfigureHardware();
    StreamTester_SDev.PreconfigureHardware();
    SysMon_SDev.PreconfigureHardware();
    LogicSysMon_SDev.PreconfigureHardware();
    Aurora_SDev.PreconfigureHardware();
    SioPll_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::OnSingleStreamAtStartConfig()
//---------------------------------------------------------------------------

void  XUDeviceMap::OnSingleStreamAtStartConfig()
{
    MultiChanDeviceMap::OnSingleStreamAtStartConfig();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUDeviceMap::ConfigureHardware()
{
    MultiChanDeviceMap::ConfigureHardware();

    Info_SDev.ConfigureHardware();
    Periph_SDev.ConfigureHardware();
    AFECommon_SDev.ConfigureHardware();
    VsProm_SDev.ConfigureHardware();
    FlashRom_SDev.ConfigureHardware();
    Alerts_SDev.ConfigureHardware();
    DioP_SDev.ConfigureHardware();
    DioN_SDev.ConfigureHardware();
    VFifo_SDev.ConfigureHardware();
    StreamTester_SDev.ConfigureHardware();
    SysMon_SDev.ConfigureHardware();
    LogicSysMon_SDev.ConfigureHardware();
    Aurora_SDev.ConfigureHardware();
    SioPll_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::OnBeforeStreamStart()
//---------------------------------------------------------------------------

void  XUDeviceMap::OnBeforeStreamStart()
{
    MultiChanDeviceMap::OnBeforeStreamStart();

    Periph_SDev.XURun(true);
}

//---------------------------------------------------------------------------
//  XUDeviceMap::OnAfterStreamStart()
//---------------------------------------------------------------------------

void  XUDeviceMap::OnAfterStreamStart()
{
    MultiChanDeviceMap::OnAfterStreamStart();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::OnBeforeStreamStop()
//---------------------------------------------------------------------------

void  XUDeviceMap::OnBeforeStreamStop()
{
    MultiChanDeviceMap::OnBeforeStreamStop();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XUDeviceMap::StreamStopHardware()
{    
    Periph_SDev.XURun(false);

    MultiChanDeviceMap::StreamStopHardware();
}

//---------------------------------------------------------------------------
//  XUDeviceMap::PeekJesdLogic() - read relative to the JESD base
//---------------------------------------------------------------------------

ii32  XUDeviceMap::PeekJesdLogic(unsigned int which, ii32 offset)
{
    AxiMasterSpace & JESD((which==0)? AXI_JESD_0 : ((which==1)? AXI_JESD_1 :
                           ((which==2)? AXI_JESD_2 : AXI_JESD_3)));
    return JESD.At(offset);
}

//---------------------------------------------------------------------------
//  XUDeviceMap::PokeJesdLogic() - write relative to the FMC base
//---------------------------------------------------------------------------

void  XUDeviceMap::PokeJesdLogic(unsigned int which, ii32 offset, ii32 data)
{
    AxiMasterSpace & JESD((which==0)? AXI_JESD_0 : ((which==1)? AXI_JESD_1 :
                           ((which==2)? AXI_JESD_2 : AXI_JESD_3)));
    JESD.At(offset, data);
}

//===========================================================================
//  CLASS XUAdcDeviceMap  -- input only card device map
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUAdcDeviceMap
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  housekeeping for class XUDac_DeviceMixin
//---------------------------------------------------------------------------

XUAFE_AdcCommon::MapRegisters XUAfeAdcCommonRegAddrs =
    {
    mmAFE_AdcEnable,        //  AfeAdcEnable_Addr
    mmAFE_AdcPowerDown,     //  AfeAdcPower_Addr
    mmAFE_AdcTriggerCfg,    //  AfeAdcTriggerCfg_Addr
    mmAFE_AdcDecimation,    //  AfeAdcDecimation_Addr
    mmAFE_AdcReset,         //  AfeAdcReset_Addr
    //mmAFE_AdcPhaseIncr,       //  AfeAdcPhaseIncr_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters XUAfeAdcPriRegAddrs =
    {
    mmAFE_AdcPriEnable,      //  PriTriggerEnable_Addr;
    mmAFE_AdcPriInterval,    //  PriInterval_Addr;
    mmAFE_AdcPriTrigParam,   //  PriTriggerParam_Addr;
    mmAFE_AdcCaptureWin      //  PriCaptureWindow_Addr;
    };

XUSpiDevice::MapRegisters XUAfeAdcSpiRegAddrs =
    {
    mmAFE_AdcSpiEnable,      //  Device_Addr;
    mmAFE_AdcSpiCtrl,        //  Write_Addr;
    mmAFE_AdcSpiStatus,      //  Read_Addr;
    };

XU_AdcCalCoefficientLoad::MapRegisters XUAfeAdcCalCoeffRegAddrs =
    {
    mmAFE_AdcGainCoeff,      //  GainCoeff_Addr
    mmAFE_AdcOffsetCoeff,    //  OffsetCoeff_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class XUAdcDeviceMap
//---------------------------------------------------------------------------

XUAdcDeviceMap::XUAdcDeviceMap(class XuBoard * owner)
    :   XUDeviceMap(owner),
        AFEAdcC_SDev  (this, AXI_AFE, XUAfeAdcCommonRegAddrs),
        AFEAdcPri_SDev(this, true, AXI_AFE, XUAfeAdcPriRegAddrs),  // true - AdcPri
        AFEAdcSpi_SDev(this, AXI_AFE, XUAfeAdcSpiRegAddrs),
        AFEAdcCal_SDev(this, AXI_AFE, XUAfeAdcCalCoeffRegAddrs, mmAFE_TotalAdcs)//,
{
}

//---------------------------------------------------------------------------
//  XUAdcDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAdcDeviceMap::ConstructCustomParts()
{
    XUDeviceMap::ConstructCustomParts();
    AFEAdcC_SDev.ConstructCustomParts();
    AFEAdcPri_SDev.ConstructCustomParts();
    AFEAdcSpi_SDev.ConstructCustomParts();
    AFEAdcCal_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XUAdcDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XUAdcDeviceMap::OpenHardware()
{
    XUDeviceMap::OpenHardware();
    AFEAdcC_SDev.OpenHardware();
    AFEAdcPri_SDev.OpenHardware();
    AFEAdcSpi_SDev.OpenHardware();
    AFEAdcCal_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  XUAdcDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XUAdcDeviceMap::PreconfigureHardware()
{
    XUDeviceMap::PreconfigureHardware();
    AFEAdcC_SDev.PreconfigureHardware();
    AFEAdcPri_SDev.PreconfigureHardware();
    AFEAdcSpi_SDev.PreconfigureHardware();
    AFEAdcCal_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XUAdcDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAdcDeviceMap::ConfigureHardware()
{
    XUDeviceMap::ConfigureHardware();
    AFEAdcC_SDev.ConfigureHardware();
    AFEAdcPri_SDev.ConfigureHardware();
    AFEAdcSpi_SDev.ConfigureHardware();
    AFEAdcCal_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XUAdcDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XUAdcDeviceMap::StreamStopHardware()
{
    XUDeviceMap::StreamStopHardware();
    AFEAdcC_SDev.StreamStopHardware();
    AFEAdcPri_SDev.StreamStopHardware();
    AFEAdcSpi_SDev.StreamStopHardware();
    AFEAdcCal_SDev.StreamStopHardware();
}

//===========================================================================
//  CLASS XUDac_DeviceMixin  -- DAC Mixin Devices for Output and Bidir cards
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUDac_DeviceMixin
//---------------------------------------------------------------------------

XUAFE_DacCommon::MapRegisters XUAfeDacCommonRegAddrs =
    {
    mmAFE_DacEnable,        //  AfeDacEnable_Addr
    mmAFE_DacPowerDown,     //  AfeDacPower_Addr
    mmAFE_DacTriggerCfg,    //  AfeDacTriggerCfg_Addr
    mmAFE_DacDecimation,    //  AfeDacDecimation_Addr
    mmAFE_DacReset,         //  AfeDacReset_Addr
    //mmAFE_DacPhaseIncr,       //  AfeDacPhaseIncr_Addr
    };

X6AFE_PulseRepetitionInterval::MapRegisters XUAfeDacPriRegAddrs =
    {
    mmAFE_DacPriEnable,      //  PriTriggerEnable_Addr;
    mmAFE_DacPriInterval,    //  PriInterval_Addr;
    mmAFE_DacPriTrigParam,   //  PriTriggerParam_Addr;
    mmAFE_DacCaptureWin      //  PriCaptureWindow_Addr;
    };

XUSpiDevice::MapRegisters XUAfeDacSpiRegAddrs =
    {
    mmAFE_DacSpiEnable,      //  Device_Addr;
    mmAFE_DacSpiCtrl,        //  Write_Addr;
    mmAFE_DacSpiStatus,      //  Read_Addr;
    };

XU_DacCalCoefficientLoad::MapRegisters XUAfeDacCalCoeffRegAddrs =
    {
    mmAFE_DacGainCoeff,      //  GainCoeff_Addr
    mmAFE_DacOffsetCoeff,    //  OffsetCoeff_Addr
    };

XU_DacPatternModeSystem::MapRegisters XUDacPatternRegAddrs =
    {
    mmDdr4Status,       //   PatternModeEnable_Addr
    mmArbPB_Control,    //   PlaybackCommand_Addr
    mmArbPB_Data0,      //   CmdFifo0_Addr
    mmArbPB_Data1,      //   CmdFifo1_Addr
    mmArbPB_Data2,      //   CmdFifo2_Addr
    mmArbPB_Data3,      //   CmdFifo3_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class XUDac_DeviceMixin
//---------------------------------------------------------------------------

XUDac_DeviceMixin::XUDac_DeviceMixin(XUDeviceMap * owner)
    :   DDMxOwner(owner),
        AFEDacC_SDev  (this, DDMxOwner->AXI_AFE, XUAfeDacCommonRegAddrs),
        AFEDacPri_SDev(this, false, DDMxOwner->AXI_AFE, XUAfeDacPriRegAddrs),  // false=DacPri
        AFEDacSpi_SDev(this, DDMxOwner->AXI_AFE, XUAfeDacSpiRegAddrs),
        AFEDacCal_SDev(this, DDMxOwner->AXI_AFE, XUAfeDacCalCoeffRegAddrs, mmAFE_TotalDacs),
        DacPattern_SDev(this, DDMxOwner->AXI_Periph, XUDacPatternRegAddrs)
{
}

//---------------------------------------------------------------------------
//  XUDac_DeviceMixin::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUDac_DeviceMixin::ConstructCustomParts()
{
    // Subdevice Parts Construction
    AFEDacC_SDev.ConstructCustomParts();
    AFEDacPri_SDev.ConstructCustomParts();
    AFEDacSpi_SDev.ConstructCustomParts();
    AFEDacCal_SDev.ConstructCustomParts();
    DacPattern_SDev.ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XUDac_DeviceMixin::OpenHardware()
//---------------------------------------------------------------------------

void  XUDac_DeviceMixin::OpenHardware()
{
    AFEDacC_SDev.OpenHardware();
    AFEDacPri_SDev.OpenHardware();
    AFEDacSpi_SDev.OpenHardware();
    AFEDacCal_SDev.OpenHardware();
    DacPattern_SDev.OpenHardware();
}

//---------------------------------------------------------------------------
//  XUDac_DeviceMixin::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XUDac_DeviceMixin::PreconfigureHardware()
{
    // Subdevice Configuration
    AFEDacC_SDev.PreconfigureHardware();
    AFEDacPri_SDev.PreconfigureHardware();
    AFEDacSpi_SDev.PreconfigureHardware();
    AFEDacCal_SDev.PreconfigureHardware();
    DacPattern_SDev.PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XUDac_DeviceMixin::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUDac_DeviceMixin::ConfigureHardware()
{
    AFEDacC_SDev.ConfigureHardware();
    AFEDacPri_SDev.ConfigureHardware();
    AFEDacSpi_SDev.ConfigureHardware();
    AFEDacCal_SDev.ConfigureHardware();
    DacPattern_SDev.ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XUDac_DeviceMixin::StreamStopHardware()
//---------------------------------------------------------------------------

void  XUDac_DeviceMixin::StreamStopHardware()
{
    AFEDacC_SDev.StreamStopHardware();
    AFEDacPri_SDev.StreamStopHardware();
    AFEDacSpi_SDev.StreamStopHardware();
    AFEDacCal_SDev.StreamStopHardware();
    DacPattern_SDev.StreamStopHardware();
}

//===========================================================================
//  CLASS XUDacDeviceMap  -- output only card device map
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUDacDeviceMap
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XUDacDeviceMap
//---------------------------------------------------------------------------

XUDacDeviceMap::XUDacDeviceMap(class XuBoard * owner)
    :   XUDeviceMap(owner),
        XUDac_DeviceMixin(this)
{
}

//---------------------------------------------------------------------------
//  XUDacDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUDacDeviceMap::ConstructCustomParts()
{
    XUDeviceMap::ConstructCustomParts();
    XUDac_DeviceMixin::ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XUDacDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XUDacDeviceMap::OpenHardware()
{
    XUDeviceMap::OpenHardware();
    XUDac_DeviceMixin::OpenHardware();
}

//---------------------------------------------------------------------------
//  XUDacDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XUDacDeviceMap::PreconfigureHardware()
{
    XUDeviceMap::PreconfigureHardware();
    XUDac_DeviceMixin::PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XUDacDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUDacDeviceMap::ConfigureHardware()
{
    XUDeviceMap::ConfigureHardware();
    XUDac_DeviceMixin::ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XUDacDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XUDacDeviceMap::StreamStopHardware()
{
    XUDeviceMap::StreamStopHardware();
    XUDac_DeviceMixin::StreamStopHardware();
}

//===========================================================================
//  CLASS XUBidirDeviceMap  -- output only card device map
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUBidirDeviceMap
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XUBidirDeviceMap
//---------------------------------------------------------------------------

XUBidirDeviceMap::XUBidirDeviceMap(class XuBoard * owner)
    :   XUAdcDeviceMap(owner),
        XUDac_DeviceMixin(this)
{
}

//---------------------------------------------------------------------------
//  XUBidirDeviceMap::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUBidirDeviceMap::ConstructCustomParts()
{
    XUAdcDeviceMap::ConstructCustomParts();
    XUDac_DeviceMixin::ConstructCustomParts();
}

//---------------------------------------------------------------------------
//  XUBidirDeviceMap::OpenHardware()
//---------------------------------------------------------------------------

void  XUBidirDeviceMap::OpenHardware()
{
    XUAdcDeviceMap::OpenHardware();
    XUDac_DeviceMixin::OpenHardware();
}

//---------------------------------------------------------------------------
//  XUBidirDeviceMap::PreconfigureHardware() - configuration outside streaming
//---------------------------------------------------------------------------

void  XUBidirDeviceMap::PreconfigureHardware()
{
    XUAdcDeviceMap::PreconfigureHardware();
    XUDac_DeviceMixin::PreconfigureHardware();
}

//---------------------------------------------------------------------------
//  XUBidirDeviceMap::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUBidirDeviceMap::ConfigureHardware()
{
    XUAdcDeviceMap::ConfigureHardware();
    XUDac_DeviceMixin::ConfigureHardware();
}

//---------------------------------------------------------------------------
//  XUBidirDeviceMap::StreamStopHardware()
//---------------------------------------------------------------------------

void  XUBidirDeviceMap::StreamStopHardware()
{
    XUAdcDeviceMap::StreamStopHardware();
    XUDac_DeviceMixin::StreamStopHardware();
}

//===========================================================================
//  CLASS XUInputDevice  -- Partially Implemented X6 Input Device
//===========================================================================
//---------------------------------------------------------------------------
//  XUInputDevice::Devices() -- return device count
//---------------------------------------------------------------------------

unsigned int  XUInputDevice::Devices()
{
     return AdcCommon().Devices();
}

//---------------------------------------------------------------------------
//  XUInputDevice::ChannelsPerDevice() -- return channels per device
//---------------------------------------------------------------------------

unsigned int  XUInputDevice::ChannelsPerDevice()
{
     return AdcCommon().ChannelsPerDevice();
}

//---------------------------------------------------------------------------
//  XUInputDevice::Channels() -- return channels
//---------------------------------------------------------------------------

unsigned int  XUInputDevice::Channels()
{
     return AdcCommon().Channels();
}

//---------------------------------------------------------------------------
//  XUInputDevice::DeviceEnabled() -- Enable specified I/O devices
//---------------------------------------------------------------------------

void  XUInputDevice::DeviceEnabled(int dev, bool state)
{
     AdcCommon().DeviceIsEnabled(dev, state);
     AdcCommon().DevicePowered(dev, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XUInputDevice::DeviceEnabled() -- Query state of specified I/O devices
//---------------------------------------------------------------------------

bool XUInputDevice::DeviceEnabled(int dev) const
{
    return AdcCommon().DeviceIsEnabled(dev);
}

//---------------------------------------------------------------------------
//  XUInputDevice::ActiveDevices() -- Return number of active devices
//---------------------------------------------------------------------------

unsigned int XUInputDevice::ActiveDevices()
{
    return AdcCommon().ActiveDevices();
}

//---------------------------------------------------------------------------
//  XUInputDevice::DevicePowered() -- Power specified I/O devices
//---------------------------------------------------------------------------

void  XUInputDevice::DevicePowered(int ch, bool state)
{
     AdcCommon().DevicePowered(ch, state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::DevicePowered() -- Query power state of an I/O devices
//---------------------------------------------------------------------------

bool XUInputDevice::DevicePowered(int ch) const
{
    return AdcCommon().DevicePowered(ch);
}

//---------------------------------------------------------------------------
//  XUInputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void  XUInputDevice::ChannelDisableAll()
{
     AdcCommon().ChannelDisableAll();
     AdcCommon().ChannelPowerDownAll();
}

//---------------------------------------------------------------------------
//  XUInputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void  XUInputDevice::ChannelEnabled(int ch, bool state)
{
     AdcCommon().ChannelEnabled(ch, state);
     ChannelPowered(ch, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XUInputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XUInputDevice::ChannelEnabled(int ch) const
{
    return AdcCommon().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  XUInputDevice::ActiveChannels() -- Return number of active channels
//---------------------------------------------------------------------------

unsigned int XUInputDevice::ActiveChannels()
{
    return AdcCommon().ActiveChannels();
}

//---------------------------------------------------------------------------
//  XUInputDevice::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XUInputDevice::ChannelPowered(int ch, bool state)
{
     AdcCommon().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::ChannelPowered() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

bool XUInputDevice::ChannelPowered(int ch) const
{
    return AdcCommon().ChannelPowered(ch);
}

//---------------------------------------------------------------------------
//  XUInputDevice::SoftwareTrigger() -- Enable Test Mode on Input
//---------------------------------------------------------------------------

void  XUInputDevice::SoftwareTrigger(bool state)
{
    Common().SoftwareTrigger(XUAFE_Common::tInput, state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::SoftwareTrigger() -- Enable Test Mode on Input
//---------------------------------------------------------------------------

void  XUInputDevice::SoftwareTrigger(AfeDirection which, bool state)
{
    XUAFE_Common::AfeDirection which2 = static_cast<XUAFE_Common::AfeDirection>(which);
    Common().SoftwareTrigger(which2, state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::TestModeEnabled() -- Enable Test Mode on Input
//---------------------------------------------------------------------------

void  XUInputDevice::TestModeEnabled(bool state, unsigned int mode)
{
    Common().TestModeEnabled(XUAFE_Common::tInput, state, mode);
}

//---------------------------------------------------------------------------
//  XUInputDevice::Decimation() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XUInputDevice::Decimation(unsigned int samples)
{
     AdcCommon().Decimation(samples);
}

//---------------------------------------------------------------------------
//  XUInputDevice::Decimation() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

unsigned int XUInputDevice::Decimation() const
{
    return AdcCommon().Decimation();
}

//---------------------------------------------------------------------------
//  XUInputDevice::TestFrequency() -- Set frequency for Test Mode on Input
//---------------------------------------------------------------------------

//void  XUInputDevice::TestFrequency(double frequency_hz)
//{
//    unsigned int channels = AdcCommon().ActiveChannels();
//    //    unsigned int dacdevs = ActiveDevices();
//    //    unsigned int factor;
//    //    if (channels==0 || dacdevs==0)
//    //        factor = 1;
//    //    else
//    //        factor = channels/dacdevs;
//    unsigned int factor;
//    if (channels==0)
//        factor = 1;
//    else
//        factor = channels;

//    AdcCommon().DacTestToneFrequency(Owner->DacClockFrequency(), frequency_hz, factor);
//}

//---------------------------------------------------------------------------
//  XUInputDevice::TestFrequency() -- Get frequency for Test Mode on Input
//---------------------------------------------------------------------------

//double  XUInputDevice::TestFrequency()
//{
//    unsigned int channels = AdcCommon().ActiveChannels();
//    if (channels==0)
//         channels = 1;
//    return AdcCommon().DacTestToneFrequency(Owner->DacClockFrequency(), channels);
//}

//---------------------------------------------------------------------------
//  XUInputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XUInputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    AdcSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XUInputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XUInputDevice::Spi(unsigned int device, unsigned int address)
{
    return AdcSpi().Spi(device, address);
}

//===========================================================================
//  CLASS XUInputDevice::TriggerCfgImpl  --
//===========================================================================
//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::FramedMode() -- Setter
//---------------------------------------------------------------------------

void  XUInputDevice::TriggerCfgImpl::FramedMode(bool state)
{
    InD->AdcCommon().FramedTriggerEnabled(state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::FramedMode() -- Getter
//---------------------------------------------------------------------------

bool XUInputDevice::TriggerCfgImpl::FramedMode() const
{
    return InD->AdcCommon().FramedTriggerEnabled();
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::Edge() -- Setter
//---------------------------------------------------------------------------

void  XUInputDevice::TriggerCfgImpl::Edge(bool state)
{
    InD->AdcCommon().EdgeTrigger(state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::Edge() -- Getter
//---------------------------------------------------------------------------

bool XUInputDevice::TriggerCfgImpl::Edge() const
{
    return InD->AdcCommon().EdgeTrigger();
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::External() -- Setter
//---------------------------------------------------------------------------

void  XUInputDevice::TriggerCfgImpl::External(bool state)
{
    InD->AdcCommon().ExternalTrigger(state);
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::External() -- Getter
//---------------------------------------------------------------------------

bool XUInputDevice::TriggerCfgImpl::External() const
{
    return InD->AdcCommon().ExternalTrigger();
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::FrameSize() -- Setter
//---------------------------------------------------------------------------

void  XUInputDevice::TriggerCfgImpl::FrameSize(unsigned int samples)
{
    InD->AdcCommon().FramedTriggerSize(samples);
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::FrameSize() -- Getter
//---------------------------------------------------------------------------

unsigned int XUInputDevice::TriggerCfgImpl::FrameSize() const
{
    return InD->AdcCommon().FramedTriggerSize();
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::ExternalSyncSource() -- Setter
//---------------------------------------------------------------------------

void  XUInputDevice::TriggerCfgImpl::ExternalSyncSource(AfeExtSyncOptions src)
{
    InD->Common().ExternalSyncSelection(
                static_cast<XUAFE_Common::AfeExtSyncOptions>(src) );
//    InD->CustomDacExternalSyncSource(src);
}

//---------------------------------------------------------------------------
//  XUInputDevice::TriggerCfgImpl::ExternalSyncSource() -- Getter
//---------------------------------------------------------------------------

XUInputDevice::AfeExtSyncOptions XUInputDevice::TriggerCfgImpl::ExternalSyncSource() const
{
    XUAFE_Common::AfeExtSyncOptions opt = InD->Common().ExternalSyncSelection();
    return static_cast<XUInputDevice::AfeExtSyncOptions>( opt );
}


//===========================================================================
//  CLASS XUOutputDevice  -- Partially Implemented X6 Output Device
//===========================================================================
//---------------------------------------------------------------------------
//  XUOutputDevice::Devices() -- return device count
//---------------------------------------------------------------------------

unsigned int  XUOutputDevice::Devices()
{
     return DacCommon().Devices();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ChannelsPerDevice() -- return channels per device
//---------------------------------------------------------------------------

unsigned int  XUOutputDevice::ChannelsPerDevice()
{
     return DacCommon().ChannelsPerDevice();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::Channels() -- return channels
//---------------------------------------------------------------------------

unsigned int  XUOutputDevice::Channels()
{
     return DacCommon().Channels();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::DeviceEnabled() -- Enable specified I/O devices
//---------------------------------------------------------------------------

void  XUOutputDevice::DeviceEnabled(int dev, bool state)
{
     DacCommon().DeviceIsEnabled(dev, state);
     DacCommon().DevicePowered(dev, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XUOutputDevice::DeviceEnabled() -- Query state of specified I/O devices
//---------------------------------------------------------------------------

bool XUOutputDevice::DeviceEnabled(int dev) const
{
    return DacCommon().DeviceIsEnabled(dev);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ActiveDevices() -- Return number of active devices
//---------------------------------------------------------------------------

unsigned int XUOutputDevice::ActiveDevices()
{
    return DacCommon().ActiveDevices();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::DevicePowered() -- Power specified I/O devices
//---------------------------------------------------------------------------

void  XUOutputDevice::DevicePowered(int ch, bool state)
{
     DacCommon().DevicePowered(ch, state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::DevicePowered() -- Query power state of an I/O devices
//---------------------------------------------------------------------------

bool XUOutputDevice::DevicePowered(int ch) const
{
    return DacCommon().DevicePowered(ch);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void  XUOutputDevice::ChannelDisableAll()
{
     DacCommon().ChannelDisableAll();
     DacCommon().ChannelPowerDownAll();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void  XUOutputDevice::ChannelEnabled(int ch, bool state)
{
     DacCommon().ChannelEnabled(ch, state);
     ChannelPowered(ch, state);  // have power state follow enabled state
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XUOutputDevice::ChannelEnabled(int ch) const
{
    return DacCommon().ChannelEnabled(ch);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ActiveChannels() -- Return number of active channels
//---------------------------------------------------------------------------

unsigned int XUOutputDevice::ActiveChannels()
{
    return DacCommon().ActiveChannels();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XUOutputDevice::ChannelPowered(int ch, bool state)
{
     DacCommon().ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::ChannelPowered() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

bool XUOutputDevice::ChannelPowered(int ch) const
{
    return DacCommon().ChannelPowered(ch);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::SoftwareTrigger() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  XUOutputDevice::SoftwareTrigger(bool state)
{
    Common().SoftwareTrigger(XUAFE_Common::tOutput, state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::SoftwareTrigger() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  XUOutputDevice::SoftwareTrigger(AfeDirection which, bool state)
{
    XUAFE_Common::AfeDirection which2 = static_cast<XUAFE_Common::AfeDirection>(which);
    Common().SoftwareTrigger(which2, state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TestModeEnabled() -- Enable Test Mode on Output
//---------------------------------------------------------------------------

void  XUOutputDevice::TestModeEnabled(bool state, unsigned int mode)
{
    Common().TestModeEnabled(XUAFE_Common::tOutput, state, mode);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::Decimation() -- Power specified I/O channel
//---------------------------------------------------------------------------

void  XUOutputDevice::Decimation(unsigned int samples)
{
     DacCommon().Decimation(samples);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::Decimation() -- Query power state of an I/O channel
//---------------------------------------------------------------------------

unsigned int XUOutputDevice::Decimation() const
{
    return DacCommon().Decimation();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TestFrequency() -- Set frequency for Test Mode on Output
//---------------------------------------------------------------------------

//void  XUOutputDevice::TestFrequency(double frequency_hz)
//{
//    unsigned int channels = DacCommon().ActiveChannels();
//    //    unsigned int dacdevs = ActiveDevices();
//    //    unsigned int factor;
//    //    if (channels==0 || dacdevs==0)
//    //        factor = 1;
//    //    else
//    //        factor = channels/dacdevs;
//    unsigned int factor;
//    if (channels==0)
//        factor = 1;
//    else
//        factor = channels;

//    DacCommon().DacTestToneFrequency(Owner->DacClockFrequency(), frequency_hz, factor);
//}

//---------------------------------------------------------------------------
//  XUOutputDevice::TestFrequency() -- Get frequency for Test Mode on Output
//---------------------------------------------------------------------------

//double  XUOutputDevice::TestFrequency()
//{
//    unsigned int channels = DacCommon().ActiveChannels();
//    if (channels==0)
//         channels = 1;
//    return DacCommon().DacTestToneFrequency(Owner->DacClockFrequency(), channels);
//}

//---------------------------------------------------------------------------
//  XUOutputDevice::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XUOutputDevice::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    DacSpi().Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XUOutputDevice::Spi(unsigned int device, unsigned int address)
{
    return DacSpi().Spi(device, address);
}

//===========================================================================
//  CLASS XUOutputDevice::TriggerCfgImpl  --
//===========================================================================
//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::FramedMode() -- Setter
//---------------------------------------------------------------------------

void  XUOutputDevice::TriggerCfgImpl::FramedMode(bool state)
{
    OutD->DacCommon().FramedTriggerEnabled(state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::FramedMode() -- Getter
//---------------------------------------------------------------------------

bool XUOutputDevice::TriggerCfgImpl::FramedMode() const
{
    return OutD->DacCommon().FramedTriggerEnabled();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::Edge() -- Setter
//---------------------------------------------------------------------------

void  XUOutputDevice::TriggerCfgImpl::Edge(bool state)
{
    OutD->DacCommon().EdgeTrigger(state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::Edge() -- Getter
//---------------------------------------------------------------------------

bool XUOutputDevice::TriggerCfgImpl::Edge() const
{
    return OutD->DacCommon().EdgeTrigger();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::External() -- Setter
//---------------------------------------------------------------------------

void  XUOutputDevice::TriggerCfgImpl::External(bool state)
{
    OutD->DacCommon().ExternalTrigger(state);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::External() -- Getter
//---------------------------------------------------------------------------

bool XUOutputDevice::TriggerCfgImpl::External() const
{
    return OutD->DacCommon().ExternalTrigger();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::FrameSize() -- Setter
//---------------------------------------------------------------------------

void  XUOutputDevice::TriggerCfgImpl::FrameSize(unsigned int samples)
{
    OutD->DacCommon().FramedTriggerSize(samples);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::FrameSize() -- Getter
//---------------------------------------------------------------------------

unsigned int XUOutputDevice::TriggerCfgImpl::FrameSize() const
{
    return OutD->DacCommon().FramedTriggerSize();
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::ExternalSyncSource() -- Setter
//---------------------------------------------------------------------------

void  XUOutputDevice::TriggerCfgImpl::ExternalSyncSource(AfeExtSyncOptions src)
{
    OutD->Common().ExternalSyncSelection(
                static_cast<XUAFE_Common::AfeExtSyncOptions>(src) );
//    OutD->CustomDacExternalSyncSource(src);
}

//---------------------------------------------------------------------------
//  XUOutputDevice::TriggerCfgImpl::ExternalSyncSource() -- Getter
//---------------------------------------------------------------------------

XUOutputDevice::AfeExtSyncOptions XUOutputDevice::TriggerCfgImpl::ExternalSyncSource() const
{
    XUAFE_Common::AfeExtSyncOptions opt = OutD->Common().ExternalSyncSelection();
    return static_cast<XUOutputDevice::AfeExtSyncOptions>( opt );
}

#ifdef _MSC_VER
    #pragma warning(pop)
#endif


} // namespace Innovative


