// XUDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef XUDeviceMap_MbH
#define XUDeviceMap_MbH

#include <MultiChanDeviceMap_Mb.h>
#include <X6SubDevice_Misc_Mb.h>
#include <XUSubDevice_Misc_Mb.h>
#include <X6SubDevice_Stream_Mb.h>
#include <XUSubDevice_Stream_Mb.h>
#include <XUSubDevice_AnalogIn_Mb.h>
#include <XUSubDevice_AnalogOut_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

class XUDeviceDriver;

//===========================================================================
//  CLASS IXUIoDevice  -- Common XU Board Configuration Interface
//===========================================================================

class IXUIoDevice
{
public:
    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essCarrier, essB2b };

    virtual ~IXUIoDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS IXUIoDevice::ITriggerCfg  -- Trigger Configuration Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ITriggerCfg
    {
    public:
        virtual ~ITriggerCfg()  {  }

        virtual void          FramedMode(bool state) = 0;
        virtual bool          FramedMode() const = 0;

        virtual void          Edge(bool state) = 0;
        virtual bool          Edge() const = 0;

        virtual void          External(bool state) = 0;
        virtual bool          External() const = 0;

        virtual void          FrameSize(unsigned int samples) = 0;
        virtual unsigned int  FrameSize() const = 0;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src) = 0;
        virtual AfeExtSyncOptions  ExternalSyncSource() const = 0;
    };
    //
    //  Interface Functions

    virtual unsigned int  Devices() = 0;
    virtual unsigned int  ChannelsPerDevice() = 0;
    virtual unsigned int  Channels() = 0;

    virtual void  DeviceDisableAll() = 0;
    virtual void  DeviceEnabled(int dev, bool state) = 0;
    virtual bool  DeviceEnabled(int dev) const = 0;

    virtual void  DevicePowered(int dev, bool state) = 0;
    virtual bool  DevicePowered(int dev) const = 0;

    virtual unsigned int  ActiveDevices() = 0;

    virtual void  ChannelDisableAll() = 0;
    virtual void  ChannelEnabled(int ch, bool state) = 0;
    virtual bool  ChannelEnabled(int ch) const = 0;

    virtual void  ChannelPowered(int ch, bool state) = 0;
    virtual bool  ChannelPowered(int ch) const = 0;

    virtual unsigned int  ActiveChannels() = 0;

    virtual ITriggerCfg &  Trigger() = 0;

    virtual void SoftwareTrigger(bool state) = 0;
    virtual void SoftwareTrigger(AfeDirection which, bool state) = 0;

    virtual void TestModeEnabled(bool state, unsigned int mode) = 0;
//    virtual void   TestFrequency(double frequency_hz) = 0;
//    virtual double TestFrequency() = 0;

    virtual void  Decimation(unsigned int samples) = 0;
    virtual unsigned int Decimation() const = 0;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data) = 0;
    virtual unsigned int Spi(unsigned int device, unsigned int address) = 0;

};

//===========================================================================
//  CLASS ISuppliesXUDeviceMapRequirements -- interfaces needed by XU DMap
//===========================================================================

class ISuppliesXUDeviceMapRequirements : public IRequires_Sysmon_AD5592R_Init,
                                         public IRequires_XUSioPll,
                                         public IRequires_Nothing
{
public:
    virtual ~ISuppliesXUDeviceMapRequirements()
        {}

};

//===========================================================================
//  CLASS XUDeviceMap  -- Register support for common XU Boards
//===========================================================================

class XUDeviceMap :  public MultiChanDeviceMap, public ISuppliesXUDeviceMapRequirements
{
public:
    friend class XUDac_DeviceMixin;
    //
    //  Member Functions
    XUDeviceMap(class XuBoard * owner);
    virtual ~XUDeviceMap();

    const IX6LogicVersionInfo&  Info()    {  return Info_SDev;  }
    X6Peripherals&              Periph()  {  return Periph_SDev;  }
    IFpaLoader &                VsProm()  {  return VsProm_SDev.VsProm();  }

    IIdRom &                    IdRom()   {  return IdRom_Intf;   }

    IUsesXUAlerts &             Alert()   {  return Alerts_SDev;  }
    XUDioPortIntf &             DioP()    {  return DioP_SDev;  }
    XUDioPortIntf &             DioN()    {  return DioN_SDev;  }
    XU_VFifoInfo &              VFifo()   {  return VFifo_SDev;    }
    XU_StreamTester &           StreamTester()   {  return StreamTester_SDev;    }
    XU_LogicSystemMonitor &     LogicSysMon()    {  return LogicSysMon_SDev;    }
    IMuxedAurora &              Aurora()         {  return Aurora_SDev;     }
    IXUSioPllControl &          SioPll()         {  return SioPll_SDev;   }


public: // pseudo-protected

protected:
    // Methods
    virtual void    ConstructCustomParts();            // board constructor time
    virtual void    PreOpenHardware();                 // before open time
    virtual void    OpenHardware();                    // after open time
    virtual void    PreconfigureHardware();            // preconfigure time
    virtual void    OnSingleStreamAtStartConfig();     // runs at first stream start (only) after PreconfigureHardware()
    virtual void    ConfigureHardware();               // stream configuration
    virtual void    OnBeforeStreamStart();             // before stream start customization
    virtual void    OnAfterStreamStart();              // after stream start customization
    virtual void    OnBeforeStreamStop();              // before stream stop customization
    virtual void    StreamStopHardware();              // after stream stop subdevice configuration

    XuBoard *                Owner;
    //
    //  AXI Master Bus Spaces
    AxiMasterSpace           AXI_Periph;
    AxiMasterSpace           AXI_SysMgmt;
    AxiMasterSpace           AXI_AFE;
    AxiMasterSpace           AXI_JESD_0;
    AxiMasterSpace           AXI_JESD_1;
    AxiMasterSpace           AXI_JESD_2;
    AxiMasterSpace           AXI_JESD_3;
    AxiMasterSpace           AXI_Custom;
    //
    //  Subdevices
    X6Info                   Info_SDev;
    X6Peripherals            Periph_SDev;
    XUAFE_Common             AFECommon_SDev;   // analog common registers
    XU_FpaLoader             VsProm_SDev;
    XU_FlashRom              FlashRom_SDev;
    XUAlertSystem            Alerts_SDev;
    XUDigitalIo              DioP_SDev;
    XUDigitalIo              DioN_SDev;
    XU_VFifoInfo             VFifo_SDev;
    XU_StreamTester          StreamTester_SDev;
    XUSystemMonitor          SysMon_SDev;
    XU_LogicSystemMonitor    LogicSysMon_SDev;
    XUAurora                 Aurora_SDev;
    XUSioPll                 SioPll_SDev;
    //
    //  Predefined IDROM pages
    X6SystemIdromPage            System_PG;
    X6IdromPage                  AdcCal_PG;
    X6IdromPage                  DacCal_PG;
    X6IdromPage                  UserRom_PG;

    //=======================================================================
    //  CLASS XUIdromIo  -- Access to IDROM Objects
    //=======================================================================
    class XUIdromIo : public IIdRom
    {
    public:

        virtual IIdRomDevice &   Device() { return Owner->FlashRom_SDev;  }
        //  Page access
        virtual X6SystemIdromPage &   System()   { return Owner->System_PG;  }
        virtual X6IdromPage &         AdcCal()   { return Owner->AdcCal_PG;  }
        virtual X6IdromPage &         DacCal()   { return Owner->DacCal_PG;  }
        virtual X6IdromPage &         User()     { return Owner->UserRom_PG;  }

        XUIdromIo(XUDeviceMap * owner) :  Owner(owner)
            {}
        virtual ~XUIdromIo()
            {}

        private:
            XUDeviceMap *  Owner;
    };

    XUIdromIo                    IdRom_Intf;
    friend class XUIdromIo;

    ii32 PeekJesdLogic(unsigned int which, ii32 offset);
    void PokeJesdLogic(unsigned int which, ii32 offset, ii32 data);

public:
    //
    //  Mixin Interface
    XUAFE_Common & Get_AFECommon_SDev() {  return AFECommon_SDev;  }
    //virtual double  Get_AdcClockRate() = 0;
    virtual double  Get_DacClockRate() = 0;

    virtual double  StandardClockFrequency() = 0;   // PRI Frequency

};

//===========================================================================
//  CLASS ISuppliesAdded_XUAdcDeviceMap_Requirements -- interfaces added
//===========================================================================

class ISuppliesAdded_XUAdcDeviceMap_Requirements :  public IRequires_XUCommonParts,
                                                    public IRequires_PriClockDevice,
                                                    public IRequires_MuxedAdcSpi,
                                                    public IRequires_AdcCalParts
{
public:
    virtual ~ISuppliesAdded_XUAdcDeviceMap_Requirements()
        {}

};

//===========================================================================
//  CLASS XUAdcDeviceMap  -- input only card device map
//===========================================================================

class XUAdcDeviceMap : public XUDeviceMap,
                       public ISuppliesAdded_XUAdcDeviceMap_Requirements
{
public:
    friend class XUInputDevice;

    XUAdcDeviceMap(class XuBoard * owner);
    virtual ~XUAdcDeviceMap()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:
    XUAFE_AdcCommon                  AFEAdcC_SDev;
    X6AFE_PulseRepetitionInterval    AFEAdcPri_SDev;
    XUAFE_AdcSpi                     AFEAdcSpi_SDev;
    XU_AdcCalCoefficientLoad         AFEAdcCal_SDev;

};

//===========================================================================
//  CLASS ISuppliesAdded_XUDac_DeviceMixin_Requirements
//===========================================================================
//  XUDac_DeviceMixin requirements

class ISuppliesAdded_XUDac_DeviceMixin_Requirements : public IRequires_XUCommonParts,
                                                      public IRequires_PriClockDevice,
                                                      public IRequires_MuxedDacSpi,
                                                      public IRequires_DacCalParts,
                                                      public IRequires_XUPatternMode
{
public:
    virtual ~ISuppliesAdded_XUDac_DeviceMixin_Requirements()
        {}

};

//===========================================================================
//  CLASS XUDac_DeviceMixin - Dac standard hardware device pack
//===========================================================================

class XUDac_DeviceMixin : public ISuppliesAdded_XUDac_DeviceMixin_Requirements
{
public:
    friend class XUOutputDevice;

    XUDac_DeviceMixin(XUDeviceMap * owner);
    virtual ~XUDac_DeviceMixin()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();
    //
    //  Subdevice Customization - implement board specific pieces
    virtual unsigned int GetDacSpiDeviceCount() = 0;

protected:
    //
    //  Subdevices
    XUDeviceMap *                    DDMxOwner;
    XUAFE_DacCommon                  AFEDacC_SDev;
    X6AFE_PulseRepetitionInterval    AFEDacPri_SDev;
    XUAFE_DacSpi                     AFEDacSpi_SDev;
    XU_DacCalCoefficientLoad         AFEDacCal_SDev;
    XU_DacPatternModeSystem          DacPattern_SDev;


    XUAFE_Common &  Common() {  return DDMxOwner->Get_AFECommon_SDev();  }
    double          DacClockFrequency() { return DDMxOwner->Get_DacClockRate(); }

};

//===========================================================================
//  CLASS XUDacDeviceMap  -- output only card device map
//===========================================================================

class XUDacDeviceMap : public XUDeviceMap, public XUDac_DeviceMixin//,
                      //public ISuppliesAddedXUDacDMRequirements
{
public:
    friend class XUOutputDevice;

    XUDacDeviceMap(class XuBoard * owner);
    virtual ~XUDacDeviceMap()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:

};

//===========================================================================
//  CLASS XUBidirDeviceMap  -- output only card device map
//===========================================================================

class XUBidirDeviceMap : public XUAdcDeviceMap, public XUDac_DeviceMixin//,
                        //public ISuppliesAddedXUBidirDMRequirements
{
public:
    friend class XUOutputDevice;

    XUBidirDeviceMap(class XuBoard * owner);
    virtual ~XUBidirDeviceMap()
        {}

    virtual void    ConstructCustomParts();    // call in most derived ctor
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();
    virtual void    StreamStopHardware();

protected:

};

//===========================================================================
//  CLASS XUInputDevice  -- Partially Implemented XUBoard Input Device
//===========================================================================

class XUInputDevice : public IXUIoDevice
{
public:

    XUInputDevice(XUAdcDeviceMap * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~XUInputDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUInputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IXUIoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(XUInputDevice * in_dev)
            :  InD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        XUInputDevice * InD;
    };

    friend class XUInputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual IX6AnalogIoInfo &  Info() = 0;

    virtual unsigned int  Devices();
    virtual unsigned int  ChannelsPerDevice();
    virtual unsigned int  Channels();

    virtual void  DeviceDisableAll() {  ChannelDisableAll();  }
    virtual void  DeviceEnabled(int dev, bool state);
    virtual bool  DeviceEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual ITriggerCfg &                   Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return AdcPri();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    virtual void TestFrequency(double /*frequency_hz*/) {}
    virtual double TestFrequency() {  return 0.0;  }

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void CustomAdcExternalSyncSource(AfeExtSyncOptions /*src*/) {}

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

protected:
    XUAdcDeviceMap *        Owner;
    TriggerCfgImpl          TriggerCfg;

    //  shortcuts to subdevices
    XUAFE_AdcCommon &       AdcCommon() {  return Owner->AFEAdcC_SDev;  }
    const XUAFE_AdcCommon & AdcCommon() const {  return Owner->AFEAdcC_SDev;  }
    XUAFE_Common &          Common() {  return Owner->AFECommon_SDev;  }
    const XUAFE_Common &    Common() const {  return Owner->AFECommon_SDev;  }
    X6AFE_PulseRepetitionInterval &        AdcPri() {  return Owner->AFEAdcPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  AdcPri() const {  return Owner->AFEAdcPri_SDev;  }
    XU_AdcCalCoefficientLoad &          AdcCal() {  return Owner->AFEAdcCal_SDev;  }
    const XU_AdcCalCoefficientLoad &    AdcCal() const {  return Owner->AFEAdcCal_SDev;  }
    XUAFE_AdcSpi &          AdcSpi() {  return Owner->AFEAdcSpi_SDev;  }
    const XUAFE_AdcSpi &    AdcSpi() const {  return Owner->AFEAdcSpi_SDev;  }

};


//===========================================================================
//  CLASS XUOutputDevice  -- Base for XU Carrier Output Devices
//===========================================================================

class XUOutputDevice : public IXUIoDevice
{
public:

    XUOutputDevice(XUDac_DeviceMixin * owner)
        :  Owner(owner), TriggerCfg(this)
        {  }
    virtual ~XUOutputDevice()  {  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUOutputDevice::TriggerCfgImpl  -- Trigger Configuration Impl
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerCfgImpl : public IXUIoDevice::ITriggerCfg
    {
    public:
        TriggerCfgImpl(XUOutputDevice * in_dev)
            :  OutD(in_dev)
            {  }
        virtual ~TriggerCfgImpl()  {  }

        virtual void          FramedMode(bool state);
        virtual bool          FramedMode() const;

        virtual void          Edge(bool state);
        virtual bool          Edge() const;

        virtual void          External(bool state);
        virtual bool          External() const;

        virtual void          FrameSize(unsigned int samples);
        virtual unsigned int  FrameSize() const;

        virtual void               ExternalSyncSource(AfeExtSyncOptions src);
        virtual AfeExtSyncOptions  ExternalSyncSource() const;

    protected:
        XUOutputDevice * OutD;
    };

    friend class XUOutputDevice::TriggerCfgImpl;
    //
    //  Interface Functions
    virtual unsigned int  Devices();
    virtual unsigned int  ChannelsPerDevice();
    virtual unsigned int  Channels();

    virtual void  DeviceDisableAll() {  ChannelDisableAll();  }
    virtual void  DeviceEnabled(int dev, bool state);
    virtual bool  DeviceEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    virtual void  ChannelDisableAll();
    virtual void  ChannelEnabled(int ch, bool state);
    virtual bool  ChannelEnabled(int ch) const;

    virtual unsigned int  ActiveChannels();

    virtual void  ChannelPowered(int ch, bool state);
    virtual bool  ChannelPowered(int ch) const;

    virtual ITriggerCfg &  Trigger() {  return TriggerCfg;  }
    virtual IPulseRepetitionIntervalIntf &  Pulse()  {  return DacPri();  }
    virtual IXUPatternModeSystem & Pattern() {  return DacPattern();  }

    virtual void SoftwareTrigger(bool state);
    virtual void SoftwareTrigger(AfeDirection which, bool state);

    virtual void TestModeEnabled(bool state, unsigned int mode);
    //virtual void TestFrequency(double /*frequency_hz*/);
    //virtual double TestFrequency();

    virtual void  Decimation(unsigned int samples);
    virtual unsigned int Decimation() const;

    virtual void  Spi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int Spi(unsigned int device, unsigned int address);

//    virtual void CustomDacExternalSyncSource(AfeExtSyncOptions /*src*/) {}


protected:
    XUDac_DeviceMixin *   Owner;
    TriggerCfgImpl  TriggerCfg;

    //  shortcuts to subdevices
    XUAFE_DacCommon &        DacCommon() {  return Owner->AFEDacC_SDev;  }
    const XUAFE_DacCommon &  DacCommon() const {  return Owner->AFEDacC_SDev;  }
    XUAFE_Common &           Common() {  return Owner->Common();  }
    const XUAFE_Common &     Common() const {  return Owner->Common();  }
    XUAFE_DacSpi &          DacSpi() {  return Owner->AFEDacSpi_SDev;  }
    const XUAFE_DacSpi &    DacSpi() const {  return Owner->AFEDacSpi_SDev;  }
    X6AFE_PulseRepetitionInterval &        DacPri() {  return Owner->AFEDacPri_SDev;  }
    const X6AFE_PulseRepetitionInterval &  DacPri() const {  return Owner->AFEDacPri_SDev;  }
    XU_DacCalCoefficientLoad &          DacCal() {  return Owner->AFEDacCal_SDev;  }
    const XU_DacCalCoefficientLoad &    DacCal() const {  return Owner->AFEDacCal_SDev;  }
    XU_DacPatternModeSystem &        DacPattern() {  return Owner->DacPattern_SDev;  }
    const XU_DacPatternModeSystem &  DacPattern() const {  return Owner->DacPattern_SDev;  }

};

//=============================================================================
//  CLASS XUStandardPriClockDevice  --  Normal impl to access clock for PRI
//=============================================================================

class XUStandardPriClockDevice : public IPriClockDevice
{
public:
    virtual double   ClockFrequency()
        {  return Owner->StandardClockFrequency();  }
    virtual int   CalculatePriDivisor()
        {  return Divisor;  }

    XUStandardPriClockDevice(XUDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~XUStandardPriClockDevice()
        {}

private:
    XUDeviceMap * Owner;
    int Divisor;
};

//=============================================================================
//  CLASS XUStandardFrameCalcDevice  --  Supply Divider on Frame ticks
//=============================================================================

class XUStandardFrameCalcDevice : public IFrameCalcDevice
{
public:
    virtual unsigned int  CalculateFrameDivisor()
        {  return Divisor;  }

    XUStandardFrameCalcDevice(XUDeviceMap * owner, int divisor)
        :  Owner(owner), Divisor(divisor)
        {}
    virtual ~XUStandardFrameCalcDevice()
        {}

private:
    XUDeviceMap * Owner;
    int Divisor;
};



#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
