//---------------------------------------------------------------------------
// XUSubDevice_AnalogIn_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosureas
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.

#include <XUSubDevice_AnalogIn_Mb.h>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS XUAFE_AdcCommon -- X6 AFE Adc Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUAFE_AdcCommon
//---------------------------------------------------------------------------

//namespace {

//    double ii_power_of_two(int exponent)
//    {
//        const int max_exponent = 30;
//        double result = 1.0;
//        while (exponent > 0)
//            {
//            int shift;
//            //  we can't shift by more than 31 bits accurately, so break up
//            //    into groups of max_exponent
//            if (exponent > max_exponent)
//                shift = max_exponent;
//            else
//                shift = exponent;
//            unsigned int partial = 1<<shift;
//            result *= static_cast<double>(partial);

//            exponent -= shift;
//            }
//        return result;
//    }
//}

const unsigned int MaxXUAdcChannels = 64;

//---------------------------------------------------------------------------
//  constructor for class XUAFE_AdcCommon
//---------------------------------------------------------------------------

XUAFE_AdcCommon::XUAFE_AdcCommon(IRequires_XUCommonParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       AdcEnable_Reg(space, regs.AfeAdcEnable_Addr),
       AdcPower_Reg(space, regs.AfeAdcPower_Addr),
       AdcTriggerCfgReg(space, regs.AfeAdcTriggerCfg_Addr),
       AdcDecimationReg(space, regs.AfeAdcDecimation_Addr),
       //AdcPhaseIncrReg(space, regs.AfeAdcPhaseIncr_Addr),
       AdcResetReg(space, regs.AfeAdcReset_Addr),
       FDevices(0), FChannelsPerDevice(0), FPhysDevices(0), FChannelsPerPhysDevice(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAFE_AdcCommon
//---------------------------------------------------------------------------

XUAFE_AdcCommon::~XUAFE_AdcCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAFE_AdcCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_XUCommonParts::devAdc);
    MDC->GetAnalogDeviceInfo(IRequires_XUCommonParts::devAdc, FDevices, FChannelsPerDevice);
    MDC->GetAnalogPhysicalDeviceInfo(IRequires_XUCommonParts::devAdc, FPhysDevices, FChannelsPerPhysDevice);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::OpenHardware()
//---------------------------------------------------------------------------

void  XUAFE_AdcCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAFE_AdcCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::ChannelDisableAll()
{
    AdcEnable_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  XUAFE_AdcCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<MaxXUAdcChannels; i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::ChannelEnabled(int ch, bool state)
{
    MDC->SetXuStreamChannelActive(IRequires_XUCommonParts::devAdc, ch, state);  // stream state follows Adc state
    AdcEnable_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::ChannelEnabled(int ch) const
{
    return AdcEnable_Reg.Bit(ch);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::ChannelPowerDownAll()
{
    AdcPower_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::ChannelPowered(int ch, bool state)
{
    AdcPower_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::ChannelPowered(int ch) const
{
    return AdcPower_Reg.Bit(ch);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::DeviceIsEnabled() -- Enable an I/O device
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::DeviceIsEnabled(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::DeviceIsEnabled() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::DeviceIsEnabled(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelEnabled(base);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::DevicePowered() -- Enable an I/O device
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::DevicePowered(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::DevicePowered() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::DevicePowered(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelPowered(base);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ActiveDevices() -- return count of active devices
//---------------------------------------------------------------------------

unsigned int  XUAFE_AdcCommon::ActiveDevices()
{
    unsigned int active_devices(0);
    for (unsigned int i=0; i<Devices(); i++)
        if (DeviceIsEnabled(i)==true)
            active_devices++;

    return active_devices;
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    AdcTriggerCfgReg.AdcFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int XUAFE_AdcCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = AdcTriggerCfgReg.AdcFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::EdgeTrigger(bool state)
{
    AdcTriggerCfgReg.AdcRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::EdgeTrigger() const
{
    return AdcTriggerCfgReg.AdcRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::ExternalTrigger(bool state)
{
    AdcTriggerCfgReg.AdcExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::ExternalTrigger() const
{
    return AdcTriggerCfgReg.AdcExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::FramedTriggerEnabled(bool state)
{
    AdcTriggerCfgReg.AdcFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool XUAFE_AdcCommon::FramedTriggerEnabled() const
{
    return AdcTriggerCfgReg.AdcFramedMode.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::Decimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::Decimation(unsigned int samples)
{
    AdcDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::Decimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int XUAFE_AdcCommon::Decimation() const
{
    return AdcDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::AdcTestToneFrequency() -- Set Reset tone_freq specified I/O channel
//---------------------------------------------------------------------------

//void XUAFE_AdcCommon::AdcTestToneFrequency(double Adc_clock_freq, double tone_freq, int channels)
//{
//    Debug.Log("Test Frequency", tone_freq);
//    Debug.Log("Adc Frequency", Adc_clock_freq);
//    Debug.Log("Channels", channels);

//    double Two32 = ii_power_of_two(32);

//    double phase_incr = tone_freq * (Two32) / Adc_clock_freq ;
//    Debug.Log("Phase Incr", phase_incr);

//    AdcPhaseIncrReg.Value(static_cast<unsigned int>(phase_incr));
//}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::AdcTestToneFrequency() -- Query Reset tone_freq of an I/O channel
//---------------------------------------------------------------------------

//double XUAFE_AdcCommon::AdcTestToneFrequency(double Adc_clock_freq, int /*channels*/) const
//{
//    double Two32 = ii_power_of_two(32);
//    unsigned int raw_phase_incr = AdcPhaseIncrReg.Value();

//    double tone_freq = static_cast<double>(raw_phase_incr) / (Two32) * Adc_clock_freq;
//    return tone_freq;
//}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::AdcReset() -- Reset Adcs
//---------------------------------------------------------------------------

void XUAFE_AdcCommon::AdcReset(bool state)
{
    AdcResetReg.Bit(0, state);
    AdcResetReg.Bit(1, state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::DRR_Bit() -- write DRR register bit (board specific)
//---------------------------------------------------------------------------

void  XUAFE_AdcCommon::DRR_Bit(unsigned int bit, bool state)
{
    AdcResetReg.Bit(bit, state);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::DRR_Bit() -- read DRR register bit (board specific)
//---------------------------------------------------------------------------

bool  XUAFE_AdcCommon::DRR_Bit(unsigned int bit)
{
    return AdcResetReg.Bit(bit);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::PhysicalDeviceEnabled() --
//---------------------------------------------------------------------------
//  Physical device should be enabled if any channel on it is enabled...

bool XUAFE_AdcCommon::PhysicalDeviceEnabled(unsigned int phys_dev) const
{
    bool enabled(false);
    if (phys_dev>=FPhysDevices)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int end   = start + FChannelsPerPhysDevice;

    for (unsigned int ch=start;  ch<end; ch++)
        enabled |= ChannelEnabled(ch);
    return enabled;
}

//---------------------------------------------------------------------------
//  XUAFE_AdcCommon::PhysicalChannelEnabled() --
//---------------------------------------------------------------------------
//  return enabled state for the Nth channel on device D...

bool XUAFE_AdcCommon::PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const
{
    if (phys_dev>=FPhysDevices || device_ch>=FChannelsPerPhysDevice)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int ch  = start + device_ch;

    return ChannelEnabled(ch);
}

//===========================================================================
//  CLASS XUAFE_AdcSpi -- XU AFE Adc Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUAFE_AdcSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XUAFE_AdcSpi
//---------------------------------------------------------------------------

XUAFE_AdcSpi::XUAFE_AdcSpi(IRequires_MuxedAdcSpi * mdc, IAddressingSpace & space, XUSpiDevice::MapRegisters & regs)
    :  MDC(mdc), SpiDev(space, regs, "XU Adc"), SpiCount(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAFE_AdcSpi
//---------------------------------------------------------------------------

XUAFE_AdcSpi::~XUAFE_AdcSpi()
{
}

//---------------------------------------------------------------------------
//  XUAFE_AdcSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAFE_AdcSpi::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SpiCount = MDC->GetAdcSpiDeviceCount();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  XUAFE_AdcSpi::OpenHardware()
//---------------------------------------------------------------------------

void  XUAFE_AdcSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_AdcSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAFE_AdcSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_AdcSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XUAFE_AdcSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiCount)
        SpiDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XUAFE_AdcSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XUAFE_AdcSpi::Spi(unsigned int device, unsigned int address)
{
    if (device < SpiCount)
        return SpiDev.Spi(device, address);
    else
        return 0;
}

//---------------------------------------------------------------------------
//  XUAFE_AdcSpi::SpiWriteAll() -- write to all SPI device
//---------------------------------------------------------------------------

void  XUAFE_AdcSpi::SpiWriteAll(unsigned int address, unsigned int data)
{
    SpiDev.SpiWriteAll(address, data);
}



}  // namespace Innovative



