// XUSubDevice_AnalogIn_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef XUSubDevice_AnalogIn_MbH
#define XUSubDevice_AnalogIn_MbH

#include <X6Support_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SpiDevice_Mb.h>
#include <X6SubDevice_AnalogIn_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS XUAFE_AdcCommon -- XU AFE Adc Common Registers "SubDevice"
//===========================================================================

class XUAFE_AdcCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_AdcCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeAdcEnable_Addr;
        unsigned int   AfeAdcPower_Addr;
        unsigned int   AfeAdcTriggerCfg_Addr;
        unsigned int   AfeAdcDecimation_Addr;
        unsigned int   AfeAdcReset_Addr;
    };

    XUAFE_AdcCommon(IRequires_XUCommonParts * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XUAFE_AdcCommon();

    //
    //  Methods
    unsigned int  Devices() const  {  return FDevices;  }
    unsigned int  ChannelsPerDevice() const {  return FChannelsPerDevice;  }
    unsigned int  Channels() const {  return FDevices * FChannelsPerDevice;  }

    virtual void  DeviceIsEnabled(int dev, bool state);
    virtual bool  DeviceIsEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    //  Physical Device Enable Mapping
    bool  PhysicalDeviceEnabled(unsigned int phys_dev) const;
    bool  PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const;

    //void   AdcTestToneFrequency(double Adc_clock_freq, double tone_freq, int channels); // in Hz
    //double AdcTestToneFrequency(double Adc_clock_freq, int channels) const;

    void   AdcReset(bool state);
    //  Hit (other) bits in AdcReset Register
    void   DRR_Bit(unsigned int bit, bool state);
    bool   DRR_Bit(unsigned int bit);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_AdcCommon::AdcTriggerCfgRegister -- Configure Adc Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  AdcFrameSize;
        RegisterBit       AdcRisingEdge;
        RegisterBit       AdcFramedMode;
        RegisterBit       AdcExternalTrigger;

    public:
        AdcTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcFrameSize(*this, 0, 24),   AdcRisingEdge(*this, 29),
               AdcFramedMode(*this, 30),     AdcExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_AdcCommon::AdcDecimationRegister  -- Adc Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AdcDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        AdcDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_XUCommonParts *     MDC;
    IFrameCalcDevice *            Dev;
    Register                      AdcEnable_Reg;
    Register                      AdcPower_Reg;
    AdcTriggerCfgRegister         AdcTriggerCfgReg;
    AdcDecimationRegister         AdcDecimationReg;
    //Register                      AdcPhaseIncrReg;
    Register                      AdcResetReg;

    unsigned int                  FDevices;
    unsigned int                  FChannelsPerDevice;
    unsigned int                  FPhysDevices;
    unsigned int                  FChannelsPerPhysDevice;

};

//===========================================================================
//  CLASS XUAFE_AdcSpi -- Fmc Muxed Adc Spi Device Manager "SubDevice"
//===========================================================================

class XUAFE_AdcSpi : public IX6SubDevice
{
public:

    XUAFE_AdcSpi(IRequires_MuxedAdcSpi * owner, IAddressingSpace & space, FmcSpiDevice::MapRegisters & regs);
    virtual ~XUAFE_AdcSpi();

    unsigned int AdcSpiCount() {  return SpiCount;  }
    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address);

    void  SpiWriteAll(unsigned int address, unsigned int data);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_MuxedAdcSpi *  MDC;
    XUSpiDevice              SpiDev;
    unsigned int             SpiCount;
};

typedef X6AFE_AdcCalibration  XU_AdcCalCoefficientLoad;


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
