//---------------------------------------------------------------------------
// XUSubDevice_AnalogOut_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.

#include <XUSubDevice_AnalogOut_Mb.h>
#include <SystemSupport_Mb.h>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS XUAFE_DacCommon -- X6 AFE Dac Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUAFE_DacCommon
//---------------------------------------------------------------------------

//namespace {

//    double ii_power_of_two(int exponent)
//    {
//        const int max_exponent = 30;
//        double result = 1.0;
//        while (exponent > 0)
//            {
//            int shift;
//            //  we can't shift by more than 31 bits accurately, so break up
//            //    into groups of max_exponent
//            if (exponent > max_exponent)
//                shift = max_exponent;
//            else
//                shift = exponent;
//            unsigned int partial = 1<<shift;
//            result *= static_cast<double>(partial);

//            exponent -= shift;
//            }
//        return result;
//    }
//}

const unsigned int MaxXUDacChannels = 64;

//---------------------------------------------------------------------------
//  constructor for class XUAFE_DacCommon
//---------------------------------------------------------------------------

XUAFE_DacCommon::XUAFE_DacCommon(IRequires_XUCommonParts * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc), Dev(0),
       DacEnable_Reg(space, regs.AfeDacEnable_Addr),
       DacPower_Reg(space, regs.AfeDacPower_Addr),
       DacTriggerCfgReg(space, regs.AfeDacTriggerCfg_Addr),
       DacDecimationReg(space, regs.AfeDacDecimation_Addr),
       //DacPhaseIncrReg(space, regs.AfeDacPhaseIncr_Addr),
       DacResetReg(space, regs.AfeDacReset_Addr),
       FDevices(0), FChannelsPerDevice(0), FPhysDevices(0), FChannelsPerPhysDevice(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAFE_DacCommon
//---------------------------------------------------------------------------

XUAFE_DacCommon::~XUAFE_DacCommon()
{
    delete Dev;  Dev = 0;
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAFE_DacCommon::ConstructCustomParts()
{
    Dev = MDC->MakeFrameCalcDevice(IRequires_XUCommonParts::devDac);
    MDC->GetAnalogDeviceInfo(IRequires_XUCommonParts::devDac, FDevices, FChannelsPerDevice);
    MDC->GetAnalogPhysicalDeviceInfo(IRequires_XUCommonParts::devDac, FPhysDevices, FChannelsPerPhysDevice);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::OpenHardware()
//---------------------------------------------------------------------------

void  XUAFE_DacCommon::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAFE_DacCommon::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ChannelDisableAll() -- Disable all channels
//---------------------------------------------------------------------------

void XUAFE_DacCommon::ChannelDisableAll()
{
    DacEnable_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ActiveChannels() -- return count of active channels
//---------------------------------------------------------------------------

unsigned int  XUAFE_DacCommon::ActiveChannels()
{
    unsigned int active_channels(0);
    for (unsigned int i=0; i<MaxXUDacChannels; i++)
        if (ChannelEnabled(i)==true)
            active_channels++;

    return active_channels;
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ChannelEnabled() -- Enable specified I/O channel
//---------------------------------------------------------------------------

void XUAFE_DacCommon::ChannelEnabled(int ch, bool state)
{
    MDC->SetXuStreamChannelActive(IRequires_XUCommonParts::devDac, ch, state);  // stream state follows DAC state
    DacEnable_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ChannelEnabled() -- Query state of specified I/O channel
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::ChannelEnabled(int ch) const
{
    return DacEnable_Reg.Bit(ch);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ChannelPowerDownAll() -- Power down all channels
//---------------------------------------------------------------------------

void XUAFE_DacCommon::ChannelPowerDownAll()
{
    DacPower_Reg.Value(0);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ChannelPowered() -- Power specified I/O channel
//---------------------------------------------------------------------------

void XUAFE_DacCommon::ChannelPowered(int ch, bool state)
{
    DacPower_Reg.Bit(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ChannelPowered() -- Query Power state of an I/O channel
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::ChannelPowered(int ch) const
{
    return DacPower_Reg.Bit(ch);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DeviceIsEnabled() -- Enable an I/O device
//---------------------------------------------------------------------------

void XUAFE_DacCommon::DeviceIsEnabled(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelEnabled(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DeviceIsEnabled() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::DeviceIsEnabled(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelEnabled(base);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DevicePowered() -- Enable an I/O device
//---------------------------------------------------------------------------

void XUAFE_DacCommon::DevicePowered(int dev, bool state)
{
    unsigned int base = dev*ChannelsPerDevice();
    for (unsigned int ch=base; ch<(base+ChannelsPerDevice()); ch++)
        ChannelPowered(ch, state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DevicePowered() -- Query state of specified I/O dev
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::DevicePowered(int dev) const
{
    // if in device mode, all are on or off together
    unsigned int base = dev*ChannelsPerDevice();

    return ChannelPowered(base);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ActiveDevices() -- return count of active devices
//---------------------------------------------------------------------------

unsigned int  XUAFE_DacCommon::ActiveDevices()
{
    unsigned int active_devices(0);
    for (unsigned int i=0; i<Devices(); i++)
        if (DeviceIsEnabled(i)==true)
            active_devices++;

    return active_devices;
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::FramedTriggerSize() -- Set Frame Trigger Window Size
//---------------------------------------------------------------------------

void XUAFE_DacCommon::FramedTriggerSize(unsigned int samples)
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = samples / divisor;
    DacTriggerCfgReg.DacFrameSize.Value(reg_val);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::FramedTriggerSize() -- Read Frame Trigger Window Size
//---------------------------------------------------------------------------

unsigned int XUAFE_DacCommon::FramedTriggerSize() const
{
    unsigned int divisor = Dev->CalculateFrameDivisor();
    unsigned int reg_val = DacTriggerCfgReg.DacFrameSize.Value();
    return reg_val * divisor;
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::EdgeTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XUAFE_DacCommon::EdgeTrigger(bool state)
{
    DacTriggerCfgReg.DacRisingEdge.Value(state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::EdgeTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::EdgeTrigger() const
{
    return DacTriggerCfgReg.DacRisingEdge.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ExternalTrigger() -- Set Edge/Level Trigger Mode
//---------------------------------------------------------------------------

void XUAFE_DacCommon::ExternalTrigger(bool state)
{
    DacTriggerCfgReg.DacExternalTrigger.Value(state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::ExternalTrigger() -- Read Edge/Level Trigger Mode
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::ExternalTrigger() const
{
    return DacTriggerCfgReg.DacExternalTrigger.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::FramedTriggerEnabled() -- Set Frame Trigger Mode
//---------------------------------------------------------------------------

void XUAFE_DacCommon::FramedTriggerEnabled(bool state)
{
    DacTriggerCfgReg.DacFramedMode.Value(state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::FramedTriggerEnabled() -- Read Frame Trigger Mode
//---------------------------------------------------------------------------

bool XUAFE_DacCommon::FramedTriggerEnabled() const
{
    return DacTriggerCfgReg.DacFramedMode.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::Decimation() -- Set Decimation Count
//---------------------------------------------------------------------------

void XUAFE_DacCommon::Decimation(unsigned int samples)
{
    DacDecimationReg.Decimation.Value(samples);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::Decimation() -- Read Decimation Count
//---------------------------------------------------------------------------

unsigned int XUAFE_DacCommon::Decimation() const
{
    return DacDecimationReg.Decimation.Value();
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DacTestToneFrequency() -- Set Reset tone_freq specified I/O channel
//---------------------------------------------------------------------------

//void XUAFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, double tone_freq, int channels)
//{
//    Debug.Log("Test Frequency", tone_freq);
//    Debug.Log("Dac Frequency", dac_clock_freq);
//    Debug.Log("Channels", channels);

//    double Two32 = ii_power_of_two(32);

//    double phase_incr = tone_freq * (Two32) / dac_clock_freq ;
//    Debug.Log("Phase Incr", phase_incr);

//    DacPhaseIncrReg.Value(static_cast<unsigned int>(phase_incr));
//}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DacTestToneFrequency() -- Query Reset tone_freq of an I/O channel
//---------------------------------------------------------------------------

//double XUAFE_DacCommon::DacTestToneFrequency(double dac_clock_freq, int /*channels*/) const
//{
//    double Two32 = ii_power_of_two(32);
//    unsigned int raw_phase_incr = DacPhaseIncrReg.Value();

//    double tone_freq = static_cast<double>(raw_phase_incr) / (Two32) * dac_clock_freq;
//    return tone_freq;
//}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DacReset() -- Reset DACs
//---------------------------------------------------------------------------

void XUAFE_DacCommon::DacReset(bool state)
{
    DacResetReg.Bit(0, state);
    DacResetReg.Bit(1, state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DRR_Bit() -- write DRR register bit (board specific)
//---------------------------------------------------------------------------

void  XUAFE_DacCommon::DRR_Bit(unsigned int bit, bool state)
{
    DacResetReg.Bit(bit, state);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::DRR_Bit() -- read DRR register bit (board specific)
//---------------------------------------------------------------------------

bool  XUAFE_DacCommon::DRR_Bit(unsigned int bit)
{
    return DacResetReg.Bit(bit);
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::PhysicalDeviceEnabled() --
//---------------------------------------------------------------------------
//  Physical device should be enabled if any channel on it is enabled...

bool XUAFE_DacCommon::PhysicalDeviceEnabled(unsigned int phys_dev) const
{
    bool enabled(false);
    if (phys_dev>=FPhysDevices)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int end   = start + FChannelsPerPhysDevice;

    for (unsigned int ch=start;  ch<end; ch++)
        enabled |= ChannelEnabled(ch);
    return enabled;
}

//---------------------------------------------------------------------------
//  XUAFE_DacCommon::PhysicalChannelEnabled() --
//---------------------------------------------------------------------------
//  return enabled state for the Nth channel on device D...

bool XUAFE_DacCommon::PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const
{
    if (phys_dev>=FPhysDevices || device_ch>=FChannelsPerPhysDevice)
        return false;

    unsigned int start = phys_dev*FChannelsPerPhysDevice;
    unsigned int ch  = start + device_ch;

    return ChannelEnabled(ch);
}

//===========================================================================
//  CLASS XUAFE_DacSpi -- XU AFE DAC Spi Device Manager "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUAFE_DacSpi
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XUAFE_DacSpi
//---------------------------------------------------------------------------

XUAFE_DacSpi::XUAFE_DacSpi(IRequires_MuxedDacSpi * mdc, IAddressingSpace & space, XUSpiDevice::MapRegisters & regs)
    :  MDC(mdc), SpiDev(space, regs, "XU Dac"), SpiCount(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAFE_DacSpi
//---------------------------------------------------------------------------

XUAFE_DacSpi::~XUAFE_DacSpi()
{
}

//---------------------------------------------------------------------------
//  XUAFE_DacSpi::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAFE_DacSpi::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    SpiCount = MDC->GetDacSpiDeviceCount();  // Request mdc to build SPI devices
}

//---------------------------------------------------------------------------
//  XUAFE_DacSpi::OpenHardware()
//---------------------------------------------------------------------------

void  XUAFE_DacSpi::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_DacSpi::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAFE_DacSpi::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XUAFE_DacSpi::Spi() -- write to a SPI device
//---------------------------------------------------------------------------

void  XUAFE_DacSpi::Spi(unsigned int device, unsigned int address, unsigned int data)
{
    if (device < SpiCount)
        SpiDev.Spi(device, address, data);
}

//---------------------------------------------------------------------------
//  XUAFE_DacSpi::Spi() -- read from a SPI device
//---------------------------------------------------------------------------

unsigned int  XUAFE_DacSpi::Spi(unsigned int device, unsigned int address)
{
    if (device < SpiCount)
        return SpiDev.Spi(device, address);
    else
        return 0;
}

//---------------------------------------------------------------------------
//  XUAFE_DacSpi::SpiWriteAll() -- write to all SPI device
//---------------------------------------------------------------------------

void  XUAFE_DacSpi::SpiWriteAll(unsigned int address, unsigned int data)
{
    SpiDev.SpiWriteAll(address, data);
}

//===========================================================================
//  CLASS XU_DacPatternModeSystem -- X6 ADC Calibration "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class XU_DacPatternModeSystem
//---------------------------------------------------------------------------

const int DefaultPatternModeOnValue =  0x3;
const int DefaultPatternModeOffValue = 0x0;

//---------------------------------------------------------------------------
//  constructor for class XU_DacPatternModeSystem
//---------------------------------------------------------------------------

XU_DacPatternModeSystem::XU_DacPatternModeSystem(IRequires_XUPatternMode* mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       ModeEnableReg(space, regs.PatternModeEnable_Addr),
       PlaybackCmdReg(space, regs.PlaybackCommand_Addr),
       CmdFifo0_Reg(space, regs.CmdFifo0_Addr),
       CmdFifo1_Reg(space, regs.CmdFifo1_Addr),
       CmdFifo2_Reg(space, regs.CmdFifo2_Addr),
       CmdFifo3_Reg(space, regs.CmdFifo3_Addr),
       FModeEnabled(false),
       FModeOnValue(DefaultPatternModeOnValue), FModeOffValue(DefaultPatternModeOffValue),
       PatternModeAlignment(1), PatternModeScaleFactor(1)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_DacPatternModeSystem
//---------------------------------------------------------------------------

XU_DacPatternModeSystem::~XU_DacPatternModeSystem()
{
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XU_DacPatternModeSystem::ConstructCustomParts()
{
    PatternModeAlignment = MDC->GetPatternModeAlignment();
    PatternModeScaleFactor = MDC->GetPatternModeScaleFactor();
    //SingleMemoryFlag = MDC->GetPatternModeSingleMemoryFlag();
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::OpenHardware()
//---------------------------------------------------------------------------

void  XU_DacPatternModeSystem::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_DacPatternModeSystem::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::MemoryInit()
//---------------------------------------------------------------------------

unsigned int XU_DacPatternModeSystem::MemoryInit()
{
    return ModeEnableReg.MemoryInitDone.Value();
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::GetMemoryStatus()
//---------------------------------------------------------------------------

unsigned int XU_DacPatternModeSystem::GetMemoryStatus()
{
    return ModeEnableReg.MemoryTestError.Value();
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::MemoryTestEnable()
//---------------------------------------------------------------------------

void XU_DacPatternModeSystem::MemoryTestEnable(unsigned int val)
{
    ModeEnableReg.MemoryTestEnable.Value(val);
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::DoMemoryTest() -- Runs firmware mem test for 60 seconds
//---------------------------------------------------------------------------

bool XU_DacPatternModeSystem::DoMemoryTest()
{
    bool Test = false;
    ModeEnableReg.MemoryTestEnable.Value(1);
    Innovative::Sleep(60000);
    unsigned int status = GetMemoryStatus();
    if (status == 0)
        Test = true;
    else
        Test = false;
    ModeEnableReg.MemoryTestEnable.Value(0);
    return Test;
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::PatternModeEnable() - enable pattern mode
//---------------------------------------------------------------------------

void  XU_DacPatternModeSystem::PatternModeEnable(bool state)
{
    int output_val = (state) ? FModeOnValue : FModeOffValue;
    ModeEnableReg.PlaybackEnable.Value(output_val);
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::PatternModeEnable() - return pattern mode state
//---------------------------------------------------------------------------

bool  XU_DacPatternModeSystem::PatternModeEnable()
{
    if (ModeEnableReg.PlaybackEnable.Value() == (unsigned int)FModeOnValue)
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::PatternFifoReady() - return pattern mode state
//---------------------------------------------------------------------------

bool  XU_DacPatternModeSystem::PatternFifoReady(unsigned int channel)
{
    unsigned int ch_bits = 1<<channel;
    unsigned int rdy_bits = PlaybackCmdReg.FifoReady.Value();
    if (rdy_bits & ch_bits)
        return true;
    else
        return false;
}

//---------------------------------------------------------------------------
//  XU_DacPatternModeSystem::SendPatternInfo()
//---------------------------------------------------------------------------
const unsigned int TagFieldStartBit     =  0;
const unsigned int TagFieldSize         =  8;

const unsigned int WriteCommandBit      =  16;
const unsigned int ReadCommandBit       =  17;
const unsigned int RepeatPacketBit      =  18;

bool XU_DacPatternModeSystem::SendPatternInfo(unsigned int channel,
                      char tag_value, unsigned int start_addr,
                      unsigned int buffer_size_ints, unsigned int rep_count,
                      PatternCommands cmd, PatternRepeatType repeat)
{

    unsigned int Packet[4];

    if (PatternFifoReady(channel)==false)
        return false;   // FIFO full, can't load pattern.

    // Align the start address
    start_addr = (start_addr/PatternModeAlignment)*PatternModeAlignment;
    buffer_size_ints = (buffer_size_ints/PatternModeAlignment)*PatternModeAlignment;

    Packet[0] = buffer_size_ints/PatternModeScaleFactor;
    Packet[1] = start_addr/PatternModeScaleFactor;
    Packet[2] = 0;
    Packet[3] = rep_count;

    bool write_bit_value;
    bool read_bit_value;
    switch (cmd)
        {
        case piPlay:
        default:
            write_bit_value = true;   // data follows, and play it
            read_bit_value = true;
            break;
        case piLoad:
            write_bit_value = true;
            read_bit_value = false;   // data follows, don't play it
            break;
        case piReplay:
            write_bit_value = false;
            read_bit_value = true;   // no data follows, play loaded pattern
            break;
        case piHalt:
            write_bit_value = false;
            read_bit_value = false;   // no data follows, halt playing
            break;
        }

    bool repeat_bit_value;
    switch (repeat)     // set repeat bit based off of mode
        {
        default:
        case prPlayAgain:
            repeat_bit_value = true;
            break;
        case prFlatline:
            repeat_bit_value = false;
            break;
        }

    PtrBitManipulator InfoWord( &Packet[2] );
    //  Fill Info Word
    InfoWord.Field(TagFieldStartBit, TagFieldSize, tag_value);
    InfoWord.Bit(WriteCommandBit, write_bit_value);
    InfoWord.Bit(ReadCommandBit, read_bit_value);
    InfoWord.Bit(RepeatPacketBit, repeat_bit_value);
    //
    //  Write Packet To Registers
    unsigned int ch_bits = 1<<channel;
    // Write Data
    CmdFifo0_Reg.Value(Packet[0]);
    CmdFifo1_Reg.Value(Packet[1]);
    CmdFifo2_Reg.Value(Packet[2]);
    CmdFifo3_Reg.Value(Packet[3]);
    // Write "command"
    PlaybackCmdReg.ChannelSelect.Value(ch_bits);
    return true;
}


}  // namespace Innovative



