// XUSubDevice_AnalogOut_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef XUSubDevice_AnalogOut_MbH
#define XUSubDevice_AnalogOut_MbH

#include <X6Support_Mb.h>
#include <HardwareRegister_Mb.h>
#include <SpiDevice_Mb.h>
#include <X6SubDevice_AnalogOut_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS XUAFE_DacCommon -- XU AFE DAC Common Registers "SubDevice"
//===========================================================================

class XUAFE_DacCommon : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_DacCommon::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeDacEnable_Addr;
        unsigned int   AfeDacPower_Addr;
        unsigned int   AfeDacTriggerCfg_Addr;
        unsigned int   AfeDacDecimation_Addr;
        unsigned int   AfeDacReset_Addr;
    };

    XUAFE_DacCommon(IRequires_XUCommonParts * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XUAFE_DacCommon();

    //
    //  Methods
    unsigned int  Devices() const  {  return FDevices;  }
    unsigned int  ChannelsPerDevice() const {  return FChannelsPerDevice;  }
    unsigned int  Channels() const {  return FDevices * FChannelsPerDevice;  }

    virtual void  DeviceIsEnabled(int dev, bool state);
    virtual bool  DeviceIsEnabled(int dev) const;

    virtual void  DevicePowered(int dev, bool state);
    virtual bool  DevicePowered(int dev) const;

    virtual unsigned int  ActiveDevices();

    void  ChannelDisableAll();
    void  ChannelEnabled(int ch, bool state);
    bool  ChannelEnabled(int ch) const;

    unsigned int  ActiveChannels();

    void  ChannelPowerDownAll();
    void  ChannelPowered(int ch, bool state);
    bool  ChannelPowered(int ch) const;

    void  FramedTriggerSize(unsigned int samples);
    unsigned int FramedTriggerSize() const;

    //  Trigger Modes
    void  FramedTriggerEnabled(bool state);
    bool  FramedTriggerEnabled() const;
    void  EdgeTrigger(bool state);
    bool  EdgeTrigger() const;
    void  ExternalTrigger(bool state);
    bool  ExternalTrigger() const;

    void  Decimation(unsigned int samples);
    unsigned int Decimation() const;

    //  Physical Device Enable Mapping
    bool  PhysicalDeviceEnabled(unsigned int phys_dev) const;
    bool  PhysicalChannelEnabled(unsigned int phys_dev, unsigned int device_ch) const;

    //void   DacTestToneFrequency(double dac_clock_freq, double tone_freq, int channels); // in Hz
    //double DacTestToneFrequency(double dac_clock_freq, int channels) const;

    void   DacReset(bool state);
    //  Hit (other) bits in DacReset Register
    void   DRR_Bit(unsigned int bit, bool state);
    bool   DRR_Bit(unsigned int bit);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_DacCommon::DacTriggerCfgRegister -- Configure DAC Trigger
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacTriggerCfgRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  DacFrameSize;
        RegisterBit       DacRisingEdge;
        RegisterBit       DacFramedMode;
        RegisterBit       DacExternalTrigger;

    public:
        DacTriggerCfgRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               DacFrameSize(*this, 0, 24),   DacRisingEdge(*this, 29),
               DacFramedMode(*this, 30),     DacExternalTrigger(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_DacCommon::DacDecimationRegister  -- DAC Decimation
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DacDecimationRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Decimation;

    public:
        DacDecimationRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Decimation(*this, 0, 12)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_XUCommonParts *     MDC;
    IFrameCalcDevice *            Dev;
    Register                      DacEnable_Reg;
    Register                      DacPower_Reg;
    DacTriggerCfgRegister         DacTriggerCfgReg;
    DacDecimationRegister         DacDecimationReg;
    //Register                      DacPhaseIncrReg;
    Register                      DacResetReg;

    unsigned int                  FDevices;
    unsigned int                  FChannelsPerDevice;
    unsigned int                  FPhysDevices;
    unsigned int                  FChannelsPerPhysDevice;

};

//===========================================================================
//  CLASS XUAFE_DacSpi -- Fmc Muxed DAC Spi Device Manager "SubDevice"
//===========================================================================

class XUAFE_DacSpi : public IX6SubDevice
{
public:

    XUAFE_DacSpi(IRequires_MuxedDacSpi * owner, IAddressingSpace & space, FmcSpiDevice::MapRegisters & regs);
    virtual ~XUAFE_DacSpi();

    unsigned int DacSpiCount() {  return SpiCount;  }
    //
    //  Methods
    void  Spi(unsigned int device, unsigned int address, unsigned int data);
    unsigned int Spi(unsigned int device, unsigned int address);

    void  SpiWriteAll(unsigned int address, unsigned int data);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_MuxedDacSpi *  MDC;
    XUSpiDevice              SpiDev;
    unsigned int             SpiCount;
};

typedef X6AFE_DacCalibration  XU_DacCalCoefficientLoad;

//===========================================================================
//  CLASS IRequires_XUPatternMode -- XU "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs pattern mode data plus a baseboard ptr

class IRequires_XUPatternMode
{
public:
    virtual unsigned int GetPatternModeAlignment() = 0;
    virtual unsigned int GetPatternModeScaleFactor() = 0;
    //virtual bool         GetPatternModeSingleMemoryFlag() = 0;

    virtual ~IRequires_XUPatternMode()
        {}
};

//===========================================================================
//  CLASS IXUPatternModeSystem -- X6 Pattern Mode Control Interface
//===========================================================================

class IXUPatternModeSystem
{
public:
    // first commands are "logic" commands. Latter 2 are SW commands.
    enum PatternCommands { piPlay, piLoad, piReplay, piHalt, piMemoryReplay=0x8, piHaltAll };
    enum PatternRepeatType { prFlatline, prPlayAgain };

    virtual ~IXUPatternModeSystem()
        {}

    virtual void  PatternModeEnable(bool state) = 0;
    virtual bool  PatternModeEnable() = 0;

    virtual bool  PatternFifoReady(unsigned int channel) = 0;
    virtual bool  SendPatternInfo(unsigned int channel,
                                  char tag_value, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int count,
                                  PatternCommands cmd = piPlay,
                                  PatternRepeatType repeat = prPlayAgain ) = 0;

    //Memory test
    virtual void            MemoryTestEnable(unsigned int val) = 0;
    virtual unsigned int    GetMemoryStatus() = 0;
    virtual unsigned int    MemoryInit() = 0;
    virtual bool            DoMemoryTest() = 0;

    virtual unsigned int  Alignment() = 0;
    virtual unsigned int  ScaleFactor() = 0;

};

//===========================================================================
//  CLASS XU_DacPatternModeSystem -- XU DAC Pattern Mode "SubDevice"
//===========================================================================

//const unsigned int kPatternModePacketSize = 4;

class XU_DacPatternModeSystem : public IX6SubDevice, public IXUPatternModeSystem
{
public:
    //=======================================================================
    //  CLASS XU_DacPatternModeSystem::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   PatternModeEnable_Addr;
        unsigned int   PlaybackCommand_Addr;
        unsigned int   CmdFifo0_Addr;
        unsigned int   CmdFifo1_Addr;
        unsigned int   CmdFifo2_Addr;
        unsigned int   CmdFifo3_Addr;
    };


    XU_DacPatternModeSystem(IRequires_XUPatternMode * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XU_DacPatternModeSystem();

    virtual void  PatternModeEnable(bool state);
    virtual bool  PatternModeEnable();

    virtual bool  PatternFifoReady(unsigned int channel);
    virtual bool  SendPatternInfo(unsigned int channel,
                                  char tag_value, unsigned int start_addr,
                                  unsigned int buffer_size_ints, unsigned int count,
                                  IXUPatternModeSystem::PatternCommands cmd = piPlay,
                                  IXUPatternModeSystem::PatternRepeatType repeat = prPlayAgain );

    virtual unsigned int  Alignment() {  return PatternModeAlignment;  }
    virtual unsigned int  ScaleFactor() {  return PatternModeScaleFactor;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //Memory test
    virtual void            MemoryTestEnable(unsigned int val);
    virtual unsigned int    GetMemoryStatus();
    virtual unsigned int    MemoryInit();
    virtual bool            DoMemoryTest(); //pass is true fail is false


    void            DefineModeEnableValues(int on_bits, int off_bits)
                        { FModeOnValue = on_bits,  FModeOffValue = off_bits;  }


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_DacPatternModeSystem::ModeEnableRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ModeEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  MemoryInitDone;
        RegisterBitGroup  PlaybackEnable;
        RegisterBitGroup  MemoryTestEnable;
        RegisterBitGroup  MemoryTestError;

    public:
        ModeEnableRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               MemoryInitDone(*this, 4, 2),
               PlaybackEnable(*this, 8, 2),
               MemoryTestEnable(*this, 12, 2),
               MemoryTestError(*this, 16, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_DacPatternModeSystem::PlaybackCmdRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class PlaybackCmdRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  ChannelSelect;
        RegisterBitGroup  WriteCmdPending;
        RegisterBitGroup  ReadCmdPending;
        RegisterBitGroup  FifoReady;
        RegisterBitGroup  FifoFull;
        RegisterBitGroup  FifoEmpty;

    public:
        PlaybackCmdRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ChannelSelect(*this, 0, 2),
               WriteCmdPending(*this, 8, 2),
               ReadCmdPending(*this, 12, 2),
               FifoReady(*this, 16, 2),
               FifoFull(*this, 20, 2),
               FifoEmpty(*this, 24, 2)
            {
            }
    };

private:
    IRequires_XUPatternMode *  MDC;
    ModeEnableRegister    ModeEnableReg;
    PlaybackCmdRegister   PlaybackCmdReg;
    Register              CmdFifo0_Reg;
    Register              CmdFifo1_Reg;
    Register              CmdFifo2_Reg;
    Register              CmdFifo3_Reg;

    bool  FModeEnabled;
    int   FModeOnValue;
    int   FModeOffValue;

    unsigned int  PatternModeAlignment;
    unsigned int  PatternModeScaleFactor;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
