//---------------------------------------------------------------------------
// XUSubDevice_Misc_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.

#include <XUSubDevice_Misc_Mb.h>
#include <SystemSupport_Mb.h>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif


//===========================================================================
//  CLASS XU_RomControl -- XU Rom Programming Shared Code
//===========================================================================
//---------------------------------------------------------------------------
//  XU_RomControl::FillOpcodeData() -- Initialize Opcode Bits for command
//---------------------------------------------------------------------------

void XU_RomControl::FillOpcodeData()
{
    OpcodeDecode[0x66] = 0x00;  /* Reset Enable */  OpcodeDecode[0x99] = 0x00;  /* Reset Memory */
    OpcodeDecode[0x9F] = 0x07;  /* Read ID */       OpcodeDecode[0x5A] = 0x1F;  /* Read SFDP */
    OpcodeDecode[0x03] = 0x17;  /* Read    */       OpcodeDecode[0x0B] = 0x1F;  /* Fast Read */
    OpcodeDecode[0x06] = 0x00;  /* WriteEN */       OpcodeDecode[0x04] = 0x00;  /* Write Disable */
    OpcodeDecode[0x05] = 0x03;  /* Read Status */   OpcodeDecode[0x70] = 0x03;  /* Read Flag Status */
    OpcodeDecode[0xB5] = 0x03;  /* Read NV Cfg */   OpcodeDecode[0x65] = 0x03;  /* Read ENH NV Cfg */
    OpcodeDecode[0xC8] = 0x03;  /* Read Ext Adr */  OpcodeDecode[0x01] = 0x02;  /* Write Status */
    OpcodeDecode[0xB1] = 0x04;  /* Write NV Cfg */  OpcodeDecode[0x81] = 0x02;  /* Write Vol Cfg */
    OpcodeDecode[0x61] = 0x02;  /* Write ENH Vol */ OpcodeDecode[0xC5] = 0x02;  /* Write Ext Adr Reg */
    OpcodeDecode[0x50] = 0x00;  /* Clear Flags */   OpcodeDecode[0x02] = 0x36;  /* Page Program */
    OpcodeDecode[0x52] = 0x10;  /* 32K Erase */     OpcodeDecode[0x20] = 0x10;  /* 4K Erase */
    OpcodeDecode[0xD8] = 0x10;  /* Sector Erase */  OpcodeDecode[0xC7] = 0x00;  /* Bulk Erase */
    OpcodeDecode[0x75] = 0x00;  /* Prog Suspend */  OpcodeDecode[0x7A] = 0x00;  /* Prog Resume */

    OpcodeDecode[0x2D] = 0x03;  /* Read Prot */     OpcodeDecode[0x2C] = 0x04;  /* Prog Prot */
    OpcodeDecode[0xE8] = 0x17;  /* Read Lock */     OpcodeDecode[0xE5] = 0x12;  /* Write Lock */
    OpcodeDecode[0xE4] = 0x00;  /* Erase Lock */
}

//---------------------------------------------------------------------------
//  XU_RomControl::SetBlockProtection() -- Impose all block protection
//---------------------------------------------------------------------------

void  XU_RomControl::SetBlockProtection()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x01, 0x00, 0xfc);  // WRSR (write status register)
    //    SRWD is 1, using the hardware protect signal.
    Flash_NotBusy_T_ms(1000);  // make sure RDSR busy status is complete
}

//---------------------------------------------------------------------------
//  XU_RomControl::ReleaseBlockProtection() -- Remove all block protection
//---------------------------------------------------------------------------

void  XU_RomControl::ReleaseBlockProtection()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x01, 0x00, 0x80);  // WRSR (write status register)
    // ...the above sets BP0-BP3 to 0, removing all protection
    //    SRWD is 1, using the hardware protect signal.
    Flash_NotBusy_T_ms(1000);  // make sure RDSR busy status is complete
}

//---------------------------------------------------------------------------
//  XU_RomControl::IsBlockProtectionReleased() -- Check block protection
//---------------------------------------------------------------------------

bool  XU_RomControl::IsBlockProtectionReleased()
{   
    unsigned int rdsr = Rdsr();
    if ((rdsr&0xFC)==0x80)
        {
        return true;  // unprotected
        }
    else
        {
        return false;  // protected
        }
}

//---------------------------------------------------------------------------
//  XU_RomControl::UpdateAddress() -- update 4 byte address bits
//---------------------------------------------------------------------------
//  when changing addresses, check bits above 3 bytes versus saved data. If changing,
//    apply new upper bits to hardware

void  XU_RomControl::UpdateAddress(unsigned int address)
{
    unsigned int upper_bits = address>>24;
    upper_bits &= 0x3;
    if (ExtAddrBits != upper_bits)
        {
        // update the extended address bits
        WriteExtendedAddressBits(upper_bits);
        ExtAddrBits = upper_bits;
        }
}

//---------------------------------------------------------------------------
//  XU_RomControl::WriteExtendedAddressBits() -- Write extended address bits
//---------------------------------------------------------------------------

void  XU_RomControl::WriteExtendedAddressBits(unsigned int addr_bits)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0xc5, 0x00, addr_bits);  // write extended address register
    Flash_NotBusy_T_ms(1000);  // make sure any busy status is complete
}

//---------------------------------------------------------------------------
//  XU_RomControl::ErasePage() -- erase 1 IDROM page
//---------------------------------------------------------------------------

void  XU_RomControl::ErasePage(unsigned int page)
{
    unsigned int addr = page*Map.Page_Size;
    UpdateAddress(addr);
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x20, addr);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  XU_RomControl::EraseSector() -- erase 1 IDROM sector
//---------------------------------------------------------------------------

void  XU_RomControl::EraseSector(unsigned int sector)
{
    unsigned int addr = sector*Map.Sector_Size;
    UpdateAddress(addr);
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0xd8, addr);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  XU_RomControl::EraseAll() -- erase entire IDROM
//---------------------------------------------------------------------------

void  XU_RomControl::EraseAll()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0xc7);

    Flash_NotBusy();   // wait for chip to report completed Erase
}

//---------------------------------------------------------------------------
//  XU_RomControl::ReadPage() -- read 1 IDROM page
//---------------------------------------------------------------------------
//  Size argument is in 32 bit words

void  XU_RomControl::ReadPage(unsigned int page, int * pkt, unsigned int size)
{
    unsigned int page_addr = page*Map.Page_Size;
    UpdateAddress(page_addr);

    size_t rpg_limit = std::min<size_t>(static_cast<size_t>(Map.Page_Size/sizeof(int)), size);

    //  Read one 32 bit word at a time into memory
    for (size_t i=0; i<rpg_limit; i++)
        {
        unsigned int addr = static_cast<unsigned int>(page_addr + i*sizeof(int));
        unsigned int value = Read32(addr);
        *pkt++ = value;
        }
}

//---------------------------------------------------------------------------
//  XU_RomControl::WritePage() -- write 1 IDROM page
//---------------------------------------------------------------------------
//  Size argument is in 32 bit words

void  XU_RomControl::WritePage(unsigned int page, int * pkt, unsigned int size)
{
    unsigned int page_addr = page*Map.Page_Size;
    UpdateAddress(page_addr);

    size_t wpg_limit = std::min<size_t>(static_cast<size_t>(Map.Page_Size/sizeof(int)), size);

    //  Program one 32 bit word at a time into memory
    for (size_t i=0; i<wpg_limit; i++)
        {
        unsigned int addr =  static_cast<unsigned int>(page_addr + i*sizeof(int));
        Write32(addr, pkt[i]);
        }
}

//---------------------------------------------------------------------------
//  XU_RomControl::Read32() -- read 1 IDROM page
//---------------------------------------------------------------------------

unsigned int  XU_RomControl::Read32(unsigned int addr)
{
    return FlashOp(0x03, addr);
}

//---------------------------------------------------------------------------
//  XU_RomControl::Write32() -- write 1 word
//---------------------------------------------------------------------------

void  XU_RomControl::Write32(unsigned int addr, unsigned int value)
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x02, addr, value );

    Flash_QuickNotBusy();
}

//---------------------------------------------------------------------------
//  XU_RomControl::SetProgramPageMode() -- Set Page Mode bit
//---------------------------------------------------------------------------

void  XU_RomControl::SetProgramPageMode(bool state)
{
    WritePageModeBit(state);
}

//---------------------------------------------------------------------------
//  XU_RomControl::SetPageAddress() -- Set Page Mode Address
//---------------------------------------------------------------------------

void  XU_RomControl::SetPageAddress(unsigned int addr)
{
    UpdateAddress(addr);
    WriteAddr(addr);
}

//---------------------------------------------------------------------------
//  XU_RomControl::Enqueue32() -- Add data to page mode queue
//---------------------------------------------------------------------------

void  XU_RomControl::Enqueue32(unsigned int value)
{
    WriteData(value);
}

//---------------------------------------------------------------------------
//  XU_RomControl::WaitForPageComplete() -- Wait for write to finish
//---------------------------------------------------------------------------

void  XU_RomControl::WaitForPageComplete()
{
    // Wait up to 2000 uS for pending flash write to be valid
    Flash_NotBusy_T_us(2000);
}

//---------------------------------------------------------------------------
//  XU_RomControl::PageProgram() -- Start a page mode access
//---------------------------------------------------------------------------

void  XU_RomControl::PageProgram()
{
    FlashOp(0x06);   // WREN - enable writing
    FlashOp(0x2);
}

//---------------------------------------------------------------------------
//  XU_RomControl::Rdsr() -- Flash Operation - Read status register
//---------------------------------------------------------------------------

unsigned int  XU_RomControl::Rdsr()
{
    Flash_Ready_T(1000);

    // Write opcode to trigger the transaction
    WriteOpcode(0x5, OpcodeDecode[0x5]);

    Flash_ReadValid();

    unsigned int data = ReadData();

//    std::stringstream msg;
//    msg << std::hex << "Rdsr:"  << data;
//    TraceDebug(msg.str());

    return data;
}

//---------------------------------------------------------------------------
//  XU_RomControl::JdecId() -- Flash Instruction - JDEC-ID
//---------------------------------------------------------------------------

unsigned int  XU_RomControl::JdecId()
{
    return FlashOp(0x9F, 0);   // address unused
}

//---------------------------------------------------------------------------
//  XU_RomControl::Rdid() -- Flash Instruction - RDID
//---------------------------------------------------------------------------

unsigned int  XU_RomControl::Rdid()
{
    return JdecId();
}

//---------------------------------------------------------------------------
//  XU_RomControl::Reset() -- Flash Instruction - Reset
//---------------------------------------------------------------------------

void  XU_RomControl::Reset()
{
    FlashOp(0x66);   // Reset Enable
    FlashOp(0x99);   // Reset Memory
}

//---------------------------------------------------------------------------
//  XU_RomControl::FlashOp() -- primitive Flash Operation - "command-like" ops
//---------------------------------------------------------------------------

void  XU_RomControl::FlashOp(unsigned int opcode)
{
    Flash_Ready();

    // Write opcode to trigger the transaction
    WriteOpcode(opcode, OpcodeDecode[opcode]);
}

//--------------- ------------------------------------------------------------
//  XU_RomControl::FlashOp() -- primitive Flash Operation - "read-like" ops
//---------------------------------------------------------------------------

unsigned int  XU_RomControl::FlashOp(unsigned int opcode, unsigned int addr)
{
    Flash_Ready();

    // Write address
    WriteAddr(addr);

    // Write opcode to trigger the transaction
    WriteOpcode(opcode, OpcodeDecode[opcode]);

    Flash_ReadValid();

    unsigned int data = ReadData();
    return data;
}

//---------------------------------------------------------------------------
//  XU_RomControl::FlashOp() -- primitive Flash Operation - "write-like" ops
//---------------------------------------------------------------------------

void  XU_RomControl::FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
{
    Flash_Ready();

    // Write data
    WriteData(data);

    // Write address
    WriteAddr(addr);

    // Write opcode to trigger the transaction
    WriteOpcode(opcode, OpcodeDecode[opcode]);

}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_Ready() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_Ready()
{
    Flash_Ready_T(200);
}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_Ready_T() -- wait for flash ready indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_Ready_T(unsigned int max_wait_us)
{
    const unsigned int reps = 10;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to "max_wait_us" uS for pending flash to be ready
    for (unsigned int i = 0; i < reps; ++i)
        {
        bool ready = ReadReady();
        if (ready)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Ready Failure (>" << max_wait_us << "us)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_ReadValid() -- wait for flash readvalid indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_ReadValid()
{
    // Wait up to 200 uS for pending flash read to be valid
    for (unsigned int i = 0; i < 10; ++i)
        {
        bool readvalid = ReadDataValid();
        if (readvalid)
            return;
        else
            Innovative::uSleep(20);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Read Valid Failure (>200us)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_NotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_NotBusy()
{
    Flash_NotBusy_T_ms(5000);  // wait up to 5000 ms
}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_QuickNotBusy() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_QuickNotBusy()
{
    Flash_NotBusy_T_us(200);  // wait up to 200 us
}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_NotBusy_T_ms() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_NotBusy_T_ms(unsigned int max_wait_ms)
{
    const unsigned int reps = 40;
    const unsigned int delay = max_wait_ms/reps;

    // Wait up to  "max_wait_ms" mS for Rdsr bit 0 to clear
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::Sleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Not Busy Failure (>" << max_wait_ms << "ms)";
    TraceDebug(msg.str());

}

//---------------------------------------------------------------------------
//  XU_RomControl::Flash_NotBusy_T_us() -- wait for flash busy=0 indicator
//---------------------------------------------------------------------------

void  XU_RomControl::Flash_NotBusy_T_us(unsigned int max_wait_us)
{
    const unsigned int reps = 20;
    const unsigned int delay = max_wait_us/reps;

    // Wait up to  "max_wait_us" uS for Rdsr bit 0 to clear
    for (unsigned int i = 0; i < reps; ++i)
        {
        unsigned int rdsr = Rdsr();
        if ((rdsr&0x1)==false)
            return;
        else
            Innovative::uSleep(delay);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " FlashRom Not Busy Failure (>" << max_wait_us << "us)";
    TraceDebug(msg.str());

}
//===========================================================================
//  CLASS XU_FpaLoader -- XU FPA Loader "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUFpaLoader
//---------------------------------------------------------------------------

XU_RomControl::RomMapInfo  RomMT25Q_128_MapInfo =
{
    4096,          //  Page_Count;
    4096,          //  Page_Size (bytes);
    256,           //  Sector_Count;
    65536,         //  Sector_Size (bytes);
};

XU_RomControl::RomMapInfo  RomMT25Q_512_MapInfo =
{
    16384,         //  Page_Count;
    4096,          //  Page_Size (bytes);
    1024,          //  Sector_Count;
    65536,         //  Sector_Size (bytes);
};

//---------------------------------------------------------------------------
//  constructor for class XUFpaLoader
//---------------------------------------------------------------------------

XU_FpaLoader::XU_FpaLoader(IRequires_Nothing * mdc, RomType which_rom,
                           IAddressingSpace & space, MapRegisters & regs)
    :  XU_RomControl(SelectRomMap(which_rom)),
       MDC(mdc),
       AddressControlReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr),
       Chip(MakeChipObject(which_rom)), RawProm(this),
       Thread( new XUFpaPersonality( Chip ))
{
    Thread.SetFullEraseMode(false);  // only erase Image Size
}

//---------------------------------------------------------------------------
//  destructor for class XU_FpaLoader
//---------------------------------------------------------------------------

XU_FpaLoader::~XU_FpaLoader()
{
    Thread.Shutdown();

    delete Chip;
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::OpenHardware()
//---------------------------------------------------------------------------

void  XU_FpaLoader::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Thread.OnOpen();
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_FpaLoader::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::WritePageModeBit() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FpaLoader::WritePageModeBit(bool bit)
{
    AddressControlReg.PageMode.Value(bit);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::WriteOpcode() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FpaLoader::WriteOpcode(unsigned int opcode, unsigned int opcode_decode)
{
    MemoryBitManipulator OpReg(0);
    OpReg.Field(0, 8, opcode);
    OpReg.Field(8, 6, opcode_decode);

    OpcodeReg.Value(OpReg.Value());
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::WriteAddr() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FpaLoader::WriteAddr(unsigned int addr)
{
    AddressControlReg.Addr.Value(addr);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::WriteData() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FpaLoader::WriteData(unsigned int data)
{
    DataReg.Value(data);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::ReadData() -- primitive Flash Operation
//---------------------------------------------------------------------------

unsigned int  XU_FpaLoader::ReadData()
{
    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::ReadReady() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  XU_FpaLoader::ReadReady()
{
    bool ready = AddressControlReg.Ready.Value();
    return ready;
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::ReadDataValid() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  XU_FpaLoader::ReadDataValid()
{
    bool readvalid = AddressControlReg.ReadDataValid.Value();
    return readvalid;
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::TraceDebug() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FpaLoader::TraceDebug(const std::string & text)
{
    Debug.Log(text);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::MakeChipObject() -- create proper chip object
//---------------------------------------------------------------------------

IXUFpaChip *   XU_FpaLoader::MakeChipObject(RomType /*which_rom*/)
{
   return new XUFpaChip(this);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::SelectRomMap() -- create proper rom map
//---------------------------------------------------------------------------

XU_RomControl::RomMapInfo  XU_FpaLoader::SelectRomMap(RomType which_rom)
{
    switch (which_rom)
        {
        default:
        case romMT25Q_128:  return RomMT25Q_128_MapInfo;
        case romMT25Q_512:  return RomMT25Q_512_MapInfo;
        }
}

//===========================================================================
//  CLASS XU_FpaLoader::XUFpaChip  -- CPLD Hardware Interface
//===========================================================================
//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::Initialize() -- init chip object
//---------------------------------------------------------------------------

bool  XU_FpaLoader::XUFpaChip::Initialize()
{
    if (!Owner->DeviceEnabled())
        return false;

    Owner->Reset();

    FDeviceId = Owner->Rdid();

    Owner->SetBlockProtection();
    //Owner->ReleaseBlockProtection();
    FHWProtected = true;
    FProtected = false;

    return true;   // or check if RDID is right?
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::Read32() -- Read 32 bit word
//---------------------------------------------------------------------------

unsigned int  XU_FpaLoader::XUFpaChip::Read32(unsigned int addr)
{
    if (!Owner->DeviceEnabled())
        return false;

    return Owner->Read32(addr);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::Write32() -- Write 32 bit word
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::Write32(unsigned int addr, unsigned int value)
{
    if (!Owner->DeviceEnabled())
        return;

     Owner->Write32(addr, value);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::SetProtection() -- Set block protection
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::SetProtection()
{
    Owner->SetBlockProtection();
    FHWProtected = true;   // we might as well trust it.
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::ReleaseProtection() -- Remove block protection
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::ReleaseProtection()
{
    Owner->ReleaseBlockProtection();
    if (Owner->IsBlockProtectionReleased())
        FHWProtected = false;
    else
        FHWProtected = true;
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::EraseSector() -- Erase a sector
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::EraseSector(unsigned int sector)
{
    if (!Owner->DeviceEnabled())
        return;

    Owner->EraseSector(sector);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::SetProgramPageMode() -- Prep for Page mode write
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::SetProgramPageMode(bool state)
{
    Owner->SetProgramPageMode(state);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::SetPageAddress() -- load address
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::SetPageAddress(unsigned int addr)
{
    Owner->SetPageAddress(addr);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::Enqueue32() -- load data
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::Enqueue32(unsigned int value)
{
    Owner->Enqueue32(value);
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::WaitForPageComplete() -- check for completion
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::WaitForPageComplete()
{
    Owner->WaitForPageComplete();
}

//---------------------------------------------------------------------------
//  XU_FpaLoader::XUFpaChip::PageProgram() -- start page programming process
//---------------------------------------------------------------------------

void  XU_FpaLoader::XUFpaChip::PageProgram()
{
    Owner->PageProgram();
}

//===========================================================================
//  CLASS XU_FlashRom -- XU Flash Rom "SubDevice"  - IDROM and other uses
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XU_FlashRom
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XU_FlashRom
//---------------------------------------------------------------------------

XU_FlashRom::XU_FlashRom(IRequires_Nothing * mdc, RomType which_rom,
                       IAddressingSpace & space, MapRegisters & regs)
    :  XU_RomControl(SelectRomMap(which_rom)),
       MDC(mdc),
       AddressControlReg(space, regs.AddressControl_Addr),
       DataReg(space, regs.Data_Addr),
       OpcodeReg(space, regs.Opcode_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_FlashRom
//---------------------------------------------------------------------------

XU_FlashRom::~XU_FlashRom()
{
}

//---------------------------------------------------------------------------
//  XU_FlashRom::OpenHardware()
//---------------------------------------------------------------------------

void  XU_FlashRom::OpenHardware()
{
    if (!DeviceEnabled())
        return;
    Reset();
    ReleaseBlockProtection();
}

//---------------------------------------------------------------------------
//  XU_FlashRom::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_FlashRom::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XU_FlashRom::WritePageModeBit() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FlashRom::WritePageModeBit(bool bit)
{
    AddressControlReg.PageMode.Value(bit);
}


//---------------------------------------------------------------------------
//  XU_FlashRom::WriteOpcode() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FlashRom::WriteOpcode(unsigned int opcode, unsigned int opcode_decode)
{
    MemoryBitManipulator OpReg(0);
    OpReg.Field(0, 8, opcode);
    OpReg.Field(8, 6, opcode_decode);

    OpcodeReg.Value(OpReg.Value());
}

//---------------------------------------------------------------------------
//  XU_FlashRom::WriteAddr() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FlashRom::WriteAddr(unsigned int addr)
{
    AddressControlReg.Addr.Value(addr);
}

//---------------------------------------------------------------------------
//  XU_FlashRom::WriteData() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FlashRom::WriteData(unsigned int data)
{
    DataReg.Value(data);
}

//---------------------------------------------------------------------------
//  XU_FlashRom::ReadData() -- primitive Flash Operation
//---------------------------------------------------------------------------

unsigned int  XU_FlashRom::ReadData()
{
    unsigned int data = DataReg.Value();
    return data;
}

//---------------------------------------------------------------------------
//  XU_FlashRom::ReadReady() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  XU_FlashRom::ReadReady()
{
    bool ready = AddressControlReg.Ready.Value();
    return ready;
}

//---------------------------------------------------------------------------
//  XU_FlashRom::ReadDataValid() -- primitive Flash Operation
//---------------------------------------------------------------------------

bool  XU_FlashRom::ReadDataValid()
{
    bool readvalid = AddressControlReg.ReadDataValid.Value();
    return readvalid;
}

//---------------------------------------------------------------------------
//  XU_FlashRom::TraceDebug() -- primitive Flash Operation
//---------------------------------------------------------------------------

void  XU_FlashRom::TraceDebug(const std::string & text)
{
    Debug.Log(text);
}

//---------------------------------------------------------------------------
//  XU_FlashRom::SelectRomMap() -- create proper rom map
//---------------------------------------------------------------------------

XU_RomControl::RomMapInfo  XU_FlashRom::SelectRomMap(RomType which_rom)
{
    switch (which_rom)
        {
        default:
        case romMT25Q_128:  return RomMT25Q_128_MapInfo;
        case romMT25Q_512:  return RomMT25Q_512_MapInfo;
        }
}

//===========================================================================
//  CLASS XUDigitalIo -- XUDigital Io "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class X6DigitalIo
//---------------------------------------------------------------------------

XUDigitalIo::XUDigitalIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       DataLo(space, regs.DioDataLo_Addr),
       OutEnLo(space, regs.DioOutputEnableLo_Addr),
       DataLoManip(DataLo)
{

}

//---------------------------------------------------------------------------
//  destructor for class XUDigitalIo
//---------------------------------------------------------------------------

XUDigitalIo::~XUDigitalIo()
{
}

//---------------------------------------------------------------------------
//  XUDigitalIo::OpenHardware()
//---------------------------------------------------------------------------

void  XUDigitalIo::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XUDigitalIo::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUDigitalIo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XUDigitalIo::DioPortData() -- access to Lo Dio Data
//---------------------------------------------------------------------------

IBitManipulator &  XUDigitalIo::DioPortData()
{
    return DataLoManip;
}

//---------------------------------------------------------------------------
//  XUDigitalIo::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

void  XUDigitalIo::DioPortConfig(unsigned int mask)
{
    OutEnLo.Value(mask);
}

//---------------------------------------------------------------------------
//  XUDigitalIo::DioPortConfig() -- read/change Low OE config mask
//---------------------------------------------------------------------------

unsigned int  XUDigitalIo::DioPortConfig() const
{
    return OutEnLo.Value();
}

//---------------------------------------------------------------------------
//  XUDigitalIo::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

void  XUDigitalIo::DioPortOutputEnable(int output_bit, bool output_enabled)
{
    if (output_bit<32)
        OutEnLo.Bit(output_bit, output_enabled);
}

//---------------------------------------------------------------------------
//  XUDigitalIo::DioPortOutputEnable() -- read/change single bit OE state
//---------------------------------------------------------------------------

bool  XUDigitalIo::DioPortOutputEnable(int output_bit) const
{
    if (output_bit<32)
        return OutEnLo.Bit(output_bit);
    else
        return false;
}

//===========================================================================
//  CLASS XUSystemMonitor -- XU System Monitor "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XUSystemMonitor
//---------------------------------------------------------------------------

XUSystemMonitor::XUSystemMonitor(IRequires_Sysmon_AD5592R_Init * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       Control(space, regs.SysMonControl_Addr),
       Data(space, regs.SysMonData_Addr),
       MultiIo_0(*this, 0),
       MultiIo_1(*this, 1),
       MultiIo_2(*this, 2)
{

}

//---------------------------------------------------------------------------
//  destructor for class XUSystemMonitor
//---------------------------------------------------------------------------

XUSystemMonitor::~XUSystemMonitor()
{
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::MultiIo()
//---------------------------------------------------------------------------

XUSystemMonitor::AD5592R &   XUSystemMonitor::MultiIo(unsigned int device)
{
    if (device==2)
        return MultiIo_2;
    else if (device==1)
        return MultiIo_1;
    else
        return MultiIo_0;

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::OpenHardware()
//---------------------------------------------------------------------------

void  XUSystemMonitor::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "XUSystemMonitor::OpenHardware()");
    if (!DeviceEnabled())
        return;

    Debug.Log("At SetSysmonDeviceConfiguration() call...");
    MDC->SetSysmonDeviceConfiguration();  // Board Specific Setup

    Debug.Log("At Config MultiIo 0 call...");
    MultiIo_0.Init();
    Debug.Log("At Config MultiIo 1 call...");
    MultiIo_1.Init();
    Debug.Log("At Config MultiIo 2 call...");
    MultiIo_2.Init();

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUSystemMonitor::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SetupSpiDevice() --  SPI Device Enable Setup
//---------------------------------------------------------------------------

void  XUSystemMonitor::SetupSpiDevice(int device)
{
    // enable SPI devices, one at a time
    unsigned int data = 1<<device;
    Control.DeviceSelect.Value(data);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SetupSpiMaskDevice() --  SPI Device Enable Setup - broadside
//---------------------------------------------------------------------------

void  XUSystemMonitor::SetupSpiMaskDevice(unsigned int mask)
{
    Control.DeviceSelect.Value(mask);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiRead() --  SPI Read
//---------------------------------------------------------------------------

unsigned short XUSystemMonitor::SpiRead(int device, unsigned short command)
{
    // Set up a SPI device
    SetupSpiDevice(device);

    SpiReady(device);

    // Issue the command
    Data.Command.Value(command);

    SpiNotReady(device);
    SpiReady(device);

    unsigned short result = Data.ReadData.Value();
    std::stringstream msg;
    msg << std::hex << " Sysmon Spi Read (0x" << Data.Value() << ") = [0x" << result << "]";
    Debug.Log(msg.str());

    return result;
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiWrite()  --  SPI Write
//---------------------------------------------------------------------------

void  XUSystemMonitor::SpiWrite(int device, unsigned short command)
{
    // Set up a SPI device
    SetupSpiDevice(device);

    SpiReady(device);

    // Issue the command
    Data.Command.Value(command);

    SpiNotReady(device);
    SpiReady(device);

    std::stringstream msg;
    msg << std::hex << " Sysmon Spi Write (0x" << command << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiWriteMask()  --  SPI Broadside Write
//---------------------------------------------------------------------------

void XUSystemMonitor::SpiWriteMask(unsigned int device_mask, unsigned short command)
{
    // Set up a SPI device
    SetupSpiMaskDevice(device_mask);

    SpiReadyMask(device_mask);

    // Issue the command
    Data.Command.Value(command);

    SpiNotReadyMask(device_mask);
    SpiReadyMask(device_mask);

    std::stringstream msg;
    msg << std::hex << " SysMon SpiMask(0x" << device_mask <<
             ", 0x" << (short)command << ") = [0x" << DumpSpiRegister() << "]";
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiWriteAll()  --  SPI Broadside Write
//---------------------------------------------------------------------------

void XUSystemMonitor::SpiWriteAll(unsigned short command)
{
    SpiWriteMask(0x07, command);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiReady()
//---------------------------------------------------------------------------

void  XUSystemMonitor::SpiReady(int device)
{
    unsigned int mask = 1<<device;
    SpiReadyMask(mask);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiReadyMask()
//---------------------------------------------------------------------------

void  XUSystemMonitor::SpiReadyMask(unsigned int mask)
{
    // Wait up to 5 uS for pending SPI to complete
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (CheckSpiReadyMask(mask))
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " Sysmon Spi Ready Failure, (>5us) on Dev Mask " << mask;
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiNotReady()
//---------------------------------------------------------------------------

void  XUSystemMonitor::SpiNotReady(int device)
{
    unsigned int mask = 1<<device;
    SpiNotReadyMask(mask);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::SpiNotReadyMask() - wait for SPI to start
//---------------------------------------------------------------------------

void XUSystemMonitor::SpiNotReadyMask(unsigned int mask)
{
    // Wait up to 5 uS for pending SPI to start
    for (unsigned int i = 0; i < 5; ++i)
        {
        if (CheckSpiNotReadyMask(mask))
            return;
        else
            Innovative::uSleep(1);
        }
    // Gave up
    //std::stringstream msg;
    //msg << std::hex << " Sysmon Spi NOT Ready Failure (>5us)";
    //Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::CheckSpiReadyMask() --  Determine if Ready Mask ON
//---------------------------------------------------------------------------
//  We have separate ready bits for each device.  Make sure all masked bits
//     are ON.

bool  XUSystemMonitor::CheckSpiReadyMask(unsigned int mask)
{
    unsigned int ready(Control.DeviceReady.Value());
    ready &= mask;   // unmasked bits now 0
    return (ready == mask);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::CheckSpiNotReadyMask() --  Determine if Ready Mask OFF
//---------------------------------------------------------------------------
//  We have separate ready bits for each device.  Make sure all masked bits
//     are OFF.

bool  XUSystemMonitor::CheckSpiNotReadyMask(unsigned int mask)
{
    unsigned int ready(Control.DeviceReady.Value());
    ready &= mask;   // unmasked bits now 0
    return (ready == 0);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::CheckAdcReadyMask() --  Determine if ADC ready Mask ON
//---------------------------------------------------------------------------
//  We have separate ready bits for each device.  Make sure all masked bits
//     are ON.

bool  XUSystemMonitor::CheckAdcReadyMask(unsigned int mask)
{
    unsigned int ready(Control.DeviceReady.Value());
    ready &= mask;   // unmasked bits now 0
    return (ready == mask);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AdcReady() --  Determine if ADC ready Mask ON
//---------------------------------------------------------------------------

void  XUSystemMonitor::AdcReady(int device)
{
    unsigned int mask = 1<<device;
    AdcReadyMask(mask);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AdcReadyMask()
//---------------------------------------------------------------------------

void  XUSystemMonitor::AdcReadyMask(unsigned int mask)
{
    // Wait up to 50 uS for pending SPI to complete
    for (unsigned int i = 0; i < 10; ++i)
        {
        if (CheckSpiReadyMask(mask))
            return;
        else
            Innovative::uSleep(5);
        }
    // Gave up
    std::stringstream msg;
    msg << std::hex << " Sysmon ADC Ready Failure, (>50us) on Dev Mask " << mask;
    Debug.Log(msg.str());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::DumpSpiRegister() --  Read SPI register for debugging
//---------------------------------------------------------------------------

unsigned int  XUSystemMonitor::DumpSpiRegister()
{
    return Data.Value();
}

//===========================================================================
//  CLASS  XUSystemMonitor::AD5592R -- System Monitor Chip
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XUSystemMonitor::AD5592R
//---------------------------------------------------------------------------

const int AD5592R_IoWidth = 8;

const unsigned int AD5592R_FullScaleCounts = 1<<12;
const double       AD5592R_FullScaleVolts  = 2.5;

//---------------------------------------------------------------------------
//  constructor for class XUSystemMonitor::AD5592R
//---------------------------------------------------------------------------

XUSystemMonitor::AD5592R::AD5592R(XUSystemMonitor & sub_dev, unsigned int dev)
        :  Owner(sub_dev), Device(dev), RegisterImage(16)
{
    for (size_t i=0; i<RegisterImage.size(); i++)
        {
        RegisterImage[i] = 0;
        }
    RegisterImage[6] = 0xFF;   // default all pull downs on

    // Apply other setup here before board specific customization
    SetRefEnabled(true);   // use internal reference

    // power up delay - 250 uSec, no write registers.  ???

}

//---------------------------------------------------------------------------
// XUSystemMonitor::AD5592R::Init() - Init hardware state
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::Init()
{
    SoftwareReset();   // reassert default state

    // ...after board specific customization (overrides user settings)

    EnableBusy();  // set BUSY- on output 7

    //  apply all configuration registers to hardware
    Port( 3, RegisterImage[3]);  // General Purpose Cfg
    Port( 4, RegisterImage[4]);  // ADC Pin Config
    Port( 5, RegisterImage[5]);  // DAC Pin Config
    Port( 6, RegisterImage[6]);  // Pull Down Config
    Port( 8, RegisterImage[8]);  // Gpio Out Pin Config
    Port(10, RegisterImage[10]); // Gpio Read Pin Config
    Port(11, RegisterImage[11]); // Gpio Power Down Config
    Port(12, RegisterImage[12]); // Gpio Open Drain Pin Config
    Port(13, RegisterImage[13]); // Three State Pin Config

}

//---------------------------------------------------------------------------
// XUSystemMonitor::AD5592R::ToVoltage() - convert counts to voltage
//---------------------------------------------------------------------------
//   for use with ADC data

double  XUSystemMonitor::AD5592R::ToVoltage(unsigned int counts)
{
    double trimmed_counts = std::min( AD5592R_FullScaleCounts-1, counts);
    double level = static_cast<double>(trimmed_counts);

    Trace Debug;
    Debug.Log("counts", counts);
    Debug.Log("trimmed_counts", trimmed_counts);

    double out_V =  AD5592R_FullScaleVolts * level / AD5592R_FullScaleCounts;
    Debug.Log("out_V", out_V);

    return out_V;
}

//---------------------------------------------------------------------------
// XUSystemMonitor::AD5592R::ToCounts() - convert voltage to counts
//---------------------------------------------------------------------------
//   for use by DACs

unsigned int  XUSystemMonitor::AD5592R::ToCounts(double voltage)
{
    double trimmed_volts = std::min( AD5592R_FullScaleVolts-1.0/AD5592R_FullScaleCounts, std::max(voltage, 0.0));
    double level = trimmed_volts / AD5592R_FullScaleVolts * AD5592R_FullScaleCounts;

    Trace Debug;
    Debug.Log("voltage", voltage);
    Debug.Log("trimmed voltage", trimmed_volts);
    Debug.Log("level", level);
    return static_cast<unsigned int>(level);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::SoftwareReset() -
//---------------------------------------------------------------------------

void  XUSystemMonitor::AD5592R::SoftwareReset()
{
    PtrBitManipulator Reg15(&RegisterImage[15]);
    Reg15.Field(0, 11, 0x5AC);

    // update register to hardware
    Port(15, RegisterImage[15]);

    // wait 250 uS before doing anything else
    uSleep(250);
}


////---------------------------------------------------------------------------
////  XUSystemMonitor::AD5592R::PowerDownAll() -
////---------------------------------------------------------------------------

//void XUSystemMonitor::AD5592R::PowerDownAll()
//{
//    // power up delay - 250 uSec, no write registers.  ???
//}


////---------------------------------------------------------------------------
////  XUSystemMonitor::AD5592R::PowerDownDac() -
////---------------------------------------------------------------------------

//void XUSystemMonitor::AD5592R::PowerDownDac(int /*dac*/, bool /*state*/)
//{
//    // power up delay - 250 uSec, no write registers.  ???
//}

void XUSystemMonitor::AD5592R::SetOutputConfig(int pin, unsigned int cfg_setting)
{
    switch (cfg_setting)
        {
        default:
        case ocDisabled:    // also ocTriStated
            {
            PinIsThreeState(pin, true);
            PinAsAdc(pin, false);
            PinAsDac(pin, false);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocAdc:
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, true );
            PinAsDac(pin, false);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocDac:  // also ocDac_PoweredUp
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, false);
            PinAsDac(pin, true);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocDac_PoweredDown:
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, false);
            PinAsDac(pin, true, true);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocDac_AdcCheck:  // Dac AND Adc on for analog DAC check
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, true);
            PinAsDac(pin, true);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocDac_AdcCheck_PwrDown:  // Dac AND Adc on for analog DAC check, plus DAC off
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, true);
            PinAsDac(pin, true, true);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocGpioIn:
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, false);
            PinAsDac(pin, false);
            PinAsGpioIn(pin, true);
            PinAsGpioOut(pin, false);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocGpioOut:
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, false);
            PinAsDac(pin, false);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, true);
            PinIsPulledDown(pin, false);
            }
            break;

        case ocGpioOut_OpenDrain:
            {
            PinIsThreeState(pin, false);
            PinAsAdc(pin, false);
            PinAsDac(pin, false);
            PinAsGpioIn(pin, false);
            PinAsGpioOut(pin, true, true);
            PinIsPulledDown(pin, false);
            }
            break;
        }

}

// customization config setup - affect state before open
//   not auto-applied

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::PinAsAdc() - set pin to be an ADC
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::PinAsAdc(int pin, bool state)
{
    PtrBitManipulator Reg4(&RegisterImage[4]);
    if (pin < AD5592R_IoWidth)
        Reg4.Bit(pin, state);

    // disable bit for other uses?

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::PinAsDac() - set pin to be a DAC
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::PinAsDac(int pin, bool state, bool powered_down)
{
    PtrBitManipulator Reg5(&RegisterImage[5]);
    PtrBitManipulator Reg11(&RegisterImage[11]);

    if (pin < AD5592R_IoWidth)
        {
        Reg5.Bit(pin, state);          // set as DAC

        bool masked_pd = (state) ? powered_down : false  ;   // must be DAC to set PD on.
        Reg11.Bit(pin, masked_pd);  // set as Powered Down
        }

    // disable bit for other uses?

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::PinAsGpioIn() - set pin to be a GPIO input
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::PinAsGpioIn(int pin, bool state)
{
    PtrBitManipulator Reg10(&RegisterImage[10]);
    if (pin < AD5592R_IoWidth)
        Reg10.Bit(pin, state);

    // disable bit for other uses?

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::PinAsGpioOut() - set pin to be GPIO output
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::PinAsGpioOut(int pin, bool state, bool open_drain)
{
    PtrBitManipulator Reg8(&RegisterImage[8]);
    PtrBitManipulator Reg12(&RegisterImage[12]);

    if (pin < AD5592R_IoWidth)
        Reg8.Bit(pin, state);

    bool masked_od = (state) ? open_drain : false  ;   // must be DAC to set PD on.
    Reg12.Bit(pin, masked_od);  // set as Powered Down

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::PinIsPulledDown() - set pin to be pulled down
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::PinIsPulledDown(int pin, bool state)
{
    PtrBitManipulator Reg6(&RegisterImage[6]);
    if (pin < AD5592R_IoWidth)
        Reg6.Bit(pin, state);

    // disable bit for other uses?

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::PinIsThreeState() - set pin to be tristated
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::PinIsThreeState(int pin, bool state)
{
    PtrBitManipulator Reg13(&RegisterImage[13]);
    if (pin < AD5592R_IoWidth)
        Reg13.Bit(pin, state);

    // disable bit for other uses?

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::EnableBusy() -  Standard BUSY Config
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::EnableBusy()
{
    // make sure GPIO 7 is output, BUSY is on

    PtrBitManipulator Reg(&RegisterImage[8]);
    Reg.Bit(8, true);

    SetOutputConfig(7, ocGpioOut);
}


//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::SetRefEnabled() -  Enable/Disable 2.5 Reference
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::SetRefEnabled(bool state)
{
    PtrBitManipulator Reg11(&RegisterImage[11]);
    Reg11.Bit(9, state);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::Port() -  Read hardware register from device
//---------------------------------------------------------------------------

unsigned int  XUSystemMonitor::AD5592R::Port(unsigned int address)
{
    //  Construct the output write word to command register read
    MemoryBitManipulator OutData(RegisterImage[7]);   // use the current LDAC status
    //  Bit 15 must be 0
    OutData.Bit(15, false);
    OutData.Field(11, 4, 7);  // Readback register address
    OutData.Bit(6, true);
    OutData.Field(2, 4, address);
    // Bits 0, 1 are LDAC state

    // Write Command Word to the SPI
    Owner.SpiWrite(DeviceId(), OutData.Value());

    // delay???

    unsigned int NOP_cmd(0);
    unsigned int data = Owner.SpiRead(DeviceId(), NOP_cmd);
    return data;
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::Port() -  Write hardware register to device
//---------------------------------------------------------------------------

void  XUSystemMonitor::AD5592R::Port(unsigned int address, unsigned int data)
{
    //  Construct the output write word to command register read
    MemoryBitManipulator OutData(0);
    //  Bit 15 must be 0
    OutData.Bit(15, false);
    OutData.Field(11, 4, address);  // Readback register address
    OutData.Field(0, 11, data);

    // Write Command Word to the SPI
    Owner.SpiWrite(DeviceId(), OutData.Value());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::LDacModePort() -  Change LDAC mode
//---------------------------------------------------------------------------

void XUSystemMonitor::AD5592R::LDacModePort(XUSystemMonitor::AD5592R::LDacMode mode)
{
    //  Construct the output write word to send to the SPI
    MemoryBitManipulator OutData(0);
    //  Bit 15 must be 0
    OutData.Bit(15, false);
    OutData.Field(11, 4, 0x7);
    OutData.Field(0, 2, static_cast<unsigned int>(mode));

    // Write DAC word to the SPI
    Owner.SpiWrite(DeviceId(), OutData.Value());

}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::DacPort() -  Write DAC data to device
//---------------------------------------------------------------------------

void  XUSystemMonitor::AD5592R::DacPort(unsigned int dac, unsigned int data)
{
    //  Construct the output write word to send to the SPI
    MemoryBitManipulator OutData(0);
    //  Bit 15 must be 1
    OutData.Bit(15, true);
    OutData.Field(12, 3, dac);
    OutData.Field(0, 12, data);

    // Write DAC word to the SPI
    Owner.SpiWrite(DeviceId(), OutData.Value());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::DacPort() -  Readback DAC setting from device
//---------------------------------------------------------------------------

unsigned int  XUSystemMonitor::AD5592R::DacPort(unsigned int dac)
{
    //  Construct the output write word to send to the SPI
    MemoryBitManipulator OutData(0);
    //  Bit 15 must be 0
    OutData.Bit(15, false);
    OutData.Field(11, 4, 0x1);  // DAC Readback register address
    OutData.Field(3, 2, 0x3);   // DAC Readback Enabled
    OutData.Field(0, 3, dac);

    // Write DAC word to the SPI
    Owner.SpiWrite(DeviceId(), OutData.Value());

    // delay??

    unsigned int NOP_cmd(0);
    MemoryBitManipulator DacData(Owner.SpiRead(DeviceId(), NOP_cmd));
    //  D15 should be 1
    //  D12-14 are DAC number
    //  rest is data
    return DacData.Field(0, 12);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::AdcPort() -  Readback ADC Data
//---------------------------------------------------------------------------

unsigned int  XUSystemMonitor::AD5592R::AdcPort(unsigned int adc, bool read_temp)
{
    if (adc >= 8)
        return 0;    // bad input

    //  Construct the output write word to send to the SPI
    MemoryBitManipulator AdcSeqData(0);
    //  Bit 15 must be 0
    AdcSeqData.Bit(15, false);
    AdcSeqData.Field(11, 4, 0x2);  // ADC Sequence register address
    AdcSeqData.Bit(9, false);      // no REPs
    AdcSeqData.Bit(8, read_temp);  // TEMP
    AdcSeqData.Field(0, 8, (read_temp) ? 0 : (1<<adc));  // select an ADC to read

    // Write ADC Sequence word to the SPI
    Owner.SpiWrite(DeviceId(), AdcSeqData.Value());

    // 500 ns delay before triggering ADC conversion
    uSleep(1);

    unsigned int NOP_cmd(0);
    Owner.SpiWrite(DeviceId(), NOP_cmd);  // data still not valid

    // wait for BUSY to be asserted
    Owner.AdcReady(DeviceId());
    // ADC conversion takes 2us ?
    // TEMP conversion takes 5us with ADC buffer enabled, 20us with it disabled

    MemoryBitManipulator AdcData(Owner.SpiRead(DeviceId(), NOP_cmd));
    //  D15 should be 0 for ADCs, 1 for TEMP
    //  D12-14 are ADC number, or 0 for TEMP
    //  rest is data
    return AdcData.Field(0, 12);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::TempPort() -  Read Chip Temperature
//---------------------------------------------------------------------------

unsigned int  XUSystemMonitor::AD5592R::TempPort()
{
    return AdcPort(0, true);
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::GpioPortAll() -  Write all DIO bits together
//---------------------------------------------------------------------------

void  XUSystemMonitor::AD5592R::GpioPortAll(unsigned int bit_data)
{
    //  construct data for Port Write to register 9
    MemoryBitManipulator GpioData(0);
    GpioData.Field(0, 7, bit_data);

    Port(9, GpioData.Value());
}

//---------------------------------------------------------------------------
//  XUSystemMonitor::AD5592R::GpioPortAll() -  Write all DIO bits together
//---------------------------------------------------------------------------

unsigned int  XUSystemMonitor::AD5592R::GpioPortAll()
{
    MemoryBitManipulator GpioData(RegisterImage[10]);
    GpioData.Bit(10, true);      // enable readback on next SPI

    Port(10, GpioData.Value());  // trigger data on next SPI

    // delay??

    unsigned int NOP_cmd(0);
    MemoryBitManipulator Gpio_In(Owner.SpiRead(DeviceId(), NOP_cmd));

    return Gpio_In.Field(0, 8);

}

//===========================================================================
//  CLASS SysMonInterface -- Base class for SYS MON customization
//===========================================================================
//---------------------------------------------------------------------------
//  SysMonInterface::Write_a_Dac() - write DAC, with conversions
//---------------------------------------------------------------------------

void SysMonInterface::Write_a_Dac(unsigned int pin_code, double voltage)
{
    // adjust voltage to DAC range
    double shifted_voltage = voltage / ConversionMultiplier[pin_code];

    dp Pin = Location[pin_code];
    unsigned int counts = Device->MultiIo(Pin.dev).ToCounts(shifted_voltage);
    //  write counts to DAC...
    Device->MultiIo(Pin.dev).DacPort(Pin.pin, counts);

}

//---------------------------------------------------------------------------
//  SysMonInterface::Read_an_Adc() - read ADC, with conversions
//---------------------------------------------------------------------------

double  SysMonInterface::Read_an_Adc(unsigned int pin_code)
{
    dp Pin = Location[pin_code];
    //  read counts from ADC...
    unsigned int counts = Device->MultiIo(Pin.dev).AdcPort(Pin.pin);

    double voltage = Device->MultiIo(Pin.dev).ToVoltage(counts);

    // adjust voltage to true voltage range
    double shifted_voltage = voltage * ConversionMultiplier[pin_code];
    return shifted_voltage;

}

//---------------------------------------------------------------------------
//  SysMonInterface::CommonFactorDualDacUpdate() -
//---------------------------------------------------------------------------
//  Output is (fixed common DAC - adjust_dac), fired at same time
//    Ranges from +/- .625 volts on output

void  SysMonInterface::CommonFactorDualDacUpdate(unsigned int common_pincode, unsigned int adjust_pincode,
                                                 unsigned int common_factor_idx, double adjust_volts)
{
    dp CommonPin = Location[common_pincode];  // note - all DACs must be on same device

    const double max_level = Device->MultiIo(CommonPin.dev).ToVoltage(CommonFactorLevel[common_factor_idx]);
    const double min_level = max_level-2.5;
    const double base_level = max_level;

    adjust_volts /= CommonFactorScale[common_factor_idx];

    double trimmed_volts = std::min( max_level, std::max(adjust_volts, min_level));
    double adjust_level = base_level - trimmed_volts;  // since adjust DAC is subtracted, smaller makes output rise

    // step 1:  enter LDAC load mode
    Device->MultiIo(CommonPin.dev).LDacModePort(XUSystemMonitor::AD5592R::ldacLoad);

    // step 2:  load Common DAC
    Write_a_Dac(common_pincode, base_level);

    // step 3:  load Adjust DAC
    Write_a_Dac(adjust_pincode, adjust_level);

    // step 4:  update DACs
    Device->MultiIo(CommonPin.dev).LDacModePort(XUSystemMonitor::AD5592R::ldacUpdate);

    // step 5:  restore normal state
    Device->MultiIo(CommonPin.dev).LDacModePort(XUSystemMonitor::AD5592R::ldacWrite);
}

//---------------------------------------------------------------------------
//  SysMonInterface::CommonFactorTripleDacUpdate() -
//---------------------------------------------------------------------------
//  Output is .5*(fixed common DAC - adjust_dac), fired at same time
//    Ranges from +/- .625 volts on output

void  SysMonInterface::CommonFactorTripleDacUpdate(unsigned int common_pincode,
                                                   unsigned int adjust0_pincode, unsigned int adjust1_pincode,
                                                   unsigned int common_factor_idx,
                                                   double adjust0_volts, double adjust1_volts)
{
    dp CommonPin = Location[common_pincode];  // note - all DACs must be on same device

    const double max_level = Device->MultiIo(CommonPin.dev).ToVoltage(CommonFactorLevel[common_factor_idx]);
    const double min_level = max_level-2.5;
    const double base_level = max_level;

    adjust0_volts /= CommonFactorScale[common_factor_idx];
    adjust1_volts /= CommonFactorScale[common_factor_idx];

    double trimmed0_volts = std::min( max_level, std::max(adjust0_volts, min_level));
    double adjust0_level = base_level - trimmed0_volts;  // since adjust DAC is subtracted, smaller makes output rise

    double trimmed1_volts = std::min( max_level, std::max(adjust1_volts, min_level));
    double adjust1_level = base_level - trimmed1_volts;  // since adjust DAC is subtracted, smaller makes output rise

    // step 1:  enter LDAC load mode
    Device->MultiIo(CommonPin.dev).LDacModePort(XUSystemMonitor::AD5592R::ldacLoad);

    // step 2:  load Common DAC
    Write_a_Dac(common_pincode, base_level);

    // step 3:  load Adjust DAC 0
    Write_a_Dac(adjust0_pincode, adjust0_level);

    // step 4:  load Adjust DAC 0
    Write_a_Dac(adjust1_pincode, adjust1_level);

    // step 5:  update DACs
    Device->MultiIo(CommonPin.dev).LDacModePort(XUSystemMonitor::AD5592R::ldacUpdate);

    // step 6:  restore normal state
    Device->MultiIo(CommonPin.dev).LDacModePort(XUSystemMonitor::AD5592R::ldacWrite);
}

//===========================================================================
//  CLASS XU_LogicSystemMonitor -- XU Logic Based Sys Monitor "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XU_LogicSystemMonitor -
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//  constructor for class XU_LogicSystemMonitor
//---------------------------------------------------------------------------

XU_LogicSystemMonitor::XU_LogicSystemMonitor(IRequires_Nothing* mdc, IAddressingSpace& space, XU_LogicSystemMonitor::MapRegisters& regs)
    :  MDC(mdc),
       TemperatureReg(space, regs.CurTemperature_Addr),
       VccIntReg(space, regs.CurVccInt_Addr),
       VccAuxReg(space, regs.CurVccAux_Addr),
       VbramReg(space,  regs.CurVbram_Addr),
       MaxTemperatureReg(space, regs.MaxTemperature_Addr),
       MaxVccIntReg(space, regs.MaxVccInt_Addr),
       MaxVccAuxReg(space, regs.MaxVccAux_Addr),
       MaxVbramReg(space, regs.MaxVbram_Addr),
       MinTemperatureReg(space, regs.MinTemperature_Addr),
       MinVccIntReg(space, regs.MinVccInt_Addr),
       MinVccAuxReg(space, regs.MinVccAux_Addr),
       MinVbramReg(space, regs.MinVbram_Addr),
       WarningThreshold_U_Reg(space, regs.WarnThresh_U_Addr),
       FatalThreshold_U_Reg(space, regs.FatalThresh_U_Addr),
       WarningThreshold_L_Reg(space, regs.WarnThresh_L_Addr),
       FatalThreshold_L_Reg(space, regs.FatalThresh_L_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_LogicSystemMonitor
//---------------------------------------------------------------------------

XU_LogicSystemMonitor::~XU_LogicSystemMonitor()
{
}

//---------------------------------------------------------------------------
//  XU_LogicSystemMonitor::OpenHardware()
//---------------------------------------------------------------------------

void XU_LogicSystemMonitor::OpenHardware()
{
}

//---------------------------------------------------------------------------
//  XU_LogicSystemMonitor::ConfigureHardware()
//---------------------------------------------------------------------------

void XU_LogicSystemMonitor::ConfigureHardware()
{
}

//---------------------------------------------------------------------------
//  XU_LogicSystemMonitor::ToVoltage()  - convert raw counts to volts
//---------------------------------------------------------------------------

double XU_LogicSystemMonitor::ToVoltage(unsigned int adc_code)
{
    double code = static_cast<double>(adc_code);
    double voltage = (code/1024.0)*3.0;
    return voltage;
}

//---------------------------------------------------------------------------
//  XU_LogicSystemMonitor::ToTemperature()  - convert raw counts to T
//---------------------------------------------------------------------------

double XU_LogicSystemMonitor::ToTemperature(unsigned int adc_code)
{
    double code = static_cast<double>(adc_code);
    double T = code/(1<<10)*501.3743 - 273.6777;
    return T;
}

//---------------------------------------------------------------------------
//  XU_LogicSystemMonitor::FromTemperature()  - convert T to raw counts
//---------------------------------------------------------------------------

unsigned int XU_LogicSystemMonitor::FromTemperature(double T)
{
    double code = (T+273.6777)*(1<<10)/501.3743;
    return static_cast<unsigned int>(code);   // or use NearestInt template?
}

//===========================================================================
//  CLASS XUAurora -- XU Aurora Device section "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XUAurora
//---------------------------------------------------------------------------

XUAurora::XUAurora(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       SelectReg(space, regs.Select_Addr),
       TestControlReg(space, regs.TestControl_Addr),
       ControlStatusReg(space, regs.ControlStatus_Addr),
       WritePortReg(space, regs.WritePort_Addr),
       ReadPortRegReg(space, regs.ReadPort_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAurora
//---------------------------------------------------------------------------

XUAurora::~XUAurora()
{
}

//---------------------------------------------------------------------------
//  XUAurora::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAurora::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XUAurora::OpenHardware()
//---------------------------------------------------------------------------

void  XUAurora::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XUAurora::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAurora::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    // nothing
}

//---------------------------------------------------------------------------
//  XUAurora::SetDevice() - set logic MUX to proper device
//---------------------------------------------------------------------------

void XUAurora::SetDevice(unsigned int device)
{
    unsigned int dev_code = 1<<device;
    SelectReg.DeviceSelect.Value(dev_code);
}

//---------------------------------------------------------------------------
//  XUAurora::LogicPresent()
//---------------------------------------------------------------------------

bool  XUAurora::LogicPresent(unsigned int device)
{
    if (!DeviceEnabled())
        return false;

    return SelectReg.DeviceActive.Bit(device);
}

//---------------------------------------------------------------------------
//  XUAurora::InitAuroraLink()
//---------------------------------------------------------------------------

void  XUAurora::InitAuroraLink(unsigned int device, LoopbackTypes lb)
{
    Debug.Log("Init Aurora Link for device ", device);

    //  Select the Aurora Device
    SetDevice(device);
    //  Reset the MGT
    ControlStatusReg.GtxResetN.Value(false);
    //  Power Up the MGT
    ControlStatusReg.PowerDown.Value(true);
    ControlStatusReg.PowerDown.Value(false);
    Sleep(10);
    //  Clear Errors
    ControlStatusReg.ClearError.Value(true);
    ControlStatusReg.ClearError.Value(false);
    //  Set Loopback State
    ControlStatusReg.LoopbackType.Value(static_cast<unsigned int>(lb));
    //  Out of Reset
    ControlStatusReg.GtxResetN.Value(true);
    Sleep(10);
}

//---------------------------------------------------------------------------
//  XUAurora::StartTestSingleDevice()
//---------------------------------------------------------------------------

void  XUAurora::StartTestSingleDevice(unsigned int device)
{
    Debug.Log("Starting Single device Aurora Test");

    bool DevSts = StartTestMB(device);

    if (!DevSts)
        return;

    // Enable the tx channel
    EnableChannelsTxMB(device);

    // Check test status
    DeviceStatus(device);
}

//---------------------------------------------------------------------------
//  XUAurora::StartTestDualDevices()
//---------------------------------------------------------------------------

void  XUAurora::StartTestDualDevices(unsigned int device1, unsigned int device2)
{
    Debug.Log("Starting Dual device Aurora Test");

    bool Dev1Sts = StartTestMB(device1);
    bool Dev2Sts = StartTestMB(device2);

    if (!Dev1Sts || !Dev2Sts)
        return;

    // Enable tx channels for both devices
    EnableChannelsTxMB(device1);
    EnableChannelsTxMB(device2);

    // Check test status
    DeviceStatus(device1);
    DeviceStatus(device2);
}

//---------------------------------------------------------------------------
//  XUAurora::StartTestMB()
//---------------------------------------------------------------------------

bool  XUAurora::StartTestMB(unsigned int device)
{
    // Read the device status to check the PLL lock and Channel Up bits
    AuroraDeviceStatus status = DeviceStatus(device);

    if (!status.PllLocked)
        {
        Debug.Log("Skipping the test, Aurora Pll lock failed");
        return false;
        }

    if (!status.ChannelUp)
        {
        Debug.Log("Skipping the test, Aurora link failed");
        return false;
        }

    Debug.Log("Start test for Aurora device ", device);

    // Enable test the generator/checker
    SetDevice(device);
    TestControlReg.EnableTxTest.Value(true);
    TestControlReg.EnableRxTest.Value(true);

    // Enable the rx channel
    ControlStatusReg.EnableRx.Value(true);

    // Start Run
    ControlStatusReg.Run.Value(false);

    //  Clear Errors
    ControlStatusReg.ClearError.Value(true);
    ControlStatusReg.ClearError.Value(false);

    // Start Run
    ControlStatusReg.Run.Value(true);

    return true;
}

//---------------------------------------------------------------------------
//  XUAurora::EnableChannelsTxMB()
//---------------------------------------------------------------------------

void  XUAurora::EnableChannelsTxMB(unsigned int device)
{
    Debug.Log("Aurora Enable Tx Channels");
    SetDevice(device);

    // enable tx channel
    ControlStatusReg.EnableTx.Value(true);
}

//---------------------------------------------------------------------------
//  XUAurora::EndTest()
//---------------------------------------------------------------------------

void  XUAurora::EndTest(unsigned int device)
{
    SetDevice(device);

    // Disable tx/rx channels
    ControlStatusReg.EnableRx.Value(false);
    ControlStatusReg.EnableTx.Value(false);

    //  Disable test
    TestControlReg.EnableTxTest.Value(false);
    TestControlReg.EnableRxTest.Value(false);

    //  End Run
    ControlStatusReg.Run.Value(false);

    //  Into Reset
    ControlStatusReg.GtxResetN.Value(false);

    //  Power Down
    ControlStatusReg.PowerDown.Value(true);
    Debug.Log("Ending Aurora Test");
}

//---------------------------------------------------------------------------
//  XUAurora::DeviceStatus()
//---------------------------------------------------------------------------

XUAurora::AuroraDeviceStatus XUAurora::DeviceStatus(unsigned int device)
{
    SetDevice(device);
    ControlStatusReg.Refresh();

    AuroraDeviceStatus status;

    status.HardError = ControlStatusReg.HardError.Value();
    status.SoftError = ControlStatusReg.SoftError.Value();
    status.FrameError = ControlStatusReg.FrameError.Value();
    status.ChannelUp = ControlStatusReg.ChannelUp.Value();
    status.PllLocked = ControlStatusReg.PllLocked.Value();
    status.ErrorCount = TestControlReg.TestErrorCount.Value();;

    Debug.Log("Aurora device status for device ", device);
    Debug.Log("Pll Lock State", status.PllLocked);
    Debug.Log("Channel Up State", status.ChannelUp);
    Debug.Log("Hard Error", status.HardError);
    Debug.Log("Soft Error", status.SoftError);
    Debug.Log("Frame Error", status.FrameError);
    Debug.Log("ErrorCount", status.ErrorCount);

    return status;
}

//===========================================================================
//  CLASS XUSioPll -- XU Information "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUSioPll
//---------------------------------------------------------------------------

//const int     SioPllI2C_SubAddr  = 0xE0;     // 0x70 left shifted one

//---------------------------------------------------------------------------
//  constructor for class XUSioPll
//---------------------------------------------------------------------------

XUSioPll::XUSioPll(IRequires_XUSioPll * mdc/*, IAddressingSpace & space, MapRegisters & regs*/)
    :  MDC(mdc),
       Sio_I2C_uDev(0), SioPllDev(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUSioPll
//---------------------------------------------------------------------------

XUSioPll::~XUSioPll()
{
    delete SioPllDev;  SioPllDev=0;
}

//---------------------------------------------------------------------------
//  XUSioPll::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUSioPll::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    Sio_I2C_uDev  = MDC->MakeXuSioI2CLink();
    SioPllDev = MDC->MakeXuSioPllObject(Sio_I2C_uDev->Device());
}

//---------------------------------------------------------------------------
//  XUSioPll::OpenHardware()
//---------------------------------------------------------------------------

void  XUSioPll::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    Debug.Log("Init Sio LMK61E2 Reference...");
    SioPllDev->Init();
}

//---------------------------------------------------------------------------
//  XUSioPll::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUSioPll::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

//    SioPllDev->Reapply();
}


}  // namespace Innovative



