// XUSubDevice_Misc_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef XUSubDevice_Misc_MbH
#define XUSubDevice_Misc_MbH

#include <X6Support_Mb.h>
#include <VpxSupport_Mb.h>
#include <HardwareRegister_Mb.h>
#include <FpaLoadThread_Mb.h>
#include <DioPortIntf_Mb.h>
#include <LMK61E2_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif



//===========================================================================
//  CLASS XU_RomControl -- XU Rom Programming Shared Code
//===========================================================================

class XU_RomControl : public IIdRomDevice
{
public:
    //=======================================================================
    //  CLASS XU_RomControl::RomMapInfo  -- Device Rom Layout Definitions
    //=======================================================================

    class RomMapInfo
    {
    public:
        unsigned int   Page_Count;
        unsigned int   Page_Size;
        unsigned int   Sector_Count;
        unsigned int   Sector_Size;
    };
    //
    //  Member Functions
    XU_RomControl(RomMapInfo map) :  Map(map), ExtAddrBits(0)
        {  FillOpcodeData();  }
    virtual ~XU_RomControl()
        {}

    //
    //  IIdRomDevice implementation:  Idrom Page Operations
    virtual void   ErasePage(unsigned int page);
    virtual void   ReadPage(unsigned int page, int * pkt, unsigned int size);
    virtual void   WritePage(unsigned int page, int * pkt, unsigned int size);

    virtual unsigned int  JdecId();
    virtual unsigned int  Rdid();

    void                  Reset();

    //  ROM Info Commands
    void          EraseSector(unsigned int sector);
    void          EraseAll();
    unsigned int  Read32(unsigned int addr);
    void          Write32(unsigned int addr, unsigned int value);
    unsigned int  Rdsr();

    //  Page Program Mode Commands
    virtual void  SetProgramPageMode(bool state);
    virtual void  SetPageAddress(unsigned int addr);
    virtual void  Enqueue32(unsigned int value);
    virtual void  WaitForPageComplete();
    virtual void  PageProgram();

    //---------------------------
    //
    //  Flash Primitives
    void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data);   // write operation
    unsigned int  FlashOp(unsigned int opcode, unsigned int addr);  // read op
    void          FlashOp(unsigned int opcode);  // command op
    //
    //  Internal Flash Primitives
    void  Flash_Ready();
    void  Flash_ReadValid();
    void  Flash_NotBusy();
    void  Flash_QuickNotBusy();

    void  ReleaseBlockProtection();
    void  SetBlockProtection();
    bool  IsBlockProtectionReleased();

    void  Flash_Ready_T(unsigned int max_wait_us);

    void  Flash_NotBusy_T_ms(unsigned int max_wait_ms);
    void  Flash_NotBusy_T_us(unsigned int max_wait_us);

    void  WriteExtendedAddressBits(unsigned int addr_bits);

    void  UpdateAddress(unsigned int address);

protected:
    //  Data
    RomMapInfo    Map;
    unsigned int  ExtAddrBits;
    std::map<unsigned int, unsigned int> OpcodeDecode;

    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit) = 0;
    virtual void          WriteOpcode(unsigned int opcode, unsigned int opcode_decode) = 0;
    virtual void          WriteAddr(unsigned int addr) = 0;
    virtual void          WriteData(unsigned int opcode) = 0;
    virtual unsigned int  ReadData() = 0;
    virtual bool          ReadReady() = 0;
    virtual bool          ReadDataValid() = 0;

    virtual void          TraceDebug(const std::string & text) = 0;

private:

    void  FillOpcodeData();
};

//===========================================================================
//  CLASS XU_FpaLoader -- XU FPA Loader "SubDevice"
//===========================================================================

class XU_FpaLoader : public IX6SubDevice, public XU_RomControl
{
public:

    enum RomType { romMT25Q_128, romMT25Q_512 };

    //=======================================================================
    //  CLASS XU_FpaLoader::MapRegisters  -- Device Register Offsets
    //=======================================================================

    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    //=======================================================================
    //  CLASS XU_FpaLoader::XUFpaChip  -- Chip Interface
    //=======================================================================
    //  MT25Q ROM

    class XUFpaChip : public IXUFpaChip
    {
    public:
        XUFpaChip(XU_FpaLoader * owner) : Owner(owner)
            {  FTotalSectors = Owner->Map.Sector_Count;  }  // no golden image

        virtual ~XUFpaChip()
            {  }
        virtual unsigned int  SectorSize(unsigned int /*sector*/) const  {  return Owner->Map.Sector_Size;  }
        virtual unsigned int  Sectors() const   {  return FTotalSectors;  }
        virtual unsigned int  StartSector() const {  return 0;  }
        virtual unsigned int  GoldenImageStartSector() const    // this is just the original image repeated.
                    {  return StartSector();  }
        virtual unsigned int  GoldenImageSectors() const
                    {  return Sectors();  }

        virtual bool Initialize();

        //  Page Program Mode
        virtual void  SetProgramPageMode(bool state);
        virtual void  SetPageAddress(unsigned int addr);
        virtual void  Enqueue32(unsigned int value);
        virtual void  WaitForPageComplete();
        virtual void  PageProgram();

        virtual void  SetProtection();
        virtual void  ReleaseProtection();

        virtual unsigned int  Read32(unsigned int addr);
        virtual void          Write32(unsigned int addr, unsigned int value);

        virtual void          EraseSector(unsigned int sector);

    private:
        XU_FpaLoader * Owner;
    };

    friend class XUFpaChip;

    //
    //  Member Functions
    XU_FpaLoader(IRequires_Nothing * mdc, RomType which_rom, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XU_FpaLoader();

    IFpaLoader &  VsProm()
            {  return Thread;  }

    //==============================================================================
    //  CLASS LocalProm  -- "Raw" access to PROM
    //==============================================================================

    class LocalProm : public IVpxProm
    {
    public:
        LocalProm(XU_FpaLoader * owner) : Owner(owner)
            {}
        virtual ~LocalProm()
            {}

        virtual void          EraseSector(unsigned int sector)
                                {  Owner->EraseSector(sector);  }
        virtual unsigned int  Read32(unsigned int addr)
                                {  return Owner->Read32(addr);  }
        virtual void          Write32(unsigned int addr, unsigned int value)
                                {  Owner->Write32(addr, value);  }
        virtual unsigned int  Rdsr()
                                {  return Owner->Rdsr();  }
        virtual unsigned int  Rdid()
                                {  return Owner->Rdid();  }

        //---------------------------
        //
        //  Flash Primitives
        //  ...write operation
        virtual void          FlashOp(unsigned int opcode, unsigned int addr, unsigned int data)
                                {  Owner->FlashOp(opcode, addr, data);  }
        //  ...read operation
        virtual unsigned int  FlashOp(unsigned int opcode, unsigned int addr)
                                {  return Owner->FlashOp(opcode, addr);  }
        //  ...command operation
        virtual void          FlashOp(unsigned int opcode)
                                {  Owner->FlashOp(opcode);  }

    private:
        XU_FpaLoader * Owner;
    };
    friend class LocalProm;

    LocalProm &  Raw()
            {  return RawProm;  }

public:
    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    FpaLoadThread &  FpaLT() { return Thread; }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_FpaLoader::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        PageMode;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), PageMode(*this, 26),
              Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *     MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    IXUFpaChip *        Chip;
    LocalProm           RawProm;    // debug interface
    FpaLoadThread       Thread;
    StopWatch           Rtc;

protected:
    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit);
    virtual void          WriteOpcode(unsigned int opcode, unsigned int opcode_decode);
    virtual void          WriteAddr(unsigned int addr);
    virtual void          WriteData(unsigned int opcode);
    virtual unsigned int  ReadData();
    virtual bool          ReadReady();
    virtual bool          ReadDataValid();

    virtual void          TraceDebug(const std::string & text);

    IXUFpaChip *  MakeChipObject(RomType which_rom);
    RomMapInfo    SelectRomMap(RomType which_rom);
};


//===========================================================================
//  CLASS XU_FlashRom -- XU Flash Rom "SubDevice"  - IDROM and other uses
//===========================================================================

class XU_FlashRom : public IX6SubDevice, public XU_RomControl
{
public:

    enum RomType { romMT25Q_128, romMT25Q_512 };

    //=======================================================================
    //  CLASS XU_FlashRom::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AddressControl_Addr;
        unsigned int   Data_Addr;
        unsigned int   Opcode_Addr;
    };

    XU_FlashRom(IRequires_Nothing * mdc, RomType which_rom,
               IAddressingSpace & space, MapRegisters & regs);
    virtual ~XU_FlashRom();



    //  Internal Interface
    //virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_FlashRom::AddressControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AddressControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Addr;
        RegisterBit        Hold;
        RegisterBit        WordProtect;
        RegisterBit        PageMode;
        RegisterBit        Aai_Do;
        RegisterBit        ReadDataValid;
        RegisterBit        Ready;

        AddressControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Addr(*this, 0, 24), Hold(*this, 24),
              WordProtect(*this, 25), PageMode(*this, 26),
              Aai_Do(*this, 29),
              ReadDataValid(*this, 30), Ready(*this, 31)
        {  }
    };

private:
    IRequires_Nothing *  MDC;
    AddressControlRegister  AddressControlReg;
    Register                DataReg;
    Register                OpcodeReg;

    //
    //  Register Access primitives
    virtual void          WritePageModeBit(bool bit);
    virtual void          WriteOpcode(unsigned int opcode, unsigned int opcode_decode);
    virtual void          WriteAddr(unsigned int addr);
    virtual void          WriteData(unsigned int opcode);
    virtual unsigned int  ReadData();
    virtual bool          ReadReady();
    virtual bool          ReadDataValid();

    virtual void          TraceDebug(const std::string & text);

    RomMapInfo    SelectRomMap(RomType which_rom);
};


//===========================================================================
//  CLASS XUDigitalIo -- XU Digital Io "SubDevice"
//===========================================================================

class XUDigitalIo : public IX6SubDevice, public XUDioPortIntf
{
public:
    //=======================================================================
    //  CLASS XUDigitalIo::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   DioDataLo_Addr;
        unsigned int   DioOutputEnableLo_Addr;
    };

    XUDigitalIo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XUDigitalIo();

    virtual IBitManipulator &   DioPortData();

    virtual void 			    DioPortConfig(unsigned int);
    virtual unsigned int 	    DioPortConfig() const;

    virtual void                DioPortOutputEnable(int output_bit, bool output_enabled);
    virtual bool                DioPortOutputEnable(int output_bit) const;

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


private:
    IRequires_Nothing *  MDC;
    Register  DataLo;
    Register  OutEnLo;
    RegisterBitManipulator  DataLoManip;

};

//===========================================================================
//  CLASS IRequires_Sysmon_AD5592R_Init -- "SubDevice" Requirement Interface
//===========================================================================

class IRequires_Sysmon_AD5592R_Init
{
public:

    // Customization
    virtual void SetSysmonDeviceConfiguration() = 0;


    // Customization Interface
    virtual ~IRequires_Sysmon_AD5592R_Init()
        {}
};

//===========================================================================
//  CLASS XUSystemMonitor -- XU System Monitor "SubDevice"
//===========================================================================

class XUSystemMonitor : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS XUSystemMonitor::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   SysMonControl_Addr;
        unsigned int   SysMonData_Addr;
    };

    XUSystemMonitor(IRequires_Sysmon_AD5592R_Init * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XUSystemMonitor();



    unsigned short  SpiRead(int device, unsigned short command);
    void            SpiWrite(int device, unsigned short command);

    void            SpiWriteMask(unsigned int device_mask, unsigned short command);
    void            SpiWriteAll(unsigned short command);

    bool            CheckAdcReady(int device);
    bool            CheckAdcReadyMask(unsigned int device_mask);

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUSystemMonitor::AD5592R --  AD5592R "do it all" I/O device
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class AD5592R
    {
    public:
        AD5592R(XUSystemMonitor & sub_dev, unsigned int dev);
        ~AD5592R()
            {}

        enum LDacMode { ldacWrite, ldacLoad, ldacUpdate};

        int  DeviceId()  {  return Device;   }

        // actions
        void Init();
        void Apply();

        //  chip specific conversions to Voltage
        double ToVoltage(unsigned int counts);
        unsigned int ToCounts(double voltage);

        // run time calls - are auto-applied
        void SoftwareReset();

//        void PowerDownAll();
//        void PowerDownDac(int dac, bool state);

        enum OutputCfgSettings { ocTriStated, ocDisabled=ocTriStated,
                                 ocAdc,
                                 ocDac, ocDac_PoweredUp=ocDac, ocDac_PoweredDown,
                                 ocDac_AdcCheck, ocDac_AdcCheck_PwrDown,
                                 ocGpioIn,
                                 ocGpioOut, ocGpioOut_noOpenDrain=ocGpioOut, ocGpioOut_OpenDrain,
                               };

        // customization config setup - affect state before open
        //   not auto-applied
        //  GP Control Config Methods
        //  AdcBufferPrecharge(bool)
        //  AdcBufferEnable(bool)
        //  LockSettings(bool)
        //  BroadsideWriteDacs(bool)
        //  AdcDoubleRange(bool)
        //  DacDoubleRange(bool)
        //   not auto-applied
        void SetRefEnabled(bool state);

        void SetOutputConfig(int pin, unsigned int cfg_setting);

        // raw field setup
        void PinAsAdc(int pin, bool state);
        void PinAsDac(int pin, bool state, bool powered_down=false);
        void PinAsGpioIn(int pin, bool state);
        void PinAsGpioOut(int pin, bool state, bool open_drain=false);
        void PinIsPulledDown(int pin, bool state);
        void PinIsThreeState(int pin, bool state);

        //  Low Level I/O
        unsigned int  Port(unsigned int address);
        void          Port(unsigned int address, unsigned int data);

        void          LDacModePort( LDacMode mode );

        void          DacPort(unsigned int dac, unsigned int data);
        unsigned int  DacPort(unsigned int dac);

        unsigned int  AdcPort(unsigned int adc, bool read_temp=false);
        unsigned int  TempPort();

        void          GpioPortAll(unsigned int bit_data);
        unsigned int  GpioPortAll();
        //unsigned int  GpioPortSome(unsigned int read_mask);  ...you can read only some of enabled GP inputs

//        void          GpioPort(unsigned int bit, bool state);  ...read/write just one bit state...update state data
//        bool          GpioPort(unsigned int bit);

    private:
        XUSystemMonitor &   Owner;
        int                 Device;
        std::vector<int>    RegisterImage;


        void  EnableBusy();
    };

    class AD5592R &       MultiIo(unsigned int device);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUSystemMonitor::SM_ControlRegister  --  Flash Address & Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SM_ControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   DeviceSelect;
        RegisterBitGroup   DeviceReady;
        RegisterBitGroup   DeviceAdcBusy;

        SM_ControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              DeviceSelect(*this, 0, 3), DeviceReady(*this, 16, 3),
              DeviceAdcBusy(*this, 24, 3)
        {  }
    };

    class SM_DataRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Command;
        RegisterBitGroup   ReadData;

        SM_DataRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Command(*this, 0, 16), ReadData(*this, 16, 16)
        {  }
    };

    void  AdcReady(int device);
    void  AdcReadyMask(unsigned int mask);

protected:
    void            SpiNotReady(int device);
    void            SpiReady(int device);
    void            SpiNotReadyMask(unsigned int mask);
    void            SpiReadyMask(unsigned int mask);
    void            SetupSpiDevice(int device);
    void            SetupSpiMaskDevice(unsigned int mask);
    bool            CheckSpiReadyMask(unsigned int mask);
    bool            CheckSpiNotReadyMask(unsigned int mask);
    unsigned int    DumpSpiRegister();

private:
    IRequires_Sysmon_AD5592R_Init *  MDC;
    SM_ControlRegister               Control;
    SM_DataRegister                  Data;

    AD5592R                          MultiIo_0;
    AD5592R                          MultiIo_1;
    AD5592R                          MultiIo_2;

};


//===========================================================================
//  CLASS SysMonInterface -- common SysMon converter functions
//===========================================================================

class SysMonInterface
{
public:
    SysMonInterface(XUSystemMonitor * device, unsigned int count, unsigned int common_factors)
        :  Device(device), Location(count), CommonFactorLevel(common_factors), CommonFactorScale(common_factors), ConversionMultiplier(count)
        {
        std::fill(CommonFactorLevel.begin(), CommonFactorLevel.end(), 0x800);
        std::fill(CommonFactorScale.begin(), CommonFactorScale.end(), 1.0);
        }
    ~SysMonInterface()
        {}

    void  SetCommonFactorLevel(unsigned int idx, unsigned int level)
                {  CommonFactorLevel[idx] = level;  }


    struct dp
        {
        unsigned int dev;
        unsigned int pin;

        dp(unsigned int dev_num=0, unsigned int pin_num=0)
            :  dev(dev_num),  pin(pin_num)
            {}

        };

protected:
    XUSystemMonitor *   Device;
    std::vector<dp>     Location;

    std::vector<unsigned int> CommonFactorLevel;
    std::vector<double> CommonFactorScale;
    std::vector<double> ConversionMultiplier;    // takes counts voltage -> actual voltage

    void    Write_a_Dac(unsigned int pin_code, double voltage);
    double  Read_an_Adc(unsigned int pin_code);
    void    CommonFactorDualDacUpdate(unsigned int common_pincode, unsigned int adjust_pincode,
                                      unsigned int common_factor_idx, double adjust_volts);
    void    CommonFactorTripleDacUpdate(unsigned int common_pincode,
                                        unsigned int adjust0_pincode, unsigned int adjust1_pincode,
                                        unsigned int common_factor_idx,
                                        double adjust0_volts, double adjust1_volts);
};


//===========================================================================
//  CLASS XU_LogicSystemMonitor -- XU Logic Based Sys Monitor "SubDevice"
//===========================================================================

class XU_LogicSystemMonitor : public IX6SubDevice
{
public:
    //=======================================================================
    //  CLASS XU_LogicSystemMonitor::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   CurTemperature_Addr;
        unsigned int   CurVccInt_Addr;
        unsigned int   CurVccAux_Addr;
        unsigned int   CurVbram_Addr;
        unsigned int   MaxTemperature_Addr;
        unsigned int   MaxVccInt_Addr;
        unsigned int   MaxVccAux_Addr;
        unsigned int   MaxVbram_Addr;
        unsigned int   MinTemperature_Addr;
        unsigned int   MinVccInt_Addr;
        unsigned int   MinVccAux_Addr;
        unsigned int   MinVbram_Addr;
        unsigned int   WarnThresh_U_Addr;
        unsigned int   FatalThresh_U_Addr;
        unsigned int   WarnThresh_L_Addr;
        unsigned int   FatalThresh_L_Addr;
    };

    XU_LogicSystemMonitor(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XU_LogicSystemMonitor();

    //  Internal Interface
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //  Read Raw Counts
    unsigned int  Read_Temperature_Raw()
        {  return TemperatureReg.Info.Value();  }
    unsigned int  Read_VccInt_Raw()
        {  return VccIntReg.Info.Value();  }
    unsigned int  Read_VccAux_Raw()
        {  return VccAuxReg.Info.Value();  }
    unsigned int  Read_Vbram_Raw()
        {  return VbramReg.Info.Value();  }
    unsigned int  Read_MaxTemperature_Raw()
        {  return MaxTemperatureReg.Info.Value();  }
    unsigned int  Read_MaxVccInt_Raw()
        {  return MaxVccIntReg.Info.Value();  }
    unsigned int  Read_MaxVccAux_Raw()
        {  return MaxVccAuxReg.Info.Value();  }
    unsigned int  Read_MaxVbram_Raw()
        {  return MaxVbramReg.Info.Value();  }
    unsigned int  Read_MinTemperature_Raw()
        {  return MinTemperatureReg.Info.Value();  }
    unsigned int  Read_MinVccInt_Raw()
        {  return MinVccIntReg.Info.Value();  }
    unsigned int  Read_MinVccAux_Raw()
        {  return MinVccAuxReg.Info.Value();  }
    unsigned int  Read_MinVbram_Raw()
        {  return MinVbramReg.Info.Value();  }
    unsigned int  Read_WarningThreshold_U_Raw()
        {  return WarningThreshold_U_Reg.Info.Value();  }
    unsigned int  Read_FatalThreshold_U_Raw()
        {  return FatalThreshold_U_Reg.Info.Value();  }
    unsigned int  Read_WarningThreshold_L_Raw()
        {  return WarningThreshold_L_Reg.Info.Value();  }
    unsigned int  Read_FatalThreshold_L_Raw()
        {  return FatalThreshold_L_Reg.Info.Value();  }

    //  Read Voltage
    double  Read_VccInt()
        {  return ToVoltage(Read_VccInt_Raw());  }
    double  Read_VccAux()
        {  return ToVoltage(Read_VccAux_Raw());  }
    double  Read_Vbram()
        {  return ToVoltage(Read_Vbram_Raw());  }
    double  Read_MaxVccInt()
        {  return ToVoltage(Read_MaxVccInt_Raw());  }
    double  Read_MaxVccAux()
        {  return ToVoltage(Read_MaxVccAux_Raw());  }
    double  Read_MaxVbram()
        {  return ToVoltage(Read_MaxVbram_Raw());  }
    double  Read_MinVccInt()
        {  return ToVoltage(Read_MinVccInt_Raw());  }
    double  Read_MinVccAux()
        {  return ToVoltage(Read_MinVccAux_Raw());  }
    double  Read_MinVbram()
        {  return ToVoltage(Read_MinVbram_Raw());  }

    //  Read Temperature
    double  Read_Temperature()
        {  return ToTemperature(Read_Temperature_Raw());  }
    double  Read_MaxTemperature()
        {  return ToTemperature(Read_MaxTemperature_Raw());  }
    double  Read_MinTemperature()
        {  return ToTemperature(Read_MinTemperature_Raw());  }
    double  Read_WarningThreshold_U()
        {  return ToTemperature(Read_WarningThreshold_U_Raw());  }
    double  Read_FatalThreshold_U()
        {  return ToTemperature(Read_FatalThreshold_U_Raw());  }
    double  Read_WarningThreshold_L()
        {  return ToTemperature(Read_WarningThreshold_L_Raw());  }
    double  Read_FatalThreshold_L()
        {  return ToTemperature(Read_FatalThreshold_L_Raw());  }

    //  Write Temperature
    void  Write_WarningThreshold_U(double temperature)
        {  WarningThreshold_U_Reg.Info.Value(FromTemperature(temperature));  }
    void  Write_FatalThreshold_U(double temperature)
        {  FatalThreshold_U_Reg.Info.Value(FromTemperature(temperature));  }
    void  Write_WarningThreshold_L(double temperature)
        {  WarningThreshold_L_Reg.Info.Value(FromTemperature(temperature));  }
    void  Write_FatalThreshold_L(double temperature)
        {  FatalThreshold_L_Reg.Info.Value(FromTemperature(temperature));  }

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_LogicSystemMonitor::LogicDataRegister  --  data access
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class LogicDataRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   Info;

        LogicDataRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              Info(*this, 6, 10)
        {}
    };

protected:

    double ToVoltage(unsigned int adc_code);
    double ToTemperature(unsigned int adc_code);

    unsigned int FromTemperature(double temperature);

private:
    IRequires_Nothing *              MDC;
    LogicDataRegister                TemperatureReg;
    LogicDataRegister                VccIntReg;
    LogicDataRegister                VccAuxReg;
    LogicDataRegister                VbramReg;
    LogicDataRegister                MaxTemperatureReg;
    LogicDataRegister                MaxVccIntReg;
    LogicDataRegister                MaxVccAuxReg;
    LogicDataRegister                MaxVbramReg;
    LogicDataRegister                MinTemperatureReg;
    LogicDataRegister                MinVccIntReg;
    LogicDataRegister                MinVccAuxReg;
    LogicDataRegister                MinVbramReg;
    LogicDataRegister                WarningThreshold_U_Reg;
    LogicDataRegister                FatalThreshold_U_Reg;
    LogicDataRegister                WarningThreshold_L_Reg;
    LogicDataRegister                FatalThreshold_L_Reg;

};

//===========================================================================
//  CLASS IMuxedAurora -- User interface to Muxed Aurora Device
//===========================================================================
//  Multiple Auroras using a select register to pick the device

class IMuxedAurora
{
public:
    enum LoopbackTypes { lbDisabled=0x0, lbParallel=0x1, lbSerial=0x2 };

    struct AuroraDeviceStatus {
        bool HardError;
        bool SoftError;
        bool FrameError;
        bool ChannelUp;
        bool PllLocked;
        unsigned int ErrorCount;

        AuroraDeviceStatus()
            : HardError(false), SoftError(false), FrameError(false),
              ChannelUp(false), PllLocked(false), ErrorCount(0)
        {}
    };

    //
    //  Interface Functions
    virtual bool  LogicPresent(unsigned int device) = 0;
    virtual void  InitAuroraLink(unsigned int device, LoopbackTypes lb) = 0;
    virtual void  StartTestSingleDevice(unsigned int device) = 0;
    virtual void  StartTestDualDevices(unsigned int device1, unsigned int device2) = 0;
    virtual bool  StartTestMB(unsigned int device) = 0;
    virtual void  EndTest(unsigned int device) = 0;
    virtual AuroraDeviceStatus DeviceStatus(unsigned int device) = 0;
    virtual void  EnableChannelsTxMB(unsigned int device) = 0;

    virtual ~IMuxedAurora() {}
};

//===========================================================================
//  CLASS XUAurora -- XU Muxed Aurora Device section "SubDevice"
//===========================================================================

class XUAurora : public IX6SubDevice, public IMuxedAurora
{
public:

    //=======================================================================
    //  CLASS XUAurora::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   Select_Addr;
        unsigned int   TestControl_Addr;
        unsigned int   ControlStatus_Addr;
        unsigned int   WritePort_Addr;
        unsigned int   ReadPort_Addr;
    };

    XUAurora(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XUAurora();

    virtual bool  LogicPresent(unsigned int device);
    virtual void  InitAuroraLink(unsigned int device, LoopbackTypes lb);
    virtual void  StartTestSingleDevice(unsigned int device);
    virtual void  StartTestDualDevices(unsigned int device1, unsigned int device2);
    virtual bool  StartTestMB(unsigned int device);
    virtual void  EndTest(unsigned int device);
    virtual AuroraDeviceStatus DeviceStatus(unsigned int device);
    virtual void  EnableChannelsTxMB(unsigned int device);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAurora::SelectRegister  --  Aurora IP select
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class SelectRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup   DeviceSelect;
        RegisterBitGroup   DeviceActive;

        SelectRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              DeviceSelect(*this, 0, 8), DeviceActive(*this, 8, 8)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAurora::TestControlRegister  --  Aurora Test Control
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class TestControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit        EnableTxTest;
        RegisterBit        EnableRxTest;
        RegisterBitGroup   TestErrorCount;

        TestControlRegister(IAddressingSpace & space, unsigned int offset)
            : inherited(space, offset),
              EnableTxTest(*this, 0), EnableRxTest(*this, 1),
              TestErrorCount(*this, 16, 16)
        {  }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAurora::ControlStatusRegister  -- Aurora Control/Status reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class ControlStatusRegister : public RefreshableShadowRegister
    {
        typedef RefreshableShadowRegister inherited;

    public:
        RegisterBit  GtxResetN;
        RegisterBit  PowerDown;
        RegisterBit  Run;
        RegisterBitGroup  LoopbackType;
        RegisterBit  ClearError;
        RegisterBit  EnableTx;
        RegisterBit  EnableRx;
        RegisterBit  HardError;
        RegisterBit  SoftError;
        RegisterBit  FrameError;
        RegisterBitGroup  LaneUp;
        RegisterBit  ChannelUp;
        RegisterBit  PllLocked;

    public:
        ControlStatusRegister( IAddressingSpace & space, int offset )
            :  inherited(space, offset),
               GtxResetN(*this, 0),   PowerDown(*this, 1),
               Run(*this, 2), LoopbackType(*this, 3, 3),
               ClearError(*this, 6),
               EnableTx(*this, 7), EnableRx(*this, 8),
               HardError(*this, 23),  SoftError(*this, 24),
               FrameError(*this, 25),
               LaneUp(*this, 26, 4),  ChannelUp(*this, 30),
               PllLocked(*this, 31)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAurora::PortRegister  -- Aurora Read/Write port reg
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    class PortRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  Data;
        RegisterBitGroup  Addr;
        RegisterBit       ReadWrite;
        RegisterBit       ReadyValid;

    public:
        PortRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               Data(*this, 0, 24),
               Addr(*this, 24, 6),
               ReadWrite(*this, 30),
               ReadyValid(*this, 31)
            {
            }
    };

private:
    IRequires_Nothing *      MDC;
    SelectRegister           SelectReg;
    TestControlRegister      TestControlReg;
    ControlStatusRegister    ControlStatusReg;
    PortRegister             WritePortReg;
    PortRegister             ReadPortRegReg;

    void  SetDevice(unsigned int device);
};

//===========================================================================
//  CLASS IRequires_XUSioPll -- XU "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the standard clock and vcxo devices

class IRequires_XUSioPll
{
public:
    virtual IX6_I2C_uDevice *  MakeXuSioI2CLink() = 0;
    virtual Lmk61E2 *          MakeXuSioPllObject(IRequires_I2C_RegIo & device) = 0;

    virtual ~IRequires_XUSioPll()
        {}
};

//===========================================================================
//  CLASS IXUSioPllControl -- User interface to Sio Pll Control Functions
//===========================================================================

class IXUSioPllControl
{
public:
    //
    //  Interface Functions
    virtual Lmk61E2 &  Pll() = 0;

    virtual ~IXUSioPllControl() {}
};

//===========================================================================
//  CLASS XUSioPll -- X6 Sio Pll Periph section "SubDevice"
//===========================================================================

class XUSioPll : public IX6SubDevice, public IXUSioPllControl
{
public:
    //=======================================================================
    //  CLASS XUSioPll::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   SioPll_Addr;
    };

    XUSioPll(IRequires_XUSioPll * mdc/*, IAddressingSpace & space, MapRegisters & regs*/);
    virtual ~XUSioPll();

    //  Methods
    virtual Lmk61E2 &  Pll()  {  return *SioPllDev;  }

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

private:
    IRequires_XUSioPll *   MDC;
private:
    IX6_I2C_uDevice *    Sio_I2C_uDev;
    Lmk61E2 *            SioPllDev;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative

#endif
