//---------------------------------------------------------------------------
// XUSubDevice_Stream_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.

#include <XUSubDevice_Stream_Mb.h>

using namespace std;

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XUAFE_Common -- XU AFE Common Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XUAFE_Common
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XUAFE_Common
//---------------------------------------------------------------------------

XUAFE_Common::XUAFE_Common(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TestControl(space, regs.AfeTestControl_Addr),
       SwTrigger(space, regs.AfeSoftwareTrigger_Addr),
       ExtSyncSelect(space, regs.AfeExtSyncSelect_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAFE_Common
//---------------------------------------------------------------------------

XUAFE_Common::~XUAFE_Common()
{
}

//---------------------------------------------------------------------------
//  XUAFE_Common::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAFE_Common::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  XUAFE_Common::OpenHardware()
//---------------------------------------------------------------------------

void  XUAFE_Common::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    AllOff();
    TestModeEnabled( tInputOutput, false, 0 );
}

//---------------------------------------------------------------------------
//  XUAFE_Common::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAFE_Common::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;

    AllOff();
}

//---------------------------------------------------------------------------
//  XUAFE_Common::AllOff()
//---------------------------------------------------------------------------

void  XUAFE_Common::AllOff()
{
    SoftwareTrigger( tInputOutput, false );
}

//---------------------------------------------------------------------------
//  XUAFE_Common::TestModeEnabled() --  Set Test Mode state
//---------------------------------------------------------------------------

void XUAFE_Common::TestModeEnabled(AfeDirection which, bool state, unsigned int mode)
{
    switch (which)
        {
        case tInput:
            TestControl.AdcTestMode.Value(mode);
            TestControl.AdcTestEnable.Value(state);
            break;
        case tOutput:
            TestControl.DacTestMode.Value(mode);
            TestControl.DacTestEnable.Value(state);
            break;
        case tInputOutput:
            TestControl.AdcTestMode.Value(mode);
            TestControl.DacTestMode.Value(mode);
            TestControl.AdcTestEnable.Value(state);
            TestControl.DacTestEnable.Value(state);
            break;
        }
}

//---------------------------------------------------------------------------
//  XUAFE_Common::SoftwareTrigger() --  Set SW Trigger state
//---------------------------------------------------------------------------

void XUAFE_Common::SoftwareTrigger(AfeDirection which, bool state)
{
    switch (which)
        {
        case tInput:
            SwTrigger.AdcTrigger.Value(state);
            break;
        case tOutput:
            SwTrigger.DacTrigger.Value(state);
            break;
        case tInputOutput:
            SwTrigger.AdcTrigger.Value(state);
            SwTrigger.DacTrigger.Value(state);
            break;
        }

    SwTrigger.Apply();
}

//---------------------------------------------------------------------------
//  XUAFE_Common::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

void XUAFE_Common::ExternalSyncSelection(AfeExtSyncOptions option)
{
    switch (option)
        {
        case essFrontPanel:
            ExtSyncSelect.ExtSyncOption.Value(0);
            break;
        case essCarrier:
            ExtSyncSelect.ExtSyncOption.Value(1);
            break;
        case essB2b:
            ExtSyncSelect.ExtSyncOption.Value(2);
            break;
        }
}

//---------------------------------------------------------------------------
//  XUAFE_Common::ExternalSyncSelection() --  Set Ext Trigger source option
//---------------------------------------------------------------------------

XUAFE_Common::AfeExtSyncOptions XUAFE_Common::ExternalSyncSelection() const
{
    unsigned int option =  ExtSyncSelect.ExtSyncOption.Value();
    if (option == 0)
        return essFrontPanel;
    else if (option == 1)
        return essCarrier;
    else // if (option == 2)
        return essB2b;
}

//===========================================================================
//  CLASS XUAlertSystem -- XU Alert "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  Housekeeping for class XUAlertSystem
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XUAlertSystem
//---------------------------------------------------------------------------

XUAlertSystem::XUAlertSystem(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       AlertEnableReg(space, regs.AlertEnable_Addr),
       AlertControlReg(space, regs.AlertControl_Addr),
       SoftwareAlertReg(space, regs.SoftwareAlert_Addr),
       AlertClearReg(space, regs.AlertClear_Addr),
       AlertEnableCache(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XUAlertSystem
//---------------------------------------------------------------------------

XUAlertSystem::~XUAlertSystem()
{
}

//---------------------------------------------------------------------------
//  XUAlertSystem::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XUAlertSystem::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  XUAlertSystem::OpenHardware()
//---------------------------------------------------------------------------

void  XUAlertSystem::OpenHardware()
{
    if (!DeviceEnabled())
        return;

    AlertEnableCache.Value(0);   // all alerts off
}

//---------------------------------------------------------------------------
//  XUAlertSystem::ConfigureHardware()
//---------------------------------------------------------------------------

void  XUAlertSystem::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
    //
    //  Turn on Timestamp Clock
    AlertTimestampEnable(true);

    // Apply enabled state for all supported alerts
    AlertEnableReg.Value(AlertEnableCache.Value());
}

//---------------------------------------------------------------------------
//  XUAlertSystem::AlertTimestampEnable() - enable timestamping of alerts
//---------------------------------------------------------------------------

void XUAlertSystem::AlertTimestampEnable(bool state)
{
    AlertControlReg.TimestampOn.Value(state);
}

//------------------------------------------------------------------------------
//   XUAlertSystem::AlertEnable() -- Change Enable state of alert
//------------------------------------------------------------------------------

void  XUAlertSystem::AlertEnable(AlertType alert, bool state)
{
    unsigned int alert_bit = static_cast<unsigned int>(alert);
    AlertEnableCache.Bit(alert_bit, state);
}

//------------------------------------------------------------------------------
//   XUAlertSystem::AlertClear() -- Clear alert logic blocking
//------------------------------------------------------------------------------

void  XUAlertSystem::AlertClear(AlertType alert)
{
    unsigned int alert_bit = static_cast<unsigned int>(alert);

    AlertClearReg.Bit(alert_bit, true);
    AlertClearReg.Bit(alert_bit, false);
}

//------------------------------------------------------------------------------
//   XUAlertSystem::SoftwareAlert() -- trigger software alert
//------------------------------------------------------------------------------

void  XUAlertSystem::SoftwareAlert(unsigned int tag)
{
    SoftwareAlertReg.Value(tag);
}

//---------------------------------------------------------------------------
//  XUAlertSystem::AlertInfoString() --  Produce string display for alert
//---------------------------------------------------------------------------

std::string  XUAlertSystem::AlertInfoString(const Innovative::McAlert & alert, bool add_raw_data)
{
    AccessDatagram<unsigned int> AlertDG(alert);

    MemoryBitManipulator ActiveAlertBits(AlertDG[0]);

    std::stringstream msg;
    unsigned int Index=2;

    // separate alerts
    msg << "-----------------------------------";
    msg << '\r' << '\n' << "Timestamp: " << std::dec << AlertDG[1];;


    for (unsigned int i=0; i<32; i++)
        {
        // Check if the alert bit is set
        if (ActiveAlertBits.Bit(i) == 0)
            continue;

        msg << '\r' << '\n';

        IUsesXUAlerts::AlertType AlertType = static_cast<IUsesXUAlerts::AlertType>(i);
        switch (AlertType)
            {
            case IUsesXUAlerts::alertTimeStampRollover:
                {
                msg << "Time stamp rollover 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertSoftware:
                {
                msg << "Software Alert 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertWarningTemperature:
                {
                msg << "Temperature Warning Alert 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertVFifoDiagnostic:
                {
                msg << "VFIFO 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertPatternMode:
                    {
                    msg << "Pattern Done Alert: Tag " << std::hex << '(' << AlertDG[Index] << ')';
                    }
                    break;
            case IUsesXUAlerts::alertTrigger:
                {
                std::string triggerType;
                switch (AlertDG[Index] & 0x3)
                    {
                    case 0:  triggerType = "? ";  break;
                    case 1:  triggerType = "Input ";  break;
                    case 2:  triggerType = "Output ";  break;
                    case 3:  triggerType = "Input and Output ";  break;
                    }
                msg << "Trigger 0x" << std::hex << AlertDG[Index]
                    << " Type: " <<  triggerType;
                }
                break;
            case IUsesXUAlerts::alertOutputUnderflow:
                {
                msg << "Output FIFO underflow 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertInputOverflow:
                {
                msg << "Input FIFO overflow 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertInputOverrange:
                {
                msg << "Input overrange 0x" << std::hex << AlertDG[Index];
                }
                break;
            case IUsesXUAlerts::alertDacChip:
                {
                std::string dacs;
                switch (AlertDG[Index] & 0x3)
                    {
                    case 0:  dacs = "? ";  break;
                    case 1:  dacs = "Dac 0";  break;
                    case 2:  dacs = "Dac 1";  break;
                    case 3:  dacs = "Dac 0 & 1";  break;
                    }
                msg << "DAC Chip Alert: " << dacs;
                }
                break;
            default:
                msg << "Unknown Alert, Type " << AlertType;
                break;
            }

        Index++;
        }

    //
    //  Optionally append raw dump of alert to data set
    if (add_raw_data)
        {
        msg << '\r' << '\n' << "Alert: ";
        for (unsigned int i=0; i<AlertDG.size(); i++)
            msg << '\r' << '\n' << std::hex << "[" << i << "]=" << AlertDG[i];
        }

    // separate alerts
    msg << '\r' << '\n' << "-----------------------------------";

    return std::string(msg.str());
}

//===========================================================================
//  CLASS XU_Chrono_Type1 -- XU AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XU_Chrono_Type1
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XU_Chrono_Type1
//---------------------------------------------------------------------------

XU_Chrono_Type1::XU_Chrono_Type1(IRequires_XU_Type1_Clocks * mdc,  IAddressingSpace & /*space*/, MapRegisters & /*regs*/)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  VcxoDev(0), XoDev(0), TxPllDev(0),
       ClockIoIntf(this), ClockDevs(this), TimeRate(this),
       CallBack(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_Chrono_Type1
//---------------------------------------------------------------------------

XU_Chrono_Type1::~XU_Chrono_Type1()
{
    delete PllDev;     PllDev  = 0;
    delete VcxoDev;    VcxoDev  = 0;
    delete XoDev;      XoDev  = 0;
    delete TxPllDev;   TxPllDev  = 0;
    delete CallBack;   CallBack  = 0;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XU_Chrono_Type1::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev      = MDC->Make_AfePllObject();
    I2C_uDev    = MDC->Make_AfeVcxoI2CLink();
    VcxoDev     = MDC->Make_AfeVcxoObject(I2C_uDev->Device());
    XoI2C_uDev  = MDC->Make_AfeXoI2CLink();
    XoDev       = MDC->Make_AfeXoObject(XoI2C_uDev->Device());
    TxPllDev    = MDC->Make_AfeTxPllObject();
    CallBack    = MDC->Make_Xu_Chrono_CB();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::OpenHardware()
//---------------------------------------------------------------------------

void  XU_Chrono_Type1::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "XU_Chrono_Type1::OpenHardware()");
    if (!DeviceEnabled())
        return;

    Debug.Log("Init Xo LMK61E2 Reference...");
    XoDev->Init();

    Debug.Log("InitSi570 Reference...");
    VcxoDev->Init();

    Debug.Log("Init LMK PLL...");
    PllDev->Powered(true);
    PllDev->Init();

    Debug.Log("Init AD4356 TX PLL...");
    TxPllDev->Init();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::PreconfigureHardware()
//---------------------------------------------------------------------------

void  XU_Chrono_Type1::PreconfigureHardware()
{
    ScopedTraceRef ccc(Debug, "XU_Chrono_Type1::PreconfigureHardware()");
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_Chrono_Type1::ConfigureHardware()
{
    ScopedTraceRef ccc(Debug, "XU_Chrono_Type1::ConfigureHardware()");
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::Pll()
//---------------------------------------------------------------------------

Lmk04820 & XU_Chrono_Type1::Pll()
{
    return *PllDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::Vcxo()
//---------------------------------------------------------------------------

Si570 & XU_Chrono_Type1::Vcxo()
{
    return *VcxoDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::Xo()
//---------------------------------------------------------------------------

Lmk61E2 & XU_Chrono_Type1::Xo()
{
    return *XoDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::TxPll()
//---------------------------------------------------------------------------

ADF4355 & XU_Chrono_Type1::TxPll()
{
    return *TxPllDev;
}

//===========================================================================
//  CLASS XU_Chrono_Type1::ClockIo -- XU Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  XU_Chrono_Type1::ClockIo::FrequencyActual() const
{
    return Owner->TimeRate.FrequencyActual();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  XU_Chrono_Type1::ClockIo::Frequency(double value)
{
    Owner->TimeRate.Frequency(value);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  XU_Chrono_Type1::ClockIo::Frequency() const
{
    return Owner->TimeRate.Frequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XU_Chrono_Type1::ClockIo::System()
{
    return Owner->TimeRate;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XU_Chrono_Type1::ClockIo::Adc()
{
    return Owner->TimeRate;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XU_Chrono_Type1::ClockIo::Dac()
{
    return Owner->TimeRate;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IXUClockIo::IIClockSource XU_Chrono_Type1::ClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void XU_Chrono_Type1::ClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IXUClockIo::IIReferenceSource XU_Chrono_Type1::ClockIo::Reference() const
{
    Lmk04820::LmkReferenceSource current_source = Owner->PllDev->ReferenceSource();
    return static_cast<IXUClockIo::IIReferenceSource>(Owner->PllDev->GetSourceMapping(current_source));
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void XU_Chrono_Type1::ClockIo::Reference(IIReferenceSource src)
{
    Lmk04820::LmkReferenceSource new_source = static_cast<Lmk04820::LmkReferenceSource>(Owner->PllDev->GetSourceMapping_Reverse(src));
    if (Owner->PllDev->SourceInUse(new_source))
        Owner->PllDev->ReferenceSource(new_source);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void XU_Chrono_Type1::ClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double XU_Chrono_Type1::ClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  XU_Chrono_Type1::ClockIo::Locked() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return (Owner->PllDev->Locked() && Owner->TxPllDev->Locked());
    else
        return Owner->TxPllDev->Locked();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::Frequency() -- Chrono "Set Frequency"
//---------------------------------------------------------------------------
//  all frequency changes come through here...Adc, Dac, System

void  XU_Chrono_Type1::ChronoTBR::Frequency(double freq)
{
    // Board Specific Time Setting Ops
    if (Chrono->ClockIoIntf.Reference()==IXUClockIo::rsInternal)
        {
        Chrono->XoDev->Frequency(Chrono->PllDev->Reference());
        Chrono->XoDev->Reapply();
        }

    Chrono->VcxoDev->SetVcoFrequency(100e6);  // temporary

    Chrono->CallBack->OnChrono_SetFrequency(Chrono, freq);
    Chrono->PllDev->Frequency(freq);

    bool Ch0Enabled = Chrono->MDC->XU_Type1_ChannelEnabled(0);
    bool Ch1Enabled = Chrono->MDC->XU_Type1_ChannelEnabled(1);

    ADF4355::AdfDevice DeviceEnabled;

    if (Ch0Enabled && Ch1Enabled)
        DeviceEnabled = ADF4355::Dev_Both;
    else if (Ch1Enabled)
        DeviceEnabled = ADF4355::Dev1;
    else
        DeviceEnabled = ADF4355::Dev0;

    Chrono->TxPllDev->DeviceRouting(DeviceEnabled);
    Chrono->TxPllDev->Reference(Chrono->LmkClkRates.TxPllRefClk);
    Chrono->TxPllDev->Frequency(freq);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::Frequency() -- Chrono "Get Frequency"
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type1::ChronoTBR::Frequency() const
{
    return Chrono->TxPllDev->Frequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::FrequencyActual() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type1::ChronoTBR::FrequencyActual() const
{
    return Chrono->TxPllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::EffectiveFrequency() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type1::ChronoTBR::EffectiveFrequency() const
{
    return Chrono->PllDev->EffectiveFrequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::EffectiveFrequencyActual() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type1::ChronoTBR::EffectiveFrequencyActual() const
{
    return Chrono->PllDev->EffectiveFrequencyActual();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::MultipliedFrequency() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type1::ChronoTBR::MultipliedFrequency() const
{
    return Chrono->PllDev->MultipliedFrequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type1::ChronoTBR::MultipliedFrequencyActual() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type1::ChronoTBR::MultipliedFrequencyActual() const
{
    return Chrono->PllDev->MultipliedFrequencyActual();
}

//===========================================================================
//  CLASS XU_Chrono_Type2 -- XU AFE Clock "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XU_Chrono_Type2
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XU_Chrono_Type2
//---------------------------------------------------------------------------

XU_Chrono_Type2::XU_Chrono_Type2(IRequires_XU_Type2_Clocks * mdc,  IAddressingSpace & /*space*/, MapRegisters & /*regs*/)
    :  MDC(mdc),
       //  PllControl(space, regs.AfePllControl_Addr),
       //  PllSpi(space, regs.AfePllSpi_Addr),
       PllDev(0),  VcxoDev(0), XoDev(0), TxPllDev(0), RxPllDev(0),
       ClockIoIntf(this), ClockDevs(this), TimeRate(this),
       CallBack(0)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_Chrono_Type2
//---------------------------------------------------------------------------

XU_Chrono_Type2::~XU_Chrono_Type2()
{
    delete PllDev;     PllDev  = 0;
    delete VcxoDev;    VcxoDev  = 0;
    delete XoDev;      XoDev  = 0;
    delete TxPllDev;   TxPllDev  = 0;
    delete RxPllDev;   RxPllDev  = 0;
    delete CallBack;   CallBack  = 0;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XU_Chrono_Type2::ConstructCustomParts()
{
    if (!DeviceEnabled())
        return;

    PllDev      = MDC->Make_AfePllObject();
    I2C_uDev    = MDC->Make_AfeVcxoI2CLink();
    VcxoDev     = MDC->Make_AfeVcxoObject(I2C_uDev->Device());
    XoI2C_uDev  = MDC->Make_AfeXoI2CLink();
    XoDev       = MDC->Make_AfeXoObject(XoI2C_uDev->Device());
    TxPllDev    = MDC->Make_AfeTxPllObject();
    RxPllDev    = MDC->Make_AfeRxPllObject();
    CallBack    = MDC->Make_Xu_Chrono_CB();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::OpenHardware()
//---------------------------------------------------------------------------

void  XU_Chrono_Type2::OpenHardware()
{
    ScopedTraceRef ccc(Debug, "XU_Chrono_Type2::OpenHardware()");
    if (!DeviceEnabled())
        return;

    Debug.Log("Init Xo LMK61E2 Reference...");
    XoDev->Init();

    Debug.Log("InitSi570 Reference...");
    VcxoDev->Init();

    Debug.Log("Init LMK PLL...");
    PllDev->Powered(true);
    PllDev->Init();

    Debug.Log("Init AD4356 TX PLL...");
    TxPllDev->Init();

    Debug.Log("Init AD4356 RX PLL...");
    TxPllDev->Init();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::PreconfigureHardware()
//---------------------------------------------------------------------------

void  XU_Chrono_Type2::PreconfigureHardware()
{
    ScopedTraceRef ccc(Debug, "XU_Chrono_Type2::PreconfigureHardware()");
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_Chrono_Type2::ConfigureHardware()
{
    ScopedTraceRef ccc(Debug, "XU_Chrono_Type2::ConfigureHardware()");
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::Pll()
//---------------------------------------------------------------------------

Lmk04820 & XU_Chrono_Type2::Pll()
{
    return *PllDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::Vcxo()
//---------------------------------------------------------------------------

Si570 & XU_Chrono_Type2::Vcxo()
{
    return *VcxoDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::Xo()
//---------------------------------------------------------------------------

Lmk61E2 & XU_Chrono_Type2::Xo()
{
    return *XoDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::TxPll()
//---------------------------------------------------------------------------

ADF4355 & XU_Chrono_Type2::TxPll()
{
    return *TxPllDev;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::RxPll()
//---------------------------------------------------------------------------

ADF4355 & XU_Chrono_Type2::RxPll()
{
    return *RxPllDev;
}

//===========================================================================
//  CLASS XU_Chrono_Type2::ClockIo -- XU Clock "SubDevice" User Interface
//===========================================================================
//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::FrequencyActual()
//---------------------------------------------------------------------------

double  XU_Chrono_Type2::ClockIo::FrequencyActual() const
{
    return Owner->TimeRate.FrequencyActual();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Frequency(double) -- write frequency
//---------------------------------------------------------------------------

void  XU_Chrono_Type2::ClockIo::Frequency(double value)
{
    Owner->TimeRate.Frequency(value);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Frequency() --  read frequency
//---------------------------------------------------------------------------

double  XU_Chrono_Type2::ClockIo::Frequency() const
{
    return Owner->TimeRate.Frequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::System() --  return total group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XU_Chrono_Type2::ClockIo::System()
{
    return Owner->TimeRate;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Adc() --  return "Adc" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XU_Chrono_Type2::ClockIo::Adc()
{
    return Owner->TimeRate;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Dac() --  return "Dac" group frequency I/O
//---------------------------------------------------------------------------

ITimebaseRate &  XU_Chrono_Type2::ClockIo::Dac()
{
    return Owner->TimeRate;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Source() -- Retrieve clock source
//---------------------------------------------------------------------------

IXUClockIo::IIClockSource XU_Chrono_Type2::ClockIo::Source() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return csInternal;
    else
        return csExternal;
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Source() -- Select clock source
//---------------------------------------------------------------------------

void XU_Chrono_Type2::ClockIo::Source(IIClockSource src)
{
    if (src == csInternal)
        Owner->PllDev->Pll_Mode();
    else
        Owner->PllDev->ExtClock_Mode();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Reference() -- Retrieve reference source
//---------------------------------------------------------------------------

IXUClockIo::IIReferenceSource XU_Chrono_Type2::ClockIo::Reference() const
{
    Lmk04820::LmkReferenceSource current_source = Owner->PllDev->ReferenceSource();
    return static_cast<IXUClockIo::IIReferenceSource>(Owner->PllDev->GetSourceMapping(current_source));
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Reference() -- Select reference source
//---------------------------------------------------------------------------

void XU_Chrono_Type2::ClockIo::Reference(IIReferenceSource src)
{
    Lmk04820::LmkReferenceSource new_source = static_cast<Lmk04820::LmkReferenceSource>(Owner->PllDev->GetSourceMapping_Reverse(src));
    if (Owner->PllDev->SourceInUse(new_source))
        Owner->PllDev->ReferenceSource(new_source);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::ReferenceFrequency() -- Change reference frequency
//---------------------------------------------------------------------------

void XU_Chrono_Type2::ClockIo::ReferenceFrequency(double value)
{
    Owner->PllDev->Reference(value);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::ReferenceFrequency() -- Query reference frequency
//---------------------------------------------------------------------------

double XU_Chrono_Type2::ClockIo::ReferenceFrequency() const
{
    return Owner->PllDev->Reference();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ClockIo::Locked() -- Query PLL Locked state
//---------------------------------------------------------------------------

bool  XU_Chrono_Type2::ClockIo::Locked() const
{
    if (Owner->PllDev->IsIn_Pll_Mode())
        return (Owner->PllDev->Locked() && Owner->TxPllDev->Locked());
    else
        return Owner->TxPllDev->Locked();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::Frequency() -- Chrono "Set Frequency"
//---------------------------------------------------------------------------
//  all frequency changes come through here...Adc, Dac, System

void  XU_Chrono_Type2::ChronoTBR::Frequency(double freq)
{
    // Board Specific Time Setting Ops
    if (Chrono->ClockIoIntf.Reference()==IXUClockIo::rsInternal)
        {
        Chrono->XoDev->Frequency(Chrono->PllDev->Reference());
        Chrono->XoDev->Reapply();
        }

    Chrono->VcxoDev->SetVcoFrequency(100e6);  // temporary

    Chrono->CallBack->OnChrono_SetFrequency(Chrono, freq);
    Chrono->PllDev->Frequency(freq);

    // Probably need one of these for both TX and RX
    bool Ch0Enabled = Chrono->MDC->XU_Type2_ChannelEnabled(0);
    bool Ch1Enabled = Chrono->MDC->XU_Type2_ChannelEnabled(1);

    ADF4355::AdfDevice TxDeviceEnabled;

    if (Ch0Enabled && Ch1Enabled)
        TxDeviceEnabled = ADF4355::Dev_Both;
    else if (Ch1Enabled)
        TxDeviceEnabled = ADF4355::Dev1;
    else
        TxDeviceEnabled = ADF4355::Dev0;

    Chrono->TxPllDev->DeviceRouting(TxDeviceEnabled);
    Chrono->TxPllDev->Reference(Chrono->LmkClkRates.TxPllRefClk);
    Chrono->TxPllDev->Frequency(freq);

    ADF4355::AdfDevice RxDeviceEnabled;

    if (Ch0Enabled && Ch1Enabled)
        RxDeviceEnabled = ADF4355::Dev_Both;
    else if (Ch1Enabled)
        RxDeviceEnabled = ADF4355::Dev1;
    else
        RxDeviceEnabled = ADF4355::Dev0;

    Chrono->RxPllDev->DeviceRouting(RxDeviceEnabled);
    Chrono->RxPllDev->Reference(Chrono->LmkClkRates.TxPllRefClk);
    Chrono->RxPllDev->Frequency(freq);
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::Frequency() -- Chrono "Get Frequency"
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type2::ChronoTBR::Frequency() const
{
    return Chrono->TxPllDev->Frequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::FrequencyActual() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type2::ChronoTBR::FrequencyActual() const
{
    return Chrono->TxPllDev->FrequencyActual();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::EffectiveFrequency() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type2::ChronoTBR::EffectiveFrequency() const
{
    return Chrono->PllDev->EffectiveFrequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::EffectiveFrequencyActual() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type2::ChronoTBR::EffectiveFrequencyActual() const
{
    return Chrono->PllDev->EffectiveFrequencyActual();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::MultipliedFrequency() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type2::ChronoTBR::MultipliedFrequency() const
{
    return Chrono->PllDev->MultipliedFrequency();
}

//---------------------------------------------------------------------------
//  XU_Chrono_Type2::ChronoTBR::MultipliedFrequencyActual() --
//---------------------------------------------------------------------------
//  all frequency requests come through here...Adc, Dac, System

double  XU_Chrono_Type2::ChronoTBR::MultipliedFrequencyActual() const
{
    return Chrono->PllDev->MultipliedFrequencyActual();
}

//===========================================================================
//  CLASS XU_VFifoInfo -- XU VFIFO Info Registers "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XU_VFifoInfo
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XU_VFifoInfo
//---------------------------------------------------------------------------

XU_VFifoInfo::XU_VFifoInfo(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       VFifoStatusReg(space, regs.VFifoStatus_Addr),
       VFifoWordCountReg(space, regs.VFifoWordCount_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_VFifoInfo
//---------------------------------------------------------------------------

XU_VFifoInfo::~XU_VFifoInfo()
{
}

//---------------------------------------------------------------------------
//  XU_VFifoInfo::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XU_VFifoInfo::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  XU_VFifoInfo::OpenHardware()
//---------------------------------------------------------------------------

void  XU_VFifoInfo::OpenHardware()
{
    if (!DeviceEnabled())
        return;

}

//---------------------------------------------------------------------------
//  XU_VFifoInfo::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_VFifoInfo::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_VFifoInfo::AlmostEmpty()
//---------------------------------------------------------------------------

bool  XU_VFifoInfo::AlmostEmpty( unsigned int vfifo_idx )
{
    unsigned int mask = 1<<vfifo_idx;
    unsigned int ae_data = VFifoStatusReg.FifoAlmostEmpty.Value();
    return (mask&ae_data) ? true : false ;
}

//---------------------------------------------------------------------------
//  XU_VFifoInfo::AlmostFull()
//---------------------------------------------------------------------------

bool  XU_VFifoInfo::AlmostFull( unsigned int vfifo_idx )
{
    unsigned int mask = 1<<vfifo_idx;
    unsigned int af_data = VFifoStatusReg.FifoAlmostFull.Value();
    return (mask&af_data) ? true : false ;
}

//---------------------------------------------------------------------------
//  XU_VFifoInfo::WordCount() --  read fifo word count
//---------------------------------------------------------------------------

unsigned int XU_VFifoInfo::WordCount( unsigned int vfifo_idx )
{
    unsigned int mask = 1<<vfifo_idx;
    VFifoStatusReg.FifoSelect.Value(mask);

    return VFifoWordCountReg.Value();
}


//===========================================================================
//  CLASS XU_StreamTester -- XU Stream Checker/Generator "SubDevice"
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class XU_StreamTester
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class XU_StreamTester
//---------------------------------------------------------------------------

XU_StreamTester::XU_StreamTester(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs)
    :  MDC(mdc),
       TestEnableReg(space, regs.XuTestEnable_Addr),
       PacketSizeReg(space, regs.XuTestPacketSize_Addr),
       TxDecimationReg(space, regs.XuTestOutDecimation_Addr),
       TxCheckErrorReg(space, regs.XuTestChkErrors_Addr),
       ExtIntTrigger(space, regs.XuTestSignalExtInt_Addr)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_StreamTester
//---------------------------------------------------------------------------

XU_StreamTester::~XU_StreamTester()
{
}


//---------------------------------------------------------------------------
//  XU_StreamTester::ConstructCustomParts()
//---------------------------------------------------------------------------

void  XU_StreamTester::ConstructCustomParts()
{
}

//---------------------------------------------------------------------------
//  XU_StreamTester::OpenHardware()
//---------------------------------------------------------------------------

void  XU_StreamTester::OpenHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_StreamTester::ConfigureHardware()
//---------------------------------------------------------------------------

void  XU_StreamTester::ConfigureHardware()
{
    if (!DeviceEnabled())
        return;
}

//---------------------------------------------------------------------------
//  XU_StreamTester::TxCheckerEnable() - turn on test data checker
//---------------------------------------------------------------------------

void  XU_StreamTester::TxCheckerEnable(unsigned int ch, bool state)
{
    MemoryBitManipulator TxEnBits(TestEnableReg.TxCheckerEnable.Value());
    TxEnBits.Bit(ch, state);
    TestEnableReg.TxCheckerEnable.Value(TxEnBits.Value());
}

//---------------------------------------------------------------------------
//  XU_StreamTester::RxTestGenEnable() - turn on test data generator
//---------------------------------------------------------------------------

void  XU_StreamTester::RxTestGenEnable(unsigned int ch, bool state)
{
    MemoryBitManipulator RxGenBits(TestEnableReg.RxRampGenEnable.Value());
    RxGenBits.Bit(ch, state);
    TestEnableReg.RxRampGenEnable.Value(RxGenBits.Value());
}

//---------------------------------------------------------------------------
//  XU_StreamTester::RxPacketSize() - test generator burst size
//---------------------------------------------------------------------------
//  will generate this many words before moving to next enabled channel

void  XU_StreamTester::RxPacketSize(unsigned int size_words)
{
    PacketSizeReg.Value(size_words);
}

//---------------------------------------------------------------------------
//  XU_StreamTester::TxDecimation() --  reduces output rate
//---------------------------------------------------------------------------

void  XU_StreamTester::TxDecimation(unsigned int factor)
{
    unsigned int logic_factor = (factor==0) ? 0 : factor-1;
    TxDecimationReg.Value(logic_factor);
}

//---------------------------------------------------------------------------
//  XU_StreamTester::TxCheckerHasError() --  Checker error on channel
//---------------------------------------------------------------------------

bool XU_StreamTester::TxCheckerHasError(unsigned int ch)
{
    MemoryBitManipulator TxCheckBits(TxCheckErrorReg.Value());
    return TxCheckBits.Bit(ch);
}


//---------------------------------------------------------------------------
//  XU_StreamTester::TxDecimation() --  reduces output rate
//---------------------------------------------------------------------------

void XU_StreamTester::ExtInt()
{
    ExtIntTrigger.Value(0);
}

}  // namespace Innovative



