// XUSubDevice_Stream_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//
//

#ifndef XUSubDevice_Stream_MbH
#define XUSubDevice_Stream_MbH

#include <X6Support_Mb.h>
#include <HardwareRegister_Mb.h>
#include <Si570_Mb.h>
#include <ADF4355_Mb.h>
#include <LMK61E2_Mb.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//
//==============================================================================
//  CLASS IXUClockIo -- User interface to control clock functionality
//==============================================================================

class IXUClockIo
{
public:
    enum IIClockSource { csExternal, csInternal };   // clock routing
    enum IIReferenceSource { rsInternal, rsFrontPanel, rsCarrier };
//    enum IIClockSelect {  cslFrontPanel, cslCarrier };


    virtual ~IXUClockIo()  {  }

    //
    //  Interface Functions
    // Methods
    virtual double              FrequencyActual() const = 0;

    // Properties
    virtual void                Frequency(double value) = 0;
    virtual double              Frequency() const = 0;

    virtual ITimebaseRate &     Adc() = 0;
    virtual ITimebaseRate &     Dac() = 0;
    virtual ITimebaseRate &     System() = 0;

    virtual void                Source(IIClockSource src) = 0;
    virtual IIClockSource       Source() const = 0;

//    virtual void                ExternalClkSelect(IIClockSelect /*src*/) {  }
//    virtual IIClockSelect       ExternalClkSelect() const {  return cslFrontPanel;  }

    virtual void                Reference(IIReferenceSource /*src*/)  {  }
    virtual IIReferenceSource   Reference() const  {  return rsInternal;  }

    virtual void                ReferenceFrequency(double /*value*/)  {  }
    virtual double              ReferenceFrequency() const  {  return 0.0;  }

    virtual bool                Locked()  const = 0;
};

//==============================================================================
//  CLASS IXU_Type1_RawClockDevices -- Interface to XU Type1 "Raw" Devices
//==============================================================================

class IXU_Type1_RawClockDevices
{
public:
    virtual ~IXU_Type1_RawClockDevices()  {  }

    virtual Lmk04820 &   Pll() = 0;
    virtual Si570 &      Vcxo() = 0;
    virtual Lmk61E2 &    Xo() = 0;
    virtual ADF4355 &    TxPll() = 0;
};

//==============================================================================
//  CLASS IXU_Type2_RawClockDevices -- Interface to XU Type1 "Raw" Devices
//==============================================================================

class IXU_Type2_RawClockDevices
{
public:
    virtual ~IXU_Type2_RawClockDevices()  {  }

    virtual Lmk04820 &   Pll() = 0;
    virtual Si570 &      Vcxo() = 0;
    virtual Lmk61E2 &    Xo() = 0;
    virtual ADF4355 &    TxPll() = 0;
    virtual ADF4355 &    RxPll() = 0;
};

//
//  end Interfaces
//

//===========================================================================
//  CLASS XUAFE_Common -- XU AFE Common Registers "SubDevice"
//===========================================================================

class XUAFE_Common : public IX6SubDevice
{
public:

    enum AfeDirection { tInput, tOutput, tInputOutput };
    enum AfeExtSyncOptions { essFrontPanel, essCarrier, essB2b };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_Common::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfeTestControl_Addr;
        unsigned int   AfeSoftwareTrigger_Addr;
        unsigned int   AfeExtSyncSelect_Addr;
    };

    XUAFE_Common(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XUAFE_Common();

    //
    //  Methods
    void TestModeEnabled( AfeDirection which, bool state, unsigned int mode );
    void SoftwareTrigger( AfeDirection which, bool state );
    void AllOff( );

    AfeExtSyncOptions  ExternalSyncSelection() const;
    void               ExternalSyncSelection(AfeExtSyncOptions option);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_Common::TestControlRegister  -- TestControl Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TestControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit       AdcTestEnable;
        RegisterBitGroup  AdcTestMode;
        RegisterBit       DacTestEnable;
        RegisterBitGroup  DacTestMode;

    public:
        TestControlRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcTestEnable(*this, 0),   AdcTestMode(*this,  4, 8),
               DacTestEnable(*this, 16),  DacTestMode(*this, 20, 8)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_Common::SoftwareTriggerRegister  -- SW Trigger Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SoftwareTriggerRegister : public CachedShadowRegister
    {
        typedef CachedShadowRegister inherited;

    public:
        RegisterBit  AdcTrigger;
        RegisterBit  DacTrigger;

    public:
        SoftwareTriggerRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               AdcTrigger(*this, 0),   DacTrigger(*this, 16)
            {
            }
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_Common::ExtSyncSelectRegister  -- Select ext trigger source
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ExtSyncSelectRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  ExtSyncOption;

    public:
        ExtSyncSelectRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               ExtSyncOption(*this, 0, 2)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    TestControlRegister       TestControl;
    SoftwareTriggerRegister   SwTrigger;
    ExtSyncSelectRegister     ExtSyncSelect;

};

//===========================================================================
//  CLASS IUsesXUAlerts -- XU Alert User Interface
//===========================================================================

class IUsesXUAlerts
{
public:
    virtual ~IUsesXUAlerts()
        {  }

    enum AlertType { alertTimeStampRollover=0,  alertSoftware=1,
                     alertWarningTemperature=2, alertVFifoDiagnostic=3,
                     alertPatternMode=4,
                     alertTrigger=6, alertOutputUnderflow=7,
                     alertInputOverflow=8, alertInputOverrange=9,
                     alertDacChip=10
                     };  // enum code is alert bit #

    virtual void            AlertEnable(AlertType alert, bool state) = 0;
    virtual void            AlertClear(AlertType alert) = 0;
    virtual void            AlertTimestampEnable(bool state) = 0;
    virtual void            SoftwareAlert(unsigned int tag=0) = 0;
    virtual std::string     AlertInfoString(const Innovative::McAlert &alert, bool add_raw_data) = 0;
};

//===========================================================================
//  CLASS XUAlertSystem -- XU Alert "SubDevice"
//===========================================================================

class XUAlertSystem : public IX6SubDevice, public IUsesXUAlerts
{
public:
    //=======================================================================
    //  CLASS XUAlertSystem::MapRegisters  -- Device Register Offsets
    //=======================================================================
    class MapRegisters
    {
    public:
        unsigned int   AlertEnable_Addr;
        unsigned int   AlertControl_Addr;
        unsigned int   SoftwareAlert_Addr;
        unsigned int   AlertClear_Addr;
    };

    XUAlertSystem(IRequires_Nothing * mdc, IAddressingSpace & space, MapRegisters & regs);
    virtual ~XUAlertSystem();

    virtual void            AlertEnable(AlertType alert, bool state);
    virtual void            AlertClear(AlertType alert);
    virtual void            AlertTimestampEnable(bool state);
    virtual void            SoftwareAlert(unsigned int tag=0);
    virtual std::string     AlertInfoString(const Innovative::McAlert &alert, bool add_raw_data);

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS AlertControlRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AlertControlRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBit         TimestampOn;

    public:
        AlertControlRegister( IAddressingSpace &space, int offset )
            :  inherited( space, offset ),
               TimestampOn( *this, 0 )
            {   }
    };

private:
    IRequires_Nothing *   MDC;
    Register              AlertEnableReg;
    AlertControlRegister  AlertControlReg;
    Register              SoftwareAlertReg;
    Register              AlertClearReg;

    MemoryBitManipulator  AlertEnableCache;

//    void  InitAlerts();
//    virtual int  ExtractAlertArgument(const Buffer & alert, int bit, int field);

};

//===========================================================================
//  CLASS IXU_Chrono_Type1_Callbacks -- Callback Customization Interface
//===========================================================================
//  The Chrono needs to call back to the actual board for some functions

class IXU_Chrono_Type1_Callbacks
{
public:
    virtual void  OnChrono_SetFrequency(class XU_Chrono_Type1 * chrono, double freq) = 0;

    virtual ~IXU_Chrono_Type1_Callbacks()
        {}
};

//===========================================================================
//  CLASS IRequires_XU_Type1_Clocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the clocks/vcos needed for XU Type 1 clock system

class IRequires_XU_Type1_Clocks
{
public:
    virtual Lmk04820 *                    Make_AfePllObject() = 0;
    virtual IX6_I2C_uDevice *             Make_AfeVcxoI2CLink() = 0;
    virtual Si570 *                       Make_AfeVcxoObject(IRequires_I2C_RegIo & device) = 0;
    virtual IX6_I2C_uDevice *             Make_AfeXoI2CLink() = 0;
    virtual Lmk61E2 *                     Make_AfeXoObject(IRequires_I2C_RegIo & device) = 0;
    virtual ADF4355 *                     Make_AfeTxPllObject() = 0;
    virtual IXU_Chrono_Type1_Callbacks *  Make_Xu_Chrono_CB() = 0;
    virtual bool                          XU_Type1_ChannelEnabled(int ch) = 0;

    virtual ~IRequires_XU_Type1_Clocks()
        {}
};


//===========================================================================
//  CLASS XU_Chrono_Type1 -- XU AFE Clock "SubDevice"
//===========================================================================
//  PLL, VCO and devices required for Analog Clock Configuation
//  Type 1:  XU_TX

class XU_Chrono_Type1 : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_Chrono_Type1::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        //unsigned int   AfeExtClkSourceSelect_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_Chrono_Type1::ClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ClockIo : public IXUClockIo
    {
    public:
        ClockIo(XU_Chrono_Type1 * owner): Owner(owner)
            {  }
        virtual ~ClockIo()
            {  }
        //
        //  IXUClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        XU_Chrono_Type1 *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_Chrono_Type1::ChronoTBR - Chrono-wide TimebaseRate
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ChronoTBR : public ITimebaseRate
    {
    public:
        ChronoTBR( XU_Chrono_Type1 * chrono )
            :  Chrono(chrono)//,
            {
            }

        void    Frequency(double freq);
        double  Frequency() const;
        double  FrequencyActual() const;
        double  EffectiveFrequency() const;
        double  EffectiveFrequencyActual() const;
        double  MultipliedFrequency() const;
        double  MultipliedFrequencyActual() const;

    private:
        XU_Chrono_Type1 *  Chrono;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoLMK20_Clock::RawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class RawClockDevices : public IXU_Type1_RawClockDevices
    {
    public:
        RawClockDevices(XU_Chrono_Type1 * owner): Owner(owner)
            {  }
        virtual ~RawClockDevices()
            {  }

        virtual Lmk04820 &   Pll()      {  return Owner->Pll();  }
        virtual Si570 &      Vcxo()     {  return Owner->Vcxo();  }
        virtual Lmk61E2 &    Xo()       {  return Owner->Xo();  }
        virtual ADF4355 &    TxPll()    {  return Owner->TxPll();  }

    protected:
        XU_Chrono_Type1 *  Owner;
    };

    struct LmkClkRatesStruct
    {
        double       FpgaDacClk;
        double       MgtRefClk;
        double       SysRefClk;
        double       TxPllRefClk;
        unsigned int MasterClkIdx;
    };
    LmkClkRatesStruct LmkClkRates;

    friend class ClockIo;
    friend class RawClockDevices;
    friend class ChronoTBR;

    XU_Chrono_Type1(IRequires_XU_Type1_Clocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XU_Chrono_Type1();

    ClockIo &           Clock() {  return ClockIoIntf;  }
    RawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04820 &   Pll();
    Si570 &      Vcxo();
    Lmk61E2 &    Xo();
    ADF4355 &    TxPll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


private:
    //
    //  Registers
    IRequires_XU_Type1_Clocks *  MDC;

    Lmk04820 *                    PllDev;
    IX6_I2C_uDevice *             I2C_uDev;
    Si570 *                       VcxoDev;
    IX6_I2C_uDevice *             XoI2C_uDev;
    Lmk61E2 *                     XoDev;
    ADF4355 *                     TxPllDev;
    ClockIo                       ClockIoIntf;
    RawClockDevices               ClockDevs;
    ChronoTBR                     TimeRate;
    IXU_Chrono_Type1_Callbacks *  CallBack;
};

//===========================================================================
//  CLASS IXU_Chrono_Type2_Callbacks -- Callback Customization Interface
//===========================================================================
//  The Chrono needs to call back to the actual board for some functions

class IXU_Chrono_Type2_Callbacks
{
public:
    virtual void  OnChrono_SetFrequency(class XU_Chrono_Type2 * chrono, double freq) = 0;

    virtual ~IXU_Chrono_Type2_Callbacks()
        {}
};

//===========================================================================
//  CLASS IRequires_XU_Type2_Clocks -- X6 "SubDevice" Requirement Interface
//===========================================================================
//  This subdevice needs the clocks/vcos needed for XU Type 2 clock system

class IRequires_XU_Type2_Clocks
{
public:
    virtual Lmk04820 *                    Make_AfePllObject() = 0;
    virtual IX6_I2C_uDevice *             Make_AfeVcxoI2CLink() = 0;
    virtual Si570 *                       Make_AfeVcxoObject(IRequires_I2C_RegIo & device) = 0;
    virtual IX6_I2C_uDevice *             Make_AfeXoI2CLink() = 0;
    virtual Lmk61E2 *                     Make_AfeXoObject(IRequires_I2C_RegIo & device) = 0;
    virtual ADF4355 *                     Make_AfeTxPllObject() = 0;
    virtual ADF4355 *                     Make_AfeRxPllObject() = 0;
    virtual IXU_Chrono_Type2_Callbacks *  Make_Xu_Chrono_CB() = 0;
    virtual bool                          XU_Type2_ChannelEnabled(int ch) = 0;

    virtual ~IRequires_XU_Type2_Clocks()
        {}
};


//===========================================================================
//  CLASS XU_Chrono_Type2 -- XU AFE Clock "SubDevice"
//===========================================================================
//  PLL, VCO and devices required for Analog Clock Configuation
//  Type 2:  XU_RT

class XU_Chrono_Type2 : public IX6SubDevice
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_Chrono_Type2::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AfePllControl_Addr;
        unsigned int   AfePllSpi_Addr;
        //unsigned int   AfeExtClkSourceSelect_Addr;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_Chrono_Type2::ClockIo  -- Clock System User Interface
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ClockIo : public IXUClockIo
    {
    public:
        ClockIo(XU_Chrono_Type2 * owner): Owner(owner)
            {  }
        virtual ~ClockIo()
            {  }
        //
        //  IXUClockIo Interface Functions
        virtual double              FrequencyActual() const;

        virtual void                Frequency(double value);
        virtual double              Frequency() const;

        virtual ITimebaseRate &     Adc();
        virtual ITimebaseRate &     Dac();
        virtual ITimebaseRate &     System();

        virtual void                Source(IIClockSource src);
        virtual IIClockSource       Source() const;

        virtual void                Reference(IIReferenceSource src);
        virtual IIReferenceSource   Reference() const;

        virtual void                ReferenceFrequency(double value);
        virtual double              ReferenceFrequency() const;

        virtual bool                Locked() const;

    protected:
        XU_Chrono_Type2 *  Owner;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_Chrono_Type2::ChronoTBR - Chrono-wide TimebaseRate
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ChronoTBR : public ITimebaseRate
    {
    public:
        ChronoTBR( XU_Chrono_Type2 * chrono )
            :  Chrono(chrono)//,
            {
            }

        void    Frequency(double freq);
        double  Frequency() const;
        double  FrequencyActual() const;
        double  EffectiveFrequency() const;
        double  EffectiveFrequencyActual() const;
        double  MultipliedFrequency() const;
        double  MultipliedFrequencyActual() const;

    private:
        XU_Chrono_Type2 *  Chrono;
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SbcDuoLMK20_Clock::RawClockDevices  -- Interface to raw clock devices
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class RawClockDevices : public IXU_Type2_RawClockDevices
    {
    public:
        RawClockDevices(XU_Chrono_Type2 * owner): Owner(owner)
            {  }
        virtual ~RawClockDevices()
            {  }

        virtual Lmk04820 &   Pll()      {  return Owner->Pll();  }
        virtual Si570 &      Vcxo()     {  return Owner->Vcxo();  }
        virtual Lmk61E2 &    Xo()       {  return Owner->Xo();  }
        virtual ADF4355 &    TxPll()    {  return Owner->TxPll();  }
        virtual ADF4355 &    RxPll()    {  return Owner->RxPll();  }

    protected:
        XU_Chrono_Type2 *  Owner;
    };

    struct LmkClkRatesStruct
    {
        double       FpgaDacClk;
        double       MgtRefClk;
        double       SysRefClk;
        double       TxPllRefClk;
        unsigned int MasterClkIdx;
    };
    LmkClkRatesStruct LmkClkRates;

    friend class ClockIo;
    friend class RawClockDevices;
    friend class ChronoTBR;

    XU_Chrono_Type2(IRequires_XU_Type2_Clocks * mdc, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XU_Chrono_Type2();

    ClockIo &           Clock() {  return ClockIoIntf;  }
    RawClockDevices &   RawClockDevice() {  return ClockDevs;  }

    Lmk04820 &   Pll();
    Si570 &      Vcxo();
    Lmk61E2 &    Xo();
    ADF4355 &    TxPll();
    ADF4355 &    RxPll();
    //
    //  Methods

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    PreconfigureHardware();
    virtual void    ConfigureHardware();


private:
    //
    //  Registers
    IRequires_XU_Type2_Clocks *  MDC;

    Lmk04820 *                    PllDev;
    IX6_I2C_uDevice *             I2C_uDev;
    Si570 *                       VcxoDev;
    IX6_I2C_uDevice *             XoI2C_uDev;
    Lmk61E2 *                     XoDev;
    ADF4355 *                     TxPllDev;
    ADF4355 *                     RxPllDev;
    ClockIo                       ClockIoIntf;
    RawClockDevices               ClockDevs;
    ChronoTBR                     TimeRate;
    IXU_Chrono_Type2_Callbacks *  CallBack;
};

//===========================================================================
//  CLASS XU_VFifoInfo -- XU VFIFO Info Registers "SubDevice"
//===========================================================================

class XU_VFifoInfo : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_VFifoInfo::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   VFifoStatus_Addr;
        unsigned int   VFifoWordCount_Addr;
    };

    XU_VFifoInfo(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XU_VFifoInfo();

    //
    //  Methods
    bool AlmostEmpty( unsigned int vfifo_idx );
    bool AlmostFull( unsigned int vfifo_idx );
    unsigned int  WordCount( unsigned int vfifo_idx );

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XUAFE_Common::VFifoStatusRegister  -- TestControl Register
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class VFifoStatusRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  FifoSelect;
        RegisterBitGroup  FifoAlmostEmpty;
        RegisterBitGroup  FifoAlmostFull;

    public:
        VFifoStatusRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               FifoSelect(*this, 0, 2),
               FifoAlmostEmpty(*this, 16, 2),
               FifoAlmostFull(*this, 24, 2)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    VFifoStatusRegister  VFifoStatusReg;
    Register             VFifoWordCountReg;
};

//===========================================================================
//  CLASS XU_StreamTester -- XU Stream Checker/Generator "SubDevice"
//===========================================================================

class XU_StreamTester : public IX6SubDevice
{
public:

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_StreamTester::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   XuTestEnable_Addr;
        unsigned int   XuTestPacketSize_Addr;
        unsigned int   XuTestOutDecimation_Addr;
        unsigned int   XuTestChkErrors_Addr;
        unsigned int   XuTestSignalExtInt_Addr;
    };

    XU_StreamTester(IRequires_Nothing * owner, IAddressingSpace & space, MapRegisters & regs );
    virtual ~XU_StreamTester();

    //
    //  Methods
    void  TxCheckerEnable( unsigned int ch, bool state );
    void  RxTestGenEnable( unsigned int ch, bool state );
    void  RxPacketSize(unsigned int size_words);
    void  TxDecimation(unsigned int factor);
    bool  TxCheckerHasError(unsigned int ch);
    void  ExtInt();

    //  Internal Interface
    virtual void    ConstructCustomParts();
    virtual void    OpenHardware();
    virtual void    ConfigureHardware();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS XU_StreamTester::TestEnableRegister  --
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TestEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup  TxCheckerEnable;
        RegisterBitGroup  RxRampGenEnable;

    public:
        TestEnableRegister( IAddressingSpace &space, int offset )
            :  inherited(space, offset),
               TxCheckerEnable(*this, 0, 8),
               RxRampGenEnable(*this, 8, 8)
            {
            }
    };

private:
    //
    //  Registers
    IRequires_Nothing *  MDC;
    TestEnableRegister   TestEnableReg;
    Register             PacketSizeReg;
    Register             TxDecimationReg;
    Register             TxCheckErrorReg;
    Register             ExtIntTrigger;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif


}  // end namespace Innovative

#endif
