// XU_RTDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2017 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XU_RTDeviceMap_MbH
#define XU_RTDeviceMap_MbH

#include <XU_RT_Mb.h>
#include <JESDSubDevice_Stream_Mb.h>
#include <AD9162_Dac_SubDevice_Mb.h>
#include <AD9689_SubDevice_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class XU_RTDeviceMap;

//==============================================================================
//  CLASS XU_RT_Pll -- XU_RT PLL Chip (Specialization of Lmk04820)
//==============================================================================

class XU_RT_Pll : public Lmk04820
{
public:
    XU_RT_Pll(XU_RTDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~XU_RT_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XU_RTDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double /*dac_rate*/);

private:
    unsigned int FMasterOutIdx;
};

//=============================================================================
//  CLASS XU_RT_Xo  --  Programmable XO/VCXO Chip Support
//=============================================================================

class XU_RT_Xo : public Lmk61E2
{
public:
    XU_RT_Xo(IRequires_I2C_RegIo & device);

    virtual void    OnInit();         // derived class customization
    virtual void    OnReapply();

};

//=============================================================================
//  CLASS XU_RT_TxPll  --  PRT PLL timebase control
//=============================================================================

class XU_RT_TxPll : public ADF4355
{
public:
    //
    //  Member Functions
    XU_RT_TxPll(XU_RTDeviceMap * owner, IAddressingSpace &space, ADF4355Registers & regs);
    virtual ~XU_RT_TxPll();

    //void  EnableAllOutputs(bool enabled);

    // ...ADF4355 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 125e6;   }
    virtual unsigned int    DefaultChannelSpacingValue() const
                                {  return static_cast<unsigned int>(1e6);   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XU_RTDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//=============================================================================
//  CLASS XU_RT_RxPll  --  PRT PLL timebase control
//=============================================================================

class XU_RT_RxPll : public ADF4355
{
public:
    //
    //  Member Functions
    XU_RT_RxPll(XU_RTDeviceMap * owner, IAddressingSpace &space, ADF4355Registers & regs);
    virtual ~XU_RT_RxPll();

    //void  EnableAllOutputs(bool enabled);

    // ...ADF4355 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 125e6;   }
    virtual unsigned int    DefaultChannelSpacingValue() const
                                {  return static_cast<unsigned int>(1e6);   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XU_RTDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};

//===========================================================================
//  CLASS XU_RT_SysMonInterface -- Device Specific System Monitor Methods
//===========================================================================

class XU_RT_SysMonInterface : public SysMonInterface
{
public:
    enum XU_RT_pins {
           /* Dev 0 */  pin_Io0, pin_Io1, pin_Io2, pin_Io3, pin_Io4, pin_Io5, pin_V3p3,
           /* Dev 1 */  pin_Vp1p5_cnfg, pin_Vp1p8_cnfg, pin_Vp5p0, pin_Vp1p3_dac, pin_Vp1p2, pin_Vp1p0, pin_VPwr,
           /* Dev 2 */  pin_T_U78, pin_T_U77, pin_T_U76, pin_T_U75, pin_T_U74, pin_T_U73, pin_Vp1p8,
                        TotalPins
                     };

    XU_RT_SysMonInterface(XU_RTDeviceMap * owner);
    ~XU_RT_SysMonInterface()
        {}

    // access methods
    XUSystemMonitor * Raw()  {  return Device;  }

    void  Pin_Init();
    void  CustomInit();
    void  OpenHardware();

    void    Init_Trig(double trig_plus_volts, double trig_minus_volts)
        {  CommonFactorTripleDacUpdate(pin_Io5, pin_Io3, pin_Io2, 0, trig_plus_volts, trig_minus_volts);  }
    void    Init_RT(double RT0_volts, double RT1_volts)
        {  CommonFactorTripleDacUpdate(pin_Io4, pin_Io0, pin_Io1, 1, RT0_volts, RT1_volts);  }


    void    WriteDac(XU_RT_pins pin_code, double voltage)
                {  Write_a_Dac(static_cast<unsigned int>(pin_code), voltage);  }
    double  ReadAdc(XU_RT_pins pin_code)
                {  return Read_an_Adc(static_cast<unsigned int>(pin_code));  }
    double  ReadTemperature(XU_RT_pins pin_code);

    void    Adjust_TrigPlus(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io5, pin_Io3, 0, adjust_volts);  }
    void    Adjust_TrigMinus(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io5, pin_Io2, 0, adjust_volts);  }
    void    Adjust_RT0(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io4, pin_Io0, 1, adjust_volts);  }
    void    Adjust_RT1(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io4, pin_Io1, 1, adjust_volts);  }


};

//=============================================================================
//  CLASS XU_RT_SioPll  --  Programmable XO/VCXO Chip Support
//=============================================================================

class XU_RT_SioPll : public Lmk61E2
{
public:
    XU_RT_SioPll(IRequires_I2C_RegIo & device);

    virtual void    OnInit();         // derived class customization
    virtual void    OnReapply();

};

//===========================================================================
//  CLASS ISuppliesXU_RTRequirements -- Vpx Device Map Requirement List
//===========================================================================

class ISuppliesXU_RTRequirements : public IRequires_Nothing,
                                   public IRequires_XU_Type2_Clocks,
                                   public IRequires_JESDPHY_Io,
                                   public IRequires_JESDADC_Io,
                                   public IRequires_JESDDAC_Io,
                                   public IRequires_AD9689_ADC_Io,
                                   public IRequires_AD9162_Dac_Io
{
public:
    virtual ~ISuppliesXU_RTRequirements()
        {}

};

//===========================================================================
//  CLASS XU_RTDeviceMap  -- Register & low level support for XU_RT module
//===========================================================================
class XU_RT;

class XU_RTDeviceMap :  public XUBidirDeviceMap, public ISuppliesXU_RTRequirements

{
    friend class XU_RT;
    friend class XU_RTInputDevice;
    friend class XU_RTOutputDevice;
    friend class XU_RT_Pll;
    friend class XU_RT_SysMonInterface;
    friend class XU_RT_DacCalOverride;

public:
    //
    //  Member Functions
    XU_RTDeviceMap(XU_RT * owner);
    virtual ~XU_RTDeviceMap();

    XU_Chrono_Type2::ClockIo & Clock()  { return Clock_SDev.Clock(); }

    double MgtRefActual()               { return Clock_SDev.RawClockDevice().Pll().OutputFrequencyActual(0); }

    class XU_RTInputDevice &   Input();
    class XU_RTOutputDevice &  Output();

    JESD_PHY &        JesdPhy(unsigned int dev) {  return (dev==0) ? JESD_PHY0_SDev : JESD_PHY1_SDev;  }
    AD9162_DAC &      Dac(unsigned int dev=0)     {  return (dev==0) ? DAC0_SDev : DAC0_SDev;    }
    AD9689_ADC &      Adc(unsigned int dev=0)     {  return (dev==0) ? ADC_SDev : ADC_SDev;    }

    // are these valid for XU_RT
//    void TestGeneratorRun(bool state);
//    void TestGeneratorReset();
//    void TestGeneratorEnable(unsigned int enable_mask, unsigned int chirp_size);

    void SetLmkClkRates(double sampling_rate, XU_Chrono_Type2::LmkClkRatesStruct & rates);

public: // pseudo-protected

    virtual double   Get_DacClockRate()  { return Clock().Dac().Frequency(); }

protected:
    // Data
    XU_RT *         Owner;

    //
    //  Subdevices
    XU_Chrono_Type2        Clock_SDev;
    JESD_PHY               JESD_PHY0_SDev;       // JESD PHY IP
    JESD_PHY               JESD_PHY1_SDev;       // JESD PHY IP
    JESD_ADCDevice         JESD_ADC_SDev;
    JESD_DACDevice         JESD_DAC0_SDev;       //
    //JESD_DACDevice         JESD_DAC1_SDev;       //
    AD9689_ADC             ADC_SDev;             // ADC Chip Management
    AD9162_DAC             DAC0_SDev;            // DAC Chip Management
    //AD9162_DAC             DAC1_SDev;            // DAC Chip Management

    XU_RTInputDevice *     FInput;
    XU_RTOutputDevice *    FOutput;

    XU_RT_SysMonInterface  SysMonInterface;


    Trace             Debug;

    // Methods
    virtual void    ConstructCustomParts();            // board construtor time
    virtual void    PreOpenHardware();                 // before open time
    virtual void    OpenHardware();                    // after open time
    virtual void    PreconfigureHardware();            // preconfigure time
    virtual void    OnSingleStreamAtStartConfig();     // runs at first stream start (only) after PreconfigureHardware()
    virtual void    ConfigureHardware();               // stream configuration
    virtual void    OnBeforeStreamStart();             // before stream start customization
    virtual void    OnAfterStreamStart();              // after stream start customization
    virtual void    OnBeforeStreamStop();              // before stream stop customization
    virtual void    StreamStopHardware();              // after stream stop subdevice configuration

    virtual double  StandardClockFrequency()
                        {  return Clock().Frequency();   }

    //  Subdevice Customization Handlers
    //  Type2 Chrono
    virtual Lmk04820 *                    Make_AfePllObject();
    virtual IX6_I2C_uDevice *             Make_AfeVcxoI2CLink();
    virtual Si570 *                       Make_AfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *             Make_AfeXoI2CLink();
    virtual Lmk61E2 *                     Make_AfeXoObject(IRequires_I2C_RegIo & device);
    virtual ADF4355 *                     Make_AfeTxPllObject();
    virtual ADF4355 *                     Make_AfeRxPllObject();
    virtual IXU_Chrono_Type2_Callbacks *  Make_Xu_Chrono_CB();
    virtual bool                          XU_Type2_ChannelEnabled(int ch);

    //  XU Dac Device Mixin
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        SetXuStreamChannelActive(FrameDeviceCode which, int ch, bool state);
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    virtual unsigned int                GetAdcSpiDeviceCount();
    virtual unsigned int                GetCalibrationAdcCount();
    virtual ICalibrationValueOverride * GetAdcCalibrationOverride();


    // JESD FPGA MGT
    virtual void                        PokeJESDPHY(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDPHY(unsigned int device, unsigned int address);
    virtual IJESD_IoDevice *            GetAdcJesd_IoDevice(unsigned int device);
    virtual IJESD_IoDevice *            GetDacJesd_IoDevice(unsigned int device);
    virtual IJESD_PllRates *            GetJesd_PllRatesDevice(unsigned int device);

    // JESD ADC
    virtual void                        PokeJESDADC(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDADC(unsigned int device, unsigned int address);

    // JESD DAC
    virtual void                        PokeJESDDAC(unsigned int device_id, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDDAC(unsigned int device_id, unsigned int address);

    // AD9689 ADC
    virtual void                        SetAdcLaneConfiguration(unsigned int device);
    virtual void                        AdcSpi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                AdcSpi(unsigned int device, unsigned int address);
    JESD_ADCDevice &                    JesdAdc(unsigned int /*device*/) {  return JESD_ADC_SDev;  }
    virtual bool                        AdcChannelEnabled(int ch);
    virtual void                        AdcChannelEnabled(int ch, bool state);
    virtual unsigned int                AdcActiveChannels();

    // ADC latency calibration
//    virtual DS90CP04Switch &            Sync_XPS() {  return SyncXPS;  }
//    virtual void                        PulsorModeEnter();
//    virtual void                        PulsorModeFire(unsigned int Log2PulseCnt);
//    virtual void                        PulsorModeExit();

    // AD9162_DAC
    virtual void                        SetDacLaneConfiguration(unsigned int device);
    virtual void                        DacSpi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                DacSpi(unsigned int device, unsigned int address);
    virtual bool                        DacChannelEnabled(int ch);
    virtual void                        DacChannelEnabled(int ch, bool state);
    JESD_DACDevice &                    JesdDac(unsigned int dev) {  return (dev==0) ? JESD_DAC0_SDev : JESD_DAC0_SDev;  }

    //  Pattern Mode
    virtual unsigned int                GetPatternModeScaleFactor();
    virtual unsigned int                GetPatternModeAlignment();

    //  System Monitor
    virtual void                        SetSysmonDeviceConfiguration();

    //  Aurora Sio Pll
    virtual IX6_I2C_uDevice *           MakeXuSioI2CLink();
    virtual Lmk61E2 *                   MakeXuSioPllObject(IRequires_I2C_RegIo & device);

private:
    void     ConfigureDacs();
};

//===========================================================================
//  CLASS XU_RTInputDevice  -- XU_RT Input Devices
//===========================================================================

class XU_RTInputDevice : public XUInputDevice
{
public:
    typedef XUInputDevice inherited;

    XU_RTInputDevice(XU_RTDeviceMap* owner, IX6AnalogIoInfo * info );
    virtual ~XU_RTInputDevice();

    virtual IX6AnalogIoInfo &  Info();

    // Pseudo-protected
    void  OpenHardware();
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

private:
    XU_RTDeviceMap *            OwnerRT;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     AdcCalIdrom()  {  return OwnerRT->AdcCal_PG;  }

};

//===========================================================================
//  CLASS XU_RTOutputDevice  -- XU_RT Output Devices
//===========================================================================

class XU_RTOutputDevice : public XUOutputDevice
{
public:
    typedef XUOutputDevice inherited;

    XU_RTOutputDevice(XU_RTDeviceMap* owner, IX6AnalogIoInfo * info );
    virtual ~XU_RTOutputDevice();

    virtual IX6AnalogIoInfo &  Info();

//    void  DacReset(bool state)
//        {
//        DacCommon().DacReset(state);
//        }

    // Pseudo-protected
    void  OpenHardware();
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

private:
    XU_RTDeviceMap *            OwnerRT;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     DacCalIdrom()  {  return OwnerRT->DacCal_PG;  }

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
