// XU_TXDeviceMap_Mb.h
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//


#ifndef XU_TXDeviceMap_MbH
#define XU_TXDeviceMap_MbH

#include <XU_TX_Mb.h>
#include <JESDSubDevice_Stream_Mb.h>
#include <AD9162_Dac_SubDevice_Mb.h>
#include <Trace_Mb.h>

namespace Innovative
{

#ifdef __CLR_VER
#pragma managed(push, off)
#endif

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

class XU_TXDeviceMap;

//==============================================================================
//  CLASS XU_TX_Pll -- XU_TX PLL Chip (Specialization of Lmk04820)
//==============================================================================

class XU_TX_Pll : public Lmk04820
{
public:
    XU_TX_Pll(XU_TXDeviceMap * owner, IAddressingSpace &space, Lmk04820Registers & reg);
    virtual ~XU_TX_Pll();

    // ...Lmk04820 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 100e6;   }
    virtual double          DefaultFrontEndPhaseDetectorFrequency() const
                                {  return 20e6;   }
    virtual double          DefaultFrontEndVcxoFrequency() const
                                {  return 100e6;   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XU_TXDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double /*dac_rate*/);

private:
    unsigned int FMasterOutIdx;
};

//=============================================================================
//  CLASS XU_TX_Xo  --  Programmable XO/VCXO Chip Support
//=============================================================================

class XU_TX_Xo : public Lmk61E2
{
public:
    XU_TX_Xo(IRequires_I2C_RegIo & device);

    virtual void    OnInit();         // derived class customization
    virtual void    OnReapply();

};

//=============================================================================
//  CLASS XU_TX_TxPll  --  PTX PLL timebase control
//=============================================================================

class XU_TX_TxPll : public ADF4355
{
public:
    //
    //  Member Functions
    XU_TX_TxPll(XU_TXDeviceMap * owner, IAddressingSpace &space, ADF4355Registers & regs);
    virtual ~XU_TX_TxPll();

    //void  EnableAllOutputs(bool enabled);

    // ...ADF4355 interface impl
    virtual void            OnInit();
    virtual void            OnReapply();

    virtual double          DefaultPhaseDetectorFrequency() const
                                {  return 125e6;   }
    virtual unsigned int    DefaultChannelSpacingValue() const
                                {  return static_cast<unsigned int>(1e6);   }

    virtual bool IsFeedbackOutputAdc();

protected:
    XU_TXDeviceMap *   Owner;

    virtual unsigned int AdcMasterOutput();
    virtual unsigned int DacMasterOutput();

    virtual void  SetAdcOutputRates(double adc_rate);
    virtual void  SetDacOutputRates(double dac_rate);

};


//===========================================================================
//  CLASS XU_TX_SysMonInterface -- Device Specific System Monitor Methods
//===========================================================================

class XU_TX_SysMonInterface : public SysMonInterface
{
public:
    enum XU_TX_pins {
           /* Dev 0 */  pin_Io0, pin_Io1, pin_Io2, pin_Io3, pin_Io4, pin_Io5, pin_V3p3,
           /* Dev 1 */  pin_Vp1p5_cnfg, pin_Vp1p8_cnfg, pin_Vp5p0, pin_Vp1p3_dac, pin_Vp1p2, pin_Vp1p0, pin_VPwr,
           /* Dev 2 */  pin_T_U78, pin_T_U77, pin_T_U76, pin_T_U75, pin_T_U74, pin_T_U73, pin_Vp1p8,
                        TotalPins
                     };

    XU_TX_SysMonInterface(XU_TXDeviceMap * owner);
    ~XU_TX_SysMonInterface()
        {}

    // access methods
    XUSystemMonitor * Raw()  {  return Device;  }

    void  Pin_Init();
    void  CustomInit();
    void  OpenHardware();

    void    Init_Trig(double trig_plus_volts, double trig_minus_volts)
        {  CommonFactorTripleDacUpdate(pin_Io5, pin_Io3, pin_Io2, 0, trig_plus_volts, trig_minus_volts);  }
    void    Init_Tx(double tx0_volts, double tx1_volts)
        {  CommonFactorTripleDacUpdate(pin_Io4, pin_Io0, pin_Io1, 1, tx0_volts, tx1_volts);  }


    void    WriteDac(XU_TX_pins pin_code, double voltage)
                {  Write_a_Dac(static_cast<unsigned int>(pin_code), voltage);  }
    double  ReadAdc(XU_TX_pins pin_code)
                {  return Read_an_Adc(static_cast<unsigned int>(pin_code));  }
    double  ReadTemperature(XU_TX_pins pin_code);

    void    Adjust_TrigPlus(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io5, pin_Io3, 0, adjust_volts);  }
    void    Adjust_TrigMinus(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io5, pin_Io2, 0, adjust_volts);  }
    void    Adjust_Tx0(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io4, pin_Io0, 1, adjust_volts);  }
    void    Adjust_Tx1(double adjust_volts)
                {  CommonFactorDualDacUpdate(pin_Io4, pin_Io1, 1, adjust_volts);  }


};

//=============================================================================
//  CLASS XU_TX_SioPll  --  Programmable XO/VCXO Chip Support
//=============================================================================

class XU_TX_SioPll : public Lmk61E2
{
public:
    XU_TX_SioPll(IRequires_I2C_RegIo & device);

    virtual void    OnInit();         // derived class customization
    virtual void    OnReapply();

};

//===========================================================================
//  CLASS ISuppliesXU_TXRequirements -- Vpx Device Map Requirement List
//===========================================================================

class ISuppliesXU_TXRequirements : public IRequires_Nothing,
                                   public IRequires_XU_Type1_Clocks,
                                   public IRequires_JESDPHY_Io,
                                   //public IRequires_JESDADC_Io,
                                   public IRequires_JESDDAC_Io,
                                   public IRequires_AD9162_Dac_Io
{
public:
    virtual ~ISuppliesXU_TXRequirements()
        {}

};

//===========================================================================
//  CLASS XU_TXDeviceMap  -- Register & low level support for XU_TX module
//===========================================================================
class XU_TX;

class XU_TXDeviceMap :  public XUDacDeviceMap, public ISuppliesXU_TXRequirements

{
    friend class XU_TX;
    friend class XU_TXOutputDevice;
    friend class XU_TX_Pll;
    friend class XU_TX_SysMonInterface;
    friend class XU_TX_DacCalOverride;

public:
    //
    //  Member Functions
    XU_TXDeviceMap(XU_TX * owner);
    virtual ~XU_TXDeviceMap();

    XU_Chrono_Type1::ClockIo & Clock()  { return Clock_SDev.Clock(); }

    double MgtRefActual()               { return Clock_SDev.RawClockDevice().Pll().OutputFrequencyActual(0); }
    class XU_TXOutputDevice &  Output();

    JESD_PHY &        JesdPhy(unsigned int dev) {  return (dev==0) ? JESD_PHY0_SDev : JESD_PHY1_SDev;  }
    AD9162_DAC &      Dac(unsigned int dev)     {  return (dev==0) ? DAC0_SDev : DAC1_SDev;    }

    // are these valid for XU_TX
//    void TestGeneratorRun(bool state);
//    void TestGeneratorReset();
//    void TestGeneratorEnable(unsigned int enable_mask, unsigned int chirp_size);

    void SetLmkClkRates(double sampling_rate, XU_Chrono_Type1::LmkClkRatesStruct & rates);

public: // pseudo-protected

    virtual double   Get_DacClockRate()  { return Clock().Dac().Frequency(); }

protected:
    // Data
    XU_TX *         Owner;

    //
    //  Subdevices
    XU_Chrono_Type1        Clock_SDev;
    JESD_PHY               JESD_PHY0_SDev;       // JESD PHY IP
    JESD_PHY               JESD_PHY1_SDev;       // JESD PHY IP
    JESD_DACDevice         JESD_DAC0_SDev;       //
    JESD_DACDevice         JESD_DAC1_SDev;       //
    AD9162_DAC             DAC0_SDev;            // DAC Chip Management
    AD9162_DAC             DAC1_SDev;            // DAC Chip Management

    XU_TXOutputDevice *    FOutput;

    XU_TX_SysMonInterface  SysMonInterface;


    Trace             Debug;

    // Methods
    virtual void    ConstructCustomParts();            // board construtor time
    virtual void    PreOpenHardware();                 // before open time
    virtual void    OpenHardware();                    // after open time
    virtual void    PreconfigureHardware();            // preconfigure time
    virtual void    OnSingleStreamAtStartConfig();     // runs at first stream start (only) after PreconfigureHardware()
    virtual void    ConfigureHardware();               // stream configuration
    virtual void    OnBeforeStreamStart();             // before stream start customization
    virtual void    OnAfterStreamStart();              // after stream start customization
    virtual void    OnBeforeStreamStop();              // before stream stop customization
    virtual void    StreamStopHardware();              // after stream stop subdevice configuration

    virtual double  StandardClockFrequency()
                        {  return Clock().Frequency();   }

    //  Subdevice Customization Handlers
    //  Type1 Chrono
    virtual Lmk04820 *                    Make_AfePllObject();
    virtual IX6_I2C_uDevice *             Make_AfeVcxoI2CLink();
    virtual Si570 *                       Make_AfeVcxoObject(IRequires_I2C_RegIo & device);
    virtual IX6_I2C_uDevice *             Make_AfeXoI2CLink();
    virtual Lmk61E2 *                     Make_AfeXoObject(IRequires_I2C_RegIo & device);
    virtual ADF4355 *                     Make_AfeTxPllObject();
    virtual IXU_Chrono_Type1_Callbacks *  Make_Xu_Chrono_CB();
    virtual bool                          XU_Type1_ChannelEnabled(int ch);

    //  XU Dac Device Mixin
    virtual class IPriClockDevice *     MakePriClockDevice(PriDeviceCode which);
    virtual IFrameCalcDevice *          MakeFrameCalcDevice(FrameDeviceCode which);
    virtual void                        GetAnalogDeviceInfo(FrameDeviceCode, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        GetAnalogPhysicalDeviceInfo(FrameDeviceCode, unsigned int& devices, unsigned int& ch_per_dev);
    virtual void                        SetXuStreamChannelActive(FrameDeviceCode which, int ch, bool state);
    virtual unsigned int                GetDacSpiDeviceCount();
    virtual unsigned int                GetCalibrationDacCount();
    virtual ICalibrationValueOverride * GetDacCalibrationOverride();

    // JESD FPGA MGT
    virtual void                        PokeJESDPHY(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDPHY(unsigned int device, unsigned int address);
    virtual IJESD_IoDevice *            GetAdcJesd_IoDevice(unsigned int device);
    virtual IJESD_IoDevice *            GetDacJesd_IoDevice(unsigned int device);
    virtual IJESD_PllRates *            GetJesd_PllRatesDevice(unsigned int device);

    // JESD DAC
    virtual void                        PokeJESDDAC(unsigned int device_id, unsigned int address, unsigned int data);
    virtual unsigned int                PeekJESDDAC(unsigned int device_id, unsigned int address);

    // AD9162_DAC
    virtual void                        SetDacLaneConfiguration(unsigned int device);
    virtual void                        DacSpi(unsigned int device, unsigned int address, unsigned int data);
    virtual unsigned int                DacSpi(unsigned int device, unsigned int address);
    virtual bool                        DacChannelEnabled(int ch);
    virtual void                        DacChannelEnabled(int ch, bool state);
    JESD_DACDevice &                    JesdDac(unsigned int dev) {  return (dev==0) ? JESD_DAC0_SDev : JESD_DAC1_SDev;  }

    //  Pattern Mode
    virtual unsigned int                GetPatternModeScaleFactor();
    virtual unsigned int                GetPatternModeAlignment();

    //  System Monitor
    virtual void                        SetSysmonDeviceConfiguration();

    //  Aurora Sio Pll
    virtual IX6_I2C_uDevice *           MakeXuSioI2CLink();
    virtual Lmk61E2 *                   MakeXuSioPllObject(IRequires_I2C_RegIo & device);

private:
    void     ConfigureDacs();
};

//===========================================================================
//  CLASS XU_TXOutputDevice  -- XU_TX Output Devices
//===========================================================================

class XU_TXOutputDevice : public XUOutputDevice
{
public:
    typedef XUOutputDevice inherited;

    XU_TXOutputDevice(XU_TXDeviceMap* owner, IX6AnalogIoInfo * info );
    virtual ~XU_TXOutputDevice();

    virtual IX6AnalogIoInfo &  Info();

//    void  DacReset(bool state)
//        {
//        DacCommon().DacReset(state);
//        }

    void   OpenHardware();

    // Pseudo-protected
    void  PreconfigureHardware();
    void  ConfigureHardware();

    virtual X6CalibrationSection & Cal() {  return CalRomSection;  }

private:
    XU_TXDeviceMap *            OwnerTX;
    IX6AnalogIoInfo *           FInfo;
    X6CalibrationSection        CalRomSection;

    X6IdromPage &     DacCalIdrom()  {  return OwnerTX->DacCal_PG;  }

};


#ifdef _MSC_VER
    #pragma warning(pop)
#endif

#ifdef __CLR_VER
#pragma managed(pop)
#endif
} // Innovative
//---------------------------------------------------------------------------
#endif
