//---------------------------------------------------------------------------
// XU_TX_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2016 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <XU_TX_Mb.h>

namespace Innovative
{

#ifdef _MSC_VER
    #pragma warning(push)
    #pragma warning(disable : 4355)
#endif

//===========================================================================
//  CLASS XU_TX  -- XU_TX baseboard object
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class XU_TX
//---------------------------------------------------------------------------

XU_TX::XU_TX()
    :  XuBoard(Device), Device(this)
{
}

//---------------------------------------------------------------------------
//  destructor for class XU_TX
//---------------------------------------------------------------------------

XU_TX::~XU_TX()
{
    Close();
}

//---------------------------------------------------------------------------
//  XU_TX::Output() --  Return reference to Output object
//---------------------------------------------------------------------------

XU_TXOutputDevice & XU_TX::Output()
{
    return Device.Output();
}

//---------------------------------------------------------------------------
//  XU_TX::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

Innovative::XU_Chrono_Type1::ClockIo &  XU_TX::Clock()
{
    return Device.Clock();
}

//---------------------------------------------------------------------------
//  XU_TX::RawClockDevice() --  Return reference to Output object
//---------------------------------------------------------------------------

XU_Chrono_Type1::RawClockDevices & XU_TX::RawClockDevice()
{
    return Device.Clock_SDev.RawClockDevice();
}

//---------------------------------------------------------------------------
//  XU_TX::JesdPhy() --  Access JESD PHY devices
//---------------------------------------------------------------------------

JESD_PHY & XU_TX::JesdPhy(unsigned int dev)
{
    return Device.JesdPhy(dev);
}

//---------------------------------------------------------------------------
//  XU_TX::Dac() --  Access JESD PHY devices
//---------------------------------------------------------------------------

AD9162_DAC & XU_TX::Dac(unsigned int dev)
{
    return Device.Dac(dev);
}


//---------------------------------------------------------------------------
//  XU_TX::SysMon() --  XU_TX Specific SysMon interface
//---------------------------------------------------------------------------

XU_TX_SysMonInterface & XU_TX::SysMon()
{
    return Device.SysMonInterface;
}


//---------------------------------------------------------------------------
//  XU_TX::SetTriggerThreshold() --  Set hardware trigger threshold level
//---------------------------------------------------------------------------

void XU_TX::SetTriggerThreshold(double volts)
{
    SysMon().Adjust_TrigMinus(volts);
}

//---------------------------------------------------------------------------
//  XU_TX::AfterFiclStart() --
//---------------------------------------------------------------------------

void  XU_TX::AfterFiclStart(FiclSystem & engine)
{
    engine.Evaluate("  0 constant Pll   1 constant Vcxo  2 constant Xo  3 constant TxPll "
                    "  4 constant Dac0Spi     5 constant Dac1Spi");
    engine.Evaluate("  6 constant Dac0Jesd    7 constant Dac1Jesd");
    engine.Evaluate("  8 constant Dac0JesdPhy 9 constant Dac1JesdPhy");
    engine.Evaluate("  10 constant SioPll  ");

    engine.Evaluate(": ?XU_TX "
                    "    cr .\" --- SubDevice Constants --- \" "
                    "    cr .\"   Pll   Vcxo   Xo   TxPll  Dac0Spi  Dac1Spi\" "
                    "    cr .\"   Dac0Jesd   Dac1Jesd   Dac0JesdPhy   Dac1JesdPhy\" "
                    "    cr .\"   SioPll\" "
                    "    cr  "
                    "    cr .\" ---- XU_TX Definitions ---- \" "
                    "    cr .\"   l@  ( addr -- data )  fetch from logic space  \" "
                    "    cr .\"   l!  ( data addr  -- ) : store to logic space   \" "
                    "    cr .\"   p@  ( addr -- data ) : fetch from pci space  \" "
                    "    cr .\"   p!  ( data addr -- ) : store to pci space   \" "
                    "    cr .\"   d@  ( addr dev -- data ) : fetch from sub-device space \" "
                    "    cr .\"   d!  ( data addr dev -- ) : store to sub-device space \" "
                    "    cr .\"   dFreq  ( f:freq dev -- ) : Set device frequency \" "
                    "    cr .\"   dFreqActual  ( dev -- f:freq) : read actual frequency \" "
                    "    cr .\"   ?XU_TX  : display this message \" "
                    "    cr  ; ");
    engine.Evaluate("marker app");

    engine.Display("XU_TX-specific wordset loaded \n" );
    //
    //  Call base class FICL init
    MultiChanBoard::AfterFiclStart(engine);
}

//---------------------------------------------------------------------------
//  XU_TX::Do_DFetch() --  default implementation
//---------------------------------------------------------------------------

int  XU_TX::Do_DFetch(int dev, int addr)
{
    const int kPll = 0;        const int kVcxo = 1;
    const int kXo = 2;         const int kTxPll = 3;
    const int kDac0Spi = 4;    const int kDac1Spi = 5;
    const int kDac0Jesd = 6;   const int kDac1Jesd = 7;
    const int Dac0JesdPhy = 8; const int Dac1JesdPhy = 9;
    const int kSioPll = 10;

    switch (dev)
        {
        default:
            return 0;
        case kPll:
            return RawClockDevice().Pll().Port(addr);
        case kVcxo:
            return RawClockDevice().Vcxo().Port(addr);
        case kXo:
            return RawClockDevice().Xo().Port(addr);
        case kTxPll:
            return RawClockDevice().TxPll().Port(addr);
        case kDac0Spi:
            return Device.Output().Spi(0, addr);
        case kDac1Spi:
            return Device.Output().Spi(1, addr);
        case kDac0Jesd :
            return static_cast<int>(Device.PeekJESDDAC(0, addr));
        case kDac1Jesd :
            return static_cast<int>(Device.PeekJESDDAC(1, addr));
        case Dac0JesdPhy :
            return static_cast<int>(Device.PeekJESDPHY(0, addr));
        case Dac1JesdPhy :
            return static_cast<int>(Device.PeekJESDPHY(1, addr));
        case kSioPll:
            return SioPll().Pll().Port(addr);
    }
}

//---------------------------------------------------------------------------
//  XU_TX::Do_DStore() --  default implementation
//---------------------------------------------------------------------------

void  XU_TX::Do_DStore(int dev, int addr, int value)
{
    const int kPll = 0;        const int kVcxo = 1;
    const int kXo = 2;         const int kTxPll = 3;
    const int kDac0Spi = 4;    const int kDac1Spi = 5;
    const int kDac0Jesd = 6;   const int kDac1Jesd = 7;
    const int Dac0JesdPhy = 8; const int Dac1JesdPhy = 9;
    const int kSioPll = 10;

    switch (dev)
        {
        default:
            break;
        case kPll:
            RawClockDevice().Pll().Port(addr, value);
            break;
        case kVcxo:
            RawClockDevice().Vcxo().Port(addr, value);
            break;
        case kXo:
            RawClockDevice().Xo().Port(addr, value);
            break;
        case kTxPll:
            RawClockDevice().TxPll().Port(addr, value);
            break;
        case kDac0Spi:
            Device.Output().Spi(0, addr, value);
            break;
        case kDac1Spi:
            Device.Output().Spi(1, addr, value);
            break;
        case kDac0Jesd :
            Device.PokeJESDDAC(0, addr, value);
            break;
        case kDac1Jesd :
            Device.PokeJESDDAC(1, addr, value);
            break;
        case Dac0JesdPhy :
            Device.PokeJESDPHY(0, addr, value);
            break;
        case Dac1JesdPhy :
            Device.PokeJESDPHY(1, addr, value);
            break;
        case kSioPll:
            return SioPll().Pll().Port(addr, value);
    }
}

//---------------------------------------------------------------------------
//  XU_TX::Do_DFreq() --  default implementation
//---------------------------------------------------------------------------

void  XU_TX::Do_DFreq(int dev, float freq)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kXo = 2;  const int kTxPll = 3;
    const int kSioPll = 10;

    switch (dev)
        {
        default:
            break;
        case kPll:
            RawClockDevice().Pll().Frequency(freq);
            RawClockDevice().Pll().Reapply();
            break;
        case kVcxo:
            RawClockDevice().Vcxo().Frequency(freq);
            break;
        case kXo:
            RawClockDevice().Xo().Frequency(freq);
            RawClockDevice().Xo().Reapply();
            break;
        case kTxPll:
            RawClockDevice().TxPll().Frequency(freq);
            break;
        case kSioPll:
            SioPll().Pll().Frequency(freq);
            SioPll().Pll().Reapply();
            break;
        }
}

//---------------------------------------------------------------------------
//  XU_TX::Do_DFreqActual() --  default implementation
//---------------------------------------------------------------------------

float  XU_TX::Do_DFreqActual(int dev)
{
    const int kPll = 0;   const int kVcxo = 1;  const int kXo = 2;  const int kTxPll = 3;
    const int kSioPll = 10;
    switch (dev)
        {
        default:
            return 0.0f;
        case kPll:
            return static_cast<float>(RawClockDevice().Pll().FrequencyActual());
        case kVcxo:
            return static_cast<float>(RawClockDevice().Vcxo().FrequencyActual());
        case kXo:
            return static_cast<float>(RawClockDevice().Xo().FrequencyActual());
        case kTxPll:
            return static_cast<float>(RawClockDevice().TxPll().FrequencyActual());
        case kSioPll:
            return static_cast<float>(SioPll().Pll().FrequencyActual());
        }
}


#ifdef _MSC_VER
    #pragma warning(pop)
#endif


}  // namespace Innovative
