/*******************************************************/
/* file: Ports_Mb.cpp                                  */
/* abstract:  This file contains the routines to       */
/*            output values on the JTAG ports, to read */
/*            the TDO bit, and to read a byte of data  */
/*            from the prom                            */
/*                                                     */
/*******************************************************/
#include <Ports_Mb.h>
#include <PmcJtagPort_Mb.h>
#include <StopWatch_Mb.h>
#include <SingletonManager_Mb.h>
#include <SystemSupport_Mb.h>
#include <StopWatch_Mb.h>
#include <cstdlib>

namespace
{

//==============================================================================
//  CLASS StopWatchSingleton --
//==============================================================================

class StopWatchSingleton : public OpenWire::SingletonService
{
public:
    Innovative::StopWatch Rtc;

private:
    static StopWatchSingleton *Instance;

public:
    static StopWatchSingleton *GetInstance()
    {
        if( ! Instance )
            Instance = new StopWatchSingleton();

        return Instance;
    }

};

//------------------------------------------------------------------------------
StopWatchSingleton *StopWatchSingleton::Instance = NULL;

} // namespace

xsvfProgressFtnPtr xsvfProgress = 0;

extern xsvfFileInformation xsvfFileInfo;
extern Innovative::PmcJtagPort * PmcJtag;

/* if in debugging mode, then just set the variables */
void setPort(short p,short val)
{
    if (p==TMS)
        PmcJtag->Tms(val);
    if (p==TDI)
        PmcJtag->Tdi(val);
    if (p==TCK)
        PmcJtag->Tck(val);
}


/* toggle tck LH */
void pulseClock()
{
    PmcJtag->Tck(0);
    PmcJtag->Tck(1);
}

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4996)
#endif

/* read in a byte of data from the prom */
void readByte(unsigned char *data)
{
	Innovative::StopWatch & SW =StopWatchSingleton::GetInstance()->Rtc;
	
    static double then = 0;
    if (std::abs(static_cast<int>(SW.Now()-then)) > 20.0)
        {
        Innovative::Sleep(1);
        then = SW.Now();
        }

    /* pretend reading using a file */
    fscanf(xsvfFileInfo.in,"%c",data);
    /**data=*xsvf_data++;*/

    if (xsvfProgress)
        xsvfProgress(((++xsvfFileInfo.offset) * 100) / xsvfFileInfo.size);
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif


/* read the TDO bit from port */
unsigned char readTDOBit()
{
    return PmcJtag->Tdo();
}


/* Wait at least the specified number of microsec.                           */
/* Use a timer if possible; otherwise estimate the number of instructions    */
/* necessary to be run based on the microcontroller speed.  For this example */
/* we pulse the TCK port a number of times based on the processor speed.     */
void waitTime(long microsec)
{
    setPort( TCK, 0 );

    StopWatchSingleton::GetInstance()->Rtc.uS(microsec);
}
