// XsvfLoadThread_Mb.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2005 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <PmcJtagPort_Mb.h>
#include <algorithm>
#include <XsvfLoadThread_Mb.h>
#include <SingletonManager_Mb.h>
#include <sstream>
#include <Ports_Mb.h>
#include <Micro_Mb.h>
#include <time.h>
#include <stdio.h>
#include <SystemSupport_Mb.h>

extern xsvfFileInformation xsvfFileInfo;
extern xsvfDebugPrintFtnPtr xsvfDebugPrint;
extern xsvfProgressFtnPtr xsvfProgress;
Innovative::PmcJtagPort * PmcJtag = 0;

namespace   Innovative
{

static XsvfLoadThread * This = 0;

//==============================================================================
//  CLASS XsvfLoadThreadLockSingleton --
//==============================================================================

class XsvfLoadThreadLockSingleton : public OpenWire::SingletonService
{
public:
    CriticalSection Lock;

private:
    static XsvfLoadThreadLockSingleton *Instance;

public:
    static XsvfLoadThreadLockSingleton *GetInstance()
    {
        if( ! Instance )
            Instance = new XsvfLoadThreadLockSingleton();

        return Instance;
    }

};

//------------------------------------------------------------------------------
XsvfLoadThreadLockSingleton *XsvfLoadThreadLockSingleton::Instance = NULL;


//---------------------------------------------------------------------------
//  LoadMessage() --
//---------------------------------------------------------------------------

static void LoadMessage(const char * msg)
{
    This->HandleLoadMessage(msg);
}

//---------------------------------------------------------------------------
//  ProgressMessage() --
//---------------------------------------------------------------------------

static void ProgressMessage(int percent)
{
    if (This->Percent != percent)
        {
        This->HandleLoadProgress(percent);
        This->Percent = percent;
        }
}


//==============================================================================
//  CLASS XsvfLoadThread -- manages Logic load via select map
//==============================================================================

//---------------------------------------------------------------------------
//  XsvfLoadThread::XsvfLoadThread() -- Ctor
//---------------------------------------------------------------------------

XsvfLoadThread::XsvfLoadThread(PmcJtagPort * port)
    : inherited("XsvfLoadThread"), Port(port), Loading(Event::ManualReset)
{
    xsvfDebugPrint = LoadMessage;
    xsvfProgress = ProgressMessage;

    Priority(tpNormal);
    Resume();
}

//---------------------------------------------------------------------------
//  XsvfLoadThread::Init() --
//---------------------------------------------------------------------------

void XsvfLoadThread::Init()
{
    inherited::Init();
	AddObject(Loading, sLoading);
}

//---------------------------------------------------------------------------
//  XsvfLoadThread::Load() -- Execute Xsvf script
//---------------------------------------------------------------------------

void XsvfLoadThread::Load(std::string file, int debug_level)
{
    File = file;
    DebugLevel = debug_level;

    Loading.Set();
}

//------------------------------------------------------------------------------
//  XsvfLoadThread::HandleLoadMessage() --
//------------------------------------------------------------------------------

void XsvfLoadThread::HandleLoadMessage(const char * msg)
{
    ProcessStatusEvent e(msg);
    OnLoadMessage.Execute(e);
}

//------------------------------------------------------------------------------
//  XsvfLoadThread::HandleLoadProgress() --
//------------------------------------------------------------------------------

void XsvfLoadThread::HandleLoadProgress(int percent)
{
    if (Percent != percent)
        {
        ProcessProgressEvent e(percent);
        OnLoadProgress.Execute(e);
        Percent = percent;
        }
}

//---------------------------------------------------------------------------
//  XsvfLoadThread::DoLoad() --
//---------------------------------------------------------------------------

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4996)
#endif

void XsvfLoadThread::DoLoad()
{
    // xsvf functions provided by Xilinx are not re-entrant
    // So, only one task may execute this method at-a-time
    CriticalSectionArea Function(XsvfLoadThreadLockSingleton::GetInstance()->Lock);

    This = this;
    Percent = -1;

    PmcJtag = Port;

    Port->Enabled(true);

    // Open file
    xsvfFileInfo.in = fopen(File.c_str(), "rb");
    if (!xsvfFileInfo.in)
        {
        ProcessStatusEvent e("File open failure");
        OnLoadMessage.Execute(e);

        ProcessCompletionEvent c(0);
        OnLoadComplete.Execute(c);
        return;
        }
    // Determine file size
    fseek(xsvfFileInfo.in, 0, SEEK_END);
    xsvfFileInfo.size = ftell(xsvfFileInfo.in);
    fseek(xsvfFileInfo.in, 0, SEEK_SET);
    xsvfFileInfo.offset = 0;

    /* Initialize the I/O.  SetPort initializes I/O on first call */
    setPort( TMS, 1 );

    /* Execute the XSVF in the file */
    int iErrorCode = xsvfExecute(DebugLevel);

    fclose(xsvfFileInfo.in);

    Port->Enabled(false);

    ProcessCompletionEvent e(iErrorCode);
    OnLoadComplete.Execute(e);
}

#ifdef _MSC_VER
	#pragma warning(pop)	
#endif

//---------------------------------------------------------------------------
//  XsvfLoadThread::Execute() --
//---------------------------------------------------------------------------

void XsvfLoadThread::Execute()
{
    while (!Terminated())
        {
        // WaitResult is either a) Suicide event, b) Configure event
        int WaitResult = WaitFor();

        if (WaitResult == sLoading)
            {
            DoLoad();
            Loading.Clear();
            }
        }
}


}   // namespace Innovative
