//
// ZbtRam.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure 
//  agreement with Innovative Integration Corporation and may not be copied or 
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2003 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <ZbtRam_Mb.h>

namespace Innovative
{

//=============================================================================
//  CLASS ZbtCell  --  ZbtCell access class
//=============================================================================

//-----------------------------------------------------------------------------
//  Ctor
//-----------------------------------------------------------------------------

ZbtCell::ZbtCell(ZbtRam * parent, AddressingSpace & space, int addr_offset,
				 int read_offset, int write_offset)
	: AddressReg(space, addr_offset),
	  ReadReg(space, read_offset), WriteReg(space, write_offset),
	  Ram(parent), Mode(false)
{
}

//-----------------------------------------------------------------------------
//  Ctor
//-----------------------------------------------------------------------------

ZbtCell::ZbtCell(ZbtRam * parent, AddressingSpace & space, int addr_offset,
				 int data_offset)
	: AddressReg(space, addr_offset),
	  ReadReg(space, data_offset), WriteReg(space, data_offset), Ram(parent), 
      Mode(true)
{
}

//------------------------------------------------------------------------------
//  ZbtCell::operator =() --
//------------------------------------------------------------------------------

ZbtCell & ZbtCell::operator=(int value)
{
	if (Mode)
		{
		AddressReg.Value(Ram->Idx);
		ReadReg.Value(value);
		}
	else
		{
		int addr = Ram->Idx | (3 << 20);
		int data = (value & 0xffff) | (value << 16);
		WriteReg.Value(data);
		AddressReg.Value(addr);
		}

	return *this;
}

//------------------------------------------------------------------------------
//  ZbtCell::operator int() --
//------------------------------------------------------------------------------

ZbtCell::operator int() const
{
	int result;

	if (Mode)
		{
		AddressReg.Value(Ram->Idx);
		result = ReadReg.Value();
		}
	else
		{
		int addr = Ram->Idx | (1 << 20);
		AddressReg.Value(addr);
		result = ReadReg.Value();
		}

	return result;
}


//=============================================================================
//  CLASS ZbtRam  --  ZbtRam access class
//=============================================================================

#ifdef _MSC_VER
	#pragma warning(push)
	#pragma warning(disable : 4355)
#endif

//-----------------------------------------------------------------------------
//  ZbtRam Ctor
//-----------------------------------------------------------------------------

ZbtRam::ZbtRam(AddressingSpace & space, int addr_offset, int read_offset,
			   int write_offset)
	: Cell(this, space, addr_offset, read_offset, write_offset)
{
}

//-----------------------------------------------------------------------------
//  ZbtRam Ctor
//-----------------------------------------------------------------------------

ZbtRam::ZbtRam(AddressingSpace & space, int addr_offset, int data_offset)
	: Cell(this, space, addr_offset, data_offset)
{
}

#ifdef _MSC_VER
	#pragma warning(pop)
#endif

//------------------------------------------------------------------------------
//  ZbtRam::operator []() --
//------------------------------------------------------------------------------

ZbtCell ZbtRam::operator [] (int idx) const
{
	Idx = idx;
	return Cell;
}

//------------------------------------------------------------------------------
//  ZbtRam::operator []() --
//------------------------------------------------------------------------------

ZbtCell & ZbtRam::operator [] (int idx)
{
	Idx = idx;
	return Cell;
}

}  // namespace Innovative
