#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <GpDspManager_Gp.h>
#include <Framework/Qt/GuiXfr_Qt.h>
#include <Framework/GuiLoader_Mb.h>
#include <sstream>
#include "SoftwareTimer_Mb.h"
#include <QMenu>
#include <QSignalMapper>
#include "GraphTool_Mb.h"
#include "QTableWidgetEx.h"
#include <QScopedPointer>

using namespace Innovative;

namespace Ui {
    class MainWindow;
}

class QTableWidgetEx;
class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::MainWindow *ui;

friend class UserInterface;

private slots:
    void on_Tabs_currentChanged(int index );
    void on_ArithmeticOverflowGroup_clicked();
    void on_PowerMeterGroup_clicked();
    void on_ArithmeticOverflowGroup_clicked(bool checked);
    void on_SnapshotButton_clicked();
    void on_FftSetupOutputSpectrumComboBox_currentIndexChanged(int index);
    void on_FftSetupWindowingCombo_currentIndexChanged(int index);
    void FftSetupTableCurrentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);
    void on_FftSetupInputSourceCombo_currentIndexChanged(int index);
    void on_FiclCommandEdit_returnPressed();
    void on_FiclCommandEdit_textEdited(QString );
    void on_DeviceWriteButton_clicked();
    void on_DeviceReadButton_clicked();
    void on_PortWriteButton_clicked();
    void on_PortReadButton_clicked();
    void on_LogicWriteButton_clicked();
    void on_LogicReadButton_clicked();
    void on_DebugScriptExecuteButton_clicked();
    void on_DebugScriptBrowseButton_clicked();
    void on_StreamStopBtn_clicked();
    void on_StreamStartBtn_clicked();
    void on_CloseBtn_clicked();
    void on_OpenBtn_clicked();
    void on_tableStatus_cellChanged(int row, int column);
    void on_DDCTestSignalUpdateButton_clicked();
    void on_DDCChanConfigFileSelBtn_clicked();
    void on_DDCUpdateChanButton_clicked();
    void DdcTableCurrentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);
    void on_StreamLogForeverCheckBox_clicked(bool checked);
    void on_DdcChannelEnableButton_clicked();
    void on_DdcChannelDisableButton_clicked();
    void on_FftSetupPeakMonitorFileSelectButton_clicked();
    void on_FftSetupUpdateButton_clicked();
    void on_FftSetupManualTriggerButton_clicked();

    void EmbeddedCombo_currentIndexChanged(const QString &);

    void ShowContextMenu(const QPoint& pos);
    void EnableAutoScale();
    void EnablePlotting();
    void SavePlot();
    void TogglexScale();
    void CheckOverflowTableCell(bool,int,int);

    void on_tableStatus_currentCellChanged(int currentRow, int currentColumn, int previousRow, int previousColumn);

    void EmbeddedCombo_currentIndexChangedEmitCall(QString position, bool update);
    void FftSetupWindowingComboEmitCall(int idx, bool update);
    void FftSetupInputSourceComboEmitCall(int idx, bool update);

    void on_TT_SoftwareTriggerEnableB_clicked();

    void on_TT_SoftwareTriggerDisableB_clicked();

    void on_TT_GPSDisableB_clicked();

    void on_TT_GPSEnableB_clicked();

    void ClockModuleGuiConfig();

    void on_TT_PpsArmButton_clicked();

    void on_FrameModeFreqsListFileButton_clicked();

    void on_TT_TriggerInBtn_clicked();

public slots:
    void DdcTableChanged(const std::vector<int> &selected);

protected:
    void resizeEvent(QResizeEvent *event);

private: // User declarations
    //
    //  Manager Objects
    friend class MGR_GUIInterface;
    friend class MGR_FiclInterface;

    IManager_GUIInterface *    MUI;
    IManager_FiclInterface *   MFICL;
    GpDspManager *             Mgr;
    GraphTool                  FFTGraph;
    QMenu                      ContextMenu;
    QAction                    *pEnableAutoScale;
    QAction                    *pEnablePlot;
    QAction                    *pSavePlot;
    QAction                    *pRawxAxis;
    QSignalMapper              *psignalMapper;
    //QTableWidgetEx             *ddcTable;
    //QTableWidgetEx             *fftTable;

    QScopedPointer<QTableWidgetEx> ddcTable;
    QScopedPointer<QTableWidgetEx> fftTable;

    //
    // Application Logic/Hardware I/O Logic
    SoftwareTimer            StatusTimer;
    unsigned int             GSPSStatusSelect, x6400mStatusSelect;
    std::vector<class QStatusBar *>    PerformanceStatusBarFrames;
    Innovative::GuiLoader GL;

    bool         Inited;
    unsigned int SelectedFftRowIndex;

    //  Ficl Interface
    void         FiclInit();
    std::string  FiclReadCommandString();
    void         FiclCommandComplete();

    void  LoadFiclTargetSelector();

    void  Log(const std::string & a_string);
    void  Status(const std::string & a_string);
    void  PeriodicStatus(unsigned int ordinal);
    void  ProgressUpdate(int percent);
    void  DisplayGpsStatus();
    bool  WarnChannelSelect(const vector<int> &rows);
    void  ChannelEnable(bool value);
    void  InstallSettings(Drc_ProviderSettings &Settings);
    void  InstallComponentSettings();
    void  SetSettings();
    void  GetSettings();
    void  PopulateFftSourceCombo(unsigned int ordinal);
    void UpdateBoardStatus();
    void  HandleStatusTimer(OpenWire::NotifyEvent & Event);
    void SetupBoardsTable();
    void UpdateBoardsTable();
    void LoadPowerMeterTargetSelector();
    void BeforeStreamStart(unsigned int ordinal);
    void AfterStreamStart(unsigned int ordinal);
    void AfterStreamAutoStop(unsigned int ordinal);
    map<string, vector<string> > InitMaps(unsigned int ordinal);
    unsigned int SelectedFftBoard();
    unsigned int SelectedFftIndex();
    void ComponentInit();
    void CreateFftSourceMap();
    void DrawSelectedFftRow(int crow);
    void NameUpperLeftCornerOfTheTableWidget();
    void AddComboWidgetToTableWidget(int row, int col, const vector<string> &items, int idx);
    bool eventFilter(QObject* o, QEvent* e);
    void Plot(unsigned int ordinal, VeloBuffer &buffer, int type);
    ii64 SamplingDurationToSamples(float duration);
    void DataPacketAvailable(unsigned int ordinal,VeloBuffer &buffer);
    void  BeforeDrcOpen();
    void  AfterDrcOpen();
    void  BeforeDrcClose();
    void  AfterDrcClose();
    void  AfterDrcStreamStart();
    void  BeforeDrcStreamStart();
    void  AfterDrcStreamStop();
    void  BeforeDrcStreamStop();
    void  InstallSettings(ClockGUISettings &Settings);
};

//===========================================================================
//  CLASS MGR_FiclInterface  --
//===========================================================================

class MGR_FiclInterface : public IManager_FiclInterface
{
public:
    MGR_FiclInterface(MainWindow * form)
            :  Form(form)
            {}
    virtual ~MGR_FiclInterface()  {  }

    //  FICL User Interface Ftns
    void  FiclLog(const std::string & a_string)
            {  Form->Log(a_string);  }
    std::string  FiclReadCommandString()
            {  return Form->FiclReadCommandString();  }
    void  FiclCommandComplete()
            {  Form->FiclCommandComplete();  }
private:
    MainWindow *  Form;
};

//===========================================================================
//  CLASS MGR_GUIInterface  --
//===========================================================================

class MGR_GUIInterface : public IManager_GUIInterface
{
public:
    MGR_GUIInterface(MainWindow * form)
            :  Form(form)
            {}
    virtual ~MGR_GUIInterface()  {  }

    virtual void  Log(const std::string & a_string)
        {  Form->Log(a_string);  }
    virtual void  PeriodicStatus(unsigned int ordinal)
        {  Form->PeriodicStatus(ordinal);  }
    virtual void  BeforeStreamStart(unsigned int ordinal)
        {  Form->BeforeStreamStart(ordinal);  }
    virtual void  AfterStreamStart(unsigned int ordinal)
        {  Form->AfterStreamStart(ordinal);  }
    virtual void  AfterStreamAutoStop(unsigned int ordinal)
        {  Form->AfterStreamAutoStop(ordinal);  }
    virtual void Plot(unsigned int ordinal, VeloBuffer &buffer, int type)
        {  Form->Plot(ordinal, buffer, type);  }
    virtual void  DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer)
        {  Form->DataPacketAvailable(ordinal, buffer); }
    virtual void  BeforeDrcOpen()
        { Form->BeforeDrcOpen(); }
    virtual void  AfterDrcOpen()
        { Form->AfterDrcOpen(); }
    virtual void  BeforeDrcClose()
        { Form->BeforeDrcClose(); }
    virtual void  AfterDrcClose()
        { Form->AfterDrcClose(); }
    virtual void  AfterDrcStreamStart()
        { Form->AfterDrcStreamStart(); }
    virtual void  BeforeDrcStreamStart()
        { Form->BeforeDrcStreamStart(); }
    virtual void  AfterDrcStreamStop()
        { Form->AfterDrcStreamStop(); }
    virtual void  BeforeDrcStreamStop()
        { Form->BeforeDrcStreamStop(); }

private:
    MainWindow *  Form;
};


#endif // MAINWINDOW_H
