#ifndef DATAGRID_H
#define DATAGRID_H

#include <vector>
#include <map>
#include <algorithm>

#include <QTableWidget>

struct BoardStatusInfo
{
    BoardStatusInfo(int argTarget, bool argEnabled, string argStatus, string argCoreVersion,
                    string argSubVersion, string argLogicVersion, int argDdcCount,
                    int argDucCount, int argFftCount) :
        Target(argTarget), Status(argStatus), Enabled(argEnabled),
        CoreVersion(argCoreVersion), SubVersion(argSubVersion),
        LogicVersion(argLogicVersion), DdcCount(argDdcCount),
        DucCount(argDucCount), fftCount(argFftCount)
    {}

    int     Target;
    string  Status;
    bool    Enabled;
    string  CoreVersion;
    string  SubVersion;
    string  LogicVersion;
    int     DdcCount;
    int     DucCount;
    int     fftCount;

};

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(QTableWidget * control, const DdcChannelInfo &Item, std::pair<std::vector<std::string>, size_t> pa)
{
     static std::string OnOff[] = {"Off", "On"};

     Innovative::ToControl(control, 0, pa.second, OnOff[Item.Enable], true);
     Innovative::ToControl(control, 1, pa.second, pa.first[Item.Source], true);
     Innovative::ToControl(control, 2, pa.second, Item.Frequency, 4, true);
     Innovative::ToControl(control, 3, pa.second, OnOff[Item.SpectrumInvert], true);
     Innovative::ToControl(control, 4, pa.second, Item.File, true);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(QTableWidget *control, DdcChannelInfo &Item, std::pair<std::map<std::string, int>, int> pa)
{
     std::string ch_str, src_str, spec_str, freq_str;

     Innovative::FromControl(control, 0, pa.second, ch_str);
     Item.Enable = pa.first[ch_str];
     Innovative::FromControl(control, 1, pa.second, src_str);
     Item.Source = pa.first[src_str];
     Innovative::FromControl(control, 2, pa.second, freq_str);
     Item.Frequency = Innovative::StringToDouble(freq_str, 0.0);
     Innovative::FromControl(control, 3, pa.second, spec_str);
     Item.SpectrumInvert = pa.first[spec_str];
     Innovative::FromControl(control, 4, pa.second, Item.File);
}


//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(QTableWidget * control, const DucChannelInfo &Item, std::pair<std::vector<std::string>, size_t> pa)
{
     static std::string OnOff[] = {"Off", "On"};

     Innovative::ToControl(control, 0, pa.second, pa.first[Item.Source], true);
     Innovative::ToControl(control, 1, pa.second, Item.StreamId, 16,  true);
     Innovative::ToControl(control, 2, pa.second, Item.Frequency, 3,  true);
     Innovative::ToControl(control, 3, pa.second, OnOff[Item.SpectrumInvert], true);
     Innovative::ToControl(control, 4, pa.second, Item.File, true);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(QTableWidget *control, DucChannelInfo &Item, std::pair<std::map<std::string, int>, int> pa)
{
    std::string src_str, spec_str, freq_str;

    Innovative::FromControl(control, 0, pa.second, src_str);
    Item.Source = pa.first[src_str];
    Innovative::FromControl(control, 1, pa.second, Item.StreamId);
    Innovative::FromControl(control, 2, pa.second, freq_str);
    Item.Frequency = Innovative::StringToDouble(freq_str, 0.0);
    Innovative::FromControl(control, 3, pa.second, spec_str);
    Item.SpectrumInvert = pa.first[spec_str];
    Innovative::FromControl(control, 4, pa.second, Item.File);
}


//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(QTableWidget * control, const ToneGeneratorDataType &Item, int row)
{
     Innovative::ToControl(control, 0, row, Item.Freq, 3);
     control->setColumnHidden(1, true);
     //Innovative::ToControl(control, 1, row, Item.Phase,3);
     Innovative::ToControl(control, 2, row, Item.Gain, 3);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(QTableWidget *control, ToneGeneratorDataType &Item, int row)
{
    Innovative::FromControl(control, 0, row, Item.Freq);
    Innovative::FromControl(control, 1, row, Item.Phase);
    Innovative::FromControl(control, 2, row, Item.Gain);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(QTableWidget * /*control*/, const FftInfo &/*Item*/, int /*row*/, const vector<std::string> &/*Sources*/)
{
     static std::string OnOff[] = {"Off", "On"};

     //Innovative::ToControl(control, 0, row, OnOff[Item.Enable], true);

}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(QTableWidget * control, const FftInfo &Item, int row, map<string, vector<std::string> > maps)
{
    Innovative::ToControl(control, 0, row, maps["TriggerMode"][Item.TriggerMode], true);
    Innovative::ToControl(control, 1, row, maps["Source"][Item.Source], true);
    Innovative::ToControl(control, 2, row, maps["Window"][Item.Window], true);
    Innovative::ToControl(control, 3, row, maps["OnOff"][Item.FasdEnable], true);
    Innovative::ToControl(control, 4, row, Item.Fasd, 4, true);
    Innovative::ToControl(control, 5, row, maps["OutputMode"][Item.OutputMode], true);
    Innovative::ToControl(control, 6, row, Item.Fps, 2, true);
    Innovative::ToControl(control, 7, row, Item.ThresholdFile, true);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(QTableWidget */*control*/, FftInfo &/*Item*/, int /*row*/)
{

}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(QTableWidget *control, FftPlotRowDataType &Item, int row)
{
    Innovative::FromControl(control, 1, row, Item.SourceIndex);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(QTableWidget * control, const TimerStatusInfo &Item, int row)
{
    Innovative::ToControl(control, 0, row, Item._name, true);
    Innovative::ToControl(control, 1, row, Item._rev, true);
    Innovative::ToControl(control, 2, row, Item._locked, true);
    Innovative::ToControl(control, 3, row, Item._striggered, true);
    Innovative::ToControl(control, 4, row, Item._temp, 3, true);
    Innovative::ToControl(control, 5, row, Item._sat, 10, true);
    Innovative::ToControl(control, 6, row, Item._time, true);
    Innovative::ToControl(control, 7, row, Item._lat, true);
    Innovative::ToControl(control, 8, row, Item._longitude, true);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(QTableWidget */*control*/, TimerStatusInfo &/*Item*/, int /*row*/)
{
    // Nothing to do
}


//---------------------------------------------------------------------------
//  SelectedRows() -- >> Return selected row(s)
//---------------------------------------------------------------------------

void SelectedRows(QTableWidget * control, vector<int> &rows)
{
    QList<QTableWidgetItem *> selected = control->selectedItems();

    for (int i = 0; i < selected.size(); i++)
        if (selected[i]->column() == 0) // only header
            rows.push_back(selected[i]->row());
}

//---------------------------------------------------------------------------
//  DrcTableInit() -- >> Init table
//---------------------------------------------------------------------------

void SetSelectedRow(QTableWidget * control, int row)
{
    control->selectRow(row);
}

//---------------------------------------------------------------------------
//  TableInit() -- >> Init table
//---------------------------------------------------------------------------

void TableInit(QTableWidget * control, const vector<std::string> &RowsHeader, const vector<std::string> &ColsHeader, bool RowSelect = true)
{
    control->clear();

    int rows = RowsHeader.size();
    int cols = ColsHeader.size();
    SetDimensions(control, cols, rows);
    for (int i = 0; i < cols; i++)
        ToColumnHeading(control, i, ColsHeader[i]);
    for (int i = 0; i < rows; i++)
        ToRowHeading(control, i, RowsHeader[i]);

    if (RowSelect)
        control->setSelectionBehavior(QAbstractItemView::SelectRows);
    // Headers fit to content
#if QT_VERSION >= 0x050000
    control->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    control->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif


}

//---------------------------------------------------------------------------
//  SetTableSelectionBehavior() -- >> Init table
//---------------------------------------------------------------------------

void TableSelectionBehavior(QTableWidget * control)
{
    control->setSelectionBehavior(QAbstractItemView::SelectRows);
}

//---------------------------------------------------------------------------
//  DucTableInit() -- >> Init table
//---------------------------------------------------------------------------

void DucTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Source", "Stream ID", "Tune[Mhz]", "Spect Inv", "Configuration File"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader, false);
}


//---------------------------------------------------------------------------
//  DrcTableInit() -- >> Init table
//---------------------------------------------------------------------------

void DrcTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Ch En", "Source", "Tune[Mhz]", "Spect Inv", "Configuration File"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader, false);
}

//---------------------------------------------------------------------------
//  FftPlotTableInit() -- >> Init table
//---------------------------------------------------------------------------

void FftPlotTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Source", "Window"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
    control->setSelectionMode(QAbstractItemView::SingleSelection);
}

//---------------------------------------------------------------------------
//  FftSetupTableInit() -- >> Init table
//---------------------------------------------------------------------------

void FftSetupTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Trig Mode", "Source", "Window", "FASD En.", "FASD Coef.", "Spectrum", "Frames/sec", "Threshold File"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
    control->setSelectionMode(QAbstractItemView::SingleSelection);
}

//---------------------------------------------------------------------------
//  BoardStatusTableInit() -- >> Init table
//---------------------------------------------------------------------------

void BoardStatusTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Target", "Status", "Enabled", "Project Info", "Project Version",
                                "DDC Cores","DUC Cores","FFT Cores"/*, "Module Info"*/};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  StreamStatusTableInitDdc() -- >> Init table
//---------------------------------------------------------------------------

void StreamStatusTableInitDdc(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Rx (M Samples)", "Rate (MB/s)", "FFT Rx (M Samples)",
                                "FFT Rate (MB/s)", "Temp (C)", "PLL Lock"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  StreamStatusTableInitDuc() -- >> Init table
//---------------------------------------------------------------------------

void StreamStatusTableInitDuc(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Rx (M Samples)", "Rate (MB/s)","Tx (Packets)", "Tx Rate (MB/s)", "FFT Rx (M Samples)",
                                "FFT Rate (MB/s)", "Temp (C)",  "PLL Lock"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  PowerTableInit() -- >> Init table
//---------------------------------------------------------------------------

void PowerTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"dBm"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  OverflowTableInit() -- >> Init table
//---------------------------------------------------------------------------

void OverflowTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Tun", "Chz", "Ppf", "Cic", "CFir", "PFir", "Fir1", "Fft"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  TestGeneratorTableInit() -- >> Init table
//---------------------------------------------------------------------------

void TestGeneratorTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Freq (MHz)", "Ph. (Deg)", "Gain (-2, 2)"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  PowerMeterTableInit() -- >> Init table
//---------------------------------------------------------------------------

void PowerMeterTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Pwr(dB)"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  TimingModulesTableInit() -- >> Init table
//---------------------------------------------------------------------------

void TimingModulesTableInit(QTableWidget * control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Name", "Version", "PLL Lock", "SW Trig", "Temp",
                                "GPS Sat", "GPS Time", "GPS Lat", "GPS Long"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));
    TableInit(control, RowsHeader, ColsHeader);
}


//---------------------------------------------------------------------------
//  UpdateBoardEntry() -- >>
//---------------------------------------------------------------------------

void UpdateBoardEntry(QTableWidget * control, const BoardStatusInfo &Item, int Row)
{
    Innovative::ToControl(control, 0, Row, Item.Target);
    Innovative::ToControl(control, 1, Row, Item.Status, true);
    Innovative::ToControl(control, 2, Row, Item.Enabled);
    Innovative::ToControl(control, 3, Row, Item.CoreVersion, true);
    Innovative::ToControl(control, 4, Row, Item.SubVersion, true);    
    Innovative::ToControl(control, 5, Row, Item.DdcCount);
    Innovative::ToControl(control, 6, Row, Item.DucCount);
    Innovative::ToControl(control, 7, Row, Item.fftCount);
    //Innovative::ToControl(control, 5, Row, Item.LogicVersion, true);
}

//---------------------------------------------------------------------------
//  ReadBoardEntry() -- >>
//---------------------------------------------------------------------------

void ReadBoardEntry(QTableWidget * control, int &Target, int &Enabled, int Row)
{
    bool tEnabled = false;
    Innovative::FromControl(control, 0, Row, Target);
    Innovative::FromControl(control, 2, Row, tEnabled);
    Enabled = tEnabled ? 1 : 0;

}

//---------------------------------------------------------------------------
//  TargetForRow() -- >>
//---------------------------------------------------------------------------

void TargetForRow(QTableWidget * control, const int &Row, int &Target)
{
    Innovative::FromControl(control, 0, Row, Target);
}



#endif // DATAGRID_H
