#ifndef DATAGRID_H
#define DATAGRID_H

#include <vector>
#include <map>


//---------------------------------------------------------------------------
namespace Innovative
{

struct BoardStatusInfo
{
    BoardStatusInfo(int argTarget, bool argEnabled, string argStatus, string argCoreVersion,
		string argSubVersion, string argLogicVersion, int argDdcCount,
		int argDucCount, int argFftCount) :
        Target(argTarget), Status(argStatus), Enabled(argEnabled),
        CoreVersion(argCoreVersion), SubVersion(argSubVersion),
		LogicVersion(argLogicVersion), DdcCount(argDdcCount),
		DucCount(argDucCount), fftCount(argFftCount)
    {}

    int     Target;
    string  Status;
    bool    Enabled;
    string  CoreVersion;
    string  SubVersion;
	string  LogicVersion;
	int	    DdcCount;
    int		DucCount;
	int	    fftCount;
};

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::DataGridView^ control,const GP::DdcChannelInfo &Item, int row, cli::array<System::String ^> ^sources)
{
     static std::string OnOff[] = {"Off", "On"};

     Innovative::ToControl(control, 0, row, OnOff[Item.Enable]);
     Innovative::ToControl(control, 1, row, StdString(sources[Item.Source]));
     Innovative::ToControl(control, 2, row, Item.Frequency, 3);
     Innovative::ToControl(control, 3, row, OnOff[Item.SpectrumInvert]);
     Innovative::ToControl(control, 4, row, Item.File);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::DataGridView^ control, const GP::DucChannelInfo &Item, int row, cli::array<System::String ^> ^sources)
{
     static std::string OnOff[] = {"Off", "On"};

     Innovative::ToControl(control, 0, row, StdString(sources[Item.Source]));
	 Innovative::ToControl(control, 1, row, Item.StreamId, 16);
     Innovative::ToControl(control, 2, row, Item.Frequency, 3);
     Innovative::ToControl(control, 3, row, OnOff[Item.SpectrumInvert]);
     Innovative::ToControl(control, 4, row, Item.File);
}


//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::DataGridView^, GP::DdcChannelInfo &, int row)
{
	// Nothing to do
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::DataGridView^, GP::DucChannelInfo &, int row)
{
	 // Nothing to do
}
//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::DataGridView^ control, const ToneGeneratorDataType &Item, int row)
{
     Innovative::ToControl(control, 0, row, Item.Freq, 3);
     Innovative::ToControl(control, 1, row, Item.Phase,3);
     Innovative::ToControl(control, 2, row, Item.Gain, 3);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::DataGridView^control, ToneGeneratorDataType &Item, int row)
{
    Innovative::FromControl(control, 0, row, Item.Freq);
    Innovative::FromControl(control, 1, row, Item.Phase);
    Innovative::FromControl(control, 2, row, Item.Gain);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::DataGridView^ control, const FftInfo &Item, int row, const vector<std::string> &Sources)
{
     static std::string OnOff[] = {"Off", "On"};

    // Innovative::ToControl(control, 0, row, OnOff[Item.Enable]);
    // Innovative::ToControl(control, 1, row, Sources[Item.Source]);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::DataGridView^ control, const FftInfo &Item, int row, Dictionary<System::String^, cli::array<System::String^ >^ > ^ maps)
{
    Innovative::ToControl(control, 0, row, StdString(maps["TriggerMode"][Item.TriggerMode]));
    Innovative::ToControl(control, 1, row, StdString(maps["Source"][Item.Source]));
    Innovative::ToControl(control, 2, row, StdString(maps["Window"][Item.Window]));
    Innovative::ToControl(control, 3, row, StdString(maps["OnOff"][Item.FasdEnable]));
    Innovative::ToControl(control, 4, row, Item.Fasd, 3);
    Innovative::ToControl(control, 5, row, StdString(maps["OutputMode"][Item.OutputMode]));
    Innovative::ToControl(control, 6, row, Item.Fps, 3);
    Innovative::ToControl(control, 7, row, Item.ThresholdFile);
}

//---------------------------------------------------------------------------
//  ToControl() -- >> Control
//---------------------------------------------------------------------------

void ToControl(System::Windows::Forms::DataGridView^ control, const TimerStatusInfo &Item, int row)
{
	Innovative::ToControl(control, 0, row, Item._name);
	Innovative::ToControl(control, 1, row, Item._rev);
	Innovative::ToControl(control, 2, row, Item._locked);
	Innovative::ToControl(control, 3, row, Item._striggered);
	Innovative::ToControl(control, 4, row, Item._temp, 3);
	Innovative::ToControl(control, 5, row, Item._sat, 10);
	Innovative::ToControl(control, 6, row, Item._time);
	Innovative::ToControl(control, 7, row, Item._lat);
	Innovative::ToControl(control, 8, row, Item._longitude);
}


//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::DataGridView^control, FftInfo &Item, int row)
{

}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control
//---------------------------------------------------------------------------

void FromControl(System::Windows::Forms::DataGridView^ control, FftPlotRowDataType &Item, int row)
{
    Innovative::FromControl(control, 1, row, Item.SourceIndex);
}

//---------------------------------------------------------------------------
//  SelectedRows() -- >> Return selected row(s)
//---------------------------------------------------------------------------

void SelectedRows(System::Windows::Forms::DataGridView^ control, vector<int> &rows)
{
	 for (int i = 0; i < control->SelectedRows->Count ; i++)					
            rows.push_back( control->SelectedRows[i]->Index );  //Passing the row index 
}

//---------------------------------------------------------------------------
//  SetSelectedRow() -- >> 
//---------------------------------------------------------------------------

void SetSelectedRow(System::Windows::Forms::DataGridView^ control, int row)
{
    control->Rows[row]->Selected = true;
}


//---------------------------------------------------------------------------
//  TableInit() -- >> Init table
//---------------------------------------------------------------------------

void TableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader, const vector<std::string> &ColsHeader)
{
	control->Rows->Clear();

    int rows = RowsHeader.size();
    int cols = ColsHeader.size();

	if (rows > 0 && cols > 0)
		{
    SetDimensions(control, cols, rows);
    for (int i = 0; i < cols; i++)
        ToColumnHeading(control, i, ColsHeader[i]);
    for (int i = 0; i < rows; i++)
        ToRowHeading(control, i, RowsHeader[i]);
		}

	// Whole rows are selected
//		control->SelectAll=true;
    // Headers fit to content
	    control->AutoResizeColumns(System::Windows::Forms::DataGridViewAutoSizeColumnsMode::AllCells);
}


//---------------------------------------------------------------------------
//  DrcTableInit() -- >> Init table
//---------------------------------------------------------------------------

void DrcTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Ch En", "Source", "Tune[Mhz]", "Spect Inv", "Configuration File"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  DucTableInit() -- >> Init table
//---------------------------------------------------------------------------

void DucTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Source", "Stream ID", "Tune[Mhz]", "Spect Inv", "Configuration File"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  FftPlotTableInit() -- >> Init table
//---------------------------------------------------------------------------

void FftPlotTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Source", "Window"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  FftSetupTableInit() -- >> Init table
//---------------------------------------------------------------------------

void FftSetupTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Trig Mode", "Source", "Window", "FASD En.", "FASD Coef.", "Spectrum", "Frames/sec", "Threshold File"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  BoardStatusTableInit() -- >> Init table
//---------------------------------------------------------------------------

void BoardStatusTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Target", "Status", "Enabled", "Project Info", "Project Version",
								"DDC Cores", "DUC Cores", "FFT Cores"/*, "Module Info"*/ };
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  StreamStatusTableInitDdc() -- >> Init table
//---------------------------------------------------------------------------

void StreamStatusTableInitDdc(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = { "Rx (M Samples)", "Rx Rate (MB/s)", "FFT Rx (M Samples)",
        "FFT Rate (MB/s)", "Temp (C)", "PLL Lock" };
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  StreamStatusTableInitDuc() -- >> Init table
//---------------------------------------------------------------------------

void StreamStatusTableInitDuc(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = { "Rx (M Samples)", "Rx Rate (MB/s)", "Tx (Packets)", "Tx Rate (MB/s)", "FFT Rx (M Samples)",
        "FFT Rate (MB/s)", "Temp (C)", "PLL Lock" };
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  OverflowTableInit() -- >> Init table
//---------------------------------------------------------------------------

void OverflowTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Tun", "Chz", "Ppf", "Cic", "CFir", "PFir", "Fir1", "Fft"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  TestGeneratorTableInit() -- >> Init table
//---------------------------------------------------------------------------

void TestGeneratorTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Freq (MHz)", "Ph. (Deg)", "Gain (-2, 2)"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  PowerMeterTableInit() -- >> Init table
//---------------------------------------------------------------------------

void PowerMeterTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
    std::string horizantal[] = {"Pwr(dBm)"};
    vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));

    TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  TimingModulesTableInit() -- >> Init table
//---------------------------------------------------------------------------

void TimingModulesTableInit(System::Windows::Forms::DataGridView^ control, const vector<std::string> &RowsHeader)
{
	std::string horizantal[] = { "Name", "Version", "PLL Lock", "SW Trig", "Temp",
		"GPS Sat", "GPS Time", "GPS Lat", "GPS Long" };
	vector<std::string> ColsHeader(horizantal, horizantal + sizeof(horizantal) / sizeof(string));
	TableInit(control, RowsHeader, ColsHeader);
}

//---------------------------------------------------------------------------
//  UpdateBoardEntry() -- >>
//---------------------------------------------------------------------------

void UpdateBoardEntry(System::Windows::Forms::DataGridView^ control, const BoardStatusInfo &Item, int Row)
{
    Innovative::ToControl(control, 0, Row, Item.Target);
    Innovative::ToControl(control, 1, Row, Item.Status);
    Innovative::ToControl(control, 2, Row, Item.Enabled);
    Innovative::ToControl(control, 3, Row, Item.CoreVersion);
	Innovative::ToControl(control, 4, Row, Item.SubVersion);
	Innovative::ToControl(control, 5, Row, Item.DdcCount);
	Innovative::ToControl(control, 6, Row, Item.DucCount);
	Innovative::ToControl(control, 7, Row, Item.fftCount);
    //Innovative::ToControl(control, 5, Row, Item.LogicVersion);
}

//---------------------------------------------------------------------------
//  FromControl() -- >> Control (DataGridCell | string)
//---------------------------------------------------------------------------

void  FromControl(System::Windows::Forms::DataGridView^ control, 
    const int col, const int row, bool & var)
{
	System::String ^ name = control[col,row]->Value->ToString();
	var = StringToInt(StdString(name)) == 1;
}

//---------------------------------------------------------------------------
//  ReadBoardEntry() -- >>
//---------------------------------------------------------------------------

void ReadBoardEntry(System::Windows::Forms::DataGridView^ control, int &Target, int &Enabled, int Row)
{
    bool tEnabled = false;
    Innovative::FromControl(control, 0, Row, Target);
    Innovative::FromControl(control, 2, Row, tEnabled);
    Enabled = tEnabled ? 1 : 0;

}

//---------------------------------------------------------------------------
//  TargetForRow() -- >>
//---------------------------------------------------------------------------

void TargetForRow(System::Windows::Forms::DataGridView^ control, const int &Row, int &Target)
{
    Innovative::FromControl(control, 0, Row, Target);
}

}  // end namespace Innovative

#endif // DATAGRID_H
