
#include <SystemSupport_Mb.h>
#include "Filter.h"

#ifdef __BORLANDC__
    #pragma option push
    #pragma warn -8004
#endif

const double PI = 3.14159265;

//===========================================================================
//  CLASS Filter  --
//===========================================================================


//---------------------------------------------------------------------------
//  Filter::Clear() --
//---------------------------------------------------------------------------
void Filter::Clear()
{
    DataLoaded = false;
    DataApplied = false;

    PolyFirCoef.PolyVec.clear();
    AuxCoef.Coef.clear();
    CFirCoef.Coef.clear();
    PFirCoef.Coef.clear();
    Fir1Coef.Coef.clear();
	Fir2Coef.Coef.clear();
}

//---------------------------------------------------------------------------
//  Filter::Load() - load all data from ini
//---------------------------------------------------------------------------
void Filter::Load(const string & file)
{
    FileName = file;
    Ini=new IniFile(FileName, false, 1e5, false);

    DecimationFactor = Ini->Read("DecimationFactor", 1.0f);
	InterpolationRate = Ini->Read("InterpolationFactor", 1.0f);
    //Filter Selects
    Ini->Section = "Filt_Sel";
    FilterSelection_1 = Ini->Read("filt_sel1", 0);
    FilterSelection_2 = Ini->Read("filt_sel2", 0);

    //Poly
    Ini->Section = "Poly_Overview";
    PolyFirCoef.Name = Ini->Section;
    PolyFirCoef.BStartNum = Ini->Read("ShiftNum", 0);
    PolyFirCoef.BStartDir = Ini->Read("ShiftDir", 0);

    PolyFirCoef.NPoly =  Ini->Read("NPoly", 0);
    PolyFirCoef.PolyVec.resize(PolyFirCoef.NPoly);

    stringstream s;
    for(unsigned int p=0; p<PolyFirCoef.NPoly; p++)
    {
        s.str("");
        s<<"Poly"<<p;
        LoadSection(s.str(), PolyFirCoef.PolyVec[p]);
    }

    //CIC
    Ini->Section = "CIC";
    CicBStartNum = Ini->Read("ShiftNum", 0);
    CicBStartDir = Ini->Read("ShiftDir", 0);

    CicRate = Ini->Read("CicRate", 0);

    //others
    LoadSection("Fir_i3", AuxCoef);  //DUC
    LoadSection("fir_d3", AuxCoef);  //DDC
    LoadSection("Cfir", CFirCoef);
    LoadSection("PFir", PFirCoef);
    LoadSection("Fir1", Fir1Coef);
	LoadSection("Fir2", Fir2Coef);
}

//---------------------------------------------------------------------------
//  Filter::LoadSection() -- load one data section from ini
//---------------------------------------------------------------------------
void Filter::LoadSection(const string &SectionName, FilterData &Coef)
{
    FilterData TempData;

    Ini->Section = SectionName;

    //  Read the BStart value
    TempData.Name = SectionName;
    TempData.BStartNum =  Ini->Read("ShiftNum", 0);
    TempData.BStartDir =  Ini->Read("ShiftDir", 0);

    //  Read the Taps from the INI file
    TempData.Taps = Ini->Read("Taps", 0);
    TempData.TapsToLoad = Ini->Read("TapsToLoad", 0);

    if (TempData.TapsToLoad<=0)
        TempData.TapsToLoad = TempData.Taps;

    int x=0;
    for (unsigned int i = 0; i < TempData.TapsToLoad; ++i)
    {
        std::string idx = "Index" + IntToString(i);
        std::string tag = "Coeff" + IntToString(Ini->Read(idx, x));
        TempData.Coef.push_back(Ini->Read(tag, x));
    }

    Coef = TempData;
}

#ifdef __BORLANDC__
    #pragma option pop
#endif
