#ifdef GCC
#include <QFrame>
#endif

#include <GraphTool_Mb.h>
#include "Poco/SharedLibrary.h"
#include "Poco/Exception.h"
#include "Poco/Path.h"
#include <Trace_Mb.h>

#if defined(GCC)
#include <Framework/Qt/GuiXfr_Qt.h>
#endif

using Poco::Path;
using Poco::SharedLibrary;

namespace Innovative
{

//---------------------------------------------------------------------------
//  Explicit template instantiate for two types
//---------------------------------------------------------------------------
#if !defined(GCC)
template class GraphTool::Loader<HWND>;
#else
template class GraphTool::Loader<QFrame*>;
#endif

//---------------------------------------------------------------------------
//   GraphTool::Loader<T>::Load()
//---------------------------------------------------------------------------
template <class T>
void GraphTool::Loader<T>::Load()
{
    typedef GraphWidget* (*FactoryMethodType) (T);
    Innovative::Trace Debug;

#if defined(GCC)
    if (!QtVersionMatch()) {
        Debug.Log("!!!! Qt version mismatch  !!!! => compiled : " +
                  CompileTimeQtVersion() + " run time : " + RunTimeQtVersion());
    }
#endif

    if (_handle)
        {
        try {
            // Load DLL
            Poco::SharedLibrary library(_outer._path + _outer._name);
            // Get the factory method ptr
            FactoryMethodType FactoryMethod = (FactoryMethodType) library.getSymbol("CreateGraphTool");
            // Create Graph
            _outer._graph = FactoryMethod(_handle);
        } catch(const Poco::LibraryLoadException& e) {
            _outer._graph = new NullGraphWidget("LibraryLoadException " + e.displayText());
        } catch(const Poco::NotFoundException& e) {
            _outer._graph = new NullGraphWidget("NotFoundException " + e.displayText());
        }
    }
    else
    {
        _outer._graph = new NullGraphWidget("GraphTool::Loader : Empty handle");
    }

    _outer._graph->init();
}

//---------------------------------------------------------------------------
//  GraphTool::GraphTool()
//---------------------------------------------------------------------------

GraphTool::GraphTool(string name, string path)
    : _name(name), _path(path), _graph(0), _loader(0)
{
#ifdef GCC
    _name.append("_Qt");
#else
    _name.append("_Vc");
#endif

    if (path == ".")
                _path = Poco::Path::current();

    _name.append(SharedLibrary::suffix()); //adds ".dll" or ".so"
}

//---------------------------------------------------------------------------
//  GraphTool::~GraphTool()
//---------------------------------------------------------------------------

GraphTool::~GraphTool()
{
    if (_graph)  delete _graph;
    if (_loader) delete _loader;
}

//---------------------------------------------------------------------------
//  GraphTool::Attach()
//---------------------------------------------------------------------------

string GraphTool::Path()
{
    return _path;
}

//---------------------------------------------------------------------------
//  GraphTool::Attach()
//---------------------------------------------------------------------------

void GraphTool::Path(const string &value)
{
    if (value == ".") 
        _path = Poco::Path::current();

    _path = value;
}

//---------------------------------------------------------------------------
//  GraphTool::operator->()
//---------------------------------------------------------------------------

GraphWidget *GraphTool::operator->()
{
    // If not loaded, Load
    if (!_graph) _loader->Load();

    return _graph;
}

//---------------------------------------------------------------------------
//  GraphTool::Ref()
//---------------------------------------------------------------------------

GraphWidget &GraphTool::Ref()
{
    // If not loaded, Load
    if (!_graph) _loader->Load();

    return *_graph;
}

}

