#ifndef GRAPHTOOL_H
#define GRAPHTOOL_H


#include <Buffer_Mb.h>
#if !defined(LINUX)
#include <Windows.h>
#endif

#if GCC
class QWidget;
#endif

using std::string;

namespace GraphToolNS {

//===========================================================================
//  CLASS GraphWidget --
//===========================================================================

class GraphWidget
{
public:
    GraphWidget() {}
    virtual ~GraphWidget() {}

    virtual void init() = 0;
    virtual void setFftSamples(int curve, const Innovative::VeloBuffer &buffer) = 0;
    virtual void setTlsSamples(int curve, const Innovative::VeloBuffer &buffer) = 0;
    virtual void setGraphOffset(float value) = 0;
    virtual void takeSnapshot(string filename) = 0;
    virtual void setxAxis(int left, int right) = 0;
    virtual void setyAxis() = 0;
    virtual void setRealAxis() = 0;
    virtual void setComplexAxis() = 0;
    virtual void setBandwidth(float bw) = 0;
    virtual void setRate(float rate) = 0;
    virtual void setSamplingRate(float srate) = 0;
    virtual void setDecimationRate(float drate) = 0;
    virtual void showSamples(int curve, bool show) = 0;
    virtual void clearAll() = 0;
    virtual void resize() = 0;
    virtual  int version() = 0;
};

//===========================================================================
//  CLASS NullGraphWidget --
//===========================================================================

class NullGraphWidget : public GraphWidget
{
public:
    NullGraphWidget()  {}
    NullGraphWidget(const std::string &msg) : emsg(msg)
    {}

    ~NullGraphWidget() {}

    void init() {}
    void setFftSamples(int /*curve*/, const Innovative::VeloBuffer &/*buffer*/) {}
    void setTlsSamples(int /*curve*/, const Innovative::VeloBuffer &/*buffer*/) {}
    void setGraphOffset(float /*value*/) {}
    void takeSnapshot(string /*filename*/) {}
    void setxAxis(int /*left*/, int /*right*/) {}
    void setyAxis() {}
    void setRealAxis() {}
    void setComplexAxis() {}
    void setBandwidth(float /*bw*/) {}
    void setRate(float /*rate*/) {}
    void setSamplingRate(float /*srate*/) {}
    void setDecimationRate(float /*drate*/) {}
    void setFftSamples(int /*curve*/, std::vector<unsigned short> &/*rys*/)  {}
    void resize() {}
    int  version() { return 0; }
    void showSamples(int /*curve*/, bool /*show*/) {}
    void clearAll() {}

    std::string getMessage()
    { return emsg; }

private:

    std::string emsg;
};


#if defined(_WIN32) && !defined(GCC)
#    ifdef LIBRARY_EXPORTS
#        define LIBRARY_API extern "C" __declspec(dllexport)
#    else
#        define LIBRARY_API extern "C" __declspec(dllimport)
#    endif
#else
#    include <QtCore/qglobal.h>
#    define LIBRARY_API extern "C" Q_DECL_EXPORT
#endif

#if GCC
    LIBRARY_API GraphWidget* ManageGraphTool(QWidget *parent);
    LIBRARY_API GraphWidget* CreateGraphTool(QWidget *parent);
#else
    LIBRARY_API GraphWidget* ManageGraphTool(HWND handle);
    LIBRARY_API GraphWidget* CreateGraphTool(HWND handle);
#endif

}


#endif
