
#include "QTableWidgetEx.h"

//---------------------------------------------------------------------------
//  QTableWidgetEx::QTableWidgetEx() -- >>
//---------------------------------------------------------------------------
QTableWidgetEx::QTableWidgetEx(QWidget *parent) :
    QTableWidget(parent)
{
    parent->installEventFilter(this);
}

//---------------------------------------------------------------------------
//  QTableWidgetEx::~QTableWidgetEx() -- >>
//---------------------------------------------------------------------------
QTableWidgetEx::~QTableWidgetEx()
{

}

//---------------------------------------------------------------------------
//  QTableWidgetEx::eventFilter() -- >>
//---------------------------------------------------------------------------
bool QTableWidgetEx::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == QEvent::Resize)
    {
        QResizeEvent *resizeEvent = static_cast<QResizeEvent*>(event);
        if (
            resizeEvent->size().height() >= viewport()->contentsRect().size().height() &&
            resizeEvent->size().width()  >= viewport()->contentsRect().size().width()
           )
            resize(resizeEvent->size());
    }
    return QWidget::eventFilter(obj, event);
}

//---------------------------------------------------------------------------
//  QTableWidgetEx::keyPressEvent() -- >>
//---------------------------------------------------------------------------
void QTableWidgetEx::keyPressEvent(QKeyEvent* event)
{

    if( event->matches( QKeySequence::Copy ) )
    {
        src_cells.clear();
        foreach (QTableWidgetItem *cell, selectedItems())
           src_cells.push_back(std::make_pair(cell->text(), cell->column()));
    }
    else if( event->matches( QKeySequence::Cut ) )
    {
        // nothing
    }
    else if( event->matches( QKeySequence::Paste ) && !src_cells.empty())
    {
        bool update = false;
        int  index  = 0;
        std::vector<int> selected;

        foreach (QTableWidgetItem *cell, selectedItems())
        {
             index %= src_cells.size();
             if (cell->column() == src_cells[index].second)
                {
                selected.push_back(cell->row());
                setItem(cell->row(), cell->column(), new QTableWidgetItem(src_cells[index++].first));
                update = true;
                }
        }

        resizeColumnsToContents();
        if (update) emit updateRequired(selected);
    }
    QTableWidget::keyPressEvent(event);
}
