#include "ClockProxy_Gp.h"
#include <algorithm>
#include <functional>




//---------------------------------------------------------------------------
//  X3ClockWrapper::X3ClockWrapper() -
//---------------------------------------------------------------------------

X3ClockWrapper::X3ClockWrapper(const string &atag, size_t target) :
    _pimpl(new ns1::ApplicationIo(target)), _tag(atag), _target(target)
{
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::~X3ClockWrapper() -
//---------------------------------------------------------------------------

X3ClockWrapper::~X3ClockWrapper()
{
    if (_pimpl) delete _pimpl;
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Open() -
//---------------------------------------------------------------------------

void X3ClockWrapper::Open()
{
    _pimpl->Open();
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Close() -
//---------------------------------------------------------------------------

void X3ClockWrapper::Close()
{
    _pimpl->Close();
}


//---------------------------------------------------------------------------
//  X3ClockWrapper::Start() -
//---------------------------------------------------------------------------

void X3ClockWrapper::Start()
{
    _pimpl->Start();
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Stop() -
//---------------------------------------------------------------------------

void X3ClockWrapper::Stop()
{
    _pimpl->Stop();
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Peek() -
//---------------------------------------------------------------------------

unsigned int X3ClockWrapper::Peek(const string &type, int offset)
{
    return _pimpl->Peek(type, offset);
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Poke() -
//---------------------------------------------------------------------------

void X3ClockWrapper::Poke(const string &type, int offset, unsigned int value)
{
    _pimpl->Poke(type, offset, value);
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Target() -
//---------------------------------------------------------------------------

void X3ClockWrapper::Target(int target)
{
    _target = target;
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::Target() -
//---------------------------------------------------------------------------

size_t X3ClockWrapper::Target() const
{
    return _target;
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::PpsArm() -
//---------------------------------------------------------------------------
void X3ClockWrapper::PpsArm()
{

}

//---------------------------------------------------------------------------
//  X3ClockWrapper::GpsEnable() -
//---------------------------------------------------------------------------
void X3ClockWrapper::GpsEnable(bool value)
{
    _pimpl->GpsEnable(value);
}


//---------------------------------------------------------------------------
//  X3ClockWrapper::GpsEnable() -
//---------------------------------------------------------------------------
void X3ClockWrapper::GpsTriggerSoon(int secs)
{
    _pimpl->GpsTriggerSoon(secs);
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::SoftwareTrigger() -
//---------------------------------------------------------------------------

void X3ClockWrapper::SoftwareTrigger(bool value)
{
    _pimpl->TriggerActivate(value);
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::PllInfo() -
//---------------------------------------------------------------------------

void X3ClockWrapper::PllInfo()
{
    _pimpl->DisplayPllData();
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::GetStatus() -
//---------------------------------------------------------------------------
TimerStatusInfo X3ClockWrapper::GetStatus()
{
    TimerStatusInfo temp;

    temp._name       = _pimpl->ModuleName();
    temp._rev        = _pimpl->ModuleRevision();
    temp._locked     = _pimpl->PllLockedStr();
    temp._striggered = _pimpl->TriggerStatus();
    temp._temp       = _pimpl->Temperature();
    temp._time       = _pimpl->GpsDateTime();
    temp._lat        = _pimpl->GpsLatitude();
    temp._longitude  = _pimpl->GpsLongitude();
    temp._sat        = _pimpl->GpsSatellites();

    return temp;
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::GetSettings() -
//---------------------------------------------------------------------------

void X3ClockWrapper::GetSettings(ClockGUISettings &s)
{
    s.ReferenceRate       = _pimpl->Settings.Vcxo.FrontPanelInRateMhz;
    s.RefSource               = _pimpl->Settings.InMux.Channel[0].Source;

    s.Frequency               = _pimpl->Settings.Pll.Frequency;
    s.SampleSource         = _pimpl->Settings.OutMux.Channel[0].Source;

    s.ExternalTrigger      = _pimpl->Settings.Trigger.ExternalTrigger;
    s.Framed                   = _pimpl->Settings.Trigger.Framed;
    s.FrameSize              = _pimpl->Settings.Trigger.FrameSize;
    s.PpsSrc                    = _pimpl->Settings.Trigger.PpsSource;
   s.TriggerType             =  _pimpl->Settings.Trigger.TriggerType;
   s.ExtClockSrc             = _pimpl->Settings.Pll.ExtClockSrc;
   s.TriggerInSec            = _pimpl->Settings.Trigger.TriggerInSec;
}

//---------------------------------------------------------------------------
//  X3ClockWrapper::SetSettings() -
//---------------------------------------------------------------------------

void X3ClockWrapper::SetSettings(const ClockGUISettings &s)
{
    _pimpl->Settings.Vcxo.FrontPanelInRateMhz =  s.ReferenceRate;
    _pimpl->Settings.InMux.Channel[0].Source  =  s.RefSource;
    _pimpl->Settings.InMux.Channel[1].Source  =  s.RefSource;
    _pimpl->Settings.InMux.Channel[2].Source  =  s.RefSource;
    _pimpl->Settings.InMux.Channel[3].Source  =  s.RefSource;
    _pimpl->Settings.InMux.Channel[4].Source  =  s.RefSource;

    _pimpl->Settings.Pll.Frequency            =  s.Frequency;
    _pimpl->Settings.OutMux.Channel[0].Source =  s.SampleSource;
    _pimpl->Settings.OutMux.Channel[1].Source =  s.SampleSource;
    _pimpl->Settings.OutMux.Channel[2].Source =  s.SampleSource;
    _pimpl->Settings.OutMux.Channel[3].Source =  s.SampleSource;

    _pimpl->Settings.Trigger.ExternalTrigger  =  s.ExternalTrigger;
    _pimpl->Settings.Trigger.Framed           =  s.Framed;
    _pimpl->Settings.Trigger.FrameSize        =  s.FrameSize;
    _pimpl->Settings.Trigger.PpsSource        =  s.PpsSrc;
    _pimpl->Settings.Trigger.TriggerType = s.TriggerType;
    _pimpl->Settings.Pll.ExtClockSrc = s.ExtClockSrc;
     _pimpl->Settings.Trigger.TriggerInSec = s.TriggerInSec;
}

// --

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::SbcDuoClockWrapper() -
//---------------------------------------------------------------------------

SbcDuoClockWrapper::SbcDuoClockWrapper(const string &atag, size_t target) :
    _pimpl(new ns2::ApplicationIo(0)), _tag(atag), _target(target)
{
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::~SbcDuoClockWrapper() -
//---------------------------------------------------------------------------

SbcDuoClockWrapper::~SbcDuoClockWrapper()
{
    if (_pimpl) delete _pimpl;
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Open() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::Open()
{
    _pimpl->Open();
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Close() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::Close()
{
    _pimpl->Close();
}


//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Start() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::Start()
{
    _pimpl->Start();
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Stop() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::Stop()
{
    _pimpl->Stop();
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Peek() -
//---------------------------------------------------------------------------

unsigned int SbcDuoClockWrapper::Peek(const string &type, int offset)
{
    return _pimpl->Peek(type, offset);
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Poke() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::Poke(const string &type, int offset, unsigned int value)
{
    _pimpl->Poke(type, offset, value);
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Target() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::Target(int target)
{
    _target = target;
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::Target() -
//---------------------------------------------------------------------------

size_t SbcDuoClockWrapper::Target() const
{
    return _target;
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::PpsArm() -
//---------------------------------------------------------------------------
void SbcDuoClockWrapper::PpsArm()
{
    _pimpl->PpsTriggerArm();
}
//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::GpsEnable() -
//---------------------------------------------------------------------------
void SbcDuoClockWrapper::GpsEnable(bool value)
{
    _pimpl->GpsEnable(value);
}


//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::GpsEnable() -
//---------------------------------------------------------------------------
void SbcDuoClockWrapper::GpsTriggerSoon(int /*secs*/)
{
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::SoftwareTrigger() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::SoftwareTrigger(bool value)
{
    _pimpl->SoftwareTrigger(value);
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::PllInfo() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::PllInfo()
{

}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::GetStatus() -
//---------------------------------------------------------------------------
TimerStatusInfo SbcDuoClockWrapper::GetStatus()
{
    TimerStatusInfo temp;

    temp._name       = _pimpl->ModuleName();
    temp._rev        = _pimpl->ModuleRevision();
    temp._locked     = _pimpl->PllLockedStr();
    temp._striggered = _pimpl->TriggerStatus();
    temp._temp       = _pimpl->Temperature();
    temp._time       = _pimpl->GpsDateTime();
    temp._lat        = _pimpl->GpsLatitude();
    temp._longitude  = _pimpl->GpsLongitude();
    temp._sat        = _pimpl->GpsSatellites();

    return temp;
}


//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::GetSettings() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::GetSettings(ClockGUISettings &s)
{
    s.ReferenceRate   = _pimpl->Settings.ReferenceRate;
    s.RefSource       = _pimpl->Settings.ReferenceClockSource;
    s.Frequency       = _pimpl->Settings.OutputFrequency;
    s.SampleSource    = _pimpl->Settings.SampleClockSource;
    s.ExternalTrigger = _pimpl->Settings.Trig.TriggerSource;
    s.Framed          = _pimpl->Settings.Trig.Framed;
    s.FrameSize       = _pimpl->Settings.Trig.FrameSize;
    s.PpsSrc             = _pimpl->Settings.Trig.PpsSource;
    s.ExtClockSrc = _pimpl->Settings.ExternalClockSource;
    s.TriggerType = _pimpl->Settings.Trig.TriggerType;
    s.TriggerInSec  = _pimpl->Settings.Trig.TriggerInSec;
}

//---------------------------------------------------------------------------
//  SbcDuoClockWrapper::SetSettings() -
//---------------------------------------------------------------------------

void SbcDuoClockWrapper::SetSettings(const ClockGUISettings &s)
{
    _pimpl->Settings.ReferenceRate          = s.ReferenceRate;
    _pimpl->Settings.ReferenceClockSource   = s.RefSource;
    _pimpl->Settings.OutputFrequency        = s.Frequency;
    _pimpl->Settings.SampleClockSource      = s.SampleSource;
    _pimpl->Settings.Trig.TriggerSource     = s.ExternalTrigger;
    _pimpl->Settings.Trig.Framed            = s.Framed;
    _pimpl->Settings.Trig.FrameSize         = s.FrameSize;
   _pimpl->Settings.Trig.PpsSource          = s.PpsSrc;
   _pimpl->Settings.ExternalClockSource =  s.ExtClockSrc;
    _pimpl->Settings.Trig.TriggerType = s.TriggerType;
     _pimpl->Settings.Trig.TriggerInSec = s.TriggerInSec ;
}

// --

//---------------------------------------------------------------------------
//  AtroposClockWrapper::AtroposClockWrapper() -
//---------------------------------------------------------------------------

AtroposClockWrapper::AtroposClockWrapper(const string &atag, size_t target) :
    _pimpl(new ns3::ApplicationIo(0)), _tag(atag), _target(target)
{
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::~AtroposClockWrapper() -
//---------------------------------------------------------------------------

AtroposClockWrapper::~AtroposClockWrapper()
{
    if (_pimpl) delete _pimpl;
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::Open() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::Open()
{
    _pimpl->Open();
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::Close() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::Close()
{
    _pimpl->Close();
}


//---------------------------------------------------------------------------
//  AtroposClockWrapper::Start() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::Start()
{
    _pimpl->StartTimer();
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::Stop() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::Stop()
{
    _pimpl->StopTimer();
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::Peek() -
//---------------------------------------------------------------------------

unsigned int AtroposClockWrapper::Peek(const string &/*type*/, int /*offset*/)
{
    return 0;
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::Poke() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::Poke(const string &/*type*/, int /*offset*/, unsigned int /*value*/)
{

}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::Target() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::Target(int target)
{
    _target = target;
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::PpsArm() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::PpsArm()
{

}
//---------------------------------------------------------------------------
//  AtroposClockWrapper::Target() -
//---------------------------------------------------------------------------

size_t AtroposClockWrapper::Target() const
{
    return _target;
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::GpsEnable() -
//---------------------------------------------------------------------------
void AtroposClockWrapper::GpsEnable(bool /*value*/)
{

}


//---------------------------------------------------------------------------
//  AtroposClockWrapper::GpsEnable() -
//---------------------------------------------------------------------------
void AtroposClockWrapper::GpsTriggerSoon(int /*secs*/)
{

}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::SoftwareTrigger() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::SoftwareTrigger(bool value)
{
    _pimpl->SoftwareTrigger(value);
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::PllInfo() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::PllInfo()
{
    _pimpl->ClockInfo();
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::GetStatus() -
//---------------------------------------------------------------------------
TimerStatusInfo AtroposClockWrapper::GetStatus()
{
    TimerStatusInfo temp;

    temp._name       = _pimpl->ModuleName();
    temp._rev        = _pimpl->ModuleRevision();
    temp._locked     = _pimpl->PllLockedStr();
    temp._striggered = _pimpl->TriggerStatus();
    temp._temp       = _pimpl->Temperature();
    temp._time       = string("N/A");
    temp._lat        = string("N/A");
    temp._longitude  = string("N/A");
    temp._sat        = 0;

    return temp;
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::GetSettings() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::GetSettings(ClockGUISettings &s)
{
    s.ReferenceRate   = _pimpl->Settings.ReferenceRate;
    s.RefSource       = _pimpl->Settings.ReferenceClockSource;
    s.Frequency       = _pimpl->Settings.Clk.SampleRate;
    s.SampleSource    = _pimpl->Settings.SampleClockSource;
    s.ExternalTrigger = _pimpl->Settings.Trig.ExternalTrigger;
    s.Framed          = _pimpl->Settings.Trig.Framed;
    s.FrameSize       = _pimpl->Settings.Trig.FrameSize;
    s.PpsSrc             = _pimpl->Settings.Trig.PpsSource;
	s.ExtClockSrc     = _pimpl->Settings.ExtClockSrcSelection;
    s.TriggerType     = _pimpl->Settings.Trig.TriggerType;
    s.TriggerInSec  = _pimpl->Settings.Trig.TriggerInSec;
}

//---------------------------------------------------------------------------
//  AtroposClockWrapper::SetSettings() -
//---------------------------------------------------------------------------

void AtroposClockWrapper::SetSettings(const ClockGUISettings &s)
{
    _pimpl->Settings.ReferenceRate          = s.ReferenceRate;
    _pimpl->Settings.ReferenceClockSource   = s.RefSource;
    _pimpl->Settings.Clk.SampleRate         = s.Frequency;
    _pimpl->Settings.SampleClockSource      = s.SampleSource;
    _pimpl->Settings.Trig.ExternalTrigger   = s.ExternalTrigger;
    _pimpl->Settings.Trig.Framed            = s.Framed;
    _pimpl->Settings.Trig.FrameSize         = s.FrameSize;
    _pimpl->Settings.Trig.PpsSource         = s.PpsSrc;
	_pimpl->Settings.ExtClockSrcSelection   = s.ExtClockSrc;
     _pimpl->Settings.Trig.TriggerType      = s.TriggerType;
     _pimpl->Settings.Trig.TriggerInSec = s.TriggerInSec;
}


// --


//---------------------------------------------------------------------------
//  ClockProxy::ClockProxy() -
//---------------------------------------------------------------------------

ClockProxy::ClockProxy()
{

}

//---------------------------------------------------------------------------
//  ClockProxy::~ClockProxy() -
//---------------------------------------------------------------------------

ClockProxy::~ClockProxy()
{
    std::for_each(_group.begin(), _group.end(),
                  std::bind2nd(std::mem_fun(&IClockCtrl::SetSettings), _cache));

    while (_group.size())
        {
        delete _group.back();
        _group.pop_back();
        }
}

//---------------------------------------------------------------------------
//  ClockProxy::Append() -
//---------------------------------------------------------------------------

void ClockProxy::Append(IClockCtrl *value)
{
    _group.push_back(value);

	for (size_t i = 0; i < _group.size(); i++) {
		_group[i]->GetSettings(_cache);
	}
}


//---------------------------------------------------------------------------
//  ClockProxy::Proxy() -
//---------------------------------------------------------------------------

IClockCtrl &ClockProxy::Proxy(unsigned int index)
{
    return *_group.at(index);
}


//---------------------------------------------------------------------------
//  ClockProxy::Open() -
//---------------------------------------------------------------------------

void ClockProxy::Open()
{
   std::for_each(_group.begin(), _group.end(),
                 std::bind2nd(std::mem_fun(&IClockCtrl::SetSettings), _cache));
   std::for_each(_group.begin(), _group.end(), std::mem_fun(&IClockCtrl::Open));
}

//---------------------------------------------------------------------------
//  ClockProxy::Close() -
//---------------------------------------------------------------------------

void ClockProxy::Close()
{
    std::for_each(_group.begin(), _group.end(), std::mem_fun(&IClockCtrl::Close));
}

//---------------------------------------------------------------------------
//  ClockProxy::Start() -
//---------------------------------------------------------------------------

void ClockProxy::Start()
{
    std::for_each(_group.begin(), _group.end(),
                  std::bind2nd(std::mem_fun(&IClockCtrl::SetSettings), _cache));
    std::for_each(_group.begin(), _group.end(), std::mem_fun(&IClockCtrl::Start));
}

//---------------------------------------------------------------------------
//  ClockProxy::Stop() -
//---------------------------------------------------------------------------

void ClockProxy::Stop()
{
    std::for_each(_group.begin(), _group.end(), std::mem_fun(&IClockCtrl::Stop));
}

//---------------------------------------------------------------------------
//  ClockProxy::PpsArm() -
//---------------------------------------------------------------------------

void ClockProxy::PpsArm()
{
    std::for_each(_group.begin(), _group.end(), std::mem_fun(&IClockCtrl::PpsArm));
}

//---------------------------------------------------------------------------
//  ClockProxy::GpsEnable() -
//---------------------------------------------------------------------------

void ClockProxy::GpsEnable(bool value)
{
    std::for_each(_group.begin(), _group.end(),
                  std::bind2nd(std::mem_fun(&IClockCtrl::GpsEnable), value));
}


//---------------------------------------------------------------------------
//  ClockProxy::IsGpsEnabled() -
//---------------------------------------------------------------------------

bool ClockProxy::IsGpsEnabled()
{
    // bool en = Mgr->ClockModules._cache.Gps.Enabled;
    return true; // TBD
}

//---------------------------------------------------------------------------
//  ClockProxy::Size() -
//---------------------------------------------------------------------------

size_t ClockProxy::Size()
{
    return _group.size();
}

//---------------------------------------------------------------------------
//  ClockProxy::PllInfo() -
//---------------------------------------------------------------------------

void ClockProxy::PllInfo()
{
    std::for_each(_group.begin(), _group.end(), std::mem_fun(&IClockCtrl::PllInfo));
}

//---------------------------------------------------------------------------
//  ClockProxy::GetStatus() -
//---------------------------------------------------------------------------

vector<TimerStatusInfo> ClockProxy::GetStatus()
{
    vector<TimerStatusInfo> temp(_group.size());
    transform(_group.begin(), _group.end(), temp.begin(), std::mem_fun(&IClockCtrl::GetStatus));

    return temp;
}

//---------------------------------------------------------------------------
//  ClockProxy::SoftwareTrigger() -
//---------------------------------------------------------------------------

void ClockProxy::SoftwareTrigger(bool value)
{
    std::for_each(_group.begin(), _group.end(),
                  std::bind2nd(std::mem_fun(&IClockCtrl::SoftwareTrigger), value));
}


//---------------------------------------------------------------------------
//  ClockProxy::IsSoftwareTriggerActivated() -
//---------------------------------------------------------------------------

bool ClockProxy::IsSoftwareTriggerActivated()
{
    // en = Mgr->ClockModules._cache.Trigger.SoftwareTriggerActivated;
    return true; // TBD
}

//---------------------------------------------------------------------------
//  ClockProxy::GpsTriggerSoon() -
//---------------------------------------------------------------------------

void ClockProxy::GpsTriggerSoon(int secs)
{
    std::for_each(_group.begin(), _group.end(),
                  std::bind2nd(std::mem_fun(&IClockCtrl::GpsTriggerSoon), secs));
}

//---------------------------------------------------------------------------
//  ClockProxy::isAtropos() -
//---------------------------------------------------------------------------

bool ClockProxy::isAtropos()
{
    //check if any clock modules that avalible is Atropos
   bool exist = false;
    for(unsigned int i = 0; i < _group.size(); i++)
         if(_group[i]->isAtropos())
            exist = true;

      return exist;
}

//---------------------------------------------------------------------------
//  ClockProxy::isDuo() -
//---------------------------------------------------------------------------

bool ClockProxy::isDuo()
{
    //check if any clock modules that avalible is Duo
    bool exist = false;
     for(unsigned int i = 0; i < _group.size(); i++)
          if(_group[i]->isDuo())
             exist = true;

       return exist;

}
//---------------------------------------------------------------------------
//  ClockProxy::isX3Timing() -
//---------------------------------------------------------------------------

bool ClockProxy::isX3Timing()
{
    //check if any clock modules that avalible is X3Timing
    bool exist = false;
     for(unsigned int i = 0; i < _group.size(); i++)
          if(_group[i]->isX3Timing())
             exist = true;

       return exist;

}
