#ifndef CLOCKPROXY_GP_H
#define CLOCKPROXY_GP_H

#include <string>
#include <vector>
#include <Application/IniSaver_App.h>


using std::string;
using std::vector;

using Innovative::IniSaver;


#include <X3ClockModule_Gp.h>
#include <SbcDuoClockModule_Gp.h>
#include <AtroposClockModule_Gp.h>

namespace ns1 = Innovative::X3_Clock;
namespace ns2 = Innovative::SbcDuo_Clock;
namespace ns3 = Innovative::Atropos_Clock;


//===========================================================================
//  CLASS TimerStatusInfo -- Transfer structure to/from TableWidget
//===========================================================================

struct TimerStatusInfo
{
    TimerStatusInfo()
    {}

    TimerStatusInfo(string name, string rev, string locked, string striggered, float temp,
                   int sat, string ti, string lat, string longitude) :
        _name(name), _rev(rev), _locked(locked), _striggered(striggered), _temp(temp),
    _sat(sat), _time(ti), _lat(lat), _longitude(longitude)
    { }

    string   _name;
    string   _rev;
    string   _locked;
    string   _striggered;
    float    _temp;
    int      _sat;
    string   _time;
    string   _lat;
    string   _longitude;
};

//===========================================================================
//  CLASS  ClockGUISettings -- Common GUI Settings
//===========================================================================

struct ClockGUISettings
{
    double ReferenceRate;
    int    RefSource;
    double Frequency;
    int    SampleSource;
    int    ExternalTrigger;
    int    Framed;
    int    FrameSize;
    int    TriggerType;
    int    ExtClockSrc;
    int    PpsSrc;
    int    TriggerInSec;
};

//===========================================================================
//  CLASS IClockCtrl --
//===========================================================================

struct IClockCtrl
{
    virtual ~IClockCtrl()
    {}

    virtual void         Open() = 0;
    virtual void         Close() = 0;
    virtual void         Start() = 0;
    virtual void         Stop() = 0;

    virtual void        PpsArm() = 0;
    virtual void        GpsEnable(bool value) = 0;
    virtual void        GpsTriggerSoon(int secs) = 0;
    virtual void        SoftwareTrigger(bool value) = 0;
    virtual TimerStatusInfo GetStatus() = 0;
    virtual void        PllInfo() = 0;

    virtual bool        isAtropos() = 0;
    virtual bool        isDuo() = 0;
    virtual bool        isX3Timing() = 0;

    virtual unsigned int Peek(const string &type, int offset) = 0;
    virtual void         Poke(const string &type, int offset, unsigned int value) = 0;
    // Properties
    virtual void        Target(int target) = 0;
    virtual size_t      Target() const = 0;
    // Translation interface
    virtual void        GetSettings(ClockGUISettings &s)       = 0;
    virtual void        SetSettings(const ClockGUISettings &s) = 0;
    //
    virtual vector<string> ReferenceSources() = 0;
    virtual vector<string> ClockSources() = 0;
    virtual vector<string> ExternalClockSources() = 0;
    virtual vector<string> ExternalTriggerSources() = 0;

    virtual bool GpsSupported() = 0;
};


//===========================================================================
//  CLASS AtroposClockWrapper --
//===========================================================================
class AtroposClockWrapper : public IClockCtrl
{
public:
    AtroposClockWrapper(const string &atag, size_t target);
    virtual ~AtroposClockWrapper();

    virtual void   Open();
    virtual void   Close();
    virtual void   Start();
    virtual void   Stop();

    virtual void   PpsArm();
    virtual void   GpsEnable(bool value);
    virtual void   GpsTriggerSoon(int secs);
    virtual void   SoftwareTrigger(bool value);
    virtual TimerStatusInfo GetStatus();
    virtual void PllInfo();

    virtual unsigned int Peek(const string &type, int offset);
    virtual void         Poke(const string &type, int offset, unsigned int value);
    // Properties
    virtual void   Target(int target);
    virtual size_t Target() const;

    virtual void   GetSettings(ClockGUISettings &s);
    virtual void   SetSettings(const ClockGUISettings &s);

    virtual bool        isAtropos() { return true;}
    virtual bool        isDuo()  { return false; }
    virtual bool        isX3Timing()  { return false; }

    virtual vector<string> ReferenceSources()
    {
        vector<string> temp;
        temp.push_back("External");
        temp.push_back("Internal");
        return temp;
    }

    virtual vector<string> ClockSources()
    {
        vector<string> temp;
        temp.push_back("External");
        temp.push_back("Internal");
        return temp;
    }

    virtual vector<string> ExternalClockSources()
    {
        vector<string> temp;
        temp.push_back("Front Panel");
        temp.push_back("P16");
        return temp;
    }

    virtual vector<string> ExternalTriggerSources()
    {
        vector<string> temp;
        temp.push_back("Front Panel");
        temp.push_back("P16");

        return temp;
    }

    virtual bool GpsSupported()
    {
        return false;
    }

private:
    ns3::ApplicationIo* const _pimpl;
    string                    _tag;
    size_t                    _target;

private:
    AtroposClockWrapper(const AtroposClockWrapper &other);
    AtroposClockWrapper &operator=(const AtroposClockWrapper &other);
};

//===========================================================================
//  CLASS SbcDuoClockWrapper --
//===========================================================================
class SbcDuoClockWrapper : public IClockCtrl
{
public:
    SbcDuoClockWrapper(const string &atag, size_t target);
    virtual ~SbcDuoClockWrapper();

    virtual void   Open();
    virtual void   Close();
    virtual void   Start();
    virtual void   Stop();

    virtual void   PpsArm();
    virtual void   GpsEnable(bool value);
    virtual void   GpsTriggerSoon(int secs);
    virtual void   SoftwareTrigger(bool value);
    virtual TimerStatusInfo GetStatus();
    virtual void PllInfo();

    virtual unsigned int Peek(const string &type, int offset);
    virtual void         Poke(const string &type, int offset, unsigned int value);
    // Properties
    virtual void   Target(int target);
    virtual size_t Target() const;

    virtual void   GetSettings(ClockGUISettings &s);
    virtual void   SetSettings(const ClockGUISettings &s);

    virtual bool        isAtropos() { return false; }
    virtual bool        isDuo()  { return true; }
    virtual bool        isX3Timing()  { return false; }

    virtual vector<string> ReferenceSources()
    {
        vector<string> temp;

        temp.push_back("External");
        temp.push_back("Internal");

        return temp;
    }

    virtual vector<string> ClockSources()
    {
        vector<string> temp;

        temp.push_back("External");
        temp.push_back("Internal");

        return temp;
    }

    virtual vector<string> ExternalClockSources()
    {
        vector<string> temp;
        temp.push_back("Front Panel");
        temp.push_back("Gps");

        return temp;
    }

    virtual vector<string> ExternalTriggerSources()
    {
        vector<string> temp;
        temp.push_back("Front Panel");
        temp.push_back("P16");
        return temp;
    }

    virtual bool GpsSupported()
    {
        return true;
    }

private:
    ns2::ApplicationIo* const _pimpl;
    string                    _tag;
    size_t                    _target;

private:
    SbcDuoClockWrapper(const SbcDuoClockWrapper &other);
    SbcDuoClockWrapper &operator=(const SbcDuoClockWrapper &other);
};


//===========================================================================
//  CLASS X3ClockWrapper --
//===========================================================================
class X3ClockWrapper : public IClockCtrl
{
public:
    X3ClockWrapper(const string &atag, size_t target);
    virtual ~X3ClockWrapper();

    virtual void   Open();
    virtual void   Close();
    virtual void   Start();
    virtual void   Stop();

    virtual void   PpsArm();
    virtual void   GpsEnable(bool value);
    virtual void   GpsTriggerSoon(int secs);
    virtual void   SoftwareTrigger(bool value);
    virtual TimerStatusInfo GetStatus();
    virtual void PllInfo();

    virtual unsigned int Peek(const string &type, int offset);
    virtual void         Poke(const string &type, int offset, unsigned int value);
    // Properties
    virtual void   Target(int target);
    virtual size_t Target() const;

    virtual void   GetSettings(ClockGUISettings &s);
    virtual void   SetSettings(const ClockGUISettings &s);

    virtual bool        isAtropos() { return false;}
    virtual bool        isDuo()  { return false; }
    virtual bool        isX3Timing()  { return true; }

    virtual vector<string> ReferenceSources()
    {
        vector<string> temp;
        temp.push_back("Internal");
        temp.push_back("Front Panel");
        temp.push_back("Gps");
        return temp;
    }

    virtual vector<string> ClockSources()
    {
        vector<string> temp;
        temp.push_back("Internal");
        temp.push_back("External");
        return temp;
    }

    virtual vector<string> ExternalClockSources()
    {
        vector<string> temp;
        temp.push_back("Front Panel");
        return temp;
    }

    virtual vector<string> ExternalTriggerSources()
    {
        vector<string> temp;
        temp.push_back("Front Panel");

        return temp;
    }

    virtual bool GpsSupported()
    {
        return true;
    }

private:
    ns1::ApplicationIo* const _pimpl;
    string                    _tag;
    size_t                    _target;

private:
    X3ClockWrapper(const X3ClockWrapper &other);
    X3ClockWrapper &operator=(const X3ClockWrapper &other);
};

//===========================================================================
//  CLASS ClockProxy --
//===========================================================================
class ClockProxy
{
public:
    ClockProxy();
    virtual ~ClockProxy();

    void Append(IClockCtrl *value);
    IClockCtrl &Proxy(unsigned int index);

    void Open();
    void Close();
    void Start();
    void Stop();

    size_t Size();
    void PllInfo();

    void PpsArm();
    void GpsEnable(bool value);
    bool IsGpsEnabled();
    void GpsTriggerSoon(int secs);
    void SoftwareTrigger(bool value);
    bool IsSoftwareTriggerActivated();
    bool        isAtropos();
   bool        isDuo();
     bool        isX3Timing();

    vector<TimerStatusInfo> GetStatus();

    ClockGUISettings    _cache;

private:
    vector<IClockCtrl*> _group;

private:
    ClockProxy(const ClockProxy &other);
    ClockProxy &operator=(const ClockProxy &other);
};





#endif // CLOCKPROXY_GP_H
