#include <CommonModule_Gp.h>
#include <SystemSupport_Mb.h>
#include <cmath>


namespace   Innovative
{
#ifdef __CLR_VER
#pragma warning(disable: 4355)
#pragma managed(push, off)
#endif

using namespace GpDsp_MemoryMap;

//===========================================================================
//  CLASS DspCommon_Module  -- DSPCommon Module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DspCommon_Module
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class DspCommon_Module
//---------------------------------------------------------------------------

DspCommon_Module::DspCommon_Module(GpDspMemoryExtension & GP)
    :  ComponentModuleBase(GP.WB_DspCommon, ewbDspCommon)
{
}

//===========================================================================
//  CLASS DspPowerMeterSM  -- Power Meter 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DspPowerMeterSM
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters PM_Sub_RegAddrs =
    {  mmPowerMeterInfo,      //  Version_Addr
       mmPowerMeterConfig,     //  Config_Addr
       mmPowerMeterReset
    };

static DspPowerMeterSM::MapRegisters PM_Core_RegAddrs =
    {
    mmPowerMeterAccPoints,  //  AccumulatePoints_Addr;
    mmPowerMeterSource,     //  SourceSelection_Addr;
    mmPowerMeterData,       //  Data_Addr;
    mmPowerMeterReady,      //  Ready_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class DspPowerMeterSM
//---------------------------------------------------------------------------

DspPowerMeterSM::DspPowerMeterSM(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_1D_Submodule(cbase, GP.WB_DspCommon, PM_Sub_RegAddrs, ewbDspCommon),
       AccumulatePointsReg(GP.WB_DspCommon, PM_Core_RegAddrs.AccumulatePoints_Addr),
       SourceSelectionReg(GP.WB_DspCommon, PM_Core_RegAddrs.SourceSelection_Addr),
       DataReg(GP.WB_DspCommon, PM_Core_RegAddrs.Data_Addr),
       ReadyReg(GP.WB_DspCommon, PM_Core_RegAddrs.Ready_Addr)//,
{
}

//---------------------------------------------------------------------------
//  DspPowerMeterSM::AccumulatePoints()
//---------------------------------------------------------------------------

void  DspPowerMeterSM::AccumulatePoints(unsigned int core, unsigned int power_of_2)
{
    SetIndex(core);
    AccumulatePointsReg.AccPoints.Value(power_of_2);
}

//---------------------------------------------------------------------------
//  DspPowerMeterSM::Source()
//---------------------------------------------------------------------------

void  DspPowerMeterSM::Source(unsigned int core, unsigned int source)
{
    SetIndex(core);
    SourceSelectionReg.Source.Value(source);
}

//---------------------------------------------------------------------------
//  DspPowerMeterSM::PowerData_dBm(unsigned int core) - return power value
//---------------------------------------------------------------------------

double  DspPowerMeterSM::PowerData_dBm(unsigned int core)
{
    SetIndex(core);
    WaitForReady();
    unsigned int data = DataReg.Value();
    return 10.0 * std::log10(static_cast<double>(data))-92.3296;
}

//---------------------------------------------------------------------------
//  DspPowerMeterSM::WaitForReady() -
//---------------------------------------------------------------------------

void  DspPowerMeterSM::WaitForReady()
{
    // we have already set core here
    // ...perform wait (how long?) - must bail out eventually
    Sleep(10);
}


//===========================================================================
//  CLASS DspOverflowSM  -- Overflow 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DspOverflowSM
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters Ov_Sub_RegAddrs =
    {  mmOverflowInfo,      //  Version_Addr
       mmOverflowConfig,    //  Config_Addr
       mmOverflowReset
    };

static DspOverflowSM::MapRegisters Ov_Core_RegAddrs =
    {
       mmOverflowData,  //  Data_Addr;
    };
//---------------------------------------------------------------------------
//  constructor for class DspOverflowSM
//---------------------------------------------------------------------------

DspOverflowSM::DspOverflowSM(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_1D_Submodule(cbase, GP.WB_DspCommon, Ov_Sub_RegAddrs, ewbDspCommon),
       DataReg(GP.WB_DspCommon, Ov_Core_RegAddrs.Data_Addr)
{
}

//---------------------------------------------------------------------------
//  DspPowerMeterSM::StatusOverflow() -
//---------------------------------------------------------------------------

unsigned int DspOverflowSM::OverflowStatus(unsigned int core)
{
    SetIndex(core);
    return DataReg.ovflo_data.Value();
}


//===========================================================================
//  CLASS DspToneGeneratorSM  -- ToneGenerator 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DspToneGeneratorSM
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters TG_Sub_RegAddrs =
    {  mmToneGenInfo,      //  Version_Addr
       mmToneGenConfig,    //  Config_Addr
       mmToneGenReset
    };

static DspToneGeneratorSM::MapRegisters TG_Core_RegAddrs =
    {
       mmToneGenFreq1,  //  Frequency1_Addr;
       mmToneGenPhase1, //  Offset1_Addr;
       mmToneGenGain1,  //  Gain1_Addr;
       mmToneGenFreq2,  //  Frequency2_Addr;
       mmToneGenPhase2, //  Offset2_Addr;
       mmToneGenGain2   //  Gain2_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class DspToneGeneratorSM
//---------------------------------------------------------------------------

DspToneGeneratorSM::DspToneGeneratorSM(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_1D_Submodule(cbase, GP.WB_DspCommon, TG_Sub_RegAddrs, ewbDspCommon),
       Frequency1Reg(GP.WB_DspCommon, TG_Core_RegAddrs.Frequency1_Addr),
       Phase1Reg(GP.WB_DspCommon, TG_Core_RegAddrs.Phase1_Addr),
       Gain1Reg(GP.WB_DspCommon, TG_Core_RegAddrs.Gain1_Addr),
       Frequency2Reg(GP.WB_DspCommon, TG_Core_RegAddrs.Frequency2_Addr),
       Phase2Reg(GP.WB_DspCommon, TG_Core_RegAddrs.Phase2_Addr),
       Gain2Reg(GP.WB_DspCommon, TG_Core_RegAddrs.Gain2_Addr)
{
}

//---------------------------------------------------------------------------
//  DspToneGeneratorSM::Frequency1() -
//---------------------------------------------------------------------------

void DspToneGeneratorSM::Frequency1(unsigned int core, double freq, double srate)
{
    unsigned long Multiplier = 2147483648u;
    unsigned int pinc = static_cast<unsigned int>((freq * Multiplier * 2) / srate);
    SetIndex(core);
    Frequency1Reg.Value(pinc);
}

//---------------------------------------------------------------------------
//  DspToneGeneratorSM::Phase1() -
//---------------------------------------------------------------------------

void DspToneGeneratorSM::Phase1(unsigned int core, double value)
{
    unsigned long Multiplier = 2147483648u;
    SetIndex(core);
    unsigned int poff =  static_cast<unsigned int>((value * Multiplier * 2) / 360);
    Phase1Reg.Value(poff);
}

//---------------------------------------------------------------------------
//  DspToneGeneratorSM::Gain1() -
//---------------------------------------------------------------------------

void DspToneGeneratorSM::Gain1(unsigned int core, double value)
{
    const unsigned int Multiplier = 32767;
    SetIndex(core);
    unsigned int gain =  static_cast<unsigned int>((value * Multiplier));
    Gain1Reg.Value(gain);
}

//---------------------------------------------------------------------------
//  DspToneGeneratorSM::Frequency2() -
//---------------------------------------------------------------------------

void DspToneGeneratorSM::Frequency2(unsigned int core, double /*freq*/, double /*srate*/)
{
    SetIndex(core);
}

//---------------------------------------------------------------------------
//  DspToneGeneratorSM::Phase2() -
//---------------------------------------------------------------------------

void DspToneGeneratorSM::Phase2(unsigned int core, double /*value*/)
{
    SetIndex(core);
}

//---------------------------------------------------------------------------
//  DspToneGeneratorSM::Gain2() -
//---------------------------------------------------------------------------

void DspToneGeneratorSM::Gain2(unsigned int core, double /*value*/)
{
    SetIndex(core);
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative



