#ifndef CommonModule_GpH
#define CommonModule_GpH

#include <Components_Gp.h>

#include <GpDspMemoryStringMap_Gp.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS DspCommon_Module  -- DSPCommon Module shared features
//===========================================================================

class DspCommon_Module : public ComponentModuleBase
{
public:
    DspCommon_Module(GpDspMemoryExtension & GP);
    virtual ~DspCommon_Module() {}

};

//===========================================================================
//  CLASS DspPowerMeterSM  -- Power Meter 'submodule'
//===========================================================================

class DspPowerMeterSM : public GpDsp_1D_Submodule   //  public IPowerMeter (?)  interface ftns...
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspPowerMeterSM::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   AccumulatePoints_Addr;
        unsigned int   SourceSelection_Addr;
        unsigned int   Data_Addr;
        unsigned int   Ready_Addr;
    };

    DspPowerMeterSM(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DspPowerMeterSM()  {}

    void  AccumulatePoints(unsigned int core, unsigned int power_of_2);
    void  Source(unsigned int core, unsigned int source);

    double  PowerData_dBm(unsigned int core);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspPowerMeterSM::AccumulatePointsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class AccumulatePointsRegister : public DspCommonShadowRegister
    {
        typedef DspCommonShadowRegister inherited;

    public:
        RegisterBitGroup      AccPoints;

        AccumulatePointsRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              AccPoints(*this, 0, 5)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspPowerMeterSM::SourceSelectionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SourceSelectionRegister : public DspCommonShadowRegister
    {
        typedef DspCommonShadowRegister inherited;

    public:
        RegisterBitGroup      Source;

        SourceSelectionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Source(*this, 0, 3)
            {}
    };

private:
    AccumulatePointsRegister  AccumulatePointsReg;
    SourceSelectionRegister   SourceSelectionReg;
    Register                  DataReg;
    Register                  ReadyReg;

    void  WaitForReady();
};

//===========================================================================
//  CLASS DspOverflowSM  -- Overflow 'submodule'
//===========================================================================

class DspOverflowSM : public GpDsp_1D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspOverflowSM::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Data_Addr;
    };

    DspOverflowSM(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DspOverflowSM()  {}

    unsigned int OverflowStatus(unsigned int core);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspOverflowSM::DataRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class DataRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      ovflo_data;

        DataRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ovflo_data(*this, 0, 8)
            {}
    };


private:
    DataRegister            DataReg;
};



//===========================================================================
//  CLASS DspToneGeneratorSM  -- ToneGenerator 'submodule'
//===========================================================================

class DspToneGeneratorSM : public GpDsp_1D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspToneGeneratorSM::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Frequency1_Addr;
        unsigned int   Phase1_Addr;
        unsigned int   Gain1_Addr;
        unsigned int   Frequency2_Addr;
        unsigned int   Phase2_Addr;
        unsigned int   Gain2_Addr;

    };

    DspToneGeneratorSM(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DspToneGeneratorSM()  {}

    void Frequency1(unsigned int core, double freq, double srate);
    void Phase1(unsigned int core, double value);
    void Gain1(unsigned int core, double value);
    void Frequency2(unsigned int core,  double freq, double srate);
    void Phase2(unsigned int core, double value);
    void Gain2(unsigned int core, double value);


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DspToneGeneratorSM::GainRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class GainRegister : public DspCommonShadowRegister
    {
        typedef DspCommonShadowRegister inherited;

    public:
        RegisterBitGroup      tone_gainx;

        GainRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              tone_gainx(*this, 0, 16)
            {}
    };


private:
    DspCommonShadowRegister Frequency1Reg;
    DspCommonShadowRegister Phase1Reg;
    GainRegister            Gain1Reg;
    DspCommonShadowRegister Frequency2Reg;
    DspCommonShadowRegister Phase2Reg;
    GainRegister            Gain2Reg;

};


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

#endif // CommonModule_GpH
