#ifndef CommonRegisters_GpH
#define CommonRegisters_GpH

#include <GpDspMemoryMap_Gp.h>
#include <HardwareRegister_Mb.h>

#include <GpDspMemoryStringMap_Gp.h>



namespace Innovative
{
#ifdef __CLR_VER
#pragma warning(disable: 4355)
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS GpDspMemoryExtension -- Template to extend a board and define WBs
//==============================================================================

class GpDspMemoryExtension
{
public:

    GpDspMemoryExtension(IAddressingSpace & logic_space)
        :  WB_Custom           (logic_space,  GpDsp_MemoryMap::wbCustom),
           WB_DspCommon        (logic_space,  GpDsp_MemoryMap::wbDspCommon),
           WB_Ddc              (logic_space,  GpDsp_MemoryMap::wbDdc),
           WB_Duc              (logic_space,  GpDsp_MemoryMap::wbDuc),
           WB_SpectrumAnalysis (logic_space,  GpDsp_MemoryMap::wbSpectrumAnalysis),
           WB_Beamforming      (logic_space,  GpDsp_MemoryMap::wbBeamforming),
           WB_PSK              (logic_space,  GpDsp_MemoryMap::wbPSK),
           WB_FSK              (logic_space,  GpDsp_MemoryMap::wbFSK)
          {}
      virtual ~GpDspMemoryExtension()
          {}

      WishboneBusSpace  WB_Custom;
      WishboneBusSpace  WB_DspCommon;
      WishboneBusSpace  WB_Ddc;
      WishboneBusSpace  WB_Duc;
      WishboneBusSpace  WB_SpectrumAnalysis;
      WishboneBusSpace  WB_Beamforming;
      WishboneBusSpace  WB_PSK;
      WishboneBusSpace  WB_FSK;

};

//==============================================================================
//  CLASS GpDspMemoryExtension_BindTo<T> -- Template  define WBs
//==============================================================================

template <class T>
class GpDspMemoryExtension_BindTo : public BaseboardExtension<T>, public GpDspMemoryExtension
{
public:
    typedef BaseboardExtension<T> inherited;

    GpDspMemoryExtension_BindTo(T& owner)
        :  inherited(owner), GpDspMemoryExtension(inherited::LogicMemory)
        {}
    virtual ~GpDspMemoryExtension_BindTo()
        {}

};

//==============================================================================
//  CLASS ProjectVersionRegister
//==============================================================================
//  Definition of a common register in the map
//  ...RxCoreType Field Defs
//      0: unspecified
//      1: IP-DDC
//      2: IP-DDC-2GSPS
//      3: IP-UWBDDC
//      4: IP-DDC-500MSPS
//      5-7: unused
//      8: IP-DDC128 (Channelizer DDC with 128 channels)
//      9: IP-DDC16 (Channelizer DDC with 16 channels)
//
//  ...TxCoreType Field Defs
//      0: unspecified
//      4: IP-DUC-500MSPS
//
//  ...Additions
//      0: none
//      1: IP-FFT32K-SER
//      2: IP-FFT32K
//      3: PSK Demodulator

class ProjectVersionRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup      RxCores;
    RegisterBitGroup      RxCoreType;
    RegisterBitGroup      TxCores;
    RegisterBitGroup      TxCoreType;
    RegisterBitGroup      Additions;

    ProjectVersionRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           RxCores(*this, 0, 4),
           RxCoreType(*this, 4, 8),
           TxCores(*this, 12, 4),
           TxCoreType(*this, 16, 8),
           Additions(*this, 24, 8)
        {}
};

//==============================================================================
//  CLASS ProjectSubVersionRegister
//==============================================================================
//  Definition of a common register in the map

class ProjectSubVersionRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup      Revision;
    RegisterBitGroup      Revision_Lo;
    RegisterBitGroup      Revision_Hi;
    RegisterBitGroup      SubRevInternal;
    RegisterBitGroup      EngineerID;

    ProjectSubVersionRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           Revision(*this, 0, 16),
           Revision_Lo(*this, 0, 8),  // Yes, these overlap Revision
           Revision_Hi(*this, 8, 8),
           SubRevInternal(*this, 16, 8),
           EngineerID(*this, 24, 8)
        {}

};

//==============================================================================
//  CLASS ProjectCoreEnableRegister
//==============================================================================
//  Definition of a common register in the map

class ProjectCoreEnableRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup      CoreEnabled;

    ProjectCoreEnableRegister(IAddressingSpace &space, int offset)
        : inherited(space, offset),
          CoreEnabled(*this, 0, 16)
        {}
};

//==============================================================================
//  CLASS ComponentModuleEnableRegister
//==============================================================================
//  Definition of a common register in the map

class ComponentModuleEnableRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup      ModuleEnabled;

    ComponentModuleEnableRegister(IAddressingSpace &space, int offset)
        : inherited(space, offset),
          ModuleEnabled(*this, 0, 16)
        {}
};

//==============================================================================
//  CLASS ComponentCommandTriggerRegister
//==============================================================================
//  Definition of a common register in the map

class ComponentCommandTriggerRegister : public CommonShadowRegister
{
    typedef CommonShadowRegister inherited;

public:
    RegisterBit      Trigger;

    ComponentCommandTriggerRegister(IAddressingSpace &space, int offset, WishboneType awtype)
        : inherited(space, offset, awtype),
          Trigger(*this, 0)
        {}
};

//==============================================================================
//  CLASS ComponentCoreIndexRegister
//==============================================================================
//  Definition of a common register in the map

class ComponentCoreIndexRegister : public CommonShadowRegister
{
    typedef CommonShadowRegister inherited;

public:
    RegisterBitGroup      CoreIndex;

    ComponentCoreIndexRegister(IAddressingSpace &space, int offset, WishboneType awtype)
        : inherited(space, offset, awtype),
          CoreIndex(*this, 0, 12)
        {}
};

//==============================================================================
//  CLASS ComponentParentIndexRegister
//==============================================================================
//  Definition of a common register in the map

class ComponentParentIndexRegister : public CommonShadowRegister
{
    typedef CommonShadowRegister inherited;

public:
    RegisterBitGroup      ParentIndex;

    ComponentParentIndexRegister(IAddressingSpace &space, int offset, WishboneType awtype)
        : inherited(space, offset, awtype),
          ParentIndex(*this, 0, 12)
        {}
};

//==============================================================================
//  CLASS SubModuleVersionRegister
//==============================================================================
//  Definition of a common register in the map

class SubModuleVersionRegister : public Register
{
    typedef Register inherited;

public:
    RegisterBitGroup      CoreCount;
    RegisterBitGroup      ChannelPerCore;
    RegisterBitGroup      Version;

    SubModuleVersionRegister(IAddressingSpace &space, int offset)
        : inherited(space, offset),
          CoreCount(*this, 0, 6),
          ChannelPerCore(*this, 6, 10),
          Version(*this, 16, 16)
        {}
};

//==============================================================================
//  CLASS SubModuleConfigRegister
//==============================================================================
//  Definition of a common register in the map

class SubModuleConfigRegister : public Register
{
    typedef Register inherited;

public:
    enum SampleWidthVals { sw16, sw08, sw24, sw32 };

    RegisterBitGroup   SampleWidth;
    RegisterBit        Unsigned;
    RegisterBit        NonInterleaved;
    RegisterBit        ComplexInput;
    RegisterBit        OutputFormat;
    RegisterBitGroup   Architecture;

    SubModuleConfigRegister(IAddressingSpace &space, int offset)
        :  inherited(space, offset),
           SampleWidth(*this, 0, 4),
           Unsigned(*this, 4),
           NonInterleaved(*this, 5),
           ComplexInput(*this, 6),
           OutputFormat(*this, 7),
           Architecture(*this, 8, 4)
        {}
};

//==============================================================================
//  CLASS SubModuleResetRegister
//==============================================================================
//  Definition of a common register in the map

class SubModuleResetRegister : public CommonShadowRegister
{
    typedef CommonShadowRegister inherited;

public:
    RegisterBit      Reset;

    SubModuleResetRegister(IAddressingSpace &space, int offset, WishboneType awtype)
        : inherited(space, offset, awtype),
          Reset(*this, 0)
        {}
};



#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // end namespace Innovative


#endif // CommonRegisters_GpH
