#include <DdcModule_Gp.h>

namespace   Innovative
{
#ifdef __CLR_VER
#pragma warning(disable: 4355)
#pragma managed(push, off)
#endif

using namespace GpDsp_MemoryMap;

//===========================================================================
//  CLASS Ddc_Module  -- DDC Module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Ddc_Module
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class DspCommon_Module
//---------------------------------------------------------------------------

Ddc_Module::Ddc_Module(GpDspMemoryExtension & GP)
    :  DdcModuleBase(GP.WB_Ddc, ewbDdc)
{
}

//===========================================================================
//  CLASS DdcPeripheralSM  -- Offset 0x10
//===========================================================================
//===========================================================================
//  CLASS DdcPeripheral  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DdcPeripheral
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DdcPeripheral_Sub_RegAddrs =
    {  mmDdcPeriphInfo,      //  Version_Addr
       mmDdcPeriphConfig,    //  Config_Addr
       mmDdcPeriphReset
    };

static DdcPeripheralSM::MapRegisters Periph_Core_RegAddrs =
    {
    mmDdcPeriphBypass,        //  Bypass_Addr;
    mmDdcPeriphSource,        //  SourceSelection_Addr;
    mmDdcPeriphSpectralInv,   //  SpectrumInv_Addr;
    mmDdcPeriphFilterSel1,    //  FilterSelect1_Addr;
    mmDdcPeriphFilterSel2     //  FilterSelect2_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class DdcPeripheral
//---------------------------------------------------------------------------

DdcPeripheralSM::DdcPeripheralSM(GpDspMemoryExtension & GP, DdcModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Ddc, DdcPeripheral_Sub_RegAddrs, ewbDdc),
       Base(cbase),
       BypassReg(GP.WB_Ddc, Periph_Core_RegAddrs.Bypass_Addr),
       SourceSelectionReg(GP.WB_Ddc, Periph_Core_RegAddrs.SourceSelection_Addr),
       SpectrumInversionReg(GP.WB_Ddc, Periph_Core_RegAddrs.SpectrumInv_Addr),
       Filter1_SelectReg(GP.WB_Ddc, Periph_Core_RegAddrs.FilterSelect1_Addr),
       Filter2_SelectReg(GP.WB_Ddc, Periph_Core_RegAddrs.FilterSelect2_Addr)//,
{
}

//---------------------------------------------------------------------------
//  DdcPeripheralSM::BypassCore() -
//---------------------------------------------------------------------------

void DdcPeripheralSM::BypassCore(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    BypassReg.Enabled.Value(value);
}

//---------------------------------------------------------------------------
//  DdcPeripheralSM::Source()
//---------------------------------------------------------------------------

void  DdcPeripheralSM::Source(unsigned int parent, unsigned int source)
{
    SetIndex(parent, 0);
    SourceSelectionReg.Source.Value(source);
}

//---------------------------------------------------------------------------
//  DdcPeripheralSM::EnableSpectrumInversion() -
//---------------------------------------------------------------------------

void DdcPeripheralSM::EnableSpectrumInversion(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    SpectrumInversionReg.Enabled.Value(value);
}

//---------------------------------------------------------------------------
//  DdcPeripheralSM::Filter1_Select()
//---------------------------------------------------------------------------

void  DdcPeripheralSM::Filter1_Select(unsigned int parent, unsigned int source)
{
    SetIndex(parent, 0);
    Filter1_SelectReg.Select.Value(source);
}

//---------------------------------------------------------------------------
//  DdcPeripheralSM::Filter2_Select()
//---------------------------------------------------------------------------

void  DdcPeripheralSM::Filter2_Select(unsigned int parent, unsigned int source)
{
    SetIndex(parent, 0);
    Filter2_SelectReg.Select.Value(source);
}

//---------------------------------------------------------------------------
//  DdcPeripheralSM::Decimation()
//---------------------------------------------------------------------------

void  DdcPeripheralSM::Decimation(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    Base.Decimation(value);
}

//===========================================================================
//  CLASS DdcTuner  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DdcTuner
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DdcTuner_Sub_RegAddrs =
    {  mmDdcTunerInfo,      //  Version_Addr
       mmDdcTunerConfig,    //  Config_Addr
       mmDdcTunerReset
    };

static DdcTuner::MapRegisters DdcTuner_Core_RegAddrs =
    {
    mmDdcTunerFrequency,    //  Frequency_Addr;
    mmDdcTunerPhaseOfs,     //  PhaseOfs_Addr;
    mmDdcTunerNumFreq,
    mmDdcTunerProgEn,
    mmDdcTunerSchAddr,
    mmDdcTunerSchData,
    mmDdcTunerSchDout
    };

//---------------------------------------------------------------------------
//  constructor for class DdcTuner
//---------------------------------------------------------------------------

DdcTuner::DdcTuner(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Ddc, DdcTuner_Sub_RegAddrs, ewbDdc),
       FrequencyReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.Frequency_Addr),
       PhaseOffsetReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.Phase_Addr),
       TunerScheduleFreqsReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.TuneFreqs_Addr),
       TunerScheduleProgEnableReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.TuneEnable_Addr),
       TunerScheduleAddrReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.TuneAddr_Addr),
       TunerScheduleDataReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.TuneData_Addr),
       TunerScheduleDataOutReg(GP.WB_Ddc, DdcTuner_Core_RegAddrs.TuneDataOut_Addr)
{
}

//---------------------------------------------------------------------------
//  DdcTuner::Frequency() -
//---------------------------------------------------------------------------

void DdcTuner::Frequency(unsigned int parent, double freq, double srate)
{
    unsigned long long Multiplier = 1LL << 32;
    unsigned int pinc = (freq / srate) * Multiplier + .5;
    SetIndex(parent, 0);
    FrequencyReg.Value(pinc);
}

//---------------------------------------------------------------------------
//  DdcTuner::ModFrequency() -
//---------------------------------------------------------------------------

void DdcTuner::ModFrequency(unsigned int parent, double freq, double /*srate*/)
{
    unsigned long long Multiplier = 1LL << 32;
    unsigned int pinc = (freq / 20) * Multiplier + .5;
    SetIndex(parent, 1);
    FrequencyReg.Value(pinc);
}

//---------------------------------------------------------------------------
//  DdcTuner::PhaseOffset() -
//---------------------------------------------------------------------------

void DdcTuner::PhaseOffset(unsigned int parent, double value)
{
    unsigned long long Multiplier = 1LL << 32;
    SetIndex(parent, 0);
    unsigned int poff = (value / 360) * Multiplier + .5;
    PhaseOffsetReg.Value(poff);
}

//---------------------------------------------------------------------------
//  DdcTuner::TuningFrequencies() -
//---------------------------------------------------------------------------

void DdcTuner::TuningFrequencies(unsigned int parent, const vector<double> &freqs, double srate)
{
    unsigned long long Multiplier = 1LL << 32;

    SetIndex(parent, 0);
    TunerScheduleProgEnableReg.Value(true);
    TunerScheduleFreqsReg.num_tuning_freqs.Value(freqs.size());
    TunerScheduleFreqsReg.en_tuning_sched.Value(true);
    TunerScheduleFreqsReg.Apply();

    for (size_t i = 0; i < freqs.size(); i++) {
        unsigned int pinc = (freqs[i] / srate) * Multiplier + .5;
        TunerScheduleAddrReg.Value(i);
        TunerScheduleDataReg.Value(pinc);
    }
    TunerScheduleProgEnableReg.Value(false);
}

//---------------------------------------------------------------------------
//  DdcTuner::TuningEnable() -
//---------------------------------------------------------------------------

void DdcTuner::TuningEnable(unsigned int parent, bool en)
{
    SetIndex(parent, 0);
    TunerScheduleFreqsReg.en_tuning_sched.Value(en);
    TunerScheduleFreqsReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcTuner::TuningDataOut() -
//---------------------------------------------------------------------------

unsigned int DdcTuner::TuningDataOut(unsigned int parent)
{
    SetIndex(parent, 0);
    return TunerScheduleDataOutReg.Value();
}

//===========================================================================
//  CLASS DdcPolyPhaseFilter  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DdcPolyPhaseFilter
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DdcPolyPhase_Sub_RegAddrs =
    {  mmDdcPolyPhaseInfo,      //  Version_Addr
       mmDdcPolyPhaseConfig,    //  Config_Addr
       mmDdcPolyPhaseReset
    };

static DdcPolyPhaseFilter::MapRegisters DdcPolyPhase_Core_RegAddrs =
    {
    mmDdcPolyPhaseCoeffs,       //  Coeffs_Addr;
    mmDdcPolyPhaseBStart        //  BStart_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class DdcPolyPhaseFilter
//---------------------------------------------------------------------------

DdcPolyPhaseFilter::DdcPolyPhaseFilter(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Ddc, DdcPolyPhase_Sub_RegAddrs, ewbDdc),
      CoeffsReg(GP.WB_Ddc, DdcPolyPhase_Core_RegAddrs.Coeffs_Addr),
      BStartReg(GP.WB_Ddc, DdcPolyPhase_Core_RegAddrs.BStart_Addr)
{
}

//---------------------------------------------------------------------------
//  DdcPolyPhaseFilter::BStart() -
//---------------------------------------------------------------------------

void DdcPolyPhaseFilter::BStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    BStartReg.ppf_shift_num.Value(num);
    BStartReg.ppf_shift_dir.Value(dir ? true : false);

    BStartReg.Apply();
}
//---------------------------------------------------------------------------
//  DdcPolyPhaseFilter::Coeffs() -
//---------------------------------------------------------------------------

void DdcPolyPhaseFilter::Coeffs(unsigned int parent, unsigned int id, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    CoeffsReg.ppf_coef.Value(cfs.size());
    // id
    CoeffsReg.ppf_coef.Value(id);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        CoeffsReg.ppf_coef.Value(cfs[i]);
}

//===========================================================================
//  CLASS DdcLowPassFilter  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DdcLowPassFilter
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DdcLowPass_Sub_RegAddrs =
    {  mmDdcLowpassInfo,      //  Version_Addr
       mmDdcLowpassConfig,    //  Config_Addr
       mmDdcLowpassReset
    };

static DdcLowPassFilter::MapRegisters DdcLowPass_Core_RegAddrs =
    {   mmDdcLowpassCicRate,
        mmDdcLowpassCicLsb,
        mmDdcLowpassCfirCoeff,
        mmDdcLowpassCfirBStart,
        mmDdcLowpassPfirCoeff,
        mmDdcLowpassPfirBStart,
        mmDdcLowpassFir1Coeff,
        mmDdcLowpassFir1BStart,
        mmDdcLowpassFir2Coeff,
        mmDdcLowpassFir2BStart
    };

//---------------------------------------------------------------------------
//  constructor for class DdcLowPassFilter
//---------------------------------------------------------------------------

DdcLowPassFilter::DdcLowPassFilter(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Ddc, DdcLowPass_Sub_RegAddrs, ewbDdc),
      CicRateReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.CicRate_Addr),
      CicBStartReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.CicBStart_Addr),
      CFirCoeffsReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.CFirCoeff_Addr),
      CFirBStartReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.CFirBStart_Addr),
      PFirCoeffsReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.PFirCoeff_Addr),
      PFirBStartReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.PFirBStart_Addr),
      FirlCoeffsReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.FirlCoeff_Addr),
      FirlBStartReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.FirlBStart_Addr),
      Fir2CoeffsReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.Fir2Coeff_Addr),
      Fir2BStartReg(GP.WB_Ddc, DdcLowPass_Core_RegAddrs.Fir2BStart_Addr)
    {
    }

//---------------------------------------------------------------------------
//  DdcLowPassFilter::CicRate() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::CicRate(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    CicRateReg.cic_rate.Value(value);
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::CicBStart() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::CicBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    CicBStartReg.ppf_shift_num.Value(num);
    CicBStartReg.ppf_shift_dir.Value(dir ? true : false);

    CicBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::CFirCoeffs() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::CFirCoeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    CFirCoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    CFirCoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        CFirCoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::CFirBStart() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::CFirBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    CFirBStartReg.ppf_shift_num.Value(num);
    CFirBStartReg.ppf_shift_dir.Value(dir ? true : false );

    CFirBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::PFirCoeffs() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::PFirCoeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    PFirCoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    PFirCoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        PFirCoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::PFirBStart() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::PFirBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    PFirBStartReg.ppf_shift_num.Value(num);
    PFirBStartReg.ppf_shift_dir.Value(dir ? true : false);

    PFirBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::FirlCoeffs() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::FirlCoeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    FirlCoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    FirlCoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        FirlCoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::FirlBStart() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::FirlBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    FirlBStartReg.ppf_shift_num.Value(num);
    FirlBStartReg.ppf_shift_dir.Value(dir ? true : false );

    FirlBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::Fir2Coeffs() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::Fir2Coeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    Fir2CoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    Fir2CoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        Fir2CoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DdcLowPassFilter::Fir2BStart() -
//---------------------------------------------------------------------------

void DdcLowPassFilter::Fir2BStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    Fir2BStartReg.ppf_shift_num.Value(num);
    Fir2BStartReg.ppf_shift_dir.Value(dir ? true : false );

    Fir2BStartReg.Apply();
}

//===========================================================================
//  CLASS DdcVitaFramer  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DdcVitaFramer
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DdcVitaFramer_Sub_RegAddrs =
    {  mmDdcVitaInfo,      //  Version_Addr
       mmDdcVitaConfig,    //  Config_Addr
       mmDdcVitaReset      //  Reset_Addr;
    };

static DdcVitaFramer::MapRegisters DdcVitaFramer_Core_RegAddrs =
    {   mmDdcVitaEnable,
        mmDdcVitaSize,
        mmDdcVitaSid,
        mmDdcVitaFrmMode,
        mmDdcVitaFrmSize,
        mmDdcVitaFrmCount,
        mmDdcVitaFrmDelay,
        mmDdcVitaFrmReEnable,
    };

//---------------------------------------------------------------------------
//  constructor for class DdcVitaFramer
//---------------------------------------------------------------------------

DdcVitaFramer::DdcVitaFramer(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Ddc, DdcVitaFramer_Sub_RegAddrs, ewbDdc),
       EnableReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.Enable_Addr),
       SizeReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.Size_Addr),
       StreamIDReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.StreamID_Addr),
       FrameModeReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.FrameMode_Addr),
       FrameSizeReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.FrameSize_Addr),
       FrameCountReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.FrameCount_Addr),
       FrameDelayReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.FrameDelay_Addr),
       FrameReEnableReg(GP.WB_Ddc, DdcVitaFramer_Core_RegAddrs.FrameReEneable_Addr)
{
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::Enable() -
//---------------------------------------------------------------------------

void DdcVitaFramer::Enable(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    EnableReg.vframer_enable.Value(value);
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::Size() -
//---------------------------------------------------------------------------

void DdcVitaFramer::Size(unsigned int parent, unsigned int size)
{
    SetIndex(parent, 0);
    SizeReg.vframer_size.Value(size);
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::Size() -
//---------------------------------------------------------------------------

unsigned int DdcVitaFramer::Size(unsigned int parent)
{
    SetIndex(parent, 0);
    return SizeReg.vframer_size.Value();
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::Identity() -
//---------------------------------------------------------------------------

void DdcVitaFramer::Identity(unsigned int parent, unsigned int did, unsigned int sid)
{
    SetIndex(parent, 0);
    StreamIDReg.vframer_stream_id.Value(sid);
    StreamIDReg.vframer_dest_id.Value(did);
    StreamIDReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::FrameMode() -
//---------------------------------------------------------------------------

void DdcVitaFramer::FrameMode(unsigned int parent, unsigned int mode, bool loc, unsigned int src)
{
    SetIndex(parent, 0);
    FrameModeReg.frame_mode.Value(mode);
    FrameModeReg.trig_loc.Value(loc);
    FrameModeReg.trig_src.Value(src);
    FrameModeReg.Apply();
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::FrameSize() -
//---------------------------------------------------------------------------

void DdcVitaFramer::FrameSize(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    FrameSizeReg.Value(value);
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::FrameCount() -
//---------------------------------------------------------------------------

unsigned int DdcVitaFramer::FrameCount(unsigned int parent)
{
    SetIndex(parent, 0);
    return FrameCountReg.Value();
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::FrameDelay() -
//---------------------------------------------------------------------------

void DdcVitaFramer::FrameDelay(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    FrameDelayReg.Value(value);
}

//---------------------------------------------------------------------------
//  DdcVitaFramer::ReEnable() -
//---------------------------------------------------------------------------

void DdcVitaFramer::ReEnable(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    FrameReEnableReg.Value(value);
}

//===========================================================================
//  CLASS DdcChannelizer  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DdcChannelizer
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DdcChannelizer_Sub_RegAddrs =
    {  mmDdcChannelizerInfo,      //  Version_Addr
       mmDdcChannelizerConfig,    //  Config_Addr
       mmDdcChannelizerReset
    };

static DdcChannelizer::MapRegisters DdcChannelizer_Core_RegAddrs =
    {   mmDdcChannelizerScale
    };

//---------------------------------------------------------------------------
//  constructor for class DdcChannelizer
//---------------------------------------------------------------------------

DdcChannelizer::DdcChannelizer(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Ddc, DdcChannelizer_Sub_RegAddrs, ewbDdc),
       ScaleScheduleReg(GP.WB_Ddc, DdcChannelizer_Core_RegAddrs.ScaleSchedule_Addr)
{
}

//---------------------------------------------------------------------------
//  DdcChannelizer::ScaleSchedule() -
//---------------------------------------------------------------------------

void DdcChannelizer::ScaleSchedule(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    ScaleScheduleReg.Value(value);
}

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative



