#ifndef DdcModule_GpH
#define DdcModule_GpH

#include <Components_Gp.h>
#include <GpDspMemoryStringMap_Gp.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

using std::vector;

//===========================================================================
//  struct FrameModeInfo  -- Helper type
//===========================================================================
struct FrameModeInfo
{
    FrameModeInfo() : mode(0), sz(0), delay(0), en(false), sched(0) {}

    unsigned int mode;
    unsigned int sz;
    unsigned int delay;
    bool         en;
    unsigned int sched;
};

//===========================================================================
//  CLASS Ddc_Module  -- DDC Module shared features
//===========================================================================

class Ddc_Module : public DdcModuleBase
{
public:
    Ddc_Module(GpDspMemoryExtension & GP);
    virtual ~Ddc_Module() {}

};

//===========================================================================
//  CLASS DdcPeripheralSM  -- Offset 0x10
//===========================================================================

class DdcPeripheralSM : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPeripheralSM::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Bypass_Addr;
        unsigned int   SourceSelection_Addr;
        unsigned int   SpectrumInv_Addr;
        unsigned int   FilterSelect1_Addr;
        unsigned int   FilterSelect2_Addr;
    };

    // definition should really be in cpp...
    DdcPeripheralSM(GpDspMemoryExtension & GP, DdcModuleBase & base);
    virtual ~DdcPeripheralSM()  {}

    void BypassCore(unsigned int parent, bool value);
    void Source(unsigned int parent, unsigned int in);
    void EnableSpectrumInversion(unsigned int parent, bool value);
    void Filter1_Select(unsigned int parent, unsigned int value);
    void Filter2_Select(unsigned int parent, unsigned int value);
    void Decimation(unsigned int parent, unsigned int value);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPeripheralSM::BypassRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BypassRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBit      Enabled;

        BypassRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Enabled(*this, 0)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPeripheralSM::SourceSelectionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SourceSelectionRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      Source;

        SourceSelectionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Source(*this, 0, 8)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPeripheralSM::SpectrumInversionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpectrumInversionRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBit      Enabled;

        SpectrumInversionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Enabled(*this, 0)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPeripheralSM::FilterSelectionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FilterSelectionRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      Select;

        FilterSelectionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Select(*this, 0, 3)
            {}
    };

private:
    DdcModuleBase             &Base;
    BypassRegister             BypassReg;
    SourceSelectionRegister    SourceSelectionReg;
    SpectrumInversionRegister  SpectrumInversionReg;
    FilterSelectionRegister    Filter1_SelectReg;
    FilterSelectionRegister    Filter2_SelectReg;

};

//===========================================================================
//  CLASS DdcTuner  --
//===========================================================================

class DdcTuner : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcTuner::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Frequency_Addr;
        unsigned int   Phase_Addr;
        unsigned int   TuneFreqs_Addr;
        unsigned int   TuneEnable_Addr;
        unsigned int   TuneAddr_Addr;
        unsigned int   TuneData_Addr;
        unsigned int   TuneDataOut_Addr;
    };

    // definition should really be in cpp...
    DdcTuner(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DdcTuner()  {}
    //  custom register class defs here...

    void Frequency(unsigned int parent, double freq,  double srate);
    void PhaseOffset(unsigned int parent, double value);
	void ModFrequency(unsigned int parent, double freq, double srate);
    void TuningFrequencies(unsigned int parent, const vector<double> &freqs, double srate);
    void TuningEnable(unsigned int parent, bool en);
    unsigned int TuningDataOut(unsigned int parent);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcTuner::TunerScheduleProgAddrRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TunerScheduleProgAddrRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      tuner_sch_addr;

        TunerScheduleProgAddrRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              tuner_sch_addr(*this, 0, 12)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcTuner::TunerSchedNumOfFreqsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TunerSchedNumOfFreqsRegister : public DdcCachedShadowRegister
    {
        typedef DdcCachedShadowRegister inherited;

    public:
        RegisterBitGroup      num_tuning_freqs;
        RegisterBit           en_tuning_sched;

        TunerSchedNumOfFreqsRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              num_tuning_freqs(*this, 0, 16),
              en_tuning_sched(*this, 31)
            {}
    };

private:
    DdcShadowRegister             FrequencyReg;
    DdcShadowRegister             PhaseOffsetReg;
    TunerSchedNumOfFreqsRegister  TunerScheduleFreqsReg;
    DdcShadowRegister             TunerScheduleProgEnableReg;
    TunerScheduleProgAddrRegister TunerScheduleAddrReg;
    DdcShadowRegister             TunerScheduleDataReg;
    DdcShadowRegister             TunerScheduleDataOutReg;
};

//===========================================================================
//  CLASS DdcPolyPhaseFilter --
//===========================================================================

class DdcPolyPhaseFilter : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPolyPhaseFilter::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Coeffs_Addr;
        unsigned int   BStart_Addr;
    };


    // definition should really be in cpp...
    DdcPolyPhaseFilter(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DdcPolyPhaseFilter()  {}
    //  custom register class defs here...

    void Coeffs(unsigned int parent, unsigned int id, std::vector<int> cfs);
    void BStart(unsigned int parent, unsigned int num, unsigned int dir);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPolyPhaseFilter::CoeffsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CoeffsRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_coef;

        CoeffsRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_coef(*this, 0, 18)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcPolyPhaseFilter::BStartRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BStartRegister : public DdcCachedShadowRegister
    {
        typedef DdcCachedShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_shift_num;
        RegisterBit           ppf_shift_dir;

        BStartRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_shift_num(*this, 0, 8),
              ppf_shift_dir(*this, 8)
            {}
    };

private:
    CoeffsRegister CoeffsReg;
    BStartRegister BStartReg;
};


//===========================================================================
//  CLASS DdcLowPassFilter --
//===========================================================================

class DdcLowPassFilter : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcLowPassFilter::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   CicRate_Addr;
        unsigned int   CicBStart_Addr;
        unsigned int   CFirCoeff_Addr;
        unsigned int   CFirBStart_Addr;
        unsigned int   PFirCoeff_Addr;
        unsigned int   PFirBStart_Addr;
        unsigned int   FirlCoeff_Addr;
        unsigned int   FirlBStart_Addr;
        unsigned int   Fir2Coeff_Addr;
        unsigned int   Fir2BStart_Addr;

    };
    // definition should really be in cpp...
    DdcLowPassFilter(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DdcLowPassFilter()  {}
    //  custom register class defs here...

    void CicRate(unsigned int parent, unsigned int value);
    void CicBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void CFirCoeffs(unsigned int parent, std::vector<int> cfs);
    void CFirBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void PFirCoeffs(unsigned int parent, std::vector<int> cfs);
    void PFirBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void FirlCoeffs(unsigned int parent, std::vector<int> cfs);
    void FirlBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void Fir2Coeffs(unsigned int parent, std::vector<int> cfs);
    void Fir2BStart(unsigned int parent, unsigned int num, unsigned int dir);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcLowPassFilter::CoeffsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CicRateRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      cic_rate;

        CicRateRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              cic_rate(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcLowPassFilter::CoeffsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CoeffsRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_coef;

        CoeffsRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_coef(*this, 0, 18)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcLowPassFilter::BStartRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BStartRegister : public DdcCachedShadowRegister
    {
        typedef DdcCachedShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_shift_num;
        RegisterBit           ppf_shift_dir;

        BStartRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_shift_num(*this, 0, 8),
              ppf_shift_dir(*this, 8)
            {}
    };


private:
    CicRateRegister CicRateReg;
    BStartRegister  CicBStartReg;
    CoeffsRegister  CFirCoeffsReg;
    BStartRegister  CFirBStartReg;
    CoeffsRegister  PFirCoeffsReg;
    BStartRegister  PFirBStartReg;
    CoeffsRegister  FirlCoeffsReg;
    BStartRegister  FirlBStartReg;
    CoeffsRegister  Fir2CoeffsReg;
    BStartRegister  Fir2BStartReg;
};


//===========================================================================
//  CLASS DdcVitaFramer  -- VitaFramer 'submodule'
//===========================================================================

class DdcVitaFramer : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Enable_Addr;
        unsigned int   Size_Addr;
        unsigned int   StreamID_Addr;
        unsigned int   FrameMode_Addr;
        unsigned int   FrameSize_Addr;
        unsigned int   FrameCount_Addr;
        unsigned int   FrameDelay_Addr;
        unsigned int   FrameReEneable_Addr;
    };

    // definition should really be in cpp...
    DdcVitaFramer(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DdcVitaFramer()  {}

    void Enable(unsigned int parent, bool value);
    void Size(unsigned int parent, unsigned int size);
    unsigned int Size(unsigned int parent);
    void Identity(unsigned int parent, unsigned int did, unsigned int sid);
    void FrameMode(unsigned int parent, unsigned int mode, bool loc, unsigned int src);
    void FrameSize(unsigned int parent, unsigned int value);
    unsigned int FrameCount(unsigned int parent);
    void FrameDelay(unsigned int parent, unsigned int value);
    void ReEnable(unsigned int parent, bool value);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::EnableRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class EnableRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBit      vframer_enable;

        EnableRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_enable(*this, 0)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::SizeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SizeRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      vframer_size;

        SizeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_size(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::StreamIDRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class StreamIDRegister : public DdcCachedShadowRegister
    {
        typedef DdcCachedShadowRegister inherited;

    public:
        RegisterBitGroup      vframer_stream_id;
        RegisterBitGroup      vframer_dest_id;

        StreamIDRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_stream_id(*this, 0, 16),
              vframer_dest_id(*this, 16, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::FrameModeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FrameModeRegister : public DdcCachedShadowRegister
    {
        typedef DdcCachedShadowRegister inherited;

    public:
        RegisterBit   frame_mode;
        RegisterBit   trig_loc;
        RegisterBit   trig_src;

        FrameModeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              frame_mode(*this, 0), trig_loc(*this, 4),
              trig_src(*this, 8)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::FrameSizeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FrameSizeRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup   frame_size;

        FrameSizeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              frame_size(*this, 0, 24)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::FrameCountRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FrameCountRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup   frame_count;

        FrameCountRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              frame_count(*this, 0, 24)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcVitaFramer::FrameDelayRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FrameDelayRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup   frame_delay;

        FrameDelayRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              frame_delay(*this, 0, 24)
            {}
    };

private:
    EnableRegister     EnableReg;
    SizeRegister       SizeReg;
    StreamIDRegister   StreamIDReg;
    FrameModeRegister  FrameModeReg;
    FrameSizeRegister  FrameSizeReg;
    FrameCountRegister FrameCountReg;
    FrameDelayRegister FrameDelayReg;
    DdcShadowRegister  FrameReEnableReg;
};

//===========================================================================
//  CLASS DdcChannelizer  -- DdcChannelizer 'submodule'
//===========================================================================

class DdcChannelizer : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcChannelizer::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   ScaleSchedule_Addr;
    };

    // definition should really be in cpp...
    DdcChannelizer(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DdcChannelizer()  {}

    void ScaleSchedule(unsigned int parent, unsigned int size);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DdcChannelizer::EnableRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ScaleScheduleRegister : public DdcShadowRegister
    {
        typedef DdcShadowRegister inherited;

    public:
        RegisterBitGroup      scale_schedule;

        ScaleScheduleRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              scale_schedule(*this, 0, 10)
            {}
    };

private:
    ScaleScheduleRegister   ScaleScheduleReg;
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

#endif // DdcModule_GpH
