#include <DucModule_Gp.h>

namespace   Innovative
{
#ifdef __CLR_VER
#pragma warning(disable: 4355)
#pragma managed(push, off)
#endif

using namespace GpDsp_MemoryMap;

//===========================================================================
//  CLASS Duc_Module  -- DUC Module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Duc_Module
//---------------------------------------------------------------------------

Duc_Module::Duc_Module(GpDspMemoryExtension & GP)
    :  DdcModuleBase(GP.WB_Duc, ewbDuc)
{
}

//===========================================================================
//  CLASS DucPeripheralSM  -- Offset 0x10
//===========================================================================
//===========================================================================
//  CLASS DucPeripheral  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DucPeripheral
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DucPeripheral_Sub_RegAddrs =
    {  mmDucPeriphInfo,      //  Version_Addr
       mmDucPeriphConfig,     //  Config_Addr
       mmDucPeriphReset
    };

static DucPeripheralSM::MapRegisters Periph_Core_RegAddrs =
    {
    mmDucDecimation,
    mmDucPeriphBypass,        //  Bypass_Addr;
    mmDucPeriphSource,        //  SourceSelection_Addr;
    mmDucPeriphSpectralInv,   //  SpectrumInv_Addr;
    mmDucPeriphFilterSel1,    //  FilterSelect1_Addr;
    mmDucPeriphFilterSel2,    //  FilterSelect2_Addr;
	mmDucInputStreamId        //  DucInputStreamId;
    };

//---------------------------------------------------------------------------
//  constructor for class DucPeripheral
//---------------------------------------------------------------------------

DucPeripheralSM::DucPeripheralSM(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Duc, DucPeripheral_Sub_RegAddrs, ewbDuc),
       DecimationReg(GP.WB_Duc, Periph_Core_RegAddrs.Decimation_Addr),
       BypassReg(GP.WB_Duc, Periph_Core_RegAddrs.Bypass_Addr),
       SourceSelectionReg(GP.WB_Duc, Periph_Core_RegAddrs.SourceSelection_Addr),
       SpectrumInversionReg(GP.WB_Duc, Periph_Core_RegAddrs.SpectrumInv_Addr),
       Filter1_SelectReg(GP.WB_Duc, Periph_Core_RegAddrs.FilterSelect1_Addr),
       Filter2_SelectReg(GP.WB_Duc, Periph_Core_RegAddrs.FilterSelect2_Addr),
	   InputStreamIdReg(GP.WB_Duc, Periph_Core_RegAddrs.InputStreamId_Addr)
{
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::BypassCore() -
//---------------------------------------------------------------------------

void DucPeripheralSM::BypassCore(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    BypassReg.Enabled.Value(value);
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::Source()
//---------------------------------------------------------------------------

void  DucPeripheralSM::Source(unsigned int parent, unsigned int source)
{
    SetIndex(parent, 0);
    SourceSelectionReg.Source.Value(source);
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::Sid()
//---------------------------------------------------------------------------

void  DucPeripheralSM::Sid(unsigned int parent, unsigned int sid)
{
    SetIndex(parent, 0);
    InputStreamIdReg.Sid.Value(sid);
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::EnableSpectrumInversion() -
//---------------------------------------------------------------------------

void DucPeripheralSM::EnableSpectrumInversion(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    SpectrumInversionReg.Enabled.Value(value);
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::Filter1_Select()
//---------------------------------------------------------------------------

void  DucPeripheralSM::Filter1_Select(unsigned int parent, unsigned int source)
{
    SetIndex(parent, 0);
    Filter1_SelectReg.Select.Value(source);
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::Filter2_Select()
//---------------------------------------------------------------------------

void  DucPeripheralSM::Filter2_Select(unsigned int parent, unsigned int source)
{
    SetIndex(parent, 0);
    Filter2_SelectReg.Select.Value(source);
}

//---------------------------------------------------------------------------
//  DucPeripheralSM::Decimation()
//---------------------------------------------------------------------------

void  DucPeripheralSM::Decimation(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    DecimationReg.Value(value);
}

//===========================================================================
//  CLASS DucTuner  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DucTuner
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DucTuner_Sub_RegAddrs =
    {  mmDucTunerInfo,      //  Version_Addr
       mmDucTunerConfig,    //  Config_Addr
       mmDucTunerReset
    };

static DucTuner::MapRegisters DucTuner_Core_RegAddrs =
    {
    mmDucTunerFrequency,    //  Frequency_Addr;
    mmDucTunerPhaseOfs      //  PhaseOfs_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class DucTuner
//---------------------------------------------------------------------------

DucTuner::DucTuner(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Duc, DucTuner_Sub_RegAddrs, ewbDuc),
       FrequencyReg(GP.WB_Duc, DucTuner_Core_RegAddrs.Frequency_Addr),
       PhaseOffsetReg(GP.WB_Duc, DucTuner_Core_RegAddrs.Phase_Addr)
{
}

//---------------------------------------------------------------------------
//  DucTuner::Frequency() -
//---------------------------------------------------------------------------

void DucTuner::Frequency(unsigned int parent, double freq, double srate)
{
    unsigned long Multiplier = 2147483648u;
    unsigned int pinc = (freq * Multiplier * 2) / srate;
    SetIndex(parent, 0);
    FrequencyReg.Value(pinc);
}

//---------------------------------------------------------------------------
//  DucTuner::ModFrequency() -
//---------------------------------------------------------------------------

void DucTuner::ModFrequency(unsigned int parent, double freq, double /*srate*/)
{
    unsigned long Multiplier = 2147483648u;
    unsigned int pinc = (freq * Multiplier * 2) / 20;
    SetIndex(parent, 1);
    FrequencyReg.Value(pinc);
}

//---------------------------------------------------------------------------
//  DucTuner::PhaseOffset() -
//---------------------------------------------------------------------------

void DucTuner::PhaseOffset(unsigned int parent, double value)
{
    unsigned long Multiplier = 2147483648u;
    SetIndex(parent, 0);
    unsigned int poff = (value * Multiplier * 2) / 360;
    PhaseOffsetReg.Value(poff);
}

//===========================================================================
//  CLASS DucPolyPhaseFilter  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DucPolyPhaseFilter
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DucPolyPhase_Sub_RegAddrs =
    {  mmDucPolyPhaseInfo,      //  Version_Addr
       mmDucPolyPhaseConfig,     //  Config_Addr
       mmDucPolyPhaseReset
    };

static DucPolyPhaseFilter::MapRegisters DucPolyPhase_Core_RegAddrs =
    {
    mmDucPolyPhaseCoeffs,       //  Coeffs_Addr;
    mmDucPolyPhaseBStart        //  BStart_Addr;
    };

//---------------------------------------------------------------------------
//  constructor for class DucPolyPhaseFilter
//---------------------------------------------------------------------------

DucPolyPhaseFilter::DucPolyPhaseFilter(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Duc, DucPolyPhase_Sub_RegAddrs, ewbDuc),
      CoeffsReg(GP.WB_Duc, DucPolyPhase_Core_RegAddrs.Coeffs_Addr),
      BStartReg(GP.WB_Duc, DucPolyPhase_Core_RegAddrs.BStart_Addr)
{
}

//---------------------------------------------------------------------------
//  DucPolyPhaseFilter::BStart() -
//---------------------------------------------------------------------------

void DucPolyPhaseFilter::BStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    BStartReg.ppf_shift_num.Value(num);
    BStartReg.ppf_shift_dir.Value(dir ? true : false);

    BStartReg.Apply();
}
//---------------------------------------------------------------------------
//  DucPolyPhaseFilter::Coeffs() -
//---------------------------------------------------------------------------

void DucPolyPhaseFilter::Coeffs(unsigned int parent, unsigned int id, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    CoeffsReg.ppf_coef.Value(cfs.size());
    // id
    CoeffsReg.ppf_coef.Value(id);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        CoeffsReg.ppf_coef.Value(cfs[i]);
}


//===========================================================================
//  CLASS DucLowPassFilter  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DucLowPassFilter
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DucLowPass_Sub_RegAddrs =
    {  mmDucLowpassInfo,      //  Version_Addr
       mmDucLowpassConfig,     //  Config_Addr
       mmDucLowpassReset
    };

static DucLowPassFilter::MapRegisters DucLowPass_Core_RegAddrs =
    {   mmDucLowpassCicRate,
        mmDucLowpassCicLsb,
        mmDucLowpassCfirCoeff,
        mmDucLowpassCfirBStart,
        mmDucLowpassPfirCoeff,
        mmDucLowpassPfirBStart,
        mmDucLowpassFir1Coeff,
        mmDucLowpassFir1BStart,
        mmDucLowpassFir2Coeff,
        mmDucLowpassFir2BStart
    };

//---------------------------------------------------------------------------
//  constructor for class DucLowPassFilter
//---------------------------------------------------------------------------

DucLowPassFilter::DucLowPassFilter(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Duc, DucLowPass_Sub_RegAddrs, ewbDuc),
      CicRateReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.CicRate_Addr),
      CicBStartReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.CicBStart_Addr),
      CFirCoeffsReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.CFirCoeff_Addr),
      CFirBStartReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.CFirBStart_Addr),
      PFirCoeffsReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.PFirCoeff_Addr),
      PFirBStartReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.PFirBStart_Addr),
      FirlCoeffsReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.FirlCoeff_Addr),
      FirlBStartReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.FirlBStart_Addr),
      Fir2CoeffsReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.Fir2Coeff_Addr),
      Fir2BStartReg(GP.WB_Duc, DucLowPass_Core_RegAddrs.Fir2BStart_Addr)
    {
    }

//---------------------------------------------------------------------------
//  DucLowPassFilter::CicRate() -
//---------------------------------------------------------------------------

void DucLowPassFilter::CicRate(unsigned int parent, unsigned int value)
{
    SetIndex(parent, 0);
    CicRateReg.cic_rate.Value(value);
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::CicBStart() -
//---------------------------------------------------------------------------

void DucLowPassFilter::CicBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    CicBStartReg.ppf_shift_num.Value(num);
    CicBStartReg.ppf_shift_dir.Value(dir ? true : false);

    CicBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::CFirCoeffs() -
//---------------------------------------------------------------------------

void DucLowPassFilter::CFirCoeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    CFirCoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    CFirCoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        CFirCoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::CFirBStart() -
//---------------------------------------------------------------------------

void DucLowPassFilter::CFirBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    CFirBStartReg.ppf_shift_num.Value(num);
    CFirBStartReg.ppf_shift_dir.Value(dir ? true : false );

    CFirBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::PFirCoeffs() -
//---------------------------------------------------------------------------

void DucLowPassFilter::PFirCoeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    PFirCoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    PFirCoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        PFirCoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::PFirBStart() -
//---------------------------------------------------------------------------

void DucLowPassFilter::PFirBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    PFirBStartReg.ppf_shift_num.Value(num);
    PFirBStartReg.ppf_shift_dir.Value(dir ? true : false);

    PFirBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::FirlCoeffs() -
//---------------------------------------------------------------------------

void DucLowPassFilter::FirlCoeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    FirlCoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    FirlCoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        FirlCoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::FirlBStart() -
//---------------------------------------------------------------------------

void DucLowPassFilter::FirlBStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    FirlBStartReg.ppf_shift_num.Value(num);
    FirlBStartReg.ppf_shift_dir.Value(dir ? true : false );

    FirlBStartReg.Apply();
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::Fir2Coeffs() -
//---------------------------------------------------------------------------

void DucLowPassFilter::Fir2Coeffs(unsigned int parent, std::vector<int> cfs)
{
    SetIndex(parent, 0);
    // taps to load
    Fir2CoeffsReg.ppf_coef.Value(cfs.size());
    // zero
    Fir2CoeffsReg.ppf_coef.Value(0);
    // coeffs
    for (size_t i = 0; i < cfs.size(); i++)
        Fir2CoeffsReg.ppf_coef.Value(cfs[i]);
}

//---------------------------------------------------------------------------
//  DucLowPassFilter::Fir2BStart() -
//---------------------------------------------------------------------------

void DucLowPassFilter::Fir2BStart(unsigned int parent, unsigned int num, unsigned int dir)
{
    SetIndex(parent, 0);
    Fir2BStartReg.ppf_shift_num.Value(num);
    Fir2BStartReg.ppf_shift_dir.Value(dir ? true : false );

    Fir2BStartReg.Apply();
}

//===========================================================================
//  CLASS DucVitaFramer  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DucVitaFramer
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters DucVitaFramer_Sub_RegAddrs =
    {  mmDucVitaInfo,      //  Version_Addr
       mmDucVitaConfig,     //  Config_Addr
       mmDucVitaReset
    };

static DucVitaFramer::MapRegisters DucVitaFramer_Core_RegAddrs =
    {   mmDucVitaEnable,
        mmDucVitaSize,
        mmDucVitaSid
    };

//---------------------------------------------------------------------------
//  constructor for class DucVitaFramer
//---------------------------------------------------------------------------

DucVitaFramer::DucVitaFramer(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_Duc, DucVitaFramer_Sub_RegAddrs, ewbDuc),
       EnableReg(GP.WB_Duc, DucVitaFramer_Core_RegAddrs.Enable_Addr),
       SizeReg(GP.WB_Duc, DucVitaFramer_Core_RegAddrs.Size_Addr),
       StreamIDReg(GP.WB_Duc, DucVitaFramer_Core_RegAddrs.StreamID_Addr)
{
}

//---------------------------------------------------------------------------
//  DucVitaFramer::Enable() -
//---------------------------------------------------------------------------

void DucVitaFramer::Enable(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    EnableReg.vframer_enable.Value(value);
}

//---------------------------------------------------------------------------
//  DucVitaFramer::Enable() -
//---------------------------------------------------------------------------

void DucVitaFramer::Size(unsigned int parent, unsigned int size)
{
    SetIndex(parent, 0);
    SizeReg.vframer_size.Value(size);
}

//---------------------------------------------------------------------------
//  DucVitaFramer::Enable() -
//---------------------------------------------------------------------------

void DucVitaFramer::Identity(unsigned int parent, unsigned int did, unsigned int sid)
{
    SetIndex(parent, 0);
    StreamIDReg.vframer_stream_id.Value(sid);
    StreamIDReg.vframer_dest_id.Value(did);
    StreamIDReg.Apply();
}

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative


