#ifndef DucModule_GpH
#define DucModule_GpH

#include <Components_Gp.h>
#include <GpDspMemoryStringMap_Gp.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS Duc_Module  -- DUC Module shared features
//===========================================================================

class Duc_Module : public DdcModuleBase
{
public:
    Duc_Module(GpDspMemoryExtension & GP);
    virtual ~Duc_Module() {}

};


//===========================================================================
//  CLASS DucPeripheralSM  -- Offset 0x10
//===========================================================================

class DucPeripheralSM : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPeripheralSM::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Decimation_Addr;
        unsigned int   Bypass_Addr;
        unsigned int   SourceSelection_Addr;
        unsigned int   SpectrumInv_Addr;
        unsigned int   FilterSelect1_Addr;
        unsigned int   FilterSelect2_Addr;
		unsigned int   InputStreamId_Addr;
    };

    // definition should really be in cpp...
    DucPeripheralSM(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DucPeripheralSM() {}

    void BypassCore(unsigned int parent, bool value);
    void Source(unsigned int parent, unsigned int in);
	void Sid(unsigned int parent, unsigned int sid);
    void EnableSpectrumInversion(unsigned int parent, bool value);
    void Filter1_Select(unsigned int parent, unsigned int value);
    void Filter2_Select(unsigned int parent, unsigned int value);
    void Decimation(unsigned int parent, unsigned int value);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPeripheralSM::BypassRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BypassRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBit      Enabled;

        BypassRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Enabled(*this, 0)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPeripheralSM::SourceSelectionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SourceSelectionRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      Source;

        SourceSelectionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Source(*this, 0, 8)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPeripheralSM::SpectrumInversionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SpectrumInversionRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBit      Enabled;

        SpectrumInversionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Enabled(*this, 0)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPeripheralSM::FilterSelectionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FilterSelectionRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      Select;

        FilterSelectionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Select(*this, 0, 3)
            {}
    };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPeripheralSM::InputStreamIdRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class InputStreamIdRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      Sid;

        InputStreamIdRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Sid(*this, 0, 16)
            {}
    };

private:
    Register                   DecimationReg;
    BypassRegister             BypassReg;
    SourceSelectionRegister    SourceSelectionReg;
    SpectrumInversionRegister  SpectrumInversionReg;
    FilterSelectionRegister    Filter1_SelectReg;
    FilterSelectionRegister    Filter2_SelectReg;
	InputStreamIdRegister      InputStreamIdReg;

};

//===========================================================================
//  CLASS DucTuner  --
//===========================================================================

class DucTuner : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucTuner::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Frequency_Addr;
        unsigned int   Phase_Addr;
    };

    // definition should really be in cpp...
    DucTuner(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DucTuner()  {}
    //  custom register class defs here...

    void Frequency(unsigned int parent, double freq,  double srate);
    void PhaseOffset(unsigned int parent, double value);
	void ModFrequency(unsigned int parent, double freq, double srate);

private:
    DucShadowRegister FrequencyReg;
    DucShadowRegister PhaseOffsetReg;
};

//===========================================================================
//  CLASS DucPolyPhaseFilter --
//===========================================================================

class DucPolyPhaseFilter : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPolyPhaseFilter::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Coeffs_Addr;
        unsigned int   BStart_Addr;
    };


    // definition should really be in cpp...
    DucPolyPhaseFilter(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DucPolyPhaseFilter()  {}
    //  custom register class defs here...

    void Coeffs(unsigned int parent, unsigned int id, std::vector<int> cfs);
    void BStart(unsigned int parent, unsigned int num, unsigned int dir);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPolyPhaseFilter::CoeffsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CoeffsRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_coef;

        CoeffsRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_coef(*this, 0, 18)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucPolyPhaseFilter::BStartRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BStartRegister : public DucCachedShadowRegister
    {
        typedef DucCachedShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_shift_num;
        RegisterBit           ppf_shift_dir;

        BStartRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_shift_num(*this, 0, 8),
              ppf_shift_dir(*this, 8)
            {}
    };

private:
    CoeffsRegister CoeffsReg;
    BStartRegister BStartReg;
};

//===========================================================================
//  CLASS DucLowPassFilter --
//===========================================================================

class DucLowPassFilter : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucLowPassFilter::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   CicRate_Addr;
        unsigned int   CicBStart_Addr;
        unsigned int   CFirCoeff_Addr;
        unsigned int   CFirBStart_Addr;
        unsigned int   PFirCoeff_Addr;
        unsigned int   PFirBStart_Addr;
        unsigned int   FirlCoeff_Addr;
        unsigned int   FirlBStart_Addr;
        unsigned int   Fir2Coeff_Addr;
        unsigned int   Fir2BStart_Addr;

    };
    // definition should really be in cpp...
    DucLowPassFilter(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DucLowPassFilter()  {}
    //  custom register class defs here...

    void CicRate(unsigned int parent, unsigned int value);
    void CicBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void CFirCoeffs(unsigned int parent, std::vector<int> cfs);
    void CFirBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void PFirCoeffs(unsigned int parent, std::vector<int> cfs);
    void PFirBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void FirlCoeffs(unsigned int parent, std::vector<int> cfs);
    void FirlBStart(unsigned int parent, unsigned int num, unsigned int dir);
    void Fir2Coeffs(unsigned int parent, std::vector<int> cfs);
    void Fir2BStart(unsigned int parent, unsigned int num, unsigned int dir);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucLowPassFilter::CoeffsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CicRateRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      cic_rate;

        CicRateRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              cic_rate(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucLowPassFilter::CoeffsRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class CoeffsRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_coef;

        CoeffsRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_coef(*this, 0, 18)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucLowPassFilter::BStartRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BStartRegister : public DucCachedShadowRegister
    {
        typedef DucCachedShadowRegister inherited;

    public:
        RegisterBitGroup      ppf_shift_num;
        RegisterBit           ppf_shift_dir;

        BStartRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              ppf_shift_num(*this, 0, 8),
              ppf_shift_dir(*this, 8)
            {}
    };


private:
    CicRateRegister CicRateReg;
    BStartRegister  CicBStartReg;
    CoeffsRegister  CFirCoeffsReg;
    BStartRegister  CFirBStartReg;
    CoeffsRegister  PFirCoeffsReg;
    BStartRegister  PFirBStartReg;
    CoeffsRegister  FirlCoeffsReg;
    BStartRegister  FirlBStartReg;
    CoeffsRegister  Fir2CoeffsReg;
    BStartRegister  Fir2BStartReg;
};


//===========================================================================
//  CLASS DucVitaFramer  -- VitaFramer 'submodule'
//===========================================================================

class DucVitaFramer : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucVitaFramer::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Enable_Addr;
        unsigned int   Size_Addr;
        unsigned int   StreamID_Addr;
    };

    // definition should really be in cpp...
    DucVitaFramer(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~DucVitaFramer()  {}

    void Enable(unsigned int parent, bool value);
    void Size(unsigned int parent, unsigned int size);
    void Identity(unsigned int parent, unsigned int did, unsigned int sid);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucVitaFramer::EnableRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class EnableRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBit      vframer_enable;

        EnableRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_enable(*this, 0)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucVitaFramer::SizeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SizeRegister : public DucShadowRegister
    {
        typedef DucShadowRegister inherited;

    public:
        RegisterBitGroup      vframer_size;

        SizeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_size(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS DucVitaFramer::StreamIDRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class StreamIDRegister : public DucCachedShadowRegister
    {
        typedef DucCachedShadowRegister inherited;

    public:
        RegisterBitGroup      vframer_stream_id;
        RegisterBitGroup      vframer_dest_id;

        StreamIDRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_stream_id(*this, 0, 16),
              vframer_dest_id(*this, 16, 16)
            {}
    };

private:
    EnableRegister   EnableReg;
    SizeRegister     SizeReg;
    StreamIDRegister StreamIDReg;

};
#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

#endif // DucModule_GpH
