#include "FftProxy_Gp.h"

#include <Application/IniSaver_App.h>
#include <Fft_WindowingCoeffs.h>
#include <math.h>
#include <SystemSupport_Mb.h>
#include <cmath>

inline double round(double x) { return (floor(x + 0.5)); }

///---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::SpectralPeripheralWrapper() -
//---------------------------------------------------------------------------
SpectralPeripheralWrapper::SpectralPeripheralWrapper(SpectralPeripheral *avf, unsigned int acore)
    : vf(avf), core(acore)
{

}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::~SpectralPeripheralWrapper() -
//---------------------------------------------------------------------------
SpectralPeripheralWrapper::~SpectralPeripheralWrapper()
{

}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::BypassSpectrumAnalysis() -
//---------------------------------------------------------------------------
void SpectralPeripheralWrapper::BypassSpectrumAnalysis(unsigned int parent, bool value)
{
    vf->BypassSpectAnalysis(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::SourceSelect() -
//---------------------------------------------------------------------------
void SpectralPeripheralWrapper::SourceSelect(unsigned int parent, unsigned int value)
{
    vf->SourceSelect(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::BypassWindowing() -
//---------------------------------------------------------------------------
void SpectralPeripheralWrapper::BypassWindowing(unsigned int parent, bool value)
{
    vf->BypassWindowing(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::ModeSelect() -
//---------------------------------------------------------------------------
void SpectralPeripheralWrapper::ModeSelect(unsigned int parent, unsigned int value)
{
    switch(value)
        {
        case 0:
            vf->ModeSelect(parent, SpectralPeripheral::oFullSpectrum);
            break;
        case 1:
            vf->ModeSelect(parent, SpectralPeripheral::oThresholLimited);
            break;
        default:
            break;
        }
}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::Reset() -
//---------------------------------------------------------------------------
void SpectralPeripheralWrapper::Reset(unsigned int /*parent*/, bool /*value*/)
{
}

//---------------------------------------------------------------------------
//  SpectralPeripheralWrapper::SetSourceMap() -
//---------------------------------------------------------------------------

void SpectralPeripheralWrapper::SetSourceMap(const vector<int> &values)
{
    vf->SetSourceMap(values);
}

//---------------------------------------------------------------------------
//  SpectralMultiBufferWrapper::SpectralMultiBufferWrapper() -
//---------------------------------------------------------------------------
SpectralMultiBufferWrapper::SpectralMultiBufferWrapper(SpectralMultiBuffer *avf, unsigned int acore)
    : vf(avf), core(acore)
{

}

//---------------------------------------------------------------------------
//  SpectralMultiBufferWrapper::~SpectralMultiBufferWrapper() -
//---------------------------------------------------------------------------
SpectralMultiBufferWrapper::~SpectralMultiBufferWrapper()
{

}

//---------------------------------------------------------------------------
//  SpectralMultiBufferWrapper::WindowSize() -
//---------------------------------------------------------------------------
void SpectralMultiBufferWrapper::WindowSize(unsigned int parent, int value)
{
    vf->WindowSize(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralMultiBufferWrapper::CalculateWindow() -
//---------------------------------------------------------------------------

void SpectralMultiBufferWrapper::CalculateWindow(unsigned int parent, unsigned int drate, unsigned int srate, float fpersec, bool arch)
{
    vf->setFasd(fpersec);

    srate *= 1e6;
    float frame = (32768.0 * drate) / srate;
    if (frame < 40.0 / 1e6) frame = 40.0 / 1e6;
    float desired = 1.0f / fpersec;
    unsigned int window = 8192;
    if (desired > frame / 2)
        {
        if (arch)
            window = desired / 2.0 * srate / drate; // parallel
        else
            window = desired * 2.0 * srate / drate; // serial
        }
    WindowSize(parent, window);
    Delay(parent, window / 2);

}

//---------------------------------------------------------------------------
//  SpectralMultiBufferWrapper::Delay() -
//---------------------------------------------------------------------------
void SpectralMultiBufferWrapper::Delay(unsigned int parent, int value)
{
    vf->Delay(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralMultiBufferWrapper::Reset() -
//---------------------------------------------------------------------------
void SpectralMultiBufferWrapper::Reset(unsigned int /*parent*/, bool /*value*/)
{
}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::SpectralWindowingWrapper() -
//---------------------------------------------------------------------------
SpectralWindowingWrapper::SpectralWindowingWrapper(SpectralWindowing *avf, unsigned int acore)
    : vf(avf), core(acore)
{

}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::~SpectralWindowingWrapper() -
//---------------------------------------------------------------------------
SpectralWindowingWrapper::~SpectralWindowingWrapper()
{

}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::CoeffEnable() -
//---------------------------------------------------------------------------
void SpectralWindowingWrapper::CoeffEnable(unsigned int parent, unsigned int value)
{
    switch(value)
        {
        case 1:
            vf->CoeffEnable(parent, SpectralWindowing::wCoeffs);
            break;
        case 0:
            vf->CoeffEnable(parent, SpectralWindowing::wController);
            break;
        default:
            break;
        }
}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::CoeffAddr() -
//---------------------------------------------------------------------------
void SpectralWindowingWrapper::CoeffAddr(unsigned int parent, int value)
{
    vf->CoeffAddr(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::CoeffData() -
//---------------------------------------------------------------------------
void SpectralWindowingWrapper::CoeffData(unsigned int parent, int value)
{
    vf->CoeffData(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::Coeffs() -
//---------------------------------------------------------------------------
void SpectralWindowingWrapper::Coeffs(unsigned int parent, const vector<int> &cs)
{
    vf->Coeffs(parent, cs);
}

//---------------------------------------------------------------------------
//  SpectralWindowingWrapper::Reset() -
//---------------------------------------------------------------------------
void SpectralWindowingWrapper::Reset(unsigned int /*parent*/, bool /*value*/)
{
}

//---------------------------------------------------------------------------
//  SpectralFftWrapper::SpectralFftWrapper() -
//---------------------------------------------------------------------------
SpectralFftWrapper::SpectralFftWrapper(SpectralFft *avf, unsigned int acore)
    : vf(avf), core(acore)
{

}

//---------------------------------------------------------------------------
//  SpectralFftWrapper::~SpectralFftWrapper() -
//---------------------------------------------------------------------------
SpectralFftWrapper::~SpectralFftWrapper()
{

}

//---------------------------------------------------------------------------
//  SpectralFftWrapper::Reset() -
//---------------------------------------------------------------------------
void SpectralFftWrapper::Reset(unsigned int /*parent*/, bool /*value*/)
{
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::SpectralPostProcWrapper() -
//---------------------------------------------------------------------------
SpectralPostProcWrapper::SpectralPostProcWrapper(SpectralPostProc *avf, unsigned int acore)
    : vf(avf), core(acore)
{

}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::~SpectralPostProcWrapper() -
//---------------------------------------------------------------------------
SpectralPostProcWrapper::~SpectralPostProcWrapper()
{

}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::TriggerMode() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::TriggerMode(unsigned int parent, unsigned int value)
{
    switch(value)
        {
        case 0:
            vf->TriggerMode(parent, SpectralPostProc::pDisabled);
            break;
        case 1:
            vf->TriggerMode(parent, SpectralPostProc::pSingleFrame);
            break;
        case 2:
            vf->TriggerMode(parent, SpectralPostProc::pContinuous);
            break;
        default:
            break;
        }
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::SingleTrigger() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::SingleTrigger(unsigned int parent, int value)
{
    vf->SingleTrigger(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::ThresholdProgEnable() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::ThresholdProgEnable(unsigned int parent, unsigned int value)
{
    switch(value)
        {
        case 0:
            vf->ThresholdProgEnable(parent, SpectralPostProc::pTls);
            break;
        case 1:
            vf->ThresholdProgEnable(parent, SpectralPostProc::pController);
            break;
        default:
            break;
        }
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::ThresholdProgEnable() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::ThresholdProgAddr(unsigned int parent, unsigned int addr)
{
    vf->ThresholdProgAddr(parent, addr);
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::ThresholdProgData() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::ThresholdProgData(unsigned int parent, int value)
{
    vf->ThresholdProgData(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::ThresholdProg() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::ThresholdProg(unsigned int parent, const vector<int> &coeffs)
{
    vf->ThresholdProg(parent, coeffs);
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::ThresholdProgData() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::SetThresholds(unsigned int parent, std::string file)
{
    vector<int> coeffs;
    Innovative::IniReader Ini(file);
    Ini.Install( ToIni("Coeffs", "Threshold", coeffs, 0) );
    Ini.Load();

    ThresholdProgEnable(parent, 1);

    ThresholdProg(parent, coeffs);

    ThresholdProgEnable(parent, 0);
}


//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::FasdFilterCoeff() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::FasdFilterCoeff(unsigned int parent, float value, bool enabled)
{
    const int MULTIPLIER = 32768;

    if (enabled)
        vf->FasdFilterCoeff(parent, static_cast<unsigned int>(value * MULTIPLIER));
    else
        vf->FasdFilterCoeff(parent, static_cast<unsigned int>(1     * MULTIPLIER));
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::Reset() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::Reset(unsigned int /*parent*/, bool /*value*/)
{
}

//---------------------------------------------------------------------------
//  SpectralPostProcWrapper::FrameDecimation() -
//---------------------------------------------------------------------------
void SpectralPostProcWrapper::FrameDecimation(unsigned int parent, int value)
{
    vf->FrameDecimation(parent, value);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::SpectralVitaFramerWrapper() -
//---------------------------------------------------------------------------
SpectralVitaFramerWrapper::SpectralVitaFramerWrapper(SpectralVitaFramer *avf, unsigned int acore)
    : vf(avf), core(acore)
{

}

//---------------------------------------------------------------------------
//  FftChannelWrapper::Clear() -
//---------------------------------------------------------------------------
SpectralVitaFramerWrapper::~SpectralVitaFramerWrapper()
{

}

//---------------------------------------------------------------------------
//  SpectralVitaFramerWrapper::Enable() -
//---------------------------------------------------------------------------

void SpectralVitaFramerWrapper::Enable(unsigned int parent, bool value)
{
    vf->Enable(parent, value);
}

//---------------------------------------------------------------------------
//  SpectralVitaFramerWrapper::Size() -
//---------------------------------------------------------------------------

void SpectralVitaFramerWrapper::Size(unsigned int parent, unsigned int value)
{
    vf->Size(parent, value);
}
//---------------------------------------------------------------------------
//  SpectralVitaFramerWrapper::Identity() -
//---------------------------------------------------------------------------

void SpectralVitaFramerWrapper::Identity(unsigned int parent, unsigned int did, unsigned int sid)
{
    vf->Identity(parent, did, sid);
}

//---------------------------------------------------------------------------
//  SpectralVitaFramerWrapper::Reset() -
//---------------------------------------------------------------------------

void SpectralVitaFramerWrapper::Reset(unsigned int /*parent*/, bool /*value*/)
{
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::FftChannelWrapper() -
//---------------------------------------------------------------------------
FftChannelWrapper::FftChannelWrapper(SpectralModule *amod, int aid, int abid)
    : vf(amod), id(aid), bid(abid)
{
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::FftChannelWrapper() -
//---------------------------------------------------------------------------
FftChannelWrapper::FftChannelWrapper(SpectralModule *amod, int aid, const FftInfo& /*ainfo*/, int abid)
    : NumberOfDDC(0), vf(amod), id(aid), bid(abid)
{
    SRate   = 1.0;
    DecRate = 1;

}

//---------------------------------------------------------------------------
//  FftChannelWrapper::~FftChannelWrapper() -
//---------------------------------------------------------------------------
FftChannelWrapper::~FftChannelWrapper()
{
    while (PeripheralGroup.size())
        {
        delete PeripheralGroup.back();
        PeripheralGroup.pop_back();
        }
    while (MultiBufferGroup.size())
        {
        delete MultiBufferGroup.back();
        MultiBufferGroup.pop_back();
        }
    while (WindowingPGroup.size())
        {
        delete WindowingPGroup.back();
        WindowingPGroup.pop_back();
        }
    while (FftGroup.size())
        {
        delete FftGroup.back();
        FftGroup.pop_back();
        }
    while (PostProcGroup.size())
        {
        delete PostProcGroup.back();
        PostProcGroup.pop_back();
        }
    while (VitaFramerGroup.size())
        {
        delete VitaFramerGroup.back();
        VitaFramerGroup.pop_back();
        }

}

//---------------------------------------------------------------------------
//  FftChannelWrapper::Tag() -
//---------------------------------------------------------------------------
string FftChannelWrapper::Tag()
{
    return string ("FFT [" + Innovative::IntToString(bid) + "][" + Innovative::IntToString(id) + "]");
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------
void FftChannelWrapper::AddWrapper(SpectralPeripheralWrapper *awrp)
{
    PeripheralGroup.push_back(awrp);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------
void FftChannelWrapper::AddWrapper(SpectralMultiBufferWrapper *awrp)
{
    MultiBufferGroup.push_back(awrp);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------
void FftChannelWrapper::AddWrapper(SpectralWindowingWrapper *awrp)
{
    WindowingPGroup.push_back(awrp);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------
void FftChannelWrapper::AddWrapper(SpectralFftWrapper *awrp)
{
    FftGroup.push_back(awrp);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------
void FftChannelWrapper::AddWrapper(SpectralPostProcWrapper *awrp)
{
    PostProcGroup.push_back(awrp);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------
void FftChannelWrapper::AddWrapper(SpectralVitaFramerWrapper *awrp)
{
    VitaFramerGroup.push_back(awrp);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::CommandTrigger() -
//---------------------------------------------------------------------------

void FftChannelWrapper::CommandTrigger()
{
    vf->CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::WhichBoard() -
//---------------------------------------------------------------------------

unsigned int FftChannelWrapper::WhichBoard()
{
    return bid;
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::SingleTrigger() -
//---------------------------------------------------------------------------

void FftChannelWrapper::SingleTrigger()
{
    for (size_t i = 0; i < PostProcGroup.size(); i++)
        {
        if (Info.TriggerMode == 1)
            PostProcGroup[i]->SingleTrigger(id, true);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::Apply() -
//---------------------------------------------------------------------------

void FftChannelWrapper::Apply()
{
    for (size_t i = 0; i < PeripheralGroup.size(); i++)
        {
        PeripheralGroup[i]->ModeSelect(id, Info.OutputMode);
        PeripheralGroup[i]->SourceSelect(id, Info.Source);
        }
    for (size_t i = 0; i < MultiBufferGroup.size(); i++)
        {
        MultiBufferGroup[i]->CalculateWindow(id, DecRate, SRate, Info.Fps, Architecture);
        }
    for (size_t i = 0; i < WindowingPGroup.size(); i++)
        {
        WindowingPGroup[i]->Coeffs(id, coeffs[Info.Window]);
        }
    for (size_t i = 0; i < PostProcGroup.size(); i++)
        {
        PostProcGroup[i]->FasdFilterCoeff(id, Info.Fasd, Info.FasdEnable);
        PostProcGroup[i]->SetThresholds(id, Info.ThresholdFile);
        PostProcGroup[i]->TriggerMode(id, Info.TriggerMode);
        PostProcGroup[i]->FrameDecimation(id, CalcFrameDecimation());
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::ApplyFrameDecimation() -
//---------------------------------------------------------------------------

void FftChannelWrapper::ApplyFrameDecimation()
{
    for (size_t i = 0; i < PostProcGroup.size(); i++)
        {
        PostProcGroup[i]->FrameDecimation(id, CalcFrameDecimation());
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::InputRate() - Calculates FFT Input rate
//---------------------------------------------------------------------------
double FftChannelWrapper::InputRate(int Source, int NumOfDdcs, int DecRate, double SampleRate)
{
    double FFT_Input_Rate   = 0;

    SampleRate = SampleRate * 1e6;
    // DDC
    if(NumOfDdcs > Source)
    {
        float DDC_Output_Rate = SampleRate / DecRate;

        if(250 * 1e6 >= DDC_Output_Rate)
           FFT_Input_Rate = DDC_Output_Rate;
        else if(500 * 1e6 >= DDC_Output_Rate && DDC_Output_Rate > 250 * 1e6)
           FFT_Input_Rate = DDC_Output_Rate / 2;
        else if(1000 * 1e6 >= DDC_Output_Rate && DDC_Output_Rate > 500 * 1e6)
           FFT_Input_Rate = DDC_Output_Rate / 4;
    }
    // Adc
    else
    {
       if(250 * 1e6 >= SampleRate)
          FFT_Input_Rate = SampleRate;
       else if(500 * 1e6 >= SampleRate && SampleRate > 250 * 1e6)
          FFT_Input_Rate = SampleRate / 2;
       else if(1000 * 1e6 >= SampleRate && SampleRate > 500 * 1e6)
          FFT_Input_Rate = SampleRate / 4;
       else if(2000 * 1e6 >= SampleRate && SampleRate > 1000 * 1e6)
          FFT_Input_Rate = SampleRate / 8;
    }

    return FFT_Input_Rate;
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::CalcFrameDecimation() - Helper
//---------------------------------------------------------------------------

int FftChannelWrapper::CalcFrameDecimation()
{
    double FftInputRate = InputRate(Info.Source, NumberOfDDC, DecRate, SRate);
    return CalcFrameDecimation(Info.Fps, FftInputRate);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::CalcFrameDecimation() -
//---------------------------------------------------------------------------
int FftChannelWrapper::CalcFrameDecimation(float FramePerSec, double FftInputRate)
{
   int Frame_Decimation = 0;

   if (FrameMode.mode == 0)
        Frame_Decimation = round(FftInputRate / 32768 / FramePerSec)-1;
   else
        Frame_Decimation = round(FftInputRate / (FrameMode.sz + FrameMode.delay) / FramePerSec)-1;

   if(Frame_Decimation < 1)
   {
       Frame_Decimation = 0;
   }
   if(Frame_Decimation > 65534)
   {
       Frame_Decimation = 65534;
   }
   return Frame_Decimation;
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::SetSourceMap() -
//---------------------------------------------------------------------------

void FftChannelWrapper::SetSourceMap(const vector<int> &values)
{
    for (size_t i = 0; i < PeripheralGroup.size(); i++)
        {
        PeripheralGroup[i]->SetSourceMap(values);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::ReCalculateWindow() -
//---------------------------------------------------------------------------

void FftChannelWrapper::ReCalculateWindow()
{
    for (size_t i = 0; i < MultiBufferGroup.size(); i++)
        {
        MultiBufferGroup[i]->CalculateWindow(id, DecRate, SRate, Info.Fps, Architecture);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::WindowSizeAndDelay() -
//---------------------------------------------------------------------------

void FftChannelWrapper::WindowSizeAndDelay(int size, int delay)
{
    for (size_t i = 0; i < MultiBufferGroup.size(); i++)
        {
        MultiBufferGroup[i]->WindowSize(id, size);
        MultiBufferGroup[i]->Delay(id, delay);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::PeakChangeApply() -
//---------------------------------------------------------------------------

void FftChannelWrapper::PeakChangeApply()
{
    for (size_t i = 0; i < PeripheralGroup.size(); i++)
        {
        PeripheralGroup[i]->ModeSelect(id, Info.OutputMode);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::SourceChangeApply() -
//---------------------------------------------------------------------------

void FftChannelWrapper::SourceChangeApply()
{
    for (size_t i = 0; i < PeripheralGroup.size(); i++)
        {
        PeripheralGroup[i]->SourceSelect(id, Info.Source);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::WindowChangeApply() -
//---------------------------------------------------------------------------

void FftChannelWrapper::WindowChangeApply()
{
    for (size_t i = 0; i < WindowingPGroup.size(); i++)
        {
        WindowingPGroup[i]->Coeffs(id, coeffs[Info.Window]);
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::TriggerModeApply() -
//---------------------------------------------------------------------------

void FftChannelWrapper::TriggerModeApply()
{
    for (size_t i = 0; i < PostProcGroup.size(); i++)
        {
        PostProcGroup[i]->TriggerMode(id, Info.TriggerMode);
        }
}


//---------------------------------------------------------------------------
//  FftChannelWrapper::ConfigureWindowCoeffs() -
//---------------------------------------------------------------------------

void FftChannelWrapper::ConfigureWindowCoeffs(GpDspSubmodule::ArhitectureType atype)
{
    Architecture = atype == GpDspSubmodule::aParallel;

    // Load coeffs
    for (size_t i = 0; i < sizeof(win_coeff_info) / sizeof(win_coeff_info[0]); i++)
        {
        int        *first = Architecture ? win_coeff_info[i].coef : swin_coeff_info[i].coef;
        int        *last  = Architecture ? (win_coeff_info[i].coef  + win_coeff_info[i].size)
                                     : (swin_coeff_info[i].coef + swin_coeff_info[i].size);

        coeffs.push_back( vector<int>(first, last) );
        }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::DecimationRate() -
//---------------------------------------------------------------------------

 void FftChannelWrapper::DecimationRate(int value)
 {
     DecRate = value;
 }

 //---------------------------------------------------------------------------
 //  FftChannelWrapper::DecimationRate() -
 //---------------------------------------------------------------------------

  int FftChannelWrapper::DecimationRate()
  {
      return DecRate;
  }


//---------------------------------------------------------------------------
//  FftChannelsProxy::SetNumberOfDDC() -
//---------------------------------------------------------------------------

void FftChannelWrapper::SetNumberOfDDC(int value)
{
   NumberOfDDC = value;
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::SetFrameModeInfo() -
//---------------------------------------------------------------------------

void FftChannelWrapper::SetFrameModeInfo(const FrameModeInfo& value)
{
   FrameMode = value;
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::ApplyEnable() -
//---------------------------------------------------------------------------

void FftChannelWrapper::ApplyEnable()
{
    for (size_t i = 0; i < VitaFramerGroup.size(); i++)
       {
       int dest_id = Info.Enable ? 0x01 : 0x8000;
       VitaFramerGroup[i]->Identity(id, dest_id, 0x400 + id);
       }
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::Reset() -
//---------------------------------------------------------------------------

void FftChannelWrapper::Reset(bool state)
{
    vf->Reset(state);
}

//---------------------------------------------------------------------------
//  FftChannelWrapper::AddWrapper() -
//---------------------------------------------------------------------------

void FftChannelWrapper::AttachSettings(Innovative::IniSaver& saver)
{
    saver.Install( ToIni(Tag() + "-FftChannel.Enable",          Info.Enable,   true)  );
    saver.Install( ToIni(Tag() + "-FftChannel.Source",          Info.Source,    0)  );
    saver.Install( ToIni(Tag() + "-FftChannel.Window",          Info.Window,    0)  );
    saver.Install( ToIni(Tag() + "-FftChannel.Fasd",            Info.Fasd,   1.0f)  );
    saver.Install( ToIni(Tag() + "-FftChannel.FasdEnable",      Info.FasdEnable, false)  );
    saver.Install( ToIni(Tag() + "-FftChannel.OutputMode",      Info.OutputMode,  0)  );
    saver.Install( ToIni(Tag() + "-FftChannel.TriggerMode",     Info.TriggerMode, 2)  );
    saver.Install( ToIni(Tag() + "-FftChannel.Fps",             Info.Fps,  10.0f)  );
    saver.Install( ToIni(Tag() + "-FftChannel.ThresholdFile",   Info.ThresholdFile,   string("..\\..\\Config\\tlsRAM_40db.ini"))  );
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::Append() -
//---------------------------------------------------------------------------
void FftChannelsProxy::Append(FftChannelWrapper *val)
{
    ChannelGroup.push_back(val);
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::AttachSettings() -
//---------------------------------------------------------------------------
void FftChannelsProxy::AttachSettings(IniSaver& saver)
{
    for (size_t i = 0; i < ChannelGroup.size(); i++)
        ChannelGroup[i]->AttachSettings(saver);

    // Atttach cache settings
    saver.Install( ToIni("FftCached.Enable",          Cache.Enable,   true)  );
    saver.Install( ToIni("FftCached.Source",          Cache.Source,    0)  );
    saver.Install( ToIni("FftCached.Window",          Cache.Window,    0)  );
    saver.Install( ToIni("FftCached.Fasd",            Cache.Fasd,    1.0f)  );
    saver.Install( ToIni("FftCached.FasdEnable",      Cache.FasdEnable,   false)  );
    saver.Install( ToIni("FftCached.OutputMode",      Cache.OutputMode,  0)  );
    saver.Install( ToIni("FftCached.TriggerMode",     Cache.TriggerMode, 2)  );
    saver.Install( ToIni("FftCached.Fps",             Cache.Fps,  10.0f)  );
    saver.Install( ToIni("FftCached.ThresholdFile",   Cache.ThresholdFile,   string("..\\..\\Config\\tlsRAM_40db.ini"))  );
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::Clear() -
//---------------------------------------------------------------------------
void FftChannelsProxy::Clear()
{
    while (ChannelGroup.size())
    {
        delete ChannelGroup.back();
        ChannelGroup.pop_back();
    }
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::Size() -
//---------------------------------------------------------------------------
size_t FftChannelsProxy::Size()
{
    return ChannelGroup.size();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::ApplyCached() -
//---------------------------------------------------------------------------

void FftChannelsProxy::ApplyCached()
{
    for (size_t i = 0; i < ChannelGroup.size(); i++)
        {
        ChannelGroup[i]->SRate   = SRate;
        ChannelGroup[i]->Apply();
        }
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::Apply() - Change settings of a single channel
//---------------------------------------------------------------------------

void FftChannelsProxy::Apply(unsigned int value)
{
    if (value < ChannelGroup.size())
        {
        ChannelGroup[value]->Info  = Cache;
        ChannelGroup[value]->SRate = SRate;
        ChannelGroup[value]->Apply();
        }
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::Apply() - Apply to one or more
//---------------------------------------------------------------------------

void FftChannelsProxy::Apply(const vector<int> &values)
{
    for (size_t i = 0; i < values.size(); i++)
         Apply(values[i]);
    CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::PeakChangeApply() - Change settings of a single channel
//---------------------------------------------------------------------------

void FftChannelsProxy::PeakChangeApply(unsigned int value)
{
    if (value < ChannelGroup.size())
        {
        ChannelGroup[value]->Info.OutputMode  = Cache.OutputMode;
        ChannelGroup[value]->PeakChangeApply();
        }
    CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::SourceChangeApply() - Change settings of a single channel
//---------------------------------------------------------------------------

void FftChannelsProxy::SourceChangeApply(unsigned int value)
{
    if (value < ChannelGroup.size())
        {
        ChannelGroup[value]->Info.Source  = Cache.Source;
        ChannelGroup[value]->SourceChangeApply();
        }
    CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::WindowChangeApply() - Change settings of a single channel
//---------------------------------------------------------------------------

void FftChannelsProxy::WindowChangeApply(unsigned int value)
{
    if (value < ChannelGroup.size())
        {
        ChannelGroup[value]->Info.Window  = Cache.Window;
        ChannelGroup[value]->WindowChangeApply();
        }

    CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::TriggerModeApply() -
//---------------------------------------------------------------------------

void FftChannelsProxy::TriggerModeApply(unsigned int value)
{
    if (value < ChannelGroup.size())
        {
        ChannelGroup[value]->Info.TriggerMode  = Cache.TriggerMode;
        ChannelGroup[value]->TriggerModeApply();
        }
    CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::SamplingRate() -
//---------------------------------------------------------------------------

void FftChannelsProxy::SamplingRate(double value)
{
    SRate = value;
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::ApplyEnable() - Enable/disable channel
//---------------------------------------------------------------------------

void FftChannelsProxy::ApplyEnable(unsigned int value)
{
    if (value < ChannelGroup.size())
        {
        ChannelGroup[value]->Info.Enable = Cache.Enable;
        ChannelGroup[value]->ApplyEnable();
        }
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::ApplyEnable() - Apply to one or more
//---------------------------------------------------------------------------

void FftChannelsProxy::ApplyEnable(const vector<int> &values)
{
    for (size_t i = 0; i < values.size(); i++)
         ApplyEnable(values[i]);
    CommandTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::Wrapper() -
//---------------------------------------------------------------------------
FftChannelWrapper &FftChannelsProxy::Wrapper(unsigned int channel)
{
    return *ChannelGroup[channel];
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::ChannelTags() -
//---------------------------------------------------------------------------

void FftChannelsProxy::ChannelTags(vector<std::string> &tags)
{
    for (size_t i = 0; i < ChannelGroup.size(); i++)
        tags.push_back(ChannelGroup[i]->Tag());
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::CommandTrigger() - Trigger each board once
//---------------------------------------------------------------------------

void FftChannelsProxy::CommandTrigger()
{
    unsigned int board = 0;
    for (size_t i = 0; i < ChannelGroup.size(); i++)
        if (board == ChannelGroup[i]->WhichBoard())
            {
            ChannelGroup[i]->CommandTrigger();
            board++; // trigger next board
            }
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::SingleTrigger() -
//---------------------------------------------------------------------------

void FftChannelsProxy::SingleTrigger(const vector<int> &values)
{
    for (size_t i = 0; i < values.size(); i++)
        ChannelGroup[values[i]]->SingleTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::SingleTrigger() - Trigger each board once
//---------------------------------------------------------------------------

void FftChannelsProxy::SingleTrigger()
{
    for (size_t i = 0; i <  ChannelGroup.size(); i++)
        ChannelGroup[i]->SingleTrigger();
}

//---------------------------------------------------------------------------
//  FftChannelsProxy::SingleTrigger() - Trigger each board once
//---------------------------------------------------------------------------

void FftChannelsProxy::Reset(bool state)
{
    unsigned int board = 0;
    for (size_t i = 0; i < ChannelGroup.size(); i++)
        if (board == ChannelGroup[i]->WhichBoard())
            {
            ChannelGroup[i]->Reset(state);
            ChannelGroup[i]->Reset(!state);
            board++; // trigger next board
            }
}
