#ifndef FFTPROXY_GP_H
#define FFTPROXY_GP_H

#include "SpectralModule_Gp.h"
#include <Application/IniSaver_App.h>
#include "DdcModule_Gp.h"

using namespace Innovative;
using namespace std;

struct FftInfo
{
    bool    Enable;
    int     Source;
    int     Window;
    bool    FasdEnable;
    float   Fasd;
    string  ThresholdFile;   
    int     TriggerMode;
    int     OutputMode;
    float   Fps;


    bool operator==(const FftInfo &other)
    {
        return
           other.Enable         == Enable       &&
           other.Source         == Source       &&
           other.Window         == Window       &&
           other.FasdEnable     == FasdEnable   &&
           other.Fasd           == Fasd         &&
           other.ThresholdFile  == ThresholdFile&&
           other.TriggerMode    == TriggerMode  &&
           other.OutputMode     == OutputMode   &&
           other.Fps            == Fps;
    }

};

struct FftPlotRowDataType
{
    bool           Enable;
    int            SourceIndex;
};

//===========================================================================
//  CLASS SpectralPeripheralWrapper --
//===========================================================================
class SpectralPeripheralWrapper
{
public:
    SpectralPeripheralWrapper(SpectralPeripheral *avf, unsigned int acore);
    virtual ~SpectralPeripheralWrapper();

    void BypassSpectrumAnalysis(unsigned int parent, bool value);
    void SourceSelect(unsigned int parent, unsigned int val);
    void BypassWindowing(unsigned int parent, bool value);
    void ModeSelect(unsigned int parent, unsigned int value);
    void Reset(unsigned int parent, bool value);
    void SetSourceMap(const vector<int> &values);

private:
    SpectralPeripheral    * const vf;
    const unsigned int       core;

private:
    SpectralPeripheralWrapper(const SpectralPeripheralWrapper &other);
    SpectralPeripheralWrapper &operator=(const SpectralPeripheralWrapper &other);
};

//===========================================================================
//  CLASS SpectralMultiBufferWrapper --
//===========================================================================
class SpectralMultiBufferWrapper
{
public:
    SpectralMultiBufferWrapper(SpectralMultiBuffer *avf, unsigned int acore);
    virtual ~SpectralMultiBufferWrapper();

    void WindowSize(unsigned int parent, int value);
    void Delay(unsigned int parent, int value);
    void Reset(unsigned int parent, bool value);
    void CalculateWindow(unsigned int parent, unsigned int drate, unsigned int srate, float fpersec, bool arch);
private:
    SpectralMultiBuffer    * const vf;
    const unsigned int       core;

private:
    SpectralMultiBufferWrapper(const SpectralMultiBufferWrapper &other);
    SpectralMultiBufferWrapper &operator=(const SpectralMultiBufferWrapper &other);
};

//===========================================================================
//  CLASS SpectralWindowingWrapper --
//===========================================================================
class SpectralWindowingWrapper
{
public:
    SpectralWindowingWrapper(SpectralWindowing *avf, unsigned int acore);
    virtual ~SpectralWindowingWrapper();


    void Coeffs(unsigned int parent, const vector<int> &cs);
    void Reset(unsigned int parent, bool value);

private:
    SpectralWindowing    * const vf;
    const unsigned int       core;

    void CoeffEnable(unsigned int parent, unsigned int value);
    void CoeffAddr(unsigned int parent, int value);
    void CoeffData(unsigned int parent, int value);
private:
    SpectralWindowingWrapper(const SpectralWindowingWrapper &other);
    SpectralWindowingWrapper &operator=(const SpectralWindowingWrapper &other);
};

//===========================================================================
//  CLASS SpectralFftWrapper --
//===========================================================================
class SpectralFftWrapper
{
public:
    SpectralFftWrapper(SpectralFft *avf, unsigned int acore);
    virtual ~SpectralFftWrapper();

    void Reset(unsigned int parent, bool value);

private:
    SpectralFft    * const vf;
    const unsigned int       core;

private:
    SpectralFftWrapper(const SpectralFftWrapper &other);
    SpectralFftWrapper &operator=(const SpectralFftWrapper &other);
};


//===========================================================================
//  CLASS SpectralPostProcWrapper --
//===========================================================================
class SpectralPostProcWrapper
{
public:
    SpectralPostProcWrapper(SpectralPostProc *avf, unsigned int acore);
    virtual ~SpectralPostProcWrapper();

    void TriggerMode(unsigned int parent, unsigned int value);
    void SingleTrigger(unsigned int parent, int value);

    void FasdFilterCoeff(unsigned int parent, float value, bool enabled);
    void Reset(unsigned int parent, bool value);
    void SetThresholds(unsigned int parent, std::string file);
    void FrameDecimation(unsigned int parent, int value);

private:
    SpectralPostProc    * const vf;
    const unsigned int       core;

    void ThresholdProgEnable(unsigned int parent, unsigned int value);
    void ThresholdProgAddr(unsigned int parent, unsigned int addr);
    void ThresholdProgData(unsigned int parent, int value);
    void ThresholdProg(unsigned int parent, const vector<int> &coeffs);
private:
    SpectralPostProcWrapper(const SpectralPostProcWrapper &other);
    SpectralPostProcWrapper &operator=(const SpectralPostProcWrapper &other);
};

//===========================================================================
//  CLASS SpectralVitaFramerWrapper --
//===========================================================================
class SpectralVitaFramerWrapper
{
public:
    SpectralVitaFramerWrapper(SpectralVitaFramer *avf, unsigned int acore);
    virtual ~SpectralVitaFramerWrapper();

    void Enable(unsigned int parent, bool value);
    void Size(unsigned int parent, unsigned int value);
    void Identity(unsigned int parent, unsigned int did, unsigned int sid);
    void Reset(unsigned int parent, bool value);

private:
    SpectralVitaFramer    * const vf;
    const unsigned int       core;

private:
    SpectralVitaFramerWrapper(const SpectralVitaFramerWrapper &other);
    SpectralVitaFramerWrapper &operator=(const SpectralVitaFramerWrapper &other);
};

//===========================================================================
//  CLASS FftChannelWrapper --
//===========================================================================

class FftChannelWrapper
{
public:
    FftChannelWrapper(SpectralModule *amod, int aid, int abid);
    FftChannelWrapper(SpectralModule *amod, int aid, const FftInfo &ainfo, int abid);
    virtual ~FftChannelWrapper();

    void AddWrapper(SpectralPeripheralWrapper *awrp);
    void AddWrapper(SpectralMultiBufferWrapper *awrp);
    void AddWrapper(SpectralWindowingWrapper *awrp);
    void AddWrapper(SpectralFftWrapper *awrp);
    void AddWrapper(SpectralPostProcWrapper *awrp);
    void AddWrapper(SpectralVitaFramerWrapper *awrp);

    string Tag();
    void AttachSettings(Innovative::IniSaver& saver);
    void CommandTrigger();
    void Apply();
    void ApplyEnable();
    void Reset(bool value);
    unsigned int WhichBoard();
    void SingleTrigger();
    void PeakChangeApply();
    void SourceChangeApply();
    void WindowChangeApply();
    void ConfigureWindowCoeffs(GpDspSubmodule::ArhitectureType atype);
    void DecimationRate(int);
    int  DecimationRate();
    void ReCalculateWindow();
    void WindowSizeAndDelay(int size, int delay);
    void SetSourceMap(const vector<int> &values);
    void TriggerModeApply();
    void SetNumberOfDDC(int value);
    void ApplyFrameDecimation();

    FftInfo Info;
    double  SRate;
    int     DecRate;
    bool    Architecture;
    int     NumberOfDDC;

    void SetFrameModeInfo(const FrameModeInfo &value);
private:
    SpectralModule     * const vf;
    const unsigned int         id;
    const unsigned int        bid;

    FrameModeInfo FrameMode;
    int CalcFrameDecimation();
    int CalcFrameDecimation(float FramePerSec, double FftInputRate);

    vector<vector<int> > coeffs;

    vector<SpectralPeripheralWrapper *>    PeripheralGroup;
    vector<SpectralMultiBufferWrapper *>   MultiBufferGroup;
    vector<SpectralWindowingWrapper *>     WindowingPGroup;
    vector<SpectralFftWrapper *>           FftGroup;
    vector<SpectralPostProcWrapper *>      PostProcGroup;
    vector<SpectralVitaFramerWrapper *>    VitaFramerGroup;


private:
    FftChannelWrapper(const FftChannelWrapper &other);
    FftChannelWrapper &operator=(const FftChannelWrapper &other);


    double InputRate(int Source, int NumOfDdcs, int DecRate, double SampleRate);
};

//===========================================================================
//  CLASS FftChannelWrapper --
//===========================================================================

class FftChannelsProxy
{
public:
    FftChannelsProxy() : SRate(1.0)
    {}

    void Append(FftChannelWrapper *val);
    void AttachSettings(IniSaver& saver);

    void Clear();
    size_t Size();
    FftChannelWrapper &operator[](int i)
    { return *ChannelGroup[i]; }

    FftChannelWrapper &Wrapper(unsigned int channel);
    void ChannelTags(vector<std::string> &tags);
    void ApplyCached();
    void Apply(const vector<int> &values);
    void Apply(unsigned int values);
    void ApplyEnable(const vector<int> &values);
    void ApplyEnable(unsigned int values);
    void CommandTrigger();
    void SingleTrigger(const vector<int> &values);
    void SingleTrigger();
    void SamplingRate(double value);
    void PeakChangeApply(unsigned int value);
    void SourceChangeApply(unsigned int value);
    void WindowChangeApply(unsigned int value);
    void Reset(bool value);
    void TriggerModeApply(unsigned int value);

    FftInfo     Cache;
    double      SRate;

    vector<FftChannelWrapper *> ChannelGroup;

private:
    FftChannelsProxy(const FftChannelsProxy &other);
    FftChannelsProxy &operator=(const FftChannelsProxy &other);
};

#endif // FFTPROXY_GP_H
