// GpDspManager_Gp.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <GpDspManager_Gp.h>
#include <Malibu_Mb.h>
#include <IppMemoryUtils_Mb.h>
#include <GpDsp_Config_Gp.h>

namespace   Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS GpDspManager  -- Hardware Access and Application I/O Class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class GpDspManager
//---------------------------------------------------------------------------

GpDspManager::GpDspManager(IManager_GUIInterface & ui, IManager_FiclInterface & ficl)
    :  GpManagerBase(ui, ficl), MGR(new GpDspManagerInterface(this))
{
}

//---------------------------------------------------------------------------
//  destructor for class GpDspManager
//---------------------------------------------------------------------------

GpDspManager::~GpDspManager()
{
    DrcSettings.Save();

    delete MGR;
}

//---------------------------------------------------------------------------
//  GpDspManager::ConfigureSystem() -
//---------------------------------------------------------------------------

void GpDspManager::ConfigureSystem()
{
    GpManagerBase::ConfigureSystem();
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_Components() - These objects require opened board
//---------------------------------------------------------------------------

void GpDspManager::Make_Components()
{
    GpManagerBase::Make_Components();
}

//---------------------------------------------------------------------------
//  GpDspManager::Reset() -
//---------------------------------------------------------------------------

void GpDspManager::Reset(bool state)
{
    GpManagerBase::Reset(state);
}

//---------------------------------------------------------------------------
//  GpDspManager::Open() - open all hardware
//---------------------------------------------------------------------------

void GpDspManager::Open()
{
    GpManagerBase::Open();
}

//---------------------------------------------------------------------------
//  GpDspManager::IsOpen()
//---------------------------------------------------------------------------

bool GpDspManager::IsOpen(unsigned int which)
{
    if (which < DrcUnits.Size() && Settings.DRC_Targets_Enabled[which])
        return  DrcUnits.Ref_At(which).IsOpen();
    else
        return false;
}

//---------------------------------------------------------------------------
//  GpDspManager::DdcClockFrequencyActual()
//---------------------------------------------------------------------------

double GpDspManager::DdcClockFrequencyActual(unsigned int ordinal)
{
    return Boards(ordinal).AdcClockFrequencyActual();
}

//---------------------------------------------------------------------------
//  GpDspManager::DucClockFrequencyActual()
//---------------------------------------------------------------------------

double GpDspManager::DucClockFrequencyActual(unsigned int ordinal)
{
    return Boards(ordinal).DacClockFrequencyActual();
}

//---------------------------------------------------------------------------
//  GpDspManager::DdcSampleFrequency()
//---------------------------------------------------------------------------

double GpDspManager::DdcSampleFrequency(unsigned int /*ordinal*/)
{
    return DrcSettings.SampleRate;
}

//---------------------------------------------------------------------------
//  GpDspManager::DataPacketAvailable()
//---------------------------------------------------------------------------

void GpDspManager::DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer)
{
    GpManagerBase::Drc_DataPacketAvailable(ordinal, buffer);
}

//---------------------------------------------------------------------------
//  GpDspManager::BeforeStreamStart()
//---------------------------------------------------------------------------

void GpDspManager::BeforeStreamStart(unsigned int ordinal)
{
    GpManagerBase::BeforeStreamStart(ordinal);
}

//---------------------------------------------------------------------------
//  GpDspManager::LogicVersion()
//---------------------------------------------------------------------------

string GpDspManager::LogicVersion(unsigned int which)
{
    if (which < DrcUnits.Size() && Settings.DRC_Targets_Enabled[which] && IsOpen(which))
        return DrcUnits.Ref_At(which).LogicVersion();
    else
        return "N/A";
}

//---------------------------------------------------------------------------
//  GpDspManager::CoreVersion()
//---------------------------------------------------------------------------

pair<string, string> GpDspManager::CoreVersion(unsigned int which)
{
    pair<string, string> result;
    result.first = result.second = "N/A";

    if (which < DrcUnits.Size() && Settings.DRC_Targets_Enabled[which]&& IsOpen(which))
        return DrcUnits.Ref_At(which).CoreVersion();
    else
        return result;
}

//---------------------------------------------------------------------------
//  GpDspManager::Close() - close all hardware
//---------------------------------------------------------------------------

void GpDspManager::Close()
{
    GpManagerBase::Close();
}

//---------------------------------------------------------------------------
//  GpDspManager::StartStreaming() - start streaming on boards
//---------------------------------------------------------------------------

void GpDspManager::StartStreaming()
{
    GpManagerBase::StartStreaming();
}

//---------------------------------------------------------------------------
//  GpDspManager::StopStreaming()  - stop streaming on boards
//---------------------------------------------------------------------------

void GpDspManager::StopStreaming()
{
    GpManagerBase::StopStreaming();
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_Drc()
//---------------------------------------------------------------------------

void GpDspManager::Make_Drc()
{
    PMix_Make_Drc(DrcSettings, MGR, Settings);
    // Let modules update defaults
    for (size_t i = 0; i < DrcUnits.Size(); i++)
            DrcUnits.Ref_At(i).PassDefaults();
    // Load Settings
    DrcSettings.LoadINISettings();
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_Clocks()
//---------------------------------------------------------------------------

void GpDspManager::Make_Clocks()
{
    Innovative::Trace Debug;

    Debug.Log("Clocks ", Settings.Clock_Module_Enabled ? "Enabled" : "Disabled");
    Debug.Log("Selected Module ", Settings.Clock_Module_Type);

    if (Settings.Clock_Module_Enabled) {
        if (Settings.Clock_Module_Type == "x3-timing") {
            ClockModules.Append(new X3ClockWrapper("x3-timing", 0));
            Debug.Log("Created X3ClockWrapper");
        } else if (Settings.Clock_Module_Type == "sbc-duo") {
            ClockModules.Append(new SbcDuoClockWrapper("sbc-duo", 0));
            Debug.Log("Created SbcDuoClockWrapper");
        } else if (Settings.Clock_Module_Type == "atropos") {
            ClockModules.Append(new AtroposClockWrapper("atropos", 0));
            Debug.Log("Created AtroposClockWrapper");
        } else {
            ClockModules.Append(new X3ClockWrapper("x3-timing", 0));
            Debug.Log("Defaulted to X3ClockWrapper");
        }
    }

}

//---------------------------------------------------------------------------
//  GpDspManager::Open_Drc()
//---------------------------------------------------------------------------

void GpDspManager::Open_Drc()
{
    PMix_Open_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDspManager::Close_Drc()
//---------------------------------------------------------------------------

void GpDspManager::Close_Drc()
{
    PMix_Close_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDspManager::StartStreaming_Drc()
//---------------------------------------------------------------------------

void GpDspManager::StartStreaming_Drc()
{
    PMix_StartStreaming_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDspManager::StopStreaming_Drc()
//---------------------------------------------------------------------------

void GpDspManager::StopStreaming_Drc()
{
    PMix_StopStreaming_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_Ficls()
//---------------------------------------------------------------------------

void GpDspManager::Make_Ficls()
{
    PMix_Make(FiclUnits, FICL);
}

//---------------------------------------------------------------------------
//  GpDspManager::MakeDdcChannels()
//---------------------------------------------------------------------------

void GpDspManager::Make_DdcChannels()
{
    PMix_Make(DdcChannels, Settings.DRC_Targets_Enabled);
    // Add more settings
    DdcChannels.AttachSettings(DrcSettings);
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_DucChannels()
//---------------------------------------------------------------------------

void GpDspManager::Make_DucChannels()
{

}

//---------------------------------------------------------------------------
//  GpDspManager::Make_FftChannels()
//---------------------------------------------------------------------------

void GpDspManager::Make_FftChannels()
{
    PMix_Make(FftChannels, Settings.DRC_Targets_Enabled);
    // Add more settings
    FftChannels.AttachSettings(DrcSettings);
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_PowerMeter()
//---------------------------------------------------------------------------

void GpDspManager::Make_PowerMeter()
{
    PMix_Make(PowerMeter, Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_OverflowDetector()
//---------------------------------------------------------------------------

void GpDspManager::Make_OverflowDetector()
{
    PMix_Make(OverflowDetector, Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDspManager::Make_ToneGenerator()
//---------------------------------------------------------------------------

void GpDspManager::Make_ToneGenerator()
{
    PMix_Make(ToneGenerator, Settings.DRC_Targets_Enabled);
    // Add more settings
    ToneGenerator.AttachSettings(DrcSettings);
    // Load settings again
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDspManager::LoadDrcSettings() we want to call this once after making all
//                                  Components
//---------------------------------------------------------------------------

void GpDspManager::LoadDrcSettings()
{
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDspManager::Debug_SoftwareAlert() - pass software alert down to a board
//---------------------------------------------------------------------------

void GpDspManager::Debug_SoftwareAlert(unsigned int ordinal, unsigned int value)
{
    if (ordinal<DrcUnits.Size())
        DrcUnits[ordinal].SoftwareAlert(value);
}

//---------------------------------------------------------------------------
//  GpDspManager::Debug_ClockInfo() - pass clock info request down to a board
//---------------------------------------------------------------------------

void GpDspManager::Debug_ClockInfo(unsigned int ordinal)
{
    if (ordinal<DrcUnits.Size())
        DrcUnits[ordinal].ClockInfo();
}

//---------------------------------------------------------------------------
//  GpDspManager::PeekLogic() --
//---------------------------------------------------------------------------

unsigned int GpDspManager::PeekLogic(unsigned int ordinal, int offset)
{
    if (ordinal<DrcUnits.Size())
        return DrcUnits[ordinal].PeekLogic(offset);
    else
        return 0;
}

//---------------------------------------------------------------------------
//  GpDspManager::PokeLogic() --
//---------------------------------------------------------------------------

void GpDspManager::PokeLogic(unsigned int ordinal, int offset, unsigned int value)
{
    if (ordinal<DrcUnits.Size())
        DrcUnits[ordinal].PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  GpDspManager::SamplingDurationToSamples() --
//---------------------------------------------------------------------------

ii64 GpDspManager::SamplingDurationToSamples(float duration)
{
    //
    // If in raw mode, use simple calculation
    if (DdcChannels.BypassDdc)
        {
        int sum = 0;
        for (size_t i = 0; i < DdcChannels.Size(); i++)
            {
            if (DdcChannels[i]->info.Enable)
                sum++;
            }
        return duration * DrcSettings.SampleRate * 1e6 * sum;
        }

    // Otherwise use DDC calculation.

    ii64 sum = 0;
    // If IQ multipy by 2
    float multiplier = Boards(0).IsIQEnabled() ? 2 : 1;
    // If 8-bit divide by 2
    multiplier /= (Boards(0).SampleWidth() == 8) ? 2 : 1;

    // Total Sample Rate in 1 sec
    for (size_t i = 0; i < DdcChannels.Size(); i++)
        {
        if (DdcChannels[i]->info.Enable)
            {
            DdcChannels.Wrapper(i).ReloadFile();
            sum += ((DrcSettings.SampleRate * 1e6 / DdcChannels.DecimationCoeffs(i)) );
            }
        }

    // Number of enabled boards
    size_t nboards = std::count(Settings.DRC_Targets_Enabled.begin(),
                                Settings.DRC_Targets_Enabled.end(), true);

    return (sum * duration * multiplier) / nboards;
}



#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

