// GpDspManager_Gp.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef GpDspManager_GpH
#define GpDspManager_GpH

#include <GpManagerBase_Gp.h>
#include <DrcProvider_Gp.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif



//==============================================================================
//  CLASS  GpDspManager  --  Manages interaction with GUI, Board(s)
//==============================================================================

class GpDspManager : public GpManagerBase, public ProviderMixin<Drc_Provider>
{
public:
    GpDspManager(IManager_GUIInterface & ui, IManager_FiclInterface & ficl);
    ~GpDspManager();

    friend class GpDspManagerInterface;
    //enum CommandTriggerType {eDDC, eCommon, eBoth};

    //  Standard Operations
    void ConfigureSystem();
    void Make_Components();
    void Open();
    void Close();
    void StartStreaming();
    void StopStreaming();

    void Reset(bool state);
    void BeforeStreamStart(unsigned int ordinal);

    //  ...other operations
    void Debug_SoftwareAlert(unsigned int ordinal, unsigned int value);
    void Debug_ClockInfo(unsigned int ordinal);

    ii64          SamplingDurationToSamples(float duration);

    Drc_ProviderSettings       DrcSettings;

    virtual bool    IsOpen(unsigned int which);
    virtual int     BoardCount() const
                        {  return PMix_BoardCount();  }
    virtual string  LogicVersion(unsigned int which);
    virtual pair<string, string>  CoreVersion(unsigned int which);
    virtual double  DdcClockFrequencyActual(unsigned int ordinal);
    virtual double  DucClockFrequencyActual(unsigned int ordinal);
    virtual double  DdcSampleFrequency(unsigned int ordinal);
    virtual void    DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer);
    unsigned int    PeekLogic(unsigned int ordinal, int offset);
    void            PokeLogic(unsigned int ordinal, int offset, unsigned int value);

protected:

    class GpDspManagerInterface *  MGR;

    //
    //  Unique Interface for Specific Managers....
    virtual void  Make_Drc();  // Make Provider Lists
    virtual void  Open_Drc();
    virtual void  Close_Drc();
    virtual void  StartStreaming_Drc();
    virtual void  StopStreaming_Drc();

    virtual  void  Make_Ficls();
    virtual  void  Make_DdcChannels();
	virtual  void  Make_DucChannels();
    virtual  void  Make_FftChannels();
    virtual  void  Make_PowerMeter();
    virtual  void  Make_OverflowDetector();
    virtual  void  Make_ToneGenerator();
    virtual  void  Make_Clocks();
    virtual  void  LoadDrcSettings();

};

//===========================================================================
//  CLASS GpDspManagerInterface  -- Callback from DRC provider to manager
//===========================================================================

class GpDspManagerInterface : public IDrcManagerInterface
{
public:
    GpDspManagerInterface(GpDspManager * mgr)
                : MGR(mgr)
                    {  }
    ~GpDspManagerInterface()  {  }

    virtual void  Log(unsigned int ordinal, const std::string & a_string)
        {  MGR->Drc_Log(ordinal, a_string);  }
    virtual void  AfterStreamAutoStop(unsigned int ordinal)
        {  MGR->Drc_AfterStreamAutoStop(ordinal);  }
    virtual void  PeriodicStatusUpdate(unsigned int ordinal)
        {  MGR->Drc_PeriodicStatusUpdate(ordinal);  }
    virtual void  BeforeStreamStart(unsigned int ordinal)
        {  MGR->Drc_BeforeStreamStart(ordinal);  }
    virtual void  AfterStreamStart(unsigned int ordinal)
        {  MGR->Drc_AfterStreamStart(ordinal);  }
    virtual unsigned int EnabledChannels(unsigned int ordinal)
        { return MGR->EnabledChannels(ordinal); }
    virtual unsigned int EnabledDevices(unsigned int ordinal)
        { return MGR->EnabledDevices(ordinal); }
    virtual float DecimationFactor(unsigned int ordinal)
        { return MGR->DecimationFactor(ordinal); }
    virtual unsigned int EnabledChannelizerChannels(unsigned int ordinal)
        { return MGR->EnabledChannelizerChannels(ordinal); }
    virtual void Plot(unsigned int ordinal, VeloBuffer &buffer, int type)
        { MGR->Plot(ordinal, buffer, type); }
    virtual void DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer)
        { MGR->DataPacketAvailable(ordinal, buffer); }
    virtual string ChannelMap(unsigned int ordinal)
        { return MGR->ChannelMap(ordinal); }

private:
    GpDspManager *  MGR;
};



#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // end namespace Innovative


#endif // GpDspManager_GpH
