#ifndef GPDSPMEMORYSTRINGMAP_GP_H
#define GPDSPMEMORYSTRINGMAP_GP_H

#include <HardwareRegister_Mb.h>

//===========================================================================
//  ENUM WishboneType  --  This should match the memory maps file
//===========================================================================

enum WishboneType {
                   ewbCustom,
                   ewbDspCommon,
                   ewbDdc,
                   ewbDuc,
                   ewbSpectrumAnalysis,
                   ewbBeamforming,
                   ewbPSK,
                   ewbFSK
                  };

// Enable only for extended register diagnosis
#if  1
//EnableDebuggingSupport

#include <Trace_Mb.h>
#include <map>
#include <string>



//===========================================================================
//  CLASS RegisterNames  --  Map register offset to name
//===========================================================================

class RegisterNames
{
public:

    RegisterNames()
        {
        Map_wbCustom[0x00] = "mmPrjVersion";
        Map_wbCustom[0x01] = "mmPrj_MajorCores";

        Map_wbDspCommon[0x00] = "mmCommonHdr_CoresPresent";
        Map_wbDspCommon[0x01] = "mmCommonHdr_CmdTrigger";
        Map_wbDspCommon[0x02] = "mmCommonHdr_CoreIndex";
        Map_wbDspCommon[0x03] = "mmCommonHdr_ParentIndex";
        // ...
        Map_wbDspCommon[0x20] = "mmPowerMeterInfo";
        Map_wbDspCommon[0x21] = "mmPowerMeterConfig";
        Map_wbDspCommon[0x23] = "mmPowerMeterReset";
        Map_wbDspCommon[0x24] = "mmPowerMeterAccPoints";
        Map_wbDspCommon[0x25] = "mmPowerMeterSource";
        Map_wbDspCommon[0x26] = "mmPowerMeterData";
        Map_wbDspCommon[0x27] = "mmPowerMeterReady";

        Map_wbDspCommon[0x30] = "mmOverflowInfo";
        Map_wbDspCommon[0x31] = "mmOverflowConfig";
        Map_wbDspCommon[0x33] = "mmOverflowReset";
        Map_wbDspCommon[0x34] = "mmOverflowData";

        Map_wbDspCommon[0x40] = "mmFifoStatusInfo";
        Map_wbDspCommon[0x41] = "mmFifoStatusConfig";
        Map_wbDspCommon[0x43] = "mmFifoStatusReset";
        Map_wbDspCommon[0x44] = "mmFifoStatusOverflow";
        Map_wbDspCommon[0x45] = "mmFifoStatusUnderflow";

        Map_wbDspCommon[0x50] = "mmToneGenInfo";
        Map_wbDspCommon[0x51] = "mmToneGenConfig";
        Map_wbDspCommon[0x53] = "mmToneGenReset";
        Map_wbDspCommon[0x54] = "mmToneGenFreq1";
        Map_wbDspCommon[0x55] = "mmToneGenPhase1";
        Map_wbDspCommon[0x56] = "mmToneGenGain1";
        Map_wbDspCommon[0x58] = "mmToneGenFreq2";
        Map_wbDspCommon[0x59] = "mmToneGenPhase2";
        Map_wbDspCommon[0x5A] = "mmToneGenGain2";

        Map_wbDspCommon[0x60] = "mmAWGNGenInfo";
        Map_wbDspCommon[0x61] = "mmAWGNGenConfig";
        Map_wbDspCommon[0x63] = "mmAWGNGenReset";
        Map_wbDspCommon[0x64] = "mmAWGNGenGain";

        Map_wbDspCommon[0x70] = "mmPowerMgrInfo";
        Map_wbDspCommon[0x71] = "mmPowerMgrConfig";
        Map_wbDspCommon[0x73] = "mmPowerMgrReset";
        Map_wbDspCommon[0x74] = "mmPowerMgrGain";

        Map_wbDdc[0x00] = "mmDdcHdr_CoresPresent";
        Map_wbDdc[0x01] = "mmDdcHdr_CmdTrigger";
        Map_wbDdc[0x02] = "mmDdcHdr_CoreIndex";
        Map_wbDdc[0x03] = "mmDdcHdr_ParentIndex";

        Map_wbDdc[0x06] = "mmDdcDecimation";

        Map_wbDdc[0x10] = "mmDdcPeriphInfo";
        Map_wbDdc[0x11] = "mmDdcPeriphConfig";

        Map_wbDdc[0x13] = "mmDdcPeriphReset";
        Map_wbDdc[0x14] = "mmDdcPeriphBypass";
        Map_wbDdc[0x15] = "mmDdcPeriphSource";
        Map_wbDdc[0x16] = "mmDdcPeriphSpectralInv";
        Map_wbDdc[0x17] = "mmDdcPeriphFilterSel1";
        Map_wbDdc[0x18] = "mmDdcPeriphFilterSel2";

        // DUC
		Map_wbDuc[0x00] = "mmDucHdr_CoresPresent";
        Map_wbDuc[0x01] = "mmDucHdr_CmdTrigger";
        Map_wbDuc[0x02] = "mmDucHdr_CoreIndex";
        Map_wbDuc[0x03] = "mmDucHdr_ParentIndex";

		Map_wbDuc[0x10] = "mmDucPeriphInfo";
        Map_wbDuc[0x11] = "mmDucPeriphConfig";

		Map_wbDuc[0x13] = "mmDucPeriphReset";
        Map_wbDuc[0x14] = "mmDucPeriphBypass";
        Map_wbDuc[0x15] = "mmDucPeriphSource";
        Map_wbDuc[0x16] = "mmDucPeriphSpectralInv";
        Map_wbDuc[0x17] = "mmDucPeriphFilterSel1";
        Map_wbDuc[0x18] = "mmDucPeriphFilterSel2";
		Map_wbDuc[0x1f] = "mmDucPeriphInputStreamId";

		Map_wbDuc[0x20] = "mmDucTunerInfo";
        Map_wbDuc[0x21] = "mmDucTunerConfig";
        Map_wbDuc[0x23] = "mmDucTunerReset";
        Map_wbDuc[0x24] = "mmDucTunerFrequency";
        Map_wbDuc[0x25] = "mmDucTunerPhaseOfs";

        Map_wbDuc[0x30] = "mmDucPolyPhaseInfo";
        Map_wbDuc[0x31] = "mmDucPolyPhaseConfig";
        Map_wbDuc[0x33] = "mmDucPolyPhaseReset";
        Map_wbDuc[0x34] = "mmDucPolyPhaseCoeffs";
		Map_wbDuc[0x35] = "mmDucPolyPhaseBStart";

        Map_wbDuc[0x40] = "mmDucLowpassInfo";
        Map_wbDuc[0x41] = "mmDucLowpassConfig";
        Map_wbDuc[0x43] = "mmDucLowpassReset";
        Map_wbDuc[0x44] = "mmDucLowpassCicRate";
        Map_wbDuc[0x45] = "mmDucLowpassCicLsb";
        Map_wbDuc[0x46] = "mmDucLowpassCfirCoeff";
        Map_wbDuc[0x47] = "mmDucLowpassCfirBStart";
        Map_wbDuc[0x48] = "mmDucLowpassPfirCoeff";
        Map_wbDuc[0x49] = "mmDucLowpassPfirBStart";
        Map_wbDuc[0x4A] = "mmDucLowpassFir1Coeff";
        Map_wbDuc[0x4B] = "mmDucLowpassFir1BStart";
        Map_wbDuc[0x4C] = "mmDucLowpassFir2Coeff";
        Map_wbDuc[0x4D] = "mmDucLowpassFir2BStart";

		Map_wbDdc[0x20] = "mmDdcTunerInfo";
        Map_wbDdc[0x21] = "mmDdcTunerConfig";
        Map_wbDdc[0x23] = "mmDdcTunerReset";
        Map_wbDdc[0x24] = "mmDdcTunerFrequency";
        Map_wbDdc[0x25] = "mmDdcTunerPhaseOfs";
        Map_wbDdc[0x26] = "mmDdcTunerNumFreq";
        Map_wbDdc[0x27] = "mmDdcTunerProgEn";
        Map_wbDdc[0x28] = "mmDdcTunerSchAddr";
        Map_wbDdc[0x29] = "mmDdcTunerSchData";
        Map_wbDdc[0x2A] = "mmDdcTunerSchDout";

//        Map_wbDdc[0x30] = "mmDdcPolyPhaseInfo";
//        Map_wbDdc[0x31] = "mmDdcPolyPhaseConfig";
//        Map_wbDdc[0x33] = "mmDdcPolyPhaseReset";
//        Map_wbDdc[0x34] = "mmDdcPolyPhaseCoeffs";
//        Map_wbDdc[0x35] = "mmDdcPolyPhaseBStart";

//        Map_wbDdc[0x40] = "mmDdcLowpassInfo";
//        Map_wbDdc[0x41] = "mmDdcLowpassConfig";
//        Map_wbDdc[0x43] = "mmDdcLowpassReset";
//        Map_wbDdc[0x44] = "mmDdcLowpassCicRate";
//        Map_wbDdc[0x45] = "mmDdcLowpassCicLsb";
//        Map_wbDdc[0x46] = "mmDdcLowpassCfirCoeff";
//        Map_wbDdc[0x47] = "mmDdcLowpassCfirBStart";
//        Map_wbDdc[0x48] = "mmDdcLowpassPfirCoeff";
//        Map_wbDdc[0x49] = "mmDdcLowpassPfirBStart";
//        Map_wbDdc[0x4A] = "mmDdcLowpassFir1Coeff";
//        Map_wbDdc[0x4B] = "mmDdcLowpassFir1BStart";
//        Map_wbDdc[0x4C] = "mmDdcLowpassFir2Coeff";
//        Map_wbDdc[0x4D] = "mmDdcLowpassFir2BStart";

        Map_wbDdc[0x60] = "mmDdcCalibInfo";
        Map_wbDdc[0x61] = "mmDdcCalibConfig";
        Map_wbDdc[0x63] = "mmDdcCalibReset";
        Map_wbDdc[0x64] = "mmDdcCalibOffset";
        Map_wbDdc[0x65] = "mmDdcCalibGain";

        Map_wbDdc[0x70] = "mmDdcChannelizerInfo";
        Map_wbDdc[0x71] = "mmDdcChannelizerConfig";
        Map_wbDdc[0x73] = "mmDdcChannelizerReset";
        Map_wbDdc[0x74] = "mmDdcChannelizerScale";

        Map_wbDdc[0xE0] = "mmDdcVitaInfo";
        Map_wbDdc[0xE1] = "mmDdcVitaConfig";
        Map_wbDdc[0xE3] = "mmDdcVitaReset";
        Map_wbDdc[0xE4] = "mmDdcVitaEnable";
        Map_wbDdc[0xE5] = "mmDdcVitaSize";
        Map_wbDdc[0xE6] = "mmDdcVitaSid";
        Map_wbDdc[0xE8] = "mmDdcVitaFrmMode ";
        Map_wbDdc[0xE9] = "mmDdcVitaFrmSize ";
        Map_wbDdc[0xEA] = "mmDdcVitaFrmCount";
        Map_wbDdc[0xEB] = "mmDdcVitaFrmDelay";
        Map_wbDdc[0xEC] = "mmDdcVitaFrmReEnable";

        Map_wbDdc[0xF0] = "mmDdcDebugInfo";
        Map_wbDdc[0xF1] = "mmDdcDebugConfig";
        Map_wbDdc[0xF3] = "mmDdcDebugReset";
        Map_wbDdc[0xF4] = "mmDdcDebugReg1";

        Map_wbDuc[0xE0] = "mmDucVitaInfo";
        Map_wbDuc[0xE1] = "mmDucVitaConfig";
        Map_wbDuc[0xE3] = "mmDucVitaReset";
        Map_wbDuc[0xE4] = "mmDucVitaEnable";
        Map_wbDuc[0xE5] = "mmDucVitaSize";
        Map_wbDuc[0xE6] = "mmDucVitaSid";

        Map_wbDuc[0xF0] = "mmDucDebugInfo";
        Map_wbDuc[0xF1] = "mmDucDebugConfig";
        Map_wbDuc[0xF3] = "mmDucDebugReset";
        Map_wbDuc[0xF4] = "mmDucDebugReg1";

        Map_wbSpectrumAnalysis[0x00] = "mmSaHdr_CoresPresent";
        Map_wbSpectrumAnalysis[0x01] = "mmSaHdr_CmdTrigger";
        Map_wbSpectrumAnalysis[0x02] = "mmSaHdr_CoreIndex";
        Map_wbSpectrumAnalysis[0x03] = "mmSaHdr_ParentIndex";

        Map_wbSpectrumAnalysis[0x10] = "mmSaPeriphInfo";
        Map_wbSpectrumAnalysis[0x11] = "mmSaPeriphConfig";
        Map_wbSpectrumAnalysis[0x13] = "mmSaPeriphReset";
        Map_wbSpectrumAnalysis[0x14] = "mmSaPeriphBypass";
        Map_wbSpectrumAnalysis[0x15] = "mmSaPeriphSource";
        Map_wbSpectrumAnalysis[0x16] = "mmSaPeriphBypassWin";
        Map_wbSpectrumAnalysis[0x17] = "mmSaPeriphOutputMode";

        Map_wbSpectrumAnalysis[0x20] = "mmSaMultiBufInfo";
        Map_wbSpectrumAnalysis[0x21] = "mmSaMultiBufConfig";
        Map_wbSpectrumAnalysis[0x23] = "mmSaMultiBufReset";
        Map_wbSpectrumAnalysis[0x24] = "mmSaMultiBufWinSize";
        Map_wbSpectrumAnalysis[0x25] = "mmSaMultiBufDelay";

        Map_wbSpectrumAnalysis[0x30] = "mmSaWindowingInfo";
        Map_wbSpectrumAnalysis[0x31] = "mmSaWindowingConfig";
        Map_wbSpectrumAnalysis[0x33] = "mmSaWindowingReset";
        Map_wbSpectrumAnalysis[0x34] = "mmSaWindowingCoeffAddr";
        Map_wbSpectrumAnalysis[0x35] = "mmSaWindowingCoeffData";

        Map_wbSpectrumAnalysis[0x40] = "mmSaFftInfo";
        Map_wbSpectrumAnalysis[0x41] = "mmSaFftConfig";
        Map_wbSpectrumAnalysis[0x43] = "mmSaFftReset";

        Map_wbSpectrumAnalysis[0x50] = "mmSaPostProcInfo";
        Map_wbSpectrumAnalysis[0x51] = "mmSaPostProcConfig";
        Map_wbSpectrumAnalysis[0x53] = "mmSaPostProcReset";
        Map_wbSpectrumAnalysis[0x54] = "mmSaPostProcTrigMode";
        Map_wbSpectrumAnalysis[0x55] = "mmSaPostProcSingleTrig";
        Map_wbSpectrumAnalysis[0x58] = "mmSaPostProcProgEn";
//        Map_wbSpectrumAnalysis[0x59] = "mmSaPostProcProgAddr";
//        Map_wbSpectrumAnalysis[0x5A] = "mmSaPostProcProgData";
        Map_wbSpectrumAnalysis[0x5B] = "mmSaPostProcDataReadback";
        Map_wbSpectrumAnalysis[0x5C] = "mmSaPostProcFasdCoeff";

        Map_wbSpectrumAnalysis[0xE0] = "mmSaVitaInfo";
        Map_wbSpectrumAnalysis[0xE1] = "mmSaVitaConfig";
        Map_wbSpectrumAnalysis[0xE3] = "mmSaVitaReset";
        Map_wbSpectrumAnalysis[0xE4] = "mmSaVitaEnable";
        Map_wbSpectrumAnalysis[0xE5] = "mmSaVitaSize";
        Map_wbSpectrumAnalysis[0xE6] = "mmSaVitaSid";

        Map_wbSpectrumAnalysis[0xF0] = "mmSaDebugInfo";
        Map_wbSpectrumAnalysis[0xF1] = "mmSaDebugConfig";
        Map_wbSpectrumAnalysis[0xF3] = "mmSaDebugReset";
        Map_wbSpectrumAnalysis[0xF4] = "mmSaDebugReg1";

        Map_wbBeamforming[0xE0] = "mmBfVitaInfo";
        Map_wbBeamforming[0xE1] = "mmBfVitaConfig";
        Map_wbBeamforming[0xE3] = "mmBfVitaReset";
        Map_wbBeamforming[0xE4] = "mmBfVitaEnable";
        Map_wbBeamforming[0xE5] = "mmBfVitaSize";
        Map_wbBeamforming[0xE6] = "mmBfVitaSid";

        Map_wbBeamforming[0xF0] = "mmBfDebugInfo";
        Map_wbBeamforming[0xF1] = "mmBfDebugConfig";
        Map_wbBeamforming[0xF3] = "mmBfDebugReset";
        Map_wbBeamforming[0xF4] = "mmBfDebugReg1";

        Map_wbPSK[0xE0] = "mmPskVitaInfo";
        Map_wbPSK[0xE1] = "mmPskVitaConfig";
        Map_wbPSK[0xE3] = "mmPskVitaReset";
        Map_wbPSK[0xE4] = "mmPskVitaEnable";
        Map_wbPSK[0xE5] = "mmPskVitaSize";
        Map_wbPSK[0xE6] = "mmPskVitaSid";

        Map_wbPSK[0xF0] = "mmPskDebugInfo";
        Map_wbPSK[0xF1] = "mmPskDebugConfig";
        Map_wbPSK[0xF3] = "mmPskDebugReset";
        Map_wbPSK[0xF4] = "mmPskDebugReg1";

        Map_wbFSK[0xE0] = "mmFskVitaInfo";
        Map_wbFSK[0xE1] = "mmFskVitaConfig";
        Map_wbFSK[0xE3] = "mmFskVitaReset";
        Map_wbFSK[0xE4] = "mmFskVitaEnable";
        Map_wbFSK[0xE5] = "mmFskVitaSize";
        Map_wbFSK[0xE6] = "mmFskVitaSid";

        Map_wbFSK[0xF0] = "mmFskDebugInfo";
        Map_wbFSK[0xF1] = "mmFskDebugConfig";
        Map_wbFSK[0xF3] = "mmFskDebugReset";
        Map_wbFSK[0xF4] = "mmFskDebugReg1";

        Map_wb[ewbCustom]           = Map_wbCustom;
        Map_wb[ewbDspCommon]        = Map_wbDspCommon;
        Map_wb[ewbDdc]              = Map_wbDdc;
        Map_wb[ewbDuc]              = Map_wbDuc;
        Map_wb[ewbSpectrumAnalysis] = Map_wbSpectrumAnalysis;
        Map_wb[ewbBeamforming]      = Map_wbBeamforming;
        Map_wb[ewbPSK]              = Map_wbPSK;
        Map_wb[ewbFSK]              = Map_wbFSK;

        }

    std::string Name(int offset, WishboneType type)
        {
         if (Map_wb.count(type))
            if (Map_wb.find(type)->second.count(offset))
                return Map_wb.find(type)->second.find(offset)->second;
             else
                return std::string(); // no reg.
         else
            return "no wish.";
        }

private:
    typedef std::map<int, std::string>  StringMapType;

    StringMapType   Map_wbCustom;
    StringMapType   Map_wbDspCommon;
    StringMapType   Map_wbDdc;
    StringMapType   Map_wbDuc;
    StringMapType   Map_wbSpectrumAnalysis;
    StringMapType   Map_wbBeamforming;
    StringMapType   Map_wbPSK;
    StringMapType   Map_wbFSK;

    std::map<WishboneType, StringMapType>  Map_wb;
};



class CommonShadowRegister : public Innovative::ShadowRegister
{
    typedef ShadowRegister inherited;

protected:
    CommonShadowRegister(Innovative::IAddressingSpace &space, int offset, WishboneType awtype)
        : inherited(space, offset), Offset(offset), wtype(awtype), NoApply(false)
        {
            Innovative::TraceVerbosity(Innovative::Trace::vNormal);
        }

protected:
    Innovative::Trace           Debug;
    int                         Offset;
    RegisterNames               Reg;
    WishboneType                wtype;
    bool                        NoApply;

    virtual void ShadowRegisterChanged()
    {
        std::stringstream msg;
        std::string name = Reg.Name(Offset, wtype);

        if (!name.empty()) {
            msg << "Reg[0x" << std::hex << Offset << "] = 0x"
                << mShadowValue  << "  (" << name << ")";
            Debug.Log(msg.str().c_str());
        }

        if (!NoApply) Apply();
    }
};

class CustomShadowRegister : public CommonShadowRegister
{
public:
    CustomShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbCustom)
        {
        }
};

class DspCommonShadowRegister : public CommonShadowRegister
{
public:
    DspCommonShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbDspCommon)
        {
        }
};

class DdcShadowRegister : public CommonShadowRegister
{
public:
    DdcShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbDdc)
        {
        }
};

class DdcCachedShadowRegister : public CommonShadowRegister
{
public:
    DdcCachedShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbDdc)
        {
        NoApply = true;
        }
};

class DucShadowRegister : public CommonShadowRegister
{
public:
    DucShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbDuc)
        {
        }
};

class DucCachedShadowRegister : public CommonShadowRegister
{
public:
    DucCachedShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbDuc)
        {
        NoApply = true;
        }
};

class SaShadowRegister : public CommonShadowRegister
{
public:
    SaShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbSpectrumAnalysis)
        {
        }
};

class SaCachedShadowRegister : public CommonShadowRegister
{
public:
    SaCachedShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbSpectrumAnalysis)
        {
        NoApply = true;
        }
};

class BfShadowRegister : public CommonShadowRegister
{
public:
    BfShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbBeamforming)
        {
        }
};

class PSKShadowRegister : public CommonShadowRegister
{
public:
    PSKShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbPSK)
        {
        }
};

class FSKShadowRegister : public CommonShadowRegister
{
public:
    FSKShadowRegister(Innovative::IAddressingSpace &space, int offset)
        : CommonShadowRegister(space, offset, ewbFSK)
        {
        }
};

#else

class NullCommonShadowRegister : public Innovative::ShadowRegister
{
    typedef Innovative::ShadowRegister inherited;

protected:
    NullCommonShadowRegister(Innovative::IAddressingSpace &space, int offset, WishboneType)
        : inherited(space, offset)
        {}
};

typedef NullCommonShadowRegister   CommonShadowRegister;
typedef Innovative::ShadowRegister CustomShadowRegister;
typedef Innovative::ShadowRegister DspCommonShadowRegister;
typedef Innovative::CachedShadowRegister DdcCachedShadowRegister;
typedef Innovative::CachedShadowRegister DucCachedShadowRegister;
typedef Innovative::CachedShadowRegister SaCachedShadowRegister;
typedef Innovative::ShadowRegister DdcShadowRegister;
typedef Innovative::ShadowRegister DucShadowRegister;
typedef Innovative::ShadowRegister DucShadowRegister;
typedef Innovative::ShadowRegister SaShadowRegister;
typedef Innovative::ShadowRegister BfShadowRegister;
typedef Innovative::ShadowRegister PSKShadowRegister;
typedef Innovative::ShadowRegister FSKShadowRegister;
#endif

#endif // GPDSPMEMORYSTRINGMAP_GP_H
