// GpDucManager_Gp.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//

#include <GpDucManager_Gp.h>
#include <GpDsp_Config_Gp.h>

namespace   Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS GpDucManager  -- Hardware Access and Application I/O Class
//===========================================================================
//---------------------------------------------------------------------------
//  constructor for class GpDucManager
//---------------------------------------------------------------------------

GpDucManager::GpDucManager(IManager_GUIInterface & ui, IManager_FiclInterface & ficl)
    :  GpManagerBase(ui, ficl), MGR(new GpDucManagerInterface(this))
{
}

//---------------------------------------------------------------------------
//  destructor for class GpDucManager
//---------------------------------------------------------------------------

GpDucManager::~GpDucManager()
{
	Close();

    DrcSettings.Save();

    delete MGR;
}

//---------------------------------------------------------------------------
//  GpDucManager::ConfigureSystem() -
//---------------------------------------------------------------------------

void GpDucManager::ConfigureSystem()
{
    GpManagerBase::ConfigureSystem();
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_Components() - These objects require opened board
//---------------------------------------------------------------------------

void GpDucManager::Make_Components()
{
    GpManagerBase::Make_Components();
}

//---------------------------------------------------------------------------
//  GpDucManager::Reset() -
//---------------------------------------------------------------------------

void GpDucManager::Reset(bool state)
{
    GpManagerBase::Reset(state);
}

//---------------------------------------------------------------------------
//  GpDucManager::Open() - open all hardware
//---------------------------------------------------------------------------

void GpDucManager::Open()
{
	GpManagerBase::Open();
}

//---------------------------------------------------------------------------
//  GpDucManager::IsOpen()
//---------------------------------------------------------------------------

bool GpDucManager::IsOpen(unsigned int which)
{
    if (which < DrcUnits.Size() && Settings.DRC_Targets_Enabled[which])
        return  DrcUnits.Ref_At(which).IsOpen();
    else
        return false;
}

//---------------------------------------------------------------------------
//  GpDucManager::DdcClockFrequencyActual()
//---------------------------------------------------------------------------

double GpDucManager::DdcClockFrequencyActual(unsigned int ordinal)
{
    return Boards(ordinal).AdcClockFrequencyActual();
}

//---------------------------------------------------------------------------
//  GpDucManager::DucClockFrequencyActual()
//---------------------------------------------------------------------------

double GpDucManager::DucClockFrequencyActual(unsigned int ordinal)
{
    return Boards(ordinal).DacClockFrequencyActual();
}


//---------------------------------------------------------------------------
//  GpDucManager::DdcSampleFrequency()
//---------------------------------------------------------------------------

double GpDucManager::DdcSampleFrequency(unsigned int /*ordinal*/)
{
    return DrcSettings.Rx.SampleRate;
}

//---------------------------------------------------------------------------
//  GpDucManager::BeforeStreamStart()
//---------------------------------------------------------------------------

void  GpDucManager::BeforeStreamStart(unsigned int ordinal)
{
    GpManagerBase::BeforeStreamStart(ordinal);
}

//---------------------------------------------------------------------------
//  GpDucManager::LogicVersion()
//---------------------------------------------------------------------------

string GpDucManager::LogicVersion(unsigned int which)
{
    if (which < DrcUnits.Size() && Settings.DRC_Targets_Enabled[which] && IsOpen(which))
        return DrcUnits.Ref_At(which).LogicVersion();
    else
        return "N/A";
}

//---------------------------------------------------------------------------
//  GpDucManager::CoreVersion()
//---------------------------------------------------------------------------

pair<string,string> GpDucManager::CoreVersion(unsigned int which)
{
    pair<string, string> result;
    result.first = result.second = "N/A";

   if (which < DrcUnits.Size() && Settings.DRC_Targets_Enabled[which]&& IsOpen(which))
      return DrcUnits.Ref_At(which).CoreVersion();
   else
      return result;
}

//---------------------------------------------------------------------------
//  GpDucManager::Close() - close all hardware
//---------------------------------------------------------------------------

void GpDucManager::Close()
{
    GpManagerBase::Close();
}

//---------------------------------------------------------------------------
//  GpDucManager::StartStreaming() - start streaming on boards
//---------------------------------------------------------------------------

void GpDucManager::StartStreaming()
{
    GpManagerBase::StartStreaming();
}

//---------------------------------------------------------------------------
//  GpDucManager::StopStreaming()  - stop streaming on boards
//---------------------------------------------------------------------------

void GpDucManager::StopStreaming()
{
    GpManagerBase::StopStreaming();
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_Drc()
//---------------------------------------------------------------------------

void GpDucManager::Make_Drc()
{
    PMix_Make_Drc(DrcSettings, MGR, Settings);
    // Let modules update defaults
    for (size_t i = 0; i < DrcUnits.Size(); i++)
            DrcUnits.Ref_At(i).PassDefaults();
    // Load Settings
    DrcSettings.LoadINISettings();
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_Clocks()
//------------------------------------------------------------------------

void GpDucManager::Make_Clocks()
{
    Innovative::Trace Debug;

    Debug.Log("Clocks ", Settings.Clock_Module_Enabled ? "Enabled" : "Disabled");
    Debug.Log("Selected Module ", Settings.Clock_Module_Type);

    if (Settings.Clock_Module_Enabled) {
        if (Settings.Clock_Module_Type == "x3-timing") {
            ClockModules.Append(new X3ClockWrapper("x3-timing", 0));
            Debug.Log("Created X3ClockWrapper");
        } else if (Settings.Clock_Module_Type == "sbc-duo") {
            ClockModules.Append(new SbcDuoClockWrapper("sbc-duo", 0));
            Debug.Log("Created SbcDuoClockWrapper");
        } else if (Settings.Clock_Module_Type == "atropos") {
            ClockModules.Append(new AtroposClockWrapper("atropos", 0));
            Debug.Log("Created AtroposClockWrapper");
        } else {
            ClockModules.Append(new X3ClockWrapper("x3-timing", 0));
            Debug.Log("Defaulted to X3ClockWrapper");
        }
    }
}

//---------------------------------------------------------------------------
//  GpDucManager::Open_Drc()
//---------------------------------------------------------------------------

void GpDucManager::Open_Drc()
{
    PMix_Open_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDucManager::Close_Drc()
//---------------------------------------------------------------------------

void GpDucManager::Close_Drc()
{
    PMix_Close_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDucManager::StartStreaming_Drc()
//---------------------------------------------------------------------------

void GpDucManager::StartStreaming_Drc()
{
    PMix_StartStreaming_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDucManager::StopStreaming_Drc()
//---------------------------------------------------------------------------

void GpDucManager::StopStreaming_Drc()
{
    PMix_StopStreaming_Drc(Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_Ficls()
//---------------------------------------------------------------------------

void GpDucManager::Make_Ficls()
{
    PMix_Make(FiclUnits, FICL);
}

//---------------------------------------------------------------------------
//  GpDucManager::MakeDdcChannels()
//---------------------------------------------------------------------------

void GpDucManager::Make_DdcChannels()
{
    PMix_Make(DdcChannels, Settings.DRC_Targets_Enabled);
    // Add more settings
    DdcChannels.AttachSettings(DrcSettings);
    // Load settings again
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDucManager::MakeDucChannels()
//---------------------------------------------------------------------------

void GpDucManager::Make_DucChannels()
{
    PMix_Make(DucChannels, Settings.DRC_Targets_Enabled);
    // Add more settings
    DucChannels.AttachSettings(DrcSettings);
    // Load settings again
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_FftChannels()
//---------------------------------------------------------------------------

void GpDucManager::Make_FftChannels()
{
    PMix_Make(FftChannels, Settings.DRC_Targets_Enabled);
    // Add more settings
    FftChannels.AttachSettings(DrcSettings);
    // Load settings again
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_PowerMeter()
//---------------------------------------------------------------------------

void GpDucManager::Make_PowerMeter()
{
    PMix_Make(PowerMeter, Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_OverflowDetector()
//---------------------------------------------------------------------------

void GpDucManager::Make_OverflowDetector()
{
    PMix_Make(OverflowDetector, Settings.DRC_Targets_Enabled);
}

//---------------------------------------------------------------------------
//  GpDucManager::Make_ToneGenerator()
//---------------------------------------------------------------------------

void GpDucManager::Make_ToneGenerator()
{
    PMix_Make(ToneGenerator, Settings.DRC_Targets_Enabled);
    // Add more settings
    ToneGenerator.AttachSettings(DrcSettings);
    // Load settings again
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDucManager::LoadDrcSettings() we want to call this once after making all
//                                  Components
//---------------------------------------------------------------------------

void GpDucManager::LoadDrcSettings()
{
    DrcSettings.Load();
}

//---------------------------------------------------------------------------
//  GpDucManager::Debug_SoftwareAlert() - pass software alert down to a board
//---------------------------------------------------------------------------

void GpDucManager::Debug_SoftwareAlert(unsigned int ordinal, unsigned int value)
{
    if (ordinal<DrcUnits.Size())
        DrcUnits[ordinal].SoftwareAlert(value);
}

//---------------------------------------------------------------------------
//  GpDucManager::Debug_ClockInfo() - pass clock info request down to a board
//---------------------------------------------------------------------------

void GpDucManager::Debug_ClockInfo(unsigned int ordinal)
{
    if (ordinal<DrcUnits.Size())
        DrcUnits[ordinal].ClockInfo();
}

//---------------------------------------------------------------------------
//  GpDucManager::PeekLogic() --
//---------------------------------------------------------------------------

unsigned int GpDucManager::PeekLogic(unsigned int ordinal, int offset)
{
    if (ordinal<DrcUnits.Size())
        return DrcUnits[ordinal].PeekLogic(offset);
    else
        return 0;
}

//---------------------------------------------------------------------------
//  GpDucManager::PokeLogic() --
//---------------------------------------------------------------------------

void GpDucManager::PokeLogic(unsigned int ordinal, int offset, unsigned int value)
{
    if (ordinal<DrcUnits.Size())
        DrcUnits[ordinal].PokeLogic(offset, value);
}

//---------------------------------------------------------------------------
//  GpDucManager::SamplingDurationToSamples() --
//---------------------------------------------------------------------------

ii64 GpDucManager::SamplingDurationToSamples(float duration)
{
    //
    // If in raw mode, use simple calculation
    if (DdcChannels.BypassDdc)
        {
        int sum = 0;
        for (size_t i = 0; i < DdcChannels.Size(); i++)
            {
            if (DdcChannels[i]->info.Enable)
                sum++;
            }
        return duration * DrcSettings.Rx.SampleRate * 1e6 * sum;
        }

    // Otherwise use DDC calculation.

    ii64 sum = 0;
    // If IQ multiply by 2
    float multiplier = Boards(0).IsIQEnabled() ? 2 : 1;
    // If 8-bit divide by 2
    multiplier /= (Boards(0).SampleWidth() == 8) ? 2 : 1;

    // Total Sample Rate in 1 sec
    for (size_t i = 0; i < DdcChannels.Size(); i++)
        {
        if (DdcChannels[i]->info.Enable)
            {
            DdcChannels.Wrapper(i).ReloadFile();
            sum += ((DrcSettings.Rx.SampleRate * 1e6 / DdcChannels.DecimationCoeffs(i)) );
            }
        }

    return sum * duration * multiplier;
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

