// GpDucManager_Gp.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef GpDucManager_GpH
#define GpDucManager_GpH

#include <GpManagerBase_Gp.h>

#include <DucProvider_Gp.h>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//==============================================================================
//  CLASS  GpDucManager  --  Manages interaction with GUI, Board(s)
//==============================================================================

class GpDucManager : public GpManagerBase, public ProviderMixin<Duc_Provider>
{
public:
    GpDucManager(IManager_GUIInterface & ui, IManager_FiclInterface & ficl);
    virtual ~GpDucManager();

    friend class GpDucManagerInterface;
    enum CommandTriggerType {eDDC, eCommon, eBoth};

    //  Standard Operations
    virtual void  ConfigureSystem();
    virtual void  Make_Components();
    virtual void  Open();
    virtual void  Close();
    virtual void  StartStreaming();
    virtual void  StopStreaming();

    virtual void  Reset(bool state);
    virtual void  BeforeStreamStart(unsigned int ordinal);

    //  ...other operations
    virtual void Debug_SoftwareAlert(unsigned int ordinal, unsigned int value);
    virtual void Debug_ClockInfo(unsigned int ordinal);

    ii64         SamplingDurationToSamples(float duration);

    //  Periodic Timers

    //  Public Data (hiss!)
    Duc_ProviderSettings       DrcSettings;

    virtual bool    IsOpen(unsigned int which);
    virtual int     BoardCount() const
                        {  return PMix_BoardCount();  }
    virtual string  LogicVersion(unsigned int which);
    virtual pair<string,string> CoreVersion(unsigned int which);
    virtual double  DdcClockFrequencyActual(unsigned int ordinal);
    virtual double  DucClockFrequencyActual(unsigned int ordinal);
    virtual double  DdcSampleFrequency(unsigned int ordinal);
    unsigned int    PeekLogic(unsigned int ordinal, int offset);
    void            PokeLogic(unsigned int ordinal, int offset, unsigned int value);

protected:

    class GpDucManagerInterface *  MGR;

    //
    //  Unique Interface for Specific Managers....
    virtual void  Make_Drc();  // Make Provider Lists
    virtual void  Open_Drc();
    virtual void  Close_Drc();
    virtual void  StartStreaming_Drc();
    virtual void  StopStreaming_Drc();

    virtual  void  Make_Ficls();
    virtual  void  Make_DdcChannels();
	virtual  void  Make_DucChannels();
    virtual  void  Make_FftChannels();
    virtual  void  Make_PowerMeter();
    virtual  void  Make_OverflowDetector();
    virtual  void  Make_ToneGenerator();
    virtual  void  Make_Clocks();

    virtual  void  LoadDrcSettings();
};

//===========================================================================
//  CLASS GpDucManagerInterface  -- Callback from DRC provider to manager
//===========================================================================

class GpDucManagerInterface : public IDucManagerInterface
{
public:
    GpDucManagerInterface(GpDucManager * mgr)
                : MGR(mgr)
                    {  }
    ~GpDucManagerInterface()  {  }

    virtual void  Log(unsigned int ordinal, const std::string & a_string)
        {  MGR->Drc_Log(ordinal, a_string);  }
    virtual void  AfterStreamAutoStop(unsigned int ordinal)
        {  MGR->Drc_AfterStreamAutoStop(ordinal);  }
    virtual void  PeriodicStatusUpdate(unsigned int ordinal)
        {  MGR->Drc_PeriodicStatusUpdate(ordinal);  }
    virtual void  BeforeStreamStart(unsigned int ordinal)
        {  MGR->Drc_BeforeStreamStart(ordinal);  }
    virtual void  AfterStreamStart(unsigned int ordinal)
        {  MGR->Drc_AfterStreamStart(ordinal);  }
    virtual unsigned int EnabledChannels(unsigned int ordinal)
        { return MGR->EnabledChannels(ordinal); }
    virtual unsigned int EnabledDevices(unsigned int ordinal)
        { return MGR->EnabledDevices(ordinal); }
    virtual float DecimationFactor(unsigned int ordinal)
        { return MGR->DecimationFactor(ordinal); }
    virtual unsigned int EnabledChannelizerChannels(unsigned int ordinal)
        { return MGR->EnabledChannelizerChannels(ordinal); }
    virtual void Plot(unsigned int ordinal, VeloBuffer &buffer, int type)
        { MGR->Plot(ordinal, buffer, type); }
    virtual void DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer)
        { MGR->Drc_DataPacketAvailable(ordinal, buffer); }
private:
    GpDucManager *  MGR;
};


#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // end namespace Innovative


#endif // GpDucManager_GpH
