// GpManagerBase_Gp.cpp
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2014 Innovative Integration Corporation.
//  All Rights Reserved.
//

#ifndef GpManagerBase_GpH
#define GpManagerBase_GpH

#include <ChannelsProxy_Gp.h>
#include <FftProxy_Gp.h>
#include <SupportProxy_Gp.h>
#include <ClockProxy_Gp.h>

#include "Application/IniSaver_App.h"
#include "Application/FiclIo_App.h"

#include <vector>

using GP::DdcChannelsProxy;
using GP::DucChannelsProxy;
using GP::DucChannelInfo;

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS IManager_GUIInterface  --
//===========================================================================

class IManager_GUIInterface // : public IFicl_UserInterface (needed?)
{
public:
    virtual ~IManager_GUIInterface()  {  }

    virtual void  Log(const std::string & a_string) = 0;
    virtual void  PeriodicStatus(unsigned int ordinal) = 0;
    virtual void  BeforeStreamStart(unsigned int ordinal) = 0;
    virtual void  AfterStreamStart(unsigned int ordinal) = 0;
    virtual void  AfterStreamAutoStop(unsigned int ordinal) = 0;
    virtual void  Plot(unsigned int ordinal, VeloBuffer &buffer, int type) = 0;
    virtual void  DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer) = 0;
    virtual void  BeforeDrcOpen() = 0;  
	virtual void  AfterDrcOpen() = 0;
    virtual void  BeforeDrcClose() = 0;
	virtual void  AfterDrcClose() = 0;
    virtual void  AfterDrcStreamStart() = 0;
    virtual void  BeforeDrcStreamStart() = 0;
    virtual void  AfterDrcStreamStop() = 0;
    virtual void  BeforeDrcStreamStop() = 0;
				 
};

//===========================================================================
//  CLASS IManager_FiclInterface  --
//===========================================================================

class IManager_FiclInterface : public IFicl_UserInterface
{
public:
    virtual ~IManager_FiclInterface()  {  }

};

//==============================================================================
//  CLASS  RoleGroup  -- collection of hardware in a particular role
//==============================================================================
//  Objects are owned by the RoleGroup

template <class T>
class RoleGroup
{
public:
    RoleGroup()
        {}
    ~RoleGroup()
        {
        Clear();
        }

    void Clear()
        {
        while( Group.size() > 0)
            {
            delete Group.back();
            Group.pop_back();
            }
        }

    void Append(T* value)
        {   Group.push_back(value);   }

    size_t  Size() const
        {   return Group.size();  }

    T&  Ref_At(size_t idx)
        {   return *Group[idx];   }
    T&  operator [](size_t idx)
        {   return Ref_At(idx);   }

private:
    //
    //
    std::vector<T *> Group;
};


//==============================================================================
//  CLASS  GpDrcManagerSettings  --
//==============================================================================

class GpDrcManagerSettings :  public Innovative::IniSaver
{
public:
    GpDrcManagerSettings();
    ~GpDrcManagerSettings();

    //  Board Configuration
    unsigned int          DRC_Count;
    vector<int>           DRC_Targets;
    vector<int>           DRC_Targets_Enabled;


    unsigned int          Clock_Module_Enabled;
    string                Clock_Module_Type;
    //  Odds and Ends
    int                   DebugVerbosity;

};

//==============================================================================
//  CLASS  GpManagerBase  --  Base for Manager of GUI, Board(s)
//==============================================================================

class GpManagerBase
{
public:
    GpManagerBase(IManager_GUIInterface & ui, IManager_FiclInterface & ficl);
    virtual ~GpManagerBase();

    //friend class GpDucManagerInterface;
	enum CommandTriggerType {eDDC, eDUC, eCommon, eBoth};

    //  Standard Operations
    virtual void  ConfigureSystem();
    virtual void  Make_Components();
    virtual void  Open();
    virtual void  Close();
    virtual void  StartStreaming();
    virtual void  StopStreaming();

    virtual void  Reset(bool state);
    virtual void  BeforeStreamStart(unsigned int ordinal);

    //  non-virtuals (not changed in derived classes
    void          CommandTrigger(CommandTriggerType which = eBoth);
    void          CoreReset();

    string        BoardTag(unsigned int b);
    void          BoardList(vector<string> &List, bool only_enabled);
    void          AdjustTargets();
    vector<std::string> FftSources(unsigned int ordinal);
    void          SetSourceMaps(unsigned int adc_count);
    void          UpdateAdcInfo(unsigned int ch);
    void          UpdateDdcInfo(unsigned int ch);

    //  Debug Support
    void         SetTraceVerbosity(unsigned int level);
    //  ...other operations
    virtual void Debug_SoftwareAlert(unsigned int ordinal, unsigned int value) = 0;
    virtual void Debug_ClockInfo(unsigned int ordinal) = 0;
    //  ...Ficl
    void        GetFiclTargets(std::vector<std::string>& ficl_targets);
    void        Ficl_OnCommandKeyPress(unsigned int ficlT, char &Key);
    void        Ficl_ExecuteDebugScript(unsigned int ficlT, const std::string & script_file);
    void        Ficl_Logic(unsigned int ficlT, const std::string & address, const std::string & data);
    void        Ficl_Logic(unsigned int ficlT, const std::string & address);
    void        Ficl_Port(unsigned int ficlT, const std::string & address, const std::string & data);
    void        Ficl_Port(unsigned int ficlT, const std::string & address);
    void        Ficl_Device(unsigned int ficlT, const std::string & address, const std::string & device,
                     const std::string & data);
    void        Ficl_Device(unsigned int ficlT, const std::string & address, const std::string & device);

    void        DdcChannelList(std::vector<std::string>& channel_list);
    void        DucChannelList(std::vector<std::string>& channel_list);

    //  Public Data (hiss!)
    GpDrcManagerSettings       Settings;
    DdcChannelsProxy           DdcChannels;
    DucChannelsProxy           DucChannels;
    FftChannelsProxy           FftChannels;
    PowerMeterProxy            PowerMeter;
    OverflowProxy              OverflowDetector;
    ToneGeneratorProxy         ToneGenerator;
    ClockProxy                     ClockModules;

    //
    //  Board Interface (defined in derived classes
    virtual bool    IsOpen(unsigned int ordinal) = 0;
    virtual int     BoardCount() const = 0;
    virtual string  LogicVersion(unsigned int ordinal) = 0;
    virtual pair<string, string>  CoreVersion(unsigned int ordinal) = 0;
    virtual double  DdcClockFrequencyActual(unsigned int ordinal) = 0;
    virtual double  DucClockFrequencyActual(unsigned int ordinal) = 0;
    virtual double  DdcSampleFrequency(unsigned int ordinal) = 0;

protected:
    IManager_GUIInterface &        UI;
    IManager_FiclInterface &       FICL;

    //
    //  FICL systems for all boards
    FiclGroup                FiclUnits;

    //
    //  Unique Interface for Specific Managers....
    virtual void  Make_Drc() = 0;  // Make Provider Lists
    virtual void  Open_Drc() = 0;
    virtual void  Close_Drc() = 0;
    virtual void  StartStreaming_Drc() = 0;
    virtual void  StopStreaming_Drc() = 0;

    // ...other "Makers"
    virtual void  Make_Ficls() = 0;
    virtual void  Make_DdcChannels() = 0;
    virtual void  Make_DucChannels() = 0;
    virtual void  Make_FftChannels() = 0;
    virtual void  Make_PowerMeter() = 0;
    virtual void  Make_OverflowDetector() = 0;
    virtual void  Make_ToneGenerator() = 0;
    virtual void  Make_Clocks() = 0;
    virtual void  LoadDrcSettings() = 0;

    //  Base class methods
    std::string  ChannelMap(unsigned int ordinal);
    void         Plot(unsigned int ordinal, VeloBuffer &buffer, int type);
    void         Drc_DataPacketAvailable(unsigned int ordinal, VeloBuffer &buffer);
    void         Drc_Log(unsigned int ordinal, const std::string & a_string);
    void         Drc_AfterStreamAutoStop(unsigned int ordinal);
    void         Drc_BeforeStreamStart(unsigned int ordinal);
    void         Drc_AfterStreamStart(unsigned int ordinal);
    void         Drc_PeriodicStatusUpdate(unsigned int ordinal);
    float        DecimationFactor(unsigned int ordinal);
    unsigned int EnabledChannelizerChannels(unsigned int ordinal);
    unsigned int EnabledChannels(unsigned int ordinal);
    unsigned int EnabledDevices(unsigned int ordinal);

};

//==============================================================================
//  TEMPLATE CLASS  ProviderMixin  --  Manages "Generic" Provider Interface
//==============================================================================

template <class ProviderT>
class ProviderMixin
{
public:
    ProviderT  &  Boards(unsigned int b)
        { return DrcUnits[b]; }

protected:
    //
    //  DRC boards and operations
    RoleGroup<ProviderT>  DrcUnits;   //  Collection of Board Providers

    size_t   PMix_BoardCount() const
        {  return DrcUnits.Size();  }

    //---------------------------------------------------------------------------
    template <class SettingsT, class MgrT>
    void PMix_Make_Drc(SettingsT& DrcSettings, MgrT* MGR, GpDrcManagerSettings& Settings)
    {
        //  erase old DRC boards (?)
        DrcUnits.Clear();

        {
        ProviderT * unit = new ProviderT(DrcSettings, MGR);

        //  make DRC boards
        Settings.DRC_Count = unit->BoardCount();

        if (Settings.DRC_Count > Settings.DRC_Targets.size())
            Settings.DRC_Targets.resize(Settings.DRC_Count);
        if (Settings.DRC_Count > Settings.DRC_Targets_Enabled.size())
            Settings.DRC_Targets_Enabled.resize(Settings.DRC_Count);

        delete unit;
        }

        size_t count = static_cast<size_t>(Settings.DRC_Count);
        for (size_t i = 0; i < count; i++)
            {
            ProviderT * unit = new ProviderT(DrcSettings, MGR);
            unit->Personality.Ordinal = i;

            if (i<Settings.DRC_Targets.size())
                {
                unit->Personality.Target  = Settings.DRC_Targets[i];
                unit->Personality.Enabled = Settings.DRC_Targets_Enabled[i] ? true : false;
                }
            else
                {
                unit->Personality.Target  = 0;
                unit->Personality.Enabled = true;
                }
            DrcUnits.Append(unit);
            }
    }
    //---------------------------------------------------------------------------
    void PMix_Open_Drc(const vector<int>& enabled)
    {
        //  open DRC boards
        for (size_t i = 0; i < DrcUnits.Size(); i++)
            {
            if (enabled[i])
                DrcUnits.Ref_At(i).Open();
            }
    }
    //---------------------------------------------------------------------------
    void PMix_Close_Drc(const vector<int>& enabled)
    {
        //  close DRC boards
        for (size_t i = 0; i < DrcUnits.Size(); i++)
            {
            if (enabled[i])
                DrcUnits.Ref_At(i).Close();
            }
    }
    //---------------------------------------------------------------------------
    void PMix_StartStreaming_Drc(const vector<int>& enabled)
    {
        //  start streaming on DRC boards
        for (size_t i = 0; i < DrcUnits.Size(); i++)
            {
            if (enabled[i])
                DrcUnits.Ref_At(i).StartStreaming();
            }
    }
    //---------------------------------------------------------------------------
    void PMix_StopStreaming_Drc(const vector<int>& enabled)
    {
        //  stop streaming on DRC boards
        for (size_t i = 0; i < DrcUnits.Size(); i++)
            {
            if (enabled[i])
                DrcUnits.Ref_At(i).StopStreaming();
            }
    }
    //---------------------------------------------------------------------------
    void  PMix_Make(FiclGroup & FiclUnits, IManager_FiclInterface & FICL)
    {
        //  erase old FICL boards
        FiclUnits.Clear();

        //  ask DRC boards to add FICL boards
        for (size_t i = 0; i < DrcUnits.Size(); i++)
            {
                DrcUnits.Ref_At(i).Make(FiclUnits, &FICL);
            }
    }
    //---------------------------------------------------------------------------
	template<class T>
    void  PMix_Make(T & Object, const vector<int>& enabled)
    {
        Object.Clear();
        for (size_t i = 0; i < DrcUnits.Size(); i++)
            {
            if (enabled[i])
                DrcUnits.Ref_At(i).Make(Object);
            }
    }
};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

} // end namespace Innovative


#endif // GpDucManager_GpH
