// GpProviderBase_Gp.cpp
//
// Data flow and hardware I/O for Duc units
//
//    INNOVATIVE INTEGRATION CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or nondisclosure
//  agreement with Innovative Integration Corporation and may not be copied or
//  disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000..2013 Innovative Integration Corporation.
//  All Rights Reserved.
//
//---------------------------------------------------------------------------

#include "GpProviderBase_Gp.h"

using namespace std;
using namespace GP;


namespace   Innovative
{

//===========================================================================
//  CLASS IDrc_Module  -- Interface to a Drc Module Implementation
//===========================================================================
//---------------------------------------------------------------------------
//  IDrc_Module::Tag() --
//---------------------------------------------------------------------------

string IDrc_Module::Tag()
{
    std::stringstream msg;
    msg << "Module[" << Personality().Ordinal << "]";

    return msg.str();
}
//---------------------------------------------------------------------------
//  IDrc_Module::Log() --  Log message thunked to owner
//---------------------------------------------------------------------------

void  IDrc_Module::Log(const std::string & msg)
{
    ProcessStatusEvent e(msg);
    OnLog.Execute(e);
}

//===========================================================================
//  CLASS IDrc_MakerModule  -- IDuc_Module with implementation for Make_ ers
//===========================================================================
//---------------------------------------------------------------------------
//  IDrc_MakerModule::Make_DdcChannels_Impl() --
//---------------------------------------------------------------------------

void  IDrc_MakerModule::Make_DdcChannels_Impl(GpProject_DDC &GpRegs, DdcChannelsProxy & group, const vector<DdcChannelInfo> &defs)
{
    if (!GpRegs.ProjectInfo.Enabled(GspDspProjectInfo::DDC))
    {
        //double check
        if(GpRegs.DdcM.GetEnableBits() == 0xda55) //No DDC
        {
            Log("No DDC Cores.");
            return;
        }
    }

    int group_size = GpRegs.DdcM.ChannelCountPerCore() == 0 ? 1 : GpRegs.DdcM.ChannelCountPerCore();
    int group_id   = -1;

	const size_t max_cores = 200;

	size_t total_cores = GpRegs.DdcM.CoreCount() * group_size;

	if (total_cores > max_cores)
		{
		total_cores = max_cores;
		Log("Core count is more than MAX. Defaulted to MAX");
		}

    std::stringstream msg;
    msg << "Number of DDC Cores : " << total_cores;
    Log(msg.str());

	for (size_t i = 0; i < total_cores; i++)
        {
        DdcChannelInfo ChanInfo = (i < defs.size()) ? defs[i] : defs[defs.size() - 1];

        ChannelWrapper *ch = new ChannelWrapper(&GpRegs.DdcM, Tag(), i, ChanInfo, &group, Personality().Ordinal);
        group.Append(ch);

        if (GpRegs.DdcM.IsBitEnabled(GspDspModuleBase::eTuner))
            ch->AddTunerWrapper(new DdcTunerWrapper(&GpRegs.DdcTunerSM, Tag(), 0));
        if (GpRegs.DdcM.IsBitEnabled(GspDspModuleBase::eVitaFramer))
            ch->AddVitaFramerWrapper(new DdcVitaFramerWrapper(&GpRegs.DdcVitaFramerSM, Tag(), 0));
        if (GpRegs.DdcM.IsBitEnabled(GspDspModuleBase::ePeripheral))
            ch->AddPeripheralWrapper(new DdcPeripheralWrapper(&GpRegs.DdcPeriphSM, Tag(), 0));
        if (GpRegs.DdcM.IsBitEnabled(GspDspModuleBase::eLowPassFilter))
            ch->AddLowPassFilterWrapper(new DdcLowPassFilterWrapper(&GpRegs.DdcLowPassFilterSM, Tag(), 0));
        if (GpRegs.DdcM.IsBitEnabled(GspDspModuleBase::ePolyFilter))
            ch->AddPolyPassFilterWrapper(new DdcPolyphaseFilterWrapper(&GpRegs.DdcPolyPhaseFilterSM, Tag(), 0));
        // Assign group id
        if ((i % group_size) == 0) {
            group_id++;
            ch->AddChannelizerWrapper(new ChannelizerWrapper(&GpRegs.DdcChannelizerSM, Tag(), 0));
        }

        ch->Group(group_id);
        }
}

//---------------------------------------------------------------------------
//  IDrc_MakerModule::Make_DucChannels_Impl() --
//---------------------------------------------------------------------------

void  IDrc_MakerModule::Make_DucChannels_Impl(GpProject_DDC &GpRegs, DucChannelsProxy & group, const vector<DucChannelInfo> &defs)
{
    if (!GpRegs.ProjectInfo.Enabled(GspDspProjectInfo::DUC))
    {
        //double check
        if(GpRegs.DucM.GetEnableBits() == 0xda55) //No DUC
        {
            Log("No DUC Cores.");
            return;
        }
    }
    std::stringstream msg;
    msg << "Number of DUC Cores : " << GpRegs.DucM.CoreCount();
    Log(msg.str());

    for (size_t i = 0; i < GpRegs.DucM.CoreCount(); i++)
        {
		DucChannelInfo ChanInfo = (i < defs.size()) ? defs[i] : defs[defs.size() - 1];


        DucChannelWrapper *ch = new DucChannelWrapper(&GpRegs.DucM, Tag(), i, ChanInfo, &group, Personality().Ordinal);
		
		if (GpRegs.DucM.IsBitEnabled(GspDspModuleBase::eTuner))
			ch->AddTunerWrapper(new DucTunerWrapper(&GpRegs.DucTunerSM, Tag(), 0));
		if (GpRegs.DucM.IsBitEnabled(GspDspModuleBase::eVitaFramer))
			ch->AddVitaFramerWrapper(new DucVitaFramerWrapper(&GpRegs.DucVitaFramerSM, Tag(), 0));
		if (GpRegs.DucM.IsBitEnabled(GspDspModuleBase::ePeripheral))
			ch->AddPeripheralWrapper(new DucPeripheralWrapper(&GpRegs.DucPeriphSM, Tag(), 0));
		if (GpRegs.DucM.IsBitEnabled(GspDspModuleBase::eLowPassFilter))
			ch->AddLowPassFilterWrapper(new DucLowPassFilterWrapper(&GpRegs.DucLowPassFilterSM, Tag(), 0));
		if (GpRegs.DucM.IsBitEnabled(GspDspModuleBase::ePolyFilter))
			ch->AddPolyPassFilterWrapper(new DucPolyphaseFilterWrapper(&GpRegs.DucPolyPhaseFilterSM, Tag(), 0));

		group.Append(ch);
        }
}

//---------------------------------------------------------------------------
//  IDrc_MakerModule::Make_PowerMeter_Impl() --
//---------------------------------------------------------------------------

void IDrc_MakerModule::Make_PowerMeter_Impl(GpProject_DDC &GpRegs, PowerMeterProxy &group)
{
    std::stringstream msg;
    msg << "Number of Power Meter Cores : " << GpRegs.PowerMeterSM.CoreCount();
    Log(msg.str());

    for (size_t i = 0; i < GpRegs.PowerMeterSM.CoreCount(); i++)
        {
        PowerMeterWrapper *pw = new PowerMeterWrapper(&GpRegs.PowerMeterSM, Tag(), i);
        pw->WhichBoard(Personality().Ordinal);
        group.Append(pw);
        }
}

//---------------------------------------------------------------------------
//  IDrc_MakerModule::Make_OverflowDetector_Impl() --
//---------------------------------------------------------------------------

void IDrc_MakerModule::Make_OverflowDetector_Impl(GpProject_DDC &GpRegs, OverflowProxy &group)
{
    std::stringstream msg;
    msg << "Number of Overflow Cores : " << GpRegs.OverflowSM.CoreCount();
    Log(msg.str());
    
    msg.str("");
    msg << "DDC[" << Personality().Ordinal << "]";

    for (size_t i = 0; i < GpRegs.OverflowSM.CoreCount(); i++)
        {
        OverflowWrapper *of = new OverflowWrapper(&GpRegs.OverflowSM, msg.str(), i);
        of->WhichBoard(Personality().Ordinal);
        group.Append(of);
        }
}

//---------------------------------------------------------------------------
//  IDrc_MakerModule::Make_ToneGenerator_Impl() --
//---------------------------------------------------------------------------
void IDrc_MakerModule::Make_ToneGenerator_Impl(GpProject_DDC &GpRegs, ToneGeneratorProxy &group, const vector<ToneGeneratorDataType> &tonedef)
{
	std::stringstream msg;
    msg << "Number of Tone Generator Cores : " << GpRegs.ToneGeneratorSM.CoreCount();
    Log(msg.str());
	
    for (size_t i = 0; i < GpRegs.ToneGeneratorSM.CoreCount(); i++)
        {
        if (i == 0)
            group.Append(new ToneGeneratorWrapper(&GpRegs.ToneGeneratorSM, Tag(),    0, tonedef[0])); // DDC Tone Gen.
        else
            group.Append(new DucToneGeneratorWrapper(&GpRegs.ToneGeneratorSM, Tag(), 1, tonedef[1])); // DUC Tone Gen.
        }
}

//==============================================================================
//  CLASS  GpProviderBase  --  Base for all Providers
//==============================================================================
//------------------------------------------------------------------------------
//  constructor for class GpProviderBase
//------------------------------------------------------------------------------

GpProviderBase::GpProviderBase()
{
}

//------------------------------------------------------------------------------
//  destructor for class GpProviderBase
//------------------------------------------------------------------------------

GpProviderBase::~GpProviderBase()
{
}

}  // namespace Innovative

