#include <SpectralModule_Gp.h>
#include <SystemSupport_Mb.h>
#include <cmath>

namespace   Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

using namespace GpDsp_MemoryMap;

//===========================================================================
//  CLASS DspCommon_Module  -- DSPCommon Module
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class DspCommon_Module
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//  constructor for class DspCommon_Module
//---------------------------------------------------------------------------

SpectralModule::SpectralModule(GpDspMemoryExtension & GP)
    :  GspDspModuleBase(GP.WB_SpectrumAnalysis, ewbSpectrumAnalysis)
{
}

//===========================================================================
//  CLASS SpectralPeripheral  -- Spectral Peripheral 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SpectralPeripheral
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters SP_Sub_RegAddrs =
    {  mmSaPeriphInfo,      //  Version_Addr
       mmSaPeriphConfig,     //  Config_Addr
       mmSaPeriphReset
    };

static SpectralPeripheral::MapRegisters SP_Core_RegAddrs =
    {
    mmSaPeriphBypass,       //  Bypass_Addr
    mmSaPeriphSource,       //  SourceSelection_Addr
    mmSaPeriphBypassWin,    //  BypassWindowing_Addr
    mmSaPeriphOutputMode    //  OutputMode_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class SpectralPeripheral
//---------------------------------------------------------------------------

SpectralPeripheral::SpectralPeripheral(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_SpectrumAnalysis, SP_Sub_RegAddrs, ewbSpectrumAnalysis),
       BypassReg(GP.WB_SpectrumAnalysis, SP_Core_RegAddrs.Bypass_Addr),
       SourceSelectionReg(GP.WB_SpectrumAnalysis, SP_Core_RegAddrs.SourceSelection_Addr),
       BypassWindowingReg(GP.WB_SpectrumAnalysis, SP_Core_RegAddrs.BypassWindowing_Addr),
       OutputModeReg(GP.WB_SpectrumAnalysis, SP_Core_RegAddrs.OutputMode_Addr)
{
}

//---------------------------------------------------------------------------
//  SpectralPeripheral::BypassSpectAnalysis() -
//---------------------------------------------------------------------------

void SpectralPeripheral::SetSourceMap(const vector<int> &values)
{
    ddc_map = values;
}

//---------------------------------------------------------------------------
//  SpectralPeripheral::BypassSpectAnalysis() -
//---------------------------------------------------------------------------

void SpectralPeripheral::BypassSpectAnalysis(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    BypassReg.Bypass.Value(value ? 1 : 0);
}

//---------------------------------------------------------------------------
//  SpectralPeripheral::SourceSelect() -
//---------------------------------------------------------------------------

void SpectralPeripheral::SourceSelect(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    if (value < static_cast<int>(ddc_map.size()))
        SourceSelectionReg.Select.Value(ddc_map[value]);
}

//---------------------------------------------------------------------------
//  SpectralPeripheral::BypassWindowing() -
//---------------------------------------------------------------------------

void SpectralPeripheral::BypassWindowing(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    BypassWindowingReg.Bypass.Value(value ? 1 : 0);
}

//---------------------------------------------------------------------------
//  SpectralPeripheral::BypassWindowing() -
//---------------------------------------------------------------------------

void SpectralPeripheral::ModeSelect(unsigned int parent, OutputModeType value)
{
    SetIndex(parent, 0);
    OutputModeReg.Mode.Value(value == oThresholLimited);
}


//===========================================================================
//  CLASS SpectralMultiBuffer  -- Spectral MultiBuffer 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SpectralMultiBuffer
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters MB_Sub_RegAddrs =
    {  mmSaMultiBufInfo,      //  Version_Addr
       mmSaMultiBufConfig,    //  Config_Addr
       mmSaMultiBufReset
    };

static SpectralMultiBuffer::MapRegisters MB_Core_RegAddrs =
    {
    mmSaMultiBufWinSize,       //  WinSize_Addr
    mmSaMultiBufDelay          //  Delay_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class SpectralPeripheral
//---------------------------------------------------------------------------

SpectralMultiBuffer::SpectralMultiBuffer(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_SpectrumAnalysis, MB_Sub_RegAddrs, ewbSpectrumAnalysis),
       WindowSizeReg(GP.WB_SpectrumAnalysis, MB_Core_RegAddrs.WinSize_Addr),
       BufferDelayReg(GP.WB_SpectrumAnalysis, MB_Core_RegAddrs.Delay_Addr)
{
}

//---------------------------------------------------------------------------
//  SpectralMultiBuffer::WindowSize() -
//---------------------------------------------------------------------------

void SpectralMultiBuffer::WindowSize(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    WindowSizeReg.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralMultiBuffer::WindowSize() -
//---------------------------------------------------------------------------

void SpectralMultiBuffer::Delay(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    BufferDelayReg.Value(value);

    Sleep(2 / fasd * 1000);
    CommandTrigger(); //
}

//===========================================================================
//  CLASS SpectralWindowing  -- Spectral Windowing 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Spectral Windowing
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters Win_Sub_RegAddrs =
    {  mmSaWindowingInfo,      //  Version_Addr
       mmSaWindowingConfig,    //  Config_Addr
       mmSaWindowingReset
    };

static SpectralWindowing::MapRegisters Win_Core_RegAddrs =
    {
    mmSaWindowingEn,              //  CoefEn_Addr
    mmSaWindowingCoeffAddr,       //  WinCoef_Addr
    mmSaWindowingCoeffData        //  CoefData_Addr
    };

//---------------------------------------------------------------------------
//  constructor for class SpectralWindowing
//---------------------------------------------------------------------------

SpectralWindowing::SpectralWindowing(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_SpectrumAnalysis, Win_Sub_RegAddrs, ewbSpectrumAnalysis),
       WindowingCoeffEnReg(GP.WB_SpectrumAnalysis, Win_Core_RegAddrs.CoefEn_Addr),
       WindowingCoeffAddrReg(GP.WB_SpectrumAnalysis, Win_Core_RegAddrs.WinCoef_Addr),
       WindowingCoeffDataReg(GP.WB_SpectrumAnalysis, Win_Core_RegAddrs.CoefData_Addr)
{
}

//---------------------------------------------------------------------------
//  SpectralWindowing::CoeffAddr() -
//---------------------------------------------------------------------------

void SpectralWindowing::CoeffEnable(unsigned int parent, WindowingModeType value)
{
    SetIndex(parent, 0);

    WindowingCoeffEnReg.Mode.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralWindowing::CoeffAddr() -
//---------------------------------------------------------------------------

void SpectralWindowing::CoeffAddr(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    WindowingCoeffAddrReg.Addr.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralWindowing::CoeffData() -
//---------------------------------------------------------------------------

void SpectralWindowing::CoeffData(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    WindowingCoeffDataReg.Data.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralWindowing::Coeffs() -
//---------------------------------------------------------------------------

void SpectralWindowing::Coeffs(unsigned int parent, const vector<int>& cs)
{
    SetIndex(parent, 0);

    WindowingCoeffEnReg.Mode.Value(SpectralWindowing::wCoeffs);

    for (size_t i = 0; i < cs.size(); i++)
        {
        WindowingCoeffAddrReg.Addr.Value(i);
        WindowingCoeffDataReg.Data.Value(cs[i]);
        }

    WindowingCoeffEnReg.Mode.Value(SpectralWindowing::wController);
}

//===========================================================================
//  CLASS SpectralFft  -- Spectral FFT 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Spectral FFT
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters Fft_Sub_RegAddrs =
    {  mmSaFftInfo,            //  Version_Addr
       mmSaFftConfig,          //  Config_Addr
       mmSaFftReset
    };

//---------------------------------------------------------------------------
//  constructor for class SpectralFft
//---------------------------------------------------------------------------

SpectralFft::SpectralFft(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_SpectrumAnalysis, Fft_Sub_RegAddrs, ewbSpectrumAnalysis)
{
}


//===========================================================================
//  CLASS SpectralPostProc  -- Spectral Post Proc 'submodule'
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class Spectral Windowing
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters Pp_Sub_RegAddrs =
    {  mmSaPostProcInfo,      //  Version_Addr
       mmSaPostProcConfig,    //  Config_Addr
       mmSaPostProcReset
    };

static SpectralPostProc::MapRegisters Pp_Core_RegAddrs =
    {
    mmSaPostProcTrigMode,       //  TrigMode_Addr
    mmSaPostProcSingleTrig,     //  SingleTrig_Addr
    mmSaPostProcProgEn,         //  ProgEn_Addr
    mmSaPostProcProgAddr,       //  ProgAddr_Addr
    mmSaPostProcProgData,       //  ProgData_Addr
    mmSaPostProcDataReadback,   //  DataReadback_Addr
    mmSaPostProcFasdCoeff,      //  FasdCoeff_Addr
	mmSaPostProcDecimation      //  SaPostProcDecimation
    };

//---------------------------------------------------------------------------
//  constructor for class SpectralPostProc
//---------------------------------------------------------------------------

SpectralPostProc::SpectralPostProc(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_SpectrumAnalysis, Pp_Sub_RegAddrs, ewbSpectrumAnalysis),
       TriggerModeReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.TrigMode_Addr),
       SingleTriggerReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.SingleTrig_Addr),
       TLSProgEnableReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.ProgEn_Addr),
       TLSAddrReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.ProgAddr_Addr),
       TLSDataReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.ProgData_Addr),
       ReadbackReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.DataReadback_Addr),
       FasdCoeffReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.FasdCoeff_Addr),
	   FrameDecimationReg(GP.WB_SpectrumAnalysis, Pp_Core_RegAddrs.Decimation_Addr)
{
}


//---------------------------------------------------------------------------
//  SpectralPostProc::TriggerMode() -
//---------------------------------------------------------------------------

void SpectralPostProc::TriggerMode(unsigned int parent, PostProcTriggerModeType value)
{
    SetIndex(parent, 0);
    TriggerModeReg.Mode.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralPostProc::SingleTrigger() -
//---------------------------------------------------------------------------

void SpectralPostProc::SingleTrigger(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    SingleTriggerReg.Single.Value(value ? true : false);
}

//---------------------------------------------------------------------------
//  SpectralPostProc::ThresholdProgEnable() -
//---------------------------------------------------------------------------

void SpectralPostProc::ThresholdProgEnable(unsigned int parent, PostProcTlsModeType value)
{
    SetIndex(parent, 0);
    TLSProgEnableReg.Mode.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralPostProc::ThresholdProgAddr() -
//---------------------------------------------------------------------------

void SpectralPostProc::ThresholdProgAddr(unsigned int parent, unsigned int addr)
{
    SetIndex(parent, 0);
    TLSAddrReg.Addr.Value(addr);

    TLSAddrReg.Apply();
}

//---------------------------------------------------------------------------
//  SpectralPostProc::ThresholdProgData() -
//---------------------------------------------------------------------------

void SpectralPostProc::ThresholdProgData(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    TLSDataReg.Data.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralPostProc::ThresholdProg() -
//---------------------------------------------------------------------------

void SpectralPostProc::ThresholdProg(unsigned int parent, const vector<int> &coeffs)
{
    SetIndex(parent, 0);

    for (size_t i = 0; i < coeffs.size(); i++)
        {
        // Set addr
        TLSAddrReg.Addr.Value(i);
        TLSAddrReg.Apply();
        // Set data
        TLSDataReg.Data.Value(coeffs[i]);
        }
}

//---------------------------------------------------------------------------
//  SpectralPostProc::FasdFilterCoeff() -
//---------------------------------------------------------------------------

void SpectralPostProc::FasdFilterCoeff(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    FasdCoeffReg.Coeff.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralPostProc::FrameDecimation() -
//---------------------------------------------------------------------------

void SpectralPostProc::FrameDecimation(unsigned int parent, int value)
{
    SetIndex(parent, 0);
    FrameDecimationReg.DecFactor.Value(value);
}

//===========================================================================
//  CLASS SpectralVitaFramer  --
//===========================================================================
//---------------------------------------------------------------------------
//  housekeeping for class SpectralVitaFramer
//---------------------------------------------------------------------------

static GpDspSubmodule::MapRegisters Sv_Sub_RegAddrs =
    {  mmSaVitaInfo,      //  Version_Addr
       mmSaVitaConfig,    //  Config_Addr
       mmSaVitaReset
    };

static SpectralVitaFramer::MapRegisters Sv_Core_RegAddrs =
    {   mmSaVitaEnable,
        mmSaVitaSize,
        mmSaVitaSid
    };

//---------------------------------------------------------------------------
//  constructor for class SpectralVitaFramer
//---------------------------------------------------------------------------

SpectralVitaFramer::SpectralVitaFramer(GpDspMemoryExtension & GP, GspDspModuleBase & cbase)
    :  GpDsp_2D_Submodule(cbase, GP.WB_SpectrumAnalysis, Sv_Sub_RegAddrs, ewbSpectrumAnalysis),
       EnableReg(GP.WB_SpectrumAnalysis, Sv_Core_RegAddrs.Enable_Addr),
       SizeReg(GP.WB_SpectrumAnalysis, Sv_Core_RegAddrs.Size_Addr),
       StreamIDReg(GP.WB_SpectrumAnalysis, Sv_Core_RegAddrs.StreamID_Addr)
{
}

//---------------------------------------------------------------------------
//  SpectralVitaFramer::Enable() -
//---------------------------------------------------------------------------

void SpectralVitaFramer::Enable(unsigned int parent, bool value)
{
    SetIndex(parent, 0);
    EnableReg.vframer_enable.Value(value);
}

//---------------------------------------------------------------------------
//  SpectralVitaFramer::Enable() -
//---------------------------------------------------------------------------

void SpectralVitaFramer::Size(unsigned int parent, unsigned int size)
{
    SetIndex(parent, 0);
    SizeReg.vframer_size.Value(size);
}

//---------------------------------------------------------------------------
//  SpectralVitaFramer::Enable() -
//---------------------------------------------------------------------------

void SpectralVitaFramer::Identity(unsigned int parent, unsigned int did, unsigned int sid)
{
    SetIndex(parent, 0);
    StreamIDReg.vframer_stream_id.Value(sid);
    StreamIDReg.vframer_dest_id.Value(did);
    StreamIDReg.Apply();
}


#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative
