#ifndef SPECTRALMODULE_GP_H
#define SPECTRALMODULE_GP_H

#include <Components_Gp.h>
#include <GpDspMemoryStringMap_Gp.h>
#include <vector>

namespace Innovative
{
#ifdef __CLR_VER
#pragma managed(push, off)
#endif

//===========================================================================
//  CLASS SpectralModule  -- Module shared features
//===========================================================================

class SpectralModule : public GspDspModuleBase
{
public:
    SpectralModule(GpDspMemoryExtension & GP);
    virtual ~SpectralModule() {}

};

//===========================================================================
//  CLASS GpDsp_2D_Submodule  -- Shared Submodule w/two_D indexing
//===========================================================================

class SpectralPeripheral : public GpDsp_2D_Submodule
{
    std::vector<int> ddc_map;

public:
    enum OutputModeType {oFullSpectrum, oThresholLimited};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct MapRegisters
    {
        unsigned int   Bypass_Addr;
        unsigned int   SourceSelection_Addr;
        unsigned int   BypassWindowing_Addr;
        unsigned int   OutputMode_Addr;
    };

    // definition should really be in cpp...
    SpectralPeripheral(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~SpectralPeripheral()  {}

    void BypassSpectAnalysis(unsigned int parent, bool value);
    void SourceSelect(unsigned int parent, int value);
    void BypassWindowing(unsigned int parent, bool value);
    void ModeSelect(unsigned int parent, OutputModeType value);
    void SetSourceMap(const std::vector<int> &values);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::BypassRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BypassRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBit      Bypass;

        BypassRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Bypass(*this, 0)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::SourceSelectionRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SourceSelectionRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        Register          Select;

        SourceSelectionRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Select(*this)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::BypassWindowingRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class BypassWindowingRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBit      Bypass;

        BypassWindowingRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Bypass(*this, 0)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::OutputModeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class OutputModeRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBit      Mode;

        OutputModeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Mode(*this, 0)
            {}
    };

private:
    BypassRegister          BypassReg;
    SourceSelectionRegister SourceSelectionReg;
    BypassWindowingRegister BypassWindowingReg;
    OutputModeRegister      OutputModeReg;

};

//===========================================================================
//  CLASS GpDsp_2D_Submodule  -- Shared Submodule w/two_D indexing
//===========================================================================

class SpectralMultiBuffer : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct MapRegisters
    {
        unsigned int   WinSize_Addr;
        unsigned int   Delay_Addr;
    };

    // definition should really be in cpp...
    SpectralMultiBuffer(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~SpectralMultiBuffer()  {}

    void WindowSize(unsigned int parent, int value);
    void Delay(unsigned int parent, int value);

    void setFasd(float value) { fasd = value; }

private:

    float fasd;

    SaShadowRegister WindowSizeReg;
    SaShadowRegister BufferDelayReg;
};

//===========================================================================
//  CLASS GpDsp_2D_Submodule  -- Shared Submodule w/two_D indexing
//===========================================================================

class SpectralWindowing : public GpDsp_2D_Submodule
{
public:
    enum WindowingModeType { wController, wCoeffs };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPeripheral::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct MapRegisters
    {
        unsigned int   CoefEn_Addr;
        unsigned int   WinCoef_Addr;
        unsigned int   CoefData_Addr;
    };

    // definition should really be in cpp...
    SpectralWindowing(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~SpectralWindowing()  {}

    void CoeffEnable(unsigned int parent, WindowingModeType value);
    void CoeffAddr(unsigned int parent, int value);
    void CoeffData(unsigned int parent, int value);
    void Coeffs(unsigned int parent, const std::vector<int>& cs);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::WindowingEnableRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class WindowingEnableRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Mode;

        WindowingEnableRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Mode(*this, 0, 2)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralWindowing::WindowingCoeffAddrRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class WindowingCoeffAddrRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Addr;

        WindowingCoeffAddrRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Addr(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralWindowing::WindowingCoeffDataRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class WindowingCoeffDataRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Data;

        WindowingCoeffDataRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Data(*this, 0, 16)
            {}
    };

private:
    WindowingEnableRegister    WindowingCoeffEnReg;
    WindowingCoeffAddrRegister WindowingCoeffAddrReg;
    WindowingCoeffDataRegister WindowingCoeffDataReg;
};


//===========================================================================
//  CLASS GpDsp_2D_Submodule  -- Shared Submodule w/two_D indexing
//===========================================================================

class SpectralFft : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralFft::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct MapRegisters
    {
        unsigned int   TrigMode_Addr;
        unsigned int   SingleFft_Addr;
    };

    // definition should really be in cpp...
    SpectralFft(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~SpectralFft()  {}

};

//===========================================================================
//  CLASS GpDsp_2D_Submodule  -- Shared Submodule w/two_D indexing
//===========================================================================

class SpectralPostProc : public GpDsp_2D_Submodule
{
public:
     enum PostProcTriggerModeType { pDisabled, pSingleFrame, pContinuous };
     enum PostProcTlsModeType { pTls,  pController };
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    struct MapRegisters
    {
        unsigned int   TrigMode_Addr;
        unsigned int   SingleTrig_Addr;
        unsigned int   ProgEn_Addr;
        unsigned int   ProgAddr_Addr;
        unsigned int   ProgData_Addr;
        unsigned int   DataReadback_Addr;
        unsigned int   FasdCoeff_Addr;
		unsigned int   Decimation_Addr;
    };

    // definition should really be in cpp...
    SpectralPostProc(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~SpectralPostProc()  {}

    void TriggerMode(unsigned int parent, PostProcTriggerModeType value);
    void SingleTrigger(unsigned int parent, int value);
    void ThresholdProgEnable(unsigned int parent, PostProcTlsModeType value);
    void ThresholdProgAddr(unsigned int parent, unsigned int addr);
    void ThresholdProgData(unsigned int parent, int value);
    void ThresholdProg(unsigned int parent, const std::vector<int> &coeffs);
    void FasdFilterCoeff(unsigned int parent, int value);
    void FrameDecimation(unsigned int parent, int value);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::TriggerModeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TriggerModeRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBitGroup      Mode;

        TriggerModeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Mode(*this, 0, 2)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::SingleTriggerRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SingleTriggerRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBit      Single;

        SingleTriggerRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Single(*this, 0)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::TLSProgEnableRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TLSProgEnableRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBitGroup      Mode;

        TLSProgEnableRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Mode(*this, 0, 2)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::TLSAddrRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TLSAddrRegister : public SaCachedShadowRegister
    {
        typedef SaCachedShadowRegister inherited;

    public:
        RegisterBitGroup      Addr;

        TLSAddrRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Addr(*this, 0, 15)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::TLSDataRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class TLSDataRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBitGroup      Data;

        TLSDataRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Data(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::ReadbackRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class ReadbackRegister : public Register
    {
        typedef Register inherited;

    public:
        RegisterBitGroup      Data;

        ReadbackRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Data(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::FasdCoeffRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FasdCoeffRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBitGroup      Coeff;

        FasdCoeffRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              Coeff(*this, 0, 16)
            {}
    };

   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralPostProc::FrameDecimationRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class FrameDecimationRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBitGroup      DecFactor;

        FrameDecimationRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              DecFactor(*this, 0, 16)
            {}
    };

private:
    TriggerModeRegister   TriggerModeReg;
    SingleTriggerRegister SingleTriggerReg;
    TLSProgEnableRegister TLSProgEnableReg;
    TLSAddrRegister       TLSAddrReg;
    TLSDataRegister       TLSDataReg;
    ReadbackRegister      ReadbackReg;
    FasdCoeffRegister     FasdCoeffReg;
	FrameDecimationRegister FrameDecimationReg;
};

//===========================================================================
//  CLASS SpectralVitaFramer  -- VitaFramer 'submodule'
//===========================================================================

class SpectralVitaFramer : public GpDsp_2D_Submodule
{
public:
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralVitaFramer::MapRegisters  -- Device Register Offsets
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class MapRegisters
    {
    public:
        unsigned int   Enable_Addr;
        unsigned int   Size_Addr;
        unsigned int   StreamID_Addr;
    };

    // definition should really be in cpp...
    SpectralVitaFramer(GpDspMemoryExtension & GP, GspDspModuleBase & base);
    virtual ~SpectralVitaFramer()  {}

    void Enable(unsigned int parent, bool value);
    void Size(unsigned int parent, unsigned int size);
    void Identity(unsigned int parent, unsigned int did, unsigned int sid);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralVitaFramer::EnableRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class EnableRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBit      vframer_enable;

        EnableRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_enable(*this, 0)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralVitaFramer::SizeRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class SizeRegister : public SaShadowRegister
    {
        typedef SaShadowRegister inherited;

    public:
        RegisterBitGroup      vframer_size;

        SizeRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_size(*this, 0, 16)
            {}
    };

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    //  CLASS SpectralVitaFramer::StreamIDRegister
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    class StreamIDRegister : public SaCachedShadowRegister
    {
        typedef SaCachedShadowRegister inherited;

    public:
        RegisterBitGroup      vframer_stream_id;
        RegisterBitGroup      vframer_dest_id;

        StreamIDRegister(IAddressingSpace &space, int offset)
            : inherited(space, offset),
              vframer_stream_id(*this, 0, 16),
              vframer_dest_id(*this, 16, 16)
            {}
    };

private:
    EnableRegister   EnableReg;
    SizeRegister     SizeReg;
    StreamIDRegister StreamIDReg;

};

#ifdef __CLR_VER
#pragma managed(pop)
#endif

}  // namespace Innovative

#endif // SPECTRALMODULE_GP_H
