#include <SupportProxy_Gp.h>
#include <SystemSupport_Mb.h>
#include <StringSupport_Mb.h>

//---------------------------------------------------------------------------
//  PowerMeterWrapper::PowerMeterWrapper()
//---------------------------------------------------------------------------

PowerMeterWrapper::PowerMeterWrapper(DspPowerMeterSM *apm, string atag, unsigned int acore)
    : pm(apm), tag(atag), core(acore)
{
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::~PowerMeterWrapper()
//---------------------------------------------------------------------------

PowerMeterWrapper::~PowerMeterWrapper()
{
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::Init()
//---------------------------------------------------------------------------

void PowerMeterWrapper::Init()
{
    AccumulatePoints(0x10);
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::Make_PowerMeter()
//---------------------------------------------------------------------------

void PowerMeterWrapper::AccumulatePoints(unsigned int power_of_2)
{
    pm->AccumulatePoints(core, power_of_2);
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::Reset()
//---------------------------------------------------------------------------

void PowerMeterWrapper::Reset(bool state)
{
    pm->Reset(core, state);
    pm->Reset(core, !state);
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::CommandTrigger()
//---------------------------------------------------------------------------

void PowerMeterWrapper::CommandTrigger()
{
    pm->CommandTrigger();
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::PowerData_dBm()
//---------------------------------------------------------------------------

void PowerMeterWrapper::Source(unsigned int source)
{
    pm->Source(core, source);
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::PowerData_dBm()
//---------------------------------------------------------------------------

double PowerMeterWrapper::PowerData_dBm(unsigned int source)
{
    Source(source);
    return pm->PowerData_dBm(core);
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::Tag()
//---------------------------------------------------------------------------

string PowerMeterWrapper::Tag() const
{
    return tag;
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::WhichBoard()
//---------------------------------------------------------------------------

void PowerMeterWrapper::WhichBoard(unsigned int ordinal)
{
    BoardOrdinal = ordinal;
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::WhichBoard()
//---------------------------------------------------------------------------
unsigned int PowerMeterWrapper::WhichBoard()
{
    return BoardOrdinal;
}

//---------------------------------------------------------------------------
//***************************************************************************
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  PowerMeterProxy::PowerMeterProxy()
//---------------------------------------------------------------------------

PowerMeterProxy::PowerMeterProxy()
{
}

//---------------------------------------------------------------------------
//  PowerMeterProxy::~PowerMeterProxy()
//---------------------------------------------------------------------------

PowerMeterProxy::~PowerMeterProxy()
{
    Clear();
}

//---------------------------------------------------------------------------
//  PowerMeterProxy::Clear()
//---------------------------------------------------------------------------

void PowerMeterProxy::Clear()
{
    while (group.size() > 0)
        {
        delete group.back();
        group.pop_back();
        }
}

//---------------------------------------------------------------------------
//  PowerMeterProxy::Init()
//---------------------------------------------------------------------------

void PowerMeterProxy::Init()
{
    for (size_t i = 0; i < group.size(); i++)
        group[i]->Init();
}

//---------------------------------------------------------------------------
//  PowerMeterProxy::PowerMeterList()
//---------------------------------------------------------------------------

void PowerMeterProxy::PowerMeterList(vector<string> &list)
{
    for (size_t i = 0; i < group.size(); i++)
        list.push_back(group[i]->Tag());
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::Reset()
//---------------------------------------------------------------------------

void PowerMeterProxy::Reset(bool state)
{
    for (size_t i = 0; i < group.size(); i++)
        group[i]->Reset(state);
}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::OverflowDetectors()
//---------------------------------------------------------------------------

PowerMeterProxy::PowerMeterGroupType PowerMeterProxy::PowerMeters(unsigned int board)
{
    PowerMeterProxy::PowerMeterGroupType bs;

    for (size_t i = 0; i < group.size(); i++)
        if (group[i]->WhichBoard() == board)
            bs.push_back(group[i]);
    return bs;

}

//---------------------------------------------------------------------------
//  PowerMeterWrapper::CommandTrigger()
//---------------------------------------------------------------------------

void PowerMeterProxy::CommandTrigger()
{
    if (group.size())
        group[0]->CommandTrigger();
}

//---------------------------------------------------------------------------
//  PowerMeterProxy::Proxy()
//---------------------------------------------------------------------------

PowerMeterWrapper &PowerMeterProxy::Proxy(unsigned int index)
{
    return *group[index];
}

//---------------------------------------------------------------------------
//  PowerMeterProxy::Append()
//---------------------------------------------------------------------------

void PowerMeterProxy::Append(PowerMeterWrapper *value)
{
    group.push_back(value);
}

//---------------------------------------------------------------------------
//  OverflowWrapper::OverflowWrapper()
//---------------------------------------------------------------------------

OverflowWrapper::OverflowWrapper(DspOverflowSM *aov, string atag, unsigned int acore)
    : ov(aov), tag(atag), core(acore)
{
}

//---------------------------------------------------------------------------
//  OverflowWrapper::~OverflowWrapper()
//---------------------------------------------------------------------------

OverflowWrapper::~OverflowWrapper()
{
}

//---------------------------------------------------------------------------
//  OverflowWrapper::Tag()
//---------------------------------------------------------------------------

string OverflowWrapper::Tag() const
{
    return tag + "[" + IntToString(core) + "]";
}

//---------------------------------------------------------------------------
//  OverflowWrapper::OverflowStatus()
//---------------------------------------------------------------------------

unsigned int  OverflowWrapper::OverflowStatus()
{
    return ov->OverflowStatus(core);
}

//---------------------------------------------------------------------------
//  OverflowWrapper::Reset()
//---------------------------------------------------------------------------

void OverflowWrapper::Reset(bool state)
{
    ov->Reset(core, state);
    ov->Reset(core, !state);
}

//---------------------------------------------------------------------------
//  OverflowWrapper::CommandTrigger()
//---------------------------------------------------------------------------

void OverflowWrapper::CommandTrigger()
{
     // nothing to do
}

//---------------------------------------------------------------------------
//  OverflowWrapper::WhichBoard()
//---------------------------------------------------------------------------

void OverflowWrapper::WhichBoard(unsigned int ordinal)
{
    BoardOrdinal = ordinal;
}

//---------------------------------------------------------------------------
//  OverflowWrapper::WhichBoard()
//---------------------------------------------------------------------------
unsigned int OverflowWrapper::WhichBoard()
{
    return BoardOrdinal;
}

//---------------------------------------------------------------------------
//  OverflowProxy::OverflowProxy()
//---------------------------------------------------------------------------

OverflowProxy::OverflowProxy()
{
}

//---------------------------------------------------------------------------
//  OverflowProxy::~OverflowProxy()
//---------------------------------------------------------------------------

OverflowProxy::~OverflowProxy()
{
    Clear();
}


//---------------------------------------------------------------------------
//  OverflowProxy::Clear()
//---------------------------------------------------------------------------

void OverflowProxy::Clear()
{
    while (group.size() > 0)
        {
        delete group.back();
        group.pop_back();
        }
}

//---------------------------------------------------------------------------
//  OverflowProxy::OverflowDetectorList()
//---------------------------------------------------------------------------

void OverflowProxy::OverflowDetectorList(vector<string> &list)
{
    for (size_t i = 0; i < group.size(); i++)
        list.push_back(group[i]->Tag());
}

//---------------------------------------------------------------------------
//  OverflowProxy::OverflowDetectorList()
//---------------------------------------------------------------------------

void OverflowProxy::OverflowDetectorList(vector<string> &list, unsigned int board)
{
    for (size_t i = 0; i < group.size(); i++)
        if (group[i]->WhichBoard() == board)
            list.push_back(group[i]->Tag());
}

//---------------------------------------------------------------------------
//  OverflowProxy::OverflowDetectors()
//---------------------------------------------------------------------------

OverflowProxy::OverflowGroupType OverflowProxy::OverflowDetectors(unsigned int board)
{
    OverflowProxy::OverflowGroupType bs;

    for (size_t i = 0; i < group.size(); i++)
        if (group[i]->WhichBoard() == board)
            bs.push_back(group[i]);
    return bs;

}

//---------------------------------------------------------------------------
//  OverflowProxy::Proxy()
//---------------------------------------------------------------------------

OverflowWrapper &OverflowProxy::Proxy(unsigned int index)
{
    return *group[index];
}

//---------------------------------------------------------------------------
//  OverflowProxy::Proxy()
//---------------------------------------------------------------------------

//OverflowWrapper &OverflowProxy::Proxy(unsigned int index, unsigned int board)
//{
//    return *boards[index];
//}


//---------------------------------------------------------------------------
//  OverflowProxy::Append()
//---------------------------------------------------------------------------

void OverflowProxy::Append(OverflowWrapper *value)
{
    group.push_back(value);
}

//---------------------------------------------------------------------------
//  OverflowProxy::Reset()
//---------------------------------------------------------------------------

void OverflowProxy::Reset(bool state)
{
    for (size_t i = 0; i < group.size(); i++)
        group[i]->Reset(state);
}

//---------------------------------------------------------------------------
//  OverflowProxy::CommandTrigger()
//---------------------------------------------------------------------------

void OverflowProxy::CommandTrigger()
{
    group[0]->CommandTrigger();
}

//---------------------------------------------------------------------------
//  OverflowProxy::Count()
//---------------------------------------------------------------------------

size_t OverflowProxy::Count() const
{
    return group.size();
}

//---------------------------------------------------------------------------
//  OverflowProxy::Count()
//---------------------------------------------------------------------------

size_t OverflowProxy::Count(unsigned int board) const
{
    size_t count = 0;
    for (size_t i = 0; i < group.size(); i++)
        if (group[i]->WhichBoard() == board)
            count++;

    return count;
}


//---------------------------------------------------------------------------
//  OverflowWrapper::OverflowWrapper()
//---------------------------------------------------------------------------

ToneGeneratorWrapper::ToneGeneratorWrapper(DspToneGeneratorSM *atone, string atag, unsigned int acore, const ToneGeneratorDataType &tonedef)
    : tone(atone), tag(atag + IntToString(acore)), core(acore)
{
	tones[0] = tonedef;
	tones[1] = tonedef;
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::~ToneGeneratorWrapper()
//---------------------------------------------------------------------------

ToneGeneratorWrapper::~ToneGeneratorWrapper()
{
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::AttachSettings() -
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::AttachSettings(Innovative::IniSaver& saver)
{
    saver.Install( ToIni(tag + "-tone_freq1",         tones[0].Freq,    tones[0].Freq)  );
    saver.Install( ToIni(tag + "-tone_ph1",           tones[0].Phase,   tones[0].Phase)  );
    saver.Install( ToIni(tag + "-tone_gain1",         tones[0].Gain,    tones[0].Gain)  );
    saver.Install( ToIni(tag + "-tone_freq2",         tones[1].Freq,    tones[1].Freq)  );
    saver.Install( ToIni(tag + "-tone_ph2",           tones[1].Phase,   tones[1].Phase)  );
    saver.Install( ToIni(tag + "-tone_gain2",         tones[1].Gain,    tones[1].Gain)  );
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Frequency1()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Frequency1(double freq, double srate[2], double /*fdiv*/)
{
    Set_Frequency1(freq, srate[0]);   // use the ADC frequency
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Phase1()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Phase1(double value)
{
    tone->Phase1(core, value);
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Gain1()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Gain1(double value)
{
    tone->Gain1(core, value);
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Frequency2()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Frequency2(double freq, double srate)
{
    tone->Frequency2(core, freq, srate);
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Phase2()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Phase2(double value)
{
    tone->Phase2(core, value);
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Gain2()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Gain2(double value)
{
    tone->Gain2(core, value);
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Reset()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::Reset(bool state)
{
    tone->Reset(core, state);
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::CommandTrigger()
//---------------------------------------------------------------------------

void ToneGeneratorWrapper::CommandTrigger()
{
    // Nothing to do
}

//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::Set_Frequency1() - actually set Freq 1
//---------------------------------------------------------------------------

void  ToneGeneratorWrapper::Set_Frequency1(double freq, double srate)
{
    tone->Frequency1(core, freq, srate);
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::ToneGeneratorProxy()
//---------------------------------------------------------------------------

ToneGeneratorProxy::ToneGeneratorProxy() : IpolRate(1.0)
{
    SRate[0] = SRate[1] = 1.0;
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::~ToneGeneratorProxy()
//---------------------------------------------------------------------------

ToneGeneratorProxy::~ToneGeneratorProxy()
{
    Clear();
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::Clear()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::Clear()
{
    while (group.size() > 0)
        {
        delete group.back();
        group.pop_back();
        }
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::Append()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::Append(ToneGeneratorWrapper *value)
{
    group.push_back(value);
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::Proxy()
//---------------------------------------------------------------------------

ToneGeneratorWrapper &ToneGeneratorProxy::Proxy(unsigned int index)
{
    return *group[index];
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::Reset()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::Reset(bool state)
{
    for (size_t i = 0; i < group.size(); i++)
        group[i]->Reset(state);
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::CommandTrigger()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::CommandTrigger()
{
    for (size_t i = 0; i < group.size(); i++)
        group[i]->CommandTrigger();
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::CommandTrigger()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::CommandTrigger(unsigned int n)
{
    if (n < group.size())
        group[n]->CommandTrigger();
}


//---------------------------------------------------------------------------
//  ToneGeneratorWrapper::AttachSettings() -
//---------------------------------------------------------------------------

void ToneGeneratorProxy::AttachSettings(Innovative::IniSaver& saver)
{
    for (size_t i = 0; i < group.size(); i++)
        group[i]->AttachSettings(saver);
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::Size()
//---------------------------------------------------------------------------

size_t ToneGeneratorProxy::Size()
{
    return group.size();
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::ApplyCached()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::ApplyCached()
{
    for (size_t i = 0; i < group.size(); i++)
        {
        group[i]->Frequency1(group[i]->tones[0].Freq, SRate, IpolRate);
        group[i]->Phase1(group[i]->tones[0].Phase);
        group[i]->Gain1(group[i]->tones[0].Gain);
        }
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::ApplyCached()
//---------------------------------------------------------------------------

void ToneGeneratorProxy::ApplyCached(unsigned int n)
{
    if (n < group.size())
        {
        group[n]->Frequency1(group[n]->tones[0].Freq, SRate, IpolRate);
        group[n]->Phase1(group[n]->tones[0].Phase);
        group[n]->Gain1(group[n]->tones[0].Gain);
        }
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::SamplingRate() -
//---------------------------------------------------------------------------

void ToneGeneratorProxy::SamplingRate(double adc_rate, double dac_rate)
{
    SRate[0] = adc_rate;    SRate[1] = dac_rate;
}

//---------------------------------------------------------------------------
//  ToneGeneratorProxy::InterpolationRate() -
//---------------------------------------------------------------------------

void ToneGeneratorProxy::InterpolationRate(double value)
{
    IpolRate = value;
}
